/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AboutPanel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:41 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.beans;

/* 
// For testing under 1.2.x
import com.klg.jclass.util.swing.JCExitFrame;
*/
import com.klg.jclass.util.internal.JClassInfo;
import com.klg.jclass.util.swing.JCHTMLPane;
import javax.swing.*;
import java.awt.*;
import java.lang.reflect.*;

/**
 * Extension of JCHTMLPane that holds information about a product :
 * Its name, package, version, configuration, and possible evaluation
 * message.
 * For internal use only.
 */
public class AboutPanel extends JCHTMLPane {

/**
 * Constructs a default About panel with no version information.
 * For internal use only.
 */
public AboutPanel()
{
    this("");
}

/**
 * Constructs a default About panel with the provided version information.
 * For internal use only.
 */
public AboutPanel(String vStr)
{
    super();
    setVersion(vStr);
}

/**
 * Constructs an About panel, builds version info from the provided strings.
 * For internal use only.
 */
public AboutPanel(String product, String pkg_name, String version,
                  String config_msg, String eval_msg)
{
    this(product + " " + pkg_name + " " + version + " " + config_msg +
         " " + eval_msg);
}

transient protected Image logo = null;

protected Image getLogo()
{
    if (logo == null) {
        logo = JClassInfo.makeKLImage(this);
    }
    return(logo);
}

/**
 * Sets the version string.
 * For internal use only.
 */
public void setVersion(String vStr)
{
    setText("<html><BODY BGCOLOR=\"#DDDDDD\"><Big><Big><Strong>About JClass</Strong></Big></Big>&nbsp;&nbsp;&nbsp;(" + vStr + ")<br>" +
            "<br>" + JClassInfo.product_info + JClassInfo.docsupMessage);
}

/**
 * Builds the version string from the provided strings.
 * For internal use only.
 */
public void setVersion(String product, String pkg_name, String version,
                       String config_msg, String eval_msg)
{
    setVersion(product + " " + pkg_name + " " + version + " " +
               config_msg + " " + eval_msg);
}

// A hack to properly display the about panel the first time.
// It seems that the second time and on will work properly.
// The Dimension used is based on the current correct size after the second time
// and should be updated if the text is updated.
public Dimension getPreferredSize()
{
    Dimension prefSize = super.getPreferredSize();
    if (prefSize.height < 100) {
        prefSize = new Dimension(614, 305);
    }
    return prefSize;
}

protected void followLink(java.net.URL url)
{
    Class borland = null;
    boolean windows = false;
    String redirectUrl = url.toString();

    try {
        // See if we can locate borland's help viewer
        // We'll use it as a browser and send it to the company website
        borland = Class.forName("com.borland.primetime.help.HelpManager");
    }
    catch (Exception e) {
        String os = System.getProperty("os.name");
        if (os != null && os.startsWith("Windows")) {
            windows = true;
        }
        else {
            windows = false;
        }
    }
    try {
        if (borland != null) {
            // First, lets find the dialog that is up
            JDialog dialog = null;
            Container parent = this;
            while (parent != null && parent instanceof Container) {
                parent = parent.getParent();
                if (parent instanceof JDialog) {
                    dialog = (JDialog)parent;
                    break;
                }
            }

            // Now find the showURL method that we need to call
            Class[] parms = new Class[2];
            parms[0] = Class.forName("java.lang.String");
            parms[1] = Class.forName("javax.swing.JDialog");
            Method method = borland.getMethod("showURL", parms);

            // And then call the method
            if (method != null) {
                Object[] args = new Object[2];
                args[0] = redirectUrl;
                args[1] = dialog;
                method.invoke(null, args);
            }
        }
        else if (windows) {
            String cmd = "rundll32 url.dll,FileProtocolHandler " +
                redirectUrl;
            Runtime.getRuntime().exec(cmd);
        }
        else {
            // Launching browser on unix platform

            // Under Unix, Netscape has to be running for the "-remote"
            // command to work. So, we try sending the command and
            // check for an exit value. If the exit command is 0,
            // it worked, otherwise we need to start the browser.

            String cmd = "netscape -remote openURL(" + redirectUrl + ")";
            Process p = Runtime.getRuntime().exec(cmd);

            try {
                int exitCode = p.waitFor();

                if (exitCode != 0) {
                    cmd = "netscape " + redirectUrl;
                    p = Runtime.getRuntime().exec(cmd);
                }
            }
            catch (InterruptedException ie) {
                System.err.println("Error launching browser");
            }
        }
    }
    catch (Exception exc) {}
}

/*
public static void main(String args[])
{
// For testing under 1.2.x - use these 3 lines if you're using JCExitFrame
//   JCExitFrame f = new JCExitFrame("Test");
//   f.getContentPane().setLayout(new java.awt.GridLayout(1,1));
//   f.getContentPane().add(new AboutPanel("Test"));
   // Use these 3 lines if you're using java.awt.Frame
	java.awt.Frame f = new java.awt.Frame("Test");
	f.setLayout(new java.awt.GridLayout(1,1));
	f.add(new AboutPanel("JClass Beans"));

	f.pack();
	f.show();
}
*/

}
