/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AbstractBeanEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:41 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.beans;

import com.klg.jclass.util.swing.JCBrace;
import com.klg.jclass.util.swing.JCColumnLayout;
import com.klg.jclass.util.swing.JCElasticLayout;
import com.klg.jclass.util.swing.JCHTMLPane;

import javax.swing.*;
import java.awt.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyEditor;

/**
 * AbstractBeanEditor wraps the editor with an HTML pane. The pane can
 * contain useful descriptions/information on how to use the editor.
 */
abstract public class AbstractBeanEditor extends JPanel
                                         implements PropertyEditor
{

protected PropertyChangeSupport  support = new PropertyChangeSupport(this);
protected JCHTMLPane             htmlPane;
protected String                 editorString;

/**
 * Wraps an editor with an HTMLPane which will contain localized
 * help information.
 * @param editor_string String will be displayed in property sheets.
 */
public AbstractBeanEditor(String locale_bundle, String help_key,
                          String editor_string)
{
    // Set the editorString
    editorString = editor_string;
    if (editorString == null) {
        editorString = "";
    }

    // Get the localized help text for the resource bundle
    JCBeanInfo.getLocaleManager().add(locale_bundle);
    String html_help_text = JCBeanInfo.getLocaleManager().getString(help_key);
    if (html_help_text == null) {
        throw new IllegalArgumentException("Invalid help key \"" +
                                           help_key + "\" specified.");
    }
    setLayout(new JCColumnLayout());

    // Add the actual editor component
    add(getEditorComponent());
    add(JCBrace.createVerticalBrace(2));

    // Create a scroll pane and an html pane. This pane will contain help
    // messages on how to use the editor.
    htmlPane = new JCHTMLPane(html_help_text);
    //	htmlPane.setPreferredSize(new Dimension(150,100));

    JScrollPane scroll_pane = new JScrollPane(htmlPane);
    add(scroll_pane, JCElasticLayout.COMPLETELY_ELASTIC_CONSTRAINT);

    // Always show the vertical scrollbar
    scroll_pane.setVerticalScrollBarPolicy(
        ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
}

/**
 * Sets the help text given the lookup key in the resource bundle.
 */
public void setHelpText(String help_key)
{
    String html_help_text = JCBeanInfo.getLocaleManager().getString(help_key);
    if (html_help_text == null) {
        throw new IllegalArgumentException("Invalid help key \"" +
                                           help_key + "\" specified.");
    }
    htmlPane.setText(html_help_text);
}

public void paintValue(Graphics gc, Rectangle box)
{
    FontMetrics fm = gc.getFontMetrics();
    int vpad = (box.height - fm.getAscent()) / 2;
    String s = getValueAsText();
    if (s == null || s.length() == 0) {
        s = editorString;
    }
    gc.drawString(s, box.x + 2, box.y + box.height - vpad);
}

/**
 * Returns the actual editor component which will be wrapped with the html
 * help pane.
 */
abstract public Component getEditorComponent();

/**
 * Returns the code generation string.
 */
abstract public String getJavaInitializationString();

/**
 * Returns the value of the property.
 */
abstract public Object getValue();

/**
 * Returns a text representation of the value.
 */
abstract protected String getValueAsText();

/**
 * Sets the value of the property.
 */
abstract public void setValue(Object o);

/**
 * Sets the html text in the html pane.
 * @param html_help_text text must be in HTML format.
 */
public void setHTMLHelpText(String html_help_text)
{
    htmlPane.setText(html_help_text);
}

/**
 * Returns null since we are assuming that the value cannot be represented as
 * text.
 */
public String getAsText()
{
    return(null);
}

/**
 * This method currently does nothing since an assumption is made that
 * since this is a custom editor, the value could not be set as text.
 */
public void setAsText(String text) throws IllegalArgumentException {}

public String[] getTags()
{
    return(null);
}

/**
 * Always returns True meaning the paintValue method is honored.
 */
public boolean isPaintable()
{
    return(true);
}

/**
 * Returns the custom editor.
 */
public Component getCustomEditor()
{
    return(this);
}

/**
 * Always returns true since this class is for custom editors.
 */
public boolean supportsCustomEditor()
{
    return(true);
}

public void addPropertyChangeListener(PropertyChangeListener l)
{
    support.addPropertyChangeListener(l);
}

public void removePropertyChangeListener(PropertyChangeListener l)
{
    support.removePropertyChangeListener(l);
}

}

