/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AbstractBeanLocaleInfo.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:41 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.beans;

import com.klg.jclass.beans.JCBeanInfo;

import java.util.ListResourceBundle;
import java.awt.event.KeyEvent;

/** 
 *  Abstact class for bean locale strings that allows bean text to be
 *  internationalized.  It implements ListResourceBundle's
 *  getContents() method by constructing an array from information received
 *  from its own abstract method getItems(), which must be implemented
 *  by any implementors.
 */

public abstract class AbstractBeanLocaleInfo extends ListResourceBundle {

protected Object[][]  strings = null;
protected boolean     isDefaultLocale;

/** Returns the list of items for the locale */
abstract protected Item [] getItems();

public Object[][] getContents()
{
    if (strings == null) {
        Item items[] = getItems();

        int num_strings = 2 * items.length;
        // Create strings array
        strings = new Object[num_strings][2];

        // Populate strings
        int j = 0;
        for (int i = 0; i < num_strings; i++) {
            if ((i % 2) == 0) {
                // Even values
                strings[i][0] = items[j].key + JCBeanInfo.DISPLAY_NAME_SUFFIX;
                strings[i][1] = items[j].displayName;
            }
            else {
                strings[i][0] = items[j].key + JCBeanInfo.SHORT_DESCRIPTION_SUFFIX;
                strings[i][1] = items[j].shortDescription;
                j++;
            }
        }

    }
    return(strings);
}

/**
 * Class to create a list of objects to return
 */
public static class Item
{
    public String key;
    public String displayName;
    public String shortDescription;

    /**
     * Constructor used when constructing locale bundles
     */
    public Item(String key, String display_name, String short_description)
    {
        this.key = key;
        this.displayName = display_name;
        this.shortDescription = short_description;
    }

}

}
