/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AbstractListEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:41 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.beans;

import com.klg.jclass.util.swing.JCBrace;
import com.klg.jclass.util.swing.JCColumnLayout;
import com.klg.jclass.util.swing.JCElasticLayout;
import com.klg.jclass.util.swing.JCRowLayout;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Abstract class for implementation of an editor that allows selection
 * from a list of values.  Implementations vary as per the type of the value.
 */
public abstract class AbstractListEditor extends AbstractBeanEditor {

protected JList       list;
protected JTextField  text;

public AbstractListEditor(String locale_bundle, String help_key,
                          String editor_string)
{
    super(locale_bundle, help_key, editor_string);

    // Check to see if subclass provides a list renderer
    if (hasListCellRenderer() == true) {
        ListCellRenderer renderer = getCellRenderer();
        if (renderer != null) {
            list.setCellRenderer(renderer);
        }
    }
}

/**
 * Set the preferred size for the editor.
 */
public Dimension getPreferredSize()
{
    return(new Dimension(300, 280));
}

/**
 * Returns the actual editor component which will be wrapped with the html
 * help pane.
 */
public Component getEditorComponent()
{
    // create container for entire editor component
    JPanel panel = new JPanel();
    panel.setLayout(new JCRowLayout());

    // Add the JList to the layout
    list = new JList(new DefaultListModel());
    list.setVisibleRowCount(10);
    list.setBorder(new TitledBorder("Current list:"));
    list.setToolTipText("Current list of values");
    list.setFixedCellWidth(120);
    list.setFixedCellHeight(15);

    // Add the list to a scroll pane
    JScrollPane scroll_pane = new JScrollPane(list);
    scroll_pane.setVerticalScrollBarPolicy(
        ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
    scroll_pane.setHorizontalScrollBarPolicy(
        ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);

    // Create a container for the buttons and the text field
    JPanel panel2 = new JPanel();
    panel2.setLayout(new JCColumnLayout(SwingConstants.CENTER));
    panel2.add(JCBrace.createVerticalBrace(10));

    // Add textfield with tooltip
    panel2.add(new JLabel("Enter a value:"));
    text = new JTextField(9);
    panel2.add(text, JCElasticLayout.HORIZONTALLY_ELASTIC_CONSTRAINT);
    text.setToolTipText("Enter new value:");
    text.addActionListener(new Add());

    // Create a brace between the textfield and the panel of buttons
    panel2.add(JCBrace.createVerticalBrace(10));

    // Create buttons and add tooltips
    JButton add_button = new JButton("Add");
    add_button.addActionListener(new Add());
    add_button.setToolTipText("Adds the new value to list");

    JButton remove_button = new JButton("Remove");
    remove_button.addActionListener(new Remove());
    remove_button.setToolTipText("Removes selected items from the list");

    // Add the buttons to the button panel and space buttons 10 pixels apart
    // with braces
    panel2.add(add_button,
               JCElasticLayout.HORIZONTALLY_ELASTIC_CONSTRAINT);
    panel2.add(JCBrace.createVerticalBrace(10));
    panel2.add(remove_button,
               JCElasticLayout.HORIZONTALLY_ELASTIC_CONSTRAINT);
    panel2.add(JCBrace.createVerticalBrace(10));

    // Add all components to the root container
    panel.add(scroll_pane, JCElasticLayout.COMPLETELY_ELASTIC_CONSTRAINT);
    panel.add(JCBrace.createHorizontalBrace(25));
    panel.add(panel2);
    panel.add(JCBrace.createHorizontalBrace(20));

    panel.setMinimumSize(panel.getPreferredSize());
    return(panel);
}

/**
 * This method will convert the String value into an object.
 */
abstract protected Object parse(String text);

/**
 * Determines if a custom list cell renderer is needed to display the list
 * values in the list.  If true, the getCellRenderer method must be
 * overridden to return the ListCellRenderer.
 */
abstract protected boolean hasListCellRenderer();

/**
 * Returns a custom cell renderer. Default will return null. If an
 * editor needs a custom cell renderer, it must override this method
 * and return the ListCellRenderer.  This method will only be called if
 * hasListCellRenderer returns true.
 */
protected ListCellRenderer getCellRenderer()
{
    return(null);
}

/**
 * Returns a text representation of the value.
 */
protected String getValueAsText()
{
    return(null);
}

/**
 * Adds the item to the list.
 */
public class Add implements ActionListener
{
    public void actionPerformed(ActionEvent e)
    {
        DefaultListModel lm = (DefaultListModel)list.getModel();
        Object value = parse(text.getText());
        if (value != null && !lm.contains(value)) {
            lm.addElement(value);
        }
        text.setText("");
        text.requestFocus();
        //Object[] array = toArray();
        //setValue((Object)array);
        support.firePropertyChange(null, null, null);
    }
} // Add

/**
 * Removes the selected item from the list.
 */
public class Remove implements ActionListener
{
    public void actionPerformed(ActionEvent e)
    {
        DefaultListModel lm = (DefaultListModel)list.getModel();
        int[] index = list.getSelectedIndices();
        for (int i = index.length - 1; i >= 0; i--) {
            lm.removeElementAt(index[i]);
        }
        //Object[] array = toArray();
        //setValue((Object)array);
        list.requestFocus();
        if (index.length > 0) {
            int size = lm.getSize();
            if (index[0] >= size) {
                list.setSelectedIndex(size - 1);
            }
            else {
                list.setSelectedIndex(index[0]);
            }
        }
        support.firePropertyChange(null, null, null);
    }
} // Remove

/**
 * This method will convert the ListModel to an Object Array.
 */
public Object[] toArray()
{
    DefaultListModel lm = (DefaultListModel)list.getModel();
    return(lm.toArray());
}

}
