/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: BorderEditorPanel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:41 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.beans;

import com.klg.jclass.util.swing.JCAlignLayout;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

/**
 * The BorderEditorPanel class is a JPanel containing components for
 * editing the javax.swing.Border type.
 */
public class BorderEditorPanel extends JPanel
                               implements ActionListener,
                                          ItemListener,
                                          FocusListener,
                                          PropertyChangeListener
{

protected JComboBox         borderCB = null;
protected JPanel            subPanel = null;

protected JLabel            matteColorLabel = null;

protected ColorEditorPanel  lineColorEditor = null;
protected ColorEditorPanel  matteColorEditor = null;
protected ColorEditorPanel  shadowColorEditor = null;
protected ColorEditorPanel  highlightColorEditor = null;

protected JTextField        topTF = null;
protected JTextField        leftTF = null;
protected JTextField        bottomTF = null;
protected JTextField        rightTF = null;

protected ButtonGroup       bevelGroup = null;
protected JRadioButton      bevelLoweredRB = null;
protected JRadioButton      bevelRaisedRB = null;

protected JTextField        thickTF = null;
protected JTextField        titledTF = null;

/* Valid Border Types -- indexes for the following arrays */
final static int EMPTY = 0;
final static int BEVEL = 1;
final static int ETCHED = 2;
final static int LINE = 3;
final static int SOFTBEVEL = 4;
final static int TITLED = 5;
final static int MATTE = 6;

//Localization Keys
final public static String BORDER_EMPTY          = "Empty";
final public static String BORDER_BEVEL          = "Bevel";
final public static String BORDER_ETCHED         = "Etched";
final public static String BORDER_LINE           = "Line";
final public static String BORDER_SOFT_BEVEL     = "Soft Bevel";
final public static String BORDER_TITLED         = "Titled";
final public static String BORDER_PROP_TITLE     = "Border Properties";
final public static String BORDER_LOWERED        = "Lowered";
final public static String BORDER_RAISED         = "Raised";
final public static String BORDER_HIGH_COLOR     = "Highlight Color";
final public static String BORDER_SHAD_COLOR     = "Shadow Color";
final public static String BORDER_LINE_COLOR     = "Line Color";
final public static String BORDER_LINE_THICKNESS = "Line Thickness";
final public static String BORDER_TOP_MARGIN     = "Top Margin";
final public static String BORDER_LEFT_MARGIN    = "Left Margin";
final public static String BORDER_BOTTOM_MARGIN  = "Bottom Margin";
final public static String BORDER_RIGHT_MARGIN   = "Right Margin";
final public static String BORDER_MATTE_COLOR    = "Matte Color";
final public static String BORDER_TITLE_TEXT     = "Title Text";

// MATTE not included because the MatteBorder class doesn't implement some
// methods needed to query property values like color.
/** Array of border types */
static int border_values[] = {
    EMPTY, BEVEL, ETCHED, LINE, SOFTBEVEL, TITLED
};

static String border_i18n_names[] = null;

/** Corresponding array of card names, the strings that define the page
 *  name for each border type.  (Some border types use the same card
 *  page for editing.)
 */
static String border_cards[] = {
    "Matte", "Bevel", "Bevel", "Line", "Bevel", "Titled", "Matte"
};

private int     currentBorderType = 0;
private Border  currentBorder = BorderFactory.createEmptyBorder();

/*
*  Constructor creates super() and calls init()
*/
public BorderEditorPanel()
{
    super();
    init();
}

/**
 * Initializes the editor with a combo box containing the border types,
 * a card layout containing the pages for specific parameters for the
 * various border types, and a button for committing the border type.
 */
public void init()
{
    JCBeanInfo.getLocaleManager().add("com.klg.jclass.beans.resources.LocaleInfo");
    //can't initialize this array up top because we can't get the strings
    //until the above line has been exectued.

    border_i18n_names = new String[6];
    border_i18n_names[0] =
        JCBeanInfo.getLocaleManager().getString(BORDER_EMPTY);
    border_i18n_names[1] =
        JCBeanInfo.getLocaleManager().getString(BORDER_BEVEL);
    border_i18n_names[2] =
        JCBeanInfo.getLocaleManager().getString(BORDER_ETCHED);
    border_i18n_names[3] =
        JCBeanInfo.getLocaleManager().getString(BORDER_LINE);
    border_i18n_names[4] =
        JCBeanInfo.getLocaleManager().getString(BORDER_SOFT_BEVEL);
    border_i18n_names[5] =
        JCBeanInfo.getLocaleManager().getString(BORDER_TITLED);

    setLayout(new BorderLayout());

    borderCB = new JComboBox(border_i18n_names);
    borderCB.addItemListener(this);
    add("North", borderCB);

    subPanel = new JPanel();
    subPanel.setBorder(BorderFactory.createTitledBorder(
        JCBeanInfo.getLocaleManager().getString(BORDER_PROP_TITLE)
    ));
    subPanel.setLayout(new CardLayout());
    subPanel.add(createMatteCard(), border_cards[MATTE]);
    subPanel.add(createBevelCard(), border_cards[BEVEL]);
    subPanel.add(createLineCard(), border_cards[LINE]);
    subPanel.add(createTitledCard(), border_cards[TITLED]);

    add("Center", subPanel);

    // Update to the correct card and border parameters
    cardChanging();
    changeBorder();
}

/**
 * Create card for Bevel, Soft Bevel, and Etched borders.
 */
private JPanel createBevelCard()
{
    JPanel bevelPanel = new JPanel();
    bevelPanel.setLayout(new BorderLayout());

    JPanel lowRaisePanel = new JPanel();
    lowRaisePanel.setLayout(new GridLayout(1, 2));
    lowRaisePanel.setBorder(BorderFactory.createLineBorder(getForeground(), 1));
    bevelGroup = new ButtonGroup();
    bevelLoweredRB = new JRadioButton(
        JCBeanInfo.getLocaleManager().getString(BORDER_LOWERED)
    );
    bevelRaisedRB = new JRadioButton(
        JCBeanInfo.getLocaleManager().getString(BORDER_RAISED)
    );
    bevelLoweredRB.addActionListener(this);
    bevelRaisedRB.addActionListener(this);
    bevelGroup.add(bevelLoweredRB);
    bevelGroup.add(bevelRaisedRB);
    lowRaisePanel.add(bevelLoweredRB);
    lowRaisePanel.add(bevelRaisedRB);

    bevelPanel.add("North", lowRaisePanel);

    JPanel colorPanel = new JPanel();
    colorPanel.setLayout(new JCAlignLayout());
    JLabel highlightLabel = new JLabel(
        JCBeanInfo.getLocaleManager().getString(BORDER_HIGH_COLOR)
    );
    JLabel shadowLabel = new JLabel(
        JCBeanInfo.getLocaleManager().getString(BORDER_SHAD_COLOR)
    );
    highlightColorEditor = new ColorEditorPanel(null);
    highlightColorEditor.addPropertyChangeListener(this);
    shadowColorEditor = new ColorEditorPanel(null);
    shadowColorEditor.addPropertyChangeListener(this);
    colorPanel.add(highlightLabel);
    colorPanel.add(highlightColorEditor);
    colorPanel.add(shadowLabel);
    colorPanel.add(shadowColorEditor);

    bevelPanel.add("Center", colorPanel);
    return(bevelPanel);
}

/**
 * Create card for Line borders.
 */
private JPanel createLineCard()
{
    JPanel linePanel = new JPanel();
    linePanel.setLayout(new JCAlignLayout());

    JLabel colorLabel = new JLabel(
        JCBeanInfo.getLocaleManager().getString(BORDER_LINE_COLOR)
    );
    lineColorEditor = new ColorEditorPanel(getForeground());
    lineColorEditor.addPropertyChangeListener(this);
    JLabel thickLabel = new JLabel(
        JCBeanInfo.getLocaleManager().getString(BORDER_LINE_THICKNESS)
    );
    thickTF = new JTextField(5);
    thickTF.addActionListener(this);
    thickTF.addFocusListener(this);

    linePanel.add(colorLabel);
    linePanel.add(lineColorEditor);
    linePanel.add(thickLabel);
    linePanel.add(thickTF);

    return linePanel;
}

/**
 * Create card for Matte and Empty borders
 */
private JPanel createMatteCard()
{
    JPanel mattePanel = new JPanel();
    mattePanel.setLayout(new JCAlignLayout());

    JLabel topLabel = new JLabel(
        JCBeanInfo.getLocaleManager().getString(BORDER_TOP_MARGIN)
    );
    JLabel leftLabel = new JLabel(
        JCBeanInfo.getLocaleManager().getString(BORDER_LEFT_MARGIN)
    );
    JLabel bottomLabel = new JLabel(
        JCBeanInfo.getLocaleManager().getString(BORDER_BOTTOM_MARGIN)
    );
    JLabel rightLabel = new JLabel(
        JCBeanInfo.getLocaleManager().getString(BORDER_RIGHT_MARGIN)
    );

    topTF = new JTextField(5);
    leftTF = new JTextField(5);
    bottomTF = new JTextField(5);
    rightTF = new JTextField(5);

    topTF.addActionListener(this);
    topTF.addFocusListener(this);
    leftTF.addActionListener(this);
    leftTF.addFocusListener(this);
    bottomTF.addActionListener(this);
    bottomTF.addFocusListener(this);
    rightTF.addActionListener(this);
    rightTF.addFocusListener(this);

    matteColorLabel = new JLabel(
        JCBeanInfo.getLocaleManager().getString(BORDER_MATTE_COLOR)
    );
    matteColorEditor = new ColorEditorPanel(getForeground());
    matteColorEditor.addPropertyChangeListener(this);

    mattePanel.add(topLabel);
    mattePanel.add(topTF);
    mattePanel.add(leftLabel);
    mattePanel.add(leftTF);
    mattePanel.add(bottomLabel);
    mattePanel.add(bottomTF);
    mattePanel.add(rightLabel);
    mattePanel.add(rightTF);
    mattePanel.add(matteColorLabel);
    mattePanel.add(matteColorEditor);
    return(mattePanel);
}

/**
 * Create card for Titled borders
 */
private JPanel createTitledCard()
{
    JPanel titledPanel = new JPanel();
    titledPanel.setLayout(new JCAlignLayout());

    JLabel titledLabel = new JLabel(
        JCBeanInfo.getLocaleManager().getString(BORDER_TITLE_TEXT)
    );
    titledPanel.add(titledLabel);

    titledTF = new JTextField(20);
    titledTF.addActionListener(this);
    titledTF.addFocusListener(this);
    titledPanel.add(titledTF);
    return(titledPanel);
}

/**
 * Called when the border type has changed and the existing card needs to
 * be changed.  This method makes fields visible or invisible
 * depending on the fields that are needed for the current border type.
 */
private void cardChanging()
{
    switch (currentBorderType) {
    case EMPTY:
        matteColorLabel.setVisible(false);
        matteColorEditor.setVisible(false);
        break;
    case BEVEL:
        break;
    case ETCHED:
        break;
    case LINE:
        break;
    case MATTE:
        matteColorLabel.setVisible(true);
        matteColorEditor.setVisible(true);
        break;
    case SOFTBEVEL:
        break;
    case TITLED:
        break;
    }
}

/**
 * Called to initialize the data fields with the correct values for the
 * current border.
 */
private void updateCard(Border border, JComponent borderOwner)
{
    // Empty Border -- MatteBorder is a special case of EmptyBorder
    if (border instanceof EmptyBorder) {
        EmptyBorder emptyBorder = (EmptyBorder)border;
        Insets emptyInsets = emptyBorder.getBorderInsets(null);
        topTF.setText("" + emptyInsets.top);
        leftTF.setText("" + emptyInsets.left);
        bottomTF.setText("" + emptyInsets.bottom);
        rightTF.setText("" + emptyInsets.right);

        if (border instanceof MatteBorder) {
            borderCB.setSelectedIndex(MATTE);
        }
        else {
            borderCB.setSelectedIndex(EMPTY);
        }
        // Bevel Border -- Soft Bevel is a special case of Bevel
    }
    else if (border instanceof BevelBorder) {
        BevelBorder bevelBorder = (BevelBorder)border;

        Color bevelHighlightColor =
            bevelBorder.getHighlightInnerColor(borderOwner);
        Color bevelShadowColor = bevelBorder.getShadowOuterColor(borderOwner);
        highlightColorEditor.setValue(bevelHighlightColor);
        shadowColorEditor.setValue(bevelShadowColor);

        int bevelType = bevelBorder.getBevelType();
        if (bevelType == BevelBorder.RAISED) {
            bevelRaisedRB.setSelected(true);
        }
        else {
            bevelLoweredRB.setSelected(true);
        }

        if (border instanceof SoftBevelBorder) {
            borderCB.setSelectedIndex(SOFTBEVEL);
        }
        else {
            borderCB.setSelectedIndex(BEVEL);
        }
    }
    else if (border instanceof EtchedBorder) {
        EtchedBorder etchedBorder = (EtchedBorder)border;

        Color etchedHighlightColor =
            etchedBorder.getHighlightColor(borderOwner);
        Color etchedShadowColor = etchedBorder.getShadowColor(borderOwner);
        highlightColorEditor.setValue(etchedHighlightColor);
        shadowColorEditor.setValue(etchedShadowColor);

        int etchedType = etchedBorder.getEtchType();
        if (etchedType == EtchedBorder.RAISED) {
            bevelRaisedRB.setSelected(true);
        }
        else {
            bevelLoweredRB.setSelected(true);
        }
        borderCB.setSelectedIndex(ETCHED);
    }
    else if (border instanceof LineBorder) {
        LineBorder lineBorder = (LineBorder)border;
        thickTF.setText("" + lineBorder.getThickness());
        lineColorEditor.setValue(lineBorder.getLineColor());
        borderCB.setSelectedIndex(LINE);
    }
    else if (border instanceof TitledBorder) {
        titledTF.setText(((TitledBorder)currentBorder).getTitle());
        borderCB.setSelectedIndex(TITLED);
    }

}

/**
 * Set the value of the current border.
 */
public void setValue(Object value)
{
    if (value instanceof Border) {
        if (currentBorder == null ||
            !currentBorder.equals(value))
        {
            currentBorder = (Border)value;
            updateCard((Border)value, this);
        }
    }
    else if (value instanceof JComponent) {
        JComponent component = (JComponent)value;
        Border newBorder = component.getBorder();
        if ((currentBorder == null && newBorder != null) ||
            (currentBorder != null && newBorder == null) ||
            (currentBorder != null && newBorder != null &&
            !currentBorder.equals(newBorder)))
        {
            currentBorder = newBorder;
            updateCard(newBorder, component);
        }
    }
}

/**
 * Get the value of the current border.
 */
public Object getValue()
{
    return(currentBorder);
}

/**
 * ItemListener method.  When border type combo box has changed, update the
 * current border and change the current card.
 */
public void itemStateChanged(ItemEvent ev)
{
    Object src = ev.getSource();
    if (src == borderCB && ev.getStateChange() == ItemEvent.SELECTED) {
        currentBorderType = borderCB.getSelectedIndex();
        CardLayout cl = (CardLayout)subPanel.getLayout();
        changeBorder();
        cardChanging();
        cl.show(subPanel, border_cards[currentBorderType]);
    }
}

/**
 * ActionListener method.  If UseBorderButton pressed, notify listeners
 * that we have a border for them.  Otherwise, update the current border
 * with newly set parameters.
 */
public void actionPerformed(ActionEvent ev)
{
    changeBorder();
}

/**
 * FocusListener method.  Do nothing.
 */
public void focusGained(FocusEvent ev) {}

/**
 * FocusListener method.  Update the current border with newly set parameters.
 */
public void focusLost(FocusEvent ev)
{
    changeBorder();
}

/**
 * PropertyChangeListener method.  Update the current border with newly
 * set parameters.
 */
public void propertyChange(PropertyChangeEvent ev)
{
    changeBorder();
}

/**
 * Called to create a new border from the selected border type and
 * parameter fields.  Sets it as the current border.
 */
public void changeBorder()
{
    switch (currentBorderType) {
    case EMPTY:
        Insets emptyInsets = getBorderInsets();
        currentBorder =
            BorderFactory.createEmptyBorder(emptyInsets.top, emptyInsets.left,
                                            emptyInsets.bottom,
                                            emptyInsets.right);
        break;
    case BEVEL:
        Color bevelHighlightColor = (Color)highlightColorEditor.getValue();
        Color bevelShadowColor = (Color)shadowColorEditor.getValue();

        int bevelType = getBevelType();
        if (bevelType < 0) {
            bevelType = BevelBorder.LOWERED;
        }
        if (bevelHighlightColor == null || bevelShadowColor == null) {
            currentBorder = BorderFactory.createBevelBorder(bevelType);
        }
        else {
            currentBorder =
                BorderFactory.createBevelBorder(bevelType, bevelHighlightColor,
                                                bevelShadowColor);
        }
/* DAM should handle default case with no colors
        currentBorder =
                BorderFactory.createBevelBorder(bevelType);
*/

        break;
    case ETCHED:
        Color etchedHighlightColor = (Color)highlightColorEditor.getValue();
        Color etchedShadowColor = (Color)shadowColorEditor.getValue();

        int etchedType = getBevelType();
        if (etchedType >= 0) {
            currentBorder = new EtchedBorder(etchedType, etchedHighlightColor,
                                             etchedShadowColor);
        }
        else {
            currentBorder =
                BorderFactory.createEtchedBorder(etchedHighlightColor,
                                                 etchedShadowColor);
        }
/* DAM should handle default case with no colors
        if (etchedType >= 0) {
            currentBorder = new EtchedBorder(etchedType);
        }
        else {
            currentBorder =  BorderFactory.createEtchedBorder();
        }
*/
        break;
    case LINE:
        Color lcolor = (Color)lineColorEditor.getValue();
        int thickness = 0;
        try {
            thickness = Integer.valueOf(thickTF.getText()).intValue();
        }
        catch (NumberFormatException e) {}
        currentBorder = BorderFactory.createLineBorder(lcolor, thickness);
        break;
    case MATTE:
        Color mcolor = (Color)matteColorEditor.getValue();
        Insets matteInsets = getBorderInsets();
        currentBorder =
            BorderFactory.createMatteBorder(matteInsets.top, matteInsets.left,
                                            matteInsets.bottom,
                                            matteInsets.right, mcolor);
        break;
    case SOFTBEVEL:
        Color softBevelHighlightColor = (Color)highlightColorEditor.getValue();
        Color softBevelShadowColor = (Color)shadowColorEditor.getValue();
        int softBevelType = getBevelType();
        if (softBevelType < 0) {
            softBevelType = SoftBevelBorder.LOWERED;
        }
        currentBorder = new SoftBevelBorder(softBevelType,
                                            softBevelHighlightColor,
                                            softBevelShadowColor);
/* DAM should handle default case with no colors
        currentBorder =
            new SoftBevelBorder(softBevelType);
*/
        break;
    case TITLED:
        currentBorder = BorderFactory.createTitledBorder(titledTF.getText());
        break;
    default :
        currentBorder = BorderFactory.createEmptyBorder();
        break;
    }

    if (listeners != null) {
        listeners.firePropertyChange("", null, currentBorder);
    }
}

/**
 * Get the bevel/etched type from the radio buttons that represent it.
 */
private int getBevelType()
{
    int type = -1;
    if (bevelLoweredRB.isSelected()) {
        type = (currentBorderType == ETCHED ? EtchedBorder.LOWERED
                                            : BevelBorder.LOWERED);
    }
    else if (bevelRaisedRB.isSelected()) {
        type = (currentBorderType == ETCHED ? EtchedBorder.RAISED
                                            : BevelBorder.RAISED);
    }
    return(type);
}

/**
 * Get the insets from the text fields that represent it.
 */
private Insets getBorderInsets()
{
    Insets insets = new Insets(0, 0, 0, 0);
    try {
        insets.top = Integer.valueOf(topTF.getText()).intValue();
    }
    catch (NumberFormatException e) {}
    try {
        insets.left = Integer.valueOf(leftTF.getText()).intValue();
    }
    catch (NumberFormatException e) {}
    try {
        insets.bottom = Integer.valueOf(bottomTF.getText()).intValue();
    }
    catch (NumberFormatException e) {}
    try {
        insets.right = Integer.valueOf(rightTF.getText()).intValue();
    }
    catch (NumberFormatException e) {}
    return(insets);
}

/**
 * Set the background color of the contained text fields
 */
public void setBackground(java.awt.Color color)
{
//DAM do something here ?
}

/**
 * Support for border property change listeners.
 */
protected PropertyChangeSupport listeners = null;

/**
 * Remove property listener.
 */
public void removePropertyChangeListener(PropertyChangeListener listener)
{
    if (listeners == null) {
        return;
    }
    listeners.removePropertyChangeListener(listener);
}

/**
 * Add property listener.
 */
public void addPropertyChangeListener(PropertyChangeListener listener)
{
    if (listeners == null) {
        listeners = new PropertyChangeSupport(this);
    }
    listeners.addPropertyChangeListener(listener);
}

}
