/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: BorderStringEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:41 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.beans;

import com.klg.jclass.util.swing.JCSwingTypeConverter;

import javax.swing.border.Border;
import java.awt.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyEditor;

/**
 * Bean editor for setting border via string values.  Uses the
 * JClass BorderEditorPanel as the actual border choosing component.
 */
public class BorderStringEditor implements PropertyEditor, 
									       PropertyChangeListener
{

static {
	JCBeanInfo.getLocaleManager().add(
		"com.klg.jclass.beans.resources.LocaleInfo");
}

public final static String BORDER_EDITOR = "Border Editor";

protected PropertyChangeSupport  support = new PropertyChangeSupport( this );
protected String                 target = ""; // Actual object that gets edited
protected BorderEditorPanel      borderChooser = null;

public void setValue(Object value)
{
    target = (String)value;
}

public Object getValue()
{
    return(target);
}

public boolean isPaintable()
{
    return(true);
}

public void paintValue(Graphics gc, Rectangle box)
{
    FontMetrics fm = gc.getFontMetrics();
    int vpad = (box.height - fm.getAscent()) / 2;
    gc.drawString(JCBeanInfo.getLocaleManager().getString(BORDER_EDITOR),
                  box.x + 2, box.y + box.height - vpad);
}

public String getAsText()
{
    return(target);
}

public void setAsText(String t)
{
    target = t;
    if (support != null) {
        support.firePropertyChange("", null, getValue());
    }
}

public String[] getTags()
{
    return(null);
}

public Component getCustomEditor()
{
    if (borderChooser == null) {
        borderChooser = new BorderEditorPanel();
        borderChooser.addPropertyChangeListener(this);
    }
    borderChooser.setValue(JCSwingTypeConverter.toBorder(target));
    return(borderChooser);
}

/* For testing purposes only
public static void main(String args[])
{
    javax.swing.JFrame frame;
    try {
        String s = "com.klg.jclass.chart.customizer.JCustomizerCustomFrame";
        java.lang.Class newclass = java.lang.Class.forName(s);
        frame = (javax.swing.JFrame)newclass.newInstance();
    }
    catch (Exception ex) {
        System.out.println(ex);
        frame = new javax.swing.JFrame();
    }

    BorderStringEditor dataSourceEditor = new BorderStringEditor();

    dataSourceEditor.setValue(null);
    frame.getContentPane().add(dataSourceEditor.getCustomEditor());
    frame.pack();
    frame.setVisible(true);
}
*/

public boolean supportsCustomEditor()
{
    return(true);
}

public String getJavaInitializationString()
{
    return("\"" + target + "\"");
}

public void addPropertyChangeListener(PropertyChangeListener l)
{
    support.addPropertyChangeListener(l);
}

public void removePropertyChangeListener(PropertyChangeListener l)
{
    support.removePropertyChangeListener(l);
}

/** Border has been changed. */
public void propertyChange(PropertyChangeEvent ev)
{
    Object src = ev.getSource();
    if (src == borderChooser) {
        String newTarget =
            JCSwingTypeConverter.fromBorder((Border)borderChooser.getValue(),
                                            borderChooser);
        if (!target.equals(newTarget)) {
            target = newTarget;
            support.firePropertyChange("", null, null);
        }
    }
}

}
