/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ColorEditorPanel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:41 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.beans;

import com.klg.jclass.util.swing.JCIconCreator;
import com.klg.jclass.util.JCLocaleManager;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.StringTokenizer;

/**
 *  Color editor component based on JPanel.
 */
public class ColorEditorPanel extends JPanel implements ActionListener {
	
//Localization keys
final public static String COLOR_INHERIT =   "inherit";
final public static String COLOR_BLACK =     "black";
final public static String COLOR_BLUE =      "blue";
final public static String COLOR_CYAN =      "cyan";
final public static String COLOR_DARKGRAY =  "darkGray";
final public static String COLOR_GRAY =      "gray";
final public static String COLOR_GREEN =     "green";
final public static String COLOR_LIGHTGRAY = "lightGray";
final public static String COLOR_MAGENTA =   "magenta";
final public static String COLOR_ORANGE =    "orange";
final public static String COLOR_PINK =      "pink";
final public static String COLOR_RED =       "red";
final public static String COLOR_WHITE =     "white";
final public static String COLOR_YELLOW =    "yellow";

private String         inherit_string = "";

private JButton	       colorButton = null;
private JPanel         colorLabel = null;
private JTextField     colorField = null;

private JColorChooser  colorChooser = null;
private JDialog	       colorChooserDialog = null;

/**
 * Default constructor
 */
public ColorEditorPanel()
{
    this(null);
}

/**
 * Constructor taking a color value
 */
public ColorEditorPanel(Color c)
{
    setLayout(new GridBagLayout());
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.gridwidth = 3;
    gbc.gridx = GridBagConstraints.RELATIVE;
    gbc.gridy = 0;
    gbc.anchor = GridBagConstraints.WEST;
    gbc.insets = new Insets(0, 3, 0, 3);

    colorField = new JTextField("", 15);
    colorField.addActionListener(this);
    add(colorField, gbc);
    colorLabel = new JPanel();
    colorLabel.setBorder(BorderFactory.createEmptyBorder(6, 6, 6, 6));
    add(colorLabel, gbc);
    colorButton = new JButton(new ImageIcon(rainbowImage()));
    colorButton.addActionListener(this);
    add(colorButton, gbc);

    String bundle_name = "com.klg.jclass.beans.resources.LocaleInfo";
    JCBeanInfo.getLocaleManager().add(bundle_name);
    inherit_string = JCBeanInfo.getLocaleManager().getString(COLOR_INHERIT);

    setValue(c);
}

/**
 * Set (or change) the object that is to be edited.
 *
 * @param value The new target object to be edited.  Note that this
 * object should not be modified by the PropertyEditor, rather
 * the PropertyEditor should create a new object to hold any
 * modified value.
 */
public void setValue(Object value)
{
    if (value == null) {
        colorField.setText(inherit_string);
        colorLabel.setBackground(null);
        colorLabel.paintImmediately(new Rectangle(colorLabel.getWidth(),
                                                  colorLabel.getHeight()));
    }
    else if (value instanceof Color) {
        Color color = ((Color)value);
        String colorName = getColorName(color);

        colorField.setText(colorName);
        colorLabel.setBackground(color);
        colorLabel.paintImmediately(new Rectangle(colorLabel.getWidth(),
                                                  colorLabel.getHeight()));
    }
    else if (value instanceof String) {
        String colorName = (String)value;
        Color color = toColor(colorName);
        if (color != null) {
            colorField.setText(colorName);
            colorLabel.setBackground(color);
            colorLabel.paintImmediately(new Rectangle(colorLabel.getWidth(),
                                                      colorLabel.getHeight()));
        }
    }
}

/**
 * @return The value of the property.  Builtin types such as "int" will
 * be wrapped as the corresponding object type such as "java.lang.Integer".
 */
public Object getValue()
{
    return(colorLabel.getBackground());
}

/**
 * A PropertyEditor may choose to make available a full custom Component
 * that edits its property value.  It is the responsibility of the
 * PropertyEditor to hook itself up to its editor Component itself and
 * to report property value changes by firing a PropertyChange event.
 * <P>
 * The higher-level code that calls getCustomEditor may either embed
 * the Component in some larger property sheet, or it may put it in
 * its own individual dialog, or ...
 *
 * @return A java.awt.Component that will allow a human to directly
 *      edit the current property value.  May be null if this is
 *      not supported.
 */
public Component getCustomEditor()
{
    if (colorChooser == null) {
        colorChooser = new JColorChooser();
        colorChooser.setColor((Color)getValue());
    }
    if (colorChooserDialog == null) {
        colorChooserDialog = JColorChooser.createDialog(colorButton,
                                                        "Choose Color", true,
                                                        colorChooser, this,
                                                        this);
    }
    return(colorChooserDialog);
}

/**
 * ActionListener method.  Deals with selection of colors from within editors
 * subcomponents.
 */
public void actionPerformed(ActionEvent ev)
{
    if (ev.getSource() == colorField) {
        String colorName = colorField.getText();
        Color color = toColor(colorName);

        if (color == null                      ||
            colorName.equalsIgnoreCase("null") ||
            colorName.indexOf(inherit_string) >= 0)
        {
            colorField.setText(inherit_string);
            colorLabel.setBackground(null);
            if (listeners != null) {
                listeners.firePropertyChange("", null, null);
            }
        }
        else {
            colorLabel.setBackground(color);
            if (listeners != null) {
                listeners.firePropertyChange("", null, getValue());
            }
        }
        colorLabel.paintImmediately(new Rectangle(colorLabel.getWidth(),
                                                  colorLabel.getHeight()));
    }
    else if (ev.getSource() == colorButton) {
        if (colorChooserDialog == null) {
            colorChooserDialog = (JDialog)getCustomEditor();
        }
        if (colorChooser != null) {
            colorChooser.setColor((Color)getValue());
        }
        colorChooserDialog.show();
    }
    else if (ev.getSource() instanceof JButton &&
             ev.getActionCommand().compareTo("OK") == 0)
    {
        Color color = colorChooser.getColor();

        colorField.setText(getColorName(color));
        colorLabel.setBackground(color);
        colorLabel.paintImmediately(new Rectangle(colorLabel.getWidth(),
                                                  colorLabel.getHeight()));
        if (listeners != null) {
            listeners.firePropertyChange("", null, getValue());
        }
    }
}

/**
 * Support for property listeners.
 */
protected PropertyChangeSupport listeners = null;

public void removePropertyChangeListener(PropertyChangeListener listener)
{
    if (listeners == null) {
        return;
    }
    listeners.removePropertyChangeListener(listener);
}

public void addPropertyChangeListener(PropertyChangeListener listener)
{
    if (listeners == null) {
        listeners = new PropertyChangeSupport(this);
    }
    listeners.addPropertyChangeListener(listener);
}

/*
* Convert a Color to a nicely readable string
*/
private String getColorName(Color color)
{
    JCLocaleManager lm = JCBeanInfo.getLocaleManager();
    if (color == null) {
        return(inherit_string);
    }
    else if (color.equals(Color.black)) {
        return lm.getString(COLOR_BLACK);
    }
    else if (color.equals(Color.blue)) {
        return lm.getString(COLOR_BLUE);
    }
    else if (color.equals(Color.cyan)) {
        return lm.getString(COLOR_CYAN);
    }
    else if (color.equals(Color.darkGray)) {
        return lm.getString(COLOR_DARKGRAY);
    }
    else if (color.equals(Color.gray)) {
        return lm.getString(COLOR_GRAY);
    }
    else if (color.equals(Color.green)) {
        return lm.getString(COLOR_GREEN);
    }
    else if (color.equals(Color.lightGray)) {
        return lm.getString(COLOR_LIGHTGRAY);
    }
    else if (color.equals(Color.magenta)) {
        return lm.getString(COLOR_MAGENTA);
    }
    else if (color.equals(Color.orange)) {
        return lm.getString(COLOR_ORANGE);
    }
    else if (color.equals(Color.pink)) {
        return lm.getString(COLOR_PINK);
    }
    else if (color.equals(Color.red)) {
        return lm.getString(COLOR_RED);
    }
    else if (color.equals(Color.white)) {
        return lm.getString(COLOR_WHITE);
    }
    else if (color.equals(Color.yellow)) {
        return lm.getString(COLOR_YELLOW);
    }
    else {
        return "" + color.getRed() + "-" + color.getGreen() + "-" +
               color.getBlue();
    }
}

/**
 * Converts a string to a Color.
 * @param s A color enum ("black", "pink", etc.);
 * hex value in format "#RRGGBB", "#RRRRGGGGBBBB" or "#N";
 * or rgb values in format "r-g-b"
 */
private static Color toColor(String s)
{
    if (s == null) {
        return(null);
    }
    StringTokenizer st = new StringTokenizer(s);
    String token = st.nextToken("-");
    if (token == null) {
        return(null);
    }
    token = token.trim();

    JCLocaleManager lm = JCBeanInfo.getLocaleManager();
    if (token.equalsIgnoreCase(lm.getString(COLOR_BLACK))) {
        return Color.black;
    }
    if (token.equalsIgnoreCase(lm.getString(COLOR_BLUE))) {
        return Color.blue;
    }
    if (token.equalsIgnoreCase(lm.getString(COLOR_CYAN))) {
        return Color.cyan;
    }
    if (token.equalsIgnoreCase(lm.getString(COLOR_DARKGRAY))) {
        return Color.darkGray;
    }
    if (token.equalsIgnoreCase(lm.getString(COLOR_GRAY))) {
        return Color.gray;
    }
    if (token.equalsIgnoreCase(lm.getString(COLOR_GREEN))) {
        return Color.green;
    }
    if (token.equalsIgnoreCase(lm.getString(COLOR_LIGHTGRAY))) {
        return Color.lightGray;
    }
    if (token.equalsIgnoreCase(lm.getString(COLOR_MAGENTA))) {
        return Color.magenta;
    }
    if (token.equalsIgnoreCase(lm.getString(COLOR_ORANGE))) {
        return Color.orange;
    }
    if (token.equalsIgnoreCase(lm.getString(COLOR_PINK))) {
        return Color.pink;
    }
    if (token.equalsIgnoreCase(lm.getString(COLOR_RED))) {
        return Color.red;
    }
    if (token.equalsIgnoreCase(lm.getString(COLOR_WHITE))) {
        return Color.white;
    }
    if (token.equalsIgnoreCase(lm.getString(COLOR_YELLOW))) {
        return Color.yellow;
    }
    try {
        // Parse a hex string
        if (token.startsWith("#")) {
            if (token.length() >= 13) {
                return new Color(Integer.valueOf(token.substring(1, 5),
                                                 16).intValue() / 256,
                                 Integer.valueOf(token.substring(5, 9),
                                                 16).intValue() / 256,
                                 Integer.valueOf(token.substring(9, 13),
                                                 16).intValue() / 256);
            }
            else if (token.length() >= 7) {
                return new Color(Integer.valueOf(token.substring(1, 3),
                                                 16).intValue(),
                                 Integer.valueOf(token.substring(3, 5),
                                                 16).intValue(),
                                 Integer.valueOf(token.substring(5, 7),
                                                 16).intValue());
            }
            else {
                return new Color(Integer.valueOf(token.substring(1), 16).intValue());
            }
        }

        // Otherwise parse red-green-blue values
        int r = Integer.parseInt(token);
        int g = Integer.parseInt(st.nextToken("-"));
        int b = Integer.parseInt(st.nextToken("-"));
        return(new Color(r, g, b));
    }
    catch (Exception e) {
        return(null);
    }
}

private static String[] rainbow = {
    "rrrr gggg bbbb",
    "rrrr gggg bbbb",
    "rrrr gggg bbbb",
    "rrrr gggg bbbb",
    "rrrr gggg bbbb",
    "rrrr gggg bbbb",
    "rrrr gggg bbbb",
    "rrrr gggg bbbb",
    "              ",
    "              ",
    "              ",
    "  ..  ..  ..  ",
    "  ..  ..  ..  "
};

/**
 * Creates the Image used in the Customer Editor button
 */
private static Image rainbowImage()
{
    JCIconCreator ic = new JCIconCreator(rainbow[0].length(), rainbow.length);
    ic.setColor('.', new Color(0x000000));
    ic.setColor('r', new Color(0xff0000));
    ic.setColor('g', new Color(0x00ff00));
    ic.setColor('b', new Color(0x0000ff));
    ic.setPixels(rainbow);
    return(ic.getIcon().getImage());
}

}
