/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ColorStringEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:41 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.beans;

import com.klg.jclass.util.swing.JCSwingTypeConverter;

import javax.swing.*;
import javax.swing.colorchooser.ColorSelectionModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyEditor;

/**
 * Bean editor for setting colors via string values.  Uses the
 * Swing JColorChooser as the actual color choosing component.
 */

public class ColorStringEditor implements PropertyEditor, ChangeListener {

protected PropertyChangeSupport  support = new PropertyChangeSupport( this );
protected String                 target = ""; // Actual object that gets edited
protected JColorChooser          colorChooser = null;

public void setValue(Object value)
{
    target = (String)value;
}

public Object getValue()
{
    return(target);
}

public boolean isPaintable()
{
    return(true);
}

public void paintValue(Graphics gc, Rectangle box)
{
    Color col = gc.getColor();
    gc.setColor(JCSwingTypeConverter.toColor(target));
    gc.fillRect(box.x, box.y, box.width, box.height);
    gc.setColor(col);
    gc.drawRect(box.x, box.y, box.width, box.height);
}

public String getAsText()
{
    return(target);
}

public void setAsText(String t)
{
    target = t;
    if (support != null) {
        support.firePropertyChange("", null, getValue());
    }
}

public String[] getTags()
{
    return(null);
}

public Component getCustomEditor()
{
    if (colorChooser == null) {
        colorChooser = new JColorChooser();
        Color currentColor = JCSwingTypeConverter.toColor(target);
        if (currentColor != null) {
            colorChooser.setColor(currentColor);
        }
        colorChooser.getSelectionModel().addChangeListener(this);
    }
    return(colorChooser);
}

/* For testing purposes only
public static void main(String args[])
{
    JFrame frame;
    try {
        String s = "com.klg.jclass.chart.customizer.JCustomizerCustomFrame";
        java.lang.Class newclass  = java.lang.Class.forName(s);
        frame = (JFrame) newclass.newInstance();
    }
    catch (Exception ex) {
        System.out.println(ex);
        frame = new JFrame();
    }

    ColorStringEditor dataSourceEditor = new ColorStringEditor();
    dataSourceEditor.setValue(null);

    frame.getContentPane().add( dataSourceEditor.getCustomEditor() );
    frame.pack();
    frame.setVisible(true);
}
*/

public boolean supportsCustomEditor()
{
    return(true);
}

public String getJavaInitializationString()
{
    return("\"" + target + "\"");
}

public void addPropertyChangeListener(PropertyChangeListener l)
{
    support.addPropertyChangeListener(l);
}

public void removePropertyChangeListener(PropertyChangeListener l)
{
    support.removePropertyChangeListener(l);
}

public void stateChanged(ChangeEvent ev)
{
    Object src = ev.getSource();
    ColorSelectionModel selectionModel = colorChooser.getSelectionModel();
    if (src == selectionModel) {
        String newTarget =
            JCSwingTypeConverter.fromColor(selectionModel.getSelectedColor());
        if (!target.equals(newTarget)) {
            target = newTarget;
            support.firePropertyChange("", null, null);
        }
    }
}

}
