/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: EnumEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:41 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.beans;

import com.klg.jclass.util.JCTypeConverter;
import java.beans.*;

/**
 * This is an abstract class to use in property editors and customizers,
 * when an enumerated list of values is required.
 * It maps a list of strings to a list of enumerated values.
 */
public abstract class EnumEditor extends PropertyEditorSupport {

protected String[]  strings;               // strings for code generation
protected String[]  i18n_strings;          // strings to appear in visual editor
                                           // (i.e. combo box)
protected int[]     values;
protected String    prepend = "";
protected boolean   upperCaseEnum = true;

/**
 * Constructor
 * @param strings the list of enumerated values to display as Strings and
 *                to use for code generation
 * @param values the list of enumerated values as ints
 */
public EnumEditor(String[] strings, int[] values)
{
    this.strings = strings;
    this.i18n_strings = strings;
    this.values = values;
}

/**
 * Constructor
 * @param strings the list of enumerated values to display as Strings and
 *                to use for code generation
 * @param values the list of enumerated values as ints
 * @param prepend a String value to prepend when generating code
 */
public EnumEditor(String[] strings, int[] values, String prepend)
{
    this(strings, values);
    this.prepend = prepend;
}

/**
 * Constructor
 * @param strings the list of enumerated values to use for code generation
 * @param values the list of enumerated values as ints
 * @param i18n_strings the list of internationalized enumerated values
 *                     to display as Strings
 * @param prepend a String value to prepend when generating code
 */
public EnumEditor(String[] strings, int[] values, String[] i18n_strings,
                  String prepend)
{
    this.strings = strings;
    this.values = values;
    this.i18n_strings = i18n_strings;
    this.prepend = prepend;
}

/*
* Sets the enum's value from the passed String value
* @param text the new value as a String
*/
public void setAsText(String text) throws IllegalArgumentException
{
    int v = JCTypeConverter.toEnum(text, i18n_strings, values, -999);
    if (v == -999) {
        throw new IllegalArgumentException("invalid enum: " + text);
    }
    setValue(new Integer(v));
}

/*
* Returns the enum's value as a String
*/
public String getAsText()
{
    return(JCTypeConverter.fromEnum(((Integer)getValue()).intValue(),
                                    i18n_strings, values));
}

public String[] getTags()
{
    return(i18n_strings);
}

public String getJavaInitializationString()
{
    if (getValue() == null) {
        return("NULL");
    }
    // Code generation uses english string array, not internationalized one
    String codeText =
        JCTypeConverter.fromEnum(((Integer)getValue()).intValue(),
                                 strings, values);
    if (upperCaseEnum) {
        return(prepend + codeText.toUpperCase());
    }
    else {
        return(prepend + codeText);
    }
}

}

