/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: FontChooserEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:41 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.beans;

import com.klg.jclass.util.swing.JCFontChooser;
import com.klg.jclass.util.swing.JCFontChooserPane;
import com.klg.jclass.util.swing.JCFontEvent;
import com.klg.jclass.util.swing.JCFontListener;

import javax.swing.*;
import java.awt.*;

/**
 * Bean editor for choosing fonts implemented using the
 * com.klg.jclass.util.swing.JCFontChooser class.
 */
public class FontChooserEditor extends AbstractBeanEditor
                               implements JCFontListener
{

public static final String FONT_CHOOSER_HELP = "Font Chooser";

protected Font               font;
protected boolean            underline;
protected JCFontChooserPane  fontChooser;

public FontChooserEditor()
{
    super("com.klg.jclass.beans.resources.LocaleInfo", FONT_CHOOSER_HELP,
          "Font Editor");
}

/**
 * Returns the actual editor component which will be wrapped with the html
 * help pane.
 */
public Component getEditorComponent()
{
    // Add the style checkboxes
    JPanel panel = new JPanel();
    panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
    fontChooser = new JCFontChooserPane();
    fontChooser.addJCFontListener(this);
    panel.add(fontChooser);
    return(panel);
}

/**
 * Returns the code generation string.
 */
public String getJavaInitializationString()
{
    if (font == null) {
        return("null");
    }
    if (underline) {
        return("com.klg.jclass.util.swing.JCFontChooser" +
               ".setUnderline(new java.awt.Font(\"" + font.getFontName() +
               "\", " + font.getStyle() + ", " + font.getSize() + ")," +
               fontChooser.isUnderline() + ")");
    }
    else {
        return("new java.awt.Font(\"" + font.getFontName() + "\", " +
               (font.getStyle() & ~JCFontChooserPane.UNDERLINE) + ", " +
               font.getSize() + ")");
    }
}

/**
 * Returns the value of the property.
 */
public Object getValue()
{
    if (font != null ) {
        return(JCFontChooser.setUnderline(font, fontChooser.isUnderline()));
    }
    return null;
}

/**
 * Returns a text representation of the value.
 */
protected String getValueAsText()
{
    return(null);
}

/**
 * Sets the value of the property.
 */
public void setValue(Object o)
{
    if (o == null) {
        font = null;
    }
    else {
        Font f = (Font)o;
        font = new Font(f.getAttributes());
        fontChooser.setSelectedFont(font);
        underline = fontChooser.isUnderline();
    }
}

public void fontChanging(JCFontEvent e) {}

/** Font has been changed. */
public void fontChanged(JCFontEvent e)
{
    if (e.getSource() == fontChooser) {
        font = fontChooser.getSelectedFont();
        underline = fontChooser.isUnderline();
        support.firePropertyChange(null, null, null);
    }
}

public static void main(String[] args)
{
    JFrame f = new JFrame();
    f.getContentPane().add(new FontChooserEditor());
    f.pack();
    f.show();
}

}
