/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: FontStringEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:41 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.beans;

import com.klg.jclass.util.swing.JCFontChooserPane;
import com.klg.jclass.util.swing.JCFontEvent;
import com.klg.jclass.util.swing.JCFontListener;
import com.klg.jclass.util.swing.JCSwingTypeConverter;

import java.awt.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyEditor;

/**
 * Bean editor for setting fonts via string values.  Uses the
 * JClass JCFontChooser as the actual font choosing component.
 */
public class FontStringEditor implements PropertyEditor, 
									     JCFontListener
{

static {
    JCBeanInfo.getLocaleManager().add(
        "com.klg.jclass.beans.resources.LocaleInfo");
}

public final static String FONT_EDITOR	= "Font Editor";

protected PropertyChangeSupport  support = new PropertyChangeSupport( this );
protected String                 target = ""; // Actual object that gets edited
protected JCFontChooserPane      fontChooser = null;

public void setValue(Object value)
{
    target = (String)value;
}

public Object getValue()
{
    return(target);
}

public boolean isPaintable()
{
    return(true);
}

public void paintValue(Graphics gc, Rectangle box)
{
    FontMetrics fm = gc.getFontMetrics();
    int vpad = (box.height - fm.getAscent()) / 2;
    gc.drawString(JCBeanInfo.getLocaleManager().getString(FONT_EDITOR),
                  box.x + 2, box.y + box.height - vpad);
}

public String getAsText()
{
    return(target);
}

public void setAsText(String t)
{
    target = t;
    if (support != null) {
        support.firePropertyChange("", null, getValue());
    }
}

public String[] getTags()
{
    return(null);
}

public Component getCustomEditor()
{
    if (fontChooser == null) {
        fontChooser = new JCFontChooserPane();
        fontChooser.addJCFontListener(this);
        fontChooser.setStyleControls(JCFontChooserPane.BOLD |
                                     JCFontChooserPane.ITALIC);
    }
    fontChooser.setSelectedFont(JCSwingTypeConverter.toFont(target));
    return(fontChooser);
}

/* For testing purposes only
public static void main(String args[])
{
    javax.swing.JFrame frame;
    try {
        String s = "com.klg.jclass.chart.customizer.JCustomizerCustomFrame";
        java.lang.Class newclass = java.lang.Class.forName(s);
        frame = (javax.swing.JFrame) newclass.newInstance();
    }
    catch (Exception ex) {
        System.out.println(ex);
        frame = new javax.swing.JFrame();
    }

    FontStringEditor dataSourceEditor = new FontStringEditor();

    dataSourceEditor.setValue(null);
    frame.getContentPane().add(dataSourceEditor.getCustomEditor());
    frame.pack();
    frame.setVisible(true);
}
*/

public boolean supportsCustomEditor()
{
    return(true);
}

public String getJavaInitializationString()
{
    return("\"" + target + "\"");
}

public void addPropertyChangeListener(PropertyChangeListener l)
{
    support.addPropertyChangeListener(l);
}

public void removePropertyChangeListener(PropertyChangeListener l)
{
    support.removePropertyChangeListener(l);
}

public void fontChanging(JCFontEvent e) {}

/** Font has been changed. */
public void fontChanged(JCFontEvent e)
{
    if (e.getSource() == fontChooser) {
        String newTarget =
            JCSwingTypeConverter.fromFont(fontChooser.getSelectedFont());
        if (!target.equals(newTarget)) {
            target = newTarget;
            support.firePropertyChange("", null, null);
        }
    }
}

}
