/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCBeanInfo.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:41 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.beans;


import com.klg.jclass.util.JCLocaleManager;

import java.beans.*;
import java.util.Vector;

/**
 * Class to implement Bean info for JClass components.
 * <p>This class can be used as a complete Bean definition.  All the properties
 * of the Bean are defined in the list of <code>JCProperties</code> passed to
 * the constructor.
 * <p>To use an objects superclass <code>BeanInfo</code> as well, use the
 * <code>JCSuperBeanInfo</code> class instead of this one.
 */
public class JCBeanInfo extends SimpleBeanInfo {

/** Constant for use when specifying the modifier in the
 * <code>JCPropertyDescriptor</code>. */
public static final int NORMAL = JCPropertyDescriptor.NORMAL;
/** Constant for use when specifying the modifier in the
 * <code>JCPropertyDescriptor</code>. */
public static final int EXPERT = JCPropertyDescriptor.EXPERT;
/** Constant for use when specifying the modifier in the
 * <code>JCPropertyDescriptor</code>>. */
public static final int HIDDEN = JCPropertyDescriptor.HIDDEN;

	//
	// List of Bean properties and Events that we provide locale information
	// for.
	//
public static final String CONTAINER_EVENT = "Container";

public static final JCEventSetDescriptor CONTAINER_EVENTSET
    = new JCEventSetDescriptor(CONTAINER_EVENT, "java.awt.event",
							   new String[] {"componentAdded",
											 "componentRemoved"});

/** Suffix for use with locale key. */
static public final String DISPLAY_NAME_SUFFIX = "DisplayName";
/** Suffix for use with locale key. */
static public final String SHORT_DESCRIPTION_SUFFIX = "ShortDescription";

/** Array index location of the mono 16 x 16 icon. */
public static final int MONO16  = 0;
/** Array index location of the color 16 x 16 icon. */
public static final int COLOR16 = 1;
/** Array index location of the mono 32 x 32 icon. */
public static final int MONO32  = 2;
/** Array index location of the color 32 x 32 icon. */
public static final int COLOR32 = 3;

/** Creates a locale bundle manager for all the Bean properties. */
public static final JCLocaleManager localeManager =
    new JCLocaleManager("com.klg.jclass.beans.resources.BeanLocaleInfo");

protected String                name;
protected JCPropertyDescriptor  properties[];
protected JCEventSetDescriptor  events[];
protected String                icons[];
protected Class                 beanClass;

public static JCLocaleManager getLocaleManager()
{
	return(localeManager);
}

/**
 * @param properties list of properties the Bean wants exposed
 * @param events list of events postable by the Bean
 * @param resource_bundle the name of the bundle from which to extract
 * localized text; this should be relative to the <code>BeanInfo</code>'s
 * package, unless the name begins with a ":"
 * @param icons Array of String paths to icons; paths are relative to the Bean's
 * location in the jar, UNLESS the "path" begins with a "/".  Relative order of
 * the icons should be <code>MONO16x16</code>, <code>COLOR16x16</code>,
 * <code>MONO32x32</code>, or <code>COLOR32x32</CODE>.
 */
public JCBeanInfo(JCPropertyDescriptor properties[],
                  JCEventSetDescriptor events[],
                  String resource_bundle, String icons[])
{
    // Infer the name of the bean we are supporting from the name of this class
    name = this.getClass().getName();
    name = name.substring(0, name.lastIndexOf("BeanInfo"));
    try {
        beanClass = Class.forName(name);
    }
    catch (Exception e) {
        throw new Error(e.toString());
    }
    this.properties = properties;
    this.events = events;
    this.icons = icons;
    // Get current package
    String pkg = name.substring(0, name.lastIndexOf("."));

    // Work out the resource bundle location by prepending the
    // package of the BeanInfo class or by using the full bundle
    // name if it is prepended with a ":"
    String patched_resource_bundle;
    if (resource_bundle.charAt(0) == ':')
        patched_resource_bundle = resource_bundle.substring(1);
    else
        patched_resource_bundle = pkg + "." + resource_bundle;

    try {
        Class.forName(patched_resource_bundle);
    }
    catch (ClassNotFoundException e) {
        /*
        try {
            // We couldn't find the resource bundle as expected so
            // now we try the name without prepending it to see if a full
            // path was specified
            Class.forName(resource_bundle);
            patched_resource_bundle = resource_bundle;
        }
        catch (ClassNotFoundException e2) {
            throw new Error(e2.toString());
        }
        */
        throw new Error(e.toString());

    }
    localeManager.add(patched_resource_bundle);
}

///**
// * This constructor is for internal use only, when we need to construct
// * concrete <CODE>JCBeanInfo</CODE>s.
// * @param events
// * @param icons
// * @param name
// * @param properties
// * @param resource_bundle
// */
//private JCBeanInfo(String name, JCPropertyDescriptor properties[],
//                   JCEventSetDescriptor events[], String resource_bundle,
//                   String icons[])
//{
//    this.name = name;
//    try {
//        beanClass = Class.forName(name);
//    }
//    catch (Exception e) {
//        throw new Error(e.toString());
//    }
//    this.properties = properties;
//    this.events = events;
//    this.icons = icons;
//    localeManager.add(resource_bundle);
//}

/**
 * CreateS the descriptor.
 * @return
 * @param jcpd
 * @throws IntrospectionException
 */
protected PropertyDescriptor createDescriptor(JCPropertyDescriptor jcpd)
    throws IntrospectionException
{
    PropertyDescriptor pd = null;
    try {
        pd = new PropertyDescriptor(jcpd.getName(), beanClass,
                                    jcpd.getGetterName(),
                                    jcpd.getSetterName());
    }
    catch (Exception e) {
        
        pd = null;
    }
    if (pd == null) {
        try {
            // Assume read-only
            pd = new PropertyDescriptor(jcpd.getName(), beanClass,
                                        jcpd.getGetterName(), null);
        }
        catch (Exception e) {
            
            pd = null;
        }
    }

    if (pd == null) {
        return(null);
    }

    pd.setHidden(jcpd.isHidden());
    pd.setExpert(jcpd.isExpert());
    pd.setDisplayName(jcpd.getDisplayName());
    pd.setShortDescription(jcpd.getShortDescription());
    pd.setPropertyEditorClass(jcpd.getPropertyEditorClass());
    return(pd);
}

/**
 * Returns an array of <code>PropertyDescriptors</code> that describe
 * the editable properties supported by this Bean.
 * @return an array of <code>PropertyDescriptors</code> describing the editable
 * properties supported by this Bean
 */
public PropertyDescriptor[] getPropertyDescriptors()
{
    if (properties == null) {
        return(null);
    }

    int offset = 0;
    PropertyDescriptor descriptors[] =
        new PropertyDescriptor[properties.length];
    try {
        for (int i = 0; i < properties.length; i++) {
            PropertyDescriptor descriptor = createDescriptor(properties[i]);
            descriptors[offset + i] = descriptor;
        }
    }
    catch (Exception e) {
        
        throw new Error(e.toString());
    }

    // Reform the property descriptor list, minus null descriptors
    // remove and invalid descriptors
    Vector list = filterPropertyDescriptors(descriptors);

    // Rebuild descriptors based on new accumulated non-null list
    descriptors = new PropertyDescriptor[list.size()];
    for (int i = 0; i < list.size(); i++) {
        descriptors[i] = (PropertyDescriptor)list.elementAt(i);
    }
    return(descriptors);
}

/**
 * This method removes any "null" property descriptors from the list.
 * This is implemented as a separate method so that it can be overridden
 * by subclasses to add or delete additional properties. <br><B>NOTE:</b>
 * <code>super.filterPropertyDescriptors()</code> should always be invoked to
 * ensure that any nulls are removed from the final property list.
 * @return
 * @param descriptors
 */
protected Vector filterPropertyDescriptors(PropertyDescriptor descriptors[])
{
    Vector list = new Vector();
    for (int i = 0; i < descriptors.length; i++) {
        // Possible property descriptor failure - skip it
        if (descriptors[i] != null) {
            list.addElement(descriptors[i]);
        }
    }
    return(list);
}

/**
 * Returns an array of <code>EventSetDescriptors</code> that describe the
 * kinds of events fired by this Bean.
 * @return  an array of <code>EventSetDescriptors</code> describing the kinds of
 * events fired by this Bean
 */
public EventSetDescriptor[] getEventSetDescriptors()
{
    if (events == null) {
        
        return(null);
    }
    

    Vector lv = new Vector();
    for (int i = 0; i < events.length; i++) {
        
        
        
        
        
        
        
        try {
            EventSetDescriptor esd =
                new EventSetDescriptor(beanClass, events[i].getName(),
                                       events[i].getListenerClass(),
                                       events[i].getListenerMethods(),
                                       events[i].getAddListenerMethod(),
                                       events[i].getRemoveListenerMethod());
            lv.addElement(esd);
        }
        catch (Exception e) {
            
        }
    }
    if (lv == null || lv.size() <= 0) {
        
        return(null);
    }

    EventSetDescriptor[] list = new EventSetDescriptor[lv.size()];
    for (int j = 0; j < lv.size(); j++) {
        list[j] = (EventSetDescriptor)lv.elementAt(j);
    }
    return(list);
}

/**
 * Gets the specified icon image.
 * @return
 * @param iconKind
 */
public java.awt.Image getIcon(int iconKind)
{
    String resource_name = null;

    switch (iconKind) {
    case ICON_MONO_16x16:
        resource_name = icons[MONO16];
        break;
    case ICON_COLOR_16x16:
        resource_name = icons[COLOR16];
        break;
    case ICON_MONO_32x32:
        resource_name = icons[MONO32];
        break;
    case ICON_COLOR_32x32:
        resource_name = icons[COLOR32];
        break;
    }

    if (resource_name != null) {
        return(loadImage(resource_name));
    }

    return(null);
}

/**
 *  Merges property arrays.  Handles null arrays as gracefully as possible.
 * @param subProps
 * @param theseProps
 * @return
 */
public static JCPropertyDescriptor[]
    mergeProperties(JCPropertyDescriptor[] theseProps,
                    JCPropertyDescriptor[] subProps)
{
    if (subProps == null) {
        return(theseProps);
    }

    if (theseProps == null) {
        return(subProps);
    }

    try {
        JCPropertyDescriptor[] thisArray =
            new JCPropertyDescriptor[theseProps.length + subProps.length];
        System.arraycopy(theseProps, 0, thisArray, 0, theseProps.length);
        System.arraycopy(subProps, 0,
                         thisArray, theseProps.length, subProps.length);

        return(thisArray);
    }
    catch (Exception e) {
        e.printStackTrace();
        return(null);
    }
}

/**
 *  Merges event arrays.  Handles null arrays as gracefully as possible.
 * @return
 * @param theseEvents
 * @param subEvents
 */
public static JCEventSetDescriptor[]
    mergeEvents(JCEventSetDescriptor[] theseEvents,
                JCEventSetDescriptor[] subEvents)
{

    if (subEvents == null) {
        return(theseEvents);
    }

    if (theseEvents == null) {
        return(subEvents);
    }

    try {
        JCEventSetDescriptor[] thisArray =
            new JCEventSetDescriptor[theseEvents.length + subEvents.length];
        System.arraycopy(theseEvents, 0, thisArray, 0, theseEvents.length);
        System.arraycopy(subEvents, 0,
                         thisArray, theseEvents.length, subEvents.length);
        return(thisArray);
    }
    catch (Exception e) {
        e.printStackTrace();
        return(null);
    }
}

/**
 * Gets the Bean's <code>BeanDescriptor</code>.
 * @return
 */
public BeanDescriptor getBeanDescriptor()
{
    try {
        BeanDescriptor bd = new BeanDescriptor(beanClass);
        bd.setValue("FOLDER", "JClass");
        bd.setValue("TOOLBAR", "JClass");
        return(bd);
    }
    catch (Exception e) {}
    return(null);
}

}
