/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCEventSetDescriptor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:41 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.beans;



/**
 * Class used to encapsulate an event description for use in Bean
 * creation.  Used by all Bean info classes subclassing from
 * <code>JCBeanInfo</code> or one of its subclasses.
 */
public class JCEventSetDescriptor extends JCFeatureDescriptor {

protected String listenerClass;
protected String addRemoveRoot;
protected String listenerMethods[];
protected String eventPackage;

/**
 * Constructor.
 * @param name the name of the event
 * @param key_root the root of the key used to look up localized event name and
 *        short text descriptions of the event
 * @param event_package root package for events and listeners
 * @param listener_class the name of the listener class relative to the class
 *        location of the containing <code>JCBeanInfo</code> subclass
 * @param addremove_root the name of the add/remove methods minus the
 *        "add"/"remove" at the beginning and the "Listener" at the end
 * @param listener_methods an array of the names of the methods contained in the
 *        listener class
 */
public JCEventSetDescriptor(String name, String key_root, String event_package,
                            String listener_class, String addremove_root,
                            String listener_methods[])
{
    super(name, key_root);
    this.eventPackage = event_package;
    this.listenerClass = listener_class;
    this.addRemoveRoot = addremove_root;
    this.listenerMethods = listener_methods;
}

/**
 * Constructor to use when the value for <code>name</code>,
 * <code>key_root</code>, and <code>addremove_root</code> from the main
 * constructor are all the same.
 * @param event_package
 * @param listener_methods
 * @param root
 */
public JCEventSetDescriptor(String root, String event_package,
                            String listener_methods[])
{
    this(root, root, event_package, root + "Listener", root, listener_methods);
}

public Class getListenerClass()
{
    try {
        return(Class.forName(eventPackage + "." + listenerClass));
    }
    catch (ClassNotFoundException e) {
        
        return(null);
    }
}

public Class getEventClass()
{
    try {
        return(Class.forName(eventPackage + "." + name));
    }
    catch (ClassNotFoundException e) {
        return(null);
    }
}

public String[] getListenerMethods()
{
    return(listenerMethods);
}

public String getAddListenerMethod()
{
    return("add" + addRemoveRoot + "Listener");
}

public String getRemoveListenerMethod()
{
   return("remove" + addRemoveRoot + "Listener");
}

}
