/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCFeatureDescriptor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:41 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.beans;

/**
 * This is a temporary class to use when constructing <code>JCBeanInfo</code>s.
 * The <code>JCBeanInfo</code> takes objects of this class and converts them
 * into real Property or EventSet Descriptors.
 */
public class JCFeatureDescriptor {

protected String name;
protected String keyRoot;

/**
 * Constructor.
 * @param name the name of the property
 * @param key_root the root of the key used to look up localized property name
 *        and short text descriptions of the property
 */
public JCFeatureDescriptor(String name, String key_root)
{
    if (name != null && name.length() > 0) {
        char first_char = name.charAt(0);
        if (Character.isUpperCase(first_char)) {
            // Names should always start with a lover case so we convert
            // it here
            String new_name;
            if (name.length() > 1) {
                new_name = "" + Character.toLowerCase(first_char) +
                    name.substring(1);
            }
            else {
                new_name = "" + Character.toLowerCase(first_char);
            }
            name = new_name;
        }
    }

    this.name = name;
    this.keyRoot = key_root;
}

/**
 * Returns the name of the property.
 * @return name of the property
 */
public String getName()
{
    return(name);
}

/**
 * Returns the key root of the property for retrieving Strings from a locale
 * bundle.
 * @return key root of the property for retrieving Strings from a locale bundle
 */
public String getKeyRoot()
{
    return(keyRoot);
}

/**
 * Returns the localized name.
 * @return localized name
 */
public String getDisplayName()
{
    try {
        String key = keyRoot + JCBeanInfo.DISPLAY_NAME_SUFFIX;
        return(JCBeanInfo.getLocaleManager().getString(key));
    }
    catch (Exception e) {
        // ??? bad code
        return(null);
    }
}

/**
 * Returns the localized descriptions.
 * @return localized descriptions
 */
public String getShortDescription()
{
    try {
        String key = keyRoot + JCBeanInfo.SHORT_DESCRIPTION_SUFFIX;
        return(JCBeanInfo.getLocaleManager().getString(key));
    }
    catch (Exception e) {
        // ??? bad code
        return(null);
    }
}

}
