/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCPropertyDescriptor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:41 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.beans;

/**
 * This is a temporary class to use when constructing <code>JCBeanInfo</code>s.
 * The <code>JCBeanInfo</code> takes objects of this class and converts them
 * into real <code>PropertyDescriptors</code>.
 */
public class JCPropertyDescriptor extends JCFeatureDescriptor {

/** Property is normal. */
static public final int NORMAL = 0;
/** Property is expert. */
static public final int EXPERT = 1;
/** Property is hidden. */
static public final int HIDDEN = 2;

protected boolean  isBoolean;
protected int      modifier;
protected String   propertyEditor;
protected String   methodRoot;

/**
 * Constructor.
 * @param name the name of the property
 * @param key_root the root of the key used to look up localized property name
 *        and short text descriptions of the property
 * @param method_root the name of the property used with the get/is and set
 *        methods
 * @param is_boolean if the property is a boolean then its accessor method
 *        begins with "is"; otherwise, it begins with "get"
 * @param modifier this describes whether the propery is <code>NORMAL</code>,
 *        <code>EXPERT</code>, or <code>HIDDEN</code>
 * @param property_editor this is the name of the property editor to use; "null"
 *        indicates that the default editor should be used
 */
public JCPropertyDescriptor(String name, String key_root, String method_root,
                            boolean is_boolean, int modifier,
                            String property_editor)
{
    super(name, key_root);
    this.methodRoot = capitalize(method_root);
    this.isBoolean = is_boolean;
    this.modifier = modifier;
    this.propertyEditor = property_editor;
}

public JCPropertyDescriptor(String root, String property_editor)
{
    this(root, root, root, false, NORMAL, property_editor);
}

public JCPropertyDescriptor(String root, String property_editor,
                            boolean is_boolean)
{
    this(root, root, root, is_boolean, NORMAL, property_editor);
}

/**
 * Returns the name of the <code>get</code> method.
 * @return name of the <code>get</code> method
 */
public String getGetterName()
{
    return((isBoolean ? "is" : "get") + methodRoot);
}

/**
 * Returns the name of the <code>set</code> method.
 * @return name of the <code>set</code> method
 */
public String getSetterName()
{
    return("set" + methodRoot);
}

/**
 * Returns whether the property is hidden or not.
 * @return whether the property is hidden or not
 */
public boolean isHidden()
{
    return(modifier == HIDDEN);
}

/**
 * Returns whether the property is expert or not.
 * @return whether the property is expert or not
 */
public boolean isExpert()
{
    return(modifier == EXPERT);
}

/**
 * Returns the class name of the property editor used to edit this property.
 * @return class name of the property editor used to edit this property
 */
public Class getPropertyEditorClass()
{
    if (propertyEditor != null && propertyEditor.length() > 0) {
        // Get the class from the string name of the property editor
        try {
            return(Class.forName(propertyEditor));
        }
        catch (ClassNotFoundException e) {
            // Do nothing since we fall through to return null
        }
    }
    return(null);
}

/**
 * An exact duplicate of <code>PropertyDescriptor.capitalize()</code>,
 * which is UNFORTUNATELY a package-level routine.
 * @param s The string to capitalize.
 * @return The new capitalized string.
 */
private static String capitalize(String s)
{
    if (s == null || s.length() == 0) {
        return(s);
    }
    char chars[] = s.toCharArray();
    chars[0] = Character.toUpperCase(chars[0]);
    return(new String(chars));
}

}								  
