/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: StringChoiceEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:41 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.beans;

import java.beans.PropertyEditorSupport;

/**
 * This is an abstract class to use in property editors and customizers,
 * when an list of string values is required.
 * It maps a list of strings used as values within the bean to a list 
 * of internationalized strings that are shown to the user.
 */
public abstract class StringChoiceEditor extends PropertyEditorSupport {

protected String[] strings;       // value strings for code generation
protected String[] i18n_strings;  // i18n strings to appear in visual editor
						          // (i.e. combo box)

/**
 * Constructor
 * @param strings the list of values to display as Strings and
 *                to use for code generation
 */
public StringChoiceEditor(String[] strings)
{
    this.strings = strings;
    this.i18n_strings = strings;
}

/**
 * Constructor
 * @param strings the list of values to use for code generation
 * @param i18n_strings the list of internationalized values
 *                     to display as Strings
 */
public StringChoiceEditor(String[] strings, String[] i18n_strings)
{
    this.strings = strings;
    this.i18n_strings = i18n_strings;
}

/**
 * Sets the property's value from the passed String value, which should
 * be an internationalized string.  To use a value string, call setValue().
 * @param text the new value as an internationalized String
 */
public void setAsText(String text) throws IllegalArgumentException
{
    String v = null;
    for (int i = 0; i < i18n_strings.length; i++) {
        if (i18n_strings[i].equals(text)) {
            v = strings[i];
            break;
        }
    }


    if (v == null) {
        throw new IllegalArgumentException("invalid enum: " + text);
    }
    setValue(v);
}

/**
 * Returns the property's value as an internationalized String
 * To get the actual value string, call getValue().
 */
public String getAsText()
{
    String text = (String)getValue();
    String v = null;
    for (int i = 0; i < strings.length; i++) {
        if (strings[i].equals(text)) {
            v = i18n_strings[i];
            break;
        }
    }
    if (v == null) {
        return("");
    }
    else {
        return(v);
    }
}

public String[] getTags()
{
    return(i18n_strings);
}

public String getJavaInitializationString()
{
    String codeString = (String)getValue();
    if (codeString == null) {
        return("NULL");
    }
    // Code generation uses english string array, not internationalized one
    return("\"" + codeString + "\"");
}

}
