/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: StringListEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:41 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.beans;

import javax.swing.*;
import java.awt.*;

/**
 *  Bean editor for selecting from a list of strings.
 */
public class StringListEditor extends AbstractListEditor {

public static final String  STRINGLIST_HELP = "StringList";
protected Object[]          stringArray = null;

public StringListEditor()
{
    super("com.klg.jclass.beans.resources.LocaleInfo", STRINGLIST_HELP,
          "StringList Editor");
    setPreferredSize(new Dimension(300, 200));
}

public StringListEditor(String locale_bundle, String help_key,
                        String editor_string)
{
    super(locale_bundle, help_key, editor_string);
}

/**
 * This method will convert the String value into an object.
 */
protected Object parse(String text)
{
    // If text is empty, don't bother parsing so it won't be added to the
    // list
    if (text.equals("")) {
        return(null);
    }
    return(text);
}

/**
 * Determines if a custom list cell renderer is needed to display the list
 * values in the list.  If true, the getCellRenderer method must be
 * overridden to return the ListCellRenderer.
 */
protected boolean hasListCellRenderer()
{
    return(false);
}

/**
 * Returns the code generation string.
 */
public String getJavaInitializationString()
{
    if (stringArray == null || stringArray.length == 0) {
        return(null);
    }

    // build the string array
    String init_string = "new String[] {";

    for (int i = 0; i < stringArray.length; i++) {
        init_string += "\"" + stringArray[i] + "\"";
        if (i != stringArray.length - 1) {
            init_string += ",";
        }
        else {
            init_string += "}";
        }
    }
    return(init_string);
}

/**
 * Returns the value of the property.
 */
public Object getValue()
{
    DefaultListModel model = (DefaultListModel)list.getModel();
    stringArray = model.toArray();
    return(stringArray);
}

/**
 * Returns a text representation of the value.
 */
protected String getValueAsText()
{
    return(null);
}

/**
 * Sets the value of the property.
 */
public void setValue(Object o)
{
    // If stringArray is the same, don't do anything.
    // If it's not, clear the list model and add the new elements
    if (stringArray != o) {
        stringArray = (Object[])o;
        DefaultListModel model = (DefaultListModel)list.getModel();
        model.removeAllElements();
        for (int i = 0; i < stringArray.length; i++) {
            model.addElement(stringArray[i]);
        }
    }
}


public static void main(String[] args)
{
    JFrame f = new JFrame();
    f.getContentPane().add(new StringListEditor());
    f.pack();
    f.show();
}

}


