/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCCellEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:41 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.cell;

import java.awt.*;
import java.io.Serializable;

/**
 * The <CODE>JCCellEditor</CODE> interface allows objects to specify
 * custom cell editors.
 *
 * <P>A cell editor is essentially an AWT component, but
 * the cell editor interface adds functions that allow the
 * container to initialize, commit, or cancel the edit.
 * 
 * <P><b>Special note:</b> If an editor contains multiple AWT components, it
 * is important that all key events migrate to the component 
 * returned by <CODE>getComponent()</CODE>.  Most containers listen for key
 * events on this component in order to determine whether to
 * traverse or cancel.
 *
 */
public interface JCCellEditor extends JCCellEditorEventSource, Serializable {

/**
 * The container calls <CODE>initialize()</CODE> before the edit starts to
 * let the editor know what kind of event started the edit
 * (via the <CODE>AWTEvent</CODE> object).
 * <br><b>Note:</b> The size of the cell comes from <code>info.getDrawingArea()</code>.
 * <p>Implementors must handle null <code>AWTEvent</code>s.
 * @param ev event that initiated the edit operation
 * @param info cell that displays information for the editor
 * @param o item to be edited
 */
public void initialize(AWTEvent ev, JCCellInfo info, Object o);

/**
 * Returns the actual component that will be used to do the editing.
 * Typically, the <CODE>JCCellEditor</CODE> will extend an Swing component and
 * implement the <CODE>JCCellEditor</CODE> interface, so
 * <CODE>getComponent()</CODE> usually returns 'this'.
 * @return actual component that will be used to do the editing
 */
public Component getComponent();


/** 
 * Returns the value contained in the editor.
 * @return value contained in the editor
 */
public Object getCellEditorValue();

/**
 * Terminates editing, accepting any partially-edited value as the value
 * of the editor. The editor should return <code>false</code> if editing was not
 * stopped; this is useful for value-validating editors that cannot accept invalid
 * edits.
 *
 * @return  <code>true</code> if editing was stopped
 */
public boolean stopCellEditing();

/**
 * Determines whether any changes have been made.  When the container
 * wants to commit the edit, it should call the <CODE>JCCellEditor</CODE>'s
 * <CODE>isModified()</CODE> method to determine whether any changes have
 * been made.
 * @return
 */
public boolean isModified();

/**
 * Cancels the edit. When the container wants to cancel the edit and
 * discard any changes, it should call <CODE>cancel()</CODE>. <br><B>Note:</b>
 * <CODE>JCCellEditor.cancel()</CODE> should never be called from within a
 * cell editor.
 */
public void cancelCellEditing();

/**
 * Retrieves the keys that the editor would like to reserve for itself. JDK
 * event handling does not allow the order of event processing to be
 * specified. This makes it possible for the container to override key
 * processing in the editor. To avoid this, the editor can pass back a list
 * of keys it wishes to reserve. <br><B>Note:</b> The container can choose whether
 * to honor the editor's attempt to reserve the keys.
 *
 * <P>Most editors can simply return <CODE>null</CODE>.
 * @return keys that the editor would like to reserve for itself
 */
public JCKeyModifier[] getReservedKeys();

}
