/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCCellEditorSupport.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:41 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.cell;

import com.klg.jclass.util.JCListenerList;

import java.io.Serializable;
import java.util.Enumeration;

/**
 * <code>JCCellEditorSupport</code> is a convenience class that can be used
 * by cell editors to implement standard editor event handling.
 * In addition to registering edit event listeners, it provides
 * easy methods for firing events.
 */
public class JCCellEditorSupport implements JCCellEditorEventSource, 
	                                        Serializable {
 
/** Manages <code>JCCellEditorListeners</code>. */
protected JCListenerList cellEditorListeners;


/**
 * Creates a <code>JCCellEditorSupport</code> instance.
 */
public JCCellEditorSupport() {}

/**
 * Adds a new <code>JCCellEditorListener</code> to the list.
 * @param l
 */
public void addCellEditorListener(JCCellEditorListener l)
{
	cellEditorListeners = JCListenerList.add(cellEditorListeners, l);
}

/**
 * Removes a <code>JCCellEditorListener</code> from the list.
 * @param l
 */
public void removeCellEditorListener(JCCellEditorListener l)
{
	cellEditorListeners = JCListenerList.remove(cellEditorListeners, l);
}

/**
 * Sends an <code>EditingStopped</code> message/event to all listeners.
 * @param event
 */
public void fireStopEditing(JCCellEditorEvent event)
{
	if (event == null) {
        return;
    }
	Enumeration e = JCListenerList.elements(cellEditorListeners);
	while (e.hasMoreElements()) {
		JCCellEditorListener listener = (JCCellEditorListener)e.nextElement();
		listener.editingStopped(event);
	}
}

/**
 * Sends an <code>EditingCanceled</code> message/event to all listeners.
 * @param event
 */
public void fireCancelEditing(JCCellEditorEvent event)
{
	if (event == null) {
        return;
    }
	Enumeration e = JCListenerList.elements(cellEditorListeners);
	while (e.hasMoreElements()) {
		JCCellEditorListener listener = (JCCellEditorListener)e.nextElement();
		listener.editingCanceled(event);
	}
}

}
