/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCCellInfo.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:41 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.cell;

import java.awt.Color;
import java.awt.Rectangle;
import java.awt.Font;
import java.awt.Insets;
import javax.swing.SwingConstants;

/**
 * The <CODE>JCCellInfo</CODE> class is passed to <CODE>JCCellRenderer</CODE>
 * and <CODE>JCCellEditor</CODE> objects to provide cell information that
 * can be used in drawing or determining the type of editor to use.
 */
public interface JCCellInfo extends java.io.Serializable {

/** Alignment value. */
public static final int LEFT = SwingConstants.LEFT;
/** Alignment value. */
public static final int CENTER = SwingConstants.CENTER;
/** Alignment value. */
public static final int RIGHT = SwingConstants.RIGHT;
/** Alignment value. */
public static final int TOP = SwingConstants.TOP;
/** Alignment value. */
public static final int BOTTOM = SwingConstants.BOTTOM;

/** <code>clipHints</code> value. */
public static final int SHOW_NONE = 0;
/** <code>clipHints</code> value. */
public static final int SHOW_HORIZONTAL = 1;
/** <code>clipHints</code> value. */
public static final int SHOW_VERTICAL = 2;
/** <code>clipHints</code> value. */
public static final int SHOW_ALL = 3;

/**
 * Returns the background color of the cell being displayed or edited.
 * @return background color of the cell being displayed or edited
 */
public Color getBackground();

/**
 * Returns the foreground color of the cell being displayed or edited.  
 * <CODE>Container</CODE> sets the foreground color on the graphics object it
 * passes to  <CODE>JCCellRenderer.draw()</CODE>.
 * @return foreground color of the cell being displayed or edited
 */
public Color getForeground();

/**
 * Returns the background color of the cell being displayed or edited when
 * the cell is selected.
 * @return background color of the cell being displayed or edited when the
 * cell is selected
 */
public Color getSelectedBackground();

/**
 * Returns the foreground color of the cell being displayed or edited when
 * the cell is selected.
 * @return foreground color of the cell being displayed or edited when the
 * cell is selected
 */
public Color getSelectedForeground();

/**
 * Returns the font of the cell being displayed or edited.  
 * <CODE>Container</CODE> sets the
 * font on the graphics object it passes to <CODE>JCCellRenderer.draw()</CODE>.
 * @return font of the cell being displayed or edited
 */
public Font getFont();

/**
 * Returns the horizontal alignment for the cell being displayed or edited.
 * Valid return values are <CODE>LEFT</CODE>, <CODE>CENTER</CODE>, and
 * <CODE>RIGHT</CODE>.
 * @return horizontal alignment for the cell being displayed or edited
 */
public int getHorizontalAlignment();

/**
 * Returns the vertical alignment for the cell being displayed or edited.
 * Valid return values are <CODE>TOP</CODE>, <CODE>CENTER</CODE>, and
 * <CODE>BOTTOM</CODE>.
 * @return vertical alignment for the cell being displayed or edited
 */
public int getVerticalAlignment();

/**
 * Returns the <code>Insets</code> object representing the margins for the cell.
 * <p> This method should be applied to editor components, but not
 * renderer components since the renderer has already been mapped inside
 * the area bounded by the margins and borders.
 * @return <code>Insets</code> object representing the margins for the cell
 */
public Insets getMarginInsets();

/**
 * Returns the <code>Insets</code> object representing the borders for the cell.
 * <p> This method should be ignored by editor and renderer components
 * since the editor is deliberatly made bigger than the renderer by this
 * amount.
 * @return <code>Insets</code> object representing the borders for the cell
 */
public Insets getBorderInsets();

/**
 * Returns the border style for the cell.  Renderers and Editors should
 * ignore this value.
 * @return border style for the cell
 */
public int getBorderStyle();

/**
 * Returns the dimensions of the drawing area for the cell being displayed
 * or edited. When the renderer's draw method is called, the gc is translated
 * to the upper left-hand corner of the drawing area for the cell.  As
 * a result, the (x,y) of the Rectangle are always (0,0).
 * @return size of the drawing area.  
 */
public Rectangle getDrawingArea();

/**
 * Determines whether the cell is editable.
 * @return
 */
public boolean isEditable();

/**
 * Determines whether the cell is enabled.
 * @return
 */
public boolean isEnabled();

/**
 * Determines whether the editor should select the contents before editing.
 * @return
 */
public boolean getSelectAll();

/**
 * Determines whether the renderer should draw some indication if the
 * entire contents of the cell cannot be rendered within the given area.
 * @return
 */
public int getClipHints();

/**
 * Returns the cell's associated data type.
 * @return cell's associated data type
 */
public Class getDataType();

}
