/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCComponentCellRenderer.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:41 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.cell;

import java.awt.Component;

/**
 * A component-based cell renderer interface.
 * <p>
 * This interface is used to render cells in LiveTable and
 * HiGrid.  It uses a component as a rubber stamp for painting
 * cells.  This is very helpful if you already have a component
 * or component hierarchy that displays data the way you want.
 * <p>
 * This model differs from <code>JCLightCellRenderer</code> in that the
 * implementor does not explicitly control drawing.  Drawing
 * is handled by the component returned by <code>getRendererComponent()</code>.
 * 
 * @see com.klg.jclass.cell.JCCellRenderer
 * @see com.klg.jclass.cell.JCLightCellRenderer
 */
public interface JCComponentCellRenderer extends JCCellRenderer {

/**
 * Method that returns the cell rendering component.  The
 * container uses the component as a rubber stamp for painting
 * inside the cell.  
 *
 * @param cellInfo cell property information
 * @param o object to be rendered
 * @param selected boolean value indicating whether the cell is selected
 * @return a lightweight component, including Swing components
 * and JClass components
 */
public Component getRendererComponent(JCCellInfo cellInfo, Object o,
                                      boolean selected);

}
