/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCKeyModifier.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:41 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.cell;

import java.io.Serializable;

/**
 * This class is a container for a key code and a modifier
 * value.  It is used by <CODE>JCCellEditor</CODE> to pass back reserved
 * key/modifier combinations.
 */
public class JCKeyModifier implements Serializable {

/** Key code. */
public int key;

/** Modifer for the key code. */
public int modifier;

/** Determines whether the key and modifier can be used to initialize an edit. */
public boolean canInitializeEdit;

/** Modifier value that indicates all modifiers active. */
public static final int ALL = 1 << 4;
/** Modifier value that indicates no modifiers active. */
public static final int NONE = 0;

/**
 * Convenience constructor.  Use <CODE>ALL</CODE> to specify that all modifiers
 * apply to this keycode.  If a key plus modifier are flagged <code>true</code> for
 * <CODE>canInitializeEdit</CODE>, then that key combination will invoke the
 * editor.
 * @param canInitializeEdit
 * @param key
 * @param modifier
 */
public JCKeyModifier(int key, int modifier, boolean canInitializeEdit)
{
	this.key = key;
	this.modifier = modifier;
	this.canInitializeEdit = canInitializeEdit;
}

/**
 * Convenience constructor.  Use <CODE>ALL</CODE> to specify that all modifiers
 * apply to this keycode.
 * @param key
 * @param modifier
 */
public JCKeyModifier(int key, int modifier)
{
	this.key = key;	
	this.modifier = modifier;
	this.canInitializeEdit = false;
}

/**
 * Constructor used to represent a key press with all 
 * modifiers active.
 * @param key
 */
public JCKeyModifier(int key)
{
	this.key = key;
	this.modifier = ALL;
	this.canInitializeEdit = false;
}

/**
 * Determines whether the specified key/modifier combination matches
 * the one stored in this object.
 * @param key
 * @param modifier
 * @return
 */
public boolean match(int key, int modifier)
{
	// Keys don't match
	if (this.key != key) {
        return(false);
    }

	// Keys match, all modifiers accepted
	if (this.modifier == ALL) {
        return(true);
    }

	// Modifiers match exactly
	if (this.modifier == modifier) {
        return(true);
    }

	// Modifiers don't match exactly, and ALL not specified.
	return(false);
}

}
