/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCLightCellRenderer.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:41 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.cell;

import java.awt.*;

/**
 * A drawing-based cell renderer interface.
 *
 * This interface is used to draw cell data within
 * contraints provided by the container.  This technique
 * is faster than that provided by <code>JCComponentCellRenderer</code>,
 * and offers more control.  It can also be more work.
 *
 * @see com.klg.jclass.cell.JCCellRenderer
 * @see com.klg.jclass.cell.JCComponentCellRenderer
 */
public interface JCLightCellRenderer extends JCCellRenderer {

/**
 * Used to draw the cell. <br><B>Note:</b> The size of the cell comes from
 * <CODE>cellInfo.getDrawingArea()</CODE>. <p>When the renderer's draw method is
 * called, the gc is translated to the upper-left corner of the drawing area 
 * for the cell.  The "drawing area" of the cell is the region <EM>inside</EM> 
 * the borders and the margins.
 * 
 * @param gc graphics context to use for drawing
 * @param cellInfo cell property information
 * @param o object to be rendered
 * @param selected boolean value indicating whether the cell is selected
 */
public void draw(Graphics gc, JCCellInfo cellInfo, Object o, boolean selected);

/**
 * Returns the preferred size of the cell's drawing area.  The "drawing area" 
 * of the cell is the region <EM>inside</EM> the borders and the margins.
 * 
 * @param gc graphics context for calculating size
 * @param cellInfo cell property information
 * @param o object to be rendered
 * @return <code>java.awt.Dimension</code> instance containing preferrred height and width
 */
public Dimension getPreferredSize(Graphics gc, JCCellInfo cellInfo, Object o);

}
