/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCCheckBoxCellEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:42 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.cell.editors;

import java.awt.*;
import java.awt.event.*;

import com.klg.jclass.cell.*;

import javax.swing.*;
import javax.swing.border.EtchedBorder;
import javax.swing.event.*;

/**
 * <CODE>JCCheckBoxCellEditor</CODE> provides an editor using a check box
 * field for boolean objects.
 */
public class JCCheckBoxCellEditor extends JCheckBox
                                  implements JCCellEditor,
								             KeyListener,
											 MouseListener,
											 ChangeListener
{

protected Object               data;
protected JCCellInfo           cellInfo;
protected JCCellEditorSupport  support = new JCCellEditorSupport();
protected JCKeyModifier[]      keys = null;

protected static final int BORDER_SIZE = 2;

/**
 * Default constructor.
 */
public JCCheckBoxCellEditor()
{
	this(null);
}

public JCCheckBoxCellEditor(String text)
{
	super(text);
    setBorder(new EtchedBorder());
    addChangeListener(this);
	addKeyListener(this);
	addMouseListener(this);
}

public void initialize(AWTEvent ev, JCCellInfo info, Object o)
{

    data = o;
    initData();
    cellInfo = info;
    setBorderPainted(true);
    setMargin(info.getMarginInsets());
    setEnabled(info.isEditable());
	// ATTN GV Lightweight technique if event not automatically set
    if (ev != null) {
        toggleValue(ev);
    }
    setBackground(info.getBackground());
    setForeground(info.getForeground());
}

/**
 * Returns the object to be committed.
 * @return object to be committed
 */
public Object getCellEditorValue()
{
    return(new Boolean(isSelected()));
}

/**
 * Returns the original unedited boolean value of this editor.
 * @return the original boolean value
 */
public boolean getOriginalValue()
{
    if (data == null) {
        return(false);
    }
    return(((Boolean)data).booleanValue());
}

/**
 * Returns the edit component.
 * @return edit component
 */
public Component getComponent()
{
	return(this);
}

/**
 * Returns <code>true</code> if the data has been modified.
 * @return <code>true</code> if the data has been modified
 */
public boolean isModified()
{
    // Data is null implies a false original value
    if (data != null && !(data instanceof Boolean)) {
        return(false);
    }
    boolean currentValue = ((Boolean)getCellEditorValue()).booleanValue();
    return(getOriginalValue() != currentValue);
}

/**
 * Called when the edit process is stopped.
 * @return
 */
public boolean stopCellEditing()
{
	return(true);
}

/**
 * Called when the edit process is cancelled.
 */
public void cancelCellEditing()
{
    initData();
    repaint();
}

/**
 * This editor reserves no keys for its own use.
 * @return
 */
public JCKeyModifier[] getReservedKeys()
{
	return(keys);
}


/**
 * Adds a listener to the list that's notified when the editor starts,
 * stops, or cancels editing.
 *
 * @param l the <code>JCCellEditorListener</code> to be added
 */
public void addCellEditorListener(JCCellEditorListener l)
{
	support.addCellEditorListener(l);
}

/**
 * Removes a listener from the list that is notified when the editor starts,
 * stops, or cancels editing.
 *
 * @param l the <code>JCCellEditorListener</code> to be removed
 */
public void removeCellEditorListener(JCCellEditorListener l)
{
	support.removeCellEditorListener(l);
}

protected void initData()
{
    if (data == null) {
        setSelected(false);
        return;
    }
    if (!(data instanceof Boolean)) {
        return;
    }
	boolean currentValue = getOriginalValue();
	if (isSelected() != currentValue) {
	    setSelected(currentValue);
	}
}

protected void toggleValue(AWTEvent e)
{
    if (isEnabled()) {
        setSelected(!isSelected());
    }
}

public void stateChanged(ChangeEvent e)
{
    SwingUtilities.invokeLater(new EventInvoker(e));
}

public void keyPressed(KeyEvent e)
{
	switch(e.getKeyCode()) {
	case KeyEvent.VK_ENTER:
		support.fireStopEditing(new JCCellEditorEvent(e));
		break;
	case KeyEvent.VK_ESCAPE:
		support.fireCancelEditing(new JCCellEditorEvent(e));
		break;
	}
}

public void keyReleased(KeyEvent e) {}

public void keyTyped(KeyEvent e) {}

public void mousePressed(MouseEvent e) {}

public void mouseReleased(MouseEvent e) {}

public void mouseClicked(MouseEvent e) {}

public void mouseEntered(MouseEvent e)
{
    setCursor(Cursor.getDefaultCursor());
}

public void mouseExited(MouseEvent e) {}

/**
 * The sole purpose of the class is to avoid thread races and fire the
 * event after the editor has gotten its value.
 */
private class EventInvoker implements Runnable
{
 	private ChangeEvent e;

    EventInvoker(ChangeEvent e)
    {
    	this.e = e;
    }

    public void run()
    {
        JCCellEditorEvent event = new JCCellEditorEvent(e);
   	    JCCheckBoxCellEditor.this.support.fireStopEditing(event);
    }

} // EventInvoker

}
