/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCComboBoxCellEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:42 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.cell.editors;

import com.klg.jclass.cell.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;

/**
 * <CODE>JCComboBoxCellEditor</CODE> provides an editor using a simple <code>ComboBox</code>
 * component.
 */
public class JCComboBoxCellEditor extends JComboBox
			                      implements JCCellEditor,
											 KeyListener
{

protected JCCellEditorSupport   support = new JCCellEditorSupport();
protected int                   values[];
protected boolean               ignoreEnterKey = false;
private   AWTEvent              initialEvent;
private   boolean               showPopupLater = false;
protected Object                originalValue = null;

protected JCKeyModifier[] key_modifiers = {
    new JCKeyModifier(KeyEvent.VK_DOWN, JCKeyModifier.ALL),
    new JCKeyModifier(KeyEvent.VK_UP, JCKeyModifier.ALL)
};

/**
 * Creates a new <CODE>JCComboBoxCellEditor</CODE> object.
 */
public JCComboBoxCellEditor()
{
    super();
    addKeyListener(this);
}

/**
 * Creates a new <CODE>JCComboBoxCellEditor</CODE> object with no values.
 * @param items an array of String items
 */
public JCComboBoxCellEditor(Object items[])
{
    this(items, null);
}

/**
 * Creates a new <CODE>JCComboBoxCellEditor</CODE> object.
 * @param items an array of String items
 * @param values an array of integers that represent the enumerated values
 *               associated with the String items
 */
public JCComboBoxCellEditor(Object items[], int values[])
{
    super(items);
    this.values = values;
    addKeyListener(this);
}

/**
 * Initializes the <code>ComboBox</code> component by selecting the appropriate
 * index associated with the data.
 * @param ev
 * @param info
 * @param o
 */
public void initialize(AWTEvent ev, JCCellInfo info, Object o)
{
    // If the event that invokes the editor is the ENTER key, that key
    // needs to be ignored in key processing so that the editor doesn't
    // immediately close.
    // ATTN GV What happens when event sent automatically?
    if (ev != null              &&
	    ev instanceof KeyEvent  &&
        ((KeyEvent)ev).getKeyCode() == KeyEvent.VK_ENTER)
	{
        ignoreEnterKey = true;
    }

    initialEvent = ev;
	if (values != null && o instanceof Integer) {
        setSelectedIndex(getIndexFromValue(((Integer)o).intValue()));
    }
	else {
        setSelectedItem(o);
    }
    originalValue = o;
    setBackground(info.getBackground());
    setForeground(info.getForeground());
    if (info.isEditable()) {
        setBackground(info.getBackground().brighter());
    }
    showPopupLater = (!isEditable()             &&
				      ev != null                &&
					  ev instanceof MouseEvent  &&
                      ev.getID() == MouseEvent.MOUSE_RELEASED);
}

/**
 * Overwrites superclass method to invoke popup.
 * @param g
 */
public void paint(java.awt.Graphics g)
{
    super.paint(g);
    if (showPopupLater) {
        showPopupLater = false;
        SwingUtilities.invokeLater(new PopupInvoker());
    }
}

/**
 * Returns the <code>ComboBox</code> component.
 * @return <code>ComboBox</code> component
 */
public Component getComponent()
{
    return(this);
}

/**
 * Returns the object to be committed.
 * @return object to be committed
 */
public Object getCellEditorValue()
{
    if (values == null) {
        return(getSelectedItem());
    }
    return(new Integer(values[getSelectedIndex()]));
}

public boolean isModified()
{
	Object o = getCellEditorValue();
    if (originalValue == null) {
        return(o != null);
    }
    return(!originalValue.equals(o));
}

/**
 * Called when the edit process is stopped.
 * @return
 */
public boolean stopCellEditing()
{
    return(true);
}

/**
 * Called when the edit process is cancelled.
 */
public void cancelCellEditing()
{
    return;
}

/**
 * This editor reserves no keys for its own use.
 * @return
 */
public JCKeyModifier[] getReservedKeys()
{
    return(key_modifiers);
}

public void keyTyped(KeyEvent e) {}

public void keyPressed(KeyEvent e) {}

public void keyReleased(KeyEvent e)
{
    if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
        support.fireCancelEditing(new JCCellEditorEvent(e));
    }
    else if (e.getKeyCode() == KeyEvent.VK_ENTER && !ignoreEnterKey) {
        support.fireStopEditing(new JCCellEditorEvent(e));
    }
    ignoreEnterKey = false;
}

/**
 * Adds a listener to the list that's notified when the editor starts,
 * stops, or cancels editing.
 *
 * @param l the <code>JCCellEditorListener</code>
 */
public void addCellEditorListener(JCCellEditorListener l)
{
    support.addCellEditorListener(l);
}

/**
 * Removes a listener from the list that's notified when the editor starts,
 * stops, or cancels editing.
 *
 * @param l the <code>JCCellEditorListener</code>
 */
public void removeCellEditorListener(JCCellEditorListener l)
{
    support.removeCellEditorListener(l);
}

/**
 * Returns the index number for a value.
 * @return index number for a value
 * @param value
 */
private int getIndexFromValue(int value)
{
    if (values == null) {
        return(value);
    }
    for (int i = 0; i < values.length; i++) {
        if (values[i] == value) {
            return(i);
        }
    }
    return(-1);
}

public void addNotify()
{
	super.addNotify();
    if (initialEvent != null) {
        if (isVisible()) {
            // This fixes the PR 10201 for an IllegalComponentStateException
            try {
                setPopupVisible(true);
	        }
			catch (Exception e) {}
        }

        // Reset, fix pr 11111
        initialEvent = null;
    }
}


/**
 * The sole purpose of the class is to invoke popup after the combobox is
 * opened with mouse click.
 */
private class PopupInvoker implements Runnable {

    public void run()
	{
        if (JCComboBoxCellEditor.this.isShowing() &&
            !JCComboBoxCellEditor.this.isPopupVisible())
		{
            JCComboBoxCellEditor.this.showPopup();
        }
    }
}

}
