/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCDateCellEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:42 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.cell.editors;

import java.util.Date;
import java.text.DateFormat;
import java.text.ParseException;
import com.klg.jclass.cell.validate.JCValidateEvent; 

/**
 * <CODE>DateCellEditor</CODE> provides an editor using a simple text 
 * field for <CODE>Date</CODE> objects.
 */
public class JCDateCellEditor extends BaseCellEditor
{

/**
 * Default constructor.
 */
public JCDateCellEditor()
{
	super();
}

/**
 * Called when the edit process is stopped.
 * @return <code>true</code> if the data entered is valid
 */
public boolean stopCellEditing()
{
    Date result = null;
	boolean valid = false;
    String str = getText();
	if (str == null || str.length() == 0) {
		result = null;
		valid = true; // null is considered a valid entry
	}
	else {
		try {
			result = DateFormat.getDateTimeInstance().parse(str.trim());
			valid = true;
		}
		catch (ParseException e) {
			result = null;
			valid = false; 
		}
	}

	JCValidateEvent ev = new JCValidateEvent(this, data, result, valid);
	return(fireValidated(ev));
}

public void initData()
{
	if (data == null || !(data instanceof Date)) {
		super.initData();
	}
	else {
		setText(DateFormat.getDateTimeInstance().format((Date)data));
	}
}

/**
 * Returns the object to be committed.
 * @return object to be committed
 */
public Object getCellEditorValue()
{
	Date result = null;
    String str = getText();
	if (str == null || str.length() == 0) {
		result = null;
    }
	else {
		try {
			result = DateFormat.getDateTimeInstance().parse(str.trim());
		}
		catch (ParseException e) {
			return(null);
		}
	}
	return(result);
}

}
