/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCMultilineCellEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:42 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.cell.editors;

import com.klg.jclass.cell.*;
import com.klg.jclass.cell.validate.JCValidateEvent;
import com.klg.jclass.cell.validate.JCValidateListener;
import com.klg.jclass.cell.validate.ValidateSupport;

import com.klg.jclass.util.JCEnvironment;

import javax.swing.FocusManager;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * <CODE>JCMultilineCellEditor</CODE> provides a simple text editing component
 * for multiline data.
 */
public class JCMultilineCellEditor extends JScrollPane
                                   implements JCCellTextEditor,
                                              KeyListener,
                                              FocusListener
{

protected Object                        data;
protected JCCellEditorSupport           support = new JCCellEditorSupport();
protected boolean                       selectAll = false;
protected ValidateSupport               validateSupport = new ValidateSupport();
protected JCKeyModifier[]               keys = null;
protected JCMultilineCellEditorTextArea text = null;
protected int                           caretPosition;
protected AWTEvent                      initialEvent;

/**
 * Default constructor.
 */
public JCMultilineCellEditor()
{
	addKeyListener(this);
    addFocusListener(this);
	text = new JCMultilineCellEditorTextArea();
	getViewport().add(text);
	setDoubleBuffered(true);
	text.setDoubleBuffered(true);
	text.addKeyListener(this);
}

public void initialize(AWTEvent ev, JCCellInfo info, Object o)
{
    this.initialEvent = ev;
	data = o;
    selectAll = info.getSelectAll();
    text.setEditable(info.isEditable());
    if (!text.isEditable()) {
        selectAll = false;
    }

    if (ev instanceof KeyEvent && data != null) {
        caretPosition = data.toString().length();
    }
    else {
        // Will be handled by the show() method later on
        caretPosition = -1;
    }

    initData();

 	setBackground(info.getBackground());
	setForeground(info.getForeground());

	// Move the focus to this container so that when
	// focus is moved to the text component child the change
	// is recognized : (
	requestFocus();
}

/**
 * Returns the edit component.
 * @return edit component
 */
public Component getComponent()
{
	validate();
	text.setVisible(true);
	text.requestFocus();
	return(this);
}

/**
 * Returns the object contained in the editor.
 * @return object contained in the editor
 */
public Object getCellEditorValue()
{
	return(getText());
}

/**
 * Gets the text for the editor (as a String).
 * @return the current text in the editor
 */
public String getCellEditorText()
{
    return(getText());
}

/**
 * Sets the text for the editor.
 * @param text the new cell editor text value
 */
public void setCellEditorText(String text)
{
    setText(text);
}

protected boolean fireValidated(JCValidateEvent ev)
{
	boolean validated = validateSupport.fireValidateEvents(ev);
	if (validated) {
	    setVisible(true);
	}
	return(validated);
}

/**
 * Tells the editor to stop editing and accept the current value.
 * @return
 */
public boolean stopCellEditing()
{
	String str = getText();
	JCValidateEvent ev = new JCValidateEvent(this, data, str, true);
	// All values are considered valid but someone registering
	// a ValidateListener can decide otherwise in the valueChangedEnd()
	// event
	boolean status = fireValidated(ev);
	if (status == true) {
        selectAllText();
    }
    return(status);
}

/**
 * Returns <CODE>true</CODE> if the data has been modified.
 * @return <CODE>true</CODE> if the data has been modified
 */
public boolean isModified()
{
	Object currentValue = getCellEditorValue();
	if ((data == null) || (data.equals(""))) {
	    if ((currentValue == null) || (currentValue.equals(""))) {
	        return(false);
	    }
	    else {
	        return(true);
	    }
	}
	return(!data.equals(currentValue));
}

/**
 * Called when the edit process is cancelled.
 */
public void cancelCellEditing()
{
    initData();
    selectAllText();
}

/**
 * <code>setText()</code> must map to <code>JTextArea</code> in Swing version.
 * @param s
 */
public void setText(String s)
{
	text.setText(s);
}

/**
 * <code>getText()</code> must map to <code>JTextArea</code> in Swing version.
 * @return
 */
public String getText() {
	return(text.getText());
}

/**
 * Returns whether this editor will select the text on initialization.
 * @return the value of <code>selectAll</code>
 */
public boolean getSelectAll()
{
    return(selectAll);
}

/**
 * Returns the internal <code>JTextArea</code> object used for editing.
 * @return the internal <code>JTextArea</code> editor
 */
public JTextArea getEditor()
{
    return(text);
}

/**
 * Overrides the <CODE>minimumSize</CODE> method to get the
 * <CODE>minimumSize</CODE> based on an abitrary row/column pair.
 * @return
 */
public Dimension getMinimumSize()
{
	return(new Dimension(0,0));
}

public Dimension getPreferredSize(Graphics gc, JCCellInfo cellInfo, Object o)
{
	return(getPreferredSize());
}

/**
 * This editor reserves the enter key and navigation keys.
 * @return
 */
public JCKeyModifier[] getReservedKeys()
{
	if (keys == null) {
		keys = new JCKeyModifier[9];
		keys[0] = new JCKeyModifier(KeyEvent.VK_ENTER);
		keys[1] = new JCKeyModifier(KeyEvent.VK_DOWN);
		keys[2] = new JCKeyModifier(KeyEvent.VK_UP);
		keys[3] = new JCKeyModifier(KeyEvent.VK_LEFT);
		keys[4] = new JCKeyModifier(KeyEvent.VK_RIGHT);
		keys[5] = new JCKeyModifier(KeyEvent.VK_PAGE_UP);
		keys[6] = new JCKeyModifier(KeyEvent.VK_PAGE_DOWN);
		keys[7] = new JCKeyModifier(KeyEvent.VK_HOME);
		keys[8] = new JCKeyModifier(KeyEvent.VK_END);
	}
	return(keys);
}

/**
 * Adds a listener to the list that is notified when the edit starts, stops,
 * or is canceled.
 * @param l
 */
public void addCellEditorListener(JCCellEditorListener l)
{
	support.addCellEditorListener(l);
}

/**
 * Removes a listener from the list that is notified when the edit starts, stops,
 * or is canceled.
 * @param l
 */
public void removeCellEditorListener(JCCellEditorListener l)
{
	support.removeCellEditorListener(l);
}

/**
 * Override the key event processor.  This allows the parent to pass a key
 * event to this editor that it otherwise might not get.
 * @param e The event to pass
 */
public void processKeyEvent(KeyEvent e)
{
    
    if (isVisible()) {
        if (JCEnvironment.getJavaVersion() >= 140) {
            super.processKeyEvent(e);
            text.processKeyEvent(e);
        }
        else {
            // In Swing versions prior to JKD1.4, we have to disable the focus
            // manager in orderto get the key processing we want
            FocusManager fm = FocusManager.getCurrentManager();
            FocusManager.disableSwingFocusManager();
            super.processKeyEvent(e);
            fm.setCurrentManager(fm);
        }
    }
}

public void keyPressed(KeyEvent e)
{
	switch(e.getKeyCode()) {
	case KeyEvent.VK_ENTER:
		if ((e.getModifiers() & KeyEvent.CTRL_MASK) != 0) {
			support.fireStopEditing(new JCCellEditorEvent(e));
        }
		break;
	case KeyEvent.VK_ESCAPE:
		support.fireCancelEditing(new JCCellEditorEvent(e));
		break;
	default:
		// Forward text keys to the pane so the container can listen to them.
		if (e.getComponent() == text) {
			processKeyEvent(new KeyEvent(this, e.getID(), e.getWhen(),
										 e.getModifiers(), e.getKeyCode(),
										 e.getKeyChar()));
		}
		break;
	}
}
public void keyReleased(KeyEvent e) {}

public void keyTyped(KeyEvent e) {}

void initData()
{
    setText((data == null) ? "" : data.toString());
}

/**
 * Uses hint from <code>CellInfo</code> to select all the text in the component.
 */
public void selectAllText()
{
    if (!isDisplayable()) {
        return;
    }

    if (getSelectAll()) {

        // Setting of the caret position is a hack to ensure that the
        // text is fully scrolled into view
        try {
            text.setCaretPosition(0);
            text.setCaretPosition((data == null) ? 0 : data.toString().length());
        }
        catch (Exception e) {
            // Checking for non-null peer above doesn't always work
        }

        text.selectAll();
    }

    // Request focus (yet again) so the caret will appear
    text.requestFocus();
}

/**
 * Overridden in order to show selected text.
 * @param b
 */
public void setVisible(boolean b)
{
	super.setVisible(b);
    JTextArea editor = getEditor();
    editor.setVisible(b);
    if (!b) {
        return;
    }

    // This code attempts to position the caret based
    // on where the user clicked the mouse within this cell
    String text = getText();
    if (text == null) {
        return;
    }
    if (caretPosition == -1) {
        caretPosition = text.length();
        if (initialEvent != null && initialEvent instanceof MouseEvent) {
            MouseEvent me = (MouseEvent)initialEvent;
            Component source = (Component)me.getSource();
            Component comp = getComponent();
            Point offset = new Point(0, 0);
            if (comp != source) {
                Container parent = getParent();
                offset.setLocation(comp.getLocation());
                while (parent != null && parent != source) {
                    offset.translate(parent.getLocation().x,
                                     parent.getLocation().y);
                    comp = parent;
                    parent = comp.getParent();
                }
                if (parent == null) {
                    offset.setLocation(0, 0);
                }
            }
            caretPosition = editor.viewToModel(new Point(me.getX() - offset.x,
                                                         me.getY() - offset.y));
        }
    }
    caretPosition = Math.min(caretPosition, text.length());
    editor.select(caretPosition, caretPosition);
    selectAllText();
}

//
// JCValidateInterface Implementation
//
/**
 * Adds a new <code>ValidateListener</code> to the list.
 * @param l
 */
public void addValidateListener(JCValidateListener l)
{
	validateSupport.addValidateListener(l);
}

/**
 * Removes a <code>ValidateListener</code> from the list.
 * @param l
 */
public void removeValidateListener(JCValidateListener l)
{
	validateSupport.removeValidateListener(l);
}

public void focusGained(FocusEvent e)
{
    text.requestFocus();
}

public void focusLost(FocusEvent e) {}

public class JCMultilineCellEditorTextArea extends JTextArea
{
    public void processKeyEvent(KeyEvent e)
    {
        super.processKeyEvent(e);
    }
}

}
