/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCWordWrapCellEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:42 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.cell.editors;

import com.klg.jclass.cell.JCCellInfo;
import com.klg.jclass.util.swing.JCWordWrap;

import java.awt.*;

/**
 * <CODE>JCWordWrapCellEditor</CODE> provides a simple text editing component
 * that wraps text.
 */
public class JCWordWrapCellEditor extends JCMultilineCellEditor
{

protected JCCellInfo  cellInfo;
protected boolean     editable = true;

protected final int VERTICAL_SB_WIDTH = 25;

/**
 * Default constructor.
 */
public JCWordWrapCellEditor()
{
    super();
    text.setLineWrap(true);
    text.setWrapStyleWord(true);
}

public void initialize(AWTEvent ev, JCCellInfo info, Object o)
{
    super.initialize(ev, info, o);
    cellInfo = info;
    editable = info.isEditable();
}

/**
 * Returns the object contained in the editor.
 * @return object contained in the editor
 */
public Object getCellEditorValue()
{
    if (editable) {
    	return(JCWordWrap.replace(getText(), "\n", " "));
    }
    else {
        return(data);
    }
}

protected String unwrapText(String s)
{
    return(JCWordWrap.replace(s, "\n", ""));
}

// No longer used.  Kept for backwards compatibility.
protected String wrapText(String s)
{
    String wrappedString = "";
	if (s != null) {
		// ATTN GV Nothing to be done on this one
		Graphics gc = getGraphics();
		if (gc == null) {
            return(s);
        }
		FontMetrics fm = getGraphics().getFontMetrics();
		Insets in = cellInfo.getBorderInsets();
		int width = cellInfo.getDrawingArea().width + in.left + in.right;
		wrappedString = JCWordWrap.wrapText(s, fm, width - VERTICAL_SB_WIDTH,
		                                    "\n", true);
	}
	return(wrappedString);
}

/**
 * Overridden to undo dynamic word wrapping on editor resize so that the
 * text area then redoes the wrapping.
 * @param height
 * @param width
 * @param x
 * @param y
 */
public void setBounds(int x, int y, int width, int height)
{
    super.setBounds(x, y, width, height);
    setText(unwrapText(getText()));
}

}
