/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCImageCellRenderer.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:42 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.cell.renderers;

import com.klg.jclass.cell.JCCellInfo;
import com.klg.jclass.cell.JCLightCellRenderer;
import com.klg.jclass.cell.Utilities;

import java.awt.*;
import java.awt.image.ImageObserver;

/**
 * The <CODE>JCImageCellRenderer</CODE> class defines a 
 * <CODE>JCCellRenderer</CODE> 
 * object that paints <code>Image</code> objects in a table cell.
 */
public class JCImageCellRenderer implements JCLightCellRenderer, ImageObserver {

protected Utilities  utilities = new Utilities();
protected boolean    drawClipArrowOutline = true;

/**
 * Draws the image.
 * @param cellInfo
 * @param gc
 * @param o
 * @param selected
 */
public void draw(Graphics gc, JCCellInfo cellInfo, Object o, boolean selected)
{
	if (o == null || !(o instanceof Image)) {
		return;
	}

    Rectangle cell_size = cellInfo.getDrawingArea();

 

	gc.setFont(cellInfo.getFont());
	gc.setColor(selected ? cellInfo.getSelectedForeground()
                         : cellInfo.getForeground());

    Point offset = getAlignmentOffset(cellInfo, o);
	gc.drawImage((Image)o, offset.x, offset.y, this);

    // Arrows are drawn using the current foreground color
    Dimension ps = getPreferredSize(gc, cellInfo, o);
    if ((ps.width > cell_size.width || ps.height > cell_size.height) &&
        cellInfo.getClipHints() != JCCellInfo.SHOW_NONE)
    {
        utilities.drawClipArrows(gc, cellInfo, ps,
                                 Utilities.DEFAULT_CLIP_ARROW_SIZE,
                                 drawClipArrowOutline);
    }
}

/**
 * Computes the alignment offset.
 * @return
 * @param cellInfo
 * @param o
 */
public Point getAlignmentOffset(JCCellInfo cellInfo, Object o)
{
    Point offset = new Point(0, 0);

	// Adjust for horizontal alignment
	int align = cellInfo.getHorizontalAlignment();
	if (align == JCCellInfo.RIGHT) {
		offset.x +=
            cellInfo.getDrawingArea().width - ((Image)o).getWidth(this);
    }
	else if (align == JCCellInfo.CENTER) {
		offset.x +=
            (cellInfo.getDrawingArea().width - ((Image)o).getWidth(this)) / 2;
    }

	// Adjust for vertical alignment
	align = cellInfo.getVerticalAlignment();
	if (align == JCCellInfo.CENTER) {
		offset.y +=
            (cellInfo.getDrawingArea().height - ((Image)o).getHeight(this)) / 2;
    }
	else if (align == JCCellInfo.BOTTOM) {
		offset.y +=
            cellInfo.getDrawingArea().height - ((Image)o).getHeight(this);
    }

    return(offset);
}

/**
 * Returns the size of the cell required to display the image.
 * @return
 * @param gc
 * @param info
 * @param o
 */
public Dimension getPreferredSize(Graphics gc, JCCellInfo info, Object o)
{
	if (o == null || !(o instanceof Image)) {
		return(null);
    }
	else {
		Image i = (Image)o;
		return(new Dimension(i.getWidth(this), i.getHeight(this)));
	}
}

public boolean imageUpdate(Image img, int flags, int x, int y, int width,
                           int height)
{
	switch(flags) {
		case ImageObserver.HEIGHT:
			//this.height = height;
			break;
		case ImageObserver.WIDTH:
			//this.width = width;
			break;
	}

	return(true);
}

}
