/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCWordWrapCellRenderer.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:42 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.cell.renderers;

import com.klg.jclass.cell.Utilities;
import com.klg.jclass.cell.JCCellInfo;
import com.klg.jclass.cell.JCLightCellRenderer;

import java.awt.*;
import java.util.StringTokenizer;

import com.klg.jclass.util.swing.JCWordWrap;

/**
 * A multi-line String cell renderer that will
 * wrap lines to fit the cell contraints.
 *
 * @see com.klg.jclass.cell.renderers.JCStringCellRenderer
 */
public class JCWordWrapCellRenderer implements JCLightCellRenderer {

protected Utilities   utilities = new Utilities();
protected boolean     drawClipArrowOutline = true;

/**
 * Used to draw a String using word-wrap logic.
 *
 * @param gc graphics context to use for drawing
 * @param cellInfo cell property information
 * @param o the object to be rendered
 * @param selected if <code>true</code>, the selected foreground color is used
 */
public void draw(Graphics gc, JCCellInfo cellInfo, Object o, boolean selected)
{
	if (o == null) {
		return;
    }
	String display = o.toString();

	if (display == null || display.length() == 0) {
		return;
    }

	Rectangle cell_size = cellInfo.getDrawingArea();
	gc.setFont(cellInfo.getFont());
	FontMetrics fm = gc.getFontMetrics();
	gc.setColor(selected ? cellInfo.getSelectedForeground() :
				cellInfo.getForeground());

    int hAlign = cellInfo.getHorizontalAlignment();
    int vAlign = cellInfo.getVerticalAlignment();
	display = JCWordWrap.wrapText(JCWordWrap.replace(display, "\n", " "),
								  fm, cell_size.width, "\n",
								  (hAlign != JCCellInfo.RIGHT));

    int h = fm.getHeight();
    int offset_x = 0, offset_y = h - 4;
	if (display.indexOf('\n') != -1) {

		int height = getHeight(fm, display);
		if (vAlign == JCCellInfo.CENTER) {
			offset_y += (cell_size.height - height) / 2;
        }
		else if (vAlign == JCCellInfo.BOTTOM) {
			offset_y += cell_size.height - height;
        }

		StringTokenizer tokenizer = new StringTokenizer(display, "\n", false);
		while (tokenizer.hasMoreTokens()) {
			String token = tokenizer.nextToken().trim();
			if (hAlign == JCCellInfo.CENTER) {
				offset_x = (cell_size.width - fm.stringWidth(token)) / 2;
            }
			else if (hAlign == JCCellInfo.RIGHT) {
				offset_x = cell_size.width - fm.stringWidth(token);
            }

			gc.drawString(token, offset_x, offset_y);
			offset_y += h;
		}
	}
	else {
		// Adjust for horizontal alignment
		if (hAlign == JCCellInfo.RIGHT) {
			offset_x += cell_size.width - fm.stringWidth(display);
        }
		else if (hAlign == JCCellInfo.CENTER) {
			offset_x += (cell_size.width - fm.stringWidth(display)) / 2;
        }

		// Adjust for vertical alignment
		if (vAlign == JCCellInfo.CENTER) {
			offset_y += (cell_size.height - fm.getHeight()) / 2;
        }
		else if(vAlign == JCCellInfo.BOTTOM) {
			offset_y += cell_size.height - fm.getHeight();
        }

		gc.drawString(display, offset_x, offset_y);
	}

    // Arrows are drawn using the current foreground color
	Dimension ps = getPreferredSize(gc, cellInfo, display);
	if ((ps.width > cell_size.width || ps.height > cell_size.height) &&
		cellInfo.getClipHints() != JCCellInfo.SHOW_NONE)
    {
		utilities.drawClipArrows(gc, cellInfo, ps,
								 Utilities.DEFAULT_CLIP_ARROW_SIZE,
								 drawClipArrowOutline);
	}
}

/**
 * Returns the height required to paint the String.
 * @return height required to paint the String
 * @param fm
 * @param s
 */
public int getHeight(FontMetrics fm, String s)
{
	if (s == null || s.length() == 0) {
		return(0);
    }
	int lines = 1;
	for (int i = 0; i < s.length(); i++) {
		if (s.charAt(i) == '\n') {
            lines++;
        }
    }
	return(fm.getHeight() * lines);
}

/**
 * Returns the width required to paint a multi-line String.
 * @return width required to paint a multi-line String
 * @param fm
 * @param text
 */
public int getWidth(FontMetrics fm, String text)
{
	if (text == null || text.length() == 0) {
		return(0);
    }

	// Manually calculate width of multi-line string
	if (text.indexOf('\n') != -1) {
		int start = 0, end = 0, width = 0;
		while ((end = text.indexOf('\n', start)) != -1) {
			width = Math.max(width, fm.stringWidth(text.substring(start, end)));
            start = end + 1;
        }
		return(Math.max(width,
                        fm.stringWidth(text.substring(start, text.length()))));
	}
	else {
		return(fm.stringWidth(text));
    }
}

/**
 * Determines the preferred size of the cell.
 *
 * @param gc the graphics context
 * @param cellInfo cell property information
 * @param o the object to be rendered
 * @return <code>java.awt.Dimension</code> instance containing preferred height and width
 */
public Dimension getPreferredSize(Graphics gc, JCCellInfo cellInfo, Object o)
{
	if (o == null) {
		return(null);
    }

    Font f = cellInfo.getFont();
    if (gc != null && f != null) {
        gc.setFont(f);
    }

	String display = o.toString();
	if (display == null || display.length() == 0) {
		return(new Dimension(0,0));
    }

	FontMetrics fm = gc.getFontMetrics();
	int width = cellInfo.getDrawingArea().width;

    int hAlign = cellInfo.getHorizontalAlignment();
	display = JCWordWrap.wrapText(JCWordWrap.replace(display, "\n", " "),
                                  fm, width, "\n",
                                  (hAlign != JCCellInfo.RIGHT));
	return(new Dimension(getWidth(fm, display), getHeight(fm, display)));
}

}
