/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AbstractRendererToComponent.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:43 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.cell.swing;

import com.klg.jclass.cell.JCCellInfo;
import com.klg.jclass.cell.JCCellRenderer;
import com.klg.jclass.cell.JCComponentCellRenderer;
import com.klg.jclass.cell.JCLightCellRenderer;

import javax.swing.*;
import java.awt.*;

public class AbstractRendererToComponent implements JCCellInfo
 {

protected JCComponentCellRenderer  renderer;
protected JComponent               parentComponent;
protected JComponent               rendererComponent;
protected Object                   value;
protected int                      horizontalAlignment = JCCellInfo.LEFT;

public AbstractRendererToComponent() {}

public AbstractRendererToComponent(JCCellRenderer renderer)
{
    setRenderer(renderer);
}

public void setRenderer(JCCellRenderer renderer)
{
    if (renderer instanceof JCLightCellRenderer) {
        this.renderer =
		    new LightRendererToComponentRenderer((JCLightCellRenderer)renderer);
    }
    else {
        this.renderer = (JCComponentCellRenderer) renderer;
    }
}

public JCCellRenderer getRenderer()
{
    return(renderer);
}

public void setParentComponent(JComponent parentComponent)
{
    this.parentComponent = parentComponent;
}

public JComponent getParentComponent()
{
    return(parentComponent);
}

public void setRendererComponent(JComponent rendererComponent)
{
    this.rendererComponent = rendererComponent;
}

public JComponent getRendererComponent()
{
    return(rendererComponent);
}

public void setValue(Object value)
{
    this.value = value;
}

public Object getValue()
{
    return(value);
}

// JCCellInfo
public Color getBackground()
{
    return((parentComponent == null) ? Color.gray
                                     : parentComponent.getBackground());
}

public Color getForeground()
{
  	return((parentComponent == null) ? Color.black
                                     : parentComponent.getForeground());
}

public Color getSelectedBackground()
{
    return((parentComponent == null) ? Color.blue
                                     : parentComponent.getBackground());
}

public Color getSelectedForeground()
{
    return((parentComponent == null) ? Color.black
                                     : parentComponent.getForeground());
}

public Font getFont()
{
    return((parentComponent == null) ? null
                                     : parentComponent.getFont());
}

public int getHorizontalAlignment()
{
	return(horizontalAlignment);
}

public void setHorizontalAlignment(int horizontalAlignment)
{
    this.horizontalAlignment = horizontalAlignment;
}

public int getVerticalAlignment()
{
	return(JCCellInfo.CENTER);
}

public Insets getMarginInsets()
{
    return((rendererComponent == null) ? new Insets(1, 1, 1, 1)
                                       : rendererComponent.getInsets());
}

public Insets getBorderInsets()
{
    return((rendererComponent == null) ? new Insets(1, 1, 1, 1)
                                       : rendererComponent.getInsets());
}

public int getBorderStyle()
{
    return(0);
}

public Rectangle getDrawingArea()
{
    return((rendererComponent == null) ? null
                                       : rendererComponent.getBounds());
}

public boolean isEditable()
{
    return(true);
}

public boolean isEnabled()
{
    return(true);
}

public boolean getSelectAll()
{
    return(false);
}

public int getClipHints()
{
    return(0);
}

public Class getDataType()
{
    return((value == null) ? null : value.getClass());
}

}
