/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCCellEditorToJTable.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:43 $  $Locker:  $  Quest Software Inc.
package com.klg.jclass.cell.swing;

import com.klg.jclass.cell.JCCellEditor;
import com.klg.jclass.cell.JCCellEditorEvent;
import com.klg.jclass.cell.JCCellEditorListener;

import javax.swing.*;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.table.TableCellEditor;
import java.awt.*;
import java.util.Enumeration;
import java.util.EventObject;
import java.util.Vector;

public class JCCellEditorToJTable extends AbstractEditorToComponent
                                  implements TableCellEditor
{

protected Vector                 cellListeners = new Vector();
protected Vector                 jcCellListeners = new Vector();

transient protected ChangeEvent  changeEvent = null;
transient protected EventObject  lastEvent = null;

public JCCellEditorToJTable() {}

public JCCellEditorToJTable(JCCellEditor editor)
{
    super(editor);
}

public Component getTableCellEditorComponent(JTable table, Object value,
                                             boolean isSelected, int row,
                                             int column)
{
    setComponent(table);
    editor.initialize((AWTEvent)lastEvent, this, value);
    return(editor.getComponent());
}

public Object getCellEditorValue()
{
    return(editor.getCellEditorValue());
}

public boolean isCellEditable(EventObject anEvent)
{
    lastEvent = anEvent;
    return(true);
}

public boolean shouldSelectCell(EventObject anEvent)
{
    lastEvent = anEvent;
    return(true);
}

public boolean stopCellEditing()
{
    if (editor.stopCellEditing()) {
        Enumeration e = cellListeners.elements();
    	while (e.hasMoreElements()) {
    		if (changeEvent == null) {
	    	    changeEvent = new ChangeEvent(this);
            }
		    CellEditorListener listener = (CellEditorListener)e.nextElement();
            listener.editingStopped(changeEvent);
        }
        return(true);
    }
    return(false);
}

public void cancelCellEditing()
{
    editor.cancelCellEditing();
    Enumeration e = cellListeners.elements();
    while (e.hasMoreElements()) {
        if (changeEvent == null) {
            changeEvent = new ChangeEvent(this);
        }
        CellEditorListener listener = (CellEditorListener)e.nextElement();
        listener.editingStopped(changeEvent);
    }
}


public void addCellEditorListener(CellEditorListener l)
{
    JCCellEditorListener listener = new JCCellEditorToCellEditorListener(l);
    cellListeners.add(l);
    jcCellListeners.add(listener);
    editor.addCellEditorListener(listener);
}

public void removeCellEditorListener(CellEditorListener l)
{
    for (int i = 0; i < cellListeners.size(); i++) {
        if (l == cellListeners.elementAt(i)) {
            cellListeners.remove(i);
            JCCellEditorListener jcCellListener =
                (JCCellEditorListener)jcCellListeners.remove(i);
            editor.removeCellEditorListener(jcCellListener);
            i--;
        }
    }
}


public Color getSelectedBackground()
{
    return(((JTable)component).getSelectionBackground());
}

public Color getSelectedForeground() {
    return(((JTable)component).getSelectionForeground());
}

public Insets getMarginInsets()
{
    return(component.getInsets());
}

public Insets getBorderInsets()
{
    return(component.getInsets());
}


public class JCCellEditorToCellEditorListener implements JCCellEditorListener
{
    protected CellEditorListener cellListener;

    public JCCellEditorToCellEditorListener(CellEditorListener cellListener)
    {
        this.cellListener = cellListener;
    }

    public void editingStopped(JCCellEditorEvent e)
    {
        cellListener.editingStopped(new ChangeEvent(e.getSource()));
    }

    public void editingCanceled(JCCellEditorEvent e)
    {
        cellListener.editingCanceled(new ChangeEvent(e.getSource()));
    }
}

}
