/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: LightRendererToComponentRenderer.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:43 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.cell.swing;

import com.klg.jclass.cell.JCCellInfo;
import com.klg.jclass.cell.JCComponentCellRenderer;
import com.klg.jclass.cell.JCLightCellRenderer;

import javax.swing.*;
import java.awt.*;

/**
 * This class wraps a light cell renderer and turns it into a component cell
 * renderer. It doesn't work for all cases.  JComboBox and JTree both require
 * the preferred size to be calculated be for a Graphics Context is available
 * for the light renderer to calculate its preferred size with.  This means the
 * renderer will not look right if it is used with these controls.
 */
public class LightRendererToComponentRenderer extends JComponent
                                              implements JCComponentCellRenderer
{

protected JCLightCellRenderer  lightRenderer;
protected JCCellInfo           cellInfo;
protected Object               value;
protected boolean              selected;

public LightRendererToComponentRenderer(JCLightCellRenderer renderer)
{
    lightRenderer = renderer;
}

public Component getRendererComponent(JCCellInfo cellInfo, Object o,
                                      boolean selected)
{
    this.cellInfo = cellInfo;
    value         = o;
    this.selected = selected;
    return(this);
}

public Object getValue()
{
	return(value);
}

public void paintComponent(Graphics gc)
{
    Dimension size = getSize();
    if (selected) {
	    gc.setColor(cellInfo.getSelectedBackground());
    }
    else {
        gc.setColor(cellInfo.getBackground());
    }
	gc.fillRect(0, 0, size.width, size.height);

	super.paintComponent(gc);

	lightRenderer.draw(gc, cellInfo, value, selected);

	// JDK1.2.2 does not remove the renderer from the CellRendererPane
	// after it is drawn. See Sun's bug parade,
	// http://developer.java.sun.com/developer/bugParade/bugs/4226215.html
	// This is fixed in jdk1.3, but for now remove it ourselves.
	Container parent = this.getParent();
	if (parent != null && parent instanceof CellRendererPane) {
	    parent.remove(this);
	}
}

public Dimension getPreferredSize()
{
	// We need a gc to get the preferred size info.
    Graphics gc = this.getGraphics();

	// If the renderer has no parent, i.e. its currently not the child of a
	// CellRendererPane, the gc returned will be null.  This means
	// that if getPreferredSize is called, we need to get the information
	// somewhere else.  The cellInfo object may store the parent object
	// from which it gets its information.  Use this if it is available.
    if (gc == null && cellInfo != null) {
		if (cellInfo instanceof AbstractRendererToComponent) {
			AbstractRendererToComponent abs =
                (AbstractRendererToComponent)cellInfo;
			JComponent parent = abs.getParentComponent();
			if (parent != null) {
				gc = parent.getGraphics();
			}
		}
    }

	if (gc == null) {
        return(new Dimension(17, 17));
    }
    else {
        return(lightRenderer.getPreferredSize(gc, cellInfo, value));
    }
}

}
