/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCValidateEvent.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:43 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.cell.validate;

import java.util.EventObject;

/**
 * Data validation event.  Validate events are typically
 * fired when <code>stopCellEditing()</code> is called.
 * 
 * @see com.klg.jclass.cell.JCCellEditor
 */
public class JCValidateEvent extends EventObject {

protected Object   old_value;
protected Object   value;
protected boolean  beep;
protected boolean  valid;

public JCValidateEvent(Object src)
{
	super(src);
}

public JCValidateEvent(Object source, Object old_value, Object value,
                       boolean valid)
{
	this(source);
	this.old_value = old_value;
	this.value = value;
	this.beep = false;
	this.valid = valid;
}

public JCValidateEvent(Object source, Object old_value, Object value,
                       boolean valid, boolean beep)
{
	this(source, old_value, value, valid);
	this.beep = beep;
}

/**
 * Returns whether the event will cause a beep or not.
 * @return whether the event will cause a beep or not
 */
public boolean getBeep()
{
	return(beep);
}

/**
 * Sets whether the event will cause a beep or not.
 * @param beep
 */
public void setBeep(boolean beep)
{
	this.beep = beep;
}

/**
 * Returns the new value of the field.
 * @return new value of the field
 */
public Object getValue()
{
	return(value);
}

/**
 * In the <code>valueChangedBegin</code> method, this may be called to changed
 * the new value. The new value is run through the validator associated with
 * the source to ensure that it is valid.
 * @param value
 */
public void setValue(Object value)
{
	this.value = value;
}

/**
 * Returns the previous value of the field.
 * @return previous value of the field
 */
public Object getOldValue()
{
	return(old_value);
}

/**
 * Returns whether the value is considered valid or not.
 * @return whether the value is considered valid or not
 */
public boolean isValid()
{
	return(valid);
}

/**
 * Sets whether the current value is considered valid or not.
 * @param valid
 */
public void setValid(boolean valid)
{
	this.valid = valid;
}

}


