/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ValidateSupport.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:43 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.cell.validate;

import com.klg.jclass.util.JCListenerList;

import java.io.*;
import java.util.Enumeration;

/**
 * ValidateSupport is a convenience class that can be used
 * by cell editors to implement standard validate event handling.
 * In addition to registering validate event listeners, it provides
 * convenient methods for firing events.
 */
public class ValidateSupport implements Serializable
{

/** Manages ValidateListeners */
protected JCListenerList validateListeners;

/**
 * Creates a ValidateSupport instance.
 */
public ValidateSupport() {}

/**
 * Adds a new ValidateListener to the list.
 */
public void addValidateListener(JCValidateListener l)
{
	validateListeners = JCListenerList.add(validateListeners, l);
}

/**
 * Removes a ValidateListener from the list.
 */
public void removeValidateListener(JCValidateListener l)
{
	validateListeners = JCListenerList.remove(validateListeners, l);
}

/**
 * Sends a valueChangedBegin message to all listeners.
 */
public void fireValueChangedBegin(JCValidateEvent event)
{
	if (event == null) {
        return;
    }
	Enumeration e = JCListenerList.elements(validateListeners);
	while (e.hasMoreElements()) {
		JCValidateListener listener = (JCValidateListener)e.nextElement();
		listener.valueChangedBegin(event);
	}
}

/**
 * Sends a valueChangedEnd message to all listeners.
 */
public void fireValueChangedEnd(JCValidateEvent event)
{
	if (event == null) {
        return;
    }
	Enumeration e = JCListenerList.elements(validateListeners);
	while (e.hasMoreElements()) {
		JCValidateListener listener = (JCValidateListener)e.nextElement();
		listener.valueChangedEnd(event);
	}
}

/**
 * Sends a stateIsInvalid message to all listeners.
 */
public void fireStateIsInvalid(JCValidateEvent event)
{
	if (event == null) {
        return;
    }
	Enumeration e = JCListenerList.elements(validateListeners);
	while (e.hasMoreElements()) {
		JCValidateListener listener = (JCValidateListener)e.nextElement();
		listener.stateIsInvalid(event);
	}
}

/**
 * Fires the validation events in the standard order.
 */
public boolean fireValidateEvents(JCValidateEvent ev)
{
	// Post event to inform any listeners that validation is about to begin
	fireValueChangedBegin(ev);

	// Get the valid state back out of the event to make sure we aren't
	// affected if someone trys to illegally change it in the events
	// we are about to post
	boolean valid = ev.isValid();

	if (ev.isValid() == false) {
		// Data is invalid
		ev.setBeep(true);
		fireStateIsInvalid(ev);
	}
	else {
		// Data is valid
		fireValueChangedEnd(ev);
	}

	return(valid);
}

}
