/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ActionHandler.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:43 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

/**
 * ActionHandler is an abstract base class for any
 * action handling object in JCChart.  It defines
 * methods for starting, animating and ending an
 * action.
 * 
 * JCChartArea will create an instance of an ActionHandler
 * when an action is started, and will call start(), animate()
 * and end during mouseDown(), mouseDrag() and mouseUp()
 * respectively.  The appropraite ActionHandler is defined 
 * by the EventTrigger object.  
 * @see com.klg.jclass.chart.EventTrigger
 */
public abstract class ActionHandler {

/** Parent object on which the action occurs */
protected JCChartArea parent = null;

/**
 * Constructs an action handler connected to a JCChartArea instance.
 */
protected ActionHandler() {}

protected void setParent(JCChartArea p)
{
	parent = p;
}

static protected ActionHandler makeHandler(int triggerEnum, JCChartArea pr)
{
	// These event triggers don't have corresponding handlers.
	if (triggerEnum == EventTrigger.CUSTOMIZE ||
		triggerEnum == EventTrigger.PICK)
    {
		return(null);
    }

	ActionHandler rval = null;
	try {
        String name = "com.klg.jclass.chart." +
                      JCChartEnumMappings.trigger_strings[triggerEnum + 1] +
                      "Handler";
		rval = (ActionHandler)Class.forName(name).newInstance();
	}
	catch (Exception e) {
		e.printStackTrace(System.out);
		return null;
	}
	rval.setParent(pr);
	return(rval);
}

/**
 * Called by JCChartArea when the action is started.
 */
protected abstract void start(int x, int y);

/**
 * Called by JCChartArea when the action is to be
 * animated
 */
protected abstract void animate(int x, int y);

/**
 * Called by JCChartArea when the chart area has been
 * refreshed and the animation must be redrawn.
 */
protected abstract void reanimate(int x, int y);

/**
 * Called by JCChartArea when the action is over.
 */
protected abstract void end(int x, int y);

protected JCChart getChart()
{
	if (parent == null) {
        return(null);
    }
	return((JCChart) parent.getParent());
}

}

