/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AnnotationHandler.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:43 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import com.klg.jclass.util.legend.JCLegend;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.awt.geom.Line2D;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Vector;
import java.util.List;

class AnnotationHandler implements java.io.Serializable {

/** Vector of JCValueLabel instances. */
protected Vector valueLabels = new Vector();

/** user value labels list */
protected Vector userValueLabels = new Vector();

/** Holds major, minor and ortho extents for axis */
protected Extents extents = new Extents();

/** Internal property holding the real annotation placement */
protected int realPlacement = JCAxis.MIN;

protected int      anchor = JCLegend.NORTH;
protected JCAxis   parent = null;
protected boolean  annotationVisible = true;

transient private JCChartDateFormat dateFormatter = null;

public AnnotationHandler(JCAxis ax)
{
    parent = ax;
}

public JCChartDateFormat getDateFormatter()
{
    if (dateFormatter == null) {
        dateFormatter = new JCChartDateFormat();
    }
    return(dateFormatter);
}

/**
 * Calculates the major, minor and orthogonal extents of the axis.
 */
protected void calcAnnotationExtents(List tickList)
{
    extents.major = extents.minor = extents.ortho = 0;
    if (tickList == null) {
        return;
    }
    int nTicks = tickList.size();
    Extents ext = null;
    JCAnno tickObj = null;
    for (int i=0; i < nTicks; i++) {
        tickObj = (JCAnno)tickList.get(i);
        if (tickObj != null &&
            (tickObj.getDrawLabels() ||
             parent.annotationMethod != JCAxis.VALUE))
        {
            ext = calcAnnotationExtents(tickObj);
            if (ext != null) {
                // Get the maximum extent values:
                extents.minor = Math.max(extents.minor, ext.minor);
                extents.major = Math.max(extents.major, ext.major);
                extents.ortho = Math.max(extents.ortho, ext.ortho);
            }
        }
    }
}

/**
 * Calculates the major, minor and orthogonal extents for a given
 * JCAnno object.
 */
protected Extents calcAnnotationExtents(JCAnno tickObj)
{
    if (!parent.visible || !annotationVisible || tickObj == null) {
        return null;
    }

    Extents extents = new Extents();

    // Calculate extents based on axis label

    if (parent.getAnnotationRotation() == JCAxis.ROTATE_OTHER &&
        !parent.isPolar() && !parent.isRadar())
    {
        double theta = parent.getAnnotationRotationAngle(JCChartUtil.RADIANS);
        // If the annotation rotation angle is negative,
        // convert it to a positive value.
        if (theta < 0) {
            theta += JCChartUtil.M_2PI;
        }
        extents.minor = getRotatedAnnoMinorExtent(
                            theta, tickObj.getLabelExtent());
        extents.major = getRotatedAnnoMajorExtent(
                            extents.minor, theta, tickObj.getLabelExtent());
        extents.ortho = getRotatedAnnoOrthoExtent(
                            theta, tickObj.getLabelExtent());
    }
    else {
        // Calculate the minor and major extents.  We need to check
        // the extents of the min and max visible labels.
        int minVisibleIndex = getMinVisibleLabel();
        int maxVisibleIndex = getMaxVisibleLabel();
        if (minVisibleIndex >= 0 && minVisibleIndex < valueLabels.size() &&
            maxVisibleIndex >= 0 && maxVisibleIndex < valueLabels.size())
        {
            JCValueLabel minVisibleLabel =
                (JCValueLabel)valueLabels.elementAt(minVisibleIndex);
            JCValueLabel maxVisibleLabel =
                (JCValueLabel)valueLabels.elementAt(maxVisibleIndex);
            minVisibleLabel.label.recalc();
            maxVisibleLabel.label.recalc();

            if (parent.i_vertical) {
                extents.minor = maxVisibleLabel.label.getHeight()/2+1;
                extents.major = minVisibleLabel.label.getHeight()/2+1;
            }
            else {
                extents.minor = minVisibleLabel.label.getWidth()/2+1;
                extents.major = maxVisibleLabel.label.getWidth()/2+1;
            }
        }
        // Calculate orthogonal extent.  For this, we need to check
        // the extents of all visible labels, not just the min and max
        // labels.
        int first = firstVisibleLabel();
        int last = lastVisibleLabel();
        if (first >= 0 && first < valueLabels.size() &&
            last >= 0 && last < valueLabels.size())
        {
            for (int i = first; i <= last; i++) {
                JCValueLabel vl = (JCValueLabel)valueLabels.elementAt(i);
                vl.label.recalc();
                int newOrth = parent.i_vertical ? vl.label.getWidth()
                                                : vl.label.getHeight();
                newOrth += tickObj.getLabelExtent() + 1;
                if (newOrth > extents.ortho) {
                    extents.ortho = newOrth;
                }
            }
        }
    }

    // Account for the axis being reversed.
    if (parent.reversed &&
        parent.getAnnotationRotation() != JCAxis.ROTATE_OTHER)
    {
        int temp = extents.minor;
        extents.minor = extents.major;
        extents.major = temp;
    }

    // Account for the title
    if (parent.title.visible) {
        if (parent.i_vertical) {
            switch (parent.title.getEffectivePlacement()) {
            case JCLegend.NORTHEAST:
            case JCLegend.NORTHWEST:
                extents.minor += parent.title.getHeight()+1;
                //extents.ortho += parent.title.getWidth()+1;
                extents.ortho = Math.max(parent.title.getWidth()+1,
                                         extents.ortho);
                break;
            case JCLegend.SOUTHEAST:
            case JCLegend.SOUTHWEST:
                extents.major += parent.title.getHeight()+1;
                //extents.ortho += parent.title.getWidth()+1;
                extents.ortho = Math.max(parent.title.getWidth()+1,
                                         extents.ortho);
                break;
            case JCLegend.EAST:
            case JCLegend.WEST:
                extents.ortho += parent.title.getWidth()+1;
                break;
            }
        }
        else {
            // Only affects orth extents
            switch (parent.title.getEffectivePlacement()) {
            case JCLegend.NORTH:
            case JCLegend.SOUTH:
                extents.ortho += parent.title.getHeight()+1;
                break;
            }
        }
    }
    return extents;
}


/**
 * Makes an educated guess regarding the location of the annotations
 * relative to the axis.
 */
protected void calcAnchor()
{
    realPlacement = parent.placement.value;
    if (realPlacement == JCAxis.AUTOMATIC) {
        realPlacement = JCAxis.MIN;
    }

    boolean orgNearerMax = false;
    if (parent.otherAxis != null)
        orgNearerMax =
            (parent.otherAxis.origin.value - parent.otherAxis.min.value >
             parent.otherAxis.max.value - parent.otherAxis.origin.value);

    if (parent.i_vertical) {
        if (realPlacement == JCAxis.MAX ||
            (realPlacement == JCAxis.ORIGIN && orgNearerMax))
        {
            anchor = JCLegend.EAST;
        }
        else {
            anchor = JCLegend.WEST;
        }
        if (parent.otherAxis != null && parent.otherAxis.reversed) {
            switch (anchor) {
            case JCLegend.EAST:
                anchor = JCLegend.WEST;
                break;
            case JCLegend.WEST:
                anchor = JCLegend.EAST;
                break;
            }
        }
    }
    else {
        if (realPlacement == JCAxis.MAX ||
            (realPlacement == JCAxis.ORIGIN && orgNearerMax))
        {
            anchor = JCLegend.NORTH;
        }
        else {
            anchor = JCLegend.SOUTH;
        }
        if (parent.otherAxis != null && parent.otherAxis.reversed) {
            switch (anchor) {
            case JCLegend.NORTH:
                anchor = JCLegend.SOUTH;
                break;
            case JCLegend.SOUTH:
                anchor = JCLegend.NORTH;
                break;
            }
        }
    }
}

/**
 * Checks to see if all the annotations will fit, and makes the
 * appropriate adjustments.
 */
protected void checkAnnotations()
{
    if (!annotationVisible) {
        return;
    }
    if (valueLabels == null || valueLabels.size() < 1) {
        return;
    }
    if (parent.annotationMethod == JCAxis.VALUE) {
        checkValueAnnotations();
    }
    else if (parent.annotationMethod == JCAxis.TIME_LABELS) {
        checkTimeAnnotations();
    }
    else {
        checkUserProvidedAnnotations();
    }
}

/**
 * Moves all the axis annotations to the correct positions.
 */
protected void layout() {
    if (parent.isPolarRadar()) {
        return;
    }
    int arraySize = valueLabels.size();
    for (int i = 0; i < arraySize; i++) {
        JCValueLabel vl = (JCValueLabel)valueLabels.elementAt(i);
        if (!(parent.isLogarithmic() && vl.value <= 0.0)) {
            int offset = parent.toPixel(vl.value);
            JCAnno annoObj = vl.getAnno();
            int labelExtent = JCAnno.DEFAULT_LABEL_EXTENT;
            if (annoObj != null) {
                labelExtent = annoObj.getLabelExtent();
            }
            // Position label based on legend anchor
            layout(vl.label, offset, labelExtent);
        }
    }
}

/**
 * Convert a data value in unit time units to a Date value.  Note that
 * interpolation may need to be done.  Units that are days or longer can be
 * different lengths because of daylight savings time, leap years, and the
 * simple fact that all months are not the same length.
 * @param value data value to convert
 * @param unit Time unit to use for conversion
 * @return Date instance representing time value
 */
protected Date dataToTime(double value, long unit)
{
    Date base = parent.timeBase;
    if (unit < JCAxis.DAYS) {
        // For these units a linear transformation is exact
        return(new Date(base.getTime() + (long)(value * unit)));
    }
    Date t = new Date(base.getTime());

    boolean weekly = unit >= 7 * JCAxis.DAYS && unit < JCAxis.MONTHS;
    long ix = (long)value;
    double f = value - ix;
    long inc = weekly ? ix * 7 : ix;

    getDateFormatter().addToUnit(t, unit, (int)inc);

    // Watch out for integer overflow and any possible bugs in the Date or
    // Calendar code.
    try {
        if (value < 0 && t.after(base)) {
            return(null);
        }
        if (value > 0 && t.before(base)) {
            return(null);
        }
    }
    catch (Exception e) {
        // Any kind of exception likely means t is out-of-range
        return(null);
    }

    if (10.0 + f != 10.0) {
        //  Interpolate fractional part of value
        Date t1 = new Date(base.getTime());
        if (value < 0) {
            ix -= 1;
            f = -f;
        }
        else {
            ix += 1;
        }
        inc = weekly ? ix * 7 : ix;
        getDateFormatter().addToUnit(t1, unit, (int)inc);
        Date t0 = t;
        t = new Date(t0.getTime() + (long)(f * (t1.getTime() - t0.getTime())));
    }
    return(t);
}

/**
 * Converts a date value into a data value in the provided units.  Note that the
 * transformation may not be exact.  Units that are days or longer can be
 * different lengths because of daylight savings time, leap years, and the
 * simple fact that all months are not the same length.
 * @param value Date object to be converted
 * @param unit Time unit to use for conversion
 * @return Data value
 */
protected double timeToData(Date value, long unit)
{
    double  hi, lo, mid, approx;
    double  d_unit = (double)unit;

    if (value == null) {
        return(0.0);
    }

    long base = parent.timeBase.getTime();
    Date minT = JCChartDateFormat.getMinTime();
    Date maxT = JCChartDateFormat.getMaxTime();

    // Check bounds of input value
    if (value.before(minT)) {
        return((minT.getTime() - base) / d_unit);
    }
    if (value.after(maxT)) {
        return((maxT.getTime() - base) / d_unit);
    }

    // Try a simple linear transformation.
    approx = mid = ((value.getTime() - base) / d_unit);

    if (unit < JCAxis.DAYS) {
        // For these units the above transform is exact
        return(mid);
    }

    // Must use bisection algorithm to find exact value
    lo = mid - 2.0;
    hi = mid + 2.0;
    Date t0 = dataToTime(lo, unit);
    Date t1 = dataToTime(hi, unit);

    if (t0 == null || t0.after(value)) {
        lo = (minT.getTime() - base)/d_unit;
        t0 = dataToTime(lo, unit);
    }
    if (t1 == null || t1.before(value)) {
        hi = (maxT.getTime() - base)/d_unit;
        t1 = dataToTime(hi, unit);
    }

    while ((hi-lo) > 1e-6) { // i.e. while (hi>lo)
        mid = (lo + hi) / 2;
        Date t = dataToTime(mid, unit);
        if (t == null) {
            // Error - bail out with initial approximation
            return approx;
        }
        if (t.before(value) || t0 == null) {
            lo = mid;
            t0 = t;
        }
        else if (t.after(value) || t1 == null) {
            hi = mid;
            t1 = t;
        }
        else {
            return(mid);    // Exact!
        }
    }

    return(mid); // Have to settle for an approximation
}

protected void checkValueAnnotations() {
    checkRotatedValueLabels();
}

private void checkTimeAnnotations()
{
    if (parent.getAnnotationRotation() == JCAxis.ROTATE_OTHER &&
        !parent.isPolar() && !parent.isRadar())
    {
        checkRotatedValueLabels();
        return;
    }

    double range = (parent.max.value - parent.min.value);
    int nlabels = valueLabels.size();
    int first = firstVisibleLabel();
    int last = lastVisibleLabel();

    Dimension ext = new Dimension(0, 0);
    timeLabelExtents(ext);

    int size = (parent.i_vertical) ? parent.getHeight() : parent.getWidth();
    int labelSpace = 0;
    if (nlabels >= 2) {
        // Determine labelSpace from the 1st and 2nd time labels.  Search
        // for the first pair of labels that have the drawLabel flag set
        // to true.
        for (int i=0; i < valueLabels.size()-1; i++) {
            JCValueLabel vl1 = (JCValueLabel)valueLabels.elementAt(i);
            JCValueLabel vl2 = (JCValueLabel)valueLabels.elementAt(i+1);
            if (vl1.drawLabel && vl2.drawLabel) {
                labelSpace =
                    (int)JCChartUtil.precCorrect(0,
                         size* (Math.abs(vl2.value- vl1.value) / range));
                break;
            }
        }
        labelSpace = Math.max(labelSpace, 1);
    }
    else {
        labelSpace = 1;
    }
    int sz = ((parent.i_vertical) ? ext.height : ext.width);
    int nskip = (sz + parent.gap*2) / labelSpace;
    boolean skip = (nskip >= 1);
    nskip++;
    if (first >= 0) {
        for (int i = first; i <= last; i++) {
            JCValueLabel vl = (JCValueLabel)valueLabels.elementAt(i);
            ChartText ct = vl.getChartText();
            ct.visible = checkLocation(vl);
            if (!ct.visible) continue;
            if (skip) {
                vl.getChartText().visible =
                                    (((i-first) % nskip) == 0);
            }
        }
    }
}

private boolean checkLocation(JCValueLabel vl)
{
    if (vl == null) {
        return false;
    }
    ChartText ct = vl.getChartText();
    if (parent.i_vertical) {
        int middle = ct.getTop() + ct.getHeight()/2;
        if (middle < parent.getTop() ||
            middle > (parent.getTop() + parent.getHeight()))
        {
            return(false);
        }
        return(true);
    }
    else {
        int middle = ct.getLeft() + ct.getWidth()/2;
        if (middle < parent.getLeft() ||
            middle > (parent.getLeft() + parent.getWidth()))
        {
            return(false);
        }
        return(true);
    }
}

/**
 * This method is used to check for overlap between rotated
 * annotations.  Unlike checkUserProvidedAnnotations(),
 * this method looks for overlap between any pair of visible labels
 * on the axis.  Overlap may occur between non-adjacent labels if
 * the label height is very large, and the in-between labels are
 * short.
 */
private void checkRotatedValueLabels()
{
    if (!annotationVisible) {
        return;
    }
    if (valueLabels == null || valueLabels.size() < 1) {
        return;
    }

    double theta = parent.getAnnotationRotationAngle(JCChartUtil.RADIANS);

    // First, set the visible flag of all labels that are within range.
    for (int i = 0; i < valueLabels.size(); i++) {
        JCValueLabel vl = (JCValueLabel)valueLabels.elementAt(i);
        ChartText ct = vl.getChartText();
        ct.visible = (vl.value >= parent.min.value &&
                      vl.value <= parent.max.value &&
                      vl.drawLabel);
    }

    // Check for overlap between visible labels.
    for (int j = 0; j < valueLabels.size(); j++) {
        JCValueLabel vl1 = (JCValueLabel)valueLabels.elementAt(j);
        ChartText ct1 = vl1.getChartText();
        if (!ct1.visible) {
            continue;
        }
        Line2D.Double[] lineArray1 = getLine2DArray(ct1, theta);

        // Check for overlap between ct1 and any other visible label
        for (int k = j + 1; k < valueLabels.size(); k++) {
            JCValueLabel vl2 = (JCValueLabel)valueLabels.elementAt(k);
            ChartText ct2 = vl2.getChartText();
            if (!ct2.visible) {
                continue;
            }
            Line2D.Double[] lineArray2 = getLine2DArray(ct2, theta);
            if (doLinesIntersect(lineArray1, lineArray2)) {
                ct2.visible = false;
            }
        }
    }
}

/**
 * This method is overridden by the CircularChartAnnoHandler subclass.
 */
protected boolean skipToNext(JCValueLabel vl)
{
    boolean retVal = false;
    if (vl.value < parent.min.value) {
        ChartText cl = vl.getChartText();
        cl.visible = false;
        retVal = true;
    }
    return retVal;
}

/**
 * This method is overridden by the CircularChartAnnoHandler subclass.
 */
protected boolean jumpToNext(JCValueLabel vl)
{
    boolean retVal = false;
    if (vl.value > parent.max.value) {
        ChartText cl = vl.getChartText();
        cl.visible = false;
        retVal = true;
    }
    return retVal;
}

protected void checkUserProvidedLabels()
{
    if (!annotationVisible) {
        return;
    }
    if (valueLabels == null || valueLabels.size() < 1) {
        return;
    }

    int i = 0;
    Rectangle last = null;
    // First, find a visible label
    for (; i < valueLabels.size(); i++) {
        JCValueLabel vl = (JCValueLabel)valueLabels.elementAt(i);
        if (skipToNext(vl) || !vl.drawLabel) {
            continue;
        }
        ChartText cl = vl.getChartText();
        cl.visible = true;

        last = new Rectangle(cl.getLeft(), cl.getTop(),
                             cl.getWidth() + parent.gap,
                             cl.getHeight() + parent.gap);
        break;

    }

    if (last == null) {
        return;
    }

    for (i += 1; i < valueLabels.size(); i++) {
        JCValueLabel vl = (JCValueLabel)valueLabels.elementAt(i);
        if (jumpToNext(vl) || !vl.drawLabel) {
            continue;
        }
        ChartText cl = vl.getChartText();

        Rectangle c = new Rectangle(cl.getLeft(), cl.getTop(),
                                    cl.getWidth() + parent.gap,
                                    cl.getHeight() + parent.gap);
        if (c.intersects(last)) {
            cl.visible = false;
        }
        else {
            cl.visible = true;
            last = c;
        }
    }
}

/**
 * Used in the point label and value label case to make sure
 * user-provided annotations don't grossly overlap.
 */
protected void checkUserProvidedAnnotations()
{
    if (!annotationVisible) {
        return;
    }
    if (parent.getAnnotationRotation() == JCAxis.ROTATE_OTHER) {
        checkRotatedValueLabels();
        return;
    }
    checkUserProvidedLabels();
}

/**
 * This method returns an array of Line2D.Double representing the
 * outline of a ChartText object rotated about the upper left corner by
 * theta radians.
 */
private Line2D.Double[] getLine2DArray(ChartText cl, double theta)
{
    Rectangle rect = new Rectangle(cl.getLeft(), cl.getTop(),
                                   cl.getWidth() + parent.gap,
                                   cl.getHeight() + parent.gap);
    Line2D.Double[] lineArray = new Line2D.Double[4];
    AffineTransform at = new AffineTransform();
    // Rotate by theta radians about a point at the upper left
    // corner of the ChartText object.
    at.setToRotation(theta, rect.x, rect.y);
    PathIterator iter = rect.getPathIterator(at);
    int i = 0;
    double[] coords = new double[6];
    Point2D.Double lastPoint = null;
    while (!iter.isDone()) {
        int type = iter.currentSegment(coords);
        switch(type) {
        case PathIterator.SEG_MOVETO:
            lastPoint = new Point2D.Double(coords[0], coords[1]);
            break;
        case PathIterator.SEG_LINETO:
            lineArray[i] = new Line2D.Double(lastPoint.x, lastPoint.y,
                                             coords[0], coords[1]);
//          System.out.println("i=" + i + ", x1=" + lastPoint.x +
//                              ", y1=" + lastPoint.y +
//                              ", x2=" + coords[0] + ", y2=" + coords[1]);
            i++;
            lastPoint = new Point2D.Double(coords[0], coords[1]);
            break;
        case PathIterator.SEG_CLOSE:
            break;
        default:
            throw new RuntimeException("Bad PathIterator enum");
        }
        iter.next();
    }
    return lineArray;
}

/**
 * This method determines if any of the line segments in the lines1
 * array intersects any of the line segments in the lines2 array.
 */
private boolean doLinesIntersect(Line2D.Double[] lines1,
                                Line2D.Double[] lines2)
{
    if (lines1 == null || lines2 == null) {
        throw new IllegalArgumentException("Line2D array is null.");
    }
    for (int i = 0; i < lines1.length; i++) {
        for (int j = 0; j < lines2.length; j++) {
            if (lines1[i].intersectsLine(lines2[j])) {
                return(true);
            }
        }
    }
    return(false);
}

/**
 * This method computes the x and y components of each of the width and
 * height parameters, and returns those values in an XYProjection data
 * object.
 * @param width An integer width value.
 * @param height An integer height value.
 * @param theta The rotation angle in radians.
 */
protected XYProjection getXYProjection(int width, int height, double theta)
{
    XYProjection projection = new XYProjection();
    projection.x1 = Math.abs(height * Math.sin(theta));
    projection.y1 = Math.abs(height * Math.cos(theta));
    projection.x2 = Math.abs(width * Math.cos(theta));
    projection.y2 = Math.abs(width * Math.sin(theta));
    return(projection);
}

/**
 * This method computes the x and y components of a ChartText
 * object's width and height, and returns those values in an
 * XYProjection data object.
 * @param cl The rotated ChartText object.
 * @param theta The angle of rotation in radians.
 */
private XYProjection getXYProjection(ChartText cl, double theta)
{
    return(getXYProjection(cl.width.value, cl.height.value, theta));
}

private static final double angleCenteredLabel = Math.PI/18;  // 10 degrees.

/**
 * An array of angle limits which defines a set of computational
 * cases for the display of rotated labels on a vertical axis.
 */
private static final double[] vertAxisAnnoRotationLimit =
{
    Math.PI/2 - angleCenteredLabel,   //  80 degrees
    Math.PI/2,                        //  90 degrees
    Math.PI/2 + angleCenteredLabel,   // 100 degrees
    Math.PI,                          // 180 degrees
    3*Math.PI/2 - angleCenteredLabel, // 260 degrees
    3*Math.PI/2,                      // 270 degrees
    3*Math.PI/2 + angleCenteredLabel, // 280 degrees
    2*Math.PI                         // 360 degrees
};

/**
 * An array of angle limits which defines a set of computational
 * cases for the display of rotated labels on a horizontal axis.
 */
private static final double[] horizAxisAnnoRotationLimit =
{
    angleCenteredLabel,               //  10 degrees
    Math.PI/2,                        //  90 degrees
    Math.PI - angleCenteredLabel,     // 170 degrees
    Math.PI,                          // 180 degrees
    Math.PI + angleCenteredLabel,     // 190 degrees
    3*Math.PI/2,                      // 270 degrees
    2*Math.PI - angleCenteredLabel,   // 350 degrees
    2*Math.PI                         // 360 degrees
};

/**
 * This method returns an integer value identifying the
 * computational case that corresponds to the specified
 * axis label rotation.
 * @param fVertical True if the axis is vertical.
 * @param theta The angle in radians at which axis labels are rotated.
 * @return An integer value in the range 1 to 8 identifying the
 * computational case.
 */
private int getAnnoRotationCase(boolean fVertical, double theta)
{
    int i = -1;
    double[] angleArray = fVertical ?
                vertAxisAnnoRotationLimit : horizAxisAnnoRotationLimit;
    for (i = 0; i < angleArray.length; i++) {
        if (theta <= angleArray[i]) {
            break;
        }
    }
    i++;
    return(i);
}

/**
 * An internal data structure containing x and y deltas.
 */
private class Translation
{
    int dx = 0;
    int dy = 0;
}

/**
 * This method returns the x and y translations required to properly
 * anchor a rotated axis label.
 * @param xyP A data object containing the x and y components of
 * a rotated label's width and height.
 * @param theta The rotation angle in radians.
 * @return A Translation data object containing the required x and y deltas.
 */
private Translation getTranslation(
            XYProjection xyP, double theta, int labelExtent)
{
    // If the annotation rotation angle is negative,
    // convert it to a positive value.
    if (theta < 0) {
        theta += JCChartUtil.M_2PI;
    }
    Translation trans = new Translation();
    if (parent.i_vertical) {
        switch(getAnnoRotationCase(true, theta)) {
        case 1:
            // Compute the horizontal projection of the end of the label
            // onto the y axis.  Anchor the label so that the tick mark
            // is located at the centre of that projection.
            if (anchor == JCLegend.WEST) {
                trans.dx = -(int)Math.round(labelExtent + xyP.x2);
                trans.dy = (int)Math.round(-xyP.y1/2 - xyP.y2 - 1);
            }
            else {
                trans.dx = (int)Math.round(xyP.x1 + labelExtent + 1);
                trans.dy = (int)Math.round(-xyP.y1/2)-1;
            }
            break;
        case 2:
            // Compute the horizontal projection of the label onto the axis.
            // The tick mark is located at the centre of that projection.
            if (anchor == JCLegend.WEST) {
                trans.dx = -(int)Math.round(labelExtent +
                                            xyP.x2);
                trans.dy = 1-(int)Math.round((xyP.y1 + xyP.y2)/2 - xyP.y1);
            }
            else {
                trans.dx = (int)Math.round(xyP.x1 + xyP.x2 +
                                            labelExtent + 1);
                trans.dy = 1-(int)Math.round((xyP.y1 + xyP.y2)/2 - xyP.y1);
            }
            break;
        case 3:
            // Compute the horizontal projection of the label onto the axis.
            // The tick mark is located at the centre of that projection.
            if (anchor == JCLegend.WEST) {
                trans.dx = -Math.round(labelExtent);
                trans.dy = 1-(int)Math.round((xyP.y1 + xyP.y2)/2);
            }
            else {
                trans.dx = (int)Math.round(xyP.x1 + xyP.x2 +
                                            labelExtent + 1);
                trans.dy = (int)Math.round((xyP.y1 - xyP.y2)/2);
            }
            break;
        case 4:
            // Compute the projection of the end of the label onto the y axis.
            // Anchor the label so that the tick mark is located at the centre
            // of that projection.
            if (anchor == JCLegend.WEST) {
                trans.dx = -Math.round(labelExtent + 1);
                trans.dy = (int)Math.round(xyP.y1/2);
            }
            else {
                trans.dx = (int)Math.round(xyP.x1 + xyP.x2 +
                                            labelExtent);
                trans.dy = (int)Math.round(1-xyP.y2 + xyP.y1/2);
            }
            break;
        case 5:
            // Compute the projection of the end of the label onto the y axis.
            // Anchor the label so that the tick mark is located at the centre
            // of that projection.
            if (anchor == JCLegend.WEST) {
                trans.dx = -(int)Math.round(xyP.x1 +
                                            labelExtent + 1);
                trans.dy = (int)Math.round(xyP.y1/2) + 1;
            }
            else {
                trans.dx = (int)Math.round(xyP.x2 +
                                            labelExtent + 1);
                trans.dy = (int)Math.round(xyP.y2 + xyP.y1/2) + 1;
            }
            break;
        case 6:
            // Compute the horizontal projection of the label onto the axis.
            // The tick mark is located at the centre of that projection.
            if (anchor == JCLegend.WEST) {
                trans.dx = -(int)Math.round(xyP.x1 +
                                            labelExtent + 1);
                trans.dy = (int)Math.round((xyP.y1 + xyP.y2)/2 - xyP.y1);
            }
            else {
                trans.dx = Math.round(labelExtent + 1);
                trans.dy = (int)Math.round((xyP.y1 + xyP.y2)/2 - xyP.y1) + 1;
            }
            break;
        case 7:
            // Compute the horizontal projection of the label onto the axis.
            // The tick mark is located at the centre of that projection.
            if (anchor == JCLegend.WEST) {
                trans.dx = -(int)Math.round(labelExtent +
                                            xyP.x1 + xyP.x2);
                trans.dy = (int)Math.round((xyP.y1 + xyP.y2)/2);
            }
            else {
                trans.dx = Math.round(labelExtent);
                trans.dy = (int)Math.round((xyP.y2 + xyP.y1)/2);
            }
            break;
        case 8:
            // Compute the projection of the end of the label onto the axis.
            // The tick mark is located at the centre of that projection.
            if (anchor == JCLegend.WEST) {
                trans.dx = -(int)Math.round(labelExtent +
                                            xyP.x1 + xyP.x2);
                trans.dy = (int)Math.round(xyP.y2 - xyP.y1/2 - 1);
            }
            else {
                trans.dx = Math.round(labelExtent + 1);
                trans.dy = -(int)Math.round(xyP.y1/2 + 1);
            }
            break;
        default:
            throw new RuntimeException(
                "Invalid rotation case for vertical axis");
        }
    }
    else {
        //  Horizontal Axis:
        switch(getAnnoRotationCase(false, theta)) {
        case 1:
            // Compute the vertical projection of the label onto the x axis.
            // Anchor the label so that the tick mark is located at the
            // centre of that projection.
            if (anchor == JCLegend.NORTH) {
                trans.dx = 1-(int)Math.round((xyP.x1 + xyP.x2)/2);
                trans.dy = -(int)Math.round(xyP.y1 + xyP.y2 +
                                    labelExtent + 2);
            }
            else {
                trans.dx = 1-(int)Math.round((xyP.x1 + xyP.x2)/2);
                trans.dy = Math.round(labelExtent + 1);
            }
            break;
        case 2:
            // Compute the vertical projection of the end of the label
            // onto the axis.  The tick mark is located at the centre
            // of that projection.
            if (anchor == JCLegend.NORTH) {
                trans.dx = 1-(int)Math.round(xyP.x2 - xyP.x1/2);
                trans.dy = -(int)Math.round(xyP.y1 + xyP.y2 +
                                    labelExtent + 2);
            }
            else {
                trans.dx = (int)Math.round(xyP.x1/2 + 1);
                trans.dy = Math.round(labelExtent + 1);
            }
            break;
        case 3:
            // Compute the vertical projection of the end of the label
            // onto the axis.  The tick mark is located at the centre
            // of that projection.
            if (anchor == JCLegend.NORTH) {
                trans.dx = (int)Math.round(xyP.x2 + xyP.x1/2);
                trans.dy = -(int)Math.round(xyP.y2 +
                                            labelExtent + 1);
            }
            else {
                trans.dx = (int)Math.round(xyP.x1/2);
                trans.dy = (int)Math.round(xyP.y1 + labelExtent + 2);
            }
            break;
        case 4:
            // Compute the vertical projection of the label onto the
            // x axis.  Anchor the label so that the tick mark is
            // located at the centre of that projection.
            if (anchor == JCLegend.NORTH) {
                trans.dx = (int)Math.round((xyP.x1 + xyP.x2)/2 - xyP.x1);
                trans.dy = -(int)Math.round(xyP.y2 +
                                        labelExtent + 1);
            }
            else {
                trans.dx = (int)Math.round((xyP.x1 + xyP.x2)/2 - xyP.x1);
                trans.dy = (int)Math.round(xyP.y1 + labelExtent + 2);
            }
            break;
        case 5:
            // Compute the vertical projection of the label onto the
            // x axis.  Anchor the label so that the tick mark is
            // located at the centre of that projection.
            if (anchor == JCLegend.NORTH) {
                trans.dx = (int)Math.round((xyP.x2 - xyP.x1)/2);
                trans.dy = -Math.round(labelExtent + 1);
            }
            else {
                trans.dx = (int)Math.round((xyP.x2 - xyP.x1)/2);
                trans.dy = (int)Math.round(xyP.y1 + xyP.y2 +
                                            labelExtent + 2);
            }
            break;
        case 6:
            // Compute the vertical projection of the end of the label
            // onto the axis. The tick mark is located at the centre
            // of that projection.
            if (anchor == JCLegend.NORTH) {
                trans.dx = -(int)Math.round(xyP.x1/2 + 1);
                trans.dy = -Math.round(labelExtent + 1);
            }
            else {
                trans.dx = (int)Math.round(-xyP.x1/2 + xyP.x2) - 1;
                trans.dy = (int)Math.round(xyP.y1 + xyP.y2 +
                                    labelExtent + 1);
            }
            break;
        case 7:
            // Compute the vertical projection of the end of the label
            // onto the axis. The tick mark is located at the centre
            // of that projection.
            if (anchor == JCLegend.NORTH) {
                trans.dx = -(int)Math.round(xyP.x1/2 + 1);
                trans.dy = -(int)Math.round(xyP.y1 +
                                            labelExtent + 2);
            }
            else {
                trans.dx = -(int)Math.round(xyP.x2 + xyP.x1/2);
                trans.dy = (int)Math.round(xyP.y2 +
                                    labelExtent);
            }
            break;
        case 8:
            // Compute the vertical projection of the label onto the
            // x axis.  The tick mark is located at the centre of
            // that projection.
            if (anchor == JCLegend.NORTH) {
                trans.dx = -(int)Math.round((xyP.x1 + xyP.x2)/2);
                trans.dy = -(int)Math.round(xyP.y1 +
                                            labelExtent + 2);
            }
            else {
                trans.dx = -(int)Math.round((xyP.x1 + xyP.x2)/2);
                trans.dy = (int)Math.round(xyP.y2 + labelExtent);
            }
            break;
        default:
            throw new RuntimeException(
                "Invalid rotation case for horizontal axis");
        }
    }
    return(trans);
}

/**
 * Returns the minor extent for axis labels rotated at angle theta.
 * @param theta The angle (in radians) at which the labels are rotated.
 * @return The minor extent in pixels.
 */
private int getRotatedAnnoMinorExtent(double theta, int labelExtent)
{
    int extent = 0;
    int maxExtent = 0;
    int first = firstVisibleLabel();
    int last = lastVisibleLabel();
    // Check all visible labels:
    for (int i = first; i <= last; i++) {
        JCValueLabel vl = (JCValueLabel)valueLabels.elementAt(i);
        ChartText cl = vl.getChartText();
        cl.recalc();
        double value = vl.getValue();
        if (!cl.isVisible()         ||
            value < parent.getMin() ||
            value > parent.getMax())
        {
            continue;
        }
        XYProjection xyP =
            getXYProjection(cl.width.value, cl.height.value, theta);
        Translation trans = getTranslation(xyP, theta, labelExtent);
        if (parent.i_vertical) {
            int offset = parent.toPixel(value) + parent.pixelStart();
            if (offset < 0) {
                continue;
            }
            // Note:  for a vertical axis, parent.pixelStart()
            //        returns -parent.getTop()
            switch(getAnnoRotationCase(true, theta)) {
            case 1:
            case 2:
                extent = Math.round(-trans.dy) - offset;
                break;
            case 3:
            case 4:
                extent = (int)Math.round(-trans.dy + xyP.y1) - offset;
                break;
            case 5:
            case 6:
                extent = (int)Math.round(-trans.dy + xyP.y1 + xyP.y2) - offset;
                break;
            case 7:
            case 8:
                extent = (int)Math.round(-trans.dy + xyP.y2) - offset;
                break;
            default:
                throw new RuntimeException(
                    "Invalid annotation rotation case for vertical axis");
            }
        }
        else {
            // Horizontal Axis:
            int offset = parent.toPixel(value) - parent.pixelStart();
            if (offset < 0) {
                continue;
            }
            switch(getAnnoRotationCase(false, theta)) {
            case 1:
            case 2:
                extent = (int)Math.round(-trans.dx + xyP.x1) - offset;
                break;
            case 3:
            case 4:
                extent = (int)Math.round(-trans.dx + xyP.x1 + xyP.x2) - offset;
                break;
            case 5:
            case 6:
                extent = (int)Math.round(-trans.dx + xyP.x2) - offset;
                break;
            case 7:
            case 8:
                extent = Math.round(-trans.dx) - offset;
                break;
            default:
                throw new RuntimeException(
                    "Invalid annotation rotation case for horizontal axis");
            }
        }
        maxExtent = Math.max(extent, maxExtent);
    }
    return(maxExtent);
}

/**
 * Returns the major extent for axis labels rotated at angle theta.
 * @param minorExtent The minor extent (in pixels) for this axis.
 * @param theta The angle (in radians) at which the labels are rotated.
 * @return The major extent in pixels.
 */
private int getRotatedAnnoMajorExtent(int minorExtent, double theta,
                                      int labelExtent) {
    int extent = 0;
    int maxExtent = 0;
    int first = firstVisibleLabel();
    int last = lastVisibleLabel();
    for (int i = first; i <= last; i++) {
        JCValueLabel vl = (JCValueLabel)valueLabels.elementAt(i);
        ChartText cl = vl.getChartText();
        cl.recalc();
        double value = vl.getValue();
        if (!cl.isVisible()         ||
            value < parent.getMin() ||
            value > parent.getMax())
        {
            continue;
        }
        XYProjection xyP =
            getXYProjection(cl.width.value, cl.height.value, theta);
        Translation trans = getTranslation(xyP, theta, labelExtent);
        if (parent.i_vertical) {
            int offset = 0;
            if (parent.reversed) {
                offset = (-parent.pixelStart() + parent.pixelLength() -
                          parent.toPixel(value));
            }
            else {
                offset = (minorExtent + parent.pixelLength() -
                          parent.toPixel(value));
            }
            if (offset < 0) {
                continue;
            }
            switch(getAnnoRotationCase(true, theta)) {
            case 1:
            case 2:
                extent = (int)Math.round(trans.dy + xyP.y1 + xyP.y2) - offset;
                break;
            case 3:
            case 4:
                extent = (int)Math.round(trans.dy + xyP.y2) - offset;
                break;
            case 5:
            case 6:
                extent = Math.round(trans.dy) - offset;
                break;
            case 7:
            case 8:
                extent = (int)Math.round(trans.dy + xyP.y1) - offset;
                break;
            default:
                throw new RuntimeException(
                    "Invalid annotation rotation case for horizontal axis");
            }
        }
        else {
            // Horizontal Axis:
            int offset = parent.toPixel(vl.getValue()) - parent.pixelStart();
            if (offset < 0) {
                continue;
            }
            switch(getAnnoRotationCase(false, theta)) {
            case 1:
            case 2:
                extent = (int)Math.round(offset + trans.dx + xyP.x2) -
                                         parent.pixelLength();
                break;
            case 3:
            case 4:
                extent = Math.round(offset + trans.dx) -
                                         parent.pixelLength();
                break;
            case 5:
            case 6:
                extent = (int)Math.round(offset + trans.dx + xyP.x1) -
                                         parent.pixelLength();
                break;
            case 7:
            case 8:
                extent = (int)Math.round(offset + trans.dx + xyP.x1 + xyP.x2) -
                                         parent.pixelLength();
                break;
            default:
                throw new RuntimeException(
                    "Invalid annotation rotation case for vertical axis");
            }
        }
        maxExtent = Math.max(extent, maxExtent);
    }
    return(maxExtent);
}

/**
 * Returns the ortho extent of axis labels rotated at angle theta.
 * @param theta The angle of rotation in radians.
 * @return The ortho extent in pixels.
 */
protected int getRotatedAnnoOrthoExtent(double theta, int labelExtent)
{
    // For ortho extents, we can ignore the
    // annotation rotation case.
    int extent = 0;
    int maxExtent = 0;
    int first = firstVisibleLabel();
    int last = lastVisibleLabel();
    for (int i = first; i <= last; i++) {
        JCValueLabel vl = (JCValueLabel)valueLabels.elementAt(i);
        ChartText cl = vl.getChartText();
        cl.recalc();
        double value = vl.getValue();
        if (!cl.isVisible()         ||
            value < parent.getMin() ||
            value > parent.getMax())
        {
            continue;
        }

        XYProjection xyP =
            getXYProjection(cl.width.value, cl.height.value, theta);
        if (parent.i_vertical) {
            extent = (int)Math.round(labelExtent + 1
                                        + xyP.x1 + xyP.x2);
        }
        else {
            extent = (int)Math.round(labelExtent + 1
                                        + xyP.y1 + xyP.y2);
//            System.out.println("labelExtent=" + labelExtent +
//                ", xyP.y1=" + xyP.y1 + ", xyP.y2=" + xyP.y2);
        }
        maxExtent = Math.max(extent, maxExtent);
    }
    return(maxExtent);
}

/**
 * This method is invoked when the annotation rotation is JCAxis.ROTATE_OTHER
 * and user-provided annotations are used.  It goes through the list of value
 * labels and computes the largest orthogonal extent as well as the largest
 * projection of a label onto the axis.  The labels' ChartText width and
 * height are used in these calculations.  We don't use FontMetrics in case
 * the labels display HTML text.
 */
protected Dimension getMaxRotatedAnnoDimensions()
{
    int width = 0;
    int height = 0;
    int maxWidth = 0;
    int maxHeight = 0;

    double theta = parent.getAnnotationRotationAngle(JCChartUtil.RADIANS);
    // If the annotation rotation angle is negative,
    // convert it to a positive value.
    if (theta < 0) {
        theta += JCChartUtil.M_2PI;
    }
    int first = firstVisibleLabel();
    int last = lastVisibleLabel();
    for (int i = first; i <= last; i++) {
        JCValueLabel vl = (JCValueLabel)valueLabels.elementAt(i);
        ChartText cl = vl.getChartText();
        cl.recalc();
        double value = vl.getValue();
        if (!cl.isVisible()         ||
            value < parent.getMin() ||
            value > parent.getMax())
        {
            continue;
        }

        XYProjection xyP = getXYProjection(cl.width.value, cl.height.value,
                                           theta);

        width = (parent.i_vertical) ? (int)Math.round(xyP.x1 + xyP.x2)
                                  : calcRotatedAnnoSpacing(cl.width.value,
                                                           cl.height.value,
                                                           theta);
        height = (parent.i_vertical) ? calcRotatedAnnoSpacing(cl.width.value,
                                                            cl.height.value,
                                                            theta)
                                   : (int)Math.round(xyP.y1 + xyP.y2);
        maxWidth = Math.max(width, maxWidth);
        maxHeight = Math.max(height, maxHeight);
    }

    return(new Dimension(maxWidth, maxHeight));
}

private int calcVertRotatedAnnoSpacing(XYProjection xyP, double angle) {
    int spacing = 0;
    try {
        spacing = (int)Math.round(xyP.y1 + xyP.x1/Math.tan(angle));
    }
    catch(ArithmeticException e) {
        // Assume theta is very close to 0 degrees.
        // Just use label height.
        spacing = (int)Math.round(xyP.y1);
    }
    return spacing;
}

private int calcHorizRotatedAnnoSpacing(XYProjection xyP, double angle) {
    int spacing = 0;
    try {
        spacing = (int)Math.round(xyP.x1 + xyP.y1/Math.tan(angle));
    }
    catch(ArithmeticException e) {
        // Assume theta is very close to 90 degrees.
        spacing = (int)Math.round(xyP.x1);
    }
    return spacing;
}

/**
 * Returns the minimum pixel spacing between axis tick marks for labels
 * rotated by angle theta.
 * @param width The pixel width of a label.
 * @param height The pixel height of a label.
 * @param theta The angle in radians by which the axis label is rotated.
 * @return The minimum pixel spacing between tick marks.
 */
protected int calcRotatedAnnoSpacing(int width, int height, double theta)
{
    int spacing = 0;

    // If the annotation rotation angle is negative,
    // convert it to a positive value.
    if (theta < 0) {
        theta += JCChartUtil.M_2PI;
    }
    XYProjection xyP = getXYProjection(width, height, theta);
    int maxSpacing = (parent.i_vertical) ? (int)Math.round(xyP.y1 + xyP.y2)
                                       : (int)Math.round(xyP.x1 + xyP.x2);
    if (parent.i_vertical) {
        switch(getAnnoRotationCase(true, theta)) {
        case 1:
            // use y1 + x1/tan(Math.PI/2 - theta)
            spacing = calcVertRotatedAnnoSpacing(xyP,
                                            JCChartUtil.M_PI_2 - theta);
            break;
        case 2:
        case 3:
            // use y1 + y2
            spacing = maxSpacing;
            break;
        case 4:
            // use y1 + x1/tan(theta - Math.PI/2)
            spacing = calcVertRotatedAnnoSpacing(xyP,
                                            theta - JCChartUtil.M_PI_2);
            break;
        case 5:
            // use y1 + x1/tan(3 * Math.PI/2 - theta)
            spacing = calcVertRotatedAnnoSpacing(xyP,
                                        3 * JCChartUtil.M_PI_2 - theta);
            break;
        case 6:
        case 7:
            // use y1 + y2
            spacing = maxSpacing;
            break;
        case 8:
            // use y1 + x1/tan(theta - 1.5*Math.PI)
            spacing = calcVertRotatedAnnoSpacing(xyP,
                                        theta - 3 * JCChartUtil.M_PI_2);
            break;
        default:
            throw new RuntimeException(
                "Invalid rotational case for vertical axis");
        }
    }
    else {
        // Horizontal axis:
        switch(getAnnoRotationCase(false, theta)) {
        case 1:
            // use x1 + x2
            spacing = maxSpacing;
            break;
        case 2:
            // use x1 + y1/tan(theta)
            spacing = calcHorizRotatedAnnoSpacing(xyP, theta);
            break;
        case 3:
            // use x1 + y1/tan(Math.PI - theta)
            spacing = calcHorizRotatedAnnoSpacing(xyP, Math.PI - theta);
            break;
        case 4:
        case 5:
            // use x1 + x2
            spacing = maxSpacing;
            break;
        case 6:
            // use x1 + y1/tan(theta - Math.PI)
            spacing = calcHorizRotatedAnnoSpacing(xyP, theta - Math.PI);
            break;
        case 7:
            // use x1 + y1/tan(2*Math.PI - theta)
            spacing = calcHorizRotatedAnnoSpacing(xyP,
                                        JCChartUtil.M_2PI - theta);
            break;
        case 8:
            // use x1 + x2
            spacing = maxSpacing;
            break;
        default:
            throw new RuntimeException(
                "Invalid rotational case for horizontal axis");
        }
    }

    if (spacing > maxSpacing) {
        spacing = maxSpacing;
    }
    return spacing;
}

/**
 * Determines the proper location for a chart label.
 */
private void layout(ChartText cl, int offset, int labelExtent)
{
    cl.recalc();

    if (parent.getAnnotationRotation() == JCAxis.ROTATE_OTHER) {
        double theta = parent.getAnnotationRotationAngle(JCChartUtil.RADIANS);
        XYProjection xyP = getXYProjection(cl, theta);
        Translation trans = getTranslation(xyP, theta, labelExtent);
        if (parent.i_vertical) {
            cl.setLocation(parent.linePosition() + trans.dx, offset + trans.dy);
        }
        else {
            cl.setLocation(offset + trans.dx, parent.linePosition() + trans.dy);
        }
        return;
    }

    if (parent.i_vertical) {
        switch (anchor) {
        case JCLegend.WEST:
            cl.setLocation(parent.getLeft() + parent.getWidth() -
                           cl.getWidth() - labelExtent - 1,
                           offset - cl.getHeight()/2);
            break;
        case JCLegend.EAST:
            cl.setLocation(parent.getLeft()+labelExtent + 1,
                           offset - cl.getHeight()/2);
        default:
            break;
        }
    }
    else {
        switch(anchor) {
        case JCLegend.NORTH:
            cl.setLocation(offset - cl.getWidth()/2,
                           parent.getTop() + parent.getHeight() -
                           cl.getHeight() - labelExtent - 1);
            break;
        case JCLegend.SOUTH:
        default:
            cl.setLocation(offset - cl.getWidth()/2,
                           parent.getTop() + labelExtent + 1);
            break;
        }
    }
}

/**
 * Draws each JCValueLabel on the screen.
 */
protected void draw(Graphics gc)
{
    if (!annotationVisible) {
        return;
    }

    int first = firstVisibleLabel();
    int last  = lastVisibleLabel();

    if (first < 0 || first > valueLabels.size() ||
        last < 0 || last >= valueLabels.size())
    {
        return;
    }

    for (int i = first; i <= last; i++) {
        try {
            JCValueLabel vl = (JCValueLabel)valueLabels.elementAt(i);
            if (vl != null && vl.drawLabel) {
                Mapper mapper = parent.getMapper();
                if (mapper != null) {
                    mapper.setDataIndex(vl.getDataIndex());
                }
                vl.label.draw(gc);
            }
        }
        catch (Exception e) {
            e.printStackTrace(System.out);
        }
    }
}

protected void makeValueAnnotations(ChartDataView dv)
{
    valueLabels = new Vector();
    parent.makeTickLabels();
}

/**
 * Creates the axis annotations for display. The axis annotations
 * are stored internally as an array of JCValueLabel objects. Each
 * JCValueLabel represents a label at a particular value along the
 * axis.
 */
protected void makeAnnotations()
{
    if (!annotationVisible) {
        return;
    }
    // Find first data view that references this axis
    ChartDataView dv =
            ((JCChart)parent.chartArea.getParent()).findData(parent);

    if (parent.annotationMethod == JCAxis.VALUE_LABELS) {
        sortValueLabels();
        makeValueLabels(dv);
    }
    else if (parent.annotationMethod == JCAxis.TIME_LABELS) {
        makeTimeLabels();
    }
    else if (parent.annotationMethod == JCAxis.POINT_LABELS) {
        makePointLabels(dv);
    }
    else if (parent.annotationMethod == JCAxis.VALUE) {
        makeValueAnnotations(dv);
    }
}

/**
 * Returns the index of the first visible value label in the
 * value label array.
 * @return index of first visible label
 */
protected int firstVisibleLabel()
{
    int i = 0;
    for (; i < valueLabels.size(); i++) {
        JCValueLabel vl = (JCValueLabel)valueLabels.elementAt(i);
        ChartText ct = vl.getChartText();
        if (ct.visible && ct.text != null) {
            break;
        }
    }
    return(i);
}

/**
 * Returns the index of the last visible value label in the
 * value label array
 * @return index of last visible label
 */
protected int lastVisibleLabel()
{
    int i = valueLabels.size() - 1;
    for (; i >= 0; i--) {
        JCValueLabel vl = (JCValueLabel)valueLabels.elementAt(i);
        ChartText ct = vl.getChartText();
        if (ct.visible && ct.text != null) {
            break;
        }
    }
    return(i);
}

/**
 * Returns the index of the visible value label with minimum value,
 * or Integer.MAX_VALUE if there are no visible value labels.
 */
protected int getMinVisibleLabel()
{
    int indexMinValue = Integer.MAX_VALUE;
    double minValue = Double.MAX_VALUE;
    for (int i=0; i < valueLabels.size(); i++) {
        JCValueLabel vl = (JCValueLabel)valueLabels.elementAt(i);
        ChartText ct = vl.getChartText();
        if (ct.visible &&
            ct.text != null &&
            !ct.text.equals("") &&
            vl.value < minValue)
        {
            minValue = vl.value;
            indexMinValue = i;
        }
    }
    return(indexMinValue);
}

/**
 * Returns the index of the visible value label with maximum value,
 * or Integer.MAX_VALUE if there are no visible value labels.
 */
protected int getMaxVisibleLabel()
{
    int indexMaxValue = Integer.MAX_VALUE;
    double maxValue = Double.MIN_VALUE;
    for (int i=0; i < valueLabels.size(); i++) {
        JCValueLabel vl = (JCValueLabel)valueLabels.elementAt(i);
        ChartText ct = vl.getChartText();
        if (ct.visible &&
            ct.text != null &&
            !ct.text.equals("") &&
            vl.value > maxValue)
        {
            maxValue = vl.value;
            indexMaxValue = i;
        }
    }
    return(indexMaxValue);
}

/**
 * Sorts user-provided value labels so that they are in increasing
 * order.
 */
void sortValueLabels()
{
    Vector oldLabels = userValueLabels;
    userValueLabels = new Vector();
    try {
        for (int i = 0; i < oldLabels.size(); i++) {
            JCValueLabel vl = (JCValueLabel)oldLabels.elementAt(i);
            int j = 0;
            for (; j < userValueLabels.size(); j++) {
                JCValueLabel nvl = (JCValueLabel)userValueLabels.elementAt(j);
                if (vl.value < nvl.value) break;
            }
            userValueLabels.insertElementAt(vl, j);
        }
    }
    catch (Exception e) {
        e.printStackTrace(System.out);
    }
}

/**
 * make the chart value labels using the user value label list
 */
private void makeValueLabels(ChartDataView dv)
{
    int firstPoint = 0;
    int lastPoint = userValueLabels.size() - 1;

    if (dv != null && parent.isRadar() && !parent.i_vertical) {
        int[] limits = ChartDataView.getFirstLastPoints(dv);
        if (limits != null) {
            firstPoint = limits[0];
            lastPoint = limits[1];
        }
        else {
            //  Don't display any labels on the theta axis if the
            //  number of value labels doesn't match the number of
            //  data points.
            return;
        }
    }

    valueLabels = new Vector();
    if (parent.useAnnoTicks) {
        // If the useAnnoTicks property of JCAxis is set, draw ticks
        // specified by the List of JCAnno objects (don't automatically
        // draw ticks at the labels).
        parent.makeTickLabels();
    }

    try {
        for (int i = firstPoint; i <= lastPoint; i++) {
            // PR 37301 -- ensure that the index is within range.
            if (i < 0 || i >= userValueLabels.size()) {
                continue;
            }
            JCValueLabel vl = (JCValueLabel)userValueLabels.elementAt(i);
            JCValueLabel nvl = new JCValueLabel(vl.value,
                                                vl.label.text,
                                                parent,
                                                vl.drawTick,
                                                vl.drawLabel,
                                                vl.drawGrid);
            nvl.label.setRotation(parent.annotationRotation);
            //Careful here, have to use actual background, foreground, and
            //font variables rather than the get methods since the get methods
            //calculate values if the variable value is null.
            nvl.label.setBackground(vl.label.background);
            nvl.label.setForeground(vl.label.foreground);
            nvl.label.setFont(vl.label.font);
            // If the useAnnoTicks property of JCAxis is set, draw ticks
            // specified by the List of JCAnno objects (don't automatically
            // draw ticks at the labels).
            if (!parent.useAnnoTicks) {
                nvl.anno = parent.getDefaultTicks();
            }
            nvl.setParent(parent);
            valueLabels.insertElementAt(nvl, i - firstPoint);
        }
    }
    catch (Exception e) {
        e.printStackTrace(System.out);
    }
}

/**
 * This method is overridden in the CircularChartAnnoHandler subclass.
 */
protected void makePointLabels(ChartDataView dv)
{
    // Clear the annotations on the axis if the dataview is null.
    valueLabels = new Vector();
    if (dv != null) {
        // Use first data series to generate value labels.  If
        // the representation is array, we're fine.  Otherwise,
        // we have to make some kind of assumption.
        ChartDataViewSeries series = null;
        try {
            series = dv.getSeries(0);
        }
        catch (IllegalArgumentException e) {}

        if (series != null) {
            int firstPoint = 0;
            int lastPoint = Math.min(dv.getPointLabels().size(),
                                     series.maxXIndex()) - 1;

            if (parent.useAnnoTicks) {
                // If the useAnnoTicks property of JCAxis is set, draw ticks
                // specified by the List of JCAnno objects (don't automatically
                // draw ticks at the labels).
                parent.makeTickLabels();
            }
            for (int i = firstPoint; i <= lastPoint; i++) {
                String label = (String)dv.getPointLabels().get(i);
                if (label == null || label.equals("")) {
                    continue;
                }
                JCValueLabel vl = new JCValueLabel(series.getX(i),
                                                   label, parent);

                // Set a data index to point back to the dataView and point.
                // Used for image maps.
                vl.setDataIndex(new JCDataIndex(dv, null, -1, i));

                // If the useAnnoTicks property of JCAxis is set, draw ticks
                // specified by the List of JCAnno objects (don't automatically
                // draw ticks at the labels).
                if (!parent.useAnnoTicks) {
                    vl.anno = parent.getDefaultTicks();
                }
                vl.label.setRotation(parent.annotationRotation);
                vl.setParent(parent);
                valueLabels.addElement(vl);
            }
        }
    }
}

/**
 * This method creates time labels for a given parent axis.  This is somewhat
 * of an involved process. There are two spaces involved here, the time space
 * and the data space.  The user defines the axis in the time space by defining
 * a time base along with a time unit.  The user then defines the data based on
 * these values (see the manual for a more detailed explanation of this).  Thus,
 * one could think of the axis as being defined in the time unit (seconds, days,
 * months, etc) or in the data space using real values like 1.0, 2.0, 5.2, etc.
 * Unfortunately, the mapping between time space and data space is not always
 * exact.  For instance, if the time unit is months, every 1 unit in data space
 * maps to a month.  However, all months are not the same size.  But they need
 * to be represented as equally spaced along the axis.  This happens for any
 * unit that is days or longer because of day light savings time and leap years.
 * Thus the methods that convert between time and data must use interpolation
 * as they are converting a different span of milliseconds to one data unit
 * depending on where they are in time.
 * @see #dataToTime
 * @see #timeToData
 */
private void makeTimeLabels()
{
    valueLabels = new Vector();
    if (parent.useAnnoTicks) {
        // If the useAnnoTicks property of JCAxis is set, draw ticks
        // specified by the List of JCAnno objects (don't automatically
        // draw ticks at the labels).
        parent.makeTickLabels();
    }

    // Always make sure there is a TimeBase.
    if (parent.timeBase == null) {
        parent.timeBase = new Date();
    }

    // Possibly calculate a time format, based on the range of the data. Note
    // that unit here is the unit for the data not the unit for the annotation.
    // That gets calculated later.
    long unit = parent.timeUnit;
    double range = (parent.max.value - parent.min.value) * unit;
    if (parent.getTimeFormatIsDefault() ||
        (parent.getTimeFormat() == null ||
         parent.getTimeFormat().equals("")))
    {
        parent.setTimeFormat(getDateFormatter().getDefaultTimeFormat(range),
                             false);
        parent.setTimeFormatIsDefault(true, false);
    }

    // Calculate the size of the largest time label.  This is needed to
    // calculate how many labels fit on the axis.  First we need to get the
    // largest label.
    String tf = parent.getTimeFormat();
    String tl = getDateFormatter().getLargestTimeLabel(tf);

    // Next we need to calculate its size.  Don't localize the number as the
    // date formatter has already handled localization.
    boolean initialNumberLocalization = parent.getNumberLocalization();
    parent.setNumberLocalization(false);
    JCValueLabel test = new JCValueLabel(0.0, tl, parent);
    if (initialNumberLocalization) {
        parent.setNumberLocalization(true);
    }
    int rot = parent.annotationRotation;
    test.label.setRotation(rot);
    test.setParent(parent);
    Dimension testD = test.label.calcSize();

    // Calculate the width and height extensions based on the rotation
    Dimension ext;
    if (rot == JCAxis.ROTATE_NONE || rot == JCAxis.ROTATE_180) {
        ext = new Dimension(testD.width, testD.height);
    }
    else {
        ext = new Dimension(testD.height, testD.width);
    }

    // Calculate a nice tik spacing and number of labels
    int gap = parent.gap * 2;
    int nlabels = parent.i_vertical ? parent.getHeight() / (ext.height + gap)
                                    : parent.getWidth()  / (ext.width  + gap);
    nlabels = Math.max(nlabels, 2);
    long tik = (long)(range / nlabels);
    tik = getDateFormatter().niceInc(tik / 2, tf);
    nlabels = (int)(range / tik) + 2;

    // Make sure time base is in valid range
    Date timeBase = parent.timeBase;
    long minT = JCChartDateFormat.getMinTime().getTime();
    long maxT = JCChartDateFormat.getMaxTime().getTime();
    timeBase.setTime(JCChartUtil.clamp(timeBase.getTime(), minT, maxT));

    // Round the time base down to the nearest unit defined by tik.  Then
    // find the time difference.  This is the offset needed to "normalize" a
    // time label so that its value is at the bottom of the unit interval to
    // which it belongs.  We want to have the labels be the same "date" in each
    // interval.
    Date xtime = new Date(timeBase.getTime());
    getDateFormatter().roundTime(xtime, tik, false);
    long timeOffset = timeBase.getTime() - xtime.getTime();

    // Find the year, month, day, and daylight savings offset
    GregorianCalendar gregCal = new GregorianCalendar();
    gregCal.setTime(timeBase);
    int baseYear  = gregCal.get(Calendar.YEAR);
    int baseMonth = gregCal.get(Calendar.MONTH);
    int baseDay   = gregCal.get(Calendar.DAY_OF_MONTH);
    int baseDST   = gregCal.get(Calendar.DST_OFFSET);

    // Find the month and daylight savings offset of the rounded time
    gregCal.setTime(xtime);
    int roundMonth = gregCal.get(Calendar.MONTH);
    int roundDST   = gregCal.get(Calendar.DST_OFFSET);

    // See if we need to adjust for any leap year
    if (gregCal.isLeapYear(baseYear)) {
        // If base time is on one side of the leap year divide and
        // if the rounded time is on the other, we must compensate.
        // We want the offset to force the labels back to the same
        // date in every year.
        if (baseMonth >= 2 && roundMonth <= 1) {
            timeOffset -= JCAxis.DAYS;
        }
    }

    // If the changeover to daylight savings time happens in the days
    // between the base time and the rounded time, adjust the offset as if no
    // change has happened (we will adjust individual affected dates later on).
    if (roundDST < baseDST) {
        // Missing an hour, add it back in
        timeOffset += JCAxis.HOURS;
    }
    else if (roundDST > baseDST) {
        // Have extra hour, take it out
        timeOffset -= JCAxis.HOURS;
    }

    if (timeOffset < 0) {
        timeOffset += JCAxis.HOURS;
    }

    // Calculate the unit for the annotation.  From this, calculate the
    // annotation increment along the axis (in data units).  Note that
    // annotation units must be greater than or equal to the data units.  One
    // cannot define the data in months and hope to display hours.
    long annoUnit = getDateFormatter().timeIncUnits(tik);
    double xinc = tik /(double)annoUnit;

    // Calculate the begin point of the axis annotation (in data units).  First
    // get the time for the data value using data units.  Convert this time
    // value back to a data value using the annotation units.  Finally round the
    // starting data value down to a multiple of the annotation increment.  If
    // we are below the axis min value, this doesn't matter as out of bounds
    // labels will be dropped later.
    xtime = dataToTime(parent.min.value, unit);
    double xanno = timeToData(xtime, annoUnit);
    long tval = (long)(xanno / xinc);
    xanno = xinc *  tval;

    // Calculate the time for the axis min and max so we know which labels
    // are outside the axis range.
    long axisMinT = minT;
    long axisMaxT = maxT;
    Date temp = null;
    try {
        temp = dataToTime(parent.min.value, unit);
        if (temp != null) {
            getDateFormatter().roundTime(temp, tik, false);
            axisMinT = temp.getTime();
        }
    }
    catch (Exception e) {}
    try {
        temp = dataToTime(parent.max.value, unit);
        if (temp != null) {
            getDateFormatter().roundTime(temp, tik, true);
            axisMaxT = temp.getTime();
        }
    }
    catch (Exception e) {}
    boolean outOfRange = false;

    // Generate value labels
    Date tempTime = new Date();
    for (int i = 0; i < nlabels; i++, xanno += xinc) {
        String label;
        try {
            // Calculate the time corresponding to the anntation value
            xtime = dataToTime(xanno, annoUnit);
            if (xtime == null) {
                continue;
            }

            // Subtract the time offset to get the value to the beginning of
            // the data time interval.
            long tempX = xtime.getTime() - timeOffset;

            // Now we need to adjust the offset in case things like day light
            // savings time or the leap year day happen in the offset interval.
            gregCal.setTime(xtime);
            int beginDay   = gregCal.get(Calendar.DAY_OF_MONTH);
            int beginYear  = gregCal.get(Calendar.YEAR);
            int beginMonth = gregCal.get(Calendar.MONTH);
            int beginDST   = gregCal.get(Calendar.DST_OFFSET);

            // If we're trying to adjust back beyond the beginning of the
            // month because the offset is too big for certain months
            // (like, say, February, which only has 28 days), then
            // readjust back up to the beginning of the month
            if ((annoUnit == JCAxis.MONTHS) && (baseDay != beginDay)) {
                if (baseDay > beginDay) {
                    tempX += ((baseDay - beginDay) * JCAxis.DAYS);
                }
            }

            // Adjust by an hour if daylight savings time has changed
            // between the initial label time and the new offset-derived time
            tempTime.setTime(tempX);
            gregCal.setTime(tempTime);
            int tempDST = gregCal.get(Calendar.DST_OFFSET);
            if (tempDST < beginDST) {
                tempX += JCAxis.HOURS;
            }
            else if (tempDST > beginDST) {
                tempX -= JCAxis.HOURS;
            }

            // If initial label date is on one side of the leap year divide
            // and our new offset-derived label date is on the other, adjust
            // for the one day difference
            if (gregCal.isLeapYear(beginYear)) {
                tempTime.setTime(tempX);
                gregCal.setTime(tempTime);
                int tempMonth = gregCal.get(Calendar.MONTH);
                if (beginMonth >= 2 && tempMonth <= 1) {
                    tempX -= JCAxis.DAYS;
                }
            }

            // Check label boundaries.  Throw out label if outside time or axis
            // bounds.
            if (minT > tempX || maxT < tempX) {
                outOfRange = true;
                nlabels--;
                i--;
                continue;
            }
            if (axisMinT > tempX || axisMaxT < tempX) {
                nlabels--;
                i--;
                continue;
            }

            // Finally, everything is set.  Create the label string.
            xtime.setTime(tempX);
            label = getDateFormatter().timeLabel(tf, xtime);
        }
        catch (Exception e) {
            e.printStackTrace(System.out);
            label = new String(JCChartBundle.string(JCChartBundle.key31,
                                                    parent.locale));
        }

        // Get the data value for the label.  Turn off localization when
        // creating the value label as it has been done by the date formatter.
        double value = timeToData(xtime, unit);
        parent.setNumberLocalization(false);
        JCValueLabel vl = new JCValueLabel(value, label, parent);
        if (initialNumberLocalization) {
            parent.setNumberLocalization(true);
        }
        vl.label.setRotation(rot);

        // If the useAnnoTicks property of JCAxis is set, draw ticks
        // specified by the List of JCAnno objects (don't automatically
        // draw ticks at the labels).
        if (!parent.useAnnoTicks) {
            vl.anno = parent.getDefaultTicks();
        }
        vl.setParent(parent);
        valueLabels.addElement(vl);
    }
    if (outOfRange) {
        ErrorDialog.raise(parent, JCChartBundle.string(JCChartBundle.key118,
                                                       parent.locale));
    }
}

private void timeLabelExtents(Dimension ext)
{
    JCValueLabel biggestLabel = null;
    boolean rotated = (parent.annotationRotation == JCAxis.ROTATE_90 ||
                       parent.annotationRotation == JCAxis.ROTATE_270);
    int biggest = 0;
    ext.width = ext.height = 0;

    for (int i = 0; i < valueLabels.size(); i++) {
        JCValueLabel vl = (JCValueLabel)valueLabels.elementAt(i);
        String labelText = vl.getChartText().getText();
        if (labelText != null) {
            int l = labelText.length();
            if (l > biggest) {
                biggest = l;
                biggestLabel = vl;
            }
        }
    }

    if (biggestLabel == null) {
        return;
    }
    Dimension t = biggestLabel.getChartText().calcSize();
    if (rotated) {
        ext.width = t.height;
        ext.height = t.width;
    }
    else {
        ext.width = t.width;
        ext.height = t.height;
    }
}

/**
 * Set the parent for the value labels list
 */
protected void setValueLabelParent(JCChart p)
{
    if (valueLabels != null) {
        for (int i = 0; i < valueLabels.size(); i++) {
            JCValueLabel vl = (JCValueLabel)valueLabels.elementAt(i);
            vl.getChartText().setParentChart(p);
        }
    }
    if (userValueLabels != null) {
        for (int i = 0; i < userValueLabels.size(); i++) {
            JCValueLabel vl = (JCValueLabel)userValueLabels.elementAt(i);
            vl.getChartText().setParentChart(p);
        }
    }
}

protected Vector getValueLabels()
{
    return(valueLabels);
}

}
