/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ChartDataEvent.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:44 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.util.EventObject;

/**
 * Class used to encapsulate a chart data change event.
 */
public class ChartDataEvent extends EventObject {

/** Type of data change. One of the enum values specified in this class. */
protected int  type;

/** Data series impacted by change (if applicable). */
protected int  seriesIndex;

/** Point index impacted by change (if applicable). */
protected int  pointIndex;

/** 
 * Enum value indicating a single data value has changed. 
 * In this case, <code>seriesIndex</code> and <code>pointIndex</code> are used to
 * index the point.  An event of this type should be used
 * when a value has been changed in a series value array that
 * has previously been queried by the chart from the data source.
 * If the array needs to be regenerated, <code>RELOAD_SERIES</code> should
 * be used instead.
 */
public final static int  RELOAD_POINT = 0;

/** 
 * Enum value indicating a data series has changed.
 * In this case, <code>seriesIndex</code> points to the series to reload.
 * An event of this type will requery the series value array
 * from the data source.
 */
public final static int  RELOAD_SERIES = 1;

/** 
 * Enum value indicating a data series has been added to the
 * end of the set of data series.  
 */
public final static int  ADD_SERIES=2;

/** 
 * Enum value indicating a data series has ben inserted at
 * a particular index in the set of data series.  In this
 * case, <code>seriesIndex</code> holds the index at which to insert the
 * series.
 */
public final static int INSERT_SERIES = 3;

/**
 * Enum value indicating a data series has been removed.
 * In this case, <code>seriesIndex</code> holds the index of the series
 * to be removed.
 */
public final static int  REMOVE_SERIES = 5;

/**
 * Enum value indicating all point labels need to be 
 * reloaded.
 */
public final static int RELOAD_ALL_POINT_LABELS = 6;

/**
 * Enum value indicating a particular point label needs
 * to be reloaded.  <code>pointIndex</code> indicates which label
 * should be reloaded
 */
public final static int RELOAD_POINT_LABEL = 7;

/**
 * Enum value indicating that all series labels need to
 * be reloaded.
 */
public final static int RELOAD_ALL_SERIES_LABELS = 8;

/**
 * Enum value indicating a particular series label needs
 * to be reloaded.  <code>seriesIndex</code> indicates which label
 * should be reloaded.
 */
public final static int RELOAD_SERIES_LABEL = 9;

/**
 * Enum value indicating the data source name that needs to
 * be reloaded.
 */
public final static int RELOAD_DATA_SOURCE_NAME = 10;

/**
 * Enum value indicating that data has been appended to the end of the
 * specified data series.  This event refreshes the data and does a fast
 * update.
 * <br>
 * <b>Note:</b> A fast update can be done only for charts of type PLOT,
 *       SCATTER_PLOT and AREA.  Also, the fastUpdate property of the
 *       corresponding ChartDataView must be set to true.
 */
 public final static int APPEND_DATA = 11;

/**
 * Enum indicating that the data source has been completely reset.
 */
public final static int RESET = 12;

/**
 * Enum indicating that the data should be reloaded.
 */
public final static int RELOAD = 13;

/**
 * Enum indicating that the chart type has changed.
 * The chart type is specified by <code>seriesIndex</code>.
 */
public final static int CHANGE_CHART_TYPE = 14;

/**
 * Enum value indicating that the hole value needs to be reloaded.
 */
public final static int RELOAD_HOLE_VALUE = 15;

/**
 * Enum value indicating that the image map information needs to be reloaded.
 */
public final static int RELOAD_IMAGE_MAP_INFO = 16;

/**
 * Enum value used for <code>seriesIndex</code> to indicate all series
 * are affected.
 */
public final static int ALL_SERIES = -1;

/**
 * Enum value used for <code>pointIndex</code> to indicate all points
 * are affected.
 */
public final static int ALL_POINTS = -1;

/**
 * Constructor used to create a <code>ChartDataEvent</code> object, with the =
 * chart data changes and the data source.
 * @param dataSrc the data source
 * @param type data change type to pass to the views
 * @param seriesIndex index of series affected by the change
 * @param pointIndex index of point affected by the change
 */
public ChartDataEvent(Object dataSrc, int type, int seriesIndex, int pointIndex)
{
	super(dataSrc);
	this.type = type;
	this.seriesIndex = seriesIndex;
	this.pointIndex = pointIndex;
}

/**
 * Gets the <code>Type</code> property.  It contains the type of change that has
 * happened to the chart data.
 * @return the <code>Type</code> property, which is one of the enums above
 */
public int getType()
{
	return(type);
}

/**
 * Gets the <code>SeriesIndex</code> property.
 * @return <code>SeriesIndex</code> property
 */
public int getSeriesIndex()
{
	return(seriesIndex);
}

/**
 * Gets the <code>PointIndex</code> property.
 * @return <code>PointIndex</code> property
 */
public int getPointIndex()
{
	return(pointIndex);
}

}
