/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ChartDataModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:44 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

/**
 * The core data model interface for JClass Chart.
 * 
 * <P>In JClass Chart, data is specified in terms of data series.
 * Chart asks for the <i>x</i> or <i>y</i> data for a
 * particular series, and the data source implementing this
 * class is expected to return an array of double values
 * (<code>double[]</code>).
 * 
 * <p>JClass Chart will attempt to recognize cases in which the
 * same <i>x</i> data is returned for all series and then will
 * re-use that data.
 * 
 * <P>It is recommended that implementors honor the <code>NumSeries</code>
 * property. Thus, be sure you can handle <i>x</i> or <i>y</i> data
 * queries in the range 0 to <code>NumSeries</code>.
 * 
 * <P>Point labels, series labels, and data source names are
 * specified by implementing <code>LabelledChartDataModel</code>.
 * 
 * @see LabelledChartDataModel
 */
public interface ChartDataModel {

/** In certain contexts, used to specify all series or all points. */
public final static int ALL = -20;

/**
 * Retrieves the <i>x</i> values of the specified data series.
 * @param index data series index
 * @return array of double values representing <i>x</i> value data
 */
public double[] getXSeries(int index);

/**
 * Retrieves the <i>y</i> values of the specified data series.
 * @param index data series index
 * @return array of double values representing <i>y</i> value data
 */
public double[] getYSeries(int index);

/**
 * Retrieves the number of data series.
 * @return the number of data series
 */
public int getNumSeries();

}


