/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ChartDataSupport.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:44 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.util.Enumeration;

import com.klg.jclass.util.JCListenerList;

/**
 * Class used to implement the {@link ChartDataManager} interface.
 */
public class ChartDataSupport implements ChartDataManager,
                                         java.io.Serializable
{

private JCListenerList  listeners;
protected Object        source;

/**
 * The default constructor for a <code>ChartDataSupport</code> object.
 */
public ChartDataSupport()
{
	source = this;
}

/**
 * Constructor used to create a <code>ChartDataSupport</code> object with a
 * given data source.
 * @param dataSrc the data source
 */
public ChartDataSupport(Object dataSrc)
{
	source = dataSrc;
}

/**
 * Adds a chart data listener.
 * @param l the chart data listener object to be added
 */
public void addChartDataListener(ChartDataListener l)
{
	listeners = JCListenerList.add(listeners, l);
}

/**
 * Removes a chart data listener.
 * @param l chart data listener object to be removed
 */
public void removeChartDataListener(ChartDataListener l)
{
	listeners = JCListenerList.remove(listeners, l);
}

/**
 * Fires a chart data event to any registered listeners.  This
 * version of <code>fireChartDataEvent()</code> will create a
 * <code>ChartDataEvent</code> object out of the type, series index, and point
 * index.
* @param type data change type to pass to the views
 * @param seriesIndex index of series affected by the change
 * @param pointIndex index of point affected by the change
 */
public void fireChartDataEvent(int type, int seriesIndex, int pointIndex)
{
	fireChartDataEvent(new ChartDataEvent(source, type, seriesIndex,
                                          pointIndex));
}

/**
 * Fires a chart data event to any registered listeners.
 * @param evt the chart data model update
 */
public void fireChartDataEvent(ChartDataEvent evt)
{
    Enumeration e = JCListenerList.elements(listeners);
	while (e.hasMoreElements()) {
	    ((ChartDataListener)e.nextElement()).chartDataChange(evt);
	}
}

}
