/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ChartDrawUtil.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:45 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.util.List;

/**
 * A collection of static methods and static variables 
 * that are used by the drawables. No method in this class
 * should make any explicit reference to a drawable that would make
 * that drawable be downloaded.
 */
public class ChartDrawUtil {

/**
 * Calculates stacking values from series of y values, and stores it in the
 * provided arrays.
 * @param ysum array of sums of positive y values, one for each series
 * @param ysum_neg array of sums of negative y values, one for each series
 * @param ytotal array of total absolute value of all y values in each series
 * @param ywork array of stacking y values, one for each point in each series
 *        (these are the values that are actually charted)
 * @param firstPoint the first point to use in each series
 * @param lastPoint the last point to use in each series
 * @param seriesList the list of data series
 * @param is_100_percent <code>true</code> if using the percentage option, where
 *        values are calculated as a percentage of the total value;
 *        <code>false</code> otherwise
 * @param holeValue the hole value
 * @param yminmax the max and min values of the y values
 * @param yaxis the y axis
 */
static void calcStackingInfo(double ysum[], double ysum_neg[],
                             double ytotal[], double ywork[][],
                             int firstPoint, int lastPoint, List seriesList,
                             boolean is_100_percent, double holeValue,
                             MinMax yminmax, JCAxis yaxis)
{
	int nsets = seriesList.size();
    double xp, yp;
	for (int i = firstPoint; i <= lastPoint; i++) {
		if (nsets == 0) {
			break;
        }
		int thisIndex = i - firstPoint;

		ysum[thisIndex] = 0.0;
		ysum_neg[thisIndex] = 0.0;

		// Calculate the total value of each point
		if (is_100_percent) {
			ytotal[thisIndex] = 0.0;
			for (int j = 0; j < nsets; j++) {
				ChartDataViewSeries thisSeries = 
					(ChartDataViewSeries) seriesList.get(j);
				xp = thisSeries.getX(i);
				yp = thisSeries.getY(i);
				if (xp == holeValue || yp == holeValue ||
					(yp <= 0.0 && yaxis.logarithmic))
                {
					continue;
				}
				if (is_100_percent && thisSeries.isIncluded()) {
					ytotal[thisIndex] += Math.abs(yp);
				}
				if (!thisSeries.isVisible()) {
					continue;
                }
			}
			if (ytotal[thisIndex] == 0.0) {
				// Just in case.
				ytotal[thisIndex] = 1.0;
			}
		}

		for (int j = 0; j < seriesList.size(); j++) {
			ChartDataViewSeries thisSeries =
                (ChartDataViewSeries)seriesList.get(j);
			if ((!thisSeries.isIncluded()) || (!thisSeries.isVisible())) {
				ywork[j][thisIndex] = Double.MAX_VALUE;
				continue;
			}

			xp = thisSeries.getX(i);
			yp = thisSeries.getY(i);

			if (xp == holeValue || yp == holeValue ||
				(yp <= 0.0 && yaxis.logarithmic))
            {
				ywork[j][thisIndex] = Double.MAX_VALUE;
				continue;
			}

			boolean y_negative = (yp < 0.0);

			if (y_negative) {
				ysum_neg[thisIndex] += Math.min(0.0, yp);
				ywork[j][thisIndex] =
                    is_100_percent ? (ysum_neg[thisIndex] /
                                      ytotal[thisIndex] * 100.0)
                                   : ysum_neg[thisIndex];
				if (ywork[j][thisIndex] > yminmax.max) {
					continue;
				}
			}
            else {
				ysum[thisIndex] += Math.max(0.0, yp);
				ywork[j][thisIndex] =
                    is_100_percent ? (ysum[thisIndex] /
                                      ytotal[thisIndex] * 100.0)
                                   : ysum[thisIndex];
				if (ywork[j][thisIndex] < yminmax.min) {
					continue;
				}
			}
		}
	}
}


}
