/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ChartInteriorRegion.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:45 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.awt.*;
import java.util.Locale;
import com.klg.jclass.util.JCName;

/**
 * A utility class used inside JClass Chart to represent a drawable region on a
 * chart. It is used for all axis annotations as a lightweight component or
 * gadget.
 */
public class ChartInteriorRegion extends TrackChange
			           implements Drawable, java.io.Serializable, JCName {

protected Insets insets = new Insets(0, 0, 0, 0);

protected DerivedInt left   = new DerivedInt(0, true);
protected DerivedInt top    = new DerivedInt(0, true);
protected DerivedInt width  = new DerivedInt(0, true);
protected DerivedInt height = new DerivedInt(0, true);

/** Font used to draw text in the chart region.  Inherited from parent. */
protected Font font = null;

/** Name of the <code>ChartInteriorRegion</code> object. */
protected String name = null;

/** Foreground color used in the chart region.  Inherited from parent. */
protected Color foreground = null;

/** Background color used in the chart region.  Inherited from parent. */
protected Color background = null;

/** Connection to the chart on which the <code>ChartInteriorRegion</code> appears. */
protected JCChart parentChart = null;

protected ChartRegion parentRegion = null;

/** Determines whether the chart region is visible. Used in <code>JCAxis</code>. */
protected boolean visible = true;

/** Used to turn off updates to parent in certain cases. */
protected boolean updateParent = true;

protected boolean dimIsDefaultChangedToTrue = false;
protected boolean inUpdate = false;
protected boolean needsRepaint = true;
protected boolean transparent = true;

/**
 * Default constructor for chart region.
 */
public ChartInteriorRegion() {
}

/**
 * Constructor that connects the <code>ChartInteriorRegion</code> to a
 * parent <code>JCChart</code>.
 * @param p parent chart
 */
public ChartInteriorRegion(JCChart p)
{
	setParentChart(p);
}

public ChartInteriorRegion(ChartRegion r)
{
	setParentRegion(r);
	if (r != null) {
		setParentChart((JCChart)r.getParent());
	}
}

/**
 * Connects the chart region to a <code>JCChart</code>.
 * @param p parent chart
 */
public void setParentChart(JCChart p)
{
	parentChart = p;
}

public JCChart getParentChart()
{
	return(parentChart);
}

public void setParentRegion(ChartRegion r)
{
	parentRegion = r;
}

public ChartRegion getParentRegion()
{
	return(parentRegion);
}

/**
 * Gets the value of the <code>Font</code> property.  The <code>Font</code>
 * property determines what font is used to render text inside the
 * chart region. <br><B>Note:</b> The <code>Font</code> property is inherited
 * from the parent <code>JCChart</code>.
 * @return font used to render text in the chart region
 */
public Font getFont()
{
	Font parentFont = (parentChart == null) ? null : parentChart.getFont();
	return((font == null) ? parentFont : font);
}

/**
 * Sets the value of the <code>Font</code> property.  The <code>Font</code>
 * property determines what font is used to render text inside the
 * chart region. <br><B>Note:</b> The <code>Font</code> property is inherited
 * from the parent <code>JCChart</code>.
 * @param f font to use to render text in the chart region
 */
public void setFont(Font f)
{
	if (f == font) {
		return;
	}
	font = f;

	// Only force a change if we have a width already
	if (getWidth() > 0) {
		setChanged(true, Changeable.RECALC);
	}
}

/**
 * Gets the value of the <code>Name</code> property.
 * The <code>Name</code> property specifies a String identifier for
 * the <code>ChartInteriorRegion</code> object.
 * @return new name for the <code>object.n</code>
 */
public String getName()
{
	return(name);
}

/**
 * Sets the value of the <code>Name</code> property.
 * The <code>Name</code> property specifies a String identifier for
 * the <code>ChartInteriorRegion</code> object.
 * @param s new name for the <code>object.n</code>
 */
public void setName(String s)
{
	name = s;
}

/**
 * Gets the value of the <code>Foreground</code> property.  The
 * <code>Foreground</code> property determines the foreground color used to
 * draw inside the chart region. <br><B>Note:</b> The <code>Foreground</code>
 * property is inherited from the parent <code>JCChart</code>.
 * @return foreground color used to draw the chart region
 */
public Color getForeground()
{
	Color parentFG = (parentChart == null) ? null : parentChart.getForeground();
	return((foreground == null) ? parentFG : foreground);
}

/**
 * Sets the value of the <code>Foreground</code> property.  The
 * <code>Foreground</code> property determines the foreground color used to
 * draw inside the chart region. <br><B>Note:</b> The <code>Foreground</code>
 * property is inherited from the parent <code>JCChart</code>.
 * @param c foreground color used to draw the chart region
 */
public void setForeground(Color c)
{
	if (c == foreground) {
		return;
	}
	foreground = c;
	setChanged(true, Changeable.RECALC);
}

/**
 * Gets the value of the <code>Background</code> property.  The
 * <code>Background</code> property determines the foreground color used to
 * draw inside the chart region. <br><B>Note:</b> The <code>Background</code>
 * property is inherited from the parent <code>JCChart</code>.
 * @return background color used to draw the chart region
 */
public Color getBackground()
{
	Color parentBG = (parentChart == null) ? null : parentChart.getBackground();
	return((background == null) ? parentBG : background);
}

/**
 * Sets the value of the <code>Background</code> property.  The
 * <code>Background</code> property determines the foreground color used to
 * draw inside the chart region. <br><B>Note:</b> The <code>Background</code>
 * property is inherited from the parent <code>JCChart</code>.
 * @param c background color used to draw the chart region
 */
public void setBackground(Color c)
{
	if (c == background) {
		return;
	}
	background = c;
	setChanged(true, Changeable.RECALC);
}

/**
 * Gets the value of the <code>Visible</code> property.
 * The <code>Visible</code> property determines whether the associated
 * <code>ChartInteriorRegion</code> is currently visible.
 * @return <code>true</code> if the chart region is showing;
 * <code>false</code> otherwise
 */
public boolean isVisible()
{
	return(visible);
}

/**
 * Sets the value of the <code>Visible</code> property.
 * The <code>Visible</code> property determines whether the associated
 * <code>ChartInteriorRegion</code> is currently visible.
 * @param is <code>true</code> if the chart region is showing;
 * <code>false</code> otherwise
 */
public void setVisible(boolean is)
{
	if (visible == is) {
		return;
	}
	visible = is;
	setChanged(true, Changeable.RECALC);
}

/**
 * Recalculates the contents of the chart region.  Not used in the base class.
 */
public void recalc() {}

/**
 * Reshapes (in ohter words, resizes and moves) the chart region according
 * to the provided parameters.
 * @param newx new <TT>x</TT> position relative to the parent chart
 * @param newy new <TT>y</TT> position relative to the parent chart
 * @param neww new width
 * @param newh new height
 */
public void setBounds(int newx, int newy, int neww, int newh)
{
	if (left.isDefault) {
		left.value = newx;
	}
	if (top.isDefault) {
		top.value = newy;
	}
	if (width.isDefault) {
		width.value = neww;
	}
	if (height.isDefault) {
		height.value = newh;
	}
}

/**
 * Moves the chart region to the provided position relative to
 * the chart.
 * @param newx new <TT>x</TT> position relative to the parent chart
 * @param newy new <TT>y</TT> position relative to the parent chart
 */
public void setLocation(int newx, int newy)
{
	setBounds(newx, newy, width.value, height.value);
}

/**
 * Resizes the chart region to the provided width and height.
 * @param neww new width
 * @param newh new height
 */
public void setSize(int neww, int newh)
{
	// Take care of insets
	setBounds(left.value, top.value, neww + insets.left + insets.right,
			  newh + insets.top + insets.bottom);
}

/**
 * Returns the size of the region.
 * @return the size of this <code>ChartInteriorRegion</code>
 */
public Dimension getSize()
{
    return(new Dimension(width.value, height.value));
}

/**
 * Sets the value of the <code>Top</code> property.
 * The <code>Top</code> property determines the location of the
 * top of the <code>ChartInteriorRegion</code>.
 * @param t top value in pixels
 */
public void setTop(int t)
{
	if (top.value == t) {
		return;
	}
	top.value = t;
	top.isDefault = false;
	setChanged(true, Changeable.RECALC | Changeable.NOTIFY_PARENT_LAYOUT);
}

/**
 * Gets the value of the <code>Top</code> property.
 * The <code>Top</code> property determines the location of the
 * top of the <code>ChartInteriorRegion</code>.
 * @return top value in pixels
 */
public int getTop()
{
	return(top.value);
}


/**
 * Sets the value of the <code>TopIsDefault</code> property.
 * The <code>TopIsDefault</code> property determines whether the top
 * position of the chart region is calculated by Chart (<code>true</code>) or
 * is taken from the <code>Top</code> property (<code>false</code>).
 * @param d <code>true</code> if the position is to be calculated;
 * <code>false</code> otherwise
 */
public void setTopIsDefault(boolean d)
{
	if (top.isDefault == d) {
		return;
	}
	top.isDefault = d;
	dimIsDefaultChangedToTrue = d;
	setChanged(true, Changeable.RECALC | Changeable.NOTIFY_PARENT_LAYOUT);
}

/**
 * Gets the value of the <code>TopIsDefault</code> property.
 * The <code>TopIsDefault</code> property determines whether the top
 * position of the chart region is calculated by Chart (<code>true</code>) or
 * is taken from the Top property (<code>false</code>).
 * @return <code>true</code> if the position is to be calculated;
 * <code>false</code> otherwise
 */
public boolean getTopIsDefault()
{
	return(top.isDefault);
}


/**
 * Sets the value of the <code>Left</code> property.
 * The <code>Left</code> property determines the location of the
 * left of the <code>ChartInteriorRegion</code>.
 * @param t left value in pixels
 */
public void setLeft(int t)
{
	if (left.value == t) {
		return;
	}
	left.value = t;
	left.isDefault = false;
	setChanged(true, Changeable.RECALC | Changeable.NOTIFY_PARENT_LAYOUT);
}

/**
 * Gets the value of the <code>Left</code> property.
 * The <code>Left</code> property determines the location of the
 * left of the <code>ChartInteriorRegion</code>.
 * @return left value in pixels
 */
public int getLeft()
{
	return(left.value);
}

/**
 * Sets the value of the <code>LeftIsDefault</code> property.
 * The <code>LeftIsDefault</code> property determines whether the left
 * position of the chart region is calculated by Chart (<code>true</code>) or
 * is taken from the <code>Left</code> property (<code>false</code>).
 * @param d <code>true</code> if the position is to be calculated;
 * <code>false</code> otherwise
 */
public void setLeftIsDefault(boolean d)
{
	if (left.isDefault == d) {
		return;
	}
	left.isDefault = d;
	dimIsDefaultChangedToTrue = d;
	setChanged(true, Changeable.RECALC | Changeable.NOTIFY_PARENT_LAYOUT);
}

/**
 * Gets the value of the <code>LeftIsDefault</code> property.
 * The <code>LeftIsDefault</code> property determines whether the left
 * position of the chart region is calculated by Chart (<code>true</code>) or
 * is taken from the <code>Left</code> property (<code>false</code>).
 * @return <code>true</code> if the position is to be calculated;
 * <code>false</code> otherwise
 */
public boolean getLeftIsDefault()
{
	return(left.isDefault);
}

/**
 * Sets the value of the <code>Width</CODE> property.
 * The <code>Width</code> property determines the width of the
 * <code>ChartInteriorRegion</code>.
 * @param t width value in pixels
 */
public void setWidth(int t)
{
	if (width.value == t) {
		return;
	}
	width.value = t;
	width.isDefault = false;
	setChanged(true, Changeable.RECALC);
}

/**
 * Gets the value of the <code>Width</code> property.
 * The <code>Width<code> property determines the location of the
 * width of the <code>ChartInteriorRegion</code>.
 * @return width value in pixels
 */
public int getWidth()
{
	return(width.value);
}

/**
 * Sets the value of the <code>WidthIsDefault</code> property.
 * The <code>WidthIsDefault</code> property determines whether the width
 * of the chart region is calculated by Chart (<code>true</code>) or
 * taken from the <code>Width</code> property (<code>false</code>).
 * @param d <code>true</code> if the width is to be calculated; <code>false</code>
 * otherwise
 */
public void setWidthIsDefault(boolean d)
{
	if (width.isDefault == d) {
		return;
	}
	width.isDefault = d;
	dimIsDefaultChangedToTrue = d;
	setChanged(true, Changeable.RECALC);
}

/**
 * Gets the value of the <code>WidthIsDefault</code> property.
 * The <code>WidthIsDefault</code> property determines whether the width
 * of the chart region is calculated by Chart (<code>true</code>) or
 * taken from the <code>Width</code> property (<code>false</code>).
 * @return <code>true</code> if the width is to be calculated; <code>false</code>
 * otherwise
 */
public boolean getWidthIsDefault()
{
	return(width.isDefault);
}


/**
 * Sets the value of the <code>Height</code> property.
 * The <code>Height</code> property determines the height of the
 * <code>ChartInteriorRegion</code>.
 * @param t height value in pixels
 */
public void setHeight(int t)
{
	if (height.value == t) {
		return;
	}
	height.value = t;
	height.isDefault = false;
	setChanged(true, Changeable.RECALC);
}

/**
 * Gets the value of the <code>Height</code> property.
 * The <code>Height</code> property determines the location of the
 * height of the <code>ChartInteriorRegion</code>.
 * @return height value in pixels
 */
public int getHeight()
{
	return(height.value);
}


/**
 * Sets the value of the <code>HeightIsDefault</code> property.
 * The <code>HeightIsDefault</code> property determines whether the height
 * of the chart region is calculated by Chart (<code>true</code>) or
 * taken from the <code>Height</code> property (<code>false</code>).
 * @param d <code>true</code> if the height is to be calculated;
 * <code>false</code> otherwise
 */
public void setHeightIsDefault(boolean d)
{
	if (height.isDefault == d) {
		return;
	}
	height.isDefault = d;
	dimIsDefaultChangedToTrue = d;
	setChanged(true, Changeable.RECALC);
}

/**
 * Gets the value of the <code>HeightIsDefault</code> property.
 * The <code>HeightIsDefault</code> property determines whether the height
 * of the chart region is calculated by Chart (<code>true</code>) or
 * taken from the <code>Height</code> property (<code>false</code>).
 * @return <code>true</code> if the height is to be calculated;
 * <code>false</code> otherwise
 */
public boolean getHeightIsDefault()
{
	return(height.isDefault);
}


transient private JCChartNumberFormat numberFormatter = null;
protected boolean numberLocalization = true;
protected boolean groupingUsed = false;

/**
 * Gets the number formatter (used for number localization).
 * @return the current <code>JCChartNumberFormat</code> object used for
 *     localizing numbers
 */
public JCChartNumberFormat getNumberFormatter()
{
	if (numberFormatter == null) {
		numberFormatter = new JCChartNumberFormat(this);
	}
	return(numberFormatter);
}

/**
 * Sets whether to localize numbers.
 * @param numberLocalization the new value of the
 * <code>NumberLocalization</code> property
 */
public void setNumberLocalization(boolean numberLocalization)
{
	if (this.numberLocalization == numberLocalization) {
		return;
	}
	this.numberLocalization = numberLocalization;
}

/**
 * Gets the value of the <code>NumberLocalization</code> property.
 * @return <code>true</code> if number localization is being used
 */
public boolean getNumberLocalization()
{
	return(numberLocalization);
}


/**
 * Gets the value of the <code>GroupingUsed</code> property.
 * The <code>GroupingUsed</code> property determines whether or not grouping
 * will be used in formatting numbers.
 * @return the current grouping used
 */
public boolean getGroupingUsed()
{
    return(groupingUsed);
}

/**
 * Sets the value of the <code>GroupingUsed</code> property.
 * The <code>GroupingUsed</code> property determines whether or not grouping
 * will be used in formatting numbers. Default value is <code>false</code>.
 * @param groupingUsed the new value of the <code>GroupingUsed</code> property
 */
public void setGroupingUsed(boolean groupingUsed)
{
	if (this.groupingUsed == groupingUsed) {
		return;
	}
    this.groupingUsed = groupingUsed;
    setChanged(true, Changeable.RECALC);
}

/**
 * Sets the value of the number formatting locale.
 * Default value is the current locale.
 * @param l the new number formatting locale
 */
public void setNumberFormatterLocale(Locale l)
{
    getNumberFormatter().setNumberFormatterLocale(l);
    setChanged(true, Changeable.RECALC);
}

/**
 * Draws the chart region.
 * @param gc graphics context to use for drawing
 */
public void draw(Graphics gc)
{
	if (!visible) {
		return;
	}
	if (gc == null) {
		return;
	}

	if (parentChart == null) {
		return;
	}
	if (!transparent) {
		if (getBackground() == null) {
			if (parentChart != null && parentChart.getBackground() != null) {
				gc.setColor(parentChart.getBackground());
				gc.fillRect(left.value, top.value, width.value, height.value);
			}
		}
		else {
			gc.setColor(getBackground());
			gc.fillRect(left.value, top.value, width.value, height.value);
		}
	}
}

/**
 * Sets the value of the <code>Changed</code> property and updates the parent
 * object. The <code>Changed</code> property determines whether the object needs
 * to be recalculated.
 * @param c new value for <code>Changed</code>
 * @param cf the value with which to update the changed flag
 */
public void setChanged(boolean c, int cf)
{
	if (c != getChanged() || !getChanged(cf)) {
		if (parentRegion != null) {
			parentRegion.setChanged(c, cf, false);
		}
		super.setChanged(c, cf);
	}
	if (c && parentChart != null && updateParent) {
		updateParent(cf);
	}
}

/**
 * Sets the value of the <code>Changed</code> property. The <code>Changed</code>
 * property determines whether the object needs to be recalculated.
 * @param n new value for <code>Changed</code>
 * @param cf the value with which to update the changed flag
 * @param updateParent if <code>true</code>, the parent is updated; otherwise,
 * the parent is not updated (the "update" semantics are up to the implementor)
 */
public void setChanged(boolean n, int cf, boolean updateParent)
{
	if (parentRegion != null) {
		parentRegion.setChanged(n, cf, false);
	}
	super.setChanged(n, cf, updateParent);
}

/**
 * Updates the parent based on the passed in flag.
 * @param cf the flag which determines the kind of update
 */
public void updateParent(int cf)
{
	JCChart p = getParentChart();
	if (p != null) {
		if (((cf & Changeable.RECALC) > 0) ||
			((cf & Changeable.NOTIFY_PARENT_LAYOUT) > 0))
		{
			p.invalidate();
			p.getChartArea().invalidate();
			if (!p.isBatched()) {
				p.validate();
				// This repaint may be redundant in some cases.
				p.repaint();
			}
		}
		else if ((cf & Changeable.REDRAW) > 0) {
			if (!p.isBatched()) {
				p.repaint();
			}
		}
	}
}

/**
 * Gets the current graphics object used.
 * @return the current graphics object used
 */
public Graphics getGraphics()
{
	if (parentChart == null) {
		return(null);
	}
	return(parentChart.getGraphics());
}

/**
 * Repaints, taking into account the parent region.  Uses the gc of the parent
 * chart.
 */
public void repaint()
{
	if (parentRegion != null) {
		// Pass the repaint up to the top level
		parentRegion.repaint();
		return;
	}
	if (parentChart == null) {
		return;
	}
	Graphics gc = parentChart.getGraphics();
	if (gc == null) {
		// Fail-safe
		parentChart.repaint();
		return;
	}
	Rectangle r = new Rectangle(left.value, top.value,
								width.value, height.value);
	gc.clipRect(r.x, r.y, r.width, r.height);
	draw(gc);
	gc.dispose();
}

/**
 * Returns the preferred size for the <code>ChartInteriorRegion</code> instance.
 * @return preferred size
 */
public Dimension preferredSize()
{
	Dimension ps = new Dimension();
	if (!width.isDefault) {
		ps.width += width.value;
	}
	else {
		ps.width = insets.left + insets.right;
	}
	if (!height.isDefault) {
		ps.height += height.value;
	}
	else {
		ps.height += insets.top + insets.bottom;
	}
	return(ps);
}

/**
 * Gets the location of the drawing region based on the margins.
 * @return the current location of the drawing region
 */
public Point location()
{
	return(new Point(left.value, top.value));
}

/**
 * Gets the size of the region.
 * @return the current size of the region
 */
public Dimension size()
{
	return(new Dimension(width.value, height.value));
}

/**
 * Gets the bounding rectangle of the component's drawing area,
 * (its area minus the shadows and insets).
 * @return the current bounding rectangle of the component's drawing area
 * @see #setInsets
 */
public Rectangle getDrawingArea()
{
	Rectangle rect = new Rectangle();
	getDrawingArea(rect);
	return(rect);
}

/** Gets the bounding rectangle of the component's drawing area,
 * (its area minus the shadows and insets).
 * @param rect an object in which the bounding rectangle of the component's
 *     drawing area is returned
 * @see #setInsets
 */
public void getDrawingArea(Rectangle rect)
{
	rect.setBounds(insets.left + left.value,
				   insets.top + top.value,
			       Math.max(0, size().width - (insets.left + insets.right)),
			       Math.max(0, size().height - (insets.top + insets.bottom)));
}

/**
 * Is the point (x,y) within the drawing area?.
 * @param x the x value of the point
 * @param y the y value of the point
 * @return <code>true</code>, if the point is within the drawing area
 */
public boolean inside(int x, int y)
{
	Rectangle rect = getDrawingArea();
	return(rect.contains(x, y));
}

/**
 * Makes the region visible and repaints the parent.
 */
public void show()
{
	setVisible(true);
	parentChart.repaint();
}

/**
 * Hides the region and repaints the parent.
 */
public void hide()
{
	if (!visible) {
		return;
	}
	setVisible(false);
	if (parentChart != null) {
		// Simple out is to repaint entire parent
		parentChart.repaint();

		// Better solution is to repaint only the exposed region
		// of the chart
		//parentChart.repaint(top.value, left.value, width.value, height.value);
	}
}

/** Sets the component's margins (default is <code>0</code>).<p>
 * <strong>HTML param name/value:</strong> "Insets"/Insets
 * @param insets the new component margins
 * @see com.klg.jclass.util.swing.JCSwingTypeConverter#toInsets
*/
public void setInsets(Insets insets)
{
	this.insets = insets;
	setChanged(true, Changeable.RECALC);
}

/** Returns the previously set insets.
 * @return the current component margins
 * @see #setInsets
 */
public Insets insets()
{
	return(insets);
}

/** Returns the previously set insets.
 * @return the current component margins
 * @see #setInsets
 */
public Insets getInsets()
{
	return(insets);
}

/**
 * Checks if the size has changed.
 * @return <code>true</code>, if the size has changed
 */
public boolean sizeChanged()
{
	if (dimIsDefaultChangedToTrue) {
		dimIsDefaultChangedToTrue = false;
		return(true);
	}
	if (getLeft()   == location().x  &&
	    getTop()    == location().y  &&
		getWidth()  == size().width  &&
		getHeight() == size().height)
	{
		return(false);
	}
	return(true);
}

}



