/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ChartRegion.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:45 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import javax.swing.*;
import java.awt.*;
import java.io.IOException;

/**
 * A utility class used inside JClass Chart to represent 
 * a drawable region on a chart. It is used
 * for child components.
 */
public class ChartRegion extends JComponent
	                     implements Changeable, java.io.Serializable
{

/** Used to turn off updates to parent in certain cases. */
protected boolean updateParent = true;

/** Boolean representing the value of the <code>Changed</code> property. */
protected boolean           changed = true;
protected int               changedFlag = Changeable.NO_CHANGE;
protected Dimension         preferredSize = null;
protected CellRendererPane  cellRendererPane = null;

/**
 * Default constructor for chart region.
 */
public ChartRegion()
{
	super();

	cellRendererPane = new CellRendererPane();
	add(cellRendererPane);
}

/**
 * Constructor that connects the <code>ChartRegion</code> to a parent <code>JCChart</code>.
 * @param p parent chart
 */
public ChartRegion(JCChart p)
{
	super();
	setBorder(BorderFactory.createEmptyBorder(3,3,3,3));

	cellRendererPane = new CellRendererPane();
	add(cellRendererPane);
}

public void paintComponent(Graphics gc)
{
	if (gc == null) {
        return;
    }

	// Draw own opaque backgrounds.  Shouldn't this be
	// done by the superclass?  That code appears to be in the UI, which
	// is null for this component it seems.
	if (isOpaque()) {
        if (!gc.getColor().equals(getBackground())) {
            gc.setColor(getBackground());
        }
		//Rectangle r = getDrawingArea();
		Rectangle r = getBounds();
        gc.fillRect(0, 0, r.width, r.height);
	}
}

/**
 * Returns the manually set preferred size for the <code>ChartRegion</code> instance.
 * @return manually set preferred size, or null is none manually set
 */
public Dimension getPreferredSize()
{
	return(preferredSize);
}

/**
 * Sets the preferred size manually for this <code>ChartRegion</code> instance.
 * @param dim the preferred size
 */
public void setPreferredSize(Dimension dim)
{
    preferredSize = dim;
}

/** Gets the bounding rectangle of the component's drawing area,
 * (its area minus the shadows and insets).
 * @see javax.swing.JComponent#setBorder
 * @return
 */
public Rectangle getDrawingArea()
{
	Rectangle rect = new Rectangle();
	getDrawingArea(rect);
	return(rect);
}

/** Gets the bounding rectangle of the component's drawing area,
 * (its area minus the insets).
 * @see javax.swing.JComponent#setBorder
 * @param rect
 */
public void getDrawingArea(Rectangle rect)
{
	Insets insets = getInsets();
	Dimension sz = getSize();
    rect.setBounds(insets.left, insets.top,
                   Math.max(0, sz.width - (insets.left + insets.right)),
                   Math.max(0, sz.height - (insets.top + insets.bottom)));
}

/**
 * Gets the cell renderer pane for this <code>ChartRegion</code>. Used to render HTML
 * encoded text with a <code>JLabel</code>.
 * @return
 */
public CellRendererPane getCellRendererPane()
{
	return(cellRendererPane);
}


// Changeable implementation begins here

/**
 * Gets the value of the <code>Changed</code> property.
 * The <code>Changed</code> property determines whether the object needs
 * to be recalculated.
 * @return <code>true</code> if the object has been changed and needs
 * recalculation; <code>false</code> otherwise
 */
public boolean getChanged()
{
	return(changed);
}

/**
 * Checks the value of the <code>ChangedFlag</code> property for the specified mask
 * (see <code>Changeable</code> for a list of valid masks).
 * The <code>ChangedFlag</code> property determines the action the object needs to
 * take in regards to the current change.
 * @return <code>true</code> if the specified mask exists in the <code>ChangedFlag</code> property;
 * <code>false</code> otherwise
 * @param mask
 */
public boolean getChanged(int mask)
{
	return((changedFlag & mask) == mask);
}

/**
 * Gets the value of the <code>Changed</code> property.
 * The <code>Changed</code> property determines whether the object needs
 * to be recalculated.
 * @return <code>true</code> if the object has been changed and needs
 * recalculation; <code>false</code> otherwise
 */
final public boolean isChanged()
{
	return(getChanged());
}

/**
 * Checks the value of the <code>ChangedFlag</code> property for the specified mask
 * (see <code>Changeable</code> for a list of valid masks).
 * The <code>ChangedFlag</code> property determines the action the object needs to
 * take in regards to the current change.
 * @return <code>true</code> if the specified mask exists in the <code>ChangedFlag</code> property;
 * <code>false</code> otherwise
 * @param mask
 */
final public boolean isChanged(int mask)
{
	return(getChanged(mask));
}

/**
 * Sets the value of the <code>Changed</code> property and updates the parent object.
 * The <code>Changed</code> property determines whether the object needs to be recalculated.
 * @param c new value for <code>Changed</code>
 * @param cf
 */
public void setChanged(boolean c, int cf)
{
	if (c != getChanged() || !getChanged(cf)) {
		changed = c;
		if (c) {
			changedFlag |= cf;
        }
		else {
			changedFlag = cf;
        }
	}
	if (c && getParent() != null && updateParent) {
		updateParent(cf);
	}
}

/**
 * Sets the value of the <code>Changed</code> property. The <code>Changed</code>
 * property determines whether the object needs to be recalculated.
 * @param n new value for <code>Changed</code>
 * @param updateParent if <code>true</code>, the parent is updated; otherwise,
 * the parent is not updated (the "update" semantics are up to the
 * implementor)
 * @param cf
 */
public void setChanged(boolean n, int cf, boolean updateParent)
{

	if (updateParent) {
		setChanged(n, cf);
    }
	else {
		changed = n;
		if (n) {
			changedFlag |= cf;
        }
		else {
			changedFlag = cf;
        }
	}
}

public void updateParent(int cf)
{
	JCChart p = (JCChart) getParent();
	if (p != null) {
		if (((cf & Changeable.RECALC) > 0) ||
			((cf & Changeable.NOTIFY_PARENT_LAYOUT) > 0))
        {
			p.invalidate();
			if (!p.isBatched()) {
				p.validate();
				// This repaint may be redundant in some cases.
				p.repaint();
			}
		}
		else if ((cf & Changeable.REDRAW) > 0) {
			if (!p.isBatched()) {
				p.repaint();
			}
		}
	}
}

public boolean isBatched()
{
	return(false);
}


/**
 * Overrides write object to change the state of the changed flags being
 * saved; this allows the chart to actually recalc and draw when re-serialized.
 * @param out
 * @throws IOException
 */
private void writeObject(java.io.ObjectOutputStream out) throws IOException
{
  // Save current state first
  boolean oldChanged = changed;
  int oldChangedFlag = changedFlag;

  // Save state that will generate a recalc
  changed = true;
  changedFlag = Changeable.RECALC;
  out.defaultWriteObject();

  // Restore current state
  changed = oldChanged;
  changedFlag = oldChangedFlag;
}

}



