/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ChartText.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:45 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import com.klg.jclass.util.swing.TextRenderer;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.AffineTransform;

/**
 * Any label that appears on a JClass Chart. A <code>ChartText</code> can be
 * rotated by increments of 90 degrees.  The 2D API is used to rotate the drawn
 * text.
 */
public class ChartText extends ChartInteriorRegion
			           implements java.io.Serializable
{

protected int      adjust = CENTER;
protected int      rotation = 0;
protected String   text = null;
protected JCAxis   parentAxis = null;
protected boolean  isHTML = false;

protected TextRenderer rendComp = null;

protected transient Image buffer;

public final static int  LEFT   = 0;
public final static int  CENTER = 1;
public final static int  RIGHT  = 2;

public final static int  DEG_0     = JCAxis.ROTATE_NONE;
public final static int  DEG_90    = JCAxis.ROTATE_90;
public final static int  DEG_180   = JCAxis.ROTATE_180;
public final static int  DEG_270   = JCAxis.ROTATE_270;
public final static int  DEG_OTHER = JCAxis.ROTATE_OTHER;

/**
 * Public constructor for <code>ChartText</code>. Creates a parentless
 * <code>ChartText</code> that can be attached to any <code>JCChart</code>
 * component.
 */
public ChartText() {}

/**
 * A smart <code>ChartText</code> constructor; it knows how to deal with String
 * and <code>ChartText</code> objects passed in.
 * @param p the parent chart
 * @param lab the text to be used for this <code>ChartText</code> object
 * @param axis the parent axis
 */
public ChartText(JCChart p, String lab, JCAxis axis)
{
	// Note: set parent after setting text to avoid unnecessary update
	setParentAxis(axis);
	setText(lab);
	setParentChart(p);
}

/**
 * A smart <code>ChartText</code> constructor; it knows how to deal with
 * String and <code>ChartText</code> objects passed in.
 * @param p the parent chart
 * @param lab the text to be used for this <code>ChartText</code> object
 * @param cr the parent chart region
 */
public ChartText(JCChart p, String lab, ChartRegion cr)
{
	// Note: set parent after setting text to avoid unnecessary update
	setParentRegion(cr);
	setText(lab);
	setParentChart(p);
}

/**
 * Sets the <code>ParentAxis</code> property. The <code>ParentAxis</code>
 * property may hold a reference to the axis to which the chart text belongs.
 * @param pa the new <code>ParentAxis</code>
 */
void setParentAxis(JCAxis pa)
{
	parentAxis = pa;
}

/**
 * Gets the font metrics of the font passed in.
 * @param f the font whose font metrics are required
 * @return the font metrics of the passed in font (null on error)
 */
public FontMetrics getFontMetrics(Font f)
{
	FontMetrics pFont = null;
   	try {
        // we query the font metrics from the chart graphics object
        // in case a subsitute graphics object (like those used for
        // various encoders or the ServerReport/PageLayout products)
        // with its own font metrics is being used
		Graphics chartQueryGC = parentChart.getQueryGraphics();
		if (chartQueryGC != null) {
       		pFont = chartQueryGC.getFontMetrics(f);
		}
		else {
			pFont = parentChart.getToolkit().getFontMetrics(f);
		}
   	}
	catch (Exception e) {}
	return(pFont);
}

/**
 * Renders the chart label using the provided graphics context.
 * @param gc graphics context
 */
public void draw(java.awt.Graphics gc)
{
	if (!visible || parentChart == null) {
		return;
	}

    Graphics localGC = null;
    if (gc != null) {
        boolean ot = transparent;
        if (transparent && (background != null)) {
            transparent = false;
        }
        super.draw(gc);
        transparent = ot;

        localGC = gc.create();
        localGC.setColor(getForeground());
        localGC.setFont(getFont());
    }

	if (text != null) {
        Rectangle rect = getDrawingArea();
		Graphics2D gc2 = (Graphics2D) localGC;

		// Do rotation on Graphics object
        boolean rotate = true;
        int tx = 0;
        int ty = 0;
        double theta = 0.0;
		if (rotation == DEG_90) {
            tx = rect.width;
            theta = Math.PI / 2.0;
		}
		else if (rotation == DEG_180) {
            tx = rect.width;
            ty = rect.height;
            theta = Math.PI;
		}
		else if (rotation == DEG_270) {
            ty = rect.height + 1;
            theta = 3 * Math.PI / 2.0;
		}
		else if (rotation == DEG_OTHER &&
				 !parentAxis.isPolar() &&
                 !parentAxis.isRadar())
        {
			theta = parentAxis.getAnnotationRotationAngle(JCChartUtil.RADIANS);
		}
        else {
            rotate = false;
        }

        if (gc2 != null) {
            if (rotate) {
                gc2.translate(tx, ty);
                gc2.rotate(theta, rect.x, rect.y);
            }
        }
        else {
            if (parentAxis != null) {
                // If a mapper exists we are generating an image map.  In this
                // case, we do not want to draw but we want to find the
                // enclosing shape and add it to the mapper's list.
                Mapper mapper = parentAxis.getMapper();
                if (mapper != null) {
                    Shape s = rect;
                    AffineTransform at = new AffineTransform();
                    JCChartArea chartArea = parentAxis.getChartArea();
                    if (chartArea != null) {
                        Point location = chartArea.getLocation();
                        Rectangle offset = chartArea.getDrawingArea();
                        at.translate(offset.x + location.x,
                                     offset.y + location.y);
                    }
                    if (rotate) {
                        at.translate(tx, ty);
                        at.rotate(theta, rect.x, rect.y);
                    }
                    s = at.createTransformedShape(s);
                    mapper.addShape(s);
                }
            }
            return;
        }

		// If text is html encoded, render it as html text
		if (isHTML) {
            ChartRegion drawRegion =
                (parentAxis == null ? getParentRegion()
                                    : parentAxis.getParentRegion());
			// Set text in rendering component, get parent region to draw
			// into and get the renderer pane we can use for that
			CellRendererPane crp = drawRegion.getCellRendererPane();

			if (crp != null) {
                if (rendComp == null) {
                    rendComp = new TextRenderer();
                }
				if (rotation == DEG_OTHER ||
                    rotation == DEG_0 ||
					rotation == DEG_180)
                {
					rendComp.drawHTMLText(gc2, crp, drawRegion, text, getLeft(),
										  getTop(), getWidth(), getHeight());
				}
				else {
					// For 90 and 270 degree rotation, rotating the gc will
					// automatically rotate the renderer component, so we
					// have to flip the width and height back
					rendComp.drawHTMLText(gc2, crp, drawRegion, text, getLeft(),
					                      getTop(), getHeight(), getWidth());
				}
			}
		}
		// For non-HTML text, render as per normal
		else {
			// Don't try to render empty text -- it throws an exception
			// during printing
            FontMetrics pFont = localGC.getFontMetrics();
			if (text.length() > 0 && pFont != null) {
				// Don't know why, but I seem to need a +1 on the x value when
				// drawing the text string.  Otherwise, it draws a pixel
				// too early.
				gc2.drawString(text, rect.x + 1,
							   rect.y + 1 + pFont.getAscent());
			}
		}
	}

	if (localGC != null) {
		localGC.dispose();
	}

	setChanged(false, Changeable.NO_CHANGE);
}

/**
 * Sets the <code>Rotation</code> property, which specifies the rotation
 * (<code>counterclockwise</code>) of the label.
 * @param rot one of <code>DEG_90</code>, <code>DEG_180</code>,
 * <code>DEG_270</code>, or <code>DEG_0</code>
 * @exception IllegalArgumentException If the rotation is not one of
 * <code>DEG_0</code>, <code>DEG_90</code>, <code>DEG_180</code>, or
 * <code>DEG_270</code>.
 */
public void setRotation(int rot)
{
	if (rot == rotation) {
		return;
	}
	rotation = rot;
	setChanged(true, Changeable.RECALC);
}

/**
 * Gets the value of the <code>Rotation</code> property, which specifies the
 * rotation (<code>counterclockwise</code>) of the label.
 * @return one of <code>DEG_0</code>, <code>DEG_90</code>, <code>DEG_180</code>,
 * or <code>DEG_270</code>
 */
public int getRotation()
{
    return(rotation);
}

/**
 * Gets the <code>Adjust</code> property, which determines how text is
 * justified (positioned) in the label.
 * @return one of <code>LEFT</code>, <code>CENTER</code>, or <code>RIGHT</code>
 */
public int getAdjust()
{
    return(adjust);
}

/**
 * Sets the <code>Adjust</code> property, which determines how text is
 * justified (positioned) in the label.
 * @param justify one of <code>LEFT</code>, <code>CENTER</code>, or
 * <code>RIGHT</code>
 * @exception IllegalArgumentException If the text justification is not
 * one of <code>LEFT</code>, <code>CENTER</code>, or <code>RIGHT</code>
 */
public void setAdjust(int justify)
{
	if (justify == adjust) {
		return;
	}
	switch (justify) {
	case LEFT:
	case CENTER:
	case RIGHT:
		adjust = justify;
		break;
	default:
		throw new IllegalArgumentException("Invalid Adjust in ChartText.  " +
		                                   "Must be one of LEFT, CENTER " +
                                           "or RIGHT");
	}
	setChanged(true, Changeable.REDRAW);
}

/**
 * Gets the value of the <code>Text</code> property. A <code>ChartText</code>
 * can render plain text.
 * @return returns a String instance if the <code>ChartText</code> is rendering
 * plain text, or returns null if the <code>ChartText</code> has no text
 */
public String getText()
{
    return(text);
}

/**
 * Sets the <code>Text</code> property.
 * @param nc new String contents of the <code>ChartText</code> instance;
 * otherwise, the raw text is used in the label
 */
public void setText(String nc)
{
	setText(nc, true);
}

/**
 * Sets the <code>Text</code> property.
 * @param nc new String contents of the <code>ChartText</code> instance.
 * @param isVisible if <code>true</code>, visible indication is set to
 * <code>true</code>
 */
public void setText(String nc, boolean isVisible)
{
	text = nc;

	// Check for HTML encoding
	if (TextRenderer.isHTML(text)) {
		isHTML = true;
	}
	else {
		isHTML = false;
	}

	if (isVisible) {
		setVisible(true);
	}
	if (parentChart == null) {
		return;
	}

	setChanged(true, Changeable.RECALC);
}

/**
 * Recalculates the extents of the label.
 * @param forceRecalc force a recalculation
 */
protected void recalc(boolean forceRecalc)
{
	boolean localChanged = false;

	if (isChanged(Changeable.RECALC)) {
		localChanged = true;
	}
	else {
		if (!forceRecalc) {
			return;
		}
	}
	Dimension s = calcSize();

	if (s.width == 0 || s.height == 0) {
		setSize(0, 0);
	}
	else if (rotation == DEG_90 || rotation == DEG_270) {
		// Flip dimensions if rotated by 90 or 270 degrees
		setSize(s.height, s.width);
	}
	else {
		// For all other angles, set width and height in the usual manner.
		setSize(s.width, s.height);
	}

	if (localChanged) {
		setChanged(false, Changeable.NO_CHANGE);
	}
}

public void recalc()
{
	recalc(false);
}

/**
 * Connects the chart region to a <code>JCChart</code>.
 * @param p parent chart
 */
public void setParentChart(JCChart p)
{
	super.setParentChart(p);
	setChanged(true, Changeable.RECALC, false);
	recalc();
}

/**
 * Calculates the size needed to display the existing text in this object.
 * @return the calculated size
 */
protected Dimension calcSize()
{
	// If html encoded text, let the renderer component figure out the size
	if (isHTML) {
        if (rendComp == null) {
            rendComp = new TextRenderer();
        }
		return(rendComp.calcHTMLTextSize(text));
	}

	// if standard text, use font metrics to calculate the size
	Font thisFont = getFont();
	if (thisFont == null) {
		return(new Dimension(0,0));
	}

	FontMetrics pFont = getFontMetrics(getFont());
	if (pFont != null) {
		// Guard against text changing during size calculation.
		if (text == null) {
			return(new Dimension(0, pFont.getHeight()));
		}
		Dimension dim = new Dimension(pFont.stringWidth(text),
							 		  pFont.getHeight());
		JCChartUtil.fudgeItalicLabel(thisFont, dim);
		return(dim);
	}
	else {
		return(new Dimension(0,0));
	}
}

/**
 * Attempts to calculate the size of one line of a multi-line piece of text.
 * If this is just straight text, it can only be one line anyways, so we
 * just return the ascent from the font metrics.  If this object contains
 * HTML encoded text, however, we try to make a guess at the height of the
 * first line by counting line breaks.  If the text uses different font
 * sizes on multiple lines, this calculation will be off.
 * <br><B>Note:</b> A full recalc should have been done on this object first.
 * @return the calculated line size
 */
public Dimension calcLineSize()
{
	// If HTML encoded string, try to figure out number of line breaks
	// and divide the full height by that value
	if (isHTML) {
		int paraCount = 1;
		paraCount += JCChartUtil.countInString("<p>", text);
		paraCount += JCChartUtil.countInString("<br>", text);
		paraCount += JCChartUtil.countInString("<P>", text);
		paraCount += JCChartUtil.countInString("<BR>", text);

		return(new Dimension(getWidth(), getHeight()/paraCount));
	}

	// If standard text, it can only be a single line, so just get the
	// ascent from the font metrics

	Font f = getFont();
	if (f == null) {
		return(new Dimension(0,0));
	}
	FontMetrics pFont = getFontMetrics(f);
	return(new Dimension(getWidth(), pFont.getAscent()));
}

/**
 * Calculates the desired foreground color of this <code>ChartText</code>
 * object.  If no foreground color has been set, we can take the color of the
 * parent region, the parent axis, or the parent chart.
 * @return the calculated foreground color
 */
public Color getForeground()
{
	Color fg = foreground;
	if (fg == null && parentRegion != null) {
		fg = parentRegion.getForeground();
	}
	if (fg == null && parentAxis != null) {
		fg = parentAxis.getForeground();
	}
	if (fg == null && parentChart != null) {
		fg = parentChart.getForeground();
	}
	return(fg);
}

/**
 * Calculates the desired background color of this <code>ChartText</code>
 * object.  If no background color has been set, we can take the color of the
 * parent axis or the parent chart, if it is not opaque.
 * @return the calculated background color
 */
public Color getBackground()
{
	Color bg = background;
	if (bg == null && parentAxis != null) {
		bg = parentAxis.getBackground();
	}
	if (bg == null && parentChart != null) {
	    bg = parentChart.isOpaque() ? parentChart.getBackground() : null;
	}
	return(bg);
}

/**
 * Caculates the desired font of this <code>ChartText</code> object.  If no
 * font has been set, we can take the font of the parent axis or the parent
 * chart.
 * @return the calculated font
 */
public Font getFont()
{
	Font f = font;
	if (f == null && parentAxis != null) {
		f = parentAxis.getFont();
	}
	if (f == null && parentChart != null) {
		f = parentChart.getFont();
	}
	return(f);
}

/**
 * Gets the preferred size of this <code>ChartText</code> object.
 * @return the preferred size
 */
public Dimension preferredSize()
{
	recalc();
	Dimension ps = size();
	return(ps);
}

/**
 * Gets a String version of this object, which is just the
 * <code>ChartText</code>'s text if it is not null.
 * @return a String version of this object
 */
public String toString()
{
	if (text == null) {
		return(super.toString());
	}
	return(text);
}

}
