/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CircularChartAnnoHandler.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:45 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import com.klg.jclass.util.legend.JCLegend;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.FontMetrics;
import java.util.Vector;

/**
 * An AnnotationHandler for Polar/Radar charts.
 */
class CircularChartAnnoHandler extends AnnotationHandler {

int thetaAxisLabelOffset = 4;
// Number of pixels between a label and the
// theta axis of a Polar/Radar chart.

CircularChartAnnoHandler(JCAxis ax) {
    super(ax);
}

/**
 * Identifies the axis labels that extend furthest to the top, bottom, left,
 * and right of a Polar/Radar chart.  The returned object is a Rectangle that
 * defines the bounds of the annotation.
 */
protected AnnoBoundsInfo getPolarAnnotationBounds()
{
    AnnoBoundsInfo annoBoundsInfo = null;
    int left = Integer.MAX_VALUE;
    int top = Integer.MAX_VALUE;
    int right = Integer.MIN_VALUE;
    int bottom = Integer.MIN_VALUE;

    JCValueLabel vlMinBoundsX = null;
    JCValueLabel vlMaxBoundsX = null;
    JCValueLabel vlMinBoundsY = null;
    JCValueLabel vlMaxBoundsY = null;

    int first = firstVisibleLabel();
    int last = lastVisibleLabel();

    if (first >= 0 && first < valueLabels.size() &&
        last >= 0 && last < valueLabels.size())
    {
        for (int i = first; i <= last; i++) {
            JCValueLabel vl = (JCValueLabel)valueLabels.elementAt(i);
            vl.label.recalc();
//            System.out.println("vl.label.text=" + vl.label.text
//                               + ", vl.label.top.value=" + vl.label.top.value
//                               + ", vl.label.left.value="
//                               + vl.label.left.value
//                               + ", vl.label.width.value="
//                               + vl.label.width.value
//                               + ", vl.label.height.value="
//                               + vl.label.height.value);
            if (left > vl.label.left.value) {
                left = vl.label.left.value;
                vlMinBoundsX = vl;
            }
            if (top > vl.label.top.value) {
                top = vl.label.top.value;
                vlMinBoundsY = vl;
            }
            if (right < vl.label.left.value + vl.label.width.value) {
                right = vl.label.left.value + vl.label.width.value;
                vlMaxBoundsX = vl;
            }
            if (bottom < vl.label.top.value + vl.label.height.value) {
                bottom = vl.label.top.value + vl.label.height.value;
                vlMaxBoundsY = vl;
            }
        }
        annoBoundsInfo = new AnnoBoundsInfo();
        annoBoundsInfo.x = left;
        annoBoundsInfo.y = top;
        annoBoundsInfo.width = right-left;
        annoBoundsInfo.height = bottom-top;
        annoBoundsInfo.valueLabelMinBoundsX = vlMinBoundsX;
        annoBoundsInfo.valueLabelMaxBoundsX = vlMaxBoundsX;
        annoBoundsInfo.valueLabelMinBoundsY = vlMinBoundsY;
        annoBoundsInfo.valueLabelMaxBoundsY = vlMaxBoundsY;
//        System.out.println("x=" + annoBoundsInfo.x +
//                           ", y=" + annoBoundsInfo.y +
//                           ", width=" + annoBoundsInfo.width +
//                           ", height=" + annoBoundsInfo.height +
//                           ", vertical=" + parent.vertical);
    }
    return(annoBoundsInfo);
}

protected double getCircumference(int angleUnit)
{
    double circumference;
    switch (angleUnit) {
    case JCChartUtil.RADIANS:
        circumference = JCChartUtil.M_2PI;
        break;
    case JCChartUtil.GRADS:
        circumference = 400.0;
        break;
    case JCChartUtil.DEGREES:   //  fall through to default
    default:
        circumference = 360.0;
    }
    return(circumference);
}

private boolean isPolarRadarThetaAxis()
{
    return((parent.isPolar() || parent.isRadar()) && !parent.i_vertical);
}

/**
 * Compute the anchor position of an axis label.
 *
 * @param cl The ChartText object to contain the anchor
 * coordinates (in pixels).
 * @param value The value of the label.
 */
private void layoutPolar(ChartText cl, double value, int labelExtent)
{
    cl.recalc();

    if (parent.i_vertical) {
        layoutPolarRAxis(cl, value, labelExtent);
    }
    else {
        layoutPolarThetaAxis(cl, value, labelExtent);
    }
}

/**
 * Called by layoutPolar() to compute the anchor position of
 * a radial axis label.
 *
 * @param cl The ChartText object to contain the anchor
 * coordinates (in pixels).
 * @param value The value of the label.
 */
private void layoutPolarRAxis(ChartText cl, double value, int labelExtent)
{
    double textOffset;
    Point pnt = null;
    JCAxis rAxis = (parent.i_vertical) ? parent : parent.otherAxis;
    JCAxis thetaAxis = (parent.i_vertical) ? parent.otherAxis : parent;
    //  Convert from data to pixel units:
    double scaledValue = Double.MAX_VALUE;
    if (parent.isLogarithmic()) {
        double logValue = JCChartUtil.log10(value);
        if (rAxis.isReversed()) {
            scaledValue = parent.applyDataToPixelScaling(
                            JCChartUtil.log10(parent.max.value) )
                            - parent.applyDataToPixelScaling(logValue);
        }
        else {
            scaledValue = parent.applyDataToPixelScaling(logValue)
                            - parent.applyDataToPixelScaling(
                                        JCChartUtil.log10(parent.min.value));
        }
    }
    else {
        if (rAxis.isReversed()) {
            scaledValue = parent.applyDataToPixelScaling(parent.max.value) -
                            parent.applyDataToPixelScaling(value);
        }
        else {
            scaledValue = parent.applyDataToPixelScaling(value) -
                            parent.applyDataToPixelScaling(parent.min.value);
        }
    }

    double axisAngle = parent.yAxisAngle;
    double angleRatio = JCChartUtil.getAngleRatio(JCChartUtil.RADIANS,
                                            axisAngle,
                                            parent.polarAxisParams.originBase,
                                            false);
    //  Pythagorean theorem:
    double r1 = Math.sqrt(labelExtent * labelExtent + scaledValue * scaledValue);
    double angle1 = 0.0;
    if (r1 != 0.0) {
        angle1 = Math.asin(labelExtent/r1);   // radians
        if (thetaAxis.reversed) {
            angle1 = -angle1;
        }
    }  //  else use angle1 = 0

    switch (anchor) {
    case JCLegend.WEST:
        if (angleRatio <= 0.0625) {   //  near 0 degrees
            pnt = parent.toPixelPolar(axisAngle + angle1, r1, false);
            textOffset = cl.getWidth()/2;
            pnt.x -= (int)Math.round(textOffset);
            pnt.y -= cl.getHeight();
        }
        else if (angleRatio > 0.0625 && angleRatio < 0.4375) {
            // greater than 1/16 and less than 7/16, i.e., top half of chart
            pnt = parent.toPixelPolar(axisAngle + angle1, r1, false);
            textOffset = cl.getWidth();
            pnt.x -= (int)Math.round(textOffset);
            pnt.y -= cl.getHeight()/2;
        }
        else if (angleRatio >= 0.4375 && angleRatio <= 0.5625) {
            //  near 180 degrees
            pnt = parent.toPixelPolar(axisAngle + angle1, r1, false);
            //  labels appear below the axis
            textOffset = cl.getWidth()/2;
            pnt.x -= (int)Math.round(textOffset);
            //  pnt.y is unchanged.
        }
        else if (angleRatio > 0.5625 && angleRatio < 0.9375) {
            //  greater than 9/16 and less than 15/16, i.e., bottom half of the chart
            pnt = parent.toPixelPolar(axisAngle - angle1, r1, false);
            textOffset = cl.getWidth();
            pnt.x -= (int)Math.round(textOffset);
            pnt.y -= cl.getHeight()/2;
        }
        else if (angleRatio >= 0.9375) {   //  near 360 degrees
            pnt = parent.toPixelPolar(axisAngle - angle1, r1, false);
            //  labels appear below the axis
            textOffset = cl.getWidth()/2;
            pnt.x -= (int)Math.round(textOffset);
            //  pnt.y is unchanged.
        }
        break;
    case JCLegend.EAST:

        if (angleRatio <= 0.0625) {   //  near 0 degrees
            pnt = parent.toPixelPolar(axisAngle - angle1, r1, false);
            //  labels appear below the axis
            textOffset = cl.getWidth()/2;
            pnt.x -= (int)Math.round(textOffset);
            // pnt.y is unchanged
        }
        else if (angleRatio > 0.0625 && angleRatio < 0.4375) {
            // greater than 1/16 and less than 7/16, i.e., top half of chart
            pnt = parent.toPixelPolar(axisAngle - angle1, r1, false);
            pnt.y -= cl.getHeight()/2;
        }
        else if (angleRatio >= 0.4375 && angleRatio <= 0.5625) {
            //  near 180 degrees
            pnt = parent.toPixelPolar(axisAngle - angle1, r1, false);
            //  labels appear above the axis
            textOffset = cl.getWidth()/2;
            pnt.x -= (int)Math.round(textOffset);
            pnt.y -= cl.getHeight();
        }
        else if (angleRatio > 0.5625 && angleRatio < 0.9375) {
            //  greater than 9/16 and less than 15/16, i.e., bottom half
            //  of the chart
            pnt = parent.toPixelPolar(axisAngle + angle1, r1, false);
            pnt.y -= cl.getHeight()/2;
        }
        else if (angleRatio >= 0.9375) {   //  near 360 degrees
            pnt = parent.toPixelPolar(axisAngle - angle1, r1, false);
            //  labels appear below the axis
            textOffset = cl.getWidth()/2;
            pnt.x -= (int)Math.round(textOffset);
            //  pnt.y is unchanged.
        }
        break;
    default:   //  Centered on axis
        pnt = parent.toPixelPolar(axisAngle, r1, false);
        textOffset = cl.getWidth()/2;
        pnt.x -= (int)Math.round(textOffset);
        pnt.y -= cl.getHeight()/2;
        break;
    }
    cl.setLocation(pnt.x, pnt.y);
}

/*
 * Called by layoutPolar() to compute the anchor position of a
 * theta axis label.
 * Assumes that the label is displayed horizontally
 *
 * @param cl The ChartText object to contain the anchor
 * coordinates (in pixels).
 * @param value The value of the label.
 * @param labelExtent The labelExtent property of the
 * associated JCAnno object.
 */
private void layoutPolarThetaAxis(ChartText cl, double value, int labelExtent)
{
    thetaAxisLabelOffset = labelExtent;
    double yOffset = 0.0;
    double xOffset = 0.0;
    double annoHeight = cl.getHeight();
    double angleRadians =
                JCChartUtil.convertAngle(parent.chartArea.getAngleUnit(),
                                         JCChartUtil.RADIANS, value);
    Point pnt = parent.toPixelPolar(angleRadians,
                    (int)Math.round(parent.polarAxisParams.radius) +
                    thetaAxisLabelOffset,
                    false);
    //  false means don't convert from data to pixel units
    JCAxis thetaAxis = (parent.i_vertical) ? parent.otherAxis : parent;
    double angleRatio = JCChartUtil.getAngleRatio(JCChartUtil.RADIANS,
                                            angleRadians,
                                            parent.polarAxisParams.originBase,
                                            thetaAxis.isReversed());
    if (angleRatio <= 0.25 || angleRatio >= 0.75) {
        /* right half */
        if (angleRatio <= 0.25) {
            /* top right quadrant */
            yOffset = (int)(2.0 * annoHeight * (- angleRatio - 0.25));
            //  If angleRatio is zero, yOffset is -0.5 * annoHeight.
            //  If angleRatio is 0.25, yOffset is -annoHeight.
        }
        else {
            /* bottom right quadrant */
            yOffset = (int)(2.0 * annoHeight * (1.0 - angleRatio) -
                                                        (annoHeight / 2));
            //  If angleRatio is 1, yOffset is -0.5 * annoHeight.
            //  If angleRatio is 0.75, yOffset is 0.
        }

        if (angleRatio < 0.25 || angleRatio > 0.75) {
            xOffset = 0;
        }
        else {   //  angleRatio is 0.25 or 0.75
            xOffset = -cl.getWidth()/2;
        }
    }
    else {
        /* left half */
        if (angleRatio <= 0.5) {
            //  top left quadrant
            yOffset = (int) (2.0 * annoHeight * (- 1.0 + angleRatio) +
                             (annoHeight / 2));
            //  If angleRatio is 0.5, yOffset is -0.5 * annoHeight.
            //  If angleRatio is 0.25, yOffset is -annoHeight
        }
        else {
            //  bottom left quadrant
            yOffset = (int) (2.0 * annoHeight * (angleRatio - 0.75));
            //  If angleRatio is 0.5, yOffset is -0.5 * annoHeight.
            //  If angleRatio is 0.75, yOffset is 0.
        }
        xOffset = - cl.getWidth();
    }
//    System.out.println("layoutPolarThetaAxisHorizText():  pnt.x + xOffset=" +
//                       (pnt.x + xOffset) + ", pnt.y + yOffset=" +
//                       (pnt.y + yOffset));
    cl.setLocation((int)(pnt.x + xOffset), (int)(pnt.y + yOffset));
}

private void makePointLabelsRadar(ChartDataView dv)
{
    //  Go through all of the series to find the highest
    //  value of firstPoint, and the lowest value of
    //  lastPoint.
    if (dv != null) {
        int[] limits = ChartDataView.getFirstLastPoints(dv);
        if (limits != null) {
            valueLabels = new Vector();
            ChartDataViewSeries series = dv.getSeries(0);
            int maxIndex = Math.min(limits[1], dv.getPointLabels().size()-1);
            for (int i = limits[0]; i <= maxIndex; i++) {
                String label = (String)dv.getPointLabels().get(i);
                if (label == null) {
                    continue;
                }
                JCValueLabel vl = new JCValueLabel(series.getX(i),
                                                   label, parent);

                // Set a data index to point back to the dataView and point.
                // Used for image maps.
                vl.setDataIndex(new JCDataIndex(dv, null, -1, i));

                // If the useAnnoTicks property of JCAxis is set, draw ticks
                // specified by the List of JCAnno objects (don't automatically
                // draw ticks at the labels).
                if (!parent.useAnnoTicks) {
                    vl.anno = parent.getDefaultTicks();
                }
                vl.label.setRotation(parent.annotationRotation);
                vl.setParent(parent);
                valueLabels.addElement(vl);
            }
        }
    }
}

private void makeValueAnnotationsRadar(ChartDataView dv, JCAnno annoObj)
{
    //  Theta axis of a Radar chart.
    //  Number the spokes of the Radar chart 0,1,2,3,....
    if (dv != null) {
        int[] limits = ChartDataView.getFirstLastPoints(dv);
        if (limits != null) {
            JCValueLabel vl = null;
            int nSpokes = limits[1] - limits[0] + 1;
            for (int i = 0; i < nSpokes; i++) {
                int angleUnit = parent.getChartArea().getAngleUnit();
                double radarAngle = JCChartUtil.getRadarAngle(angleUnit,
                                                              nSpokes, i,
                                                              false);
                // We pass false for the last parameter because
                // toPixelPolar() handles theta axis reversal.
                vl = new JCValueLabel(radarAngle, Integer.toString(i), parent);
                //  setParent() calls label.setParentChart()
                vl.setParent(parent);
                vl.setAnno(annoObj);
                valueLabels.addElement(vl);
            }
        }
    }
}

/**
 * Returns the font ascent, or -1 if the ascent is not available.
 */
private int getFontAscent(ChartText ct)
{
    int ascent = -1;
    FontMetrics pFont = ct.getFontMetrics(ct.getFont());
    if (pFont != null) {
        ascent = pFont.getAscent();
    }
    return ascent;
}

/**
 * Returns true if the ChartTexts overlap.
 */
private boolean isOverlapping(ChartText ct1, ChartText ct2)
{
    boolean fOverlap = false;
    if (ct1 != null && ct1.visible && ct2 != null && ct2.visible) {
        int ascent = getFontAscent(ct1);
        if (ascent < 0) {
            ascent = ct1.getHeight();
        }
        Rectangle rect1 = new Rectangle(ct1.getLeft(), ct1.getTop(),
                                        ct1.getWidth(), ascent);
        ascent = getFontAscent(ct2);
        if (ascent < 0) {
            ascent = ct2.getHeight();
        }
        Rectangle rect2 = new Rectangle(ct2.getLeft(), ct2.getTop(),
                                        ct2.getWidth(), ascent);
        fOverlap = rect1.intersects(rect2);
    }
    return fOverlap;
}

protected void checkValueAnnotations()
{
    if (parent.isPolar() && !parent.i_vertical) {
        // Check for overlap between visible labels.
        for (int j = 0; j < valueLabels.size()-1; j++) {
            JCValueLabel vl1 = (JCValueLabel)valueLabels.elementAt(j);
            ChartText ct1 = vl1.getChartText();
            if (!ct1.visible) {
                continue;
            }

            // Check for overlap between ct1 and the next visible label
            JCValueLabel vl2 = (JCValueLabel)valueLabels.elementAt(j+1);
            ChartText ct2 = vl2.getChartText();
            if (!ct2.visible) {
                continue;
            }
            if (isOverlapping(ct1, ct2)) {
                ct2.visible = false;
            }
        }
        layout();
    }
    else {
        super.checkValueAnnotations();
    }
}

/**
 * Override the base class method.
 * Moves all the axis annotations to the correct positions.
 */
protected void layout()
{
    int arraySize = valueLabels.size();
    for (int i = 0; i < arraySize; i++) {
        JCValueLabel vl = (JCValueLabel)valueLabels.elementAt(i);
        JCAnno annoObj = vl.getAnno();
        int labelExtent = 4;
        if (annoObj != null) {
            labelExtent = annoObj.getLabelExtent();
        }
        if (!(parent.isLogarithmic() && vl.value <= 0.0) ||
            ((parent.isPolar() || parent.isRadar()) && !parent.i_vertical))
        {
            // We allow the user to set the logarithmic flag for the x axis,
            // even if the chart type is Polar/Radar.  If the chart is
            // Polar/Radar, we ignore the logarithmic flag.
            if ( parent.isPolar() || (parent.isRadar() && parent.i_vertical) ) {
                layoutPolar(vl.label, vl.value, labelExtent);
            }
            else if (parent.isRadar() && !parent.i_vertical) {
                //  if x axis of a Radar chart
                double radarAngle =
                    JCChartUtil.getRadarAngle(parent.chartArea.getAngleUnit(),
                                              arraySize, i, false);
                // We pass false for the last parameter because toPixelPolar()
                // handles theta axis reversal.
                layoutPolar(vl.label, radarAngle, labelExtent);
            }
        }
    }
}

/**
 * Override the base class method.
 */
protected boolean skipToNext(JCValueLabel vl)
{
    boolean retVal = false;
    if (!isPolarRadarThetaAxis()) {
        if (vl.value < parent.min.value) {
            ChartText cl = vl.getChartText();
            cl.visible = false;
            retVal = true;
        }
    }
    return retVal;
}

/**
 * Override the base class method.
 */
protected boolean jumpToNext(JCValueLabel vl)
{
    boolean retVal = false;
    ChartText cl = vl.getChartText();
    if (!isPolarRadarThetaAxis()) {
        if (vl.value > parent.max.value) {
            cl.visible = false;
            retVal = true;
        }
    }
    else {
        //  Show all labels for theta axis of Polar/Radar chart
        cl.visible = true;
        retVal = true;
    }
    return retVal;
}

/**
 * Override the base class method.
 * Used in the point label and value label case to make sure
 * user-provided annotations don't grossly overlap.
 */
protected void checkUserProvidedAnnotations()
{
    checkUserProvidedLabels();
}

/**
 * Override the base class method.
 */
protected void makePointLabels(ChartDataView dv)
{
    makePointLabelsRadar(dv);
}

/**
 * Override the base class method.
 */
protected void makeValueAnnotations(ChartDataView dv)
{
    if (parent.isRadar() && !parent.i_vertical) {
        //  Theta axis of a Radar chart.
        //  Number the spokes of the Radar chart 0,1,2,3,....
        valueLabels = new Vector();
        makeValueAnnotationsRadar(dv, parent.getDefaultLabels());
    }
    else {
        super.makeValueAnnotations(dv);
    }
}

}
