/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CircularChartLinearAxisHandler.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:45 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.FontMetrics;
import java.util.List;

/**
 * A linear axis handler for Polar/Radar charts.
 */
class CircularChartLinearAxisHandler extends CircularChartAxisHandler
                                     implements LinearAxisHandler {

CircularChartLinearAxisHandler()
{
    axisScaleHandler = new LinearAxisScaleHandler();
}

protected double getScaledValue(double value)
{
    //  Because tick marks are evenly spaced, doesn't matter
    //  if y axis is reversed.
    return(scale * (value - parent.min.value));
}

/**
 * Performs pixel-to-data conversion for a Polar or Radar chart.
 *
 * @param x chart area x coordinate of pixel
 * @param y chart area y coordinate of pixel
 * @return Data space coordinates of this pixel.  JCDataCoord.x
 * is the polar angle value, and JCDataCoord.y is the radius.
 */
protected JCDataCoord toDataPolar(int x, int y)
{
    double dx = (double)x - parent.polarAxisParams.xCenter;
    double dy = (double)y - parent.polarAxisParams.yCenter;
    JCAxis rAxis = getRadialAxis();
    if (rAxis == null) {
        return null;
    }

    double pixelDistance = Math.sqrt(dx * dx + dy * dy);
    double r = Double.MAX_VALUE;
    if (rAxis.isReversed()) {
        r = rAxis.max.value - (pixelDistance / -scale);
    }
    else {
        r = (pixelDistance / scale) + rAxis.min.value;
    }
    double theta = calcThetaAngle(dx, dy);
    return(new JCDataCoord(theta, r));
}

/*
* Returns the chart area pixel location of a point at polar coordinates (theta,r).
*
* @param theta angle in radians.
* @param r radius in data units or pixel units
* @param convertDataToPixel true if radius is in data units, false if radius
* is in pixel units.
* @return pixel location.
*/
protected Point toPixelPolar(double theta, double r,
                             boolean convertDataToPixel)
{
    if (convertDataToPixel) {
        if (parent.isPolar()) {
            r = polarSubtractMinimum(r);
        }
        else {   // this is a Radar chart
            r = radarSubtractMinimum(r);
        }
    }

    JCAxis thetaAxis = (parent.i_vertical) ? parent.otherAxis : parent;
    if (thetaAxis.isReversed()) {
        theta *= -1.0;
    }

    JCDataCoord screenLocation =
        toScreenLocationPolar(theta, r, convertDataToPixel);

    double value = screenLocation.getX();
    int iX = (value < parent.polarAxisParams.xCenter) ?
        (int)Math.ceil(value) : (int)Math.round(value);

    value = screenLocation.getY();
    int iY = (value < parent.polarAxisParams.yCenter) ?
        (int)Math.ceil(value) : (int)Math.round(value);

    return(new Point(iX, iY));
}

private double polarSubtractMinimum(double r)
{
    JCAxis rAxis = (parent.i_vertical) ? parent : parent.otherAxis;
    if (rAxis.isReversed()) {
        r = rAxis.max.value - r;
    }
    else {
        boolean fMinNegative = (rAxis.dataLimits.min < 0.0);
        r -= (fMinNegative) ? -rAxis.min.value : rAxis.min.value;
    }
    return(r);
}

private double radarSubtractMinimum(double r)
{
    JCAxis rAxis = (parent.i_vertical) ? parent : parent.otherAxis;
    if (rAxis.isReversed()) {
        r = rAxis.max.value - r;
    }
    else {
        r -= rAxis.min.value;
    }
    return(r);
}

private void setDefaultThetaMinMax()
{
    if (!parent.i_vertical) {
        int angleUnit = parent.chartArea.getAngleUnit();
        double minReal = (parent.isHalfRange()) ?
            JCChartUtil.convertAngle(JCChartUtil.RADIANS,
                                     angleUnit, -JCChartUtil.M_PI) : 0.0;

        double maxReal = (parent.isHalfRange()) ?
            JCChartUtil.M_PI : JCChartUtil.M_2PI;
        maxReal = JCChartUtil.convertAngle(JCChartUtil.RADIANS,
                                           angleUnit,
                                           maxReal);
        parent.setMinIsDefault(true);
        parent.setMaxIsDefault(true);
        // For a theta axis, the lmin and minReal parameters
        // are identical, as are lmax and maxReal.
        LinearAxisUtil.setAxisMinMax(parent,
                                     minReal, maxReal,
                                     minReal, maxReal);
    }
}

protected void calcParams()
{

    double dataMin = (parent.dataLimits == null) ?
        0.0 : parent.dataLimits.min;
    double dataMax = (parent.dataLimits == null) ?
        100.0 : parent.dataLimits.max;
    double maxAxisValue = Double.MAX_VALUE;
    if (parent.i_vertical) {
        if (parent.isReversed()) {
            maxAxisValue = (parent.dataLimits == null) ?
                100.0 : parent.dataLimits.max;
        }
        else {
            maxAxisValue = Math.max(Math.abs(dataMin), Math.abs(dataMax));
            //  Take the absolute value of each, since both min and max
            //  may be negative.
        }
    }

    double dlmin;
    double dlmax;

    if (!parent.i_vertical) {
        //  The theta axis of a POLAR/RADAR chart always has these
        //  max and min values:
        dlmin = 0.0;
        dlmax = JCChartUtil.convertAngle(JCChartUtil.RADIANS,
                                         parent.chartArea.getAngleUnit(),
                                         JCChartUtil.M_2PI);
    }
    else {
        //  The y axis minimum of a POLAR chart cannot be negative,
        //  unless the axis is reversed.
        double yMinValue = dataMin;
        //  We want to avoid clipping.
        if (!parent.isReversed() && yMinValue < 0.) {
            yMinValue = 0.;
        }
        dlmin = yMinValue;
        dlmax = maxAxisValue;
    }

    boolean polarChart = (parent.chartType == JCChart.POLAR);
    /*  For Polar charts, if rAxis.reversed is false, the r axis values
     *  can't be negative.  (theta,-r) is interpreted as (theta+180,r).
     *  So if dlmin is negative, reset it to 0 for calculation of axis max
     *  and min values.  Because points with negative r values are plotted
     *  as positive r (shifted 180 degrees), we have to ensure that dlmax
     *  is large enough to bound very negative values.  Hence, if
     *  abs(dlmin) is greater than dlmax, set dlmax to abs(dlmin).
     *
     *  If the rAxis of a Polar chart is reversed, we allow rAxis to extend
     *  out to negative values.
     */
    if (polarChart && parent.i_vertical) {
        if (!parent.isReversed()) {
            //  If both dlmin and dlmax are negative:
            if (dlmax < 0.0) {
                double tmp = dlmin;
                dlmin = Math.abs(dlmax);
                dlmax = Math.abs(tmp);
                //  For example, if dlmin=-70 and dlmax=-50,
                //  dlmin becomes 50, and dlmax becomes 70.
            }
            else {
                double dlminAbs = Math.abs(dlmin);
                if (dlminAbs > dlmax) {
                    dlmax = dlminAbs;
                }
                if (dlmin < 0.0) {
                    dlmin = 0.0;
                }
            }
        }
    }

    double lmax = dlmax;
    double lmin = dlmin;
    double org = 0.0;
    double inc = parent.numSpacing.value;
    double tik = parent.tickSpacing.value;

    if (!parent.i_vertical) {
        setDefaultThetaMinMax();
    }
    else if (polarChart && parent.i_vertical) {
        //  parent is the y axis of a Polar chart.
        //  The y axis minimum cannot be negative
        //  unless the y axis is reversed.
        if (parent.min.value < 0. && !parent.isReversed()) {
            parent.min.value = 0;
            parent.setMinIsDefault(true);
        }
        else if (parent.min.value > dataMin) {
            if ((dataMin < 0.) && !parent.isReversed()) {
                parent.min.value = 0;
            }
            else {
                parent.min.value = dataMin;
            }
            parent.setMinIsDefault(true);
        }
        else if ((dataMin < 0) &&
            (dataMax < 0) &&
            !parent.isReversed()) {
            //  All data points are negative.  Set the y axis
            //  minimum to zero if the axis is not reversed.
            parent.min.value = 0;
            parent.setMinIsDefault(true);
        }

        if (parent.max.value < maxAxisValue) {
            parent.max.value = maxAxisValue;
            parent.setMaxIsDefault(true);
        }
    }
    else {
        //  parent is the y axis of a Radar chart.
        //  We want to avoid clipping.
        if (parent.min.value > dataMin) {
            parent.min.value = dataMin;
            parent.setMinIsDefault(true);
        }
        if (parent.max.value < dataMax) {
            parent.max.value = dataMax;
            parent.setMaxIsDefault(true);
        }
    }

    do {
        calcPrecision();
    }
    while (axisScaleHandler.fixBounds());

    double maxOrig = lmax;
    double minOrig = lmin;

    int tprec = parent.getPrecision();

    lmin = JCChartUtil.precFloor(tprec, dlmin);
    lmax = JCChartUtil.precCeil(tprec, dlmax);

    if (lmax <= lmin) {
        // Move the max. and min. values based on the precision.
        // Keep adjusting the spacing until we get some movement
        //
        //  Note that something very similar to this is also done again later
        //  in this routine for Time labeled axes which could have a 0
        //  data range but for which precision is irrelevant.
        tprec = parent.getPrecision();
        if (parent.max.isDefault) {
            // Move max
            do {
                lmax = (lmin + JCChartUtil.pow10(-tprec));
                tprec--;
            }
            while (lmax <= lmin);
        }
        else {
            // Move min
            do {
                lmin = (lmax - JCChartUtil.pow10(-tprec));
                tprec--;
            }
            while (lmax <= lmin);
        }
    }

    lmin = parent.min.isDefault ? lmin : parent.min.value;
    lmax = parent.max.isDefault ? lmax : parent.max.value;

    org = calcOrigin(lmin, lmax);

    double minReal = parent.min.isDefault ? JCChartUtil.min(lmin, org) :
        lmin;
    double maxReal = parent.max.isDefault ? JCChartUtil.max(lmax, org) :
        lmax;

    double error = 0.0;

    boolean pass = false;

    int numTicks = 0;
    if (parent.i_vertical) {
        for (int retryCount = 0; retryCount < 2 && !pass; retryCount++) {

            double minUsed = LinearAxisUtil.calcMinUsed(parent, lmin, org);
            double maxUsed = LinearAxisUtil.calcMaxUsed(parent, lmax, org);

            lmin = minUsed;
            lmax = maxUsed;

            error = (0.5 * JCChartUtil.pow10(-parent.getPrecision() - 1));

            inc = parent.numSpacing.value;
            tik = parent.tickSpacing.value;

            // resize the axis as well
            int annoSize = calcMaxAnnoSize(lmin, lmax);

            // Determine tick and numbering spacings
            if (parent.numSpacing.isDefault && parent.tickSpacing.isDefault) {
                // Determine default tick and num spacing.  Give a couple of pixels
                // breathing room around each annotation to counter rounding errors
                // and annotations being placed too close to each other.
                numTicks = 0;
                if (parent.i_vertical) {
                    //  y axis of a Polar or Radar chart
                    if (useAnnoWidthForRAxisTickCalculation()) {
                        numTicks = (annoSize == 0) ? 0 :
                            (int)(parent.polarAxisParams.radius) / (annoSize + 2);
                    }
                    else {
                        //  Divide the y component of the r Axis by
                        //  the annotation height.
                        double rAxisAngle = JCChartUtil.boundAngle(
                            JCChartUtil.RADIANS,
                            parent.yAxisAngle +
                            parent.polarAxisParams.originBase);
                        double yVector = Math.sin(rAxisAngle) *
                            parent.polarAxisParams.radius;
                        numTicks = (annoSize == 0) ? 0 :
                            (int)Math.abs(yVector) / (annoSize + 2);
                    }
                }

                double sz = JCChartUtil.abs(maxUsed - minUsed);
                if (numTicks <= 0) {
                    tik = inc = 1.0;
                    //  Don't set to 0.0 -- Otherwise, will cause an
                    //  infinite recursion.
                }
                else {
                    double tmp = sz / numTicks;
                    DoubleCoord r =
                        JCChartUtil.calcTik(tmp, parent.getPrecision());
                    tik = r.x;
                    inc = r.y;
                }
            }
            else {
                // At least one of the two values has been given, so
                // calculate the other.  Note that NumSpacing is sensitive
                // to the precision, but tickSpacing is not.

                if (parent.tickSpacing.isDefault) {
                    inc = JCChartUtil.precCorrect(parent.getPrecision(), inc);
                    tik = parent.numSpacing.value / 2.0;
                }
                else if (parent.numSpacing.isDefault) {
                    inc = 2 * tik;
                    inc = JCChartUtil.precCorrect(parent.getPrecision(), inc);
                }
            }

            if (inc != 0) {
                lmin = LinearAxisUtil.updateMinValue(lmin, org, inc);
                lmax = LinearAxisUtil.updateMaxValue(lmax, org, inc);
                if (JCChartUtil.abs(lmax - lmin) < error) {
                    lmax = lmin + inc;
                }
            }
            else {
                if (JCChartUtil.abs(lmax - lmin) < error) {
                    lmax = lmin + 1;
                }
            }

            // Try this once ... if either of the axis min-max have
            // been moved, it might be too crowded to add these
            // extra labels, so try to determine these values
            // again.
            if ((JCChartUtil.abs(lmin - minUsed) > error) ||
                (JCChartUtil.abs(lmax - maxUsed) > error)) {
                pass = true;
                if ((lmax - inc) >= maxReal)
                    lmax -= inc;
                else
                    pass = false;
                if ((lmin + inc) <= minReal)
                    lmin += inc;
                else
                    pass = false;
            }
        }
    }
    else if (polarChart) {
        // For the theta axis of a Polar chart...

        // Start with an estimate of numTicks based on the
        // height of the annotations.
        int ascent = 13;
        FontMetrics fm = getFontMetrics();
        if (fm != null) {
            ascent = fm.getAscent();
        }
        double labelExtent = 4;
        JCAnno tickObj = parent.getDefaultLabels();
        if (tickObj != null && tickObj.getDrawLabels()) {
            labelExtent = tickObj.getLabelExtent();
        }
        double radius = parent.polarAxisParams.radius + labelExtent;
        double minAngleIncrement =
            Math.acos(1.0 - 0.3 * ascent / radius);  // Radians
        // Note that by using a factor 0.3 in the calculation
        // of minAngleIncrement, vertical overlap of annotations
        // may occur.  Without that factor, the tick spacing
        // appears unnecessarily large in many cases.  If
        // overlap does occur (for labels with large precision),
        // the checkValueAnnotations() method of the annotation
        // handler will drop labels.
        numTicks = 2 * (int)(JCChartUtil.M_2PI / minAngleIncrement);
        // Ensure that numTicks is greater than zero to avoid
        // a divide-by-zero exception below.
        numTicks = Math.max(1, numTicks);

        double tmp_val = 0;
        // If the angle unit is RADIANS, calculate a number of
        // tick marks that is an exact divisor of 64.  For
        // DEGREES, the number should be a divisor of 144.  For
        // GRADS, the number should be a divisor of 80.
        int n = 64;  // RADIANS
        if (parent.chartArea.getAngleUnit() == JCChartUtil.DEGREES) {
            n = 144;
        }
        else if (parent.chartArea.getAngleUnit() == JCChartUtil.GRADS) {
            n = 80;
        }
        while ((n % numTicks) != 0 && numTicks > 1) {
            numTicks--;
        }
        tmp_val = JCChartUtil.convertAngle(
                        JCChartUtil.RADIANS,
                        parent.chartArea.getAngleUnit(),
                        JCChartUtil.M_2PI) / numTicks;
        // Fix for PR 32130. Don't use sz which depends on
        // the precision.
        tik = tmp_val;
        inc = 2 * tik;
    }

    // For time labels and polar charts, horizontal precision is ignored
    if (!parent.i_vertical &&
        (parent.annotationMethod == JCAxis.TIME_LABELS)) {
        lmin = dlmin;
        lmax = dlmax;
        if (lmax <= lmin) {
            tprec = 0;
            do {
                lmax = (lmin + JCChartUtil.pow10(tprec));
                lmin -= lmax - lmin;
                tprec++;
            }
            while (lmax <= lmin);
        }
        minReal = lmin;
        maxReal = lmax;
    }

    //  For point_labels, if axis min/max use_default is true then do not
    //  use the precision corrected xmin or xmax.  We want it to default
    //  to the what it would have been when XRT/graph still had a
    //  discrete axis (i.e. a half_step lower/higher than the data
    //  min/max value where half_step = 0.5 * cg->xaxis_step).
    //
    if (parent.dataLimits != null &&
        (parent.annotationMethod == JCAxis.POINT_LABELS)) {
        double new_min1 = minOrig;
        double new_max1 = maxOrig;
        if (parent.min.isDefault) {
            new_min1 = dlmin - parent.step / 2.0;
        }
        if (parent.max.isDefault) {
            new_max1 = dlmax + parent.step / 2.0;
        }
        lmin = minReal = JCChartUtil.min(minOrig, new_min1);
        lmax = maxReal = JCChartUtil.max(maxOrig, new_max1);
    }

    if (!parent.i_vertical) {
        setDefaultThetaMinMax();
    }
    else {
        LinearAxisUtil.setAxisMinMax(parent, lmin, lmax, minReal, maxReal);
    }

    org = calcOrigin(parent.min.value, parent.max.value);
    if (parent.origin.isDefault) {
        parent.origin.value = org;
    }
    if (parent.tickSpacing.isDefault) {
        parent.tickSpacing.value = tik;
    }
    if (parent.numSpacing.isDefault) {
        parent.numSpacing.value = inc;
    }
    // Get the default JCGrid object.
    List gridList = parent.getGridList();
    if (gridList.size() > 0) {
        JCGrid gridObj = (JCGrid)gridList.get(0);
        if (gridObj != null && gridObj.getIncrementIsDefault()) {
            gridObj.incrementValue.value = parent.numSpacing.value;
        }
    }
}

private boolean useAnnoWidthForRAxisTickCalculation()
{
    // If the y axis angle is between -45 and +45 degrees, or between
    // 135 and 225 degrees, use the annotation width to calculate tick
    // spacing.  For all other y axis angles, use the annotation height
    // to calculate tick spacing.
    double angleRatio = JCChartUtil.getAngleRatio(JCChartUtil.RADIANS,
                                                  parent.yAxisAngle,
                                                  parent.polarAxisParams.originBase,
                                                  false);
    // An angle delta of 0.125 corresponds to 45 degrees.
    final double ANGLE_DELTA = 0.125;
    return ((angleRatio < ANGLE_DELTA) ||
        (angleRatio > 1.0 - ANGLE_DELTA) ||
        ((angleRatio > 0.5 - ANGLE_DELTA)
        && (angleRatio < 0.5 + ANGLE_DELTA)));
}

/**
 * Calculates the maximum annotation size
 */
private int calcMaxAnnoSize(double min, double max)
{
    int maxAnnoSize = -1;
    Dimension d = new Dimension();

    // Find widest string
    if (parent.getAnnotationRotation() == JCAxis.ROTATE_OTHER &&
        parent.getAnnotationMethod() != JCAxis.VALUE)
    {
        d = parent.getAnnotationHandler().getMaxRotatedAnnoDimensions();
    }
    else {
        getAnnoExtent(d, max, parent.getPrecision());
        getAnnoExtent(d, min, parent.getPrecision());
    }

    if (useAnnoWidthForRAxisTickCalculation()) {
        maxAnnoSize = d.width;
    }
    else {
        maxAnnoSize = d.height;
    }

    return(maxAnnoSize);
}

/**
 * Calculates the value of scale and shift given the axis position
 * and start point. scale and shift are used by toPixel() and toData().
 */
void calcTransformation()
{
    if (!parent.i_vertical) {
        // This method only applies to the r axis of a circular chart.
        return;
    }
    int length = (int)Math.rint(parent.polarAxisParams.radius);
    computeTransformation(parent.min.value,
                          parent.max.value,
                          length);
}

/**
 * Override the base class method to hide the maximum label on
 * the theta axis of a Polar chart.  Otherwise, 360 will overwrite
 * 0.
 */
public void makeTickLabels(JCAnno tick) {
    super.makeTickLabels(tick);
    PolarAxisParams polarAxisParams = parent.getPolarAxisParams();
    if (parent.isPolar() && !parent.i_vertical &&
        polarAxisParams != null && !polarAxisParams.isPolarHalfRange())
    {
        // The parent is the theta axis of a Polar chart and the
        // fHalfRange flag is not set.
        List valueLabels = parent.getAnnotations();
        int maxLabelIndex =
                parent.getAnnotationHandler().getMaxVisibleLabel();
        if (maxLabelIndex >= 0 && maxLabelIndex < valueLabels.size()) {
            JCValueLabel maxVisibleLabel =
                    (JCValueLabel)valueLabels.get(maxLabelIndex);
            double radianValue =
                JCChartUtil.convertAngle(parent.chartArea.getAngleUnit(),
                                JCChartUtil.RADIANS, maxVisibleLabel.value);
            // Need to deal with round-off error:
            radianValue = JCChartUtil.precCorrect(5, radianValue);
            double M_2PI = JCChartUtil.precCorrect(5, JCChartUtil.M_2PI);
            maxVisibleLabel.label.visible = (radianValue != M_2PI);
        }
    }
}

}
