/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CircularChartLogAxisHandler.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:45 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.awt.Dimension;
import java.awt.Point;

/**
 * Handler for a logarithmic y axis on a circular chart (Polar or Radar).
 */
class CircularChartLogAxisHandler extends CircularChartAxisHandler
                                  implements LogAxisHandler {

CircularChartLogAxisHandler()
{
    axisScaleHandler = new LogAxisScaleHandler();
}

protected double getScaledValue(double value)
{
    double scaledValue = 0.0;
    if (parent.isReversed()) {
        scaledValue = scale * (-JCChartUtil.log10(value) +
            JCChartUtil.log10(parent.max.value));
    }
    else {
        scaledValue = scale * (JCChartUtil.log10(value) -
            JCChartUtil.log10(parent.min.value));
    }
    return scaledValue;
}

/**
 * Performs pixel-to-data conversion for a Polar or Radar chart
 * with logarithmic r axis.
 *
 * @param x chart area x coordinate of pixel
 * @param y chart area y coordinate of pixel
 * @return Data space coordinates of this pixel.  JCDataCoord.x
 * is the polar angle value, and JCDataCoord.y is the radius.
 */
protected JCDataCoord toDataPolar(int x, int y)
{
    double dx = (double)x - parent.polarAxisParams.xCenter;
    double dy = (double)y - parent.polarAxisParams.yCenter;
    JCAxis rAxis = getRadialAxis();
    if (rAxis == null) {
        return null;
    }

    double pixelDistance = Math.sqrt(dx * dx + dy * dy);
    double logVal = Double.MAX_VALUE;
    if (rAxis.isReversed()) {
        logVal = JCChartUtil.log10(rAxis.max.value) - (pixelDistance / -scale);
    }
    else {
        logVal = (pixelDistance / scale) + JCChartUtil.log10(rAxis.min.value);
//      System.out.println("logVal=" + logVal + ", pixelDistance=" +
//                      pixelDistance + ", scale=" + scale +
//                      ", log10(min)=" + JCChartUtil.log10(rAxis.min.value));
    }

    double r = Math.pow(10.0, logVal);
    double theta = calcThetaAngle(dx, dy);
    return (new JCDataCoord(theta, r));
}

/*
* Returns the chart area pixel location of a point at polar
* coordinates (theta,r).
*
* @param theta angle in radians.
* @param r radius in data units or pixel units
* @param convertDataToPixel true if radius is in data units,
* false if radius is in pixel units.
* @return pixel location.
*/
protected Point toPixelPolar(double theta, double r,
                             boolean convertDataToPixel)
{
    if (convertDataToPixel) {
        //  Need to subtract axis minimum from r.
        JCAxis rAxis = (parent.i_vertical) ? parent : parent.otherAxis;
        if (rAxis.isReversed()) {
            if ((r > 0.0) && (rAxis.max.value > 0.0)) {
                r = JCChartUtil.log10(rAxis.max.value)
                    - JCChartUtil.log10(r);
            }
            else {
                r = 0.0;
            }
        }
        else {
            if ((r > 0.0) && (rAxis.min.value > 0.0)) {
                r = JCChartUtil.log10(r)
                    - JCChartUtil.log10(rAxis.min.value);
            }
            else {
                r = 0.0;
            }
        }
    }

    JCAxis thetaAxis = (parent.i_vertical) ? parent.otherAxis : parent;
    if (thetaAxis.isReversed()) {
        theta *= -1.0;
    }

    JCDataCoord screenLocation =
        toScreenLocationPolar(theta, r, convertDataToPixel);

    double value = screenLocation.getX();
    int iX = (value < parent.polarAxisParams.xCenter) ?
        (int)Math.ceil(value) : (int)Math.round(value);

    value = screenLocation.getY();
    int iY = (value < parent.polarAxisParams.yCenter) ?
        (int)Math.ceil(value) : (int)Math.round(value);

    return (new Point(iX, iY));
}

protected void calcParams()
{
    //  If this is the theta axis of a Polar/Radar chart, return
    //  immediately.
    if (!parent.i_vertical) {
        return;
    }

    double lmin = (parent.dataLimits == null) ? 1.0 : parent.dataLimits.min;
    double lmax = (parent.dataLimits == null) ? 100.0 : parent.dataLimits.max;
    int startMin = 0;
    int startMax = 0;
    int logMin = 0;
    int logMax = 0;
    int logOrg = 0;
    int logInc = 0;

    lmin = parent.min.isDefault ? lmin : parent.min.value;
    lmax = parent.max.isDefault ? lmax : parent.max.value;

    final int MAX_PRECISION = 10;
    startMin = JCChartUtil.ifloor(
        JCChartUtil.precCorrect(MAX_PRECISION, JCChartUtil.log10(lmin)));
    startMax = JCChartUtil.iceil(JCChartUtil.log10(lmax));

    do {
        calcPrecision();
    }
    while (axisScaleHandler.fixBounds());

    if (parent.i_vertical) {
        if (!parent.origin.isDefault && parent.origin.value > 0.0) {
            logOrg = JCChartUtil.ifloor(JCChartUtil.log10(parent.origin.value));
            if (logOrg < startMin && parent.min.isDefault) {
                startMin = logOrg;
            }
            if (logOrg > startMax && parent.max.isDefault) {
                startMax = logOrg;
            }
        }
    }
    else {
        if (!parent.origin.isDefault && parent.origin.value > 0.0) {
            logOrg = JCChartUtil.ifloor(JCChartUtil.log10(parent.origin.value));
            if (logOrg < startMin && parent.min.isDefault) {
                startMin = logOrg;
            }
            if (logOrg > startMax && parent.max.isDefault) {
                startMax = logOrg;
            }
        }
    }

    int annoSize = 0;
    Dimension d = new Dimension();
    int pixelLength = (int)Math.round(parent.polarAxisParams.radius);
    for (logInc = 1; logInc <= 5; logInc++) {
        logMin = startMin;
        logMax = startMax;
        boolean on_top = (JCChartUtil.abs(logMax) > JCChartUtil.abs(logMin));
        while ((logMax - logMin) % logInc != 0) {
            if (on_top)
                logMax++;
            else
                logMin--;
            on_top = !on_top;
        }
        if (parent.annotationRotation == JCAxis.ROTATE_OTHER) {
            annoSize = calcRotatedAnnoSpacing((double)logMin, (double)logMax);
        }
        else {
            getAnnoExtent(d, (double)logMax, parent.getPrecision());
            getAnnoExtent(d, (double)logMin, parent.getPrecision());
            if (parent.i_vertical) {
                annoSize = (parent.annotationRotation == JCAxis.ROTATE_NONE) ?
                    d.height * 2 : d.width + d.width / 2;
            }
            else {
                annoSize = (parent.annotationRotation == JCAxis.ROTATE_NONE) ?
                    d.width + d.width / 2 : d.height * 2;
            }
        }
        if ((logMax - logMin) / logInc * annoSize < pixelLength) {
            break;
        }
    }
    if (logInc <= 5) {
        // Found a divisor
        parent.tickSpacing.value = JCChartUtil.pow10(logInc);
    }
    else {
        // No divisor.  Increment is the entire range
        logMin = startMin;
        logMax = startMax;
        int diff = Math.max(logMax - logMin, 1);
        parent.tickSpacing.value = JCChartUtil.pow10(diff);
    }
    parent.numSpacing.value = parent.tickSpacing.value;

    if (parent.min.isDefault)
        parent.min.value = JCChartUtil.pow10(logMin);
    if (parent.max.isDefault)
        parent.max.value = JCChartUtil.pow10(logMax);

    if (parent.i_vertical) {
        parent.origin.value = parent.max.value;
    }

    if (parent.i_vertical) {
        parent.setSize(annoSize + 1 + parent.tickLength / 2, parent.getHeight());
    }
    else {
        parent.setSize(parent.getWidth(), d.height + 1 + parent.tickLength / 2);
    }
}

/**
 * Calculates the value of scale and shift given the axis position
 * and start point. scale and shift are used by toPixel() and toData().
 */
void calcTransformation()
{
    if (!parent.i_vertical) {
        // This method only applies to the r axis of a circular chart.
        return;
    }
    int length = (int)Math.rint(parent.polarAxisParams.radius);
    computeTransformation(JCChartUtil.log10(parent.min.value),
                          JCChartUtil.log10(parent.max.value),
                          length);
}

}
