/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DepthHandler.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:45 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.awt.Point;

/**
 * Handler object for depth change actions within JCChart.  When
 * an septh action is started, JCChartArea will create an
 * instance of DepthHandler and call start(), animate() and
 * end() at appropriate points.  The mapping between key
 * combinations and JCChart events is specified by EventTrigger.
 * @see com.klg.jclass.chart.ActionHandler
 * @see com.klg.jclass.chart.EventTrigger
 */
public class DepthHandler extends ActionHandler {

/** should we be doing a depth change at all? */
protected boolean doDepthChange = false;

/** Point in chart area at which the depth change action was started */
protected Point first = null;

/** Last point used for depth change */
protected Point last = null;

/** Original depth value */
protected int orig_depth;

/** True if the chart being changed is a pie chart */
protected boolean pieChart = false;

/**
 * Default constructor
 */
protected DepthHandler() {}

/**
 * Called by JCChartArea when the action is started.  For
 * the depth change action, this method sets a series of initial
 * values.
 */
protected void start(int x, int y)
{
    parent.setInAction(true);
    first = new Point(x, y);
    last = null;

    if (parent == null || getChart() == null ||
        getChart().getChartArea() == null)
        return;
    if (!getChart().getChartArea().getHasBar()) {
        doDepthChange = false;
    }
    else {
        doDepthChange = true;
    }

    orig_depth = parent.getDepth();
    pieChart = (getChart().getDataView(0).getChartType() == JCChart.PIE);
}

/**
 * Called by JCChartArea when the action is to be
 * animated.  For the depth change action, this method calculates
 * new depth values for the chart area.
 */
protected void animate(int x, int y)
{
    if (doDepthChange == false) {
        return;
    }

    if (first == null) {
        return;
    }
    if (last == null) {
        last = new Point(x, y);
        return;
    }
    if ((y == last.y) && (x == last.x)) {
        return;
    }

    // calculate new depth based on distance pointer
    // moved relative to the height of chartArea

    int dy = y - first.y;

    int new_depth = orig_depth;
    if (pieChart) {
        new_depth -= dy;
    }
    else {
        new_depth += dy;
    }
    new_depth = (int)JCChartUtil.clamp((double)new_depth, 0.0,
                                       (double)JCChartArea.MAX_DEPTH);

    last.x = x;
    last.y = y;

    // if depth has changed, set and redraw
    if (new_depth != parent.getDepth()) {
        // setDepth now calls setChanged
        parent.setDepth(new_depth);
    }
}

/**
 * Called by JCChartArea when the chart area has been
 * refreshed and the animation must be redrawn.
 */
protected void reanimate(int x, int y) {}

/**
 * Called by JCChartArea when the action is over.  Does nothing
 * for the depth change action.
 */
protected void end(int x, int y)
{
    parent.setInAction(false);
    JCChart chart = getChart();
    if (chart != null) {
        chart.sendEvent(null);
    }
    if (parent.getFastAction()) {
        parent.setChanged(true, Changeable.RECALC);
    }
}

}
