/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ErrorDialog.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:45 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Locale;

/**
 * Error dialog object used inside JCChart to show an
 * internal error to the user.  Better than System.out.println,
 * but not much better - yet.
 */
class ErrorDialog extends JDialog implements ActionListener {

/** Global ErrorDialog for JCChart. */
static ErrorDialog dialog;

/** Label that contains the error text */
protected JTextArea text;

/** OK button that dismisses the dialog */
protected JButton ok;

/**
 * The default and only constructor.  Creates the
 * visual objects inside the dialog - label for the
 * text, and a button to dismiss the dialog.
 */
ErrorDialog()
{
    super();
    getContentPane().setLayout(new BorderLayout());
    text = new JTextArea();
    getContentPane().add("Center", text);
    JPanel footer = new JPanel();
    getContentPane().add("South", footer);
    footer.setLayout(new FlowLayout());
    ok = new JButton(JCChartBundle.string(JCChartBundle.key1,
                                          Locale.getDefault()));
    footer.add(ok);
    ok.addActionListener(this);
}

/**
 * Handle locale change by redoing the text on the ok button.
 */
public void setLocalizedText(Locale locale)
{
    if (ok != null) {
        ok.setText(JCChartBundle.string(JCChartBundle.key1, locale));
    }
}

/**
 * Get the dialog.  If it hasn't been created yet, do so.
 */
static ErrorDialog getDialog()
{
    if (dialog == null) {
        dialog = new ErrorDialog();
    }
    return(dialog);
}

/**
 * Static method used to show an error using the global
 * error dialog.  If the global dialog does not exist, it
 * is created.
 * @param s Error text to be displayed inside the dialog.
 */
static void raise(String s)
{
    raise(s, Locale.getDefault());
}

/**
 * Static method used to show an error using the global
 * error dialog.  If the global dialog does not exist, it is created.
 * The specified locale is used for dialog-specific text,
 * such as the ok button.
 * @param s Error text to be displayed inside the dialog.
 */
static void raise(String s, Locale locale)
{
    try {
        if (dialog == null) {
            dialog = new ErrorDialog();
        }
        dialog.setLocalizedText(locale);
        dialog.text.setText(s);
        dialog.text.invalidate();
        dialog.invalidate();
        dialog.validate();
        dialog.pack();
        dialog.setVisible(true);
        dialog.toFront();
    }
    catch (Exception e) {
        System.err.println(s);
    }
}

/**
 * Static method used to show an error using the global
 * error dialog.  If the global dialog does not exist, it
 * is created.
 * The locale used for dialog-specific text, such as the ok button,
 * is retrieved from the passed component.
 * @param comp The instance the error occurred in (Colors imported)
 * @param s Error text to be displayed inside the dialog.
 */
static void raise(Component comp, String s)
{
    if (comp instanceof JCChart) {
        if (!((JCChart)comp).isWarningDialog()) {
            return;
        }
    }

    if (dialog == null) {
        dialog = new ErrorDialog();
    }

    Locale locale = Locale.getDefault();
    if (comp != null) {
        locale = comp.getLocale();
        dialog.setBackground(comp.getBackground());
        dialog.setForeground(comp.getForeground());
    }
    raise(s, locale);
}

/**
 * Static method used to show an error using the global
 * error dialog.  If the global dialog does not exist, it
 * is created.
 * The locale used for dialog-specific text, such as the ok button,
 * is retrieved from the passed axis.
 * @param comp The instance the error occurred in (Colors imported)
 * @param s Error text to be displayed inside the dialog.
 */
static void raise(JCAxis comp, String s)
{
    JCChart chart = (comp != null) ? comp.getParentChart() : null;
    if (chart != null) {
        if (!chart.isWarningDialog()) {
            return;
        }
    }

    if (dialog == null) {
        dialog = new ErrorDialog();
    }

    Locale locale = Locale.getDefault();
    if (comp != null) {
        locale = comp.locale;
        dialog.setBackground(comp.getBackground());
        dialog.setForeground(comp.getForeground());
    }
    raise(s, locale);
}

/**
 * Static method used to show an error using the global
 * error dialog.  If the global dialog does not exist, it
 * is created.
 * The locale used for dialog-specific text, such as the ok button,
 * is retrieved from the passed object if the object is a component.
 * @param obj The instance the error occurred in (Colors imported)
 * @param s Error text to be displayed inside the dialog.
 */
static void raise(Object obj, String s)
{
    raise(obj, s, Locale.getDefault());
}

/**
 * Static method used to show an error using the global
 * error dialog.  If the global dialog does not exist, it
 * is created.
 * The locale used for dialog-specific text, such as the ok button,
 * is retrieved from the passed object if the object is a component
 * or from the passed locale otherwise.
 * @param obj The instance the error occurred in (Colors imported)
 * @param s Error text to be displayed inside the dialog.
 */
static void raise(Object obj, String s, Locale locale)
{
    if (obj instanceof Component) {
        raise((Component)obj, s);
    }
    else {
        raise(s, locale);
    }
}

/**
 * Static method used to show an error using the global
 * error dialog.  If the global dialog does not exist, it
 * is created.
 * @param chart The instance the error occurred in (Colors imported)
 * @param s Error text to be displayed inside the dialog.
 * @param e The Exception the occured
 */

/*
 * Still broken
static void showException(JCChart chart, String s, Exception e)
{
    if (dialog == null) {
        dialog = new ErrorDialog();
        dialog.setBackground(chart.getBackground());
        dialog.setForeground(chart.getForeground());
    }
    if (dialog.text == null) {
        dialog.text = new JTextArea();
        add("Center", dialog.text);
    }
    dialog.text.setText(s);
    dialog.text.setText(e.printStackTrace());
    dialog.text.showPosition(0);
    dialog.text.invalidate();
    dialog.text.invalidate();
    dialog.invalidate();
    dialog.validate();
    dialog.pack();
    dialog.setVisible(true);
    dialog.toFront();
}
*/

/**
 * Action event for the error dialog - makes the dialog
 * go away.
 */
public void actionPerformed(ActionEvent evt)
{
    setVisible(false);
}

}
