/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: FastHLOCChartDraw.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:45 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;

import java.util.List;
import java.awt.geom.Line2D;
import java.awt.Graphics2D;

import com.klg.jclass.util.graphics.DefineShape;

/**
 * Drawable object for Hi-Lo-Open-Close charts.  This object
 * is used for rendering a Hi-Lo-Open-Close chart based on data
 * contained in the <code>dataObject</code>.
 */
public class FastHLOCChartDraw extends ChartDraw {

// number of chart data view series per HLOC data series
public final static int NUM_SERIES_PER_DATA = 4;
public final static int HI_SERIES_INDEX = 0;
public final static int LO_SERIES_INDEX = 1;
public final static int OPEN_SERIES_INDEX = 2;
public final static int CLOSE_SERIES_INDEX = 3;

// Local copy of data used by draw().  Set by recalc().
protected JCHLOCChartFormat hlocf = null;

protected MinMax xlim;
protected MinMax ylim;

/**
 * Default constructor.
 */
public FastHLOCChartDraw() {
	super();
	xlim = new MinMax();
	ylim = new MinMax();
}

/**
 * Method used to recalculate extents of related objects.
 */
public void recalc() {
	recalc(-1);
}

public void recalc(int seriesIndex) {
	super.recalc();

	// Make a list of points
	// If calling for a specific series, use existing series list.
	// Otherwise, refresh series list.
	if (seriesIndex == -1) {
		seriesList = dataObject.getSeries();
	}
	if (seriesList == null) return;

	hlocf = (JCHLOCChartFormat) dataObject.getChartFormat();
	xaxis = dataObject.getXAxis();
	yaxis = dataObject.getYAxis();
	holeValue = dataObject.getHoleValue();
	inverted = xaxis.i_vertical;

	int numSeries = seriesList.size() / NUM_SERIES_PER_DATA;

	if (seriesIndex == (-1) || seriesIndex == 0) {
		// This may take awhile for large data set
		if (dataObject.getXLimits() == null) {
			return;
		}
		if (dataObject.getYLimits() == null) {
			return;
		}
		xlim = new MinMax(dataObject.getXLimits().min,
						  dataObject.getXLimits().max);
		ylim = new MinMax(dataObject.getYLimits().min,
						  dataObject.getYLimits().max);
		dataObject.getDataBounds(xlim, ylim);
	}
}

/**
 * Method used to draw related objects.
 * @param gc
 */
public void draw(Graphics gc) {

	try {
	if (seriesList == null) return;

   	Rectangle clipBounds = gc.getClipBounds();
   	if (clipBounds.width == 0 || clipBounds.height == 0)
       	return;


	boolean fullTickWidth = hlocf.isOpenCloseFullWidth();
	boolean showOpen = hlocf.isShowingOpen();
	boolean showClose = hlocf.isShowingClose();

	boolean xLog = xaxis.isLogarithmic();
	boolean yLog = yaxis.isLogarithmic();

	// Setup parameters to loop thru the data series
	int numSeries = seriesList.size() / NUM_SERIES_PER_DATA;
	int firstSeries = 0;
	int lastSeries = numSeries;
	for (int i = firstSeries; i < lastSeries; i++) {

		int tickSize = hlocf.getTickSize(i);
		boolean hasData = true;
		boolean showSeries = true;

		int firstPoint = Integer.MAX_VALUE;
		int lastPoint = Integer.MAX_VALUE;
		for (int s = NUM_SERIES_PER_DATA * i, count = 0;
				count < NUM_SERIES_PER_DATA; count++, s++) {
			ChartDataViewSeries series =
				(ChartDataViewSeries) seriesList.get(s);

			if (!series.isVisible()) {
				showSeries = false;
				break;
			}
			// Drop out if data is uninteresting
			if (series.maxXIndex() == 0 || series.maxYIndex() == 0) {
				hasData = false;
				break;
			}

			// Determine first and last point in data series
			firstPoint = Math.min(firstPoint, series.getFirstPoint());
			lastPoint = Math.min(lastPoint, series.getLastPoint());
		}
		if (!hasData) continue;
		if (!showSeries) continue;

		JCChartStyle style = ((ChartDataViewSeries) seriesList.get(
									NUM_SERIES_PER_DATA * i)).getStyle();
		if (style == null) continue;
		// Set up the line style
		JCLineStyle lstyle = style.getLineStyle();
		if (!(lstyle.updateGraphics(gc)))
			continue;

		// Create a series of PlotPoint arrays holding the various lines that
		// make up this plot chart.
		PlotPoint newpts[] = new PlotPoint[2];
		newpts[0] = new PlotPoint();
		newpts[1] = new PlotPoint();

		for (int j = firstPoint; j <= lastPoint; j++) {
			drawPoints(gc, seriesList, i, j,
				newpts, lstyle,
				fullTickWidth, showOpen, showClose, tickSize,
				xLog, yLog);

		}
		lstyle.resetGraphics(gc);
	}

	}
	catch (Exception e) {
		e.printStackTrace(System.out);
	}
}

// Had to move this out of draw() because it crashes the jdk 1.1.6 virtual
// machine
final private void drawPoints(Graphics gc,
	List seriesList, int i, int j,
	PlotPoint newpts[], JCLineStyle lstyle,
	boolean fullTickWidth, boolean showOpen, boolean showClose, int tickSize,
	boolean xLog, boolean yLog)
{

	Graphics2D gc2D = (Graphics2D) gc;

    DefineShape defShape = null;
    if (gc instanceof DefineShape) {
        defShape = (DefineShape) gc;
    }
	int hlocX = Integer.MAX_VALUE;
	int highY = Integer.MAX_VALUE;
	int lowY = Integer.MAX_VALUE;

			// Do the Hilo points:
			int s = NUM_SERIES_PER_DATA * i;
			ChartDataViewSeries highSeries =
				(ChartDataViewSeries) seriesList.get(s++);

			double x1 = highSeries.getX(j);
			if (xLog && x1 <= 0.0)
				x1 = holeValue;
			double y1 = highSeries.getY(j);
			if (yLog && y1 <= 0.0)
				y1 = holeValue;

			ChartDataViewSeries lowSeries =
				(ChartDataViewSeries) seriesList.get(s++);
			double x2 = lowSeries.getX(j);
			if (xLog && x2 <= 0.0)
				x2 = holeValue;
			double y2 = lowSeries.getY(j);
			if (yLog && y2 <= 0.0)
				y2 = holeValue;

			newpts[0].valid = false;
			newpts[1].valid = false;
			financialClip(x1, y1, x2, y2, xlim, ylim,
						newpts,
						inverted, false);
			if (newpts[0].valid && newpts[1].valid) {



				gc2D.draw(new Line2D.Double(
						Math.floor(newpts[0].px), Math.floor(newpts[0].py),
						Math.floor(newpts[1].px), Math.floor(newpts[1].py)));

				// if using DefineShape, keep track of high/low info
				if (defShape != null) {
					if (!inverted) {
						hlocX = (int) newpts[0].px;
						highY = (int) newpts[0].py;
						lowY = (int) newpts[1].py;
					} else {
						hlocX = (int) newpts[0].py;
						highY = (int) newpts[0].px;
						lowY = (int) newpts[1].px;
					}
					if (!newpts[0].drawPoint) {
						highSeries = null;
					}
					if (!newpts[1].drawPoint) {
						lowSeries = null;
					}
				}
			}

			ChartDataViewSeries openSeries = null;
			int openY = Integer.MAX_VALUE;
			// Do the Open points.  Don't bother calcing if not showing or
			// if tick size is 0
			if (tickSize > 0 && showOpen) {
				openSeries =
					(ChartDataViewSeries) seriesList.get(s++);
				double x3 = openSeries.getX(j);
				if (xLog && x3 <= 0.0)
					x3 = holeValue;
				double y3 = openSeries.getY(j);
				if (yLog && y3 <= 0.0)
					y3 = holeValue;
				if (x3 != holeValue && y3 != holeValue) {
					int point_x = xaxis.toPixel(x3, drawFront);

					x1 = xaxis.toData(point_x - tickSize, drawFront);
					y1 = y3;

					x2 = fullTickWidth ?
						xaxis.toData(point_x + tickSize, drawFront) : x3;
					y2 = y3;

					newpts[0].valid = false;
					newpts[1].valid = false;
					financialClip(x1, y1, x2, y2, xlim, ylim,
							newpts,
							inverted, false);
					if (newpts[0].valid && newpts[1].valid) {



						gc2D.draw(new Line2D.Double(
							Math.floor(newpts[0].px), Math.floor(newpts[0].py),
							Math.floor(newpts[1].px), Math.floor(newpts[1].py)));

						// if using DefineShape, keep track of open info
						if (defShape != null) {
							if (!inverted) {
								openY = (int) newpts[0].py;
							} else {
								openY = (int) newpts[0].px;
							}
							if (!newpts[0].drawPoint && !newpts[1].drawPoint) {
								openSeries = null;
							}
						}
					}
				}
			} else {
				s++;
			}

			ChartDataViewSeries closeSeries = null;
			int closeY = Integer.MAX_VALUE;
			// Do the Close points.  Don't bother calcing if not showing or
			// if tick size is 0
			if (tickSize > 0 && showClose) {

				closeSeries =
					(ChartDataViewSeries) seriesList.get(s++);
				double x3 = closeSeries.getX(j);
				if (xLog && x3 <= 0.0)
					x3 = holeValue;
				double y3 = closeSeries.getY(j);
				if (yLog && y3 <= 0.0)
					y3 = holeValue;

				if (x3 != holeValue && y3 != holeValue) {
					int point_x = xaxis.toPixel(x3, drawFront);

					x1 = fullTickWidth ?
						xaxis.toData(point_x - tickSize, drawFront) : x3;
					y1 = y3;

					x2 = xaxis.toData(point_x + tickSize, drawFront);
					y2 = y3;

					newpts[0].valid = false;
					newpts[1].valid = false;
					financialClip(x1, y1, x2, y2, xlim, ylim,
							newpts,
							inverted, false);
					if (newpts[0].valid && newpts[1].valid) {



						gc2D.draw(new Line2D.Double(
							Math.floor(newpts[0].px), Math.floor(newpts[0].py),
							Math.floor(newpts[1].px), Math.floor(newpts[1].py)));

						// if using DefineShape, keep track of close info
						if (defShape != null) {
							if (!inverted) {
								closeY = (int) newpts[0].py;
							} else {
								closeY = (int) newpts[0].px;
							}
							if (!newpts[0].drawPoint && !newpts[1].drawPoint) {
								closeSeries = null;
							}
						}
					}
				}

			} else {
				s++;
			}

			// is using DefineShape, associate HLOC shape with data indices
			if (defShape != null) {
				defineHLOCShapes(defShape, gc, inverted, j,
					highSeries, lowSeries, openSeries, closeSeries,
					highY, lowY, openY, closeY,
					hlocX, tickSize);
			}
}

/**
 * Associate shapes covering areas of the overall Hi-Lo-Open-Close shape with
 * the data indices they represent.
 * @param closeSeries
 * @param closeY
 * @param defShape
 * @param gc
 * @param highSeries
 * @param highY
 * @param hlocX
 * @param inverted
 * @param j
 * @param lowSeries
 * @param lowY
 * @param openSeries
 * @param openY
 * @param tickWidth
 */
protected void defineHLOCShapes(DefineShape defShape, Graphics gc,
    boolean inverted, int j,
	ChartDataViewSeries highSeries, ChartDataViewSeries lowSeries,
	ChartDataViewSeries openSeries, ChartDataViewSeries closeSeries,
	int highY, int lowY, int openY, int closeY, int hlocX, int tickWidth)
{
	if (tickWidth < 5) {
		tickWidth = 5;
	}

	// first, arrange high, low, open and close pixel values
	// in order smallest to biggest
	ChartDataViewSeries hlocSeries[] = new ChartDataViewSeries[4];
	int hlocY[] = new int[4];
	for (int i = 0; i < 4; i++) {
		int tmpY = Integer.MAX_VALUE;
		int pick = 0;
		if (highY != Integer.MAX_VALUE && highY < tmpY) {
			tmpY = highY;
			pick = 1;
		}
		if (lowY != Integer.MAX_VALUE && lowY < tmpY) {
			tmpY = lowY;
			pick = 2;
		}
		if (openY != Integer.MAX_VALUE && openY < tmpY) {
			tmpY = openY;
			pick = 3;
		}
		if (closeY != Integer.MAX_VALUE && closeY < tmpY) {
			tmpY = closeY;
			pick = 4;
		}

		hlocY[i] = tmpY;
		if (pick == 1) {
			hlocSeries[i] = highSeries;
			highY = Integer.MAX_VALUE;
		} else if (pick == 2) {
			hlocSeries[i] = lowSeries;
			lowY = Integer.MAX_VALUE;
		} else if (pick == 3) {
			hlocSeries[i] = openSeries;
			openY = Integer.MAX_VALUE;
		} else if (pick == 4) {
			hlocSeries[i] = closeSeries;
			closeY = Integer.MAX_VALUE;
		}
	}

	// now loop through list, creating shapes for area from
	// top to halfway to next point, then from that last point
	// to halfway to next point and so on until we reach the
	// end of the list.  The end of the list is defined either
	// by the length of the array or by an Integer.MAX_VALUE
	// value being encountered.
	int startY = Integer.MAX_VALUE;
	int endY = Integer.MAX_VALUE;
	int thisY = Integer.MAX_VALUE;
	int nextY = Integer.MAX_VALUE;
	int i = 0;
	while (i < hlocY.length && hlocY[i] != Integer.MAX_VALUE) {
		thisY = hlocY[i];
		if (i == 0) {
			startY = thisY;
		} else {
			startY = endY;
		}
		if (i + 1 < hlocY.length && hlocY[i+1] != Integer.MAX_VALUE) {
			nextY = hlocY[i+1];
		} else {
			nextY = hlocY[i];
		}

		if (i + 1 < hlocY.length && hlocSeries[i+1] != null) {
			endY = thisY + ((nextY - thisY) / 2);
		} else {
			endY = nextY;
		}

		if (hlocSeries[i] != null) {
			if (endY > startY) {
   				defShape.beginShape(new JCDataIndex(j, hlocSeries[i]));
	    		if (!inverted) {
            		gc.fillRect(
               			hlocX - tickWidth, startY,
               			(tickWidth * 2) + 1, endY - startY);
				} else {
            		gc.fillRect(
               			startY, hlocX - tickWidth,
               			endY - startY, (tickWidth * 2) + 1);
				}
       			defShape.endShape(false);
			}
		} else {
			endY = thisY;
		}

		i++;
	}
}

/**
 * Pick method for Hi-Lo-Open-Close charts.  Given a point relative to the
 * chart area, this method finds the closest point.
 * @param pt (x,y) value relative to the chart area
 * @param focus pick focus used to determine distance between pick point
 * and data point
 * @return <code>JCDataIndex</code> encapsulating the point index and data
 * series of the selected point
 */
public JCDataIndex pick(Point pt, int focus) {
	Rectangle r = chartArea.getDrawingArea();
    int px = pt.x - r.x;
    int py = pt.y - r.y;
	int series = -1;
	int point = -1;
	JCDataIndex di = null;
	double length = Double.MAX_VALUE;
	if (seriesList == null) return di;

	double xmax = xlim.max;
	double xmin = xlim.min;
	double ymax = ylim.max;
	double ymin = ylim.min;

	boolean xLog = xaxis.isLogarithmic();
	boolean yLog = yaxis.isLogarithmic();

	boolean showOpen = hlocf.isShowingOpen();
	boolean showClose = hlocf.isShowingClose();
	boolean fullTickWidth = hlocf.isOpenCloseFullWidth();

	int tickMod;
	if ((!xaxis.i_vertical && !xaxis.reversed) ||
		(xaxis.i_vertical && xaxis.reversed))
		tickMod = -1;
	else
		tickMod = 1;

	double holeValue = dataObject.getHoleValue();

	int numSeries = seriesList.size() / NUM_SERIES_PER_DATA;
	int firstSeries = 0;
	int lastSeries = numSeries;
	int pix_x, pix_y;
	try {
	for (int i = firstSeries; i < lastSeries; i++) {

		int tickWidth = hlocf.getTickSize(i);
		double tickSize = (tickWidth == 0) ? 0.0 :
			Math.abs(xaxis.toData(0) - xaxis.toData(tickWidth));

		boolean hasData = true;
		boolean showSeries = true;
		int firstPoint = Integer.MAX_VALUE;
		int lastPoint = Integer.MAX_VALUE;
		for (int s = NUM_SERIES_PER_DATA * i, count = 0;
				count < NUM_SERIES_PER_DATA; count++, s++) {
			ChartDataViewSeries thisSeries =
				(ChartDataViewSeries) seriesList.get(s);

			if (!thisSeries.isVisible()) {
				showSeries = false;
				break;
			}
			// Drop out if data is uninteresting
			if (thisSeries.maxXIndex() == 0 || thisSeries.maxYIndex() == 0) {
				hasData = false;
				break;
			}

			// Determine first and last point in data series
			firstPoint = Math.min(firstPoint, thisSeries.getFirstPoint());
			lastPoint = Math.min(lastPoint, thisSeries.getLastPoint());

		}
		if (!hasData) continue;
		if (!showSeries) continue;

		int s = NUM_SERIES_PER_DATA * i;

		double highx[] =
			((ChartDataViewSeries) seriesList.get(s+HI_SERIES_INDEX)).getX();
		double highy[] =
			((ChartDataViewSeries) seriesList.get(s+HI_SERIES_INDEX)).getY();
		double lowx[] =
			((ChartDataViewSeries) seriesList.get(s+LO_SERIES_INDEX)).getX();
		double lowy[] =
			((ChartDataViewSeries) seriesList.get(s+LO_SERIES_INDEX)).getY();
		double openx[] =
			((ChartDataViewSeries) seriesList.get(s+OPEN_SERIES_INDEX)).getX();
		double openy[] =
			((ChartDataViewSeries) seriesList.get(s+OPEN_SERIES_INDEX)).getY();
		double closex[] =
			((ChartDataViewSeries) seriesList.get(s+CLOSE_SERIES_INDEX)).getX();
		double closey[] =
			((ChartDataViewSeries) seriesList.get(s+CLOSE_SERIES_INDEX)).getY();

		double thispx = 0.0;
		double thispy = 0.0;
		for (int j = firstPoint; j <= lastPoint; j++) {

			boolean other = false;
			boolean hasHiLo = !(highx[j] == holeValue ||
						highy[j] == holeValue || lowx[j] == holeValue ||
						lowy[j] == holeValue ||
						(xLog && highx[j] <= 0.0) ||
						(yLog && highy[j] <= 0.0) ||
						(xLog && lowx[j] <= 0.0) ||
						(yLog && lowy[j] <= 0.0)
						);
			boolean hasOpen = !(openx[j] == holeValue ||
									openy[j] == holeValue ||
									(xLog && openx[j] <= 0.0) ||
									(yLog && openy[j] <= 0.0));
			boolean hasClose = !(closex[j] == holeValue ||
									closey[j] == holeValue ||
									(xLog && closex[j] <= 0.0) ||
									(yLog && closey[j] <= 0.0));
			for (int k = hasHiLo ? 0 : OPEN_SERIES_INDEX;
											k < NUM_SERIES_PER_DATA; ) {
				switch (k) {
					case HI_SERIES_INDEX:
						thispx = highx[j];
						thispy = highy[j];
						break;
					case LO_SERIES_INDEX:
						thispx = lowx[j];
						thispy = lowy[j];
						break;
					case OPEN_SERIES_INDEX:
						if (!showOpen || !hasOpen || tickWidth == 0) {
					 		k++;
							continue;
						}
						if (!other) {
							thispx = openx[j];
							thispy = openy[j];
							if (fullTickWidth)
								thispx += (-tickMod) * tickSize;
							other = true;
						} else {
							thispx = openx[j] + (tickMod * tickSize);
							other = false;
						}
						break;
					case CLOSE_SERIES_INDEX:
						if (!showClose || !hasClose || tickWidth == 0) {
					 		k++;
							continue;
						}
						if (!other) {
							thispx = closex[j];
							thispy = closey[j];
							if (fullTickWidth)
								thispx += (tickMod * tickSize);
							other = true;
						} else {
							thispx = closex[j] - (tickMod * tickSize);
							other = false;
						}
						break;
				}

				if (thispx > xmax ||
					xmin > thispx ||
					thispy > ymax ||
					ymin > thispy) {
					 if (!other) k++;
					 continue;
				}

				double len;
				switch(focus) {
                case (ChartDataView.PICK_FOCUS_X) :
                    pix_x = xaxis.toPixel(thispx, drawFront);
                    len = inverted ?
                        Math.abs(pix_x - py) : Math.abs(pix_x - px);
                    break;
                case (ChartDataView.PICK_FOCUS_Y) :
                    pix_y = yaxis.toPixel(thispy, drawFront);
                    len = inverted ?
                        Math.abs(pix_y - px) : Math.abs(pix_y - py);
                    break;
                case (ChartDataView.PICK_FOCUS_XY) :
                default :
                    pix_x = xaxis.toPixel(thispx, drawFront);
                    pix_y = yaxis.toPixel(thispy, drawFront);
                    len = inverted ?
                        (Math.sqrt(((pix_y - px)*(pix_y - px)) +
                            ((pix_x - py)*(pix_x - py)) ) ) :
                        (Math.sqrt(((pix_x - px)*(pix_x - px)) +
                            ((pix_y - py)*(pix_y - py)) ) ) ;
                    break;
				}
				if (length > len) {
					series = i * NUM_SERIES_PER_DATA + k;
					point = j;
					length = len;
				}

				if (!other) k++;
			}
		}
	}
	if (series >= 0 && point >= 0) {
		di = new JCDataIndex(point,
				(ChartDataViewSeries) seriesList.get(series), series);
		di.distance = (int) length;
	}
	return di;
	} catch (Exception e) {
		e.printStackTrace(System.out);
		return null;
	}
}

/**
 * Unpick method for Hi-Lo-Open-Close charts.  Given a data series and a datapoint
 * within that series, the function returns pixel coordinates of
 * that point relative to the chart area.
 * @param pt point index
 * @param seriesIndex data series
 * @return point object representing the position in screen pixels
 */
public Point unpick(int pt, int seriesIndex) {

	ChartDataViewSeries series =
				(ChartDataViewSeries) seriesList.get(seriesIndex);
	if (series == null) return null;

	double yvalue = series.getY(pt);
	double xvalue = series.getX(pt);
	int point_x = -1;
	int point_y = -1;

	if (yvalue != holeValue && xvalue != holeValue) {
		if (inverted) {
			point_x = yaxis.toPixel(yvalue, drawFront);
			point_y = xaxis.toPixel(xvalue, drawFront);
		} else {
			point_x = xaxis.toPixel(xvalue, drawFront);
			point_y = yaxis.toPixel(yvalue, drawFront);
		}
		return new Point(point_x, point_y);
	}
	return null;
}

/** Location on the chart area of the point before the point being edited. */
Point start = null;

/** Last position on the chart area of the edited point. */
Point end = null;

/** Short cut to drawing area for the chart area. */
Rectangle r = null;

/**
 * Method called by the <code>EditHandler</code> when an edit is started
 * on the data set that is plotted by this drawable.
 * @param di <code>JCDataIndex</code> instance specifying the data series and
 * point values for the point being edited
 */
void startEdit(JCDataIndex di) {
	if (di == null || di.dataView != dataObject) return;
	int seriesIndex = di.series.getDrawingOrder() / NUM_SERIES_PER_DATA;
	r = chartArea.getDrawingArea();
	start = null;
	end = null;

	int startInd = seriesIndex * NUM_SERIES_PER_DATA;
	int sInd = di.seriesIndex - startInd;
	double thispx =
		((ChartDataViewSeries) seriesList.get(startInd + sInd)).getX(di.point);
	double thispy =
		((ChartDataViewSeries) seriesList.get(startInd + sInd)).getY(di.point);
	if (inverted)
		start = new Point(yaxis.toPixel(thispy)+r.x,
							xaxis.toPixel(thispx)+r.y);
	else
		start = new Point(xaxis.toPixel(thispx)+r.x,
							yaxis.toPixel(thispy)+r.y);
}

/**
 * Method called by the <code>EditHandler</code> when an edit is occuring
 * on the data set that is plotted by this drawable.
 * @param di <code>JCDataIndex</code> instance specifying the data series and
 * point values for the point being edited
 * @param x <i>x</i> location of the action, relative to the chart area
 * @param y <i>y</i> location of the action, relative to the chart area
 */
void showEdit(JCDataIndex di, int x, int y) {
	if (di == null || di.dataView != dataObject) return;
	boolean inverted = di.dataView.getXAxis().i_vertical;
	chartArea.drawRubberLine(start, end);

	// Edit for HLOC charts only edits in the y direction
	if (end == null)
		// Create new point at x axis value of piced point with varying
		// y axis value.
		if (inverted)
			end = new Point(x,
				di.dataView.getXAxis().toPixel(di.series.getX(di.point),
														drawFront) + r.y);
		else
			end = new Point(
				di.dataView.getXAxis().toPixel(di.series.getX(di.point),
														drawFront) + r.x, y);
	else {
		if (inverted)
			end.x = x;
		else
			end.y = y;
	}
	chartArea.drawRubberLine(start, end);
}

/**
 * Method called by the <code>EditHandler</code> when an edit is finished
 * on the data set that is plotted by this drawable.
 * @param di <code>JCDataIndex</code> instance specifying the data series and
 * point values for the point being edited
 * @param x <i>x</i> location of the action, relative to the chart area
 * @param y <i>y</i> location of the action, relative to the chart area
 * @return
 */
int endEdit(JCDataIndex di, int x, int y) {
	if (di == null || di.dataView != dataObject) return -1;
	chartArea.drawRubberLine(start, end);
	return (di.dataView.getXAxis().i_vertical ? x - r.x : y - r.y);
}

public int getNumSeriesPerData() {
	return NUM_SERIES_PER_DATA;
}

}
