/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

package com.klg.jclass.chart;

import com.klg.jclass.util.ImageMapInfo;

/**
 * An interface used to specify image map url and extra information
 * for a JClass Chart data model.  Image maps information will only be
 * processed by JClass ServerChart (i.e. ignored by JClass Chart).
 *
 * <P><b>Note:</b> The <code>ImageMapChartDataModel</code> interface
 * is only used in conjunction with the <code>ChartDataModel</code>
 * interface; that is, in order for an object to be recognized as a chart
 * data source, it needs to implement the <code>ChartDataModel</code> interface.
 *
 * @see ChartDataModel
 */
public interface ImageMapChartDataModel {

/**
 * Retrieves a array of image map information indexed by series.
 * @param seriesIndex The series whose image map information is required.
 * @return The current point image map information list for the given series.
 */
public ImageMapInfo[] getPointImageMapInfo(int seriesIndex);

/**
 * Retrieves a array of image map information, one for each point cluster.
 * @return The current cluster image map information list.
 */
public ImageMapInfo[] getClusterImageMapInfo();

/**
 * Retrieves a array of image map information, one for each series.
 * @return The current series image map information list.
 */
public ImageMapInfo[] getSeriesImageMapInfo();

/**
 * Retrieves a array of image map information, one for each series.
 * @return The current series image map information list.
 */
public ImageMapInfo[] getLegendImageMapInfo();

}
