/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ImageMapShape.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.awt.*;
import java.awt.geom.Arc2D;
import java.awt.geom.PathIterator;

/**
 * This class is used for Pie chart image mapping.  For internal use only.
 */
public class ImageMapShape implements ProcessShapeModel {

protected Mapper  mapper = null;

public ImageMapShape(Mapper mapper)
{
    this.mapper = mapper;
}

public void updateFillOrientation(Rectangle rect, int newOrient)
{
    //  Do nothing.
}

public void resetFillOrientation()
{
    //  Do nothing.
}

/**
 * Saves the <code>Shape</code> parameter to a Vector.  Also
 * adds the shape and its associated <code>JCDataIndex</code>
 * to a hashtable.
 * @param shape
 */
public void fillPolygon(Shape shape)
{
    if (mapper != null) {
        mapper.addShape(shape);
    }
}

/**
 * Same as <code>fillPolygon</code>.
 * @param shape
 */
public void fillPolygonShadow(Shape shape)
{
    fillPolygon(shape);
}

/**
 * Creates a polygon that approximates the shape of a filled Arc
 * and saves it to a Vector.  Also adds the Polygon and its associated
 * <code>JCDataIndex</code> to a hashtable.
 * @param arcAngle
 * @param height
 * @param startAngle
 * @param width
 * @param x
 * @param y
 */
public void fillArc(double x, double y, double width, double height,
                    double startAngle, double arcAngle)
{

    final double fraction = 0.02;

    if (mapper != null) {
        double flatness = Math.min(width, height) * fraction;
        flatness = Math.max(0.0, flatness);
        width *= (1.0 + fraction);
        height *= (1.0 + fraction);
        x -= (width * fraction / 2.0);
        y -= (height * fraction / 2.0);
        Arc2D.Double shape = new Arc2D.Double(x, y, width, height,
                                              startAngle, arcAngle, Arc2D.PIE);
        PathIterator pathIterator = shape.getPathIterator(null, flatness);
        Polygon polygon = new Polygon();
        while (!pathIterator.isDone()) {
            double[] coords = new double[6];
            int type = pathIterator.currentSegment(coords);
            switch (type) {
            case PathIterator.SEG_MOVETO:  //  fall through to next case.
            case PathIterator.SEG_LINETO:
                polygon.addPoint((int)Math.round(coords[0]),
                                 (int)Math.round(coords[1]));
                break;
            case PathIterator.SEG_CLOSE:
                //  Do nothing.
                break;
            default:
                System.out.println(
                    "ImageMapShape.fillArc()  Bad PathIterator enum!");
            }
            pathIterator.next();
        }
        mapper.addShape(polygon);
    }
}

/**
 * Same as <code>fillArc</code>.
 * @param arcAngle
 * @param height
 * @param startAngle
 * @param width
 * @param x
 * @param y
 */
public void fillArcShadow(double x, double y, double width, double height,
                          double startAngle, double arcAngle)
{
    fillArc(x, y, width, height, startAngle, arcAngle);
}

public void drawLine(double x1, double y1, double x2, double y2)
{
    //  Do nothing.
}

public void drawArc(double x, double y, double width, double height,
                    double startAngle, double arcAngle)
{
    //  Do nothing.
}

}
