/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCAnno.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.awt.*;

/**
 * An instance of this class is used to define a series of tick
 * marks on a JCAxis.
 */
public class JCAnno implements java.io.Serializable {

public final static int DEFAULT_LABELS             = 10;
public final static int DEFAULT_TICKS              = 20;
public final static int USER_DEFINED               = 30;
public final static int DEFAULT_INNER_EXTENT       =  2;
public final static int DEFAULT_OUTER_EXTENT       =  2;
public final static int DEFAULT_LABEL_EXTENT       =  3;

protected JCAxis            parent              = null;

protected String            name                = null;

/**
 * The <code>startValue</code> is the scale value at which to begin this set of
 * tick marks.
 */
protected DerivedDouble     startValue          = new DerivedDouble(0., true);

/**
 * The <code>stopValue</code> is the scale value at which this set of tick
 * marks ends.
 */
protected DerivedDouble     stopValue           = new DerivedDouble(100., true);

/**
 * If not automatic, the scale values at which tick marks are to be placed.
 */
protected DerivedDouble     incrementValue      = new DerivedDouble(10., true);

protected int               defPrecision        = 0;
/**
 * If <code>precisionUseDefault</code> is <code>false</code>, use this
 * precision.  If it is <code>true</code>, the value of this property will be
 * automatically calculated and the calculated value will overwrite any
 * previous value this property might have.
 */
protected DerivedInt        precision           = new DerivedInt(
                                                    defPrecision, true);

/**
 * If <code>true</code>, draw tick labels; otherwise, no tick labels are drawn.
 */
protected boolean           drawLabels          = true;

/**
 * If <code>true</code>, draw tick marks; otherwise, no tick marks are drawn.
 */
protected boolean           drawTicks           = true;

protected int               defLabelExtent      = DEFAULT_LABEL_EXTENT;
/**
 * The distance in pixels of a label from the axis.
 */
protected DerivedInt        labelExtent         = new DerivedInt(
                                                    defLabelExtent, true);

protected int               defInnerExtent      = DEFAULT_INNER_EXTENT;
/**
 * The pixel length of a tick within the plot area.
 */
protected DerivedInt        innerExtent         =  new DerivedInt(
                                                    defInnerExtent, true);

protected int               defOuterExtent      = DEFAULT_OUTER_EXTENT;
/**
 * The pixel length of a tick outside of the plot area.
 */
protected DerivedInt        outerExtent         = new DerivedInt(
                                                    defOuterExtent, true);

/**
 * The color in which to draw ticks.
 */
protected Color             tickColor           = null;

/**
 * The color in which to draw labels.
 */
protected Color             labelColor          = null;

protected int               type                = USER_DEFINED;

/**
 * Creates a JCAnno object with default properties.
 */
public JCAnno() {
}

/**
 * Creates a JCAnno with startValue, stopValue, and
 * incrementValue initialized to the min, max, and tickSpacing
 * values of the parent JCAxis.
 */
public JCAnno(int type, JCAxis parent, Color tickColor) {
    this.type = type;
    this.parent = parent;
    if (parent != null) {
        startValue.value = parent.min.value;
        stopValue.value = parent.max.value;
        incrementValue.value = (type == DEFAULT_LABELS) ?
            parent.numSpacing.value : parent.tickSpacing.value;
        this.tickColor = (tickColor != null) ?
            tickColor : parent.getForeground();
    }
    drawLabels = (type == DEFAULT_LABELS);
    drawTicks = (type != DEFAULT_LABELS);
}

/**
 * Creates a tick object and sets its properties.
 *
 * @param type The tick object type. Valid values are DEFAULT_LABELS,
 *      DEFAULT_TICKS, and USER_DEFINED.
 * @param parent The JCAxis associated with this JCAnno.
 * @param startValue where the tick marks are to begin
 * @param stopValue where the tick marks are to end
 * @param incrementValue the (non-default) tick interval
 * @param precision The format of the tick label; positive values denote the
 *      number of places after the decimal point (for example, 3 means multiples
 *      of 0.001), and negative values indicate the positive number of zeros to
 *      be used before the decimal place (for example, -3 means numbering will
 *      be in multiples of 1000). Overridden if <code>precisionIsDefault</code>
 *      is set to <code>true</code>.
 * @param drawLabels whether tick labels are to be drawn
 * @param drawTicks whether tick marks are to be drawn
 * @param labelExtent the radial or linear distance to the center of a tick
 *      label, expressed as a decimal fraction of the scale's radius or
 *      width/height
 * @param innerExtent the place where the tick mark starts, expressed in pixels
 * @param outerExtent the place where the tick mark ends, expressed in pixels
 * @param tickColor the color of the tick mark
 * @param labelColor the color of the labels.
 */
public JCAnno(int               type,
              JCAxis            parent,
              double            startValue,
              double            stopValue,
              double            incrementValue,
              int               precision,
              boolean           drawLabels,
              boolean           drawTicks,
              int               labelExtent,
              int               innerExtent,
              int               outerExtent,
              Color             tickColor,
              Color             labelColor) {
    this.type            = type;
    this.parent          = parent;
    this.startValue      = new DerivedDouble(startValue, false);
    this.stopValue       = new DerivedDouble(stopValue, false);
    this.incrementValue  = new DerivedDouble(incrementValue, false);
    this.precision       = new DerivedInt(precision, false);
    this.drawLabels      = drawLabels;
    this.drawTicks       = drawTicks;
    this.labelExtent     = new DerivedInt(labelExtent, false);
    this.innerExtent     = new DerivedInt(innerExtent, false);
    this.outerExtent     = new DerivedInt(outerExtent, false);
    this.tickColor       = tickColor;
    this.labelColor      = labelColor;
}

/**
 * Returns the parent JCAxis.
 */
public JCAxis getParent() {
    return parent;
}

/**
 * Sets the parent JCAxis.
 */
public void setParent(JCAxis axis) {
    this.parent = axis;
}

/**
 * Returns the name of this tick object.
 */
public String getName() {
    return name;
}

/**
 * Sets the name of this tick object.
 */
public void setName(String name) {
    this.name = name;
}

/**
 * Returns the Boolean controlling the drawing of labels. <code>true</code>
 * means draw the labels, <code>false</code> means hide the labels.
 *
 * @return the <code>boolean</code> controlling the drawing of labels
 */
public boolean getDrawLabels() {
    return(drawLabels);
}

/**
 * Returns true if the drawLabels property has the default value.
 */
public boolean getDrawLabelsIsDefault() {
    return (type == DEFAULT_LABELS) ?
            drawLabels : !drawLabels;
}

/**
 * Returns true if the drawTicks property has the default value.
 */
public boolean getDrawTicksIsDefault() {
    return (type == DEFAULT_LABELS) ?
            !drawTicks : drawTicks;
}

/**
 * Returns the Boolean controlling the drawing of ticks. <code>true</code>
 * means draw the ticks, <code>false</code> means hide the ticks.
 *
 * @return the <code>boolean</code> controlling the drawing of ticks
 */
public boolean getDrawTicks() {
    return(drawTicks);
}

/**
 * In non-automatic mode, use this value as the increment value; that
 * is, it places ticks at every <code>incrementValue</code> along the scale.
 * In automatic mode, the calculated increment value will overwrite this value.
 *
 * @return the spacing between adjacent tick marks
 */
public double getIncrementValue() {
    return(incrementValue.value);
}

public boolean getIncrementIsDefault() {
    return(incrementValue.isDefault);
}

/**
 * This method returns the distance (in pixels) that tick marks extend into the
 * plot area.
 * @return the inner extent of the tick mark
 */
public int getInnerExtent() {
    return(innerExtent.value);
}

public boolean getInnerExtentIsDefault() {
    return(innerExtent.isDefault);
}

public int getDefaultInnerExtent() {
    return defInnerExtent;
}

/**
 * Returns the distance (in pixels) of tick labels from the axis.
 *
 * @return The pixel distance of labels from the axis.
 */
public int getLabelExtent() {
    return(labelExtent.value);
}

public boolean getLabelExtentIsDefault() {
    return(labelExtent.isDefault);
}

public int getDefaultLabelExtent() {
    return(defLabelExtent);
}

/**
 * This method returns the distance (in pixels) that tick marks extend beyond the
 * plot area.
 * @return the outer extent of the tick mark
 */
public int getOuterExtent() {
    return(outerExtent.value);
}

public boolean getOuterExtentIsDefault() {
    return(outerExtent.isDefault);
}

public int getDefaultOuterExtent() {
    return defOuterExtent;
}

/**
 * Returns the format for numbers in a tick label.  Positive values denote the
 * number of places after the decimal point (for example, 3 means multiples
 * of 0.001); negative values indicate the positive number of zeros that are
 * used before the decimal place (for example, -3 means numbering will be in
 * multiples of 1000). Overridden if <code>precisionUseDefault</code> is set
 * to <code>true</code>.
 *
 * @return the precision code for formatting numbers in a tick label
 */
public int getPrecision() {
    return(precision.value);
}

/**
 * If <code>true</code>, uses the axis-determined precision for tick labels;
 * otherwise, uses the current value of <code>precision</code> property to
 * label tick marks.
 *
 * @return a Boolean indicating whether or not a label's precision is
 *     calculated by the gauge or gotten from the <code>precision</code>
 *     property
 */
public boolean getPrecisionIsDefault() {
    return(precision.isDefault);
}

public int getDefaultPrecision() {
    return defPrecision;
}

/**
 * In non-automatic mode, uses this value as the start value.  In automatic
 * mode, the calculated start value will overwrite this value.
 *
 * @return the start value
 */
public double getStartValue() {
    return(startValue.value);
}

public boolean getStartValueIsDefault() {
    return(startValue.isDefault);
}

/**
 * In non-automatic mode, uses this value as the stop value.  In automatic
 * mode, the calculated stop value will overwrite this value.
 *
 * @return the stop value
 */
public double getStopValue() {
    return(stopValue.value);
}

public boolean getStopValueIsDefault() {
    return(stopValue.isDefault);
}

/**
 * Returns the color in which to draw ticks.
 * If a tick color has not been specified for this JCAnno object,
 * return the parent's foreground color. If the parent is
 * undefined, or if the parent's foreground color is null,
 * return Color.black.
 *
 * @return the color of the tick mark
 */
public Color getTickColor() {
    Color color = Color.black;
    if (tickColor != null) {
        color = tickColor;
    }
    else if (parent != null && parent.getForeground() != null) {
        color = parent.getForeground();
    }
    return(color);
}

/**
 * Returns the color in which to draw labels.
 * If a label color has not been specified for this JCAnno object,
 * return the parent's foreground color. If the parent is
 * undefined, or if the parent's foreground color is null,
 * return Color.black.
 *
 * @return the color of the tick mark
 */
public Color getLabelColor() {
    Color color = Color.black;
    if (labelColor != null) {
        color = labelColor;
    }
    else if (parent != null && parent.getForeground() != null) {
        color = parent.getForeground();
    }
    return(color);
}

/**
 * Returns the Boolean controlling the drawing of labels. <code>true</code>
 * means draw the labels; <code>false</code> means hide the labels.
 *
 * @param drawLabels the boolean controlling the drawing of
 *     labels
 */
public void setDrawLabels(boolean drawLabels) {
    this.drawLabels = drawLabels;
    setChanged(true, Changeable.RECALC);
}

/**
 * Sets the boolean controlling the drawing of ticks. <code>true</code>
 * means draw the ticks; <code>false</code> means hide the ticks.
 *
 * @param drawTicks the boolean controlling the drawing of
 *     ticks
 */
public void setDrawTicks(boolean drawTicks) {
    this.drawTicks = drawTicks;
    setChanged(true, Changeable.RECALC);
}

/**
 * In non-automatic mode, use this value as the increment value; that
 * is, it places ticks at every <code>incrementValue</code> along the scale.
 * In automatic mode, the calculated increment value will overwrite this value.
 *
 * @param incrementValue the spacing between adjacent tick marks
 */
public void setIncrementValue(double incrementValue) {
    this.incrementValue.value = incrementValue;
    this.incrementValue.isDefault = false;
    if (parent != null) {
        if (type == DEFAULT_LABELS) {
            parent.numSpacing.value = incrementValue;
            parent.numSpacing.isDefault = false;
        }
        else if (type == DEFAULT_TICKS) {
            parent.tickSpacing.value = incrementValue;
            parent.tickSpacing.isDefault = false;
        }
    }
}

public void setIncrementIsDefault(boolean isDefault) {
    this.incrementValue.isDefault = isDefault;
    if (parent != null) {
        if (type == DEFAULT_LABELS) {
            parent.numSpacing.isDefault = isDefault;
        }
        else if (type == DEFAULT_TICKS) {
            parent.tickSpacing.isDefault = isDefault;
        }
    }
    setChanged(true, Changeable.RECALC);
}

/**
 * Sets the pixel length of tick marks within the plot area.
 * @param innerExtent the inner extent of the tick mark
 */
public void setInnerExtent(int innerExtent) {
    this.innerExtent.value = innerExtent;
    this.innerExtent.isDefault = false;
    setChanged(true, Changeable.RECALC);
}

public void setInnerExtentIsDefault(boolean isDefault) {
    this.innerExtent.isDefault = isDefault;
    if (isDefault) {
        innerExtent.value = defInnerExtent;
    }
    setChanged(true, Changeable.RECALC);
}

public void setDefaultInnerExtent(int defValue) {
    this.defInnerExtent = defValue;
}

/**
 * Sets the pixel distance at which to place the tick label objects.
 *
 * @param labelExtent the label extent
 */
public void setLabelExtent(int labelExtent) {
    this.labelExtent.value = labelExtent;
    this.labelExtent.isDefault = false;
    setChanged(true, Changeable.RECALC);
}

public void setLabelExtentIsDefault(boolean isDefault) {
    this.labelExtent.isDefault = isDefault;
    if (isDefault) {
        labelExtent.value = defLabelExtent;
    }
    setChanged(true, Changeable.RECALC);
}

public void setDefaultLabelExtent(int defValue) {
    this.defLabelExtent = defValue;
}

/**
 * Sets the pixel length of tick marks outside of the plot area.
 * @param outerExtent the outer extent of the tick mark
 */
public void setOuterExtent(int outerExtent) {
    this.outerExtent.value = outerExtent;
    this.outerExtent.isDefault = false;
    setChanged(true, Changeable.RECALC);
}

public void setOuterExtentIsDefault(boolean isDefault) {
    this.outerExtent.isDefault = isDefault;
    if (isDefault) {
        outerExtent.value = defOuterExtent;
    }
    setChanged(true, Changeable.RECALC);
}

public void setDefaultOuterExtent(int defValue) {
    this.defOuterExtent = defValue;
}

/**
 * Sets the format for numbers in a tick label; positive values denote the
 * number of places after the decimal point (for example, 3 means multiples
 * of 0.001), negative values indicate the positive number of zeros are
 * used before the decimal place (for example, -3 means numbering will be in
 * multiples of 1000). Overridden if <code>precisionUseDefault</code> is set
 * to <code>true</code>.
 *
 * @param precision an integer that specifies the number of decimal places
 *      for the values in tick labels
 */
public void setPrecision(int precision) {
    this.precision.value = precision;
    this.precision.isDefault = false;
    if (type == DEFAULT_LABELS && parent != null) {
        parent.precision.value = precision;
        parent.precision.isDefault = false;
    }
    setChanged(true, Changeable.RECALC);
}

/**
 * If <code>true</code>, uses the axis-determined precision for tick labels;
 * otherwise, uses the current value of <code>precision</code> property to
 * label tick marks.
 *
 * @param isDefault whether or not to use the axis-determined
 *     precision
 */
public void setPrecisionIsDefault(boolean isDefault) {
    this.precision.isDefault = isDefault;
    if (isDefault) {
        precision.value = defPrecision;
    }
    setChanged(true, Changeable.RECALC);
}

public void setDefaultPrecision(int defValue) {
    this.defPrecision = defValue;
}

/**
 * In non-automatic mode, use this value as the start value.  In automatic
 * mode, the calculated start value will overwrite this value.
 *
 * @param startValue the start value
 */
public void setStartValue(double startValue) {
    this.startValue.value = startValue;
    this.startValue.isDefault = false;
}

public void setStartValueIsDefault(boolean isDefault) {
    this.startValue.isDefault = isDefault;
}

/**
 * In non-automatic mode, use this value as the stop value.  In automatic
 * mode, the calculated stop value will overwrite this value.
 *
 * @param stopValue the stop value
 */
public void setStopValue(double stopValue) {
    this.stopValue.value = stopValue;
    this.stopValue.isDefault = false;
}

public void setStopValueIsDefault(boolean isDefault) {
    this.stopValue.isDefault = isDefault;
}

/**
 * The color to use to draw ticks.
 *
 * @param tickColor the color of the tick mark
 */
public void setTickColor(Color tickColor) {
    this.tickColor = tickColor;
    setChanged(true, Changeable.RECALC);
}

/**
 * The color to use to draw labels.
 *
 * @param labelColor the color of the label
 */
public void setLabelColor(Color labelColor) {
    this.labelColor = labelColor;
    setChanged(true, Changeable.RECALC);
}

/**
 * Intended for internal use.
 * Sets the type property of this JCAnno object.
 * Valid values are JCAnno.DEFAULT_LABELS, JCAnno.DEFAULT_TICKS,
 * or JCAnno.USER_DEFINED.
 */
public void setType(int type) {
    this.type = type;
    drawLabels = (type == DEFAULT_LABELS);
    drawTicks = (type != DEFAULT_LABELS);
}

/**
 * Intended for internal use.
 * Returns the type property of this JCAnno object.
 * Possible values are JCAnno.DEFAULT_LABELS, JCAnno.DEFAULT_TICKS,
 * or JCAnno.USER_DEFINED.
 */
public int getType() {
    return type;
}

/**
 * Calls the setChanged() method of the parent axis.
 * @param c new value for <code>Changed</code>
 * @param cf the value with which to update the changed flag
 */
protected void setChanged(boolean c, int cf) {
    if (parent != null) {
        parent.setChanged(c, cf);
    }
}

public String toString() {
    String str = super.toString();
    str = str + ", name=" + name +
                ", startValue=" + startValue.value +
                ", stopValue=" + stopValue.value +
                ", incrementValue=" + incrementValue.value +
                ", precision=" + precision.value +
                ", innerExtent=" + innerExtent.value +
                ", outerExtent=" + outerExtent.value +
                ", labelExtent=" + labelExtent.value +
                ", drawTicks=" + drawTicks +
                ", drawLabels=" + drawLabels +
                ", color=" + tickColor;
    return str;
}

}
