/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCAxis.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import com.klg.jclass.util.graphics.DefineShape;
import com.klg.jclass.util.legend.JCLegend;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.util.*;
import java.util.List;

/**
 * Class controlling all attributes relating to rendering axes.
 */
public class JCAxis extends ChartInteriorRegion implements java.io.Serializable
{

/** Property that controls the gap between visible value labels, in pixels. */
protected int gap = 3;

/** The axis title. */
protected JCAxisTitle title = null;
/** Axis miniumum. */
protected DerivedDouble min = new DerivedDouble(0.0, true);
/** Axis maxiumum. */
protected DerivedDouble max = new DerivedDouble(0.0, true);
/** Axis origin. */
protected DerivedDouble origin = new DerivedDouble(0.0, true);
/** Placement of origin on axis. */
protected DerivedInt originPlacement = new DerivedInt(AUTOMATIC, true);
/** Placement of axis with respect to another axis. */
protected DerivedInt  placement= new DerivedInt(AUTOMATIC, true);

/** True is grid lines are visible.  False otherwise. */
protected boolean gridVisible = false;

/** Formula used to define axis relationships. */
protected JCAxisFormula formula = null;

/** True if axis is editable. */
protected boolean  editable = true;
/** True if axis is logarithmic. */
protected boolean  logarithmic = false;
/** True if axis is reversed. */
protected boolean  reversed = false;
/** True if axis is vertical. */
protected boolean  vertical = false;
/** Internal flag */
protected boolean  i_vertical = false;
/**
 * Rotation of axis annotations.  Value should be one ROTATE_NONE,
 * ROTATE_90, ROTATE_180, ROTATE_270.
 */
protected int  annotationRotation = 0;
/** Axis to place this axis on. */
protected JCAxis placementAxis = null;
/** Location on placementAxis where this axis is placed. */
protected double placementLocation = 0.0;
/** Generator for custom annotations. */
protected JCLabelGenerator labelGenerator = null;


/** True if decimals in annotations should be padded with zeroes. */
protected boolean  decimalPadded = true;
/** Local copy of x axis annotation method. */
protected int savedXAxisAnnotationMethod;

/** Time unit used for time labels. */
protected long timeUnit = SECONDS;
/** Format used in displaying time labels. */
protected DerivedString timeFormat = new DerivedString("", true);
/** Base time used for time labels. */
protected Date timeBase = null;
/**
 * True if the user has explicitly set the time base value. Gets reset if
 * the user sets the time base to null.
 */
public boolean isTimeBaseSet = false;

/** The polygon that defines the axis trough. */
protected Polygon sideplane;

/** Internal property specifying the length of the tick marks in pixels */
protected int tickLength = 4;

/** Pointer to the parent chart area */
protected JCChartArea chartArea = null;

/** Locale for this axis. */
protected Locale locale = Locale.getDefault();

/** TimeZone for this axis.  Used only for time labels. */
protected TimeZone timeZone = TimeZone.getDefault();

/** Internal property that tracks the data space tick spacing */
protected double step = 1.0;

//Enums for time units.
//Also equals the number of milliseconds in one of those units.
/** Enums for TimeUnit property representing value of seconds.*/
final public static long SECONDS = 1000;
/** Enums for TimeUnit property representing value of minutes.*/
final public static long MINUTES = 60*SECONDS;
/** Enums for TimeUnit property representing value of hours.*/
final public static long HOURS   = (60*MINUTES);
/** Enums for TimeUnit property representing value of days.*/
final public static long DAYS    = (24*HOURS);
/** Enums for TimeUnit property representing value of weeks.*/
final public static long WEEKS   = (7*DAYS);
/** Enums for TimeUnit property representing value of months.*/
final public static long MONTHS  = (31*DAYS);
/** Enums for TimeUnit property representing value of years.*/
final public static long YEARS   = (365*DAYS);

/** The mininum and maximum value of data being plotted against this axis.*/
protected MinMax dataLimits = null;

/** Object that holds the axis numbering precision specified by the user */
protected DerivedInt precision = new DerivedInt(1, true);

/**
 * Object that holds the numbering precision for a Polar chart x axis
 * when the angle unit is RADIANS.
 */
protected DerivedInt radianPrecision = new DerivedInt(2, true);

/**
 * Object that holds the numbering precision for a Polar chart x axis
 * when the angle unit is DEGREES or GRADS.
 */
protected DerivedInt degreeGradPrecision = new DerivedInt(0, true);

/**  The default maximum precision. */
protected static final int MAX_PRECISION = 14;

/** The maximum value the axis precision can have. */
protected int maximumPrecision = MAX_PRECISION;

/** Object that holds the axis numbering spacing specified by the user */
protected DerivedDouble numSpacing = new DerivedDouble(0.0, true);

/** Object that holds the axis tick spacing specified by the user */
protected DerivedDouble tickSpacing = new DerivedDouble(0.0, true);

/// Enums tracking the origin placement
/** Enum value for automatic origin placement. */
public final static int  AUTOMATIC = 0;
/** Enum value for origin placement at axis minimum. */
public final static int  MIN = 1;
/** Enum value for origin placement at axis maximum. */
public final static int  MAX = 2;
/** Enum value for origin placement at axis zero. */
public final static int  ZERO = 3;
/** Enum value for origin placement at a particular axis value. */
public final static int  VALUE_ANCHORED = 4;
/** Enum value for origin placement at axis origin. */
public final static int  ORIGIN = 5;

// Enum for annotation methods
/** Enum value representing value annotations. */
public final static int VALUE = 0;
/** Enum value representing value label annotations. */
public final static int VALUE_LABELS = 1;
/** Enum value representing time label annotations. */
public final static int TIME_LABELS = 2;
/** Enum value representing point label annotations. */
public final static int POINT_LABELS = 3;

/** AnnotationMethod property, determines how the axis is labelled */
protected int annotationMethod = VALUE;

/** Label rotation in radians */
protected double annotationRotationAngle = 0.0;

// Enum for annotation rotation
/** Enum value representing no rotation. */
public final static int ROTATE_NONE = 0;
/** Enum value representing a rotation of 90 degrees. */
public final static int ROTATE_90 = 1;
/** Enum value representing a rotation of 180 degrees. */
public final static int ROTATE_180 = 2;
/** Enum value representing a rotation of 270 degrees. */
public final static int ROTATE_270 = 3;
/** Enum value representing a rotation specified by annotationRotationAngle. */
public final static int ROTATE_OTHER = 4;

/** The type of the chart this axis is associated with. */
protected transient int chartType;
/** True if this is the primary axis for this chart. */
protected transient boolean isPrimary;
/**
 *  The other axis associated with this axis.  If this is an X axis,
 *  the other axis is the associated Y axis, and vice versa.
 */
protected transient JCAxis otherAxis;

/** Handler for type-specific axis code. */
transient private AxisHandler axisHandler = null;
/** Handler for code specific to axis annotations. */
protected AnnotationHandler annoHandler = null;

private boolean axisDrawn = false;
private static int axisNum = 0;
private boolean hasBar = false;
boolean frontPlane = true;

protected PolarAxisParams polarAxisParams = null;
/** Angle (in RADIANS) of y axis in a Polar chart. */
protected double yAxisAngle = 0.0;

protected boolean usingDoublePixels = false;

/**  List of JCAnno objects.  */
protected List annoList = null;

/**  List of JCGrid objects.  */
protected List gridList = null;

protected boolean gridDefault = true;

/**
 * If true, and the annotation method is TIME_LABELS, POINT_LABELS,
 * or VALUE_LABELS, draw ticks specified by all the JCAnno objects.
 * If false, draw ticks at the positions of the labels.  This flag
 * is ignored if the annotation method is VALUE.  In that case,
 * always draw ticks specified by the JCAnno objects.
 */
protected boolean useAnnoTicks = false;

/** Used for point label image mapping. */
protected Mapper mapper = null;

/**
 * Performs common initialization tasks
 */
private void init()
{
    setTitle(new JCAxisTitle());
    setName(JCChartBundle.string(JCChartBundle.key6, locale)
                                                    + " " + (++axisNum));
    if (chartArea != null) {
        polarAxisParams = chartArea.polarChartParams;
    }
    annoList = new ArrayList();
    makeDefaultAnnotations();
    gridList = new ArrayList();
    // Add the default JCGrid object.
    gridList.add(new JCGrid(this));
}

/**
 * Constructs a JCAxis object given the parent JCChartArea and a
 * boolean indicating whether the axis is vertical or horizontal.
 * @param parent parent JCChartArea object. All axes must be attached
 * to a JCChartArea.
 * @param vert if true, the axis is a vertical axis. Otherwise, the
 * axis is considered horizontal.
 * @param meth Axis annotation method
 */
public JCAxis(JCChartArea parent, boolean vert, int meth)
{
    super();
    vertical = vert;
    i_vertical = vert;
    annotationMethod = meth;
    chartArea = parent;
    createAnnoHandler();
    setLocale(chartArea.getLocale());
    init();
}

/**
 * Default constructor
 */
public JCAxis()
{
    super();
    createAnnoHandler();
    init();
}

/**
 * Default constructor with locale.
 */
public JCAxis(Locale locale)
{
    super();
    setLocale(locale);
    createAnnoHandler();
    init();
}

/**
 * This method must be called before setLocale().
 */
private void createAnnoHandler() {
    annoHandler = isPolarRadar() ?
        new CircularChartAnnoHandler(this) :
        new AnnotationHandler(this);
}

/**
 * This method instantiates either an AnnotationHandler or a
 * CircularChartAnnoHandler depending on the chart type parameter.
 */
protected void updateAnnoHandler(int type) {
    if (annoHandler instanceof CircularChartAnnoHandler &&
        !isPolarRadar(type))
    {
        annoHandler = new AnnotationHandler(this);
    }
    else if (!(annoHandler instanceof CircularChartAnnoHandler) &&
             isPolarRadar(type))
    {
        annoHandler = new CircularChartAnnoHandler(this);
    }
}

/**
 * Sets the parent chart of this axis. This involves noting it locally,
 * setting the parent region to be the chart area associated with the
 * specified chart, setting these values on any titles this axis may have,
 * and marking all annotations to use this parent.
 * @param p The parent chart to use
 */
public void setParentChart(JCChart p)
{
    if (parentChart == p) {
        return;
    }
    super.setParentChart(p);
    if (p != null) {
        chartArea = p.getChartArea();
        setLocale(chartArea.getLocale());
    }
    if (chartArea != null) {
        polarAxisParams = chartArea.polarChartParams;
    }
    setParentRegion(chartArea);
    if (title != null) {
        title.setParentAxis(this);
        title.setParentChart(p);
        title.setParentRegion(chartArea);
    }
    if (annoHandler != null) {
        annoHandler.setValueLabelParent(p);
    }
}

/**
 * Set the locale for this axis.
 */
public void setLocale(Locale locale)
{
    if (locale == null) {
        locale = Locale.getDefault();
    }
    this.locale = locale;

    // Propogate locale to dependent formatters
    getNumberFormatter().setNumberFormatterLocale(locale);
    getAnnotationHandler().getDateFormatter().setLocale(locale);

    setChanged(true, Changeable.REDRAW);
}

/**
 * Retrieve the locale for this axis.
 */
public Locale getLocale()
{
    return(locale);
}

/**
 * Set the time zone for formatting.  This is used for time labels.
 * @param timeZone The new time zone.
 */
public void setTimeZone(TimeZone timeZone)
{
    if (timeZone == null) {
        timeZone = TimeZone.getDefault();
    }
    this.timeZone = timeZone;

    // Propogate time zone to dependent formatters
    getAnnotationHandler().getDateFormatter().setTimeZone(timeZone);
    setChanged(true, Changeable.REDRAW);
}

/**
 * Retrieve the time zone for this axis.
 * @return The current time zone.
 */
public TimeZone getTimeZone()
{
    return(timeZone);
}

/**
 * Gets the value of the annotationVisible property.
 */
public boolean isAnnotationVisible() {
    return annoHandler.annotationVisible;
}

/**
 * Sets the value of the annotationVisible property.
 */
public void setAnnotationVisible(boolean visible) {
    annoHandler.annotationVisible = visible;
}

/**
 * Gets the value of the GridVisible property.
 * The GridVisible property determines whether a grid is drawn
 * for the axis.
 * @return true if the grid should appear, false otherwise
 */
public boolean isGridVisible()
{
    return(gridVisible);
}

/**
 * Sets the value of the GridVisible property.
 * The GridVisible property determines whether a grid is drawn
 * for the axis.
 * @param isGridVisible Set to true if the grid should appear, false otherwise
 */
public void setGridVisible(boolean isGridVisible)
{
    if (isGridVisible == gridVisible) {
        return;
    }
    gridVisible = isGridVisible;
    setChanged(true, Changeable.RECALC);
}

/**
 * Sets the value of the gridDefault property.
 * The gridDefault property determines whether grid lines are drawn
 * at the labels.
 * @param isDefault If true, grid lines will be drawn at the labels.
 * If false, grid line positions will be determined by JCGrid
 * properties.
 */
public void setGridDefault(boolean isDefault)
{
    if (gridDefault == isDefault) {
        return;
    }
    gridDefault = isDefault;
    setChanged(true, Changeable.RECALC);
}

public boolean isGridDefault() {
    return(gridDefault);
}

/**
 * Gets the GridSpacing property of the first JCGrid object.
 * The GridSpacing property controls the spacing between grid
 * lines relative to the axis.
 * @return floating-point number representing the axis
 * value spacing between grid lines. A value of 0.0 indicates that
 * grid lines will not be shown.
 */
public double getGridSpacing()
{
    double gridSpacing = 0.0;
    if (gridList.size() > 0) {
        JCGrid gridObj = (JCGrid)gridList.get(0);
        if (gridObj != null) {
            gridSpacing = gridObj.getIncrementValue();
        }
    }
    return(gridSpacing);
}

/**
 * Sets the GridSpacing property of the first JCGrid object on this axis.
 * The GridSpacing property controls the spacing between grid
 * lines relative to the axis.
 * @param space floating-point number representing the axis
 * value spacing between grid lines. A value of 0.0 indicates that
 * grid lines will not be shown.
 */
public void setGridSpacing(double space)
{
    if (gridList.size() > 0) {
        JCGrid gridObj = (JCGrid)gridList.get(0);
        if (gridObj != null) {
            if (gridObj.getIncrementValue() == space) {
                return;
            }
            gridObj.setIncrementValue(space);
            gridDefault = false;
            setChanged(true, Changeable.RECALC);
        }
    }
}

/**
 * Sets the isDefault property of the first JCGrid object on this axis.
 * The GridSpacingIsDefault property determines whether Chart is responsible
 * for calculating the grid spacing value. If true, Chart will
 * calculate the grid spacing. If false, Chart will use the provided
 * grid spacing.
 * @param useDef true indicates that chart calculates the GridSpacing property.
 * If false, the GridSpacing property is already specified.
 */
public void setGridSpacingIsDefault(boolean useDef)
{
    this.gridDefault = useDef;
    if (gridList.size() > 0) {
        JCGrid gridObj = (JCGrid)gridList.get(0);
        if (gridObj != null) {
            if (gridObj.getIncrementIsDefault() == useDef) {
                return;
            }
            gridObj.setIncrementIsDefault(useDef);
            setChanged(true, Changeable.RECALC);
        }
    }
}

/**
 * Gets the isDefault property of the first JCGrid object on this axis.
 * The GridSpacingIsDefault property determines whether Chart is responsible
 * for calculating the grid spacing value. If true, Chart will
 * calculate the grid spacing. If false, Chart will use the provided
 * grid spacing.
 * @return true indicates that chart calculates the GridSpacing property.
 * If false, the GridSpacing property is already specified.
 */
public boolean getGridSpacingIsDefault()
{
    boolean isDefault = false;
    if (gridList.size() > 0) {
        JCGrid gridObj = (JCGrid)gridList.get(0);
        if (gridObj != null) {
            isDefault = gridObj.getIncrementIsDefault();
        }
    }
    return(isDefault);
}


/**
 * Gets the GridStyle property of the first JCGrid object on this axis.
 * The GridStyle property controls the appearance of the grid.
 * @return JCChartStyle object that controls the grid's
 * appearance
 */
public JCChartStyle getGridStyle()
{
    JCChartStyle gridStyle = null;
    if (gridList.size() > 0) {
        JCGrid gridObj = (JCGrid)gridList.get(0);
        if (gridObj != null) {
            gridStyle = gridObj.getGridStyle();
        }
    }
    // If null, create one
    if (gridStyle == null) {
        gridStyle = new JCChartStyle();
        gridStyle.setLineStyle(new JCLineStyle(1, getForeground(), 1));
        gridStyle.addParent(this);
    }
    return(gridStyle);
}

/**
 * Sets the GridStyle property of the first JCGrid object on this axis.
 * The GridStyle property controls the appearance of the grid.
 * @param style JCChartStyle object that controls the grid's
 * appearance
 */
public void setGridStyle(JCChartStyle style)
{
    if (gridList.size() > 0) {
        JCGrid gridObj = (JCGrid)gridList.get(0);
        if (gridObj != null) {
            if (gridObj.getGridStyle().equals(style)) {
                return;
            }
            gridObj.setGridStyle(style);
            setChanged(true, Changeable.RECALC);
        }
    }
}


/**
 * Sets the Logarithmic property of JCAxis, which determines
 * whether the axis will be logarithmic (true) or linear (false).
 * @param log If true, the axis will be logarithmic (base 10).
 * Otherwise, the axis will be linear.
 */
public void setLogarithmic(boolean log)
{
    if (logarithmic == log) {
        return;
    }
    logarithmic = log;
    axisHandler = null;

    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the Logarithmic property of JCAxis, which determines
 * whether the axis will be logarithmic (true) or linear (false).
 * @return if true, the axis will be logarithmic (base 10).
 * Otherwise, the axis will be linear.
 */
public boolean isLogarithmic()
{
    return(logarithmic);
}

/**
 * Sets the Editable property.
 * The Editable property determines whether the axis can
 * be affected by edit/translation/zooming.
 * @param b if true, the axis can be affected by an edit, translation
 * or zoom action.  If false, the axis is static.
 */
public void setEditable(boolean b)
{
    editable = b;
}

/**
 * Gets the Editable property.
 * The Editable property determines whether the axis can
 * be affected by edit/translation/zooming.
 * @return if true, the axis can be affected by an edit, translation
 * or zoom action.  If false, the axis is static.
 */
public boolean isEditable()
{
    return(editable);
}

/**
 * Sets the DecimalPadded property.
 * The DecimalPadded property determines whether the axis annotations will
 * have their decimal portions padded with zeroes to the size of the precision.
 * @param b if true, the axis annotations will be decimal padded.
 */
public void setDecimalPadded(boolean b)
{
    decimalPadded = b;
}

/**
 * Gets the DecimalPadded property.
 * The DecimalPadded property determines whether the axis annotations will
 * have their decimal portions padded with zeroes to the size of the precision.
 * @return if true, the axis annotations are decimal padded.
 */
public boolean isDecimalPadded()
{
    return(decimalPadded);
}

/**
 * Sets the Min property of the JCAxis, which controls the
 * minimum value shown on the axis.
 * @param nmin minimum value
 */
public void setMin(double nmin)
{
    min.value = nmin;
    min.isDefault = false;

    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the Min property of the JCAxis, which controls the
 * minimum value shown on the axis.
 * @return minimum value
 */
public double getMin()
{
    return(min.value);
}

/**
 * Sets the MinIsDefault property.
 * The MinIsDefault property determines whether Chart is responsible
 * for calculating the minimum axis value. If true, Chart will
 * calculate the axis min. If false, Chart will use the provided
 * axis min.
 * @param useDef if true, Chart calculates the Min property. If
 * false, the Min property is already specified.
 */
public void setMinIsDefault(boolean useDef)
{
    if (min.isDefault == useDef) {
        return;
    }
    min.isDefault = useDef;

    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the MinIsDefault property.
 * The MinIsDefault property determines whether Chart is responsible
 * for calculating the minimum axis value. If true, Chart will
 * calculate the axis min. If false, Chart will use the provided
 * axis min.
 * @return true Chart calculates the Min property.
 * If false, the Min property is already specified.
 */
public boolean getMinIsDefault()
{
    return(min.isDefault);
}

/**
 * Sets the Max property of the axis, which controls the
 * maximum value shown on the axis.
 * @param nmax maximum value
 */
public void setMax(double nmax)
{
    max.value = nmax;
    max.isDefault = false;

    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the Max property of the axis, which controls the
 * maximum value shown on the axis.
 * @return maximum value
 */
public double getMax()
{
    return(max.value);
}

/**
 * Sets the MaxIsDefault property.
 * The MaxIsDefault property determines whether Chart is responsible
 * for calculating the maximum axis value. If true, Chart
 * calculates the axis max. If false, Chart uses the provided
 * axis max.
 * @param useDef if true, Chart calculates the Max property.
 * If false, the Max property is already specified.
 */
public void setMaxIsDefault(boolean useDef)
{
    if (max.isDefault == useDef) {
        return;
    }
    max.isDefault = useDef;

    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the MaxIsDefault property.
 * The MaxIsDefault property determines whether Chart is responsible
 * for calculating the maximum axis value. If true, Chart
 * calculates the axis max. If false, Chart uses the provided
 * axis max.
 * @return true Chart calculates the Max property.
 * If false, the Max property is already specified.
 */
public boolean getMaxIsDefault()
{
    return(max.isDefault);
}

/**
 * Sets the Origin property of the Axis, which controls the
 * location of the origin along the axis.
 * @param oval origin value
 */
public void setOrigin(double oval)
{
    origin.value = oval;
    origin.isDefault = false;

    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the Origin property of the Axis, which controls the
 * location of the origin along the axis.
 * @return origin value
 */
public double getOrigin()
{
    return(origin.value);
}

/**
 * Sets the OriginIsDefault property.
 * The OriginIsDefault property determines whether Chart is responsible
 * for positioning the axis origin. If true, Chart
 * calculates the axis origin. If false, Chart uses the provided
 * axis origin value.
 * @param useDef if true Chart calculates the Origin property.
 * If false, the Origin property is already specified.
 */
public void setOriginIsDefault(boolean useDef)
{
    if (origin.isDefault == useDef) {
        return;
    }
    origin.isDefault = useDef;

    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the OriginIsDefault property.
 * The OriginIsDefault property determines whether Chart is responsible
 * for positioning the axis origin. If true, Chart will
 * calculate the axis origin. If false, Chart will use the provided
 * axis origin value.
 * @return true Chart calculates the Origin property.
 * If false, the Origin property is already specified.
 */
public boolean getOriginIsDefault()
{
    return(origin.isDefault);
}

/**
 * Sets the OriginPlacement property of the JCAxis, which
 * specifies where the origin is placed. Note that the OriginPlacement
 * property is only active if the Origin property has not been set.
 * @param place one of AUTOMATIC (places origin at minimum value
 * or at zero if there are negative and positive values), ZERO (places
 * origin at zero), MIN (places origin at minimum value on axis) or
 * MAX (places origin at maximum value on axis)
 * @exception IllegalArgumentException If OriginPlacement isn't one of
 * AUTOMATIC, ZERO, MIN or MAX.
 */
public void setOriginPlacement(int place)
{
    if (originPlacement.value == place) {
        return;
    }
    switch (place) {
    case AUTOMATIC:
    case MIN:
    case MAX:
    case ZERO:
        originPlacement.value = place;
        originPlacement.isDefault = false;
        break;
    default:
        throw new IllegalArgumentException("Invalid OriginPlacement in JCAxis");
    }

    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the OriginPlacement property of the JCAxis, which
 * specifies where the origin is placed.
 * @return one of AUTOMATIC (places origin at minimum value
 * or at zero if there are negative and positive values), ZERO (places
 * origin at zero), MIN (places origin at minimum value on axis) or
 * MAX (places origin at maximum value on axis)
 */
public int getOriginPlacement()
{
    return(originPlacement.value);
}

/**
 * Sets the OriginPlacementIsDefault property.
 * The OriginPlacementIsDefault property determines whether Chart
 * is responsible for determining the location of the axis origin.
 * If true, Chart calculates the origin positioning. If false,
 * Chart uses the provided origin placement.
 * @param useDef if true, Chart calculates the OriginPlacement property.
 * If false, the OriginPlacement property is already specified.
 */
public void setOriginPlacementIsDefault(boolean useDef)
{
    if (originPlacement.isDefault == useDef) {
        return;
    }
    originPlacement.isDefault = useDef;
    if (useDef) {
        originPlacement.value = AUTOMATIC;
    }

    setChanged(true, Changeable.RECALC);
}


/**
 * Gets the OriginPlacementIsDefault property.
 * The OriginPlacementIsDefault property determines whether Chart
 * is responsible for determining the location of the axis origin.
 * If true, Chart calculates the origin positioning. If false,
 * Chart uses the provided origin placement.
 * @return true Chart calculates the OriginPlacement property.
 * If false, the OriginPlacement property is already specified.
 */
public boolean getOriginPlacementIsDefault()
{
    return(originPlacement.isDefault);
}


/**
 * Retrieves the JCAxisTitle object, which controls the
 * appearance of the axis title.
 * @return JCAxisTitle object
 * @see com.klg.jclass.chart.JCAxisTitle
 */
public JCAxisTitle getTitle()
{
    if (title == null) {
        title = new JCAxisTitle();
        title.setParentAxis(this);
        title.setParentChart(parentChart);
        title.setParentRegion(chartArea);
    }
    return(title);
}

/**
 * Sets the JCAxisTitle object associated with the JCAxis.
 * JCAxisTitle controls the appearance of the axis title.
 * This method is only accessible inside the chart package.
 * @param props new JCAxisTitle object.
 * @see com.klg.jclass.chart.JCAxisTitle
 */
public void setTitle(JCAxisTitle props)
{
    if (props == null) {
        return;
    }
    title = props;
    title.setParentAxis(this);
    title.setParentChart(parentChart);
    title.setParentRegion(chartArea);

    setChanged(true, Changeable.RECALC);
}

/**
 * Sets the Reversed property of JCAxis, which determines if the
 * axis order is reversed.
 * @param rev true if the axis is reversed, false otherwise
 */
public void setReversed(boolean rev)
{
    reversed = rev;

    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the Reversed property of JCAxis, which determines if the
 * axis order is reversed.
 * @return true if the axis is reversed, false otherwise
 */
public boolean isReversed()
{
    return(reversed);
}

/**
 * Retrieves the JCAxisFormula object, which specifies a relationship
 * between this JCAxis object and another JCAxis object.
 * @return JCAxisFormula object
 * @see com.klg.jclass.chart.JCAxisFormula
 */
public JCAxisFormula getFormula()
{
    if (formula == null) {
        formula = new JCAxisFormula(this);
    }
    return(formula);
}

/**
 * Sets the AnnotationRotation property of JCAxis.
 * <P>The AnnotationRotation property specifies the rotation (clockwise)
 * of each axis label.
 * The 2D API is used to rotate the drawn text.
 * Valid values are JCAxis.ROTATE_NONE, JCAxis.ROTATE_90, JCAxis.ROTATE_180,
 * JCAxis.ROTATE_270, or ROTATE_OTHER.
 * @param rot one of ROTATE_NONE, ROTATE_90, ROTATE_180, ROTATE_270
 * or ROTATE_OTHER.
 * @exception IllegalArgumentException If the provided rotation value is
 * not part of the ROTATE_ enum.
 */
public void setAnnotationRotation(int rot)
{
    if (rot == annotationRotation) {
        return;
    }
    switch (rot) {
    case ROTATE_NONE:
    case ROTATE_90:
    case ROTATE_180:
    case ROTATE_270:
    case ROTATE_OTHER:
        annotationRotation = rot;
        break;
    default:
        throw new IllegalArgumentException(
            "Invalid AnnotationRotation value in JCAxis");
    }

    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the AnnotationRotation property of JCAxis.
 * <P>The AnnotationRotation property specifies the rotation (clockwise)
 * of each axis label.
 * In JDK 1.2, the 2D API is used to rotate the drawn text.
 * Valid values are JCAxis.ROTATE_NONE, JCAxis.ROTATE_90, JCAxis.ROTATE_180,
 * JCAxis.ROTATE_270 or  JCAxis.ROTATE_OTHER.
 * @return One of ROTATE_NONE, ROTATE_90, ROTATE_180, ROTATE_270
 * or ROTATE_OTHER.
 */
public int getAnnotationRotation() {
    return annotationRotation;
}

/**
 * Sets the AnnotationRotationAngle property of JCAxis.
 * The AnnotationRotationAngle property represents the angle of rotation
 * in radians of labels on this axis.
 * @param angleUnit The units of the specified angle.  Allowed
 * values are JCChartUtil.DEGREES, JCChartUtil.RADIANS, and JCChartUtil.GRADS.
 * If an invalid angle unit is specified, the method will not update
 * the AnnotationRotationAngle property.
 * @param rotationAngle The rotation angle of axis labels.
 */
public void setAnnotationRotationAngle(int angleUnit, double rotationAngle)
{
    String unit = "radians";
    switch(angleUnit) {
    case JCChartUtil.RADIANS:
        break;
    case JCChartUtil.DEGREES:
        unit = "degrees";
        break;
    case JCChartUtil.GRADS:
        unit = "grads";
        break;
    default:
        return;
    }
    double boundAngle = JCChartUtil.boundAngle(angleUnit, rotationAngle);
    // boundAngle() fails for huge rotation angles.  See if the returned
    // value is between -360 and +360 degrees.  If not, throw an
    // IllegalArgumentException.
    if (!isBoundAngle(angleUnit, boundAngle)) {
        throw new IllegalArgumentException(
            "Unable to normalize input angle " + rotationAngle + " " + unit);
    }
    annotationRotationAngle = JCChartUtil.convertAngle(angleUnit,
                                                       JCChartUtil.RADIANS,
                                                       boundAngle);
    setChanged(true, Changeable.RECALC);
}

private boolean isBoundAngle(int angleUnit, double angle) {
    double degrees = JCChartUtil.convertAngle(angleUnit,
                        JCChartUtil.DEGREES, angle);
    return (-360.0 <= degrees && degrees <= 360.0);
}

/**
 * Returns the AnnotationRotationAngle property of JCAxis.
 * The AnnotationRotationAngle property represents the angle of rotation
 * of the labels on this axis.
 * @param angleUnit The desired units for the returned angle.  Allowed
 * values are JCChartUtil.DEGREES, JCChartUtil.RADIANS, and JCChartUtil.GRADS.
 * @return The rotation angle of the axis labels in the specified angle unit.
 * If an invalid angle unit is specified, the returned angle will be
 * in RADIANS.
 */
public double getAnnotationRotationAngle(int angleUnit)
{
    switch(angleUnit) {
    case JCChartUtil.RADIANS:
    case JCChartUtil.DEGREES:
    case JCChartUtil.GRADS:
        break;
    default:
        angleUnit = JCChartUtil.RADIANS;
    }
    return(JCChartUtil.convertAngle(JCChartUtil.RADIANS,
                                    angleUnit, annotationRotationAngle));
}

/**
 * Sets the Placement property of the JCAxis, which
 * controls where the axis is placed.
 * @param place one of AUTOMATIC (Chart chooses an appropriate
 * location), ORIGIN (appears at the origin value), MIN (appears at
 * the minimum axis value), MAX (appears at the maximum axis
 * value) or VALUE_ANCHORED (appears at a particular value along another axis.
 * Note that <TT>x</TT> axis labels, for example, will be placed at
 * either the <TT>y</TT> axis origin, <TT>y</TT> axis max or <TT>y</TT>
 * axis min value.
 * @exception IllegalArgumentException If the Placement value is not one
 * of AUTOMATIC, ORIGIN, MIN, MAX or VALUE_ANCHORED.
 */
public void setPlacement(int place)
{
    if (placement.value == place) {
        return;
    }
    switch (place) {
    case AUTOMATIC:
    case ORIGIN:
    case MIN:
    case MAX:
    case VALUE_ANCHORED:
        placement.value = place;
        placement.isDefault = false;
        break;
    default:
        throw new IllegalArgumentException("Invalid Placement value in JCAxis");
    }

    setChanged(true, Changeable.RECALC);
}

/**
 * Convenience method that sets the Placement property to
 * ORIGIN, and sets the PlacementAxis property to the
 * provided value
 * @param ax new value for PlacementAxis.
 */
public void setPlacement(JCAxis ax)
{
    placementAxis = ax;

    setPlacement(ORIGIN);
    // Already called by setPlacement()
    //setChanged(true, Changeable.RECALC);
}

/**
 * Convenience method that sets the Placement property to
 * VALUE_ANCHORED, the PlacementAxis property to the provided axis,
 * and the PlacementLocation property to the provided floating-
 * point value.
 * @param ax new value for PlacementAxis
 * @param v new value for PlacementLocation
 */
public void setPlacement(JCAxis ax, double v)
{
    placementAxis = ax;
    placementLocation = v;

    setPlacement(VALUE_ANCHORED);
    // Already called by setPlacement()
    //setChanged(true, Changeable.RECALC);
}

/**
 * Gets the Placement property of the JCAxis, which
 * controls where the axis is placed.
 * @return one of AUTOMATIC (Chart chooses an appropriate
 * location), ORIGIN (appears at the origin value), MIN (Appears at
 * the minimum axis value) and MAX (appears at the maximum axis
 * value).  Note that <TT>x</TT> axis labels, for example, will be placed at
 * either the <TT>y</TT> axis origin, <TT>y</TT> axis max or <TT>y</TT> axis min value.
 */
public int getPlacement()
{
    return(placement.value);
}

/**
 * Sets the PlacementIsDefault property.
 * The PlacementIsDefault property determines whether Chart
 * is responsible for determining the location of the axis.
 * If true, Chart calculates the origin positioning. If false,
 * Chart uses the provided axus placement.
 * @param useDef if true, Chart calculates the Placement property.
 * If false, the Placement property is already specified.
 */
public void setPlacementIsDefault(boolean useDef)
{
    if (placement.isDefault == useDef) {
        return;
    }
    placement.isDefault = useDef;
    if (useDef) {
        placement.value = AUTOMATIC;
    }

    setChanged(true, Changeable.RECALC);
}


/**
 * Gets the PlacementIsDefault property.
 * The PlacementIsDefault property determines whether Chart
 * is responsible for determining the location of the axis.
 * If true, Chart calculates the axis positioning. If false,
 * Chart uses the provided  placement.
 * @return true Chart calculates the Placement property.
 * If false, the Placement property is already specified.
 */
public boolean getPlacementIsDefault()
{
    return(placement.isDefault);
}


/**
 * Sets the PlacementAxis property of the axis.  The PlacementAxis
 * property determines the axis that controls the placement of
 * this axis.  In JCChart, it is possible to position an axis
 * at a particular position on another axis (in conjunction with
 * the PlacementLocation property or the Placement property).
 * @param ax axis to be used to position this axis.
 */
public void setPlacementAxis(JCAxis ax)
{
    placementAxis = ax;

    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the PlacementAxis property of the axis.  The PlacementAxis
 * property determines the axis that controls the placement of
 * this axis.  In JCChart, it is possible to position an axis
 * at a particular position on another axis (in conjunction with
 * the PlacementLocation property or the Placement property).
 * @return axis to be used to position this axis.
 */
public JCAxis getPlacementAxis()
{
    return(placementAxis);
}

/**
 * Sets the value of the PlacementLocation property.  The
 * PlacementLocation property is used with the PlacementAxis
 * property to position the current axis object at a particular
 * point on another axis.
 * @param v Floating-point data value on PlacementAxis at which to
 * place the current axis.
 */
public void setPlacementLocation(double v)
{
    placementLocation = v;

    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the value of the PlacementLocation property.  The
 * PlacementLocation property is used with the PlacementAxis
 * property to position the current axis object at a particular
 * point on another axis.
 * @return Floating-point data value on PlacementAxis at which to
 * place the current axis.
 */
public double getPlacementLocation()
{
    return(placementLocation);
}

/**
 * Sets the value of the AnnotationMethod property. The
 * AnnotationMethod property determines how axis annotations
 * are generated.
 * @param newMethod one of VALUE, VALUE_LABELS,
 * TIME_LABELS or POINT_LABELS
 * @exception IllegalArgumentException If the AnnotationMethod value
 * is not one of VALUE, VALUE_LABELS, TIME_LABELS or
 * POINT_LABELS.
 */
public void setAnnotationMethod(int newMethod)
{
    if (newMethod == annotationMethod) {
        savedXAxisAnnotationMethod = annotationMethod;
        return;
    }

    switch (newMethod) {
    case POINT_LABELS:
        if (chartArea != null && !chartArea.isXAxis(this))
            throw new IllegalArgumentException(
                "Only X axes can use point labels");
    case VALUE:
        if (useAnnoTicks) {
            JCAnno annoObj = getDefaultLabels();
            if (annoObj != null) {
                annoObj.drawLabels = true;
            }
        }
        // fall through to the next case
    case VALUE_LABELS:
        annotationMethod = newMethod;
        savedXAxisAnnotationMethod = annotationMethod;
        break;
    case TIME_LABELS:
        //  Don't allow time labels for the theta axis a Radar chart.
        if (isRadar() && !i_vertical) {
            return;
        }
        else {
            annotationMethod = newMethod;
            savedXAxisAnnotationMethod = annotationMethod;
        }
        break;
    default:
        throw new IllegalArgumentException("Invalid AnnotationMethod value in JCAxis");
    }

    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the value of the AnnotationMethod property. The
 * AnnotationMethod property determines how axis annotations
 * are generated.
 * @return One of VALUE, VALUE_LABELS,
 * TIME_LABELS or POINT_LABELS
 */
public int getAnnotationMethod()
{
    return(annotationMethod);
}

/**
 * Sets the Precision property of the Axis, which controls
 * the number of zeros that appear after the decimal place in
 * the axis label.
 * @param prec number of zeros after the decimal place. If negative,
 * it specifies the number of zeros that appear before the decimal place.
 */
public void setPrecision(int prec)
{
    DerivedInt precObj = getPrecisionObject();
    precObj.value = prec;
    precObj.isDefault = false;
    // Set the default precision of the DEFAULT_LABELS JCAnno.
    JCAnno defLabels = getDefaultLabels();
    if (defLabels != null && defLabels.precision.isDefault) {
        defLabels.precision.value = prec;
        defLabels.precision.isDefault = false;
        // Must clear the anno object's isDefault flag.
        // The Financial demo illustrates the need for this.
    }
    setChanged(true, Changeable.RECALC);
}

protected void setPrecisionInternal(int prec)
{
    DerivedInt precObj = getPrecisionObject();
    precObj.value = prec;
    updateAnnoDefPrecision(prec);
}

/**
 * This method updates the default precision of
 * the DEFAULT_LABELS JCAnno object.
 */
protected void updateAnnoDefPrecision(int prec) {
    JCAnno defLabels = getDefaultLabels();
    if (defLabels != null) {
        defLabels.setDefaultPrecision(prec);
        if (defLabels.getPrecisionIsDefault()) {
            defLabels.precision.value = prec;
        }
    }
}

/**
 * Gets the Precision property of the Axis, which controls
 * the number of zeros that appear after the decimal place in
 * the axis label.
 * @return number of zeros after the decimal place. If negative,
 * specifies the number of zeros that appear before the decimal place.
 */
public int getPrecision()
{
    int prec = getPrecisionObject().value;
    JCAnno defLabels = getDefaultLabels();
    if (defLabels != null) {
        prec = defLabels.getPrecision();
    }
    return(prec);
}

/**
 * Sets the PrecisionIsDefault property of JCAxis. The
 * PrecisionIsDefault determines whether Chart is responsible
 * for calculating the numbering precision.  If true, Chart will
 * calculate the precision.  If false, Chart will use the provided
 * precision.
 * @param useDef true tells Chart to calculate the Precision property,
 * whereas false means that the Precision property is already specified
 */
public void setPrecisionIsDefault(boolean useDef)
{
    DerivedInt prec = getPrecisionObject();
    prec.isDefault = useDef;

    JCAnno annoObj = getDefaultLabels();
    if (annoObj != null) {
        annoObj.precision.isDefault = useDef;
        if (useDef) {
            annoObj.precision.value = annoObj.defPrecision;
        }
    }

    setChanged(true, Changeable.RECALC);
}

private DerivedInt getPrecisionObject()
{
    DerivedInt prec = precision;
    if (isPolar() && !i_vertical) {
        //  For the theta axis of a Polar chart...
        int angleUnit = chartArea.getAngleUnit();
        switch(angleUnit) {
        case JCChartUtil.RADIANS:
            prec = radianPrecision;
            break;
        case JCChartUtil.DEGREES:
        case JCChartUtil.GRADS:
            prec = degreeGradPrecision;
            break;
        default:
        }
    }
    return(prec);
}

/**
 * Returns 2 if this is the theta axis of a Polar chart and
 * the angleUnit is RADIANS.  Otherwise, the return value is 0.
 * At the time that this method is invoked, the JCAxis.chartType
 * property may not have been set yet, so we pass in the chart type
 * as a parameter.
 */
private int getDefaultPrecision(int chartType) {
    int defPrec = 0;
    if (chartType == JCChart.POLAR && !i_vertical) {
        //  If the theta axis of a Polar chart...
        int angleUnit = chartArea.getAngleUnit();
        if (angleUnit == JCChartUtil.RADIANS) {
            defPrec = 2;
        }
    }
    return defPrec;
}

/**
 * Sets the MaxPrecision property of the Axis, which controls
 * the maximum value of the Precision property.  The default value is 14.
 * @param maxprec the maximum value the Precision property can take on.
 */
public void setMaxPrecision(int maxprec)
{
    maximumPrecision = maxprec;
}

/**
 * Gets the MaxPrecision property of the Axis, which controls
 * the maximum value of the Precision property.  The default value is 14.
 * @return the maximum value the Precision property can take on.
 */
public int getMaxPrecision()
{
    return(maximumPrecision);
}

/**
 * Gets the PrecisionIsDefault property of JCAxis. The
 * PrecisionIsDefault determines whether Chart is responsible
 * for calculating the numbering precision.  If true, Chart will
 * calculate the precision.  If false, Chart will use the provided
 * precision.
 * @return true tells Chart to calculate the Precision property,
 * whereas false means that the Precision property is already specified
 */
public boolean getPrecisionIsDefault()
{
    DerivedInt prec = getPrecisionObject();
    return(prec.isDefault);
}

/**
 * This method is invoked by the setChartType() method of
 * ChartDataView.  When switching between chart types, we
 * need to update the default annotation precision.
 */
protected void updateAnnoDefaultPrecision(int chartType) {
    if (annoList != null) {
        JCAnno anno = null;
        int nObjects = annoList.size();
        for (int i=0; i < nObjects; i++) {
            anno = (JCAnno)annoList.get(i);
            if (anno != null) {
                int defPrec = getDefaultPrecision(chartType);
                anno.setDefaultPrecision(defPrec);
                // Now update the DerivedInt value.
                if (anno.precision.isDefault) {
                    anno.precision.value = defPrec;
                }
            }
        }
    }
}

/**
 * Sets the NumSpacing property of the Axis, which controls
 * the number interval of the axis labels. NumSpacing is only meant to
 * work with Value or ValueLabels annotations. This method has no
 * effect when time labels or point labels are being used.
 * @param inc interval between axis labels
 */
public void setNumSpacing(double inc)
{
    numSpacing.value = inc;
    numSpacing.isDefault = false;

    JCAnno labels = getDefaultLabels();
    if (labels != null && labels.incrementValue.isDefault) {
        labels.incrementValue.value = inc;
    }

    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the NumSpacing property of the Axis, which controls
 * the interval between axis labels. NumSpacing is only meant to
 * work with Value or ValueLabels annotations. This method has no
 * effect when time labels or point labels are being used.
 * @return interval between axis labels
 */
public double getNumSpacing()
{
    return(numSpacing.value);
}

/**
 * Sets the NumSpacingIsDefault property of JCAxis. The
 * NumSpacingIsDefault determines whether Chart is responsible
 * for calculating the numbering spacing. If true, Chart will
 * calculate the spacing. If false, Chart will use the provided
 * numbering spacing.
 * @param useDef true tells Chart to calculate the NumSpacing property,
 * whereas false means the NumSpacing property is already specified
 */
public void setNumSpacingIsDefault(boolean useDef)
{
    numSpacing.isDefault = useDef;

    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the NumSpacingIsDefault property of JCAxis.  The
 * NumSpacingIsDefault determines whether Chart is responsible
 * for calculating the numbering spacing.  If true, Chart will
 * calculate the spacing.  If false, Chart will use the provided
 * numbering spacing.
 * @return true tells Chart to calculate the NumSpacing property,
 * whereas false means the NumSpacing property is already specified
 */
public boolean getNumSpacingIsDefault()
{
    if (!this.numSpacing.isDefault) {
        return false;
    }
    JCAnno anno = getDefaultLabels();
    boolean fDefault = false;
    if (anno != null) {
        fDefault = anno.getIncrementIsDefault();
    }
    return fDefault;
}

/**
 * Gets the TickSpacing property, which controls the interval
 * between tick lines on the axis. Note: if the
 * AnnotationMethod property is set to POINT_LABELS, tick
 * lines appear at point values.
 * @return interval between ticks
 */
public double getTickSpacing()
{
    return(tickSpacing.value);
}

/**
 * Sets the TickSpacing property, which controls the interval
 * between tick lines on the axis.
 * @param tick interval between ticks
 */
public void setTickSpacing(double tick)
{
    this.tickSpacing.value = tick;
    this.tickSpacing.isDefault = false;
    // If the tickSpacing is non-default, draw ticks specified
    // by the JCAnno objects when the annotation method is
    // TIME_LABELS, POINT_LABELS, or VALUE_LABELS.  If useAnnoTicks
    // is false, draw ticks at the positions of the labels.  useAnnoTicks
    // is ignored when the annotation method is VALUE.  In that
    // case, (visible) ticks specified by the JCAnno objects are always
    // drawn.
    this.useAnnoTicks = true;

    if (this.numSpacing.isDefault) {
        setNumSpacing(tick);
    }

    JCAnno ticks = getDefaultTicks();
    if (ticks != null && ticks.incrementValue.isDefault) {
        // JCAxis.setTickSpacing() is retained for backwards compatibility.
        // If the user has set the incrementValue property of a
        // JCAnno to a nondefault value, we shouldn't overwrite it.
        // So we change the value of tick.incrementValue only if its
        // isDefault flag is true.  RE (19/01/04)
        ticks.incrementValue.value = tick;
    }

    setChanged(true, Changeable.RECALC);
}

/**
 * Sets the TickSpacingIsDefault property of JCAxis. The
 * TickSpacingIsDefault determines whether Chart is responsible
 * for calculating the tick spacing. If true, Chart will
 * calculate the tick spacing. If false, Chart will use the provided
 * tick spacing.
 * @param useDef true tells Chart to calculate the TickSpacing property,
 * whereas false means that the TickSpacing property is already specified
 */
public void setTickSpacingIsDefault(boolean useDef)
{
    this.tickSpacing.isDefault = useDef;
    this.useAnnoTicks = !useDef;
    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the TickSpacingIsDefault property of JCAxis.  The
 * TickSpacingIsDefault determines whether Chart is responsible
 * for calculating the tick spacing. If true, Chart will
 * calculate the tick spacing. If false, Chart will use the provided
 * tick spacing.
 * @return true tells Chart to calculate the TickSpacing property,
 * whereas false means that the TickSpacing property is already specified
 */
public boolean getTickSpacingIsDefault() {
    if (!this.tickSpacing.isDefault) {
        return false;
    }
    JCAnno ticks = getDefaultTicks();
    boolean fDefault = false;
    if (ticks != null) {
        fDefault = ticks.getIncrementIsDefault();
    }
    return fDefault;
}

/**
 * Sets the value of the LabelGenerator property.  The LabelGenerator
 * property holds a reference to an object that implements the
 * JCLabelGenerator interface.  This interface is used to externally
 * generate labels.
 * @param lg reference to object instance used to generate labels.
 */
public void setLabelGenerator(JCLabelGenerator lg) {
    if (labelGenerator == lg) return;
    labelGenerator = lg;
    numberLocalization = false;

    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the value of the LabelGenerator property.  The LabelGenerator
 * property holds a reference to an object that implements the
 * JCLabelGenerator interface.  This interface is used to externally
 * generate labels.
 * @return reference to object instance used to generate labels.
 */
public JCLabelGenerator getLabelGenerator()
{
    return(labelGenerator);
}

/**
 * Sets the value of the TimeUnit property.
 * The TimeUnit property controls the unit of time
 * used for labelling a time labelled axis.
 * @param tu Unit of time; one of SECONDS, MINUTES,
 * HOURS, DAYS, WEEKS, MONTHS or YEARS.
 */
public void setTimeUnit(long tu)
{
    if (timeUnit == tu) {
        return;
    }
    if (tu == SECONDS || tu == MINUTES || tu == HOURS ||
        tu == DAYS || tu == WEEKS || tu == MONTHS || tu == YEARS)
    {
        timeUnit = tu;
    }
    else {
        throw new IllegalArgumentException("Invalid TimeUnit value in JCAxis");
    }

    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the value of the TimeUnit property.
 * The TimeUnit property controls the unit of time
 * used for labelling a time labelled axis.
 * @return Unit of time; one of SECONDS, MINUTES,
 * HOURS, DAYS, WEEKS, MONTHS or YEARS.
 */
public long getTimeUnit()
{
    return(timeUnit);
}

/**
 * Sets the value of the TimeBase property.
 * The TimeBase property defines the start time for the
 * axis.
 * @param nb Date object representing the start time for the
 * axis.
 */
public void setTimeBase(Date nb)
{
    if (nb == null) {
        // Nothing happens but we reset the isTimeBaseSet flag
        isTimeBaseSet = false;
        return;
    }
    if (timeBase != null && timeBase.equals(nb)) {
        return;
    }
    timeBase = nb;
    isTimeBaseSet = true;

    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the value of the TimeBase property.
 * The TimeBase property defines the start time for the
 * axis.
 * @return Date object representing the start time for the
 * axis.
 */
public Date getTimeBase()
{
    if (timeBase == null) {
        timeBase = new Date();
    }
    return(timeBase);
}

/**
 * Convert the time base to a time label string.
 * @param useTimeFormat if true, will use TimeFormat property in generating
 *                      label.  If false, will use default format.
 */
public String timeBaseToString(boolean useTimeFormat)
{
    JCChartDateFormat df = annoHandler.getDateFormatter();
    return(useTimeFormat ? df.timeLabel(timeFormat.value, getTimeBase())
                         : df.timeLabel(getTimeBase()));
}

/**
 * Sets the value of the TimeFormat property
 * The TimeFormat property controls the format used to
 * generate time labels for time labelled axes. This is JDK 1.1 compliant.
 * Uses the date and time format patterns described in JDK 1.1 documentation
 * If this date and time format is used then the old time format does not reflect the changes
 * @param tf format string for time labels.
 * @see java.text.SimpleDateFormat
 */
public void setTimeFormat(String tf)
{
    setTimeFormat(tf, true);
}

/**
 * Sets the value of the TimeFormat property and recalcs as specified.
 * @param tf format string for time labels.
 * @param update force a recalc if true.
 * @see java.text.SimpleDateFormat
 */
public void setTimeFormat(String tf, boolean update)
{
    if (timeFormat.value != null && timeFormat.value.equals(tf)) {
        return;
    }
    timeFormat.value = tf;
    timeFormat.isDefault = false;

    if (update) {
        setChanged(true, Changeable.RECALC);
    }
}

/**
 * Gets the value of the TimeFormat property
 * The TimeFormat property controls the format used to
 * generate time labels for time labelled axes.
 * @return format string for time labels.
 */
public String getTimeFormat()
{
    return(timeFormat.value);
}

/**
 * Sets the value of the TimeFormatIsDefault property.
 * The TimeFormatIsDefault property determines whether
 * a time label format is generated automatically, or the
 * user value for TimeFormat is used.
 * @param ud true if a default will be generated, false if
 * the user value will be used.
 */
public void setTimeFormatIsDefault(boolean ud)
{
    setTimeFormatIsDefault(ud, true);
}

/**
 * Sets the value of the TimeFormatIsDefault property and recalcs as specified.
 * @param ud true if a default will be generated, false if
 * the user value will be used.
 * @param update will recalc if true.
 */
public void setTimeFormatIsDefault(boolean ud, boolean update)
{
    if (timeFormat.isDefault == ud) {
        return;
    }
    timeFormat.isDefault = ud;

    if (update) {
        setChanged(true, Changeable.RECALC);
    }
}

/**
 * Gets the value of the TimeFormatIsDefault property.
 * The TimeFormatIsDefault property determines whether
 * a time label format is generated automatically, or the
 * user value for TimeFormat is used.
 * @return true if a default will be generated, false if
 * the user value will be used.
 */
public boolean getTimeFormatIsDefault()
{
    return(timeFormat.isDefault);
}


/**
 * Sets the value of the Vertical property.  The Vertical
 * property determines whether the axis is vertically oriented
 * or horizontally oriented.
 * @param v If true, the axis is vertical.  Otherwise, it is horizontal.
 */
public void setVertical(boolean v)
{
    if (vertical == v) {
        return;
    }
    vertical = v;
    i_vertical = v;

    setChanged(true, Changeable.RECALC);
}

protected void setVerticalInternal(boolean v)
{
    if (i_vertical == v) {
        return;
    }
    i_vertical = v;
    // Don't call setChanged().
}

/**
 * Gets the value of the Vertical property.  The Vertical
 * property determines whether the axis is vertically oriented
 * or horizontally oriented.
 * @return If true, the axis is vertical.  Otherwise, it is horizontal.
 */
public boolean isVertical()
{
    return(vertical);
}

/**
 * Sets the value of the Gap property.
 * The Gap property determines the amount of space left between
 * adjacent axis annotations, in pixels.
 * @param gp Space between labels. To ensure that the labels do not overlap,
 * <code>gp</code> should be &#62;=0. If overlapping labels is desired, <code>gp</code> can be &#60;0.
 */
public void setGap(int gp)
{
    if (gap == gp) {
        return;
    }
    gap = gp;

    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the value of the Gap property.
 * The Gap property determines the amount of space left between
 * adjacent axis annotations, in pixels.
 * @return Space between labels. To ensure that the labels do not overlap,
 * <code>gp</code> should be &#62;=0. If overlapping labels is desired, <code>gp</code> can be &#60;0.
 */
public int getGap()
{
    return(gap);
}

/**
 * Retrieves the value label for the specified value from a list of value
 * value labels store in a Vector.  Note that if there is no value label that
 * is an exact floating point match of the value, the value label with a value
 * closest to the value is returned.
 * @param val data value for which to find the value label
 * @return JCValueLabel instance.  Returns null if there are no labels.
*/
private JCValueLabel getValueLabel(Vector vLabels, double val)
{
    if (vLabels == null) {
        return (null);
    }

    JCValueLabel ret = null;
    double distance = Double.MAX_VALUE;
    for (int i = 0; i < vLabels.size(); i++) {
        JCValueLabel vl = (JCValueLabel) vLabels.elementAt(i);
        // Exact match, we have our label
        if (val == vl.value) {
            ret = vl;
            break;
        }
        // Not exact, see if it is closer to the value than the previous label
        else {
            double diff = Math.abs(val - vl.value);
            if (diff < distance) {
                ret = vl;
                distance = diff;
            }
        }
    }
    return(ret);
}

/**
 * Retrieves the value label for the specified value from the list of
 * user-specified value labels.  Note that if there is no value label that is
 * an exact floating point match of the value, the value label with a value
 * closest to the value is returned.
 * @param val data value for which to find the value label
 * @return JCValueLabel instance.  Returns null if there are no user-specified
 * value labels.
 */
public JCValueLabel getValueLabel(double val)
{
    return(getValueLabel(annoHandler.userValueLabels, val));
}

/**
 * Retrieves the value label for the specified value from the list of
 * value labels generated for this axis.  If the AnnotationMethod is
 * set to VALUE_LABELS, this list will be the same as the user-specified
 * list of value labels.  Note that if there is no value label that is an
 * exact floating point match of the value, the value label with a value
 * closest to the value is returned.
 * @param val data value for which to find the value label
 * @return JCValueLabel instance.  Returns null if there are no generated
 * value labels.
 */
public JCValueLabel getGeneratedValueLabel(double val)
{
    return(getValueLabel(annoHandler.valueLabels, val));
}

/**
 * Adds the provided value label to the list of value labels.
 * If the AnnotationMethod is VALUE_LABELS, then the value
 * label will appear at the appropriate point along the axis.
 * @param vl JCValueLabel instance to add
 */
public void addValueLabel(JCValueLabel vl)
{
    int duplicate = -1;
    int try_insert = -1;
    // If there is an existing label with the same value, replace it
    for (int i = 0; i < annoHandler.userValueLabels.size(); i++) {
        JCValueLabel lbl = (JCValueLabel)annoHandler.userValueLabels.elementAt(i);
        if (lbl.value > vl.value && try_insert < 0) {
            try_insert = i;
            // ignore comments below, we now assume the list is sorted
            break;
            // If we knew the valueLabel list was sorted, we could
            // break from the loop here.
            // For now, let's play it safe and keep looking for
            // a duplicate label.
        }
        if (lbl.value == vl.value) {
            duplicate = i;
            break;
        }
    }
    if (duplicate >= 0) {
        annoHandler.userValueLabels.setElementAt(vl, duplicate);
    }
    else if (try_insert >= 0) {
        annoHandler.userValueLabels.insertElementAt(vl, try_insert);
    }
    else {
        annoHandler.userValueLabels.addElement(vl);
    }
    vl.setParent(this);
    // re-calc, re-sort and re-draw
    setChanged(true, Changeable.RECALC);
}

/**
 * Intended for internal use only.
 */
public void removeValueLabel(int index) {
    annoHandler.userValueLabels.remove(index);
    // re-calc, re-sort and re-draw
    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the value of the ValueLabels property.
 * The ValueLabels property is an indexed property containing
 * a list of all annotations specified by the user for an axis.
 */
public JCValueLabel[] getValueLabels()
{
    if (annoHandler.userValueLabels == null ||
        annoHandler.userValueLabels.size() == 0)
    {
        return(null);
    }
    JCValueLabel[] vl = new JCValueLabel[annoHandler.userValueLabels.size()];
    for (int i = 0; i < vl.length; i++) {
        vl[i] = (JCValueLabel)(annoHandler.userValueLabels.elementAt(i));
    }
    return(vl);
}

/**
 * Intended for internal use only.
 */
public Vector getUserValueLabels() {
    return(annoHandler.userValueLabels);
}

/**
 * Gets the list of value labels generated for this axis.  If the
 * AnnotationMethod is set to VALUE_LABELS, this list will be the
 * same as the user-specified list of value labels.
 */
public JCValueLabel[] getGeneratedValueLabels()
{
    if (annoHandler.valueLabels == null ||
        annoHandler.valueLabels.size() == 0)
    {
        return(null);
    }
    JCValueLabel[] vl = new JCValueLabel[annoHandler.valueLabels.size()];
    for (int i = 0; i < vl.length; i++) {
        vl[i] = (JCValueLabel)(annoHandler.valueLabels.elementAt(i));
    }
    return(vl);
}

/**
 * Sets the value of the ValueLabels property.
 * The ValueLabels property is an indexed property containing
 * a list of all annotations specified by the user for an axis.
 */
public void setValueLabels(JCValueLabel[] vl)
{
    annoHandler.userValueLabels = new Vector();
    if (vl == null) {
        return;
    }
    for (int i = 0; i < vl.length; i++) {
        if (vl[i] != null) {
            vl[i].setParent(this);
        }
        annoHandler.userValueLabels.addElement(vl[i]);
    }
    annoHandler.sortValueLabels();
    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the value of the ValueLabels property at a specified index.
 * The ValueLabels property is an indexed property containing
 * a list of all annotations specified by the user for an axis.
 */
public JCValueLabel getValueLabels(int indx)
{
    if (annoHandler.userValueLabels == null ||
        annoHandler.userValueLabels.size() == 0)
    {
        return(null);
    }
    JCValueLabel vl = null;
    try {
        vl = (JCValueLabel)annoHandler.userValueLabels.elementAt(indx);
    }
    catch (Exception e) {}
    return(vl);
}

/**
 * Gets the value label at the specified index in the list of value labels
 * generated for this axis.  If the AnnotationMethod is set to VALUE_LABELS,
 * this list will be the same as the user-specified list of value labels.
 */
public JCValueLabel getGeneratedValueLabels(int indx)
{
    if (annoHandler.valueLabels == null ||
        annoHandler.valueLabels.size() == 0)
    {
        return(null);
    }
    JCValueLabel vl = null;
    try {
        vl = (JCValueLabel)annoHandler.valueLabels.elementAt(indx);
    }
    catch (Exception e) {}
    return(vl);
}

/**
 * Sets the value of one one value label in the ValueLabels property.
 * The ValueLabels property is an indexed property containing
 * a list of all annotations specified by the user for an axis.
 * This method is required by the beanbox, but somewhat
 * ill-formed because the index specified will not
 * necessarily be honoured because the ValueLabel list
 * is sorted by value.  It is not recommended for use.
 */
public void setValueLabels(int index, JCValueLabel vl)
{
    if (annoHandler.userValueLabels == null) {
        annoHandler.userValueLabels = new Vector();
    }
    if (index < 0 || index > annoHandler.userValueLabels.size()) {
        throw new IllegalArgumentException(
            "Invalid ValueLabels index in JCAxis");
    }
    if (vl != null) {
        vl.setParent(this);
    }
    if (index < annoHandler.userValueLabels.size()) {
        annoHandler.userValueLabels.setElementAt(vl, index);
    }
    else {
        annoHandler.userValueLabels.insertElementAt(vl, index);
    }

    setChanged(true, Changeable.RECALC);
}

/**
 * String representation of the axis.  Used by
 * com.klg.jclass.beans.IndexedPropertyEditor to name the
 * axis.
 */
public String toString()
{
    String s = getName();
    return (s == null) ?
        ( JCChartBundle.string(JCChartBundle.key141, locale) +
        (i_vertical ?  JCChartBundle.string(JCChartBundle.key147, locale) :
        JCChartBundle.string(JCChartBundle.key44, locale) ) + " " +
        JCChartBundle.string(JCChartBundle.key6, locale) ) : s;
}

/**
 * Converts a data value along the axis to a pixel value. The
 * pixel value is valid inside the parent JCChartArea.
 * @param v data value to convert to a screen location
 * @return screen location in pixels
 */
public int toPixel(double v)
{
    AxisHandler axisHandler = getAxisHandler();
    if (axisHandler instanceof RectangularChartAxisHandler) {
        return((RectangularChartAxisHandler)axisHandler).toPixel(v);
    }
    return Integer.MAX_VALUE;
}

/**
 * Converts a data value along the axis to a pixel value. The
 * pixel value is valid inside the parent JCChartArea.
 * @param v data value to convert to a screen location
 * @return screen location in pixels
 */
public double toPixelDouble(double v)
{
    AxisHandler axisHandler = getAxisHandler();
    if (axisHandler instanceof RectangularChartAxisHandler) {
        return((RectangularChartAxisHandler)axisHandler).toPixelDouble(v);
    }
    return Double.MAX_VALUE;
}

/**
 * Converts polar data (angle,radius) to pixel (x,y) coordinates. The
 * pixel value is valid inside the parent JCChartArea.
 *
 * @param angle theta value in radians
 * @param r radius in data or pixel units
 * @param convertDataToPixel false if r is in pixel units.
 * @return (x,y) chart area location in pixels.  Returns null if r axis doesn't exist.
 */
public Point toPixelPolar(double angle, double r, boolean convertDataToPixel)
{
    Point location = null;
    if (i_vertical) {   // if this is the r axis
        AxisHandler axisHandler = getAxisHandler();
        if (axisHandler instanceof CircularChartAxisHandler) {
            location = ((CircularChartAxisHandler)
                axisHandler).toPixelPolar(angle, r, convertDataToPixel);
        }
    }
    else if (otherAxis != null) {   // if r axis exists
        AxisHandler axisHandler = otherAxis.getAxisHandler();
        if (axisHandler instanceof CircularChartAxisHandler) {
            location = ((CircularChartAxisHandler)
                axisHandler).toPixelPolar(angle, r, convertDataToPixel);
        }
    }
    return(location);
}

/**
 * Convert from data to pixel units (i.e., apply scaling factor
 * only, no translation).
 *
 * @param dataValue Data value to be scaled.
 * @return The scaled value in pixel units.  Double.MAX_VALUE if
 * the r axis doesn't exist.
 */
protected double applyDataToPixelScaling(double dataValue)
{
    double scaledValue = Double.MAX_VALUE;
    if (i_vertical) {   // if this is the r axis
        scaledValue = getAxisHandler().scale * dataValue;
    }
    else if (otherAxis != null) {   // if r axis exists
        scaledValue = otherAxis.getAxisHandler().scale * dataValue;
    }
    return(scaledValue);
}

/**
 * Convert from pixel to data units (i.e., apply scaling factor
 * only, no translation).
 *
 * @param pixels Pixel value to be scaled.
 * @return The scaled value in data units.  Double.MAX_VALUE if
 * the r axis doesn't exist.
 */
protected double applyPixelToDataScaling(double pixels)
{
    double scaledValue = Double.MAX_VALUE;
    if (i_vertical) {   // if this is the r axis
        scaledValue = pixels / getAxisHandler().scale;
    }
    else if (otherAxis != null) {   // if r axis exists
        scaledValue = pixels / otherAxis.getAxisHandler().scale;
    }
    return(scaledValue);
}

/**
 * Converts a data value along the axis to a pixel value, shifting the
 * value to the front or back plane of the chart from the plane this
 * axis is on. The pixel value is valid inside the parent JCChartArea.
 * @param v data value to convert to a screen location
 * @param drawFront true if the conversion to the front plane is desired,
 *                  false if the conversion to the back plane is desired.
 * @return screen location in pixels
 */
public int toPixel(double v, boolean drawFront)
{
    // Axis is on front, but we want the corresponding pixel on the back plane
    int delta = (i_vertical ? chartArea.getDeltaY()
                            : parentChart.getChartArea().getDeltaX());
    if (frontPlane && !drawFront) {
        return(toPixel(v) + delta);
    }
    // Axis is on back, but we want the corresponding pixel on the front plane
    else if (!frontPlane && drawFront) {
        return(toPixel(v) - delta);
    }
    // Axis and pixel are on the same plane.
    else {
        return(toPixel(v));
    }
}

/**
 * Converts a data value along the axis to a pixel value, shifting the
 * value to the front or back plane of the chart from the plane this
 * axis is on. The pixel value is valid inside the parent JCChartArea.
 * @param v data value to convert to a screen location
 * @param drawFront true if the conversion to the front plane is desired,
 *                  false if the conversion to the back plane is desired.
 * @return screen location in pixels
 */
public double toPixelDouble(double v, boolean drawFront)
{
    // axis is on front, but we want the corresponding pixel on the back plane
    if (frontPlane && !drawFront) {
        return toPixelDouble(v) +
            (i_vertical ? chartArea.getDeltaY() : parentChart.getChartArea().getDeltaX());
    // axis is on back, but we want the corresponding pixel on the front plane
    }
    else if (!frontPlane && drawFront) {
        return toPixelDouble(v) -
            (i_vertical ? chartArea.getDeltaY() : parentChart.getChartArea().getDeltaX());
    // axis and pixel are on the same plane.
    } else {
        return toPixelDouble(v);
    }
}

/**
 * Converts a pixel value relative to the parent JCChartArea to
 * a value in the data space. In effect, this routine performs
 * mapping along a particular axis.
 * @param v screen value (in pixels) to convert to a data value.  Must
 * be relative to the plot area.
 * @return data value corresponding to the screen location
 */
public double toData(int v)
{
    AxisHandler axisHandler = getAxisHandler();
    if (axisHandler instanceof RectangularChartAxisHandler) {
        return((RectangularChartAxisHandler)axisHandler).toData(v);
    }
    return Double.MAX_VALUE;
}

/**
 * Converts a pixel value relative to the parent JCChartArea to
 * a value in the data space, first shifting the value from the front or
 * back plane of the chart to the plane that holds this axis. In effect,
 * this routine performs mapping along a particular axis.
 * @param v screen value (in pixels) to convert to a data value.  Must
 * be relative to the plot area.
 * @param drawFront true if the conversion from the front plane is desired,
 * false if the conversion from the back plane is desired.
 * @return data value corresponding to the screen location
 */
public double toData(int v, boolean drawFront)
{
    // axis is on front, but we want the corresponding pixel on the back plane
    if (frontPlane && !drawFront) {
        return toData(v -
            (i_vertical ? chartArea.getDeltaY() : parentChart.getChartArea().getDeltaX()));
    // axis is on back, but we want the corresponding pixel on the front plane
    }
    else if (!frontPlane && drawFront) {
        return toData(v +
            (i_vertical ? chartArea.getDeltaY() : parentChart.getChartArea().getDeltaX()));
    // axis and pixel are on the same plane.
    }
    else {
        return toData(v);
    }
}

/**
 * Converts a pixel value relative to the parent JCChartArea to
 * a value in the data space.
 * @param x screen value x coordinate.  Must be relative to the plot area.
 * @param y screen value y coordinate.
 * @return data value corresponding to the screen location.  Null if r axis doesn't exist.
 */
public JCDataCoord toDataPolar(int x, int y)
{
    JCDataCoord dataValue = null;
    if (i_vertical) {
        AxisHandler axisHandler = getAxisHandler();
        if (axisHandler instanceof CircularChartAxisHandler) {
            dataValue = ((CircularChartAxisHandler)
                          axisHandler).toDataPolar(x, y);
        }
    }
    else if (otherAxis != null) {
        AxisHandler axisHandler = otherAxis.getAxisHandler();
        if (axisHandler instanceof CircularChartAxisHandler) {
            dataValue = ((CircularChartAxisHandler)
                          axisHandler).toDataPolar(x, y);
        }
    }
    return(dataValue);
}

/**
 * Overrides ChartInteriorRegion.draw() to ensure that a border is not
 * drawn around the axis.
 */
public void draw(Graphics gc)
{
    if (!visible) {
        return;
    }
    if (gc == null) {
        return;
    }

    DefineShape shapeDef = null;
    if (gc instanceof DefineShape) {
        shapeDef = (DefineShape) gc;
        shapeDef.beginShape();
    }

    axisDrawn = false;
    if (i_vertical) {
        axisDrawn = drawVertical(gc);
    }
    else {
        axisDrawn = drawHorizontal(gc);
    }

    if (shapeDef != null) {
        shapeDef.endShape(true);
    }
}

public void drawAnnotationsAndTitle(Graphics gc)
{
    drawAnnotationsAndTitle(gc, true, true);
}

public void drawAnnotationsAndTitle(Graphics gc, boolean drawAnno,
                                    boolean drawTitle)
{
    if (!visible || (!drawAnno && !drawTitle)) {
        return;
    }
    //Graphics localGC = clipToExtents(gc);
    Graphics localGC = (gc == null) ? null : gc.create();
    if (axisDrawn) {
        if (drawAnno) {
            annoHandler.draw(localGC);
        }
        if (drawTitle && title != null && !isPolar() && !isRadar()) {
            title.draw(localGC);
        }
    }
    if (localGC != null) {
        localGC.dispose();
    }
}

/**
 * Overridden setBounds() method that makes sure the scale and shift
 * values are recalculated when the axis size or position changes.
 */
public void setBounds(int newx, int newy, int neww, int newh)
{
    if (getLeft() == newx && getTop() == newy &&
        getWidth() == neww && getHeight() == newh)
    {
        return;
    }
    super.setBounds(newx, newy, neww, newh);
    setChanged(true, Changeable.RECALC, false);
    calcTransformation();
}

/**
 * Recalculates the axis parameters. This method is just a placeholder
 * for now, since axis recalculation is distributed inside recalc()
 * for JCChartArea.
 */
public void recalc()
{
    setChanged(false, Changeable.NO_CHANGE, false);
}

/**
 * Converts a date to a value along this axis, taking into consideration
 * the current TimeBase and TimeUnit properties.
 * @param d date to convert
 * @return value along this axis represented by specified date.
 */
public double dateToValue(Date d)
{
    // Guarantees that time base has been allocated.
    getTimeBase();
    return(annoHandler.timeToData(d, timeUnit));
}

/**
 * Converts a value along this axis to a Date, taking into consideration
 * the current TimeBase and TimeUnit properties.
 * @param v value to convert
 * @return date represented by specified value.
 */
public Date valueToDate(double v)
{
    // Guarantees that time base has been allocated.
    getTimeBase();
    return(annoHandler.dataToTime(v, timeUnit));
}

/********
 * Package-level methods
 */

/** Calc the axis trough if the axis is 3D.  It is stored in 'sideplane'
 *  for later use.
 */
void preDrawCalc()
{
    if (!visible) {
        return;
    }

    if (isBar() && is3D()) {
        // Calc sideplane
        int x[] = new int[5];
        int y[] = new int[5];

        if (i_vertical) {
            x[0] = x[1] = linePosition();
            y[0] = getTop()+getHeight();
            y[1] = getTop();
            x[3] = x[2] = x[1] + chartArea.getDeltaX();
            y[2] = y[1] + chartArea.getDeltaY();
            y[3] = y[0] + chartArea.getDeltaY();
        }
        else {
            x[0] = getLeft()+getWidth();
            x[1] = getLeft();
            y[0] = y[1] = linePosition();
            x[2] = x[1] + chartArea.getDeltaX();
            y[2] = y[3] = y[1] + chartArea.getDeltaY();
            x[3] = x[0] + chartArea.getDeltaX();
        }
        x[4] = x[0];
        y[4] = y[0];

        sideplane = new Polygon();
        for (int i = 0; i < 5; i++) {
            sideplane.addPoint(x[i], y[i]);
        }
    }
}

/** Draws the 3D axis trough. If 'all' is true, the entire trough is
 * drawn regardless of the value of 'start'.  If 'all' is false,
 * the value of start will determine which portion of the trough will
 * be drawn.  If 'start' is true, the portion up to the point where the
 * axis is intersected by the other axis will be drawn.  If 'start' is
 * false, the portion after that point will be drawn.
 */
void preDraw(Graphics gc, boolean all, boolean start)
{
    if (!visible) {
        return;
    }

    if (isBar() && is3D()) {
        Color fillColor = null;
        if (chartArea.isOpaque()) {
            fillColor = chartArea.getPlotArea().getBackground();
        }
        else {
            // Traverse the component hierarchy looking for a opaque component
            // to give us it's background color
            Component caParent = chartArea.getParent();
            while (caParent != null               &&
                   caParent instanceof JComponent &&
                   !caParent.isOpaque())
            {
                caParent = caParent.getParent();
            }
            if (caParent != null                &&
                caParent instanceof JComponent  &&
                caParent.isOpaque())
            {
                fillColor = caParent.getBackground();
            }
            else if (caParent != null && !(caParent instanceof JComponent)) {
                fillColor = caParent.getBackground();
            }
        }

        if (fillColor == null) {
            return;
        }

        JCFillStyle fs = new JCFillStyle(fillColor, JCFillStyle.SOLID);
        if (all) {
            if ((!i_vertical && chartArea.getDeltaY() <= 0) ||
                (i_vertical && chartArea.getDeltaX() <= 0))
            {
                fs.fillPolygonHighlight(gc, sideplane);
            }
            else {
                fs.fillPolygonShadow(gc, sideplane);
            }
        }
        else {
            int otherPos = otherAxis.linePosition();
            if (start) {
                Polygon splitSidePlane1 = new Polygon(sideplane.xpoints,
                                    sideplane.ypoints, sideplane.npoints);
                if (i_vertical) {
                    splitSidePlane1.ypoints[1] = otherPos;
                    splitSidePlane1.ypoints[2] = otherPos +
                                                    chartArea.getDeltaY();
                }
                else {
                    splitSidePlane1.xpoints[0] = otherPos;
                    splitSidePlane1.xpoints[4] = otherPos;
                    splitSidePlane1.xpoints[3] = otherPos +
                                                    chartArea.getDeltaX();
                }
                if ((!i_vertical && chartArea.getDeltaY() <= 0) ||
                    (i_vertical && chartArea.getDeltaX() <= 0))
                {
                    fs.fillPolygonHighlight(gc, splitSidePlane1);
                }
                else {
                    fs.fillPolygonShadow(gc, splitSidePlane1);
                }
            }
            else {
                Polygon splitSidePlane2 = new Polygon(sideplane.xpoints,
                                                      sideplane.ypoints,
                                                      sideplane.npoints);
                if (i_vertical) {
                    splitSidePlane2.ypoints[0] = otherPos;
                    splitSidePlane2.ypoints[4] = otherPos;
                    splitSidePlane2.ypoints[3] = otherPos +
                                                    chartArea.getDeltaY();

                }
                else {
                    splitSidePlane2.xpoints[1] = otherPos;
                    splitSidePlane2.xpoints[2] = otherPos +
                                                    chartArea.getDeltaX();
                }
                if ((!i_vertical && chartArea.getDeltaY() <= 0) ||
                    (i_vertical && chartArea.getDeltaX() <= 0))
                {
                    fs.fillPolygonHighlight(gc, splitSidePlane2);
                }
                else {
                    fs.fillPolygonShadow(gc, splitSidePlane2);
                }
            }
        }
    }
}

/** Draw the outline of the 3D axis trough. */
void preDrawOutlineTrough(Graphics gc)
{
    if (!visible) {
        return;
    }

    if (isBar() && is3D()) {
        // temporarily increase the clip by 1 so that
        // the trough will line up with the bounding box
        Rectangle defClipRect = gc.getClipBounds();
        gc.setClip(defClipRect.x - 1, defClipRect.y - 1,
                   defClipRect.width + 2, defClipRect.height + 2);
        JCFillStyle fs =
            new JCFillStyle(chartArea.getPlotArea().getBackground(),
                            JCFillStyle.SOLID);
        fs.drawOutlinePolygon(gc, sideplane, getForeground());
        gc.setClip(defClipRect.x, defClipRect.y,
                   defClipRect.width, defClipRect.height);
    }
}

final static int TROUGH_TOP = 1;
final static int TROUGH_BOTTOM = 2;
final static int TROUGH_LEFT = TROUGH_TOP;
final static int TROUGH_RIGHT = TROUGH_BOTTOM;

/** Draws the specified sides out the 3D Axis Trough.
 *  @param gc the graphics context to draw with.
 *  @param side Or'ed values of TROUGH_TOP, TROUGH_BOTTOM, TROUGH_LEFT,
 *  and TROUGH_RIGHT.  Used to specify which side of trough to draw.
 */
void postDrawOutlineTrough(Graphics gc, int side)
{
    if (!visible) {
        return;
    }

    if (isBar() && is3D() && sideplane != null) {
        // temporarily increase the clip by 1 so that
        // the trough will line up with the bounding box
        Rectangle defClipRect = gc.getClipBounds();
        gc.setClip(defClipRect.x - 1, defClipRect.y - 1,
                    defClipRect.width + 2, defClipRect.height + 2);
        JCFillStyle fs =
            new JCFillStyle(chartArea.getPlotArea().getBackground(),
                            JCFillStyle.SOLID);
        boolean isInverted = (isPrimary && i_vertical) ||
            (!isPrimary && !i_vertical);

        if ((!isInverted && !reversed && (side & TROUGH_TOP) != 0) ||
            (!isInverted && reversed && (side & TROUGH_BOTTOM) != 0) ||
            (isInverted &&  !reversed && (side & TROUGH_BOTTOM) != 0) ||
            (isInverted &&  reversed && (side & TROUGH_TOP) != 0))
        {
            fs.drawOutlineLine(gc, sideplane, 1, 2, getForeground());
        }
        if ((!isInverted && !reversed && (side & TROUGH_BOTTOM) != 0) ||
            (!isInverted && reversed && (side & TROUGH_TOP) != 0) ||
            (isInverted && !reversed && (side & TROUGH_TOP) != 0) ||
            (isInverted && reversed && (side & TROUGH_BOTTOM) != 0))
        {
            fs.drawOutlineLine(gc, sideplane, 3, 4, getForeground());
        }
        gc.setClip(defClipRect.x, defClipRect.y,
                   defClipRect.width, defClipRect.height);
    }
}

/**
 * Resets the data limits.
 */
void clearLimits()
{
    dataLimits = null;
}

void setLimits(MinMax newLimits)
{
    if (dataLimits == null) {
        dataLimits = new MinMax();
    }
    dataLimits.union(newLimits);
    if (dataLimits.min > dataLimits.max) {
        dataLimits = null;
    }
}

/**
 * Sets the size of this axis to an initial size for calculations to
 * work with.  This size will be changed by annotation calculations.
 */
void setInitialSize()
{
    if (i_vertical) {
        setBounds(0, 0, 30, chartArea.getSize().height);
    }
    else {
        setBounds(0, chartArea.getSize().height-30,
                  chartArea.getSize().width, 30);
    }
}

/**
 * Makes an educated guess at the precision, tick spacing and numbering
 * spacing
 */
void calcAxisParams(boolean primary)
{
    // Removed - calculation should occur if axis not shown because
    // related axis may rely on results
    //if (!visible) return;

    setInitialSize();

    // Calculate whether this axis is a primary axis, and what its
    // type is, based on the chart data view(s) to which it is connected.
    chartType = JCChart.NONE;
    JCAxis target = this;
    boolean related = (formula != null && formula.getOriginator() != null);
    if (related) {
        target = formula.getOriginator();
    }
    hasBar = false;
    boolean foundAxis = false;
    for (Iterator i = getParentChart().getOriginalDataView().iterator();
         i.hasNext(); )
    {
        ChartDataView cdv = (ChartDataView)i.next();
        usingDoublePixels = cdv.isUsingDoublePixels();
        if (cdv.getXAxis() == target) {
            int thisChartType = cdv.getChartType();
            if (!foundAxis) {
                chartType = thisChartType;
                isPrimary = !related;
                otherAxis = cdv.getYAxis();
                foundAxis = true;
            }
            // mark axis if it has a 3d type (i.e. a bar chart type)
            // related axes aren't marked so that they'll appear
            // on the backplane
            if (!related && (thisChartType == JCChart.BAR ||
                thisChartType == JCChart.STACKING_BAR))
            {
                hasBar = true;
            }
        }
        if (cdv.getYAxis() == target) {
            int thisChartType = cdv.getChartType();
            if (!foundAxis) {
                chartType = thisChartType;
                isPrimary = false;
                otherAxis = cdv.getXAxis();
                foundAxis = true;
            }
            // mark axis if it has a 3d type (i.e. a bar chart type)
            // related axes aren't marked so that they'll appear
            // on the backplane
            if (!related && (thisChartType == JCChart.BAR ||
                thisChartType == JCChart.STACKING_BAR))
            {
                hasBar = true;
            }
        }
    }
    if (!hasBar && is3D()) {
        frontPlane = false;
    }
    else {
        frontPlane = true;
    }

    getAxisHandler().calcParams();

    // Not very interesting
    recalc();
}

/**
 * Checks to see if all the annotations will fit, and makes the
 * appropriate adjustments.
 */
void checkAnnotations()
{
    if (!visible) {
        return;
    }
    annoHandler.checkAnnotations();
}

/**
 * Makes an educated guess regarding the location of the annotations
 * relative to the axis.
 */
void calcAnchors()
{
    if (!visible) {
        return;
    }
    annoHandler.calcAnchor();
    if (title.visible) {
        title.recalc(true);
        title.fixPlacement(this);
    }
}

/**
 * Calculates the major, minor and orthogonal extents of the axis.
 */
void calcAnnotationExtents()
{
    annoHandler.calcAnnotationExtents(annoList);
}

/**
 * Returns true if the axis is on the right side of the chart.
 */
protected boolean isRightSide()
{
    boolean othReversed = false;
    if (otherAxis != null) {
        othReversed = otherAxis.reversed;
    }
    if (i_vertical) {
        if ((annoHandler.realPlacement == MAX && !othReversed) ||
            (annoHandler.realPlacement == MIN && othReversed))
        {
            return(true);
        }
    }
    return(false);
}

/**
 * Returns true if the axis is on the left side of the chart.
 */
protected boolean isLeftSide()
{
    boolean othReversed = false;
    if (otherAxis != null) {
        othReversed = otherAxis.reversed;
    }
    if (i_vertical) {
        if ((annoHandler.realPlacement == MIN && !othReversed) ||
            (annoHandler.realPlacement == MAX && othReversed))
        {
            return(true);
        }
    }
    return(false);
}

/**
 * Returns true if the axis is on the bottom of the chart.
 */
protected boolean isBottomSide()
{
    boolean othReversed = false;
    if (otherAxis != null) {
        othReversed = otherAxis.reversed;
    }
    if (!i_vertical) {
        if ((annoHandler.realPlacement == MIN && !othReversed) ||
            (annoHandler.realPlacement == MAX && othReversed))
        {
            return(true);
        }
    }
    return(false);
}

/**
 * Returns true if the axis is on the top of the chart.
 */
protected boolean isTopSide()
{
    boolean othReversed = false;
    if (otherAxis != null) {
        othReversed = otherAxis.reversed;
    }
    if (!i_vertical) {
        if ((annoHandler.realPlacement == MAX && !othReversed) ||
            (annoHandler.realPlacement == MIN && othReversed))
        {
            return(true);
        }
    }
    return(false);
}

/**
 * Special adjustment to make sure that non MIN/MAX axes have
 * enough room for their ortho extents. Useful for bar graphs.
 */
void adjustForOrthoExtent()
{
    if (!visible) {
        return;
    }
    if (annoHandler.realPlacement == MIN ||
        annoHandler.realPlacement == MAX)
    {
        return;
    }
    if (placementAxis == null) {
        return;
    }
    Rectangle pa = chartArea.getPlotRect();
    Rectangle total = chartArea.getChartAreaRect();
    int offset;

    if (i_vertical) {
        int x;
        if (annoHandler.realPlacement == VALUE_ANCHORED) {
            x = placementAxis.toPixel(placementLocation);
        }
        else {
            x = placementAxis.toPixel(placementAxis.origin.value);
        }

        // If axis outside of min/max of placementAxis,
        // axis won't be drawn, so don't do offsets
        if (x < placementAxis.getLeft() ||
            x > placementAxis.getLeft() + placementAxis.getWidth())
        {
            return;
        }

        if (x < annoHandler.extents.ortho) {
            // Squeezed at left side
            if (chartArea.getPlotArea().getLeftIsDefault()) {
                offset = annoHandler.extents.ortho - x;
                pa.width -= offset;
                pa.x += offset;
            }
            setSize(annoHandler.extents.ortho, getHeight());
        }
        else if (total.width-x < annoHandler.extents.ortho) {
            // Squeezed at right side
            if (chartArea.getPlotArea().getRightIsDefault()) {
                offset = annoHandler.extents.ortho - (total.width - x);
                pa.width -= offset;
            }
            setSize(annoHandler.extents.ortho, getHeight());
        }
    }
    else {
        int y;
        if (annoHandler.realPlacement == VALUE_ANCHORED) {
            y = placementAxis.toPixel(placementLocation);
        }
        else {
            y = placementAxis.toPixel(placementAxis.origin.value);
        }

        // if axis above or below min/max of placementAxis,
        // axis won't be drawn, so don't do offsets
        if (y < placementAxis.getTop() ||
            y > placementAxis.getTop() + placementAxis.getHeight())
        {
            return;
        }

        if (total.height-y < annoHandler.extents.ortho) {
            if (chartArea.getPlotArea().getBottomIsDefault()) {
                offset = annoHandler.extents.ortho - (total.height - y);
                pa.height -= offset;
            }
            setSize(getWidth(), annoHandler.extents.ortho);
        }
        else if (y < annoHandler.extents.ortho) {
            if (chartArea.getPlotArea().getTopIsDefault()) {
                offset = annoHandler.extents.ortho - y;
                pa.height -= offset;
                pa.y += offset;
            }
            setSize(getWidth(), annoHandler.extents.ortho);
        }
    }
}

protected void setPolarAxisParams(PolarAxisParams axisParams)
{
    polarAxisParams = axisParams;
}

protected PolarAxisParams getPolarAxisParams() {
    return polarAxisParams;
}

/**
 * Axis layout for Polar or Radar charts.  This method is called
 * twice.  In the first invocation, polarAxisParams properties are
 * initialized, and chartArea.recalc() is called.  While processing
 * chartArea.recalc(), layoutPolarAxis() gets called again.  We
 * don't want to reinitialize polarAxisParams or call chartArea.recalc()
 * again, so if polarAxisParams.radius is already equal to the
 * initial value, return immediately.
 */
protected void layoutPolarAxis()
{
    if (!isPolar() && !isRadar()) {
        return;
    }

    Rectangle chartAreaRect = chartArea.getChartAreaRect();
    double dWidth = (double)chartAreaRect.width;
    double dHeight = (double)chartAreaRect.height;

    double factor = 0.95;
    if (isRadar() && !polarAxisParams.isCircularGrid()) {
        // If a web-shaped Radar chart, use a larger scale factor
        factor = 1.414;  // square root of 2
    }
    double dRadius = factor * Math.min(dWidth, dHeight)/2.0;  // pixels

    // check that values are close -- that's good enough
    // otherwise we could end up in an endless loop
    if (Math.abs(polarAxisParams.radius - dRadius) < 1) {
        return;
    }

    polarAxisParams.radius = dRadius;
    polarAxisParams.xCenter = (double)chartAreaRect.x + dWidth/2.0;
    polarAxisParams.yCenter = (double)chartAreaRect.y + dHeight/2.0;

    // Call recalc on the chart area
    // For some reason, polar/radar axes need to have the recalc
    // called twice so that all parameters calculate correctly.
    // This needs to be fixed.
    chartArea.recalc();
}

/**
 * Adjusts the position and size of a Polar/Radar chart to keep all of the
 * annotation within view.  This method can be called for either the theta
 * axis, or r axis, but not both.
 *
 * @param xOffset x coordinate offset of chart center
 * @param yOffset y coordinate offset of chart center
 * @param radiusDelta Change in value of chart radius
 */
protected void adjustCenterAndRadius(int xOffset, int yOffset,
                                     int radiusDelta)
{
    polarAxisParams.xCenter += xOffset;
    polarAxisParams.yCenter += yOffset;
    polarAxisParams.radius += radiusDelta;

    //  Compute new scaling factors for the theta and r axes:
    getAxisHandler().calcTransformation();
}

/**
 * Returns an AnnoBoundsInfo object for this axis.
 */
protected AnnoBoundsInfo getPolarAnnotationBounds()
{
    AnnotationHandler annoHandler = getAnnotationHandler();
    if (annoHandler instanceof CircularChartAnnoHandler) {
        return ((CircularChartAnnoHandler)
                annoHandler).getPolarAnnotationBounds();
    }
    return null;
}

/**
 * Returns the pixel distance between the theta axis and a
 * theta axis label in a Polar/Radar chart.
 */
protected int getThetaAxisLabelOffset()
{
    if (i_vertical) {
        return ((CircularChartAnnoHandler)
            otherAxis.getAnnotationHandler()).thetaAxisLabelOffset;
    }
    else {
        return ((CircularChartAnnoHandler)
            getAnnotationHandler()).thetaAxisLabelOffset;
    }
}

/**
 * Basic layout that assumes we are dealing with a straight-line
 * axis.
 * @param last If true, this is the last axis layout.  This means that
 * 3D axes should be adjusted for the deltaX/Y values along their
 * minor dimension (ie move vertical axes by deltaX, move horizontal
 * axes by deltaY).
 */
void layoutAxis(boolean last)
{
    // Fix for bug 4072 - layout must occur even if axis is not visible
    //if (!visible) return;
    int hadj = Math.max(-chartArea.getDeltaX(), 0);
    int vadj = Math.max(-chartArea.getDeltaY(), 0);
    int ax = chartArea.getDeltaX();
    int ay = chartArea.getDeltaY();
    int fac = (last ? 1 : 0);
    if (i_vertical) {
        if (isPolar() || isRadar()) {
            //  layoutPolarAxis() called for x axis.  Don't call
            //  that method twice.
        }
        else if (!is3D()) {
            setBounds(getLeft(), chartArea.getPlotRect().y,
                      getWidth(), chartArea.getPlotRect().height);
        }
        else if (isBar()) {
            if (getLeft() < chartArea.getPlotRect().x) {
                setBounds(getLeft() + hadj * fac,
                          chartArea.getPlotRect().y + vadj,
                          getWidth(),
                          chartArea.getPlotRect().height-JCChartUtil.abs(ay));
            }
            else {
                hadj = Math.max(chartArea.getDeltaX(), 0);
                setBounds(getLeft() - hadj * fac,
                          chartArea.getPlotRect().y + vadj,
                          getWidth(),
                          chartArea.getPlotRect().height - JCChartUtil.abs(ay));
            }
        }
        else {
            vadj = Math.max(chartArea.getDeltaY(), 0);
            if (getLeft() <= chartArea.getPlotRect().x) {
                hadj = Math.max(chartArea.getDeltaX(), 0);
                setBounds(chartArea.getPlotRect().x + hadj * fac - getWidth(),
                          chartArea.getPlotRect().y + vadj,
                          getWidth(),
                          chartArea.getPlotRect().height - JCChartUtil.abs(ay));
            }
            else {
                setBounds(chartArea.getPlotRect().x +
                          chartArea.getPlotRect().width - hadj * fac,
                          chartArea.getPlotRect().y+vadj,
                          getWidth(),
                          chartArea.getPlotRect().height - JCChartUtil.abs(ay));
            }
        }
    }
    else {
        if (isPolar() || isRadar()) {
            layoutPolarAxis();
        }
        else if (!is3D()) {
            setBounds(chartArea.getPlotRect().x, getTop(),
                    chartArea.getPlotRect().width, getHeight());
        }
        else if (isBar()) {
            if (getTop() <= chartArea.getPlotRect().y) {
                setBounds(chartArea.getPlotRect().x+hadj, getTop()+vadj*fac,
                          chartArea.getPlotRect().width-JCChartUtil.abs(ax),
                          getHeight());
            }
            else {
                vadj = Math.max(chartArea.getDeltaY(), 0);
                setBounds(chartArea.getPlotRect().x+hadj, getTop()-vadj*fac,
                          chartArea.getPlotRect().width-JCChartUtil.abs(ax),
                          getHeight());
            }
        }
        else {
            hadj = Math.max(chartArea.getDeltaX(), 0);
            if (getTop() <= chartArea.getPlotRect().y) {
                vadj = Math.max(chartArea.getDeltaY(), 0);
                setBounds(chartArea.getPlotRect().x + hadj,
                          chartArea.getPlotRect().y + vadj * fac - getHeight(),
                          chartArea.getPlotRect().width - JCChartUtil.abs(ax),
                          getHeight());
            }
            else {
                setBounds(chartArea.getPlotRect().x + hadj,
                          chartArea.getPlotRect().y +
                          chartArea.getPlotRect().height - vadj * fac,
                          chartArea.getPlotRect().width - JCChartUtil.abs(ax),
                          getHeight());
            }
        }
    }
}

/**
 * Moves the axis so that it lines up with the origin of
 * another axis.
 */
void layoutOrigin()
{
    if ((annoHandler.realPlacement != ORIGIN &&
         annoHandler.realPlacement != VALUE_ANCHORED))
    {
        return;
    }
    if (placementAxis == null) {
        placementAxis = otherAxis;
    }
    if (placementAxis == null) {
        return;
    }

    double v =
        (annoHandler.realPlacement == VALUE_ANCHORED) ? placementLocation
                                                      : placementAxis.origin.value;

    int loc = placementAxis.toPixel(v);
    if (i_vertical) {
        if (annoHandler.anchor == JCLegend.EAST) {
            setLocation(loc, getTop());
        }
        else {
            setLocation(loc-getWidth(),getTop());
        }
    }
    else {
        if (annoHandler.anchor == JCLegend.NORTH) {
            setLocation(getLeft(), loc-getHeight());
        }
        else {
            setLocation(getLeft(), loc);
        }
    }
}

/**
 * Moves all the axis annotations to the correct positions.
 */
void layoutAnnotations()
{
    if (!visible) {
        return;
    }
    annoHandler.layout();
    if (title != null) {
        title.layout();
    }
}

/**
 * Returns the "length" of the axis in pixels. Uses Vertical
 * to determine the major dimension of the axis.
 */
int pixelLength()
{
    int pixelLength;
    if ((isPolar() || isRadar()) && i_vertical) {
        pixelLength = (int)Math.round(polarAxisParams.radius);
    }
    else {
        pixelLength = i_vertical ? getHeight() : getWidth();
    }

    return(pixelLength);
}

/**
 * Returns the "start" of the axis in pixels, relative to the
 * parent JCChartArea. Uses Vertical to determine the major dimension
 * of the axis.
 */
int pixelStart()
{
    return(i_vertical ? -getTop(): getLeft());
}

/**
 * Calculates the value of scale and shift given the axis position
 * and start point. Scale and shift are used by toPixel() and toData().
 */
void calcTransformation()
{
    getAxisHandler().calcTransformation();
}

int linePosition()
{
    int ret;
    if (i_vertical) {
        if (annoHandler.anchor == JCLegend.WEST) {
            ret = getLeft() + getWidth();
        }
        else if (annoHandler.anchor == JCLegend.EAST) {
            ret = getLeft();
        }
        else {
            ret = getLeft() + getWidth()/2;
        }
    }
    else {
        if (annoHandler.anchor == JCLegend.NORTH) {
            ret = getTop() + getHeight();
        }
        else if (annoHandler.anchor == JCLegend.SOUTH) {
            ret = getTop();
        }
        else {
            ret = getTop() + getHeight()/2;
        }
    }
    return(ret);
}


/**
 * Creates the axis annotations for display. The axis annotations
 * are stored internally as an array of JCValueLabel objects. Each
 * JCValueLabel represents a label at a particular value along the
 * axis.
 */
void makeAnnotations() {
    annoHandler.makeAnnotations();
}

AxisHandler getAxisHandler()
{
    /* Create a new AxisHandler if
     *   - one doesn't already exist
     *   - this is the theta axis of a Polar/Radar chart, and
     *     the existing AxisHandler is for a logarithmic axis
     *   - this is a logarithmic axis in a cartesian chart, and
     *     the existing AxisHandler is not logarithmic
     *   - this is a linear axis in a cartesian chart, and the
     *     existing AxisHandler is for a logarithmic axis.
     */
    boolean fPolarRadar = (isPolar() || isRadar());
    boolean fThetaAxis = (fPolarRadar && !i_vertical);
    if ( (axisHandler == null) ||
         (fPolarRadar && axisHandler instanceof RectangularChartAxisHandler) ||
         (!fPolarRadar && axisHandler instanceof CircularChartAxisHandler) ||
         (fThetaAxis && axisHandler instanceof CircularChartLogAxisHandler) ||
         (logarithmic && !(axisHandler instanceof LogAxisHandler)) ||
         (!logarithmic && (axisHandler instanceof LogAxisHandler)) )
    {
        axisHandler = AxisHandler.makeAxisHandler(this);
    }
    return(axisHandler);
}

Extents getExtents()
{
    return(annoHandler.extents);
}

Vector getAnnotations()
{
    return(annoHandler.valueLabels);
}

int getAdjustedRealPlacement()
{
    if (otherAxis != null && otherAxis.reversed) {
        if (annoHandler.realPlacement == MIN) {
            return(MAX);
        }
        else {
            return(MIN);
        }
    }
    return(annoHandler.realPlacement);
}

/**
 *  Gets the location at which the current axis is placed by translating
 *  the 'realPlacement' member to a value on the other axis.
 */
double getRealPlacementLocation()
{
    switch (annoHandler.realPlacement) {
    case JCAxis.MIN:
        if (otherAxis != null) {
            return(otherAxis.getMin());
        }
        break;
    case JCAxis.MAX:
        if (otherAxis != null) {
            return(otherAxis.getMax());
        }
        break;
    case JCAxis.ORIGIN:
        if (otherAxis != null) {
            return(otherAxis.getOrigin());
        }
        break;
    case JCAxis.VALUE_ANCHORED:
        return(getPlacementLocation());
    }
    return(Double.MAX_VALUE);
}

int getAnnotationAnchor()
{
    return(annoHandler.anchor);
}

/** Finds the first grid line along the axis. Determines this by working
 *  backwards from the first annotation.
 */
double firstGridInc(double inc)
{
    double first = firstInc(inc);

    double diff = first - min.value;
    if (diff >= inc) {
        first -= inc * Math.floor(diff / inc);
    }
    return(first);
}

/** Finds the first annotation along the axis, given the
 *  increment to be used.
 */
double firstInc(double inc)
{
    double lmin = min.value;
    double r;
    if (inc == 0.0) {
        return(lmin);
    }
    lmin = JCChartUtil.precCorrect(getPrecision(), lmin);
    int numIncs = (int)Math.ceil((min.value-lmin) / inc);
    if (numIncs < 0) {
        return(lmin);
    }
    r = lmin + inc * numIncs;
    r = JCChartUtil.precCorrect(getPrecision(), r);
    if (r > max.value) {
        return(lmin);
    }
    return(r);
}

double range()
{
    return(getAxisHandler().range());

}

void translate(double t)
{
    getAxisHandler().translate(t);
    setChanged(true, Changeable.RECALC, false);
}

/**
 * Is the chart a polar plot?
 * @return true if the chart type is JCChart.POLAR
 */
public boolean isPolar()
{
    return(chartType == JCChart.POLAR);
}

/**
 * Is the chart a radar plot?
 * @return true if the chart type is JCChart.RADAR or JCChart.AREA_RADAR
 */
public boolean isRadar()
{
    return(chartType == JCChart.RADAR || chartType == JCChart.AREA_RADAR);
}

/**
 * Is the chart a polar or radar plot?
 * @return true if the chart type is JCChart.POLAR, JCChart.RADAR or
 * JCChart.AREA_RADAR
 */
public boolean isPolarRadar() {
    return (chartType == JCChart.POLAR ||
            chartType == JCChart.RADAR ||
            chartType == JCChart.AREA_RADAR);
}

protected boolean isPolarRadar(int type) {
    return (type == JCChart.POLAR ||
            type == JCChart.RADAR ||
            type == JCChart.AREA_RADAR);
}

/********
 * Internal methods
 */

public boolean isBar()
{
    return(hasBar);
}

private boolean is3D()
{
    return(chartArea != null && chartArea.getDraw3D());
}

/**
 * Determines the length of the axis in data space.
 */
private double dataLength()
{
    return(max.value - min.value);
}

/**
 * Draws the grid lines defined by each of the JCGrid objects.
 */
private void drawGrid(Graphics gc, boolean trough, double start, double end) {
    if (!gridVisible) {
        return;
    }
    JCGrid gridObj = null;
    if (gridDefault) {
        // Use the first JCGrid object to draw grid lines at the labels.
        if (gridList.size() > 0) {
            gridObj = (JCGrid)gridList.get(0);
            if (gridObj != null) {
                drawGridLinesAtLabels(gc, gridObj, trough, start, end);
            }
        }
    }
    else {
        Iterator iterator = gridList.iterator();
        while (iterator.hasNext()) {
            gridObj = (JCGrid)iterator.next();
            if (gridObj != null) {
                drawLinearGridLines(gc, gridObj, trough, start, end);
            }
        }
    }
}

/**
 * Draws the grid lines defined by each of the JCGrid objects.
 */
void drawGrid(Graphics gc)
{
    drawGrid(gc, false, Double.MAX_VALUE, Double.MAX_VALUE);
}

/**
 * Draws the grid lines in the 3D trough for the given value range.
 */
void drawGridInTrough(Graphics gc, double start, double end)
{
    drawGrid(gc, true, start, end);
}

/**
 *  Gets a list of the axis positions that grid lines have to be drawn at.
 *  @return an array of grid line positions.
 */
protected double[] getGridLineList()
{
    // Go thru the list of grid objects, determine the positions of
    // grid lines for each of them (whether default or not), and
    // combine them in a single ArrayList.  Finally, return an
    // array of double values created from that ArrayList.
    ArrayList gridLineList = new ArrayList();
    Iterator iterator = gridList.iterator();
    while (iterator.hasNext()) {
        JCGrid gridObj = (JCGrid)iterator.next();
        if (gridObj != null) {
            ArrayList list = isGridDefault() ?
                getGridLinesAtLabelsList() :
                getLinearGridLinesList(gridObj);
            gridLineList.addAll(list);
        }
    }
    Object[] arr = gridLineList.toArray();
    // Convert the array of Double objects to an array of doubles.
    double[] array = new double[arr.length];
    for (int i=0; i < array.length; i++) {
        array[i] = ((Double)arr[i]).doubleValue();
    }
    return array;
}

/**
 * Draws a grid line at the specified floating point value
 * @param gc Graphics context to use for drawing.
 * @param ls Line style to use for the grid line.
 * @param v Data-space value at which to draw the grid line.
 */
private void drawGridLine(Graphics gc, JCLineStyle ls, double v)
{
    double pixOff = Double.MAX_VALUE;
    boolean fPolarRadar = (isPolar() || isRadar());
    if (!fPolarRadar) {
        if (usingDoublePixels) {
            pixOff = toPixelDouble(v);
        }
        else {
            pixOff = (double) toPixel(v);
        }
    }
    Rectangle c = chartArea.getPlotRect();
    int dx = chartArea.getDeltaX();
    int dy = chartArea.getDeltaY();
    Graphics2D gc2D = (Graphics2D) gc;
    if (i_vertical) {
        if (fPolarRadar) {
            double scale = getAxisHandler().scale;
            double r = 0.0;
            if (reversed) {
                if (isLogarithmic()) {
                    if (max.value > 0 && v > 0) {
                        r = -scale * (JCChartUtil.log10(max.value)
                                        - JCChartUtil.log10(v));
                    }
                }
                else {
                    r = -scale * (max.value - v);
                }
            }
            else {
                if (isLogarithmic()) {
                    if (min.value > 0 && v > 0) {
                        r = scale * (JCChartUtil.log10(v)
                                    - JCChartUtil.log10(min.value));
                    }
                }
                else {
                    r = scale * (v - min.value);
                }
            }

            if ((isPolar() &&
                 (!otherAxis.visible ||
                  r != Math.rint(polarAxisParams.radius))) ||
                (isRadar() && polarAxisParams.circularGridLines))
            {
                //  Don't want a circular grid line to overlap with
                //  the theta axis of a Polar chart.
                ls.drawCircle(gc,
                              (int)Math.round(polarAxisParams.xCenter),
                              (int)Math.round(polarAxisParams.yCenter),
                              (int)Math.round(r));
            }
            else if (isRadar() && !polarAxisParams.circularGridLines) {
                //  This code assumes that in a Radar chart, all of
                //  the spokes are labelled.
                //  This is the vertical axis -- need to get the
                //  value labels for otherAxis.
                int nSpokes = otherAxis.annoHandler.valueLabels.size();
                drawWebLine(gc, ls, r, nSpokes);
            }
        }
        else {
            int clipCorrect = (reversed) ? 1 : -1;
            boolean fValue = (annotationMethod == VALUE);
            int ymax = fValue ?
                (toPixel(max.value) + clipCorrect) : (c.y + c.height);
            int ymin = fValue ? toPixel(min.value) : c.y;
            int deltaY = is3D() ? dy : 0;

            Rectangle defClipRect = gc.getClipBounds();
            gc.clipRect(defClipRect.x, Math.min(ymax, ymin) + deltaY,
                        defClipRect.width, Math.abs(ymax - ymin));

            if (is3D()) {
                // Core grid lines go on the backplane.

                // Backplane grid lines are already shifted to the back plane
                if (!frontPlane) {
                    dy = 0;
                }

                if (dx < 0) {
                    gc2D.draw(new Line2D.Double((double)c.x,
                                                pixOff + (double)dy,
                                                (double)(c.x + c.width + dx),
                                                pixOff + (double)dy));
                }
                else {
                    gc2D.draw(new Line2D.Double((double) (c.x + dx),
                                                pixOff + (double)dy,
                                                (double)(c.x+c.width),
                                                pixOff + (double)dy));
                }
            }
            else {
                gc2D.draw(new Line2D.Double((double)c.x, pixOff,
                                            (double)(c.x + c.width), pixOff));
            }
            gc.setClip(defClipRect.x, defClipRect.y,
                       defClipRect.width, defClipRect.height);
        }
    }
    else {
        if (fPolarRadar) {
            //  Polar or Radar chart, theta axis.
            double angleRadians =
                JCChartUtil.convertAngle(chartArea.getAngleUnit(),
                                         JCChartUtil.RADIANS, v);
            //  Avoid rendering artifacts due to round-off error --
            //  don't draw radial lines for both 0 and 2PI
            if (angleRadians != JCChartUtil.M_2PI) {
                drawRadialGridLine(gc, angleRadians);
            }
        }
        else {
            int clipCorrect = (reversed) ? -1 : 1;

            boolean fValue = (annotationMethod == VALUE);
            int xmax = fValue ?
                (toPixel(max.value) + clipCorrect) : (c.x + c.width);
            int xmin = fValue ? toPixel(min.value) : c.x ;
            int deltaX = is3D() ? dx : 0;
            Rectangle defClipRect = gc.getClipBounds();
            gc.clipRect(Math.min(xmax, xmin) + deltaX, defClipRect.y,
                        Math.abs(xmax - xmin), defClipRect.height);

            if (is3D()) {
                // Core grid lines go on the backplane.

                // Backplane grid lines are already shifted to the back plane
                if (!frontPlane) {
                    dx = 0;
                }

                if (dy < 0) {
                    gc2D.draw(new Line2D.Double((double)dx + pixOff,
                                                (double)c.y,
                                                (double)dx + pixOff,
                                                (double)(c.y + c.height + dy)));
                }
                else {
                    gc2D.draw(new Line2D.Double((double)dx + pixOff,
                                                (double)(c.y + dy),
                                                (double)dx + pixOff,
                                                (double)(c.y + c.height)));
                }
            }
            else {
                gc2D.draw(new Line2D.Double(pixOff, (double)c.y,
                                            pixOff, (double)(c.y + c.height)));
            }
            gc.setClip(defClipRect.x, defClipRect.y,
                       defClipRect.width, defClipRect.height);
        }
    }
}

/**
 * Draws a radial line of length radius at the specified angle.
 *
 * @param gc The graphics context
 * @param angle Angle in radians.
 */
protected void drawRadialGridLine(Graphics gc, double angle)
{
    AxisHandler axisHandler = getAxisHandler();
    if (axisHandler instanceof CircularChartAxisHandler) {
        Point startPoint = ((CircularChartAxisHandler)
                            axisHandler).toPixelPolar(angle, 0.0, false);
        Point endPoint = ((CircularChartAxisHandler)
            axisHandler).toPixelPolar(angle, polarAxisParams.radius, false);
        gc.drawLine(startPoint.x, startPoint.y, endPoint.x, endPoint.y);
    }
}

/**
 * Draw a grid line in the axis trough.
 * @param gc Graphics object used to draw grid line
 * @param ls JCLineStyle object associated with this grid line
 * @param v  the value on the axis representing the grid line to draw.
 */
protected void drawGridLineInTrough(Graphics gc, JCLineStyle ls, double v)
{
    double pixOff;
    if (usingDoublePixels) {
        pixOff = toPixelDouble(v);
    }
    else {
        pixOff = (double) toPixel(v);
    }
    int dx = chartArea.getDeltaX();
    int dy = chartArea.getDeltaY();
    if (i_vertical) {
        if (is3D() && isBar() && dx != 0 &&
            pixOff >= getTop() && pixOff <= (getTop() + getHeight()))
        {
            int x = linePosition();
            ls.updateGraphics(gc);
            ((Graphics2D)gc).draw(new Line2D.Double((double) x, pixOff,
                                  (double) (x + dx), pixOff + (double)dy));
            ls.resetGraphics(gc);
        }
    }
    else {
        if (is3D() && isBar() && dy != 0 &&
            pixOff >= getLeft() && pixOff <= (getLeft() + getWidth()))
        {
            int y = linePosition();
            ls.updateGraphics(gc);
            ((Graphics2D)gc).draw(new Line2D.Double(pixOff, (double) y,
                                  pixOff + (double)dx, (double) (y + dy)));
            ls.resetGraphics(gc);
        }
    }
}

/**
 * Draws a linear grid based on the tick marks for this axis in the specified
 * range.
 */
private void drawLinearGridLines(Graphics gc,
                                 JCGrid gridObj,
                                 boolean trough,
                                 double start,
                                 double end)
{
    if (gridObj == null) {
        return;
    }
    double gridStart = gridObj.getStartValue();
    double gridStop = gridObj.getStopValue();
    double gs = gridObj.getIncrementValue();
    int length = pixelLength();
    double total = dataLength();

    if ((((gs * length) / total) + .5) < 2) {
        if (gs <= 0.0) {
            // Could only have intended to turn grid off
            setGridVisible(false);
//            setGridSpacingIsDefault(true);
        }
        else {
            ErrorDialog.raise(this,
                JCChartBundle.string(JCChartBundle.key123, locale));
        }
        return;
    }
    JCLineStyle ls = (gridObj != null) ?
        gridObj.getGridStyle().getLineStyle() : getGridStyle().getLineStyle();
    if (!(ls.updateGraphics(gc))) {
        return;
    }
    // If the color of the JCLineStyle isn't valid, set the
    // color of the Graphics object to black.  Don't change
    // the color of the JCLineStyle.
    if (ls.getColor() == null) {
        Color foreground = getForeground();
        if (foreground == null) {
            foreground = Color.black;
        }
        gc.setColor(foreground);
    }

    for (double v = gridStart; v <= gridStop; v += gs) {
        if (trough) {
            if (start == Double.MAX_VALUE || (v >= start && v <= end)) {
                drawGridLineInTrough(gc, ls, v);
            }
        }
        else  {
            drawGridLine(gc, ls, v);
        }
    }
    ls.resetGraphics(gc);
}

/**
 *  Gets a list of the axis positions at which linear grid lines have to be drawn.
 *  @return an ArrayList of Doubles representing linear grid line positions.
 */
private ArrayList getLinearGridLinesList(JCGrid gridObj)
{
    double gs = gridObj.getIncrementValue();

    int length = pixelLength();
    double total = dataLength();
    double err = gs / 2;
    if (err < 0) err = -err;

    if ((((gs * length) / total) + .5) < 2) {
        if (gs <= 0.0) {
            // Could only have intended to turn grid off
            setGridVisible(false);
//            setGridSpacingIsDefault(true);
        }
        else {
            ErrorDialog.raise(this,
                JCChartBundle.string(JCChartBundle.key123, locale));
        }
        return new ArrayList();  // return an empty ArrayList
    }
    ArrayList arrayList = new ArrayList(
        (int) Math.ceil((max.value + err - min.value) / gs) );
    for (double v = firstGridInc(gs); v < max.value + err; v += gs) {
        arrayList.add(new Double(v));
    }
    return(arrayList);
}

/**
 * This method draws grid lines at labels only.
 */
private void drawGridLinesAtLabels(Graphics gc,
                                   JCGrid gridObj,
                                   boolean trough,
                                   double start,
                                   double end)
{
    JCLineStyle ls = (gridObj != null) ?
        gridObj.getGridStyle().getLineStyle() : getGridStyle().getLineStyle();

    if (!(ls.updateGraphics(gc))) {
        return;
    }
    // If the color of the JCLineStyle isn't valid, set the
    // color of the Graphics object to black.  Don't change
    // the color of the JCLineStyle.
    if (ls.getColor() == null) {
        Color foreground = getForeground();
        if (foreground == null) {
            foreground = Color.black;
        }
        gc.setColor(foreground);
    }
    List valueLabels = annoHandler.valueLabels;
    int arraySize = valueLabels.size();

    for (int i = 0; i < arraySize; i++) {
        JCValueLabel vl = (JCValueLabel)valueLabels.get(i);
        if (vl == null || !vl.isDrawGrid() || !vl.isDrawLabel()) {
            continue;
        }

        if (trough)  {
            if (start == Double.MAX_VALUE ||
                (vl.value >= start && vl.value <= end))
            {
                drawGridLineInTrough(gc, ls, vl.value);
            }
        }
        else {
            if (isRadar() && !i_vertical) {
                //  Ignore values of JCValueLabel data for Radar chart x axis.
                //  That is, use i rather than vl.value.
                double radarAngle =
                    JCChartUtil.getRadarAngle(chartArea.getAngleUnit(),
                                              arraySize, i, isReversed());
                drawGridLine(gc, ls, radarAngle);
            }
            else {
                drawGridLine(gc, ls, vl.value);
            }
        }
    }
    ls.resetGraphics(gc);
}

/**
 *  Gets a list of the axis positions at which label-based grid lines have to
 *  be drawn.
 *  @return an ArrayList of Doubles representing linear grid line positions.
 */
private ArrayList getGridLinesAtLabelsList()
{
    ArrayList arrayList = new ArrayList();
    for (int i = 0; i < annoHandler.valueLabels.size(); i++) {
        JCValueLabel vl = (JCValueLabel)annoHandler.valueLabels.elementAt(i);
        ChartText cl = vl.getChartText();
        if (!cl.visible || !vl.isDrawLabel()) {
            continue;
        }
        arrayList.add(new Double(vl.value));
    }
    return(arrayList);
}

/**
 * Draws a vertical axis using the provided graphics context.
 * @param gc graphics context to use for drawing.
 */
private boolean drawVertical(Graphics gc)
{
    int localx;
    int localh = getHeight();
    int localy = getTop();
    localx = linePosition();

    if (annoHandler.realPlacement == ORIGIN ||
        annoHandler.realPlacement == VALUE_ANCHORED)
    {
        Rectangle r = chartArea.getPlotRect();
        if ((localx > r.x + r.width) || (localx < r.x)) {
            return(false);
        }
    }

    if (gc.getColor() != getForeground()) {
        gc.setColor(getForeground());
    }

    if (isPolar()) {
        drawRadialGridLine(gc, yAxisAngle);
    }
    else if (isRadar()) {
        //  In a Radar chart, don't draw a radial line for the y axis.
        //  Just display the axis labels and tick marks.
    }
    else {
        gc.drawLine(localx, localy, localx, localy+localh);
    }

    getAxisHandler().drawTicks(gc);
    return(true);
}

/**
 * Draws a horizontal axis using the provided graphics context.
 * Note that for a Polar chart, "horizontal" is the theta axis.
 * @param gc graphics context to use for drawing
 */
private boolean drawHorizontal(Graphics gc)
{
    int localw = getWidth();
    int localx = getLeft();
    int localy = linePosition();

    if (annoHandler.realPlacement == ORIGIN ||
        annoHandler.realPlacement == VALUE_ANCHORED)
    {
        Rectangle r = chartArea.getPlotRect();
        if ((localy > r.y + r.height) || (localy < r.y)) {
            return(false);
        }
    }

    if (gc.getColor() != getForeground()) {
        gc.setColor(getForeground());
    }

    if (isPolar()) {
        int radius = (int)Math.round(polarAxisParams.radius);
        int diameter = 2 * radius;
        int x = (int)Math.round(polarAxisParams.xCenter) - radius;
        int y = (int)Math.round(polarAxisParams.yCenter) - radius;
        gc.drawArc(x, y, diameter, diameter, 0, 360);
    }
    else if (isRadar()) {
        //  Don't draw x axis for a Radar chart.
    }
    else {
        gc.drawLine(localx, localy, localx+localw, localy);
    }

    getAxisHandler().drawTicks(gc);
    return(true);
}

/*
 * Called by drawWebLine() to draw a single line segment.
 */
private Point drawWebSegment(Graphics gc, double r, double radarAngle,
                             Point prevPt)
{
    if (prevPt == null) {
        //  This is the start of a web grid line.
        prevPt = toPixelPolar(0.0, r, false);
    }
    Point currPt = toPixelPolar(radarAngle, r, false);

    Graphics2D gc2D = (Graphics2D) gc;
    gc2D.draw(new Line2D.Double((double)prevPt.x, (double)prevPt.y,
                                (double)currPt.x, (double)currPt.y));
    return(currPt);
}

/*
 * Called by drawGridLine() for Radar charts.  Also called
 * by JCChartArea.drawWebBoundingBox().
 */
protected void drawWebLine(Graphics gc, JCLineStyle ls, double r,
                           int nSpokes)
{
    double radarAngle = JCChartUtil.getRadarAngle(JCChartUtil.RADIANS, nSpokes,
                                                  1, false);
    Point pnt1 = null;
    for (int i = 0; i < nSpokes; i++) {
        pnt1 = drawWebSegment(gc, r, i * radarAngle, pnt1);
    }
    drawWebSegment(gc, r, 0., pnt1);  //  Close off the web line.
}

//private Graphics clipToExtents(Graphics gc)
//{
//  Graphics tempGC = gc.create();
//  Extents ex = getExtents();
//  if (vertical) {
//      tempGC.clipRect(0, Math.max(0, getTop()-ex.minor),
//                      chartArea.getDrawingArea().width+1,
//                      Math.min(chartArea.getDrawingArea().height,
//                               getHeight()+ex.major+ex.minor)+1);
//    }
//  else {
//      tempGC.clipRect(Math.max(0, getLeft()-ex.minor), 0 ,
//                      Math.min(chartArea.getDrawingArea().width,
//                               getWidth()+ex.major+ex.minor)+1,
//                      chartArea.getDrawingArea().height+1);
//    }
//  return(tempGC);
//}

/**
 * Gets the foreground color of this axis, which is derived according to
 * the following hierarchy: set locally, set on parent region JCChartArea
 * instance, and set on parent JCChart instance.
 * @return the derived foreground color of this axis.
 */
public Color getForeground()
{
    return (foreground == null) ? (chartArea == null) ? (parentChart == null) ?
        null : parentChart.getForeground() : chartArea.getForeground() : foreground;
}

/**
 * Gets the background color of this axis, which is derived according to
 * the following hierarchy: set locally, set on parent region JCChartArea
 * instance and region is opaque, and set on parent JCChart instance and chart
 * is opaque.
 * <BR>Note: this is here for completeness sake -- there is really no background color
 * to an axis.
 * @return the derived background color of this axis.
 */
public Color getBackground()
{
    // No such thing as an axis background color
    return (chartArea == null) ? (
        (parentChart == null) ? null : (parentChart.isOpaque() ?
                parentChart.getBackground() : null)
        ) : (chartArea.isOpaque() ? chartArea.getBackground() : null);
}

/**
 * Gets the font property of this axis, which is derived according to
 * the following hierarchy: set locally, set on parent region JCChartArea
 * instance, and set on parent JCChart instance.
 * @return the derived font of this axis.
 */
public Font getFont()
{
    return (font == null) ? (chartArea == null) ? (parentChart == null) ?
        null : parentChart.getFont() : chartArea.getFont() : font;
}

/**
 * Gets the parent region JCChartArea instance.
 */
public JCChartArea getChartArea()
{
    return(chartArea);
}

/**
 * Gets the AnnotationHandler for this axis
 */
public AnnotationHandler getAnnotationHandler()
{
    return(this.annoHandler);
}

/**
 * Method to set the angle of the y axis in a Polar/Radar chart.
 *
 * @param units The units of the specified angle.  Valid values are
 * JCChartUtil.DEGREES, JCChartUtil.GRADS, or JCChartUtil.RADIANS.
 * @param angle The desired angle for the r axis.
 */
public void setYAxisAngle(int units, double angle)
{
    yAxisAngle = JCChartUtil.convertAngle(units, JCChartUtil.RADIANS, angle);
    setChanged(true, Changeable.RECALC);
}

/**
 * Method to get the angle of the y axis in a Polar/Radar chart.
 *
 * @param units The desired units of the returned angle.  Valid values are
 * JCChartUtil.DEGREES, JCChartUtil.GRADS, or JCChartUtil.RADIANS.
 */
public double getYAxisAngle(int units)
{
    return(JCChartUtil.convertAngle(JCChartUtil.RADIANS, units, yAxisAngle));
}

/**
 * Method to set the angle of the theta axis origin.
 *
 * @param units The units of the specified angle.  Valid values are
 * JCChartUtil.DEGREES, JCChartUtil.GRADS, or JCChartUtil.RADIANS.
 * @param angle The desired angle for the theta axis origin.
 */
public void setOriginBase(int units, double angle)
{
    polarAxisParams.setOriginBase(units, angle);
    setChanged(true, Changeable.RECALC);
}

/**
 * Method to get the angle of the theta axis origin.
 *
 * @param units The desired units of the returned angle.  Valid values are
 * JCChartUtil.DEGREES, JCChartUtil.GRADS, or JCChartUtil.RADIANS.
 */
public double getOriginBase(int units)
{
    return(polarAxisParams.getOriginBase(units));
}

/**
 * Method to set the half range flag for Polar charts.
 *
 * @param fHalfRange If true, theta axis labels on a Polar chart will
 * have a maximum value of 180 degrees.
 */
public void setHalfRange(boolean fHalfRange)
{
    polarAxisParams.setPolarHalfRange(fHalfRange);
    setChanged(true, Changeable.RECALC);
}

/**
 * Method to get the half range flag for Polar charts.
 *
 * @return true if theta axis labels have a maximum value of 180 degrees.
 */
public boolean isHalfRange()
{
    return(polarAxisParams.isPolarHalfRange());
}

/**
 * Makes the y grid lines of a Radar chart circular.
 *
 * @param fCircular true for circular grid lines, false for webbed
 * grid lines.
 */
public void setRadarCircularGrid(boolean fCircular)
{
    polarAxisParams.setCircularGrid(fCircular);
    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the state of the circular flag for Radar chart grid lines.
 *
 * @return true if the y grid lines are circular, false if the y grid
 * lines are webbed.
 */
public boolean isRadarCircularGrid()
{
    return(polarAxisParams.isCircularGrid());
}

/**
 * Gets the center of the plot area for polar/radar charts.
 *
 * @return the center of the plot area for polar/radar charts.
 */
public Point2D.Double getPolarRadarCenter()
{
    return(new Point2D.Double(polarAxisParams.xCenter,
                              polarAxisParams.yCenter));
}

/**
 * Gets the radius of the plot area for polar/radar charts.
 *
 * @return the radius of the plot area for polar/radar charts.
 */
public double getPolarRadarRadius()
{
    return(polarAxisParams.radius);
}

/**
 * Adds a JCAnno object to this axis.
 */
public void addAnnotation(JCAnno anno)
{
    if (annoList != null) {
        anno.setParent(this);
        int type = anno.getType();
        if (type == JCAnno.DEFAULT_LABELS) {
            addDefaultLabels(anno);
        }
        else if (type == JCAnno.DEFAULT_TICKS) {
            addDefaultAnnotations(anno);
        }
        else {
            annoList.add(anno);
        }
        setChanged(true, Changeable.RECALC);
    }
}

/**
 * Removes the specified JCAnno object from this axis.
 */
public void removeAnnotation(JCAnno anno)
{
    if (annoList != null && anno != null) {
        annoList.remove(anno);
        setChanged(true, Changeable.RECALC);
    }
}

/**
 * Removes the JCAnno object at the specified index from this axis.
 */
public void removeAnnotation(int index)
{
    if (annoList != null && index >= 0 && index < annoList.size()) {
        annoList.remove(index);
        setChanged(true, Changeable.RECALC);
    }
}

/**
 * Determines if the specified JCAnno has already been
 * added to the axis.
 * @param anno The JCAnno to look for
 * @return true if the JCAnno was found
 */
public boolean containsAnnotation(JCAnno anno)
{
    boolean found = false;
    if (annoList != null && anno != null) {
        found = annoList.contains(anno);
    }
    return(found);
}

/**
 * Determines if a JCAnno with the specified name has
 * already been added to the axis.
 * @param annoName The name of the JCAnno.  If the parameter
 * is null, the method will return false;
 * @return true if a JCAnno of the specified name was found
 */
public boolean containsAnnotation(String annoName)
{
    boolean found = false;
    if (annoList != null && annoName != null) {
        int nObjects = annoList.size();
        JCAnno anno = null;
        String name = null;
        for (int i = 0; i < nObjects; i++) {
            anno = (JCAnno)annoList.get(i);
            name = anno.getName();
            if (name != null && name.equals(annoName)) {
                found = true;
                break;
            }
        }
    }
    return(found);
}

/**
 * Sets a List of JCAnno objects on this axis.
 * @param annoList A List of JCAnno objects
 */
public void setAnnotationList(List annoList)
{
    this.annoList = annoList;
    setChanged(true, Changeable.RECALC);
}

/**
 * Returns the List of JCAnno objects that have been set on this axis.
 */
public List getAnnotationList()
{
    return(annoList);
}

/**
 * Removes all JCAnno objects from the internal List.
 */
public void removeAllAnnotations()
{
    if (annoList != null) {
        annoList.clear();
    }
}

/**
 * For internal use only.
 */
public void removeDefaultLabels()
{
    removeAnnotation(getDefaultLabels());
}

/**
 * For internal use only.
 */
public void removeDefaultTicks()
{
    removeAnnotation(getDefaultTicks());
}

/**
 * For internal use only.
 */
public void removeDefaultGrid()
{
    gridList.clear();
}

protected void updateGridObject(JCGrid chartGrid, double start, double stop,
                                double increment)
{
    if (chartGrid != null) {
        if (chartGrid.getStartValueIsDefault()) {
            chartGrid.startValue.value = start;
            // Don't call setStartValue() which sets isDefault to false.
        }
        if (chartGrid.getStopValueIsDefault()) {
            chartGrid.stopValue.value = stop;
        }
        if (chartGrid.getIncrementIsDefault()) {
            chartGrid.incrementValue.value = increment;
        }
    }
}

protected void updateGrids()
{
    if (gridList != null) {
        for (int i = 0; i < gridList.size(); i++) {
            updateGridObject((JCGrid)gridList.get(i), min.value, max.value,
                             numSpacing.value);
        }
    }
}

/**
 * This method puts the specified anno object at the top of
 * the ArrayList.
 */
private void addDefaultLabels(JCAnno labels)
{
    if (annoList == null) {
        annoList = new ArrayList();
    }
    // The default labels object is always at index 0.
    if (annoList.size() == 0) {
        annoList.add(labels);
    }
    else {
        annoList.add(0, labels);
    }
}

/**
 * This method puts the specified anno object at the
 * appropriate position in the ArrayList.
 */
private void addDefaultAnnotations(JCAnno anno)
{
    if (annoList == null) {
        annoList = new ArrayList();
    }
    // If the default labels object exists (always at index 0),
    // add the default anno as the 2nd object.
    // If the default labels object doesn't exist, add the
    // default anno as the first anno object.
    if (getDefaultLabels() == null) {
        if (annoList.size() == 0) {
            annoList.add(anno);
        }
        else {
            annoList.add(0, anno);
        }
    }
    else {
        if (annoList.size() == 1) {
            annoList.add(anno);
        }
        else {
            annoList.add(1, anno);
        }
    }
}

/**
 * Intended for internal use.
 */
public void addDefaultLabels()
{
    JCAnno annoObj = new JCAnno(JCAnno.DEFAULT_LABELS, this, null);
    // Because no Color is specified, the axis foreground color will be used.
    addDefaultLabels(annoObj);
    setChanged(true, Changeable.RECALC);
}

/**
 * Intended for internal use.
 */
public void addDefaultTicks()
{
    JCAnno annoObj = new JCAnno(JCAnno.DEFAULT_TICKS, this, null);
    // Because no Color is specified, the axis foreground
    // color will be used.
    addDefaultAnnotations(annoObj);
    setChanged(true, Changeable.RECALC);
}

/**
 * Create default JCAnno objects.
 */
protected void makeDefaultAnnotations()
{
    addDefaultLabels();
    addDefaultTicks();
}

/**
 * At the end of the layout process, update the default start, stop,
 * and increment values of tick objects.
 */
protected void updateAnnotations()
{
    if (annoList != null && axisHandler != null) {
        JCAnno annoObj = null;
        for (int i=0; i < annoList.size(); i++) {
            annoObj = (JCAnno)annoList.get(i);
            if (annoObj != null) {
                axisHandler.updateAnnoObject(annoObj);
            }
        }
    }
}

/**
 * This method searches an internal List for a JCAnno
 * object of the specified type.
 * @param type The JCAnno type.  Valid values include
 * JCAnno.DEFAULT_LABELS, JCAnno.DEFAULT_TICKS, or
 * JCAnno.USER_DEFINED.
 * @return A JCAnno object of the specified type, or null
 * if such an object does not exist.
 */
protected JCAnno getAnnoObject(int type)
{
    int nTickObjects = annoList.size();
    JCAnno anno = null;
    JCAnno retVal = null;
    for (int i=0; i < nTickObjects; i++) {
        anno = (JCAnno)annoList.get(i);
        if (anno.getType() == type) {
            retVal = anno;
            break;
        }
    }
    return(retVal);
}

/**
 * For internal use only.
 * This method returns a JCAnno object of type DEFAULT_LABELS,
 * or null if an object of that type doesn't exist.
 */
public JCAnno getDefaultLabels()
{
    return getAnnoObject(JCAnno.DEFAULT_LABELS);
}

/**
 * For internal use only.
 * This method returns a JCAnno object of type DEFAULT_TICKS,
 * or null if an object of that type doesn't exist.
 */
public JCAnno getDefaultTicks()
{
    return getAnnoObject(JCAnno.DEFAULT_TICKS);
}

/**
 * If the annotation method is VALUE or if the useAnnoTicks
 * flag is set, iterate through the list of JCAnno objects
 * and create JCValueLabels for the tick marks defined by
 * each JCAnno.  If useAnnoTicks is true, skip JCAnno
 * objects that have the drawLabel flag set.
 */
protected void makeTickLabels()
{
    JCAnno annoObj = null;
    if (annotationMethod == JCAxis.VALUE || useAnnoTicks) {
        int nObjects = annoList.size();
        for (int i=0; i < nObjects; i++) {
            annoObj = (JCAnno)annoList.get(i);
            if (annoObj != null) {
                if (annotationMethod != JCAxis.VALUE &&
                    useAnnoTicks &&
                    annoObj.drawLabels)
                {
                    continue;
                }
                getAxisHandler().makeTickLabels(annoObj);
            }
        }
    }
}

/**
 * This method is used to set the value of the useAnnoTicks flag.
 * If the flag is true, ticks specified by the JCAnno objects
 * will be drawn when the annotation method is TIME_LABELS, POINT_LABELS,
 * or VALUE_LABELS.  If false, ticks will be drawn at the positions of
 * the labels.  The useAnnoTicks flag is ignored when the annotation
 * method is VALUE.
 */
public void setUseAnnoTicks(boolean useAnnoTicks)
{
    this.useAnnoTicks = useAnnoTicks;
    setChanged(true, Changeable.RECALC);
}

/**
 * This method returns the value of the useAnnoTicks flag.
 * If the flag is true, ticks specified by the JCAnno objects
 * will be drawn when the annotation method is TIME_LABELS, POINT_LABELS,
 * or VALUE_LABELS.  If false, ticks will be drawn at the positions of
 * the labels.  The useAnnoTicks flag is ignored when the annotation
 * method is VALUE.
 */
public boolean getUseAnnoTicks()
{
    return(useAnnoTicks);
}

/**
 * This method returns the maximum outer extent of tick marks
 * on this axis.
 */
protected int getMaxTickOuterExtent()
{
    int outerExtent = tickLength / 2;
    JCAnno tick = null;
    for (int i=0; i < annoList.size(); i++) {
        tick = (JCAnno)annoList.get(i);
        if (tick.getOuterExtent() > outerExtent) {
            outerExtent = tick.getOuterExtent();
        }
    }
    return(outerExtent);
}

/**
 * Returns the List of JCGrid objects.
 */
public List getGridList() {
    return(gridList);
}

/**
 * This method returns the JCGrid object at the
 * specified index.
 */
public JCGrid getGrid(int index)
{
    JCGrid grid = null;
    if (gridList != null &&
        index >= 0 &&
        index < gridList.size())
    {
        grid = (JCGrid)gridList.get(index);
    }
    return(grid);
}

/**
 * Adds a JCGrid object to this axis.
 */
public void addGrid(JCGrid grid)
{
    if (gridList != null) {
        grid.setParent(this);
        gridList.add(grid);
        setChanged(true, Changeable.RECALC);
    }
}

/**
 * Removes the specified JCGrid object from this axis.
 */
public void removeGrid(JCGrid grid) {
    if (gridList != null) {
        gridList.remove(grid);
        setChanged(true, Changeable.RECALC);
    }
}

/**
 * Removes the JCGrid object at the specified index from this axis.
 */
public void removeGrid(int index)
{
    if (gridList != null &&
        index >= 0 &&
        index < gridList.size())
    {
        gridList.remove(index);
        setChanged(true, Changeable.RECALC);
    }
}

public void removeAllGrids()
{
    if (gridList != null) {
        gridList.clear();
    }
}

/**
 * Determines if the specified JCGrid has already been
 * added to the axis.
 * @param grid The JCGrid to look for
 * @return true if the JCGrid was found
 */
public boolean containsGrid(JCGrid grid)
{
    boolean found = false;
    if (gridList != null) {
        found = gridList.contains(grid);
    }
    return(found);
}

/**
 * Determines if a JCGrid with the specified name has
 * already been added to the axis.
 * @param gridName The name of the JCGrid.  If the parameter
 * is null, the method will return false;
 * @return true if a JCGrid of the specified name was found
 */
public boolean containsGrid(String gridName)
{
    boolean found = false;
    if (gridList != null && gridName != null) {
        int nGrids = gridList.size();
        JCGrid grid = null;
        String name = null;
        for (int i = 0; i < nGrids; i++) {
            grid = (JCGrid)gridList.get(i);
            name = grid.getName();
            if (name != null && name.equals(gridName)) {
                found = true;
                break;
            }
        }
    }
    return(found);
}

/**
 * For internal use only.
 * Returns the first JCGrid object in the internal List
 * that has all properties set to their defaults.
 */
public JCGrid getDefaultGrid()
{
    JCGrid defGrid = null;
    if (gridList != null) {
        JCGrid gridObj = null;
        for (int i=0; i < gridList.size(); i++) {
            gridObj = (JCGrid)gridList.get(i);
            if (gridObj != null && gridObj.isDefault()) {
                defGrid = gridObj;
                break;
            }
        }
    }
    return(defGrid);
}

/**
 * Get the current image map mapper.  For internal use only.
 * @return The current image map mapper.
 */
public Mapper getMapper()
{
    return(mapper);
}

/**
 * Set the current image map mapper.  For internal use only.
 * @param mapper The new image map mapper.
 */
public void setMapper(Mapper mapper)
{
    this.mapper = mapper;
}

}
