/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCAxisTitle.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import com.klg.jclass.util.legend.JCLegend;

/**
 * Class that controls the appearance of an axis title
 * (that is, the title placement and text).
 */
public class JCAxisTitle extends ChartText {

/** The placement value as set by the user (or not, if using default) */
protected DerivedInt placement = new DerivedInt(JCLegend.NORTH, true);

/** The actual placement used to place the axis title.  Calculated in
    fixPlacement() from the user-set or default placement as well as
    other facotrs. */
protected int effectivePlacement = JCLegend.NORTH;

/**
 * Default constructor for <code>JCAxisTitle</code>. Makes an
 * unattached axis title.
 */
public JCAxisTitle()
{
    super();
    setVisible(false);
}

/**
 * Constructor for <code>JCAxisTitle</code> that sets the contents of
 * the title as a convenience.
 * @param title String representing the axis title
 */
public JCAxisTitle(String title)
{
    super();
    setText(title);
}

/**
 * Sets the <code>ParentAxis</code> property. The <code>ParentAxis</code>
 * property may hold a reference to the axis to which the
 * <code>JCAxisTitle</code> belongs.
 * @param pa
 */
void setParentAxis(JCAxis pa)
{
    // let the super class do the work
    super.setParentAxis(pa);
    // calculate the current effective placement now that we have an axis
    fixPlacement(pa);
}

/**
 * Sets the <code>Text</code> property.
 * @param nc new string contents of the <code>ChartText</code> instance
 * @param setVisible if <code>true</code>, Visible indication is set to
 * <code>true</code>
 */
public void setText(String nc, boolean setVisible)
{
    // Let the super class do the work
    super.setText(nc, setVisible);
    // Only if no text was there before
    if (getWidth() == 0) {
        setChanged(true, Changeable.RECALC);
    }
}

/**
 * Sets the <code>Placement</code> property of the <code>JCTitleAxis</code>.
 * The <code>Placement</code> property controls where the <code>JCAxis</code>
 * title is placed relative to the &quot;opposing&quot; axis. Valid values
 * include <code>JCLegend.NORTH</code> or <code>JCLegend.SOUTH</code>
 * for horizontal axes, and <code>JCLegend.EAST</code>,
 * <code>JCLegend.WEST</code>, <code>JCLegend.NORTHEAST</code>,
 * <code>JCLegend.SOUTHEAST</code>, <code>JCLegend.NORTHWEST</code> or
 * <code>JCLegend.SOUTHEAST</code> for vertical axes.
 * @param place one of NORTH or SOUTH for
 * horizontal axes, and EAST/WEST/NORTHEAST/SOUTHEAST/NORTHWEST/SOUTHEAST
 * for vertical axes.
 * @exception IllegalArgumentException If <code>Placement</code> is not one
 * of NORTH, SOUTH, EAST, WEST, NORTHEAST, NORTHWEST, SOUTHEAST or SOUTHWEST.
 */
public void setPlacement(int place)
{
    if (!placement.isDefault && place == placement.value) {
        return;
    }

    switch (place) {
    case JCLegend.EAST:
    case JCLegend.WEST:
    case JCLegend.NORTHEAST:
    case JCLegend.NORTHWEST:
    case JCLegend.SOUTHEAST:
    case JCLegend.SOUTHWEST:
    case JCLegend.NORTH:
    case JCLegend.SOUTH:
        placement.value = place;
        placement.isDefault = false;
        // update the effective placement
        fixPlacement(parentAxis);
        break;
    default:
        throw new IllegalArgumentException("Invalid title Placement in " +
                                           "JCAxisTitle");
    }
    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the actual value of the
 * <code>Placement</code> property of the <code>JCTitleAxis</code>.
 * The <code>Placement</code> property controls where the <code>JCAxis</code>
 * title is placed relative to the &quot;opposing&quot; axis. Valid values
 * include <code>JCLegend.NORTH</code> or <code>JCLegend.SOUTH</code>
 * for horizontal axes, and <code>JCLegend.EAST</code>,
 * <code>JCLegend.WEST</code>, <code>JCLegend.NORTHEAST</code>,
 * <code>JCLegend.SOUTHEAST</code>, <code>JCLegend.NORTHWEST</code> or
 * <code>JCLegend.SOUTHEAST</code> for vertical axes.
 * @return the actual value of the <code>Placement</code> property, one of
 * NORTH,SOUTH,EAST,WEST,NORTHEAST,SOUTHEAST,NORTHWEST or SOUTHEAST.
 */
public int getPlacement()
{
    return(placement.value);
}

/**
 * Sets the value of the <code>PlacementIsDefault</code> property.
 * The <code>PlacementIsDefault</code> property determines whether
 * Chart is responsible for calculating a reasonable default placement
 * for the axis title.
 * @param pid <code>true</code> if the default placement is to be used,
 *            <code>false</code> otherwise.
 */
public void setPlacementIsDefault(boolean pid)
{
    if (placement.isDefault == pid) {
        return;
    }
    placement.isDefault = pid;
    // update the effective placement
    fixPlacement(parentAxis);
    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the value of the <code>PlacementIsDefault</code> property.
 * The <code>PlacementIsDefault</code> property determines whether
 * Chart is responsible for calculating a reasonable default placement
 * for the axis title.
 * @return <code>true</code> if the default placement is to be used,
 * <code>false</code> otherwise.
 */
public boolean getPlacementIsDefault()
{
    return(placement.isDefault);
}

/**
 * Gets the effective value of the
 * <code>Placement</code> property of the <code>JCTitleAxis</code>
 * given current information.
 * This is the actual value used to control where the
 * <code>JCAxis</code> title is placed.
 * If the value of the <code>Placement</code> property is a value
 * that is not allowed considering the orientation of the current axis,
 * a default value is used.
 * @return One of NORTH or SOUTH for
 * horizontal axes, and EAST/WEST/NORTHEAST/SOUTHEAST/NORTHWEST/SOUTHEAST
 * for vertical axes.
 */
public int getEffectivePlacement()
{
    return(effectivePlacement);
}

/**
 * Fix the placement, generating the effectivePlacement property.
 * This is done by looking at existing value of the placement property
 * and determining whether it is allowable on the existing axes.  If
 * not a suitable value is chosen.
 * @param axis JCAxis instance that this title is being placed against
 */
protected void fixPlacement(JCAxis axis)
{

    // start with the current placement
    effectivePlacement = placement.value;

    if (axis == null) {
        return;
    }

    // calculate correct default placement
    if (placement.isDefault) {
        // Here is the "old" placement code
        // Note: Parent axis must be non-null
        if (!parentAxis.i_vertical && rotation != ChartText.DEG_0) {
            effectivePlacement = axis.getAnnotationAnchor();
        }
        else {
            if (parentAxis.otherAxis != null &&
                parentAxis.otherAxis.getAnnotationAnchor() == JCLegend.NORTH) {
                if (parentAxis.getAnnotationAnchor() == JCLegend.EAST) {
                    effectivePlacement = JCLegend.SOUTHEAST;
                }
                else {
                    effectivePlacement = JCLegend.SOUTHWEST;
                }
            }
            else {
                if (parentAxis.getAnnotationAnchor() == JCLegend.EAST) {
                    effectivePlacement = JCLegend.NORTHEAST;
                }
                else {
                    effectivePlacement = JCLegend.NORTHWEST;
                }
            }
        }
    }

    // Make sure effective placement conforms to constraints depending on
    // axis direction
    if (axis.i_vertical) {
        switch (effectivePlacement) {
        case JCLegend.EAST:
        case JCLegend.WEST:
        case JCLegend.NORTHEAST:
        case JCLegend.NORTHWEST:
        case JCLegend.SOUTHEAST:
        case JCLegend.SOUTHWEST:
            break;
        default:
            effectivePlacement = axis.getAnnotationAnchor();
            break;
        }
    }
    else {
        switch (effectivePlacement) {
        case JCLegend.NORTH:
        case JCLegend.SOUTH:
            break;
        default:
            effectivePlacement = axis.getAnnotationAnchor();
            break;
        }
    }
}

/**
 * Performs layout on the axis title.
 */
public void layout()
{
    if (visible) {
        if (parentAxis.i_vertical) {
            int y = parentAxis.getTop() + (parentAxis.getHeight() -
                    getHeight()) / 2;
            int leftx = parentAxis.getLeft();
            int rightx = parentAxis.getLeft() + parentAxis.getWidth() -
                         getWidth();
            Extents ex = parentAxis.getExtents();
            switch (effectivePlacement) {
            case JCLegend.NORTHWEST:
                setLocation(leftx, parentAxis.getTop() - ex.minor);
                break;
            case JCLegend.WEST:
                setLocation(leftx, y);
                break;
            case JCLegend.SOUTHWEST:
                setLocation(leftx, parentAxis.getTop() +
                                   parentAxis.getHeight() +
                                   ex.major - getHeight() - 1);
                break;
            case JCLegend.NORTHEAST:
                setLocation(rightx, parentAxis.getTop() -
                                    ex.minor);
                break;
            case JCLegend.EAST:
                setLocation(rightx, y);
                break;
            case JCLegend.SOUTHEAST:
                setLocation(rightx, parentAxis.getTop() +
                                    parentAxis.getHeight() +
                                    ex.major - getHeight() - 1);
                break;
            }
        }
        else {
            int x = parentAxis.getLeft() +
                    (parentAxis.getWidth() - getWidth()) / 2;
            switch (effectivePlacement) {
            case JCLegend.NORTH:
                setLocation(x, parentAxis.getTop() + 1);
                break;
            case JCLegend.SOUTH:
                setLocation(x, parentAxis.getTop() + parentAxis.getHeight() -
                               getHeight());
                break;
            }
        }
    }
}

}


