/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCBarChartFormat.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

/**
 * An object that manages chart properties specific to bar charts.
 */
public class JCBarChartFormat implements JCChartTypeFormat,
                                         java.io.Serializable
{

public final static int  BAR_CLUSTER_WIDTH_MAX = 100;
public final static int  BAR_CLUSTER_OVERLAP_MIN = -100;
public final static int  BAR_CLUSTER_OVERLAP_MAX = 100;

protected int         clusterWidth = 80;
protected int         clusterOverlap = 0;
protected boolean     is100Percent = false;
protected Changeable  parent = null;


public JCBarChartFormat() {}

public JCBarChartFormat(Changeable p)
{
    parent = p;
}

public void setParent(ChartDataView cdv)
{
    parent = cdv;
}

/**
 * Gets the <code>ClusterWidth</code> property, which determines the
 * percentage of available space to be occupied by the
 * bars.
 * @return A value between 0 and 100 representing the percentage
 * of available space used by the bars
 */
public int getClusterWidth()
{
    return(clusterWidth);
}

/**
 * Sets the <code>ClusterWidth</code> property, which determines the
 * percentage of available space that will be occupied by the
 * bars.
 * @param per value between 0 and 100 representing the percentage
 * of available space used by the bars
 * @exception IllegalArgumentException If <code>ClusterWidth</code> is not
 * between 0 and 100.
 */
public void setClusterWidth(int per)
{
    if (per < 0 || per > BAR_CLUSTER_WIDTH_MAX) {
        throw new IllegalArgumentException("ClusterWidth must be between " +
                                           "0 and 100");
    }
    if (per == clusterWidth) {
        return;
    }
    clusterWidth = per;
    if (parent != null) {
        parent.setChanged(true, Changeable.RECALC);
    }
}

/**
 * Gets the <code>ClusterOverlap</code> property, which specifies the
 * overlap between bars.
 * @return percentage of overlap as a percentage of the bar's
 * width. Valid values are between -100 and +100.
 */
public int getClusterOverlap()
{
    return(clusterOverlap);
}

/**
 * Sets the <code>ClusterOverlap</code> property, which specifies the
 * overlap between bars.
 * @param per percentage of overlap as a percentage of the bar's
 * width. Valid values are between -100 and +100.
 * @exception IllegalArgumentException If <code>ClusterOverlap</code> is not
 * between -100 and 100.
 */
public void setClusterOverlap(int per)
{
    if (per < BAR_CLUSTER_OVERLAP_MIN || per > BAR_CLUSTER_OVERLAP_MAX) {
        throw new IllegalArgumentException("ClusterOverlap must be between " +
                                           "-100 and 100");
    }
    if (per == clusterOverlap) {
        return;
    }
    clusterOverlap = per;
    if (parent != null) {
        parent.setChanged(true, Changeable.RECALC);
    }
}

/**
 * Gets the value of the <code>100Percent</code> property. A <code>true</code>
 * value indicates that the stacking bar chart will be charted against
 * an axis representing values from 0 to 100 percent.
 * @return A boolean indicating whether the Y-axis will range from
 * 0 to 100.
 */
public boolean is100Percent()
{
    return(is100Percent);
}

/**
 * Sets the value of the <code>100Percent</code> property. A <code>true</code>
 * value indicates that the stacking bar chart will be charted against
 * an axis representing values from 0 to 100 percent.
 * @param b boolean indicating whether the y axis will range from
 * 0 to 100
 */
public void set100Percent(boolean b)
{
    if (is100Percent == b) {
        return;
    }
    is100Percent = b;
    if (parent != null) {
        parent.setChanged(true, Changeable.RECALC);
    }
}

}
