/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCChart.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import com.klg.jclass.util.legend.*;
import com.klg.jclass.util.swing.JCExitFrame;
import com.klg.jclass.util.*;

import javax.swing.*;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;
import javax.swing.plaf.ComponentUI;
import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.io.FileInputStream;
import java.io.ObjectInputStream;
import java.util.*;
import java.util.List;


 

/**
 * The core charting class. <code>JCChart</code> allows the
 * retrieval of related objects, such as the chart area, the
 * legend, the header, and the footer. Bean classes are represented
 * by a charting class derived from <code>JCChart</code>.
 */
public class JCChart extends JComponent
                     implements Changeable, LegendComponentLayoutUser,
                                ServerRenderable, java.io.Serializable,
                                KeyListener, MouseListener, MouseMotionListener,
                                AncestorListener, JCName, RenderProperties
{

/**
 * The UI class ID.
 * @see #getUIClassID
 * @see #writeObject
 */
private static final String uiClassID = "JCChartUI";

protected JCChartArea           chartArea = null;
protected JCLegend              legend = null;
protected JCChartLegendManager  legendManager = null;
protected JComponent            footer = null;
protected JComponent            header = null;
protected Vector                data = new Vector();
protected Vector                triggers = new Vector();
protected JCListenerList        listeners = null;
protected JCListenerList        pickListeners = null;
protected JCChartLabelManager   chartLabelManager = null;
protected Applet                chartApplet = null;
protected Object                userObject = null;

/** The image map information for the chart. */
protected ImageMapInfo imageMapInfo = null;

/** The image map information for the footer. */
protected ImageMapInfo footerImageMapInfo = null;

/** The image map information for the header. */
protected ImageMapInfo headerImageMapInfo = null;

private boolean                 warningOn = true;
private boolean                 batched = false;
private boolean                 allowUserChanges = false;

/** Chart Types. */
public final static int  NONE = -1;
public final static int  PLOT = 0;
public final static int  SCATTER_PLOT = 1;
public final static int  POLAR = 2;
public final static int  RADAR = 3;
public final static int  AREA_RADAR = 4;
public final static int  HILO = 5;
public final static int  HILO_OPEN_CLOSE = 6;
public final static int  CANDLE = 7;
public final static int  AREA = 8;
public final static int  BAR = 9;
public final static int  STACKING_BAR = 10;
public final static int  PIE = 11;
public final static int  STACKING_AREA = 12;
public final static int  LAST_PREDEFINED_CHART_TYPE = 12;

protected int antiAliasing = ANTI_ALIASING_DEFAULT;

/** Error message for invalid trigger index. */
public final static String ERRMSG_INVALID_TRIGGER_INDEX =
    "Invalid EventTrigger index.";

protected String customizerName =
    "com.klg.jclass.chart.customizer.ChartCustomizer";

protected int lineColorIndex;
protected int symbolColorIndex;
protected int fillColorIndex;
protected int symbolShapeIndex;

private Dimension curSize = new Dimension(-1, -1);

protected Rectangle chartAreaLayoutHints = new Rectangle(
    Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE);
protected Rectangle legendLayoutHints = new Rectangle(
    Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE);
protected Rectangle headerLayoutHints = new Rectangle(
    Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE);
protected Rectangle footerLayoutHints = new Rectangle(
    Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE);

protected int resetKey  = 'r';
protected int cancelKey = 'c';

private boolean changed = false;

/** Temporary Graphics object used to draw without showing to screen.*/
protected transient Graphics prevSetGraphics = null;

/** Temporary Graphics object created from current graphics object.  Used
    for querying only (for font metrics, for example).  Created and
    disposed (with methods createQueryGraphics() and disposeQueryGraphics())
    with each recalc/draw cycle. */
protected Graphics queryGraphics = null;

protected Locale locale = Locale.getDefault();

// Don't javadoc -- internal use only
protected boolean usingDoublePixels = true;
protected boolean layoutDone = false;


 

/**
 * Initializes assorted chart member variables. Used as a
 * convenience when writing a new <code>JCChart</code> constructor.
 */
protected void init()
{
    setOpaque(true);

    super.setLayout(new LegendComponentLayout());

    createHeader();
    createFooter();
    createLegend();
    createChartArea();

    resetDefaults();
    addDataView(0);

    enableEvents(AWTEvent.KEY_EVENT_MASK);
    enableEvents(AWTEvent.MOUSE_EVENT_MASK);
    enableEvents(AWTEvent.MOUSE_MOTION_EVENT_MASK);

    registerUI();
    updateUI();
}

/**
 * Creates and adds header to chart.
 */
protected void createHeader()
{
    header = new JLabel();
    header.setLocale(locale);
    header.setVisible(false);
    add(header);
}

/**
 * Creates and adds footer to chart.
 */
protected void createFooter()
{
    footer = new JLabel();
    footer.setLocale(locale);
    footer.setVisible(false);
    add(footer);
}

/**
 * Creates and adds legend to chart.
 */
protected void createLegend()
{
    legend = new JCGridLegend();
    legendManager = new JCChartLegendManager(this);
    legend.setLocale(locale);
    legend.setLegendPopulator(legendManager);
    legend.setLegendRenderer(legendManager);
    add(legend);
}

/**
 * Creates and adds legend to chart.
 */
protected void createChartArea()
{
    chartArea = new JCChartArea(locale);
    add(chartArea);
    // must do this manually.  Can't rely on addNotify() because axes
    // need to know their parent chart before that is finally called.
    chartArea.setParentOnAxes(this);


 
}

/**
 * Enables the events defined by the specified event mask parameter.
 * @param eventsToEnable
 */
public void enableChartEvents(long eventsToEnable) {
    enableEvents(eventsToEnable);
}

/**
 * Disables the events defined by the specified event mask parameter.
 * @param eventsToDisable
 */
public void disableChartEvents(long eventsToDisable) {
    disableEvents(eventsToDisable);
}

/**
 * Processes mouse click events on the chart.
 * @param e the mouse event to process
 */
protected void processMouseEvent(MouseEvent e)
{
    super.processMouseEvent(e);

    if (e.isConsumed()) {
        return;
    }

    int id = e.getID();
    switch(id) {
    case MouseEvent.MOUSE_PRESSED:
        mousePressed(e);
        break;
    case MouseEvent.MOUSE_RELEASED:
        mouseReleased(e);
        break;
    case MouseEvent.MOUSE_CLICKED:
        mouseClicked(e);
        break;
    case MouseEvent.MOUSE_EXITED:
        mouseExited(e);
        break;
    case MouseEvent.MOUSE_ENTERED:
        mouseEntered(e);
        break;
    }
}

/**
 * Processes mouse motion events on the chart.
 * @param e the mouse motion event to process
 */
protected void processMouseMotionEvent(MouseEvent e)
{
    super.processMouseMotionEvent(e);

    if (e.isConsumed()) {
        return;
    }

    int id = e.getID();
    switch(id) {
    case MouseEvent.MOUSE_MOVED:
        mouseMoved(e);
        break;
    case MouseEvent.MOUSE_DRAGGED:
        mouseDragged(e);
        break;
    }
}

/**
 * Processes key events on the chart.
 * @param e the key events to process
 */
protected void processKeyEvent(KeyEvent e)
{
    super.processKeyEvent(e);

    int id = e.getID();
    switch(id) {
    case KeyEvent.KEY_PRESSED:
        keyPressed(e);
        break;
    case KeyEvent.KEY_RELEASED:
        keyReleased(e);
        break;
    case KeyEvent.KEY_TYPED:
        keyTyped(e);
        break;
    }
}

/**
 * Processes <code>mousePressed</code> events for the chart: request focus and
 * perform associated actions.
 * @param event the <code>MouseEvent</code> instance that represents this mouse
 *     click
 */
public void mousePressed(MouseEvent event)
{
    int x = event.getX();
    int y = event.getY();

    if (!event.isMetaDown() &&
        (event.getModifiers() & InputEvent.ALT_MASK) == 0 && // left button clicked
        isFocusTraversable())
    {
        requestFocus();
    }

    EventTrigger et = findMatchingTrigger(event.getModifiers());
    if (et != null) {
        if (!allowUserChanges &&
            (et.action == EventTrigger.EDIT ||
             et.action == EventTrigger.CUSTOMIZE)) return;

        if (et.action == EventTrigger.PICK) {
            sendPickEvent(pick(new Point(x, y), null));
        }
        else if (et.action == EventTrigger.CUSTOMIZE) {
            launchPropertyPage(new Point(x,y));
        }
        else if (et.action != EventTrigger.NONE &&
                    getComponentAt(x, y) == chartArea) {
            chartArea.startAction(et,
                x - chartArea.getLocation().x,
                y - chartArea.getLocation().y);
        }
    }
}

/**
 * Processes <code>mouseClicked</code> events for the chart: do nothing.
 * @param event the <code>MouseEvent</code> instance that represents this mouse
 * event
 */
public void mouseClicked(MouseEvent event) {}

/**
 * Processes <code>mouseReleased</code> events for the chart: finish associated
 * actions.
 * @param event the <code>MouseEvent</code> instance that represents this mouse
 * event
 */
public void mouseReleased(MouseEvent event)
{
    int x = event.getX();
    int y = event.getY();

    if (chartArea.getActionHandler() != null) {
        chartArea.endAction(x - chartArea.getLocation().x,
                            y - chartArea.getLocation().y);
    }
}

/**
 * Processes <code>mouseEntered</code> events for the chart: do nothing.
 * @param event the <code>MouseEvent</code> instance that represents this mouse
 * event
 */
public void mouseEntered(MouseEvent event) {}

/**
 * Processes <code>mouseExited</code> events for the chart: do nothing.
 * @param event the <code>MouseEvent</code> instance that represents this mouse
 * event
 */
public void mouseExited(MouseEvent event) {}

/**
 * Processes <code>mouseDragged</code> events for the chart: perform associated
 * actions.
 * @param event the <code>MouseEvent</code> instance that represents this mouse
 * event
 */
public void mouseDragged(MouseEvent event)
{
    int x = event.getX();
    int y = event.getY();

    if (chartArea.getActionHandler() == null) {
        return;
    }
    chartArea.showAction(x - chartArea.getLocation().x,
                         y - chartArea.getLocation().y);
}

/**
 * Processes <code>mouseMoved</code> events for the chart: do nothing.
 * @param event the <code>MouseEvent</Code> instance that represents this mouse
 * event
 */
public void mouseMoved(MouseEvent event) {}

/**
 * Processes <code>keyTyped</code> events for the chart: do nothing.
 * @param event the <code>KeyEvent</code> instance that represents this keyboard
 * event
 */
public void keyTyped(KeyEvent event) {}

/**
 * Processes <code>keyPressed</code> events for the chart: reset chart or cancel
 * action as specified.
 * @param event the <code>KeyEvent</code> instance that represents this keyboard
 * event
 */
public void keyPressed(KeyEvent event)
{
    int key = (int) event.getKeyChar();
    if (key == resetKey) {
        reset();
    }
    else if (key == cancelKey) {
        cancel();
    }
}

/**
 * Processes <code>keyReleased</code> events for the chart: do nothing.
 * @param event the <code>KeyEvent</code> instance that represents this keyboard
 * event
 */
public void keyReleased(KeyEvent event) {}

/**
 * Ancestor Listener method. Do nothing in this case.
 * @param e the ancestor event
 */
public void ancestorAdded(AncestorEvent e) {}

/**
 * Ancestor Listener method. Do nothing in this case.
 * @param e the ancestor event
 */
public void ancestorMoved(AncestorEvent e) {}

/**
 * Ancestor Listener method. Removes ancestor listener and repaints chart to
 * get rid of attach lines on dwell labels.
 * @param e the ancestor event
 */
public void ancestorRemoved(AncestorEvent e)
{
    ((JComponent)e.getSource()).removeAncestorListener(this);
    JCChartLabel cl = chartLabelManager.getCurrentDwellLabel();
    if ((cl != null && cl.isConnected()) || (cl == null)) {
        repaint();
    }
}

public transient JCCustomizerPage customizer = null;

/**
 * Launches customizer.
 * @param p the point at which to launch the customizer
 */
protected void launchPropertyPage(Point p)
{
    if (!allowUserChanges) {
        return;
    }
    if (customizer == null) {
        setCursor( Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR) );
        try {
            Class newclass = Class.forName(getCustomizerName());
            JCCustomizerPage pp = (JCCustomizerPage) newclass.newInstance();
            pp.init();
            customizer = pp;
        }
        catch (Exception e) {
            e.printStackTrace(System.out);
            customizer = null;
            ErrorDialog.raise(this,
                JCChartBundle.string(JCChartBundle.key20, locale) +
                "\n" + e );
        }
        setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR) );
    }
    if (customizer != null) {
        customizer.setObject( this );
        customizer.launch();
    }
}

/**
 * Constructor that creates a <code>JCChart</code> of a particular type.
 * Note that the <code>ChartType</code> is actually a property of
 * <code>ChartDataView</code>. This constructor is provided as a convenience.
 * @param tp the chart type
 * @see com.klg.jclass.chart.ChartDataView
 */
public JCChart(int tp)
{
    this(tp, Locale.getDefault());
}

/**
 * Default constructor, required by Java Beans.
 */
public JCChart()
{
    this(PLOT, Locale.getDefault());
}

/**
 *  Constructor that creates a <code>JCChart</code> for a specified locale and
 *  of type PLOT.
 *  @param locale the Locale object
 */
public JCChart(Locale locale)
{
    this(PLOT, locale);
}

/**
 *  Constructor that creates a <code>JCChart</code> for a specified type and
 *  locale.
 *  @param type the chart type
 *  @param locale the Locale object
 */
public JCChart(int type, Locale locale)
{
    super();
    if (locale == null) {
        locale = Locale.getDefault();
    }
    else {
        this.locale = locale;
    }
    init();
    ((ChartDataView)data.elementAt(0)).setChartType(type);
}

/**
 * Static function to allow creation of a chart from a serialized file. The
 * serialized file can be created by using the JClass Chart Customizer's
 * <b>Save As</b> option.
 * JDK 1.1 only.
 * @param fileName the name of the serialization file
 * @return The newly created chart.
 */
public static JCChart makeChart(String fileName)
{
    JCChart c = null;
    try {
        FileInputStream fis = new FileInputStream(fileName);
        ObjectInputStream ois = new ObjectInputStream(fis);
        c = (JCChart)ois.readObject();
        ois.close();
    }
    catch (Exception e) {
        e.printStackTrace(System.out);
        c = null;
    }
    return(c);
}

/**
 * Gets the value of the <code>Changed</code> property, which determines
 * whether the chart requires a recalculation.
 * @return The current value of the changed flag.
 */
final public boolean isChanged()
{
    return(getChanged());
}

/**
 * Checks the value of the <code>ChangedFlag</code> property for the specified
 * mask (see {@link Changeable} for a list of valid masks). The
 * <code>ChangedFlag</code> property determines the type of action that the
 * chart needs to take regarding the current change.
 * @param mask the mask
 * @return The current value of the changed flag based on the mask.
 */
final public boolean isChanged(int mask)
{
    return(getChanged(mask));
}

/**
 * Gets the value of the <code>Changed</code> property, which determines
 * whether the chart requires a recalculation.
 * @return The current value of the changed flag.
 */
public boolean getChanged()
{
    // Chart is changed if one of its subcomponents has changed
    return(chartArea.isChanged()                 ||
           legend.isChanged()                    ||
           !chartArea.isValid()                  ||
           !legend.isValid()                     ||
           (header != null && !header.isValid()) ||
           (footer != null && !footer.isValid()));
}

/**
 * Checks the value of the <code>ChangedFlag</code> property for the specified
 * mask (see {@link Changeable} for a list of valid masks). The
 * <code>ChangedFlag</code> property determines the type of action the chart
 * needs to take regarding the current change.
 * @param mask the mask
 * @return The current value of the changed flag based on the mask.
 */
public boolean getChanged(int mask)
{
    // Chart is changed if one of its subcomponents has changed
    return(chartArea.isChanged(mask)             ||
           legend.isChanged(mask)                ||
           !chartArea.isValid()                  ||
           !legend.isValid()                     ||
           (header != null && !header.isValid()) ||
           (footer != null && !footer.isValid()));
}

/**
 * Recalculates the entire chart if it has been marked for
 * recalculation.
 */
public void recalc()
{
    if (!isChanged(Changeable.RECALC)) {
        return;
    }

	// create special graphics copy used to query font metrics, etc
	createQueryGraphics();
    try {
        // Recalc chart area first so that any new drawables will be created
        // in case the legend recalc needs them.
        chartArea.recalc();

        // Recalc legend here in case no layout manager is being used.
        legend.recalc();

    }
    catch (Exception e) {
        // Don't fail as a result of child exceptions.
        e.printStackTrace(System.out);
    }
	// dispose of special graphics copy used to query font metrics, etc
	disposeQueryGraphics();
    setChanged(false, Changeable.NO_CHANGE);
}

/**
 * Sets the value of the <code>Changed</code> property, which determines whether
 * the chart requires recalculation. This value is propagated to all
 * child components.
 * @param n boolean value of the <code>Changed</code> property
 * @param cf the changed flag to add in
 */
public void setChanged(boolean n, int cf)
{
    if (n == changed) {
        return;
    }
    changed = n;
    if (changed) {
        int legendCF = Changeable.NO_CHANGE;
        if ((cf & Changeable.NOTIFY_LEGEND_RECALC) > 0) {
            legendCF |= Changeable.RECALC;
        }
        if ((cf & Changeable.NOTIFY_LEGEND_REDRAW) > 0) {
            legendCF |= Changeable.REDRAW;
        }
        if (legendCF != Changeable.NO_CHANGE) {
            legend.setChanged(true, legendCF, false);
        }

        int caCF = Changeable.NO_CHANGE;
        if ((cf & Changeable.LAYOUT) > 0) {
            caCF |= Changeable.RECALC;
        }
        if ((cf & Changeable.NOTIFY_CA_RECALC) > 0) {
            caCF |= Changeable.RECALC;
        }
        if ((cf & Changeable.NOTIFY_CA_REDRAW) > 0) {
            caCF |= Changeable.REDRAW;
        }
        if (caCF != Changeable.NO_CHANGE) {
            chartArea.setChanged(true, caCF, false);
        }

        if (isBatched()) {
            invalidate();
        }
        else if (((cf & Changeable.RECALC) > 0) ||
                 ((cf & Changeable.LAYOUT) > 0))
        {
            update();
        }
        else if ((cf & Changeable.REDRAW) > 0) {
            repaint();
        }
    }
}

/**
 * Sets the value of the <code>Changed</code> property, which determines whether the
 * chart requires recalculation. This value is propagated to all
 * the child components. <P><b>Note:</b> This method is mandated by the
 * {@link Changeable} interface, but has no use in <code>JCChart</code>.
 * It is usually used for cases in which the <code>Changed</code> property must
 * be changed without forcing a recalculation.
 * @param n boolean value of the <code>Changed</code> property
 * @param cf the changed flag to add in
 * @param updateParent
 */
public void setChanged(boolean n, int cf, boolean updateParent)
{
    if (updateParent) {
        setChanged(n, cf);
    }
    else {
        changed = n;
    }
}

/**
 * Overrides the superclass's <code>paint()</code> method so that the chart is
 * fully painted and flushed before the listeners are called.
 * @param gc the graphics context
 */
public void paint(Graphics gc)
{
    if (antiAliasing == ANTI_ALIASING_DEFAULT) {
        super.paint(gc);
    }
    else {
        Graphics2D g2d = (Graphics2D)gc;
        Object save = g2d.getRenderingHint(RenderingHints.KEY_ANTIALIASING);
        Object antiAlias = RenderingHints.VALUE_ANTIALIAS_OFF;
        if (antiAliasing == ANTI_ALIASING_ON) {
            antiAlias = RenderingHints.VALUE_ANTIALIAS_ON;
        }
        if (antiAlias != save) {
            g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, antiAlias);
        }
        super.paint(gc);
        if (save != null && antiAlias != save) {
            g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, save);
        }
    }
    firePaint();
}

/**
 * Fires a paint event to interested listeners.
 */
public void firePaint()
{
    Enumeration e = JCListenerList.elements(listeners);
    while (e.hasMoreElements()) {
        ((JCChartListener)e.nextElement()).paintChart(this);
    }
}

/**
 * Gets the drawing area represented by this chart.
 * @return Rectangle object containing the drawing area.
 */
public Rectangle getDrawingArea()
{
    Rectangle rect = new Rectangle();
    getDrawingArea(rect);
    return(rect);
}

/**
 * Gets the drawing area represented by this chart and places it in the
 * provided rectangle.
 * @param rect Rectangle object to fill with drawing area coordinates
 */
public void getDrawingArea(Rectangle rect)
{
    Insets insets = getInsets();
    if (insets == null) {
        insets = new Insets(0, 0, 0, 0);
    }
    rect.setBounds(insets.left, insets.top,
                   Math.max(0, getSize().width - (insets.left + insets.right)),
                   Math.max(0, getSize().height - (insets.top + insets.bottom)));
}

/**
 * Gets the height of the drawing area represented by this chart.
 * @return The current drawing area height.
 */
public int getDrawingAreaHeight()
{
    Insets insets = getInsets();
    int k = 0;
    if (insets != null) {
        k = insets.top + insets.bottom;
    }
    return(Math.max(0, getSize().height - k));
}

/**
 * Gets the width of the drawing area represented by this chart.
 * @return The current drawing area width.
 */
public int getDrawingAreaWidth()
{
    Insets insets = getInsets();
    int k = 0;
    if (insets != null) {
        k = insets.left + insets.right;
    }
    return(Math.max(0, getSize().width - k));
}

/**
 * The top-level paint routine in <code>JCChart</code>.
 * @param gc the graphics context
 */
public void paintComponent(java.awt.Graphics gc)
{
    if (gc == null) {
        return;
    }

	// create special graphics copy used to query font metrics, etc
	createQueryGraphics();

    Graphics newgc = gc.create();

    // Draw our own opaque backgrounds.  Shouldn't this be
    // done by the superclass?  That code appears to be in the UI, which
    // is null for this component it seems.
    if (isOpaque()) {
        if (!newgc.getColor().equals(getBackground())) {
            newgc.setColor(getBackground());
        }
        Rectangle r = getBounds();
        newgc.fillRect(0, 0, r.width, r.height);
    }

    if (newgc != null) {
        newgc.dispose();
    }

	// dispose of special graphics copy used to query font metrics, etc
	disposeQueryGraphics();

    setChanged(false, Changeable.NO_CHANGE);
}

/**
 * Overrides the superclass's method to draw chart label connectors in
 * addition to chart children.
 * @param gc the graphics context
 */
protected void paintChildren(Graphics gc)
{
    super.paintChildren(gc);

    // paint the connectors for any dwell labels that happen to be up
    if (chartLabelManager != null) {
        chartLabelManager.paintConnectors(gc);
    }
}

/**
 * Prints this component and all of its subcomponents.
 * Overridden from <code>java.awt.Component</code>, but should be used
 * in the same way.
 * @param g the graphics context
 */
public void printAll(Graphics g)
{
    super.printAll(g);
}

/**
 * <code>JCChart</code> layout method. It lays out the header, footer, legend,
 *  and chart area.
 */
public void doLayout()
{
    layoutDone = true;
    Dimension sz = new Dimension();

    sz.width = getDrawingAreaWidth();
    sz.height = getDrawingAreaHeight();

    if (sz.width <= 0 || sz.height <= 0) {
        return;
    }

    // If the size of the chart has not changed, layout should only be
    // done if the header, footer, or legend has changed.  (This is because
    // the chart area always ends up with the leftover space.)  However if
    // the chart area has changed, we still need to call its recalc method to
    // force its redraw.
    if ((!chartArea.isChanged(Changeable.NOTIFY_PARENT_LAYOUT)) &&
        chartArea.isValid() &&
        (sz.width == curSize.width && sz.height == curSize.height))
    {

        if (!legend.isChanged(Changeable.RECALC) &&
            !legend.isChanged(Changeable.NOTIFY_PARENT_LAYOUT) &&
            legend.isValid() &&
            (header != null && header.isValid()) &&
            (footer != null && footer.isValid()))
        {
            if (chartArea.isChanged(Changeable.RECALC) ||
                !chartArea.isValid())
            {
                recalc();
            }
            repaint();
            return;
        }
    }

    recalc();

    // Since we're doing a full relayout, mark the chart labels for later
    // recalculation (they can't actually be recalced until the chart area
    // has been drawn).
    if (chartLabelManager != null) {
        chartLabelManager.markAllChartLabelsForRecalc();
    }

    curSize = sz;

    // Now that we've decided to actually layout and all the objects have
    // been calced, do the layout.
	createQueryGraphics();
    super.doLayout();
	disposeQueryGraphics();

    repaint();
}

/**
 * Sets layout hints for chart children. Hints are rectangle objects.
 * A value of <code>Integer.MAX_VALUE</code> in the rectangle's members
 * indicates to calculate default values during layout. Other values
 * indicate to the layout to use that value. For example, a rectangle
 * with members x=5, y=10, width=MAX_VALUE, and height=200, would indicate
 * to the layout mechanism that the chart child should be placed at (5,10),
 * have a height of 200, and use the default width.
 * <P>Layout hints are only used by the <code>DefaultChartLayout</code>
 * layout manager.
 * @param child chart child -- either the chartArea, legend, header, or footer
 * @param layoutHints Rectangle object containing the desired layout hints
 */
public void setLayoutHints(JComponent child, Rectangle layoutHints)
{
    if (child == null) {
        return;
    }

    if (child == chartArea) {
        chartAreaLayoutHints = new Rectangle(layoutHints.x, layoutHints.y,
                                             layoutHints.width,
                                             layoutHints.height);
    }
    else if (child == legend) {
        legendLayoutHints = new Rectangle(layoutHints.x, layoutHints.y,
                                          layoutHints.width,
                                          layoutHints.height);
    }
    else if (child == header) {
        headerLayoutHints = new Rectangle(layoutHints.x, layoutHints.y,
                                          layoutHints.width,
                                          layoutHints.height);
    }
    else if (child == footer) {
        footerLayoutHints = new Rectangle(layoutHints.x, layoutHints.y,
                                          layoutHints.width,
                                          layoutHints.height);
    }

    child.invalidate();
    validate();
}

/**
 * Gets layout hints for the specified chart child.
 * @param child chart child -- either the chartArea, legend, header, or footer
 * @return      Rectangle object containing the desired layout hints. Null is
 *              not one of the valid children of chart.
 */
public Rectangle getLayoutHints(JComponent child)
{
    if (child != null) {
        if (child == chartArea) {
            return(chartAreaLayoutHints);
        }
        else if (child == legend) {
            return(legendLayoutHints);
        }
        else if (child == header) {
            return(headerLayoutHints);
        }
        else if (child == footer) {
            return(footerLayoutHints);
        }
    }
    return(new Rectangle(Integer.MAX_VALUE, Integer.MAX_VALUE,
                         Integer.MAX_VALUE, Integer.MAX_VALUE));
}

/**
 * Sets the value of the <code>Batched</code> property, which controls whether
 * chart updates are accumulated.
 * @param bt if <code>true</code>, it will accumulate chart updates; if
 * <code>false</code>, it forces the accumulated updates to be processed
 */
public void setBatched(boolean bt)
{
    if (batched == bt) {
        return;
    }
    batched = bt;

    if (!bt) {
        update();
    }
}

/**
 * Gets the value of the <code>Batched</code> property.
 * @return <code>true</code> if updates are being accumulated;
 * <code>false</code> if updates are not accumulated.
 */
public boolean isBatched()
{
    return(batched);
}

/**
 * Store an object on this instance of JClass Chart.  This
 * feature can be used to store info related to a chart
 * for later access.
 * @param userObject user-specified object to store on this
 *                   instance of JClass Chart.
 */
public void setUserObject(Object userObject)
{
    this.userObject = userObject;
}

/**
 * Get the user-specified object being stored on this instance
 * of JClass Chart.
 * @return user-specified object being stored on this
 *         instance of JClass Chart.
 */
public Object getUserObject()
{
    return userObject;
}

/**
 * Forces the chart to re-layout and recalculate.
 */
public void update()
{
    invalidate();
    if (!isShowing()) {
        // Fix for PR 18172.  If the data is changed while a Pie chart
        // is not visible, the PieChartDraw's calcTransientData() method
        // doesn't get called.  Set a flag to ensure that the transient
        // data is recalculated.
        ChartDataView dv = getDataView(0);
        if (dv.isChanged()) {
            ChartDraw drawable = getChartArea().getDrawable(dv);
            if (drawable instanceof PieChartDraw) {
                ((PieChartDraw)drawable).setRecalcFlag();
            }
        }
        return;
    }
    validate();
}

/**
 * Overrides <code>javax.swing.JComponent addNotify()</code>.
 */
// largely used to put up various dialogs in limited and other versions of chart
public void addNotify()
{
    super.addNotify();
    moreAddNotify();
}

/**
 * Continues the add notify process.
 */
protected void moreAddNotify()
{

 


 

    doLayout();

 
}

/**
 * Overrides the superclass's method so that changes will generate redraws.
 * @param c the new background color
 */
public void setBackground(Color c)
{
    super.setBackground(c);
    setChanged(true, Changeable.RECALC | Changeable.NOTIFY_CA_RECALC);
}

/**
 * Overrides the superclass's method so that changes will generate redraws.
 * @param c the new foreground color
 */
public void setForeground(Color c)
{
    super.setForeground(c);
    setChanged(true, Changeable.RECALC | Changeable.NOTIFY_CA_RECALC |
                     Changeable.NOTIFY_LEGEND_RECALC);
}

/**
 * Overrides the superclass's method so that changes will generate redraws.
 * @param f the new font
 */
public void setFont(Font f)
{
    super.setFont(f);
    setChanged(true, Changeable.RECALC | Changeable.NOTIFY_CA_RECALC |
                Changeable.NOTIFY_LEGEND_RECALC);
}

/**
 * Overrides the superclass's method so that changes will generate redraws.
 * @param op the new value of the <code>opaque</code> property
 */
public void setOpaque(boolean op)
{
    super.setOpaque(op);
    setChanged(true, Changeable.REDRAW);
}

/**
 * Sets the value of the <code>AllowUserChanges</code> property, which
 * determines whether the user viewing the chart can modify chart values. Used
 * to allow edits to values and changes to parameters via the customizer.
 * @param change <code>true</code> means the user is allowed to modify graph
 * values; <code>false</code> means the user is not allowed to modify graph
 * values
 */
public void setAllowUserChanges(boolean change)
{
    allowUserChanges = change;
}

/**
 * Gets the value of the <code>AllowUserChanges</code> property, which
 * determines whether the user viewing the chart can modify chart values. Used
 * to allow edits to values and changes to parameters via the customizer.
 * @return <code>true</code> if the user is allowed to modify the graph values,
 * <code>false</code> otherwise
 */
public boolean getAllowUserChanges()
{
    return(allowUserChanges);
}

/**
 * Sets the value of the <code>WarningDialog</code> property, which controls
 * whether a dialog will appear when the chart has warning messages.
 * @param on if <code>true</code> (default), the warning dialog will appear; if
 * <code>false</code>, chart will not warn the user of anything
 */
public void setWarningDialog(boolean on)
{
    if (warningOn == on) {
        return;
    }
    warningOn = on;
}

/**
 * Gets the value of the <code>WarningDialog</code> property.
 * @return <code>true</code> (default) if the warning dialog is to appear;
 * <code>false</code> if chart is not to warn the user of anything
 */
public boolean isWarningDialog()
{
    return(warningOn);
}


/**
 * Gets the <code>Header</code> property.
 * <p>The <code>Header</code> property controls the object that controls the
 * display of the header.
 * @return The <code>JComponent</code> object associated with the header.  By
 * default, it is an instance of <code>JLabel</code>.
 */
public JComponent getHeader()
{
    return(header);
}

/**
 * Sets the <code>Header</code> property.
 * <p>The <code>Header</code> property controls the object that controls the
 * display of the header.
 * @param head the <code>JComponent</code> object associated with the header
 */
public void setHeader(JComponent head)
{
    if (head == null) {
        return;
    }
    if (header != null) {
        remove(header);
    }
    header = head;
    add(header);
    setChanged(true, Changeable.LAYOUT           |
                     Changeable.NOTIFY_CA_RECALC |
                     Changeable.NOTIFY_LEGEND_RECALC);
}

/**
 * Sets the <code>Footer</code> property.
 * <p>The <code>Footer</code> property controls the object that controls the
 * display of the footer.
 * @return The <code>JComponent</code> object associated with the footer.  By
 * default it is an instance of <code>JLabel</code>.
 */
public JComponent getFooter()
{
    return(footer);
}

/**
 * Sets the <code>Footer</code> property.
 * <p>The <code>Footer</code> property controls the object that controls the
 * display of the footer.
 * @param foot the <code>JComponent</code> object associated with the footer
 */
public void setFooter(JComponent foot)
{
    if (foot == null) {
        return;
    }
    if (footer != null) {
        remove(footer);
    }
    footer = foot;
    add(footer);
    setChanged(true, Changeable.LAYOUT           |
                     Changeable.NOTIFY_CA_RECALC |
                     Changeable.NOTIFY_LEGEND_RECALC);
}

/**
 * Gets the <code>Legend</code> property.
 * The <code>Legend</code> property controls the object that controls
 * the display of the legend.
 * @return The <code>JCLegend</code> object associated with the legend.
 */
public JCLegend getLegend()
{
    return(legend);
}

/**
 * Sets the <code>Legend</code> property.
 * <p>The <code>Legend</code> property controls the object that controls
 * the display of the legend. If the legend has no populator
 * or renderer set, they will be set with JClass Chart's default
 * legend populator and renderer.
 * @param jl the <code>JCLegend</code> object associated with the legend
 */
public void setLegend(JCLegend jl)
{
    if (jl == null) {
        return;
    }
    if (legend != null) {
        remove(legend);
    }
    legend = jl;
    if (legend.getLegendPopulator() == null) {
        legend.setLegendPopulator(legendManager);
    }
    if (legend.getLegendRenderer() == null) {
        legend.setLegendRenderer(legendManager);
    }
    add(legend);
    setChanged(true, Changeable.LAYOUT           |
                     Changeable.NOTIFY_CA_RECALC |
                     Changeable.NOTIFY_LEGEND_RECALC);
}

/**
 * Gets the value of the <code>ChartArea</code> property.
 * <p>The <code>ChartArea</code> property controls the object that controls the
 * display of the graph.
 * @return The <code>JCChartArea</code> object associated with this
 * <code>JCChart</code>.
 */
public JCChartArea getChartArea()
{
    return(chartArea);
}

/**
 * Used to implement the
 * {@link com.klg.jclass.util.legend.LegendComponentLayoutUser} interface.
 * <p>Returns the component used to draw the actual chart.
 * @return The subcomponent used to draw the actual chart.
 */
public JComponent getComponentArea()
{
    return(getChartArea());
}

/**
 * Sets the value of the <code>ChartArea</code> property.
 * Use of this method is an unsupported feature.
 * The <code>ChartArea</code> property controls
 * the object that controls the display
 * of the graph.
 * @param ca the new <code>JCChartArea</code> object
 */
public void setChartArea(JCChartArea ca)
{
    if (ca == null) {
        return;
    }
    if (chartArea != null) {
        remove(chartArea);
    }
    chartArea = ca;
    add(chartArea);
    setChanged(true, Changeable.LAYOUT           |
                     Changeable.NOTIFY_CA_RECALC |
                     Changeable.NOTIFY_LEGEND_RECALC);
}

/**
 * Gets the value of the specified index within the <code>Data</code> property.
 * The <code>Data</code>
 * property is an indexed property of <code>JCChart</code> that contains
 * one or more <code>ChartDataView</code> objects. Each
 * <code>ChartDataView</code> object controls the display of data using a
 * particular chart type.
 * @param index index of the data object to return
 * @return <code>ChartDataView</code> object.
 */
public ChartDataView getDataView(int index)
{
    ChartDataView d = null;
    try {
        d = (ChartDataView)data.get(index);
    }
    catch (Exception e) {}
    return(d);
}

/**
 * Creates a new <code>ChartDataView</code> and adds it to the chart
 * at the specified index, <B>replacing</B> the existing
 * <code>ChartDataView</code>
 * object at that specified index. If the index is one past the final occupied
 * index, the <code>ChartDataView</code> will be added to the end of the list.
 * Also see {@link #setDataView setDataView}.
 * @param index the index at which to set the new <code>ChartDataView</code>
 * @return The newly created <code>ChartDataView</code>.
 */
public ChartDataView addDataView(int index)
{
    ChartDataView ndv = new ChartDataView(locale);
    setDataView(index, ndv);
    return(ndv);
}

/**
 * Takes the specified <code>ChartDataView</code> and adds it to the chart
 * at the index indicated. The act of adding this new <code>ChartDataView</code>
 * will cause all existing <code>ChartDataView</code> objects to re-order
 * themselves. Also see {@link #addDataView addDataView}.
 * @param index position at which to add the new object
 * @param ndv new data view object
 * @exception IllegalArgumentException If the index does not address
 * a valid <code>ChartDataView</code>.
 */
public void setDataView(int index, ChartDataView ndv)
{
    if (index < 0 || index > data.size()) {
        throw new IllegalArgumentException("Invalid ChartDataView index " +
                                           "in JCChart");
    }
    if (ndv != null) {
        ndv.setParent(this);
    }
    if (index >= 0 && index < data.size()) {
        data.set(index, ndv);
    }
    else {
        data.add(index, ndv);
    }
    reorderDataViews();

    chartArea.clearDrawables();
    setChanged(true, Changeable.LAYOUT           |
                     Changeable.NOTIFY_CA_RECALC |
                     Changeable.NOTIFY_LEGEND_RECALC);
}

/**
 * Sets the value of the <code>DataView</code> property, which is an
 * indexed property that contains all the data to be
 * displayed in the chart.
 * @param views array of data views to be included in the <code>DataView</code>
 * property
 */
public void setDataView(List views)
{
    data = new Vector();
    if (views == null) {
        return;
    }
    for (Iterator i = views.iterator(); i.hasNext();) {
        ChartDataView cdv = (ChartDataView) i.next();
        if (cdv != null) {
            cdv.setParent(this);
        }
        data.add(cdv);
    }

    chartArea.clearDrawables();
    setChanged(true, Changeable.LAYOUT           |
                     Changeable.NOTIFY_CA_RECALC |
                     Changeable.NOTIFY_LEGEND_RECALC);
}

/**
 * Method used to find a data view, given its name.
 * @param name the name by which to find the data view
 * @return The <code>ChartDataView</code> which matches the
 * name (<code>null</code> if none found).
 */
public ChartDataView findDataView(String name)
{
    if (name == null || name.length() == 0) {
        return(null);
    }
    if (data == null || data.size() == 0) {
        return(null);
    }
    for (int i = 0; i < data.size(); i++) {
        ChartDataView dv = (ChartDataView)(data.elementAt(i));
        if (dv.getName().equals(name)) {
            return(dv);
        }
    }
    return(null);
}

/**
 * Gets the value of the <code>DataView</code> property, which
 * is an indexed property that contains all the data to be
 * displayed in the chart.
 * @return An array of data views that are included in the
 * <code>DataView</code> property.
 */
public List getDataView()
{
    if (data == null || data.size() == 0) {
        return(null);
    }
    return(new Vector(data));
}

/**
 * Gets the actual data view collection. Used within the chart
 * instead of using <code>getDataView()</code> in order to avoid copying.
 * @return The current list of data views.
 */
protected List getOriginalDataView()
{
    return(data);
}

/**
 * Find the index within the data view collection of the specified
 * ChartDataView.
 * @param dv The ChartDataView whose index is required.
 * @return The index of the dataView.  Returns -1 if not found.
 */
public int findDataViewIndex(ChartDataView dv)
{
    int dataViewIndex = -1;
    if (dv != null && data != null && data.size() > 0) {
        // Check to see if specified dataView is in the chart's list
        for (int i = 0; i < data.size(); i++) {
            ChartDataView dataView = (ChartDataView)(data.elementAt(i));
            if (dv == dataView) {
                dataViewIndex = i;
                break;
            }
        }
    }
    return(dataViewIndex);
}

/**
 * Removes a <code>ChartDataView</code> from the index specified.
 * The act of removing a <code>ChartDataView</code> to the chart will cause
 * all existing <code>ChartDataView</code> objects to re-order themselves.
 * @param index position from which to remove the object
 */
public void removeDataView(int index)
{
    try {
        chartArea.removeDrawable((ChartDataView)(data.elementAt(index)));
        data.remove(index);
        reorderDataViews();
        setChanged(true, Changeable.LAYOUT           |
                         Changeable.NOTIFY_CA_RECALC |
                         Changeable.NOTIFY_LEGEND_RECALC);
    }
    catch (Exception e) {}
}

/**
 * Gets the value of the <code>NumData</code> property, which
 * indicates how many <code>ChartDataView</code> objects are stored
 * in JClass Chart. It is a read-only property.
 * @return The number of <code>ChartDataView</code> objects inside
 * this <code>JCChart</code> instance.
 */
public int getNumData()
{
    int rval = 0;
    try {
        if (data != null) {
            rval = data.size();
        }
    }
    catch (Exception e) {
    }
    return(rval);
}

/**
 * Gets a single trigger from the <code>Triggers</code> property of JClass
 * Chart. The <code>Triggers</code> property is an indexed property that
 * contains a series of mappings, from AWT events to JClass Chart actions.
 * @param index index of trigger
 * @return EventTrigger instance.
 */
public EventTrigger getTrigger(int index)
{
    EventTrigger t = null;
    try {
        t = (EventTrigger)triggers.elementAt(index);
    }
    catch (Exception e) {}
    return(t);
}

/**
 * Gets the <code>NumTriggers</code> property, which indicates how many
 * event triggers have been specified.
 * @return The number of EventTrigger instances in the <code>Triggers</code>
 * property.
 */
public int getNumTriggers()
{
    int rval = 0;
    try {
        if (triggers != null) {
            rval = triggers.size();
        }
    }
    catch (Exception e) {}
    return(rval);
}

/**
 * Adds a new event trigger to the <code>Triggers</code> property.
 * @return EventTrigger instance, to be manipulated by the
 * developer.
 */
public EventTrigger addTrigger()
{
    EventTrigger rval = null;
    try {
        EventTrigger et = new EventTrigger();
        triggers.addElement(et);
        rval = et;
    }
    catch (Exception e) {}
    return(rval);
}

/**
 * Adds a new event trigger to the <code>Triggers</code> property.
 *
 * @param mod bitfield of modifiers from Event. Can include
 * <code>Event.SHIFT_MASK</code>, <code>Event.CTRL_MASK</code>,
 * <code>Event.META_MASK</code>, and <code>Event.ALT_MASK</code>. If not
 * including <code>META_MASK</code> (which is the same as the
 * third mouse button 3 mask), the bitfield is assumed to be intended to
 * be applied to the button 1 (the left mouse button).
 *
 * @param act action enum, one of <code>ROTATE</code>, <code>ZOOM</code>,
 * <code>TRANSLATE</code>, <code>EDIT</code>, <code>PICK</code>,
 * <code>DEPTH</code>, or <code>CUSTOMIZE</code>.
 *
 * @return EventTrigger instance, or <code>null</code> if error occurred.
 */
public EventTrigger addTrigger(int mod, int act)
{
    EventTrigger rval = null;
    try {
        EventTrigger et = new EventTrigger(mod, act);
        triggers.addElement(et);
        rval = et;
    }
    catch (Exception e) {}
    return(rval);
}


/**
 * Sets the value of the <code>Trigger</code> property at a specified
 * index. The <code>Trigger</code> property is an indexed property and
 * it contains all the information necessary to map user events
 * into JClass Chart actions. The <code>Trigger</code> property comprises a
 * number of <code>EventTrigger</code> objects.
 * @param index index within the <code>Trigger</code> property
 * @param trig <code>EventTrigger</code> object to be placed at the specified
 * index
 * @exception IllegalArgumentException If the index does not
 * address a valid event trigger.
 */
public void setTrigger(int index, EventTrigger trig)
{
    if (triggers == null) {
        return;
    }
    if (trig == null) {
        if (index < 0 || index >= triggers.size()) {
            throw new IllegalArgumentException(ERRMSG_INVALID_TRIGGER_INDEX);
        }
        removeTrigger(index);
    }
    else {
        if (index < 0 || index > triggers.size()) {
            throw new IllegalArgumentException(ERRMSG_INVALID_TRIGGER_INDEX);
        }
        triggers.insertElementAt(trig, index);
    }
}

/**
 * Removes the event trigger at the specified index.
 * @param index index of the <code>EventTrigger</code> instance to
 * be removed
 */
public void removeTrigger(int index)
{
    try {
        triggers.removeElementAt(index);
    }
    catch (Exception e) {}
}


/**
 *  Searches through the list of event triggers looking for the one
 *  that has been attached to the specified modifier.
 *  @param mods the modifier being searched for
 *  @return The <code>EventTrigger</code> object found or null if none matches.
 */
public EventTrigger findMatchingTrigger(int mods)
{
    if (triggers == null) {
        return(null);
    }
    EventTrigger ret = null;
    for (int i = 0; i < triggers.size(); i++) {
        EventTrigger e = (EventTrigger)triggers.elementAt(i);
        if (e.modifiers == mods) {
            ret = e;
            break;
        }
    }
    return(ret);
}

/**
 * Sets the value of the <code>ResetKey</code> property.
 * The <code>ResetKey</code> property specifies the key used to
 * perform a reset operation.
 * @param k Key that will trigger a reset
 */
public void setResetKey(int k)
{
    resetKey = k;
}

/**
 * Gets the value of the <code>ResetKey</code> property.
 * The <code>ResetKey</code> property specifies the key used to
 * perform a reset operation.
 * @return Key that will trigger a reset.
 */
public int getResetKey()
{
    return(resetKey);
}

/**
 * Sets the value of the <code>CancelKey</code> property.
 * The <code>CancelKey</code> property specifies the key used to
 * perform a cancel operation.
 * @param k Key that will trigger a cancel
 */
public void setCancelKey(int k)
{
    cancelKey = k;
}

/**
 * Gets the value of the <code>CancelKey</code> property.
 * The <code>CancelKey</code> property specifies the key used to
 * perform a cancel operation.
 * @return Key that will trigger a cancel.
 */
public int getCancelKey()
{
    return(cancelKey);
}


/**
 * A call to signify the beginning of a rotate event.
 * <BR><b>Note:</b> Not currently implemented.
 */
public void rotateStart() {}

/**
 * A call to rotate a graph to the specified locations.
 * <BR><b>Note:</b> Not currently implemented.
 */
public void rotate() {}

/**
 * A call to signify the end of a rotate event.
 * <BR><b>Note:</b> Not currently implemented.
 */
public void rotateEnd() {}

/**
 * Performs a zoom operation along the specified axis.
 * @param start new axis minimum value
 * @param end new axis maximum value
 * @param axis axis along which to perform zoom
 * @param update if <code>true</code>, a redisplay is forced; if
 * <code>false</code>, the redisplay does not occur. If translating along two
 * axes, the first <code>translate()</code> call should have
 * <code>update=false</code>, and the second should have
 * <code>update=true</code>.
 */
public void zoom(double start, double end, JCAxis axis, boolean update)
{
    if (axis == null) {
        return;
    }

    // Don't allow zoom if requested axis limits are too tight with regards
    // the precision
    double err = JCChartUtil.calcError(axis.getPrecision());
    double diff = 0.0;
    if (axis.logarithmic) {
        diff = Math.abs(JCChartUtil.log10(start) - JCChartUtil.log10(end));
    }
    else {
        diff = Math.abs(start - end);
    }

    if (diff < err) {
        return;
    }

    // Don't allow zoom if there's a chance that the precision will
    // be exceeded during calculation of axis annotations and tick marks.
    double inc = diff / 30.0;   // Assuming maximum number of tick marks is 30.
    if (start + inc == start) {
        return;
    }

    // EGH - should the chart area be informed of this?
    // JCChartArea.setZoomX

    if (start > end) {
        double tmp = start;
        start = end;
        end = tmp;
    }

    axis.min.value = start;
    axis.min.isDefault = false;
    axis.max.value = end;
    axis.max.isDefault = false;
    axis.setChanged(true, Changeable.RECALC, false);

    sendEvent(axis);

    if (update) {
        chartArea.setChanged(true, Changeable.RECALC);
    }
}

/**
 * Performs a scale operation.
 * <br>A scale factor of 0.5 is 50% and makes everything smaller (zoom out);
 * a scale factor of 2 is 200% and makes everything bigger (zoom in).
 * @param sc scale percentage relative to the default appearance of
 *     the graph
 * @param axis the axis on which to perform the scale operation
 * @param update whether or not to set the changed flag on the chart area
 */
public void scale(double sc, JCAxis axis, boolean update)
{
    if (axis == null) {
        return;
    }

    // inform the ChartArea?

    // scale towards the origin
    axis.min.value /= sc;
    axis.min.isDefault = false;
    axis.max.value /= sc;
    axis.max.isDefault = false;

    axis.setChanged(true, Changeable.RECALC,false);

    sendEvent(axis);

    if (update) {
        chartArea.setChanged(true, Changeable.RECALC);
    }
}

/**
 * A call to signify the start of a translate event.
 * @param axis axis along which to begin translating. Note that
 * it is possible to translate along multiple axes simultaneously
 */
public void translateStart(JCAxis axis)
{
    if (axis == null) {
        return;
    }
    axis.min.isDefault = false;
    axis.max.isDefault = false;

    sendEvent(axis);

    axis.setChanged(true, Changeable.RECALC, false);
}

/**
 * A call to translate a specified amount along the specified axis.
 * @param delta amount to translate along the axis
 * @param axis axis along which to perform the translation
 * @param update if <code>true</code>, a redisplay is forced; if
 * <code>false</code>, the redisplay does not occur. If translating along two
 * axes,
 * the first <code>translate()</code> call should have
 * <code>update=false</code>, and the second should have
 * <code>update=true</code>.
 */
public void translate(double delta, JCAxis axis, boolean update)
{
    if (axis == null) {
        return;
    }
    axis.translate(delta);

    sendEvent(axis);

    if (update) {
        chartArea.setChanged(true, Changeable.RECALC);
    }
}

/**
 * Performs a reset on the chart. Returns the chart back to the
 * default min and max.
 */
public void reset()
{
    chartArea.reset();
}

/**
 * Performs a cancel on the chart. Cancel will stop any current
 * action.
 */
public void cancel()
{
    chartArea.cancel();
}

/**
 * Sets the order attribute of each attached <code>ChartDataView</code>
 * object to its index in the data array.
 */
protected void reorderDataViews()
{
    for (int i = 0; i < data.size(); i++) {
        ChartDataView dv = (ChartDataView)data.elementAt(i);
        dv.setDrawingOrder(i);
    }
}

/**
 * Gets the value of the <code>About</code> property.
 * The <code>About</code> property displays contact information for
 * Quest Software in the BeanBox.
 * @return The about String.
 */
public String getAbout()
{
    return(JCVersion.getVersionString());
}

/**
 * Sets the value of the <code>About</code> property.
 * The <code>About</code> property displays contact information for
 * Quest Software in the BeanBox.
 * @param s The new about String (ignored)
 */
public void setAbout(String s) {}

/*
 * Gets the value of the <code>License</code> property.
 * The <code>License</code> property displays license status information for
 * Quest Software in the BeanBox if it's invalid.
 */

 

/*
 * Sets the value of the <code>License</code> property.
 * The <code>License</code> property displays license status information for
 * Quest Software in the BeanBox if it's invalid.
 */

 

/**
 * Internal convenience method that finds the first visible
 * <code>ChartDataView</code> object that references a particular axis.
 * @param ax axis for which the corresponding data object is to be found
 * @return The ChartDataView that was found (null if none found).
 */
protected ChartDataView findData(JCAxis ax)
{
    if (ax == null) {
        return(null);
    }
    for (int i = 0; i < data.size(); i++) {
        ChartDataView dv = (ChartDataView)data.elementAt(i);
        if ((dv.getXAxis() == ax || dv.getYAxis() == ax) && dv.isVisible()) {
             return(dv);
        }
    }
    return(null);
}

/**
 * Given a screen position in pixels, returns a <code>JCDataIndex</code>
 * object that represents the closest point in a data series in the specified
 * <code>ChartDataView</code> instance. If no data view is supplied, all data
 * views are considered when finding the closest point. If the data in a
 * data view is being updated when <code>pick()</code> is called, the result
 * may be incorrect.
 * @param p pick point in pixels relative to the <code>JCChart</code> object
 * @param data data view on which to perform pick; if <code>null</code>,
 * all data views are used
 * @return <code>JCDataIndex</code> object representing the picked point.
 */
public JCDataIndex pick(Point p, ChartDataView data)
{
    Component comp = getComponentAt(p.x, p.y);
    if (comp == null) {
        return(null);
    }

    Point pp = new Point(p.x - comp.getLocation().x,
                         p.y - comp.getLocation().y);
    if (comp instanceof JCLegend) {
        return(pickLegend(pp, data));
    }
    else if (comp instanceof JCChartArea) {
        return(chartArea.pick(pp, data, ChartDataView.PICK_FOCUS_LOCAL));
    }
    else if (comp != null) {
        JCDataIndex di = new JCDataIndex(-1, null, -1);
        di.obj = comp;
        return(di);
    }
    return(null);
}

/**
 * Internal pick method.
 * Chooses the data series represented by the legend item at the
 * provided position. If a data view is supplied, picks a series
 * on the provided data view only. Otherwise, tries to find the
 * appropriate series from any data view.
 * @param p pick point relative to legend
 * @param data data view on which to perform pick; if <code>null</code>,
 * all data views are used
 * @return <code>JCDataIndex</code> object representing the picked point.
 */
JCDataIndex pickLegend(Point p, ChartDataView data)
{
    JCLegendItem pickItem = legend.pick(p);

    JCDataIndex di = null;
    di = new JCDataIndex(-1, null, -1);
    di.obj = legend;

    if (pickItem == null) {
        return(di);
    }

    JCDataIndex pickIndex = (JCDataIndex)pickItem.itemInfo;

    ChartDataView dv = pickIndex.getDataView();
    if ((data != null) && (dv != data)) {
        return(di);
    }
    di.dataView = dv;
    di.series = pickIndex.getSeries();
    di.seriesIndex = pickIndex.getSeriesIndex();

    return(di);
}

/**
 * Returns the position in screen pixels of a particular point in a
 * particular data series.
 * @param dv the data view containing the specified series
 * @param pt point index
 * @param series data series index or <code>JCPieChartFormat.OTHER_SLICE</code>
 * if specifying the other slice of a pie chart
 * @return AWT Point object representing position in screen pixels
 * relative to the <code>JCChart</code> object or <code>null</code> if series or point does not
 * exist or is not showing.
 */
public Point unpick(ChartDataView dv, int pt, int series)
{
    if (series == -1) {
        return(null);
    }
    if (series >= 0) {
        ChartDataViewSeries seriesObject = dv.getSeries(series);
        int first = seriesObject.getFirstPoint();
        int last = seriesObject.getLastPoint();
        if (pt < first || pt > last) {
            return(null);
        }
    }

    Point p = chartArea.getDrawable(dv).unpick(pt, series);
    // Adjust unpicked point to screen pixels relative to JCChart object.
    if (p != null) {
        Point l = chartArea.getLocation();
        p.x += l.x;
        p.y += l.y;

        Rectangle r = chartArea.getDrawingArea();
        p.x += r.x;
        p.y += r.y;
    }
    return(p);
}

/**
 * Returns the position in screen pixels of a particular point in a
 * particular data series.
 * @param pt point index
 * @param series data series
 * @return AWT Point object representing position in screen pixels
 * relative to the <code>JCChart</code> object or <code>null</code> if series or
 * point does not exist or is not showing.
 */
public Point unpick(int pt, ChartDataViewSeries series)
{
    if (series == null) {
        return(null);
    }
    return(unpick(series.getParent(), pt,
                  series.getParent().getSeriesIndex(series)));
}

/*
 * JDK-1.1 style event handling methods
 */
/**
 * Adds listener to changes in chart. Called after zoom, translate, and scale
 * (when used either through method calls or interactive triggers), as well
 * as following rotate, depth change, or edit (interactive only).
 * @param l the chart listener to be added
 */
public void addChartListener(JCChartListener l)
{
    listeners = JCListenerList.add(listeners, l);
}

/**
 * Removes listener to changes in chart from list of listeners.
 * @param l the chart listener to be removed
 */
public void removeChartListener(JCChartListener l)
{
    listeners = JCListenerList.remove(listeners, l);
}

/**
 * Send an event to each of the chart listeners.
 * @param axis the axis to which the event pertains
 */
protected void sendEvent(JCAxis axis)
{
    JCChartEvent ev = new JCChartEvent(this, axis);

    Enumeration e = JCListenerList.elements(listeners);
    while(e.hasMoreElements()) {
        ((JCChartListener)e.nextElement()).changeChart(ev);
    }
}

/**
 * Adds listener to chart pick events.
 * @param l the pick listener to add
 */
public void addPickListener(JCPickListener l)
{
    pickListeners = JCListenerList.add(pickListeners,l);
}

/**
 * Removes listener to chart pick events from list of listeners.
 * @param l the pick listener to remove
 */
public void removePickListener(JCPickListener l)
{
    pickListeners = JCListenerList.remove(pickListeners,l);
}

/**
 * Sends pick event to all objects listening for it.
 * @param di the data index to which the event pertains
 */
public void sendPickEvent(JCDataIndex di)
{
    JCPickEvent ev = new JCPickEvent(this,di);

    Enumeration e = JCListenerList.elements(pickListeners);
    while (e.hasMoreElements()) {
        ((JCPickListener)e.nextElement()).pick(ev);
    }
}

/**
 * Requests the input focus. A <code>FocusEvent</code> will be generated
 * to indicate that the focus has been gained.
 * @see java.awt.event.FocusEvent
 */
public void requestFocus()
{
    super.requestFocus();
    if (isFocusTraversable()) {
        processEvent(new FocusEvent(this, FocusEvent.FOCUS_GAINED));
    }
}

/**
 * Overrides the superclass's <code>isFocusTraversable</code> method to always
 * return <code>true</code>, which is required to get keyPress events.
 * @return Always returns true
 */
public boolean isFocusTraversable()
{
    return(true);
}

/**
 * Resets the line, symbol, and fill color and pattern
 * to the default index values.
 * The colors are defined in {@link JCStyle}.
 */
public void resetDefaults()
{
    lineColorIndex   = 0;
    fillColorIndex   = 0;
    symbolColorIndex = 0;
    symbolShapeIndex = 1;
}

/**
 * Sets the initial line color index.
 * @param colorIndex the new color index
 */
public void setLineColorIndex(int colorIndex)
{
    lineColorIndex = colorIndex;
}

/**
 * Sets the initial fill color index.
 * @param colorIndex the new fill color index
 */
public void setFillColorIndex(int colorIndex)
{
    fillColorIndex = colorIndex;
}

/**
 * Sets the initial symbol color index.
 * @param colorIndex the new symbol color index
 */
public void setSymbolColorIndex(int colorIndex)
{
    symbolColorIndex = colorIndex;
}

/**
 * Sets the initial symbol shape index.
 * @param shapeIndex the new symbol shape index
 */
public void setSymbolShapeIndex(int shapeIndex)
{
    symbolShapeIndex = shapeIndex;
}

/**
 * Gets the initial line color index.
 * @return The current initial line color index.
 */
public int getLineColorIndex()
{
    return(lineColorIndex);
}

/**
 * Gets the initial fill color index.
 * @return The current initial fill color index.
 */
public int getFillColorIndex()
{
    return(fillColorIndex);
}

/**
 * Gets the initial symbol color index.
 * @return The current initial symbol color index.
 */
public int getSymbolColorIndex()
{
    return(symbolColorIndex);
}

/**
 * Gets the initial symbol shape index.
 * @return The current initial symbol shape index.
 */
public int getSymbolShapeIndex()
{
    return(symbolShapeIndex);
}

/**
 * Override this method to provide blank tool tip text when you want
 * to bring up a dwell label. Text tells the tool tip manager to bring
 * up the tool tip, to which the dwell label is later added.
 * @param event the mouse event for the tool tip
 * @return The tool tip text.
 */
public String getToolTipText(MouseEvent event)
{
    int x = event.getX();
    int y = event.getY();
    Component comp = getComponentAt(x, y);

    // Is there a dwell label that corresponds to this section of the screen?
    if (comp instanceof JCChartArea) {
        if (chartLabelManager != null) {
            if (chartLabelManager.locateCurrentDwellLabel(x, y)) {
                return("");
            }
            else {
                return(super.getToolTipText(event));
            }
        }
    }
    return(super.getToolTipText(event));
}

/**
 * Override this method to return the location of the dwell label
 * being brought up.
 * @param event the mouse event for the tool tip
 * @return The tool tip location.
 */
public Point getToolTipLocation(MouseEvent event)
{
    //Return position of saved Dwell Label above
    if (chartLabelManager != null) {
        Point ap = chartLabelManager.getCurrentDwellLabelAttachPoint();
        if (ap != null) {
            return(ap);
        }
    }

    return(super.getToolTipLocation(event));
}

/**
 * Override this method to add a dwell label to the tool tip.
 * @return The newly created tool tip.
 */
public JToolTip createToolTip()
{
    // Create tool tip
    JToolTip tip = super.createToolTip();

    // Add dwell label to tool tip.  Make sure to set size of tip and
    // location of dwell label child correctly.
    if (chartLabelManager != null) {
        JCChartLabel dwellLabel = chartLabelManager.getCurrentDwellLabel();
        JComponent dwellComp =
            chartLabelManager.getCurrentDwellLabelComponent();

        if (dwellComp != null) {
            tip.add(dwellComp);
            //tip.setOpaque(false);

            //If dwell label needs connector, draw it, but register with
            //ancestor listener to redraw chart when dwell pops down
            if (dwellLabel.isConnected()) {
                tip.addAncestorListener(this);
                Graphics gc = getGraphics();
                if (gc != null) {
                    gc.setColor(getForeground());
                    dwellLabel.drawConnector(gc);
					gc.dispose();
                }
            }
            dwellComp.setLocation(0,0);
            tip.setPreferredSize(dwellComp.getPreferredSize());
            return(tip);
        }
    }

    return(super.createToolTip());
}

/**
 * Gets the {@link JCChartLabelManager} interface for managing chart labels. If
 * no manager has been set and the default manager has not yet been created,
 * calling this function will create an instance of the default manager.
 * @return The chart label manager class.
 */
public JCChartLabelManager getChartLabelManager()
{
    if (chartLabelManager == null) {
        try {
            String lmString = "com.klg.jclass.chart.JCDefaultChartLabelManager";
            chartLabelManager =
                (JCChartLabelManager) Class.forName(lmString).newInstance();
            chartLabelManager.setParentChart(this);
        }
        catch (Exception e) {}
    }
    return(chartLabelManager);
}

/**
 * Sets the <code>ChartLabelManager</code> property, a class that
 * implements the {@link JCChartLabelManager} interface.  This method is
 * not often used; usually the user would use the default manager provided
 * with JClass Chart.
 * @param clm a class the implements the <code>JCChartLabelManager</code>
 *     interface
 */
public void setChartLabelManager(JCChartLabelManager clm)
{
    if (chartLabelManager != clm) {
        chartLabelManager = clm;
        chartLabelManager.setParentChart(this);
    }
}


/**
 * Returns <code>true</code> if a chart label manager instance has
 * been created.
 * @return Does this <code>JCChart</code> component have a chart label manager?
 */
public boolean hasChartLabelManager()
{
    return(chartLabelManager != null);
}

/**
 * Gets the value of the <code>CustomizerName</code> property.
 * The <code>CustomzierName</code> property controls the name of the customizer
 * used by JClass Chart. It must be a full class name. The default
 * value is <code>com.klg.jclass.chart.customizer.ChartCustomizer</code>.
 * @return The current customizer name.
 */
public String getCustomizerName()
{
    return(customizerName);
}

/**
 * Sets the value of the CustomizerName property.
 * The CustomzierName property controls the name of the customizer
 * used by JClass Chart. It must be a full class name. The default
 * value is <code>com.klg.jclass.chart.customizer.ChartCustomizer</code>.
 * @param s the new customizer name
 */
public void setCustomizerName(String s)
{
    if (s == null) {
        return;
    }
    customizerName = s;
}

/**
 * Returns the UI for <code>JCChart</code>.
 * @return The current JCChartUI objet.
 */
public JCChartUI getUI()
{
    return((JCChartUI)ui);
}

/**
 * Sets the UI for <code>JCChart</code>.
 * @param newUI the new UI object
 */
public void setUI(ComponentUI newUI)
{
    super.setUI(newUI);
}

/**
 * Updates the UI for <code>JCChart</code>.
 */
public void updateUI()
{
    ComponentUI compUI = UIManager.getUI(this);

    // Because Swing's UI Manager caches not only our UI string, but also
    // the resulting UI class that it refers too, UIManager.getUI() can
    // return a UI class from a different copy of the chart jar rather
    // than the one this instance of class resides in.
    // To test for this case, do a simple cast of the retrieved UI.
    // If the cast fails, ask the UI class to create a new instance
    // of itself for use with this class.
    try {
        JCChartUI chartUI = (JCChartUI) compUI;
    }
    catch (Exception ee) {
        compUI = JCChartUI.createUI(this);
    }
    setUI(compUI);
}

/**
 * Returns the UIClass ID for <code>JCChart</code>.
 * @return The current UIClassID for this component.
 */
public String getUIClassID()
{
    return(uiClassID);
}

/**
 * Tells the UIManager class which class to use for JCChart.
 */
public void registerUI()
{
    UIDefaults uiDefaults = UIManager.getDefaults();
    uiDefaults.put(uiClassID, "com.klg.jclass.chart.JCChartUI");
}

// Hack for Visual Cafe event usage.  Cafe insists all object passed
// to listener methods are events that a getSource() can be done on.
// One of our listeners takes a chart, so we need this.
/**
 * Returns this chart as an object.
 * @return This chart as an object.
 */
public Object getSource()
{
    return(this);
}

/**
 * Overrides the <code>Container</code> class's method to return only
 * components that are actually showing.
 * @param x the x position of the component
 * @param y the y position of the component
 * @return The component at (x,y).
 */
public Component getComponentAt(int x, int y)
{
    if (!contains(x, y)) {
        return(null);
    }
    int ncomponents = getComponentCount();
    Component component[] = getComponents();
    for (int i = 0 ; i < ncomponents ; i++) {
        Component comp = component[i];
        if (comp != null) {
            Point p = comp.getLocation();
            if (comp.isShowing() && comp.contains(x - p.x, y - p.y)) {
                return(comp);
            }
        }
    }
    return(this);
}

/**
 * Takes a snapshot of the current chart.
 * @return Image object containing snapshot of chart.
 */
public Image snapshot()
{
    return snapshot(BufferedImage.TYPE_INT_ARGB);
}

/**
 * Takes a snapshot of the current chart and places it in an image of the
 * specified type. The image types are as specified in the
 * <code>BufferedImage</code> class. <code>BufferedImage.TYPE_INT_ARGB</code>
 * is a good default for representing many possible colors.  If using less than
 * 256 colors, <code>BufferedImage.TYPE_BYTE_INDEXED</code> may prove to
 * generate faster and smaller images.<br>
 * @param imagetype the type of image to write to, as defined in the
 *        java.awt.image.BufferedImage class
 * @return Image object containing snapshot of chart.
 */
public Image snapshot(int imagetype)
{
    Dimension size = getSize();
    Image img;
    try {
        img = new BufferedImage(size.width, size.height, imagetype);
    }
    catch (Exception e) {
        img = createImage(size.width,size.height);
    }
    snapshot(img);
    return(img);
}

/*
 * Takes a snapshot of the current chart and places it in the supplied image.
 */
public void snapshot(Image img)
{
    if (img != null) {
        Graphics igc = img.getGraphics();
        //Gotta set the clip, or else paint throws an exception
        igc.setClip(0, 0, getSize().width, getSize().height);
        paint(igc);
		igc.dispose();
    }
}

/**
 * Creates image for use in drawing rotated text and other components.
 * @param width the width of the new image
 * @param height the height of the new image
 * @return The newly created image.
 */
protected Image createLocalImage(int width, int height)
{
    return(createImage(width, height));
}

//ServerRenderable methods
/**
 * Overrides parent <code>getGraphics()</code>. If another Graphics object has
 * been set to be the current one, a copy of that will be returned. Otherwise,
 * call the superclass' <code>getGraphics()</code> method.
 * @return The Graphics object to use to draw this chart.
 */
public Graphics getGraphics()
{
    if (prevSetGraphics == null) {
        return(super.getGraphics());
    }
	// make sure to return a copy of the user-set graphics object
	// this is in keeping with the contract for getGraphics()
	// some Java code calls dispose() on the object returned from this method
    return(prevSetGraphics.create());
}

/**
 * Provides a way for the chart to draw to a custom <code>Graphics</code>
 * object. Call with a null to specify use of the default <code>Graphics</code>
 * object used to draw to the screen.
 * @param g <code>Graphics</code> object to use for drawing
 */
public void setGraphics(Graphics g)
{
    prevSetGraphics = g;
	if (queryGraphics != null) {
		disposeQueryGraphics();
		createQueryGraphics();
	}
    getChartArea().setGraphics(g);
    getLegend().setGraphics(g);
}

/**
 * Create a copy of the current graphics object that will be used for
 * querying only.  It should not be disposed directly, but only by
 * calling disposeQueryGraphics().  This technique is used to cache
 * a Graphics object for such things as querying FontMetrics over
 * the course of a chart recalc/draw cycle, preventing unnecessary
 * object creation.
 */
public void createQueryGraphics()
{
	if (queryGraphics != null) {
		queryGraphics.dispose();
		queryGraphics = null;
	}
	queryGraphics = getGraphics();
}

/**
 * Dispose of the current query graphics object, which is a copy of the
 * current graphics object that is used for querying only.  It should
 * not be disposed of directly, but only by calling this method.  This
 * technique is used to cache a Graphics object for such things as
 * querying FontMetrics over the course of a chart recalc/draw cycle,
 * preventing unnecessary object creation.
 */
public void disposeQueryGraphics()
{
	if (queryGraphics != null) {
		queryGraphics.dispose();
		queryGraphics = null;
	}
}

/**
 * Return the current query graphics object, if one exists.  Otherwise,
 * one will be created.  The query graphics object is a
 * copy of the current graphics object that will be used for
 * querying only.  It should not be disposed directly, but only by
 * calling disposeQueryGraphics().  This technique is used to cache
 * a Graphics object for such things as querying FontMetrics over
 * the course of a chart recalc/draw cycle, preventing unnecessary
 * object creation.
 */
public Graphics getQueryGraphics()
{
	// Fail-Safe: if query graphics doesn't exist, create one.
	// It'll be disposed at the end of the next draw cycle anyways
	if (queryGraphics == null) {
		createQueryGraphics();
	}

	return queryGraphics;
}

/**
 * Internal use only.  Don't use.
 */
public Image serverSnapshot()
{
    return(ServerRenderer.snapshot(this));
}

/**
 * Internal use only.  Don't use.
 */
public void serverPaint(Graphics igc)
{
    ServerRenderer.paintComponent(igc, this);
}

/**
 * Internal use only.  Don't use.
 */
public void setUsingDoublePixels(boolean usingDoubles)
{
    usingDoublePixels = usingDoubles;
}

/**
 * Internal use only.  Don't use.
 */
public boolean isUsingDoublePixels()
{
    return(usingDoublePixels);
}

/**
 * Gets the antiAliasing value.  This value determines whether antiAliasing is
 * on when painting the chart and all its subcomponents.  If set to
 * <code>ANTI_ALIASING_ON</code> or <code>ANTI_ALIASING_OFF</code>, the anti
 * aliasing rendering hint will be explicity turned on or off on the graphics
 * object used by the chart to paint its children.  If set to
 * <code>ANTI_ALIASING_DEFAULT</code>, the graphics object will be untouched
 * with respect to anti aliasing when the chart is painted.
 * @return The current antiAliasing value.
 */
public int getAntiAliasing()
{
    return(antiAliasing);
}

/**
 * Sets the antiAliasing value.  This value determines whether antiAliasing is
 * on when painting the chart and all its subcomponents.  If set to
 * <code>ANTI_ALIASING_ON</code> or <code>ANTI_ALIASING_OFF</code>, the anti
 * aliasing rendering hint will be explicity turned on or off on the graphics
 * object used by the chart to paint its children.  If set to
 * <code>ANTI_ALIASING_DEFAULT</code>, the graphics object will be untouched
 * with respect to anti aliasing when the chart is painted.
 * @param antiAliasing The new antiAliasing value.
 */
public void setAntiAliasing(int antiAliasing)
{
    if (antiAliasing < ANTI_ALIASING_DEFAULT ||
        antiAliasing > ANTI_ALIASING_OFF)
    {
        throw new IllegalArgumentException("The antiAliasing value must be " +
                                           "one of ANTI_ALIASING_DEFAULT," +
                                           "ANTI_ALIASING_ON, or " +
                                           "ANTI_ALIASING_OFF");
    }
    if (this.antiAliasing == antiAliasing) {
        return;
    }
    this.antiAliasing = antiAliasing;
    setChanged(true, Changeable.REDRAW);
}

/**
 * Retrieve the image map information for this <code>JCChart</code>.
 * @return The current image map information.
 */
public ImageMapInfo getImageMapInfo()
{
    return(imageMapInfo);
}

/**
 * Set the image map information for this <code>JChart</code>.
 * @param imageMapInfo The new image map information.
 */
public void setImageMapInfo(ImageMapInfo imageMapInfo)
{
    this.imageMapInfo = imageMapInfo;
}

/**
 * Retrieve the image map information for the footer.
 * @return The current footer image map information.
 */
public ImageMapInfo getFooterImageMapInfo()
{
    return(footerImageMapInfo);
}

/**
 * Set the image map information for the footer.
 * @param footerImageMapInfo The new footer image map information.
 */
public void setFooterImageMapInfo(ImageMapInfo footerImageMapInfo)
{
    this.footerImageMapInfo = footerImageMapInfo;
}

/**
 * Retrieve the image map information for the header.
 * @return The current header image map information.
 */
public ImageMapInfo getHeaderImageMapInfo()
{
    return(headerImageMapInfo);
}

/**
 * Set the image map information for the header.
 * @param headerImageMapInfo The new header image map information.
 */
public void setHeaderImageMapInfo(ImageMapInfo headerImageMapInfo)
{
    this.headerImageMapInfo = headerImageMapInfo;
}

/**
 * Sets the locale for chart.
 * This will also set the locale for all children of the chart (chart area,
 * legend, header, footer, chart labels) and pertinent formatting classes,
 * but default names generated under the old locale (such as the names of
 * data views and axes) will not be changed.  Also, auto label chart labels
 * will not regenerate their text.
 *
 * @param locale the Locale object
 * @see java.util.Locale
 */
public void setLocale(Locale locale)
{
    if (locale == null) {
        locale = Locale.getDefault();
    }

    // gotta check for null here due to weird jdk 1.4.0 initialization condition
    if (this.locale != null && this.locale.equals(locale)) {
        return;
    }

    Locale oldValue = locale;
    try {
        this.locale = locale;
    }
    catch (MissingResourceException e) {
        this.locale = Locale.getDefault();
    }

    if (legend != null) {
        legend.setLocale(locale);
    }

    // set locale on chartArea and child axes
    if (chartArea != null) {
        chartArea.setLocale(locale);

        List xAxes = chartArea.getXAxes();
        if (xAxes != null && xAxes.size() > 0) {
            for (int i = 0; i < xAxes.size(); i++) {
                JCAxis xAxis = (JCAxis)xAxes.get(i);
                xAxis.setLocale(locale);
            }
        }

        List yAxes = chartArea.getYAxes();
        if (yAxes != null && yAxes.size() != 0) {
           for (int i = 0; i < yAxes.size(); i++ ) {
                JCAxis yAxis = (JCAxis)yAxes.get(i);
                yAxis.setLocale(locale);
            }
        }
    }

    if (data != null) {
        for (int i = 0; i < data.size(); i++) {
            ((ChartDataView)data.get(i)).setLocale(locale);
        }
    }

    // set locale on child chart labels
    JCChartLabelManager clm = getChartLabelManager();
    if (clm != null) {
        List labels = clm.getChartLabels();
        if (labels != null) {
            for (int i = 0; i < labels.size(); i++) {
                JComponent comp = ((JCChartLabel)labels.get(i)).component;
                if (comp != null) {
                    comp.setLocale(locale);
                }
            }
        }
    }

    // This is a bound property, so report the change to
    // any registered listeners.  (Cheap if there are none.)
    firePropertyChange("locale", oldValue, locale);

    // This could change the preferred size of the Component.
    if (isValid()) {
        invalidate();
    }
}

/**
 * Returns the chart locale.
 *
 * @return Locale The chart locale
 */
public Locale getLocale()
{
    return(this.locale);
}

/**
 * Overrides <code>readObject()</code> to first register UI class before
 * deserialiaing. Otherwise, the <code>UIManager</code> will throw a null
 * pointer exception when trying to find it during object reading.
 * @param in the input stream
 * @throws java.io.IOException
 */
private void readObject(java.io.ObjectInputStream in)
    throws java.io.IOException
{
    try {
        registerUI();
        in.defaultReadObject();
    }
    catch (Exception e) {
        e.printStackTrace(System.out);
    }
}


 


/**
 * Main method for running chart as an application.
 * @param args the command line arguments
 */
public static void main(String args[])
{
    JCExitFrame f = new JCExitFrame(
        JCChartBundle.string(JCChartBundle.key5, Locale.getDefault()));
    JCChart chart = new JCChart();
    chart.setAllowUserChanges(true);
    chart.setTrigger(0, new EventTrigger(InputEvent.META_MASK,
                                         EventTrigger.CUSTOMIZE));
    f.getContentPane().setLayout(new GridLayout(1,1));
    f.getContentPane().add(chart);
    f.pack();
    f.setSize(500, 300);
    f.show();
}

}
