/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCChartEnumMappings.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import com.klg.jclass.util.RenderProperties;

import java.awt.event.InputEvent;
import javax.swing.SwingConstants;
import java.awt.BasicStroke;

/**
 * Class consisting of static arrays that hold enum values and
 * their corresponding String representations (for things like HTML/XML save and
 * loading) and, in turn, their corresponding internationalizable String
 * representations (for things like combo box choices in the customizer
 * and the beans).  Note that some enum types have three arrays and some have
 * four arrays.  The extra array (with an <i>xml</i> in the name) was needed so
 * that the enum names in the xml document would be consistent (also xml enums
 * cannot start with a digit).
 *
 * All three (or four) arrays must match -- enum value at position <i>n</i> in
 * the enum value array must correspond to the String description at position
 * <i>n</i> in the other two arrays.
 */
public class JCChartEnumMappings
{

// Chart Type values/strings
public final static int[] chartType_values = {
    JCChart.PLOT, JCChart.SCATTER_PLOT, JCChart.AREA, JCChart.STACKING_AREA,
    JCChart.BAR, JCChart.STACKING_BAR, JCChart.PIE,
    JCChart.HILO, JCChart.HILO_OPEN_CLOSE, JCChart.CANDLE, JCChart.POLAR,
    JCChart.RADAR, JCChart.AREA_RADAR
};
public final static String[] chartType_strings = {
    "PLOT", "SCATTER_PLOT", "AREA", "STACKING_AREA",
    "BAR", "STACKING_BAR", "PIE" , "HILO",
    "HILO_OPEN_CLOSE", "CANDLE", "POLAR", "RADAR",
    "AREA_RADAR"
};
public final static String[] chartType_xml_strings = {
    "Plot", "Scatter_Plot", "Area", "Stacking_Area",
    "Bar", "Stacking_Bar", "Pie" , "Hi_Lo",
    "Hi_Lo_Open_Close", "Candle", "Polar", "Radar",
    "Area_Radar"
};
public final static String[] chartType_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key188),
    JCChartBundle.string(JCChartBundle.key189),
    JCChartBundle.string(JCChartBundle.key190),
    JCChartBundle.string(JCChartBundle.key191),
    JCChartBundle.string(JCChartBundle.key192),
    JCChartBundle.string(JCChartBundle.key193),
    JCChartBundle.string(JCChartBundle.key194),
    JCChartBundle.string(JCChartBundle.key195),
    JCChartBundle.string(JCChartBundle.key196),
    JCChartBundle.string(JCChartBundle.key197),
    JCChartBundle.string(JCChartBundle.key198),
    JCChartBundle.string(JCChartBundle.key199),
    JCChartBundle.string(JCChartBundle.key200)
};

// ChartText adjust values/strings
//DAM check -- may no longer be used?
public final static int[] adjust_values = {
    ChartText.LEFT, ChartText.CENTER, ChartText.RIGHT
};
public final static String[] adjust_strings = {
    "Left", "Center", "Right"
};
public final static String[] adjust_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key201),
    JCChartBundle.string(JCChartBundle.key202),
    JCChartBundle.string(JCChartBundle.key203)
};

// ChartText rotation values/strings
public final static int[] rotation_values = {
    ChartText.DEG_0, ChartText.DEG_90,
    ChartText.DEG_180, ChartText.DEG_270
};
public final static String[] rotation_strings = {
    "0", "90", "180", "270"
};
public final static String[] rotation_xml_strings = {
    "None", "Rotate_90", "Rotate_180", "Rotate_270"
};
public final static String[] rotation_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key204),
    JCChartBundle.string(JCChartBundle.key205),
    JCChartBundle.string(JCChartBundle.key206),
    JCChartBundle.string(JCChartBundle.key207)
};

// EventTrigger values/strings
public final static int[] trigger_values = { EventTrigger.NONE,
        EventTrigger.ROTATE, EventTrigger.ZOOM,
        EventTrigger.TRANSLATE, EventTrigger.EDIT,
        EventTrigger.PICK, EventTrigger.DEPTH,
        EventTrigger.CUSTOMIZE,
};
public final static String[] trigger_strings = {
    "None", "Rotate", "Zoom", "Translate",
    "Edit", "Pick", "Depth", "Customize"
};
public final static String[] trigger_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key180),
    JCChartBundle.string(JCChartBundle.key181),
    JCChartBundle.string(JCChartBundle.key182),
    JCChartBundle.string(JCChartBundle.key183),
    JCChartBundle.string(JCChartBundle.key184),
    JCChartBundle.string(JCChartBundle.key185),
    JCChartBundle.string(JCChartBundle.key186),
    JCChartBundle.string(JCChartBundle.key187)
};

// JCAxis AxisPlacement values/strings
public final static int[] placement_values = {
    JCAxis.AUTOMATIC, JCAxis.ORIGIN, JCAxis.MIN, JCAxis.MAX,
    JCAxis.VALUE_ANCHORED
};
public final static String[] placement_strings = {
    "Automatic", "Origin", "Min", "Max", "Value_Anchored"
};
public final static String[] placement_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key208),
    JCChartBundle.string(JCChartBundle.key209),
    JCChartBundle.string(JCChartBundle.key210),
    JCChartBundle.string(JCChartBundle.key211),
    JCChartBundle.string(JCChartBundle.key212)
};

// JCAxis AnnotationRotation values/strings
public final static int[] annRotation_values = {
    JCAxis.ROTATE_NONE, JCAxis.ROTATE_90,
    JCAxis.ROTATE_180, JCAxis.ROTATE_270,
    JCAxis.ROTATE_OTHER
};
public final static String[] annRotation_strings = {
    "None", "90", "180", "270", "Other"
};
public final static String[] annRotation_xml_strings = {
    "None", "Rotate_90", "Rotate_180", "Rotate_270", "Rotate_Other"
};
public final static String[] annRotation_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key180),
    JCChartBundle.string(JCChartBundle.key205),
    JCChartBundle.string(JCChartBundle.key206),
    JCChartBundle.string(JCChartBundle.key207),
    JCChartBundle.string(JCChartBundle.key3)
};

// JCAxis AnnotationMethod values/strings
public final static int[] annotationMethod_values = {
    JCAxis.VALUE, JCAxis.VALUE_LABELS,
    JCAxis.TIME_LABELS, JCAxis.POINT_LABELS
};
public final static String[] annotationMethod_strings = {
    "Value", "Value_Labels", "Time_Labels", "Point_Labels"
};
public final static String[] annotationMethod_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key213),
    JCChartBundle.string(JCChartBundle.key214),
    JCChartBundle.string(JCChartBundle.key215),
    JCChartBundle.string(JCChartBundle.key216)
};


// JCAxis AnnotationMethod values/strings
public final static int[] yAxisAnnotationMethod_values = {
    JCAxis.VALUE, JCAxis.VALUE_LABELS,
    JCAxis.TIME_LABELS
};
public final static String[] yAxisAnnotationMethod_strings = {
    "Value", "Value_Labels", "Time_Labels"
};
public final static String[] yAxisAnnotationMethod_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key213),
    JCChartBundle.string(JCChartBundle.key214),
    JCChartBundle.string(JCChartBundle.key215)
};


// JCAxis AnnotationMethod values/strings
public final static int[] yAxisAnnotationPolar_values = {
    JCAxis.VALUE, JCAxis.VALUE_LABELS
};
public final static String[] yAxisAnnotationPolar_strings = {
    "Value", "Value_Labels"
};
public final static String[] yAxisAnnotationPolar_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key213),
    JCChartBundle.string(JCChartBundle.key214)
};

// JCAxis AnnotationMethod values/strings
public final static int[] annotationPolar_values = {
    JCAxis.VALUE, JCAxis.VALUE_LABELS, JCAxis.POINT_LABELS
};
public final static String[] annotationPolar_strings = {
    "Value", "Value_Labels", "Point_Labels"
};
public final static String[] annotationPolar_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key213),
    JCChartBundle.string(JCChartBundle.key214),
    JCChartBundle.string(JCChartBundle.key216)
};

// JCAxis TimeUnit values/strings
    // careful - in JCAxis these are longs, but they fit in ints...
public final static long[] timeUnit_values = {
    JCAxis.SECONDS, JCAxis.MINUTES, JCAxis.HOURS,
    JCAxis.DAYS, JCAxis.WEEKS, JCAxis.MONTHS, JCAxis.YEARS
};
public final static String[] timeUnit_strings = {
    "Seconds", "Minutes", "Hours",
    "Days", "Weeks","Months", "Years"
};
public final static String[] timeUnit_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key217),
    JCChartBundle.string(JCChartBundle.key218),
    JCChartBundle.string(JCChartBundle.key219),
    JCChartBundle.string(JCChartBundle.key220),
    JCChartBundle.string(JCChartBundle.key221),
    JCChartBundle.string(JCChartBundle.key222),
    JCChartBundle.string(JCChartBundle.key223)
};

// JCAxis OriginPlacement values/strings
public final static int[] originPlacement_values = {
    JCAxis.AUTOMATIC, JCAxis.ZERO, JCAxis.MIN, JCAxis.MAX
};
public final static String[] originPlacement_strings = {
    "Automatic", "Zero", "Min", "Max"
};
public final static String[] originPlacement_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key208),
    JCChartBundle.string(JCChartBundle.key224),
    JCChartBundle.string(JCChartBundle.key210),
    JCChartBundle.string(JCChartBundle.key211)
};

// JCChart EventTrigger modifier values/strings
public final static int[] eventMod_values = {
    0,
    InputEvent.CTRL_MASK,
    InputEvent.SHIFT_MASK,
    InputEvent.ALT_MASK,
    InputEvent.META_MASK
};
public final static String[] eventMod_strings = {
    "None", "Ctrl", "Shift", "Alt", "Meta"
};
public final static String[] eventMod_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key180),
    JCChartBundle.string(JCChartBundle.key233),
    JCChartBundle.string(JCChartBundle.key234),
    JCChartBundle.string(JCChartBundle.key235),
    JCChartBundle.string(JCChartBundle.key236)
};

// JCChart header and footer Alignment values/strings for those that
// are JLabels
public final static int[] halign_values = {
    SwingConstants.LEFT, SwingConstants.CENTER, SwingConstants.RIGHT,
    SwingConstants.LEADING, SwingConstants.TRAILING
};
public final static String[] halign_strings = {
    "Left", "Center", "Right", "Leading", "Trailing"
};
public final static String[] halign_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key201),
    JCChartBundle.string(JCChartBundle.key202),
    JCChartBundle.string(JCChartBundle.key203),
    JCChartBundle.string(JCChartBundle.key237),
    JCChartBundle.string(JCChartBundle.key238)
};
public final static int[] valign_values = {
    SwingConstants.TOP, SwingConstants.CENTER, SwingConstants.BOTTOM
};
public final static String[] valign_strings = {
    "Top", "Center", "Bottom"
};
public final static String[] valign_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key239),
    JCChartBundle.string(JCChartBundle.key202),
    JCChartBundle.string(JCChartBundle.key240)
};


// JCChartArea AngleUnit values/strings
public final static int[] angleUnit_values = {
    JCChartUtil.DEGREES, JCChartUtil.RADIANS, JCChartUtil.GRADS
};
public final static String[] angleUnit_strings = {
    "DEGREES", "RADIANS", "GRADS"
};
public final static String[] angleUnit_xml_strings = {
    "Degrees", "Radians", "Grads"
};
public final static String[] angleUnit_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key241),
    JCChartBundle.string(JCChartBundle.key242),
    JCChartBundle.string(JCChartBundle.key243),
};

// JCChartLabel AttachMethod values/strings
public final static int attachMethod_values[] = {
    JCChartLabel.ATTACH_NONE, JCChartLabel.ATTACH_COORD,
    JCChartLabel.ATTACH_DATACOORD, JCChartLabel.ATTACH_DATAINDEX
};
public final static String[] attachMethod_strings = {
    "Attach_None", "Attach_Coord", "Attach_Datacoord", "Attach_Dataindex"
};
public final static String[] attachMethod_xml_strings = {
    "None", "Coord", "Data_Coord", "Data_Index"
};
public final static String[] attachMethod_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key244),
    JCChartBundle.string(JCChartBundle.key245),
    JCChartBundle.string(JCChartBundle.key246),
    JCChartBundle.string(JCChartBundle.key247)
};

// JCChartLabel Anchor values/strings
public final static int chartLabelAnchor_values[] = {
    JCChartLabel.AUTO, JCChartLabel.CENTER,
    JCChartLabel.NORTHEAST, JCChartLabel.SOUTHEAST,
    JCChartLabel.NORTHWEST, JCChartLabel.SOUTHWEST,
    JCChartLabel.NORTH, JCChartLabel.SOUTH,
    JCChartLabel.EAST, JCChartLabel.WEST
};
public final static String[] chartLabelAnchor_strings = {
    "Auto" , "Center",
    "Northeast", "Southeast", "Northwest", "Southwest",
    "North", "South", "East", "West"
};
public final static String[] chartLabelAnchor_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key248),
    JCChartBundle.string(JCChartBundle.key202),
    JCChartBundle.string(JCChartBundle.key225),
    JCChartBundle.string(JCChartBundle.key230),
    JCChartBundle.string(JCChartBundle.key227),
    JCChartBundle.string(JCChartBundle.key232),
    JCChartBundle.string(JCChartBundle.key226),
    JCChartBundle.string(JCChartBundle.key231),
    JCChartBundle.string(JCChartBundle.key228),
    JCChartBundle.string(JCChartBundle.key229)
};

// JCFillStyle FillPattern values/strings
public final static int[] fillPattern_values = {
    JCFillStyle.NONE, JCFillStyle.SOLID,
    JCFillStyle.PER_25, JCFillStyle.PER_50, JCFillStyle.PER_75,
    JCFillStyle.HORIZ_STRIPE, JCFillStyle.VERT_STRIPE,
    JCFillStyle.STRIPE_45, JCFillStyle.STRIPE_135,
    JCFillStyle.DIAG_HATCHED, JCFillStyle.CROSS_HATCHED,
    JCFillStyle.CUSTOM_FILL, JCFillStyle.CUSTOM_STACK
};
public final static String[] fillPattern_strings = {
    "None", "Solid", "Per_25", "Per_50", "Per_75",
    "Horiz_Stripe", "Vert_Stripe", "Stripe_45", "Stripe_135",
    "Diag_Hatched", "Cross_Hatched", "Custom_Fill", "Custom_Stack"
};
public final static String[] fillPattern_xml_strings = {
    "None", "Solid", "Per_25", "Per_50", "Per_75",
    "Horiz_Stripe", "Vert_Stripe", "Stripe_45", "Stripe_135",
    "Diag_Hatched", "Cross_Hatched", "Custom_Fill", "Custom_Stacked"
};
public final static String[] fillPattern_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key180),
    JCChartBundle.string(JCChartBundle.key249),
    JCChartBundle.string(JCChartBundle.key250),
    JCChartBundle.string(JCChartBundle.key251),
    JCChartBundle.string(JCChartBundle.key252),
    JCChartBundle.string(JCChartBundle.key253),
    JCChartBundle.string(JCChartBundle.key254),
    JCChartBundle.string(JCChartBundle.key255),
    JCChartBundle.string(JCChartBundle.key256),
    JCChartBundle.string(JCChartBundle.key257),
    JCChartBundle.string(JCChartBundle.key258),
    JCChartBundle.string(JCChartBundle.key259),
    JCChartBundle.string(JCChartBundle.key260)
};

// JCLineStyle LinePattern values/strings
public final static int[] linePattern_values = {
    JCLineStyle.NONE, JCLineStyle.SOLID,
    JCLineStyle.LONG_DASH, JCLineStyle.SHORT_DASH,
    JCLineStyle.LSL_DASH, JCLineStyle.DASH_DOT
};
public final static String[] linePattern_strings = {
    "None", "Solid",
    "Long_Dash", "Short_Dash",
    "LSL_Dash", "Dash_Dot"
};
public final static String[] linePattern_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key180),
    JCChartBundle.string(JCChartBundle.key249),
    JCChartBundle.string(JCChartBundle.key261),
    JCChartBundle.string(JCChartBundle.key262),
    JCChartBundle.string(JCChartBundle.key263),
    JCChartBundle.string(JCChartBundle.key264)
};

// JCLineStyle Line Join values/strings
public final static int[] lineJoin_values = {
    BasicStroke.JOIN_MITER, BasicStroke.JOIN_BEVEL, BasicStroke.JOIN_ROUND
};
public final static String[] lineJoin_strings = {
    "Miter", "Bevel", "Round"
};
public final static String[] lineJoin_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key265),
    JCChartBundle.string(JCChartBundle.key266),
    JCChartBundle.string(JCChartBundle.key267)
};

// JCLineStyle Line Cap values/strings
public final static int[] lineCap_values = {
    BasicStroke.CAP_BUTT, BasicStroke.CAP_ROUND, BasicStroke.CAP_SQUARE
};
public final static String[] lineCap_strings = {
    "Butt", "Round", "Square"
};
public final static String[] lineCap_xml_strings = {
    "Butt", "Round", "Squared"
};
public final static String[] lineCap_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key268),
    JCChartBundle.string(JCChartBundle.key267),
    JCChartBundle.string(JCChartBundle.key269)
};

// JCPieChartFormat ThresholdMethod values/strings
public final static int[] thresholdMethod_values = {
    JCPieChartFormat.SLICE_CUTOFF, JCPieChartFormat.PIE_PERCENTILE
};
public final static String[] thresholdMethod_strings = {
    "Slice_Cutoff", "Percentile"
};
public final static String[] thresholdMethod_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key270),
    JCChartBundle.string(JCChartBundle.key271)
};

// JCPieChartFormat SortOrder values/strings
public final static int[] sortOrder_values = {
    JCPieChartFormat.DATA_ORDER, JCPieChartFormat.ASCENDING_ORDER,
    JCPieChartFormat.DESCENDING_ORDER
};
public final static String[] sortOrder_strings = {
    "Data_Order", "Ascending", "Descending"
};
public final static String[] sortOrder_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key272),
    JCChartBundle.string(JCChartBundle.key273),
    JCChartBundle.string(JCChartBundle.key274)
};

// JCSymbolStyle Shape values/strings
public final static int[] shape_values = {
    JCSymbolStyle.NONE, JCSymbolStyle.DOT, JCSymbolStyle.BOX,
    JCSymbolStyle.TRIANGLE, JCSymbolStyle.DIAMOND, JCSymbolStyle.STAR,
    JCSymbolStyle.VERT_LINE, JCSymbolStyle.HORIZ_LINE, JCSymbolStyle.CROSS,
    JCSymbolStyle.CIRCLE, JCSymbolStyle.SQUARE
};
public final static String[] shape_strings = {
    "None", "Dot", "Box",
    "Triangle", "Diamond", "Star",
    "Vert_Line", "Horiz_Line", "Cross",
    "Circle", "Square"
};
public final static String[] shape_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key180),
    JCChartBundle.string(JCChartBundle.key275),
    JCChartBundle.string(JCChartBundle.key276),
    JCChartBundle.string(JCChartBundle.key277),
    JCChartBundle.string(JCChartBundle.key278),
    JCChartBundle.string(JCChartBundle.key279),
    JCChartBundle.string(JCChartBundle.key280),
    JCChartBundle.string(JCChartBundle.key281),
    JCChartBundle.string(JCChartBundle.key282),
    JCChartBundle.string(JCChartBundle.key283),
    JCChartBundle.string(JCChartBundle.key284)
};

// Anti aliasing values/strings
public final static int[] anti_aliasing_values = {
    RenderProperties.ANTI_ALIASING_DEFAULT,
    RenderProperties.ANTI_ALIASING_ON,
    RenderProperties.ANTI_ALIASING_OFF
};
public final static String[] anti_aliasing_strings = {
    "Default", "On", "Off"
};
public final static String[] anti_aliasing_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key285),
    JCChartBundle.string(JCChartBundle.key286),
    JCChartBundle.string(JCChartBundle.key287)
};

// JCAnno values/strings
public final static int[] anno_type_values = {
    JCAnno.DEFAULT_LABELS,
    JCAnno.DEFAULT_TICKS,
    JCAnno.USER_DEFINED
};
public final static String[] anno_type_strings = {
    "Default_Labels", "Default_Ticks", "User_Defined"
};
public final static String[] anno_type_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key288),
    JCChartBundle.string(JCChartBundle.key289),
    JCChartBundle.string(JCChartBundle.key290)
};

// Data Format values/strings
public final static int[] data_format_values = {
    ChartDataModel.ALL,
    JCPieChartFormat.OTHER_SLICE
};
public final static String[] data_format_strings = {
    "All", "Other_Slice"
};
public final static String[] data_format_i18n_strings = {
    JCChartBundle.string(JCChartBundle.key291),
    JCChartBundle.string(JCChartBundle.key292)
};

}
