/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCChartEvent.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.util.EventObject;

/**
 * Class used to encapsulate an action (for example, zoom or scale) 
 * on a chart. It holds the chart that was acted upon and, if the 
 * chart axis was modified, holds that axis -- zoom,
 * scale, and tranlate actions modify axes; rotate, depth change, and
 * edit actions do not.
 */

public class JCChartEvent extends EventObject {

protected JCAxis modifiedAxis;

/**
 * Constructor to create a <code>JCChartEvent</code> object given the chart 
 * and the modified axis.
 * @param chart the chart that was acted on; it is stored in the
 * <code>Source</code> property of the superclass
 * @param axis the axis that was modified; if no axis was modified, use
 * <code>null</code>
 */
JCChartEvent(JCChart chart, JCAxis axis)
{
    super(chart);
    modifiedAxis = axis;
}

/**
 * Gets the <code>ModifiedAxis</code> property. It holds the axis that was
 * modified by an action on the chart. If no axis was modified, the value is
 * <code>null</code>.
 * @return the <code>ModifiedAxis</code> property, a <code>JCAxis</code>
 * instance
 */
public JCAxis getModifiedAxis()
{
    return(modifiedAxis);
}

/**
 * Returns this object as a String representation.
 * @return object as a String representation
 */
public String toString()
{
    return(super.toString() + " " +
           JCChartBundle.string(JCChartBundle.key6,
                                ((JCChart)getSource()).getLocale()) +
           " : " + modifiedAxis);
}

}
