/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCChartLabel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;


 




import com.klg.jclass.util.swing.TextRenderer;

import javax.swing.*;
import java.awt.*;

/**
 * <code>JCChartLabel</code> manages and renders a floating label attached
 * somewhere on the chart.
 */
public class JCChartLabel extends TrackChange implements java.io.Serializable
{

public final static int NORTHEAST = 1;
public final static int NORTH     = 2;
public final static int NORTHWEST = 3;
public final static int EAST      = 4;
public final static int WEST      = 5;
public final static int SOUTHEAST = 6;
public final static int SOUTH     = 7;
public final static int SOUTHWEST = 8;
public final static int CENTER    = 9;
public final static int AUTO      = 10;

public final static int ATTACH_NONE      = 0;
public final static int ATTACH_COORD     = 1;
public final static int ATTACH_DATACOORD = 2;
public final static int ATTACH_DATAINDEX = 3;

/**
 * Instance variables.
 */
protected int                  anchor = CENTER;
protected int                  attachMethod = ATTACH_COORD;
protected int                  attachX = 0;
protected int                  attachY = 0;
protected double               dataAttachX = 0.0;
protected double               dataAttachY = 0.0;
protected int                  offsetX = 0;
protected int                  offsetY = 0;
protected JCDataIndex          attachDataIndex = null;
protected ChartDataView        dataView = null;
protected boolean              dwellLabel = false;
protected int                  dwellDelay = 0;
protected boolean              connected = false;

protected JComponent           component = new JLabel();
protected JCChartLabelLine     line = null;
protected String               text = null;
protected String               defaultText = null;
protected JCChartLabelManager  parent = null;

// Save the point to attach data at before calculating the offsets due
// to anchor values.
private Point                  attachBeforeAnchor = null;
// Save the current attachement point
protected Point                attachPoint = null;

/**
 * Constructor for <code>JCChartLabel</code>.
 * @param contents the text to be displayed in the label
 */
public JCChartLabel(String contents)
{
    super();
    setText(contents);
    component.setVisible(true);
}

/**
 * Default constructor for <code>JCChartLabel</code>, required by JavaBeans.
 */
public JCChartLabel()
{
    super();
    setText(null);
    component.setVisible(true);
}

/**
 * Sets the component portion of this chart label.  The component portion
 * is the object derived from <code>JComponent</code> that actually draws on
 * the chart.
 * @param comp the component portion of the chart label
 */
public void setComponent(JComponent comp)
{
    if (component == comp) {
        return;
    }

    JCChart chart = getParentChart();
    if (chart != null && !dwellLabel && component != null) {
        if (isConnected() && line != null) {
            chart.remove(line);
        }
        chart.remove(component);
    }

    component = comp;

    if (chart != null && !dwellLabel && component != null) {
        if (isConnected() && line != null) {
            chart.add(line, 0);
        }
        chart.add(component, 0);
    }

	// If custom component, just set text to empty string, since
	// it doesnt apply
    if (!(component instanceof JLabel)) {
        text = "";
    }
	// If setting a custom jlabel, make sure text gets noted so that
	// default text is turned off
    else {
		text = ((JLabel)component).getText();
		defaultText = null;
    }

    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the component portion of this chart label.  The component portion
 * is the object derived from <code>JComponent</code> that actually draws on the
 * chart.
 * @return the component portion of the chart label
 */
public JComponent getComponent()
{
    return(component);
}

/**
 * Convenience method for setting text.  If the component property is null,
 * a <code>JLabel</code> will be created with the specified text.  If a
 * component already exists and it is a <code>JLabel</code>, that instance will
 * be set with the specified text.  Otherwise, nothing will happen.
 * @param s The text to set. If the text is null, text will automatically
 * be generated.
 */
public void setText(String s)
{
    text = s;
    String textToSet = s;
    if (textToSet == null) {
        textToSet = "";
    }

    if (component == null) {
        component = new JLabel(textToSet);
    }
    else if (component instanceof JLabel) {



        	((JLabel)component).setText(textToSet);



    }
	defaultText = null;
    setChanged(true, Changeable.RECALC);
}

/**
 * Internal use only. Returns the value of the internal text variable
 * that is set with the setText() method.  The return value may be null
 * if default text is being used.
 */
public String getSetText()
{
	return(text);
}

/**
 * Internal use only. Returns the value of the internal default text variable
 * that contains the last calculated instance of default text.
 */
public String getDefaultText()
{
	return(defaultText);
}

/**
 * Returns the parent chart to this chart label.
 * @return parent chart
 */
public JCChart getParentChart()
{
    JCChartLabelManager man = getParentManager();
    if (man != null) {
        return(man.getParentChart());
    }
    return(null);
}

/**
 * Returns the manager chart to this chart label.
 * @return manager chart
 */
public JCChartLabelManager getParentManager()
{
    return(parent);
}

/**
 * Sets the parent manager of this chart label.
 * @param manager parent manager
 */
public void setParentManager(JCChartLabelManager manager)
{
    parent = manager;
}

/**
 * Determines the correct location of the label and moves there.
 */
public void recalc()
{
    if (component == null) {
        return;
    }

    if (text == null) {
        getDefaultLabel();
    }

    setChanged(true, Changeable.RECALC, false);

    Dimension dim = null;

    dim = component.getPreferredSize();


	// DAM I probably should determine whether the user has not set
	// his own size, that is, whether the default size should be
	// used or not.  But I cannot figure out how to determine this.


 

    component.setSize(dim.width, dim.height);
    component.validate();

	// make sure we're using the right data view
    ChartDataView dv = getEffectiveDataView();

	// can't determine data view, so bail out of recalc
    if (dv == null) {
        return;
    }

    Point attach = null;
    int w = component.getWidth() / 2, h = component.getHeight() / 2;

    boolean checkData = true;
    int chartType = dv.getChartType();

    // for use in clipping
    double data_x = 0.0;
    double data_y = 0.0;

    try {
        switch (attachMethod) {
        case ATTACH_COORD:
            attach = new Point(attachX, attachY);
            break;
        case ATTACH_DATACOORD:
            if (chartType != JCChart.PIE &&
                chartType != JCChart.RADAR &&
                chartType != JCChart.AREA_RADAR)
            {
                attach = dv.dataCoordToCoord(dataAttachX, dataAttachY);
                data_x = dataAttachX;
                data_y = dataAttachY;
            }
            break;
        case ATTACH_DATAINDEX:
            attach = dv.dataIndexToCoord(attachDataIndex);
            int seriesIndex = attachDataIndex.getSeriesIndex();
            if (seriesIndex != JCPieChartFormat.OTHER_SLICE) {
                ChartDataViewSeries series = attachDataIndex.getSeries();
                data_x = series.getX(attachDataIndex.point);
                data_y = series.getY(attachDataIndex.point);
            }
            else {
                checkData = false;
            }
            break;
        case ATTACH_NONE:
        default:
            // System.out.println("Default?");
            return;
        }

        // clip attach points to axis min/max depending
        // on attach method
        JCAxis xaxis = dv.getXAxis();
        JCAxis yaxis = dv.getYAxis();
        switch (attachMethod) {
        case ATTACH_COORD:
            break;
        case ATTACH_DATACOORD:
            if (chartType == JCChart.RADAR ||
                chartType == JCChart.AREA_RADAR)
            {
                attach = null;
            }
            else if (chartType == JCChart.POLAR) {
                // Don't check for data_y < yaxis.getMin().
                // Points with y values less than the axis
                // minimum are displayed at theta + 2PI.
                if (data_y > yaxis.getMax()) {
                    attach = null;
                }
            }
            else if (data_x < xaxis.getMin() ||
                     data_x > xaxis.getMax() ||
                     data_y < yaxis.getMin() ||
                     data_y > yaxis.getMax())
            {
                attach = null;
            }
            break;
        case ATTACH_DATAINDEX:
            //clip attach point to axes min/max
            if (checkData) {
                if (chartType == JCChart.POLAR) {
                    // Don't check for data_y < yaxis.getMin().
                    if (data_y > yaxis.getMax()) {
                        attach = null;
                    }
                }
                else if (chartType == JCChart.RADAR ||
                    chartType == JCChart.AREA_RADAR) {
                    if (data_y < yaxis.getMin() || data_y > yaxis.getMax()) {
                        attach = null;
                    }
                }
                else if (data_x < xaxis.getMin() ||
                         data_x > xaxis.getMax() ||
                         data_y < yaxis.getMin() ||
                         data_y > yaxis.getMax())
                {
                    attach = null;
                }
            }
            break;
        }
    }
    catch (Exception e) {
        attach = null;
    }

    if (attach == null) {
        // Couldn't place this label: move it out of the way
        // maybe we'll have better luck next time
        attachBeforeAnchor = null;
        attach = new Point(-9999, -9999);
        // System.out.println("null attachment point");
    }
    else {
        attachBeforeAnchor = new Point(attach.x, attach.y);
        attach.x -= w;
        attach.y -= h;

        // adjust x
        switch (anchor) {
        case EAST:
        case SOUTHEAST:
        case NORTHEAST:
            attach.x += w;
            break;
        case WEST:
        case SOUTHWEST:
        case NORTHWEST:
            attach.x -= w;
            break;
        }

        // adjust y
        switch (anchor) {
        case NORTH:
        case NORTHWEST:
        case NORTHEAST:
            attach.y -= h;
            break;
        case SOUTH:
        case SOUTHWEST:
        case SOUTHEAST:
            attach.y += h;
            break;
        }

        attach.x += offsetX;
        attach.y += offsetY;

        setChanged(false, Changeable.NO_CHANGE);
    }
    component.setLocation(attach.x, attach.y);
    attachPoint = new Point(attach.x, attach.y);

    if (line != null && isConnected() && !dwellLabel) {
        calcLineBounds(line);
    }
}

/**
 * Gets the point on the chart that this label is attached to.
 * @return the point the label is attached to
 */
protected Point getAttachPoint()
{
    return(attachPoint);
}

/**
 * For internal use only.
 * Draws the connector between the attach point and this chart label.
 * Used only with dwell labels, since for dwell labels the component
 * and it's line are not actually added to the parent chart.
 * @param g
 */
public void drawConnector(Graphics g)
{
    if (component == null) {
        return;
    }

    Point attachLine = new Point(0, 0);
    if (calcConnectingLine(attachLine)) {
        g.drawLine(attachBeforeAnchor.x, attachBeforeAnchor.y,
                   attachLine.x, attachLine.y);
    }
}

/**
 * Calculates the connector between the attach point and this chart label.
 * Used only with non-dwell labels, since for them the component and the
 * line are actually added to the parent chart.
 * @param thisLine
 */
protected void calcLineBounds(JCChartLabelLine thisLine)
{
    if (component == null || line == null) {
        thisLine.setBounds(new Rectangle(0, 0, 0, 0));
        return;
    }

    Point attachLine = new Point(0, 0);
    if (calcConnectingLine(attachLine)) {
        Rectangle bounds = new Rectangle();
        bounds.x = Math.min(attachBeforeAnchor.x, attachLine.x);
        bounds.y = Math.min(attachBeforeAnchor.y, attachLine.y);
        bounds.width  = Math.abs(attachBeforeAnchor.x - attachLine.x) + 1;
        bounds.height = Math.abs(attachBeforeAnchor.y - attachLine.y) + 1;
        thisLine.setBounds(bounds);
        thisLine.setStartPoint(new Point(attachBeforeAnchor.x - bounds.x,
                                         attachBeforeAnchor.y - bounds.y));
        thisLine.setEndPoint(new Point(attachLine.x - bounds.x,
                                       attachLine.y - bounds.y));
        return;
    }

    thisLine.setBounds(new Rectangle(0, 0, 0, 0));
}

/**
 * Calculates the connecting line.  Returns <code>true</code> if line is to be
 * drawn and the endpoint will be returned in the provided <code>Point</code>
 * object. Returns <code>false</code> and doesn't modify the provided point
 * object if line is not to be drawn.
 * @return <code>true</code> if the line is to be drawn and the endpoint will
 * be returned in the provided <code>Point</code> object; <code>false</code>
 * otherwise.
 * @param linePoint
 */
protected boolean calcConnectingLine(Point linePoint)
{
    if (attachBeforeAnchor != null) {
        // If a data attached label, check that the data point to attach the
        // label at is actually currently visible.  If not, don't display the
        // label.  All data is always visible in a pie chart.
    	ChartDataView dv = getEffectiveDataView();

        if ((dv != null) && (attachMethod == ATTACH_DATACOORD ||
            attachMethod == ATTACH_DATAINDEX))
        {
            Point l = getParentChart().getChartArea().getLocation();
            Rectangle pr = getParentChart().getChartArea().getPlotRect();
            Insets ins = getParentChart().getChartArea().getInsets();
            if (attachBeforeAnchor.x < pr.x + l.x ||
                attachBeforeAnchor.x > pr.x + l.x +
                pr.width + ins.left + ins.right ||
                attachBeforeAnchor.y < pr.y + l.y ||
                attachBeforeAnchor.y > pr.y + l.y +
                pr.height + ins.top + ins.bottom)
            {
                return(false);
            }
        }
        if (connected && attachPoint != null) {
            int fullwidth = component.getWidth();
            int fullheight = component.getHeight();
            // Quick and dirty way to draw lines.
            // Could be replaced by more comprehensive method to
            // attach to anchored side or find away to draw lines
            // before labels (like it used to be done)
            Point attachLine = null;
            // label fully above point
            if (attachPoint.y + fullheight < attachBeforeAnchor.y) {
                attachLine = new Point(attachPoint.x + fullwidth / 2,
                                       attachPoint.y + fullheight);
            }
            // label fully below point
            else if (attachPoint.y > attachBeforeAnchor.y) {
                attachLine = new Point(attachPoint.x + fullwidth / 2,
                                       attachPoint.y);
            }
            // label fully right of point
            else if (attachPoint.x > attachBeforeAnchor.x) {
                attachLine = new Point(attachPoint.x,
                                       attachPoint.y + fullheight / 2);
            }
            // label fully left of point
            else if (attachPoint.x + fullwidth < attachBeforeAnchor.x) {
                attachLine = new Point(attachPoint.x + fullwidth,
                                       attachPoint.y + fullheight / 2);
            }
            if (attachLine != null) {
                linePoint.x = attachLine.x;
                linePoint.y = attachLine.y;
                return(true);
            }
            else {
                return(false);
            }
        }
    }
    return(false);
}

/**
 * Sets the <code>Anchor</code> property.
 * This property determines the position of the
 * label relative to the its attachment location.
 * @param a one of <code>NORTH</code>, <code>SOUTH</code>,
 * <code>EAST</code>, <code>WEST</code>, <code>NORTHWEST</code>,
 * <code>SOUTHWEST</code>, <code>NORTHEAST</code>, or <code>SOUTHEAST</code>
 * @throws IllegalArgumentException If the anchor value is not one
 * of <code>NORTH</code>, <code>SOUTH</code>, <code>EAST</code>,
 * <code>WEST</code>, <code>NORTHEAST</code>, <code>NORTHWEST</code>,
 * <code>SOUTHEAST</code>, <code>SOUTHWEST</code>, <code>CENTER</code> or
 * <code>AUTO</code>.
 */
public void setAnchor(int a)
{
    if (a == anchor) {
        return;
    }
    switch (a) {
    case NORTH:
    case SOUTH:
    case EAST:
    case WEST:
    case NORTHEAST:
    case SOUTHEAST:
    case NORTHWEST:
    case SOUTHWEST:
    case CENTER:
    case AUTO:
        anchor = a;
        break;
    default:
        throw new IllegalArgumentException(
            "Invalid Anchor value in JCChartLabel.  Should be one of " +
            "NORTH, SOUTH, EAST, WEST, NORTHEAST, SOUTHEAST, NORTHWEST, " +
            "SOUTHWEST, CENTER or AUTO."
        );
    }

    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the <code>Anchor</code> property.
 * This property determines the position of the
 * label relative to the its attachment location.
 * @return one of <code>NORTH</code>, <code>SOUTH</code>, <code>EAST</code>,
 * <code>WEST</code>, <code>NORTHWEST</code>, <code>SOUTHWEST</code>,
 * <code>NORTHEAST</code>, <code>SOUTHEAST</code>, <code>CENTER</code> or
 * <code>AUTO</code>.
 */
public int getAnchor()
{
    return(anchor);
}

/**
 * Sets the Attachment Method of the label.
 * @param newMethod one of <code>ATTACH_NONE</code>, <code>ATTACH_COORD</code>,
 * <code>ATTACH_DATACOORD</code>, or <code>ATTACH_DATAINDEX</code>.
 * <br><code>ATTACH_NONE</code> means the label is not shown.
 * <br><code>ATTACH_COORD</code> means the label is attached is screen (pixel)
 * coordinates relative to the chart.
 * <br><code>ATTACH_DATACOORD</code> means the label is attached in axis
 * coordinates relative to the associated data view.
 * <br><code>ATTACH_DATAINDEX</code> means the label is attached to a data item.
 * @throws IllegalArgumentException If the anchor value is not one
 * of <code>ATTACH_NONE</code>, <code>ATTACH_COORD</code>,
 * <code>ATTACH_DATACOORD</code>, <code>ATTACH_DATAINDEX</code>.
 */
public void setAttachMethod(int newMethod)
{
    if (newMethod == attachMethod) {
        return;
    }

    switch (newMethod) {
    case ATTACH_NONE:
    case ATTACH_COORD:
    case ATTACH_DATACOORD:
    case ATTACH_DATAINDEX:
        attachMethod = newMethod;
        setChanged(true, Changeable.RECALC);
        break;
    default:
        throw new IllegalArgumentException(
            "Invalid AttachMethod in JCChartLabel. Should be one of " +
            "ATTACH_NONE, ATTACH_COORD, ATTACH_DATACOORD, ATTACH_DATAINDEX."
        );
    }
}

/**
 * Gets the Attachment Method of the label.
 * @return one of <code>ATTACH_NONE</code>, <code>ATTACH_COORD</code>,
 * <code>ATTACH_DATACOORD</code>, or <code>ATTACH_DATAINDEX</code>
 */
public int getAttachMethod()
{
    return(attachMethod);
}

/**
 * Sets the coordinates that the label is attached to.
 * This is used when the label's <code>attachMethod</code> is
 * <code>ATTACH_COORD</code>.
 * @param p the point in chart pixel coordinates where the label is attached
 */
public void setCoord(Point p)
{
    attachX = p.x;
    attachY = p.y;
    setChanged(true, Changeable.RECALC);
    recalc();
}

/**
 * Gets the coordinates that the label is attached to.
 * @return p the point in chart pixel coordinates where the label is attached
 */
public Point getCoord()
{
    return(new Point(attachX, attachY));
}

/**
 * Sets the current data view for this label.  The property specifies
 * the data view that labels using the <code>ATTACH_DATACOORD</code>
 * <code>AttachMethod</code> will be attached to.  It also specifies
 * the backup data view to use when a label using the
 * <code>ATTACH_DATAINDEX</code> <code>AttachMethod</code> does not
 * specify an <code>AttachDataIndex</code> or does not specify a data
 * view within the <code>AttachDataIndex</code>.  Data views used to
 * determine the correct axes against which to position the label.
 * @param dv the <code>ChartDataView</code> relative to which the label is
 * attached
 */
public void setDataView(ChartDataView dv)
{
    dataView = dv;
    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the data view specified with the setDataView() method.
 * If no data view has been previously set the effective data view
 * from getEffectiveDataView() will be returned.
 * @return the <code>ChartDataView</code> relative to which
 * the label is attached
 */
public ChartDataView getDataView()
{
    ChartDataView chartDataView = dataView;
    if (chartDataView == null) {
		chartDataView = getEffectiveDataView();
	}
	return chartDataView;
}

/**
 * Internal use only.  Contains the actual value of the DataView
 * property set with the setDataView() method.  The value of this
 * property may be null.
 * @return the <code>ChartDataView</code> previously set by calling
 *         setDataView.  This value may be null.
 */
public ChartDataView getSetDataView()
{
	return dataView;
}

/**
 * Gets the effective data view, that is the data view that will be
 * used to determine the correct attachment axes for this label.  The
 * value will change depending on the value of the <code>AttachMethod</code>
 * and <code>AttachDataIndex</code> properties.  The returned value may be
 * null.
 * If the label is using an <code>AttachMethod</code> of
 * <code>ATTACH_DATAINDEX</code>,  the effective data view is the data
 * view specified in the <code>AttachDataIndex</code> property.  If this
 * data view is null or if the label is using another <code>AttachMethod</code>,
 * the effective data view is the data view previously set with the
 * setDataView() method.  If this value is also null, the effective
 * data view is the first data view of the parent chart.
 * @return the <code>ChartDataView</code> currently being used to attach
 * this label.  The value may be null.
 */
public ChartDataView getEffectiveDataView()
{
    ChartDataView dv = null;

	// If attaching to data index, get from data index
	if (attachMethod == ATTACH_DATAINDEX &&
			attachDataIndex != null &&
			attachDataIndex.getDataView() != null) {
		dv = attachDataIndex.getDataView();
	}

	// If not using data index (or if data index contains null data view),
    // use currently set data view property value
	if (dv == null) {
		dv = getSetDataView();
	}

	// if data view still null, try to grab from parent chart
    if (dv == null && getParentChart() != null) {
        dv = getParentChart().getDataView(0);
    }

    return(dv);
}

/**
 * Sets the data coordinates that the label is attached to,
 * if the label's <code>attachMethod</code> is <code>ATTACH_DATACOORD</code>.
 * @param p the coordinate in chart axis coordinates where the label is attached
 */
public void setDataCoord(JCDataCoord p)
{
    dataAttachX = p.x;
    dataAttachY = p.y;
    setChanged(true, Changeable.RECALC);
    recalc();
}

/**
 * Gets the data coordinates that the label is attached to,
 * if the label's <code>attachMethod</code> is <code>ATTACH_DATACOORD</code>.
 * @return p the coordinate in chart axis coordinates where the
 * label is attached
 */
public JCDataCoord getDataCoord()
{
    return(new JCDataCoord(dataAttachX, dataAttachY));
}

/**
 * Sets the data item that the label is attached to.
 * This is used when the label's <code>attachMethod</code> is
 * <code>ATTACH_DATAINDEX</code>.
 * @param di the data index the label is attached to
 */
public void setDataIndex(JCDataIndex di)
{
    attachDataIndex = di;
    setChanged(true, Changeable.RECALC);
    recalc();
}

/**
 * Gets the data item that the label is attached to.
 * @return di the data index the label is attached to
 */
public JCDataIndex getDataIndex()
{
    return(attachDataIndex);
}

/**
 * Sets the Offset to adjust the location of the label relative to
 * the position determined by the <code>attachMethod</code>.
 * @param point the x and y adjustments applied to the calculated
 * location
 */
public void setOffset(Point point)
{
    offsetX = point.x;
    offsetY = point.y;
    setChanged(true, Changeable.RECALC);
    recalc();
}

/**
 * Gets the adjustment to the location of the label relative to
 * the position determined by the <code>attachMethod</code>.
 * @return the x and y corrections applied to the calculated
 * location
 */
public Point getOffset()
{
    return(new Point(offsetX, offsetY));
}

/**
 * Determines whether a line is drawn from the chart label
 * to the attachment position. Works only if the attachment
 * method is either <code>ATTACH_DATACOORD</code> or
 * <code>ATTACH_DATAINDEX</code>.
 * @param b if <code>true</code>, a line is drawn from the chart
 * label to the attachement position; if <code>false</code>, the line
 * will not be drawn
 */
public void setConnected(boolean b)
{
    if (b == connected) {
        return;
    }

    connected = b;
    if (connected) {
        addConnectedLine();
    }
    else if (!connected) {
        removeConnectedLine();
    }

    setChanged(true, Changeable.RECALC);
}

/**
 * Gets a value indicating if this is a label that is connected or not.
 * @return <code>true</code> if this is a connected label; <code>false</code> if this
 * is not a connected label
 */
public boolean isConnected()
{
    return(connected);
}

/**
 * Sets this label to always display, or to display only when the cursor is over
 * the data to which the label is attached.
 * @param b if <code>true</code>, this is a transient label which will be
 * displayed only when the mouse is over the data item; if <code>false</code>,
 * the label will always be displayed
 */
public void setDwellLabel(boolean b)
{
    if (b == dwellLabel) {
        return;
    }

    dwellLabel = b;


    // if this was a non-dwell label, remove it from the chart and add it
    // to the dwell list.  Otherwise, add it to the chart and remove it
    // from the dwell list.
    JCChart chart = getParentChart();
    if (chart != null) {
        if (b == true) {
            if (component != null) {
                chart.remove(component);
            }
            if (isConnected() && line != null) {
                chart.remove(line);
            }
            getParentManager().addToDwellLabels(this);
        }
        else {
            chart.getChartLabelManager().removeFromDwellLabels(this);
            if (component != null) {
                if (isConnected() && line != null) {
                    chart.add(line, 0);
                }
                chart.add(component, 0);
            }
        }
    }

    setChanged(true, Changeable.RECALC);
}

/**
 * Gets the value indicating if this is a dwell label or not.
 * @return <code>true</code> if this is a transient label; <code>false</code> if this
 * is a 'permanent' label
 */
public boolean isDwellLabel()
{
    return dwellLabel;
}


/**
 * Determines if the label is attached to the specified data index. Used by
 * <code>JCChart</code> to determine when the mouse is over a dwell label.
 * @return whether or not the given point cooresponds to the attachment
 * of this label
 * @param di
 */
public boolean shouldPopupHere(JCDataIndex di)
{
    if (attachMethod != ATTACH_DATAINDEX)
        return(false);

    if (di != null && di.equals(attachDataIndex)) {
        if (component != null && component.isVisible()) {
            return(true);
        }
    }

    return(false);
}

/**
 * Adds the <code>JCChartLabel</code> line component associated with this chart
 * label to the chart at the appropriate position, right after the component
 * associated with this chart label.
 */
protected void addConnectedLine()
{
    line = new JCChartLabelLine();
    JCChart chart = getParentChart();
    if (chart != null && component != null && !dwellLabel) {
        Component[] comps = chart.getComponents();
        for (int i = 0; i < comps.length; i++) {
            if (comps[i] == component) {
                chart.add(line, i + 1);
                break;
            }
        }
    }
}

/**
 * Removes the <code>JCChartLabel</code> line component associated with this
 * chart label from the chart.
 */
protected void removeConnectedLine()
{
    JCChart chart = getParentChart();
    if (chart != null && line != null && !dwellLabel) {
        chart.remove(line);
    }
    line = null;
}

/* Set Changed Stuff  -- maybe this should be in a separate class */

/**
 * Sets the value of the <code>Changed</code> property and updates the
 * parent object.
 * <P>The <code>Changed</code> property determines whether the object needs
 * to be recalculated.
 * @param c new value for <code>Changed</code>
 * @param cf
 */
public void setChanged(boolean c, int cf)
{
    if (c != getChanged() || !getChanged(cf)) {
        super.setChanged(c, cf);
    }
    if (c && parent != null && parent.getParentChart() != null) {
        parent.updateParent(cf);
    }
}

public void setChanged(boolean n, int cf, boolean updateParent)
{
    super.setChanged(n, cf, updateParent);
}

protected void getDefaultLabel()
{
    if (!(component instanceof JLabel)) {
        return;
    }

	String defaultText = "";

    // Try to get text from data point
    if (attachMethod == ATTACH_DATAINDEX && attachDataIndex != null) {
        ChartDataView dv = getEffectiveDataView();
        int si = attachDataIndex.getSeriesIndex();
        int p = attachDataIndex.getPoint();

        if (dv != null &&
            (si >= 0 || si == JCPieChartFormat.OTHER_SLICE) &&
            p >= 0)
        {
            if (getParentManager() != null) {
                defaultText = getParentManager().createAutoLabelText(dv, si, p);
            }
        }
    }
    else if (attachMethod == ATTACH_DATACOORD) {
        StringBuffer sb = new StringBuffer();
        sb.append(" (");
        sb.append(dataAttachX);
        sb.append(",");
        sb.append(dataAttachY);
        sb.append(") ");
        defaultText = sb.toString();
    }
    else if (attachMethod == ATTACH_COORD) {
        StringBuffer sb = new StringBuffer();
        sb.append(" (");
        sb.append(attachX);
        sb.append(",");
        sb.append(attachY);
        sb.append(") ");
        defaultText = sb.toString();
    }

    JLabel lbl = (JLabel)component;
	lbl.setText(defaultText);

	this.defaultText = defaultText;
}

}
