/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCChartLabelManager.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.awt.Graphics;
import java.awt.Point;
import javax.swing.JComponent;
import java.util.List;

/**
 * An interface to be used by any class wanting to
 * implement chart label functionality within JClass Chart.
 */
public interface JCChartLabelManager
{

/**
 * Sets the chart for which this class is managing chart labels.
 * @param chart parent <code>JCChart</code> instance
 */
public void setParentChart(JCChart chart);

/**
 * Gets the chart for which this class is managing chart labels.
 * @return parent <code>JCChart</code> instance
 */
public JCChart getParentChart();

/**
 *  Paints the connectors between the chart labels and the points to which
 *  they are connected.
 *  @param gc Graphics object of parent chart used for painting connectors
 */
public void paintConnectors(Graphics gc);


/**
 *  Adds a chart label, forcing a recalculation of it and a redraw of the parent
 *  chart.
 *  @param label the <code>JCChartLabel</code> instance to add
 */
public void addChartLabel(JCChartLabel label);

/**
 * Adds a new chart label to the list of dwell labels.
 * @param label the <code>JCChartLabel</code> instance being added to the dwell
 * label list
 */
public void addToDwellLabels(JCChartLabel label);

/**
 * Locates the dwell label that is associated with the given
 * coordinates and marks it as the current dwell label.
 * @param x the <i>x</i> coordinate within the <code>JCChartArea</code>
 * @param y the <i>y</i> coordinate within the <code>JCChartArea</code>
 * @return <code>true</code> if dwell label is found; <code>false</code>
 * otherwise
 */
public boolean locateCurrentDwellLabel(int x, int y);

/**
 * Gets the <code>AttachPoint</code> property of the current dwell label, if
 * one exists.
 * @return the <code>AttachPoint</code> property of the current dwell label, an
 *     instance of the <code>Point</code> object, or null
 */
public Point getCurrentDwellLabelAttachPoint();

/**
 * Gets the <code>Component</code> property of the current dwell label.
 * @return the component part of the current dwell label, an instance of
 * the <code>JComponent</code> class, or null
 */
public JComponent getCurrentDwellLabelComponent();

/**
 * Gets the current dwell label.
 * @return the current dwell label, an instance of
 * the <code>JChartLabel</code> class, or null
 */
public JCChartLabel getCurrentDwellLabel();

/**
 * Removes a chart label, forcing a redraw of
 * the parent chart.
 * @param label the <code>JCChartLabel</code> instance to remove from the chart
 *     label list
 */
public void removeChartLabel(JCChartLabel label);

/**
 * Removes a chart label from the list of dwell labels.
 * @param label the <code>JCChartLabel</code> instance to remove from the dwell
 *     label list
 */
public void removeFromDwellLabels(JCChartLabel label);

/**
 * Gets the current number of chart labels.
 * @return the number of chart labels this manager holds
 */
public int getNumChartLabels();

/**
 * Removes all of the chart labels. If the <code>AutoLabel</code> property has
 * been set on any of this chart's <code>DataViews</code>, the property should
 * be set to <code>false</code> and the list of auto labels nulled out.
 */
public void removeAllChartLabels();

/**
 * Gets a value of the <code>ChartLabels</code> indexed property.
 * The <code>ChartLabels</code> property controls a list of chart labels
 * managed by the chart.
 * @param index position at which to add the new object
 * @return requested chart label
 */
public JCChartLabel getChartLabels(int index);

/**
 * Sets a value of the <code>ChartLabels</code> indexed property.
 * The <code>ChartLabels</code> property controls a list of chart labels
 * managed by the chart.
 * @param index position at which to add the new object; to add a new
 * item at the end, set the position to the current size of the labels
 * array
 * @param lbl new chart label to be added
 * @exception IllegalArgumentException Thrown if the index is larger than
 * the size of the array.
 */
public void setChartLabels(int index, JCChartLabel lbl);

/**
 * Gets the value of the <code>ChartLabels</code> indexed property.
 * The <code>ChartLabels</code> property controls a list of chart labels
 * managed by the chart.
 * @return List object containing all the chart labels for this
 * <code>JCChart</code>
 */
public List getChartLabels();

/**
 * Sets the value of the <code>ChartLabels</code> indexed property.
 * The <code>ChartLabels</code> property controls a list of chart labels
 * managed by the chart.
 * @param s List object containg chart labels to add to this
 * <code>JCChart</code>
 */
public void setChartLabels(List s);

/**
 * Marks all chart labels for later recalculation.
 */
public void markAllChartLabelsForRecalc();

/**
 * Forces recalculation of any chart labels marked for a recalc. This
 * is necessary when chart label property changes force a chart layout.
 */
public void recalcChangedChartLabels();

/**
 * Gets the <code>AutoLabelList</code> property for the
 * <code>ChartDataView</code> object. The <code>AutoLabelList</code> property
 * is a two-dimensional array of automatically generated
 * <code>JCChartLabel</code> instances, one for every point and series. The
 * inner array is indexed by point; the outer array by series.
 * @param cdv the <code>ChartDataView</code> instance that defines the auto
 *     label list
 * @return array of <code>JCChartLabel</code> objects representing the
 *     automatically generated chart labels
 */
public JCChartLabel[][] getAutoLabelList(ChartDataView cdv);

/**
 * Automatically creates <code>JCChartLabels</code> for every series and point,
 * if requested.
 * @param cdv the <code>ChartDataView</code> instance that defines the auto
 *     label list
 */
public void generateAutoLabels(ChartDataView cdv);

/**
 * Creates the text for an auto label given the series and point.
 * @param cdv the <code>ChartDataView</code> instance that defines the auto
 *     label list
 * @param series the series number
 * @param point the point number within the series
 * @return the newly created text
 */
public String createAutoLabelText(ChartDataView cdv, int series, int point);

/**
 * Sets the text on an auto label given the series and point.
 * @param cdv the <code>ChartDataView</code> instance that defines the auto
 *     label list
 * @param series the series number
 * @param point the point number within the series
 */
public void setAutoLabelText(ChartDataView cdv, int series, int point);

/**
 * Adds some automatically created <code>JCChartLabels</code>.
 * @param cdv the <code>ChartDataView</code> instance that defines the auto
 *     label list
 * @param addSeries the series number to add auto labels for (if -1,
 * add applies to all series)
 * @param addPoint the point number within a series to add auto labels
 * for (if -1, add applies to all points within the given series)
 */
public void addAutoLabels(ChartDataView cdv, int addSeries, int addPoint);

/**
 * Removes some of the automatically created <code>JCChartLabels</code>.
 * @param cdv the <code>ChartDataView</code> instance that defines the auto
 *     label list
 * @param remSeries the series number to remove auto labels for (if -1,
 * remove applies to all series)
 * @param remPoint the point number within a series to remove auto labels
 * for (if -1, remove applies to all points within the given series)
 */
public void removeAutoLabels(ChartDataView cdv, int remSeries, int remPoint);

/**
 * Removes all of the automatically created <code>JCChartLabels</code>.
 * @param cdv the <code>ChartDataView</code> instance that defines the auto
 *     label list
 */
public void removeAutoLabels(ChartDataView cdv);

/**
 * Clears the auto labels list for this data view. Simply removes the
 * lists' references internally. Does nothing to remove the
 * labels from the chart.
 * @param cdv <code>ChartDataView</code> instance from which the auto labels
 *     are to be cleared
 */
public void clearAutoLabels(ChartDataView cdv);

/**
 * Updates the parent chart based on the type of change made to the child
 * chart label.
 * @param cf the changed flag
 */
public void updateParent(int cf);

}
