/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCChartStyle.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.awt.Color;
import java.awt.Image;
import java.util.Vector;

/**
 * Represents a style for rendering a
 * data series. It controls fill styles,
 * symbol styles, and line styles.
 * <P><STRONG><CENTER>Properties</CENTER></STRONG></P>
 * <P></P>
 * <TABLE BORDER=1 WIDTH=100%>
 * <TR>
 * <TH>Name</TH>
 * <TH>Description</TH></TR>
 * <TR>
 * <TD><code>FillColor</code>
 * </TD>
 * <TD>
 * The <code>FillColor</code> property determines the color used to fill regions
 * in chart. Default value is generated.
 * </TD></TR>
 * <TR>
 * <TD><code>FillImage</code>
 * </TD>
 * <TD>
 * The <code>FillImage</code> property determines the
 * image used to paint the fill region of bar charts. Default value is null.
 * </TD></TR>
 * <TR>
 * <TD><code>FillPattern</code>
 * </TD>
 * <TD>
 * The <code>FillPattern</code> property determines the fill pattern used to
 * fill regions in chart. Default value is <code>JCFillStyle.SOLID</code>.
 * </TD></TR>
 * <TR>
 * <TD><code>FillStyle</code>
 * </TD>
 * <TD>
 * The <code>FillStyle</code> property controls the appearance of filled areas in
 * chart. See <code>JCFillStyle</code> for additional properties.<br>
 * <b>Note:</b> All <code>JCChartStyle</code> properties of the format
 * <i>Fill*</i> are virtual properties that map to properties of
 * <code>JCFillStyle</code>.
 * </TD></TR>
 * <TR>
 * <TD><code>LineColor</code>
 * </TD>
 * <TD>
 * The <code>LineColor</code> property determines the color used to draw a line.
 * Default value is generated.
 * </TD></TR>
 * <TR>
 * <TD><code>LinePattern</code>
 * </TD>
 * <TD>
 * The <code>LinePattern</code> property dictates the pattern used to draw a
 * line. Valid values include <code>JCLineStyle.NONE</code>,
 * <code>JCLineStyle.SOLID</code>, <code>JCLineStyle.LONG_DASH</code>,
 * <code>JCLineStyle.SHORT_DASH</code>, <code>JCLineStyle.LSL_DASH</code>, and
 * <code>JCLineStyle.DASH_DOT</code>. Default value is
 * <code>JCLineStyle.SOLID</code>.
 * </TD></TR>
 * <TR>
 * <TD><code>LineStyle</code>
 * </TD>
 * <TD>
 * The <code>LineStyle</code> property controls the appearance of lines in
 * chart. See <code>JCLineStyle</code> for additional properties.
 * </TD></TR>
 * <TR>
 * <TD><code>LineWidth</code>
 * </TD>
 * <TD>
 * The <code>LineWidth</code> property controls the line width.
 * Default value is 1.
 * </TD></TR>
 * <TR>
 * <TD><code>SymbolColor</code>
 * </TD>
 * <TD>The <code>SymbolColor</code> property determines the color used to paint
 * the symbol. Default value is generated.
 * </TD></TR>
 * <TR>
 * <TD><code>SymbolCustomShape</code>
 * </TD>
 * <TD>
 * The <code>SymbolCustomShape</code> property contains an object derived from
 * <code>JCShape</code> that is used to draw points. See <code>JCShape</code>
 * for details. Default value is null.
 * </TD></TR>
 * <TR>
 * <TD><code>SymbolShape</code>
 * </TD>
 * <TD>
 * The <code>SymbolShape</code> property determines the type of symbol that will
 * be drawn. Valid values include <code>JCSymbolStyle.NONE</code>,
 * <code>JCSymbolStyle.DOT</code>, <code>JCSymbolStyle.BOX</code>,
 * <code>JCSymbolStyle.TRIANGLE</code>, <code>JCSymbolStyle.DIAMOND</code>,
 * <code>JCSymbolStyle.STAR</code>, <code>JCSymbolStyle.VERT_LINE</code>,
 * <code>JCSymbolStyle.HORIZ_LINE</code>, <code>JCSymbolStyle.CROSS</code>,
 * <code>JCSymbolStyle.CIRCLE</code>, and <code>JCSymbolStyle.SQUARE</code>.
 * Default value is generated.
 * </TD></TR>
 * <TR>
 * <TD><code>SymbolSize</code>
 * </TD>
 * <TD>
 * The <code>SymbolSize</code> property determines the size of the symbol.
 * Default value is 6.
 * </TD></TR>
 * <TR>
 * <TD><code>SymbolStyle</code>
 * </TD>
 * <TD>
 * The <code>SymbolStyle</code> property controls the symbol that represents
 * an individual point.  See <code>JCSymbolStyle</code> for additional properties.
 * <br><b>Note:</B> all <code>JCChartStyle</code> properties of the format
 * <i>Symbol*</i> are virtual properties that map to properties of
 * <code>JCSymbolStyle</code>.
 * </TD></TR>
 * </TABLE>
 * <P>
 * @see com.klg.jclass.chart.JCFillStyle
 * @see com.klg.jclass.chart.JCLineStyle
 * @see com.klg.jclass.chart.JCSymbolStyle
 */
public class JCChartStyle implements java.io.Serializable {

protected JCLineStyle    lineStyle = null;
protected JCFillStyle    fillStyle = null;
protected JCSymbolStyle  symbolStyle = null;

// Vector of Changeable objects used to hold multiple parents so that
// and instance of this object may be shared between multiple parents.
protected Vector parents = new Vector();

/**
 * Constructor for chart style objects.
 * @param ls <CODE>JCLineStyle</CODE> instance representing the line style for
 * this chart style object
 * @param fs <code>JCFillStyle</code> instance representing the fill style for
 * this chart style object
 * @param ps <code>JCSymbolStyle</code> instance representing the symbol style
 * for this chart style object
 */
public JCChartStyle(JCLineStyle ls, JCFillStyle fs, JCSymbolStyle ps)
{
    lineStyle = ls;
    fillStyle = fs;
    symbolStyle = ps;
    lineStyle.parent = fillStyle.parent = symbolStyle.parent = this;
}

/**
 * Default constructor for chart style objects.  Creates default
 * line style, fill style, and symbol style objects.
 */
public JCChartStyle()
{
    lineStyle = JCLineStyle.makeDefault(null);
    fillStyle = JCFillStyle.makeDefault(null);
    symbolStyle = JCSymbolStyle.makeDefault(null);
    lineStyle.parent = fillStyle.parent = symbolStyle.parent = this;
}

/**
 * Creates a default chart style.
 * @param c <code>JCChart</code> instance that is the parent of the created
 * <code>JCChartStyle</code> object
 * @return <code>JCChartStyle</code> object with default line, fill, and
 * symbol styles.  The defaults are cycled, so consecutive calls
 * will return different chart styles.
 */
public static JCChartStyle makeDefault(JCChart c)
{
    return(makeDefault(c, null, false));
}

/**
 * Creates a default chart style.
 * @param c <code>JCChart</code> instance that will use this
 * <code>JCChartStyle</code>
 * @param parent an instance of <code>Changeable</code> that will be the parent
 * of the created <code>JCChartStyle</code> object
 * @return <code>JCChartStyle</code> object with default line, fill, and
 * symbol styles. The defaults are cycled, so consecutive calls
 * will return different chart styles.
 */
public static JCChartStyle makeDefault(JCChart c, Changeable parent)
{
    return(makeDefault(c, parent, false));
}

/**
 * Creates a default chart style.
 * @param c <code>JCChart</code> instance that will use this
 * <code>JCChartStyle</code>
 * @param parent an instance of <code>Changeable</code> that will be the parent
 * of the created <code>JCChartStyle</code> object
 * @param other <code>true</code> if creating from a pie chart another style;
 * <code>false</code> otherwise
 * @return <code>JCChartStyle</code> object with default line, fill, and
 * symbol styles.  The defaults are cycled, so consecutive calls
 * will return different chart styles.
 */
public static JCChartStyle makeDefault(JCChart c, Changeable parent,
                                       boolean other)
{
    JCChartStyle r = new JCChartStyle(JCLineStyle.makeDefault(c),
                                      JCFillStyle.makeDefault(c, other),
                                      JCSymbolStyle.makeDefault(c));
    if (parent != null) {
        r.parents.addElement(parent);
    }
    else if (c != null) {
        r.parents.addElement(c.getChartArea());
    }
    return(r);
}

/**
 * Gets the <code>JCLineStyle</code> property, which controls how lines are
 * drawn.
 * @return <code>JCLineStyle</code> instance representing the line drawing style
 */
public JCLineStyle getLineStyle()
{
    return(lineStyle);
}

/**
 * Sets the <code>JCLineStyle</code> property, which controls how lines are
 * drawn.
 * @param ls <code>JCLineStyle</code> instance representing the line drawing
 * style
 */
public void setLineStyle(JCLineStyle ls)
{
    if (lineStyle == ls) {
        return;
    }
    lineStyle = ls;
    lineStyle.parent = this;

    callParentSetChanged(true, Changeable.REDRAW |
                               Changeable.NOTIFY_LEGEND_REDRAW);
}

/**
 * Gets the value of the <code>LinePattern</code> property.
 * The <code>LinePattern</code> property dictates the pattern used to draw a
 * line. Valid values include <code>JCLineStyle.NONE</code>,
 * <code>JCLineStyle.SOLID</code>, <code>JCLineStyle.LONG_DASH</code>,
 * <code>JCLineStyle.SHORT_DASH</code>, <code>JCLineStyle.LSL_DASH</code>, and
 * <code>JCLineStyle.DASH_DOT</code>.
 * @return line pattern enum
 */
public int getLinePattern()
{
    if (lineStyle == null) return JCLineStyle.NONE;
    return lineStyle.getPattern();
}

/**
 * Sets the value of the <code>LinePattern</code> property.
 * The <code>LinePattern</code> property dictates the pattern used to draw a
 * line. Valid values include <code>JCLineStyle.NONE</code>,
 * <code>JCLineStyle.SOLID</code>, <code>JCLineStyle.LONG_DASH</code>,
 * <code>JCLineStyle.SHORT_DASH</code>, <code>JCLineStyle.LSL_DASH</code>, and
 * <code>JCLineStyle.DASH_DOT</code>.
 * @param p pattern enum
 */
public void setLinePattern(int p)
{
    if (lineStyle == null) {
        lineStyle = JCLineStyle.makeDefault(null);
    }
    lineStyle.setPattern(p);
}

/**
 * Gets the value of the <code>LineJoin</code> property.
 * The <code>LineJoin</code> property dictates the join style used to join two
 * lines. Valid values include <code>BasicStroke.JOIN_MITER</code>,
 * <code>BasicStroke.JOIN_BEVEL</code>, and <code>BasicStroke.JOIN_ROUND</code>.
 * <I>This feature is supported only in JDK 1.2 and higher.</I>
 * @return line pattern enum
 */
public int getLineJoin()
{
    if (lineStyle == null) {
        return(-1);
    }
    return(lineStyle.getJoin());
}

/**
 * Sets the value of the <code>LineJoin</code> property.
 * The <code>LineJoin</code> property dictates the join style used to join two
 * lines. Valid values include <code>BasicStroke.JOIN_MITER</code>,
 * <code>BasicStroke.JOIN_BEVEL</code>, and <code>BasicStroke.JOIN_ROUND</code>.
 * <I>This feature is supported only in JDK 1.2 and higher.</I>
 * @param j join enum
 */
public void setLineJoin(int j)
{
    if (lineStyle == null) {
        lineStyle = JCLineStyle.makeDefault(null);
    }
    lineStyle.setJoin(j);
}

/**
 * Gets the value of the <code>LineCap</code> property.
 * The <code>LineCap</code> property dictates the cap style used to end a line.
 * Valid values include <code>BasicStroke.CAP_BUTT</code>,
 * <code>BasicStroke.CAP_ROUND</code>, and <code>BasicStroke.CAP_SQUARE</code>.
 * <I>This feature is supported only in JDK 1.2 and higher.</I>
 * @return line pattern enum
 */
public int getLineCap()
{
    if (lineStyle == null) {
        return(-1);
    }
    return(lineStyle.getCap());
}

/**
 * Sets the value of the <code>LineCap</code> property.
 * The <code>LineCap</code> property dictates the cap style used to end a line.
 * Valid values include <code>BasicStroke.CAP_BUTT</code>,
 * <code>BasicStroke.CAP_ROUND</code>, and <code>BasicStroke.CAP_SQUARE</code>.
 * <I>This feature is supported only in JDK 1.2 and higher.</I>
 * @param c cap enum
 */
public void setLineCap(int c)
{
    if (lineStyle == null) {
        lineStyle = JCLineStyle.makeDefault(null);
    }
    lineStyle.setCap(c);
}

/**
 * Gets the value of the <code>LineWidth</code> property.
 * The <code>LineWidth</code> property controls the line width.
 * @return line width
 */
public int getLineWidth()
{
    if (lineStyle == null) {
        return(0);
    }
    return(lineStyle.getWidth());
}

/**
 * Sets the value of the <code>LineWidth</code> property.
 * The <code>LineWidth</code> property controls the line width.
 * @param lw line width
 */
public void setLineWidth(int lw)
{
    if (lineStyle == null) {
        lineStyle = JCLineStyle.makeDefault(null);
    }
    lineStyle.setWidth(lw);
}

/**
 * Gets the value of the <code>LineColor</code> property.
 * The <code>LineColor</code> property determines the color used to draw a line.
 * @return line color
 */
public Color getLineColor()
{
    if (lineStyle == null) {
        return(null);
    }
    return(lineStyle.getColor());
}

/**
 * Sets the value of the <code>LineColor</code> property.
 * The <code>LineColor</code> property determines the color used to draw a line.
 * @param lc line color
 */
public void setLineColor(Color lc)
{
    if (lineStyle == null) {
        lineStyle = JCLineStyle.makeDefault(null);
    }
    lineStyle.setColor(lc);
}

/**
 * Gets the <code>SymbolStyle</code> property.
 * The <code>SymbolStyle</code> property controls the symbol that represents
 * an individual point.
 * @return <code>JCSymbolStyle</code> instance representing the symbol drawing
 * style
 */
public JCSymbolStyle getSymbolStyle()
{
    return(symbolStyle);
}

/**
 * Gets the <code>SymbolStyle</code> property.
 * The <code>SymbolStyle</code> property controls the symbol that represents
 * an individual point.
 * @param ps <code>JCSymbolStyle</code> instance representing the symbol drawing
 * style
 */
public void setSymbolStyle(JCSymbolStyle ps)
{
    if (symbolStyle == ps) {
        return;
    }
    symbolStyle = ps;
    symbolStyle.parent = this;

    callParentSetChanged(true, Changeable.REDRAW |
                               Changeable.NOTIFY_LEGEND_REDRAW);
}

/**
 * Gets the value of the <code>SymbolShape</code> property.
 * The <code>SymbolShape</code> property determines the type of symbol that will
 * be drawn.  Valid values include <code>JCSymbolStyle.NONE</code>,
 * <code>JCSymbolStyle.DOT</code>, <code>JCSymbolStyle.BOX</code>,
 * <code>JCSymbolStyle.TRIANGLE</code>, <code>JCSymbolStyle.DIAMOND</code>,
 * <code>JCSymbolStyle.STAR</code>, <code>JCSymbolStyle.VERT_LINE</code>,
 * <code>JCSymbolStyle.HORIZ_LINE</code>, <code>JCSymbolStyle.CROSS</code>,
 * <code>JCSymbolStyle.CIRCLE</code>, and <code>JCSymbolStyle.SQUARE</code>.
 * @return symbol shape
 */
public int getSymbolShape()
{
    if (symbolStyle == null) {
        return(JCSymbolStyle.NONE);
    }
    return(symbolStyle.getShape());
}

/**
 * Sets the value of the <code>SymbolShape</code> property.
 * The <code>SymbolShape</code> property determines the type of symbol that will
 * be drawn.  Valid values include <code>JCSymbolStyle.NONE</code>,
 * <code>JCSymbolStyle.DOT</code>, <code>JCSymbolStyle.BOX</code>,
 * <code>JCSymbolStyle.TRIANGLE</code>, <code>JCSymbolStyle.DIAMOND</code>,
 * <code>JCSymbolStyle.STAR</code>, <code>JCSymbolStyle.VERT_LINE</code>,
 * <code>JCSymbolStyle.HORIZ_LINE</code>, <code>JCSymbolStyle.CROSS</code>,
 * <code>JCSymbolStyle.CIRCLE</code>, and <code>JCSymbolStyle.SQUARE</code>.
 * @param st symbol shape
 */
public void setSymbolShape(int st)
{
    if (symbolStyle == null) {
        symbolStyle = JCSymbolStyle.makeDefault(null);
    }
    symbolStyle.setShape(st);
}

/**
 * Gets the value of the <code>SymbolColor</code> property.
 * The <code>SymbolColor</code> property determines color used to paint the
 * symbol.
 * @return symbol color
 */
public Color getSymbolColor()
{
    if (symbolStyle == null) {
        return(null);
    }
    return(symbolStyle.getColor());
}

/**
 * Sets the value of the <code>SymbolColor</code> property.
 * The <code>SymbolColor</code> property determines color used to paint the
 * symbol.
 * @param sc symbol color
 */
public void setSymbolColor(Color sc)
{
    if (symbolStyle == null) {
        symbolStyle = JCSymbolStyle.makeDefault(null);
    }
    symbolStyle.setColor(sc);
}

/**
 * Gets the value of the <code>SymbolSize</code> property.
 * The <code>SymbolSize</code> property determines the size of the symbols.
 * @return positive integer representing symbol size
 */
public int getSymbolSize()
{
    if (symbolStyle == null) {
        return(0);
    }
    return(symbolStyle.getSize());
}

/**
 * Sets the value of the <code>SymbolSize</code> property.
 * The <code>SymbolSize</code> property determines the size of the symbols.
 * When set to 0, no symbol is drawn.
 * @param sz symbol size
 */
public void setSymbolSize(int sz)
{
    if (symbolStyle == null) {
        symbolStyle = JCSymbolStyle.makeDefault(null);
    }
    symbolStyle.setSize(sz);
}

/**
 * Gets the value of the <code>SymbolCustomShape</code> property.
 * The <code>SymbolCustomShape</code> property contains an object derived from
 * <code>JCShape</code> that is used to draw points.  See <code>JCShape</code>
 * for details.
 * @return custom shape object
 */
public JCShape getSymbolCustomShape()
{
    if (symbolStyle == null) {
        return(null);
    }
    return(symbolStyle.getCustomShape());
}

/**
 * Sets the value of the <code>SymbolCustomShape</code> property.
 * The <code>SymbolCustomShape</code> property contains an object derived from
 * <code>JCShape</code> that is used to draw points. See <code>JCShape</code>
 * for details.
 * @param cs custom shape object
 */
public void setSymbolCustomShape(JCShape cs)
{
    if (symbolStyle == null) {
        symbolStyle = JCSymbolStyle.makeDefault(null);
    }

    symbolStyle.setCustomShape(cs);
    // don't call callParentSetChanged() here because it is already
    // called in symbolStyle
}

/**
 * Gets the <code>JCFillStyle</code> property, which controls how fills are
 * drawn.
 * @return <code>JCFillStyle</code> instance representing the fill drawing style
 */
public JCFillStyle getFillStyle()
{
    return(fillStyle);
}

/**
 * Sets the <code>JCFillStyle</code> property, which controls how fills are
 * drawn.
 * @param fs <code>JCFillStyle</code> instance representing the fill drawing
 * style
 */
public void setFillStyle(JCFillStyle fs)
{
    if (fillStyle == fs) {
        return;
    }
    fillStyle = fs;
    fillStyle.parent = this;

    callParentSetChanged(true, Changeable.REDRAW |
                               Changeable.NOTIFY_LEGEND_RECALC);
}

/**
 * Gets the value of the <code>FillColor</code> property.
 * The <code>FillColor</code> property determines the color used to fill regions
 * in a chart.
 * @return fill color
 */
public Color getFillColor()
{
    if (fillStyle == null) {
        return(null);
    }
    return(fillStyle.getColor());
}

/**
 * Sets the value of the <code>FillColor</code> property.
 * The <code>FillColor</code> property determines the color used to fill regions
 * in a chart.
 * @param fc fill color
 */
public void setFillColor(Color fc)
{
    if (fillStyle == null) {
        fillStyle = JCFillStyle.makeDefault(null);
    }
    fillStyle.setColor(fc);
}

/**
 * Gets the value of the <code>FillBackground</code> property.
 * The <code>FillBackground</code> property determines the color used for the
 * background when doing patterned fills in chart.
 * @return fill background color
 */
public Color getFillBackground()
{
    if (fillStyle == null) {
        return(null);
    }
    return(fillStyle.getBackground());
}

/**
 * Sets the value of the <code>FillBackground</code> property.
 * The <code>FillBackground</code> property determines the color used for the
 * background when doing patterned fills in chart.
 * @param fc fill background color
 */
public void setFillBackground(Color fc)
{
    if (fillStyle == null) {
        fillStyle = JCFillStyle.makeDefault(null);
    }
    fillStyle.setBackground(fc);
}

/**
 * Gets the value of the <code>FillPattern</code> property.
 * The <code>FillPattern</code> property determines the fill pattern used to
 * fill regions in chart.
 * @return fill pattern
 */
public int getFillPattern()
{
    if (fillStyle == null) {
        return(0);
    }
    return(fillStyle.getPattern());
}

/**
 * Sets the value of the <code>FillPattern</code> property.
 * The <code>FillPattern</code> property determines the fill pattern used to
 * fill regions in chart.
 * @param fp fill pattern
 */
public void setFillPattern(int fp)
{
    if (fillStyle == null) {
        fillStyle = JCFillStyle.makeDefault(null);
    }
    fillStyle.setPattern(fp);
}

/**
 * Gets the value of the <code>FillImage</code> property.
 * The <code>FillImage</code> property determines the
 * image used to paint the fill region of bar charts.
 * @return fill image
 */
public Image getFillImage()
{
    if (fillStyle == null) {
        return(null);
    }
    return(fillStyle.getImage());
}

/**
 * Sets the value of the <code>FillImage</code> property.
 * The <code>FillImage</code> property determines the
 * image used to paint the fill region of bar charts.
 * @param fi fill image
 */
public void setFillImage(Image fi)
{
    if (fillStyle == null) {
        fillStyle = JCFillStyle.makeDefault(null);
    }
    fillStyle.setImage(fi);
}

/**
 * Calls the <code>setChanged</code> methods of all of the parents of this
 * <code>JCChartStyle</code>.
 * @param c
 * @param cf
 */
void callParentSetChanged(boolean c, int cf)
{
    if (parents == null) {
        return;
    }

    for (int i = 0; i < parents.size(); i++) {
        ((Changeable)parents.elementAt(i)).setChanged(true, cf);
    }
}

/**
 * Adds a parent to the list of parents for the <code>JCChartStyle</code>
 * object.
 * @param parent the <code>Changeable</code> object to add to the parent list
 */
public void addParent(Changeable parent)
{
    for (int i = 0; i < parents.size(); i++) {
        if (parents.elementAt(i) == parent) {
            return;
        }
    }
    parents.addElement(parent);

}

/**
 * Removes a parent from the list of parents for the <code>JCChartStyle</code>
 * object.
 * @param parent the <code>Changeable</code> object to remove from the parent
 * list.
 */
public void removeParent(Changeable parent)
{
    for (int i = 0; i < parents.size(); i++) {
        if (parents.elementAt(i) == parent) {
            parents.removeElementAt(i);
        }
    }
}

/**
 * Clears the list of parents of this <code>JCChartStyle</code> object.
 */
public void clearParents()
{
    parents = new Vector();
}

}
