/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCChartUI.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.UIResource;
import java.awt.*;

/**
 * <code>JCChartUI</code> implements pluggable look and feel for JCChart.
 * This class handles the UI for all possible UI's (which allows for
 * similarities between JClass Chart and Swing). Background and border
 * UI attributes have been taken from <code>JPanel</code>, and the
 * foreground and font attributes are from <code>JLabel</code>.
 * <p>If a user has set these properties themselves, their values
 * take precedence.
 */
public class JCChartUI extends ComponentUI {
	
protected static JCChartUI chartUI;

public void installUI(JComponent c)
{
    LookAndFeel.installColorsAndFont(c, "Panel.background",
                                     "Label.foreground",
                                     "Label.font");
    installDefaults(c);
}

protected void installDefaults(JComponent c)
{
    JCChart chart = (JCChart)c;

    // Only change properties if user hasn't set them yet.

    Color bgColor = chart.getBackground();
    if (bgColor == null || bgColor instanceof UIResource) {
        chart.setBackground(UIManager.getColor("Panel.background"));
    }

    Color fgColor = chart.getForeground();
    if (fgColor == null || fgColor instanceof UIResource) {
        chart.setForeground(UIManager.getColor("Label.foreground"));
    }

    Font chFont = chart.getFont();
    if (chFont == null || chFont instanceof UIResource) {
        chart.setFont(UIManager.getFont("Label.font"));
    }

    Border chBorder = chart.getBorder();
    if (chBorder == null || chBorder instanceof UIResource) {
        chart.setBorder(UIManager.getBorder("Panel.border"));
    }
}

public static ComponentUI createUI(JComponent c)
{
    if (chartUI == null) {
        chartUI = new JCChartUI();
    }
    return(chartUI);
}

public void uninstallUI(JComponent c) {}

}
