/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCCustomizerPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import javax.swing.*;
import java.awt.*;

/**
 * Base class for Property Pages. This class is used internally by JClass Chart
 * and should not be used by applications.
 */
public abstract class JCCustomizerPage extends JPanel {

protected Object target = null;

/**
 * Method called to initialize the page.
 */
public void init() {}

/**
 * Method called to return the target that this page is editing.
 * @return target that the page is editing
 */
public Object getObject()
{
    return(target);
}

/**
 * Method called to update the page with values from the given source.
 * @param source
 */
public void setObject(Object source)
{
    target = source;
}

/**
 * Method called to update the page with values from the current target.
 */
public void setObject()
{
    setObject(getObject());
}

/**
 * Method to <code>show()</code> an Info Dialog containing important messages.
 * @param msg
 */
public static void showError(String msg)
{
    ErrorDialog.raise(msg);
}

/**
 * Utility method to return a frame in which to place a propery page.
 * Will be overridden in subclasses.
 * @return frame for property page
 */
public JFrame getPropertyFrame()
{
    return(new JFrame());
}

/**
 * Utility method to show the Customizer.
 */
public void launch()
{
    JFrame frame = getFrame(this);
    if (frame == null) {
		try {
			frame = getPropertyFrame();
			frame.setTitle(JCChartBundle.string(JCChartBundle.key90));
		    frame.getContentPane().setLayout(new java.awt.GridLayout(1, 1));
			frame.getContentPane().add( this );
			frame.pack();
			java.awt.Dimension bigness = this.getPreferredSize();
			// Fix bug #2825: Customizer coming up too small
			// under Netscape and Internet Explorer.
			// Seems that getPreferredSize() is not reliable on
			// these platforms. Work around by detecting
			// a too-small 'bigness'.
			if (bigness.width < 400 || bigness.height < 200) {
			    // Hard-code 'optimal' size of JCChart Customizer
				frame.setSize(740, 440);
			}
			else {
			    // 'Nice' size allows for frame's menubar
				frame.setSize(bigness.width + 10, bigness.height + 50);
			}
		}
		catch (Throwable e) {
			showError(JCChartBundle.string(JCChartBundle.key87) + "\n" + e);
			frame = null;
		}
    }
    if (frame != null) {
		frame.show();
    }
}

/**
 * Gets the component's parent frame.
 * @return parent frame
 * @param comp component
 */
public static JFrame getFrame( Component comp )
{
    Component tmp = comp, parent = comp;
    while (true) {
        if (parent == null || parent instanceof JFrame) {
            break;
        }
        parent = tmp;
        if (tmp == null) {
            return(null);
        }
        tmp = tmp.getParent();
    }
    return((JFrame)parent);
}

}


