/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCFillStyle.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import com.klg.jclass.util.swing.JCIconCreator;
import com.klg.jclass.util.io.OutputProperties;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.Arc2D;
import java.awt.image.BufferedImage;

/**
 * Controls the appearance of a fill, including the fill pattern and the fill
 * color.
 */
public class JCFillStyle extends JCStyle {

protected Color                    color = null;
protected Color                    backgroundColor = null;
protected int                      pattern = SOLID;
protected int                      colorIndex;
protected Rectangle                anchorRect = null;
protected int                      orientation = NONE;
protected OutputProperties         outputProperties;

// Transient properties
protected transient Image          image = null;
protected transient BufferedImage  buffImage = null;
protected transient TexturePaint   customPaint = null;
protected transient Paint          fillPaint = null;
protected transient Paint          hiFillPaint = null;
protected transient Paint          shFillPaint = null;

// Added to width and height in fillArc(), fillArcShadow(), and fillCircle().
protected int fillCorrection = 0;

// Orientation for fill patterns (use NONE = 0 from fill pattern list)
public final static int  TOP          = 1;
public final static int  BOTTOM       = 2;
public final static int  LEFT         = 3;
public final static int  RIGHT        = 4;
public final static int  CENTER       = 5;
public final static int  BOTTOM_LEFT  = 6;
public final static int  TOP_LEFT     = 7;
public final static int  BOTTOM_RIGHT = 8;
public final static int  TOP_RIGHT    = 9;

// Fill Patterns
public final static int  NONE          = 0;
public final static int  SOLID         = 1;
public final static int  PER_25        = 2;
public final static int  PER_50        = 3;
public final static int  PER_75        = 4;
public final static int  HORIZ_STRIPE  = 5;
public final static int  VERT_STRIPE   = 6;
public final static int  STRIPE_45     = 7;
public final static int  STRIPE_135    = 8;
public final static int  DIAG_HATCHED  = 9;
public final static int  CROSS_HATCHED = 10;
public final static int  CUSTOM_FILL   = 11;  // Fill with images set by user
public final static int  CUSTOM_STACK  = 12;  // Stack images set by user
public final static int  CUSTOM_PAINT  = 13;  // Fill with TexturePaint
                                              // object set by user

// fill pattern definitions
public final static String[] PER_25_PIXELS = {
	"C C C C C C C C C C C C C C C C ",
	"                                ",
	"C C C C C C C C C C C C C C C C ",
	"                                ",
	"C C C C C C C C C C C C C C C C ",
	"                                ",
	"C C C C C C C C C C C C C C C C ",
	"                                ",
	"C C C C C C C C C C C C C C C C ",
	"                                ",
	"C C C C C C C C C C C C C C C C ",
	"                                ",
	"C C C C C C C C C C C C C C C C ",
	"                                ",
	"C C C C C C C C C C C C C C C C ",
	"                                "
	};
public final static String[] PER_50_PIXELS = {
	"C C C C C C C C C C C C C C C C ",
	" C C C C C C C C C C C C C C C C",
	"C C C C C C C C C C C C C C C C ",
	" C C C C C C C C C C C C C C C C",
	"C C C C C C C C C C C C C C C C ",
	" C C C C C C C C C C C C C C C C",
	"C C C C C C C C C C C C C C C C ",
	" C C C C C C C C C C C C C C C C",
	"C C C C C C C C C C C C C C C C ",
	" C C C C C C C C C C C C C C C C",
	"C C C C C C C C C C C C C C C C ",
	" C C C C C C C C C C C C C C C C",
	"C C C C C C C C C C C C C C C C ",
	" C C C C C C C C C C C C C C C C",
	"C C C C C C C C C C C C C C C C ",
	" C C C C C C C C C C C C C C C C",
	};
public final static String[] PER_75_PIXELS = {
    "CCC CCC CCC CCC CCC CCC CCC CCC ",
    "C CCC CCC CCC CCC CCC CCC CCC CC",
    "CCC CCC CCC CCC CCC CCC CCC CCC ",
    "C CCC CCC CCC CCC CCC CCC CCC CC",
    "CCC CCC CCC CCC CCC CCC CCC CCC ",
    "C CCC CCC CCC CCC CCC CCC CCC CC",
    "CCC CCC CCC CCC CCC CCC CCC CCC ",
    "C CCC CCC CCC CCC CCC CCC CCC CC",
    "CCC CCC CCC CCC CCC CCC CCC CCC ",
    "C CCC CCC CCC CCC CCC CCC CCC CC",
    "CCC CCC CCC CCC CCC CCC CCC CCC ",
    "C CCC CCC CCC CCC CCC CCC CCC CC",
    "CCC CCC CCC CCC CCC CCC CCC CCC ",
    "C CCC CCC CCC CCC CCC CCC CCC CC",
    "CCC CCC CCC CCC CCC CCC CCC CCC ",
    "C CCC CCC CCC CCC CCC CCC CCC CC",
    };
public final static String[] HORIZ_STRIPE_PIXELS = {
    "                                ",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "                                ",
    "                                ",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "                                ",
    "                                ",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "                                ",
    "                                ",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "                                ",
    };
public final static String[] VERT_STRIPE_PIXELS = {
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    };
public final static String[] STRIPE_45_PIXELS = {
    "   CC      CC      CC      CC   ",
    "  CC      CC      CC      CC    ",
    " CC      CC      CC      CC     ",
    "CC      CC      CC      CC      ",
    "C      CC      CC      CC      C",
    "      CC      CC      CC      CC",
    "     CC      CC      CC      CC ",
    "    CC      CC      CC      CC  ",
    "   CC      CC      CC      CC   ",
    "  CC      CC      CC      CC    ",
    " CC      CC      CC      CC     ",
    "CC      CC      CC      CC      ",
    "C      CC      CC      CC      C",
    "      CC      CC      CC      CC",
    "     CC      CC      CC      CC ",
    "    CC      CC      CC      CC  ",
    };
public final static String[] STRIPE_135_PIXELS = {
    "C      CC      CC      CC      C",
    "CC      CC      CC      CC      ",
    " CC      CC      CC      CC     ",
    "  CC      CC      CC      CC    ",
    "   CC      CC      CC      CC   ",
    "    CC      CC      CC      CC  ",
    "     CC      CC      CC      CC ",
    "      CC      CC      CC      CC",
    "C      CC      CC      CC      C",
    "CC      CC      CC      CC      ",
    " CC      CC      CC      CC     ",
    "  CC      CC      CC      CC    ",
    "   CC      CC      CC      CC   ",
    "    CC      CC      CC      CC  ",
    "     CC      CC      CC      CC ",
    "      CC      CC      CC      CC",
    };
public final static String[] DIAG_HATCHED_PIXELS = {
    "CC      CC      CC      CC      ",
    "CCC    CCCC    CCCC    CCCC    C",
    "  CC  CC  CC  CC  CC  CC  CC  CC",
    "   CCCC    CCCC    CCCC    CCCC ",
    "    CC      CC      CC      CC  ",
    "   CCCC    CCCC    CCCC    CCCC ",
    "  CC  CC  CC  CC  CC  CC  CC  CC",
    "CCC    CCCC    CCCC    CCCC    C",
    "CC      CC      CC      CC      ",
    "CCC    CCCC    CCCC    CCCC    C",
    "  CC  CC  CC  CC  CC  CC  CC  CC",
    "   CCCC    CCCC    CCCC    CCCC ",
    "    CC      CC      CC      CC  ",
    "   CCCC    CCCC    CCCC    CCCC ",
    "  CC  CC  CC  CC  CC  CC  CC  CC",
    "CCC    CCCC    CCCC    CCCC    C",
    };
public final static String[] CROSS_HATCHED_PIXELS = {
    "CC      CC      CC      CC      ",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CC      CC      CC      CC      ",
    "CC      CC      CC      CC      ",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CC      CC      CC      CC      ",
    "CC      CC      CC      CC      ",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CC      CC      CC      CC      ",
    "CC      CC      CC      CC      ",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CC      CC      CC      CC      "
};

/**
 * Constructor for fill style objects.
 * @param c fill color
 * @param p fill pattern.  One of <code>NONE</code>, <code>SOLID</code>,
 * <code>PER_25</code>, <code>PER_50</code>, <code>PER_75</code>,
 * <code>HORIZ_STRIPE</code>, <code>VERT_STRIPE</code>, <code>STRIPE_45</code>,
 * <code>STRIPE_135</code>, <code>DIAG_HATCHED</code>, or
 * <code>CROSS_HATCHED</code>.
 */
public JCFillStyle(Color c, int p)
{
	color = c;
	pattern = p;
	buffImage = makeBufferedImage();
	fillPaint = null;
	hiFillPaint = null;
	shFillPaint = null;
}

/**
 * Creates a default fill style.
 * @return <code>JCFillStyle</code> object with default color and fill
 * pattern. The defaults are cycled, so consecutive calls
 * will return different fill styles.
 * @param c <code>JCChart</code> instance
 */
public static JCFillStyle makeDefault(JCChart c)
{
	return(makeDefault(c, false));
}

/**
 * Creates a default fill style.
 * @param c <code>JCChart</code> instance
 * @param other <code>true</code> if calculating the other style for pie charts;
 * <code>false</code> otherwise
 * @return <code>JCFillStyle</code> object with default color and fill
 * pattern. The defaults are cycled, so consecutive calls
 * will return different fill styles.
 */
public static JCFillStyle makeDefault(JCChart c, boolean other)
{
	JCFillStyle style = new JCFillStyle(getDefaultColor(c, other),
                                        getDefaultPattern(c));
	int index = 0;
	if (c != null) {
		index = c.getFillColorIndex();
		if (index == 0) {
			index = defaultColors.length - 1;
        }
		else {
			index--;
        }
	}
	style.colorIndex = index;
	return(style);
}

/**
 * Updates the <code>Graphics</code> object with the elements of this line style
 * (paint fills and colors), with the patterned fills being centered around the
 * anchor rectangle that may have been set.
 * @param gc <code>Graphics</code> object to change
 * @return <code>true</code> if you should now use this object to draw;
 * <code>false</code> if the style is such that no drawing should occur
 */
public boolean updateGraphics(Graphics gc)
{
    if (pattern == NONE) {
        return(false);
    }

    Graphics2D gc2D = (Graphics2D) gc;

	// Set the backup fillstyle color on the gc before setting the paint.
	// This is not needed for on-screen chart usage, but it is needed
	// when chart is used in enivironments (such as PageLayout/ServerReport)
	// that do not currently support Paint objects.
    if (!gc.getColor().equals(color)) {
        gc.setColor(color);
    }

	// Make the fill paint object for this fill style wrt to the given
	// rectangle. If the pattern is SOLID, cache the fill paint object
	// for reuse. If the pattern is CUSTOM_PAINT, use the user-provided
	// paint object instead.
	if (pattern != CUSTOM_STACK || image == null) {
		if (pattern == CUSTOM_PAINT) {
			if (customPaint != null) {
				fillPaint = makeFillPaint(customPaint.getImage(), color,
										  anchorRect, orientation);
			}
            else {
				fillPaint = color;
			}
		}
        else if (pattern != SOLID || fillPaint == null) {
			fillPaint = makeFillPaint(buffImage, color, anchorRect,
                                      orientation);
		}
		gc2D.setPaint(fillPaint);
	}
	return(true);
}

/**
 * Updates the <code>Graphics</code> object with the shadow elements of this
 * line style (paint fills and colors) centered around the anchor rectangle
 * that may have been set.
 * @param gc <code>Graphics</code> object to change
 * @return <code>true</code> if you should now use this object to draw;
 * <code>false</code> if the style is such that no drawing should occur
 */
public boolean updateGraphicsToShadow(Graphics gc)
{
    if (pattern == NONE) {
        return(false);
    }

	Color shadowColor = JCChartUtil.darker(color);
    Graphics2D gc2D = (Graphics2D) gc;
	// Make the fill paint object for this fill style wrt to the given
	// rectangle. If the pattern is SOLID, cache the fill paint object
	// for reuse. If the pattern is CUSTOM_PAINT, use the user-provided
	// paint object instead.
	if (pattern != CUSTOM_STACK || image == null) {
		if (pattern == CUSTOM_PAINT) {
			if (customPaint != null) {
				shFillPaint = makeFillPaint(customPaint.getImage(), shadowColor,
											anchorRect, orientation);
			}
            else {
				shFillPaint = shadowColor;
			}
		}
        else if (pattern != SOLID || shFillPaint == null) {
			if (pattern == CUSTOM_FILL) {
				shFillPaint = makeFillPaint(buffImage, shadowColor, anchorRect,
                                            orientation);
			}
            else {
				shFillPaint = makeFillPaint(
					makeBufferedImage(shadowColor), shadowColor, anchorRect,
                    orientation);
			}
		}
		gc2D.setPaint(shFillPaint);
	}
	return(true);
}

/**
 * Updates the <code>Graphics</code> object with the highlight elements of this
 * line style (paint fills and colors) centered around the anchor rectangle
 * that may have been set.
 * @param gc a<code>Graphics</code> object to change
 * @return <code>true</code> if you should now use this object to draw;
 * <code>false</code> if the style is such that no drawing should occur
 */
public boolean updateGraphicsToHighlight(Graphics gc)
{
    if (pattern == NONE) {
        return(false);
    }

	Color highlightColor = JCChartUtil.brighter(color);
    Graphics2D gc2D = (Graphics2D) gc;
	// Make the fill paint object for this fill style wrt to the given
	// rectangle. If the pattern is SOLID, cache the fill paint object
	// for reuse. If the pattern is CUSTOM_PAINT, use the user-provided
	// paint object instead.
	if (pattern != CUSTOM_STACK || image == null) {
		if (pattern == CUSTOM_PAINT) {
			if (customPaint != null) {
				hiFillPaint = makeFillPaint(customPaint.getImage(),
							                highlightColor, anchorRect,
                                            orientation);
			}
            else {
				hiFillPaint = highlightColor;
			}
		}
        else if (pattern != SOLID || hiFillPaint == null) {
			if (pattern == CUSTOM_FILL) {
				hiFillPaint = makeFillPaint(buffImage, highlightColor,
                                            anchorRect, orientation);
			}
            else {
				hiFillPaint = makeFillPaint(makeBufferedImage(highlightColor),
                                            highlightColor, anchorRect,
                                            orientation);
			}
		}
		gc2D.setPaint(hiFillPaint);
	}
	return(true);
}

/**
 * Resets the graphics object to a default drawing style.
 * @param gc <code>Graphics</code> object to reset
 */
public void resetGraphics(Graphics gc)
{
    Graphics2D gc2D = (Graphics2D) gc;
    gc2D.setColor(color);
}

/**
 * Updates the fill orientation properties for a pending fill operation.
 * These properties are calculated by the drawable and set here so that they
 * don't need to passed into the fill methods, but are not truly properties
 * related to the fill style.
 * @param rect
 * @param newOrient
 */
public void updateFillOrientation(Rectangle rect, int newOrient)
{
	anchorRect = rect;
	orientation = newOrient;
}

/**
 *  Resets the fill orientation properties before the next fill operation.
 */
public void resetFillOrientation()
{
	anchorRect = null;
	orientation = NONE;
}

/**
 * Performs the requested fill given the values of the properties
 * of the <code>JCFillStyle</code> instance.
 * @param gc graphics context to use for drawing
 * @param x x origin of fill region (in pixels)
 * @param y y origin of fill region (in pixels)
 * @param width width of fill region (in pixels)
 * @param height height of fill region (in pixels)
 */
public void fillRect(java.awt.Graphics gc,int x,int y,int width,int height)
{
	if (updateGraphics(gc)) {
		gc.fillRect(x, y, width, height);
		resetGraphics(gc);
	}
}

/**
 * Performs the requested fill with an image given the values of the properties
 * of the <code>JCFillStyle</code> instance.
 * @param gc graphics context to use for drawing
 * @param x x origin of fill region (in pixels)
 * @param y y origin of fill region (in pixels)
 * @param width width of fill region (in pixels)
 * @param height height of fill region (in pixels)
 * @param positive true if the fill is occurring above the axis.
 * @param inverted true if the axes are inverted.
 */
public void fillImageRect(java.awt.Graphics gc,int x,int y,int width,
						  int height, boolean positive, boolean inverted)
{
	if (image == null) {
        return;
    }

	int iHeight = image.getHeight(null);
	int iWidth  = image.getWidth(null);
	if (iHeight <= 0 || iWidth <= 0) {
		return;
    }

	Graphics tempGC = gc.create();
	tempGC.clipRect(x, y, width, height);
	if (!inverted) {
		int draw_x = x + ((width - iWidth) / 2);
		if (positive) {
			int draw_y = y + height - iHeight;
			while (draw_y > y) {
				tempGC.drawImage(image, draw_x, draw_y, null);
				draw_y -= iHeight;
			}
			tempGC.drawImage(image, draw_x, draw_y, null);
		}
        else {
			int draw_y = y + 1;
			while (draw_y < y + height) {
				tempGC.drawImage(image, draw_x, draw_y, null);
				draw_y += iHeight;
			}
		}
	}
    else {
		int draw_y = y + ((height - iHeight) / 2);
		if (positive) {
			int draw_x = x + 1;
			while (draw_x < x + width) {
				tempGC.drawImage(image, draw_x, draw_y, null);
				draw_x += iWidth;
			}
		}
        else {
			int draw_x = x + width - iWidth;
			while (draw_x > x) {
				tempGC.drawImage(image, draw_x, draw_y, null);
				draw_x -= iWidth;
			}
			tempGC.drawImage(image, draw_x, draw_y, null);
		}
	}
	tempGC.dispose();
}

/**
 * Performs the requested fill with a drawn outline given the values of
 * the properties of the <code>JCFillStyle</code> instance.
 * @param gc graphics context to use for drawing
 * @param x x origin of fill region (in pixels)
 * @param y y origin of fill region (in pixels)
 * @param width width of fill region (in pixels)
 * @param height height of fill region (in pixels)
 */
public void fillOutlineRect(java.awt.Graphics gc,int x,int y,
											int width, int height)
{
	fillOutlineRect(gc, x, y, width, height, (Color)null);
}

/**
 * Performs the requested fill with a drawn outline given the values of
 * the properties of the <code>JCFillStyle</code> instance.
 * @param gc graphics context to use for drawing
 * @param x x origin of fill region (in pixels)
 * @param y y origin of fill region (in pixels)
 * @param width width of fill region (in pixels)
 * @param height height of fill region (in pixels)
 * @param localColor color of outline. If null, color will be calculated from
 * fill color.
 */
public void fillOutlineRect(java.awt.Graphics gc, int x, int y,
		int width, int height, Color localColor)
{
	if (localColor == null) {
		localColor = calcOutlineColor();
	}
	fillOutlineRect(gc, x, y, width, height,
					new JCLineStyle(1, localColor, JCLineStyle.SOLID));
}

/**
 * Performs the requested fill with a drawn outline given the values of
 * the properties of the <code>JCFillStyle</code> instance.
 * @param gc graphics context to use for drawing
 * @param x x origin of fill region (in pixels)
 * @param y y origin of fill region (in pixels)
 * @param width width of fill region (in pixels)
 * @param height height of fill region (in pixels)
 * @param lstyle the outline style
 */
public void fillOutlineRect(java.awt.Graphics gc, int x, int y,
							int width, int height, JCLineStyle lstyle)
{
	fillRect(gc, x, y, width, height);
	drawOutlineRect(gc, x, y, width, height, lstyle);
}

/**
 * Draws an outline around the specified rectangle.
 * @param gc graphics context to use for drawing
 * @param x x origin of rectangle (in pixels)
 * @param y y origin of outline (in pixels)
 * @param width width of outline (in pixels)
 * @param height height of outline (in pixels)
 * @param localColor color of outline. If null, color will be calculated from
 * fill color.
 */
public void drawOutlineRect(java.awt.Graphics gc, int x, int y,
							int width, int height, Color localColor)
{
	if (localColor == null) {
		localColor = calcOutlineColor();
	}
	drawOutlineRect(gc, x, y, width, height,
					new JCLineStyle(1, localColor, JCLineStyle.SOLID));
}

/**
 * Draws an outline around the specified rectangle.
 * @param gc graphics context to use for drawing
 * @param x x origin of rectangle (in pixels)
 * @param y y origin of outline (in pixels)
 * @param width width of outline (in pixels)
 * @param height height of outline (in pixels)
 * @param lstyle the outline style
 */
public void drawOutlineRect(java.awt.Graphics gc, int x, int y,
							int width, int height, JCLineStyle lstyle)
{
	if (lstyle != null) {
		lstyle.drawRect(gc, x, y, width, height);
	}
}

/**
 * Performs the requested fill given the values of the properties
 * of the <code>JCFillStyle</code> instance.
 * @param gc graphics context to use for drawing
 * @param poly polygon to fill
 */
public void fillPolygon(java.awt.Graphics gc, Polygon poly)
{
	if (updateGraphics(gc)) {
		gc.fillPolygon(poly);
		resetGraphics(gc);
	}
}

/**
 * Performs the requested fill given the values of the properties
 * of the <code>JCFillStyle</code> instance.
 * @param gc graphics context to use for drawing
 * @param shape shape to fill
 */
public void fillPolygon(java.awt.Graphics gc, Shape shape)
{
	if (updateGraphics(gc)) {
		((Graphics2D)gc).fill(shape);
		resetGraphics(gc);
	}
}

/**
 * Performs the requested fill with a drawn outline given the values of the
 * properties of the <code>JCFillStyle</code> instance.
 * @param gc graphics context to use for drawing
 * @param poly polygon to fill
 * @param localColor color of outline. If null, color will be calculated from
 * fill color.
 */
public void fillOutlinePolygon(java.awt.Graphics gc,
							   java.awt.Polygon poly, Color localColor)
{
	if (localColor == null) {
		localColor = calcOutlineColor();
	}
	fillOutlinePolygon(gc, poly, new JCLineStyle(1, localColor,
                                                 JCLineStyle.SOLID));
}

/**
 * Performs the requested fill with a drawn outline given the values of the
 * properties of the <code>JCFillStyle</code> instance.
 * @param gc graphics context to use for drawing
 * @param poly polygon to fill
 * @param lstyle the outline style
 */
public void fillOutlinePolygon(java.awt.Graphics gc, java.awt.Polygon poly,
							   JCLineStyle lstyle)
{
	fillPolygon(gc, poly);
	drawOutlinePolygon(gc, poly, lstyle);
}

/**
 * Draws an outline around the given polygon.
 * @param gc graphics context to use for drawing
 * @param poly polygon to outline
 * @param localColor color of outline. If null, color will be calculated from
 * fill color.
 */
public void drawOutlinePolygon(java.awt.Graphics gc,
							   java.awt.Polygon poly, Color localColor)
{
	if (localColor == null) {
		localColor = calcOutlineColor();
	}
	drawOutlinePolygon(gc, poly, new JCLineStyle(1, localColor,
                                                 JCLineStyle.SOLID));
}

/**
 * Draws an outline around the given polygon.
 * @param gc graphics context to use for drawing
 * @param poly polygon to outline
 * @param lstyle the outline style
 */
public void drawOutlinePolygon(java.awt.Graphics gc, java.awt.Polygon poly,
								JCLineStyle lstyle)
{
	drawOutlineLine(gc, poly, 0, 0, lstyle);
}

/**
 * Draws a portion of the outline around the given polygon.
 * @param gc graphics context to use for drawing
 * @param poly polygon to outline
 * @param start the index of the point within the polygon from which to begin
 * drawing
 * @param end the index of the point within the polygon from which to stop
 * drawing
 * @param localColor color of outline. If null, color will be calculated from
 * fill color.
 */
public void drawOutlineLine(java.awt.Graphics gc, java.awt.Polygon poly,
                            int start, int end, Color localColor)
{
	if (localColor == null) {
		localColor = calcOutlineColor();
	}
	drawOutlineLine(gc, poly, start, end,
					new JCLineStyle(1, localColor, JCLineStyle.SOLID));
}

/**
 * Draws a portion of the outline around the given polygon.
 * @param gc graphics context to use for drawing
 * @param poly polygon to outline
 * @param start the index of the point within the polygon from which to begin drawing
 * @param end the index of the point within the polygon from which to stop drawing.
 * To outline the polygon completely, pass the same value for both <code>start</code>
 * and <code>end</code>.
 * @param lstyle the outline style
 */
public void drawOutlineLine(java.awt.Graphics gc, java.awt.Polygon poly,
							int start, int end, JCLineStyle lstyle)
{
	if (lstyle == null) {
		return;
	}
	if (end <= start) {
		for (int i = start; i < poly.npoints-1; i++) {
			lstyle.drawLine(gc, (double)poly.xpoints[i],
								(double)poly.ypoints[i],
								(double)poly.xpoints[i+1],
								(double)poly.ypoints[i+1]);
		}

		lstyle.drawLine(gc, (double)poly.xpoints[poly.npoints-1],
							(double)poly.ypoints[poly.npoints-1],
							(double)poly.xpoints[0],
							(double)poly.ypoints[0]);

		for (int i = 0; i < end; i++) {
			lstyle.drawLine(gc, (double)poly.xpoints[i],
								(double)poly.ypoints[i],
								(double)poly.xpoints[i+1],
								(double)poly.ypoints[i+1]);
		}
	}
	else {
		for (int i = start; (i < end) && (i < poly.npoints-1); i++) {
			lstyle.drawLine(gc, (double)poly.xpoints[i],
								(double)poly.ypoints[i],
								(double)poly.xpoints[i+1],
								(double)poly.ypoints[i+1]);
		}
	}
}

/**
 * Performs the requested fill for a partial polygon with a drawn outline,
 * given the values of the * properties of the <code>JCFillStyle</code> instance. A
 * partial polygon is a polygon that is not closed. A line between the
 * first and last points will be drawn in the fill color. The line outlining
 * the rest of the polygon will be drawn in the color specified by <code>localColor</code>.
 * @param gc graphics context to use for drawing
 * @param poly polygon to fill
 * @param localColor color of outline. If null, color will be calculated from
 * fill color.
 */
public void fillOutlinePartialPolygon(java.awt.Graphics gc,
							          java.awt.Polygon poly, Color localColor)
{
	if (localColor == null) {
		localColor = calcOutlineColor();
	}
	fillOutlinePartialPolygon(gc, poly, new JCLineStyle(1, localColor,
                                                        JCLineStyle.SOLID));
}

/**
 * Performs the requested fill for a partial polygon with a drawn outline,
 * given the values of the * properties of the <code>JCFillStyle</code> instance. A
 * partial polygon is a polygon that is not closed. A solid line between the
 * first and last points will be drawn in the fill color. The line outlining
 * the rest of the polygon will be drawn in the specified line style.
 * @param gc graphics context to use for drawing
 * @param poly polygon to fill
 * @param lstyle the outline style
 */
public void fillOutlinePartialPolygon(java.awt.Graphics gc,
							   java.awt.Polygon poly, JCLineStyle lstyle)
{
	fillPolygon(gc, poly);

	if (lstyle == null) {
		return;
	}

	// Fudge the open side of the polygon with a line of the fill color
	Color lsColor = lstyle.getColor();
	int lsPattern = lstyle.getPattern();
	Color gcColor = gc.getColor();
	if (!lsColor.equals(gcColor)) {
		lstyle.setColor(gcColor);
	}

	lstyle.setPattern(JCLineStyle.SOLID);

	lstyle.drawLine(gc, (double)poly.xpoints[0],
						(double)poly.ypoints[0],
						(double)poly.xpoints[poly.npoints - 1],
						(double)poly.ypoints[poly.npoints - 1]);

	// Restore the line style pattern
	lstyle.setPattern(lsPattern);

	// Restore the line style color
	if (!lsColor.equals(gcColor)) {
		lstyle.setColor(lsColor);
	}

	// Draw the outline using the lstyle color, width, and pattern
	for (int i = 0; i < poly.npoints - 1; i++) {
		lstyle.drawLine(gc, (double)poly.xpoints[i],
							(double)poly.ypoints[i],
							(double)poly.xpoints[i + 1],
							(double)poly.ypoints[i + 1]);
	}
}

/**
 * Performs the requested fill in a color darker than that specified in the
 * <code>JCFillStyle</code> instance.
 * @param gc graphics context to use for drawing
 * @param poly polygon to fill
 */
public void fillPolygonShadow(java.awt.Graphics gc, Polygon poly)
{
	if (updateGraphicsToShadow(gc)) {
		gc.fillPolygon(poly);
		resetGraphics(gc);
	}
}

public void fillPolygonShadow(java.awt.Graphics gc, Shape shape)
{
	if (updateGraphicsToShadow(gc)) {
		((Graphics2D)gc).fill(shape);
		resetGraphics(gc);
	}
}

/**
 * Performs the requested fill in a color brighter than that specified in the
 * <code>JCFillStyle</code> instance.
 * @param gc graphics context to use for drawing
 * @param poly polygon to fill
 */
public void fillPolygonHighlight(java.awt.Graphics gc, Polygon poly)
{
	if (updateGraphicsToHighlight(gc)) {
		gc.fillPolygon(poly);
		resetGraphics(gc);
	}
}

/**
 * Performs the requested fill in a color darker than that specified in the
 * <code>JCFillStyle</code> instance, including a drawn outline.
 * @param gc graphics context to use for drawing
 * @param poly polygon to fill
 */
public void fillOutlinePolygonShadow(java.awt.Graphics gc, Polygon poly)
{
	fillOutlinePolygonShadow(gc, poly, null);
}

/**
 * Performs the requested fill in a color darker than that specified in the
 * <code>JCFillStyle</code> instance, including a drawn outline.
 * @param gc graphics context to use for drawing
 * @param poly polygon to fill
 * @param localColor color of outline. If null, color will be calculated from
 * fill color.
 */
public void fillOutlinePolygonShadow(java.awt.Graphics gc, Polygon poly,
									 Color localColor)
{
	fillPolygonShadow(gc, poly);
	drawOutlinePolygon(gc, poly, localColor);
}

/**
 * Performs the requested fill in a color brighter than that specified in the
 * <code>JCFillStyle</code> instance, including a drawn outline.
 * @param gc graphics context to use for drawing
 * @param poly polygon to fill
 */
public void fillOutlinePolygonHighlight(java.awt.Graphics gc, Polygon poly)
{
	fillOutlinePolygonHighlight(gc, poly, null);
}

/**
 * Performs the requested fill in a color brighter than that specified in the
 * <code>JCFillStyle</code> instance, including a drawn outline.
 * @param gc graphics context to use for drawing
 * @param poly polygon to fill
 * @param localColor color of outline. If null, color will be calculated from
 * fill color.
 */
public void fillOutlinePolygonHighlight(java.awt.Graphics gc, Polygon poly,
										Color localColor)
{
	fillPolygonHighlight(gc, poly);
	drawOutlinePolygon(gc, poly, localColor);
}

Color calcOutlineColor()
{
	Color localColor = JCChartUtil.darker(color);
	if (localColor.equals(color)) {
		localColor = JCChartUtil.brighter(color);
	}
	return(localColor);
}

/**
 * Performs the requested fill given the values of the properties
 * of the <code>JCFillStyle</code> instance.
 * @param gc graphics context to use for drawing
 * @param x x origin of fill region (in pixels)
 * @param y y origin of fill region (in pixels)
 * @param width width of fill region (in pixels)
 * @param height height of fill region (in pixels)
 * @param startAngle start angle of the arc (in degrees)
 * @param arcAngle size of arc to fill (in degrees)
 */
public void fillArc(java.awt.Graphics gc,int x,int y,int width,int height,
					int startAngle, int arcAngle)
{
	if (updateGraphics(gc)) {
		gc.fillArc(x, y, width + fillCorrection,
				   height + fillCorrection, startAngle, arcAngle);
		resetGraphics(gc);
	}
}

/**
 * Performs the requested fill in a color darker than that specified in the
 * <code>JCFillStyle</code> instance.
 * @param gc graphics context to use for drawing
 * @param x x origin of fill region (in pixels)
 * @param y y origin of fill region (in pixels)
 * @param width width of fill region (in pixels)
 * @param height height of fill region (in pixels)
 * @param startAngle start angle of the arc (in degrees)
 * @param arcAngle size of arc to fill (in degrees)
 */
public void fillArcShadow(java.awt.Graphics gc,int x,int y, int width,
						  int height,	int startAngle, int arcAngle)
{
	if (updateGraphicsToShadow(gc)) {
		gc.fillArc(x, y, width + fillCorrection,
				   height + fillCorrection, startAngle, arcAngle);
		resetGraphics(gc);
	}
}

public void fillArc(java.awt.Graphics gc, double x, double y,
		double width, double height, double startAngle, double arcAngle)
{
	if (updateGraphics(gc)) {
		((Graphics2D)gc).fill( new Arc2D.Double(x, y, width, height,
									startAngle, arcAngle, Arc2D.PIE) );
		resetGraphics(gc);
	}
}

public void fillArcShadow(java.awt.Graphics gc, double x, double y,
                          double width, double height, double startAngle,
                          double arcAngle)
{
	if (updateGraphicsToShadow(gc)) {
		((Graphics2D)gc).fill( new Arc2D.Double(x, y, width, height,
										startAngle, arcAngle, Arc2D.PIE) );
		resetGraphics(gc);
	}
}


/**
 * Performs the requested draw circle operation given the properties
 * of the <code>JCFillStyle</code> instance.
 *
 * @param gc graphics context to use for drawing
 * @param xCenter x coordinate of center (in pixels)
 * @param yCenter y coordinate of center (in pixels)
 * @param radius radius of circle (in pixels)
 */
public void fillCircle(java.awt.Graphics gc, int xCenter,
						int yCenter, int radius)
{
	if (updateGraphics(gc)) {
		int diameter = 2 * radius + fillCorrection;
		gc.fillArc(xCenter - radius, yCenter - radius,
					diameter, diameter, 0, 360);
		resetGraphics(gc);
	}
}

public void setFillCorrection(int nPixels)
{
	fillCorrection = nPixels;
}

public int getFillCorrection()
{
	return(fillCorrection);
}

/**
 * Gets the <code>Color</code> property of <code>JCFillStyle</code>, which determines the
 * color used to paint the fill region.
 * @return AWT Color class representing the color to be used
 * to paint the fill region. If null, the current color of the <code>Graphics</code>
 * object is used to paint the fill.
 */
public Color getColor()
{
    return(color);
}

/**
 * Sets the <code>Color</code> property of <code>JCFillStyle</code>, which determines the
 * color used to paint the fill region.
 * @param clr AWT Color class representing the color to be used
 * to paint the fill region. If null, the current color of the <code>Graphics</code>
 * object is used to paint the fill.
 */
public void setColor(Color clr)
{
	color = clr;
	if (pattern != CUSTOM_FILL  &&
        pattern != CUSTOM_STACK &&
		pattern != CUSTOM_PAINT)
    {
		buffImage = makeBufferedImage();
		fillPaint = null;
		hiFillPaint = null;
		shFillPaint = null;
	}

	setChanged(true, Changeable.REDRAW | Changeable.NOTIFY_LEGEND_REDRAW);
}

/**
 * Gets the <code>Background</code> property of <code>JCFillStyle</code>, which determines the
 * background color used in painting patterned fills. If null, the
 * chart's background color will be used.
 * @return AWT Color class representing the color to be used for the
 * background when painting patterned fills.
 */
public Color getBackground()
{
    return(backgroundColor);
}

/**
 * Sets the <code>Background</code> property of <code>JCFillStyle</code>, which determines the
 * background color used in painting patterned fills. If null, the
 * chart's background color will be used.
 * @param clr AWT Color class representing the color to be used for the
 * background when painting patterned fills.
 */
public void setBackground(Color clr)
{
	backgroundColor = clr;
	if (pattern != CUSTOM_FILL  &&
        pattern != CUSTOM_STACK &&
		pattern != CUSTOM_PAINT)
    {
		buffImage = makeBufferedImage();
		fillPaint = null;
		hiFillPaint = null;
		shFillPaint = null;
	}

	setChanged(true, Changeable.REDRAW | Changeable.NOTIFY_LEGEND_REDRAW);
}

/**
 * Gets the <code>Pattern</code> property of <code>JCFillStyle</code>, which
 * controls the pattern used to fill the specified region.
 * @return fill pattern, which is one of <code>NONE</code>, <code>SOLID</code>,
 * <code>25_PERCENT</code>, <code>50_PERCENT</code>, <code>75_PERCENT</code>,
 * <code>HORIZ_STRIPE</code>, <code>VERT_STRIPE</code>, <code>45_STRIPE</code>,
 * <code>135_STRIPE</code>, <code>DIAG_HATCHED</code>,
 * <code>CROSS_HATCHED</code>, <code>CUSTOM_FILL</code>, or
 * <code>CUSTOM_STACK</code>.
 */
public int getPattern()
{
    return(pattern);
}

/**
 * Sets the <code>Pattern</code> property of <code>JCFillStyle</code>, which
 * controls the pattern used to fill the specified region.
 * @param pat fill pattern, which is one of <code>NONE</code>,
 * <code>SOLID</code>, <code>25_PERCENT</code>, <code>50_PERCENT</code>,
 * <code>75_PERCENT</code>, <code>HORIZ_STRIPE</code>, <code>VERT_STRIPE</code>,
 * <code>45_STRIPE</code>, <code>135_STRIPE</code>, <code>DIAG_HATCHED</code>,
 * <code>CROSS_HATCHED</code>, <code>CUSTOM_FILL</code>, or
 * <code>CUSTOM_STACK</code>. (<code>CUSTOM_STACK</code> is only used by bar
 * charts.)
 * @exception IllegalArgumentException If <code>Pattern</code> is not one of
 * the valid enum values.
 */
public void setPattern(int pat) {

	if (pat == pattern) {
        return;
    }
	switch (pat) {
	case NONE:
	case SOLID:
	case PER_25:
	case PER_50:
	case PER_75:
	case HORIZ_STRIPE:
	case VERT_STRIPE:
	case STRIPE_45:
	case STRIPE_135:
	case DIAG_HATCHED:
	case CROSS_HATCHED:
		pattern = pat;
		buffImage = makeBufferedImage();
		fillPaint = null;
		hiFillPaint = null;
		shFillPaint = null;
		break;
	case CUSTOM_FILL:
		pattern = pat;
		buffImage = makeBufferedImage(image);
		fillPaint = null;
		hiFillPaint = null;
		shFillPaint = null;
		break;
	case CUSTOM_STACK:
		pattern = pat;
		fillPaint = null;
		hiFillPaint = null;
		shFillPaint = null;
		break;
	case CUSTOM_PAINT:
		pattern = pat;
		break;
	default:
		throw new IllegalArgumentException(
		   "Invalid value for Pattern in JCFillStyle.  Must be one of " +
           "NONE, SOLID, PER_25, PER_50, PER_75, HORIZ_STRIPE, VERT_STRIPE, " +
           "STRIPE_45, STRIPE_135, DIAG_HATCHED, CROSS_HATCHED, CUSTOM_FILL, " +
           "or CUSTOM_STACK."
		   );
	}

	setChanged(true, Changeable.REDRAW | Changeable.NOTIFY_LEGEND_RECALC);
}

/**
 * Gets the custom fill paint property that has been set by the user.
 * @return the custom fill paint object that was set by the user. Null if not
 * set.
 */
public TexturePaint getCustomPaint()
{
	return(customPaint);
}

/**
 * Sets a custom fill paint property in <code>JCFillStyle</code>, which controls
 * the pattern used to fill the specified region. This property is used when
 * the pattern has been set to <code>CUSTOM_PAINT</code>.
 * @param paint custom <code>TexturePaint</code> object to use for fill pattern
 */
public void setCustomPaint(TexturePaint paint)
{
	if (paint == customPaint) {
        return;
    }
	buffImage = null;
	customPaint = paint;
	setChanged(true, Changeable.REDRAW | Changeable.NOTIFY_LEGEND_RECALC);
}

/**
 * Gets the <code>Image</code> property.<br> The <code>Image</code> property
 * determines the custom image used to paint the fill region.
 * @return AWT Image class representing the image to be used
 * to paint image fills. If null, no image fill is done.
 */
public Image getImage()
{
    return(image);
}

/**
 * Sets the <code>Image</code> property. The <code>Image</code> property
 * determines the custom image used to paint the fill region.
 * @param img AWT Image class representing the image to be used
 * to paint image fills. If null, no image fill is done.
 */
public void setImage(Image img)
{
	image = img;
	if (image != null) {
		int iHeight = image.getHeight(null);
		int iWidth = image.getWidth(null);

		if (iHeight <= 0 || iWidth <=0) {
			image = null;
        }
	}

	if (pattern == CUSTOM_FILL || pattern == CUSTOM_STACK) {
		buffImage = makeBufferedImage(image);
		fillPaint = null;
		hiFillPaint = null;
		shFillPaint = null;
	}

	setChanged(true, Changeable.REDRAW | Changeable.NOTIFY_LEGEND_RECALC);
}

/**
 * Sets the <code>Image</code> property of <code>JCFillStyle</code>, which
 * determines the image used to paint the fill region. Only bar charts use
 * this property.  Note that the fileName will be discarded after loading.  If
 * you wish to save this information, you can do so by storing it in an
 * <code>OutputProperties</code> object and setting it on this
 * <code>JCFillStyle</code> object.
 * @param fileName AWT Image class representing the image to be used
 * to paint image fills. If null, no image fill is done.
 * @see #getOutputProperties
 * @see #setOutputProperties
 */
public void setImage(String fileName)
{
	Image thisImage = (new ImageIcon(fileName)).getImage();

	setImage(thisImage);
}

/**
 * Get the current object which holds the output properties for saving the image
 * to html or xml.
 * @return The current output properties object
 */
public OutputProperties getOutputProperties()
{
    return(outputProperties);
}

/**
 * Set the current object which holds the output properties for saving the image
 * to html or xml.
 * @param outputProperties The new output properties object
 */
public void setOutputProperties(OutputProperties outputProperties)
{
    this.outputProperties = outputProperties;
}

/**
 * Returns the index used to generate the color. Only set
 * if default colors are used.
 * @return index used to generate the color
 */
public int getColorIndex()
{
	return(colorIndex);
}

static Color getDefaultColor(JCChart c)
{
	return(getDefaultColor(c, false));
}

static boolean nearPieColor(ChartDataView cdv, boolean other, Color color)
{
    if (cdv == null || cdv.getChartType() != JCChart.PIE || other) {
        return(false);
    }
    JCPieChartFormat format = (JCPieChartFormat)cdv.getChartFormat();
    return(JCChartUtil.colorsNear(color,
                                  format.getOtherStyle().getFillColor()));
}

static Color getDefaultColor(JCChart c, boolean other)
{
	Color rval;
	int localIndex = 0;
	if (c == null) {
		rval = defaultColors[0];
	}
	else {
		localIndex = c.getFillColorIndex();
		rval = JCSymbolStyle.defaultColors[localIndex++];
		if (localIndex >= JCSymbolStyle.defaultColors.length) {
			localIndex = 0;
        }
	}

	if (c != null && c.getChartArea() != null) {
        JCChartArea area = c.getChartArea();
		ChartDataView cdv = c.getDataView(0);

		// Don't allow default fill color to be the same color as the Other
		// slice color in a pie chart or the background color of the chart area.
		int startIndex = localIndex;
        Color background = area.getPlotArea().getBackground();
		while (nearPieColor(cdv, other, rval) ||
			   JCChartUtil.colorsNear(rval, background))
        {
			rval = JCSymbolStyle.defaultColors[localIndex++];
			if (localIndex >= JCSymbolStyle.defaultColors.length) {
				localIndex = 0;
            }

			if (localIndex == startIndex) {
				break;
            }
		}
		c.setFillColorIndex(localIndex);
	}
	return(rval);
}

static int getDefaultPattern(JCChart c)
{
	return(SOLID);
}

/**
 * Makes the <code>BufferedImage</code> for future fill paint operations using
 * the color specified in the <code>Color</code> property.
 */
public BufferedImage makeBufferedImage()
{
	return(makeBufferedImage(color));
}

/**
 * Makes the <code>BufferedImage</code> object for fill operations using a
 * specified color.
 */
public BufferedImage makeBufferedImage(Color useFgColor)
{
	if (pattern == NONE || pattern == SOLID) {
		return(null);
    }

	JCIconCreator ic = new JCIconCreator(32, 16);
	ic.setColor('C', useFgColor);

	Color useBgColor = backgroundColor;
	// If no background color set, make background transparent!
	if (useBgColor == null) {
		useBgColor = new Color(0.0f, 0.0f, 0.0f, 0.0f);
	}
	ic.setColor(' ', useBgColor);

	String pixels[] = null;
	switch(pattern) {
	case PER_25:
		pixels = PER_25_PIXELS;
		break;
	case PER_50:
		pixels = PER_50_PIXELS;
		break;
	case PER_75:
		pixels = PER_75_PIXELS;
		break;
	case HORIZ_STRIPE:
		pixels = HORIZ_STRIPE_PIXELS;
		break;
	case VERT_STRIPE:
		pixels = VERT_STRIPE_PIXELS;
		break;
	case STRIPE_45:
		pixels = STRIPE_45_PIXELS;
		break;
	case STRIPE_135:
		pixels = STRIPE_135_PIXELS;
		break;
	case DIAG_HATCHED:
		pixels = DIAG_HATCHED_PIXELS;
		break;
	case CROSS_HATCHED:
		pixels = CROSS_HATCHED_PIXELS;
		break;
	}
	if (pixels == null) {
		return(null);
    }

	ic.setPixels(pixels);
	Image im = ic.getIcon().getImage();
	return(makeBufferedImage(im));
}

/**
 * Makes the <code>BufferedImage</code> object for fill operations using a
 * specified image.
 */
public BufferedImage makeBufferedImage(Image im)
{
	if (im == null) {
        return(null);
    }

	int iw = im.getWidth(null);
	int ih = im.getHeight(null);
	if (iw < 0 || ih < 0) {
        return null;
    }

	BufferedImage bi = new BufferedImage(iw, ih, BufferedImage.TYPE_INT_ARGB);
	Graphics2D biGC = bi.createGraphics();
	biGC.drawImage(im, 0, 0, null);
	return(bi);
}

/**
 * Makes the <code>Paint</code> object for fill operations using the previously
 * created buffered image, the default foreground color, and no anchor
 * rectangle.
 */
public Paint makeFillPaint()
{
	return(makeFillPaint(buffImage, color, null, NONE));
}

/**
 * Makes the <code>Paint</code> object for fill operations using specified
 * buffered image anchored in the center of the provided rectangle.
 * @param useImage <code>BufferedImage</code> object to use for
 * <code>Paint</code> object creation. If null, return default color as
 * <code>Paint</code>.
 * @param useFgColor color to use as default paint if <code>useImage</code> is
 * null, or in the case of a Solid pattern.
 * @param anchor anchor rectangle to center <code>Paint</code> pattern within.
 * If null, default <code>Paint</code> behavior is used and there is no
 * rectangle.
 */
public Paint makeFillPaint(BufferedImage useImage, Color useFgColor,
                           Rectangle anchor, int orientation)
{

    if (pattern == NONE) {
        return(null);
    }

    if (pattern == SOLID) {
        return(useFgColor);
    }

    if (useImage == null) {
        return(useFgColor);
    }

    int iw = useImage.getWidth();
    int ih = useImage.getHeight();
    Rectangle r = null;
    if (anchor != null && orientation != NONE) {
        int hoffset = (anchor.width - iw) / 2;
        int voffset = (anchor.height - ih) / 2;
        switch (orientation) {
        case BOTTOM:
            r = new Rectangle(hoffset + anchor.x, anchor.y + anchor.height - ih,
                              iw, ih);
            break;
        case TOP:
            r = new Rectangle(hoffset + anchor.x, anchor.y, iw, ih);
            break;
        case LEFT:
            r = new Rectangle(anchor.x, voffset + anchor.y, iw, ih);
            break;
        case RIGHT:
            r = new Rectangle(anchor.x + anchor.width - iw, voffset + anchor.y,
                              iw, ih);
            break;
        case CENTER:
            r = new Rectangle(hoffset + anchor.x, voffset + anchor.y, iw, ih);
            break;
        case BOTTOM_LEFT:
            r = new Rectangle(anchor.x, anchor.y + anchor.height - ih, iw, ih);
            break;
        case TOP_LEFT:
            r = new Rectangle(anchor.x, anchor.y, iw, ih);
            break;
        case BOTTOM_RIGHT:
            r = new Rectangle(anchor.x + anchor.width - iw,
                              anchor.y + anchor.height - ih, iw, ih);
            break;
        case TOP_RIGHT:
            r = new Rectangle(anchor.x + anchor.width - iw, anchor.y, iw, ih);
            break;
        default :
            r = null;
        }
    }
    else {
        r = new Rectangle(0, 0, iw, ih);
    }

    return(new TexturePaint(useImage, r));
}

/**
 *  Returns <code>true</code> if this fill style can be used for custom image stacking;
 *  that is, if the pattern is <code>CUSTOM_STACK</code> and an image is present.
 *  Otherwise, returns <code>false</code>.
 * @return <code>true</code> if the fill style can be used for custom image stacking;
 * <code>false</code> otherwise
 */
protected boolean isCustomStackable()
{
	return(pattern == CUSTOM_STACK && image != null);
}

}
