/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCGrid.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.awt.Color;
import java.util.Arrays;

/**
 * This class is used to define a series of grid
 * lines on a JCAxis.
 */
public class JCGrid implements java.io.Serializable {

protected JCAxis            parent              = null;

protected String            name                = null;

/**
 * The <code>startValue</code> is the scale value at which to begin this set of
 * grid lines.
 */
protected DerivedDouble     startValue          = new DerivedDouble(0, true);

/**
 * The <code>stopValue</code> is the scale value at which this set of grid
 * lines ends.
 */
protected DerivedDouble     stopValue           = new DerivedDouble(100, true);

/**
 * If not default, the scale values at which grid lines are to be placed.
 */
protected DerivedDouble     incrementValue      = new DerivedDouble(10, true);

/** Style of grid lines. */
protected JCChartStyle      gridStyle           = new JCChartStyle(
                                            makeDefaultLineStyle(null),
                                            JCFillStyle.makeDefault(null),
                                            JCSymbolStyle.makeDefault(null));

/**
 * Creates a JCGrid object with default properties.
 */
public JCGrid() {
}

/**
 * Creates a JCGrid object with startValue, stopValue, and incrementValue
 * initialized to the min, max, and numSpacing values of the parent axis.
 */
public JCGrid(JCAxis ax) {
    this.parent         = ax;
    this.startValue     = new DerivedDouble(ax.min.value, true);
    this.stopValue      = new DerivedDouble(ax.max.value, true);
    this.incrementValue = new DerivedDouble(ax.numSpacing.value, true);
    setGridStyle(new JCChartStyle(makeDefaultLineStyle(ax),
                          JCFillStyle.makeDefault(null),
                          JCSymbolStyle.makeDefault(null)));
}

/**
 * Creates a grid object and sets its properties.
 *
 * @param startValue where the grid lines are to begin
 * @param stopValue where the grid lines are to end
 * @param incrementValue the grid interval
 * @param gridStyle the grid line style
 */
public JCGrid(JCAxis        parent,
              double        startValue,
              double        stopValue,
              double        incrementValue,
              JCChartStyle  gridStyle)
{
    this.parent         = parent;
    this.startValue     = new DerivedDouble(startValue, false);
    this.stopValue      = new DerivedDouble(stopValue, false);
    this.incrementValue = new DerivedDouble(incrementValue, false);
    this.gridStyle      = gridStyle;
}

/**
 * Returns the parent JCAxis.
 */
public JCAxis getParent() {
    return parent;
}

/**
 * Sets the parent JCAxis.
 */
public void setParent(JCAxis ax) {
    this.parent = ax;
}

/**
 * Returns the name of this grid object.
 */
public String getName() {
    return name;
}

/**
 * Sets the name of this grid object.
 */
public void setName(String name) {
    this.name = name;
}

/**
 * In non-default mode, use this value as the increment value; that
 * is, it places grids at every <code>incrementValue</code> along the scale.
 * In default mode, the calculated increment value will overwrite this value.
 *
 * @return the spacing between adjacent grid lines
 */
public double getIncrementValue() {
    return(incrementValue.value);
}

public boolean getIncrementIsDefault() {
    return(incrementValue.isDefault);
}

/**
 * In non-default mode, use this value as the start value.  In default
 * mode, the calculated start value will overwrite this value.
 *
 * @return the start value
 */
public double getStartValue() {
    return(startValue.value);
}

public boolean getStartValueIsDefault() {
    return(startValue.isDefault);
}

/**
 * In non-default mode, use this value as the stop value.  In default
 * mode, the calculated stop value will overwrite this value.
 *
 * @return the stop value
 */
public double getStopValue() {
    return(stopValue.value);
}

public boolean getStopValueIsDefault() {
    return(stopValue.isDefault);
}

/**
 * Returns the style in which to draw grids.
 *
 * @return the style of the grid lines
 */
public JCChartStyle getGridStyle() {
    return(gridStyle);
}

/**
 * In non-default mode, use this value as the increment value; that
 * is, it places grid lines at every <code>incrementValue</code> along
 * the scale. In default mode, the calculated increment value will
 * overwrite this value.
 *
 * @param incrementValue the spacing between adjacent grid lines
 */
public void setIncrementValue(double incrementValue) {
    this.incrementValue.value = incrementValue;
    this.incrementValue.isDefault = false;
}

public void setIncrementIsDefault(boolean isDefault) {
    this.incrementValue.isDefault = isDefault;
}

/**
 * In non-default mode, use this value as the start value.  In default
 * mode, the calculated start value will overwrite this value.
 *
 * @param startValue the start value
 */
public void setStartValue(double startValue) {
    this.startValue.value = startValue;
    this.startValue.isDefault = false;
}

public void setStartValueIsDefault(boolean isDefault) {
    this.startValue.isDefault = isDefault;
}

/**
 * In non-default mode, use this value as the stop value.  In default
 * mode, the calculated stop value will overwrite this value.
 *
 * @param stopValue the stop value
 */
public void setStopValue(double stopValue) {
    this.stopValue.value = stopValue;
    this.stopValue.isDefault = false;
}

public void setStopValueIsDefault(boolean isDefault) {
    this.stopValue.isDefault = isDefault;
}

/**
 * The style to use to draw grid lines.
 *
 * @param gridStyle the style of the grid lines
 */
public void setGridStyle(JCChartStyle gridStyle) {
    this.gridStyle = gridStyle;
}

public String toString() {
    String str = super.toString();
    str = str + ", name=" + name +
                ", startValue=" + startValue.value +
                ", stopValue=" + stopValue.value +
                ", incrementValue=" + incrementValue.value +
                ", lineStyle.color=" + gridStyle.getLineStyle().getColor() +
                ", lineStyle.width=" + gridStyle.getLineStyle().getWidth() +
                ", lineStyle.pattern=" + gridStyle.getLineStyle().getPattern() +
                ", lineStyle.join=" + gridStyle.getLineStyle().getJoin() +
                ", lineStyle.cap=" + gridStyle.getLineStyle().getCap();
    return str;
}

/**
 * This method creates a JCLineStyle with default attributes for
 * a JCGrid.
 */
public static JCLineStyle makeDefaultLineStyle(JCAxis axis) {
    Color lineColor = (axis != null) ? axis.getForeground() : null;
    return new JCLineStyle(1, lineColor, JCLineStyle.SOLID);
}

/**
 * Returns true if the colors of the JCLineStyle parameters are equal.
 * If one of the colors is null, and the other color is equal to the
 * axis foreground color, then the line style colors will be considered
 * equal.
 */
private static boolean equivalentGridLineColors(
                            JCLineStyle ls1, JCLineStyle ls2, JCAxis ax) {
    Color clr1 = ls1.getColor();
    Color clr2 = ls2.getColor();
    Color foreground = (ax != null) ? ax.getForeground() : null;

    boolean fEqual = false;
    if (clr1 == null) {
        fEqual = (clr2 == null ||
                  (foreground != null && clr2.equals(foreground)));
    }
    else if (clr2 == null) {
        fEqual = (clr1 == null ||
                  (foreground != null && clr1.equals(foreground)));
    }
    else {
        fEqual = clr1.equals(clr2);
    }

    return fEqual;
}

/**
 * Intended for internal use only.
 * Returns true if the JCLineStyle parameters are equal.
 */
public static boolean equivalentGridLineStyles(
                            JCLineStyle ls1, JCLineStyle ls2, JCAxis ax) {
    return (ls1.getJoin() == ls2.getJoin() &&
            ls1.getCap() == ls2.getCap() &&
            ls1.getPattern() == ls2.getPattern() &&
            ls1.getWidth() == ls2.getWidth() &&
            equivalentGridLineColors(ls1, ls2, ax) &&
            Arrays.equals(ls1.getCustomPatternArray(),
                          ls2.getCustomPatternArray()) &&
            Arrays.equals(ls1.getCustomLegendPatternArray(),
                          ls2.getCustomLegendPatternArray()));
}

/**
 * Returns true if all the JCGrid properties have default values.
 */
public boolean isDefault() {
    return (getStartValueIsDefault() &&
            getStopValueIsDefault()  &&
            getIncrementIsDefault()  &&
            equivalentGridLineStyles(getGridStyle().getLineStyle(),
                                     makeDefaultLineStyle(parent),
                                     parent));
}

}
