/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCHLOCChartFormat.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

/**
 * Manages chart properties specific to Hi-Lo-Open-Close charts.
 * <P>Hi-Lo-Open-Close charts take the Y values of <EM>4</EM> data series
 * to construct each "logical" bar.  The number of data points defines the
 * number of bars in each row.
 */

public class JCHLOCChartFormat implements JCChartTypeFormat,
										  java.io.Serializable
{

protected boolean isShowingOpen = true;
protected boolean isShowingClose = true;
protected boolean isOpenCloseFullWidth = false;

protected ChartDataView parent = null;

public JCHLOCChartFormat() {}

public JCHLOCChartFormat(ChartDataView p)
{
    parent = p;
}

public void setParent(ChartDataView cdv)
{
    parent = cdv;
}

public ChartDataView getParent()
{
    return(parent);
}

/**
 * Gets the <code>ShowingOpen</code> property of the Hi-Lo-Open-Close chart.
 * This property determines whether the open tick indication is shown or not.
 * @return <code>true</code> if the open tick is shown; <code>false</code>
 * otherwise
 */
public boolean isShowingOpen()
{
    return(isShowingOpen);
}

/**
 * Sets the <code>ShowingOpen</code> property of the Hi-Lo-Open-Close chart.
 * This property determines whether the open tick indication is shown or not.
 * @param b if <code>true</code>, the open tick is shown
 */
public void setShowingOpen(boolean b)
{
    if (isShowingOpen == b) {
        return;
    }
    isShowingOpen = b;

    if (parent != null) {
        parent.setChanged(true, Changeable.RECALC |
                                Changeable.NOTIFY_LEGEND_RECALC);
    }
}

/**
 * Gets the <code>ShowingClose</code> property of the Hi-Lo-Open-Close chart.
 * This property determines whether the close tick indication is shown or not.
 * @return <code>true</code> if close tick shown; <code>false</code> otherwise
 */
public boolean isShowingClose()
{
    return(isShowingClose);
}

/**
 * Sets the <code>ShowingClose</code> property of the Hi-Lo-Open-Close chart.
 * This property determines whether the close tick indication is shown or not.
 * @param b if <code>true</code>, the close tick is shown
 */
public void setShowingClose(boolean b)
{
    if (isShowingClose == b) {
        return;
    }
    isShowingClose = b;

    if (parent != null) {
        parent.setChanged(true, Changeable.RECALC |
                                Changeable.NOTIFY_LEGEND_RECALC);
    }
}

/**
 * Gets the <code>OpenCloseFullWidth</code> property of the Hi-Lo-Open-Close
 * chart. This property determines whether the open and close tick indications
 * are shown full width from left of the Hi-Lo line to the right of
 * the Hi-Lo line.
 * @return <code>true</code> if full open and close tick indications are shown
 * full width; <code>flase</code> otherwise
 */
public boolean isOpenCloseFullWidth()
{
    return(isOpenCloseFullWidth);
}

/**
 * Sets the <code>OpenCloseFullWidth</code> property of the Hi-Lo-Open-Close
 * chart. This property determines whether the open and close tick indications
 * are shown full width from left of the Hi-Lo line to the right of
 * the Hi-Lo line.
 * @param b if <code>true</code>, full width is shown
 */
public void setOpenCloseFullWidth(boolean b)
{
    if (isOpenCloseFullWidth == b) {
        return;
    }
    isOpenCloseFullWidth = b;

    if (parent != null) {
        parent.setChanged(true, Changeable.RECALC |
                                Changeable.NOTIFY_LEGEND_RECALC);
    }
}

/**
 * Gets the <code>JCChartStyle</code> that controls the style of the
 * Hi-Lo-Open-Close chart.
 * @param index the logical index of the Hi-Lo-Open-Close series
 * @return the <code>JCChartStyle</code> object
 */
public JCChartStyle getHiloStyle(int index)
{
    JCChartStyle cs = null;
    if (parent != null) {
        cs = parent.getChartStyle(index * parent.getNumSeriesPerData());
    }
    return(cs);
}

/**
 * Sets the style of the Hi-Lo-Open-Close chart.
 * @param index the logical index of the Hi-Lo-Open-Close series
 * @param value the <code>JCChartStyle</code> object containing the desired
 * style
 */
public void setHiloStyle(int index, JCChartStyle value)
{
    if (parent != null) {
        parent.setChartStyle(index * parent.getNumSeriesPerData(), value);
    }
}

/**
 * Gets the tick size for the open and close ticks.
 * @param index the logical index of the Hi-Lo-Open-Close series
 * @return the tick size
 */
public int getTickSize(int index)
{
    int size = 0;
    if (parent != null) {
        JCChartStyle cs = parent.getChartStyle(index *
                                               parent.getNumSeriesPerData());
        if (cs != null) {
            size = cs.getSymbolSize();
        }
    }
    return(size);
}

/**
 * Sets the tick size for the open and close ticks.
 * @param index the logical index of the Hi-Lo-Open-Close series
 * @param value the tick size
 */
public void setTickSize(int index, int value)
{
    if (parent != null) {
        JCChartStyle cs = parent.getChartStyle(index *
                                               parent.getNumSeriesPerData());
        if (cs != null) {
            cs.setSymbolSize(value);
        }
    }
}

}


