/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCHiloChartFormat.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

/**
 * Manages chart properties specific to Hi-Lo charts.
 * <P>Hi-Lo charts take the Y values of <EM>2</EM> data series
 * to construct each "logical" bar.  The number of data points defines the
 * number of bars in each row.
 */

public class JCHiloChartFormat implements JCChartTypeFormat, 
								          java.io.Serializable
{

protected ChartDataView parent = null;

public JCHiloChartFormat() {}

public JCHiloChartFormat(ChartDataView p)
{
    parent = p;
}

public void setParent(ChartDataView cdv)
{
    parent = cdv;
}

public ChartDataView getParent()
{
    return(parent);
}

/**
 * Gets the <code>JCChartStyle</code> that controls the style of the
 * Hi-Lo chart.
 * @param index the logical index of the Hi-Lo series
 * @return the <code>JCChartStyle</code> object
 */
public JCChartStyle getHiloStyle(int index)
{
    JCChartStyle cs = null;
    if (parent != null) {
        cs = parent.getChartStyle(index * parent.getNumSeriesPerData());
    }
    return(cs);
}

/**
 * Sets the style of the Hi-Lo chart.
 * @param index the logical index of the Hi-Lo series
 * @param value the <code>JCChartStyle</code> object containing the desired style
 */
public void setHiloStyle(int index, JCChartStyle value)
{
    if (parent != null) {
        parent.setChartStyle(index * parent.getNumSeriesPerData(), value);
    }
}

}


