/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCPieChartFormat.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import com.klg.jclass.util.ImageMapInfo;

import java.awt.*;
import java.util.Locale;

/**
 * An object that manages chart properties specific to pie charts.
 */
public class JCPieChartFormat implements JCChartTypeFormat,
                                         java.io.Serializable
{

protected String          otherLabel = null;
protected JCChartStyle    otherStyle = null;
protected ImageMapInfo[]  otherImageMapInfo = null;
protected ImageMapInfo    otherLegendImageMapInfo = null;

protected int             thresholdMethod = SLICE_CUTOFF;
protected double          thresholdValue = 0.0;
protected int             sortOrder = DATA_ORDER;
protected int             minSlices = 1;
protected Point[]         explodeList;
protected int             explodeOffset = 10;
protected double          startAngle = 135.0;
protected Changeable      parent = null;

public final static int  SLICE_CUTOFF     = 0;
public final static int  PIE_PERCENTILE   = 1;
public final static int  DATA_ORDER       = 0;
public final static int  ASCENDING_ORDER  = 1;
public final static int  DESCENDING_ORDER = 2;

/** Used to specify the other or all slices of a pie chart. */
public final static int  OTHER_SLICE = -10;
public final static int  ALL         = ChartDataModel.ALL;

public JCPieChartFormat() {}

public JCPieChartFormat(Changeable p)
{
    parent = p;
}

public void setParent(ChartDataView cdv)
{
    parent = cdv;
}

/**
 * Gets the <code>ThresholdMethod</code> property of <code>PieChartStyle</code>.
 * This property determines how the <code>ThresholdValue</code> property is used.
 * If the method is <code>SLICE_CUTOFF</code>, the <code>ThresholdValue</code>
 * is used as a cutoff to determine what items are lumped into the other
 * slice. If the method is <code>PIE_PERCENTILE</code>, items are grouped
 * into the other slice until it represents "ThresholdValue" percent of
 * the pie.
 * @return one of <code>SLICE_CUTOFF</code> or <code>PIE_PERCENTILE</code>
 */
public int getThresholdMethod()
{
    return(thresholdMethod);
}

/**
 * Sets the <code>ThresholdMethod</code> property of the
 * <code>PieChartStyle</code>. This property determines how the
 * <code>ThresholdValue</code> property is used.  If the method is
 * <code>SLICE_CUTOFF</code>, the <code>ThresholdValue</code> is used as a
 * cutoff to determine what items are lumped into the other slice. If the method
 * is <code>PIE_PERCENTILE</code>, items are grouped into the other slice until
 * it represents "ThresholdValue" percent of the pie.
 * @param meth one of <code>SLICE_CUTOFF</code> or <code>PIE_PERCENTILE</code>
 * @exception IllegalArgumentException If the value for
 * <code>ThresholdMethod</code> is not one of <code>SLICE_CUTOFF</code> or
 * <code>PIE_PERCENTILE</code>.
 */
public void setThresholdMethod(int meth)
{
    if (meth == thresholdMethod) {
        return;
    }

    switch (meth) {
    case SLICE_CUTOFF:
    case PIE_PERCENTILE:
        thresholdMethod = meth;
        break;
    default:
        throw new IllegalArgumentException("Invalid value for " +
                                           "ThresholdMethod in " +
                                           "JCPieChartFormat.  Must be one " +
                                           "of SLICE_CUTOFF or PIE_PERCENTILE");
    }
    if (parent != null) {
        parent.setChanged(true,
                          Changeable.RECALC | Changeable.NOTIFY_LEGEND_RECALC);
    }
}

/**
 * Gets the <code>ThresholdValue</code> property, which is a percentage value
 * (that is, between 0.0 and 100.0). How this value is used depends on
 * the <code>ThresholdMethod</code> property.
 * @return percentage value between 0.0 and 100.0
 * @see #getThresholdMethod
 */
public double getThresholdValue()
{
    return(thresholdValue);
}

/**
 * Sets the <code>ThresholdValue</code> property, which is a percentage value
 * (that is, between 0.0 and 100.0). How this value is used depends on
 * the <code>ThresholdMethod</code> property.
 * @param per percentage value between 0.0 and 100.0
 * @see #getThresholdMethod
 * @exception IllegalArgumentException If the new threshold value is less
 * than 0.0 or greater than 100.0.
 */
public void setThresholdValue(double per)
{
    if (per == thresholdValue) {
        return;
    }
    if (per < 0.0 || per > 100.0) {
        throw new IllegalArgumentException("ThresholdValue must be between " +
                                           "0.0 and 100.0");
    }

    thresholdValue = per;

    if (parent != null) {
        parent.setChanged(true,
                          Changeable.RECALC | Changeable.NOTIFY_LEGEND_RECALC);
    }
}

/**
 * Gets the <code>SortOrder</code> property, which is used to determine the
 * order in which pie slices will be displayed. Note that the
 * "other" slice is always last in any ordering.
 * @return one of <code>ASCENDING_ORDER</code>, <code>DESCENDING_ORDER</code>,
 * or <code>DATA_ORDER</code>
 */
public int getSortOrder()
{
    return(sortOrder);
}

/**
 * Gets the <code>SortOrder</code> property, which is used to determine the
 * order in which pie slices will be displayed.  Note that the
 * "other" slice is always last in any ordering.
 * @param order one of <code>ASCENDING_ORDER</code>,
 * <code>DESCENDING_ORDER</code>, or <code>DATA_ORDER</code>
 * @exception IllegalArgumentException If the ordering value is not
 * one of <code>ASCENDING_ORDER</code>, <code>DESCENDING_ORDER</code>, or
 * <code>DATA_ORDER</code>.
 */
public void setSortOrder(int order)
{
    if (order == sortOrder) {
        return;
    }
    switch (order) {
    case ASCENDING_ORDER:
    case DESCENDING_ORDER:
    case DATA_ORDER:
        sortOrder = order;
        break;
    default:
        throw new IllegalArgumentException("Invalid value for SortOrder in " +
                                           "JCPieChartFormat.  Must be one " +
                                           "of ASCENDING_ORDER, " +
                                           "DESCENDING_ORDER or DATA_ORDER");
    }

    if (parent != null) {
        parent.setChanged(true, Changeable.RECALC);
    }
}

/**
 * Gets the <code>MinSlices</code> property, which represents the minimum
 * number of pie slices that the chart must display, overriding the
 * placing of series into the "other" slice.
 * @return a positive integer representing the number of slices
 * to attempt to display
 */
public int getMinSlices()
{
    return(minSlices);
}

/**
 * Sets the <code>MinSlices</code> property, which represents the minimum
 * number of pie slices that the chart must display, overriding the
 * the placing of series into the "other" slice.
 * @param min a positive integer representing the number of slices
 * to attempt to display
 */
public void setMinSlices(int min)
{
    if (minSlices == min) {
        return;
    }
    minSlices = min;

    if (parent != null) {
        parent.setChanged(true,
                          Changeable.RECALC | Changeable.NOTIFY_LEGEND_RECALC);
    }
}

/**
 * Gets the <code>OtherLabel</code> property, which represents the label
 * used on the "other" pie slice. It can be unparsed text.
 * @return text used as the label for the "other" pie slice
 */
public String getOtherLabel()
{
    if (otherLabel == null) {
        Locale locale = Locale.getDefault();
        if (parent != null && parent instanceof ChartDataView) {
            locale = ((ChartDataView)parent).locale;
        }
        otherLabel = new String(JCChartBundle.string(JCChartBundle.key3,
                                                     locale));
    }
    return(otherLabel);
}

/**
 * Sets the <code>OtherLabel</code> property, which represents the label
 * used on the "other" pie slice. It can be unparsed text.
 * @param label String used as the label for the "other" pie slice
 */
public void setOtherLabel(String label)
{
    if (otherLabel != null && otherLabel.equals(label)) {
        return;
    }
    otherLabel = label;

    if (parent != null) {
        parent.setChanged(true, Changeable.REDRAW |
                                Changeable.NOTIFY_LEGEND_RECALC);
    }
}

/**
 * Gets the <code>OtherStyle</code> property, which specifies the style
 * used to render the "other" pie slice.
 * @return <code>JCChartStyle</code> instance outlining rendering information
 * for the "other" slice
 */
public JCChartStyle getOtherStyle()
{
    if (otherStyle == null) {
        if (parent != null && parent instanceof ChartDataView) {
            otherStyle =
                JCChartStyle.makeDefault(((ChartDataView) parent).getParent(),
                                         parent, true);
        }
        else {
            otherStyle = new JCChartStyle();
        }
    }
    return(otherStyle);
}

/**
 * Sets the <code>OtherStyle</code> property, which specifies the style
 * used to render the "other" pie slice.
 * @param style <code>JCChartStyle</code> instance outlining rendering
 * information for the "other" slice
 */
public void setOtherStyle(JCChartStyle style)
{
    if (style == null) {
        return;
    }
    otherStyle = style;

    if (parent != null) {
        parent.setChanged(true, Changeable.REDRAW |
                                Changeable.NOTIFY_LEGEND_REDRAW);
    }
}

/**
 * Gets the <code>ExplodeList</code> property, which specifies a list of
 * exploded pie slices in the pie charts.
 * @return an array of Point objects. Each Point object contains the
 * data point index (pie number) in the <i>x</i> value and the series number
 * (slice index) in the <i>y</i> value, specifying the pie slice to explode.
 * If the "other" slice is exploded, the series number returned is
 * <code>OTHER_SLICE</code>. If no slices are exploded, this function will
 * return null.
 */
public Point[] getExplodeList()
{
    return(explodeList);
}

/**
 * Sets the <code>ExplodeList</code> property, which specifies a list of
 * exploded pie slices in the pie charts.
 * @param pts an array of Point objects. Each Point object contains the
 * data point index (pie number) in the <i>x</i> value and the series number
 * (slice index) in the <i>y</i> value, specifying the pie slice to explode.
 * To explode the "other" slice, the series number should be
 * <code>OTHER_SLICE</code>. If null, no slices are exploded.
 */
public void setExplodeList(Point[] pts)
{
    explodeList = pts;

    if (parent != null) {
        parent.setChanged(true, Changeable.RECALC);
    }
}

/**
 * Gets the <code>ExplodeOffset</code> property, which specifies the distance a
 * slice is exploded from the center of a pie chart.
 * @return The explode offset
 */
public int getExplodeOffset()
{
    return(explodeOffset);
}

/**
 * Sets the <code>ExplodeOffset</code> property, which specifies the distance a
 * slice is exploded from the center of a pie chart.
 * @param offset The explode offset
 */
public void setExplodeOffset(int offset)
{
    explodeOffset = offset;

    if (parent != null) {
        parent.setChanged(true, Changeable.RECALC);
    }
}

/**
 * Gets the <code>StartAngle</code> property, which specifies the angle (in
 * degrees) at which the first pie slice is drawn.  A value of zero represents
 * a horizontal line from the center of the chart to the right edge of
 * the chart.  Increasing values move clockwise from there. Only the
 * range of values from zero degrees to 360 degrees is valid.
 * @return the start angle
 */
public double getStartAngle()
{
    return(startAngle);
}

/**
 * Sets the <code>StartAngle</code> property, which specifies the angle (in
 * degrees) at which the first pie slice is drawn.  A value of zero represents
 * a horizontal line from the center of the chart to the right edge of
 * the chart.  Increasing values move clockwise from there.  Only the
 * range of values from zero degrees to 360 degrees is valid.
 * @param sa the start angle
 */
public void setStartAngle(double sa)
{
    if (sa == startAngle) {
        return;
    }
    if (sa < 0.0 || sa > 360.0) {
        throw new IllegalArgumentException("StartAngle must be between 0.0 " +
                                           "and 360.0 degrees");
    }
    startAngle = sa;

    if (parent != null) {
        parent.setChanged(true, Changeable.RECALC);
    }
}

/**
 * Retrieves an array of image map information for the other slice, one for each
 * point (i.e. one for each pie).
 * @return The current image map information for the other slice.
 */
public ImageMapInfo[] getOtherImageMapInfo()
{
    return(otherImageMapInfo);
}

/**
 * Sets an array of image map information for the other slice, one for each
 * point (i.e. one for each pie).
 * @param otherImageMapInfo The image map information for the other slice.
 */
public void setOtherImageMapInfo(ImageMapInfo[] otherImageMapInfo)
{
    this.otherImageMapInfo = otherImageMapInfo;
}

/**
 * Retrieves the image map information for the other slice in the legend.
 * @return The current image map information for the other slice in the legend.
 */
public ImageMapInfo getOtherLegendImageMapInfo()
{
    return(otherLegendImageMapInfo);
}

/**
 * Sets the image map information for the other slice in the legend.
 * @param otherLegendImageMapInfo The new image map information for the other
 * slice in the legend.
 */
public void setOtherLegendImageMapInfo(ImageMapInfo otherLegendImageMapInfo)
{
    this.otherLegendImageMapInfo = otherLegendImageMapInfo;
}

}


