/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCPolarRadarChartFormat.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

/**
 * An object that manages chart properties
 * specific to <code>POLAR</code>, <code>RADAR</code>,
 * or <code>AREA_RADAR</code> charts.
 */

public class JCPolarRadarChartFormat implements JCChartTypeFormat,
												java.io.Serializable {

protected Changeable parent = null;


public JCPolarRadarChartFormat() {}

public JCPolarRadarChartFormat(Changeable p)
{
	parent = p;
}

public void setParent(ChartDataView cdv)
{
    parent = cdv;
}

protected JCAxis getYAxis()
{
	JCAxis rAxis = null;
	if ((parent != null) && (parent instanceof ChartDataView)) {
		rAxis = ((ChartDataView)parent).getYAxis();
	}
	return(rAxis);
}

protected JCAxis getThetaAxis()
{
	JCAxis thetaAxis = null;
	if ((parent != null) && (parent instanceof ChartDataView)) {
		thetaAxis = ((ChartDataView)parent).getXAxis();
	}
	return(thetaAxis);
}

protected int getCurrentAngleUnit()
{
    if ((parent != null) && (parent instanceof ChartDataView)) {
        ChartDataView cdv = (ChartDataView)parent;
        return(cdv.getParent().getChartArea().getAngleUnit());
    }
    return(JCChartUtil.RADIANS);
}

/**
 * Sets the angle of the y axis in a Polar or Radar chart.  Angles are
 * relative to the current origin base.  The angle units are assumed to be the
 * current value of the chart area's <code>angleUnit</code> property.
 *
 * @param angle the desired angle for the y axis
 */
public void setYAxisAngle(double angle)
{
    setYAxisAngle(getCurrentAngleUnit(), angle);
}

/**
 * Sets the angle of the y axis in a Polar or Radar chart.  Angles are
 * relative to the current origin base.
 *
 * @param units The units of the specified angle.  Valid values are
 * <code>JCChartUtil.DEGREES</code>, <code>JCChartUtil.GRADS</code>, or
 * <code>JCChartUtil.RADIANS</code>.
 * @param angle the desired angle for the y axis
 */
public void setYAxisAngle(int units, double angle)
{
    JCAxis rAxis = getYAxis();
    if (rAxis != null) {
        rAxis.setYAxisAngle(units, angle);
        if (parent != null) {
            parent.setChanged(true, Changeable.RECALC);
        }
    }
}

/**
 * Gets the angle of the y axis in a Polar or Radar chart.  Angles are
 * relative to the current origin base.  The angle units are assumed to be the
 * current value of the chart area's angleUnit property.
 *
 * @return the current y axis angle of a Polar or Radar chart
 */
public double getYAxisAngle()
{
    return(getYAxisAngle(getCurrentAngleUnit()));
}

/**
 * Gets the angle of the y axis in a Polar or Radar chart.  Angles are
 * relative to the current origin base.
 *
 * @param units The desired units of the returned angle.  Valid values are
 * <code>JCChartUtil.DEGREES</code>, <code>JCChartUtil.GRADS</code>, or
 * <code>JCChartUtil.RADIANS</code>.
 * @return  The current y axis angle of a Polar or Radar chart based on the
 * units specified.  On error, returns the default value of 0.0.
 */
public double getYAxisAngle(int units)
{
    JCAxis rAxis = getYAxis();
    if (rAxis != null) {
        return (rAxis.getYAxisAngle(units));
    }
    // If no rAxis, return the default value.  Needed for HTML & XML output
    return(0.0);
}

/**
 * Sets the angle of the theta axis origin.   Angles are based on
 * zero degrees pointing east (the normal rectangular x axis direction) with
 * positive angles going counter-clockwise.  The angle units are assumed to be
 * the current value of the chart area's angleUnit property.
 *
 * @param angle the desired angle for the theta axis origin
 */
public void setOriginBase(double angle)
{
    setOriginBase(getCurrentAngleUnit(), angle);
}

/**
 * Sets the angle of the theta axis origin.  Angles are based on
 * zero degrees pointing east (the normal rectangular x axis direction) with
 * positive angles going counter-clockwise.
 *
 * @param units The units of the specified angle.  Valid values are
 * <code>JCChartUtil.DEGREES</code>, <code>JCChartUtil.GRADS</code>, or
 * <code>JCChartUtil.RADIANS</code>.
 * @param angle the desired angle for the theta axis origin
 */
public void setOriginBase(int units, double angle)
{
    JCAxis thetaAxis = getThetaAxis();
    if (thetaAxis != null) {
        thetaAxis.polarAxisParams.setOriginBase(units, angle);
        if (parent != null) {
            parent.setChanged(true, Changeable.RECALC);
        }
    }
}

/**
 * Gets the angle of the theta axis origin.   Angles are based on
 * zero degrees pointing east (the normal rectangular x axis direction) with
 * positive angles going counter-clockwise.  The angle units are assumed to be
 * the current value of the chart area's angleUnit property.
 *
 * @return the current theta axis origin
 */
public double getOriginBase()
{
    return(getOriginBase(getCurrentAngleUnit()));
}

/**
 * Gets the angle of the theta axis origin.   Angles are based on
 * zero degrees pointing east (the normal rectangular x axis direction) with
 * positive angles going counter-clockwise.
 *
 * @param units The desired units of the returned angle.  Valid values are
 * <code>JCChartUtil.DEGREES</code>, <code>JCChartUtil.GRADS</code>, or
 * <code>JCChartUtil.RADIANS</code>.
 * @return The current theta axis origin based on the angle units specified.
 * On error, returns the default value of 0.0.
 */
public double getOriginBase(int units)
{
    JCAxis thetaAxis = getThetaAxis();
    if (thetaAxis != null) {
        return(thetaAxis.polarAxisParams.getOriginBase(units));
    }
    // If no rAxis, return the default value.  Needed for HTML & XML output
    return(0.0);
}

/**
 * Sets the half range flag for Polar charts.
 *
 * @param fHalfRange if <code>true</code>, theta axis labels on a Polar chart
 * will have a maximum value of 180 degrees
 */
public void setHalfRange(boolean fHalfRange)
{
    JCAxis thetaAxis = getThetaAxis();
    if (thetaAxis != null) {
        thetaAxis.polarAxisParams.setPolarHalfRange(fHalfRange);
        if (parent != null) {
            parent.setChanged(true, Changeable.RECALC);
        }
    }
}

/**
 * Gets the half range flag for Polar charts.
 *
 * @return <code>true</code> if theta axis labels have a maximum value of 180
 * degrees; on error, return <code>false</code> (the default value)
 */
public boolean isHalfRange()
{
    JCAxis thetaAxis = getThetaAxis();
    if (thetaAxis != null) {
        return (thetaAxis.polarAxisParams.isPolarHalfRange());
    }
    return (false);
}

/**
 * Makes the y gridlines of a Radar chart circular.
 *
 * @param fCircular <code>true</code> for circular gridlines;
 * <code>false</code> for webbed gridlines
 */
public void setRadarCircularGrid(boolean fCircular)
{
    JCAxis rAxis = getYAxis();
    if (rAxis != null) {
        rAxis.polarAxisParams.setCircularGrid(fCircular);
        if (parent != null) {
            parent.setChanged(true, Changeable.RECALC);
        }
    }
}

/**
 * Gets the state of the circular flag for Radar chart gridlines.
 *
 * @return <code>true</code> if the y gridlines are circular; <code>false</code>
 * if the y gridlines are webbed.  On error, return <code>true</code> (the
 * default value).
 */
public boolean isRadarCircularGrid()
{
    JCAxis rAxis = getYAxis();
    if (rAxis != null) {
        return (rAxis.polarAxisParams.isCircularGrid());
    }
    return(true);
}

}
