/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCShape.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.*;

/**
 * <code>JCShape</code> is an abstract base class used for defining custom
 * point styles in JClass Chart.  To create a new shape class, extend
 * <code>JCShape</code> and provide a <code>resize()</code> method.  The
 * <code>resize()</code> method must populate the <i>x[]</i> and <i>y[]</i>
 * arrays with point values for the shape to be drawn.  The point values are
 * relative to (0,0).  Chart will automatically move the shape to the
 * appropriate location. 
 */
abstract public class JCShape {

/** Size of the point.  Used inside resize() to populate x[] and y[] */
protected int size = 0;

/** Array of x values. */
protected int x[];
/** Array of y values. */
protected int y[];

/** 
 * Populates the <i>x[]</i> and <i>y[]</i> arrays based on the
 * provided size.  
 * @param sz value to be assigned to size member variable
 */
abstract protected void resize(int sz);

/**
 * Draws the point at the specified integer pixel coordinates.
 * @param gc graphics context to use for drawing
 * @param xorg <i>x</i> origin point as an integer
 * @param yorg <i>y</i> origin point as an integer
 */
public void draw(Graphics gc, int xorg, int yorg)
{
    if (x == null || y == null || x.length == 0 || y.length == 0) {
        return;
    }

    int localx[] = new int[x.length];
    int localy[] = new int[y.length];
    for (int i = 0; i < localx.length; i++) {
        localx[i] = x[i] + xorg;
        localy[i] = y[i] + yorg;
    }
    gc.fillPolygon(localx, localy, localx.length);
}

/**
 * Draws the point at the specified floating point pixel coordinates.
 * @param gc graphics context to use for drawing
 * @param xorg <i>x</i> origin point as a double
 * @param yorg <i>y</i> origin point as a double
 */
public void draw(Graphics gc, double xorg, double yorg)
{
    if (x == null || y == null || x.length == 0 || y.length == 0) {
        return;
    }

    GeneralPath symbolPath = new GeneralPath();
    symbolPath.moveTo((float)((double)x[0] + xorg),
                      (float)((double)y[0] + yorg));
    for (int i = 1; i < x.length; i++) {
        symbolPath.lineTo((float)((double)x[i] + xorg),
                          (float)((double)y[i] + yorg));
    }
    ((Graphics2D)gc).fill(symbolPath);
}

}
