/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: LinearAxisScaleHandler.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:47 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.util.List;
import java.util.Iterator;

/**
 * An AxisScaleHandler for linear axes.
 */
class LinearAxisScaleHandler extends BaseAxisScaleHandler {


public void calcPrecision() {
    if (parent == null || parent.getAnnotationList() == null) {
        return;
    }

    // Don't change the precision if the parent is the theta
    // axis of a Polar chart.
    if (parent.chartType == JCChart.POLAR && !parent.i_vertical) {
        return;
    }

    double lmin = (parent.min.isDefault && parent.dataLimits != null) ?
                    parent.dataLimits.min : parent.min.value;
    double lmax = (parent.max.isDefault && parent.dataLimits != null) ?
                    parent.dataLimits.max : parent.max.value;

    double delta = (lmax <= lmin) ? lmax : lmax - lmin;
    int prec = JCChartUtil.nicePrecision(delta);

    // Set the default precision for each of the JCAnno
    // objects.
    List annoList = parent.getAnnotationList();
    JCAnno tickObj = null;
    Iterator iterator = annoList.iterator();
    while (iterator.hasNext()) {
        tickObj = (JCAnno)iterator.next();
        if (tickObj != null) {
            if (!tickObj.getIncrementIsDefault()) {
                tickObj.defPrecision =
                    JCChartUtil.tickBasedPrecision(
                            tickObj.getIncrementValue());
            }
            else {
                tickObj.defPrecision =
                    JCChartUtil.clamp(prec, -parent.getMaxPrecision(),
                                      parent.getMaxPrecision());
            }
            if (tickObj.getPrecisionIsDefault()) {
                tickObj.precision.value = tickObj.defPrecision;
            }
        }
    }
}

/**
 * Intended for internal use only.
 */
public void updateAnnoObject(JCAnno anno) {
    if (anno != null) {
        int type = anno.getType();
        double increment = (type == JCAnno.DEFAULT_LABELS) ?
            parent.numSpacing.value : parent.tickSpacing.value;
        updateAnnoObject(anno, parent.min.value,
                         parent.max.value, increment);
    }
}

/**
 * Converts each of the ticks specified by the JCAnno parameter
 * into a JCValueLabel.  The new JCValueLabels are added to the
 * AnnotationHandler's internal List of value labels.
 *
 * The calcParams() method of AxisHandler must be called first.
 */
public void makeTickLabels(JCAnno anno) {
    if (anno == null) {
        return;
    }
    List valueLabels = parent.getAnnotations();
    double startValue = anno.getStartValue();
    double stopValue = anno.getStopValue();
    double increment = anno.getIncrementValue();
    double err = JCChartUtil.calcError(anno.getPrecision());
    if (JCChartUtil.abs(increment) <= err) {
        return;
    }

    JCValueLabel tickLabel = null;
    //  Make sure we don't exceed the precision.  Otherwise,
    //  we will get caught in an infinite loop.
    if (startValue + increment == startValue) {
        return;
    }

    //  Impose a limit on the maximum number of tick marks.
    int maxTicks = parent.pixelLength() * 2;
    int count = 0;
    if (startValue <= stopValue && increment > 0) {
        for (double d = startValue;
             d <= (stopValue + err) && count < maxTicks;
             d += increment, count++)
        {
            if (!isDuplicateLabel(anno, d)) {
                tickLabel = new JCValueLabel(anno, d);
                valueLabels.add(tickLabel);
            }
        }
    }
    else if (startValue >= stopValue && increment < 0) {
        for (double d = startValue;
             d >= (stopValue - err) && count < maxTicks;
             d += increment, count++) {
            if (!isDuplicateLabel(anno, d)) {
                tickLabel = new JCValueLabel(anno, d);
                valueLabels.add(tickLabel);
            }
        }
    }
}

public double range() {
    double r = Double.MAX_VALUE;
    if (parent != null) {
        r = parent.max.value - parent.min.value;
    }
    return r;
}

public void translate(double t) {
    if (parent == null) {
        return;
    }
    if (parent.reversed) {
        t = -t;
    }
    parent.min.value += t;
    parent.max.value += t;
}

public String format(double value, int precision) {
    return JCChartUtil.format(value, precision);
}

}
