/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: LogAxisScaleHandler.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:47 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.util.List;

/**
 * An AxisScaleHandler for logarithmic axes.
 */
class LogAxisScaleHandler extends BaseAxisScaleHandler {

public boolean fixBounds() {
    boolean modified = super.fixBounds();
    double amin, amax;
    if (!parent.max.isDefault) {
        if (parent.max.value <= 0) {
            String s =
                JCChartBundle.string(JCChartBundle.key104, parent.locale);
            s +=  JCChartBundle.string(JCChartBundle.key130, parent.locale);
            ErrorDialog.raise(parent, s);
            parent.max.isDefault = true;
            modified = true;
        }
    }

    if (!parent.min.isDefault) {
        if (parent.min.value <= 0) {
            String s =
                JCChartBundle.string(JCChartBundle.key106, parent.locale);
            s += JCChartBundle.string(JCChartBundle.key132, parent.locale);
            ErrorDialog.raise(parent, s);
            parent.min.isDefault = true;
            modified = true;
        }
    }

    if (!parent.min.isDefault && !parent.max.isDefault) {
        amin = JCChartUtil.log10(parent.min.value);
        amax = JCChartUtil.log10(parent.max.value);
        if ((amax - amin) < Double.MIN_VALUE) {
            String s =
                JCChartBundle.string(JCChartBundle.key7, parent.locale);
            s += JCChartBundle.string(JCChartBundle.key130, parent.locale);
            ErrorDialog.raise(parent, s);
            parent.max.isDefault = true;
            modified = true;
        }
    }

    return modified;

}

public void calcPrecision() {
    if (parent == null) {
        return;
    }
    if (parent.precision.isDefault) {
        if (parent.annotationMethod == JCAxis.TIME_LABELS ||
            parent.dataLimits == null) {
            parent.precision.value = 0;
            parent.getDefaultLabels().precision.value = 0;
        }
        else {
            int prec = -JCAxis.MAX_PRECISION;
            parent.precision.value = prec;
            parent.updateAnnoDefPrecision(prec);
        }
    }
    int prec = JCChartUtil.clamp(parent.getPrecision(),
                                 -parent.getMaxPrecision(),
                                 parent.getMaxPrecision());

    parent.setPrecisionInternal(prec);
}

/**
 * Intended for internal use only.
 */
public void updateAnnoObject(JCAnno anno) {
    updateAnnoObject(anno, parent.min.value, parent.max.value, 0.0);
}

/**
 * Converts each of the ticks specified by the JCAnno parameter
 * into a JCValueLabel.  The new JCValueLabels are added to the
 * AnnotationHandler's internal List of value labels.
 *
 * The calcParams() method of AxisHandler must be called first.
 */
public void makeTickLabels(JCAnno anno) {
    if (anno == null) {
        return;
    }
    List valueLabels = parent.getAnnotations();

    int ilog1, ilog2;
    double amax = anno.getStopValueIsDefault() ?
                        parent.max.value : anno.getStopValue();
    double amin = anno.getStartValueIsDefault() ?
                        parent.min.value : anno.getStartValue();
    if (amax <= 0.0) {
        amax = 1.0;
        amin = 0.1;
    }
    else {
        if (amin < 0.0) {
            amin = amax / 10.0;
        }
    }

    // Need to apply precCorrect() on the log10 values.  Otherwise,
    // incorrect ilog values may result due to floating point error.
    final int MAX_PRECISION = 10;
    double log10_amin =
        JCChartUtil.precCorrect(MAX_PRECISION, JCChartUtil.log10(amin));
    double log10_amax =
        JCChartUtil.precCorrect(MAX_PRECISION, JCChartUtil.log10(amax));
    ilog1 = JCChartUtil.ifloor(log10_amin);
    ilog2 = JCChartUtil.iceil(log10_amax);

    JCValueLabel tickLabel = null;
    for (int i = ilog1; i <= ilog2; i++) {
        double pnum = JCChartUtil.pow10(i);
        if (anno.getType() == JCAnno.DEFAULT_LABELS) {
            if (!isDuplicateLabel(anno, pnum)) {
                tickLabel = new JCValueLabel(parent, i, anno.getPrecision());
                tickLabel.setAnno(anno);
                valueLabels.add(tickLabel);
            }
        }
        else {
            for (int j = 1; j < 10; j++) {
                double val = j * pnum;
                if (val < parent.min.value || val > parent.max.value) {
                    continue;
                }
                if (!isDuplicateLabel(anno, val)) {
                    tickLabel = new JCValueLabel(anno, val);
                    valueLabels.add(tickLabel);
                }
            }
        }
    }
}

public double range() {
    if (parent == null) {
        return Double.MAX_VALUE;
    }
    int min = JCChartUtil.ifloor(JCChartUtil.log10(parent.min.value));
    int max = JCChartUtil.iceil(JCChartUtil.log10(parent.max.value));
    return (double)(max - min);
}

public void translate(double t) {
    if (parent == null) {
        return;
    }
    double lmin = JCChartUtil.log10(parent.min.value);
    double lmax = JCChartUtil.log10(parent.max.value);
    if (parent.reversed) {
        t = -t;
    }
    parent.min.value = Math.pow(10.0, lmin+t);
    parent.max.value = Math.pow(10.0, lmax+t);
}

public String format(double value, int precision) {
    return JCChartUtil.logFormat((int)value);
}

}
