/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Mapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:47 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.awt.*;
import java.util.Map;
import java.util.Vector;

/**
 * This class is used for Pie chart image mapping.  For internal use only.
 */
public class Mapper implements PieTagModel {

protected PieData        pieData = null;
protected Vector         shapesInDrawingOrder = null;
protected Map            hash = null;
protected JCDataIndex    currentDataIndex = null;
protected ChartDataView  restrictedDataView = null;
protected int            restrictedSeries = -1;
protected int            restrictedPoint = -1;

/**
 * Constructor.
 * @param hashTable
 */
public Mapper(Map hashTable)
{
    this(null, hashTable);
}

/**
 * Constructor.
 * @param hashTable
 * @param pd
 */
public Mapper(PieData pd, Map hashTable)
{
    shapesInDrawingOrder = new Vector();
    hash = hashTable;
    pieData = pd;
}

/**
 * Add shape to shape list and hash.  However, if the series has been restricted
 * and the series index of the current data index does not match the restricted
 * series, don't add it.
 * @param shape The shape to be added.
 */
public void addShape(Shape shape)
{
    // If we've restricted the series/point and the series/point index of the
    // current data index does not match the restricted series/point, don't add
    // it.
    if (currentDataIndex != null &&
        ((restrictedDataView != null &&
          restrictedDataView != currentDataIndex.getDataView()) ||
         (restrictedSeries != -1 &&
          restrictedSeries != currentDataIndex.getSeriesIndex()) ||
         (restrictedPoint != -1 &&
          restrictedPoint != currentDataIndex.getPoint())))
    {
        return;
    }
    shapesInDrawingOrder.add(shape);
    addToHash(shape);
}

public void addToHash(Shape shape)
{
    hash.put(shape, currentDataIndex);
}

public Vector getShapeList()
{
    return(shapesInDrawingOrder);
}

public void setPieData(PieData pd)
{
    pieData = pd;
}

public PieData getPieData()
{
    return(pieData);
}

public void setDataIndex(JCDataIndex index)
{
    currentDataIndex = index;
}

public JCDataIndex getDataIndex()
{
    return(currentDataIndex);
}

public ChartDataView getRestrictedDataView()
{
    return(restrictedDataView);
}

public void setRestrictedDataView(ChartDataView restrictedDataView)
{
    this.restrictedDataView = restrictedDataView;
}

public int getRestrictedSeries()
{
    return(restrictedSeries);
}

public void setRestrictedSeries(int restrictedSeries)
{
    this.restrictedSeries = restrictedSeries;
}

public int getRestrictedPoint()
{
    return(restrictedPoint);
}

public void setRestrictedPoint(int restrictedPoint)
{
    this.restrictedPoint = restrictedPoint;
}

}
