/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: MinMax.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:47 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

/**
 * Internal object used by <code>JCChart</code> to represent the minimum
 * and maximum in a series of data values.
 */
public class MinMax implements java.io.Serializable {

/** Minimum value. */
public double min = Double.MAX_VALUE;

/** Maximum value. */
public double max = -min;

/**
 * Constructs an instance of <code>MinMax</code> using the provided array
 * to determine the minimum and maximum values.
 * @param ar array of floating point values from which the min and
 * max are retrieved
 * @param hv hole value for the data set
 */
MinMax(double ar[], double hv)
{
	setMinMax(ar, hv);
}

/**
 * Default constructor for <code>MinMax</code>.
 */
MinMax() {}

/**
 * Copy constructor for <code>MinMax</code>.  Copies the min and max values
 * from the provided <code>MinMax</code> object.
 * @param source object from which to copy min and max values
 */
MinMax(MinMax source)
{
	min = source.min;
	max = source.max;
}

/**
 * Constructor for <code>MinMax</code> that sets the min and max values
 * explicitly.
 * @param newMin minimum value
 * @param newMax maximum value
 */
MinMax(double newMin, double newMax)
{
	min = newMin;
	max = newMax;
}

/**
 * Determines the minimum and maximum of the provided array.
 * @param ar array of floating point values from which the min and max are
 *           retrieved
 * @param holeValue hole value for the data set
 */
void setMinMax(double ar[], double holeValue)
{
	setMinMax(ar, holeValue, false);
}

/**
 * Determines the minimum and maximum of the provided array.
 * @param ar array of floating point values from which the min and
 * max are retrieved
 * @param holeValue hole value for the data set
 * @param isLog <code>true</code> if dealing with logarithmic values, where
 *              values less than or equal to zero should be ignored;
 *              <code>false</code> otherwise
 */
void setMinMax(double ar[], double holeValue, boolean isLog)
{
	if (ar == null) {
		return;
	}
	for (int i = 0; i < ar.length; i++) {
		setMinMax(ar[i], holeValue, isLog);
	}
}

/**
 * Changes the minimum and maximum depending on the provided value.
 * @param val floating point value
 * @param holeValue hole value for the data set
 */
void setMinMax(double val, double holeValue)
{
	setMinMax(val, holeValue, false);
}

/**
 * Changes the minimum and maximum depending on the provided value.
 * @param val floating point value
 * @param holeValue hole value for the data set
 * @param isLog <code>true</code> if dealing with logarithmic values, where
 *              values less than or equal to zero should be ignored;
 * <code>false</code> otherwise
 */
void setMinMax(double val, double holeValue, boolean isLog)
{
	if (val == holeValue || (isLog && (val <= 0.0))) {
		return;
	}
	if (val < min) {
		min = val;
	}
	if (val > max) {
		max = val;
	}
}

/**
 * Calculates the union of two <code>MinMax</code> objects.
 * @param mm <code>MinMax</code> object with which to perform union
 */
void union(MinMax mm)
{
	min = Math.min(min, mm.min);
	max = Math.max(max, mm.max);
}

/**
 * Copies the min and max values from the provided <code>MinMax</code>
 * object.
 * @param mm <code>MinMax</code> object from which to copy min and max values
 */
void reset(MinMax mm)
{
	min = mm.min;
	max = mm.max;
}

/**
 * Returns a String with the current min and max.
 * @return a String representation of this object
 */
public String toString()
{
    StringBuffer sb = new StringBuffer("min= ");
    sb.append(min);
    sb.append(" max=");
    sb.append(max);
    return(sb.toString());
}

}
