/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PickShape.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:47 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.Point2D;

public class PickShape implements ProcessShapeModel {

protected Picker picker = null;

public PickShape(Picker picker) {
	this.picker = picker;
}

public void updateFillOrientation(Rectangle rect, int newOrient) {
	//  Do nothing.
}

public void resetFillOrientation() {
	//  Do nothing.
}

/**
 * Determines the distance between a pick point and a polygon.
 * The pick point coordinates are contained in a <code>Picker</code> object
 * passed to the <code>PickShape</code> constructor.  The same <code>Picker</code> object
 * will contain the distance computed in this method.  If the
 * pick point is inside the polygon, the distance will be zero.
 * @param shape
 */
public void fillPolygon(Shape shape) {
	int distance = -1;

	distance = getDistanceToShape(shape,
		picker.pix_x, picker.pix_y,
		ChartDataView.PICK_FOCUS_X | ChartDataView.PICK_FOCUS_Y);
	if (distance <= picker.distance) {
		// This slice is closer than any other slice so far.
		picker.distance = distance;
		picker.set = picker.tag_set;
		picker.point = picker.tag_point;
	}
}

/**
 * Same as <code>fillPolygon</code>.
 * @param shape
 */
public void fillPolygonShadow(Shape shape) {
	fillPolygon(shape);
}

/**
 * Determines the distance between a pick point and a filled arc.
 * The pick point coordinates are contained in a <code>Picker</code> object
 * passed to the <code>PickShape</code> constructor.  The same <code>Picker</code> object
 * will contain the distance computed in this method.  If the
 * pick point is inside the arc region, the distance will be zero.
 * @param arcAngle
 * @param height
 * @param startAngle
 * @param width
 * @param x
 * @param y
 */
public void fillArc(double x, double y, double width, double height,
					double startAngle, double arcAngle) {
	double xRadius = width / 2.0;
	double yRadius = height / 2.0;
	double xCenter = x + xRadius;
	double yCenter = y + yRadius;

	if (width == 0 || height == 0) {
		// Nonexistant arcs can't be picked.
	}
	else if (picker.pix_x == (int)Math.round(xCenter) &&
			 picker.pix_y == (int)Math.round(yCenter)) {
		// The pick point is on the center point.
		picker.distance = 0;
		picker.set = picker.tag_set;
		picker.point = picker.tag_point;
	}
	else {
		// Determine the angle of the pick point on the pie and
		// the distance from the pick point to the center of the
		// pie.
		double xDiff = picker.pix_x - xCenter;
		double yDiff = picker.pix_y - yCenter;
		double distanceCenterPick = Math.sqrt(xDiff * xDiff + yDiff * yDiff);
		double pickAngle = Math.atan2(-yDiff/height, xDiff/width);
		pickAngle = JCChartUtil.convertAngle(JCChartUtil.RADIANS,
									JCChartUtil.DEGREES, pickAngle);
		//  Ensure that pickAngle is between 0 and 360.
		if (pickAngle < 0.0) {
			pickAngle += 360.0;
		}

		if (startAngle < 0.0) {
			pickAngle += ((int)Math.round(startAngle) / 360 - 1) * 360.0;
		}
		else {
			pickAngle += ((int)Math.round(startAngle) / 360) * 360.0;
		}

		double distance = -1.0;
		double endAngle = startAngle + arcAngle;

//		System.out.println("picker.pix_x=" + picker.pix_x + ", picker.pix_y="
//			+ picker.pix_y + ", xCenter=" + xCenter + ", yCenter=" + yCenter
//			+ ", xDiff=" + xDiff + ", yDiff=" + yDiff + ", pickAngle="
//			+ pickAngle + ", startAngle=" + startAngle + ", endAngle="
//			+ endAngle);

		if (withinRange(pickAngle, startAngle, arcAngle)) {
			// The pick point is inside the arc defined by [start, end].
			// Determine where the pick radial intersects the arc and
			// the distance from the intersection to the center point of
			// the pie.
			pickAngle = JCChartUtil.convertAngle(JCChartUtil.DEGREES,
									JCChartUtil.RADIANS, pickAngle);
			double xIntersect = xCenter + Math.cos(pickAngle) * xRadius;
			double yIntersect = yCenter - Math.sin(pickAngle) * yRadius;
			xDiff = xIntersect - xCenter;
			yDiff = yIntersect - yCenter;
			double distanceCenterIntersection =
									Math.sqrt(xDiff * xDiff + yDiff * yDiff);

			if (distanceCenterPick > distanceCenterIntersection) {
				// The pick point is outside the slice.
				distance = distanceCenterPick - distanceCenterIntersection;
			}
			else {
				// The pick point is inside the slice.
				distance = 0.0;
			}
		}
		else {
			// The pick point is outside the arc defined by [start, end].
			// Calculate the end points of the start and end vectors.
			double angle = JCChartUtil.convertAngle(JCChartUtil.DEGREES,
									JCChartUtil.RADIANS, startAngle);
			double xStart = xCenter + Math.cos(angle) * xRadius;
			double yStart = yCenter - Math.sin(angle) * yRadius;

			angle = JCChartUtil.convertAngle(JCChartUtil.DEGREES,
									JCChartUtil.RADIANS, endAngle);
			double xEnd = xCenter + Math.cos(angle) * xRadius;
			double yEnd = yCenter - Math.sin(angle) * yRadius;

			double xPick = picker.pix_x;
			double yPick = picker.pix_y;

			// Determine the distance to the start and end vectors.
			double distancePickStart = JCChartUtil.getDistanceToVector(
										new Point2D.Double(xCenter, yCenter),
										new Point2D.Double(xStart, yStart),
										new Point2D.Double(xPick, yPick));
			double distancePickEnd = JCChartUtil.getDistanceToVector(
										new Point2D.Double(xCenter, yCenter),
										new Point2D.Double(xEnd, yEnd),
										new Point2D.Double(xPick, yPick));
			distance = Math.min(distancePickStart, distancePickEnd);
		}

		if (distance <= picker.distance) {
			// This slice is closer than any other slice so far.
			picker.distance = (int)Math.round(distance);
			picker.set = picker.tag_set;
			picker.point = picker.tag_point;
		}
	}
}

/**
 * Same as <code>fillArc</code>.
 * @param arcAngle
 * @param height
 * @param startAngle
 * @param width
 * @param x
 * @param y
 */
public void fillArcShadow(double x, double y, double width, double height,
						  double startAngle, double arcAngle) {
	fillArc(x, y, width, height, startAngle, arcAngle);
}

public void drawLine(double x1, double y1, double x2, double y2) {
	//  Do nothing.
}

public void drawArc(double x, double y, double width, double height,
					double startAngle, double arcAngle) {
	//  Do nothing.
}

// Returns the pixel distance between a pick point and a Shape.
protected int getDistanceToShape(Shape shape, int x, int y, int focus) {
	int distance = -1;

	switch(focus) {
	case ChartDataView.PICK_FOCUS_X:
		Rectangle bounds = shape.getBounds();
		int right = bounds.x + bounds.width;
		if (x < bounds.x) {
			//  The point is to the left of the shape.
			distance = bounds.x - x;
		}
		else if (x > right) {
			//  The point is to the right of the shape.
			distance = x - right;
		}
		else {
			//  The point is within the x range of the shape
			distance = 0;
		}
		break;

	case ChartDataView.PICK_FOCUS_Y:
		bounds = shape.getBounds();
		int bottom = bounds.y + bounds.height;
		if (y < bounds.y) {
			//  The point is above the shape.
			distance = bounds.y - y;
		}
		else if (y > bottom) {
			//  The point is below the shape.
			distance = y - bottom;
		}
		else {
			//  The point is within the y range of the shape
			distance = 0;
		}
		break;

	case ChartDataView.PICK_FOCUS_XY:  //  Fall through to default case.
	default:
		distance = (int)Math.round(JCChartUtil.getShapeDistance(
									new Point2D.Double(x,y), shape, 2.0));
	}

	return distance;
}

//  Angle is assumed to be in degrees.
private double boundAngle(double angle) {
	if (angle < 0.0) {
		angle += 360.0;
	}
	else if (angle > 360.0) {
		angle -= 360.0;
	}
	return angle;
}

//  All angles are assumed to be in degrees.
protected boolean withinRange(double pickAngle, double startAngle,
								double arcAngle) {

	boolean fWithinRange = false;

	pickAngle = boundAngle(pickAngle);
	startAngle = boundAngle(startAngle);
	double endAngle = startAngle + arcAngle;
	if (arcAngle < 0.0) {
		if (endAngle >= 0.0) {
			if (pickAngle <= startAngle && pickAngle >= endAngle) {
				fWithinRange = true;
			}
		}
		else {
			endAngle += 360.0;
			if (pickAngle <= startAngle ||
				pickAngle >= endAngle) {
				fWithinRange = true;
			}

		}
	}
	else {  //  arcAngle >= 0.0
		if (endAngle <= 360.0) {
			if (pickAngle >= startAngle && pickAngle <= endAngle) {
				fWithinRange = true;
			}
		}
		else {
			endAngle = boundAngle(endAngle);
			if (pickAngle >= startAngle ||
				pickAngle <= endAngle) {
				fWithinRange = true;
			}
		}
	}

	return fWithinRange;
}

}
