/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PlotArea.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:47 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import com.klg.jclass.util.ImageMapInfo;

import java.awt.*;

/**
 * <code>PlotArea</code> is a utility class used inside <code>JCChart</code> to
 * represent the background of the <code>ChartArea</code>.
 */
public class PlotArea implements java.io.Serializable {

protected DerivedInt left   = new DerivedInt(0, true);
protected DerivedInt top    = new DerivedInt(0, true);
protected DerivedInt bottom  = new DerivedInt(0, true);
protected DerivedInt right = new DerivedInt(0, true);

/** The image map information for the <code>PlotArea</code>. */
protected ImageMapInfo imageMapInfo = null;

/** Foreground color used in the chart region.  Inherited from parent. */
private Color foreground = null;

/** Background color used in the chart region.  Inherited from parent. */
private Color background = null;

/** Connetion to the chart region on which the <code>PlotArea</code> appears. */
protected ChartRegion parent = null;

public PlotArea() {}

/**
 * Constructor that connects the <code>PlotArea</code> to a
 * parent <code>ChartRegion</code>.
 * @param p parent chart region
 */
public PlotArea(ChartRegion p)
{
    parent = p;
}

/**
 * Gets the value of the <code>Foreground</code> property.  The
 * <code>Foreground</code> property determines the color used to draw the axis
 * bounding box controlled by <code>JCChartArea</code>.  <br><B>Note:</b> The
 * <code>Foreground</code> property is inherited from the parent
 * <code>ChartRegion</code>.
 * @return color used to draw the axis bounding box
 */
public Color getForeground()
{
    return(foreground == null) ? (parent == null) ? null
                                                  : parent.getForeground()
                               : foreground;
}

/**
 * Sets the value of the <code>Foreground</code> property.  The
 * <code>Foreground</code> property determines the color used to draw the axis
 * bounding box controlled by <code>JCChartArea</code>.  <br><B>Note:</b> The
 * <code>Foreground</code> property is inherited from the parent
 * <code>ChartRegion</code>.
 * @param c color used to draw the axis bounding box
 * @see com.klg.jclass.chart.JCChartArea#setAxisBoundingBox
 */
public void setForeground(Color c)
{
    if (c == foreground) {
        return;
    }
    foreground = c;

    if (parent != null) {
        parent.setChanged(true, Changeable.RECALC);
    }
}

/**
 * Gets the value of the <code>Background</code> property. The
 * <code>Background</code> property determines the background color used to
 * draw inside the chart region.  The <code>Background</code> property is
 * derived as follows:
 * If the property has been set by a user, that value is used.
 * Otherwise, it is inherited from the object's parent.  If the parent
 * is null, null is returned.
 * @return background color used to draw the chart region
 */
public Color getBackground()
{
    return(background == null) ? (parent == null) ? null
                                                   : parent.getBackground()
                               : background;
}

/**
 * Sets the value of the <code>Background</code> property.  The
 * <code>Background</code> property determines the foreground color used to
 * draw inside the chart region. <br><B>Note:</b> The <code>Background</code>
 * property is inherited from the parent <code>ChartRegion</code>.
 * Also note that the <code>Opaque</code> property of the parent
 * <code>JCChartArea</code> object needs to be set to <code>true</code> for the
 * background color to be visible. <br><code>JCChartArea</code> is transparent
 * by default.
 * @param c background color used to draw the chart region
 */
public void setBackground(Color c)
{
    if (c == background) {
        return;
    }
    background = c;

    if (parent != null) {
        parent.setChanged(true, Changeable.RECALC);
    }
}

/**
 * Sets the value of the <code>Top</code> property.
 * The <code>Top</code> property determines the location of the
 * top of the <code>ChartRegion</code>.
 * @param t top value in pixels
 */
public void setTop(int t)
{
    top.isDefault = false;
    if (top.value == t) {
        return;
    }
    top.value = t;

    if (parent != null) {
        parent.setChanged(true, Changeable.RECALC);
    }
}

/**
 * Gets the value of the <code>Top</code> property.
 * The <code>Top</code> property determines the location of the
 * top of the <code>ChartRegion</code>.
 * @return top value in pixels.
 */
public int getTop()
{
    return(top.value);
}

/**
 * Sets the value of the <code>TopIsDefault</code> property.
 * The <code>TopIsDefault</code> property determines whether the top
 * position of the chart region is calculated by Chart (<code>true</code>) or
 * taken from the <code>Top</code> property (<code>false</code>).
 * @param d <code>true</code> if the position is to be calculated,
 * <code>false</code> otherwise
 */
public void setTopIsDefault(boolean d)
{
    if (top.isDefault == d) {
        return;
    }
    top.isDefault = d;

    if (parent != null) {
        parent.setChanged(true, Changeable.RECALC);
    }
}

/**
 * Gets the value of the <code>TopIsDefault</code> property.
 * The <code>TopIsDefault</code> property determines whether the top
 * position of the chart region is calculated by Chart (<code>true</code>) or
 * taken from the <code>Top</code> property (<code>false</code>).
 * @return <code>true</code> if the position is to be calculated,
 * <code>false</code> otherwise
 */
public boolean getTopIsDefault()
{
    return(top.isDefault);
}

/**
 * Sets the value of the <code>Left</code> property.
 * The <code>Left</code> property determines the location of the
 * left of the <code>ChartRegion</code>.
 * @param t left value in pixels
 */
public void setLeft(int t)
{
    left.isDefault = false;
    if (left.value == t) {
        return;
    }
    left.value = t;

    if (parent != null) {
        parent.setChanged(true, Changeable.RECALC);
    }
}

/**
 * Gets the value of the <code>Left</code> property.
 * The <code>Left</code> property determines the location of the
 * left of the <code>ChartRegion</code>.
 * @return left value in pixels
 */
public int getLeft()
{
    return(left.value);
}

/**
 * Sets the value of the <code>LeftIsDefault</code> property.
 * The <code>LeftIsDefault</code> property determines whether the left
 * position of the chart region is calculated by Chart (<code>true</code>) or
 * taken from the <code>Left</code> property (<code>false</code>).
 * @param d <code>true</code> if the position is to be calculated,
 * <code>false</code> otherwise
 */
public void setLeftIsDefault(boolean d)
{
    if (left.isDefault == d) {
        return;
    }
    left.isDefault = d;

    if (parent != null) {
        parent.setChanged(true, Changeable.RECALC);
    }
}

/**
 * Gets the value of the <code>LeftIsDefault</code> property.
 * The <code>LeftIsDefault</code> property determines whether the left
 * position of the chart region is calculated by Chart (<code>true</code>) or
 * taken from the <code>Left</code> property (<code>false</code>).
 * @return <code>true</code> if the position is to be calculated,
 * <code>false</code> otherwise
 */
public boolean getLeftIsDefault()
{
    return(left.isDefault);
}

/**
 * Sets the value of the <code>Bottom</code> property.
 * The <code>Bottom</code> property determines the bottom of the
 * <code>ChartRegion</code>.
 * @param t bottom value in pixels
 */
public void setBottom(int t)
{
    bottom.isDefault = false;
    if (bottom.value == t) {
        return;
    }
    bottom.value = t;

    if (parent != null) {
        parent.setChanged(true, Changeable.RECALC);
    }
}

/**
 * Gets the value of the <code>Bottom</code> property.
 * The <code>Bottom</code> property determines the location of the
 * bottom of the <code>ChartRegion</code>.
 * @return bottom value in pixels
 */
public int getBottom()
{
    return(bottom.value);
}

/**
 * Sets the value of the <code>BottomIsDefault</code> property.
 * The <code>BottomIsDefault</code> property determines whether the bottom
 * of the chart region is calculated by Chart (<code>true</code>) or
 * taken from the <code>Bottom</code> property (<code>false</code>).
 * @param d <code>true</code> if the bottom is to be calculated;
 * <code>false</code> otherwise
 */
public void setBottomIsDefault(boolean d)
{
    if (bottom.isDefault == d) {
        return;
    }
    bottom.isDefault = d;

    if (parent != null) {
        parent.setChanged(true, Changeable.RECALC);
    }
}

/**
 * Gets the value of the <code>BottomIsDefault</code> property.
 * The <code>BottomIsDefault</code> property determines whether the bottom
 * of the chart region is calculated by Chart (<code>true</code>) or
 * taken from the <code>Bottom</code> property (<code>false</code>).
 * @return <code>true</code> if the bottom is to be calculated;
 * <code>false</code> otherwise
 */
public boolean getBottomIsDefault()
{
    return(bottom.isDefault);
}

/**
 * Sets the value of the <code>Right</code> property.
 * The <code>Right</code> property determines the right of the
 * <code>ChartRegion</code>.
 * @param t right value in pixels
 */
public void setRight(int t)
{
    right.isDefault = false;
    if (right.value == t) {
        return;
    }
    right.value = t;

    if (parent != null) {
        parent.setChanged(true, Changeable.RECALC);
    }
}

/**
 * Gets the value of the <code>Right</code> property.
 * The <code>Right</code> property determines the location of the
 * right of the <code>ChartRegion</code>.
 * @return right value in pixels
 */
public int getRight()
{
    return(right.value);
}

/**
 * Sets the value of the <code>RightIsDefault</code> property.
 * The <code>RightIsDefault</code> property determines whether the right
 * of the chart region is calculated by Chart (<code>true</code>) or
 * taken from the <code>Right</code> property (<code>false</code>).
 * @param d <code>true</code> if the right is to be calculated;
 * <code>false</code> otherwise
 */
public void setRightIsDefault(boolean d)
{
    if (right.isDefault == d) {
        return;
    }
    right.isDefault = d;

    if (parent != null) {
        parent.setChanged(true, Changeable.RECALC);
    }
}

/**
 * Gets the value of the <code>RightIsDefault</code> property.
 * The <code>RightIsDefault</code> property determines whether the right
 * of the chart region is calculated by Chart (<code>true</code>) or
 * taken from the <code>Right</code> property (<code>false</code>).
 * @return <code>true</code> if the right is to be calculated;
 * <code>false</code> otherwise
 */
public boolean getRightIsDefault()
{
    return(right.isDefault);
}

/**
 * Gets the <code>ChartArea</code> that contains this Plot Area.
 * @return the parent <code>ChartRegion</code>
 */
public ChartRegion getParent()
{
    return(parent);
}

/**
 * Retrieve the image map information for this <code>PlotArea</code>.
 * @return The current image map information.
 */
public ImageMapInfo getImageMapInfo()
{
    return(imageMapInfo);
}

/**
 * Set the image map information for this <code>PlotArea</code>.
 * @param imageMapInfo The new image map information.
 */
public void setImageMapInfo(ImageMapInfo imageMapInfo)
{
    this.imageMapInfo = imageMapInfo;
}

}

