/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PolarAxisParams.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:47 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

/**
 * This class contains size, position, and orientation parameters for Polar/Radar
 * chart axes.
 */
public class PolarAxisParams implements java.io.Serializable {

protected double radius = 0.0;       // radius of theta axis in Polar chart (pixel units)
protected double xCenter = 0.0;
protected double yCenter = 0.0;
protected double originBase = 0.0;   // angle of theta axis origin in POLAR chart (radians)

protected boolean fHalfRange = false;   //  Polar charts only.
protected boolean circularGridLines = true;   //  false for webbed grid lines.  Radar charts only.

/**
 * Method to set the angle of the theta axis origin.
 *
 * @param units The units of the specified angle.  Valid values are
 * <code>JCChartUtil.DEGREES</code>, <code>JCChartUtil.GRADS</code>, or <code>JCChartUtil.RADIANS</code>.
 * @param angle the desired angle for the theta axis origin
 */
protected void setOriginBase(int units, double angle) {
	originBase = JCChartUtil.convertAngle(units, JCChartUtil.RADIANS, angle);
}

/**
 * Method to get the angle of the theta axis origin.
 *
 * @param units The desired units of the returned angle.  Valid values are
 * <code>JCChartUtil.DEGREES</code>, <code>JCChartUtil.GRADS</code>, or <code>JCChartUtil.RADIANS</code>.
 * @return
 */
protected double getOriginBase(int units) {
	return JCChartUtil.convertAngle(JCChartUtil.RADIANS, units, originBase);
}

/**
 * Method to set the half range flag for Polar charts.
 *
 * @param halfRange if <code>true</code>, theta axis labels on a Polar chart will
 * have a maximum value of 180 degrees
 */
protected void setPolarHalfRange(boolean halfRange) {
	fHalfRange = halfRange;
}

/**
 * Method to get the half range flag for Polar charts.
 *
 * @return <code>true</code> if theta axis labels have a maximum value of 180 degrees
 */
protected boolean isPolarHalfRange() {
	return fHalfRange;
}

protected void setCircularGrid(boolean fCircular) {
	circularGridLines = fCircular;
}

protected boolean isCircularGrid() {
	return circularGridLines;
}

}
