/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: RectangularChartAxisHandler.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:47 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.awt.*;
import java.awt.geom.Line2D;

/**
 * A base class for rectangular chart axis handlers.
 */
abstract class RectangularChartAxisHandler extends AxisHandler {

protected void drawTickMark(Graphics gc, JCValueLabel label, int axisLoc)
{
    if (label != null) {
        JCAnno anno = label.getAnno();
        if (anno != null && anno.getDrawTicks()) {
            double err = JCChartUtil.calcError(anno.getPrecision());
            double value = label.getValue();
            if (parent.usingDoublePixels) {
                double pixOff = parent.toPixelDouble(value);
                // Draw the tick only if it lies between the
                // axis min and max.
                if (value >= (parent.min.value - err) &&
                    value <= (parent.max.value + err))
                {
                    drawTick(gc, axisLoc, pixOff, anno);
                }
            }
            else {
                int pixOff = parent.toPixel(value);
                // Draw the tick only if it lies between the
                // axis min and max.
                if (value >= (parent.min.value - err) &&
                    value <= (parent.max.value + err)) {
                    drawTick(gc, axisLoc, pixOff, anno);
                }
            }
        }
    }
}

/**
 * Internal method used to draw a tick mark at a location
 * along the axis.
 * @param gc Graphics context to use for drawing
 * @param axisLoc Location of the axis line
 * @param pixLoc Location of the tick along the axis
 */
protected void drawTick(Graphics gc, int axisLoc,
                        int pixLoc, JCAnno tick)
{
    int innerExtent = (int)Math.round(getInnerExtent(tick));
    int outerExtent = (int)Math.round(getOuterExtent(tick));

    if (parent.reversed) {
        innerExtent *= -1;
        outerExtent *= -1;
    }

    Graphics localGC = gc.create();
    localGC.setColor(getTickColor(tick));

    if (parent.i_vertical) {
        localGC.drawLine(axisLoc + innerExtent, pixLoc,
                         axisLoc - outerExtent, pixLoc);
    }
    else {
        localGC.drawLine(pixLoc, axisLoc - innerExtent,
                         pixLoc, axisLoc + outerExtent);
    }

    localGC.dispose();
}

protected void drawTick(Graphics gc, int axisLoc,
                        double pixLoc, JCAnno tick)
{
    double innerExtent = getInnerExtent(tick);
    double outerExtent = getOuterExtent(tick);

    if (parent.reversed) {
        innerExtent *= -1;
        outerExtent *= -1;
    }

    Graphics2D localGC = (Graphics2D)gc.create();
    localGC.setColor(getTickColor(tick));

    if (parent.i_vertical) {
        localGC.draw(new Line2D.Double(
            (double)axisLoc + innerExtent, pixLoc,
            (double)axisLoc - outerExtent, pixLoc));
    }
    else {
        localGC.draw(new Line2D.Double(
            pixLoc, (double)axisLoc - innerExtent,
            pixLoc, (double)axisLoc + outerExtent));
    }

    localGC.dispose();
}

/**
 * Gets the width and height of an annotation generated for the
 * provided value and precision.  For rectangular charts, the
 * JCAxis.annotationRotation property can have values JCAxis.ROTATE_OTHER,
 * JCAxis.ROTATE_90, JCAxis.ROTATE_180, JCAxis.ROTATE_270, or
 * JCAxis.ROTATE_NONE.
 *
 * @param p Dimension object in which to return the size
 * @param value Value to display
 * @param precision Precision to use to display the value
 */
void getAnnoExtent(Dimension p, double value, int precision)
{
    Dimension dim = getAnnoDimension(value, precision);
    if (dim == null) {
        return;
    }

    if (parent.annotationRotation == JCAxis.ROTATE_OTHER) {
        // Compute the full x and y extents of a label rotated at
        // angle theta.
        int localWidth = 0;
        int localHeight = 0;
        double theta = parent.getAnnotationRotationAngle(JCChartUtil.RADIANS);

        XYProjection proj =
            parent.getAnnotationHandler().getXYProjection(
                                    dim.width, dim.height, theta);

        localWidth = (int)Math.round(proj.x1 + proj.x2);
        if (p.width < localWidth) {
            p.width = localWidth;
        }
        localHeight = (int)Math.round(proj.y1 + proj.y2);
        if (p.height < localHeight) {
            p.height = localHeight;
        }
    }
    else {
        getAnnotationExtent(p, value, precision);
    }
}

abstract double toData(int v);

abstract int toPixel(double v);

abstract double toPixelDouble(double v);

}
