/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: RectangularChartLinearAxisHandler.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:47 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.awt.Dimension;
import java.util.List;

/**
 * A linear axis handler for rectangular charts.
 */
class RectangularChartLinearAxisHandler extends RectangularChartAxisHandler
                                        implements LinearAxisHandler {

RectangularChartLinearAxisHandler()
{
    axisScaleHandler = new LinearAxisScaleHandler();
}

/**
 * Performs pixel-to-data conversion for a linear axis
 * @param v screen value (in pixels) to convert to a data value.  Must
 * be relative to the plot area.
 * @return data value corresponding to the screen location
 */
double toData(int v)
{
    if (parent.i_vertical) {
        return((parent.getHeight() - v - shift) / scale);
    }
    return((v - shift) / scale);
}

/**
 * Performs data-to-screen conversion for a linear axis
 * @param v data value to convert to screen value.
 * @return screen value corresponding to the data value
 */
protected double toScreenLocation(double v)
{
    return(scale * v + shift);
}

/**
 * Performs data-to-pixel conversion for a linear axis
 * @param v data value to convert to a screen location
 * @return screen location in pixels
 */
int toPixel(double v)
{
    v = toScreenLocation(v) + .5;
    int iv;
    if (v >= 0.0) {
        iv = (int)v;
    }
    else {
        iv = (int)Math.floor(v);
    }
    return(parent.i_vertical ? parent.getHeight() - iv : iv);
}

/**
 * Performs data-to-pixel conversion for a linear axis
 * @param v data value to convert to a screen location
 * @return screen location in pixels
 */
double toPixelDouble(double v)
{
    v = toScreenLocation(v);
    return(parent.i_vertical ? parent.getHeight() - v : v);
}

/**
 * Calculates/estimates axis parameters for a linear axis
 * Roughly the equivalent of xrt_calc_parms().
 */
void calcParams()
{
    boolean barCharts = (parent.chartType == JCChart.BAR ||
                         parent.chartType == JCChart.STACKING_BAR);
    boolean hiloCharts = (parent.chartType == JCChart.HILO ||
                          parent.chartType == JCChart.HILO_OPEN_CLOSE);
    boolean candleCharts = (parent.chartType == JCChart.CANDLE);

    double dataMin = (parent.dataLimits == null) ? 0.0 : parent.dataLimits.min;
    double dataMax = (parent.dataLimits == null) ? 100.0 : parent.dataLimits.max;

    double dlmin = dataMin;
    double dlmax = dataMax;

    double lmax = dlmax;
    double lmin = dlmin;
    double org = 0.0;
    double inc = parent.numSpacing.value;
    double tik = parent.tickSpacing.value;

    do {
        calcPrecision();
    }
    while (axisScaleHandler.fixBounds());

    double maxOrig = lmax;
    double minOrig = lmin;

    int tprec = parent.getPrecision();

    lmin = JCChartUtil.precFloor(tprec, dlmin);
    lmax = JCChartUtil.precCeil(tprec, dlmax);

    if (lmax <= lmin) {
        // Move the max. and min. values based on the precision.
        // Keep adjusting the spacing until we get some movement
        //
        //  Note that something very similar to this is also done again later
        //  in this routine for Time labeled axes which could have a 0
        //  data range but for which precision is irrelevant.
        tprec = parent.getPrecision();
        if (parent.max.isDefault) {
            // Move max
            do {
                lmax = (lmin + JCChartUtil.pow10(-tprec));
                tprec--;
            }
            while (lmax <= lmin);
        }
        else {
            // Move min
            do {
                lmin = (lmax - JCChartUtil.pow10(-tprec));
                tprec--;
            }
            while (lmax <= lmin);
        }
    }

    lmin = parent.min.isDefault ? lmin : parent.min.value;
    lmax = parent.max.isDefault ? lmax : parent.max.value;

    org = calcOrigin(lmin, lmax);

    double minReal = parent.min.isDefault ? JCChartUtil.min(lmin, org) : lmin;
    double maxReal = parent.max.isDefault ? JCChartUtil.max(lmax, org) : lmax;

    double error = 0.0;

    boolean pass = false;

    for (int retryCount = 0; retryCount < 2 && !pass; retryCount++) {

        double minUsed = LinearAxisUtil.calcMinUsed(parent, lmin, org);
        double maxUsed = LinearAxisUtil.calcMaxUsed(parent, lmax, org);

        lmin = minUsed;
        lmax = maxUsed;

        error = (0.5 * JCChartUtil.pow10(-parent.getPrecision() - 1));

        inc = parent.numSpacing.value;
        tik = parent.tickSpacing.value;

        // resize the axis as well
        int annoSize = calcMaxAnnoSize(lmin, lmax);

        // Determine tick and numbering spacings
        if (parent.numSpacing.isDefault && parent.tickSpacing.isDefault) {
            // Determine default tick and num spacing.  Give a couple of pixels
            // breathing room around each annotation to counter rounding errors
            // and annotations being placed too close to each other.
            int numTicks = 0;
            if (parent.annotationRotation == JCAxis.ROTATE_OTHER) {
                annoSize = calcRotatedAnnoSpacing(lmin, lmax);
            }
            numTicks = (annoSize == 0) ? 0 :
                parent.pixelLength() / (annoSize + parent.gap + ANNO_PADDING);

            double sz = JCChartUtil.abs(maxUsed - minUsed);
            if (numTicks <= 0) {
                tik = inc = 1.0;
                //  Don't set to 0.0 -- Otherwise, will cause an
                //  infinite recursion.
            }
            else {
                double tmp = sz / numTicks;
                DoubleCoord r =
                    JCChartUtil.calcTik(tmp, parent.getPrecision());
                tik = r.x;
                inc = r.y;
            }
        }
        else {
            // At least one of the two values has been given, so
            // calculate the other.  Note that NumSpacing is sensitive
            // to the precision, but tickSpacing is not.

            if (parent.tickSpacing.isDefault) {
                inc = JCChartUtil.precCorrect(parent.getPrecision(), inc);
                tik = (parent.annotationMethod == JCAxis.VALUE) ?
                      parent.numSpacing.value / 2.0 :
                      parent.numSpacing.value;
            }
            else if (parent.numSpacing.isDefault) {
                inc = 2 * tik;
                inc = JCChartUtil.precCorrect(parent.getPrecision(), inc);
            }
        }

        if (inc != 0) {
            lmin = LinearAxisUtil.updateMinValue(lmin, org, inc);
            lmax = LinearAxisUtil.updateMaxValue(lmax, org, inc);
            if (JCChartUtil.abs(lmax - lmin) < error) {
                lmax = lmin + inc;
            }
        }
        else {
            if (JCChartUtil.abs(lmax - lmin) < error) {
                lmax = lmin + 1;
            }
        }

        // Try this once ... if either of the axis min-max have
        // been moved, it might be too crowded to add these
        // extra labels, so try to determine these values
        // again.
        if ((JCChartUtil.abs(lmin - minUsed) > error) ||
            (JCChartUtil.abs(lmax - maxUsed) > error))
        {
            pass = true;
            if ((lmax - inc) >= maxReal) {
                lmax -= inc;
            }
            else {
                pass = false;
            }
            if ((lmin + inc) <= minReal) {
                lmin += inc;
            }
            else {
                pass = false;
            }
        }
    }

    // For time labels, horizontal precision is ignored
    if (!parent.i_vertical &&
        (parent.annotationMethod == JCAxis.TIME_LABELS))
    {
        lmin = dlmin;
        lmax = dlmax;
        if (lmax <= lmin) {
            tprec = 0;
            do {
                lmax = (lmin + JCChartUtil.pow10(tprec));
                lmin -= lmax - lmin;
                tprec++;
            }
            while (lmax <= lmin);
        }
        minReal = lmin;
        maxReal = lmax;
    }

    //  For point_labels, if axis min/max use_default is true then do not
    //  use the precision corrected xmin or xmax.  We want it to default
    //  to the what it would have been when XRT/graph still had a
    //  discrete axis (i.e. a half_step lower/higher than the data
    //  min/max value where half_step = 0.5 * cg->xaxis_step).
    //
    //  Use the same logic to extend the axis for hilo and candle chart types
    if (parent.dataLimits != null &&
        (parent.annotationMethod == JCAxis.POINT_LABELS ||
        (parent.isPrimary && (barCharts || hiloCharts || candleCharts))))
    {
        double new_min1 = minOrig;
        double new_max1 = maxOrig;
        if (parent.min.isDefault) {
            new_min1 = dlmin - parent.step / 2.0;
        }
        if (parent.max.isDefault) {
            new_max1 = dlmax + parent.step / 2.0;
        }
        lmin = minReal = JCChartUtil.min(minOrig, new_min1);
        lmax = maxReal = JCChartUtil.max(maxOrig, new_max1);
    }

    LinearAxisUtil.setAxisMinMax(parent, lmin, lmax, minReal, maxReal);

    org = calcOrigin(parent.min.value, parent.max.value);
    if (parent.origin.isDefault) {
        parent.origin.value = org;
    }
    if (parent.tickSpacing.isDefault) {
        parent.tickSpacing.value = tik;
    }
    if (parent.numSpacing.isDefault) {
        parent.numSpacing.value = inc;
    }
    List gridList = parent.getGridList();
    if (gridList.size() > 0) {
        JCGrid gridObj = (JCGrid)gridList.get(0);
        if (gridObj != null && gridObj.getIncrementIsDefault()) {
            gridObj.incrementValue.value = parent.numSpacing.value;
        }
    }
}

/**
 * Calculates the value of scale and shift given the axis position
 * and start point. scale and shift are used by toPixel() and toData().
 */
void calcTransformation()
{
    int length = Math.max(parent.pixelLength(), 1);
    computeTransformation(parent.min.value,
                          parent.max.value,
                          length);
}

/**
 * Calculates the maximum annotation size
 */
private int calcMaxAnnoSize(double min, double max)
{
    int maxAnnoSize = -1;
    Dimension d = new Dimension();

    // Find widest string
    if (parent.getAnnotationRotation() == JCAxis.ROTATE_OTHER &&
        parent.getAnnotationMethod() != JCAxis.VALUE) {
        d = parent.getAnnotationHandler().getMaxRotatedAnnoDimensions();
    }
    else {
        getAnnoExtent(d, max, parent.getPrecision());
        getAnnoExtent(d, min, parent.getPrecision());
    }

    // Resize axes.
    if (parent.i_vertical) {
        parent.setSize(d.width + 1 + parent.tickLength / 2, parent.getHeight());
    }
    else {
        parent.setSize(parent.getWidth(), d.height + 1 + parent.tickLength / 2);
    }
    maxAnnoSize = (parent.i_vertical) ? d.height : d.width;

    return(maxAnnoSize);
}

}
