/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: RectangularChartLogAxisHandler.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:47 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.awt.Dimension;

/**
 * A logarithmic axis handler for rectangular charts.
 */
class RectangularChartLogAxisHandler extends RectangularChartAxisHandler
                                     implements LogAxisHandler {

RectangularChartLogAxisHandler()
{
    axisScaleHandler = new LogAxisScaleHandler();
}

/**
 * Performs data-to-pixel conversion for a logarithmic axis
 * @param v screen value (in pixels) to convert to a data value.  Must
 * be relative to the plot area.
 * @return data value corresponding to the screen location
 */
double toData(int v)
{
    if (parent.i_vertical) {
        v = parent.getHeight() - v;
    }
    return(Math.pow(10.0, (v - shift) / scale));
}

/**
 * Performs data-to-pixel conversion for a logarithmic axis
 * @param v data value to convert to a screen location
 * @return screen location in pixels
 */
int toPixel(double v)
{
    if (v <= 0.0) {
        v = parent.min.value;
    }
    int iv;
    iv = JCChartUtil.ifloor(scale * JCChartUtil.log10(v) +
                            shift + 0.5);
    return(parent.i_vertical ? parent.getHeight() - iv : iv);
}

/**
 * Performs data-to-pixel conversion for a logarithmic axis
 * @param v data value to convert to a screen location
 * @return screen location in pixels
 */
double toPixelDouble(double v)
{
    if (v <= 0.0) {
        v = parent.min.value;
    }
    double iv;
    iv = scale * JCChartUtil.log10(v) + shift;
    return(parent.i_vertical ? parent.getHeight() - iv : iv);
}

/**
 * Calculates/estimates axis parameters for a logarithmic axis
 */
void calcParams()
{

    double lmin = (parent.dataLimits == null) ? 1.0 : parent.dataLimits.min;
    double lmax = (parent.dataLimits == null) ? 100.0 : parent.dataLimits.max;
    int startMin = 0;
    int startMax = 0;
    int logMin = 0;
    int logMax = 0;
    int logOrg = 0;
    int logInc = 0;

    lmin = parent.min.isDefault ? lmin : parent.min.value;
    lmax = parent.max.isDefault ? lmax : parent.max.value;

    final int MAX_PRECISION = 10;
    startMin = JCChartUtil.ifloor(
        JCChartUtil.precCorrect(MAX_PRECISION, JCChartUtil.log10(lmin)));
    startMax = JCChartUtil.iceil(JCChartUtil.log10(lmax));

    do {
        calcPrecision();
    }
    while (axisScaleHandler.fixBounds());

    if (parent.i_vertical) {
        if (parent.origin.isDefault || parent.origin.value <= 0.0) {
            if (parent.chartType == JCChart.BAR ||
                parent.chartType == JCChart.STACKING_BAR ||
                parent.chartType == JCChart.STACKING_AREA ||
                parent.chartType == JCChart.AREA)
            {
                startMin--;
            }
        }
        else if (!parent.origin.isDefault && parent.origin.value > 0.0) {
            logOrg = JCChartUtil.ifloor(JCChartUtil.log10(parent.origin.value));
            if (logOrg < startMin && parent.min.isDefault) {
                startMin = logOrg;
            }
            if (logOrg > startMax && parent.max.isDefault) {
                startMax = logOrg;
            }
        }
    }
    else {
        if (!parent.origin.isDefault && parent.origin.value > 0.0) {
            logOrg = JCChartUtil.ifloor(JCChartUtil.log10(parent.origin.value));
            if (logOrg < startMin && parent.min.isDefault) {
                startMin = logOrg;
            }
            if (logOrg > startMax && parent.max.isDefault) {
                startMax = logOrg;
            }
        }
    }

    int annoSize = 0;
    Dimension d = new Dimension();
    int pixelLength = parent.pixelLength();
    for (logInc = 1; logInc <= 5; logInc++) {
        logMin = startMin;
        logMax = startMax;
        boolean on_top = (JCChartUtil.abs(logMax) > JCChartUtil.abs(logMin));
        while ((logMax - logMin) % logInc != 0) {
            if (on_top) {
                logMax++;
            }
            else {
                logMin--;
            }
            on_top = !on_top;
        }
        if (parent.annotationRotation == JCAxis.ROTATE_OTHER) {
            annoSize = calcRotatedAnnoSpacing((double)logMin, (double)logMax);
        }
        else {
            getAnnoExtent(d, (double)logMax, parent.getPrecision());
            getAnnoExtent(d, (double)logMin, parent.getPrecision());
            if (parent.i_vertical) {
                annoSize = (parent.annotationRotation == JCAxis.ROTATE_NONE) ?
                    d.height * 2 : d.width + d.width / 2;
            }
            else {
                annoSize = (parent.annotationRotation == JCAxis.ROTATE_NONE) ?
                    d.width + d.width / 2 : d.height * 2;
            }
        }
        if ((logMax - logMin) / logInc * annoSize < pixelLength) {
            break;
        }
    }
    if (logInc <= 5) {
        // Found a divisor
        parent.tickSpacing.value = JCChartUtil.pow10(logInc);
    }
    else {
        // No divisor.  Increment is the entire range
        logMin = startMin;
        logMax = startMax;
        int diff = Math.max(logMax - logMin, 1);
        parent.tickSpacing.value = JCChartUtil.pow10(diff);
    }
    parent.numSpacing.value = parent.tickSpacing.value;

    if (parent.min.isDefault)
        parent.min.value = JCChartUtil.pow10(logMin);
    if (parent.max.isDefault)
        parent.max.value = JCChartUtil.pow10(logMax);

    if (parent.origin.isDefault || parent.origin.value <= 0.0) {
        switch (parent.originPlacement.value) {
        case JCAxis.MAX:
            parent.origin.value = parent.max.value;
            break;
        case JCAxis.ZERO:
        case JCAxis.MIN:
        case JCAxis.AUTOMATIC:
            parent.origin.value = parent.min.value;
            break;
        }
    }

    if (parent.i_vertical) {
        parent.setSize(annoSize + 1 + parent.tickLength / 2, parent.getHeight());
    }
    else {
        parent.setSize(parent.getWidth(), d.height + 1 + parent.tickLength / 2);
    }
}

/**
 * Calculates the value of scale and shift given the axis position
 * and start point. scale and shift are used by toPixel() and toData().
 */
void calcTransformation()
{
    int length = parent.pixelLength();
    computeTransformation(JCChartUtil.log10(parent.min.value),
                          JCChartUtil.log10(parent.max.value),
                          length);
}

}

