/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: RenderShape.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:47 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Shape;

public class RenderShape implements ProcessShapeModel {

protected Graphics gc = null;
protected JCLineStyle lstyle = null;
protected JCFillStyle fstyle = null;

public RenderShape(Graphics gc, JCLineStyle lstyle, JCFillStyle fstyle) {
	this.gc = gc;
	this.lstyle = lstyle;
	this.fstyle = fstyle;
}

public void setFillStyle(JCFillStyle fstyle) {
	this.fstyle = fstyle;
}

public void updateFillOrientation(Rectangle rect, int newOrient) {
	if (fstyle != null) {
		fstyle.updateFillOrientation(rect, newOrient);
	}
}

public void resetFillOrientation() {
	if (fstyle != null) {
		fstyle.resetFillOrientation();
	}
}

/**
 * Draws a filled polygon using double precision coordinates.
 * @param shape
 */
public void fillPolygon(Shape shape) {
	if (gc != null && fstyle != null) {
		fstyle.fillPolygon(gc, shape);
	}
}

/**
 * Draws a filled polygon in the shadow color using
 * double precision coordinates.
 * @param shape
 */
public void fillPolygonShadow(Shape shape) {
	if (gc != null && fstyle != null) {
		fstyle.fillPolygonShadow(gc, shape);
	}
}

/**
 * Draws a filled arc using double precision coordinates.
 * @param arcAngle
 * @param height
 * @param startAngle
 * @param width
 * @param x
 * @param y
 */
public void fillArc(double x, double y, double width, double height,
					double startAngle, double arcAngle) {
	if (gc != null && fstyle != null) {
		fstyle.fillArc(gc, x, y, width, height, startAngle, arcAngle);
	}
}

/**
 * Draws a filled arc in the shadow color using double precision coordinates.
 * @param arcAngle
 * @param height
 * @param startAngle
 * @param width
 * @param x
 * @param y
 */
public void fillArcShadow(double x, double y, double width, double height,
						  double startAngle, double arcAngle) {
	if (gc != null && fstyle != null) {
		fstyle.fillArcShadow(gc, x, y, width, height, startAngle, arcAngle);
	}
}

/**
 * Draws a double precision line segment.
 * @param x1
 * @param x2
 * @param y1
 * @param y2
 */
public void drawLine(double x1, double y1, double x2, double y2) {
	if (gc != null && lstyle != null) {
		lstyle.drawLine(gc, x1, y1, x2, y2);
	}
}

/**
 * Draws a double precision arc.
 * @param arcAngle
 * @param height
 * @param startAngle
 * @param width
 * @param x
 * @param y
 */
public void drawArc(double x, double y, double width, double height,
					double startAngle, double arcAngle) {
	if (gc != null && lstyle != null) {
		lstyle.drawArc(gc, x, y, width, height, startAngle,
								arcAngle);
	}
}

}
