/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: RotateHandler.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:47 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.awt.Point;

/**
 * Handler object for rotate actions within JCChart.  When
 * an rotate action is started, JCChartArea will create an
 * instance of RotateHandler and call start(), animate() and
 * end() at appropriate points.  The mapping between key
 * combinations and JCChart events is specified by EventTrigger.
 * @see com.klg.jclass.chart.ActionHandler
 * @see com.klg.jclass.chart.EventTrigger
 */
public class RotateHandler extends ActionHandler {

/** Should we be doing a rotation at all? */
protected boolean doRotation = false;

/** Point in chart area at which the rotate action was started */
protected Point first = null;

/** Last point used for rotation */
protected Point last = null;

/** Original elevation value */
protected int orig_elevation;

/** Original rotation value */
protected int orig_rotation;

/** True if the chart being rotated is a pie chart */
protected boolean pieChart = false;

/**
 * Default constructor
 */
protected RotateHandler() {}

/**
 * Called by JCChartArea when the action is started.  For
 * the rotate action, this method sets a series of initial
 * values.
 */
protected void start(int x, int y)
{
    parent.setInAction(true);
    first = new Point(x, y);
    last = null;

    doRotation = true;
    JCChart chart = getChart();
    JCChartArea area = (chart == null) ? null : chart.getChartArea();
    if (parent.getDepth() == 0 || (area != null && !area.getHasBar())) {
        doRotation = false;
    }

    orig_elevation = parent.getElevation();
    orig_rotation = parent.getRotation();
    pieChart = (chart != null &&
                chart.getDataView(0).getChartType() == JCChart.PIE);
}

/**
 * Called by JCChartArea when the action is to be
 * animated.  For the rotate action, this method calculates
 * new rotation and elevation values for the chart area.
 */
protected void animate(int x, int y)
{
    if (doRotation == false) {
        return;
    }

    if (first == null) {
        return;
    }
    if (last == null) {
        last = new Point(x, y);
        return;
    }
    if ((y == last.y) && (x == last.x)) {
        return;
    }

    // Calculate new elevation and rotation based on distance pointer
    // moved relative to the height or width of chartArea

    int dy = y - first.y;
    int dx = x - first.x;
    double xscale =
        2 * JCChartArea.MAX_ROTATION / (double)parent.getPlotRect().width;
    double yscale =
        2 * JCChartArea.MAX_ELEVATION / (double)parent.getPlotRect().height;

    int new_elevation = orig_elevation;
    int new_rotation = orig_rotation;
    if (pieChart) {
        new_elevation -= dy * yscale;
    }
    else {
        new_rotation -= dx * xscale;
        new_elevation += dy * yscale;
    }
    new_elevation = (int)JCChartUtil.clamp((double)new_elevation,
                                           (double)-JCChartArea.MAX_ELEVATION,
                                           (double)JCChartArea.MAX_ELEVATION);
    new_rotation = (int)JCChartUtil.clamp((double)new_rotation,
                                          (double)-JCChartArea.MAX_ROTATION,
                                          (double)JCChartArea.MAX_ROTATION);

    last.x = x;
    last.y = y;

    // If elevation or rotation has changed, set and redraw
    if ((new_elevation != parent.getElevation()) ||
        (new_rotation != parent.getRotation()))
    {
        parent.setElevation(new_elevation);
        parent.setRotation(new_rotation);
    }
}

/**
 * Called by JCChartArea when the chart area has been
 * refreshed and the animation must be redrawn.
 */
protected void reanimate(int x, int y) {}

/**
 * Called by JCChartArea when the action is over.  Does nothing
 * for the rotate action.
 */
protected void end(int x, int y)
{
    parent.setInAction(false);
    JCChart chart = getChart();
    if (chart != null) {
        chart.sendEvent(null);
    }
    if (parent.getFastAction()) {
        parent.setChanged(true, Changeable.RECALC);
    }
}

}
