/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: SBarChartDraw.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:47 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import com.klg.jclass.util.graphics.DefineShape;

import java.awt.*;

/**
 * Drawable object for stacking bar charts.  This object
 * is used for rendering a stacking bar chart based on data
 * contained in the <code>dataObject</code>.
 */
public class SBarChartDraw extends ChartDraw {

/** If width or height of the bar is greater than this value, it will be outlined.*/
public final static int OUTLINE_SENSITIVITY = 3;

// Local copies of data needed for draw(), pick(), and unpick().  They are
// set in recalc().
protected Point        firstLast = null;
protected JCLineStyle  outlineStyle = null;
protected int          x_reversed = 0;
protected int          y_reversed = 0;
protected int          delta_x = 0;
protected int          delta_y = 0;
protected boolean      is_3d = false;
protected double       xAxisPos = Double.MAX_VALUE;
protected double[]     xGridLineList = null;
protected JCLineStyle  gridLineStyle = null;
protected boolean      fastAction = false;
protected boolean      inAction = false;
protected double       yOrigin = 0.0;

/** Array of information representing each bar. */
protected BarPoint[][] pts = null;

/** Actual width of a bar. */
protected double       bar_base;

/** Outer bounds of drawing area. */
protected int          left, right, top, bottom;

/** Height of drawing area. */
protected int          height;

/** Extent of x axis. */
protected MinMax       x;
/** Extent of y axis. */
protected MinMax       y;

/** Extent of x data. */
protected MinMax       datax;
/** Extent of y data. */
protected MinMax       datay;

/** <code>True</code> if portraying data as percentage out of 100. */
protected boolean      is_100_percent;

/** <code>True</code> if an outline is drawn around the bar. */
protected boolean      drawOutline = true;

/** Array of positive stacking bars. */
protected SBarVal[][]  set_pos;

/** Array of negative stacking bars. */
protected SBarVal[][]  set_neg;

/** Array of totals of each point.  Used to calculate total bar height
    or percentages. */
protected double[]     ytotal;
protected double[]     ysum;
protected double[]     ysum_neg;
protected double[][]   ywork;

/** Opposite x or y axis.  0 if <code>false</code>, 1 if <code>true</code>. */
protected int          opp_x, opp_y;

/**
 * Default constructor.
 */
public SBarChartDraw()
{
    super();
}

/**
 * Method used to recalculate extents of related objects.
 */
public void recalc()
{
    super.recalc();
    JCBarChartFormat format = (JCBarChartFormat)dataObject.getChartFormat();
    if (format == null) {
        return;
    }

    is_100_percent = format.is100Percent();
    if (format.clusterWidth == 0) {
        return;
    }

    holeValue = dataObject.getHoleValue();
    if (dataObject.getXLimits() == null) {
        return;
    }
    if (dataObject.getYLimits() == null) {
        return;
    }

    datax = new MinMax(dataObject.getXLimits().min,
                       dataObject.getXLimits().max);
    datay = new MinMax(dataObject.getYLimits().min,
                       dataObject.getYLimits().max);
    dataObject.getDataBounds(datax, datay);

    xaxis = dataObject.getXAxis();
    yaxis = dataObject.getYAxis();
    bar_base = ((dataObject.step * format.clusterWidth) / (100.0));
    inverted = xaxis.i_vertical;

    x = new MinMax();
    y = new MinMax();
    //    If data_min/data_max use_defaults are FALSE, honor those
    //    values, otherwise see if we can extend the bounds to include
    //    half of a cluster.
    x.min = datax.min - bar_base / 2.0;
    x.max = datax.max + bar_base / 2.0;
    xaxis.setLimits(x);
    // Have to recalculate some axis parameters
    xaxis.calcTransformation();
    xaxis.layoutAnnotations();
    xaxis.checkAnnotations();

    x = xaxis.dataLimits;
    y = yaxis.dataLimits;

    // If width of bar is greater than sensitivity, draw an outline around
    // all bars.  The outline needs to be consistently drawn or not drawn
    // for all bars.  Check width from axis min for consistency when bars
    // are translated.
    double xMinValue = xaxis.min.value;
    drawOutline =
        (Math.abs(xaxis.toPixel(xMinValue + bar_base, drawFront) -
                  xaxis.toPixel(xMinValue, drawFront)) > OUTLINE_SENSITIVITY);

    // Compute plotting area limits (this is the front plane in 3D)
    int width =  (inverted ? yaxis.getWidth() : xaxis.getHeight());
    int height = (inverted ? xaxis.getWidth() : yaxis.getHeight());

    left = (inverted ? yaxis.getLeft() : xaxis.getLeft());
    right = left + width - 1;

    int top = (inverted ? xaxis.getTop() : yaxis.getTop());
    //int bottom = top + height - 1;

    // Transform data points to pixel coordinates.  The information
    // is stored in the array "pts".  beg_pix and end_pix are the
    // pixel values of beginning and ending of the bar in the x
    // direction and len_pix in the y direction (even if inverted).
    //
    // "x" and "y" store the x and y values of the bars in data space
    // to distinguish between valid and invalid values (invalid would
    // be holes and ones outside of data and/or axis bounds).

    yOrigin = yaxis.origin.value;
    int pix_org = yaxis.toPixel(yOrigin, drawFront);

    firstLast = dataObject.getFirstLast();
    x_reversed = xaxis.reversed ? 1 : 0;
    y_reversed = yaxis.reversed ? 1 : 0;
    delta_x = chartArea.getDeltaX();
    delta_y = chartArea.getDeltaY();
    is_3d = (delta_x != 0 || delta_y != 0);
    seriesList = dataObject.getSeries();
    int nsets = seriesList.size();
    if ((nsets == 0) || (firstLast == null)) {
        return;
    }
    int npoints = firstLast.y - firstLast.x + 1;

    fastAction = chartArea.getFastAction();
    inAction = chartArea.getInAction();

    if (!fastAction || !inAction) {
        xAxisPos = xaxis.getRealPlacementLocation();

        // Get list of grid line values for the x axis.  If null, make into an
        // empty array
        xGridLineList = xaxis.getGridLineList();
        if (xGridLineList == null) {
            xGridLineList = new double[0];
        }

        gridLineStyle = xaxis.getGridStyle().getLineStyle();
    }

    double xbeg_value, xend_value;
    double yp;

    set_pos = new SBarVal[npoints][nsets];
    set_neg = new SBarVal[npoints][nsets];
    ytotal = new double[npoints];
    ysum = new double[npoints];
    ysum_neg = new double[npoints];
    ywork = new double[nsets][npoints];

    ChartDrawUtil.calcStackingInfo(ysum, ysum_neg, ytotal, ywork,
                                   firstLast.x, firstLast.y, seriesList,
                                   is_100_percent, holeValue, y, yaxis);

    Point[] top_points = new Point[4];
    top_points[0] = new Point(0, 0);
    top_points[1] = new Point(delta_x, delta_y);
    top_points[2] = new Point(0, 0);
    top_points[3] = new Point(-delta_x, -delta_y);
    //top_points[4] = new Point(0, 0);

    Point[] side_points = new Point[4];
    side_points[0] = new Point(0, 0);
    side_points[1] = new Point(delta_x, delta_y);
    side_points[2] = new Point(0, 0);
    side_points[3] = new Point(-delta_x, -delta_y);
    //side_points[4] = new Point(0, 0);

    int pix_x = 0;
    int pix_y = 0;
    int pix_top = 0;
    int pix_side = 0;
    int this_bar_width = 0;
    int this_bar_height = 0;

    pts = new BarPoint[firstLast.y - firstLast.x + 1][nsets];

    if (is_3d) {
        if (inverted) {
            opp_x = ((delta_y > 0 ? 1 : 0) ^ x_reversed);
            opp_y = ((delta_x <= 0 ? 1 : 0) ^ y_reversed);
        }
        else {
            opp_x = ((delta_x <= 0 ? 1 : 0) ^ x_reversed);
            opp_y = ((delta_y > 0 ? 1 : 0) ^ y_reversed);
        }

        for (int i = firstLast.x; i <= firstLast.y; i++) {
            int thisIndex = i - firstLast.x;
            xbeg_value = ((ChartDataViewSeries)seriesList.get(0)).getX(i) -
                         bar_base / 2.0;
            xend_value = xbeg_value + bar_base;
            if (xend_value < x.min           ||
                xbeg_value > x.max           ||
                xend_value < xaxis.min.value ||
                xbeg_value > xaxis.max.value)
            {
                continue;
            }
            if (xbeg_value < x.min) {
                xbeg_value = x.min;
            }
            if (xend_value > x.max) {
                xend_value = x.max;
            }

            // Calculate the total value of each point
            if (inverted) {
                pix_y = xaxis.toPixel(xbeg_value, drawFront);
                this_bar_height = pix_y - xaxis.toPixel(xend_value, drawFront);
                pix_top = (opp_x != 0) ? pix_y : pix_y - this_bar_height;
                top_points[0].y = pix_top;
                side_points[0].y = pix_y - this_bar_height;
                side_points[2].y = this_bar_height;
            }
            else {
                pix_x = xaxis.toPixel(xbeg_value, drawFront);
                this_bar_width =
                    xaxis.toPixel(xend_value, drawFront) - pix_x;
                pix_side = (opp_x != 0) ? pix_x : pix_x + this_bar_width;
                side_points[0].x = pix_side;
                top_points[0].x = pix_x;
                top_points[2].x = this_bar_width;
            }

            // Calculate and save the positions to draw the side and top
            // or bottom faces of the bar.
            int nsets_pos = 0;
            int nsets_neg = 0;
            int last_pixel_pos = pix_org;
            int last_pixel_neg = pix_org;
            SBarVal set_val;
            int this_pixel = 0, last_pixel = 0;
            boolean y_negative = false;

            for (int j = 0; j < nsets; j++) {
                if (ywork[j][thisIndex] == Double.MAX_VALUE) {
                    continue;
                }
                y_negative = (ywork[j][thisIndex] < 0.0);
                if (y_negative) {
                    if (ywork[j][thisIndex] > y.max) {
                        continue;
                    }
                    this_pixel = yaxis.toPixel(Math.max(ywork[j][thisIndex],
                                                        y.min),
                                               drawFront);
                    last_pixel = last_pixel_neg;
                }
                else {
                    if (ywork[j][thisIndex] < y.min) {
                        continue;
                    }
                    this_pixel = yaxis.toPixel(Math.min(ywork[j][thisIndex],
                                                        y.max),
                                               drawFront);
                    last_pixel = last_pixel_pos;
                }

                // Adjust if y-axis is on the back plane
                if (inverted) {
                    if (y_negative) {
                        set_val = set_neg[thisIndex][nsets_neg] = new SBarVal();
                        nsets_neg++;
                    }
                    else {
                        set_val = set_pos[thisIndex][nsets_pos] = new SBarVal();
                        nsets_pos++;
                    }
                    set_val.nset_index = j;
                    set_val.side_point0.x = this_pixel;
                    set_val.side_point2.x = last_pixel - this_pixel;

                    if ((opp_y == 0 && !y_negative) ||
                        (opp_y != 0 && y_negative))
                    {
                        set_val.top_point.x = this_pixel;
                    }
                    else {
                        set_val.top_point.x = last_pixel;
                    }
                }
                else {
                    if (y_negative) {
                        set_val = set_neg[thisIndex][nsets_neg] = new SBarVal();
                        nsets_neg++;
                    }
                    else {
                        set_val = set_pos[thisIndex][nsets_pos] = new SBarVal();
                        nsets_pos++;
                    }
                    set_val.nset_index = j;
                    set_val.side_point0.y = last_pixel;
                    set_val.side_point2.y = this_pixel - last_pixel;

                    if ((opp_y == 0 && !y_negative) ||
                        (opp_y != 0 && y_negative))
                    {
                        set_val.top_point.y = this_pixel;
                    }
                    else {
                        set_val.top_point.y = last_pixel;
                    }
                }
                if (y_negative) {
                    last_pixel_neg = this_pixel;
                }
                else {
                    last_pixel_pos = this_pixel;
                }
            }

            // Draw the side and top or bottom faces of the bar
            int limit_1, limit_2;
            if (opp_y != 0) {
                limit_1 = nsets_pos - 1;
                limit_2 = nsets_neg;
            }
            else {
                limit_1 = nsets_neg - 1;
                limit_2 = nsets_pos;
            }

            BarPoint lp[] = pts[thisIndex];
            for (int k = limit_1; k >= 0; k--) {
                set_val = (opp_y != 0) ? set_pos[thisIndex][k]
                                       : set_neg[thisIndex][k];
                int j = set_val.nset_index;
                if (lp[j] == null) {
                    lp[j] = new BarPoint();
                }
                BarPoint bp = lp[j];
                if (inverted) {
                    top_points[0].x = set_val.side_point0.x;
                    top_points[2].x = set_val.side_point2.x;
                    bp.top = copyPointArray(top_points);
                    side_points[0].x = set_val.top_point.x;
                    bp.side = copyPointArray(side_points);
                }
                else {
                    side_points[0].y = set_val.side_point0.y;
                    side_points[2].y = set_val.side_point2.y;
                    bp.side = copyPointArray(side_points);
                    top_points[0].y = set_val.top_point.y;
                    bp.top = copyPointArray(top_points);
                }
            }
            for (int k = 0; k < limit_2; k++) {
                set_val = (opp_y != 0) ? set_neg[thisIndex][k]
                                       : set_pos[thisIndex][k];
                int j = set_val.nset_index;
                if (lp[j] == null) {
                    lp[j] = new BarPoint();
                }
                BarPoint bp = lp[j];

                if (inverted) {
                    top_points[0].x = set_val.side_point0.x;
                    top_points[2].x = set_val.side_point2.x;
                    bp.top = copyPointArray(top_points);
                    side_points[0].x = set_val.top_point.x;
                    bp.side = copyPointArray(side_points);
                }
                else {
                    side_points[0].y = set_val.side_point0.y;
                    side_points[2].y = set_val.side_point2.y;
                    bp.side = copyPointArray(side_points);
                    top_points[0].y = set_val.top_point.y;
                    bp.top = copyPointArray(top_points);
                }
            }
        }

        if (is_3d) {
            if (inverted) {
                pix_y = (delta_y > 0) ? top + height - 1 : top;
            }
            else {
                pix_x = (delta_x <= 0) ? left : right;
            }
        }
    }

    for (int i = firstLast.x; i <= firstLast.y; i++) {
        if (seriesList.size() == 0) {
            break;
        }
        int thisIndex = i - firstLast.x;

        xbeg_value =
            ((ChartDataViewSeries)seriesList.get(0)).getX(i) - bar_base / 2.0;
        xend_value = xbeg_value + bar_base;
        if (xend_value < x.min           ||
            xbeg_value > x.max           ||
            xend_value < xaxis.min.value ||
            xbeg_value > xaxis.max.value)
        {
            continue;
        }

        if (xbeg_value < x.min) {
            xbeg_value = x.min;
        }
        if (xend_value > x.max) {
            xend_value = x.max;
        }

        //  Draw the bar interior.
        if (inverted) {
            pix_y = xaxis.toPixel(xbeg_value, drawFront);
            this_bar_height =
                Math.abs(xaxis.toPixel(xend_value, drawFront) - pix_y);
            if (x_reversed == 0) {
                pix_y -= this_bar_height;
            }
        }
        else {
            pix_x = xaxis.toPixel(xbeg_value, drawFront);
            this_bar_width =
                Math.abs(xaxis.toPixel(xend_value, drawFront) - pix_x);
            if (x_reversed != 0) {
                pix_x -= this_bar_width;
            }
        }

        // Draw the front face of the bar.
        int last_pixel_pos = pix_org;
        int last_pixel_neg = pix_org;
        int this_pixel = 0, last_pixel = 0;

        BarPoint lp[] = pts[thisIndex];
        for (int j = 0; j < seriesList.size(); j++) {
            if (ywork[j][thisIndex] == Double.MAX_VALUE) {
                continue;
            }

            if (lp[j] == null) {
                lp[j] = new BarPoint();
            }
            BarPoint bp = lp[j];

            yp = ((ChartDataViewSeries)seriesList.get(j)).getY(i);

            bp.x = xbeg_value;
            bp.y = yp;

            boolean y_negative = (ywork[j][thisIndex] < 0.0);

            if (y_negative) {
                if (ywork[j][thisIndex] > y.max) {
                    continue;
                }
                this_pixel = yaxis.toPixel(Math.max(ywork[j][thisIndex],
                                                    y.min),
                                           drawFront);
                last_pixel = last_pixel_neg;
            }
            else {
                if (ywork[j][thisIndex] < y.min) {
                    continue;
                }
                this_pixel = yaxis.toPixel(Math.min(ywork[j][thisIndex],
                                                    y.max),
                                           drawFront);
                last_pixel = last_pixel_pos;
            }

            if (inverted) {
                if ((y_reversed ^ (y_negative ? 1 : 0)) != 0) {
                    this_bar_width = last_pixel - this_pixel;
                    pix_x = this_pixel;
                }
                else {
                    pix_x = last_pixel;
                    this_bar_width = this_pixel - last_pixel;
                }
            }
            else {
                if ((y_reversed ^ (y_negative ? 1 : 0)) != 0) {
                    pix_y = last_pixel;
                    this_bar_height = this_pixel - last_pixel;
                }
                else {
                    pix_y = this_pixel;
                    this_bar_height = last_pixel - this_pixel;
                }
            }

            if (inverted) {
                this_bar_height = Math.max(1, this_bar_height);
            }
            else {
                this_bar_width = Math.max(1, this_bar_width);
            }
            bp.front = new Rectangle(pix_x, pix_y, this_bar_width,
                                     this_bar_height);

            if (y_negative) {
                last_pixel_neg = this_pixel;
            }
            else {
                last_pixel_pos = this_pixel;
            }
        }
    }
}

/**
 * Method used to draw related objects.
 * @param gc
 */
public void draw(Graphics gc)
{
    if (pts == null) {
        return;
    }

    Rectangle clipBounds = gc.getClipBounds();
    if (clipBounds.width == 0 || clipBounds.height == 0) {
        return;
    }

    int nsets = seriesList.size();
    if ((nsets == 0) || (firstLast == null)) {
        return;
    }
    //int npoints = firstLast.y - firstLast.x + 1;

    outlineStyle = dataObject.getOutlineStyle();

    // Draw 3d shadows
    if (is_3d || is_100_percent) {

        int ibeg, inc;
        boolean more_sbars;

        boolean yTroughGridDrawn = false;
        boolean finishTroughAfterBar = false;
        int xGridIndex = 0;

        if (opp_x != 0) {
            ibeg = firstLast.y;
            inc = -1;
            more_sbars = (ibeg >= firstLast.x);
            xGridIndex = xGridLineList.length - 1;
        }
        else {
            ibeg = firstLast.x;
            inc = 1;
            more_sbars = (ibeg <= firstLast.y);
            xGridIndex = 0;
        }

        BarPoint lastBar = null;
        int lastBarIndex = -1;
        for (int i = ibeg; more_sbars; i += inc) {
            more_sbars = (opp_x != 0) ? (i > firstLast.x) : (i < firstLast.y);
            int thisIndex = i - firstLast.x;

            // First pull out the first segment we can find from the current
            // stacking bar.  We need the x axis position for grid line drawing.
            if (!fastAction || !inAction) {
                BarPoint sampleBP = null;
                for (int k = set_neg[thisIndex].length - 1; k >= 0; k--) {
                    if (set_neg[thisIndex][k] != null) {
                        int j = set_neg[thisIndex][k].nset_index;

                        BarPoint bp = null;
                        try {
                            bp = pts[thisIndex][j];
                        }
                        catch (ArrayIndexOutOfBoundsException e) {}

                        if (bp != null && bp.front != null) {
                            sampleBP = bp;
                            break;
                        }
                    }
                }

                if (sampleBP == null) {
                    for (int k = 0; k < set_pos[thisIndex].length &&
                         set_pos[thisIndex][k] != null;
                         k++)
                    {
                        int j = set_pos[thisIndex][k].nset_index;
                        BarPoint bp = null;
                        try {
                            bp = pts[thisIndex][j];
                        }
                        catch (ArrayIndexOutOfBoundsException e) {}
                        if (bp != null && bp.front != null) {
                            sampleBP = bp;
                        }
                    }
                }

                if (sampleBP != null) {

                    // Axis trough grid lines need to be drawn here, when the
                    // bars near the axes are drawn, so that we can achieve
                    // the proper 3D look.

                    // the pixel values of the bar front along the x axis,
                    // used to check if the bar straddles the y axis or not.
                    int bar_x1 = 0;
                    int bar_x2 = 0;
                    if (inverted) {
                        bar_x2 = sampleBP.front.y + sampleBP.front.height;
                        bar_x1 = sampleBP.front.y;
                    }
                    else {
                        bar_x1 = sampleBP.front.x;
                        bar_x2 = sampleBP.front.x + sampleBP.front.width;
                    }
                    lastBar = sampleBP;
                    lastBarIndex = thisIndex;

                    // If y axis trough lines not drawn yet, see if it is
                    // time to draw them

                    // Don't check gridVisible here, check it inside
                    // tryDrawYAxisTroughGrid().  The reason?  We must
                    // check the trough ends the same way we check the
                    // grid lines, and the trough ends are always drawn.
                    if (!yTroughGridDrawn) {
                        int result = tryDrawYAxisTroughGrid(gc, delta_x,
                                                            delta_y, yaxis,
                                                            bar_x1, bar_x2);
                        if (result == TROUGH_DRAWN) {
                            yTroughGridDrawn = true;
                        }
                        else if (result == FINISH_TROUGH_LATER) {
                            finishTroughAfterBar = true;
                            yTroughGridDrawn = true;
                        }
                    }

                    // draw undrawn x axis trough lines up to current bar
                    if (xaxis.gridVisible) {
                        xGridIndex = drawXAxisTroughGrid(gc, delta_x, delta_y,
                                                         xaxis, y_reversed,
                                                         xAxisPos, bar_x1,
                                                         bar_x2,
                                                         ysum[thisIndex],
                                                         ysum_neg[thisIndex],
                                                         xGridLineList,
                                                         xGridIndex, inc,
                                                         gridLineStyle);
                    }
                }
            }

            // Draw the shadows
            SBarVal[][] set_val;
            set_val = (opp_y != 0) ? set_pos : set_neg;
            for (int k = set_val[thisIndex].length - 1; k >= 0; k--) {
                if (set_val[thisIndex][k] != null) {
                    int j = set_val[thisIndex][k].nset_index;
                    BarPoint bp = null;
                    try {
                        bp = pts[thisIndex][j];
                    }
                    catch (ArrayIndexOutOfBoundsException e) {}

                    if (bp != null) {
                        ChartDataViewSeries series =
                            ((ChartDataViewSeries)seriesList.get(j));
                        JCFillStyle fs = series.getStyle().getFillStyle();
                        if (!fs.isCustomStackable()) {
                            int fillOrientation =
                                calcFillOrientation(bp.y, 0.0, inverted,
                                                    yaxis.reversed);
                            if (inverted) {
                                DrawBarFace(gc, bp.top, j, i, true,
                                            fillOrientation, drawOutline);
                                DrawBarFace(gc, bp.side, j, i, false,
                                            fillOrientation, drawOutline);
                            }
                            else {
                                DrawBarFace(gc, bp.side, j, i, false,
                                            fillOrientation, drawOutline);
                                DrawBarFace(gc, bp.top, j, i, true,
                                            fillOrientation, drawOutline);
                            }
                        }
                    }
                }
            }

            set_val = (opp_y != 0) ? set_neg : set_pos;
            for (int k = 0;
                 k < set_val[thisIndex].length && set_val[thisIndex][k] != null;
                 k++)
            {
                int j = set_val[thisIndex][k].nset_index;
                BarPoint bp = null;
                try {
                    bp = pts[thisIndex][j];
                }
                catch (ArrayIndexOutOfBoundsException e) {}
                if (bp != null) {
                    ChartDataViewSeries series =
                        ((ChartDataViewSeries)seriesList.get(j));
                    JCFillStyle fs = series.getStyle().getFillStyle();
                    if (!fs.isCustomStackable()) {
                        int fillOrientation =
                            calcFillOrientation(bp.y, 0.0, inverted,
                                                yaxis.reversed);
                        if (inverted) {
                            DrawBarFace(gc, bp.top, j, i, true,
                                        fillOrientation, drawOutline);
                            DrawBarFace(gc, bp.side, j, i, false,
                                        fillOrientation, drawOutline);
                        }
                        else {
                            DrawBarFace(gc, bp.side, j, i, false,
                                        fillOrientation, drawOutline);
                            DrawBarFace(gc, bp.top, j, i, true,
                                        fillOrientation, drawOutline);
                        }
                    }
                }
            }


            // Finish drawing y axis trough grid lines that draw over
            // bar shadow to simulate 3D
            if (finishTroughAfterBar) {
                finishDrawYAxisTroughGrid(gc, delta_x, delta_y,
                                          yaxis, y_reversed,
                                          ysum_neg[thisIndex], ysum[thisIndex]);
                finishTroughAfterBar = false;
            }
        }

        if (!fastAction || !inAction) {
            // Draw the rest of the y axis grid lines.
            // Don't draw last grid lines and re-outline trough
            // if last bar overhangs axis.
            boolean drawOrNot = true;
            if (lastBar != null) {
                drawOrNot = isDrawFinalGridAndTrough(
                    lastBar,
                    isDrawOverBar(inverted, y_reversed,
                                  delta_y, delta_x, xAxisPos,
                                  ysum[lastBarIndex], ysum_neg[lastBarIndex]),
                    inverted,
                    xaxis.toPixel(xaxis.max.value, drawFront),
                    xaxis.toPixel(xaxis.min.value, drawFront));
            }
            if (drawOrNot) {
                finalDrawAxisTroughGrid(gc,
                                        xaxis, yaxis, opp_x,
                                        yTroughGridDrawn, xGridLineList,
                                        xGridIndex, inc, gridLineStyle);
            }
        }

    }

    // Draw the bar fronts
    for (int i = firstLast.x; i <= firstLast.y; i++) {
        if (seriesList.size() == 0) {
            break;
        }

        int thisIndex = i - firstLast.x;
        for (int j = 0; j < seriesList.size(); j++) {
            ChartDataViewSeries thisSeries =
                (ChartDataViewSeries)seriesList.get(j);
            if ((!thisSeries.isVisible()) ||
                (!thisSeries.isIncluded()))
            {
                continue;
            }

            BarPoint bp = null;
            try {
                bp = pts[thisIndex][j];
            }
            catch (ArrayIndexOutOfBoundsException e) {}

            JCChartStyle cs = thisSeries.getStyle();
            JCFillStyle fs = cs.getFillStyle();
            if (bp != null && bp.front != null) {
                if (fs.isCustomStackable()) {
                    fs.fillImageRect(gc, bp.front.x, bp.front.y,
                                     bp.front.width, bp.front.height,
                                     (bp.y >= 0), inverted);
                }
                else {
                    int newOrient = calcFillOrientation(bp.y, 0.0, inverted,
                                                        yaxis.reversed);
                    fs.updateFillOrientation(new Rectangle(bp.front.x,
                                                           bp.front.y,
                                                           bp.front.width,
                                                           bp.front.height),
                                             newOrient);
//DAM
                    DefineShape defShape = null;
                    if (gc instanceof DefineShape) {
                        defShape = (DefineShape)gc;
                    }

                    // If using DefineShape, begin a shape definition for
                    // this index
                    if (defShape != null) {
                        defShape.beginShape(new JCDataIndex(i, thisSeries));
                    }
                    fs.fillRect(gc, bp.front.x, bp.front.y,
                                bp.front.width, bp.front.height);

                    // if using DefineShape, finish up the shape definition
                    if (defShape != null) {
                        defShape.endShape(true);
                    }
                    if (drawOutline) {
                        fs.drawOutlineRect(gc, bp.front.x, bp.front.y,
                                           bp.front.width, bp.front.height,
                                           outlineStyle);
                    }
                    fs.resetFillOrientation();
                }
            }
        }
    }
}

private Point[] copyPointArray(Point[] in)
{
    int sz = in.length;
    Point[] out = new Point[sz];
    for (int i = 0; i < sz; i++) {
        out[i] = new Point(in[i].x, in[i].y);
    }
    return(out);
}

/**
 * Draws the bar interior.
 * @param gc graphics context for drawing
 * @param points array of points representing bar face
 * @param index index of series represented by this bar
 * @param is_top <code>true</code> if front plane; <code>false</code> if back
 *               plane.
 * @param fillOrientation orient the fill inside the polygon this way
 * @param drawOutline <code>true</code> if bar is drawn with outline
 * @param cluster
 */
void DrawBarFace(Graphics gc, Point[] points, int index, int cluster,
                 boolean is_top, int fillOrientation, boolean drawOutline)
{
    ChartDataViewSeries indexSeries =
        ((ChartDataViewSeries)seriesList.get(index));
    JCFillStyle fstyle = indexSeries.getStyle().getFillStyle();
    int delta_x = chartArea.getDeltaX();
    int delta_y = chartArea.getDeltaY();

    boolean is_hilite = (is_top && delta_y <= 0) || (!is_top && delta_x <= 0);

    DefineShape defShape = null;
    if (gc instanceof DefineShape) {
        defShape = (DefineShape)gc;
    }

    Polygon poly = new Polygon();
    for (int j = 0; j < points.length; j++) {
        if (j == 0) {
            poly.addPoint(points[j].x, points[j].y);
        }
        else {
            poly.addPoint(poly.xpoints[j - 1] + points[j].x,
                          poly.ypoints[j - 1] + points[j].y);
        }
    }

    fstyle.updateFillOrientation(poly.getBounds(), fillOrientation);

    // If using DefineShape, begin a shape definition for this index
    if (defShape != null) {
        defShape.beginShape(
            new JCDataIndex(cluster, indexSeries));
    }

    if (is_hilite) {
        fstyle.fillPolygonHighlight(gc, poly);
    }
    else {
        fstyle.fillPolygonShadow(gc, poly);
    }

    // If using DefineShape, finish up the shape definition
    if (defShape != null) {
        defShape.endShape(true);
    }

    if (drawOutline) {
        fstyle.drawOutlinePolygon(gc, poly, outlineStyle);
    }
    fstyle.resetFillOrientation();
}

/**
 * Calculates the fill orientation for images based on the yval being
 * graphed and the inversion of the axes.  The following code is used
 * by bar and stacking bar charts.  Other drawables may override this
 * method as needed.
 * @param inverted
 * @param origin
 * @param reversed
 * @param yval
 * @return
 */
protected int calcFillOrientation(double yval, double origin,
                                  boolean inverted, boolean reversed)
{
    if (!inverted) {
        if ((!reversed && yval >= origin) || (reversed && yval < origin)) {
            return(JCFillStyle.BOTTOM);
        }
        else {
            return(JCFillStyle.TOP);
        }
    }
    else {
        if ((!reversed && yval >= origin) || (reversed && yval < origin)) {
            return(JCFillStyle.LEFT);
        }
        else {
            return(JCFillStyle.RIGHT);
        }
    }
}

/**
 * Pick the nearest bar to the given point.
 * @param pt point used to find the nearest bar
 * @param focus pick focus used to determine distance between point and bars
 * @return an index to the data point in the series that is represented by
 * the picked bar
 */
public JCDataIndex pick(Point pt, int focus)
{
    Rectangle r = chartArea.getDrawingArea();
    // Make changes to a copy of the Point parameter, in case
    // pick() is called multiple times with the same Point object.
    Point p = (Point)pt.clone();
    p.x -= r.x;
    p.y -= r.y;
    int series = -1;
    int point = -1;
    JCDataIndex di = null;
    int length = Integer.MAX_VALUE;
    int delta_x = chartArea.getDeltaX();
    int delta_y = chartArea.getDeltaY();

    boolean is_3d = (delta_x != 0 || delta_y != 0);

    int nsets = seriesList.size();
    if (nsets == 0 || firstLast == null || pts == null) {
        return(null);
    }
    //int npoints = firstLast.y - firstLast.x + 1;

    // First check distance from 3d shadows in the order that they are drawn
    if (is_3d || is_100_percent) {

        int ibeg, inc;
        boolean more_sbars;

        if (opp_x != 0) {
            ibeg = firstLast.y;
            inc = -1;
            more_sbars = (ibeg >= firstLast.x);
        }
        else {
            ibeg = firstLast.x;
            inc = 1;
            more_sbars = (ibeg <= firstLast.y);
        }

        for (int i = ibeg; more_sbars; i += inc) {
            more_sbars = (opp_x != 0) ? (i > firstLast.x) : (i < firstLast.y);
            int thisIndex = i - firstLast.x;
            BarPoint lp[] = pts[thisIndex];
            for (int j = 0; j < seriesList.size(); j++) {
                if (lp[j] != null) {
                    int len_top =
                        JCChartUtil.distanceTo3dPolygon(p.x, p.y, lp[j].top,
                                                        inverted, focus);
                    int len_side =
                        JCChartUtil.distanceTo3dPolygon(p.x, p.y, lp[j].side,
                                                        inverted, focus);
                    if ((len_top <= length) || (len_side <= length)) {
                        series = j;
                        point = i;
                        length = Math.min(len_side, len_top);
                    }
                }
            }
        }
    }

    // Then check distance to front bar.  If the distance is the same between
    // a shadow and a front bar, the front bar wins out.
    for (int i = firstLast.x; i <= firstLast.y; i++) {
        int thisIndex = i - firstLast.x;
        if (pts[thisIndex] == null) {
            continue;
        }
        for (int j = 0; j < pts[thisIndex].length; j++) {
            if (!((ChartDataViewSeries)seriesList.get(j)).isVisible()) {
                continue;
            }
            BarPoint bp = null;
            try {
                bp = pts[thisIndex][j];
            }
            catch (ArrayIndexOutOfBoundsException e) {}
            if (bp == null || bp.x == holeValue || bp.y == holeValue) {
                continue;
            }
            if (bp.front == null) {
                continue;
            }
            int len = JCChartUtil.distance(p.x, p.y, bp.front);
            if (len <= length) {
                series = j;
                point = i;
                length = len;
            }
        }
    }
    if (series >= 0 && point >= 0) {
        di = new JCDataIndex(point,
                             (ChartDataViewSeries)seriesList.get(series),
                             series);
        di.distance = length;
    }
    return(di);
}

/**
 * Unpick method for stacking bar charts.  Given a data series and a datapoint
 * within that series, the function will return pixel coordinates of
 * that point relative to the chart area.
 * @param pt point index
 * @param seriesIndex data series
 * @return point object representing the position in screen pixels
 */
public Point unpick(int pt, int seriesIndex)
{
    BarPoint bp = null;
    try {
        bp = pts[pt - firstLast.x][seriesIndex];
    }
    catch (ArrayIndexOutOfBoundsException e) {}

    if (bp == null) {
        return(null);
    }

    double yvalue = bp.y;
    double xvalue = bp.x;
    int point_x = -1;
    int point_y = -1;

    if ((yvalue != holeValue) && (xvalue != holeValue)) {
        point_y = bp.front.y + (bp.front.height / 2);
        point_x = bp.front.x + (bp.front.width / 2);
        return(new Point(point_x, point_y));
    }

    return(null);
}

/** Rectangle of drawing area. Used by edit routines.*/
Rectangle r = null;
/** <code>true</code> if axes are inverted. Used by edit routines.*/
boolean editInverted = false;

/**
 * Method called by the <code>EditHandler</code> when an edit is started
 * on the data set plotted by this drawable.
 * @param di <code>JCDataIndex</code> instance specifying the data series and
 * point values for the point being edited
 */
void startEdit(JCDataIndex di)
{
    r = chartArea.getDrawingArea();
    editInverted = di.dataView.getXAxis().i_vertical;
    start = end = null;
}

/** Start point of edit. */
Point start = null;

/** End point of edit. */
Point end = null;

/**
 * Method called by the <code>EditHandler</code> when an edit is occuring
 * on the data set plotted by this drawable.
 * @param di <code>JCDataIndex</code> instance specifying the data series and
 * point values for the point being edited
 * @param x <i>x</i> location of the action, relative to the chart area
 * @param y <i>y</i> location of the action, relative to the chart area
 */
void showEdit(JCDataIndex di, int x, int y)
{
    if (di == null || di.dataView != dataObject) {
        return;
    }
    BarPoint bp = null;
    try {
        bp = pts[di.point - firstLast.x][di.series.getDrawingOrder()];
    }
    catch (ArrayIndexOutOfBoundsException e) {}
    if (bp == null) {
        return;
    }

    if (start == null || end == null) {
        start = new Point(0, 0);
        end = new Point(0, 0);
    }
    else {
        chartArea.drawRubberBand(end, start);
    }
    if (editInverted) {
        start.x = x;
        start.y = bp.front.y + r.y;
        end.x = bp.front.x + r.x;
        end.y = bp.front.y + r.y + bp.front.height;
    }
    else {
        start.x = bp.front.x + r.x;
        start.y = y;
        end.x = bp.front.x + r.x + bp.front.width;
        end.y = bp.front.height + bp.front.y + r.y;
    }
    chartArea.drawRubberBand(end, start);
}

/**
 * Method called by the <code>EditHandler</code> when an edit is finished
 * on the data set plotted by this drawable.
 * @param di <code>JCDataIndex</code> instance specifying the data series and
 * point values for the point being edited
 * @param x <i>x</i> location of the action, relative to the chart area
 * @param y <i>y</i> location of the action, relative to the chart area
 * @return pixel value of edited data
 */
int endEdit(JCDataIndex di, int x, int y)
{
    if (di == null || di.dataView != dataObject) {
        return(-1);
    }
    chartArea.drawRubberBand(end, start);
    BarPoint bp = pts[di.point - firstLast.x][di.series.getDrawingOrder()];
    int axis_pos = yaxis.toPixel(yaxis.origin.value, drawFront);
    int newh;
    if (editInverted ? bp.front.x >= axis_pos : bp.front.y < axis_pos) {
        // If stacking bar above axis
        newh = editInverted ? (x - r.x) - bp.front.x
                            : bp.front.y + bp.front.height - (y - r.y);
    }
    else {
        // If stacking bar below axis
        newh = editInverted ? x - r.x - (bp.front.x + bp.front.width)
                            : bp.front.y - (y - r.y);
    }
    // Fake pixel value by pretending the bar starts at the axis
    // (this is converted to an appropriate data value by the edit handler)
    int off = editInverted ? axis_pos + newh : axis_pos - newh;
    return(off);
}

}
