/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TrackChange.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:47 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.awt.Graphics;
import java.io.IOException;

/**
 * Concrete implementation of the <code>Changeable</code> interface,
 * for use inside <code>JCChart</code>.
 */
abstract public class TrackChange implements Changeable, java.io.Serializable {

/** Boolean representing the value of the <code>Changed</code> property. */
protected boolean  changed     = true;
protected int      changedFlag = Changeable.NO_CHANGE;

/**
 * Method used to recalculate extents of related objects.
 */
abstract public void recalc();

/**
 * Gets the value of the <code>Changed</code> property. The <code>Changed</code> property determines
 * whether the object needs to be recalculated.
 * @return <code>true</code> if the object has been changed and needs
 * recalculation; <code>false</code> otherwise
 */
public boolean getChanged()
{
	return(changed);
}

/**
 * Checks the value of the <code>ChangedFlag</code> property for the specified mask
 * (see <code>Changeable</code> for a list of valid masks). The <code>ChangedFlag</code> property
 * determines the action the object needs to take in regards to the current
 * change.
 * @param mask the mask with which to check
 * @return <code>true</code> if the specified mask exists in the <code>ChangedFlag</code> property;
 * <code>false</code> otherwise
 */
public boolean getChanged(int mask)
{
	return((changedFlag & mask) == mask);
}

/**
 * Gets the value of the <code>Changed</code> property. The <code>Changed</code> property determines
 * whether the object needs to be recalculated.
 * @return <code>true</code> if the object has been changed and needs recalculation; <code>false</code>
 *     otherwise
 */
final public boolean isChanged()
{
	return(getChanged());
}

/**
 * Checks the value of the <code>ChangedFlag</code> property for the specified mask
 * (see <code>Changeable</code> for a list of valid masks). The <code>ChangedFlag</code> property
 * determines the action the object needs to take in regards to the current
 * change.
 * @param mask the mask with which to check
 * @return <code>True</code> if the specified mask exists in the <code>ChangedFlag</code> property;
 *     <code>false</code> otherwise
 */
final public boolean isChanged(int mask)
{
	return(getChanged(mask));
}

/**
 * Sets the value of the <code>Changed</code> property and updates the parent object. The
 * <code>Changed</code> property determines whether the object needs to be recalculated.
 * @param n new value for <code>Changed</code>
 * @param cf the value with which to update the changed flag
 */
public void setChanged(boolean n, int cf)
{
	changed = n;
	if (n) {
		changedFlag |= cf;
	}
	else {
		changedFlag = cf;
	}
}

/**
 * Sets the value of the <code>Changed</code> property. The <code>Changed</code>
 * property determines whether the object needs to be recalculated.
 * @param n new value for <code>Changed</code>
 * @param cf the value with which to update the changed flag
 * @param updateParent if <code>true</code>, the parent is updated; otherwise, the parent
 *     is not updated (the "update" semantics are up to the implementor)
 */
public void setChanged(boolean n, int cf, boolean updateParent)
{
	if (updateParent) {
		setChanged(n, cf);
	}
	else {
		// Note that this code is identical to the setChanged(n, cf) method
		// above. However, do not replace this code with the method call because
		// setChanged(n, cf) is overridden in classes that extend this one.
		changed = n;
		if (n) {
			changedFlag |= cf;
		}
		else {
			changedFlag = cf;
		}
	}
}

/**
 * By default do not batch.
 * @return always return <code>false</code>
 */
public boolean isBatched()
{
	return(false);
}

/**
 * Overrides write object to change the state of the changed flags being
 * saved, so that the chart will actually recalc and draw when re-serialized.
 * @param out the output stream to which the object is to be written
 * @throws IOException
 */
private void writeObject(java.io.ObjectOutputStream out)
     throws IOException
{
    // Save current state first
    boolean oldChanged = changed;
    int oldChangedFlag = changedFlag;

    // Save state that will generate a recalc
    changed = true;
    changedFlag = Changeable.RECALC;
    out.defaultWriteObject();

    // Restore current state
    changed = oldChanged;
    changedFlag = oldChangedFlag;
}

}
