/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TranslateHandler.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:47 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart;

import java.awt.Point;

/**
 * Handler object for translate actions within JCChart.  When
 * an translate action is started, JCChartArea will create an
 * instance of TranslateHandler and call start(), animate() and
 * end() at appropriate points.  The mapping between key
 * combinations and JCChart events is specified by EventTrigger.
 * @see com.klg.jclass.chart.ActionHandler
 * @see com.klg.jclass.chart.EventTrigger
 */
public class TranslateHandler extends ActionHandler {

/** The axis going in the x direction and the axis going in the y direction */
protected JCAxis xAxis, yAxis;

/** Last point for translate action.  Used to determine size of translate */
protected Point last = null;

/** Data ranges */
protected double xRange, yRange;

/**
 * Default constructor
 */
protected TranslateHandler() {}

/**
 * Called by JCChartArea when the action is started.
 * For the translate action, this method calls JCChart.translateStart()
 * and calculates the data ranges along the horizontal and vertical axes.
 */
protected void start(int x, int y)
{
    JCAxis horizActionAxis = parent.getHorizActionAxis();
    JCAxis vertActionAxis = parent.getVertActionAxis();
    if (horizActionAxis != null && vertActionAxis != null) {
        if (horizActionAxis.isVertical() == vertActionAxis.isVertical()) {
            ErrorDialog.raise(parent.getParent(),
                              JCChartBundle.string(JCChartBundle.key45,
                                                   parent.getLocale()));
            return;
        }
    }

    parent.setInAction(true);
    last = new Point(x, y);

    if (horizActionAxis != null) {
        if (horizActionAxis.isVertical()) {
            yAxis = horizActionAxis;
        }
        else {
            xAxis = horizActionAxis;
        }
    }
    if (vertActionAxis != null) {
        if (vertActionAxis.isVertical()) {
            yAxis = vertActionAxis;
        }
        else {
            xAxis = vertActionAxis;
        }
    }

    if (xAxis != null && xAxis.editable && getChart() != null)
        getChart().translateStart(xAxis);
    if (yAxis != null && yAxis.editable && getChart() != null)
        getChart().translateStart(yAxis);

    if (xAxis != null) {
        xRange = xAxis.range();
    }
    if (yAxis != null) {
        yRange = yAxis.range();
    }
}

/**
 * Called by JCChartArea when the action is to be
 * animated.  For the translate action, this method
 * moves the axis limits relative to the distance from the
 * last point.
 */
protected void animate(int x, int y)
{
    if (last == null) {
        return;
    }

    boolean changed = false;
    JCChart chart = getChart();
    if (xAxis != null && xAxis.editable) {
        int xper = 100 * (last.x - x) / parent.getPlotRect().width;
        if (xper != 0) {
            double xTrans = xRange * xper / 100.0;
            if (chart != null) {
                chart.translate(xTrans, xAxis, false);
            }
            changed = true;
            last.x = x;
        }
    }

    if (yAxis != null && yAxis.editable) {
        int yper = 100 * (y - last.y) / parent.getPlotRect().height;
        if (yper != 0) {
            double yTrans = yRange * yper / 100.0;
            if (chart != null) {
                chart.translate(yTrans, yAxis, false);
            }
            changed = true;
            last.y = y;
        }
    }
    if (changed) {
        parent.setChanged(true, Changeable.RECALC);
    }
}

/**
 * Called by JCChartArea when the chart area has been
 * refreshed and the animation must be redrawn.
 */
protected void reanimate(int x, int y) {}

/**
 * Called by JCChartArea when the action is over.  This
 * method does nothing for translate actions.
 */
protected void end(int x, int y)
{
    parent.setInAction(false);
    if (parent.getFastAction()) {
        parent.setChanged(true, Changeable.RECALC);
    }
}

}
