/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AppearanceWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:47 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.ChartRegion;
import com.klg.jclass.chart.PlotArea;
import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.swing.JCSwingTypeConverter;

import java.awt.Color;
import java.awt.Component;

/**
 * Wrapper class for appearance related properties.
 * Used by the <code>MultiChart</code> Bean.
 */
public class AppearanceWrapper 
	implements java.io.Serializable
{

public Color    background;
public Color    foreground;
public boolean  visible;
public boolean  opaque;
public boolean  axisbounding;
public String   border;

// not to be set, just for reference
public int thisobj;
public String prefix; 

public int  bw_def = 0;    	// default value for border width

/**
 * Default constructor.  Members will be empty.
 */
public AppearanceWrapper() {
}

/**
 * Constructor.  Fills members with parameter Strings.
 * @param backgroundString     background color String.  Value should be a
 *                       String representing a color name (for example, "blue") or
 *                       an rgb value represented as "r-g-b".
 * @param foregroundString     foreground color String.  Value should be a
 *                       string representing a color name (for example, "blue") or
 *                       an rgb value represented as "r-g-b".
 * @param visibleBoolean       visible value
 * @param opaqueBoolean        opaque value
 * @param thisBorder           border value
 * @param axisBoundingBoolean  axis bounding box value
 */
public AppearanceWrapper(String backgroundString, String foregroundString,
	boolean visibleBoolean, boolean opaqueBoolean, String thisBorder,
	boolean axisBoundingBoolean)
{
	background = JCSwingTypeConverter.toColor(backgroundString);
	foreground = JCSwingTypeConverter.toColor(foregroundString);
    visible = visibleBoolean;
    opaque = opaqueBoolean;
	border = thisBorder;

    axisbounding = axisBoundingBoolean;

	thisobj = MultiChart.OBJ_UNKNOWN;
	prefix = "";
}

/**
 * Stores the appearance properties from 
 * the specified object.
 * <br><B>Note:</b> Not currently used.  Wrapper value setting is done in the
 * Bean's <code>get...Appearance()</code> methods (for example, <code>getLegendAppearance</code>).
 * @param obj the object that contains the properties
 */
public void setWrapperValues(Object obj)
{
	// not currently used due to serialization problems
}

/**
 * Sets the appearance properties on 
 * the specified object.
 * <br><B>Note:</b> Not currently used.  Property setting is done in the
 * Bean's <code>set...Appearance()</code> methods (for example, <code>setLegendAppearance</code>).
 * @param obj the object that we need to set these values on
 */
public void setPropertyValues(Object obj)
{
	// not currently used due to serialization problems
}

/**
 * Converts properties to String for code generation.
 * @return String containing properties for code generation
 */
public String toString()
{
	String s = "";

	s += "\"" + JCSwingTypeConverter.fromColor(background) + "\"";
	s += ",";
	s += "\"" + JCSwingTypeConverter.fromColor(foreground) + "\"";
	s += ",";
	s += visible;
	s += ",";
	s += opaque;
	s += ",";
	s += "\"" + border + "\"";
	s += ",";
	s += axisbounding;

	return s;
}

public boolean equals(Object obj)
{
	boolean eq = true;
	if (obj instanceof AppearanceWrapper) {
		AppearanceWrapper compareAW = (AppearanceWrapper) obj;
		if (compareAW.background != null && this.background != null) {
			if (!compareAW.background.equals(this.background))
				eq = false;
		} else {
			if (compareAW.background != this.background)
				eq = false;
		}
		if (compareAW.foreground != null && this.foreground != null) {
			if (!compareAW.foreground.equals(this.foreground))
				eq = false;
		} else {
			if (compareAW.foreground != this.foreground)
				eq = false;
		}
		if (compareAW.border != null && this.border != null) {
			if (!compareAW.border.equals(this.border))
				eq = false;
		} else {
			if (compareAW.border != this.border)
				eq = false;
		}
		if (compareAW.visible != this.visible)
			eq = false;
		if (compareAW.opaque != this.opaque)
			eq = false;
		if (compareAW.axisbounding != this.axisbounding)
			eq = false;
	} else {
		eq = false;
	}
	return eq;
}

}
