/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisAnnoEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCChartEnumMappings;

import java.awt.Font;



import java.awt.GraphicsEnvironment;

import java.awt.Insets;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.BorderFactory;




import java.util.Vector;


/**
 * Editor for Axis Annotation Properties.
 */

public class AxisAnnoEditor extends RadioEditor
					  implements ItemListener, ActionListener, FocusListener {

protected JComboBox annometh_ch;
protected JComboBox annorot_ch;
protected JComboBox fname_ch;
protected JComboBox fstyle_ch;
protected JComboBox fsize_ch;
protected JTextField gap_tf;

private String fname_vals[];
static final private String[] fstyle_names = {"plain", "bold", "italic", "bold+iitalic"};
static final private int[] fstyle_vals = {Font.PLAIN, Font.BOLD, Font.ITALIC,
											Font.BOLD+Font.ITALIC};
static final private int[] fsize_vals = {3, 5, 8, 10, 12, 14, 18, 24, 28, 30, 32, 36, 48 };
protected String[] annometh_names;
protected int[] annometh_vals;
protected String[] annorot_names;
protected int[] annorot_vals;

public AxisAnnoEditor() {
	super(MultiChart.AXIS_RADIO_TITLE, MultiChart.AXIS_RADIO_NAMES);

}

protected void setEnumArrays()
{
	annometh_names = 
		new String[JCChartEnumMappings.annotationMethod_i18n_strings.length];
	System.arraycopy(JCChartEnumMappings.annotationMethod_i18n_strings, 
			0, annometh_names, 0,
			JCChartEnumMappings.annotationMethod_i18n_strings.length);
	annometh_vals = JCChartEnumMappings.annotationMethod_values;

 
	annorot_names = JCChartEnumMappings.annRotation_i18n_strings;
	annorot_vals = JCChartEnumMappings.annRotation_values;
}

/**
 * Creates a panel containing the GUI elements for the properties
 * associated with the radio buttons.
 * @return the panel
 */
protected JPanel makePropertyPanel() 
{
    JPanel p2 = new JPanel();
    p2.setBorder(BorderFactory.createTitledBorder(
    	JCChartBeanBundle.string(JCChartBeanBundle.key195)));
	p2.setLayout(new GridLayout(2,1));

    JPanel p3 = new JPanel();
    p3.setBorder(BorderFactory.createTitledBorder(
    	JCChartBeanBundle.string(JCChartBeanBundle.key196)));
	//p3.setLayout(new GridLayout(2,2));
	p3.setLayout(new GridBagLayout());
	GridBagConstraints gbc3  = new GridBagConstraints();
	gbc3.gridwidth = 6;
	gbc3.gridx = GridBagConstraints.RELATIVE;
	gbc3.gridy = 0;
	gbc3.anchor = GridBagConstraints.WEST;
	gbc3.insets = new Insets(4,4,4,4);

    JPanel p4 = new JPanel();
	p4.setLayout(new GridBagLayout());
	GridBagConstraints gbc4  = new GridBagConstraints();
	gbc4.gridwidth = 6;
	gbc4.gridx = GridBagConstraints.RELATIVE;
	gbc4.gridy = 0;
	gbc4.anchor = GridBagConstraints.WEST;
	gbc4.insets = new Insets(4,4,4,4);

	fname_ch = new JComboBox();
	fstyle_ch = new JComboBox();
	fsize_ch = new JComboBox();
	annometh_ch = new JComboBox();
	annorot_ch = new JComboBox();




    // Root through list of valid font names removing the sub-family names
    String tmp_fnames[] = GraphicsEnvironment.getLocalGraphicsEnvironment().
                                                getAvailableFontFamilyNames();
    int num_fnames = 0;
    for (int i = 0; i < tmp_fnames.length; i++) {
        if (tmp_fnames[i].indexOf('.') < 0)
            num_fnames++;
    }
    fname_vals = new String[num_fnames];
    int fname_ind = 0;
    for (int i = 0; i < tmp_fnames.length; i++) {
        if (tmp_fnames[i].indexOf('.') < 0) {
            fname_vals[fname_ind] = new String(tmp_fnames[i]);
            fname_ind++;
        }
    }

	setEnumArrays();

	for (int i = 0; i < fname_vals.length; i++)
		fname_ch.addItem(fname_vals[i]);
	for (int i = 0; i < fstyle_names.length; i++)
		fstyle_ch.addItem(fstyle_names[i]);
	for (int i = 0; i < fsize_vals.length; i++)
		fsize_ch.addItem("" + fsize_vals[i]);
	for (int i = 0; i < annometh_names.length; i++)
		annometh_ch.addItem(annometh_names[i]);
	for (int i = 0; i < annorot_names.length; i++)
		annorot_ch.addItem(annorot_names[i]);

	JLabel namelabel = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key197));
	JLabel stylelabel = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key198));
	JLabel sizelabel = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key199));
	JLabel methlabel = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key200));
	JLabel rotlabel = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key201));
	JLabel gaplabel = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key202));
	gap_tf = new JTextField(5);

	p3.add(namelabel, gbc3);
	p3.add(fname_ch, gbc3);
	p3.add(stylelabel, gbc3);
	p3.add(fstyle_ch, gbc3);
	p3.add(sizelabel, gbc3);
	p3.add(fsize_ch, gbc3);

	p4.add(methlabel, gbc4);
	p4.add(annometh_ch, gbc4);
	p4.add(rotlabel, gbc4);
	p4.add(annorot_ch, gbc4);
	p4.add(gaplabel, gbc4);
	p4.add(gap_tf, gbc4);

    p2.add(p4);

    p2.add(p3);
	add("Center", p2);

	fname_ch.addItemListener(this);
	fstyle_ch.addItemListener(this);
	fsize_ch.addItemListener(this);
	annometh_ch.addItemListener(this);
	annorot_ch.addItemListener(this);
	gap_tf.addActionListener(this);
	gap_tf.addFocusListener(this);

	return p2;
}


 

AxisAnnoWrapper wrap = null;

public void setValue(Object value) {

	if (value != null) {
		try {
			wrap = (AxisAnnoWrapper) value;
		} catch (ClassCastException e) {
		}
	}

	if (wrap == null)
		wrap = new AxisAnnoWrapper(numOfRadios);

	switchValues(radioSel);
}

/**
 * Fills the property sheet with values associated with the newly selected
 * radio button.
 * @param sel the index of the newly selected radio button
 */
protected void switchValues(int sel)
{
	if (sel < 0 || wrap == null)
		return;

	for (int i = 0; i < annometh_vals.length; i++) {
		if (wrap.annometh[sel] == annometh_vals[i]) {

			if (annometh_ch.getSelectedIndex() != i)
				annometh_ch.setSelectedIndex(i);
			break;

		}
	}
	for (int i = 0; i < annorot_vals.length; i++) {
		if (wrap.annorot[sel] == annorot_vals[i]) {

			if (annorot_ch.getSelectedIndex() != i)
				annorot_ch.setSelectedIndex(i);
			break;

		}
	}
	Font tmpFont = wrap.font[sel];
	if (tmpFont != null) {
		for (int i = 0; i < fname_vals.length; i++) {
			String cmpName = tmpFont.getName();
			if (cmpName.equalsIgnoreCase(fname_vals[i])) {

				if (fname_ch.getSelectedIndex() != i)
					fname_ch.setSelectedIndex(i);
				break;

			}
		}
		for (int i = 0; i < fstyle_vals.length; i++) {
			int cmpStyle = tmpFont.getStyle();
			if (cmpStyle == fstyle_vals[i]) {

				if (fstyle_ch.getSelectedIndex() != i)
					fstyle_ch.setSelectedIndex(i);
				break;

			}
		}
		for (int i = 0; i < fsize_vals.length; i++) {
			int cmpSize = tmpFont.getSize();
			if (cmpSize == fsize_vals[i]) {

				if (fsize_ch.getSelectedIndex() != i)
					fsize_ch.setSelectedIndex(i);
				break;

			}
		}
	}
	gap_tf.setText("" + wrap.gap[sel]);
}

public Object getValue() {
	return (Object) wrap;
}

public void setAsText(String text) throws IllegalArgumentException {
    Vector paramVector = BeanTextParser.parse(text);
    if (paramVector.size() != 4) {
        throw new IllegalArgumentException();
    }
    AxisAnnoWrapper setWrapper = null;
    try {
        setWrapper = new AxisAnnoWrapper(
            (String) paramVector.elementAt(0),
            (String) paramVector.elementAt(1),
            (String) paramVector.elementAt(2),
            (String) paramVector.elementAt(3)
            );
        setValue(setWrapper);
        support.firePropertyChange("", null, null);
    } catch (Exception e) {
        throw new IllegalArgumentException();
    }
}

public String getAsText() {
    if (wrap != null)
        return wrap.toString();
    else
        return null;
}

public String getJavaInitializationString() {
	return "new com.klg.jclass.chart.beans.AxisAnnoWrapper(" + wrap + ")";
}

public Dimension getPreferredSize() {
    Dimension dim = super.getPreferredSize();
    dim.width = Math.max(dim.width, 500);
    dim.height += 250;
    return dim;
}

public void itemStateChanged( ItemEvent ev ) {
    super.itemStateChanged(ev);
    Object src;
    if( (src = ev.getSource()) instanceof JComboBox && 
			ev.getStateChange() != ItemEvent.DESELECTED ) {
		checkProperties(src);
    }
}

public void focusGained(FocusEvent ev) {}

public void focusLost(FocusEvent ev) 
{
	Object src = ev.getSource();
	checkProperties(src);
}

public void actionPerformed( ActionEvent ev ) 
{
	Object src = ev.getSource();
	checkProperties(src);
}

public void checkProperties(Object src)
{
	try {
	if (src	== annometh_ch) {

 

		wrap.annometh[radioSel] = annometh_vals[annometh_ch.getSelectedIndex()];
		support.firePropertyChange("", null, null);
	} else if (src	== annorot_ch) {
		wrap.annorot[radioSel] = annorot_vals[annorot_ch.getSelectedIndex()];
		support.firePropertyChange("", null, null);
	} else if (src	== fname_ch || src == fstyle_ch || src == fsize_ch) {
		wrap.font[radioSel] = new Font(
			fname_vals[fname_ch.getSelectedIndex()],
			fstyle_vals[fstyle_ch.getSelectedIndex()],
			fsize_vals[fsize_ch.getSelectedIndex()] );
		support.firePropertyChange("", null, null);
	} else if (src	== gap_tf) {
		setIntProperty(gap_tf.getText(), wrap.gap, radioSel, wrap.gap_def);
		switchValues(radioSel);
	}
	} catch (Exception ex) {
	}
}

/* for testing purposes
public static void main( String args[] ) {
    JFrame frame;
    try {
    java.lang.Class newclass
        = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
    frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
        System.out.println(ex);
    frame = new JFrame();
    }

    AxisAnnoEditor axisAnnoEditor = new AxisAnnoEditor();
    axisAnnoEditor.setBackground( java.awt.Color.lightGray );
	AxisAnnoWrapper thiswrap = new AxisAnnoWrapper(
		"(x1 value_labels)(x2 value)(y1 time_labels)(y2 value_labels)", 
		"", "", "(x1 0)(x2 3)(y1 2)(y2 4)");
    axisAnnoEditor.setValue( thiswrap );
    frame.getContentPane().add( axisAnnoEditor );
    frame.pack();
    java.awt.Dimension bigness = axisAnnoEditor.getPreferredSize();
    frame.setSize( bigness.width + 100, bigness.height + 50 );
    frame.setVisible(true);
}
*/

}
