/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisAnnoWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCChartEnumMappings;

import java.awt.Font;

/**
 * Wrapper class for Axis Annotation properties.  
 * Used by the MultiChart Bean.
 */
public class AxisAnnoWrapper extends RadioAxisWrapper {

// properties
public int annometh[];
public int annorot[];
public Font font[];
public int    gap[];

// default values
protected Font default_font = new Font("helvetica", Font.PLAIN, 12);
public int  gap_def = 3;       

/**
 * Default constructor.  Members will be empty.
 */
public AxisAnnoWrapper() {
	super();
}

/**
 * Constructor.  Allocates space for the arrays but does not fill them.
 * @param size
 */
public AxisAnnoWrapper(int size) {
	super();
	setWrapperArraySize(size);
}

/**
 * Constructor.  Allocates space for the arrays and fills them by parsing
 * series Strings of format <code>(x1 value)(y1 value)(x2 value)(y2 value)</code>.
 * @param annomethString annotation method string in above format.  Possible
 *                       values are <code>Value</code>, <code>Value_Labels</code>,
 * <code>Time_Labels</code>, or <code>Point_Labels</code>.
 * @param annorotString  annotation rotation string in above format.  Possible
 * values are <code>None</code>, <code>90</code>, <code>180</code>, or <code>270</code>.
 * @param fontString     font String in above format.  Values should be
 *                       of the format <i>style-name-pointsize</i>.
 * @param gapString      gap String in above format.  Values are integer
 *                       values.
 */
public AxisAnnoWrapper(String annomethString, String annorotString,
	String fontString, String gapString)
{
    setWrapperArraySize(MultiChart.AXIS_RADIO_NAMES.length);

    setWrapperEnumValues(annometh, new RadioSeries(annomethString),
		JCChartEnumMappings.annotationMethod_strings, 
		JCChartEnumMappings.annotationMethod_values,
		"AnnotationMethod", JCAxis.VALUE);
    setWrapperEnumValues(annorot, new RadioSeries(annorotString),
		JCChartEnumMappings.annRotation_strings, 
		JCChartEnumMappings.annRotation_values,
		"AnnotationRotation", JCAxis.ROTATE_NONE);
    setWrapperFontValues(font, new RadioSeries(fontString));
	setWrapperIntValues(gap, new RadioSeries(gapString));
}

/**
 * Sets the size of the internal arrays that store properties for
 * axes (should be equal to the number of axes).
 * @param size the size of the internal arrays
 */
public void setWrapperArraySize(int size)
{
	annometh = new int[size];
	annorot = new int[size];
	font = new Font[size];
	// Set a default font so that we don't end up with null fonts in the array
	for (int i = 0; i < size; i++) {
		font[i] = default_font;
	}
	gap = new int[size];
}

/**
 * Stores the properties from the specified axis in 
 * internal arrays at the specified index.
 * @param index the index of the array that corresponds to this axis
 * @param axis the axis that contains the properties for this index
 */
public void setWrapperValue(int index, JCAxis axis)
{
	annometh[index] = axis.getAnnotationMethod();
	annorot[index] = axis.getAnnotationRotation();
	font[index] = axis.getFont();
	// Set a default font so that we don't end up with null fonts in the array
	if (font[index] == null) {
		font[index] = default_font;
	}
	gap[index] = axis.getGap();
}

/**
 * Sets the properties on 
 * the specified axis from values stored in internal arrays 
 * at the specified index.
 * @param index the index where the values for this axis are stored
 * @param axis the axis that we need to set these values on
 */
public void setPropertyValue(int index, JCAxis axis)
{
	// Watch out, this call can throw an exception.  If so, put the correct
	// value in the array.
	try {
		axis.setAnnotationMethod(annometh[index]);
	} catch (Exception e) {
		annometh[index] = axis.getAnnotationMethod();
	}
	
	try {
		axis.setAnnotationRotation(annorot[index]);
	} catch (Exception e) {
		annorot[index] = axis.getAnnotationRotation();
	}
	if (!(axis.getFont().equals(font[index]))) {
		axis.setFont(font[index]);
	}
	axis.setGap(gap[index]);
}

/**
 * Converts properties to String for code generation.
 * @return String containing properties for code generation
 */
public String toString()
{
    String s = "";
    s += enumValuesToString(annometh,
		JCChartEnumMappings.annotationMethod_strings, 
			JCChartEnumMappings.annotationMethod_values);
    s += ",";
    s += enumValuesToString(annorot,
		JCChartEnumMappings.annRotation_strings, 
			JCChartEnumMappings.annRotation_values);
    s += ",";
    s += fontValuesToString(font);
    s += ",";
    s += intValuesToString(gap);

    return s;
}

public boolean equals(Object obj)
{
	boolean eq = true;
	if (obj instanceof AxisAnnoWrapper) {
		AxisAnnoWrapper compareWrap = (AxisAnnoWrapper) obj;
		if (compareWrap.annometh != null && this.annometh != null) {
			for (int i = 0; i < compareWrap.annometh.length; i++) {
				if (compareWrap.annometh[i] != this.annometh[i])
					eq = false;
			}
		} else {
			if (compareWrap.annometh != this.annometh)
				eq = false;
		}
		if (compareWrap.annorot != null && this.annorot != null) {
			for (int i = 0; i < compareWrap.annorot.length; i++) {
				if (compareWrap.annorot[i] != this.annorot[i])
					eq = false;
			}
		} else {
			if (compareWrap.annorot != this.annorot)
				eq = false;
		}
		if (compareWrap.gap != null && this.gap != null) {
			for (int i = 0; i < compareWrap.gap.length; i++) {
				if (compareWrap.gap[i] != this.gap[i])
					eq = false;
			}
		} else {
			if (compareWrap.gap != this.gap)
				eq = false;
		}
		if (compareWrap.font != null && this.font != null) {
			for (int i = 0; i < compareWrap.font.length; i++) {
				if (compareWrap.font[i] != null && this.font[i] != null) {
					if (!compareWrap.font[i].equals(this.font[i]))
						eq = false;
				} else {
					if (compareWrap.font[i] != this.font[i])
						eq = false;
				}
			}
		} else {
			if (compareWrap.font != this.font)
				eq = false;
		}

	} else {
		eq = false;
	}
	return eq;
}

}
