/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisGridWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.chart.JCChartStyle;
import com.klg.jclass.chart.JCLineStyle;

import java.awt.Color;

/**
 * Wrapper class for Axis Grid properties.  
 * Used by the MultiChart Bean.
 */
public class AxisGridWrapper extends RadioAxisWrapper {

public boolean showing[];
public double  spacing[];
public int     pattern[];
public int     width[];
public Color   color[];

public boolean spacing_isdef[];

public int  width_def = 1;         	// default value for width
private boolean defs_done = false;  // defaults set yet? 

/**
 * Default constructor.  Members arrays will be empty.
 */
public AxisGridWrapper() {
	super();
}

/**
 * Constructor. Allocates space for the arrays but does not fill them.
 * @param size
 */
public AxisGridWrapper(int size) {
	super();
	setWrapperArraySize(size);
}

/**
 * Constructor.  Allocates space for the arrays and fills them by parsing
 * series Strings of format <code>(x1 value)(y1 value)(x2 value)(y2 value)</code>.
 * @param showingString  showing String in above format.  Values are boolean
 *                       values.
 * @param spacingString  spacing String in above format.  Values are double
 *                       values.
 * @param patternString  pattern String in above format.  Possible values
 * are <code>None</code>, <code>Solid</code>, <code>Long_Dash</code>, <code>Short_Dash</code>,
 * <code>LSL_Dash</code>, or <code>Dash_Dot</code>. <br><B>Note:</b> The pattern property is
 * currently disabled and setting this String will do nothing.
 * @param widthString    width String in above format.  Values are integer
 *                       values.
 * @param colorString    color String in above format.  Values should be a
 *                       String representing a color name (for example, "blue") or
 *                       an rgb value represented as "r-g-b".
 */
public AxisGridWrapper(String showingString, String spacingString,
    String patternString, String widthString, String colorString)
{
    setWrapperArraySize(MultiChart.AXIS_RADIO_NAMES.length);

    setWrapperBooleanValues(showing, new RadioSeries(showingString));
    setWrapperDoubleIsDefValues(spacing, spacing_isdef,
								 new RadioSeries(spacingString));
    setWrapperEnumValues(pattern, new RadioSeries(patternString),
        JCChartEnumMappings.linePattern_strings, 
		JCChartEnumMappings.linePattern_values,
        "AxisGridPattern", JCLineStyle.SOLID);
    setWrapperIntValues(width, new RadioSeries(widthString));
    setWrapperColorValues(color, new RadioSeries(colorString));
}

/**
 * Sets the size of the internal arrays that store properties for
 * axes (should be equal to the number of axes).
 * @param size the size of the internal arrays
 */
public void setWrapperArraySize(int size)
{
	showing = new boolean[size];
	spacing = new double[size];
	pattern = new int[size];
	width = new int[size];
	color = new Color[size];

	spacing_isdef = new boolean[size];
}

/**
 * Stores the properties from 
 * the specified axis in internal arrays at the specified index.
 * @param index the index of the array that corresponds to this axis
 * @param axis the axis that contains the properties for this index
 */
public void setWrapperValue(int index, JCAxis axis)
{
	showing[index] = axis.isGridVisible();
	spacing[index] = axis.getGridSpacing();
	pattern[index] = axis.getGridStyle().getLinePattern();
	width[index] = axis.getGridStyle().getLineWidth();
	color[index] = axis.getGridStyle().getLineColor();

	spacing_isdef[index] = axis.getGridSpacingIsDefault();
}

/**
 * Sets the properties on 
 * the specified axis from values stored in internal arrays 
 * at the specified index.
 * @param index the index where the values for this axis are stored
 * @param axis the axis that we need to set these values on
 */
public void setPropertyValue(int index, JCAxis axis)
{
	axis.setGridVisible(showing[index]);
	
	if (spacing_isdef[index]) {
		axis.setGridSpacingIsDefault(spacing_isdef[index]);
	} else {
		axis.setGridSpacing(spacing[index]);
	}

	try {
		axis.getGridStyle().setLinePattern(pattern[index]);
	} catch (Exception e) {
		pattern[index] = axis.getGridStyle().getLinePattern();
	}

	axis.getGridStyle().setLineWidth(width[index]);
	axis.getGridStyle().setLineColor(color[index]);
}
  
/**
 * Converts properties to String for code generation.
 * @return String containing properties for code generation
 */
public String toString()
{
    String s = "";
    s += booleanValuesToString(showing);
    s += ",";
    s += doubleIsDefValuesToString(spacing, spacing_isdef);
    s += ",";
    s += enumValuesToString(pattern,
        JCChartEnumMappings.linePattern_strings, 
		JCChartEnumMappings.linePattern_values);
    s += ",";
    s += intValuesToString(width);
    s += ",";
    s += colorValuesToString(color);

    return s;
}

public boolean equals(Object obj)
{
	boolean eq = true;
	if (obj instanceof AxisGridWrapper) {
		AxisGridWrapper compareWrap = (AxisGridWrapper) obj;
		if (compareWrap.showing != null && this.showing != null) {
			for (int i = 0; i < compareWrap.showing.length; i++) {
				if (compareWrap.showing[i] != this.showing[i])
					eq = false;
			}
		} else {
			if (compareWrap.showing != this.showing)
				eq = false;
		}
		if (compareWrap.spacing != null && this.spacing != null) {
			for (int i = 0; i < compareWrap.spacing.length; i++) {
				if (!compareWrap.spacing_isdef[i] && !this.spacing_isdef[i]) {
					if (compareWrap.spacing[i] != this.spacing[i])
						eq = false;
				} else if (compareWrap.spacing_isdef[i] != this.spacing_isdef[i]) {
					eq = false;
				}
			}
		} else {
			if (compareWrap.spacing != this.spacing)
				eq = false;
		}
		if (compareWrap.pattern != null && this.pattern != null) {
			for (int i = 0; i < compareWrap.pattern.length; i++) {
				if (compareWrap.pattern[i] != this.pattern[i])
					eq = false;
			}
		} else {
			if (compareWrap.pattern != this.pattern)
				eq = false;
		}
		if (compareWrap.width != null && this.width != null) {
			for (int i = 0; i < compareWrap.width.length; i++) {
				if (compareWrap.width[i] != this.width[i])
					eq = false;
			}
		} else {
			if (compareWrap.width != this.width)
				eq = false;
		}

		if (compareWrap.color != null && this.color != null) {
			for (int i = 0; i < compareWrap.color.length; i++) {
				if (compareWrap.color[i] != null && this.color[i] != null) {
					if (!compareWrap.color[i].equals(this.color[i]))
						eq = false;
				} else {
					if (compareWrap.color[i] != this.color[i])
						eq = false;
				}
			}
		} else {
			if (compareWrap.color != this.color)
				eq = false;
		}

	} else {
		eq = false;
	}
	return eq;
}

}
