/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisMiscEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import java.awt.GridLayout;
import java.awt.Dimension;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import javax.swing.JFrame;
import javax.swing.JCheckBox;
import javax.swing.JRadioButton;
import javax.swing.JPanel;
import javax.swing.BorderFactory;




import java.util.Vector;


/**
 * Editor for Misc Axis Properties.
 */

public class AxisMiscEditor extends RadioEditor
						  implements ItemListener {

protected JCheckBox show_cb;
protected JCheckBox edit_cb;
protected JCheckBox log_cb;

public AxisMiscEditor() {
	super(MultiChart.AXIS_RADIO_TITLE, MultiChart.AXIS_RADIO_NAMES);
}

/**
 * Creates a panel containing the GUI elements for the properties
 * associated with the radio buttons.
 * @return the panel
 */
protected JPanel makePropertyPanel() 
{

    JPanel p2 = new JPanel();
    p2.setBorder(BorderFactory.createTitledBorder(
    	JCChartBeanBundle.string(JCChartBeanBundle.key210)));
	p2.setLayout(new GridLayout(1,3));
	show_cb = new JCheckBox(JCChartBeanBundle.string(JCChartBeanBundle.key191));
	edit_cb = new JCheckBox(JCChartBeanBundle.string(JCChartBeanBundle.key211));
	log_cb = new JCheckBox(JCChartBeanBundle.string(JCChartBeanBundle.key212));

    p2.add(show_cb);
    p2.add(edit_cb);
    p2.add(log_cb);
	add("Center", p2);

	show_cb.addItemListener(this);
	edit_cb.addItemListener(this);
	log_cb.addItemListener(this);

	return p2;
}

AxisMiscWrapper wrap = null;

public void setValue(Object value) {

    if (value != null) {
        try {
			wrap = (AxisMiscWrapper) value;
        } catch (ClassCastException e) {
        }
    }

	if (wrap == null)
		wrap = new AxisMiscWrapper(numOfRadios);

	switchValues(radioSel);
}

/**
 * Fills the property sheet with values associated with the newly selected
 * radio button.
 * @param sel the index of the newly selected radio buttons
 */
protected void switchValues(int sel)
{
	if (sel < 0 || wrap == null)
		return;
	if (show_cb.isSelected() != wrap.showing[sel])
		show_cb.setSelected(wrap.showing[sel]);
	if (edit_cb.isSelected() != wrap.editable[sel])
		edit_cb.setSelected(wrap.editable[sel]);
	if (log_cb.isSelected() != wrap.logarithmic[sel])
		log_cb.setSelected(wrap.logarithmic[sel]);
}

public Object getValue() {
	return (Object) wrap;
}

public void setAsText(String text) throws IllegalArgumentException {
    Vector paramVector = BeanTextParser.parse(text);
    if (paramVector.size() != 3) {
        throw new IllegalArgumentException();
    }
    AxisMiscWrapper setWrapper = null;
    try {
        setWrapper = new AxisMiscWrapper(
            (String) paramVector.elementAt(0),
            (String) paramVector.elementAt(1),
            (String) paramVector.elementAt(2)
            );
        setValue(setWrapper);
        support.firePropertyChange("", null, null);
    } catch (Exception e) {
        throw new IllegalArgumentException();
    }
}

public String getAsText() {
    if (wrap != null)
        return wrap.toString();
    else
        return null;
}

public String getJavaInitializationString() {
	return "new com.klg.jclass.chart.beans.AxisMiscWrapper(" + wrap + ")";
}

public Dimension getPreferredSize() {
    Dimension dim = super.getPreferredSize();
    dim.width = Math.max(dim.width, 400);
    dim.height += 200;
    return dim;
}

// ActionListener method
public void itemStateChanged( ItemEvent ev ) {
	super.itemStateChanged(ev);
	Object src;
    if( (src = ev.getSource()) instanceof JCheckBox ) {
		if (src	== show_cb) {
			wrap.showing[radioSel] = show_cb.isSelected();
			support.firePropertyChange("", null, null);
		} else if (src == edit_cb) { 
			wrap.editable[radioSel] = edit_cb.isSelected();
			support.firePropertyChange("", null, null);
		} else if (src == log_cb) {
			wrap.logarithmic[radioSel] = log_cb.isSelected();
			support.firePropertyChange("", null, null);
		}
	}
}

/* for testing purposes
public static void main( String args[] ) {
    JFrame frame;
    try {
    java.lang.Class newclass
        = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
    frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
        System.out.println(ex);
    frame = new JFrame();
    }

    AxisMiscEditor axisGenEditor = new AxisMiscEditor();
    axisGenEditor.setBackground( java.awt.Color.lightGray );
    //axisGenEditor.setValue( null);
	AxisMiscWrapper thiswrap = new AxisMiscWrapper(4);
	thiswrap.showing[2] = true;
	thiswrap.editable[3] = true;
	thiswrap.logarithmic[0] = true;
    axisGenEditor.setValue( thiswrap );
    frame.getContentPane().add( axisGenEditor );
    frame.pack();
    java.awt.Dimension bigness = axisGenEditor.getPreferredSize();
    frame.setSize( bigness.width + 100, bigness.height + 50 );
    frame.setVisible(true);
}
*/
}
