/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisMiscWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCAxis;

/**
 * Wrapper class for miscellaneous Axis properties.  
 * Used by the MultiChart Bean.
 */
public class AxisMiscWrapper extends RadioAxisWrapper {

public boolean showing[];
public boolean editable[];
public boolean logarithmic[];

/**
 * Default constructor.  Member arrays will be empty.
 */
public AxisMiscWrapper() {
	super();
}

/**
 * Constructor.  Allocates space for the arrays but does not fill them.
 * @param size
 */
public AxisMiscWrapper(int size) {
	super();
	setWrapperArraySize(size);
}

/**
 * Constructor.  Allocates space for the arrays and fills them by parsing
 * series Strings of format <code>(x1 value)(y1 value)(x2 value)(y2 value)</code>.
 * @param showingString      showing String in above format.  Values are
 *                           boolean values.
 * @param editableString     editable String in above format.  Values are
 *                           boolean values.
 * @param logarithmicString  logarithmic String in above format.  Values are
 *                           boolean values.
 */
public AxisMiscWrapper(String showingString, String editableString,
    											String logarithmicString)
{
    setWrapperArraySize(MultiChart.AXIS_RADIO_NAMES.length);

    setWrapperBooleanValues(showing, new RadioSeries(showingString));
    setWrapperBooleanValues(editable, new RadioSeries(editableString));
    setWrapperBooleanValues(logarithmic, new RadioSeries(logarithmicString));
}

/**
 * Sets the size of the internal arrays that store properties for
 * axes (should be equal to the number of axes).
 * @param size the size of the internal arrays
 */
public void setWrapperArraySize(int size)
{
	showing = new boolean[size];
	editable = new boolean[size];
	logarithmic = new boolean[size];
}

/**
 * Stores the properties from 
 * the specified axis in internal arrays at the specified index.
 * @param index the index of the array that corresponds to this axis
 * @param axis the axis that contains the properties for this index
 */
public void setWrapperValue(int index, JCAxis axis)
{
	showing[index] = axis.isVisible();
	editable[index] = axis.isEditable();
	logarithmic[index] = axis.isLogarithmic();
}

/**
 * Sets the properties on 
 * the specified axis from values stored in internal arrays 
 * at the specified index.
 * @param index the index where the values for this axis are stored
 * @param axis the axis that we need to set these values on
 */
public void setPropertyValue(int index, JCAxis axis)
{
	axis.setVisible(showing[index]);
	axis.setEditable(editable[index]);
	axis.setLogarithmic(logarithmic[index]);
}

/**
 * Converts properties to String for code generation.
 * @return String containing properties for code generation
 */
public String toString()
{
    String s = "";
    s += booleanValuesToString(showing);
    s += ",";
    s += booleanValuesToString(editable);
    s += ",";
    s += booleanValuesToString(logarithmic);

    return s;
}

public boolean equals(Object obj)
{
	boolean eq = true;
	if (obj instanceof AxisMiscWrapper) {
		AxisMiscWrapper compareWrap = (AxisMiscWrapper) obj;
		if (compareWrap.showing != null && this.showing != null) {
			for (int i = 0; i < compareWrap.showing.length; i++) {
				if (compareWrap.showing[i] != this.showing[i])
					eq = false;
			}
		} else {
			if (compareWrap.showing != this.showing)
				eq = false;
		}
		if (compareWrap.editable != null && this.editable != null) {
			for (int i = 0; i < compareWrap.editable.length; i++) {
				if (compareWrap.editable[i] != this.editable[i])
					eq = false;
			}
		} else {
			if (compareWrap.editable != this.editable)
				eq = false;
		}
		if (compareWrap.logarithmic != null && this.logarithmic != null) {
			for (int i = 0; i < compareWrap.logarithmic.length; i++) {
				if (compareWrap.logarithmic[i] != this.logarithmic[i])
					eq = false;
			}
		} else {
			if (compareWrap.logarithmic != this.logarithmic)
				eq = false;
		}

	} else {
		eq = false;
	}
	return eq;
}

}
