/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisOrientationEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import java.awt.Insets;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.BorderFactory;




import java.util.Vector;


/**
 * Editor for Orientation Axis Properties.
 */

public class AxisOrientationEditor extends RadioEditor {

protected AxisOrientationAdapter editor;
protected JLabel otheraxis_lb;

protected String[] paxis_names;

public AxisOrientationEditor() {
	super(MultiChart.AXIS_RADIO_TITLE, MultiChart.AXIS_RADIO_NAMES);
}

/**
 * Creates a panel containing the gui elements for the properties 
 * associated with the radio buttons.
 * @return the panel.
 */
protected JPanel makePropertyPanel() 
{
    JPanel p2 = new JPanel();
    p2.setBorder(BorderFactory.createTitledBorder(
    	JCChartBeanBundle.string(JCChartBeanBundle.key213)));
	p2.setLayout(new BorderLayout(1,1));

    JPanel p3 = new JPanel();
	p3.setLayout(new GridLayout(1, 2));
	p3.add(new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key214) + " : ", 
															JLabel.RIGHT));
	otheraxis_lb = new JLabel("  ", JLabel.LEFT);
	p3.add(otheraxis_lb); 
	
	editor = new AxisOrientationAdapter(this);

	p2.add("North", p3);
	p2.add("Center", editor);

	add("Center", p2);

	return p2;
}

AxisOrientationWrapper wrap = null;

public void setValue(Object value) {

    if (value != null) {
        try {
			wrap = (AxisOrientationWrapper) value;
        } catch (ClassCastException e) {
        }
    }

	if (wrap == null)
		wrap = new AxisOrientationWrapper(numOfRadios);

    paxis_names = MultiChart.AXIS_RADIO_NAMES;

	switchValues(radioSel);
}

/**
 * Fills the property sheet with values associated with the newly selected
 * radio button.
 * @param sel the index of the newly selected radio button.
 */
protected void switchValues(int sel)
{
	if (sel < 0 || wrap == null)
		return;
	editor.setSelectedValue(wrap.orientation[sel], true, false);
	if (wrap.other_axis_index[sel] >= 0)
		otheraxis_lb.setText(paxis_names[wrap.other_axis_index[sel]]);
	else
		otheraxis_lb.setText(JCChartBeanBundle.string(JCChartBeanBundle.key215));

	validate();
}

public Object getValue() {
	return (Object) wrap;
}

public void setAsText(String text) throws IllegalArgumentException {
    Vector paramVector = BeanTextParser.parse(text);
    if (paramVector.size() != 1) {
        throw new IllegalArgumentException();
    }
    AxisOrientationWrapper setWrapper = null;
    try {
        setWrapper = new AxisOrientationWrapper(
            (String) paramVector.elementAt(0)
            );
        setValue(setWrapper);
        support.firePropertyChange("", null, null);
    } catch (Exception e) {
        throw new IllegalArgumentException();
    }
}

public String getAsText() {
    if (wrap != null)
        return wrap.toString();
    else
        return null;
}

public String getJavaInitializationString() {
	return "new com.klg.jclass.chart.beans.AxisOrientationWrapper(" + wrap + ")";
}

public Dimension getPreferredSize() {
    Dimension dim = super.getPreferredSize();
    dim.width = Math.max(dim.width, 300);
    dim.height += 300;
    return dim;
}

protected void setProperty(int v)
{
	wrap.orientation[radioSel] = v;
	if (wrap.other_axis_index[radioSel] >= 0)
		wrap.orientation[wrap.other_axis_index[radioSel]] = v;
	wrap.trigger_index = radioSel;
}

/* for testing purposes
public static void main( String args[] ) {
    JFrame frame;
    try {
    java.lang.Class newclass
        = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
    frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
        System.out.println(ex);
    frame = new JFrame();
    }

    AxisOrientationEditor axisOrientEditor = new AxisOrientationEditor();
    axisOrientEditor.setBackground( java.awt.Color.lightGray );
	AxisOrientationWrapper thiswrap = new AxisOrientationWrapper(4);
    axisOrientEditor.setValue( thiswrap );
    frame.getContentPane().add( axisOrientEditor );
    frame.pack();
    java.awt.Dimension bigness = axisOrientEditor.getPreferredSize();
    frame.setSize( bigness.width + 100, bigness.height + 50 );
    frame.setVisible(true);
}
*/
}
