/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisOrientationWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.ChartDataView;

/**
 * Wrapper class for Axis Orientation properties.  
 * Used by the MultiChart Bean.
 */
public class AxisOrientationWrapper extends RadioAxisWrapper {

public int    orientation[];

// Purely informational
public int		 other_axis_index[];

public int		     trigger_index;

/**
 * Default constructor.  Member arrays will be empty.
 */
public AxisOrientationWrapper() {
	super();
}

/**
 * Constructor.  Allocates space for the arrays but does not fill them.
 * @param size
 */
public AxisOrientationWrapper(int size) {
	super();
	setWrapperArraySize(size);
}

/**
 * Constructor.  Allocates space for the arrays and fills them by parsing
 * series Strings of format <code>(x1 value)(y1 value)(x2 value)(y2 value)</code>.
 * @param orientationString  orientation String in above format.  Possible
 * values are: <code>ORIENT_X_NORMAL_Y_NORMAL</code>, <code>ORIENT_INVERTED_X_NORMAL_Y_NORMAL</code>,
 * <code>ORIENT_X_REVERSED_Y_NORMAL</code>, <code>ORIENT_INVERTED_X_REVERSED_Y_NORMAL</code>,
 * <code>ORIENT_X_NORMAL_Y_REVERSED</code>, <code>ORIENT_INVERTED_X_NORMAL_Y_REVERSED</code>,
 * <code>ORIENT_X_REVERSED_Y_REVERSED</code>, <code>ORIENT_INVERTED_X_REVERSED_Y_REVERSED</code>.
 */
public AxisOrientationWrapper(String orientationString)
{
	setWrapperArraySize(MultiChart.AXIS_RADIO_NAMES.length);
	
	setWrapperEnumValues(orientation, new RadioSeries(orientationString),
		BaseChart.axesOrientationStrings, BaseChart.axesOrientationValues,
		"AxisOrientation", BaseChart.ORIENT_X_NORMAL_Y_NORMAL);
}

/**
 * Sets the size of the internal arrays that store properties for
 * axes (should be equal to the number of axes).
 * @param size the size of the internal arrays
 */
public void setWrapperArraySize(int size)
{
	orientation = new int[size];

	//other_axis = new String[size];
	other_axis_index = new int[size];
}

/**
 * Stores the properties from 
 * the specified axis in internal arrays at the specified index.
 * <br><B>Note:</b> This is no longer used.  Wrapper values are set from inside
 * the MultiChart Bean.
 * @param index the index of the array that corresponds to this axis
 * @param axis the axis that contains the properties for this index
 */
public void setWrapperValue(int index, JCAxis axis)
{
	// not currently used due to serialization problems
}

/**
 * Sets the properties on 
 * the specified axis from values stored in internal arrays 
 * at the specified index.
 * <br><B>Note:</b> This is no longer used.  Property values are set from inside
 * the MultiChart Bean.
 * @param index the index where the values for this axis are stored
 * @param axis the axis that we need to set these values on
 */
public void setPropertyValue(int index, JCAxis axis)
{
	// not currently used due to serialization problems
}

/**
 * Converts properties to String for code generation.
 * @return String containing properties for code generation
 */
public String toString()
{
	String s = "";
	s += enumValuesToString(orientation,
		BaseChart.axesOrientationStrings, BaseChart.axesOrientationValues);

	return s;
}

public boolean equals(Object obj)
{
	boolean eq = true;
	if (obj instanceof AxisOrientationWrapper) {
		AxisOrientationWrapper compareWrap = (AxisOrientationWrapper) obj;
		if (compareWrap.orientation != null && this.orientation != null) {
			for (int i = 0; i < compareWrap.orientation.length; i++) {
				if (compareWrap.orientation[i] != this.orientation[i])
					eq = false;
			}
		} else {
			if (compareWrap.orientation != this.orientation)
				eq = false;
		}

	} else {
		eq = false;
	}
	return eq;
}

}
