/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisOriginEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCChartEnumMappings;

import java.awt.Insets;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.BorderFactory;




import java.util.Vector;


/**
 * Editor for Axis Origin Properties.
 */

public class AxisOriginEditor extends RadioEditor
					  implements ItemListener, ActionListener, FocusListener {

protected JTextField origin_tf;
protected JComboBox placement_ch;

protected String[] placement_names;
protected int[] placement_vals;

public AxisOriginEditor() {
	super(MultiChart.AXIS_RADIO_TITLE, MultiChart.AXIS_RADIO_NAMES);

}

protected void setEnumArrays()
{
	placement_names = 
		new String[JCChartEnumMappings.originPlacement_i18n_strings.length];
	System.arraycopy(JCChartEnumMappings.originPlacement_i18n_strings, 
		0, placement_names, 0,
		JCChartEnumMappings.originPlacement_i18n_strings.length);
	placement_vals = JCChartEnumMappings.originPlacement_values;

 
}

/**
 * Creates a panel containing the GUI elements for the properties
 * associated with the radio buttons.
 * @return the panel
 */
protected JPanel makePropertyPanel() 
{

    JPanel p2 = new JPanel();
    p2.setBorder(BorderFactory.createTitledBorder(
    	JCChartBeanBundle.string(JCChartBeanBundle.key216)));
	p2.setLayout(new GridBagLayout());
	GridBagConstraints gbc2  = new GridBagConstraints();
	gbc2.gridwidth = 2;
	gbc2.gridx = GridBagConstraints.RELATIVE;
	gbc2.gridy = 0;
	gbc2.anchor = GridBagConstraints.WEST;
	gbc2.insets = new Insets(4,4,4,4);


	placement_ch = new JComboBox();
	JLabel origin_label = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key217));
	JLabel placement_label = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key218));
	origin_tf = new JTextField(5);

	setEnumArrays();

	for (int i = 0; i < placement_names.length; i++)
		placement_ch.addItem(placement_names[i]);
	placement_ch.addItem(JCChartBeanBundle.string(JCChartBeanBundle.key219));

	p2.add(origin_label, gbc2);
	p2.add(origin_tf, gbc2);
	gbc2.gridy++;
	p2.add(placement_label, gbc2);
	p2.add(placement_ch, gbc2);


	add("Center", p2);

	origin_tf.addActionListener(this);
	origin_tf.addFocusListener(this);
	placement_ch.addItemListener(this);

	return p2;
}


 

AxisOriginWrapper wrap = null;

public void setValue(Object value) {

    if (value != null) {
        try {
			wrap = (AxisOriginWrapper) value;
        } catch (ClassCastException e) {
        }
    }

	if (wrap == null)
		wrap = new AxisOriginWrapper(numOfRadios);

	switchValues(radioSel);
}

/**
 * Fills the property sheet with values associated with the newly selected
 * radio button.
 * @param sel the index of the newly selected radio button
 */
protected void switchValues(int sel)
{
	if (sel < 0 || wrap == null)
		return;

	if (wrap.origin[sel] != Double.MAX_VALUE)
		origin_tf.setText("" + wrap.origin[sel]);
	else
		origin_tf.setText("");

    int check_placement = -1;
    // default == AUTOMATIC
    if (wrap.placement_isdef[sel]) {
        check_placement = JCAxis.AUTOMATIC;
    } else {
        check_placement = wrap.placement[sel];
    }
	for (int i = 0; i < placement_vals.length; i++) {
		if (check_placement == placement_vals[i]) {
			if (placement_ch.getSelectedIndex() != i)
				placement_ch.setSelectedIndex(i);
			break;
		}
	}
	placement_ch.setEnabled(wrap.origin_isdef[sel]);
}

public Object getValue() {
	return (Object) wrap;
}

public void setAsText(String text) throws IllegalArgumentException {
    Vector paramVector = BeanTextParser.parse(text);
    if (paramVector.size() != 2) {
        throw new IllegalArgumentException();
    }
    AxisOriginWrapper setWrapper = null;
    try {
        setWrapper = new AxisOriginWrapper(
            (String) paramVector.elementAt(0),
            (String) paramVector.elementAt(1)
            );
        setValue(setWrapper);
        support.firePropertyChange("", null, null);
    } catch (Exception e) {
        throw new IllegalArgumentException();
    }
}

public String getAsText() {
    if (wrap != null)
        return wrap.toString();
    else
        return null;
}

public String getJavaInitializationString() {
	return "new com.klg.jclass.chart.beans.AxisOriginWrapper(" + wrap + ")";
}

public Dimension getPreferredSize() {
    Dimension dim = super.getPreferredSize();
    dim.width = Math.max(dim.width, 400);
    dim.height += 200;
    return dim;
}

public void itemStateChanged( ItemEvent ev ) {
    super.itemStateChanged(ev);
    Object src;
    if( (src = ev.getSource()) instanceof JComboBox &&
			ev.getStateChange() != ItemEvent.DESELECTED) {
		checkProperties(src);
    }
}

public void focusGained(FocusEvent ev) {}

public void focusLost(FocusEvent ev) 
{
	Object src = ev.getSource();
	checkProperties(src);
}

public void actionPerformed( ActionEvent ev ) 
{
	Object src = ev.getSource();
	checkProperties(src);
}

public void checkProperties(Object src)
{
	try {

	if (src	== origin_tf) {

 
		setDoubleIsDefProperty(origin_tf.getText(),
			wrap.origin, wrap.origin_isdef, radioSel);
		switchValues(radioSel);
	} else if (src	== placement_ch) {
		int select = placement_ch.getSelectedIndex();

 
		// selection not in value array -- return to default
		if (select >= placement_vals.length) {
			if (!wrap.placement_isdef[radioSel]) {
				wrap.placement_isdef[radioSel] = true;
				support.firePropertyChange("", null, null);
			}
		// accept selected value
		} else {
			if (wrap.placement[radioSel] != placement_vals[select]) {
				wrap.placement[radioSel] = placement_vals[select];
				wrap.placement_isdef[radioSel] = false;
				support.firePropertyChange("", null, null);
			}
		}
	}
	} catch (Exception ex) {
	}
	switchValues(radioSel);
}

/* for testing purposes
public static void main( String args[] ) {
    JFrame frame;
    try {
    java.lang.Class newclass
        = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
    frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
        System.out.println(ex);
    frame = new JFrame();
    }

    AxisOriginEditor axisOriginEditor = new AxisOriginEditor();
    axisOriginEditor.setBackground( java.awt.Color.lightGray );
	AxisOriginWrapper thiswrap = new AxisOriginWrapper("", "");
    axisOriginEditor.setValue( thiswrap );
    frame.getContentPane().add( axisOriginEditor );
    frame.pack();
    java.awt.Dimension bigness = axisOriginEditor.getPreferredSize();
    frame.setSize( bigness.width + 100, bigness.height + 50 );
    frame.setVisible(true);
}
*/

}
