/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisOriginWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCChartEnumMappings;

import java.awt.Font;

/**
 * Wrapper class for Axis Origin properties.  
 * Used by the MultiChart Bean.
 */
public class AxisOriginWrapper extends RadioAxisWrapper {

public double origin[];
public int    placement[];

public boolean origin_isdef[];
public boolean placement_isdef[];

/**
 * Default constructor.  Members will be empty.
 */
public AxisOriginWrapper() {
	super();
}

/**
 * Constructor.  Allocates space for the arrays but does not fill them.
 * @param size
 */
public AxisOriginWrapper(int size) {
	super();
	setWrapperArraySize(size);
}
/**
 * Constructor.  Allocates space for the arrays and fills them by parsing
 * series Strings of format <code>(x1 value)(y1 value)(x2 value)(y2 value)</code>.
 * @param originString    origin location String in above format.  Values
 *                        are double values.
 *                        <p>This property has intelligent defaults.  If a
 *                        value is not provided for an axis, it will 
 *                        use them.
 * @param placementString origin placement String in above format.  Possible
 * values are <code>Automatic</code>, <code>Zero</code>, <code>Min</code>, or <code>Max</code>.
 *                        This property has intelligent defaults.  If a 
 *                        value is not provided for an axis, it will 
 *                        use them.
 */
public AxisOriginWrapper(String originString, String placementString)
{
    setWrapperArraySize(MultiChart.AXIS_RADIO_NAMES.length);

    setWrapperDoubleIsDefValues(origin, origin_isdef,
									 new RadioSeries(originString));
    setWrapperEnumIsDefValues(placement, placement_isdef,
		new RadioSeries(placementString),
        JCChartEnumMappings.originPlacement_strings, 
		JCChartEnumMappings.originPlacement_values,
        "AxisOriginPlacement", JCAxis.AUTOMATIC);
}
/**
 * Sets the size of the internal arrays that store properties for
 * axes (should be equal to the number of axes).
 * @param size the size of the internal arrays
 */
public void setWrapperArraySize(int size)
{
	origin = new double[size];
	placement = new int[size];

	origin_isdef = new boolean[size];
	placement_isdef = new boolean[size];
}

/**
 * Stores the properties from 
 * the specified axis in internal arrays at the specified index.
 * @param index the index of the array that corresponds to this axis
 * @param axis the axis that contains the properties for this index
 */
public void setWrapperValue(int index, JCAxis axis)
{
	origin[index] = axis.getOrigin();
	placement[index] = axis.getOriginPlacement();

	origin_isdef[index] = axis.getOriginIsDefault();
	placement_isdef[index] = axis.getOriginPlacementIsDefault();
}

/**
 * Sets the properties on 
 * the specified axis from values stored in internal arrays 
 * at the specified index.
 * @param index the index where the values for this axis are stored
 * @param axis the axis that we need to set these values on
 */
public void setPropertyValue(int index, JCAxis axis)
{
	if (origin_isdef[index]) {
		axis.setOriginIsDefault(origin_isdef[index]);
	} else {
		axis.setOrigin(origin[index]);
	}

	if (placement_isdef[index]) {
		axis.setOriginPlacementIsDefault(placement_isdef[index]);
	} else {
		try {
			axis.setOriginPlacement(placement[index]);
		} catch (Exception e) {
			placement[index] = axis.getOriginPlacement();
		}
	}
}

/**
 * Converts properties to String for code generation.
 * @return String containing properties for code generation.
 */
public String toString()
{
    String s = "";
    s += doubleIsDefValuesToString(origin, origin_isdef);
    s += ",";
    s += enumIsDefValuesToString(placement, placement_isdef,
        JCChartEnumMappings.originPlacement_strings, 
		JCChartEnumMappings.originPlacement_values);

    return s;
}

public boolean equals(Object obj)
{
	boolean eq = true;
	if (obj instanceof AxisOriginWrapper) {
		AxisOriginWrapper compareWrap = (AxisOriginWrapper) obj;
		if (compareWrap.origin != null && this.origin != null) {
			for (int i = 0; i < compareWrap.origin.length; i++) {
				if (!compareWrap.origin_isdef[i] && !this.origin_isdef[i]) {
					if (compareWrap.origin[i] != this.origin[i])
						eq = false;
				} else if (compareWrap.origin_isdef[i] != this.origin_isdef[i]) {
					eq = false;
				}
			}
		} else {
			if (compareWrap.origin != this.origin)
				eq = false;
		}
		if (compareWrap.placement != null && this.placement != null) {
			for (int i = 0; i < compareWrap.placement.length; i++) {
				if (!compareWrap.placement_isdef[i] && !this.placement_isdef[i]) {
					if (compareWrap.placement[i] != this.placement[i])
						eq = false;
				} else if (compareWrap.placement_isdef[i] != this.placement_isdef[i]) {
					eq = false;
				}
			}
		} else {
			if (compareWrap.placement != this.placement)
				eq = false;
		}

	} else {
		eq = false;
	}
	return eq;
}

}
