/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisPlacementEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCChartEnumMappings;

import java.awt.Insets;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.BorderFactory;




import java.util.Vector;


/**
 * Editor for Axis Placement Properties.
 */

public class AxisPlacementEditor extends RadioEditor
					  implements ItemListener, ActionListener, FocusListener {

protected JComboBox placement_ch;
protected JTextField location_tf;
protected JLabel    paxis_lb;
protected JComboBox paxis_ch;

protected String[] placement_names;
protected int[] placement_vals;
protected String[] paxis_names;

public AxisPlacementEditor() {
	super(MultiChart.AXIS_RADIO_TITLE, MultiChart.AXIS_RADIO_NAMES);

}

protected void setEnumArrays()
{
	placement_names = new String[JCChartEnumMappings.placement_i18n_strings.length];
	System.arraycopy(JCChartEnumMappings.placement_i18n_strings, 
		0, placement_names, 0,
		JCChartEnumMappings.placement_i18n_strings.length);
	placement_vals = JCChartEnumMappings.placement_values;

 

    paxis_names = new String[MultiChart.AXIS_RADIO_NAMES.length];
    System.arraycopy(MultiChart.AXIS_RADIO_NAMES, 0, paxis_names, 0,
    								MultiChart.AXIS_RADIO_NAMES.length);
}

/**
 * Creates a panel containing the GUI elements for the properties
 * associated with the radio buttons.
 * @return the panel
 */
protected JPanel makePropertyPanel() 
{

    JPanel p2 = new JPanel();
    p2.setBorder(BorderFactory.createTitledBorder(
    	JCChartBeanBundle.string(JCChartBeanBundle.key221)));
	p2.setLayout(new GridBagLayout());
	GridBagConstraints gbc2  = new GridBagConstraints();
	gbc2.gridwidth = 2;
	gbc2.gridx = GridBagConstraints.RELATIVE;
	gbc2.gridy = 0;
	gbc2.anchor = GridBagConstraints.WEST;
	gbc2.insets = new Insets(4,4,4,4);

	placement_ch = new JComboBox();
	paxis_ch = new JComboBox();

	setEnumArrays();

	for (int i = 0; i < placement_names.length; i++)
		placement_ch.addItem(placement_names[i]);
	placement_ch.addItem(JCChartBeanBundle.string(JCChartBeanBundle.key219));
	for (int i = 0; i < paxis_names.length; i++)
		paxis_ch.addItem(paxis_names[i]);
	paxis_ch.addItem(JCChartBeanBundle.string(JCChartBeanBundle.key215));

	JLabel placement_label = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key222));
	paxis_lb = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key223));
	JLabel location_label = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key224));
	location_tf = new JTextField(5);

	p2.add(placement_label, gbc2);
	p2.add(placement_ch, gbc2);
	gbc2.gridy++;
	p2.add(paxis_lb, gbc2);
	p2.add(paxis_ch, gbc2);
	gbc2.gridy++;
	p2.add(location_label, gbc2);
	p2.add(location_tf, gbc2);

	add("Center", p2);

	placement_ch.addItemListener(this);
	paxis_ch.addItemListener(this);
	location_tf.addActionListener(this);
	location_tf.addFocusListener(this);

	return p2;
}


 

AxisPlacementWrapper wrap = null;

public void setValue(Object value) {

    if (value != null) {
        try {
			wrap = (AxisPlacementWrapper) value;
        } catch (ClassCastException e) {
        }
    }

	if (wrap == null)
		wrap = new AxisPlacementWrapper(numOfRadios);

	switchValues(radioSel);
}

/**
 * Fills the property sheet with values associated with the newly selected
 * radio button.
 * @param sel the index of the newly selected radio button
 */
protected void switchValues(int sel)
{
	if (sel < 0 || wrap == null)
		return;

	switchIsDefValues(sel);

	setPAxis(wrap.paxis, sel, paxis_names, paxis_ch);
	paxis_ch.setEnabled(!wrap.placement_isdef[sel]);

	location_tf.setText("" + wrap.location[sel]);

	location_tf.setEnabled(!wrap.placement_isdef[sel] &&
						wrap.placement[sel] == JCAxis.VALUE_ANCHORED);
	validate();
}

/**
 * Refreshes the property sheet values that have <code>IsDefault</code> boooleans
 * associated with them.
 * @param sel the index of the newly selected radio button
 */
protected void switchIsDefValues(int sel)
{
	if (sel < 0 || wrap == null)
		return;

	int check_placement = -1;
	// default == AUTOMATIC
	if (wrap.placement_isdef[sel]) {
		check_placement = JCAxis.AUTOMATIC;
	} else {
		check_placement = wrap.placement[sel];
	}
	for (int i = 0; i < placement_vals.length; i++) {
		if (check_placement == placement_vals[i]) {
			if (placement_ch.getSelectedIndex() != i)
				placement_ch.setSelectedIndex(i);
			break;
		}
	}
}

public void setPAxis(int[] paxis, int sel, String[] paxis_names, 
													JComboBox paxis_ch)
{
	if (wrap.paxis[sel] == -1) {
		if (paxis_ch.getSelectedIndex() != paxis_ch.getItemCount() - 1)
			paxis_ch.setSelectedIndex(paxis_ch.getItemCount() - 1);
	} else {
        for (int i = 0; i < paxis_names.length; i++) {
            if (MultiChart.AXIS_RADIO_NAMES[paxis[sel]].
                                            equals(paxis_names[i])) {
                if (paxis_ch.getSelectedIndex() != i)
                    paxis_ch.setSelectedIndex(i);
            }
        }
	}
}

public Object getValue() {
	return (Object) wrap;
}

public void setAsText(String text) throws IllegalArgumentException {
    Vector paramVector = BeanTextParser.parse(text);
    if (paramVector.size() != 3) {
        throw new IllegalArgumentException();
    }
    AxisPlacementWrapper setWrapper = null;
    try {
        setWrapper = new AxisPlacementWrapper(
            (String) paramVector.elementAt(0),
            (String) paramVector.elementAt(1),
            (String) paramVector.elementAt(2)
            );
        setValue(setWrapper);
        support.firePropertyChange("", null, null);
    } catch (Exception e) {
        throw new IllegalArgumentException();
    }
}

public String getAsText() {
    if (wrap != null)
        return wrap.toString();
    else
        return null;
}

public String getJavaInitializationString() {
	return "new com.klg.jclass.chart.beans.AxisPlacementWrapper(" + wrap + ")";
}

public Dimension getPreferredSize() {
    Dimension dim = super.getPreferredSize();
    dim.width = Math.max(dim.width, 400);
    dim.height += 200;
    return dim;
}

public void itemStateChanged( ItemEvent ev ) {
    super.itemStateChanged(ev);
    Object src;
    if( (src = ev.getSource()) instanceof JComboBox && 
			(ev.getStateChange() != ItemEvent.DESELECTED)) {
		checkProperties(src);
    }
}

public void focusGained(FocusEvent ev) {}

public void focusLost(FocusEvent ev) 
{
	Object src = ev.getSource();
	checkProperties(src);
}

public void actionPerformed( ActionEvent ev ) 
{
	Object src = ev.getSource();
	checkProperties(src);
}

public void checkProperties(Object src)
{
	try {
	if (src	== placement_ch) {
		int select = 0;

 
		select = placement_ch.getSelectedIndex();
		// selection not in value array -- return to default
		if (select >= placement_vals.length) {
			if (!wrap.placement_isdef[radioSel]) {
				wrap.placement_isdef[radioSel] = true;
				support.firePropertyChange("", null, null);
			}
		// accept selected value
		} else if (wrap.placement[radioSel] != placement_vals[select]) {
			wrap.placement[radioSel] = placement_vals[select];
			wrap.placement_isdef[radioSel] = false;
			support.firePropertyChange("", null, null);
		}
		switchValues(radioSel);
	} else if (src == paxis_ch) {
		int select = paxis_ch.getSelectedIndex();
		if (select == paxis_names.length) {
			wrap.paxis[radioSel] = -1;
		} else {
			wrap.paxis[radioSel] = select;
		}
		support.firePropertyChange("", null, null);
	} else if (src	== location_tf) {
		setDoubleProperty(location_tf.getText(), wrap.location, radioSel,
														wrap.location_def);
		switchValues(radioSel);
	}
	} catch (Exception ex) {
	}
	switchIsDefValues(radioSel);
}

/* for testing purposes
public static void main( String args[] ) {
    JFrame frame;
    try {
    java.lang.Class newclass
        = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
    frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
        System.out.println(ex);
    frame = new JFrame();
    }

    AxisPlacementEditor axisPlacementEditor = new AxisPlacementEditor();
    axisPlacementEditor.setBackground( java.awt.Color.lightGray );
	AxisPlacementWrapper thiswrap = new AxisPlacementWrapper(
		"", "", "(x1 0.0)(x2 0.0)(y1 0.0)(y2 0.0)");
    axisPlacementEditor.setValue( thiswrap );
    frame.getContentPane().add( axisPlacementEditor );
    frame.pack();
    java.awt.Dimension bigness = axisPlacementEditor.getPreferredSize();
    frame.setSize( bigness.width + 100, bigness.height + 50 );
    frame.setVisible(true);
}
*/

}
