/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisPlacementWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCChartEnumMappings;

import java.awt.Font;

/**
 * Wrapper class for Axis Placement properties.  
 * Used by the MultiChart Bean.
 */
public class AxisPlacementWrapper extends RadioAxisWrapper {

public int    placement[];
public int    paxis[];			// placement axis index
public double location[];

public boolean placement_isdef[];

public double  location_def = 0.0;         // default value for location

/**
 * Default constructor.  Members will be empty.
 */
public AxisPlacementWrapper() {
	super();
}

/**
 * Constructor.  Allocates space for the arrays but does not fill them.
 * @param size
 */
public AxisPlacementWrapper(int size) {
	super();
	setWrapperArraySize(size);
}

/**
 * Constructor.  Allocates space for the arrays and fills them by parsing
 * series Strings of format <code>(x1 value)(y1 value)(x2 value)(y2 value)</code>.
 * @param placementString  axis placement String in above format.
 * Possible values are <code>Automatic</code>, <code>Min</code>, <code>Max</code>,
 * <code>Origin</code>, or <code>Value_Anchored</code>.
 *                         <p>This property has intelligent defaults.  If a
 *                         value is not provided for an axis, it will 
 *                         use them.
 * @param paxisString      placement axis String in above format.  Values are
 *                         the names of the axis being referenced (for example, <i>x1</i>).
 * @param locationString   placement location String in above format.  Values
 *                         double values.
 */
public AxisPlacementWrapper(String placementString, String paxisString,
    String locationString)
{
    setWrapperArraySize(MultiChart.AXIS_RADIO_NAMES.length);

    setWrapperEnumIsDefValues(placement, placement_isdef,
		 new RadioSeries(placementString),
        JCChartEnumMappings.placement_strings, 
		JCChartEnumMappings.placement_values,
        "AxisPlacement", JCAxis.AUTOMATIC);
	setWrapperRadioIDValues(paxis, new RadioSeries(paxisString));
    setWrapperDoubleValues(location, new RadioSeries(locationString));
}


/**
 * Sets the size of the internal arrays that store properties for
 * axes (should be equal to the number of axes).
 * @param size the size of the internal arrays
 */
public void setWrapperArraySize(int size)
{
	placement = new int[size];
	paxis = new int[size];
	location = new double[size];

	placement_isdef = new boolean[size];
}

/**
 * Stores the properties from 
 * the specified axis in internal arrays at the specified index.
 * <br><B>Note:</b> The axis wrapper value is set by the Bean's <code>getAxisPlacement()</code>
 * method.
 * @param index the index of the array that corresponds to this axis
 * @param axis the axis that contains the properties for this index
 */
public void setWrapperValue(int index, JCAxis axis)
{
	placement[index] = axis.getPlacement();
	location[index] = axis.getPlacementLocation();

	placement_isdef[index] = axis.getPlacementIsDefault();
	// paxis no longer set due to serialization problems -- see bean's
	// getAxisPlacement() method
}

/**
 * Sets the properties on 
 * the specified axis from values stored in internal arrays 
 * at the specified index.
 * <br><B>Note:</b> The placement axis property is set by the Bean's <code>setAxisPlacement()</code>
 * method.
 * @param index the index where the values for this axis are stored
 * @param axis the axis that we need to set these values on
 */
public void setPropertyValue(int index, JCAxis axis)
{
	if (placement_isdef[index]) {
		axis.setPlacementIsDefault(placement_isdef[index]);
	} else {
		try {
			axis.setPlacement(placement[index]);
		} catch (Exception e) {
			placement[index] = axis.getPlacement();
		}
	}

	axis.setPlacementLocation(location[index]);
	// paxis no longer set due to serialization problems -- see bean's
	// setAxisPlacement() method
}

/**
 * Converts properties to String for code generation.
 * @return String containing properties for code generation
 */
public String toString()
{
    String s = "";
    s += enumIsDefValuesToString(placement, placement_isdef,
        JCChartEnumMappings.placement_strings, 
		JCChartEnumMappings.placement_values);
    s += ",";
    s += radioIDValuesToString(paxis);
    s += ",";
    s += doubleValuesToString(location);

    return s;
}

public boolean equals(Object obj)
{
	boolean eq = true;
	if (obj instanceof AxisPlacementWrapper) {
		AxisPlacementWrapper compareWrap = (AxisPlacementWrapper) obj;
		if (compareWrap.placement != null && this.placement != null) {
			for (int i = 0; i < compareWrap.placement.length; i++) {
				if (!compareWrap.placement_isdef[i] && !this.placement_isdef[i]) {
					if (compareWrap.placement[i] != this.placement[i])
						eq = false;
				} else if (compareWrap.placement_isdef[i] != this.placement_isdef[i]) {
					eq = false;
				}
			}
		} else {
			if (compareWrap.placement != this.placement)
				eq = false;
		}
		if (compareWrap.paxis != null && this.paxis != null) {
			for (int i = 0; i < compareWrap.paxis.length; i++) {
				if (compareWrap.paxis[i] != this.paxis[i])
					eq = false;
			}
		} else {
			if (compareWrap.paxis != this.paxis)
				eq = false;
		}
		if (compareWrap.location != null && this.location != null) {
			for (int i = 0; i < compareWrap.location.length; i++) {
				if (compareWrap.location[i] != this.location[i])
					eq = false;
			}
		} else {
			if (compareWrap.location != this.location)
				eq = false;
		}

	} else {
		eq = false;
	}
	return eq;
}

}
