/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisPointLabelEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import java.awt.Insets;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Dimension;
import java.util.StringTokenizer;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextArea;
import javax.swing.BorderFactory;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;




import java.util.Vector;


/**
 * Editor for PointLabel Axis Properties.
 */

public class AxisPointLabelEditor extends RadioEditor
					  implements DocumentListener {

protected JTextArea pointlabels_ta;

public static final String explanation = 
	JCChartBeanBundle.string(JCChartBeanBundle.key227);
protected String editString;

public AxisPointLabelEditor() {
	super(MultiChart.AXIS_RADIO_TITLE, MultiChart.AXIS_RADIO_NAMES);
}

protected void init()
{
	super.init();
	radioB[1].setEnabled(false);
	radioB[3].setEnabled(false);
}

/**
 * Creates a panel containing the GUI elements for the properties
 * associated with the radio buttons.
 * @return the panel
 */
protected JPanel makePropertyPanel() 
{

    JPanel p2 = new JPanel();
    p2.setBorder(BorderFactory.createTitledBorder(
    	JCChartBeanBundle.string(JCChartBeanBundle.key228)));
	p2.setLayout(new GridLayout(1, 1));

    pointlabels_ta = new JTextArea(explanation);
	p2.add(pointlabels_ta);

	add("Center", p2);

	pointlabels_ta.getDocument().addDocumentListener(this);

	return p2;
}

AxisPointLabelWrapper wrap = null;

public void setValue(Object value) {

    if (value != null) {
        try {
			wrap = (AxisPointLabelWrapper) value;
        } catch (ClassCastException e) {
        }
    }

	if (wrap == null)
		wrap = new AxisPointLabelWrapper(numOfRadios);

	if (editString == null) {
		switchValues(radioSel);
	}
}

/**
 * Fills the property sheet with values associated with the newly selected
 * radio button.
 * @param sel the index of the newly selected radio button
 */
protected void switchValues(int sel)
{
	if (sel < 0 || wrap == null)
		return;

	StringBuffer sb = new StringBuffer(explanation);
	sb.append('\n');
	if (wrap.pointlabels[sel] != null) {
		sb.append(wrap.pointlabels[sel]);
	}
	editString = sb.toString();
	pointlabels_ta.setText(editString);
}

public Object getValue() {
	return (Object) wrap;
}

public void setAsText(String text) throws IllegalArgumentException {
    Vector paramVector = BeanTextParser.parse(text);
    if (paramVector.size() != 1) {
        throw new IllegalArgumentException();
    }
    AxisPointLabelWrapper setWrapper = null;
    try {
        setWrapper = new AxisPointLabelWrapper(
            (String) paramVector.elementAt(0)
            );
        setValue(setWrapper);
        support.firePropertyChange("", null, null);
    } catch (Exception e) {
        throw new IllegalArgumentException();
    }
}

public String getAsText() {
    if (wrap != null)
        return wrap.toString();
    else
        return null;
}

public String getJavaInitializationString() {
	return "new com.klg.jclass.chart.beans.AxisPointLabelWrapper(" + wrap + ")";
}

public Dimension getPreferredSize() {
    Dimension dim = super.getPreferredSize();
    dim.width = Math.max(dim.width, 400);
    dim.height += 250;
    return dim;
}

public void changedUpdate( DocumentEvent ev ) 
{
	Object src = ev.getDocument();
	checkProperties(src);
}
public void insertUpdate( DocumentEvent ev ) 
{
	Object src = ev.getDocument();
	checkProperties(src);
}
public void removeUpdate( DocumentEvent ev ) 
{
	Object src = ev.getDocument();
	checkProperties(src);
}

public void checkProperties(Object src)
{
	try {
	if (src	== pointlabels_ta.getDocument()) {
		editString = pointlabels_ta.getText();

        StringTokenizer strtok = new StringTokenizer(editString, "\n\r");
        StringBuffer sb = new StringBuffer();
        boolean firstTime = true;
        while (strtok.hasMoreTokens()) {
            String line = strtok.nextToken();

            if (line.charAt(0) == '#')
                continue;

            if (firstTime)
                firstTime = false;
            else
                sb.append('\n');
            sb.append(line);
        }

		wrap.pointlabels[radioSel] = sb.toString();
		support.firePropertyChange("", null, null);
	}
	} catch (Exception ex) {
	}
}

/* for testing purposes
public static void main( String args[] ) {
    JFrame frame;
    try {
    java.lang.Class newclass
        = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
    frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
        System.out.println(ex);
    frame = new JFrame();
    }

    AxisPointLabelEditor axisPointLabelEditor = new AxisPointLabelEditor();
    axisPointLabelEditor.setBackground( java.awt.Color.lightGray );
    AxisPointLabelWrapper thiswrap = new AxisPointLabelWrapper("");
    axisPointLabelEditor.setValue( thiswrap );
    frame.getContentPane().add( axisPointLabelEditor );
    frame.pack();
    java.awt.Dimension bigness = axisPointLabelEditor.getPreferredSize();
    frame.setSize( bigness.width + 100, bigness.height + 50 );
    frame.setVisible(true);
}
*/

}
