/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisPointLabelWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.ChartDataView;
import java.util.Date;
import java.util.StringTokenizer;




import java.util.Iterator;
import java.util.List;
import java.util.Vector;


/**
 * Wrapper class for Axis Point Label properties.  
 * Used by the MultiChart Bean.
 */
public class AxisPointLabelWrapper extends RadioAxisWrapper {

public String   pointlabels[];

/**
 * Default constructor.  Members will be empty.
 */
public AxisPointLabelWrapper() {
	super();
}

/**
 * Constructor.  Allocates space for the arrays but does not fill them.
 * @param size
 */
public AxisPointLabelWrapper(int size) {
	super();
	setWrapperArraySize(size);
}

/**
 * Constructor.  Allocates space for the arrays and fills them by parsing
 * series Strings of format <code>(x1 value)(y1 value)(x2 value)(y2 value)</code>.
 * @param pointLabelString  point label String in above format.  Values are
 * Strings, each a list of point label pairs separated by a newline (<code>\n</code>).
 * For example, the series String might look like :
 *                          <pre>(x1 1.0,one\n2.0,two\n3.0,three)</pre>
 */
public AxisPointLabelWrapper(String pointLabelString) {
    setWrapperArraySize(MultiChart.AXIS_RADIO_NAMES.length);

    setWrapperStringValues(pointlabels, new RadioSeries(pointLabelString));
}

/**
 * Sets the size of the internal arrays that store properties for
 * axes (should be equal to the number of axes).
 * @param size the size of the internal arrays
 */
public void setWrapperArraySize(int size)
{
	pointlabels = new String[size];
}

/**
 * Stores the properties from 
 * the specified axis in internal arrays at the specified index.
 * @param index the index of the array that corresponds to this axis
 * @param axis the axis that contains the properties for this index
 */
public void setWrapperValue(int index, JCAxis axis)
{
	// find mathing axis in chart data view list
	ChartDataView cdv = null;
	List cdvList = axis.getParentChart().getDataView();
	for (Iterator i = cdvList.iterator(); i.hasNext();) {
		ChartDataView thisCDV = (ChartDataView) i.next();
		if (thisCDV.getXAxis() == axis) {
			cdv = thisCDV;
			break;
		}
	}
	if (cdv != null) {
		pointlabels[index] = pointLabelsToString(cdv.getPointLabels(), 
													cdv.getXArrayCopy());
	}
}

/**
 * Sets the properties on 
 * the specified axis from values stored in internal arrays 
 * at the specified index.
 * @param index the index where the values for this axis are stored
 * @param axis the axis that we need to set these values on
 */
public void setPropertyValue(int index, JCAxis axis)
{
	// find mathing axis in chart data view list
	ChartDataView cdv = null;
	List cdvList = axis.getParentChart().getDataView();
	for (Iterator i = cdvList.iterator(); i.hasNext();) {
		ChartDataView thisCDV = (ChartDataView) i.next();
		if (thisCDV.getXAxis() == axis) {
			cdv = thisCDV;
			break;
		}
	}
	if (cdv != null) {
		cdv.setPointLabels(stringToPointLabels(pointlabels[index], 
											   cdv.getXArrayCopy() ));
	}
}

/**
 * Converts properties to String for code generation.
 * @return String containing properties for code generation
 */
public String toString()
{
    String s = "";
    s += stringValuesToString(pointlabels);

    return s;
}

/**
 * Converts list of point labels to a String for code generation.
 * @param pointLabelList array of Strings for point labels
 * @param pointValueList matching array of values for point labels
 * @return String containing list of point labels. Each line is of
 * the format "point,label" with each of these pairs separated by 
 * newlines.
 */
public String pointLabelsToString(List pointLabelList, 
										double[] pointValueList)
{
   StringBuffer sb = new StringBuffer();
   if (pointLabelList != null && pointValueList != null) {
       for (int i = 0; i < pointLabelList.size(); i++) {
           sb.append(pointValueList[i]);
           sb.append(" , ");
           sb.append(pointLabelList.get(i));
           sb.append('\n');
       }
   }

   return sb.toString();
}

/**
 * Converts String to list of point labels.
 * @param pointLabelString String containing entire point label list.
 *        Each line has a pair of the form "value,label".
 * @param pointValueList array of possible point values
 * @return array of Strings representing the list of point labels
 */
public List stringToPointLabels(String pointLabelString,
										double[] pointValueList)
{
	if (pointLabelString == null || pointValueList == null)
		return null;

	Vector labelArray = new Vector();
    StringTokenizer strtok = new StringTokenizer(pointLabelString, "\n\r");
    while (strtok.hasMoreTokens()) {
        String line = strtok.nextToken();

        if (line.charAt(0) == '#')
            continue;

        int commapos;
        if ((commapos = line.indexOf(',')) < 0)
            continue;

        try {
        double pointval = Double.valueOf(
                    line.substring(0, commapos).trim()).doubleValue();
        for (int i = 0; i < pointValueList.length; i++) {
            if (pointval == pointValueList[i]) {
				String s = line.substring(commapos + 1).trim();
				if (s.equals("null")) {
					labelArray.add(null);
				}
				else {
					labelArray.add(s);
				}
            }
        }
        } catch (Exception ex) {
            //ex.printStackTrace();
        }
    }
    return labelArray;
}

public boolean equals(Object obj)
{
	boolean eq = true;
	if (obj instanceof AxisPointLabelWrapper) {
		AxisPointLabelWrapper compareWrap = (AxisPointLabelWrapper) obj;
		if (compareWrap.pointlabels != null && this.pointlabels != null) {
			for (int i = 0; i < compareWrap.pointlabels.length; i++) {
				if (compareWrap.pointlabels[i] != null && this.pointlabels[i] != null) {
					if (!compareWrap.pointlabels[i].equals(this.pointlabels[i]))
						eq = false;
				} else {
					if (compareWrap.pointlabels[i] != this.pointlabels[i])
						eq = false;
				}
			}
		} else {
			if (compareWrap.pointlabels != this.pointlabels)
				eq = false;
		}

	} else {
		eq = false;
	}
	return eq;
}

}
