/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisRelationshipEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import java.awt.Insets;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.BorderFactory;




import java.util.Vector;


/**
 * Editor for Axis Relationship Properties.
 */

public class AxisRelationshipEditor extends RadioEditor
					  implements ItemListener, ActionListener, FocusListener {

protected JLabel raxis_lb;
protected JComboBox raxis_ch;
protected JTextField constant_tf;
protected JTextField multiplier_tf;

protected String[] raxis_names;


 

public AxisRelationshipEditor() {
	super(MultiChart.AXIS_RADIO_TITLE, MultiChart.AXIS_RADIO_NAMES);
}

protected void setEnumArrays()
{
	raxis_names = new String[MultiChart.AXIS_RADIO_NAMES.length];
	System.arraycopy(MultiChart.AXIS_RADIO_NAMES, 0, raxis_names, 0,
								MultiChart.AXIS_RADIO_NAMES.length);
}

/**
 * Creates a panel containing the GUI elements for the properties
 * associated with the radio buttons.
 * @return the panel
 */
protected JPanel makePropertyPanel() 
{

    JPanel p2 = new JPanel();
    p2.setBorder(BorderFactory.createTitledBorder(
    	JCChartBeanBundle.string(JCChartBeanBundle.key229)));
	p2.setLayout(new GridBagLayout());
	GridBagConstraints gbc2  = new GridBagConstraints();
	gbc2.gridwidth = 2;
	gbc2.gridx = GridBagConstraints.RELATIVE;
	gbc2.gridy = 0;
	gbc2.anchor = GridBagConstraints.WEST;
	gbc2.insets = new Insets(4,4,4,4);

	raxis_ch = new JComboBox();

	setEnumArrays();

	for (int i = 0; i < raxis_names.length; i++)
		raxis_ch.addItem(raxis_names[i]);
	raxis_ch.addItem(JCChartBeanBundle.string(JCChartBeanBundle.key215));

	raxis_lb = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key230));
	JLabel multiplier_label = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key231));
	JLabel constant_label = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key232));
	multiplier_tf = new JTextField(5);
	constant_tf = new JTextField(5);

	p2.add(raxis_lb, gbc2);
	p2.add(raxis_ch, gbc2);
	gbc2.gridy++;
	p2.add(multiplier_label, gbc2);
	p2.add(multiplier_tf, gbc2);
	gbc2.gridy++;
	p2.add(constant_label, gbc2);
	p2.add(constant_tf, gbc2);

	add("Center", p2);

	raxis_ch.addItemListener(this);
	multiplier_tf.addActionListener(this);
	multiplier_tf.addFocusListener(this);
	constant_tf.addActionListener(this);
	constant_tf.addFocusListener(this);

	return p2;
}


 

AxisRelationshipWrapper wrap = null;

public void setValue(Object value) {

    if (value != null) {
        try {
			wrap = (AxisRelationshipWrapper) value;
        } catch (ClassCastException e) {
        }
    }

	if (wrap == null)
		wrap = new AxisRelationshipWrapper(numOfRadios);

	switchValues(radioSel);
}

/**
 * Fills the property sheet with values associated with the newly selected
 * radio button.
 * @param sel the index of the newly selected radio button
 */
protected void switchValues(int sel)
{
	if (sel < 0 || wrap == null)
		return;

    setRAxis(wrap.raxis, sel, raxis_names, raxis_ch);

	multiplier_tf.setText("" + wrap.multiplier[sel]);
	constant_tf.setText("" + wrap.constant[sel]);
	validate();
}

public void setRAxis(int[] raxis, int sel, String[] raxis_names,
                                                    JComboBox raxis_ch)
{
    if (wrap.raxis[sel] == -1) {
        if (raxis_ch.getSelectedIndex() != raxis_ch.getItemCount() - 1)
            raxis_ch.setSelectedIndex(raxis_ch.getItemCount() - 1);
    } else {
        for (int i = 0; i < raxis_names.length; i++) {
            if (MultiChart.AXIS_RADIO_NAMES[raxis[sel]].
                                            equals(raxis_names[i])) {
                if (raxis_ch.getSelectedIndex() != i)
                    raxis_ch.setSelectedIndex(i);
            }
        }
    }
}

public Object getValue() {
	return (Object) wrap;
}

public void setAsText(String text) throws IllegalArgumentException {
    Vector paramVector = BeanTextParser.parse(text);
    if (paramVector.size() != 3) {
        throw new IllegalArgumentException();
    }
    AxisRelationshipWrapper setWrapper = null;
    try {
        setWrapper = new AxisRelationshipWrapper(
            (String) paramVector.elementAt(0),
            (String) paramVector.elementAt(1),
            (String) paramVector.elementAt(2)
            );
        setValue(setWrapper);
        support.firePropertyChange("", null, null);
    } catch (Exception e) {
        throw new IllegalArgumentException();
    }
}

public String getAsText() {
    if (wrap != null)
        return wrap.toString();
    else
        return null;
}

public String getJavaInitializationString() {
	return "new com.klg.jclass.chart.beans.AxisRelationshipWrapper(" + wrap + ")";
}

public Dimension getPreferredSize() {
    Dimension dim = super.getPreferredSize();
    dim.width = Math.max(dim.width, 400);
    dim.height += 250;
    return dim;
}

public void itemStateChanged( ItemEvent ev ) {
    super.itemStateChanged(ev);
    Object src;
    if( (src = ev.getSource()) instanceof JComboBox  &&
			ev.getStateChange() != ItemEvent.DESELECTED) {
		checkProperties(src);
    }
}

public void focusGained(FocusEvent ev) {}

public void focusLost(FocusEvent ev) 
{
	Object src = ev.getSource();
	checkProperties(src);
}

public void actionPerformed( ActionEvent ev ) 
{
	Object src = ev.getSource();
	checkProperties(src);
}

public void checkProperties(Object src)
{
	try {
	if (src	== constant_tf) {

 
		setDoubleProperty(constant_tf.getText(), wrap.constant, radioSel,
														wrap.constant_def);
		switchValues(radioSel);
	} else if (src	== multiplier_tf) {

 
		setDoubleProperty(multiplier_tf.getText(), wrap.multiplier, radioSel,
													wrap.multiplier_def);
		switchValues(radioSel);
    } else if (src == raxis_ch) {
        int select = raxis_ch.getSelectedIndex();

 
       	if (select == raxis_names.length) {
           	wrap.raxis[radioSel] = -1;
       	} else {
           	wrap.raxis[radioSel] = select;
       	}
       	support.firePropertyChange("", null, null);
	}
	} catch (Exception ex) {
	}
}

/* for testing purposes
public static void main( String args[] ) {
    JFrame frame;
    try {
    java.lang.Class newclass
        = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
    frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
        System.out.println(ex);
    frame = new JFrame();
    }

    AxisRelationshipEditor axisRelationshipEditor = new AxisRelationshipEditor();
    axisRelationshipEditor.setBackground( java.awt.Color.lightGray );
	AxisRelationshipWrapper thiswrap = new AxisRelationshipWrapper("", "", "");
    axisRelationshipEditor.setValue( thiswrap );
    frame.getContentPane().add( axisRelationshipEditor );
    frame.pack();
    java.awt.Dimension bigness = axisRelationshipEditor.getPreferredSize();
    frame.setSize( bigness.width + 100, bigness.height + 50 );
    frame.setVisible(true);
}
*/

}
