/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisRelationshipWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCAxis;

import java.awt.Font;

/**
 * Wrapper class for Axis Relationship Label properties.  
 * Used by the MultiChart Bean.
 */
public class AxisRelationshipWrapper extends RadioAxisWrapper {

public int raxis[];
public double constant[];
public double multiplier[];

public double  constant_def = 0.0;       	// default value for constant
public double  multiplier_def = 1.0;     	// default value for multiplier

/**
 * Default constructor.  Members will be empty.
 */
public AxisRelationshipWrapper() {
	super();
}

/**
 * Constructor.  Allocates space for the arrays but does not fill them.
 * @param size
 */
public AxisRelationshipWrapper(int size) {
	super();
	setWrapperArraySize(size);
}

/**
 * Constructor.  Allocate space for the arrays and fills them by parsing
 * series Strings of format <code>(x1 value)(y1 value)(x2 value)(y2 value)</code>.
 * @param raxisString       related axis string in above format.  Values are
 *                          the names of the axis being referenced (for example, <code>x1</code>).
 * @param constantString    constant String in above format.  Values are double
 *                          values.
 * @param multiplierString  multiplier String in above format.  Values are
 *                          double values.
 */
public AxisRelationshipWrapper(String raxisString, String constantString,
    String multiplierString)
{
    setWrapperArraySize(MultiChart.AXIS_RADIO_NAMES.length);

    setWrapperRadioIDValues(raxis, new RadioSeries(raxisString));
    setWrapperDoubleValues(constant, new RadioSeries(constantString));
    setWrapperDoubleValues(multiplier, new RadioSeries(multiplierString));
}
/**
 * Sets the size of the internal arrays that store properties for
 * axes (should be equal to the number of axes).
 * @param size the size of the internal arrays
 */
public void setWrapperArraySize(int size)
{
	raxis = new int[size];
	constant = new double[size];
	multiplier = new double[size];
}

/**
 * Stores the properties from 
 * the specified axis in internal arrays at the specified index.
 * <br><B>Note:</b> The axis wrapper value is set by the Bean's
 * <code>getAxisRelationship()</code> method.
 * @param index the index of the array that corresponds to this axis
 * @param axis the axis that contains the properties for this index
 */
public void setWrapperValue(int index, JCAxis axis)
{
	constant[index] = axis.getFormula().getConstant();
	multiplier[index] = axis.getFormula().getMultiplier();
	// raxis no longer set due to serialization problems -- see bean's
	// setAxisRelationship() method
}

/**
 * Sets the properties on 
 * the specified axis from values stored in internal arrays 
 * at the specified index.
 * <br><B>Note:</b> The relationship axis property is set by the Bean's
 * <code>setAxisRelationship()</code> method.
 * @param index the index where the values for this axis are stored
 * @param axis the axis that we need to set these values on
 */
public void setPropertyValue(int index, JCAxis axis)
{
    axis.getFormula().setConstant(constant[index]);
    axis.getFormula().setMultiplier(multiplier[index]);
	// raxis no longer set due to serialization problems -- see bean's
	// setAxisRelationship() method
}

/**
 * Converts properties to String for code generation.
 * @return String containing properties for code generation
 */
public String toString()
{
    String s = "";
    s += radioIDValuesToString(raxis);
    s += ",";
    s += doubleValuesToString(constant);
    s += ",";
    s += doubleValuesToString(multiplier);

    return s;
}

public boolean equals(Object obj)
{
	boolean eq = true;
	if (obj instanceof AxisRelationshipWrapper) {
		AxisRelationshipWrapper compareWrap = (AxisRelationshipWrapper) obj;
		if (compareWrap.raxis != null && this.raxis != null) {
			for (int i = 0; i < compareWrap.raxis.length; i++) {
				if (compareWrap.raxis[i] != this.raxis[i])
					eq = false;
			}
		} else {
			if (compareWrap.raxis != this.raxis)
				eq = false;
		}
		if (compareWrap.constant != null && this.constant != null) {
			for (int i = 0; i < compareWrap.constant.length; i++) {
				if (compareWrap.constant[i] != this.constant[i])
					eq = false;
			}
		} else {
			if (compareWrap.constant != this.constant)
				eq = false;
		}
		if (compareWrap.multiplier != null && this.multiplier != null) {
			for (int i = 0; i < compareWrap.multiplier.length; i++) {
				if (compareWrap.multiplier[i] != this.multiplier[i])
					eq = false;
			}
		} else {
			if (compareWrap.multiplier != this.multiplier)
				eq = false;
		}

	} else {
		eq = false;
	}
	return eq;
}

}
