/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisScaleWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCAxis;

/**
 * Wrapper class for Axis Scale Label properties.  
 * Used by the MultiChart Bean.
 */
public class AxisScaleWrapper extends RadioAxisWrapper {

public double min[];
public double max[];
public double numspacing[];
public double tickspacing[];
public int    precision[];

public boolean min_isdef[];
public boolean max_isdef[];
public boolean numspacing_isdef[];
public boolean tickspacing_isdef[];
public boolean precision_isdef[];

/**
 * Default constructor.  Members will be empty.
 */
public AxisScaleWrapper() {
	super();
}

/**
 * Constructor.  Allocates space for the arrays but does not fill them.
 * @param size
 */
public AxisScaleWrapper(int size) {
	super();
	setWrapperArraySize(size);
}

/**
 * Constructor.  Allocates space for the arrays and fills them by parsing
 * series Strings of format <code>(x1 value)(y1 value)(x2 value)(y2 value)</code>.
 * @param minString          min String in above format.  Values are
 *                           double values.
 *                           This property has intelligent defaults.  If a 
 *                           value is not provided for an axis, it will 
 *                           use them.
 * @param maxString          max String in above format.  Values are
 *                           double values.
 *                           This property has intelligent defaults.  If a 
 *                           value is not provided for an axis, it will 
 *                           use them.
 * @param numSpacingString   numSpacing String in above format.  Values are
 *                           double values.
 *                           This property has intelligent defaults.  If a 
 *                           value is not provided for an axis, it will 
 *                           use them.
 * @param tickSpacingString  tickSpacing String in above format.  Values are
 *                           double values.
 *                           This property has intelligent defaults.  If a 
 *                           value is not provided for an axis, it will 
 *                           use them.
 * @param precisionString    precision String in above format.  Values are
 *                           double values.
 *                           This property has intelligent defaults.  If a 
 *                           value is not provided for an axis, it will 
 *                           use them.
 */
public AxisScaleWrapper(String minString, String maxString,
	String numSpacingString, String tickSpacingString, String precisionString)
{ 
	setWrapperArraySize(MultiChart.AXIS_RADIO_NAMES.length);

	setWrapperDoubleIsDefValues(min, 
		min_isdef, new RadioSeries(minString));
	setWrapperDoubleIsDefValues(max, 
		max_isdef, new RadioSeries(maxString));
	setWrapperDoubleIsDefValues(numspacing, 
		numspacing_isdef, new RadioSeries(numSpacingString));
	setWrapperDoubleIsDefValues(tickspacing, 
		tickspacing_isdef, new RadioSeries(tickSpacingString));
	setWrapperIntIsDefValues(precision, 
		precision_isdef, new RadioSeries(precisionString));
}


/**
 * Sets the size of the internal arrays that store properties for
 * axes (should be equal to the number of axes).
 * @param size the size of the internal arrays
 */
public void setWrapperArraySize(int size)
{
	min = new double[size];
	max = new double[size];
	numspacing = new double[size];
	tickspacing = new double[size];
	precision = new int[size];

	min_isdef = new boolean[size];
	max_isdef = new boolean[size];
	numspacing_isdef = new boolean[size];
	tickspacing_isdef = new boolean[size];
	precision_isdef = new boolean[size];
}

/**
 * Stores the properties from 
 * the specified axis in internal arrays at the specified index.
 * @param index the index of the array that corresponds to this axis
 * @param axis the axis that contains the properties for this index
 */
public void setWrapperValue(int index, JCAxis axis)
{
	min[index] = axis.getMin();
	max[index] = axis.getMax();
	numspacing[index] = axis.getNumSpacing();
	tickspacing[index] = axis.getTickSpacing();
	precision[index] = axis.getPrecision();

	min_isdef[index] = axis.getMinIsDefault();
	max_isdef[index] = axis.getMaxIsDefault();
	numspacing_isdef[index] = axis.getNumSpacingIsDefault();
	tickspacing_isdef[index] = axis.getTickSpacingIsDefault();
	precision_isdef[index] = axis.getPrecisionIsDefault();
}

/**
 * Sets the properties on 
 * the specified axis from values stored in internal arrays 
 * at the specified index.
 * @param index the index where the values for this axis are stored
 * @param axis the axis that we need to set these values on
 */
public void setPropertyValue(int index, JCAxis axis)
{
	if (min_isdef[index]) {
		axis.setMinIsDefault(min_isdef[index]);
	} else {
		axis.setMin(min[index]);
	}
	if (max_isdef[index]) {
		axis.setMaxIsDefault(max_isdef[index]);
	} else {
		axis.setMax(max[index]);
	}
	if (numspacing_isdef[index]) {
		axis.setNumSpacingIsDefault(numspacing_isdef[index]);
	} else {
		axis.setNumSpacing(numspacing[index]);
	}
	if (tickspacing_isdef[index]) {
		axis.setTickSpacingIsDefault(tickspacing_isdef[index]);
	} else {
		axis.setTickSpacing(tickspacing[index]);
	}
	if (precision_isdef[index]) {
		axis.setPrecisionIsDefault(precision_isdef[index]);
	} else {
		axis.setPrecision(precision[index]);
	}
}

/**
 * Converts properties to String for code generation.
 * @return String containing properties for code generation
 */
public String toString()
{
	String s = "";
	s += doubleIsDefValuesToString(min, min_isdef);
	s += ",";
	s += doubleIsDefValuesToString(max, max_isdef);
	s += ",";
	s += doubleIsDefValuesToString(numspacing, numspacing_isdef);
	s += ",";
	s += doubleIsDefValuesToString(tickspacing, tickspacing_isdef);
	s += ",";
	s += intIsDefValuesToString(precision, precision_isdef);

	return s;
}

public boolean equals(Object obj)
{
	boolean eq = true;
	if (obj instanceof AxisScaleWrapper) {
		AxisScaleWrapper compareWrap = (AxisScaleWrapper) obj;
		if (compareWrap.min != null && this.min != null) {
			for (int i = 0; i < compareWrap.min.length; i++) {
				if (!compareWrap.min_isdef[i] && !this.min_isdef[i]) {
					if (compareWrap.min[i] != this.min[i])
						eq = false;
				} else if (compareWrap.min_isdef[i] != this.min_isdef[i]) {
					eq = false;
				}
			}
		} else {
			if (compareWrap.min != this.min)
				eq = false;
		}
		if (compareWrap.max != null && this.max != null) {
			for (int i = 0; i < compareWrap.max.length; i++) {
				if (!compareWrap.max_isdef[i] && !this.max_isdef[i]) {
					if (compareWrap.max[i] != this.max[i])
						eq = false;
				} else if (compareWrap.max_isdef[i] != this.max_isdef[i]) {
					eq = false;
				}
			}
		} else {
			if (compareWrap.max != this.max)
				eq = false;
		}
		if (compareWrap.tickspacing != null && this.tickspacing != null) {
			for (int i = 0; i < compareWrap.tickspacing.length; i++) {
				if (!compareWrap.tickspacing_isdef[i] && !this.tickspacing_isdef[i]) {
					if (compareWrap.tickspacing[i] != this.tickspacing[i])
						eq = false;
				} else if (compareWrap.tickspacing_isdef[i] != this.tickspacing_isdef[i]) {
					eq = false;
				}
			}
		} else {
			if (compareWrap.tickspacing != this.tickspacing)
				eq = false;
		}
		if (compareWrap.numspacing != null && this.numspacing != null) {
			for (int i = 0; i < compareWrap.numspacing.length; i++) {
				if (!compareWrap.numspacing_isdef[i] && !this.numspacing_isdef[i]) {
					if (compareWrap.numspacing[i] != this.numspacing[i])
						eq = false;
				} else if (compareWrap.numspacing_isdef[i] != this.numspacing_isdef[i]) {
					eq = false;
				}
			}
		} else {
			if (compareWrap.numspacing != this.numspacing)
				eq = false;
		}
		if (compareWrap.precision != null && this.precision != null) {
			for (int i = 0; i < compareWrap.precision.length; i++) {
				if (!compareWrap.precision_isdef[i] && !this.precision_isdef[i]) {
					if (compareWrap.precision[i] != this.precision[i])
						eq = false;
				} else if (compareWrap.precision_isdef[i] != this.precision_isdef[i]) {
					eq = false;
				}
			}
		} else {
			if (compareWrap.precision != this.precision)
				eq = false;
		}

	} else {
		eq = false;
	}
	return eq;
}

}
