/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisTimeLabelEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCChartEnumMappings;

import java.awt.Insets;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.text.DateFormat;
import java.util.Date;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.BorderFactory;




import java.util.Vector;


/**
 * Editor for TimeLabel Axis Properties.
 */

public class AxisTimeLabelEditor extends RadioEditor
					  implements ItemListener, ActionListener, FocusListener {

protected JTextField timebase_tf;
protected JTextField timeformat_tf;
protected JComboBox  timeunit_ch;

protected String[] timeunit_names;
protected long[] timeunit_vals;

public AxisTimeLabelEditor() {
	super(MultiChart.AXIS_RADIO_TITLE, MultiChart.AXIS_RADIO_NAMES);
}

protected void setEnumArrays()
{
    timeunit_names = JCChartEnumMappings.timeUnit_i18n_strings;
    timeunit_vals = JCChartEnumMappings.timeUnit_values;
}

/**
 * Creates a panel containing the GUI elements for the properties
 * associated with the radio buttons.
 * @return the panel
 */
protected JPanel makePropertyPanel() 
{

    JPanel p2 = new JPanel();
    p2.setBorder(BorderFactory.createTitledBorder(
    			JCChartBeanBundle.string(JCChartBeanBundle.key245)));
	p2.setLayout(new GridBagLayout());
	GridBagConstraints gbc2  = new GridBagConstraints();
	gbc2.gridwidth = 1;
	gbc2.gridx = GridBagConstraints.RELATIVE;
	gbc2.gridy = 0;
	gbc2.anchor = GridBagConstraints.WEST;
	gbc2.insets = new Insets(4,4,4,4);

	timeunit_ch = new JComboBox();

	setEnumArrays();
    for (int i = 0; i < timeunit_names.length; i++)
        timeunit_ch.addItem(timeunit_names[i]);

	JLabel timebase_label = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key246));
	JLabel timeformat_label = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key247));
	JLabel timeunit_label = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key248));
	timebase_tf = new JTextField(30);
	timeformat_tf = new JTextField(30);
	p2.add(timebase_label, gbc2);
	p2.add(timebase_tf, gbc2);
	gbc2.gridy++;
	p2.add(timeformat_label, gbc2);
	p2.add(timeformat_tf, gbc2);
	gbc2.gridy++;
	p2.add(timeunit_label, gbc2);
	p2.add(timeunit_ch, gbc2);

	add("Center", p2);

	timebase_tf.addActionListener(this);
	timebase_tf.addFocusListener(this);
	timeformat_tf.addActionListener(this);
	timeformat_tf.addFocusListener(this);
	timeunit_ch.addItemListener(this);

	return p2;
}

AxisTimeLabelWrapper wrap = null;

public void setValue(Object value) {

    if (value != null) {
        try {
			wrap = (AxisTimeLabelWrapper) value;
        } catch (ClassCastException e) {
        }
    }

	if (wrap == null)
		wrap = new AxisTimeLabelWrapper(numOfRadios);

	switchValues(radioSel);
}

/**
 * Fills the property sheet with values associated with the newly selected
 * radio button.
 * @param sel the index of the newly selected radio button
 */
protected void switchValues(int sel)
{
	if (sel < 0 || wrap == null)
		return;
	timebase_tf.setText(
		DateFormat.getInstance().format(wrap.timebase[sel]) );
	switchIsDefValues(sel);
    for (int i = 0; i < timeunit_vals.length; i++) {
        if (wrap.timeunit[sel] == timeunit_vals[i]) {
            if (timeunit_ch.getSelectedIndex() != i)
            	timeunit_ch.setSelectedIndex(i);
            break;
        }
    }
}

/**
 * Refreshes the properties that have <code>IsDefault</code> booleans associated with them.
 * @param sel the index of the newly selected radio button
 */
protected void switchIsDefValues(int sel)
{
	if (sel < 0 || wrap == null)
		return;
	timeformat_tf.setText("" + wrap.timeformat[sel]);
}

public Object getValue() {
	return (Object) wrap;
}

public void setAsText(String text) throws IllegalArgumentException {
    Vector paramVector = BeanTextParser.parse(text);
    if (paramVector.size() != 3) {
        throw new IllegalArgumentException();
    }
    AxisTimeLabelWrapper setWrapper = null;
    try {
        setWrapper = new AxisTimeLabelWrapper(
            (String) paramVector.elementAt(0),
            (String) paramVector.elementAt(1),
            (String) paramVector.elementAt(2)
            );
        setValue(setWrapper);
        support.firePropertyChange("", null, null);
    } catch (Exception e) {
        throw new IllegalArgumentException();
    }
}

public String getAsText() {
    if (wrap != null)
        return wrap.toString();
    else
        return null;
}

public String getJavaInitializationString() {
	return "new com.klg.jclass.chart.beans.AxisTimeLabelWrapper(" + wrap + ")";
}

public Dimension getPreferredSize() {
    Dimension dim = super.getPreferredSize();
    dim.width = Math.max(dim.width, 500);
    dim.height += 250;
    return dim;
}

public void focusGained(FocusEvent ev) {}

public void focusLost(FocusEvent ev) 
{
	Object src = ev.getSource();
	checkProperties(src);
}

public void actionPerformed( ActionEvent ev ) 
{
	Object src = ev.getSource();
	checkProperties(src);
}

public void itemStateChanged( ItemEvent ev ) {
    super.itemStateChanged(ev);
    Object src;
    if( (src = ev.getSource()) instanceof JComboBox &&
			ev.getStateChange() != ItemEvent.DESELECTED) {
        checkProperties(src);
    }
}

public void checkProperties(Object src)
{
	try {
	if (src	== timebase_tf) {
		wrap.timebase[radioSel] = 
			DateFormat.getInstance().parse(timebase_tf.getText());
		support.firePropertyChange("", null, null);
	} else if (src	== timeformat_tf) {
		setStringIsDefProperty(timeformat_tf.getText(), 
			wrap.timeformat, wrap.timeformat_isdef, radioSel);
		switchValues(radioSel);
	} else if (src	== timeunit_ch) {
		wrap.timeunit[radioSel] = timeunit_vals[timeunit_ch.getSelectedIndex()];
		support.firePropertyChange("", null, null);
		switchValues(radioSel);
	}
	} catch (Exception ex) {
	}
	switchIsDefValues(radioSel);
}

/* for testing purposes
public static void main( String args[] ) {
    JFrame frame;
    try {
    java.lang.Class newclass
        = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
    frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
        System.out.println(ex);
    frame = new JFrame();
    }

    AxisTimeLabelEditor axisTimeLabelEditor = new AxisTimeLabelEditor();
    axisTimeLabelEditor.setBackground( java.awt.Color.lightGray );
    AxisTimeLabelWrapper thiswrap = new AxisTimeLabelWrapper("", "", "");
    axisTimeLabelEditor.setValue( thiswrap );
    frame.getContentPane().add( axisTimeLabelEditor );
    frame.pack();
    java.awt.Dimension bigness = axisTimeLabelEditor.getPreferredSize();
    frame.setSize( bigness.width + 100, bigness.height + 50 );
    frame.setVisible(true);
}
*/

}
