/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisTimeLabelWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCChartEnumMappings;
import java.util.Date;

/**
 * Wrapper class for Axis Time Label properties.  
 * Used by the MultiChart Bean.
 */
public class AxisTimeLabelWrapper extends RadioAxisWrapper {

public Date   timebase[];
public String timeformat[];
public long   timeunit[];

public boolean timeformat_isdef[];

/**
 * Default constructor.  Members will be empty.
 */
public AxisTimeLabelWrapper() {
	super();
}

/**
 * Constructor.  Allocates space for the arrays but does not fill them.
 * @param size
 */
public AxisTimeLabelWrapper(int size) {
	super();
	setWrapperArraySize(size);
}

/**
 * Constructor.  Allocates space for the arrays and fills them by parsing
 * series Strings of format <code>(x1 value)(y1 value)(x2 value)(y2 value)</code>.
 * @param timebaseString    timebase String in above format.  Values should be
 *                          string representing a date.
 * @param timeformatString  timeformat String in above format.  For possible
 *                          values see the <code>setTimeFormat()</code> method of
 *                          <code>JCAxis</code>. This property has intelligent defaults.
 *                          If a value is not provided for an axis, it will
 *                          use them.
 * @param timeunitString    timeunit String in above format.  Possible values
 * are <code>Seconds</code>, <code>Minutes</code>, <code>Hours</code>, <code>Days</code>,
 * <code>Weeks</code>, <code>Months</code>, or <code>Years</code>.
 */
public AxisTimeLabelWrapper(String timebaseString, String timeformatString,
    String timeunitString)
{
    setWrapperArraySize(MultiChart.AXIS_RADIO_NAMES.length);

    setWrapperDateValues(timebase, new RadioSeries(timebaseString));
    setWrapperStringIsDefValues(timeformat, timeformat_isdef,
									 new RadioSeries(timeformatString));
    setWrapperEnumValues(timeunit, new RadioSeries(timeunitString),
        JCChartEnumMappings.timeUnit_strings, 
		JCChartEnumMappings.timeUnit_values,
        "AxisTimeLabelUnit", JCAxis.SECONDS);
}

/**
 * Sets the size of the internal arrays that store properties for
 * axes (should be equal to the number of axes).
 * @param size the size of the internal arrays
 */
public void setWrapperArraySize(int size)
{
	timebase = new Date[size];
	timeformat = new String[size];
	timeunit = new long[size];

	timeformat_isdef = new boolean[size];
}

/**
 * Stores the properties from 
 * the specified axis in internal arrays at the specified index.
 * @param index the index of the array that corresponds to this axis
 * @param axis the axis that contains the properties for this index
 */
public void setWrapperValue(int index, JCAxis axis)
{
	timebase[index] = axis.getTimeBase();
	timeformat[index] = axis.getTimeFormat();
	timeunit[index] = axis.getTimeUnit();

	timeformat_isdef[index] = axis.getTimeFormatIsDefault();
}

/**
 * Sets the properties on 
 * the specified axis from values stored in internal arrays 
 * at the specified index.
 * @param index the index where the values for this axis are stored
 * @param axis the axis that we need to set these values on
 */
public void setPropertyValue(int index, JCAxis axis)
{
	axis.setTimeBase(timebase[index]);

	if (timeformat_isdef[index]) {
		axis.setTimeFormatIsDefault(timeformat_isdef[index]);
	} else {
		axis.setTimeFormat(timeformat[index]);
	}

    try {
        axis.setTimeUnit(timeunit[index]);
    } catch (Exception e) {
        timeunit[index] = axis.getTimeUnit();
    }
}

/**
 * Converts properties to String for code generation.
 * @return String containing properties for code generation
 */
public String toString()
{
    String s = "";
    s += dateValuesToString(timebase);
    s += ",";
    s += stringIsDefValuesToString(timeformat, timeformat_isdef);
    s += ",";
    s += enumValuesToString(timeunit,
        JCChartEnumMappings.timeUnit_strings, 
		JCChartEnumMappings.timeUnit_values);

    return s;
}

public boolean equals(Object obj)
{
	boolean eq = true;
	if (obj instanceof AxisTimeLabelWrapper) {
		AxisTimeLabelWrapper compareWrap = (AxisTimeLabelWrapper) obj;
		if (compareWrap.timeunit != null && this.timeunit != null) {
			for (int i = 0; i < compareWrap.timeunit.length; i++) {
				if (compareWrap.timeunit[i] != this.timeunit[i])
					eq = false;
			}
		} else {
			if (compareWrap.timeunit != this.timeunit)
				eq = false;
		}

		if (compareWrap.timebase != null && this.timebase != null) {
			for (int i = 0; i < compareWrap.timebase.length; i++) {
				if (compareWrap.timebase[i] != null && this.timebase[i] != null) {
					if (!compareWrap.timebase[i].equals(this.timebase[i]))
						eq = false;
				} else {
					if (compareWrap.timebase[i] != this.timebase[i])
						eq = false;
				}
			}
		} else {
			if (compareWrap.timebase != this.timebase)
				eq = false;
		}
		if (compareWrap.timeformat != null && this.timeformat != null) {
			for (int i = 0; i < compareWrap.timeformat.length; i++) {
				if (!compareWrap.timeformat_isdef[i] && !this.timeformat_isdef[i]) {
					if (compareWrap.timeformat[i] != null && this.timeformat[i] != null) {
						if (!compareWrap.timeformat[i].equals(this.timeformat[i]))
							eq = false;
					} else {
						if (compareWrap.timeformat[i] != this.timeformat[i])
							eq = false;
					}
				} else if (compareWrap.timeformat_isdef[i] != this.timeformat_isdef[i]) {
					eq = false;
				}
			}
		} else {
			if (compareWrap.timeformat != this.timeformat)
				eq = false;
		}

	} else {
		eq = false;
	}
	return eq;
}

}
