/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisTitleEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCAxisTitle;
import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.util.legend.JCLegendEnumMappings;
import com.klg.jclass.chart.ChartText;

import java.awt.Font;



import java.awt.GraphicsEnvironment;

import java.awt.Insets;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.BorderFactory;




import java.util.Vector;


/**
 * Editor for Axis Title Properties.
 */

public class AxisTitleEditor extends RadioEditor
					  implements ItemListener, ActionListener, FocusListener {

protected JTextField text_tf;
protected JComboBox rotation_ch;
protected JComboBox placement_ch;
protected JComboBox fname_ch;
protected JComboBox fstyle_ch;
protected JComboBox fsize_ch;

private String fname_vals[];
static final private String[] fstyle_names = {"plain", "bold", "italic",
														"bold+italic"};
static final private int[] fstyle_vals = {Font.PLAIN, Font.BOLD, Font.ITALIC,
												Font.BOLD+Font.ITALIC };
static final private int[] fsize_vals = {3, 5, 8, 10, 12, 14, 18, 24, 28, 30, 32, 36, 48 };
protected String[] rotation_names;
protected int[] rotation_vals;
protected String[] placement_names;
protected int[] placement_vals;

public AxisTitleEditor() {
	super(MultiChart.AXIS_RADIO_TITLE, MultiChart.AXIS_RADIO_NAMES);

}

protected void setEnumArrays()
{
	rotation_names = JCChartEnumMappings.rotation_i18n_strings;
	rotation_vals = JCChartEnumMappings.rotation_values;
	placement_names = JCLegendEnumMappings.anchor_i18n_strings;
	placement_vals = JCLegendEnumMappings.anchor_values;
}

/**
 * Creates a panel containing the GUI elements for the properties
 * associated with the radio buttons.
 * @return the panel
 */
protected JPanel makePropertyPanel() 
{

    JPanel p2 = new JPanel();
    p2.setBorder(BorderFactory.createTitledBorder(
    	JCChartBeanBundle.string(JCChartBeanBundle.key249)));
	p2.setLayout(new GridLayout(2,1));

    JPanel p3 = new JPanel();
    p3.setBorder(BorderFactory.createTitledBorder(
    	JCChartBeanBundle.string(JCChartBeanBundle.key196)));
	p3.setLayout(new GridBagLayout());
	GridBagConstraints gbc3  = new GridBagConstraints();
	gbc3.gridwidth = 6;
	gbc3.gridx = GridBagConstraints.RELATIVE;
	gbc3.gridy = 0;
	gbc3.anchor = GridBagConstraints.WEST;
	gbc3.insets = new Insets(4,4,4,4);

    JPanel p4 = new JPanel();
    JPanel p5 = new JPanel();
    JPanel p6 = new JPanel();
	p4.setLayout(new GridLayout(2,1));

	p5.setLayout(new GridBagLayout());
	GridBagConstraints gbc5  = new GridBagConstraints();
	gbc5.gridwidth = 2;
	gbc5.gridx = GridBagConstraints.RELATIVE;
	gbc5.gridy = 0;
	gbc5.anchor = GridBagConstraints.WEST;
	gbc5.insets = new Insets(4,4,4,4);

	p6.setLayout(new GridBagLayout());
	GridBagConstraints gbc6  = new GridBagConstraints();
	gbc6.gridwidth = 4;
	gbc6.gridx = GridBagConstraints.RELATIVE;
	gbc6.gridy = 0;
	gbc6.anchor = GridBagConstraints.WEST;
	gbc6.insets = new Insets(4,4,4,4);

	fname_ch = new JComboBox();
	fstyle_ch = new JComboBox();
	fsize_ch = new JComboBox();
	rotation_ch = new JComboBox();
	placement_ch = new JComboBox();




    // Root through list of valid font names removing the sub-family names
    String tmp_fnames[] = GraphicsEnvironment.getLocalGraphicsEnvironment().
                                                getAvailableFontFamilyNames();
    int num_fnames = 0;
    for (int i = 0; i < tmp_fnames.length; i++) {
        if (tmp_fnames[i].indexOf('.') < 0)
            num_fnames++;
    }
    fname_vals = new String[num_fnames];
    int fname_ind = 0;
    for (int i = 0; i < tmp_fnames.length; i++) {
        if (tmp_fnames[i].indexOf('.') < 0) {
            fname_vals[fname_ind] = new String(tmp_fnames[i]);
            fname_ind++;
        }
    }

	setEnumArrays();

	for (int i = 0; i < fname_vals.length; i++)
		fname_ch.addItem(fname_vals[i]);
	for (int i = 0; i < fstyle_names.length; i++)
		fstyle_ch.addItem(fstyle_names[i]);
	for (int i = 0; i < fsize_vals.length; i++)
		fsize_ch.addItem("" + fsize_vals[i]);
	for (int i = 0; i < rotation_names.length; i++)
		rotation_ch.addItem(rotation_names[i]);
	for (int i = 0; i < placement_names.length; i++)
		placement_ch.addItem(placement_names[i]);
	placement_ch.addItem(JCChartBeanBundle.string(JCChartBeanBundle.key219));

	JLabel name_label = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key197));
	JLabel style_label = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key198));
	JLabel size_label = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key199));
	JLabel text_label = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key250));
	JLabel rotation_label = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key201));
	JLabel placement_label = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key222));
	text_tf = new JTextField(30);
	p3.add(name_label, gbc3);
	p3.add(fname_ch, gbc3);
	p3.add(style_label, gbc3);
	p3.add(fstyle_ch, gbc3);
	p3.add(size_label, gbc3);
	p3.add(fsize_ch, gbc3);

	p5.add(text_label, gbc5);
	p5.add(text_tf, gbc5);

	p6.add(rotation_label, gbc6);
	p6.add(rotation_ch, gbc6);
	p6.add(placement_label, gbc6);
	p6.add(placement_ch, gbc6);

	p4.add(p5);
	p4.add(p6);

    p2.add(p4);
    p2.add(p3);
	add("Center", p2);

	text_tf.addActionListener(this);
	text_tf.addFocusListener(this);
	fname_ch.addItemListener(this);
	fstyle_ch.addItemListener(this);
	fsize_ch.addItemListener(this);
	rotation_ch.addItemListener(this);
	placement_ch.addItemListener(this);

	return p2;
}

AxisTitleWrapper wrap = null;

public void setValue(Object value) {

    if (value != null) {
        try {
			wrap = (AxisTitleWrapper) value;
        } catch (ClassCastException e) {
        }
    }

	if (wrap == null)
		wrap = new AxisTitleWrapper(numOfRadios);

	switchValues(radioSel);
}

/**
 * Fills the property sheet with values associated with the newly selected
 * radio button.
 * @param sel the index of the newly selected radio button
 */
protected void switchValues(int sel)
{
	if (sel < 0 || wrap == null)
		return;
	text_tf.setText(wrap.text[sel]);
	for (int i = 0; i < rotation_vals.length; i++) {
		if (wrap.rotation[sel] == rotation_vals[i]) {
			if (rotation_ch.getSelectedIndex() != i)
				rotation_ch.setSelectedIndex(i);
			break;
		}
	}
	for (int i = 0; i < placement_vals.length; i++) {
		if (wrap.placement[sel] == placement_vals[i]) {
			if (placement_ch.getSelectedIndex() != i)
				placement_ch.setSelectedIndex(i);
			break;
		}
	}
	Font tmpFont = wrap.font[sel];
	if (tmpFont != null) {
		for (int i = 0; i < fname_vals.length; i++) {
			String cmpName = tmpFont.getName();
			if (cmpName.equalsIgnoreCase(fname_vals[i])) {
				if (fname_ch.getSelectedIndex() != i)
					fname_ch.setSelectedIndex(i);
				break;
			}
		}
		for (int i = 0; i < fstyle_vals.length; i++) {
			int cmpStyle = tmpFont.getStyle();
			if (cmpStyle == fstyle_vals[i]) {
				if (fstyle_ch.getSelectedIndex() != i)
					fstyle_ch.setSelectedIndex(i);
				break;
			}
		}
		for (int i = 0; i < fsize_vals.length; i++) {
			int cmpSize = tmpFont.getSize();
			if (cmpSize == fsize_vals[i]) {
				if (fsize_ch.getSelectedIndex() != i)
					fsize_ch.setSelectedIndex(i);
				break;
			}
		}
	}
}

public Object getValue() {
	return (Object) wrap;
}

public void setAsText(String text) throws IllegalArgumentException {
    Vector paramVector = BeanTextParser.parse(text);
    if (paramVector.size() != 4) {
        throw new IllegalArgumentException();
    }
    AxisTitleWrapper setWrapper = null;
    try {
        setWrapper = new AxisTitleWrapper(
            (String) paramVector.elementAt(0),
            (String) paramVector.elementAt(1),
            (String) paramVector.elementAt(2),
            (String) paramVector.elementAt(3)
            );
        setValue(setWrapper);
        support.firePropertyChange("", null, null);
    } catch (Exception e) {
        throw new IllegalArgumentException();
    }
}

public String getAsText() {
    if (wrap != null)
        return wrap.toString();
    else
        return null;
}

public String getJavaInitializationString() {
	return "new com.klg.jclass.chart.beans.AxisTitleWrapper(" + wrap + ")";
}

public Dimension getPreferredSize() {
    Dimension dim = super.getPreferredSize();
    dim.width = Math.max(dim.width, 500);
    dim.height += 250;
    return dim;
}

public void itemStateChanged( ItemEvent ev ) {
    super.itemStateChanged(ev);
    Object src;
    if( (src = ev.getSource()) instanceof JComboBox &&
			ev.getStateChange() != ItemEvent.DESELECTED) {
		checkProperties(src);
    }
}

public void focusGained(FocusEvent ev) {}

public void focusLost(FocusEvent ev) 
{
	Object src = ev.getSource();
	checkProperties(src);
}

public void actionPerformed( ActionEvent ev ) 
{
	Object src = ev.getSource();
	checkProperties(src);
}

public void checkProperties(Object src)
{
	try {
	if (src	== text_tf) {
		wrap.text[radioSel] = text_tf.getText();
		support.firePropertyChange("", null, null);
	} else if (src	== rotation_ch) {
		wrap.rotation[radioSel] = rotation_vals[rotation_ch.getSelectedIndex()];
		support.firePropertyChange("", null, null);
	} else if (src	== placement_ch) {
		int select = placement_ch.getSelectedIndex();
		// selection not in valua array -- return to default
		if (select >= placement_vals.length) {
			if (!wrap.placement_isdef[radioSel]) {
				wrap.placement_isdef[radioSel] = true;
				support.firePropertyChange("", null, null);
			}
		// accept selected value
		} else {
			if (wrap.placement[radioSel] != placement_vals[select]) {
				wrap.placement[radioSel] = placement_vals[select];
				wrap.placement_isdef[radioSel] = false;
				support.firePropertyChange("", null, null);
			}
		}
	} else if (src	== fname_ch || src == fstyle_ch || src == fsize_ch) {
		wrap.font[radioSel] = new Font(
			fname_vals[fname_ch.getSelectedIndex()],
			fstyle_vals[fstyle_ch.getSelectedIndex()],
			fsize_vals[fsize_ch.getSelectedIndex()] );
		support.firePropertyChange("", null, null);
	}
	} catch (Exception ex) {
	}
}

/* for testing purposes
public static void main( String args[] ) {
    JFrame frame;
    try {
    java.lang.Class newclass
        = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
    frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
        System.out.println(ex);
    frame = new JFrame();
    }

    AxisTitleEditor axisTitleEditor = new AxisTitleEditor();
    axisTitleEditor.setBackground( java.awt.Color.lightGray );
	AxisTitleWrapper thiswrap = new AxisTitleWrapper("", "", "", "");
    axisTitleEditor.setValue( thiswrap );
    frame.getContentPane().add( axisTitleEditor );
    frame.pack();
    java.awt.Dimension bigness = axisTitleEditor.getPreferredSize();
    frame.setSize( bigness.width + 100, bigness.height + 50 );
    frame.setVisible(true);
}
*/

}
