/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisTitleWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.util.legend.JCLegendEnumMappings;
import com.klg.jclass.chart.JCAxisTitle;
import com.klg.jclass.chart.ChartText;
import com.klg.jclass.util.legend.JCLegend;

import java.awt.Font;

/**
 * Wrapper class for Axis Title properties.  
 * Used by the MultiChart Bean.
 */
public class AxisTitleWrapper extends RadioAxisWrapper {

public String text[];
public int    rotation[];
public int    placement[];
public Font   font[];

public boolean placement_isdef[];

protected Font default_font = new Font("helvetica", Font.PLAIN, 12);

/**
 * Default constructor.  Members will be empty.
 */
public AxisTitleWrapper() {
	super();
}

/**
 * Constructor. Allocates space for the arrays but does not fill them.
 * @param size
 */
public AxisTitleWrapper(int size) {
	super();
	setWrapperArraySize(size);
}

/**
 * Constructor.  Allocates space for the arrays and fills them by parsing
 * series Strings of format <code>(x1 value)(y1 value)(x2 value)(y2 value)</code>.
 * @param textString       text String in above format.  Values are Strings.
 * @param rotationString   rotation String in above format.  Possible values
 * are <code>0</code>, <code>90</code>, <code>180</code>, or <code>270</code>.
 * @param placementString  placement String in above format.  Possible values
 * are <code>North</code>, <code>South</code>, <code>East</code>, <code>West</code>,
 * <code>Northeast</code>, <code>Northwest</code>, <code>Southeast</code>, or <code>Southwest</code>.
 *                         This property has intelligent defaults.  If a 
 *                         value is not provided for an axis, it will 
 *                         use them.
 * @param fontString       font String in above format.  Values should be
 *                         of the format <i>style-name-pointsize</i>.
 */
public AxisTitleWrapper(String textString, String rotationString,
    String placementString, String fontString)
{
    setWrapperArraySize(MultiChart.AXIS_RADIO_NAMES.length);

    setWrapperStringValues(text, new RadioSeries(textString));
    setWrapperEnumValues(rotation, new RadioSeries(rotationString),
        JCChartEnumMappings.rotation_strings, 
		JCChartEnumMappings.rotation_values,
        "AxisTitleRotation", ChartText.DEG_0);
    setWrapperEnumIsDefValues(placement, placement_isdef,
		new RadioSeries(placementString),
        JCLegendEnumMappings.anchor_strings, JCLegendEnumMappings.anchor_values,
        "AxisTitlePlacement", JCLegend.NORTH);
    setWrapperFontValues(font, new RadioSeries(fontString));
}

/**
 * Sets the size of the internal arrays that store properties for
 * axes (should be equal to the number of axes).
 * @param size the size of the internal arrays
 */
public void setWrapperArraySize(int size)
{
	text = new String[size];
	rotation = new int[size];
	placement = new int[size];
	font = new Font[size];
	// Set a default font so that we don't end up with null fonts in the array
	for (int i = 0; i < size; i++) {
		font[i] = default_font;
	}

	placement_isdef = new boolean[size];
}

/**
 * Stores the properties from 
 * the specified axis in internal arrays at the specified index.
 * @param index the index of the array that corresponds to this axis
 * @param axis the axis that contains the properties for this index
 */
public void setWrapperValue(int index, JCAxis axis)
{
	text[index] = axis.getTitle().getText();

	rotation[index] = axis.getTitle().getRotation();
	placement[index] = axis.getTitle().getPlacement();
	font[index] = axis.getTitle().getFont();
	// Set a default font so that we don't end up with null fonts in the array
	if (font[index] == null) {
		font[index] = default_font;
	}

	placement_isdef[index] = axis.getTitle().getPlacementIsDefault();
}

/**
 * Sets the properties on 
 * the specified axis from values stored in internal arrays 
 * at the specified index.
 * @param index the index where the values for this axis are stored
 * @param axis the axis that we need to set these values on
 */
public void setPropertyValue(int index, JCAxis axis)
{
	if (text[index] != null)
		axis.getTitle().setText(text[index]);

	// Watch out, this call can throw an exception.  If so, put the correct
	// value in the array.
	try {
		axis.getTitle().setRotation(rotation[index]);
	} catch (Exception e) {
		rotation[index] = axis.getTitle().getRotation();
	}
	
	if (placement_isdef[index]) {
		axis.getTitle().setPlacementIsDefault(placement_isdef[index]);
	} else {
		try {
			axis.getTitle().setPlacement(placement[index]);
		} catch (Exception e) {
			placement[index] = axis.getTitle().getPlacement();
		}
	}
	if (!(font[index].equals(axis.getTitle().getFont()))) {
		axis.getTitle().setFont(font[index]);
	}
}

/**
 * Converts properties to String for code generation.
 * @return String containing properties for code generation
 */
public String toString()
{
    String s = "";
    s += stringValuesToString(text);
    s += ",";
    s += enumValuesToString(rotation,
        JCChartEnumMappings.rotation_strings, 
		JCChartEnumMappings.rotation_values);
    s += ",";
    s += enumIsDefValuesToString(placement, placement_isdef,
        JCLegendEnumMappings.anchor_strings, 
		JCLegendEnumMappings.anchor_values);
    s += ",";
    s += fontValuesToString(font);

    return s;
}

public boolean equals(Object obj)
{
	boolean eq = true;
	if (obj instanceof AxisTitleWrapper) {
		AxisTitleWrapper compareWrap = (AxisTitleWrapper) obj;
		if (compareWrap.rotation != null && this.rotation != null) {
			for (int i = 0; i < compareWrap.rotation.length; i++) {
				if (compareWrap.rotation[i] != this.rotation[i])
					eq = false;
			}
		} else {
			if (compareWrap.rotation != this.rotation)
				eq = false;
		}
		if (compareWrap.placement != null && this.placement != null) {
			for (int i = 0; i < compareWrap.placement.length; i++) {
				if (compareWrap.placement[i] != this.placement[i])
					eq = false;
			}
		} else {
			if (compareWrap.placement != this.placement)
				eq = false;
		}

		if (compareWrap.text != null && this.text != null) {
			for (int i = 0; i < compareWrap.text.length; i++) {
				if (compareWrap.text[i] != null && this.text[i] != null) {
					if (!compareWrap.text[i].equals(this.text[i]))
						eq = false;
				} else {
					if (compareWrap.text[i] != this.text[i])
						eq = false;
				}
			}
		} else {
			if (compareWrap.text != this.text)
				eq = false;
		}
		if (compareWrap.font != null && this.font != null) {
			for (int i = 0; i < compareWrap.font.length; i++) {
				if (compareWrap.font[i] != null && this.font[i] != null) {
					if (!compareWrap.font[i].equals(this.font[i]))
						eq = false;
				} else {
					if (compareWrap.font[i] != this.font[i])
						eq = false;
				}
			}
		} else {
			if (compareWrap.font != this.font)
				eq = false;
		}

	} else {
		eq = false;
	}
	return eq;
}

}
