/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisValueLabelEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import java.awt.Insets;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Dimension;
import java.util.StringTokenizer;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextArea;
import javax.swing.BorderFactory;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;




import java.util.Vector;


/**
 * Editor for ValueLabel Axis Properties.
 */

public class AxisValueLabelEditor extends RadioEditor
					  implements DocumentListener {

protected JTextArea valuelabels_ta;

public static final String explanation = 
		JCChartBeanBundle.string(JCChartBeanBundle.key251);

protected String editString;

public AxisValueLabelEditor() {
	super(MultiChart.AXIS_RADIO_TITLE, MultiChart.AXIS_RADIO_NAMES);
}

/**
 * Creates a panel containing the GUI elements for the properties
 * associated with the radio buttons.
 * @return the panel
 */
protected JPanel makePropertyPanel() 
{

    JPanel p2 = new JPanel();
    p2.setBorder(BorderFactory.createTitledBorder(
    	JCChartBeanBundle.string(JCChartBeanBundle.key252)));
	p2.setLayout(new GridLayout(1, 1));

    valuelabels_ta = new JTextArea(explanation);

	p2.add(valuelabels_ta);

	add("Center", p2);

	valuelabels_ta.getDocument().addDocumentListener(this);

	return p2;
}

AxisValueLabelWrapper wrap = null;

public void setValue(Object value) {

    if (value != null) {
        try {
			wrap = (AxisValueLabelWrapper) value;
        } catch (ClassCastException e) {
        }
    }

	if (wrap == null)
		wrap = new AxisValueLabelWrapper(numOfRadios);

	if (editString == null) {
		switchValues(radioSel);
	}
}

/**
 * Fills the property sheet with values associated with the newly selected
 * radio button.
 * @param sel the index of the newly selected radio button
 */
protected void switchValues(int sel)
{
	if (sel < 0 || wrap == null)
		return;

	StringBuffer sb = new StringBuffer(explanation);
	sb.append('\n');
	sb.append(wrap.valuelabels[sel]);
	editString = sb.toString();
	valuelabels_ta.setText(editString);
}

public Object getValue() {
	return (Object) wrap;
}

public void setAsText(String text) throws IllegalArgumentException {
    Vector paramVector = BeanTextParser.parse(text);
    if (paramVector.size() != 1) {
        throw new IllegalArgumentException();
    }
    AxisValueLabelWrapper setWrapper = null;
    try {
        setWrapper = new AxisValueLabelWrapper(
            (String) paramVector.elementAt(0)
            );
        setValue(setWrapper);
        support.firePropertyChange("", null, null);
    } catch (Exception e) {
        throw new IllegalArgumentException();
    }
}

public String getAsText() {
    if (wrap != null)
        return wrap.toString();
    else
        return null;
}

public String getJavaInitializationString() {
	return "new com.klg.jclass.chart.beans.AxisValueLabelWrapper(" + wrap + ")";
}

public Dimension getPreferredSize() {
    Dimension dim = super.getPreferredSize();
    dim.width = Math.max(dim.width, 400);
    dim.height += 250;
    return dim;
}

public void changedUpdate( DocumentEvent ev )
{
    Object src = ev.getDocument();
    checkProperties(src);
}
public void insertUpdate( DocumentEvent ev )
{
    Object src = ev.getDocument();
    checkProperties(src);
}
public void removeUpdate( DocumentEvent ev )
{
    Object src = ev.getDocument();
    checkProperties(src);
}

public void checkProperties(Object src)
{
	try {
	if (src	== valuelabels_ta.getDocument()) {
		editString = valuelabels_ta.getText();

        StringTokenizer strtok = new StringTokenizer(editString, "\n\r");
		StringBuffer sb = new StringBuffer();
		boolean firstTime = true;
        while (strtok.hasMoreTokens()) {
            String line = strtok.nextToken();

            if (line.charAt(0) == '#')
                continue;

			if (firstTime)
				firstTime = false;
			else
				sb.append('\n');
			sb.append(line);
		}
		wrap.valuelabels[radioSel] = sb.toString();
		support.firePropertyChange("", null, null);
	}
	} catch (Exception ex) {
	}
}

/* for testing purposes
public static void main( String args[] ) {
    JFrame frame;
    try {
    java.lang.Class newclass
        = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
    frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
        System.out.println(ex);
    frame = new JFrame();
    }

    AxisValueLabelEditor axisValueLabelEditor = new AxisValueLabelEditor();
    axisValueLabelEditor.setBackground( java.awt.Color.lightGray );
    AxisValueLabelWrapper thiswrap = new AxisValueLabelWrapper("");
    axisValueLabelEditor.setValue( thiswrap );
    frame.getContentPane().add( axisValueLabelEditor );
    frame.pack();
    java.awt.Dimension bigness = axisValueLabelEditor.getPreferredSize();
    frame.setSize( bigness.width + 100, bigness.height + 50 );
    frame.setVisible(true);
}
*/

}
