/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisValueLabelWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCValueLabel;
import java.util.Date;
import java.util.StringTokenizer;




import java.util.Vector;


/**
 * Wrapper class for Axis Value Label properties.  
 * Used by the MultiChart Bean.
 */
public class AxisValueLabelWrapper extends RadioAxisWrapper {

public String	valuelabels[];

/**
 * Default constructor.  Members will be empty.
 */
public AxisValueLabelWrapper() {
	super();
}

/**
 * Constructor. Allocates space for the arrays but does not fill them.
 * @param size
 */
public AxisValueLabelWrapper(int size) {
	super();
	setWrapperArraySize(size);
}

/**
 * Constructor. Allocates space for the arrays and fills them by parsing
 * series Strings of format <code>(x1 value)(y1 value)(x2 value)(y2 value)</code>.
 * @param valueLabelString  value label String in above format.  Values
 *                          are a Strings, each a list of point label pairs
 *                          separated by a newline (<code>\n</code>).  For example, the
 *                          series String might look like :
 *                          <pre>(x1 1.0,one\n2.0,two\n3.0,three)</pre>
 */
public AxisValueLabelWrapper(String valueLabelString)
{
	setWrapperArraySize(MultiChart.AXIS_RADIO_NAMES.length);

	setWrapperStringValues(valuelabels, new RadioSeries(valueLabelString));
}

/**
 * Sets the size of the internal arrays that store properties for
 * axes (should be equal to the number of axes).
 * @param size the size of the internal arrays
 */
public void setWrapperArraySize(int size)
{
	valuelabels = new String[size];
}

/**
 * Stores the properties from 
 * the specified axis in internal arrays at the specified index.
 * @param index the index of the array the corresponds to this axis
 * @param axis the axis that contains the properties for this index
 */
public void setWrapperValue(int index, JCAxis axis)
{
	valuelabels[index] = valueLabelsToString(axis.getValueLabels());
}

/**
 * Sets the properties on 
 * the specified axis from values stored in internal arrays 
 * at the specified index.
 * @param index the index where the values for this axis are stored
 * @param axis the axis that we need to set these values on
 */
public void setPropertyValue(int index, JCAxis axis)
{
	axis.setValueLabels(stringToValueLabels(valuelabels[index]));
}

/**
 * Convers properties to String for code generation.
 * @return String containing properties for code generation
 */
public String toString()
{
    String s = "";
    s += stringValuesToString(valuelabels);

	return s;
}

/**
 * Converts list of value labels to a String for code generation.
 * @param valList array of <code>JCValueLabel</code> instances to convert
 * @return String containing list of value labels. Each line is of
 * the format <i>point,label</i>, with one pair per line.
 */
public String valueLabelsToString(JCValueLabel[] valList)
{
	StringBuffer sb = new StringBuffer();
    if (valList != null) {
        for (int i = 0; i < valList.length; i++) {
            sb.append(valList[i].getValue());
            sb.append(" , ");
            sb.append(valList[i].getText());
            sb.append('\n');
        }
    }
	return sb.toString();
}

/**
 * Converts String to list of value labels.
 * @param s String containing entire value label list.
 *        Each line has a pair of the form <i>value,label</i>.
 * @return array of <code>JCValueLabels</code> representing the list of value labels
 */
public JCValueLabel[] stringToValueLabels(String s)
{
    Vector valList = new Vector();
    StringTokenizer strtok = new StringTokenizer(s, "\n\r");
    while (strtok.hasMoreTokens()) {
        String line = strtok.nextToken();

        if (line.charAt(0) == '#')
            continue;

        int commapos;
        if ((commapos = line.indexOf(',')) < 0)
            continue;

        try {
            valList.addElement(new JCValueLabel(
               Double.valueOf(
                    line.substring(0, commapos).trim()).doubleValue(),
                line.substring(commapos + 1).trim())
                );
        } catch (Exception ex) {
            //ex.printStackTrace();
        }

    }
    if (valList.size() == 0)
        return null;
    else {
        JCValueLabel[] valArr = new JCValueLabel[valList.size()];
        for (int i = 0; i < valList.size(); i++) {
            valArr[i] = (JCValueLabel) valList.elementAt(i);
        }
		return valArr;
    }
}


public boolean equals(Object obj)
{
	boolean eq = true;
	if (obj instanceof AxisValueLabelWrapper) {
		AxisValueLabelWrapper compareWrap = (AxisValueLabelWrapper) obj;
		if (compareWrap.valuelabels != null && this.valuelabels != null) {
			for (int i = 0; i < compareWrap.valuelabels.length; i++) {
				if (compareWrap.valuelabels[i] != null && this.valuelabels[i] != null) {
					if (!compareWrap.valuelabels[i].equals(this.valuelabels[i]))
						eq = false;
				} else {
					if (compareWrap.valuelabels[i] != this.valuelabels[i])
						eq = false;
				}
			}
		} else {
			if (compareWrap.valuelabels != this.valuelabels)
				eq = false;
		}

	} else {
		eq = false;
	}
	return eq;
}

}
