/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: BaseChart.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.ChartDataSupport;
import com.klg.jclass.chart.JCChartArea;
import com.klg.jclass.chart.data.JCChartSwingDataSource;
import com.klg.jclass.chart.ChartDataModel;
import com.klg.jclass.chart.Changeable;
import com.klg.jclass.chart.ChartDataView;
import com.klg.jclass.chart.JCChartUtil;

import com.klg.jclass.util.JCStringTokenizer;
import com.klg.jclass.util.legend.JCLegend;

import java.util.StringTokenizer;

import java.awt.Font;

import javax.swing.table.TableModel;
import javax.swing.JComponent;
import javax.swing.JLabel;

/**
 * Base class for most chart Beans. Wraps many useful chart and chart subclass
 * properties in simple get/set methods. Subclassed by a number of chart
 * Bean classes, such as <code>SimpleChart</code> and the databound
 * chart Beans.
 */
public abstract class BaseChart extends JCChart {

/** Enums for axis orientation values. */
public final static int ORIENT_X_NORMAL_Y_NORMAL = 0;
public final static int ORIENT_X_REVERSED_Y_NORMAL = 1;
public final static int ORIENT_X_NORMAL_Y_REVERSED = 2;
public final static int ORIENT_X_REVERSED_Y_REVERSED = 3;
public final static int ORIENT_INVERTED_X_NORMAL_Y_NORMAL = 4;
public final static int ORIENT_INVERTED_X_REVERSED_Y_NORMAL = 5;
public final static int ORIENT_INVERTED_X_NORMAL_Y_REVERSED = 6;
public final static int ORIENT_INVERTED_X_REVERSED_Y_REVERSED = 7;

/** Array of possible axis orientation values. */
public final static int[] axesOrientationValues = {
			ORIENT_X_NORMAL_Y_NORMAL,
			ORIENT_INVERTED_X_NORMAL_Y_NORMAL,
			ORIENT_X_REVERSED_Y_NORMAL,
			ORIENT_INVERTED_X_REVERSED_Y_NORMAL,
			ORIENT_X_NORMAL_Y_REVERSED,
			ORIENT_INVERTED_X_NORMAL_Y_REVERSED,
			ORIENT_X_REVERSED_Y_REVERSED,
			ORIENT_INVERTED_X_REVERSED_Y_REVERSED };

/** Array of matching String descriptions for axis orientation values. */
public final static String[] axesOrientationStrings = {
			"ORIENT_X_NORMAL_Y_NORMAL",
			"ORIENT_INVERTED_X_NORMAL_Y_NORMAL",
			"ORIENT_X_REVERSED_Y_NORMAL",
			"ORIENT_INVERTED_X_REVERSED_Y_NORMAL",
			"ORIENT_X_NORMAL_Y_REVERSED",
			"ORIENT_INVERTED_X_NORMAL_Y_REVERSED",
			"ORIENT_X_REVERSED_Y_REVERSED",
			"ORIENT_INVERTED_X_REVERSED_Y_REVERSED" };

/** Enums for annotation rotation values. */
public final static int ROTATE_NONE = JCAxis.ROTATE_NONE;
public final static int ROTATE_90  =  JCAxis.ROTATE_90;
public final static int ROTATE_270 = JCAxis.ROTATE_270;

/** Enums for axis placement values. */
public final static int AUTOMATIC = JCAxis.AUTOMATIC;
public final static int ORIGIN    = JCAxis.ORIGIN;
public final static int MIN       = JCAxis.MIN;
public final static int MAX       = JCAxis.MAX;
public final static int VALUE_ANCHORED = JCAxis.VALUE_ANCHORED;

/** Enums for annotation method values. */
public final static int VALUE        = JCAxis.VALUE;
public final static int VALUE_LABELS = JCAxis.VALUE_LABELS;
public final static int TIME_LABELS  = JCAxis.TIME_LABELS;
public final static int POINT_LABELS = JCAxis.POINT_LABELS;

/** Enums for time unit method values. */
final public static int SECONDS = 0;
final public static int MINUTES = 1;
final public static int HOURS   = 2;
final public static int DAYS    = 3;
final public static int WEEKS   = 4;
final public static int MONTHS  = 5;
final public static int YEARS   = 6;

/** Array of time unit values. */
final static int[] timeUnit_intvalues = { SECONDS, MINUTES, HOURS, DAYS,
										  WEEKS, MONTHS, YEARS };

/** Enums for legend anchor values. */
public final static int  NORTHEAST = JCLegend.NORTHEAST;
public final static int  NORTH     = JCLegend.NORTH;
public final static int  NORTHWEST = JCLegend.NORTHWEST;
public final static int  EAST      = JCLegend.EAST;
public final static int  WEST      = JCLegend.WEST;
public final static int  SOUTHEAST = JCLegend.SOUTHEAST;
public final static int  SOUTH     = JCLegend.SOUTH;
public final static int  SOUTHWEST = JCLegend.SOUTHWEST;

/** Enums for legend orientation values. */
public final static int  HORIZONTAL= JCLegend.HORIZONTAL;
public final static int  VERTICAL  = JCLegend.VERTICAL;

/** <code>TableModel</code> used to populate data into this chart, if one is being used. */
public TableModel tableModel = null;

/**
 * <code>JCSwingDataSource</code> used to populate data into this chart,
 * if one is being used.
 */
public ChartDataSupport dataSource = null;

/**
 * Basic constructor.
 */
public BaseChart() {
	super();
}

/************** JCChart properties *************/

/**
 * Gets the value of the <code>chartType</code> property, which determines the chart
 * type of the first set of data in the chart.
 * @return value of the <code>chartType</code> property
 */
public int getChartType() {
	return getDataView(0).getChartType();
}

/**
 * Sets the value of the <code>chartType</code> property, which determines the chart
 * type of the first set of data in the chart.
 * @param ct
 */
public void setChartType(int ct) {
	try {
	getDataView(0).setChartType(ct);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}



/**
 * Gets the value of the <code>axisOrientation</code> property, which determines whether the
 * <i>x</i> and <i>y</i> axes are inverted or not, and whether the axes are
 * reversed or not.
 * <P>Possible axis orientation values are: <BR><code>ORIENT_X_NORMAL_Y_NORMAL</code>
 * <BR><code>ORIENT_INVERTED_X_NORMAL_Y_NORMAL</code>
 * <BR><code>ORIENT_X_REVERSED_Y_NORMAL</code>
 * <BR><code>ORIENT_INVERTED_X_REVERSED_Y_NORMAL</code>
 * <BR><code>ORIENT_X_NORMAL_Y_REVERSED</code>
 * <BR><code>ORIENT_INVERTED_X_NORMAL_Y_REVERSED</code>
 * <BR><code>ORIENT_X_REVERSED_Y_REVERSED</code> or
 * <BR><code>ORIENT_INVERTED_X_REVERSED_Y_REVERSED</code>.
 * @return value of the <code>axisOrientation</code> property
 */
public int getAxisOrientation() {
	// dependent on current constant values
	int orient = ORIENT_X_NORMAL_Y_NORMAL;
	if (getDataView(0).isInverted()) {
		orient += ORIENT_INVERTED_X_NORMAL_Y_NORMAL;
	}
	if (getChartArea().getXAxis(0).isReversed()) {
		orient += ORIENT_X_REVERSED_Y_NORMAL;
	}
	if (getChartArea().getYAxis(0).isReversed()) {
		orient += ORIENT_X_NORMAL_Y_REVERSED;
	}
	return orient;
}

/**
 * Sets the value of the <code>axisOrientation</code> property, which determines whether
 * the <i>x</i> and <i>y</i> axes are inverted or not, and whether the axes are
 * reversed or not.
 * <P>Possible axis orientation values are: <BR><code>ORIENT_X_NORMAL_Y_NORMAL</code>
 * <BR><code>ORIENT_INVERTED_X_NORMAL_Y_NORMAL</code>
 * <BR><code>ORIENT_X_REVERSED_Y_NORMAL</code>
 * <BR><code>ORIENT_INVERTED_X_REVERSED_Y_NORMAL</code>
 * <BR><code>ORIENT_X_NORMAL_Y_REVERSED</code>
 * <BR><code>ORIENT_INVERTED_X_NORMAL_Y_REVERSED</code>
 * <BR><code>ORIENT_X_REVERSED_Y_REVERSED</code> or
 * <BR><code>ORIENT_INVERTED_X_REVERSED_Y_REVERSED</code>.
 * @param newOrient
 */
public void setAxisOrientation(int newOrient) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	// dependent on current constant values
	if (newOrient > ORIENT_X_REVERSED_Y_REVERSED) {
		getDataView(0).setInverted(true);
		newOrient -= ORIENT_INVERTED_X_NORMAL_Y_NORMAL;
	}
	else {
		getDataView(0).setInverted(false);
	}
	if (newOrient > ORIENT_X_REVERSED_Y_NORMAL) {
		getChartArea().getYAxis(0).setReversed(true);
		newOrient -= ORIENT_X_NORMAL_Y_REVERSED;
	}
	else {
		getChartArea().getYAxis(0).setReversed(false);
	}
	getChartArea().getXAxis(0).setReversed(
		newOrient == ORIENT_X_NORMAL_Y_NORMAL ? false : true);

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/************** Chart Area *************/

/**
 * Gets the value of the <TT>View3D</TT> property, which combines the
 * values of the <TT>Depth</TT>, <TT>Elevation</TT>, and <TT>Rotation</TT>
 * properties defined in <TT>JCChartArea</TT>.
 * <BR><TT>Depth</TT> controls the apparent depth of a graph.
 * <TT>Elevation</TT> controls the distance above the <i>x</i> axis
 * for the 3D effect. <TT>Rotation</TT> controls the position of the
 * eye relative to the <i>y</i> axis for the 3D effect.
 * @return    value of <TT>Depth</TT>, <TT>Elevation</TT>, and <TT>Rotation</TT>
 * properties respectively, separated by commas
 * @see       JCChartArea#getDepth
 * @see       JCChartArea#getElevation
 * @see       JCChartArea#getRotation
 */
public String getView3D() {
	JCChartArea ca = getChartArea();
	return ""+ca.getDepth()+","+ca.getElevation()+","+ca.getRotation();
}

/**
 * Sets the value of the <TT>View3D</TT> property, which controls the
 * apparent depth of a graph by combining the values <TT>Depth</TT>,
 * <TT>Elevation</TT>, and <TT>Rotation</TT> properties
 * defined in <TT>JCChartArea</TT>.
 * <BR><TT>Depth</TT> controls the apparent depth of a graph.
 * <TT>Elevation</TT> controls the distance above the <i>x</i> axis
 * for the 3D effect. <TT>Rotation</TT> controls the position of the
 * eye relative to the <i>y</i> axis for the 3D effect.
 * @param     s value of <TT>Depth</TT>, <TT>Elevation</TT>, and <TT>Rotation</TT>
 * properties respectively, separated by commas
 * @see       JCChartArea#setDepth
 * @see       JCChartArea#setElevation
 * @see       JCChartArea#setRotation
 */
public void setView3D(String s) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	JCChartArea ca = getChartArea();
	StringTokenizer st = new StringTokenizer(s, ",");
	for (int j = 0; j < 3; j++) {
		int i = 0;
		if (st.hasMoreTokens()) i = Integer.parseInt(st.nextToken());
		switch (j) {
		case 0:
			ca.setDepth(i); break;
		case 1:
			ca.setElevation(i); break;
		case 2:
			ca.setRotation(i); break;
		}
	}

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}

}

/************* X Axis Properties ***********/

/**
 * Gets the value of the <code>XAxisLogarithmic</code> property, which determines
 * whether the first <i>x</i> axis will be logarithmic (<code>true</code>) or
 * linear (<code>false</code>).
 * @return value of the <code>XAxisLogarithmic</code> property
 */
public boolean getXAxisLogarithmic() {
	return getChartArea().getXAxis(0).isLogarithmic();
}

/**
 * Sets the value of the <code>XAxisLogarithmic</code> property, which determines
 * whether the first <i>x</i> axis will be logarithmic (<code>true</code>) or
 * linear (<code>false</code>).
 * @param b
 */
public void setXAxisLogarithmic(boolean b) {
	try {
	getChartArea().getXAxis(0).setLogarithmic(b);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/**
 * Gets the value of the <code>XAxisVisible</code> property, which determines whether
 * the first <i>x</i> axis is currently visible.
 * @return value of the <code>XAxisVisible</code> property
 */
public boolean getXAxisVisible() {
	return getChartArea().getXAxis(0).isVisible();
}

/**
 * Sets the value of the <code>XAxisVisible</code> property, which determines whether
 * the first <i>x</i> axis is currently visible.
 * @param b
 */
public void setXAxisVisible(boolean b) {
	try {
	getChartArea().getXAxis(0).setVisible(b);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/**
 * Gets the value of the <code>XAxisAnnotationMethod</code> property, which determines
 * how axis annotations are generated.
 * @return one of <code>VALUE</code>, <code>VALUE_LABELS</code>,
 * <code>TIME_LABELS</code>, or <code>POINT_LABELS</code>
 */
public int getXAxisAnnotationMethod() {
	return getChartArea().getXAxis(0).getAnnotationMethod();
}


/**
 * Sets the value of the <code>XAxisAnnotationMethod</code> property, which determines
 * how axis annotations are generated.
 * @param r one of <code>VALUE</code>, <code>VALUE_LABELS</code>,
 * <code>TIME_LABELS</code>, or <code>POINT_LABELS</code>
 * @exception IllegalArgumentException If the <code>AnnotationMethod</code> value
 * is not one of <code>VALUE</code>, <code>VALUE_LABELS</code>, <code>TIME_LABELS</code>, or
 * <code>POINT_LABELS</code>.
 */
public void setXAxisAnnotationMethod(int r) {
	try {
	getChartArea().getXAxis(0).setAnnotationMethod(r);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/**
 * Gets the value of the <code>XAxisNumSpacing</code> property -- this property controls the
 * interval between axis labels.
 * @return value of the <code>XAxisNumSpacing</code> property
 */
public String getXAxisNumSpacing() {
	if (getChartArea().getXAxis(0).getNumSpacingIsDefault()) return "";
	return "" + getChartArea().getXAxis(0).getNumSpacing();
}

/**
 * Sets the value of the <code>XAxisNumSpacing</code> property -- this property controls
 * the interval between axis labels.
 * @param r
 */
public void setXAxisNumSpacing(String r) {
	try {
	JCAxis ax = getChartArea().getXAxis(0);
	String s = (r == null) ? null : r.trim();
	if (s == null || s.length() == 0) {
		ax.setNumSpacingIsDefault(true);
		return;
	}
	double dr = Double.valueOf(s).doubleValue();
	ax.setNumSpacing(dr);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/**
 * Gets the value of the <code>XAxisGridVisible</code> property, which determines whether
 * a grid is drawn for the axis.
 * @return value of the <code>XAxisGridVisible</code> property
 */
public boolean getXAxisGridVisible() {
	return getChartArea().getXAxis(0).isGridVisible();
}

/**
 * Sets the value of the <code>XAxisGridVisible</code> property, which determines whether
 * a grid is drawn for the axis.
 * @param s
 *
 */
public void setXAxisGridVisible(boolean s) {
	try {
	getChartArea().getXAxis(0).setGridVisible(s);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/**
 * Gets the value of the <code>XAxisMinMax</code> property, which controls both the
 * <code>XAxisMin</code> and <code>XAxisMax</code> properties.
 * @return value of the <code>XAxisMinMax</code> property
 */
public String getXAxisMinMax() {
	return "" + getXAxisMin() + "," + getXAxisMax();
}

/**
 * Sets the value of the <code>XAxisMinMax</code> property, which controls
 * both the <code>XAxisMin</code> and <code>XAxisMax</code> properties.
 * @param r
 */
public void setXAxisMinMax(String r) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	JCStringTokenizer st = new JCStringTokenizer(r);
	String sMin = st.nextToken(',');
	String sMax = st.nextToken(',');
	setXAxisMin(sMin);
	setXAxisMax(sMax);

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/**
 * Gets the value of the <code>XAxisMin</code> property; this property controls the
 * minimum value shown on the axis.
 * @return value of the <code>XAxisMin</code> property
 */
public String getXAxisMin() {
	if (getChartArea().getXAxis(0).getMinIsDefault()) return "";
	return "" + getChartArea().getXAxis(0).getMin();
}

/**
 * Sets the value of the <code>XAxisMin</code> property; this property
 * controls the minimum value shown on the axis.
 * @param r
 */
public void setXAxisMin(String r) {
	try {
	JCAxis ax = getChartArea().getXAxis(0);
	String s = (r == null) ? null : r.trim();
	if (s == null || s.length() == 0) {
		ax.setMinIsDefault(true);
		return;
	}
	double dr = Double.valueOf(s).doubleValue();
	ax.setMin(dr);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/**
 * Gets the value of the <code>XAxisMax</code> property, which controls the
 * maximum value shown on the axis.
 * @return value of the <code>XAxisMax</code> property
 */
public String getXAxisMax() {
	if (getChartArea().getXAxis(0).getMaxIsDefault()) return "";
	return "" + getChartArea().getXAxis(0).getMax();
}

/**
 * Sets the value of the <code>XAxisMax</code> property, which controls the
 * maximum value shown on the axis.
 * @param r
 */
public void setXAxisMax(String r) {
	try {
	JCAxis ax = getChartArea().getXAxis(0);
	String s = (r == null) ? null : r.trim();
	if (s == null || s.length() == 0) {
		ax.setMaxIsDefault(true);
		return;
	}
	double dr = Double.valueOf(s).doubleValue();
	ax.setMax(dr);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/**
 * Gets the value of the <code>XAxisTitleText</code> property, which specifies
 * the text that will appear as the <i>x</i> axis title.
 * @return value of the <code>XAxisTitleText</code> property
 */
public String getXAxisTitleText() {
	String retStr = getChartArea().getXAxis(0).getTitle().getText();
	if (retStr == null)
		retStr = new String("");
	return retStr;
}

/**
 * Sets the value of the <code>XAxisTitleText</code> property, which specifies
 * the text that will appear as the <i>x</i> axis title.
 * @param t
 */
public void setXAxisTitleText(String t) {
	try {
	getChartArea().getXAxis(0).getTitle().setText(t, true);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/******************** Y Axis ************************/


/**
 * Gets the value of the <code>YAxisLogarithmic</code> property, which determines
 * whether the first <i>y</i> axis will be logarithmic (<code>true</code>) or
 * linear (<code>false</code>).
 * @return value of the <code>YAxisLogarithmic</code> property
 */
public boolean getYAxisLogarithmic() {
	return getChartArea().getYAxis(0).isLogarithmic();
}

/**
 * Sets the value of the <code>YAxisLogarithmic</code> property, which
 * determines whether the first <i>y</i> axis will be logarithmic
 * (<code>true</code>) or linear (<code>false</code>).
 * @param b
 */
public void setYAxisLogarithmic(boolean b) {
	try {
	getChartArea().getYAxis(0).setLogarithmic(b);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/**
 * Gets the value of the <code>YAxisVisible</code> property -- this property determines
 * whether the first <i>y</i> axis is currently visible.
 * @return value of the <code>YAxisVisible</code> property
 */
public boolean getYAxisVisible() {
	return getChartArea().getYAxis(0).isVisible();
}

/**
 * Sets the value of the <code>YAxisVisible</code> property -- this property
 * determines whether the first <i>y</i> axis is currently visible.
 * @param b
 */
public void setYAxisVisible(boolean b) {
	try {
	getChartArea().getYAxis(0).setVisible(b);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/**
 * Gets the value of the <code>YAxisAnnotationMethod</code> property, which determines
 * how axis annotations are generated.
 * @return one of <code>VALUE</code>, <code>VALUE_LABELS</code>,
 * <code>TIME_LABELS</code>, or <code>POINT_LABELS</code>
 */
public int getYAxisAnnotationMethod() {
	return getChartArea().getYAxis(0).getAnnotationMethod();
}

/**
 * Sets the value of the <code>YAxisAnnotationMethod property, which determines
 * how axis annotations are generated.
 * @param r one of <code>VALUE</code>, <code>VALUE_LABELS</code>,
 * <code>TIME_LABELS</code>, or <code>POINT_LABELS</code>
 * @exception IllegalArgumentException If the <code>AnnotationMethod</code> value
 * is not one of <code>VALUE</code>, <code>VALUE_LABELS</code>, <code>TIME_LABELS</code>, or
 * <code>POINT_LABELS</code>.
 */
public void setYAxisAnnotationMethod(int r) {
	try {
		getChartArea().getYAxis(0).setAnnotationMethod(r);
	}
	catch( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/**
 * Gets the value of the <code>YAxisNumSpacing</code> property, which controls
 * the interval between axis labels.
 * @return value of the <code>YAxisNumSpacing</code> property
 */
public String getYAxisNumSpacing() {
	if (getChartArea().getYAxis(0).getNumSpacingIsDefault()) return "";
	return "" + getChartArea().getYAxis(0).getNumSpacing();
}

/**
 * Sets the value of the <code>YAxisNumSpacing</code> property, which controls
 * the interval between axis labels.
 * @param r
 */
public void setYAxisNumSpacing(String r) {
	try {
	JCAxis ax = getChartArea().getYAxis(0);
	String s = (r == null) ? null : r.trim();
	if (s == null || s.length() == 0) {
		ax.setNumSpacingIsDefault(true);
		return;
	}
	double dr = Double.valueOf(s).doubleValue();
	ax.setNumSpacing(dr);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/**
 * Gets the value of the <code>YAxisGridVisible</code> property, which determines
 * whether a grid is drawn for the axis.
 * @return value of the <code>YAxisGridVisible</code> property
 */
public boolean getYAxisGridVisible() {
	return getChartArea().getYAxis(0).isGridVisible();
}

/**
 * Sets the value of the <code>YAxisGridVisible</code> property, which determines
 * whether a grid is drawn for the axis.
 * @param s
 */
public void setYAxisGridVisible(boolean s) {
	try {
	getChartArea().getYAxis(0).setGridVisible(s);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/**
 * Gets the value of the <code>YAxisMinMax</code> property, which controls both
 * the YAxisMin and YAxisMax properties.
 * @return value of the <code>YAxisMinMax</code> property
 */
public String getYAxisMinMax() {
	return "" + getYAxisMin() + "," + getYAxisMax();
}

/**
 * Sets the value of the <code>YAxisMinMax</code> property, which controls both
 * the <code>YAxisMin</code> and <code>YAxisMax</code> properties.
 * @param r
 */
public void setYAxisMinMax(String r) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	JCStringTokenizer st = new JCStringTokenizer(r);
	String sMin = st.nextToken(',');
	String sMax = st.nextToken(',');
	setYAxisMin(sMin);
	setYAxisMax(sMax);

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/**
 * Gets the value of the <code>YAxisMin</code> property -- this property
 * controls the minimum value shown on the axis.
 * @return value of the <code>YAxisMin</code> property
 */
public String getYAxisMin() {
	if (getChartArea().getYAxis(0).getMinIsDefault()) return "";
	return "" + getChartArea().getYAxis(0).getMin();
}

/**
 * Sets the value of the <code>YAxisMin</code> property -- this property controls
 * the minimum value shown on the axis.
 * @param r
 */
public void setYAxisMin(String r) {
	try {
	JCAxis ax = getChartArea().getYAxis(0);
	String s = (r == null) ? null : r.trim();
	if (s == null || s.length() == 0) {
		ax.setMinIsDefault(true);
		return;
	}
	double dr = Double.valueOf(s).doubleValue();
	ax.setMin(dr);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/**
 * Gets the value of the <code>YAxisMax</code> property -- this property controls
 * the maximum value shown on the axis.
 * @return value of the <code>YAxisMax</code> property
 */
public String getYAxisMax() {
	if (getChartArea().getYAxis(0).getMaxIsDefault()) return "";
	return "" + getChartArea().getYAxis(0).getMax();
}

/**
 * Sets the value of the <code>YAxisMax</code> property -- this property controls
 * the maximum value shown on the axis.
 * @param r
 */
public void setYAxisMax(String r) {
	try {
	JCAxis ax = getChartArea().getYAxis(0);
	String s = (r == null) ? null : r.trim();
	if (s == null || s.length() == 0) {
		ax.setMaxIsDefault(true);
		return;
	}
	double dr = Double.valueOf(s).doubleValue();
	ax.setMax(dr);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/**
 * Gets the value of the <code>YAxisTitleText</code> property, which specifies
 * the text that will appear as the <i>y</i> axis title.
 * @return value of the <code>YAxisTitleText</code> property
 */
public String getYAxisTitleText() {
	String retStr = getChartArea().getYAxis(0).getTitle().getText();
	if (retStr == null)
		retStr = new String("");
	return retStr;
}

/**
 * Sets the value of the <code>YAxisTitleText</code> property, which specifies
 * the text that will appear as the <i>y</i> axis title.
 * @param t
 */
public void setYAxisTitleText(String t) {
	try {
	getChartArea().getYAxis(0).getTitle().setText(t, true);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/****************** Header ******************/

/**
 * Gets the value of the <code>HeaderText</code> property; the <code>HeaderText</code> property
 * holds the text that is displayed in the header.
 * @return value of the <code>HeaderText</code> property
 */
public String getHeaderText() {
	JComponent header = getHeader();
	String retStr = null;
	if (header instanceof JLabel) {
		retStr = ((JLabel)header).getText();
	}
	if (retStr == null)
		retStr = new String("");
	return retStr;
}

/**
 * Sets the value of the <code>HeaderText</code> property; the <code>HeaderText</code> property
 * holds the text that is displayed in the header. It also dictates
 * the <code>Visible</code> property.
 * @param t
 */
public void setHeaderText(String t) {
	try {
	JComponent header = getHeader();
	if (!(header instanceof JLabel))
		return;

	if (t == null || t.length() == 0)
		header.setVisible(false);
	else
		header.setVisible(true);
	((JLabel)header).setText(t);
	update();
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/**
 * Gets the value of the <code>HeaderFont</code> property, which holds the font
 * that is used to display the header.
 * @return value of the <code>HeaderFont</code> property
 */
public Font getHeaderFont() {
	JComponent header = getHeader();
	if (header != null)
		return header.getFont();
	return getFont();
}

/**
 * Sets the value of the <code>HeaderFont</code> property, which holds the font
 * that is used to display the header.
 * @param f
 */
public void setHeaderFont(Font f) {
	try {
	JComponent header = getHeader();
	if (header != null)
		header.setFont(f);

	update();
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/****************** Footer ******************/

/**
 * Gets the value of the <code>FooterText</code> property, which holds the text
 * that is displayed in the footer.
 * @return value of the <code>FooterText</code> property
 */
public String getFooterText() {
	JComponent footer = getFooter();
	String retStr = null;
	if (footer instanceof JLabel) {
		retStr = ((JLabel)footer).getText();
	}
	if (retStr == null)
		retStr = new String("");
	return retStr;
}

/**
 * Sets the value of the <code>FooterText</code> property, which holds the text
 * that is displayed in the footer. It also dictates the <code>Visible</code>
 * property.
 * @param t
 */
public void setFooterText(String t) {
	try {
	JComponent footer = getFooter();
	if (!(footer instanceof JLabel))
		return;
	if (t == null || t.length() == 0)
		footer.setVisible(false);
	else
		footer.setVisible(true);
	((JLabel)footer).setText(t);
	update();
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/**
 * Gets the value of the <code>FooterFont</code> property, which holds the font that
 * is used to display the footer.
 * @return value of the <code>FooterFont</code> property
 */
public Font getFooterFont() {
	JComponent footer = getFooter();
	if (footer != null)
		return footer.getFont();
	return getFont();
}

/**
 * Sets the value of the <code>FooterFont</code> property, which holds the font that
 * is used to display the footer.
 * @param f
 */
public void setFooterFont(Font f) {
	try {
	JComponent footer = getFooter();
	if (footer != null)
		footer.setFont(f);

	update();
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/****************** Legend ******************/

/**
 * Gets the value of the <code>LegendVisible</code> property; this property determines
 * whether the legend is visible.
 * @return value of the <code>LegendVisible</code> property
 */
public boolean getLegendVisible() {
	return getLegend().isVisible();
}

/**
 * Sets the value of the <code>LegendVisible</code> property; this property determines
 * whether the legend is visible.
 * @param is
 */
public void setLegendVisible(boolean is) {
	try {
	getLegend().setVisible(is);
	getLegend().setChanged(true, Changeable.RECALC);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/**
 * Gets the value of the <code>LegendAnchor</code> property, which determines the
 * position of the legend relative to the ChartArea. Valid values include
 * <code>NORTH</code>, <code>SOUTH</code>, <code>EAST</code>, <code>WEST</code>,
 * <code>NORTHWEST</code>, <code>SOUTHWEST</code>, <code>NORTHEAST</code> and
 * <code>SOUTHEAST</code>.
 * @return the value of the <code>LegendAnchor</code> property
 */
public int getLegendAnchor() {
	return getLegend().getAnchor();
}

/**
 * Sets the value of the <code>LegendAnchor</code> property, which determines the
 * position of the legend relative to the ChartArea. Valid values include
 * <code>NORTH</code>, <code>SOUTH</code>, <code>EAST</code>, <code>WEST</code>,
 * <code>NORTHWEST</code>, <code>SOUTHWEST</code>, <code>NORTHEAST</code> and
 * <code>SOUTHEAST</code>.
 * @param a
 */
public void setLegendAnchor(int a) {
	try {
	getLegend().setAnchor(a);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/**
 * Gets the value of the <code>LegendOrientation</code> property, which determines
 * how legend information is laid out. Valid values include
 * <code>VERTICAL</code> and <code>HORIZONTAL</code>.
 * @return value of the <code>LegendOrientation</code> property
 */
public int getLegendOrientation() {
	return getLegend().getOrientation();
}

/**
 * Sets the value of the <code>LegendOrientation</code> property, which determines
 * how legend information is laid out. Valid values include <code>VERTICAL</code>
 * and <code>HORIZONTAL</code>.
 * @param a
 */
public void setLegendOrientation(int a) {
	try {
	getLegend().setOrientation(a);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}
}

/**
 * Sets the data source to be derived from the
 * provided instance of the Swing <code>TableModel</code>.
 * @param model
 */
public void setSwingDataModel(TableModel model) {
    tableModel = model;
	ChartDataView dv = getDataView(0);
    dataSource = new JCChartSwingDataSource(model);
    dv.setDataSource((ChartDataModel)dataSource);
}

/**
 * Gets the Swing <code>TableModel</code> instance being used to populate
 * the data source.
 * @return Swing <code>TableModel</code> instance being used to populate
 * the data source
 */
public TableModel getSwingDataModel() {
    return tableModel;
}

}

