/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataSourceEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;
import java.beans.PropertyEditor;
import java.util.StringTokenizer;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.FontMetrics;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import javax.swing.event.CaretListener;
import javax.swing.event.CaretEvent;
import javax.swing.JLabel;
import javax.swing.JCheckBox;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JFrame;
import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.BorderFactory;




import java.util.Vector;


import com.klg.jclass.chart.data.JCDefaultDataSource;

/**
 * Bean editor for chart data source property.
 */

public class DataSourceEditor implements PropertyEditor {

protected PropertyChangeSupport support = new PropertyChangeSupport( this );
private boolean isFile = false;
protected String target = null; //the actual object that gets edited


public void setValue( Object value ) {
	if( value != null && !value.equals( "" ) ) {
		target = (String)value;
		isFile = target.indexOf( "ARRAY" ) < 0;
	}
	else {
		target = new String(new JCDefaultDataSource().buildDataSourceString());
	}
}

public Object getValue() {
	return target;
}

public boolean isPaintable() {
	return true;
}

public void paintValue( Graphics gc, Rectangle box ) {
    String s = JCChartBeanBundle.string(JCChartBeanBundle.key163);
    if (s == null) {
        s = new String("");
    }
	gc.setClip( box );
    FontMetrics fm = gc.getFontMetrics();
    int vpad = (box.height - fm.getAscent())/2;
    gc.drawString(s, box.x, box.y+box.height-vpad);
}

public String getAsText() {
	return target;
}

public void setAsText( String t ) {
    target = t;
    if( support != null )
        support.firePropertyChange( "", null, getValue() );
}

public String[] getTags() {
	return null;
}

public Component getCustomEditor() {
	final JPanel editor = new JPanel();
	final JPanel boxes = new JPanel();
	final JPanel fileLocation = new JPanel();
	final JScrollPane textScrollPane = new JScrollPane();
	final JTextArea t = new JTextArea( isFile ? new JCDefaultDataSource().buildDataSourceString() : target );
	final ButtonGroup cbg = new ButtonGroup();
	final JCheckBox fileBox = new JCheckBox( JCChartBeanBundle.string( JCChartBeanBundle.key171 ), isFile );
	final JCheckBox areaBox = new JCheckBox( JCChartBeanBundle.string( JCChartBeanBundle.key172 ), !isFile );
	final JTextField fileField = new JTextField( isFile ? target : "", 30 );
	
	cbg.add( fileBox );
	cbg.add( areaBox );
	editor.setLayout( new BorderLayout() );
	boxes.setLayout( new FlowLayout( FlowLayout.CENTER, 4, 4 ) );
	fileLocation.setLayout( new FlowLayout( FlowLayout.CENTER, 4, 4 ) );
	boxes.add( areaBox );
	boxes.add( fileBox );
	fileLocation.add( new JLabel( JCChartBeanBundle.string( JCChartBeanBundle.key173 ) + ":" ) );
	fileLocation.add( fileField );
	t.setEnabled( !isFile );
	fileField.setEnabled( isFile );
	t.setSize( 300, 150 );
	textScrollPane.setPreferredSize( new java.awt.Dimension( t.getPreferredScrollableViewportSize() ) );
    textScrollPane.getViewport().add( t );
	t.addCaretListener( new CaretListener() {
		public void caretUpdate( CaretEvent e ) {
			target = t.getText();
			if( support != null )
				support.firePropertyChange( "", null, getValue() );
		}
	});
	fileField.addCaretListener( new CaretListener() {
		public void caretUpdate( CaretEvent e ) {
			target = fileField.getText();
			if( support != null )
				support.firePropertyChange( "", null, getValue() );
		}
	});
	fileBox.addItemListener( new ItemListener() {
		public void itemStateChanged( ItemEvent e ) {
			fileField.setEnabled( true );
			t.setEnabled( false );
			isFile = true;
			target = fileField.getText();
			if( support != null )
				support.firePropertyChange( "", null, getValue() );
		}
	});
	areaBox.addItemListener( new ItemListener() {
		public void itemStateChanged( ItemEvent e ) {
			fileField.setEnabled( false );
			t.setEnabled( true );
			isFile = false;
			target = t.getText();
			if( support != null )
				support.firePropertyChange( "", null, getValue() );
		}
	});
	editor.add( boxes, BorderLayout.NORTH );
	editor.add( textScrollPane, BorderLayout.CENTER );
	editor.add( fileLocation, BorderLayout.SOUTH );
	return editor;
}

/* for testing purposes only
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }

	DataSourceEditor dataSourceEditor = new DataSourceEditor();

	dataSourceEditor.setValue( null );
    frame.getContentPane().add( dataSourceEditor.getCustomEditor() );
    frame.pack();
    frame.setVisible(true);
}
*/

public boolean supportsCustomEditor() {
	return true;
}

public String getJavaInitializationString() {
	StringBuffer statementString = new StringBuffer();

	if( !isFile ) {
		StringTokenizer tokenizer = new StringTokenizer( target, "\n ", true );
		boolean append = false;

		//stripping out the comments and the \n
		while( tokenizer.hasMoreElements() ) {
			String token = tokenizer.nextToken();

			if( token.compareTo("ARRAY") == 0 )
				append = true;
			if( append && token.compareTo("\n") != 0 )
				statementString.append( token );
		}
	}
	else {
		StringTokenizer tokenizer = new StringTokenizer( target, "\\", true );
		while( tokenizer.hasMoreElements() ) {
			String token = tokenizer.nextToken();

			if( token.compareTo( "\\" ) == 0 )
				statementString.append( token );
			statementString.append( token );
		}
	}
	return "\"" + statementString + "\"";
}

public void addPropertyChangeListener(PropertyChangeListener l) {
	support.addPropertyChangeListener(l);
}

public void removePropertyChangeListener(PropertyChangeListener l) {
	support.removePropertyChangeListener(l);
}

}
