/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: LegendEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.util.legend.JCLegend;
import com.klg.jclass.util.legend.JCLegendEnumMappings;

import java.awt.Insets;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Dimension;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import javax.swing.JPanel;
import javax.swing.BorderFactory;




import java.util.Vector;


/**
 * Editor for Legend Properties.
 */

public class LegendEditor extends BaseEditor
	implements ItemListener {

protected JComboBox orientation_ch;
protected JComboBox anchor_ch;

protected String[] orientation_names;
protected int[] orientation_vals;
protected String[] anchor_names;
protected int[] anchor_vals;

public LegendEditor() {
	super();
}

protected void setEnumArrays()
{
	orientation_names = JCLegendEnumMappings.orientation_i18n_strings;
	orientation_vals = JCLegendEnumMappings.orientation_values;
	anchor_names = JCLegendEnumMappings.anchor_i18n_strings;
	anchor_vals = JCLegendEnumMappings.anchor_values;
}

/**
 * Creates a panel containing the GUI elements for the properties
 * associated with the radio buttons.
 * @return the panel
 */
protected JPanel makePropertyPanel() 
{
    JPanel p2 = new JPanel();
    p2.setBorder(BorderFactory.createTitledBorder(
    	JCChartBeanBundle.string(JCChartBeanBundle.key254)));
	p2.setLayout(new GridBagLayout());

    GridBagConstraints gbc2  = new GridBagConstraints();
    gbc2.gridwidth = 1;
    gbc2.gridx = GridBagConstraints.RELATIVE;
    gbc2.gridy = 0;
    gbc2.anchor = GridBagConstraints.NORTH;
    gbc2.insets = new Insets(4,4,4,4);

	orientation_ch = new JComboBox();
	anchor_ch = new JComboBox();

	setEnumArrays();
	for (int i = 0; i < orientation_names.length; i++)
		orientation_ch.addItem(orientation_names[i]);
	for (int i = 0; i < anchor_names.length; i++)
		anchor_ch.addItem(anchor_names[i]);

	JLabel orientation_label = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key255));
	JLabel anchor_label = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key256));
	p2.add(orientation_label, gbc2);
	p2.add(orientation_ch, gbc2);
	gbc2.gridy++;
	p2.add(anchor_label, gbc2);
	p2.add(anchor_ch, gbc2);

	orientation_ch.addItemListener(this);
	anchor_ch.addItemListener(this);

	add("Center", p2);

	return p2;
}

LegendWrapper wrap = null;

public void setValue(Object value) {

    if (value != null) {
        try {
			wrap = (LegendWrapper) value;
        } catch (ClassCastException e) {
        }
    }

	if (wrap == null)
		wrap = new LegendWrapper();

	switchValues();
}

/**
 * Fills the property sheet with values associated with the newly selected
 * radio button.
 */
protected void switchValues()
{
	if (wrap == null)
		return;

	for (int i = 0; i < orientation_vals.length; i++) {
		if (wrap.orientation == orientation_vals[i]) {
			if (orientation_ch.getSelectedIndex() != i)
				orientation_ch.setSelectedIndex(i);
		}
	}
	for (int i = 0; i < anchor_vals.length; i++) {
		if (wrap.anchor == anchor_vals[i]) {
			if (anchor_ch.getSelectedIndex() != i)
				anchor_ch.setSelectedIndex(i);
		}
	}
}

public Object getValue() {
	return (Object) wrap;
}

public void setAsText(String text) throws IllegalArgumentException {
    Vector paramVector = BeanTextParser.parse(text);
    if (paramVector.size() != 2) {
        throw new IllegalArgumentException();
    }
    LegendWrapper setWrapper = null;
    try {
        setWrapper = new LegendWrapper(
            (String) paramVector.elementAt(0),
            (String) paramVector.elementAt(1)
            );
        setValue(setWrapper);
        support.firePropertyChange("", null, null);
    } catch (Exception e) {
        throw new IllegalArgumentException();
    }
}

public String getAsText() {
    if (wrap != null)
        return wrap.toString();
    else
        return null;
}


public String getJavaInitializationString() {
	return "new com.klg.jclass.chart.beans.LegendWrapper(" + wrap + ")";
}

public Dimension getPreferredSize() {
	Dimension dim = super.getPreferredSize();
	dim.width = Math.max(dim.width, 300);
	dim.height += 200;
	return dim;
}

public void itemStateChanged( ItemEvent ev ) 
{
	Object src = ev.getSource();
	if (src instanceof JComboBox &&
            ev.getStateChange() == ItemEvent.DESELECTED)
        return;
	checkProperties(src);
}

public void checkProperties( Object src)
{
	try {
		if (src == orientation_ch) {
			wrap.orientation = 
				orientation_vals[orientation_ch.getSelectedIndex()];
			if( support != null )
				support.firePropertyChange("", null, null);
		} else if (src == anchor_ch) {
			wrap.anchor = 
				anchor_vals[anchor_ch.getSelectedIndex()];
			if( support != null )
				support.firePropertyChange("", null, null);
		}
	} catch (Exception ex) {
	}
}

/* for testing purposes
public static void main( String args[] ) {
    JFrame frame;
    try {
    java.lang.Class newclass
        = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
    frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
        System.out.println(ex);
    frame = new JFrame();
    }

    LegendEditor axisGenEditor = new LegendEditor();
    axisGenEditor.setBackground( java.awt.Color.lightGray );
    //axisGenEditor.setValue( null);
	LegendWrapper thiswrap = new LegendWrapper("", "");
    axisGenEditor.setValue( thiswrap );
    frame.getContentPane().add( axisGenEditor );
    frame.pack();
    java.awt.Dimension bigness = axisGenEditor.getPreferredSize();
    frame.setSize( bigness.width + 100, bigness.height + 50 );
    frame.setVisible(true);
}
*/
}
