/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: LegendWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.util.legend.JCLegendEnumMappings;
import com.klg.jclass.util.JCTypeConverter;

import com.klg.jclass.util.legend.JCLegend;

/**
 * Wrapper class for Legend properties.
 * Used by the MultiChart Bean.
 */
public class LegendWrapper 
implements java.io.Serializable
{

public int  orientation;
public int  anchor;

/**
 * Default constructor.  Members will be empty.
 */
public LegendWrapper() {
}

/**
 * Constructor.  Fills members with parameter Strings.
 * @param orientationString     orientation String.  Possible values are
 *                       are <code>Horizontal</code> or <code>Vertical</code>.
 * @param anchorString          anchor String.  Possible values are
 * <code>Northeast</code>, <code>North</code>, <code>Northwest</code>, <code>East</code>,
 * <code>West</code>, <code>Southeast</code>, <code>South</code>, or <code>Southwest</code>.
 */
public LegendWrapper(String orientationString, String anchorString)
{
    orientation = JCTypeConverter.toEnum(orientationString, "LegendOrientation",
        JCLegendEnumMappings.orientation_strings, 
		JCLegendEnumMappings.orientation_values,
        JCLegend.VERTICAL);
    anchor = JCTypeConverter.toEnum(anchorString, "LegendAnchor",
        JCLegendEnumMappings.anchor_strings, JCLegendEnumMappings.anchor_values,
        JCLegend.EAST);
}

/**
 * Stores the properties from the specified legend.
 * @param legend the legend that contains the properties
 */
public void setWrapperValues(JCLegend legend)
{
	orientation = legend.getOrientation();
	anchor = legend.getAnchor();
}

/**
 * Sets the properties on the specified legend from values stored internally.
 * @param legend the legend that we need to set these values on
 */
public void setPropertyValues(JCLegend legend)
{
	legend.setOrientation(orientation);
	legend.setAnchor(anchor);
}

/**
 * Converts properties to String for code generation.
 * @return String containing properties for code generation
 */
public String toString()
{
    String s = "";

    s += "\"" + JCTypeConverter.fromEnum(orientation,
        JCLegendEnumMappings.orientation_strings, 
		JCLegendEnumMappings.orientation_values) + "\"";
    s += ",";
    s += "\"" + JCTypeConverter.fromEnum(anchor,
        JCLegendEnumMappings.anchor_strings, 
		JCLegendEnumMappings.anchor_values) + "\"";

    return s;
}

public boolean equals(Object obj)
{
	boolean eq = true;
	if (obj instanceof LegendWrapper) {
		LegendWrapper compareAW = (LegendWrapper) obj;
		if (compareAW.orientation != this.orientation)
			eq = false;
		if (compareAW.anchor != this.anchor)
			eq = false;
	} else {
		eq = false;
	}
	return eq;
}
}
