/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: MultiChart.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import java.awt.event.InputEvent;
import java.awt.Color;
import java.util.StringTokenizer;
import javax.swing.table.TableModel;
import javax.swing.JComponent;
import com.klg.jclass.util.JCStringTokenizer;
import com.klg.jclass.util.swing.JCSwingTypeConverter;

import java.util.List;




import com.klg.jclass.chart.Changeable;
import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.JCChartArea;
import com.klg.jclass.chart.ChartDataView;
import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.ChartDataSupport;
import com.klg.jclass.chart.EventTrigger;
import com.klg.jclass.chart.data.JCDefaultDataSource;
import com.klg.jclass.chart.data.JCFileDataSource;
import com.klg.jclass.chart.data.JCURLDataSource;
import com.klg.jclass.chart.data.JCStringDataSource;
import com.klg.jclass.chart.data.JCChartSwingDataSource;
import com.klg.jclass.chart.ChartDataModel;
import com.klg.jclass.chart.PlotArea;
import com.klg.jclass.chart.JCPolarRadarChartFormat;

/**
 * A derived class that enables JClass Chart to
 * be used at design-time with Bean-compliant Java IDEs (Integrated
 * Development Environments).
 * <P>This class differs from <code>JCChart</code> in the following ways:
 * <OL>
 * <LI><code>MultiChart</code> exposes properties that are actually defined
 * in <code>JCChart</code>'s sub-objects so that IDEs can access them.
 * <LI><code>MultiChart</code> manages two data views, two <i>x</i> axes and two <i>y</i>
 * axes. Properties on these items can be set for all instances of the item.
 * </OL>
 * <P>The following table lists the properties this class can set/get.
 * <P><STRONG>Properties</STRONG></P>
 * <P></P>
 * <TABLE BORDER=1 WIDTH=100%>
 * <TR>
 * <TH>Name</TH>
 * <TH>Description</TH></TR>
 * <TR>
 * <TD>AxisAnnotations
 * </TD>
 * <TD>
 * The <code>AxisAnnotations</code> property accesses the <code>AnnotationMethod</code>,
 * <code>AnnotationRotation</code>, <code>font</code>, and <code>gap</code> properties of an axis.
 * </TD></TR>
 * <TR><TD>AxisGrid
 * </TD>
 * <TD>
 * The <code>AxisGrid</code> property accesses the <code>GridVisible</code> and <code>GridSpacing</code>
 * properties of the axis, and the <code>LinePattern</code>, <code>LineColor</code>, and
 * <code>LineWidth</code> properties of the axis' <code>GridStyle</code> property.
 * </TD></TR>
 * <TR><TD>AxisAnnotations
 * </TD>
 * <TD>
 * The <code>AxisAnnotations</code> property accesses the <code>AnnotationMethod</code>,
 * <code>AnnotationRotation</code>, <code>font</code>, and <code>gap</code> properties of an axis.
 * </TD></TR>
 * <TR><TD>AxisMisc
 * </TD>
 * <TD>
 * The <code>AxisMisc</code> property accesses the <code>Visible</code>, <code>Editable</code>, and
 * <code>Logarithmic</code> properties of an axis.
 * </TD></TR>
 * <TR><TD>AxisOrientation
 * </TD>
 * <TD>
 * The <code>AxisOrientation</code> property accesses the <code>Reversed</code> property of
 * an axis and the <code>Inverted</code> property of the related data view.
 * </TD></TR>
 * <TR><TD>AxisOrigin
 * </TD>
 * <TD>
 * The <code>AxisOrigin</code> property accesses the <code>Origin</code> and <code>OriginPlacement</code>
 * properties of an axis.
 * </TD></TR>
 * <TR><TD>AxisPlacement
 * </TD>
 * <TD>
 * The <code>AxisPlacement</code> property accesses the <code>Placement</code>, <code>PlacementAxis</code>,
 * and <code>PlacementLocation</code> properties of an axis.
 * </TD></TR>
 * <TR><TD>AxisPointLabels
 * </TD>
 * <TD>
 * The <code>AxisPointLabels</code> property accesses the <code>PointLabels</code> property of
 * the data view related to an axis.
 * </TD></TR>
 * <TR><TD>AxisRelationship
 * </TD>
 * <TD>
 * The <code>AxisRelationship</code> property accesses the <code>Originator</code>, <code>Constant</code>,
 * and <code>Multiplier</code> properties of an axis' <code>Formula</code> property.
 * </TD></TR>
 * <TR><TD>AxisScale
 * </TD>
 * <TD>
 * The <code>AxisScale</code> property accesses the <code>Min</code>, <code>Max</code>,
 * <code>NumSpacing</code>, <code>TickSpacing</code>, and <code>Precision</code> properties of an axis.
 * </TD></TR>
 * <TR><TD>AxisTimeLabels
 * </TD>
 * <TD>
 * The <code>AxisTimeLabels</code> property accesses the <code>TimeBase</code>, <code>TimeFormat</code>,
 * and <code>TimeUnit</code> properties of an axis.
 * </TD></TR>
 * <TR><TD>AxisTitle
 * </TD>
 * <TD>
 * The <code>AxisTitle</code> property accesses the <code>Text</code>, <code>Rotation</code>,
 * <code>Placement</code>, and <code>Font</code> properties of an axis' <code>Title</code> property.
 * </TD></TR>
 * <TR><TD>AxisValueLabels
 * </TD>
 * <TD>
 * The <code>AxisValueLabels</code> property accesses the <code>ValueLabels</code>
 * property of an axis.
 * </TD></TR>
 * <TR><TD>PolarRadar
 * </TD>
 * <TD>
 * The <code>PolarRadar</code> property accesses the <code>OriginBase</code>, <code>YAxisAngle</code>,
 * <code>AngleUnit</code>, <code>HalfRange</code>, and <code>RadarCircularGrid</code> properties of a
 * Polar/Radar chart.
 * </TD></TR>
 * <TR><TD>ChartAppearance
 * </TD>
 * <TD>
 * The <code>ChartAppearance</code> property accesses the <code>Background</code>, <code>Foreground</code>,
 * <code>BorderType</code>, and <code>BorderWidth</code> properties of the <code>Chart</code> object.
 * </TD></TR>
 * <TR><TD>ChartAreaAppearance
 * </TD>
 * <TD>
 * The <code>ChartAreaAppearance</code> property accesses the <code>Background</code>, <code>Foreground</code>,
 * <code>Visible</code>, and <code>Opaque</code> properties of the <code>ChartArea</code> object.
 * </TD></TR>
 * <TR><TD>DataChart
 * </TD>
 * <TD>
 * The <code>DataChart</code> property accesses the <code>ChartType</code>, <code>XAxis</code>,
 * and <code>YAxis</code> properties of a data view.
 * </TD></TR>
 * <TR><TD>DataMisc
 * </TD>
 * <TD>
 * The <code>DataChart</code> property accesses the <code>Visible</code>, <code>VisibleInLegend</code>,
 * <code>DrawFrontPlane</code>, and <code>AutoLabel</code> properties of a data view.
 * </TD></TR>
 * <TR><TD>DataSource
 * </TD>
 * <TD>
 * The <code>DataSource</code> property accesses the <code>DataSource</code> property of a
 * data view.
 * </TD></TR>
 * <TR><TD>FooterAppearance
 * </TD>
 * <TD>
 * The <code>FooterAppearance</code> property accesses the <code>Background</code>, <code>Foreground</code>,
 * <code>Visible</code>, and <code>Opaque</code> properties of the <code>Footer</code> object.
 * </TD></TR>
 * <TR><TD>FooterText
 * </TD>
 * <TD>
 * The <code>FooterText</code> property accesses the <code>Font</code> and <code>Text</code>
 * properties of the chart's footer (by default, a <code>JLabel</code> instance).
 * </TD></TR>
 * <TR><TD>HeaderAppearance
 * </TD>
 * <TD>
 * The <code>HeaderAppearance</code> property accesses the <code>Background</code>, <code>Foreground</code>,
 * <code>Visible</code>, and <code>Opaque</code> properties of the <code>Header</code> object.
 * </TD></TR>
 * <TR><TD>HeaderText
 * </TD>
 * <TD>
 * The <code>HeaderText</code> property accesses the <code>Font</code> and <code>Text</code>
 * properties of the chart's header (by default, a <code>JLabel</code> instance).
 * </TD></TR>
 * <TR><TD>LegendAppearance
 * </TD>
 * <TD>
 * The <CODE>LegendAppearance</code> property accesses the <code>Background</code>, <code>Foreground</code>,
 * <code>Visible</code>, and <code>Opaque</code> properties of the <code>Legend</code> object.
 * </TD></TR>
 * <TR><TD>LegendLayout
 * </TD>
 * <TD>
 * The <code>LegendLayout</code> property accesses the <code>Orientation</code> and <code>Anchor</code>
 * properties of the <code>Legend</code> object.
 * </TD></TR>
 * <TR><TD>PlotAreaAppearance
 * </TD>
 * <TD>
 * The <code>PlotAreaAppearance</code> property accesses the <code>Background</code>, <code>Foreground</code>,
 * and <code>AxisBoundingBox</code> properties of the <code>PlotArea</code> object.
 * </TD></TR>
 * <TR><TD>SwingDataModel1<BR>SwingDataModel2
 * </TD><TD>
 * <code>SwingDataModel1</code> sets the data source for the first data view to use a specified
 * Swing <code>TableModel</code> object, instead of the <TT>DataSource</TT> property.
 * The <code>SwingDataModel2</code> property does the same for the second data view.
 * </TD></TR>
 * <TR><TD>TriggerList
 * </TD>
 * <TD>
 * The <code>TriggerList</code> property accesses the <code>Trigger</code> property of the
 * <code>Chart</code> object.
 * </TD></TR>
 * </TABLE>
 */
public class MultiChart extends JCChart {

/**
 * List of axis names for use in multi-axis property editors.
 * Must match logically with <code>AXIS_RADIO_IDS</code> array below.
 */
public final static String AXIS_RADIO_NAMES[] = {
    JCChartBeanBundle.string(JCChartBeanBundle.key259) + " 1",
    JCChartBeanBundle.string(JCChartBeanBundle.key260) + " 1",
    JCChartBeanBundle.string(JCChartBeanBundle.key259) + " 2",
    JCChartBeanBundle.string(JCChartBeanBundle.key260) + " 2"
    };

/**
 * List of axis IDs for use in multi-axis property editors.
 * Must match logically with <code>AXIS_RADIO_NAMES</code> array above.
 */
public final static String AXIS_RADIO_IDS[] = {
    "x1", "y1", "x2", "y2"
    };

/** Title of multi-axis property editors. */
public final static String AXIS_RADIO_TITLE =
    JCChartBeanBundle.string(JCChartBeanBundle.key288);

/**
 * List of data view names for use in multi-data property editors.
 * Must match logically with <code>DATA_RADIO_IDS</code> array below.
 */
public final static String DATA_RADIO_NAMES[] = {
    JCChartBeanBundle.string(JCChartBeanBundle.key287) + " 1",
    JCChartBeanBundle.string(JCChartBeanBundle.key287) + " 2"
    };

/**
 * List of data view IDs for use in multi-data property editors.
 * Must match logically with <code>AXIS_RADIO_NAMES</code> array below.
 */
public final static String DATA_RADIO_IDS[] = {
    "data1", "data2"
    };

/** Title of multi-data property editors. */
public final static String DATA_RADIO_TITLE =
    JCChartBeanBundle.string(JCChartBeanBundle.key289);

/** Enums for axis orientation values. */
public final static int ORIENT_X_NORMAL_Y_NORMAL = 0;
public final static int ORIENT_X_REVERSED_Y_NORMAL = 1;
public final static int ORIENT_X_NORMAL_Y_REVERSED = 2;
public final static int ORIENT_X_REVERSED_Y_REVERSED = 3;
public final static int ORIENT_INVERTED_X_NORMAL_Y_NORMAL = 4;
public final static int ORIENT_INVERTED_X_REVERSED_Y_NORMAL = 5;
public final static int ORIENT_INVERTED_X_NORMAL_Y_REVERSED = 6;
public final static int ORIENT_INVERTED_X_REVERSED_Y_REVERSED = 7;

/** Array of axis orientation values. */
public final static int[] axesOrientationValues = {
			ORIENT_X_NORMAL_Y_NORMAL,
			ORIENT_INVERTED_X_NORMAL_Y_NORMAL,
			ORIENT_X_REVERSED_Y_NORMAL,
			ORIENT_INVERTED_X_REVERSED_Y_NORMAL,
			ORIENT_X_NORMAL_Y_REVERSED,
			ORIENT_INVERTED_X_NORMAL_Y_REVERSED,
			ORIENT_X_REVERSED_Y_REVERSED,
			ORIENT_INVERTED_X_REVERSED_Y_REVERSED };

/** Array of String representations of axis orientation values. */
public final static String[] axesOrientationStrings = {
			"ORIENT_X_NORMAL_Y_NORMAL",
			"ORIENT_INVERTED_X_NORMAL_Y_NORMAL",
			"ORIENT_X_REVERSED_Y_NORMAL",
			"ORIENT_INVERTED_X_REVERSED_Y_NORMAL",
			"ORIENT_X_NORMAL_Y_REVERSED",
			"ORIENT_INVERTED_X_NORMAL_Y_REVERSED",
			"ORIENT_X_REVERSED_Y_REVERSED",
			"ORIENT_INVERTED_X_REVERSED_Y_REVERSED" };

/** Enums for possible objects for <code>Appearance</code> properties. */
public final static int  OBJ_UNKNOWN = -1;
public final static int  OBJ_JCOMPONENT = 0;
public final static int  OBJ_PLOT_AREA = 1;
public final static int  OBJ_CHART = 2;

protected JCAxis[] axisArray;					// array of axes in this chart
protected ChartDataView[] dataViewArray;		// array of data views in this chart

protected String data[];
protected String dataOther[];

/** <code>TableModel</code> used to populate first data view, if one is being used. */
public TableModel tableModel = null;

/** <code>TableModel</code> used to populate second data view, if one is being used. */
public TableModel tableModel2 = null;

/** <code>JCSwingDataSource</code> used to populate first data view, if one is being used. */
public ChartDataSupport dataSource = null;

/** <code>JCSwingDataSource</code> used to populate second data view, if one is being used. */
public ChartDataSupport dataSource2 = null;

/**
 * Basic constructor.  Use default data.
 */
public MultiChart() {
	super();

	dataOther = new String[2];
	dataOther[0] = "";
	dataOther[1] = "";

	data = new String[2];
	JCDefaultDataSource defDS1 = new JCDefaultDataSource();
	JCDefaultDataSource defDS2 = new JCDefaultDataSource();
	defDS1.setName("Data View 1");
	defDS2.setName("Data View 2");
	setData(defDS1.buildDataSourceString(), 0 );
	setData(defDS2.buildDataSourceString(), 1 );

	JCAxis second_x = new JCAxis();
	second_x.setVisible(false);
	second_x.setEditable(false);
	JCAxis second_y = new JCAxis();
	second_y.setVertical(true);
	second_y.setVisible(false);
	second_y.setEditable(false);

	getChartArea().setXAxis(1, second_x);
	getChartArea().setYAxis(1, second_y);

	axisArray = new JCAxis[4];
	axisArray[0] = getChartArea().getXAxis(0);
	axisArray[1] = getChartArea().getYAxis(0);
	axisArray[2] = second_x;
	axisArray[3] = second_y;

	setTrigger(0, new EventTrigger(0, -1));
	setTrigger(1, new EventTrigger(InputEvent.SHIFT_MASK, -1));
	setTrigger(2, new EventTrigger(InputEvent.CTRL_MASK, -1));
	setTrigger(3, new EventTrigger(InputEvent.ALT_MASK, -1));
	setTrigger(4, new EventTrigger(InputEvent.META_MASK, -1));

    List l = getDataView();
    dataViewArray = new ChartDataView[l.size()];
	for (int i = 0; i < l.size(); i++) {
		dataViewArray[i] = (ChartDataView) l.get(i);
	}
	dataViewArray[1].setXAxis(axisArray[0]);
	dataViewArray[1].setYAxis(axisArray[1]);
	dataViewArray[0].setChartType(JCChart.BAR);
	axisArray[0].setAnnotationMethod(JCAxis.VALUE);

}

/***** Data Properties *****/

/**
 * Gets the value of the data property.
 * The data property controls the file or URL used for the
 * sets of data in the chart.
 * @return value of the data property
 */
public MultiDataSourceWrapper getDataSource() {
	MultiDataSourceWrapper dsw = new MultiDataSourceWrapper();

	dsw.setWrapperValues(dataViewArray);

	dsw.data[0] = getData(0);
	dsw.data[1] = getData(1);
	dsw.data_other[0] = dataOther[0];
	dsw.data_other[1] = dataOther[1];

	return dsw;
}

/**
 * Sets the value of the data property.
 * The data property controls the file or URL used for the
 * sets of data in the chart.
 * @param dsw
 */
public void setDataSource(MultiDataSourceWrapper dsw) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	//dsw.setPropertyValues(dataViewArray);
	for (int i = 0; i < dsw.data.length; i++) {
		setData(dsw.data[i], i);
	}
	dataOther[0] = dsw.data_other[0];
	dataOther[1] = dsw.data_other[1];

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}
}

/**
 *  Get the specified data view as a String (which is cached
 *  for this purpose).
 *  @param ind index of the data view
 *  @return String representing the requested data view -- either a
 *                 file name or a String specification of data
 */
String getData(int ind)
{
	if (data[ind] == null)
		data[ind] = new String("");
	return data[ind];
}

/**
 *  Sets the specified data view from a String.
 *  @param s String containing data view specification -- either a file
 *           name or a String representation of data
 *  @param ind index of data view to set
 */
void setData(String s, int ind) {
	if (getDataView(ind) == null) {
		setDataView(ind, new ChartDataView());
	}
	if (s == null || s.equals("")) {
		getDataView(ind).setDataSource(null);
	}
	else if (s.indexOf("http") != -1) {
		try {
			getDataView(ind).setDataSource(new JCURLDataSource(s));
		}
		catch (Exception e) {
			getDataView(ind).setDataSource(null);
		}
	}
	else {
        String testString = s.toLowerCase();

        // check if definitely trying to be a string data source
        int pos = -1;
        if ((pos = testString.indexOf("array")) >= 0) {
            if (testString.length() == pos + 5 ||
                !Character.isWhitespace(testString.charAt(pos + 5))) {
                pos = -1;
            }
        } else if ((pos = testString.indexOf("general")) >= 0) {
            if (testString.length() == pos + 7 ||
                !Character.isWhitespace(testString.charAt(pos + 7))) {
                pos = -1;
            }
        }
        // has "array" or "general" follwed by whitespace,
        // must be a string source
        if (pos != -1) {
            try {
                getDataView(ind).setDataSource(new JCStringDataSource(s));
            }
            catch( Exception e2 ) {
                getDataView(ind).setDataSource(null);
            }
        // not of right format, then must be a file name
        } else {
            try {
                getDataView(ind).setDataSource(new JCFileDataSource(s));
            }
            catch( Exception e1 ) {
                getDataView(ind).setDataSource(null);
            }
        }
	}
	data[ind] = s;
}

/**
 *  Gets the values of the properties encapsulated in the <code>DataChart</code>
 *  property.
 * @return values of the properties encapsulated in the <code>DataChart</code>
 *  property
 */
public MultiDataChartWrapper getDataChart() {
	MultiDataChartWrapper mdcw = new MultiDataChartWrapper();
	mdcw.setWrapperValues(dataViewArray);
	for (int i = 0; i < dataViewArray.length; i++) {
		mdcw.xaxis[i] = findAxisIndex(dataViewArray[i].getXAxis());
		mdcw.yaxis[i] = findAxisIndex(dataViewArray[i].getYAxis());
	}

	return mdcw;
}

/**
 *  Sets the values of the properties encapsulated in the <code>DataChart</code>
 *  property.
 * @param mdcw
 */
public void setDataChart(MultiDataChartWrapper mdcw) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	mdcw.setPropertyValues(dataViewArray);
	for (int i = 0; i < dataViewArray.length; i++) {
    	if (mdcw.xaxis[i] == -1) {
        	dataViewArray[i].setXAxis(null);
		} else {
        	dataViewArray[i].setXAxis(axisArray[mdcw.xaxis[i]]);
    	}
    	if (mdcw.yaxis[i] == -1) {
        	dataViewArray[i].setYAxis(null);
		} else {
        	dataViewArray[i].setYAxis(axisArray[mdcw.yaxis[i]]);
    	}
	}

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}

}

/**
 *  Gets the values of the properties encapsulated in the <code>DataMisc</code>
 *  property.
 * @return values of the properties encapsulated in the <code>DataMisc</code>
 *  property
 */
public MultiDataMiscWrapper getDataMisc() {
	MultiDataMiscWrapper mdgw = new MultiDataMiscWrapper();
	mdgw.setWrapperValues(dataViewArray);

	return mdgw;
}

/**
 *  Sets the values of the properties encapsulated in the <code>DataMisc</code>
 *  property.
 * @param mdgw
 */
public void setDataMisc(MultiDataMiscWrapper mdgw) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	mdgw.setPropertyValues(dataViewArray);

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}

}

/************** General JCChart properties *************/

/**
 * Gets the value of the <TT>View3D</TT> property.  This property combines the
 * values of the <TT>Depth</TT>, <TT>Elevation</TT>, and <TT>Rotation</TT> properties
 * defined in <TT>JCChartArea</TT>.
 * <TT>Depth</TT> controls the apparent depth of a graph.
 * <TT>Elevation</TT> controls the distance above the <i>x</i> axis
 * for the 3D effect. <TT>Rotation</TT> controls the position of the
 * eye relative to the <i>y</i> axis for the 3D effect.
 * @return    value of <TT>Depth</TT>, <TT>Elevation</TT>, and <TT>Rotation</TT>
 * properties respectively, separated by commas
 * @see       JCChartArea#getDepth
 * @see       JCChartArea#getElevation
 * @see       JCChartArea#getRotation
 */
public String getView3D() {
	JCChartArea ca = getChartArea();
	return ""+ca.getDepth()+","+ca.getElevation()+","+ca.getRotation();
}

/**
 * Sets the value of the <TT>View3D</TT> property.  This property controls the
 * apparent depth of a graph by combining the values of the <TT>Depth</TT>, <TT>Elevation</TT>,
 * and <TT>Rotation</TT> properties defined in <TT>JCChartArea</TT>.
 * <TT>Depth</TT> controls the apparent depth of a graph.
 * <TT>Elevation</TT> controls the distance above the <i>x</i> axis
 * for the 3D effect. <TT>Rotation</TT> controls the position of the
 * eye relative to the <i>y</i> axis for the 3D effect.
 * @param     s value of <TT>Depth</TT>, <TT>Elevation</TT>, and <TT>Rotation</TT>
 * properties respectively, separated by commas
 * @see       JCChartArea#setDepth
 * @see       JCChartArea#setElevation
 * @see       JCChartArea#setRotation
 */
public void setView3D(String s) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	JCChartArea ca = getChartArea();
	StringTokenizer st = new StringTokenizer(s, ",");
	for (int j = 0; j < 3; j++) {
		int i = 0;
		if (st.hasMoreTokens()) i = Integer.parseInt(st.nextToken());
		switch (j) {
		case 0:
			ca.setDepth(i); break;
		case 1:
			ca.setElevation(i); break;
		case 2:
			ca.setRotation(i); break;
		}
	}

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}

}

/**
 * Gets the value of the <TT>OutlineStyle</TT> property.  This property combines
 * the values of <TT>Width</TT>, <TT>Color</TT>, <TT>Pattern</TT>, <TT>Join</TT>,
 * and <TT>Cap</TT> defined in the <TT>JCLineStyle</TT> class.
 * @return value of the <TT>OutlineStyle</TT> property
 */
public OutlineStyleWrapper getOutlineStyle() {
	ChartDataView cdv = getDataView(0);
	OutlineStyleWrapper osw = new OutlineStyleWrapper();
	osw.setWrapperValues(cdv);
	return osw;
}

/**
 * Sets the value of the <TT>OutlineStyle</TT> property.  This property combines
 * the values of <TT>Width</TT>, <TT>Color</TT>, <TT>Pattern</TT>, <TT>Join</TT>,
 * and <TT>Cap</TT> defined in the <TT>JCLineStyle</TT> class.
 * @param osw
 */
public void setOutlineStyle(OutlineStyleWrapper osw) {
	boolean oldBatched = false;
	try {
	oldBatched = isBatched();
	setBatched(true);

	ChartDataView cdv = getDataView(0);
	osw.setPropertyValues(cdv);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}
	finally {
		setBatched(oldBatched);
	}
}

/************** Polar/Radar chart properties ****************/

/**
 * Gets the value of the <TT>PolarRadar</TT> property.  This property combines
 * the values of <TT>OriginBase</TT>, <TT>YAxisAngle</TT>, <TT>AngleUnit</TT>,
 * <TT>HalfRange</TT>, and <TT>CircularRadarGrid</TT>.
 * @return value of the <TT>PolarRadar</TT> property
 */
public PolarRadarWrapper getPolarRadar() {
	JCChartArea ca = getChartArea();
	ChartDataView cdv = getDataView(0);
	JCPolarRadarChartFormat cf =
			(JCPolarRadarChartFormat)cdv.getChartFormat(JCChart.POLAR);
			// Specifying POLAR, RADAR, or AREA_RADAR will return the
			// same JCPolarRadarChartFormat object.
	PolarRadarWrapper prw = new PolarRadarWrapper();
	prw.setWrapperValues(cf, ca);
	return prw;
}

/**
 * Sets the value of the <TT>PolarRadar</TT> property.  This property combines
 * the values of <TT>OriginBase</TT>, <TT>YAxisAngle</TT>, <TT>AngleUnit</TT>,
 * <TT>HalfRange</TT>, and <TT>CircularRadarGrid</TT>.
 * @param prw
 */
public void setPolarRadar(PolarRadarWrapper prw) {
	boolean oldBatched = false;
	try {
	oldBatched = isBatched();
	setBatched(true);

	ChartDataView cdv = getDataView(0);
	JCPolarRadarChartFormat cf =
			(JCPolarRadarChartFormat)cdv.getChartFormat(JCChart.POLAR);
			// Specifying POLAR, RADAR, or AREA_RADAR will return the
			// same JCPolarRadarChartFormat object.
	JCChartArea ca = getChartArea();
	prw.setPropertyValues(cf, ca);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}
	finally {
		setBatched(oldBatched);
	}

}



/************** JCAxis properties *************/

/**
 *  Gets the values of the properties encapsulated in the <code>AxisMisc</code>
 *  property.
 * @return values of the properties encapsulated in the <code>AxisMisc</code>
 *  property
 */
public AxisMiscWrapper getAxisMisc() {
	AxisMiscWrapper agw = new AxisMiscWrapper();
	agw.setWrapperValues(axisArray);

	return agw;
}

/**
 *  Sets the values of the properties encapsulated in the <code>AxisMisc</code>
 *  property.
 * @param agw
 */
public void setAxisMisc(AxisMiscWrapper agw) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	agw.setPropertyValues(axisArray);

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}

}

/**
 *  Gets the values of the properties encapsulated in the <code>AxisScale</code>
 *  property.
 * @return values of the properties encapsulated in the <code>AxisScale</code>
 *  property
 */
public AxisScaleWrapper getAxisScale() {
	AxisScaleWrapper asw = new AxisScaleWrapper();
	asw.setWrapperValues(axisArray);

	return asw;
}

/**
 *  Sets the values of the properties encapsulated in the <code>AxisScale</code>
 *  property.
 * @param asw
 */
public void setAxisScale(AxisScaleWrapper asw) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	asw.setPropertyValues(axisArray);

	setBatched(oldBatched);

	}
	catch ( Exception e ) {
		e.printStackTrace();
	}

	// refresh wrapper values -- values change when isdef is set
	asw.setWrapperValues(axisArray);

}

/**
 *  Gets the values of the properties encapsulated in the <code>AxisAnnotations</code>
 *  property.
 * @return values of the properties encapsulated in the <code>AxisAnnotations</code>
 *  property
 */
public AxisAnnoWrapper getAxisAnnotations() {
	AxisAnnoWrapper aaw = new AxisAnnoWrapper();
	aaw.setWrapperValues(axisArray);

	return aaw;
}

/**
 *  Sets the values of the properties encapsulated in the <code>AxisAnnotations</code>
 *  property.
 * @param aaw
 */
public void setAxisAnnotations(AxisAnnoWrapper aaw) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	aaw.setPropertyValues(axisArray);

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}
}

/**
 *  Gets the values of the properties encapsulated in the <code>AxisTimeLabels</code>
 *  property.
 * @return values of the properties encapsulated in the <code>AxisTimeLabels</code>
 *  property
 */
public AxisTimeLabelWrapper getAxisTimeLabels() {
	AxisTimeLabelWrapper axis_time_label_wrapper = new AxisTimeLabelWrapper();

	if (axis_time_label_wrapper == null) {
		axis_time_label_wrapper = new AxisTimeLabelWrapper();
	}
	axis_time_label_wrapper.setWrapperValues(axisArray);

	return axis_time_label_wrapper;
}

/**
 *  Sets the values of the properties encapsulated in the <code>AxisTimeLabels</code>
 *  property.
 * @param axis_time_label_wrapper
 */
public void setAxisTimeLabels(AxisTimeLabelWrapper axis_time_label_wrapper) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	axis_time_label_wrapper.setPropertyValues(axisArray);

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}

	// refresh wrapper values -- values change when isdef is set
	axis_time_label_wrapper.setWrapperValues(axisArray);
}

/**
 *  Gets the values of the properties encapsulated in the <code>AxisValueLabels</code>
 *  property.
 * @return values of the properties encapsulated in the <code>AxisValueLabels</code>
 *  property
 */
public AxisValueLabelWrapper getAxisValueLabels() {
	AxisValueLabelWrapper avw = new AxisValueLabelWrapper();
	avw.setWrapperValues(axisArray);

	return avw;
}

/**
 *  Sets the values of the properties encapsulated in the <code>AxisValueLabels</code>
 *  property.
 * @param avw
 */
public void setAxisValueLabels(AxisValueLabelWrapper avw) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	avw.setPropertyValues(axisArray);

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}

}

/**
 *  Gets the values of the properties encapsulated in the <code>AxisPointLabels</code>
 *  property.
 * @return values of the properties encapsulated in the <code>AxisPointLabels</code>
 *  property
 */
public AxisPointLabelWrapper getAxisPointLabels() {
	AxisPointLabelWrapper apw = new AxisPointLabelWrapper();

	apw.setWrapperValues(axisArray);

	return apw;
}

/**
 *  Sets the values of the properties encapsulated in the <code>AxisPointLabels</code>
 *  property.
 * @param apw
 */
public void setAxisPointLabels(AxisPointLabelWrapper apw) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);


	apw.setPropertyValues(axisArray);

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}
}

/**
 *  Gets the values of the properties encapsulated in the <code>AxisTitle</code>
 *  property.
 * @return values of the properties encapsulated in the <code>AxisTitle</code>
 *  property
 */
public AxisTitleWrapper getAxisTitle() {
	AxisTitleWrapper atw = new AxisTitleWrapper();
	atw.setWrapperValues(axisArray);

	return atw;
}

/**
 *  Sets the values of the properties encapsulated in the <code>AxisTitle</code>
 *  property.
 * @param atw
 */
public void setAxisTitle(AxisTitleWrapper atw) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	atw.setPropertyValues(axisArray);

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}

}

/**
 *  Gets the values of the properties encapsulated in the <code>AxisGrid</code>
 *  property.
 * @return values of the properties encapsulated in the <code>AxisGrid</code>
 *  property
 */
public AxisGridWrapper getAxisGrid() {
	AxisGridWrapper agw = new AxisGridWrapper();
	agw.setWrapperValues(axisArray);

	return agw;
}

/**
 *  Sets the values of the properties encapsulated in the <code>AxisGrid</code>
 *  property.
 * @param agw
 */
public void setAxisGrid(AxisGridWrapper agw) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	agw.setPropertyValues(axisArray);

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}

	// refresh grid wrapper values -- values change when isdef is set
	agw.setWrapperValues(axisArray);
}

/**
 *  Gets the values of the properties encapsulated in the <code>AxisOrigin</code>
 *  property.
 * @return values of the properties encapsulated in the <code>AxisOrigin</code>
 *  property
 */
public AxisOriginWrapper getAxisOrigin() {
	AxisOriginWrapper aow = new AxisOriginWrapper();
	aow.setWrapperValues(axisArray);

	return aow;
}

/**
 *  Sets the values of the properties encapsulated in the <code>AxisOrigin</code>
 *  property.
 * @param aow
 */
public void setAxisOrigin(AxisOriginWrapper aow) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	aow.setPropertyValues(axisArray);

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}
	aow.setWrapperValues(axisArray);
}

/**
 *  Gets the values of the properties encapsulated in the <code>AxisPlacement</code>
 *  property.
 * @return values of the properties encapsulated in the <code>AxisPlacement</code>
 *  property
 */
public AxisPlacementWrapper getAxisPlacement() {
	AxisPlacementWrapper apw = new AxisPlacementWrapper();
	apw.setWrapperValues(axisArray);
	for (int i = 0; i < axisArray.length; i++) {
		apw.paxis[i] = findAxisIndex(axisArray[i].getPlacementAxis());
	}

	return apw;
}

/**
 *  Sets the values of the properties encapsulated in the <code>AxisPlacement</code>
 *  property.
 * @param apw
 */
public void setAxisPlacement(AxisPlacementWrapper apw) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	apw.setPropertyValues(axisArray);

	for (int i = 0; i < axisArray.length; i++) {
    	if (apw.paxis[i] == -1) {
        	axisArray[i].setPlacementAxis(null);
		} else if (axisArray[i].isVertical() !=
				axisArray[apw.paxis[i]].isVertical()) {
        	axisArray[i].setPlacementAxis(axisArray[apw.paxis[i]]);
    	} else {
        	apw.paxis[i] = findAxisIndex(axisArray[i].getPlacementAxis());
    	}
	}

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}

}

/**
 * Given an axis, searches the axis array for it and returns its index.
 * @param checkAxis the axis to look for
 * @return the index of the found axis, or -1 if not found
 */
private int findAxisIndex(JCAxis checkAxis)
{
	int ind = -1;
	for (int j = 0; j < axisArray.length; j++) {
		if (checkAxis == axisArray[j]) {
			ind = j;
		}
	}
	return ind;
}

/**
 *  Gets the values of the properties encapsulated in the <code>AxisRelationship</code>
 *  property.
 * @return values of the properties encapsulated in the <code>AxisRelationship</code>
 *  property
 */
public AxisRelationshipWrapper getAxisRelationship() {
	AxisRelationshipWrapper arw = new AxisRelationshipWrapper();
	arw.setWrapperValues(axisArray);
	for (int i = 0; i < axisArray.length; i++) {
		arw.raxis[i] = findAxisIndex(axisArray[i].getFormula().getOriginator());
	}

	return arw;
}

/**
 *  Sets the values of the properties encapsulated in the <code>AxisRelationship</code>
 *  property.
 * @param arw
 */
public void setAxisRelationship(AxisRelationshipWrapper arw) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	arw.setPropertyValues(axisArray);
	for (int i = 0; i < axisArray.length; i++) {
    	if (arw.raxis[i] == -1 || arw.raxis[i] == i) {
        	axisArray[i].getFormula().setOriginator(null);
		} else {
        	axisArray[i].getFormula().setOriginator(axisArray[arw.raxis[i]]);
    	}
	}

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}

}

/**
 *  Gets the values of the properties encapsulated in the <code>AxisOrientation</code>
 *  property.
 * @return values of the properties encapsulated in the <code>AxisOrientation</code>
 *  property
 */
public AxisOrientationWrapper getAxisOrientation() {
	AxisOrientationWrapper aow = new AxisOrientationWrapper(axisArray.length);

	for (int i = 0; i < axisArray.length; i++) {
		AxisInfoWrapper axisInfo = new AxisInfoWrapper();
		axisInfo.getAxisInformation(axisArray[i]);
		aow.orientation[i] = getActualAxisOrientation(axisArray[i], axisInfo);
		aow.other_axis_index[i] = -1;
		for (int j = 0; j < axisArray.length; j++) {
			if (axisArray[j] == axisInfo.axis_other_axis) {
				aow.other_axis_index[i] = j;
			}
		}
	}
	aow.trigger_index = -1;

	return aow;
}

/**
 *  Sets the values of the properties encapsulated in the <code>AxisOrientation</code>
 *  property.
 * @param aow
 */
public void setAxisOrientation(AxisOrientationWrapper aow) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	for (int i = 0; i < axisArray.length; i++) {
		if (aow.trigger_index >= 0 && aow.trigger_index != i)
			continue;
		AxisInfoWrapper axisInfo = new AxisInfoWrapper();
		axisInfo.getAxisInformation(axisArray[i]);
		setActualAxisOrientation(axisArray[i], axisInfo,
			aow.orientation[i]);
	}

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}
}

/**
 *  Calculates an axis' orientation enum value depending on whether it
 *  is reversed, its data view is inverted, or it is an <i>x</i> or <i>y</i> axis.
 *  @param axis the axis to calculate the orientation value for
 *  @param axisInfo a wrapper containing important information for this axis
 *  @return the axis' orientation enum value
 */
private int getActualAxisOrientation(JCAxis axis, AxisInfoWrapper axisInfo)
{
    ChartDataView cdv = axisInfo.axis_cdv;
    JCAxis otherAxis = axisInfo.axis_other_axis;
    boolean xOrY = axisInfo.axis_xory;

    int orient = BaseChart.ORIENT_X_NORMAL_Y_NORMAL;
    if (cdv != null && cdv.isInverted())
        orient += BaseChart.ORIENT_INVERTED_X_NORMAL_Y_NORMAL;

    JCAxis checkAxis;

    if (xOrY)
        checkAxis = axis;
    else
        checkAxis = otherAxis;
    if (checkAxis != null && checkAxis.isReversed())
        orient += BaseChart.ORIENT_X_REVERSED_Y_NORMAL;

    if (!xOrY)
        checkAxis = axis;
    else
        checkAxis = otherAxis;
    if (checkAxis != null && checkAxis.isReversed())
        orient += BaseChart.ORIENT_X_NORMAL_Y_REVERSED;

    return orient;
}

/**
 *  Takes an axis' orientation enum value and sets the appropriate properties
 *  that are related to it: the axis' <code>Reversed</code> property and its
 *  data view's <code>Inverted</code> property.
 *  @param axis the axis to convert the orientation value for
 *  @param axisInfo a wrapper containing important information for this axis
 *  @param orientation the axis' orientation enum value
 */
private void setActualAxisOrientation(JCAxis axis, AxisInfoWrapper axisInfo,
															int orientation)
{
    ChartDataView cdv = axisInfo.axis_cdv;
    JCAxis otherAxis = axisInfo.axis_other_axis;
    boolean xOrY = axisInfo.axis_xory;

    int newOrient = orientation;

    if (newOrient > BaseChart.ORIENT_X_REVERSED_Y_REVERSED) {
        if (cdv != null) {
            cdv.setInverted(true);
        }
        newOrient -= BaseChart.ORIENT_INVERTED_X_NORMAL_Y_NORMAL;
    }
    else {
        if (cdv != null) {
            cdv.setInverted(false);
        }
    }
    if (newOrient > BaseChart.ORIENT_X_REVERSED_Y_NORMAL) {
        if (!xOrY) {
            axis.setReversed(true);
        } else if (otherAxis != null) {
            otherAxis.setReversed(true);
        }
        newOrient -= BaseChart.ORIENT_X_NORMAL_Y_REVERSED;
    }
    else {
        if (!xOrY) {
            axis.setReversed(false);
        } else if (otherAxis != null) {
            otherAxis.setReversed(false);
        }
    }
    if (xOrY) {
        axis.setReversed(
            newOrient == BaseChart.ORIENT_X_NORMAL_Y_NORMAL ? false : true);
    } else if (otherAxis != null) {
        otherAxis.setReversed(
            newOrient == BaseChart.ORIENT_X_NORMAL_Y_NORMAL ? false : true);
    }
}

/****************** Non-array properties ******************/

/**
 *  Gets the values of the properties encapsulated in the <code>LegendLayout</code>
 *  property.
 * @return values of the properties encapsulated in the <code>LegendLayout</code>
 *  property
 */
public LegendWrapper getLegendLayout() {
	LegendWrapper lw = new LegendWrapper();
	lw.setWrapperValues(getLegend());

	return lw;
}

/**
 *  Sets the values of the properties encapsulated in the <code>LegendLayout</code>
 *  property.
 * @param lw
 */
public void setLegendLayout(LegendWrapper lw) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	lw.setPropertyValues(getLegend());

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}

}

/**
 *  Gets the values of the properties encapsulated in the <code>LegendAppearance</code>
 *  property.
 * @return values of the properties encapsulated in the <code>LegendAppearance</code>
 *  property
 */
public AppearanceWrapper getLegendAppearance() {
	AppearanceWrapper lw = new AppearanceWrapper();
	getAppearance(getLegend(), lw);
	lw.prefix = JCChartBeanBundle.string(JCChartBeanBundle.key290);

	return lw;
}

/**
 *  Sets the values of the properties encapsulated in the <code>LegendAppearance</code>
 *  property.
 * @param lw
 */
public void setLegendAppearance(AppearanceWrapper lw) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	setAppearance(getLegend(), lw);
	getLegend().invalidate();

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}

}

/**
 *  Gets the values of the properties encapsulated in the <code>FooterAppearance</code>
 *  property.
 * @return values of the properties encapsulated in the <code>FooterAppearance</code>
 *  property
 */
public AppearanceWrapper getFooterAppearance() {
	AppearanceWrapper fw = new AppearanceWrapper();
	getAppearance(getFooter(), fw);
	fw.prefix = JCChartBeanBundle.string(JCChartBeanBundle.key291);

	return fw;
}

/**
 *  Sets the values of the properties encapsulated in the <code>FooterAppearance</code>
 *  property.
 * @param fw
 */
public void setFooterAppearance(AppearanceWrapper fw) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	//fw.setPropertyValues(getFooter());
	setAppearance(getFooter(), fw);

	getFooter().invalidate();

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}

}
/**
 *  Gets the values of the properties encapsulated in the <code>HeaderAppearance</code>
 *  property.
 * @return values of the properties encapsulated in the <code>HeaderAppearance</code>
 *  property
 */
public AppearanceWrapper getHeaderAppearance() {
	AppearanceWrapper hw = new AppearanceWrapper();
	getAppearance(getHeader(), hw);
	hw.prefix = JCChartBeanBundle.string(JCChartBeanBundle.key292);

	return hw;
}

/**
 *  Sets the values of the properties encapsulated in the <code>HeaderAppearance</code>
 *  property.
 * @param hw
 */
public void setHeaderAppearance(AppearanceWrapper hw) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	setAppearance(getHeader(), hw);

	getHeader().invalidate();

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}

}
/**
 *  Gets the values of the properties encapsulated in the <code>ChartAreaAppearance</code>
 *  property.
 * @return values of the properties encapsulated in the <code>ChartAreaAppearance</code>
 *  property
 */
public AppearanceWrapper getChartAreaAppearance() {
	AppearanceWrapper cw = new AppearanceWrapper();
	getAppearance(getChartArea(), cw);
	cw.prefix = JCChartBeanBundle.string(JCChartBeanBundle.key293);

	return cw;
}

/**
 *  Sets the values of the properties encapsulated in the <code>ChartAreaAppearance</code>
 *  property.
 * @param cw
 */
public void setChartAreaAppearance(AppearanceWrapper cw) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	setAppearance(getChartArea(), cw);
	getChartArea().invalidate();

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}

}
/**
 *  Gets the values of the properties encapsulated in the <code>ChartAppearance</code>
 *  property.
 * @return values of the properties encapsulated in the <code>ChartAppearance</code>
 *  property
 */
public AppearanceWrapper getChartAppearance() {
	AppearanceWrapper cw = new AppearanceWrapper();
	getAppearance(this, cw);
	cw.prefix = JCChartBeanBundle.string(JCChartBeanBundle.key294);

	return cw;
}

/**
 *  Sets the values of the properties encapsulated in the <code>ChartAppearance</code>
 *  property.
 * @param cw
 */
public void setChartAppearance(AppearanceWrapper cw) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	setAppearance(this, cw);

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}

}

/**
 *  Gets the values of the properties encapsulated in the <code>PlotAreaAppearance</code>
 *  property.
 * @return values of the properties encapsulated in the <code>PlotAreaAppearance</code>
 *  property
 */
public AppearanceWrapper getPlotAreaAppearance() {
	AppearanceWrapper pw = new AppearanceWrapper();
	getAppearance(getChartArea().getPlotArea(), pw);
	pw.prefix = JCChartBeanBundle.string(JCChartBeanBundle.key295);

	return pw;
}

/**
 *  Sets the values of the properties encapsulated in the <code>PlotAreaAppearance</code>
 *  property.
 * @param pw
 */
public void setPlotAreaAppearance(AppearanceWrapper pw) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	setAppearance(getChartArea().getPlotArea(), pw);

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}

}

/**
 * Sets the appearance values in the provided <code>AppearanceWrapper</code> object
 * by querying the passed object.
 * @param obj the object from which to query appearance values
 * @param aw the <code>AppearanceWrapper</code> object to fill
 */
private void getAppearance(Object obj, AppearanceWrapper aw)
{
    if (obj instanceof JCChart) {
        JCChart ch = (JCChart) obj;

        aw.background = ch.getBackground();
        aw.foreground = ch.getForeground();
        aw.opaque = ch.isOpaque();

        aw.border = JCSwingTypeConverter.fromBorder(ch.getBorder(), ch);
        aw.axisbounding = false;

        aw.thisobj = OBJ_CHART;
    } else if (obj instanceof PlotArea) {
        PlotArea pa = (PlotArea) obj;

        aw.background = pa.getBackground();
        aw.foreground = pa.getForeground();
        aw.visible = true;
		aw.border = null;
        aw.opaque = true;
        aw.axisbounding = ((JCChartArea)pa.getParent()).getAxisBoundingBox();

        aw.thisobj = OBJ_PLOT_AREA;
    } else if (obj instanceof JComponent) {
        JComponent jc = (JComponent) obj;

        aw.background = jc.getBackground();
        aw.foreground = jc.getForeground();
        aw.visible = jc.isVisible();
        aw.opaque = jc.isOpaque();
        aw.border = JCSwingTypeConverter.fromBorder(jc.getBorder(), jc);
        aw.axisbounding = false;

        aw.thisobj = OBJ_JCOMPONENT;
    } else {
        aw.background = Color.white;
        aw.foreground = Color.black;
        aw.visible = true;
        aw.opaque = false;
		aw.border = null;
        aw.axisbounding = false;

        aw.thisobj = OBJ_UNKNOWN;
	}
}

/**
 * Sets the appearance values of the specified object from the provided
 * <code>AppearanceWrapper</code> object.
 * @param obj the object to set appearance values in
 * @param aw the <code>AppearanceWrapper</code> object to use
 */
private void setAppearance(Object obj, AppearanceWrapper aw)
{
    if (obj instanceof JCChart) {
        JCChart ch = (JCChart) obj;

        ch.setBackground(aw.background);
        ch.setForeground(aw.foreground);
        ch.setOpaque(aw.opaque);
        ch.setBorder(JCSwingTypeConverter.toBorder(aw.border));
		aw.thisobj = OBJ_CHART;
    } else if (obj instanceof PlotArea) {
        PlotArea pa = (PlotArea) obj;

        pa.setBackground(aw.background);
        pa.setForeground(aw.foreground);
        ((JCChartArea)pa.getParent()).setAxisBoundingBox(aw.axisbounding);
		aw.thisobj = OBJ_PLOT_AREA;
    } else if (obj instanceof JComponent) {
        JComponent jc = (JComponent) obj;

        jc.setBackground(aw.background);
        jc.setForeground(aw.foreground);
        jc.setVisible(aw.visible);
        jc.setOpaque(aw.opaque);
        jc.setBorder(JCSwingTypeConverter.toBorder(aw.border));
		aw.thisobj = OBJ_JCOMPONENT;
    }
}

/**
 *  Gets the values of the properties encapsulated in the <code>HeaderText</code>
 *  property.
 * @return values of the properties encapsulated in the <code>HeaderText</code>
 *  property
 */
public TitleWrapper getHeaderText() {
	TitleWrapper hw = new TitleWrapper();
	hw.setWrapperValues(getHeader());
	hw.prefix = JCChartBeanBundle.string(JCChartBeanBundle.key292);

	return hw;
}

/**
 *  Sets the values of the properties encapsulated in the <code>HeaderText</code>
 *  property.
 * @param hw
 */
public void setHeaderText(TitleWrapper hw) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	hw.setPropertyValues(getHeader());

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}

}

/**
 *  Gets the values of the properties encapsulated in the <code>FooterText</code>
 *  property.
 * @return values of the properties encapsulated in the <code>FooterText</code>
 *  property
 */
public TitleWrapper getFooterText() {
	TitleWrapper fw = new TitleWrapper();
	fw.setWrapperValues(getFooter());
	fw.prefix = JCChartBeanBundle.string(JCChartBeanBundle.key291);

	return fw;
}

/**
 *  Sets the values of the properties encapsulated in the <code>FooterText</code>
 *  property.
 * @param fw
 */
public void setFooterText(TitleWrapper fw) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	fw.setPropertyValues(getFooter());

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}

}

/**
 *  Gets the values of the properties encapsulated in the <code>TriggerList</code>
 *  property.
 * @return values of the properties encapsulated in the <code>TriggerList</code>
 *  property
 */
public TriggerWrapper getTriggerList() {
	TriggerWrapper tw = new TriggerWrapper();
	tw.setWrapperValues(this);

	return tw;
}

/**
 *  Sets the values of the properties encapsulated in the <code>TriggerList</code>
 *  property.
 * @param tw
 */
public void setTriggerList(TriggerWrapper tw) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	tw.setPropertyValues(this);

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}

}

/**
 *  Sets the data source of the first data view to be derived from the
 *  provided instance of the Swing <code>TableModel</code>.
 * @param model
 */
public void setSwingDataModel1(TableModel model) {
    tableModel = model;
    dataSource = new JCChartSwingDataSource(model);
    getDataView(0).setDataSource((ChartDataModel) dataSource);
}

/**
 *  Gets the Swing <code>TableModel</code> instance being used to populate the data source
 *  of the first data view.
 * @return Swing <code>TableModel</code> instance being used to populate the data source
 *  of the first data view
 */
public TableModel getSwingDataModel1() {
    return tableModel;
}

/**
 *  Sets the data source of the second data view to be derived from the
 *  provided instance of the Swing <code>TableModel</code>.
 * @param model
 */
public void setSwingDataModel2(TableModel model) {
    tableModel2 = model;
    dataSource2 = new JCChartSwingDataSource(model);
    getDataView(1).setDataSource((ChartDataModel) dataSource2);
}

/**
 *  Gets the Swing <code>TableModel</code> instance being used to populate the data source
 *  of the second data view.
 * @return the Swing <code>TableModel</code> instance being used to populate the data source
 *  of the second data view
 */
public TableModel getSwingDataModel2() {
    return tableModel2;
}

}
