/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: MultiDataChartEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.ChartDataView;
import com.klg.jclass.chart.JCChartEnumMappings;

import java.awt.Insets;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Dimension;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import javax.swing.JPanel;
import javax.swing.BorderFactory;




import java.util.Vector;


/**
 * Editor for Multiple Data View Properties.
 */

public class MultiDataChartEditor extends RadioEditor 
	implements ItemListener {

protected JComboBox charttype_ch;
protected JComboBox xaxis_ch;
protected JComboBox yaxis_ch;

protected String[] charttype_names;
protected int[] charttype_vals;
protected String[] xaxis_names;
protected String[] yaxis_names;

public MultiDataChartEditor() {
	super(MultiChart.DATA_RADIO_TITLE, MultiChart.DATA_RADIO_NAMES);
}

protected void setEnumArrays()
{
	xaxis_names = new String[MultiChart.AXIS_RADIO_NAMES.length / 2];
	yaxis_names = new String[MultiChart.AXIS_RADIO_NAMES.length / 2];
	int xinc = 0;
	int yinc = 0;
	for (int i = 0; i < MultiChart.AXIS_RADIO_NAMES.length; i++) {
		if (MultiChart.AXIS_RADIO_NAMES[i].charAt(0) == 'X') {
			xaxis_names[xinc] = MultiChart.AXIS_RADIO_NAMES[i];

 
			xinc++;
		} else if (MultiChart.AXIS_RADIO_NAMES[i].charAt(0) == 'Y') {
			yaxis_names[yinc] = MultiChart.AXIS_RADIO_NAMES[i];

 
			yinc++;
		}
	}

	charttype_names = JCChartEnumMappings.chartType_i18n_strings;
	charttype_vals = JCChartEnumMappings.chartType_values;
}

/**
 * Creates a panel containing the GUI elements for the properties
 * associated with the radio buttons.
 * @return the panel
 */
protected JPanel makePropertyPanel() 
{
    JPanel p2 = new JPanel();
    p2.setBorder(BorderFactory.createTitledBorder(
    	JCChartBeanBundle.string(JCChartBeanBundle.key257)));
	p2.setLayout(new GridBagLayout());

    GridBagConstraints gbc2  = new GridBagConstraints();
    gbc2.gridwidth = 1;
    gbc2.gridx = GridBagConstraints.RELATIVE;
    gbc2.gridy = 0;
    gbc2.anchor = GridBagConstraints.WEST;
    gbc2.insets = new Insets(4,4,4,4);

	charttype_ch = new JComboBox();
	xaxis_ch = new JComboBox();
	yaxis_ch = new JComboBox();

	setEnumArrays();
	for (int i = 0; i < charttype_names.length; i++)
		charttype_ch.addItem(charttype_names[i]);
	for (int i = 0; i < xaxis_names.length; i++)
		xaxis_ch.addItem(xaxis_names[i]);
	for (int i = 0; i < yaxis_names.length; i++)
		yaxis_ch.addItem(yaxis_names[i]);

	JLabel charttype_label = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key258));
	JLabel xaxis_label = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key259));
	JLabel yaxis_label = new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key260));

	p2.add(charttype_label, gbc2);
	p2.add(charttype_ch, gbc2);
	gbc2.gridy++;
	p2.add(xaxis_label, gbc2);
	p2.add(xaxis_ch, gbc2);
	gbc2.gridy++;
	p2.add(yaxis_label, gbc2);
	p2.add(yaxis_ch, gbc2);

	charttype_ch.addItemListener(this);
	xaxis_ch.addItemListener(this);
	yaxis_ch.addItemListener(this);

	add("Center", p2);

	return p2;
}


 

MultiDataChartWrapper wrap = null;

public void setValue(Object value) {

    if (value != null) {
        try {
			wrap = (MultiDataChartWrapper) value;
        } catch (ClassCastException e) {
        }
    }

	if (wrap == null)
		wrap = new MultiDataChartWrapper(numOfRadios);

	switchValues(radioSel);
}

/**
 * Fills the property sheet with values associated with the newly selected
 * radio button.
 * @param sel the index of the newly selected radio button
 */
protected void switchValues(int sel)
{
	if (sel < 0 || wrap == null)
		return;

	selectAxis(wrap.xaxis, sel, xaxis_names, xaxis_ch);
	selectAxis(wrap.yaxis, sel, yaxis_names, yaxis_ch);

	for (int i = 0; i < charttype_vals.length; i++) {
		if (wrap.charttype[sel] == charttype_vals[i]) {
			if (charttype_ch.getSelectedIndex() != i)
				charttype_ch.setSelectedIndex(i);
		}
	}
}

private void selectAxis(int[] axisArray, int index, String[] axis_names, 
															JComboBox axis_ch)
{
	if (axisArray[index] != -1) {
		for (int i = 0; i < axis_names.length; i++) {
			if (MultiChart.AXIS_RADIO_NAMES[axisArray[index]].
											equals(axis_names[i])) {
				if (axis_ch.getSelectedIndex() != i)
					axis_ch.setSelectedIndex(i);
			}
		}
	}
}

public Object getValue() {
	return (Object) wrap;
}

public void setAsText(String text) throws IllegalArgumentException {
    Vector paramVector = BeanTextParser.parse(text);
    if (paramVector.size() != 3) {
        throw new IllegalArgumentException();
    }
    MultiDataChartWrapper setWrapper = null;
    try {
        setWrapper = new MultiDataChartWrapper(
            (String) paramVector.elementAt(0),
            (String) paramVector.elementAt(1),
            (String) paramVector.elementAt(2)
            );
        setValue(setWrapper);
        support.firePropertyChange("", null, null);
    } catch (Exception e) {
        throw new IllegalArgumentException();
    }
}

public String getAsText() {
    if (wrap != null)
        return wrap.toString();
    else
        return null;
}

public String getJavaInitializationString() {
	return "new com.klg.jclass.chart.beans.MultiDataChartWrapper(" + wrap + ")";
}

public Dimension getPreferredSize() {
    Dimension dim = super.getPreferredSize();
    dim.width = Math.max(dim.width, 300);
    dim.height += 350;
    return dim;
}

public void itemStateChanged( ItemEvent ev ) 
{
	super.itemStateChanged(ev);
	Object src = ev.getSource();
    if( src instanceof JComboBox &&
            ev.getStateChange() == ItemEvent.DESELECTED)
        return;
	checkProperties(src);
}

public void checkProperties( Object src)
{
	try {
		if (src == charttype_ch) {
			wrap.charttype[radioSel] = 
				charttype_vals[charttype_ch.getSelectedIndex()];
			if( support != null )
				support.firePropertyChange("", null, null);
		} else if (src == xaxis_ch) {
			int selected = xaxis_ch.getSelectedIndex();
			for (int i = 0; i < MultiChart.AXIS_RADIO_NAMES.length; i++) {
				if (MultiChart.AXIS_RADIO_NAMES[i].equals(
							xaxis_names[selected]) ) {
					wrap.xaxis[radioSel] = i;
				}
			}

 
			if(support != null )
				support.firePropertyChange("", null, null);
		} else if (src == yaxis_ch) {
			int selected = yaxis_ch.getSelectedIndex();
			for (int i = 0; i < MultiChart.AXIS_RADIO_NAMES.length; i++) {
				if (MultiChart.AXIS_RADIO_NAMES[i].equals(
							yaxis_names[selected]) ) {
					wrap.yaxis[radioSel] = i;
				}
			}

 
			if( support != null )
				support.firePropertyChange("", null, null);
		}
	} catch (Exception ex) {
	}
}

/* for testing purposes
public static void main( String args[] ) {
    JFrame frame;
    try {
    java.lang.Class newclass
        = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
    frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
        System.out.println(ex);
    frame = new JFrame();
    }

    MultiDataChartEditor axisGenEditor = new MultiDataChartEditor();
    axisGenEditor.setBackground( java.awt.Color.lightGray );
    //axisGenEditor.setValue( null);
	MultiDataChartWrapper thiswrap = new MultiDataChartWrapper("", "", "");
    axisGenEditor.setValue( thiswrap );
    frame.getContentPane().add( axisGenEditor );
    frame.pack();
    java.awt.Dimension bigness = axisGenEditor.getPreferredSize();
    frame.setSize( bigness.width + 100, bigness.height + 50 );
    frame.setVisible(true);
}
*/
}
