/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: MultiDataChartWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.ChartDataView;
import com.klg.jclass.chart.JCChartEnumMappings;
import java.util.Date;




import java.util.Vector;


/**
 * Wrapper class for Data Chart-related properties.
 * Used by the MultiChart Bean.
 */
public class MultiDataChartWrapper extends RadioDataWrapper {

public int      charttype[];
public int      xaxis[];
public int      yaxis[];

/**
 * Default constructor.  Members will be empty.
 */
public MultiDataChartWrapper() {
	super();
}

/**
 * Constructor.  Allocates space for the arrays, but does not fill them.
 * @param size
 */
public MultiDataChartWrapper(int size) {
	super();
	setWrapperArraySize(size);
}

/**
 * Constructor. Allocates space for the arrays and fills them by parsing
 * series Strings of format "<code>(x1 value)(y1 value)(x2 value)(y2 value)</code>".
 * @param charttypeString   chart type String in above format.  Possible values
 * are <code>PLOT</code>, <code>SCATTER_PLOT</code>, <code>AREA</code>, <code>STACKING_AREA</code>,
 * <code>BAR</code>, <code>STACKING_BAR</code>, <code>PIE</Code>, <code>HILO</code>,
 * <code>HILO_OPEN_CLOSE</code>, or <code>CANDLE</code>.
 * @param xaxisString       x axis String in above format.  Values are
 *                          the names of the axis being referenced (for example, <code>x1</code>).
 * @param yaxisString       y axis String in above format.  Values are
 *                          the names of the axis being referenced (for example, <code>y1</code>).
 */
public MultiDataChartWrapper(String charttypeString, String xaxisString,
    String yaxisString)
{
    setWrapperArraySize(MultiChart.DATA_RADIO_NAMES.length);

    setWrapperEnumValues(charttype, new RadioSeries(charttypeString),
        JCChartEnumMappings.chartType_strings, 
		JCChartEnumMappings.chartType_values,
        "AxisPlacement", JCChart.PLOT);
    setWrapperRadioIDValues(xaxis, new RadioSeries(xaxisString),
		MultiChart.AXIS_RADIO_IDS);
    setWrapperRadioIDValues(yaxis, new RadioSeries(yaxisString),
		MultiChart.AXIS_RADIO_IDS);
}

/**
 * Sets the size of the internal arrays that store properties for
 * axes (should be equal to the number of axes).
 * @param size the size of the internal arrays
 */
public void setWrapperArraySize(int size)
{
	charttype = new int[size];
	xaxis = new int[size];
	yaxis = new int[size];
}

/**
 * Stores the properties from 
 * the specified dataview in internal arrays at the specified index.
 * <br><B>Note:</b> The x-axis and y-axis wrapper values are set by the
 * Bean's <code>getDataChart()</code> method.
 * @param index the index of the array the corresponds to this dataview
 * @param dataview the dataview that contains the properties for this index
 */
public void setWrapperValue(int index, ChartDataView dataview)
{
	charttype[index] = dataview.getChartType();
	// xaxis and yaxis no longer set due to serialization problems.
	// now done in the bean's getDataChart() method.
}

/**
 * Sets the properties on 
 * the specified dataview from values stored in internal arrays 
 * at the specified index.
 * <br><B>Note:</b> The x-axis and y-axis properties are set by the
 * Bean's <code>setDataChart()</code> method.
 * @param index the index where the values for this dataview are stored
 * @param dataview the dataview that we need to set these values on
 */
public void setPropertyValue(int index, ChartDataView dataview)
{
	dataview.setChartType(charttype[index]);
	// xaxis and yaxis no longer set due to serialization problems.
	// now done in the bean's setDataChart() method.
}

/**
 * Converts properties to String for code generation.
 * @return String containing properties for code generation
 */
public String toString()
{
    String s = "";
    s += enumValuesToString(charttype,
        JCChartEnumMappings.chartType_strings, 
		JCChartEnumMappings.chartType_values);
    s += ",";
    s += radioIDValuesToString(xaxis, MultiChart.AXIS_RADIO_IDS);
    s += ",";
    s += radioIDValuesToString(yaxis, MultiChart.AXIS_RADIO_IDS);

    return s;
}

public boolean equals(Object obj)
{
	boolean eq = true;
	if (obj instanceof MultiDataChartWrapper) {
		MultiDataChartWrapper compareWrap = (MultiDataChartWrapper) obj;
		if (compareWrap.charttype != null && this.charttype != null) {
			for (int i = 0; i < compareWrap.charttype.length; i++) {
				if (compareWrap.charttype[i] != this.charttype[i])
					eq = false;
			}
		} else {
			if (compareWrap.charttype != this.charttype)
				eq = false;
		}
		if (compareWrap.xaxis != null && this.xaxis != null) {
			for (int i = 0; i < compareWrap.xaxis.length; i++) {
				if (compareWrap.xaxis[i] != this.xaxis[i])
					eq = false;
			}
		} else {
			if (compareWrap.xaxis != this.xaxis)
				eq = false;
		}
		if (compareWrap.yaxis != null && this.yaxis != null) {
			for (int i = 0; i < compareWrap.yaxis.length; i++) {
				if (compareWrap.yaxis[i] != this.yaxis[i])
					eq = false;
			}
		} else {
			if (compareWrap.yaxis != this.yaxis)
				eq = false;
		}

	} else {
		eq = false;
	}
	return eq;
}

}
