/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: MultiDataMiscEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import java.awt.Insets;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Dimension;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.BorderFactory;




import java.util.Vector;


/**
 * Editor for Multiple Data View Miscellaneous Properties.
 */

public class MultiDataMiscEditor extends RadioEditor 
	implements ItemListener {

protected JCheckBox showing_cb;
protected JCheckBox showinglegend_cb;
protected JCheckBox drawfront_cb;
protected JCheckBox autolabel_cb;

public MultiDataMiscEditor() {
	super(MultiChart.DATA_RADIO_TITLE, MultiChart.DATA_RADIO_NAMES);
}

/**
 * Creates a panel containing the GUI elements for the properties
 * associated with the radio buttons.
 * @return the panel
 */
protected JPanel makePropertyPanel() 
{
    JPanel p2 = new JPanel();
    p2.setBorder(BorderFactory.createTitledBorder(
    		JCChartBeanBundle.string(JCChartBeanBundle.key263)));
	p2.setLayout(new GridLayout(4,1));

	showing_cb = new JCheckBox(JCChartBeanBundle.string(JCChartBeanBundle.key191));
	showinglegend_cb = new JCheckBox(JCChartBeanBundle.string(JCChartBeanBundle.key264));
	drawfront_cb = new JCheckBox(JCChartBeanBundle.string(JCChartBeanBundle.key265));
	autolabel_cb = new JCheckBox(JCChartBeanBundle.string(JCChartBeanBundle.key266));

	p2.add(showing_cb);
	p2.add(showinglegend_cb);
	p2.add(drawfront_cb);
	p2.add(autolabel_cb);

	showing_cb.addItemListener(this);
	showinglegend_cb.addItemListener(this);
	drawfront_cb.addItemListener(this);
	autolabel_cb.addItemListener(this);

	add("Center", p2);

	return p2;
}


 

MultiDataMiscWrapper wrap = null;

public void setValue(Object value) {

    if (value != null) {
        try {
			wrap = (MultiDataMiscWrapper) value;
        } catch (ClassCastException e) {
        }
    }

	if (wrap == null)
		wrap = new MultiDataMiscWrapper(numOfRadios);

	switchValues(radioSel);
}

/**
 * Fills the property sheet with values associated with the newly selected
 * radio button.
 * @param sel the index of the newly selected radio button
 */
protected void switchValues(int sel)
{
	if (sel < 0 || wrap == null)
		return;
	if (showing_cb.isSelected() != wrap.showing[sel])
		showing_cb.setSelected(wrap.showing[sel]);
	if (showinglegend_cb.isSelected() != wrap.showinglegend[sel])
		showinglegend_cb.setSelected(wrap.showinglegend[sel]);
	if (drawfront_cb.isSelected() != wrap.drawfront[sel])
		drawfront_cb.setSelected(wrap.drawfront[sel]);
	if (autolabel_cb.isSelected() != wrap.autolabel[sel])
		autolabel_cb.setSelected(wrap.autolabel[sel]);
}

public Object getValue() {
	return (Object) wrap;
}

public void setAsText(String text) throws IllegalArgumentException {
    Vector paramVector = BeanTextParser.parse(text);
    if (paramVector.size() != 4) {
        throw new IllegalArgumentException();
    }
    MultiDataMiscWrapper setWrapper = null;
    try {
        setWrapper = new MultiDataMiscWrapper(
            (String) paramVector.elementAt(0),
            (String) paramVector.elementAt(1),
            (String) paramVector.elementAt(2),
            (String) paramVector.elementAt(3)
            );
        setValue(setWrapper);
        support.firePropertyChange("", null, null);
    } catch (Exception e) {
        throw new IllegalArgumentException();
    }
}

public String getAsText() {
    if (wrap != null)
        return wrap.toString();
    else
        return null;
}

public String getJavaInitializationString() {
	return "new com.klg.jclass.chart.beans.MultiDataMiscWrapper(" + wrap + ")";
}

public Dimension getPreferredSize() {
    Dimension dim = super.getPreferredSize();
    dim.width = Math.max(dim.width, 300);
    dim.height += 350;
    return dim;
}

public void itemStateChanged( ItemEvent ev ) 
{
	super.itemStateChanged(ev);
	Object src = ev.getSource();
	checkProperties(src);
}

public void checkProperties( Object src)
{
	try {
		if (src == showing_cb) {
			wrap.showing[radioSel] = showing_cb.isSelected();
			if( support != null )
				support.firePropertyChange("", null, null);
		} else if (src == showinglegend_cb) {
			wrap.showinglegend[radioSel] = showinglegend_cb.isSelected();
			if( support != null )
				support.firePropertyChange("", null, null);
		} else if (src == drawfront_cb) {
			wrap.drawfront[radioSel] = drawfront_cb.isSelected();
			if( support != null )
				support.firePropertyChange("", null, null);
		} else if (src == autolabel_cb) {

 
			wrap.autolabel[radioSel] = autolabel_cb.isSelected();
			if( support != null )
				support.firePropertyChange("", null, null);
		}
	} catch (Exception ex) {
	}
}

/* for testing purposes
public static void main( String args[] ) {
    JFrame frame;
    try {
    java.lang.Class newclass
        = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
    frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
        System.out.println(ex);
    frame = new JFrame();
    }

    MultiDataMiscEditor axisGenEditor = new MultiDataMiscEditor();
    axisGenEditor.setBackground( java.awt.Color.lightGray );
    //axisGenEditor.setValue( null);
	MultiDataMiscWrapper thiswrap = new MultiDataMiscWrapper("", "", "", "");
    axisGenEditor.setValue( thiswrap );
    frame.getContentPane().add( axisGenEditor );
    frame.pack();
    java.awt.Dimension bigness = axisGenEditor.getPreferredSize();
    frame.setSize( bigness.width + 100, bigness.height + 50 );
    frame.setVisible(true);
}
*/
}
