/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: MultiDataMiscWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.ChartDataView;
import java.util.Date;




import java.util.Vector;


/**
 * Wrapper class for miscellaneous Data properties.
 * Used by the MultiChart Bean.
 */
public class MultiDataMiscWrapper extends RadioDataWrapper {

public boolean  showing[];
public boolean  showinglegend[];
public boolean  drawfront[];
public boolean  autolabel[];

/**
 * Default constructor.  Members will be empty.
 */
public MultiDataMiscWrapper() {
	super();
}

/**
 * Constructor. Allocates space for the arrays, but does not fill them.
 * @param size
 */
public MultiDataMiscWrapper(int size) {
	super();
	setWrapperArraySize(size);
}

/**
 * Constructor. Allocates space for the arrays and fills them by parsing
 * series Strings of format "<code>(x1 value)(y1 value)(x2 value)(y2 value)</code>".
 * @param showingString        showing String in above format.  Values are
 *                             boolean values.
 * @param showingLegendString  <code>showingLegend</code> String in above format.  Values
 *                             are boolean values.
 * @param drawFrontString      <code>drawFront</code> String in above format.  Values are
 *                             boolean values.
 * @param autoLabelString      <code>autoLabel</code> String in above format.  Values are
 *                             boolean values.
 */
public MultiDataMiscWrapper(String showingString, String showingLegendString,
    String drawFrontString, String autoLabelString)
{
    setWrapperArraySize(MultiChart.DATA_RADIO_NAMES.length);

    setWrapperBooleanValues(showing, new RadioSeries(showingString));
    setWrapperBooleanValues(showinglegend, new RadioSeries(showingLegendString));
    setWrapperBooleanValues(drawfront, new RadioSeries(drawFrontString));
    setWrapperBooleanValues(autolabel, new RadioSeries(autoLabelString));
}

/**
 * Sets the size of the internal arrays that store properties for
 * axes (should be equal to the number of axes).
 * @param size the size of the internal arrays
 */
public void setWrapperArraySize(int size)
{
	showing = new boolean[size];
	showinglegend = new boolean[size];
	drawfront = new boolean[size];
	autolabel = new boolean[size];
}

/**
 * Stores the properties from 
 * the specified dataview in internal arrays at the specified index.
 * @param index the index of the array that corresponds to this dataview
 * @param dataview the dataview that contains the properties for this index
 */
public void setWrapperValue(int index, ChartDataView dataview)
{
	showing[index] = dataview.isVisible();
	showinglegend[index] = dataview.isVisibleInLegend();
	drawfront[index] = dataview.getDrawFrontPlane();
	autolabel[index] = dataview.getAutoLabel();
}

/**
 * Sets the properties on 
 * the specified dataview from values stored in internal arrays 
 * at the specified index.
 * @param index the index where the values for this dataview are stored
 * @param dataview the dataview that we need to set these values on
 */
public void setPropertyValue(int index, ChartDataView dataview)
{

	dataview.setVisible(showing[index]);
	dataview.setVisibleInLegend(showinglegend[index]);
	dataview.setDrawFrontPlane(drawfront[index]);
	dataview.setAutoLabel(autolabel[index]);
}

/**
 * Converts properties to String for code generation.
 * @return String containing properties for code generation
 */
public String toString()
{
    String s = "";
    s += booleanValuesToString(showing);
    s += ",";
    s += booleanValuesToString(showinglegend);
    s += ",";
    s += booleanValuesToString(drawfront);
    s += ",";
    s += booleanValuesToString(autolabel);

    return s;
}

public boolean equals(Object obj)
{
	boolean eq = true;
	if (obj instanceof MultiDataMiscWrapper) {
		MultiDataMiscWrapper compareWrap = (MultiDataMiscWrapper) obj;
		if (compareWrap.showing != null && this.showing != null) {
			for (int i = 0; i < compareWrap.showing.length; i++) {
				if (compareWrap.showing[i] != this.showing[i])
					eq = false;
			}
		} else {
			if (compareWrap.showing != this.showing)
				eq = false;
		}
		if (compareWrap.showinglegend != null && this.showinglegend != null) {
			for (int i = 0; i < compareWrap.showinglegend.length; i++) {
				if (compareWrap.showinglegend[i] != this.showinglegend[i])
					eq = false;
			}
		} else {
			if (compareWrap.showinglegend != this.showinglegend)
				eq = false;
		}
		if (compareWrap.drawfront != null && this.drawfront != null) {
			for (int i = 0; i < compareWrap.drawfront.length; i++) {
				if (compareWrap.drawfront[i] != this.drawfront[i])
					eq = false;
			}
		} else {
			if (compareWrap.drawfront != this.drawfront)
				eq = false;
		}
		if (compareWrap.autolabel != null && this.autolabel != null) {
			for (int i = 0; i < compareWrap.autolabel.length; i++) {
				if (compareWrap.autolabel[i] != this.autolabel[i])
					eq = false;
			}
		} else {
			if (compareWrap.autolabel != this.autolabel)
				eq = false;
		}

	} else {
		eq = false;
	}
	return eq;
}

}
