/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: MultiDataSourceEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import java.awt.Insets;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Dimension;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.BorderFactory;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;




import java.util.Vector;


/**
 * Editor for Multiple Data View Source Properties.
 */

public class MultiDataSourceEditor extends RadioEditor
	implements DocumentListener, ItemListener {

protected JTextArea data_ta;
protected ButtonGroup data_cbg;
protected JCheckBox datafile_cb;
protected JCheckBox dataarea_cb;
protected JTextField datafile_tf;

public MultiDataSourceEditor() {
	super(MultiChart.DATA_RADIO_TITLE, MultiChart.DATA_RADIO_NAMES);
}

/**
 * Creates a panel containing the GUI elements for the properties
 * associated with the radio buttons.
 * @return the panel
 */
protected JPanel makePropertyPanel() 
{
    JPanel p2 = new JPanel();
    p2.setBorder(BorderFactory.createTitledBorder(
    	JCChartBeanBundle.string(JCChartBeanBundle.key269)));
	p2.setLayout(new GridLayout(1,1));

	p2.add(makeDataEditor());

	add("Center", p2);

	return p2;
}

boolean isFile = false;

public JPanel makeDataEditor()
{
	JPanel editor = new JPanel();
	JPanel boxes = new JPanel();
	JPanel fileLocation = new JPanel();
	data_ta = new JTextArea();
	data_ta.setText("");
	data_cbg = new ButtonGroup();
	datafile_cb = new JCheckBox( JCChartBeanBundle.string( JCChartBeanBundle.key171 ),
																		false);
	dataarea_cb = new JCheckBox( JCChartBeanBundle.string( JCChartBeanBundle.key172 ),
																		true);
	data_cbg.add(datafile_cb);
	data_cbg.add(dataarea_cb);
	datafile_tf = new JTextField("", 30 );
	
	editor.setLayout( new BorderLayout() );
	boxes.setLayout( new FlowLayout( FlowLayout.CENTER, 4, 4 ) );
	fileLocation.setLayout( new FlowLayout( FlowLayout.CENTER, 4, 4 ) );
	boxes.add( dataarea_cb );
	boxes.add( datafile_cb );
	fileLocation.add( new JLabel( JCChartBeanBundle.string( JCChartBeanBundle.key173 ) + ":" ) );
	fileLocation.add( datafile_tf );

	data_ta.setEnabled(true);
	datafile_tf.setEnabled(false);

	data_ta.getDocument().addDocumentListener(this);
	datafile_tf.getDocument().addDocumentListener(this);
	datafile_cb.addItemListener(this);
	dataarea_cb.addItemListener(this);

	editor.add( boxes, BorderLayout.NORTH );
	editor.add( data_ta, BorderLayout.CENTER );
	editor.add( fileLocation, BorderLayout.SOUTH );
	return editor;
}

MultiDataSourceWrapper wrap = null;
boolean firstTime = true;

public void setValue(Object value) {

    if (value != null) {
        try {
			wrap = (MultiDataSourceWrapper) value;
        } catch (ClassCastException e) {
        }
    }

	if (wrap == null)
		wrap = new MultiDataSourceWrapper(numOfRadios);

	if (firstTime) {
	switchValues(radioSel);
	firstTime = false;
	}
}

/**
 * Fills the property sheet with values associated with the newly selected
 * radio button.
 * @param sel the index of the newly selected radio button
 */
protected void switchValues(int sel)
{
	if (sel < 0 || wrap == null)
		return;

	if (wrap.data[sel] != null && !wrap.data[sel].equals("")) {
		isFile = wrap.data[sel].indexOf("ARRAY") < 0;
	} else {
		isFile = true;
	}

	if (isFile) {
			datafile_tf.setText(wrap.data[sel]);
			data_ta.setText(wrap.data_other[sel]);
			if (datafile_cb.isSelected() != true)
				datafile_cb.setSelected(true);
	} else {
			data_ta.setText(wrap.data[sel]);
			datafile_tf.setText(wrap.data_other[sel]);
			if (dataarea_cb.isSelected() != true) {
				dataarea_cb.setSelected(true);
			}
	}
	data_ta.setEnabled(!isFile);
	datafile_tf.setEnabled(isFile);

}

public Object getValue() {
	return (Object) wrap;
}

public void setAsText(String text) throws IllegalArgumentException {
    Vector paramVector = BeanTextParser.parse(text);
    if (paramVector.size() != 1) {
        throw new IllegalArgumentException();
    }
    MultiDataSourceWrapper setWrapper = null;
    try {
        setWrapper = new MultiDataSourceWrapper(
            (String) paramVector.elementAt(0)
            );
        setValue(setWrapper);
        support.firePropertyChange("", null, null);
    } catch (Exception e) {
        throw new IllegalArgumentException();
    }
}

public String getAsText() {
    if (wrap != null)
        return wrap.toString();
    else
        return null;
}


public String getJavaInitializationString() {
	return "new com.klg.jclass.chart.beans.MultiDataSourceWrapper(" + wrap + ")";
}

public Dimension getPreferredSize() {
    Dimension dim = super.getPreferredSize();
    dim.width = Math.max(dim.width, 450);
    dim.height += 350;
    return dim;
}

public void itemStateChanged( ItemEvent ev ) 
{
	super.itemStateChanged(ev);
	Object src = ev.getSource();
	if (src instanceof JRadioButton)
		return;
	checkProperties(src);
}

public void changedUpdate( DocumentEvent ev )
{
    Object src = ev.getDocument();
    checkProperties(src);
}
public void insertUpdate( DocumentEvent ev )
{
    Object src = ev.getDocument();
    checkProperties(src);
}
public void removeUpdate( DocumentEvent ev )
{
    Object src = ev.getDocument();
    checkProperties(src);
}

public void checkProperties( Object src)
{
	try {
		if (src == data_ta.getDocument()) {
			if (!isFile) {
				String data_text = data_ta.getText();
				if (!data_text.equals(wrap.data[radioSel])) {
					wrap.data[radioSel] = data_ta.getText();
					if( support != null )
						support.firePropertyChange("", null, null);
				}
			} else {
				wrap.data_other[radioSel] = data_ta.getText();
			}
		} else if (src == datafile_tf.getDocument()) {
			if (isFile) {
				wrap.data[radioSel] = datafile_tf.getText();
				if( support != null )
					support.firePropertyChange("", null, null);
			} else {
				wrap.data_other[radioSel] = datafile_tf.getText();
			}
		} else if (src == datafile_cb) {
			datafile_tf.setEnabled(true);
			data_ta.setEnabled(false);
			isFile = true;
			wrap.data[radioSel] = datafile_tf.getText();
			wrap.data_other[radioSel] = data_ta.getText();
			if( support != null )
				support.firePropertyChange("", null, null);
		} else if (src == dataarea_cb) {
			data_ta.setEnabled(true);
			datafile_tf.setEnabled(false);
			isFile = false;
			wrap.data[radioSel] = data_ta.getText();
			wrap.data_other[radioSel] = datafile_tf.getText();
			if( support != null )
				support.firePropertyChange("", null, null);
		}
	} catch (Exception ex) {
	}
}



/* for testing purposes
public static void main( String args[] ) {
    JFrame frame;
    try {
    java.lang.Class newclass
        = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
    frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
        System.out.println(ex);
    frame = new JFrame();
    }

    MultiDataSourceEditor axisGenEditor = new MultiDataSourceEditor();
    axisGenEditor.setBackground( java.awt.Color.lightGray );
    //axisGenEditor.setValue( null);
	MultiDataSourceWrapper thiswrap = new MultiDataSourceWrapper("");
    axisGenEditor.setValue( thiswrap );
    frame.getContentPane().add( axisGenEditor );
    frame.pack();
    java.awt.Dimension bigness = axisGenEditor.getPreferredSize();
    frame.setSize( bigness.width + 100, bigness.height + 50 );
    frame.setVisible(true);
}
*/
}
