/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: MultiDataSourceWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.ChartDataView;
import java.util.Date;




import java.util.Vector;


/**
 * Wrapper class for Data Source properties.
 * Used by the MultiChart Bean.
 */
public class MultiDataSourceWrapper extends RadioDataWrapper {

public String   data[];        // string for the active data source

public String   data_other[];  // string for the inactive data source
							   // needed for switching back to it at user's
							   // request

/**
 * Default constructor.  Members will be empty.
 */
public MultiDataSourceWrapper() {
	super();
}

/**
 * Constructor. Allocates space for the arrays, but does not fill them.
 * @param size
 */
public MultiDataSourceWrapper(int size) {
	super();
	setWrapperArraySize(size);
}

/**
 * Constructor. Allocates space for the arrays and fills them by parsing
 * series Strings of format "<code>(x1 value)(y1 value)(x2 value)(y2 value)</code>".
 * @param dataSourceString  data source String in above format.  Values
 *                          can either be a file name or a String
 *                          representing a data source.  See the
 *                          <code>JCInputStreamDataSource</code> docs for information on
 *                          the format.  Use <i>'\n'</i> for newline characters.
 */
public MultiDataSourceWrapper(String dataSourceString)
{
	setWrapperArraySize(MultiChart.DATA_RADIO_NAMES.length);

	setWrapperStringValues(data, new RadioSeries(dataSourceString));

	for (int i = 0; i < data_other.length; i++) {
		data_other[i] = "";
	}
}

/**
 * Sets the size of the internal arrays that store properties for
 * axes (should be equal to the number of axes).
 * @param size the size of the internal arrays
 */
public void setWrapperArraySize(int size)
{
	data = new String[size];
	data_other = new String[size];
}

/**
 * Stores the properties from 
 * the specified dataview in internal arrays at the specified index.
 * <br><B>Note:</b> Not currently used.  Wrapper value setting is done in the
 * Bean's <code>getDataSource()</code> method.
 * @param index the index of the array the corresponds to this dataview
 * @param dataview the dataview that contains the properties for this index
 */
public void setWrapperValue(int index, ChartDataView dataview)
{
}

/**
 * Sets the properties on 
 * the specified dataview from values stored in internal arrays 
 * at the specified index.
 * <br><B>Note:</b> Not currently used.  Property setting is done in the
 * Bean's <code>setDataSource()</code> method.
 * @param index the index where the values for this dataview are stored
 * @param dataview the dataview that we need to set these values on
 */
public void setPropertyValue(int index, ChartDataView dataview)
{
}

/**
 * Converts properties to String for code generation.
 * @return String containing properties for code generation.
 */
public String toString()
{
    String s = "";
    s += stringValuesToBigString(data);
	
	s = s.replace('\r', ' ');

    return s;
}

public boolean equals(Object obj)
{
	boolean eq = true;
	if (obj instanceof MultiDataSourceWrapper) {
		MultiDataSourceWrapper compareWrap = (MultiDataSourceWrapper) obj;
		if (compareWrap.data != null && this.data != null) {
			for (int i = 0; i < compareWrap.data.length; i++) {
				if (compareWrap.data[i] != null && this.data[i] != null) {
					if (!compareWrap.data[i].equals(this.data[i]))
						eq = false;
				} else {
					if (compareWrap.data[i] != this.data[i])
						eq = false;
				}
			}
		} else {
			if (compareWrap.data != this.data)
				eq = false;
		}
		if (compareWrap.data_other != null && this.data_other != null) {
			for (int i = 0; i < compareWrap.data_other.length; i++) {
				if (compareWrap.data_other[i] != null && this.data_other[i] != null) {
					if (!compareWrap.data_other[i].equals(this.data_other[i]))
						eq = false;
				} else {
					if (compareWrap.data_other[i] != this.data_other[i])
						eq = false;
				}
			}
		} else {
			if (compareWrap.data_other != this.data_other)
				eq = false;
		}

	} else {
		eq = false;
	}
	return eq;
}

}
