/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: OutlineStyleWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import java.awt.Color;

import com.klg.jclass.chart.ChartDataView;

/**
 * Wrapper class for <code>OutlineStyle</code> properties.
 */
public class OutlineStyleWrapper implements java.io.Serializable {

protected int width;
protected Color color;
protected int pattern;
protected int join;
protected int cap;

/**
 * Default constructor.
 */
public OutlineStyleWrapper() {
}

/**
 * Constructor.
 * @param width pixel width of outline
 * @param color outline color
 * @param pattern <code>JCLineStyle.NONE</code>, <code>JCLineStyle.SOLID</code>,
 * <code>JCLineStyle.LONG_DASH</code>, <code>JCLineStyle.SHORT_DASH</code>,
 * <code>JCLineStyle.LSL_DASH</code>, or <code>JCLineStyle.DASH_DOT</code>
 * @param join <code>BasicStroke.JOIN_MITER</code>, <code>BasicStroke.JOIN_BEVEL</code>,
 * or <code>BasicStroke.JOIN_ROUND</code>
 * @param cap <code>BasicStroke.CAP_BUTT</code>, <code>BasicStroke.CAP_ROUND</code>,
 * or <code>BasicStroke.CAP_SQUARE</code>
 */
public OutlineStyleWrapper(int width, Color color, int pattern,
							int join, int cap) {
	this.width = width;
	this.color = color;
	this.pattern = pattern;
	this.join = join;
	this.cap = cap;
}

/**
 * Stores <code>OutlineStyle</code> properties.
 * @param view <code>ChartDataView</code> object with get methods for
 * <code>OutlineStyle</code> properties
 */
public void setWrapperValues(ChartDataView view) {
	if (view != null) {
		width = view.getOutlineWidth();
		color = view.getOutlineColor();
		pattern = view.getOutlinePattern();
		join = view.getOutlineJoin();
		cap = view.getOutlineCap();
	}
}

/**
 * Sets <code>OutlineStyle</code> properties using methods provided by the
 * <code>ChartDataView</code> object.
 * @param view <code>ChartDataView</code> object
 */
public void setPropertyValues(ChartDataView view) {
	if (view != null) {
		view.setOutlineWidth(width);
		view.setOutlineColor(color);
		view.setOutlinePattern(pattern);
		view.setOutlineJoin(join);
		view.setOutlineCap(cap);
	}
}

/**
 * Converts properties to String for code generation.
 * @return String containing properties for code generation
 */
public String toString() {
    String s = "";
	s += width;
	s += ",";
	s += color;
	s += ",";
	s += pattern;
	s += ",";
	s += join;
	s += ",";
	s += cap;

    return s;
}

public boolean equals(Object obj) {
	boolean eq = true;
	if (obj instanceof OutlineStyleWrapper) {
		OutlineStyleWrapper compareWrap = (OutlineStyleWrapper) obj;
		if (compareWrap.width != this.width) {
			eq = false;
		}
		if (!compareWrap.color.equals(this.color)) {
			eq = false;
		}
		if (compareWrap.pattern != this.pattern) {
			eq = false;
		}
		if (compareWrap.join != this.join) {
			eq = false;
		}
		if (compareWrap.cap != this.cap) {
			eq = false;
		}

	} else {
		eq = false;
	}
	return eq;
}

}
