/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PolarRadarEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCChartArea;
import com.klg.jclass.chart.JCChartUtil;

import java.beans.PropertyEditor;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.BorderLayout;
import java.awt.Insets;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.BorderFactory;




import java.util.Vector;


/**
 * Bean editor for Polar/Radar chart properties.
 */
public class PolarRadarEditor extends JPanel
				implements PropertyEditor, ItemListener,
				FocusListener, ActionListener {

protected PropertyChangeSupport support = new PropertyChangeSupport(this);

protected JTextField textOriginBase;
protected JTextField textYAxisAngle;
protected JComboBox comboAngleUnit;
protected JCheckBox cbHalfRange;
protected JCheckBox cbCircularGrid;

protected String[] strAngleUnit = {
	JCChartBeanBundle.string(JCChartBeanBundle.key180),  // Degrees
	JCChartBeanBundle.string(JCChartBeanBundle.key181),  // Radians
	JCChartBeanBundle.string(JCChartBeanBundle.key182),  // Grads
	};

protected int[] angleUnits = {
	JCChartUtil.DEGREES,
	JCChartUtil.RADIANS,
	JCChartUtil.GRADS,
	};

public PolarRadarEditor() {
	init();
}


protected void init() {
	setLayout( new BorderLayout() );
	setBorder( BorderFactory.createEmptyBorder( 2, 2, 2, 2 ) );

    JPanel p1 = new JPanel();
	p1.setLayout( new GridLayout( 3, 2) );

	textOriginBase = new JTextField( "", 5 );
	textYAxisAngle = new JTextField( "", 5 );
	comboAngleUnit = new JComboBox();
	textOriginBase.addActionListener( this );
	textOriginBase.addFocusListener( this );
	textYAxisAngle.addActionListener( this );
	textYAxisAngle.addFocusListener( this );

	comboAngleUnit.addActionListener( this );

    p1.add(new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key175)));
	p1.add(textOriginBase);
    p1.add(new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key176)));
	p1.add(textYAxisAngle);

    p1.add(new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key177)));
    for (int i=0; i < strAngleUnit.length; i++) {
		comboAngleUnit.addItem(strAngleUnit[i]);
	}
	p1.add(comboAngleUnit);

	add("North", p1);

    JPanel p2 = new JPanel();
	p2.setLayout(new GridLayout(2,1));
    if ((cbHalfRange =
    		new JCheckBox(JCChartBeanBundle.string(JCChartBeanBundle.key178),
    														false)) != null) {
		cbHalfRange.addItemListener( this );
        p2.add(cbHalfRange);
    }
    if ((cbCircularGrid =
    		new JCheckBox(JCChartBeanBundle.string(JCChartBeanBundle.key179),
    														false)) != null) {
		cbCircularGrid.addItemListener( this );
        p2.add(cbCircularGrid);
    }
	add("South", p2);
}

PolarRadarWrapper wrap = null;

public void setValue(Object value) {
    if (value != null) {
        try {
			wrap = (PolarRadarWrapper) value;
        } catch (ClassCastException e) {
        }
    }

	if (wrap == null)
		wrap = new PolarRadarWrapper();

	switchValues();
}

/**
 * Fills the property sheet with values.
 */
protected void switchValues() {
	if (wrap == null) {
		return;
	}
	textOriginBase.setText("" + wrap.originBase);
	textYAxisAngle.setText("" + wrap.yAxisAngle);
	comboAngleUnit.setSelectedIndex(getAngleUnitIndex(wrap.angleUnit));
	cbHalfRange.setSelected(wrap.halfRange);
	cbCircularGrid.setSelected(wrap.circularGrid);
}

protected int getAngleUnitIndex(int angleUnit) {
	int index = Integer.MIN_VALUE;
	for (int i=0; i < angleUnits.length; i++) {
		if (angleUnits[i] == angleUnit) {
			index = i;
			break;
		}
	}
	return index;
}


protected void setSelectedValue(Object src) {
	if (wrap == null) {
		return;
	}

	if (src == textOriginBase) {
		try {
			wrap.originBase = Double.parseDouble(textOriginBase.getText());
		}
		catch (NumberFormatException e) {
		}
	}
	else if (src == textYAxisAngle) {
		try {
			wrap.yAxisAngle = Double.parseDouble(textYAxisAngle.getText());
		}
		catch (NumberFormatException e) {
		}
	}
	else if (src == comboAngleUnit) {
		int index = comboAngleUnit.getSelectedIndex();
		if (index >=0 && index < angleUnits.length) {
			wrap.angleUnit = angleUnits[index];
		}
	}
	else if (src == cbHalfRange) {
		wrap.halfRange = cbHalfRange.isSelected();
	}
	else if (src == cbCircularGrid) {
		wrap.circularGrid = cbCircularGrid.isSelected();
	}

	support.firePropertyChange("", null, null);
}


public Object getValue() {
	return (Object) wrap;
}

public void setAsText(String text) throws IllegalArgumentException {
    Vector paramVector = BeanTextParser.parse(text);
    if (paramVector.size() != 5) {
        throw new IllegalArgumentException();
    }
    PolarRadarWrapper setWrapper = null;
    try {
        setWrapper = new PolarRadarWrapper(
            Double.parseDouble((String) paramVector.elementAt(0)),
            Double.parseDouble((String) paramVector.elementAt(1)),
            Integer.parseInt((String) paramVector.elementAt(2)),
            Boolean.valueOf((String) paramVector.elementAt(3)).booleanValue(),
            Boolean.valueOf((String) paramVector.elementAt(4)).booleanValue()
            );
        setValue(setWrapper);
        support.firePropertyChange("", null, null);
    } catch (Exception e) {
        throw new IllegalArgumentException();
    }
}

public String getAsText() {
    if (wrap != null)
        return wrap.toString();
    else
        return null;
}

public String[] getTags() { return null; }

public String getJavaInitializationString() {
	return "new com.klg.jclass.chart.beans.PolarRadarWrapper(" + wrap + ")";
}

public boolean isPaintable() {
	return true;
}

public void paintValue(Graphics gc, Rectangle box) {
	String s = JCChartBeanBundle.string(JCChartBeanBundle.key163);
	if (s == null) {
		s = new String("");
	}
	FontMetrics fm = gc.getFontMetrics();
	int vpad = (box.height - fm.getAscent())/2;
	gc.drawString(s, box.x, box.y+box.height-vpad);
}

public Component getCustomEditor() {
	return this;
}

public boolean supportsCustomEditor() {
	return true;
}

public Dimension getPreferredSize() {
	return new Dimension(400, 300);
}

public void addPropertyChangeListener(PropertyChangeListener l) {
	support.addPropertyChangeListener(l);
}

public void removePropertyChangeListener(PropertyChangeListener l) {
	support.removePropertyChangeListener(l);
}

public void focusGained(FocusEvent ev) {}

public void focusLost(FocusEvent ev)
{
	Object src = ev.getSource();
	if (src instanceof JTextField) {
		setSelectedValue(src);
	}
}

// ItemListener method
public void itemStateChanged( ItemEvent ev ) {
	Object src = ev.getSource();
    if(src instanceof JCheckBox) {
		setSelectedValue(src);
	}
}

// ActionListener method
public void actionPerformed( ActionEvent ev ) {
	Object src = ev.getSource();
	if (src instanceof JTextField ||
		src instanceof JComboBox) {
		setSelectedValue(src);
	}
}

/* for testing purposes only
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }

	PolarRadarEditor polarRadarEditor = new PolarRadarEditor();
    polarRadarEditor.setBackground( java.awt.Color.lightGray );
	polarRadarEditor.setValue( "0,0,1,false,true" );
    frame.getContentPane().add( polarRadarEditor );
    frame.pack();
    java.awt.Dimension bigness = polarRadarEditor.getPreferredSize();
    frame.setSize( bigness.width + 100, bigness.height + 50 );
    frame.setVisible(true);
}
*/

}
