/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PolarRadarWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCPolarRadarChartFormat;
import com.klg.jclass.chart.JCChartArea;

/**
 * Wrapper class for Polar/Radar properties.
 * Used by the MultiChart bean.
 */
public class PolarRadarWrapper implements java.io.Serializable {

protected double originBase;
protected double yAxisAngle;
protected int angleUnit;
protected boolean halfRange;
protected boolean circularGrid;

/**
 * Default constructor.
 */
public PolarRadarWrapper() {
}

/**
 * Constructor.
 * @param originBase origin base angle.
 * @param yAxisAngle y axis angle.
 * @param angleUnitInteger JCChartUtil.DEGREES, JCChartUtil.RADIANS, or
 * JCChartUtil.GRADS.
 * @param halfRangeBoolean If true, the theta axis of a Polar chart will
 * show half-range values (0 to 180, 0 to -180).
 * @param circularGridBoolean If true, the y axis grid lines in a Radar
 * chart will be circular.  Otherwise, the grid will be web-shaped.
 */
public PolarRadarWrapper(double originBase, double yAxisAngle,
    					 int angleUnitInteger, boolean halfRangeBoolean,
    					 boolean circularGridBoolean) {
	this.originBase = originBase;
	this.yAxisAngle = yAxisAngle;
	this.angleUnit = angleUnitInteger;
	this.halfRange = halfRangeBoolean;
	this.circularGrid = circularGridBoolean;
}

/**
 * Stores Polar/Radar properties.
 * @param cf JCPolarRadarChartFormat object with get methods for Polar/Radar
 * properties.
 * @param ca JCChartArea object.
 */
public void setWrapperValues(JCPolarRadarChartFormat cf, JCChartArea ca) {
	originBase = cf.getOriginBase();
	yAxisAngle = cf.getYAxisAngle();
	angleUnit = ca.getAngleUnit();
	halfRange = cf.isHalfRange();
	circularGrid = cf.isRadarCircularGrid();
}

/**
 * Sets Polar/Radar properties using methods provided by the
 * JCPolarRadarChartFormat and JCChartArea objects.
 * @param JCPolarRadarChartFormat object with set methods for Polar/Radar
 * properties.
 * @param ca JCChartArea object.
 */
public void setPropertyValues(JCPolarRadarChartFormat cf, JCChartArea ca) {
	ca.setAngleUnit(angleUnit);  // Set this BEFORE originBase and yAxisAngle!
	cf.setOriginBase(originBase);
	cf.setYAxisAngle(yAxisAngle);
	cf.setHalfRange(halfRange);
	cf.setRadarCircularGrid(circularGrid);
}

/**
 * Convert properties to string for code generation.
 * @return string containing properties for code generation.
 */
public String toString() {
    String s = "";
	s += originBase;
	s += ",";
	s += yAxisAngle;
	s += ",";
	s += angleUnit;
	s += ",";
	s += halfRange;
	s += ",";
	s += circularGrid;

    return s;
}

public boolean equals(Object obj) {
	boolean eq = true;
	if (obj instanceof PolarRadarWrapper) {
		PolarRadarWrapper compareWrap = (PolarRadarWrapper) obj;
		if (compareWrap.originBase != this.originBase) {
			eq = false;
		}
		if (compareWrap.yAxisAngle != this.yAxisAngle) {
			eq = false;
		}
		if (compareWrap.angleUnit != this.angleUnit) {
			eq = false;
		}
		if (compareWrap.halfRange != this.halfRange) {
			eq = false;
		}
		if (compareWrap.circularGrid != this.circularGrid) {
			eq = false;
		}

	} else {
		eq = false;
	}
	return eq;
}

}
