/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: RadioAxisWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCAxis;

/**
 *  More specific (though still abstract) implementation of <code>RadioWrapper</code>
 *  for axis property wrappers.  All Axis*Wrapper classes subclass from this.
 */
public abstract class RadioAxisWrapper extends RadioWrapper
{

public RadioAxisWrapper() {
	// set the axis radio ids (x1, x2, etc.)
	radioIds = MultiChart.AXIS_RADIO_IDS;
}

/**
 * Caches properties from the specified axes.
 * @param axisList list of axes to cache properties from
 */
public void setWrapperValues(JCAxis[] axisList)
{
	if (axisList == null)
		return;

	setWrapperArraySize(axisList.length);
	for (int i = 0; i < axisList.length; i++) {
		setWrapperValue(i, axisList[i]);
	}
}

/**
 * Caches properties from the specified axis.  To be implemented by subclass
 * for specific properties.
 * @param index the index of the cached arrays that corresponds to this axis
 * @param axis the axis that contains the properties to be cached
 */
abstract public void setWrapperValue(int index, JCAxis axis);

/**
 * Sets the size of the internal arrays that store properties for multiple
 * axes.  To be implemented by subclass for specific properties.
 * @param size the size of the internal arrays
 */
abstract public void setWrapperArraySize(int size);

/**
 * Sets new property values for the specified axes.
 * @param axisList list of axes to set properties in (should match list
 * of axes passed to <code>setWrapperValues()</code>)
 */
public void setPropertyValues(JCAxis[] axisList)
{
	if (axisList == null)
		return;

	for (int i = 0; i < axisList.length; i++) {
		setPropertyValue(i, axisList[i]);
	}
}

/**
 * Sets property values for the specified axis from the stored values.
 * To be implemented by subclass for specific properties.
 * @param index the index where the values for this axis are stored
 * @param axis the axis that we need to set these values on
 */
abstract public void setPropertyValue(int index, JCAxis axis);

}
