/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: RadioButtonImageEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import java.beans.PropertyEditor;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Image;
import java.awt.GridLayout;
import java.awt.Component;
import java.awt.FontMetrics;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import com.klg.jclass.util.JCTypeConverter;

import javax.swing.JFrame;
import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;
import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.BorderFactory;




import java.util.Vector;


/**
 * Presents a group of radio buttons with images.
 */
public class RadioButtonImageEditor extends JPanel 
				implements PropertyEditor, ItemListener {

protected transient Image[] images;
protected String[] strings;
protected int[] values;
protected String prepend = "";
protected int rows;
protected int cols;
protected int numImages;
protected boolean upperCaseEnum = true;

private JPanel orientPanel = null;
private Object[] radioButtons = null;
private JRadioButton currentButton = null;

protected PropertyChangeSupport support = new PropertyChangeSupport(this);

public RadioButtonImageEditor() {
}

public void init( int numImages, String[] strings, int[] values, Image[] images,
				  int rows, int cols, String prepend ) {
	this.numImages = numImages;
	this.strings = strings;
	this.values = values;
	this.images = images;
	this.rows = rows;
	this.cols = cols;
	this.prepend = prepend;
	init();
}

protected void init() {
	setLayout( new GridLayout( 1, 1 ) );
	setBorder( BorderFactory.createEmptyBorder( 2, 4, 2, 2 ) );
	orientPanel = new JPanel();
	orientPanel.setBorder( BorderFactory.createTitledBorder( JCChartBeanBundle.string( JCChartBeanBundle.key174 ) ) );
	orientPanel.setLayout( new GridLayout( rows, cols ) );
	radioButtons = new Object[ numImages ];

	ButtonGroup orientBox = new ButtonGroup();

	for( int i = 0; i < numImages; i++ ) {
		JRadioButton radio = new JRadioButton( new ImageIcon( images[ i ] ) );

		radioButtons[ i ] = radio;
		orientBox.add( radio );
		orientPanel.add( radio );
		radio.addItemListener( this );
	}
	add( orientPanel );
}

public void setAsText(String text) throws IllegalArgumentException {
}

public String getAsText() {
	return "Click to edit ...";
}

public String[] getTags() { return null; }

public String getJavaInitializationString() {
	if (getValue() == null) return "NULL";
	if (upperCaseEnum) 
		return prepend + getValueAsText().toUpperCase();
	else
		return prepend + getValueAsText();
}

public boolean isPaintable() {
	return true;
}

public void paintValue(Graphics gc, Rectangle box) {
	String s = JCChartBeanBundle.string(JCChartBeanBundle.key163);
	if (s == null) {
		s = new String("");
	}
	FontMetrics fm = gc.getFontMetrics();
	int vpad = (box.height - fm.getAscent())/2;
	gc.drawString(s, box.x, box.y+box.height-vpad);
}

public Component getCustomEditor() {
	return this;
}

public boolean supportsCustomEditor() {
	return true;
}

public void addPropertyChangeListener(PropertyChangeListener l) {
	support.addPropertyChangeListener(l);
}

public void removePropertyChangeListener(PropertyChangeListener l) {
	support.removePropertyChangeListener(l);
}

// JCItemListener method
protected String getValueAsText() {
	return JCTypeConverter.fromEnum( getButtonValue( currentButton ), strings, values );
}

public Object getValue() {
	return new Integer( getButtonValue( currentButton ) );
}

public void setValue( Object o ) {
	setCurrentButton( getRadioButton( ((Number)o).intValue() ), true );
}

public void itemStateChanged( ItemEvent ev ) {
    if( ev.getSource() instanceof JRadioButton ) {
		JRadioButton radio = (JRadioButton)ev.getSource();

		if( radio == null || 
				ev.getStateChange() != ItemEvent.SELECTED ||
				radio == currentButton) 
			return;

		boolean value = radio.isSelected();

		if( value == false ) return;

		setCurrentButton( radio, true );
    }
}

protected void setCurrentButton( JRadioButton radio, boolean fire ) {
	if( radio != null ) {
		if( currentButton != null )
			currentButton.setBorderPainted( false );
		radio.setBorderPainted( true );
		currentButton = radio;
		currentButton.setSelected( true );
		if (fire)
			support.firePropertyChange( "", null, null );
	}
}

protected int getButtonValue( JRadioButton radio ) {
	int index = -1;

	for( int i = 0; i < radioButtons.length; i++ ) {
		if( radioButtons[ i ] == radio ) {
			index = i;
			break;
		}
	}
	if (index >= 0)
		return values[index];
	else
		return index;
}

protected JRadioButton getRadioButton( int v ) {
	JRadioButton radio = null;

	for( int i = 0; i < radioButtons.length; i++ ) {
		if( values[i] == v ) {
			radio = (JRadioButton)radioButtons[ i ];
			break;
		}
	}
	return radio;

}

/* For testing puposes only -- causes Class Circularity Errors in Bean Box
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }

	AxesOrientationEditor axesOrientationEditor = new AxesOrientationEditor();
    axesOrientationEditor.setBackground( java.awt.Color.lightGray );
    frame.getContentPane().add( axesOrientationEditor );
    frame.pack();
    java.awt.Dimension bigness = axesOrientationEditor.getPreferredSize();
    frame.setSize( bigness.width + 100, bigness.height + 50 );
    frame.setVisible(true);
}
*/

}

