/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: RadioDataWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.ChartDataView;

/**
 *  More specific (though still abstract) implementation of <code>RadioWrapper</code>
 *  for data view property wrappers.  All MultiData*Wrapper classes 
 *  subclass from this.
 */
public abstract class RadioDataWrapper extends RadioWrapper
{

public RadioDataWrapper() {
    // set the data view radio ids (data1, data2, etc.)
	radioIds = MultiChart.DATA_RADIO_IDS;
}

/**
 * Caches properties from the specified data views.
 * @param dataList list of data views to cache properties from
 */
public void setWrapperValues(ChartDataView[] dataList)
{
	if (dataList == null)
		return;

	setWrapperArraySize(dataList.length);
	for (int i = 0; i < dataList.length; i++) {
		setWrapperValue(i, dataList[i]);
	}
}

/**
 * Caches properties from the specified dataview.  To be implemented by subclass
 * for specific properties.
 * @param index the index of the cached arrays that corresponds to this dataview
 * @param dataview the dataview that contains the properties to be cached
 */
abstract public void setWrapperValue(int index, ChartDataView dataview);

/**
 * Sets the size of the internal arrays that store properties for multiple
 * data views.  To be implemented by subclass for specific properties.
 * @param size the size of the internal arrays
 */
abstract public void setWrapperArraySize(int size);

/**
 * Sets new property values for the specified data views.
 * @param dataList list of data views to set properties in (should match list
 * of data views passed to <code>setWrapperValues()</code>)
 */
public void setPropertyValues(ChartDataView[] dataList)
{
	if (dataList == null)
		return;

	for (int i = 0; i < dataList.length; i++) {
		setPropertyValue(i, dataList[i]);
	}
}

/**
 * Sets property values for the specified dataview from the stored values.
 * To be implemented by subclass for specific properties.
 * @param index the index where the values for this dataview are stored
 * @param dataview the dataview that we need to set these values on
 */
abstract public void setPropertyValue(int index, ChartDataView dataview);

}
