/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: RadioEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import java.beans.PropertyEditor;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;
import java.awt.Dimension;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.BorderLayout;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.FontMetrics;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import javax.swing.JFrame;
import javax.swing.JCheckBox;
import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;
import javax.swing.JPanel;
import javax.swing.BorderFactory;




import java.util.Vector;


/**
 * This editor provides the framework to edit properties across
 * multiple instances of an object (for example, properties across several axes)
 * by implementing a series of radio buttons, one for each instance,
 * that allows switching between the properties for each instance.
 */

abstract public class RadioEditor extends JPanel
						  implements PropertyEditor, ItemListener {

protected PropertyChangeSupport support = new PropertyChangeSupport(this);

protected ButtonGroup radioG;
protected JRadioButton[] radioB;


 

protected int radioSel = -1;
protected int numOfRadios = 0;
protected String[] radioNames = null;
protected String radioTitle = null;
protected String propertyTitle = null;

public RadioEditor(String title, String[] names) {
	numOfRadios = names.length;
	radioNames = new String[names.length];
	System.arraycopy(names, 0, radioNames, 0, names.length);

 
	radioTitle = title;
	init();
}

protected void init() {
	if (radioNames == null)
		return;

	setLayout(new BorderLayout());

    JPanel p1 = new JPanel();
    p1.setBorder(BorderFactory.createTitledBorder(radioTitle));
	p1.setLayout(new GridLayout(1,radioNames.length));

	radioG = new ButtonGroup();
	radioB = new JRadioButton[radioNames.length];
	for (int i = 0; i < radioNames.length; i++) {
		radioB[i] = new JRadioButton(radioNames[i]);
		radioG.add(radioB[i]);
		p1.add(radioB[i]);
		radioB[i].addItemListener(this);
	}
	radioB[0].setSelected(true);
	add("North", p1);

	JPanel p2 = makePropertyPanel();

	if (p2 != null)
		add("Center", p2);


 
}


 

/**
 * Creates a panel containing the GUI elements for the properties
 * associated with the radio buttons.
 * @return the panel
 */
abstract protected JPanel makePropertyPanel();

Object valueObj = null;

public void setValue(Object value) {

	valueObj = value;
}

/**
 * Attempts to switch to new radio button.
 * @param sel the radio button to switch to
 */
protected void switchItem(JRadioButton sel)
{
	if (radioB == null)
		return;

	for (int i = 0; i < radioB.length; i++) {
		if (sel == radioB[i]) {
			radioSel = i;
			switchValues(radioSel);
			break;
		}
	}
}

/**
 * Fills the property sheet with values associated with the newly selected
 * radio button.
 * @param sel the index of the newly selected radio button
 */
abstract protected void switchValues(int sel);

public Object getValue() {
	return valueObj;
}

public void setAsText(String text) throws IllegalArgumentException {
}

public String getAsText() {
	return "Click to edit...";
}

public String[] getTags() { return null; }

public String getJavaInitializationString() {
	return "\"" + valueObj + "\"";
}

public Dimension getPreferredSize() {
    Dimension dim = new Dimension(200, 0);

 
    return dim;
}

public boolean isPaintable() {
	return true;
}

public void paintValue(Graphics gc, Rectangle box) {
	String s = JCChartBeanBundle.string(JCChartBeanBundle.key163);
	if (s == null) {
		s = new String("");
	}
	FontMetrics fm = gc.getFontMetrics();
	int vpad = (box.height - fm.getAscent())/2;
	gc.drawString(s, box.x, box.y+box.height-vpad);
}

public Component getCustomEditor() {
	return this;
}

public boolean supportsCustomEditor() {
	return true;
}

public void addPropertyChangeListener(PropertyChangeListener l) {
	support.addPropertyChangeListener(l);
}

public void removePropertyChangeListener(PropertyChangeListener l) {
	support.removePropertyChangeListener(l);
}

// ActionListener method
public void itemStateChanged( ItemEvent ev ) {

	Object src;
    if( (src = ev.getSource()) instanceof JRadioButton ) {
		JRadioButton thisRadio = (JRadioButton) src;

		if (ev.getStateChange() == ItemEvent.SELECTED) {

 

			switchItem(thisRadio);
		}
	}
}

/**
 * Sets a int is-def'ed property given a String containing the
 * value to set.  If the String is empty, the is-default element of the
 * property is marked <code>true</code>.  If the String is not empty and is not equal
 * to the current value, the value element of the property is marked with
 * the new value and the id-default element is marked as <code>false</code>.
 * @param sval String containing value to set
 * @param propArray array containing int values for this property
 * @param propIsDefArray array containing is-default values for this property
 * @param idx index in array that indicated which property is being set
 */
public void setIntIsDefProperty(String sval, int propArray[],
                            boolean propIsDefArray[], int idx)
{
    String s = checkForEmptyString(sval);
    if (s == null) {
        if (!propIsDefArray[idx]) {
            propIsDefArray[idx] = true;
			propArray[idx] = Integer.MAX_VALUE;
            support.firePropertyChange("", null, null);
        }
    } else {
        int val = Integer.valueOf(s).intValue();
        if (val != propArray[idx]) {
            propArray[idx] = val;
            propIsDefArray[idx] = false;
            support.firePropertyChange("", null, null);
        }
    }
}

/**
 * Sets an int property given a String containing the value to set.
 * If the String is empty, the property is set to a default of 0.
 * If the String is not empty and is not equal to the current value,
 * the value  element of the property is marked with the new value.
 * @param sval String containing value to set
 * @param propArray array containing int values for this property
 * @param idx index in array that indicated which property is being set
 */
public void setIntProperty(String sval, int propArray[], int idx)
{
	setIntProperty(sval, propArray, idx, 0);
}

/**
 * Sets an int property given a String containing the value to set.
 * If the String is empty, the property set to the provided default.
 * If the String is not empty and is not equal to the current value,
 * the value element of the property is marked with the new value.
 * @param sval String containing value to set
 * @param propArray array containing int values for this property
 * @param idx index in array that indicated which property is being set
 * @param def the default value to use if the String is empty
 */
public void setIntProperty(String sval, int propArray[], int idx, int def)
{
    String s = checkForEmptyString(sval);
	int val;
    if (s != null) {
        val = Integer.valueOf(s).intValue();
	} else {
		val = def;
	}
    if (val != propArray[idx]) {
        propArray[idx] = val;
        support.firePropertyChange("", null, null);
    }
}

/**
 * Sets a double is-def'ed property given a String containing the
 * value to set.  If the String is empty, the is-default element of the
 * property is marked <code>true</code>.  If the String is not empty and is not equal
 * to the current value, the value element of the property is marked with
 * the new value and the id-default element is marked as <code>false</code>.
 * @param sval String containing value to set
 * @param propArray array containing double values for this property
 * @param propIsDefArray array containing is-default values for this property
 * @param idx index in array that indicated which property is being set
 */
public void setDoubleIsDefProperty(String sval, double propArray[],
                            boolean propIsDefArray[], int idx)
{
    String s = checkForEmptyString(sval);
    if (s == null) {
        if (!propIsDefArray[idx]) {
            propIsDefArray[idx] = true;
			propArray[idx] = Double.MAX_VALUE;
            support.firePropertyChange("", null, null);
        }
    } else {
        double val = Double.valueOf(s).doubleValue();
        if (val != propArray[idx]) {
            propArray[idx] = val;
            propIsDefArray[idx] = false;
            support.firePropertyChange("", null, null);
        }
    }
}

/**
 * Sets a double property given a String containing the value to set.
 * If the String is empty, the property is set to a default of 0.0.
 * If the String is not empty and is not equal to the current value,
 * the value element of the property is marked with the new value.
 * @param sval String containing value to set
 * @param propArray array containing double values for this property
 * @param idx index in array that indicated which property is being set
 */
public void setDoubleProperty(String sval, double propArray[], int idx)
{
	setDoubleProperty(sval, propArray, idx, 0.0);
}

/**
 * Sets a double property given a String containing the value to set.
 * If the String is empty, the property is set to the provided default.
 * If the String is not empty and is not equal to the current value,
 * the value  element of the property is marked with the new value.
 * @param sval String containing value to set
 * @param propArray array containing double values for this property
 * @param idx index in array that indicated which property is being set
 * @param def default value to use if String is empty
 */
public void setDoubleProperty(String sval, double propArray[], int idx,
		double def)
{
    String s = checkForEmptyString(sval);
	double val;
    if (s != null) {
        val = Double.valueOf(s).doubleValue();
	} else {
		val = def;
	}
    if (val != propArray[idx]) {
        propArray[idx] = val;
        support.firePropertyChange("", null, null);
    }
}

/**
 * Sets a String is-def'ed property given a String containing the
 * value to set.  If the String is empty, the is-default element of the
 * property is marked <code>true</code>.  If the String is not empty and is not equal
 * to the current value, the value element of the property is marked with
 * the new value and the id-default element is marked as <code>false</code>.
 * @param sval String containing value to set
 * @param propArray array containing String values for this property
 * @param propIsDefArray array containing is-default values for this property
 * @param idx index in array that indicated which property is being set
 */
public void setStringIsDefProperty(String sval, String propArray[],
                            boolean propIsDefArray[], int idx)
{
    String s = checkForEmptyString(sval);
    if (s == null) {
        if (!propIsDefArray[idx]) {
            propIsDefArray[idx] = true;
			propArray[idx] = "";
            support.firePropertyChange("", null, null);
        }
    } else {
        if (!s.equals(propArray[idx])) {
            propArray[idx] = s;
            propIsDefArray[idx] = false;
            support.firePropertyChange("", null, null);
        }
    }
}

/**
 * Check for empty string.
 * @param str string to check.
 * @return Null if empty.  Otherwise, trimmed string.
 */
public String checkForEmptyString(String str)
{
    String s = (str == null) ? null : str.trim();
    if (s == null || s.length() == 0)
        return null;
    else
        return s;
}

}
