/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: RadioSeries.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import java.util.Hashtable;
import java.util.Enumeration;

import com.klg.jclass.util.JCStringTokenizer;

/** 
 * Encapsulates a series String of the format
 *   "<code>(radioId1 value1)(radioId2 value2) ...</code>" and parses it to allow
 *   for extraction of each value associated with a radioId.
 */
public class RadioSeries extends Hashtable {

/**
 * Parses the provided series String for later extraction.
 * @param series series String of the format:<br>
 *               "<code>(radioId1 value1)(radioId2 value2) ...</code>"
 */
public RadioSeries(String series) {
	parse(series);
}

/**
 * Parses the provided series String for later extraction.
 * @param s series String of the format:<br>
 *               "<code>(radioId1 value1)(radioId2 value2) ...</code>"
 */
public void parse(String s) {
	clear();
	if (s == null) return;
	String[] tokens = JCStringTokenizer.parse(s, '(');
	if (tokens == null) return;
	int p;
	for (int i = 0; i < tokens.length; i++) {
		String token = tokens[i];
		if (token == null) continue;
		if ((p = token.indexOf(')')) == -1) continue;
		token = token.substring(0, p);
		token = token.trim();
		if (token.length() == 0) continue;
		for (p = 0; 
			 p < token.length() && !Character.isWhitespace(token.charAt(p)); 
			 p++);
		if (p >= token.length()) continue;
		String axisValue = token.substring(0, p);
		String value = token.substring(p+1);
		if (axisValue == null || value == null) return;
		value = value.trim();
		put(axisValue, value);
	}
}

/**
 * Converts this series back to a String representation.
 * @return a series String of the format :
 *               "<code>(radioId1 value1)(radioId2 value2) ...</code>"
 */
public String toString() {
	int max = size() - 1;
	StringBuffer buf = new StringBuffer();
	Enumeration k = keys();
	Enumeration e = elements();

	for (int i = 0; i <= max; i++) {
		buf.append("(");
	    String s1 = k.nextElement().toString();
	    String s2 = e.nextElement().toString();
	    buf.append(s1 + " " + s2 + ")");
	}
	return buf.toString();
}

/**
 *  Gets the double value associated with the given key.
 *  @param key the radioId associated with the value
 *  @return the assocatied value as a double
 */
public double getAsDouble(String key) {
	Object value = get(key);
	// ATTN GV Questionable
	if (value == null) return Double.MAX_VALUE;
	return Double.valueOf(value.toString()).doubleValue();
}

/**
 *  Gets the integer value associated with the given key.
 *  @param key the radioId associated with the value
 *  @return the assocatied value as an integer
 */
public int getAsInteger(String key) {
	Object value = get(key);
	// ATTN GV Questionable
	if (value == null) return Integer.MAX_VALUE;
	return Integer.valueOf(value.toString()).intValue();
}

/**
 *  Gets the boolean value associated with the given key.
 *  @param key the radioId associated with the value
 *  @return the assocatied value as a boolean
 */
public boolean getAsBoolean(String key) {
	Object value = get(key);
	// ATTN GV Questionable
	if (value == null) return false;
	return Boolean.valueOf(value.toString()).booleanValue();
}

/**
 *  Gets the String value associated with the given key.
 *  @param key the radioId associated with the value
 *  @return the assocatied value as a String
 */
public String getAsString(String key) {
	Object value = get(key);
	// ATTN GV Questionable
	if (value == null) return "";
	return value.toString();
}

/* for testing puposes only
public static void main(String args[]) {
	RadioSeries ax = new RadioSeries("(x1 1)(x2 3.2) ((( () ())( (y3)(y3) (y3) (y3  ) (  y3  )(  y3)(x4   4)");
	Object foo = ax.get("x2");
	System.out.println(foo);
	System.out.println(ax);
}
*/

}


