/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: RadioWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import java.awt.Color;
import java.awt.Font;
import java.util.Date;

import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.swing.JCSwingTypeConverter;

/**
 * Abstract base class for wrappers dealing with multiple instances of 
 * the same properties.  In the related editors, these instances are
 * often represented by a series of radio buttons, hence the name.  
 * <p>
 * In the subclass wrappers (for example, for axes or dataviews), the
 * wrappers store property values for each axis or dataview in arrays.
 * This class contains many methods for converting from series Strings to
 * arrays of these datatypes and vice versa.
 */
public abstract class RadioWrapper 
implements java.io.Serializable 
{

// Array of Ids to represent each instance of data.  Used in code generation
// and in parsing series strings.
protected String[] radioIds = null;

public RadioWrapper() {
}

/**
 * Fills the array of double values from the provided <code>RadioSeries</code>
 * object.
 * @param valArray array of double values
 * @param series <code>RadioSeries</code> object derived from String representation
 *               of values
 */
public void setWrapperDoubleValues(double[] valArray, RadioSeries series)
{
    for (int i = 0; i < valArray.length; i++) {
        valArray[i] =
        	series.getAsDouble(radioIds[i]);
    }
}

/**
 * Fills the array of double values and matching array of boolean isdef 
 * values from the provided <code>RadioSeries</code> object.
 * @param valArray array of double values
 * @param valIsDefArray matching array of boolean isdef values -- <code>true</code>
 *         means to calculate a default rather than use the set value
 * @param series <code>RadioSeries</code> object derived from String representation
 *               of values
 */
public void setWrapperDoubleIsDefValues(
	double[] valArray, boolean[] valIsDefArray, RadioSeries series)
{
    for (int i = 0; i < valArray.length; i++) {
        double val  = series.getAsDouble(radioIds[i]);
        if (val != Double.MAX_VALUE) {
            valArray[i] = val;
            valIsDefArray[i] = false;
        } else {
            valIsDefArray[i] = true;
        }
    }
}

/**
 * Fills the array of integer values from the provided <code>RadioSeries</code>
 * object.
 * @param valArray array of integer values
 * @param series <code>RadioSeries</code> object derived from String representation
 *               of values
 */
public void setWrapperIntValues(int[] valArray, RadioSeries series)
{
    for (int i = 0; i < valArray.length; i++) {
        valArray[i] =
        	series.getAsInteger(radioIds[i]);
    }
}

/**
 * Fills the array of integer values and matching array of boolean isdef 
 * values from the provided <code>RadioSeries</code> object.
 * @param valArray array of integer values
 * @param valIsDefArray matching array of boolean isdef values -- <code>true</code>
 *         means to calculate a default rather than use the set value
 * @param series <code>RadioSeries</code> object derived from String representation
 *               of values
 */
public void setWrapperIntIsDefValues(
	int[] valArray, boolean[] valIsDefArray, RadioSeries series)
{
    for (int i = 0; i < valArray.length; i++) {
        int val  = series.getAsInteger(radioIds[i]);
        if (val != Integer.MAX_VALUE) {
            valArray[i] = val;
            valIsDefArray[i] = false;
        } else {
            valIsDefArray[i] = true;
        }
    }
}

/**
 * Fills the array of boolean values from the provided <code>RadioSeries</code>
 * object.
 * @param valArray array of boolean values
 * @param series <code>RadioSeries</code> object derived from String representation
 *               of values
 */
public void setWrapperBooleanValues(boolean[] valArray, RadioSeries series)
{
    for (int i = 0; i < valArray.length; i++) {
        valArray[i] =
        	series.getAsBoolean(radioIds[i]);
    }
}

/**
 * Fills the array of String values from the provided <code>RadioSeries</code>
 * object.
 * @param valArray array of String values
 * @param series <code>RadioSeries</Code> object derived from String representation
 *               of values
 */
public void setWrapperStringValues(String[] valArray, RadioSeries series)
{
    for (int i = 0; i < valArray.length; i++) {
		valArray[i] = 
			JCTypeConverter.toNewLine(series.getAsString(radioIds[i]));
    }
}

/**
 * Fills the array of String values and matching array of boolean isdef 
 * values from the provided <code>RadioSeries</code> object.
 * @param valArray array of String values
 * @param valIsDefArray matching array of boolean isdef values -- <code>true</code>
 *         means to calculate a default rather than use the set value
 * @param series <code>RadioSeries</code> object derived from String representation
 *               of values
 */
public void setWrapperStringIsDefValues(String[] valArray, 
						boolean[] valIsDefArray, RadioSeries series)
{
    for (int i = 0; i < valArray.length; i++) {
        String val  = series.getAsString(radioIds[i]);
        if (val.length() > 0) {
            valArray[i] = val;
            valIsDefArray[i] = false;
        } else {
            valIsDefArray[i] = true;
        }
    }
}

/**
 * Fills the array of enum integer values from the provided <code>RadioSeries</code>
 * object containing enum Strings.
 * @param valArray array of enum integer values
 * @param series <code>RadioSeries</code> object derived from String representation
 *               of values
 * @param enumStrings array of enum String values for conversion
 * @param enumVals    matching array of enum integer values for conversion
 * @param type        enum name used for error messages
 * @param def         default value to use if enum String can not be converted
 */
public void setWrapperEnumValues(int[] valArray, RadioSeries series,
	String[] enumStrings, int[] enumVals, String type, int def)
{
    for (int i = 0; i < valArray.length; i++) {
        valArray[i] = JCTypeConverter.toEnum(
       		series.getAsString(radioIds[i]), type, enumStrings, enumVals, def);
    }
}

/**
 * Fills the array of enum long values from the provided <code>RadioSeries</code>
 * object containing enum Strings.
 * @param valArray array of enum long values
 * @param series <code>RadioSeries</code> object derived from String representation
 *               of values
 * @param enumStrings array of enum String values for conversion
 * @param enumVals    matching array of enum long values for conversion
 * @param type        enum name used for error messages
 * @param def         default value to use if enum String can not be converted
 */
public void setWrapperEnumValues(long[] valArray, RadioSeries series,
	String[] enumStrings, long[] enumVals, String type, long def)
{
    for (int i = 0; i < valArray.length; i++) {
        valArray[i] = JCTypeConverter.toEnum(
       		series.getAsString(radioIds[i]), type, enumStrings, enumVals, def);
    }
}

/**
 * Fills the array of enum integer values and the matching array of
 * boolean isdef values from the provided <code>RadioSeries</code>
 * object containing enum Strings.
 * @param valArray array of enum integer values
 * @param valIsDefArray matching array of boolean isdef values -- <code>true</code>
 *         means to calculate a default rather than use the set value
 * @param series <code>RadioSeries</code> object derived from String representation
 *               of values
 * @param enumStrings array of enum String values for conversion
 * @param enumVals    matching array of enum integer values for conversion
 * @param type        enum name used for error messages
 * @param def         default value to use if enum String can not be converted
 */
public void setWrapperEnumIsDefValues(int[] valArray, boolean[] valIsDefArray,
	RadioSeries series,
	String[] enumStrings, int[] enumVals, String type, int def)
{
    for (int i = 0; i < valArray.length; i++) {
        String val  = series.getAsString(radioIds[i]);
        if (val.length() > 0 ) {
        	valArray[i] = JCTypeConverter.toEnum(
       			series.getAsString(radioIds[i]), type, 
				enumStrings, enumVals, -1);
            valIsDefArray[i] = false;
        } else {
            valIsDefArray[i] = true;
        }
    }
}
/**
 * Fills the array of Font values from the provided <code>RadioSeries</code>
 * object.
 * @param valArray array of Font values
 * @param series <code>RadioSeries</code> object derived from String representation
 *               of values
 */
public void setWrapperFontValues(Font[] valArray, RadioSeries series)
{
    for (int i = 0; i < valArray.length; i++) {
        valArray[i] = JCSwingTypeConverter.toFont(series.getAsString(radioIds[i]));
    }
}

/**
 * Fills the array of Color values from the provided <code>RadioSeries</code>
 * object.
 * @param valArray array of Color values
 * @param series <code>RadioSeries</code> object derived from String representation
 *               of values
 */
public void setWrapperColorValues(Color[] valArray, RadioSeries series)
{
    for (int i = 0; i < valArray.length; i++) {
        valArray[i] = JCSwingTypeConverter.toColor(series.getAsString(radioIds[i]));
    }
}

/**
 * Fills the array of Date values from the provided <code>RadioSeries</code>
 * object.
 * @param valArray array of Date values
 * @param series <code>RadioSeries</code> object derived from String representation
 *               of values
 */
public void setWrapperDateValues(Date[] valArray, RadioSeries series)
{
    for (int i = 0; i < valArray.length; i++) {
        valArray[i] = JCTypeConverter.toDate(series.getAsString(radioIds[i]),
																new Date());
    }
}

/**
 * Fills the array of radio id integer values from the provided <code>RadioSeries</code>
 * object containing String representations of the radio IDs.  Uses the
 * current set <code>radioIds</code> String as the default list of radio IDs.
 * @param valArray array of double values
 * @param series <code>RadioSeries</code> object derived from String representation
 *               of values
 */
public void setWrapperRadioIDValues(int[] valArray, RadioSeries series)
{
	setWrapperRadioIDValues(valArray, series, radioIds);
}

/**
 * Fills the array of radio ID integer values from the provided <code>RadioSeries</code>
 * object containing String representations of the radio IDs.
 * @param valArray array of double values
 * @param series RadioSeries object derived from String representation
 *               of values
 * @param radioIds the array of radioIds to use for conversion.
 */
public void setWrapperRadioIDValues(int[] valArray, RadioSeries series,
									String[] radioIds)
{
    for (int i = 0; i < valArray.length; i++) {
		String val = series.getAsString(this.radioIds[i]);
	    valArray[i] = -1;
    	for (int j = 0; j < radioIds.length; j++) {
        	if (val.equals(radioIds[j])) {
            	valArray[i] = j;
        	}
    	}
	}
}

/* ---- */

/** 
 *  Converts array of double values to a series String of the format
 *  "<code>(radioId1 value1)(radioId2 value2) ...</code>".
 *  @param valArray array of double values to convert
 *  @return series String
 */
public String doubleValuesToString(double[] valArray)
{
	String s = "\"";
    for (int i = 0; i < valArray.length; i++) {
		s += seriesStringPrefix(radioIds[i]);
		s += valArray[i];
		s += seriesStringSuffix();
    }
	s += "\"";
	return s;
}

/** 
 *  Converts array of double values to a series String of the format
 *  "<code>(radioId1 value1)(radioId2 value2) ...</code>".  If the value's matching
 *  IsDef value is <code>true</code>, no String will be created for that instance.
 *  @param valArray array of double values to convert
 *  @param valIsDefArray matching array of boolean IsDef values
 *  @return series String
 */
public String doubleIsDefValuesToString(
	double[] valArray, boolean[] valIsDefArray)
{
	String s = "\"";
    for (int i = 0; i < valArray.length; i++) {
		if (!valIsDefArray[i]) {
			s += seriesStringPrefix(radioIds[i]);
			s += valArray[i];
			s += seriesStringSuffix();
		}
    }
	s += "\"";
	return s;
}

/** 
 *  Converts array of integer values to a series String of the format
 *  "<code>(radioId1 value1)(radioId2 value2) ...</code>".
 *  @param valArray array of integer values to convert
 *  @return series String
 */
public String intValuesToString(int[] valArray)
{
	String s = "\"";
    for (int i = 0; i < valArray.length; i++) {
		s += seriesStringPrefix(radioIds[i]);
		s += valArray[i];
		s += seriesStringSuffix();
    }
	s += "\"";
	return s;
}

/** 
 *  Converts array of integer values to a series String of the format
 *  "<code>(radioId1 value1)(radioId2 value2) ...</code>".  If the value's matching
 *  IsDef value is <code>true</code>, no String will be created for that instance.
 *  @param valArray array of integer values to convert
 *  @param valIsDefArray matching array of boolean IsDef values
 *  @return series String
 */
public String intIsDefValuesToString(
	int[] valArray, boolean[] valIsDefArray)
{
	String s = "\"";
    for (int i = 0; i < valArray.length; i++) {
		if (!valIsDefArray[i]) {
			s += seriesStringPrefix(radioIds[i]);
			s += valArray[i];
			s += seriesStringSuffix();
		}
    }
	s += "\"";
	return s;
}

/** 
 *  Converts array of boolean values to a series String of the format
 *  "<code>(radioId1 value1)(radioId2 value2) ...</code>".
 *  @param valArray array of boolean values to convert
 *  @return series String
 */
public String booleanValuesToString(boolean[] valArray)
{
	String s = "\"";
    for (int i = 0; i < valArray.length; i++) {
		s += seriesStringPrefix(radioIds[i]);
		s += valArray[i];
		s += seriesStringSuffix();
	}
	s += "\"";
	return s;
}

/** 
 *  Converts array of String values to a series String of the format
 *  "<code>(radioId1 value1)(radioId2 value2) ...</code>".
 *  @param valArray array of String values to convert
 *  @return series String
 */
public String stringValuesToString(String[] valArray)
{
	String s = "\"";
    for (int i = 0; i < valArray.length; i++) {
		s += seriesStringPrefix(radioIds[i]);
		if (valArray[i] != null) {
			s += JCTypeConverter.fromNewLine(valArray[i]);
		}
		s += seriesStringSuffix();
	}
	s += "\"";
	return s;
}

/** 
 *  Converts array of String values to a series String of the format
 *  "<code>(radioId1 value1)(radioId2 value2) ...</code>".  Use when expecting large
 *  Strings to result.  This method breaks up the resulting String into
 *  different lines at any occurrence of a <i>'\n'</i> character.  (It leaves
 *  the <i>'\n'</i> characters in the String, though, as it should.)
 *  @param valArray array of double values to convert
 *  @return series String
 */
public String stringValuesToBigString(String[] valArray)
{
	String s = "\"";
    for (int i = 0; i < valArray.length; i++) {
		s += seriesStringPrefix(radioIds[i]);
		if (valArray[i] != null) {
			String convString = JCTypeConverter.fromNewLine(valArray[i]);
			int startPos = 0;
			int endPos;
			while ((endPos = convString.indexOf("\\n")) >= 0) {
				s += convString.substring(startPos, endPos + 2);
				if (endPos + 2 < convString.length()) {
					s += "\" + \n\r\"";
					convString = convString.substring(endPos + 2);
				} else {
					convString = "";
				}
			}
			// Now handle remainder of convString
			if (convString.length() > 0) {
			    s += convString;
			}
		}
		s += seriesStringSuffix();
	}
	s += "\"";
	return s;
}

/** 
 *  Converts array of String values to a series String of the format
 *  "<code>(radioId1 value1)(radioId2 value2) ...</code>".  If the value's matching
 *  IsDef value is <code>true</code>, no String will be created for that instance.
 *  @param valArray array of String values to convert
 *  @param valIsDefArray matching array of boolean IsDef values
 *  @return series string
 */
public String stringIsDefValuesToString(String[] valArray, 
										boolean[] valIsDefArray)
{
	String s = "\"";
    for (int i = 0; i < valArray.length; i++) {
		if (!valIsDefArray[i]) {
			s += seriesStringPrefix(radioIds[i]);
			s += valArray[i];
			s += seriesStringSuffix();
		}
	}
	s += "\"";
	return s;
}

/** 
 *  Converts array of enum integer values to a series String of the format
 *  "<code>(radioId1 value1)(radioId2 value2) ...</code>".
 *  @param valArray array of enum integer values to convert
 *  @param enumStrings array of enum String names
 *  @param enumVals matching array of enum String values
 *  @return series String
 */
public String enumValuesToString(int[] valArray,
	String[] enumStrings, int[] enumVals)
{
	String s = "\"";
    for (int i = 0; i < valArray.length; i++) {
		s += seriesStringPrefix(radioIds[i]);
		s += JCTypeConverter.fromEnum(valArray[i], enumStrings, enumVals);
		s += seriesStringSuffix();
	}
	s += "\"";
	return s;
}

/** 
 *  Converts array of enum long values to a series String of the format
 *  "<code>(radioId1 value1)(radioId2 value2) ...</code>".
 *  @param valArray array of enum long values to convert
 *  @param enumStrings array of enum String names
 *  @param enumVals matching array of enum String values
 *  @return series String
 */
public String enumValuesToString(long[] valArray,
	String[] enumStrings, long[] enumVals)
{
	String s = "\"";
    for (int i = 0; i < valArray.length; i++) {
		s += seriesStringPrefix(radioIds[i]);
		s += JCTypeConverter.fromEnum(valArray[i], enumStrings, enumVals);
		s += seriesStringSuffix();
	}
	s += "\"";
	return s;
}

/** 
 *  Converts array of enum intger values to a series String of the format
 *  "<code>(radioId1 value1)(radioId2 value2) ...</code>".  If the value's matching
 *  IsDef value is <code>true</code>, no String will be created for that instance.
 *  @param valArray array of enum integer values to convert
 *  @param valIsDefArray matching array of boolean IsDef values
 *  @param enumStrings array of enum String names
 *  @param enumVals matching array of enum String values
 *  @return series String
 */
public String enumIsDefValuesToString(int[] valArray, boolean[] valIsDefArray,
	String[] enumStrings, int[] enumVals)
{
	String s = "\"";
    for (int i = 0; i < valArray.length; i++) {
		if (!valIsDefArray[i]) {
			s += seriesStringPrefix(radioIds[i]);
			s += JCTypeConverter.fromEnum(valArray[i], enumStrings, enumVals);
			s += seriesStringSuffix();
		}
	}
	s += "\"";
	return s;
}

/** 
 *  Converts array of Font values to a series String of the format
 *  "<code>(radioId1 value1)(radioId2 value2) ...</code>".
 *  @param valArray array of Font values to convert
 *  @return series String
 */
public String fontValuesToString(Font[] valArray)
{
	String s = "\"";
    for (int i = 0; i < valArray.length; i++) {
		s += seriesStringPrefix(radioIds[i]);
		s += JCSwingTypeConverter.fromFont(valArray[i]);
		s += seriesStringSuffix();
	}
	s += "\"";
	return s;
}

/** 
 *  Converts array of Color values to a series String of the format
 *  "<code>(radioId1 value1)(radioId2 value2) ...</code>".
 *  @param valArray array of Color values to convert
 *  @return series String
 */
public String colorValuesToString(Color[] valArray)
{
	String s = "\"";
    for (int i = 0; i < valArray.length; i++) {
		s += seriesStringPrefix(radioIds[i]);
		s += JCSwingTypeConverter.fromColor(valArray[i]);
		s += seriesStringSuffix();
	}
	s += "\"";
	return s;
}

/** 
 *  Converts array of Date values to a series String of the format
 *  "<code>(radioId1 value1)(radioId2 value2) ...</code>".
 *  @param valArray array of Date values to convert
 *  @return series String
 */
public String dateValuesToString(Date[] valArray)
{
	String s = "\"";
    for (int i = 0; i < valArray.length; i++) {
		s += seriesStringPrefix(radioIds[i]);
		s += valArray[i];
		s += seriesStringSuffix();
	}
	s += "\"";
	return s;
}

/** 
 *  Converts array of radioId values to a series String of the format
 *  "<code>(radioId1 value1)(radioId2 value2) ...</code>".  This uses the default
 *  radioIds Strings.
 *  @param valArray array of radioId values to convert
 *  @return series String
 */
public String radioIDValuesToString(int[] valArray)
{
	return radioIDValuesToString(valArray, radioIds);
}

/** 
 *  Converts array of radioId values to a series String of the format
 *  "<code>(radioId1 value1)(radioId2 value2) ...</code>".
 *  @param valArray array of radioId values to convert
 *  @param radioIds aray of radioIds to use for the conversion
 *  @return series String
 */
public String radioIDValuesToString(int[] valArray, String[] radioIds)
{
	String s = "\"";
    for (int i = 0; i < valArray.length; i++) {
		s += seriesStringPrefix(this.radioIds[i]);
		if (valArray[i] >= 0)
			s += radioIds[valArray[i]];
		s += seriesStringSuffix();
	}
	s += "\"";
	return s;
}

/* ------ */

/** 
 *  Contructs series String beginning using radioId.
 *  @param radioId radioId to use to build the beginining of the series String
 *  @return beginning of series String "(radioId"
 */
public String seriesStringPrefix(String radioId)
{
	return "(" + radioId + " ";
}

/**
 *  Constructs closing portion of series String.
 *  @return closing portion of series String : ")" .
 */
public String seriesStringSuffix()
{
	return ")";
}

}
