/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: SimpleChart.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import java.util.StringTokenizer;
import com.klg.jclass.util.JCStringTokenizer;

import com.klg.jclass.chart.ChartDataView;
import com.klg.jclass.chart.data.JCDefaultDataSource;
import com.klg.jclass.chart.data.JCURLDataSource;
import com.klg.jclass.chart.data.JCFileDataSource;
import com.klg.jclass.chart.data.JCStringDataSource;

/**
 * A derived class that enables JClass Chart to
 * be used at design-time with Bean-compliant Java IDEs (Integrated
 * Development Environments).
 * <P>This class differs from <code>JCChart</code> in the following ways:
 * <OL>
 * <LI><code>SimpleChart</code> exposes properties that are actually defined
 * in <code>JCChart</code>'s sub-objects so that IDEs can access them.
 * <LI><code>SimpleChart</code> duplicates enums that are actually defined
 * in <code>JCChart</code>'s sub-objects.
 * </OL>
 * <P>The following table lists the properties this class can set/get.
 * <P><STRONG>Properties</STRONG></P>
 * <P></P>
 * <TABLE BORDER=1 WIDTH=100%>
 * <TR>
 * <TH>Name</TH>
 * <TH>Description</TH></TR>
 * <TR>
 * <TD>AxisOrientation
 * </TD>
 * <TD>
 * The <code>AxisOrientation</code> property determines if the <i>x</i> and <i>y</i> axes
 * are inverted and reversed.
 * </TD></TR>
 * <TD>Background
 * </TD>
 * <TD>The <code>Background</code> property determines the background color used to draw
 * inside the chart region. <br><B>Note:</b> The <code>Background</code> property is inherited
 * from the parent <code>JComponent</code>.
 * </TD></TR>
 * <TR>
 * <TD>ChartType
 * </TD>
 * <TD>
 * The <code>ChartType</code> property determines the chart type of the
 * first set of data in the chart.
 * </TD></TR>
 * <TR>
 * <TD>Data
 * </TD>
 * <TD>
 * The <code>Data</code> property controls the file or URL used for the
 * first set of data in chart.
 * </TD></TR>
 * <TR>
 * <TD>Font
 * </TD>
 * <TD>The <code>Font</code> property determines what font is used to render text inside
 * the chart region. <br><B>Note:</b> The <code>Font</code> property is inherited from the
 * parent <code>JComponent</code>.
 * </TD></TR>
 * <TR>
 * <TD>FooterText
 * </TD>
 * <TD>
 * The <code>FooterText</code> property holds the text that is displayed
 * in the footer. The default value is " " (empty String).
 * </TD></TR>
 * <TR>
 * <TD>Foreground
 * </TD>
 * <TD>The <code>Foreground</code> property determines the foreground color used to draw
 * inside the chart region. <br><B>Note:</b> The <code>Foreground</code> property is inherited
 * from the parent <code>JComponent</code>.
 * </TD></TR>
 * <TR>
 * <TD>HeaderText
 * </TD>
 * <TD>
 * The <code>HeaderText</code> property holds the text that is displayed
 * in the header. The default value is " " (empty String).
 * </TD></TR>
 * <TR>
 * <TD>Legend
 * </TD>
 * <TD>
 * The <code>Legend</code> property controls the object that controls
 * the display of the legend. The default value is null.
 * </TD></TR>
 * <TR>
 * <TD>LegendAnchor
 * </TD>
 * <TD>
 * The <code>LegendAnchor</code> property determines the position of the
 * legend relative to the <code>ChartArea</code>.  Valid values include <code>NORTH</code>,
 * <code>SOUTH</code>, <code>EAST</code>, <code>WEST</code>, <code>NORTHWEST</code>,
 * <code>SOUTHWEST</code>, <code>NORTHEAST</code>, and <code>SOUTHEAST</code>. The default
 * value is <code>EAST</code>.
 * </TD></TR>
 * <TR>
 * <TD>LegendVisible
 * </TD>
 * <TD>
 * The <code>LegendVisible</code> property determines whether the
 * legend is visible. The default value is <code>false</code>.
 * </TD></TR>
 * <TR>
 * <TD>LegendOrientation
 * </TD>
 * <TD>
 * The <code>LegendOrientation</code> property determines how legend
 * information is laid out. Valid values include <code>VERTICAL</code> and
 * <code>HORIZONTAL</code>. The default value is <code>VERTICAL</code>.
 * </TD></TR>
 * <TR>
 * <TD>SwingDataModel
 * </TD><TD>
 * Sets the chart's data source to use a specified Swing <code>TableModel</code> object,
 * instead of the <TT>Data</TT> property.
 * </TD></TR>
 * <TR>
 * <TD>View3D
 * </TD>
 * <TD>
 * The <TT>View3D</TT> property combines the values of the <TT>Depth</TT>,
 * <TT>Elevation</TT>, and <TT>Rotation</TT> properties defined in <TT>JCChartArea</TT>.
 * <TT>Depth</TT> controls the apparent depth of a graph.
 * <TT>Elevation</TT> controls the distance above the <i>x</i> axis
 * for the 3D effect. <TT>Rotation</TT> controls the position of the
 * eye relative to the <i>y</i> axis for the 3D effect.  The default value is "0.0,0.0,0.0".
 * </TD></TR>
 * <TR>
 * <TD>Version
 * </TD>
 * <TD>
 * The <code>Version</code> property specifies the JClass Chart version number. This property
 * is read-only. The default value is calculated.
 * </TD></TR>
 * <TR>
 * <TD>XAxisAnnotationMethod
 * </TD>
 * <TD>
 * The <code>XAxisAnnotationMethod</code> property determines how axis annotations
 * are generated. Valid values include <code>VALUE</code> (annotation is generated
 * by JClass Chart, with possible callbacks to a label generator);
 * <code>VALUE_LABELS</code> (annotation is taken from a list of value
 * labels provided by the user -- a value label is a label that appears
 * at a particular axis value); <code>POINT_LABELS</code>
 * (annotation comes from the data source's point labels
 * that are associated with particular data points); and
 * <code>TIME_LABELS</code> (JClass Chart generates time/date labels based on the
 * <code>TimeUnit</code>, <code>TimeBase</code> and <code>TimeFormat</code> properties).
 * The default value is <code>VALUE</code>.
 * </TD></TR>
 * <TR>
 * <TD>XAxisGridVisible
 * </TD>
 * <TD>
 * The <code>XAxisGridVisible</code> property determines whether a grid is drawn
 * for the axis. The default value is <code>false</code>.
 * </TD></TR>
 * <TR>
 * <TD>XAxisLogarithmic
 * </TD>
 * <TD>
 * The <code>XAxisLogarithmic</code> property determines whether the
 * first <i>x</i> axis will be logarithmic (<code>true</code>) or linear (<code>false</code>).
 * The default value is <code>false</code>.
 * </TD></TR>
 * <TR>
 * <TD>XAxisVisible
 * </TD>
 * <TD>
 * The <code>XAxisVisible</code> property determines whether the
 * first <i>x</i> axis is currently visible. The default value is <code>true</code>.
 * </TD></TR>
 * <TR>
 * <TD>XAxisMinMax
 * </TD>
 * <TD>
 * The <code>XAxisMinMax</code> controls both the <code>XAxisMin</code> and
 * <code>XAxisMax</code> properties. The <code>XAxisMin</code> property controls the
 * minimum value shown on the axis. If a null String is used, JClass Chart will calculate
 * the axis min. The data min is determined by JClass Chart. The default value is calculated.
 * The <code>XAxisMax</code> property controls the maximum value shown on the axis.
 * If a null String is used, JClass Chart will calculate the axis max.
 * The data max is determined by JClass Chart. The default value is calculated.
 * </TD></TR>
 * <TR>
 * <TD>XAxisNumSpacing
 * </TD>
 * <TD>
 * The <code>XAxisNumSpacing</code> property controls the interval between
 * axis labels.
 * If a null String is used, JClass Chart will calculate the interval.
 * The default value is calculated.
 * </TD></TR>
 * <TR>
 * <TD>XAxisTitleText
 * </TD>
 * <TD>
 * The <code>XAxisTitleText</code> property specifies the text that
 * will appear as the <i>x</i> axis title. The default value is " " (empty String).
 * </TD></TR>
 * <TR>
 * <TD>YAxisAnnotationMethod
 * </TD>
 * <TD>
 * The <code>YAxisAnnotationMethod</Code> property determines how axis annotations
 * are generated. Valid values include <code>VALUE</code> (annotation is generated
 * by JClass Chart, with possible callbacks to a label generator);
 * <code>VALUE_LABELS</code> (annotation is taken from a list of value
 * labels provided by the user -- a value label is a label that appears
 * at a particular axis value); <code>POINT_LABELS</code> (annotation comes from the
 * data source's point labels that are associated with particular data points); and
 * <code>TIME_LABELS</code> (JClass Chart generates time/date labels based on the
 * <code>TimeUnit</code>, <code>TimeBase</code>, and <code>TimeFormat</code> properties).
 * The default value is <code>VALUE</code>.
 * </TD></TR>
 * <TR>
 * <TD>YAxisGridVisible
 * </TD>
 * <TD>
 * The <code>YAxisGridVisible</code> property determines whether a grid is drawn for the axis.
 * </TD></TR>
 * <TR>
 * <TD>YAxisLogarithmic
 * </TD>
 * <TD>
 * The <code>YAxisLogarithmic</code> property determines whether the
 * first <i>y</i> axis will be logarithmic (<code>true</code>) or linear (<code>false</code>).
 * The default value is <code>false</code>.
 * </TD></TR>
 * <TR>
 * <TD>YAxisVisible
 * </TD>
 * <TD>
 * The <code>YAxisVisible</code> property determines whether the
 * first <TT>y</TT> axis is currently visible. The default value is <code>true</code>.
 * </TD></TR>
 * <TR>
 * <TD>YAxisMinMax
 * </TD>
 * <TD>
 * The <code>YAxisMinMax</code> controls both the <code>YAxisMin</code> and
 * <code>YAxisMax</code> properties. The <code>YAxisMin</code> property controls the
 * minimum value shown on the axis. If a null String is used, JClass Chart will
 * calculate the axis min. The data min is determined by JClass Chart. The default
 * value is calculated. The <code>YAxisMax</code> property controls the maximum value
 * shown on the axis. If a null String is used, JClass Chart will calculate the axis max.
 * The data max is determined by JClass Chart. The default value is calculated.
 * </TD></TR>
 * <TR>
 * <TD>YAxisNumSpacing
 * </TD>
 * <TD>
 * The <code>YAxisNumSpacing</code> property controls the interval between
 * axis labels.
 * If a null String is used, JClass Chart will calculate the interval.
 * The default value is calculated.
 * </TD></TR>
 * <TR>
 * <TD>YAxisTitleText
 * </TD>
 * <TD>
 * The <code>YAxisTitleText</code> property specifies the text that
 * will appear as the <i>y</i> axis title. The default value is " " (empty String).
 * </TD></TR>
 * </TABLE>
 */
public class SimpleChart extends BaseChart {


/**
 * Basic constructor.  Sets data to default data.
 */
public SimpleChart() {
	super();
	setData( new JCDefaultDataSource().buildDataSourceString() );
}

/** String representation of data held by this chart. */
protected String data;

/**
 * Gets the value of the <code>data</code> property.
 * The <code>data</code> property controls the file or URL used for the
 * first set of data in the chart.
 * @return
 */
public String getData() {
	if (data == null)
		data = new String("");
	return data;
}

/**
 * Sets the value of the <code>data</code> property.
 * The <code>data</code> property controls the file or URL used for the
 * first set of data in the chart.
 * @param s
 */
public void setData(String s) {
	if (getDataView(0) == null) {
		setDataView(0, new ChartDataView());
	}
	if (s == null || s.equals("")) {
		setDataView(0, new ChartDataView());
	}
	else if (s.indexOf("http") != -1) {
		try {
			getDataView(0).setDataSource(new JCURLDataSource(s));
		}
		catch (Exception e) {
			getDataView(0).setDataSource(null);
		}
	}
	else {
        String testString = s.toLowerCase();

        // check if definitely trying to be a string data source
        int pos = -1;
        if ((pos = testString.indexOf("array")) >= 0) {
            if (testString.length() == pos + 5 ||
                !Character.isWhitespace(testString.charAt(pos + 5))) {
                pos = -1;
            }
        } else if ((pos = testString.indexOf("general")) >= 0) {
            if (testString.length() == pos + 7 ||
                !Character.isWhitespace(testString.charAt(pos + 7))) {
                pos = -1;
            }
        }
        // has "array" or "general" follwed by whitespace,
        // must be a string source
        if (pos != -1) {
            try {
                getDataView(0).setDataSource(new JCStringDataSource(s));
            }
            catch( Exception e2 ) {
                getDataView(0).setDataSource(null);
            }
        // not of right format, then must be a file name
        } else {
            try {
                getDataView(0).setDataSource(new JCFileDataSource(s));
            }
            catch( Exception e1 ) {
                getDataView(0).setDataSource(null);
            }
        }
	}
	data = s;
}


}
