/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TitleWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:49 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.SwingConstants;

import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.swing.JCSwingTypeConverter;

import java.awt.Font;

/**
 * Wrapper class for Title properties -- headers and footers.
 * Used by the MultiChart Bean.
 */
public class TitleWrapper 
implements java.io.Serializable
{

public Font    font;
public String  text;

// not to be set, just for reference
public Font default_font = new Font("helvetica", Font.PLAIN, 12);
public String  prefix;

/**
 * Default constructor.  Members will be empty.
 */
public TitleWrapper() {
	font = default_font;
	prefix = "";
}

/**
 * Constructor.  Fills members with parameter Strings.
 * @param fontString     font String.  Values should be of the format
 *                       "style-name-pointsize".
 * @param textString     text String
 */
public TitleWrapper(String fontString, String textString)
{
    font = JCSwingTypeConverter.toFont(fontString);
	text = new String(textString);

	prefix = "";
}

/**
 * Stores the properties from the specified title (for example, header, footer).
 * @param title the title object that contains the properties
 */
public void setWrapperValues(JComponent title)
{
	if (title instanceof JLabel) {
		JLabel label = (JLabel) title;
		text = label.getText();
	}

	if (text == null)
		text = new String("");

	font = title.getFont();
	if (font == null)
		font = default_font;
}

/**
 * Sets the properties on the specified title
 * (for example, header, footer) from values stored internally.
 * @param title the title object that we need to set these values on
 */
public void setPropertyValues(JComponent title)
{
	if (title instanceof JLabel) {
		JLabel label = (JLabel) title;
		label.setText(text);
	}
	if (!(font.equals(title.getFont()))) {
		title.setFont(font);
	}
}

/**
 * Converts properties to String for code generation.
 * @return String containing properties for code generation
 */
public String toString()
{
    String s = "";

    s += "\"" + JCSwingTypeConverter.fromFont(font) + "\"";
    s += ",";
    s += "\"" + text + "\"";

    return s;
}

public boolean equals(Object obj)
{
	boolean eq = true;
	if (obj instanceof TitleWrapper) {
		TitleWrapper compareAW = (TitleWrapper) obj;
		if (compareAW.font != null && this.font != null) {
			if (!compareAW.font.equals(this.font))
				eq = false;
		} else {
			if (compareAW.font != this.font)
				eq = false;
		}
		if (compareAW.text != null && this.text != null) {
			if (!compareAW.text.equals(this.text))
				eq = false;
		} else {
			if (compareAW.text != this.text)
				eq = false;
		}
	} else {
		eq = false;
	}
	return eq;
}
}
