/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TriggerWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:49 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.EventTrigger;
import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.util.JCTypeConverter;

import java.awt.event.InputEvent;

/**
 * Wrapper class for Trigger properties.
 * Used by the MultiChart Bean.
 */
public class TriggerWrapper 
implements java.io.Serializable
{

public int  norm_action;
public int  shift_action;
public int  ctrl_action;
public int  alt_action;
public int  meta_action;
public boolean  allow;

/**
 * Default constructor.  Members will be empty.
 */
/**
 * Constructor.  Allocates space for the arrays, but does not fill them.
 */
/**
 * Constructor. Allocates space for the arrays and fills them by parsing
 * series Strings of format "<code>(x1 value)(y1 value)(x2 value)(y2 value)</code>".
 */
public TriggerWrapper() {
}

/**
 * Constructor. Fills members with parameter Strings.
 * @param normString   String for normal trigger.  Possible values are
 * <code>None</code>, <code>Rotate</code>, <code>Zoom</code>, <code>Translate</code>,
 * <code>Edit</code>, <code>Pick</code>, <code>Depth</code>, or <code>Customize</code>.
 * @param shiftString  String for shift trigger.  Possible values are
 * <code>None</code>, <code>Rotate</code>, <code>Zoom</code>, <code>Translate</code>,
 * <code>Edit</code>, <code>Pick</code>, <code>Depth</code>, or <code>Customize</code>.
 * @param ctrlString   String for ctrl trigger.  Possible values are
 * <code>None</code>, <code>Rotate</code>, <code>Zoom</code>, <code>Translate</code>,
 * <code>Edit</code>, <code>Pick</code>, <code>Depth</code>, or <code>Customize</code>.
 * @param altString    String for alt trigger.  Possible values are
 * <code>None</code>, <code>Rotate</code>, <code>Zoom</code>, <code>Translate</code>,
 * <code>Edit</code>, <code>Pick</code>, <code>Depth</code>, or <code>Customize</code>.
 * @param metaString   String for meta trigger.  Possible values are
 * <code>None</code>, <code>Rotate</code>, <code>Zoom</code>, <code>Translate</code>,
 * <code>Edit</code>, <code>Pick</code>, <code>Depth</code>, or <code>Customize</code>.
 * @param allowBoolean value for allow customer changes property
 */
public TriggerWrapper(String normString, String shiftString, String ctrlString,
	String altString, String metaString, boolean allowBoolean)
{
    norm_action = JCTypeConverter.toEnum(normString, "Trigger",
        JCChartEnumMappings.trigger_strings, JCChartEnumMappings.trigger_values,
        -1);
    shift_action = JCTypeConverter.toEnum(shiftString, "Trigger",
        JCChartEnumMappings.trigger_strings, JCChartEnumMappings.trigger_values,
        -1);
    ctrl_action = JCTypeConverter.toEnum(ctrlString, "Trigger",
        JCChartEnumMappings.trigger_strings, JCChartEnumMappings.trigger_values,
        -1);
    alt_action = JCTypeConverter.toEnum(altString, "Trigger",
        JCChartEnumMappings.trigger_strings, JCChartEnumMappings.trigger_values,
        -1);
    meta_action = JCTypeConverter.toEnum(metaString, "Trigger",
        JCChartEnumMappings.trigger_strings, JCChartEnumMappings.trigger_values,
        -1);

    allow = allowBoolean;
}

/**
 * Stores the trigger properties from the specified chart.
 * @param chart the chart that contains the trigger properties
 */
public void setWrapperValues(JCChart chart)
{
	EventTrigger et = null;
	
	if ((et = chart.findMatchingTrigger(0)) != null) {
		norm_action = et.getAction();
	} else {
		norm_action = -1;
	}
	if ((et = chart.findMatchingTrigger(InputEvent.SHIFT_MASK)) != null) {
		shift_action = et.getAction();
	} else {
		shift_action = -1;
	}
	if ((et = chart.findMatchingTrigger(InputEvent.CTRL_MASK)) != null) {
		ctrl_action = et.getAction();
	} else {
		ctrl_action = -1;
	}
	if ((et = chart.findMatchingTrigger(InputEvent.ALT_MASK)) != null) {
		alt_action = et.getAction();
	} else {
		alt_action = -1;
	}
	if ((et = chart.findMatchingTrigger(InputEvent.META_MASK)) != null) {
		meta_action = et.getAction();
	} else {
		meta_action = -1;
	}

	allow = chart.getAllowUserChanges();
}

/**
 * Sets the trigger properties on 
 * the specified chart from values stored internally.
 * @param chart the chart that we need to set these trigger values on
 */
public void setPropertyValues(JCChart chart)
{
	EventTrigger et = null;
	
	if ((et = chart.findMatchingTrigger(0)) != null) {
		et.setAction(norm_action);
	}
	if ((et = chart.findMatchingTrigger(InputEvent.SHIFT_MASK)) != null) {
		et.setAction(shift_action);
	}
	if ((et = chart.findMatchingTrigger(InputEvent.CTRL_MASK)) != null) {
		et.setAction(ctrl_action);
	}
	if ((et = chart.findMatchingTrigger(InputEvent.ALT_MASK)) != null) {
		et.setAction(alt_action);
	}
	if ((et = chart.findMatchingTrigger(InputEvent.META_MASK)) != null) {
		et.setAction(meta_action);
	}

	chart.setAllowUserChanges(allow);
}

/**
 * Converts properties to String for code generation.
 * @return String containing properties for code generation
 */
public String toString()
{
    String s = "";

    s += "\"" + JCTypeConverter.fromEnum(norm_action,
        JCChartEnumMappings.trigger_strings, 
		JCChartEnumMappings.trigger_values) + "\"";
    s += ",";
    s += "\"" + JCTypeConverter.fromEnum(shift_action,
        JCChartEnumMappings.trigger_strings, 
		JCChartEnumMappings.trigger_values) + "\"";
    s += ",";
    s += "\"" + JCTypeConverter.fromEnum(ctrl_action,
        JCChartEnumMappings.trigger_strings, 
		JCChartEnumMappings.trigger_values) + "\"";
    s += ",";
    s += "\"" + JCTypeConverter.fromEnum(alt_action,
        JCChartEnumMappings.trigger_strings, 
		JCChartEnumMappings.trigger_values) + "\"";
    s += ",";
    s += "\"" + JCTypeConverter.fromEnum(meta_action,
        JCChartEnumMappings.trigger_strings, 
		JCChartEnumMappings.trigger_values) + "\"";
    s += ",";
    s += allow;

    return s;
}

public boolean equals(Object obj)
{
	boolean eq = true;
	if (obj instanceof TriggerWrapper) {
		TriggerWrapper compareAW = (TriggerWrapper) obj;
		if (compareAW.norm_action != this.norm_action)
			eq = false;
		if (compareAW.shift_action != this.shift_action)
			eq = false;
		if (compareAW.ctrl_action != this.ctrl_action)
			eq = false;
		if (compareAW.alt_action != this.alt_action)
			eq = false;
		if (compareAW.meta_action != this.meta_action)
			eq = false;
		if (compareAW.allow != this.allow)
			eq = false;
	} else {
		eq = false;
	}
	return eq;
}

}
