/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: View3DCube.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:49 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import java.awt.Graphics;
import java.awt.Color;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.Insets;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

import javax.swing.JPanel;
import javax.swing.BorderFactory;

import com.klg.jclass.chart.JCFillStyle;
import com.klg.jclass.chart.JCChartUtil;
import com.klg.jclass.chart.JCChartArea;

 /**
 * <code>View3DCube</code> is used in the <code>View3DEditor</code> to visually set
 * the values of <code>Elevation</code>, <code>Rotation</code>, and <code>Depth</code>.
 */
public class View3DCube extends JPanel 
				implements MouseListener, MouseMotionListener{

protected View3DEditor parent;
protected int deltaX;
protected int deltaY;
protected boolean inDepthChange = false;
protected Point first;
protected Point last;
protected int firstDepth;
protected int firstElevation;
protected int firstRotation;
protected int scaleWidth;
protected int scaleHeight;

View3DCube(View3DEditor editor) {
	parent = editor;
	setBorder( BorderFactory.createEmptyBorder( 2, 2, 2, 2 ) );
	this.addMouseListener(this);
	this.addMouseMotionListener(this);
}

private int getDepth() {
	return Integer.parseInt(parent.textDepth.getText());
}

private void setDepth(int iValue) {
	parent.textDepth.setText(Integer.toString(iValue));
	parent.setSelectedValue(true);
}

private int getElevation() {
	return Integer.parseInt(parent.textElevation.getText());
}

private void setElevation(int iValue) {
	parent.textElevation.setText(Integer.toString(iValue));
	parent.setSelectedValue(true);
}

private int getRotation() {
	return Integer.parseInt(parent.textRotation.getText());
}

private void setRotation(int iValue) {
	parent.textRotation.setText(Integer.toString(iValue));
	parent.setSelectedValue(true);
}

private boolean getChangeDepth() {
	return parent.cbDepth.isSelected();
}

/**
 * Rendering method for <code>View3DCube</code>.
 * @param gc graphics context for drawing
 */
public void paintComponent(java.awt.Graphics gc) {
	if (gc == null) return;
	super.paintComponent(gc);
	gc = gc.create();
	Rectangle r = null;
	r = getBounds( r );

	//  draw the face
	int cWidth = r.width / 3;
	int cHeight = r.height / 3;
	boolean wide = cWidth > cHeight;
	Color outlineColor = Color.black;
	cWidth = Math.min(cWidth, cHeight);
	cHeight = cWidth;
	Rectangle ri = new Rectangle((r.width - cWidth) / 2, 
								 (r.height - cHeight) / 2,
								 cWidth, cHeight);
	JCFillStyle fs = new JCFillStyle(Color.red, JCFillStyle.SOLID);
	fs.fillOutlineRect(gc, ri.x, ri.y, ri.width, ri.height, outlineColor);

	// draw the top
	scaleWidth = (r.width - ri.width) / 2 - 8;
	scaleHeight = (r.height - ri.height) / 2 - 8;
	calculateDeltas(scaleWidth, scaleHeight);
	if (deltaX == 0 && deltaY == 0) return;
   	Point[] points = new Point[4];
	if (deltaY < 0)
   		points[0] = new Point(ri.x, ri.y);
	else 
   		points[0] = new Point(ri.x, ri.y + ri.height);
   	points[1] = new Point(deltaX, deltaY);
   	points[2] = new Point(ri.width, 0);
   	points[3] = new Point(-deltaX, -deltaY);
	Polygon poly = JCChartUtil.pointsToPolygon(points);
   	if (deltaY <= 0)
		fs.fillOutlinePolygonHighlight(gc, poly, outlineColor);
	else
		fs.fillOutlinePolygonShadow(gc, poly, outlineColor);

	// draw the side
	if (deltaX < 0)
   		points[0] = new Point(ri.x, ri.y);
	else 
   		points[0] = new Point(ri.x + ri.width, ri.y);
   	points[1] = new Point(deltaX, deltaY);
   	points[2] = new Point(0, ri.height);
   	points[3] = new Point(-deltaX, -deltaY);
	poly = JCChartUtil.pointsToPolygon(points);
   	if (deltaX <= 0)
		fs.fillOutlinePolygonHighlight(gc, poly, outlineColor);
	else
		fs.fillOutlinePolygonShadow(gc, poly, outlineColor);

	gc.dispose();
}

void calculateDeltas(int rwidth, int rheight) {
	if (parent == null) return;
	int depth = getDepth();
	int elevation = getElevation();
	int rotation = getRotation();
	double dRotation = JCChartUtil.degToRad(rotation);
	double dElevation = JCChartUtil.degToRad(elevation);
	double sin_rotation = Math.sin(dRotation);
	double sin_elevation = Math.sin(dElevation);
	double cos_elevation = Math.cos(dElevation);
	double factor = depth / 80.0;
	double dwidth = JCChartUtil.max(1, rwidth);
	dwidth /= (1 + JCChartUtil.abs(sin_rotation) * cos_elevation * factor);
	double dDepth = dwidth * factor;
	deltaX = (int) (dDepth * sin_rotation * cos_elevation);
	deltaY = (int) (-dDepth * sin_elevation);

	// ensure the delta values stay within limits 
	int wh = rwidth;
    if (Math.abs(deltaX) > wh) {
		deltaX = (deltaX < 0) ? -wh : wh;
	}
	wh = rheight;
    if (Math.abs(deltaY) > wh) {
		deltaY = (deltaY < 0) ? -wh : wh;
	}
}

/*
 * <code>MouseListener</code> methods.
 */
public void mousePressed(MouseEvent event) {
	inDepthChange = getChangeDepth();
	/* once we start moving, set default values 
	if (!inDepthChange && getDepth() == 0) {
		first = null;
		return;
	}
	*/
	first = new Point(event.getX(), event.getY());
	last = null;
	firstDepth = getDepth();
	firstElevation = getElevation();
	firstRotation = getRotation();
}

public void mouseClicked(MouseEvent event) {
}

public void mouseReleased(MouseEvent event) {
	first = null;
}

public void mouseEntered(MouseEvent event) {
}

public void mouseExited(MouseEvent event) {
}

/*
 * <code>MouseMotionListener</code> methods.
 */
public void mouseDragged(MouseEvent event) {
	if (first == null) return;
	int x = event.getX();
	int y = event.getY();
	if (last == null) {
		last = new Point(x, y);
		return;
	}
	if ((x == last.x) && (y == last.y)) return;
	int dy = y - first.y;
	int dx = x - first.x;
	last.x = x;
	last.y = y;

	if (inDepthChange) {
		int newDepth = firstDepth;
		newDepth += dy;
		setDepth(newDepth);
	}
	else {
		double xscale = 2 * JCChartArea.MAX_ROTATION / (double) scaleWidth;
		double yscale = 2 * JCChartArea.MAX_ELEVATION / (double) scaleHeight;
		int newElevation = firstElevation;
		int newRotation = firstRotation;
		newElevation += (int) dy * yscale;
		newRotation -= (int) dx * xscale;
		setElevation(newElevation);
		setRotation(newRotation);
	}	
}

public void mouseMoved(MouseEvent event) {
}

}



