/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: View3DEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:49 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.beans;

import com.klg.jclass.chart.JCChartArea;
import com.klg.jclass.chart.JCChartUtil;

import java.beans.PropertyEditor;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.BorderLayout;
import java.awt.Insets;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.util.StringTokenizer;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JCheckBox;
import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;
import javax.swing.JPanel;
import javax.swing.BorderFactory;

/**
 * Bean editor for chart 3D properties.
 */
public class View3DEditor extends JPanel
						  implements PropertyEditor, ActionListener {

protected PropertyChangeSupport support = new PropertyChangeSupport(this);

protected JTextField textDepth;
protected JTextField textElevation;
protected JTextField textRotation;
protected JCheckBox cbDepth;
protected View3DCube cube;
protected String selectedView;

public View3DEditor() {
	init();
}


protected void init() {
	setLayout( new BorderLayout() );
	setBorder( BorderFactory.createEmptyBorder( 2, 2, 2, 2 ) );

    JPanel p1 = new JPanel();
	p1.setLayout( new GridLayout( 1, 6) );

	textDepth = new JTextField( "", 5 );
	textElevation = new JTextField( "", 5 );
	textRotation = new JTextField( "", 5 );
	textDepth.addActionListener( this );
	textElevation.addActionListener( this );
	textRotation.addActionListener( this );

    p1.add(new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key158)));
	p1.add(textDepth);
    p1.add(new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key159)));
	p1.add(textElevation);
    p1.add(new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key160)));
	p1.add(textRotation);

	add("North", p1);

    JPanel p2 = new JPanel();
	p2.setLayout(new GridLayout(2,1));
    if ((cbDepth = new JCheckBox(JCChartBeanBundle.string(JCChartBeanBundle.key161), false)) != null) { 
        p2.add(cbDepth);
    }
	p2.add(new JLabel(JCChartBeanBundle.string(JCChartBeanBundle.key162)));
	add("South", p2);

	cube = new View3DCube(this);
	add("Center", cube);
}

public void setValue(Object o) {
	setSelectedValue((String)o);
}

protected void setSelectedValue(String s) {
	selectedView = s;
	StringTokenizer st = new StringTokenizer(s, ",");
	for (int j = 0; j < 3; j++) {
		String t = null;
		if (st.hasMoreTokens()) t = st.nextToken();
		switch (j) {
		case 0: 
			textDepth.setText(t); break;
		case 1: 
			textElevation.setText(t); break;
		case 2: 
			textRotation.setText(t); break;
		}
	}
	support.firePropertyChange("", null, null);
	cube.repaint();
}

protected void setSelectedValue(boolean fromCube) {
	// validate the input
	int iDepth = 0;
	int iElevation = 0;
	int iRotation = 0;
	try {
		iDepth = JCChartUtil.clamp(Integer.parseInt(textDepth.getText()), 
								   0, JCChartArea.MAX_DEPTH);
	}
	catch( NumberFormatException e ) { iDepth = 0; }
	try {
		iElevation = JCChartUtil.clamp(Integer.parseInt(textElevation.getText()), 
									   (int) -JCChartArea.MAX_ELEVATION, 
									   (int) JCChartArea.MAX_ELEVATION);
	}
	catch( NumberFormatException e ) { iElevation = 0; }
	try {
		iRotation = JCChartUtil.clamp(Integer.parseInt(textRotation.getText()), 
									  (int) -JCChartArea.MAX_ROTATION, 
									  (int) JCChartArea.MAX_ROTATION);
	}
	catch( NumberFormatException e ) { iRotation = 0; }
	if (fromCube) {
		/* once we start moving, set default values */
		if (iDepth > 0 && iElevation == 0 && iRotation == 0) {
			iElevation = 5;
			iRotation = 5;
		}
		if (iDepth == 0 && (iElevation != 0 || iRotation != 0)) iDepth = 5;
	}
	String s = ""+Integer.toString(iDepth)+","+Integer.toString(iElevation)+","+
				Integer.toString(iRotation);
	setSelectedValue(s);
}

public Object getValue() {
	return (Object) selectedView;
}

public void setAsText(String text) throws IllegalArgumentException {
    setSelectedValue(text);
}

public String getAsText() {
    return selectedView;
}

public String[] getTags() { return null; }

public String getJavaInitializationString() {
	return "\"" + selectedView + "\"";
}

public boolean isPaintable() {
	return true;
}

public void paintValue(Graphics gc, Rectangle box) {
	String s = JCChartBeanBundle.string(JCChartBeanBundle.key163);
	if (s == null) {
		s = new String("");
	}
	FontMetrics fm = gc.getFontMetrics();
	int vpad = (box.height - fm.getAscent())/2;
	gc.drawString(s, box.x, box.y+box.height-vpad);
}

public Component getCustomEditor() {
	return this;
}

public boolean supportsCustomEditor() {
	return true;
}

public Dimension getPreferredSize() {
	return new Dimension(400, 300);
}

public void addPropertyChangeListener(PropertyChangeListener l) {
	support.addPropertyChangeListener(l);
}

public void removePropertyChangeListener(PropertyChangeListener l) {
	support.removePropertyChangeListener(l);
}

// ActionListener method
public void actionPerformed( ActionEvent ev ) {
    if( ev.getSource() instanceof JTextField ) {
		JTextField tf = (JTextField) ev.getSource();
		if( tf == null ) return;
		setSelectedValue( false );
    }
}

/* for testing purposes only
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }

	View3DEditor view3DEditor = new View3DEditor();
    view3DEditor.setBackground( java.awt.Color.lightGray );
	view3DEditor.setValue( "10,45,45" );
    frame.getContentPane().add( view3DEditor );
    frame.pack();
    java.awt.Dimension bigness = view3DEditor.getPreferredSize();
    frame.setSize( bigness.width + 100, bigness.height + 50 );
    frame.setVisible(true);
}
*/

}
