/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AreaLocationPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:49 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;


import java.awt.Container;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Insets;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.FlowLayout;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.BorderFactory;

import com.klg.jclass.chart.JCChart;

/**
 *  Chart Region Location and Size Customizing Page
 */
public class AreaLocationPage extends JPropertyPage {


private JIntegerEditor xField;
private JBooleanEditor xIsDefault;
private JIntegerEditor yField;
private JBooleanEditor yIsDefault;
private JIntegerEditor widthField;
private JBooleanEditor widthIsDefault;
private JIntegerEditor heightField;
private JBooleanEditor heightIsDefault;

private JComponent target = null;
private JCChart targetChart;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );

	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 10, 10, 5, 10 );
	constraints.weightx = 1.0;

	add( new JLabel( "X:" ) );
    xField = new JIntegerEditor();
	xField.setMinimum( -9999 );
    xField.addPropertyChangeListener(this);
    add(xField);
    xIsDefault = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key124 ) );
    xIsDefault.addPropertyChangeListener( this );
	gridbag.setConstraints( xIsDefault, constraints );
    add( xIsDefault );

    add( new JLabel( "Y:" ) );
    yField = new JIntegerEditor();
	yField.setMinimum( -9999 );
    yField.addPropertyChangeListener(this);
    add( yField );
    yIsDefault = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key124 ) );
    yIsDefault.addPropertyChangeListener( this );
	gridbag.setConstraints( yIsDefault, constraints );
    add( yIsDefault );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key116 ) + ":" ) );
    widthField = new JIntegerEditor();
    widthField.addPropertyChangeListener( this );
    add( widthField );
    widthIsDefault = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key124 ) );
    widthIsDefault.addPropertyChangeListener( this );
	gridbag.setConstraints( widthIsDefault, constraints );
    add( widthIsDefault );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key125 ) + ":" ) );
    heightField = new JIntegerEditor();
    heightField.addPropertyChangeListener(this);
    add( heightField );
    heightIsDefault = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key124 ) );
    heightIsDefault.addPropertyChangeListener( this );
	gridbag.setConstraints( heightIsDefault, constraints );
    add( heightIsDefault );
}

/**
 * Return the object being edited
 */
public Object getObject() {
    return target;
}

/**
 * Method called to update the page with values from the supplied source
 */
public void setObject(Object source) {
    //System.out.println("CLabelLP setObject: "+source);
    if (source instanceof JComponent) {
		target = (JComponent) source;
		Container targetParent = target.getParent();
		if (targetParent instanceof JCChart) {
			targetChart = (JCChart) targetParent;
		} else {
			return;
		}
		Rectangle layoutHints = targetChart.getLayoutHints(target);

		boolean isdef;
	
		Point location = target.getLocation();
		isdef = layoutHints.x == Integer.MAX_VALUE;
		xIsDefault.setValue(new Boolean(isdef));
		if (isdef)
			xField.setValue(new Integer(location.x));
		else
			xField.setValue(new Integer(layoutHints.x));

		isdef = layoutHints.y == Integer.MAX_VALUE;
		yIsDefault.setValue(new Boolean(isdef));
		if (isdef)
			yField.setValue(new Integer(location.y));
		else
			yField.setValue(new Integer(layoutHints.y));
	
		Dimension size = target.getSize();
		isdef = layoutHints.width == Integer.MAX_VALUE;
		widthIsDefault.setValue(new Boolean(isdef));
		if (isdef)
			widthField.setValue(new Integer(size.width));
		else
			widthField.setValue(new Integer(layoutHints.width));

		isdef = layoutHints.height == Integer.MAX_VALUE;
		heightIsDefault.setValue(new Boolean(isdef));
		if (isdef)
			heightField.setValue(new Integer(size.height));
		else
			heightField.setValue(new Integer(layoutHints.height));
	}
}

/**
 *  Property Changed listener
 *  Called when a property editor is updated
 */
public void propertyChanged(Object source, Object value) {
    if (target == null || targetChart == null) return;
	Rectangle layoutHints = targetChart.getLayoutHints(target);
    if (value instanceof Integer) {
		if (source == xField && xField.isEnabled()) { 
			layoutHints.x = ((Integer) value).intValue();
			targetChart.setLayoutHints(target, layoutHints);
		} else if (source == yField && yField.isEnabled()) {
	   		layoutHints.y = ((Integer) value).intValue();
			targetChart.setLayoutHints(target, layoutHints);
		} else if (source == widthField && widthField.isEnabled()) {
	   		layoutHints.width = ((Integer) value).intValue();
			targetChart.setLayoutHints(target, layoutHints);
		} else if (source == heightField && heightField.isEnabled()) {
	   		layoutHints.height = ((Integer) value).intValue();
			targetChart.setLayoutHints(target, layoutHints);
		}
		this.setObject();
    }
    else if (value instanceof Boolean) {
		boolean id = ((Boolean)value).booleanValue();
		if (source == xIsDefault) {
			if (id)
				layoutHints.x = Integer.MAX_VALUE;
			else
				layoutHints.x = target.getLocation().x;
			xField.setEnabled(!id);
		} else if (source == yIsDefault) {
			if (id)
				layoutHints.y = Integer.MAX_VALUE;
			else
				layoutHints.y = target.getLocation().y;
			yField.setEnabled(!id);
		} else if (source == widthIsDefault) {
			if (id)
				layoutHints.width = Integer.MAX_VALUE;
			else
				layoutHints.width = target.getSize().width;
			widthField.setEnabled(!id);
		} else if (source == heightIsDefault) {
			if (id)
				layoutHints.height = Integer.MAX_VALUE;
			else
				layoutHints.height = target.getSize().height;
			heightField.setEnabled(!id);
		}
		targetChart.setLayoutHints(target, layoutHints);
		this.setObject();
    }
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java AreaLocationPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    AreaLocationPage areaLocationPage = new AreaLocationPage();
    areaLocationPage.setBackground( java.awt.Color.lightGray );
    areaLocationPage.init();
    frame.getContentPane().add( areaLocationPage );
    frame.pack();
    java.awt.Dimension bigness = areaLocationPage.getPreferredSize();
    frame.setSize(bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key126 );
}

public static String getPageName() {
    return "AreaLocationPage";
}

}

