/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisAnnoPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:49 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Insets;
import java.awt.FlowLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.BorderFactory;

import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCChartUtil;
import com.klg.jclass.chart.JCChartEnumMappings;

/**
 *  Axis Annotation Property Page
 */
public class AxisAnnoPage extends JPropertyPage {

private JEnumEditor   aMethodCombo = null;
private JEnumEditor   aRotCombo = null;
private JFontEditor   annoFontCombo = null;
private JDoubleEditor aRotAngleField = null;

private JCAxis target;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 5, 5, 5, 5 );
	constraints.weightx = 1.0;
    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key234 ) + ":" ) );
     // Anno Method combo
    aMethodCombo = new JEnumEditor(JCChartEnumMappings.annotationMethod_i18n_strings,
							JCChartEnumMappings.annotationMethod_values );
    aMethodCombo.addPropertyChangeListener( this );
	gridbag.setConstraints( aMethodCombo, constraints );
    add( aMethodCombo );

     // Anno Rotation combo
    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key235 ) + ":" ) );
    aRotCombo = new JEnumEditor( JCChartEnumMappings.annRotation_i18n_strings,
									JCChartEnumMappings.annRotation_values );
    aRotCombo.addPropertyChangeListener( this );
	gridbag.setConstraints( aRotCombo, constraints );
    add( aRotCombo );
    add(new JLabel( JCustomizerBundle.string( JCustomizerBundle.key63 ) + ":" ));
	aRotAngleField = new JDoubleEditor( 8 );
    aRotAngleField.addPropertyChangeListener(this);
	gridbag.setConstraints( aRotAngleField, constraints );
	add( aRotAngleField );
    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key237 ) + ":" ) );
    annoFontCombo = new JFontEditor();
    annoFontCombo.addPropertyChangeListener( this );
	gridbag.setConstraints( annoFontCombo, constraints );
    add( annoFontCombo );
}

/**
 * Method called to return the current target
 */
public Object getObject() {
    return target;
}

/**
 * Method called to initialize the page with values from the
 * supplied target
 */
public void setObject(Object source) {
    boolean useDefault;
    String name, current;
    //System.out.println("AAP setObject:"+source);
    if (source instanceof JCAxis) {
		target = (JCAxis) source;

		int annoMethod = target.getAnnotationMethod();
		aMethodCombo.setValue(new Integer(annoMethod));

		int annoRotation = target.getAnnotationRotation();
		aRotCombo.setValue(new Integer(annoRotation));
		// Disable annotation rotation for the theta axis of
		// a POLAR/RADAR chart.
		aRotCombo.setEnabled( !((target.isPolar() || target.isRadar()) &&
													!target.isVertical()) );
		double annoRotationAngle =
			target.getAnnotationRotationAngle(
							target.getChartArea().getAngleUnit());
		annoRotationAngle = JCChartUtil.precCorrect(10, annoRotationAngle);
		Object obj = aRotAngleField.getValue();
		if (obj == null) {
			aRotAngleField.setValue(new Double(annoRotationAngle));
		}
		else if (((Double)obj).doubleValue() != annoRotationAngle) {
			// Don't update the JDoubleEditor component if annoRotationAngle
			// hasn't changed.
			aRotAngleField.setValue(new Double(annoRotationAngle));
		}
		//  Disable annotation rotation at arbitrary angles if the the chart
		//  type is POLAR or RADAR.
		aRotAngleField.setEnabled( !(target.isPolar() || target.isRadar()) );

		annoFontCombo.setValue( target.getFont() );
    }
}

/**
 * This method is called when a specialized property editor's
 * value is changed.
 */
public void propertyChanged(Object source, Object value) {
    if (target == null) return;
    if (source == annoFontCombo) {
		target.setFont((java.awt.Font)value);
    }
    else if (source == aMethodCombo) {
		try {
	    	target.setAnnotationMethod(((Integer)value).intValue());
		} catch( Exception e ) {
	    	showError( JCustomizerBundle.string( JCustomizerBundle.key124 ) + ": \n" + e );
	    	target.setAnnotationMethod(JCAxis.VALUE);
		}
    } else if (source == aRotCombo) {
		target.setAnnotationRotation(((Integer)value).intValue());
    }
    else if (source == aRotAngleField) {
	    target.setAnnotationRotationAngle(target.getChartArea().getAngleUnit(),
	    									((Double)value).doubleValue());
	}
    setObject();
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java AxisAnnoPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    AxisAnnoPage axisAnnoPage = new AxisAnnoPage();
    axisAnnoPage.setBackground( java.awt.Color.lightGray );
    axisAnnoPage.init();
	axisAnnoPage.setObject( new com.klg.jclass.chart.JCAxis() );
    frame.getContentPane().add( axisAnnoPage );
    frame.pack();
    java.awt.Dimension bigness = axisAnnoPage.getPreferredSize();
    frame.setSize( bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key238 );
}

public static String getPageName() {
    return "AxisAnnoPage";
}

}

