/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisAttachPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:49 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Font;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Insets;

import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.ButtonGroup;
import javax.swing.BorderFactory;

import com.klg.jclass.chart.ChartDataView;
import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.JCChartEnumMappings;

/**
 *  Axis Property Page
 */
public class AxisAttachPage extends JPropertyPage {

private JPanel attachCards = null;
private JBooleanEditor attachViewToggle = null;
private JBooleanEditor attachAxisToggle = null;
private JBooleanEditor attachValueToggle = null;

private ViewAxisPage viewGroup = null;
private AxisRelationPage relationGroup = null;
private AxisPlacementPage placementGroup = null;
private CardLayout card_layout = null;

private JCChart chart = null;
private JCAxis target = null;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
    setLayout( new BorderLayout() );

    JPanel boxesPanel = new JPanel();

	boxesPanel.setBorder( BorderFactory.createTitledBorder( JCustomizerBundle.string( JCustomizerBundle.key286 ) + ":" ) );
	ButtonGroup boxesGroup = new ButtonGroup();
    boxesPanel.setLayout( new GridLayout( 1, 3 ) );

    attachViewToggle = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key83 ) );
    attachViewToggle.addPropertyChangeListener( this );
	boxesGroup.add( attachViewToggle );
    boxesPanel.add( attachViewToggle );

    attachAxisToggle = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key84 ) );
    attachAxisToggle.addPropertyChangeListener( this );
	boxesGroup.add( attachAxisToggle );
    boxesPanel.add( attachAxisToggle );

    attachValueToggle = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key85 ) );
    attachValueToggle.addPropertyChangeListener( this );
	boxesGroup.add( attachValueToggle );
    boxesPanel.add( attachValueToggle );

    attachCards = new JPanel();
    attachCards.setLayout( card_layout = new CardLayout() );
    viewGroup = new ViewAxisPage();
    viewGroup.init();
    relationGroup = new AxisRelationPage();
    relationGroup.init();
    placementGroup = new AxisPlacementPage();
    placementGroup.init();
    attachCards.add( JCustomizerBundle.string( JCustomizerBundle.key83 ), viewGroup );
    attachCards.add( JCustomizerBundle.string( JCustomizerBundle.key286 ), relationGroup );
    attachCards.add( JCustomizerBundle.string( JCustomizerBundle.key202 ), placementGroup );

    add( boxesPanel, BorderLayout.NORTH );
    add( attachCards, BorderLayout.CENTER );
}

/**
 * Method called to attempt to attach the current target and return it
 */
public JCAxis attachAxis() {
    JCAxis axis = (JCAxis) getObject();
    if (axis == null) return null;
    if( ((Boolean)attachViewToggle.getValue()).booleanValue() ) {
	ChartDataView view = (ChartDataView) viewGroup.viewCombo.getItem( ((Integer)viewGroup.viewCombo.getValue()).intValue() );
	if (view != null) {
	    if( ((Boolean)viewGroup.xCheck.getValue()).booleanValue() ) {
		axis.setVertical(view.getXAxis().isVertical());
		view.setXAxis(axis);
		return(axis);
	    }
	    else if ( ((Boolean)viewGroup.yCheck.getValue()).booleanValue() ) {
		axis.setVertical(view.getYAxis().isVertical());
		view.setYAxis( axis );
		return(axis);
	    }
	}
	else {
	    showError( JCustomizerBundle.string( JCustomizerBundle.key287 ) );
	}
    }
    else if( ((Boolean)attachAxisToggle.getValue()).booleanValue() ) {
		JCAxis orig = (JCAxis) relationGroup.axisRelCombo.getItem(
				((Integer)relationGroup.axisRelCombo.getValue()).intValue());
		Double mult = (Double) relationGroup.multiplierField.getValue();
		Double cons = (Double) relationGroup.constantField.getValue();
		if (orig != null && mult != null && cons != null) {
			axis.setVertical(orig.isVertical());
			axis.getFormula().setOriginator(orig);
			axis.getFormula().setMultiplier(mult.doubleValue());
			axis.getFormula().setConstant(cons.doubleValue());
			return(axis);
		}   
		showError( JCustomizerBundle.string( JCustomizerBundle.key288 ) );
    }
    else if ( ((Boolean)attachValueToggle.getValue()).booleanValue() ) {
		Object placementType = placementGroup.aPlaceCombo.getValue();
		Object placement = placementGroup.placementField.getValue();
		Object place = placementGroup.axisValCombo.getItem(
				((Integer)placementGroup.axisValCombo.getValue()).intValue());
		if (placementType instanceof Integer && place instanceof JCAxis) {
			axis.setVertical(!((JCAxis)place).isVertical());
			if (((Integer)placementType).intValue() == JCAxis.VALUE_ANCHORED) {
				if (placement instanceof Double) {
					axis.setPlacement((JCAxis)place, 
					  	((Double)placement).doubleValue());
					return(axis);
				}
			} else {
				axis.setPlacement((JCAxis)place);
				axis.setPlacement(((Integer)placementType).intValue());
				return(axis);
			}
		}
		showError( JCustomizerBundle.string( JCustomizerBundle.key289 ) );
    }
    return null;
}

/**
 * Method called to return the current target
 */
public Object getObject() {
    return target;
}

/**
 * Method called to provide this dialog with a model object
 * to emulate, particularly in the preview
 */
public void setModel(Object source) {
    //System.out.println("AxisAttach setModel: "+source);
    if (source == null) return;
    if (source instanceof JCChart) {
	chart = (JCChart) source;
    }
}

/**
 * Method called to populate the dialog with the source's properties
 */
public void setObject(Object source) {
    if (source == null) return;
    //System.out.println("AAP setObject:"+source);
    if( source instanceof JCAxis ) {
		target = (JCAxis) source;
		if (target.getFormula().getOriginator() != null) {
			attachAxisToggle.setValue( new Boolean( true ) );
		}
		else if (target.getPlacementAxis() != null) {
			attachValueToggle.setValue( new Boolean( true ) );
		}
		else {
			attachViewToggle.setValue( new Boolean( true ) );
		}
		viewGroup.setObject(source);
    }
}

public void propertyChanged( Object source, Object value ) {
	if( target == null ) return;
	if( source == null ) return;
	if( source instanceof JBooleanEditor ) {
		JBooleanEditor box = (JBooleanEditor)source;
		boolean val = ((Boolean)value).booleanValue();
		if( !val ) return;

	if (box == attachValueToggle) {
	    card_layout.show( attachCards, JCustomizerBundle.string( JCustomizerBundle.key202 ) );
	    placementGroup.setObject( target );
	    placementGroup.setObject( chart );
	} else if (box == attachAxisToggle) {
	    card_layout.show(attachCards, JCustomizerBundle.string( JCustomizerBundle.key286 ) );
	    relationGroup.setObject( target );
	    relationGroup.setObject( chart );
	} else {
	    card_layout.show( attachCards, JCustomizerBundle.string( JCustomizerBundle.key83 ) );
	    viewGroup.setObject( target );
	    viewGroup.setObject( chart );
	}
	}

}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java AxisAttachPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    AxisAttachPage axisAttachPage = new AxisAttachPage();
    axisAttachPage.setBackground( java.awt.Color.lightGray );
    axisAttachPage.init();
	axisAttachPage.setObject( new com.klg.jclass.chart.JCAxis() );
    frame.getContentPane().add( axisAttachPage );
    frame.pack();
    java.awt.Dimension bigness = axisAttachPage.getPreferredSize();
    frame.setSize(bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key290 );
}

public static String getPageName() {
    return "AxisAttachPage";
}

}
