/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisGeneralPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:49 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Insets;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.BorderFactory;

import com.klg.jclass.chart.JCAxis;

/**
 *  General Chart Property Page
 */
public class AxisGeneralPage extends JPropertyPage {

private JBooleanEditor showingCheck;
private JBooleanEditor logarithmicCheck;
private JBooleanEditor reversedCheck;
private JBooleanEditor editableCheck;
private JBooleanEditor verticalCheck;
private AxisAttachTabs  axisAttachTabs;

private JCAxis target;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
	setLayout( new BorderLayout( 5, 5 ) );

    JPanel controls = new JPanel();

	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	controls.setLayout( gridbag );
	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 10, 10, 5, 10 );
	constraints.weightx = 1.0;
    showingCheck = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key27 ) );
    showingCheck.addPropertyChangeListener(this);
	gridbag.setConstraints( showingCheck, constraints );
    controls.add( showingCheck );
    editableCheck = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key193 ) );
    editableCheck.addPropertyChangeListener( this );
	gridbag.setConstraints( editableCheck, constraints );
    controls.add(editableCheck);
    logarithmicCheck = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key194 ) );
    logarithmicCheck.addPropertyChangeListener( this );
	gridbag.setConstraints( logarithmicCheck, constraints );
    controls.add( logarithmicCheck );
    reversedCheck = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key195 ) );
    reversedCheck.addPropertyChangeListener( this );
    reversedCheck.setEnabled( false );
	gridbag.setConstraints( reversedCheck, constraints );
    controls.add( reversedCheck );
    verticalCheck = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key196 ) );
    verticalCheck.addPropertyChangeListener(this);
    verticalCheck.setEnabled( false );
	gridbag.setConstraints( verticalCheck, constraints );
    controls.add( verticalCheck );
    add( "West", controls );
    axisAttachTabs = new AxisAttachTabs();
    axisAttachTabs.init();
    add( "Center", axisAttachTabs );
}

/**
 * Method called to return the current target
 */
public Object getObject() {
    return target;
}

/**
 * Method called to initialize the page
 */
public void setObject(Object source) {
    //System.out.println("AGP setObject:"+source);
    //attach.setObject(source);
    if (source instanceof JCAxis) {
		target = (JCAxis) source;
		showingCheck.setValue(new Boolean(target.isVisible()));
		verticalCheck.setValue(new Boolean(target.isVertical()));
		boolean fPolarRadarThetaAxis =
				( (target.isPolar() || target.isRadar()) && !target.isVertical() );
		logarithmicCheck.setValue(new Boolean(target.isLogarithmic() &&
											  !fPolarRadarThetaAxis));
		logarithmicCheck.setEnabled(!fPolarRadarThetaAxis);
		reversedCheck.setValue(new Boolean(target.isReversed()));
		editableCheck.setValue(new Boolean(target.isEditable()));
    }
    axisAttachTabs.setObject(source);
}

/**
 * This method is called when a specialized property editor's
 * value is changed.
 */

public void propertyChanged(Object source, Object value) {
    if (target == null || value == null) return;
    if (value instanceof Boolean) {
	boolean toggle = ((Boolean)value).booleanValue();
	if (source == verticalCheck) {
	    target.setVertical(toggle);
	} else if (source == logarithmicCheck) {
	    target.setLogarithmic(toggle);
	} else if (source == reversedCheck) {
	    target.setReversed(toggle);
	} else if (source == editableCheck) {
	    target.setEditable(toggle);
	} else if (source == showingCheck) {
	    target.setVisible(toggle);
	}
	this.setObject();
    }
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java AxisGeneralPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    AxisGeneralPage axisGeneralPage = new AxisGeneralPage();
    axisGeneralPage.setBackground( java.awt.Color.lightGray );
    axisGeneralPage.init();
	axisGeneralPage.setObject( new com.klg.jclass.chart.JCChart() );
    frame.getContentPane().add( axisGeneralPage );
    frame.pack();
    java.awt.Dimension bigness = axisGeneralPage.getPreferredSize();
    frame.setSize( bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key197 );
}

public static String getPageName() {
    return "AxisGeneralPage";
}

}

