/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisGridPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:49 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Insets;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.BorderFactory;

import com.klg.jclass.chart.JCAxis;

/**
 *  Axis Grid Property Page
 */
public class AxisGridPage extends JPropertyPage {

private JBooleanEditor showGridCheck = null;
private JDoubleEditor gridField = null;
private JBooleanEditor gridDefault = null;
private LineStylePage stylePage = null;

private JCAxis target;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );

    JPanel top = new JPanel();
	GridBagConstraints topConstraints = new GridBagConstraints();
	GridBagLayout topGridbag = new GridBagLayout();

	top.setLayout( topGridbag );
	top.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	topConstraints.anchor = GridBagConstraints.WEST;
	topConstraints.gridwidth = GridBagConstraints.REMAINDER;
	topConstraints.insets = new Insets( 5, 5, 5, 5 );
	topConstraints.weightx = 1.0;
	setLayout( topGridbag );

    top.add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key198 ) + ":" ) );
    showGridCheck = new JBooleanEditor( "" );
    showGridCheck.addPropertyChangeListener( this );
	topGridbag.setConstraints( showGridCheck, topConstraints );
    top.add( showGridCheck );
    top.add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key199 ) + ":" ) );
    gridField = new JDoubleEditor( 9 );
    gridField.addPropertyChangeListener( this );
    top.add( gridField );
    gridDefault = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key124 ) );
    gridDefault.addPropertyChangeListener( this );
	topGridbag.setConstraints( gridDefault, topConstraints );
    top.add( gridDefault );
	topGridbag.setConstraints( top, topConstraints );
    add( top );

    JPanel bottom = new JPanel();

    bottom.setBorder( BorderFactory.createEmptyBorder( 0, 3, 3, 0 ) ); // top, left, bottom, right
	bottom.setBorder( BorderFactory.createTitledBorder( JCustomizerBundle.string( JCustomizerBundle.key200 ) ) );
    bottom.setLayout( new GridLayout( 1, 1 ) );
    stylePage = new LineStylePage();
    stylePage.init();
    bottom.add( stylePage );
	topGridbag.setConstraints( bottom, topConstraints );
    add( bottom );
}

/**
 * Method called to initialize the page
 */
public void setObject(Object source) {
    //System.out.println("AP setObject:"+source);
	boolean isGridShowing;
    boolean useDefault;
    Double  gridSpacing;
    if (source instanceof JCAxis) {
		target = (JCAxis) source;
		isGridShowing = target.isGridVisible();
		showGridCheck.setSelected( isGridShowing );
		gridSpacing = new Double( target.getGridSpacing() );
		gridField.setValue( gridSpacing.toString() );
		useDefault = target.getGridSpacingIsDefault();
		gridDefault.setSelected( useDefault );
		gridDefault.setEnabled( !useDefault );
		stylePage.setObject( target.getGridStyle() );
	}
}

public void propertyChanged( Object source, Object value ) {
    if( target == null || source == null ) return;
	if( source instanceof JBooleanEditor ) {
		JBooleanEditor box = (JBooleanEditor) source;
		boolean val = ((Boolean)value).booleanValue();

		if( box == showGridCheck )
			target.setGridVisible( val );
		if( box == gridDefault )
			target.setGridSpacingIsDefault( val );
    }
	if( source instanceof JDoubleEditor ) {
		JDoubleEditor dblE = (JDoubleEditor)source;
		double val = ((Double)value).doubleValue();

		if( dblE == gridField ) {
			target.setGridSpacing( val );
			this.setObject( target );
		}
	}
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java AxisGridPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    AxisGridPage axisGridPage = new AxisGridPage();
    axisGridPage.setBackground( java.awt.Color.lightGray );
    axisGridPage.init();
	axisGridPage.setObject( new com.klg.jclass.chart.JCChart() );
    frame.getContentPane().add( axisGridPage );
    frame.pack();
    java.awt.Dimension bigness = axisGridPage.getPreferredSize();
    frame.setSize( bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key201 );
}

public static String getPageName() {
    return "AxisGridPage";
}

}

