/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisOrientPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:49 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Component;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.FlowLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.awt.Insets;
import java.awt.Color;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;

import com.klg.jclass.chart.ChartDataView;
import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.beans.AxesOrientationImages;

/**
 *  Axis Orientation (inverted, reversed) Property Page
 */
public class AxisOrientPage extends JPropertyPage  
                            implements ActionListener, ItemListener {

private JBooleanEditor isInvertedCheck = null;
private ButtonGroup orientBox = null ;
private JButton xAxisButton = null; 
private JButton yAxisButton = null;
private JAddRemoveDialog dialog = null;
private Object[] radioButtons = null;
private JRadioButton currentButton = null;

private ChartDataView target;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
    
	JPanel all = new JPanel();
    JPanel controls = new JPanel();
	JPanel orient = new JPanel();
    JPanel contain = new JPanel();

	all.setLayout( new FlowLayout( FlowLayout.LEFT, 2, 3 ) );
	orient.setLayout( new GridLayout( 4, 2 ) );
	orient.setBorder( BorderFactory.createTitledBorder( JCustomizerBundle.string( JCustomizerBundle.key224 ) ) );
    contain.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );

    GridBagConstraints controlsConstraints = new GridBagConstraints();
	GridBagLayout controlsGridbag = new GridBagLayout();

	contain.setLayout( controlsGridbag );
	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	controlsConstraints.anchor = GridBagConstraints.WEST;
	controlsConstraints.gridwidth = GridBagConstraints.REMAINDER;
	controlsConstraints.insets = new Insets( 5, 5, 5, 5 );
	controlsConstraints.weightx = 1.0;

    isInvertedCheck = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key223 ) );
    isInvertedCheck.addPropertyChangeListener( this );
	controlsGridbag.setConstraints( isInvertedCheck, controlsConstraints );
    controls.add( isInvertedCheck );

    {
	int ni = AxesOrientationImages.getOrientationImageLength();

	radioButtons = new Object[ ni ];
	orientBox = new ButtonGroup();

	for (int i = 0; i < ni; i++ ) {
		JRadioButton radio = new JRadioButton( 
			new ImageIcon ( AxesOrientationImages.getOrientationImage( 
											   i, AxesOrientationImages.ENABLED ) ) );
	    radioButtons[ i ] = radio;
		orientBox.add( radio );
		orient.add( radio );
		radio.addItemListener( this );
	}
    }
	controlsGridbag.setConstraints( orient, controlsConstraints );
    controls.add( orient );
    all.add( controls );

    GridBagConstraints containConstraints = new GridBagConstraints();
	GridBagLayout containGridbag = new GridBagLayout();

	contain.setLayout( containGridbag );
	containConstraints.anchor = GridBagConstraints.WEST;
	containConstraints.gridwidth = GridBagConstraints.REMAINDER;
	containConstraints.insets = new Insets( 5, 5, 5, 5 );
	containConstraints.weightx = 1.0;

    all.add( contain );
    add( all );
}

/**
 * Return the Object that this Property Page is editing
 */
public Object getObject() {
    return target;
}

/**
 * Method called to initialize the page
 */
public void setObject(Object source) {
    Integer value;
    //System.out.println("AGP setObject:"+source);
    if( source instanceof ChartDataView ) {
		target = (ChartDataView) source;
		boolean inverted = target.isInverted();
		isInvertedCheck.setValue( new Boolean( inverted ) );
		setViewOrient( target );
    }
}

void setViewOrient( ChartDataView dv ) {
    int radioSelected = AxesOrientationImages.viewToValue(dv);
    int orientation = dv.isInverted() ? 1 : 0;
    for( int i = 0; i < radioButtons.length; i++,i++ ) {
		for( int j = 0; j < 2; j++ ) {
	    	JRadioButton item = (JRadioButton)radioButtons[ i + j ];
	
			if( i + j == radioSelected ) {
				if( currentButton != null ) {
					currentButton.setBorderPainted( false );
				}
				setCurrentButton( item );
			}
			if( orientation == j ) {
				item.setIcon( new ImageIcon( 
					AxesOrientationImages.getOrientationImage( i + j, 
					AxesOrientationImages.ENABLED ) ) );
				item.setEnabled( true );
	    	} else {
				item.setIcon( new ImageIcon( 
					AxesOrientationImages.getOrientationImage( i + j, 
					AxesOrientationImages.DISABLED ) ) );
				item.setEnabled( false );
	    	}
		}
    }
}

static Image getEnabledOrientationImage( Component c, int i ) {
	return AxesOrientationImages.getOrientationImage(i, AxesOrientationImages.ENABLED );
}

static Image getEnabledOrientationImage( Component c, ChartDataView dv ) {
	return AxesOrientationImages.getOrientationImage(dv, AxesOrientationImages.ENABLED );
}

/**
 * This method is called when a specialized property editor's 
 * value is changed.
 */

public void propertyChanged( Object source, Object value ) {
    if (target == null || value == null) return;
    if (source == isInvertedCheck) {
		boolean toggle = ((Boolean)value).booleanValue();
		target.setInverted( toggle );
		setObject();
	}
    else if( source == dialog ) {
			JPropertyPage.getFrame( dialog ).setVisible( false );
	}
}


public void itemStateChanged( ItemEvent ev ) {
	if( target == null ) return;
    if (ev.getSource() instanceof JRadioButton ) {
		JRadioButton radio = (JRadioButton)ev.getSource();

		if( radio == null || target == null ) return;
		if( radio == currentButton ) return;
		boolean value = radio.isSelected();
		
		if (value == false) return;
		int orientation = getButtonIndex( radio );
		boolean inverted = (orientation & 1) == 1;
		boolean vertRev = (orientation & 4) == 4;
		boolean horzRev = (orientation & 2) == 2;

		ChartDataView dv = target;

		setCurrentButton( radio );
		dv.setBatched( true );
		if( dv.getXAxis() != null && dv.getYAxis() != null ) {
			dv.getYAxis().setReversed( vertRev );
			dv.getXAxis().setReversed( horzRev );
		}
		dv.setBatched( false );
		setObject();
    }
}

private void setCurrentButton( JRadioButton radio ) {
	if( radio != null ) {
		if( currentButton != null )
			currentButton.setBorderPainted( false );
		radio.setBorderPainted( true );
		currentButton = radio;
		currentButton.setSelected( true );
	}
}

private int getButtonIndex( JRadioButton radio ) {
	int index = -1;

	for( int i = 0; i < radioButtons.length; i++ ) {
		if( radioButtons[ i ] == radio ) {
			index = i;
			break;
		}
	}
	return index;
}
/**
 * JCActionListener method
 * Called when a button pressed
 */
public void actionPerformed( ActionEvent evt) {
    if (target == null) return;
}


/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java AxisOrientPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    AxisOrientPage axisOrientPage = new AxisOrientPage();
    axisOrientPage.setBackground( java.awt.Color.lightGray );
    axisOrientPage.init();
	axisOrientPage.setObject( new com.klg.jclass.chart.JCChart() );
    frame.getContentPane().add( axisOrientPage );
    frame.pack();
    java.awt.Dimension bigness = axisOrientPage.getPreferredSize();
    frame.setSize( bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key231 );
}

public static String getPageName() {
    return "AxisOrientPage";
}

}

