/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisPlacementPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:49 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.FlowLayout;




import java.util.List;


import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.BorderFactory;

import com.klg.jclass.chart.ChartDataView;
import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.JCChartEnumMappings;

/**
 *  Axis Property Page
 */
public class AxisPlacementPage extends JPropertyPage {

protected JEnumEditor axisValCombo = null;
protected JDoubleEditor placementField = null;
protected JEnumEditor aPlaceCombo = null;
protected JBooleanEditor aPlaceDefault = null;

private JCAxis target;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 5, 10, 5, 5 );
	constraints.weightx = 1.0;

     // Anno Placement combo
    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key85 ) + ":" ));
    JPanel panel = new JPanel();
    panel.setLayout( new FlowLayout( FlowLayout.LEFT, 1, 5 ) );
    aPlaceCombo = new JEnumEditor( JCChartEnumMappings.placement_i18n_strings,
                                    JCChartEnumMappings.placement_values );
    aPlaceCombo.addPropertyChangeListener( this );
    panel.add( aPlaceCombo );
    aPlaceDefault = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key124 ) );
    aPlaceDefault.addPropertyChangeListener( this );
    panel.add( aPlaceDefault );
    gridbag.setConstraints( panel, constraints );
    add( panel );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key70 ) + ":" ) );
    axisValCombo = new JEnumEditor();
    axisValCombo.addPropertyChangeListener( this );
	gridbag.setConstraints( axisValCombo, constraints );
    add( axisValCombo );
    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key97 ) + ":" ) );
    placementField = new JDoubleEditor( 6 );
    placementField.addPropertyChangeListener( this );
	gridbag.setConstraints( placementField, constraints );
    add( placementField );
}

private void setAxisList(JCChart chart) {
    int numx = 0;
    int numy = 0;
    List xaxes = chart.getChartArea().getXAxes();
    if (xaxes != null) {
		numx = xaxes.size();
    }
    List yaxes = chart.getChartArea().getYAxes();
    if (yaxes != null) {
		numy = yaxes.size();
    }
    //System.out.println("AAP setList="+numx+numy);
    String list[] = new String[numx + numy];
    JCAxis axes[] = new JCAxis[numx + numy];
    int index = 0;
    for (int i=0; i<numx; i++, index++) {
		Object t = xaxes.get(i);
		axes[index] = (JCAxis)t;
		if (axes[index] == null) {
			list[index] = "";
		}
		else {
			list[index] = new String(""+((JCAxis)t).getName());
		}
    }
    for( int i = 0; i < numy; i++, index++ ) {
		Object t = yaxes.get(i);
		axes[index] = (JCAxis)t;
		if (axes[index] == null) {
			list[index] = "";
		}
		else {
			list[index] = new String(""+((JCAxis)t).getName());
		}
    }
    if (index > 0) {
		axisValCombo.setItems(list, axes);
		//System.out.println("AAP setAxis:"+list);
    }
}


/**
 * Method called to return the current target
 */
public Object getObject() {
    return target;
}

/**
 * Method called to initialize the page
 */
public void setObject( Object source ) {
    //System.out.println("APP setObject:"+source);
    if( source instanceof JCChart ) {
		setAxisList( (JCChart)source );
    }
    else if( source instanceof JCAxis ) {
		target = (JCAxis) source;

		JCChart parent = target.getParentChart();

		if (parent == null) return;

    int annoPlacement = target.getPlacement();

    aPlaceCombo.setValue(new Integer(annoPlacement));
    boolean useDefault = target.getPlacementIsDefault();
    aPlaceDefault.setValue(new Boolean(useDefault));
    aPlaceDefault.setEnabled( !useDefault );
    aPlaceCombo.setEnabled( !useDefault );

		setAxisList(parent);

		JCAxis place = target.getPlacementAxis();
		
		if( place == null ) {
			axisValCombo.setValue("");
			axisValCombo.setEnabled( true );
			placementField.setValue( "" );
			placementField.textField.setEditable( false );
		} else {
			axisValCombo.setValue( place );
			axisValCombo.setEnabled( false );
			placementField.setValue( new Double( target.getPlacementLocation() ) );
			placementField.textField.setEditable( true );
		}
    }
}

/**
 *  Property Change listener
 *  Called when a property editor is updated
 */
public void propertyChanged( Object source, Object value ) {
    if( target == null ) return;
    if( value instanceof Double ) {
		if( source == placementField ) {
	    	target.setPlacementLocation( ((Double)value).doubleValue());
		}
    } else if( value instanceof Integer && source == axisValCombo ) {
		JCAxis newAxis = (JCAxis)axisValCombo.getItem( ((Integer)value).intValue() );
		JCAxis old = target.getPlacementAxis();
		if( old != newAxis ) {
			target.setPlacementAxis( newAxis );
		}
    } else if (source == aPlaceCombo) {
		int placeValue = ((Integer)value).intValue();
    	target.setPlacement(placeValue);
		if (placeValue == JCAxis.VALUE_ANCHORED) {
			placementField.textField.setEditable(true);
			placementField.setValue( new Double( target.getPlacementLocation() ) );
		} else {
			placementField.setValue("");
			placementField.textField.setEditable(false);
		}
    } else if (value instanceof Boolean && source == aPlaceDefault) {
        boolean toggle = ((Boolean)value).booleanValue();
        target.setPlacementIsDefault(toggle);
	}
	this.setObject();
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java AxisPlacementPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    AxisPlacementPage axisPlacementPage = new AxisPlacementPage();
    axisPlacementPage.setBackground( java.awt.Color.lightGray );
    axisPlacementPage.init();
	axisPlacementPage.setObject( new com.klg.jclass.chart.JCAxis() );
    frame.getContentPane().add( axisPlacementPage );
    frame.pack();
    java.awt.Dimension bigness =  axisPlacementPage.getPreferredSize();
    frame.setSize( bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key208 );
}

public static String getPageName() {
    return "AxisPlacementPage";
}

}
