/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisRelationPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:49 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;




import java.util.List;


import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.BorderFactory;

import com.klg.jclass.chart.ChartDataView;
import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCChart;

/**
 *  Axis Relation Page
 */
public class AxisRelationPage extends JPropertyPage {

protected JEnumEditor axisRelCombo = null;
protected JDoubleEditor multiplierField = null;
protected JDoubleEditor constantField = null;

private JCAxis target;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 5, 10, 5, 5 );
	constraints.weightx = 1.0;
    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key204 ) + ":" ) );
    axisRelCombo = new JEnumEditor();
    axisRelCombo.addPropertyChangeListener( this );
	gridbag.setConstraints( axisRelCombo, constraints );
    add( axisRelCombo );
    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key205 ) + ":" ) );
    multiplierField = new JDoubleEditor(7);
    multiplierField.addPropertyChangeListener(this);
	gridbag.setConstraints( multiplierField, constraints );
    add( multiplierField );
	add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key206 ) + ":" ) );
    constantField = new JDoubleEditor(7);
    constantField.addPropertyChangeListener(this);
	gridbag.setConstraints( constantField, constraints );
    add( constantField );
}

private void setAxisList(JCChart chart) {
    int numx = 0;
    int numy = 0;
    List xaxes = chart.getChartArea().getXAxes();
    if (xaxes != null) {
		numx = xaxes.size();
    }
    List yaxes = chart.getChartArea().getYAxes();
    if (yaxes != null) {
		numy = yaxes.size();
    }
    //System.out.println("ARP setList="+numx+numy);
    String list[] = new String[numx + numy + 1];
    JCAxis axes[] = new JCAxis[numx + numy + 1];
	list[0] = "None";
	axes[0] = null;
    int index = 1;
    for (int i=0; i<numx; i++, index++) {
		Object t = xaxes.get(i);
		axes[index] = (JCAxis)t;
		if (axes[index] == null) {
			list[index] = "";
		}
		else {
			list[index] = new String(""+((JCAxis)t).getName());
		}
    }
    for( int i = 0; i < numy; i++, index++ ) {
		Object t = yaxes.get(i);
		axes[index] = (JCAxis)t;
		if (axes[index] == null) {
			list[index] = "";
		}
		else {
			list[index] = new String(""+((JCAxis)t).getName());
		}
    }
    if (index > 0) {
		axisRelCombo.setItems( list, axes );
		//System.out.println("ARP setAxis:"+list);
    }
}


/**
 * Method called to return the current target
 */
public Object getObject() {
    return target;
}

/**
 * Method called to initialize the page
 */
public void setObject(Object source) {
    //System.out.println("ARP setObject:"+source);
    if( source instanceof JCChart) {
		setAxisList((JCChart) source);
    }
    else if( source instanceof JCAxis ) {
		target = (JCAxis) source;
		JCChart parent = target.getParentChart();
		if (parent == null) return;
		setAxisList( parent );
		List views = parent.getDataView();
		if (views == null) return;

		JCAxis orig;
		if( ( orig = target.getFormula().getOriginator() ) != null ) {
	    	axisRelCombo.setValue( orig );
	    	axisRelCombo.setEnabled( false );
	    	Double value;
	    	value = new Double(target.getFormula().getMultiplier());
	    	multiplierField.setValue( value );
	    	multiplierField.textField.setEditable( true );
	    	value = new Double( target.getFormula().getConstant() );
	    	constantField.setValue( value );
	    	constantField.textField.setEditable( true );
		}
		else {
	    	axisRelCombo.setValue( "" );
	    	axisRelCombo.setEnabled( true );
	    	multiplierField.setValue("");
	    	multiplierField.textField.setEditable( false );
	    	constantField.setValue( "" );
	    	constantField.textField.setEditable( false );
		}
    }
}

/**
 *  Property Change listener
 *  Called when a property editor is updated
 */
public void propertyChanged(Object source, Object value) {
    if (target == null) return;
    if (value instanceof Double) {
		if (source == multiplierField) {
	    	target.getFormula().setMultiplier(((Double)value).doubleValue());
		}
		else if (source == constantField) {
	    	target.getFormula().setConstant(((Double)value).doubleValue());
		}
		this.setObject();
    } else if (value instanceof Integer) {
		if (source == axisRelCombo) {
        	JCAxis newAxis = (JCAxis)axisRelCombo.getItem( ((Integer)value).intValue () );
        	JCAxis old = target.getFormula().getOriginator();
        	if( old != newAxis && newAxis != target ) {
            	target.getFormula().setOriginator( newAxis );
				if (newAxis != null) {
	    			multiplierField.textField.setEditable( true );
	    			constantField.textField.setEditable( true );
				} else {
	    			multiplierField.textField.setEditable( false );
	    			constantField.textField.setEditable( false );
        		}
			} else if (newAxis == target) {
	    		axisRelCombo.setValue( old );
			}
		}
		this.setObject();
	}
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java AxisRelationPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    AxisRelationPage axisRelationPage = new AxisRelationPage();
    axisRelationPage.setBackground( java.awt.Color.lightGray );
    axisRelationPage.init();
	axisRelationPage.setObject( new com.klg.jclass.chart.JCAxis() );
    frame.getContentPane().add( axisRelationPage );
    frame.pack();
    java.awt.Dimension bigness =  axisRelationPage.getPreferredSize();
    frame.setSize( bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key207 );
}

public static String getPageName() {
    return "AxisRelationPage";
}

}
