/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisScalePage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:49 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Insets;
import java.awt.GridBagLayout;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.BorderFactory;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;

import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCChartEnumMappings;

/**
 *  Axis Property Page
 */
public class AxisScalePage extends JPropertyPage {

private JDoubleEditor axisMinField = null;
private JBooleanEditor axisMinDefault = null;
private JDoubleEditor axisMaxField = null;
private JBooleanEditor axisMaxDefault = null;
private JDoubleEditor originField = null;
private JBooleanEditor originDefault = null;

private JEnumEditor oPlaceCombo = null;
private JBooleanEditor oPlaceDefault = null;

private JCAxis target;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 5, 5, 5, 5 );
	constraints.weightx = 1.0;

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key219 ) + ":" ) );

	JPanel min = new JPanel();

    min.setLayout( new FlowLayout( FlowLayout.LEFT, 0, 4 ) );
	axisMinField = new JDoubleEditor( 8 );
    axisMinField.addPropertyChangeListener(this);
	min.add( axisMinField );
    axisMinDefault = new JBooleanEditor( ( JCustomizerBundle.string( JCustomizerBundle.key124 ) ) );
    axisMinDefault.addPropertyChangeListener(this);
	min.add( axisMinDefault );
	gridbag.setConstraints( min, constraints );
    add( min );

    add(new JLabel( JCustomizerBundle.string( JCustomizerBundle.key220 ) + ":" ) );
	
	JPanel max = new JPanel();

    max.setLayout( new FlowLayout( FlowLayout.LEFT, 0, 4 ) );
	axisMaxField = new JDoubleEditor( 8 );
    axisMaxField.addPropertyChangeListener(this);
	max.add( axisMaxField );
    axisMaxDefault = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key124 ) );
    axisMaxDefault.addPropertyChangeListener(this);
	max.add( axisMaxDefault );
	gridbag.setConstraints( max, constraints );
    add( max );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key236 ) + ":") );
     // Origin Placement combo
    JPanel panel = new JPanel();
    panel.setLayout( new FlowLayout( FlowLayout.LEFT, 1, 5 ) );
    oPlaceCombo = new JEnumEditor(JCChartEnumMappings.originPlacement_i18n_strings,
                                JCChartEnumMappings.originPlacement_values );
    oPlaceCombo.addPropertyChangeListener( this );
    panel.add( oPlaceCombo );
    oPlaceDefault = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key124 ) );
    oPlaceDefault.addPropertyChangeListener( this );
    panel.add( oPlaceDefault );
    gridbag.setConstraints( panel, constraints );
    add( panel );

    add(new JLabel( JCustomizerBundle.string( JCustomizerBundle.key221 ) + ":" ) );

	JPanel orig = new JPanel();

    orig.setLayout( new FlowLayout( FlowLayout.LEFT, 0, 4 ) );
    originField = new JDoubleEditor( 8 );
    originField.addPropertyChangeListener(this);
    orig.add( originField );
    originDefault = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key124 ) );
    originDefault.addPropertyChangeListener(this);
    orig.add( originDefault );
	gridbag.setConstraints( orig, constraints );
	add( orig );

}

/**
 * Method called to return the current target
 */
public Object getObject() {
    return target;
}

/**
 * Method called to initialize the page
 */
public void setObject(Object source) {
    boolean useDefault;
    Double  value;
    //System.out.println("AP setObject:"+source);
    if (source instanceof JCAxis) {
		target = (JCAxis) source;

		value = new Double(target.getMin());
		axisMinField.setValue(value);

		axisMinDefault.setValue( new Boolean( target.getMinIsDefault() ) );
		axisMinDefault.setEnabled( !target.getMinIsDefault() );

		value = new Double(target.getMax());
		axisMaxField.setValue(value);

		axisMaxDefault.setValue( new Boolean( target.getMaxIsDefault() ) );
		axisMaxDefault.setEnabled( !target.getMaxIsDefault() );

    	int originPlacement = target.getOriginPlacement();
	
    	oPlaceCombo.setValue(new Integer(originPlacement));
    	useDefault = target.getOriginPlacementIsDefault();
    	oPlaceDefault.setValue(new Boolean(useDefault));
    	oPlaceDefault.setEnabled( !useDefault );
    	oPlaceCombo.setEnabled( !useDefault );

		value = new Double(target.getOrigin());
		originField.setValue(value);
	
		originDefault.setValue( new Boolean( target.getOriginIsDefault() ) );
		originDefault.setEnabled( !target.getOriginIsDefault() );

    }
}

/**
 *  Property Change listener
 *  Called when a property editor is updated
 */
public void propertyChanged(Object source, Object value) {
    if (target == null) return;
    if (value instanceof Double) {
		if (source == axisMinField) {
	    	target.setMin(((Double)value).doubleValue());
		}
		else if (source == axisMaxField) {
	    	target.setMax(((Double)value).doubleValue());
		}
		else if (source == originField) {
	    	target.setOrigin(((Double)value).doubleValue());
		}
		this.setObject();
    } else if( value instanceof Boolean ) {
		if( source == axisMinDefault ) {
			target.setMinIsDefault(((Boolean)value).booleanValue());
		} else if( source == axisMaxDefault ) {
			target.setMaxIsDefault(((Boolean)value).booleanValue());
		} else if( source == originDefault ) {
			target.setOriginIsDefault(((Boolean)value).booleanValue());
        } else if (source == oPlaceDefault) {
        	boolean toggle = ((Boolean)value).booleanValue();
            target.setOriginPlacementIsDefault(toggle);
		}
		this.setObject();
    } else if (value instanceof Integer) {
    	if (source == oPlaceCombo) {
        	target.setOriginPlacement(((Integer)value).intValue());
		}
		this.setObject();
	}
}



/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java AxisScalePage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    AxisScalePage axisScalePage = new AxisScalePage();
    axisScalePage.setBackground( java.awt.Color.lightGray );
    axisScalePage.init();
	axisScalePage.setObject( new com.klg.jclass.chart.JCAxis() );
    frame.getContentPane().add( axisScalePage );
    frame.pack();
    java.awt.Dimension bigness = axisScalePage.getPreferredSize();
    frame.setSize( bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key222 );
}

public static String getPageName() {
    return "AxisScalePage";
}

}

