/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisTitlePage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:49 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Insets;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.FlowLayout;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.BorderFactory;

import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCAxisTitle;
import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.util.legend.JCLegendEnumMappings;

/**
 *  Axis Title Property Page
 */
public class AxisTitlePage extends JPropertyPage  {

private JFieldEditor titleField = null;
private JEnumEditor tRotCombo = null;
private JEnumEditor tPlaceCombo = null;
private JFontEditor titleFontCombo = null;
private JBooleanEditor tPlaceDefault = null;

private JCAxis target;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 5, 5, 5, 5 );
	constraints.weightx = 1.0;

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key109 ) + ":" ) );
    titleField = new JFieldEditor( "", 30 );
    titleField.addPropertyChangeListener( this );
	gridbag.setConstraints( titleField, constraints );
    add( titleField );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key175 ) + ":" ) );
    tRotCombo = new JEnumEditor( JCChartEnumMappings.annRotation_i18n_strings, 
									JCChartEnumMappings.annRotation_values );
    tRotCombo.addPropertyChangeListener( this );
	gridbag.setConstraints( tRotCombo, constraints );
    add( tRotCombo );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key202 ) + ":" ) );

	JPanel panel = new JPanel();

    panel.setLayout( new FlowLayout( FlowLayout.LEFT, 5, 5 ) );
	tPlaceCombo = new JEnumEditor( JCLegendEnumMappings.anchor_i18n_strings, 
										JCLegendEnumMappings.anchor_values ); 
    tPlaceCombo.addPropertyChangeListener( this );
    panel.add( tPlaceCombo );
    tPlaceDefault = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key124 ) );
    tPlaceDefault.addPropertyChangeListener( this );
    panel.add( tPlaceDefault );
	gridbag.setConstraints( panel, constraints );
	add( panel );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key112 ) + ":" ) );
    titleFontCombo = new JFontEditor();
    titleFontCombo.addPropertyChangeListener( this );
	gridbag.setConstraints( titleFontCombo, constraints );
    add( titleFontCombo );
}

/**
 * Method called to initialize the page
 */
public void setObject(Object source) {
    //System.out.println("AP setObject:"+source);
    if (source instanceof JCAxis) {
	target = (JCAxis) source;
	titleField.setValue(target.getTitle().getText());

	int titleRotation = target.getTitle().getRotation();
	tRotCombo.setValue(new Integer(titleRotation));

	int titlePlacement = target.getTitle().getPlacement();
	tPlaceCombo.setValue(new Integer(titlePlacement));

	boolean isDefault = target.getTitle().getPlacementIsDefault();
	tPlaceDefault.setValue(new Boolean(isDefault));
	tPlaceDefault.setEnabled( !isDefault );
	titleFontCombo.setValue(target.getTitle().getFont());
    }
}

/**
 * This method is called when a specialized property editor's 
 * value is changed.
 */
public void propertyChanged(Object source, Object value) {
    if (target == null) return;
	boolean doSet = false;
    if (source == titleField) {
	target.getTitle().setText((String)value, true);
    }
    else if (source == tRotCombo) {
	target.getTitle().setRotation(((Integer)value).intValue());
    } 
    else if (source == tPlaceCombo) {
	doSet = true;
	try {
	    target.getTitle().setPlacement(((Integer)value).intValue());
	} catch (Exception e1) {
	    target.getTitle().setPlacementIsDefault(true);
	}
    }
    else if (source == tPlaceDefault) {
	    target.getTitle().setPlacementIsDefault(((Boolean)value).booleanValue());
		doSet = true;
    }
    else if (source == titleFontCombo) {
	target.getTitle().setFont((java.awt.Font)value);
    }
	if (doSet)
		setObject(target);
	else
		this.setObject();
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java AxisTitlePage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    AxisTitlePage axisTitlePage = new AxisTitlePage();
    axisTitlePage.setBackground( java.awt.Color.lightGray );
    axisTitlePage.init();
	axisTitlePage.setObject( new com.klg.jclass.chart.JCAxis() );
    frame.getContentPane().add( axisTitlePage );
    frame.pack();
    java.awt.Dimension bigness =  axisTitlePage.getPreferredSize();
    frame.setSize( bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key203 );
}

public static String getPageName() {
    return "AxisTitlePage";
}

}

