/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: BarChartPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:49 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Insets;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.BorderFactory;

import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.JCBarChartFormat;
import com.klg.jclass.chart.ChartDataView;

/**
 *  Bar Chart Specific Property Page
 */
public class BarChartPage extends JPropertyPage {

private JBooleanEditor is100PercentCheck;
private JIntegerEditor clusterWidthField;
private JIntegerEditor clusterOverlapField;

private ChartDataView view = null;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 10, 10, 5, 10 );
	constraints.weightx = 1.0;
    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key177 ) + ":" ) );
    is100PercentCheck = new JBooleanEditor("");
    is100PercentCheck.addPropertyChangeListener( this );
	gridbag.setConstraints( is100PercentCheck, constraints );
    add( is100PercentCheck );

    add(new JLabel( JCustomizerBundle.string( JCustomizerBundle.key178 ) + ":" ) );
    clusterWidthField = new JIntegerEditor();
    clusterWidthField.addPropertyChangeListener( this );
	clusterWidthField.setMaximum( JCBarChartFormat.BAR_CLUSTER_WIDTH_MAX );
	gridbag.setConstraints( clusterWidthField, constraints );
    add( clusterWidthField );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key179 ) + ":" ) );
    clusterOverlapField = new JIntegerEditor();
    clusterOverlapField.addPropertyChangeListener(this);
	clusterOverlapField.setMinimum( JCBarChartFormat.BAR_CLUSTER_OVERLAP_MIN );
	clusterOverlapField.setMaximum( JCBarChartFormat.BAR_CLUSTER_OVERLAP_MAX );
	gridbag.setConstraints( clusterOverlapField, constraints );
    add( clusterOverlapField );
}

/**
 * Method called to populate the dialog with the source's properties
 */
public void setObject(Object source) {
    //System.out.println("DVGP setObject: "+source);
    if (source instanceof JCChart) {
	view = ((JCChart) source).getDataView(0);
    }
    else if (source instanceof ChartDataView) {
	view = (ChartDataView) source;
    }
    if (view != null && 
	(view.getChartType() == JCChart.BAR 
	 || view.getChartType() == JCChart.STACKING_BAR)) {
	JCBarChartFormat barFormat = (JCBarChartFormat) view.getChartFormat();
	if (barFormat != null) {	    
	    is100PercentCheck.setValue(new 
				Boolean(barFormat.is100Percent()));
	    is100PercentCheck.setEnabled( true );

	    clusterWidthField.setValue(new 
				Integer(barFormat.getClusterWidth()));
	    clusterWidthField.setEnabled( true );

	    clusterOverlapField.setValue(new 
				Integer(barFormat.getClusterOverlap()));
	    clusterOverlapField.setEnabled( true );
	}
    } else {
	is100PercentCheck.setEnabled( false );
	clusterWidthField.setEnabled( false );
	clusterOverlapField.setEnabled( false );
    }
}

/**
 * This method is called when a specialized property editor's 
 * value is changed.
 */
public void propertyChanged(Object source, Object value) {
    if (view == null) return;
	if (view.getChartType() != JCChart.BAR 
	   && view.getChartType() != JCChart.STACKING_BAR)
		return;
	JCBarChartFormat fmt = (JCBarChartFormat) view.getChartFormat();
    if (source == is100PercentCheck) {
	boolean toggle = ((Boolean)value).booleanValue();
	fmt.set100Percent(toggle);
    }
    else if (source == clusterWidthField) {
	Integer clusterWidth = (Integer) value;
	fmt.setClusterWidth(clusterWidth.intValue());
    }
    else if (source == clusterOverlapField) {
	Integer overlap = (Integer) value;
	fmt.setClusterOverlap(overlap.intValue());
    }
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java BarChartPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    BarChartPage barChartPage = new BarChartPage();
    barChartPage.setBackground( java.awt.Color.lightGray );
    barChartPage.init();
	barChartPage.setObject( new com.klg.jclass.chart.JCChart( JCChart.BAR ) );
    frame.getContentPane().add( barChartPage );
    frame.pack();
    java.awt.Dimension bigness = barChartPage.getPreferredSize();
    frame.setSize( bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key180 );
}

public static String getPageName() {
    return "BarChartPage";
}

}

