/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CandleChartPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:49 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;

import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.BorderFactory;

import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.JCCandleChartFormat;
import com.klg.jclass.chart.ChartDataView;

/**
 *  Candle Chart Specific Property Page
 */
public class CandleChartPage extends JPropertyPage {

private JBooleanEditor complexCheck;

private ChartDataView view = null;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 10, 10, 5, 10 );
	constraints.weightx = 1.0;
	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
    complexCheck = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key181 ) );
    complexCheck.addPropertyChangeListener( this );
	gridbag.setConstraints( complexCheck, constraints );
    add( complexCheck );
}

/**
 * Method called to populate the dialog with the source's properties
 */
public void setObject(Object source) {
    //System.out.println("DVGP setObject: "+source);
    if (source instanceof JCChart) {
	view = ((JCChart) source).getDataView(0);
    }
    else if (source instanceof ChartDataView) {
	view = (ChartDataView) source;
    }
    if (view != null && 
	(view.getChartType() == JCChart.CANDLE)) {
	JCCandleChartFormat candleFormat = (JCCandleChartFormat) view.getChartFormat();
	if (candleFormat != null) {	    
	    complexCheck.setSelected( candleFormat.isComplex() );
	    complexCheck.setEnabled( true );
	}
    } else {
	complexCheck.setEnabled( false );
    }
}

public void propertyChanged( Object source, Object value ) {
    if( source instanceof JBooleanEditor ) {
	JBooleanEditor box = (JBooleanEditor) source;
	if( box == null || view == null ||
			view.getChartType() != JCChart.CANDLE)
		return;
	if( box == complexCheck ) {
	    ((JCCandleChartFormat)view.getChartFormat()).
						setComplex( ((Boolean)value).booleanValue() );
	}
    }
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java CandleChartPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    CandleChartPage candleChartPage = new CandleChartPage();
    candleChartPage.setBackground( java.awt.Color.lightGray );
    candleChartPage.init();
	candleChartPage.setObject( new com.klg.jclass.chart.JCChart( JCChart.CANDLE ) );
    frame.getContentPane().add( candleChartPage );
    frame.pack();
    java.awt.Dimension bigness = candleChartPage.getPreferredSize();
    frame.setSize( bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key182 );
}

public static String getPageName() {
    return "CandleChartPage";
}

}

