/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ChartAreaGeneralPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:49 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Insets;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.BorderFactory;

import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.JCChartArea;
import com.klg.jclass.chart.JCChartEnumMappings;

/**
 *  ChartArea General Property Page
 */
public class ChartAreaGeneralPage extends JPropertyPage {

private JBooleanEditor visibleCheck;
private JBooleanEditor fastActionCheck;
private JFontEditor areaFont;
private JEnumEditor angleUnitCombo;

private JCChartArea chartArea = null;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 5, 5, 5, 5 );
	constraints.weightx = 1.0;

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key27 ) + ":" ) );
    visibleCheck = new JBooleanEditor("");
    visibleCheck.addPropertyChangeListener(this);
	gridbag.setConstraints( visibleCheck, constraints );
    add( visibleCheck );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key127 ) + ":" ) );
    fastActionCheck = new JBooleanEditor("");
    fastActionCheck.addPropertyChangeListener(this);
	gridbag.setConstraints( fastActionCheck, constraints );
    add( fastActionCheck );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key112 ) + ":" ) );
    areaFont = new JFontEditor();
    areaFont.addPropertyChangeListener( this );
	gridbag.setConstraints( areaFont, constraints );
    add( areaFont );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key62 ) + ":" ) );
    angleUnitCombo = new JEnumEditor(JCChartEnumMappings.angleUnit_i18n_strings,
    								 JCChartEnumMappings.angleUnit_values );
    angleUnitCombo.addPropertyChangeListener( this );
    gridbag.setConstraints( angleUnitCombo, constraints );
    add( angleUnitCombo );
}

/**
 * Method called to return the target that this page is editing
 */
public Object getObject() {
    return chartArea;
}

/**
 * Method called to populate the dialog with the source's properties
 */
public void setObject(Object source) {
    Integer value;
    //System.out.println("CAGP setObject: "+source);
    if (source instanceof JCChart) {
		chartArea = ((JCChart) source).getChartArea();
    }
    else if (source instanceof JCChartArea) {
		chartArea = (JCChartArea) source;
    }
    if (chartArea != null) {
		visibleCheck.setValue(new Boolean(chartArea.isVisible()));
		fastActionCheck.setValue(new Boolean(chartArea.getFastAction()));
		areaFont.setValue( chartArea.getFont() );
    	int angleUnit = chartArea.getAngleUnit();
    	angleUnitCombo.setValue(new Integer(angleUnit));
    }
}

/**
 * This method is called when a specialized property editor's
 * value is changed.
 */
public void propertyChanged(Object source, Object value) {
    if (chartArea == null || value == null) return;
    if (value instanceof Boolean) {
		boolean toggle = ((Boolean)value).booleanValue();
		if (source == visibleCheck) {
		    chartArea.setVisible(toggle);
		} else if (source == fastActionCheck) {
		    chartArea.setFastAction(toggle);
		}
    }
	else if (value instanceof Integer) {
		if (source == angleUnitCombo) {
			chartArea.setAngleUnit( ((Integer)value).intValue() );
		}
	}
    else if (source == areaFont) {
		chartArea.setFont((java.awt.Font)value);
    }
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java ChartAreaGeneralPage
 */
public static void main(String args[]) {
    JFrame frame;
    try {
	java.lang.Class newclass
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    ChartAreaGeneralPage chartAreaGeneralPage = new ChartAreaGeneralPage();
    chartAreaGeneralPage.setBackground( java.awt.Color.lightGray );
    chartAreaGeneralPage.init();
    frame.getContentPane().add( chartAreaGeneralPage );
    frame.pack();
    java.awt.Dimension bigness = chartAreaGeneralPage.getPreferredSize();
    frame.setSize(bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key128 );
}

public static String getPageName() {
    return "ChartAreaGeneralPage";
}

}

