/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ChartAxisList.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:50 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.util.Enumeration;

import javax.swing.JFrame;

import com.klg.jclass.chart.ChartDataView;
import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.JCChartArea;




import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.util.Arrays;


/**
 *  Axes Customizer Property Page
 */
public class ChartAxisList extends OutlinedPropertyPage {

private JCChart target = null;

/**
 * return a list of (Outline item label , item Class) pairs
 */
public String[] getOutlineDesc() {
    return axesOutline;
}

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
    setToolTipText( getPageTitle() );
    title.setText( JCustomizerBundle.string( JCustomizerBundle.key70 ) + ":" );
}

/**
 * Return the Object that this Property Page is showing
 */
public Object getObject() {
    return (Object) target;
}
/**
 * Method called to populate the dialog with the source's properties
 */
public void setObject( Object source ) {
    int index = 0;
	JCustomNode node = null;
	JCustomNode root = tree.getRoot();
    if( source instanceof JCChart ) {
		//System.out.println("CAL setObj with a chart");
		target = (JCChart) source;
		List views = target.getDataView();
		int xindex = 0;
		int yindex = 0;
		int treeIndex = 0;

		Iterator iter = views.iterator();
		for(int i = 0; iter.hasNext(); i++) {
			ChartDataView view = (ChartDataView) iter.next();
			JCustomNode viewNode = tree.findNode( root, view );

			if( viewNode == null ) {
				viewNode = new JCustomNode( view.getName() );
				tree.addNode( viewNode, root );
				viewNode.setUserData( (Object)view );
			}
			else {
				this.setNode( root, view.getName(), i, (Object)view );
			}
			treeIndex++;

			JCAxis xaxis = view.getXAxis();
			JCAxis yaxis = view.getYAxis();

			node = tree.findNode( root, xaxis );
			if( node == null ) {
				this.setNode( viewNode, "X"+ xindex + "(" + xaxis.getName() + ")", 0, (Object)xaxis );
			} else {
				this.setNode( viewNode, node.toString(), 0, (Object)xaxis);
			}
			xindex++;

			node = tree.findNode( root, yaxis );
			if( node == null ) {
				this.setNode( viewNode, "Y"+ yindex + "(" + yaxis.getName() + ")", 1, (Object) yaxis);
			} else {
				this.setNode( viewNode, node.toString(), 1, (Object) yaxis );
		    }
			yindex++;

			this.setNode( viewNode, null, 2, null ); // Clear remaining children
		}
		
		JCChartArea chartarea = target.getChartArea();
		List xaxes = chartarea.getXAxes();
		List yaxes = chartarea.getYAxes();

		if( xaxes != null ) {
			for( int i = 0; i < xaxes.size(); i++ ) {
				Object t = xaxes.get(i);
				if (t == null || !(t instanceof JCAxis)) continue;
				JCAxis xaxis = (JCAxis)t;

				node = tree.findNode( root, xaxis );
				if( node == null ) {
					JCAxis originator = xaxis.getFormula().getOriginator();
					
					if( originator != null ) {
						JCustomNode orig = tree.findNode( root, originator );
						
						if( orig != null ) {
							String label = " X" + xindex + " ("+
								xaxis.getName() + " " + 
								xaxis.getFormula().getMultiplier() + "x+" +
										   xaxis.getFormula().getConstant() + ")";
							
							node = new JCustomNode( label );
							xindex++;
							tree.addNode( node, (JCustomNode)orig.getParent() );
							node.setUserData( xaxis );
						} else {
							xindex++;
						}
					}
					else {
						this.setNode( root, "X" + xindex + "(" + xaxis.getName() + ")", treeIndex, (Object)xaxis );
						xindex++;
						treeIndex++;
					}
				// if node parent is root, then it was in the data view and
				// has already been counted.  If not, it needs to be counted.
				} else if (node.getParent() == root) {
						xindex++;
						treeIndex++;
				}
			}
		}
		if( yaxes != null ) {
			for( int i = 0; i < yaxes.size(); i++ ) {
				Object t = yaxes.get(i);
				if (t == null || !(t instanceof JCAxis)) continue;
				JCAxis yaxis = (JCAxis)t;

				node = tree.findNode( root, yaxis );
				if( node == null ) {
					JCAxis originator = yaxis.getFormula().getOriginator();

					if (originator != null) {
						JCustomNode orig = tree.findNode( root, originator );
						
						if( orig != null ) {
							String label = " Y" + yindex + " (" +
								yaxis.getName() + " " +
								yaxis.getFormula().getMultiplier()+ "y+" + 
								yaxis.getFormula().getConstant() + ")";
							node = new JCustomNode( label );
							yindex++;
							tree.addNode( node, (JCustomNode)orig.getParent() );
							node.setUserData( yaxis );
						} else {
							yindex++;
						}
					}
					else {
						this.setNode(root, "Y" + yindex + "(" + yaxis.getName() + ")", treeIndex, (Object)yaxis);
						yindex++;
						treeIndex++;
					}
				// if node parent is root, then it was in the data view and
				// has already been counted.  If not, it needs to be counted.
				} else if (node.getParent() == root) {
					yindex++;
					treeIndex++;
				}

			}
		} 
		this.setObject( (xaxes == null) ? ( (yaxes == null) ? null : yaxes.get(0) ) : xaxes.get(0) );
		initSelection( false );
    } else if( source instanceof JCChartArea ) {
		//System.out.println("CAL setObj with a chart area");
		JCChartArea chartarea = (JCChartArea) source;
		List xaxes = chartarea.getXAxes();
		List yaxes = chartarea.getYAxes();

		if( xaxes != null ) {
			for( index = 0; index < xaxes.size(); index++ )
				this.setNode( root, "X" + index, index, 
							  xaxes.get(index) );
		}
		if( yaxes != null ) {
			for( int j = 0; j < yaxes.size(); j++, index++ )
				this.setNode( root, "Y" + j, index, xaxes.get(j));
		}
		this.setNode( root, null, index, null ); // Clear the rest of the children
		this.setObject( (xaxes == null) ? ( (yaxes == null) ? null : yaxes.get(0) ) : xaxes.get(0) );
	} else if (source instanceof ChartDataView) {
		//System.out.println("CAL setObj with a cdv");
		ChartDataView view = (ChartDataView) source;
		JCAxis xaxis = view.getXAxis();
		JCAxis yaxis = view.getYAxis();
			
		if( xaxis != null ) {
			this.setNode( root, "X", index, (Object) xaxis );
			index++;
		}
		if (yaxis != null) {
			this.setNode( root, "Y", index, (Object) yaxis );
			index++;
		}
		this.setNode( root, null, index, null ); // Clear the rest of the children
		this.setObject((xaxis == null) ? yaxis : xaxis);
		addButton.setEnabled( false );
		removeButton.setEnabled( false );
	} else if( source instanceof JCAxis ) {
		//System.out.println("CAL setObj with an axis");
		JCAxis axis = (JCAxis)source;
		boolean found = false;
		node = tree.findNode( root, axis );
		if( node == null ) {
			node = new JCustomNode( "" + axis.getTitle().getText() );
			node.setUserData( (Object)axis );
			tree.addNode( node, root );
		}
		else {
			editor.setObject( axis );
		}
	}
}

/**
 * This method is called when a specialized property editor's 
 * value is changed.
 */
public void propertyChanged( Object source, Object value ) {
    if (target == null) return;
    if (source == dialog) {
	JCAxis axis = (JCAxis) value;
	if (dialog.getMode() == JAddRemoveDialog.MODE_REMOVE) {
	    //System.out.println("CAL propertyChanged() Remove "+axis);
	    Vector axes;
	    int i, j;
	    boolean found = false;
	    i = 0;
	    for (j=0; j<2; j++) {
			if (j==0) {
		    	axes = new Vector(target.getChartArea().getXAxes());
			} else {
		    	axes = new Vector(target.getChartArea().getYAxes());
			}
			if (axes != null) {
		    	for (i=0; i<axes.size(); i++) {
					if (axes.get(i) == axis) {
			    		found = true;
			    		break;
					}
		    	}
			}
			if (found) {
		    	axes.remove(i);
		    	Vector axisarr = new Vector(axes);
		    	if (j == 0) {
					target.getChartArea().setXAxes(axisarr);
				}
		    	else {
					target.getChartArea().setYAxes(axisarr);
				}
				JCustomNode root = tree.getRoot();
				JCustomNode node = tree.findNode( root, axis );
				tree.removeNode(node);
		    	JPropertyPage.getFrame(dialog).dispose();
		    	break;
			}
	    }
	}
	else if (dialog.getMode() == JAddRemoveDialog.MODE_ADD) {
	    // add value to current Object
	    //System.out.println("CAL propertyChanged() Add "+axis);
	    List axes = target.getChartArea().getXAxes();
	    if (axes != null && axes.size() >= 1 
			&& (((JCAxis)axes.get(0)).isVertical()==axis.isVertical())) {
			target.getChartArea().setXAxis(axes.size(), axis);
	    }
	    else {
			axes = target.getChartArea().getYAxes();
			int index = (axes == null) ? 0 : axes.size();
			target.getChartArea().setYAxis(index, axis);
	    }
	    JPropertyPage.getFrame(dialog).dispose();
	}
	// Update
	setObject();
    }	
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java ChartAxisList
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		//System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    ChartAxisList chartAxisList = new ChartAxisList();
    chartAxisList.setBackground( java.awt.Color.lightGray );
    chartAxisList.init();
	chartAxisList.setObject( new com.klg.jclass.chart.JCChart() );
    frame.getContentPane().add( chartAxisList );
    frame.pack();
    java.awt.Dimension bigness = chartAxisList.getPreferredSize();
    frame.setSize( bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key192 );
}

public static String getPageName() {
    return "ChartAxisList";
}

}

