/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ChartBorderPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:50 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Insets;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Container;

import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.BorderFactory;
import javax.swing.border.Border;
import javax.swing.JComponent;

import com.klg.jclass.chart.ChartRegion;
import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.chart.Changeable;
import com.klg.jclass.chart.JCChartLabel;

import com.klg.jclass.beans.BorderEditorPanel;

/**
 *  Chart Border Property Page
 */
public class ChartBorderPage extends JPropertyPage {

private BorderEditorPanel borderEditor;

private JComponent comp = null;
private JCChartLabel chartLabel = null;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 10, 10, 5, 10 );
	constraints.weightx = 1.0;

    borderEditor = new BorderEditorPanel(); 
    borderEditor.addPropertyChangeListener( this );
	gridbag.setConstraints( borderEditor, constraints );
    add( borderEditor );
}

/**
 * Method called to populate the dialog with the source's properties
 */
public void setObject(Object source) {
    //System.out.println("CBP setObject: "+source);
    if (source instanceof JComponent) {
		comp = (JComponent) source;
		chartLabel = null;
    } else if (source instanceof JCChartLabel) {
		chartLabel = ((JCChartLabel) source);
		comp = chartLabel.getComponent();
	}
	if (comp != null)
		borderEditor.setValue(comp);
}

/**
 * Return the Object that this Property Page is editing
 */
public Object getObject() {
	return (Object) comp;
}

/**
 * This method is called when a specialized property editor's 
 * value is changed.
 */
public void propertyChanged(Object source, Object value) {
    if (value == null) return;

    if (comp != null) {
		if (source == borderEditor) {
	    	comp.setBorder((Border)value);
			Container parent = comp.getParent();
			if (chartLabel != null) {
                chartLabel.setChanged(true, Changeable.RECALC);
			} else if (comp instanceof ChartRegion) {
				((ChartRegion)comp).setChanged(true, Changeable.RECALC);
			} else if (comp instanceof JCChart) {
				comp.invalidate();
				comp.validate();
			} else if (parent != null && parent instanceof JCChart) {
				comp.invalidate();
				parent.invalidate();
				parent.validate();
			}
		}
    }
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java ChartAreaBorderPage
 */
public static void main(String args[]) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    ChartBorderPage chartBorderPage = new ChartBorderPage();
    chartBorderPage.setBackground( java.awt.Color.lightGray );
    chartBorderPage.init();
    frame.getContentPane().add( chartBorderPage );
    frame.pack();
    java.awt.Dimension bigness = chartBorderPage.getPreferredSize();
    frame.setSize(bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key165 );
}

public static String getPageName() {
    return "ChartBorderPage";
}

}

