/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ChartColorPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:50 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.JComponent;

import com.klg.jclass.chart.PlotArea;
import com.klg.jclass.chart.JCChartLabel;
import com.klg.jclass.chart.JCChartArea;

/**
 *  Chart Color Property Page
 */
public class ChartColorPage extends JPropertyPage {

private JColorEditor backgroundColor = null;
private JColorEditor foregroundColor = null;
private JBooleanEditor opaqueCheck = null;
private JComponent component = null;
private PlotArea plotArea = null;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 20, 10, 10, 10 );
	constraints.weightx = 1.0;

    opaqueCheck = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key123 ));
    opaqueCheck.addPropertyChangeListener(this);
	gridbag.setConstraints( opaqueCheck, constraints );
	add( opaqueCheck );

	JPanel backgroundPanel = new JPanel();
	backgroundPanel.setLayout( new FlowLayout( FlowLayout.LEFT ) );
	JPanel foregroundPanel = new JPanel();
	foregroundPanel.setLayout( new FlowLayout( FlowLayout.LEFT ) );

    backgroundColor = new JColorEditor( textBG );
    backgroundColor.addPropertyChangeListener( this );
	foregroundColor = new JColorEditor( textBG );
    foregroundColor.addPropertyChangeListener( this );
    backgroundPanel.add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key20 ) ) );
	backgroundPanel.add( backgroundColor );
    foregroundPanel.add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key21 ) ) );
	foregroundPanel.add( foregroundColor );
	gridbag.setConstraints( backgroundPanel, constraints );
	add( backgroundPanel );
	gridbag.setConstraints( foregroundPanel, constraints );
	add( foregroundPanel );
}

/**
 * Method called to populate the dialog with the source's properties
 */
public void setObject( Object source ) {
    //System.out.println("CCP setObject: "+source);
    if (source instanceof JComponent || source instanceof JCChartLabel) {
    	if (source instanceof JComponent)
			component = (JComponent) source;
		else if (source instanceof JCChartLabel)
			component = ((JCChartLabel) source).getComponent();
	opaqueCheck.setValue(new Boolean(component.isOpaque()));
	backgroundColor.setValue( component.getBackground() );
	foregroundColor.setValue( component.getForeground() );
    }
    else if (source instanceof PlotArea) {
	plotArea = (PlotArea) source;
	opaqueCheck.setValue(new Boolean(false));
	opaqueCheck.setVisible(false);
	backgroundColor.setValue(plotArea.getBackground());
	foregroundColor.setValue(plotArea.getForeground());
    }
}

/**
 * Return the Object that this Property Page is editing
 */
public Object getObject() {
	//System.out.println("CCP getObject: ");
    if (plotArea == null) {
	   	return (Object) component;
    } else
		return (Object) plotArea;	 
}

/**
 * This method is called when a specialized property editor's 
 * value is changed.
 */
public void propertyChanged( Object source, Object value)  {
    if (source == opaqueCheck) {
		if (plotArea != null) {
		} else if (component != null) {
	    	component.setOpaque( ((Boolean)value).booleanValue() );
			// force repaint on all but chart area -- it does it's own
			if (!(component instanceof JCChartArea)) {
				component.invalidate();
				if (component.getParent() != null)
					component.getParent().validate();
			}
		}
    } else if (source == backgroundColor) {
		if (plotArea != null) {
	    	plotArea.setBackground( (Color)backgroundColor.getValue() );
		} else if (component != null) {
	    	component.setBackground( (Color)backgroundColor.getValue() );
		}
    }
    else if (source == foregroundColor) {
		if (plotArea != null) {
	    	plotArea.setForeground( (Color)foregroundColor.getValue() );
		} else if (component != null) {
	    	component.setForeground( (Color)foregroundColor.getValue() );
		}
    }
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java ChartColorPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    ChartColorPage chartColorPage = new ChartColorPage();
    chartColorPage.setBackground(java.awt.Color.lightGray);
    chartColorPage.init();
    frame.getContentPane().add( chartColorPage );
    frame.pack();
    java.awt.Dimension bigness = chartColorPage.getPreferredSize();
    frame.setSize(bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key22 );
}

public static String getPageName() {
    return "ChartColorPage";
}

}

