/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ChartCustomizerFrame.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:50 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.JCChartFactory;
import com.klg.jclass.chart.property.PropertyAccessModel;
import com.klg.jclass.chart.property.PropertyLoadFactory;
import com.klg.jclass.chart.property.PropertyLoadModel;
import com.klg.jclass.chart.property.html.JCHTMLFileAccessor;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

public class ChartCustomizerFrame extends JFrame implements ActionListener {

public final static int NONE = 0;
public final static int LOAD = 1;
public final static int APPLY = 2;
public final static int SAVE = 3;

private static boolean exitOnClose = false;
private static int instances = 0;
private JMenuItem loadItem, applyItem, saveItem, closeItem;

private FileDialog dialog = null;
private int dialogAction = NONE;

protected static Vector frames = new Vector();

public ChartCustomizerFrame()
{
    super();
    init();
}

public ChartCustomizerFrame( String s )
{
    super( s );
    init();
}

private void init()
{
    addInstance();

    JMenuBar menubar = new JMenuBar();
    this.setJMenuBar( menubar );
    JMenu file = new JMenu( JCustomizerBundle.string( JCustomizerBundle.key106 ) );

    menubar.add( file );

    loadItem = new JMenuItem( JCustomizerBundle.string( JCustomizerBundle.key53 ) );
	loadItem.addActionListener(this);
	file.setMnemonic( JCustomizerBundle.string( JCustomizerBundle.key308 ).charAt( 0 ) );
    file.add( loadItem );

    applyItem = new JMenuItem( JCustomizerBundle.string( JCustomizerBundle.key29 ) );
	applyItem.addActionListener(this);
    file.add( applyItem );

    saveItem = new JMenuItem( JCustomizerBundle.string( JCustomizerBundle.key107 ) );
	saveItem.addActionListener(this);

    file.add( saveItem );
    file.addSeparator();
    closeItem = new JMenuItem( JCustomizerBundle.string( JCustomizerBundle.key108 ) );
	closeItem.addActionListener( new ActionListener() {
		public void actionPerformed( ActionEvent e ) {
			setVisible( false );
		}
	});
    file.add( closeItem );

	JMenu lookAndFeel = new JMenu( JCustomizerBundle.string( JCustomizerBundle.key310 ) );
	JMenuItem windowsItem = new JMenuItem( JCustomizerBundle.string( JCustomizerBundle.key311 ) );
	JMenuItem motifItem = new JMenuItem( JCustomizerBundle.string( JCustomizerBundle.key312 ) );
	JMenuItem metalItem = new JMenuItem( JCustomizerBundle.string( JCustomizerBundle.key313 ) );

	lookAndFeel.setMnemonic( JCustomizerBundle.string( JCustomizerBundle.key309 ).charAt( 0 ) );
	lookAndFeel.add( windowsItem );
	lookAndFeel.add( motifItem );
	lookAndFeel.add( metalItem );
	menubar.add( lookAndFeel );
	windowsItem.addActionListener( new ActionListener() {
		public void actionPerformed( ActionEvent e ) {
			setFramesLookAndFeel( "com.sun.java.swing.plaf.windows.WindowsLookAndFeel" );
		}
	});
	motifItem.addActionListener( new ActionListener() {
		public void actionPerformed( ActionEvent e ) {
			setFramesLookAndFeel( "com.sun.java.swing.plaf.motif.MotifLookAndFeel" );
		}
	});
	metalItem.addActionListener( new ActionListener() {
		public void actionPerformed( ActionEvent e ) {
			setFramesLookAndFeel( "javax.swing.plaf.metal.MetalLookAndFeel" );
		}
	});
}

public void actionPerformed( ActionEvent e )
{
	Object source = e.getSource();
	if (source == saveItem || source == loadItem || source == applyItem) {
		if (dialog == null) {
			dialog = new FileDialog(new JFrame());
		}
		if (source == saveItem) {
			dialog.setTitle(JCustomizerBundle.string( JCustomizerBundle.key36 ));
			dialog.setMode(FileDialog.SAVE);
			dialogAction = SAVE;
		} else if (source == loadItem) {
			dialog.setTitle(JCustomizerBundle.string( JCustomizerBundle.key35 ));
			dialog.setMode(FileDialog.LOAD);
			dialogAction = LOAD;
		} else {
			dialog.setTitle(JCustomizerBundle.string( JCustomizerBundle.key35 ));
			dialog.setMode(FileDialog.LOAD);
			dialogAction = APPLY;
		}
		dialog.setVisible(true);
		String directory = dialog.getDirectory();
		String file = dialog.getFile();
		if (directory != null && file != null && !(file.equals("")))
			fileAction(directory+file);
	}
}

/**
 *  Process the selected file with respect to the expected action -- load,
 *  apply or save from html.
 */
protected void fileAction(String fullFileName)
{
	if (dialogAction == SAVE) {
		JCChart chart = (JCChart) getObject();
		try {
            JCChartFactory.setOutputDataPropertiesOnChart(chart,
                                                          JCChartFactory.EMBED_DATA,
                                                          null);
			JCChartFactory.saveChartToFile(chart, fullFileName,
			                               JCChartFactory.HTML);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	else if (dialogAction == LOAD) {
		JCChart oldChart = (JCChart) getObject();
		JCChart newChart = null;
		String newName = oldChart.getName();
		if (newName == null) {
			newName = "";
		}

		try {
			newChart = JCChartFactory.makeChartFromFile(fullFileName, newName,
			                                            JCChartFactory.HTML);
		}
		catch (Exception e) {
			e.printStackTrace();
			newChart = new JCChart();
			newChart.setName(newName);
		}

		Container chartParent = oldChart.getParent();
		chartParent.remove(oldChart);
		chartParent.add(newChart);
		newChart.customizer = oldChart.customizer;
		oldChart.customizer = null;
		setObject(newChart);
		/*
		java.awt.Frame f =new java.awt.Frame();
		f.setLayout(new java.awt.GridLayout(1,1));
		f.add(newChart);
		f.pack();
		f.show();
		*/
		chartParent.validate();
	}
	else if (dialogAction == APPLY) {
		JCChart chart = (JCChart) getObject();
		if (chart.getName() == null) {
			chart.setName("");
		}
		PropertyAccessModel pam = new JCHTMLFileAccessor(chart);
		try {
            pam.getProperties(fullFileName);
		}
		catch (Exception e) {     // Problems accessing file
			e.printStackTrace();
		}
		PropertyLoadModel loader = PropertyLoadFactory.makeLoader(chart);
		if (loader != null) {
			try {
			    loader.loadProperties(pam, null);
			}
			catch (Exception e) { // Problems accessing data file
				e.printStackTrace();
			}
		}
	}
}

public void setExitOnClose(boolean exit)
{
    exitOnClose = exit;
}

public void setObject( Object chart )
{
    JPropertyPage child = getPropertyPageChild();

    if (child != null) {
		child.setObject(chart);
    }
}

private JPropertyPage getPropertyPageChild()
{
    JPropertyPage answer = null;
    Component comps[] = getContentPane().getComponents();
    for ( int i = 0; i < comps.length; i++ ) {
	    Component child = comps[ i ];
	    if( child instanceof JPropertyPage ) {
	        answer = (JPropertyPage)child;
	        break;
	    }
	    else if( child instanceof Container ) {
	        answer = getPropertyPageChild();
		}
    }
    return(answer);
}

public Object getObject()
{
    JPropertyPage child = getPropertyPageChild();

    if (child == null) {
		return null;
    }
	else {
		return child.getObject();
    }
}

public synchronized void dispose()
{
    Component [] comps = getComponents();
    for(int i = 0; i < comps.length; i++) {
	if (comps[i] instanceof JApplet)
	    ((JApplet)comps[i]).destroy();
    }
    super.dispose();
    removeInstance();
}

private void addInstance()
{
	setDefaultCloseOperation( DISPOSE_ON_CLOSE );
    instances++;
	frames.addElement( this );
}

private void removeInstance()
{
    instances--;
    if (instances == 0 && exitOnClose ) {
		System.exit(0);
    }
}

private static void setFramesLookAndFeel( String lookAndFeelClass )
{
	int pagesSize = JPropertyPage.pages.size();
	int framesSize = frames.size();
	StringBuffer exception = new StringBuffer();

	((JFrame)frames.elementAt( 0 )).setCursor( Cursor.getPredefinedCursor( Cursor.WAIT_CURSOR ) );

	for( int i = 0; i < pagesSize; i++ ) {
		JPropertyPage page = (JPropertyPage)JPropertyPage.pages.elementAt( i );

		if( !lookAndFeel( lookAndFeelClass, page, exception ) ) {
			JOptionPane.showMessageDialog( null, JCustomizerBundle.string( JCustomizerBundle.key314 ) + "\n" + exception.toString(),
										   JCustomizerBundle.string( JCustomizerBundle.key315 ), JOptionPane.ERROR_MESSAGE );
			return;
		}
	}
	for( int i = 0; i < framesSize; i++ ) {
		JFrame frame = (JFrame)frames.elementAt( i );

		if( !lookAndFeel( lookAndFeelClass, frame, exception ) ) {
			JOptionPane.showMessageDialog( null, JCustomizerBundle.string( JCustomizerBundle.key314 ) + "\n" + exception.toString(),
										   JCustomizerBundle.string( JCustomizerBundle.key315 ), JOptionPane.ERROR_MESSAGE );
			return;
		}
	}
	((JFrame)frames.elementAt( 0 )).setCursor( Cursor.getPredefinedCursor( Cursor.DEFAULT_CURSOR ) );
}

private static boolean lookAndFeel( String lookAndFeelClass, Component c, StringBuffer exception )
{
	boolean error = false;

	if( c != null ) {
		try {
			UIManager.setLookAndFeel( lookAndFeelClass );
			SwingUtilities.updateComponentTreeUI( c );
		}
		catch( ClassNotFoundException cfe ) {
			error = true;
			exception.append( "ClassNotFoundException " + cfe.getMessage() );
		}
		catch( InstantiationException  ie ) {
			error = true;
			exception.append( "InstantiationException " + ie.getMessage() );
		}
		catch( IllegalAccessException iae ) {
			error = true;
			exception.append( "IllegalAccessException " + iae.getMessage() );
		}
		catch( UnsupportedLookAndFeelException ulfe ) {
			error = true;
			exception.append( "UnsupportedLookAndFeelException " + ulfe.getMessage() );
		}
	}
	return(!error);
}

}
