/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ChartGeneralPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:50 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Container;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.BorderFactory;

import com.klg.jclass.chart.JCChart;

/**
 *  General Chart Property Page
 */
public class ChartGeneralPage extends JPropertyPage {

private JBooleanEditor doubleBufferedCheck = null;
private JBooleanEditor batchedCheck = null;
private JFieldEditor nameField = null;
private JCChart target;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
    setLayout( new BorderLayout() );
    setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );

    JPanel controls = new JPanel();
	GridBagLayout gridbag = new GridBagLayout();
	GridBagConstraints gbc = new GridBagConstraints();
    controls.setLayout(gridbag);
	gbc.anchor = GridBagConstraints.WEST;
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	gbc.insets = new Insets(5, 10, 5, 10);
 	gbc.weightx = 1.0;
	gbc.gridy = 0;

    controls.add( new JLabel(JCustomizerBundle.string( JCustomizerBundle.key30 ) + ":" ) );

    nameField = new JFieldEditor("", 14);
    nameField.addPropertyChangeListener(this);
	gridbag.setConstraints(nameField, gbc);
    controls.add( nameField );
	gbc.gridy++;

    doubleBufferedCheck = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key14 ) );
    doubleBufferedCheck.addPropertyChangeListener(this);
	gridbag.setConstraints(doubleBufferedCheck, gbc);
    controls.add( doubleBufferedCheck );
	gbc.gridy++;

    batchedCheck = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key15 ));
    batchedCheck.addPropertyChangeListener(this);
	gridbag.setConstraints(batchedCheck, gbc);
    controls.add( batchedCheck );

//    ChartAboutPage about = new ChartAboutPage();
//    about.init();

    add("North", controls);
//    add("South", about);
}

/**
 * Method called to initialize the page
 */
public void setObject(Object source) {
    Integer value;
    if (source instanceof JCChart) {
		target = (JCChart) source;
		String name = target.getName();
		if (name == null)
			name = new String();
		nameField.setValue(name);
		doubleBufferedCheck.setValue(new Boolean(target.isDoubleBuffered()));
		batchedCheck.setValue(new Boolean(target.isBatched()));
    }
}

public void propertyChanged(Object source, Object value) {
    if (target == null) return;
    if (source == doubleBufferedCheck) {
		boolean db = ((Boolean)value).booleanValue();
		//System.out.println("Setting DoubleBuffer to: "+db);
		target.setDoubleBuffered(db);
    } else if (source == batchedCheck) {
		boolean batched = ((Boolean)value).booleanValue();
		target.setBatched(batched);
    } else if (source == nameField) {
		target.setName((String) value);
    }
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java ChartGeneralPage
 */
public static void main(String args[]) {
    JFrame frame;
    try {
	java.lang.Class newclass
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    ChartGeneralPage chartGeneralPage = new ChartGeneralPage();
    chartGeneralPage.setBackground( java.awt.Color.lightGray );
    chartGeneralPage.init();
    frame.getContentPane().add( chartGeneralPage );
    frame.pack();
    java.awt.Dimension bigness = chartGeneralPage.getPreferredSize();
    frame.setSize(bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key4 );
}

public static String getPageName() {
    return "ChartGeneralPage";
}

}
