/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ChartLabelLocationPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:50 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.BorderFactory;

import com.klg.jclass.chart.JCChartLabel;
import com.klg.jclass.chart.JCDataCoord;
import com.klg.jclass.chart.JCDataIndex;
import com.klg.jclass.chart.Changeable;
import com.klg.jclass.chart.JCChartEnumMappings;

/**
 *  Chart Label Location and Size Customizing Page
 */
public class ChartLabelLocationPage extends JPropertyPage implements ActionListener {

private JEnumEditor anchorCombo;
private JIntegerEditor xoffset;
private JIntegerEditor yoffset;
private JIntegerEditor widthField;
private JIntegerEditor heightField;
private JButton resetSizeButton;

private JCChartLabel target;

private Dimension currentPS = new Dimension(-1, -1);

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
    setLayout( new FlowLayout( FlowLayout.LEFT, 2, 3 ) );

    JPanel labelControl = new JPanel();
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	labelControl.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	labelControl.setLayout( gridbag );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 5, 10, 5, 10 );
	constraints.weightx = 1.0;

    add( labelControl );
    labelControl.add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key252 ) + ":" ) );
    anchorCombo = new JEnumEditor(JCChartEnumMappings.chartLabelAnchor_i18n_strings,
								JCChartEnumMappings.chartLabelAnchor_values );
    anchorCombo.addPropertyChangeListener( this );
	gridbag.setConstraints( anchorCombo, constraints );
    labelControl.add( anchorCombo );
    labelControl.add( new JLabel( "X " + JCustomizerBundle.string( JCustomizerBundle.key253 ) + ":" ) );
    xoffset = new JIntegerEditor( 4 );
    xoffset.setMinimum( -9999 );
    xoffset.addPropertyChangeListener( this );
	gridbag.setConstraints( xoffset, constraints );
    labelControl.add( xoffset );

    labelControl.add( new JLabel( "Y " + JCustomizerBundle.string( JCustomizerBundle.key253 ) + ":" ) );
    yoffset = new JIntegerEditor( 4 );
    yoffset.setMinimum( -9999 );
    yoffset.addPropertyChangeListener( this );
	gridbag.setConstraints( yoffset, constraints );
    labelControl.add( yoffset );
    add( labelControl );

    JPanel labelLocation = new JPanel();
    
	labelLocation.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	labelLocation.setLayout( gridbag );

    labelLocation.add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key116 ) + ":" ) );
    widthField = new JIntegerEditor(4);
    widthField.addPropertyChangeListener(this);
	gridbag.setConstraints( widthField, constraints );
    labelLocation.add( widthField );

    // Add some padding to this label to ensure enough space
    labelLocation.add( new JLabel("  " + JCustomizerBundle.string( JCustomizerBundle.key125 ) + ":" ) );
    heightField = new JIntegerEditor(4);
    heightField.addPropertyChangeListener( this );
	gridbag.setConstraints( heightField, constraints );
    labelLocation.add( heightField );

    // Reset pref size?
    labelLocation.add( new JLabel(""));
    labelLocation.add( resetSizeButton = new JButton(
    		JCustomizerBundle.string( JCustomizerBundle.key254 )) );
    resetSizeButton.addActionListener( this );

    add( labelLocation );
}

/**
 * Return the object being edited
 */
public Object getObject() {
    return target;
}

/**
 * Method called to update the page with values from the supplied source
 */
public void setObject(Object source) {
    //System.out.println("CLabelLP setObject: "+source);
    if (source instanceof JCChartLabel) {
	target = (JCChartLabel) source;
	//System.out.println("label.anchor: "+target.getAnchor());
	anchorCombo.setValue(new Integer(target.getAnchor()));

	Point offset = target.getOffset();
	xoffset.setValue(new Integer(offset.x));
	yoffset.setValue(new Integer(offset.y));

	JComponent comp = target.getComponent();
	if (comp != null) {
		currentPS = comp.getPreferredSize();
		widthField.setValue(new Integer(currentPS.width));
		heightField.setValue(new Integer(currentPS.height));
	}
    } 
}

/**
 *  Property Change listener
 *  Called when a property editor is updated
 */
public void propertyChanged(Object source, Object value) {
    if (target == null) return;
	if (source == anchorCombo) {
	int anchor = ((Integer)value).intValue();
	target.setAnchor(anchor);
    }
    else if (source == xoffset || source == yoffset) {
	int val1;
	int val2;
	if (source == xoffset) {
	    val1 = ((Integer)value).intValue();
	    val2 = ((Integer)yoffset.getValue()).intValue();
	} else {
	    val1 = ((Integer)xoffset.getValue()).intValue();
	    val2 = ((Integer)value).intValue();
	}
	target.setOffset(new Point(val1, val2));
    } else if (source == widthField || source == heightField) {
		int val1;
		int val2;
		// width and height are intrinsically linked, but we can't rely
		// on the correct value of the other half having been put in the field
		// yet.  Grab it from the current preferred size.
		if (source == widthField) {
	    	val1 = ((Integer)value).intValue();
			val2 = currentPS.height;
		} else {
			val1 = currentPS.width;
	    	val2 = ((Integer)value).intValue();
		}
		JComponent comp = target.getComponent();
		if (comp != null) {
			// only set preferred size if someone has changed it
			if (currentPS.width != val1 || currentPS.height != val2) {
				currentPS = new Dimension(val1, val2);
				comp.setPreferredSize(new Dimension(currentPS));
				target.setChanged(true, Changeable.RECALC);
			}
		}
    }
    this.setObject();
}

public void actionPerformed(ActionEvent evt) {
	Object source = evt.getSource();
    if (target == null) return;

	if (source == resetSizeButton) {
		JComponent comp = target.getComponent();
		if (comp != null) {
			// reset preferred size
			comp.setPreferredSize(null);
			target.setChanged(true, Changeable.RECALC);
			currentPS = comp.getPreferredSize();
		}
    }
    this.setObject();
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java ChartLabelLocationPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    ChartLabelLocationPage chartLabelLocationPage = new ChartLabelLocationPage();
    chartLabelLocationPage.setBackground( java.awt.Color.lightGray );
    chartLabelLocationPage.init();
    frame.getContentPane().add( chartLabelLocationPage );
    frame.pack();
    java.awt.Dimension bigness = chartLabelLocationPage.getPreferredSize();
    frame.setSize(bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key256 );
}

public static String getPageName() {
    return "ChartLabelLocationPage";
}

}

