/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ChartTriggerPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:50 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;

import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.DefaultTableModel;

import com.klg.jclass.chart.EventTrigger;
import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.JCChartEnumMappings;

/**
 *  Chart Trigger Property Page
 */
public class ChartTriggerPage extends JPropertyPage implements ActionListener {

private JCustomTable triggerList = null;
private ComboBoxRenderer comboBoxRenderer = null;
private JButton applyButton = null;
private JCChart target = null;

final static String[] triggerList_columns = { 
    JCustomizerBundle.string( JCustomizerBundle.key262 ), 
	JCustomizerBundle.string( JCustomizerBundle.key263 ),
	JCustomizerBundle.string( JCustomizerBundle.key302 )
};

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
    setLayout( new BorderLayout() );
	setBorder( BorderFactory.createEmptyBorder( 10, 10, 10, 10 ) );
	int rows = JCChartEnumMappings.eventMod_strings.length;
	JComboBox actionCombo = new JComboBox();

    triggerList = new JCustomTable();
	triggerList.setRowHeight( 20 );
	triggerList.setIntercellSpacing( new java.awt.Dimension( 3, 3 ) );
	triggerList.setToolTipText( JCustomizerBundle.string( JCustomizerBundle.key303 ) );
	triggerList.setModel( new DefaultTableModel( new Object[ rows ][ 3 ], triggerList_columns ) {
		public boolean isCellEditable( int rowIndex, int columnIndex ) {
			return columnIndex == 1 ? false : true;
		}
	});
	triggerList.getTableHeader().setReorderingAllowed(false);
	comboBoxRenderer = new ComboBoxRenderer();
	for( int i = 0; i < JCChartEnumMappings.trigger_values.length; i++ ) {
		actionCombo.addItem( JCChartEnumMappings.trigger_i18n_strings[ i ] );
		comboBoxRenderer.addItem( JCChartEnumMappings.trigger_i18n_strings[ i ] );
	}
	TableColumn column0 = triggerList.getColumn( JCustomizerBundle.string( JCustomizerBundle.key262 ) );
	TableColumn column1 = triggerList.getColumn( JCustomizerBundle.string( JCustomizerBundle.key263 ) );
	TableColumn column2 = triggerList.getColumn( JCustomizerBundle.string( JCustomizerBundle.key302 ) );

	column0.setCellEditor( new DefaultCellEditor( actionCombo ) );
	column2.setCellEditor( new DefaultCellEditor( new JCheckBox() ) );
	column0.setCellRenderer( comboBoxRenderer );
	column2.setCellRenderer( new CheckBoxRenderer() );
	for( int i = 0; i < rows; i++ ) {
		triggerList.setValueAt( new String( JCChartEnumMappings.trigger_i18n_strings[ 0 ] ), i, 0 );
		triggerList.setValueAt( JCChartEnumMappings.eventMod_i18n_strings[ i ], i, 1 );
		triggerList.setValueAt( new Boolean( false ), i, 2 );
	}
	add( triggerList.createTablePanel(), BorderLayout.CENTER );
	applyButton = new JButton( JCustomizerBundle.string( JCustomizerBundle.key247 ) );
	applyButton.addActionListener( this );
	add( applyButton, BorderLayout.SOUTH );
}

/**
 * Method called to return the object of the page
 */
public Object getObject() {
    return target;
}

/**
 * Method called to set the contents of the page
 */
public void setObject(Object source) {
    //System.out.println("CTP setObject:"+source);
    if( source instanceof JCChart ) {
		target = (JCChart) source;

		int numtriggers = target.getNumTriggers();
		for( int i = 0; i < numtriggers; i++ ) {
			EventTrigger trigger = target.getTrigger( i );

			if( trigger != null ) {
				int row = getModifierIndex( trigger.getModifiers() );

				triggerList.setValueAt( new String( JCChartEnumMappings.trigger_i18n_strings[ getActionIndex( trigger.getAction() ) ] ), 
													row, 0 );
				triggerList.setValueAt( new Boolean( true ), row, 2 );
			}
		}
    }
}

// ActionListener method
public void actionPerformed( ActionEvent ev ) {
    if (target == null) return;
    if( ev.getSource() instanceof JButton ) {
		JButton button = (JButton) ev.getSource();

		if( button == applyButton ) {
			for( int i = 0; i < target.getNumTriggers(); i++ ) {
				target.removeTrigger( i );
			}
			int triggerNo = 0;
			for( int row = 0; row < triggerList.getRowCount(); row++ ) {
				if( ((Boolean)triggerList.getValueAt( row, 2 )).booleanValue() ) {
					int action = getActionValue( triggerList.getValueAt( row, 0 ).toString() );
					int modifier = JCChartEnumMappings.eventMod_values[ row ];
					
					if( action != -1 ) {
						target.setTrigger( triggerNo++, new EventTrigger( modifier, action ) );
					}
				}
			}
		}
	}
}

/**
 * This method is called when a specialized property editor's 
 * value is changed.
 */
public void propertyChanged(Object source, Object value) {
}

private int getModifierIndex( int modifier ) {
	int index = 0;

	for( int i = 0; i < JCChartEnumMappings.eventMod_values.length; i++ ) {
		if( modifier == JCChartEnumMappings.eventMod_values[ i ] ) {
			index = i;
			break;
		}
	}
	return index;
}

private int getActionIndex( int action ) {
	int index = 0;

	for( int i = 0; i < JCChartEnumMappings.trigger_values.length; i++ ) {
		if( action == JCChartEnumMappings.trigger_values[ i ] ) {
			index = i;
			break;
		}
	}
	return index;
}

private int getActionValue( String actionName ) {
	int value = -1;

	for( int i = 0; i < JCChartEnumMappings.trigger_i18n_strings.length; i++ ) {
		if( actionName.compareTo( JCChartEnumMappings.trigger_i18n_strings[ i ] ) == 0 ) {
			value = JCChartEnumMappings.trigger_values[ i ];
			break;
		}
	}
	return value;	
}

private class CheckBoxRenderer extends JCheckBox implements TableCellRenderer {
    public java.awt.Component getTableCellRendererComponent( JTable table, Object value, 
															 boolean isSelected, boolean hasFocus, 
															 int row, int column) {
		if( value != null ) {
			if( value instanceof Boolean )
				setSelected( ((Boolean)value).booleanValue() );
			// If 'None' selected, boolean must always be false.  Second if
			// needed to prevent endless rendering loop!
			if( triggerList.getValueAt( row, 0 ).toString().
										compareTo( "None" ) == 0 ) {
				if( ((Boolean) triggerList.getValueAt( row, 2 )).
											booleanValue() != false ) {
					triggerList.setValueAt( new Boolean( false ), row, 2 );
				}
			}
		}
		return this;
    }
}

private class ComboBoxRenderer extends JComboBox implements TableCellRenderer {
    public java.awt.Component getTableCellRendererComponent( JTable table, Object value, 
															 boolean isSelected, boolean hasFocus, 
															 int row, int column) {
		if( value != null ) {
			if( value instanceof Integer ) {
				setSelectedIndex( ((Integer)value).intValue() );
			}
			if( value instanceof String ) {
				setSelectedItem( value );
				if( value.toString().compareTo( "None" ) == 0 ) {
					triggerList.setValueAt( new Boolean( false ), row, 2 );
				}
			}
		}
        return this;
    }
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java ChartTriggerPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    ChartTriggerPage chartTriggerPage = new ChartTriggerPage();
    chartTriggerPage.setBackground( java.awt.Color.lightGray );
    chartTriggerPage.init();
	chartTriggerPage.setObject( new com.klg.jclass.chart.JCChart());
    frame.getContentPane().add( chartTriggerPage );
    frame.pack();
    java.awt.Dimension bigness = chartTriggerPage.getPreferredSize();
    frame.setSize(bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key264 );
}

public static String getPageName() {
    return "ChartTriggerPage";
}

}

