/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataViewGeneralPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:50 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.BorderLayout;
import java.awt.Insets;
import java.awt.FlowLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.BorderFactory;

import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.ChartDataModel;
import com.klg.jclass.chart.ChartDataView;
import com.klg.jclass.chart.JCChartEnumMappings;

/**
 *  ChartDataView General Property Page
 */
public class DataViewGeneralPage extends JPropertyPage {

private JEnumEditor chartTypeCombo;
private JFieldEditor viewNameField;
private JBooleanEditor visibleCheck;
private JBooleanEditor visibleLegendCheck;
private JBooleanEditor invertedCheck;
private ChartTypeTabs chartTypeTabs;
private DataSourceEditor dataSource;

private ChartDataView view = null;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
    setLayout( new BorderLayout( 5, 5 ) );

    JPanel left = new JPanel();
	GridBagConstraints leftConstraints = new GridBagConstraints();
	GridBagLayout leftGridbag = new GridBagLayout();

	left.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	left.setLayout( leftGridbag );
	leftConstraints.anchor = GridBagConstraints.WEST;
	leftConstraints.gridwidth = GridBagConstraints.REMAINDER;
	leftConstraints.insets = new Insets( 5, 10, 5, 10 );
	leftConstraints.weightx = 1.0;
    left.add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key25 ) + ":" ) );
    chartTypeCombo = new JEnumEditor( JCChartEnumMappings.chartType_i18n_strings, 
										JCChartEnumMappings.chartType_values );
    chartTypeCombo.addPropertyChangeListener(this);
	leftGridbag.setConstraints( chartTypeCombo, leftConstraints );
    left.add( chartTypeCombo );

    left.add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key26 ) + ":" ) );
    viewNameField = new JFieldEditor("", 14);
    viewNameField.addPropertyChangeListener(this);
	leftGridbag.setConstraints( viewNameField, leftConstraints );
    left.add( viewNameField );

    visibleCheck = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key27 ) );
    visibleCheck.addPropertyChangeListener(this);
	leftGridbag.setConstraints( visibleCheck, leftConstraints );
    left.add( visibleCheck );

    visibleLegendCheck = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key28 ) );
    visibleLegendCheck.addPropertyChangeListener(this);
	leftGridbag.setConstraints( visibleLegendCheck, leftConstraints );
    left.add( visibleLegendCheck );

    add("West", left);
    chartTypeTabs = new ChartTypeTabs();
    add("Center", chartTypeTabs);
    chartTypeTabs.init();

    JPanel dataSourceArea = new JPanel();
    dataSourceArea.setLayout( new FlowLayout( FlowLayout.LEFT, 0, 0 ) );
    dataSourceArea.add( new JLabel( " " + JCustomizerBundle.string( JCustomizerBundle.key31 ) ) );
    dataSource = new DataSourceEditor();
    dataSource.init();
    dataSource.addPropertyChangeListener( this );
    dataSourceArea.add( dataSource );
    add( "South", dataSourceArea );
}

/**
 * Return the Object that this Property Page is editing
 */
public Object getObject() {
    return view;
}

/**
 * Method called to populate the dialog with the source's properties
 */
public void setObject(Object source) {
    //System.out.println("DVGP setObject: "+source);
    if (source instanceof JCChart) {
		view = ((JCChart) source).getDataView(0);
    }
    else if (source instanceof ChartDataView) {
		view = (ChartDataView) source;
    }
    if (view != null) {
		chartTypeCombo.setValue(new Integer(view.getChartType()));
		String name = view.getName();
		if (name == null) name = "noName";
		viewNameField.setValue(name);
		visibleCheck.setValue(new Boolean(view.isVisible()));
		visibleLegendCheck.setValue(new 
				      Boolean(view.isVisibleInLegend()));
		/*
		invertedCheck.setValue(new Boolean(view.isInverted()));
		*/
		dataSource.setValue( view );
    }
    chartTypeTabs.setObject(view);
}

/**
 * This method is called when a specialized property editor's 
 * value is changed.
 */

public void propertyChanged(Object source, Object value) {
    if (view == null || value == null) return;
    if (source == chartTypeCombo) {
		try {
	    	view.setChartType(((Integer)value).intValue());
		} catch (Exception ex) {
	    	showError( JCustomizerBundle.string( JCustomizerBundle.key32 ) + "\n" + ex);
	    	this.setObject();
		} finally {
	    	chartTypeTabs.setObject(view);
		}
    }
    else if (source == viewNameField) {
		view.setName((String)value);
		// Update the Outline view parent to reflect the new name
		java.awt.Component parent;
		if( (( parent = getParent() ) != null ) &&
			(( parent = parent.getParent() ) != null ) && 
	    	(( parent = parent.getParent() ) != null ) ) {
	    	if( parent instanceof JPropertyPage ) {
				JPropertyPage page = (JPropertyPage)parent;
				page.setObject();
				page.repaint();
	    	}
		}
		setObject( view );
    }
    else if (source == visibleCheck) {
		boolean toggle = ((Boolean)value).booleanValue();
		view.setVisible(toggle);
    }
    else if (source == visibleLegendCheck) {
		boolean toggle = ((Boolean)value).booleanValue();
		view.setVisibleInLegend(toggle);
    }
    else if (source == invertedCheck) {
		boolean toggle = ((Boolean)value).booleanValue();
		view.setInverted(toggle);
    }
    else if (source == dataSource) {
	   	//System.out.println("DVGP setting DataSource: ");
		if( value instanceof ChartDataModel ) {
	    	//System.out.println("DVGP setting DataSource: ");
	    	view.setDataSource( (ChartDataModel) value );
		}
		// refresh this page
		setObject( view );
    }
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java DataViewGeneralPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    DataViewGeneralPage dataViewGeneralPage = new DataViewGeneralPage();
    dataViewGeneralPage.setBackground( java.awt.Color.lightGray );
    dataViewGeneralPage.init();
	dataViewGeneralPage.setObject( new com.klg.jclass.chart.JCChart() );
    frame.getContentPane().add( dataViewGeneralPage );
    frame.pack();
    java.awt.Dimension bigness = dataViewGeneralPage.getPreferredSize();
    frame.setSize(bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key8 );
}

public static String getPageName() {
    return "DataViewGeneralPage";
}

}

