/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataViewList.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:50 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.util.Enumeration;

import javax.swing.JFrame;
import javax.swing.tree.DefaultTreeModel;



import java.util.Iterator;
import java.util.List;


import com.klg.jclass.chart.ChartDataView;
import com.klg.jclass.chart.JCChart;

/**
 *  Data Views Customizer Property Page
 */
public class DataViewList extends OutlinedPropertyPage {

private JCChart chart = null;

/**
 * return a list of ( Outline item label, item Class ) pairs
 */
public String[] getOutlineDesc() {
    return viewsOutline;
}

/**
 * Return the Object that this Property Page is showing
 */
public Object getObject() {
    return (Object)chart;
}

/**
 * Method called to populate the dialog with the source's properties
 */
public void setObject( Object source ) {
    int i = 0;
    JCustomNode root = tree.getRoot();
    //System.out.println("DataViewList setObject: "+source);
    if( source instanceof JCChart ) {
        chart = (JCChart) source;
        if( chart != null ) {
            JCustomNode node = null;
            List views = chart.getDataView();
            if( views == null ) {
                title.setText( JCustomizerBundle.string( JCustomizerBundle.key140 ) );
            }
            else {
                String label = "" + views.size() + " " + ( (views.size() == 1) ? JCustomizerBundle.string( JCustomizerBundle.key83 ) :
                                                                                 JCustomizerBundle.string( JCustomizerBundle.key141 ) ) + ":";
                title.setText( label );
                Iterator iter = views.iterator();
                for( i = 0; iter.hasNext(); i++ ) {
                    Object thisObj = iter.next();
                    if(thisObj instanceof ChartDataView ) {
                        ChartDataView view = (ChartDataView) thisObj;
                        this.setNode( root, "" + i + ": " + view.getName(), i, (Object)view );
                    }
                }
            }
            this.setNode( root, null, i, null ); // Clear remaining children
            this.initSelection( false );
        }
    }
    else if( source instanceof ChartDataView ) {
        ChartDataView view = (ChartDataView)source;
        boolean found = false;
        JCustomNode node = null;
        Enumeration nodes = root.children();

        while( nodes.hasMoreElements() ) {
            node = (JCustomNode)nodes.nextElement();

            Object ud = node.getUserData();
            if (ud != null && view == (ChartDataView) ud) {
                //System.out.println("DataViewList: selecting node='"+node+ "' out of "+nodes.size());
                editor.setObject( view );
                found = true;
                break;
            }
        }
        if (!found) {
            node = new JCustomNode( "" + view.getName() );
            node.setUserData( (Object)view );
            tree.addNode( node, root, root.getChildCount() + 1 );
        }
    }
}

/**
 * This method is called when a specialized property editor's
 * value is changed.
 */
public void propertyChanged( Object source, Object value ) {
    if (chart == null || value == null) return;
    if( source == dialog ) {
        ChartDataView view = (ChartDataView)value;

        if (dialog.getMode() == JAddRemoveDialog.MODE_ADD ) {
        // add value to current Object
            List views = chart.getDataView();
            int size = ( (views == null) ? 0 : views.size() );
                chart.setDataView( size, view );
                setObject();
                JPropertyPage.getFrame( dialog ).dispose();
        }
        else if ( dialog.getMode() == JAddRemoveDialog.MODE_REMOVE ) {
            // remove value from current Object
            //System.out.println("Remove "+view);
            List views = chart.getDataView();
            if( views != null ) {
                Iterator iter = views.iterator();
                for( int i=0; iter.hasNext(); i++ ) {
                    ChartDataView thisView = (ChartDataView) iter.next();
                    if (view == thisView) {
                        chart.removeDataView( i );
                        JPropertyPage.getFrame( dialog ).dispose();
                    }
                }
            }
        }
    // Update
    setObject();
    }
}

// Override base class method.  Disable the remove button
// if there is only one data view.
protected void updateRemoveButton() {
    DefaultTreeModel treeModel = (DefaultTreeModel)tree.getModel();
    int nDataViews = treeModel.getChildCount(treeModel.getRoot());
    removeButton.setEnabled(nDataViews > 1);
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java DataViewGeneralPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
    java.lang.Class newclass
        = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
    frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
        System.out.println(ex);
    frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    DataViewList dataViewList = new DataViewList();
    dataViewList.setBackground( java.awt.Color.lightGray );
    dataViewList.init();
    dataViewList.setObject( new com.klg.jclass.chart.JCChart() );
    frame.getContentPane().add( dataViewList );
    frame.pack();
    java.awt.Dimension bigness = dataViewList.getPreferredSize();
    frame.setSize( bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key139 );
}

public static String getPageName() {
    return "DataViewList";
}


}

