/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: FillStylePage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:50 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Color;
import java.awt.Image;
import java.awt.Insets;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;

import com.klg.jclass.chart.ChartDataView;
import com.klg.jclass.chart.JCChartStyle;
import com.klg.jclass.chart.JCFillStyle;
import com.klg.jclass.chart.JCChartEnumMappings;

/**
 *  Fill Style Property Page
 */
public class FillStylePage extends JPropertyPage {

private JColorEditor fillColor;
private JColorEditor fillBackColor;
private JFileNameEditor fillImage;
private JEnumEditor fillPattern;

private JCChartStyle chartStyle = null;
private JCFillStyle fillStyle = null;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 10, 10, 5, 10 );
	constraints.weightx = 1.0;
    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key155 ) + ":" ) );
    fillColor = new JColorEditor();
    fillColor.addPropertyChangeListener(this);
	gridbag.setConstraints( fillColor, constraints );
    add( fillColor );

    add( new JLabel(JCustomizerBundle.string( JCustomizerBundle.key20 ) + ":" ) );
    fillBackColor = new JColorEditor();
    fillBackColor.addPropertyChangeListener(this);
	gridbag.setConstraints( fillBackColor, constraints );
    add( fillBackColor );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key156 ) + ":" ) );
    fillImage = new JFileNameEditor();
    fillImage.init();
    fillImage.saveButton.setEnabled( false );
    fillImage.addPropertyChangeListener( this );
	gridbag.setConstraints( fillImage, constraints );
    add( fillImage );
    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key157 ) + ":" ) );
    fillPattern = new JEnumEditor(JCChartEnumMappings.fillPattern_i18n_strings,
									JCChartEnumMappings.fillPattern_values );
    fillPattern.addPropertyChangeListener(this);
	gridbag.setConstraints( fillPattern, constraints );
    add( fillPattern );
    add( new JLabel("") );
}

/**
 * Method called to populate the dialog with the source's properties
 */
public void setObject(Object source) {
    //System.out.println("FSP setObject: "+source);
    if (source instanceof JCChartStyle) {
		chartStyle = (JCChartStyle) source;
		fillColor.setValue(chartStyle.getFillColor());
		fillBackColor.setValue(chartStyle.getFillBackground());
		fillPattern.setValue(new Integer(chartStyle.getFillPattern()));
		if (chartStyle.getFillImage() == null) {
	    	fillImage.setValue("*.gif");
		} else {
	    	fillImage.setValue("");
		}
    }
    else if (source instanceof JCFillStyle) {
		fillStyle = (JCFillStyle) source;
		fillColor.setValue(fillStyle.getColor());
		fillBackColor.setValue(fillStyle.getBackground());
		fillPattern.setValue(new Integer(fillStyle.getPattern()));
		if (fillStyle.getImage() == null) {
	    	fillImage.setValue("*.gif");
		} else {
	    	fillImage.setValue("");
		}
    }
}

/**
 * Return the Object that this Property Page is editing
 */
public Object getObject() {
    if (fillStyle != null) {
		return (Object) fillStyle;
    } else {
		return (Object) chartStyle;
    }
}

/**
 * This method is called when a specialized property editor's
 * value is changed.
 */
public void propertyChanged(Object source, Object value) {
    if (value == null) return;
    if (source == fillColor) {
		if (fillStyle != null) {
	    	fillStyle.setColor((Color)value);
		} else if (chartStyle != null) {
	    	chartStyle.setFillColor((Color)value);
		}
    }
    else if (source == fillBackColor) {
		if (fillStyle != null) {
	    	fillStyle.setBackground((Color)value);
		} else if (chartStyle != null) {
	    	chartStyle.setFillBackground((Color)value);
		}
    }
    else if (source == fillImage && value != null) {
		String filename = fillImage.getFile();
		Image image = null;
		if (filename != null && filename.length() > 0) {
			String directory = fillImage.getDirectory();
			String fullFileName = directory + filename;
			fullFileName = fullFileName.replace('\\', '/');
			if (filename != null && filename.indexOf('*') < 0) {
				image = (new ImageIcon(fullFileName)).getImage();
			}
		}
	   	if (fillStyle != null) {
			fillStyle.setImage(image);
		} else if (chartStyle != null) {
			chartStyle.setFillImage(image);
	   	}
	//System.out.println("JFileEditor fillImage= '" + value + "'" );
    }
    else if (source == fillPattern) {
		if (fillStyle != null) {
	    	fillStyle.setPattern(((Integer)value).intValue());
		} else if (chartStyle != null) {
	    	chartStyle.setFillPattern(((Integer)value).intValue());
		}
    }
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java FillStylePage
 */
public static void main(String args[]) {
    JFrame frame;
    try {
	java.lang.Class newclass
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    FillStylePage fillStylePage = new FillStylePage();
    fillStylePage.setBackground( java.awt.Color.lightGray );
    fillStylePage.init();
    frame.getContentPane().add( fillStylePage );
    frame.pack();
    java.awt.Dimension bigness = fillStylePage.getPreferredSize();
    frame.setSize(bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key158 );
}

public static String getPageName() {
    return "FillStylePage";
}

}

