/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: HiLoChartPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:50 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Insets;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;


import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.BorderFactory;

import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.JCHLOCChartFormat;
import com.klg.jclass.chart.ChartDataView;

/**
 *  HiLowOpenClose Chart Specific Property Page
 */
public class HiLoChartPage extends JPropertyPage {

private JBooleanEditor isShowingOpen;
private JBooleanEditor isShowingClose;
private JBooleanEditor isOpenCloseFullWidth;

private ChartDataView view = null;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 5, 5, 5, 5 );
	constraints.weightx = 1.0;
	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
    isShowingOpen = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key183 ) );
    isShowingOpen.addPropertyChangeListener( this );
	gridbag.setConstraints( isShowingOpen, constraints );
    add( isShowingOpen );
    isShowingClose = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key184 ) );
    isShowingClose.addPropertyChangeListener( this );
	gridbag.setConstraints( isShowingClose, constraints );
    add( isShowingClose );
    isOpenCloseFullWidth = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key185 ) );
    isOpenCloseFullWidth.addPropertyChangeListener( this );
	gridbag.setConstraints( isOpenCloseFullWidth, constraints );
    add( isOpenCloseFullWidth );
}

/**
 * Method called to populate the dialog with the source's properties
 */
public void setObject(Object source) {
    //System.out.println("DVGP setObject: "+source);
    if (source instanceof JCChart) {
	view = ((JCChart) source).getDataView(0);
    }
    else if (source instanceof ChartDataView) {
	view = (ChartDataView) source;
    }
    if (view != null && 
	(view.getChartType() == JCChart.HILO_OPEN_CLOSE)) {
	JCHLOCChartFormat hiloFormat = (JCHLOCChartFormat) view.getChartFormat();
	if (hiloFormat != null) {	    
	    isShowingOpen.setSelected( hiloFormat.isShowingOpen() );
	    isShowingOpen.setEnabled( true );
	    isShowingClose.setSelected( hiloFormat.isShowingClose() );
	    isShowingClose.setEnabled( true );
	    isOpenCloseFullWidth.setSelected( hiloFormat.isOpenCloseFullWidth() );
	    isOpenCloseFullWidth.setEnabled( true );
	}
    } else {
	isShowingOpen.setEnabled( false );
	isShowingClose.setEnabled( false );
	isOpenCloseFullWidth.setEnabled( false );
    }
}

public void propertyChanged( Object source, Object value ) {
    if( source instanceof JBooleanEditor ) {
		JBooleanEditor box = (JBooleanEditor) source;

		if( box == null || view == null ||
				view.getChartType() != JCChart.HILO_OPEN_CLOSE)
			return;
		boolean val = ((Boolean)value).booleanValue();

		JCHLOCChartFormat fmt = (JCHLOCChartFormat) view.getChartFormat();

		if( box == isShowingOpen ) {
			fmt.setShowingOpen( val );
		}
		else if (box == isShowingClose) {
			fmt.setShowingClose( val );
		}
		if (box == isOpenCloseFullWidth) {
			fmt.setOpenCloseFullWidth( val );
		}
    }
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java HiLoChartPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    HiLoChartPage hiLoChartPage = new HiLoChartPage();
    hiLoChartPage.setBackground( java.awt.Color.lightGray );
    hiLoChartPage.init();
	hiLoChartPage.setObject( new com.klg.jclass.chart.JCChart( JCChart.HILO_OPEN_CLOSE ) );
    frame.getContentPane().add( hiLoChartPage );
    frame.pack();
    java.awt.Dimension bigness = hiLoChartPage.getPreferredSize();
    frame.setSize( bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key186 );
}

public static String getPageName() {
    return "HiLoChartPage";
}

}

