/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JAddRemoveDialog.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:50 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.BorderFactory;

/**
 *  Add or remove an class from a JClass Component
 */
public class JAddRemoveDialog extends JPropertyPage implements ActionListener {

private JLabel title;
protected JPropertyPage editor;
protected JPanel controls;
protected JButton addButton, previewButton, removeButton, cancelButton;

public static int MODE_ADD = 1;
public static int MODE_REMOVE = 3;
private int currentMode = 0;

private Object container;
private Object target;
protected Object model;

private static String editorDialogs[][] = {
    { "ChartDataViewDialog", "DataViewCustomizer" },
    { "JCAxisDialog", "AxisCustomizer" },
    { "JChartLabelDialog", "ChartLabelTabs" },
};

/**
 * Method called to initialize the page
 */
public void init() {
    super.init();
	setToolTipText( getPageTitle() );
    setLayout( new BorderLayout() );
    controls = new JPanel();
    controls.setLayout( new BorderLayout() );
    editor = getPage( getEditorName() );
    editor.init();
    controls.add( editor, BorderLayout.CENTER );

    JPanel controlButtons = new JPanel();

    controlButtons.setLayout( new FlowLayout( FlowLayout.CENTER, 10, 2 ) );
    addButton = new JButton( JCustomizerBundle.string( JCustomizerBundle.key133 ) );
    addButton.addActionListener( this );
    controlButtons.add( addButton );
    if( hasPreview() ) {
		previewButton = new JButton( JCustomizerBundle.string( JCustomizerBundle.key134 ) );
		previewButton.addActionListener( this );
		controlButtons.add( previewButton );
    }
    removeButton = new JButton( JCustomizerBundle.string( JCustomizerBundle.key135 ) );
    removeButton.addActionListener( this );
    controlButtons.add( removeButton );
    cancelButton = new JButton( JCustomizerBundle.string( JCustomizerBundle.key136 ) );
    cancelButton.addActionListener( this );
    controlButtons.add( cancelButton );
    controls.add( controlButtons, BorderLayout.SOUTH );

    add( controls, BorderLayout.CENTER );
    setMode( MODE_ADD );
}

/**
 * This dialog can be used to Add or Remove an object
 * Set the mode to indicate which visually
 */
public void setMode( int mode ) {
    //System.out.println("Add DataView setObject: "+source);
    if( mode != currentMode ) {
		currentMode = mode;
	if( mode == MODE_ADD ) {
	    addButton.setEnabled( true );
	    removeButton.setEnabled( false );
		}
	else if( mode == MODE_REMOVE ) {
	    addButton.setEnabled( false );
	    removeButton.setEnabled( true );
		}
    }
}

/**
 * This dialog can be used to Add or Remove an object
 * Return the mode this dialog is currently in
 */
public int getMode() {
    return currentMode;
}

/**
 * Determine if this Dialog has a Preview
 * Subclasses will override this
 */
public boolean hasPreview() {
    return false;
}

/**
 * Show the current object in some kind of preview
 * Subclasses will override this
 */
public void showPreview() {
}

/**
 * Method called to populate the dialog with the source's properties
 */
public void setObject( Object source ) {
    //System.out.println("Add DataView setObject: "+source);
    if (source == null) return;
		target = source;
}

/**
 * Method called to provide this dialog with a model object
 * to emulate, particularly in the preview
 */
public void setModel( Object source ) {
    //System.out.println("Add/Remove setModel: "+source);
    if (source == null) return;
		model = source;
}

public void setValue(Object value) {
    setObject(value);
};

public Object getValue() {
    return target;
}

public void cancelAction() {
    JPropertyPage.getFrame( this ).dispose();
}

public void previewAction() {
    showPreview();
}

public void removeAction() {
	//System.out.println( "remove JAdd" + getValue() );
	if( listeners != null )
        listeners.firePropertyChange( "", null, getValue() );
}


public void addAction() {
	//System.out.println( "add in Jremove:" + getValue() );
	if( listeners != null )
	    listeners.firePropertyChange( "", null, getValue() );
}

/**
 * ActionListener method
 */
public void actionPerformed( ActionEvent ev ) {
    if (ev.getSource() == cancelButton) {
	cancelAction();
    } 
    else if ( ev.getSource() == previewButton ) {
	previewAction();
    } 
    else if ( ev.getSource() == removeButton ) {
	removeAction();
    } 
    else {
	addAction();
    }
}

public String getEditorName() {
    String fullname = getClass().getName();
    String shortname = fullname.substring( fullname.lastIndexOf(".") + 1 );
    for (int i = 0; i < editorDialogs.length; i++) {
	if( editorDialogs[i][0].equals( shortname ) ) {
	    return editorDialogs[i][1];
	}
    }
    return "JFieldEditor";
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java JAddRemoveDialog
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    JAddRemoveDialog jAddRemoveDialog = new JAddRemoveDialog();
    jAddRemoveDialog.setBackground( java.awt.Color.lightGray );
    jAddRemoveDialog.init();
    frame.getContentPane().add( jAddRemoveDialog );
    frame.pack();
    java.awt.Dimension bigness = jAddRemoveDialog.getPreferredSize();
    frame.setSize(bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key137 );
}

public static String getPageName() {
    return "JAddRemoveDialog";
}

}
