/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JColorEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:50 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.FlowLayout;
import java.awt.Image;
import java.awt.Rectangle;
import java.beans.PropertyEditor;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.beans.Customizer;
import java.util.StringTokenizer;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JColorChooser;
import javax.swing.JTextField;
import javax.swing.JDialog;
import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.BorderFactory;
import javax.swing.border.BevelBorder;

import com.klg.jclass.util.swing.JCIconCreator;
import com.klg.jclass.beans.ColorEditorPanel;

/**
 *  Chart Color Property Page
 */
public class JColorEditor extends JPropertyPage 
						implements PropertyEditor, PropertyChangeListener {
	
private ColorEditorPanel colorEditor = null;

public JColorEditor() {
	this( null );
}

public JColorEditor( Color c ) {
	Color color = c != null ? c : getBackground();

	setLayout( new FlowLayout( FlowLayout.LEFT, 0, 0 ) );
	colorEditor = new ColorEditorPanel(c);
	colorEditor.addPropertyChangeListener(this);
	add(colorEditor);
}

/**
 *  java.beans.PropertyEditor support
 */

    /**
     * Set (or change) the object that is to be edited.
     *
     * @param value The new target object to be edited.  Note that this
     * object should not be modified by the PropertyEditor, rather 
     * the PropertyEditor should create a new object to hold any
     * modified value.
     */
public void setValue( Object value ) {
	colorEditor.setValue(value);
}

    /**
     * @return The value of the property.  Builtin types such as "int" will
     * be wrapped as the corresponding object type such as "java.lang.Integer".
     */

public Object getValue() {
	return colorEditor.getValue();
}

    //----------------------------------------------------------------------

    /**
     * @return  True if the class will honor the paintValue method.
     */

public boolean isPaintable() {
    return true; 
}

    /**
     * Paint a representation of the value into a given area of screen
     * real estate.  Note that the propertyEditor is responsible for doing
     * its own clipping so that it fits into the given rectangle.
     * <p>
     * If the PropertyEditor doesn't honor paint requests (see isPaintable)
     * this method should be a silent noop.
     *
     * @param gfx  Graphics object to paint into.
     * @param box  Rectangle within graphics object into which we should paint.
     */
public void paintValue(java.awt.Graphics gfx, java.awt.Rectangle box) {
    gfx.setColor((Color) getValue());
    gfx.fillRect(box.x, box.y, box.width, box.height);
}

    //----------------------------------------------------------------------

    /**
     * This method is intended for use when generating Java code to set
     * the value of the property.  It should return a fragment of Java code
     * that can be used to initialize a variable with the current property
     * value.
     * <p>
     * Example results are "2", "new Color(127,127,34)", "Color.orange", etc.
     *
     * @return A fragment of Java code representing an initializer for the
     *   	current value.
     */
public String getJavaInitializationString() {
    Color color = (Color) getValue();
    return "new Color(" + color.getRed()+ ", " + color.getGreen()+ ", " + color.getBlue() + ")";
}

    //----------------------------------------------------------------------

    /**
     * @return The property value as a human editable string.
     * <p>   Returns null if the value can't be expressed as an editable string.
     * <p>   If a non-null value is returned, then the PropertyEditor should
     *	     be prepared to parse that string back in setAsText().
     */
public String getAsText() {
    return getJavaInitializationString();
}

    /**
     * Set the property value by parsing a given String.  May raise
     * java.lang.IllegalArgumentException if either the String is
     * badly formatted or if this kind of property can't be expressed
     * as text.
     * @param text  The string to be parsed.
     */
public void setAsText( String text ) throws java.lang.IllegalArgumentException {
    setValue( text );
    return;
}

    //----------------------------------------------------------------------

    /**
     * If the property value must be one of a set of known tagged values, 
     * then this method should return an array of the tags.  This can
     * be used to represent (for example) enum values.  If a PropertyEditor
     * supports tags, then it should support the use of setAsText with
     * a tag value as a way of setting the value and the use of getAsText
     * to identify the current value.
     *
     * @return The tag values for this property.  May be null if this 
     *   property cannot be represented as a tagged value.
     *	
     */
public String[] getTags() {
    return null;
}

//----------------------------------------------------------------------

/**
 * A PropertyEditor may choose to make available a full custom Component
 * that edits its property value.  It is the responsibility of the
 * PropertyEditor to hook itself up to its editor Component itself and
 * to report property value changes by firing a PropertyChange event.
 * <P>
 * The higher-level code that calls getCustomEditor may either embed
 * the Component in some larger property sheet, or it may put it in
 * its own individual dialog, or ...
 *
 * @return A java.awt.Component that will allow a human to directly
 *      edit the current property value.  May be null if this is
 *	    not supported.
 */
public Component getCustomEditor() {
	return colorEditor.getCustomEditor();
}

/**
 * @return  True if the propertyEditor can provide a custom editor.
 */
public boolean supportsCustomEditor() {
    return true;
}

protected PropertyChangeSupport listeners = null;

public void removePropertyChangeListener( PropertyChangeListener listener ) {
    if (listeners == null) {
        return;
    }
    listeners.removePropertyChangeListener( listener );
}

public void addPropertyChangeListener( PropertyChangeListener listener ) {
    if( listeners == null ) {
        listeners = new PropertyChangeSupport( this );
    }
    listeners.addPropertyChangeListener( listener );
}

public void propertyChange(PropertyChangeEvent ev) {
	if (listeners != null) {
		listeners.firePropertyChange("", null, getValue());
	}
}

}
