/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCustomTree.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:50 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Color;
import java.util.Enumeration;

import javax.swing.JTree;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeSelectionModel;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.tree.TreePath;

public class JCustomTree extends JTree {

	private DefaultTreeModel treeModel = null;
	private DefaultTreeSelectionModel selectionModel = null;

	public JCustomTree( JCustomNode root ) {
		super( root );
		treeModel = new DefaultTreeModel( root );
		selectionModel = new DefaultTreeSelectionModel();
		selectionModel.setSelectionMode( DefaultTreeSelectionModel.SINGLE_TREE_SELECTION );
		setModel( treeModel );
		setSelectionModel( selectionModel );
		setRootVisible( true );
		setBackground( Color.white );
	}

	public JCustomNode getRoot() {
		return (JCustomNode)treeModel.getRoot();
	}

	public JCustomNode getSelectedNode() {
		JCustomNode selectedNode = null;
		TreePath selectionPath = getSelectionPath();
		
		if( selectionPath != null )
			selectedNode = (JCustomNode)selectionPath.getLastPathComponent();
		
		return selectedNode;
   }

	public void removeNode( JCustomNode node ) {
		if( node != null && node != getRoot() )
			treeModel.removeNodeFromParent( node );
	}

	public void fireValueChanged( TreeSelectionEvent e ) {
		super.fireValueChanged( e );
	}

	public JCustomNode addNode( JCustomNode parent, String label, int index ) {
		JCustomNode newNode = null;

		if( parent != null ) {
			Enumeration children = parent.children();
			int childrenCount = parent.getChildCount();

			if( children != null && index < childrenCount ) {
				newNode = (JCustomNode)parent.getChildAt( index );
				newNode.setUserObject( new String( label ) );
			} 
			else {
				newNode = new JCustomNode( label );
				treeModel.insertNodeInto( newNode, parent, index );
			}
		}
		return newNode;
	}

	public void addNode( JCustomNode node, JCustomNode parent, int index ) {
		if( parent != null && node != null && index > parent.getChildCount() )
			treeModel.insertNodeInto( node, parent, index );
	}

	public void addNode( JCustomNode node, JCustomNode parent ) {
		if( parent != null && node != null )
			treeModel.insertNodeInto( node, parent, parent.getChildCount() );
	}

	public void initSelection( boolean selectFirst ) {
		JCustomNode root = getRoot();
		int rowToExpand = 0;

		while( true ) {
			if( treeModel.getChildCount( root ) > 0 ) {
				JCustomNode child = (JCustomNode)treeModel.getChild( root, 0 );

				expandRow( rowToExpand );
				if( treeModel.isLeaf( child ) ) {
					if( selectFirst )
						setSelectionRow( rowToExpand + 1 );
					break;
				}
				else {
					root = child;
					rowToExpand++;
				}
			}
			else
				break;
		}
	}

	public void removeAllChildren( JCustomNode rootNode ) {
		int childrenCount = treeModel.getChildCount( rootNode );

		for( int i = 0; i < childrenCount; i++ ) {
			JCustomNode child = (JCustomNode)treeModel.getChild( rootNode, i );

			if( !treeModel.isLeaf( child ) )
				removeAllChildren( child );
			removeNode( child );
		}
	}

	public JCustomNode findNode( JCustomNode root, Object userData ) {
		JCustomNode node = null;
		int childrenCount = treeModel.getChildCount( root );

		for( int i = 0; i < childrenCount; i++ ) {
			node = (JCustomNode)treeModel.getChild( root, i );

			Object ud = node.getUserData();

			if( ud != null && ud == userData ) {
				return node;
			}
			if( !treeModel.isLeaf( node ) ) {
				if( ( node = findNode( node, userData ) ) != null )
					return node;
			}
		}
		return null;
	}

	protected void printTree( JCustomNode rootNode ) {
		int childrenCount = treeModel.getChildCount( rootNode );

		for( int i = 0; i < childrenCount; i++ ) {
			JCustomNode child = (JCustomNode)treeModel.getChild( rootNode, i );

			if( !treeModel.isLeaf( child ) )
				printTree( child );
			System.out.println( child );
		}
	}

}
