/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCustomizerBundle.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:50 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.util.Locale;
import java.util.ResourceBundle;
import java.text.MessageFormat;
import java.text.ChoiceFormat;


public class JCustomizerBundle {

private static final String BUNDLE_PATH = "com.klg.jclass.chart.customizer.resources.LocaleInfo";
private static ResourceBundle chart_bundle = ResourceBundle.getBundle( BUNDLE_PATH, Locale.getDefault() );


/**
 * Static function used to localize a string based on its key,
 * language and country
 * @param language desired language
 * @param country desired country
 * @param key the key of the string that is to be retrived
 */
public synchronized static String string( String language, String country, String key ) {
	return ResourceBundle.getBundle( BUNDLE_PATH, new Locale( language, country ) ).getString( key );
}

/**
 * Static function used to retrive a string based on its key;
 * In JDK 1.1 the default locale it is used.
 * @param key the key of the string that is to be retrived
 */
public synchronized static String string( String key ) {
	String s = null;
	s = chart_bundle.getString( key );
	return s;
}

/**
 * Static function used to format a string based on its key.
 * It produces concatenated messages in la anguage-neutral way.
 * @param key the key of the string that is to be retrived
 * @param args set of objects used in formatting
 */
public synchronized static String string( String key, Object[] args ) {
	String s = null;

	s = MessageFormat.format( chart_bundle.getString( key ), args );

	return s;
}

 /**
 * Static function used to format a string based on its key.
 * It produces concatenated messages in la anguage-neutral way.
 * @param key the key of the string that is to be retrived
 * @param formatter a formatter provided by the user
 * @param args set of objects used in formatting
 */
public synchronized static String string( String key, MessageFormat formatter, Object args[] ) {
	String s = null;

	s = formatter.format( chart_bundle.getString( key ), args );
	return s;
}

 /**
 * Static function used to programatically change the default locale
 * @param lnew locale
 */
public synchronized static void setBundleLocale( Locale l ) {
	chart_bundle = ResourceBundle.getBundle( BUNDLE_PATH, l );
}

 /**
 * Static function used to format a string based on its key.
 * It produces concatenated messages in laanguage-neutral way.
 * Allows to attach a format to a range of numbers. It is generally used for handling plurals
 * The choice is specified with an ascending list of doubles, where each item
 * specifies a half-open interval up to the next item:  X matches j if and only if limit[j] <= X < limit[j+1]
 * @param key the key of the string that is to be retrived
 * @param args set of objects used in formatting
 * @param limits the list of doubles that specifies a half-open interval
 * @param formats it is an array of choices: one choice for each interval defined by the limits array
 * @param choiceVar it is used to identify which of the variable from the string that contains
 * the formatting information uses the choice formatting feature. Note the limitation of only one choice variable per string.
 */
public synchronized static String string( String key, Object[] args, double[] limits, String[] formats, int choiceVar ) {
	String s = null;
	MessageFormat formatter = new MessageFormat( chart_bundle.getString( key ) );
	ChoiceFormat choicer = new ChoiceFormat( limits, formats );

	formatter.setFormat( choiceVar, choicer );
	s = formatter.format( args );
	return s;
}


// Key constants

public static final String key0   = "com.klg.jclass.chart.customizer.Key0";
public static final String key1   = "com.klg.jclass.chart.customizer.Key1";
public static final String key2   = "com.klg.jclass.chart.customizer.Key2";
public static final String key3   = "com.klg.jclass.chart.customizer.Key3";
public static final String key4   = "com.klg.jclass.chart.customizer.Key4";
public static final String key5   = "com.klg.jclass.chart.customizer.Key5";
public static final String key6   = "com.klg.jclass.chart.customizer.Key6";
public static final String key7   = "com.klg.jclass.chart.customizer.Key7";
public static final String key8   = "com.klg.jclass.chart.customizer.Key8";
public static final String key9   = "com.klg.jclass.chart.customizer.Key9";
public static final String key10  = "com.klg.jclass.chart.customizer.Key10";
public static final String key11  = "com.klg.jclass.chart.customizer.Key11";
public static final String key12  = "com.klg.jclass.chart.customizer.Key12";
public static final String key13  = "com.klg.jclass.chart.customizer.Key13";
public static final String key14  = "com.klg.jclass.chart.customizer.Key14";
public static final String key15  = "com.klg.jclass.chart.customizer.Key15";
public static final String key16  = "com.klg.jclass.chart.customizer.Key16";
public static final String key17  = "com.klg.jclass.chart.customizer.Key17";
public static final String key18  = "com.klg.jclass.chart.customizer.Key18";
public static final String key19  = "com.klg.jclass.chart.customizer.Key19";
public static final String key20  = "com.klg.jclass.chart.customizer.Key20";
public static final String key21  = "com.klg.jclass.chart.customizer.Key21";
public static final String key22  = "com.klg.jclass.chart.customizer.Key22";
public static final String key23  = "com.klg.jclass.chart.customizer.Key23";
public static final String key24  = "com.klg.jclass.chart.customizer.Key24";
public static final String key25  = "com.klg.jclass.chart.customizer.Key25";
public static final String key26  = "com.klg.jclass.chart.customizer.Key26";
public static final String key27  = "com.klg.jclass.chart.customizer.Key27";
public static final String key28  = "com.klg.jclass.chart.customizer.Key28";
public static final String key29  = "com.klg.jclass.chart.customizer.Key29";
public static final String key30  = "com.klg.jclass.chart.customizer.Key30";
public static final String key31  = "com.klg.jclass.chart.customizer.Key31";
public static final String key32  = "com.klg.jclass.chart.customizer.Key32";
public static final String key33  = "com.klg.jclass.chart.customizer.Key33";
public static final String key34  = "com.klg.jclass.chart.customizer.Key34";
public static final String key35  = "com.klg.jclass.chart.customizer.Key35";
public static final String key36  = "com.klg.jclass.chart.customizer.Key36";
public static final String key37  = "com.klg.jclass.chart.customizer.Key37";
public static final String key38  = "com.klg.jclass.chart.customizer.Key38";
public static final String key39  = "com.klg.jclass.chart.customizer.Key39";
public static final String key40  = "com.klg.jclass.chart.customizer.Key40";
public static final String key41  = "com.klg.jclass.chart.customizer.Key41";
public static final String key42  = "com.klg.jclass.chart.customizer.Key42";
public static final String key43  = "com.klg.jclass.chart.customizer.Key43";
public static final String key44  = "com.klg.jclass.chart.customizer.Key44";
public static final String key45  = "com.klg.jclass.chart.customizer.Key45";
public static final String key46  = "com.klg.jclass.chart.customizer.Key46";
public static final String key47  = "com.klg.jclass.chart.customizer.Key47";
public static final String key48  = "com.klg.jclass.chart.customizer.Key48";
public static final String key49  = "com.klg.jclass.chart.customizer.Key49";
public static final String key50  = "com.klg.jclass.chart.customizer.Key50";
public static final String key51  = "com.klg.jclass.chart.customizer.Key51";
public static final String key52  = "com.klg.jclass.chart.customizer.Key52";
public static final String key53  = "com.klg.jclass.chart.customizer.Key53";
public static final String key54  = "com.klg.jclass.chart.customizer.Key54";
public static final String key55  = "com.klg.jclass.chart.customizer.Key55";
public static final String key56  = "com.klg.jclass.chart.customizer.Key56";
public static final String key57  = "com.klg.jclass.chart.customizer.Key57";
public static final String key58  = "com.klg.jclass.chart.customizer.Key58";
public static final String key59  = "com.klg.jclass.chart.customizer.Key59";
public static final String key60  = "com.klg.jclass.chart.customizer.Key60";
public static final String key61  = "com.klg.jclass.chart.customizer.Key61";
public static final String key62  = "com.klg.jclass.chart.customizer.Key62";
public static final String key63  = "com.klg.jclass.chart.customizer.Key63";

public static final String key67  = "com.klg.jclass.chart.customizer.Key67";
public static final String key68  = "com.klg.jclass.chart.customizer.Key68";
public static final String key69  = "com.klg.jclass.chart.customizer.Key69";
public static final String key70  = "com.klg.jclass.chart.customizer.Key70";
public static final String key71  = "com.klg.jclass.chart.customizer.Key71";
public static final String key72  = "com.klg.jclass.chart.customizer.Key72";
public static final String key73  = "com.klg.jclass.chart.customizer.Key73";
public static final String key74  = "com.klg.jclass.chart.customizer.Key74";
public static final String key75  = "com.klg.jclass.chart.customizer.Key75";
public static final String key76  = "com.klg.jclass.chart.customizer.Key76";
public static final String key77  = "com.klg.jclass.chart.customizer.Key77";
public static final String key78  = "com.klg.jclass.chart.customizer.Key78";
public static final String key79  = "com.klg.jclass.chart.customizer.Key79";
public static final String key80  = "com.klg.jclass.chart.customizer.Key80";
public static final String key81  = "com.klg.jclass.chart.customizer.Key81";
public static final String key82  = "com.klg.jclass.chart.customizer.Key82";
public static final String key83  = "com.klg.jclass.chart.customizer.Key83";
public static final String key84  = "com.klg.jclass.chart.customizer.Key84";
public static final String key85  = "com.klg.jclass.chart.customizer.Key85";
public static final String key86  = "com.klg.jclass.chart.customizer.Key86";
public static final String key87  = "com.klg.jclass.chart.customizer.Key87";
public static final String key88  = "com.klg.jclass.chart.customizer.Key88";
public static final String key89  = "com.klg.jclass.chart.customizer.Key89";
public static final String key90  = "com.klg.jclass.chart.customizer.Key90";
public static final String key91  = "com.klg.jclass.chart.customizer.Key91";
public static final String key92  = "com.klg.jclass.chart.customizer.Key92";
public static final String key93  = "com.klg.jclass.chart.customizer.Key93";
public static final String key94  = "com.klg.jclass.chart.customizer.Key94";
public static final String key95  = "com.klg.jclass.chart.customizer.Key95";
public static final String key96  = "com.klg.jclass.chart.customizer.Key96";
public static final String key97  = "com.klg.jclass.chart.customizer.Key97";
public static final String key98  = "com.klg.jclass.chart.customizer.Key98";
public static final String key99  = "com.klg.jclass.chart.customizer.Key99";
public static final String key100 = "com.klg.jclass.chart.customizer.Key100";
public static final String key101 = "com.klg.jclass.chart.customizer.Key101";
public static final String key102 = "com.klg.jclass.chart.customizer.Key102";
public static final String key103 = "com.klg.jclass.chart.customizer.Key103";
public static final String key104 = "com.klg.jclass.chart.customizer.Key104";
public static final String key105 = "com.klg.jclass.chart.customizer.Key105";
public static final String key106 = "com.klg.jclass.chart.customizer.Key106";
public static final String key107 = "com.klg.jclass.chart.customizer.Key107";
public static final String key108 = "com.klg.jclass.chart.customizer.Key108";
public static final String key109 = "com.klg.jclass.chart.customizer.Key109";
public static final String key110 = "com.klg.jclass.chart.customizer.Key110";
public static final String key111 = "com.klg.jclass.chart.customizer.Key111";
public static final String key112 = "com.klg.jclass.chart.customizer.Key112";
public static final String key113 = "com.klg.jclass.chart.customizer.Key113";
public static final String key114 = "com.klg.jclass.chart.customizer.Key114";
public static final String key115 = "com.klg.jclass.chart.customizer.Key115";
public static final String key116 = "com.klg.jclass.chart.customizer.Key116";
public static final String key117 = "com.klg.jclass.chart.customizer.Key117";
public static final String key118 = "com.klg.jclass.chart.customizer.Key118";
public static final String key119 = "com.klg.jclass.chart.customizer.Key119";
public static final String key120 = "com.klg.jclass.chart.customizer.Key120";
public static final String key121 = "com.klg.jclass.chart.customizer.Key121";
public static final String key122 = "com.klg.jclass.chart.customizer.Key122";
public static final String key123 = "com.klg.jclass.chart.customizer.Key123";
public static final String key124 = "com.klg.jclass.chart.customizer.Key124";
public static final String key125 = "com.klg.jclass.chart.customizer.Key125";
public static final String key126 = "com.klg.jclass.chart.customizer.Key126";
public static final String key127 = "com.klg.jclass.chart.customizer.Key127";
public static final String key128 = "com.klg.jclass.chart.customizer.Key128";
public static final String key129 = "com.klg.jclass.chart.customizer.Key129";
public static final String key130 = "com.klg.jclass.chart.customizer.Key130";
public static final String key131 = "com.klg.jclass.chart.customizer.Key131";
public static final String key132 = "com.klg.jclass.chart.customizer.Key132";
public static final String key133 = "com.klg.jclass.chart.customizer.Key133";
public static final String key134 = "com.klg.jclass.chart.customizer.Key134";
public static final String key135 = "com.klg.jclass.chart.customizer.Key135";
public static final String key136 = "com.klg.jclass.chart.customizer.Key136";
public static final String key137 = "com.klg.jclass.chart.customizer.Key137";
public static final String key138 = "com.klg.jclass.chart.customizer.Key138";
public static final String key139 = "com.klg.jclass.chart.customizer.Key139";
public static final String key140 = "com.klg.jclass.chart.customizer.Key140";
public static final String key141 = "com.klg.jclass.chart.customizer.Key141";
public static final String key142 = "com.klg.jclass.chart.customizer.Key142";
public static final String key143 = "com.klg.jclass.chart.customizer.Key143";
public static final String key144 = "com.klg.jclass.chart.customizer.Key144";
public static final String key145 = "com.klg.jclass.chart.customizer.Key145";
public static final String key146 = "com.klg.jclass.chart.customizer.Key146";
public static final String key147 = "com.klg.jclass.chart.customizer.Key147";
public static final String key148 = "com.klg.jclass.chart.customizer.Key148";
public static final String key149 = "com.klg.jclass.chart.customizer.Key149";
public static final String key150 = "com.klg.jclass.chart.customizer.Key150";
public static final String key151 = "com.klg.jclass.chart.customizer.Key151";
public static final String key152 = "com.klg.jclass.chart.customizer.Key152";
public static final String key153 = "com.klg.jclass.chart.customizer.Key153";
public static final String key154 = "com.klg.jclass.chart.customizer.Key154";
public static final String key155 = "com.klg.jclass.chart.customizer.Key155";
public static final String key156 = "com.klg.jclass.chart.customizer.Key156";
public static final String key157 = "com.klg.jclass.chart.customizer.Key157";
public static final String key158 = "com.klg.jclass.chart.customizer.Key158";
public static final String key159 = "com.klg.jclass.chart.customizer.Key159";
public static final String key160 = "com.klg.jclass.chart.customizer.Key160";
public static final String key161 = "com.klg.jclass.chart.customizer.Key161";
public static final String key162 = "com.klg.jclass.chart.customizer.Key162";
public static final String key163 = "com.klg.jclass.chart.customizer.Key163";
public static final String key164 = "com.klg.jclass.chart.customizer.Key164";
public static final String key165 = "com.klg.jclass.chart.customizer.Key165";
public static final String key166 = "com.klg.jclass.chart.customizer.Key166";
public static final String key167 = "com.klg.jclass.chart.customizer.Key167";
public static final String key168 = "com.klg.jclass.chart.customizer.Key168";
public static final String key169 = "com.klg.jclass.chart.customizer.Key169";
public static final String key170 = "com.klg.jclass.chart.customizer.Key170";
public static final String key171 = "com.klg.jclass.chart.customizer.Key171";
public static final String key172 = "com.klg.jclass.chart.customizer.Key172";
public static final String key173 = "com.klg.jclass.chart.customizer.Key173";
public static final String key174 = "com.klg.jclass.chart.customizer.Key174";
public static final String key175 = "com.klg.jclass.chart.customizer.Key175";
public static final String key176 = "com.klg.jclass.chart.customizer.Key176";
public static final String key177 = "com.klg.jclass.chart.customizer.Key177";
public static final String key178 = "com.klg.jclass.chart.customizer.Key178";
public static final String key179 = "com.klg.jclass.chart.customizer.Key179";
public static final String key180 = "com.klg.jclass.chart.customizer.Key180";
public static final String key181 = "com.klg.jclass.chart.customizer.Key181";
public static final String key182 = "com.klg.jclass.chart.customizer.Key182";
public static final String key183 = "com.klg.jclass.chart.customizer.Key183";
public static final String key184 = "com.klg.jclass.chart.customizer.Key184";
public static final String key185 = "com.klg.jclass.chart.customizer.Key185";
public static final String key186 = "com.klg.jclass.chart.customizer.Key186";
public static final String key187 = "com.klg.jclass.chart.customizer.Key187";
public static final String key188 = "com.klg.jclass.chart.customizer.Key188";
public static final String key189 = "com.klg.jclass.chart.customizer.Key189";
public static final String key190 = "com.klg.jclass.chart.customizer.Key190";
public static final String key191 = "com.klg.jclass.chart.customizer.Key191";
public static final String key192 = "com.klg.jclass.chart.customizer.Key192";
public static final String key193 = "com.klg.jclass.chart.customizer.Key193";
public static final String key194 = "com.klg.jclass.chart.customizer.Key194";
public static final String key195 = "com.klg.jclass.chart.customizer.Key195";
public static final String key196 = "com.klg.jclass.chart.customizer.Key196";
public static final String key197 = "com.klg.jclass.chart.customizer.Key197";
public static final String key198 = "com.klg.jclass.chart.customizer.Key198";
public static final String key199 = "com.klg.jclass.chart.customizer.Key199";
public static final String key200 = "com.klg.jclass.chart.customizer.Key200";
public static final String key201 = "com.klg.jclass.chart.customizer.Key201";
public static final String key202 = "com.klg.jclass.chart.customizer.Key202";
public static final String key203 = "com.klg.jclass.chart.customizer.Key203";
public static final String key204 = "com.klg.jclass.chart.customizer.Key204";
public static final String key205 = "com.klg.jclass.chart.customizer.Key205";
public static final String key206 = "com.klg.jclass.chart.customizer.Key206";
public static final String key207 = "com.klg.jclass.chart.customizer.Key207";
public static final String key208 = "com.klg.jclass.chart.customizer.Key208";
public static final String key209 = "com.klg.jclass.chart.customizer.Key209";
public static final String key210 = "com.klg.jclass.chart.customizer.Key210";
public static final String key211 = "com.klg.jclass.chart.customizer.Key211";
public static final String key212 = "com.klg.jclass.chart.customizer.Key212";
public static final String key213 = "com.klg.jclass.chart.customizer.Key213";
public static final String key214 = "com.klg.jclass.chart.customizer.Key214";
public static final String key215 = "com.klg.jclass.chart.customizer.Key215";
public static final String key216 = "com.klg.jclass.chart.customizer.Key216";
public static final String key217 = "com.klg.jclass.chart.customizer.Key217";
public static final String key218 = "com.klg.jclass.chart.customizer.Key218";
public static final String key219 = "com.klg.jclass.chart.customizer.Key219";
public static final String key220 = "com.klg.jclass.chart.customizer.Key220";
public static final String key221 = "com.klg.jclass.chart.customizer.Key221";
public static final String key222 = "com.klg.jclass.chart.customizer.Key222";
public static final String key223 = "com.klg.jclass.chart.customizer.Key223";
public static final String key224 = "com.klg.jclass.chart.customizer.Key224";
public static final String key225 = "com.klg.jclass.chart.customizer.Key225";
public static final String key226 = "com.klg.jclass.chart.customizer.Key226";
public static final String key227 = "com.klg.jclass.chart.customizer.Key227";
public static final String key228 = "com.klg.jclass.chart.customizer.Key228";
public static final String key229 = "com.klg.jclass.chart.customizer.Key229";
public static final String key230 = "com.klg.jclass.chart.customizer.Key230";
public static final String key231 = "com.klg.jclass.chart.customizer.Key231";
public static final String key232 = "com.klg.jclass.chart.customizer.Key232";
public static final String key233 = "com.klg.jclass.chart.customizer.Key233";
public static final String key234 = "com.klg.jclass.chart.customizer.Key234";
public static final String key235 = "com.klg.jclass.chart.customizer.Key235";
public static final String key236 = "com.klg.jclass.chart.customizer.Key236";
public static final String key237 = "com.klg.jclass.chart.customizer.Key237";
public static final String key238 = "com.klg.jclass.chart.customizer.Key238";
public static final String key239 = "com.klg.jclass.chart.customizer.Key239";
public static final String key240 = "com.klg.jclass.chart.customizer.Key240";
public static final String key241 = "com.klg.jclass.chart.customizer.Key241";
public static final String key242 = "com.klg.jclass.chart.customizer.Key242";
public static final String key243 = "com.klg.jclass.chart.customizer.Key243";
public static final String key244 = "com.klg.jclass.chart.customizer.Key244";
public static final String key245 = "com.klg.jclass.chart.customizer.Key245";
public static final String key246 = "com.klg.jclass.chart.customizer.Key246";
public static final String key247 = "com.klg.jclass.chart.customizer.Key247";
public static final String key248 = "com.klg.jclass.chart.customizer.Key248";
public static final String key249 = "com.klg.jclass.chart.customizer.Key249";
public static final String key250 = "com.klg.jclass.chart.customizer.Key250";
public static final String key251 = "com.klg.jclass.chart.customizer.Key251";
public static final String key252 = "com.klg.jclass.chart.customizer.Key252";
public static final String key253 = "com.klg.jclass.chart.customizer.Key253";
public static final String key254 = "com.klg.jclass.chart.customizer.Key254";
public static final String key255 = "com.klg.jclass.chart.customizer.Key255";
public static final String key256 = "com.klg.jclass.chart.customizer.Key256";
public static final String key257 = "com.klg.jclass.chart.customizer.Key257";
public static final String key258 = "com.klg.jclass.chart.customizer.Key258";
public static final String key259 = "com.klg.jclass.chart.customizer.Key259";
public static final String key260 = "com.klg.jclass.chart.customizer.Key260";
public static final String key261 = "com.klg.jclass.chart.customizer.Key261";
public static final String key262 = "com.klg.jclass.chart.customizer.Key262";
public static final String key263 = "com.klg.jclass.chart.customizer.Key263";
public static final String key264 = "com.klg.jclass.chart.customizer.Key264";
public static final String key265 = "com.klg.jclass.chart.customizer.Key265";
public static final String key266 = "com.klg.jclass.chart.customizer.Key266";
public static final String key267 = "com.klg.jclass.chart.customizer.Key267";
public static final String key268 = "com.klg.jclass.chart.customizer.Key268";
public static final String key269 = "com.klg.jclass.chart.customizer.Key269";
public static final String key270 = "com.klg.jclass.chart.customizer.Key270";
public static final String key271 = "com.klg.jclass.chart.customizer.Key271";
public static final String key272 = "com.klg.jclass.chart.customizer.Key272";
public static final String key273 = "com.klg.jclass.chart.customizer.Key273";
public static final String key274 = "com.klg.jclass.chart.customizer.Key274";
public static final String key275 = "com.klg.jclass.chart.customizer.Key275";
public static final String key276 = "com.klg.jclass.chart.customizer.Key276";
public static final String key277 = "com.klg.jclass.chart.customizer.Key277";
public static final String key278 = "com.klg.jclass.chart.customizer.Key278";
public static final String key279 = "com.klg.jclass.chart.customizer.Key279";
public static final String key280 = "com.klg.jclass.chart.customizer.Key280";
public static final String key281 = "com.klg.jclass.chart.customizer.Key281";
public static final String key282 = "com.klg.jclass.chart.customizer.Key282";
public static final String key283 = "com.klg.jclass.chart.customizer.Key283";
public static final String key284 = "com.klg.jclass.chart.customizer.Key284";
public static final String key285 = "com.klg.jclass.chart.customizer.Key285";
public static final String key286 = "com.klg.jclass.chart.customizer.Key286";
public static final String key287 = "com.klg.jclass.chart.customizer.Key287";
public static final String key288 = "com.klg.jclass.chart.customizer.Key288";
public static final String key289 = "com.klg.jclass.chart.customizer.Key289";
public static final String key290 = "com.klg.jclass.chart.customizer.Key290";
public static final String key291 = "com.klg.jclass.chart.customizer.Key291";
public static final String key292 = "com.klg.jclass.chart.customizer.Key292";
public static final String key293 = "com.klg.jclass.chart.customizer.Key293";
public static final String key294 = "com.klg.jclass.chart.customizer.Key294";
public static final String key295 = "com.klg.jclass.chart.customizer.Key295";
public static final String key296 = "com.klg.jclass.chart.customizer.Key296";
public static final String key297 = "com.klg.jclass.chart.customizer.Key297";
public static final String key298 = "com.klg.jclass.chart.customizer.Key298";
public static final String key299 = "com.klg.jclass.chart.customizer.Key299";
public static final String key300 = "com.klg.jclass.chart.customizer.Key300";
public static final String key301 = "com.klg.jclass.chart.customizer.Key301";
public static final String key302 = "com.klg.jclass.chart.customizer.Key302";
public static final String key303 = "com.klg.jclass.chart.customizer.Key303";
public static final String key304 = "com.klg.jclass.chart.customizer.Key304";
public static final String key305 = "com.klg.jclass.chart.customizer.Key305";
public static final String key306 = "com.klg.jclass.chart.customizer.Key306";
public static final String key307 = "com.klg.jclass.chart.customizer.Key307";
public static final String key308 = "com.klg.jclass.chart.customizer.Key308";
public static final String key309 = "com.klg.jclass.chart.customizer.Key309";
public static final String key310 = "com.klg.jclass.chart.customizer.Key310";
public static final String key311 = "com.klg.jclass.chart.customizer.Key311";
public static final String key312 = "com.klg.jclass.chart.customizer.Key312";
public static final String key313 = "com.klg.jclass.chart.customizer.Key313";
public static final String key314 = "com.klg.jclass.chart.customizer.Key314";
public static final String key315 = "com.klg.jclass.chart.customizer.Key315";
public static final String key316 = "com.klg.jclass.chart.customizer.Key316";
public static final String key317 = "com.klg.jclass.chart.customizer.Key317";
public static final String key318 = "com.klg.jclass.chart.customizer.Key318";
public static final String key319 = "com.klg.jclass.chart.customizer.Key319";
public static final String key320 = "com.klg.jclass.chart.customizer.Key320";
public static final String key321 = "com.klg.jclass.chart.customizer.Key321";
public static final String key322 = "com.klg.jclass.chart.customizer.Key322";
public static final String key323 = "com.klg.jclass.chart.customizer.Key323";
public static final String key324 = "com.klg.jclass.chart.customizer.Key324";
public static final String key325 = "com.klg.jclass.chart.customizer.Key325";
public static final String key326 = "com.klg.jclass.chart.customizer.Key326";
public static final String key327 = "com.klg.jclass.chart.customizer.Key327";
public static final String key328 = "com.klg.jclass.chart.customizer.Key328";
public static final String key329 = "com.klg.jclass.chart.customizer.Key329";
public static final String key330 = "com.klg.jclass.chart.customizer.Key330";
public static final String key331 = "com.klg.jclass.chart.customizer.Key331";
public static final String key332 = "com.klg.jclass.chart.customizer.Key332";
public static final String key333 = "com.klg.jclass.chart.customizer.Key333";
public static final String key334 = "com.klg.jclass.chart.customizer.Key334";
public static final String key335 = "com.klg.jclass.chart.customizer.Key335";
public static final String key336 = "com.klg.jclass.chart.customizer.Key336";
public static final String key337 = "com.klg.jclass.chart.customizer.Key337";
public static final String key338 = "com.klg.jclass.chart.customizer.Key338";
public static final String key339 = "com.klg.jclass.chart.customizer.Key339";
public static final String key340 = "com.klg.jclass.chart.customizer.Key340";
public static final String key341 = "com.klg.jclass.chart.customizer.Key341";
public static final String key342 = "com.klg.jclass.chart.customizer.Key342";
public static final String key343 = "com.klg.jclass.chart.customizer.Key343";
public static final String key344 = "com.klg.jclass.chart.customizer.Key344";
public static final String key345 = "com.klg.jclass.chart.customizer.Key345";
public static final String key346 = "com.klg.jclass.chart.customizer.Key346";
}
