/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JDoubleEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:50 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.beans.PropertyEditor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class JDoubleEditor extends JFieldEditor 
					implements ActionListener, PropertyEditor {

/** 
 * Creates a double field with the given size (characters)
 * No parameters are read from an HTML file.
 */
public JDoubleEditor( int size ) {
    super( "", size );
}


/**
 *  java.beans.PropertyEditor support
 */

    /**
     * This method is intended for use when generating Java code to set
     * the value of the property.  It should return a fragment of Java code
     * that can be used to initialize a variable with the current property
     * value.
     * <p>
     * Example results are "2", "new Color(127,127,34)", "Color.orange", etc.
     *
     * @return A fragment of Java code representing an initializer for the
     *   	current value.
     */
public String getJavaInitializationString() {
    Object value = getValue();

    if( value != null ) {
		return value.toString();
    }
    else 
		return null;
}

    //----------------------------------------------------------------------
    /**
     * Set (or change) the object that is to be edited.  Builtin types such
     * as "int" must be wrapped as the corresponding object type such as
     * "java.lang.Integer".
     *
     * @param value The new target object to be edited.  Note that this
     *     object should not be modified by the PropertyEditor, rather 
     *     the PropertyEditor should create a new object to hold any
     *     modified value.
     */
public void setValue( Object value ) {
    if( value instanceof String ) {
		textField.setText((String)value);
    }
    else if (value instanceof Double) {
		double dval = ((Double)value).doubleValue();
		if (dval == Double.MAX_VALUE) {
			textField.setText("MAX");
		} else if (dval == Double.MIN_VALUE) {
			textField.setText("MIN");
		} else {
			textField.setText(((Double)value).toString());
		}
    }
}

   /**
    * @return The value of the property.  
    * Returns the Integer cooresponding to the Enum value.
    */

public Object getValue() {
	String text = textField.getText();
    try {
		Double value = new Double(text);
		return (Object) value;
    } catch (Exception e) {
		if (text.equalsIgnoreCase("MAX"))
			return new Double(Double.MAX_VALUE);
		else if (text.equalsIgnoreCase("MIN"))
			return new Double(Double.MIN_VALUE);
		else
			return null;
    }
}

/**
 * ActionListener method
 * Just fire off a PropertyChange Event if the current value is valid
 */
public void actionPerformed( ActionEvent ev ) {
    Object value = getValue();

    if( value == null ) {
		//beep();
    } 
	else {
		if( listeners != null )
			listeners.firePropertyChange( "", null, value );
    }
}

}
