/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JEnumEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:50 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.awt.GridLayout;
import java.beans.PropertyChangeListener;
import java.beans.PropertyEditor;
import java.beans.PropertyChangeSupport;

import javax.swing.JComboBox;
import javax.swing.JFrame;

public class JEnumEditor extends JPropertyPage
				implements ItemListener, PropertyEditor {

private String[] strings;
private int[] enumValues;
private JComboBox comboBox = null;
private String prepend = "";
private Object[] itemValues = null;
private boolean settingObjectItems = false;

/**
 * Default Constructor: Creates an empty ComboBox.
 * No parameters are read from an HTML file.
 */
public JEnumEditor() {
    this( null, null );
}

/**
 * Creates a comboBox with the specified items and name.
 * No parameters are read from an HTML file.
 */
public JEnumEditor( String[] items, int[] values ) {
    super.init();
	setToolTipText( getPageTitle() );
	setLayout( new GridLayout( 1, 1 ) );
	comboBox = new JComboBox();
    strings = items;
    enumValues = values;
	itemValues = strings;
	if( items != null && values != null ) {
		setItems( items );
	}
	comboBox.addItemListener( this );
	add( comboBox );
}


public void setItems( String items[], Object values[] ) {
	settingObjectItems = true;
	if( !sameStrings( items ) ) {
		strings = items;
		itemValues = values;
		if( items != null && values != null ) {
			setItems( items );
		}
	}
	settingObjectItems = false;
}

public Object[] getItems() {
	return itemValues;
}

public Object getItem( int index ) {
	Object item = null;

	if( itemValues != null && index < itemValues.length )
		item = itemValues[ index ];
	return item;
}

/**
 * Creates a comboBox with the specified items and name and string
 * to prepend for Java initialization.
 * No parameters are read from an HTML file.
 */
public JEnumEditor( String[] items, int[] values, String prepend ) {
    this( items, values );
    this.prepend = prepend;
}
/**
 *  java.beans.PropertyEditor support
 */

    /**
     * Set (or change) the object that is to be edited.  Builtin types such
     * as "int" must be wrapped as the corresponding object type such as
     * "java.lang.Integer".
     *
     * @param value The new target object to be edited.  Note that this
     *     object should not be modified by the PropertyEditor, rather
     *     the PropertyEditor should create a new object to hold any
     *     modified value.
     */
public void setValue( Object value ) {
	if( value == null ) return;
    if (value instanceof Integer) {
		int val = ((Integer)value).intValue();
		int index = indexOf( val );
		if (index < 0) index = val;
		int old = comboBox.getSelectedIndex();

		if (index != old) {
			try {
				comboBox.setSelectedIndex( index );
			}
			catch (Exception e) {
				comboBox.insertItemAt( new String( JCustomizerBundle.string( JCustomizerBundle.key24 ) + index), 0 );
				comboBox.setSelectedIndex( 0 );
			}
		}
    }
	else
		if (value instanceof String) {
			int index = indexOf((String)value);
			int old = comboBox.getSelectedIndex();

			if (index >= 0 && index != old) {
				comboBox.setSelectedIndex( index );
			}
		}
	else
		if (value instanceof Object) {
			int index = indexOf( value );
			int old = comboBox.getSelectedIndex();

			if (index >= 0 && index != old) {
				comboBox.setSelectedIndex( index );
			}
		}
}

   /**
    * @return The value of the property.
    * Returns the Integer cooresponding to the Enum value or the
	* index of the selected item.
    */

public Object getValue() {
    int selected = comboBox.getSelectedIndex();
	int index = -1;

    if( enumValues != null )
		index = enumValues[ selected ];
	else
		index = selected;
	return new Integer( index );
}

    //----------------------------------------------------------------------

    /**
     * @return  True if the class will honor the paintValue method.
     */

public boolean isPaintable() {
    return false;
}

    /**
     * Paint a representation of the value into a given area of screen
     * real estate.  Note that the propertyEditor is responsible for doing
     * its own clipping so that it fits into the given rectangle.
     * <p>
     * If the PropertyEditor doesn't honor paint requests (see isPaintable)
     * this method should be a silent noop.
     *
     * @param gfx  Graphics object to paint into.
     * @param box  Rectangle within graphics object into which we should paint.
     */
public void paintValue(java.awt.Graphics gfx, java.awt.Rectangle box) {}

    //----------------------------------------------------------------------

    /**
     * This method is intended for use when generating Java code to set
     * the value of the property.  It should return a fragment of Java code
     * that can be used to initialize a variable with the current property
     * value.
     * <p>
     * Example results are "2", "new Color(127,127,34)", "Color.orange", etc.
     *
     * @return A fragment of Java code representing an initializer for the
     *   	current value.
     */
public String getJavaInitializationString() {
    if (getValue() == null) return "NULL";
    return prepend + getAsText();
}

    //----------------------------------------------------------------------

    /**
     * @return The property value as a human editable string.
     * <p>   Returns null if the value can't be expressed as an editable string.
     * <p>   If a non-null value is returned, then the PropertyEditor should
     *	     be prepared to parse that string back in setAsText().
     */
public String getAsText() {
    return (String)comboBox.getSelectedItem();
}

    /**
     * Set the property value by parsing a given String.  May raise
     * java.lang.IllegalArgumentException if either the String is
     * badly formatted or if this kind of property can't be expressed
     * as text.
     * @param text  The string to be parsed.
     */
public void setAsText(String text) throws java.lang.IllegalArgumentException {
    setValue(text);
    return;
}

    //----------------------------------------------------------------------

    /**
     * If the property value must be one of a set of known tagged values,
     * then this method should return an array of the tags.  This can
     * be used to represent (for example) enum values.  If a PropertyEditor
     * supports tags, then it should support the use of setAsText with
     * a tag value as a way of setting the value and the use of getAsText
     * to identify the current value.
     *
     * @return The tag values for this property.  May be null if this
     *   property cannot be represented as a tagged value.
     *
     */
public String[] getTags() {
    return strings;
}
    //----------------------------------------------------------------------

    /**
     * A PropertyEditor may choose to make available a full custom Component
     * that edits its property value.  It is the responsibility of the
     * PropertyEditor to hook itself up to its editor Component itself and
     * to report property value changes by firing a PropertyChange event.
     * <P>
     * The higher-level code that calls getCustomEditor may either embed
     * the Component in some larger property sheet, or it may put it in
     * its own individual dialog, or ...
     *
     * @return A java.awt.Component that will allow a human to directly
     *      edit the current property value.  May be null if this is
     *	    not supported.
     */

public java.awt.Component getCustomEditor() {
return null;
}
    /**
     * @return  True if the propertyEditor can provide a custom editor.
     */
public boolean supportsCustomEditor() {
    return false;
}

protected PropertyChangeSupport listeners = null;

public void removePropertyChangeListener( PropertyChangeListener listener ) {
    if (listeners == null) {
		return;
    }
	listeners.removePropertyChangeListener( listener );
}

public void addPropertyChangeListener( PropertyChangeListener listener ) {
    if( listeners == null ) {
		listeners = new PropertyChangeSupport( this );
    }
	listeners.addPropertyChangeListener( listener );
}


/**
 * ItemListener method
 * Just fire off a PropertyChangeEvent
 */
public void itemStateChanged( ItemEvent ev ) {
	if( settingObjectItems ) return;
    if( ev.getStateChange() == ItemEvent.DESELECTED ) return;
	if( listeners != null )
		listeners.firePropertyChange( "", null, getValue() );
}

/**
 * Find the index of the provided value
 *
 */
public int indexOf( int enumval ) {
    if( enumValues != null ) {
	for( int i = 0; i < enumValues.length; i++ ) {
	    if( enumValues[ i ] == enumval ) {
		return i;
	    }
	}
    }
    return -1;
}

/**
 * Find the index of the provided value
 *
 */
public int indexOf( String value ) {
    if (enumValues != null) {
	for (int i=0; i < enumValues.length; i++) {
	    if (value.equalsIgnoreCase((String) comboBox.getItemAt( i ))) {
		return i;
	    }
	}
    }
    return -1;
}

public int indexOf( Object item ) {
    if( itemValues != null ) {
		for( int i = 0; i < itemValues.length; i++ ) {
			if( itemValues[i] == item ) {
				return i;
			}
		}
    }
    return -1;
}

/**
 * Enables/disables the control
 *
 * @param enabled true to enable the control,
 * false to disable it.
 */
public void setEnabled(boolean enabled) {
	comboBox.setEnabled(enabled);
}

/**
 * Returns true if the control is enabled.
 */
public boolean isEnabled() {
	return comboBox.isEnabled();
}

private void setItems( String[] items ) {
	comboBox.removeAllItems();
	for( int i = 0; i < items.length; i++ ) {
		comboBox.addItem( new String( items[ i ] ) );
	}
}

private boolean sameStrings( String[] itemStrings ) {
	if( strings == null ) return false;
	boolean same = itemStrings.length == strings.length;

	for( int i = 0; (i < itemStrings.length) && same; i ++ ) {
		if( strings[ i ].compareTo( itemStrings[ i ] ) != 0 )
			same = false;
	}
	return same;
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java JEnumEditor
 */
public static void main(String args[]) {
    JFrame frame;
    try {
	java.lang.Class newclass
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    JEnumEditor jEnumEditor = new JEnumEditor();
    jEnumEditor.setBackground( java.awt.Color.lightGray );
    jEnumEditor.init();
    frame.getContentPane().add( jEnumEditor );
    frame.pack();
    java.awt.Dimension bigness = jEnumEditor.getPreferredSize();
    frame.setSize(bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key162 );
}


public static String getPageName() {
    return "JEnumEditor";
}

}
