/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JFileNameEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:50 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Color;
import java.awt.Component;
import java.awt.FileDialog;
import java.awt.FlowLayout;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyEditor;
import java.beans. PropertyChangeSupport;
import java.io.File;
import java.io.FilenameFilter;



import java.util.Vector;


import javax.swing.JButton;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JFrame;

import com.klg.jclass.util.swing.JCIconCreator;

/**
 * This class is a a simple FilenameFilter. It defines the required
 * accept() method to determine whether a specified file should
 * be listed. A file will be listed if its name ends with the
 * specified extension, or if it is a directory.
 */

class EndsWithFilter implements FilenameFilter {
    private String extension;
    public EndsWithFilter(String extension) {
		this.extension = extension;
    }
    public boolean accept(File dir, String name) {
		if (name.endsWith(extension)) 
			return true;
		else 
			return (new File(dir, name)).isDirectory();
   	}
}

/**
 *  File Name Selector Toolbar
 */
public class JFileNameEditor extends JPropertyPage 
				implements ActionListener, PropertyEditor  {


public JFieldEditor nameField;
public JButton loadButton, saveButton;
private FilenameFilter filter;
protected FileDialog loadDialog = null;
protected FileDialog saveDialog = null;
private String directory;
private String oldDirectory = new String("");
private String filename;
private boolean loadOrSave = true;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
    setLayout(new FlowLayout( FlowLayout.LEFT, 2, 2 ) );

    nameField = new JFieldEditor( "", 12 );
    nameField.setBackground( textBG );
    nameField.addPropertyChangeListener( this );
    add( nameField );

    loadButton = new JButton( JCustomizerBundle.string( JCustomizerBundle.key33 ), new ImageIcon( makeLoadImage( this ) ) );
    loadButton.addActionListener(this);
    add( loadButton );
    saveButton = new JButton( JCustomizerBundle.string( JCustomizerBundle.key34 ), new ImageIcon( makeSaveImage( this ) ) );
    saveButton.addActionListener(this);
    add( saveButton );
}


/**
 * Gets the directory of the Dialog.
 */
public String getDirectory() {
    return directory;
}

/**
 * Gets the file of the Dialog.
 */
public String getFile() {
    return filename;
}

/**
 * Gets a boolean representing the file action to be taken.  True means
 * load, false means save.
 */
public boolean getLoadOrSave() {
    return loadOrSave;
}

private void setDialogFile(FileDialog dialog, String text) {
    if ((text == null) == text.equals("")) {
		dialog.setFile("*");
    }
    if (text.indexOf('*') >= 0) {
		dialog.setFile(text);
    }
    else {
		int index = text.lastIndexOf('.');
		if (index >= 0) {
/* Filters don't work -- java thing, I think
	    	dialog.setFilenameFilter(new 
				     	EndsWithFilter(text.substring(index)));
*/
	      	dialog.setFile("*" + text.substring(index));
		}
    }
}

/**
 * PropertyChangeListener Method
 */
public void propertyChanged( Object source, Object value ) {
    boolean fire = false;
    if (source == nameField) {
		//System.out.println("Name: "+value);
		if( value instanceof String ) {
			filename = (String) value;
		}
		loadOrSave = true;
		fire = true;
    }
    if (fire) {
	   	listeners.firePropertyChange("", null, getValue());
    }
}

protected PropertyChangeSupport listeners = null;

public void removePropertyChangeListener( PropertyChangeListener listener ) {
    if (listeners == null) {
        return;
    }
    listeners.removePropertyChangeListener( listener );
}

public void addPropertyChangeListener( PropertyChangeListener listener ) {
    if( listeners == null ) {
        listeners = new PropertyChangeSupport( this );
    }
    listeners.addPropertyChangeListener( listener );
}

/**
 * ActionListener method
 * Respond to a Load button press by popping up a file
 * dialog and prompting for a data file.
 */
public void actionPerformed( ActionEvent ev ) {
    if (ev.getSource() instanceof JButton) {
	JButton button = (JButton) ev.getSource();
	if (button == loadButton) {
	    if (loadDialog == null) {
			loadDialog = new FileDialog(getDialogFrame(),
						   JCustomizerBundle.string( JCustomizerBundle.key35 ),
						   FileDialog.LOAD);
	    }
	    setDialogFile(loadDialog, nameField.textField.getText());
	    loadDialog.show();
	    directory = loadDialog.getDirectory();
	    filename = loadDialog.getFile();
		loadOrSave = true;
	    boolean update = false;
	    if ((directory != null)
				&& !directory.equals("") 
				&& (!oldDirectory.equals(directory))) {
			oldDirectory = new String(directory);
			update = true;
	    }
	    if ((filename != null)
				&& !filename.equals("")
				&& (!nameField.textField.getText().equals(filename))) {
			nameField.textField.setText(filename);
			update = true;
	    }
	    if (update) {
			//System.out.println("JFileNameEditor fire: "+getValue());
			listeners.firePropertyChange("", null, getValue());
	    }
	}
	else if (button == saveButton) {
	    if (saveDialog == null) {
			saveDialog = new FileDialog(getDialogFrame(),
					    JCustomizerBundle.string( JCustomizerBundle.key36 ),
					    FileDialog.SAVE);
			String name = nameField.textField.getText();
			if (name.indexOf('*') >= 0) {
		    	saveDialog.setFile(name);
			}
	    }
	    setDialogFile(saveDialog, nameField.textField.getText());
	    saveDialog.show();
	    directory = saveDialog.getDirectory();
	    filename = saveDialog.getFile();
		loadOrSave = false;
	    boolean update = false;
	    if ((directory != null)
				&& !directory.equals("")
				&& (!oldDirectory.equals(directory))) {
			oldDirectory = new String(directory);
			update = true;
	    }
	    if ((filename != null)
				&& !filename.equals("")
				&& (!nameField.textField.getText().equals(filename))) {
			nameField.textField.setText(filename);
			update = true;
	    }
	    if (update) {
			//System.out.println("JFileNameEditor fire: "+getValue());
			listeners.firePropertyChange("", null, getValue());
	    }
	}
    }
}

/**
 *  java.beans.PropertyEditor support
 */

    /**
     * Set (or change) the object that is to be edited.  Builtin types such
     * as "int" must be wrapped as the corresponding object type such as
     * "java.lang.Integer".
     *
     * @param value The new target object to be edited.  Note that this
     *     object should not be modified by the PropertyEditor, rather 
     *     the PropertyEditor should create a new object to hold any
     *     modified value.
     */
public void setValue(Object value) {
    if (value instanceof String) {
	// try as a URL? try as a File?
	//System.out.println("JCFileEditor setValue= '"+value+"'");
	String input = (String) value;
	// Is '/' adequately cross-platform and i18n?
	int sepIndex = input.lastIndexOf('/');
	if (sepIndex > 0) {
	    oldDirectory = new String(input.substring(0, sepIndex));
	}
	nameField.setValue(input.substring(sepIndex+1));
    }
}

/**
 * @return The value of the property.  Builtin types such as "int" will
 * be wrapped as the corresponding object type such as "java.lang.Integer".
 */

public Object getValue() {
    return directory + filename;
}

    //----------------------------------------------------------------------

    /**
     * @return  True if the class will honor the paintValue method.
     */

public boolean isPaintable() {
    return false; 
}

/**
 * Paint a representation of the value into a given area of screen
 * real estate.  Note that the propertyEditor is responsible for doing
 * its own clipping so that it fits into the given rectangle.
 * <p>
 * If the PropertyEditor doesn't honor paint requests (see isPaintable)
 * this method should be a silent noop.
 *
 * @param gfx  Graphics object to paint into.
 * @param box  Rectangle within graphics object into which we should paint.
 */
public void paintValue(java.awt.Graphics gfx, java.awt.Rectangle box) {}

    //----------------------------------------------------------------------

/**
 * This method is intended for use when generating Java code to set
 * the value of the property.  It should return a fragment of Java code
 * that can be used to initialize a variable with the current property
 * value.
 * <p>
 * Example results are "2", "new Color(127,127,34)", "Color.orange", etc.
 *
 * @return A fragment of Java code representing an initializer for the
 *   	current value.
 */
public String getJavaInitializationString() {
    // Beef this up!
    return getAsText();
}

  //----------------------------------------------------------------------

/**
 * @return The property value as a human editable string.
 * <p>   Returns null if the value can't be expressed as an editable string.
 * <p>   If a non-null value is returned, then the PropertyEditor should
 *	     be prepared to parse that string back in setAsText().
 */
public String getAsText() {
    return (String) getValue();
}

    /**
     * Set the property value by parsing a given String.  May raise
     * java.lang.IllegalArgumentException if either the String is
     * badly formatted or if this kind of property can't be expressed
     * as text.
     * @param text  The string to be parsed.
     */
public void setAsText(String text) throws java.lang.IllegalArgumentException {
    setValue(text);
    return;
}

    //----------------------------------------------------------------------

    /**
     * If the property value must be one of a set of known tagged values, 
     * then this method should return an array of the tags.  This can
     * be used to represent (for example) enum values.  If a PropertyEditor
     * supports tags, then it should support the use of setAsText with
     * a tag value as a way of setting the value and the use of getAsText
     * to identify the current value.
     *
     * @return The tag values for this property.  May be null if this 
     *   property cannot be represented as a tagged value.
     *	
     */
public String[] getTags() {
    return null;
}
    //----------------------------------------------------------------------

    /**
     * A PropertyEditor may choose to make available a full custom Component
     * that edits its property value.  It is the responsibility of the
     * PropertyEditor to hook itself up to its editor Component itself and
     * to report property value changes by firing a PropertyChange event.
     * <P>
     * The higher-level code that calls getCustomEditor may either embed
     * the Component in some larger property sheet, or it may put it in
     * its own individual dialog, or ...
     *
     * @return A java.awt.Component that will allow a human to directly
     *      edit the current property value.  May be null if this is
     *	    not supported.
     */

public java.awt.Component getCustomEditor() {
return null;
}
    /**
     * @return  True if the propertyEditor can provide a custom editor.
     */
public boolean supportsCustomEditor() {
    return false;
}
  
/**
 * Set the background color of the contained text field
 */
public void setBackground(java.awt.Color color) {
    if (nameField != null) {
	nameField.setBackground(color);
    }
}

static private String[] loadPixels = {
    "                    ",
    "           ddd      ",
    "          d   d d   ",
    "               dd   ",
    "   ...        ddd   ",
    "  .ccc.......       ",
    "  .ggggggggg.       ",
    "  .ggggggggg.       ",
    "  .gggg...........  ",
    "  .ggg.bbbbbbbbb.   ",
    "  .gg.bbbbbbbbb.    ",
    "  .g.bbbbbbbbb.     ",
    "  ..bbbbbbbbb.      ",
    "  ...........       ",
    "                    ",
    "                    "
};

static private String[] savePixels = {
    "                    ",
    "  dddrrrrrrrrrrddd  ",
    "  dcdrrrrrrrrrrdcd  ",
    "  dddccccccccccddd  ",
    "  dddcc.c..c.ccddd  ",
    "  dddccccccccccddd  ",
    "  dddcc...c..ccddd  ",
    "  dddccccccccccddd  ",
    "  dddcc..c.c.ccddd  ",
    "  ddddccccccccdddd  ",
    "  dddddddddddddddd  ",
    "  ddddaaaaaaaadddd  ",
    "  dddda..aaaaadddd  ",
    "  dddda..aaaaadddd  ",
    "  dddda..aaaaadddd  ",
    "   ddda..aaaaadddd  ",
    "    dddddddddddddd  ",
    "                    "
};

/**
 * Creates the Image used in the load button
 */
static Image makeLoadImage( Component comp ) {
    JCIconCreator ic = new JCIconCreator(
					   loadPixels[0].length(),
					   loadPixels.length);
    ic.setColor('.', new Color(0x000000));
    ic.setColor('g', new Color(0x00ff00));
    ic.setColor('b', new Color(0x848400));
    ic.setColor('c', new Color(0xffffff));
    ic.setColor('d', new Color(0x0000ff));
    ic.setPixels(loadPixels);
    return ic.getIcon().getImage();
}


/**
 * Creates the Image used in the save button
 */
static Image makeSaveImage( Component comp ) {
    JCIconCreator ic = new JCIconCreator(
					   savePixels[0].length(),
					   savePixels.length);
    ic.setColor('.', new Color(0x000000));
    ic.setColor('b', new Color(0x848400));
    ic.setColor('c', new Color(0xffffff));
    ic.setColor('d', new Color(0x0000ff));
    ic.setPixels(savePixels);
    return ic.getIcon().getImage();
}


/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java JFileNameEditor
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    JFileNameEditor jFileNameEditor = new JFileNameEditor();
    jFileNameEditor.setBackground( java.awt.Color.lightGray );
    jFileNameEditor.init();
    frame.getContentPane().add( jFileNameEditor );
    frame.pack();
    java.awt.Dimension bigness = jFileNameEditor.getPreferredSize();
    frame.setSize(bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key37 );
}

public static String getPageName() {
    return "JFileNameEditor";
}

}
