/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JFontEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:50 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Color;
import java.awt.Font;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyEditor;

import javax.swing.JButton;
import javax.swing.JTextField;
import javax.swing.JFrame;
import javax.swing.JComboBox;




import java.awt.GraphicsEnvironment;


import com.klg.jclass.util.JCTypeConverter;

/**
 * The JFontEditor class is a general Font selection
 * dialog for setting fonts
 */
public class JFontEditor extends JPropertyPage
			implements ActionListener, ItemListener, AdjustmentListener, PropertyEditor {

protected JComboBox fontNameCombo = null;
protected JComboBox fontStyleCombo = null;
protected JSpinBox fontSizeSpin = null;
protected JTextField fontSizeField = null;
protected JButton useFontButton = null;

static int font_style_values[] = {
    Font.PLAIN, Font.BOLD, Font.ITALIC, Font.BOLD+Font.ITALIC
};
static String font_style_names[] = {
    "Plain", "Bold", "Italic", "BoldItalic"
};

/*
 *  Constructor creates super() and calls init()
 */
public JFontEditor() {
	super.init();
	setToolTipText( getPageTitle() );
	init();
}


String long_label = "      " + JCustomizerBundle.string( JCustomizerBundle.key129 ) + "       ";
String short_label = " " + JCustomizerBundle.string( JCustomizerBundle.key129 ) + " ";

/**
 * Initializes the dialog
 */
public void init() {
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	this.setLayout( gridbag );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.insets = new Insets( 3, 3, 3, 3 );
	constraints.gridy = 0;
	constraints.gridx = 0;




    // Root through list of valid font names removing the sub-family names
    String tmp_fnames[] = GraphicsEnvironment.getLocalGraphicsEnvironment().
                                                getAvailableFontFamilyNames();
    int num_fnames = 0;
    for (int i = 0; i < tmp_fnames.length; i++) {
        if (tmp_fnames[i].indexOf('.') < 0)
            num_fnames++;
    }
    String font_names[] = new String[num_fnames];
    int fname_ind = 0;
    for (int i = 0; i < tmp_fnames.length; i++) {
        if (tmp_fnames[i].indexOf('.') < 0) {
            font_names[fname_ind] = new String(tmp_fnames[i]);
            fname_ind++;
        }
    }


    fontNameCombo = new JComboBox( font_names );
    fontNameCombo.addItemListener( this );
	gridbag.setConstraints( fontNameCombo, constraints );
	constraints.gridx++;
    this.add( fontNameCombo );

    fontStyleCombo = new JComboBox( font_style_names );
    fontStyleCombo.addItemListener( this );
	gridbag.setConstraints( fontStyleCombo, constraints );
	constraints.gridx++;
    this.add( fontStyleCombo );

    fontSizeSpin = new JSpinBox();
    fontSizeSpin.setMinimum( 8 );
    fontSizeSpin.setMaximum( 36 );
    fontSizeField = fontSizeSpin.getTextField();
    fontSizeField.addActionListener( this );
    fontSizeSpin.getScrollBar().addAdjustmentListener( this );
	gridbag.setConstraints( fontSizeSpin, constraints );
	constraints.gridx++;
    this.add( fontSizeSpin );
    
    useFontButton = new JButton( long_label );
    useFontButton.addActionListener(this);
	constraints.gridy++;
	constraints.gridx = 0;
	constraints.gridwidth = 3;
	constraints.anchor = GridBagConstraints.CENTER;
	gridbag.setConstraints( useFontButton, constraints );
    this.add( useFontButton );
}

/**
 *  java.beans.PropertyEditor support
 */

    /**
     * Set (or change) the object that is to be edited.  Builtin types such
     * as "int" must be wrapped as the corresponding object type such as
     * "java.lang.Integer".
     *
     * @param value The new target object to be edited.  Note that this
     *     object should not be modified by the PropertyEditor, rather 
     *     the PropertyEditor should create a new object to hold any
     *     modified value.
     */
public void setValue(Object value) {
    if (value instanceof Font) {
		this.setEditFont( (Font)value );
    }
};

    /**
     * @return The value of the property.  Builtin types such as "int" will
     * be wrapped as the corresponding object type such as "java.lang.Integer".
     */

public Object getValue() {
    return getEditFont();
}

    //----------------------------------------------------------------------

    /**
     * @return  True if the class will honor the paintValue method.
     */

public boolean isPaintable() {
    return false; 
}

    /**
     * Paint a representation of the value into a given area of screen
     * real estate.  Note that the propertyEditor is responsible for doing
     * its own clipping so that it fits into the given rectangle.
     * <p>
     * If the PropertyEditor doesn't honor paint requests (see isPaintable)
     * this method should be a silent noop.
     *
     * @param gfx  Graphics object to paint into.
     * @param box  Rectangle within graphics object into which we should paint.
     */
public void paintValue(java.awt.Graphics gfx, java.awt.Rectangle box) {}

    //----------------------------------------------------------------------

    /**
     * This method is intended for use when generating Java code to set
     * the value of the property.  It should return a fragment of Java code
     * that can be used to initialize a variable with the current property
     * value.
     * <p>
     * Example results are "2", "new Color(127,127,34)", "Color.orange", etc.
     *
     * @return A fragment of Java code representing an initializer for the
     *   	current value.
     */
public String getJavaInitializationString() {
    Font font = getEditFont();
    String style;
    switch (font.getStyle()) {
    case (Font.BOLD):
	style = "Font.BOLD";
	break;
    case (Font.ITALIC):
	style = "Font.ITALIC";
	break;
    case (Font.BOLD + Font.ITALIC):
	style = "Font.BOLD + Font.ITALIC";
	break;
    default:
	style = "Font.PLAIN";
    }

    return "new Font("+font.getName() +", " +style+ ", " +font.getSize() +")";
}

    //----------------------------------------------------------------------

    /**
     * @return The property value as a human editable string.
     * <p>   Returns null if the value can't be expressed as an editable string.
     * <p>   If a non-null value is returned, then the PropertyEditor should
     *	     be prepared to parse that string back in setAsText().
     */
public String getAsText() {
    Font font = getEditFont();
    return font.toString();
}

    /**
     * Set the property value by parsing a given String.  May raise
     * java.lang.IllegalArgumentException if either the String is
     * badly formatted or if this kind of property can't be expressed
     * as text.
     * @param text  The string to be parsed.
     */
public void setAsText(String text) throws java.lang.IllegalArgumentException {
    setValue(text);
    return;
}

    //----------------------------------------------------------------------

    /**
     * If the property value must be one of a set of known tagged values, 
     * then this method should return an array of the tags.  This can
     * be used to represent (for example) enum values.  If a PropertyEditor
     * supports tags, then it should support the use of setAsText with
     * a tag value as a way of setting the value and the use of getAsText
     * to identify the current value.
     *
     * @return The tag values for this property.  May be null if this 
     *   property cannot be represented as a tagged value.
     *	
     */
public String[] getTags() {
    return null;
}
    //----------------------------------------------------------------------

    /**
     * A PropertyEditor may choose to make available a full custom Component
     * that edits its property value.  It is the responsibility of the
     * PropertyEditor to hook itself up to its editor Component itself and
     * to report property value changes by firing a PropertyChange event.
     * <P>
     * The higher-level code that calls getCustomEditor may either embed
     * the Component in some larger property sheet, or it may put it in
     * its own individual dialog, or ...
     *
     * @return A java.awt.Component that will allow a human to directly
     *      edit the current property value.  May be null if this is
     *	    not supported.
     */

public java.awt.Component getCustomEditor() {
return null;
}
    /**
     * @return  True if the propertyEditor can provide a custom editor.
     */
public boolean supportsCustomEditor() {
    return false;
}
  
public Font getEditFont() {
    Font font = null;
    try {
	String fontName = (String)fontNameCombo.getSelectedItem();
	int styleIndex = fontStyleCombo.getSelectedIndex();
	int style = font_style_values[ styleIndex ];
	String fontSize = fontSizeField.getText();
	int size = Integer.parseInt( fontSize );

	font = new Font( fontName, style, size );
    } catch (Exception f) {
    }
    return font;
}

public void setEditFont( Font font ) {
    // Select the items in each list which match the current font.
    String fontName = font.getName();
	int fontNamesNo = fontNameCombo.getItemCount();
    boolean found = false;

    for( int i = 0; i < fontNamesNo; i++ ) {
	if( fontName.equalsIgnoreCase( (String)fontNameCombo.getItemAt( i ) ) ) {
	    if( fontNameCombo.getSelectedIndex() != i ) {
			fontNameCombo.setSelectedIndex( i );
	    }
	    found = true;
	}
    }
    if (!found) {
	try {
	    // Insert a new element into the list
	    fontNameCombo.addItem( fontName );
	    fontNameCombo.setSelectedItem( (Object)fontName );
	} catch (Exception f) {
	    System.out.println( JCustomizerBundle.string( JCustomizerBundle.key24 ) + " "  + 
						    JCustomizerBundle.string( JCustomizerBundle.key112 ) + " " + fontName );
	}
    }

    int style = font.getStyle();
    for( int i=0; i< font_style_values.length; i++ ) {
		if( style == font_style_values[i] ) {
			if( fontStyleCombo.getSelectedIndex() != i ) {
				fontStyleCombo.setSelectedIndex( i );
			}
			break;
		}
    }
    int size = font.getSize();
    fontSizeSpin.setIntValue( size );
    fontChanged();
}

private void fontChanged() {
    try {
	String name = (String)fontNameCombo.getSelectedItem();
	String styleString = (String)fontStyleCombo.getSelectedItem();
	String sizeString = fontSizeField.getText();
	if( name != "" && styleString != "" && sizeString != "" ) {
	    int style = JCTypeConverter.toEnum( styleString, font_style_names, font_style_values, Font.PLAIN );
	    Integer item = new Integer( sizeString );
	    int size = item.intValue();
	    Font font = new Font( name, style, size );
	    if( size > 14 ) {
			useFontButton.setText( short_label );
	    }
	    else {
			useFontButton.setText( long_label );
	    }
	    useFontButton.setFont( font );
	}
    } catch (Exception e) {
	System.out.println( JCustomizerBundle.string( JCustomizerBundle.key130 ) +  
						JCustomizerBundle.string( JCustomizerBundle.key112 ) + ":" + e.getMessage() );
    }
}

private void comboChanged( ItemEvent ev ) {
    if( ev.getStateChange() == ItemEvent.DESELECTED ) return;
	JComboBox box = (JComboBox) ev.getSource();

    if( box == fontNameCombo || box == fontStyleCombo ) {
		fontChanged();
    }
}

private void textChanged( ActionEvent ev ) {
    JTextField field = (JTextField) ev.getSource();

    if( field == fontSizeField ) {
		fontChanged();
    }
}

// ItemListener method
public void itemStateChanged( ItemEvent ev ) {
    if( ev.getSource() instanceof JComboBox) {
	this.comboChanged( ev );
    }
}

// ActionListener method
public void actionPerformed( ActionEvent ev ) {
    if( ev.getSource() instanceof JTextField ) {
	try {
		fontSizeSpin.setTextValue( ev.getActionCommand() );
	    this.textChanged( ev );
	}
	catch( Exception e ) {
	    System.out.println( JCustomizerBundle.string( JCustomizerBundle.key130 ) + ": " + e.getMessage() );
	}
    } else if (ev.getSource() instanceof JButton) {
		if( listeners != null )
			listeners.firePropertyChange( "", null, getValue() );
    }
}


public void adjustmentValueChanged( AdjustmentEvent e ) {
	if( e != null && e.getSource() == fontSizeSpin.getScrollBar() ) {
		fontSizeSpin.setIntValue( e.getValue() );
		fontChanged();
	}
}


/**
 * Set the background color of the contained text fields
 */
public void setBackground( java.awt.Color color ) {
	if( fontNameCombo != null )
		fontNameCombo.setBackground( color );
	if( fontStyleCombo != null )
		fontStyleCombo.setBackground( color );
	if( fontSizeField != null )
		fontSizeField.setBackground( color );
}

protected PropertyChangeSupport listeners = null;

public void removePropertyChangeListener( PropertyChangeListener listener ) {
    if (listeners == null) {
		return;
    }
	listeners.removePropertyChangeListener( listener );
}

public void addPropertyChangeListener( PropertyChangeListener listener ) {
    if( listeners == null ) {
		listeners = new PropertyChangeSupport( this );
    }
	listeners.addPropertyChangeListener( listener );
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java JFontEditor
 */
public static void main(String args[]) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    JFontEditor jFontEditor = new JFontEditor();
    jFontEditor.setBackground( java.awt.Color.lightGray );
    frame.getContentPane().add( jFontEditor );
	jFontEditor.setValue( new Font("Dialog", Font.BOLD, 12 ) );
    frame.pack();
    java.awt.Dimension bigness = jFontEditor.getPreferredSize();
    frame.setSize(bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key131 );
}


public static String getPageName() {
    return "JFontEditor";
}

}
