/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JPropertyPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:50 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Color;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.Component;
import java.awt.Frame;



import java.util.Vector;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.beans.Customizer;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JApplet;

import com.klg.jclass.chart.JCCustomizerPage;
/**
 * Utility Base class for Dialogs
 */
class DialogFrame extends JFrame {

private static int instances = 0;

public DialogFrame() {
    super();
    addInstance();
}

public DialogFrame(String s) {
    super(s);
    addInstance();
}

public synchronized void dispose() {
	Component [] comps = getContentPane().getComponents();
	for(int i = 0; i < comps.length; i++) {
		if(comps[i] instanceof JApplet) 
			((JApplet)comps[i]).destroy();
	}
	super.dispose();
	removeInstance();
}

private void addInstance() {
	setDefaultCloseOperation( DISPOSE_ON_CLOSE );
    instances++;
}

private void removeInstance() {
    instances--;
}

}

/**
 * Base class for Property Pages
 */
public abstract class JPropertyPage extends JCCustomizerPage 
							implements Customizer, PropertyChangeListener {

protected static Color textBG = new Color( 232, 232, 232 );
protected static Color customBG = Color.lightGray;
protected static Color outlinerBG = Color.white;
protected static Color testBG = Color.cyan;
protected static Color impactBG = Color.darkGray;
protected static Color impactFG = Color.yellow;

protected static Vector pages = new Vector();

protected Object target = null;
protected boolean init = false;

/**
 * Method called to initialize the page
 */
public void init() {
    setLayout( new GridLayout( 1, 1 ) );
	setToolTipText( getPageTitle() );
	pages.addElement( this );
	init = true;
}

/**
 * Method called to update the page with values from the current target
 * equivalent to 'update'.
 */
public void setObject( Object source ) {
	target = source;
}

public Object getObject() {
    return target;
}

public void setObject() {
    setObject( getObject() );
}

public void propertyChange( PropertyChangeEvent ev ) {
    propertyChanged( ev.getSource(), ev.getNewValue() );
}


public void propertyChanged( Object source, Object value ) {
    if (target == null) return;
}

protected PropertyChangeSupport listeners = null;

public void removePropertyChangeListener( PropertyChangeListener listener ) {
    if (listeners == null) {
		return;
    }
	listeners.removePropertyChangeListener( listener );
}

public void addPropertyChangeListener( PropertyChangeListener listener ) {
    if( listeners == null ) {
		listeners = new PropertyChangeSupport( this );
    }
	listeners.addPropertyChangeListener( listener );
}

/**
 * Utility method to return the instance of the given target class
 * being edited by a parent of the give page.
 */
public static Object getObjectByClass( JPropertyPage page, String target ) {
    Container parent;
    do {
	Object object = page.getObject();
	if (object.getClass().getName().equals(target)) {
	    return object;
	}
	parent = page.getParent();
	while( (parent != null ) && !( parent instanceof JPropertyPage) ) {
	    parent = parent.getParent();
	} 
	if (parent == null) break;
	page = (JPropertyPage) parent;
    } while ( page instanceof JPropertyPage);
    return null;
}

/**
 * Utility method to return a frame for placing a propery page in
 */
public JFrame getDialogFrame() {
    JFrame frame = new DialogFrame();
    return frame;
}
/**
 * Utility method to return a frame for placing a propery page in
 */
public JFrame getPropertyFrame() {
    JFrame frame;
    String bestname = CUSTOM_PACKAGE + "." + CUSTOM_FRAME;
    //System.out.println("Creating " + bestname);
    try {
	java.lang.Class newclass = java.lang.Class.forName( bestname );
	frame = (JFrame) newclass.newInstance();
	return frame;
    } catch( Exception ex ) {
	showError( JCustomizerBundle.string( JCustomizerBundle.key2 ) + " " + bestname );
	return super.getPropertyFrame();
    }
}

/**
 * Utility method to return a PropertyPage subclass
 * given a string
 * For example, getPage("JColorEditor") would return
 * and instance of this class ready to be initialized.
 *
 * If this class cannot be created, this method
 * attepts to returns a default Property Page.
 *
 * If this attempt fails too, return null 
 * Developers: If this occurs, verity CUSTOM_PACKAGE
 */

public static JPropertyPage getPage( String classname ) {
    String fullname = CUSTOM_PACKAGE + "." + classname;
    JPropertyPage test;
    try {
	test = (JPropertyPage) Class.forName( fullname ).newInstance();
    } catch (Throwable e) {
	//System.out.println("Error1: "+ e);
	try {
	    String defaultpage = CUSTOM_PACKAGE + "." + DEFAULT_PAGE_NAME;
	    test = (JPropertyPage) 
		java.lang.Class.forName(defaultpage).newInstance();
	    test.setObject(classname);
	} 
	catch (Throwable e2) {
	    //System.out.println("Error2: "+ e2);
	    System.out.println( JCustomizerBundle.string( JCustomizerBundle.key2 ) + " " + classname );
	    test = null;
	}
    }
    return test;
}


/**
 * Utility method to return a PropertyPage subclass
 * in a default frame given a class name string
 * For example, getPage("JColorEditor") would return
 * and instance of this class initialized
 * and ready to be shown
 */
public JPropertyPage getFramedPage( String classname ) {
    JPropertyPage test = getPage( classname );
    if (test != null) {
	test.init();

	JFrame frame = (JFrame)getPropertyFrame();

	if( classname.compareTo( OutlinedPropertyPage.labelOutline[ 1 ] ) == 0 || 
		classname.compareTo( OutlinedPropertyPage.axesOutline[ 1 ] ) == 0 ||
		classname.compareTo( OutlinedPropertyPage.viewsOutline[ 1 ] ) == 0 )
		frame.getJMenuBar().remove( 0 );
	frame.getContentPane().setLayout( new GridLayout( 1, 1 ) );
	frame.setBackground( Color.lightGray );
	frame.getContentPane().add( test );
	resizeFrame( test );
	frame.show();
    }
    return test;
}

/**
 * Utility method resize the frame to fit the current contents
 */
public static void resizeFrame( JComponent test) {
    JFrame frame = (JFrame)getFrame( test );
    frame.pack();
    java.awt.Dimension bigness = test.getPreferredSize();
    // Leave room for left and right spacing,
    // Frame title area, Frame Menubar area
	frame.setSize( bigness.width + 10, bigness.height + 50 );
}


public static String CUSTOM_PACKAGE = "com.klg.jclass.chart.customizer";
public static String CUSTOM_FRAME = "ChartCustomizerFrame";
public static String DEFAULT_PAGE_NAME = "DefaultPropertyPage";

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key3 );
}

public static String getPageName() {
    return "JPropertyPage";
}

}


