/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JSpinBox.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:51 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.FlowLayout;
import java.awt.Dimension;

import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JTextField;
import javax.swing.BorderFactory;
import javax.swing.border.BevelBorder;

public class JSpinBox extends JPanel {

	private JTextField textField = null;
	private JScrollBar scrollBar = null;
	private int minValue = 0;//Integer.MIN_VALUE;
	private int maxValue = Integer.MAX_VALUE;
	private int value = 0;
	private int extent = 1;
	private String[] strings = null;
	private boolean isStringsSpin = false;

	public JSpinBox() {
		this( 0 );
	}

	public JSpinBox( int start ) {
		super();
		setLayout( new FlowLayout( FlowLayout.LEFT, 2 , 2 ) );
		setBorder( BorderFactory.createBevelBorder( BevelBorder.LOWERED ) );
		value = start;
		textField = new JTextField( "", 5 );
		textField.setText( String.valueOf( start ) );
		add( textField );
		scrollBar = new JScrollBar( JScrollBar.VERTICAL, start, extent , minValue, maxValue );
		scrollBar.setPreferredSize( new Dimension( 20, 25 ) );
		scrollBar.setUnitIncrement( -1 );
		add( scrollBar );
	}

	public JSpinBox( String[] _strings ) {
		super();
		strings = _strings;
		setLayout( new FlowLayout( FlowLayout.LEFT, 2 , 2 ) );
		setBorder( BorderFactory.createBevelBorder( BevelBorder.RAISED ) );
		value = 0;
		textField = new JTextField( "", 10 );
		textField.setText( strings[ value ] );
		add( textField );
		scrollBar = new JScrollBar( JScrollBar.VERTICAL, 0, 1 , 0, strings.length );
		minValue = 0;
		maxValue = strings.length;
		scrollBar.setPreferredSize( new Dimension( 20, 25 ) );
		scrollBar.setUnitIncrement( -1 );
		add( scrollBar );
		isStringsSpin = true;
	}

	public JScrollBar getScrollBar() {
		return scrollBar;
	}

	public JTextField getTextField() {
		return textField;
	}

	public void setIntValue( int val ) {
		setValue( val );
	}

	public void setTextValue( String stringValue ) {
		try {
			int val = isStringsSpin ? getStringIndex( stringValue ) : Integer.valueOf( stringValue ).intValue();

			if( isStringsSpin && val == -1 )
				throw new NumberFormatException();
			setValue( val );
		}
		catch( NumberFormatException e ) {
			if( !isStringsSpin )
				textField.setText( String.valueOf( value ) );
			else
				textField.setText( strings[ value ] );
			scrollBar.setValue( value );
		}
	}

	public int getIntValue() {
		return value;
	}

	public String getTextValue() {
		return textField.getText();
	}

	public void setMaximum( int max ) {
		maxValue = max;
		if (max != Integer.MAX_VALUE) {
			scrollBar.setMaximum( max + extent );
		} else {
			scrollBar.setMaximum( max );
		}
		if( maxValue < value ) {
			value = maxValue;
			setIntValue( value );
		}
	}

	public int getMaximum() {
		return maxValue;
	}

	public void setMinimum( int min ) {
		minValue = min;
		scrollBar.setMinimum( min );
		if( value < minValue ) {
			value = minValue;
			setIntValue( value );
		}
	}

	public int getMinimum() {
		return minValue;
	}

	public void setBackground( java.awt.Color color ) {
		if( textField != null )
			textField.setBackground( color );
	}

	private void setValue( int val ) {
		if( val >= minValue && val <= maxValue ) {
			value = val;
			scrollBar.setValue( value );
			if( !isStringsSpin )
				textField.setText( String.valueOf( value ) );
			else
				textField.setText( strings[ value ] );
		}
		else if( val < minValue ) {
				scrollBar.setValue( minValue );
			if( !isStringsSpin )
				textField.setText( String.valueOf( minValue ) );
			else
				textField.setText( strings[ minValue ] );
			}
			else {
				scrollBar.setValue( maxValue );
			if( !isStringsSpin )
				textField.setText( String.valueOf( maxValue ) );
			else
				textField.setText( strings[ maxValue ] );
			}
	}

	private int getStringIndex( String s ) {
		int index = -1;

		for( int i = 0; i < strings.length; i++ )
			if( s.compareTo( strings[ i ] ) == 0 ) {
				index = i;
				break;
			}
		return index;
	}

}
