/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: LabelValuePage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:51 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.AdjustmentListener;
import java.awt.event.AdjustmentEvent;

import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.BorderFactory;
import javax.swing.JButton;

import com.klg.jclass.chart.JCAxis;

/**
 *  Axis Label by Value Property Page
 */
public class LabelValuePage extends JPropertyPage implements AdjustmentListener {

private JDoubleEditor numField = null;
private JBooleanEditor numDefault = null;
private JDoubleEditor tickField = null;
private JBooleanEditor tickDefault = null;
private JSpinBox precisionField = null;
private JBooleanEditor precisionDefault = null;

private JCAxis target;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 10, 10, 5, 10 );
	constraints.weightx = 1.0;

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key269 ) + ":" ) );
    numField = new JDoubleEditor( 4 );
    numField.addPropertyChangeListener( this );
    add( numField );
    numDefault = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key124 ) );
    numDefault.addPropertyChangeListener(this);
	gridbag.setConstraints( numDefault, constraints );
    add( numDefault );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key270 ) + ":" ) );
    tickField = new JDoubleEditor( 4 );
    tickField.addPropertyChangeListener(this);
    add( tickField );
    tickDefault = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key124 ) );
    tickDefault.addPropertyChangeListener( this );
	gridbag.setConstraints( tickDefault, constraints );
    add( tickDefault );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key271 ) + ":" ) );
    precisionField = new JSpinBox( 4 );
    precisionField.setMinimum( -6 );
    precisionField.setMaximum( 6 );
    precisionField.getScrollBar().addAdjustmentListener( this );
    add( precisionField );
    precisionDefault = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key124 ) );
    precisionDefault.addPropertyChangeListener( this );
	gridbag.setConstraints( precisionDefault, constraints );
    add( precisionDefault );
}

/**
 * Method called to initialize the page
 */
public void setObject(Object source) {
    //System.out.println("AP setObject:"+source);
    boolean useDefault;
    String  value;
    if( source instanceof JCAxis ) {
		target = (JCAxis) source;

	value = String.valueOf( target.getNumSpacing() );
	if( value != numField.textField.getText() ) {
	    numField.setValue( value );
	}
	useDefault = target.getNumSpacingIsDefault();
	numDefault.setValue( new Boolean( useDefault ) );
	numDefault.setEnabled( !useDefault );
	value = String.valueOf( target.getTickSpacing() );
	if( value != tickField.textField.getText() ) {
	    tickField.setValue( value );
	}
	useDefault = target.getTickSpacingIsDefault();
	tickDefault.setValue( new Boolean( useDefault ) );
	tickDefault.setEnabled( !useDefault );

	useDefault = target.getPrecisionIsDefault();
	int precision = target.getPrecision();
	if( precision != precisionField.getIntValue() ) {
	    precisionField.setIntValue( precision );
	}
	precisionDefault.setValue( new Boolean( useDefault ) );
	precisionDefault.setEnabled( !useDefault );
    }
}

public void propertyChanged( Object source, Object value ) {
    if( target == null ) return;
	if( source == null ) return;
    if( source instanceof JFieldEditor ) {
		JFieldEditor field = (JFieldEditor)source;
		Double val = (Double)value;

		if( field == numField ) {
			target.setNumSpacing( val.doubleValue() );
		}
		else if( field == tickField) {
			target.setTickSpacing( val.doubleValue() );
		}
		this.setObject( target );
	}
	else if( source instanceof JBooleanEditor ) {
		JBooleanEditor box = (JBooleanEditor)source;
		Boolean val = (Boolean)value;

		if(	box == numDefault ) {
			target.setNumSpacingIsDefault( val.booleanValue() );
		}
		else if( box == tickDefault ) {
			target.setTickSpacingIsDefault( val.booleanValue() );
		} 
		else if( box == precisionDefault ) {
			target.setPrecisionIsDefault( val.booleanValue() );
		}
		this.setObject( target );
	}
}

public void adjustmentValueChanged( AdjustmentEvent e ) {
    if( target == null) return;
	if( e != null && e.getSource() instanceof JScrollBar ) {
		if( e.getSource() == precisionField.getScrollBar() ) {
			precisionField.setIntValue( e.getValue() );
			target.setPrecision( precisionField.getIntValue() );
			this.setObject( target );
		}
	}
}


/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java LabelValuePage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    LabelValuePage labelValuePage = new LabelValuePage();
    labelValuePage.setBackground( java.awt.Color.lightGray );
    labelValuePage.init();
	labelValuePage.setObject( new com.klg.jclass.chart.JCAxis());
    frame.getContentPane().add( labelValuePage );
    frame.pack();
    java.awt.Dimension bigness = labelValuePage.getPreferredSize();
    frame.setSize(bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key272 );
}


public static String getPageName() {
    return "LabelValuePage";
}

}

