/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: LegendGeneralPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:51 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Insets;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.BorderFactory;

import com.klg.jclass.chart.JCChart;
import com.klg.jclass.util.legend.JCLegendEnumMappings;
import com.klg.jclass.util.legend.JCLegend;

/**
 * Chart Legend Property Page
 */
public class LegendGeneralPage extends JPropertyPage {

private JBooleanEditor visibleCheck = null;
private JEnumEditor anchorCombo;
private JEnumEditor orientCombo;
private JFontEditor legendFont = null;

private JCLegend legend;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 10, 10, 5, 10 );
	constraints.weightx = 1.0;
    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key27 ) + ":" ) );
    visibleCheck = new JBooleanEditor( "" );
    visibleCheck.addPropertyChangeListener( this );
	gridbag.setConstraints( visibleCheck, constraints );
    add( visibleCheck );
    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key168 ) + ":" ) );
    anchorCombo = new JEnumEditor( JCLegendEnumMappings.anchor_i18n_strings, 
									JCLegendEnumMappings.anchor_values );
    anchorCombo.addPropertyChangeListener( this );
	gridbag.setConstraints( anchorCombo, constraints );
    add( anchorCombo );
    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key169 ) + ":" ) );
    orientCombo = new JEnumEditor(JCLegendEnumMappings.orientation_i18n_strings, 
									JCLegendEnumMappings.orientation_values );
    orientCombo.addPropertyChangeListener(this);
	gridbag.setConstraints( orientCombo, constraints );
    add( orientCombo );
    add(new JLabel( JCustomizerBundle.string( JCustomizerBundle.key112 ) + ":" ) );
    legendFont = new JFontEditor();
    legendFont.addPropertyChangeListener( this );
	gridbag.setConstraints( legendFont, constraints );
    add( legendFont );
}

/**
 * Method called to return the target that this page is editing
 */
public Object getObject() {
    return legend;
}

/**
 * Method called to initialize the page with values extracted from
 * the given target
 */
public void setObject(Object source) {
    if (source instanceof JCChart) {
	legend = ((JCChart) source).getLegend();
    }
    else if (source instanceof JCLegend) {
	legend = (JCLegend) source;
    }
    if (legend != null) {
	visibleCheck.setValue(new Boolean(legend.isVisible()));
	anchorCombo.setValue(new Integer(legend.getAnchor()));
	orientCombo.setValue(new Integer(legend.getOrientation()));
	legendFont.setValue(legend.getFont());	
    }
}

/**
 * This method is called when a specialized property editor's 
 * value is changed.
 */
public void propertyChanged(Object source, Object value) {
    if (legend == null || value == null) return;
    if (source == visibleCheck) {
	boolean toggle = ((Boolean)value).booleanValue();
	legend.setVisible(toggle);
    } else if (source == anchorCombo) {
	try {
	    legend.setAnchor(((Integer)value).intValue());
	} catch (Exception ex) {
	    System.out.println( JCustomizerBundle.string( JCustomizerBundle.key170 ) + ": " + ex.getMessage() );
	}
    } else if (source == orientCombo) {
	try {
	    legend.setOrientation(((Integer)value).intValue());
	} catch (Exception ex) {
	    System.out.println( JCustomizerBundle.string( JCustomizerBundle.key170 ) + ": " + ex.getMessage() );
	}
    }
    else if (source == legendFont) {
	legend.setFont((java.awt.Font)value);
    }

}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java LegendGeneralPage
 */
public static void main(String args[]) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    LegendGeneralPage legendGeneralPage = new LegendGeneralPage();
    legendGeneralPage.setBackground( java.awt.Color.lightGray );
    legendGeneralPage.init();
    frame.getContentPane().add( legendGeneralPage );
    frame.pack();
    java.awt.Dimension bigness = legendGeneralPage.getPreferredSize();
    frame.setSize(bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key171 );
}

public static String getPageName() {
    return "LegendGeneralPage";
}

}

