/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: LineStylePage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:51 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Insets;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Color;

import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.BorderFactory;


import com.klg.jclass.chart.JCChartStyle;
import com.klg.jclass.chart.JCLineStyle;
import com.klg.jclass.chart.JCChartEnumMappings;
/**
 *  Line Style Property Page
 *  exposes Line style properties
 */
public class LineStylePage extends JPropertyPage {

protected JColorEditor   lineColor;
protected JIntegerEditor lineWidth;
protected JEnumEditor    linePattern;

protected JEnumEditor    lineJoin;
protected JEnumEditor    lineCap;


private JCChartStyle chartStyle = null;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 10, 10, 5, 10 );
	constraints.weightx = 1.0;
    JLabel sample = new JLabel( JCustomizerBundle.string( JCustomizerBundle.key115 ) + ":" );
    add( sample );
    lineColor = new JColorEditor();
    lineColor.addPropertyChangeListener( this );
	gridbag.setConstraints( lineColor, constraints );
    add( lineColor );

    sample = new JLabel( JCustomizerBundle.string( JCustomizerBundle.key116 ) + ":" );
    add( sample );
    lineWidth = new JIntegerEditor();
    lineWidth.addPropertyChangeListener( this );
	gridbag.setConstraints( lineWidth, constraints );
    add( lineWidth );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key117 ) + ":" ) ) ;
    linePattern = new JEnumEditor( JCChartEnumMappings.linePattern_i18n_strings,
									JCChartEnumMappings.linePattern_values );
    linePattern.addPropertyChangeListener( this );
	gridbag.setConstraints( linePattern, constraints );
    add( linePattern );


    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key54 ) + ":" ) ) ;
    lineJoin = new JEnumEditor( JCChartEnumMappings.lineJoin_i18n_strings,
									JCChartEnumMappings.lineJoin_values );
    lineJoin.addPropertyChangeListener( this );
	gridbag.setConstraints( lineJoin, constraints );
    add( lineJoin );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key55 ) + ":" ) ) ;
    lineCap = new JEnumEditor( JCChartEnumMappings.lineCap_i18n_strings,
									JCChartEnumMappings.lineCap_values );
    lineCap.addPropertyChangeListener( this );
	gridbag.setConstraints( lineCap, constraints );
    add( lineCap );

}

/**
 * Method called to populate the dialog with the source's properties
 */
public void setObject(Object source) {
    //System.out.println("PSP setObject: "+source);
    if (source instanceof JCChartStyle) {
		chartStyle = (JCChartStyle) source;
		lineColor.setValue(chartStyle.getLineColor());
		lineWidth.setValue(new Integer(chartStyle.getLineWidth()));
		linePattern.setValue(new Integer(chartStyle.getLinePattern()));

		lineJoin.setValue(new Integer(chartStyle.getLineJoin()));
		lineCap.setValue(new Integer(chartStyle.getLineCap()));

    }
}

/**
 * Return the Object that this Property Page is editing
 */
public Object getObject() {
    return (Object) chartStyle;
}

/**
 * This method is called when a specialized property editor's
 * value is changed.
 */
public void propertyChanged(Object source, Object value) {
    if (value == null) return;
    if (source == lineColor) {
	if (chartStyle != null) {
	    chartStyle.setLineColor((Color)value);
	}
    }
    else if (source == lineWidth) {
	if (chartStyle != null) {
	    chartStyle.setLineWidth(((Integer)value).intValue());
	}
    }
    else if (source == linePattern) {
	if (chartStyle != null) {
	    chartStyle.setLinePattern(((Integer)value).intValue());
	}
    }

    else if (source == lineJoin) {
	if (chartStyle != null) {
	    chartStyle.setLineJoin(((Integer)value).intValue());
	}
    }
    else if (source == lineCap) {
	if (chartStyle != null) {
	    chartStyle.setLineCap(((Integer)value).intValue());
	}
    }

}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java LineStylePage
 */
public static void main(String args[]) {
    JFrame frame;
    try {
	java.lang.Class newclass
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    LineStylePage lineStylePage = new LineStylePage();
    lineStylePage.setBackground( java.awt.Color.lightGray );
    lineStylePage.init();
    frame.getContentPane().add( lineStylePage );
    frame.pack();
    java.awt.Dimension bigness = lineStylePage.getPreferredSize();
    frame.setSize(bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key118 );
}

public static String getPageName() {
    return "LineStylePage";
}

}

