/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: OutlinedPropertyPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:51 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Rectangle;
import java.util.Enumeration;
import java.util.StringTokenizer;

import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.tree.TreePath;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.BorderFactory;

import com.klg.jclass.chart.JCChart;

/**
 *  Base Class for JClass Chart Customizer with
 *  a tree on the left selecting the target of
 *  the a Property Page on the right
 */
public class OutlinedPropertyPage extends JPropertyPage
                    implements TreeSelectionListener, ActionListener {

protected JButtonUserData addButton, removeButton;
protected JPropertyPage editor = null;
protected JAddRemoveDialog dialog = null;
protected JLabel title = null;
protected JCustomTree tree = null;

private static int EDITOR_NAME = 0;
private static int DIALOG_NAME = 1;
protected static String outlineDesc[] = { "PropertyPage", null };
protected static String labelOutline[] = { "ChartLabelTabs", "JChartLabelDialog" };
protected static String axesOutline[] = { "AxisCustomizer", "JCAxisDialog" };
protected static String viewsOutline[] = { "DataViewCustomizer", "ChartDataViewDialog" };
protected static String seriesOutline[] = { "DataViewDataPage", null };
protected static String dataStyleOutline[] = {"DataStyleTabs", null };

private boolean new_page = true;
/**
 * return a list of (Tab Label, TabPage Class) pairs
 */
public String[] getOutlineDesc() {
    return outlineDesc;
}

/**
 * Method called to initialize the page
 */
public void init() {
    super.init();
    setToolTipText( getPageTitle() );
    setLayout( new BorderLayout() );

    String desc[] = this.getOutlineDesc();

    if( desc != null) {
        if( tree == null ) {
            tree = new JCustomTree( new JCustomNode() );
            tree.addTreeSelectionListener( this );
        }
    }
    JPanel p = new JPanel();
    p.setBorder( BorderFactory.createEmptyBorder( 0, 4, 2, 2 ) );
    p.setLayout( new BorderLayout() );
    title = new JLabel( JCustomizerBundle.string( JCustomizerBundle.key138 ) + ":" );
    title.setVerticalAlignment( SwingConstants.BOTTOM );
    title.setHorizontalAlignment( SwingConstants.LEFT );
    title.setFont( new Font( "Dialog", Font.BOLD, 14 ) );
    title.setForeground( Color.darkGray );
    p.add( title, BorderLayout.NORTH );
    JScrollPane treePanel = new JScrollPane( tree );
    p.add( treePanel, BorderLayout.CENTER );

    JPanel buttons = new JPanel();

    buttons.setLayout( new GridLayout( 2, 1 ) );
    addButton = new JButtonUserData( JCustomizerBundle.string( JCustomizerBundle.key133 ) );
    addButton.setUserData( (Object)desc[ DIALOG_NAME ] );
    addButton.addActionListener( this );
    buttons.add( addButton );
    removeButton = new JButtonUserData( JCustomizerBundle.string( JCustomizerBundle.key135 ) );
    removeButton.setUserData( (Object) desc[DIALOG_NAME] );
    removeButton.addActionListener( this );
    buttons.add( removeButton );
    p.add( buttons, BorderLayout.SOUTH );
    if( desc[DIALOG_NAME] == null ) {
        addButton.setEnabled( false );
        removeButton.setEnabled( false );
    }
    add( p, BorderLayout.WEST );

    editor = null;
    try {
        String className = CUSTOM_PACKAGE + "." + desc[ EDITOR_NAME ];
        java.lang.Class newclass = java.lang.Class.forName( className );
        editor = (JPropertyPage) newclass.newInstance();
        editor.init();
    } catch (Exception ex) {
        // if failed with our package, try on it's own
        try {
            String className = desc[ EDITOR_NAME ];
            java.lang.Class newclass = java.lang.Class.forName( className );
            editor = (JPropertyPage) newclass.newInstance();
            editor.init();
        } catch (Exception ex2) {
            editor = (JPropertyPage) new DefaultPropertyPage();
            editor.init();
            editor.setObject( desc[ EDITOR_NAME ] );
        }
    } finally {
        add( editor, BorderLayout.CENTER );
    }
}

/**
 * Method called to populate the dialog with the source's properties
 */
public void setObject( Object source ) {
}

/**
 * Method called to populate the tree the source's items
 */
public void setNode( JCustomNode parent, String label, int index, Object userData ) {
    //System.out.println("OPP setNode:"+label+index+userData);
    int childrenCount = parent.getChildCount();
    if( label != null ) {
        tree.addNode( parent, label, index ).setUserData( userData );
    }
    else if( parent.children() != null && index < childrenCount ) {
        // remove all children below this point
        for( int i = childrenCount - 1; i > (index - 1); i-- ) {
            tree.removeNode( (JCustomNode)parent.getChildAt( i ) );
        }
    }
}

/**
 * Utility routine to return the Object stored in the
 * currently selected item
 */
public Object getSelectedData() {
    Object data = null;
    JCustomNode node = tree.getSelectedNode();

    if( node != null ) {
        data = node.getUserData();
    }
    return data;
}

/**
 * Utility routine select the first item in the list
 * unless an item is already selected
 */
public void initSelection( boolean selectFirst ) {
    TreePath oldSelectionPath = tree.getSelectionPath();
    tree.initSelection( selectFirst || new_page );
    if( new_page )
        new_page = false;
    tree.fireValueChanged( new TreeSelectionEvent( tree, tree.getSelectionPath(), true, oldSelectionPath, tree.getSelectionPath() ) );
}

/**
 * Utility routine to return the name of the class being
 * edited in the current series
 */
public String getItemName() {
    Object target = editor.getObject();
    if (target == null) return null;
    String bigclass = target.getClass().getName();
    if (bigclass == null) return null;
    return bigclass.substring( bigclass.lastIndexOf( "." ) + 1 );
}

/**
 * ActionListener method
 * Called when a node is selected or a button pressed
 */
public void actionPerformed( ActionEvent evt ) {
    if( evt.getSource() instanceof JButtonUserData ) {
        JButtonUserData button = (JButtonUserData)evt.getSource();
        if( dialog == null ) {
            String classname = (String)button.getUserData();

            if (classname == null) return;
            try {
                dialog = (JAddRemoveDialog)getFramedPage( classname );
            }
            catch( Throwable e ) {
                dialog = null;
            }
            if( dialog == null ) {
                showError( classname + " " + JCustomizerBundle.string( JCustomizerBundle.key6 ) + "." );
                button.setEnabled( false );
            }
            else {
                dialog.addPropertyChangeListener( this );
            }
        }
        if( dialog != null ) {
            dialog.setModel( this.getObject() );

            JFrame frame = (JFrame)JPropertyPage.getFrame( dialog );

            if( button == removeButton ) {
                dialog.setObject( getSelectedData() );
                frame.setTitle( JCustomizerBundle.string( JCustomizerBundle.key135 ) + " " + getItemName() );
                dialog.setMode(JAddRemoveDialog.MODE_REMOVE );
            }
            else if( button == addButton ) {
                try {
                    Object test = editor.getObject().getClass().newInstance();

                    dialog.setObject( test );
                    frame.setTitle( JCustomizerBundle.string( JCustomizerBundle.key133 ) + " " + getItemName() );
                    dialog.setMode( JAddRemoveDialog.MODE_ADD );
                }
                catch( Throwable e ) {
                    frame.setTitle( JCustomizerBundle.string( JCustomizerBundle.key2 ) + " " + getItemName() );
                }
            }
            frame.show();
        }
    }
}

// Overridden in DataViewList subclass.
protected void updateRemoveButton() {
    // Do nothing.
}

public void valueChanged( TreeSelectionEvent e ) {
    if( e != null && e.getSource() instanceof JTree ) {
        if( e.getSource() == tree ) {
            Object userData = getSelectedData();
            if( userData != null && editor != null ) {
                editor.setObject( userData );
            }
            updateRemoveButton();
        }
    }
}

/**
 * This method is called when a specialized property editor's
 * value is changed.
 */
public void propertyChanged( Object source, Object value ) {
    if (source == dialog) {
    if (dialog.getMode() == JAddRemoveDialog.MODE_REMOVE) {
        // remove value from current Object
    }
    else if (dialog.getMode() == JAddRemoveDialog.MODE_ADD) {
    }
    // Update
    setObject();
    }
}

/**
 * Reduce text to a size of 10 characters (plus ellipses).
 * @param text text to reduce
 * @return reduced text
 */
public String reduceText(String text)
{
    return reduceText(text, 10);
}

/**
 * Reduce text of large labels down to something manageable by first
 * getting rid of HTML tags and limiting to the specified number of
 * characters (plus ellipses).
 * @param text text to reduce
 * @param numChars the max number of characters to have
 * @return reduced text
 */
public String reduceText(String text, int numChars)
{
    StringBuffer newTextSB = new StringBuffer();

    StringTokenizer tokenizer = new StringTokenizer(text, "<>", true);
    boolean ignore = false;
    while (tokenizer.hasMoreTokens()) {
        String token = tokenizer.nextToken();
        if (token.charAt(0) == '<') {
            ignore = true;
        } else if (token.charAt(0) == '>') {
            ignore = false;
        } else {
            if (!ignore) {
                newTextSB.append(token);
            }
        }
    }

    if (newTextSB.length() > numChars) {
        return (newTextSB.toString().substring(0, numChars-1) + "...");
    } else {
        return (newTextSB.toString());
    }
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java OutlinedPropertyPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
    java.lang.Class newclass
        = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
    frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
        System.out.println(ex);
    frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    OutlinedPropertyPage outlinedPropertyPage = new OutlinedPropertyPage();
    outlinedPropertyPage.setBackground( java.awt.Color.lightGray );
    outlinedPropertyPage.init();
    frame.getContentPane().add( outlinedPropertyPage );
    frame.pack();
    java.awt.Dimension bigness = outlinedPropertyPage.getPreferredSize();
    frame.setSize(bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key138 ) + " " +
           JCustomizerBundle.string( JCustomizerBundle.key83 );
}

public static String getPageName() {
    return "OutlinedPropertyPage";
}

}

