/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PieChartPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:51 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;


import java.awt.Insets;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.AdjustmentListener;
import java.awt.event.AdjustmentEvent;

import javax.swing.JFrame;
import javax.swing.JScrollBar;
import javax.swing.JLabel;
import javax.swing.BorderFactory;

import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.JCPieChartFormat;
import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.chart.ChartDataView;
import com.klg.jclass.util.JCTypeConverter;

/**
 *  Pie Chart Specific Property Page
 */
public class PieChartPage extends JPropertyPage implements AdjustmentListener {

private JSpinBox thresholdMethod;
private JDoubleEditor thresholdValueField;
private JSpinBox sortOrder;
private JIntegerEditor minSlicesField;
private JDoubleEditor startAngleField;

private ChartDataView view = null;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 10, 10, 5, 10 );
	constraints.weightx = 1.0;

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key187 ) + ":" ) );
	thresholdMethod = new JSpinBox( JCChartEnumMappings.thresholdMethod_i18n_strings );
	thresholdMethod.getTextField().setEditable( false );
	thresholdMethod.getScrollBar().addAdjustmentListener( this );
	gridbag.setConstraints( thresholdMethod, constraints );
    add( thresholdMethod );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key188 ) + ":" ) );
    thresholdValueField = new JDoubleEditor( 4 );
    thresholdValueField.addPropertyChangeListener(this);
	gridbag.setConstraints( thresholdValueField, constraints );
    add( thresholdValueField );

    add(new JLabel( JCustomizerBundle.string( JCustomizerBundle.key189 ) + ":" ) );
	sortOrder = new JSpinBox( JCChartEnumMappings.sortOrder_i18n_strings );
	sortOrder.getTextField().setEditable( false );
	sortOrder.getScrollBar().addAdjustmentListener( this );
	gridbag.setConstraints( sortOrder, constraints );
	add( sortOrder );
    
    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key190 ) + ":" ) );
    minSlicesField = new JIntegerEditor();
    minSlicesField.addPropertyChangeListener( this );
	gridbag.setConstraints( minSlicesField, constraints );
    add( minSlicesField );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key345 ) + ":" ) );
    startAngleField = new JDoubleEditor( 4 );
    startAngleField.addPropertyChangeListener(this);
	gridbag.setConstraints( startAngleField, constraints );
    add( startAngleField );
}

/**
 * Return the object that this page is editing
 */
public Object getObject() {
    return view;
}

/**
 * Method called to populate the dialog with the source's properties
 */
public void setObject(Object source) {
    //System.out.println("DVGP setObject: "+source);
    if (source instanceof JCChart) {
	view = ((JCChart) source).getDataView(0);
    }
    else if (source instanceof ChartDataView) {
	view = (ChartDataView) source;
    }
    if (view != null && (view.getChartType() == JCChart.PIE)) {
	JCPieChartFormat pieFormat = (JCPieChartFormat) view.getChartFormat();
	if (pieFormat != null) {
	    int method = pieFormat.getThresholdMethod();
	    String name = JCTypeConverter.fromEnum(method, 
					JCChartEnumMappings.thresholdMethod_i18n_strings, 
					JCChartEnumMappings.thresholdMethod_values);
	    String current = thresholdMethod.getTextValue();
	    if (!name.equals(current)) {
			thresholdMethod.setTextValue(name);
	    }
		thresholdMethod.getScrollBar().setEnabled( true );
	    thresholdMethod.setIntValue( 0 );

	    thresholdValueField.setValue( new Double( pieFormat.getThresholdValue() ) );
		thresholdValueField.textField.setEnabled( true );

	    int order = pieFormat.getSortOrder();
	    name = JCTypeConverter.fromEnum(order, 
					JCChartEnumMappings.sortOrder_i18n_strings, 
					JCChartEnumMappings.sortOrder_values);
	    current = sortOrder.getTextValue();
	    if (!name.equals(current)) {
			sortOrder.setTextValue( name );
	   	}
		sortOrder.getScrollBar().setEnabled( true );
	    sortOrder.setIntValue( 0 );

	    minSlicesField.setValue(new Integer(pieFormat.getMinSlices()));
		minSlicesField.box.getScrollBar().setEnabled( true );
		minSlicesField.box.getTextField().setEnabled( true );

	    startAngleField.setValue( new Double( pieFormat.getStartAngle() ) );
		startAngleField.textField.setEnabled( true );
	}
    } 
	else {
		thresholdMethod.getScrollBar().setEnabled( false );
		thresholdValueField.textField.setEnabled( false );
		sortOrder.getScrollBar().setEnabled( false );
		minSlicesField.box.getScrollBar().setEnabled( false );
		minSlicesField.box.getTextField().setEnabled( false );
		startAngleField.textField.setEnabled( false );
    }
}

/**
 * This method is called when a specialized property editor's 
 * value is changed.
 */
public void propertyChanged(Object source, Object value) {
    if( view == null || view.getChartType() != JCChart.PIE) return;
	JCPieChartFormat fmt = (JCPieChartFormat) view.getChartFormat();
    if( source == thresholdValueField ) {
		Double threshold = (Double) value;
		try {
			fmt.setThresholdValue(threshold.doubleValue());
		} catch (Exception e) {
			setObject(view);
		}
    }
    else if( source == startAngleField ) {
		Double startAngle = (Double) value;
		try {
			fmt.setStartAngle(startAngle.doubleValue());
		} catch (Exception e) {
			setObject(view);
		}
    }
    else if (source == minSlicesField) {
		Integer minSlices = (Integer) value;
		fmt.setMinSlices(minSlices.intValue());
    }
}

public void adjustmentValueChanged( AdjustmentEvent e ) {
    if( view == null || view.getChartType() != JCChart.PIE) return;
	if( e != null && e.getSource() instanceof JScrollBar ) {
		JCPieChartFormat fmt = (JCPieChartFormat) view.getChartFormat();
		if( e.getSource() == thresholdMethod.getScrollBar() ) {
			thresholdMethod.setIntValue( e.getValue() );
			fmt.setThresholdMethod(JCTypeConverter.toEnum(
						thresholdMethod.getTextValue(),
						JCChartEnumMappings.thresholdMethod_i18n_strings,
						JCChartEnumMappings.thresholdMethod_values,
						JCPieChartFormat.SLICE_CUTOFF));
		}
		if( e.getSource() == sortOrder.getScrollBar() ) {
			sortOrder.setIntValue( e.getValue() );
			fmt.setSortOrder(JCTypeConverter.toEnum(
						sortOrder.getTextValue(),
						JCChartEnumMappings.sortOrder_i18n_strings,
						JCChartEnumMappings.sortOrder_values,
						JCPieChartFormat.DATA_ORDER));
		}
	}
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java PieChartPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    PieChartPage pieChartPage = new PieChartPage();
    pieChartPage.setBackground( java.awt.Color.lightGray );
    pieChartPage.init();
	pieChartPage.setObject( new com.klg.jclass.chart.JCChart( JCChart.PIE ) );
    frame.getContentPane().add( pieChartPage );
    frame.pack();
    java.awt.Dimension bigness = pieChartPage.getPreferredSize();
    frame.setSize( bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key191 );
}

public static String getPageName() {
    return "PieChartPage";
}

}

