/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PlotAreaBorderPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:51 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Insets;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.BorderFactory;

import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.JCChartArea;
import com.klg.jclass.chart.PlotArea;

/**
 *  Plot Area Style Property Page
 */
public class PlotAreaBorderPage extends JPropertyPage {

private JBooleanEditor axisBBox;

private JCChartArea chartarea = null;
private PlotArea region = null;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	//constraints.insets = new Insets( 10, 10, 5, 10 );
	constraints.weightx = 1.0;

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key166 ) + ":" ) );
    axisBBox = new JBooleanEditor("");
    axisBBox.addPropertyChangeListener( this );
	gridbag.setConstraints( axisBBox, constraints );
    add( axisBBox );
}

/**
 * Method called to populate the dialog with the source's properties
 */
public void setObject(Object source) {
    //System.out.println("PASP setObject: "+source);
    if (source instanceof JCChart) {
	chartarea = ((JCChart) source).getChartArea();
	region = chartarea.getPlotArea();
    }
    if (source instanceof JCChartArea) {
	chartarea = (JCChartArea) source;
	region = chartarea.getPlotArea();
    }
    else if (source instanceof PlotArea) {
	region = (PlotArea) source;
	chartarea = (JCChartArea) region.getParent();
    }
    if (chartarea != null) {
	axisBBox.setValue(new Boolean(chartarea.getAxisBoundingBox()));
    }
}

/**
 * Return the Object that this Property Page is editing
 */
public Object getObject() {
    if (region == null) 
	return (Object) chartarea;
    else
	return (Object) region;
}

/**
 * This method is called when a specialized property editor's 
 * value is changed.
 */
public void propertyChanged(Object source, Object value) {
    if (value == null) return;
    if (source == axisBBox) {
	if (chartarea != null) {
	    boolean toggle = ((Boolean)value).booleanValue();
	    chartarea.setAxisBoundingBox(toggle);
	}
    }
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java PlotAreaBorderPage
 */
public static void main(String args[]) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    PlotAreaBorderPage plotAreaBorderPage = new PlotAreaBorderPage();
    plotAreaBorderPage.setBackground( java.awt.Color.lightGray );
    plotAreaBorderPage.init();
    frame.getContentPane().add( plotAreaBorderPage );
    frame.pack();
    java.awt.Dimension bigness = plotAreaBorderPage.getPreferredSize();
    frame.setSize(bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key167 );
}

public static String getPageName() {
    return "PlotAreaBorderPage";
}

}

