/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PlotAreaMarginPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:51 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Insets;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.FlowLayout;

import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.BorderFactory;

import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.JCChartArea;
import com.klg.jclass.chart.PlotArea;

/**
 *  Plot Area Margin Page
 */

public class PlotAreaMarginPage extends JPropertyPage {

private JIntegerEditor marginTop;
private JBooleanEditor topIsDefault;
private JIntegerEditor marginLeft;
private JBooleanEditor leftIsDefault;
private JIntegerEditor marginBottom;
private JBooleanEditor bottomIsDefault;
private JIntegerEditor marginRight;
private JBooleanEditor rightIsDefault;

private PlotArea target;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 10, 10, 5, 10 );
	constraints.weightx = 1.0;

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key119 ) + ":" ) );
    marginTop = new JIntegerEditor();
	marginTop.setMinimum( -9999 );
    marginTop.addPropertyChangeListener( this );
    add( marginTop );
    topIsDefault = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key124 ) );
    topIsDefault.addPropertyChangeListener( this );
	gridbag.setConstraints( topIsDefault, constraints );
    add( topIsDefault );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key120 ) + ":" ) );
    marginLeft = new JIntegerEditor();
    marginLeft.addPropertyChangeListener(this);
	marginLeft.setMinimum( -9999 );
    add( marginLeft );
    leftIsDefault = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key124 ) );
    leftIsDefault.addPropertyChangeListener( this );
	gridbag.setConstraints( leftIsDefault, constraints );
    add( leftIsDefault );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key121 ) + ":" ) );
    marginBottom = new JIntegerEditor();
	marginBottom.setMinimum( -9999 );
    marginBottom.addPropertyChangeListener( this );
    add( marginBottom );
    bottomIsDefault = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key124 ) );
    bottomIsDefault.addPropertyChangeListener( this );
	gridbag.setConstraints( bottomIsDefault, constraints );
    add( bottomIsDefault );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key122 ) + ":" ) );
    marginRight = new JIntegerEditor();
	marginRight.setMinimum( -9999 );
    marginRight.addPropertyChangeListener( this );
    add( marginRight );
    rightIsDefault = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key124 ) );
    rightIsDefault.addPropertyChangeListener( this );
	gridbag.setConstraints( rightIsDefault, constraints );
    add( rightIsDefault );
}

/**
 * Method called to initialize the page
 */
public void setObject( Object source ) {
    Integer value;
    if (source instanceof JCChart) {
	target = ((JCChart) source).getChartArea().getPlotArea();
    }
    else if (source instanceof JCChartArea) {
	target = ((JCChartArea) source).getPlotArea();
    }
    else if (source instanceof PlotArea) {
	target = (PlotArea) source;
    }
    if (target != null) {
	value = new Integer(target.getTop());
	marginTop.setValue(value);
	topIsDefault.setValue(new Boolean(target.getTopIsDefault()));
	topIsDefault.setEnabled( !target.getTopIsDefault() );
	value = new Integer(target.getLeft());
	marginLeft.setValue(value);
	leftIsDefault.setValue(new Boolean(target.getLeftIsDefault()));
	leftIsDefault.setEnabled( !target.getLeftIsDefault() );
	value = new Integer(target.getBottom());
	marginBottom.setValue(value);
	bottomIsDefault.setValue(new Boolean(target.getBottomIsDefault()));
	bottomIsDefault.setEnabled( !target.getBottomIsDefault() );
	value = new Integer(target.getRight());
	marginRight.setValue(value);
	rightIsDefault.setValue(new Boolean(target.getRightIsDefault()));
	rightIsDefault.setEnabled( !target.getRightIsDefault() );
    }
}


public void propertyChanged(Object source, Object value) {
    if (target == null) return;
    if (value instanceof Integer) {
	if (source == marginTop) {
	    target.setTop(((Integer) marginTop.getValue()).intValue());
	}
	else if (source == marginLeft) {
	    target.setLeft(((Integer) marginLeft.getValue()).intValue());
	}
	else if (source == marginBottom) {
	    target.setBottom(((Integer)marginBottom.getValue()).intValue());
	}
	else if (source == marginRight) {
	    target.setRight(((Integer) marginRight.getValue()).intValue());
	}
    }
    else if (value instanceof Boolean) {
	boolean id = ((Boolean)value).booleanValue();
	if (source == leftIsDefault) {
	    target.setLeftIsDefault(id);
	} else if (source == topIsDefault) {
	    target.setTopIsDefault(id);
	} else if (source == bottomIsDefault) {
	    target.setBottomIsDefault(id);
	} else if (source == rightIsDefault) {
	    target.setRightIsDefault(id);
	}
    }
    this.setObject();
}
/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java PlotAreaMarginPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    PlotAreaMarginPage plotAreaMarginPage = new PlotAreaMarginPage();
    plotAreaMarginPage.setBackground( java.awt.Color.lightGray );
    plotAreaMarginPage.init();
    frame.getContentPane().add( plotAreaMarginPage );
    frame.pack();
    java.awt.Dimension bigness = plotAreaMarginPage.getPreferredSize();
    frame.setSize(bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key163 );
}

public static String getPageName() {
    return "PlotAreaMArginPage";
}

}

