/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PolarRadarChartPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:51 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Insets;
import java.awt.GridLayout;
import java.awt.FlowLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.BorderFactory;

import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCChartUtil;
import com.klg.jclass.chart.JCPolarRadarChartFormat;
import com.klg.jclass.chart.ChartDataView;

/**
 *  DataView General Polar/Radar Property Page
 */
public class PolarRadarChartPage extends JPropertyPage {

private JDoubleEditor originBaseField = null;
private JDoubleEditor rAxisAngleField = null;
private JBooleanEditor halfRangeCheckBox = null;
private JBooleanEditor webGridLinesCheckBox = null;
private ChartDataView view = null;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 5, 5, 5, 5 );
	constraints.weightx = 1.0;

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key58 )
    					+ ":" ) );
    JPanel originBasePanel = new JPanel();
    originBasePanel.setLayout( new FlowLayout( FlowLayout.LEFT, 0, 4 ) );
    originBaseField = new JDoubleEditor( 7 );
    originBaseField.addPropertyChangeListener( this );
    originBasePanel.add( originBaseField );
    gridbag.setConstraints( originBasePanel, constraints );
    add(originBasePanel);

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key59 )
    					+ ":" ) );
    JPanel rAxisAnglePanel = new JPanel();
    rAxisAnglePanel.setLayout( new FlowLayout( FlowLayout.LEFT, 0, 4 ) );
    rAxisAngleField = new JDoubleEditor( 7 );
    rAxisAngleField.addPropertyChangeListener( this );
    rAxisAnglePanel.add( rAxisAngleField );
    gridbag.setConstraints( rAxisAnglePanel, constraints );
    add(rAxisAnglePanel);

    JPanel halfRangePanel = new JPanel();
    halfRangePanel.setLayout( new FlowLayout( FlowLayout.LEFT, 0, 4 ) );
    halfRangeCheckBox =
    	new JBooleanEditor(JCustomizerBundle.string(JCustomizerBundle.key60));
    halfRangeCheckBox.addPropertyChangeListener( this );
    halfRangePanel.add( halfRangeCheckBox );
    gridbag.setConstraints( halfRangePanel, constraints );
    add(halfRangePanel);

    JPanel webGridLinesPanel = new JPanel();
    webGridLinesPanel.setLayout( new FlowLayout( FlowLayout.LEFT, 0, 4 ) );
    webGridLinesCheckBox =
    	new JBooleanEditor(JCustomizerBundle.string(JCustomizerBundle.key61));
    webGridLinesCheckBox.addPropertyChangeListener( this );
    webGridLinesPanel.add( webGridLinesCheckBox );
    gridbag.setConstraints( webGridLinesPanel, constraints );
    add(webGridLinesPanel);
}

/**
 * Method called to initialize the page
 */
public void setObject(Object source) {
	if (source instanceof JCChart) {
		view = ((JCChart)source).getDataView(0);
	}
	else if (source instanceof ChartDataView) {
		view = (ChartDataView)source;
	}
    if (isPolarRadar(view)) {
		JCPolarRadarChartFormat chartFormat =
							(JCPolarRadarChartFormat)view.getChartFormat();
		if (chartFormat != null) {
			Double value =
				new Double(chartFormat.getOriginBase(JCChartUtil.DEGREES));
			originBaseField.setValue(value);

			value = new Double(chartFormat.getYAxisAngle(JCChartUtil.DEGREES));
			rAxisAngleField.setValue(value);

			Boolean fHalfRange = new Boolean(chartFormat.isHalfRange());
			halfRangeCheckBox.setValue(fHalfRange);

			Boolean fWebbed = new Boolean(!chartFormat.isRadarCircularGrid());
			webGridLinesCheckBox.setValue(fWebbed);
		}
	}
}

public void propertyChanged( Object source, Object value ) {
    if( !isPolarRadar(view) || source == null ) return;

	JCPolarRadarChartFormat chartFormat =
							(JCPolarRadarChartFormat)view.getChartFormat();
	if (chartFormat == null) return;

    if (value instanceof Double) {
		if (source == originBaseField) {
	    	chartFormat.setOriginBase(JCChartUtil.DEGREES,
	    							  ((Double)value).doubleValue());
		}
		else if (source == rAxisAngleField) {
	    	chartFormat.setYAxisAngle(JCChartUtil.DEGREES,
	    							  ((Double)value).doubleValue());
		}
		this.setObject(view);
    }
    else if (value instanceof Boolean) {
		if (source == halfRangeCheckBox) {
			chartFormat.setHalfRange( ((Boolean)value).booleanValue() );
		}
		else if (source == webGridLinesCheckBox) {
			chartFormat.setRadarCircularGrid(!((Boolean)value).booleanValue());
		}
	}
}

private boolean isPolarRadar(ChartDataView cdv) {
	if (cdv == null) return false;
	int chartType = cdv.getChartType();
	return (chartType == JCChart.POLAR ||
			chartType == JCChart.RADAR ||
			chartType == JCChart.AREA_RADAR);
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java PolarRadarChartPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass
	    = java.lang.Class.forName(
				"com.klg.jclass.chart.customizer.JCustomizerCustomFrame");
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    PolarRadarChartPage polarRadarPage = new PolarRadarChartPage();
    polarRadarPage.setBackground( java.awt.Color.lightGray );
    polarRadarPage.init();
	polarRadarPage.setObject( new com.klg.jclass.chart.JCChart() );
    frame.getContentPane().add( polarRadarPage );
    frame.pack();
    java.awt.Dimension bigness = polarRadarPage.getPreferredSize();
    frame.setSize( bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key57 );
}

public static String getPageName() {
    return "PolarRadarChartPage";
}

}
