/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: SymbolStylePage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:51 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Color;
import java.awt.Insets;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.BorderFactory;

import com.klg.jclass.chart.JCChartStyle;
import com.klg.jclass.chart.JCSymbolStyle;
import com.klg.jclass.chart.JCChartEnumMappings;

/**
 *  Symbol Style Property Page
 *  exposes Point style Properties
 */
public class SymbolStylePage extends JPropertyPage {

private JColorEditor pointColor;
private JIntegerEditor pointWidth;
private JEnumEditor symbolShape;

private JCChartStyle chartStyle = null;


/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 10, 10, 5, 10 );
	constraints.weightx = 1.0;

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key115 ) + ":" ) );
    pointColor = new JColorEditor();
    pointColor.addPropertyChangeListener( this );
	gridbag.setConstraints( pointColor, constraints );
    add( pointColor );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key160 ) + ":" ) );
    pointWidth = new JIntegerEditor();
    pointWidth.addPropertyChangeListener( this );
	gridbag.setConstraints( pointWidth, constraints );
    add( pointWidth );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key161 ) + ":" ) );
    symbolShape = new JEnumEditor(JCChartEnumMappings.shape_i18n_strings, 
									JCChartEnumMappings.shape_values );
    symbolShape.addPropertyChangeListener( this );
	gridbag.setConstraints( symbolShape, constraints );
    add( symbolShape );
}

/**
 * Method called to populate the dialog with the source's properties
 */
public void setObject(Object source) {
    //System.out.println("PSP setObject: "+source);
    if (source instanceof JCChartStyle) {
	chartStyle = (JCChartStyle) source;

	pointColor.setValue(chartStyle.getSymbolColor());
	pointWidth.setValue(new Integer(chartStyle.getSymbolSize()));
	symbolShape.setValue(new Integer(chartStyle.getSymbolShape()));
    }
}

/**
 * Return the Object that this Property Page is editing
 */
public Object getObject() {
    return (Object) chartStyle;
}

/**
 * This method is called when a specialized property editor's 
 * value is changed.
 */
public void propertyChanged(Object source, Object value) {
    if (value == null) return;
    if (source == pointColor) {
	if (chartStyle != null) {
	    chartStyle.setSymbolColor((Color)value);
	}
    }
    else if (source == pointWidth) {
	if (chartStyle != null) {
	    chartStyle.setSymbolSize(((Integer)value).intValue());
	}
    }
    else if (source == symbolShape) {
	if (chartStyle != null) {
	    chartStyle.setSymbolShape(((Integer)value).intValue());
	}	
    }
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java SymbolStylePage
 */
public static void main(String args[]) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    SymbolStylePage symbolStylePage = new SymbolStylePage();
    symbolStylePage.setBackground( java.awt.Color.lightGray );
    symbolStylePage.init();
    frame.getContentPane().add( symbolStylePage );
    frame.pack();
    java.awt.Dimension bigness = symbolStylePage.getPreferredSize();
    frame.setSize(bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key159 );
}


public static String getPageName() {
    return "SymbolStylePage";
}

}
