/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TimeLabelPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:51 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.text.SimpleDateFormat;
import java.text.DateFormat;
import java.util.SimpleTimeZone;
import java.util.TimeZone;
import java.awt.Insets;
import java.awt.FlowLayout;
import java.awt.Color;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Date;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.BorderFactory;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;

import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCChartEnumMappings;

/**
 *  Time-labelled Axis Property Page
 */
public class TimeLabelPage extends JPropertyPage implements ActionListener {

private JFieldEditor baseField = null;
private JFieldEditor formatField = null;
private JEnumEditor unitCombo = null;
private JBooleanEditor formatIsDefault = null;
private JRadioButton chartFormat = null;
private String chartFormatString = JCustomizerBundle.string( JCustomizerBundle.key215 );
private String shortFormatString = JCustomizerBundle.string( JCustomizerBundle.key216 );
private String fullFormatString = JCustomizerBundle.string( JCustomizerBundle.key217 );

final private long SECONDS = 1000;
final private long MINUTES = 60*SECONDS;
final private long HOURS   = (60*MINUTES);
final private long DAYS    = (24*HOURS);
final private long WEEKS   = (7*DAYS);
final private long MONTHS  = (31*DAYS);
final private long YEARS   = (365*DAYS);

private JCAxis target;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 5, 5, 5, 5 );
	constraints.weightx = 1.0;

    JPanel baseTime = new JPanel();
	chartFormat = new JRadioButton( chartFormatString );
	JRadioButton shortFormat = new JRadioButton( shortFormatString );
	JRadioButton fullFormat = new JRadioButton( fullFormatString );
	ButtonGroup radioGroup = new ButtonGroup();

	baseTime.setLayout( new FlowLayout( FlowLayout.LEFT, 3, 3 ) );
	baseTime.setBorder( BorderFactory.createTitledBorder( JCustomizerBundle.string( JCustomizerBundle.key218 ) ) );
	chartFormat.addActionListener( this );
	baseTime.add( chartFormat );
	shortFormat.addActionListener( this );
	baseTime.add( shortFormat );
	fullFormat.addActionListener( this );
	baseTime.add( fullFormat );
	radioGroup.add( chartFormat );
	radioGroup.add( shortFormat );
	radioGroup.add( fullFormat );
	chartFormat.setEnabled( true );
	gridbag.setConstraints( baseTime, constraints );
	add( baseTime );
	
	add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key211 ) + ":" ) );
     // Time Base Field
    baseField = new JFieldEditor( "", 30 );
    baseField.addPropertyChangeListener( this );
	gridbag.setConstraints( baseField, constraints );
    add( baseField );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key212 ) + ":" ) );
	
	JPanel p = new JPanel();

	p.setLayout( new FlowLayout( FlowLayout.LEFT, 3, 3 ) );
     // Time Format Field
    formatField = new JFieldEditor( "", 20 );
    formatField.addPropertyChangeListener( this );
    p.add( formatField );
	formatIsDefault = new JBooleanEditor( JCustomizerBundle.string( JCustomizerBundle.key124 ) );
	formatIsDefault.addPropertyChangeListener( this );
	p.add( formatIsDefault );
	gridbag.setConstraints( p, constraints );
	add( p );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key213 ) + ":" ) );
     // Time unit combo
    // careful - in JCAxis these are longs, but they fit in ints...
    long units[] = JCChartEnumMappings.timeUnit_values;
    int useunits[] = new int[ units.length ];
    for( int i = 0; i < units.length; i++ ) {
		useunits[i] = (int) units[i];
    }
    unitCombo = new JEnumEditor( JCChartEnumMappings.timeUnit_i18n_strings, useunits );
    unitCombo.addPropertyChangeListener( this );
	gridbag.setConstraints( unitCombo, constraints );
    add( unitCombo );
}

/**
 * Method called to return the current target
 */
public Object getObject() {
    return target;
}

/**
 * Method called to initialize the page
 */
public void setObject( Object source ) {
    //System.out.println("AP setObject:"+source);
    String timeFormat;
    if( source instanceof JCAxis ) {
		target = (JCAxis) source;

		SimpleDateFormat dateFormat = (SimpleDateFormat)DateFormat.getDateTimeInstance();

		dateFormat.setTimeZone( TimeZone.getDefault() );
		dateFormat.applyPattern( target.getTimeFormat() );
		baseField.setValue( dateFormat.format( target.getTimeBase() ) );
		boolean isdef = target.getTimeFormatIsDefault();
		formatField.setValue( new String( target.getTimeFormat() ) );
		formatIsDefault.setValue( new Boolean( isdef ) );
		formatIsDefault.setEnabled( !isdef );

		unitCombo.setValue( new Integer( (int)target.getTimeUnit() ) );
		chartFormat.setSelected( true );
    }
}

/**
 * This method is called when a specialized property editor's 
 * value is changed.
 */
public void propertyChanged( Object source, Object value ) {
    if (target == null) return;
    if (source == unitCombo) {
		target.setTimeUnit( convertTimeUnitToLong( ((Integer)value).intValue() ) );
		setObject();
    }
	if( source instanceof JBooleanEditor ) {
		if( source == formatIsDefault ) {
			boolean val = ((Boolean)value).booleanValue();
			if (val != target.getTimeFormatIsDefault()) {
				target.setTimeFormatIsDefault( val );
				setObject();
			}
		}
	}
    if( source instanceof JFieldEditor ) {
		JFieldEditor field = (JFieldEditor)source;
		SimpleDateFormat dateFormat = (SimpleDateFormat)DateFormat.getDateTimeInstance();
		dateFormat.setTimeZone( TimeZone.getDefault() );

		if( field == baseField ) {
			try {
				dateFormat.applyPattern( target.getTimeFormat() );
				System.out.println( value );
				System.out.println( dateFormat.parse( (String)value ) );
				target.setTimeBase( dateFormat.parse( (String)value ) );
			}
			catch( Exception e ) { //ParseException or IllegalArgumentException
				target.setTimeBase( target.getTimeBase() );
			}
			setObject();
		}
		else 
			if( field == formatField ) {
				try {
					String pattern = (String)value;

					dateFormat.applyPattern( pattern );
					dateFormat.parse( dateFormat.format( new Date() ) );
					target.setTimeFormat( pattern );
				}
				catch( Exception e ) { //ParseException or IllegalArgumentException
					target.setTimeFormat( target.getTimeFormat() );
				}
				setObject();
			}
	}
}

public void actionPerformed( ActionEvent e ) {
	if( target == null ) return;
	if( e.getSource() instanceof JRadioButton ) {
		JRadioButton radio = (JRadioButton)e.getSource();
		String action = e.getActionCommand();
		String pattern = formatField.textField.getText();
		SimpleDateFormat dateFormat = null;

		if(  action.compareTo( chartFormatString ) == 0 ) {
			dateFormat = (SimpleDateFormat)DateFormat.getDateTimeInstance();
			
			dateFormat.applyPattern( target.getTimeFormat() );
		} else if( action.compareTo( shortFormatString ) == 0 ) {
			dateFormat = (SimpleDateFormat)DateFormat.getDateTimeInstance( DateFormat.SHORT, DateFormat.SHORT );
			
		} else if ( action.compareTo( fullFormatString ) == 0 ) {
			dateFormat = (SimpleDateFormat)DateFormat.getDateTimeInstance( DateFormat.FULL, DateFormat.FULL );
		}
		dateFormat.setTimeZone( TimeZone.getDefault() );
		baseField.textField.setText( (dateFormat.format( target.getTimeBase() ).toString() ) );
	}
}
//this is a hack due to the bug no. 3218;
//The problem appears because a time unit > WEEKS doesn't fit in a java.lang.Integer value
//it cannot be solved otherwise beacause a change int the JCEnumEditor to keep an array of longs instead
//an array of integers wiil trigger a long chain of changes;
private long convertTimeUnitToLong( int timeUnit ) {
//this are the long numbers that don't fit in an Integer
	if( timeUnit == 1471228928 )
		return YEARS;
	if( timeUnit == -1616567296 )
		return MONTHS;
	return timeUnit;
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java TimeLabelPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    TimeLabelPage timeLabelPage = new TimeLabelPage();
    timeLabelPage.setBackground( java.awt.Color.lightGray );
    timeLabelPage.init();
	timeLabelPage.setObject( new com.klg.jclass.chart.JCAxis() );
    frame.getContentPane().add( timeLabelPage );
    frame.pack();
    java.awt.Dimension bigness = timeLabelPage.getPreferredSize();
    frame.setSize( bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key214 );
}

public static String getPageName() {
    return "TimeLabelPage";
}

}

