/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TitleGeneralPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:51 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.event.ActionListener;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.BorderFactory;
import javax.swing.JTextArea;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JComponent;

import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.JCChartLabel;
import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.chart.Changeable;

/**
 *  Title General Property Page
 */
public class TitleGeneralPage extends JPropertyPage 
                                 implements ActionListener {

private JBooleanEditor visibleCheck = null;
private JTextArea titleTextArea = null;
private JEnumEditor halignCombo = null;
private JEnumEditor valignCombo = null;
private JFontEditor titleFont = null;

private Object title = null;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );

	setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );

	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 5, 10, 5, 10 );
	constraints.weightx = 1.0;

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key27 ) + ":" ) );
    visibleCheck = new JBooleanEditor("");
    visibleCheck.addPropertyChangeListener( this );
	gridbag.setConstraints( visibleCheck, constraints );
    add( visibleCheck );

    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key109 ) + ":" ) );
	titleTextArea = new JTextArea( 5, 30 );

	JScrollPane scrollPane = new JScrollPane();

    scrollPane.setBorder( BorderFactory.createLoweredBevelBorder() );
	scrollPane.setPreferredSize( new Dimension( titleTextArea.getPreferredScrollableViewportSize() ) );
    scrollPane.getViewport().add( titleTextArea );
	add( scrollPane );
    JButton textButton = new JButton( JCustomizerBundle.string( JCustomizerBundle.key110 ) + 
									  JCustomizerBundle.string( JCustomizerBundle.key109 ) );
    textButton.addActionListener( this );
	gridbag.setConstraints( textButton, constraints );
	add( textButton );

	add(new JLabel(""));
	JPanel comboPanel = new JPanel();
	comboPanel.setLayout(new FlowLayout());
    comboPanel.add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key111 ) + ":" ) );
    halignCombo = new JEnumEditor(JCChartEnumMappings.halign_i18n_strings, 
									JCChartEnumMappings.halign_values ); 
    halignCombo.addPropertyChangeListener( this );
	//gridbag.setConstraints( halignCombo, constraints );
    comboPanel.add( halignCombo );
    comboPanel.add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key333 ) + ":" ) );
    valignCombo = new JEnumEditor(JCChartEnumMappings.valign_i18n_strings, 
									JCChartEnumMappings.valign_values ); 
    valignCombo.addPropertyChangeListener( this );
    comboPanel.add( valignCombo );
	gridbag.setConstraints( comboPanel, constraints );
	add(comboPanel);
	
    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key112 ) + ":"  ) );
    titleFont = new JFontEditor();
    titleFont.addPropertyChangeListener(this);
	gridbag.setConstraints( titleFont, constraints );
    add( titleFont );
}

/**
 * Return the object being edited
 */
public Object getObject() {
    return title;
}

/**
 * Method called to populate the dialog with the source's properties
 */
public void setObject(Object source) {
    //System.out.println("TitleGP setObject: "+source);
    if( source instanceof JLabel ) {
		title = source;
    	if( title != null ) {
			JLabel label = (JLabel) source;
			String textAreaString = label.getText();
			if( textAreaString != null )
				titleTextArea.setText( new String( textAreaString ) );
			visibleCheck.setValue( new Boolean( label.isVisible() ) );
			halignCombo.setValue( new Integer( label.getHorizontalAlignment() ) );
			valignCombo.setValue( new Integer( label.getVerticalAlignment() ) );
			titleFont.setValue( label.getFont() );
    	}
	} else if (source instanceof JCChartLabel) {
		title = source;
    	if( title != null ) {
			JCChartLabel label = (JCChartLabel) source;
			JComponent comp = label.getComponent();
			if (comp instanceof JLabel) {
				String textAreaString = ((JLabel) comp).getText();
				if( textAreaString != null )
					titleTextArea.setText( new String( textAreaString ) );
				halignCombo.setValue( new Integer( 
					((JLabel)comp).getHorizontalAlignment() ) );
				valignCombo.setValue( new Integer( 
					((JLabel)comp).getVerticalAlignment() ) );
			}
			visibleCheck.setValue( new Boolean( label.getComponent().isVisible() ) );
			titleFont.setValue( label.getComponent().getFont() );
    	}
	}
}

/**
 * ActionListener method
 * Update the text of the title
 */
public void actionPerformed( ActionEvent ev ) {
    if (title == null) return;
	if (title instanceof JLabel) {
		((JLabel)title).setText( titleTextArea.getText() );
	} else if (title instanceof JCChartLabel) {
		((JCChartLabel)title).setText( titleTextArea.getText() );

		// Gotta go up 4 levels to get the parent outliner list to update it.
		java.awt.Component parent;
        if( (( parent = getParent() ) != null ) &&
            (( parent = parent.getParent() ) != null ) &&
            (( parent = parent.getParent() ) != null ) &&
            (( parent = parent.getParent() ) != null ) ) {
            if( parent instanceof JPropertyPage ) {
                JPropertyPage page = (JPropertyPage)parent;
                page.setObject();
                page.repaint();
            }
        }
	}
}

/**
 * This method is called when a specialized property editor's 
 * value is changed.
 */
public void propertyChanged( Object source, Object value ) {
    if (title == null || value == null) return;

	JLabel thisLabel = null;
	JComponent thisComponent = null;
	JCChartLabel thisChartLabel = null;
	if (title instanceof JLabel) {
		thisLabel = (JLabel) title;
		thisComponent = (JComponent) title;
	} else if (title instanceof JCChartLabel) {
		thisChartLabel = (JCChartLabel) title;
		thisComponent = ((JCChartLabel) title).getComponent();
		if (thisComponent instanceof JLabel) {
			thisLabel = (JLabel) thisComponent;
		}
	}

    if (source == halignCombo) {
		if (thisLabel != null)
			thisLabel.setHorizontalAlignment(((Integer)value).intValue());
    } else if (source == valignCombo) {
		if (thisLabel != null)
			thisLabel.setVerticalAlignment(((Integer)value).intValue());
    }
    else if (source == visibleCheck) {
		boolean toggle = ((Boolean)value).booleanValue();
		if (thisComponent != null)
			thisComponent.setVisible(toggle);
    }
    else if (source == titleFont) {
		if (thisComponent != null) {
			thisComponent.setFont( (java.awt.Font)value );
			if (thisChartLabel != null) {
				thisChartLabel.setChanged(true, Changeable.RECALC);
			}
		}
    }
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java TitleGeneralPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    TitleGeneralPage titleGeneralPage = new TitleGeneralPage();
    titleGeneralPage.setBackground( java.awt.Color.lightGray );
    titleGeneralPage.init();
    frame.getContentPane().add( titleGeneralPage );
    frame.pack();
    java.awt.Dimension bigness = titleGeneralPage.getPreferredSize();
    frame.setSize(bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key113 );
}

public static String getPageName() {
    return "TitleGeneralPage";
}

}

