/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ValueLabelPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:51 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.Vector;

import javax.swing.JTextField;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JButton;
import javax.swing.JTable;
import javax.swing.DefaultCellEditor;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.JCValueLabel;

/**
 *  Value-labelled Axis Property Page
 */
public class ValueLabelPage extends JPropertyPage implements ActionListener {

private JCustomTable valueLabelList = null;
private JButton applyButton = null;
private JButton addRow = null;
private JButton removeRow = null;
private static final Object[] newRow = { new String(""), new String("") };

private JCAxis target;

final static String[] valueLabel_columns = { 
    JCustomizerBundle.string( JCustomizerBundle.key97 ), 
	JCustomizerBundle.string( JCustomizerBundle.key267 ),
};

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
    setLayout( new BorderLayout() );
	setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
    valueLabelList = new JCustomTable();
	valueLabelList.setRowHeight( 20 );
	valueLabelList.setIntercellSpacing( new java.awt.Dimension( 3, 3 ) );
	valueLabelList.setToolTipText( JCustomizerBundle.string( JCustomizerBundle.key305 ) );
	valueLabelList.setModel( new DefaultTableModel( new Object[ 5 ][ 3 ], valueLabel_columns ) {
		public boolean isCellEditable( int rowIndex, int columnIndex ) {
			return true;
		}
	});
    add( valueLabelList.createTablePanel(), BorderLayout.CENTER );

	for( int i = 0; i < 5; i++ ) {
		valueLabelList.setValueAt( new String( "" ), i, 0 );
		valueLabelList.setValueAt( new String( "" ), i, 1 );
	}

    JPanel buttonsPanel = new JPanel();

    buttonsPanel.setLayout( new FlowLayout( FlowLayout.CENTER, 3, 3 ) );
    buttonsPanel.add( applyButton = new JButton( JCustomizerBundle.string( JCustomizerBundle.key247 ) ) );
	buttonsPanel.add( addRow = new JButton( JCustomizerBundle.string( JCustomizerBundle.key306 ) ) );
	buttonsPanel.add( removeRow = new JButton( JCustomizerBundle.string( JCustomizerBundle.key307 ) ) );
	applyButton.addActionListener( this );
	addRow.addActionListener( this );
	removeRow.addActionListener( this );
    add( buttonsPanel, BorderLayout.SOUTH );
}

/**
 * Method called to initialize the page
 */
public void setObject(Object source) {
    //System.out.println("AP setObject:"+source);
    Vector vlabels = null;
	//reseting the table
	for( int i = 0; i < valueLabelList.getRowCount(); i++ ) {
		valueLabelList.setValueAt( "", i, 0 );
		valueLabelList.setValueAt( "", i, 1 );
	}
    if( source instanceof JCAxis ) {
		target = (JCAxis) source;
		JCValueLabel labels[] = target.getValueLabels();

		if( labels != null ) {
			for( int i = 0; i < labels.length; i++ ) {
				if( valueLabelList.getRowCount() <= i )
					valueLabelList.addRow( newRow );
				valueLabelList.setValueAt( Double.toString( labels[ i ].getValue() ), i, 0 );
				valueLabelList.setValueAt( labels[ i ].getText(), i, 1 );
				//valueLabelList.setValueAt( new Boolean( true ), i, 2 );
			}
		}
    }
    valueLabelList.setItems( vlabels );
    if( target.getAnnotationMethod() == JCAxis.VALUE_LABELS ) {
		valueLabelList.setEnabled( true );
		applyButton.setEnabled( true );
		addRow.setEnabled( true );
		removeRow.setEnabled( true );
	} 
	else {
		valueLabelList.setEnabled( false );
		applyButton.setEnabled( false );
		addRow.setEnabled( false );
		removeRow.setEnabled( false );
    }
}

// ActionListener method
public void actionPerformed( ActionEvent ev ) {
	if( target == null ) return;
	if( ev.getSource() instanceof JButton) {
		JButton button = (JButton) ev.getSource();

		if( button == addRow ) {
			valueLabelList.addRow( newRow );
		}
		else if( button == removeRow ) {
			int rowSelected = valueLabelList.getSelectedRow();

			if( rowSelected >= 0 ) {
				try {
					double remove = new Double( (String)valueLabelList.getValueAt( rowSelected, 0 ) ).doubleValue();
					JCValueLabel labels[] = target.getValueLabels();

					if( labels != null ) {
						JCValueLabel vlabels[] = new JCValueLabel[ labels.length - 1 ];
						int j = 0;

						for( int i = 0; i < labels.length; i++ )
							if( labels[i].getValue() != remove ) {
								vlabels[ j ] = labels[ i ];
								j++;
							}
					target.setValueLabels( vlabels );
					}
				}
				catch( Exception e ) {
					//System.out.println("Value Label Remove: "+e.getMessage());
				}
				valueLabelList.removeRow( rowSelected );
			}
		} else if( button == applyButton ) {
			int rowsNo = valueLabelList.getRowCount();
			Vector newLabels = new Vector();

			for( int i = 0; i < rowsNo; i++ )
				try { 
					double value = new Double( (String)valueLabelList.getValueAt( i, 0 ) ).doubleValue();
					String text = (String)valueLabelList.getValueAt( i, 1 );
					JCValueLabel label = new JCValueLabel( value, text );

					newLabels.addElement( label );
				}
				catch( Exception e ) {
					//System.out.println("Value Label Apply: "+e.getMessage());
				}

			int size = newLabels.size();
			JCValueLabel labels[] = new JCValueLabel[ size ];

			for( int i = 0; i < size; i++ )
				labels[ i ] = (JCValueLabel)newLabels.elementAt( i );
			target.setValueLabels( labels );
		}
		this.setObject( target );
	}
}

/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java ValueLabelPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    ValueLabelPage valueLabelPage = new ValueLabelPage();
    valueLabelPage.setBackground( java.awt.Color.lightGray );
    valueLabelPage.init();
	valueLabelPage.setObject( new com.klg.jclass.chart.JCAxis());
    frame.getContentPane().add( valueLabelPage );
    frame.pack();
    java.awt.Dimension bigness = valueLabelPage.getPreferredSize();
    frame.setSize(bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key268 );
}


public static String getPageName() {
    return "ValueLabelPage";
}

}

