/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ViewAxisPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:51 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.customizer;

import java.awt.Insets;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.GridLayout;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.BorderFactory;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;




import java.util.Iterator;
import java.util.List;


import com.klg.jclass.chart.ChartDataView;
import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCChart;

/**
 *  Axis Property Page
 */
public class ViewAxisPage extends JPropertyPage {

protected JEnumEditor viewCombo = null;
private JLabel viewLabel = null;
protected JBooleanEditor xCheck = null;
protected JBooleanEditor yCheck = null;
private JPanel buttonsPanel = null;

private JCAxis target;

/**
 * Method called to initialize the page
 */
public void init() {
	super.init();
	setToolTipText( getPageTitle() );
	GridBagConstraints constraints = new GridBagConstraints();
	GridBagLayout gridbag = new GridBagLayout();

	setLayout( gridbag );
	setBorder( BorderFactory.createEmptyBorder( 2, 2, 2, 2 ) );
	constraints.anchor = GridBagConstraints.WEST;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.insets = new Insets( 5, 5, 5, 5 );
	constraints.weightx = 1.0;

    viewLabel = new JLabel( "" );
    viewLabel.setIcon( new ImageIcon( AxisOrientPage.getEnabledOrientationImage( (java.awt.Component)this, 0 ) ) );
    add( viewLabel );

    /*
     * Axis selector
     */
	ButtonGroup xyGroup = new ButtonGroup();

    buttonsPanel = new JPanel();
	buttonsPanel.setLayout( new GridLayout( 2, 1 ) );
	buttonsPanel.setBorder( BorderFactory.createTitledBorder("") );
    xCheck = new JBooleanEditor( "X " + JCustomizerBundle.string( JCustomizerBundle.key83 ) );
    yCheck = new JBooleanEditor( "Y " + JCustomizerBundle.string( JCustomizerBundle.key83 ) );
	xCheck.addPropertyChangeListener( this );
	yCheck.addPropertyChangeListener( this );
	xyGroup.add( xCheck );
	xyGroup.add( yCheck );
	buttonsPanel.add( xCheck );
	buttonsPanel.add( yCheck );
	gridbag.setConstraints( buttonsPanel, constraints );
	add( buttonsPanel );
    add( new JLabel( JCustomizerBundle.string( JCustomizerBundle.key69 ) + ":" ) );
    viewCombo = new JEnumEditor();
    viewCombo.addPropertyChangeListener( this );
	gridbag.setConstraints( viewCombo, constraints );
    add( viewCombo );
}

private void setViewList( JCChart chart ) {
    List views = chart.getDataView();
	boolean add = true;

	if( views != null ) {
		String list[] = new String[ views.size() + 1 ];
		ChartDataView viewArray[] = new ChartDataView[ views.size() + 1];
		list[0] = "None";
		viewArray[0] = null;

		Iterator iter = views.iterator();
		for(int i = 1; iter.hasNext(); i++) {
			ChartDataView view = (ChartDataView) iter.next();
			list[ i ] = new String( "" + i + ": " + view.getName() );
			viewArray[i] = view;
		}
		viewCombo.setItems( list, viewArray );
	}
}


/**
 * Method called to return the current target
 */
public Object getObject() {
    return target;
}

/**
 * Method called to initialize the page
 */
public void setObject(Object source) {
    //System.out.println("VAxisP setObject:"+source);
    if( source instanceof JCChart ) {
		setViewList((JCChart) source);
		viewCombo.setEnabled( true );
		buttonsPanel.setEnabled( true );
    }
    else if( source instanceof JCAxis ) {
		target = (JCAxis) source;
		JCChart parent = target.getParentChart();
		if (parent == null) return;
		List views = parent.getDataView();
		if (views == null) return;

		JCAxis axis;
		JCAxis orig = target.getFormula().getOriginator();
		if (orig != null) {
			axis = orig;
		} else {
			axis = target;
		}
		setViewList( parent );

		String list[] = new String[ views.size() ];
		int selected = -1;
		Iterator iter = views.iterator();
		for(int i = 0; iter.hasNext(); i++) {
			ChartDataView view = (ChartDataView) iter.next();
			list[i] = new String( "" + i + ": " + view.getName() );
			if( (axis == view.getXAxis() )
					|| (axis == view.getYAxis() ) ) {
				selected = i;
			}
		}
		if( selected >= 0 ) {
			ChartDataView selView = (ChartDataView) views.get(selected);
	    	viewCombo.setValue( selView );
	    	viewLabel.setIcon( new ImageIcon( AxisOrientPage.getEnabledOrientationImage( (java.awt.Component)this, selView ) ) );
	    	if( axis == selView.getXAxis() ) {
				xCheck.setValue( new Boolean( true ) );
				yCheck.setValue( new Boolean( false ) );
	    	} else {
				yCheck.setValue( new Boolean( true ) );
				xCheck.setValue( new Boolean( false ) );
	    	}
	    	//viewCombo.setEnabled( false );
	    	buttonsPanel.setEnabled( false );
		} else {
	    	viewCombo.setValue( null );
	    	buttonsPanel.setEnabled( true );
		}
    }
}

/**
 *  Property Change listener
 *  Called when a property editor is updated
 */
public void propertyChanged( Object source, Object value ) {
    if( target == null ) return;
    if( value instanceof ChartDataView ) {
		ChartDataView view = (ChartDataView) value;

		viewLabel.setIcon( new ImageIcon( AxisOrientPage.getEnabledOrientationImage((java.awt.Component)this, view) ) );
		if (target == view.getXAxis()) {
			//xCheck.setValue( new Boolean( true ) );
			//yCheck.setValue( new Boolean( false ) );
		} else if (target == view.getYAxis()) {
			//yCheck.setValue( new Boolean( true ) );
			//xCheck.setValue( new Boolean( false ) );
		}
    } 
	else if( source instanceof JBooleanEditor ) {
		JBooleanEditor editor = (JBooleanEditor)source;

		if( editor == null ) return;
		boolean val = ((Boolean)value).booleanValue();

		if( val ) this.setObject( target );
	} 
	else if( source instanceof JEnumEditor ) {
		JEnumEditor editor = (JEnumEditor)source;
		ChartDataView view = (ChartDataView)editor.getItem( ((Integer)value).intValue() );
		JCChart chart = target.getParentChart();
		if (chart != null && view != null) {

			ChartDataView oldView = null;
			boolean xOrY = false;
			List list = chart.getDataView();
			for (Iterator i = list.iterator(); i.hasNext(); ) {
				ChartDataView cdv = (ChartDataView) i.next();
				if (cdv.getXAxis() == target) {
					oldView = cdv;
					xOrY = true;
					break;
				} else if (cdv.getYAxis() == target) {
					oldView = cdv;
					xOrY = false;
				}
			}
			if (oldView != view) {
				if (xOrY) {
					view.setXAxis(target);
				} else {
					view.setYAxis(target);
				}
				viewLabel.setIcon( new ImageIcon( AxisOrientPage.getEnabledOrientationImage((java.awt.Component)this, view) ) );
			}
			setObject();
		}
	}
}


/**
 * Static main method that allows this class to be run independently
 * in the interpreter with the command:
 * java ViewAxisPage
 */
public static void main( String args[] ) {
    JFrame frame;
    try {
	java.lang.Class newclass 
	    = java.lang.Class.forName( "com.klg.jclass.chart.customizer.JCustomizerCustomFrame" );
	frame = (JFrame) newclass.newInstance();
    } catch (Exception ex) {
		System.out.println(ex);
	frame = new JFrame();
    }
    frame.setTitle( getPageTitle() );

    ViewAxisPage viewAxisPage = new ViewAxisPage();
    viewAxisPage.setBackground( java.awt.Color.lightGray );
    viewAxisPage.init();
	viewAxisPage.setObject( new com.klg.jclass.chart.JCChart() );
    frame.getContentPane().add( viewAxisPage );
    frame.pack();
    java.awt.Dimension bigness = viewAxisPage.getPreferredSize();
    frame.setSize( bigness.width + 200, bigness.height + 100 );
    frame.show();
}

public static String getPageTitle() {
    return JCustomizerBundle.string( JCustomizerBundle.key233 );
}

public static String getPageName() {
    return "ViewAxisPage";
}

}
