/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCAppletDataSource.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:51 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.data;

import java.applet.Applet;
import java.io.IOException;
import java.io.StringReader;

/**
 * Data source for reading data from the <code>DATA</code> param tag
 * in an applet.<p>
 * For details on the acceptable data formats, please refer to
 * <code>JCDefaultDataInterpreter</code> and
 * <code>JCXMLDataInterpreter</code>.<p>
 * <b>Note:</b> In Netscape, newlines are stripped. Make sure that if you
 *  spread the data across multiple line, that you also put a space at the
 *  beginning of each line.
 * @see JCDefaultDataInterpreter
 * @see JCXMLDataInterpreter
 */
public class JCAppletDataSource extends JCInputStreamDataSource {

/**
 * Pulls the data from the <code>DATA</code> param tag associated with
 * the specified applet, and interprets it via the default interpreter class.
 * @param applet Applet class to retrieve data from
 * @exception IOException An IOException will be thrown if
 * there's a problem reading the data or if there is no <code>DATA</code>
 * parameter.
 */
public JCAppletDataSource(Applet applet) throws IOException
{
    this(applet, (JCDataInterpreter)null);
}

/**
 * Pulls the data from the <code>DATA</code> param tag associated with
 * the specified applet, and interprets it via the specified interpreter class.
 * @param applet Applet class to retrieve data from
 * @param ipr interpreter that will interpret data stream
 * @exception IOException An IOException will be thrown if
 * there's a problem reading the data or if there is no <code>DATA</code>
 * parameter.
 */
public JCAppletDataSource(Applet applet, JCDataInterpreter ipr)
    throws IOException
{
    String s = applet.getParameter("DATA");

    if (s == null) {
        throw new IOException("No DATA tag");
    }

    System.out.println("1:Parsing data from:" + s);

    StringReader sr = new StringReader(s);
    interpreter = ipr;
    ctor(sr);
}

/**
 * Pulls the data from the <code>&#60param&#62</code> tag associated with
 * the specified applet and interprets it via the default interpreter class.
 * @param applet Applet class to retrieve data from
 * @param param parameter name in Applet class that holds data
 * @exception IOException An <code>IOException</code> will be thrown if
 * there's a problem reading the data, or if there is no
 * <code>&#60param&#62</code> parameter.
 */
public JCAppletDataSource(Applet applet, String param) throws IOException
{
    this(applet, param, null);
}

/**
 * Pulls the data from the <code>&#60param&#62</code> param tag associated with
 * the specified applet and interprets it via the specified interpreter class.
 * @param applet Applet class to retrieve data from
 * @param param Parameter name in Applet class that holds data
 * @param ipr Interpreter that will interpret data stream
 * @exception IOException An <code>IOException</code> will be thrown if
 * there's a problem reading the data, or if there is no
 * <code>&#60param&#62</code> parameter.
 */
public JCAppletDataSource(Applet applet, String param, JCDataInterpreter ipr)
    throws IOException
{
    String s = applet.getParameter(param);

    if (s == null) {
        // i18n
        throw new IOException("No " +
                              param +
                              " tag");
    }

    System.out.println("2:Parsing data from: " + s);

    StringReader sr = new StringReader(s);
    interpreter = ipr;
    ctor(sr);
}

}
