/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCDataInterpreter.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:51 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.data;

import java.io.IOException;
import java.io.Reader;

/**
 * The base interpreter class for any data source that relies on input
 * from some reader.  Subclasses that are implemented will interpret for specific
 * data formats.
 */
public abstract class JCDataInterpreter {

/* Data source to populate. */
protected BaseDataSource baseDS = null;

final static int PARSE_ERR_DATA_STYLE = 1;
final static int PARSE_ERR_HOLE_OR_VALUE = 2;
final static int PARSE_ERR_POINT_LABEL = 3;

final static String ERRMSG_FILE_FORMAT = "Error in data file format - ";
final static String ERRMSG_READ_POINT_LABELS = "Error in reading point labels - ";
final static String ERRMSG_MUST_START = "Data must start with ARRAY or GENERAL";
final static String ERRMSG_DATA_LINE = "Error reading data: line ";
final static String ERRMSG_EXPECT_HOLE = "Expect either a hole or numeric value and not ";
final static String ERRMSG_EXPECT_POINT_LABEL = "Expect a point label and not ";

/**
 * This constructor creates an instance of a data interpreter and associates it
 * with a data source. It does not yet do any interpreting.
 * @param bds the data source to eventually be populated with the interpreted
 *     data
 */
public JCDataInterpreter(BaseDataSource bds)
{
	baseDS = bds;
}

/**
 * Default no-argument constructor.
 */
public JCDataInterpreter() {}

/**
 * This interpreter should be associated with a data source that will eventually
 * be populated with interpreted data.
 * @param bds the data source to eventually be populated with the interpreted
 * data
 */
public void setBaseDataSource(BaseDataSource bds)
{
	baseDS = bds;
}

/**
 * Abstract method to be overridden by specific interpreter implementations.<p>
 * Parses the specified input stream and populates the previously set
 * data source. Called by the other <code>ctor()</code> method, and is also used
 * for String input where you can't have an input stream.
 * @param r the Reader object used to read data
 * @throws IOException
 */
protected abstract void ctor(Reader r) throws IOException;

}
