/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCDefaultDataSource.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:51 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.data;

import java.util.Locale;

import com.klg.jclass.chart.JCChartBundle;
import com.klg.jclass.chart.ChartDataEvent;
import com.klg.jclass.util.ImageMapInfo;

/**
 * <code>JCDefaultDataSource</code> extends <code>BaseDataSource</code> to create
 * a more useful default container for JClass Chart data.
 * <p>
 * (Formerly <code>VectorDataSource</code>).
 */
public class JCDefaultDataSource extends BaseDataSource {

/**
 * Used to suppress generation of x series data in all cases.
 */
protected boolean suppressMultipleGeneration = false;

/**
 * Default data used in Beans.
 */
protected static double[][] defaultY = {
    {20.0, 22.0, 19.0, 24.0, 25.0, },
    {8.0, 12.0, 10.0, 12.0, 15.0, },
    {10.0, 16.0, 17.0, 15.0, 23.0, },
    {16.0, 19.0, 15.0, 22.0, 18.0, },
};

/**
 * The default constructor. Used mostly for the <code>DataSourceEditor</code>.
 */
public JCDefaultDataSource()
{
	this(Locale.getDefault());
}

/**
 * The default constructor with specified locale.  Locale is used
 * to generate default labels.
 */
public JCDefaultDataSource(Locale locale)
{
    this(null, defaultY, null, null, null);
    generateLabels(locale);
}

/**
 * Creates a data source using the provided data.
 *
 * @param xvalues X series arrays; can be null
 * @param yvalues Y series arrays; can be null
 * @param pointLabels array of point labels; can be null
 * @param seriesLabels array of series labels; can be null
 * @param dataSourceName name to be applied to this data source
 */
public JCDefaultDataSource(double[][] xvalues, double[][] yvalues,
                           String[] pointLabels, String[] seriesLabels,
                           String dataSourceName)
{
    this(xvalues, yvalues, pointLabels, seriesLabels, dataSourceName,
         null, null, null, null);
}

/**
 * Creates a data source using the provided data and image map information.
 * Note that the image map information will only be used for JCServerChart
 * (i.e. it will be ignored for JCChart).
 *
 * @param xvalues X series arrays; can be null.
 * @param yvalues Y series arrays; can be null.
 * @param pointLabels array of point labels; can be null.
 * @param seriesLabels array of series labels; can be null.
 * @param dataSourceName name to be applied to this data source.
 * @param pointURLs An array of URLs, indexed by series and point.
 * @param pointExtras An array of extra URL information, indexed by series and
 * point.
 * @param clusterURLs An array of URLs, one for each point cluster.
 * @param clusterExtras An array of extra URL information, one for each point
 * cluster.
 * @param seriesURLs An array of URLs, one for each series.
 * @param seriesExtras An array of extra URL information, one for each series.
 * @param legendURLs An array of legend URLs, one for each series.
 * @param legendExtras An array of extra legend URL information, one for each
 * series.
 */
public JCDefaultDataSource(double[][] xvalues, double[][] yvalues,
                           String[] pointLabels, String[] seriesLabels,
                           String dataSourceName,
                           String[][] pointURLs, String[][] pointExtras,
                           String[] clusterURLs, String[] clusterExtras,
                           String[] seriesURLs, String[] seriesExtras,
                           String[] legendURLs, String[] legendExtras)
{
    this(xvalues, yvalues, pointLabels, seriesLabels, dataSourceName,
         ImageMapInfo.createImageMapInfoArray(pointURLs, pointExtras),
         ImageMapInfo.createImageMapInfoArray(clusterURLs, clusterExtras),
         ImageMapInfo.createImageMapInfoArray(seriesURLs, seriesExtras),
         ImageMapInfo.createImageMapInfoArray(legendURLs, legendExtras));
}

/**
 * Creates a data source using the provided data.
 *
 * @param xvalues X series arrays; can be null
 * @param yvalues Y series arrays; can be null
 * @param pointLabels array of point labels; can be null
 * @param seriesLabels array of series labels; can be null
 * @param dataSourceName name to be applied to this data source
 * @param pointImageMapInfo An array of image map information, indexed by
 * series and point.
 * @param clusterImageMapInfo An array of image map information, one for each
 * point cluster.
 * @param seriesImageMapInfo An array of image map information, one for each
 * series.
 * @param legendImageMapInfo An array of legend image map information, one for
 * each series.
 */
public JCDefaultDataSource(double[][] xvalues, double[][] yvalues,
                           String[] pointLabels, String[] seriesLabels,
                           String dataSourceName,
                           ImageMapInfo[][] pointImageMapInfo,
                           ImageMapInfo[] clusterImageMapInfo,
                           ImageMapInfo[] seriesImageMapInfo,
                           ImageMapInfo[] legendImageMapInfo)
{
    this.xvalues = xvalues;
    this.yvalues = yvalues;
    this.pointLabels = pointLabels;
    this.seriesLabels = seriesLabels;
    this.name = dataSourceName;
    this.pointImageMapInfo = pointImageMapInfo;
    this.clusterImageMapInfo = clusterImageMapInfo;
    this.seriesImageMapInfo = seriesImageMapInfo;
    this.legendImageMapInfo = legendImageMapInfo;

    check();
}

/**
 * Creates a JCDefaultDataSource from a BaseDataSource.
 * @param baseDataSource The BaseDataSource from which this object is derived.
 */
public JCDefaultDataSource(BaseDataSource baseDataSource)
{
	if (baseDataSource != null) {
		xvalues = baseDataSource.xvalues;
		yvalues = baseDataSource.yvalues;
		pointLabels = baseDataSource.pointLabels;
		seriesLabels = baseDataSource.seriesLabels;
		name = baseDataSource.name;
		holeValue = baseDataSource.holeValue;
        pointImageMapInfo = baseDataSource.pointImageMapInfo;
        clusterImageMapInfo = baseDataSource.clusterImageMapInfo;
        seriesImageMapInfo = baseDataSource.seriesImageMapInfo;
        legendImageMapInfo = baseDataSource.legendImageMapInfo;
		cloneArrays = baseDataSource.cloneArrays;
		check();
	}
}

/**
 * Checks the supplied data to see if x data has been provided.  If it hasn't,
 * it is generated.
 */
protected void check()
{
    if (yvalues == null || yvalues.length <= 0) {
        return;
    }

    // Check for x data.
    if (xvalues == null) {
        // suppressMultipleGeneration is used to
        // force generation of a single series of
        // x data in all cases.
        int baseLength = yvalues[0].length;
        if (suppressMultipleGeneration) {
           xvalues = ChartDataSourceUtil.generateSingleXSeries(baseLength);
           return;
        }

        // Need to determine whether to generate a single
        // x series or an x series for each y series.  If
        // any of the y series lengths is unique, we will
        // generate an x series.
        boolean differentSeriesLength = false;
        for (int i = 1; i < yvalues.length; i++) {
            if (yvalues[i].length != baseLength) {
                differentSeriesLength = true;
                break;
            }
        }
        if (differentSeriesLength) {
            xvalues = ChartDataSourceUtil.generateMultipleXSeries(yvalues);
        }
        else {
            xvalues = ChartDataSourceUtil.generateSingleXSeries(baseLength);
        }
    }
}

/**
 * Generates and sets the data source name, series labels, and point labels
 * from the existing y values.
 */
protected void generateLabels()
{
	generateLabels(Locale.getDefault());
}

/**
 * Generates and sets the data source name, series labels, and point labels
 * from the existing y values for the specified locale.
 */
protected void generateLabels(Locale locale)
{
    name = new String(JCChartBundle.string(JCChartBundle.key165, locale));

    if (yvalues == null) {
        return;
    }

    seriesLabels = new String[getNumSeries()];
    for (int i = 0; i < seriesLabels.length; i++) {
        seriesLabels[i] = JCChartBundle.string(JCChartBundle.key127, locale)
																	 + (i + 1);
    }

    pointLabels = new String[getXSeries(0).length];
    for (int i = 0; i < pointLabels.length; i++) {
        pointLabels[i] = JCChartBundle.string(JCChartBundle.key166, locale) + i;
    }
}

/**
 * Internal method to inform the listeners when the chart data has changed.
 * @param type data change type to pass to the views
 * @param seriesIndex index of series affected by the change
 * @param pointIndex index of point affected by the change
 */
protected void updateChart(int type, int seriesIndex, int pointIndex)
{
    try {
        ChartDataEvent de = new ChartDataEvent(this, type, seriesIndex,
                                               pointIndex);
        fireChartDataEvent(de);
    }
    catch (Exception e) {
        e.printStackTrace(System.out);
    }
}

/**
 * Sets the x and y data contained by this data source.
 * @param xvalues X series arrays; can be null
 * @param yvalues Y series arrays; can be null
 */
public void setData(double xvalues[][], double yvalues[][])
{
    this.xvalues = xvalues;
    this.yvalues = yvalues;
    check();
    updateChart(ChartDataEvent.RESET, 0, 0);
}

/**
 * Sets the specified x-value series.  If the current
 * array of x series data is not large enough to accommodate
 * the index, it will automatically be expanded, with
 * missing data series values set to null.  <b>Note:</b>
 * <code>NumSeries</code> will not be adjusted automatically.
 * @param index data series index
 * @param xvalues array of x values
 */
public void setXSeries(int index, double xvalues[])
{
    if (index >= this.xvalues.length) {
        double newValues[][] = new double[index + 1][];
        System.arraycopy(this.xvalues, 0, newValues, 0, this.xvalues.length);
        for (int i = this.xvalues.length; i <= index; i++) {
            newValues[i] = null;
        }
        this.xvalues = newValues;
        this.xvalues[index] = xvalues;
        updateChart(ChartDataEvent.RESET, 0, 0);
    }
    else {
        this.xvalues[index] = xvalues;
        updateChart(ChartDataEvent.RELOAD_SERIES, index, 0);
    }
}

/**
 * Sets the specified y-value series.  If the current
 * array of y series data is not large enough to accommodate
 * the index, it will automatically be expanded, with
 * missing data series values set to null. <br><b>Note:</b>
 * <code>NumSeries</code> will not be adjusted automatically.
 * @param index data series index
 * @param yvalues array of y values
 */
public void setYSeries(int index, double yvalues[])
{
    if (index >= this.yvalues.length) {
        double newValues[][] = new double[index + 1][];
        System.arraycopy(this.yvalues, 0, newValues, 0, this.yvalues.length);
        for (int i = this.yvalues.length; i <= index; i++) {
            newValues[i] = null;
        }
        this.yvalues = newValues;
        this.yvalues[index] = yvalues;
        updateChart(ChartDataEvent.RESET, 0, 0);
    }
    else {
        this.yvalues[index] = yvalues;
        updateChart(ChartDataEvent.RELOAD_SERIES, index, 0);
    }
}

/**
 * Sets the value of the <code>pointLabels</code> property. The
 * <code>pointLabels</code> property is an indexed property that contains a list
 * of point labels provided by the data source.
 * @param ps the array of point label Strings
 */
public void setPointLabels(String[] ps)
{
    pointLabels = ps;
    updateChart(ChartDataEvent.RELOAD_ALL_POINT_LABELS, 0, 0);
}

/**
 * Sets the <code>seriesLabel</code> properties. The <code>seriesLabel</code>
 * property defines the label that is associated with a data
 * series.
 * @param ss the array of series label Strings
 */
public void setSeriesLabels(String[] ss)
{
    seriesLabels = ss;
    updateChart(ChartDataEvent.RELOAD_ALL_SERIES_LABELS, 0, 0);
}

/**
 * Sets the value of the <code>Name</code> property. The <code>Name</code>
 * property defines the name of the data object.
 * @param s the name String
 */
public void setName(String s)
{
    name = s;
}

/**
 * Sets an array of image map information indexed by series and point.
 * @param pointImageMapInfo The new point image map information array.
 */
public void setPointImageMapInfo(ImageMapInfo[][] pointImageMapInfo)
{
    this.pointImageMapInfo = pointImageMapInfo;
    updateChart(ChartDataEvent.RELOAD_IMAGE_MAP_INFO, 0, 0);
}

/**
 * Sets an array of image map information indexed by series and point.  The
 * arrays get combined into an array of ImageMapInfo objects.
 * @param URLs An array of URLs, indexed by series and point.
 * @param extras An array of extra URL information, indexed by series and point.
 */
public void setPointImageMapInfo(String[][] URLs, String[][] extras)
{
    setPointImageMapInfo(ImageMapInfo.createImageMapInfoArray(URLs, extras));
}

/**
 * Sets an array of image map information for a given series.
 * @param seriesIndex The index of the series.
 * @param pointImageMapInfo The new point image map information array.
 */
public void setPointImageMapInfo(int seriesIndex,
                                 ImageMapInfo[] pointImageMapInfo)
{
    if (seriesIndex < 0) {
        throw new IllegalArgumentException("seriesIndex must be non-negative");
    }
    if (this.pointImageMapInfo == null) {
        this.pointImageMapInfo = new ImageMapInfo[0][];
    }
    if (seriesIndex >= this.pointImageMapInfo.length) {
        ImageMapInfo newValues[][] = new ImageMapInfo[seriesIndex + 1][];
        if (this.pointImageMapInfo.length > 0) {
            System.arraycopy(this.pointImageMapInfo, 0, newValues, 0,
                             this.pointImageMapInfo.length);
        }
        for (int i = this.pointImageMapInfo.length; i <= seriesIndex; i++) {
            newValues[i] = null;
        }
        this.pointImageMapInfo = newValues;
        this.pointImageMapInfo[seriesIndex] = pointImageMapInfo;
    }
    else {
        this.pointImageMapInfo[seriesIndex] = pointImageMapInfo;
    }
    updateChart(ChartDataEvent.RELOAD_IMAGE_MAP_INFO, 0, 0);
}

/**
 * Sets an array of image map information for a given series.
 * @param seriesIndex The index of the series.
 * @param URLs An array of URLs.
 * @param extras An array of extra URL information.
 */
public void setPointImageMapInfo(int seriesIndex, String[] URLs,
                                 String[] extras)
{
    setPointImageMapInfo(seriesIndex,
                         ImageMapInfo.createImageMapInfoArray(URLs, extras));
}

/**
 * Sets an array of image map information, one for each point cluster.
 * @param clusterImageMapInfo The new cluster image map information array.
 */
public void setClusterImageMapInfo(ImageMapInfo[] clusterImageMapInfo)
{
    this.clusterImageMapInfo = clusterImageMapInfo;
    updateChart(ChartDataEvent.RELOAD_IMAGE_MAP_INFO, 0, 0);
}

/**
 * Sets an array of image map information, one for each point cluster.  The
 * arrays get combined into an array of ImageMapInfo objects.
 * @param URLs An array of URLs, one for each point cluster.
 * @param extras An array of extra URL information, one for each point cluster.
 */
public void setClusterImageMapInfo(String[] URLs, String[] extras)
{
    setClusterImageMapInfo(ImageMapInfo.createImageMapInfoArray(URLs, extras));
}

/**
 * Sets an array of image map information, one for each series.
 * @param seriesImageMapInfo  The new series image map information array.
 */
public void setSeriesImageMapInfo(ImageMapInfo[] seriesImageMapInfo)
{
    this.seriesImageMapInfo = seriesImageMapInfo;
    updateChart(ChartDataEvent.RELOAD_IMAGE_MAP_INFO, 0, 0);
}

/**
 * Sets an array of image map information, one for each series.  The
 * arrays get combined into an array of ImageMapInfo objects.
 * @param URLs An array of URLs, one for each series.
 * @param extras An array of extra URL information, one for each series.
 */
public void setSeriesImageMapInfo(String[] URLs, String[] extras)
{
    setSeriesImageMapInfo(ImageMapInfo.createImageMapInfoArray(URLs, extras));
}

/**
 * Sets an array of legend image map information, one for each series.
 * @param legendImageMapInfo The new legend image map information array.
 */
public void setLegendImageMapInfo(ImageMapInfo[] legendImageMapInfo)
{
    this.legendImageMapInfo = legendImageMapInfo;
    updateChart(ChartDataEvent.RELOAD_IMAGE_MAP_INFO, 0, 0);
}

/**
 * Sets an array of legend image map information, one for each series.  The
 * arrays get combined into an array of ImageMapInfo objects.
 * @param URLs An array of URLs, one for each series.
 * @param extras An array of extra URL information, one for each series.
 */
public void setLegendImageMapInfo(String[] URLs, String[] extras)
{
    setLegendImageMapInfo(ImageMapInfo.createImageMapInfoArray(URLs, extras));
}

/**
 * Converts the current data source into a String form.
 * @see com.klg.jclass.chart.data.ChartDataSourceUtil
 * @return
 */
public String buildDataSourceString()
{
    return(ChartDataSourceUtil.buildDataSourceString(this));
}

}

