/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCEditableDataSource.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:52 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.data;

import java.util.Locale;

import com.klg.jclass.chart.EditableChartDataModel;
import com.klg.jclass.util.ImageMapInfo;

/**
 *  A quick and easy default editable data source.  It inherits the vast
 *  majority of it's behavior from <code>JCDefaultDataSource</code>.  It
 *  implements the <code>EditableChartDataModel</code> interface to
 *  update the Y-values array with the edited value.
 */
public class JCEditableDataSource extends JCDefaultDataSource
				                  implements EditableChartDataModel
{

/**
 * The default constructor. Used mostly for the <code>DataSourceEditor</code>.
 */
public JCEditableDataSource()
{
	this(Locale.getDefault());
}

/**
 * The default constructor specifying a locale.
 */
public JCEditableDataSource(Locale locale)
{
	super(locale);
}

/**
 * Creates a data source using the provided data.
 *
 * @param xvalues X series arrays; can be null
 * @param yvalues Y series arrays; can be null
 * @param pointLabels array of point labels; can be null
 * @param seriesLabels array of series labels; can be null
 * @param dataSourceName name applied to this data source
 */
public JCEditableDataSource(double[][] xvalues, double[][] yvalues,
                            String[] pointLabels, String[] seriesLabels,
                            String dataSourceName)
{
	super(xvalues, yvalues, pointLabels, seriesLabels, dataSourceName);
}

/**
 * Creates a data source using the provided data.
 *
 * @param xvalues X series arrays; can be null
 * @param yvalues Y series arrays; can be null
 * @param pointLabels array of point labels; can be null
 * @param seriesLabels array of series labels; can be null
 * @param dataSourceName name to be applied to this data source
 * @param pointImageMapInfo An array of image map information, indexed by
 * series and point.
 * @param clusterImageMapInfo An array of image map information, one for each
 * point cluster.
 * @param seriesImageMapInfo An array of image map information, one for each
 * series.
 * @param legendImageMapInfo An array of legend image map information, one for
 * each series.
 */
public JCEditableDataSource(double[][] xvalues, double[][] yvalues,
                            String[] pointLabels, String[] seriesLabels,
                            String dataSourceName,
                            ImageMapInfo[][] pointImageMapInfo,
                            ImageMapInfo[] clusterImageMapInfo,
                            ImageMapInfo[] seriesImageMapInfo,
                            ImageMapInfo[] legendImageMapInfo)
{
    super(xvalues, yvalues, pointLabels, seriesLabels, dataSourceName,
          pointImageMapInfo, clusterImageMapInfo, seriesImageMapInfo,
          legendImageMapInfo);
}

/**
 * Changes the specified Y data value.
 * @param seriesIndex series index for the changed point
 * @param pointIndex point index for the changed point
 * @param newValue new y value for the specified point
 * @return boolean value indicating whether the new value was
 * accepted; <code>true</code> means the value was accepted
 */
public boolean setDataItem(int seriesIndex, int pointIndex, double newValue)
{
	if (yvalues.length <= seriesIndex) {
		return(false);
    }

	if (yvalues[seriesIndex].length <= pointIndex) {
		return(false);
    }

	yvalues[seriesIndex][pointIndex] = newValue;
	return(true);
}

}
