/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCFileDataSource.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:52 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.data;

import java.io.*;

/**
 * Loads data from a file.
 * <p>
 * For details on the acceptable data formats, please refer to
 * <code>JCDefaultDataInterpreter</code> and <code>JCXMLDataInterpreter</code>.
 *
 * @see JCDefaultDataInterpreter
 * @see JCXMLDataInterpreter
 */
public class JCFileDataSource extends JCInputStreamDataSource {

/**
 * Creates a file data source from the specified file name using the
 * default interpreter class.
 *
 * @param fname the name of the file to be used
 * @exception java.io.FileNotFoundException A <code>FileNotFoundException</code>
 * will be thrown if the file cannot be found.
 * @exception java.io.IOException A <code>IOException</code> will be thrown if
 * a different i/o problem is encountered.
 */
public JCFileDataSource(String fname)
    throws FileNotFoundException, IOException
{
	super(new FileInputStream(fname));
}

/**
 * Creates a file data source from the specified file name using the
 * specified <code>Interpreter</code> class.
 *
 * @param fname the name of the file to be used
 * @param ipr Interpreter class to use to interpret the file
 * @exception java.io.FileNotFoundException A <code>FileNotFoundException</code>
 * will be thrown if the file cannot be found.
 * @exception java.io.IOException A <code>IOException</code> will be thrown if a
 * different i/o problem is encountered.
 */
public JCFileDataSource(String fname, JCDataInterpreter ipr)
    throws FileNotFoundException, IOException
{
	super(new FileInputStream(fname), ipr);
}

/**
 * Creates a file data source using the given charset from the specified file
 * name using the specified <code>Interpreter</code> class.
 *
 * @param fname the name of the file to be used
 * @param ipr Interpreter class to use to interpret the file
 * @param charsetName The name of a supported charset
 * @exception java.io.FileNotFoundException A <code>FileNotFoundException</code>
 * will be thrown if the file cannot be found.
 * @exception java.io.IOException A <code>IOException</code> will be thrown if a
 * different i/o problem is encountered.
 */
public JCFileDataSource(String fname, JCDataInterpreter ipr, String charsetName)
    throws FileNotFoundException, IOException
{
    super(new FileInputStream(fname), ipr, charsetName);
}

/**
 * Creates a file data source from the specified <code>File</code> object using
 * the default <code>Interpreter</code> class.
 *
 * @param f a <code>File</code> object representing the file to be opened
 * @exception java.io.FileNotFoundException A <code>FileNotFoundException</code>
 * will be thrown if the file cannot be found.
 * @exception java.io.IOException A <code>IOException</code> will be thrown if
 * a different i/o problem is encountered.
 * @see java.io.File
 */
public JCFileDataSource(File f)
    throws FileNotFoundException, IOException
{
	super(new FileInputStream(f));
}

/**
 * Creates a file data source from the specified <code>File</code> object using
 * the specified <code>Interpreter</code> class.
 *
 * @param f a <code>File</code> object representing the file to be opened
 * @param ipr <code>Interpreter</code> class to use to interpret the file
 * @exception java.io.FileNotFoundException A <code>FileNotFoundException</code>
 * will be thrown if the file cannot be found.
 * @exception java.io.IOException A <code>IOException</code> will be thrown if
 * a different i/o problem is encountered.
 * @see java.io.File
 */
public JCFileDataSource(File f, JCDataInterpreter ipr)
    throws FileNotFoundException, IOException
{
	super(new FileInputStream(f), ipr);
}

/**
 * Creates a file data source based on the given charset from the specified
 * <code>File</code> object using the specified <code>Interpreter</code> class.
 *
 * @param f a <code>File</code> object representing the file to be opened
 * @param ipr <code>Interpreter</code> class to use to interpret the file
 * @param charsetName The name of a supported charset
 * @exception java.io.FileNotFoundException A <code>FileNotFoundException</code>
 * will be thrown if the file cannot be found.
 * @exception java.io.IOException A <code>IOException</code> will be thrown if
 * a different i/o problem is encountered.
 * @see java.io.File
 */
public JCFileDataSource(File f, JCDataInterpreter ipr, String charsetName)
    throws FileNotFoundException, IOException
{
    super(new FileInputStream(f), ipr, charsetName);
}

}
