/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCReaderDataSource.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:52 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.data;

import java.io.*;

/**
 * This class is the base for any data source that relies on input
 * from a reader. Data is interpreted according by a interpreter classes that
 * subclasses from <code>JCDataInterpreter</code>.
 */
public class JCReaderDataSource extends JCInputStreamDataSource {

/**
 * This constructor takes the <code>Reader</code>, reads in all the
 * appropriate data and interprets it according to the default interpreter
 * class.
 *
 * @param r <code>Reader</code> object used to read the data
 * @exception IOException An <code>IOException</code> will be thrown if there
 * are any errors while reading in the data.
 */
public JCReaderDataSource(Reader r) throws IOException
{
    ctor(r);
}

/**
 * This constructor takes the <code>Reader</code>, reads in all the
 * appropriate data, and interprets it according to the specified interpreter
 * class.
 *
 * @param r <code>Reader</code> object used to read the data
 * @param p <code>Interpreter</code> used to make sense of the data
 * @exception IOException An <code>IOException</code> will be thrown if there
 * are any errors while reading in the data.
 */
public JCReaderDataSource(Reader r, JCDataInterpreter p)
    throws IOException
{
    super(p);
    ctor(r);
}


/**
 * This constructor sets up a <code>JCReaderDataSource</code> class to use
 * the default interpreter. It does not yet read or interpret any data.
 * @throws IOException An <code>IOException</code> will be thrown if there are
 * any errors while reading in the data.
 */
protected JCReaderDataSource() throws IOException {}

/**
 * This constructor sets up a <code>JCReaderDataSource</code> class to use
 * the specified interpreter. It does not yet read or interpret any data.
 * @param p the data interpreter to be used
 * @throws IOException An <code>IOException</code> will be thrown if there are
 * any errors while reading in the data.
 */
protected JCReaderDataSource(JCDataInterpreter p) throws IOException
{
    super(p);
}

}
