/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCURLDataSource.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:52 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.data;

import java.io.*;
import java.net.*;

/**
 * This class takes a URL and turns it into a JClass Chart data source.
 */
public class JCURLDataSource extends JCInputStreamDataSource {

/**
 * Pulls data from specified URL according to the specified data interpreter.
 * @param url a String representing the location of the data file
 * (for example, http://www.quest.com/data.file)
 * @exception MalformedURLException A <code>MalformedURLException</code> is
 * thrown if the specified URL is not in a valid format.
 * @exception IOException An <code>IOException</code> is thrown if there is some
 * problem reading the file.
 */
public JCURLDataSource(String url)
    throws MalformedURLException, IOException
{
    super((new URL(url)).openStream());
}

/**
 * Pulls data from specified URL according to the specified data interpreter.
 * @param url a String representing the location of the data file
 * (for example, http://www.quest.com/data.file)
 * @param ipr <code>Interpreter</code> class used to interpret the data
 * @exception MalformedURLException A <code>MalformedURLException</code> is
 * thrown if the specified URL is not in a valid format.
 * @exception IOException An <code>IOException</code> is thrown if there is some
 * problem reading the file.
 */
public JCURLDataSource(String url, JCDataInterpreter ipr)
    throws MalformedURLException, IOException
{
    super((new URL(url)).openStream(), ipr);
}

/**
 * Pulls data from specified URL using the given charset according to the
 * specified data interpreter.
 * @param url a String representing the location of the data file
 * (for example, http://www.quest.com/data.file)
 * @param ipr <code>Interpreter</code> class used to interpret the data
 * @param charsetName The name of a supported charset
 * @exception MalformedURLException A <code>MalformedURLException</code> is
 * thrown if the specified URL is not in a valid format.
 * @exception IOException An <code>IOException</code> is thrown if there is some
 * problem reading the file.
 */
public JCURLDataSource(String url, JCDataInterpreter ipr, String charsetName)
    throws MalformedURLException, IOException
{
    super((new URL(url)).openStream(), ipr, charsetName);
}

/**
 * Pulls data from specified URL according to the default data interpreter.
 * @param host the WWW hostname
 * @param file the fully-qualified name of the file on the server
 * @exception MalformedURLException A <code>MalformedURLException</code> is
 * thrown if the specified URL is not in a valid format.
 * @exception IOException An <code>IOException</code> is thrown if there is some
 * problem reading the file.
 */
public JCURLDataSource(String host, String file)
    throws MalformedURLException, IOException
{
    super((new URL("http", host, file)).openStream());
}

/**
 * Pulls data from specified URL according to the specified data interpreter.
 * @param host the WWW hostname
 * @param file the fully-qualified name of the file on the server
 * @param ipr <code>Interpreter</code> class used to interpret the data
 * @exception MalformedURLException A <code>MalformedURLException</code> is
 * thrown if the specified URL is not in a valid format.
 * @exception IOException An <code>IOException</code> is thrown if there is some
 * problem reading the file.
 */
public JCURLDataSource(String host, String file, JCDataInterpreter ipr)
    throws MalformedURLException, IOException
{
    super((new URL("http", host, file)).openStream(), ipr);
}

/**
 * Pulls data from specified URL using the given charset according to the
 * specified data interpreter.
 * @param host the WWW hostname
 * @param file the fully-qualified name of the file on the server
 * @param ipr <code>Interpreter</code> class used to interpret the data
 * @param charsetName The name of a supported charset
 * @exception MalformedURLException A <code>MalformedURLException</code> is
 * thrown if the specified URL is not in a valid format.
 * @exception IOException An <code>IOException</code> is thrown if there is some
 * problem reading the file.
 */
public JCURLDataSource(String host, String file, JCDataInterpreter ipr,
                       String charsetName)
    throws MalformedURLException, IOException
{
    super((new URL("http", host, file)).openStream(), ipr, charsetName);
}

/**
 * Pulls data from specified URL according to the default data interpreter.
 * @param protocol The protocol to use to access the file. Currently only
 * http is supported.
 * @param host the WWW hostname
 * @param file the fully-qualified name of the file on the server
 * @exception MalformedURLException A <code>MalformedURLException</code> is
 * thrown if the specified URL is not in a valid format.
 * @exception IOException An <code>IOException</code> is thrown if there is some
 * problem reading the file.
 */
public JCURLDataSource(String protocol, String host, String file)
    throws MalformedURLException, IOException
{
    super((new URL(protocol, host, file)).openStream());
}

/**
 * Pulls data from specified URL according to the specified data interpreter.
 * @param protocol the protocol to use to access the file (currently only
 * http is supported)
 * @param host the WWW hostname
 * @param file the fully-qualified name of the file on the server
 * @param ipr <code>Interpreter</code> class used to interpret the data
 * @exception MalformedURLException A <code>MalformedURLException</code> is
 * thrown if the specified URL is not in a valid format.
 * @exception IOException An <code>IOException</code> is thrown if there is some
 * problem reading the file.
 */
public JCURLDataSource(String protocol, String host, String file,
                       JCDataInterpreter ipr)
    throws MalformedURLException, IOException
{
    super((new URL(protocol, host, file)).openStream(), ipr);
}

/**
 * Pulls data from specified URL using the given charset according to the
 * specified data interpreter.
 * @param protocol the protocol to use to access the file (currently only
 * http is supported)
 * @param host the WWW hostname
 * @param file the fully-qualified name of the file on the server
 * @param ipr <code>Interpreter</code> class used to interpret the data
 * @param charsetName The name of a supported charset
 * @exception MalformedURLException A <code>MalformedURLException</code> is
 * thrown if the specified URL is not in a valid format.
 * @exception IOException An <code>IOException</code> is thrown if there is some
 * problem reading the file.
 */
public JCURLDataSource(String protocol, String host, String file,
                       JCDataInterpreter ipr, String charsetName)
    throws MalformedURLException, IOException
{
    super((new URL(protocol, host, file)).openStream(), ipr, charsetName);
}

/**
 * Pulls data from specified URL according to the default data interpreter.
 * @param base a URL object representing the directory in which to find
 * the file
 * @param file the name of the file on the server
 * @exception MalformedURLException A <code>MalformedURLException</code> is
 * thrown if the specified URL is not in a valid format.
 * @exception IOException An <code>IOException</code> is thrown if there is some
 * problem reading the file.
 * @see java.net.URL
 */
public JCURLDataSource(URL base, String file)
    throws MalformedURLException, IOException
{
    super((new URL(base, file)).openStream());
}

/**
 * Pulls data from specified URL according to the specified data interpreter.
 * @param base a URL object representing the directory in which to find
 * the file
 * @param file the name of the file on the server
 * @param ipr <code>Interpreter</code> class used to interpret the data
 * @exception MalformedURLException A <code>MalformedURLException</code> is
 * thrown if the specified URL is not in a valid format.
 * @exception IOException An <code>IOException</code> is thrown if there is some
 * problem reading the file.
 * @see java.net.URL
 */
public JCURLDataSource(URL base, String file, JCDataInterpreter ipr)
    throws MalformedURLException, IOException
{
    super((new URL(base, file)).openStream(), ipr);
}

/**
 * Pulls data from specified URL using the given charset according to the
 * specified data interpreter.
 * @param base a URL object representing the directory in which to find
 * the file
 * @param file the name of the file on the server
 * @param ipr <code>Interpreter</code> class used to interpret the data
 * @param charsetName The name of a supported charset
 * @exception MalformedURLException A <code>MalformedURLException</code> is
 * thrown if the specified URL is not in a valid format.
 * @exception IOException An <code>IOException</code> is thrown if there is some
 * problem reading the file.
 * @see java.net.URL
 */
public JCURLDataSource(URL base, String file, JCDataInterpreter ipr,
                       String charsetName)
    throws MalformedURLException, IOException
{
    super((new URL(base, file)).openStream(), ipr, charsetName);
}

}
