/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: XMLDataHandler.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:52 $  $Locker:  $  Quest Software Inc.
package com.klg.jclass.chart.data;

import java.util.ArrayList;
import java.util.List;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import java.io.*;

import com.klg.jclass.chart.ChartDataModel;
import com.klg.jclass.chart.LabelledChartDataModel;
import com.klg.jclass.chart.HoleValueChartDataModel;
import com.klg.jclass.chart.ImageMapChartDataModel;
import com.klg.jclass.util.ImageMapInfo;


/**
 * Convenience class extends DefaultHandler to give parsing instructions for
 * JCChart data objects. Public only so chart classes can access it.
 */
public class XMLDataHandler extends DefaultHandler {

private static int lowCharacterBound  = 32;
private static int highCharacterBound = 126;

protected StringBuffer    sb                  = null;
protected Double          curValue            = null;
protected BaseDataSource  baseDS              = null;
protected List            xData               = null;
protected List            yData               = null;
protected List            xDataArrays         = null;
protected List            yDataArrays         = null;
protected List            seriesLabels        = null;
protected List            pointLabels         = null;

// ImageMapInfo lists
protected List            pointImageMapInfo   = null;
protected List            pointImageMapArrays = null;
protected List            clusterImageMapInfo = null;
protected List            seriesImageMapInfo  = null;
protected List            legendImageMapInfo  = null;

protected String          seriesLabel         = null;
protected String          pointLabel          = null;

protected boolean         inSeries            = false;
protected boolean         inPoint             = false;
protected boolean         usingSeries         = true;
protected boolean         newTags             = true;

/** String for top level tag of chart XML datasource. */
protected String XML_DS = null;
/** String for name attribute of top level tag in the chart XML datasource. */
protected String XML_DS_ATT_NAME = null;
/** String for hole attriubute of top level tag in the chart XML datasource. */
protected String XML_DS_ATT_HOLE = null;
/** String for series tag of chart XML datasource. */
protected String XML_DS_SERIES = null;
/** String for series label tag of chart XML datasource. */
protected String XML_DS_SERIESLABEL = null;
/** String for point tag of chart XML datasource. */
protected String XML_DS_POINT = null;
/** String for point label tag of chart XML datasource. */
protected String XML_DS_POINTLABEL = null;
/** String for x data tag of chart XML datasource. */
protected String XML_DS_XDATA = null;
/** String for y data tag of chart XML datasource. */
protected String XML_DS_YDATA = null;
/** String for point url attribute of the YDATA tag. */
protected String XML_DS_POINT_URL = "pointImageMapURL";
/** String for point extra attribute of the YDATA tag. */
protected String XML_DS_POINT_EXTRA = "pointImageMapExtra";
/** String for cluster url attribute of the XDATA tag. */
protected String XML_DS_CLUSTER_URL = "clusterImageMapURL";
/** String for cluster extra attribute of the XDATA tag. */
protected String XML_DS_CLUSTER_EXTRA = "clusterImageMapExtra";
/** String for series url attribute of the SERIES tag. */
protected String XML_DS_SERIES_URL = "seriesImageMapURL";
/** String for series extra attribute of the SERIES tag. */
protected String XML_DS_SERIES_EXTRA = "seriesImageMapExtra";
/** String for legend url attribute of the SERIES tag. */
protected String XML_DS_LEGEND_URL = "legendImageMapURL";
/** String for legend extra attribute of the SERIES tag. */
protected String XML_DS_LEGEND_EXTRA = "legendImageMapExtra";

/**
 * Default constructor that provides a datasource into which the data in the
 * file is to be read.
 * @param baseDS The datasource to populate
 */
public XMLDataHandler(BaseDataSource baseDS)
{
    super();
    this.baseDS = baseDS;
    useNewTags();
}

/**
 * Use the new XML tags (after JClass 6.0).
 */
public void useNewTags()
{
    XML_DS             = "chart-data";
    XML_DS_ATT_NAME    = "name";
    XML_DS_ATT_HOLE    = "hole";
    XML_DS_SERIES      = "data-series";
    XML_DS_SERIESLABEL = "data-series-label";
    XML_DS_POINT       = "data-point";
    XML_DS_POINTLABEL  = "data-point-label";
    XML_DS_XDATA       = "x-data";
    XML_DS_YDATA       = "y-data";
    newTags            = true;
}

/**
 * Use the old XML tags (JClass 6.0 and earlier).
 */
public void useOldTags()
{
    XML_DS             = "JCChartData";
    XML_DS_ATT_NAME    = "Name";
    XML_DS_ATT_HOLE    = "Hole";
    XML_DS_SERIES      = "Series";
    XML_DS_SERIESLABEL = "SeriesLabel";
    XML_DS_POINT       = "Point";
    XML_DS_POINTLABEL  = "PointLabel";
    XML_DS_XDATA       = "XData";
    XML_DS_YDATA       = "YData";
    newTags            = false;
}

/**
 * Set the lower and upper character bounds as integers.  By default this
 * range is set to be low=32 and high=126 which is the normal ascii printable
 * range.
 * @param lower The new lower bound
 * @param higher The new higher bound
 */
public static void setCharacterBounds(int lower, int higher)
{
    if (lower > higher || lower < 0) {
        throw new IllegalArgumentException("Illegal bounds - lower:" + lower +
                                           " higher:" + higher);
    }
    lowCharacterBound  = lower;
    highCharacterBound = higher;
}

/**
 * Get the low character bound.
 * @return The current low character bound
 */
public static int getLowCharacterBound()
{
    return(lowCharacterBound);
}

/**
 * Get the high character bound.
 * @return The current high character bound
 */
public static int getHighCharacterBound()
{
    return(highCharacterBound);
}

/**
 * Get the data source into which the data will or has been read in.
 * @return The current data source
 */
public BaseDataSource getDataSource()
{
    return(baseDS);
}

/**
 * Get the data source into which the data will or has been read in.
 * @param baseDS The new data source
 */
public void setDataSource(BaseDataSource baseDS)
{
    this.baseDS = baseDS;
}

/**
 * Event triggered when the start of an Element is encountered.
 * @param namespaceURI the namespaceURI for this element.
 * @param localName the localName value of this element.
 * @param qName the qualified name of this element.
 * @param atts the Attributes for this element
 * @throws SAXException
 */
public void startElement(String namespaceURI,
                         String localName,
                         String qName,
                         Attributes atts)
    throws SAXException
{
    handleStartElement(namespaceURI, localName, qName, atts);
}

/**
 * Implementation of endElement, cleans up objects no longer required.
 * @param namespaceURI the namespaceURI for this element.
 * @param localName the localName value of this element.
 * @param qName the qualified name of this element.
 * @throws SAXException
 */
public void endElement(String namespaceURI,
                       String localName,
                       String qName)
    throws SAXException
{
    handleEndElement(namespaceURI, localName, qName);
}

/**
 * Receive notification of character data inside an element.
 *
 * @param ch The characters.
 * @param start The start position in the character array.
 * @param length The number of characters to use from the
 *               character array.
 * @exception org.xml.sax.SAXException Any SAX exception, possibly
 *            wrapping another exception.
 * @see org.xml.sax.ContentHandler#characters
 */
public void characters(char ch[],
                       int start,
                       int length)
    throws SAXException
{
    handleCharacters(ch, start, length);
}

/**
 * Event triggered when the start of an Element is encountered.
 * @param namespaceURI the namespaceURI for this element.
 * @param localName the localName value of this element.
 * @param qName the qualified name of this element.
 * @param atts the Attributes for this element
 * @throws SAXException
 */
public void handleStartElement(String namespaceURI,
                               String localName,
                               String qName,
                               Attributes atts)
    throws SAXException
{
    if (baseDS == null) {
        return;
    }

    if (qName.equals("JCChartData") && newTags) {
        useOldTags();
    }
    else if (qName.equals("chart-data") && !newTags) {
        useNewTags();
    }

    if (qName.equals(XML_DS_SERIES)) {
        inSeries = true;
        usingSeries = true;
        if (xData == null) {
            xData = new ArrayList();
        }
        if (yData == null) {
            yData = new ArrayList();
        }
        if (pointImageMapInfo == null) {
            pointImageMapInfo = new ArrayList();
        }
       // If we have no valid xArrays yet but do have cluster URL info, reset
        // so that, the cluster info matches the first valid xArray.
        if (xDataArrays.size() == 0 && clusterImageMapInfo.size() > 0) {
            clusterImageMapInfo = new ArrayList();
        }

        processURL(atts, XML_DS_SERIES_URL, XML_DS_SERIES_EXTRA,
                   seriesImageMapInfo);
        processURL(atts, XML_DS_LEGEND_URL, XML_DS_LEGEND_EXTRA,
                   legendImageMapInfo);
    }
    else if (qName.equals(XML_DS_POINT)) {
        inPoint = true;
        usingSeries = false;
        if (xData == null) {
            xData = new ArrayList();
        }
        if (yData == null) {
            yData = new ArrayList();
        }
        if (pointImageMapInfo == null) {
            pointImageMapInfo = new ArrayList();
        }
    }
    else if (qName.equals(XML_DS_XDATA)) {
        // If we're in series mode and we already have a valid xArray, don't add
        // anymore values to the cluster arrays.
        if ((inSeries && xDataArrays.size() == 0) || inPoint) {
            processURL(atts, XML_DS_CLUSTER_URL, XML_DS_CLUSTER_EXTRA,
                       clusterImageMapInfo);
        }
    }
    else if (qName.equals(XML_DS_YDATA)) {
        processURL(atts, XML_DS_POINT_URL, XML_DS_POINT_EXTRA,
                   pointImageMapInfo);
    }
    else if (qName.equals(XML_DS)) {
        if (atts != null) {
            int index = atts.getIndex(XML_DS_ATT_NAME);
            if (index >-1) {
                baseDS.name = atts.getValue(index);
            }

            index = atts.getIndex(XML_DS_ATT_HOLE);
            if (index > -1) {
                baseDS.holeValue =
                    getNextDouble(atts.getValue(index)).doubleValue();
            }
        }

        xDataArrays         = new ArrayList();
        yDataArrays         = new ArrayList();
        seriesLabels        = new ArrayList();
        pointLabels         = new ArrayList();

        pointImageMapArrays = new ArrayList();
        clusterImageMapInfo = new ArrayList();
        seriesImageMapInfo  = new ArrayList();
        legendImageMapInfo  = new ArrayList();
    }
    
}

    /**
 * Implementation of endElement, cleans up objects no longer required.
 * @param namespaceURI the namespaceURI for this element.
 * @param localName the localName value of this element.
 * @param qName the qualified name of this element.
 * @throws SAXException
 */
public void handleEndElement(String namespaceURI,
                             String localName,
                             String qName)
    throws SAXException
{
    String sbString = (sb == null) ? null : sb.toString();
    
    if (qName.equals(XML_DS_XDATA)) {
        curValue = getNextDouble(sbString);
        
        xData.add(curValue);
    }
    else if (qName.equals(XML_DS_YDATA)) {
        curValue = getNextDouble(sbString);
        
        yData.add(curValue);
    }
    else if (qName.equals(XML_DS_SERIESLABEL)) {
        seriesLabel = sbString;
        if (!inSeries) {
            seriesLabels.add(seriesLabel);
            seriesLabel = null;
        }
    }
    else if (qName.equals(XML_DS_POINTLABEL)) {
        pointLabel = sbString;
        if (!inPoint) {
            pointLabels.add(pointLabel);
            pointLabel = null;
        }
    }
    else if (qName.equals(XML_DS_SERIES)) {
        processXYData();
        if (seriesLabel != null) {
            seriesLabels.add(seriesLabel);
        }
        seriesLabel = null;
        inSeries = false;
    }
    else if (qName.equals(XML_DS_POINT)) {
        processXYData();
        if (pointLabel != null) {
            pointLabels.add(pointLabel);
        }
        pointLabel = null;
        inPoint = false;
    }
    else if (qName.equals(XML_DS)) {
        if (xData != null) {              // Point data - must process first
            double[] xArray = processListToDoubles(xData);
            xDataArrays.add(xArray);
            xData = null;
        }
        processDataArrays();
    }
    sb = null;
}

/**
 * Receive notification of character data inside an element.
 *
 * @param ch The characters.
 * @param start The start position in the character array.
 * @param length The number of characters to use from the
 *               character array.
 * @exception org.xml.sax.SAXException Any SAX exception, possibly
 *            wrapping another exception.
 * @see org.xml.sax.ContentHandler#characters
 */
public void handleCharacters(char ch[],
                             int start,
                             int length)
    throws SAXException
{
    String s = new String(ch, start, length);
    
    if (sb == null) {
        sb = new StringBuffer(s);
    }
    else {
        sb.append(s);
    }
}

/**
 * Given an list of Double objects, store the double values in a doubles
 * array.
 * @param a The list of Double objects
 * @return The newly created doubles array
 */
protected double[] processListToDoubles(List a)
{
    if (a == null || a.size() == 0) {
        return(null);
    }
    double[] d = new double[a.size()];
    for (int i = 0; i < a.size(); i++) {
        curValue = (Double)a.get(i);
        d[i] = curValue.doubleValue();
    }
    return(d);
}

/**
 * Given an list of ImageMapInfo objects, store them in a ImageMapInfo array.
 * @param a The list of ImageMapInfo objects
 * @return The newly created ImageMapInfo array
 */
protected ImageMapInfo[] processListToImageMapInfoArray(List a)
{
    if (a == null || a.size() == 0) {
        return(null);
    }
    ImageMapInfo[] imageMaps = new ImageMapInfo[a.size()];
    for (int i = 0; i < a.size(); i++) {
        imageMaps[i] = (ImageMapInfo)a.get(i);
    }
    return(imageMaps);
}

/**
 * Take the Double objects from the array lists, convert to double arrays and
 * store them in xDataArrays and yDataArrays.
 * @throws SAXException if trouble with doubles conversion
 */
protected void processXYData() throws SAXException
{
    

    double[] yArray = processListToDoubles(yData);
    yDataArrays.add(yArray);
    pointImageMapArrays.add(processListToImageMapInfoArray(pointImageMapInfo));
    yData = null;
    pointImageMapInfo = null;

    if (inSeries) {
        double[] xArray = processListToDoubles(xData);
        if (xArray != null) {
            xDataArrays.add(xArray);
        }
        xData = null;
    }
    if (inPoint) {
        // Check if xData was given for the current point.  If not provide some.
        if (xData.size() < yDataArrays.size()) {
            xData.add(new Double((double)(xData.size() - 1)));
            clusterImageMapInfo.add(new ImageMapInfo());
        }
    }
}

/**
 * Given a double array and a size n, extend the array out to be of size n
 * and pad with hole values.  Do nothing if size of array is already at least n.
 * @param d The input double array
 * @param n The size the array must be
 * @return A possibly new array padded out with doubles
 */
protected double[] fillOutDoubleArray(double[] d, int n)
{
    if (d != null && d.length >= n) {
        return(d);
    }

    double[] newd = new double[n];
    int curSize = (d == null) ? 0 : d.length;
    if (curSize > 0) {
        System.arraycopy(d, 0, newd, 0, curSize);
    }
    for (int i = curSize; i < n; i++) {
        newd[i] = baseDS.getHoleValue();
    }
    return(newd);
}

/**
 * Given a ImageMapInfo array and a size n, extend the array out to be of size n
 * and pad with null values.  Do nothing if size of array is already at least n.
 * @param info The input ImageMapInfo array
 * @param n The size the array must be
 * @return A possibly new array padded out with nulls
 */
protected ImageMapInfo[] fillOutImageMapInfoArray(ImageMapInfo[] info, int n)
{
    if (info != null && info.length >= n) {
        return(info);
    }

    ImageMapInfo[] newInfo = new ImageMapInfo[n];
    int curSize = (info == null) ? 0 : info.length;
    if (curSize > 0) {
        System.arraycopy(info, 0, newInfo, 0, curSize);
    }
    for (int i = curSize; i < n; i++) {
        newInfo[i] = null;
    }
    return(newInfo);
}

/**
 * Create a string array of size n from the list of strings in the passed in
 * List. The reason we don't use List.toArray() is because we need
 * make sure that empty or null strings are set to be empty strings (if its
 * position in the array is less than the size of the List).  Pad out the
 * string array with nulls if the passed in List is less than n.
 *
 * @param a list the labels are in
 * @param n The number of items in the array of strings to return
 * @return A string array with the labels in it
 */
protected String[] processLabels(List a, int n)
{
    if (n <= 0 || a == null || a.size() == 0) {
        return(null);
    }

    String[] labels = new String[n];
    for (int i = 0; i < n; i++) {
        if (i < a.size()) {
            String l = (String)a.get(i);
            if (l == null || l.length() == 0) {
                l = "";
            }
            labels[i] = l;
        }
        else {
            labels[i] = null;
        }
    }
    return(labels);
}

/**
 * Check the List containing arrays of ImageMapInfo objects for non-empty
 * values. Return true if we find one non-empty value.
 * @param a The input List that contains arrays of ImageMapInfo objects.
 * @return true, if one of the arrays of ImageMapInfo objects contains a
 * non-empty value.
 */
protected boolean checkListofImageMapInfoArrays(List a)
{
    if (a == null || a.size() == 0) {
        return(false);
    }
    for (int i = 0; i < a.size(); i++) {
        ImageMapInfo[] curList = (ImageMapInfo[])a.get(i);
        if (!ImageMapInfo.isEmpty(curList)) {
            return(true);
        }
    }
    return(false);
}

/**
 * Process the data array strings, currently stored in xDataArrays and
 * yDataArrays.  Convert the strings to doubles and store them in the data
 * source.  Note that the data could have been read in by series (usingSeries
 * is true) or by point.
 *
 * Read in by series:
 *    - 0 or more x arrays.  0 or 1 ==> Array data.  If more than one there
 *      must be exacly the same number of x arrays as y arrays.
 *    - y arrays represent a list of point values for each series
 *
 * Read in by point:
 *    - Exactly one x array.
 *    - y arrays represent a list of series values for each point and must
 *      thus be transposed.
 *
 * After reading in data, the series and point labels need to be processed.
 */
protected void processDataArrays()
{
    double[] x, y;

    int nXSeries = xDataArrays.size();
    int nSeries = 0;
    if (usingSeries) {
        nSeries = yDataArrays.size();
    }
    else {
        for (int i = 0; i < yDataArrays.size(); i++) {
            y = (double[])yDataArrays.get(i);
            if (y != null) {
                nSeries = Math.max(nSeries, y.length);
            }
        }
    }

    // Initialize point url info
    boolean usingPointURLs = checkListofImageMapInfoArrays(pointImageMapArrays);
    if (usingPointURLs) {
        baseDS.pointImageMapInfo = new ImageMapInfo[nSeries][];
    }

    // Initialize cluster url ifno
    boolean usingClusterURLs = false;
    ImageMapInfo[] clusterURLInfo = null;
    if (nXSeries > 0) {
        clusterURLInfo = processListToImageMapInfoArray(clusterImageMapInfo);
        usingClusterURLs = !ImageMapInfo.isEmpty(clusterURLInfo);
    }

    // Always have at least one X series (if not we will create one later).
    // If more than one X series we have general mode. Otherwise its array mode.
    // Note that if usingSeries is false (data read in by points) then the data
    // is automatically in array mode.
    baseDS.xvalues = new double[(nXSeries > 1) ? nXSeries : 1][];
    baseDS.yvalues = new double[nSeries][];

    // For array mode, find the maximum number of points in the arrays.  We
    // need to know this so we can pad out arrays that aren't long enough with
    // hole values.
    int nPoints = 0;
    if (nXSeries <= 1) {
        x = (nXSeries == 1) ? (double[])xDataArrays.get(0) : null;
        nPoints = (x == null) ? 0 : x.length;
        if (usingSeries) {
            for (int i = 0; i < nSeries; i++) {
                y = (double[])yDataArrays.get(i);
                if (y != null) {
                    nPoints = Math.max(nPoints, y.length);
                }
            }
        }
        else {
            nPoints = Math.max(nPoints, yDataArrays.size());
        }
    }

    // For array mode, the x series is invariant so set it now
    x = null;
    if (nXSeries == 0) {  // Create an X series if there isn't one
        x = new double[nPoints];
        for (int j = 0; j < nPoints; j++) {
            x[j] = (double) (j + 1);
        }
    }
    else if (nXSeries == 1) {
        x = (double[])xDataArrays.get(0);
    }
    int xLength = (x == null) ? 0 : x.length;

    // Put the x and y arrays in the datasource
    int maxPoints = nPoints;
    for (int i = 0; i < yDataArrays.size(); i++) {
        y = (double[])yDataArrays.get(i);
        int yLength = (y == null) ? 0 : y.length;

        // X data
        if (nXSeries <= 1) {  // Array mode
            // Only need to do this once
            if (i == 0) {
                baseDS.xvalues[0] = fillOutDoubleArray(x, nPoints);
            }
        }
        else {                // General mode
            x = (double[])xDataArrays.get(i);
            xLength = (x == null) ? 0 : x.length;
            nPoints = Math.max(xLength, yLength);
            maxPoints = Math.max(nPoints, maxPoints);
            baseDS.xvalues[i] = fillOutDoubleArray(x, nPoints);
        }

        // Cluster URL and Extra strings.  Only do first time as cluster info is
        // valid only for array data situations (bars, stacking bars, and pies).
        // We do this for both array and general mode because the user may have
        // specified the same x values for each series.  This way we pick up
        // the URL info, for the first series.
        if (i == 0) {
            if (usingClusterURLs) {
                baseDS.clusterImageMapInfo =
                    fillOutImageMapInfoArray(clusterURLInfo, nPoints);
            }
        }

        // Y data
        ImageMapInfo[] pointInfo = (ImageMapInfo[])pointImageMapArrays.get(i);
        if (usingSeries) {    // Store arrays directly in data source
            baseDS.yvalues[i] = fillOutDoubleArray(y, nPoints);
            if (usingPointURLs) {
                baseDS.pointImageMapInfo[i] =
                    fillOutImageMapInfoArray(pointInfo, nPoints);
            }
        }
        else {                // Transpose data
            for (int j = 0; j < nSeries; j++) {
                // First time allocate arrays
                if (i == 0) {
                    baseDS.yvalues[j] = new double[nPoints];
                    if (usingPointURLs) {
                        baseDS.pointImageMapInfo[j] = new ImageMapInfo[nPoints];
                    }
                }
                // If not enough points, pad out with hole or null values
                boolean pad = (j >= yLength);
                baseDS.yvalues[j][i] = pad ? baseDS.getHoleValue() : y[j];
                if (usingPointURLs) {
                    baseDS.pointImageMapInfo[j][i] = pad ? null : pointInfo[j];
                }
            }
        }
    }
    xDataArrays         = null;
    yDataArrays         = null;
    pointImageMapArrays = null;

    // Process series URL strings
    ImageMapInfo[] info = processListToImageMapInfoArray(seriesImageMapInfo);
    if (!ImageMapInfo.isEmpty(info)) {
        baseDS.seriesImageMapInfo = info;
    }
    // Process legend URL strings
    info = processListToImageMapInfoArray(legendImageMapInfo);
    if (!ImageMapInfo.isEmpty(info)) {
        baseDS.legendImageMapInfo = info;
    }

    // Process series labels and point labels
    baseDS.seriesLabels = processLabels(seriesLabels, nSeries);
    baseDS.pointLabels  = processLabels(pointLabels, maxPoints);

    seriesLabels = null;
    pointLabels  = null;
}

/**
 * Process the URL and extra attributes for a given element.
 * @param atts The Attributes for this element
 * @param URLName The URL attribute name
 * @param extraName The extra attribute name
 * @param imageMapInfoList The list to which the URL and extra values are added.
 */
protected void processURL(Attributes atts, String URLName, String extraName,
                          List imageMapInfoList)
{
    String URL = null;
    String extra = null;
    if (atts != null) {
        int index = atts.getIndex(URLName);
        if (index > -1) {
            URL = atts.getValue(index);
        }

        index = atts.getIndex(extraName);
        if (index > -1) {
            extra = atts.getValue(index);
        }
    }
    ImageMapInfo imageMapInfo = new ImageMapInfo(URL, extra);
    imageMapInfoList.add(imageMapInfo);
}

/**
 * Utility function used to retrieve next double value from a string.
 * @param s The string from which to retrieve the double
 * @return The retrieved double value
 * @throws SAXException if an error occurs in the conversion
 */
protected Double getNextDouble(String s) throws SAXException
{
    Double d = null;
    if (s == null || s.equalsIgnoreCase("HOLE")) {
        d = new Double(baseDS.getHoleValue());
    }
    else if (s.equalsIgnoreCase("MAX")) {
        d = new Double(Double.MAX_VALUE);
    }
    else if (s.equalsIgnoreCase("MIN")) {
        d = new Double(Double.MIN_VALUE);
    }
    else if (s.equalsIgnoreCase("INFINITY")) {
        d = new Double(Double.POSITIVE_INFINITY);
    }
    else {
        try {
            d = Double.valueOf(s);
        }
        catch (Exception e) {
            displayErrorMessage(JCDataInterpreter.PARSE_ERR_HOLE_OR_VALUE, s);
            throw new SAXException(JCDataInterpreter.ERRMSG_FILE_FORMAT);
        }
        if (d == null) {
            d = new Double(0.0);
        }
    }
    return (d);
}

/**
 * Print the error message given an error enum and a message to append to
 * the end.
 * @param nErrorIndex The error enum
 * @param tagVal The message to append to the canned message
 */
protected void displayErrorMessage(int nErrorIndex, String tagVal)
{
    String strError = "";

    switch (nErrorIndex) {
    case JCDataInterpreter.PARSE_ERR_DATA_STYLE:
        strError += JCDataInterpreter.ERRMSG_MUST_START + " and not ";
        break;
    case JCDataInterpreter.PARSE_ERR_HOLE_OR_VALUE:
        strError += JCDataInterpreter.ERRMSG_EXPECT_HOLE;
        break;
    case JCDataInterpreter.PARSE_ERR_POINT_LABEL:
        strError += JCDataInterpreter.ERRMSG_EXPECT_POINT_LABEL;
        break;
    default:
        return;
    }
    if (tagVal != null && tagVal.length() > 0) {
        strError += "'";
        strError += tagVal;
        strError += "'";
    }
    System.err.println(strError);
}

/**
 * Convert this double to a string, optionally taking into account the hole
 * value.  Use "max"/"min" to write out Double.MAX_VALUE/Double.MIN_VALUE.
 * @param d The double to write out
 * @param hole The hole value
 * @param doHoles If true, write "hole" instead of the actual hole value
 * @return The double as a string
 */
public static String doubleToString(double d, double hole, boolean doHoles)
{
    String doubleString = null;
    if (doHoles && d == hole) {
        doubleString = "hole";
    }
    else if (d == Double.MAX_VALUE) {
        doubleString = "max";
    }
    else if (d == Double.MIN_VALUE) {
        doubleString = "min";
    }
    else {
        doubleString = String.valueOf(d);
    }
    return(doubleString);
}

/**
 * Write the string at the given tab value to the given output stream and/or
 * string buffer.
 * @param writer The OutputStreamWriter to write the output
 * @param sbuf The string buffer
 * @param s The string to write
 * @param tab The amount to tab over before writing
 * @throws IOException
 */
protected void writeString(OutputStreamWriter writer, StringBuffer sbuf,
                           String s, int tab)
    throws IOException
{
    if ((writer == null && sbuf == null) || s == null) {
        return;
    }

    StringBuffer sbTab = null;
    if (tab > 0) {
        sbTab = new StringBuffer();
        for (int i = 0; i < tab; i++) {
            sbTab.append(" ");
        }
    }

    if (writer != null) {
        if (sbTab != null) {
            writer.write(sbTab.toString());
        }
        writer.write(s);
    }
    if (sbuf != null) {
        if (sbTab != null) {
            sbuf.append(sbTab.toString());
        }
        sbuf.append(s);
    }
}

/**
 * Write the given image map attribute to a given output stream and/or
 * string buffer.
 * @param writer The OutputStreamWriter to write the output
 * @param sbuf The string buffer
 * @param attribute The name of the attribute
 * @param value The attributes current value
 * @param tabString The element string
 * @param tab The current element tab
 * @param tabAttribute The attribute tab
 * @return The new value of the attribute tab
 * @throws IOException
 */
protected int writeImageMapAttribute(OutputStreamWriter writer,
                                     StringBuffer sbuf, String attribute,
                                     String value,  String tabString, int tab,
                                     int tabAttribute)
    throws IOException
{
    if (value != null) {
        if (tabAttribute > 0) {
            writeString(writer, sbuf, "\n", 0);
        }
        writeString(writer, sbuf, " " + attribute + "=\"" +
                                  expandReservedCharacters(value) + "\"",
                    (tabAttribute > 0) ? tab + tabAttribute : 0);
        tabAttribute = (tabString == null) ? 0 : tabString.length();
    }
    return(tabAttribute);
}

/**
 * Write the data in XML format to a given output stream and/or string buffer.
 * @param data The data source to write out
 * @param writer The OutputStreamWriter to write the output
 * @param sbuf The string buffer
 * @param tab The initial tab
 * @param tabIncrement The tabbing increment between nested elements
 * @param writeHeader Write intial "xml" and "DOCTYPE" elements
 * @throws IOException
 */
protected void writeDataToXML(ChartDataModel data, OutputStreamWriter writer,
                              StringBuffer sbuf, int tab,
                              int tabIncrement, boolean writeHeader)
    throws IOException
{
    if (data == null || data.getNumSeries() <= 0) {
        return;
    }

    if (writer == null && sbuf == null) {
        return;
    }

    int numSeries = data.getNumSeries();
    String[] pointLabels = null;
    String[] seriesLabels = null;
    String name = null;
    if (data instanceof LabelledChartDataModel) {
        LabelledChartDataModel lData = (LabelledChartDataModel)data;
        pointLabels = lData.getPointLabels();
        seriesLabels = lData.getSeriesLabels();
        name = lData.getDataSourceName();
    }
    boolean cvtHoles = false;
    double holeValue = Double.MAX_VALUE;
    // Convert hole values to word 'hole' if we can find out what the
    // hole value is
    if (data instanceof HoleValueChartDataModel) {
        holeValue = ((HoleValueChartDataModel)data).getHoleValue();
        cvtHoles = true;
    }

    ImageMapInfo pointInfo[][] = new ImageMapInfo[numSeries][];
    ImageMapInfo clusterInfo[] = null;
    ImageMapInfo seriesInfo[]  = null;
    ImageMapInfo legendInfo[]  = null;
    if (data instanceof ImageMapChartDataModel) {
        ImageMapChartDataModel imageData = (ImageMapChartDataModel)data;
        for (int i = 0; i < numSeries; i++) {
            pointInfo[i] = imageData.getPointImageMapInfo(i);
        }
        clusterInfo = imageData.getClusterImageMapInfo();
        seriesInfo  = imageData.getSeriesImageMapInfo();
        legendInfo  = imageData.getLegendImageMapInfo();
    }

    double[][] xvalues = new double[numSeries][];
    double[][] yvalues = new double[numSeries][];
    for (int i = 0; i < numSeries; i++) {
        // Deal with x
        double[] temp = data.getXSeries(i);
        if (i == 0 || temp != xvalues[0]) {
            xvalues[i] = temp;
        }
        // Deal with y
        yvalues[i] = data.getYSeries(i);
    }
    if (xvalues.length > 1 && xvalues[1] == null) {
        double[] temp = xvalues[0];
        xvalues = new double[1][];
        xvalues[0] = temp;
    }

    tab          = Math.max(tab, 0);
    tabIncrement = Math.max(tabIncrement, 0);
    if (writeHeader) {
        writeString(writer, sbuf, "<?xml version=\"1.0\"?>\n", tab);
        writeString(writer, sbuf,
                    "<!DOCTYPE chart-data SYSTEM \"JCChartData.dtd\">\n", tab);
    }
    writeString(writer, sbuf, "<" + XML_DS, tab);
    if (name != null) {
        writeString(writer, sbuf, " " + XML_DS_ATT_NAME + "=\"" +
                    expandReservedCharacters(name) + "\"", 0);
    }
    if (cvtHoles && holeValue != Double.MAX_VALUE) {
        writeString(writer, sbuf, " " + XML_DS_ATT_HOLE + "=\"" +
                    doubleToString(holeValue, holeValue, false) + "\"", 0);
    }
    writeString(writer, sbuf, ">\n", 0);
    tab += tabIncrement;

    // Write point labels
    if (pointLabels != null && pointLabels.length > 0) {
        // Only write em out if there is at least one non-empty point label
        boolean pointLabelFound = false;
        for (int i = 0; i < pointLabels.length; i++) {
            if (pointLabels[i] != null && pointLabels[i].length() > 0) {
                pointLabelFound = true;
                break;
            }
        }
        if (pointLabelFound) {
            for (int j = 0; j < pointLabels.length; j++) {
                writeString(writer, sbuf, "<" + XML_DS_POINTLABEL + ">", tab);
                writeString(writer, sbuf,
                            expandReservedCharacters(pointLabels[j]), 0);
                writeString(writer, sbuf, "</" + XML_DS_POINTLABEL + ">\n", 0);
            }
        }
    }

    for (int i = 0; i < numSeries; i++) {
        String seriesTag = "<" + XML_DS_SERIES;
        writeString(writer, sbuf, seriesTag, tab);
        int infoTab = 0;
        ImageMapInfo info =
            (seriesInfo != null && i < seriesInfo.length) ? seriesInfo[i]
                                                          : null;
        if (info != null && !info.isEmpty()) {
            infoTab = writeImageMapAttribute(writer, sbuf, XML_DS_SERIES_URL,
                                             info.getUrl(), seriesTag,
                                             tab, infoTab);
            infoTab = writeImageMapAttribute(writer, sbuf, XML_DS_SERIES_EXTRA,
                                             info.getExtra(), seriesTag,
                                             tab, infoTab);
        }
        info = (legendInfo != null && i < legendInfo.length) ? legendInfo[i]
                                                             : null;
        if (info != null && !info.isEmpty()) {
            infoTab = writeImageMapAttribute(writer, sbuf, XML_DS_LEGEND_URL,
                                             info.getUrl(), seriesTag,
                                             tab, infoTab);
            infoTab = writeImageMapAttribute(writer, sbuf, XML_DS_LEGEND_EXTRA,
                                             info.getExtra(), seriesTag,
                                             tab, infoTab);
        }
        writeString(writer, sbuf, ">\n", 0);
        tab += tabIncrement;

        // If there is a series label, write it out
        if (seriesLabels != null    &&
            i < seriesLabels.length &&
            seriesLabels[i] != null)
        {
            writeString(writer, sbuf, "<" + XML_DS_SERIESLABEL + ">", tab);
            writeString(writer, sbuf,
                        expandReservedCharacters(seriesLabels[i]), 0);
            writeString(writer, sbuf, "</" + XML_DS_SERIESLABEL + ">\n", 0);
        }

        // X Data
        if (i == 0 || xvalues.length > 1) {
            for (int j = 0; j < xvalues[i].length; j++) {
                String xDataTag = "<" + XML_DS_XDATA;
                writeString(writer, sbuf, xDataTag, tab);
                infoTab = 0;
                if (i == 0 && clusterInfo != null && j < clusterInfo.length) {
                    info = clusterInfo[j];
                    if (info != null && !info.isEmpty()) {
                        infoTab = writeImageMapAttribute(writer, sbuf,
                                                         XML_DS_CLUSTER_URL,
                                                         info.getUrl(),
                                                         xDataTag, tab,
                                                         infoTab);
                        infoTab = writeImageMapAttribute(writer, sbuf,
                                                         XML_DS_CLUSTER_EXTRA,
                                                         info.getExtra(),
                                                         xDataTag, tab,
                                                         infoTab);
                    }
                }
                String newLine = (infoTab > 0)  ? "\n" : "";
                writeString(writer, sbuf, ">" + newLine, 0);
                writeString(writer, sbuf,
                            doubleToString(xvalues[i][j], holeValue, cvtHoles) +
                            newLine,
                            (infoTab > 0) ? tab : 0);
                writeString(writer, sbuf, "</" + XML_DS_XDATA + ">\n",
                            (infoTab > 0) ? tab : 0);
            }
        }

        // Y Data
        for (int j = 0; j < yvalues[i].length; j++) {
            String yDataTag = "<" + XML_DS_YDATA;
            writeString(writer, sbuf, yDataTag, tab);
            infoTab = 0;
            if (pointInfo[i] != null && j < pointInfo[i].length) {
                info = pointInfo[i][j];
                if (info != null && !info.isEmpty()) {
                    infoTab = writeImageMapAttribute(writer, sbuf,
                                                     XML_DS_POINT_URL,
                                                     info.getUrl(), yDataTag,
                                                     tab, infoTab);
                   infoTab = writeImageMapAttribute(writer, sbuf,
                                                     XML_DS_POINT_EXTRA,
                                                     info.getExtra(), yDataTag,
                                                     tab, infoTab);
                }
            }
            String newLine = (infoTab > 0) ? "\n" : "";
            writeString(writer, sbuf, ">" + newLine, 0);
            writeString(writer, sbuf,
                        doubleToString(yvalues[i][j], holeValue, cvtHoles) +
                        newLine,
                        (infoTab > 0) ? tab : 0);
            writeString(writer, sbuf, "</" + XML_DS_YDATA + ">\n",
                        (infoTab > 0) ? tab : 0);
        }

        tab -= tabIncrement;
        writeString(writer, sbuf, "</" + XML_DS_SERIES + ">\n", tab);
    }

    tab -= tabIncrement;
    writeString(writer, sbuf, "</" + XML_DS + ">\n", tab);
    if (writer != null) {
        writer.flush();
    }
}

/**
 * Write the data in XML format to a string.
 * @param data The data source to write out
 * @param tab The intial tab
 * @param tabIncrement The tabbing increment between nested elements
 * @param writeHeader Write intial "xml" and "DOCTYPE" elements
 * @return The data in XML format as a string
 */
public String writeDataToXML(ChartDataModel data, int tab, int tabIncrement,
                             boolean writeHeader)
{
    StringBuffer sbuf = new StringBuffer();
    try {
        writeDataToXML(data, null, sbuf, tab, tabIncrement, writeHeader);
    }
    catch (Exception e) {
        e.printStackTrace();
    }
    return(sbuf.toString());
}

/**
 * Write the data in XML format using a given OutputStreamWriter.
 * @param data The data source to write out
 * @param writer The OutputStreamWriter to write the output
 * @param tab The initial tab
 * @param tabIncrement The tabbing increment between nested elements
 * @param writeHeader Write intial "xml" and "DOCTYPE" elements
 * @throws IOException
 */
public void writeDataToXML(ChartDataModel data, OutputStreamWriter writer,
                           int tab, int tabIncrement, boolean writeHeader)
    throws IOException
{
    if (data == null || data.getNumSeries() <= 0 || writer == null) {
        return;
    }

    writeDataToXML(data, writer, null, tab, tabIncrement, writeHeader);
}

/**
 * Write the data to an XML file in the format expected by JClass Chart.
 * @param data The data source to write out
 * @param fileName The file in which to write the output
 * @param tab The intial tab
 * @param tabIncrement The tabbing increment between nested elements
 * @throws IOException
 */
public void writeDataToXML(ChartDataModel data, String fileName, int tab,
                           int tabIncrement)
    throws IOException
{
    if (fileName == null) {
        return;
    }

    FileOutputStream fos = new FileOutputStream(fileName);
    writeDataToXML(data, fos, tab, tabIncrement, true);
    fos.close();
}

/**
 * Write the data in XML format to a given output stream.
 * @param data The data source to write out
 * @param os The output stream to write the output
 * @param tab The initial tab
 * @param tabIncrement The tabbing increment between nested elements
 * @param writeHeader Write intial "xml" and "DOCTYPE" elements
 * @throws IOException
 */
public void writeDataToXML(ChartDataModel data, OutputStream os, int tab,
                           int tabIncrement, boolean writeHeader)
    throws IOException
{
    if (data == null || data.getNumSeries() <= 0 || os == null) {
        return;
    }

    DataOutputStream out = null;
    if (os instanceof DataOutputStream) {
        out = (DataOutputStream)os;
    }
    else {
        out = new DataOutputStream(os);
    }
    OutputStreamWriter writer = new OutputStreamWriter(out);
    writeDataToXML(data, writer, null, tab, tabIncrement, writeHeader);
}

/**
 * Write the data in XML format to a given string buffer
 * @param data The data source to write out
 * @param sbuf The string buffer to write the output
 * @param tab The initial tab
 * @param tabIncrement The tabbing increment between nested elements
 * @param writeHeader Write intial "xml" and "DOCTYPE" elements
 */
public void writeDataToXML(ChartDataModel data, StringBuffer sbuf, int tab,
                           int tabIncrement, boolean writeHeader)
{
    if (data == null || data.getNumSeries() <= 0 || sbuf == null) {
        return;
    }

    try {
        writeDataToXML(data, null, sbuf, tab, tabIncrement, writeHeader);
    }
    catch (Exception e) {
        e.printStackTrace();
    }
}

/**
 * Expand reserved characters in xml strings to their entity references.
 * @param s The string to expand
 * @return The newly expanded string
 */
public static String expandReservedCharacters(String s)
{
    if (s == null) {
        return(null);
    }

    // Make sure that ampersand's are done first
    String[] reserved = { "&", "<", ">", "'", "\"" };
    String[] replace  = { "&amp;", "&lt;", "&gt;", "&apos;", "&quot;" };

    boolean found = false;
    for (int i = 0; i < reserved.length; i++) {
        if (s.indexOf(reserved[i]) >= 0) {
            found = true;
        }
    }

    StringBuffer sbuf = null;
    if (found) {
        sbuf = new StringBuffer(s);
        for (int i = 0; i < reserved.length; i++) {
            // We need the temp string here because prior to JDK 1.4,
            // StringBuffer does not have the indexOf methods.
            String temp = sbuf.toString();
            int index = temp.indexOf(reserved[i]);
            while (index >= 0) {
                sbuf.replace(index, index + 1, replace[i]);
                temp = sbuf.toString();
                index = temp.indexOf(reserved[i], index + 1);
            }
        }
    }

    if (sbuf == null) {
        sbuf = new StringBuffer(s);
    }
    // Replace unprintable characters with their unicode equivalents
    for (int i = sbuf.length() - 1; i >= 0; i--) {
        int c = (int)sbuf.charAt(i);
        // Check if unprintable character
        if (c < lowCharacterBound || c > highCharacterBound) {
            String hex = Integer.toHexString(c);
            StringBuffer sbufNew = new StringBuffer("&#x");
            for (int j = 4; j > hex.length(); j--) {
                sbufNew.append("0");
            }
            sbufNew.append(hex);
            sbufNew.append(";");
            sbuf.replace(i, i + 1, sbufNew.toString());
        }
    }
    return(sbuf.toString());
}

}


