/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataBindingConfigEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:52 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.db;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.FontMetrics;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyEditor;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.klg.jclass.chart.JCChartBundle;

/**
 * This is the DataBindingConfigEditor
 * used in JBuilder. However, different
 * custom editors are used for each IDE.
 */
public class DataBindingConfigEditor extends JPanel implements PropertyEditor,
	ActionListener, ItemListener, ListSelectionListener, PropertyChangeListener,
    FocusListener, java.io.Serializable{

public PropertyChangeSupport support = new PropertyChangeSupport(this);

private JTextField startPoint = null;
private JTextField endPoint = null;
private JCheckBox allRows = null;
private JList seriesColumns = null;
protected JComboBox labelsColumn;

protected String[] allColumns = new String[0];
protected int[] numericalColumns = new int[0];

protected boolean updateComponents = true;

/**
 * Constructor for the custom editor used to represent a DataBindingMetaData
 */
public DataBindingConfigEditor() {
	super();
	setLayout(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();
    c.fill = GridBagConstraints.BOTH;
	c.insets = new Insets(2,2,2,2);
	c.anchor = GridBagConstraints.NORTHWEST;
	c.gridheight = 1;
	c.gridwidth = 1;
	c.gridx = 0;
	c.gridy = 0;

	// Data range panel
	GroupPanel dataRange = new GroupPanel("Data Range");
//	dataRange.setBorder(BorderFactory.createTitledBorder(JCChartBundle.string(JCChartBundle.key180)));
	dataRange.add(new JLabel("Start row"));
//	dataRange.add(new JLabel(JCChartBundle.string(JCChartBundle.key176) + ":"));
	startPoint = new JTextField(Integer.toString(0), 5);
	dataRange.add(startPoint);
	dataRange.add(new JLabel("End row"));
//	dataRange.add(new JLabel(JCChartBundle.string(JCChartBundle.key177) + ":"));
	endPoint = new JTextField(Integer.toString(100), 5);
	allRows = new JCheckBox("All rows", false);
//	allRows = new JCheckBox(JCChartBundle.string(JCChartBundle.key298),dbMetaData.getAllRows());
	startPoint.setEnabled(!allRows.isSelected());
	endPoint.setEnabled(!allRows.isSelected());
	dataRange.add(endPoint);
	dataRange.add(allRows);

	// Point labels column panel
	GroupPanel pointLabelsPanel = new GroupPanel("Potential point labels column");
	labelsColumn = new JComboBox();
//	panel.setBorder(BorderFactory.createTitledBorder(JCChartBundle.string(JCChartBundle.key182)));
	pointLabelsPanel.add(new JLabel("Column name"));
	pointLabelsPanel.add(new JScrollPane(labelsColumn), BorderLayout.CENTER);

	// Columns panel
	GroupPanel columnsPanel = new GroupPanel("Potential series columns");
	seriesColumns = new JList();
	seriesColumns.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
//	panel.setBorder(BorderFactory.createTitledBorder(JCChartBundle.string(JCChartBundle.key181)));
	columnsPanel.add(new JScrollPane(seriesColumns), BorderLayout.CENTER);

	add(dataRange, c);
	c.gridy++;
	add(pointLabelsPanel, c);
	c.gridy++;
	add(columnsPanel, c);

	startPoint.addActionListener(this);
	startPoint.addFocusListener(this);
	endPoint.addActionListener(this);
	endPoint.addFocusListener(this);
	allRows.addActionListener(this);
	labelsColumn.addItemListener(this);
	seriesColumns.addListSelectionListener(this);
}

/**
 * Set (or change) the object that is to be edited.
 */
public void setValue(Object value) {
	if (value == null || !(value instanceof DataBindingConfigWrapper)) {
    	return;
    }

    DataBindingConfigWrapper wrapper = (DataBindingConfigWrapper)value;
    allColumns = wrapper.allColumns;
    if (allColumns == null) {
    	allColumns = new String[0];
    }
    numericalColumns = wrapper.numericalColumns;
    if (numericalColumns == null) {
    	numericalColumns = new int[0];
    }

	synchronized(this) {
        // Disable event handling in components listeners
        updateComponents = false;

        allRows.setSelected(wrapper.allRows);
        startPoint.setText(wrapper.startPoint+"");
        endPoint.setText(wrapper.endPoint+"");

        if (labelsColumn.getItemCount() > 0) {
	        labelsColumn.removeAllItems();
        }
        for (int i=0; i<allColumns.length; i++) {
            labelsColumn.addItem(allColumns[i]);
        }
//		System.out.println("setValue(), wrapper.labelsColumn="+wrapper.labelsColumn);
        if (findColumnIndex(allColumns, wrapper.labelsColumn) != -1) {
            labelsColumn.setSelectedItem(wrapper.labelsColumn);
//			System.out.println("setValue(), getSelectedItem()="+labelsColumn.getSelectedItem());
        }
        String[] numNames = numericalColumnNames();
        seriesColumns.setListData(numNames);
        int[] indices = columnsIndices(allColumns, numNames, wrapper.seriesColumns);
        if (indices != null) {
            seriesColumns.setSelectedIndices(indices);
        }

        // Enable event handling in components listeners
        updateComponents = true;
    }
}

/**
 * The value of the property
 */
public Object getValue() {
    Object[] selectedValues = seriesColumns.getSelectedValues();
    String[] selectedColumns = new String[selectedValues.length];
    for (int i=0; i<selectedValues.length; i++) {
        selectedColumns[i] = (String)selectedValues[i];
    }
	return new DataBindingConfigWrapper(
        allRows.isSelected(),
        Integer.valueOf(startPoint.getText()).intValue(),
        Integer.valueOf(endPoint.getText()).intValue(),
        (String)labelsColumn.getSelectedItem(),
        selectedColumns,
        numericalColumns, allColumns);
}

public static int findColumnIndex(String[] names, String column) {
	for (int i=0; i<names.length; i++) {
        if (column.equalsIgnoreCase(names[i])) {
        	return i;
        }
	}
    return -1;
}

protected boolean isNumerical(int index) {
	for (int i=0; i<numericalColumns.length; i++) {
        if (index == numericalColumns[i]) {
        	return true;
        }
	}
    return false;
}

protected String[] numericalColumnNames() {
	String[] names = new String[numericalColumns.length];
	for (int i=0; i<numericalColumns.length; i++) {
        names[i] = allColumns[numericalColumns[i]];
	}
    return names;
}

protected int[] columnsIndices(String[] allNames,
	String[] numNames, String[] names)
{
	int[] indices = new int[names.length];
	for (int i=0; i<names.length; i++) {
    	int index = findColumnIndex(allNames, names[i]);
        if (!isNumerical(index)) {
        	return null;
        }
        indices[i] = findColumnIndex(numNames, names[i]);
	}
    return indices;
}

/**
 * True if the class will honor the paintValue method
 */
public boolean isPaintable() {
	return true;
}

/**
 * Set the property value by parsing a given String.
 * May raise java.lang.IllegalArgumentException if either
 * the String is badly formatted or if this kind of property
 * can't be expressed as text.
 */
public void setAsText(String text) throws IllegalArgumentException {
}

/**
 * The property value as a human editable string
 */
public String getAsText() {
	return null;
}

/**
 * If the property value must be one of a set of known
 * tagged values, then this method should return an array of the tags
 */
public String[] getTags() {
	return null;
}

/**
 * True if the propertyEditor can provide a custom editor
 */
public Component getCustomEditor() {
	return this;
}

public boolean supportsCustomEditor() {
	return true;
}

/**
 * A fragment of Java code representing an initializer for the current value
 */
public String getJavaInitializationString() {
	return ((DataBindingConfigWrapper)getValue()).getJavaInitializationString();
}

/**
 * Paint a representation of the value into a given area of screen real estate.
 */
public void paintValue(Graphics gc, Rectangle box) {
	String s = JCChartBundle.string(JCChartBundle.key163);
	if (s == null) {
		s = new String("");
	}
	FontMetrics fm = gc.getFontMetrics();
	int vpad = (box.height - fm.getAscent())/2;
	gc.drawString(s, box.x, box.y+box.height-vpad);
}

/**
 * This method gets called when a bound property is changed.
 * @param e a PropertyChangeEvent object describing the event
 * source and the property that has changed.
 */
public void propertyChange(PropertyChangeEvent e) {
	firePropertyChange();
}

protected void firePropertyChange() {
//	System.out.println("firePropertyChange()");
	support.firePropertyChange("dataBindingConfig", null, getValue());
}

/**
 * Register a listener for the PropertyChange event.
 */
public void addPropertyChangeListener(PropertyChangeListener l) {
	support.addPropertyChangeListener(l);
}

/**
 * Remove a listener for the PropertyChange event
 */
public void removePropertyChangeListener(PropertyChangeListener l) {
	support.removePropertyChangeListener(l);
}

//================== ActionListener interface methods ==================

public void actionPerformed(ActionEvent e) {
	if (!updateComponents) {
		return;
    }
    Object source = e.getSource();
    if (startPoint.equals(source) || endPoint.equals(source)) {
        firePropertyChange();
	}
    else if (allRows.equals(source)) {
    	boolean enabled = !allRows.isSelected();
        startPoint.setEnabled(enabled);
        endPoint.setEnabled(enabled);
        firePropertyChange();
	}
}

//================== FocusListener interface methods ==================

public void focusGained(FocusEvent ev)
{
}

public void focusLost(FocusEvent ev)
{
	if (!updateComponents) {
		return;
    }
    Object source = ev.getSource();
    if (startPoint.equals(source) || endPoint.equals(source)) {
        firePropertyChange();
	}
}

//================== ItemListener interface methods ==================

public void itemStateChanged(ItemEvent e) {
	if (!updateComponents) {
		return;
    }
    if (labelsColumn.equals(e.getSource())) {
        firePropertyChange();
	}
}

//================== ListSelectionListener interface methods ==================

public void valueChanged(ListSelectionEvent e) {
	if (!updateComponents) {
		return;
    }
    if (seriesColumns.equals(e.getSource())) {
        firePropertyChange();
    }
}

protected class GroupPanel extends JPanel {

GroupPanel(String title) {
	super();
	setBorder(BorderFactory.createTitledBorder(title));
}

} // GroupPanel


}


