/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataBindingConfigWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:52 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.db;

/**
 * DataBindingConfigWrapper wraps up the data binding configuration information.
 * It is used for the dataBindingConfig property in DataChart
 */
public class DataBindingConfigWrapper implements java.io.Serializable {

protected boolean allRows = false;
protected int startPoint = 0;
protected int endPoint = 0;

/** Point labels column */
protected String labelsColumn = null;

/** Data column */
protected String[] seriesColumns = null;

protected int[] numericalColumns = null;
protected String[] allColumns = null;

/**
 * Creates an empty DataBindingConfigWrapper.
 */
public DataBindingConfigWrapper()
{
	allRows = false;
	startPoint = 0;
	endPoint = 0;
	labelsColumn = null;
	seriesColumns = new String[0];
}

/**
 * Creates an DataBindingConfigWrapper given the integers representing the
 * configuration information for data binding.
 * @param allRows defines whether to grab all rows
 * @param startPoint is the starting point row number
 * @param endPoint is the ending point row number
 * @param specifies point labels column
 * @param specifies data column
 */
public DataBindingConfigWrapper(boolean allRows, int startPoint, int endPoint,
	String labelsColumn, String[] seriesColumns)
{
	this.allRows = allRows;
	this.startPoint = startPoint;
	this.endPoint = endPoint;
	this.labelsColumn = labelsColumn;
	this.seriesColumns = seriesColumns;
}

/**
 * Creates an DataBindingConfigWrapper given the integers representing the
 * configuration information for data binding.
 * @param allRows defines whether to grab all rows
 * @param startPoint is the starting point row number
 * @param endPoint is the ending point row number
 * @param specifies point labels column
 * @param specifies data column
 */
public DataBindingConfigWrapper(boolean allRows, int startPoint, int endPoint,
	String labelsColumn, String[] seriesColumns,
    int[] numericalColumns, String[] allColumns)
{
	this.allRows = allRows;
	this.startPoint = startPoint;
	this.endPoint = endPoint;
	this.labelsColumn = labelsColumn;
    this.seriesColumns = seriesColumns;
    this.numericalColumns = numericalColumns;
    this.allColumns = allColumns;
}

public String toString() {
	return allRows+", "+startPoint+", "+endPoint+ ", "+labelsColumn+", "+
    	"("+makeArrayString(seriesColumns)+")";
}

public String getJavaInitializationString() {
	// Use "\n" as a delimiter
	return "new "+getClass().getName()+"(\n        "+
    	allRows+", "+startPoint+", "+endPoint+", \""+
        labelsColumn+"\",\n        "+
    	"new String[] {"+makeArrayString(seriesColumns)+"})";
}

protected String makeArrayString(String[] stringArray) {
	String s = "";
    for (int i=0; i<stringArray.length; i++) {
		s += "\""+stringArray[i]+"\""+(i<stringArray.length-1 ? "," : "");
    }
    return s;
}

}	// DataBindingConfigWrapper

