/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DSdbChart.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:52 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.db.datasource;

import com.klg.jclass.chart.db.DataChart;
import com.klg.jclass.datasource.DataModel;

/**
  * <code>DSdbChart</code> is a derived class that enables JClass Chart to
  * bind to a database at design-time using the JClass DataSource.
  * <P>This class differs from JCChart in the following ways:
  * <OL>
  * <LI><code>DSdbChart</code> exposes properties that are actually defined
  * in JCChart's sub-objects so that IDE's can access them.
  * <LI><code>DSdbChart</code> duplicates enums that are actually defined
  * in JCChart's sub-objects.
  * </OL>
  * <P>The following table lists the properties this class can set/get.
  * <P><STRONG>Properties</STRONG></P>
  * <P></P>
  * <TABLE BORDER=1 WIDTH=100%>
  * <TR>
  * <TH>Name</TH>
  * <TH>Description</TH></TR>
  * <TR>
  * <TD>AxisOrientation
  * </TD>
  * <TD>
  * The <code>AxisOrientation</code> property determines if
  * the <i>x</i> and <i>y</i> axes are inverted and reversed.
  * </TD></TR>
  * <TD>Background
  * </TD>
  * <TD>The <code>Background</code> property determines
  * the background color used to draw
  * inside the chart region.  Note that the Background property is inherited
  * from the parent <code>JCComponent</code>.
  * </TD></TR>
  * <TR>
  * <TD>ChartType
  * </TD>
  * <TD>
  * The <code>ChartType</code> property determines the chart type of the
  * first set of data in the chart.
  * </TD></TR>
  * <TR>
  * <TD>DataBinding
  * </TD>
  * <TD>
  * The <code>DataBinding</code> property specifies a string,
  * describing a <code>JCTreeData</code> instance and
  * metadata level of the instance this chart is bound with. The format of the property
  * is the following:
  * <br>
  * &lt;JCTreeData name&gt;:&lt;MetaDataModel description&gt;[|&lt;MetaDataModel description&gt;...]
  * </TD></TR>
  * <TR>
  * <TD>DataBindingMetaData
  * </TD>
  * <TD>
  * The <code>DataBindingMetaData</code> property provides information necessary for
  * <code>DSdbChart</code> to display data from a database, including the columns and
  * range of data to be used. For more information, see,
  * <A HREF="com.klg.jclass.chart.db.DataBindingMetaData.html#_top_">
  * com.klg.jclass.chart.db.DataBindingMetaData</A>
  * </TD></TR>
  * <TR>
  * <TD>Font
  * </TD>
  * <TD>The <code>Font</code> property determines
  * what font is used to render text inside
  * the chart region.  Note that the <code>Font</code> property is inherited from the
  * parent <code>JCComponent</code>.
  * </TD></TR>
  * <TR>
  * <TD>FooterText
  * </TD>
  * <TD>
  * The <code>FooterText</code> property holds the text that is displayed
  * in the footer. The default value is " " (empty string).
  * </TD></TR>
  * <TR>
  * <TD>Foreground
  * </TD>
  * <TD>The <code>Foreground</code> property determines
  * the foreground color used to draw
  * inside the chart region.  Note that the <code>Foreground</code>
  * property is inherited from the parent <code>JCComponent</code>.
  * </TD></TR>
  * <TR>
  * <TD>HeaderText
  * </TD>
  * <TD>
  * The <code>HeaderText</code> property holds the text that is displayed
  * in the header. The default value is " " (empty string).
  * </TD></TR>
  * <TR>
  * <TD>Legend
  * </TD>
  * <TD>
  * The <code>Legend</code> property controls the object that controls
  * the display of the legend. The default value is null.
  * </TD></TR>
  * <TR>
  * <TD>LegendAnchor
  * </TD>
  * <TD>
  * The <code>LegendAnchor</code> property determines the position of the
  * legend relative to the ChartArea.  Valid values include <code>NORTH</code>,
  * <code>SOUTH</code>, <code>EAST</code>, <code>WEST</code>,  <code>NORTHWEST</code>,
  * <code>SOUTHWEST</code>, <code>NORTHEAST</code> and <code>SOUTHEAST</code>.
  * The default value is <code>EAST</code>.
  * </TD></TR>
  * <TR>
  * <TD>LegendVisible
  * </TD>
  * <TD>
  * The <code>LegendVisible</code> property determines whether the
  * legend is visible. The default value is <code>false</code>.
  * </TD></TR>
  * <TR>
  * <TD>LegendOrientation
  * </TD>
  * <TD>
  * The <code>LegendOrientation</code> property determines how legend
  * information is laid out. Valid values include <code>VERTICAL</code> and
  * <code>HORIZONTAL</code>. The default value is <code>VERTICAL</code>.
  * </TD></TR>
  * <TR>
  * <TD>View3D
  * </TD>
  * <TD>
  * The <TT>View3D</TT> property combines the values of the <TT>Depth</TT>,
  * <TT>Elevation</TT>, and <TT>Rotation</TT> properties defined in <TT>JCChartArea</TT>.
  * <TT>Depth</TT> controls the apparent depth of a graph.
  * <TT>Elevation</TT> controls the distance above the X-axis
  * for the 3D effect. <TT>Rotation</TT> controls the position of the
  * eye relative to the Y-axis for the 3D effect.  The default value is "0.0, 0.0, 0.0".
  * </TD></TR>
  * <TR>
  * <TD>Version
  * </TD>
  * <TD>
  * The <code>Version</code> property specifies the JClass Chart version number.
  * This property is read-only. The default value is calculated.
  * </TD></TR>
  * <TR>
  * <TD>XAxisAnnotationMethod
  * </TD>
  * <TD>
  * The <code>XAxisAnnotationMethod</code> property determines how axis annotations
  * are generated.  Valid values include <code>VALUE</code> (annotation is generated
  * by Chart, with possible callbacks to a label generator);
  * <code>VALUE_LABELS</code> (annotation is taken from a list of value
  * labels provided by the user -- a value label is a label that appears
  * at a particular axis value);  <code>POINT_LABELS</code>
  * (annotation comes from the data source's point labels
  * that are associated with particular data points);  and
  * <code>TIME_LABELS</code> (Chart generates time/date labels based on the
  * TimeUnit, TimeBase and TimeFormat properties). The default value is <code>VALUE</code>.
  * </TD></TR>
  * <TR>
  * <TD>XAxisGridVisible
  * </TD>
  * <TD>
  * The <code>XAxisGridVisible</code> property determines whether a grid is drawn
  * for the axis. The default value is <code>false</code>.
  * </TD></TR>
  * <TR>
  * <TD>XAxisLogarithmic
  * </TD>
  * <TD>
  * The <code>XAxisLogarithmic</code> property determines whether the
  * first <i>x</i> axis will be logarithmic (<code>true</code>)
  * or linear (<code>false</code>). The default value is false.
  * </TD></TR>
  * <TR>
  * <TD>XAxisVisible
  * </TD>
  * <TD>
  * The <code>XAxisVisible</code> property determines whether the
  * first <i>x</i> axis is currently visible. The default value is <code>true</code>.
  * </TD></TR>
  * <TR>
  * <TD>XAxisMinMax
  * </TD>
  * <TD>
  * The <code>XAxisMinMax</code> controls both the
  * <code>XAxisMin</code> and <code>XAxisMax</code> properties.
  * The <code>XAxisMin</code> property controls the minimum value shown on the axis.
  * If a null string is used, Chart will calculate the axis min.
  * The data min is determined by Chart. The default value is calculated.
  * The <code>XAxisMax</code> property controls the maximum value shown on the axis.
  * If a null string is used, Chart will calculate the axis max.
  * The data max is determined by Chart. The default value is calculated.
  * </TD></TR>
  * <TR>
  * <TD>XAxisNumSpacing
  * </TD>
  * <TD>
  * The <code>XAxisNumSpacing</code> property controls the interval between
  * axis labels.
  * If a null string is used, Chart will calculate the interval.
  * The default value is calculated.
  * </TD></TR>
  * <TR>
  * <TD>XAxisTitleText
  * </TD>
  * <TD>
  * The <code>XAxisTitleText</code> property specifies the text that
  * will appear as the x axis title. The default value is " " (empty string).
  * </TD></TR>
  * <TR>
  * <TD>YAxisAnnotationMethod
  * </TD>
  * <TD>
  * The <code>YAxisAnnotationMethod</code> property determines how axis annotations
  * are generated.  Valid values include <code>VALUE</code> (annotation is generated
  * by Chart, with possible callbacks to a label generator);
  * <code>VALUE_LABELS</code> (annotation is taken from a list of value
  * labels provided by the user -- a value label is a label that appears
  * at a particular axis value);  <code>POINT_LABELS</code>
  * (annotation comes from the data source's point labels
  * that are associated with particular data points);  and
  * <code>TIME_LABELS</code> (Chart generates time/date labels based on the
  * TimeUnit, TimeBase and TimeFormat properties). The default value is <code>VALUE</code>.
  * </TD></TR>
  * <TR>
  * <TD>YAxisGridVisible
  * </TD>
  * <TD>
  * The <code>YAxisGridVisible</code> property determines whether a grid is drawn for the axis.
  * </TD></TR>
  * <TR>
  * <TD>YAxisLogarithmic
  * </TD>
  * <TD>
  * The <code>YAxisLogarithmic</code> property determines whether the
  * first <TT>y</TT> axis will be logarithmic
  * (<code>true</code>) or linear (<code>false</code>).
  * The default value is <code>false</code>.
  * </TD></TR>
  * <TR>
  * <TD>YAxisVisible
  * </TD>
  * <TD>
  * The <code>YAxisVisible</code> property determines whether the
  * first <i>y</i> axis is currently visible. The default value is <code>true</code>.
  * </TD></TR>
  * <TR>
  * <TD>YAxisMinMax
  * </TD>
  * <TD>
  * The <code>YAxisMinMax</code> controls both the <code>YAxisMin</code>
  * and <code>YAxisMax</code> properties.
  * The <code>YAxisMin</code> property controls the minimum value shown on the axis.
  * If a null string is used, Chart will calculate the axis min.
  * The data min is determined by Chart. The default value is calculated.
  * The <code>YAxisMax</code> property controls the maximum value shown on the axis.
  * If a null string is used, Chart will calculate the axis max.
  * The data max is determined by Chart. The default value is calculated.
  * </TD></TR>
  * <TR>
  * <TD>YAxisNumSpacing
  * </TD>
  * <TD>
  * The <code>YAxisNumSpacing</code> property controls the interval between
  * axis labels.
  * If a null string is used, Chart will calculate the interval.
  * The default value is calculated.
  * </TD></TR>
  * <TR>
  * <TD>YAxisTitleText
  * </TD>
  * <TD>
  * The <code>YAxisTitleText</code> property specifies the text that
  * will appear as the Y axis title. The default value is " " (empty string).
  * </TD></TR>
  * </TABLE>
 */

public class DSdbChart extends DataChart {

/**
 * Simple bean constructor. If no data set for this bean
 * then the sample default datasource defined in JCDefaultDataSource
 * will be used
 */
public DSdbChart() {
	super();
}

/**
 * Returns the chart's value of a <code>DataBinding</code> object property.
 * @see #setDataBinding
 *
 * The <code>DataBinding</code> property specifies a string
 * describing a <code>JCTreeData</code> instance and
 * metadata level of the instance this chart is bound with. The format of the property
 * is the following:
 * <pre>
 *   <JCTreeData name>:<MetaDataModel description>[|<MetaDataModel description>...]
 * </pre>
 * @see com.klg.jclass.datasource.TreeData
 * @see com.klg.jclass.datasource.MetaDataModel
 */
public String getDataBinding() {
	return getDataBindingString();
}

/**
 * Sets the <code>DataBinding</code> property value to the chart.
 * @see #getDataBinding
 */
public void setDataBinding( String dataBinding ) {
    super.setDataSource(dataBinding);
}

/**
 * Sets the data source to a <code>DataModel</code> and binds the MetaData to
 * the <code>MetaDataModel</code> represented by a "path" of <code>MetaDataModel</code>
 * descriptions separated by '|' (e.g. Orders|Customers).
 */
public void setDataSource(DataModel dataModel, String metaDescription) {
    super.setDataSource(dataModel, metaDescription);
}

}
