/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JBdbChart.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:12:52 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.db.jbuilder;

import com.borland.dx.dataset.DataSet;
import com.borland.dx.dataset.DataSetAware;

import com.klg.jclass.datasource.MetaDataModel;
import com.klg.jclass.datasource.jbuilder.JBuilderDataBinding;
import com.klg.jclass.chart.ChartDataModel;
import com.klg.jclass.chart.db.DataChart;

/**
  * JBdbChart is a derived class that enables JClass Chart to
  * bind to a database at design-time in Borland JBuilder 2.0.
  * <P>This class differs from JCChart in the following ways:
  * <OL>
  * <LI>JBdbChart exposes properties that are actually defined 
  * in JCChart's sub-objects so that JBuilder can access them.
  * <LI>JBdbChart duplicates enums that are actually defined 
  * in JCChart's sub-objects.
  * </OL>
  * <P>The following table lists the properties this class can set/get.
  * <P><STRONG>Properties</STRONG></P>
  * <P></P>
  * <TABLE BORDER=1 WIDTH=100%>
  * <TR>
  * <TH>Name</TH>
  * <TH>Description</TH></TR>
  * <TR>
  * <TD>AxisOrientation
  * </TD>
  * <TD>
  * The AxisOrientation property determines if the x and y axis
  * are inverted and reversed.
  * </TD></TR>
  * <TD>Background
  * </TD>
  * <TD>The Background property determines the background color used to draw
  * inside the chart region.  Note that the Background property is inherited 
  * from the parent JCComponent.
  * </TD></TR>
  * <TR>
  * <TD>ChartType
  * </TD>
  * <TD>
  * The ChartType property determines the chart type of the
  * first set of data in the chart.
  * </TD></TR>
  * <TR>
  * <TD>DataBindingMetaData
  * </TD>
  * <TD>
  * The DataBindingMetaData property provides information necessary for 
  * JBdbChart to display data from a database, including the columns and 
  * range of data to be used. For more information, see, 
  * <A HREF="com.klg.jclass.chart.db.DataBindingMetaData.html#_top_">com.klg.jclass.chart.db.DataBindingMetaData</A>
  * </TD></TR>
  * <TR>
  * <TD>DataSet
  * </TD>
  * <TD>
  * The DataSet property specifies the JBuilder DataSet instance being used for data binding.
  * </TD></TR>  
  * <TR>
  * <TD>Font
  * </TD>
  * <TD>The Font property determines what font is used to render text inside
  * the chart region.  Note that the Font property is inherited from the 
  * parent JCComponent.
  * </TD></TR>
  * <TR>
  * <TD>FooterText
  * </TD>
  * <TD>
  * The FooterText property holds the text that is displayed
  * in the footer. The default value is " " (empty string).
  * </TD></TR>
  * <TR>
  * <TD>Foreground
  * </TD>
  * <TD>The Foreground property determines the foreground color used to draw
  * inside the chart region.  Note that the Foreground property is inherited 
  * from the parent JCComponent.
  * </TD></TR>
  * <TR>
  * <TD>HeaderText
  * </TD>
  * <TD>
  * The HeaderText property holds the text that is displayed
  * in the header. The default value is " " (empty string).
  * </TD></TR>
  * <TR>
  * <TD>Legend
  * </TD>
  * <TD>
  * The Legend property controls the object that controls
  * the display of the legend. The default value is null.
  * </TD></TR>
  * <TR>
  * <TD>LegendAnchor
  * </TD>
  * <TD>
  * The LegendAnchor property determines the position of the 
  * legend relative to the ChartArea.  Valid values include NORTH, 
  * SOUTH, EAST, WEST,  NORTHWEST, SOUTHWEST, NORTHEAST and SOUTHEAST. The default value is EAST.
  * </TD></TR>
  * <TR>
  * <TD>LegendVisible
  * </TD>
  * <TD>
  * The LegendVisible property determines whether the
  * legend is visible. The default value is false.
  * </TD></TR>
  * <TR>
  * <TD>LegendOrientation
  * </TD>
  * <TD>
  * The LegendOrientation property determines how legend 
  * information is laid out. Valid values include VERTICAL and  
  * HORIZONTAL. The default value is VERTICAL.
  * </TD></TR>
  * <TR>
  * <TD>View3D
  * </TD>
  * <TD>
  * The <TT>View3D</TT> property combines the values of the <TT>Depth</TT>, 
  * <TT>Elevation</TT>, and <TT>Rotation</TT> properties defined in <TT>JCChartArea</TT>. 
  * <TT>Depth</TT> controls the apparent depth of a graph. 
  * <TT>Elevation</TT> controls the distance above the X-axis
  * for the 3D effect. <TT>Rotation</TT> controls the position of the
  * eye relative to the Y-axis for the 3D effect.  The default value is "0.0,0.0,0.0".
  * </TD></TR>
  * <TR>
  * <TD>Version
  * </TD>
  * <TD>
  * The Version property specifies the JClass Chart version number. This property is read-only. The default value is calculated.
  * </TD></TR>
  * <TR>
  * <TD>XAxisAnnotationMethod
  * </TD>
  * <TD>
  * The XAxisAnnotationMethod property determines how axis annotations 
  * are generated.  Valid values include VALUE (annotation is generated
  * by Chart, with possible callbacks to a label generator);
  * VALUE_LABELS (annotation is taken from a list of value
  * labels provided by the user -- a value label is a label that appears
  * at a particular axis value);  POINT_LABELS 
  * (annotation comes from the data source's point labels 
  * that are associated with particular data points);  and 
  * TIME_LABELS (Chart generates time/date labels based on the 
  * TimeUnit, TimeBase and TimeFormat properties). The default value is VALUE.
  * </TD></TR>
  * <TR>
  * <TD>XAxisGridVisible
  * </TD>
  * <TD>
  * The XAxisGridVisible property determines whether a grid is drawn
  * for the axis. The default value is false.
  * </TD></TR>
  * <TR>
  * <TD>XAxisLogarithmic
  * </TD>
  * <TD>
  * The XAxisLogarithmic property determines whether the
  * first x axis will be logarithmic (true) or linear (false). The default value is false.
  * </TD></TR>
  * <TR>
  * <TD>XAxisVisible
  * </TD>
  * <TD>
  * The XAxisVisible property determines whether the
  * first x axis is currently visible. The default value is true.
  * </TD></TR>
  * <TR>
  * <TD>XAxisMinMax
  * </TD>
  * <TD>
  * The XAxisMinMax controls both the XAxisMin and XAxisMax properties.
  * The XAxisMin property controls the minimum value shown on the axis. 
  * If a null string is used, Chart will calculate the axis min.
  * The data min is determined by Chart. The default value is calculated.
  * The XAxisMax property controls the maximum value shown on the axis.
  * If a null string is used, Chart will calculate the axis max.
  * The data max is determined by Chart. The default value is calculated.
  * </TD></TR>
  * <TR>
  * <TD>XAxisNumSpacing
  * </TD>
  * <TD>
  * The XAxisNumSpacing property controls the interval between 
  * axis labels. 
  * If a null string is used, Chart will calculate the interval.
  * The default value is calculated.
  * </TD></TR>
  * <TR>
  * <TD>XAxisTitleText
  * </TD>
  * <TD>
  * The XAxisTitleText property specifies the text that
  * will appear as the x axis title. The default value is " " (empty string).
  * </TD></TR>
  * <TR>
  * <TD>YAxisAnnotationMethod
  * </TD>
  * <TD>
  * The YAxisAnnotationMethod property determines how axis annotations 
  * are generated.  Valid values include VALUE (annotation is generated
  * by Chart, with possible callbacks to a label generator);
  * VALUE_LABELS (annotation is taken from a list of value
  * labels provided by the user -- a value label is a label that appears
  * at a particular axis value);  POINT_LABELS 
  * (annotation comes from the data source's point labels 
  * that are associated with particular data points);  and
  * TIME_LABELS (Chart generates time/date labels based on the 
  * TimeUnit, TimeBase and TimeFormat properties). The default value is VALUE.
  * </TD></TR>
  * <TR>
  * <TD>YAxisGridVisible
  * </TD>
  * <TD>
  * The YAxisGridVisible property determines whether a grid is drawn for the axis.
  * </TD></TR>
  * <TR>
  * <TD>YAxisLogarithmic
  * </TD>
  * <TD>
  * The YAxisLogarithmic property determines whether the
  * first <TT>y</TT> axis will be logarithmic (true) or linear (false). The default value is false.
  * </TD></TR>
  * <TR>
  * <TD>YAxisVisible
  * </TD>
  * <TD>
  * The YAxisVisible property determines whether the
  * first <TT>y</TT> axis is currently visible. The default value is true.
  * </TD></TR>
  * <TR>
  * <TD>YAxisMinMax
  * </TD>
  * <TD>
  * The YAxisMinMax controls both the YAxisMin and YAxisMax properties.
  * The YAxisMin property controls the minimum value shown on the axis. 
  * If a null string is used, Chart will calculate the axis min.
  * The data min is determined by Chart. The default value is calculated.
  * The YAxisMax property controls the maximum value shown on the axis. 
  * If a null string is used, Chart will calculate the axis max.
  * The data max is determined by Chart. The default value is calculated.
  * </TD></TR>
  * <TR>
  * <TD>YAxisNumSpacing
  * </TD>
  * <TD>
  * The YAxisNumSpacing property controls the interval between 
  * axis labels. 
  * If a null string is used, Chart will calculate the interval.
  * The default value is calculated.
  * </TD></TR>
  * <TR>
  * <TD>YAxisTitleText
  * </TD>
  * <TD>
  * The YAxisTitleText property specifies the text that
  * will appear as the Y axis title. The default value is " " (empty string).
  * </TD></TR>
  * </TABLE>
 */


public class JBdbChart extends DataChart implements DataSetAware {

/**
 * Default constructor - creates an empty chart
 */
public JBdbChart() {
	super();
}

/**
 * Disposes data binding resources.
 */
protected void finalize() throws Throwable {
	JBuilderDataBinding.detach(getDataSet());
    super.finalize();
}

/**
 * Returns the DataSet object bound to the chart.
 */
public DataSet getDataSet() {
	return (DataSet)super.getNativeMetaData();
}

/**
 * Sets the DataSet data source to the chart.
 */
public void setDataSet(DataSet dataSet) {
	try {
		if (dataSet != null) {
        	JBuilderDataBinding jbBinding = JBuilderDataBinding.getBinding(dataSet);
            if (jbBinding != null) {
            	if (!setDataBinding(jbBinding.getDataModel(), (MetaDataModel)null))
                	setDataSource(jbBinding.getDataModel());
            	return;
            }
		}
	} catch(Exception e) {
		e.printStackTrace();
	}
    setDataSource((ChartDataModel)null);
}

}

