/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ChartDataViewSeriesPropertyLoad.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:53 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.ChartDataViewSeries;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.io.JCIOException;

/**
 * Responsible for loading properties of ChartDataViewSeries
 * objects using a specified PropertyAcessModel.  Not
 * really intended for public consumption
 *
 * @see com.klg.jclass.chart.ChartDataViewSeries
 * @see com.klg.jclass.chart.property.ChartDataViewSeriesPropertySave
 */
public class ChartDataViewSeriesPropertyLoad implements PropertyLoadModel {

protected ChartDataViewSeries series = null;

/**
 * Sets the source object ie the object on which to set properties.
 *
 * @param o Object that is loading properties
 */
public void setSource(Object o)
{
	if (o instanceof ChartDataViewSeries) {
		this.series = (ChartDataViewSeries)o;
	}
}

/**
 * Loads properties using the given property accessor object
 *
 * @param prop Property access object.
 * @param name Name of object to be loaded.
 * @throws JCIOException
 */
public void loadProperties(PropertyAccessModel prop, String name)
    throws JCIOException
{
	// We don't load styles here anymore. They're always loaded
	// through the data view since that's their true parent.  This
	// allows styles to be specified beyond the end of data.

    // Set the name property
    String pval = prop.getProperty(name + "name");
    if (pval != null) {
        series.setName(pval);
    }

	pval = prop.getProperty(name + "visible");
	series.setVisible(JCTypeConverter.toBoolean(pval, series.isVisible()));

	pval = prop.getProperty(name + "included");
	series.setIncluded(JCTypeConverter.toBoolean(pval, series.isIncluded()));

	pval = prop.getProperty(name + "visibleInLegend");
	series.setVisibleInLegend(JCTypeConverter.toBoolean(pval,
												series.isVisibleInLegend()));

	pval = prop.getProperty(name + "firstPoint");
	if (pval != null) {
		series.setFirstPoint(JCTypeConverter.toInt(pval,
												   series.getFirstPoint()));
	}

	pval = prop.getProperty(name + "lastPoint");
	if (pval != null) {
		series.setLastPoint(JCTypeConverter.toInt(pval, series.getLastPoint()));
	}

	pval = prop.getProperty(name + "label");
	if (pval != null && !pval.equals(series.getLabel())) {
		series.setLabel(pval);
	}
}

}
