/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ChartDataViewSeriesPropertySave.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:53 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.ChartDataViewSeries;
import com.klg.jclass.chart.JCChartStyle;
import com.klg.jclass.util.io.JCIOException;

/**
 * Responsible for saving properties of ChartDataViewSeries
 * objects to a destiation specified by PropertyPersistorModel.
 * Not really intended for public consumption.
 *
 * @see com.klg.jclass.chart.ChartDataViewSeries
 * @see com.klg.jclass.chart.property.ChartDataViewSeriesPropertyLoad
 */
public class ChartDataViewSeriesPropertySave implements PropertySaveModel {

protected ChartDataViewSeries series = null;
protected ChartDataViewSeries defaultSeries = null;

/**
 * Set the source component whose properties are to be saved
 * @param o The new source component
 */
public void setSource(Object o)
{
	if (o instanceof ChartDataViewSeries) {
		this.series = (ChartDataViewSeries)o;
	}
}

/**
 * Sets default object instance.  This object is typically used by implementors
 * to determine whether a property really needs to be saved.
 * @param def The new default object
 */
public void setDefault(Object def)
{
	if (def instanceof ChartDataViewSeries) {
		this.defaultSeries = (ChartDataViewSeries)def;
	}
}

/**
 * Check whether are any properties different from their default.  If so, then
 * then this saver will have something to write
 * @param prop Property persistor object
 * @return Whether any properties are different from their default.
 */
public boolean checkProperties(PropertyPersistorModel prop)
{
	if (series == null || defaultSeries == null) {
		return(false);
    }

	if (series.getName() != defaultSeries.getName()                     ||
        series.isVisible() != defaultSeries.isVisible()                 ||
	    series.isIncluded() != defaultSeries.isIncluded()               ||
		series.isVisibleInLegend() != defaultSeries.isVisibleInLegend() ||
		series.getFirstPoint() != defaultSeries.getFirstPoint()         ||
		!series.getLastPointIsDefault()                                 ||
		series.getLabel() != defaultSeries.getLabel())
	{
		return(true);
	}
	return(PropertySaveFactory.check(prop, series.getStyle(),
	                                 new JCChartStyle()));
}

/**
 * Click the method name to see which properties are saved.<p>
 * Saves the following values:
 *   <pre>
 *   All style values
 *   visible
 *   included
 *   visibleInLegend
 *   firstPoint
 *   lastPoint
 *   label
 *   </pre>
 * @param prop Property persistor object
 * @param name Name of object to be saved
 * @param tab The tab indent for this group of properties
 * @throws JCIOException
 */
public void saveProperties(PropertyPersistorModel prop, String name, int tab)
    throws JCIOException
{
	if (series == null || defaultSeries == null) {
		System.out.println("FAILURE: no data view series set");
		return;
	}

	// Check if something to do.
	if (!checkProperties(prop)) {
		return;
	}

	int newTab = prop.writeBegin("chart-data-view-series", tab);

    if (series.getName() != defaultSeries.getName()) {
        prop.writeProperty(name, "name", newTab, series.getName());
    }

	if (series.isVisible() != defaultSeries.isVisible()) {
		prop.writeProperty(name, "visible", newTab,
		                   new Boolean(series.isVisible()));
	}

	if (series.isIncluded() != defaultSeries.isIncluded()) {
		prop.writeProperty(name, "included", newTab,
		                   new Boolean(series.isIncluded()));
	}

	if (series.isVisibleInLegend() != defaultSeries.isVisibleInLegend()) {
		prop.writeProperty(name, "visibleInLegend", newTab,
						   new Boolean(series.isVisibleInLegend()));
	}

	if (series.getFirstPoint() != defaultSeries.getFirstPoint()) {
		prop.writeProperty(name, "firstPoint", newTab,
						   new Integer(series.getFirstPoint()));
	}

	if (!series.getLastPointIsDefault()) {
		prop.writeProperty(name, "lastPoint", newTab,
						   new Integer(series.getLastPoint()));
	}

	if (series.getLabel() != defaultSeries.getLabel()) {
		prop.writeProperty(name, "label", newTab,
		                   prop.expandText(series.getLabel()));
	}

	prop.writeEnd(null, tab, true, true);

	// Save chart style
	PropertySaveFactory.save(prop, series.getStyle(), new JCChartStyle(),
	                         name, tab);
	prop.writeEnd("chart-data-view-series", tab, true, false);
}

}
