/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ChartInteriorRegionPropertyLoad.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:53 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.ChartInteriorRegion;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.io.JCIOException;
import com.klg.jclass.util.swing.JCSwingTypeConverter;

import java.awt.*;

/**
 * Responsible for loading properties of ChartInteriorRegion
 * objects using a specified PropertyAcessModel.  Not
 * really intended for public consumption
 *
 * @see com.klg.jclass.chart.ChartInteriorRegion
 * @see com.klg.jclass.chart.property.ChartInteriorRegionPropertySave
 */
public class ChartInteriorRegionPropertyLoad implements PropertyLoadModel {

protected ChartInteriorRegion reg = null;

/**
 * Sets the source object ie the object on which to set properties.
 *
 * @param o Object that is loading properties
 */
public void setSource(Object o)
{
	if (o instanceof ChartInteriorRegion) {
		this.reg = (ChartInteriorRegion)o;
	}
}

/**
 * Loads properties using the given property accessor object
 *
 * @param prop Property access object.
 * @param name Name of object to be loaded.
 * @throws JCIOException
 */
public void loadProperties(PropertyAccessModel prop, String name)
    throws JCIOException
{
	if (reg == null) {
		System.out.println("FAILURE: No region set");
		return;
	}

	// Get font
	String pval = prop.getProperty(name + "font");
	if (pval != null) {
		reg.setFont(JCSwingTypeConverter.toFont(pval));
	}

	// Get foreground
	pval = prop.getProperty(name + "foreground");
	if (pval != null) {
		reg.setForeground(JCSwingTypeConverter.toColor(pval));
	}

	// Get background
	pval = prop.getProperty(name + "background");
	if (pval != null) {
		reg.setBackground(JCSwingTypeConverter.toColor(pval));
	}

	// Get groupingUsed
	pval = prop.getProperty(name + "groupingUsed");
	if (pval != null) {
		reg.setGroupingUsed(JCTypeConverter.toBoolean(pval,
													  reg.getGroupingUsed()));
	}

	// Get numberLocalization
	pval = prop.getProperty(name + "numberLocalization");
	if (pval != null) {
		reg.setNumberLocalization(JCTypeConverter.toBoolean(pval,
												reg.getNumberLocalization()));
	}

	// Get visible
	pval = prop.getProperty(name + "visible");
	reg.setVisible(JCTypeConverter.toBoolean(pval, reg.isVisible()));

	// Get insets
	Object o = prop.getPropertyObject(name + "insets");
	if (o != null) {
		if (o instanceof String) {
			pval = (String)o;
	        reg.setInsets(JCSwingTypeConverter.toInsets(pval, reg.getInsets()));
		}
		else if (o instanceof Insets) {
			reg.setInsets((Insets)o);
		}
	}

    // Set left placement
	pval = prop.getProperty(name + "left");
	if (pval != null ) {
		try {
			if (Integer.valueOf(pval).intValue() == 0) {
				reg.setLeft(0);
				reg.setLeftIsDefault( false );
			}
			else {
				reg.setLeft(JCTypeConverter.toInt(pval, reg.getLeft()));
			}
		}
		catch (NumberFormatException e) {}
	}

    // Set width placement
	pval = prop.getProperty(name + "width");
	if (pval != null ) {
		try {
			if (Integer.valueOf(pval).intValue() == 0) {
				reg.setWidth(0);
				reg.setWidthIsDefault( false );
			}
			else {
				reg.setWidth(JCTypeConverter.toInt(pval, reg.getWidth()));
			}
		}
		catch (NumberFormatException e) {}
	}

    // Set top placement
	pval = prop.getProperty(name + "top");
	if (pval != null ) {
		try {
			if (Integer.valueOf(pval).intValue() == 0) {
				reg.setTop(0);
				reg.setTopIsDefault( false );
			}
			else {
				reg.setTop(JCTypeConverter.toInt(pval, reg.getTop()));
			}
		}
		catch (NumberFormatException e) {}
	}

    // Set height placement
	pval = prop.getProperty(name + "height");
	if (pval != null ) {
		try {
			if (Integer.valueOf(pval).intValue() == 0) {
				reg.setHeight(0);
				reg.setHeightIsDefault( false );
			}
			else {
				reg.setHeight(JCTypeConverter.toInt(pval, reg.getHeight()));
			}
		}
		catch (NumberFormatException e) {}
	}
}

}
