/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ChartInteriorRegionPropertySave.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:53 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.ChartInteriorRegion;
import com.klg.jclass.util.io.JCIOException;
import com.klg.jclass.util.swing.JCSwingTypeConverter;

import java.awt.*;

/**
 * Responsible for saving properties of ChartInteriorRegion
 * objects to a destiation specified by PropertyPersistorModel.
 * Not really intended for public consumption.
 *
 * @see com.klg.jclass.chart.ChartInteriorRegion
 * @see com.klg.jclass.chart.property.ChartInteriorRegionPropertyLoad
 */
public class ChartInteriorRegionPropertySave implements PropertySaveModel {

protected ChartInteriorRegion   reg = null;
protected ChartInteriorRegion   defaultReg = null;

/**
 * Set the source component whose properties are to be saved
 * @param o The new source component
 */
public void setSource(Object o)
{
	if (o instanceof ChartInteriorRegion) {
		this.reg = (ChartInteriorRegion)o;
	}
}

/**
 * Sets default object instance.  This object is typically used by implementors
 * to determine whether a property really needs to be saved.
 * @param def The new default object
 */
public void setDefault(Object def)
{
	if (def instanceof ChartInteriorRegion) {
		this.defaultReg = (ChartInteriorRegion)def;
	}
}

/**
 * Check whether are any properties different from their default.  If so, then
 * then this saver will have something to write
 * @param prop Property persistor object
 * @return Whether any properties are different from their default.
 */
public boolean checkProperties(PropertyPersistorModel prop)
{
	if (reg == null || defaultReg == null) {
		return(false);
	}
	Insets insets = reg.getInsets();
	if (reg.getFont() != defaultReg.getFont()                              ||
	    reg.getForeground() != defaultReg.getForeground()                  ||
		reg.getBackground() != defaultReg.getBackground()                  ||
	    reg.getGroupingUsed() != defaultReg.getGroupingUsed()              ||
	    reg.getNumberLocalization() != defaultReg.getNumberLocalization()  ||
	    reg.isVisible() != defaultReg.isVisible()                          ||
	    !reg.getLeftIsDefault()                                            ||
		!reg.getWidthIsDefault()                                           ||
	    !reg.getTopIsDefault()                                             ||
	    !reg.getHeightIsDefault()                                          ||
		(insets != null && !insets.equals(defaultReg.getInsets())))
	{
		return(true);
	}

	return(false);
}

/**
 * Saves HTML parameters from the current param source;
 * click the method name to see which properties are saved.<p>
 * Saves the following properties:
 *  <pre>
 *   font (Font)
 *   foreground (Color)
 *   background (Color)
 *   groupingUsed (boolean)
 *   numberLOcalization (boolean)
 *   visible (boolean)
 *   insets (Insets)
 *   left (int)
 *   width (int)
 *   top (int)
 *   height (int)
 *   </pre>
 * @param prop Property persistor object
 * @param name Name of object to be saved
 * @param tab The tab indent for this group of properties
 * @throws JCIOException
 */
public void saveProperties(PropertyPersistorModel prop, String name, int tab)
    throws JCIOException
{
	if (reg == null || defaultReg == null) {
		System.out.println("FAILURE: No region set");
		return;
	}

	int newTab = prop.writeBegin("chart-interior-region", tab);

	// Set font
	if (reg.getFont() != defaultReg.getFont()) {
		prop.writeProperty(name, "font", newTab,
						   JCSwingTypeConverter.fromFont(reg.getFont()));
	}

	// Set foreground
	if (reg.getForeground() != defaultReg.getForeground()) {
		prop.writeProperty(name, "foreground", newTab,
					       JCSwingTypeConverter.fromColor(reg.getForeground()));
	}

	// Set background
	if (reg.getBackground() != defaultReg.getBackground()) {
		prop.writeProperty(name, "background", newTab,
					       JCSwingTypeConverter.fromColor(reg.getBackground()));
	}


	// Set groupingUsed
	if (reg.getGroupingUsed() != defaultReg.getGroupingUsed()) {
		prop.writeProperty(name, "groupingUsed", newTab,
						   new Boolean(reg.getGroupingUsed()));
	}

	// Set numberLocalization
	if (reg.getNumberLocalization() != defaultReg.getNumberLocalization()) {
		prop.writeProperty(name, "numberLocalization", newTab,
						   new Boolean(reg.getNumberLocalization()));
	}

	// Set visible
	if (reg.isVisible() != defaultReg.isVisible()) {
		prop.writeProperty(name, "visible", newTab,
						   new Boolean(reg.isVisible()));
	}

	// Set Left placement
	if (!reg.getLeftIsDefault()) {
	    prop.writeProperty(name, "left", newTab, new Integer(reg.getLeft()));
	}

    // Set width placement
	if (!reg.getWidthIsDefault()) {
	    prop.writeProperty(name, "width", newTab, new Integer(reg.getWidth()));
	}

    // Set top placement
	if (!reg.getTopIsDefault()) {
	    prop.writeProperty(name, "top", newTab, new Integer(reg.getTop()));
	}

    // Set height placement
	if (!reg.getHeightIsDefault()) {
	    prop.writeProperty(name, "height", newTab, new Integer(reg.getHeight()));
	}
	prop.writeEnd(null, tab, true, true);

	saveInsets(prop, name, tab + PropertySaveFactory.tabIncrement);
	prop.writeEnd("chart-interior-region", tab, true, false);
}

protected void saveInsets(PropertyPersistorModel prop, String name, int tab)
{
	Insets insets = reg.getInsets();
	if (insets == null || insets.equals(defaultReg.getInsets())) {
		return;
	}

	if (prop.getType().equals("HTML")) {
		prop.writeProperty(name, "insets", tab,
		                   "" + insets.top + "," + insets.left + "," +
						   insets.bottom + "," + insets.right);
	}
	else {
		ComponentPropertySave.writeInsets(prop, name, tab, insets);
	}
}

}
