/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ChartTextPropertyLoad.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:53 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.ChartText;
import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.io.JCIOException;

/**
 * Responsible for loading properties of ChartText
 * objects using a specified PropertyAcessModel.  Not
 * really intended for public consumption
 *
 * @see com.klg.jclass.chart.ChartText
 * @see com.klg.jclass.chart.property.ChartTextPropertySave
 */
public class ChartTextPropertyLoad extends ChartInteriorRegionPropertyLoad {

protected ChartText text = null;

/**
 * Sets the source object ie the object on which to set properties.
 *
 * @param o Object that is loading properties
 */
public void setSource(Object o)
{
	super.setSource(o);
	if (o instanceof ChartText) {
		this.text = (ChartText)o;
	}
}

/**
 * Loads properties using the given property accessor object
 *
 * @param prop Property access object.
 * @param name Name of object to be loaded.
 * @throws JCIOException
 */
public void loadProperties(PropertyAccessModel prop, String name)
    throws JCIOException
{
	super.loadProperties(prop, name);

	if (text == null) {
		System.out.println("FAILURE: No text set");
		return;
	}

	// Text
	String pval = prop.getProperty(name + "text");
	if (pval != null) {
		text.setText(pval);
	}

	// Rotation
	boolean isXML    = prop.getType().equals("XML");
	String[] strings = isXML ? JCChartEnumMappings.rotation_xml_strings
		                     : JCChartEnumMappings.rotation_strings;
	int[] values     = JCChartEnumMappings.rotation_values;
	pval = prop.getProperty(name + "rotation");
	text.setRotation(JCTypeConverter.toEnum(pval, "titleRotation", strings,
	                                        values, text.getRotation()));

	// Adjust
	pval = prop.getProperty(name + "adjust");
	strings = JCChartEnumMappings.adjust_strings;
	values  = JCChartEnumMappings.adjust_values;
	text.setAdjust(JCTypeConverter.toEnum(pval, "titleAdjust", strings, values,
	                                      text.getAdjust()));
}

}
