/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ChartTextPropertySave.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:53 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.ChartText;
import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.io.JCIOException;

/**
 * Responsible for saving properties of ChartText
 * objects to a destiation specified by PropertyPersistorModel.
 * Not really intended for public consumption.
 *
 * @see com.klg.jclass.chart.ChartText
 * @see com.klg.jclass.chart.property.ChartTextPropertyLoad
 */
public class ChartTextPropertySave extends ChartInteriorRegionPropertySave {

protected ChartText text = null;
protected ChartText defaultText = null;

/**
 * Set the source component whose properties are to be saved
 * @param o The new source component
 */
public void setSource(Object o)
{
	super.setSource(o);
	if (o instanceof ChartText) {
		this.text = (ChartText)o;
	}
}

/**
 * Sets default object instance.  This object is typically used by implementors
 * to determine whether a property really needs to be saved.
 * @param def The new default object
 */
public void setDefault(Object def)
{
	super.setDefault(def);
	if (def instanceof ChartText) {
		this.defaultText = (ChartText)def;
	}
}

/**
 * Check whether are any properties different from their default.  If so, then
 * then this saver will have something to write
 * @param prop Property persistor object
 * @return Whether any properties are different from their default.
 */
public boolean checkProperties(PropertyPersistorModel prop)
{
	if (text == null || defaultText == null) {
		return(false);
	}

	if (text.getText() != defaultText.getText()          ||
	    text.getRotation() != defaultText.getRotation()  ||
		text.getAdjust() != defaultText.getAdjust()      ||
	    super.checkProperties(prop))
	{
		return(true);
	}

	return(false);
}

/**
 * Saves HTML parameters from the current param source;
 * click the method name to see which properties are saved.<p>
 * Sets the following properties:
 * <pre>
 *   text (string)
 *   titleRotation (enum)
 *   adjust (enum)
 * </pre>
 * @param prop Property persistor object
 * @param name Name of object to be saved
 * @param tab The tab indent for this group of properties
 * @throws JCIOException
 */
public void saveProperties(PropertyPersistorModel prop, String name, int tab)
    throws JCIOException
{
	if (text == null || defaultText == null) {
		System.out.println("FAILURE: No text set");
		return;
	}

	// Since we are continuing the axis-title element from the axis title
	// saveProperties, we need to compute the tab length
	String s = "<axis-title ";
	int newTab = tab + s.length();

	// Set text
	if (text.getText() != defaultText.getText()) {
		prop.writeProperty(name, "text", newTab,
		                   prop.expandText(text.getText()));
	}

	// Set titleRotation
	boolean isXML    = prop.getType().equals("XML");
	String[] strings = isXML ? JCChartEnumMappings.rotation_xml_strings
		                     : JCChartEnumMappings.rotation_strings;
	int[] values     = JCChartEnumMappings.rotation_values;
	if (text.getRotation() != defaultText.getRotation()) {
		prop.writeProperty(name, "rotation", newTab,
						   JCTypeConverter.fromEnum(text.getRotation(),
										            strings, values));
	}

	// Set adjust
	strings = JCChartEnumMappings.adjust_strings;
	values  = JCChartEnumMappings.adjust_values;
	if (text.getAdjust() != defaultText.getAdjust()) {
		prop.writeProperty(name, "adjust", newTab,
						   JCTypeConverter.fromEnum(text.getAdjust(),
										            strings, values));
	}
	prop.writeEnd(null, tab, true, true);

	if (super.checkProperties(prop)) {
	    super.saveProperties(prop, name,
		                     tab + PropertySaveFactory.tabIncrement);
	}
}

}
