/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ComponentPropertyLoad.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:53 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.io.JCIOException;
import com.klg.jclass.util.swing.JCSwingTypeConverter;

import javax.swing.*;
import javax.swing.border.Border;
import java.awt.*;

/**
 * Responsible for loading properties of java.awt.Component
 * objects using a specified PropertyAcessModel.  This is
 * a base class extended by other chart property loaders,
 * like JCChartAreaPropertyLoad, JCChartPropertyLoad,
 * JCLegendPropertyLoad and JLabelPropertyLoad.  Not
 * really intended for public consumptiono
 *
 * @see java.awt.Component
 * @see com.klg.jclass.chart.property.ComponentPropertySave
 */
public class ComponentPropertyLoad implements PropertyLoadModel {

protected Component component = null;

/**
 * Sets the source object ie the object on which to set properties.
 *
 * @param o Object that is loading properties
 */
public void setSource(Object o)
{
	if (o instanceof Component) {
		this.component = (Component)o;
	}
}

/**
 * Loads properties using the given property accessor object
 *
 * @param prop Property access object.
 * @param name Name of object to be loaded.
 * @throws JCIOException
 */
public void loadProperties(PropertyAccessModel prop, String name)
    throws JCIOException
{
	if (component == null || !(component instanceof JComponent)) {
		System.out.println("FAILURE: No JComponent set");
		return;
	}

	JComponent jComp = (JComponent)component;
	String opaqueval = prop.getProperty(name + "opaque");
	if (opaqueval != null) {
    	jComp.setOpaque(JCTypeConverter.toBoolean(opaqueval, jComp.isOpaque()));
	}

	String pval = prop.getProperty(name + "background");
	if (pval != null) {
    	jComp.setBackground(JCSwingTypeConverter.toColor(pval,
							component.getBackground()));
	}

	// If background is set but opaque is not mentioned, assume true
	// for backwards compatibility sake
	if (pval != null && opaqueval == null) {
		jComp.setOpaque(true);
	}

	pval = prop.getProperty(name + "foreground");
	if (pval != null) {
    	jComp.setForeground(JCSwingTypeConverter.toColor(pval,
							component.getForeground()));
	}

	pval = prop.getProperty(name + "font");
	if (pval != null) {
    	jComp.setFont(JCSwingTypeConverter.toFont(pval, component.getFont()));
	}

	Object o = prop.getPropertyObject(name + "border");
	if (o != null) {
		if (o instanceof String) {
			pval = (String)o;
			jComp.setBorder(JCSwingTypeConverter.toBorder(pval));
		}
		else if (o instanceof Border) {
			jComp.setBorder((Border)o);
		}
	}

	pval = prop.getProperty(name + "visible");
	if (pval != null) {
    	jComp.setVisible(JCTypeConverter.toBoolean(pval,
												   component.isVisible()));
	}
}

}
