/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: FilePropertyAccessModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:53 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.util.io.OutputProperties;
import com.klg.jclass.util.io.JCIOException;

import java.io.InputStream;

/**
 * A model for accessing property values based on property names where the
 * property values are file names.  This differs from the standard
 * PropertyAccessModel in that it allows an Accessor to provide specific
 * rules for the formatting of file names, such as establishing the full
 * path.
 * This class should probably not be implemented on it's own, only in
 * conjunction with the PropertyAccessModel interface.
 */
public interface FilePropertyAccessModel {

/**
 * Get property value based on specified key.  Property value is a file
 * name.  The implementing accessor will provide the rules that determine
 * the formatting of the file name and will return an input stream based
 * on the file.
 *
 * @param key Name of property to be retrieved.
 * @param outputProperties Object which records the output properties to be
 * used from loading in the input properties
 * @return Property value, a file name, as an input stream.
 *
 * @see com.klg.jclass.chart.property.html.JCAppletAccessor
 * @see com.klg.jclass.chart.property.JCBaseAccessor
 * @see com.klg.jclass.chart.property.JCFileAccessor
 * @see com.klg.jclass.chart.property.JCStringAccessor
 * @throws JCIOException
 */
public InputStream getFilePropertyStream(String key,
                                         OutputProperties outputProperties)
    throws JCIOException;

}
