/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ImageMapInfoPropertySave.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:53 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.util.ImageMapInfo;
import com.klg.jclass.util.io.JCIOException;

/**
 * Responsible for saving properties of <code>ImageMapInfo</code>
 * objects to a destination specified by <code>PropertyPersistorModel</code>.
 * Not intended for public use.
 *
 * @see com.klg.jclass.util.ImageMapInfo
 * @see com.klg.jclass.chart.property.ImageMapInfoPropertyLoad
 */
public class ImageMapInfoPropertySave implements PropertySaveModel {

protected ImageMapInfo imageMapInfo;
protected ImageMapInfo defaultImageMapInfo;

/**
 * Set the source ImageMapInfo object whose properties are to be saved.
 * @param o The new source component.
 */
public void setSource(Object o)
{
    if (o instanceof ImageMapInfo) {
        this.imageMapInfo = (ImageMapInfo)o;
    }
}

/**
 * Sets default object instance.  This object is typically used by implementors
 * to determine whether a property really needs to be saved.
 * @param def The new default object.
 */
public void setDefault(Object def)
{
    if (def instanceof ImageMapInfo) {
        this.defaultImageMapInfo = (ImageMapInfo)def;
    }
}

/**
 * Check whether are any properties different from their default.  If so, then
 * then this saver will have something to write
 * @param prop Property persistor object
 * @return Whether any properties are different from their default.
 */
public boolean checkProperties(PropertyPersistorModel prop)
{
    if (imageMapInfo == null || defaultImageMapInfo == null) {
        return(false);
    }
    if (imageMapInfo.isEmpty()) {
        return(false);
    }
    return(true);
}

/**
 * Saves HTML parameters from the current param source;
 * click the method name to see which properties are saved.<p>
 * Saves the following properties:
 *  <pre>
 *   imageMapUrl (String)
 *   imageMapExtra (String)
 *  </pre>
 * @param prop property persistor object
 * @param name name of object to be saved
 * @param tab the tab indent for this group of properties
 * @throws JCIOException
 */
public void saveProperties(PropertyPersistorModel prop, String name, int tab)
    throws JCIOException
{
	// Check is something to do
	if (!checkProperties(prop)) {
		return;
	}

    int newTab = prop.writeBegin("image-map-info", tab);

    String url = imageMapInfo.getUrl();
    if (url != null && !url.equals(defaultImageMapInfo.getUrl())) {
        prop.writeProperty(name, "url", newTab, prop.expandText(url));
    }

    String extra = imageMapInfo.getExtra();
    if (extra != null && !extra.equals(defaultImageMapInfo.getExtra())) {
        prop.writeProperty(name, "extra", newTab, prop.expandText(extra));
    }

    prop.writeEnd(null, tab, true, false);
}

}
