/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCAnnoPropertyLoad.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:53 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import java.awt.Color;

import com.klg.jclass.chart.JCAnno;
import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.swing.JCSwingTypeConverter;
import com.klg.jclass.util.io.JCIOException;

/**
 * Responsible for loading properties of <code>JCAnno</code>
 * objects using a specified <code>PropertyAcessModel</code>. Not
 * intended for public use.
 *
 * @see com.klg.jclass.chart.JCAnno
 */
public class JCAnnoPropertyLoad implements PropertyLoadModel {

protected JCAnno anno = null;

/**
 * Sets the source object (the object on which to set properties).
 *
 * @param o object that is loading properties
 */
public void setSource(Object o)
{
    if (o instanceof JCAnno) {
        this.anno = (JCAnno)o;
    }
}

/**
 * Loads properties using the given property accessor object.
 *
 * @param prop property access object.
 * @param name name of object to be loaded.
 * @throws JCIOException
 */
public void loadProperties(PropertyAccessModel prop, String name)
    throws JCIOException
{
    if (anno == null) {
        System.out.println("FAILURE: No anno object is set");
        return;
    }

    // Remove the last period from name.
    String annoName = name.substring(0, name.length()-1);
    anno.setName(annoName);

    String pval = prop.getProperty(name + "type");
    if (pval != null) {
        anno.setType(JCTypeConverter.toEnum(pval,
                     JCChartEnumMappings.anno_type_strings,
                     JCChartEnumMappings.anno_type_values,
                     JCAnno.USER_DEFINED));
    }

    pval = prop.getProperty(name + "startValue");
    if (pval != null) {
        anno.setStartValue(JCTypeConverter.toDouble(pval,
                           anno.getStartValue()));
    }

    pval = prop.getProperty(name + "stopValue");
    if (pval != null) {
        anno.setStopValue(JCTypeConverter.toDouble(pval,
                          anno.getStopValue()));
    }

    pval = prop.getProperty(name + "incrementValue");
    if (pval != null) {
        anno.setIncrementValue(JCTypeConverter.toDouble(pval,
                               anno.getIncrementValue()));
    }

    pval = prop.getProperty(name + "innerExtent");
    if (pval != null) {
        anno.setInnerExtent(JCTypeConverter.toInt(pval,
                               anno.getInnerExtent()));
    }

    pval = prop.getProperty(name + "outerExtent");
    if (pval != null) {
        anno.setOuterExtent(JCTypeConverter.toInt(pval,
                               anno.getOuterExtent()));
    }

    pval = prop.getProperty(name + "labelExtent");
    if (pval != null) {
        anno.setLabelExtent(JCTypeConverter.toInt(pval,
                               anno.getLabelExtent()));
    }

    pval = prop.getProperty(name + "precision");
    if (pval != null) {
        anno.setPrecision(JCTypeConverter.toInt(pval,
                               anno.getPrecision()));
    }

    pval = prop.getProperty(name + "drawTicks");
    if (pval != null) {
        anno.setDrawTicks(JCTypeConverter.toBoolean(pval,
                          anno.getDrawTicks()));
    }

    pval = prop.getProperty(name + "drawLabels");
    if (pval != null) {
        anno.setDrawLabels(JCTypeConverter.toBoolean(pval,
                          anno.getDrawLabels()));
    }

    pval = prop.getProperty(name + "tickColor");
    if (pval != null) {
        anno.setTickColor(JCSwingTypeConverter.toColor(pval,
                          anno.getTickColor()));
    }

    pval = prop.getProperty(name + "labelColor");
    if (pval != null) {
        anno.setLabelColor(JCSwingTypeConverter.toColor(pval,
                          anno.getLabelColor()));
    }
}

}
