/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCAreaChartFormatPropertySave.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:53 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.JCAreaChartFormat;
import com.klg.jclass.util.io.JCIOException;

/**
 * Responsible for saving properties of <code>JCAreaChartFormat</code>
 * objects to a destiation specified by <code>PropertyPersistorModel</code>.
 * Not intended for public use.
 *
 * @see com.klg.jclass.chart.JCAreaChartFormat
 * @see com.klg.jclass.chart.property.JCAreaChartFormatPropertyLoad
 */
public class JCAreaChartFormatPropertySave implements PropertySaveModel {

protected JCAreaChartFormat format = null;
protected JCAreaChartFormat defaultFormat = null;

/**
 * Sets the source component, whose properties are to be saved.
 * @param o the new source component
 */
public void setSource(Object o)
{
	if (o instanceof JCAreaChartFormat) {
		this.format = (JCAreaChartFormat)o;
	}
}

/**
 * Sets the default object instance. This object is typically used by implementors
 * to determine whether a property really needs to be saved.
 * @param def the new default object
 */
public void setDefault(Object def)
{
	if (def instanceof JCAreaChartFormat) {
		this.defaultFormat = (JCAreaChartFormat)def;
	}
}

/**
 * Checks whether there are any properties different from their default. If so,
 * then this saver has information to write.
 * @param prop property persistor object
 * @return whether any properties are different from their default
 */
public boolean checkProperties(PropertyPersistorModel prop)
{
    if (format == null || defaultFormat == null) {
		return(false);
    }

	if (format.is100Percent() != defaultFormat.is100Percent()) {
		return(true);
	}
	return(false);
}

/**
 * Saves properties, including <code>100percent (boolean);</code>.
 *
 * @param prop property persistor object
 * @param name name of object to be saved
 * @param tab the tab indent for this group of properties
 * @throws JCIOException
 */
public void saveProperties(PropertyPersistorModel prop, String name, int tab)
    throws JCIOException
{
	if (format == null || defaultFormat == null) {
		System.out.println("FAILURE: No area format set");
		return;
	}

	// Check if something to do.
	if (!checkProperties(prop)) {
		return;
	}

	int newTab = prop.writeBegin("area-format", tab);

	// Set 100Percent
	String percentName = prop.getType().equals("XML") ? "percent100"
	                                                  : "100Percent";
	if (format.is100Percent() != defaultFormat.is100Percent()) {
		prop.writeProperty(name, percentName, newTab,
						   new Boolean(format.is100Percent()));
	}
	prop.writeEnd(null, tab, true, false);
}

}
