/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCAxisFormulaPropertyLoad.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:53 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCAxisFormula;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.io.JCIOException;

/**
 * Responsible for loading properties of <code>JCAxisFormula</code>
 * objects using a specified <code>PropertyAcessModel</code>. Not
 * intended for public use.
 *
 * @see com.klg.jclass.chart.JCAxisFormula
 * @see com.klg.jclass.chart.property.JCAxisFormulaPropertySave
 */
public class JCAxisFormulaPropertyLoad implements PropertyLoadModel {

protected JCAxisFormula formula = null;

/**
 * Sets the source object (the object on which to set properties).
 *
 * @param o object that is loading properties
 */
public void setSource(Object o)
{
	if (o instanceof JCAxisFormula) {
		this.formula = (JCAxisFormula)o;
	}
}

/**
 * Loads properties using the given property accessor object.
 *
 * @param prop property access object.
 * @param name name of object to be loaded.
 * @throws JCIOException
 */
public void loadProperties(PropertyAccessModel prop, String name)
    throws JCIOException
{
	if (formula == null) {
		System.out.println("FAILURE: No axis formula set");
		return;
	}

	String pval = prop.getProperty(name + "multiplier");
	formula.setMultiplier(JCTypeConverter.toDouble(pval,
												   formula.getMultiplier()));

	pval = prop.getProperty(name + "constant");
	formula.setConstant(JCTypeConverter.toDouble(pval,
												   formula.getConstant()));

	pval = prop.getProperty(name + "originator");
	if (pval != null) {
		JCAxis ax = formula.getParent().getChartArea().findAxisByName(pval);
		if (ax != null) {
			formula.setOriginator(ax);
		}
	}
}

}
