/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCAxisFormulaPropertySave.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:53 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.JCAxisFormula;
import com.klg.jclass.util.io.JCIOException;

/**
 * Responsible for saving properties of <code>JCAxisFormula</code>
 * objects to a destiation specified by <code>PropertyPersistorModel</code>.
 * Not intended for public use.
 *
 * @see com.klg.jclass.chart.JCAxisFormula
 * @see com.klg.jclass.chart.property.JCAxisFormulaPropertyLoad
 */
public class JCAxisFormulaPropertySave implements PropertySaveModel {

protected JCAxisFormula formula = null;
protected JCAxisFormula defaultFormula = null;

/**
 * Sets the source component, whose properties are to be saved.
 * @param o the new source component
 */
public void setSource(Object o)
{
	if (o instanceof JCAxisFormula) {
		this.formula = (JCAxisFormula)o;
	}
}

/**
 * Sets default object instance.  This object is typically used by implementors
 * to determine whether a property really needs to be saved.
 * @param def the new default object
 */
public void setDefault(Object def)
{
	if (def instanceof JCAxisFormula) {
		this.defaultFormula = (JCAxisFormula)def;
	}
}

/**
 * Checks whether there are any properties different from their default.  If so,
 * then this saver has information to write.
 * @param prop property persistor object
 * @return whether any properties are different from their default
 */
public boolean checkProperties(PropertyPersistorModel prop)
{
	if (formula == null || defaultFormula == null) {
		return(false);
	}

	// Check if anything to do
	if (formula.getMultiplier() == defaultFormula.getMultiplier() &&
	    formula.getConstant() == defaultFormula.getConstant()     &&
		formula.getOriginator() == null)
	{
		return(false);
	}
	return(true);
}

/**
 * Saves HTML parameters from the current param source;
 * click the method name to see which properties are saved.<p>
 * Sets the following properties:
 * <pre>
 *   multiplier (double)
 *   constant (double)
 *   originator (JCAxis, through string)
 * </pre>
 * @param prop property persistor object
 * @param name name of object to be saved
 * @param tab the tab indent for this group of properties
 * @throws JCIOException
 */
public void saveProperties(PropertyPersistorModel prop, String name, int tab)
    throws JCIOException
{
	if (formula == null || defaultFormula == null) {
		System.out.println("FAILURE: No axis formula set");
		return;
	}

	// Check if something to do.
	if (!checkProperties(prop)) {
		return;
	}

	int newTab = prop.writeBegin("axis-formula", tab);

	// Set multiplier
	if (formula.getMultiplier() != defaultFormula.getMultiplier()) {
		prop.writeProperty(name, "multiplier", newTab,
						   new Double(formula.getMultiplier()));
	}

	// Set constant
	if (formula.getConstant() != defaultFormula.getConstant()) {
		prop.writeProperty(name, "constant", newTab,
						   new Double(formula.getConstant()));
	}

	// Set originator
	if (formula.getOriginator() != null) {
		prop.writeProperty(name, "originator", newTab,
						   prop.expandText(formula.getOriginator().getName()));
	}
	prop.writeEnd(null, tab, true, false);
}

}
