/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCAxisPropertyLoad.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:53 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import java.util.ArrayList;
import java.util.TimeZone;

import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.chart.JCChartUtil;
import com.klg.jclass.chart.JCValueLabel;
import com.klg.jclass.chart.JCAnno;
import com.klg.jclass.chart.JCGrid;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.io.JCIOException;

/**
 * Responsible for loading properties of <code>JCAxis</code>
 * objects using a specified <code>PropertyAcessModel</code>. Not
 * intended for public use.
 *
 * @see com.klg.jclass.chart.JCAxis
 * @see com.klg.jclass.chart.property.JCAxisPropertySave
 */
public class JCAxisPropertyLoad extends ChartInteriorRegionPropertyLoad {

protected JCAxis axis = null;

/**
 * Sets the source object (the object on which to set properties).
 *
 * @param o object that is loading properties
 */
public void setSource(Object o)
{
    super.setSource(o);
    if (o instanceof JCAxis) {
        this.axis = (JCAxis)o;
    }
}

/**
 * Loads properties using the given property accessor object.
 *
 * @param prop property access object.
 * @param name name of object to be loaded.
 * @throws JCIOException
 */
public void loadProperties(PropertyAccessModel prop, String name)
    throws JCIOException
{
    super.loadProperties(prop, name);

    if (axis == null) {
        System.out.println("FAILURE: No axis set");
        return;
    }

    String pval = prop.getProperty(name + "vertical");
    axis.setVertical(JCTypeConverter.toBoolean(pval, axis.isVertical()));

    pval = prop.getProperty(name + "logarithmic");
    axis.setLogarithmic(JCTypeConverter.toBoolean(pval, axis.isLogarithmic()));

    boolean isXML    = prop.getType().equals("XML");
    String[] strings = isXML ? JCChartEnumMappings.annRotation_xml_strings
                             : JCChartEnumMappings.annRotation_strings;
    int[] values     = JCChartEnumMappings.annRotation_values;
    pval = prop.getProperty(name + "annotationRotation");
    axis.setAnnotationRotation(JCTypeConverter.toEnum(pval,
                                                      "annotationRotation",
                                                      strings, values,
                                                      axis.getAnnotationRotation()));
    // Set Annotation Rotation Angle
    pval = prop.getProperty(name + "annotationRotationAngle");
    if (pval != null) {
        double def = axis.getAnnotationRotationAngle(JCChartUtil.DEGREES);
        axis.setAnnotationRotationAngle(JCChartUtil.DEGREES,
                                        JCTypeConverter.toDouble(pval, def));
    }

    strings = JCChartEnumMappings.annotationMethod_strings;
    values  = JCChartEnumMappings.annotationMethod_values;
    pval = prop.getProperty(name + "annotationMethod");
    axis.setAnnotationMethod(JCTypeConverter.toEnum(pval, "annotationMethod",
                                                    strings, values,
                                                    axis.getAnnotationMethod()));

    strings = JCChartEnumMappings.placement_strings;
    values  = JCChartEnumMappings.placement_values;
    pval = prop.getProperty(name + "placement");
    if (pval != null) {
        axis.setPlacement(JCTypeConverter.toEnum(pval, "Placement",
                                                 strings, values,
                                                 axis.getPlacement()));
    }

    pval = prop.getProperty(name + "placementAxis");
    if (pval != null) {
        JCAxis plax = axis.getChartArea().findAxisByName(pval);
        if (plax != axis.getPlacementAxis()) {
            axis.setPlacementAxis(plax);
        }
    }

    // Set isReversed property
    pval = prop.getProperty(name + "reversed");
    axis.setReversed(JCTypeConverter.toBoolean(pval, axis.isReversed()));

    // Set placementLocation
    pval = prop.getProperty(name + "placementLocation");
    if (pval != null) {
        axis.setPlacementLocation(JCTypeConverter.toDouble(pval,
                                               axis.getPlacementLocation()));
    }

    // Set gap
    pval = prop.getProperty(name + "gap");
    axis.setGap(JCTypeConverter.toInt(pval, axis.getGap()));

    // time related stuff

    // TimeBase
    pval = prop.getProperty(name + "timeBase");
    if (pval != null) {
        axis.setTimeBase(JCTypeConverter.toDate(pval, axis.getTimeBase()));
    }

    // TimeUnit
    strings         = JCChartEnumMappings.timeUnit_strings;
    long[] lValues  = JCChartEnumMappings.timeUnit_values;
    pval = prop.getProperty(name + "timeUnit");
    if (pval != null) {
        axis.setTimeUnit(JCTypeConverter.toEnum(pval, "timeUnit", strings,
                                                lValues, axis.getTimeUnit()));
    }

    // TimeFormat
    pval = prop.getProperty(name + "timeFormat");
    if (pval != null) {
        axis.setTimeFormat(pval);
    }

    // TimeZone
    pval = prop.getProperty(name + "timeZone");
    if (pval != null) {
        axis.setTimeZone(TimeZone.getTimeZone(pval));
    }

    // Min
    pval = prop.getProperty(name + "min");
    if (pval != null) {
        axis.setMin(JCTypeConverter.toDouble(pval, axis.getMin()));
    }

    // Max
    pval = prop.getProperty(name + "max");
    if (pval != null) {
        axis.setMax(JCTypeConverter.toDouble(pval, axis.getMax()));
    }

    // Precision
    pval = prop.getProperty(name + "precision");
    if (pval != null) {
        axis.setPrecision(JCTypeConverter.toInt(pval, axis.getPrecision()));
    }

    // TickSpacing
    pval = prop.getProperty(name + "tickSpacing");
    if (pval != null) {
        axis.setTickSpacing(JCTypeConverter.toDouble(pval,
                                                     axis.getTickSpacing()));
    }

    // NumSpacing
    pval = prop.getProperty(name + "numSpacing");
    if (pval != null) {
        axis.setNumSpacing(JCTypeConverter.toDouble(pval,
                                                    axis.getNumSpacing()));
    }

    // Origin
    pval = prop.getProperty(name + "origin");
    if (pval != null) {
        axis.setOrigin(JCTypeConverter.toDouble(pval, axis.getOrigin()));
    }

    // OriginPlacement
    strings = JCChartEnumMappings.originPlacement_strings;
    values  = JCChartEnumMappings.originPlacement_values;
    pval = prop.getProperty(name + "originPlacement");
    if (pval != null) {
        axis.setOriginPlacement(JCTypeConverter.toEnum(pval, "originPlacement",
                                                       strings, values,
                                                       axis.getOriginPlacement()));
    }

    // Grid properties

    // GridVisible
    pval = prop.getProperty(name + "grid.visible");
    if (pval != null) {
        axis.setGridVisible(JCTypeConverter.toBoolean(pval,
                                                      axis.isGridVisible()));
    }

    // GridDefault
    pval = prop.getProperty(name + "grid.default");
    if (pval != null) {
        axis.setGridDefault(JCTypeConverter.toBoolean(pval,
                                                      axis.isGridDefault()));
    }

    // GridSpacing
    pval = prop.getProperty(name + "grid.spacing");
    if (pval != null) {
        axis.setGridSpacing(JCTypeConverter.toDouble(pval,
                                                     axis.getGridSpacing()));
    }

    PropertyLoadFactory.load(prop, axis.getGridStyle().getLineStyle(),
                             name + "grid.");

    Object o = prop.getPropertyObject(name + "valueLabels");
    if (o != null) {
        if (o instanceof String) {
            pval = (String)o;
            String sArr[] = JCTypeConverter.toStringList(pval, ';', false);
            for (int i = 0; i < sArr.length; i += 2) {
                double val = JCTypeConverter.toDouble(sArr[i], 0.0);
                axis.addValueLabel(new JCValueLabel(val, sArr[i+1]));
            }
        }
        else if (o instanceof JCValueLabel[]) {
            axis.setValueLabels((JCValueLabel[])o);
        }
    }

    pval = prop.getProperty(name + "editable");
    axis.setEditable(JCTypeConverter.toBoolean(pval, axis.isEditable()));

    PropertyLoadFactory.load(prop, axis.getTitle(), name + "title.");
    PropertyLoadFactory.load(prop, axis.getFormula(), name + "formula.");

    // JCAnno properties
    loadAnnoProperties(prop, name);

    pval = prop.getProperty(name + "useAnnoTicks");
    if (pval != null) {
        boolean useAnnoTicks = JCTypeConverter.toBoolean(pval, false);
        axis.setUseAnnoTicks(useAnnoTicks);
        if (useAnnoTicks) {
            updateAxisSpacingProperties();
        }
    }

    // JCGrid properties
    loadGridProperties(prop, name);
}

/**
 * Sets the JCAxis tickSpacing or numSpacing property to a value determined
 * from the JCAnno objects.
 */
private void updateAxisSpacingProperties() {
    JCAnno defLabels = axis.getDefaultLabels();
    if (defLabels != null) {
        if (!defLabels.getIncrementIsDefault()) {
            axis.setNumSpacing(defLabels.getIncrementValue());
        }
    }
    JCAnno defTicks = axis.getDefaultTicks();
    if (defTicks != null) {
        if (!defTicks.getIncrementIsDefault()) {
            axis.setTickSpacing(defTicks.getIncrementValue());
        }
    }
}

private boolean annoObjectSpecified(PropertyAccessModel prop, String name)
{
    if (prop == null || name == null) {
        return false;
    }
    boolean annoObjectFound = false;
    int annoNum = 1;
    while (annoNum < 100) {
        String annoName = prop.getProperty(name + "anno" +
                                           annoNum + ".annoName");
        if (annoName != null) {
            annoObjectFound = true;
            break;
        }
        annoNum++;
    }
    return(annoObjectFound);
}

protected void loadAnnoProperties(PropertyAccessModel prop, String name)
    throws JCIOException
{
    if (prop.getType().equals("HTML")) {
        String defLabelsAnno = prop.getProperty(name + "useDefaultLabels");
        String defTicksAnno = prop.getProperty(name + "useDefaultTicks");
        if (annoObjectSpecified(prop, name)) {
            // If the defLabelsAnno flag is false, or if the
            // flag doesn't exist, remove the default tick object
            // that was created during instantiation of the JCAxis.
            if (defLabelsAnno != null) {
                if (!JCTypeConverter.toBoolean(defLabelsAnno, false)) {
                    axis.removeDefaultLabels();
                }
            }
            else {
                axis.removeDefaultLabels();
            }

            // If the defTicksAnno flag is false, or if the
            // flag doesn't exist, remove the default tick object
            // that was created during instantiation of the JCAxis.
            if (defTicksAnno != null) {
                if (!JCTypeConverter.toBoolean(defTicksAnno, false)) {
                    axis.removeDefaultTicks();
                }
            }
            else {
                axis.removeDefaultTicks();
            }

            // Check if there are multiple anno objects
            // There is a limit of 100 anno objects
            int annoNum = 1;
            while (annoNum < 100) {
                String annoName = prop.getProperty(name + "anno" +
                                                   annoNum + ".annoName");
                if (annoName != null) {
                    JCAnno anno = new JCAnno();
                    PropertyLoadFactory.load(prop,
                                             anno, name + annoName + ".");
                    if (!axis.containsAnnotation(anno.getName())) {
                        axis.addAnnotation(anno);
                    }
                }
                annoNum++;
            }
        }
        else {
            if (defLabelsAnno != null) {
                if (!JCTypeConverter.toBoolean(defLabelsAnno, false)) {
                    axis.removeDefaultLabels();
                }
                if (defTicksAnno == null) {
                    // If useDefaultLabels is specified, but
                    // useDefaultTicks isn't specified, remove the default
                    // tick object.
                    axis.removeDefaultTicks();
                }
            }
            if (defTicksAnno != null) {
                if (!JCTypeConverter.toBoolean(defTicksAnno, false)) {
                    axis.removeDefaultTicks();
                }
                if (defLabelsAnno == null) {
                    // If useDefaultTicks is specified, but
                    // useDefaultLabels isn't specified, remove the default
                    // label object.
                    axis.removeDefaultLabels();
                }
            }
        }
    }
    else {
        // Loading properties from XML
        Boolean useDefaultLabels = null;
        Boolean useDefaultTicks = null;
        Object o = prop.getPropertyObject(name + "useDefaultLabels");
        if (o != null && o instanceof String) {
            useDefaultLabels =
                new Boolean(JCTypeConverter.toBoolean((String)o, false));
        }
        o = prop.getPropertyObject(name + "useDefaultTicks");
        if (o != null && o instanceof String) {
            useDefaultTicks =
                new Boolean(JCTypeConverter.toBoolean((String)o, false));
        }
        o = prop.getPropertyObject(name + "annoList");
        if (o != null && o instanceof ArrayList) {
            // Non-default tick objects have been specified.
            ArrayList annoList = (ArrayList)o;

            // If the useDefaultLabels flag is false, or if the
            // flag doesn't exist, remove the default labels object
            // that was created during instantiation of the JCAxis.
            if (useDefaultLabels != null) {
                if (!useDefaultLabels.booleanValue()) {
                    axis.removeDefaultLabels();
                }
            }
            else {
                axis.removeDefaultLabels();
            }

            // If the useDefaultTicks flag is false, or if the
            // flag doesn't exist, remove the default ticks object
            // that was created during instantiation of the JCAxis.
            if (useDefaultTicks != null) {
                if (!useDefaultTicks.booleanValue()) {
                    axis.removeDefaultTicks();
                }
            }
            else {
                axis.removeDefaultTicks();
            }

            for (int i=0; i < annoList.size(); i++) {
                JCAnno anno = (JCAnno)annoList.get(i);
                // Make sure we don't add the same JCAnno more than once.
                if (!axis.containsAnnotation(anno)) {
                    axis.addAnnotation(anno);
                }
            }
        }
        else {
            // There are no non-default anno objects.
            if (useDefaultLabels != null) {
                if (!useDefaultLabels.booleanValue()) {
                    axis.removeDefaultLabels();
                }
                if (useDefaultTicks == null) {
                    // If useDefaultLabels is specified, but
                    // useDefaultTicks isn't specified, remove the default
                    // minor tick object.
                    axis.removeDefaultTicks();
                }
            }
            if (useDefaultTicks != null) {
                if (!useDefaultTicks.booleanValue()) {
                    axis.removeDefaultTicks();
                }
                if (useDefaultLabels == null) {
                    // If useDefaultTicks is specified, but
                    // useDefaultLabels isn't specified, remove the default
                    // major tick object.
                    axis.removeDefaultLabels();
                }
            }
        }
    }
    updateAxisSpacingProperties();
}

private boolean gridObjectSpecified(PropertyAccessModel prop, String name)
{
    if (prop == null || name == null) {
        return(false);
    }
    boolean gridObjectFound = false;
    int gridNum = 1;
    while (gridNum < 100) {
        String gridName = prop.getProperty(name + "grid" +
                                           gridNum + ".gridName");
        if (gridName != null) {
            gridObjectFound = true;
            break;
        }
        gridNum++;
    }
    return(gridObjectFound);
}

protected void loadGridProperties(PropertyAccessModel prop, String name)
    throws JCIOException
{
    if (prop.getType().equals("HTML")) {
        String defGrid = prop.getProperty(name + "useDefaultGrid");
        if (gridObjectSpecified(prop, name)) {
            // Check if there are multiple grid objects
            // There is a limit of 100 grid objects
            int gridNum = 1;
            while (gridNum < 100) {
                String gridName =
                    prop.getProperty(name + "grid" + gridNum + ".gridName");
                if (gridName != null) {
                    // If the useDefaultGrid flag is false, or if the
                    // flag doesn't exist, remove the default grid object
                    // that was created during instantiation of the JCAxis.
                    if (defGrid != null) {
                        if (!JCTypeConverter.toBoolean(defGrid, false)) {
                            axis.removeDefaultGrid();
                        }
                    }
                    else {
                        axis.removeDefaultGrid();
                    }
                    JCGrid grid = new JCGrid(axis);
                    PropertyLoadFactory.load(prop, grid,
                                             name + gridName + ".");
                    if (!axis.containsGrid(grid.getName())) {
                        axis.addGrid(grid);
                    }
                }
                gridNum++;
            }
        }
        else if (defGrid != null) {
            if (!JCTypeConverter.toBoolean(defGrid, false)) {
                axis.removeDefaultGrid();
            }
        }
    }
    else {
        // Data was loaded from XML.
        Boolean useDefaultGrid = null;
        Object o = prop.getPropertyObject(name + "useDefaultGrid");
        if (o != null && o instanceof String) {
            boolean value = JCTypeConverter.toBoolean((String)o, false);
            useDefaultGrid = new Boolean(value);
        }

        o = prop.getPropertyObject(name + "gridList");
        if (o != null && o instanceof ArrayList) {
            ArrayList gridList = (ArrayList)o;

            // If the useDefaultGrid flag is false, or if the
            // flag doesn't exist, remove the default grid object
            // that was created during instantiation of the JCAxis.
            if (useDefaultGrid != null) {
                if (!useDefaultGrid.booleanValue()) {
                    axis.removeDefaultGrid();
                }
            }
            else {
                axis.removeDefaultGrid();
            }

            for (int i=0; i < gridList.size(); i++) {
                JCGrid grid = (JCGrid)gridList.get(i);
                grid.setParent(axis);
                // Get the JCLineStyle for this grid object.
                String lsName = name + "grid" + i + ".";
                PropertyLoadFactory.load(prop,
                                         grid.getGridStyle().getLineStyle(),
                                         lsName);
                // Make sure we don't add the same JCGrid more than once.
                if (!axis.containsGrid(grid)) {
                    axis.addGrid(grid);
                }
            }
        }
        else if (useDefaultGrid != null) {
            if (!useDefaultGrid.booleanValue()) {
                axis.removeDefaultGrid();
            }
        }
    }
}

}
