/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCAxisTitlePropertySave.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:53 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.JCAxisTitle;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.io.JCIOException;
import com.klg.jclass.util.legend.JCLegendEnumMappings;

/**
 * Responsible for saving properties of <code>JCAxisTitle</code>
 * objects to a destiation specified by <code>PropertyPersistorModel</code>.
 * Not intended for public use.
 *
 * @see com.klg.jclass.chart.JCAxisTitle
 * @see com.klg.jclass.chart.property.JCAxisTitlePropertyLoad
 */
public class JCAxisTitlePropertySave extends ChartTextPropertySave {

protected JCAxisTitle title = null;
protected JCAxisTitle defaultTitle = null;

/**
 * Sets the source component whose properties are to be saved.
 * @param o the new source component
 */
public void setSource(Object o)
{
	super.setSource(o);
	if (o instanceof JCAxisTitle) {
		this.title = (JCAxisTitle)o;
	}
}

/**
 * Sets default object instance.  This object is typically used by implementors
 * to determine whether a property really needs to be saved.
 * @param def the new default object
 */
public void setDefault(Object def)
{
	super.setDefault(def);
	if (def instanceof JCAxisTitle) {
		this.defaultTitle = (JCAxisTitle)def;
	}
}

/**
 * Checks whether there are any properties different from their default.  If so,
 * then this saver has information to write.
 * @param prop property persistor object
 * @return whether any properties are different from their default
 */
public boolean checkProperties(PropertyPersistorModel prop)
{
	if (title == null || defaultTitle == null) {
		return(false);
	}

	if (!title.getPlacementIsDefault() || super.checkProperties(prop)) {
		return(true);
	}

	return(false);
}

/**
 * Saves HTML parameters from the current param source;
 * click the method name to see which properties are saved.<p>
 * Sets the following properties:
 * <pre>
 *   placement (enum)
 * </pre>
 * @param prop property persistor object
 * @param name name of object to be saved
 * @param tab the tab indent for this group of properties
 */
public void saveProperties(PropertyPersistorModel prop, String name, int tab)
    throws JCIOException
{
	if (title == null || defaultTitle == null) {
		System.out.println("FAILURE: No axis title set");
		return;
	}

	// Check if something to do.
	if (!checkProperties(prop)) {
		return;
	}

	int newTab = prop.writeBegin("axis-title", tab);

	// Set placement
	String[] strings = JCLegendEnumMappings.anchor_strings;
	int[]    values  = JCLegendEnumMappings.anchor_values;
	if (!title.getPlacementIsDefault()) {
		prop.writeProperty(name, "placement", newTab,
				           JCTypeConverter.fromEnum(title.getPlacement(),
									                strings, values));
	}

	// We continue the axis-title element in the super class
	// ChartTextPropertySave.  Therefore we don't call checkProperties here
	// because if we get here we want to finish the partially written
	// axis-title
	super.saveProperties(prop, name, tab);

	prop.writeEnd("axis-title", tab, true, false);
}

}
