/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCBasePersistor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:53 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

/**
 * Abstract property persistor class that saves property settings. Subclasses
 * define methods like the <code>writeProperty</code> method which determine how
 * to write in the appropriate format for a specific type of file.
 *
 * @see com.klg.jclass.chart.property.PropertyPersistorModel
 */
public abstract class JCBasePersistor implements PropertyPersistorModel {

protected Object     source = null;
protected Object     helperObject = null;

/**
 * Constructor which provides the source (which holds the properties to write)
 * and a SaveProperties object which stores information on how to write the
 * chart (such as data file information)
 * @param source The source (the Chart)
 */
public JCBasePersistor(Object source)
{
	setSource(source);
}

/**
 * Sets the source object (which holds the properties to write)
 * @param source The source (the Chart)
 */
public void setSource(Object source)
{
    this.source = source;
}

/**
 * Gets the source object (which holds the properties to write)
 * @return The source object (the Chart)
 */
public Object getSource()
{
    return(source);
}

/**
 * Gets the persistor type String.
 * @return the persistor type String
 */
public abstract String getType();

/**
 * Gets the peristor subDirectory (if any).
 * @return the persistor subDirectory (null if no subDirectory)
 */
public abstract String getSubDirectory();

/**
 * Store a helper object.  This is used to facilitate communication
 * between PropertyLoad/Save classes.  For example, a Load/Save class
 * may set a helper object that it knows will be needed by the
 * Load/Save class that will be invoked by its call to save().
 * @param helper Helper object to store.
 */
public void setHelperObject(Object helper)
{
	this.helperObject = helper;
}

/**
 * Get the helper object set with setHelperObject().  If no helper
 * object is set, this is null.
 */
public Object getHelperObject()
{
	return helperObject;
}

/**
 * Persists the String property using the value provided.
 *
 * @param name class hierarchy name
 * @param property property name
 * @param tab the amount to tab before writing property
 * @param value actual object to persist
 */
public abstract void writeProperty(String name, String property, int tab,
                                   Object value);

/**
 * Writes a beginning to a list of properties.
 * @param name the name of the group of properties
 * @param tab the amount to tab before writing the beginning
 * @return a new tabstop based on output String
 */
public abstract int writeBegin(String name, int tab);

/**
 * Writes an ending to a list of properties.
 * @param name the name of the group of properties
 * @param tab the amount to tab before writing the ending
 * @param newLine if <code>true</code>, a <code>newLine</code> character is
 *     printed at the end
 * @param partial Is this a partial ending?
 */
public abstract void writeEnd(String name, int tab, boolean newLine,
                              boolean partial);

/**
 * Writes the String to the output stream.
 * @param s the String to write
 * @param tab the number of spaces to tab over before writing
 */
public abstract void writeString(String s, int tab);

/**
 * Expands the given String to escape reserved characters.
 * @param s the String to expand
 * @return the newly expanded String
 */
public abstract String expandText(String s);

}
