/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCCandleChartFormatPropertyLoad.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:54 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.JCCandleChartFormat;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.io.JCIOException;

/**
 * Responsible for loading properties of <code>JCCandleChartFormat</code>
 * objects using a specified <code>PropertyAcessModel</code>. Not
 * intended for public use.
 *
 * @see com.klg.jclass.chart.JCCandleChartFormat
 * @see com.klg.jclass.chart.property.JCCandleChartFormatPropertySave
 */
public class JCCandleChartFormatPropertyLoad implements PropertyLoadModel {

protected JCCandleChartFormat format = null;

/**
 * Sets the source object (the object on which to set properties).
 *
 * @param o object that is loading properties
 */
public void setSource(Object o)
{
	if (o instanceof JCCandleChartFormat) {
		this.format = (JCCandleChartFormat)o;
	}
}

/**
 * Loads properties using the given property accessor object.
 *
 * @param prop property access object
 * @param name name of object to be loaded
 * @throws JCIOException
 */
public void loadProperties(PropertyAccessModel prop, String name)
    throws JCIOException
{
	if (format == null) {
		System.out.println("FAILURE: No candle format set");
		return;
	}

	// Load style data.  Note that this may be redundant information,
	// possibly having been already loaded with the data view styles.
	// For this reason, it isn't saved by the save class from this
	// class.  Still, we load it for backwards compatibility and to
	// allow a way to get this information into the chart that is
	// a little easier understood.

	int n = format.getParent().getNumSeriesPerData();
	for (int i = 0; i < format.getParent().getNumSeries(); i += n) {
		String prefix = name + "series" + ((i/n) + 1);
		PropertyLoadFactory.load(prop, format.getCandleOutlineStyle(i/n),
								 prefix + ".outline.");
		PropertyLoadFactory.load(prop, format.getRisingCandleStyle(i/n),
								 prefix + ".rising.");
		PropertyLoadFactory.load(prop, format.getFallingCandleStyle(i/n),
								 prefix + ".falling.");
		PropertyLoadFactory.load(prop, format.getHiloStyle(i/n),
								 prefix + ".hilo.");
	}

	String pval = prop.getProperty(name + "complex");
	format.setComplex(JCTypeConverter.toBoolean(pval, format.isComplex()));
}

}
