/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCCandleChartFormatPropertySave.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:54 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.JCCandleChartFormat;
import com.klg.jclass.util.io.JCIOException;

/**
 * Responsible for saving properties of <code>JCCandleChartFormat</code>
 * objects to a destiation specified by <code>PropertyPersistorModel</code>.
 * Not intended for public use.
 *
 * @see com.klg.jclass.chart.JCCandleChartFormat
 * @see com.klg.jclass.chart.property.JCCandleChartFormatPropertyLoad
 */
public class JCCandleChartFormatPropertySave implements PropertySaveModel {

protected JCCandleChartFormat format = null;
protected JCCandleChartFormat defaultFormat = null;

/**
 * Sets the source component whose properties are to be saved.
 * @param o the new source component
 */
public void setSource(Object o)
{
	if (o instanceof JCCandleChartFormat) {
		this.format = (JCCandleChartFormat)o;
	}
}

/**
 * Sets default object instance.  This object is typically used by implementors
 * to determine whether a property really needs to be saved.
 * @param def the new default object
 */
public void setDefault(Object def)
{
	if (def instanceof JCCandleChartFormat) {
		this.defaultFormat = (JCCandleChartFormat)def;
	}
}

/**
 * Checks whether there are any properties different from their default.  If so,
 * then this saver has information to write.
 * @param prop property persistor object
 * @return whether any properties are different from their default
 */
public boolean checkProperties(PropertyPersistorModel prop)
{
    if (format == null || defaultFormat == null) {
		return(false);
    }

	if (format.isComplex() != defaultFormat.isComplex()) {
		return(true);
	}
	return(false);
}

/**
 * Click the method name to see which properties are saved.<p>
 * Saves the following properties:
 * <pre>
 *   outline.* (See JCChartStyle)
 *   rising.* (See JCChartStyle)
 *   falling.* (See JCChartStyle)
 *   hilo.* (See JCChartStyle)
 *   complex (boolean)
 * </pre>
 * @param prop property persistor object
 * @param name name of object to be saved
 * @param tab the tab indent for this group of properties
 * @throws JCIOException
 */
public void saveProperties(PropertyPersistorModel prop, String name, int tab)
    throws JCIOException
{
	if (format == null || defaultFormat == null) {
		System.out.println("FAILURE: No candle format set");
		return;
	}

	// Check if something to do.
	if (!checkProperties(prop)) {
		return;
	}

	int newTab = prop.writeBegin("candle-format", tab);

    // Do not save hilo style and tick size information from format
    // class.  This is redundant information, having already been
    // saved with the chart styles of the series.  (It is loaded by
    // the loader class however, to maintain backwards compatibility
    // and to allow slightly more obvious ways of getting that info
    // into the chart.)

	// Set complex
	if (format.isComplex() != defaultFormat.isComplex()) {
		prop.writeProperty(name, "complex", newTab,
		                   new Boolean(format.isComplex()));
	}
	prop.writeEnd(null, tab, true, false);
}

}
