/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCChartAreaPropertyLoad.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:54 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCChartArea;
import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.ImageMapInfo;
import com.klg.jclass.util.io.JCIOException;

/**
 * Responsible for loading properties of <code>JCChartArea</code>
 * objects using a specified <code>PropertyAcessModel</code>. Not
 * intended for public use.
 *
 * @see com.klg.jclass.chart.JCChartArea
 * @see com.klg.jclass.chart.property.JCChartAreaPropertySave
 */
public class JCChartAreaPropertyLoad extends ComponentPropertyLoad {

protected JCChartArea chartArea = null;

/**
 * Sets the source object (the object on which to set properties).
 *
 * @param o object that is loading properties
 */
public void setSource(Object o)
{
	super.setSource(o);
	if (o instanceof JCChartArea) {
		this.chartArea = (JCChartArea)o;
	}
}

/**
 * Loads properties using the given property accessor object.
 *
 * @param prop property access object.
 * @param name name of object to be loaded.
 * @throws JCIOException
 */
public void loadProperties(PropertyAccessModel prop, String name)
    throws JCIOException
{
	super.loadProperties(prop, name);

	if (chartArea == null) {
		System.out.println("FAILURE: No chart area set");
		return;
	}

	// Set axis bounding box
	String pval = prop.getProperty(name + "axisBoundingBox");
	chartArea.setAxisBoundingBox(JCTypeConverter.toBoolean(pval,
										   chartArea.getAxisBoundingBox()));

    // Set 3d depth
	pval = prop.getProperty(name + "depth");
	chartArea.setDepth(JCTypeConverter.toInt(pval, chartArea.getDepth()));

    // Set 3d elevation
	pval = prop.getProperty(name + "elevation");
	chartArea.setElevation(JCTypeConverter.toInt(pval,
												 chartArea.getElevation()));

    // Set 3d rotation
	pval = prop.getProperty(name + "rotation");
	chartArea.setRotation(JCTypeConverter.toInt(pval,
												chartArea.getRotation()));

    // Set angle unit
	boolean isXML    = prop.getType().equals("XML");
	String[] strings = isXML ? JCChartEnumMappings.angleUnit_xml_strings
		                     : JCChartEnumMappings.angleUnit_strings;
	int[] values     = JCChartEnumMappings.angleUnit_values;
	pval = prop.getProperty(name + "angleUnit");
	if (pval != null) {
		chartArea.setAngleUnit(JCTypeConverter.toEnum(pval, "angle unit",
									                  strings, values,
									                  chartArea.getAngleUnit()));
	}

    // Set fastaction
	pval = prop.getProperty(name + "fastAction");
	chartArea.setFastAction(JCTypeConverter.toBoolean(pval,
												  chartArea.getFastAction()));

    // Load the chartArea's imageMapInfo, if set.
    ImageMapInfo info = new ImageMapInfo();
    PropertyLoadFactory.load(prop, info, name + "imageMapInfo.");
    if (!info.isEmpty()) {
        chartArea.setImageMapInfo(info);
    }

	PropertyLoadFactory.load(prop, chartArea.getPlotArea(), name + "plotArea.");

	// Load default x axis
	JCAxis ax = chartArea.getXAxis(0);
	ax.setName("xaxis");
	PropertyLoadFactory.load(prop, ax, ax.getName() + ".");

	// Load default y axis
	ax = chartArea.getYAxis(0);
	ax.setName("yaxis");
	PropertyLoadFactory.load(prop, ax, ax.getName() + ".");

    // Get any additional axis. There is a limit of 100 extra axis.
	// Create needed axes but do not load properties.  That is done in
	// the parent loading class because data loading needs to have axes
	// existing but without properties set.
    int axis = 1;
    while (axis < 100) {
		String xAxisName = prop.getProperty(name + "xaxisName" + axis);
        if (xAxisName != null) {
            JCAxis xaxis = new JCAxis(chartArea, false, JCAxis.VALUE);
			chartArea.setXAxis(chartArea.getXAxes().size(), xaxis);
			xaxis.setName(xAxisName);
        }
		String yAxisName = prop.getProperty(name + "yaxisName" + axis);
        if (yAxisName != null) {
            JCAxis yaxis = new JCAxis(chartArea, true, JCAxis.VALUE);
			chartArea.setYAxis(chartArea.getYAxes().size(), yaxis);
			yaxis.setName(yAxisName);
        }
        axis++;
    }

    // Set horizontal action axis
	pval = prop.getProperty(name + "horizActionAxis");
	if (pval != null) {
		ax = chartArea.findAxisByName(pval);
		if (ax != null) {
			chartArea.setHorizActionAxis(ax);
		}
	}

	// Set vertical action axis
	pval = prop.getProperty(name + "vertActionAxis");
	if (pval != null) {
		ax = chartArea.findAxisByName(pval);
		if (ax != null) {
			chartArea.setVertActionAxis(ax);
		}
	}
}

}
