/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCChartAreaPropertySave.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:54 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.JCAxis;
import com.klg.jclass.chart.JCChart;
import com.klg.jclass.chart.JCChartArea;
import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.ImageMapInfo;
import com.klg.jclass.util.io.JCIOException;

import javax.swing.*;
import java.awt.*;
import java.util.ListIterator;

/**
 * Responsible for saving properties of <code>JCChartArea</code>
 * objects to a destiation specified by <code>PropertyPersistorModel</code>.
 * Not intended for public use.
 *
 * @see com.klg.jclass.chart.JCChartArea
 * @see com.klg.jclass.chart.property.JCChartAreaPropertyLoad
 */
public class JCChartAreaPropertySave extends ComponentPropertySave {

protected JCChartArea chartArea = null;
protected JCChartArea defaultChartArea = null;

/**
 * Sets the source component whose properties are to be saved.
 * @param o the new source component
 */
public void setSource(Object o)
{
	super.setSource(o);
	if (o instanceof JCChartArea) {
		this.chartArea = (JCChartArea)o;
	}
}

/**
 * Sets default object instance.  This object is typically used by implementors
 * to determine whether a property really needs to be saved.
 * @param def the new default object
 */
public void setDefault(Object def)
{
	super.setDefault(def);
	if (def instanceof JCChartArea) {
		this.defaultChartArea = (JCChartArea)def;
	}
}

/**
 * Checks whether there are any properties different from their default.  If so,
 * then this saver has information to write.
 * @return whether any properties are different from their default
 */
public boolean checkProperties()
{
	return(true);
}

/**
 * Saves object properties using the specified persistance object.
 *
 * @param prop Property persistor object
 * @param name Name of object to be saved
 * @param tab The tab indent for this group of properties
 * @throws JCIOException
 */
public void saveProperties(PropertyPersistorModel prop, String name, int tab)
    throws JCIOException
{
	if (chartArea == null || defaultChartArea == null) {
		System.out.println("FAILURE: No chart area set");
		return;
	}

	// First, rename axes for more convenient saving.  Note this needs to
	// be done before axes are saved and horizActionAxis and vertActionAxis
	// properties are saved.
	int axisIndex = 0;
	ListIterator li = chartArea.getXAxes().listIterator();
	while (li.hasNext()) {
		JCAxis axis = (JCAxis)li.next();
		String axname = "xaxis" + (axisIndex == 0 ? "" : "" + axisIndex);
		axis.setName(axname);
		axisIndex++;
	}
	axisIndex = 0;
	li = chartArea.getYAxes().listIterator();
	while (li.hasNext()) {
		JCAxis axis = (JCAxis)li.next();
		String axname = "yaxis" +  (axisIndex == 0 ? "" : "" + axisIndex);
		axis.setName(axname);
		axisIndex++;
	}

	int newTab = prop.writeBegin("chart-area", tab);

	// Set axis bounding box
	if (chartArea.getAxisBoundingBox() != defaultChartArea.getAxisBoundingBox())
	{
		prop.writeProperty(name, "axisBoundingBox", newTab,
						   new Boolean(chartArea.getAxisBoundingBox()));
	}

    // Set 3d depth
	if (chartArea.getDepth() != defaultChartArea.getDepth()) {
		prop.writeProperty(name, "depth", newTab,
						   new Integer(chartArea.getDepth()));
	}

    // Set 3d elevation
	if (chartArea.getElevation() != defaultChartArea.getElevation()) {
		prop.writeProperty(name, "elevation", newTab,
						   new Integer(chartArea.getElevation()));
	}

    // Set 3d rotation
	if (chartArea.getRotation() != defaultChartArea.getRotation()) {
		prop.writeProperty(name, "rotation", newTab,
						   new Integer(chartArea.getRotation()));
	}

    // Set angle unit
	boolean isXML    = prop.getType().equals("XML");
	String[] strings = isXML ? JCChartEnumMappings.angleUnit_xml_strings
		                     : JCChartEnumMappings.angleUnit_strings;
	int[] values     = JCChartEnumMappings.angleUnit_values;
	if (chartArea.getAngleUnit() != defaultChartArea.getAngleUnit()) {
		prop.writeProperty(name, "angleUnit", newTab,
		                   JCTypeConverter.fromEnum(chartArea.getAngleUnit(),
						                            strings, values));
	}

    // Set fastaction
	if (chartArea.getFastAction() != defaultChartArea.getFastAction()) {
		prop.writeProperty(name, "fastAction", newTab,
						   new Boolean(chartArea.getFastAction()));
	}

    // Set horizontal action axis
	JCAxis hAxis = chartArea.getHorizActionAxis();
	if (hAxis != null && !hAxis.getName().equals("xaxis")) {
		prop.writeProperty(name, "horizActionAxis", newTab,
		                   prop.expandText(hAxis.getName()));
	}

	// Set vertical action axis
	JCAxis vAxis = chartArea.getVertActionAxis();
	if (vAxis != null && !vAxis.getName().equals("yaxis")) {
		prop.writeProperty(name, "vertActionAxis", newTab,
		                   prop.expandText(vAxis.getName()));
	}

	prop.writeEnd(null, tab, true, true);

	if (super.checkProperties(prop)) {
	    super.saveProperties(prop, name,
		                     tab + PropertySaveFactory.tabIncrement);
	}

	Container parent = component.getParent();
	if (parent != null && parent instanceof JCChart) {
	    saveLayoutHints(prop, (JCChart)parent, (JComponent)component,
		                name, tab + PropertySaveFactory.tabIncrement);
	}

    // Save ImageMapInfo properties
    PropertySaveFactory.save(prop, chartArea.getImageMapInfo(),
                             new ImageMapInfo(), name + "imageMapInfo.", tab);

	PropertySaveFactory.save(prop, chartArea.getPlotArea(),
							 defaultChartArea.getPlotArea(),
							 name + "plotArea.", tab);

	// Save x axes
	JCAxis defaultAxis = defaultChartArea.getXAxis(0);
	li = chartArea.getXAxes().listIterator();
	while (li.hasNext()) {
		JCAxis axis = (JCAxis)li.next();
		if (axis != null) {
			PropertySaveFactory.save(prop, axis, defaultAxis,
			                         axis.getName() + ".", tab);
		}
	}

	// Save y axes
	defaultAxis = defaultChartArea.getYAxis(0);
	li = chartArea.getYAxes().listIterator();
	while (li.hasNext()) {
		JCAxis axis = (JCAxis)li.next();
		if (axis != null) {
			PropertySaveFactory.save(prop, axis, defaultAxis,
			                         axis.getName() + ".", tab);
		}
	}

	prop.writeEnd("chart-area", tab, true, false);
}

}
