/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCChartLabelPropertyLoad.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:54 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.chart.JCChartLabel;
import com.klg.jclass.chart.JCDataCoord;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.io.JCIOException;
import com.klg.jclass.util.swing.JCSwingTypeConverter;

/**
 * Responsible for loading properties of <code>JCChartLabel</code>
 * objects using a specified <code>PropertyAcessModel</code>. Not
 * intended for public use.
 *
 * @see com.klg.jclass.chart.JCChartLabel
 * @see com.klg.jclass.chart.property.JCChartLabelPropertySave
 */
public class JCChartLabelPropertyLoad implements PropertyLoadModel {

protected JCChartLabel label = null;

/**
 * Sets the source object (the object on which to set properties).
 *
 * @param o object that is loading properties
 */
public void setSource(Object o)
{
	if (o instanceof JCChartLabel) {
		this.label = (JCChartLabel)o;
	}
}

/**
 * Loads properties using the given property accessor object.
 *
 * @param prop property access object.
 * @param name name of object to be loaded.
 * @throws JCIOException
 */
public void loadProperties(PropertyAccessModel prop, String name)
    throws JCIOException
{
	if (label == null) {
		System.out.println("FAILURE: No label set");
		return;
	}

	// First, sneak in component-based properties
	PropertyLoadFactory.load(prop, label.getComponent(), name);

	//this may seem redundant (seeing as there is one in JLabelPropertyLoad)
	//but it's needed to ensure that the chart labels knows enough to
	//not generate an automatic label
	String pval = prop.getProperty(name + "text");
	label.setText(pval);

	// Set anchor method
	String[] strings = JCChartEnumMappings.chartLabelAnchor_strings;
	int[] values     = JCChartEnumMappings.chartLabelAnchor_values;
	pval = prop.getProperty(name + "anchor");
	label.setAnchor(JCTypeConverter.toEnum(pval, "anchor", strings, values,
								           label.getAnchor()));

	// Set attach method
	boolean isXML = prop.getType().equals("XML");
	strings       = isXML ? JCChartEnumMappings.attachMethod_xml_strings
		                  : JCChartEnumMappings.attachMethod_strings;
	values        = JCChartEnumMappings.attachMethod_values;
	pval = prop.getProperty(name + "attachMethod");
	label.setAttachMethod(JCTypeConverter.toEnum(pval, "AttachMethod",
	                                             strings, values,
												 label.getAttachMethod()));

	// Set attach coordinates
	pval = prop.getProperty(name + "coord");
	label.setCoord(JCSwingTypeConverter.toPoint(pval, label.getCoord()));

	// DataView

	// Data Coord

	// Data Index

	// Set offset values
	pval = prop.getProperty(name + "offset");
	label.setOffset(JCSwingTypeConverter.toPoint(pval, label.getOffset()));

	// Set connected
	pval = prop.getProperty(name + "connected");
	label.setConnected(JCTypeConverter.toBoolean(pval, label.isConnected()));

	// Set dwell label
	pval = prop.getProperty(name + "dwellLabel");
	label.setDwellLabel(JCTypeConverter.toBoolean(pval, label.isDwellLabel()));

	// Set dataAttachX and dataAttachY
	JCDataCoord dc = label.getDataCoord();
	double x = dc.getX();
	double y = dc.getY();
	pval = prop.getProperty(name + "dataAttachX");
	x = JCTypeConverter.toDouble(pval, x);
	pval = prop.getProperty(name + "dataAttachY");
	y = JCTypeConverter.toDouble(pval, y);
	label.setDataCoord(new JCDataCoord(x, y));
}

}
