/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCChartLabelPropertySave.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:54 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.chart.JCChartLabel;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.io.JCIOException;

import java.awt.*;

/**
 * Responsible for saving properties of <code>JCChartLabel</code>
 * objects to a destiation specified by <code>PropertyPersistorModel</code>.
 * Not intended for public use.
 *
 * @see com.klg.jclass.chart.JCChartLabel
 * @see com.klg.jclass.chart.property.JCChartLabelPropertyLoad
 */
public abstract class JCChartLabelPropertySave implements PropertySaveModel,
												          PropertyRefreshModel
{

protected JCChartLabel  label = null;
protected JCChartLabel  defaultLabel = null;
protected static int    dataIndex = 1;

/**
 * Sets the source component whose properties are to be saved.
 * @param o the new source component
 */
public void setSource(Object o)
{
	if (o instanceof JCChartLabel) {
		this.label = (JCChartLabel)o;
	}
}

/**
 * Sets default object instance.  This object is typically used by implementors
 * to determine whether a property really needs to be saved.
 * @param def the new default object
 */
public void setDefault(Object def)
{
	if (def instanceof JCChartLabel) {
		this.defaultLabel = (JCChartLabel)def;
	}
}

/**
 * Checks whether there are any properties different from their default.  If so,
 * then this saver has information to write.
 * @param prop property persistor object
 * @return whether any properties are different from their default
 */
public boolean checkProperties(PropertyPersistorModel prop)
{
	return(true);
}

/**
 * Refreshes properties using the specified persistor model
 *
 * @param prop Property peristor object.
 * @param name Name of object to persist.
 */
public void refreshProperties(PropertyPersistorModel prop, String name)
{
	dataIndex = 1;
}

/**
 * Saves object properties using the specified persistance object.
 *
 * @param prop Property persistor object
 * @param name Name of object to be saved
 * @param tab The tab indent for this group of properties
 * @throws JCIOException
 */
public void saveProperties(PropertyPersistorModel prop, String name, int tab)
    throws JCIOException
{
	if (label == null || defaultLabel == null) {
		System.out.println("FAILURE: No label set");
		return;
	}

	int newTab = prop.writeBegin("chart-label", tab);

	saveDataViewProperty(prop, name, newTab);

	// Set anchor
	String[] strings = JCChartEnumMappings.chartLabelAnchor_strings;
	int[] values     = JCChartEnumMappings.chartLabelAnchor_values;
	if (label.getAnchor() != defaultLabel.getAnchor()) {
		prop.writeProperty(name, "anchor", newTab,
						   JCTypeConverter.fromEnum(label.getAnchor(),
								                    strings, values));
	}

	// Set attachMethod
	boolean isXML = prop.getType().equals("XML");
	strings       = isXML ? JCChartEnumMappings.attachMethod_xml_strings
		                  : JCChartEnumMappings.attachMethod_strings;
	values        = JCChartEnumMappings.attachMethod_values;
	if (label.getAttachMethod() != defaultLabel.getAttachMethod()) {
		prop.writeProperty(name, "attachMethod", newTab,
						   JCTypeConverter.fromEnum(label.getAttachMethod(),
											        strings, values));
	}

	// Set connected
	if (label.isConnected() != defaultLabel.isConnected()) {
		prop.writeProperty(name, "connected", newTab,
						   new Boolean(label.isConnected()));
	}

	// Set dwellLabel
	if (label.isDwellLabel() != defaultLabel.isDwellLabel()) {
		prop.writeProperty(name, "dwellLabel", newTab,
						   new Boolean(label.isDwellLabel()));
	}
	prop.writeEnd(null, tab, true, true);

	// Component-based properties
	prop.setHelperObject(label);
	PropertySaveFactory.save(prop, label.getComponent(),
							 defaultLabel.getComponent(), name, tab);
	prop.setHelperObject(null);

	// Set offset
	Point current = label.getOffset();
	Point dflt = defaultLabel.getOffset();
	if (current.x != dflt.x || current.y != dflt.y) {
	    if (prop.getType().equals("HTML")) {
		    prop.writeProperty(name, "offset", newTab,
		                       current.x + "," + current.y);
	    }
	    else {
			newTab = prop.writeBegin("offset",
			                         tab + PropertySaveFactory.tabIncrement);
			prop.writeProperty(name, "x", newTab, new Integer(current.x));
			prop.writeProperty(name, "y", newTab, new Integer(current.y));
			prop.writeEnd(null, tab, true, false);
	    }
	}

	saveAttachProperties(prop, name, tab);
	prop.writeEnd("chart-label", tab, true, false);
}

public abstract void saveDataViewProperty(PropertyPersistorModel prop,
                                          String name, int tab);

public abstract void saveAttachProperties(PropertyPersistorModel prop,
                                          String name, int tab)
    throws JCIOException;

}
