/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCDataIndexPropertyLoad.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:54 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.JCDataIndex;
import com.klg.jclass.chart.JCPieChartFormat;
import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.io.JCIOException;

/**
 * Responsible for loading properties of <code>JCDataIndex</code>
 * objects using a specified <code>PropertyAcessModel</code>. Not
 * intended for public use.
 *
 * @see com.klg.jclass.chart.JCDataIndex
 * @see com.klg.jclass.chart.property.JCDataIndexPropertySave
 */
public class JCDataIndexPropertyLoad implements PropertyLoadModel {

protected JCDataIndex index = null;

/**
 * Sets the source object (the object on which to set properties).
 *
 * @param o object that is loading properties
 */
public void setSource(Object o)
{
	if (o instanceof JCDataIndex) {
		this.index = (JCDataIndex)o;
	}
}

/**
 * Loads properties using the given property accessor object.
 *
 * @param prop property access object.
 * @param name name of object to be loaded.
 * @throws JCIOException
 */
public void loadProperties(PropertyAccessModel prop, String name)
    throws JCIOException
{
	if (index == null) {
		System.out.println("FAILURE: No index set");
		return;
	}

	String pval = prop.getProperty(name + "point");
	index.setPoint(JCTypeConverter.toInt(pval, index.getPoint()));

	pval = prop.getProperty(name + "seriesIndex");
	// Special case for other slice
    String other = JCChartEnumMappings.data_format_strings[1];
	if (pval != null && pval.equalsIgnoreCase(other)) {
		index.setSeriesIndex(JCPieChartFormat.OTHER_SLICE);
	}
    // All other series indices are numbers
    else {
		index.setSeriesIndex(JCTypeConverter.toInt(pval,
                                                   index.getSeriesIndex()));
	}
}

}
