/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCDataIndexPropertySave.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:54 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.ChartDataView;
import com.klg.jclass.chart.JCDataIndex;
import com.klg.jclass.chart.JCPieChartFormat;
import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.util.io.JCIOException;

/**
 * Responsible for saving properties of <code>JCDataIndex</code>
 * objects to a destiation specified by <code>PropertyPersistorModel</code>.
 * Not intended for public use.
 *
 * @see com.klg.jclass.chart.JCDataIndex
 * @see com.klg.jclass.chart.property.JCDataIndexPropertyLoad
 */
public class JCDataIndexPropertySave implements PropertySaveModel {

protected JCDataIndex index = null;
protected JCDataIndex defaultIndex = null;

/**
 * Sets the source component whose properties are to be saved.
 * @param o the new source component
 */
public void setSource(Object o)
{
	if (o instanceof JCDataIndex) {
		this.index = (JCDataIndex)o;
	}
}

/**
 * Sets default object instance.  This object is typically used by implementors
 * to determine whether a property really needs to be saved.
 * @param def the new default object
 */
public void setDefault(Object def)
{
	if (def instanceof JCDataIndex) {
		this.defaultIndex = (JCDataIndex)def;
	}
}

/**
 * Checks whether there are any properties different from their default.  If so,
 * then this saver has information to write.
 * @param prop property persistor object
 * @return whether any properties are different from their default
 */
public boolean checkProperties(PropertyPersistorModel prop)
{
	if (index == null || defaultIndex == null) {
		return(false);
    }

	if (index.getPoint() != defaultIndex.getPoint()       ||
	    index.getDistance() != defaultIndex.getDistance() ||
		index.getSeriesIndex() != defaultIndex.getSeriesIndex())
	{
		return(true);
	}
	return(false);
}

/**
 * Saves HTML parameters from the current param source;
 * click the method name to see which properties are saved.<p>
 * Saves the following properties:
 *  <pre>
 *   point (integer)
 *   distance (integer)
 *   seriesIndex (integer)
 *  </pre>
 *
 * @param prop property persistor object
 * @param name name of object to be saved
 * @param tab the tab indent for this group of properties
 * @throws JCIOException
 */
public void saveProperties(PropertyPersistorModel prop, String name, int tab)
    throws JCIOException
{
    if (index == null || defaultIndex == null) {
        System.out.println("FAILURE: No index set");
        return;
    }

    // Check if something to do.
    if (!checkProperties(prop)) {
        return;
    }

    int newTab = prop.writeBegin("data-index", tab);

    if (!prop.getType().equals("HTML")) {
        ChartDataView indexView = index.getDataView();
        if (indexView != null) {
            prop.writeProperty(name, "dataView", tab,
                               prop.expandText(indexView.getName()));
        }
    }

    // Set point
    if (index.getPoint() != defaultIndex.getPoint()) {
        prop.writeProperty(name, "point", newTab,
                           new Integer(index.getPoint()));
    }

    // Set seriesIndex
    String seriesName = prop.getType().equals("XML") ? "series"
                                                     : "seriesIndex";
    if (index.getSeriesIndex() != defaultIndex.getSeriesIndex()) {
        int seriesIndex = index.getSeriesIndex();
        // Special case for other slice
        if (seriesIndex == JCPieChartFormat.OTHER_SLICE) {
            String other = JCChartEnumMappings.data_format_strings[1];
            prop.writeProperty(name, seriesName, newTab, other);
        }
        // All other series indices are numbers
        else {
            prop.writeProperty(name, seriesName, newTab,
                               new Integer(index.getSeriesIndex()));
        }
    }
    prop.writeEnd(null, tab, true, false);
}

}
