/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCFileAccessor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:54 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.util.io.LoadProperties;
import com.klg.jclass.util.io.Properties;

import java.awt.*;
import java.io.*;

/**
 * A property accessor class that gets properties from a file.
 *
 * @see com.klg.jclass.chart.property.PropertyAccessModel
 * @see com.klg.jclass.chart.property.JCBaseAccessor
 */
public abstract class JCFileAccessor extends JCBaseAccessor {

/**
 * Default constructor - does nothing.
 */
public JCFileAccessor() {}

/**
 * Constructor which provides the  component for which properties will be loaded
 * @param c the component (the Chart)
 */
public JCFileAccessor(Component c)
{
	this.component = c;
    loadProperties = new LoadProperties();
}

/**
 * Constructor which provides the component for which properties will be
 * loaded and an instance of LoadProperties containing properties
 * that will be used during the load.
 * @param c the component (the Chart)
 */
public JCFileAccessor(Component c, LoadProperties loadProperties)
{
    this.component = c;
	this.loadProperties = loadProperties;
}

/**
 * Loads properties from a file (with the given filename) into a String.
 * @param fileName the name of the file
 * @return the String with the property information
 * @throws java.io.IOException
 */
protected static String loadFile(String fileName) throws java.io.IOException
{
	InputStream stream = null;
	stream = new FileInputStream(fileName);

	if (stream == null) {
		return("");
	}

	return(loadFile(stream));
}

/**
 * Loads properties from a stream into a String.
 * @param stream the input stream
 * @return the String with the property information
 * @throws java.io.IOException
 */
protected static String loadFile(InputStream stream) throws java.io.IOException
{
	if (stream == null) {
		return(null);
	}

	BufferedReader    data = null;
	InputStreamReader isr  = null;
	try {
		isr = new InputStreamReader(new BufferedInputStream(stream),
									Properties.DEFAULT_CHARSET_NAME);
		data = new BufferedReader(isr);
	}
	catch (UnsupportedEncodingException uee) {
		isr = new InputStreamReader(new BufferedInputStream(stream));
		data = new BufferedReader(isr);
	}
    return(loadFile(data));
}

/**
 * Loads properties from a reader into a String.
 * @param reader the reader
 * @return the String with the property information
 * @throws java.io.IOException
 */
protected static String loadFile(Reader reader) throws java.io.IOException
{
    if (reader == null) {
        return(null);
    }

    StringBuffer sb = new StringBuffer();
    BufferedReader data = null;
    if (reader instanceof BufferedReader) {
        data = (BufferedReader)reader;
    }
    else {
        data = new BufferedReader(reader);
    }
    for (String s = data.readLine(); s != null; s = data.readLine()) {
        sb.append(s);
        sb.append("\n");
    }
    return(new String(sb));
}

}
