/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCFilePersistor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:54 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import java.io.DataOutputStream;
import java.io.FileOutputStream;
import java.io.OutputStream;

/**
 * Abstract property persistor class that saves property settings to a file.
 * Subclasses define the <code>writeProperty</code> method to write in the
 * appropriate format for a specific type of file.
 *
 * @see com.klg.jclass.chart.property.PropertyPersistorModel
 */
public abstract class JCFilePersistor extends JCBasePersistor {

protected DataOutputStream   out = null;

/**
 * Constructor which provides the source (which holds the properties to write)
 * and a filename (to which the output is to be written).
 * @param fileName The filename
 * @param source The source (the Chart)
 * @throws java.io.IOException
 */
public JCFilePersistor(String fileName, Object source)
    throws java.io.IOException
{
	this(new FileOutputStream(fileName), source);
}

/**
 * Constructor which provides the source (which holds the properties to write)
 * and an output stream (to which the output is to be written).
 * @param os The output stream
 * @param source The source (the Chart)
 * @throws java.io.IOException
 */
public JCFilePersistor(OutputStream os, Object source)
    throws java.io.IOException
{
    super(source);
    setOutputStream(os);
}

/**
 * Returns the output stream used for this file persistor.
 * @return The file persistor's output stream
 */
public OutputStream getOutputStream()
{
	return(out);
}

/**
 * Sets the output stream used for this file persistor.
 * @param os The new file persistor output stream
 */
public void setOutputStream(OutputStream os)
{
    if (os == null) {
        throw new IllegalArgumentException("Outputstream cannot be null");
    }
    if (os instanceof DataOutputStream) {
        out = (DataOutputStream)os;
    }
    else {
        out = new DataOutputStream(os);
    }
}

}
