/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCFillStylePropertyLoad.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:54 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.chart.JCFillStyle;
import com.klg.jclass.chart.JCStyle;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.io.JCIOException;
import com.klg.jclass.util.io.OutputProperties;
import com.klg.jclass.util.swing.JCSwingTypeConverter;

import javax.swing.*;
import java.awt.*;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * Responsible for loading properties of <code>JCFillStyle</code>
 * objects using a specified <code>PropertyAcessModel</code>. Not
 * intended for public use.
 *
 * @see com.klg.jclass.chart.JCFillStyle
 * @see com.klg.jclass.chart.property.JCFillStylePropertySave
 */
public class JCFillStylePropertyLoad implements PropertyLoadModel {

protected JCFillStyle style = null;

/**
 * Sets the source object (the object on which to set properties).
 *
 * @param o object that is loading properties
 */
public void setSource(Object o)
{
	if (o instanceof JCFillStyle) {
		this.style = (JCFillStyle)o;
	}
}

/**
 * Loads properties using the given property accessor object.
 *
 * @param prop property access object.
 * @param name name of object to be loaded.
 * @throws JCIOException
 */
public void loadProperties(PropertyAccessModel prop, String name)
    throws JCIOException
{
	if (style == null) {
		System.out.println("FAILURE: No fill style set");
		return;
	}

	// Get color value, either from color or colorIndex
	String pval = prop.getProperty(name + "color");
	if (pval != null) {
		style.setColor(JCSwingTypeConverter.toColor(pval, style.getColor()));
	}
	else {
		// For backwards compatibility only
		pval = prop.getProperty(name + "colorIndex");
		if (pval != null) {
			int i = JCTypeConverter.toInt(pval, -1);
			if (i != style.getColorIndex()) {
				style.setColor(JCStyle.getDefaultColors()[i]);
			}
		}
	}

	// Get pattern
    boolean isXML    = prop.getType().equals("XML");
    String[] strings = isXML ? JCChartEnumMappings.fillPattern_xml_strings
                             : JCChartEnumMappings.fillPattern_strings;
	int[] values     = JCChartEnumMappings.fillPattern_values;
	pval = prop.getProperty(name + "pattern");
	style.setPattern(JCTypeConverter.toEnum(pval, "fillPattern", strings,
	                                        values, style.getPattern()));

	// Get background
	pval = prop.getProperty(name + "background");
	style.setBackground(JCSwingTypeConverter.toColor(pval,
													 style.getBackground()));

	// Get fill image
    URL url = null;
    OutputProperties outProps = new OutputProperties();
    final boolean ingnoreExceptions =
            prop.getLoadProperties().ignoreExternalResourceExceptions();
    try {
        url = prop.getPropertyURL(name, outProps);
    }
    catch (MalformedURLException mle) {
        if (!ingnoreExceptions) {
            throw new JCIOException(mle.getMessage(), mle);
        }
    }

    style.setOutputProperties(outProps);
    if (url != null) {
        ImageIcon imageIcon = null;

       
       

       
       imageIcon = new ImageIcon(url);
       

        if (imageIcon != null &&
                imageIcon.getImageLoadStatus() == MediaTracker.COMPLETE) {
            style.setImage(imageIcon.getImage());
        }
        else {
            if (!ingnoreExceptions) {
                throw new JCIOException("Image from URL " + url +
                                        " could not be loaded!!!");
            }
        }
    }
}

}

