/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCFillStylePropertySave.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:54 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.chart.JCFillStyle;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.io.OutputProperties;
import com.klg.jclass.util.io.JCIOException;
import com.klg.jclass.util.swing.JCSwingTypeConverter;

/**
 * Responsible for saving properties of <code>JCFillStyle</code>
 * objects to a destiation specified by <code>PropertyPersistorModel</code>.
 * Not intended for public use.
 *
 * @see com.klg.jclass.chart.JCFillStyle
 * @see com.klg.jclass.chart.property.JCFillStylePropertyLoad
 */
public class JCFillStylePropertySave implements PropertySaveModel {

protected JCFillStyle style = null;
protected JCFillStyle defaultStyle = null;

/**
 * Sets the source component whose properties are to be saved.
 * @param o the new source component
 */
public void setSource(Object o)
{
	if (o instanceof JCFillStyle) {
		this.style = (JCFillStyle)o;
	}
}

/**
 * Sets default object instance.  This object is typically used by implementors
 * to determine whether a property really needs to be saved.
 * @param def the new default object
 */
public void setDefault(Object def)
{
	if (def instanceof JCFillStyle) {
		this.defaultStyle = (JCFillStyle)def;
	}
}

/**
 * Checks whether there are any properties different from their default.  If so,
 * then this saver has information to write.
 * @param prop property persistor object
 * @return whether any properties are different from their default
 */
public boolean checkProperties(PropertyPersistorModel prop)
{
	return(true);
}

/**
 * Click the method name to see which properties are saved.<p>
 * Saves the following properties:
 * <pre>
 *   color
 *   pattern
 *   background
 * </pre>
 *
 * @param prop property persistor object
 * @param name name of object to be saved
 * @param tab the tab indent for this group of properties
 * @throws JCIOException
 */
public void saveProperties(PropertyPersistorModel prop, String name, int tab)
    throws JCIOException
{
	if (style == null || defaultStyle == null) {
		System.out.println("FAILURE: No fill style set");
		return;
	}

	int newTab = prop.writeBegin("fill-style", tab);

	// Always write out the color
	prop.writeProperty(name, "color", newTab,
					   JCSwingTypeConverter.fromColor(style.getColor()));

	// Save pattern
    boolean isXML    = prop.getType().equals("XML");
    String[] strings = isXML ? JCChartEnumMappings.fillPattern_xml_strings
                             : JCChartEnumMappings.fillPattern_strings;
	int[] values     = JCChartEnumMappings.fillPattern_values;
	if (style.getPattern() != defaultStyle.getPattern()) {
		prop.writeProperty(name, "pattern", newTab,
						   JCTypeConverter.fromEnum(style.getPattern(),
									                strings, values));
	}

	// Save background color
	if (style.getBackground() != defaultStyle.getBackground()) {
		prop.writeProperty(name, "background", newTab,
					       JCSwingTypeConverter.fromColor(style.getBackground()));
	}

    // Save image output properties
    OutputProperties outProps = style.getOutputProperties();
    if (outProps != null) {
        String propertyName = outProps.getPropertyName();
        int fileAccess = outProps.getFileAccess();
        if (propertyName != null && propertyName.length() > 0) {
            prop.writeEnd(null, tab, true, true);

            // Write out image-file element
            int newerTab =
                prop.writeBegin("image-file",
                                tab + PropertySaveFactory.tabIncrement);
            prop.writeProperty(name + "image.", "fileName", newerTab,
                               propertyName);
            String s = outProps.getFileAccessStringFromEnum(fileAccess,
                                                            false);
            prop.writeProperty(name + "image.", "fileAccess", newerTab, s);
            prop.writeEnd(null, newTab, true, false);

            prop.writeEnd("fill-style", tab, true, false);
            return;
        }
    }
    prop.writeEnd(null, tab, true, false);
}

}

