/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCGridPropertyLoad.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:54 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.JCGrid;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.io.JCIOException;

/**
 * Responsible for loading properties of <code>JCGrid</code>
 * objects using a specified <code>PropertyAcessModel</code>. Not
 * intended for public use.
 *
 * @see com.klg.jclass.chart.JCGrid
 */
public class JCGridPropertyLoad implements PropertyLoadModel {

protected JCGrid grid = null;

/**
 * Sets the source object (the object on which to set properties).
 *
 * @param o object that is loading properties
 */
public void setSource(Object o)
{
    if (o instanceof JCGrid) {
        this.grid = (JCGrid)o;
    }
}

/**
 * Loads properties using the given property accessor object.
 *
 * @param prop property access object.
 * @param name name of object to be loaded.
 * @throws JCIOException
 */
public void loadProperties(PropertyAccessModel prop, String name)
    throws JCIOException
{
    if (grid == null) {
        System.out.println("FAILURE: No grid object is set");
        return;
    }

    // Remove the last period from name.
    String gridName = name.substring(0, name.length()-1);
    grid.setName(gridName);

    String pval = prop.getProperty(name + "startValue");
    if (pval != null) {
        grid.setStartValue(JCTypeConverter.toDouble(pval,
                           grid.getStartValue()));
    }

    pval = prop.getProperty(name + "startValueIsDefault");
    if (pval != null) {
        grid.setStartValueIsDefault(JCTypeConverter.toBoolean(pval,
                          grid.getStartValueIsDefault()));
    }

    pval = prop.getProperty(name + "stopValue");
    if (pval != null) {
        grid.setStopValue(JCTypeConverter.toDouble(pval,
                          grid.getStopValue()));
    }

    pval = prop.getProperty(name + "stopValueIsDefault");
    if (pval != null) {
        grid.setStopValueIsDefault(JCTypeConverter.toBoolean(pval,
                          grid.getStopValueIsDefault()));
    }

    pval = prop.getProperty(name + "incrementValue");
    if (pval != null) {
        grid.setIncrementValue(JCTypeConverter.toDouble(pval,
                               grid.getIncrementValue()));
    }

    pval = prop.getProperty(name + "incrementIsDefault");
    if (pval != null) {
        grid.setIncrementIsDefault(JCTypeConverter.toBoolean(pval,
                          grid.getIncrementIsDefault()));
    }

    PropertyLoadFactory.load(prop, grid.getGridStyle().getLineStyle(),
                             name);
}

}
