/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCHLOCChartFormatPropertySave.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:54 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.JCHLOCChartFormat;
import com.klg.jclass.util.io.JCIOException;

/**
 * Responsible for saving properties of <code>JCHLOCChartFormat</code>
 * objects to a destiation specified by <code>PropertyPersistorModel</code>.
 * Not intended for public use.
 *
 * @see com.klg.jclass.chart.JCHLOCChartFormat
 * @see com.klg.jclass.chart.property.JCHLOCChartFormatPropertyLoad
 */
public class JCHLOCChartFormatPropertySave implements PropertySaveModel {

protected JCHLOCChartFormat format = null;
protected JCHLOCChartFormat defaultFormat = null;

/**
 * Sets the source component whose properties are to be saved.
 * @param o the new source component
 */
public void setSource(Object o)
{
	if (o instanceof JCHLOCChartFormat) {
		this.format = (JCHLOCChartFormat)o;
	}
}

/**
 * Sets default object instance.  This object is typically used by implementors
 * to determine whether a property really needs to be saved.
 * @param def the new default object
 */
public void setDefault(Object def)
{
	if (def instanceof JCHLOCChartFormat) {
		this.defaultFormat = (JCHLOCChartFormat)def;
	}
}

/**
 * Checks whether there are any properties different from their default.  If so,
 * then this saver has information to write.
 * @param prop Property persistor object
 * @return Whether any properties are different from their default.
 */
public boolean checkProperties(PropertyPersistorModel prop)
{
    if (format == null || defaultFormat == null) {
		return(false);
    }

	if (format.isShowingOpen() != defaultFormat.isShowingOpen()   ||
	    format.isShowingClose() != defaultFormat.isShowingClose() ||
		format.isOpenCloseFullWidth() != defaultFormat.isOpenCloseFullWidth())
	{
		return(true);
	}
	return(false);
}

/**
 * Saves HTML parameters from the current param source;
 * click the method name to see which properties are saved.<p>
 * Saves the following properties:
 *   <pre>
 *   series&lt;n&gt;.hilo.&lt;style&gt; (JCChartStyle)
 *   series&lt;n&gt;.tickSize (integer)
 *   showingOpen (boolean)
 *   showingClose (boolean)
 *   openCloseFullWidth (boolean)
 *  </pre>
 * @param prop property persistor object
 * @param name name of object to be saved
 * @param tab the tab indent for this group of properties
 * @throws JCIOException
 */
public void saveProperties(PropertyPersistorModel prop, String name, int tab)
    throws JCIOException
{
	if (format == null || defaultFormat == null) {
		System.out.println("FAILURE: No HLOC format set");
		return;
	}

	// Check if something to do.
	if (!checkProperties(prop)) {
		return;
	}

	int newTab = prop.writeBegin("hi-lo-open-close-format", tab);

	// Do not save hilo style and tick size information from format
	// class.  This is redundant information, having already been
    // saved with the chart styles of the series.  (It is loaded by
	// the loader class however, to maintain backwards compatibility
	// and to allow slightly more obvious ways of getting that info
	// into the chart.)

	// Set showingOpen
	if (format.isShowingOpen() != defaultFormat.isShowingOpen()) {
		prop.writeProperty(name, "showingOpen", newTab,
						   new Boolean(format.isShowingOpen()));
	}

	// Set showingClose
	if (format.isShowingClose() != defaultFormat.isShowingClose()) {
		prop.writeProperty(name, "showingClose", newTab,
						   new Boolean(format.isShowingClose()));
	}

	// Set openCloseFullWidth
	if (format.isOpenCloseFullWidth() != defaultFormat.isOpenCloseFullWidth()) {
		prop.writeProperty(name, "openCloseFullWidth", newTab,
						   new Boolean(format.isOpenCloseFullWidth()));
	}
	prop.writeEnd(null, tab, true, false);
}

}
