/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCLegendPropertyLoad.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:54 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.ImageMapInfo;
import com.klg.jclass.util.io.JCIOException;
import com.klg.jclass.util.legend.JCLegend;
import com.klg.jclass.util.legend.JCMultiColLegend;
import com.klg.jclass.util.legend.JCLegendEnumMappings;

/**
 * Responsible for loading properties of <code>JCLegend</code>
 * objects using a specified <code>PropertyAcessModel</code>. Not
 * intended for public use.
 *
 * @see com.klg.jclass.util.legend.JCLegend
 * @see com.klg.jclass.chart.property.JCLegendPropertySave
 */
public class JCLegendPropertyLoad extends ComponentPropertyLoad {

protected JCLegend legend = null;

/**
 * Sets the source object (the object on which to set properties).
 *
 * @param o object that is loading properties
 */
public void setSource(Object o)
{
	super.setSource(o);
	if (o instanceof JCLegend) {
		this.legend = (JCLegend)o;
	}
}

/**
 * Loads properties using the given property accessor object.
 *
 * @param prop property access object.
 * @param name name of object to be loaded.
 * @throws JCIOException
 */
public void loadProperties(PropertyAccessModel prop, String name)
    throws JCIOException
{
	super.loadProperties(prop, name);

	if (legend == null) {
		System.out.println("FAILURE: No legend set");
		return;
	}

	String[] strings = JCLegendEnumMappings.anchor_strings;
	int[]    values  = JCLegendEnumMappings.anchor_values;
	String pval = prop.getProperty(name + "anchor");
	legend.setAnchor(JCTypeConverter.toEnum(pval, "Anchor", strings, values,
									        legend.getAnchor()));

	strings = JCLegendEnumMappings.orientation_strings;
	values  = JCLegendEnumMappings.orientation_values;
	pval = prop.getProperty(name + "orientation");
	legend.setOrientation(JCTypeConverter.toEnum(pval, "Orientation",
									             strings, values,
									             legend.getOrientation()));

    // Load the legend's imageMapInfo, if set.
    ImageMapInfo info = new ImageMapInfo();
    PropertyLoadFactory.load(prop, info, name + "imageMapInfo.");
    if (!info.isEmpty()) {
        legend.setImageMapInfo(info);
    }
}

}
