/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCLegendPropertySave.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:54 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.JCChart;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.ImageMapInfo;
import com.klg.jclass.util.io.JCIOException;
import com.klg.jclass.util.legend.JCLegend;
import com.klg.jclass.util.legend.JCGridLegend;
import com.klg.jclass.util.legend.JCMultiColLegend;
import com.klg.jclass.util.legend.JCLegendEnumMappings;

import javax.swing.*;
import java.awt.*;

/**
 * Responsible for saving properties of <code>JCLegend</code>
 * objects to a destiation specified by <code>PropertyPersistorModel</code>.
 * Not intended for public use.
 *
 * @see com.klg.jclass.util.legend.JCLegend
 * @see com.klg.jclass.chart.property.JCLegendPropertyLoad
 */
public class JCLegendPropertySave extends ComponentPropertySave {

protected JCLegend legend = null;
protected JCLegend defaultLegend = null;

/**
 * Sets the source object ie the object to be persisted.
 * @param o object that is being persisted.
 */
public void setSource(Object o)
{
	super.setSource(o);
	if (o instanceof JCLegend) {
		this.legend = (JCLegend)o;
	}
}

/**
 * Sets default object instance.  This object is typically used by implementors
 * to determine whether a property really needs to be saved.
 * @param def The new default object
 */
public void setDefault(Object def)
{
	super.setDefault(def);
	if (def instanceof JCLegend) {
		this.defaultLegend = (JCLegend)def;
	}
}

/**
 * Checks whether there are any properties different from their default.  If so,
 * then this saver has information to write.
 * @param prop property persistor object
 * @return whether any properties are different from their default
 */
public boolean checkProperties(PropertyPersistorModel prop)
{
	if (legend == null || defaultLegend == null) {
		return(false);
	}

	boolean writeProperties = false;

	Container parent = component.getParent();
	if (parent != null && parent instanceof JCChart) {
		if (checkLayoutHints((JCChart)parent, (JComponent)component)) {
			writeProperties = true;
		}
	}

	if (!writeProperties &&
        (super.checkProperties(prop)                     ||
	     legend.getAnchor() != defaultLegend.getAnchor() ||
	     legend.getOrientation() != defaultLegend.getOrientation()))
	{
		writeProperties = true;
	}

    ImageMapInfo info = legend.getImageMapInfo();
    if (!writeProperties && info != null && !info.isEmpty()) {
        writeProperties = true;
    }

	return(writeProperties);
}

/**
 * Saves HTML parameters from the current param source;
 * click the method name to see which properties are saved.<p>
 * Saves the following properties
 *  <pre>
 *   anchor (enum)
 *   orientation (enum)
 *  </pre>
 * @param prop property persistor object
 * @param name name of object to be saved
 * @param tab the tab indent for this group of properties
 * @throws JCIOException
 */
public void saveProperties(PropertyPersistorModel prop, String name, int tab)
    throws JCIOException
{
	if (legend == null || defaultLegend == null) {
		System.out.println("FAILURE: No legend set");
		return;
	}

	// Check if something to do.
	if (!checkProperties(prop)) {
		return;
	}

	int newTab = prop.writeBegin("legend", tab);

	// save legend type
	if (legend instanceof JCMultiColLegend) {
		prop.writeProperty(name, "type", newTab, "MultiCol");
	} else if (legend instanceof JCGridLegend) {
		prop.writeProperty(name, "type", newTab, "Grid");
	}

	String[] strings = JCLegendEnumMappings.anchor_strings;
	int[]    values  = JCLegendEnumMappings.anchor_values;
	if (legend.getAnchor() != defaultLegend.getAnchor()) {
		prop.writeProperty(name, "anchor", newTab,
						   JCTypeConverter.fromEnum(legend.getAnchor(),
										            strings, values));
	}

	strings = JCLegendEnumMappings.orientation_strings;
	values  = JCLegendEnumMappings.orientation_values;
	if (legend.getOrientation() != defaultLegend.getOrientation()) {
		prop.writeProperty(name, "orientation", newTab,
						   JCTypeConverter.fromEnum(legend.getOrientation(),
								                    strings, values));
	}

	prop.writeEnd(null, tab, true, true);

	if (super.checkProperties(prop)) {
	    super.saveProperties(prop, name,
		                     tab + PropertySaveFactory.tabIncrement);
	}

	Container parent = component.getParent();
	if (parent != null && parent instanceof JCChart) {
	    saveLayoutHints(prop, (JCChart)parent, (JComponent)component,
		                name, tab + PropertySaveFactory.tabIncrement);
	}

    // Save ImageMapInfo properties
    PropertySaveFactory.save(prop, legend.getImageMapInfo(),
                             new ImageMapInfo(), name + "imageMapInfo.", tab);

	saveSubclassProperties(prop, name, tab);

	prop.writeEnd("legend", tab, true, false);
}

/**
 * Save properties specific to legend subclasses.  For obvious reasons,
 * this is empty here.  It should be overridden by subclasses representing
 * subclasses of the JCLegend class.
 */
public void saveSubclassProperties(PropertyPersistorModel prop, String name,
                                   int tab) {}

}

