/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCLineStylePropertyLoad.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:54 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.chart.JCLineStyle;
import com.klg.jclass.chart.JCStyle;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.io.JCIOException;
import com.klg.jclass.util.swing.JCSwingTypeConverter;

/**
 * Responsible for loading properties of <code>JCLineStyle</code>
 * objects using a specified <code>PropertyAcessModel</code>. Not
 * intended for public use.
 *
 * @see com.klg.jclass.chart.JCLineStyle
 * @see com.klg.jclass.chart.property.JCLineStylePropertySave
 */
public class JCLineStylePropertyLoad implements PropertyLoadModel {

protected JCLineStyle style = null;

/**
 * Sets the source object (the object on which to set properties).
 *
 * @param o object that is loading properties
 */
public void setSource(Object o)
{
	if (o instanceof JCLineStyle) {
		this.style = (JCLineStyle)o;
	}
}

/**
 * Loads properties using the given property accessor object.
 *
 * @param prop property access object.
 * @param name name of object to be loaded.
 * @throws JCIOException
 */
public void loadProperties(PropertyAccessModel prop, String name)
    throws JCIOException
{
	if (style == null) {
		System.out.println("FAILURE: No line style set");
		return;
	}

	// Get color value, either from color or colorIndex
	String pval = prop.getProperty(name + "color");
	if (pval != null) {
		style.setColor(JCSwingTypeConverter.toColor(pval, style.getColor()));
	}
	else {
		// For backwards compatibility only
		pval = prop.getProperty(name + "colorIndex");
		if (pval != null) {
			int i = JCTypeConverter.toInt(pval, -1);
			if (i != style.getColorIndex()) {
				style.setColor(JCStyle.getDefaultColors()[i]);
			}
		}
	}

	// Get line pattern
	String[] strings = JCChartEnumMappings.linePattern_strings;
	int[] values     = JCChartEnumMappings.linePattern_values;
	pval = prop.getProperty(name + "pattern");
	style.setPattern(JCTypeConverter.toEnum(pval, "linePattern", strings,
									        values, style.getPattern()));

	// Get line width
	pval = prop.getProperty(name + "width");
	style.setWidth(JCTypeConverter.toInt(pval, style.getWidth()));

	// Get line join style
	strings = JCChartEnumMappings.lineJoin_strings;
	values  = JCChartEnumMappings.lineJoin_values;
	pval = prop.getProperty(name + "join");
	style.setJoin(JCTypeConverter.toEnum(pval, "join", strings, values,
										 style.getJoin()));

	// Get line cap
    boolean isXML = prop.getType().equals("XML");
    strings = isXML ? JCChartEnumMappings.lineCap_xml_strings
                    : JCChartEnumMappings.lineCap_strings;
	values  = JCChartEnumMappings.lineCap_values;
	pval = prop.getProperty(name + "cap");
	style.setCap(JCTypeConverter.toEnum(pval, "cap", strings, values,
										style.getCap()));
}

}

