/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCLineStylePropertySave.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:54 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.chart.JCLineStyle;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.io.JCIOException;
import com.klg.jclass.util.swing.JCSwingTypeConverter;

/**
 * Responsible for saving properties of <code>JCLineStyle</code>
 * objects to a destiation specified by <code>PropertyPersistorModel</code>.
 * Not intended for public use.
 *
 * @see com.klg.jclass.chart.JCLineStyle
 * @see com.klg.jclass.chart.property.JCLineStylePropertyLoad
 */
public class JCLineStylePropertySave implements PropertySaveModel {

protected JCLineStyle style = null;
protected JCLineStyle defaultStyle = null;

/**
 * Sets the source component whose properties are to be saved.
 * @param o the new source component
 */
public void setSource(Object o)
{
	if (o instanceof JCLineStyle) {
		this.style = (JCLineStyle)o;
	}
}

/**
 * Sets default object instance.  This object is typically used by implementors
 * to determine whether a property really needs to be saved.
 * @param def the new default object
 */
public void setDefault(Object def)
{
	if (def instanceof JCLineStyle) {
		this.defaultStyle = (JCLineStyle)def;
	}
}

/**
 * Checks whether there are any properties different from their default.  If so,
 * then this saver has information to write.
 * @param prop property persistor object
 * @return whether any properties are different from their default
 */
public boolean checkProperties(PropertyPersistorModel prop)
{
	return(true);
}

/**
 * Saves HTML parameters from the current param source;
 * click the method name to see which properties are saved.<p>
 * Saves the following properties:
 *   <pre>
 *   color (Color)
 *   pattern (enum)
 *   width (int)
 *   join (enum)
 *   cap (enum)
 *  </pre>
 * @param prop property persistor object
 * @param name name of object to be saved
 * @param tab the tab indent for this group of properties
 * @throws JCIOException
 */
public void saveProperties(PropertyPersistorModel prop, String name, int tab)
    throws JCIOException
{
	if (style == null || defaultStyle == null) {
		System.out.println("FAILURE: No line style set");
		return;
	}


	int newTab = prop.writeBegin("line-style", tab);

	// Always write out the color
	prop.writeProperty(name, "color", newTab,
					   JCSwingTypeConverter.fromColor(style.getColor()));

	// Save pattern
	String[] strings = JCChartEnumMappings.linePattern_strings;
	int[] values     = JCChartEnumMappings.linePattern_values;
	if (style.getPattern() != defaultStyle.getPattern()) {
		prop.writeProperty(name, "pattern", newTab,
						   JCTypeConverter.fromEnum(style.getPattern(),
		                                            strings, values));
	}

	// Save line width
	if (style.getWidth() != defaultStyle.getWidth()) {
		prop.writeProperty(name, "width", newTab,
		                   new Integer(style.getWidth()));
	}

	// Save line join style
	strings = JCChartEnumMappings.lineJoin_strings;
	values  = JCChartEnumMappings.lineJoin_values;
	if (style.getJoin() != defaultStyle.getJoin()) {
		prop.writeProperty(name, "join", newTab,
						   JCTypeConverter.fromEnum(style.getJoin(),
		                                            strings, values));
	}

	// Save line cap style
    boolean isXML = prop.getType().equals("XML");
    strings = isXML ? JCChartEnumMappings.lineCap_xml_strings
                    : JCChartEnumMappings.lineCap_strings;
	values  = JCChartEnumMappings.lineCap_values;
	if (style.getCap() != defaultStyle.getCap()) {
		prop.writeProperty(name, "cap", newTab,
						   JCTypeConverter.fromEnum(style.getCap(),
		                                            strings, values));
	}
    prop.writeEnd(null, tab, true, false);
}

}

