/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCMultiColLegendPropertyLoad.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:54 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.io.JCIOException;
import com.klg.jclass.util.legend.JCLegend;
import com.klg.jclass.util.legend.JCMultiColLegend;
import com.klg.jclass.util.legend.JCLegendEnumMappings;

/**
 * Responsible for loading properties of <code>JCMultiColLegend</code>
 * objects using a specified <code>PropertyAcessModel</code>. Not
 * intended for public use.
 *
 * @see com.klg.jclass.util.legend.JCLegend
 * @see com.klg.jclass.util.legend.JCMultiColLegend
 * @see com.klg.jclass.chart.property.JCLegendPropertySave
 */
public class JCMultiColLegendPropertyLoad extends JCGridLegendPropertyLoad {

protected JCMultiColLegend mcLegend = null;

/**
 * Sets the source object (the object on which to set properties).
 *
 * @param o object that is loading properties
 */
public void setSource(Object o)
{
	super.setSource(o);
	if (o instanceof JCMultiColLegend) {
		this.mcLegend = (JCMultiColLegend)o;
	}
}

/**
 * Loads properties using the given property accessor object.
 *
 * @param prop property access object.
 * @param name name of object to be loaded.
 * @throws JCIOException
 */
public void loadProperties(PropertyAccessModel prop, String name)
    throws JCIOException
{
	super.loadProperties(prop, name);

	if (mcLegend == null) {
		System.out.println("FAILURE: No legend set");
		return;
	}

	// handle multi-column properties if legend is a multi-column legend
	String pval = prop.getProperty(name + "numRows");
	if (pval != null) {
		mcLegend.setNumRows(JCTypeConverter.toInt(pval, -1));
	}

	pval = prop.getProperty(name + "numColumns");
	if (pval != null) {
		mcLegend.setNumColumns(JCTypeConverter.toInt(pval, -1));
	}
}

}
