/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCMultiColLegendPropertySave.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:54 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.JCChart;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.io.JCIOException;
import com.klg.jclass.util.legend.JCLegend;
import com.klg.jclass.util.legend.JCMultiColLegend;
import com.klg.jclass.util.legend.JCLegendEnumMappings;

import javax.swing.*;
import java.awt.*;

/**
 * Responsible for saving properties of <code>JCMultiColLegend</code>
 * objects to a destiation specified by <code>PropertyPersistorModel</code>.
 * Not intended for public use.
 *
 * @see com.klg.jclass.util.legend.JCLegend
 * @see com.klg.jclass.util.legend.JCMultiColLegend
 * @see com.klg.jclass.chart.property.JCLegendPropertyLoad
 */
public class JCMultiColLegendPropertySave extends JCGridLegendPropertySave {

protected JCMultiColLegend mcLegend = null;
protected JCMultiColLegend defaultMCLegend = null;

/**
 * Sets the source object ie the object to be persisted.
 * @param o object that is being persisted.
 */
public void setSource(Object o)
{
	super.setSource(o);
	if (o instanceof JCMultiColLegend) {
		this.mcLegend = (JCMultiColLegend)o;
	}
}

/**
 * Sets default object instance.  This object is typically used by implementors
 * to determine whether a property really needs to be saved.
 * @param def The new default object
 */
public void setDefault(Object def)
{
	super.setDefault(def);
	if (def instanceof JCMultiColLegend) {
		this.defaultMCLegend = (JCMultiColLegend)def;
	}
}

/**
 * Checks whether there are any properties different from their default.  If so,
 * then this saver has information to write.
 * @param prop property persistor object
 * @return whether any properties are different from their default
 */
public boolean checkProperties(PropertyPersistorModel prop)
{
	if (mcLegend == null) {
		return(false);
	}

	// always write properties for multi-col legend -- it's a
	// non-default case

	return true;
}

/**
 * Saves HTML parameters from the current param source;
 * click the method name to see which properties are saved.<p>
 * Saves the following properties
 *  <pre>
 *   anchor (enum)
 *   orientation (enum)
 *  </pre>
 * @param prop property persistor object
 * @param name name of object to be saved
 * @param tab the tab indent for this group of properties
 * @throws JCIOException
 */
public void saveProperties(PropertyPersistorModel prop, String name, int tab)
    throws JCIOException
{
	if (mcLegend == null) {
		System.out.println("FAILURE: No multi column legend set");
		return;
	}

	super.saveProperties(prop, name, tab);
}

/**
 * Save MultiCol specific properties.  I have to do this here and not
 * in saveProperties() so that they end up inside the actual legend tag
 * that is created by the superclass' saveProperties() method.
 */
public void saveSubclassProperties(PropertyPersistorModel prop, 
												String name, int tab) {

    int mcNumRows = mcLegend.getNumRows();
    int mcNumColumns = mcLegend.getNumColumns();

    if (mcNumRows >= 0 || mcNumColumns >= 0) {

        int mcTab = tab + PropertySaveFactory.tabIncrement;
        int newTab = prop.writeBegin("multi-col", mcTab);
        if (mcNumRows >= 0) {
            prop.writeProperty(name, "numRows", newTab,
                        new Integer(mcNumRows));
        }
        if (mcNumColumns >= 0) {
            prop.writeProperty(name, "numColumns", newTab,
                            new Integer(mcNumColumns));
        }
        prop.writeEnd(null, mcTab, true, false);
    }
}

}
