/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCPieChartFormatPropertyLoad.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:54 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.chart.JCPieChartFormat;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.ImageMapInfo;
import com.klg.jclass.util.io.JCIOException;

/**
 * Responsible for loading properties of <code>JCPieChartFormat</code>
 * objects using a specified <code>PropertyAcessModel</code>. Not
 * intended for public use.
 *
 * @see com.klg.jclass.chart.JCPieChartFormat
 * @see com.klg.jclass.chart.property.JCPieChartFormatPropertySave
 */
public class JCPieChartFormatPropertyLoad implements PropertyLoadModel {

protected JCPieChartFormat format = null;

/**
 * Sets the source object (the object on which to set properties).
 *
 * @param o object that is loading properties
 */
public void setSource(Object o)
{
    if (o instanceof JCPieChartFormat) {
        this.format = (JCPieChartFormat)o;
    }
}

/**
 * Loads properties using the given property accessor object.
 *
 * @param prop property access object.
 * @param name name of object to be loaded.
 * @throws JCIOException
 */
public void loadProperties(PropertyAccessModel prop, String name)
    throws JCIOException
{
    if (format == null) {
        System.out.println("FAILURE: No pie format set");
        return;
    }

    String pval = prop.getProperty(name + "minSlices");
    format.setMinSlices(JCTypeConverter.toInt(pval, format.getMinSlices()));

    pval = prop.getProperty(name + "explodeOffset");
    format.setExplodeOffset(JCTypeConverter.toInt(pval,
                                                  format.getExplodeOffset()));

    String[] strings = JCChartEnumMappings.thresholdMethod_strings;
    int[] values     = JCChartEnumMappings.thresholdMethod_values;
    pval = prop.getProperty(name + "thresholdMethod");
    format.setThresholdMethod(JCTypeConverter.toEnum(pval, "thresholdMethod",
                                                     strings, values,
                                                     format.getThresholdMethod()));

    pval = prop.getProperty(name + "thresholdValue");
    format.setThresholdValue(JCTypeConverter.toDouble(pval,
                                                      format.getThresholdValue()));


    strings = JCChartEnumMappings.sortOrder_strings;
    values  = JCChartEnumMappings.sortOrder_values;
    pval = prop.getProperty(name + "sortOrder");
    format.setSortOrder(JCTypeConverter.toEnum(pval, "sortOrder", strings,
                                               values, format.getSortOrder()));

    pval = prop.getProperty(name + "startAngle");
    format.setStartAngle(JCTypeConverter.toDouble(pval,
                                                  format.getStartAngle()));

    pval = prop.getProperty(name + "other.label");
    if (pval != null) {
        format.setOtherLabel(pval);
    }

    PropertyLoadFactory.load(prop, format.getOtherStyle(), name + "other.");

    loadExplodedSliceProperties(prop, name);

    // Load other slice image map info
    pval = prop.getProperty(name + "otherSliceImageMapInfoCount");
    if (pval != null) {
        int count = Math.max(JCTypeConverter.toInt(pval, 0), 0);
        ImageMapInfo[] infoArray = new ImageMapInfo[count];
        for (int i = 0; i < count; i++) {
            ImageMapInfo info = new ImageMapInfo();
            String s = (i == 0) ? "" : "" + i;
            PropertyLoadFactory.load(prop, info,
                                     name + "otherSliceImageMapInfo" + s + ".");
            infoArray[i] = info;
        }
        if (!ImageMapInfo.isEmpty(infoArray)) {
            format.setOtherImageMapInfo(infoArray);
        }
    }
    ImageMapInfo info = new ImageMapInfo();
    PropertyLoadFactory.load(prop, info,
                             name + "otherSliceImageMapInfoLegend.");
    if (!info.isEmpty()) {
        format.setOtherLegendImageMapInfo(info);
    }
}

protected void loadExplodedSliceProperties(PropertyAccessModel prop,
                                           String name)
{
    Object o = prop.getPropertyObject(name + "explodeList");
    if (o != null) {
        format.setExplodeList(JCChartPropertyLoad.convertObjectToPointList(o));
    }
}

}
