/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCPolarRadarChartFormatPropertySave.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:54 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.JCChartUtil;
import com.klg.jclass.chart.JCPolarRadarChartFormat;
import com.klg.jclass.util.io.JCIOException;

/**
 * Responsible for saving properties of <code>JCPolarRadarChartFormat</code>
 * objects to a destination specified by <code>PropertyPersistorModel</code>.
 * Not intended for public use.
 *
 * @see com.klg.jclass.chart.JCPolarRadarChartFormat
 * @see com.klg.jclass.chart.property.JCPolarRadarChartFormatPropertyLoad
 */
public class JCPolarRadarChartFormatPropertySave implements PropertySaveModel {

protected JCPolarRadarChartFormat format = null;
protected JCPolarRadarChartFormat defaultFormat = null;

protected double   yAxisAngle;
protected double   originBase;
protected boolean  radarCircularGrid;
protected boolean  halfRange;

/**
 * Sets the source object (the object to be persisted).
 * @param o object that is being persisted.
 */
public void setSource(Object o)
{
	if (o instanceof JCPolarRadarChartFormat) {
		this.format = (JCPolarRadarChartFormat)o;
	}
}

/**
 * Sets default object instance.  This object is typically used by implementors
 * to determine whether a property really needs to be saved.
 * @param def the new default object
 */
public void setDefault(Object def)
{
	if (def instanceof JCPolarRadarChartFormat) {
		this.defaultFormat = (JCPolarRadarChartFormat)def;
	}
}

/**
 * Checks whether there are any properties different from their default.  If so,
 * then this saver has information to write.
 * @param prop property persistor object
 * @return whether any properties are different from their default
 */
public boolean checkProperties(PropertyPersistorModel prop)
{
    if (format == null || defaultFormat == null) {
		return(false);
    }

	yAxisAngle        = format.getYAxisAngle(JCChartUtil.DEGREES);
	originBase        = format.getOriginBase(JCChartUtil.DEGREES);
	radarCircularGrid = format.isRadarCircularGrid();
	halfRange         = format.isHalfRange();

	if (yAxisAngle != defaultFormat.getYAxisAngle(JCChartUtil.DEGREES) ||
	    originBase != defaultFormat.getOriginBase(JCChartUtil.DEGREES) ||
		radarCircularGrid != defaultFormat.isRadarCircularGrid()       ||
		halfRange != defaultFormat.isHalfRange())
	{
		return(true);
	}
	return(false);
}

/**
 * Saves HTML parameters from the current param source;
 * click the method name to see which properties are saved.<p>
 * Saves the following properties
 *  <pre>
 *   yAxisAngle (double)
 *   originBase (double)
 *   radarCircularGrid (boolean)
 *   halfRange (boolean)
 *  </pre>
 * @param prop property persistor object
 * @param name name of object to be saved
 * @param tab the tab indent for this group of properties
 * @throws JCIOException
 */
public void saveProperties(PropertyPersistorModel prop, String name, int tab)
    throws JCIOException
{
	if (format == null || defaultFormat == null) {
		System.out.println("FAILURE: No Polar/Radar format set");
		return;
	}

	// Check if something to do.
	if (!checkProperties(prop)) {
		return;
	}

	int newTab = prop.writeBegin("polar-radar-format", tab);

	// Set y Axis Angle:
	if (yAxisAngle != defaultFormat.getYAxisAngle(JCChartUtil.DEGREES)) {
		prop.writeProperty(name, "yAxisAngle", newTab,
		                   new Double(yAxisAngle));
	}

	// Set origin base:
	if (originBase != defaultFormat.getOriginBase(JCChartUtil.DEGREES)) {
		prop.writeProperty(name, "originBase", newTab,
		                   new Double(originBase));
	}

	// Set radar circular grid:
	if (radarCircularGrid != defaultFormat.isRadarCircularGrid()) {
		prop.writeProperty(name, "radarCircularGrid", newTab,
		                   new Boolean(radarCircularGrid));
	}

	// Set half range:
	if (halfRange != defaultFormat.isHalfRange()) {
		prop.writeProperty(name, "halfRange", newTab,
		                   new Boolean(halfRange));
	}
	prop.writeEnd(null, tab, true, false);
}

}
