/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCStringPersistor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:55 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

/**
 * Persists the properties of a chart to a String.  This can
 * be done in two ways.  Either provide a <code>StringBuffer</code> class
 * in the constructor which you can convert to a String
 * when you have finished writing properties with <code>writeProperty()</code>,
 * or use the other constructor (which creates a default
 * <code>StringBuffer</code>) and call <code>getParamString()</code> when you
 * are done.
 */
public abstract class JCStringPersistor extends JCBasePersistor {

protected StringBuffer    sb = null;

/**
 * Constructor which provides the source which holds the properties.
 * @param source the source (the Chart)
 */
public JCStringPersistor(Object source)
{
	this(new StringBuffer(), source);
}

/**
 * Constructor which provides the source (which holds the properties) and a
 * StringBuffer (to store the output of writeProperty).
 * @param sb String buffer
 * @param source the source (the Chart)
 */
public JCStringPersistor(StringBuffer sb, Object source)
{
    super(source);
    setStringBuffer(sb);
}

/**
 * Gets the StringBuffer in which the output is being written.
 * @return the currently used output String buffer
 */
public StringBuffer getStringBuffer()
{
	return(sb);
}

/**
 * Sets the StringBuffer in which the output is being written.
 * @param sb
 */
public void setStringBuffer(StringBuffer sb)
{
    if (sb == null) {
        throw new IllegalArgumentException("Provided StringBuffer may not " +
                                           "be null");
    }
    this.sb = sb;
}

}
