/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCSymbolStylePropertySave.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:55 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.chart.JCSymbolStyle;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.io.JCIOException;
import com.klg.jclass.util.swing.JCSwingTypeConverter;

/**
 * Responsible for saving properties of <code>JCSymbolStyle</code>
 * objects to a destiation specified by <code>PropertyPersistorModel</code>.
 * Not intended for public use.
 *
 * @see com.klg.jclass.chart.JCSymbolStyle
 * @see com.klg.jclass.chart.property.JCSymbolStylePropertyLoad
 */
public class JCSymbolStylePropertySave implements PropertySaveModel {

protected JCSymbolStyle style = null;
protected JCSymbolStyle defaultStyle = null;

/**
 * Sets the source component whose properties are to be saved.
 * @param o the new source component
 */
public void setSource(Object o)
{
	if (o instanceof JCSymbolStyle) {
		this.style = (JCSymbolStyle)o;
	}
}

/**
 * Sets default object instance.  This object is typically used by implementors
 * to determine whether a property really needs to be saved.
 * @param def the new default object
 */
public void setDefault(Object def)
{
	if (def instanceof JCSymbolStyle) {
		this.defaultStyle = (JCSymbolStyle)def;
	}
}

/**
 * Checks whether there are any properties different from their default.  If so,
 * then this saver has information to write.
 * @param prop property persistor object
 * @return whether any properties are different from their default
 */
public boolean checkProperties(PropertyPersistorModel prop)
{
	return(true);
}

/**
 * Click the method name to see which properties are saved.<p>
 * Saves the following properties:
 *   <pre>
 *   color (Color)
 *   shape (enum)
 *   size (int)
 * </pre>
 *
 * @param prop property persistor object
 * @param name name of object to be saved
 * @param tab the tab indent for this group of properties
 * @throws JCIOException
 */
public void saveProperties(PropertyPersistorModel prop, String name, int tab)
    throws JCIOException
{
	if (style == null || defaultStyle == null) {
		System.out.println("FAILURE: No symbol style set");
		return;
	}

	int newTab = prop.writeBegin("symbol-style", tab);

	// Always write out the color
	prop.writeProperty(name, "color", newTab,
					   JCSwingTypeConverter.fromColor(style.getColor()));

	// Write out the shape if its one of the standard ones.  Note that saving
	// and loading custom shapes (i.e. shape == JCSymbolStyle.OTHER) is not
	// supported
	String[] strings = JCChartEnumMappings.shape_strings;
	int[] values     = JCChartEnumMappings.shape_values;
	String shape     = JCTypeConverter.fromEnum(style.getShape(), strings,
					                            values);
	if (shape != null) {
	    prop.writeProperty(name, "shape", newTab, shape);
	}

	// Save size
	if (style.getSize() != defaultStyle.getSize()) {
		prop.writeProperty(name, "size", newTab,
		                   new Integer(style.getSize()));
	}
	prop.writeEnd(null, tab, true, false);
}

}

