/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PlotAreaPropertyLoad.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:55 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.PlotArea;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.ImageMapInfo;
import com.klg.jclass.util.io.JCIOException;
import com.klg.jclass.util.swing.JCSwingTypeConverter;

/**
 * Responsible for loading properties of PlotArea
 * objects using a specified PropertyAcessModel. Not really
 * intended for public consumption.
 *
 * @see com.klg.jclass.chart.PlotArea
 * @see com.klg.jclass.chart.property.PlotAreaPropertySave
 */
public class PlotAreaPropertyLoad implements PropertyLoadModel {

protected PlotArea plotArea = null;

/**
 * Sets the source object ie the object on which to set properties.
 *
 * @param o Object that is loading properties
 */
public void setSource(Object o)
{
	if (o instanceof PlotArea) {
		this.plotArea = (PlotArea)o;
	}
}

/**
 * Loads properties using the given property accessor object
 *
 * @param prop Property access object.
 * @param name Name of object to be loaded.
 * @throws JCIOException
 */
public void loadProperties(PropertyAccessModel prop, String name)
    throws JCIOException
{
	if (plotArea == null) {
		System.out.println("FAILURE: No plot area set");
		return;
	}

	//
	// PlotArea is different from other objects in that it inherits
	// color from its parent.  Therefore only set values if a value
	// appears in the property list.
	//

    // Set foreground
	String pval = prop.getProperty(name + "foreground");
	if (pval != null) {
		plotArea.setForeground(JCSwingTypeConverter.toColor(pval));
	}

    // Set background
	pval = prop.getProperty(name + "background");
	if (pval != null) {
		plotArea.setBackground(JCSwingTypeConverter.toColor(pval));
	}

    // Set left placement
	pval = prop.getProperty(name + "left");
	if (pval != null ) {
		try {
			if (Integer.valueOf(pval).intValue() == 0) {
				plotArea.setLeft(0);
				plotArea.setLeftIsDefault( false );
			}
			else {
				plotArea.setLeft(JCTypeConverter.toInt(pval,
													   plotArea.getLeft()));
			}
		}
		catch (NumberFormatException e) {}
	}

    // Set right placement
	pval = prop.getProperty(name + "right");
	if (pval != null ) {
		try {
			if (Integer.valueOf(pval).intValue() == 0) {
				plotArea.setRight(0);
				plotArea.setRightIsDefault( false );
			}
			else {
				plotArea.setRight(JCTypeConverter.toInt(pval,
													    plotArea.getRight()));
			}
		}
		catch (NumberFormatException e) {}
	}

    // Set top placement
	pval = prop.getProperty(name + "top");
	if (pval != null ) {
		try {
			if (Integer.valueOf(pval).intValue() == 0) {
				plotArea.setTop(0);
				plotArea.setTopIsDefault( false );
			}
			else {
				plotArea.setTop(JCTypeConverter.toInt(pval,
													  plotArea.getTop()));
			}
		}
		catch (NumberFormatException e) {}
	}

    // Set bottom placement
	pval = prop.getProperty(name + "bottom");
	if (pval != null ) {
		try {
			if (Integer.valueOf(pval).intValue() == 0) {
				plotArea.setBottom(0);
				plotArea.setBottomIsDefault( false );
			}
			else {
				plotArea.setBottom(JCTypeConverter.toInt(pval,
													     plotArea.getBottom()));
			}
		}
		catch (NumberFormatException e) {}
	}

    // Load the plotArea's imageMapInfo, if set.
    ImageMapInfo info = new ImageMapInfo();
    PropertyLoadFactory.load(prop, info, name + "imageMapInfo.");
    if (!info.isEmpty()) {
        plotArea.setImageMapInfo(info);
    }
}

}
