/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PlotAreaPropertySave.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:55 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.chart.PlotArea;
import com.klg.jclass.util.swing.JCSwingTypeConverter;
import com.klg.jclass.util.io.JCIOException;
import com.klg.jclass.util.ImageMapInfo;

/**
 * Responsible for saving properties of PlotArea
 * objects to a destiation specified by PropertyPersistorModel.
 * Not really intended for public consumption.
 *
 * @see com.klg.jclass.chart.PlotArea
 * @see com.klg.jclass.chart.property.PlotAreaPropertyLoad
 */
public class PlotAreaPropertySave implements PropertySaveModel {

protected PlotArea plotArea = null;
protected PlotArea defaultPlotArea = null;

/**
 * Set the source component whose properties are to be saved
 * @param o The new source component
 */
public void setSource(Object o)
{
	if (o instanceof PlotArea) {
		this.plotArea = (PlotArea)o;
	}
}

/**
 * Sets default object instance.  This object is typically used by implementors
 * to determine whether a property really needs to be saved.
 * @param def The new default object
 */
public void setDefault(Object def)
{
	if (def instanceof PlotArea) {
		this.defaultPlotArea = (PlotArea)def;
	}
}

/**
 * Check whether are any properties different from their default.  If so, then
 * then this saver will have something to write
 * @param prop Property persistor object
 * @return Whether any properties are different from their default.
 */
public boolean checkProperties(PropertyPersistorModel prop)
{
	if (plotArea == null || defaultPlotArea == null) {
		return(false);
	}

    ImageMapInfo info = plotArea.getImageMapInfo();
    if (info != null && !info.isEmpty()) {
        return(true);
    }

	// Check if anything to do
	if (plotArea.getForeground() == defaultPlotArea.getForeground()  &&
	    plotArea.getBackground() == defaultPlotArea.getBackground()  &&
	    plotArea.getLeftIsDefault()                                  &&
	    plotArea.getRightIsDefault()                                 &&
	    plotArea.getTopIsDefault()                                   &&
	    plotArea.getBottomIsDefault())
	{
		return(false);
	}
	return(true);
}

/**
 * Click the method name to see which properties are saved.<p>
 * Saves the following properties:
 *   <pre>
 *   foreground (Color)
 *   background (Color)
 *   left (int)
 *   right (int)
 *   top (int)
 *   bottom (int)
 * </pre>
 * @param prop Property persistor object
 * @param name Name of object to be saved
 * @param tab The tab indent for this group of properties
 * @throws JCIOException
 */
public void saveProperties(PropertyPersistorModel prop, String name, int tab)
    throws JCIOException
{
	if (plotArea == null || defaultPlotArea == null) {
		System.out.println("FAILURE: No plot area set");
		return;
	}

	// Check if something to do.
	if (!checkProperties(prop)) {
		return;
	}

	int newTab = prop.writeBegin("plot-area", tab);
	// Save foreground
	if (plotArea.getForeground() != defaultPlotArea.getForeground()) {
		prop.writeProperty(name, "foreground", newTab,
				   JCSwingTypeConverter.fromColor(plotArea.getForeground()));
	}

	// Save background
	if (plotArea.getBackground() != defaultPlotArea.getBackground()) {
		prop.writeProperty(name, "background", newTab,
				   JCSwingTypeConverter.fromColor(plotArea.getBackground()));
	}

	// Set Left placement
	if (!plotArea.getLeftIsDefault()) {
	    prop.writeProperty(name, "left", newTab,
		                   new Integer(plotArea.getLeft()));
	}

	// Set Right placement
	if (!plotArea.getRightIsDefault()) {
	    prop.writeProperty(name, "right", newTab,
		                   new Integer(plotArea.getRight()));
	}

	// Set Top placement
	if (!plotArea.getTopIsDefault()) {
	    prop.writeProperty(name, "top", newTab,
		                   new Integer(plotArea.getTop()));
	}

	// Set Bottom placement
	if (!plotArea.getBottomIsDefault()) {
	    prop.writeProperty(name, "bottom", newTab,
		                   new Integer(plotArea.getBottom()));
	}

    ImageMapInfo info = plotArea.getImageMapInfo();
    if (info != null && !info.isEmpty()) {
        prop.writeEnd(null, tab, true, true);
        // Save ImageMapInfo properties
        PropertySaveFactory.save(prop, info, new ImageMapInfo(),
                                 name + "imageMapInfo.", tab);
        prop.writeEnd("plot-area", tab, true, false);
    }
    else {
	    prop.writeEnd(null, tab, true, false);
    }
}

}
