/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PropertyAccessModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:55 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

import com.klg.jclass.util.io.LoadProperties;
import com.klg.jclass.util.io.OutputProperties;
import com.klg.jclass.util.io.JCParseException;

import java.net.URL;
import java.net.MalformedURLException;
import java.io.IOException;

/**
 * A model for accessing property values based on property names.
 */
public interface PropertyAccessModel {

/**
 * Get the properties from the string, file, etc, parse them, and store them.
 * Later a loader, actually loads the parsed properties into the chart in the
 * order it needs.
 * @param o The object from which to get the properties (fileName, inputStream,
 * reader, ...)
 * @throws IOException
 * @throws JCParseException
 */
public void getProperties(Object o) throws IOException, JCParseException;

/**
 * Get the accessor type string
 * @return The accessor type string
 */
public String getType();

/**
 * Get the accessor subDirectory (if any)
 * @return The accessor subDirectory (null if no subDirectory)
 */
public String getSubDirectory();

/**
 * Get property as a string value based on specified key
 *
 * @param key Name of property to be retrieved.
 * @return Property value as a string
 *
 * @see com.klg.jclass.chart.property.html.JCAppletAccessor
 * @see com.klg.jclass.chart.property.JCBaseAccessor
 * @see com.klg.jclass.chart.property.JCFileAccessor
 * @see com.klg.jclass.chart.property.JCStringAccessor
 */
public String getProperty(String key);

/**
 * Set the property to a string value based on a specified key
 * @param key Name of property to be set.
 * @param value The new string value of the property
 *
 * @see com.klg.jclass.chart.property.JCBaseAccessor
 */
public void setProperty(String key, String value);

/**
 * Get property value as an object based on specified key
 *
 * @param key Name of property to be retrieved.
 * @return Property value
 *
 * @see com.klg.jclass.chart.property.html.JCAppletAccessor
 * @see com.klg.jclass.chart.property.JCBaseAccessor
 * @see com.klg.jclass.chart.property.JCFileAccessor
 * @see com.klg.jclass.chart.property.JCStringAccessor
 */
public Object getPropertyObject(String key);

/**
 * Set the property to value based on a specified key
 * @param key Name of property to be set.
 * @param value The new value of the property
 *
 * @see com.klg.jclass.chart.property.JCBaseAccessor
 */
public void setPropertyObject(String key, Object value);

/**
 * Get URL based on specified key.
 * @param key Name of property to be retrieved as URL
 * @param outputProperties Object which records the output properties to be
 * used from loading in the input properties
 * @return Property value as URL.
 * @throws MalformedURLException
 */
public URL getPropertyURL(String key, OutputProperties outputProperties)
    throws MalformedURLException;

/**
 * Store an instance of LoadProperties containing the information
 * that will be used during the load of component properties.
 * @param loadProperties instance of LoadProperties
 */
public void setLoadProperties(LoadProperties loadProperties);

/**
 * Get the instance of LoadProperties
 * that will be used during the load of component properties.
 * @return instance of LoadProperties
 */
public LoadProperties getLoadProperties();

}
