/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PropertyPersistorModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:55 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property;

/**
 * A model for persisting properties of an object.
 */
public interface PropertyPersistorModel {

/**
 * Get the persistor type string
 * @return The persistor type string
 */
public String getType();

/**
 * Get the peristor subDirectory (if any)
 * @return The persistor subDirectory (null if no subDirectory)
 */
public String getSubDirectory();

/**
 * Store a helper object.
 * @param helper A helper object.
 */
public void setHelperObject(Object helper);

/**
 * Get the helper object set with setHelperObject().
 */
public Object getHelperObject();

/**
 * Persists the string property using the value provided
 *
 * @param name Class hierarchy name
 * @param property Property name
 * @param tab The amount to tab before writing property
 * @param value Actual object to persist.
 */
public void writeProperty(String name, String property, int tab, Object value);

/**
 * Write a beginning to a list of properties
 * @param name The name of the group of properties
 * @param tab The amount to tab before writing the beginning
 * @return A new tabstop based on output string
 */
public int writeBegin(String name, int tab);

/**
 * Write a ending to a list of properties
 * @param name The name of the group of properties
 * @param tab The amount to tab before writing the ending
 * @param newLine If true, a newLine character is printed at the end
 * @param partial Is this a partial ending?
 */
public void writeEnd(String name, int tab, boolean newLine, boolean partial);

/**
 * Write the string to the output stream
 * @param s The string to write
 * @param tab The number of spaces to tab over before writing
 */
public void writeString(String s, int tab);

/**
 * Expands the given string to escape reserved characters
 * @param s The string to expand
 * @return The newly expanded string.
 */
public String expandText(String s);

}
