/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCAppletAccessor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:55 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property.html;

import com.klg.jclass.util.io.LoadProperties;
import com.klg.jclass.util.io.OutputProperties;
import com.klg.jclass.util.io.JCParseException;
import com.klg.jclass.chart.data.ChartDataSourceUtil;
import com.klg.jclass.chart.property.PropertyAccessModel;

import java.applet.Applet;
import java.awt.*;
import java.net.URL;
import java.net.MalformedURLException;
import java.io.IOException;

/**
 * A property accessor class that gets properties from an applet.
 * @see com.klg.jclass.chart.property.PropertyAccessModel
 */
public class JCAppletAccessor implements PropertyAccessModel {

protected Applet          applet;
protected Component       component;
protected LoadProperties  loadProperties;

/**
 * Constructor which specifies the applet and the component from which to get
 * properties.
 * @param a the applet
 * @param c the component from which to get properties
 */
public JCAppletAccessor(Applet a, Component c)
{
	applet         = a;
	component      = c;
	loadProperties = new LoadProperties();
}

/**
 * Constructor which specifies the applet, the component from which to get
 * properties, and an instance of LoadProperties
 * @param a the applet
 * @param c the component from which to get properties
 */
public JCAppletAccessor(Applet a, Component c, LoadProperties loadProperties)
{
    applet = a;
    component = c;
    this.loadProperties = loadProperties;
}

/**
 * Gets the accessor type String.
 * @return the accessor type String
 */
public String getType()
{
	return("HTML");
}

/**
 * Gets the accessor subDirectory (if any).
 * @return the accessor subDirectory (null if no subDirectory)
 */
public String getSubDirectory()
{
	return("html");
}

/**
 * Returns the applet from which this accessor gets its property values.
 * @return the applet for this accessor
 */
public Applet getApplet()
{
	return(applet);
}

/**
 * Not necessary here as the properties are gotten one at a time from the
 * applet in the getProperty method.
 *
 * Get the properties from the string, file, etc, parse them, and store them.
 * Later a loader, actually loads the parsed properties into the chart in the
 * order it needs.
 * @param o The object from which to get the properties (fileName, inputStream,
 * reader, ...)
 * @throws IOException
 * @throws JCParseException
 * @see #getProperty
 */
public void getProperties(Object o) throws IOException, JCParseException {}

/**
 * Gets a property value as a String based on specified key.
 *
 * @param key name of property to be retrieved
 * @return property value as a String
 */
public String getProperty(String key)
{
	if (applet == null) {
		return(null);
	}
	String rval = applet.getParameter(key);
	if (rval == null && component != null) {
		rval = applet.getParameter(component.getName() + "." + key);
	}
	rval = ChartDataSourceUtil.processText(rval);
	return(rval);
}

/**
 * Gets property value as an object based on specified key.  <br><b>Note:</b>
 * The property type is always a String, so this just returns what
 * <code>getProperty()</code> would return.
 *
 * @param key name of property to be retrieved
 * @return property value
 */
public Object getPropertyObject(String key)
{
	return(getProperty(key));
}

/**
 * Empty method. It does not make sense to set a property here.
 * @param key name of property to be set
 * @param value the new value of the property
 */
public void setProperty(String key, String value) {}

/**
 * Empty method. It does not make sense to set a property here.
 * @param key name of property to be set.
 * @param value the new value of the property
 */
public void setPropertyObject(String key, Object value) {}

/**
 * Get URL based on specified key.
 * @param key Name of property to be retrieved as URL
 * @return Property value as URL.
 * @throws MalformedURLException
 */
public URL getPropertyURL(String key, OutputProperties outputProperties)
    throws MalformedURLException
{
    String fileName = getProperty(key + "image.fileName");
    if (fileName == null || fileName.length() == 0) {
        fileName = getProperty(key + "image");
        if (fileName == null || fileName.length() == 0) {
            return(null);
        }
    }
    if (outputProperties != null) {
        outputProperties.setPropertyName(fileName);
        outputProperties.setFileAccess(OutputProperties.ABSOLUTE);
    }
    URL url = applet.getDocumentBase();
    return(new URL(url, fileName));
}

/**
 * Store an instance of LoadProperties containing the information
 * that will be used during the load of component properties.
 * @param loadProperties instance of LoadProperties
 */
public void setLoadProperties(LoadProperties loadProperties)
{
    this.loadProperties = loadProperties;
}

/**
 * Get the instance of LoadProperties
 * that will be used during the load of component properties.
 * @return instance of LoadProperties
 */
public LoadProperties getLoadProperties()
{
    return(loadProperties);
}

}
