/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCAxisHTMLPropertySave.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:55 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property.html;

import com.klg.jclass.chart.JCGrid;
import com.klg.jclass.chart.JCAnno;
import com.klg.jclass.chart.JCChartEnumMappings;
import com.klg.jclass.chart.property.JCAxisPropertySave;
import com.klg.jclass.chart.property.PropertyPersistorModel;
import com.klg.jclass.chart.property.PropertySaveFactory;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.io.JCIOException;
import com.klg.jclass.util.swing.JCSwingTypeConverter;

import java.util.List;

/**
 * Responsible for saving properties of <code>JCAxis</code>
 * objects to a destination specified by <code>PropertyPersistorModel</code>.
 * Not intended for public use.
 *
 * @see com.klg.jclass.chart.JCAxis
 * @see com.klg.jclass.chart.property.JCAxisPropertyLoad
 */
public class JCAxisHTMLPropertySave extends JCAxisPropertySave {


protected void saveAxisGridProperties(PropertyPersistorModel prop, String name,
                                      int tab)
{
    // Save axis.gridVisible
    if (axis.isGridVisible() != defaultAxis.isGridVisible()) {
        prop.writeProperty(name, "grid.visible", tab,
                           new Boolean(axis.isGridVisible()));
    }

    // Save axis.gridDefault
    if (!axis.isGridDefault()) {
        prop.writeProperty(name, "grid.default", tab,
                           new Boolean(axis.isGridDefault()));
    }
}

protected void saveGridProperties(PropertyPersistorModel prop, String name,
                                  int tab, int newTab) throws JCIOException
{
    List gridList = axis.getGridList();
    if (gridList != null) {
        int nGrids = gridList.size();
        for (int i=0; i < nGrids; i++) {
            JCGrid grid = (JCGrid)gridList.get(i);
            if (!checkGridProperties(grid)) {
                continue;
            }
            String gridStr = "grid" + (i + 1);
            String value = grid.getName();
            // If the grid object doesn't have a name, assign one.
            if (value == null) {
                value = gridStr;
            }
            prop.writeProperty(name + gridStr + ".", "gridName",
                               newTab, value);
            String propName = name + value + ".";

            if (!grid.getStartValueIsDefault()) {
                value = Double.toString(grid.getStartValue());
                prop.writeProperty(propName, "startValue", newTab, value);
            }

            if (!grid.getStopValueIsDefault()) {
                value = Double.toString(grid.getStopValue());
                prop.writeProperty(propName, "stopValue", newTab, value);
            }

            if (!grid.getIncrementIsDefault()) {
                value = Double.toString(grid.getIncrementValue());
                prop.writeProperty(propName, "incrementValue", newTab, value);
            }

            if (!isDefaultGridLineStyle(grid)) {
                PropertySaveFactory.save(prop, grid.getGridStyle().getLineStyle(),
                                         JCGrid.makeDefaultLineStyle(axis),
                                         propName,
                                         tab + PropertySaveFactory.tabIncrement);
            }
        }
    }
}

protected void saveTypeAndName(PropertyPersistorModel prop, String name,
                               boolean isXAxis, int tab)
{
    // Save name for axes with indices 1 or greater.  No need to save type.
    String axName = axis.getName();
    if (axName == null || axName.length() <= 5) {
        return;
    }
    String axisIndex = axName.substring(5);
    String axisPropertyName = isXAxis ? "xaxisName" : "yaxisName";
    prop.writeProperty("chartArea.", axisPropertyName + axisIndex, tab,
                       prop.expandText(axName));
}

protected void saveAnnoProperties(PropertyPersistorModel prop,
                                  String name, int tab, int newTab)
{
    List annoList = axis.getAnnotationList();
    if (annoList != null) {
        int nAnno = annoList.size();
        for (int i=0; i < nAnno; i++) {
            JCAnno anno = (JCAnno)annoList.get(i);
            // Save all user-defined anno objects, as well as
            // modified default anno objects.
            if (anno == null ||
                (anno.getType() != JCAnno.USER_DEFINED &&
                !checkAnnoProperties(anno)))
            {
                continue;
            }
            String annoStr = "anno" + (i + 1);
            String value = anno.getName();
            // If the anno object doesn't have a name, assign one.
            if (value == null) {
                value = annoStr;
            }
            prop.writeProperty(name + annoStr + ".", "annoName",
                               newTab, value);
            String propName = name + value + ".";

            value = JCTypeConverter.fromEnum(anno.getType(),
                        JCChartEnumMappings.anno_type_strings,
                        JCChartEnumMappings.anno_type_values);
            prop.writeProperty(propName, "type", newTab, value);

            if (!anno.getStartValueIsDefault()) {
                value = Double.toString(anno.getStartValue());
                prop.writeProperty(propName, "startValue", newTab, value);
            }

            if (!anno.getStopValueIsDefault()) {
                value = Double.toString(anno.getStopValue());
                prop.writeProperty(propName, "stopValue", newTab, value);
            }

            if (!isDefaultIncrement(anno)) {
                value = Double.toString(anno.getIncrementValue());
                prop.writeProperty(propName, "incrementValue", newTab, value);
            }

            if (!isDefaultPrecision(anno)) {
                value = Integer.toString(anno.getPrecision());
                prop.writeProperty(propName, "precision", newTab, value);
            }

            if (!anno.getDrawLabelsIsDefault()) {
                value = Boolean.toString(anno.getDrawLabels());
                prop.writeProperty(propName, "drawLabels", newTab, value);
            }

            if (!anno.getDrawTicksIsDefault()) {
                value = Boolean.toString(anno.getDrawTicks());
                prop.writeProperty(propName, "drawTicks", newTab, value);
            }

            if (!anno.getLabelExtentIsDefault()) {
                value = Double.toString(anno.getLabelExtent());
                prop.writeProperty(propName, "labelExtent", newTab, value);
            }

            if (!anno.getInnerExtentIsDefault()) {
                value = Integer.toString(anno.getInnerExtent());
                prop.writeProperty(propName, "innerExtent", newTab, value);
            }

            if (!anno.getOuterExtentIsDefault()) {
                value = Integer.toString(anno.getOuterExtent());
                prop.writeProperty(propName, "outerExtent", newTab, value);
            }

            if (anno.getTickColor() != null &&
                !anno.getTickColor().equals(axis.getForeground()))
            {
                value = JCSwingTypeConverter.fromColor(anno.getTickColor());
                prop.writeProperty(propName, "tickColor", newTab, value);
            }

            if (anno.getLabelColor() != null &&
                !anno.getLabelColor().equals(axis.getForeground()))
            {
                value = JCSwingTypeConverter.fromColor(anno.getLabelColor());
                prop.writeProperty(propName, "labelColor", newTab, value);
            }
        }
    }
}

}
