/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCChartHTMLPropertySave.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:55 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property.html;

import com.klg.jclass.chart.*;
import com.klg.jclass.chart.property.JCChartPropertySave;
import com.klg.jclass.chart.property.PropertyPersistorModel;
import com.klg.jclass.chart.property.PropertySaveFactory;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.io.JCIOException;

import java.util.ListIterator;
import java.util.Locale;
import java.awt.*;

/**
 * Responsible for saving properties of <code>JCChart</code>
 * objects to a destiation specified by <code>PropertyPersistorModel</code>.
 * Not intended for public use.
 *
 * @see com.klg.jclass.chart.JCChart
 * @see com.klg.jclass.chart.property.JCChartPropertyLoad
 */
public class JCChartHTMLPropertySave extends JCChartPropertySave {

/**
 * Saves the header for the chart.
 * @param prop property persistor object
 * @param tab the tab indent
 */
protected void saveHeader(PropertyPersistorModel prop, int tab) {}

/**
 * Saves the properties for the given data view.
 * @param view the <code>ChartDataView</code> to save
 * @param defaultView The default <code>ChartDataView</code> to compare
 * @param numString the String which indicates the data view number
 * @param prop property persistor object
 * @param name name of object to be saved
 * @param tab the tab indent for this group of properties
 */
protected void saveDataView(ChartDataView view, ChartDataView defaultView,
                            String numString, PropertyPersistorModel prop,
							String name, int tab)
    throws JCIOException
{
    String dvName = name + "data" + numString;
	if (numString != null && numString.length() > 0) {
	    prop.writeProperty(name, "dataName" + numString, tab, dvName);
	}
	super.saveDataView(view, defaultView, numString, prop, name, tab);
}

/**
 * Saves the chart labels for the chart.
 * @param prop property persistor object
 * @param name name of object to be saved
 * @param tab the tab indent for this group of properties
 */
protected void saveChartLabels(PropertyPersistorModel prop, String name,
                               int tab)
    throws JCIOException
{
    // Get JCDataIndices for the labels
    // No default DataIndex set
    // Write last data index value out as property lastLabelIndex

    int indexNum = 1;
	JCChartLabelManager mn = chart.getChartLabelManager();
	if (mn != null && mn.getChartLabels() != null) {
		JCChartLabel dflt = new JCChartLabel();
		for (ListIterator li = mn.getChartLabels().listIterator();
		     li.hasNext();
			 indexNum++)
		{
			JCChartLabel lbl = (JCChartLabel)li.next();
			if (indexNum == 1) {
				PropertySaveFactory.refresh(prop, lbl, dflt,
											"label" + indexNum + ".");
			}

			prop.writeProperty(name, "labelName" + indexNum, tab,
							   "label" + indexNum);

			PropertySaveFactory.save(prop, lbl, dflt,
			                         "label" + indexNum + ".", tab);
		}
		prop.writeProperty(name, "lastLabelIndex", tab,
		                   "" + (indexNum - 1));
	}
}

/**
 * Saves all event triggers from the <code>PropertyPersistorModel</code>.
 * @param prop property persistor object
 * @param name name of object to be saved
 * @param tab the tab indent for this group of properties
 */
protected void saveEventTriggers(PropertyPersistorModel prop, String name,
                                 int tab)
{
	if (chart == null) {
		return;
	}

    // Set EventTriggers
	for (int i = 0; i < chart.getNumTriggers(); i++) {
		EventTrigger et = chart.getTrigger(i);
		if (et == null) {
			continue;
		}

	    String[] strings   = JCChartEnumMappings.trigger_strings;
	    int[] values       = JCChartEnumMappings.trigger_values;
		String triggerName = JCTypeConverter.fromEnum(et.getAction(),
		                                              strings, values);

	    strings        = JCChartEnumMappings.eventMod_strings;
	    values         = JCChartEnumMappings.eventMod_values;
		String modName = JCTypeConverter.fromEnum(et.getModifiers(),
									              strings, values);

		prop.writeProperty(name, triggerName + "Trigger", tab, modName);
	}

    // Set Keys
	if (chart.getResetKey() != defaultChart.getResetKey()) {
		prop.writeProperty(name, "resetKey", tab,
		                   new Integer(chart.getResetKey()));
	}
	if (chart.getCancelKey() != defaultChart.getCancelKey()) {
		prop.writeProperty(name, "cancelKey", tab,
		                   new Integer(chart.getCancelKey()));
	}
}

/**
 * Saves locale info from the <code>PropertyPersistorModel</code>.
 * @param prop property persistor object
 * @param name name of object to be saved
 * @param tab the tab indent for this group of properties
 */
protected void saveLocale(PropertyPersistorModel prop, String name, int tab)
{
	if (chart == null) {
		return;
	}

    Locale chartLocale = chart.getLocale();

    // If using default locale, no need to write out locale
    if (chartLocale == Locale.getDefault()) {
        return;
    }

    String localeLanguage = chartLocale.getLanguage();
    if (localeLanguage == null) {
        localeLanguage = "";
    }

    String localeCountry = chartLocale.getCountry();
    String localeVariant = chartLocale.getVariant();

    // Write language attribute
    prop.writeProperty(name, "locale.language", tab, localeLanguage);

    // Don't write country attribute if one isn't specified
    if (localeCountry != null && !localeCountry.equals("")) {
    	prop.writeProperty(name, "locale.country", tab, localeCountry);
    }

    // Don't write variant attribute if one isn't specified
    if (localeVariant != null && !localeVariant.equals("")) {
    	prop.writeProperty(name, "locale.variant", tab, localeVariant);
    }

    // Close tag
    prop.writeEnd(null, tab, true, false);
}

public static void savePointList(PropertyPersistorModel prop, Point[] pointList,
                                 String name, String propertyName, int tab)
{
    if (pointList == null || pointList.length == 0) {
        return;
    }

    String[] dataStrings = JCChartEnumMappings.data_format_strings;
    String allString = dataStrings[0];
    String otherString = dataStrings[1];
    String string = JCTypeConverter.fromPointList(pointList, '|');
    String all = Integer.toString(JCPieChartFormat.ALL);
    string = string.replaceAll(all, allString);
    String other = Integer.toString(JCPieChartFormat.OTHER_SLICE);
    string = string.replaceAll(other, otherString);
    prop.writeProperty(name, propertyName, tab, string);
}

/**
 * Saves the image map rules for the chart.
 * @param prop property persistor object
 * @param name name of object to be saved
 * @param tab the tab indent for this group of properties
 */
protected void saveImageMapRules(PropertyPersistorModel prop, String name,
                                 int tab)
    throws JCIOException
{
}

}
