/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCChartLabelHTMLPropertySave.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:55 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property.html;

import com.klg.jclass.chart.*;
import com.klg.jclass.chart.property.JCChartLabelPropertySave;
import com.klg.jclass.chart.property.PropertyPersistorModel;
import com.klg.jclass.chart.property.PropertySaveFactory;
import com.klg.jclass.chart.property.JCChartPropertySave;
import com.klg.jclass.util.io.JCIOException;

import java.awt.*;

/**
 * Responsible for saving properties of <code>JCChartLabel</code>
 * objects to a destiation specified by <code>PropertyPersistorModel</code>.
 * Not intended for public use.
 *
 * @see com.klg.jclass.chart.JCChartLabel
 * @see com.klg.jclass.chart.property.JCChartLabelPropertyLoad
 */
public class JCChartLabelHTMLPropertySave extends JCChartLabelPropertySave {

protected String getDataViewName()
{
    return(JCChartPropertySave.getDataViewName(label.getParentChart(),
                                               label.getDataView()));
}

public void saveDataViewProperty(PropertyPersistorModel prop, String name,
                                 int tab)
{
    ChartDataView userDataView = label.getSetDataView();
    // Only save data view property if it was explicitly set
    if (userDataView != null) {
		prop.writeProperty(name, "dataView", tab, getDataViewName());
	}
}

public void saveAttachProperties(PropertyPersistorModel prop, String name,
                                 int tab)
    throws JCIOException
{
	switch (label.getAttachMethod()) {
		case JCChartLabel.ATTACH_COORD:
			// Set coord
			Point current = label.getCoord();
			Point dflt = defaultLabel.getCoord();
			if (current.x != dflt.x || current.y != dflt.y) {
				prop.writeProperty(name, "coord", tab,
				                   current.x + "," + current.y);
			}
			break;
		case JCChartLabel.ATTACH_DATACOORD:
			JCDataCoord currentDC = label.getDataCoord();
			JCDataCoord defaultDC = defaultLabel.getDataCoord();
			if (currentDC.getX() != defaultDC.getX() ||
				currentDC.getY() != defaultDC.getY())
			{
				prop.writeProperty(name, "dataAttachX", tab,
								   new Double(currentDC.getX()));
				prop.writeProperty(name, "dataAttachY", tab,
								   new Double(currentDC.getY()));
			}
			break;
		case JCChartLabel.ATTACH_DATAINDEX:
		    String attachIndexName =  "attachIndex" + dataIndex;

			// Write out an indexname for the data index so all the data indices
			// can be found when loaded.  Then write out a data index name for
			// the label so the label knows which data index it needs.
			prop.writeProperty("", "indexName" + dataIndex, tab,
							   attachIndexName);
			prop.writeProperty(name, "dataIndex", tab, attachIndexName);

			// Finally, write out the data index (starting with the dataview)
			prop.writeProperty("", attachIndexName + ".dataView", tab,
			                   getDataViewName());
			PropertySaveFactory.save(prop, label.getDataIndex(),
			                         new JCDataIndex(),
									 attachIndexName + ".", tab);
			dataIndex++;
			break;
	}
}

}
