/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCHTMLStringPersistor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:55 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart.property.html;

import com.klg.jclass.chart.data.ChartDataSourceUtil;
import com.klg.jclass.chart.property.JCStringPersistor;

/**
 * Persists the properties of a chart to a String.  This can
 * be done in two ways.  Either provide a <code>StringBuffer</code> class
 * in the constructor which you can then convert to a String
 * when you're done writing properties <code>writeProperty()</code>, or use the
 * other constructor (which creates a default <code>StringBuffer</code>) and
 * call <code>getParamString()</code> when you are done.
 */
public class JCHTMLStringPersistor extends JCStringPersistor {

/**
 * Constructor which provides the source which holds the property.
 * @param sr the source (the Chart)
 */
public JCHTMLStringPersistor(Object sr)
{
	super(sr);
}

/**
 * Constructor which provides the source which holds the property and a String
 * buffer to store the output of <code>writeProperty</code>.
 * @param sb String buffer
 * @param sr the source (the Chart)
 */
public JCHTMLStringPersistor(StringBuffer sb, Object sr)
{
	super(sb, sr);
}

/**
 * Gets the persistor type String.
 * @return the persistor type String
 */
public String getType()
{
	return("HTML");
}

/**
 * Gets the peristor subDirectory (if any).
 * @return the persistor subDirectory (null if no subDirectory)
 */
public String getSubDirectory()
{
	return("html");
}

/**
 * Persists the String property to an html param-value pair using the
 * value provided. <br><b>Note:</b> The tab parameter is ignored.
 *
 * @param name class hierarchy name
 * @param property property name
 * @param tab the amount to tab before writing property
 * @param value actual object to persist
 */
public void writeProperty(String name, String property, int tab, Object value)
{
   if (sb == null || value == null) {
      return;
   }

   sb.append("<PARAM NAME=\"");
   sb.append(name);
   sb.append(property);
   sb.append("\" VALUE=\"");
   sb.append(value.toString());
   sb.append("\">\n");
}

/**
 * Writes a beginning to a list of properties.
 * @param name the name of the group of properties
 * @param tab the amount to tab before writing the beginning
 * @return a new tabstop based on output String
 */
public int writeBegin(String name, int tab)
{
	return(tab);
}

/**
 * Writes a ending to a list of properties.
 * @param name the name of the group of properties
 * @param tab the amount to tab before writing the ending
 * @param newLine if <code>true</code>, a <code>newLine</code> character is
 *     printed at the end
 * @param partial Is this a partial ending?
 */
public void writeEnd(String name, int tab, boolean newLine, boolean partial) {}

/**
 * Writes the String to the output stream.
 * @param s the String to write
 * @param tab the number of spaces to tab over before writing
 */
public void writeString(String s, int tab)
{
	if (sb != null) {
		sb.append(s);
	}
}

/**
 * Gets the String buffer which holds the param/value output.
 * @return the String which which holds the param/value output
 */
public String getParamString()
{
   return(sb.toString());
}

/**
 * Expands the given String to escape reserved characters.
 * @param s the String to expand
 * @return the newly expanded String
 */
public String expandText(String s)
{
	return(ChartDataSourceUtil.expandText(s));
}

}
