/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Chart3dDataHandler.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:56 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import com.klg.jclass.chart3d.event.*;

/**
 * Data interpretation class (this class is used internally
 * by JClass Chart 3D and should not be used by applications).
 * Ambitious users could attempt to create and plug
 * in their own data handlers for specific handling,
 * but this is not common use.
 *
 */
public abstract class Chart3dDataHandler implements java.io.Serializable {

/**  */
protected Chart3dData data = null;

/**
 * Creates a data handler instance for the given data
 *
 * @param data The internal data object for this data handler
 */
public Chart3dDataHandler(Chart3dData data)
{
    this.data = data;
}

/**
 * Gets the internal data object for this data handler.
 *
 * @return The internal data object that this object handles
 */
public Chart3dData getData()
{
    return(data);
}

/**
 * Implements <code>Chart3dDataEvent.RELOAD_DATA_SOURCE_NAME
 * </code>.
 *
 * @param source
 * @see Chart3dDataEvent
 */
public boolean reloadDataSourceName(Chart3dDataModel source)
{
	if (source == null) {
        return(false);
	}
	if (source instanceof LabelledChart3dDataModel) {
		LabelledChart3dDataModel labelSource = (LabelledChart3dDataModel)source;
        data.setName(labelSource.getDataSourceName());
        return(true);
	}
    return(false);
}

/**
 * Implements <code>Chart3dDataEvent.RELOAD_HOLE_VALUE
 * </code>.
 *
 * @param source
 * @see Chart3dDataEvent
 */
public boolean reloadHoleValue(Chart3dDataModel source)
{
	if (source == null) {
        return(false);
	}
    if (source instanceof HoleValueChart3dDataModel) {
		HoleValueChart3dDataModel holeSource = (HoleValueChart3dDataModel)source;
        data.setHoleValue(holeSource.getHoleValue());
        return(true);
    }
    return(false);
}

/**
 * Implements <code>Chart3dDataEvent.RESET
 * </code>.
 *
 * @param source
 * @see Chart3dDataEvent
 */
public abstract boolean reset(Chart3dDataModel source);

/**
 * Implements <code>Chart3dDataEvent.RELOAD
 * </code>.
 *
 * @param source
 * @see Chart3dDataEvent
 */
public abstract boolean reload(Chart3dDataModel source);

/**
 * Given an input source and an data event, updates the parent internal data
 * object appropriately.
 *
 * @param source
 * @param event
 */
public abstract boolean handleEvent(Chart3dDataModel source,
				                    Chart3dDataEvent event);

/**
 * Modifies z value of data point in an <code>EditableChart3dDataModel</code>
 * instance.
 *
 * @param index Either a JCData3dGridIndex or a JCData3dPointIndex which
 *   indcates the index of the point being edited.
 * @param value The new z value
 * @return Whether the edit succeeded
 */
public abstract boolean editZValue(JCData3dIndex index, double value);

}
