/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Chart3dPointSeries.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:56 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import java.awt.Color;
import javax.vecmath.Point3d;

/**
 * Represents a series of points along with a <code>JCChart3dStyle</code> that
 * describes how to draw the points in this series.
 */
public class Chart3dPointSeries implements java.io.Serializable {

/** The array of points for this series. */
protected Point3d[]             points     = null;

/** The number of points for this series. */
protected int                   numPoints  = 0;

/** The parent point data object. */
protected Chart3dPointData      data       = null;

/** The <code>chartStyle</code> for this series. */
protected JCChart3dStyle        chartStyle = null;

/** The data label for this series - used in the legend. */
protected String                label      = null;

/**
 * Basic constructor. Parent and data arrays are set to null.
 */
public Chart3dPointSeries()
{
}

/**
 * Internal constructor specifying parent data and array holding data.
 *
 * @param data The parent point data object
 * @param points A list of points to intialize this series
 */
public Chart3dPointSeries(Chart3dPointData data, Point3d[] points)
{
	this.data       = data;
	this.points     = points;
	this.numPoints  = (points == null) ? 0 : points.length;
	if (data != null) {
		Chart3dDataView dataView = data.getDataView();
		this.chartStyle = JCChart3dStyle.makeDefault(dataView.getChart3d(),
													 dataView);
	}
	else {
		this.chartStyle = JCChart3dStyle.makeDefault(null);
	}
}

/**
 * Gets the parent <code>Chart3dPointData</code> for this series.
 *
 * @return The parent point data object
 */
public Chart3dPointData getData()
{
	return(data);
}

/**
 * Sets the parent <code>Chart3dPointData</code> for this series.
 *
 * @param data The new parent for this series
 */
public void setData(Chart3dPointData data)
{
	this.data = data;
	if (this.data != null) {
		this.data.setChanged(true, Changeable.RECALC |
								   Changeable.NOTIFY_LEGEND_RECALC);
	}
}

/**
 * Returns the point in the points array indexed by point.
 *
 * @param point The index of the point to be returned
 * @return The point indexed by point
 */
public Point3d getPoint(int point)
{
	return(points[point]);
}

/**
 * Returns the array of points.
 *
 * @return The array of points
 */
public Point3d[] getPoints()
{
	return(points);
}

/**
 * Sets the point array.
 *
 * @param points The new point array
 */
protected void setPoints(Point3d[] points)
{
	this.points = points;
	this.numPoints = (points == null) ? 0 : points.length;
}

/**
 * Returns the number of points in the point array (equivalent to
 * <code>points.length</code>).
 *
 * @return The number of points in the point array
 */
public int getNumPoints()
{
	return(numPoints);
}

/**
 * Sets the chart style for this series.
 *
 * @param chartStyle The new chart style for this series
 */
public void setChartStyle(JCChart3dStyle chartStyle)
{
	if (this.chartStyle == chartStyle) {
		return;
	}
	this.chartStyle = chartStyle;
	if (data != null) {
		if (this.chartStyle != null) {
	         this.chartStyle.addParent(data.getDataView());
		}
		data.setChanged(true, Changeable.REDRAW |
							  Changeable.NOTIFY_LEGEND_RECALC);
	}
}

/**
 * Gets the chart style for this series.
 *
 * @return The chart style for this series
 */
public JCChart3dStyle getChartStyle()
{
	if (this.chartStyle == null) {
		if (data != null) {
			Chart3dDataView dataView = data.getDataView();
		    chartStyle = JCChart3dStyle.makeDefault(dataView.getChart3d(),
											        dataView);
		}
		else {
		    chartStyle = JCChart3dStyle.makeDefault(null);
		}
	}
	return(chartStyle);
}

/**
 * Sets the series label.
 *
 * @param label The new series label
 */
public void setLabel(String label)
{
	if (this.label == label) {
		return;
	}
	this.label = label;
	if (data != null) {
		data.setChanged(true, Changeable.ALL);
	}
}

/**
 * Gets the series label.
 *
 * @return The current series label
 */
public String getLabel()
{
	return(label);
}

}


