/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ColorBin.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:56 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Polygon;

/**
 * Data structure internal to JClass Chart 3D.  Used to batch rectangles and polygons
 * of the same color together.  This is only used in 2D zones; if <code>zoneMethod</code>
 * is <code>ZONE_CELLS</code>, then rectangles are batched; otherwise, both polygons and
 * rectangles are batched.
 *
 */
public class ColorBin implements java.io.Serializable{

/**  */
public final static int MAX_BIN_SIZE = 100;

/**  */
protected int       binSize = 0;
/**  */
protected Object[]  fill = new Object[MAX_BIN_SIZE];
/**  */
protected Graphics  gc = null;
/**  */
protected int       level = -1;
/**  */
protected JCContour contour = null;

/**  */
ColorBin(JCContour contour, Graphics gc, int level)
{
	this.contour = contour;
	this.gc      = gc;
	this.level   = level;
}

/**
 * Adds an object to the color bin.
 *
 * @param obj
 */
public void addObjectToBin(Object obj)
{
	if (binSize == MAX_BIN_SIZE) {
		flushBin();
	}

	fill[binSize] = obj;
	binSize++;
}

/**
 * Get the value of bin size.
 * @return int The value of binSize
 */
public int getBinSize() {
    return binSize;
}

/**
 * Get the fill array of objects.
 * @return Objects[] The fill member of class
 */
public Object[] getFill() {
    return fill;
}

/**
 * Flushes the color bin.
 *
 */
protected void flushBin()
{
	contour.setFillStyle(gc, level);
	for (int i = 0; i < binSize; i++) {
		Object obj = fill[i];
	    if (obj instanceof Rectangle) {
		    Rectangle r = (Rectangle)obj;
		    gc.fillRect(r.x, r.y, r.width, r.height);
	    }
	    else if (obj instanceof Polygon) {
		    Polygon p = (Polygon)obj;
		    gc.fillPolygon(p);
	    }
	}
	binSize = 0;
}

}
