/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: Cube.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:12:56 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import javax.vecmath.Point3d;
import javax.vecmath.Point4d;
import javax.vecmath.Matrix4d;

public class Cube implements java.io.Serializable {

/**  * 
 */
static final long serialVersionUID = -5542417106751358122L;

// X axis annotation edges
/**  * 
 */
public final static int YNEG_ZNEG = 0;
/**  * 
 */
public final static int YNEG_ZPOS = 1;
/**  * 
 */
public final static int YPOS_ZNEG = 2;
/**  * 
 */
public final static int YPOS_ZPOS = 3;

// Y axis annotation edges
/**  * 
 */
public final static int XNEG_ZNEG = 4;
/**  * 
 */
public final static int XNEG_ZPOS = 5;
/**  * 
 */
public final static int XPOS_ZNEG = 6;
/**  * 
 */
public final static int XPOS_ZPOS = 7;

// Z axis annotation edges
/**  * 
 */
public final static int XNEG_YNEG = 8;
/**  * 
 */
public final static int XNEG_YPOS = 9;
/**  * 
 */
public final static int XPOS_YNEG = 10;
/**  * 
 */
public final static int XPOS_YPOS = 11;

// The six faces of the cube
/**  * 
 */
public final static int FACE_LEFT   = 0;
/**  * 
 */
public final static int FACE_RIGHT  = 1;
/**  * 
 */
public final static int FACE_BOTTOM = 2;
/**  * 
 */
public final static int FACE_TOP    = 3;
/**  * 
 */
public final static int FACE_FAR    = 4;
/**  * 
 */
public final static int FACE_NEAR   = 5;

// Cube constants
/**  * 
 */
public final static int NUM_POINTS         = 8;
/**  * 
 */
public final static int NUM_EDGES          = 12;
/**  * 
 */
public final static int NUM_FACES          = 6;
/**  * 
 */
public final static int NUM_POINTS_IN_FACE = 4;

// Codes for composing the viewing case code
/**  * 
 */
public final static int VP_XMAX = 1;
/**  * 
 */
public final static int VP_XMIN = 2;
/**  * 
 */
public final static int VP_YMAX = 4;
/**  * 
 */
public final static int VP_YMIN = 8;
/**  * 
 */
public final static int VP_ZMAX = 16;
/**  * 
 */
public final static int VP_ZMIN = 32;

/**  * 
 */
public final static int VP_XMASK	= 3;    // VP_XMAX | VP_XMIN
/**  * 
 */
public final static int VP_YMASK	= 12;   // VP_YMAX | VP_YMIN
/**  * 
 */
public final static int VP_ZMASK	= 48;   // VP_ZMAX | VP_ZMIN

// The points of the cube.  Note that its a double unit cube.
/**  * 
 */
protected final static double points[][] = {
    { -1.0, -1.0, -1.0 },
    { -1.0, -1.0,  1.0 },
    { -1.0,  1.0,  1.0 },
    { -1.0,  1.0, -1.0 },
    {  1.0,  1.0, -1.0 },
    {  1.0,  1.0,  1.0 },
    {  1.0, -1.0,  1.0 },
    {  1.0, -1.0, -1.0 }
};

// The faces of the cube by point number
/**  * 
 */
protected final static int faces[][] = {
	{ 0, 1, 2, 3 },		// FACE_LEFT
	{ 4, 5, 6, 7 },		// FACE_RIGHT
	{ 0, 7, 6, 1 },		// FACE_BOTTOM
	{ 2, 5, 4, 3 },		// FACE_TOP
	{ 0, 3, 4, 7 },		// FACE_FAR
	{ 1, 6, 5, 2 }		// FACE_NEAR
};

// The edges of the cube by point number
/**  * 
 */
protected final static int edges[][] = {
	{ 0, 7 },	// YNEG_ZNEG      X axis annotation edges
	{ 1, 6 },	// YNEG_ZPOS
	{ 3, 4 },	// YPOS_ZNEG
	{ 2, 5 },	// YPOS_ZPOS

	{ 0, 3 },	// XNEG_ZNEG      Y axis annotation edges
	{ 1, 2 },	// XNEG_ZPOS
	{ 7, 4 },	// XPOS_ZNEG
	{ 6, 5 },	// XPOS_ZPOS

	{ 0, 1 },	// XNEG_YNEG      Z axis annotation edges
	{ 3, 2 },	// XNEG_YPOS
	{ 7, 6 },	// XPOS_YNEG
	{ 4, 5 }	// XPOS_YPOS
};

// For each point, its adjacent edges
/**  * 
 */
protected final static int pointEdges[][] = {
	{ 0, 2, 8 },    // (-1.0, -1.0, -1.0)
	{ 0, 3, 9 },    // (-1.0, -1.0,  1.0)
	{ 4, 7, 9 },    // (-1.0,  1.0,  1.0)
	{ 4, 6, 8 },    // (-1.0,  1.0, -1.0)
	{ 5, 6, 10 },   // ( 1.0,  1.0, -1.0)
	{ 5, 7, 11 },   // ( 1.0,  1.0,  1.0)
	{ 1, 3, 11 },   // ( 1.0, -1.0,  1.0)
	{ 1, 2, 10 }    // ( 1.0, -1.0, -1.0)
};

/**  * 
 */
protected boolean[] faceVisible  = null;
/**  * 
 */
protected boolean[] annotateEdge = null;
/**  * 
 */
protected boolean[] drawEdge     = null;

// The points of the cube
/**  * 
 */
protected Point4d[] cubePoints = null;

/**  * 
 */
protected JCChart3dArea area = null;

/** 
 * Null constructor
 * 
 */
public Cube()
{
    faceVisible  = new boolean[NUM_FACES];
    annotateEdge = new boolean[NUM_EDGES];
    drawEdge     = new boolean[NUM_EDGES];

    // Initialize cubePoints array
    cubePoints = new Point4d[NUM_POINTS];
    for (int i = 0; i < NUM_POINTS; i++) {
        cubePoints[i] = new Point4d(points[i][0], points[i][1], points[i][2], 1.0);
    }
}

/**  * 
 * @param area 
 */
public Cube(JCChart3dArea area)
{
    this();
    this.area = area;
}

/** 
 * Gets the parent <code>chart3d</code> area.
 * 
 * @return The parent <code>JCChart3dArea</code> object
 */
public JCChart3dArea getChart3dArea()
{
	return(area);
}

/** 
 * Sets the parent <code>chart3d</code> area
 * 
 * @param area 
 */
public void setChart3dArea(JCChart3dArea area)
{
	this.area = area;
}

/** 
 * Gets the precomputed cube points in projected space.
 * 
 * @return The cube points in projected space
 */
protected Point4d[] getCubePoints()
{
    return(cubePoints);
}

/** 
 * Gets the array which determines whether faces are visible or not.
 * 
 * @return The face visible array
 */
protected boolean[] getFaceVisible()
{
	return(faceVisible);
}

/** 
 * Gets the array which determines whether edges are annotated or not.
 * 
 * @return The annotate edge array
 */
public boolean[] getAnnotateEdge()
{
	return(annotateEdge);
}

/** 
 * Gets the array which determines whether edges are drawn or not.
 * 
 * @return The draw edge array
 */
protected boolean[] getDrawEdge()
{
	return(drawEdge);
}

/** 
 * Determines if the given face is backfacing or not.  Projects the points of the
 * face, in clockwise order, onto the screen.
 * 
 * @param face 
 */
protected boolean faceBackfacing(int face)
{
	Point4d		    pt        = new Point4d();
	int[]           xPoints   = new int[NUM_POINTS_IN_FACE];
	int[]           yPoints   = new int[NUM_POINTS_IN_FACE];
    Transform       transform = area.getTransform();

	/* ctm = [sm][pm][rm][scales] */
    Matrix4d ctm = new Matrix4d(transform.getTextView());
    JCPlotCube plotcube = area.getPlotCube();
    Transform.scaleMatrix(ctm, plotcube.getXNormalizedScale(),
                          plotcube.getYNormalizedScale(),
                          plotcube.getZNormalizedScale());

	for (int i = 0; i < NUM_POINTS_IN_FACE; i++) {
        int n = faces[face][i];
        pt.set(points[n][0], points[n][1], points[n][2], 1.0);
        Transform.transformPoint(ctm, pt);
		int j = NUM_POINTS_IN_FACE - i - 1;
		xPoints[j] = (int) Math.round(pt.x);
		yPoints[j] = (int) Math.round(pt.y);
	}
	return(JCChart3dUtil.backfacing(xPoints, yPoints));
}

/**  * 
 */
protected int findViewPosition()
{
    Point4d     eyePosition = area.getTransform().getEyePosition();
    JCPlotCube  plotcube    = area.getPlotCube();

	// Account for scaling
	double vp_x = eyePosition.x / plotcube.getXNormalizedScale();
	double vp_y = eyePosition.y / plotcube.getYNormalizedScale();
	double vp_z = eyePosition.z / plotcube.getZNormalizedScale();

	int vp_code = 0;
	if( vp_x >=  1.0 ) {
        vp_code |= VP_XMAX;
    }
	if( vp_x <= -1.0 ) {
        vp_code |= VP_XMIN;
    }
	if( vp_y >=  1.0 ) {
        vp_code |= VP_YMAX;
    }
	if( vp_y <= -1.0 ) {
        vp_code |= VP_YMIN;
    }
	if( vp_z >=  1.0 ) {
        vp_code |= VP_ZMAX;
    }
	if( vp_z <= -1.0 ) {
        vp_code |= VP_ZMIN;
    }

	return(vp_code);
}

// Determine if "face" contains the "edge = (p1,p2)"
/**  * 
 * @param face 
 * @param edge 
 */
protected boolean edgeInFace(int face, int edge)
{
	boolean p1_code = false;
    boolean p2_code = false;

	for (int i = 0; i < NUM_POINTS_IN_FACE; i ++) {
		p1_code = p1_code || (faces[face][i] == edges[edge][0]);
		p2_code = p2_code || (faces[face][i] == edges[edge][1]);
	}
	return( (p1_code && p2_code) );
}

/**  * 
 * @param vp_code 
 */
protected void calcFace(int vp_code)
{
	int   	i;

	for (i = YNEG_ZNEG; i <= XPOS_YPOS; i++ ) {
		annotateEdge[i] = false;
		drawEdge[i]     = true;
	}

	for (i = FACE_LEFT; i <= FACE_NEAR; i++ ) {
		faceVisible[i] = true;
	}

	switch(vp_code) {
	case VP_XMAX :
		annotateEdge[XPOS_ZNEG]  = true;
		annotateEdge[XPOS_ZPOS]  = true;
		annotateEdge[XPOS_YNEG]  = true;
		annotateEdge[XPOS_YPOS]  = true;
		faceVisible[FACE_RIGHT] = false;
		break;
	case VP_XMIN :
		annotateEdge[XNEG_ZNEG] = true;
		annotateEdge[XNEG_ZPOS] = true;
		annotateEdge[XNEG_YNEG] = true;
		annotateEdge[XNEG_YPOS] = true;
		faceVisible[FACE_LEFT] = false;
		break;
	case VP_YMAX :
		annotateEdge[YPOS_ZNEG] = true;
		annotateEdge[YPOS_ZPOS] = true;
		annotateEdge[XNEG_YPOS] = true;
		annotateEdge[XPOS_YPOS] = true;
		faceVisible[FACE_TOP]  = false;
		break;
	case VP_YMIN :
		annotateEdge[YNEG_ZNEG]   = true;
		annotateEdge[YNEG_ZPOS]   = true;
		annotateEdge[XNEG_YNEG]   = true;
		annotateEdge[XPOS_YNEG]   = true;
		faceVisible[FACE_BOTTOM] = false;
		break;
	case VP_ZMAX :
		annotateEdge[YNEG_ZPOS] = true;
		annotateEdge[YPOS_ZPOS] = true;
		annotateEdge[XNEG_ZPOS] = true;
		annotateEdge[XPOS_ZPOS] = true;
		faceVisible[FACE_NEAR] = false;
		break;
	case VP_ZMIN :
		annotateEdge[YNEG_ZNEG] = true;
		annotateEdge[YPOS_ZNEG] = true;
		annotateEdge[XNEG_ZNEG] = true;
		annotateEdge[XPOS_ZNEG] = true;
		faceVisible[FACE_FAR]  = false;
		break;
	}
}

/** 
 * Constructs the id of the leading edge from the <code>vp_code
 * </code>
 * We know that exactly one of the <i>x</i>,<i>y</i>,<i>z</i> groups must be zero in the
 * <code>vp_code</code> for this routine to be called.  The <code>vp_code</code> is organized as:
 * 00 00 01 XMAX (x-pos)
 * 00 00 10 XMIN (x-neg)
 * 00 01 00 YMAX (y-pos)
 * 00 10 00 YMIN (y-neg)
 * 01 00 00 ZMAX (z-pos)
 * 10 00 00 ZMIN (z-neg)
 * 
 * @param vp_code 
 */
protected void calcEdge(int vp_code)
{
	int   	i, a_code;

	int	edge = -1;
	int d_code = 0;
	if ((vp_code & VP_XMASK) == 0) {
		d_code |= (vp_code & VP_YMAX) >> 1;
		d_code |= (vp_code & VP_ZMAX) >> 4;
		a_code  = (~d_code) & 3;

		/* X-axis : 3 drawn, 2 annotated */
		for (i = YNEG_ZNEG; i <= YPOS_ZPOS; i++) {
			annotateEdge[i] = (((i&3) != d_code) && ((i&3) != a_code));
			drawEdge[i]     = ((i&3) != d_code);
			if ((i&3) == d_code) {
				edge = i;
			}
		}

		/* Y-axis : 4 drawn, 2 annotated */
		for (i = XNEG_ZNEG; i <= XPOS_ZPOS; i++) {
			annotateEdge[i] = ((i&1) == (d_code&1));
			drawEdge[i]     = true;
		}

		/* Z-axis : 4 drawn, 2 annotated */
		for (i = XNEG_YNEG; i <= XPOS_YPOS; i++) {
			annotateEdge[i] = ((i&1) == (d_code>>1));
			drawEdge[i]     = true;
		}
	}
	else if ((vp_code & VP_YMASK) == 0) {
		d_code |= (vp_code & VP_XMAX) << 1;
		d_code |= (vp_code & VP_ZMAX) >> 4;
		a_code  = (~d_code) & 3;

		/* Y-axis : 3 drawn, 2 annotated */
		for( i = XNEG_ZNEG; i <= XPOS_ZPOS; i++ ) {
			annotateEdge[i] = (((i&3) != d_code) && ((i&3) != a_code));
			drawEdge[i]     = ((i&3) != d_code);
			if ((i&3) == d_code) {
				edge = i;
			}
		}

		/* X-axis : 4 drawn, 2 annotated */
		for( i = YNEG_ZNEG; i <= YPOS_ZPOS; i++ ) {
			annotateEdge[i] = ((i&1) == (d_code&1));
			drawEdge[i]     = true;
		}

		/* Z-axis : 4 drawn, 2 annotated */
		for( i = XNEG_YNEG; i <= XPOS_YPOS; i++ ) {
			annotateEdge[i] = ((i&2) == (d_code&2));
			drawEdge[i]     = true;
		}
	}
	else if ((vp_code & VP_ZMASK) == 0) {
		d_code |= (vp_code & VP_XMAX) << 1;
		d_code |= (vp_code & VP_YMAX) >> 2;
		a_code  = (~d_code) & 3;

		/* Z-axis : 3 drawn, 2 annotated */
		for (i = XNEG_YNEG; i <= XPOS_YPOS; i++) {
			annotateEdge[i] = (((i&3) != d_code) && ((i&3) != a_code));
			drawEdge[i]     = ((i&3) != d_code);
			if ((i&3) == d_code) {
				edge = i;
			}
		}

		/* X-axis : 4 drawn, 2 annotated */
		for (i = YNEG_ZNEG; i <= YPOS_ZPOS; i++) {
			annotateEdge[i] = ((i&2) == ((d_code&1)<<1));
			drawEdge[i] = true;
		}

		/* Y-axis : 4 drawn, 2 annotated */
		for (i = XNEG_ZNEG; i <= XPOS_ZPOS; i++) {
			annotateEdge[i] = ((i&2) == (d_code&2));
			drawEdge[i] = true;
		}
	}

	/*
	 *  "edge" represents the edge which is not drawn.  Determine
	 *  which faces include this edge and hence are not visible.
	*/

	//assert(edge != -1);
	for (i = FACE_LEFT; i <= FACE_NEAR; i++ ) {
		faceVisible[i] = edgeInFace(i, edge) ? false : true;
	}
}

/**  * 
 * @param min_pt 
 * @param max_pt 
 */
protected void calcCorner(int min_pt, int max_pt)
{
	// Determine which edges are to be drawn and annotated
	for (int i = YNEG_ZNEG; i <= XPOS_YPOS; i++) {
		// Do not annotate the edges associated with either the farthest point
        // or the nearest point
		annotateEdge[i] = (edges[i][0] != min_pt && edges[i][1] != min_pt &&
                           edges[i][0] != max_pt && edges[i][1] != max_pt);

		/* Do not even draw the edges associated with the nearest point */
		drawEdge[i] = (edges[i][0] != max_pt && edges[i][1] != max_pt);
	}

    // Don't draw any faces that contain the nearest point
	for (int i = FACE_LEFT; i <= FACE_NEAR; i++) {
		faceVisible[i] = ((faces[i][0] != max_pt) &&
			              (faces[i][1] != max_pt) &&
			              (faces[i][2] != max_pt) &&
			              (faces[i][3] != max_pt)) ? true : false;
	}
}

/** 
 * This routine determines the edges which will be used for annotations.
 * There are various possible strategies for choosing the annotation edges.
 * The best choice will depend upon the viewing angle, the data, and the
 * viewer's preferences.
 * Only one stategy is implemented here, but more may be added to this
 * routine at a later time.
 * 
 * The strategy used is to discard the 3 edges associated with the furthest
 * corner of the viewing cube, and discard the 3 edges touching the nearest
 * corner of the viewing cube.  Of the remaining 6 edges, all may be
 * annotated, or the furthest edge from the viewer for each axis may be
 * discarded to leave 3 edges.
 * 
 */
protected void calc3DAnnoEdges()
{
	// Precompute the cube corner positions in projected space.
    // The scale/offset necessary to get the actual screen coordinates is not
    // known at this time.  The effect of perspective on the cube lines
    // (annotation axes) is accounted for.  This fact is important in setting
    // up the orientation transform for the annotations.

    Transform transform = area.getTransform();

	/* ctm = [-y][pm][rm][scales] */
    Matrix4d ctm = new Matrix4d();
    ctm.setIdentity();
    ctm.m11 = -1.0;
    ctm.mul(transform.getPerspective());
    ctm.mul(transform.getRotation());
    JCPlotCube plotcube = area.getPlotCube();
    Transform.scaleMatrix(ctm, plotcube.getXNormalizedScale(),
                          plotcube.getYNormalizedScale(),
                          plotcube.getZNormalizedScale());

	/* Find the nearest & farthest cube points while projecting the points */
	double zmin = Double.MAX_VALUE;
	double zmax = Double.MIN_VALUE;

	int max_pt = -1;    // Nearest cube point
    int min_pt = -1;    // Furthest cube point
	for (int i = 0; i < NUM_POINTS; i++) {
        cubePoints[i].set(points[i][0], points[i][1], points[i][2], 1.0);
        Transform.transformPoint(ctm, cubePoints[i]);
		if (zmin > cubePoints[i].z) {
            zmin = cubePoints[i].z;
            min_pt = i;
        }
		if (zmax < cubePoints[i].z) {
            zmax = cubePoints[i].z;
            max_pt = i;
        }
	}
	//assert(min_pt != -1 && max_pt != -1);

	// Analyze the viewing case code to determine how to proceed first.
    // Choose between inside, face, edge, & corner viewing cases
	int vp_code = findViewPosition();   // view position code
	int v_code = 0;                     // view case code
	if ((vp_code & VP_XMASK) != 0) {
        v_code += 1;
    }
	if ((vp_code & VP_YMASK) != 0) {
        v_code += 2;
    }
	if ((vp_code & VP_ZMASK) != 0) {
        v_code += 4;
    }

	switch(v_code) {
	case 0 :			// cannot view from inside cube
		break;
	case 1 :			// x-axis face
		calcFace(vp_code);
		break;
	case 2 :			// y-axis face
		calcFace(vp_code);
		break;
	case 3 :			// x-axis edge
		calcEdge(vp_code);
		break;
	case 4 :			// z-axis face
		calcFace(vp_code);
		break;
	case 5 :			// y-axis edge
		calcEdge(vp_code);
		break;
	case 6 :			// z-axis edge
		calcEdge(vp_code);
		break;
	case 7 :			// corner
		calcCorner(min_pt, max_pt);
		break;
	}

	if (!area.getAxis(JCAxis.AXIS_X).isShowing()) {
		for (int i = YNEG_ZNEG; i <= YPOS_ZPOS; i++) {
			annotateEdge[i] = false;
			drawEdge[i]     = false;
		}
	}
	if (!area.getAxis(JCAxis.AXIS_Y).isShowing()) {
		for (int i = XNEG_ZNEG; i <= XPOS_ZPOS; i++) {
			annotateEdge[i] = false;
			drawEdge[i]     = false;
		}
	}
	if (!area.getAxis(JCAxis.AXIS_Z).isShowing()) {
		for (int i = XNEG_YNEG; i <= XPOS_YPOS; i++) {
			annotateEdge[i] = false;
			drawEdge[i]     = false;
		}
	}
}

/**  * 
 */
protected void calc2DAnnoEdges()
{
    // Only face 5 is visible
	for (int i = FACE_LEFT; i <= FACE_NEAR; i++) {
		faceVisible[i] = (i == 5) ? true : false;
	}
}

}
