/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DerivedDouble.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:56 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

/**
 * Object that allows the user to set a value and set whether
 * the value is used independent of each other.  Used inside
 * JClass Chart 3D to represent values that may be set by the user, but
 * may also be generated by JClass Chart 3D.
 */
class DerivedDouble implements java.io.Serializable {

/** Value to be used. */
double value;

/** If true, the value is ignored and a default is generated. */
boolean isDefault = true;

/**
 * Makes a <code>DerivedDouble</code> object using the provided value and
 * IsDefault value.
 *
 * @param v The starting value
 * @param u isDefault value.
 */
DerivedDouble(double v, boolean u)
{
	value     = v;
	isDefault = u;
}

/**
 * Sets value only if invalid or <code>isDefault</code> is true.
 *
 * @param v The value attempted to be set
 */
protected void setValue(double v)
{
	if (isDefault || value == Double.MAX_VALUE) {
		value = v;
	}
}

/**
 * Sets a value with restriction that it must be greater or less than a given
 * value.  Used to make sure that a user specified axis min or max doesn't fall
 * within data bounds.
 *
 * @param v The value attempted to be set
 * @param doGreater If true, use greater than for the test.  Otherwise use
 *   less than.
 */
protected boolean setValidValue(double v, boolean doGreater)
{
	if (isDefault) {
		value = v;
		return(true);
	}

	boolean valueBad;
	if (doGreater) {
		valueBad = (value > v);
	}
	else {
		valueBad = (value < v);
	}
	if (valueBad || value == Double.MAX_VALUE) {
		value     = v;
		isDefault = true;
		return(false);
	}
	return(true);
}

}
