/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Draw.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:56 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import java.awt.Graphics;
import java.awt.Color;



/**
 * Abstract class that draws the dataset associated with its dataView parent.
 * Child classes must implement the draw and draw2dPoints methods for the
 * given type of data.
 *
 */
public abstract class Draw implements java.io.Serializable {

/**  */
protected ScreenPoint      originPoint = null;
/**  */
protected Chart3dDataView  dataView    = null;
/**  */
protected Mapping          mapping     = null;

/**
 * Basic constructor. Parent and mapping are null!
 */
Draw()
{
}

/**
 * Internal constructor specifying parent data view and mapping object.
 */
Draw(Chart3dDataView dataView)
{
	this.dataView = dataView;
}

/**
 * Gets the value of the <code>dataView</code> property.
 * The <code>dataView</code> property points to the containing
 * <code>Chart3dDataView</code> for this data object
 *
 */
public Chart3dDataView getDataView()
{
	return(dataView);
}

/**
 * Gets the current mapping object.  It must match the type (grid or point)
 * of this draw object.
 *
 */
public Mapping getMapping()
{
	return(mapping);
}

/**
 * Sets the mapping object for this draw object.  It must match the type (grid
 * or point) of this draw object.
 *
 * @param mapping
 */
public void setMapping(Mapping mapping)
{
	this.mapping = mapping;
}

/**
 * Draws a drop line based from the calculated <code>ScreenPoint</code> to the origin.
 *
 * @param gc
 * @param pt
 * @param lineStyle
 * @param x
 * @param y
 */
protected void renderDropLine(Graphics gc, ScreenPoint pt,
			                  JCLineStyle lineStyle, double x, double y)
{
	JCChart3d       chart3d = dataView.getChart3d();
	JCChart3dArea   area    = chart3d.getChart3dArea();
	JCAxis          zAxis   = area.getAxis(JCAxis.AXIS_Z);
	JCScatter       scatter = area.getScatter();

	if (!chart3d.isProjection()	    &&
		scatter.hasDropLines()	    &&
		lineStyle.getWidth() > 0    &&
		lineStyle.getPattern() != JCLineStyle.NONE)
	{
		if (originPoint == null) {
			originPoint = new ScreenPoint();
		}
		else {
			originPoint.reset();
		}

		lineStyle.updateGraphics(gc);
		JCChart3dUtil.getScreenPoint(chart3d, x, y, zAxis.getOrigin(),
		                             zAxis.getOrigin(), originPoint);
		gc.drawLine(pt.getXPixel(), pt.getYPixel(),
		            originPoint.getXPixel(), originPoint.getYPixel());
		lineStyle.resetGraphics(gc);
	}
}

/**
 * Renders the point with given screen point.  If drawing drop lines, decide
 * whether to draw drop lines first or last.
 *
 * @param gc
 * @param symbolStyle
 * @param lineStyle
 * @param pt
 * @param x
 * @param y
 */
protected void renderPoint(Graphics gc, JCSymbolStyle symbolStyle,
			               JCLineStyle lineStyle, ScreenPoint pt,
						   double x, double y)
{
	JCChart3dArea   area = dataView.getChart3d().getChart3dArea();

	if (pt == null || lineStyle == null || symbolStyle == null) {
		return;
	}

	// Determine whether to draw the drop lines first or last.
	// We draw drop lines last if we are below the zmin plane
	// and we are off one or zero of the four sides but not
	// off a corner.  See grid.h for values.
	boolean dropLineLast = false;
	int     order        = area.getTransform().getProcessingOrder();
	if ((order & Transform.EDGE_ZMIN) != 0) {
		switch(order & 15) {
	    case Transform.FACE_ON:
	    case Transform.EDGE_XMAX:
	    case Transform.EDGE_XMIN:
	    case Transform.EDGE_YMAX:
	    case Transform.EDGE_YMIN:
			dropLineLast = true;
			break;
		default:
			dropLineLast = false;
			break;
		}
	}

	if (!dropLineLast) {
		renderDropLine(gc, pt, lineStyle, x, y);
	}

	if (symbolStyle.getSize() > 0 &&
	    symbolStyle.getShape() != JCSymbolStyle.NONE)
	{
		symbolStyle.draw(gc, pt.getXPixel(), pt.getYPixel());
	}
	pt.setDrawn(true);

	if (dropLineLast) {
		renderDropLine(gc, pt, lineStyle, x, y);
	}
}

/**
 * Draws the graph based on the data.
 *
 * @param gc
 */
protected abstract void draw(Graphics gc);

/**
 * Draws 2d points based on the data.
 *
 * @param gc
 */
protected abstract void draw2dPoints(Graphics gc);

}


