/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DrawFacetFactory.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:57 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import java.awt.Graphics;

/**
 *
 */
public class DrawFacetFactory implements java.io.Serializable {

protected int        drawCode = 0;
protected boolean    isTransparent = false;
protected DrawFacet  drawFacet = null;

public DrawFacetFactory()
{
}

/**
 *
 */
public DrawFacet createDrawFacet(DrawGrid drawGrid, Graphics gc)
{
	if (drawGrid == null || gc == null) {
		drawCode  = 0;
		drawFacet = null;
		return(drawFacet);
	}

	Chart3dDataView dataView         = drawGrid.getGridData().getDataView();
	boolean         newIsTransparent = dataView.getElevation().isTransparent();
	int             newDrawCode      = dataView.getDrawCode();
	if (newDrawCode == 0) {
		drawCode  = 0;
		drawFacet = null;
		return(drawFacet);
	}
	else if (newDrawCode == drawCode           &&
		     newIsTransparent == isTransparent &&
			 drawFacet != null)
	{
		// Use old object
		drawFacet.setDrawGrid(drawGrid);
		drawFacet.setGC(gc);
		return(drawFacet);
	}

	drawCode = newDrawCode;
	isTransparent = newIsTransparent;

	switch(drawCode) {
	case Chart3dDataView.MESHED:
		if (isTransparent) {
			drawFacet = new DrawFacetMH(drawGrid, gc);
		}
		else {
		    drawFacet = new DrawFacetM(drawGrid, gc);
		}
		break;
	case Chart3dDataView.SHADED:
		drawFacet = new DrawFacetS(drawGrid, gc);
		break;
	case Chart3dDataView.SHADED + Chart3dDataView.MESHED:
		drawFacet = new DrawFacetMS(drawGrid, gc);
		break;
	case Chart3dDataView.ZONED + Chart3dDataView.MESHED:
		if (isTransparent) {
		    drawFacet = new DrawFacetMZH(drawGrid, gc);
		}
		else {
			drawFacet = new DrawFacetMZ(drawGrid, gc);
		}
		break;
	case Chart3dDataView.ZONED + Chart3dDataView.SHADED:
		drawFacet = new DrawFacetSZ(drawGrid, gc);
		break;
	case Chart3dDataView.ZONED + Chart3dDataView.SHADED + Chart3dDataView.MESHED:
		drawFacet = new DrawFacetMSZ(drawGrid, gc);
		break;
	case Chart3dDataView.CONTOURED + Chart3dDataView.MESHED:
		if (isTransparent) {
		    drawFacet = new DrawFacetMCH(drawGrid, gc);
		}
		else {
			drawFacet = new DrawFacetMC(drawGrid, gc);
		}
		break;
	case Chart3dDataView.CONTOURED + Chart3dDataView.SHADED:
		drawFacet = new DrawFacetSC(drawGrid, gc);
		break;
	case Chart3dDataView.CONTOURED + Chart3dDataView.SHADED + Chart3dDataView.MESHED:
		drawFacet = new DrawFacetMSC(drawGrid, gc);
		break;
	case Chart3dDataView.CONTOURED + Chart3dDataView.ZONED + Chart3dDataView.MESHED:
		if (isTransparent) {
			drawFacet = new DrawFacetMZCH(drawGrid, gc);
		}
		else {
			drawFacet = new DrawFacetMZC(drawGrid, gc);
		}
		break;
	case Chart3dDataView.CONTOURED + Chart3dDataView.ZONED + Chart3dDataView.SHADED:
		drawFacet = new DrawFacetSZC(drawGrid, gc);
		break;
	case Chart3dDataView.CONTOURED + Chart3dDataView.ZONED +
         Chart3dDataView.SHADED + Chart3dDataView.MESHED:
		drawFacet = new DrawFacetMSZC(drawGrid, gc);
		break;
	}
	return(drawFacet);
}

}


