/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DrawPoint.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:57 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import java.awt.Graphics;
import java.util.ArrayList;
import javax.vecmath.Point3d;
import javax.vecmath.Point4d;
import javax.vecmath.Matrix4d;



/**  * 
 */
public class DrawPoint extends Draw implements java.io.Serializable {

/** 
 * Basic constructor.
 * 
 */
public DrawPoint()
{
	super();
}

/** 
 * Internal constructor specifying parent data view.
 * 
 * @param dataView 
 */
public DrawPoint(Chart3dDataView dataView)
{
	super(dataView);
}

/** 
 * Data must be point data.  This is a convenience method to get the data from
 * the <code>Chart3dDataView</code>.
 * 
 */
protected Chart3dPointData getPointData()
{
    if (dataView != null) {
        return((Chart3dPointData)dataView.getElevationData());
    }
    return(null);
}

/** 
 * Actually draws the graph based on the point data.
 * 
 * @param gc 
 */
protected void draw(Graphics gc)
{
	Chart3dPointData    data         = getPointData();
	ScreenPoint[][]     screenPoints = data.getScreenPoints();

	if (!data.isDataOK()) {
		return;
	}

	data.sortPoints();
	ArrayList sortPoints = data.getSortPoints();
	if (sortPoints ==  null) {
		
		return;
	}

	/*
	 *	Make our own copy of the text list -- note that this also
	 *	sorts them by distance to eye.

	xrttext   = NULL;
	text_list = NULL;
	if (xrt->text_list) {
		for (i = 0; xrt->text_list[i]; i++) {
			text_list = xrt3d_add_to_text_list(text_list, xrt->text_list[i],
											   TRUE);
		}
		xrttext = (i > 0) ? XrtTextImpl(text_list[0]) : NULL;
	}

	assert(data->nseries <= xrt->data_style_list.count);
	*/



	//	Render points and text objects in sorted order (farthest away
	//	from the eye first).

	//int text = 0;
	for (int i = 0; i < sortPoints.size(); i++) {
		SortPoint sortPoint = (SortPoint)sortPoints.get(i);

//		while (xrttext &&
//			   xrttext->distance_to_eye > pSortPoint->distance_to_eye)
//		{
//			xrt3d_draw_3d_text_object(xrt, xrttext, TRUE, TRUE);
//			text++;
//			xrttext = text_list[text] ? XrtTextImpl(text_list[text]) : NULL;
//		}

		int         series = sortPoint.getSeries();
		int         point  = sortPoint.getPoint();

		Chart3dPointSeries pointSeries = data.getSeries(series);
		JCChart3dStyle     chartStyle  = pointSeries.getChartStyle();
		Point3d            point3d     = pointSeries.getPoint(point);
		ScreenPoint        pt          = screenPoints[series][point];
		if (!pt.isDrawn() && !pt.isHoleValue() && !pt.isHidden()) {
			renderPoint(gc, chartStyle.getSymbolStyle(),
						chartStyle.getLineStyle(), pt, point3d.x, point3d.y);
		}
	}

	// Render any remaining text objects
	//while (xrttext) {
	//	xrt3d_draw_3d_text_object(xrt, xrttext, TRUE, TRUE);
	//	text++;
	//	xrttext = text_list[text] ? XrtTextImpl(text_list[text]) : NULL;
	//}

	//xrt3d_destroy_text_list(text_list, FALSE);
}

/** 
 * For point data, draws 2d points.  Just blurt them out (no sorting required).
 * 
 * @param gc 
 */
protected void draw2dPoints(Graphics gc)
{
	Chart3dPointData    data         = getPointData();
	ScreenPoint[][]     screenPoints = data.getScreenPoints();

	if (!data.isDataOK()) {
		return;
	}

	for (int series = 0; series < data.getNumSeries(); series++) {
		Chart3dPointSeries pointSeries = data.getSeries(series);
		JCChart3dStyle     chartStyle  = pointSeries.getChartStyle();
		for (int point = 0; point < pointSeries.getNumPoints(); point++) {
			Point3d     point3d = pointSeries.getPoint(point);
		    ScreenPoint pt      = screenPoints[series][point];
		    if (!pt.isDrawn() && !pt.isHoleValue() && !pt.isHidden()) {
			    renderPoint(gc, chartStyle.getSymbolStyle(),
							chartStyle.getLineStyle(), pt, point3d.x, point3d.y);
			}
		}
	}
}

}


