/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ErrorDialog.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:57 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import com.klg.jclass.chart3d.resources.LocaleBundle;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/** 
 * Error dialog object used inside JClass Chart 3D to show an
 * internal error to the user.  Better than <code>System.out.println</code>,
 * but not much better - yet.
 */
class ErrorDialog extends JDialog implements ActionListener {

/** Global <code>ErrorDialog</code> for JClass Chart 3D. */
protected static ErrorDialog dialog;

/** Label that contains the error text. */
protected JTextArea text;

/** OK button that dismisses the dialog. */
protected JButton ok;

/** 
 * The default and only constructor.  Creates the visual objects inside the
 * dialog - label for the text, and a button to dismiss the dialog.
 */
public ErrorDialog()
{
	super();
	getContentPane().setLayout(new BorderLayout());
	text = new JTextArea();
	getContentPane().add("Center", text);
	JPanel footer = new JPanel();
	getContentPane().add("South", footer);
	footer.setLayout(new FlowLayout());
    ok = new JButton(LocaleBundle.string(LocaleBundle.OKAY_LABEL));
	ok.addActionListener(this);
    footer.add(ok);
}

/** 
 * Static method used to show an error using the global
 * error dialog.  If the global dialog does not exist, it
 * is created.
 * 
 * @param s Error text to be displayed inside the dialog.
 */
public static void raise(String s)
{
	try {
		if (dialog == null) {
			dialog = new ErrorDialog();
		}
        dialog.text.setText(s);
	    dialog.text.invalidate();
	    dialog.invalidate();
	    dialog.validate();
	    dialog.pack();
	    dialog.setVisible(true);
	    dialog.toFront();
	}
	catch (Exception e) {
		System.out.println(s);
	}
}

/** 
 * Static method used to show an error using the global
 * error dialog.  If the global dialog does not exist, it
 * is created.
 * 
 * @param comp The instance the error occurred in (Colors imported)
 * @param s Error text to be displayed inside the dialog.
 */
public static void raise(Component comp, String s)
{
	// don't show error dialog if parent is not showing
	if (comp != null && !comp.isShowing()) {
		return;
	}

   	if (dialog == null) {
		dialog = new ErrorDialog();
	}
	if (comp != null) {
		if (comp instanceof JCChart3d) {
			if (!((JCChart3d) comp).isWarningDialog()) {
				return;
			}
		}
	    dialog.setBackground(comp.getBackground());
	    dialog.setForeground(comp.getForeground());
	}
	raise(s);
}

/** 
 * Static method used to show an error using the global
 * error dialog.  If the global dialog does not exist, it
 * is created.
 * 
 * @param area 
 * @param s Error text to be displayed inside the dialog.
 */
public static void raise(JCChart3dArea area, String s)
{
	// Don't show error dialog if parent is not showing
	JCChart3d chart3d = (area != null) ? area.getChart3d() : null;
	if (chart3d != null && !chart3d.isShowing()) {
		return;
	}

    if (dialog == null) {
		dialog = new ErrorDialog();
	}
	if (area != null ) {
		if (chart3d != null) {
			if (!chart3d.isWarningDialog()) {
				return;
			}
		}
	    dialog.setBackground(area.getBackground());
	    dialog.setForeground(area.getForeground());
	}
	raise(s);
}

/** 
 * Static method used to show an error using the global
 * error dialog.  If the global dialog does not exist, it
 * is created.
 * 
 * @param obj The instance the error occurred in (Colors imported)
 * @param s Error text to be displayed inside the dialog.
 */
public static void raise(Object obj, String s)
{
	if (obj instanceof Component) {
		raise((Component)obj, s);
	}
	else {
		raise(s);
	}
}

/** 
 * Action event for the error dialog - makes the dialog go away.
 * 
 * @param evt The event generated by this action
 */
public void actionPerformed(ActionEvent evt)
{
	setVisible(false);
}

}
