/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Extents.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:57 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

/** 
 * Class internal to JClass Chart 3D.  Used to store the extents in the 
 * <i>x</i>, <i>y</i>, and <i>z</i> direction of something.
 */
public class Extents implements java.io.Serializable {

/** The current x minimum extents */
protected double xMin = Double.MAX_VALUE;
/** The current x maximum extents */
protected double xMax = Double.MIN_VALUE;
/** The current y minimum extents */
protected double yMin = Double.MAX_VALUE;
/** The current y maximum extents */
protected double yMax = Double.MIN_VALUE;
/** The current z minimum extents */
protected double zMin = Double.MAX_VALUE;
/** The current z maximum extents */
protected double zMax = Double.MIN_VALUE;

/** 
 * Null constructor.  All values at default.
 */
public Extents() {}

/** 
 * Constructor that intializes the <i>x</i> and <i>y</i> extents.
 * 
 * @param xMin The initial x minimum
 * @param xMax The initial x maximum
 * @param yMin The initial y minimum
 * @param yMax The initial y maximum
 */
public Extents(double xMin, double xMax,
               double yMin, double yMax)
{
	this.xMin = xMin;
	this.xMax = xMax;
	this.yMin = yMin;
	this.yMax = yMax;
}

/** 
 * Constructor that intializes the <i>x</i>, <i>y</i> and <i>z</i> extents
 * 
 * @param xMin The initial x minimum
 * @param xMax The initial x maximum
 * @param yMin The initial y minimum
 * @param yMax The initial y maximum
 * @param zMin The initial z minimum
 * @param zMax The initial z maximum
 */
public Extents(double xMin, double xMax,
			   double yMin, double yMax,
			   double zMin, double zMax)
{
	this.xMin = xMin;
	this.xMax = xMax;
	this.yMin = yMin;
	this.yMax = yMax;
	this.zMin = zMin;
	this.zMax = zMax;
}

/** 
 * Gets the <i>x</i> minimum value for this extents object.
 * 
 * @return The current value of xMin
 */
protected double getXMin()
{
	return(xMin);
}

/** 
 * Sets the <i>x</i> minimum value for this extents object.
 * 
 * @param xMin The new value of xMin
 */
protected void setXMin(double xMin)
{
	this.xMin = xMin;
}

/** 
 * Gets the <i>x</i> maximum value for this extents object.
 * 
 * @return The current value of xMax
 */
protected double getXMax()
{
	return(xMax);
}

/** 
 * Sets the <i>x</i> maximum value for this extents object.
 * 
 * @param xMax The new value of xMax
 */
protected void setXMax(double xMax)
{
	this.xMax = xMax;
}

/** 
 * Gets the <i>y</i> minimum value for this extents object.
 * 
 * @return The current value of yMin
 */
protected double getYMin()
{
	return(yMin);
}

/** 
 * Sets the <i>y</i> minimum value for this extents object.
 * 
 * @param yMin The new value of yMin
 */
protected void setYMin(double yMin)
{
	this.yMin = yMin;
}

/** 
 * Gets the <i>y</i> maximum value for this extents object.
 * 
 * @return The current value of yMax
 */
protected double getYMax()
{
	return(yMax);
}

/** 
 * Sets the <i>y</i> maximum value for this extents object.
 * 
 * @param yMax The new value of yMax
 */
protected void setYMax(double yMax)
{
	this.yMax = yMax;
}

/** 
 * Gets the <i>z</i> minimum value for this extents object.
 * 
 * @return The current value of zMin
 */
protected double getZMin()
{
	return(zMin);
}

/** 
 * Sets the <i>z</i> minimum value for this extents object.
 * 
 * @param zMin The new value of zMin
 */
protected void setZMin(double zMin)
{
	this.zMin = zMin;
}

/** 
 * Gets the <i>z</i> maximum value for this extents object.
 * 
 * @return The current value of zMax
 */
protected double getZMax()
{
	return(zMax);
}

/** 
 * Sets the <i>z</i> maximum value for this extents object.
 * 
 * @param zMax The new value of zMax
 */
protected void setZMax(double zMax)
{
	this.zMax = zMax;
}

/** 
 * Extends the extents in the <i>x</i> direction based on <i>x</i>.
 * 
 * @param x The x value to check
 */
protected void checkXExtents(double x)
{
	if (xMin == Double.MAX_VALUE) {
		xMin = xMax = x;
	}
	else if (x < xMin) {
		xMin = x;
	}
	else if (x > xMax) {
		xMax = x;
	}
}

/** 
 * Extends the extents in the <i>y</i> direction based on <i>x</i>.
 * 
 * @param y The y value to check
 */
protected void checkYExtents(double y)
{
	if (yMin == Double.MAX_VALUE) {
		yMin = yMax = y;
	}
	else if (y < yMin) {
		yMin = y;
	}
	else if (y > yMax) {
		yMax = y;
	}
}

/** 
 * Extends the extents in the <i>z</i> direction based on <i>x</i>.
 * 
 * @param z The z value to check
 */
protected void checkZExtents(double z)
{
	if (zMin == Double.MAX_VALUE) {
		zMin = zMax = z;
	}
	else if (z < zMin) {
		zMin = z;
	}
	else if (z > zMax) {
		zMax = z;
	}
}

/** 
 * Extends the extents in the <i>x</i> and/or <i>y</i> direction based on
 * (<i>x</i>, <i>y</i>).
 * 
 * @param x The x value to check
 * @param y The y value to check
 */
protected void checkXYExtents(double x, double y)
{
	checkXExtents(x);
	checkYExtents(y);
}

/** 
 * Extends the extents in the <i>x</i>, <i>y</i> and/or <i>z</i> direction based
 * on (<i>x</i>, <i>y</i>, <i>z</i>).
 * 
 * @param x The x value to check
 * @param y The y value to check
 * @param z The z value to check
 */
protected void checkXYZExtents(double x, double y, double z)
{
	checkXExtents(x);
	checkYExtents(y);
	checkZExtents(z);
}

public String toString()
{
    String s = "";
    if (xMin == Double.MAX_VALUE) {
        s += "x[default] ";
    }
    else {
        s += "x[" + xMin + "," + xMax + "] ";
    }
    if (yMin == Double.MAX_VALUE) {
        s += "y[default] ";
    }
    else {
        s += "y[" + yMin + "," + yMax + "] ";
    }
    if (zMin == Double.MAX_VALUE) {
        s += "z[default]";
    }
    else {
        s += "z[" + zMin + "," + zMax + "]";
    }
    return(s);
}

}
