/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: FacetAbstract.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:57 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import javax.vecmath.Point4d;

public abstract class FacetAbstract implements Facet, java.io.Serializable {

protected int[] xPoints = null;
protected int[] yPoints = null;

/** Parent draw object. */
protected DrawGrid drawGrid = null;

protected Chart3dGridData data = null;

/** No args constructor */
FacetAbstract()
{
}

public FacetAbstract(DrawGrid drawGrid)
{
	this.drawGrid = drawGrid;
    if (drawGrid != null) {
        data = drawGrid.getGridData();
    }
}

protected DrawGrid getDrawGrid()
{
	return(drawGrid);
}

/**
 * @param drawGrid
 */
protected void setDrawGrid(DrawGrid drawGrid)
{
	this.drawGrid = drawGrid;
    data = null;
    if (drawGrid != null) {
        data = drawGrid.getGridData();
    }
}

/**
 * Fills the two integer point arrays from a screen point array (so they're
 * ready for input to Graphics routines like <code>fillPolygon</code>).  Use
 * the same arrays each time (extending them if needed).
 *
 * @param spts
 * @param n
 */
protected void fillPointArrays(ScreenPoint[] spts, int n)
{
	if (xPoints == null || xPoints.length < n) {
	   xPoints = new int[n];
	   yPoints = new int[n];
	}
	for (int i = 0; i < n; i++) {
		xPoints[i] = spts[i].getXPixel();
		yPoints[i] = spts[i].getYPixel();
	}
}

/**
 * Gets the transformed screen point from data point
 * (<i>x</i>, <i>y</i>, <i>z</i>)
 *
 * @param x
 * @param y
 * @param z
 */
protected ScreenPoint getScreenPoint(double x, double y, double z)
{
	JCChart3dArea   area = data.getDataView().getChart3d().getChart3dArea();
	Transform       transform = area.getTransform();
	Point4d         pt = new Point4d(x, y, z, 1.0);

	transform.transformPoint(transform.getView(), pt);
    ScreenPoint spt = new ScreenPoint((int) Math.round(pt.x),
					                  (int) Math.round(pt.y));
	return(spt);
}

/**
 * Abstract routine that renders the facet.  Each chart type will do this
 * differently.
 *
 * @param x
 * @param y
 * @param edges
 */
public abstract void processFacet(int x, int y, int edges);

}


