/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: GridLimits.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

/** 
 * Data structure internal to JClass Chart 3D.  Used to hold the data limits
 * of a <code>Chart3dGridData</code> structure based on the X-axis and Y-axis min and
 * max provided by the user
 * 
 */
public class GridLimits implements java.io.Serializable {

/** Starting offset into <i>x</i> grid array.
 * 
 */
protected int xOffset;

/** Starting offset into <i>y</i> grid array.
 * 
 */
protected int yOffset;

/** Ending offset into <i>x</i> grid array.
 * 
 */
protected int xLast;

/** Ending offset into <i>y</i> grid array.
 * 
 */
protected int yLast;

/** Number of <i>x</i> grid points in region defined by [<code>xOffset</code>, <code>xLast</code>].
 * 
 */
protected int numX;

/** Number of <i>y</i> grid points in region defined by [<code>yOffset</code>, <code>yLast</code>].
 * 
 */
protected int numY;

/** Parent data object.
 * 
 */
protected Chart3dGridData data = null;

/**  * 
 */
public GridLimits()
{
	assignGridValues((Chart3dGridData)null);
}

/**  * 
 * @param data 
 * @param assignValues 
 */
public GridLimits(Chart3dGridData data, boolean assignValues)
{
	this.data = data;
	if (assignValues) {
		assignGridValues(data);
	}
	else {
		assignGridValues((Chart3dGridData)null);
	}
}

/** 
 * Returns starting offset into <i>x</i> grid array.
 * 
 * @return The starting offset into x grid array
 */
public int getXOffset()
{
	return(xOffset);
}

/** 
 * Sets starting offset into <i>x</i> grid array.
 * 
 * @param xOffset The new starting offset into x grid array
 */
protected void setXOffset(int xOffset)
{
	this.xOffset = xOffset;
}

/** 
 * Returns starting offset into <i>y</i> grid array.
 * 
 * @return The starting offset into y grid array
 */
public int getYOffset()
{
	return(yOffset);
}

/** 
 * Sets starting offset into <i>y</i> grid array.
 * 
 * @param yOffset The new starting offset into y grid array
 */
protected void setYOffset(int yOffset)
{
	this.yOffset = yOffset;
}

/** 
 * Returns ending offset into <i>x</i> grid array.
 * 
 * @return The ending offset into x grid array
 */
public int getXLast()
{
	return(xLast);
}

/** 
 * Sets ending offset into <i>x</i> grid array.
 * 
 * @param xLast 
 */
protected void setXLast(int xLast)
{
	this.xLast = xLast;
}

/** 
 * Returns ending offset into <i>y</i> grid array.
 * 
 * @return The ending offset into y grid array
 */
public int getYLast()
{
	return(yLast);
}

/** 
 * Sets ending offset into <i>y</i> grid array.
 * 
 * @param yLast 
 */
protected void setYLast(int yLast)
{
	this.yLast = yLast;
}

/** 
 * Returns the number of <i>x</i> grid points in region defined by [<code>xOffset</code>, <code>xLast</code>].
 * 
 * @return The total number of x grid points used
 */
public int getNumX()
{
	return(numX);
}

/** 
 * Sets the number of <i>x</i> grid points in region defined by [<code>xOffset</code>, <code>xLast</code>].
 * 
 * @param numX The new  number of x grid points
 */
protected void setNumX(int numX)
{
	this.numX = numX;
}

/** 
 * Returns the number of <i>y</i> grid points in region defined by [<code>yOffset</code>, <code>yLast</code>].
 * 
 * @return The total number of y grid points used
 */
public int getNumY()
{
	return(numY);
}

/** 
 * Sets the number of <i>y</i> grid points in region defined by [<code>yOffset</code>, <code>yLast</code>].
 * 
 * @param numY The new  number of y grid points
 */
protected void setNumY(int numY)
{
	this.numY = numY;
}

/** 
 * Returns the parent grid data object.
 * 
 * @return The current grid data parent
 */
protected Chart3dGridData getGridData()
{
	return(data);
}

/** 
 * Sets the parent grid data object.
 * 
 * @param data 
 */
protected void setGridData(Chart3dGridData data)
{
	this.data = data;
	assignGridValues(data);
}

/** 
 * Assigns grid values based on given internal data.
 * 
 * @param data 
 */
protected void assignGridValues(Chart3dGridData data)
{
	if (data == null) {
		xOffset = 0;
		yOffset = 0;
		xLast   = 0;
		yLast   = 0;
		numX    = 0;
		numY    = 0;
		return;
	}

	JCChart3dArea chart3dArea = data.getDataView().getChart3d().getChart3dArea();
	double xMin               = chart3dArea.getAxis(JCAxis.AXIS_X).getMin();
	double xMax               = chart3dArea.getAxis(JCAxis.AXIS_X).getMax();
	double yMin               = chart3dArea.getAxis(JCAxis.AXIS_Y).getMin();
	double yMax               = chart3dArea.getAxis(JCAxis.AXIS_Y).getMax();
	double[] xGrid            = data.getXGrid();
	double[] yGrid            = data.getYGrid();

	// Set these to the grid data values for now
	numX = data.getNumX();
	numY = data.getNumY();

	// Compute the min/max limits in x/y
	xOffset = data.getPointIndex(xMin, xGrid, numX);
	yOffset = data.getPointIndex(yMin, yGrid, numY);
	xLast   = data.getPointIndex(xMax, xGrid, numX);
	yLast   = data.getPointIndex(yMax, yGrid, numY);

	// Exact grid matches will be off by one
	if (1.0 + (xGrid[xLast] - xMax) != 1.0) {
		xLast = Math.min(xLast + 1, numX - 1);
	}
	if (1.0 + (yGrid[yLast] - yMax) != 1.0) {
		yLast = Math.min(yLast + 1, numY - 1);
	}

	int chartType = data.getDataView().getChartType();
	if (chartType == JCChart3d.SURFACE ||
	    (chartType == JCChart3d.BAR &&
	     chart3dArea.getBar().getXFormat() != JCBar.FIXED))
	{
		/* Ensure that at least 1 grid cell is shown */
		if (xLast <= xOffset) {
			if (xLast == numX - 1) {
				xOffset = xLast - 1;
			}
			else {
				xLast = xOffset + 1;
			}
		}
	}
	if (chartType == JCChart3d.SURFACE ||
	    (chartType == JCChart3d.BAR &&
	     chart3dArea.getBar().getYFormat() != JCBar.FIXED))
	{
		/* Ensure that at least 1 grid cell is shown */
		if (yLast <= yOffset) {
			if (yLast == numY - 1) {
				yOffset = yLast - 1;
			}
			else {
				yLast = yOffset + 1;
			}
		}
	}

	// Force to be within bounds
	xOffset = Math.max(xOffset, 0);
	yOffset = Math.max(yOffset, 0);
	xLast   = Math.min(xLast, numX - 1);
	yLast   = Math.min(yLast, numY - 1);

	// Derive new number of grids from limits
	numX = xLast - xOffset + 1;
	numY = yLast - yOffset + 1;
}

}
