/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: GridRegion.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import javax.vecmath.Matrix4d;
import javax.vecmath.Point4d;

/**
 * Data structure internal to JCChart3d. Used to define nine regions within
 * a grid.  In each of the X and Y directions there is a left, center, and
 * right region, with the center being of length 1 (the intersection of the
 * X and Y center is the grid facet where the eye is located).  If the eye
 * is outside of grid (in one or both directions), some of the regions may
 * be empty (note that at least one region must be non-empty).
 */
public class GridRegion implements java.io.Serializable {

public final static int REGION_1 = 1;
public final static int REGION_2 = 2;
public final static int REGION_3 = 3;
public final static int REGION_4 = 4;
public final static int REGION_5 = 5;
public final static int REGION_6 = 6;
public final static int REGION_7 = 7;
public final static int REGION_8 = 8;
public final static int REGION_9 = 9;

/** Eye position in x grid array */
protected int xEye;

/** Eye position in y grid array */
protected int yEye;

/** Starting offset into x grid array */
protected int xBegin;

/** Starting offset into y grid array */
protected int yBegin;

/** Ending offset left of eye in x grid array */
protected int xLeft;

/** Ending offset left of eye in y grid array */
protected int yLeft;

/** Starting offset right of eye in x grid array */
protected int xRight;

/** Starting offset right of eye in y grid array */
protected int yRight;

/** Ending offset into x grid array */
protected int xEnd;

/** Ending offset into y grid array */
protected int yEnd;

/** Parent draw object */
protected DrawGrid drawGrid = null;

GridRegion()
{
}

GridRegion(DrawGrid drawGrid)
{
	this.drawGrid = drawGrid;
}

/**
 *
 */
protected DrawGrid getDrawGrid()
{
	return(drawGrid);
}

/**
 *
 */
protected void setDrawGrid(DrawGrid drawGrid)
{
	this.drawGrid = drawGrid;
}

/**
 *
 */
protected int getXEye()
{
	return(xEye);
}

/**
 *
 */
protected void setXEye(int xEye)
{
	this.xEye = xEye;
}

/**
 *
 */
protected int getYEye()
{
	return(yEye);
}

/**
 *
 */
protected void setYEye(int yEye)
{
	this.yEye = yEye;
}

/**
 *
 */
protected int getXBegin()
{
	return(xBegin);
}

/**
 *
 */
protected void setXBegin(int xBegin)
{
	this.xBegin = xBegin;
}

/**
 *
 */
protected int getYBegin()
{
	return(yBegin);
}

/**
 *
 */
protected void setYBegin(int yBegin)
{
	this.yBegin = yBegin;
}

/**
 *
 */
protected int getXLeft()
{
	return(xLeft);
}

/**
 *
 */
protected void setXLeft(int xLeft)
{
	this.xLeft = xLeft;
}

/**
 *
 */
protected int getYLeft()
{
	return(yLeft);
}

/**
 *
 */
protected void setYLeft(int yLeft)
{
	this.yLeft = yLeft;
}

/**
 *
 */
protected int getXRight()
{
	return(xRight);
}

/**
 *
 */
protected void setXRight(int xRight)
{
	this.xRight = xRight;
}

/**
 *
 */
protected int getYRight()
{
	return(yRight);
}

/**
 *
 */
protected void setYRight(int yRight)
{
	this.yRight = yRight;
}

/**
 *
 */
protected int getXEnd()
{
	return(xEnd);
}

/**
 *
 */
protected void setXEnd(int xEnd)
{
	this.xEnd = xEnd;
}

/**
 *
 */
protected int getYEnd()
{
	return(yEnd);
}

/**
 *
 */
protected void setYEnd(int yEnd)
{
	this.yEnd = yEnd;
}

/**
 *
 */
protected void calcRegions()
{
	Chart3dDataView     dataView = drawGrid.getDataView();
	JCChart3dArea       area = dataView.getChart3d().getChart3dArea();
    Chart3dGridData     data = drawGrid.getGridData();
	GridLimits          gridLimits = data.getGridLimits();

	xBegin = gridLimits.getXOffset();
	yBegin = gridLimits.getYOffset();
	xEnd   = gridLimits.getXLast() - 1;
	yEnd   = gridLimits.getYLast() - 1;

	JCBar bar = area.getBar();
	if (dataView.getChartType() == JCChart3d.BAR) {
		if (bar.getXFormat() == JCBar.FIXED) {
			xEnd++;
		}
		if (bar.getYFormat() == JCBar.FIXED) {
			yEnd++;
		}
	}
	else if (dataView.getChartType() == JCChart3d.SCATTER) {
		xEnd++;
		yEnd++;
	}

	// Figure out where eye falls on the grid.
	Transform transform = area.getTransform();
	Point4d  eye        = transform.getEyePosition();
	xEye     = data.getGridIndex(eye.x, true, true);
	yEye     = data.getGridIndex(eye.y, false, true);
	xLeft    = xEye - 1;
	yLeft    = yEye - 1;
	xRight   = xEye + 1;
	yRight   = yEye + 1;

	// Does eye fall outside of data bounds?  If so, we may need to adjust
	// the left/right values -- they may now fall on the grid boundaries.
	Point4d[] plane   = JCChart3dUtil.constructBarPlane(data, xEye, yEye);
	Matrix4d absolute = transform.getAbsolute();
	double xEyeData   = (eye.x - absolute.m03)/absolute.m00;
	double yEyeData   = (eye.y - absolute.m13)/absolute.m11;
	if (xEyeData < plane[1].x) {
		xRight--;
	}
	if (xEyeData > plane[3].x) {
		xLeft++;
	}

	if (yEyeData < plane[1].y) {
		yRight--;
	}
	if (yEyeData > plane[2].y) {
		yLeft++;
	}
}

/**
 *
 */
public int getRegion(int x, int y)
{
	boolean	xIsLeft, xIsMiddle, xIsRight, yIsBottom, yIsMiddle, yIsTop;

	xIsLeft = xIsMiddle = xIsRight = false;
	if (xLeft < xBegin) {
		xIsRight = true;
	}
	else if (xRight > xEnd) {
		xIsLeft = true;
	}
	else {
		xIsLeft   = (x <= xLeft);
		xIsMiddle = (x == xEye);
		xIsRight  = (x >= xRight);
	}
	yIsBottom = yIsMiddle = yIsTop = false;
	if (yLeft < yBegin) {
		yIsTop = true;
	}
	else if (yRight > yEnd) {
		yIsBottom = true;
	}
	else {
		yIsBottom = (y <= yLeft);
		yIsMiddle = (y == yEye);
		yIsTop    = (y >= yRight);
	}

	if (xIsMiddle && yIsMiddle) {
		return(REGION_1);
	}
	else if (xIsMiddle && yIsTop) {
		return(REGION_2);
	}
	else if (xIsMiddle && yIsBottom) {
		return(REGION_3);
	}
	else if (xIsRight && yIsMiddle) {
		return(REGION_4);
	}
	else if (xIsLeft && yIsMiddle) {
		return(REGION_5);
	}
	else if (xIsRight && yIsTop) {
		return(REGION_6);
	}
	else if (xIsRight && yIsBottom) {
		return(REGION_7);
	}
	else if (xIsLeft && yIsTop) {
		return(REGION_8);
	}
	if (xIsLeft && yIsBottom) {
		return(REGION_9);
	}
	return(REGION_9);
}

}
