/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Intersection.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;



/** 
 * Data structure internal to JClass Chart 3D.  This structure represents an
 * intersection of a contour line with a cell.  It stores the side numbers
 * of the sides the line intersects and stores the point information in
 * <code>ContourSegment</code> objects.
 * 
 */
public class Intersection implements java.io.Serializable{

/** Contour level of the contour line for this intersection.
 * 
 */
protected int    level = 0;

/** The first side of the cell that the contour line intersects.
 * 
 */
protected int    side1 = 0;

/** The second side of the cell that the contour line intersects.
 * 
 */
protected int    side2 = 0;

/** The point (stored in the segment) that corresponds to the side1 intersection.
 * 
 */
protected ContourSegment segment1 = null;

/** The point (stored in the segment) that corresponds to the side2 intersection.
 * 
 */
protected ContourSegment segment2 = null;

/**  * 
 */
Intersection()
{
}

/**  * 
 * @param level 
 * @param side1 
 * @param segment1 
 */
Intersection(int level, int side1, ContourSegment segment1)
{
	this.level    = level;
	this.side1    = side1;
	this.segment1 = segment1;
}

/** 
 * Sets the level for this intersection.
 * 
 * @param level 
 */
protected void setLevel(int level)
{
	this.level = level;
}

/** 
 * Gets the level for this intersection.
 * 
 */
public int getLevel()
{
	return(level);
}

/** 
 * Sets the first side for this intersection.
 * 
 * @param side1 
 */
protected void setSide1(int side1)
{
	this.side1 = side1;
}

/** 
 * Gets the first side for this intersection.
 * 
 */
public int getSide1()
{
	return(side1);
}

/** 
 * Sets the second side for this intersection.
 * 
 * @param side2 
 */
protected void setSide2(int side2)
{
	this.side2 = side2;
}

/** 
 * Gets the second side for this intersection.
 * 
 */
public int getSide2()
{
	return(side2);
}

/** 
 * Sets the first point for this contour intersection.
 * 
 * @param segment1 
 */
protected void setSegment1(ContourSegment segment1)
{
	this.segment1 = segment1;
}

/** 
 * Gets the first point for this contour intersection.
 * 
 */
public ContourSegment getSegment1()
{
	return(segment1);
}

/** 
 * Sets the second point for this contour intersection.
 * 
 * @param segment2 
 */
protected void setSegment2(ContourSegment segment2)
{
	this.segment2 = segment2;
}

/** 
 * Gets the second point for this contour intersection.
 * 
 */
public ContourSegment getSegment2()
{
	return(segment2);
}

}
