/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCAction.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import java.awt.event.InputEvent;

/**
 * The action interface for JClass Chart 3D.  The methods in this
 * interface are called by the JCChart3dArea object when a user action is
 * started, annimated, reanimated, ended, or cancelled.
 */
public interface JCAction {

/**
 * Sets the associated JCChart3dArea object
 *
 * @param chart3dArea The new JCChart3dArea object
 */
public void setChart3dArea(JCChart3dArea chart3dArea);


/**
 * Returns the associated JCChart3dArea object
 *
 * @return The current JCChart3dArea object
 */
public JCChart3dArea getChart3dArea();

/**
 * Called by <code>JCChart3dArea</code> when the action is initialized.
 *
 * @param object Initialization parameters
 */
public void initialize(Object object);

/**
 * Called by <code>JCChart3dArea</code> when the action is started.
 *
 * @param event The InputEvent that triggered this action
 * @param x The x position to start the action
 * @param y The y position to start the action
 */
public void start(InputEvent event, int x, int y);

/**
 * Called by <code>JCChart3dArea</code> when the action is to be animated.
 *
 * @param event The InputEvent that triggered this action
 * @param x The current x position to animate the action
 * @param y The current y position to animate the action
 */
public void animate(InputEvent event, int x, int y);

/**
 * Called by <code>JCChart3dArea</code> when the chart area has been refreshed
 * and the animation must be redrawn.
 *
 * @param x The x position to re-animate the action
 * @param y The y position to re-animate the action
 */
public void reanimate(int x, int y);

/**
 * Called by <code>JCChart3dArea</code> when the action is over.
 *
 * @param event The InputEvent that triggered this action
 * @param x The x position to end the action
 * @param y The y position to end the action
 */
public void end(InputEvent event, int x, int y);

/**
 * Called by <code>JCChart3dArea</code> when the action is cancelled
 */
public void cancel();

}


