/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCActionTable.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import java.awt.*;
import java.awt.event.*;
import java.io.Serializable;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeMap;

/**
 * <code>JCActionTable</code> allows the association of an <code>InputEvent</code> with
 * an action to be performed.  In particular, a <code>MouseActionInitiator</code> or
 * a <code>KeyActionInitiator</code> is associated with a class that implements the
 * <code>JCAction</code> interface.  At runtime, when the user clicks on the chart
 * with a mouse button, or presses a key while the chart has the
 * input focus, <code>JCActionTable</code> will interrogate its mapping for the
 * corresponding class that implements the desired <code>JCAction</code>.
 * This class is then instantiated to handle the <code>InputEvent</code>.
 * All of the default <code>JCActions</code> can be found in either the
 * com.klg.jclass.chart3d.j2d.actions package or the
 * com.klg.jclass.chart3d.j3d.action package.
 */
public abstract class JCActionTable implements Serializable {

/** Map of all mouse actions. */
protected TreeMap mouseActions;

/** Map of all key actions. */
protected TreeMap keyActions;

/** Map of all running action handlers. */
protected TreeMap handlerInstances;

/** <code>MouseActionInitiator</code> that pops up the customizer. */
public static final MouseActionInitiator DEFAULT_MOUSE_CUSTOMIZE_ACTION = new MouseActionInitiator(MouseEvent.BUTTON3_MASK, ActionInitiator.NO_MODIFIER_MASK);

/** <code>KeyActionInitiator</code> that pops up the customizer. */
public static final KeyActionInitiator   DEFAULT_KEY_CUSTOMIZE_ACTION = new KeyActionInitiator(KeyEvent.VK_ENTER, InputEvent.ALT_MASK);

/** <code>MouseActionInitiator</code> that allows interactive editing of the chart. */
public static final MouseActionInitiator DEFAULT_MOUSE_EDIT_ACTION = new MouseActionInitiator(MouseEvent.BUTTON1_MASK, ActionInitiator.NO_MODIFIER_MASK);

/** <code>MouseActionInitiator</code> that allows interactive picking of the chart. */
public static final MouseActionInitiator DEFAULT_MOUSE_PICK_ACTION = new MouseActionInitiator(MouseEvent.BUTTON1_MASK, ActionInitiator.NO_MODIFIER_MASK);

/** <code>MouseActionInitiator</code> that allows interactive rotation of the chart with no constraint. */
public static final MouseActionInitiator DEFAULT_MOUSE_ROTATE_ANY_ACTION = new MouseActionInitiator(MouseEvent.BUTTON1_MASK, InputEvent.CTRL_MASK);

/** <code>MouseActionInitiator</code> that allows interactive rotation of the chart constrained to the eye. */
public static final MouseActionInitiator DEFAULT_MOUSE_ROTATE_EYE_ACTION = new MouseActionInitiator(MouseEvent.BUTTON1_MASK, InputEvent.CTRL_MASK);

/** <code>MouseActionInitiator</code> that allows interactive rotation of the chart constrained to the X axis. */
public static final MouseActionInitiator DEFAULT_MOUSE_ROTATE_X_ACTION = new MouseActionInitiator(MouseEvent.BUTTON1_MASK, InputEvent.CTRL_MASK);

/** <code>MouseActionInitiator</code> that allows interactive rotation of the chart constrained to the Y axis. */
public static final MouseActionInitiator DEFAULT_MOUSE_ROTATE_Y_ACTION = new MouseActionInitiator(MouseEvent.BUTTON1_MASK, InputEvent.CTRL_MASK);

/** <code>MouseActionInitiator</code> that allows interactive rotation of the chart constrained to the Z axis. */
public static final MouseActionInitiator DEFAULT_MOUSE_ROTATE_Z_ACTION = new MouseActionInitiator(MouseEvent.BUTTON1_MASK, InputEvent.CTRL_MASK);

/** <code>KeyActionInitiator</code> that removes any constraint for interactive rotation. */
public static final KeyActionInitiator   DEFAULT_KEY_SWITCH_ROTATE_ANY_ACTION = new KeyActionInitiator(KeyEvent.VK_N, ActionInitiator.NO_MODIFIER_MASK);

/** <code>KeyActionInitiator</code> that constrains subsequent interactive rotation to the eye. */
public static final KeyActionInitiator   DEFAULT_KEY_SWITCH_ROTATE_EYE_ACTION = new KeyActionInitiator(KeyEvent.VK_E, ActionInitiator.NO_MODIFIER_MASK);

/** <code>KeyActionInitiator</code> that constrains subsequent interactive rotation about the X axis. */
public static final KeyActionInitiator   DEFAULT_KEY_SWITCH_ROTATE_X_ACTION = new KeyActionInitiator(KeyEvent.VK_X, ActionInitiator.NO_MODIFIER_MASK);

/** <code>KeyActionInitiator</code> that constrains subsequent interactive rotation about the Y axis. */
public static final KeyActionInitiator   DEFAULT_KEY_SWITCH_ROTATE_Y_ACTION = new KeyActionInitiator(KeyEvent.VK_Y, ActionInitiator.NO_MODIFIER_MASK);

/** <code>KeyActionInitiator</code> that constrains subsequent interactive rotation about the Z axis. */
public static final KeyActionInitiator   DEFAULT_KEY_SWITCH_ROTATE_Z_ACTION = new KeyActionInitiator(KeyEvent.VK_Z, ActionInitiator.NO_MODIFIER_MASK);

/** <code>KeyActionInitiator</code> that spins the chart counter-clockwise. */
public static final KeyActionInitiator   DEFAULT_KEY_SPIN_CCW_ACTION = new KeyActionInitiator(KeyEvent.VK_LEFT, ActionInitiator.NO_MODIFIER_MASK);

/** <code>KeyActionInitiator</code> that spins the chart clockwise. */
public static final KeyActionInitiator   DEFAULT_KEY_SPIN_CW_ACTION = new KeyActionInitiator(KeyEvent.VK_RIGHT, ActionInitiator.NO_MODIFIER_MASK);

/** <code>KeyActionInitiator</code> that tilts and spins the chart counter-clockwise. */
public static final KeyActionInitiator   DEFAULT_KEY_TILT_CCW_ACTION = new KeyActionInitiator(KeyEvent.VK_UP, ActionInitiator.NO_MODIFIER_MASK);

/** <code>KeyActionInitiator</code> that tilts and spins the chart clockwise. */
public static final KeyActionInitiator   DEFAULT_KEY_TILT_CW_ACTION = new KeyActionInitiator(KeyEvent.VK_DOWN, ActionInitiator.NO_MODIFIER_MASK);

/** <code>MouseActionInitiator</code> that allows interactive scaling of the chart. */
public static final MouseActionInitiator DEFAULT_MOUSE_SCALE_ACTION = new MouseActionInitiator(MouseEvent.BUTTON1_MASK, InputEvent.ALT_MASK);

/** <code>KeyActionInitiator</code> that allows interactive scaling increase of the chart. */
public static final KeyActionInitiator   DEFAULT_KEY_SCALE_UP_ACTION = new KeyActionInitiator(KeyEvent.VK_PAGE_UP, ActionInitiator.NO_MODIFIER_MASK);

/** <code>KeyActionInitiator</code> that allows interactive scaling decrease of the chart. */
public static final KeyActionInitiator   DEFAULT_KEY_SCALE_DOWN_ACTION = new KeyActionInitiator(KeyEvent.VK_PAGE_DOWN, ActionInitiator.NO_MODIFIER_MASK);

/** <code>MouseActionInitiator</code> that allows interactive translation (movement) of the chart. */
public static final MouseActionInitiator DEFAULT_MOUSE_TRANSLATE_ACTION = new MouseActionInitiator(MouseEvent.BUTTON1_MASK, InputEvent.SHIFT_MASK);

/** <code>KeyActionInitiator</code> that allows interactive translation (movement) of the chart to the left. */
public static final KeyActionInitiator   DEFAULT_KEY_TRANSLATE_LEFT_ACTION = new KeyActionInitiator(KeyEvent.VK_LEFT, InputEvent.SHIFT_MASK);

/** <code>KeyActionInitiator</code> that allows interactive translation (movement) of the chart to the right. */
public static final KeyActionInitiator   DEFAULT_KEY_TRANSLATE_RIGHT_ACTION = new KeyActionInitiator(KeyEvent.VK_RIGHT, InputEvent.SHIFT_MASK);

/** <code>KeyActionInitiator</code> that allows interactive translation (movement) of the chart upwards. */
public static final KeyActionInitiator   DEFAULT_KEY_TRANSLATE_UP_ACTION = new KeyActionInitiator(KeyEvent.VK_UP, InputEvent.SHIFT_MASK);

/** <code>KeyActionInitiator</code> that allows interactive translation (movement) of the chart downwards. */
public static final KeyActionInitiator   DEFAULT_KEY_TRANSLATE_DOWN_ACTION = new KeyActionInitiator(KeyEvent.VK_DOWN, InputEvent.SHIFT_MASK);

/** <code>MouseActionInitiator</code> that allows interactive zooming of the chart. */
public static final MouseActionInitiator DEFAULT_MOUSE_ZOOM_ACTION = new MouseActionInitiator(MouseEvent.BUTTON1_MASK, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK);

/** <code>KeyActionInitiator</code> that resets the chart. */
public static final KeyActionInitiator   DEFAULT_KEY_RESET_ACTION = new KeyActionInitiator(KeyEvent.VK_R, ActionInitiator.ANY_MODIFIER_MASK);

/** <code>KeyActionInitiator</code> that cancels the current action. */
public static final KeyActionInitiator   DEFAULT_KEY_CANCEL_ACTION = new KeyActionInitiator(KeyEvent.VK_C, ActionInitiator.ANY_MODIFIER_MASK);

/**
 * No args constructor.
 */
public JCActionTable() {
    mouseActions = new TreeMap();
    keyActions = new TreeMap();
    handlerInstances = new TreeMap();
}

////////////////////////
// ActionHandling

/**
 * Associates a given <code>ActionInitiator</code> with an action handler
 * (<code>JCAction</code>) that will handle an action.
 * 
 * @param initiator the <code>MouseActionInitiator</code> or <code>KeyActionInitiator</code> that
 *                  specifies the <code>InputEvent</code>
 * @param className the fully specified name of the class that implements
 *                  the <code>JCAction</code> interface
 */
public void addAction(ActionInitiator initiator, String className) {
    if (initiator instanceof MouseActionInitiator) {
        mouseActions.put(initiator, className);
    }
    else if (initiator instanceof KeyActionInitiator) {
        keyActions.put(initiator, className);
    }
}

/**
 * Returns an instance of the <code>JCAction</code> to handle the <code>InputEvent</code>.
 * This method must be overridden by subclasses.
 * @return a <code>JCAction</code> instance
 * @param event
 * @param object
 */
public abstract JCAction getActionHandlerInstance(InputEvent event, Object object);

/**
 * Returns the mouse button pressed, based on the modifers.
 * @return the mouse button pressed
 * @param modifiers
 */
private int getMouseButtonMask(int modifiers) {
    if ((modifiers & InputEvent.BUTTON1_MASK) == InputEvent.BUTTON1_MASK) {
        return InputEvent.BUTTON1_MASK;
    }
    else if ((modifiers & InputEvent.BUTTON2_MASK) == InputEvent.BUTTON2_MASK) {
        return InputEvent.BUTTON2_MASK;
    }
    else if ((modifiers & InputEvent.BUTTON3_MASK) == InputEvent.BUTTON3_MASK) {
        return InputEvent.BUTTON3_MASK;
    }
    else {
        // should never happen
        return 0;
    }
}

/**
 * Returns the action mapped to the given event. If no action
 * exists, <CODE>null</CODE> is returned.
 * @return action mapped to the given event
 * @param event
 */
public String getAction(InputEvent event) {
    // construct an ActionInitiator from the AWT event
    Object className = null;
    if (event instanceof KeyEvent) {
        KeyEvent keyEvent = (KeyEvent)event;
        KeyActionInitiator initiator = new KeyActionInitiator(keyEvent.getKeyCode(), keyEvent.getModifiers());
        className = keyActions.get(initiator);
        if (className == null) {
            // try the "none" variant
            initiator.modifiers = ActionInitiator.NO_MODIFIER_MASK;
            className = keyActions.get(initiator);
            if (className == null) {
                // try the "any" variant
                initiator.modifiers = ActionInitiator.ANY_MODIFIER_MASK;
                className = keyActions.get(initiator);
            }
        }
    }
    else if (event instanceof MouseEvent) {
        MouseEvent mouseEvent = (MouseEvent)event;
        MouseActionInitiator initiator = new MouseActionInitiator(
            getMouseButtonMask(mouseEvent.getModifiers()),
            mouseEvent.getModifiers() & (InputEvent.SHIFT_MASK | InputEvent.CTRL_MASK | InputEvent.ALT_MASK));
        className = mouseActions.get(initiator);
        if (className == null) {
            // try the "none" variant
            initiator.modifiers = ActionInitiator.NO_MODIFIER_MASK;
            className = mouseActions.get(initiator);
            if (className == null) {
                // try the "any" variant
                initiator.modifiers = ActionInitiator.ANY_MODIFIER_MASK;
                className = mouseActions.get(initiator);
            }
        }
    }
    return (String) className;
}

/**
 * Removes an action.
 *
 * @param initiator the <code>ActionInitiator</code> to remove
 */
public void removeAction(ActionInitiator initiator) {
    if (initiator instanceof KeyActionInitiator) {
        removeKeyAction((KeyActionInitiator)initiator);
    }
    else if (initiator instanceof MouseActionInitiator) {
        removeMouseAction((MouseActionInitiator)initiator);
    }
}

/**
 * Removes a key action.
 * @param initiator the <code>KeyActionInitiator</code> to remove
 */
public void removeKeyAction(KeyActionInitiator initiator) {
    keyActions.remove(initiator);
}

/**
 * Removes a mouse action.
 * @param initiator the <code>MouseActionInitiator</code> to remove
 */
public void removeMouseAction(MouseActionInitiator initiator) {
    mouseActions.remove(initiator);
}

/**
 * Removes all key actions from JClass Chart 3D.
 */
public void removeAllKeyActions() {
    keyActions.clear();
}

/**
 * Removes all mouse actions from JClass Chart 3D.
 */
public void removeAllMouseActions() {
    mouseActions.clear();
}

/**
 * Removes all actions from JClass Chart 3D.
 */
public void removeAllActions() {
    removeAllKeyActions();
    removeAllMouseActions();
}

/**
 * Returns a Set of the Key Actions.
 *
 * @return a Set of the Key Actions
 */
public Set getAllKeyActions() {
    return keyActions.keySet();
}

/**
 * Returns a Set of the Mouse Actions.
 *
 * @return a Set of the Mouse Actions
 */
public Set getAllMouseActions() {
    return mouseActions.keySet();
}

/**
 * Re-creates all standard actions. Call <CODE>removeAllActions()</CODE> first
 * if you want to start anew.
 */
public void addAllDefaultActions() {
    addAllDefaultKeyActions();
    addAllDefaultMouseActions();
}

/**
 * Re-creates all standard key actions. Call <CODE>removeAllKeyActions()</CODE> first
 * if you want to start anew with key actions.
 */
public void addAllDefaultKeyActions() {
    addAction(DEFAULT_KEY_CANCEL_ACTION, getDefaultCancelActionClass());
    addAction(DEFAULT_KEY_CUSTOMIZE_ACTION, getDefaultCustomizeActionClass());
    addAction(DEFAULT_KEY_RESET_ACTION, getDefaultResetActionClass());
    addAction(DEFAULT_KEY_SPIN_CCW_ACTION, getDefaultRotateActionClass());
    addAction(DEFAULT_KEY_SPIN_CW_ACTION, getDefaultRotateActionClass());
    addAction(DEFAULT_KEY_TILT_CCW_ACTION, getDefaultRotateActionClass());
    addAction(DEFAULT_KEY_TILT_CW_ACTION, getDefaultRotateActionClass());
    addAction(DEFAULT_KEY_SCALE_UP_ACTION, getDefaultScaleActionClass());
    addAction(DEFAULT_KEY_SCALE_DOWN_ACTION, getDefaultScaleActionClass());
    addAction(DEFAULT_KEY_TRANSLATE_LEFT_ACTION, getDefaultTranslateActionClass());
    addAction(DEFAULT_KEY_TRANSLATE_RIGHT_ACTION, getDefaultTranslateActionClass());
    addAction(DEFAULT_KEY_TRANSLATE_UP_ACTION, getDefaultTranslateActionClass());
    addAction(DEFAULT_KEY_TRANSLATE_DOWN_ACTION, getDefaultTranslateActionClass());
    addAction(DEFAULT_KEY_SWITCH_ROTATE_ANY_ACTION, getDefaultSwitchRotateAnyActionClass());
    addAction(DEFAULT_KEY_SWITCH_ROTATE_EYE_ACTION, getDefaultSwitchRotateEyeActionClass());
    addAction(DEFAULT_KEY_SWITCH_ROTATE_X_ACTION, getDefaultSwitchRotateXActionClass());
    addAction(DEFAULT_KEY_SWITCH_ROTATE_Y_ACTION, getDefaultSwitchRotateYActionClass());
    addAction(DEFAULT_KEY_SWITCH_ROTATE_Z_ACTION, getDefaultSwitchRotateZActionClass());
}

/**
 * Re-creates all standard mouse actions. Call <CODE>removeAllMouseActions()</CODE> first
 * if you want to start anew with mouse actions.
 */
public void addAllDefaultMouseActions() {
    addAction(DEFAULT_MOUSE_CUSTOMIZE_ACTION, getDefaultCustomizeActionClass());
//FIX
//    addAction(DEFAULT_MOUSE_EDIT_ACTION, getDefaultEditActionClass());
    addAction(DEFAULT_MOUSE_PICK_ACTION, getDefaultPickActionClass());
    addAction(DEFAULT_MOUSE_ROTATE_ANY_ACTION, getDefaultRotateActionClass());
    addAction(DEFAULT_MOUSE_SCALE_ACTION, getDefaultScaleActionClass());
    addAction(DEFAULT_MOUSE_TRANSLATE_ACTION, getDefaultTranslateActionClass());
    addAction(DEFAULT_MOUSE_ZOOM_ACTION, getDefaultZoomActionClass());
}

/**
 * Returns the name of the default class that implements the Cancel Action.
 * @return default class name
 */
public abstract String getDefaultCancelActionClass();

/**
 * Returns the name of the default class that implements the Customize Action.
 * @return default class name
 */
public abstract String getDefaultCustomizeActionClass();

/**
 * Returns the name of the default class that implements the Edit Action.
 * @return default class name
 */
public abstract String getDefaultEditActionClass();

/**
 * Returns the name of the default class that implements the Pick Action.
 * @return default class name
 */
public abstract String getDefaultPickActionClass();

/**
 * Returns the name of the default class that implements the Reset Action.
 * @return default class name
 */
public abstract String getDefaultResetActionClass();

/**
 * Returns the name of the default class that implements the Rotate Action.
 * @return default class name
 */
public abstract String getDefaultRotateActionClass();

/**
 * Returns the name of the default class that implements the RotateEye Action.
 * @return default class name
 */
public abstract String getDefaultRotateEyeActionClass();

/**
 * Returns the name of the default class that implements the RotateX Action.
 * @return default class name
 */
public abstract String getDefaultRotateXActionClass();

/**
 * Returns the name of the default class that implements the RotateY Action.
 * @return default class name
 */
public abstract String getDefaultRotateYActionClass();

/**
 * Returns the name of the default class that implements the RotateZ Action.
 * @return default class name
 */
public abstract String getDefaultRotateZActionClass();

/**
 * Returns the name of the default class that implements the Scale Action.
 * @return default class name
 */
public abstract String getDefaultScaleActionClass();

/**
 * Returns the name of the default class that implements the RotateAny Action.
 * @return default class name
 */
public abstract String getDefaultSwitchRotateAnyActionClass();

/**
 * Returns the name of the default class that implements the SwitchRotateEye Action.
 * @return default class name
 */
public abstract String getDefaultSwitchRotateEyeActionClass();

/**
 * Returns the name of the default class that implements the SwitchRotateX Action.
 * @return default class name
 */
public abstract String getDefaultSwitchRotateXActionClass();

/**
 * Returns the name of the default class that implements the SwitchRotateY Action.
 * @return default class name
 */
public abstract String getDefaultSwitchRotateYActionClass();

/**
 * Returns the name of the default class that implements the SwitchRotateZ Action.
 * @return default class name
 */
public abstract String getDefaultSwitchRotateZActionClass();

/**
 * Returns the name of the default class that implements the Translate Action.
 * @return default class name
 */
public abstract String getDefaultTranslateActionClass();

/**
 * Returns the name of the default class that implements the Zoom Action.
 * @return default class name
 */
public abstract String getDefaultZoomActionClass();

}
