/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCBar.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:12:58 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import com.klg.jclass.chart3d.resources.LocaleBundle;

/**
 * JClass Chart 3D class that contains properties that pertain to plots of
 * type bar. These are properties such as the bar format and spcaing.
 */
public class JCBar extends ParentTrackChange {

static final long serialVersionUID = -5542417106751358122L;

/**
 * The fixed bar format. Center of bar is at each data point. The bar spacing
 * determines the bar width.
 */
public final static int FIXED     = 1;

/**
 * The hisogram bar format.  Endpoints of bar are at datapoints.  Bar spacing
 * has no effect.
 */
public final static int HISTOGRAM = 2;

/** <i>X</i> bar format.  <code>FIXED</code> or <code>HISTOGRAM</code>. */
protected int           xFormat   = FIXED;

/** <i>Y</i> bar format.  <code>FIXED</code> or <code>HISTOGRAM</code>. */
protected int           yFormat   = FIXED;

/** <i>X</i> bar spacing.  Percentage of possible space used. */
protected double        xSpacing  = 80.0;

/** <i>Y</i> bar spacing.  Percentage of possible space used. */
protected double        ySpacing  = 80.0;

/**
 * Default constructor.  All values at their default.
 */
public JCBar()
{
}

/**
 * Constructor that intializes the <i>X</i> & <i>Y</i> formats and spacing.
 *
 * @param xFormat the <i>X</i> bar format
 * @param yFormat the <i>Y</i> bar format
 * @param xSpacing the <i>X</I> bar spacing
 * @param ySpacing the <I>Y</I> bar spacing
 */
public JCBar(int xFormat, int yFormat, double xSpacing, double ySpacing)
{
	this.xFormat  = xFormat;
	this.yFormat  = yFormat;
	this.xSpacing = xSpacing;
	this.ySpacing = ySpacing;
}

/**
 * Gets the current <i>x</i> or <i>X</i> axis bar format.  The possible values
 * are <code>FIXED</code> or <code>HISTOGRAM</code>.
 *
 * @return the current <i>X</i> bar format
 */
public int getXFormat()
{
	return(xFormat);
}

/**
 * Sets the current <i>x</i> or <i>X</i> axis bar format.  The possible values
 * are <code>FIXED</code> or <code>HISTOGRAM</code>.
 *
 * @param xFormat the new <i>X</i> bar format
 */
public void setXFormat(int xFormat)
{
	if (this.xFormat == xFormat) {
		return;
	}
	if (xFormat >= FIXED && xFormat <= HISTOGRAM) {
	   this.xFormat = xFormat;
	}
	else {
		throw new IllegalArgumentException(
            LocaleBundle.string(LocaleBundle.INVALID_FIXED_OR_HISTOGRAM_ERROR));
	}
	callParentSetChanged(true, Changeable.RECALC);
}

/**
 * Gets the current <i>y</i> or <i>Y</i> axis bar format.  The possible values
 * are <code>FIXED</code> or <code>HISTOGRAM</code>.
 *
 * @return the current <I>Y</I> format
 */
public int getYFormat()
{
	return(yFormat);
}

/**
 * Sets the current <i>y</i> or <i>Y</i> axis bar format.  The possible values
 * are <code>FIXED</code> or <code>HISTOGRAM</code>.
 *
 * @param yFormat the new <I>Y</I> bar format
 */
public void setYFormat(int yFormat)
{
	if (this.yFormat == yFormat) {
		return;
	}
	if (yFormat >= FIXED && yFormat <= HISTOGRAM) {
	   this.yFormat = yFormat;
	}
	else {
		throw new IllegalArgumentException(
            LocaleBundle.string(LocaleBundle.INVALID_FIXED_OR_HISTOGRAM_ERROR));
	}
	callParentSetChanged(true, Changeable.RECALC);
}

/**
 * Gets the current <i>X</i> bar spacing.  This value is a percentage of the
 * available bar space that is used.
 *
 * @return the current <I>X</i> bar spacing
 */
public double getXSpacing()
{
	return(xSpacing);
}

/**
 * Sets the current <i>X</i> bar spacing.  This value is a percentage of the
 * available bar space that is used.
 *
 * @param xSpacing the new <I>X</i> bar spacing value
 */
public void setXSpacing(double xSpacing)
{
	if (this.xSpacing == xSpacing) {
		return;
	}
	this.xSpacing = JCChart3dUtil.clamp(xSpacing, 0.0, 100.0);
	callParentSetChanged(true, Changeable.RECALC);
}

/**
 * Gets the current <i>Y</i> bar spacing.  This value is a percentage of the
 * available bar space that is used.
 *
 * @return the current <I>Y</I> bar spacing
 */
public double getYSpacing()
{
	return(ySpacing);
}

/**
 * Sets the current <i>Y</i> bar spacing.  This value is a percentage of the
 * available bar space that is used.
 *
 * @param ySpacing the new column bar spacing value
 */
public void setYSpacing(double ySpacing)
{
	if (this.ySpacing == ySpacing) {
		return;
	}
	this.ySpacing = JCChart3dUtil.clamp(ySpacing, 0.0, 100.0);
	callParentSetChanged(true, Changeable.RECALC);
}

}
