/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCChart3dArea.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:12:58 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import javax.swing.JComponent;
import java.awt.Container;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Point;
import java.awt.event.InputEvent;
import java.io.IOException;

import com.klg.jclass.util.legend.JCLegend;
import com.klg.jclass.chart3d.resources.LocaleBundle;


 


 

/**
 * This is the sub-portion of the overall <code>chart3d</code> real estate onto which the
 * actual linear/circular portion of the <code>chart3d</code> is drawn. Headers, footers,
 * and legends are not drawn in this area; the 3d graph is.
 */
public class JCChart3dArea extends JComponent
						   implements Changeable, java.io.Serializable {

static final long serialVersionUID = -5542417106751358122L;

// Used to turn off updates to parent in certain cases.
protected boolean   updateParent = true;

// Graphics used for drawing zoom rectangle (i.e. rubberbanding).
protected Graphics rbGC = null;

/**
 * Boolean that represents the value of the <code>Changed</code> property.
 */
protected boolean   changed     = true;

// Flag that accumulates user changes
protected int       changedFlag = Changeable.NO_CHANGE;

/** Asks if we are currently in an action. */
protected boolean   inAction = false;

/** If an action is in process, this is the handler that will handle it. */
protected transient JCAction actionHandler  = null;

/** What is our preferred size?  If null, the super's preferred size is used. */
protected Dimension preferredSize = null;

/**
 * Temporary <code>Graphics</code> object used to draw without showing to
 * screen.
 */
private transient Graphics prevSetGraphics = null;

/**
 * The chart3d to be placed in this chart3d area.
 */
protected JCChart3d     chart3d = null;

/** The axis in the <i>X</i> direction. */
protected JCAxis        xAxis  = null;

/** The axis in the <i>Y</i> direction. */
protected JCAxis        yAxis  = null;

/** The axis in the <i>Z</i> direction. */
protected JCAxis        zAxis  = null;

/** A list of the <I>X</i>, <I>Y</i>, and <I>Z</i> axis objects. */
protected JCAxis        axes[] = null;

/** An object that allows the user to control the drawing viewport. */
protected JCViewport    viewport  = null;

/** An object that controls the <I>X</i>, <I>Y</i>, and <i>Z</i> rotation. */
protected JCView3d      view3d    = null;

/* Object that stores generic cube information. */
protected Cube          cube      = null;

/** Object that controls properties of the plot cube. */
protected JCPlotCube    plotCube  = null;

/* Object that computes and handles 3D transformations. */
protected Transform     transform = null;

/* Object that computes the extents and draws annotation. */
protected Anno          anno      = null;

/** Object that controls surface only properties. */
protected JCSurface     surface   = null;

/** Object that controls bar chart only properties. */
protected JCBar         bar       = null;

/** Object that controls scatter plot only properties. */
protected JCScatter     scatter   = null;

/**
 * No arguments constructor.
 */
public JCChart3dArea()
{
}

/**
 * Creates a <code>chart3d</code> area for a 3D chart.
 *
 * @param chart3d the <code>JCChart3d</code> whose graph will be drawn in this
 *      <code>chart3d</code> area
 */
public JCChart3dArea(JCChart3d chart3d)
{
	setOpaque(false);

	this.chart3d = chart3d;
}

/**
 * Gets the chart area's associated <code>JCChart3d</code> object (normally its
 * parent).
 *
 * @return the <code>JCChart3d</code> object referenced by this
 *     <code>JCChart3dArea</code>
 */
public JCChart3d getChart3d()
{
	return(chart3d);
}

/**
 * Sets the parent <code>JCChart3d</code> object.
 *
 * @param chart3d The new <code>JCChart3d</code> object
 */
public void setChart3d(JCChart3d chart3d)
{
	if (this.chart3d == chart3d) {
		return;
	}
	this.chart3d = chart3d;
    setChanged(true, Changeable.ALL);
}

/**
 * Overrides <code>setForeground</code> to make sure legend is marked for
 * redraw.  (Needed for outline color of swatches, which is derived from
 * <code>Chart3dArea</code>.)
 *
 * @param c The new foreground color
 */
public void setForeground(Color c)
{
	super.setForeground(c);
    setChanged(true, Changeable.REDRAW | Changeable.NOTIFY_LEGEND_RECALC);
}

/**
 * Overrides <code>setBackground</code> to make sure that the entire chart is
 * redrawn if the background is set on this subcomponent.<BR>
 * <b>Note:</b> The <code>Opaque</code> property needs to be set to
 * <code>true</code> for the background color to be visible.
 * <code>JCChartArea</code> is transparent by default.
 *
 * @param bg The new background color
 */
public void setBackground(Color bg)
{
    super.setBackground(bg);
    invalidate();
    Container thisParent = getParent();
    if (thisParent != null) {
        thisParent.validate();
    }
}

/**
 * Overrides <code>setOpaque</code> to make sure entire chart is redrawn if
 * opacity is set on this subcomponent.
 *
 * @param op the new value of opaque
 */
public void setOpaque(boolean op)
{
    super.setOpaque(op);
    invalidate();
    Container thisParent = getParent();
    if (thisParent != null) {
        thisParent.validate();
    }
}

/**
 * Gets the bounding rectangle of the component's drawing area, (its area
 * minus the shadows and insets).
 *
 * @see javax.swing.JComponent#setBorder
 * @return a rectangle representing the component's drawing area
 */
public Rectangle getDrawingArea()
{
	Rectangle rect = new Rectangle();
	getDrawingArea(rect);
	return(rect);
}

/**
 * Gets the bounding rectangle of the component's drawing area, (its area
 * minus the insets).
 *
 * @param rect the rectangle in which to put the drawing area information
 * @see javax.swing.JComponent#setBorder
 */
public void getDrawingArea(Rectangle rect)
{
	Insets        insets = getInsets();
	Dimension     sz     = getSize();
	rect.setBounds(insets.left, insets.top,
			       Math.max(0, sz.width  - (insets.left + insets.right)),
			       Math.max(0, sz.height - (insets.top  + insets.bottom)));
}

/**
 * Returns the preferred size for the chart area.
 * @return the current preferred size
 */
public Dimension getPreferredSize()
{
    Dimension ps = preferredSize;
    if (ps == null || ps.width <= 0 || ps.height <= 0) {
        ps = new Dimension();
        ps.width  = 250;
        ps.height = 250;
    }
    return(ps);
}

/**
 * Sets the preferred size manually for this <code>JCChart3dArea</code>
 * instance.
 *
 * @param preferredSize the preferred size
 */
public void setPreferredSize(Dimension preferredSize)
{
    this.preferredSize = preferredSize;
}

/**
 * Returns the minimum size for the chart area.
 *
 * @return a <code>Dimension</code> object containing the minimum size
 */
public Dimension getMinimumSize()
{
    return(getPreferredSize());
}

/**
 * Returns the axis based on the given <code>axisId</code>.
 *
 * @param axisId the axis ID (either <code>AXIS_X</code>, <code>AXIS_Y</code>,
 *     or <code>AXIS_Z</code>).
 * @return the <code>JCAxis</code> object based on the given
 *     <code>axisId</code>
 */
public JCAxis getAxis(int axisId)
{
    if (axes == null) {
        return(null);
    }
	for (int i = 0; i < JCAxis.NUM_AXES; i++) {
		if (axes[i].getAxisId() == axisId) {
			return(axes[i]);
		}
	}
	return(null);
}

/**
 * Sets an axis on the chart based on <code>axisId</code>.
 *
 * @param axis the new axis
 * @param axisId Which one of <i>X</I>, <I>Y</i>, or <I>Z</i> axis is it?
 */
public void setAxis(JCAxis axis, int axisId)
{
	if (axis == null) {
		throw new IllegalArgumentException(
		    LocaleBundle.string(LocaleBundle.INVALID_VALUE_NULL_ERROR));
	}
    if (axisId < JCAxis.AXIS_X || axisId > JCAxis.AXIS_Z) {
        return;
    }
    if (axis.getAxisId() != axisId) {
		throw new IllegalArgumentException(
            LocaleBundle.string(LocaleBundle.AXIS_AND_ID_MISMATCH_ERROR));
    }
    axes[axisId - 1] = axis;
    if (axisId == JCAxis.AXIS_X) {
		if (this.xAxis == axis) {
			return;
		}
        this.xAxis = axis;
    }
    else if (axisId == JCAxis.AXIS_Y) {
		if (this.yAxis == axis) {
			return;
		}
        this.yAxis = axis;
    }
    else if (axisId == JCAxis.AXIS_Z) {
		if (this.zAxis == axis) {
			return;
		}
        this.zAxis = axis;
    }
	axis.addParent(this);
	setChanged(true, Changeable.ALL);
}

/**
 * Returns the list of axes for this 3d chart.
 *
 * @return the current list of axes for this chart
 */
public JCAxis[] getAxes()
{
	return(axes);
}

/**
 * Sets all axes for the chart via an array of axes.
 *
 * @param axes an array of axes (<I>X</i>, <I>Y</i>, and <I>Z</i>)
 */
public void setAxes(JCAxis[] axes)
{
    if (axes.length != JCAxis.NUM_AXES) {
		throw new IllegalArgumentException(
            LocaleBundle.string(LocaleBundle.ARRAY_LENGTH_INCORRECT_ERROR));
    }
    this.axes = axes;
	boolean hasChanged = false;
	if (this.xAxis != axes[0]) {
        if (axes[0].getAxisId() != JCAxis.AXIS_X) {
		    throw new IllegalArgumentException(
                LocaleBundle.string(LocaleBundle.AXIS_AND_ID_MISMATCH_ERROR));
        }
        this.xAxis = axes[0];
	    axes[0].addParent(this);
		hasChanged = true;
    }
	if (this.yAxis != axes[1]) {
        if (axes[1].getAxisId() != JCAxis.AXIS_Y) {
		    throw new IllegalArgumentException(
                LocaleBundle.string(LocaleBundle.AXIS_AND_ID_MISMATCH_ERROR));
        }
        this.yAxis = axes[1];
	    axes[1].addParent(this);
		hasChanged = true;
    }
	if (this.zAxis != axes[2]) {
        if (axes[2].getAxisId() != JCAxis.AXIS_Z) {
		    throw new IllegalArgumentException(
                LocaleBundle.string(LocaleBundle.AXIS_AND_ID_MISMATCH_ERROR));
        }
        this.zAxis = axes[2];
	    axes[2].addParent(this);
		hasChanged = true;
    }
	if (hasChanged) {
	    setChanged(true, Changeable.ALL);
	}
}

/**
 * Gets the <i>X</i> axis for this <code>chart3dArea</code> object.
 * @return the <i>X</i> axis for this <code>chart3dArea</code> object
 */
public JCAxis getXAxis()
{
	return(xAxis);
}

/**
 * Sets the <i>X</i> axis for this <code>chart3dArea</code> object.
 * <BR><B>Note:</b> The <code>axisId</code> of the axis that is passed in must
 * be equal to <code>JCAxis.AXIS_X</code>.
 * @param axis the new <i>X</i> axis
 */
public void setXAxis(JCAxis axis)
{
	if (axis == null) {
		throw new IllegalArgumentException(
            LocaleBundle.string(LocaleBundle.INVALID_VALUE_NULL_ERROR));
	}
    if (axis.getAxisId() != JCAxis.AXIS_X) {
		throw new IllegalArgumentException(
            LocaleBundle.string(LocaleBundle.AXIS_AND_ID_MISMATCH_ERROR));
    }
    axes[JCAxis.AXIS_X - 1] = axis;
	this.xAxis = axis;
	axis.addParent(this);
	setChanged(true, Changeable.ALL);
}

/**
 * Gets the <i>Y</i> axis for this <code>chart3dArea</code> object.
 * @return the <i>Y</i> axis for this <code>chart3dArea</code> object
 */
public JCAxis getYAxis()
{
	return(yAxis);
}

/**
 * Sets the <i>Y</i> axis for this <code>chart3dArea</code> object.
 * <BR><B>Note:</b> The <code>axisId</code> of the axis that is passed in must
 * be equal to <code>JCAxis.AXIS_Y</code>.
 * @param axis the new <i>Y</i> axis
 */
public void setYAxis(JCAxis axis)
{
	if (axis == null) {
		throw new IllegalArgumentException(
            LocaleBundle.string(LocaleBundle.INVALID_VALUE_NULL_ERROR));
	}
    if (axis.getAxisId() != JCAxis.AXIS_Y) {
		throw new IllegalArgumentException(
            LocaleBundle.string(LocaleBundle.AXIS_AND_ID_MISMATCH_ERROR));
    }
    axes[JCAxis.AXIS_Y - 1] = axis;
	this.yAxis = axis;
	axis.addParent(this);
	setChanged(true, Changeable.ALL);
}

/**
 * Gets the <i>Z</i> axis for this <code>chart3dArea</code> object.
 * @return the <i>Z</i> axis for this <code>chart3dArea</code> object
 */
public JCAxis getZAxis()
{
	return(zAxis);
}

/**
 * Sets the <i>Z</i> axis for this <code>chart3dArea</code> object.
 * <br><b>Note:</b> The <code>axisId</code> of the axis that is passed in must
 * be equal to <code>JCAxis.AXIS_Z</code>.
 * @param axis the new <i>Z</i> axis
 */
public void setZAxis(JCAxis axis)
{
	if (axis == null) {
		throw new IllegalArgumentException(
            LocaleBundle.string(LocaleBundle.INVALID_VALUE_NULL_ERROR));
	}
    if (axis.getAxisId() != JCAxis.AXIS_Z) {
		throw new IllegalArgumentException(
            LocaleBundle.string(LocaleBundle.AXIS_AND_ID_MISMATCH_ERROR));
    }
    axes[JCAxis.AXIS_Z - 1] = axis;
	this.zAxis = axis;
	axis.addParent(this);
	setChanged(true, Changeable.ALL);
}

/**
 * Gets the chart area's <code>JCViewport</code> object.
 *
 * @return the <code>JCViewport</code> object referenced by this
 *     <code>JCChart3dArea</code>
 */
public JCViewport getViewport()
{
	return(viewport);
}

/**
 * Sets the chart area's <code>JCViewport</code> object.
 *
 * @param viewport the new <code>JCViewport</code> object
 */
public void setViewport(JCViewport viewport)
{
	if (this.viewport == viewport) {
		return;
	}
    this.viewport = viewport;
	if (this.viewport != null) {
		this.viewport.addParent(this);
	}
	else {
		throw new IllegalArgumentException(
            LocaleBundle.string(LocaleBundle.INVALID_VALUE_NULL_ERROR));
	}
	setChanged(true, Changeable.RECALC);
}

/**
 * Gets the chart area's <code>JCView3d</code> object.
 *
 * @return the <code>JCView3d</code> object referenced by this
 *     <code>JCChart3dArea</code>
 */
public JCView3d getView3d()
{
	return(view3d);
}

/**
 * Sets the chart area's <code>JCView3d</code> object.
 *
 * @param view3d the new <code>JCView3d</code> object
 */
public void setView3d(JCView3d view3d)
{
	if (this.view3d == view3d) {
		return;
	}
    this.view3d = view3d;
	if (this.view3d != null) {
		this.view3d.addParent(this);
	}
	else {
		throw new IllegalArgumentException(
            LocaleBundle.string(LocaleBundle.INVALID_VALUE_NULL_ERROR));
	}
	setChanged(true, Changeable.RECALC);
}

/**
 * Gets the chart area's <code>JCPlotCube</code> object.
 *
 * @return the <code>JCPlotCube</code> object referenced by this
 *     <code>JCChart3dArea</code>
 */
public JCPlotCube getPlotCube()
{
	return(plotCube);
}

/**
 * Sets the chart area's <code>JCPlotCube</code> object.
 *
 * @param plotCube the new <code>JCPlotCube</code> object
 */
public void setPlotCube(JCPlotCube plotCube)
{
	if (this.plotCube == plotCube) {
		return;
	}
	this.plotCube = plotCube;
	if (this.plotCube != null) {
		// Note that setChart3dArea calls plotCube's addParent method and also
		// calls setChanged
		this.plotCube.setChart3dArea(this);
	}
	else {
		throw new IllegalArgumentException(
            LocaleBundle.string(LocaleBundle.INVALID_VALUE_NULL_ERROR));
	}
}

/**
 * Gets the chart area's <code>JCSurface</code> object.
 *
 * @return the <code>JCSurface</code> object referenced by this
 *     <code>JCChart3dArea</code>
 */
public JCSurface getSurface()
{
	return(surface);
}

/**
 * Sets the chart area's <code>JCSurface</code> object.
 *
 * @param surface the new chart area surface object
 */
public void setSurface(JCSurface surface)
{
	if (this.surface == surface) {
		return;
	}
    this.surface = surface;
	if (this.surface != null) {
		this.surface.addParent(this);
	}
	else {
		throw new IllegalArgumentException(
            LocaleBundle.string(LocaleBundle.INVALID_VALUE_NULL_ERROR));
	}
	setChanged(true, Changeable.RECALC);
}

/**
 * Gets the chart area's <code>JCBar</code> object.
 *
 * @return the <code>JCBar</code> object referenced by this
 *     <code>JCChart3dArea</code>
 */
public JCBar getBar()
{
	return(bar);
}

/**
 * Sets the chart area's <code>JCBar</code> object.
 *
 * @param bar the new chart area bar object
 */
public void setBar(JCBar bar)
{
	if (this.bar == bar) {
		return;
	}
    this.bar = bar;
	if (this.bar != null) {
		this.bar.addParent(this);
	}
	else {
		throw new IllegalArgumentException(
            LocaleBundle.string(LocaleBundle.INVALID_VALUE_NULL_ERROR));
	}
	setChanged(true, Changeable.RECALC);
}

/**
 * Gets the chart area's <code>JCScatter</code> object.
 *
 * @return the <code>JCScatter</code> object referenced by this
 *     <code>JCChart3dArea</code>
 */
public JCScatter getScatter()
{
	return(scatter);
}

/**
 * Sets the chart area's <code>JCScatter</code> object.
 *
 * @param scatter the new chart area scatter object
 */
public void setScatter(JCScatter scatter)
{
	if (this.scatter == scatter) {
		return;
	}
    this.scatter = scatter;
	if (this.scatter != null) {
		this.scatter.addParent(this);
	}
	else {
		throw new IllegalArgumentException(
            LocaleBundle.string(LocaleBundle.INVALID_VALUE_NULL_ERROR));
	}
	setChanged(true, Changeable.RECALC);
}

/**
 * Gets the chart area's <code>Cube</code> object.
 *
 * @return the <code>Cube</code> object referenced by this
 *     <code>JCChart3dArea</code>
 */
public Cube getCube()
{
	return(cube);
}

/**
 * Sets the <code>Cube</code> object for this <code>chart3dArea</code>.
 * @param cube the new <code>Cube</code> object
 */
public void setCube(Cube cube)
{
    this.cube = cube;
	cube.setChart3dArea(this);
}

/**
 * Gets the chart area's <code>Transform</code> object.
 *
 * @return the <code>Transform</code> object referenced by this
 *     <code>JCChart3dArea</code>
 */
public Transform getTransform()
{
	return(transform);
}

/** Sets the <code>Transform</code> object for this <code>chart3dArea</code>.
 * @param transform the new transform object
 */
public void setTransform(Transform transform)
{
    this.transform = transform;
	transform.setChart3dArea(this);
}

/**
 * Gets the chart area's <code>Anno</code> object.
 *
 * @return the <code>Anno</code> object referenced by this
 *     <code>JCChart3dArea</code>
 */
public Anno getAnno()
{
	return(anno);
}

/** Sets the <code>Anno</code> object for this <code>chart3dArea</code>.
 * @param anno the new <code>Anno</code> object
 */
public void setAnno(Anno anno)
{
    this.anno = anno;
	anno.setChart3dArea(this);
}

// Changeable implementation begins here

/**
 * Gets the value of the <code>Changed</code> property.
 * The <code>Changed</code> property determines whether the object needs
 * to be recalculated.
 *
 * @return <code>true</code> if the object has been changed and needs
 *     recalculation; <code>false</code> otherwise
 */
public boolean getChanged()
{
	return(changed);
}

/**
 * Checks the value of the <code>ChangedFlag</code> property for the specified
 * mask (see <code>Changeable</code> for a list of valid masks).
 * The <code>ChangedFlag</code> property determines the action the object needs
 * to take in regards to the current change.
 *
 * @param mask the mask to check against
 * @return <code>true</code> if the specified mask exists in the
 *     <code>ChangedFlag</code> property; <code>false</code> otherwise
 */
public boolean getChanged(int mask)
{
	return((changedFlag & mask) == mask);
}

/**
 * Gets the value of the <code>Changed</code> property.
 * The <code>Changed</code> property determines whether the object needs
 * to be recalculated.
 *
 * @return <code>true</code> if the object has been changed and needs
 *     recalculation; <code>false</code> otherwise
 */
final public boolean isChanged()
{
	return(getChanged());
}

/**
 * Checks the value of the <code>ChangedFlag</code> property for the specified
 * mask (see <code>Changeable</code> for a list of valid masks).
 * The <code>ChangedFlag</code> property determines the action the object needs
 * to take in regards to the current change.
 *
 * @param mask the mask to check against
 * @return <code>true</code> if the specified mask exists in the
 *     <code>ChangedFlag</code> property; <code>false</code> otherwise
 */
final public boolean isChanged(int mask)
{
	return(getChanged(mask));
}

/**
 * Sets the value of the <code>Changed</code> property and updates the
 * parent object.
 * The <code>Changed</code> property determines whether the object needs
 * to be recalculated.
 *
 * @param n new value for <code>Changed</code>
 * @param cf the change flag mask used to check which type of change is needed
 */
public void setChanged(boolean n, int cf)
{
    if (chart3d == null) {
		return;
    }

	Chart3dDataView dataView = chart3d.getDataView(0);
	if (n) {
		dataView.setChanged(true, cf, false);
	}
	else {
		chart3d.getDataView(0).setChanged(false, Changeable.NO_CHANGE, false);
	}

	changed = n;
	if (changed) {
		changedFlag |= cf;
	}
	else {
		changedFlag = cf;
	}

    if (changed) {
        if (!chart3d.isBatched()) {
        	JCLegend legend = chart3d.getLegend();
        	if (legend != null && (cf & Changeable.NOTIFY_LEGEND_RECALC) > 0) {
            	legend.setChanged(true, Changeable.RECALC, false);
				legend.recalc();
        	}
        	if (legend != null && (cf & Changeable.NOTIFY_LEGEND_REDRAW) > 0) {
            	legend.setChanged(true, Changeable.REDRAW, false);
        	}

			if ((cf & Changeable.RECALC) > 0) {
				//recalc();  Currently there is no difference with a REDRAW
				chart3d.redraw();
			}
			else if ((cf & Changeable.REDRAW) > 0) {
				chart3d.redraw();
			}
        }
        return;
    }
}

/**
 * Sets the value of the <code>Changed</code> property.
 * The <code>Changed</code> property determines whether the object needs
 * to be recalculated.
 *
 * @param n new value for <code>Changed</code>
 * @param cf the change flag mask used to check which type of change is needed
 * @param updateParent if <code>true</code>, the parent is updated; otherwise,
 *     the parent is not updated (the "update" semantics are up to the
 *     implementor)
 */
public void setChanged(boolean n, int cf, boolean updateParent)
{
	if (updateParent) {
		setChanged(n, cf);
	}
	else {
		changed = n;
		if (n) {
			changedFlag |= cf;
		}
		else {
			changedFlag = cf;
		}
	}
}

/**
 * Gets the action handler for this chart 3D area. The action handler handles
 * the interactions for this chart area.
 *
 * @return the current action handler that is in use; null if none is in use
 */
public JCAction getActionHandler()
{
	return(actionHandler);
}

/**
 * Sets the action handler for this chart 3D area. The action handler handles
 * the interactions for this chart area.
 *
 * @param handler the new action handler to use
 */
protected void setActionHandler(JCAction handler)
{
	this.actionHandler = handler;
}

/**
 * Gets the value of the <code>InAction</code> property, which is a flag that
 * determines whether an action is in process.
 *
 * @return whether we are in an action
 */
public boolean isInAction()
{
	return(inAction);
}

/**
 * Sets the value of the <code>InAction</code> property, which is a flag that
 * determines whether an action is in process.
 *
 * @param ia Are we in an action?
 */
public void setInAction(boolean ia)
{
	inAction = ia;
}

/**
 * Fills the chart area will its background color.
 *
 * @param gc the graphics object to use for filling
 */
protected void fillWithBackgroundColor(Graphics gc)
{
	if (!gc.getColor().equals(getBackground())) {
		gc.setColor(getBackground());
	}
	Rectangle r = getBounds();
	gc.fillRect(0, 0, r.width, r.height);
}

/**
 * Renders the <code>JCChart3dArea</code> contents.
 *
 * @param gc graphics context used for drawing
 */
public void paintComponent(Graphics gc)
{
	if (getParent() == null ||
        chart3d == null     ||
        gc == null          ||
        !isVisible())
    {
		return;
	}


 

	try {
	    Chart3dDataView dataView = chart3d.getDataView(0);
	    dataView.calcDefaults();
        if (transform != null) {
            transform.computeTransformations(gc);
        }
	    dataView.getContour().recalcContours();

	    // Draw own opaque backgrounds.  Shouldn't this be
	    // done by the superclass?  That code appears to be in the UI, which
	    // is null for this component it seems.
	    if (isOpaque()) {
		    fillWithBackgroundColor(gc);
	    }

	    dataView.drawGraph(gc);


 

 

 

    }
	catch(Exception e) {
		e.printStackTrace(System.out);
	}

	rbGC = gc;
	if (actionHandler != null) {
		actionHandler.reanimate(last.x, last.y);
	}
	rbGC = null;

    setChanged(false, Changeable.NO_CHANGE);
}


 

/** Updates our parent with the changed flag mask information, and possibly
 *  repaints.
 * @param cf the change flag mask used to check which type of change is needed
 */
public void updateParent(int cf)
{
	JCChart3d p = (JCChart3d) getParent();
	if (p != null) {
		if (((cf & Changeable.RECALC) > 0) ||
			((cf & Changeable.NOTIFY_PARENT_LAYOUT) > 0))
		{
			p.invalidate();
			if (!p.isBatched()) {
				p.validate();
				// This repaint may be redundant in some cases.
				p.repaint();
			}
		}
		else if ((cf & Changeable.REDRAW) > 0) {
			if (!p.isBatched()) {
				p.repaint();
			}
		}
	}
}

/**
 * If necessary, forces recalculation of the chart area.
 */
public void recalc()
{
	chart3d.getDataView(0).calcDefaults();
}

/** The point used during last <code>animateAction</code> call. */
protected Point last = new Point(-1, -1);

/**
 * Calls the handler for the current action, if the current point is different
 * from the last point. The assumption is that the action has been started
 * already.
 *
 * @param event the event for the action
 * @param x the <i>X</i> position of the current action
 * @param y the <i>Y</i> position of the current action
 */
protected void animateAction(InputEvent event, int x, int y)
{
    if (actionHandler == null) {
        return;
    }
    if (last.x == x && last.y == y) {
        return;
    }
    last.x = x;
    last.y = y;
    actionHandler.animate(event, x, y);
}

/**
 * Calls the action handler to end the current action.
 * @param event the event for the action
 * @param x the <I>X</i> position of the current action
 * @param y the <I>Y</i> position of the current action
 */
protected void endAction(InputEvent event, int x, int y)
{
    if (actionHandler == null) {
        return;
    }
    JCAction tmp = actionHandler;
    actionHandler = null;
    last.x = x;
    last.y = y;
    tmp.end(event, x, y);
}

/**
 * Starts the action stored in the <code>actionHandler</code>, if any.
 * @param event the event for the action
 * @param x the <I>X</i> position of the action
 * @param y the <I>Y</i> position of the action
 */
protected void startAction(InputEvent event, int x, int y)
{
    if (actionHandler == null) {
        return;
    }
    actionHandler.start(event, x, y);
}

/**
 * Draws the preview cube for this <code>chart3dArea</code>.
 *
 * @param scale the scale of the preview cube
 * @param horizShift the horizontal shift of the preview cube
 * @param vertShift the vertical shift of the preview cube
 * @param xRotation the xRotation angle of the preview cube
 * @param yRotation the yRotation angle of the preview cube
 * @param zRotation the zRotation angle of the preview cube
 * @param axisId this determines the type of previewing
 * @param x the current <I>X</i> position of the mouse (needed for EYE
 *     previewing)
 * @param y the current <I>Y</i> position of the mouse (needed for EYE
 *     previewing)
 */
public void drawPreview(double scale, double horizShift, double vertShift,
			            double xRotation, double yRotation, double zRotation,
                        int axisId, int x, int y)
{
	Graphics gc = getGraphics();

	viewport.scale           = scale;
	viewport.horizontalShift = horizShift;
	viewport.verticalShift   = vertShift;
	viewport.normalized      = (scale      == 1.0 &&
							    horizShift == 0.0 &&
								vertShift  == 0.0);

	view3d.xRotation = xRotation;
	view3d.yRotation = yRotation;
	view3d.zRotation = zRotation;
	transform.computeTransformations(gc);

	fillWithBackgroundColor(gc);

	if (getChart3d().getDataView(0).getElevationData().isDataOK()) {
	    plotCube.updateBoundingBox(gc, axisId, x, y);
	}
}

/**
 * Package-internal method that draws a rubber band rectangle from the
 * point in "start" to the provided point.
 *
 * @param p coordinates to use as the end point for the rectangle
 * @param start starting point for zoom rectangle
 */
public void drawZoomRectangle(Point p, Point start) {
    if (p == null) {
		return;
    }
    if (p.x == start.x || p.y == start.y) {
		return;
    }
    int ulx = start.x;
    int uly = start.y;
    int lrx = p.x;
    int lry = p.y;
    if (rbGC == null) {
        if (getParent() == null) {
			return;
        }
    }
    Graphics gc = (rbGC == null) ? getGraphics().create() : rbGC.create();
    if (lrx - ulx < 0) {
        // Swap x values
        int tempx = lrx;
        lrx = ulx;
        ulx = tempx;
    }
    if (lry - uly < 0) {
        // Swap y values
        int tempy = lry;
        lry = uly;
        uly = tempy;
    }
    gc.setXORMode(Color.white);
    gc.drawRect(ulx, uly, lrx-ulx, lry-uly);
    gc.setXORMode(Color.black);
    gc.drawRect(ulx+1, uly+1, lrx-ulx, lry-uly);
	gc.dispose();
}

/**
 * Returns the chart back to the default viewport settings.
 */
public void reset()
{
    if (chart3d == null) {
        return;
    }
    boolean batched = chart3d.isBatched();
    chart3d.setBatched(true);
    viewport.setNormalized(true);
    actionHandler = null;
    inAction      = false;
    setChanged(true, Changeable.RECALC);
    chart3d.setBatched(batched);
}

/**
 * Stops any current action.
 */
public void cancel()
{
	actionHandler.cancel();
    actionHandler = null;
    inAction      = false;
    setChanged(true, Changeable.RECALC);
}

/**
 * Returns the value of the batched property.
 * @return this method will always return <code>false</code>
 */
public boolean isBatched()
{
	return(false);
}

/**
 * Package-internal pick method.
 * Chooses the point closest to the provided position.  If
 * a data view is supplied, picks a point on the provided
 * data view only.  Otherwise, it tries to find the closest point
 * from any data view (currently only one data view is supported).
 *
 * @param p pick point in pixels relative to <code>JCChart3dArea</code> object
 * @param dataView Data view on which to perform pick.  If null, all data
 *     views are used (currently only one data view is supported).
 * @return <code>JCData3dIndex</code> object representing the picked point
 */
protected JCData3dIndex pick(Point p, Chart3dDataView dataView)
{
    JCData3dIndex index = null;
	if (p == null || chart3d == null) {
		return(index);
	}

	// Since the currently is only one dataView, use it.
	if (dataView == null) {
		dataView = chart3d.getDataView(0);
	}

	// First check to see if a label was selected
	JCValueLabel vLabel = anno.pickLabel(getGraphics(), p);
	if (vLabel != null) {
		return(new JCData3dLabelIndex(vLabel));
	}

	// Make sure that the dataView has a draw object
	dataView.createDraw();
	index = dataView.getDraw().getMapping().pick(p.x, p.y);
	if (index != null) {
		index.setObject(this);
		index.setDataView(dataView);
	}
	return(index);
}

/**
 * Overrides write object to change the state of the changed flags being
 * saved. This ensures that the chart will actually recalculate and draw when
 * re-serialized.
 *
 * @exception IOException
 * @param out
 */
private void writeObject(java.io.ObjectOutputStream out) throws IOException
{
  // Save current state first
  boolean    oldChanged     = changed;
  int        oldChangedFlag = changedFlag;

  // Save state that will generate a recalc
  changed     = true;
  changedFlag = Changeable.RECALC;
  out.defaultWriteObject();

  // Restore current state
  changed     = oldChanged;
  changedFlag = oldChangedFlag;
}

/**
 * Overrides parent <code>getGraphics()</code>. If another
 * <code>Graphics</code> object has been set to be the current one, it is
 * returned; otherwise, the superclass's <code>getGraphics()</code> method is
 * called.
 * @return the <code>Graphics</code> object to use to draw this chart area
 */
public Graphics getGraphics()
{
    if (prevSetGraphics == null) {
        return(super.getGraphics());
    }
    return(prevSetGraphics);
}

/**
 * Provides a way for the chart area to draw to a custom <code>Graphics</code>
 * object.  Call with a null to specify use of the default <code>Graphics</code>
 * object used to draw to the screen.
 * @param g <code>Graphics</code> object to use for drawing
 */
public void setGraphics(Graphics g)
{
    prevSetGraphics = g;
}

}
