/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCChart3dCustomizerPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import com.klg.jclass.chart3d.resources.LocaleBundle;

import javax.swing.*;
import java.awt.*;

/**
 * Base class for Property Pages. This class is used internally by JClass Chart
 * 3D and should not be used by applications.
 */
public abstract class JCChart3dCustomizerPage extends JPanel {

/** The target that this page is editing. */
protected Object target = null;

/**
 * Method that initializes the page.
 */
public void initialize() {}

/**
 * Method that returns the target that this page is editing.
 * @return the target
 */
public Object getObject()
{
    return(target);
}

/**
 * Method that updates the page with values from the given source.
 *
 * @param source the new target for editing
 */
public void setObject(Object source)
{
    target = source;
}

/**
 * Method that updates the page with values from the current target.
 */
public void setObject()
{
    setObject(getObject());
}

/**
 * Method to <code>show()</code> an Info Dialog containing important messages.
 *
 * @param msg the message to show
 */
public static void showError(String msg)
{
    ErrorDialog.raise(msg);
}

/**
 * Utility method that returns a frame for placing a propery page in.
 * Will be overridden in subclasses.
 * @return a frame for placing a property page in
 */
public JFrame getPropertyFrame()
{
    return(new JFrame());
}

/**
 * Utility method that shows the Customizer.
 *
 * @param p point at which to launch the customizer
 */
public void launch(Point p)
{
    JFrame frame = getFrame(this);
    if (frame == null) {
		try {
			frame = getPropertyFrame();
            frame.setTitle(
                LocaleBundle.string(LocaleBundle.CHART3D_PROPERTIES_LABEL));
			frame.getContentPane().setLayout(new java.awt.GridLayout(1, 1));
			frame.getContentPane().add(this);
			frame.pack();
			java.awt.Dimension bigness = this.getPreferredSize();
			// Customizer coming up too small
			// under Netscape and Internet Explorer.
			// Seems that getPreferredSize() is not reliable on
			// these platforms. Work around by detecting
			// a too-small 'bigness'.
			if (bigness.width < 400 || bigness.height < 200) {
				// hard-code 'optimal' size of JCChart3d Customizer
				frame.setSize(740, 440);
			}
			else {
			     // 'Nice' size allows for frame's menubar
				frame.setSize(bigness.width + 10, bigness.height + 50);
			}
		}
		catch (Throwable e) {
			showError(
                LocaleBundle.string(LocaleBundle.CUSTOMIZER_PAGE_LAUNCH_ERROR) +
                "\n" + e);
			frame = null;
		}
    }
    if (frame != null) {
		frame.show();
    }
}

/**
 * Gets the component's parent frame.
 *
 * @param comp the component whose parent frame should be returned
 * @return the parent frame
 */
public static JFrame getFrame(Component comp)
{
    Component tmp = comp, parent = comp;
    while(true) {
		if (parent == null || parent instanceof JFrame) {
	        break;
		}
		parent = tmp;
	    if (tmp == null) {
	        return(null);
	    }
	    tmp = tmp.getParent();
    }
    return((JFrame) parent);
}

}


