/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCChart3dLegendTemplate.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.
package com.klg.jclass.chart3d;

import java.util.List;

/** 
 * Implemented by the 3D chart's legend class.  Any subclass of <code>JCLegend
 * </code>
 * that correctly implements this interface can be used.  The default
 * implementation is the <code>JCChart3dLegend</code> class.
 * 
 */
public interface JCChart3dLegendTemplate
{

// layout style enums
/** Legend Style -- Continuous.
 * 
 */
public final static int  CONTINUOUS = 0;
/** Legend Style -- Stepped.
 * 
 */
public final static int  STEPPED = 1;

// distributon range enums
/** Distribution Range -- all ranges.
 * 
 */
public final static int  RANGE_ALL = 0;
/** Distribution Range -- constrain to data.
 * 
 */
public final static int  RANGE_DATA = 1;

/** Gets the list of user specified label lists used to override
 * default label generation.  The value is null if no lists have been
 * specified.  The returned List instance is indexed by data view
 * number.  Each item in the returned List is itself an List
 * instance representing the list of legend labels (either String or
 * <code>JCMultiFieldString</code> objects) to be used for the data view.;
 * 
 * @return the list of user specified label lists
 */
public List getLabels();

/** Gets the class specifying custom labels to be used in the
 * legend.  A null value indicates that another method of setting
 * legend labels is being used.
 * 
 * @return a class implementing the <code>JCChart3dLegendLabelGenerator</code>
 *      interface that is being used to specify legend labels
 */
public JCChart3dLegendLabelGenerator getLabelGenerator();

/** 
 * Gets the style being used for displaying data ranges in the legend.  One
 * of <code>CONTINUOUS</code> or <code>STEPPED</code>.
 * 
 * @return the current range style
 */
public int getLayoutStyle();

/** 
 * Gets the type of distribution range being used to display data in the legend.
 * One of <code>RANGE_DATA</code> or <code>RANGE_ALL</code>.
 * <code>RANGE_DATA</code>  constrains the legend data
 * ranges to the min and max of the data in the chart.
 * <code>RANGE_ALL</code> ensures that all data ranges are included.
 */
public int getDistributionRange();

/** Gets the gap between the legend items depending on the orientation.
 * 
 * @return the item gap
 */
public int getItemGap();

/** 
 * Called by legend manager to notify legend that CONTINUOUS layout
 * is actually continuous.  (In some cases, CONTINUOUS layout may be
 * forced to a STEPPED layout.)
 * 
 * @param continuous 
 */
public void setContinuousLayout(boolean continuous);

/** 
 * Returns true if layout is truly continuous.  False otherwise.
 * (In some cases a CONTINUOUS layout may not actually be continuous, but
 * may behave like a STEPPED layout instead.)
 * 
 */
public boolean isContinuousLayout();

}
