/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCChart3dStyle.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import java.awt.Color;
import java.awt.Image;
import java.util.Vector;

/**
 * For point data, <code>JChart3dStyle</code> represents a style for rendering a
 * series or points. It controls symbol styles and line styles.
 * <P><STRONG><CENTER>Properties</CENTER></STRONG></P>
 * <P></P>
 * <TABLE BORDER=1 WIDTH=100%>
 * <TR>
 * <TH>Name</TH>
 * <TH>Description</TH></TR>
 * <TR>
 * <TD><code>LineColor</code>
 * </TD>
 * <TD>
 * The <code>LineColor</code> property determines the color used to draw a line. Default value is generated.
 * </TD></TR>
 * <TR>
 * <TD><code>LinePattern</code>
 * </TD>
 * <TD>
 * The <code>LinePattern</code> property dictates the pattern used to draw a line.
 * Valid values include <code>JCLineStyle.NONE</code>,
 * <code>JCLineStyle.SOLID</code>, <code>JCLineStyle.LONG_DASH</code>,
 * <code>JCLineStyle.SHORT_DASH</code>, <code>JCLineStyle.LSL_DASH
 * </code>,
 * and <code>JCLineStyle.DASH_DOT</code>. <br><B>Note:</B> This property is
 * supported only in JDK 1.2 and higher. Default value is
 * <code>JCLineStyle.SOLID</code>.
 * </TD></TR>
 * <TR>
 * <TD><code>LineStyle</code>
 * </TD>
 * <TD>
 * The <code>LineStyle</code> property controls the appearance of lines in chart. See
 * <code>JCLineStyle</code> for additional properties.
 * </TD></TR>
 * <TR>
 * <TD><code>LineWidth</code>
 * </TD>
 * <TD>
 * The <code>LineWidth</code> property controls the line width.
 * Default value is 1.
 * </TD></TR>
 * <TR>
 * <TD><code>SymbolColor</code>
 * </TD>
 * <TD>The <code>SymbolColor</code> property determines the color used to paint
 * the symbol. Default value is generated.
 * </TD></TR>
 * <TR>
 * <TD>Custom Shapes
 * </TD>
 * <TD>
 * Custom shapes can made by creating an object that implements the
 * <code>JCSymbolShape</code> interface. The new shape can be used by using
 * the <code>setSymbolShape()</code> method with the filename of the class. See
 * <code>com.klg.jclass.chart3d.shape.JCSymbolShape</code> for details.
 * </TD></TR>
 * <TR>
 * <TD><code>SymbolShape</code>
 * </TD>
 * <TD>
 * The <code>SymbolShape</code> property determines the type of symbol that
 * will be drawn.  Valid values include <code>JCSymbolStyle.DOT</code>,
 * <code>JCSymbolStyle.BOX</code>, <code>JCSymbolStyle.TRIANGLE</code>,
 * <code>JCSymbolStyle.DIAMOND</code>, <code>JCSymbolStyle.STAR</code>,
 * <code>JCSymbolStyle.VERT_LINE</code>, <code>JCSymbolStyle.HORIZ_LINE</code>,
 * <code>JCSymbolStyle.CROSS</code>, <code>JCSymbolStyle.CIRCLE</code>, and
 * <code>JCSymbolStyle.SQUARE</code>.
 * Default value is generated.
 * </TD></TR>
 * <TR>
 * <TD><code>SymbolSize</code>
 * </TD>
 * <TD>
 * The <code>SymbolSize</code> property determines the size of the symbol.
 * Default value is 6.
 * </TD></TR>
 * <TR>
 * <TD><code>SymbolStyle</code>
 * </TD>
 * <TD>
 * The <code>SymbolStyle</code> property controls the symbol that represents
 * an individual point.  See <code>JCSymbolStyle</code> for additional properties.
 * <br><B>Note:</b> All <code>JCChart3dStyle</code> properties of the format <i>Symbol*</i> are
 * virtual properties that map to properties of <code>JCSymbolStyle</code>.
 * </TD></TR>
 * </TABLE>
 * <P>
 *
 * @see com.klg.jclass.chart3d.JCFillStyle
 * @see com.klg.jclass.chart3d.JCLineStyle
 * @see com.klg.jclass.chart3d.JCSymbolStyle
 */
public class JCChart3dStyle extends ParentTrackChange {

/** The line style to be used for this chart style. */
protected JCLineStyle     lineStyle   = null;

/** The symbol style to be used for this chart style. */
protected JCSymbolStyle   symbolStyle = null;

/**
 * Constructor for chart style objects.
 *
 * @param lineStyle <code>JCLineStyle</code> instance representing the line
 *     style for this chart style object
 * @param symbolStyle <code>JCSymbolStyle</code> instance representing the
 *     symbol style for this chart style object
 * @see com.klg.jclass.chart3d.JCLineStyle
 * @see com.klg.jclass.chart3d.JCSymbolStyle
 */
public JCChart3dStyle(JCLineStyle lineStyle, JCSymbolStyle symbolStyle)
{
	this.lineStyle          = lineStyle;
	this.symbolStyle        = symbolStyle;
	this.lineStyle.parent   = this;
	this.symbolStyle.parent = this;
}

/**
 * Default constructor for chart style objects.  Creates default
 * line style and symbol style objects.
 *
 */
public JCChart3dStyle()
{
	lineStyle          = JCLineStyle.makeDefault(null);
	symbolStyle        = JCSymbolStyle.makeDefault(null);
	lineStyle.parent   = this;
	symbolStyle.parent = this;
}

/**
 * Creates a default chart style.
 *
 * @param c <code>JCChart3d</code> instance that is the parent of the created
 *     <code>JCChart3dStyle</code> object
 * @return <code>JCChart3dStyle</code> object with default line, fill and
 *     symbol styles.  The defaults are cycled, so consecutive calls will return
 *      different chart styles.
 */
public static JCChart3dStyle makeDefault(JCChart3d c)
{
	return(makeDefault(c, null));
}

/**
 * Creates a default chart style.
 *
 * @param c <code>JCChart3d</code> instance that this
 *     <code>JCChart3dStyle</code> will use
 * @param parent an instance of <code>Changeable</code> that will be the
 *     parent of the created <code>JCChart3dStyle</code> object
 * @return <code>JCChart3dStyle</code> object with default line, fill and
 *     symbol styles.  The defaults are cycled, so consecutive calls will return
 *      different chart styles.
 */
public static JCChart3dStyle makeDefault(JCChart3d c, Changeable parent)
{
	JCChart3dStyle r = new JCChart3dStyle(JCLineStyle.makeDefault(c),
									      JCSymbolStyle.makeDefault(c));
	if (parent != null) {
		r.addParent(parent);
	}
	else if (c != null) {
		r.addParent(c.getDataView(0));
	}
	return(r);
}

/**
 * Gets the <code>JCLineStyle</code> property, which controls how lines are
 * drawn.
 *
 * @return <code>JCLineStyle</code> instance representing the line drawing
 *     style
 */
public JCLineStyle getLineStyle()
{
    return(lineStyle);
}

/**
 * Sets the <code>JCLineStyle</code> property, which controls how lines are
 * drawn.
 *
 * @param lineStyle
 */
public void setLineStyle(JCLineStyle lineStyle) {
	if (this.lineStyle == lineStyle) {
		return;
	}
	this.lineStyle        = lineStyle;
	this.lineStyle.parent = this;

	callParentSetChanged(true,
						 Changeable.REDRAW | Changeable.NOTIFY_LEGEND_REDRAW);
}

/**
 * Gets the value of the <code>LinePattern</code> property.
 * The <code>LinePattern</code> property dictates the pattern used to draw a line.
 * Valid values include <code>JCLineStyle.NONE</code>, <code>JCLineStyle.SOLID</code>,
 * <code>JCLineStyle.LONG_DASH</code>, <code>JCLineStyle.SHORT_DASH</code>, <code>JCLineStyle.LSL_DASH
 * </code>, and <code>JCLineStyle.DASH_DOT</code>.
 * <I>This feature is supported only in JDK 1.2 and higher.</I>
 *
 * @return line pattern enum
 */
public int getLinePattern()
{
	if (lineStyle == null) {
		return(JCLineStyle.NONE);
	}
	return(lineStyle.getPattern());
}

/**
 * Sets the value of the <code>LinePattern</code> property.
 * The <code>LinePattern</code> property dictates the pattern used to draw a line.
 * Valid values include <code>JCLineStyle.NONE</code>,
 * <code>JCLineStyle.SOLID</code>, <code>JCLineStyle.LONG_DASH</code>,
 * <code>JCLineStyle.SHORT_DASH</code>, <code>JCLineStyle.LSL_DASH</code>,
 * and <code>JCLineStyle.DASH_DOT</code>.
 * <I>This feature is supported only in JDK 1.2 and higher.</I>
 *
 * @param p the new line pattern
 */
public void setLinePattern(int p)
{
	if (lineStyle == null) {
		lineStyle = JCLineStyle.makeDefault(null);
	}
	lineStyle.setPattern(p);
}

/**
 * Gets the value of the <code>LineJoin</code> property. The
 * <code>LineJoin</code> property dictates the join style used to join two lines.
 * Valid values include <code>BasicStroke.JOIN_MITER</code>,
 * <code>BasicStroke.JOIN_BEVEL</code>, and <code>BasicStroke.JOIN_ROUND</code>.
 * <I>This feature is supported only in JDK 1.2 and higher.</I>
 *
 * @return line pattern enum
 */
public int getLineJoin()
{
	if (lineStyle == null) {
		return(-1);
	}
	return(lineStyle.getJoin());
}

/**
 * Sets the value of the <code>LineJoin</code> property. The <code>LineJoin</code> property
 * dictates the join style used to join two lines. Valid values include
 * <code>BasicStroke.JOIN_MITER</code>, <code>BasicStroke.JOIN_BEVEL</code>,
 * and <code>BasicStroke.JOIN_ROUND</code>.
 * <I>This feature is supported only in JDK 1.2 and higher.</I>
 *
 * @param j join enum
 */
public void setLineJoin(int j)
{
	if (lineStyle == null) {
		lineStyle = JCLineStyle.makeDefault(null);
	}
	lineStyle.setJoin(j);
}

/**
 * Gets the value of the <code>LineCap</code> property.
 * The <code>LineCap</code> property dictates the cap style used to end a line.
 * Valid values include <code>BasicStroke.CAP_BUTT</code>,
 * <code>BasicStroke.CAP_ROUND</code>, and <code>BasicStroke.CAP_SQUARE</code>.
 * <I>This feature is supported only in JDK 1.2 and higher.</I>
 *
 * @return line pattern enum
 */
public int getLineCap()
{
	if (lineStyle == null) {
		return(-1);
	}
	return(lineStyle.getCap());
}

/**
 * Sets the value of the <code>LineCap</code> property.
 * The <code>LineCap</code> property dictates the cap style used to end a line.
 * Valid values include <code>BasicStroke.CAP_BUTT</code>,
 * <code>BasicStroke.CAP_ROUND</code>, and <code>BasicStroke.CAP_SQUARE</code>.
 * <I>This feature is supported only in JDK 1.2 and higher.</I>
 *
 * @param c cap enum
 */
public void setLineCap(int c)
{
	if (lineStyle == null) {
		lineStyle = JCLineStyle.makeDefault(null);
	}
	lineStyle.setCap(c);
}
/**
 * Gets the value of the <code>LineWidth</code> property.
 * The <code>LineWidth</code> property controls the line width.
 *
 * @return line width
 */
public int getLineWidth()
{
	if (lineStyle == null) {
		return(0);
	}
	return(lineStyle.getWidth());
}

/**
 * Sets the value of the <code>LineWidth</code> property.
 * The <code>LineWidth</code> property controls the line width.
 *
 * @param lw line width
 */
public void setLineWidth(int lw)
{
	if (lineStyle == null) {
		lineStyle = JCLineStyle.makeDefault(null);
	}
	lineStyle.setWidth(lw);
}

/**
 * Gets the value of the <code>LineColor</code> property.
 * The <code>LineColor</code> property determines the color used to draw a line.
 *
 * @return line color
 */
public Color getLineColor()
{
	if (lineStyle == null) {
		return(null);
	}
	return(lineStyle.getColor());
}

/**
 * Sets the value of the <code>LineColor</code> property.
 * The <code>LineColor</code> property determines the color used to draw a line.
 *
 * @param lc line color
 */
public void setLineColor(Color lc)
{
	if (lineStyle == null) {
		lineStyle = JCLineStyle.makeDefault(null);
	}
	lineStyle.setColor(lc);
}

/**
 * Gets the <code>SymbolStyle</code> property
 * The <code>SymbolStyle</code> property controls the symbol that represents
 * an individual point.
 *
 * @return <code>JCSymbolStyle</code> instance representing the symbol drawing
 *      style
 */
public JCSymbolStyle getSymbolStyle()
{
    return(symbolStyle);
}

/**
 * Gets the <code>SymbolStyle</code> property
 * The <code>SymbolStyle</code> property controls the symbol that represents
 * an individual point.
 *
 * @param symbolStyle
 */
public void setSymbolStyle(JCSymbolStyle symbolStyle)
{
	if (this.symbolStyle == symbolStyle) {
		return;
	}
	this.symbolStyle        = symbolStyle;
	this.symbolStyle.parent = this;

	callParentSetChanged(true,
			             Changeable.REDRAW | Changeable.NOTIFY_LEGEND_REDRAW);
}

/**
 * Gets the value of the <code>SymbolShape</code> property. The
 * <code>SymbolShape</code> property determines the type of symbol that will be
 * drawn.  Valid values include <code>JCSymbolStyle.NONE</code>,
 * <code>JCSymbolStyle.DOT</code>, <code>JCSymbolStyle.BOX</code>,
 * <code>JCSymbolStyle.TRIANGLE</code>, <code>JCSymbolStyle.DIAMOND</code>,
 * <code>JCSymbolStyle.STAR</code>, <code>JCSymbolStyle.VERT_LINE</code>,
 * <code>JCSymbolStyle.HORIZ_LINE</code>, <code>JCSymbolStyle.CROSS</code>,
 * <code>JCSymbolStyle.CIRCLE</code>, and <code>JCSymbolStyle.SQUARE</code>.
 *
 * @return symbol shape
 */
public String getSymbolShape()
{
	if (symbolStyle == null) {
		return(JCSymbolStyle.NONE);
	}
	return(symbolStyle.getShape());
}

/**
 * Sets the value of the <code>SymbolShape</code> property.
 * The <code>SymbolShape</code> property determines the type of symbol that will be
 * drawn.  Valid values include <code>JCSymbolStyle.NONE</code>,
 * <code>JCSymbolStyle.DOT</code>, <code>JCSymbolStyle.BOX</code>,
 * <code>JCSymbolStyle.TRIANGLE</code>, <code>JCSymbolStyle.DIAMOND</code>,
 * <code>JCSymbolStyle.STAR</code>, <code>JCSymbolStyle.VERT_LINE</code>,
 * <code>JCSymbolStyle.HORIZ_LINE</code>, <code>JCSymbolStyle.CROSS</code>,
 * <code>JCSymbolStyle.CIRCLE</code>, and <code>JCSymbolStyle.SQUARE</code>.
 *
 * @param shape symbol shape
 */
public void setSymbolShape(String shape)
{
	if (symbolStyle == null) {
		symbolStyle = JCSymbolStyle.makeDefault(null);
	}
	symbolStyle.setShape(shape);
}

/**
 * Gets the value of the <code>SymbolColor</code> property.  The
 * <code>SymbolColor</code> property determines the color used to paint the symbol.
 *
 * @return symbol color
 */
public Color getSymbolColor()
{
	if (symbolStyle == null) {
		return(null);
	}
	return(symbolStyle.getColor());
}

/**
 * Sets the value of the <code>SymbolColor</code> property.  The
 * <code>SymbolColor</code> property determines the color used to paint the symbol.
 *
 * @param sc symbol color
 */
public void setSymbolColor(Color sc)
{
	if (symbolStyle == null) {
		symbolStyle = JCSymbolStyle.makeDefault(null);
	}
	symbolStyle.setColor(sc);
}

/**
 * Gets the value of the <code>SymbolSize</code> property.
 * The <code>SymbolSize</code> property determines the size of the symbols.
 *
 * @return positive integer representing symbol size
 */
public int getSymbolSize()
{
	if (symbolStyle == null) {
		return(0);
	}
	return symbolStyle.getSize();
}

/**
 * Sets the value of the <code>SymbolSize</code> property.
 * The <code>SymbolSize</code> property determines the size of the symbols.
 * When set to 0, no symbol is drawn.
 *
 * @param sz symbol size
 */
public void setSymbolSize(int sz) {
	if (symbolStyle == null) {
		symbolStyle = JCSymbolStyle.makeDefault(null);
	}
	symbolStyle.setSize(sz);
}

}
