/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCChart3dUI.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import java.awt.Color;
import java.awt.Font;
import javax.swing.JComponent;
import javax.swing.UIManager;
import javax.swing.LookAndFeel;
import javax.swing.border.Border;
import javax.swing.border.BevelBorder;
import javax.swing.border.SoftBevelBorder;
import javax.swing.border.LineBorder;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.UIResource;

/** 
 * <code>JCChart3dUI</code> implements pluggable look and feel for <code>JCChart3d</code>.
 * To alleviate the necessity of writing separate UI classes for each possible UI,
 * and to allow chart and chart 3D to have a similar appearance to Swing,
 * this class handles the UI for all possible UI's.  It steals the background
 * and border UI attributes from <code>JPanel</code>, and the foreground and font attributes
 * from <code>JLabel</code>.  If a user has set these properties themselves, their values
 * take precedence.
 * 
 */
public class JCChart3dUI extends ComponentUI {

/**  */
protected static JCChart3dUI chart3dUI;

/**  * 
 * @param c 
 */
public void installUI(JComponent c)
{
	LookAndFeel.installColorsAndFont(c, "Panel.background",
									 "Label.foreground",
									 "Label.font");
	installDefaults(c);
}

/**  * 
 * @param c 
 */
protected void installDefaults(JComponent c)
{
	JCChart3d chart3d = (JCChart3d)c;

	// Only change properties if user hasn't set them yet.

	Color bgColor = chart3d.getBackground();
	if (bgColor == null || bgColor instanceof UIResource) {
		chart3d.setBackground(UIManager.getColor("Panel.background"));
	}

	Color fgColor = chart3d.getForeground();
	if (fgColor == null || fgColor instanceof UIResource) {
		chart3d.setForeground(UIManager.getColor("Label.foreground"));
	}

	Font chFont = chart3d.getFont();
	if (chFont == null || chFont instanceof UIResource) {
		chart3d.setFont(UIManager.getFont("Label.font"));
	}

	Border chBorder = chart3d.getBorder();
	if (chBorder == null || chBorder instanceof UIResource) {
		chart3d.setBorder(UIManager.getBorder("Panel.border"));
	}
}

/**  * 
 * @param c
 * @return
 */
public static ComponentUI createUI(JComponent c)
{
	if (chart3dUI == null) {
		chart3dUI = new JCChart3dUI();
	}
	return(chart3dUI);
}

/**  * 
 * @param c 
 */
public void uninstallUI(JComponent c)
{
}

}
