/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCContourStyle.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import java.awt.Color;
import java.awt.Image;

/**
 * Object defining the style used to draw a contour.
 */
public class JCContourStyle extends ParentTrackChange {

/** The line style to be used for this contour style. */
protected JCLineStyle    lineStyle   = null;

/** The fill style to be used for this contour style. */
protected JCFillStyle    fillStyle   = null;

/** The symbol style to be used for this contour style. */
protected JCSymbolStyle  symbolStyle = null;

/**
 * Default constructor for contour style objects.  Creates default
 * line style and fill style objects.
 */
public JCContourStyle()
{
    lineStyle          = new JCLineStyle(1, Color.black, JCLineStyle.SOLID);
    fillStyle          = new JCFillStyle(Color.gray, JCFillStyle.SOLID);
    symbolStyle        = new JCSymbolStyle(JCSymbolStyle.getDefaultShape(null),
                                           Color.orange, 6);
	lineStyle.parent   = this;
	fillStyle.parent   = this;
	symbolStyle.parent = this;
}

/**
 * Constructor for contour style objects.  Only the <code>fillColor</code> of
 * the <code>fillStyle</code> is specified.  All other values are at their
 * default values.
 *
 * @param fillColor the color of the <code>fillStyle</code>
 */
public JCContourStyle(Color fillColor)
{
    lineStyle          = new JCLineStyle(1, Color.black, JCLineStyle.SOLID);
    fillStyle          = new JCFillStyle(fillColor, JCFillStyle.SOLID);
    symbolStyle        = new JCSymbolStyle(JCSymbolStyle.getDefaultShape(null),
                                           fillColor, 6);
	lineStyle.parent   = this;
	fillStyle.parent   = this;
	symbolStyle.parent = this;
}

/**
 * Constructor for contour style objects.  The line width, color, and pattern
 * are specified, as well as the fill color and pattern.  The symbol style
 * is set to the default shape and size with the specified
 * <code>fillColor</code>.
 *
 * @param width the width of the line
 * @param lineColor the color of the line
 * @param linePattern the line pattern - see
 *      {@link com.klg.jclass.chart3d.JCLineStyle} for details
 * @param fillColor the color of the <code>fillStyle</code>
 * @param fillPattern the fill pattern - see
 *      {@link com.klg.jclass.chart3d.JCFillStyle} for details
 */
public JCContourStyle(int width, Color lineColor, int linePattern,
                      Color fillColor, int fillPattern)
{
    lineStyle          = new JCLineStyle(width, lineColor, linePattern);
    fillStyle          = new JCFillStyle(fillColor, fillPattern);
    symbolStyle        = new JCSymbolStyle(JCSymbolStyle.getDefaultShape(null),
                                           fillColor, 6);
	lineStyle.parent   = this;
	fillStyle.parent   = this;
	symbolStyle.parent = this;
}

/**
 * Constructor for contour style objects.  The line width, color, and pattern
 * are specified as well as the fill color and pattern.  The symbol style
 * is set to the default shape and size with the specified
 * <code>symbolColor</code>.
 *
 * @param width the width of the line
 * @param lineColor the color of the line
 * @param linePattern the line pattern - see
 *      {@link com.klg.jclass.chart3d.JCLineStyle} for details
 * @param fillColor the color of the <code>fillStyle</code>.
 * @param fillPattern the fill pattern - see
 *      {@link com.klg.jclass.chart3d.JCFillStyle} for details
 * @param shape the shape of the <code>symbolStyle</code> - see
 *     {@link com.klg.jclass.chart3d.JCSymbolStyle} for details
 * @param symbolColor the color of the symbol
 * @param symbolSize the size of the symbol
 */
public JCContourStyle(int width, Color lineColor, int linePattern,
                      Color fillColor, int fillPattern, String shape,
					  Color symbolColor, int symbolSize)
{
    lineStyle          = new JCLineStyle(width, lineColor, linePattern);
    fillStyle          = new JCFillStyle(fillColor, fillPattern);
    symbolStyle        = new JCSymbolStyle(shape, symbolColor, symbolSize);
	lineStyle.parent   = this;
	fillStyle.parent   = this;
	symbolStyle.parent = this;
}

/**
 * Constructor for contour style objects.
 *
 * @param lineStyle <code>JCLineStyle</code> instance representing the line
 *     style for this contour style object
 * @param fillStyle <code>JCFillStyle</code> instance representing the fill
 *     style for this contour style object
 * @param symbolStyle <code>JCSymbolStyle</code> instance representing the
 *     symbol style for this contour style object
 */
public JCContourStyle(JCLineStyle lineStyle, JCFillStyle fillStyle,
	                  JCSymbolStyle symbolStyle)
{
	this.lineStyle          = lineStyle;
	this.fillStyle          = fillStyle;
	this.symbolStyle        = symbolStyle;
	this.lineStyle.parent   = this;
	this.fillStyle.parent   = this;
	this.symbolStyle.parent = this;
}

/**
 * Creates a default chart style.
 *
 * @param c <code>JCChart3d</code> instance that is the parent of the created
 *     <code>JCContourStyle</code> object
 * @return <code>JCContourStyle</code> object with default line and fill
 *     styles. The defaults are cycled, so consecutive calls will return contour
 *      styles with different fill colors.
 */
public static JCContourStyle makeDefault(JCChart3d c)
{
	return(makeDefault(c, null));
}

/**
 * Creates a default contour style.
 *
 * @param c <code>JCChart3d</code> instance that uses this
 *     <code>JCContourStyle</code>
 * @param parent an instance of <code>Changeable</code> that will be the
 *     parent of the created <code>JCContourStyle</code> object
 * @return <code>JCContourStyle</code> object with default line and fill
 *     styles. The defaults are cycled, so consecutive calls will return contour
 *     styles with different fill colors.
 */
public static JCContourStyle makeDefault(JCChart3d c, Changeable parent)
{
	Color           color = JCStyle.getDefaultColor(c, JCStyle.FILL_STYLE);
	JCContourStyle  r     = new JCContourStyle(color);

	if (parent != null) {
		r.addParent(parent);
	}
	else if (c != null) {
		r.addParent(c.getDataView(0));
	}
	return(r);
}

/**
 * Gets the <code>JCLineStyle</code> property, which controls how contour lines
 * are drawn.
 *
 * @return <code>JCLineStyle</code> instance representing the contour line
 *     drawing style
 */
public JCLineStyle getLineStyle()
{
	return(lineStyle);
}

/**
 * Sets the <code>JCLineStyle</code> property, which controls how contour lines
 * are drawn.
 *
 * @param lineStyle <code>JCLineStyle</code> instance representing the contour
 *      line drawing style
 */
public void setLineStyle(JCLineStyle lineStyle)
{
	if (this.lineStyle == lineStyle) {
		return;
	}
	this.lineStyle        = lineStyle;
	this.lineStyle.parent = this;

	callParentSetChanged(true,
						 Changeable.REDRAW | Changeable.NOTIFY_LEGEND_RECALC);
}

/**
 * Gets the value of the <code>LinePattern</code> property.
 * The <code>LinePattern</code> property dictates the pattern used to draw a line.
 * Valid values include <code>JCLineStyle.NONE</code>,
 * <code>JCLineStyle.SOLID</code>, <code>JCLineStyle.LONG_DASH</code>,
 * <code>JCLineStyle.SHORT_DASH</code>, <code>JCLineStyle.LSL_DASH</code>, and
 * <code>JCLineStyle.DASH_DOT</code>.
 *
 * @return Line pattern enum
 */
public int getLinePattern()
{
	if (lineStyle == null) {
		return(JCLineStyle.NONE);
	}
	return(lineStyle.getPattern());
}

/**
 * Sets the value of the <code>LinePattern</code> property.
 * The <code>LinePattern</code> property dictates the pattern used to draw a line.
 * Valid values include <code>JCLineStyle.NONE</code>,
 * <code>JCLineStyle.SOLID</code>, <code>JCLineStyle.LONG_DASH</code>,
 * <code>JCLineStyle.SHORT_DASH</code>, <code>JCLineStyle.LSL_DASH </code>, and
 * <code>JCLineStyle.DASH_DOT</code>.
 *
 * @param pattern Line pattern enum
 */
public void setLinePattern(int pattern)
{
	if (lineStyle == null) {
		lineStyle = new JCLineStyle(1, Color.black, JCLineStyle.SOLID);;
	}
	lineStyle.setPattern(pattern);
}

/**
 * Gets the value of the <code>LineJoin</code> property. The
 * <code>LineJoin</code> property dictates the join style used to join two
 * lines.  Valid values include <code>BasicStroke.JOIN_MITER</code>,
 * <code>BasicStroke.JOIN_BEVEL</code>, and <code>BasicStroke.JOIN_ROUND</code>.
 *
 * @return Line join enum
 */
public int getLineJoin()
{
	if (lineStyle == null) {
		return(-1);
	}
	return(lineStyle.getJoin());
}

/**
 * Sets the value of the <code>LineJoin</code> property. The
 * <code>LineJoin</code> property dictates the join style used to join two
 * lines.  Valid values include <code>BasicStroke.JOIN_MITER</code>,
 * <code>BasicStroke.JOIN_BEVEL</code>, and <code>BasicStroke.JOIN_ROUND</code>.
 *
 * @param join Line join enum
 */
public void setLineJoin(int join)
{
	if (lineStyle == null) {
		lineStyle = new JCLineStyle(1, Color.black, JCLineStyle.SOLID);;
	}
	lineStyle.setJoin(join);
}

/**
 * Gets the value of the <code>LineCap</code> property. The <code>LineCap</code>
 * property dictates the cap style used to end a line. Valid values include
 * <code>BasicStroke.CAP_BUTT</code>, <code>BasicStroke.CAP_ROUND</code>, and
 * <code>BasicStroke.CAP_SQUARE</code>.
 *
 * @return Line cap enum
 */
public int getLineCap()
{
	if (lineStyle == null) {
		return(-1);
	}
	return(lineStyle.getCap());
}

/**
 * Sets the value of the <code>LineCap</code> property. The <code>LineCap</code>
 * property dictates the cap style used to end a line. Valid values include
 * <code>BasicStroke.CAP_BUTT</code>, <code>BasicStroke.CAP_ROUND</code>, and
 * <code>BasicStroke.CAP_SQUARE</code>.
 *
 * @param cap Line cap enum
 */
public void setLineCap(int cap)
{
	if (lineStyle == null) {
		lineStyle = new JCLineStyle(1, Color.black, JCLineStyle.SOLID);;
	}
	lineStyle.setCap(cap);
}
/**
 * Gets the value of the <code>LineWidth</code> property.
 * The <code>LineWidth</code> property controls the line width.
 *
 * @return Line width
 */
public int getLineWidth()
{
	if (lineStyle == null) {
		return(0);
	}
	return(lineStyle.getWidth());
}

/**
 * Sets the value of the <code>LineWidth</code> property.
 * The <code>LineWidth</code> property controls the line width.
 *
 * @param lineWidth Line width
 */
public void setLineWidth(int lineWidth)
{
	if (lineStyle == null) {
		lineStyle = new JCLineStyle(1, Color.black, JCLineStyle.SOLID);;
	}
	lineStyle.setWidth(lineWidth);
}

/**
 * Gets the value of the <code>LineColor</code> property.
 * The <code>LineColor</code> property determines the color used to draw a line.
 *
 * @return Line color
 */
public Color getLineColor()
{
	if (lineStyle == null) {
		return(null);
	}
	return(lineStyle.getColor());
}

/**
 * Sets the value of the <code>LineColor</code> property.
 * The <code>LineColor</code> property determines the color used to draw a line.
 *
 * @param lineColor Line color
 */
public void setLineColor(Color lineColor)
{
	if (lineStyle == null) {
		lineStyle = new JCLineStyle(1, Color.black, JCLineStyle.SOLID);;
	}
	lineStyle.setColor(lineColor);
}

/**
 * Gets the <code>JCFillStyle</code> property, which controls how contour zones
 * are filled.
 *
 * @return <code>JCFillStyle</code> instance representing the contour zone
 *     fill style
 */
public JCFillStyle getFillStyle()
{
	return(fillStyle);
}

/**
 * Sets the <code>JCFillStyle</code> property, which controls how contour zones
 * are filled.
 *
 * @param fillStyle <code>JCFillStyle</code> instance representing the contour
 *      zone fill style
 */
public void setFillStyle(JCFillStyle fillStyle)
{
	if (this.fillStyle == fillStyle) {
		return;
	}
	this.fillStyle        = fillStyle;
	this.fillStyle.parent = this;

	callParentSetChanged(true,
						 Changeable.REDRAW | Changeable.NOTIFY_LEGEND_RECALC);
}

/**
 * Gets the value of the <code>FillColor</code> property. The
 * <code>FillColor</code> property determines the color used to fill regions
 * in the chart.
 *
 * @return Fill color
 */
public Color getFillColor()
{
	if (fillStyle == null) {
		return null;
	}
	return(fillStyle.getColor());
}

/**
 * Sets the value of the <code>FillColor</code> property. The
 * <code>FillColor</code> property determines the color used to fill regions
 * in the chart.
 *
 * @param fillColor Fill color
 */
public void setFillColor(Color fillColor)
{
	if (fillStyle == null) {
		fillStyle = new JCFillStyle(Color.gray, JCFillStyle.SOLID);
	}
	fillStyle.setColor(fillColor);
}

/**
 * Gets the value of the <code>FillBackground</code> property. The
 * <code>FillBackground</code> property determines the color used for the
 * background when doing patterned fills in the chart.
 *
 * @return Fill background color
 */
public Color getFillBackground()
{
	if (fillStyle == null) {
		return(null);
	}
	return(fillStyle.getBackground());
}

/**
 * Sets the value of the <code>FillBackground</code> property. The
 * <code>FillBackground</code> property determines the color used for the
 * background when doing patterned fills in the chart.
 *
 * @param fillColor Fill background color
 */
public void setFillBackground(Color fillColor)
{
	if (fillStyle == null) {
		fillStyle = new JCFillStyle(Color.gray, JCFillStyle.SOLID);
	}
	fillStyle.setBackground(fillColor);
}

/**
 * Gets the value of the <code>FillPattern</code> property. The
 * <code>FillPattern</code> property determines the fill pattern used to fill
 * regions in chart.
 *
 * @return Fill pattern
 */
public int getFillPattern()
{
	if (fillStyle == null) {
		return(0);
	}
	return(fillStyle.getPattern());
}

/**
 * Sets the value of the <code>FillPattern</code> property. The
 * <code>FillPattern</code> property determines the fill pattern used to fill
 * regions in chart.
 *
 * @param fillPattern Fill pattern
 */
public void setFillPattern(int fillPattern)
{
	if (fillStyle == null) {
		fillStyle = new JCFillStyle(Color.gray, JCFillStyle.SOLID);
	}
	fillStyle.setPattern(fillPattern);
}

/**
 * Gets the value of the <code>FillImage</code> property. The
 * <code>FillImage</code> property is not currently used in
 * <code>JCChart3d</code>.
 *
 * @return Fill image
 */
public Image getFillImage()
{
	if (fillStyle == null) {
		return(null);
	}
	return(fillStyle.getImage());
}

/**
 * Sets the value of the <code>FillImage</code> property. The
 * <code>FillImage</code> property is not currently used in
 * <code>JCChart3d</code>.
 *
 * @param fillImage fill image
 */
public void setFillImage(Image fillImage)
{
	if (fillStyle == null) {
		fillStyle = new JCFillStyle(Color.gray, JCFillStyle.SOLID);
	}
	fillStyle.setImage(fillImage);
}

/**
 * Gets the <code>SymbolStyle</code> property. The <code>SymbolStyle</code>
 * property controls the symbol that represents an individual point.
 *
 * @return <code>JCSymbolStyle</code> instance representing the symbol drawing
 *      style
 */
public JCSymbolStyle getSymbolStyle()
{
    return(symbolStyle);
}

/**
 * Sets the <code>SymbolStyle</code> property. The <code>SymbolStyle</code>
 * property controls the symbol that represents an individual point.
 *
 * @param symbolStyle The current symbol style object
 */
public void setSymbolStyle(JCSymbolStyle symbolStyle)
{
	if (this.symbolStyle == symbolStyle) {
		return;
	}
	this.symbolStyle        = symbolStyle;
	this.symbolStyle.parent = this;

	callParentSetChanged(true,
			             Changeable.REDRAW | Changeable.NOTIFY_LEGEND_REDRAW);
}

/**
 * Gets the value of the <code>SymbolShape</code> property. The
 * <code>SymbolShape</code> property determines the type of symbol that will be
 * drawn.  Valid values include <code>JCSymbolStyle.NONE</code>,
 * <code>JCSymbolStyle.DOT</code>, <code>JCSymbolStyle.BOX</code>,
 * <code>JCSymbolStyle.TRIANGLE</code>, <code>JCSymbolStyle.DIAMOND</code>,
 * <code>JCSymbolStyle.STAR</code>, <code>JCSymbolStyle.VERT_LINE</code>,
 * <code>JCSymbolStyle.HORIZ_LINE</code>, <code>JCSymbolStyle.CROSS</code>,
 * <code>JCSymbolStyle.CIRCLE</code>, and <code>JCSymbolStyle.SQUARE</code>.
 *
 * @return Symbol shape
 */
public String getSymbolShape()
{
	if (symbolStyle == null) {
		return(JCSymbolStyle.NONE);
	}
	return(symbolStyle.getShape());
}

/**
 * Sets the value of the <code>SymbolShape</code> property. The
 * <code>SymbolShape</code> property determines the type of symbol that will be
 * drawn.  Valid values include <code>JCSymbolStyle.NONE</code>,
 * <code>JCSymbolStyle.DOT</code>, <code>JCSymbolStyle.BOX</code>,
 * <code>JCSymbolStyle.TRIANGLE</code>, <code>JCSymbolStyle.DIAMOND</code>,
 * <code>JCSymbolStyle.STAR</code>, <code>JCSymbolStyle.VERT_LINE</code>,
 * <code>JCSymbolStyle.HORIZ_LINE</code>, <code>JCSymbolStyle.CROSS</code>,
 * <code>JCSymbolStyle.CIRCLE</code>, and <code>JCSymbolStyle.SQUARE</code>.
 *
 * @param shape Symbol shape
 */
public void setSymbolShape(String shape)
{
	if (symbolStyle == null) {
		symbolStyle = JCSymbolStyle.makeDefault(null);
	}
	symbolStyle.setShape(shape);
}

/**
 * Gets the value of the <code>SymbolColor</code> property.  The
 * <code>SymbolColor</code> property determines color used to paint the symbol.
 *
 * @return Symbol color
 */
public Color getSymbolColor()
{
	if (symbolStyle == null) {
		return(null);
	}
	return(symbolStyle.getColor());
}

/**
 * Sets the value of the <code>SymbolColor</code> property. The
 * <code>SymbolColor</code> property determines color used to paint the symbol.
 *
 * @param sc Symbol color
 */
public void setSymbolColor(Color sc)
{
	if (symbolStyle == null) {
		symbolStyle = JCSymbolStyle.makeDefault(null);
	}
	symbolStyle.setColor(sc);
}

/**
 * Gets the value of the <code>SymbolSize</code> property.
 * The <code>SymbolSize</code> property determines the size of the symbols.
 *
 * @return Positive integer representing symbol size
 */
public int getSymbolSize()
{
	if (symbolStyle == null) {
		return(0);
	}
	return symbolStyle.getSize();
}

/**
 * Sets the value of the <code>SymbolSize</code> property.
 * The <code>SymbolSize</code> property determines the size of the symbols.
 * When set to 0, no symbol is drawn.
 *
 * @param sz Symbol size
 */
public void setSymbolSize(int sz)
{
	if (symbolStyle == null) {
		symbolStyle = JCSymbolStyle.makeDefault(null);
	}
	symbolStyle.setSize(sz);
}

/**
 * Initializes the contour style so that it has a <code>lineStyle</code>,
 * <code>fillStyle</code>, and <code>symbolStyle</code> with valid information.
 *
 * @param contour The contour object to use to get default colors
 * @param index The index of the contour to initialize
 */
protected void initialize(JCContour contour, int index)
{
	if (lineStyle == null) {
		lineStyle = new JCLineStyle(1, Color.black, JCLineStyle.SOLID);
	}
    else if (lineStyle.getColor() == null) {
        lineStyle.setColor(Color.black);
    }
	Color fillColor = contour.getDefaultColors().getColor(index);
	if (fillStyle == null) {
        fillStyle = new JCFillStyle(fillColor, JCFillStyle.SOLID);
	}
    else if (fillStyle.getColor() == null) {
        fillStyle.setColor(fillColor);
    }
	if (symbolStyle == null) {
        symbolStyle = new JCSymbolStyle(JCSymbolStyle.getDefaultShape(null),
                                        fillColor, 6);
	}
	else if (symbolStyle.getColor() == null) {
		symbolStyle.setColor(fillColor);
	}
}

}
