/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCData3dGridIndex.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

/**
 * <code>JCData3dGridIndex</code> contains a unique index to a grid data point
 * in Chart3d consisting of a <i>x</i> and a <i>y</i>.  It is used by the
 * chart's <code>pick()</code> methods, and also contains information related to
 * these operations.
 */
public class JCData3dGridIndex extends JCData3dIndex
                               implements java.io.Serializable {

/** The <i>x</i> index of selected data point. */
protected int x = -1;

/** The <i>y</i> index of the selected data point. */
protected int y = -1;

/**
 * Simple constructor for <code>JCData3dGridIndex</code>.
 */
public JCData3dGridIndex()
{
}

/**
 * Constructor that creates a grid data index from an <i>x</i> and a <i>y</i>.
 * The <code>dataView</code>, if needed, must be set via the
 * <code>setDataView()</code> method.
 *
 * @param x the <i>x</i> index within the data
 * @param y the <i>y</i> index within the data
 */
public JCData3dGridIndex(int x, int y)
{
	super();
	this.x = x;
	this.y = y;
}

/**
 * Constructor that creates a data index from the standard information needed
 * to create a data index: a data view, an <i>x</i>, and a <i>y</i>.
 *
 * @param dataView the data view that holds the (<i>x</i>, <i>y</i>) grid
 *     index
 * @param x the <i>x</i> index within the data
 * @param y the <i>y</i> index within the data
 */
public JCData3dGridIndex(Chart3dDataView dataView, int x, int y)
{
	super(dataView);
	this.x = x;
	this.y = y;
}

/**
 * Gets the value of the <i>X</i> property.  It contains the index of the
 * <i>x</i> grid to which this data index object refers.
 *
 * @return index of the selected <i>x</i>
 */
public int getX() {
	return(x);
}

/**
 * Gets the value of the <i>X</i> property.  It contains the index of the
 * <i>x</i> grid to which this data index object refers.
 *
 * @param x index of the selected <i>x</i>
 */
public void setX(int x)
{
	if (this.x == x) {
		return;
	}
	this.x = x;
	setChanged(true, Changeable.REDRAW);
}

/**
 * Gets the value of the <i>Y</i> property.  It contains the index of the
 * <i>y</i> grid to which this data index object refers.
 *
 * @return index of the selected <i>y</i>
 */
public int getY() {
	return(y);
}

/**
 * Gets the value of the <i>Y</i> property.  It contains the index of the
 * <i>y</i> grid to which this data index object refers.
 *
 * @param y index of the selected <i>y</i>
 */
public void setY(int y)
{
	if (this.y == y) {
		return;
	}
	this.y = y;
	setChanged(true, Changeable.REDRAW);
}

/**
 * Determines if the specified object refers to the same data point as this
 * <code>JCData3dGridIndex</code> instance.
 *
 * @param o object to compare
 * @return whether the passed object is a <code>JCData3dGridIndex</code> instance
 *     and refers to the same data point as this instance
 */
public boolean equals(Object o) {
	if (o == null) {
		return(false);
	}

	if (o instanceof JCData3dGridIndex) {
		JCData3dGridIndex di = (JCData3dGridIndex)o;
		return(dataView == di.dataView && x == di.x && y == di.y);
	}
	else {
		return(false);
	}
}

}
