/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCData3dIndex.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

/**
 * <code>JCData3dIndex</code> contains a unique index to a data point in Chart3d
 * consisting of either a (<i>x</i>, <i>y</i>) grid data value or (series, point)
 * point data value.  It is used by the chart3d's <code>pick()</code> and
 * <code>unpick()</code> methods, and also contains information related to
 * these operations.
 */
public class JCData3dIndex implements java.io.Serializable {

/** Special index value that implies using all indices. */
public final static int ALL = -100;

/** <code>Chart3d</code> object picked (<code>legend</code>,
 *  <code>chart3dArea</code>).
 */
protected Object obj = null;

/** Data view object for the data point that this index references. */
protected Chart3dDataView  dataView = null;

/** If this is a child of a grid color, use this to propagate changes. */
protected JCGridColor parent = null;

/** The distance of a selected point from its closest grid point. */
protected double distance = Double.MAX_VALUE;

/**
 * Simple constructor for <code>JCData3dIndex</code>.
 */
public JCData3dIndex()
{
}

/**
 * Constructor that creates a data index from a <code>dataView</code> and data.
 *
 * @param dataView the <code>dataView</code> this index is based upon
 */
public JCData3dIndex(Chart3dDataView dataView)
{
	this.dataView = dataView;
}


/**
 * Gets the value of the <code>DataView</code> property.  It contains the <code>Chart3dDataView</code>
 * instance that holds the point that this data index class is referring to.
 *
 * @return the data view
 */
public Chart3dDataView getDataView()
{
	return(dataView);
}

/**
 * Sets the value of the <code>DataView</code> property.  It contains the
 * <code>ChartDataView </code> instance that holds the point that this data
 * index class is referring to.
 *
 * @param dataView sets a new data view for this object
 */
public void setDataView(Chart3dDataView dataView)
{
	this.dataView = dataView;
}

/**
 * Gets the value of the <code>Obj</code> property.  It is used by pick operations
 * and contains the object picked (for example, a <code>JCChart3dArea</code> or
 * <code>JCLegend</code> instance) by those operations.
 *
 * @return picked object
 */
public Object getObject()
{
	return(obj);
}

/**
 * Sets the value of the <code>Obj</code> property.  It is used by pick operations
 * and contains the object picked (for example, a <code>JCChart3dArea</code> or
 * <code>JCLegend</code> instance) by those operations.
 *
 * @param object picked object
 */
public void setObject(Object object)
{
	obj = object;
}

/**
 * Gets the value of the <code>Distance</code> property.  It is used by pick
 * operations and contains the distance between the point selected and the
 * nearest data point.
 *
 * @return distance from the selected point
 */
public double getDistance()
{
	return(distance);
}

/**
 * Sets the value of the <code>Distance</code> property.  It is used by pick
 * operations and contains the distance between the point selected and the
 * nearest data point.
 *
 * @param distance the new distance from the data point
 */
public void setDistance(double distance)
{
	this.distance = distance;
}

/**
 * Telld the parent (if there is one) that a change has happened.
 *
 * @param b has a changed happened
 * @param cf mask which tells what type of change has happened
 */
protected void setChanged(boolean b, int cf)
{
	if (parent != null) {
		parent.setChanged(b, cf);
	}
}

}


