/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCData3dLabelIndex.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

/**
 * <code>JCData3dLabelIndex</code> contains a unique index to a label
 * in Chart3d consisting of a <code>JCValueLabel</code> and its index.  It is used by the
 * chart <code>pick()</code> methods, and also contains information relating to
 * these operations.
 */
public class JCData3dLabelIndex extends JCData3dIndex
                               implements java.io.Serializable {

/** The index of the selected label. */
protected int labelIndex = -1;

/** The selected label. */
protected JCValueLabel valueLabel = null;

/**
 * Simple constructor for <code>JCData3dLabelIndex</code>.
 */
public JCData3dLabelIndex()
{
}

/**
 * Constructor that creates a label data index from a <code>JCValueLabel</code>.
 * The <code>dataView</code>, if needed, must be set via the
 * <code>setDataView()</code> method.
 *
 * @param valueLabel the selected label (stored as a <code>JCValueLabel</code>)
 */
public JCData3dLabelIndex(JCValueLabel valueLabel)
{
	super();
    this.valueLabel = valueLabel;
    if (valueLabel != null) {
        this.labelIndex = valueLabel.getIndex();
    }
}

/**
 * Gets the value of the <i>labelIndex</i> property.  It contains the index of
 * the selected label.
 *
 * @return index of the selected <i>x</i>
 */
public int getLabelIndex() {
	return(labelIndex);
}

/**
 * Sets the value of the <i>labelIndex</i> property.  It contains the index of
 * the selecteded label.
 *
 * @param labelIndex index of the selected <i>x</i>
 */
public void setLabelIndex(int labelIndex)
{
	this.labelIndex = labelIndex;
}

/**
 * Gets the selected label (stored as a <code>JCValueLabel</code>).
 *
 * @return the selected label
 */
public JCValueLabel getValueLabel()
{
    return(valueLabel);
}

/**
 * Sets the selected label (stored as a <code>JCValueLabel</code>).
 *
 * @param valueLabel the new selected label
 */
public void setValueLabel(JCValueLabel valueLabel)
{
    this.valueLabel = valueLabel;
}

/**
 * Determines if the specified object refers to the same label as this
 * <code>JCData3dLabelIndex</code> instance.
 *
 * @param o object to compare
 * @return if the passed object is a <code>JCData3dLabelIndex</code> instance,
 *     and if it refers to the same label as this instance
 */
public boolean equals(Object o) {
	if (o == null) {
		return(false);
	}

	if (o instanceof JCData3dLabelIndex) {
		JCData3dLabelIndex li = (JCData3dLabelIndex)o;
		return(dataView == li.dataView     &&
               labelIndex == li.labelIndex &&
               valueLabel == li.valueLabel);
	}
	else {
		return(false);
	}
}

}
