/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCData3dPointIndex.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

/**
 * <code>JCData3dIndex</code> contains a unique index to a data point in Chart,
 * consisting of a data series instance and an index representing the
 * point within the data series.  It is used by the chart <code>pick()</code>
 * methods and also contains information related to these operations.
 *
 */
public class JCData3dPointIndex extends JCData3dIndex
			                    implements java.io.Serializable {

/** The series index of the selected point. */
protected int series = -1;

/** The point index of the selected point. */
protected int point  = -1;

/**
 * Simple constructor for <code>JCData3dPointIndex</code>.
 */
public JCData3dPointIndex()
{
}

/**
 * Constructor that creates a point data index from a series index and a point
 * index within that series.  The <code>dataView</code>, if needed, must be set
 * via the <code>setDataView()</code> method.
 *
 * @param series the series index
 * @param point the point index
 */
public JCData3dPointIndex(int series, int point) {
	super();
	this.series = series;
	this.point  = point;
}

/**
 * Constructor that creates a data index from the standard information needed
 * to create a data index (a data view, a series index, and a point index).
 *
 * @param dataView the data view
 * @param series the series index
 * @param point the point index
 */
public JCData3dPointIndex(Chart3dDataView dataView, int series, int point) {
	super(dataView);
	this.series = series;
	this.point  = point;
}

/**
 * Gets the value of the <code>Series</code> property.  It contains the index
 * of the series that this data index object refers to.
 *
 * @return index of the selected series
 */
public int getSeries() {
	return(series);
}

/**
 * Gets the value of the <code>Series</code> property.  It contains the index
 * of the series that this data index object refers to.
 *
 * @param series index of the selected series
 */
public void setSeries(int series)
{
	this.series = series;
}

/**
 * Gets the value of the <code>Point</code> property.  It contains the index
 * of the point that this data object class refers to.
 *
 * @return index of the selected point within its data series
 */
public int getPoint() {
	return(point);
}

/**
 * Gets the value of the <code>Point</code> property.  It contains the index
 * of the point that this data index object refers to.
 *
 * @param point index of the selected point within its data series
 */
public void setPoint(int point)
{
	this.point = point;
}

/**
 * Determines if the specified object refers to the same data point as this
 * <code>JCData3dPointIndex</code> instance.
 *
 * @param o object to compare
 * @return if the passed object is a <code>JCData3dPointIndex</code> instance
 *     and refers to the same data point as this instance
 */
public boolean equals(Object o) {
	if (o == null) {
		return(false);
	}

	if (o instanceof JCData3dPointIndex) {
		JCData3dPointIndex di = (JCData3dPointIndex)o;
		return(dataView == di.dataView && series == di.series && point == di.point);
	}
	else {
		return(false);
	}
}

}


