/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCElevation.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:12:58 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import java.awt.Color;

/**
 *
 */
public class JCElevation extends ParentTrackChange {

/**  */
static final long serialVersionUID = -5542417106751358122L;

/** Asks if mesh lines are drawn. */
protected boolean meshed = true;

/** Asks if facets are shaded. */
protected boolean shaded = false;

/** Asks if surfaces/bars are transparent (for example, are hidden lines drawn). */
protected boolean transparent = false;

/** Sets the mesh top color (only drawn in certain cases). */
protected Color meshTopColor = Color.black;

/** Sets the mesh bottom color (only drawn in certain cases). */
protected Color meshBottomColor = Color.black;

/** Sets the shaded top color (only drawn in certain cases).  Defaults to light grey. */
protected Color shadedTopColor = new Color(211, 211, 211);

/** Sets the shaded bottom color (only drawn in certain cases). Defaults to dim grey. */
protected Color shadedBottomColor = new Color(112, 112, 112);

/**
 * Constructor with no arguments creates a <code>JCElevation</code> with its
 * values set at their default values.
 */
public JCElevation()
{
}

/**
 * Constructor that intializes the meshed, shaded, and transparent properties.
 *
 * @param meshed Are mesh lines drawn?
 * @param shaded Are facets shaded?
 * @param transparent Are surfaces/bars transparent (for example, are hidden lines
 *     drawn)?
 */
public JCElevation(boolean meshed, boolean shaded, boolean transparent)
{
	this.meshed      = meshed;
	this.shaded      = shaded;
	this.transparent = transparent;
}

/**
 * Sets flag which determine whether this elevation is meshed.
 *
 * @param meshed the new value of the meshed flag
 */
public void setMeshed(boolean meshed)
{
	if (this.meshed == meshed) {
		return;
	}
	this.meshed = meshed;
	callParentSetChanged(true, Changeable.ALL);
}

/**
 * Returns whether this elevation is meshed.
 *
 * @return the value of the meshed flag
 */
public boolean isMeshed()
{
	return(meshed);
}

/**
 * Sets flag which determines whether this elevation is shaded.
 *
 * @param shaded the new value of the shaded flag
 */
public void setShaded(boolean shaded)
{
	if (this.shaded == shaded) {
		return;
	}
	this.shaded = shaded;
	callParentSetChanged(true, Changeable.ALL);
}

/**
 * Returns whether this elevation is shaded.
 *
 * @return the value of the shaded flag
 */
public boolean isShaded()
{
	return(shaded);
}

/**
 * Sets flag which determines whether this elevation is transparent (hidden
 * lines are removed).
 *
 * @param transparent the new value of the transparent flag
 */
public void setTransparent(boolean transparent)
{
	if (this.transparent == transparent) {
		return;
	}
	this.transparent = transparent;
	callParentSetChanged(true, Changeable.REDRAW);
}

/**
 * Returns whether this elevation is transparent (hidden lines are removed).
 *
 * @return the value of the transparent flag
 */
public boolean isTransparent()
{
	return(transparent);
}

/**
 * Sets the top color of the mesh (only drawn in certain cases).
 *
 * @param meshTopColor the new mesh top color
 */
public void setMeshTopColor(Color meshTopColor)
{
	if (this.meshTopColor == meshTopColor) {
		return;
	}
	this.meshTopColor = meshTopColor;
	callParentSetChanged(true, Changeable.REDRAW);
}

/**
 * Sets the top color of the mesh (only drawn in certain cases).
 *
 * @return the current mesh top color
 */
public Color getMeshTopColor()
{
	return(meshTopColor);
}

/**
 * Sets the bottom color of the mesh (only drawn in certain cases).
 *
 * @param meshBottomColor the new mesh bottom color
 */
public void setMeshBottomColor(Color meshBottomColor)
{
	if (this.meshBottomColor == meshBottomColor) {
		return;
	}
	this.meshBottomColor = meshBottomColor;
	callParentSetChanged(true, Changeable.REDRAW);
}

/**
 * Sets the bottom color of the mesh (only drawn in certain cases).
 *
 * @return the current mesh bottom color
 */
public Color getMeshBottomColor()
{
	return(meshBottomColor);
}

/**
 * Sets the top color of the shaded (only drawn in certain cases).
 *
 * @param shadedTopColor the new shaded top color
 */
public void setShadedTopColor(Color shadedTopColor)
{
	if (this.shadedTopColor == shadedTopColor) {
		return;
	}
	this.shadedTopColor = shadedTopColor;
	callParentSetChanged(true, Changeable.REDRAW);
}

/**
 * Sets the top color of the shaded (only drawn in certain cases).
 *
 * @return the current shaded top color
 */
public Color getShadedTopColor()
{
	return(shadedTopColor);
}

/**
 * Sets the bottom color of the shaded (only drawn in certain cases).
 *
 * @param shadedBottomColor the new shaded bottom color
 */
public void setShadedBottomColor(Color shadedBottomColor)
{
	if (this.shadedBottomColor == shadedBottomColor) {
		return;
	}
	this.shadedBottomColor = shadedBottomColor;
	callParentSetChanged(true, Changeable.REDRAW);
}

/**
 * Sets the bottom color of the shaded (only drawn in certain cases).
 *
 * @return the current shaded bottom color
 */
public Color getShadedBottomColor()
{
	return(shadedBottomColor);
}

}
