/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCFillStyle.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import com.klg.jclass.chart3d.resources.LocaleBundle;
import com.klg.jclass.util.swing.JCIconCreator;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;

/**
 * Controls the appearance of a fill, including the fill pattern and the
 * fill color.
 */
public class JCFillStyle extends JCStyle {

/** Sets the fill color. */
protected Color          color           = null;

/** Sets the background color (for patterned fills). */
protected Color          backgroundColor = null;

/** Sets the fill pattern. */
protected int            pattern         = SOLID;

/** Sets the index of the fill color in the default color array. */
protected int            colorIndex      = 0;

/** Image used to paint the fill region. */
protected Image          image           = null;

/** Anchor rectangle for fill patterns. */
protected Rectangle      anchorRect      = null;

/** Orientation for fill patterns. */
protected int            orientation     = NONE;

/** Custom fill paint property that has been set by the user. */
protected TexturePaint   customPaint     = null;

/** Fill paint object. */
protected Paint          fillPaint       = null;

/** Highlights the fill paint object. */
protected Paint          hiFillPaint     = null;

/** Shadow fill paint object. */
protected Paint          shFillPaint     = null;

/** Image used for filling with images. */
protected BufferedImage  buffImage       = null;

// Orientation for fill patterns -- use NONE = 0 from fill pattern list
public final static int  TOP           = 1;
public final static int  BOTTOM        = 2;
public final static int  LEFT          = 3;
public final static int  RIGHT         = 4;
public final static int  CENTER        = 5;
public final static int  BOTTOM_LEFT   = 6;
public final static int  TOP_LEFT      = 7;
public final static int  BOTTOM_RIGHT  = 8;
public final static int  TOP_RIGHT     = 9;

// Fill Patterns
public final static int  NONE          = 0;
public final static int  SOLID         = 1;
public final static int  PER_25        = 2;
public final static int  PER_50        = 3;
public final static int  PER_75        = 4;
public final static int  HORIZ_STRIPE  = 5;
public final static int  VERT_STRIPE   = 6;
public final static int  STRIPE_45     = 7;
public final static int  STRIPE_135    = 8;
public final static int  DIAG_HATCHED  = 9;
public final static int  CROSS_HATCHED = 10;
public final static int  CUSTOM_FILL   = 11; // Fill with images set by user
public final static int  CUSTOM_STACK  = 12; // Stack images set by user
public final static int  CUSTOM_PAINT  = 13; // Fill with TexturePaint
                                             // object set by user

// Fill pattern definitions
public final static String[] PER_25_PIXELS = {
	"C C C C C C C C C C C C C C C C ",
	"                                ",
	"C C C C C C C C C C C C C C C C ",
	"                                ",
	"C C C C C C C C C C C C C C C C ",
	"                                ",
	"C C C C C C C C C C C C C C C C ",
	"                                ",
	"C C C C C C C C C C C C C C C C ",
	"                                ",
	"C C C C C C C C C C C C C C C C ",
	"                                ",
	"C C C C C C C C C C C C C C C C ",
	"                                ",
	"C C C C C C C C C C C C C C C C ",
	"                                "
	};
public final static String[] PER_50_PIXELS = {
	"C C C C C C C C C C C C C C C C ",
	" C C C C C C C C C C C C C C C C",
	"C C C C C C C C C C C C C C C C ",
	" C C C C C C C C C C C C C C C C",
	"C C C C C C C C C C C C C C C C ",
	" C C C C C C C C C C C C C C C C",
	"C C C C C C C C C C C C C C C C ",
	" C C C C C C C C C C C C C C C C",
	"C C C C C C C C C C C C C C C C ",
	" C C C C C C C C C C C C C C C C",
	"C C C C C C C C C C C C C C C C ",
	" C C C C C C C C C C C C C C C C",
	"C C C C C C C C C C C C C C C C ",
	" C C C C C C C C C C C C C C C C",
	"C C C C C C C C C C C C C C C C ",
	" C C C C C C C C C C C C C C C C",
	};
public final static String[] PER_75_PIXELS = {
    "CCC CCC CCC CCC CCC CCC CCC CCC ",
    "C CCC CCC CCC CCC CCC CCC CCC CC",
    "CCC CCC CCC CCC CCC CCC CCC CCC ",
    "C CCC CCC CCC CCC CCC CCC CCC CC",
    "CCC CCC CCC CCC CCC CCC CCC CCC ",
    "C CCC CCC CCC CCC CCC CCC CCC CC",
    "CCC CCC CCC CCC CCC CCC CCC CCC ",
    "C CCC CCC CCC CCC CCC CCC CCC CC",
    "CCC CCC CCC CCC CCC CCC CCC CCC ",
    "C CCC CCC CCC CCC CCC CCC CCC CC",
    "CCC CCC CCC CCC CCC CCC CCC CCC ",
    "C CCC CCC CCC CCC CCC CCC CCC CC",
    "CCC CCC CCC CCC CCC CCC CCC CCC ",
    "C CCC CCC CCC CCC CCC CCC CCC CC",
    "CCC CCC CCC CCC CCC CCC CCC CCC ",
    "C CCC CCC CCC CCC CCC CCC CCC CC",
    };
public final static String[] HORIZ_STRIPE_PIXELS = {
    "                                ",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "                                ",
    "                                ",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "                                ",
    "                                ",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "                                ",
    "                                ",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "                                ",
    };
public final static String[] VERT_STRIPE_PIXELS = {
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    "CC  CC  CC  CC  CC  CC  CC  CC  ",
    };
public final static String[] STRIPE_45_PIXELS = {
    "   CC      CC      CC      CC   ",
    "  CC      CC      CC      CC    ",
    " CC      CC      CC      CC     ",
    "CC      CC      CC      CC      ",
    "C      CC      CC      CC      C",
    "      CC      CC      CC      CC",
    "     CC      CC      CC      CC ",
    "    CC      CC      CC      CC  ",
    "   CC      CC      CC      CC   ",
    "  CC      CC      CC      CC    ",
    " CC      CC      CC      CC     ",
    "CC      CC      CC      CC      ",
    "C      CC      CC      CC      C",
    "      CC      CC      CC      CC",
    "     CC      CC      CC      CC ",
    "    CC      CC      CC      CC  ",
    };
public final static String[] STRIPE_135_PIXELS = {
    "C      CC      CC      CC      C",
    "CC      CC      CC      CC      ",
    " CC      CC      CC      CC     ",
    "  CC      CC      CC      CC    ",
    "   CC      CC      CC      CC   ",
    "    CC      CC      CC      CC  ",
    "     CC      CC      CC      CC ",
    "      CC      CC      CC      CC",
    "C      CC      CC      CC      C",
    "CC      CC      CC      CC      ",
    " CC      CC      CC      CC     ",
    "  CC      CC      CC      CC    ",
    "   CC      CC      CC      CC   ",
    "    CC      CC      CC      CC  ",
    "     CC      CC      CC      CC ",
    "      CC      CC      CC      CC",
    };
public final static String[] DIAG_HATCHED_PIXELS = {
    "CC      CC      CC      CC      ",
    "CCC    CCCC    CCCC    CCCC    C",
    "  CC  CC  CC  CC  CC  CC  CC  CC",
    "   CCCC    CCCC    CCCC    CCCC ",
    "    CC      CC      CC      CC  ",
    "   CCCC    CCCC    CCCC    CCCC ",
    "  CC  CC  CC  CC  CC  CC  CC  CC",
    "CCC    CCCC    CCCC    CCCC    C",
    "CC      CC      CC      CC      ",
    "CCC    CCCC    CCCC    CCCC    C",
    "  CC  CC  CC  CC  CC  CC  CC  CC",
    "   CCCC    CCCC    CCCC    CCCC ",
    "    CC      CC      CC      CC  ",
    "   CCCC    CCCC    CCCC    CCCC ",
    "  CC  CC  CC  CC  CC  CC  CC  CC",
    "CCC    CCCC    CCCC    CCCC    C",
    };
public final static String[] CROSS_HATCHED_PIXELS = {
    "CC      CC      CC      CC      ",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CC      CC      CC      CC      ",
    "CC      CC      CC      CC      ",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CC      CC      CC      CC      ",
    "CC      CC      CC      CC      ",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CC      CC      CC      CC      ",
    "CC      CC      CC      CC      ",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
    "CC      CC      CC      CC      "
};

/**
 * Constructor for fill style objects.
 *
 * @param color fill color
 * @param pattern Fill pattern.  One of <code>NONE</code>, <code>SOLID</code>,
 * <code>PER_25</code>, <code>PER_50</code>, <code>PER_75</code>, <code>HORIZ_STRIPE</code>,
 * <code>VERT_STRIPE</code>, <code>STRIPE_45</code>, <code>STRIPE_135</code>, <code>DIAG_HATCHED</code>,
 * or <code>CROSS_HATCHED</code>.
 */
public JCFillStyle(Color color, int pattern)
{
	this.color   = color;
	this.pattern = pattern;
	buffImage    = makeBufferedImage();
	fillPaint    = null;
	hiFillPaint  = null;
	shFillPaint  = null;
}

/**
 * Creates a default fill style.
 *
 * @param c <code>Chart3d</code> object to base default fill style on
 * @return <code>JCFillStyle</code> object with default color and fill
 *     pattern. The defaults are cycled, so consecutive calls will return
 *     different fill styles.
 */
public static JCFillStyle makeDefault(JCChart3d c)
{
	JCFillStyle style = new JCFillStyle(getDefaultColor(c, FILL_STYLE),
                                        getDefaultPattern());
	int index = 0;
	if (c != null) {
		index = c.getFillColorIndex();
		if (index == 0) {
			index = defaultColors.length - 1;
		}
		else {
			index--;
		}
	}
	style.colorIndex = index;
	return(style);
}

/**
 * Updates the graphics object with the elements of this line style (paint
 * fills and colors), with the patterned fills being centered around the
 * anchor rectangle that may have been set.
 *
 * @param gc <code>Graphics</code> object to change
 * @return <code>true</code> if you should now use this object to draw; <code>false</code>
 * if the style is such that no drawing should occur
 */
public boolean updateGraphics(Graphics gc)
{
    if (pattern == NONE) {
        return false;
    }

    Graphics2D gc2D = (Graphics2D) gc;
	// Make the fill paint object for this fill style wrt to the given
	// rectangle. If the pattern is SOLID, cache the fill paint object
	// for reuse. If the pattern is CUSTOM_PAINT, use the user-provided
	// paint object instead.
	if (pattern != CUSTOM_STACK || image == null) {
		if (pattern == CUSTOM_PAINT) {
			if (customPaint != null) {
				fillPaint = makeFillPaint(customPaint.getImage(), color,
                                          anchorRect, orientation);
			}
            else {
				fillPaint = color;
			}
		}
        else if (pattern != SOLID || fillPaint == null) {
			fillPaint = makeFillPaint(buffImage, color,
									  anchorRect, orientation);
		}
		gc2D.setPaint(fillPaint);
	}

	return(true);
}

/**
 * Updates the graphics object with the shadow elements of this line style
 * (paint fills and colors), centered around the anchor rectangle
 * that may have been set.
 *
 * @param gc <code>Graphics</code> object to change
 * @return <code>true</code> if you should now use this object to draw; <code>false</code>
 * if the style is such that no drawing should occur
 */
public boolean updateGraphicsToShadow(Graphics gc)
{
    if (pattern == NONE) {
        return false;
    }

	Color shadowColor = JCChart3dUtil.darker(color);
    Graphics2D gc2D = (Graphics2D) gc;
	// Make the fill paint object for this fill style wrt to the given
	// rectangle. If the pattern is SOLID, cache the fill paint object
	// for reuse. If the pattern is CUSTOM_PAINT, use the user-provided
	// paint object instead.
	if (pattern != CUSTOM_STACK || image == null) {
		if (pattern == CUSTOM_PAINT) {
			if (customPaint != null) {
				shFillPaint = makeFillPaint(customPaint.getImage(), shadowColor,
											anchorRect, orientation);
			}
            else {
				shFillPaint = shadowColor;
			}
		}
        else if (pattern != SOLID || shFillPaint == null) {
			if (pattern == CUSTOM_FILL) {
				shFillPaint = makeFillPaint(buffImage, shadowColor,
                                            anchorRect, orientation);
			}
            else {
				shFillPaint = makeFillPaint(makeBufferedImage(shadowColor),
                                            shadowColor, anchorRect,
                                            orientation);
			}
		}
		gc2D.setPaint(shFillPaint);
	}

	return(true);
}

/**
 * Updates the graphics object with the highlight elements of this line style
 * (paint fills and colors), centered around the anchor rectangle
 * that may have been set.
 *
 * @param gc <code>Graphics</code> object to change
 * @return <code>true</code> if you should now use this object to draw; <code>false</code>
 * if the style is such that no drawing should occur
 */
public boolean updateGraphicsToHighlight(Graphics gc)
{
    if (pattern == NONE) {
        return false;
    }

	Color highlightColor = JCChart3dUtil.brighter(color);
    Graphics2D gc2D = (Graphics2D) gc;
	// Make the fill paint object for this fill style wrt to the given
	// rectangle. If the pattern is SOLID, cache the fill paint object
	// for reuse. If the pattern is CUSTOM_PAINT, use the user-provided
	// paint object instead.
	if (pattern != CUSTOM_STACK || image == null) {
		if (pattern == CUSTOM_PAINT) {
			if (customPaint != null) {
				hiFillPaint = makeFillPaint(customPaint.getImage(),
							                highlightColor, anchorRect,
                                            orientation);
			}
            else {
				hiFillPaint = highlightColor;
			}
		}
        else if (pattern != SOLID || hiFillPaint == null) {
			if (pattern == CUSTOM_FILL) {
				hiFillPaint = makeFillPaint(buffImage, highlightColor,
                                            anchorRect, orientation);
			}
            else {
				hiFillPaint = makeFillPaint(makeBufferedImage(highlightColor),
                                            highlightColor, anchorRect,
                                            orientation);
			}
		}
		gc2D.setPaint(hiFillPaint);
	}

	return(true);
}

/**
 * Resets the graphics object to a default drawing style.
 *
 * @param gc <code>Graphics</code> object to reset
 */
public void resetGraphics(Graphics gc)
{
    Graphics2D gc2D = (Graphics2D) gc;

    gc2D.setColor(color);
}

/**
 * Updates the fill orientation properties for a pending fill operation. These are not
 * actually properties of the fill style; they are calculated by the drawable
 * and set here so that they don't need to be passed into the fill methods.
 *
 * @param rect the new anchor rectangle
 * @param newOrient the new orientatoin
 */
public void updateFillOrientation(Rectangle rect, int newOrient)
{
	anchorRect  = rect;
	orientation = newOrient;
}

/**
 * Resets the fill orientation properties before the next fill operation.
 */
public void resetFillOrientation()
{
	anchorRect  = null;
	orientation = NONE;
}

/**
 * Performs the requested fill given the values of the properties
 * of the <code>JCFillStyle</code> instance.
 *
 * @param gc graphics context to use for drawing
 * @param x <i>x</i> origin of fill region (in pixels)
 * @param y <i>y</i> origin of fill region (in pixels)
 * @param width width of fill region (in pixels)
 * @param height height of fill region (in pixels)
 */
public void fillRect(java.awt.Graphics gc, int x, int y, int width, int height)
{
	if (updateGraphics(gc)) {
		gc.fillRect(x, y, width, height);
		resetGraphics(gc);
	}
}

/**
 * Performs the requested fill with an image, given the values of the properties
 * of the <code>JCFillStyle</code> instance.
 *
 * @param gc graphics context to use for drawing
 * @param x <i>x</i> origin of fill region (in pixels)
 * @param y <i>y</i> origin of fill region (in pixels)
 * @param width width of fill region (in pixels)
 * @param height height of fill region (in pixels)
 * @param positive <code>true</code> if the fill is occurring above the axis
 * @param inverted <code>true</code> if the axes are inverted
 */
public void fillImageRect(java.awt.Graphics gc,int x,int y,int width,
						  int height, boolean positive, boolean inverted)
{
	if (image == null) {
        return;
	}

	int iHeight = image.getHeight(null);
	int iWidth  = image.getWidth(null);

	if (iHeight <= 0 || iWidth <=0) {
		return;
	}

	Graphics tempGC = gc.create();

	tempGC.clipRect(x, y, width, height);

	if (!inverted) {
		int draw_x = x + ((width - iWidth) / 2);

		if (positive) {
			int draw_y = y + height - iHeight;
			while (draw_y > y) {
				tempGC.drawImage(image, draw_x, draw_y, null);
				draw_y -= iHeight;
			}
			tempGC.drawImage(image, draw_x, draw_y, null);
		}
        else {
			int draw_y = y + 1;
			while (draw_y < y + height) {
				tempGC.drawImage(image, draw_x, draw_y, null);
				draw_y += iHeight;
			}
		}
	}
    else {
		int draw_y = y + ((height - iHeight) / 2);

		if (positive) {
			int draw_x = x + 1;
			while (draw_x < x + width) {
				tempGC.drawImage(image, draw_x, draw_y, null);
				draw_x += iWidth;
			}
		}
        else {
			int draw_x = x + width - iWidth;
			while (draw_x > x) {
				tempGC.drawImage(image, draw_x, draw_y, null);
				draw_x -= iWidth;
			}
			tempGC.drawImage(image, draw_x, draw_y, null);
		}
	}

	tempGC.dispose();
}

/**
 * Performs the requested fill with a drawn outline, given the values of
 * the properties of the <code>JCFillStyle</code> instance.
 *
 * @param gc graphics context to use for drawing
 * @param x <i>x</i> origin of fill region (in pixels)
 * @param y <i>y</i> origin of fill region (in pixels)
 * @param width width of fill region (in pixels)
 * @param height height of fill region (in pixels)
 */
public void fillOutlineRect(java.awt.Graphics gc, int x,int y,
                            int width, int height)
{
	fillOutlineRect(gc, x, y, width, height, null);
}

/**
 * Performs the requested fill with a drawn outline given the values of
 * the properties of the <code>JCFillStyle</code> instance.
 *
 * @param gc graphics context to use for drawing
 * @param x <i>x</i> origin of fill region (in pixels)
 * @param y <i>y</i> origin of fill region (in pixels)
 * @param width width of fill region (in pixels)
 * @param height height of fill region (in pixels)
 * @param localColor Color of the outline. If null, color will be calculated from
 *     the fill color.
 */
public void fillOutlineRect(java.awt.Graphics gc, int x, int y,
		                    int width, int height, Color localColor)
{
	fillRect(gc, x, y, width, height);
	drawOutlineRect(gc, x, y, width, height, localColor);
}

/**
 * Draws an outline around the specified rectangle.
 *
 * @param gc graphics context to use for drawing
 * @param x <i>x</i> origin of rectangle (in pixels)
 * @param y <i>y</i> origin of outline (in pixels)
 * @param width width of outline (in pixels)
 * @param height height of outline (in pixels)
 * @param localColor Color of the outline. If null, color will be calculated from
 *     the fill color.
 */
public void drawOutlineRect(java.awt.Graphics gc,int x,int y,
							int width,int height, Color localColor)
{
	if (localColor == null) {
		localColor = calcOutlineColor();
	}
	if (!gc.getColor().equals(localColor)) {
        gc.setColor(localColor);
	}
	gc.drawRect(x, y, width, height);
}

/**
 * Performs the requested fill given the values of the properties
 * of the <code>JCFillStyle</code> instance.
 *
 * @param gc graphics context to use for drawing
 * @param poly polygon to fill
 */
public void fillPolygon(java.awt.Graphics gc, Polygon poly)
{
	if (updateGraphics(gc)) {
		gc.fillPolygon(poly);
		resetGraphics(gc);
	}
}

/**
 * Performs the requested fill with a drawn outline given the values of the
 * properties of the <code>JCFillStyle</code> instance.
 *
 * @param gc graphics context to use for drawing
 * @param poly polygon to fill
 * @param localColor Color of the outline. If null, color will be calculated from
 *     the fill color.
 */
public void fillOutlinePolygon(java.awt.Graphics gc, java.awt.Polygon poly,
                               Color localColor)
{
	fillPolygon(gc, poly);
	drawOutlinePolygon(gc, poly, localColor);
}

/**
 * Draws an outline around the given polygon.
 *
 * @param gc graphics context to use for drawing
 * @param poly polygon to outline
 * @param localColor Color of the outline. If null, color will be calculated from
 *     the fill color.
 */
public void drawOutlinePolygon(java.awt.Graphics gc, java.awt.Polygon poly,
                               Color localColor)
{
	if (localColor == null) {
		localColor = calcOutlineColor();
	}
	if (!gc.getColor().equals(localColor)) {
        gc.setColor(localColor);
	}
	gc.drawPolygon(poly);
}

/**
 * Draws a portion of the outline around the given polygon.
 *
 * @param gc graphics context to use for drawing
 * @param poly polygon to outline
 * @param start the index of the point within the polygon to start drawing at
 * @param end the index of the point within the polygon to stop drawing at
 * @param localColor Color of the outline. If null, color will be calculated from
 *     the fill color.
 */
public void drawOutlineLine(java.awt.Graphics gc, java.awt.Polygon poly,
                            int start, int end, Color localColor)
{
	if (localColor == null) {
		localColor = calcOutlineColor();
	}
	if (!gc.getColor().equals(localColor)) {
        gc.setColor(localColor);
	}
	for (int i = start; i < end; i++) {
		gc.drawLine(poly.xpoints[i],   poly.ypoints[i],
				    poly.xpoints[i+1], poly.ypoints[i+1]);
	}
}

/**
 * Performs the requested fill for a partial polygon with a drawn outline
 * given the values of the properties of the <code>JCFillStyle</code>
 * instance. A partial polygon is a polygon that is not closed. A line between
 * the first and last points will be drawn in the fill color. The line outlining
 * the rest of the polygon will be drawn in the color specified by
 * <code>localColor</code>.
 *
 * @param gc graphics context to use for drawing
 * @param poly polygon to fill
 * @param localColor Color of the outline. If null, color will be calculated from
 *     the fill color.
 */
public void fillOutlinePartialPolygon(java.awt.Graphics gc,
							          java.awt.Polygon poly,
                                      Color localColor)
{
	fillPolygon(gc, poly);

	// Fudge the open side of the polygon with a line of the fill color
	gc.drawLine(poly.xpoints[0], poly.ypoints[0],
			    poly.xpoints[poly.npoints - 1],
                poly.ypoints[poly.npoints - 1]);

	// Draw the outline
	if (localColor == null) {
		localColor = calcOutlineColor();
	}
	if (!gc.getColor().equals(localColor)) {
        gc.setColor(localColor);
	}
	for (int i = 0; i < poly.npoints - 1; i++) {
		gc.drawLine(poly.xpoints[i], poly.ypoints[i],
			        poly.xpoints[i + 1], poly.ypoints[i + 1]);
	}
}

/**
 * Performs the requested fill in a color darker than that specified in the
 * <code>JCFillStyle</code> instance.
 *
 * @param gc graphics context to use for drawing
 * @param poly polygon to fill
 */
public void fillPolygonShadow(java.awt.Graphics gc, Polygon poly)
{
	if (updateGraphicsToShadow(gc)) {
		gc.fillPolygon(poly);
		resetGraphics(gc);
	}
}

/**
 * Performs the requested fill in a color brighter than that specified in the
 * <code>JCFillStyle</code> instance.
 *
 * @param gc graphics context to use for drawing
 * @param poly polygon to fill
 */
public void fillPolygonHighlight(java.awt.Graphics gc, Polygon poly)
{
	if (updateGraphicsToHighlight(gc)) {
		gc.fillPolygon(poly);
		resetGraphics(gc);
	}
}

/**
 * Performs the requested fill in a color darker than that specified in the
 * <code>JCFillStyle</code> instance, including a drawn outline.
 *
 * @param gc graphics context to use for drawing
 * @param poly polygon to fill
 */
public void fillOutlinePolygonShadow(java.awt.Graphics gc, Polygon poly)
{
	fillOutlinePolygonShadow(gc, poly, null);
}

/**
 * Performs the requested fill in a color darker than that specified in the
 * <code>JCFillStyle</code> instance, including a drawn outline.
 *
 * @param gc graphics context to use for drawing
 * @param poly polygon to fill
 * @param localColor Color of the outline. If null, color will be calculated from
 *     the fill color.
 */
public void fillOutlinePolygonShadow(java.awt.Graphics gc, Polygon poly,
									 Color localColor)
{
	fillPolygonShadow(gc, poly);
	drawOutlinePolygon(gc, poly, localColor);
}

/**
 * Performs the requested fill in a color brighter than that specified in the
 * <code>JCFillStyle</code> instance, including a drawn outline.
 *
 * @param gc graphics context to use for drawing
 * @param poly polygon to fill
 */
public void fillOutlinePolygonHighlight(java.awt.Graphics gc, Polygon poly)
{
	fillOutlinePolygonHighlight(gc, poly, null);
}

/**
 * Performs the requested fill in a color brighter than that specified in the
 * <code>JCFillStyle</code> instance, including a drawn outline.
 *
 * @param gc graphics context to use for drawing
 * @param poly polygon to fill
 * @param localColor Color of the outline. If null, color will be calculated from
 *     the fill color.
 */
public void fillOutlinePolygonHighlight(java.awt.Graphics gc, Polygon poly,
										Color localColor)
{
	fillPolygonHighlight(gc, poly);

	drawOutlinePolygon(gc, poly, localColor);
}

Color calcOutlineColor()
{
	Color localColor = JCChart3dUtil.darker(color);
	if (localColor == color) {
		localColor = JCChart3dUtil.brighter(color);
	}
	return(localColor);
}

/**
 * Performs the requested fill given the values of the properties
 * of the <code>JCFillStyle</code> instance.
 *
 * @param gc graphics context to use for drawing
 * @param x <i>x</i> origin of fill region (in pixels)
 * @param y <i>y</i> origin of fill region (in pixels)
 * @param width width of fill region (in pixels)
 * @param height height of fill region (in pixels)
 * @param startAngle start angle of the arc (in degrees)
 * @param arcAngle size of arc to fill (in degrees)
 */
public void fillArc(java.awt.Graphics gc,int x,int y,int width,int height,
					int startAngle, int arcAngle)
{
	if (updateGraphics(gc)) {
		gc.fillArc(x, y, width, height, startAngle, arcAngle);
		resetGraphics(gc);
	}
}

/**
 * Performs the requested fill in a color darker than that specified in the
 * <code>JCFillStyle</code> instance.
 *
 * @param gc graphics context to use for drawing
 * @param x <i>x</i> origin of fill region (in pixels)
 * @param y <i>y</i> origin of fill region (in pixels)
 * @param width width of fill region (in pixels)
 * @param height height of fill region (in pixels)
 * @param startAngle start angle of the arc (in degrees)
 * @param arcAngle size of arc to fill (in degrees)
 */
public void fillArcShadow(java.awt.Graphics gc, int x, int y, int width,
						  int height, int startAngle, int arcAngle)
{
	if (updateGraphicsToShadow(gc)) {
		gc.fillArc(x, y, width, height, startAngle, arcAngle);
		resetGraphics(gc);
	}
}

/**
 * Gets the <code>Color</code> property of <code>JCFillStyle</code>, which determines the
 * color used to paint the fill region.
 *
 * @return AWT Color class representing the color to be used to paint the
 *     fills. If null, the current color of the <code>Graphics</code> object is used to paint
 *     the fill.
 */
public Color getColor()
{
    return(color);
}

/**
 * Sets the <code>Color</code> property of <code>JCFillStyle</code>, which
 * determines the color used to paint the fill region.
 *
 * @param color AWT Color class representing the color to be used to paint the
 *     fills. If null, the current color of the <code>Graphics</code> object is used to
 *     paint the fill.
 */
public void setColor(Color color)
{
	this.color = color;

	if (pattern != CUSTOM_FILL  &&
        pattern != CUSTOM_STACK &&
        pattern != CUSTOM_PAINT)
    {
		buffImage   = makeBufferedImage();
		fillPaint   = null;
		hiFillPaint = null;
		shFillPaint = null;
	}

	setChanged(true, Changeable.REDRAW | Changeable.NOTIFY_LEGEND_REDRAW);
}

/**
 * Gets the <code>Background</code> property of <code>JCFillStyle</code>, which
 * determines the background color used in painting patterned fills. If null,
 * the plot cube's (or if null the chart area's) background color will be used.
 *
 * @return AWT Color class representing the color to be used for the
 *     background when painting patterned fills.
 */
public Color getBackground()
{
    return(backgroundColor);
}

/**
 * Sets the <code>Background</code> property of <code>JCFillStyle</code>, which
 * determines the background color used in painting patterned fills.  If null,
 * the plot cube's (or if null the chart area's) background color will be used.
 *
 * @param color AWT Color class representing the color to be used for the
 *     background when painting patterned fills.
 */
public void setBackground(Color color)
{
	backgroundColor = color;

	if (pattern != CUSTOM_FILL  &&
        pattern != CUSTOM_STACK &&
        pattern != CUSTOM_PAINT)
    {
		buffImage   = makeBufferedImage();
		fillPaint   = null;
		hiFillPaint = null;
		shFillPaint = null;
	}

	setChanged(true, Changeable.REDRAW | Changeable.NOTIFY_LEGEND_REDRAW);
}

/**
 * Gets the <code>Pattern</code> property of <code>JCFillStyle</code>, which
 * controls the pattern used to fill the specified region.
 *
 * @return fill pattern, which is one of <code>NONE</code>, <code>SOLID</code>,
 * <code>25_PERCENT</code>, <code>50_PERCENT</code>, <code>75_PERCENT</code>,
 * <code>HORIZ_STRIPE</code>, <code>VERT_STRIPE</code>, <code>45_STRIPE</code>,
 * <code>135_STRIPE</code>, <code>DIAG_HATCHED</code>, <code>CROSS_HATCHED</code>,
 * <code>CUSTOM_FILL</code>, or <code>CUSTOM_STACK</code>
 */
public int getPattern()
{
    return(pattern);
}

/**
 * Sets the <code>Pattern</code> property of <code>JCFillStyle</code>, which controls the
 * pattern used to fill the specified region.
 *
 * @param pattern fill pattern, which is one of <code>NONE</code>, <code>SOLID</code>,
 * <code>25_PERCENT</code>, <code>50_PERCENT</code>, <code>75_PERCENT</code>,
 * <code>HORIZ_STRIPE</code>, <code>VERT_STRIPE</code>, <code>45_STRIPE</code>,
 * <code>135_STRIPE</code>, <code>DIAG_HATCHED</code>, <code>CROSS_HATCHED</code>,
 * <code>CUSTOM_FILL</code>, or <code>CUSTOM_STACK</code> (<code>CUSTOM_STACK</code>
 * is only used by bar charts)
 */
public void setPattern(int pattern) {

	if (this.pattern == pattern) {
        return;
	}
	switch (pattern) {
	case NONE:
	case SOLID:
	case PER_25:
	case PER_50:
	case PER_75:
	case HORIZ_STRIPE:
	case VERT_STRIPE:
	case STRIPE_45:
	case STRIPE_135:
	case DIAG_HATCHED:
	case CROSS_HATCHED:
		this.pattern = pattern;
		buffImage    = makeBufferedImage();
		fillPaint    = null;
		hiFillPaint  = null;
		shFillPaint  = null;
		break;
	case CUSTOM_FILL:
		this.pattern = pattern;
		buffImage    = makeBufferedImage(image);
		fillPaint    = null;
		hiFillPaint  = null;
		shFillPaint  = null;
		break;
	case CUSTOM_STACK:
		this.pattern = pattern;
		fillPaint    = null;
		hiFillPaint  = null;
		shFillPaint  = null;
		break;
	case CUSTOM_PAINT:
		this.pattern = pattern;
		break;
	default:
		throw new IllegalArgumentException(
            LocaleBundle.string(LocaleBundle.INVALID_FILLSTYLE_ERROR));
	}

	setChanged(true, Changeable.REDRAW | Changeable.NOTIFY_LEGEND_RECALC);
}

/**
 * Gets the custom fill paint property that has been set by the user.
 *
 * @return the custom fill paint object that was set by the user (null if not
 *     set)
 */
public TexturePaint getCustomPaint() {
	return customPaint;
}

/**
 * Sets a <code>custom fill paint</code> property in <code>JCFillStyle</code>,
 * which controls the pattern used to fill the specified region. This property
 * is used when the pattern has been set to <code>CUSTOM_PAINT</code>.
 *
 * @param customPaint
 */
public void setCustomPaint(TexturePaint customPaint) {
	if (this.customPaint == customPaint) {
        return;
	}
	buffImage   = null;
	this.customPaint = customPaint;
	setChanged(true, Changeable.REDRAW | Changeable.NOTIFY_LEGEND_RECALC);
}

/**
 * Gets the <code>Image</code> property.
 * The <code>Image</code> property determines the custom image used to paint
 * the fill region.
 *
 * @return AWT Image class representing the image to be used to paint image
 *     fills; if null, no image fill is done
 */
public Image getImage()
{
    return(image);
}

/**
 * Sets the <code>Image</code> property.
 * The <code>Image</code> property determines the custom image used to paint
 * the fill region.
 *
 * @param image AWT Image class representing the image to be used to paint
 *     image fills; if null, no image fill is done
 */
public void setImage(Image image)
{
	this.image = image;

	if (this.image != null) {
		int iHeight = this.image.getHeight(null);
		int iWidth  = this.image.getWidth(null);

		if (iHeight <= 0 || iWidth <= 0) {
			this.image = null;
		}
	}

	if (pattern == CUSTOM_FILL || pattern == CUSTOM_STACK) {
		buffImage   = makeBufferedImage(this.image);
		fillPaint   = null;
		hiFillPaint = null;
		shFillPaint = null;
	}

	setChanged(true, Changeable.REDRAW | Changeable.NOTIFY_LEGEND_RECALC);
}

/**
 * Sets the <code>Image</code> property of <code>JCFillStyle</code>, which
 * determines the image used to paint the fill region. Only bar charts use
 * this property.
 *
 * @param file the file from which to get the image
 */
public void setImage(String file)
{
	Image thisImage = (new ImageIcon(file)).getImage();

	setImage(thisImage);
}

/**
 * Returns the index used to generate the color. Only set if default colors are used.
 *
 * @return the current color index
 */
public int getColorIndex()
{
	return(colorIndex);
}

/**
 * Returns the default pattern.
 *
 * @return the default pattern
 */
static int getDefaultPattern()
{
	return(SOLID);
}

/**
 * Makes the <code>BufferedImage</code> for future fill paint operations using
 * the color specified in the <code>Color</code> property.
 *
 * @return a buffered image based on the fill color
 */
public BufferedImage makeBufferedImage()
{
	return(makeBufferedImage(color));
}

/**
 * Makes the <code>BufferedImage</code> object for fill operations using
 * specified color.
 *
 * @param useFgColor the foreground color to use
 * @return
 */
public BufferedImage makeBufferedImage(Color useFgColor)
{
	if (pattern == NONE || pattern == SOLID) {
		return(null);
	}

	JCIconCreator ic = new JCIconCreator(32, 16);
	ic.setColor('C', useFgColor);

	Color useBgColor = backgroundColor;
	// If no background color set, make background transparent!
	if (useBgColor == null) {
		useBgColor = new Color(0.0f, 0.0f, 0.0f, 0.0f);
	}
	ic.setColor(' ', useBgColor);

	String pixels[] = null;

	switch(pattern) {
	case PER_25:
		pixels = PER_25_PIXELS;
		break;
	case PER_50:
		pixels = PER_50_PIXELS;
		break;
	case PER_75:
		pixels = PER_75_PIXELS;
		break;
	case HORIZ_STRIPE:
		pixels = HORIZ_STRIPE_PIXELS;
		break;
	case VERT_STRIPE:
		pixels = VERT_STRIPE_PIXELS;
		break;
	case STRIPE_45:
		pixels = STRIPE_45_PIXELS;
		break;
	case STRIPE_135:
		pixels = STRIPE_135_PIXELS;
		break;
	case DIAG_HATCHED:
		pixels = DIAG_HATCHED_PIXELS;
		break;
	case CROSS_HATCHED:
		pixels = CROSS_HATCHED_PIXELS;
		break;
	}
	if (pixels == null) {
		return(null);
	}

	ic.setPixels(pixels);
	Image im = ic.getIcon().getImage();

	return(makeBufferedImage(im));
}

/**
 * Makes the <code>BufferedImage</code> object for fill operations using
 * specified image.
 *
 * @param im image to base buffered image on
 * @return
 */
public BufferedImage makeBufferedImage(Image im)
{
	if (im == null) {
        return(null);
	}

	int imageWidth  = im.getWidth(null);
	int imageHeight = im.getHeight(null);

	if (imageWidth < 0 || imageWidth < 0) {
        return(null);
	}

	BufferedImage bufferedImage = new BufferedImage(imageWidth, imageHeight,
                                                    BufferedImage.TYPE_INT_ARGB);
	Graphics2D bufferedImageGC = bufferedImage.createGraphics();
	bufferedImageGC.drawImage(im, 0, 0, null);
	bufferedImageGC.dispose();
	return(bufferedImage);
}

/**
 * Makes the <code>Paint</code> object for fill operations using the previously
 * created buffered image, the default foreground color, and no anchor
 * rectangle.
 * @return
 */
public Paint makeFillPaint()
{
	return(makeFillPaint(buffImage, color, null, NONE));
}

/**
 * Makes the <code>Paint</code> object for fill operations using specified
 * buffered image anchored in the center of the provided rectangle.
 *
 * @param useImage <code>BufferedImage</code> object to use for
 *     <code>Paint</code> object creation. If null, returns default color as
 *     <code>Paint</code>.
 * @param useFgColor
 * @param anchor Anchor rectangle to center <code>Paint</code> pattern within.
 *      If null, no rectangle is used and default <code>Paint</code> behavior is
 *      used.
 * @param orientation
 * @param useFgColor color to use as default paint if <code>useImage</code> is null, or in
 *     the case of a <code>Solid</code> pattern
 * @return
 */
public Paint makeFillPaint(BufferedImage useImage, Color useFgColor,
                           Rectangle anchor, int orientation)
{
	if (pattern == NONE) {
		return(null);
	}

	if (pattern == SOLID) {
		return(useFgColor);
	}

	if (useImage == null) {
		return(useFgColor);
	}

	int imageWidth = useImage.getWidth();
	int imageHeight = useImage.getHeight();

	Rectangle r = null;
	if (anchor != null && orientation != NONE) {
		int hoffset = (anchor.width - imageWidth) / 2;
		int voffset = (anchor.height - imageHeight) / 2;
		switch (orientation) {
			case BOTTOM :
				r = new Rectangle(hoffset + anchor.x,
					              anchor.y + anchor.height - imageHeight,
					              imageWidth, imageHeight);
				break;
			case TOP :
				r = new Rectangle(hoffset + anchor.x,
					              anchor.y,
					              imageWidth, imageHeight);
				break;
			case LEFT :
				r = new Rectangle(anchor.x,
					              voffset + anchor.y,
					              imageWidth, imageHeight);
				break;
			case RIGHT :
				r = new Rectangle(anchor.x + anchor.width - imageWidth,
					              voffset + anchor.y,
					              imageWidth, imageHeight);
				break;
			case CENTER :
				r = new Rectangle(hoffset + anchor.x,
                                  voffset + anchor.y,
					              imageWidth, imageHeight);
				break;
			case BOTTOM_LEFT :
				r = new Rectangle(anchor.x,
					              anchor.y + anchor.height - imageHeight,
					              imageWidth, imageHeight);
				break;
			case TOP_LEFT :
				r = new Rectangle(anchor.x,
					              anchor.y,
					              imageWidth, imageHeight);
				break;
			case BOTTOM_RIGHT :
				r = new Rectangle(anchor.x + anchor.width - imageWidth,
					              anchor.y + anchor.height - imageHeight,
					              imageWidth, imageHeight);
				break;
			case TOP_RIGHT :
				r = new Rectangle(anchor.x + anchor.width - imageWidth,
					              anchor.y,
					              imageWidth, imageHeight);
				break;
			default :
				r = null;
		}
	}
    else {
		r = new Rectangle(0, 0, imageWidth, imageHeight);
	}

	return(new TexturePaint(useImage, r));
}

/**
 * Returns <code>true</code> if this fill style can be used for custom image stacking;
 * that is, if the pattern is <code>CUSTOM_STACK</code> and an image is
 * present. Returns <code>false</code> otherwise.
 * @return Returns <code>true</code> if this fill style can be used for custom image stacking;
 * that is, if the pattern is <code>CUSTOM_STACK</code> and an image is
 * present. Returns <code>false</code> otherwise.
 */
protected boolean isCustomStackable()
{
	return(pattern == CUSTOM_STACK && image != null);
}

}
