/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCGridColor.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:12:58 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import java.awt.Color;

/**
 * Class that allows certain bars (or rows/columns of bars) to be drawn with
 * distinct colors.  This can be used to do selection of bars.
 */
public class JCGridColor {

static final long serialVersionUID = -5542417106751358122L;

/** The grid index of the selected bar (or row/column of bars). */
protected JCData3dGridIndex   dataIndex;

/** The color to give selected bars. */
protected Color               color;

/** The parent tracking changes in this object. */
protected TrackChange parent = null;

/**
 * Default constructor.  Color is null, and <i>x</i> and <i>y</i> are
 * invalid values.
 */
public JCGridColor()
{
	this.dataIndex        = new JCData3dGridIndex();
	this.dataIndex.parent = this;
	this.color            = null;
}

/**
 * Constructor that specifies the necessary info: <i>x</i>, <i>y</i>, and color.
 *
 * @param x the <code>x</code> index for this grid color object
 * @param y the <code>y</code> index for this grid color object
 * @param color the color for this grid color object
 */
public JCGridColor(int x, int y, Color color)
{
	this.dataIndex        = new JCData3dGridIndex(x, y);
	this.dataIndex.parent = this;
	this.color            = color;
}

/**
 * Returns the <code>JCData3dGridIndex</code> object that contains the <i>x</i>
 * and <i>y</i> index for this grid color.
 *
 * @return the current data index
 */
public JCData3dGridIndex getDataIndex()
{
	return(dataIndex);
}

/**
 * Sets the <code>JCData3dGridIndex</code> object that contains the <i>x</i>
 * and <i>y</i> index for this grid color.
 *
 * @param dataIndex the new data index
 */
public void setDataIndex(JCData3dGridIndex dataIndex)
{
	if (this.dataIndex == dataIndex) {
		return;
	}
	this.dataIndex = dataIndex;
	this.dataIndex.parent = this;
	setChanged(true, Changeable.REDRAW);
}

/**
 * Returns the <i>x</i> index for this grid color.  This is a convenience method
 * that only returns the <i>x</i> index of the grid color's
 * <code>JCData3dGridIndex</code> object.
 *
 * @return the grid color's <i>x</i> index
 */
public int getX()
{
	return(dataIndex.getX());
}

/**
 * Sets the <i>x</i> index for this grid color.  This is a convenience method
 * that only sets the <i>x</i> index of the grid color's
 * <code>JCData3dGridIndex</code> object.
 *
 * @param x the new <i>x</i> index
 */
public void setX(int x)
{
	dataIndex.setX(x);
}

/**
 * Returns the <i>y</i> index for this grid color.  This is a convenience method
 * that only returns the <i>y</i> index of the grid color's
 * <code>JCData3dGridIndex</code> object.
 *
 * @return the grid color's <i>y</i> index
 */
public int getY()
{
	return(dataIndex.getY());
}

/**
 * Returns the <i>y</i> index for this grid color.  This is a convenience method
 * that only returns the <i>y</i> index of the grid color's
 * <code>JCData3dGridIndex</code> object.
 *
 * @param y the new <i>y</i> index
 */
public void setY(int y)
{
	dataIndex.setY(y);
}

/**
 * Gets the color used to draw the bar (or row/column of bars) referenced by the
 * <i>x</i> and <i>y</i> of the <code>JCData3dGridIndex</code>.
 *
 * @return the color that bars referenced by this grid color should be drawn
 */
public Color getColor()
{
	return(color);
}

/**
 * Sets the color used to draw the bar (or row/column of bars) referenced by the
 * <i>x</i> and <i>y</i> of the <code>JCData3dGridIndex</code>.
 *
 * @param color the new color used to draw bars referenced by this grid color
 */
public void setColor(Color color)
{
	if (this.color == color) {
		return;
	}
	this.color = color;
	setChanged(true, Changeable.REDRAW);
}

/**
 * Tells the parent (if there is one) that a change has happened.
 *
 * @param b Is there a change?
 * @param cf the change mask that tells which type of change
 */
protected void setChanged(boolean b, int cf)
{
	if (parent != null) {
		parent.setChanged(b, cf);
	}
}

}
