/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCProjection.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:12:58 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

/**
 * <code>JCProjection</code> is a class that controlls how floor and ceiling projections
 * are drawn on the plotcube.
 */
public class JCProjection implements java.io.Serializable {

static final long serialVersionUID = -5542417106751358122L;

/** Asks if contour lines are drawn on this projection. */
protected boolean contoured = false;

/** Asks if colored contour zones are drawn on this projection. */
protected boolean zoned     = false;

/** Parent that keeps track of changes in this object. */
protected ParentTrackChange parent = null;

/**
 * No args constructor.  Values at their default.
 */
public JCProjection()
{
}

/**
 * Constructor that sets the value of contoured and zoned.
 *
 * @param contoured Are contour lines drawn on the projection?
 * @param zoned Are colored contour zones drawn on the projection?
 */
public JCProjection(boolean contoured, boolean zoned)
{
    this.contoured = contoured;
    this.zoned     = zoned;
}

/**
 * Sets whether this projection is contoured or not.
 *
 * @param contoured the new contoured value
 */
public void setContoured(boolean contoured)
{
	if (this.contoured == contoured) {
		return;
	}
    this.contoured = contoured;
	setChanged(true, Changeable.REDRAW);
}

/**
 * Asks if this projection is contoured.
 *
 * @return whether or not the projection is contoured
 */
public boolean isContoured()
{
    return(contoured);
}

/**
 * Sets whether this projection is zoned or not.
 *
 * @param zoned the new zoned value
 */
public void setZoned(boolean zoned)
{
	if (this.zoned == zoned) {
		return;
	}
    this.zoned = zoned;
	setChanged(true, Changeable.REDRAW);
}

/**
 * Asks if this projection is zoned.
 *
 * @return whether or not the projection is zoned
 */
public boolean isZoned()
{
    return(zoned);
}

/**
 * Tells the parent (if there is one) that a change has happened.
 *
 * @param b Has a change happened?
 * @param cf change mask that determines the type of change
 */
protected void setChanged(boolean b, int cf)
{
	if (parent != null) {
		parent.callParentSetChanged(b, cf);
	}
}

}
