/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCStyle.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import java.awt.Color;



/**
 * The base class for <code>JCLineStyle</code>, <code>JCFillStyle</code>, and
 * <code>JCSymbolStyle</code>.
 */
public class JCStyle implements java.io.Serializable {

/** Parent that tracks changes in this object. */
protected ParentTrackChange parent = null;

// Used to determine whether a fill style, line style, or symbol style is used.
protected static final int FILL_STYLE   = 1;
protected static final int LINE_STYLE   = 2;
protected static final int SYMBOL_STYLE = 3;

/** A static array of default colors. */
private static Color standardDefault[] = {
    Color.red,
    Color.orange,
    Color.blue,
    Color.lightGray,
    Color.magenta,
    Color.yellow,
    Color.gray,
    Color.green,
    Color.darkGray,
    Color.cyan,
    Color.black,
    Color.pink,
    Color.white,
};

/** The default color list (intialized to the standard default array). */
protected static Color defaultColors[] = standardDefault;

/**
 * Default constructor.
 */
JCStyle()
{
}

/**
 * Allows access to the default color list.
 * @return color list
 */
public static Color[] getDefaultColors()
{
	return(defaultColors);
}

/**
 * Uses a new default color array.
 * @param newColors the new default colors array
 */
public static void setDefaultColors(Color[] newColors)
{
	if (newColors == null || newColors.length < 1) {
		defaultColors = standardDefault;
	}
	else {
		defaultColors = newColors;
	}
}

/**
 * Tells the parent (if there is one) that a change has happened.
 *
 * @param b Has a change happened?
 * @param cf changes the mask that determines the type of change
 */
void setChanged(boolean b, int cf)
{
	if (parent != null) {
		parent.callParentSetChanged(b, cf);
	}
}

/**
 * Returns the next default color in the default color array for either the
 * line style, fill style, or symbol style array.
 * @return next default color
 * @param c
 * @param style
 */
static Color getDefaultColor(JCChart3d c, int style)
{
	Color rval;

	if (c == null || style < 1 || style > SYMBOL_STYLE) {
		rval = defaultColors[0];
	}
	else {
		int localIndex = c.getColorIndex(style);
		rval = defaultColors[localIndex++];
		if (localIndex >= defaultColors.length) {
			localIndex = 0;
		}
		JCChart3dArea area = c.getChart3dArea();
		if (area != null) {
			JCPlotCube plotCube = area.getPlotCube();
			if (JCChart3dUtil.colorsNear(rval, plotCube.getDrawBackground())) {
				rval = defaultColors[localIndex++];
				if (localIndex >= defaultColors.length) {
				    localIndex = 0;
				}
			}
		}
		c.setColorIndex(localIndex, style);
	}
	return(rval);
}

}
