/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCSymbolStyle.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import com.klg.jclass.chart3d.resources.LocaleBundle;
import com.klg.jclass.chart3d.shape.JCSymbolShape;

import java.awt.*;
import java.util.Hashtable;

/**
 * <code>JCSymbolStyle</code> controls the appearance of points in a scatter
 * plot by allowing the modification of the symbol shape, symbol color, and
 * symbol size.
 */
public class JCSymbolStyle extends JCStyle {

/** The symbol shape. */
protected String shape      = DOT;

/** The index into the default color for this object. */
protected int    colorIndex = 0;

/** The index into the default shape array for this object. */
protected int    shapeIndex = 0;

/** The symbol color. */
protected Color  color      = null;

/** The symbol size in pixels. */
protected int    size       = DEFAULT_SIZE;

/** The default size of a symbol. */
public static final int DEFAULT_SIZE = 6;

/** Keeps a static table of symbol instances. */
public static Hashtable symbolInstances = new Hashtable();

// Symbol predefined shapes
/** The following are predefined shapes for the Java2D implementation. */
public final static String NONE       = null;
public final static String DOT        = "com.klg.jclass.chart3d.shape.Dot";
public final static String BOX        = "com.klg.jclass.chart3d.shape.Box";
public final static String TRIANGLE   = "com.klg.jclass.chart3d.shape.Triangle";
public final static String DIAMOND    = "com.klg.jclass.chart3d.shape.Diamond";
public final static String STAR       = "com.klg.jclass.chart3d.shape.Star";
public final static String VERT_LINE  = "com.klg.jclass.chart3d.shape.VerticalLine";
public final static String HORIZ_LINE = "com.klg.jclass.chart3d.shape.HorizontalLine";
public final static String CROSS      = "com.klg.jclass.chart3d.shape.Cross";
public final static String CIRCLE     = "com.klg.jclass.chart3d.shape.Circle";
public final static String SQUARE     = "com.klg.jclass.chart3d.shape.Square";
public final static String RECTANGLE  = "com.klg.jclass.chart3d.shape.Rectangle";

/** The following are predefined shapes for the Java3D implementation. */
public static final String SPHERE =      "com.klg.jclass.chart3d.shape3d.Sphere";
public static final String CUBE =        "com.klg.jclass.chart3d.shape3d.Cube";
public static final String CONE =        "com.klg.jclass.chart3d.shape3d.Cone";
public static final String POINT =       "com.klg.jclass.chart3d.shape3d.Point";
public static final String CYLINDER =    "com.klg.jclass.chart3d.shape3d.Cylinder";
public static final String TETRAHEDRON = "com.klg.jclass.chart3d.shape3d.Tetrahedron";

private static String[] standard2dShapes = {
	DOT, BOX, TRIANGLE, DIAMOND, STAR, VERT_LINE, HORIZ_LINE, CROSS,
    CIRCLE, SQUARE, RECTANGLE
};

private static String[] standard3dShapes = {
	SPHERE, CUBE, CONE, CYLINDER, TETRAHEDRON
};

/** The default shape list (intialized to the standard default array). */
protected static String[] defaultShapes = standard2dShapes;

/**
 * Constructor for symbol style objects.
 *
 * @param shape symbol shape (one of <code>NONE</code>, <code>DOT</code>, <code>BOX</code>,
 * <code>TRIANGLE</code>, <code>DIAMOND</code>, <code>STAR</code>, <code>VERT_LINE</code>,
 * <code>HORIZ_LINE</code>, <code>CROSS</code>, <code>CIRCLE</code>, or <code>SQUARE</code>)
 * @param color symbol color
 * @param size symbol size
 */
public JCSymbolStyle(String shape, Color color, int size) {
	setShape(shape);
	this.color = color;
	this.size = size;
}

/**
 * Creates a default symbol style.
 *
 * @param c the chart for which to generate the default symbol style
 * @return <code>JCSymbolStyle</code> object with default color and symbol
 *     style.  The defaults are cycled, so consecutive calls will return
 *     different symbol styles.
 */
public static JCSymbolStyle makeDefault(JCChart3d c)
{
	JCSymbolStyle style =
        new JCSymbolStyle(getDefaultShape(c),
                          getDefaultColor(c, JCStyle.SYMBOL_STYLE),
                          DEFAULT_SIZE);
	int sym = 0;
	int col = 0;
	if (c != null) {
		col = c.getSymbolColorIndex();
		if (col == 0) {
			col = defaultColors.length - 1;
		}
		else {
			col--;
		}

		sym = c.getSymbolShapeIndex();
		if (sym == 0) {
			sym = defaultShapes.length - 1;
		}
		else {
			sym--;
		}
	}
	style.colorIndex = col;
	style.shapeIndex = sym;

	return(style);
}

/**
 * Allows access to the default shape list.
 * @return default shape list
 */
public static String[] getDefaultShapes()
{
	return(defaultShapes);
}

/**
 * Uses a new default 2d shapes array.
 * @param newShapes the new default shapes array
 */
public static void setDefault2dShapes(String[] newShapes)
{
	if (newShapes == null || newShapes.length < 1) {
		defaultShapes = standard2dShapes;
	}
	else {
		defaultShapes = newShapes;
	}
}

/**
 * Uses a new default 3d shapes array.
 * @param newShapes the new default shapes array
 */
public static void setDefault3dShapes(String[] newShapes)
{
	if (newShapes == null || newShapes.length < 1) {
		defaultShapes = standard3dShapes;
	}
	else {
		defaultShapes = newShapes;
	}
}

/**
 * Gets a default shape's index from its String value.
 * @param shape the shape for which to get the index
 * @return the index in the <code>defaultShapes</code> array of the shape (returns -1 if
 *    not found)
 */
public static int shapeToIndex(String shape)
{
	if (shape == null) {
		return(-1);
	}

	for (int i = 0; i < defaultShapes.length; i++) {
		if (defaultShapes[i].equals(shape)) {
			return(i);
		}
	}
	return(-1);
}

/**
 * Draws a symbol at the specified location, using the properties
 * of the <code>JCSymbolStyle</code> instance.
 *
 * @param gc graphics context to use for drawing
 * @param x <i>x</i> position of symbol (in pixels)
 * @param y <i>y</i> position of symbol (in pixels)
 */
public void draw(Graphics gc, int x, int y)
{
	draw(gc, x, y, size, null);
}

/**
 * Draws a symbol at the specified location, using the properties
 * of the <code>JCSymbolStyle</code> instance.
 *
 * @param gc graphics context to use for drawing
 * @param x <i>x</i> position of symbol (in pixels)
 * @param y <i>y</i> position of symbol (in pixels)
 * @param col the color to draw with
 */
public void draw(Graphics gc, int x,int y, Color col)
{
	draw(gc, x, y, size, col);
}

/**
 * Draws a symbol at the specified location, using the properties
 * of the <code>JCSymbolStyle</code> instance.
 *
 * @param gc graphics context to use for drawing
 * @param x <i>x</i> position of symbol (in pixels)
 * @param y <i>y</i> position of symbol (in pixels)
 * @param sz size of the symbol
 */
public void draw(Graphics gc, int x, int y, int sz)
{
	draw(gc, x, y, sz, null);
}

/**
 * Draws a symbol at the specified location, using the properties
 * of the <code>JCSymbolStyle</code> instance.
 *
 * @param gc graphics context to use for drawing
 * @param x <i>x</i> position of symbol (in pixels)
 * @param y <i>y</i> position of symbol (in pixels)
 * @param sz size of the symbol
 * @param col the color to draw with
 */
public void draw(Graphics gc, int x, int y, int sz, Color col)
{
	Color thisColor;
	if (col != null) {
		thisColor = col;
	}
	else {
		thisColor = color;
	}

	gc.setColor(thisColor);

    JCSymbolShape symbolShape = getSymbolShapeInstance(shape);
    if (symbolShape != null) {
        symbolShape.resize(sz);
        symbolShape.draw(gc, x, y);
    }
}

/**
 * Gets the <code>Shape</code> property. The <code>Shape</code> property
 * determines the shape of symbol that will be drawn.
 *
 * @return one of <code>NONE</code>, <code>DOT</code>, <code>BOX</code>, <code>TRIANGLE</code>,
 * <code>DIAMOND</code>, <code>STAR</code>, <code>VERT_LINE</code>, <CODE>HORIZ_LINE</code>,
 * <code>CROSS</code>, <code>CIRCLE</code>, <code>SQUARE</code> or <code>RECTANGLE</code>
 */
public String getShape()
{
    return(shape);
}

/**
 * Sets the <code>Shape</code> property. The <code>Shape</code> property
 * determines the shape of symbol that will be drawn.
 *
 * @param shape one of <code>NONE</code>, <code>DOT</code>, <code>BOX</code>, <code>TRIANGLE</code>,
 * <code>DIAMOND</code>, <code>STAR</code>, <code>VERT_LINE</code>, <CODE>HORIZ_LINE</code>,
 * <code>CROSS</code>, <code>CIRCLE</code>, <code>SQUARE</code> or <code>RECTANGLE</code>
 */
public void setShape(String shape)
{
	if (this.shape.equals(shape)) {
        return;
	}

	if (shape == null) {
		throw new IllegalArgumentException(
            LocaleBundle.string(LocaleBundle.INVALID_VALUE_ERROR));
	}
    this.shape = shape;
	setChanged(true, Changeable.REDRAW | Changeable.NOTIFY_LEGEND_REDRAW);
}

/**
 * Gets the <code>Color</code> property of <code>JCSymbolStyle</code>, which
 * determines the color used to paint the symbols.
 *
 * @return AWT Color class representing the color to be used to paint the
 *     symbols. If null, the current color of the <code>Graphics</code> object is used.
 */
public Color getColor()
{
    return(color);
}

/**
 * Sets the <code>Color</code> property of <code>JCSymbolStyle</code>, which
 * determines the color used to paint the symbols.
 *
 * @param color AWT Color class representing the color to be used to paint the
 *      symbols. If null, the current color of the <code>Graphics</code> object is used.
 */
public void setColor(Color color) {
	if (this.color == color) {
        return;
	}
	this.color = color;

	setChanged(true, Changeable.REDRAW | Changeable.NOTIFY_LEGEND_REDRAW);
}

/**
 * Gets the <code>Size</code> property.
 * The <code>Size</code> property determines the size of the symbols. <br><B>Note:</b>
 * A value of zero size means the symbol will not be drawn.
 *
 * @return positive integer representing size
 */
public int getSize()
{
    return(size);
}

/**
 * Sets the <code>Size</code> property of <code>JCSymbolStyle</code>, which
 * determines the size of the symbols. <br><b>Note:</b> A value of zero size means
 * the symbol will not be drawn.
 *
 * @param size positive integer representing size
 */
public void setSize(int size)
{
	if (this.size == size) {
        return;
	}
	this.size = size;
	setChanged(true, Changeable.RECALC | Changeable.NOTIFY_LEGEND_REDRAW);
}

/**
 * Gets the current default shape for this chart based on its default shape
 * index.
 * @param c
 * @return the index of the default shape to be used
 */
protected static String getDefaultShape(JCChart3d c)
{
	String rval;
	if (c == null) {
		rval = defaultShapes[0];
	}
	else {
		int localIndex = c.getSymbolShapeIndex();
		localIndex++;
		if (localIndex >= defaultShapes.length) {
            localIndex = 0;
		}
		rval = defaultShapes[localIndex];
		c.setSymbolShapeIndex(localIndex);
	}
	return(rval);
}

/**
 * Returns the index used to generate the color.  Only set if default colors
 * are used.
 *
 * @return the current index into the default color array
 */
public int getColorIndex()
{
	return(colorIndex);
}

/**
 * Gets an instance of a symbol shape object for a given shape.  Look it up
 * in the <code>hashTable</code>. If it exists in the table, use it. If not, create a
 * new instance and store it in the table.
 *
 * @param shape the shape whose value points to a class to instantiate
 * @return an object which implements the <code>JCSymbolShape</code> interface, ready for
 *     use
 */
public static JCSymbolShape getSymbolShapeInstance(String shape)
{
    if (shape == null) {
        return(null);
    }
    Object symbolShapeInstance = symbolInstances.get(shape);
    if (symbolShapeInstance == null) {
        try {
            symbolShapeInstance = Class.forName(shape).newInstance();
        }
        // Unable to create new instance, for various reasons
        catch (ClassNotFoundException e) {
            //e.printStackTrace();
        }
        catch (InstantiationException e1) {
        }
        catch (IllegalAccessException e1) {
        }
        if (symbolShapeInstance instanceof JCSymbolShape) {
            symbolInstances.put(shape, symbolShapeInstance);
        }
    }
    if (symbolShapeInstance instanceof JCSymbolShape) {
        return((JCSymbolShape)symbolShapeInstance);
    }
    return(null);
}

}
