/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCView3d.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:12:58 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

/**
 * The <code>JCView3d</code> class holds information about the perspective and the rotation
 * angles at which the users view the data in the plotcube.
 */
public class JCView3d extends ParentTrackChange {

static final long serialVersionUID = -5542417106751358122L;

/** The minimum allowed perspective value. */
public final static double MIN_PERSPECTIVE = 1.0;

/** The maximum allowed perspective value. */
public final static double MAX_PERSPECTIVE = 500.0;

/**
 * The plot cube perspective value (defaults to 2.5).
 */
protected double perspective = 2.5;

/**
 * The <i>X</i> rotation angle.
 */
protected double xRotation = 45.0;

/**
 * The <i>Y</i> rotation angle.
 */
protected double yRotation = 0.0;

/**
 * The <i>Z</i> rotation angle.
 */
protected double zRotation = 45.0;

/**
 * No args constructor.  Values at their default.
 */
public JCView3d()
{
}

/**
 * Constructor that provides perspective and <i>x</i>, <i>y</i>, and <i>z</i>
 * rotation values.
 *
 * @param perspective the plot cube perspective value
 * @param xRotation the <i>X</i> rotation angle
 * @param yRotation the <i>Y</i> rotation angle
 * @param zRotation the <i>Z</i> rotation angle
 */
public JCView3d(double perspective, double xRotation, double yRotation,
	            double zRotation)
{
    this.perspective = perspective;
    this.xRotation   = xRotation;
    this.yRotation   = yRotation;
    this.zRotation   = zRotation;
}

/**
 * Sets the plot cube perspective.
 *
 * @param perspective the plot cube perspective
 */
public void setPerspective(double perspective)
{
	perspective = JCChart3dUtil.clamp(perspective, MIN_PERSPECTIVE,
									  MAX_PERSPECTIVE);
	if (this.perspective == perspective) {
		return;
	}
	this.perspective = perspective;
	callParentSetChanged(true, Changeable.RECALC);
}

/**
 * Returns the plot cube perspective.
 *
 * @return the plot cube perspective
 */
public double getPerspective()
{
    return(perspective);
}

/**
 * Sets the <i>X</i> rotation angle.
 *
 * @param xRotation the <i>X</i> rotation angle
 */
public void setXRotation(double xRotation)
{
	if (this.xRotation == xRotation) {
		return;
	}
    this.xRotation = xRotation;
	callParentSetChanged(true, Changeable.RECALC);
}

/**
 * Returns the <i>X</i> rotation angle.
 *
 * @return the <i>X</i> rotation angle
 */
public double getXRotation()
{
    return(xRotation);
}

/**
 * Sets the <i>Y</i> rotation angle.
 *
 * @param yRotation the <i>Y</i> rotation angle
 */
public void setYRotation(double yRotation)
{
	if (this.yRotation == yRotation) {
		return;
	}
    this.yRotation = yRotation;
	callParentSetChanged(true, Changeable.RECALC);
}


/**
 * Returns the <i>Y</i> rotation angle.
 *
 * @return the <i>Y</i> rotation angle
 */
public double getYRotation()
{
    return(yRotation);
}

/**
 * Sets the <i>Z</i> rotation angle.
 *
 * @param zRotation the <i>Z</i> rotation angle
 */
public void setZRotation(double zRotation)
{
	if (this.zRotation == zRotation) {
		return;
	}
    this.zRotation = zRotation;
	callParentSetChanged(true, Changeable.RECALC);
}

/**
 * Returns the <i>Z</i> rotation angle.
 *
 * @return the <i>Z</i> rotation angle
 */
public double getZRotation()
{
    return(zRotation);
}

/**
 * Sets all three rotation angles at once (with only one update).
 *
 * @param xRotation the <i>X</i> rotation angle
 * @param yRotation the <i>Y</i> rotation angle
 * @param zRotation the <i>Z</i> rotation angle
 */
public void setXYZRotation(double xRotation, double yRotation, double zRotation)
{
	// Don't put in check for equality here as the actions wants a redraw
	// even if the angles are the same as before
    this.xRotation = xRotation;
    this.yRotation = yRotation;
    this.zRotation = zRotation;
	callParentSetChanged(true, Changeable.RECALC);
}

}
