/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Justify.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import com.klg.jclass.chart3d.resources.LocaleBundle;

/** 
 * Class internal to JClass Chart 3D.  Used to store information about
 * annotation placment and justification.
 */
public class Justify {

/** Rotates annotation 0, 90, 180, or 270 degrees. */
protected int rotate = Transform.ROTATE_NONE;

/** Moves annotation above the axis. */
public boolean above = false;

/**
 * Draws annotation vertical (orthogonal to axis) or horizontal (parallel to
 * axis).
 */
protected boolean vertical = false;

/** Rotates 180 degrees about <i>X</i> axis. */
protected boolean flipX = false;

/** Rotates 180 degrees about <i>Y</i> axis. */
protected boolean flipY = false;

/** Rotates title 180 degrees about the <i>Z</i> axis. */
protected boolean spinTitle = false;

/** The amount of space between the axis and the extent of the annotation. */
protected double  easeway = 0.0;

/** 
 * Null constructor.  All values at default.
 * 
 */
public Justify() {}

/** 
 * Returns whether to rotate text 0, 90, 180, or 270 degrees.
 * @return Text rotation enum.  One of <code>Transform.ROTATE_NONE</code>,
 * <code>Transform.ROTATE_90</code>, <code>Transform.ROTATE_180</code>, or
 * <code>Transform.ROTATE_270</code>.
 */
public int getRotate()
{
	return(rotate);
}

/** 
 * Sets whether to rotate text 0, 90, 180, or 270 degrees.
 * 
 * @param rotate enum which indicates whether to rotate text 0, 90, 180, or
 *     270 degrees
 */
public void setRotate(int rotate)
{
	if (rotate < Transform.ROTATE_NONE || rotate > Transform.ROTATE_270) {
	    throw new IllegalArgumentException(
            LocaleBundle.string(LocaleBundle.INVALID_VALUE_ERROR));
	}
	this.rotate = rotate;
}

/** 
 * Returns whether the annotation is above the axis.
 * 
 * @return Is the annotation above the axis?
 */
public boolean isAbove()
{
	return(above);
}

/** 
 * Sets whether the annotation is above the axis.
 * 
 * @param above <code>true</code> if the annotation is above the axis
 */
public void setAbove(boolean above)
{
	this.above = above;
}

/** 
 * Returns whether the annotation is orthogonal to the axis (vertical is
 * <code>true</code>) or parallel to the axis (vertical is <code>false</code>).
 * 
 * @return Is the annotation orthogonal or parallel to the axis?
 */
public boolean isVertical()
{
	return(vertical);
}

/** 
 * Sets whether the annotation is orthogonal to the axis (vertical is
 * <code>true</code>) or parallel to the axis (vertical is <code>false</code>).
 * 
 * @param vertical <code>true</code> if the annotation is orthogonal to the axis
 */
protected void setVertical(boolean vertical)
{
	this.vertical = vertical;
}

/** 
 * Returns whether to rotate the annotation 180 degrees about the <i>X</i> axis.
 * 
 * @return Rotate the annotation 180 degrees about the <i>X</i> axis?
 */
public boolean doFlipX()
{
	return(flipX);
}

/** 
 * Sets whether to rotate the annotation 180 degrees about the <i>X</i> axis.
 * 
 * @param flipX <code>true</code> if the annotation is rotated 180 about the
 * <i>X</i> axis
 */
public void setFlipX(boolean flipX)
{
	this.flipX = flipX;
}

/** 
 * Returns whether to rotate the annotation 180 degrees about the <i>Y</i> axis.
 * 
 * @return Rotate the annotation 180 degrees about the <i>Y</i> axis?
 */
public boolean doFlipY()
{
	return(flipY);
}

/** 
 * Sets whether to rotate the annotation 180 degrees about the <i>Y</i> axis.
 * 
 * @param flipY <code>true</code> if the annotation is rotated 180 about the
 * <i>Y</i> axis
 */
public void setFlipY(boolean flipY)
{
	this.flipY = flipY;
}

/** 
 * Returns whether to spin the title 180 degrees about the <i>Z</i> axis.
 * 
 * @return Spin the title 180 degrees about the <i>Z</i> axis?
 */
public boolean doSpinTitle()
{
	return(spinTitle);
}

/** 
 * Sets whether to to spin the title 180 degrees about the <i>Z</i> axis.
 * 
 * @param spinTitle <code>true</code> if the title is to be rotated 180 degrees
 *     about the <i>Z</i> axis
 */
public void setSpinTitle(boolean spinTitle)
{
	this.spinTitle = spinTitle;
}

/** 
 * Returns the amount of space between the axis and the extent of the
 * annotation.
 * 
 * @return the current value of easeway
 */
protected double getEaseway()
{
	return(easeway);
}

/** 
 * Sets the amount of space between the axis and the extent of the annotation.
 * 
 * @param easeway the new value of easeway
 */
public void setEaseway(double easeway)
{
	this.easeway = easeway;
}

}
