/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: KeyActionInitiator.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import java.awt.AWTEvent;
import java.awt.event.KeyEvent;
import java.io.Serializable;

/**
 * A <CODE>KeyActionInitiator</CODE> is used by <CODE>JCActionTable</CODE> to
 * associate a combination of key code and modifiers to a specific action 
 * on Chart 3D.
 */
public class KeyActionInitiator extends ActionInitiator
                                implements Comparable, Serializable
{

static final long serialVersionUID = -8443878208369210093L;

/** Key code. */
public int keycode;

/** Modifier for the key code. */
public int modifiers;

/**
 * Creates a new <CODE>KeyActionInitiator</CODE> given a key and modifiers.
 * <P>Valid modifierss are:
 * <UL>
 * <LI><CODE>InputEvent.ALT_MASK</CODE></LI>
 * <LI><CODE>InputEvent.CTRL_MASK</CODE></LI>
 * <LI><CODE>InputEvent.SHIFT_MASK</CODE></LI>
 * <LI><CODE>ActionInitiator.NO_MODIFIER_MASK</CODE></LI>
 * <LI><CODE>ActionInitiator.ANY_MODIFIER_MASK</CODE></LI>
 * </UL>
 */
public KeyActionInitiator(int keycode, int modifiers)
{
    this.keycode = keycode;
    this.modifiers = modifiers;
}

/**
 * Returns true if the given event matches the ActionInitiator settings.
 */
public boolean isMatch(AWTEvent e)
{
    if (!(e instanceof KeyEvent)) {
        return(false);
    }
    KeyEvent ke = (KeyEvent)e;

    if (modifiers == ANY_MODIFIER_MASK) {
        return(keycode == ke.getKeyCode());
    }
    else {
        return(keycode == ke.getKeyCode() && modifiers == ke.getModifiers());
    }
}

/**
 * Implementation of Comparable interface.
 */
public int compareTo(Object obj)
{
    int difference = 0;
    if (obj instanceof KeyActionInitiator) {
        KeyActionInitiator kai = (KeyActionInitiator)obj;
        difference = this.keycode - kai.keycode;
        if (difference == 0) {
            difference = this.modifiers - kai.modifiers;
        }
    }
    return(difference);
}

/**
 * Convert to a string.
 */
public String toString()
{
    return("Keycode=" + keycode + " Modifier=" + modifiers);
}

}
