/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: LabelledChart3dGridDataModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

/**
 * An interface used to specify <i>x</i>labels and <i>y</i>labels for a JClass
 * Chart 3D data model.  <i>x</i>labels and <i>y</i>labels are used for the
 * <i>x</i> and <i>y</i> axis respectively if the annotation type of the axis
 * is JCAxis.ANNOTATION_DATA_LABELS (@see com.klg.jclass.chart3d.JCAxis for
 * details). This interface also specifies the number of <i>X</i> grid and
 * <i>Y</i> grid values.
 *
 * <P>Note that the <code>LabelledChart3dGridDataModel</code> interface
 * is only used in conjunction with the <code>Chart3dDataModel</code>
 * interface, that is, in order for an object to be recognized as a chart
 * data source, it needs to implement the <code>Chart3dDataModel</code>
 * interface.
 *
 * @see Chart3dDataModel
 */
public interface LabelledChart3dGridDataModel extends LabelledChart3dDataModel {

/**
 * Retrieves the <i>X</i> labels to be used for the <i>X</i> axis if its
 * annotation type isJCAxis.ANNOTATION_DATA_LABELS.
 *
 * @return An array of <i>X</i> labels for a data model implementing
 *   this interface
 */
public String[] getXLabels();

/**
 * Retrieves the <i>Y</i> labels to be used for the <i>Y</i> axis if its
 * annotation type is JCAxis.ANNOTATION_DATA_LABELS.
 *
 * @return An array of <i>Y</i> labels for a data model implementing
 *   this interface
 */
public String[] getYLabels();

/**
 * Retrieves the number of <i>X</i> grid values.
 *
 * @return The number of <i>X</i> grid values for a data model implementing
 *   this interface
 */
public int getNumX();

/**
 * Retrieves the number of <i>Y</i> grid values.
 *
 * @return The number of <i>Y</i> grid values for a data model implementing
 *   this interface
 */
public int getNumY();

}


