/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: LegendEntry.java,v $ $Revision: 1.2 $
// $Date: 2006-03-17 21:12:58 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import com.klg.jclass.chart3d.resources.LocaleBundle;

/**
 * This is laregely for internal use.  It encapsulates a legend entry for
 * a 3d legend. Usually, this consists of a data index, a <code>drawStyle</code>
 * and a parent draw style.
 */
public class LegendEntry implements java.io.Serializable
{

/**
 * Encapsulates data index, either contour range or point data series,
 * that is referenced by this legend entry.
 */
protected JCData3dIndex dataIndex = null;

/**
 * Primary draw style.  Likely one of <code>JCLineStyle</code>,
 * <code>JCFillStyle</code> or <code>JCSymbolStyle</code>.
 */
protected Object drawStyle = null;

/**
 * Parent draw style of the primary draw style.  This is used to retrieve
 * secondary drawing information, such as the colour of drop lines.
 * This object is likely one of <code>JCContourStyle</code> or
 * <code>JCChart3dStyle</code>.
 */
protected Object parentStyle = null;

/**
 * Constructor with data index and parent draw style.
 *
 * @param index The data index used for this legend entry
 * @param parent The data index used for this legend entry
 */
public LegendEntry(JCData3dIndex index, Object parent)
{
    dataIndex   = index;
    parentStyle = parent;
}

/**
 * Constructor with data index, parent draw style and specific draw style.
 *
 * @param index The data index used for this legend entry
 * @param parent The data index used for this legend entry
 * @param draw The drawstyle to be used
 */
public LegendEntry(JCData3dIndex index, Object parent, Object draw)
{
    dataIndex   = index;
    parentStyle = parent;
    drawStyle   = draw;
}

/**
 * Return a string version of this object.
 */
public String toString()
{
	if (dataIndex instanceof JCData3dContourIndex) {
		JCData3dContourIndex contourIndex =
			(JCData3dContourIndex) dataIndex;
		return("LegendEntry: " + " " +
            LocaleBundle.string(LocaleBundle.LEVEL_LABEL) + " " +
            contourIndex.getContourStyleIndex() + " " +
            LocaleBundle.string(LocaleBundle.MIN_LABEL) + " " +
            contourIndex.getLowerContourRangeValue() + " " +
            LocaleBundle.string(LocaleBundle.MAX_LABEL) + " " +
            contourIndex.getUpperContourRangeValue() + " " +
            LocaleBundle.string(LocaleBundle.PARENT_STYLE_LABEL) + " " +
            parentStyle + " " +
            LocaleBundle.string(LocaleBundle.DRAW_STYLE_LABEL) + " " +
            drawStyle);
	}
    else if (dataIndex instanceof JCData3dPointIndex) {
		JCData3dPointIndex pointIndex =
			(JCData3dPointIndex) dataIndex;
		return("LegendEntry: " + " " +
            LocaleBundle.string(LocaleBundle.SERIES_LABEL) + " " +
            pointIndex.getSeries() + " " +
            LocaleBundle.string(LocaleBundle.POINT_LABEL) + " " +
            pointIndex.getPoint() + " " +
            LocaleBundle.string(LocaleBundle.PARENT_STYLE_LABEL) + " " +
            parentStyle + " " +
            LocaleBundle.string(LocaleBundle.DRAW_STYLE_LABEL) + " " +
            drawStyle);
	}
    else {
		return("LegendEntry: " + " " +
            LocaleBundle.string(LocaleBundle.PARENT_STYLE_LABEL) + " " +
            parentStyle + " " +
            LocaleBundle.string(LocaleBundle.DRAW_STYLE_LABEL) + " " +
            drawStyle);
	}
}

}
