/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Mapping.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import javax.vecmath.Point3d;

/** 
 * The mapping interface for JClass Chart 3D.
 * 
 */
public interface Mapping {

/** 
 * Performs a map operation by mapping the provided screen pixel
 * information to a value in data space.
 * 
 * @param x <i>x</i> value in screen pixels
 * @param y <i>y</i> value in screen pixels
 * @return The data point that maps to the given screen point
 */
public Point3d map(int x, int y);

/** 
 * Performs an unmap operation by transforming the given data point to the
 * position on the screen where it would be drawn.
 * 
 * @param point The data point to be unmapped.
 * @return The screen point to which the data point maps
 */
public java.awt.Point unmap(Point3d point);

/** 
 * Performs a pick operation by returning the index of the closest point in
 * the data set to the given screen point.
 * 
 * @param x <i>x</i> value in screen pixels
 * @param y <i>y</i> value in screen pixels
 */
public JCData3dIndex pick(int x, int y);

/** 
 * Performs an unpick operation by returning the screen point that corresponds
 * to the given data index.
 * 
 * @param index The data index of the point whose screen point is required.
 */
public java.awt.Point unpick(JCData3dIndex index);

/** 
 * Returns the internal data object whose type determines the type of
 * <code>MapPick</code> object required.
 */
public Chart3dData getData();

/**
 * Create a mapping instance based on the data type
 * @param data The data on which to base the creation
 * @return The new mapping instance
 */
public Mapping createMapping(Chart3dData data);

}


