/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: MouseActionInitiator.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import java.awt.AWTEvent;
import java.awt.event.MouseEvent;
import java.io.Serializable;

/**
 * A <CODE>MouseActionInitiator</CODE> is used by <CODE>JCActionTable</CODE> to
 * associate a combination of mouse button and modifiers to a specific action 
 * on Chart 3D.
 */
public class MouseActionInitiator extends ActionInitiator
                                  implements Comparable, Serializable
{

static final long serialVersionUID = 2452909788192167913L;

/** Mouse button code. */
public int buttonMask;

/** Modifier for the mouse button. */
public int modifiers;

public static final int ANY_BUTTON_MASK = -1;

/**
 * Creates a new <CODE>MouseActionInitiator</CODE> given a mouse button
 * and modifiers.
 * <P>Valid buttons are: 
 * <UL>
 * <LI><CODE>MouseEvent.BUTTON1_MASK</CODE></LI>
 * <LI><CODE>MouseEvent.BUTTON2_MASK</CODE></LI> 
 * <LI><CODE>MouseEvent.BUTTON3_MASK</CODE></LI>
 * <LI><CODE>MouseActionInitiator.ANY_BUTTON_MASK</CODE></LI>
 * </UL>
 * <P>Valid modifierss are:
 * <UL>
 * <LI><CODE>InputEvent.ALT_MASK</CODE></LI>
 * <LI><CODE>InputEvent.CTRL_MASK</CODE></LI>
 * <LI><CODE>InputEvent.SHIFT_MASK</CODE></LI>
 * <LI><CODE>ActionInitiator.NO_MODIFIER_MASK</CODE></LI>
 * <LI><CODE>ActionInitiator.ANY_MODIFIER_MASK</CODE></LI>
 * </UL>
 */
public MouseActionInitiator(int buttonMask, int modifiers)
{
    this.buttonMask = buttonMask;
    this.modifiers = modifiers;
}

/**
 * Returns true if the given event matches the ActionInitiator settings.
 */
public boolean isMatch(AWTEvent e)
{
    if (!(e instanceof MouseEvent)) {
        return(false);
    }
    MouseEvent me = (MouseEvent)e;

    // check for match on the button mask
    if (buttonMask == ANY_BUTTON_MASK ||
        (me.getModifiers() & buttonMask) != 0)
    {
        // check for match on the modifierss
        if (modifiers == ActionInitiator.ANY_MODIFIER_MASK ||
            (me.getModifiers() & modifiers) != 0)
        {
            return(true);
        }
        else if (modifiers == ActionInitiator.NO_MODIFIER_MASK &&
                 (me.getModifiers() == buttonMask))
        {
            return(true);
        }
    }
    return(false);
}

/**
 * Implementation of Comparable interface.
 */
public int compareTo(Object obj)
{
    int difference = 0;
    if (obj instanceof MouseActionInitiator) {
        MouseActionInitiator mai = (MouseActionInitiator)obj;
        difference = this.buttonMask - mai.buttonMask;
        if (difference == 0) {
            difference = this.modifiers - mai.modifiers;
        }
    }
    return(difference);
}

/**
 * Convert to a string.
 */
public String toString()
{
    return("Button Mask=" + buttonMask + " Modifier=" + modifiers);
}

}
