/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ParentFormatter.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import java.util.Locale;

/** 
 * An object that keeps number formatting information needed to format axis
 * labels and contour labels.
 * 
 */
abstract public class ParentFormatter extends ParentTrackChange {

transient private JCChart3dNumberFormat   numberFormatter    = null;
protected         int                     nFrac              = 0;
protected         boolean                 numberLocalization = true;
protected         boolean                 groupingUsed       = false;

/** True if decimals in annotations should be padded with zeroes. */
protected boolean decimalPadded  = true;

/** 
 * Gets the internal number formatter for this axis.
 */
public JCChart3dNumberFormat getNumberFormatter()
{
	if (numberFormatter == null) {
		numberFormatter = new JCChart3dNumberFormat(this);
	}
	return(numberFormatter);
}

/** 
 * Sets whether number localization is done to labels generated for this axis.
 * 
 * @param numberLocalization 
 */
public void setNumberLocalization(boolean  numberLocalization)
{
	if (this.numberLocalization == numberLocalization) {
		return;
	}
	this.numberLocalization = numberLocalization;
}

/** 
 * Do number localization to labels generated for this axis?
 */
public boolean getNumberLocalization()
{
	return(numberLocalization);
}

/** 
 * Sets the number of fractional digits in the step.
 * 
 * @param nFrac The number of fractional digits in the step
 */
protected void setNFrac(int nFrac)
{
	this.nFrac = nFrac;
}

/** 
 * Gets the number of fractional digits in the step.
 * 
 * @return The number of fractional digits in the step
 */
protected int getNFrac()
{
	return(nFrac);
}

/** 
 * Sets the value of the <code>GroupingUsed</code> property.  
 * The <code>GroupingUsed</code> property
 * determines whether or not grouping  will be used in formatting numbers.
 * Default value is false
 * 
 * @param groupingUsed 
 */
public void setGroupingUsed(boolean groupingUsed)
{
	if (this.groupingUsed == groupingUsed) {
		return;
	}
    this.groupingUsed = groupingUsed;
	callParentSetChanged(true, Changeable.RECALC);
}

/** 
 * Gets the value of the <code>GroupingUsed</code> property.  
 * The <code>GroupingUsed</code> property
 * determines whether or not grouping will be used in formatting numbers.
 * 
 */
public boolean getGroupingUsed()
{
    return(groupingUsed);
}

/** 
 * Sets the <code>DecimalPadded</code> property.
 * The <code>DecimalPadded</code> property determines whether the axis
 * annotations will have their decimal portions padded with zeroes to the size
 * of the precision.
 * 
 * @param b if true, the axis annotations will be decimal padded.
 */
public void setDecimalPadded(boolean b)
{
	decimalPadded = b;
}

/** 
 * Gets the <code>DecimalPadded</code> property.
 * The <code>DecimalPadded</code> property determines whether the axis
 * annotations will have their decimal portions padded with zeroes to the size
 * of the precision.
 * 
 * @return if true, the axis annotations are decimal padded.
 */
public boolean isDecimalPadded()
{
	return(decimalPadded);
}

/** 
 * Sets the value of the number formatting locale. Default value is the
 * current locale.
 * 
 * @param l 
 */
public void setNumberFormatterLocale(Locale l)
{
    getNumberFormatter().setNumberFormatterLocale(l);
	callParentSetChanged(true, Changeable.RECALC);
}

}
