/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ParentTrackChange.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import java.util.ArrayList;

/**
 * An object that keeps a list of <code>Changeable</code> parents and updates
 * them when a change they need to know about happens.
 */
abstract public class ParentTrackChange implements java.io.Serializable {

/**
 * ArrayList of Changeable objects used to hold multiple parents so that and
 * instance of this object may be shared between multiple parents.
 */
protected ArrayList parents = null;

/**
 * Calls the <code>setChanged</code> methods of all of the
 * parents of this <code>ParentTrackChange</code>.
 *
 * @param c
 * @param cf
 */
void callParentSetChanged(boolean c, int cf)
{
	if (parents == null) {
		return;
	}

	for (int i = 0; i < parents.size(); i++) {
        Changeable parent = (Changeable)parents.get(i);
        if (parent != null) {
		    parent.setChanged(c, cf);
        }
	}
}

/**
 * Adds a parent to the list of parents of this <code>ParentTrackChange</code>
 * object.
 *
 * @param parent The <code>Changeable</code> object to add to the parent list.
 */
public void addParent(Changeable parent)
{
	if (parents == null) {
		parents = new ArrayList();
	}

	if (!parents.contains(parent)) {
		parents.add(parent);
	}
}

/**
 * Removes a parent from the list of parents of this
 * <code>ParentTrackChange</code> object.
 *
 * @param parent The <code>Changeable</code> object to remove from the parent
 *     list.
 */
public void removeParent(Changeable parent)
{
	if (parents == null) {
		return;
	}
	parents.remove(parent);
}

/**
 * Clears the list of parents of this <code>ParentTrackChange</code> object.
 *
 */
public void clearParents()
{
	parents = null;
}

}
