/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PolygonChain.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

/**
 * Data structure internal to JClass Chart 3D.  Represents a list of points that
 * define the line segments of a contour line.
 *
 */
public class PolygonChain implements java.io.Serializable {


/**
 * There is a most a two-point <code>PChain</code> for each corner, and a
 * </code>PChain</code> being filled (8 pts. max), for a maximum total of
 * 16 points.  We make it 20 to be safe.
 */
public final static int MAX_CHAIN_SIZE = 20;

/** The <i>X</i> pixel value. */
protected int xPixel = 0;

/** The <i>Y</i> pixel value. */
protected int yPixel = 0;

/** The <i>Z</i> value corresponding to this point (elevation data). */
protected double zValue = 0.0;

/** Contoured value (zone data). */
protected double cValue = 0.0;

/** Next one in chain. */
protected PolygonChain  next = null;

/**  */
PolygonChain() {}

/**
 * Sets all four values at once.
 *
 * @param xPixel
 * @param yPixel
 * @param zValue
 * @param cValue
 */
protected void setValues(int xPixel, int yPixel, double zValue, double cValue)
{
	this.xPixel = xPixel;
	this.yPixel = yPixel;
	this.zValue = zValue;
	this.cValue = cValue;
}

/**
 * Sets the <i>x</i> pixel value for this point.
 *
 * @param xPixel
 */
protected void setXPixel(int xPixel)
{
	this.xPixel = xPixel;
}

/**
 * Retrieves the <i>x</i> pixel value for this point.
 *
 */
public int getXPixel()
{
	return(xPixel);
}

/**
 * Sets the <i>x</i> pixel value for this point.
 *
 * @param yPixel
 */
protected void setYPixel(int yPixel)
{
	this.yPixel = yPixel;
}

/**
 * Retrieves the <i>y</i> pixel value for this point.
 *
 */
public int getYPixel()
{
	return(yPixel);
}

/**
 * Sets the <i>z</i> value.
 *
 * @param zValue
 */
protected void setZValue(double zValue)
{
	this.zValue = zValue;
}

/**
 * Gets the <i>z</i> value.
 *
 */
protected double getZValue()
{
	return(zValue);
}

/**
 * Sets the contoured value.
 *
 * @param cValue
 */
protected void setCValue(double cValue)
{
	this.cValue = cValue;
}

/**
 * Gets the contoured value.
 *
 */
public double getCValue()
{
	return(cValue);
}

/**
 * Setd the reference to the next polygon in this chain.
 *
 * @param next
 */
protected void setNext(PolygonChain next)
{
	this.next = next;
}

/**
 * Getd the next polygon in this chain.
 *
 */
public PolygonChain getNext()
{
	return(next);
}

}
