/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ScatterFacet.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;



import java.awt.*;

/**
 * Render facet at (x,y) as a scatter point
 */
public class ScatterFacet extends FacetAbstract {

protected transient Graphics gc = null;

ScatterFacet() {}

ScatterFacet(DrawGrid drawGrid, Graphics gc)
{
	super(drawGrid);
	this.gc = gc;
}

/**
 * @param x
 * @param y
 * @param edges
 */
public void processFacet(int x, int y, int edges)
{
    Chart3dGridData     data = drawGrid.getGridData();
	ScreenPoint[][]     screenPoints = data.getScreenPoints();
	GridLimits          gridLimits = data.getGridLimits();
	Chart3dDataView     dataView = data.getDataView();
	JCChart3dArea       area = dataView.getChart3d().getChart3dArea();

	if (x < gridLimits.getXOffset() ||
	    x > gridLimits.getXLast()   ||
		y < gridLimits.getYOffset() ||
		y > gridLimits.getYLast())
	{
		// Should never get here
	    
		return;
	}

    if (screenPoints == null) {
        return;
    }

    ScreenPoint sp = screenPoints[x][y];
	if (sp.isDrawn() || sp.isHoleValue() || sp.isHidden()) {
		return;
	}

	// Don't process any points outside chart area due to zooming
    if (!area.getViewport().isNormalized() &&
        area.getScatter().hasDropLines())
    {
		ScreenPoint zp = new ScreenPoint();
		JCAxis axis = area.getAxis(JCAxis.AXIS_Z);
		JCChart3dUtil.getScreenPoint(area.getChart3d(),
							         data.getX(x), data.getY(y),
									 axis.getOrigin(), axis.getMin(), zp);

		int x1 = Math.min(sp.getXPixel(), zp.getXPixel());
        int y1 = Math.min(sp.getYPixel(), zp.getYPixel());
        int x2 = Math.max(sp.getXPixel(), zp.getXPixel());
        int y2 = Math.max(sp.getYPixel(), zp.getYPixel());
        Rectangle rect = new Rectangle(x1, y1, x2 - x1, y2 - y1);

        if (!rect.intersects(area.getDrawingArea())) {
            return;
        }
    }

    Chart3dGridData    zoneData      = dataView.getZoneData();
	JCContour          contour       = dataView.getContour();
	JCContourLevels    contourLevels = contour.getContourLevels();
	JCLineStyle        lineStyle     = null;
	JCSymbolStyle      symbolStyle   = null;
	if (contour.isContoured() || contour.isZoned()) {
	    double value = data.getZValue(x, y);
	    if (zoneData.isDataOK()) {
		    value = zoneData.getZValue(x, y);
		}
		int level = contourLevels.getLevelFromValue(value);
		JCContourStyle contourStyle = contour.getContourStyleFromLevel(level);
		if (contourStyle == null) {
		    return;
		}
		symbolStyle = contourStyle.getSymbolStyle();
		lineStyle   = contourStyle.getLineStyle();
	}
	else {
		JCChart3dStyle chartStyle = data.getChartStyle();
	    if (chartStyle == null) {
		    return;
		}
	    symbolStyle = chartStyle.getSymbolStyle();
	    lineStyle   = chartStyle.getLineStyle();
	}
	drawGrid.renderPoint(gc, symbolStyle, lineStyle, sp,
						 data.getX(x), data.getY(y));
}

}
