/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ScreenPoint.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

/**
 * Data structure internal to JClass Chart 3D.  Used to represent the
 * pixel values and related information of a data point.
 *
 */
public class ScreenPoint implements java.io.Serializable {

/** The <i>X</i> pixel value. */
protected int xPixel = 0;

/** The <i>Y</i> pixel value. */
protected int yPixel = 0;

/** Is point hidden? */
protected boolean hidden = false;

/** Does this point correspond to a hole value in the data? */
protected boolean holeValue = false;

/** Has point been drawn? */
protected boolean drawn = false;

/**
 * Default constructor.  Values at their default.
 */
public ScreenPoint() {}

/**
 * Constructor which specifies the <i>x</i> an <i>y</i> pixel values
 *
 * @param xPixel The <i>x</i> pixel value
 * @param yPixel The <i>y</i> pixel value
 */
public ScreenPoint(int xPixel, int yPixel)
{
    this.xPixel = xPixel;
    this.yPixel = yPixel;
}

/**
 * Resets this point to default values.
 */
public void reset()
{
	xPixel    = 0;
	yPixel    = 0;
	hidden    = false;
	holeValue = false;
	drawn     = false;
}

/**
 * Sets the <i>x</i> pixel value for this point.
 *
 * @param xPixel The <i>x</i> pixel value for this point
 */
public void setXPixel(int xPixel)
{
	this.xPixel = xPixel;
}

/**
 * Retrieves the <i>x</i> pixel value for this point.
 *
 * @return The <i>x</i> pixel value for this point
 */
public int getXPixel()
{
	return(xPixel);
}

/**
 * Sets the <i>y</i> pixel value for this point.
 *
 * @param yPixel The <i>y</i> pixel value for this point
 */
public void setYPixel(int yPixel)
{
	this.yPixel = yPixel;
}

/**
 * Retrieves the <i>y</i> pixel value for this point.
 *
 * @return The <i>y</i> pixel value for this point
 */
public int getYPixel()
{
	return(yPixel);
}

/**
 * Sets whether this point is hidden or not.
 *
 * @param hidden Is this point hidden?
 */
public void setHidden(boolean hidden)
{
	this.hidden = hidden;
}

/**
 * Is this point is hidden?
 *
 * @return Whether this point is hidden or not
 */
public boolean isHidden()
{
	return(hidden);
}

/**
 * Sets whether this point corresponds to a hole value or not.
 *
 * @param holeValue Is this point a hole value?
 */
public void setHoleValue(boolean holeValue)
{
	this.holeValue = holeValue;
}

/**
 * Does this point corresponds to a hole value?
 *
 * @return Whether this point is a hole value or not
 */
public boolean isHoleValue()
{
	return(holeValue);
}

/**
 * Sets whether this point/facet has already been drawn.
 *
 * @param drawn Has this point been drawn?
 */
public void setDrawn(boolean drawn)
{
	this.drawn = drawn;
}

/**
 * Has this point/facet has already been drawn?
 *
 * @return Whether this point has been drawn or not
 */
public boolean isDrawn()
{
	return(drawn);
}

/** Override toString method */
public String toString()
{
	return("xPixel:" + xPixel + " yPixel:" + yPixel + " hole:" + holeValue +
	       " hidden:" + hidden + " drawn:" + drawn);
}

}
