/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: SortPoint.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

/**
 * Data structure internal to JClass Chart 3D.  Used to sort point data.  It
 * store the point, series, and distance to eye for each point.
 */
public class SortPoint implements java.io.Serializable {

/** The series to which this point belongs. */
protected int series = -1;

/** Point number in given series. */
protected int point = -1;

/** The distance of this point to the eye. */
protected double distanceToEye = Double.MAX_VALUE;


/**
 * Default constructor
 */
public SortPoint() {}

/**
 *  Constructor that intializes the series, point, and distanceToEye
 *  @param series The series number for this point
 *  @param point The point number for this point
 *  @param distanceToEye The distance to the eye for this point
 */
public SortPoint(int series, int point, double distanceToEye)
{
    this.series        = series;
	this.point         = point;
	this.distanceToEye = distanceToEye;
}

/**
 * Sets the series for this point.
 *
 * @param series The series number for this point
 */
public void setSeries(int series)
{
	this.series = series;
}

/**
 * Retrieves the series for this point.
 * @return The series number for this point
 */
public int getSeries()
{
	return(series);
}

/**
 * Sets the point number of this point in the given series.
 *
 * @param point The point number for this point
 */
public void setPoint(int point)
{
	this.point = point;
}

/**
 * Retrieves the point number of this point in the given series.
 * @return The point number for this point
 */
public int getPoint()
{
	return(point);
}

/**
 * Sets the <code>distanceToEye</code> for this point.
 *
 * @param distanceToEye The distance to the eye for this point
 */
public void setDistanceToEye(double distanceToEye)
{
	this.distanceToEye = distanceToEye;
}

/**
 * Retrieves the <code>distanceToEye</code> for this point.
 * @return The distance to the eye for this point
 */
public double getDistanceToEye()
{
	return(distanceToEye);
}

/** Override toString method */
public String toString()
{
	return("series:" + series + " point:" + point +
        " distanceToEye:" + distanceToEye);
}

}
