/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TrackChange.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:58 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d;

import java.io.IOException;

/** 
 * Concrete implementation of the <code>Changeable</code> interface,
 * for use inside JClass Chart 3D.
 * 
 */
abstract public class TrackChange implements Changeable, java.io.Serializable {

/** Boolean representing the value of the <code>Changed</code> property. */
protected boolean        changed     = true;
protected int            changedFlag = Changeable.NO_CHANGE;

/** 
 * Gets the value of the <code>Changed</code> property.
 * The <code>Changed</code> property determines whether the object needs
 * to be recalculated.
 * 
 * @return True if the object has been changed and needs recalculation, false 
 *     otherwise.
 */
public boolean getChanged()
{
	return(changed);
}

/** 
 * Checks the value of the <code>ChangedFlag</code> property for the specified
 * mask (see <code>Changeable</code> for a list of valid masks).
 * The </code>ChangedFlag</code> property determines the action the object needs
 * to take in regards to the current change.
 * 
 * @param mask 
 * @return True if the specified mask exists in the <code>ChangedFlag</code> 
 *     property, false otherwise.
 */
public boolean getChanged(int mask)
{
	return((changedFlag & mask) == mask);
}

/** 
 * Gets the value of the <code>Changed</code> property.
 * The <code>Changed</code> property determines whether the object needs
 * to be recalculated.
 * 
 * @return True if the object has been changed and needs recalculation, false 
 *     otherwise.
 */
final public boolean isChanged()
{
	return(getChanged());
}

/** 
 * Checks the value of the <code>ChangedFlag</code> property for the specified
 * mask (see <code>Changeable</code> for a list of valid masks).
 * The <code>ChangedFlag</code> property determines the action the object needs
 * to take in regards to the current change.
 * 
 * @param mask 
 * @return True if the specified mask exists in the <code>ChangedFlag</code> 
 *     property, false otherwise.
 */
final public boolean isChanged(int mask)
{
	return(getChanged(mask));
}

/** 
 * Update the "<code>changed</code>" flag and the "<code>ChangedFlag</code>"
 * mask.
 * 
 * @param n 
 * @param cf 
 */
protected void updateFlags(boolean n, int cf)
{
	changed = n;
	if (n) {
		changedFlag |= cf;
	}
	else {
		changedFlag = cf;
	}
}

/** 
 * Sets the value of the <code>Changed</code> property and updates the
 * parent object.
 * The <code>Changed</code> property determines whether the object needs
 * to be recalculated.
 * 
 * @param n New value for <code>Changed</code>.
 * @param cf 
 */
public void setChanged(boolean n, int cf)
{
	updateFlags(n, cf);
}

/** 
 * Sets the value of the <code>Changed</code> property.
 * The <code>Changed</code> property determines whether the object needs
 * to be recalculated.
 * 
 * @param n New value for <code>Changed</code>.
 * @param cf 
 * @param updateParent If true, the parent is updated.  Otherwise, the parent 
 *     is not updated.  The "update" semantics are up to the implementor.
 */
public void setChanged(boolean n, int cf, boolean updateParent)
{
	if (updateParent) {
		// Note that normally, setChanged is overriden in subclasses
		// which means that this call is actually different than
		// calling updateFlags
		setChanged(n, cf);
	}
	else {
		updateFlags(n, cf);
	}
}

/** 
 * Needed for <code>Changeable</code> interface.
 * 
 */
public boolean isBatched()
{
	return(false);
}

/** 
 * Overrides write object to change the state of the changed flags being
 * saved so that the chart will actually recalc and draw when re-serialized.
 * 
 * @exception IOException 
 * @param out 
 */
private void writeObject(java.io.ObjectOutputStream out) throws IOException
{
  // Save current state first
  boolean    oldChanged     = changed;
  int        oldChangedFlag = changedFlag;

  // Save state that will generate a recalc
  changed     = true;
  changedFlag = Changeable.RECALC;
  out.defaultWriteObject();

  // Restore current state
  changed     = oldChanged;
  changedFlag = oldChangedFlag;
}

}
