/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: BeanTextParser.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:59 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.beans;

import java.util.StringTokenizer;
import java.util.Vector;


/** 
 * This class serves as a parser for bean <code>setAsText()</code> strings.
 * 
 */
public class BeanTextParser {

/** 
 * Parses the <code>setAsText()</code> string from the chart beans.  This string looks
 * exactly like the list of parameters you'd pass to the constructor of
 * a wrapper.  Some parameters may be encased in quotes, which are removed.
 * All parameters are separated by a comma.  Escaped quotes and commas
 * that are within quote blocks are kept as is.
 * 
 * @param beanText 
 */
public static Vector parse(String beanText)
{

	StringTokenizer tokenizer = new StringTokenizer(
						beanText, "+\n\r\t\f\\,\"", true);
//						beanText, "\t\n\r\f\\,\"", true);

	Vector parseResults = new Vector();
	boolean inQuotes = false;      //currently in quoted text
	boolean slashPending = false;  //we've hit an escape slash, now waiting
								   //for next character
	boolean takeThisToken = false; //add this token to the tokenSoFar string
	boolean fullTokenDone = false; //we've got the entire parameter token now
	StringBuffer tokenSoFar = new StringBuffer(""); // token string we're 
											//building from the smaller tokens

	while (tokenizer.hasMoreTokens()) {

		fullTokenDone = false;
		takeThisToken = false;
		String token = tokenizer.nextToken();

		// hit a backslash -- it could be an escaped sequence (like, for
		// carriage \n carriage returns), so note it and move on.  If it's
		// the second backslash in a row, then it is an escaped backslash
		// sequence and we want this second backslash in our token.
		if (token.equals("\\")) {
			if (!slashPending) {
				slashPending = true;
			} else {
				slashPending = false;
				takeThisToken = true;
			}
		// hit a quoted character.  If we are in an escaped sequence, we
		// want this quote in our token.  Otherwise, just note whether we
		// are entering or leaving a quoted section of text and don't 
		// include those quotes in our token.
		} else if (token.equals("\"")) {
			if (slashPending) {
				slashPending = false;
				takeThisToken = true;
			} else if (!inQuotes) {
				inQuotes = true;
			} else {
				inQuotes = false;
			}
		// hit a plus sign.  If in quoted sequence, keep it in the token.
		// If outside a quoted sequence, it's there to joing strings and
		// we want to lose it.
		} else if (token.equals("+")) {
			if (inQuotes) {
				takeThisToken = true;
			} else {
			}
		// hit a comma.  If in an escaped sequence or inside a quote block, 
		// we want it in the token.  Otherwise, it marks the end of our token.
		} else if (token.equals(",")) {
			if (slashPending) {
				slashPending = false;
				takeThisToken = true;
			} else if (!inQuotes) {
				fullTokenDone = true;
			} else {
				takeThisToken = true;
			}
		// hit a carriage return.  Ignore them.
		} else if (token.equals("\n") || token.equals("\r")) {
		// hit normal text.  If we're in a possibly escaped sequence, keep
		// the backslash that got us there.  Always keep the normal text.
		} else {
			if (slashPending) {
				tokenSoFar.append("\\");
				slashPending = false;
			}
			takeThisToken = true;
		}

		if (takeThisToken) {
			tokenSoFar.append(token);
		}

		// We're done with this token.  Add it to the vector and start anew.
		if (fullTokenDone) {
			parseResults.addElement(tokenSoFar.toString().trim());
			tokenSoFar = new StringBuffer("");
		}
	}
	// get that last token -- it won't have had a comma after it
	parseResults.addElement(tokenSoFar.toString().trim());

	return parseResults;
}

}
