/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataSourceEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:59 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.beans;

import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;
import java.beans.PropertyEditor;
import java.util.StringTokenizer;
import java.awt.*;
import java.awt.event.*;

import javax.swing.event.CaretListener;
import javax.swing.event.CaretEvent;
import javax.swing.*;

import java.util.Vector;

import com.klg.jclass.chart3d.data.*;



/**
 * Bean editor for chart data source property.
 */

public class DataSourceEditor implements PropertyEditor {

protected PropertyChangeSupport support = new PropertyChangeSupport( this );
private boolean isFile = false;
protected String target = null;
protected String xText = null;
protected String yText = null;
protected String zText = null;
protected String fText = null;
private boolean edited = false;

private JTextField fileField = new JTextField(30);
private JTextField xValuesField = new JTextField(30);
private JTextField yValuesField = new JTextField(30);
private JTextArea zValuesField = new JTextArea();
private JCheckBox areaBox;
private JCheckBox fileBox;

public void setValue( Object value ) {
	if (!edited) {
	if( value != null && !value.equals( "" ) && ((String)value).indexOf(", ") >0) {

		StringTokenizer st = new StringTokenizer((String)value, "\\n");
		if (st.countTokens() < 2 ) {
			edited = true;
			return;
		}

		String temp = st.nextToken();
		if(temp.indexOf("null") <0 ) {
			xText = temp;

		}
		temp = st.nextToken();
		if(temp.indexOf("null") < 0) {
			yText = temp;


		}
		zText = "";
		while(st.hasMoreTokens() ) {
			zText +=st.nextToken();
			zText += "\n";

		}
		target = xText + " \\n";

		target += yText + " \\n";
		StringTokenizer zT = new StringTokenizer(zText, "\n");
		while (zT.hasMoreTokens()){
			target += zT.nextToken() + "\\n";
		}


		edited = true;

		return;

	}
		if (fText == null ) {
		fText = new String("");
		}
		target = fText;
		edited = true;

	}

	xValuesField.setText(xText);
	yValuesField.setText(yText);
	zValuesField.setText(zText);
	fileField.setText(fText);

    if( support != null ){
        support.firePropertyChange( "", null, getValue() );

    }




}



public Object getValue() {
	if(fileBox == null || !fileBox.isSelected()) {
	    target = xText + " \\\\n";

	    target += yText + " \\\\n";
	    if (zText != null) {
    		StringTokenizer st = new StringTokenizer(zText, "\n");
		while (st.hasMoreTokens()){
			target += st.nextToken() + "\\\\n";
		}
	    }
	} else {
		target = fText;
	}

	return  target;
}

public boolean isPaintable() {
	return true;
}

public void paintValue( Graphics gc, Rectangle box ) {
    String s = JCChart3dBeanBundle.string(JCChart3dBeanBundle.CLICK_TO_EDIT);
    if (s == null) {
        s = new String("");
    }
	gc.setClip( box );
    FontMetrics fm = gc.getFontMetrics();
    int vpad = (box.height - fm.getAscent())/2;
    gc.drawString(s, box.x, box.y+box.height-vpad);
}

public String getAsText() {
	if(areaBox != null && areaBox.isSelected()) {
		xText = xValuesField.getText();
		yText = yValuesField.getText();
		zText = zValuesField.getText();
		target = xText + " \\n";

		target += yText + " \\n";
		StringTokenizer st = new StringTokenizer(zText, "\n");
		while (st.hasMoreTokens()){
			target += st.nextToken() + "\\n";
		}
	}
	else {
		target = fText;
	}

	return target;
}

public void setAsText( String t ) {
    	if (!edited) {
	if( t != null && !t.equals( "" ) && t.indexOf(", ") >0) {

		StringTokenizer st = new StringTokenizer(t, "\\n");
		if (st.countTokens() < 2 ) {
			edited = true;
			return;
		}

		String temp = st.nextToken();
		if(temp.indexOf("null") <0 ) {
			xText = temp;

		}
		temp = st.nextToken();
		if(temp.indexOf("null") < 0) {
			yText = temp;


		}
		zText = "";
		while(st.hasMoreTokens() ) {
			zText +=st.nextToken();
			zText += "\n";

		}
		target = xText + " \\n";

		target += yText + " \\n";
		StringTokenizer zT = new StringTokenizer(zText, "\n");
		while (zT.hasMoreTokens()){
			target += zT.nextToken() + "\\n";
		}


		edited = true;

		return;

	}
		if (fText == null ) {
		fText = new String("");
		}
		target = fText;
		edited = true;

	}

	xValuesField.setText(xText);
	yValuesField.setText(yText);
	zValuesField.setText(zText);
	fileField.setText(fText);

    if( support != null ){
        support.firePropertyChange( "", null, getValue() );

    }

}

public String[] getTags() {
	return null;
}

public Component getCustomEditor() {
	final JPanel editor = new JPanel();
	final JPanel boxes = new JPanel();
	final JPanel fileLocation = new JPanel();
	final ButtonGroup cbg = new ButtonGroup();
	fileBox = new JCheckBox( JCChart3dBeanBundle.string( JCChart3dBeanBundle.LOAD_DATA_FROM_FILE ), isFile );
	areaBox = new JCheckBox( JCChart3dBeanBundle.string( JCChart3dBeanBundle.EDIT_DATA_TEXTAREA ), !isFile );
	final JPanel dataPanel = new JPanel();
	dataPanel.setLayout(new GridLayout(2, 1));
	final JPanel xyPanel = new JPanel();
	xyPanel.setLayout(new GridLayout(5, 1));
	final JLabel xValuesLabel = new JLabel(JCChart3dBeanBundle.string(JCChart3dBeanBundle.EDIT_X_VALUES));
	final JLabel yValuesLabel = new JLabel(JCChart3dBeanBundle.string(JCChart3dBeanBundle.EDIT_Y_VALUES));
	final JButton applyButton = new JButton(JCChart3dBeanBundle.APPLY);
	dataPanel.add(xyPanel);
	xyPanel.add(xValuesLabel);
	xyPanel.add(xValuesField);
	xValuesField.setText(xText);
	xyPanel.add(yValuesLabel);
	xyPanel.add(yValuesField);
	yValuesField.setText(yText);

	zValuesField.setText(zText);
	final JLabel zValuesLabel = new JLabel(JCChart3dBeanBundle.string(JCChart3dBeanBundle.EDIT_Z_VALUES));
	xyPanel.add(zValuesLabel);
	fileBox.setSelected(false);
	areaBox.setSelected(true);
	cbg.add( fileBox );
	cbg.add( areaBox );
	editor.setLayout( new BorderLayout() );
	boxes.setLayout( new FlowLayout( FlowLayout.CENTER, 4, 4 ) );
	fileLocation.setLayout( new FlowLayout( FlowLayout.CENTER, 4, 4 ) );
	boxes.add( areaBox );
	boxes.add( fileBox );
	fileLocation.add( new JLabel( JCChart3dBeanBundle.string( JCChart3dBeanBundle.FILE_LOCATION ) + ":" ) );
	fileLocation.add( fileField );
	fileLocation.add(applyButton);
	zValuesField.setEnabled( !isFile );
	fileField.setEnabled( isFile );
	fileField.setText(fText);

	applyButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			if (fileBox.isSelected()) {
				fText = fileField.getText();
				isFile = true;
			}
			else {
				xText = xValuesField.getText();
				yText = yValuesField.getText();
				zText = zValuesField.getText();
				isFile = false;
			}
			if( support != null ){
				support.firePropertyChange( "", null, getValue() );
			}

		}
	});




	fileBox.addItemListener( new ItemListener() {
		public void itemStateChanged( ItemEvent e ) {
			if (e.getStateChange() == ItemEvent.SELECTED && e.getSource() == fileBox) {
				fileField.setEnabled( true );
				xValuesField.setEnabled( false );
				yValuesField.setEnabled(false);
				zValuesField.setEnabled( false);
				isFile = true;
				fText = fileField.getText();
				if( support != null ) {
					support.firePropertyChange( "", null, getValue() );
				}
			}
		}
	});
	areaBox.addItemListener( new ItemListener() {
		public void itemStateChanged( ItemEvent e ) {
			if (e.getStateChange() == ItemEvent.SELECTED && e.getSource() == areaBox) {
				fileField.setEnabled( false );
				xValuesField.setEnabled( true);
				yValuesField.setEnabled( true);
				zValuesField.setEnabled( true );
				isFile = false;
				//target = t.getText();
				if( support != null ) {
					support.firePropertyChange( "", null, getValue() );
				}
			}
		}
	});

	dataPanel.add(zValuesField);


	editor.add( boxes, BorderLayout.NORTH );
	editor.add( dataPanel, BorderLayout.CENTER );
	editor.add( fileLocation, BorderLayout.SOUTH );
	return editor;
}


public boolean supportsCustomEditor() {
	return true;
}

public final String getJavaInitializationString() {


	return new String( "\"" + getValue() + "\"");
}

public void addPropertyChangeListener(PropertyChangeListener l) {
	support.addPropertyChangeListener(l);
}

public void removePropertyChangeListener(PropertyChangeListener l) {
	support.removePropertyChangeListener(l);
}

}
