/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCChart3dBeanBundle.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:59 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.beans;

import java.util.Locale;
import java.util.ResourceBundle;
import java.text.MessageFormat;
import java.text.ChoiceFormat;

/**
 * <code>JCChartBeanBundle</code> contains locale-specific objects and helpers to get this information.
 * When a program needs a locale-specific String it can load it from the resource bundle that
 * is appropriate for the current user's locale. Those are located in the <i>com.klg.jclass.chart\resources</i>
 * directory. In this way, program code can be written that is largely independent of the user's locale,
 * isolating most of the locale-specific information in resource bundles.
 */
public class JCChart3dBeanBundle {

private static final String BUNDLE_PATH = "com.klg.jclass.chart3d.beans.resources.LocaleInfo";
private static ResourceBundle chart_bundle = ResourceBundle.getBundle( BUNDLE_PATH, Locale.getDefault() );

/**
 * Static function that localizes a String based on its key,
 * language, and country.
 * @param language desired language
 * @param country desired country
 * @param key the key of the String that is to be retrived
 * @return
 */
public synchronized static String string( String language, String country, String key ) {
	return ResourceBundle.getBundle( BUNDLE_PATH, new Locale( language, country ) ).getString( key );
}

/**
 * Static function that retrives a String based on its key.
 * In JDK 1.1 the default locale it is used.
 * @param key the key of the String that is to be retrived
 * @return
 */
public synchronized static String string( String key ) {
	String s = null;
	s = chart_bundle.getString( key );
	return s;
}

/**
 * Static function used to format a String based on its key.
 * It produces concatenated messages in a language-neutral way.
 * @param key the key of the String that is to be retrived
 * @param args set of objects used in formatting
 * @return
 */
public synchronized static String string( String key, Object[] args ) {
	String s = null;

	s = MessageFormat.format( chart_bundle.getString( key ), args );

	return s;
}

 /**
 * Static function used to format a String based on its key.
 * It produces concatenated messages in a language-neutral way.
 * @param key the key of the String that is to be retrived
 * @param formatter a formatter provided by the user
 * @param args set of objects used in formatting
  * @return
 */
public synchronized static String string( String key, MessageFormat formatter, Object args[] ) {
	String s = null;

	s = formatter.format( chart_bundle.getString( key ), args );
	return s;
}

 /**
 * Static function used to programatically change the default locale.
 * @param l locale
 */
public synchronized static void setBundleLocale( Locale l ) {
	chart_bundle = ResourceBundle.getBundle( BUNDLE_PATH, l );
}

 /**
 * Static function used to format a String based on its key.
 * It produces concatenated messages in a language-neutral way.
 * Allows a format to be attached to a range of numbers. It is generally used for handling plurals.<br>
 * The choice is specified with an ascending list of doubles, where each item 
 * specifies a half-open interval up to the next item: <i>X</i> matches <i>j</i> if and only if
 * <i>limit[j] <= X < limit[j+1]</i>
 * @param key the key of the String that is to be retrived
 * @param args set of objects used in formatting
 * @param limits the list of doubles that specifies a half-open interval
 * @param formats it is an array of choices: one choice for each interval defined by the limits array
 * @param choiceVar It is used to identify which of the variables from the String contains
 * the formatting information used in the choice formatting feature.
  * <br><B>Note:</b> There is a limitation of only one choice variable per String.
  * @return
 */
public synchronized static String string( String key, Object[] args, double[] limits, String[] formats, int choiceVar ) {
	String s = null;
	MessageFormat formatter = new MessageFormat( chart_bundle.getString( key ) );
	ChoiceFormat choicer = new ChoiceFormat( limits, formats );

	formatter.setFormat( choiceVar, choicer );
	s = formatter.format( args );

	return s;
}
							

// bean.Key constants
public static final String CLICK_TO_EDIT = "Click to edit...";
public static final String LOAD_DATA_FROM_FILE = "Load data from a file";
public static final String EDIT_DATA_TEXTAREA = "Edit data, using comma separated values.";
public static final String EDIT_X_VALUES = "Edit x Values:";
public static final String EDIT_Y_VALUES = "Edit y Values:";
public static final String EDIT_Z_VALUES = "Edit z Values:";
public static final String FILE_LOCATION = "File Location";

public static final String TRIGGER_PROPERTIES = "Trigger Properties";
public static final String SHIFT = "Shift";
public static final String CTRL = "Ctrl";
public static final String ALT = "Alt";
public static final String META = "Meta";
public static final String MODIFIER = "Modifier";
public static final String ACTION = "Action";
public static final String NONE = "None";
public static final String ALLOW_USER_CHANGES = "Allow user changes";

public static final String APPLY = "Apply";


}
