/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AbstractChart3dCustomizer.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:59 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.JCChart3d;
import com.klg.jclass.chart3d.JCChart3dCustomizer;
import com.klg.jclass.chart3d.event.JCChart3dEvent;
import com.klg.jclass.chart3d.event.JCChart3dListener;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

public abstract class AbstractChart3dCustomizer
    extends Chart3dPropertyEditor
	implements JCChart3dCustomizer, PropertyEditorRoot,
               ActionListener, JCChart3dListener
{

private final static String			ABOUT_PATH = "com.klg.jclass.chart3d.beans";
private final static String			ABOUT_FILENAME = "banner.gif";
private final static String			REDIRECT_URL = "http://www.quest.com/jclass/";

private static final String[]		children = {
					"com.klg.jclass.chart3d.customizer.ControlEditor",
					"com.klg.jclass.chart3d.customizer.ChartAreaEditor",
					"com.klg.jclass.chart3d.customizer.View3dEditor",
					"com.klg.jclass.chart3d.customizer.PlotCubeEditor",
					"com.klg.jclass.chart3d.customizer.TitlesEditor",
					"com.klg.jclass.chart3d.customizer.LegendEditor",
					"com.klg.jclass.chart3d.customizer.AxesEditor",
					"com.klg.jclass.chart3d.customizer.DataViewEditor",
};

public JFrame						customizerFrame;

private static boolean				isOSWindows;
private static ImageIcon			aboutIcon;
//private boolean						isTree = false;
private JCChart3d					target = null;
private PropertyEditorTemplate		template = null;
private JCheckBox					isBatchedCheckBox;
//private JCheckBox					isTreeCheckBox;
private JButton						okButton;
private JButton						applyButton;
private JPanel						customizerComponent = null;

public AbstractChart3dCustomizer()
{

    super(null, children);
}

public void setTarget(JCChart3d aTarget)
{

    if (aTarget == null) {
        throw new IllegalArgumentException("setChart(JCChart3d) must be non-null.");
    }

    if (target != null) {
        target.removeChart3dListener(this);
    }

    target = aTarget;
    target.addChart3dListener(this);
}

public JCChart3d getTarget()
{
    return target;
}

public void showCustomizer(Point p)
{
    if (target == null) {
        return;
    }

    if (customizerFrame == null) {
        customizerFrame = new JFrame("JClass Chart 3D Customizer");
        customizerFrame.getContentPane().add(getCustomizerComponent());
        customizerFrame.pack();
    }

    guaranteePointOnScreen(p);

    customizerFrame.setLocation(p);
    customizerFrame.show();
    okButton.requestFocus();
}

public static void guaranteePointOnScreen(Point p)
{

    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

    if (p.x < 0) {
        p.x = 10;
    }
    if (p.y < 0) {
        p.y = 10;
    }
    if (p.x > screenSize.width) {
        p.x = screenSize.width - 10;
    }
    if (p.y > screenSize.height) {
        p.y = screenSize.height - 10;
    }
}

public void hideCustomizer()
{
    shutdown();
}

public Component getEditor()
{
    return null;
}

public Component getCustomizerComponent()
{

    if (customizerComponent == null) {

        customizerComponent = new JPanel(new BorderLayout());
        customizerComponent.setMinimumSize(new Dimension(150, 150));
        customizerComponent.setPreferredSize(new Dimension(600, 550));

//		isTreeCheckBox = createCheckBox("Tree", this);
        isBatchedCheckBox =
            createCheckBox(getLocalizedString(LocaleBundle.STRING_BATCHED),
                           this);
        okButton =
            createButton(getLocalizedString(LocaleBundle.STRING_OK), this);
        applyButton =
            createButton(getLocalizedString(LocaleBundle.STRING_APPLY), this);

        JPanel controls = new JPanel(new GridLayout(1, 0, 5, 5));
//		controls.add(isTreeCheckBox);
        controls.add(okButton);
        controls.add(applyButton);

        JPanel panel = createBorderPanel();
        panel.add(isBatchedCheckBox, BorderLayout.WEST);
        panel.add(controls, BorderLayout.EAST);

        customizerComponent.add(panel, BorderLayout.SOUTH);
//		if (!isTreeCheckBox.isSelected()) {
        template = new TabbedTemplate(this);
        customizerComponent.add(template.getEditorComponent());
//		}
//		else {
//			template = new TreeTemplate(this);
//			customizerComponent.add(template.getEditorComponent());
//		}

    }
    return customizerComponent;
}

/**
 * Method called whenever chart 3D has been modified in some way. Interested
 * listeners should implement this method and register the object with JClass
 * Chart 3D.
 *
 * @param e The event to which this method is to respond
 */
public void changeChart(JCChart3dEvent e)
{
    // If I have the focus, then I amreposnsible for the change, and do nothing...
    // Else refresh the editor tree so that any changes are reflected...

    refreshEditorTree();
}

/**
 * Method called whenever the paint method of the JCChart3d object is called.
 *
 * @param chart The chart that was painted.
 */
public void paintChart(JCChart3d chart)
{
    // Do nothing!

    refreshEditorTree();
}

public void actionPerformed(ActionEvent e)
{

    Object source = e.getSource();

    if (source == applyButton) {
        target.setBatched(false);

        if (isBatchedCheckBox.isSelected()) {
            target.setBatched(true);
        }

        return;
    }

    if (source == okButton) {
        shutdown();
        return;
    }

    if (source == isBatchedCheckBox) {
        target.setBatched(isBatchedCheckBox.isSelected());
        return;
    }

/*	if (source == isTreeCheckBox) {

    if (isTreeCheckBox.isSelected()) {
        customizerComponent.remove(template.getEditorComponent());
        template = new TreeTemplate(this);
        customizerComponent.add(template.getEditorComponent());
    }
    else {
        customizerComponent.remove(template.getEditorComponent());
        template = new TabbedTemplate(this);
        customizerComponent.add(template.getEditorComponent());
    }
    customizerComponent.invalidate();
    customizerComponent.validate();
    customizerComponent.repaint();
    return;
}
*/
}

private void shutdown()
{

    if (target == null) {
        return;
    }

    customizerFrame.dispose();
    customizerComponent = null;
    customizerFrame = null;

    target.setBatched(false);

    finalizeEditorTree();

    template = null;

//	isTreeCheckBox.removeActionListener(this);
//	isTreeCheckBox = null;

    isBatchedCheckBox.removeActionListener(this);
    isBatchedCheckBox = null;

    applyButton.removeActionListener(this);
    applyButton = null;


    return;
}

public static JLabel createAboutPage()
{

    if (aboutIcon == null) {

        String os = System.getProperty("os.name");
        if (os != null && os.startsWith("Windows")) {
            isOSWindows = true;
        }
        else {
            isOSWindows = false;
        }

        String s = "/com/klg/jclass/beans/resources/" + ABOUT_FILENAME;
        java.net.URL url = AbstractChart3dCustomizer.class.getResource(s);

        aboutIcon = new ImageIcon(url);
    }

    JLabel splashLabel = new JLabel(REDIRECT_URL);
    splashLabel.setIcon(aboutIcon);

    splashLabel.setForeground(Color.blue);
    splashLabel.setBorder(new EmptyBorder(20, 20, 20, 20));
    splashLabel.setHorizontalTextPosition(SwingConstants.CENTER);
    splashLabel.setVerticalTextPosition(SwingConstants.BOTTOM);

    splashLabel.addMouseListener(new MouseAdapter() {

        public void mousePressed(MouseEvent e)
        {

            try {

                if (isOSWindows) {
                    String cmd =
                        "rundll32 url.dll,FileProtocolHandler " + REDIRECT_URL;
                    Runtime.getRuntime().exec(cmd);
                }
                else {
                    // launching browser on unix platform

                    // Under Unix, Netscape has to be running for the "-remote"
                    // command to work. So, we try sending the command and
                    // check for an exit value. If the exit command is 0,
                    // it worked, otherwise we need to start the browser.

                    String cmd = "netscape -remote openURL(" + REDIRECT_URL + ")";
                    Process p = Runtime.getRuntime().exec(cmd);

                    try {
                        int exitCode = p.waitFor();

                        if (exitCode != 0) {
                            cmd = "netscape " + REDIRECT_URL;
                            p = Runtime.getRuntime().exec(cmd);
                        }
                    }
                    catch (InterruptedException ie) {
                        System.err.println("Error launching browser");
                    }
                }

            }
            catch (Exception exc) {
                // Do nothing!
            }
        }
    });

    return splashLabel;
}

}
