/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisAnnoFontEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:59 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.*;
import com.klg.jclass.util.swing.JCFontChooserPane;
import com.klg.jclass.util.swing.JCFontListener;
import com.klg.jclass.util.swing.JCFontEvent;

import java.awt.Component;
import java.awt.Container;
import java.awt.BorderLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.JTextField;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.Box;

public class AxisAnnoFontEditor extends Chart3dPropertyEditor
	implements ActionListener, FocusListener, JCFontListener {

private static String		nameKey = LocaleBundle.STRING_ANNO_FONT;

private boolean				refreshing = false;
private Container			content;
private JCFontChooserPane	fontChooser;
private JTextField			cubeSizeField;

public AxisAnnoFontEditor() {
	super(nameKey);
}

public Component getEditor() {

	if (content == null) {

		fontChooser = getFontChooser();
		fontChooser.addJCFontListener(this);
		fontChooser.setAlignmentX(0);

		String local;

		local = getLocalizedString(LocaleBundle.STRING_FONT_CUBE_SIZE);

		JLabel cubeSizeLabel = new JLabel(local);

		cubeSizeField = createTextField(10, this, this);

		JPanel cubeSizePanel = createBorderPanel();
		cubeSizePanel.setBorder(null);
		cubeSizePanel.add(cubeSizeLabel, BorderLayout.WEST);
		cubeSizePanel.add(cubeSizeField);
		cubeSizePanel.setAlignmentX(0);

		content = createPreferredSizePanel();
		content.setLayout(new BoxLayout(content, BoxLayout.Y_AXIS));
		content.add(cubeSizePanel);
		content.add(Box.createVerticalStrut(10));
		content.add(fontChooser);
	}

	return content;
}

public void finalizeEditor() {

	if (content != null) {

		// Remove listeners and destroy components.
		fontChooser.removeJCFontListener(this);
		fontChooser = null;

		cubeSizeField.removeActionListener(this);
		cubeSizeField.removeFocusListener(this);
		cubeSizeField = null;

		content = null;
	}
}

public void refreshEditor() {

	if (content != null) {

		refreshing = true;

		AxesEditor parent = (AxesEditor)getParent();
		JCAxis axis = parent.getSelectedAxis();

		fontChooser.setSelectedFont(axis.getAnnoFont());

		cubeSizeField.setText(String.valueOf(axis.getAnnoFontCubeSize()));

		refreshing = false;
	}
}

public void actionPerformed(ActionEvent e) {
	handleChange(e.getSource());
}

public void focusLost(FocusEvent e) {
	handleChange(e.getSource());
}

public void focusGained(FocusEvent e) {
	handleChange(e.getSource());
}

private void handleChange(Object source) {

	if (refreshing) {
		return;
	}

	if (content != null) {

		AxesEditor parent = (AxesEditor)getParent();
		JCAxis axis = parent.getSelectedAxis();

		if (source == cubeSizeField) {

			Integer i = parseIntegerInput(cubeSizeField);

			if (i == null || i.intValue() < 0 || i.intValue() > JCAxis.MAX_AXIS_FONT_CUBE_SIZE) {
				// The parse was unsuccessful, or exceeded limits!  Refresh the editor and return.
				refreshEditor();
				return;
			}
			axis.setAnnoFontCubeSize(i.intValue());
		}
	}
}

public void fontChanged(JCFontEvent e) {

	if (refreshing) {
		return;
	}

	if (content != null) {

		AxesEditor parent = (AxesEditor)getParent();
		JCAxis axis = parent.getSelectedAxis();

		axis.setAnnoFont(e.getFont());
	}
}

public void fontChanging(JCFontEvent e) {
	// Do nothing!
}

}
