/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisGeneralEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:59 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.JCAxis;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.BorderLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import java.util.Arrays;

import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JCheckBox;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.Box;

public class AxisGeneralEditor extends Chart3dPropertyEditor implements ActionListener, FocusListener {

private static final String		nameKey = LocaleBundle.STRING_GENERAL;

private boolean					refreshing = false;
private Container				content;
private JCheckBox				isVisibleCheckBox;
private JTextField				originField;
private JComboBox				annotationMethodComboBox;
private Object[]				annotationMethodModel;

public AxisGeneralEditor() {
	super(nameKey);
}

public Component getEditor() {

	if (content == null) {

		String local;

		local = getLocalizedString(LocaleBundle.STRING_VISIBLE);
		isVisibleCheckBox = createCheckBox(local, this);

		local = getLocalizedString(LocaleBundle.STRING_ANNO_METHOD);
		JLabel annotationMethodLabel =  new JLabel(local);

		annotationMethodModel = new ComboBoxEntry[] {

			new ComboBoxEntry(JCAxis.ANNOTATION_VALUES, getLocalizedString(LocaleBundle.STRING_VALUES)),
			new ComboBoxEntry(JCAxis.ANNOTATION_DATA_LABELS, getLocalizedString(LocaleBundle.STRING_DATA_LABELS)),
			new ComboBoxEntry(JCAxis.ANNOTATION_VALUE_LABELS, getLocalizedString(LocaleBundle.STRING_VALUE_LABELS))
		};

		// Sort the model so that we can map a constant to an entry using the Arrays.find
		// method.
		Arrays.sort(annotationMethodModel);

		annotationMethodComboBox = createComboBox(annotationMethodModel, this);

		local = getLocalizedString(LocaleBundle.STRING_ORIGIN);
		JLabel originLabel = new JLabel(local);

		originField = createTextField(10, this, this);

		JPanel labels = new JPanel(new GridLayout(3,0,5,5));
		labels.add(isVisibleCheckBox);
		labels.add(annotationMethodLabel);
		labels.add(originLabel);

		JPanel fields = new JPanel(new GridLayout(3,0,5,5));
		fields.add(new JPanel());
		fields.add(annotationMethodComboBox);
		fields.add(originField);

		JPanel controls = createBorderPanel();
		controls.setBorder(null);
		controls.add(labels, BorderLayout.WEST);
		controls.add(fields);

		JPanel panel = createPreferredSizePanel();
		panel.add(controls);

		content = Box.createVerticalBox();
		content.add(panel);

	}
	return content;
}

public void finalizeEditor() {

	if (content != null) {

		isVisibleCheckBox.removeActionListener(this);
		isVisibleCheckBox = null;

		originField.removeActionListener(this);
		originField.removeFocusListener(this);
		originField = null;

		annotationMethodComboBox.removeActionListener(this);
		annotationMethodComboBox = null;

		content = null;
	}
}

public void refreshEditor() {

	if (content != null) {

		refreshing = true;

		AxesEditor parent = (AxesEditor)getParent();
		JCAxis axis = parent.getSelectedAxis();

		isVisibleCheckBox.setSelected(axis.isShowing());


		MutableComboBoxEntry entry = new MutableComboBoxEntry();

		entry.setConstant(axis.getAnnotationMethod());
		annotationMethodComboBox.setSelectedIndex(Arrays.binarySearch(annotationMethodModel, entry));

		// Only the z axis uses the origin field.
		if (axis.getAxisId() == JCAxis.AXIS_Z) {
			originField.setEnabled(true);
		}
		else {
			originField.setEnabled(false);
			originField.setText(String.valueOf(axis.getOrigin()));
		}

		refreshing = false;
	}
}

public void actionPerformed(ActionEvent e) {
	handleChange(e.getSource());
}

public void focusGained(FocusEvent e) {
	handleChange(e.getSource());
}

public void focusLost(FocusEvent e) {

	if (!e.isTemporary()) {
		handleChange(e.getSource());
	}
}

private void handleChange(Object source) {

	if (refreshing) {
		return;
	}

	if (content != null) {

		AxesEditor parent = (AxesEditor)getParent();
		JCAxis axis = parent.getSelectedAxis();

		if (source == isVisibleCheckBox) {
			axis.setShowing(isVisibleCheckBox.isSelected());
		}
		else if (source == annotationMethodComboBox) {

			Object	selection = annotationMethodComboBox.getSelectedItem();
			int		constant = ((ComboBoxEntry)selection).getConstant();

			axis.setAnnotationMethod(constant);
		}
		else if (source == originField) {

			Double d = parseDoubleInput(originField);

			if (d == null) {
				refreshEditor();
			}
			else {
				axis.setOrigin(d.doubleValue());
			}
		}
	}
}

}
