/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisGridLinesEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:59 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.JCAxis;
import com.klg.jclass.chart3d.JCGridLines;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.Box;

public class AxisGridLinesEditor extends Chart3dPropertyEditor implements ActionListener {

private static final String		nameKey  = LocaleBundle.STRING_GRID_LINES;

private boolean					refreshing = false;
private Container				content;
private JCheckBox				xyCheckBox;
private JCheckBox				xzCheckBox;
private JCheckBox				yzCheckBox;

public AxisGridLinesEditor() {
	super(nameKey);
}

public Component getEditor() {

	if (content == null) {

		String local;

		local = getLocalizedString(LocaleBundle.STRING_XY_PLANE);
		xyCheckBox = createCheckBox(local, this);

		local = getLocalizedString(LocaleBundle.STRING_XZ_PLANE);
		xzCheckBox = createCheckBox(local, this);

		local = getLocalizedString(LocaleBundle.STRING_YZ_PLANE);
		yzCheckBox = createCheckBox(local, this);

		JPanel panel = createPreferredSizePanel();
		panel.setLayout(new GridLayout(0,1,5,5));
		panel.add(xyCheckBox);
		panel.add(xzCheckBox);
		panel.add(yzCheckBox);

		content = Box.createVerticalBox();
		content.add(panel);
	}
	return content;
}

public void finalizeEditor() {

	if (content != null) {

		xyCheckBox.removeActionListener(this);
		xyCheckBox = null;

		xzCheckBox.removeActionListener(this);
		xzCheckBox = null;

		yzCheckBox.removeActionListener(this);
		yzCheckBox = null;

		content = null;
	}
}

public void refreshEditor() {

	if (content != null) {

		refreshing = true;

		AxesEditor		parent = (AxesEditor)getParent();
		JCAxis			axis = parent.getSelectedAxis();
		JCGridLines 	gridLines = axis.getGridLines();

		int mask = gridLines.getPlaneMask();

		xyCheckBox.setVisible(true);
		xzCheckBox.setVisible(true);
		yzCheckBox.setVisible(true);

		if (axis.getAxisId() == JCAxis.AXIS_X) {
			yzCheckBox.setVisible(false);
		}
		else if (axis.getAxisId() == JCAxis.AXIS_Y) {
			xzCheckBox.setVisible(false);
		}
		else { // if axis.getAxisId() == JCAxis.AXIS_Z)
			xyCheckBox.setVisible(false);
		}

		xyCheckBox.setSelected((mask & JCGridLines.XY_PLANE) == JCGridLines.XY_PLANE);
		xzCheckBox.setSelected((mask & JCGridLines.XZ_PLANE) == JCGridLines.XZ_PLANE);
		yzCheckBox.setSelected((mask & JCGridLines.YZ_PLANE) == JCGridLines.YZ_PLANE);

		refreshing = false;
	}
}

public void actionPerformed(ActionEvent e) {

	if (refreshing) {
		return;
	}

	if (content != null) {

		Object src = e.getSource();

		AxesEditor parent = (AxesEditor)getParent();
		JCGridLines gridLines = parent.getSelectedAxis().getGridLines();

		int mask = gridLines.getPlaneMask(),
			plane = JCGridLines.XY_PLANE; // default

		if (src == xzCheckBox) {
			plane = JCGridLines.XZ_PLANE;
		}
		else if (src == yzCheckBox) {
			plane = JCGridLines.YZ_PLANE;
		}

		// Simply toggle the plane using an XOR.
		gridLines.setPlaneMask(mask ^ plane);
	}
}

}
