/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisLimitsEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:59 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.JCAxis;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.BorderLayout;
import java.awt.Rectangle;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JCheckBox;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.Box;

public class AxisLimitsEditor extends Chart3dPropertyEditor implements ActionListener, FocusListener {

private static final String		nameKey = LocaleBundle.STRING_LIMITS;

private boolean					refreshing = false;
private Container				content;
private JTextField[]			intFields;
private JCheckBox[]				isDefaultCheckBoxes;

public AxisLimitsEditor() {
	super(nameKey);
}

public Component getEditor() {

	if (content == null) {

		String[] keys = {
			LocaleBundle.STRING_MIN,
			LocaleBundle.STRING_MAX,
		};

		JLabel[] labels = new JLabel[2];

		intFields = new JTextField[2];
		isDefaultCheckBoxes = new JCheckBox[2];

		JPanel labelPanel = new JPanel(new GridLayout(0,1,5,5));
		JPanel intFieldPanel = new JPanel(new GridLayout(0,1,5,5));
		JPanel isDefaultCheckBoxPanel = new JPanel(new GridLayout(0,1,5,5));

		String defaultStr = getLocalizedString(LocaleBundle.STRING_DEFAULT);

		for (int each = 0; each < 2; each++) {

			labels[each] = new JLabel(getLocalizedString(keys[each]));
			labelPanel.add(labels[each]);

			intFields[each] = createTextField(7, this, this);
			intFieldPanel.add(intFields[each]);

			isDefaultCheckBoxes[each] = createCheckBox(defaultStr, this);
			isDefaultCheckBoxPanel.add(isDefaultCheckBoxes[each]);
		}

		JPanel controls = createBorderPanel();
		controls.setBorder(null);
		controls.add(labelPanel, BorderLayout.WEST);
		controls.add(intFieldPanel);
		controls.add(isDefaultCheckBoxPanel, BorderLayout.EAST);

		JPanel panel = createPreferredSizePanel();
		panel.add(controls);

		content = Box.createVerticalBox();
		content.add(panel);

	}
	return content;
}

public void finalizeEditor() {

	if (content != null) {

		// Destroy all components.
		for (int each = 0; each < 2; each++) {

			intFields[each].removeActionListener(this);
			intFields[each].removeFocusListener(this);
			intFields[each] = null;

			isDefaultCheckBoxes[each].removeActionListener(this);
			isDefaultCheckBoxes[each] = null;
		}

		intFields = null;
		isDefaultCheckBoxes = null;
		content = null;
	}
}

public void refreshEditor() {

	if (content != null) {

		refreshing = true;

		AxesEditor parent = (AxesEditor)getParent();
		JCAxis axis = parent.getSelectedAxis();

		double[]	limits = { axis.getMin(), axis.getMax() };
		boolean[]	isDefault = { axis.getMinIsDefault(), axis.getMaxIsDefault() };

		for (int each = 0; each < 2; each++) {

			// Update the isDefault check boxes.
			isDefaultCheckBoxes[each].setSelected(isDefault[each]);
			intFields[each].setEnabled(!isDefault[each]);

			intFields[each].setText(String.valueOf(limits[each]));

		}

		refreshing = false;
	}
}

public void actionPerformed(ActionEvent e) {
	handleChange(e.getSource());
}

public void focusGained(FocusEvent e) {
	handleChange(e.getSource());
}

public void focusLost(FocusEvent e) {

	if (!e.isTemporary()) {
		handleChange(e.getSource());
	}
}

private void handleChange(Object source) {

	if (refreshing) {
		return;
	}

	if (content != null) {

		AxesEditor parent = (AxesEditor)getParent();
		JCAxis axis = parent.getSelectedAxis();

		if (source instanceof JTextField) {

			Double d = parseDoubleInput((JTextField)source);

			if (d == null) {
				// Input is no good!  Refresh editor and return.
				refreshEditor();
				return;
			}

			double value = d.doubleValue();

			if (source == intFields[0]) {
				// Set axis min.
				axis.setMin(value);
			}
			else {
				// Set axis max.
				axis.setMax(value);
			}

			// In case the min is greater than the max, refresh the editor
			// to refelct the swap that is made by JCAxis.
			refreshEditor();
		}
		else if (source instanceof JCheckBox) {

			boolean isDefault = ((JCheckBox)source).isSelected();

			if (source == isDefaultCheckBoxes[0]) {
				// Axis min uses default value.
				axis.setMinIsDefault(isDefault);
				// Enable or disable the min text field.
				intFields[0].setEnabled(!isDefault);
			}
			else if (source == isDefaultCheckBoxes[1]) {
				// Axis max uses default value.
				axis.setMaxIsDefault(isDefault);
				// Enable or disable the max text field.
				intFields[1].setEnabled(!isDefault);
			}
		}
	}
}

}
