/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AxisValueLabelsEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:59 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.JCAxis;
import com.klg.jclass.chart3d.JCValueLabel;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.BorderLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.List;
import java.util.ArrayList;

import javax.swing.JList;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JButton;
import javax.swing.JTextField;
import javax.swing.JPanel;
import javax.swing.Box;
import javax.swing.AbstractListModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

public class AxisValueLabelsEditor extends Chart3dPropertyEditor
	implements ActionListener, ListDataListener {

private static final String		nameKey  = LocaleBundle.STRING_VALUE_LABELS;

private boolean					refreshing = false;
private Container				content;
private JList					valueLabelsList;
private JButton					addButton;
private JButton					insertButton;
private JButton					editButton;
private JButton					removeButton;
private ValueLabelsModel		valueLabelsModel;

public AxisValueLabelsEditor() {
	super(nameKey);
}

public Component getEditor() {

	if (content == null) {

		String local;

		local = getLocalizedString(LocaleBundle.STRING_ADD);
		addButton = createButton(local, this);

		local = getLocalizedString(LocaleBundle.STRING_INSERT);
		insertButton = createButton(local, this);

		local = getLocalizedString(LocaleBundle.STRING_EDIT);
		editButton = createButton(local, this);

		local = getLocalizedString(LocaleBundle.STRING_REMOVE);
		removeButton = createButton(local, this);

		valueLabelsModel = new ValueLabelsModel();
		valueLabelsModel.addListDataListener(this);

		valueLabelsList = new JList(valueLabelsModel);
 		valueLabelsList.setPrototypeCellValue("XXXXXXXXXXXXXXXXXXXXXX");

		JScrollPane scrollPane = new JScrollPane(
				valueLabelsList,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED
				);

		JPanel grid = createPreferredSizePanel();
		grid.setLayout(new GridLayout(0,1,5,5));
		grid.add(addButton);
		grid.add(insertButton);
		grid.add(editButton);
		grid.add(removeButton);
		grid.add(Box.createVerticalGlue());

		JPanel panel = createPreferredSizePanel();
		panel.add(grid, BorderLayout.EAST);
		panel.add(scrollPane);

		content = Box.createVerticalBox();
		content.add(panel);
	}
	return content;
}

public void finalizeEditor() {

	if (content != null) {

		addButton.removeActionListener(this);
		addButton = null;

		insertButton.removeActionListener(this);
		insertButton = null;

		editButton.removeActionListener(this);
		editButton = null;

		removeButton.removeActionListener(this);
		removeButton = null;

		valueLabelsModel.addListDataListener(this);
		valueLabelsList = null;

		content = null;
	}
}

public void refreshEditor() {

	if (content != null) {

		refreshing = true;

		AxesEditor		parent = (AxesEditor)getParent();
		JCAxis			axis = parent.getSelectedAxis();

		valueLabelsModel.setValueLabels(axis.getValueLabels());

		refreshing = false;
	}
}

public void contentsChanged(ListDataEvent e) {
	handleChange();
}

public void intervalAdded(ListDataEvent e) {
	handleChange();
}

public void intervalRemoved(ListDataEvent e) {
	handleChange();
}

public void handleChange() {

	if (refreshing) {
		return;
	}

	if (content != null) {

		AxesEditor	parent = (AxesEditor)getParent();
		JCAxis		axis = parent.getSelectedAxis();

		axis.setValueLabels(valueLabelsModel.getValueLabels());
	}
}

public void actionPerformed(ActionEvent e) {

	if (refreshing) {
		return;
	}

	if (content != null) {

		Object source = e.getSource();

		AxesEditor	parent = (AxesEditor)getParent();
		int			selection = valueLabelsList.getSelectedIndex();

		if (source == addButton || source == insertButton) {

			JCValueLabel newValueLabel = showValueLabelDialog(
				addButton,
				LocaleBundle.STRING_ADD,
				null);	// null source value label

			if (newValueLabel != null) {
				if (source == addButton || selection == -1) {
					valueLabelsModel.add(newValueLabel);
				}
				else { // source == insertButton
					valueLabelsModel.insert(selection, newValueLabel);
				}
			}
		}
		else if (source == editButton) {
			if (selection != -1) {

				showValueLabelDialog(
					removeButton,
					LocaleBundle.STRING_REMOVE,
					valueLabelsModel.getValueLabelAt(selection));

				valueLabelsModel.update(selection);
			}
		}
		else if (source == removeButton) {
			if (selection != -1) {
				valueLabelsModel.remove(selection);
			}
		}
	}
}

private JCValueLabel showValueLabelDialog(Component owner, String titleKey, JCValueLabel source) {

	String local;

	local = getLocalizedString(LocaleBundle.STRING_VALUE);
	JLabel valueLabel = new JLabel(local);

	local = getLocalizedString(LocaleBundle.STRING_LABEL);
	JLabel labelLabel = new JLabel(local);

	JPanel labelsPanel = new JPanel(new GridLayout(2,0,5,5));
	labelsPanel.add(valueLabel);
	labelsPanel.add(labelLabel);

	JTextField valueField = new JTextField(10);
	JTextField labelField = new JTextField(10);

	if (source != null) {
		valueField.setText(String.valueOf(source.getValue()));
		labelField.setText(source.getLabel());
	}

	JPanel fieldsPanel = new JPanel(new GridLayout(2,0,5,5));
	fieldsPanel.add(valueField);
	fieldsPanel.add(labelField);

	JPanel content = createPreferredSizePanel();
	content.add(labelsPanel, BorderLayout.WEST);
	content.add(fieldsPanel);

	int result = JOptionPane.showOptionDialog(
			owner,
			content,
			getLocalizedString(titleKey),
			JOptionPane.OK_CANCEL_OPTION,
			JOptionPane.PLAIN_MESSAGE,
			null,
			null,
			null);

	if (result == JOptionPane.OK_OPTION) {

		Double d = parseDoubleInput(valueField);

		if (d != null) {

			if (source != null) {
				source.setValue(d.doubleValue());
				source.setLabel(labelField.getText());
				return null;
			}
			else {
				return new JCValueLabel(d.doubleValue(), labelField.getText());
			}
		}
	}
	return null;
}

	private static class ValueLabelsModel extends AbstractListModel {

	private List	valueLabels;
	private List	valueLabelsAsStrings;

	public ValueLabelsModel() {
		valueLabels = new ArrayList();
		valueLabelsAsStrings = new ArrayList();
	}

	public void setValueLabels(JCValueLabel[] valueLabels) {

		if (getSize() > 0) {
			fireIntervalRemoved(this, 0, getSize() - 1);
			this.valueLabels.clear();
			this.valueLabelsAsStrings.clear();
		}

		if (valueLabels == null) {
			return;
		}

		for (int each = 0; each < valueLabels.length; each++) {
			this.valueLabels.add(valueLabels[each]);
			this.valueLabelsAsStrings.add(toString(valueLabels[each]));
		}

		if (getSize() > 0) {
			fireIntervalAdded(this, 0, getSize() - 1);
		}
	}

	public Object getElementAt(int index) {
		return valueLabelsAsStrings.get(index);
	}

	/**
	 * Use this method to get a value label from this model.
	 */
	public JCValueLabel getValueLabelAt(int index) {
		return (JCValueLabel)valueLabels.get(index);
	}

	public int getSize() {
		return valueLabelsAsStrings.size();
	}

	public void remove(int index) {
		valueLabels.remove(index);
		valueLabelsAsStrings.remove(index);
		fireIntervalRemoved(this, index, index);
	}

	public void replace(int index, JCValueLabel valueLabel) {
		remove(index);
		insert(index, valueLabel);
	}

	public void insert(int index, JCValueLabel valueLabel) {
		valueLabels.add(index, valueLabel);
		valueLabelsAsStrings.add(index, toString(valueLabel));
		fireIntervalAdded(this, index, index);
	}

	private String toString(JCValueLabel valueLabel) {
		return valueLabel.getValue() + "," + valueLabel.getLabel();
	}

	public void add(JCValueLabel valueLabel) {
		insert(getSize(), valueLabel);
	}

	public void update(int index) {
		fireContentsChanged(this, index, index);
	}

	public JCValueLabel[] getValueLabels() {
		return (JCValueLabel[])valueLabels.toArray(new JCValueLabel[0]);
	}

	}

}
