/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Chart3dPropertyEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:59 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.JCChart3d;

import com.klg.jclass.chart3d.customizer.util.JCBorderChooser;
import com.klg.jclass.chart3d.customizer.util.JCColorChooser;

import com.klg.jclass.util.JCLocaleManager;
import com.klg.jclass.util.swing.JCFontChooserPane;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.BorderLayout;

import java.awt.event.ActionListener;
import java.awt.event.FocusListener;
import javax.swing.event.ListSelectionListener;

import java.util.List;
import java.util.ArrayList;

import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.BorderFactory;

import javax.swing.border.EmptyBorder;

import javax.swing.text.JTextComponent;

public abstract class Chart3dPropertyEditor extends AbstractPropertyEditor {

private static JCLocaleManager		localeManager = new JCLocaleManager(
					"com.klg.jclass.chart3d.customizer.resources.LocaleInfo");

private String					nameKey;
private List					children;

public Chart3dPropertyEditor(String nameKey) {
	this(nameKey, null);
}

public Chart3dPropertyEditor(String nameKey, String[] childClasses) {
	this(nameKey, null, childClasses);
}

public Chart3dPropertyEditor(String nameKey, PropertyEditor parent, String[] childClasses) {
	super(parent);

	this.nameKey = nameKey;

	if (childClasses != null && childClasses.length > 0) {
		int childCount = childClasses.length;
		children = new ArrayList(childCount);
		for (int each = 0; each < childCount; each++) {
			children.add(each, childClasses[each]);
		}
	}

	if (localeManager == null) {
		localeManager = JCLocaleManager.getDefault();
		localeManager.add("com.klg.jclass.chart3d.customizer.resources.LocaleInfo");
	}
}

/**
 * Default implementation of PropertyEditor.getEditor() which
 * returns null.
 */
public Component getEditor() {
	return null;
}

/**
 * Default implementation of PropertyEditor.refreshEditor() which
 * does nothing.
 */
public void refreshEditor() {
}

public void refreshEditorTree() {
	refreshEditorTree(true);
}

public void refreshEditorTree(boolean refreshMeToo) {

	for (int each  = 0; each < getChildCount(); each++) {
		if (childExistsAt(each)) {
			getChildAt(each).refreshEditorTree();
		}
	}

	if (refreshMeToo) {
		refreshEditor();
	}
}

/**
 * Default implementation of PropertyEditor.finalizeEditor() which
 * does nothing.
 */
public void finalizeEditor() {
}

public void finalizeEditorTree() {

	for (int each  = 0; each < getChildCount(); each++) {
		if (childExistsAt(each)) {
			getChildAt(each).finalizeEditorTree();
		}
	}

	finalizeEditor();
}

protected boolean childExistsAt(int index) {
	return children != null && children.get(index) instanceof PropertyEditor;
}

public int getChildCount() {

	if (children == null) {
		return 0;
	}

	return children.size();
}

/**
 * Retrieves the child at the specified index, provided the
 * index is not < 0 or >= getChildCount().
 *
 * @return - the child at the specified index belonging to this
 * PropertyEditor
 * @throw - IndexOutOfBoundsException if the specified index
 * is < 0 or >= getChildCount().
 */
public PropertyEditor getChildAt(int index) {

	if (childExistsAt(index)) {
		return (PropertyEditor)children.get(index);
	}

	PropertyEditor editor = null;

	String className = (String)children.remove(index);
	try {
		Class clazz = Class.forName(className);

		editor = (PropertyEditor)clazz.newInstance();
		editor.setParent(this);

	} catch (Exception e) {
		System.out.println("Class " + className + " not found.");
		System.out.println(e.toString());
		editor = new NotFoundEditor();
	}

	// Stick the instance into the children list.
	children.add(index, editor);

	return editor;
}

public boolean isLeaf() {
	return getChildCount() == 0;
}

public String getName() {
	return getLocalizedString(nameKey);
}

public JCChart3d getChart() {
	return (JCChart3d)getCustomizer().getTarget();
}

public AbstractChart3dCustomizer getCustomizer() {
	return (AbstractChart3dCustomizer)getRoot();
}

public int getIndexOfChild(PropertyEditor aChild) {

	if (isLeaf()) {
		return -1;
	}

	return children.indexOf(aChild);
}

public String toString() {
	return getName();
}

/**
 * Returns the same instance of JCBorderChooser which is a member of this customizer.
 * By using this method, property editor's that use a border chooser may save time by
 * avoiding any object instantiation.
 */
public JCBorderChooser getBorderChooser() {
	return new JCBorderChooser();
}

/**
 * Returns the same instance of JCColorChooser which is a member of this customizer.
 * By using this method, property editor's that use a color chooser may save time by
 * avoiding any object instantiation.
 */
public JCColorChooser getColorChooser() {
	return new JCColorChooser();
}

/**
 * Returns the same instance of JCFontChooserPane which is a member of this customizer.
 * By using this method, property editor's that use a font chooser may save time by
 * avoiding any object instantiation.
 */
public JCFontChooserPane getFontChooser() {
	return new JCFontChooserPane();
}

public static JCLocaleManager getLocaleManager() {
	return localeManager;
}

public static String getLocalizedString(String key) {

	if (key == null) {
		return "";
	}

	String str = localeManager.getString(key);
	if (str == null) {
		str = key;
	}

	return str;
}

/**
 * A convenience method to construct a check box with the specified text and
 * action listener.
 */
public static JCheckBox createCheckBox(String text, ActionListener listener) {

	JCheckBox checkBox = new JCheckBox(text);
	checkBox.addActionListener(listener);

	return checkBox;
}

/**
 * A convenience method to construct a list box with the specified model and
 * list selection listener.
 */
public static JList createList(String[] model, ListSelectionListener listener) {

	JList list = new JList(model);
	list.setBorder(BorderFactory.createEtchedBorder());
	list.addListSelectionListener(listener);
	list.setSelectedIndex(0);

	return list;
}

/**
 * A convenience method to construct a radio button with the specified text and
 * action listener.
 */
public static JRadioButton createRadioButton(String text, ActionListener listener) {

	JRadioButton button = new JRadioButton(text);
	button.addActionListener(listener);

	return button;
}

/**
 * A convenience method to construct a button with the specified text and
 * action listener.
 */
public static JButton createButton(String text, ActionListener listener) {

	JButton button = new JButton(text);
	button.addActionListener(listener);

	return button;
}

/**
 * A convenience method to construct a text field with the specified width, action
 * listener and focus listener.
 */
public static JTextField createTextField(int width, ActionListener actionListener, FocusListener focusListener) {

	JTextField field  = new JTextField(width);
	field.addActionListener(actionListener);
	field.addFocusListener(focusListener);

	return field;
}

/**
 * A convenience method to construct a combo box with the specified model, and
 * action listener.
 */
public static JComboBox createComboBox(Object[] model, ActionListener actionListener) {

	JComboBox comboBox  = new JComboBox(model);
	comboBox.addActionListener(actionListener);

	return comboBox;
}

/**
 * A convenience method to construct a panel with a padded border.
 */
public static JPanel createBorderPanel() {

	JPanel panel = new JPanel(new BorderLayout(5,5));
	panel.setBorder(new EmptyBorder(5,5,5,5));

	return panel;
}

/**
 * A convenience method to construct a panel with a padded border.
 */
public static JPanel createBorderPanel(int top, int left, int bottom, int right) {

	JPanel panel = new JPanel(new BorderLayout(5,5));
	panel.setBorder(new EmptyBorder(top, left, bottom, right));

	return panel;
}

/**
 * A convenience method to construct a panel with a padded border.
 */
public static JPanel createPreferredSizePanel() {

	JPanel panel = new JPanel(new BorderLayout(5,5)) {
		public Dimension getMaximumSize() {
			return getPreferredSize();
		}
	};
	panel.setBorder(new EmptyBorder(5,5,5,5));

	return panel;
}

public static void doError(Component component, String messageKey, String titleKey) {

	JOptionPane.showMessageDialog(
			component,
			getLocalizedString(messageKey),
			getLocalizedString(titleKey),
			JOptionPane.ERROR_MESSAGE);
}

public static Integer parseIntegerInput(JTextComponent component) {

	try {
		return new Integer(component.getText());
	} catch (NumberFormatException e) {
		doError(component, LocaleBundle.ERROR_INTEGERS_ONLY, LocaleBundle.STRING_ERROR);
	}
	return null;
}

public static Double parseDoubleInput(JTextComponent component) {

	try {
		return new Double(component.getText());
	} catch (NumberFormatException e) {
		doError(component, LocaleBundle.ERROR_ILLEGAL_VALUE, LocaleBundle.STRING_ERROR);
	}
	return null;
}

}
