/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ChartAreaBarEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:59 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.JCBar;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.BorderLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import java.util.Arrays;

import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JCheckBox;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.Box;

public class ChartAreaBarEditor extends Chart3dPropertyEditor implements ActionListener, FocusListener {

private static final String		nameKey = LocaleBundle.STRING_BAR;

private boolean					refreshing = false;
private Container				content;
private JTextField				xSpacingField;
private JComboBox				xFormatComboBox;
private JTextField				ySpacingField;
private JComboBox				yFormatComboBox;
private Object[]				formatModel;

public ChartAreaBarEditor() {
	super(nameKey);
}

public Component getEditor() {

	if (content == null) {

		String local;

		local = getLocalizedString(LocaleBundle.STRING_X_FORMAT);
		JLabel xFormatLabel =  new JLabel(local);

		local = getLocalizedString(LocaleBundle.STRING_Y_FORMAT);
		JLabel yFormatLabel =  new JLabel(local);

		local = getLocalizedString(LocaleBundle.STRING_X_SPACING);
		JLabel xSpacingLabel =  new JLabel(local);

		local = getLocalizedString(LocaleBundle.STRING_Y_SPACING);
		JLabel ySpacingLabel =  new JLabel(local);

		formatModel = new ComboBoxEntry[] {
			new ComboBoxEntry(JCBar.FIXED, getLocalizedString(LocaleBundle.STRING_FIXED)),
			new ComboBoxEntry(JCBar.HISTOGRAM, getLocalizedString(LocaleBundle.STRING_HISTOGRAM))
		};

		// Sort the model so that we can map a constant to an entry using the Arrays.find
		// method.
		Arrays.sort(formatModel);

		xFormatComboBox = createComboBox(formatModel, this);

		yFormatComboBox = createComboBox(formatModel, this);

		xSpacingField = createTextField(10, this, this);

		ySpacingField = createTextField(10, this, this);

		JPanel labels = new JPanel(new GridLayout(4,0,5,5));
		labels.add(xFormatLabel);
		labels.add(xSpacingLabel);
		labels.add(yFormatLabel);
		labels.add(ySpacingLabel);

		JPanel fields = new JPanel(new GridLayout(4,0,5,5));
		fields.add(xFormatComboBox);
		fields.add(xSpacingField);
		fields.add(yFormatComboBox);
		fields.add(ySpacingField);

		JPanel controls = createBorderPanel();
		controls.setBorder(null);
		controls.add(labels, BorderLayout.WEST);
		controls.add(fields);

		JPanel panel = createPreferredSizePanel();
		panel.add(controls);

		content = Box.createVerticalBox();
		content.add(panel);
	}
	return content;
}

public void finalizeEditor() {

	if (content != null) {

		xSpacingField.removeActionListener(this);
		xSpacingField.removeFocusListener(this);
		xSpacingField = null;

		xFormatComboBox.removeActionListener(this);
		xFormatComboBox = null;

		ySpacingField.removeActionListener(this);
		ySpacingField.removeFocusListener(this);
		ySpacingField = null;

		yFormatComboBox.removeActionListener(this);
		yFormatComboBox = null;

		content = null;
	}
}

public void refreshEditor() {

	if (content != null) {

		refreshing = true;

		JCBar bar = getChart().getChart3dArea().getBar();

		MutableComboBoxEntry entry = new MutableComboBoxEntry();

		entry.setConstant(bar.getXFormat());
		xFormatComboBox.setSelectedIndex(Arrays.binarySearch(formatModel, entry));

		entry.setConstant(bar.getYFormat());
		yFormatComboBox.setSelectedIndex(Arrays.binarySearch(formatModel, entry));

		xSpacingField.setText(String.valueOf(bar.getXSpacing()));
		ySpacingField.setText(String.valueOf(bar.getYSpacing()));

		refreshing = false;
	}
}

public void actionPerformed(ActionEvent e) {
	handleChange(e.getSource());
}

public void focusGained(FocusEvent e) {
	// Do nothing!
}

public void focusLost(FocusEvent e) {

	if (!e.isTemporary()) {
		handleChange(e.getSource());
	}
}

private void handleChange(Object source) {

	if (refreshing) {
		return;
	}

	if (content != null) {

		JCBar bar = getChart().getChart3dArea().getBar();

		if (source instanceof JComboBox) {

			Object	selection = ((JComboBox)source).getSelectedItem();
			int		constant = ((ComboBoxEntry)selection).getConstant();

			if (source == xFormatComboBox) {
				bar.setXFormat(constant);
			}
			else {
				bar.setYFormat(constant);
			}
		}
		else if (source instanceof JTextField) {

			Double d = parseDoubleInput((JTextField)source);

			if (d == null || d.doubleValue() < 0) {
				refreshEditor();
				return;
			}

			if (source == xSpacingField) {
				bar.setXSpacing(d.doubleValue());
			}
			else {
				bar.setYSpacing(d.doubleValue());
			}
		}
	}
}

}
