/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ChartAreaSurfaceEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:59 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.JCSurface;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.BorderLayout;
import java.awt.FlowLayout;

import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JCheckBox;
import javax.swing.Box;

public class ChartAreaSurfaceEditor extends Chart3dPropertyEditor
	implements ActionListener, FocusListener {

private static final String		nameKey = LocaleBundle.STRING_SURFACE;

private boolean					refreshing = false;
private Container				content;
private JCheckBox 				isSolidCheckBox;
private JCheckBox 				isXMeshCheckBox;
private JCheckBox 				isYMeshCheckBox;
private JTextField 				xMeshFilterField;
private JTextField 				yMeshFilterField;

public ChartAreaSurfaceEditor() {
	super(nameKey);
}


public Component getEditor() {

	if (content == null) {

		String local;

		local = getLocalizedString(LocaleBundle.STRING_SOLID);
		isSolidCheckBox = createCheckBox(local, this);

		local = getLocalizedString(LocaleBundle.STRING_X_MESH);
		isXMeshCheckBox = createCheckBox(local, this);

		local = getLocalizedString(LocaleBundle.STRING_Y_MESH);
		isYMeshCheckBox = createCheckBox(local, this);

		local = getLocalizedString(LocaleBundle.STRING_X_MESH_FILTER);
		JLabel xMeshFilterLabel = new JLabel(local);

		xMeshFilterField = createTextField(5, this, this);

		local = getLocalizedString(LocaleBundle.STRING_Y_MESH_FILTER);
		JLabel yMeshFilterLabel = new JLabel(local);

		yMeshFilterField = createTextField(5, this, this);

		JPanel xPanel = createBorderPanel();
		xPanel.setBorder(null);
		xPanel.add(xMeshFilterLabel, BorderLayout.WEST);
		xPanel.add(xMeshFilterField);

		JPanel yPanel = createBorderPanel();
		yPanel.setBorder(null);
		yPanel.add(yMeshFilterLabel, BorderLayout.WEST);
		yPanel.add(yMeshFilterField);

		JPanel grid = createPreferredSizePanel();
		grid.setLayout(new GridLayout(0,1,5,5));
		grid.add(isSolidCheckBox);
		grid.add(isXMeshCheckBox);
		grid.add(isYMeshCheckBox);
		grid.add(xPanel);
		grid.add(yPanel);

		content = Box.createVerticalBox();
		content.add(grid);
	}
	return content;
}

public void finalizeEditor() {

	if (content != null) {

		isSolidCheckBox.removeActionListener(this);
		isSolidCheckBox = null;

		isXMeshCheckBox.removeActionListener(this);
		isXMeshCheckBox = null;

		isYMeshCheckBox.removeActionListener(this);
		isYMeshCheckBox = null;

		xMeshFilterField.removeActionListener(this);
		xMeshFilterField.removeFocusListener(this);
		xMeshFilterField = null;

		yMeshFilterField.removeActionListener(this);
		yMeshFilterField.removeFocusListener(this);
		yMeshFilterField = null;

		content = null;
	}
}

public void refreshEditor() {

	if (content != null) {

		refreshing = true;

		JCSurface surface = getChart().getChart3dArea().getSurface();

		isSolidCheckBox.setSelected(surface.isSolid());
		isXMeshCheckBox.setSelected(surface.isXMeshShowing());
		isYMeshCheckBox.setSelected(surface.isYMeshShowing());

		xMeshFilterField.setText(String.valueOf(surface.getXMeshFilter()));
		yMeshFilterField.setText(String.valueOf(surface.getYMeshFilter()));

		refreshing = false;
	}
}

public void actionPerformed(ActionEvent e) {
	changePerformed(e.getSource());
}

public void focusLost(FocusEvent e) {
	if (!e.isTemporary()) {
		changePerformed(e.getSource());
	}
}

public void focusGained(FocusEvent e) {	}

public void changePerformed(Object source) {

	if (refreshing) {
		return;
	}

	if (content != null) {

		JCSurface surface = getChart().getChart3dArea().getSurface();

		if (source == isSolidCheckBox) {
			surface.setSolid(isSolidCheckBox.isSelected());
		}
		else if (source == isXMeshCheckBox) {
			surface.setXMeshShowing(isXMeshCheckBox.isSelected());
		}
		else if (source == isYMeshCheckBox) {
			surface.setYMeshShowing(isYMeshCheckBox.isSelected());
		}
		else if (source == xMeshFilterField) {

			Integer i = parseIntegerInput(xMeshFilterField);

			if (i != null && i.intValue() >= 0) {
				surface.setXMeshFilter(i.intValue());
			}
			else {
				refreshEditor();
			}
		}
		else if (source == yMeshFilterField) {

			Integer i = parseIntegerInput(yMeshFilterField);

			if (i != null && i.intValue() >= 0) {
				surface.setYMeshFilter(i.intValue());
			}
			else {
				refreshEditor();
			}
		}
	}
}

}
