/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataViewChartStyleEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:59 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.*;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.List;

import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JComponent;
import javax.swing.Box;
import javax.swing.BoxLayout;

import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

public class DataViewChartStyleEditor extends Chart3dPropertyEditor implements ListSelectionListener {

private static String[]		children = {
		"com.klg.jclass.chart3d.customizer.DataViewChartStyleLineStyleEditor",
		"com.klg.jclass.chart3d.customizer.DataViewChartStyleSymbolStyleEditor"
};

private static final String		nameKey = LocaleBundle.STRING_GRID_STYLE;

private boolean					refreshing = false;
private Container				content;
private Container				dataListContainer;
private JList					dataList;

public DataViewChartStyleEditor() {
	super(nameKey, children);
}

public DataViewChartStyleEditor(PropertyEditor parent) {
	super(nameKey, parent, children);
}

public Component getEditor() {

	if (content == null) {

		dataList = new JList();

		dataListContainer = createPreferredSizePanel();
		dataListContainer.setLayout(new GridLayout(0,1,5,5));
		dataListContainer.add(dataList);

		content = Box.createVerticalBox();
		content.add(dataListContainer);
	}
	return content;
}

public void valueChanged(ListSelectionEvent e) {

	if (refreshing) {
		return;
	}

	// The selected series has changed, OR, it is possible that the
	// actual data model has changed.
	refreshEditorTree();
}

/**
 * Returns the JCAxis which is currently being edited by this editor.
 * Children of this editor should always modify the JCAxis instance
 * returned by this method.
 */
public Chart3dData getSelectedData() {

	if (content != null) {

		DataViewEditor		parent = (DataViewEditor)getParent();
		Chart3dDataView		dataView = parent.getSelectedDataView();

		return dataView.getElevationData();
	}
	return null;
}

public boolean hasPointData() {
	return getSelectedData() instanceof Chart3dPointData;
}

public boolean hasGridData() {
	return getSelectedData() instanceof Chart3dGridData;
}

public JCChart3dStyle getSelectedChartStyle() {

	if (hasGridData()) {
		return ((Chart3dGridData)getSelectedData()).getChartStyle();
	}

	int sel = dataList.getSelectedIndex();

	if (sel != -1) {
		return ((Chart3dPointData)getSelectedData()).getSeries(sel).getChartStyle();
	}
	return null;
}

public void finalizeEditor() {

	if (content != null) {

		dataList.removeListSelectionListener(this);
		dataList = null;

		dataListContainer = null;

		content = null;
	}
}

public void refreshEditor() {
	// The selected series has changed, OR, it is possible that the
	// actual data model has changed.

	// Reconstruct the dataList...

	if (content != null) {

		refreshing = true;

		String[] dataListModel = null;

		if (hasPointData()) {

			Chart3dPointData pointData = (Chart3dPointData)getSelectedData();

			int numSeries = pointData.getNumSeries();

			dataListModel = new String[numSeries];

			String series = getLocalizedString(LocaleBundle.STRING_SERIES);

			for (int each = 0; each < numSeries; each++) {
				dataListModel[each] = series + " " + each;
			}
		}
		else {
			String data = getLocalizedString(LocaleBundle.STRING_DATA_VIEW);
			dataListModel = new String[] { data + " " + 0 };
		}


		dataList.removeListSelectionListener(this);
		dataList = createList(dataListModel, this);
		dataList.setPrototypeCellValue("XXXXXXXXXXXX");

		dataListContainer.removeAll();
		dataListContainer.add(dataList);

		refreshing = false;
	}
}

}
