/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataViewContourLevelsEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:12:59 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.Chart3dDataView;
import com.klg.jclass.chart3d.JCContourLevels;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.BorderLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import java.util.List;
import java.util.Collections;
import java.util.ArrayList;

import javax.swing.JList;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JButton;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.JPanel;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.AbstractListModel;
import javax.swing.ScrollPaneConstants;

import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

public class DataViewContourLevelsEditor extends Chart3dPropertyEditor
	implements ActionListener, ListDataListener, FocusListener {

private static final String		nameKey  = LocaleBundle.STRING_CONTOUR_LEVELS;

private boolean					refreshing = false;
private Container				content;
private JRadioButton			useDefaultLevelsButton;
private JRadioButton			setLevelsButton;
private JTextField				numLevelsField;
private JList					levelsList;
private JButton					addButton;
private JButton					removeButton;
private LevelsModel				levelsModel;

public DataViewContourLevelsEditor() {
	super(nameKey);
}

public Component getEditor() {

	if (content == null) {

		String local;

		local = getLocalizedString(LocaleBundle.STRING_USE_DEFAULT_LEVELS);
		useDefaultLevelsButton = createRadioButton(local, this);
		useDefaultLevelsButton.setAlignmentX(0);

		local = getLocalizedString(LocaleBundle.STRING_SET_LEVELS);
		setLevelsButton = createRadioButton(local, this);
		setLevelsButton.setAlignmentX(0);

		local = getLocalizedString(LocaleBundle.STRING_LEVELS);
		JLabel numLevelsLabel = new JLabel(local);

		numLevelsField = createTextField(10, this, this);

		local = getLocalizedString(LocaleBundle.STRING_ADD);
		addButton = createButton(local, this);

		local = getLocalizedString(LocaleBundle.STRING_REMOVE);
		removeButton = createButton(local, this);

		levelsModel = new LevelsModel();
		levelsModel.addListDataListener(this);

		levelsList = new JList(levelsModel);
 		levelsList.setPrototypeCellValue("XXXXXXXXXXXX");

		JScrollPane scrollPane = new JScrollPane(
				levelsList,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED
				);
		scrollPane.setAlignmentX(0);

		JPanel numLevelsPanel = createBorderPanel(0,15,0,0);
		numLevelsPanel.add(numLevelsLabel, BorderLayout.WEST);
		numLevelsPanel.add(numLevelsField);
		numLevelsPanel.setAlignmentX(0);

		JPanel buttonPanel = new JPanel(new GridLayout(0,2,5,5));
		buttonPanel.add(addButton);
		buttonPanel.add(removeButton);
		buttonPanel.setAlignmentX(0);

//		JPanel controlPanel = createBorderPanel();
//		controlPanel.setBorder(null);
//		controlPanel.add(buttonPanel, BorderLayout.WEST);
//		controlPanel.setAlignmentX(0);

		content = createBorderPanel();
		content.setLayout(new BoxLayout(content, BoxLayout.Y_AXIS));
		content.add(useDefaultLevelsButton);
		content.add(numLevelsPanel);
		content.add(setLevelsButton);
		content.add(scrollPane);
		content.add(Box.createVerticalStrut(5));
//		content.add(controlPanel);
		content.add(buttonPanel);
	}
	return content;
}

public void finalizeEditor() {

	if (content != null) {

		useDefaultLevelsButton.removeActionListener(this);
		useDefaultLevelsButton = null;

		setLevelsButton.removeActionListener(this);
		setLevelsButton = null;

		numLevelsField.removeActionListener(this);
		numLevelsField.removeFocusListener(this);
		numLevelsField = null;

		levelsModel.removeListDataListener(this);
		levelsModel = null;

		levelsList = null;

		addButton.removeActionListener(this);
		addButton = null;

		removeButton.removeActionListener(this);
		removeButton = null;

		content = null;
	}
}

public void refreshEditor() {

	if (content != null) {

		refreshing = true;

		DataViewEditor		parent = (DataViewEditor)getParent();
		Chart3dDataView		dataView = parent.getSelectedDataView();
		JCContourLevels		levels = dataView.getContour().getContourLevels();

		boolean isDefault = levels.isDefault();

		useDefaultLevelsButton.setSelected(isDefault);

		numLevelsField.setEnabled(isDefault);
		numLevelsField.setText(String.valueOf(levels.getNumLevelsInternal()));

		setLevelsButton.setSelected(!isDefault);

		addButton.setEnabled(!isDefault);
		removeButton.setEnabled(!isDefault);

		levelsModel.setLevels(levels.getLevels());

		refreshing = false;
	}
}

public void contentsChanged(ListDataEvent e) {
	handleChange();
}

public void intervalAdded(ListDataEvent e) {
	handleChange();
}

public void intervalRemoved(ListDataEvent e) {
	handleChange();
}

public void handleChange() {

	if (refreshing) {
		return;
	}

	if (content != null) {

		DataViewEditor		parent = (DataViewEditor)getParent();
		Chart3dDataView		dataView = parent.getSelectedDataView();
		JCContourLevels 	levels = dataView.getContour().getContourLevels();

		levels.setLevels(levelsModel.getLevels());
	}
}

public void actionPerformed(ActionEvent e) {

	if (refreshing) {
		return;
	}

	if (content != null) {

		DataViewEditor		parent = (DataViewEditor)getParent();
		Chart3dDataView		dataView = parent.getSelectedDataView();
		JCContourLevels 	levels = dataView.getContour().getContourLevels();

		Object source = e.getSource();

		if (source == addButton) {

			Double newLevel = showLevelsDialog(addButton, LocaleBundle.STRING_ADD);

			if (newLevel != null) {
				levelsModel.add(newLevel.doubleValue());
			}
		}
		else if (source == removeButton) {
			if (levelsList.getSelectedIndex() != -1) {
				levelsModel.remove(levelsList.getSelectedIndex());
			}
		}
		else if (source == useDefaultLevelsButton) {
			levels.setDefault(true);
		}
		else if (source == setLevelsButton) {
			levels.setDefault(false);
		}
		else if (source == numLevelsField) {
			handleNumLevelsChange();
		}
	}
}

public void focusLost(FocusEvent e) {
	handleNumLevelsChange();
}

public void focusGained(FocusEvent e) {	}


private void handleNumLevelsChange() {

	if (refreshing) {
		return;
	}

	DataViewEditor		parent = (DataViewEditor)getParent();
	Chart3dDataView		dataView = parent.getSelectedDataView();
	JCContourLevels 	levels = dataView.getContour().getContourLevels();

	Integer i = parseIntegerInput(numLevelsField);

	if (i == null || i.intValue() <= 2) {
		refreshEditor();
		return;
	}

	levels.setNumLevels(i.intValue());
}

private Double showLevelsDialog(Component owner, String titleKey) {

	String local;

	local = getLocalizedString(LocaleBundle.STRING_LEVELS);
	JLabel levelLabel = new JLabel(local);

	JTextField levelsField = new JTextField(10);

	JPanel content = createPreferredSizePanel();
	content.add(levelLabel, BorderLayout.WEST);
	content.add(levelsField);

	int result = JOptionPane.showOptionDialog(
			owner,
			content,
			getLocalizedString(titleKey),
			JOptionPane.OK_CANCEL_OPTION,
			JOptionPane.PLAIN_MESSAGE,
			null,
			null,
			null);

	if (result == JOptionPane.OK_OPTION) {
		return parseDoubleInput(levelsField);
	}
	return null;
}

	private static class LevelsModel extends AbstractListModel {

	private List	levels;

	public LevelsModel() {
		levels = new ArrayList();
	}

	public void setLevels(double[] levels) {

		if (getSize() > 0) {
			fireIntervalRemoved(this, 0, getSize() - 1);
			this.levels.clear();
		}

		if (levels == null) {
			return;
		}

		for (int each = 0; each < levels.length; each++) {
			this.levels.add(new Double(levels[each]));
		}

		if (getSize() > 0) {
			sortLevels();
			fireIntervalAdded(this, 0, getSize() - 1);
		}
	}

	public Object getElementAt(int index) {
		return levels.get(index);
	}

	/**
	 * Use this method to get a value label from this model.
	 */
	public double getLevelAt(int index) {
		return ((Double)levels.get(index)).doubleValue();
	}

	public int getSize() {
		return levels.size();
	}

	public void remove(int index) {
		levels.remove(index);
		fireIntervalRemoved(this, index, index);
	}

	public void add(double level) {
		levels.add(new Double(level));
		sortLevels();
		fireIntervalAdded(this, getSize(), getSize());
	}

	private void sortLevels() {
		Collections.sort(levels);
	}

	public double[] getLevels() {

		double[] levelArr = new double[getSize()];

		for (int each = 0; each < getSize(); each++) {
			levelArr[each] = getLevelAt(each);
		}

		return levelArr;
	}

	}

}
