/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataViewContourStylesEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:00 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.customizer.util.JCColorChooser;
import com.klg.jclass.chart3d.*;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.BorderLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.Collections;
import java.util.ArrayList;

import javax.swing.JList;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JButton;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.JPanel;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.AbstractListModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingUtilities;

import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class DataViewContourStylesEditor extends Chart3dPropertyEditor
	implements ActionListener, ListDataListener, ListSelectionListener {

private static String[]		children = {
				"com.klg.jclass.chart3d.customizer.DataViewContourStyleLineStyleEditor",
				"com.klg.jclass.chart3d.customizer.DataViewContourStyleSymbolStyleEditor",
				"com.klg.jclass.chart3d.customizer.DataViewContourStyleFillStyleEditor"
};

private static final String		nameKey  = LocaleBundle.STRING_CONTOUR_STYLES;
private static String			style = Chart3dPropertyEditor.getLocalizedString(LocaleBundle.STRING_STYLE);

private boolean					refreshing = false;
private Container				content;
private JList					contourStylesList;
private JButton					addButton;
private JButton					removeButton;
private JButton					resetButton;
private ContourStylesModel		contourStylesModel;

public DataViewContourStylesEditor() {
	super(nameKey, children);
}

public Component getEditor() {

	if (content == null) {

		String local;

		local = getLocalizedString(LocaleBundle.STRING_ADD);
		addButton = createButton(local, this);

		local = getLocalizedString(LocaleBundle.STRING_REMOVE);
		removeButton = createButton(local, this);

		local = getLocalizedString(LocaleBundle.STRING_RESET_STYLES);
		resetButton = createButton(local, this);

		contourStylesModel = new ContourStylesModel();
		contourStylesModel.addListDataListener(this);

		contourStylesList = new JList(contourStylesModel);
		contourStylesList.addListSelectionListener(this);
 		contourStylesList.setPrototypeCellValue("XXXXXXXXXXXX");

		JScrollPane scrollPane = new JScrollPane(contourStylesList);
		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setAlignmentX(0);

		JPanel controls = createPreferredSizePanel();
		controls.setLayout(new GridLayout(3,0,5,5));
		controls.add(addButton);
		controls.add(removeButton);
		controls.add(resetButton);
		controls.setAlignmentX(0);

		content = createPreferredSizePanel();
		content.setLayout(new BoxLayout(content, BoxLayout.Y_AXIS));
		content.add(scrollPane);
		content.add(Box.createVerticalStrut(5));
		content.add(controls);
		content.add(Box.createVerticalGlue());
	}
	return content;
}

public void finalizeEditor() {

	if (content != null) {

		contourStylesList.removeListSelectionListener(this);
		contourStylesList = null;

		contourStylesModel.removeListDataListener(this);
		contourStylesModel = null;

		contourStylesList = null;

		addButton.removeActionListener(this);
		addButton = null;

		removeButton.removeActionListener(this);
		removeButton = null;

		resetButton.removeActionListener(this);
		resetButton = null;

		content = null;
	}
}

public void refreshEditor() {

	if (content != null) {

		refreshing = true;

		DataViewEditor		parent = (DataViewEditor)getParent();
		JCContour			contour = parent.getSelectedDataView().getContour();

		contourStylesModel.setContourStyles(contour.getContourStyles());

		refreshing = false;
	}
}

public void contentsChanged(ListDataEvent e) {
	handleContourStyleChange();
}

public void intervalAdded(ListDataEvent e) {
	handleContourStyleChange();
}

public void intervalRemoved(ListDataEvent e) {
	handleContourStyleChange();
}

public void valueChanged(ListSelectionEvent e) {
	// The selected contour style has changed.  Better notify the children...
	if (!refreshing && !e.getValueIsAdjusting()) {
		// But don't refresh me again!
		refreshEditorTree(false);
	}
}

public void actionPerformed(ActionEvent e) {

	if (refreshing) {
		return;
	}

	if (content != null) {

		DataViewEditor		parent = (DataViewEditor)getParent();
		JCContour			contour = parent.getSelectedDataView().getContour();

		Object source = e.getSource();

		if (source == addButton) {
			contourStylesModel.add(new JCContourStyle());
		}
		else if (source == removeButton) {
			if (getSel() != -1) {
				contourStylesModel.remove(getSel());
			}
		}
		else if (source == resetButton) {
			contour.resetContourStyles();
		}
	}
}

private void handleContourStyleChange() {

	if (refreshing) {
		return;
	}

	DataViewEditor		parent = (DataViewEditor)getParent();
	JCContour			contour = parent.getSelectedDataView().getContour();

	if (contour != null) {
		contour.setContourStyles(contourStylesModel.getContourStyles());
	}
}

private int getSel() {
	return contourStylesList.getSelectedIndex();
}

protected JCLineStyle getSelectedLineStyle() {
	if (getSel() != -1) {
		return contourStylesModel.getContourStyleAt(getSel()).getLineStyle();
	}
	return null;
}

protected JCFillStyle getSelectedFillStyle() {
	if (getSel() != -1) {
		return contourStylesModel.getContourStyleAt(getSel()).getFillStyle();
	}
	return null;
}

protected JCSymbolStyle getSelectedSymbolStyle() {
	if (getSel() != -1) {
		return contourStylesModel.getContourStyleAt(getSel()).getSymbolStyle();
	}
	return null;
}

	private static class ContourStylesModel extends AbstractListModel {

	private ArrayList	contourStyles;

	public ContourStylesModel() {
	}

	public void setContourStyles(ArrayList contourStyles) {

		if (getSize() > 0) {
			fireIntervalRemoved(this, 0, getSize() - 1);
		}

		if (contourStyles == null) {
			return;
		}

		this.contourStyles = contourStyles;

		if (getSize() > 0) {
			fireIntervalAdded(this, 0, getSize() - 1);
		}
	}

	public ArrayList getContourStyles() {
		return contourStyles;
	}

	public Object getElementAt(int index) {
		return toString(index, (JCContourStyle)contourStyles.get(index));
	}

	/**
	 * Use this method to get a value label from this model.
	 */
	public JCContourStyle getContourStyleAt(int index) {
		return (JCContourStyle)contourStyles.get(index);
	}

	public int getSize() {
		if (contourStyles == null) {
			return 0;
		}
		return contourStyles.size();
	}

	public void remove(int index) {
		contourStyles.remove(index);
		fireIntervalRemoved(this, index, index);
	}

	public void add(JCContourStyle contourStyle) {
		contourStyles.add(contourStyle);
		fireIntervalAdded(this, getSize(), getSize());
	}

	private String toString(int index, JCContourStyle contourStyle) {

		if (contourStyle == null) {
			return "";
		}

		return style + " " + index;
	}

	}
}
