/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataViewDataLabelsEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:00 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.*;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.BorderLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.List;
import java.util.Collections;
import java.util.ArrayList;

import javax.swing.JList;
import javax.swing.JLabel;
import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JButton;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.JPanel;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.AbstractListModel;
import javax.swing.ScrollPaneConstants;

import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

public class DataViewDataLabelsEditor extends Chart3dPropertyEditor
	implements ActionListener, ListDataListener {

private static final String		nameKey  = LocaleBundle.STRING_DATA_LABELS;

private boolean					refreshing = false;
private Container				content;

private JComboBox				axisComboBox;
private JList					dataLabelsList;
private JButton					addButton;
private JButton					removeButton;
private DataLabelsModel			dataLabelsModel;

public DataViewDataLabelsEditor() {
	super(nameKey);
}

public Component getEditor() {

	if (content == null) {

		String local;


		String[] axisModel = {
				getLocalizedString(LocaleBundle.STRING_X_AXIS),
				getLocalizedString(LocaleBundle.STRING_Y_AXIS)
		};

		axisComboBox = new JComboBox(axisModel);
		axisComboBox.setAlignmentX(0);
		axisComboBox.addActionListener(this);

		local = getLocalizedString(LocaleBundle.STRING_ADD);
		addButton = createButton(local, this);

		local = getLocalizedString(LocaleBundle.STRING_REMOVE);
		removeButton = createButton(local, this);

		dataLabelsModel = new DataLabelsModel();
		dataLabelsModel.addListDataListener(this);

		dataLabelsList = new JList(dataLabelsModel);
 		dataLabelsList.setPrototypeCellValue("XXXXXXXXXXXX");

		JScrollPane scrollPane = new JScrollPane(
				dataLabelsList,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED
				);
		scrollPane.setAlignmentX(0);

		JPanel buttonPanel = new JPanel(new GridLayout(0,2,5,5));
		buttonPanel.add(addButton);
		buttonPanel.add(removeButton);
		buttonPanel.setAlignmentX(0);

		content = createBorderPanel();
		content.setLayout(new BoxLayout(content, BoxLayout.Y_AXIS));
		content.add(axisComboBox);
		content.add(Box.createVerticalStrut(5));
		content.add(scrollPane);
		content.add(Box.createVerticalStrut(5));
		content.add(buttonPanel);
	}
	return content;
}

public void finalizeEditor() {

	if (content != null) {

		axisComboBox.removeActionListener(this);
		axisComboBox = null;

		dataLabelsModel.removeListDataListener(this);
		dataLabelsModel = null;

		dataLabelsList = null;

		addButton.removeActionListener(this);
		addButton = null;

		removeButton.removeActionListener(this);
		removeButton = null;

		content = null;
	}
}

public void refreshEditor() {

	if (content != null) {

		refreshing = true;

		DataViewEditor		parent = (DataViewEditor)getParent();
		Chart3dDataView		dataView = parent.getSelectedDataView();

		Chart3dData			data = dataView.getElevationData();

		if (! (data instanceof Chart3dGridData)) {
			content.setEnabled(false);
			return;
		}

		// Must be grid data!
		content.setEnabled(false);

		Chart3dGridData gridData = (Chart3dGridData)data;

		int sel = axisComboBox.getSelectedIndex();

		if (sel != -1) {

			String[] dataLabels = null;
			if (sel == 0) {
				// X axis.
				dataLabels = gridData.getXLabels();
			}
			else { // Y axis.
				dataLabels = gridData.getYLabels();
			}

			dataLabelsModel.setDataLabels(dataLabels);
		}
		refreshing = false;
	}
}

public void contentsChanged(ListDataEvent e) {
	handleChange();
}

public void intervalAdded(ListDataEvent e) {
	handleChange();
}

public void intervalRemoved(ListDataEvent e) {
	handleChange();
}

public void handleChange() {

	if (refreshing) {
		return;
	}

	if (content != null) {

		DataViewEditor		parent = (DataViewEditor)getParent();
		Chart3dDataView		dataView = parent.getSelectedDataView();

		Chart3dGridData		gridData = (Chart3dGridData)dataView.getElevationData();

		int sel = axisComboBox.getSelectedIndex();

		if (sel != -1) {

			String[] dataLabels = null;
			if (sel == 0) {
				// X axis.
				gridData.setXLabels(dataLabelsModel.getDataLabels());
			}
			else { // Y axis.
				gridData.setYLabels(dataLabelsModel.getDataLabels());
			}
		}
	}
}

public void actionPerformed(ActionEvent e) {

	if (refreshing) {
		return;
	}

	if (content != null) {

		Object source = e.getSource();

		if (source == addButton) {

			String newDataLabel = showDataLabelsDialog(addButton, LocaleBundle.STRING_ADD);

			if (newDataLabel != null) {
				dataLabelsModel.add(newDataLabel);
			}
		}
		else if (source == removeButton) {
			if (dataLabelsList.getSelectedIndex() != -1) {
				dataLabelsModel.remove(dataLabelsList.getSelectedIndex());
			}
		}
		else if (source == axisComboBox) {
			refreshEditor();
		}
	}
}

private String showDataLabelsDialog(Component owner, String titleKey) {

	String local;

	local = getLocalizedString(LocaleBundle.STRING_DATA_LABELS);
	JLabel dataLabelLabel = new JLabel(local);

	JTextField dataLabelField = new JTextField(10);

	JPanel content = createPreferredSizePanel();
	content.add(dataLabelLabel, BorderLayout.WEST);
	content.add(dataLabelField);

	int result = JOptionPane.showOptionDialog(
			owner,
			content,
			getLocalizedString(titleKey),
			JOptionPane.OK_CANCEL_OPTION,
			JOptionPane.PLAIN_MESSAGE,
			null,
			null,
			null);

	if (result == JOptionPane.OK_OPTION) {
		return dataLabelField.getText();
	}
	return null;
}

	private static class DataLabelsModel extends AbstractListModel {

	private List	dataLabels;

	public DataLabelsModel() {
		dataLabels = new ArrayList();
	}

	public void setDataLabels(String[] dataLabels) {

		if (getSize() > 0) {
			fireIntervalRemoved(this, 0, getSize() - 1);
			this.dataLabels.clear();
		}

		if (dataLabels == null) {
			return;
		}

		for (int each = 0; each < dataLabels.length; each++) {
			this.dataLabels.add(dataLabels[each]);
		}

		if (getSize() > 0) {
			fireIntervalAdded(this, 0, getSize() - 1);
		}
	}

	public Object getElementAt(int index) {
		return dataLabels.get(index);
	}

	/**
	 * Use this method to get a value label from this model.
	 */
	public String getDataLabelAt(int index) {
		return (String)dataLabels.get(index);
	}

	public int getSize() {
		return dataLabels.size();
	}

	public void remove(int index) {
		dataLabels.remove(index);
		fireIntervalRemoved(this, index, index);
	}

	public void add(String dataLabel) {
		dataLabels.add(dataLabel);
		fireIntervalAdded(this, getSize(), getSize());
	}

	public String[] getDataLabels() {

		return (String[])dataLabels.toArray(new String[0]);
	}

	}

}
