/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataViewElevationEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:00 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.*;
import com.klg.jclass.chart3d.customizer.util.JCColorChooser;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.BorderLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.BorderFactory;

public class DataViewElevationEditor extends Chart3dPropertyEditor
	implements ActionListener {

private static String		nameKey = LocaleBundle.STRING_ELEVATION;

private boolean				refreshing = false;
private Container			content;
private JCColorChooser		colorChooser;
private JComboBox			colorComboBox;

public DataViewElevationEditor() {
	super(nameKey);
}

public Component getEditor() {

	if (content == null) {

		String local;

		local = getLocalizedString(LocaleBundle.STRING_ELEVATION);
		colorChooser = getColorChooser();
		colorChooser.addActionListener(this);
		colorChooser.setAlignmentX(0);
		colorChooser.setBorder(BorderFactory.createTitledBorder(local));

		String[] colorComboBoxModel = {
			getLocalizedString(LocaleBundle.STRING_MESH_TOP_COLOR),
			getLocalizedString(LocaleBundle.STRING_MESH_BOTTOM_COLOR),
			getLocalizedString(LocaleBundle.STRING_SHADED_TOP_COLOR),
			getLocalizedString(LocaleBundle.STRING_SHADED_BOTTOM_COLOR)
		};

		colorComboBox = new JComboBox(colorComboBoxModel);
		colorComboBox.setAlignmentX(0);
		colorComboBox.addActionListener(this);

		content = createBorderPanel();
		content.setLayout(new BoxLayout(content, BoxLayout.Y_AXIS));
		content.add(colorComboBox);
		content.add(Box.createVerticalStrut(10));
		content.add(colorChooser);
	}
	return content;
}

public void finalizeEditor() {

	if (content != null) {

		// Remove listeners and destroy components.

		colorChooser.removeActionListener(this);
		colorChooser = null;

		colorComboBox.removeActionListener(this);
		colorComboBox = null;

		content = null;
	}
}

protected JCElevation getElevation() {

	DataViewEditor		parent = (DataViewEditor)getParent();
	Chart3dDataView		dataView = parent.getSelectedDataView();

	if (dataView != null) {
		return dataView.getElevation();
	}
	return null;
}

public void refreshEditor() {

	if (content != null) {

		refreshing = true;

		int sel = colorComboBox.getSelectedIndex();

		if (sel != -1) {

			JCElevation elevation = getElevation();

			Color color = null;
			switch (sel) {
				case 0:
					color = elevation.getMeshTopColor();
					break;
				case 1:
					color = elevation.getMeshBottomColor();
					break;
				case 2:
					color = elevation.getShadedTopColor();
					break;
				case 3:
					color = elevation.getShadedBottomColor();
					break;
			}

			if (color != null) {
				colorChooser.setColor(color);
			}

		}
		refreshing = false;
	}
}

public void actionPerformed(ActionEvent e) {
	handleChange(e.getSource());
}

private void handleChange(Object source) {

	if (refreshing) {
		return;
	}

	if (content != null) {

		if (source == colorChooser) {

			int sel = colorComboBox.getSelectedIndex();

			if (sel != -1) {

				JCElevation elevation = getElevation();

				Color color = colorChooser.getColor();
				switch (sel) {
					case 0:
						elevation.setMeshTopColor(color);
						break;
					case 1:
						elevation.setMeshBottomColor(color);
						break;
					case 2:
						elevation.setShadedTopColor(color);
						break;
					case 3:
						elevation.setShadedBottomColor(color);
						break;
				}
			}
		}
		else if (source == colorComboBox) {
			refreshEditor();
		}
	}
}

}
