/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataViewGeneralEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:00 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.JCChart3d;
import com.klg.jclass.chart3d.Chart3dDataView;
import com.klg.jclass.chart3d.JCContour;
import com.klg.jclass.chart3d.JCElevation;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.BorderLayout;

import java.util.Arrays;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JComboBox;
import javax.swing.JCheckBox;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.Box;

public class DataViewGeneralEditor extends Chart3dPropertyEditor implements ActionListener {

private static String		nameKey = LocaleBundle.STRING_GENERAL;

private boolean				refreshing;
private Container			content;
private JComboBox			chartTypeComboBox;
private JComboBox			zoneMethodComboBox;
private JCheckBox			isContouredCheckBox;
private JCheckBox			isZonedCheckBox;
private JCheckBox			isMeshedCheckBox;
private JCheckBox			isShadedCheckBox;
private JCheckBox			isTransparentCheckBox;
private Object[]			chartTypeModel;
private Object[]			zoneMethodModel;

public DataViewGeneralEditor() {
	super(nameKey);
}

public Component getEditor() {

	if (content == null) {

		String local;

		local = getLocalizedString(LocaleBundle.STRING_CHART_TYPE);
		JLabel chartTypeLabel = new JLabel(local);

		chartTypeModel = new ComboBoxEntry[] {

			new ComboBoxEntry(JCChart3d.SCATTER, getLocalizedString(LocaleBundle.STRING_SCATTER)),
			new ComboBoxEntry(JCChart3d.SURFACE, getLocalizedString(LocaleBundle.STRING_SURFACE)),
			new ComboBoxEntry(JCChart3d.BAR, getLocalizedString(LocaleBundle.STRING_BAR)),
		};
		// All models will be sorted so that we can map a constant to an entry using the Arrays.find
		// method.
		Arrays.sort(chartTypeModel);

		chartTypeComboBox = createComboBox(chartTypeModel, this);

		local = getLocalizedString(LocaleBundle.STRING_ZONE_METHOD);
		JLabel zoneMethodLabel = new JLabel(local);

		zoneMethodModel = new ComboBoxEntry[] {

			new ComboBoxEntry(JCContour.ZONE_CONTOURS, getLocalizedString(LocaleBundle.STRING_ZONE_CONTOUR)),
			new ComboBoxEntry(JCContour.ZONE_CELLS_AVERAGE, getLocalizedString(LocaleBundle.STRING_ZONE_CELL_AVERAGE)),
			new ComboBoxEntry(JCContour.ZONE_CELLS_CORNER, getLocalizedString(LocaleBundle.STRING_ZONE_CELL_CORNER))
		};

		Arrays.sort(zoneMethodModel);

		zoneMethodComboBox = createComboBox(zoneMethodModel, this);

		local = getLocalizedString(LocaleBundle.STRING_CONTOURED);
		isContouredCheckBox = createCheckBox(local, this);

		local = getLocalizedString(LocaleBundle.STRING_ZONED);
		isZonedCheckBox = createCheckBox(local, this);

		local = getLocalizedString(LocaleBundle.STRING_MESHED);
		isMeshedCheckBox = createCheckBox(local, this);

		local = getLocalizedString(LocaleBundle.STRING_SHADED);
		isShadedCheckBox = createCheckBox(local, this);

		local = getLocalizedString(LocaleBundle.STRING_TRANSPARENT);
		isTransparentCheckBox = createCheckBox(local, this);

		JPanel labels = new JPanel(new GridLayout(7,0,5,5));
		labels.add(chartTypeLabel);
		labels.add(zoneMethodLabel);
		labels.add(isContouredCheckBox);
		labels.add(isZonedCheckBox);
		labels.add(isMeshedCheckBox);
		labels.add(isShadedCheckBox);
		labels.add(isTransparentCheckBox);

		JPanel comboBoxes = new JPanel(new GridLayout(7,0,5,5));
		comboBoxes.add(chartTypeComboBox);
		comboBoxes.add(zoneMethodComboBox);

		JPanel controls = createBorderPanel();
		controls.setBorder(null);
		controls.add(labels, BorderLayout.WEST);
		controls.add(comboBoxes);

		JPanel panel = createPreferredSizePanel();
		panel.add(controls);

		content = Box.createVerticalBox();
		content.add(panel);

	}
	return content;
}

public void finalizeEditor() {

	if (content != null) {

		chartTypeComboBox.removeActionListener(this);
		chartTypeComboBox = null;

		zoneMethodComboBox.removeActionListener(this);
		zoneMethodComboBox = null;

		isContouredCheckBox.removeActionListener(this);
		isContouredCheckBox = null;

		isZonedCheckBox.removeActionListener(this);
		isZonedCheckBox = null;

		isMeshedCheckBox.removeActionListener(this);
		isMeshedCheckBox = null;

		isShadedCheckBox.removeActionListener(this);
		isShadedCheckBox = null;

		isTransparentCheckBox.removeActionListener(this);
		isTransparentCheckBox = null;

		content = null;
	}
}

public void refreshEditor() {

	if (content != null) {

		refreshing = true;

		DataViewEditor parent = (DataViewEditor)getParent();
		Chart3dDataView dataView = parent.getSelectedDataView();

		MutableComboBoxEntry entry = new MutableComboBoxEntry();

		entry.setConstant(dataView.getChartType());
		chartTypeComboBox.setSelectedIndex(Arrays.binarySearch(chartTypeModel, entry));

		entry.setConstant(dataView.getContour().getZoneMethod());
		zoneMethodComboBox.setSelectedIndex(Arrays.binarySearch(zoneMethodModel, entry));

		isContouredCheckBox.setSelected(dataView.getContour().isContoured());
		isZonedCheckBox.setSelected(dataView.getContour().isZoned());
		isMeshedCheckBox.setSelected(dataView.getElevation().isMeshed());
		isShadedCheckBox.setSelected(dataView.getElevation().isShaded());
		isTransparentCheckBox.setSelected(dataView.getElevation().isTransparent());

		refreshing = false;
	}
}

public void actionPerformed(ActionEvent e) {
	handleChange(e.getSource());
}

private void handleChange(Object source) {

	if (refreshing) {
		return;
	}

	if (content != null) {

		DataViewEditor parent = (DataViewEditor)getParent();
		Chart3dDataView dataView = parent.getSelectedDataView();

		if (source == isMeshedCheckBox) {
			dataView.getElevation().setMeshed(isMeshedCheckBox.isSelected());
		}
		else if (source == isShadedCheckBox) {
			dataView.getElevation().setShaded(isShadedCheckBox.isSelected());
		}
		else if (source == isTransparentCheckBox) {
			dataView.getElevation().setTransparent(isTransparentCheckBox.isSelected());
		}
		else if (source == isContouredCheckBox) {
			dataView.getContour().setContoured(isContouredCheckBox.isSelected());
		}
		else if (source == isZonedCheckBox) {
			dataView.getContour().setZoned(isZonedCheckBox.isSelected());
		}
		else {

			// Source must be one of the combo boxes.
			JComboBox comboBoxSource = (JComboBox)source;

			int selection = ((ComboBoxEntry)comboBoxSource.getSelectedItem()).getConstant();

			if (source == chartTypeComboBox) {
				dataView.setChartType(selection);
			}
			else { // source == zoneMethodComboBox
				dataView.getContour().setZoneMethod(selection);
			}
		}
	}
}

}
