/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataViewGridColorsEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:00 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.customizer.util.JCColorChooser;
import com.klg.jclass.chart3d.*;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.BorderLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.Collections;
import java.util.ArrayList;

import javax.swing.JList;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JButton;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.JPanel;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.AbstractListModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.BorderFactory;

import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

public class DataViewGridColorsEditor extends Chart3dPropertyEditor
	implements ActionListener, ListDataListener {

private static final String		nameKey  = LocaleBundle.STRING_GRID_COLORS;
private static String			all = Chart3dPropertyEditor.getLocalizedString(LocaleBundle.STRING_ALL);

private boolean					refreshing = false;
private Container				content;
private JList					gridColorsList;
private JButton					addButton;
private JButton					removeButton;
private GridColorsModel			gridColorsModel;
private JCColorChooser			colorChooser;

public DataViewGridColorsEditor() {
	super(nameKey);
}

public Component getEditor() {

	if (content == null) {

		String local;

		local = getLocalizedString(LocaleBundle.STRING_XY_GRIDS);
		JLabel gridPointsLabel = new JLabel(local);

		local = getLocalizedString(LocaleBundle.STRING_ADD);
		addButton = createButton(local, this);

		local = getLocalizedString(LocaleBundle.STRING_REMOVE);
		removeButton = createButton(local, this);

		gridColorsModel = new GridColorsModel();
		gridColorsModel.addListDataListener(this);

		gridColorsList = new JList(gridColorsModel);
 		gridColorsList.setPrototypeCellValue("XXXXXXXXXXXX");

		JScrollPane scrollPane = new JScrollPane(
				gridColorsList,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED
				);
		scrollPane.setAlignmentX(0);

		local = getLocalizedString(LocaleBundle.STRING_GRID_COLORS);

		colorChooser = getColorChooser();
		colorChooser.addActionListener(this);
		colorChooser.setBorder(BorderFactory.createTitledBorder(local));

		JPanel buttonPanel = new JPanel(new GridLayout(0,2,5,5));
		buttonPanel.add(addButton);
		buttonPanel.add(removeButton);
		buttonPanel.setAlignmentX(0);

		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		panel.add(gridPointsLabel);
		panel.add(scrollPane);
		panel.add(Box.createVerticalStrut(5));
		panel.add(buttonPanel);

		content = createPreferredSizePanel();
		content.setLayout(new BoxLayout(content, BoxLayout.X_AXIS));
		content.add(panel);
		content.add(Box.createHorizontalStrut(5));
		content.add(colorChooser);
	}
	return content;
}

public void finalizeEditor() {

	if (content != null) {

		gridColorsModel.removeListDataListener(this);
		gridColorsModel = null;

		gridColorsList = null;

		addButton.removeActionListener(this);
		addButton = null;

		removeButton.removeActionListener(this);
		removeButton = null;

		colorChooser.removeActionListener(this);
		colorChooser = null;

		content = null;
	}
}

public void refreshEditor() {

	if (content != null) {

		refreshing = true;

		DataViewEditor		parent = (DataViewEditor)getParent();
		Chart3dDataView		dataView = parent.getSelectedDataView();
		boolean				enabled = dataView.getElevationData() instanceof Chart3dGridData;

		// Disable all components if the data model is not a grid data model.
		gridColorsList.setEnabled(enabled);
		addButton.setEnabled(enabled);
		removeButton.setEnabled(enabled);
		colorChooser.setEnabled(enabled);

		if (!enabled) {
			return;
		}

		Chart3dGridData gridData = (Chart3dGridData)dataView.getElevationData();

		gridColorsModel.setGridColors(dataView.getGridColors());

//						int ys = gridData.getYGrid().length;
//
//						if ( (xValue >= 0 && xValue < xs) || xValue == JCData3dIndex.ALL) {
//							if ( (yValue >= 0 && yValue < ys) || yValue == JCData3dIndex.ALL) {
//								target.getDataView(0).addGridColor(xValue,yValue,Color.blue);
//							}
//						}
//						else {
//							throw new IllegalArgumentException(lm.getString(LocaleBundle.ERROR_INVALID_X_Y));
//						}

		refreshing = false;
	}
}

public void contentsChanged(ListDataEvent e) {
	handleGridColorChange();
}

public void intervalAdded(ListDataEvent e) {
	handleGridColorChange();
}

public void intervalRemoved(ListDataEvent e) {
	handleGridColorChange();
}

public void actionPerformed(ActionEvent e) {

	if (refreshing) {
		return;
	}

	if (content != null) {

		DataViewEditor		parent = (DataViewEditor)getParent();
		Chart3dDataView		dataView = parent.getSelectedDataView();

		Object source = e.getSource();

		if (source == addButton) {

			JCGridColor newGridColor = showGridValueDialog(addButton, LocaleBundle.STRING_ADD, null);

			if (newGridColor != null) {
				gridColorsModel.add(newGridColor);
			}
		}
		else if (source == removeButton) {
			if (gridColorsList.getSelectedIndex() != -1) {
				gridColorsModel.remove(gridColorsList.getSelectedIndex());
			}
		}
		else if (source == colorChooser) {
			int sel = gridColorsList.getSelectedIndex();
			if (sel != -1) {
				Color color = colorChooser.getColor();
				gridColorsModel.getGridColorAt(sel).setColor(color);
				handleGridColorChange();
			}
		}
	}
}

private void handleGridColorChange() {

	if (refreshing) {
		return;
	}

	DataViewEditor		parent = (DataViewEditor)getParent();
	Chart3dDataView		dataView = parent.getSelectedDataView();

	dataView.setGridColors(gridColorsModel.getGridColors());
}

private JCGridColor showGridValueDialog(Component owner, String titleKey, JCGridColor source) {

	String local;

	local = getLocalizedString(LocaleBundle.STRING_X);
	JLabel xLabel = new JLabel(local);

	local = getLocalizedString(LocaleBundle.STRING_Y);
	JLabel yLabel = new JLabel(local);

	JPanel labelsPanel = new JPanel(new GridLayout(2,0,5,5));
	labelsPanel.add(xLabel);
	labelsPanel.add(yLabel);

	JTextField xField = new JTextField(8);
	JTextField yField = new JTextField(8);

	if (source != null) {
		xField.setText(String.valueOf(source.getX()));
		yField.setText(String.valueOf(source.getY()));
	}

	JPanel fieldsPanel = new JPanel(new GridLayout(2,0,5,5));
	fieldsPanel.add(xField);
	fieldsPanel.add(yField);

	JPanel content = createPreferredSizePanel();
	content.add(labelsPanel, BorderLayout.WEST);
	content.add(fieldsPanel);

	int result = JOptionPane.showOptionDialog(
			owner,
			content,
			getLocalizedString(titleKey),
			JOptionPane.OK_CANCEL_OPTION,
			JOptionPane.PLAIN_MESSAGE,
			null,
			null,
			null);

	if (result == JOptionPane.OK_OPTION) {



		Integer	x = null,
				y = null;

		// Do x.

		if (xField.getText().equalsIgnoreCase(all)) {
			x = new Integer(JCData3dIndex.ALL);
		}
		else {
			x = parseIntegerInput(xField);
		}

		if (x == null) {
			return null;
		}

		// Now do y.

		if (yField.getText().equalsIgnoreCase(all)) {
			y = new Integer(JCData3dIndex.ALL);
		}
		else {
			y = parseIntegerInput(yField);
		}

		if (y == null) {
			return null;
		}

		// We have an x and a y grid value.
		if (y != null) {
			if (source != null) {
				source.setX(x.intValue());
				source.setY(y.intValue());
				return null;
			}
			else {
				return new JCGridColor(x.intValue(), y.intValue(), colorChooser.getColor());
			}
		}
	}
	return null;
}

	private static class GridColorsModel extends AbstractListModel {

	private ArrayList	gridColors;

	public GridColorsModel() {
		gridColors = new ArrayList();
	}

	public void setGridColors(ArrayList gridColors) {

		if (getSize() > 0) {
			fireIntervalRemoved(this, 0, getSize() - 1);
		}

		if (gridColors == null) {
			return;
		}

		this.gridColors = gridColors;

		if (getSize() > 0) {
			fireIntervalAdded(this, 0, getSize() - 1);
		}
	}

	public ArrayList getGridColors() {
		return gridColors;
	}

	public Object getElementAt(int index) {
		return toString((JCGridColor)gridColors.get(index));
	}

	/**
	 * Use this method to get a value label from this model.
	 */
	public JCGridColor getGridColorAt(int index) {
		return (JCGridColor)gridColors.get(index);
	}

	public int getSize() {
		if (gridColors == null) {
			return 0;
		}
		return gridColors.size();
	}

	public void remove(int index) {
		gridColors.remove(index);
		fireIntervalRemoved(this, index, index);
	}

	public void add(JCGridColor gridColor) {
		if (gridColor != null) {
			gridColors.add(gridColor);
			fireIntervalAdded(this, getSize(), getSize());
		}
	}

	private String toString(JCGridColor gridColor) {

		if (gridColor == null) {
			return "";
		}

		int x = gridColor.getX(),
			y = gridColor.getY();

		String xStr = (x == JCData3dIndex.ALL)
			? all
			: String.valueOf(x);

		String yStr = (y == JCData3dIndex.ALL)
			? all
			: String.valueOf(y);

		return xStr + "," + yStr;
	}

	}
}
