/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: FillStyleEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:00 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.customizer.util.JCColorChooser;

import com.klg.jclass.chart3d.JCChart3dStyle;
import com.klg.jclass.chart3d.JCFillStyle;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Image;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.BorderLayout;
import java.awt.FlowLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.File;

import java.util.Arrays;

import javax.swing.JComboBox;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.BorderFactory;
import javax.swing.SwingConstants;
import javax.swing.ImageIcon;

import javax.swing.filechooser.FileFilter;

public abstract class FillStyleEditor extends Chart3dPropertyEditor implements ActionListener {

private static String		nameKey = LocaleBundle.STRING_FILL_STYLE;

private boolean				refreshing = false;
private Container			content;
private JCColorChooser		fgColorChooser;
private JCColorChooser		bgColorChooser;
private JLabel				thumbnailLabel;
private JButton				imageButton;
private JComboBox			patternComboBox;
private JCheckBox			opaqueCheckBox;
private Object[]			patternModel;

public FillStyleEditor() {
	super(nameKey);
}

public Component getEditor() {

	if (content == null) {

		String local;

		local = getLocalizedString(LocaleBundle.STRING_FILL);

		fgColorChooser = getColorChooser();
		fgColorChooser.addActionListener(this);
		fgColorChooser.setAlignmentX(0);
		fgColorChooser.setBorder(BorderFactory.createTitledBorder(local));

		bgColorChooser = getColorChooser();
		bgColorChooser.addActionListener(this);

		local = getLocalizedString(LocaleBundle.STRING_OPAQUE);
		opaqueCheckBox = createCheckBox(local, this);

		local = getLocalizedString(LocaleBundle.STRING_BACKGROUND);

		JPanel bgPanel = createBorderPanel();
		bgPanel.setBorder(BorderFactory.createTitledBorder(local));
		bgPanel.setAlignmentX(0);
		bgPanel.add(bgColorChooser);
		bgPanel.add(opaqueCheckBox, BorderLayout.SOUTH);

		local = getLocalizedString(LocaleBundle.STRING_PATTERN);

		JLabel patternLabel = new JLabel(local);

		patternModel = new ComboBoxEntry[] {

			new ComboBoxEntry(JCFillStyle.NONE, getLocalizedString(LocaleBundle.STRING_NONE)),
			new ComboBoxEntry(JCFillStyle.SOLID, getLocalizedString(LocaleBundle.STRING_SOLID)),
			new ComboBoxEntry(JCFillStyle.PER_25, getLocalizedString(LocaleBundle.STRING_PER_25)),
			new ComboBoxEntry(JCFillStyle.PER_50, getLocalizedString(LocaleBundle.STRING_PER_50)),
			new ComboBoxEntry(JCFillStyle.PER_75, getLocalizedString(LocaleBundle.STRING_PER_75)),
			new ComboBoxEntry(JCFillStyle.HORIZ_STRIPE, getLocalizedString(LocaleBundle.STRING_HORIZ_STRIPE)),
			new ComboBoxEntry(JCFillStyle.VERT_STRIPE, getLocalizedString(LocaleBundle.STRING_VERT_STRIPE)),
			new ComboBoxEntry(JCFillStyle.STRIPE_45, getLocalizedString(LocaleBundle.STRING_STRIPE_45)),
			new ComboBoxEntry(JCFillStyle.STRIPE_135, getLocalizedString(LocaleBundle.STRING_STRIPE_135)),
			new ComboBoxEntry(JCFillStyle.DIAG_HATCHED, getLocalizedString(LocaleBundle.STRING_DIAG_HATCHED)),
			new ComboBoxEntry(JCFillStyle.CROSS_HATCHED, getLocalizedString(LocaleBundle.STRING_CROSS_HATCHED)),
			new ComboBoxEntry(JCFillStyle.CUSTOM_FILL, getLocalizedString(LocaleBundle.STRING_CUSTOM_FILL)),
			new ComboBoxEntry(JCFillStyle.CUSTOM_STACK, getLocalizedString(LocaleBundle.STRING_CUSTOM_STACK)),
			new ComboBoxEntry(JCFillStyle.CUSTOM_PAINT, getLocalizedString(LocaleBundle.STRING_CUSTOM_PAINT))
		};

		// All models will be sorted so that we can map a constant to an entry using the Arrays.find
		// method.
		Arrays.sort(patternModel);

		patternComboBox = createComboBox(patternModel, this);

		JPanel patternPanel = createBorderPanel();
		patternPanel.setBorder(null);
		patternPanel.add(patternLabel, BorderLayout.WEST);
		patternPanel.add(patternComboBox);
		patternPanel.setAlignmentX(0);

		local = getLocalizedString(LocaleBundle.STRING_IMAGE);

		thumbnailLabel = new JLabel();
		thumbnailLabel.setBorder(BorderFactory.createEtchedBorder());
		thumbnailLabel.setMaximumSize(new Dimension(50,50));  // fixed size thumbnail!
		thumbnailLabel.setMinimumSize(new Dimension(50,50));
		thumbnailLabel.setPreferredSize(new Dimension(50,50));

		local = getLocalizedString(LocaleBundle.STRING_OPEN);

		imageButton = createButton(local, this);

		local = getLocalizedString(LocaleBundle.STRING_IMAGE);

		JLabel imageLabel = new JLabel(local);
		imageLabel.setHorizontalAlignment(SwingConstants.CENTER);
		imageLabel.setVerticalAlignment(SwingConstants.CENTER);

		JPanel imagePanel = new JPanel(new FlowLayout(FlowLayout.LEFT,5,5));
		imagePanel.setAlignmentX(0);
		imagePanel.add(imageLabel);
		imagePanel.add(thumbnailLabel);
		imagePanel.add(imageButton);

		content = createBorderPanel();
		content.setLayout(new BoxLayout(content, BoxLayout.Y_AXIS));
		content.add(fgColorChooser);
		content.add(Box.createVerticalStrut(10));
		content.add(bgPanel);
		content.add(Box.createVerticalStrut(10));
		content.add(patternPanel);
		content.add(Box.createVerticalStrut(10));
		content.add(imagePanel);
	}
	return content;
}

public void finalizeEditor() {

	if (content != null) {

		// Remove listeners and destroy components.

		fgColorChooser.removeActionListener(this);
		fgColorChooser = null;

		bgColorChooser.removeActionListener(this);
		bgColorChooser = null;

		opaqueCheckBox.removeActionListener(this);
		opaqueCheckBox = null;

		patternComboBox.removeActionListener(this);
		patternComboBox = null;

		imageButton.removeActionListener(this);
		imageButton = null;

		content = null;
	}
}

public void refreshEditor() {

	if (content != null) {

		refreshing = true;

		JCFillStyle fillStyle = getSelectedFillStyle();

		if (fillStyle != null) {

			fgColorChooser.setColor(fillStyle.getColor());

			Color bg = fillStyle.getBackground();

			boolean isOpaque = (bg != null);

			bgColorChooser.setEnabled(isOpaque);

			bgColorChooser.setColor(bg);

			opaqueCheckBox.setSelected(isOpaque);

			MutableComboBoxEntry entry = new MutableComboBoxEntry();

			entry.setConstant(fillStyle.getPattern());
			patternComboBox.setSelectedIndex(Arrays.binarySearch(patternModel, entry));

			Image image = fillStyle.getImage();
			if (image == null) {
				thumbnailLabel.setIcon(null);
			}
			else {
				thumbnailLabel.setIcon(new ImageIcon(image));
			}
		}

		refreshing = false;
	}
}

public void actionPerformed(ActionEvent e) {
	handleChange(e.getSource());
}

private void handleChange(Object source) {

	if (refreshing) {
		return;
	}

	if (content != null) {

		JCFillStyle fillStyle = getSelectedFillStyle();
		if (fillStyle != null) {

			if (source == fgColorChooser) {
				fillStyle.setColor(fgColorChooser.getColor());
			}
			else if (source == bgColorChooser) {
				fillStyle.setBackground(bgColorChooser.getColor());
			}
			else if (source == opaqueCheckBox) {

				// If opaque is selected, the background color chooser should be activated.
				boolean isOpaque = opaqueCheckBox.isSelected();

				bgColorChooser.setEnabled(isOpaque);

				// If opaque is not selected, then the background color will not be used.
				// We need to set the background to null in this case (it's what JCFillStyle
				// expects.

				if (!isOpaque) {
					fillStyle.setBackground(null);
				}
				else {
					fillStyle.setBackground(bgColorChooser.getColor());
				}
			}
			else if (source == patternComboBox) {
				int sel = ((ComboBoxEntry)patternComboBox.getSelectedItem()).getConstant();
				fillStyle.setPattern(sel);
			}
			else if (source == imageButton) {
				Image image = doImageDialog(imageButton);
				if (image != null) {
					fillStyle.setImage(image);
					thumbnailLabel.setIcon(new ImageIcon(image));
				}
			}
		}
	}
}


protected Image doImageDialog(Component source) {

	JFileChooser fileChooser = new JFileChooser();
	fileChooser.setFileFilter(new FileFilter() {

		public boolean accept(File file) {

			if (file.isDirectory()) {
				return true;
			}

			String filename = file.getName();
			if (filename.endsWith(".gif") || filename.endsWith(".jpg")) {
				return true;
			}

			// Not a valid file.
			return false;
		}

		public String getDescription() {
			return "JPEG and GIF Image Files (*.jpg, *.gif)";
		}
	});

	int option = fileChooser.showOpenDialog(source);

	Image image = null;

	if (option == JFileChooser.APPROVE_OPTION) {
		String fullFilename = fileChooser.getSelectedFile().getAbsolutePath();
		// Try to load the image.
		ImageIcon icon = new ImageIcon(fullFilename);
		if (icon != null) {
			image = icon.getImage();
		}
		else {
			// Couldn't load the image.  Alert the user.
			doError(source, LocaleBundle.ERROR_IMAGE_LOAD, LocaleBundle.STRING_ERROR);
		}
	}
	return image;
}

protected abstract JCFillStyle getSelectedFillStyle();

}
