/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: LegendGeneralEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:00 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.util.legend.JCLegend;
import com.klg.jclass.chart3d.JCChart3dLegend;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.BorderLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.Arrays;

import javax.swing.JComboBox;
import javax.swing.JCheckBox;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.Box;

public class LegendGeneralEditor extends Chart3dPropertyEditor implements ActionListener {

private static String		nameKey = LocaleBundle.STRING_GENERAL;

private boolean				refreshing = false;
private Container			content;
private JCheckBox			isVisibleCheckBox;
private JComboBox			anchorComboBox;
private JComboBox			orientationComboBox;
private JComboBox			layoutStyleComboBox;
private JComboBox			distRangeComboBox;
private Object[]			anchorModel;
private Object[]			orientationModel;
private Object[]			layoutStyleModel;
private Object[]			distRangeModel;

public LegendGeneralEditor() {
	super(nameKey);
}

public Component getEditor() {

	if (content == null) {

		String local;

		local = getLocalizedString(LocaleBundle.STRING_VISIBLE);
		isVisibleCheckBox = createCheckBox(local, this);

		local = getLocalizedString(LocaleBundle.STRING_ANCHOR);
		JLabel anchorLabel = new JLabel(local);

		anchorModel = new ComboBoxEntry[] {

			new ComboBoxEntry(JCLegend.NORTH, getLocalizedString(LocaleBundle.STRING_NORTH)),
			new ComboBoxEntry(JCLegend.SOUTH, getLocalizedString(LocaleBundle.STRING_SOUTH)),
			new ComboBoxEntry(JCLegend.EAST, getLocalizedString(LocaleBundle.STRING_EAST)),
			new ComboBoxEntry(JCLegend.WEST, getLocalizedString(LocaleBundle.STRING_WEST)),
			new ComboBoxEntry(JCLegend.NORTHEAST, getLocalizedString(LocaleBundle.STRING_NORTHEAST)),
			new ComboBoxEntry(JCLegend.NORTHWEST, getLocalizedString(LocaleBundle.STRING_NORTHWEST)),
			new ComboBoxEntry(JCLegend.SOUTHEAST, getLocalizedString(LocaleBundle.STRING_SOUTHEAST)),
			new ComboBoxEntry(JCLegend.SOUTHWEST, getLocalizedString(LocaleBundle.STRING_SOUTHWEST))
		};

		// All models will be sorted so that we can map a constant to an entry using the Arrays.find
		// method.
		Arrays.sort(anchorModel);

		anchorComboBox = createComboBox(anchorModel, this);

		local = getLocalizedString(LocaleBundle.STRING_ORIENTATION);
		JLabel orientationLabel = new JLabel(local);

		orientationModel = new ComboBoxEntry[] {

			new ComboBoxEntry(JCLegend.HORIZONTAL, getLocalizedString(LocaleBundle.STRING_HORIZONTAL)),
			new ComboBoxEntry(JCLegend.VERTICAL, getLocalizedString(LocaleBundle.STRING_VERTICAL))
		};

		Arrays.sort(orientationModel);

		orientationComboBox = createComboBox(orientationModel, this);

		local = getLocalizedString(LocaleBundle.STRING_LAYOUT_STYLE);
		JLabel layoutStyleLabel = new JLabel(local);

		layoutStyleModel = new ComboBoxEntry[] {

			new ComboBoxEntry(JCChart3dLegend.CONTINUOUS, getLocalizedString(LocaleBundle.STRING_CONTINUOUS)),
			new ComboBoxEntry(JCChart3dLegend.STEPPED, getLocalizedString(LocaleBundle.STRING_STEPPED))
		};

		Arrays.sort(layoutStyleModel);

		layoutStyleComboBox = createComboBox(layoutStyleModel, this);

		local = getLocalizedString(LocaleBundle.STRING_DIST_RANGE);
		JLabel distRangeLabel = new JLabel(local);

		distRangeModel = new ComboBoxEntry[] {

			new ComboBoxEntry(JCChart3dLegend.RANGE_DATA, getLocalizedString(LocaleBundle.STRING_DATA)),
			new ComboBoxEntry(JCChart3dLegend.RANGE_ALL, getLocalizedString(LocaleBundle.STRING_ALL))
		};

		Arrays.sort(distRangeModel);

		distRangeComboBox = createComboBox(distRangeModel, this);



		JPanel labels = new JPanel(new GridLayout(5,0,5,5));
		labels.add(isVisibleCheckBox);
		labels.add(anchorLabel);
		labels.add(orientationLabel);
		labels.add(layoutStyleLabel);
		labels.add(distRangeLabel);

		JPanel fields = new JPanel(new GridLayout(5,0,5,5));
		fields.add(new JPanel());
		fields.add(anchorComboBox);
		fields.add(orientationComboBox);
		fields.add(layoutStyleComboBox);
		fields.add(distRangeComboBox);

		JPanel controls = createBorderPanel();
		controls.setBorder(null);
		controls.add(labels, BorderLayout.WEST);
		controls.add(fields);

		JPanel panel = createPreferredSizePanel();
		panel.add(controls);

		content = Box.createVerticalBox();
		content.add(panel);
	}
	return content;
}

public void finalizeEditor() {

	if (content != null) {

		// Remove listeners and destroy components.
		isVisibleCheckBox.removeActionListener(this);
		isVisibleCheckBox = null;

		anchorComboBox.removeActionListener(this);
		anchorComboBox = null;

		orientationComboBox.removeActionListener(this);
		orientationComboBox = null;

		layoutStyleComboBox.removeActionListener(this);
		layoutStyleComboBox = null;

		distRangeComboBox.removeActionListener(this);
		distRangeComboBox = null;

		content = null;
	}
}

public void refreshEditor() {

	if (content != null) {

		refreshing = true;

		JCLegend legend = getChart().getLegend();

		isVisibleCheckBox.setSelected(legend.isVisible());

		MutableComboBoxEntry entry = new MutableComboBoxEntry();

		entry.setConstant(legend.getAnchor());
		anchorComboBox.setSelectedIndex(Arrays.binarySearch(anchorModel, entry));

		entry.setConstant(legend.getOrientation());
		orientationComboBox.setSelectedIndex(Arrays.binarySearch(orientationModel, entry));

		if (!(legend instanceof JCChart3dLegend)) {
			// Disable the layout style and distribution range combo boxes -
			// then return.
			layoutStyleComboBox.setEnabled(false);
			distRangeComboBox.setEnabled(false);
			return;
		}

		layoutStyleComboBox.setEnabled(true);
		distRangeComboBox.setEnabled(true);

		entry.setConstant(((JCChart3dLegend)legend).getLayoutStyle());
		layoutStyleComboBox.setSelectedIndex(Arrays.binarySearch(layoutStyleModel, entry));

		entry.setConstant(((JCChart3dLegend)legend).getDistributionRange());
		distRangeComboBox.setSelectedIndex(Arrays.binarySearch(distRangeModel, entry));

		refreshing = false;
	}
}

public void actionPerformed(ActionEvent e) {
	handleChange(e.getSource());
}

private void handleChange(Object source) {

	if (refreshing) {
		return;
	}

	if (content != null) {

		JCLegend legend = getChart().getLegend();

		if (source == isVisibleCheckBox) {
			legend.setVisible(isVisibleCheckBox.isSelected());
			return;
		}

		// All other sources are combo boxes.

		JComboBox comboBoxSource = (JComboBox)source;

		int selection = ((ComboBoxEntry)comboBoxSource.getSelectedItem()).getConstant();

		if (source == anchorComboBox) {
			legend.setAnchor(selection);
		}
		else if (source == orientationComboBox) {
			legend.setOrientation(selection);
		}

		if (!(legend instanceof JCChart3dLegend)) {
			return;
		}

		if (source == layoutStyleComboBox) {
			((JCChart3dLegend)legend).setLayoutStyle(selection);
		}
		else if (source == distRangeComboBox) {
			((JCChart3dLegend)legend).setDistributionRange(selection);
		}
	}
}

public static class ComboBoxEntry implements Comparable {

	private int			constant;
	private String		name;


	public ComboBoxEntry(int constant, String name) {
		this.name = name;
		this.constant = constant;
	}

	public int compareTo(Object o) {

		int constantToCompare = ((ComboBoxEntry)o).getConstant();

		if (getConstant() < constantToCompare) {
			return -1;
		}

		if (getConstant() > constantToCompare) {
			return 1;
		}

		return 0;
	}

	public boolean equals(Object o) {

		if (o instanceof ComboBoxEntry) {
			return ((ComboBoxEntry)o).getConstant() == getConstant();
		}

		return false;
	}

	public int getConstant() {
		return constant;
	}

	public String getName() {
		return name;
	}

	public String toString() {
		return getName();
	}
}

public static class MutableComboBoxEntry extends ComboBoxEntry {

	private int		constant;

	public MutableComboBoxEntry() {
		super(0, "");
	}

	public void setConstant(int constant) {
		this.constant = constant;
	}

	public int getConstant() {
		return constant;
	}
}

}
