/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: LegendSpacingEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:00 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.JCChart3dLegend;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.Box;

public class LegendSpacingEditor extends Chart3dPropertyEditor implements ActionListener, FocusListener {

private static final String		nameKey = LocaleBundle.STRING_SPACING;

private boolean					refreshing = false;
private Container				content;
private JTextField[]			intFields;

public LegendSpacingEditor() {
	super(nameKey);
}

public Component getEditor() {

	if (content == null) {

		String local;

		String[] names = {
			LocaleBundle.STRING_MARGIN_GAP,
			LocaleBundle.STRING_GROUP_GAP,
			LocaleBundle.STRING_HORIZONTAL_ITEM_GAP,
			LocaleBundle.STRING_VERTICAL_ITEM_GAP,
			LocaleBundle.STRING_SYMBOL_TO_TEXT_GAP
		};

		JLabel[] labels = new JLabel[5];

		intFields = new JTextField[5];

		JPanel labelPanel = new JPanel(new GridLayout(5,0,5,5));
		JPanel intFieldPanel = new JPanel(new GridLayout(5,0,5,5));

		for (int each = 0; each < 5; each++) {

			labels[each] = new JLabel(getLocalizedString(names[each]));
			intFields[each] = createTextField(8, this, this);

			labelPanel.add(labels[each]);
			intFieldPanel.add(intFields[each]);
		}

		JPanel panel = createPreferredSizePanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		panel.add(labelPanel);
		panel.add(intFieldPanel);

		content = Box.createVerticalBox();
		content.add(panel);
	}
	return content;
}

public void finalizeEditor() {

	if (content != null) {

		for (int each = 0; each < 5; each++) {

			intFields[each].removeActionListener(this);
			intFields[each].removeFocusListener(this);
			intFields[each] = null;
		}
		intFields = null;

		content = null;
	}
}

public void refreshEditor() {

	if (content != null) {

		refreshing = true;

		if (!(getChart().getLegend() instanceof JCChart3dLegend)) {
			// The legend is not a 3d legend!  We cannot modify any
			// of the gap items, so we just disable the editor.
			content.setEnabled(false);
		}

		content.setEnabled(true);

		JCChart3dLegend legend  = (JCChart3dLegend)getChart().getLegend();

		int[] values = {
			legend.getMarginGap(),
			legend.getGroupGap(),
			legend.getHorizontalItemGap(),
			legend.getVerticalItemGap(),
			legend.getInsideItemGap()
		};

		for (int each = 0 ; each < 5; each++) {
			intFields[each].setText(String.valueOf(values[each]));
		}

		refreshing = false;
	}
}

public void actionPerformed(ActionEvent e) {

	handleChange(e.getSource());
}

public void focusGained(FocusEvent e) {
	handleChange(e.getSource());
}

public void focusLost(FocusEvent e) {
	if (!e.isTemporary()) {
		handleChange(e.getSource());
	}
}

private void handleChange(Object source) {

	if (refreshing) {
		return;
	}

	if (content != null) {

		Integer i = parseIntegerInput((JTextField)source);

		if (i == null || i.intValue() < 0) {
			// All gaps must be >= 0.
			refreshEditor();
			return;
		}

		int					value = i.intValue();
		JCChart3dLegend		legend  = (JCChart3dLegend)getChart().getLegend();

		if (source == intFields[0]) {
			legend.setMarginGap(value);
		}
		else if (source == intFields[1]) {
			legend.setGroupGap(value);
		}
		else if (source == intFields[2]) {
			legend.setHorizontalItemGap(value);
		}
		else if (source == intFields[3]) {
			legend.setVerticalItemGap(value);
		}
		else if (source == intFields[4]) {
			legend.setInsideItemGap(value);
		}
	}
}

}
