/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: LineStyleEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:00 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.customizer.util.JCColorChooser;

import com.klg.jclass.chart3d.JCChart3dStyle;
import com.klg.jclass.chart3d.JCLineStyle;

import java.awt.BasicStroke;
import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.BorderLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import java.util.Arrays;

import javax.swing.JComboBox;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.BorderFactory;

public abstract class LineStyleEditor extends Chart3dPropertyEditor
									  implements ActionListener,
												 FocusListener
{

private static String		nameKey = LocaleBundle.STRING_LINE_STYLE;

private boolean				refreshing = false;
private Container			content;
private JCColorChooser		colorChooser;
private JTextField			widthField;
private JComboBox			styleComboBox;
private JComboBox			joinComboBox;
private JComboBox			capComboBox;
private Object[]			styleModel;
private Object[]			joinModel;
private Object[]			capModel;

public LineStyleEditor()
{
	super(nameKey);
}

public Component getEditor()
{
	if (content == null) {
		String local;

		local = getLocalizedString(LocaleBundle.STRING_COLOR);
		colorChooser = getColorChooser();
		colorChooser.addActionListener(this);
		colorChooser.setAlignmentX(0);
		colorChooser.setBorder(BorderFactory.createTitledBorder(local));

		local = getLocalizedString(LocaleBundle.STRING_WIDTH);
		JLabel widthLabel = new JLabel(local);

		widthField = createTextField(8, this, this);

		local = getLocalizedString(LocaleBundle.STRING_STYLE);
		JLabel styleLabel = new JLabel(local);

		styleModel = new ComboBoxEntry[] {
			new ComboBoxEntry(JCLineStyle.NONE,
			                  getLocalizedString(LocaleBundle.STRING_NONE)),
			new ComboBoxEntry(JCLineStyle.SOLID,
			                  getLocalizedString(LocaleBundle.STRING_SOLID)),
			new ComboBoxEntry(JCLineStyle.LONG_DASH,
			                  getLocalizedString(LocaleBundle.STRING_LONG_DASH)),
			new ComboBoxEntry(JCLineStyle.SHORT_DASH,
			                  getLocalizedString(LocaleBundle.STRING_SHORT_DASH)),
			new ComboBoxEntry(JCLineStyle.LSL_DASH,
			                  getLocalizedString(LocaleBundle.STRING_LSL_DASH)),
			new ComboBoxEntry(JCLineStyle.DASH_DOT,
			                  getLocalizedString(LocaleBundle.STRING_DASH_DOT))
		};

		// All models will be sorted so that we can map a constant to an entry
		// using the Arrays.find method.
		Arrays.sort(styleModel);

		styleComboBox = createComboBox(styleModel, this);

		local = getLocalizedString(LocaleBundle.STRING_JOIN);
		JLabel joinLabel = new JLabel(local);

		joinModel = new ComboBoxEntry[] {
			new ComboBoxEntry(BasicStroke.JOIN_BEVEL,
			                 getLocalizedString(LocaleBundle.STRING_BEVEL)),
			new ComboBoxEntry(BasicStroke.JOIN_MITER,
			                  getLocalizedString(LocaleBundle.STRING_MITER)),
			new ComboBoxEntry(BasicStroke.JOIN_ROUND,
			                  getLocalizedString(LocaleBundle.STRING_ROUND))
		};

		Arrays.sort(joinModel);

		joinComboBox = createComboBox(joinModel, this);

		local = getLocalizedString(LocaleBundle.STRING_LAYOUT_STYLE);
		JLabel capLabel = new JLabel(local);

		capModel = new ComboBoxEntry[] {
			new ComboBoxEntry(BasicStroke.CAP_BUTT,
			                  getLocalizedString(LocaleBundle.STRING_BUTT)),
			new ComboBoxEntry(BasicStroke.CAP_ROUND,
			                  getLocalizedString(LocaleBundle.STRING_ROUND)),
			new ComboBoxEntry(BasicStroke.CAP_SQUARE,
			                  getLocalizedString(LocaleBundle.STRING_SQUARE))
		};

		Arrays.sort(capModel);

		capComboBox = createComboBox(capModel, this);

		JPanel labels = new JPanel(new GridLayout(4,0,5,5));
		labels.add(widthLabel);
		labels.add(styleLabel);
		labels.add(joinLabel);
		labels.add(capLabel);

		JPanel fields = new JPanel(new GridLayout(4,0,5,5));
		fields.add(widthField);
		fields.add(styleComboBox);
		fields.add(joinComboBox);
		fields.add(capComboBox);

		JPanel controls = createBorderPanel();
		controls.setAlignmentX(0);
		controls.setBorder(null);
		controls.add(labels, BorderLayout.WEST);
		controls.add(fields);

		content = createBorderPanel();
		content.setLayout(new BoxLayout(content, BoxLayout.Y_AXIS));
		content.add(colorChooser);
		content.add(Box.createVerticalStrut(10));
		content.add(controls);
	}
	return content;
}

public void finalizeEditor()
{
	if (content != null) {
		// Remove listeners and destroy components.

		colorChooser.removeActionListener(this);
		colorChooser = null;

		widthField.removeActionListener(this);
		widthField.removeFocusListener(this);
		widthField = null;

		styleComboBox.removeActionListener(this);
		styleComboBox = null;

		joinComboBox.removeActionListener(this);
		joinComboBox = null;

		capComboBox.removeActionListener(this);
		capComboBox = null;

		content = null;
	}
}

public void refreshEditor()
{
	if (content != null) {

		refreshing = true;

		JCLineStyle lineStyle = getSelectedLineStyle();

		if (lineStyle != null) {

			colorChooser.setColor(lineStyle.getColor());

			widthField.setText(String.valueOf(lineStyle.getWidth()));

			MutableComboBoxEntry entry = new MutableComboBoxEntry();

			entry.setConstant(lineStyle.getPattern());
			styleComboBox.setSelectedIndex(Arrays.binarySearch(styleModel, entry));

			entry.setConstant(lineStyle.getJoin());
			joinComboBox.setSelectedIndex(Arrays.binarySearch(joinModel, entry));

			entry.setConstant(lineStyle.getJoin());
			joinComboBox.setSelectedIndex(Arrays.binarySearch(capModel, entry));
		}

		refreshing = false;
	}
}

public void actionPerformed(ActionEvent e)
{
	handleChange(e.getSource());
}

public void focusGained(FocusEvent e)
{
	handleChange(e.getSource());
}

public void focusLost(FocusEvent e)
{
	if (!e.isTemporary()) {
		handleChange(e.getSource());
	}
}

private void handleChange(Object source)
{
	if (refreshing) {
		return;
	}

	if (content != null) {
		JCLineStyle lineStyle = getSelectedLineStyle();
		if (lineStyle != null) {
			if (source == colorChooser) {
				lineStyle.setColor(colorChooser.getColor());
			}
			else if (source == widthField) {
				Integer i = parseIntegerInput(widthField);

				if (i == null || i.intValue() < 0) {
					// The parse was unsuccessful or width is < 0!
					// Refresh the editor and return.
					refreshEditor();
					return;
				}
				lineStyle.setWidth(i.intValue());
			}
			else {
				// All other sources are combo boxes.
				JComboBox comboBoxSource = (JComboBox)source;
				ComboBoxEntry entry =
				    (ComboBoxEntry)comboBoxSource.getSelectedItem();

				int selection = entry.getConstant();
				if (source == styleComboBox) {
					lineStyle.setPattern(selection);
				}
				else if (source == joinComboBox) {
					lineStyle.setJoin(selection);
				}
				else { // if (source == capComboBox) {
					lineStyle.setCap(selection);
				}
			}
		}
	}
}

protected abstract JCLineStyle getSelectedLineStyle();

}
