/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: LocationEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:00 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.BorderLayout;
import java.awt.Rectangle;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JCheckBox;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.Box;

public abstract class LocationEditor extends Chart3dPropertyEditor implements ActionListener, FocusListener {

private boolean					refreshing = false;
private Container				content;
private JTextField[]			intFields;
private JCheckBox[]				isDefaultCheckBoxes;

public LocationEditor(String nameKey) {
	super(nameKey);
}

public Component getEditor() {

	if (content == null) {

		String[] keys = {
			LocaleBundle.STRING_X,
			LocaleBundle.STRING_Y,
			LocaleBundle.STRING_WIDTH,
			LocaleBundle.STRING_HEIGHT
		};

		JLabel[] labels = new JLabel[4];

		intFields = new JTextField[4];
		isDefaultCheckBoxes = new JCheckBox[4];

		JPanel labelPanel = new JPanel(new GridLayout(0,1,5,5));
		JPanel intFieldPanel = new JPanel(new GridLayout(0,1,5,5));
		JPanel isDefaultCheckBoxPanel = new JPanel(new GridLayout(0,1,5,5));

		String defaultStr = getLocalizedString(LocaleBundle.STRING_DEFAULT);

		for (int each = 0; each < 4; each++) {

			labels[each] = new JLabel(getLocalizedString(keys[each]));
			labelPanel.add(labels[each]);

			intFields[each] = createTextField(7, this, this);
			intFieldPanel.add(intFields[each]);

			isDefaultCheckBoxes[each] = createCheckBox(defaultStr, this);
			isDefaultCheckBoxPanel.add(isDefaultCheckBoxes[each]);
		}

		JPanel controls = createBorderPanel();
		controls.setBorder(null);
		controls.add(labelPanel, BorderLayout.WEST);
		controls.add(intFieldPanel);
		controls.add(isDefaultCheckBoxPanel, BorderLayout.EAST);

		JPanel panel = createPreferredSizePanel();
		panel.add(controls);

		content = Box.createVerticalBox();
		content.add(panel);

	}
	return content;
}

public void finalizeEditor() {

	if (content != null) {

		// Destroy all components.
		for (int each = 0; each < 4; each++) {

			intFields[each].removeActionListener(this);
			intFields[each].removeFocusListener(this);
			intFields[each] = null;

			isDefaultCheckBoxes[each].removeActionListener(this);
			isDefaultCheckBoxes[each] = null;
		}

		intFields = null;
		isDefaultCheckBoxes = null;
		content = null;
	}
}

public void refreshEditor() {

	if (content != null) {

		refreshing = true;

		JComponent c = getLocationComponent();

		Rectangle bounds = c.getBounds();
		int[] boundsDim = { bounds.x, bounds.y, bounds.width, bounds.height };

		Rectangle hints = getChart().getLayoutHints(c);
		int[] hintsDim = { hints.x, hints.y, hints.width, hints.height };

		for (int each = 0; each < 4; each++) {

			intFields[each].setText(String.valueOf(boundsDim[each]));

			boolean isDefault = (hintsDim[each] == Integer.MAX_VALUE);

			isDefaultCheckBoxes[each].setSelected(isDefault);
			intFields[each].setEnabled(!isDefault);
		}

		refreshing = false;
	}
}

public void actionPerformed(ActionEvent e) {
	handleChange(e.getSource());
}

public void focusGained(FocusEvent e) {
	handleChange(e.getSource());
}

public void focusLost(FocusEvent e) {

	if (!e.isTemporary()) {
		handleChange(e.getSource());
	}
}

private void handleChange(Object source) {

	if (refreshing) {
		return;
	}

	if (content != null) {

		JComponent c = getLocationComponent();

		int[] bounds = new int[4];
		int[] hints = new int[4];

		// To save a lot of code, don't check for the source of the event,
		// but instead just iterate through all fields and check boxes and
		// then update the hints rectangle for the target component.
		for (int each = 0; each < 4; each++) {

			Integer i = parseIntegerInput(intFields[each]);

			if (i == null) {
				// Thre parse was unsuccessful!  Refresh the editor and return.
				refreshEditor();
				return;
			}

			bounds[each] = i.intValue();

			if (isDefaultCheckBoxes[each].isSelected()) {

				intFields[each].setEnabled(false);
				hints[each] = Integer.MAX_VALUE;
			}
			else {

				intFields[each].setEnabled(true);
				hints[each] = i.intValue();
			}
		}
		c.setBounds(bounds[0], bounds[1], bounds[2], bounds[3]);
		getChart().setLayoutHints(c, new Rectangle(hints[0], hints[1], hints[2], hints[3]));
		getChart().revalidate();
	}
}

/**
 * Subclasses must implement this method and return the non-null target
 * component of this editor, ie the component whose location is being
 * edited.
 */
public abstract JComponent getLocationComponent();

}
