/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PlotCubeGeneralEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:01 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.JCPlotCube;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.BorderLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.Box;

public class PlotCubeGeneralEditor extends Chart3dPropertyEditor implements ActionListener, FocusListener {

private static final String		nameKey = LocaleBundle.STRING_GENERAL;

private boolean					refreshing = false;
private Container				content;
private JTextField				xScaleField;
private JTextField				yScaleField;
private JTextField				zScaleField;

public PlotCubeGeneralEditor() {
	super(nameKey);
}

public Component getEditor() {

	if (content == null) {

		String local;

		local = getLocalizedString(LocaleBundle.STRING_X_SCALE);
		JLabel xScaleLabel = new JLabel(local);

		xScaleField = createTextField(8, this, this);

		local = getLocalizedString(LocaleBundle.STRING_Y_SCALE);
		JLabel yScaleLabel = new JLabel(local);

		yScaleField = createTextField(8, this, this);

		local = getLocalizedString(LocaleBundle.STRING_Z_SCALE);
		JLabel zScaleLabel = new JLabel(local);

		zScaleField = createTextField(8, this, this);

		JPanel labels = new JPanel(new GridLayout(3,0,5,5));
		labels.add(xScaleLabel);
		labels.add(yScaleLabel);
		labels.add(zScaleLabel);

		JPanel fields = new JPanel(new GridLayout(3,0,5,5));
		fields.add(xScaleField);
		fields.add(yScaleField);
		fields.add(zScaleField);

		JPanel panel = createPreferredSizePanel();
//		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		panel.add(labels, BorderLayout.WEST);
		panel.add(fields);

		content = Box.createVerticalBox();
		content.add(panel);
	}
	return content;
}

public void finalizeEditor() {

	if (content != null) {

		// Destroy all components!
		xScaleField.removeActionListener(this);
		xScaleField.removeFocusListener(this);
		xScaleField = null;

		yScaleField.removeActionListener(this);
		yScaleField.removeFocusListener(this);
		yScaleField = null;

		zScaleField.removeActionListener(this);
		zScaleField.removeFocusListener(this);
		zScaleField = null;

		content = null;
	}
}

public void refreshEditor() {

	if (content != null) {

		refreshing = true;

		JCPlotCube cube = getChart().getChart3dArea().getPlotCube();

		xScaleField.setText(String.valueOf(cube.getXScale()));
		yScaleField.setText(String.valueOf(cube.getYScale()));
		zScaleField.setText(String.valueOf(cube.getZScale()));

		refreshing = false;
	}
}

public void actionPerformed(ActionEvent e) {

	handleChange(e.getSource());
}

public void focusGained(FocusEvent e) {
	// Do nothing!
}

public void focusLost(FocusEvent e) {
	if (!e.isTemporary()) {
		handleChange(e.getSource());
	}
}

private void handleChange(Object source) {

	if (refreshing) {
		return;
	}

	if (content != null) {

		Double d = parseDoubleInput((JTextField)source);

		if (d == null || d.doubleValue() <= 0) {
			// The x, y, and z scales must be greater than zero - otherwise
			// JCPlotCube throws an exception.
			refreshEditor();
			return;
		}

		double		value = d.doubleValue();
		JCPlotCube	cube = getChart().getChart3dArea().getPlotCube();

		if (source == xScaleField) {
			cube.setXScale(value);
		}
		else if (source == yScaleField) {
			cube.setYScale(value);
		}
		else if (source == zScaleField) {
			cube.setZScale(value);
		}

		// This shouldn't have to be called... but the Java 3D chart demands it.
		getChart().update();
	}
}

}
