/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PlotCubeInteriorEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:01 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.JCPlotCube;

import com.klg.jclass.chart3d.customizer.util.JCColorChooser;

import java.awt.Component;
import java.awt.Container;
import java.awt.BorderLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JPanel;
import javax.swing.JComponent;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;

public class PlotCubeInteriorEditor extends Chart3dPropertyEditor implements ActionListener {

private static final String		nameKey = LocaleBundle.STRING_INTERIOR;

private boolean					refreshing;
private Container				content;
private JCColorChooser			bgColorChooser;
private JCColorChooser			fgColorChooser;

public PlotCubeInteriorEditor() {
	super(nameKey);
}

public Component getEditor() {

	if (content == null) {

		String local;

		local = getLocalizedString(LocaleBundle.STRING_BACKGROUND);
		bgColorChooser = new JCColorChooser();
		bgColorChooser.setBorder(BorderFactory.	createTitledBorder(local));
		bgColorChooser.addActionListener(this);

		local = getLocalizedString(LocaleBundle.STRING_FOREGROUND);
		fgColorChooser = new JCColorChooser();
		fgColorChooser.setBorder(BorderFactory.createTitledBorder(local));
		fgColorChooser.addActionListener(this);

		JPanel contents = createPreferredSizePanel();
		contents.setLayout(new BoxLayout(contents, BoxLayout.Y_AXIS));
		contents.add(bgColorChooser);
		contents.add(fgColorChooser);

		content = Box.createVerticalBox();
		content.add(contents);
	}
	return content;
}

public void finalizeEditor() {

	if (content != null) {

		// Destroy all components.

		bgColorChooser.removeActionListener(this);
		bgColorChooser = null;

		fgColorChooser.removeActionListener(this);
		fgColorChooser = null;

		content = null;
	}
}

public void refreshEditor() {

	if (content != null) {

		refreshing = true;

		JCPlotCube cube = getChart().getChart3dArea().getPlotCube();

		bgColorChooser.setColor(cube.getBackground());
		fgColorChooser.setColor(cube.getForeground());

		refreshing = false;
	}
}

public void actionPerformed(ActionEvent e) {

	if (refreshing) {
		return;
	}

	if (content != null) {

		Object		src = e.getSource();
		JCPlotCube	cube = getChart().getChart3dArea().getPlotCube();

		if (src == bgColorChooser) {
			cube.setBackground(bgColorChooser.getColor());
		}
		else if (src == fgColorChooser) {
			cube.setForeground(fgColorChooser.getColor());
		}
	}
}

}
