/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PlotCubeViewportEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:01 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.JCViewport;
import com.klg.jclass.chart3d.JCChart3dUtil;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.BorderLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.JComboBox;
import javax.swing.JCheckBox;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.Box;

public class PlotCubeViewportEditor extends Chart3dPropertyEditor implements ActionListener, FocusListener {

private static String		nameKey = LocaleBundle.STRING_VIEWPORT;

private Container			content;
private JCheckBox			isNormalizedCheckBox;
private JTextField			scaleField;
private JTextField			horizontalShiftField;
private JTextField			verticalShiftField;
private JComboBox			previewMethodComboBox;

public PlotCubeViewportEditor() {
	super(nameKey);
}

public Component getEditor() {

	if (content == null) {

		String local;

		local = getLocalizedString(LocaleBundle.STRING_NORMALIZED);
		isNormalizedCheckBox = createCheckBox(local, this);

		local = getLocalizedString(LocaleBundle.STRING_SCALE);
		JLabel scaleLabel = new JLabel(local);

		scaleField = createTextField(10, this, this);

		local = getLocalizedString(LocaleBundle.STRING_HORIZONTAL_SHIFT);
		JLabel horizontalShiftLabel = new JLabel(local);

		horizontalShiftField = createTextField(10, this, this);

		local = getLocalizedString(LocaleBundle.STRING_VERTICAL_SHIFT);
		JLabel verticalShiftLabel = new JLabel(local);

		verticalShiftField = createTextField(10, this, this);

		local = getLocalizedString(LocaleBundle.STRING_PREVIEW_METHOD);
		JLabel previewMethodLabel = new JLabel(local);

		String[] model = {
				getLocalizedString(LocaleBundle.STRING_CUBE),
				getLocalizedString(LocaleBundle.STRING_FULL)
		};

		previewMethodComboBox = createComboBox(model, this);

		JPanel labels = new JPanel(new GridLayout(5,0,5,5));
		labels.add(isNormalizedCheckBox);
		labels.add(scaleLabel);
		labels.add(horizontalShiftLabel);
		labels.add(verticalShiftLabel);
		labels.add(previewMethodLabel);

		JPanel fields = new JPanel(new GridLayout(5,0,5,5));
		fields.add(new JPanel());
		fields.add(scaleField);
		fields.add(horizontalShiftField);
		fields.add(verticalShiftField);
		fields.add(previewMethodComboBox);

		JPanel controls = createBorderPanel();
		controls.setBorder(null);
		controls.add(labels, BorderLayout.WEST);
		controls.add(fields);

		JPanel panel = createPreferredSizePanel();
		panel.add(controls);

		content = Box.createVerticalBox();
		content.add(panel);

	}
	return content;
}

public void finalizeEditor() {
	content = null;
}

public void refreshEditor() {

	if (content != null) {

		JCViewport viewport = getChart().getChart3dArea().getViewport();

		isNormalizedCheckBox.setSelected(viewport.isNormalized());
		scaleField.setText(String.valueOf(viewport.getScale()));
		horizontalShiftField.setText(String.valueOf(viewport.getHorizontalShift()));
		verticalShiftField.setText(String.valueOf(viewport.getVerticalShift()));

		int index = 0;
		if (viewport.getPreviewMethod() == JCViewport.PREVIEW_CUBE) {
			index = 0;
		}
		else {
			index = 1;
		}
		previewMethodComboBox.setSelectedIndex(index);
	}
}

public void actionPerformed(ActionEvent e) {
	handleChange(e.getSource());
}

public void focusGained(FocusEvent e) {
	handleChange(e.getSource());
}

public void focusLost(FocusEvent e) {

	if (!e.isTemporary()) {
		handleChange(e.getSource());
	}
}

private void handleChange(Object source) {

	JCViewport viewport = getChart().getChart3dArea().getViewport();

	if (source == isNormalizedCheckBox) {
		viewport.setNormalized(isNormalizedCheckBox.isSelected());
		return;
	}

	if (source == previewMethodComboBox) {

		if (previewMethodComboBox.getSelectedIndex() == 0) {
			viewport.setPreviewMethod(JCViewport.PREVIEW_CUBE);
		}
		else {
			viewport.setPreviewMethod(JCViewport.PREVIEW_FULL);
		}
		return;
	}


	// Source must be a text field.

	Double d = parseDoubleInput((JTextField)source);

	if (d == null) {
		refreshEditor();
		return;
	}

	double value = d.doubleValue();

	if (source == scaleField) {

		double scale = JCChart3dUtil.clamp(value, JCViewport.MIN_VIEW_SCALE, JCViewport.MAX_VIEW_SCALE);
		viewport.setScale(scale);
		if (scale != value) {
			refreshEditor();
		}
	}
	else if (source == horizontalShiftField) {
		viewport.setHorizontalShift(value);
	}
	else if (source == verticalShiftField) {
		viewport.setVerticalShift(value);
	}
}

}
