/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ProjectionEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:01 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.JCPlotCube;
import com.klg.jclass.chart3d.JCProjection;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.Box;

public abstract class ProjectionEditor extends Chart3dPropertyEditor implements ActionListener {

private boolean					refreshing = false;
private boolean					isFloor;
private Container				content;
private JCheckBox				isZonedCheckBox;
private JCheckBox				isContouredCheckBox;

public ProjectionEditor(String nameKey, boolean isFloor) {
	super(nameKey);

	this.isFloor = isFloor;
}

public Component getEditor() {

	if (content == null) {

		String local;

		local = getLocalizedString(LocaleBundle.STRING_ZONED);
		isZonedCheckBox = createCheckBox(local, this);

		local = getLocalizedString(LocaleBundle.STRING_CONTOURED);
		isContouredCheckBox = createCheckBox(local, this);

		JPanel panel = createPreferredSizePanel();
		panel.setLayout(new GridLayout(0,1,5,5));
		panel.add(isZonedCheckBox);
		panel.add(isContouredCheckBox);

		content = Box.createVerticalBox();
		content.add(panel);
	}
	return content;
}

public void finalizeEditor() {

	if (content != null) {

		isZonedCheckBox.removeActionListener(this);
		isZonedCheckBox = null;

		isContouredCheckBox.removeActionListener(this);
		isContouredCheckBox = null;

		content = null;
	}
}

public void refreshEditor() {

	if (content != null) {

		refreshing = true;

		JCPlotCube		cube = getChart().getChart3dArea().getPlotCube();
		JCProjection	projection = (isFloor) ?
							cube.getFloor() :
							cube.getCeiling();

		isZonedCheckBox.setSelected(projection.isZoned());
		isContouredCheckBox.setSelected(projection.isContoured());

		refreshing = false;
	}
}

public void actionPerformed(ActionEvent e) {

	if (refreshing) {
		return;
	}

	if (content != null) {

		Object			src = e.getSource();
		JCPlotCube		cube = getChart().getChart3dArea().getPlotCube();
		JCProjection	projection = (isFloor) ?
							cube.getFloor() :
							cube.getCeiling();

		if (src == isZonedCheckBox) {
			projection.setZoned(isZonedCheckBox.isSelected());
			// FIX
			// This is a fix for a bug in the Java 3D implementation which
			// does not automatically update the floor/ceiling projection.
			getChart().update();
		}
		else if (src == isContouredCheckBox) {
			projection.setContoured(isContouredCheckBox.isSelected());
			// FIX
			// This is a fix for a bug in the Java 3D implementation which
			// does not automatically update the floor/ceiling projection.
			getChart().update();
		}
	}
}

}
