/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: SymbolStyleEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:01 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.customizer.util.JCColorChooser;

import com.klg.jclass.chart3d.*;
import com.klg.jclass.chart3d.shape.JCSymbolShape;

import java.awt.BasicStroke;
import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.BorderLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import java.util.List;
import java.util.ArrayList;

import javax.swing.JComboBox;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.BorderFactory;

public abstract class SymbolStyleEditor extends Chart3dPropertyEditor
	implements ActionListener, FocusListener {

private static String		nameKey = LocaleBundle.STRING_SYMBOL_STYLE;

private boolean				refreshing = false;
private Container			content;
private JCColorChooser		colorChooser;
private JTextField			sizeField;
private JComboBox			shapesComboBox;
private List				shapeNames;
private List				shapeClassNames;

public SymbolStyleEditor() {
	super(nameKey);
}

public Component getEditor() {

	if (content == null) {

		String local;

		local = getLocalizedString(LocaleBundle.STRING_SYMBOL);
		colorChooser = getColorChooser();
		colorChooser.addActionListener(this);
		colorChooser.setAlignmentX(0);
		colorChooser.setBorder(BorderFactory.createTitledBorder(local));

		local = getLocalizedString(LocaleBundle.STRING_SIZE);
		JLabel sizeLabel = new JLabel(local);

		sizeField = createTextField(8, this, this);

		local = getLocalizedString(LocaleBundle.STRING_SYMBOL);
		JLabel symbolLabel = new JLabel(local);

		String[] defaultShapeNames = JCSymbolStyle.getDefaultShapes();

		shapeClassNames = new ArrayList();
		shapeNames = new ArrayList();

		for (int each = 0; each < defaultShapeNames.length; each++) {

			JCSymbolShape shape = JCSymbolStyle.getSymbolShapeInstance(defaultShapeNames[each]);

			if (shape != null) {
				shapeClassNames.add(defaultShapeNames[each]);
				shapeNames.add(shape.getName());
			}
		}

		String[] shapesModel = (String[])shapeNames.toArray(new String[0]);

		shapesComboBox = new JComboBox(shapesModel);
        shapesComboBox.addActionListener(this);		
		shapesComboBox.setAlignmentX(0);

		JPanel sizePanel = createBorderPanel();
		sizePanel.setBorder(null);
		sizePanel.add(sizeLabel, BorderLayout.WEST);
		sizePanel.add(sizeField);
		sizePanel.setAlignmentX(0);

		content = createBorderPanel();
		content.setLayout(new BoxLayout(content, BoxLayout.Y_AXIS));
		content.add(colorChooser);
		content.add(Box.createVerticalStrut(10));
		content.add(shapesComboBox);
		content.add(Box.createVerticalStrut(10));
		content.add(sizePanel);
	}
	return content;
}

public void finalizeEditor() {

	if (content != null) {

		// Remove listeners and destroy components.

		colorChooser.removeActionListener(this);
		colorChooser = null;

		sizeField.removeActionListener(this);
		sizeField.removeFocusListener(this);
		sizeField = null;

		shapesComboBox.removeActionListener(this);
		shapesComboBox = null;

		content = null;
	}
}

public void refreshEditor() {

	if (content != null) {

		refreshing = true;

		JCSymbolStyle symbolStyle = getSelectedSymbolStyle();

		if (symbolStyle != null) {

			colorChooser.setColor(symbolStyle.getColor());

			sizeField.setText(String.valueOf(symbolStyle.getSize()));

			int index = shapeClassNames.indexOf(symbolStyle.getShape());

			if (index != -1) {
				shapesComboBox.setSelectedIndex(index);
			}
		}
		refreshing = false;
	}
}

public void actionPerformed(ActionEvent e) {
	handleChange(e.getSource());
}

public void focusGained(FocusEvent e) {
	handleChange(e.getSource());
}

public void focusLost(FocusEvent e) {

	if (!e.isTemporary()) {
		handleChange(e.getSource());
	}
}

private void handleChange(Object source) {
    
	if (refreshing) {
		return;
	}

	if (content != null) {


		JCSymbolStyle symbolStyle = getSelectedSymbolStyle();

		if (symbolStyle != null) {

			if (source == colorChooser) {
				symbolStyle.setColor(colorChooser.getColor());
			}
			else if (source == sizeField) {

				Integer i = parseIntegerInput(sizeField);

				if (i == null || i.intValue() < 0) {
					// The parse was unsuccessful or width is < 0!  Refresh the editor and return.
					refreshEditor();
					return;
				}

				symbolStyle.setSize(i.intValue());
			}
			else if (source == shapesComboBox) {

				int index = shapesComboBox.getSelectedIndex();
				if (index != -1) {
					symbolStyle.setShape((String)shapeClassNames.get(index));
				}
			}
		}
	}
}

protected abstract JCSymbolStyle getSelectedSymbolStyle();

}
