/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TabbedTemplate.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:01 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import java.awt.Component;
import java.awt.BorderLayout;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import javax.swing.border.EmptyBorder;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class TabbedTemplate extends PropertyEditorTemplate implements ChangeListener {

public static final int		DEFAULT_SELECTION = 0;

private int					selectedIndex = -1;
private PropertyEditor		selectedEditor;
private JPanel				editorComponent;
private JTabbedPane			tabbedPane;

public TabbedTemplate(PropertyEditor source) {
	super(source);
}

public Component getEditorComponent() {

	if (editorComponent == null) {

		JPanel panel = new JPanel(new BorderLayout());

		Component e = getSource().getEditor();

		if (e != null) {
			panel.add(e, BorderLayout.WEST);
		}

		int childCount = getSource().getChildCount();

		if (childCount > 0) {

			tabbedPane = new JTabbedPane();
			tabbedPane.setBorder(new EmptyBorder(5,5,5,5));
			for (int each = 0; each < childCount; each++) {

				String name = getSource().getChildAt(each).getName();
				tabbedPane.add(name, new ProxyComponent());
			}

			panel.add(tabbedPane, BorderLayout.CENTER);
		}

		editorComponent = new JPanel(new BorderLayout());
		editorComponent.add(panel, BorderLayout.NORTH);

		if (tabbedPane != null) {

			tabbedPane.addChangeListener(this);
			tabbedPane.setSelectedIndex(DEFAULT_SELECTION);
			handleChange();
		}
	}

	return editorComponent;
}

public void stateChanged(ChangeEvent e) {
	handleChange();
}

private void handleChange() {

	if (tabbedPane.getSelectedIndex() == selectedIndex) {
		return;
	}

	if (selectedEditor != null) {
		// Then there was a previously selected property editor.
		// Remove it and replace it with a proxy component - and
		// don't forget to finalize it.

		selectedEditor.finalizeEditorTree();
		tabbedPane.setComponentAt(selectedIndex, new ProxyComponent());
	}

	selectedIndex = tabbedPane.getSelectedIndex();
	selectedEditor = getSource().getChildAt(selectedIndex);

	// Construct the template for the selected property editor.
	TabbedTemplate template = new TabbedTemplate(selectedEditor);

	// And add it to the tabbed pane.
	tabbedPane.setComponentAt(selectedIndex, template.getEditorComponent());

	// Make sure the properties are current.
	selectedEditor.refreshEditor();

	// And make sure the tabbed pane is repainted.
	tabbedPane.invalidate();
	tabbedPane.revalidate();
	tabbedPane.repaint();
}

public static class ProxyComponent extends Component {
}

}
