/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TitlesGeneralEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:01 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.BorderLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JCheckBox;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.Box;


public class TitlesGeneralEditor extends Chart3dPropertyEditor implements ActionListener, FocusListener {

private static final String		nameKey = LocaleBundle.STRING_GENERAL;

private boolean					refreshing = false;
private Container				content;
private JCheckBox				isVisibleCheckBox;
private JTextField				textField;
private AlignmentComboBox		horizontalAlignmentComboBox;
private AlignmentComboBox		verticalAlignmentComboBox;

public TitlesGeneralEditor() {
	super(nameKey);
}

public Component getEditor() {

	if (content == null) {

		String local;

		local = getLocalizedString(LocaleBundle.STRING_VISIBLE);
		isVisibleCheckBox = createCheckBox(local, this);

		local = getLocalizedString(LocaleBundle.STRING_TEXT);
		JLabel textLabel = new JLabel(local);

		textField = createTextField(10, this, this);

		local = getLocalizedString(LocaleBundle.STRING_HORIZONTAL);
		JLabel horizontalAlignmentLabel =  new JLabel(local);

		horizontalAlignmentComboBox = new AlignmentComboBox(AlignmentComboBox.HORIZONTAL, this);

		local = getLocalizedString(LocaleBundle.STRING_VERTICAL);
		JLabel verticalAlignmentLabel =  new JLabel(local);

		verticalAlignmentComboBox = new AlignmentComboBox(AlignmentComboBox.VERTICAL, this);

		JPanel labels = new JPanel(new GridLayout(5,0,5,5));
		labels.add(isVisibleCheckBox);
		labels.add(textLabel);
		labels.add(horizontalAlignmentLabel);
		labels.add(verticalAlignmentLabel);

		JPanel fields = new JPanel(new GridLayout(5,0,5,5));
		fields.add(new JPanel());
		fields.add(textField);
		fields.add(horizontalAlignmentComboBox);
		fields.add(verticalAlignmentComboBox);

		JPanel controls = createBorderPanel();
		controls.setBorder(null);
		controls.add(labels, BorderLayout.WEST);
		controls.add(fields);

		JPanel panel = createPreferredSizePanel();
		panel.add(controls);

		content = Box.createVerticalBox();
		content.add(panel);

	}
	return content;
}

public void finalizeEditor() {

	if (content != null) {

		isVisibleCheckBox.removeActionListener(this);
		isVisibleCheckBox = null;

		textField.removeActionListener(this);
		textField.removeFocusListener(this);
		textField = null;

		horizontalAlignmentComboBox.removeActionListener(this);
		horizontalAlignmentComboBox = null;

		verticalAlignmentComboBox.removeActionListener(this);
		verticalAlignmentComboBox = null;

		content = null;
	}
}

public void refreshEditor() {

	if (content != null) {

		refreshing = true;

		TitlesEditor parent = (TitlesEditor)getParent();
		JComponent c = parent.getSelectedTitleComponent();

		content.setEnabled(true);

		isVisibleCheckBox.setSelected(c.isVisible());

		if (c instanceof JLabel) {
			JLabel t = (JLabel)c;
			textField.setText(t.getText());
			horizontalAlignmentComboBox.setSelectedAlignment(t.getHorizontalTextPosition());
			verticalAlignmentComboBox.setSelectedAlignment(t.getVerticalTextPosition());
		}

		refreshing = false;
	}
}

public void actionPerformed(ActionEvent e) {
	handleChange(e.getSource());
}

public void focusGained(FocusEvent e) {
	handleChange(e.getSource());
}

public void focusLost(FocusEvent e) {

	if (!e.isTemporary()) {
		handleChange(e.getSource());
	}
}

private void handleChange(Object source) {

	if (refreshing) {
		return;
	}

	if (content != null) {

		TitlesEditor parent = (TitlesEditor)getParent();
		JComponent c = parent.getSelectedTitleComponent();

		if (source == isVisibleCheckBox) {
			c.setVisible(isVisibleCheckBox.isSelected());
			return;
		}

		if (!(c instanceof JLabel)) {
			return;
		}

		JLabel t = (JLabel)c;

		if (source == textField) {
			t.setText(textField.getText());
		}
		else if (source == horizontalAlignmentComboBox) {
			t.setHorizontalAlignment(horizontalAlignmentComboBox.getSelectedAlignment());
		}
		else if (source == verticalAlignmentComboBox) {
			t.setVerticalAlignment(verticalAlignmentComboBox.getSelectedAlignment());
		}
	}
}

}
