/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TreeTemplate.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:01 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.BorderLayout;

import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;

import java.util.Enumeration;

import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.JSplitPane;

import javax.swing.border.EmptyBorder;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;

import javax.swing.tree.*;

public class TreeTemplate extends PropertyEditorTemplate {

private JSplitPane			editorComponent;
private JTree				tree;
private Container			rightComponent;

public TreeTemplate(PropertyEditor source) {
	super(source);
}

public Component getEditorComponent() {

	if (editorComponent == null) {

		tree = new JTree(new DefaultTreeModel(new PropertyEditorTreeNode(getSource())));
		tree.setRootVisible(false);
//		tree.setPreferredSize(new Dimension(400,400));
//		tree.addTreeExpansionListener(new MyTreeListener());
		tree.addMouseListener(new MouseAdapter() {

			public void mousePressed(MouseEvent e) {
				int row = tree.getRowForLocation(e.getX(), e.getY());
				TreePath path = tree.getPathForLocation(e.getX(), e.getY());
				if(row != -1) {

					PropertyEditor editor = ((PropertyEditorTreeNode)path.getLastPathComponent()).getSource();

					rightComponent.removeAll();

					if (editor.getEditor() != null) {
						rightComponent.add(editor.getEditor(), BorderLayout.NORTH);
					}
					editorComponent.setRightComponent(rightComponent);
					editor.refreshEditor();
//					editorComponent.resetToPreferredSizes();
				}
			}
		});

		rightComponent = Chart3dPropertyEditor.createBorderPanel();

		editorComponent = new JSplitPane();
		editorComponent.setLeftComponent(tree);
		editorComponent.setRightComponent(rightComponent);
	}

	return editorComponent;
}

public class MyTreeListener implements TreeExpansionListener {

	/**
	 * Called whenever an item in the tree has been collapsed.
	 */
	public void treeCollapsed(TreeExpansionEvent e) {
		handleChange(e);
	}

	public void treeExpanded(TreeExpansionEvent e) {
		handleChange(e);
	}

	private void handleChange(TreeExpansionEvent e) {
		TreePath			path = e.getPath();
		PropertyEditor		editor = ((PropertyEditorTreeNode)path.getPathComponent(0)).getSource();

		rightComponent.removeAll();
		rightComponent.add(editor.getEditor(), BorderLayout.NORTH);
		editorComponent.setRightComponent(rightComponent);
//		editorComponent.resetToPreferredSizes();
	}
}


public static class PropertyEditorTreeNode implements TreeNode {

	private int							enumIndex;
	private PropertyEditor				source;

	private PropertyEditorTreeNode[]	childrenAsTreeNodes;

	public PropertyEditorTreeNode(PropertyEditor source) {
		this.source = source;
	}

	public PropertyEditor getSource() {
		return source;
	}

	/**
	 * Returns the children of the reciever as an Enumeration.
	 */
	public Enumeration children() {

		enumIndex = 0;
		return new Enumeration() {
			public boolean hasMoreElements() {
				return enumIndex < getChildCount();
			}

			public Object nextElement() {
				enumIndex++;
				return getChildAt(enumIndex - 1);
			}
		};
	}

	/**
	 * Returns true if the receiver allows children.
	 */
	public boolean getAllowsChildren() {
		return true;
	}

	/**
	 * Returns the child TreeNode at index childIndex.
	 */
	public TreeNode getChildAt(int index) {
		return new PropertyEditorTreeNode(source.getChildAt(index));
	}

	/**
	 * Returns the number of children TreeNodes the receiver contains.
	 */
	public int getChildCount() {
		return source.getChildCount();
	}

	/**
	 * Returns the index of node in the receivers children.
	 */
	public int getIndex(TreeNode node) {
		PropertyEditor editor = ((PropertyEditorTreeNode)node).source;
		return source.getIndexOfChild(editor);
	}

	/**
	 * Returns the parent TreeNode of the receiver.
	 */
	public TreeNode getParent() {
		if (source.getParent() != null) {
			return new PropertyEditorTreeNode(source.getParent());
		}

		return null;
	}

	/**
	 * Returns true if the receiver is a leaf.
	 */
	public boolean isLeaf() {
		return source.isLeaf();
	}

	public String toString() {
		return source.toString();
	}

}

}
