/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: View3dEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:01 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer;

import com.klg.jclass.chart3d.JCView3d;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.Box;

public class View3dEditor extends Chart3dPropertyEditor implements ActionListener, FocusListener {

private static final String		nameKey = LocaleBundle.STRING_VIEW_3D;

private boolean					refreshing = false;
private Container				content;
private JTextField				perspectiveField;
private JTextField				xRotationField;
private JTextField				yRotationField;
private JTextField				zRotationField;

public View3dEditor() {
	super(nameKey);
}

public Component getEditor() {

	if (content == null) {

		String local;

		local = getLocalizedString(LocaleBundle.STRING_PERSPECTIVE);
		JLabel perspectiveLabel = new JLabel(local);

		perspectiveField = createTextField(12, this, this);

		local = getLocalizedString(LocaleBundle.STRING_X_ROTATION);
		JLabel xRotationLabel = new JLabel(local);

		xRotationField = createTextField(12, this, this);

		local = getLocalizedString(LocaleBundle.STRING_Y_ROTATION);
		JLabel yRotationLabel = new JLabel(local);

		yRotationField = createTextField(12, this, this);

		local = getLocalizedString(LocaleBundle.STRING_Z_ROTATION);
		JLabel zRotationLabel = new JLabel(local);

		zRotationField = createTextField(12, this, this);

		JPanel labels = new JPanel(new GridLayout(4,0,5,5));
		labels.add(perspectiveLabel);
		labels.add(xRotationLabel);
		labels.add(yRotationLabel);
		labels.add(zRotationLabel);

		JPanel fields = new JPanel(new GridLayout(4,0,5,5));
		fields.add(perspectiveField);
		fields.add(xRotationField);
		fields.add(yRotationField);
		fields.add(zRotationField);

		JPanel panel = createPreferredSizePanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		panel.add(labels);
		panel.add(fields);

		content = Box.createVerticalBox();
		content.add(panel);

	}
	return content;
}

public void finalizeEditor() {

	if (content != null) {

		JTextField[] fields = {	perspectiveField, xRotationField, yRotationField, zRotationField };

		for (int each = 0; each < 4; each++) {
			fields[each].removeActionListener(this);
			fields[each].removeFocusListener(this);
		}

		fields = null;

		perspectiveField = null;
		xRotationField = null;
		yRotationField = null;
		zRotationField = null;

		content = null;
	}
}

public void refreshEditor() {

	if (content != null) {

		refreshing = true;

		JCView3d view = getChart().getChart3dArea().getView3d();

		perspectiveField.setText(String.valueOf(view.getPerspective()));
		xRotationField.setText(String.valueOf(view.getXRotation()));
		yRotationField.setText(String.valueOf(view.getYRotation()));
		zRotationField.setText(String.valueOf(view.getZRotation()));

		refreshing = false;
	}
}

public void actionPerformed(ActionEvent e) {

	handleChange(e.getSource());
}

public void focusGained(FocusEvent e) {
	handleChange(e.getSource());
}

public void focusLost(FocusEvent e) {
	if (!e.isTemporary()) {
		handleChange(e.getSource());
	}
}

private void handleChange(Object source) {

	if (refreshing) {
		return;
	}

	if (content != null) {

		Double d = parseDoubleInput((JTextField)source);

		if (d == null) {
			refreshEditor();
			return;
		}

		double		value = d.doubleValue();
		JCView3d	view = getChart().getChart3dArea().getView3d();

		if (source == perspectiveField) {
			view.setPerspective(value);
		}
		else if (source == xRotationField) {
			view.setXRotation(value);
		}
		else if (source == yRotationField) {
			view.setYRotation(value);
		}
		else if (source == zRotationField) {
			view.setZRotation(value);
		}
	}
}

}
