/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCBorderChooser.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:01 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer.util;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import com.klg.jclass.chart3d.customizer.*;
import com.klg.jclass.util.JCLocaleManager;
import com.klg.jclass.chart3d.customizer.util.resources.LocaleBundle;

/* Internationalized */

public class JCBorderChooser extends JPanel implements ItemListener,ActionListener,FocusListener {

	protected JCLocaleManager lm = JCLocaleManager.getDefault();

	protected EventListenerList listeners;

	protected JComboBox borderList;

	protected JTextField topEmptyField;
	protected JTextField leftEmptyField;
	protected JTextField bottomEmptyField;
	protected JTextField rightEmptyField;

	protected JTextField topMatteField;
	protected JTextField leftMatteField;
	protected JTextField bottomMatteField;
	protected JTextField rightMatteField;

	protected ButtonGroup bevelGroup;
	protected JRadioButton bevelLoweredButton;
	protected JRadioButton bevelRaisedButton;

	protected JTextField thickField;
	protected JTextField titledField;

	protected JCColorChooser lineColorChooser;
	protected JCColorChooser matteColorChooser;
	protected JCColorChooser shadowColorChooser;
	protected JCColorChooser highlightColorChooser;

	protected JPanel bevelEditor;
	protected JPanel lineEditor;
	protected JPanel matteEditor;
	protected JPanel emptyEditor;
	protected JPanel titledEditor;

	protected JPanel activeEditor;
	protected JPanel editorPanel;

	protected Border border;

	protected ArrayList borderNames = new ArrayList(6);

	protected Object focusValue;

	protected static final int EMPTY = 1;
	protected static final int BEVEL = 2;
	protected static final int ETCHED = 3;
	protected static final int LINE = 4;
	protected static final int MATTE = 5;
	protected static final int TITLED = 6;

	private boolean isSetSelectedBorder = false;

	static {
		JCLocaleManager.getDefault().add("com.klg.jclass.chart3d.customizer.util.resources.LocaleInfo");
	}

	public JCBorderChooser() {
		super();
		initialize();
		setSelectedBorder(new EmptyBorder(0,0,0,0));
	}

	public JCBorderChooser(Border initialBorder) {
		super();
		initialize();
		setSelectedBorder(initialBorder);
	}

	private void initialize() {
		listeners = new EventListenerList();

		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		constraints.anchor = GridBagConstraints.NORTHWEST;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.insets = new Insets(3,0,3,0);

		setLayout(layout);

		borderNames.add(lm.getString(LocaleBundle.STRING_EMPTY));
		borderNames.add(lm.getString(LocaleBundle.STRING_BEVEL));
		borderNames.add(lm.getString(LocaleBundle.STRING_ETCHED));
		borderNames.add(lm.getString(LocaleBundle.STRING_LINE));
		borderNames.add(lm.getString(LocaleBundle.STRING_MATTE));
		borderNames.add(lm.getString(LocaleBundle.STRING_TITLED));

		DefaultComboBoxModel model = new DefaultComboBoxModel();
		model.addElement(new Integer(EMPTY));
		model.addElement(new Integer(BEVEL));
		model.addElement(new Integer(ETCHED));
		model.addElement(new Integer(LINE));
		model.addElement(new Integer(MATTE));
		model.addElement(new Integer(TITLED));

		borderList = new JComboBox(model);
		borderList.setRenderer(new CustomCellRenderer());
		borderList.addItemListener(this);
		layout.setConstraints(borderList,constraints);
		add(borderList);

		editorPanel = new JPanel();
		editorPanel.setLayout(new CardLayout());

		emptyEditor = createEmptyEditor();
		editorPanel.add(emptyEditor,lm.getString(LocaleBundle.STRING_EMPTY_BORDER));

		layout.setConstraints(editorPanel,constraints);
		add(editorPanel);

		activeEditor = emptyEditor;
	}

	public void setSelectedBorder(Border border) {
		isSetSelectedBorder = true;

		if (border instanceof MatteBorder) {
			MatteBorder matteBorder = (MatteBorder) border;

			borderList.setSelectedItem(new Integer(MATTE));

			Insets i = matteBorder.getBorderInsets(this);

			topMatteField.setText(String.valueOf(i.top));
			leftMatteField.setText(String.valueOf(i.left));
			bottomMatteField.setText(String.valueOf(i.bottom));
			rightMatteField.setText(String.valueOf(i.right));

			Color c = matteColorChooser.getColor();

			if (c == null)
				c = Color.black;

			matteColorChooser.setColor(c);
		}
		else if (border instanceof BevelBorder) {
			BevelBorder bevelBorder = (BevelBorder) border;

			borderList.setSelectedItem(new Integer(BEVEL));

			if (bevelBorder.getBevelType() == BevelBorder.LOWERED) {
				bevelLoweredButton.setSelected(true);
			}
			else {
				bevelRaisedButton.setSelected(true);
			}

			Color c = highlightColorChooser.getColor();

			if (c == null)
				c = Color.black;

			highlightColorChooser.setColor(c);

			c = shadowColorChooser.getColor();

			if (c == null)
				c = Color.black;

			shadowColorChooser.setColor(c);
		}
		else if (border instanceof EtchedBorder) {
			EtchedBorder etchedBorder = (EtchedBorder) border;

			borderList.setSelectedItem(new Integer(ETCHED));

			if (etchedBorder.getEtchType() == BevelBorder.LOWERED) {
				bevelLoweredButton.setSelected(true);
			}
			else {
				bevelRaisedButton.setSelected(true);
			}

//			highlightColorChooser.setColor(etchedBorder.getHighlightColor());
//			shadowColorChooser.setColor(etchedBorder.getShadowColor());
		}
		else if (border instanceof LineBorder) {
			LineBorder lineBorder = (LineBorder) border;

			borderList.setSelectedItem(new Integer(LINE));

			thickField.setText(String.valueOf(lineBorder.getThickness()));
			lineColorChooser.setColor(lineBorder.getLineColor());
		}
		else if (border instanceof EmptyBorder) {
			EmptyBorder emptyBorder = (EmptyBorder) border;

			borderList.setSelectedItem(new Integer(EMPTY));

			Insets i = emptyBorder.getBorderInsets(this);

			topEmptyField.setText(String.valueOf(i.top));
			leftEmptyField.setText(String.valueOf(i.left));
			bottomEmptyField.setText(String.valueOf(i.bottom));
			rightEmptyField.setText(String.valueOf(i.right));
		}
		else if (border instanceof TitledBorder) {
			TitledBorder titledBorder = (TitledBorder) border;

			borderList.setSelectedItem(new Integer(TITLED));

			titledField.setText(titledBorder.getTitle());
		}

		isSetSelectedBorder = false;

		this.border = border;
	}

	public JPanel createBevelEditor() {
		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		constraints.anchor = GridBagConstraints.NORTHWEST;

		JPanel panel = new JPanel();
		panel.setLayout(layout);

		JPanel subpanel = new JPanel();

		ButtonGroup bevelGroup = new ButtonGroup();

		bevelLoweredButton = new JRadioButton(lm.getString(LocaleBundle.STRING_LOWERED));
		bevelRaisedButton = new JRadioButton(lm.getString(LocaleBundle.STRING_RAISED));
		bevelLoweredButton.addActionListener(this);
		bevelRaisedButton.addActionListener(this);
		bevelLoweredButton.setSelected(true);

		bevelGroup.add(bevelLoweredButton);
		bevelGroup.add(bevelRaisedButton);

		subpanel.add(bevelLoweredButton);
		subpanel.add(new JPanel());
		subpanel.add(bevelRaisedButton);

		layout.setConstraints(subpanel,constraints);
		panel.add(subpanel);

		highlightColorChooser = new JCColorChooser(Color.white);
		highlightColorChooser.setBorder(BorderFactory.createTitledBorder(lm.getString(LocaleBundle.STRING_HIGHLIGHT_COLOR)));
		highlightColorChooser.addActionListener(this);
		layout.setConstraints(highlightColorChooser,constraints);
		panel.add(highlightColorChooser);

		shadowColorChooser = new JCColorChooser(Color.black);
		shadowColorChooser.setBorder(BorderFactory.createTitledBorder(lm.getString(LocaleBundle.STRING_SHADOW_COLOR)));
		shadowColorChooser.addActionListener(this);
		layout.setConstraints(shadowColorChooser,constraints);
		panel.add(shadowColorChooser);

		JPanel outerPanel = new JPanel();
		outerPanel.setLayout(new FlowLayout(FlowLayout.LEFT,5,5));
		outerPanel.add(panel);
		//outerPanel.setBorder(BorderFactory.createEtchedBorder());

		return outerPanel;
	}

	public JPanel createLineEditor() {
		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		constraints.anchor = GridBagConstraints.NORTHWEST;

		JPanel panel = new JPanel();
		panel.setLayout(layout);

		JPanel subpanel = new JPanel();

		thickField = new JTextField("0",11);
		thickField.addActionListener(this);
		thickField.addFocusListener(this);
		subpanel.add(new JLabel("  " + lm.getString(LocaleBundle.STRING_LINE_THICKNESS) + ": "));
		subpanel.add(thickField);

		layout.setConstraints(subpanel,constraints);

		panel.add(subpanel);

		lineColorChooser = new JCColorChooser();
		lineColorChooser.setBorder(BorderFactory.createTitledBorder(lm.getString(LocaleBundle.STRING_LINE_COLOR)));
		lineColorChooser.addActionListener(this);
		layout.setConstraints(lineColorChooser,constraints);
		panel.add(lineColorChooser);

		JPanel outerPanel = new JPanel();
		outerPanel.setLayout(new FlowLayout(FlowLayout.LEFT,5,5));
		outerPanel.add(panel);
		//outerPanel.setBorder(BorderFactory.createEtchedBorder());

		return outerPanel;
	}

	public JPanel createEmptyEditor() {
		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.anchor = GridBagConstraints.NORTHWEST;
		constraints.weightx = 1.0;
		constraints.weighty = 1.0;

		JPanel panel = new JPanel();
		panel.setLayout(layout);

		JPanel subpanel = new JPanel();
		subpanel.setLayout(new GridLayout(4,2,0,15));

		topEmptyField = new JTextField("0",8);
		topEmptyField.addActionListener(this);
		topEmptyField.addFocusListener(this);
		subpanel.add(new JLabel(lm.getString(LocaleBundle.STRING_TOP_MARGIN) + ": "));
		subpanel.add(topEmptyField);

		leftEmptyField = new JTextField("0",8);
		leftEmptyField.addActionListener(this);
		leftEmptyField.addFocusListener(this);
		subpanel.add(new JLabel(lm.getString(LocaleBundle.STRING_LEFT_MARGIN) + ": "));
		subpanel.add(leftEmptyField);

		bottomEmptyField = new JTextField("0",8);
		bottomEmptyField.addActionListener(this);
		bottomEmptyField.addFocusListener(this);
		subpanel.add(new JLabel(lm.getString(LocaleBundle.STRING_BOTTOM_MARGIN) + ": "));
		subpanel.add(bottomEmptyField);

		rightEmptyField = new JTextField("0",8);
		rightEmptyField.addActionListener(this);
		rightEmptyField.addFocusListener(this);
		subpanel.add(new JLabel(lm.getString(LocaleBundle.STRING_RIGHT_MARGIN) + ": "));
		subpanel.add(rightEmptyField);

		layout.setConstraints(subpanel,constraints);
		panel.add(subpanel);

		JPanel outerPanel = new JPanel();
		outerPanel.setLayout(new FlowLayout(FlowLayout.LEFT,5,5));
		outerPanel.add(panel);
		//outerPanel.setBorder(BorderFactory.createEtchedBorder());

		return outerPanel;
	}

	public JPanel createMatteEditor() {
		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		constraints.anchor = GridBagConstraints.NORTHWEST;
		constraints.weightx = 1.0;
		constraints.weighty = 1.0;

		JPanel panel = new JPanel();
		panel.setLayout(layout);

		matteColorChooser = new JCColorChooser();
		matteColorChooser.setBorder(BorderFactory.createTitledBorder(lm.getString(LocaleBundle.STRING_MATTE_COLOR)));
		matteColorChooser.addActionListener(this);
		layout.setConstraints(matteColorChooser,constraints);
		panel.add(matteColorChooser);
		panel.add(new JPanel());

		JPanel subpanel = new JPanel();
		subpanel.setLayout(new GridLayout(4,2,0,15));

		topMatteField = new JTextField("0",8);
		topMatteField.addActionListener(this);
		topMatteField.addFocusListener(this);
		subpanel.add(new JLabel("  " + lm.getString(LocaleBundle.STRING_TOP_MARGIN) + ": "));
		subpanel.add(topMatteField);

		leftMatteField = new JTextField("0",8);
		leftMatteField.addActionListener(this);
		leftMatteField.addFocusListener(this);
		subpanel.add(new JLabel("  " + lm.getString(LocaleBundle.STRING_LEFT_MARGIN) + ": "));
		subpanel.add(leftMatteField);

		bottomMatteField = new JTextField("0",8);
		bottomMatteField.addActionListener(this);
		bottomMatteField.addFocusListener(this);
		subpanel.add(new JLabel("  " + lm.getString(LocaleBundle.STRING_BOTTOM_MARGIN) + ": "));
		subpanel.add(bottomMatteField);

		rightMatteField = new JTextField("0",8);
		rightMatteField.addActionListener(this);
		rightMatteField.addFocusListener(this);
		subpanel.add(new JLabel("  " + lm.getString(LocaleBundle.STRING_RIGHT_MARGIN) + ": "));
		subpanel.add(rightMatteField);

		layout.setConstraints(subpanel,constraints);
		panel.add(subpanel);

		JPanel outerPanel = new JPanel();
		outerPanel.setLayout(new FlowLayout(FlowLayout.LEFT,5,5));
		outerPanel.add(panel);
		//outerPanel.setBorder(BorderFactory.createEtchedBorder());

		return outerPanel;
	}

	public JPanel createTitledEditor() {
		JPanel panel = new JPanel();
		panel.setLayout(new FlowLayout(FlowLayout.LEFT,5,5));

		titledField = new JTextField(12);
		titledField.addActionListener(this);
		titledField.addFocusListener(this);
		panel.add(new JLabel(lm.getString(LocaleBundle.STRING_TITLE_TEXT) + ": "));
		panel.add(titledField);

		//panel.setBorder(BorderFactory.createEtchedBorder());

		return panel;
	}

	public void itemStateChanged(ItemEvent e) {
		Object source = e.getSource();

		activeEditor.setVisible(false);

		Integer selection = (Integer) borderList.getSelectedItem();

		switch (selection.intValue()) {
			case EMPTY:
				activeEditor = emptyEditor;
				break;
			case BEVEL:
				if (bevelEditor == null) {
					bevelEditor = createBevelEditor();
					editorPanel.add(bevelEditor,lm.getString(LocaleBundle.STRING_BEVEL_BORDER));
				}

				activeEditor = bevelEditor;
				break;
			case ETCHED:
				if (bevelEditor == null) {
					bevelEditor = createBevelEditor();
					editorPanel.add(bevelEditor,lm.getString(LocaleBundle.STRING_BEVEL_BORDER));
				}

				activeEditor = bevelEditor;
				break;
			case LINE:
				if (lineEditor == null) {
					lineEditor = createLineEditor();
					editorPanel.add(lineEditor,lm.getString(LocaleBundle.STRING_LINE_BORDER));
				}

				activeEditor = lineEditor;
				break;
			case MATTE:
				if (matteEditor == null) {
					matteEditor = createMatteEditor();
					editorPanel.add(matteEditor,lm.getString(LocaleBundle.STRING_MATTE_BORDER));
				}

				activeEditor = matteEditor;
				break;
			case TITLED:
				if (titledEditor == null) {
					titledEditor = createTitledEditor();
					editorPanel.add(titledEditor,lm.getString(LocaleBundle.STRING_TITLED_BORDER));
				}

				activeEditor = titledEditor;
				break;
		}

		activeEditor.setVisible(true);

		if (!isSetSelectedBorder)
			actionPerformed(new ActionEvent(source,0,null));
	}

	public void setEnabled(boolean b) {
		super.setEnabled(b);

		borderList.setEnabled(b);

		if (emptyEditor != null) {
			topEmptyField.setEnabled(b);
			leftEmptyField.setEnabled(b);
			bottomEmptyField.setEnabled(b);
			rightEmptyField.setEnabled(b);
		}

		if (matteEditor != null) {
			topMatteField.setEnabled(b);
			leftMatteField.setEnabled(b);
			bottomMatteField.setEnabled(b);
			rightMatteField.setEnabled(b);
			matteColorChooser.setEnabled(b);
		}

		if (bevelEditor != null) {
			bevelLoweredButton.setEnabled(b);
			bevelRaisedButton.setEnabled(b);
			shadowColorChooser.setEnabled(b);
			highlightColorChooser.setEnabled(b);
		}

		if (lineEditor != null) {
			thickField.setEnabled(b);
			lineColorChooser.setEnabled(b);
		}

		if (titledEditor != null) {
			titledField.setEnabled(b);
		}
	}


	public void setPanelBorder(Border border) {
		bevelEditor.setBorder(border);
		lineEditor.setBorder(border);
		matteEditor.setBorder(border);
		emptyEditor.setBorder(border);
		titledEditor.setBorder(border);
	}

	public void actionPerformed(ActionEvent e) {
		changePerformed(e.getSource());
	}

	public void focusLost(FocusEvent e) {
		Object source = e.getSource();

		if (source instanceof JTextField) {
			String s1 = (String) focusValue;
			String s2 = ((JTextField) source).getText();

			if (!s1.equals(s2)) {
				changePerformed(e.getSource());
			}
		}
		else {
			changePerformed(e.getSource());
		}
	}

	public void focusGained(FocusEvent e) {
		Object source = e.getSource();

		if (source instanceof JTextField) {
			focusValue = ((JTextField) source).getText();
		}
		else {
			focusValue = null;
		}
	}

	public void changePerformed(Object source) {
		Border border = null;

		int type = BevelBorder.LOWERED;

		if (bevelEditor != null) {
			if (bevelLoweredButton.isSelected()) {
				type = BevelBorder.LOWERED;
			}
			else {
				type = BevelBorder.RAISED;
			}
		}

		Integer selection = (Integer) borderList.getSelectedItem();

		switch (selection.intValue()) {
			case EMPTY:
				border = BorderFactory.createEmptyBorder(intValue(topEmptyField),
														 intValue(leftEmptyField),
														 intValue(bottomEmptyField),
														 intValue(rightEmptyField));
				break;
			case MATTE:
				border = BorderFactory.createMatteBorder(intValue(topMatteField),
														 intValue(leftMatteField),
														 intValue(bottomMatteField),
														 intValue(rightMatteField),
														 matteColorChooser.getColor());
				break;
			case BEVEL:
				border = BorderFactory.createBevelBorder(type,
														 highlightColorChooser.getColor(),
														 shadowColorChooser.getColor());
				break;
			case ETCHED:
				border = BorderFactory.createEtchedBorder(highlightColorChooser.getColor(),
														  shadowColorChooser.getColor());
				break;
			case LINE:
				border = BorderFactory.createLineBorder(lineColorChooser.getColor(),
														intValue(thickField));
				break;
			case TITLED:
				border = BorderFactory.createTitledBorder(titledField.getText());
				break;
		}

		this.border = border;

		fireAction();
	}

	public Border getSelectedBorder() {
		return border;
	}

	public void addActionListener (ActionListener listener) {
		listeners.add(listener.getClass(),listener);
	}

	public void removeActionListener(ActionListener listener) {
		listeners.remove(listener.getClass(),listener);
	}

	private int intValue(JTextField tf) {
		String s = tf.getText();

		int value = 0;

		try {
			value = Integer.valueOf(s).intValue();
		} catch (NumberFormatException nfe) {
			tf.setText("0");
		}

		return value;
	}



	protected void fireAction() {
		ActionEvent event = new ActionEvent(this,0,null);

		Object[] listenerList = listeners.getListenerList();

		for (int i = listenerList.length - 2; i>=0; i-=2) {
			if (listenerList[i+1] instanceof ActionListener) {;
				((ActionListener)listenerList[i+1]).actionPerformed(event);
			}
		}
	}

	private class CustomCellRenderer extends JLabel implements ListCellRenderer {
		public CustomCellRenderer() {
			super();

			setOpaque(true);
			setBorder(new EmptyBorder(1,1,1,1));
		}

		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
			if (index < 0) {
				DefaultComboBoxModel model = (DefaultComboBoxModel) list.getModel();
				index = model.getIndexOf(value);
			}

			setText((String) borderNames.get(index));

			setFont(list.getFont());

			if (isSelected) {
	            setBackground(list.getSelectionBackground());
	            setForeground(list.getSelectionForeground());
			}
	        else {
	            setBackground(list.getBackground());
	            setForeground(list.getForeground());
			}

			return this;
		}
	}
}









