/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCColorChooser.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:01 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.customizer.util;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.beans.*;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import com.klg.jclass.chart3d.customizer.*;
import com.klg.jclass.util.swing.JCIconCreator;
import com.klg.jclass.util.JCLocaleManager;
import com.klg.jclass.chart3d.customizer.util.resources.LocaleBundle;

/* Internationalized */

public class JCColorChooser extends JPanel implements ActionListener,FocusListener {

	protected JCLocaleManager lm = JCLocaleManager.getDefault();

	protected EventListenerList listeners;
	protected JTextField colorRGB;
	protected JComboBox colorName;
	protected Color color;
	protected JButton colorPickButton;
	protected ArrayList colornameList;

	static {
		JCLocaleManager.getDefault().add("com.klg.jclass.chart3d.customizer.util.resources.LocaleInfo");
	}

	public JCColorChooser(Color color) {
		super();
		if (color == null) {
			color = Color.white;
		}
		this.color = color;
		initialize();
	}

	public JCColorChooser() {
		this(Color.white);
	}

	public void initialize() {

		colornameList = new ArrayList();

		Object[] colorList = JCColorList.getColornames();

		for (int i=0; i<colorList.length; i++) {
			colornameList.add(colorList[i]);
		}

		listeners = new EventListenerList();
		JPanel panel;

		GridBagLayout layout;
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.anchor = GridBagConstraints.WEST;
		constraints.gridwidth = GridBagConstraints.REMAINDER;


		JPanel colorPanel = new JPanel();

		colorPickButton = new JButton(new ImageIcon(rainbowImage()));
		colorPickButton.addActionListener(this);

		colorRGB = new JTextField(9);
		colorRGB.addActionListener(this);
		colorRGB.addFocusListener(this);

		DefaultComboBoxModel model = new DefaultComboBoxModel(JCColorList.getColors());
		model.addElement(lm.getString(LocaleBundle.STRING_UNDEFINED));
		colorName = new JComboBox(model);
		colorName.addActionListener(this);
		colorName.setRenderer(new CustomCellRenderer());

		layout = new GridBagLayout();
		colorPanel.setLayout(layout);

		panel = new JPanel();
		panel.add(new JLabel(lm.getString(LocaleBundle.STRING_RGB)));
		panel.add(new JPanel());
		panel.add(new JPanel());
		panel.add(colorRGB);
		panel.add(new JPanel());
		panel.add(colorPickButton);
		layout.setConstraints(panel,constraints);
		colorPanel.add(panel);

		panel = new JPanel();
		panel.add(new JLabel(lm.getString(LocaleBundle.STRING_NAME) + ":"));
		panel.add(new JPanel());
		panel.add(colorName);
		layout.setConstraints(panel,constraints);
		colorPanel.add(panel);

		// *** Main panel
		//setLayout(new GridLayout(2,1));
		add(colorPanel);

		// *** Display initialization
		colorRGB.setText("#" + JCColorList.getHexRGBFromColor(color));

		setColorname(color);
	}

	private void setColorname(Color color) {
		if (JCColorList.isColorInList(color)) {
			colorName.setSelectedItem(color);
		}
		else {
			colorName.setSelectedItem(lm.getString(LocaleBundle.STRING_UNDEFINED));
			colorName.repaint();
		}
	}

	public void setEnabled(boolean b) {
		super.setEnabled(b);

		colorRGB.setEnabled(b);
		colorName.setEnabled(b);
		colorPickButton.setEnabled(b);
	}

	public void actionPerformed(ActionEvent e) {
		changePerformed(e.getSource());
	}

	public void focusLost(FocusEvent e) {
		changePerformed(e.getSource());
	}

	public void focusGained(FocusEvent e) {	}

	public void changePerformed(Object source) {
		Color oldColor = color;

		if (source == colorRGB) {
			try {
				color = JCColorList.getColorFromHexRGB(colorRGB.getText());
				setColorname(color);
			} catch (NumberFormatException nfe) {
				colorRGB.setText("#" + JCColorList.getHexRGBFromColor(color));

				JOptionPane.showMessageDialog(this,
											  lm.getString(LocaleBundle.ERROR_ILLEGAL_COLOR),
											  lm.getString(LocaleBundle.STRING_ERROR),
											  JOptionPane.ERROR_MESSAGE);

				colorRGB.setText("#" + JCColorList.getHexRGBFromColor(color));
			}
		}
		else if (source == colorName) {
			Object selection = colorName.getSelectedItem();

			if (selection instanceof Color) {
				color = (Color) selection;
				colorRGB.setText("#" + JCColorList.getHexRGBFromColor(color));
			}
			else if (selection instanceof String) {
				setColorname(color);
			}
		}
		else if (source == colorPickButton) {
			JColorChooser colorChooser = new JColorChooser(color);
			Color chosenColor = colorChooser.showDialog(this,lm.getString(LocaleBundle.STRING_CHOOSE_A_COLOR),color);
			if (chosenColor != null) {
				color = chosenColor;
			}
		}

		if (!color.equals(oldColor)) {
			colorRGB.setText("#" + JCColorList.getHexRGBFromColor(color));
			setColorname(color);

			fireAction();
		}
	}

	public void setColor(Color color) {
		if (color == null) {
			return;
		}
		this.color = color;
		colorRGB.setText("#" + JCColorList.getHexRGBFromColor(color));
		setColorname(color);
	}

	public Color getColor() {
		return color;
	}

	public void addColor(String colorname,Color color) throws NullPointerException {
		if (colorname == null) {
			throw new NullPointerException("Color name cannot be null");
		}
		else if (color == null) {
			throw new NullPointerException("Color cannot be null");
		}

		DefaultComboBoxModel model = (DefaultComboBoxModel) colorName.getModel();
		model.addElement(color);
		colornameList.add(colorname);
	}

	public void addActionListener (ActionListener listener) {
		listeners.add(listener.getClass(),listener);
	}

	public void removeActionListener(ActionListener listener) {
		listeners.remove(listener.getClass(),listener);
	}

	protected void fireAction() {
		ActionEvent event = new ActionEvent(this,0,null);

		Object[] listenerList = listeners.getListenerList();

		for (int i = listenerList.length - 2; i>=0; i-=2) {
			if (listenerList[i+1] instanceof ActionListener) {;
				((ActionListener)listenerList[i+1]).actionPerformed(event);
			}
		}
	}

	private static String[] rainbow = {
	    "rrrr gggg bbbb",
	    "rrrr gggg bbbb",
	    "rrrr gggg bbbb",
	    "rrrr gggg bbbb",
	    "rrrr gggg bbbb",
	    "rrrr gggg bbbb",
	    "rrrr gggg bbbb",
	    "rrrr gggg bbbb",
	    "              ",
	    "              ",
	    "              ",
	    "  ..  ..  ..  ",
	    "  ..  ..  ..  "
	};

	private static Image rainbowImage() {
	    JCIconCreator ic = new JCIconCreator(
						   rainbow[0].length(),
						   rainbow.length);
	    ic.setColor('.', new Color(0x000000));
	    ic.setColor('r', new Color(0xff0000));
	    ic.setColor('g', new Color(0x00ff00));
	    ic.setColor('b', new Color(0x0000ff));
	    ic.setPixels(rainbow);
	    return ic.getIcon().getImage();
	}

	private class CustomCellRenderer extends JLabel implements ListCellRenderer,Icon {
		int iconWidth = 12;
		int iconHeight = 12;
		Color color;

		public CustomCellRenderer() {
			super();

			setOpaque(true);
			setIcon(this);
			setIconTextGap(8);
			setBorder(new EmptyBorder(1,1,1,1));
		}

		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {

			if (value instanceof String) {
				setText((String) value);
				color = JCColorChooser.this.color;
			}
			else if (value instanceof Color) {
				color = (Color) value;

				if (index < 0) {
					DefaultComboBoxModel model = (DefaultComboBoxModel) list.getModel();
					index = model.getIndexOf(value);
				}

				setText((String) colornameList.get(index));
			}
			else {
				setText(value.toString());
			}

			setFont(list.getFont());

			if (isSelected) {
	            setBackground(list.getSelectionBackground());
	            setForeground(list.getSelectionForeground());
			}
	        else {
	            setBackground(list.getBackground());
	            setForeground(list.getForeground());
			}

			return this;
		}

		public void paintIcon(Component c,Graphics g,int x,int y) {
			g.setColor(color);
			g.fill3DRect(x,y,iconWidth,iconHeight,true);
		}

		public int getIconWidth() {
			return iconWidth;
		}

		public int getIconHeight() {
			return iconHeight;
		}
	}
}
