/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Base3dDataSource.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:02 $  $Locker:  $  Quest Software Inc.


package com.klg.jclass.chart3d.data;

import com.klg.jclass.chart3d.*;
import com.klg.jclass.chart3d.event.Chart3dDataEvent;
import com.klg.jclass.chart3d.event.Chart3dDataSupport;

/** 
 * Empty base class which implements the <code>DataModel</code>'s common
 * ancestor. Do not attempt to instantiate.
 * 
 */
public abstract class Base3dDataSource extends Chart3dDataSupport
									   implements Chart3dDataModel,
												  HoleValueChart3dDataModel {

/** 
 * Hole Value  property for positions that are not to be plotted.
 * Defaults to <code>HoleValueChart3dDataModel.DEFAULT_HOLE_VALUE</code>.
 * 
 */
protected double holeValue = HoleValueChart3dDataModel.DEFAULT_HOLE_VALUE;

/** 
 * The default constructor for a <code>Chart3dDataSupport</code> object.
 * 
 */
public Base3dDataSource()
{
	super();
}

/** 
 * Constructor used to create a <code>Base3dDataSource</code> object with a given data
 * source.
 * 
 * @param dataSrc the data source
 */
public Base3dDataSource(Object dataSrc)
{
	super(dataSrc);
}

/** 
 * Returns the hole value for the chart 3D data.  The hole value is
 * a floating point number used to represent a hole in the data.
 * Internally, this value is placed in the arrays to represent a missing
 * data value.  If the hole value is not explicitly set, returns
 * <code>HoleValueChart3dDataModel.DEFAULT_HOLE_VALUE</code> by default.
 * 
 * @return floating-point value used as a placeholder for missing values
 */
public double getHoleValue()
{
	return(this.holeValue);
}//getHoleValue

/** 
 * Sets the hole value for the chart 3D data.  The hole value is
 * a floating point number used to represent a hole in the data.
 * Internally, this value is placed in the arrays to represent a missing
 * data value.
 * The hole value should not be set to <code>Double.NAN</code>.
 * 
 * <P>If the hole value is changed, previously designated hole values
 * will be drawn at their actual set value (that is, the old hole value).
 * If this old hole value is the default hole value as defined by
 * <code>HoleValueChart3dDataModel.DEFAULT_HOLE_VALUE</code>, chart 3D will experience
 * severe drawing slowness due to trying to scale its values to such a
 * great range. If you are using this method to change hole values,
 * it is suggested that you first change the previous hole values inside
 * the data to something realistic or to the new hole value you are about
 * to set.
 * 
 * @param holeValue floating-point value used as a placeholder for missing 
 *     values
 */
public void setHoleValue(double holeValue)
{
	this.holeValue      = holeValue;
	int type            = Chart3dDataEvent.RELOAD_HOLE_VALUE;
	JCData3dIndex index = new JCData3dIndex();
	fireChart3dDataEvent(new Chart3dDataEvent(this, type, index));
}//setHoleValue

}
