/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Base3dGridDataSource.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:02 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.data;

import com.klg.jclass.chart3d.*;
import com.klg.jclass.chart3d.event.*;
import com.klg.jclass.util.JCListenerList;

import java.io.Serializable;
import java.util.Enumeration;



/** 
 * This class is the base for any data source that chooses to
 * store data internally using gridmodel arrays.
 * 
 */
public class Base3dGridDataSource extends Base3dDataSource
			                      implements Chart3dGridDataModel,
		                                     LabelledChart3dGridDataModel,
											 Cloneable,
											 Serializable {

/** Array of doubles for <i>X</i> values (columns).
 * 
 */
protected double[] xGrid = null;

/** Array of doubles for <i>Y</i> values (rows).
 * 
 */
protected double[] yGrid = null;

/** Two-dimensional array of doubles for <i>Z</i> values (data).
 * 
 */
protected double[][] zValues = null;

/** Name of this datasource.
 * 
 */
protected String dataSourceName = null;

/** List of interested listeners.
 * 
 */
protected JCListenerList chartDataListeners;

/** The <i>X</i> axis labels.
 * 
 */
protected String[] xLabels;

/** The <i>Y</i> axis labels.
 * 
 */
protected String[] yLabels;

/** 
 * Constructor for creating a new <code>Base3dGridDataSource</code> when the data is already known,
 * includes a <code>String</code> for the name
 * .
 * 
 * @param name Name String of this datasource (optional, can be null)
 * @param xGrid Array of doubles for <i>X</i> values (columns)
 * @param yGrid Array of doubles for <i>Y</i> values (rows)
 * @param zValues Two-dimensional array of doubles for <i>Z</i> values (data)
 */
public Base3dGridDataSource(String name, double[] xGrid, double[] yGrid,
							double[][] zValues)
{
	super();
	if (name != null) {
		this.dataSourceName = name;
	}
	this.xGrid   = xGrid;
	this.yGrid   = yGrid;
	this.zValues = zValues;
	int type = Chart3dDataEvent.RESET;
	JCData3dGridIndex index = new JCData3dGridIndex(JCData3dGridIndex.ALL,
											        JCData3dGridIndex.ALL);
	fireChart3dDataEvent(new Chart3dGridDataEvent(this, type, index));
}//with data and name constructor

/** 
 * Sets the <code>datasource name</code> property
 * .
 * 
 * @param name String name of this datasource
 */
public void setDataSourceName(String name)
{
	this.dataSourceName = name;
	int type = Chart3dDataEvent.RELOAD_DATA_SOURCE_NAME;
	JCData3dGridIndex index = new JCData3dGridIndex(JCData3dGridIndex.ALL,
											        JCData3dGridIndex.ALL);
	fireChart3dDataEvent(new Chart3dGridDataEvent(this, type, index));
}//setName


/** 
 * Returns the <code>datasource name</code> property
 * .
 * 
 * @return name String name of this datsource
 */
public String getDataSourceName()
{
	return(dataSourceName);
}//getName

/** 
 * Sets the <i>x</i>Grid values
 * .
 * 
 * @param xGrid Array of doubles
 */
public void setXGrid(double[] xGrid)
{
	this.xGrid = xGrid;
	int type = Chart3dGridDataEvent.RELOAD_XGRID;
	JCData3dGridIndex index = new JCData3dGridIndex(JCData3dGridIndex.ALL,
											        JCData3dGridIndex.ALL);
	fireChart3dDataEvent(new Chart3dGridDataEvent(this, type, index));
}//setXGrid

/** 
 * Returns the <i>x</i>Grid values  If the values don't
 * exist, but the <i>z</i>Values do, a default array of values
 * with an increment of 1.0 will be generated. If
 * neither the <i>z</i>Values, nor <i>x</i>Grid has been set,
 * this method will return null.
 * 
 * @return <i>x</i>Grid Array of doubles
 */
public double[] getXGrid()
{
	//if the values don't exist but the z values do
	if (xGrid == null && zValues != null){
		//make them
		xGrid = new double[zValues.length];
		for (int i = 0; i < xGrid.length; i++){
			xGrid[i] = (double)i;
		}//for
	}//if xGrid

	return(xGrid);
}//getXGrid

/** 
 * Sets the <i>y</i>Grid values
 * .
 * 
 * @param yGrid Array of doubles
 */
public void setYGrid(double[] yGrid)
{
	this.yGrid = yGrid;
	int type = Chart3dGridDataEvent.RELOAD_YGRID;
	JCData3dGridIndex index = new JCData3dGridIndex(JCData3dGridIndex.ALL,
											        JCData3dGridIndex.ALL);
	fireChart3dDataEvent(new Chart3dGridDataEvent(this, type, index));
}//setYGrid

/** 
 * Returns the <i>y</i>Grid values.   If the values don't
 * exist, but the <i>z</i>Values do, a default array of values
 * with an increment of 1.0 will be generated. If
 * neither the <i>z</i>Values, nor <i>y</i>Grid has been set,
 * this method will return null.
 * 
 * @return <i>y</i>Grid Array of doubles
 */
public double[] getYGrid()
{
	//if the y values don't exist, but the z values do.
	if (yGrid == null && zValues != null){
		int max = 0;
		for (int i = 0; i < zValues.length; i++) {
			if (zValues[i].length > max) {
				max = zValues[i].length;
			}//if
		}//for
		yGrid = new double[max];
		for (int i = 0; i < max; i++) {
			yGrid[i] = (double)i;
		}//for
	}//if

	return(yGrid);
}//getYGrid

/** 
 * Sets the <i>Z</i>Values (data).
 * 
 * @param zValues Two-dimensional array of doubles
 */
public void setZValues(double[][] zValues)
{
	this.zValues = zValues;
	int type = Chart3dGridDataEvent.RELOAD_ZALL;
	JCData3dGridIndex index = new JCData3dGridIndex(JCData3dGridIndex.ALL,
											        JCData3dGridIndex.ALL);
	fireChart3dDataEvent(new Chart3dGridDataEvent(this, type, index));
}//setZValues

/** 
 * Returns the <i>Z</i>Values (data).
 * 
 * @return <i>z</i>Values Two-dimensional array of doubles
 */
public double[][] getZValues()
{
	return(zValues);
}//getZValues

/** 
 * Retrieves the number of <i>X</i> grid values (which is the same as the number of
 * z arrays) this datasource holds.
 * 
 * @return The number of <i>X</i> grid values (or <i>z</i> value arrays).
 */
public int getNumX()
{
	if (zValues == null) {
		return(0);
	}
	return(zValues.length);
}//getNumX

/** 
 * Retrieves the number of <i>Y</i> grid values.  This is the minimum length of all
 * the <i>z</i> value arrays.
 * 
 * @return The number of <i>Y</i> grid values
 */
public int getNumY()
{
	if (zValues == null || zValues.length == 0) {
		return(0);
	}
    int min = Integer.MAX_VALUE;
    for (int i = 0; i < zValues.length; i++) {
        if (zValues[i] == null) {
            return(0);
        }
        if (zValues[i].length < min) {
            min = zValues[i].length;
        }
    }
	return(min);
}//getNumY

/** 
 * Retrieves the <i>x</i>Labels
 * .
 * 
 * @return <i>x</i>Labels a String array containing the <i>x</i> labels
 */
public String[] getXLabels()
{
	return(this.xLabels);
}//getXLabels

/** 
 * Sets the <i>x</i>Labels
 * .
 * 
 * @param xLabels a String array
 */
public void setXLabels(String[] xLabels)
{
	this.xLabels = xLabels;
	int type = Chart3dGridDataEvent.RELOAD_ALL_XLABELS;
	JCData3dGridIndex index = new JCData3dGridIndex(JCData3dGridIndex.ALL,
											        JCData3dGridIndex.ALL);
	fireChart3dDataEvent(new Chart3dGridDataEvent(this, type, index));
}//setXLabels

/** 
 * Retrieves the yLabels
 * 
 * @return <i>y</i>Labels a String array containing the <i>y</i> labels
 */
public String[] getYLabels()
{
	return(this.yLabels);
}//getYLabels

/** 
 * Sets the <i>y</i>Labels.
 * 
 * @param yLabels a String array
 */
public void setYLabels(String[] yLabels)
{
	this.yLabels = yLabels;
	int type = Chart3dGridDataEvent.RELOAD_ALL_YLABELS;
	JCData3dGridIndex index = new JCData3dGridIndex(JCData3dGridIndex.ALL,
											        JCData3dGridIndex.ALL);
	fireChart3dDataEvent(new Chart3dGridDataEvent(this, type, index));
}//setYLabels

/** 
 * Overrides clone in class Object. This clone method retuns a deep copy of
 * all <code>Base3dGridDataSource</code> fields, except for the <code>JCListenerList</code>, which
 * is set to null.
 * 
 * Overrides @see Object#clone().
 * 
 * @throws java.lang.OutOfMemoryError - if there is not enough memory.
 * @return a clone of this instance.
 */
public Object clone()
{
	Base3dGridDataSource clone = null;

	try {
		clone = (Base3dGridDataSource)super.clone();

		// Make a deep copy of the xGrid values.
		if (xGrid != null) {
			clone.xGrid = (double[])xGrid.clone();
		}

		// Do the same for the yGrid values.
		if (yGrid != null) {
			clone.yGrid = (double[])yGrid.clone();
		}

		// Make a deep copy of the z values.
		if (zValues != null) {
			clone.zValues = (double[][])zValues.clone();
			for (int each = 0; each < zValues.length; each++) {
				if (zValues[each] != null) {
					clone.zValues[each] = (double[])zValues[each].clone();
				}
			}
		}

		// Make a deep copy of the x labels.
		if (xLabels != null) {
			clone.xLabels = (String[])xLabels.clone();
			// String objects are immutable; no need to clone each label.
		}

		// Do the same for the y labels.
		if (yLabels != null) {
			clone.yLabels = (String[])yLabels.clone();
			// String objects are immutable; no need to clone each label.
		}

		// The clone should not share the same chart data listeners of
		// it's prototype, unless ofcourse this is desired. So, by default,
		// set the list of listeners to null.
		clone.chartDataListeners = null;

	}
	catch (CloneNotSupportedException e) {
		// This should never happen.
		throw new RuntimeException("Base3dGridDataSource does not support cloneable.");
	}

	return clone;
}//clone

}//Base3dGridData
