/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCDefault3dGridDataSource.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:02 $  $Locker:  $  Quest Software Inc.
package com.klg.jclass.chart3d.data;


import com.klg.jclass.chart3d.*;

import com.klg.jclass.chart3d.event.*;

/** 
 * <code>JCDefault3dGridDataSource</code> extends <code>Base3dGridDataSource</code> to create
 * a more useful default container for JClass Chart 3D data.
 * 
 */
public class JCDefault3dGridDataSource extends Base3dGridDataSource {

/** 
 * Convenience no argument constructor sets all properties initially to null.
 * 
 */
public JCDefault3dGridDataSource()
{
	super(null, null, null, null);
}//no argument constructor

/** 
 * Regular constructor sets the data values and the name of this datasource.
 * 
 * @param name String name of the datasource
 * @param xGrid array of doubles which holds the <i>X</i> values
 * @param yGrid array of doubles which holds the <i>Y</i> values
 * @param zValues two-dimensional array of doubles holds the <i>Z</i> values.
 */
public JCDefault3dGridDataSource(String name, double[] xGrid, double[] yGrid,
								 double[][] zValues)
{
	super(name, xGrid, yGrid, zValues);
}//constructor with params

/** 
 * Retrieves the specified series as an array of doubles.
 * This method will throw <code>ArrayIndexOutOfBoundsException</code> if the index
 * is not valid.
 * 
 * @param index int which indicates which series to return
 * @return the specified series
 */
public double[] getZArray(int index)
{
	return(zValues[index]);
}//getSeries

/** 
 * Sets the values for the specified series index.
 * This method will throw <code>ArrayIndexOutOfBoundsException</code> if the index
 * is not valid.
 * 
 * @param xIndex 
 * @param values the new series
 */
public void setZArray(int xIndex, double[] values)
{
	zValues[xIndex] = values;
	int type = Chart3dGridDataEvent.RELOAD_ZARRAY;
	JCData3dGridIndex index = new JCData3dGridIndex(xIndex,
													JCData3dGridIndex.ALL);
	fireChart3dDataEvent(new Chart3dGridDataEvent(this, type, index));
}//setSeries


/** 
 * Retrieves the number of <i>X</i> values.
 * 
 * @return the int number of <i>x</i> values
 */
public int getXGridSize()
{
	if (xGrid == null) {
		return 0;
	}
	return(xGrid.length);
}//getXGridSize

/** 
 * Retrieves the number of <i>Y</i> values.
 * 
 * @return the int number of <i>y</i> values
 */
public int getYGridSize()
{
	if (yGrid == null) {
		return 0;
	}
	return(yGrid.length);
}//getYGridSize

}//JCDefault3dGridDataSource
