/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCDefault3dPointDataSource.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:02 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.data;

import com.klg.jclass.chart3d.*;

import javax.vecmath.Point3d;

import com.klg.jclass.chart3d.event.*;

/** 
 * <code>JCDefault3dPointDataSource</code> extends <code>Base3dPointDataSource</code> to create
 * a more useful default container for JClass Chart 3D data.
 * 
 */
public class JCDefault3dPointDataSource extends Base3dPointDataSource {

/** 
 * Convenience no argument constructor sets all properties initially to null.
 * 
 */
public JCDefault3dPointDataSource()
{
	super(null, null);
}//no argument constructor


/** 
 * Regular constructor sets the data values and the name of this datasource.
 * 
 * @param name String name of the datasource
 * @param points array of <code>Point3d</code> objects to be plotted on the
 *     chart
 */
public JCDefault3dPointDataSource(String name, Point3d[][] points)
{
	super(name, points);
}//constructor with params

/** 
 * Retrieves the specified series as an array of <code>Point3d</code> objects.
 * This method will throw <code>ArrayIndexOutOfBoundsException</code> if the index
 * is not valid.
 * 
 * @param index int which indicates which series to return
 * @return the specified series
 */
public Point3d[] getSeries(int index)
{
	return points[index];
}//getSeries

/** 
 * Sets the values for the specified series index.
 * This method will throw <code>ArrayIndexOutOfBoundsException</code> if the index
 * is not valid.
 * 
 * @param seriesIndex 
 * @param values the new series
 */
public void setSeries(int seriesIndex, Point3d[] values)
{
	points[seriesIndex] = values;
	int type = Chart3dPointDataEvent.RELOAD_SERIES;
	JCData3dPointIndex index = new JCData3dPointIndex(seriesIndex,
													  JCData3dPointIndex.ALL);
	fireChart3dDataEvent(new Chart3dPointDataEvent(this, type, index));
}//setSeries

}//JCDefault3dPointDataSource
