/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCEditable3dGridDataSource.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:02 $  $Locker:  $  Quest Software Inc.
package com.klg.jclass.chart3d.data;

import com.klg.jclass.chart3d.*;
import com.klg.jclass.chart3d.event.*;

/**
 * Extension to <code>JCDefault3dGridDataSource</code> with convenience methods
 * that permit editing of data.
 */
public class JCEditable3dGridDataSource extends JCDefault3dGridDataSource
			                            implements EditableChart3dDataModel {

/**
 * No argument constructor which sets all params to null.
 */
public JCEditable3dGridDataSource()
{
	super(null, null, null, null);
} //no argument constructor

/**
 * Constructor that creates a populated <code>JCEditable3dGridDataSource</code>.
 *
 * @param name String name of the datasource
 * @param xGrid array of doubles which holds the <i>X</i> values
 * @param yGrid array of doubles which holds the <i>Y</i> values
 * @param zValues two-dimensional array of doubles holds the <i>Z</i> values
 */
public JCEditable3dGridDataSource(String name, double[] xGrid, double[] yGrid,
								  double[][] zValues)
{
	super(name, xGrid, yGrid, zValues);
} //Constructor for creating a populated datasource

/**
 * Retrieves the specified point value, given series and point indices.
 *
 * @param xIndex int indicating which series contains the point
 * @param yIndex int indicating where in the series the point is
 * @return the value of that point as a double
 */
public double getZValue(int xIndex, int yIndex)
{
	if (xIndex < 0               ||
	    xIndex >= zValues.length ||
		yIndex < 0               ||
		yIndex >= zValues[xIndex].length)
	{
		return getHoleValue();
	}
	return(zValues[xIndex][yIndex]);
}//getZValue


/**
 * Retrieves the <code>zValue</code> based on the <code>JCData3dIndex</code>
 * object.
 *
 * @param   index  a <code>JCData3dGridIndex</code> object
 * @return  the double value at that location
 */
public double getZValue(JCData3dIndex index) {
	if (index == null || !(index instanceof JCData3dGridIndex)) {
		return getHoleValue();
	}
	JCData3dGridIndex gridIndex = (JCData3dGridIndex)index;

	int xIndex = gridIndex.getX();
	int yIndex = gridIndex.getY();
	return getZValue(xIndex, yIndex);
}//getZValue
	

/**
 * Sets the point for a given data index which must be of type
 * <code>JCData3dGridIndex</code>.
 *
 * @param index the data index from which the <i>x</i> and <i>y</i> indices of the point
 *   to be edited is obtained
 * @param newValue the new <i>z</i> value for the point
 * @return whether the edit succeeded
 */
public boolean setZValue(JCData3dIndex index, double newValue)
{
    if (index == null || !(index instanceof JCData3dGridIndex)) {
        return(false);
    }
    JCData3dGridIndex gridIndex = (JCData3dGridIndex)index;
    int xIndex = gridIndex.getX();
    int yIndex = gridIndex.getY();
    return setZValue( xIndex, yIndex, newValue);
}


/**
 * Sets the value for a location given the <i>x</i> and <i>y</i> indices.
 *
 * @param   xIndex  the <i>X</i> location
 * @param   yIndex  the <i>Y</i> location
 * @param   newValue  the new <i>z</i> value for the point
 * @return   whether the edit succeeded
 */
public boolean setZValue(int xIndex, int yIndex, double newValue) {
	if (xIndex < 0               ||
	    xIndex >= zValues.length ||
		yIndex < 0               ||
		yIndex >= zValues[xIndex].length)
	{
		return(false);
	}
	zValues[xIndex][yIndex] = newValue;
	int type = Chart3dGridDataEvent.RELOAD_ZVALUE;
	fireChart3dDataEvent(new Chart3dGridDataEvent(this, type, new JCData3dGridIndex(xIndex, yIndex)));
	return(true);
} //setZValue

}
