/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCEditable3dPointDataSource.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:02 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.data;


import com.klg.jclass.chart3d.*;
import com.klg.jclass.chart3d.event.*;

import javax.vecmath.Point3d;

/**
 * Extension to <code>JCDefault3dPointDataSource</code> with convenience
 * methods that permit editing of data.
 */
public class JCEditable3dPointDataSource extends JCDefault3dPointDataSource
			                             implements EditableChart3dDataModel {

/**
 * No argument constructor which sets all params to null.
 */
public JCEditable3dPointDataSource()
{
	super(null, null);
}//no argument constructor

/**
 * Regular constructor sets the data values and the name of this datasource.
 *
 * @param name String name of the datasource
 * @param points array of <code>Point3d</code> objects to be plotted on the
 *     chart
 */
public JCEditable3dPointDataSource(String name, Point3d[][] points)
{
	super(name, points);
}

/**
 * Retrieves the specified point object, given series and point indices.
 *
 * @param seriesIndex int indicating which series contains the point
 * @param pointIndex int indicating where in the series the point is
 * @return the <code>Point3d</code> object at that location
 */
public Point3d getPoint(int seriesIndex, int pointIndex)
{
	if (seriesIndex < 0              ||
	    seriesIndex >= points.length ||
		pointIndex < 0               ||
		pointIndex >= points[seriesIndex].length)
	{
		return(null);
	}
	return(points[seriesIndex][pointIndex]);
}//getPoint

/**
 * Sets the point for a given series and point index.
 *
 * @param seriesIndex int indicating which series contains the point
 * @param pointIndex int indicating where in the series the point is
 * @param point the new <code>Point3d</code> object
 * @return whether the edit succeeded
 */
public boolean setPoint(int seriesIndex, int pointIndex, Point3d point)
{
	if (seriesIndex < 0              ||
	    seriesIndex >= points.length ||
		pointIndex < 0               ||
		pointIndex >= points[seriesIndex].length)
	{
		return(false);
	}
	points[seriesIndex][pointIndex] = point;
	int type = Chart3dPointDataEvent.RELOAD_POINT;
	JCData3dPointIndex index = new JCData3dPointIndex(seriesIndex, pointIndex);
	fireChart3dDataEvent(new Chart3dPointDataEvent(this, type, index));
	return(true);
} //setPoint


/**
 * Retrieves the <code>zValue</code> from the specified
 * <code>JCData3dPointIndex</code> object.
 *
 * @param   index  the <code>JCData3dPointIndex</code>
 * @return  the <code>Point3d</code> object at that location
 */
public double getZValue(JCData3dIndex index) {
	if (index == null || !(index instanceof JCData3dPointIndex)) {
        return getHoleValue();
    }
    JCData3dPointIndex pointDataIndex = (JCData3dPointIndex)index;
    int seriesIndex = pointDataIndex.getSeries();
    int pointIndex  = pointDataIndex.getPoint();
	if (seriesIndex < 0              ||
	    seriesIndex >= points.length ||
		pointIndex < 0               ||
		pointIndex >= points[seriesIndex].length)
	{
		return getHoleValue();
	}
	Point3d point = points[seriesIndex][pointIndex];
	return point.z;
}//getZValue


/**
 * Sets the <code>zValue</code> of a point for a given series and point index.  This
 * series and point index is retrieved from the passed in data index,
 * which must be of type <code>JCData3dPointIndex</code>.
 *
 * @param index the data index from which the series and point indices of the
 *   point to be edited is obtained
 * @param newValue <code>Point3d</code> object for that position
 * @return whether the edit succeeded
 */
public boolean setZValue(JCData3dIndex index, double newValue)
{
    if (index == null || !(index instanceof JCData3dPointIndex)) {
        return(false);
    }
    JCData3dPointIndex pointDataIndex = (JCData3dPointIndex)index;
    int seriesIndex = pointDataIndex.getSeries();
    int pointIndex  = pointDataIndex.getPoint();
	if (seriesIndex < 0              ||
	    seriesIndex >= points.length ||
		pointIndex < 0               ||
		pointIndex >= points[seriesIndex].length)
	{
		return(false);
	}
	Point3d point = points[seriesIndex][pointIndex];
    point.z = newValue;
	int type = Chart3dPointDataEvent.RELOAD_POINT;
	fireChart3dDataEvent(new Chart3dPointDataEvent(this, type, pointDataIndex));
	return(true);
} //setDataValue

}
