/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCFile3dDataSource.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:02 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.data;

import java.io.*;
import java.util.*;
import javax.vecmath.Point3d;

import com.klg.jclass.chart3d.*;

/**
 * This is a convenience class which parses data from a file. Do not create
 * an instance of this class.  Instead, use the static method:
 * <code>JCFile3dDataSource.createDataSourceFromFile(String fileName)</code>.
 * The file can either contain an irregular or regular grid or point data.
 * A regular grid file has the following format:<br>
 * # Comments use the # sign<br>
 * # For this example the grid has 5 by 3 points<br>
 * # The Grid increase in <br>
 * # X steps of 1.0 and Y steps of 2.0<br>
 * # The Origin of the Grid is as x = -20.0 and y = 50.0<br>
 * GRID <br>
 * 5 'A A' 'B' 'C' 'D' 'E' <br>
 * <br>
 * 3 'Y1' 'Y2' Y3'<br>
 * <br>
 * 100.0 1.0 2.0 -20.0 50.0<br>
 * <br>
 * # 15 data points would follow, 1 for each point<br>
 * 49.875000 43.765625 38.50000 33.984375 30.12400<br>
 * 26.828125 24.0000 21.656875 19.375000 17.39062 16.222 18.444 23.555<br>
 * 58.664 37.894564<br>
 * <br>
 * <br>
 * An Irregular Grid would supply all the <i>x</i> and <i>y</i> values as well
 * as the datapoints.<br>
 * An example irregular grid file:<br>
 * # Irregular grid has 5 by 3 points<br>
 * # Holes have value 100.00<br>
 * # 5 x values are give<br>
 * # 3 y values are given<br>
 * IGRID <br>
 * 5  'A A' 'B' 'C' 'D' 'EE'<br>
 * <br>
 * 3  'Y1' 'Yahoo' 'Y3'<br>
 * <br>
 * 100.0<br>
 * <br>
 * 20 21.1 22.3 23 24.4 <br>
 * 50.3 51.3 52.6 <br>
 * <br>
 * # 15 Data values follow<br>
 * 23.34343 12.89239 11.99423 15.781212 18.18989<br>
 * 26.828125 24.0000 21.656875 19.375000 17.39062 16.222 18.444 23.555<br>
 * 58.664 37.894564<br>
 * <br>
 * <br>
 * Point data would look like this:<br>
 * <br>
 * POINT
 * 3 'Series 1' 'Series 2' 'Series 3'<br>
 * -1000<br>
 * <br>
 * 5<br>
 * 5.65    6.24    1.78<br>
 * 7.41    7.26    4.21<br>
 * 5.45    5.44    1.43<br>
 * 0.97    9.66    3.41<br>
 * 3.86    1.42    0.20<br>
 * <br>
 * 4<br>
 * 6.57    7.43    8.37<br>
 * 3.79    3.63    2.65<br>
 * 7.89    3.48    5.65<br>
 * 0.78    7.03    0.65<br>
 * <br>
 * 6<br>
 * 9.91    7.54    1.74<br>
 * 6.53    4.62    1.99<br>
 * 8.41    3.49    5.06<br>
 * 7.85    9.16    0.64<br>
 * 6.96    9.18    8.95<br>
 * 3.47    3.19    6.29<br>
 * <br>
 *
 */
public class JCFile3dDataSource  {

/** </code>StreamTokenizer</code> instance used to parse the data file. */
protected static StreamTokenizer streamTokenizer = null;

/**
 * Do not use a constructor on this class.  Use this method instead to parse the
 * file and create an instance of the appropriate datasource.
 *
 * @param fileName
 * @return <code>Base3dDataSource</code> an instance of a datasource that
 *     <code>JCChart3d</code> can use
 */
public static Base3dDataSource createDataSourceFromFile(String fileName)
{
	Base3dDataSource dm = null;
	try {
		FileReader reader = new FileReader(fileName);
		streamTokenizer = new StreamTokenizer(reader);
		dm = parseFile();
    }
	catch (FileNotFoundException ex) {
		ex.printStackTrace();
	}
	catch (SecurityException secex) {
		secex.printStackTrace();
	}
	catch (IOException e) {
		e.printStackTrace();
	}//end catch

	return(dm);
}//createDataSourceFromFile

/**
 * Method does the initial parsing to see if this is a regular or
 * irregular grid datafile, or a point datafile.
 *
 * @exception IOException
 * @return
 */
protected static Base3dDataSource parseFile() throws IOException
{
	// Configure the tokenizer to suit
	Base3dDataSource dm = null;
	streamTokenizer.commentChar('#');
	streamTokenizer.parseNumbers();
	streamTokenizer.eolIsSignificant(false);

	// Cycle through any comments
	while (streamTokenizer.sval == null) {
		streamTokenizer.nextToken();
	}//end while

	String type = streamTokenizer.sval;

	//do the parsing appropriate to the data type
	if (type.equals("GRID")) {
		dm = parseGrid();
	}
	else if (type.equals("IGRID")) {
		dm = parseIGrid();
	}
	else if (type.equals("POINT")) {
		dm = parsePoint();
	}
	else {
		System.out.println("Unknown datatype");
	}//end if

	return(dm);
}//parseFile


/**
 * Parses a data file of type <code>POINT</code>, and returns a suitable
 * datasource.
 *
 * @exception IOException
 * @return <code>JCEditable3dPointDataSource</code> a suitable datasource for
 *     the parsed data
 * @see <code>JCEditable3dPointDataSource</code>
 */
protected  static JCEditable3dPointDataSource parsePoint() throws IOException
{
	// Initialize the points
	double x, y, z;
	while (streamTokenizer.nval == 0.0) {
		streamTokenizer.nextToken();
	}

	int numSeries = (int)streamTokenizer.nval;
	Point3d[][] pts = new Point3d[numSeries][];

	streamTokenizer.nextToken();

	// Load the Series labels if there are any
	String[] serlabs = null;

	if (streamTokenizer.sval != null) {
		serlabs = new String[numSeries];
		int i = 0;
		while(streamTokenizer.sval != null) {
			serlabs[i++] = streamTokenizer.sval;
			streamTokenizer.nextToken();
		}
	}

	//load the data points
	double hole = streamTokenizer.nval;
	streamTokenizer.nextToken();

	for (int i = 0; i < numSeries; i++) {
		int numPoints = (int) streamTokenizer.nval;
		streamTokenizer.nextToken();
		Point3d[] pt = new Point3d[numPoints];
		for (int j = 0; j < numPoints; j++) {
			x = streamTokenizer.nval;
			streamTokenizer.nextToken();
			y = streamTokenizer.nval;
			streamTokenizer.nextToken();
			z = streamTokenizer.nval;
			Point3d p = new Point3d(x, y, z);
			pt[j] = p;
			streamTokenizer.nextToken();
		}
		pts[i] = pt;
	}

	// Create your new datasource
	JCEditable3dPointDataSource ds = new JCEditable3dPointDataSource(null, pts);
	ds.setHoleValue(hole);
	if (serlabs != null) {
		ds.setSeriesLabels(serlabs);
	}
	return(ds);
}

/**
 * Parses the file if it is a regular grid file.
 *
 * @exception IOException
 * @return <code>JCEditable3dGridDataSource</code> a suitable datasource for
 *     the parsed data
 * @see <code>JCEditable3dGridDataSource</code>
 */
protected static JCEditable3dGridDataSource parseGrid() throws IOException
{
	while (streamTokenizer.nval == 0.0) {
		streamTokenizer.nextToken();
	}

	int numx = (int)streamTokenizer.nval;

	streamTokenizer.nextToken();

	// Get the x labels, if there are any.
	String[] xlabs = null;

	if (streamTokenizer.sval != null) {
		xlabs = new String[numx];
		int i = 0;
		while(streamTokenizer.sval != null) {
			xlabs[i++] = streamTokenizer.sval;
			streamTokenizer.nextToken();
		}
	}

	int numy = (int)streamTokenizer.nval;

	// Get the y labels if there are any.
	streamTokenizer.nextToken();
	String[] ylabs = null;

	if (streamTokenizer.sval != null) {
		ylabs = new String[numy];
		int i = 0;
		while(streamTokenizer.sval != null) {
			ylabs[i++] = streamTokenizer.sval;
			streamTokenizer.nextToken();
		}
	}

	// Initialize the info for creating holes and the x and y values
	double hole = streamTokenizer.nval;
	streamTokenizer.nextToken();
	double stepx = streamTokenizer.nval;
	streamTokenizer.nextToken();
	double stepy = streamTokenizer.nval;
	streamTokenizer.nextToken();
	double origx = streamTokenizer.nval;
	streamTokenizer.nextToken();
	double origy = streamTokenizer.nval;
	streamTokenizer.nextToken();

	// Create the x values
	double[] xarray = new double[numx];
	for (int i = 0; i < numx; i++) {
		xarray[i] = origx + (i * stepx);
	}

	// Create the y values
	double[] yarray = new double[numy];
	for (int i = 0; i < numy; i++) {
		yarray[i] = origy + (i * stepy);
	}

	// Load the zValues
	double[][] zval = new double[numx][numy];
	for (int i = 0; i < numx; i++) {
		for (int j = 0; j < numy; j++) {
			zval[i][j]= streamTokenizer.nval;
						streamTokenizer.nextToken();

		}
	}

	// create the datasource
	JCEditable3dGridDataSource ds = new JCEditable3dGridDataSource(null,
															       xarray,
																   yarray,
																   zval);
	ds.setHoleValue(hole);
	if (xlabs != null) {
		ds.setXLabels(xlabs);
	}
	if (ylabs != null) {
		ds.setYLabels(ylabs);
	}
	return(ds);
}

/**
 * Parses an irregular grid file.
 *
 * @exception IOException
 * @return <code>JCEditable3dGridDataSource</code> a suitable datasource for
 *     the parsed data
 * @see <code>JCEditable3dGridDataSource</code>
 */
protected static JCEditable3dGridDataSource parseIGrid() throws IOException
{
	while(streamTokenizer.nval == 0.0) {
		streamTokenizer.nextToken();
	}
	int numx = (int)streamTokenizer.nval;
	streamTokenizer.nextToken();

	// Load the x labels, if there are any
	String[] xlabs = null;

	if (streamTokenizer.sval != null) {
		xlabs = new String[numx];
		int i = 0;
		while(streamTokenizer.sval != null) {
			xlabs[i++] = streamTokenizer.sval;
			streamTokenizer.nextToken();
		}
	}

	int numy = (int)streamTokenizer.nval;
	streamTokenizer.nextToken();

	// Load the y labels, if there are any.
	String[] ylabs = null;

	if (streamTokenizer.sval != null) {
		ylabs = new String[numy];
		int i = 0;
		while(streamTokenizer.sval != null) {
			ylabs[i++] = streamTokenizer.sval;
			streamTokenizer.nextToken();

		}
	}

	double hole = streamTokenizer.nval;
	streamTokenizer.nextToken();

	// Load the x values
	double[] xarray = new double[numx];
	for (int i = 0; i < numx; i++ ){
		xarray[i] = streamTokenizer.nval;
		streamTokenizer.nextToken();

	}

	// Load the y values
	double[] yarray = new double[numy];
	for (int i = 0; i < numy; i++) {
		yarray[i] = streamTokenizer.nval;
		streamTokenizer.nextToken();

	}

	// Load the z values
	double[][] zval = new double[numx][numy];
	for (int i = 0; i < numx; i++) {
		for (int j = 0; j < numy; j++) {
			zval[i][j]= streamTokenizer.nval;
			streamTokenizer.nextToken();

		}
	}

	// Create the datasource
	JCEditable3dGridDataSource ds = new JCEditable3dGridDataSource(null,
													               xarray,
																   yarray,
																   zval);
	ds.setHoleValue(hole);
	if (xlabs != null) {
		ds.setXLabels(xlabs);
	}
	if (ylabs != null) {
		ds.setYLabels(ylabs);
	}

	return(ds);
}//parseIGrid

}//JCFile3dDataSource
