/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCXML3dDataSource.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:02 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.data;

import com.klg.jclass.chart3d.JCData3dGridIndex;
import com.klg.jclass.chart3d.event.Chart3dDataEvent;
import com.klg.jclass.chart3d.event.Chart3dGridDataEvent;
import com.klg.jclass.util.xml.JCXMLDTDResolver;
import com.klg.jclass.util.xml.JCXMLErrorPrinter;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;

/**
 * This class parses data from an XML file.
 * Specification for the xml data can be found in
 * the <code>chart3d.dtd</code> file.
 * <p>
 * An example XML file would have the following appearance:<p>
 * <code>&lt;?xml version="1.0"?&gt; <br>
 * &lt;!DOCTYPE data SYSTEM "chart3d.dtd"&gt; <br> <br>
 * &lt;data hole="-100" name="my data"&gt; <br>
 * &nbsp;&nbsp;&lt;xgrid&gt; <br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;xval&gt;0.78&lt;/xval&gt;<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;xval&gt;1.565&lt;/xval&gt;<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;xval&gt;2.00&lt;/xval&gt;<br>
 * &nbsp;&nbsp;&lt;/xgrid&gt;<br> <br>
 * &nbsp;&nbsp;&lt;ygrid&gt;<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;yval&gt;1.00&lt;/yval&gt;<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;yval&gt;2.0&lt;/yval&gt;<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;yval&gt;3.0&lt;/yval&gt;<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;yval&gt;4.0&lt;/yval&gt;<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;yval&gt;5.0&lt;/yval&gt;<br>
 * &nbsp;&nbsp;	&lt;/ygrid&gt;<br>
 * <br>
 * &nbsp;&nbsp;&lt;zgrid&gt; <br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;zval&gt;15.64&lt;/zval&gt; <br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;zval&gt;23.4546&lt;/zval&gt; <br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;zval&gt;45.4545&lt;/zval&gt; <br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;zval&gt;20.4546&lt;/zval&gt; <br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;zval&gt;14.4545&lt;/zval&gt; <br>
 * &nbsp;&nbsp;&lt;/zgrid&gt; <br>
 * <br>
 * &nbsp;&nbsp;&lt;zgrid&gt; <br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;zval&gt;18.5656&lt;/zval&gt; <br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;zval&gt;23.884&lt;/zval&gt; <br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;zval&gt;35.6454&lt;/zval&gt; <br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;zval&gt;21.47&lt;/zval&gt; <br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;zval&gt;37.45&lt;/zval&gt; <br>
 * &nbsp;&nbsp;&lt;/zgrid&gt; <br>
 * <br>
 * &nbsp;&nbsp;&lt;zgrid&gt; <br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;zval&gt;16.58&lt;/zval&gt; <br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;zval&gt;10.5656&lt;/zval&gt; <br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;zval&gt;17.65&lt;/zval&gt; <br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;zval&gt;19.645&lt;/zval&gt; <br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;zval&gt;34.4561&lt;/zval&gt; <br>
 * &nbsp;&nbsp;&lt;/zgrid&gt; <br>
 * <br>
 * &nbsp;&nbsp;&lt;xlabel&gt;January&lt;/xlabel&gt; <br>
 * &nbsp;&nbsp;&lt;xlabel&gt;February&lt;/xlabel&gt; <br>
 * &nbsp;&nbsp;&lt;xlabel&gt;March&lt;/xlabel&gt; <br>
 * <br>
 * &nbsp;&nbsp;&lt;ylabel&gt;Orcs&lt;/ylabel&gt; <br>
 * &nbsp;&nbsp;&lt;ylabel&gt;Zombies&lt;/ylabel&gt; <br>
 * &nbsp;&nbsp;&lt;ylabel&gt;Skeletons&lt;/ylabel&gt; <br>
 * &nbsp;&nbsp;&lt;ylabel&gt;Vampires&lt;/ylabel&gt; <br>
 * &nbsp;&nbsp;&lt;ylabel&gt;Werewolves&lt;/ylabel&gt; <br>
 * <br>
 * &lt;/data&gt;
 * </code>
 */
public class JCXML3dDataSource extends JCDefault3dGridDataSource  {

/**
 * Static string for hole attribute of top level tag in
 * the chart XML datasource.
 */
static public final String XML_DS_ATT_HOLE = "hole";
/**
 * Static string for datasource name of chart XML datasource.
 */
static public final String XML_DS_ATT_NAME = "name";
/**
 * Static string for x Labels  of chart XML datasource.
 */
static public final String XML_DS_XLABEL = "xlabel";
/**
 * Static string for series label tag of chart XML datasource.
 */
static public final String XML_DS_YLABEL = "ylabel";
/**
 * Static string for <i>x</i> data tag of chart XML datasource.
 */
static public final String XML_DS_XDATA = "xval";
/**
 * Static string for <i>y</i> data tag of chart XML datasource.
 */
static public final String XML_DS_YDATA = "yval";
/**
 * Static string for <i>z</i> grid tag of chart XML datasource.
 */
static public final String XML_DS_ZGRID = "zgrid";
/**
 * Static string for <i>z</i> data tag of chart XML datasource.
 */
static public final String XML_DS_ZVAL = "zval";

/**
 * This constructor takes xml in String form
 * .
 *
 * @param xmlString String containing xml data
 */
public JCXML3dDataSource(String xmlString)
{
	DocumentBuilder docBuilder = getDocBuilder();
	try {
		Document doc = docBuilder.parse(xmlString);
		parseDoc(doc);
	}
	catch (SAXException se) {
		se.printStackTrace();
	}
	catch (IOException ioe) {
		ioe.printStackTrace();
	}
}//Constructor - filename argument

/**
 * This constructor takes an <code>InputStream</code> in XML form.
 *
 * @param inStream an xml InputStream
 */
public JCXML3dDataSource(InputStream inStream)
{
	DocumentBuilder docBuilder = getDocBuilder();
	try {
		Document doc = docBuilder.parse(inStream);
		parseDoc(doc);
	}
	catch (SAXException se) {
		se.printStackTrace();
	}
	catch (IOException ioe) {
		ioe.printStackTrace();
	}
}//Constructor - inputstream argument

/**
 * This constructor takes a <code>Reader</code> which contains xml.
 *
 * @param reader a Reader object
 */
public JCXML3dDataSource(Reader reader)
{
	DocumentBuilder docBuilder = getDocBuilder();
	try {
		Document doc = docBuilder.parse(new InputSource(reader));
		parseDoc(doc);
	}
	catch (SAXException se) {
		se.printStackTrace();
	}
	catch (IOException ioe) {
		ioe.printStackTrace();
	}
}//Constructor - reader argument

/**
 * This constructor takes an <code>InputSource
 * </code>.
 *
 * @param inSource InputSource
 */
public JCXML3dDataSource(InputSource inSource)
{
	DocumentBuilder docBuilder = getDocBuilder();
	try {
		Document doc = docBuilder.parse(inSource.getByteStream());
		parseDoc(doc);
	}
	catch (SAXException se) {
		se.printStackTrace();
	}
	catch (IOException ioe) {
		ioe.printStackTrace();
	}
}//Constructor - input argument

/**
 * This constructor takes an xml file as an
 * argument.
 *
 * @param xmlFile The xml file object
 */
public JCXML3dDataSource(File xmlFile)
{
	DocumentBuilder docBuilder = getDocBuilder();
	try {
		Document doc = docBuilder.parse(xmlFile);
		parseDoc(doc);
	}
	catch (SAXException se) {
		se.printStackTrace();
	}
	catch (IOException ioe) {
		ioe.printStackTrace();
	}
}//Constructor - filename argument

/**
 * Creates a DocumentBuilder object which parses the XML into a Document object,
 * which can then be parsed into a Chart3d dataSource.
 *
 * @return DocumentBuilder object
 */
public DocumentBuilder getDocBuilder()
{
    // Do some preliminary configuration of the document
    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
    dbf.setValidating(true);
    dbf.setIgnoringComments(true);
    dbf.setIgnoringElementContentWhitespace(true);
    dbf.setCoalescing(false);
    DocumentBuilder docBuilder = null;

    try {
        docBuilder = dbf.newDocumentBuilder();
        docBuilder.setErrorHandler(new JCXMLErrorPrinter());
        docBuilder.setEntityResolver(new JCXMLDTDResolver("JCChart3dData.dtd"));
    }
    catch (ParserConfigurationException e) {
        e.printStackTrace();
    }
    return(docBuilder);
}//getDocBuilder


/**
 * This is the workhorse of this class which parses the document
 * .
 * This is protected only in case we need to subclass later.  This
 * method is not meant to be called on its own.
 *
 * @param doc The document object created from the xml file.
 */
protected void parseDoc(Document doc)
{
	doc.normalize();

	// Get the root element.  All good things come from root.
	Element dataElement = doc.getDocumentElement();
	// If we have a hole value, set it.
	if (dataElement.hasAttribute(JCXML3dDataSource.XML_DS_ATT_HOLE)){
        String s = dataElement.getAttribute(JCXML3dDataSource.XML_DS_ATT_HOLE);
        setHoleValue(convertDoubleValue(s));
	}
	// If we have a name, get and set it.
	if (dataElement.hasAttribute(JCXML3dDataSource.XML_DS_ATT_NAME)) {
        String s = dataElement.getAttribute(JCXML3dDataSource.XML_DS_ATT_NAME);
        setDataSourceName(s);
	}

	// If we have xlabels set them
	NodeList xlabs =
        dataElement.getElementsByTagName(JCXML3dDataSource.XML_DS_XLABEL);
	if (xlabs != null) {
		xLabels = parseLabels(xlabs);
	}
	// If we have ylabels set them.
	NodeList ylabs =
        dataElement.getElementsByTagName(JCXML3dDataSource.XML_DS_YLABEL);
	if (ylabs != null) {
		yLabels = parseLabels(ylabs);
	}
	// Get the xGrid values
	NodeList xdat =
        dataElement.getElementsByTagName(JCXML3dDataSource.XML_DS_XDATA);
	setXGrid(parseDoubles(xdat));
	// Get the yGrid values
	NodeList ydat =
        dataElement.getElementsByTagName(JCXML3dDataSource.XML_DS_YDATA);
	setYGrid(parseDoubles(ydat));
	// Get the zValues
	NodeList zgrid =
        dataElement.getElementsByTagName(JCXML3dDataSource.XML_DS_ZGRID);
	zValues = new double[zgrid.getLength()][];
	for (int i = 0; i < zValues.length; i++ ) {
        Element element = (Element)zgrid.item(i);
        NodeList thisGrid =
            element.getElementsByTagName(JCXML3dDataSource.XML_DS_ZVAL);
		zValues[i] = parseDoubles(thisGrid);
	}

	int type = Chart3dDataEvent.RESET;
	JCData3dGridIndex index = new JCData3dGridIndex(JCData3dGridIndex.ALL,
													JCData3dGridIndex.ALL);
	fireChart3dDataEvent(new Chart3dGridDataEvent(this, type, index));
}//parseDoc

/**
 * Convenience method for  pulling <code>double[]</code> out of
 * <code>NodeLists</code>. This method assumes that the <code>NodeList</code>
 * contains bottom-level Elements which only contain doubles.
 *
 * @param dat The <code>NodeList</code> we want returned as an array of
 *     doubles
 * @return the array of doubles we got
 */
protected double[] parseDoubles(NodeList dat)
{
	double[] data = new double[dat.getLength()];
	for (int i = 0; i < data.length; i++) {
        String nodeValue = dat.item(i).getFirstChild().getNodeValue();
        data[i] = convertDoubleValue(nodeValue);
	}
	return(data);
}//parseDoubles

/**
 * Convenience method to convert a String to a double
 * .
 *
 * @param toConvert the String we wish made double
 * @return the double value of the String
 */
protected double convertDoubleValue(String toConvert)
{
	return((new Double(toConvert)).doubleValue());
}//convertDoubleValue

/**
 * Convenience method for  pulling <code>String[]</code> out of
 * <code>NodeLists</code> This method assumes that the <code>NodeList</code>
 * contains bottom-level Elements which contain text.
 *
 * @param labs The <code>NodeList</code> we want returned as an array of
 *     Strings
 * @return the array of Strings we got.
 */
protected String[] parseLabels(NodeList labs)
{
	String[] labels = new String[labs.getLength()];
	for (int i = 0; i < labels.length; i++) {
		labels[i] = labs.item(i).getFirstChild().getNodeValue();
	}
	return(labels);
}//parseXLabels

}//JCXML3dDataSource
