/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JDBC3dDataSource.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:02 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.data;

import java.awt.Point;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.Types;
import java.sql.SQLException;
import java.util.Vector;

import com.klg.jclass.chart3d.*;
import com.klg.jclass.chart3d.event.*;


/** 
 * <code>JDBC3dDataSource</code> extends <code>JCDefault3dGridDataSource</code> to create
 * a data source for use with JDBC.
 * 
 */
public class JDBC3dDataSource extends JCDefault3dGridDataSource {

/** 
 * Default constructor.
 * 
 */
public JDBC3dDataSource()
{
	super();
	int type = Chart3dDataEvent.RESET;
	JCData3dGridIndex index = new JCData3dGridIndex(JCData3dGridIndex.ALL,
											        JCData3dGridIndex.ALL);
	fireChart3dDataEvent(new Chart3dGridDataEvent(this, type, index));
}

/** 
 * The <code>JDBCDataSource</code> constructor takes a <code>ResultSet</code> as source data.
 * Default behavior is to use all numeric columns for <i>Y</i> data and automatically
 * generate <i>x</i> values.
 * 
 * @param jdbcData the input data as a <code>ResultSet</code>
 */
public JDBC3dDataSource(ResultSet jdbcData)
{
    setJDBCData(jdbcData, null, null, null);
	int type = Chart3dDataEvent.RESET;
	JCData3dGridIndex index = new JCData3dGridIndex(JCData3dGridIndex.ALL,
											        JCData3dGridIndex.ALL);
   	fireChart3dDataEvent(new Chart3dGridDataEvent(this, type, index));
}

/** 
 * The <code>JDBCDataSource</code> constructor takes a <code>ResultSet</code> as source data and
 * column names to specify the columns to be used for data.
 * 
 * @param jdbcData the input data as a <code>ResultSet</code>
 * @param xLabelColumnName the name of the column to use for <i>x</i> label 
 *     names.  If null, <i>x</i> labels are generated automatically.
 * @param yLabelColumnName the name of the column to use for <i>y</i> label 
 *     names.  If null, <i>y</i> labels are generated automatically.
 * @param activeColumnNames array of column names to use for <i>y</i> values. 
 *     If null, all numeric columns not associated with the <i>x</i> column or 
 *     the point labels column are used.
 */
public JDBC3dDataSource(ResultSet jdbcData,
			            String xLabelColumnName,
						String yLabelColumnName,
						String[] activeColumnNames)
{
    setJDBCData(jdbcData, xLabelColumnName, yLabelColumnName, activeColumnNames);
	int type = Chart3dDataEvent.RESET;
	JCData3dGridIndex index = new JCData3dGridIndex(JCData3dGridIndex.ALL,
											        JCData3dGridIndex.ALL);
	fireChart3dDataEvent(new Chart3dGridDataEvent(this, type, index));
}

/** 
 * Sets the data for this data source from the passed result set and according
 * to  the specification of columns in the passed name parameters.
 * 
 * @param jdbcData the input data as a <code>ResultSet</code>
 * @param xLabelColumnName the name of the column to use for <i>x</i> label 
 *     names.  If null, <i>x</i> labels are generated automatically.
 * @param yLabelColumnName the name of the column to use for <i>y</i> label 
 *     names.  If null, <i>y</i> labels are generated automatically.
 * @param activeColumnNames array of column names to use for <i>y</i> values. 
 *     If null, all numeric columns not associated with the <i>x</i> column or 
 *     the point labels column are used.
 */
public void setData(ResultSet jdbcData,
			        String xLabelColumnName,
					String yLabelColumnName,
					String[] activeColumnNames)
{
    setJDBCData(jdbcData,  xLabelColumnName, yLabelColumnName, activeColumnNames);
	int type = Chart3dDataEvent.RESET;
	JCData3dGridIndex index = new JCData3dGridIndex(JCData3dGridIndex.ALL,
											        JCData3dGridIndex.ALL);
	fireChart3dDataEvent(new Chart3dGridDataEvent(this, type, index));
}

/** 
 * Internal method to Populate the datasource with data from the result set
 * according to the specification of columns in the passed name parameters.
 * Series labels are the column labels of the active columns.
 * 
 * @param jdbcData <code>ResultSet</code> object to get data from.
 * @param xLabelColumnName the name of the column to use for <i>x</i> label 
 *     names.  If null, <i>x</i> labels are generated automatically.
 * @param yLabelColumnName the name of the column to use for <i>y</i> label 
 *     names.  If null, <i>y</i> labels are generated automatically.
 * @param activeColumnNames array of column names to use for <i>y</i> values. 
 *     If null, all numeric columns not associated with the <i>x</i> column or 
 *     the point labels column are used.
 */
protected void setJDBCData(ResultSet jdbcData,
			               String xLabelColumnName,
						   String yLabelColumnName,
						   String[] activeColumnNames)
{
    try {
		// No meta data?  No columns?  Clear the data and get out.
        ResultSetMetaData md = jdbcData.getMetaData();
        if (md == null) {
            clearData();
            return;
        }
        int columns = md.getColumnCount();
        if (columns <= 0) {
            clearData();
            return;
        }

		// initialize column numbers to default case where no columns
		// are explicitly specified
		int xLabelColumnNumber = -1;
		int yLabelColumnNumber = -1;
		boolean hasXLabelColumn = false;
		boolean hasYLabelColumn = false;
		Vector activeColumnNumbers = new Vector();

		// Root through columns looking for specified x, xlabel, and y, ylabel and
		// active z columns.  Note them if found.
		for (int i = 1; i <= columns; i++) {
			String columnName = md.getColumnName(i);


			if (xLabelColumnName != null &&
				columnName.equals(xLabelColumnName)) {
				xLabelColumnNumber = i;
				hasXLabelColumn = true;
			}

			if (yLabelColumnName != null &&
				columnName.equals(yLabelColumnName)) {

				yLabelColumnNumber = i;
				hasYLabelColumn = true;
			}
			if (activeColumnNames != null) {
				for (int j = 0; j < activeColumnNames.length; j++) {
					if (activeColumnNames[j] != null &&
						columnName.equals(activeColumnNames[j])) {
						activeColumnNumbers.addElement(new Integer(i));
					}
				}
			}
		}

		// If x label column not specified, take the first column if
		// it contains text
		if (xLabelColumnName == null) {
        	int SQLType = md.getColumnType(1);
        	if (SQLType == Types.CHAR || SQLType == Types.VARCHAR ||
            	SQLType == Types.LONGVARCHAR) {
            	xLabelColumnNumber = 1;
				hasXLabelColumn = true;
            }
		}

		//if no y label column is specified, we'll just leave it.

		// If we're dealing with default column selection, do a second pass
		// through the columns.  Pull out all columns containing numbers that
		// aren't already specified to be the x or y or labels columns.
		if (activeColumnNames == null) {
			for (int i = 1; i <= columns; i++) {
				if (i == xLabelColumnNumber || i == yLabelColumnNumber)
					continue;

        		int SQLType = md.getColumnType(i);
				if (SQLType == Types.DOUBLE || SQLType == Types.FLOAT ||
					SQLType == Types.BIGINT || SQLType == Types.INTEGER ||
					SQLType == Types.TINYINT || SQLType == Types.SMALLINT ||
					SQLType == Types.REAL || SQLType == Types.DECIMAL ||
					SQLType == Types.NUMERIC) {
					activeColumnNumbers.addElement(new Integer(i));
				}
			}
		}

		// number of series in data source is equal to the number of columns
		// of data we'll pull out of the query
        int nSeries = activeColumnNumbers.size();

		// Convert active column vector back into array for easier access
		int[] activeColumnArray = new int[nSeries];
		for (int i = 0; i < nSeries; i++) {
			activeColumnArray[i] =
				((Integer) activeColumnNumbers.elementAt(i)).intValue();
		}

        Vector vData = new Vector();
        Vector vLabels = new Vector();
        Vector yLabs = new Vector();

		xLabels = new String[nSeries];
		yLabels = null;

		// Go through the result set row by row
		try {
        int rowIndex = -1;
		Object xLabelObject = null;
		Object yLabelObject = null;
        while(jdbcData.next()) {
            rowIndex++;

			// get the data from the x column and the point label column
			if (hasXLabelColumn) {
				xLabelObject = jdbcData.getObject(xLabelColumnNumber);
				vLabels.addElement(xLabelObject.toString());

			}

			if (hasYLabelColumn) {
				yLabelObject = jdbcData.getObject(yLabelColumnNumber);
				yLabs.addElement(yLabelObject.toString());

			}

			// get the data from the z columns
            double[] rowData = new double[nSeries];
			for (int i = 0; i < nSeries; i++) {
				int thisColumnNumber = activeColumnArray[i];

				// be careful if y values are sharing columns with x or
				// point labels
				Object dataObject;
				if (thisColumnNumber == xLabelColumnNumber) {
					dataObject = xLabelObject;
				}
				else {
 					dataObject = jdbcData.getObject(thisColumnNumber);
				}

				// if a number, use it
				if (dataObject instanceof Number) {
					rowData[i] = ((Number)dataObject).doubleValue();
				// if a string, try to parse it
				} else if (dataObject instanceof String) {
					try {
						Double.valueOf((String)dataObject).doubleValue();
					} catch (NumberFormatException nfe) {
						rowData[i] = Double.MAX_VALUE;
					}
				// otherwise, we can't use it
				} else {
					rowData[i] = Double.MAX_VALUE;
				}


			}
           	vData.addElement(rowData);
        }
		} catch (SQLException e) {
			e.printStackTrace();
		}

        int nDataPoints = vData.size();

       	zValues = new double[nSeries][nDataPoints];
        if (hasXLabelColumn) {
            xLabels = new String[nSeries];
        }
		else {
			xLabels = null;
		}

		if (hasYLabelColumn) {
			yLabels = new String[nDataPoints];
		}
		else {
			yLabels = null;
		}
		for (int j = 0; j < nDataPoints; j++) {
			double[] rowData = (double[]) vData.elementAt(j);
			for (int i = 0; i < nSeries; i++) {
				zValues[i][j] = rowData[i];
			}



			if (hasYLabelColumn) {
				yLabels[j] = (String) yLabs.elementAt(j);

			}
		}//for

		for (int k = 0; k < nSeries; k++) {

			if (hasXLabelColumn) {
				xLabels[k] = (String) vLabels.elementAt(k);
			}
		}
    }//try
    catch (SQLException e) {
        e.printStackTrace(System.out);
        clearData();
        return;
    }
}

/** 
 * Internal method to clear the data.
 * 
 */
void clearData()
{
    xGrid = null;
    yGrid = null;
    zValues = null;
}

}
