/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Chart3dDataEvent.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:02 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.event;

import java.util.EventObject;

import com.klg.jclass.chart3d.*;

/**
 * Class used to encapsulate a JClass Chart 3D data change event.
 */
public class Chart3dDataEvent extends EventObject {

/** Type of data change. One of the enum values specified in this class */
protected int type;

/** Index object which gives info about which grid or point index affected */
protected JCData3dIndex index;

/**
 * Enum value indicating the data source name needs to be reloaded
 */
public final static int RELOAD_DATA_SOURCE_NAME = 1;

/**
 * Enum indicating that the data source has been completely reset.
 */
public final static int RESET = 2;

/**
 * Enum indicating that the data should be reloaded.
 */
public final static int RELOAD = 3;

/**
 * Enum value indicating the hole value needs to be reloaded
 */
public final static int RELOAD_HOLE_VALUE = 4;

/* The last enum value in the main data event class */
public final static int DATA_EVENT_LAST = RELOAD_HOLE_VALUE;

/**
 * Constructor used to create a ChartDataEvent object, with the chart data
 * changes and the data source
 * @param dataSrc the data source
 * @param type data change type to pass to the views
 * @param index points to series and point affected by the change
 */
public Chart3dDataEvent(Object dataSrc, int type, JCData3dIndex index)
{
	super(dataSrc);
	this.type  = type;
    this.index = index;
}//constructor

/**
 * Gets the Type property.  It contains the type of change that has happened
 * to the chart data.
 * @return the Type property, one of the enums above.
 */
public int getType()
{
	return(type);
}//getType

/**
 * Gets the data index.  This is either a JCData3dGridIndex (for grid data)
 * or a JCData3dPointIndex (for point data)
 * @return The data index that gives information about this event.
 */
public JCData3dIndex getIndex()
{
    return(index);
}//getIndex

}
