/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Chart3dDataSupport.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:02 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.event;

import java.util.Enumeration;

import com.klg.jclass.util.JCListenerList;
import com.klg.jclass.chart3d.JCData3dIndex;
import com.klg.jclass.chart3d.JCData3dGridIndex;
import com.klg.jclass.chart3d.JCData3dPointIndex;

/**
 * Class used to implement the {@link Chart3dDataManager} interface.
 */
public class Chart3dDataSupport implements Chart3dDataManager, java.io.Serializable {

private JCListenerList    listeners;
protected Object          source;

/**
 * The default constructor for a Chart3dDataSupport object
 */
public Chart3dDataSupport()
{
	source = this;
}

/**
 * Constructor used to create a Chart3dDataSupport object with a given data
 * source.
 * @param dataSrc the data source
 */
public Chart3dDataSupport(Object dataSrc)
{
	source = dataSrc;
}

/**
 * Adds a chart data listener.
 * @param l the chart data listener object to be added
 */
public void addChart3dDataListener(Chart3dDataListener l)
{
	listeners = JCListenerList.add(listeners, l);
}

/**
 * Removes a chart data listener.
 * @param l chart data listener object to be removed
 */
public void removeChart3dDataListener(Chart3dDataListener l)
{
	listeners = JCListenerList.remove(listeners, l);
}

/**
 * Fires a chart data event to any registered listeners.  This
 * version of fireChartDataEvent() will create a Chart3dDataEvent
 * object out of the message and a JCData3dIndex
 * @param type A valid message type from Chart3dDataEvent
 * @param index The data index which tells which (x, y) or (series, point)
 *   to which this event refers.
 */
public void fireChart3dDataEvent(int type, JCData3dIndex index)
{
	Chart3dDataEvent event = null;
	if (index instanceof JCData3dGridIndex) {
		event = new Chart3dGridDataEvent(source, type,
		                                 (JCData3dGridIndex)index);
	}
	else if (index instanceof JCData3dPointIndex) {
		event = new Chart3dPointDataEvent(source, type,
		                                  (JCData3dPointIndex)index);
	}
	if (event != null) {
	    fireChart3dDataEvent(event);
	}
}

/**
 * Fires a chart data event to any registered listeners.
 * @param event The event to send to registered listeners.
 */
public void fireChart3dDataEvent(Chart3dDataEvent event)
{
	Enumeration e = JCListenerList.elements(listeners);
    while (e.hasMoreElements()) {
	    ((Chart3dDataListener)e.nextElement()).chart3dDataChange(event);
	}
}

}
