/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Chart3dGridDataEvent.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:02 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.event;

import com.klg.jclass.chart3d.JCData3dGridIndex;

/**
 * Class used to encapsulate a chart 3D grid data change event.
 */
public class Chart3dGridDataEvent extends Chart3dDataEvent {

/**
 * Enum value indicating a given x value is affected.
 */
public final static int RELOAD_XVALUE = DATA_EVENT_LAST + 1;

/**
 * Enum value indicating all x values are affected.
 */
public final static int RELOAD_XGRID = DATA_EVENT_LAST + 2;

/**
 * Enum value indicating a given y value is affected.
 */
public final static int RELOAD_YVALUE = DATA_EVENT_LAST + 3;

/**
 * Enum value indicating all y values are affected.
 */
public final static int RELOAD_YGRID = DATA_EVENT_LAST + 4;

/**
 * Enum value indicating one z Value is affected.
 */
public final static int RELOAD_ZVALUE = DATA_EVENT_LAST + 5;

/**
 * Enum value indicating one array of zValues is affected.
 */
public final static int RELOAD_ZARRAY = DATA_EVENT_LAST + 6;

/**
 * Enum value indicating the entire doubly indexed array of zValues is affected.
 */
public final static int RELOAD_ZALL = DATA_EVENT_LAST + 7;

/**
 * Enum value indicating a particular xLabel is affected.
 */
public final static int RELOAD_XLABEL = DATA_EVENT_LAST + 8;

/**
 * Enum value indicating all xLabels are affected.
 */
public final static int RELOAD_ALL_XLABELS = DATA_EVENT_LAST + 9;

/**
 * Enum value indicating a particular yLabel is affected.
 */
public final static int RELOAD_YLABEL = DATA_EVENT_LAST + 10;

/**
 * Enum value indicating all yLabels are affected.
 */
public final static int RELOAD_ALL_YLABELS = DATA_EVENT_LAST + 11;

/**
 * Constructor used to create a ChartDataEvent object, with the chart data
 * changes and the data source
 * @param dataSrc the data source
 * @param type data change type to pass to the views
 * @param index location of the change
 */
public Chart3dGridDataEvent(Object dataSrc, int type, JCData3dGridIndex index)
{
	super(dataSrc, type, index);
}

/**
 * Method which returns the x index of the affected data.
 * Returns -100, if all x values are affected.
 * @return index the X index affected.
 */
public int getX()
{
    JCData3dGridIndex gridIndex = (JCData3dGridIndex)index;
	return(gridIndex.getX());
}

/**
 * Method which returns the y index of the affected data.
 * Returns -100, if all y values are affected.
 * @return index the y index affected.
 */
public int getY()
{
    JCData3dGridIndex gridIndex = (JCData3dGridIndex)index;
	return(gridIndex.getY());
}

}
