/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Chart3dPointDataEvent.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:02 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.event;

import com.klg.jclass.chart3d.JCData3dPointIndex;

/**
 * Class used to encapsulate a chart 3d point data change event.
 */
public class Chart3dPointDataEvent extends Chart3dDataEvent {

/**
 * Enum value indicating a single data value has changed.
 * In this case, seriesIndex and pointIndex are used to
 * index the point.  An event of this type should be used
 * when a value has been changed in a series value array that
 * has previously been queried by the chart from the data source.
 * If the array needs to be regenerated, RELOAD_SERIES should
 * be used instead.
 */
public final static int  RELOAD_POINT = DATA_EVENT_LAST + 1;

/**
 * Enum value indicating a data series has changed.
 * In this case, seriesIndex points to the series to reload.
 * An event of this type will requery the series value array
 * from the data source.
 */
public final static int  RELOAD_SERIES = DATA_EVENT_LAST + 2;

/**
 * Enum value indicating a data series has been added to the
 * end of the set of data series.
 */
public final static int  ADD_SERIES = DATA_EVENT_LAST + 3;

/**
 * Enum value indicating a data series has ben inserted at
 * a particular index in the set of data series.  In this
 * case, seriesIndex holds the index at which to insert the
 * series.
 */
public final static int INSERT_SERIES = DATA_EVENT_LAST + 4;

/**
 * Enum value indicating a data series has been removed.
 * In this case, seriesIndex holds the index of the series
 * to be removed.
 */
public final static int  REMOVE_SERIES = DATA_EVENT_LAST + 5;

/**
 * Enum value indicating a particular series label needs
 * to be reloaded.  seriesIndex indicates which label
 * should be reloaded.
 */
public final static int RELOAD_SERIES_LABEL = DATA_EVENT_LAST + 6;

/**
 * Enum value indicating all series labels need to be reloaded.
 */
public final static int RELOAD_ALL_SERIES_LABELS = DATA_EVENT_LAST + 7;

/**
 * Constructor used to create a ChartDataEvent object, with the chart data
 * changes and the data source
 * @param dataSrc the data source
 * @param type data change type to pass to the views
 * @param index points to series and point affected by the change
 */
public Chart3dPointDataEvent(Object dataSrc, int type,
                             JCData3dPointIndex index)
{
	super(dataSrc, type, index);
}

/**
 * Retrieves the point index associated with the event.
 *
 * @return int the index of the point affected.  Returns -100 if all
 * points are affected.
 */
public int getPoint()
{
    JCData3dPointIndex pointIndex = (JCData3dPointIndex)index;
	return(pointIndex.getPoint());
}

/**
 * Retrieves the series index associated with the event.
 *
 * @return int the index of the series affected.  Returns -100 if all
 * series are affected.
 */
public int getSeries()
{
    JCData3dPointIndex pointIndex = (JCData3dPointIndex)index;
	return(pointIndex.getSeries());
}

}
