/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCPick3dEvent.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:02 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.event;

import java.util.EventObject;

import com.klg.jclass.chart3d.JCData3dIndex;
import com.klg.jclass.chart3d.JCChart3d;

/**
 * Represents a pick event in <code>JCChart3d</code>.  A pick event occurs when a
 * <code>JCPick3dListener</code> is installed on a 3D chart and the mouse button is pressed
 * down over a <code>JCChart3d</code> object.
 */
public class JCPick3dEvent extends EventObject {

/**
 * The result of a pick event.  Holds the data index of the point picked.  If
 * this index is a <code>JCData3dGridIndex</code>, the index points to the (x, y) grid point
 * that was selected.  If it is a <code>JCData3dPointIndex</code>, it references the
 * selected point in the point data set.  If it is a <code>JCData3dContourIndex</code>,
 * it references a contour level.
 */
protected JCData3dIndex pickResult;

/**
 * Constructor to create a pick event.  It is passed in a <code>chart3d</code> object
 * and a <code>JCData3dIndex</code>, which is the result of a pick event.
 *
 * @param chart3d the chart3d object which was picked
 * @param index the pick result index
 */
public JCPick3dEvent(JCChart3d chart3d, JCData3dIndex index)
{
	super(chart3d);
	pickResult = index;
}

/**
 * Gets the result of a pick event, which is a <code>JCData3dIndex</code> object.  If no subcomponent
 * of a <code>JCChart3d</code> (header, footer, legend, chart area) was picked, this will
 * be null.
 * @return a <code>JCData3dIndex</code> instance holding the pick result
 */
public JCData3dIndex getPickResult()
{
	return(pickResult);
}

}
