/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCChart3dJava2d.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:03 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.j2d;

import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.InputEvent;
import javax.swing.JLabel;
import java.util.ArrayList;

import com.klg.jclass.chart3d.*;
import com.klg.jclass.util.swing.JCExitFrame;


/**
 * The <code>JCChart3dJava2d</code> class extends <code>JCChart3d</code> to give a JClass Chart 3D object that
 * is based on the Java 2D API.
 */
public class JCChart3dJava2d extends JCChart3d implements Chart3dGridDataModel {

protected String		customizerName = "com.klg.jclass.chart3d.j2d.Chart3dCustomizerJava2d";

/**
 * Constructor that creates a <code>JCChart3dJava2d</code> of a particular type.
 * @param tp
 */
public JCChart3dJava2d(int tp) {
	super(tp);
    createObjects(tp);
}

/**
 * Default constructor, required by JavaBeans.
 */
public JCChart3dJava2d()
{
	super();
    createObjects(SURFACE);
}

/**
 * Creates a new action table.
 */
protected void createActionTable()
{
    actionTable = new ActionTable();
}

/**
 * Gets the current customizer name.
 * @return the customizer name
 */
public String getCustomizerName() {
	return customizerName;
}

/**
 * Sets the customizer name.
 * @param name the new customizer name
 */
public void setCustomizerName(String name) {
	customizerName = name;
}

/**
 * Gets the proper action based on the given input event.
 * @param event the user event
 * @return the associated action
 */
protected JCAction getActionHandler(InputEvent event)
{
    return(actionTable.getActionHandlerInstance(event, chart3dArea));
}

/**
 * Creates the objects needed for this <code>Chart3d</code> object.
 *
 * @param chartType the initial chart type of the chart (one of <code>SURFACE</code>,
 *   <code>BAR</code>, or <code>SCATTER</code>)
 */
protected void createObjects(int chartType)
{
	JCSymbolStyle.setDefault2dShapes(null);
	boolean batchedSave = isBatched();
	setBatched(true);
	createChart3dArea();

	Chart3dDataView dv = addDataView(0, chartType);
	dv.setElevation(new JCElevation());
	JCContour contour = new JCContour(dv);
	dv.setContour(contour);
	contour.setContourLevels(new JCContourLevels());

    JCAxis[] axes = new JCAxis[JCAxis.NUM_AXES];
    chart3dArea.setAxes(axes);
    chart3dArea.setAxis(new JCAxis(JCAxis.AXIS_X, getFont()), JCAxis.AXIS_X);
    chart3dArea.setAxis(new JCAxis(JCAxis.AXIS_Y, getFont()), JCAxis.AXIS_Y);
    chart3dArea.setAxis(new JCAxis(JCAxis.AXIS_Z, getFont()), JCAxis.AXIS_Z);
    for (int i = 0; i < JCAxis.NUM_AXES; i++) {
        axes[i].setGridLines(new JCGridLines(axes[i], JCGridLines.NO_PLANE));
    }

    JCPlotCube plotCube = new JCPlotCube(chart3dArea);
    plotCube.setFloor(new JCProjection());
    plotCube.setCeiling(new JCProjection());
    chart3dArea.setPlotCube(plotCube);

    chart3dArea.setViewport(new JCViewport());
    chart3dArea.setView3d(new JCView3d());
    chart3dArea.setCube(new Cube(chart3dArea));
    chart3dArea.setTransform(new Transform(chart3dArea));
    chart3dArea.setAnno(new Anno(chart3dArea));
    chart3dArea.setSurface(new JCSurface());
    chart3dArea.setBar(new JCBar());
    chart3dArea.setScatter(new JCScatter());
	dv.setElevationDataSource(this);
	dv.setMapping(new MapPick());
	setBatched(batchedSave);
}

/**
 * Main method for running chart as an application.
 *
 * @param args command line arguments
 */
public static void main(String args[])
{
    String labels[]  = {"1.0", "3.0", "5.0", "7.0", "9.0"};
    String zLabels[] = {"0.0", "10.0", "20.0", "30.0", "40.0", "50.0"};
	String label_list[] = {"one", "two", "three", "four", "five",
						   "six", "seven", "eight", "nine", "ten"};
	String data_labels[] = new String[10];
	data_labels[0] = label_list[0];
	for (int i = 1; i < data_labels.length; i++) {
		data_labels[i] = label_list[(i - 1)/2 + 1];
	}

	JCValueLabel vLabel1 = new JCValueLabel(1.0, "one");
	vLabel1.setBackground(Color.black);
	vLabel1.setForeground(Color.yellow);
	JCValueLabel vLabel2 = new JCValueLabel(1.0, "two");
	vLabel2.setBackground(Color.red);
	vLabel2.setForeground(Color.white);
	JCValueLabel vLabel3 = new JCValueLabel(1.0, "three");
	vLabel3.setBackground(Color.yellow);
	vLabel3.setForeground(Color.black);
	JCValueLabel vLabel4 = new JCValueLabel(1.0, "four");
//	JCValueLabel vLabel5 = new JCValueLabel(1.0, "five");
//	JCValueLabel vLabel6 = new JCValueLabel(1.0, "six");
	ArrayList list = new ArrayList();
	list.add(vLabel1);
	list.add(vLabel1);
	list.add(vLabel1);
	list.add(vLabel2);
	list.add(vLabel2);
	list.add(vLabel4);
	list.add(vLabel3);
	list.add(vLabel3);
	list.add(vLabel3);
	list.add(vLabel3);

	String tags[] = {"anno", "jcchart3darea"};
	
	
	JCExitFrame f = new JCExitFrame("JClass Chart3d");
	JCChart3d chart3d = new JCChart3dJava2d();
	chart3d.setAllowUserChanges(true);
    chart3d.getActionTable().addAllDefaultActions();
	chart3d.setHeader(new JLabel("Header"));
	chart3d.setFooter(new JLabel("Footer"));
	chart3d.getLegend().setVisible(true);
	JCChart3dArea area = chart3d.getChart3dArea();
	Chart3dDataView dataView = chart3d.getDataView(0);
	dataView.setChartType(JCChart3d.SURFACE);
	area.setVisible(true);
	area.setOpaque(true);
	area.setBackground(Color.blue);
	area.setForeground(Color.white);
	area.getPlotCube().setBackground(Color.gray);
//	area.getPlotCube().getFloor().setContoured(false);
//	area.getPlotCube().getFloor().setZoned(true);
//	area.getPlotCube().getCeiling().setContoured(true);
//	area.getPlotCube().getCeiling().setZoned(false);
	area.getSurface().setSolid(false);
//	area.getPlotCube().setZScale(0.5);
	area.getView3d().setXRotation(25.0);
	area.getView3d().setYRotation(0.0);
	area.getView3d().setZRotation(45.0);
	area.getAxis(JCAxis.AXIS_X).getGridLines().setPlaneMask(7);
	area.getAxis(JCAxis.AXIS_Y).getGridLines().setPlaneMask(7);
	dataView.getElevation().setMeshed(true);
	dataView.getElevation().setShaded(true);
	dataView.getContour().setZoned(true);
	dataView.getContour().setContoured(true);
	dataView.getElevation().setTransparent(false);
    JCValueLabel[] vLabels = new JCValueLabel[5];
    for (int i = 0; i < vLabels.length; i++) {
        vLabels[i] = new JCValueLabel((double)(2 * i + 1), labels[i]);
    }
    area.getAxis(JCAxis.AXIS_X).setValueLabels(vLabels);
    area.getAxis(JCAxis.AXIS_Y).setValueLabels(vLabels);
    JCValueLabel[] vZLabels = new JCValueLabel[6];
    for (int i = 0; i < vZLabels.length; i++) {
        vZLabels[i] = new JCValueLabel((double)(10*i), zLabels[i]);
    }
    area.getAxis(JCAxis.AXIS_Z).setValueLabels(vZLabels);
	Chart3dGridData gridData = (Chart3dGridData)dataView.getElevationData();
	gridData.setXLabels(data_labels);
	gridData.setYLabelsArrayList(list);
	area.getAxis(JCAxis.AXIS_X).setAnnotationMethod(JCAxis.ANNOTATION_DATA_LABELS);
	area.getAxis(JCAxis.AXIS_X).setCombineLabels(true);
	area.getAxis(JCAxis.AXIS_Y).setAnnotationMethod(JCAxis.ANNOTATION_DATA_LABELS);
	area.getAxis(JCAxis.AXIS_Y).setCombineLabels(true);
	f.getContentPane().setLayout(new GridLayout(1,1));
	f.getContentPane().add(chart3d);
	f.setSize(600, 500);
	f.show();
}

}
