/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: MapPick.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:03 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.j2d;

import java.awt.Point;
import java.awt.Rectangle;
import javax.vecmath.Point3d;
import javax.vecmath.Point4d;

import com.klg.jclass.chart3d.*;


/**
 * A class that implements the Mapping interface.  This computes the map and
 * pick functions for the j2d version of Chart3d.
 */
public class MapPick implements Mapping, java.io.Serializable {

protected Chart3dData data = null;

/**
 * Default constructor, required by Java Beans.
 */
public MapPick()
{
}

/**
 * Default constructor, required by Java Beans.
 */
public MapPick(Chart3dData data)
{
    this.data = data;
}

/**
 * Find the euclidean distance (squared) between the two passed in points.
 */
protected double calcDist(int x1, int y1, int x2, int y2)
{
	int	diffx, diffy, value;

	diffx = x2 - x1;
	diffy = y2 - y1;
	value = diffx * diffx + diffy * diffy;
	return(Math.sqrt((double)value));
}

/**
 * Find the corresponding (x, y) coordinate in data space to (xPixel, yPixel)
 * via interpolation.
 */
protected Rectangle interpolateXY(int xPixel, int yPixel,
                                  double xMin, double yMin,
			                      double xMax, double yMax,
                                  Point3d mapPoint)
{
	if (data == null || !data.isDataOK()) {
		return(null);
	}
	JCChart3dArea area      = data.getDataView().getChart3d().getChart3dArea();
	Transform     transform = area.getTransform();

	Point4d minPoint = new Point4d(xMin, yMin, 0.0, 1.0);
	Point4d maxPoint = new Point4d(xMax, yMax, 0.0, 1.0);

	transform.transformPoint(transform.getView(), minPoint);
	int x1 = (int) Math.round(minPoint.x);
	int y1 = (int) Math.round(minPoint.y);

	transform.transformPoint(transform.getView(), maxPoint);
	int x2 = (int) Math.round(maxPoint.x);
	int y2 = (int) Math.round(maxPoint.y);

	//	To find the x,y point in data space, we linearly interpolate
	//	the x & y values from screen space to data space.
	//
	//	Compute interval of visible data in data space.
	double xInterval = xMax - xMin;
	double yInterval = yMax - yMin;

	//	Apply screen space ratio to the data space and add the
	//	data space origin to arrive at the data space coordinate.
	mapPoint.x = (xPixel - x1) * xInterval / (x2 - x1) + xMin;

	//	Negate all screen y values since there is a mirror about
	//	x-axis between the screen and data spaces
	mapPoint.y = (y1 - yPixel) * yInterval / (y1 - y2) + yMin;
    return(new Rectangle(Math.min(x1, x2), Math.min(y1, y2),
			             Math.abs(x2 - x1), Math.abs(y2 - y1)));
}

/**
 * Return the data's screen points.  Compute them if they are not already
 * there.
 */
protected ScreenPoint[][] getScreenPoints()
{
	if (data == null) {
		return(null);
	}

    ScreenPoint[][] screenPoints = data.getScreenPoints();
    if (screenPoints == null) {
        data.precomputePoints();
        screenPoints = data.getScreenPoints();
    }
	return(screenPoints);
}

/**
 * Dummy implementation.  Meant to be overridden in MapPickGrid or MapPickPoint.
 */
public Point3d map(int xPixel, int yPixel)
{
    return(null);
}

/**
 * Performs an unmap operation by transforming the given data point to the
 * position on the screen where it would be drawn.
 * @param point The data point to be unmapped.
 * @return The screen point to which the data point maps
 */
public java.awt.Point unmap(Point3d point)
{
	if (data == null || !data.isDataOK()) {
		return(null);
	}
	JCChart3dArea area      = data.getDataView().getChart3d().getChart3dArea();
	Transform     transform = area.getTransform();
	Point4d pt = new Point4d(point.x, point.y, point.z, 1.0);
	transform.transformPoint(transform.getView(), pt);
	return(new Point((int) Math.round(pt.x), (int)Math.round(pt.y)));
}

/**
 * Dummy implementation.  Meant to be overridden in MapPickGrid or MapPickPoint.
 */
public JCData3dIndex pick(int xPixel, int yPixel)
{
    return(null);
}

/**
 * Dummy implementation.  Meant to be overridden in MapPickGrid or MapPickPoint.
 */
public java.awt.Point unpick(JCData3dIndex index)
{
    return(null);
}

/**
 * Return the internal data object whose type determines the type of MapPick
 * object required.
 */
public Chart3dData getData()
{
	return(data);
}

/**
 * Create a new mapping, depending on the type of input data
 */
public Mapping createMapping(Chart3dData data)
{
    if (data == null) {
        return(null);
    }

    if (data instanceof Chart3dGridData) {
        return(new MapPickGrid(data));
    }
    if (data instanceof Chart3dPointData) {
        return(new MapPickPoint(data));
    }
    return(null);
}

}
