/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AnnoJava3d.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:04 $  $Locker:  $  Quest Software Inc.

//  AnnoJava3d.java

package com.klg.jclass.chart3d.j3d;

import com.klg.jclass.chart3d.*;
import com.klg.jclass.chart3d.JCBox;
import com.klg.jclass.chart3d.j2d.actions.RotateAction;


import javax.media.j3d.*;
import javax.vecmath.*;
import java.awt.*;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Hashtable;

class AnnoJava3d extends Anno {

// Axis annotations are created/destroyed as the plotcube is rotated.
// Java3d hangs onto the Material and ColoringAttributes objects
// that are associated with the annotation Shape3D objects, causing a
// memory leak.  To alleviate this problem, a single Material object
// is shared amongst all annotation objects.  We also pool the
// JCColoringAttributes objects -- if a ColoringAttributes object
// of the required color already exists in the Hashtable, we reuse it.
protected Material material = null;
protected Hashtable colorAttribHashtable = null;

protected AnnoJava3d(JCChart3dAreaJava3d area)
{
	super(area);
	colorAttribHashtable = new Hashtable();
}

/**
 * Need to update the values of XRotation, YRotation, and ZRotation
 * in the base package JCView3d object:
 */
protected void updateChart3dRotation()
{
	// Get the current transformation for the plotcube:
	JCPlotCubeJava3d plotCube = (JCPlotCubeJava3d)area.getPlotCube();
	TransformGroup tg = plotCube.getTransformGroup();
	Transform3D t3d = new Transform3D();
	tg.getTransform(t3d);
	// Extract the rotational component of the plotcube transformation:
	Matrix3d matrix = new Matrix3d();
	t3d.get(matrix);

	Matrix4d m4d = new Matrix4d();
	m4d.setIdentity();
	m4d.setRotation(matrix);
	RotateAction rotateAction = new RotateAction();
	rotateAction.extractAngles(m4d);
	//  Chart3dRotateHandler.extractAngles() returns angles in degrees.

	area.getView3d().setXRotation(rotateAction.getXRotation() + 90.0);
	area.getView3d().setYRotation(rotateAction.getYRotation());
	area.getView3d().setZRotation(rotateAction.getZRotation());
}

/**
 * Adjust the dataport for a given axis.  This needs to be consistent for
 * each possible rotation.  Thus we obtain the maximum possible axis
 * extention by calculating the extension for both the vertical and
 * horizontal cases and taking the maximum.
 *
 * @param gc The graphics object to use for drawing and getting font info
 * @param axis The axis whose dataport might be adjusted
 * @param dataport The object which stores the current dataport extents
 * @param startEdge The starting edge for this axis
 * @param scale The axis normalized scale
 * @param tight We want tight bounds in the Z direction for bars
 * @param justify Justify object (needed in the base class but not here)
 * @param axisMin The axis min value
 * @param axisMax The axis max value
 */
public boolean adjustDataport(Graphics gc, JCAxis axis, JCBox dataport,
                              int startEdge, double scale, boolean tight,
							  Justify justify, double axisMin, double axisMax)
{
	// Find the maximum extension of the dataport by do both extensions for
	// annotation being vertical (perpendicular to the axis) and
	// horizontal (parallel to the axis)
	extendAxisDataport(gc, axis, dataport, scale, tight,
					   true, axisMin, axisMax);
	extendAxisDataport(gc, axis, dataport, scale, tight,
					   false, axisMin, axisMax);
	return(true);
}

/**
 * Override method.
 *
 * Calculate the orientation matrix.  This matrix orients the annotation plane
 * so that it faces the viewer most.  This plane is identified by finding which
 * of the unit axes has the largest magnitude Z component after the perspective,
 * rotation, and edgeMatrix transformations have been applied.  Also compute,
 * whether to rotate the annotation, whether its above or below the axis,
 * whether to flip the annotation in the Y direction, and whether to spin the
 * axis title.  All this is stored in the justify object.
 */
public Matrix4d getOrientationMatrix(int edge, Justify justify)
{
    Transform transform = area.getTransform();
    Matrix4d  rotation  = transform.getRotation();

	if (orientationMatrix == null) {
	    orientationMatrix = new Matrix4d();
	}
    orientationMatrix.setIdentity();

    justify.setFlipX(false);
    justify.setFlipY(false);
    justify.setSpinTitle(false);

	// Find best annotation plane: matrix = [-y][pm][rm][edgeMatrix]
    if (planeMatrixInit == null) {
        planeMatrixInit = new Matrix4d();
        planeMatrixInit.setIdentity();
        Transform.scaleMatrix(planeMatrixInit, 1.0, -1.0, 1.0);
        planeMatrixInit.mul(transform.getPerspective());
        planeMatrixInit.mul(rotation);
    }
    matrix.mul(planeMatrixInit, edgeMatrix);

    // Initialize points if not done already
	if (u == null) {
		u = new Point4d[NUM_POINTS];
		for (int i = 0; i < NUM_POINTS; i++) {
			u[i] = new Point4d();
		}
	}
    u[0].set(0.0, 0.0, 0.0, 1.0);
    u[1].set(1.0, 0.0, 0.0, 1.0);
    u[2].set(0.0, 1.0, 0.0, 1.0);
    u[3].set(0.0, 0.0, 1.0, 1.0);

    Transform.transformPoint(matrix, u[0]);
    Transform.transformPoint(matrix, u[1]);
    Transform.transformPoint(matrix, u[2]);
    Transform.transformPoint(matrix, u[3]);

    // Initialize vectors if not done already
	if (v == null) {
		v = new Vector3d[NUM_VECTORS];
		for (int i = 0; i < NUM_VECTORS; i++) {
			v[i] = new Vector3d();
		}
	}
    v[0].set(u[1].x - u[0].x, u[1].y - u[0].y, u[1].z - u[0].z);
    v[1].set(u[2].x - u[0].x, u[2].y - u[0].y, u[2].z - u[0].z);
    v[2].set(u[3].x - u[0].x, u[3].y - u[0].y, u[3].z - u[0].z);

	// It is essential to renormalize the unit vectors since the perspective
	// foreshortening effect is a shear in x and y which stretches/shrinks
	// the actual length of the unit vector
    v[0].normalize();
    v[1].normalize();
    v[2].normalize();

//	System.out.println("edge=" + edge);
//	System.out.println("v[0]=" + v[0].toString());
//	System.out.println("v[1]=" + v[1].toString());
//	System.out.println("v[2]=" + v[2].toString() + "\n");

    double dx = 0.0;
    double dy = 0.0;

	double uy_z = v[1].z;
	double uz_z = v[2].z;

	switch (edge) {
	case Cube.YNEG_ZNEG:			/* X back-bottom */
        justify.setAbove(false);
		// Load up axis dx,dy for justify.rotate determination
		dx = rotation.m00;
		dy = rotation.m10;

		dy = -dy;
		// Decide best plane for display
		if (Math.abs(uz_z) < Math.abs(uy_z)) {
			// Change from xy plane to xz plane
            Transform.rotateMatrix(orientationMatrix, Transform.ROTATE_90,
                                   JCAxis.AXIS_X);
			if (-uy_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
		else {
			if (uz_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
	    break;
	case Cube.YNEG_ZPOS:			/* X front-bottom */
        justify.setAbove(false);
		// Load up axis dx,dy for justify.rotate determination
		dx = rotation.m00;
		dy = rotation.m10;

		dy = -dy;
		// Decide best plane for display
		if (Math.abs(uz_z) < Math.abs(uy_z)) {
			// Change from xy plane to xz plane
            Transform.rotateMatrix(orientationMatrix, Transform.ROTATE_270,
                                   JCAxis.AXIS_X);
			if (uy_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
		else {
			if (uz_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
	    break;
	case Cube.YPOS_ZNEG:			/* X back-top */
        justify.setAbove(true);
		// Load up axis dx,dy for justify.rotate determination
		dx = rotation.m00;
		dy = rotation.m10;

		dy = -dy;
		// Decide best plane for display
		if (Math.abs(uz_z) < Math.abs(uy_z)) {
			// Change from xy plane to xz plane
            Transform.rotateMatrix(orientationMatrix, Transform.ROTATE_270,
                                   JCAxis.AXIS_X);
			if (uy_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
		else {
			if (uz_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
	    break;
	case Cube.YPOS_ZPOS:			/* X front-top */
        justify.setAbove(true);
		// Load up axis dx,dy for justify.rotate determination
		dx = rotation.m00;
		dy = rotation.m10;

		dy = -dy;
		// Decide best plane for display
		if (Math.abs(uz_z) < Math.abs(uy_z)) {
			// Change from xy plane to xz plane
            Transform.rotateMatrix(orientationMatrix, Transform.ROTATE_90,
                                   JCAxis.AXIS_X);
			if (-uy_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
		else {
			if (uz_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
	    break;
	case Cube.XNEG_ZNEG:			/* Y back-left */
        justify.setAbove(true);
		// Load up axis dx,dy for justify.rotate determination
		dx = rotation.m01;
		dy = rotation.m11;

		dy = -dy;
		// Decide best plane for display
		if (Math.abs(uz_z) < Math.abs(uy_z)) {
			// Change from xy plane to yz plane
            Transform.rotateMatrix(orientationMatrix, Transform.ROTATE_270,
                                   JCAxis.AXIS_X);
			if (uy_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
		else {
			if (uz_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
	    break;
	case Cube.XNEG_ZPOS:
        justify.setAbove(true);
		// Load up axis dx,dy for justify.rotate determination
		dx = rotation.m01;
		dy = rotation.m11;

		dy = -dy;
		// Decide best plane for display
		if (Math.abs(uz_z) < Math.abs(uy_z)) {
			// Change from xy plane to yz plane
            Transform.rotateMatrix(orientationMatrix, Transform.ROTATE_90,
                                   JCAxis.AXIS_X);
			if (-uy_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
		else {
			if (uz_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
	    break;
	case Cube.XPOS_ZNEG:
        justify.setAbove(false);
		// Load up axis dx,dy for justify.rotate determination
		dx = rotation.m01;
		dy = rotation.m11;

		dy = -dy;
		// Decide best plane for display
		if (Math.abs(uz_z) < Math.abs(uy_z)) {
			// Change from xy plane to yz plane
            Transform.rotateMatrix(orientationMatrix, Transform.ROTATE_90,
                                   JCAxis.AXIS_X);
			if (-uy_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
		else {
			if (uz_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
	    break;
	case Cube.XPOS_ZPOS:
        justify.setAbove(false);
		// Load up axis dx,dy for justify.rotate determination
		dx = rotation.m01;
		dy = rotation.m11;

		dy = -dy;
		// Decide best plane for display
		if (Math.abs(uz_z) < Math.abs(uy_z)) {
			// Change from xy plane to yz plane
            Transform.rotateMatrix(orientationMatrix, Transform.ROTATE_270,
                                   JCAxis.AXIS_X);
			if (uy_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
		else {
			if (uz_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
	    break;
	case Cube.XNEG_YNEG:			/* Z bottom-left */
        justify.setAbove(false);
		// Load up axis dx,dy for justify.rotate determination
		dx = rotation.m02;
		dy = rotation.m12;

		dy = -dy;
		// Decide best plane for display
		if (Math.abs(uz_z) < Math.abs(uy_z)) {
			// Change from yz plane to xz plane
            Transform.rotateMatrix(orientationMatrix, Transform.ROTATE_270,
                                   JCAxis.AXIS_X);
			if (uy_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
		else {
			if (uz_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
	    break;
	case Cube.XNEG_YPOS:
        justify.setAbove(true);
		// Load up axis dx,dy for justify.rotate determination
		dx = rotation.m02;
		dy = rotation.m12;

		dy = -dy;
		// Decide best plane for display
		if (Math.abs(uz_z) < Math.abs(uy_z)) {
			// Change from yz plane to xz plane
            Transform.rotateMatrix(orientationMatrix, Transform.ROTATE_90,
                                   JCAxis.AXIS_X);
			if (-uy_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
		else {
			if (uz_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
	    break;
	case Cube.XPOS_YNEG:
        justify.setAbove(false);
		// Load up axis dx,dy for justify.rotate determination
		dx = rotation.m02;
		dy = rotation.m12;

		dy = -dy;
		// Decide best plane for display
		if (Math.abs(uz_z) < Math.abs(uy_z)) {
			// Change from yz plane to xz plane
            Transform.rotateMatrix(orientationMatrix, Transform.ROTATE_90,
                                   JCAxis.AXIS_X);
			if (-uy_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
		else {
			if (uz_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
	    break;
	case Cube.XPOS_YPOS:
        justify.setAbove(true);
		// Load up axis dx,dy for justify.rotate determination
		dx = rotation.m02;
		dy = rotation.m12;

		dy = -dy;
		// Decide best plane for display
		if (Math.abs(uz_z) < Math.abs(uy_z)) {
			// Change from yz plane to xz plane
            Transform.rotateMatrix(orientationMatrix, Transform.ROTATE_270,
                                   JCAxis.AXIS_X);
			if (uy_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
		else {
			if (uz_z > 0.0) {
                justify.setFlipY(true);
				dx = -dx;
			}
		}
	    break;
	default:
	    
	    break;
	}

	// Find rotation for best horizontal bias when annotation is drawn.
	// The annotation value calculations will force either vertical or
    // horizontal orientation.  Choose the closest.
    int rotate = Transform.ROTATE_NONE;
	if (justify.isVertical()) {
        rotate = (dy >= 0.0) ? Transform.ROTATE_90 : Transform.ROTATE_270;
	}
    else {
        rotate = (dx >= 0.0) ? Transform.ROTATE_NONE : Transform.ROTATE_180;
    }
    justify.setRotate(rotate);

	Point2d point = findEdgeDxDy(edge);
    if (justify.doFlipY()) {
        point.x = -point.x;
    }
    if (point.x < 0.0) {
        justify.setSpinTitle(true);
    }
    else {
        justify.setSpinTitle(false);
    }
	return(orientationMatrix);
}

//  Override method
public void draw3dAnno(Graphics gc)
{
	if (gc == null) {
		return;
	}
    gc.setColor(area.getPlotCube().getDrawForeground());
	((JCPlotCubeJava3d)area.getPlotCube()).removeAllAnnotation();

    Matrix4d textView = area.getTransform().getTextView();
	draw3dAxis(gc, area.getAxis(JCAxis.AXIS_Z), textView);
	draw3dAxis(gc, area.getAxis(JCAxis.AXIS_X), textView);
	draw3dAxis(gc, area.getAxis(JCAxis.AXIS_Y), textView);
}


/**
 * In 3d mode, draw the annotation for a given axis.  The internal value labels,
 * generated earlier, store the annotation.
 */
protected void draw3dAxis(Graphics gc, JCAxis axis, Matrix4d ctm)
{
	JCPlotCubeJava3d plotCube       = (JCPlotCubeJava3d)area.getPlotCube();
	boolean[]        annotateEdge   = area.getCube().getAnnotateEdge();
	Justify          justify        = new Justify();

    int     startEdge = -1;;
    double  scale     = 1.0;
    switch (axis.getAxisId()) {
    case JCAxis.AXIS_X:
        startEdge = Cube.YNEG_ZNEG;
        scale     = plotCube.getXNormalizedScale();
        break;
    case JCAxis.AXIS_Y:
        startEdge = Cube.XNEG_ZNEG;
        scale     = plotCube.getYNormalizedScale();
        break;
    case JCAxis.AXIS_Z:
        startEdge = Cube.XNEG_YNEG;
        scale     = plotCube.getZNormalizedScale();
        break;
    default:
	    
        return;
    }

	double annoHeight  = axis.getAnnoFontCubeSize() * CUBE_SCALE;

	determineVertical(startEdge, justify);
	for (int edge = startEdge; edge <= startEdge + 3; edge++) {
		if (annotateEdge[edge]) {
			// Init the annotation basis
			double annoMin = axis.getAnnoMin();
			double annoMax = axis.getAnnoMax();
			double dMin    = axis.getDataportMin();
			double dMax    = axis.getDataportMax();

			edgeMatrix = getEdgeMatrix(edge);
			orientationMatrix = getOrientationMatrix(edge, justify);

		    Matrix4d myMatrix = new Matrix4d();
			//  Don't use ctm -- it includes perspective, screen, rotation, etc.
			//  that we don't want
			myMatrix.setIdentity();
		    //  Don't use the screen matrix -- results in an offset!!
			myMatrix.mul(area.getTransform().getRotation());
			Transform.scaleMatrix(myMatrix,
								  plotCube.getXNormalizedScale(),
								  plotCube.getYNormalizedScale(),
								  plotCube.getZNormalizedScale());

			Vector3d trans = new Vector3d();
			myMatrix.get(trans);

		    axisMatrix.setIdentity();
			axisMatrix.setTranslation(trans);
			axisMatrix.setScale(myMatrix.getScale());
			axisMatrix.mul(edgeMatrix);

            ArrayList valueLabels = axis.getAnnoHandler().getValueLabels();
            if (valueLabels == null || valueLabels.size() == 0) {
                continue;
            }

            ArrayList arrayListTG = new ArrayList();

            for (int i = 0; i < valueLabels.size(); i++) {
                JCValueLabel vLabel = (JCValueLabel)valueLabels.get(i);
                double value = vLabel.getValue();
                if (annoMin <= value && value <= annoMax) {
                    String label = vLabel.getLabel();

					matrix.set(axisMatrix);
					double pos =
						(2.0 * (value - dMin) / (dMax - dMin) - 1.0) * scale;
					Transform.translateMatrix(matrix, pos, 0.0, 0.0);
					matrix.mul(orientationMatrix);
					Transform.scaleMatrix(matrix, 1.0, -1.0, 1.0);

                    double w = cubeTextWidth(gc, axis, label, false, true);
					double width = w * CUBE_SCALE;
					drawAnnotation(gc, axis, matrix, justify, annoHeight,
								   width, vLabel, arrayListTG);
               }
            }
            if (axis.getTitle() != null) {
				matrix.mul(axisMatrix, orientationMatrix);
				Transform.scaleMatrix(matrix, 1.0, -1.0, 1.0);

                double w = cubeTextWidth(gc, axis, axis.getTitle(), true, true);
                double width = w * CUBE_SCALE;
                double easeway = axis.getAnnoEaseway();
                if (justify.above) {
                    easeway = -easeway;
                }
                Transform.translateMatrix(matrix, -width/2.0, easeway, 0.0);

                if (justify.doSpinTitle()) {
                    Transform.translateMatrix(matrix, width, 0.0, 0.0);
                    Transform.rotateMatrix(matrix, Transform.ROTATE_180,
                                           JCAxis.AXIS_Z);
                }
                if (justify.doFlipY()) {
                    Transform.translateMatrix(matrix, width, 0.0, 0.0);
                    Transform.rotateMatrix(matrix, Transform.ROTATE_180,
                                           JCAxis.AXIS_Y);
                }
                if (justify.doFlipX()) {
                    Transform.rotateMatrix(matrix, Transform.ROTATE_180,
                                           JCAxis.AXIS_X);
                }
				cubeDrawString(gc, axis, matrix, axis.getTitle(),
				               true, null, arrayListTG);
            }
            JCBranchGroup bg = new JCBranchGroup();
            for (int i = 0; i < arrayListTG.size(); i++) {
				bg.addChild((TransformGroup)arrayListTG.get(i));
			}
			arrayListTG.clear();
			plotCube.addAnnotationBranchGroup(bg);
		}
	}
}

/**
 * Transform and draw string on the cube
 */
protected void cubeDrawString(Graphics gc, JCAxis axis, Matrix4d m,
                              String text, boolean isTitle, Color foreground,
							  ArrayList arrayListTG)
{
    if (gc == null || axis == null || text == null || text.length() == 0) {
		return;
    }

	Font font = (isTitle) ? axis.getTitleFont() : axis.getAnnoFont();
	if (font == null) {
		return;
	}

    Graphics2D g2 = (Graphics2D)gc;
	g2.setFont(font);
    FontRenderContext frc = g2.getFontRenderContext();
    TextLayout layout = new TextLayout(text, font, frc);

	Chart3dAnnoHandler  annoHandler  = axis.getAnnoHandler();
    Rectangle2D         bounds       = layout.getBounds();
	int                 fontCubeSize = 0;
	double              height       = 0.0;
	if (isTitle) {
		fontCubeSize = axis.getTitleFontCubeSize();
		height       = bounds.getHeight();
	}
	else {
		fontCubeSize = axis.getAnnoFontCubeSize();
		height       = annoHandler.getAnnoHeight();
	}

    double ty = isTitle ? -bounds.getY() - height / 2.0
				        : annoHandler.getAnnoShift();

    double scale = (double)fontCubeSize * CUBE_SCALE / height;

	TransformGroup tg = createTransformGroup(m, scale);

	Font3D f3d = new Font3D(new Font(font.getName(), Font.BOLD, font.getSize()),
						    new FontExtrusion());
	Text3D txt = new Text3D(f3d, text,
						    new Point3f((float)(-bounds.getX()),
                                        (float)-ty, 0.0f));
	JCShape3D sh = new JCShape3D();
	JCAppearance app = new JCAppearance();
	if (material == null) {
		material = new Material();
		material.setLightingEnable(false);
	}
	app.setMaterial(material);
	if (foreground == null) {
		foreground = area.getPlotCube().getDrawForeground();
	}
	JCColoringAttributes colorAttrib =
	     (JCColoringAttributes)colorAttribHashtable.get(foreground);
	if (colorAttrib == null) {
		colorAttrib = new JCColoringAttributes(new Color3f(foreground),
		                              JCColoringAttributes.SHADE_GOURAUD);
		colorAttribHashtable.put(foreground, colorAttrib);
	}
	app.setColoringAttributes(colorAttrib);
	sh.setGeometry(txt);
	sh.setAppearance(app);

	tg.addChild(sh);
	arrayListTG.add(tg);
}

/**
 * Given a string and a transformation matrix, draw the tick and annotation
 * based on the information in the justify object.  If tickOnly is true,
 * only a half size tick is drawn.  If the string is null only the tick (full
 * or half size depending on the value of tickOnly) is drawn.
 */
protected void drawAnnotation(Graphics gc, JCAxis axis, Matrix4d ctm,
							  Justify justify, double height, double width,
							  JCValueLabel vLabel, ArrayList arrayListTG)
{
	if (vLabel.isTickDrawn()) {
        addTickMarks(axis, ctm, justify, height, vLabel, arrayListTG);
	}
	String drawString = vLabel.getLabel();
	if (!vLabel.isLabelDrawn() ||
	    drawString == null     ||
		drawString.length() == 0)
	{
		return;
	}

	// Compose the annotation-specific transformation matrix. This matrix
	// depends on the height & width of the specific annotation to position it
	// properly with respect to its tick mark
	justifyMatrix = getJustifyMatrix(height, width, justify);

	// Apply the justification transformation
	matrix.mul(ctm, justifyMatrix);

	// Draw the string
	cubeDrawString(gc, axis, matrix, drawString, false,
	               vLabel.getForeground(), arrayListTG);
}

protected void addTickMarks(JCAxis axis, Matrix4d m, Justify justify,
                            double height, JCValueLabel vLabel,
							ArrayList arrayListTG)
{
	JCPlotCubeJava3d plotCube = (JCPlotCubeJava3d)(area.getPlotCube());
	Chart3dAnnoHandler annoHandler = axis.getAnnoHandler();
    int fontCubeSize = axis.getAnnoFontCubeSize();
    double annoHeight = annoHandler.getAnnoHeight();
    double scale = (double)fontCubeSize * CUBE_SCALE / annoHeight;

	if (u == null) {
		u = new Point4d[NUM_POINTS];
		for (int i = 0; i < NUM_POINTS; i++) {
			u[i] = new Point4d();
		}
	}
	u[0].set(0.0, 0.0, 0.0, 1.0);
    u[1].set(0.0, justify.isAbove() ? - height/(2.0 * scale)
	                                : height/(2.0 * scale), 0.0, 1.0);
    if (vLabel.isMinorTick()) {
        u[1].y *= 0.6;
    }

    TransformGroup tg = createTransformGroup(m, scale);

    LineArray lineArray =
        new LineArray(2, GeometryArray.COORDINATES | GeometryArray.COLOR_3 );
	Point3d verts[] = new Point3d[2];
	Color3f clr[] = new Color3f[2];
    verts[0] = new Point3d(u[0].x, u[0].y, u[0].z);
    verts[1] = new Point3d(u[1].x, u[1].y, u[1].z) ;
    clr[0] = new Color3f(plotCube.getForeground());
    clr[1] = new Color3f(plotCube.getForeground());
    lineArray.setCoordinates(0, verts);
	lineArray.setColors(0, clr);
	lineArray.setCapability(GeometryArray.ALLOW_INTERSECT);
	lineArray.setCapability(GeometryArray.ALLOW_FORMAT_READ);
	lineArray.setCapability(GeometryArray.ALLOW_COUNT_READ);
	lineArray.setCapability(GeometryArray.ALLOW_COORDINATE_READ);
	JCShape3D sh = new JCShape3D(lineArray);
	tg.addChild(sh);
	arrayListTG.add(tg);
}

private TransformGroup createTransformGroup(Matrix4d m, double scale)
{
	Transform3D t3d = new Transform3D();
	Vector3d trans = new Vector3d();
	m.get(trans);   //  get the translational component
    Matrix3d mRot = new Matrix3d();
    m.get(mRot);  //  get the rotational component
	t3d.setScale(scale);
	t3d.setTranslation(trans);  //  apply the translational component
    t3d.setRotation(mRot);    //  apply the rotational component
	TransformGroup tg = new TransformGroup(t3d);
    return(tg);
}

}
