/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Chart3dDataViewJava3d.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:04 $  $Locker:  $  Quest Software Inc.

//  Chart3dDataViewJava3d.java

package com.klg.jclass.chart3d.j3d;

import java.awt.Graphics;
import com.klg.jclass.chart3d.*;


public class Chart3dDataViewJava3d extends Chart3dDataView {

protected JCChart3dAreaJava3d area = null;

protected Chart3dDataViewJava3d(JCChart3dJava3d chart3d, int dataViewId)
{
	super(chart3d, dataViewId, JCChart3d.SURFACE);
	this.area = (JCChart3dAreaJava3d)chart3d.getChart3dArea();
}

protected Chart3dDataViewJava3d(JCChart3dJava3d chart3d, int dataViewId, int chartType)
{
	super(chart3d, dataViewId, chartType);
	this.area = (JCChart3dAreaJava3d)chart3d.getChart3dArea();
}

/**
 * Creates the correct draw object based on the data type.
 */
public void createDraw()
{
	// Create a grid or point draw object depending on the data
    boolean isGridData = (elevationData instanceof Chart3dGridData);
    if (draw == null) {
        if (isGridData) {
            draw = new DrawGridJava3d(this);
        }
        else {
            draw = new DrawPointJava3d(this);
        }
    }
    else {
		// Already a draw object.  Only create a new one if the type changed.
        if (isGridData && (draw instanceof DrawPoint)) {
            draw = new DrawGridJava3d(this);
        }
        if (!isGridData && (draw instanceof DrawGrid)) {
            draw = new DrawPointJava3d(this);
        }
    }

	// Set the correct mapping object on the draw object
	if (draw.getMapping() == null ||
	    draw.getMapping().getData() != elevationData)
	{
		if (mapping == null) {
			
			return;
		}
		draw.setMapping(mapping.createMapping(elevationData));
	}
}

/**
 *   In the Java3D version of Chart3d, "draw" means "create scene graph objects
 *   to represent the data".  The scene graph objects may be points, surfaces,
 *   lines, ribbons, 3D text, etc.
 */
protected void doPreprocessing()
{
	// Create the proper draw object
	createDraw();

	// Preprocess text objects here

	// Preprocess contours here

}

public boolean isBusy() {
	return area.getChart3dUniverse().getVirtualUniverse().getCanvas3D().getWaitRenderFlag();
}

public void setBusyFlag(boolean fBusy) {
    area.getChart3dUniverse().getVirtualUniverse().getCanvas3D().setWaitRenderFlag(fBusy);
}


protected void computeTransformations()
{
	JCChart3dAreaJava3d area = (JCChart3dAreaJava3d)chart3d.getChart3dArea();
	area.recalc();
	//  calls dataView.calcDefaults()
	Transform transform = area.getTransform();
    if (transform != null) {
        transform.computeTransformations(chart3d.getChart3dArea().getGraphics());
    }
}


protected void createDataObject()
{
	doPreprocessing();

    boolean isGridData = (elevationData instanceof Chart3dGridData);
    if (isGridData)
	{
	    ((DrawGridJava3d)draw).draw();
	}
	else
	{
		((DrawPointJava3d)draw).draw();
	}
}


/**
 *   Create Java3D scene graph object(s) representing the data.
 */
public void createDataViewObject()
{
	computeTransformations();
	createDataObject();
}

}
