/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CubeWall.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:04 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.j3d;

import com.sun.j3d.utils.picking.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;


class CubeWall {

protected final static float DEFAULT_POLYGON_OFFSET = 1.0f;
protected final static float DEFAULT_POLYGON_OFFSET_FACTOR = 1.0f;
protected Vector3d imageRotationAxis = null;
protected Vector3d eulerAngles = null;
protected Vector3d translation = null;
protected JCBranchGroup bg = null;
protected JCTransformGroup tg = null;
protected JCShape3D filledCubeFace = null;
protected JCShape3D outlineCubeFace = null;
private double imageRotationAngle = 0.0;

protected CubeWall(Vector3d imageRotationAxis,
                   Vector3d eulerAngles,
                   Vector3d translation,
                   Color3f fillColor,
                   Color3f outlineColor)
{
    this.imageRotationAxis = imageRotationAxis;
    this.eulerAngles = eulerAngles;
    this.translation = translation;
    this.bg = new JCBranchGroup();
    this.tg = new JCTransformGroup();
    tg.addChild(bg);

    createWall(fillColor, outlineColor);
}

protected void createWall(Color3f fillColor, Color3f outlineColor)
{
    Geometry geom = new SquareQA(fillColor);
    filledCubeFace = new JCShape3D(geom);
    createFace(filledCubeFace, fillColor,
               PolygonAttributes.POLYGON_FILL, DEFAULT_POLYGON_OFFSET,
               DEFAULT_POLYGON_OFFSET_FACTOR);

    geom = new SquareQA(outlineColor);
    outlineCubeFace = new JCShape3D(geom);
    createFace(outlineCubeFace, outlineColor,
               PolygonAttributes.POLYGON_LINE, 0.0f, 0.0f);
}

private void createFace(JCShape3D face, Color3f color,
                        int polygonMode, float polygonOffset,
                        float polygonOffsetFactor)
{
    JCAppearance appearance = new JCAppearance();
    appearance.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_READ);
    appearance.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_WRITE);

    Material m = new Material() ;
    m.setLightingEnable(false) ;
    appearance.setMaterial(m) ;

    JCRenderingAttributes renderAttrib = new JCRenderingAttributes();
    renderAttrib.setIgnoreVertexColors(true);
    appearance.setRenderingAttributes(renderAttrib);

    JCColoringAttributes colorAttrib = new JCColoringAttributes();
    colorAttrib.setColor(color);
    appearance.setColoringAttributes(colorAttrib);

    PolygonAttributes pa = new PolygonAttributes();
    pa.setCapability(PolygonAttributes.ALLOW_CULL_FACE_READ);
    pa.setCapability(PolygonAttributes.ALLOW_CULL_FACE_WRITE);
    pa.setCapability(PolygonAttributes.ALLOW_NORMAL_FLIP_READ);
    pa.setCapability(PolygonAttributes.ALLOW_NORMAL_FLIP_WRITE);
    pa.setPolygonMode(polygonMode);
    pa.setPolygonOffsetFactor(polygonOffsetFactor);
    pa.setPolygonOffset(polygonOffset);
	appearance.setPolygonAttributes(pa);

    TransparencyAttributes ta = new TransparencyAttributes();
    ta.setCapability(TransparencyAttributes.ALLOW_MODE_READ);
    ta.setCapability(TransparencyAttributes.ALLOW_MODE_WRITE);
    ta.setCapability(TransparencyAttributes.ALLOW_VALUE_READ);
    ta.setCapability(TransparencyAttributes.ALLOW_VALUE_WRITE);
    ta.setCapability(TransparencyAttributes.ALLOW_BLEND_FUNCTION_READ);
    ta.setCapability(TransparencyAttributes.ALLOW_BLEND_FUNCTION_WRITE);
    appearance.setTransparencyAttributes(ta);

    face.setAppearance(appearance);
	face.setCapability(Shape3D.ENABLE_PICK_REPORTING);
	PickTool.setCapabilities(face, PickTool.INTERSECT_FULL);
    tg.addChild(face);
}

protected void setImageAngle(double angle)
{
    double remainder = Math.IEEEremainder(angle, 90.0);
    if (remainder != 0.0) {
        return;
    }
	imageRotationAngle = angle;
	int nTurns = (int)Math.abs(angle / 90.0);
	SquareQA squareQA = (SquareQA)filledCubeFace.getGeometry();
	squareQA.rotateTextureCoordinates(nTurns, (angle > 0.0));
}

protected double getImageAngle()
{
	return imageRotationAngle;
}

protected void flipTextureCoordinates(int axis)
{
	SquareQA squareQA = (SquareQA)filledCubeFace.getGeometry();
	squareQA.flipTextureCoordinates(axis);
}

protected void addObject(JCBranchGroup bgObject)
{
	bg.addChild(bgObject);
}

protected void clearObjects()
{
	bg.removeAllChildren();
}

}

