/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataAdjustGrid.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:05 $  $Locker:  $  Quest Software Inc.

//  DataAdjustGrid.java

package com.klg.jclass.chart3d.j3d;

import javax.vecmath.*;
import com.klg.jclass.chart3d.*;

public class DataAdjustGrid extends DataAdjust {

protected int chartType = -1;
protected Chart3dGridData gridData = null;
protected double xValue;
protected double yValue;
protected double radiusOfInfluence;
protected double scaleFactor = 1.0;
protected int xIndex;
protected int yIndex;
protected double[] xGrid = null;
protected double[] yGrid = null;
protected double[][] zValues = null;

public DataAdjustGrid(Chart3dGridData gridData, double xVal, double yVal,
					  double radius) {
	xGrid = gridData.getXGrid();
	yGrid = gridData.getYGrid();
	zValues = gridData.getZValues();

	this.gridData = gridData;
	Chart3dDataView dataView = gridData.getDataView();
	chartType = dataView.getChartType();

	holeValue = gridData.getHoleValue();
	double range = gridData.getDataMax() - gridData.getDataMin();
	scaleFactor = range * dataView.getMouseEditScaleFactor();

	if (chartType == JCChart3dJava3d.SURFACE) {
		xValue = xVal;
		yValue = yVal;
		if (radius <= 0.0) {
			radius = 0.00000001;
		}
		radiusOfInfluence = radius;
	}
	else {
		//  BAR or SCATTER
		xIndex = gridData.getXClosest(xVal);
		yIndex = gridData.getYClosest(yVal);
	}
}

public void adjustData(double deltaZ) {
	double deltaX, deltaY, distance;
	double deltaXSquared, zFunction;

	if (chartType == JCChart3dJava3d.SURFACE)
	for (int i = 0; i < xGrid.length; i++) {
		deltaX = Math.abs(xGrid[i] - xValue);
		deltaXSquared = deltaX * deltaX;
		for (int j = 0; j < yGrid.length; j++) {
			if (zValues[i][j] != holeValue) {
				deltaY = Math.abs(yGrid[j] - yValue);
		    	distance = Math.sqrt( deltaXSquared + (deltaY * deltaY) );
				zFunction = scaleFactor * deltaZ * Math.exp(-distance/radiusOfInfluence);
				zValues[i][j] += zFunction;
			}
		}
	}
	else {
		//  BAR or SCATTER
		if (zValues[xIndex][yIndex] != holeValue) {
			zValues[xIndex][yIndex] += (deltaZ * scaleFactor);
		}
	}
}

}
