/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataAdjustPoint.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:05 $  $Locker:  $  Quest Software Inc.

//  DataAdjustPoint.java

package com.klg.jclass.chart3d.j3d;

import javax.vecmath.*;
import com.klg.jclass.chart3d.*;

public class DataAdjustPoint extends DataAdjust {

protected Chart3dPointData pointData = null;
protected Point3d point3d = null;
protected double scaleFactor = 1.0;

public DataAdjustPoint(Chart3dPointData pointData, double xVal, double yVal,
					  double radius)
{
	this.pointData = pointData;
	JCChart3dArea area = pointData.getDataView().getChart3d().getChart3dArea();

	holeValue = pointData.getHoleValue();
	double range = pointData.getDataMax() - pointData.getDataMin();
	scaleFactor = range * pointData.getDataView().getMouseEditScaleFactor();

	JCAxis xAxis = area.getAxis(JCAxis.AXIS_X);
	JCAxis yAxis = area.getAxis(JCAxis.AXIS_Y);

	double errX = (xAxis.getMax() - xAxis.getMin()) / 1000000.;
	double errY = (yAxis.getMax() - yAxis.getMin()) / 1000000.;

	for (int series = 0; series < pointData.getNumSeries(); series++) {
		Chart3dPointSeries pointSeries = pointData.getSeries(series);
		JCChart3dStyle chartStyle  = pointSeries.getChartStyle();
		for (int point = 0; point < pointSeries.getNumPoints(); point++) {
			Point3d tmp = pointSeries.getPoint(point);
			if ((isEqual(tmp, xVal, yVal, errX, errY)) &&
				(tmp.z != holeValue)) {
				point3d = tmp;
				return;
			}
		}
	}
}

public void adjustData(double deltaZ)
{
	if (point3d != null) {
		point3d.z += (deltaZ * scaleFactor);
	}
}

protected boolean isEqual(Point3d pnt, double xValue, double yValue,
						  double errX, double errY)
{
	if ( (pnt.x >= xValue - errX) && (pnt.x <= xValue + errX) &&
		 (pnt.y >= yValue - errY) && (pnt.y <= yValue + errY) ) {
		return true;
	}
	return false;
}

}
