/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataAdjustmentAxis.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:05 $  $Locker:  $  Quest Software Inc.

//  DataAdjustmentAxis.java

package com.klg.jclass.chart3d.j3d;

import java.util.*;
import com.sun.j3d.utils.picking.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.klg.jclass.chart3d.*;

public class DataAdjustmentAxis {

private JCChart3dJava3d chart3d = null;
private JCPlotCubeJava3d plotCube = null;
private DataAdjust dataAdjust = null;
private Chart3dData data = null;
private Chart3dDataViewJava3d dataView = null;
private JCChart3dAreaJava3d area = null;
private JCContour contour = null;
private double zStartPoint = 3.0;
private double zEndPoint = -3.0;
private Color3f axisColor = null;

//  Aligned with the z-axis
public DataAdjustmentAxis(JCChart3dJava3d chart3d, Point3d pt,
						  double radiusOfInfluence) {
	super();
	this.chart3d = chart3d;
	this.area = (JCChart3dAreaJava3d)(chart3d.getChart3dArea());
	plotCube = (JCPlotCubeJava3d)area.getPlotCube();

	//  Added to the current TransformGroup.  Contains the BranchGroup for
	//  the line.
	JCBranchGroup bg = new JCBranchGroup();

	Point3d start = new Point3d(pt.x, pt.y, zStartPoint);
	Point3d end = new Point3d(pt.x, pt.y, zEndPoint);
	axisColor = new Color3f(1.0f, 0.0f, 0.0f);
	ZLine zLine = new ZLine(start, end, axisColor);
	Appearance appearance = new Appearance();
	JCShape3D axis = new JCShape3D(zLine, appearance);
	axis.setPickable(false);
	bg.addChild(axis);
	plotCube.addPickAxis(bg);

	dataView = (Chart3dDataViewJava3d)chart3d.getDataView(0);
	data = dataView.getElevationData();

	Transform transform = area.getTransform();
	Matrix4d iAbsolute = transform.getInverseAbsolute();
	iAbsolute.transform(pt);
	if (data instanceof Chart3dGridData) {
		dataAdjust = new DataAdjustGrid((Chart3dGridData)data,
										pt.x, pt.y,
										radiusOfInfluence);
	}
	else {
		dataAdjust = new DataAdjustPoint((Chart3dPointData)data,
										 pt.x, pt.y,
										 radiusOfInfluence);
	}

	contour = dataView.getContour();
}

public boolean isBusy() {
	return area.getChart3dUniverse().getVirtualUniverse().getCanvas3D().getWaitRenderFlag();
}

public void setZCoord(double deltaZ) {
//	long start = System.currentTimeMillis();
    area.getChart3dUniverse().getVirtualUniverse().getCanvas3D().setWaitRenderFlag(true);

	// modify the grid data
	dataAdjust.adjustData(deltaZ);

	// replace the existing data view object (surface, bars, or scatter points)
	dataView.createDataViewObject();

//	long delta = System.currentTimeMillis() - start;
//	System.out.println("delta=" + delta);
}

public void setStartPointZCoord(double zCoord) {
	zStartPoint = zCoord;
}

public double getStartPointZCoord() {
	return zStartPoint;
}

public void setEndPointZCoord(double zCoord) {
	zEndPoint = zCoord;
}

public double getEndPointZCoord() {
	return zEndPoint;
}

public void setAxisColor(Color3f color) {
	axisColor = color;
}

public Color3f getAxisColor() {
	return axisColor;
}

}

class ZLine extends LineArray {

protected ZLine(Point3d start, Point3d end, Color3f clr3f) {
	super(2, GeometryArray.COORDINATES | GeometryArray.COLOR_3);
	Point3d pnts[] = new Point3d[2];
	Color3f[] clrs = new Color3f[2];

	pnts[0] = start;
	pnts[1] = end;
	clrs[0] = clr3f;
	clrs[1] = clr3f;
	setCoordinates(0, pnts);
	setColors(0, clrs);
}

}
