/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataSurfaceLA.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:05 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.j3d;

import java.awt.Color;
import java.util.ArrayList;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.klg.jclass.chart3d.*;

class DataSurfaceLA {

private LineArray meshLineArray = null;
private ArrayList gridPntList = new ArrayList();
private ArrayList gridClrList = new ArrayList();

private LineArray[] contourLineArray = null;
private ArrayList[] contourPntList = null;
private ArrayList[] contourClrList = null;

private JCContour contour = null;
private JCContourLevels contourLevels = null;
private ArrayList contourStyles = null;

private double[] xGrid = null;
private double[] yGrid = null;
private double[][] zValues = null;

private boolean[] xMesh = null;
private boolean[] yMesh = null;

private Matrix4d absolute = null;
private double holeValue = Double.MAX_VALUE;

public DataSurfaceLA( Matrix4d absolute, Chart3dGridData data,
				      JCContour contour )
{
	data.getDataView().getChart3d().getChart3dArea().getSurface().computeMeshFiltering(data.getDataView());
	xMesh = data.getDataView().getChart3d().getChart3dArea().getSurface().getXMesh();
	yMesh = data.getDataView().getChart3d().getChart3dArea().getSurface().getYMesh();
	GridLimits gridLimits = data.getGridLimits();

	this.absolute = absolute;
	this.xGrid = data.getXGrid();
	this.yGrid = data.getYGrid();
	this.zValues = data.getZValues();
    this.holeValue = data.getHoleValue();

    if (xMesh == null) {
		xMesh = new boolean[yGrid.length];
		for (int i=0; i < yGrid.length; i++) {
			xMesh[i] = false;
		}
	}

    if (yMesh == null) {
		yMesh = new boolean[xGrid.length];
		for (int i=0; i < xGrid.length; i++) {
			yMesh[i] = false;
		}
	}

	contour.calcContours();
	this.contour = contour;
	contourStyles = contour.getContourStyles();
	ContourCell[] contourCells = contour.getContourCells();
	this.contourLevels = contour.getContourLevels();

	//  Create a LineArray, pntList, and clrList for each contour level
	int numLevels = contourLevels.getNumLevelsInternal();
	contourPntList = new ArrayList[numLevels];
	contourClrList = new ArrayList[numLevels];
	for (int i = 0; i < numLevels; i++) {
		contourPntList[i] = new ArrayList();
		contourClrList[i] = new ArrayList();
	}

	ContourCell contourCell = null;
	Point3d vertex0 = null;
	Point3d vertex1 = null;
	Point3d vertex2 = null;
	Point3d vertex3 = null;
	int x, y, nIntersections;

	int xOffset = gridLimits.getXOffset();
	int yOffset = gridLimits.getYOffset();
	int xLast = gridLimits.getXLast();
	int yLast = gridLimits.getYLast();
	int cellIndex = 0;
	for (x = xOffset; x < xLast; x++) {
		for (y = yOffset; y < yLast; y++) {
		    contourCell = contourCells[cellIndex];
		    nIntersections = contourCell.getNIntersections();
			if (nIntersections == 0) {
			    //  Handle the case where there are no contour segments
			    //  in the grid cell:
				vertex0 = new Point3d( xGrid[x + 1], yGrid[y], zValues[x + 1][y] );
				vertex1 = new Point3d( xGrid[x + 1], yGrid[y + 1], zValues[x + 1][y + 1] );
				vertex2 = new Point3d( xGrid[x], yGrid[y + 1], zValues[x][y + 1] );
				vertex3 = new Point3d( xGrid[x], yGrid[y], zValues[x][y] );

				if (yMesh[x+1]) {
					//  Draw the right side (vertex0 to vertex1) of the grid cell if
					//    - it is at the right edge of the surface, and the cell does not contain a hole
					//    - it is on the right edge of a hole, and there is no hole to its right
					//    - it is on the left edge of a hole, and there is no hole to its left
					//    - the left side of the grid cell is at the left edge of the surface, and the cell does not contain a hole value
					//    - there is no hole on its left and no hole on its right.
					if ((((x+1)==xLast) && !containsHoleValue(x, y, zValues, holeValue)) ||
						(((x+1) < xLast) && containsHoleValue(x, y, zValues, holeValue) && !containsHoleValue(x+1, y, zValues, holeValue)) ||
						(((x+1) < xLast) && !containsHoleValue(x, y, zValues, holeValue) && containsHoleValue(x+1, y, zValues, holeValue)) ||
						((x==xOffset) && !containsHoleValue(x, y, zValues, holeValue)) ||
						((x > 0) && ((x+1) < xLast) && !containsHoleValue(x, y, zValues, holeValue) && !containsHoleValue(x+1, y, zValues, holeValue))) {
						defineLineSegment(vertex0, vertex1);
					}
				}
				if (xMesh[y+1]) {
					//  Draw the top side (vertex1 to vertex2) of the grid cell if
					//    - it is at the top edge of the surface, and the cell does not contain a hole
					//    - it is on the top edge of a hole, and there is no hole immediately above it
					//    - it is on the bottom edge of a hole, and there is no hole immediately below it
					//    - the bottom of the cell is at the bottom of the surface, and the cell does not contain a hole value
					//    - there is no hole below it, and no hole above it.
					if ((((y+1)==yLast) && !containsHoleValue(x, y, zValues, holeValue)) ||
						(((y+1) < yLast) && containsHoleValue(x, y, zValues, holeValue) && !containsHoleValue(x, y+1, zValues, holeValue)) ||
						(((y+1) < yLast) && !containsHoleValue(x, y, zValues, holeValue) && containsHoleValue(x, y+1, zValues, holeValue)) ||
						((y==yOffset) && !containsHoleValue(x, y, zValues, holeValue)) ||
						((y > 0) && ((y+1) < yLast) && !containsHoleValue(x, y, zValues, holeValue) && !containsHoleValue(x, y+1, zValues, holeValue))) {
						defineLineSegment(vertex1, vertex2);
					}
				}
				//  Draw the left side (vertex2 to vertex3) of the grid cell if it is at the left edge of the surface,
				//  and the cell does not contain a hole value
				if ((x==xOffset) && (yMesh[x])) {
					if (!containsHoleValue(x, y, zValues, holeValue)) {
						defineLineSegment(vertex2, vertex3);
					}
				}
				//  Draw the bottom edge (vertex3 to vertex0) of the grid cell if it is at the bottom edge of the
				//  surface and the cell does not contain a hole value.
				if ((y==yOffset) && (xMesh[y])) {
					if (!containsHoleValue(x, y, zValues, holeValue)) {
						defineLineSegment(vertex3, vertex0);
					}
				}
			}
			else {
		        defineCellSections(contourCell, contour, x, y);
		    }
			cellIndex++;
		}
	}

	meshLineArray = createLineArray(gridPntList, gridClrList);

	contourLineArray = createContourLineArrays(numLevels, contourPntList,
										       contourClrList);
}

private boolean containsHoleValue(int x, int y, double[][] zValue,
                                  double holeValue)
{
    return ( (zValue[x][y] == holeValue)     ||
             (zValue[x+1][y] == holeValue)   ||
             (zValue[x+1][y+1] == holeValue) ||
             (zValue[x][y+1] == holeValue) );
}

private LineArray createLineArray(ArrayList pntList, ArrayList clrList)
{
	if (pntList.size() == 0) {
		return null;
	}

	Point3d[] pnts = new Point3d[pntList.size()];
	pntList.toArray(pnts);

	Color3f[] clrs = new Color3f[clrList.size()];
	clrList.toArray(clrs);

	LineArray lineArray = new LineArray( pnts.length,
									     GeometryArray.COORDINATES |
									     GeometryArray.COLOR_3 );
	lineArray.setCoordinates(0, pnts);
	lineArray.setColors(0, clrs);

	return lineArray;
}

private LineArray[] createContourLineArrays(int numLevels,
										    ArrayList[] pntList,
											ArrayList[] clrList)
{
	LineArray[] lineArray = new LineArray[numLevels];
	for (int i = 0; i < numLevels; i++) {
		if (pntList[i].size() > 0) {
	        Point3d[] pnts = new Point3d[pntList[i].size()];
	        pntList[i].toArray(pnts);

	        Color3f[] clrs = new Color3f[clrList[i].size()];
	        clrList[i].toArray(clrs);

	    	lineArray[i] = new LineArray( pnts.length,
	    							      GeometryArray.COORDINATES |
	    							      GeometryArray.COLOR_3 );
	        lineArray[i].setCoordinates(0, pnts);
	        lineArray[i].setColors(0, clrs);
		}
		else {
			lineArray[i] = null;
		}
	}

	return lineArray;
}

//  Note:  returned value is unscaled coordinate.
private Point3d getVertex(int cornerIndex, int x, int y)
{
	Point3d vertex = null;

	switch(cornerIndex)
	{
	case 0:
		vertex = new Point3d( xGrid[x + 1], yGrid[y], zValues[x + 1][y] );
		break;
	case 1:
		vertex = new Point3d( xGrid[x + 1], yGrid[y + 1], zValues[x + 1][y + 1] );
		break;
	case 2:
		vertex = new Point3d( xGrid[x], yGrid[y + 1], zValues[x][y + 1] );
		break;
	case 3:
		vertex = new Point3d( xGrid[x], yGrid[y], zValues[x][y] );
		break;
	default:
		System.out.println(
		"DataSurfaceLA.getVertex():  Invalid index for cell corner.");
	}
	return vertex;
}

private void defineCellSections( ContourCell contourCell, JCContour contour,
				    		     int x, int y )
{
	Intersection intersection = null;
	ContourSegment segment1 = null;
	ContourSegment segment2 = null;
	Corner[] corner = new Corner[4];
	Point3d cornerCoord = null;
	Point3d pnt1 = null;
	Point3d pnt2 = null;
	Point3d lastSegmentPnt1 = null;
	Point3d lastSegmentPnt2 = null;
	Point3d pt0 = null;
	Point3d pt1 = null;
	Point3d pt2 = null;
	Point3d pt3 = null;
	ArrayList cellSection = null;
	int side1 = -1;
	int side2 = -1;

	//  For each corner of the cell:
	for (int cornerIndex = 0; cornerIndex < 4; cornerIndex++) {
		cornerCoord = getVertex(cornerIndex, x, y);
		corner[cornerIndex] = new Corner( cornerCoord );
		cellSection = contourCell.getIntersectionList(cornerIndex);
		if (cellSection != null) {
			for (int k = 0; k < cellSection.size(); k++) {
				//  for each intersection in this corner
				intersection = (Intersection)(cellSection.get(k));
				segment1 = intersection.getSegment1();
				segment2 = intersection.getSegment2();
				pnt1 = segment1.getPoint();
				pnt2 = segment2.getPoint();

				side1 = intersection.getSide1();
				side2 = intersection.getSide2();

				corner[cornerIndex].setEndPoints(side1, side2, pnt1, pnt2);

				if (0 == k) {
					defineCornerSegments( x, y, side1, side2,
										  pnt1, pnt2, cornerCoord, cornerCoord );
				}
				else {
					defineCornerSegments( x, y, side1, side2,
										  pnt1, pnt2, lastSegmentPnt1,
										  lastSegmentPnt2 );
				}
				lastSegmentPnt1 = pnt1;
				lastSegmentPnt2 = pnt2;
			}
		}
	}
	//  Now handle vertical or horizontal contour segments:
	cellSection = contourCell.getIntersectionList(ContourCell.SIDE);
	if (cellSection != null) {
		for (int k = 0; k < cellSection.size(); k++) {
			//  for each intersection through this side
			intersection = (Intersection)(cellSection.get(k));
			segment1 = intersection.getSegment1();
			segment2 = intersection.getSegment2();
			pnt1 = segment1.getPoint();
			pnt2 = segment2.getPoint();

			side1 = intersection.getSide1();
			side2 = intersection.getSide2();

			if (0 == k) {
				if (2 == side1) {   //  horizontal
					//  Define the region between the 1st horizontal
					//  contour segment and the cell corners adjacent to
					//  side 3.
					//  The 1st horizontal contour segment is always closest
					//  to side 3.  side1 is always 2.

					defineContourLineSegment(pnt1, pnt2);

					if (yMesh[x+1]) {
						if (corner[0].fIntersected) {
							pt0 = corner[0].point2;
						}
						else {
							pt0 = corner[0].point1;
						}
						defineLineSegment(pnt2, pt0);
					}

					if ((0==x) && (yMesh[x])) {
						pt3 = corner[3].point1;
						//  whether corner is intersected or not
					    defineLineSegment(pnt1, pt3);
					}

					if ((0==y) && (xMesh[y])) {
						pt0 = corner[0].point1;
						if (corner[3].fIntersected) {
							pt3 = corner[3].point2;
						}
						else {
							pt3 = corner[3].point1;   //  corner coordinate
						}
						//  whether corner is intersected or not
						defineLineSegment(pt3, pt0);
					}
				}
			    else {    //  vertical
					//  Define the region between the 1st vertical
					//  contour segment and the cell corners adjacent
					//  to side 0.
					//  The 1st vertical contour segment is always closest
					//  to side 0.  side1 is always 3.

					defineContourLineSegment(pnt1, pnt2);
					if (corner[1].fIntersected) {
						pt1 = corner[1].point2;
					}
					else {
					    pt1 = corner[1].point1;
					}
					if (xMesh[y+1]) {
						defineLineSegment(pnt2, pt1);
					}

					pt1 = corner[1].point1;   //  whether corner is intersected or not
					if (corner[0].fIntersected) {
						pt0 = corner[0].point2;
					}
					else {
						pt0 = corner[0].point1;
					}
					if (yMesh[x+1]) {
						defineLineSegment(pt0, pt1);
					}

					if (0==y) {
					    pt0 = corner[0].point1;  //  whether corner is intersected or not
					    if (xMesh[y]) {
							defineLineSegment(pnt1, pt0);
						}
					}
				}
			}
			else {
				defineContourLineSegment(pnt1, pnt2);
				if ( ((0==side2) && (yMesh[x+1])) ||
					 ((1==side2) && (xMesh[y+1])) ){
					defineLineSegment(pnt2, lastSegmentPnt2);
				}

				if (2==side1) {  //  horizontal
					if ((0==x) && (yMesh[x])) {
						defineLineSegment(pnt1, lastSegmentPnt1);
					}
				}
				else {  //  vertical
					if ((0==y) && (xMesh[y])) {
						defineLineSegment(pnt1, lastSegmentPnt1);
					}
				}
			}
			lastSegmentPnt1 = pnt1;
			lastSegmentPnt2 = pnt2;
		}
		//  Define the remainder of the cell.
		//  Since the intersection list for SIDE was not empty, create
		//  a polygon defined by lastSegmentPnt1, lastSegmentPnt2, and
		//  the remaining corners.

		//  From side1 of the last segment, identify the remaining
		//  corners and create a polygon chain:
		if (2 == side1) {   //  horizontal contours are always left-to-right
			//  Define the region between the last horizontal
			//  contour segment and the cell corners adjacent to
			//  side 1:

			//  draw part of side 0
			if (yMesh[x+1]) {
				pt1 = corner[1].point1;  //  whether intersected or not
				defineLineSegment(pnt2, pt1);
			}

			//  draw part of side 1
			if (xMesh[y+1]) {
				if (corner[1].fIntersected) {
					pt1 = corner[1].point2;
				}
				else {
					pt1 = corner[1].point1;
				}
				pt2 = corner[2].point1;  //  whether intersected or not
				defineLineSegment(pt2, pt1);
			}

			if ((0==x) && (yMesh[x])) {
				//  draw part of side 2
				if (corner[2].fIntersected) {
					pt2 = corner[2].point2;
				}
				else {
					pt2 = corner[2].point1;
				}
				defineLineSegment(pnt1, pt2);
			}
		}
		else {    //  vertical
			//  Define the region between the last vertical
			//  contour segment and the cell corners adjacent
			//  to side 2:

			//  draw part of side 1
			if (xMesh[y+1]) {
				pt2 = corner[2].point1;   //  whether intersected or not
				defineLineSegment(pnt2, pt2);
			}

			//  draw part of side 2
			if ((0==x) && (yMesh[x])) {
				if (corner[2].fIntersected) {
					pt2 = corner[2].point2;
				}
				else {
					pt2 = corner[2].point1;
				}
				pt3 = corner[3].point1;  //  whether intersected or not
				defineLineSegment(pt2, pt3);
			}
			//  draw part of side 3
			if ((0==y) && (xMesh[y])) {
				if (corner[3].fIntersected) {
					pt3 = corner[3].point2;
				}
				else {
					pt3 = corner[3].point1;
				}
				defineLineSegment(pt3, pnt1);
			}
		}
	}
	else {
		//  Define the remainder of the cell.
		//  Since the intersection list for SIDE is empty, create
		//  the polygon that remains after the corners have been processed.
		if (yMesh[x+1]) {
			if (corner[0].fIntersected) {
				pnt1 = corner[0].point2;
			}
			else {
				pnt1 = corner[0].point1;  //  the corner of the cell
			}
			pnt2 = corner[1].point1;  //  whether corner is intersected or not
			defineLineSegment(pnt1, pnt2);
		}

		if (xMesh[y+1]) {
			if (corner[1].fIntersected) {
				pnt1 = corner[1].point2;
			}
			else {
				pnt1 = corner[1].point1;  //  the corner of the cell
			}
			pnt2 = corner[2].point1;  //  whether corner is intersected or not
			defineLineSegment(pnt1, pnt2);
		}

		if ((0==x) && (yMesh[x])) {
			if (corner[2].fIntersected) {
				pnt1 = corner[2].point2;
			}
			else {
				pnt1 = corner[2].point1;  //  the corner of the cell
			}
			pnt2 = corner[3].point1;  //  whether corner is intersected or not
			defineLineSegment(pnt1, pnt2);
		}

		if ((0==y) && (xMesh[y])) {
			if (corner[3].fIntersected) {
				pnt1 = corner[3].point2;
			}
			else {
				pnt1 = corner[3].point1;  //  the corner of the cell
			}
			pnt2 = corner[0].point1;   //  whether intersected or not
			defineLineSegment(pnt1, pnt2);
		}
	}
}

private void defineCornerSegments( int cellColumn, int cellRow,
								   int side1, int side2,
								   Point3d pnt1, Point3d pnt2,
								   Point3d lastPnt1, Point3d lastPnt2 )
{
	defineContourLineSegment(pnt1, pnt2);
	if ((0==side2) && yMesh[cellColumn+1]) {    //  corner 0
		defineLineSegment(pnt2, lastPnt2);
	}
	if (1==side2) {    //  corner 1
		if (xMesh[cellRow+1]) {
			defineLineSegment(pnt2, lastPnt2);
		}
		if (yMesh[cellColumn+1]) {
			defineLineSegment(pnt1, lastPnt1);
		}
	}
	if (1==side1) {    // corner 2
		if (yMesh[cellColumn]) {
			defineLineSegment(pnt2, lastPnt2);
		}
		if (xMesh[cellRow+1]) {
			defineLineSegment(pnt1, lastPnt1);
		}
	}
	if (0==cellColumn) {
		if ((2==side1) && yMesh[cellColumn]) {
			defineLineSegment(pnt1, lastPnt1);
		}
		if ((2==side2) && yMesh[cellColumn]) {
			defineLineSegment(pnt2, lastPnt2);
		}
	}
	if (0==cellRow) {
		if ((3==side1) && xMesh[cellRow]) {
			defineLineSegment(pnt1, lastPnt1);
		}
		if ((3==side2) && xMesh[cellRow]) {
			defineLineSegment(pnt2, lastPnt2);
		}
	}
}

//  Returns the average z value.
private double getMeanZValue(Point3d vert1, Point3d vert2)
{
	return ( (vert1.z + vert2.z)/2.0 );
}

//  Returns the color associated with the z parameter.
private Color3f getContourColor3f(double z)
{
	int level = contourLevels.getLevelFromValue(z);
	int contourStyleIndex = contour.contourIndex(level);
	JCContourStyle contourStyle =
				     (JCContourStyle)contourStyles.get(contourStyleIndex);
	JCFillStyle fillStyle = contourStyle.getFillStyle();
	Color color = fillStyle.getColor();
	return (new Color3f(color));
}

private void defineLineSegment(Point3d v1, Point3d v2)
{
    if ( (v1.z == holeValue) || (v2.z == holeValue) ) {
        return;
    }

	Point3d scaledV1 = new Point3d(v1.x, v1.y, v1.z);
	absolute.transform(scaledV1);
	Point3d scaledV2 = new Point3d(v2.x, v2.y, v2.z);
	absolute.transform(scaledV2);

	gridPntList.add(scaledV1);
	gridPntList.add(scaledV2);

	Color3f clr = getContourColor3f( getMeanZValue(v1, v2) );

	gridClrList.add(clr);
	gridClrList.add(clr);
}

private void defineContourLineSegment(Point3d v1, Point3d v2)
{
    //  For contour lines, there shouldn't be any hole values, since
    //  those are handled by the chart3d methods that generate the
    //  ContourCell data.
	Point3d scaledV1 = new Point3d(v1.x, v1.y, v1.z);
	absolute.transform(scaledV1);
	Point3d scaledV2 = new Point3d(v2.x, v2.y, v2.z);
	absolute.transform(scaledV2);

	double zValue = getMeanZValue(v1, v2);
	int level = contourLevels.getLevelFromValue(zValue);

	contourPntList[level].add(scaledV1);
	contourPntList[level].add(scaledV2);

	Color3f clr = getContourColor3f(zValue);

	contourClrList[level].add(clr);
	contourClrList[level].add(clr);
}

public Geometry getMeshGeometry()
{
	return meshLineArray;
}

public LineArray[] getContourGeometry()
{
	return contourLineArray;
}

}
