/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataSurfaceQA.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:05 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.j3d;

import java.awt.Color;
import java.util.ArrayList;
import com.sun.j3d.utils.picking.behaviors.*;
import com.sun.j3d.utils.picking.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.klg.jclass.chart3d.*;

public class DataSurfaceQA {

private JCContour contour = null;
private JCContourLevels contourLevels = null;
private ArrayList contourStyles = null;

private QuadArray quadArray = null;

public DataSurfaceQA( Matrix4d absolute, Chart3dGridData data,
				      Color3f color, JCContour contour )
{
	if (contour != null) {
		this.contour = contour;
		contourStyles = contour.getContourStyles();
		this.contourLevels = contour.getContourLevels();
	}

	GridLimits gridLimits = data.getGridLimits();
	double[] xGrid = data.getXGrid();
	double[] yGrid = data.getYGrid();
	double[][] zValues = data.getZValues();
	double holeValue = data.getHoleValue();
	int nxQuads = gridLimits.getNumX() - 1;
	int nyQuads = gridLimits.getNumY() - 1;
	Point3d  verts[][] = new Point3d[nxQuads + 1][nyQuads + 1];
	TexCoord2f texCoords[][] = new TexCoord2f[nxQuads + 1][nyQuads + 1];

	int initX = gridLimits.getXOffset();
	int initY = gridLimits.getYOffset();
	for (int i = 0; i < (nxQuads + 1); i++) {
		for (int j = 0; j < (nyQuads + 1); j++) {
			Point3d pnt = new Point3d(xGrid[initX + i],
									  yGrid[initY + j],
									  zValues[initX + i][initY + j]);
			verts[i][j] = new Point3d(pnt);
			texCoords[i][j] = new TexCoord2f( (float)i/(float)(nxQuads),
										      1.0f - (float)j/(float)(nyQuads) );
		}
	}

    ArrayList pntList = new ArrayList();
    ArrayList clrList = new ArrayList();
    ArrayList txtrList = new ArrayList();

	Color3f cellColor = null;
	int vertexCount = 0;
	for (int i = 0; i < (nxQuads); i++)  //  for each column in the grid
	{
		for (int j = 0; j < (nyQuads); j++)  //  for each row in the grid
		{
			//  Define a set of quadrilaterals to show the frontface.
			try {
                if (containsHoleValue(initX + i, initY + j,
                                      zValues, holeValue) == false) {
			        //  Quartets of vertices defining the quadrilaterals in our surface:
			        pntList.add( new Point3d(verts[i][j]) );
			        txtrList.add(texCoords[i][j]);

			        pntList.add( new Point3d(verts[i+1][j]) );
			        txtrList.add(texCoords[i+1][j]);

			        pntList.add( new Point3d(verts[i+1][j+1]) );
			        txtrList.add(texCoords[i+1][j+1]);

			        pntList.add( new Point3d(verts[i][j+1]) );
			        txtrList.add(texCoords[i][j+1]);

			        if ((contour != null) && (contour.getZoneMethod() == JCContour.ZONE_CELLS_AVERAGE)) {
						//  Take average z value of all four vertices, and look up the zone color
						cellColor = colorAverageZValue(verts, i, j);
					}
					else if ((contour != null) && (contour.getZoneMethod() == JCContour.ZONE_CELLS_CORNER)) {
						//  Use the z value for the lower left corner of the cell
						cellColor = getContourColor3f(verts[i][j].z);
					}
					else {
						//  Use the color parameter that was passed in.
						cellColor = color;
					}

					for (int k=0; k < 4; k++) {
						clrList.add(new Color3f(cellColor));
					}
                }
			}
			catch (ArrayIndexOutOfBoundsException e) {
				System.out.println("Caught an ArrayIndexOutOfBoundsException!");
			}
		}
	}

    int nPoints = pntList.size();
	Point3d pnts[] = new Point3d[nPoints];
    pntList.toArray(pnts);
	TexCoord2f txtr[] = new TexCoord2f[nPoints];
    txtrList.toArray(txtr);
	Color3f clrs[] = new Color3f[nPoints];
	clrList.toArray(clrs);

	for (int i=0; i < nPoints; i++) {
		absolute.transform(pnts[i]);
	}

	float[] textureCoord = new float[2 * nPoints];
	int k = 0;
	for (int i=0; i < nPoints; i++) {
		textureCoord[k++] = txtr[i].x;
		textureCoord[k++] = txtr[i].y;
	}

	GeometryInfo gi = new GeometryInfo(GeometryInfo.QUAD_ARRAY);
	gi.setCoordinates(pnts);
	gi.setColors(clrs);
	gi.setTextureCoordinates2(textureCoord);
	NormalGenerator ng = new NormalGenerator();
	ng.generateNormals(gi);

	quadArray = (QuadArray)gi.getGeometryArray();
}


private boolean containsHoleValue(int x, int y, double[][] zValue,
                                  double holeValue) {

    boolean containsHole = ( (zValue[x][y] == holeValue)     ||
                             (zValue[x+1][y] == holeValue)   ||
                             (zValue[x+1][y+1] == holeValue) ||
                             (zValue[x][y+1] == holeValue) );
    return containsHole;
}


//  Returns the color associated with the z parameter.
private Color3f getContourColor3f(double z)
{
	if (contour == null) {
		return null;
	}
	int level = contourLevels.getLevelFromValue(z);
	int contourStyleIndex = contour.contourIndex(level);
	JCContourStyle contourStyle =
				     (JCContourStyle)contourStyles.get(contourStyleIndex);
	JCFillStyle fillStyle = contourStyle.getFillStyle();
	Color color = fillStyle.getColor();
	return (new Color3f(color));
}


private Color3f colorAverageZValue(Point3d verts[][], int x, int y)
{
	double averageZ = (verts[x][y].z + verts[x+1][y].z + verts[x+1][y+1].z + verts[x][y+1].z) / 4.0;
	return getContourColor3f(averageZ);
}


public GeometryArray getGeometryArray() {
    return quadArray;
}

}
