/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataSurfaceTA.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:05 $  $Locker:  $  Quest Software Inc.

//  DataSurfaceTA.java

package com.klg.jclass.chart3d.j3d;

import java.awt.Color;
import java.util.ArrayList;
import com.sun.j3d.utils.picking.behaviors.*;
import com.sun.j3d.utils.picking.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.klg.jclass.chart3d.*;

public class DataSurfaceTA {

private TriangleStripArray triangleStripArray = null;
private ArrayList pntList = new ArrayList();
private ArrayList clrList = new ArrayList();
private JCContour contour = null;
private JCContourLevels contourLevels = null;
private ArrayList contourStyles = null;
private Chart3dGridData zoneData = null;

private double[] xGrid = null;
private double[] yGrid = null;
private double[][] zValues = null;
private double holeValue = Double.MAX_VALUE;

private Matrix4d absolute = null;

public DataSurfaceTA( Matrix4d absolute,
				      Chart3dGridData data,
					  JCContour contour )
{
	GridLimits gridLimits = data.getGridLimits();

	this.zoneData = data.getDataView().getZoneData();
	this.absolute = absolute;
	this.xGrid = data.getXGrid();
	this.yGrid = data.getYGrid();
	this.zValues = data.getZValues();
    this.holeValue = data.getHoleValue();

	contour.calcContours();
	this.contour = contour;
	contourStyles = contour.getContourStyles();
	ContourCell[] contourCells = contour.getContourCells();
	this.contourLevels = contour.getContourLevels();

	ContourCell contourCell = null;
	Point3d vertex0 = null;
	Point3d vertex1 = null;
	Point3d vertex2 = null;
	Point3d vertex3 = null;
	int x, y, nIntersections;

	int xOffset = gridLimits.getXOffset();
	int yOffset = gridLimits.getYOffset();
	int xLast = gridLimits.getXLast();
	int yLast = gridLimits.getYLast();
	int cellIndex = 0;
	for (x = xOffset; x < xLast; x++) {
		for (y = yOffset; y < yLast; y++) {
		    contourCell = contourCells[cellIndex];
            if (containsHoleValue(x, y, holeValue) == false) {
                nIntersections = contourCell.getNIntersections();
		    	if (nIntersections == 0) {
		    	    //  Handle the case where there are no contour segments
		    	    //  in the grid cell:
		    		vertex0 = new Point3d( xGrid[x + 1], yGrid[y], zValues[x + 1][y] );
		    		vertex1 = new Point3d( xGrid[x + 1], yGrid[y + 1], zValues[x + 1][y + 1] );
		    		vertex2 = new Point3d( xGrid[x], yGrid[y + 1], zValues[x][y + 1] );
		    		vertex3 = new Point3d( xGrid[x], yGrid[y], zValues[x][y] );
		    		defineTriangle(vertex3, vertex0, vertex1);
		    		defineTriangle(vertex3, vertex1, vertex2);
		    	}
		    	else {
		           triangulateCell(contourCell, contour, x, y);
		        }
            }
			cellIndex++;
		}
	}

	if (pntList.size() == 0) {
		return;
	}

	Point3d[] pnts = new Point3d[pntList.size()];
	pntList.toArray(pnts);
	Color3f[] clrs = new Color3f[clrList.size()];
	clrList.toArray(clrs);

	GeometryInfo gi = new GeometryInfo(GeometryInfo.TRIANGLE_ARRAY);
	gi.setCoordinates(pnts);
	gi.setColors(clrs);
	NormalGenerator ng = new NormalGenerator();
	ng.generateNormals(gi);
	Stripifier st = new Stripifier();
	st.stripify(gi);
	triangleStripArray = (TriangleStripArray)gi.getGeometryArray();
}

private boolean containsHoleValue(int x, int y, double holeValue) {
    Point3d cornerValue = null;
    //  See if any of the cell corners is a hole value
	for (int cornerIndex = 0; cornerIndex < 4; cornerIndex++) {
        cornerValue = getVertex(cornerIndex, x, y);
        if (cornerValue.z == holeValue) {
            return true;
        }
    }
    return false;
}

//  Note:  returned value is unscaled coordinate.
private Point3d getVertex(int cornerIndex, int x, int y) {
	Point3d vertex = null;

	switch(cornerIndex)
	{
	case 0:
		vertex = new Point3d( xGrid[x + 1], yGrid[y], zValues[x + 1][y] );
		break;
	case 1:
		vertex = new Point3d( xGrid[x + 1], yGrid[y + 1], zValues[x + 1][y + 1] );
		break;
	case 2:
		vertex = new Point3d( xGrid[x], yGrid[y + 1], zValues[x][y + 1] );
		break;
	case 3:
		vertex = new Point3d( xGrid[x], yGrid[y], zValues[x][y] );
		break;
	default:
		System.out.println(
		"DataSurfaceTA.triangulateCell():  Invalid index for cell corner.");
	}
	return vertex;
}

private void triangulateCell( ContourCell contourCell, JCContour contour,
						      int x, int y ) {
	Intersection intersection = null;
	ContourSegment segment1 = null;
	ContourSegment segment2 = null;
	Corner[] corner = new Corner[4];
	Point3d cornerCoord = null;
	Point3d pnt1 = null;
	Point3d pnt2 = null;
	Point3d lastSegmentPnt1 = null;
	Point3d lastSegmentPnt2 = null;
	ArrayList cellSection = null;
	Point3d pivot = null;
	Point3d vertex2 = null;
	Point3d vertex3 = null;
	int side1 = -1;
	int side2 = -1;

	//  For each corner of the cell:
	for (int cornerIndex = 0; cornerIndex < 4; cornerIndex++) {
		cornerCoord = getVertex(cornerIndex, x, y);
		corner[cornerIndex] = new Corner( cornerCoord );
		cellSection = contourCell.getIntersectionList(cornerIndex);
		if (cellSection != null) {
			for (int k = 0; k < cellSection.size(); k++) {
				//  for each intersection in this corner
				intersection = (Intersection)(cellSection.get(k));
				segment1 = intersection.getSegment1();
				segment2 = intersection.getSegment2();
				pnt1 = segment1.getPoint();
				pnt2 = segment2.getPoint();

				side1 = intersection.getSide1();
				side2 = intersection.getSide2();

				corner[cornerIndex].setEndPoints(side1, side2, pnt1, pnt2);

				if (0 == k) {
					defineTriangle(pnt1, cornerCoord, pnt2);
				}
				else {
					defineTriangle(pnt1, lastSegmentPnt1, pnt2);
					defineTriangle(pnt2, lastSegmentPnt1, lastSegmentPnt2);
				}
				lastSegmentPnt1 = pnt1;
				lastSegmentPnt2 = pnt2;
			}
		}
	}
	//  Now handle vertical or horizontal contour segments:
	cellSection = contourCell.getIntersectionList(ContourCell.SIDE);
	if (cellSection != null) {
		for (int k = 0; k < cellSection.size(); k++) {
			//  for each intersection through this side
			intersection = (Intersection)(cellSection.get(k));
			segment1 = intersection.getSegment1();
			segment2 = intersection.getSegment2();
			pnt1 = segment1.getPoint();
			pnt2 = segment2.getPoint();

			side1 = intersection.getSide1();
			side2 = intersection.getSide2();

			if (0 == k) {
				ArrayList polygonVertices = new ArrayList();
				if (2 == side1) {   //  horizontal
					//  Triangulate the region between the 1st horizontal
					//  contour segment and the cell corners adjacent to
					//  side 3.
					//  The 1st horizontal contour segment is always closest
					//  to side 3.  side1 is always 2.

					//  First, create a polygon chain from the corner[] data:
				    for (int m = 3; m >= 0; m-=3) {
					    polygonVertices.add(corner[m].point1);
					    if ( corner[m].fIntersected ) {
						    polygonVertices.add(corner[m].point2);
					    }
				    }
				}
			    else {    //  vertical
					//  Triangulate the region between the 1st vertical
					//  contour segment and the cell corners adjacent
					//  to side 0.
					//  The 1st vertical contour segment is always closest
					//  to side 0.  side1 is always 3.

					//  First, create a polygon chain from the corner[] data:
				    for (int m = 0; m < 2; m++) {
					    polygonVertices.add(corner[m].point1);
					    if ( corner[m].fIntersected ) {
						    polygonVertices.add(corner[m].point2);
					    }
				    }
				}
				triangulateRegion(pnt2, pnt1, polygonVertices);
			}
			else {
				defineTriangle(pnt2, pnt1, lastSegmentPnt1);
				defineTriangle(pnt2, lastSegmentPnt1, lastSegmentPnt2);
			}
			lastSegmentPnt1 = pnt1;
			lastSegmentPnt2 = pnt2;
		}
		//  Triangulate the remainder of the cell.
		//  Since the intersection list for SIDE was not empty, triangulate
		//  the polygon defined by lastSegmentPnt1, lastSegmentPnt2, and
		//  the remaining corners.

		//  From side1 of the last segment, identify the remaining
		//  corners and create a polygon chain:
		if (2 == side1) {   //  horizontal contours are always left-to-right
			//  Triangulate the region between the last horizontal
			//  contour segment and the cell corners adjacent to
			//  side 1:

			//  First, create a polygon chain from the corner[] data:
			ArrayList polygonVertices = new ArrayList();
			for (int m = 1; m <= 2; m++) {
				polygonVertices.add(corner[m].point1);
				if ( corner[m].fIntersected ) {
					polygonVertices.add(corner[m].point2);
				}
			}
			triangulateRegion(pnt1, pnt2, polygonVertices);
		}
		else {    //  vertical
			//  Triangulate the region between the last vertical
			//  contour segment and the cell corners adjacent
			//  to side 2:

			//  First, create a polygon chain from the corner[] data:
			ArrayList polygonVertices = new ArrayList();
			for (int m = 2; m <= 3; m++) {
				polygonVertices.add(corner[m].point1);
				if ( corner[m].fIntersected ) {
					polygonVertices.add(corner[m].point2);
				}
			}
			triangulateRegion(pnt1, pnt2, polygonVertices);
		}
	}
	else {
		//  Triangulate the remainder of the cell.
		//  Since the intersection list for SIDE is empty, triangulate
		//  the polygon that remains after the corners have been processed.

		//  Create a polygon chain from the corner[] data:
		ArrayList polygonVertices = new ArrayList();
		for (int m = 0; m < 4; m++) {
			polygonVertices.add(corner[m].point1);
			if ( corner[m].fIntersected ) {
				polygonVertices.add(corner[m].point2);
			}
		}
		pivot = (Point3d)polygonVertices.get(0);
		triangulatePolygon(pivot, polygonVertices, 1);  // yes, 1
	}
}

private void triangulateRegion(Point3d pivot, Point3d vertex2,
							   ArrayList polygonVertices) {
	Point3d vertex3 = (Point3d)polygonVertices.get(0);
	defineTriangle(pivot, vertex2, vertex3);
	triangulatePolygon(pivot, polygonVertices, 0);
}

private void triangulatePolygon(Point3d pivot, ArrayList polygonVertices,
								int startIndex) {
	Point3d vertex2 = null;
	Point3d vertex3 = null;

	for (int m = startIndex; m < (polygonVertices.size() - 1); m++) {
		vertex2 = (Point3d)polygonVertices.get(m);
		vertex3 = (Point3d)polygonVertices.get(m+1);
		defineTriangle(pivot, vertex2, vertex3);
	}
}

//  Returns the average z value for the three coordinate parameters.
private double getMeanZValue(Point3d vert1, Point3d vert2, Point3d vert3) {
	return ( (vert1.z + vert2.z + vert3.z)/3.0 );
}

//  Returns the color associated with the z parameter.
private Color3f getContourColor3f(double z) {
	int level = contourLevels.getLevelFromValue(z);
	int contourStyleIndex = contour.contourIndex(level);
	JCContourStyle contourStyle =
				     (JCContourStyle)contourStyles.get(contourStyleIndex);
	JCFillStyle fillStyle = contourStyle.getFillStyle();
	Color color = fillStyle.getColor();
	return (new Color3f(color));
}

private void defineTriangle(Point3d v1, Point3d v2, Point3d v3) {
	Color3f clr = null;
	Point3d pnt1 = null;
	Point3d pnt2 = null;
	Point3d pnt3 = null;

	Point3d scaledV1 = new Point3d(v1.x, v1.y, v1.z);
	absolute.transform(scaledV1);
	Point3d scaledV2 = new Point3d(v2.x, v2.y, v2.z);
	absolute.transform(scaledV2);
	Point3d scaledV3 = new Point3d(v3.x, v3.y, v3.z);
	absolute.transform(scaledV3);

	pntList.add(scaledV1);
	pntList.add(scaledV2);
	pntList.add(scaledV3);

	if (zoneData.isDataOK() && contour.isZoned()) {
		//  get 4D color
		pnt1 = new Point3d(0., 0., getZoneDataZValue(v1));
		pnt2 = new Point3d(0., 0., getZoneDataZValue(v2));
		pnt3 = new Point3d(0., 0., getZoneDataZValue(v3));
	}
	else {
		//  use elevation data
		pnt1 = v1;
		pnt2 = v2;
		pnt3 = v3;
	}
	clr = getContourColor3f( getMeanZValue(pnt1, pnt2, pnt3) );
	for (int i=0; i<3; i++) {
	    clrList.add(clr);
	}
}

public TriangleStripArray getGeometryArray() {
	return triangleStripArray;
}

private double interpolate(double v, double v1, double v2, double z1, double z2)
{
	double slope = (z2 - z1) / (v2 - v1);
	return ((v - v1) * slope + z1);
}

protected boolean isEqual(double value1, double value2, double error)
{
	if ( (value1 >= (value2 - error)) && (value1 <= (value2 + error)) ) {
		return true;
	}
	return false;
}

private double getZoneDataZValue(Point3d pnt) {
	double zoneDataZValue = zoneData.getHoleValue();
	int xGridLength = zoneData.getXGrid().length;
	int yGridLength = zoneData.getYGrid().length;
	double[] zoneXGrid = zoneData.getXGrid();
	double[] zoneYGrid = zoneData.getYGrid();
	double error = 0.000000001;
	int xIndex = zoneData.getPointIndex(pnt.x + Math.abs(pnt.x * error), zoneXGrid, xGridLength);
	int yIndex = zoneData.getPointIndex(pnt.y + Math.abs(pnt.y * error), zoneYGrid, yGridLength);

//	Three possible cases:
//	(a) pnt.x == vertex.x, pnt.y == vertex.y -- use vertex.z
//	(b) pnt.x == vertex.x, pnt.y  > vertex.y -- interpolate between zValue(xIndex,yIndex) and zValue(xIndex,  yIndex+1)
//	(c) pnt.x  > vertex.x, pnt.y == vertex.y -- interpolate between zValue(xIndex,yIndex) and zValue(xIndex+1,yIndex  )

	do {
		zoneDataZValue = zoneData.getZValue(xIndex, yIndex);

		if (isEqual(pnt.x, zoneXGrid[xIndex], pnt.x * error) && isEqual(pnt.y, zoneYGrid[yIndex], pnt.y * error)) {
			break;
		}

		if (isEqual(pnt.x, zoneXGrid[xIndex], pnt.x * error)) {
			if (yIndex < yGridLength-1) {
				zoneDataZValue = interpolate(pnt.y, zoneYGrid[yIndex], zoneYGrid[yIndex+1],
											 zoneData.getZValue(xIndex, yIndex),
											 zoneData.getZValue(xIndex, yIndex+1));
			}
			break;
		}

		if (xIndex < xGridLength-1) {
			zoneDataZValue = interpolate(pnt.x, zoneXGrid[xIndex], zoneXGrid[xIndex+1],
										 zoneData.getZValue(xIndex,   yIndex),
										 zoneData.getZValue(xIndex+1, yIndex));
		}
	} while (false);

	return zoneDataZValue;
}

}
