/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DrawGridJava3d.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:05 $  $Locker:  $  Quest Software Inc.

//  DrawGridJava3d.java

package com.klg.jclass.chart3d.j3d;

import java.awt.Graphics;
import javax.vecmath.*;
import com.klg.jclass.chart3d.*;

/**
 *
 */
public class DrawGridJava3d extends DrawGrid {

/**
 * Basic constructor.
 */
protected DrawGridJava3d()
{
	super();
}

/**
 * Internal constructor specifying parent data view.
 */
protected DrawGridJava3d(Chart3dDataView dataView)
{
	super(dataView);
}

/**
 *  Create Java3D scene graph objects based on the grid data.
 */
public void draw()
{
	int              type = dataView.getChartType();
	JCChart3dAreaJava3d   area = (JCChart3dAreaJava3d)dataView.getChart3d().getChart3dArea();
	JCElevation      elevation = dataView.getElevation();
	JCSurface        surface = area.getSurface();
	JCBar            bar = area.getBar();
    Chart3dGridData  data = getGridData();

    int numX = data.getNumX();
    int numY = data.getNumY();
	if (type == JCChart3d.SURFACE) {
		if (numX < 2 || numY < 2) {
			return;
		}
	}
	else if (type == JCChart3d.BAR) {
		if ((bar.getXFormat() == JCBar.HISTOGRAM && numX < 2) ||
		    (bar.getYFormat() == JCBar.HISTOGRAM && numY < 2))
		{
			return;
		}
	}
	else {	// JCChart3d.SCATTER_PLOT
		if (numX < 1 || numY < 1) {
			return;
		}
	}

	if ((JCPlotCubeJava3d)area.getPlotCube() != null) {
	    ((JCPlotCubeJava3d)area.getPlotCube()).removeAllDataGroups();
	}

	switch (dataView.getChartType()) {
	case JCChart3d.SURFACE:
		createDataSurface(area);
		break;
	case JCChart3d.BAR:
		createDataBars(area);
		break;
	case JCChart3d.SCATTER:
		createScatterPoints(area);
		break;
	default:
		//  Shouldn't get here.
	}
}

private void createDataBars(JCChart3dAreaJava3d area)
{
	Matrix4d absolute = area.getTransform().getAbsolute();
	Chart3dGridData gd = getGridData();
	JCBar barFormat = area.getBar();
	GridLimits gridLimits = gd.getGridLimits();

	if (dataView.getElevation().isTransparent()) {
		if (dataView.getElevation().isMeshed()) {
			if (dataView.getContour().isZoned()) {
				if (dataView.getZoneData().isDataOK()) {
					//  Show entire outline in topmost zone color if we have 4D data
					JCMeshDataBars meshDataBars = new JCMeshDataBars(barFormat,
									absolute,
									gd,
									false,
									new Color3f(area.getForeground()),
									false);
					//  Show entire outline in topmost zone color:
					meshDataBars.showDataBarsZoned(true);
					((JCPlotCubeJava3d)area.getPlotCube()).addDataGroup(meshDataBars);
				}
				else {
					//  Show outline with zone colors.
					JCZonedMeshDataBars zonedMeshDataBars =
						    new JCZonedMeshDataBars(barFormat,
									absolute,
									gd,
									false,
									new Color3f(area.getForeground()));
					zonedMeshDataBars.showDataBarsZoned(true);
					((JCPlotCubeJava3d)area.getPlotCube()).addDataGroup(zonedMeshDataBars);
				}
			}
			else {
				// Mesh lines are not zoned.
				JCMeshDataBars meshDataBars = new JCMeshDataBars(barFormat,
		    	    			    absolute,
									gd,
									false,
		    	    				new Color3f(area.getForeground()),
								    false);
				meshDataBars.showDataBarsZoned(false);
				((JCPlotCubeJava3d)area.getPlotCube()).addDataGroup(meshDataBars);
			}
		}
	}
	else {   //  Solid bars
		if (dataView.getContour().isZoned()) {
			if (dataView.getZoneData().isDataOK()) {
				//  Show entire outline in topmost zone color if we have 4D data
				JCMeshDataBars meshDataBars = new JCMeshDataBars(barFormat,
									absolute,
									gd,
									false,
									new Color3f(area.getForeground()),
									true);
				//  Show entire bar in topmost zone color:
				meshDataBars.showDataBarsZoned(true);
		    	meshDataBars.setDataBarsLit(dataView.getElevation().isShaded());
				((JCPlotCubeJava3d)area.getPlotCube()).addDataGroup(meshDataBars);
			}
			else {
				//  Show zone colors on solid data bars.
	        	JCZonedDataBars zonedDataBars =
			    	new JCZonedDataBars(barFormat,
									absolute,
									gd,
									false,
									new Color3f(area.getForeground()),
									true);
	        	zonedDataBars.showDataBarsZoned(dataView.getContour().isZoned());
		    	zonedDataBars.setDataBarsLit(dataView.getElevation().isShaded());
	        	((JCPlotCubeJava3d)area.getPlotCube()).addDataGroup(zonedDataBars);
			}
		}
		else {
			// The solid data bars are not zoned.  Pass true for the fSolid
			// parameter of JCMeshDataBars to create a set of solid bars.
	        JCMeshDataBars meshDataBars = new JCMeshDataBars(barFormat,
		        				    absolute,
									gd,
									false,
		        					new Color3f(area.getForeground()),
			        				true);
	        meshDataBars.showDataBarsZoned(false);
		    meshDataBars.setDataBarsLit(dataView.getElevation().isShaded());
	        ((JCPlotCubeJava3d)area.getPlotCube()).addDataGroup(meshDataBars);
		}
		if (dataView.getElevation().isMeshed()) {
			// The solid data bars are meshed.  Create a set of mesh bars in
			// the chart area's foreground color. (We pass false for the fSolid
			// parameter of JCMeshdDataBars.)
	        JCMeshDataBars meshDataBars = new JCMeshDataBars(barFormat,
		        				    absolute,
									gd,
									true,
		        					new Color3f(area.getForeground()),
								    false);
			//  show edges of bar in a single color
	        meshDataBars.showDataBarsZoned(false);
	        ((JCPlotCubeJava3d)area.getPlotCube()).addDataGroup(meshDataBars);
		}
	}
	if ( dataView.getContour().isContoured() ) {
		// Create an outline QuadArray to represent the data bar contours.
		ContourDataBarsQA contours =
			    new ContourDataBarsQA(barFormat,
									  absolute,
									  gd,
									  false,
									  new Color3f(area.getForeground()),
									  false);
	    contours.showDataBarsZoned(dataView.getContour().isZoned() &&
								   dataView.getElevation().isTransparent());
	    ((JCPlotCubeJava3d)area.getPlotCube()).addDataGroup(contours);
	}
}

private void createScatterPoints(JCChart3dAreaJava3d area)
{
	Matrix4d absolute = area.getTransform().getAbsolute();
	Chart3dGridData gd = getGridData();

    JCDataPoints dataPoints = new JCDataPoints(absolute,
										       gd,
                                               dataView.getContour(),
                                               new Color3f(area.getForeground()),
                                               dataView.getContour().isZoned());
    ((JCPlotCubeJava3d)area.getPlotCube()).addDataGroup(dataPoints);
}

private void createDataSurface(JCChart3dAreaJava3d area)
{
	Matrix4d absolute = area.getTransform().getAbsolute();
	Chart3dGridData gd = getGridData();
	JCZonedMeshSurface zonedMeshSurface = null;

    if ( dataView.getElevation().isMeshed() ||
	     dataView.getContour().isContoured() )
	{
	    zonedMeshSurface = new JCZonedMeshSurface(absolute,
									gd, dataView.getContour(),
									new Color3f(area.getForeground()));
	    zonedMeshSurface.setContourLinesVisible(dataView.getContour().isContoured());
	    zonedMeshSurface.setMeshSurfaceVisible(dataView.getElevation().isMeshed());
	    boolean fZoned = dataView.getContour().isZoned() &
	    					dataView.getElevation().isTransparent();
		zonedMeshSurface.setMeshSurfaceZoned(fZoned);
		zonedMeshSurface.setContourLinesZoned(fZoned);
        ((JCPlotCubeJava3d)area.getPlotCube()).addDataGroup(zonedMeshSurface);
    }

	if (!dataView.getElevation().isTransparent()) {
		JCSolidDataSurface solidDataSurface =
					    new JCSolidDataSurface(absolute,
									gd, dataView.getContour(),
									new Color3f(area.getForeground()));
	    solidDataSurface.setSolidSurfaceShaded(dataView.getElevation().isShaded());
	    solidDataSurface.setSolidSurfaceZoned(dataView.getContour().isZoned());
	    ((JCPlotCubeJava3d)area.getPlotCube()).addDataGroup(solidDataSurface);
	    if (zonedMeshSurface != null) {
			// If data surface is not transparent, allow picking of the solid
			// surface only.
			zonedMeshSurface.setMeshPickable(false);
			zonedMeshSurface.setContourLinesPickable(false);
		}
	}
}

}


