/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: InputBehavior.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:05 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.j3d;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.klg.jclass.chart3d.JCChart3d;

/**
 * This Behavior forwards mouse and keyboard events to the Chart 3D action handlers
 * (<code>RotateAction</code>, <code>TranslateAction</code>, <code>ScaleAction</code>,
 * <code>EditAction</code>, etc).
 */
public class InputBehavior extends Behavior {

protected JCChart3d chart3d;

protected WakeupCriterion[] conditions;
protected WakeupOr wakeupCondition;

public InputBehavior(JCChart3d chart3d, Bounds bounds) {
	super();
	this.chart3d = chart3d;
	this.setSchedulingBounds(bounds);
}

public void initialize() {

	conditions = new WakeupCriterion[7];
	conditions[0] = new WakeupOnAWTEvent(MouseEvent.MOUSE_DRAGGED);
	conditions[1] = new WakeupOnAWTEvent(MouseEvent.MOUSE_MOVED);
	conditions[2] = new WakeupOnAWTEvent(MouseEvent.MOUSE_PRESSED);
	conditions[3] = new WakeupOnAWTEvent(MouseEvent.MOUSE_RELEASED);
	conditions[4] = new WakeupOnAWTEvent(MouseEvent.MOUSE_CLICKED);
	conditions[5] = new WakeupOnAWTEvent(KeyEvent.KEY_PRESSED);
	conditions[6] = new WakeupOnAWTEvent(KeyEvent.KEY_RELEASED);
	wakeupCondition = new WakeupOr(conditions);

	wakeupOn(wakeupCondition);
}

public void setChart3d(JCChart3d chart3d) {
    this.chart3d = chart3d;
}

public void processStimulus (Enumeration criteria) {
	MouseEvent mouseEvent;
	KeyEvent keyEvent;
	WakeupCriterion wakeup;
	AWTEvent[] evt = null;

	while(criteria.hasMoreElements()) {
		wakeup = (WakeupCriterion)criteria.nextElement();
		if (wakeup instanceof WakeupOnAWTEvent) {
			evt = ((WakeupOnAWTEvent)wakeup).getAWTEvent();
		}

		if (evt[0] instanceof MouseEvent){
		    mouseEvent = (MouseEvent)evt[0];
		    switch (mouseEvent.getID()) {
		        case MouseEvent.MOUSE_CLICKED:
		            chart3d.mouseClicked(mouseEvent);
		            break;
		        case MouseEvent.MOUSE_PRESSED:
		            chart3d.mousePressed(mouseEvent);
		            break;
		        case MouseEvent.MOUSE_RELEASED:
		            chart3d.mouseReleased(mouseEvent);
		            break;
		        case MouseEvent.MOUSE_DRAGGED:
		            chart3d.mouseDragged(mouseEvent);
		            break;
		        case MouseEvent.MOUSE_MOVED:
		            chart3d.mouseMoved(mouseEvent);
		            break;
		    }
		}
		else if (evt[0] instanceof KeyEvent){
		    keyEvent = (KeyEvent)evt[0];
		    switch (keyEvent.getID()) {
		        case KeyEvent.KEY_PRESSED:
		            chart3d.keyPressed(keyEvent);
		            break;
		        case KeyEvent.KEY_RELEASED:
		            chart3d.keyReleased(keyEvent);
		            break;
		    }
		}
	}
	wakeupOn(wakeupCondition);
}

}

