/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JC3DUniverse.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:05 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.j3d;

import com.sun.j3d.utils.picking.*;
import java.util.Enumeration;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsEnvironment;
import java.awt.Font;
import java.awt.Color;
import java.util.BitSet;
import java.util.ArrayList;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;

/**
 * This class provides low-level methods to build the Java 3D scene graph.
 * An instance of this class is created in <code>JCChart3dUniverse</code>.
 */
public class JC3DUniverse {

private Locale locale = null;
private View view = null;

private boolean fEnableDepthCue = true;
private JCBranchGroup bgFog = null;
private Switch fogSwitch = null;
private LinearFog fog = null;
private double fogFrontDistance = 0.0;
private double fogBackDistance = 0.0;

private double viewPlatformDistance = 5.0;
private double fieldOfView = Math.toRadians(43.60);

private ArrayList lights = new ArrayList();
private JCBranchGroup bgLighting = null;

private JCCanvas3D myCanvas3D = null;
private BoundingSphere boundsphere = null;

protected JCTransformGroup tgViewPlatform = null;
private JCBranchGroup bgScene = null;
private JCTransformGroup objScale = null;
private JCBranchGroup bgContent = null;
private JCBranchGroup bgBackground = null;
private Background background = null;

/**
 * Creates the Java3D scene graph with view platform.
 */
public JC3DUniverse ()
{
	GraphicsConfigTemplate3D template = new GraphicsConfigTemplate3D();
	GraphicsConfiguration cfg = GraphicsEnvironment.getLocalGraphicsEnvironment().
		    getDefaultScreenDevice().getBestConfiguration(template);
	//  Some code examples pass a null parameter to the Canvas3D constructor.
	//  To maintain backward compatibility with earlier versions of Java3D
	//  this is still allowed, but a warning will be outputted.
	myCanvas3D = new JCCanvas3D(cfg);

	view = constructView(myCanvas3D);
	view.setSceneAntialiasingEnable(false);
	//  Runs very slowly when full scene antialiasing is enabled.
	locale = constructViewBranch(view);

	// Create a scene and attach it to the virtual universe
	bgScene = createSceneGraph(myCanvas3D);

//	locale.addBranchGraph(bgScene) to be invoked later.
}

private View constructView(Canvas3D myCanvas3D)
{
	View myView = new View();
	myView.addCanvas3D(myCanvas3D);
	myView.setPhysicalBody( new PhysicalBody() );
	myView.setPhysicalEnvironment( new PhysicalEnvironment() );
	myView.setFrontClipDistance(0.04);
	myView.setFieldOfView(fieldOfView);
	return myView;
}

private Locale constructViewBranch(View myView)
{
	VirtualUniverse myUniverse = new VirtualUniverse();
	Locale myLocale = new Locale(myUniverse);
	JCBranchGroup myBranchGroup = new JCBranchGroup();
	Transform3D t3d = new Transform3D();
	t3d.setTranslation( new Vector3d(0.0, 0.0, viewPlatformDistance) );
	tgViewPlatform = new JCTransformGroup(t3d);
	ViewPlatform myViewPlatform = new ViewPlatform();
	tgViewPlatform.addChild(myViewPlatform);
	myBranchGroup.addChild(tgViewPlatform);
	myLocale.addBranchGraph(myBranchGroup);
	myView.attachViewPlatform(myViewPlatform);

	return myLocale;
}

/**
 * Creates the Java3D scene graph.
 *
 * @param canvas the <code>Canvas3D</code> in which the chart is rendered
 * @return a <code>JCBranchGroup</code> containing the scene graph
 */
public JCBranchGroup createSceneGraph(Canvas3D canvas)
{
	// Create the root of the branch graph
	JCBranchGroup objRoot = new JCBranchGroup();

	// Create a Transformgroup to scale all objects so they
	// appear in the scene.
	objScale = new JCTransformGroup();

	Transform3D t3d = new Transform3D();
	t3d.setScale(1.0);
	objScale.setTransform(t3d);
	objRoot.addChild(objScale);

	bgContent = new JCBranchGroup();
	objScale.addChild(bgContent);

	boundsphere =
		new BoundingSphere(new Point3d(0.0,0.0,0.0), 100.0);

	//  Lighting added later.
	bgLighting = new JCBranchGroup();
	objScale.addChild(bgLighting);

	//  Fog object added later.
    bgFog = new JCBranchGroup();
	// Create a switch group to hold the fog node.  This enables
	// us to turn the fog node on and off.
	fogSwitch = new Switch();
	fogSwitch.setCapability(Switch.ALLOW_SWITCH_WRITE);
	fogSwitch.setCapability(Switch.ALLOW_SWITCH_READ);

	fogSwitch.addChild(bgFog);
	objRoot.addChild(fogSwitch);
	if ( fEnableDepthCue ) {
		fogSwitch.setWhichChild( Switch.CHILD_ALL );  // on
	}
	else {
		fogSwitch.setWhichChild( Switch.CHILD_NONE );  // off
	}

    //  bgScene.compile() to be invoked later.
	return objRoot;
}

protected JCCanvas3D getCanvas3D()
{
    return myCanvas3D;
}

/**
 * Returns the <code>BoundingSphere</code> associated with <code>Fog</code>,
 * <code>Light</code>, and <code>Background</code> objects in the virtual universe.
 *
 * @return the <code>BoundingSphere</code> object
 */
public BoundingSphere getBoundingSphere()
{
	return boundsphere;
}

protected JCTransformGroup getViewPlatformTransformGroup()
{
	return tgViewPlatform;
}

/**
 * Returns the <code>BranchGroup</code> containing the scene graph.
 * @return the <code>BranchGroup</code> containing the scene graph
 */
public JCBranchGroup getSceneBranchGroup()
{
	return bgScene;
}

//  The user should call the JCChart3dUniverse.setViewPlatformDistance() method instead.
//  That method updates the perspective.
public void setViewPlatformDistance(double vpDistance)
{
	viewPlatformDistance = vpDistance;
	Transform3D t3d = new Transform3D();
	tgViewPlatform.getTransform(t3d);
	Vector3d translation = new Vector3d();
	t3d.get(translation);
	translation.z = vpDistance;
	t3d.setTranslation(translation);
	tgViewPlatform.setTransform(t3d);
}

//  The user should call the JCChart3dUniverse.getViewPlatformDistance() method instead.
public double getViewPlatformDistance()
{
	return viewPlatformDistance;
}

protected Point3d getEyeCoordinate()
{
	return ( new Point3d(0.0, 0.0, viewPlatformDistance) );
	//  To do:  handle case where eye position is not at x=0, y=0
}

/**
 * Adds a <code>Fog</code> object to the scene graph.
 *
 * @param fog the <code>Fog</code> object to be added
 */
public void addFog(Fog fog)
{
	JCBranchGroup bg = new JCBranchGroup();
	bg.addChild(fog);
	bgFog.addChild(bg);
}

/**
 * Adds a <code>LinearFog</code> object to the scene graph.  Any existing <code>Fog</code> object is replaced.
 *
 * @param fogColor the color of the fog to be added
 */
public void addLinearFog(Color3f fogColor)
{
	//  First, remove any existing fog
	removeFog();

	JCBranchGroup bg = new JCBranchGroup();
	fog = new LinearFog();
	fog.setCapability(Fog.ALLOW_COLOR_READ);
	fog.setCapability(Fog.ALLOW_COLOR_WRITE);
	fog.setCapability(Fog.ALLOW_INFLUENCING_BOUNDS_WRITE);
	fog.setCapability(LinearFog.ALLOW_DISTANCE_READ);
	fog.setCapability(LinearFog.ALLOW_DISTANCE_WRITE);
	//  Allow fog distance to be changed.
	fog.setColor(fogColor);

	// Set front distance to the distance from the current
	// viewing position to the center of the bounds.
	fogFrontDistance = viewPlatformDistance - 0.75;
	fog.setFrontDistance(fogFrontDistance);
	//  distance of fog.front from view position

	// Set back distance to the distance from the current
	// viewing position to the back of the bounds.
	fogBackDistance = viewPlatformDistance + 3.0;
	fog.setBackDistance(fogBackDistance);
	//  distance of fog.back from view position

	fog.setInfluencingBounds(boundsphere);
	bg.addChild(fog);
	bgFog.addChild(bg);
}

/**
 * Removes any <code>Fog</code> object from the scene graph.
 */
public void removeFog()
{
    bgFog.removeAllChildren();
}

/**
 * Returns the last <code>Fog</code> object that was added.
 *
 * @return the last <code>Fog</code> object added to the scene graph
 */
public Fog getFog()
{
    return fog;
}

/**
 * Displays or hides the fog.
 *
 * @param flag <code>true</code> to show the fog, <code>false</code> to hide the fog
 */
public void enableFog(boolean flag)
{
	if (fog == null) {
		return;
	}
	fEnableDepthCue = flag;
	if ( fEnableDepthCue ) {
		fogSwitch.setWhichChild( Switch.CHILD_ALL );
	}
	else {
		fogSwitch.setWhichChild( Switch.CHILD_NONE );
	}
}

/**
 * Returns the display state of the fog.
 *
 * @return a boolean; <code>true</code> if the fog is visible, <code>false</code> if the fog is hidden
 */
public boolean isFogEnabled()
{
	return fEnableDepthCue;
}

/**
 * Sets the color of the fog.
 *
 * @param color3f the desired color of the fog
 */
public void setFogColor(Color3f color3f)
{
	if (fog != null) {
	    fog.setColor(color3f);
	}
}

/**
 * Returns the color of the fog.
 *
 * @param color3f the color of the fog
 */
public void getFogColor(Color3f color3f)
{
	if (fog != null) {
	    fog.getColor(color3f);
	}
}

/**
 * Returns the front distance of the fog.
 *
 * @return the front distance of the fog (in meters)
 */
public double getFogFrontDistance()
{
	return fogFrontDistance;
}

/**
 * Returns the back distance of the fog.
 *
 * @return the back distance of the fog (in meters)
 */
public double getFogBackDistance()
{
	return fogBackDistance;
}

/**
 * Sets the front distance of the fog.
 *
 * @param front the front distance of the fog (in meters)
 */
public void setFogFrontDistance(double front)
{
	fogFrontDistance = front;
	if (fog != null) {
	    fog.setFrontDistance((float)fogFrontDistance);
	}
}

/**
 * Sets the back distance of the fog.
 *
 * @param back the back distance of the fog (in meters)
 */
public void setFogBackDistance(double back)
{
	fogBackDistance = back;
	if (fog != null) {
	    fog.setBackDistance((float)fogBackDistance);
	}
}

/**
 * Changes the front and back distances of the fog by the specified value.
 *
 * @param z the value (in meters) by which the front and back distances are to be changed
 */
public void adjustFogDistance(double z)
{
	if (fog == null) {
		return;
	}
	fogFrontDistance += z;
	fogBackDistance += z;
	fog.setFrontDistance(fogFrontDistance);
	fog.setBackDistance(fogBackDistance);
}

/**
 * Adds a specified <code>Behavior</code> to the scene graph.
 * @param behavior the <code>Behavior</code> object to be added
 */
public void addBehavior(Behavior behavior)
{
	JCBranchGroup bgBehavior = new JCBranchGroup();
	bgBehavior.addChild(behavior);
	bgScene.addChild(bgBehavior);
}

/**
 * Adds a <code>BranchGroup</code> containing the 3D chart to the scene graph.
 *
 * @param bg the <code>BranchGroup</code> to be added
 */
public void addContent(JCBranchGroup bg)
{
	bg.compile();
	bgContent.addChild(bg);
}

/**
 * Removes the <code>BranchGroup</code> containing the 3D chart from the scene graph.
 */
public void removeAllContent()
{
    bgContent.removeAllChildren();
}

/**
 * Returns the <code>BranchGroup</code> containing the 3D chart.
 *
 * @return the <code>BranchGroup</code> containing the 3D chart
 */
public JCBranchGroup getContent()
{
	return bgContent;
}

/**
 * Adds a <code>Light</code> object to the scene graph.
 *
 * @param light the <code>Light</code> object to be added
 */
public void addLighting(Light light)
{
	JCBranchGroup bg = new JCBranchGroup();
	bg.addChild(light);
	bg.compile();
	bgLighting.addChild(bg);
    lights.add(light);
}

/**
 * Removes all <code>Light</code> objects from the scene graph.
 */
public void removeLighting()
{
    bgLighting.removeAllChildren();
    lights.clear();
}

/**
 * Returns an <code>ArrayList</code> containing references to all of the <code>Light</code> objects
 * in the scene graph.
 *
 * @return an <code>ArrayList</code> containing references to all <code>Light</code> objects
 */
public ArrayList getLighting()
{
	return lights;
}

/**
 * Adds an <code>AmbientLight</code> to the scene graph.
 */
public void addAmbientLight()
{
	AmbientLight light = new AmbientLight();
	light.setInfluencingBounds(boundsphere);
	light.setEnable(true);
	addLighting(light);
}

/**
 * Adds a <code>SpotLight</code> with default characteristics to the scene graph.
 */
public void addSpotLight()
{
	Color3f lColor = new Color3f(1.0f, 1.0f, 1.0f);
	Vector3f direction  = new Vector3f(0.0f, 0.0f, -1.0f);
	Point3f position = new Point3f(0.0f, 0.0f, 3.0f);
	Point3f attenuation = new Point3f(1.0f, 0.0f, 0.0f);
	float spreadAngle = (float)Math.PI;
	float concentration = 3.0f;

	JCSpotLight light = new JCSpotLight(lColor, position, attenuation,
										direction, spreadAngle, concentration);
	light.setInfluencingBounds(boundsphere);
	light.setEnable(true);
	addLighting(light);
}

/**
 * Adds a <code>DirectionalLight</code> with default characteristics to the scene graph.
 */
public void addDirectionalLight()
{
	Color3f clr3f = new Color3f(1.0f, 1.0f, 1.0f);
	Vector3f direction  = new Vector3f(0.0f, -1.0f, -1.0f);  // Arbitrary
	DirectionalLight light = new DirectionalLight(clr3f, direction);
	light.setInfluencingBounds(boundsphere);
	light.setEnable(true);
	addLighting(light);
}

/**
 * Adds a <code>PointLight</code> with default characteristics to the scene graph.
 */
public void addPointLight()
{
	Color3f lColor = new Color3f(1.0f, 1.0f, 1.0f);
	Point3f position = new Point3f(1.0f, 1.0f, 3.0f);  // Arbitrary
	Point3f attenuation = new Point3f(1.0f, 0.0f, 0.0f);

	JCPointLight light = new JCPointLight(lColor, position, attenuation);
	light.setInfluencingBounds(boundsphere);
	light.setEnable(true);
	addLighting(light);
}

/**
 * Adds a <code>Background</code> with default characteristics to the scene graph.
 */
public void addBackground()
{
	bgBackground = new JCBranchGroup();
	background = new Background();   //  Initialized to black
	background.setCapability(Background.ALLOW_COLOR_READ);
	background.setCapability(Background.ALLOW_COLOR_WRITE);
	background.setApplicationBounds(boundsphere);
    bgBackground.addChild(background);
	bgScene.addChild(bgBackground);
}

/**
 * Removes the <code>Background</code> object from the scene graph.
 */
public void removeBackground()
{
    bgBackground.detach();
    bgBackground = null;
    background = null;
}

/**
 * Returns the <code>Background</code> object.
 *
 * @return the <code>Background</code> object
 */
public Background getBackgroundNode()
{
	return background;
}

/**
 * Enables or disables full scene antialiasing.
 *
 * @param flag <code>true</code> to enable full scene antialiasing, <code>false</code> to disable
 */
public void setSceneAntialiasing(boolean flag)
{
    view.setSceneAntialiasingEnable(flag);
}

/**
 * Returns the <code>Locale</code> object for the virtual universe.
 *
 * @return the <code>Locale</code> object
 */
public Locale getLocale()
{
    return locale;
}

/**
 *  Adjust the orientation of the view platform so that the plotcube
 *  appears to be rotated in the specified x,y, and z angles.
 *
 * @param angleX rotation about the x axis in degrees
 * @param angleY rotation about the y axis in degrees
 * @param angleZ rotation about the z axis in degrees
 */
public void setViewPlatformRotation(double angleX, double angleY, double angleZ)
{
	double radianX, radianY, radianZ;

	//  Convert degrees to radians:
	radianX = Math.toRadians(angleX);
	radianY = Math.toRadians(angleY);
	radianZ = Math.toRadians(angleZ);

	//  These are the original up vector and eye position:
	Vector3d upVector = new Vector3d(0.0, 1.0, 0.0);
	Point3d eye = new Point3d(0.0, 0.0, viewPlatformDistance);
	//  Apply the specified rotations on those up vector and eye position:
	Transform3D t = new Transform3D();
	t.setEuler( new Vector3d(-radianX, -radianY, -radianZ) );
	t.transform(upVector);
	t.transform(eye);

	//  Pass the new up vector and eye position to Transform3D.lookAt() method.
	//  We are looking at the origin (centre of our plotcube).
	Transform3D t3d = new Transform3D();
	tgViewPlatform.getTransform(t3d);
	Point3d center = new Point3d(0.0, 0.0, 0.0);
	t3d.lookAt(eye, center, upVector);
	t3d.invert(t3d);
	//  Update the transformation for the view platform.
	tgViewPlatform.setTransform(t3d);
}

}
