/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCCanvas3D.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:05 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.j3d;

import java.awt.GraphicsConfiguration;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;

import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import javax.vecmath.*;
import com.klg.jclass.util.swing.encode.*;
import java.io.*;

import java.util.EventListener;

public class JCCanvas3D extends Canvas3D {

protected JCChart3dJava3d chart3d = null;
private boolean fWaitRenderDone;
private GraphicsContext3D  last_ctx = null;
protected int bufferedImageType = BufferedImage.TYPE_INT_RGB;

    transient MouseListener myMouseListener = null;
    transient MouseMotionListener myMouseMotionListener = null;

/**
 * Creates a special <code>Canvas3D</code> needed for <code>Chart3D</code> image capture.
 *
 * @param cfg a <code>GraphicsConfiguration</code> object
 */
public JCCanvas3D(GraphicsConfiguration cfg)
{
    super(cfg);
    enableEvents(AWTEvent.MOUSE_EVENT_MASK | AWTEvent.MOUSE_MOTION_EVENT_MASK);
}

/**
 * Invoked when the Java 3D renderer is done.  Clears the <code>JCCanvas3D</code> busy flag.
 */
public void postRender()
{
    fWaitRenderDone = false;
}

/**
 * Sets the <code>JCCanvas3D</code> busy flag to the specified value.
 * @param flag
 */
public void setWaitRenderFlag(boolean flag)
{
    fWaitRenderDone = flag;
}

/**
 * Returns the value of the <code>JCCanvas3D</code> busy flag.
 * @return
 */
public boolean getWaitRenderFlag()
{
    return fWaitRenderDone;
}

public synchronized void postSwap()
{
    super.postSwap();
    last_ctx = getGraphicsContext3D();
}

/**
 * This method sets the value of the bufferedImageType
 * property.
 *
 * @param type The new value for the bufferedImageType property.
 * Supported types include BufferedImage.TYPE_INT_ARGB,
 * BufferedImage.TYPE_USHORT_555_RGB, and BufferedImage.TYPE_INT_RGB.
 */
public void setBufferedImageType(int type) {
    bufferedImageType = type;
}

/**
 * Returns the value of the bufferedImageType property.
 * The default value is BufferedImage.TYPE_INT_RGB
 */
public int getBufferedImageType() {
    return bufferedImageType;
}

/**
 * Captures an image from the <code>Canvas3D</code>.
 *
 * @return an <code>Image</code> object or null if an error
 * occurred. The default type is BufferedImage.TYPE_INT_RGB
 */
public Image getAsImage()
{
    Image paintedImage = null;
    try {
        paintedImage = getBufferedImage(bufferedImageType);
    }
    catch (IllegalArgumentException e) {
        e.printStackTrace();
    }
    return paintedImage;
}

/**
 * Captures an image from the <code>Canvas3D</code>.
 *
 * @param type An integer specifying the Image type.
 * Supported types include BufferedImage.TYPE_INT_ARGB,
 * BufferedImage.TYPE_USHORT_555_RGB, and BufferedImage.TYPE_INT_RGB.
 * @return a <code>BufferedImage</code> object
 */
public BufferedImage getBufferedImage(int type) throws IllegalArgumentException
{
    int bufferedImageType = type;
    int imageComponentType = Integer.MAX_VALUE;

    if (type == BufferedImage.TYPE_USHORT_555_RGB) {
        imageComponentType = ImageComponent.FORMAT_RGB5;
    }
    else if (type == BufferedImage.TYPE_INT_RGB) {
        imageComponentType = ImageComponent.FORMAT_RGB;
    }
    else if (type == BufferedImage.TYPE_INT_ARGB) {
        imageComponentType = ImageComponent.FORMAT_RGBA;
    }
    else {
        throw new IllegalArgumentException(
            "BufferedImage type " + type + " is not supported");
    }

    Raster ras = new Raster(
           new Point3f(-1.0f,-1.0f,-1.0f),
           Raster.RASTER_COLOR,
           0,0,
           getBounds().width,getBounds().height,
           new ImageComponent2D(
                     imageComponentType,
                     new BufferedImage(getBounds().width,getBounds().height,
                                       bufferedImageType)),
           null);

    last_ctx.readRaster(ras);

    // Now strip out the image info
    BufferedImage paintedImage = ras.getImage().getImage();

    return paintedImage;
}

public void setChart3d(JCChart3dJava3d chart3d) {
    this.chart3d = chart3d;
}

public synchronized void addMouseListener(MouseListener l) {
    if (l == null) {
        return;
    }
    myMouseListener = AWTEventMulticaster.add(myMouseListener,l);
//    newEventsOnly = true;

    // if this is a lightweight component, enable mouse events
    // in the native container.
//    if (getPeer() instanceof java.awt.peer.LightweightPeer) {
//      parent.proxyEnableEvents(AWTEvent.MOUSE_EVENT_MASK);
//    }
}
public synchronized void removeMouseListener(MouseListener l) {
    if (l == null) {
        return;
    }
    myMouseListener = AWTEventMulticaster.remove(myMouseListener, l);
}

public synchronized void addMouseMotionListener(MouseMotionListener l) {
    if (l == null) {
        return;
    }
    myMouseMotionListener = AWTEventMulticaster.add(myMouseMotionListener,l);
//    newEventsOnly = true;

    // if this is a lightweight component, enable mouse events
    // in the native container.
//    if (getPeer() instanceof java.awt.peer.LightweightPeer) {
//      parent.proxyEnableEvents(AWTEvent.MOUSE_MOTION_EVENT_MASK);
//    }
}

public synchronized void removeMouseMotionListener(MouseMotionListener l) {
    if (l == null) {
        return;
    }
    myMouseMotionListener = AWTEventMulticaster.remove(myMouseMotionListener, l);
}

protected void processMouseEvent(MouseEvent e) {
    MouseListener listener = myMouseListener;
    if (listener != null) {
        int id = e.getID();
        e.translatePoint(chart3d.convertChartAreaXToX(0),
                         chart3d.convertChartAreaYToY(0));
        switch(id) {
            case MouseEvent.MOUSE_PRESSED:
            listener.mousePressed(e);
            break;
            case MouseEvent.MOUSE_RELEASED:
            listener.mouseReleased(e);
            break;
            case MouseEvent.MOUSE_CLICKED:
            listener.mouseClicked(e);
            break;
            case MouseEvent.MOUSE_EXITED:
            listener.mouseExited(e);
            break;
            case MouseEvent.MOUSE_ENTERED:
            listener.mouseEntered(e);
            break;
        }
    }
}

protected void processMouseMotionEvent(MouseEvent e) {
    MouseMotionListener listener = myMouseMotionListener;
    if (listener != null) {
        int id = e.getID();
        e.translatePoint(chart3d.convertChartAreaXToX(0),
                         chart3d.convertChartAreaYToY(0));
        switch(id) {
            case MouseEvent.MOUSE_MOVED:
            listener.mouseMoved(e);
            break;
            case MouseEvent.MOUSE_DRAGGED:
            listener.mouseDragged(e);
            break;
        }
    }
}


/*
// JDK 1.3 and higher only
protected void processMouseMotionEvent(MouseEvent e) {
    EventListener[] list = getListeners(MouseMotionListener.class);
    for (int i = 0; i < list.length; i++) {
        MouseMotionListener listener = (MouseMotionListener )list[i];
        if (listener != null) {
            int id = e.getID();
            e.translatePoint(chart3d.convertChartAreaXToX(0),
                            chart3d.convertChartAreaYToY(0));
            switch(id) {
                case MouseEvent.MOUSE_MOVED:
                listener.mouseMoved(e);
                break;
                case MouseEvent.MOUSE_DRAGGED:
                listener.mouseDragged(e);
                break;
            }
        }
    }
}

protected void processMouseEvent(MouseEvent e) {
    EventListener[] list = getListeners(MouseListener.class);
    for (int i = 0; i < list.length; i++) {
        MouseListener listener = (MouseListener )list[i];
        if (listener != null) {
            int id = e.getID();
            e.translatePoint(chart3d.convertChartAreaXToX(0),
                            chart3d.convertChartAreaYToY(0));
            switch(id) {
                case MouseEvent.MOUSE_PRESSED:
                listener.mousePressed(e);
                break;
                case MouseEvent.MOUSE_RELEASED:
                listener.mouseReleased(e);
                break;
                case MouseEvent.MOUSE_CLICKED:
                listener.mouseClicked(e);
                break;
                case MouseEvent.MOUSE_EXITED:
                listener.mouseExited(e);
                break;
                case MouseEvent.MOUSE_ENTERED:
                listener.mouseEntered(e);
                break;
            }
        }
    }
}
*/


}
