/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCChart3dAreaJava3d.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:05 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.j3d;

import javax.swing.JComponent;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import javax.vecmath.*;
import javax.media.j3d.*;
import java.util.HashMap;
import com.klg.jclass.chart3d.*;
import com.klg.jclass.util.swing.encode.Java3dImageable;





import java.util.Vector;


/**
 * This is the sub-portion of the overall <code>Chart3d</code> real estate onto which the
 * actual 3D chart is drawn. Headers, footers, and legends are not drawn in
 * this area; the 3d graph is.
 *
 */
public class JCChart3dAreaJava3d extends JCChart3dArea
              implements Changeable, Java3dImageable, java.io.Serializable {

protected JCChart3dUniverse chart3dUniverse = null;
protected HashMap shapeInstances = null;

/**
 * Create a <code>Chart3d</code> area.
 *
 * @param chart3d the <code>JCChart3d</code> whose graph will be drawn
 *      in this <code>Chart3d</code> area
 */
JCChart3dAreaJava3d(JCChart3dJava3d chart3d)
{
    super(chart3d);
    //  Create the Java3D virtual universe:
    chart3dUniverse = new JCChart3dUniverse(chart3d);

    setLayout(new BorderLayout());
    add(chart3dUniverse);
    cube = new CubeJava3d(chart3d);
    shapeInstances = new HashMap();
}

public HashMap getShapeInstances()
{
    return shapeInstances;
}

/**
 * Returns a <code>JCChart3dUniverse</code> object containing the Java 3D scene graph.
 * <code>getChart3dUniverse().getVirtualUniverse().getSceneBranchGroup()</code> returns
 * a <code>BranchGroup</code> containing the scene graph.
 * @return a <code>BranchGroup</code> containing the scene graph
 */
public JCChart3dUniverse getChart3dUniverse()
{
    return chart3dUniverse;
}

/**
 * Sets the color of the <code>Background</code> node in the scene graph.
 *
 * @param color the desired color for the background
 */
public void setBackground(Color color) {
    Color3f color3f = new Color3f(color);
    chart3dUniverse.virtualUniverse.getBackgroundNode().setColor(color3f);
}

/**
 * Adds an <code>AmbientLight</code> object with <code>Chart3d</code> default properties to the scene graph.
 */
public void addAmbientLight() {
    chart3dUniverse.virtualUniverse.addAmbientLight();
}

/**
 * Adds a <code>SpotLight</code> object with <code>Chart3d</code> default properties to the scene graph.
 */
public void addSpotLight() {
    chart3dUniverse.virtualUniverse.addSpotLight();
}

/**
 * Adds a <code>DirectionalLight</code> object with <code>Chart3d</code> default properties to the scene graph.
 */
public void addDirectionalLight() {
    chart3dUniverse.virtualUniverse.addDirectionalLight();
}

/**
 * Adds a <code>PointLight</code> object with <code>Chart3d</code> default properties to the scene graph.
 */
public void addPointLight() {
    chart3dUniverse.virtualUniverse.addPointLight();
}

/**
 * Adds a <code>Light</code> object to the scene graph.
 *
 * @param light the <code>Light</code> object to be added
 */
public void addLighting(Light light) {
    light.setInfluencingBounds(chart3dUniverse.virtualUniverse.getBoundingSphere());
    chart3dUniverse.virtualUniverse.addLighting(light);
}

/**
 * Removes all <code>Light</code> objects from the scene graph.
 */
public void removeAllLighting() {
    chart3dUniverse.virtualUniverse.removeLighting();
}

/**
 * Adds a <code>Fog</code> object to the scene graph.
 *
 * @param fog the <code>Fog</code> object to be added
 */
public void addFog(Fog fog) {
    fog.setInfluencingBounds(chart3dUniverse.virtualUniverse.getBoundingSphere());
    chart3dUniverse.virtualUniverse.addFog(fog);
}

/**
 * Removes the current <code>Fog</code> object from the scene graph.
 */
public void removeFog() {
    chart3dUniverse.virtualUniverse.removeFog();
}

/**
 * Adds a <code>LinearFog</code> object to the scene graph.
 *
 * @param color the desired color for the <code>LinearFog</code>
 */
public void addLinearFog(Color color) {
    chart3dUniverse.virtualUniverse.addLinearFog(new Color3f(color));
}

/**
 * Enables or disables the <code>Fog</code>.
 *
 * @param flag <code>true</code> to enable the fog, <code>false</code> to disable it
 */
public void enableFog(boolean flag) {
    chart3dUniverse.virtualUniverse.enableFog(flag);
}

/**
 * Gets the on/off state of the fog.
 *
 * @return <code>true</code> if fog is enabled, <code>false</code> if it is disabled
 */
public boolean isFogEnabled() {
    return chart3dUniverse.virtualUniverse.isFogEnabled();
}

/**
 * Sets the color of the <code>Fog</code>.
 *
 * @param color the desired color for the <code>fog</code>
 */
public void setFogColor(Color color) {
    chart3dUniverse.virtualUniverse.setFogColor(new Color3f(color));
}

/**
 * Gets the color of the <code>Fog</code>.
 *
 * @return the <code>Fog</code> color
 */
public Color getFogColor() {
    Color3f color3f = new Color3f();
    chart3dUniverse.virtualUniverse.getFogColor(color3f);
    return color3f.get();
}

/**
 * Sets the front distance of the <code>Fog</code> object.
 *
 * @param front the front distance in meters
 */
public void setFogFrontDistance(double front) {
    chart3dUniverse.virtualUniverse.setFogFrontDistance(front);
}

/**
 * Gets the front distance of the <code>Fog</code> object.
 *
 * @return the front distance in meters
 */
public double getFogFrontDistance() {
    return chart3dUniverse.virtualUniverse.getFogFrontDistance();
}

/**
 * Sets the back distance of the <code>Fog</code> object.
 *
 * @param back the back distance in meters
 */
public void setFogBackDistance(double back) {
    chart3dUniverse.virtualUniverse.setFogBackDistance(back);
}

/**
 * Gets the back distance of the <code>Fog</code> object.
 *
 * @return the back distance in meters
 */
public double getFogBackDistance() {
    return chart3dUniverse.virtualUniverse.getFogBackDistance();
}

/**
 * Enables or disables full scene antialiasing.  If enabled, rendering
 * is very slow.
 *
 * @param antialias <code>true</code> to enable full scene antialiasing, <code>false</code> to disable it
 */
public void setSceneAntialiasing(boolean antialias) {
    chart3dUniverse.virtualUniverse.setSceneAntialiasing(antialias);
}

/**
 * Displays or hides the <code>Chart3d</code> area.
 *
 * @param fVisible <code>true</code> to display the <code>Chart3d</code> area, <code>false</code> to hide it
 */
public void setVisible(boolean fVisible) {
    super.setVisible(fVisible);
    chart3dUniverse.setVisible(fVisible);
}

protected void setPlotCubeJava3d(JCPlotCubeJava3d pc) {
    plotCube = pc;
    chart3dUniverse.virtualUniverse.removeAllContent();
    chart3dUniverse.virtualUniverse.addContent(((JCPlotCubeJava3d)plotCube).getContent());
}

//  Override the base class method.  Let the Java 3D renderer do all the painting.
public void paintComponent(Graphics gc)
{
}

/**
 * Overrides the base class method.
 * Chooses the point closest to the provided position.  If
 * a data view is supplied, picks a point on the provided
 * data view only.  Otherwise, tries to find the closest point
 * from any data view (currently, only one data view is supported).
 *
 * @param p pick point in pixels relative to <code>JCChart3dArea</code> object
 * @param dataView data view on which to perform pick; if null, all data
 *     views are used (currently only one data view is supported)
 * @return <code>JCData3dIndex</code> object representing the picked point
 */
protected JCData3dIndex pick(Point p, Chart3dDataView dataView)
{
    JCData3dIndex index = null;
    if (p == null || chart3d == null) {
        return(index);
    }

    // Since there currently is only one dataView, use it.
    if (dataView == null) {
        dataView = chart3d.getDataView(0);
    }

/*
    anno.pickLabel() calls draw3dAnno() -- can't do that in the Java 3D version.
    // First check to see if a label was selected
    JCValueLabel vLabel = anno.pickLabel(getGraphics(), p);
    if (vLabel != null) {
        return(new JCData3dLabelIndex(vLabel));
    }
*/

    // Make sure that the dataView has a draw object
    dataView.createDraw();
    index = dataView.getDraw().getMapping().pick(p.x, p.y);
    if (index != null) {
        index.setObject(this);
        index.setDataView(dataView);
    }
    return(index);
}

public void updateAnnotation()
{
    ((AnnoJava3d)anno).updateChart3dRotation();
    recalc();
    //  calls dataView.calcDefaults()
    if (transform != null) {
        transform.computeTransformations(getGraphics());
    }
    ((JCPlotCubeJava3d)getPlotCube()).addGridLines();
    ((AnnoJava3d)anno).draw3dAnno(getGraphics());
}

public void updatePlotCube()
{
    updateAnnotation();
    ((JCPlotCubeJava3d)getPlotCube()).updateProjections();
}

//Java3dImageable methods
/**
 * Gets the Java3d canvas used by this class as an image.  Implemented
 * as part of the <code>Java3dImageable</code> interface.
 * @param imageLocation a <code>Point</code> object that will be filled with the location
 *        of the canvas' image within this component
 * @return an <code>Image</code> class containg a drawn representation of this class'
 *         3d canvas object; null if no canvas exists
 */
public Image getJava3dAsImage(Point imageLocation)
{
    if (chart3dUniverse != null) {
        if (imageLocation != null) {
            imageLocation.x = chart3dUniverse.getLocation().x;
            imageLocation.y = chart3dUniverse.getLocation().y;
        }
        if (chart3dUniverse.myCanvas3D != null) {
            return chart3dUniverse.myCanvas3D.getAsImage();
        }
    }
    return null;
}

/**
 * Adds a <code>MouseMotionListener</code> to the <code>Chart3d</code> area.
 *
 * @param listener the <code>MouseMotionListener</code> to be added
 */
public void addMouseMotionListener(MouseMotionListener listener)
{
    super.addMouseMotionListener(listener);
    getChart3dUniverse().getVirtualUniverse().getCanvas3D().addMouseMotionListener(listener);
}

/**
 * Removes a <code>MouseMotionListener</code> from the <code>Chart3d</code> area.
 *
 * @param listener the <code>MouseMotionListener</code> to be removed
 */
public void removeMouseMotionListener(MouseMotionListener listener)
{
    super.removeMouseMotionListener(listener);
    getChart3dUniverse().getVirtualUniverse().getCanvas3D().removeMouseMotionListener(listener);
}

/**
 * Adds a <code>MouseListener</code> to the <code>Chart3d</code> area.
 *
 * @param listener the <code>MouseListener</code> to be added
 */
public void addMouseListener(MouseListener listener)
{
    super.addMouseListener(listener);
    getChart3dUniverse().getVirtualUniverse().getCanvas3D().addMouseListener(listener);
}

/**
 * Removes a <code>MouseMotionListener</code> from the <code>Chart3d</code> area.
 *
 * @param listener the <code>MouseListener</code> to be removed
 */
public void removeMouseListener(MouseListener listener)
{
    super.removeMouseListener(listener);
    getChart3dUniverse().getVirtualUniverse().getCanvas3D().removeMouseListener(listener);
}

}
