/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCChart3dJava3d.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:05 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.j3d;

import com.klg.jclass.chart3d.*;

import java.awt.Point;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.awt.event.InputEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

/**
 * The core charting class. <code>JCChart3dJava3d</code> allows the
 * retrieval of related objects, such as the chart area, the
 * legend, the header, and the footer.
 */
public class JCChart3dJava3d extends JCChart3d implements Chart3dGridDataModel {

protected String customizerName =
    "com.klg.jclass.chart3d.j3d.Chart3dCustomizerJava3d";

protected ActionHandlerParams actionHandlerParams = null;

public String getCustomizerName()
{
    return customizerName;
}

public void setCustomizerName(String name)
{
    customizerName = name;
}

protected void init(int chartType)
{
    setOpaque(true);
    initializeLayout();
    createHeader();
    createFooter();
    createLegend();
//  createChart3dArea();
    resetDefaults();
    initializeUI();
    createActionTable();
}

/**
 * Creates an <code>ActionHandlerParams</code> object containing all the
 * parameters (scene <code>BranchGroup</code>, <code>BoundingSphere</code>, and
 * so on) required by the action handlers.
 */
protected void createActionHandlerParams()
{
    JCChart3dAreaJava3d area3d = (JCChart3dAreaJava3d)getChart3dArea();
    actionHandlerParams = new ActionHandlerParams(area3d);
}

protected void createActionTable()
{
    actionTable = new ActionTable();
}

protected JCAction getActionHandler(InputEvent event)
{
    return actionTable.getActionHandlerInstance(event, actionHandlerParams);
}

/**
 * Constructor that creates a <code>JCChart3dJava3d</code> of a particular type.
 * @param tp
 */
public JCChart3dJava3d(int tp)
{
    super();
    createObjects(tp);
}

/**
 * Default constructor, required by JavaBeans.
 */
public JCChart3dJava3d()
{
    super();
    createObjects(SURFACE);
}

/**
 * Creates the objects needed for this <code>Chart3d</code> object.
 * @param chartType
 */
protected void createObjects(int chartType)
{
    JCSymbolStyle.setDefault3dShapes(null);
    boolean batchedSave = isBatched();
    setBatched(true);
    createChart3dAreaJava3d();
    createActionHandlerParams();
    Chart3dDataViewJava3d dv = (Chart3dDataViewJava3d)addDataView(0, chartType);

    //  params for JCElevation()
    //  meshed = false, shaded = true, transparent = false
    dv.setElevation(new JCElevation(false, true, false));
    JCContour contour = new JCContour(dv);
    dv.setContour(contour);
    contour.setContourLevels(new JCContourLevels());

    JCAxis[] axes = new JCAxis[JCAxis.NUM_AXES];
    chart3dArea.setAxes(axes);
    chart3dArea.setAxis(new JCAxis(JCAxis.AXIS_X, getFont()), JCAxis.AXIS_X);
    chart3dArea.setAxis(new JCAxis(JCAxis.AXIS_Y, getFont()), JCAxis.AXIS_Y);
    chart3dArea.setAxis(new JCAxis(JCAxis.AXIS_Z, getFont()), JCAxis.AXIS_Z);
    for (int i = 0; i < JCAxis.NUM_AXES; i++) {
        axes[i].setGridLines(new JCGridLinesJava3d(axes[i],
                                                   JCGridLines.NO_PLANE));
    }

    JCChart3dAreaJava3d area3d = (JCChart3dAreaJava3d)chart3dArea;
    chart3dArea.setView3d(new JCView3dJava3d(area3d));
    JCPlotCubeJava3d plotCube = new JCPlotCubeJava3d(area3d);
    plotCube.setPerspective();
    plotCube.setFloor(new JCProjection());
    plotCube.setCeiling(new JCProjection());
    area3d.setPlotCubeJava3d(plotCube);

    chart3dArea.setViewport(new JCViewportJava3d(area3d));
    chart3dArea.setCube(new CubeJava3d(this));
    chart3dArea.setTransform(new Transform(chart3dArea));
    chart3dArea.setAnno(new AnnoJava3d(area3d));
    chart3dArea.setSurface(new JCSurface());
    chart3dArea.setBar(new JCBar());
    chart3dArea.setScatter(new JCScatter());
    dv.setElevationDataSource(this);

    dv.setMapping(new MapPick());
    setBatched(batchedSave);
}

/**
 * Creates a new <code>Chart3dDataView</code> and adds it to the chart
 * at the specified index, <B>replacing</B> the existing
 * <code>ChartDataView</code> object at that specified index. If the index is
 * one past the final occupied index, the <code>ChartDataView</code> will be
 * added to the end of the list.
 * Also see {@link #setDataView setDataView}.
 * @return
 * @param index
 * @param chartType
 */
public Chart3dDataView addDataView(int index, int chartType)
{
    Chart3dDataViewJava3d ndv = new Chart3dDataViewJava3d(this, index,
                                                          chartType);
    setDataView(index, ndv);
    return(ndv);
}

/**
 * Creates and adds the legend to the chart.
 */
protected void createChart3dAreaJava3d()
{
    chart3dArea = new JCChart3dAreaJava3d(this);
    add(chart3dArea);
    // must do this manually.  Can't rely on addNotify() because axes
    // need to know their parent chart before that is finally called.
    // chart3dArea.setParentOnAxes(this);
}

/**
 * Overrides the parent method to call <code>update()</code> instead of
 * <code>repaint()</code>.
 */
public void redraw()
{
    update();
}

/**
 * Forces the chart to be recalculated and laid out again.
 */
public void update()
{
    Chart3dDataViewJava3d dv = (Chart3dDataViewJava3d)getDataView(0);
    dv.computeDataLimits();
    ((AnnoJava3d)chart3dArea.getAnno()).updateChart3dRotation();
    dv.computeTransformations();
    ((JCPlotCubeJava3d)chart3dArea.getPlotCube()).addGridLines();
    chart3dArea.getAnno().draw3dAnno(getGraphics());
    dv.createDataObject();
    ((JCPlotCubeJava3d)chart3dArea.getPlotCube()).updateProjections();

    //  Update the Legend:
    chart3dArea.updateParent(Changeable.RECALC);
}

/**
 * This method must be called by an application or applet if it contains an
 * instance of <code>JCChart3dJava3d</code> that needs to be garbage collected
 * before the application or applet terminates. <code>dispose()</code> calls the
 * <code>removeAllLocales()</code> method of the <code>VirtualUniverse</code>
 * object that contains the Chart3D, thus ensuring that Java3D does not retain
 * any references to the <code>JCChart3dJava3d</code> instance.
 */
public void dispose()
{
    JCChart3dAreaJava3d area3d = ((JCChart3dAreaJava3d)getChart3dArea());
    JC3DUniverse universe = area3d.getChart3dUniverse().getVirtualUniverse();
    universe.getLocale().getVirtualUniverse().removeAllLocales();
}

public void addMouseMotionListener(MouseMotionListener listener)
{
    super.addMouseMotionListener(listener);
    JCChart3dAreaJava3d area3d = ((JCChart3dAreaJava3d)getChart3dArea());
    JC3DUniverse universe = area3d.getChart3dUniverse().getVirtualUniverse();
    universe.getCanvas3D().addMouseMotionListener(listener);
}

public void removeMouseMotionListener(MouseMotionListener listener)
{
    super.removeMouseMotionListener(listener);
    JCChart3dAreaJava3d area3d = ((JCChart3dAreaJava3d)getChart3dArea());
    JC3DUniverse universe = area3d.getChart3dUniverse().getVirtualUniverse();
    universe.getCanvas3D().removeMouseMotionListener(listener);
}

public void addMouseListener(MouseListener listener)
{
    super.addMouseListener(listener);
    JCChart3dAreaJava3d area3d = ((JCChart3dAreaJava3d)getChart3dArea());
    JC3DUniverse universe = area3d.getChart3dUniverse().getVirtualUniverse();
    universe.getCanvas3D().addMouseListener(listener);
}

public void removeMouseListener(MouseListener listener)
{
    super.removeMouseListener(listener);
    JCChart3dAreaJava3d area3d = ((JCChart3dAreaJava3d)getChart3dArea());
    JC3DUniverse universe = area3d.getChart3dUniverse().getVirtualUniverse();
    universe.getCanvas3D().removeMouseListener(listener);
}


/**
 * Override the base class method to handle the Canvas3D chart area component.
 * @param img The image to put the snapshot in.
 */
public void snapshot(Image img) {
    super.snapshot(img);
    // Get a snapshot of the Canvas3D and place it at the location of
    // the chart area.
    if (img != null && img instanceof BufferedImage) {
        JCChart3dAreaJava3d area3d =
            ((JCChart3dAreaJava3d)getChart3dArea());
        if (area3d != null) {
            int imageType = ((BufferedImage)img).getType();
            area3d.getChart3dUniverse().getVirtualUniverse().getCanvas3D().
                                            setBufferedImageType(imageType);
            Point pt = new Point(0,0);
            BufferedImage chartAreaImage =
                (BufferedImage)area3d.getJava3dAsImage(pt);
            // pt is the location of the JCChart3dUniverse component
            // within the JCChart3dAreaJava3d object.
            if (chartAreaImage != null) {
                int x = area3d.getLocation().x + pt.x;
                int y = area3d.getLocation().y + pt.y;
                Raster raster =
                    chartAreaImage.getRaster().createTranslatedChild(x,y);
                ((BufferedImage)img).setData(raster);
            }
        }
    }
}

}
