/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCChart3dUniverse.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:05 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.j3d;

import com.sun.j3d.utils.picking.*;
import java.awt.BorderLayout;
import java.awt.Component;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import javax.swing.*;
import com.klg.jclass.chart3d.*;

/**
 * This class creates the Java 3D scene graph (indirectly, using methods from
 * <code>JC3DUniverse</code>), adding Chart 3D specific features to it.
 */
public class JCChart3dUniverse extends JComponent {

protected JCChart3dJava3d chart3d = null;
protected JC3DUniverse virtualUniverse = null;
protected JCCanvas3D myCanvas3D = null;

/**
 * Creates a Java 3D scene graph containing an <code>InputBehavior</code> object.
 *
 * @param chart3d the <code>JCChart3d</code> whose graph will be created in this
 *      virtual universe
 */
public JCChart3dUniverse (JCChart3dJava3d chart3d)
{
	super();

	this.chart3d = chart3d;
	virtualUniverse = new JC3DUniverse();
    virtualUniverse.addBackground();

	addChart3dBehavior();

	JCBranchGroup bgScene = virtualUniverse.getSceneBranchGroup();
    bgScene.compile();
    virtualUniverse.getLocale().addBranchGraph(bgScene);

	setLayout(new BorderLayout());
	myCanvas3D = virtualUniverse.getCanvas3D();
	myCanvas3D.setChart3d(chart3d);
	add("Center", myCanvas3D);
}

/**
 * Returns a reference to our virtual universe.
 *
 * @return <code>JC3DUniverse</code> object containing the scene graph
 */
public JC3DUniverse getVirtualUniverse()
{
	return virtualUniverse;
}

/**
 * Sets the distance of the view platform from the origin of the virtual universe.
 * This method updates the perspective value in the <code>JCView3d</code> object.
 *
 * @param vpDistance the distance in meters of the view platform from the origin
 */
public void setViewPlatformDistance(double vpDistance)
{
	virtualUniverse.setViewPlatformDistance(vpDistance);
	//  Update the perspective:
	TransformGroup transformGroup = ((JCPlotCubeJava3d)chart3d.getChart3dArea().getPlotCube()).getTransformGroup();
	Transform3D t3d = new Transform3D();
	transformGroup.getTransform(t3d);
	Vector3d translation = new Vector3d();
	t3d.get(translation);
	double perspective = (vpDistance - translation.z) / 2.0;
	chart3d.getChart3dArea().getView3d().setPerspective(perspective);
}

/**
 * Returns the distance of the view platform from the origin of the virtual universe.
 *
 * @return the distance in meters of the view platform from the origin
 */
public double getViewPlatformDistance()
{
	return virtualUniverse.getViewPlatformDistance();
}

protected void addChart3dBehavior()
{
	InputBehavior behavior =
		    new InputBehavior(chart3d, virtualUniverse.getBoundingSphere());
	virtualUniverse.addBehavior(behavior);
}

}
