/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCCubeSquareArray.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:05 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.j3d;

import com.sun.j3d.utils.picking.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.klg.jclass.chart3d.*;

public class JCCubeSquareArray extends JCBranchGroup {

protected final static int NUM_FACES = 6;
private CubeWall[] cubeWall = new CubeWall[NUM_FACES];

// The six faces of the cube (looking down the z axis)
protected final static int FACE_LEFT   = 0;
protected final static int FACE_RIGHT  = 1;
protected final static int FACE_BOTTOM = 2;
protected final static int FACE_TOP    = 3;
protected final static int FACE_FAR    = 4;
protected final static int FACE_NEAR   = 5;

//  Creates a double unit cube with the centre at the origin.
protected JCCubeSquareArray(Color3f fillColor, Color3f outlineColor)
{
	//  FACE_FAR:
    cubeWall[FACE_FAR] = new CubeWall(new Vector3d(0.0, 0.0, 1.0),
                                      new Vector3d(0.0, 0.0, 0.0),
                                      new Vector3d(0.0, 0.0, -1.0),
                                      fillColor, outlineColor);

	//  FACE_LEFT:
    cubeWall[FACE_LEFT] = new CubeWall(new Vector3d(0.0, -1.0, 0.0),
                                       new Vector3d(Math.PI/2.0, 0.0,
                                                    Math.PI/2.0),
                                       new Vector3d(-1.0, 0.0, 0.0),
                                       fillColor, outlineColor);

	//  FACE_NEAR:
    cubeWall[FACE_NEAR] = new CubeWall(new Vector3d(0.0, 0.0, -1.0),
                                       new Vector3d(0.0, Math.PI, Math.PI),
                                       new Vector3d(0.0, 0.0, 1.0),
                                       fillColor, outlineColor);

	//  FACE_RIGHT:
    cubeWall[FACE_RIGHT] = new CubeWall(new Vector3d(0.0, -1.0, 0.0),
                                        new Vector3d(Math.PI/2.0, Math.PI,
                                                     -Math.PI/2.0),
                                        new Vector3d(1.0, 0.0, 0.0),
                                        fillColor, outlineColor);

	//  FACE_TOP:
    cubeWall[FACE_TOP] = new CubeWall(new Vector3d(0.0, -1.0, 0.0),
                                      new Vector3d(Math.PI/2.0, 0.0, 0.0),
                                      new Vector3d(0.0, 1.0, 0.0),
                                      fillColor, outlineColor);

	//  FACE_BOTTOM:
    cubeWall[FACE_BOTTOM] = new CubeWall(new Vector3d(0.0, 1.0, 0.0),
                                         new Vector3d(-Math.PI/2.0, 0.0,
                                         0.0),
                                         new Vector3d(0.0, -1.0, 0.0),
                                         fillColor, outlineColor);

	//  Don't rotate the cube walls to change orientation of wall images.
	//  Otherwise, grid lines will not be displayed properly.
	cubeWall[FACE_RIGHT].flipTextureCoordinates(JCAxis.AXIS_X);
	cubeWall[FACE_RIGHT].flipTextureCoordinates(JCAxis.AXIS_Y);

	cubeWall[FACE_BOTTOM].flipTextureCoordinates(JCAxis.AXIS_X);
	cubeWall[FACE_BOTTOM].flipTextureCoordinates(JCAxis.AXIS_Y);

    for (int i = 0; i < NUM_FACES; i++) {
		setRotation(i, 0.0);
	    addChild(cubeWall[i].tg);
    }

    compile();
}

protected void setPolygonOffset(float polygonOffset)
{
	for (int i=0; i < NUM_FACES; i++) {
		Appearance appearance = cubeWall[i].filledCubeFace.getAppearance();
	    PolygonAttributes pa = appearance.getPolygonAttributes();
	    if (pa == null) {
		    pa = new PolygonAttributes();
		    appearance.setPolygonAttributes(pa);
	    }
		pa.setPolygonOffset(polygonOffset);
	}
}

protected void setPolygonOffsetFactor(float polygonOffsetFactor)
{
	for (int i=0; i < NUM_FACES; i++) {
		Appearance appearance = cubeWall[i].filledCubeFace.getAppearance();
	    PolygonAttributes pa = appearance.getPolygonAttributes();
	    if (pa == null) {
		    pa = new PolygonAttributes();
		    appearance.setPolygonAttributes(pa);
	    }
		pa.setPolygonOffsetFactor(polygonOffsetFactor);
	}
}

protected float getPolygonOffset()
{
	float polygonOffset = 0.0f;
    Appearance app = cubeWall[0].filledCubeFace.getAppearance();
    PolygonAttributes pa = app.getPolygonAttributes();
    if (pa != null) {
        polygonOffset = pa.getPolygonOffset();
    }
	return polygonOffset;
}

protected float getPolygonOffsetFactor()
{
	float polygonOffsetFactor = 0.0f;
    Appearance app = cubeWall[0].filledCubeFace.getAppearance();
    PolygonAttributes pa = app.getPolygonAttributes();
    if (pa != null) {
        polygonOffsetFactor = pa.getPolygonOffsetFactor();
    }
	return polygonOffsetFactor;
}

protected void applyTexture(int face, JCTexture2D texture)
{
	if ( (face > -1) && (face < NUM_FACES) ) {

	    Appearance appearance = cubeWall[face].filledCubeFace.getAppearance();
	    appearance.setTexture(texture);
	}
}

protected JCShape3D getFace(int face)
{
	if ( (face > -1) && (face < NUM_FACES) ) {
	    return cubeWall[face].filledCubeFace;
	}
	return null;
}

protected JCShape3D getFaceOutline(int face)
{
	if ( (face > -1) && (face < NUM_FACES) ) {
	    return cubeWall[face].outlineCubeFace;
	}
	return null;
}

protected void setRotation(int face, double angle)
{
    if ( (face < 0) || (face > 5) ) {
        return;
    }
    double remainder = Math.IEEEremainder(angle, 90.0);
    if (remainder != 0.0) {
        return;
    }
    double radians = Math.toRadians(angle);
	cubeWall[face].setImageAngle(angle);   // degrees
    Vector3d newAngle = new Vector3d(cubeWall[face].imageRotationAxis);
    newAngle.scale(radians);
    cubeWall[face].eulerAngles.add(newAngle);
    Transform3D t3d = new Transform3D();
    t3d.setEuler(cubeWall[face].eulerAngles);
    t3d.setTranslation(cubeWall[face].translation);
    cubeWall[face].tg.setTransform(t3d);
}

protected double getRotation(int face)
{
	return cubeWall[face].getImageAngle();
}

protected void setWallImageRotation(int face, double angle)
{
    if ( (face < 0) || (face > 5) ) {
        return;
    }
	cubeWall[face].setImageAngle(angle);   // degrees
}

protected double getWallImageRotation(int face)
{
	return cubeWall[face].getImageAngle();
}

protected void flipWallImage(int face, int axis)
{
	cubeWall[face].flipTextureCoordinates(axis);
}

protected void addObject(int face, JCBranchGroup bg)
{
	cubeWall[face].addObject(bg);
}

protected void clearObjects(int face)
{
	cubeWall[face].clearObjects();
}

private void addGridObject(int face, Point3d[] pnts, Color3f[] clrs)
{
	JCBranchGroup bgGridLine = new JCBranchGroup();
	QuadArray geom = new QuadArray( 4,
									GeometryArray.COORDINATES |
									GeometryArray.COLOR_3 );
	geom.setCoordinates(0, pnts);
	geom.setColors(0, clrs);

	JCAppearance app = new JCAppearance();
	JCPolygonAttributes pa = new JCPolygonAttributes();
	pa.setPolygonMode(PolygonAttributes.POLYGON_LINE);
	pa.setPolygonOffset(0.0f);
	pa.setPolygonOffsetFactor(0.0f);
	pa.setCullFace(PolygonAttributes.CULL_BACK);
	app.setPolygonAttributes(pa);

	JCShape3D shape = new JCShape3D(geom, app);

	shape.setCapability(Shape3D.ENABLE_PICK_REPORTING);
	PickTool.setCapabilities(shape, PickTool.INTERSECT_FULL);

	bgGridLine.addChild(shape);
	cubeWall[face].addObject(bgGridLine);
}

protected void addGridLine(JCAxis axis, double value, Color3f color)
{
	Point3d pnts[] = new Point3d[4];
	Color3f clrs[] = new Color3f[4];

	for (int i = 0; i < 4; i++) {
		clrs[i] = new Color3f(color);
	}

	if ( JCAxis.AXIS_X == axis.getAxisId() ) {
		pnts[3] = new Point3d(-1.0, 1.0, 0.0);
		pnts[2] = new Point3d(value, 1.0, 0.0);
		pnts[1] = new Point3d(value, -1.0, 0.0);
		pnts[0] = new Point3d(-1.0, -1.0, 0.0);
		if ((axis.getGridLines().getPlaneMask() & JCGridLines.XZ_PLANE) > 0) {
			addGridObject(FACE_BOTTOM, pnts, clrs);
			addGridObject(FACE_TOP, pnts, clrs);
		}
		if ((axis.getGridLines().getPlaneMask() & JCGridLines.XY_PLANE) > 0) {
			addGridObject(FACE_NEAR, pnts, clrs);
			addGridObject(FACE_FAR, pnts, clrs);
		}
	}
	else if ( JCAxis.AXIS_Y == axis.getAxisId() ) {
		if ((axis.getGridLines().getPlaneMask() & JCGridLines.YZ_PLANE) > 0) {
			pnts[3] = new Point3d(-1.0, 1.0, 0.0);
			pnts[2] = new Point3d(value, 1.0, 0.0);
			pnts[1] = new Point3d(value, -1.0, 0.0);
			pnts[0] = new Point3d(-1.0, -1.0, 0.0);
			addGridObject(FACE_RIGHT, pnts, clrs);
			addGridObject(FACE_LEFT, pnts, clrs);
		}

		if ((axis.getGridLines().getPlaneMask() & JCGridLines.XY_PLANE) > 0) {
			pnts[0] = new Point3d(-1.0, -value, 0.0);
			pnts[1] = new Point3d(1.0, -value, 0.0);
			pnts[2] = new Point3d(1.0, 1.0, 0.0);
			pnts[3] = new Point3d(-1.0, 1.0, 0.0);
			addGridObject(FACE_NEAR, pnts, clrs);

			pnts[3] = new Point3d(-1.0, value, 0.0);
			pnts[2] = new Point3d(1.0, value, 0.0);
			pnts[1] = new Point3d(1.0, -1.0, 0.0);
			pnts[0] = new Point3d(-1.0, -1.0, 0.0);
			addGridObject(FACE_FAR, pnts, clrs);
		}
	}
	else if ( JCAxis.AXIS_Z == axis.getAxisId() ) {
		pnts[0] = new Point3d(-1.0, -value, 0.0);
		pnts[1] = new Point3d(1.0, -value, 0.0);
		pnts[2] = new Point3d(1.0, 1.0, 0.0);
		pnts[3] = new Point3d(-1.0, 1.0, 0.0);
		if ((axis.getGridLines().getPlaneMask() & JCGridLines.XZ_PLANE) > 0) {
			addGridObject(FACE_BOTTOM, pnts, clrs);
		}
		if ((axis.getGridLines().getPlaneMask() & JCGridLines.YZ_PLANE) > 0) {
			addGridObject(FACE_RIGHT, pnts, clrs);
		}

		pnts[3] = new Point3d(-1.0, value, 0.0);
		pnts[2] = new Point3d(1.0, value, 0.0);
		pnts[1] = new Point3d(1.0, -1.0, 0.0);
		pnts[0] = new Point3d(-1.0, -1.0, 0.0);
		if ((axis.getGridLines().getPlaneMask() & JCGridLines.XZ_PLANE) > 0) {
			addGridObject(FACE_TOP, pnts, clrs);
		}
		if ((axis.getGridLines().getPlaneMask() & JCGridLines.YZ_PLANE) > 0) {
			addGridObject(FACE_LEFT, pnts, clrs);
		}
	}
}

}
