/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCDataBars.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:06 $  $Locker:  $  Quest Software Inc.

//  JCDataBars.java

package com.klg.jclass.chart3d.j3d;

import java.awt.Color;
import com.sun.j3d.utils.picking.behaviors.*;
import com.sun.j3d.utils.picking.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.util.ArrayList;
import com.klg.jclass.chart3d.*;
import com.klg.jclass.chart3d.JCGridColor;

public abstract class JCDataBars extends JCBranchGroup {

protected final static float DEFAULT_SHININESS = 5.0f;
protected final static float DEFAULT_POLYGON_OFFSET = 1.0f;
protected final static float DEFAULT_POLYGON_OFFSET_FACTOR = 1.0f;
protected float polygonOffset = DEFAULT_POLYGON_OFFSET;
protected float polygonOffsetFactor = DEFAULT_POLYGON_OFFSET_FACTOR;
protected Chart3dDataView dataView = null;
protected int nXBars = 0;
protected int nYBars = 0;
protected GridLimits gridLimits = null;
protected JCContour contour = null;
protected JCContourLevels contourLevels = null;
protected ArrayList contourStyles = null;
protected JCShape3D[][] dataBars = null;

public JCDataBars(JCBar barFormat,
				  Chart3dGridData gd) {
	super();
	dataView = gd.getDataView();
	this.contour = dataView.getContour();
	this.contourLevels = contour.getContourLevels();
	this.contourStyles = contour.getContourStyles();
	this.gridLimits = gd.getGridLimits();

	if (barFormat.getXFormat() == JCBar.HISTOGRAM) {
		nXBars = gridLimits.getNumX() - 1;
	}
	else {
		nXBars = gridLimits.getNumX();
	}
	if (barFormat.getYFormat() == JCBar.HISTOGRAM) {
		nYBars = gridLimits.getNumY() - 1;
	}
	else {
		nYBars = gridLimits.getNumY();
	}

}

//  Call this method in the constructor of a subclass.
protected void createDataBars(JCBar barFormat,
							Matrix4d absolute,
							Chart3dGridData gd,
							boolean fUseForegroundColor,
							//  Need to set true if plotting 4D data using
							//  solid data bars with mesh lines.
							Color3f foregroundColor,
							boolean fSolid)
{
	double[] xGrid = gd.getXGrid();
	double[] yGrid = gd.getYGrid();
	double[][] zValues = gd.getZValues();
	double holeValue = gd.getHoleValue();

	double xFullWidth = gd.getMinXStep();
	double yFullWidth = gd.getMinYStep();
	dataBars = new JCShape3D[gridLimits.getNumX()][gridLimits.getNumY()];

	double xWidth = xFullWidth * (barFormat.getXSpacing() / 100.0);
	double yWidth = yFullWidth * (barFormat.getYSpacing() / 100.0);
	Vector3d vx = new Vector3d(xWidth, 0.0, 0.0);
	absolute.transform(vx);
	xWidth = vx.length();
	Vector3d vy = new Vector3d(0.0, yWidth, 0.0);
	absolute.transform(vy);
	yWidth = vy.length();

	double x = 0.0;
	double y = 0.0;

	Color3f clr3f = null;
	double zOrigin = dataView.getChart3d().getChart3dArea().getAxis(JCAxis.AXIS_Z).getOrigin();
	int index = 0;
	int yOffset = gridLimits.getYOffset();
	int xOffset = gridLimits.getXOffset();
	for (int j = yOffset; j < (yOffset + nYBars); j++) {
	    for (int i = xOffset; i < (xOffset + nXBars); i++) {
			if (zValues[i][j] != holeValue) {
			    if (barFormat.getXFormat() == JCBar.HISTOGRAM) {
				    x = xGrid[i] - (xFullWidth);
			    }
			    else {
				    x = xGrid[i] - (xFullWidth/2.0);
			    }
			    if (barFormat.getYFormat() == JCBar.HISTOGRAM) {
				    y = yGrid[j] + (yFullWidth);
			    }
			    else {
				    y = yGrid[j] + (yFullWidth/2.0);
			    }

				if (fUseForegroundColor) {
					clr3f = foregroundColor;
				}
				else {
					clr3f = getBarColor(i, j);
				}

			    dataBars[i][j] = createDataBar(absolute, x, y,
											   zValues[i][j], zOrigin, clr3f,
											   xWidth, yWidth, foregroundColor,
											   fSolid, index);
				//  In the case of contours, might get dataBars[i][j] == null.
				if (dataBars[i][j] != null) {
			        dataBars[i][j].setCapability(Shape3D.ENABLE_PICK_REPORTING);
			        PickTool.setCapabilities(dataBars[i][j], PickTool.INTERSECT_FULL);
			        addChild(dataBars[i][j]);
				}
			}
			index++;
			//  MUST increment index even if holeValue.
			//  Otherwise, won't be able to pick the correct bar.
		}
	}

	compile();
}

protected abstract JCShape3D createDataBar(Matrix4d absolute,
										   double x, double y, double zValue,
										   double zOrigin,
										   Color3f barColor,
										   double xWidth, double yWidth,
										   Color3f foregroundColor,
										   boolean fSolid, int index);

//  Returns the color associated with the z parameter.
protected Color3f getContourColor3f(double z)
{
	int level = contourLevels.getLevelFromValue(z);
	return getZoneColor(level);
}

protected Color3f getZoneColor(int index) {
	int contourStyleIndex = contour.contourIndex(index);
	JCContourStyle contourStyle =
			(JCContourStyle)contourStyles.get(contourStyleIndex);
	JCFillStyle fillStyle = contourStyle.getFillStyle();
	Color color = fillStyle.getColor();
	return (new Color3f(color));
}


protected Color3f getBarColor(int x, int y)
{
	Color3f clr3f = null;
    Chart3dGridData zoneData = dataView.getZoneData();
	if (zoneData.isDataOK() && contour.isZoned()) {
		//  return color for 4D data
		clr3f = getContourColor3f(zoneData.getZValue(x, y));
	}
	else {
		//  return a grid color
	    JCGridColor gridColor = dataView.findGridColor(x, y);
	    if (gridColor != null) {
		    Color color = gridColor.getColor();
		    if (color != null) {
			    clr3f = new Color3f(color);
		    }
	    }
	}
	return clr3f;
}

private void showDataBarsZoneColors(boolean fZoned, JCShape3D bar)
{
	if (bar == null) {
		return;
	}
	Appearance app = bar.getAppearance();
	JCRenderingAttributes renderAttrib =
						(JCRenderingAttributes)app.getRenderingAttributes();
	renderAttrib.setIgnoreVertexColors(!fZoned);
}

public void setDataBarsLit(boolean fLit)
{
	for (int x=0; x < nXBars; x++) {
		for (int y=0; y < nYBars; y++) {
			if (dataBars[x][y] == null) {
				continue;
			}
			JCAppearance app = (JCAppearance)dataBars[x][y].getAppearance();
			JCMaterial m = (JCMaterial)app.getMaterial();
			m.setLightingEnable(fLit);
		}
	}
}

public void showDataBarsZoned(boolean fZoned)
{
	if (dataBars != null) {
		for (int x=0; x < nXBars; x++) {
			for (int y=0; y < nYBars; y++) {
				showDataBarsZoneColors(fZoned, dataBars[x][y]);
			}
		}
	}
}

private void setShape3DVisible(JCShape3D shape, boolean fVisible)
{
	if (shape == null) {
		return;
	}
	JCAppearance app = (JCAppearance)shape.getAppearance();
	JCRenderingAttributes renderAttrib =
					    (JCRenderingAttributes)app.getRenderingAttributes();
	if (renderAttrib != null) {
	    renderAttrib.setVisible(fVisible);
	}

}

public void setDataBarsVisible(boolean fVisible)
{
	if (dataBars != null) {
		for (int x=0; x < nXBars; x++) {
			for (int y=0; y < nYBars; y++) {
			    setShape3DVisible(dataBars[x][y], fVisible);
			}
		}
	}
}

public void setSpecularColor(Color3f specularColor)
{
	if (dataBars == null) {
		return;
	}
	for (int x=0; x < nXBars; x++) {
		for (int y=0; y < nYBars; y++) {
			if (dataBars[x][y] == null) {
				continue;
			}
			JCAppearance app = (JCAppearance)dataBars[x][y].getAppearance();
			JCMaterial m = (JCMaterial)app.getMaterial();
			if (m != null) {
				m.setSpecularColor(specularColor);
			}
		}
	}
}

public void getSpecularColor(Color3f specularColor)
{
	if ((dataBars == null) || (dataBars[0][0] == null)) {
		return;
	}
	JCAppearance app = (JCAppearance)dataBars[0][0].getAppearance();
	JCMaterial m = (JCMaterial)app.getMaterial();
	if (m != null) {
		m.getSpecularColor(specularColor);
	}
}

public void setEmissiveColor(Color3f emissiveColor)
{
	if (dataBars == null) {
		return;
	}
	for (int x=0; x < nXBars; x++) {
		for (int y=0; y < nYBars; y++) {
			if (dataBars[x][y] == null) {
				continue;
			}
			JCAppearance app = (JCAppearance)dataBars[x][y].getAppearance();
			JCMaterial m = (JCMaterial)app.getMaterial();
			if (m != null) {
				m.setEmissiveColor(emissiveColor);
			}
		}
	}
}

public void getEmissiveColor(Color3f emissiveColor)
{
	if ((dataBars == null) || (dataBars[0][0] == null)) {
		return;
	}
	JCAppearance app = (JCAppearance)dataBars[0][0].getAppearance();
	JCMaterial m = (JCMaterial)app.getMaterial();
	if (m != null) {
		m.getEmissiveColor(emissiveColor);
	}
}

public void setAmbientColor(Color3f ambientColor)
{
	if (dataBars == null) {
		return;
	}
	for (int x=0; x < nXBars; x++) {
		for (int y=0; y < nYBars; y++) {
			if (dataBars[x][y] == null) {
				continue;
			}
			JCAppearance app = (JCAppearance)dataBars[x][y].getAppearance();
			JCMaterial m = (JCMaterial)app.getMaterial();
			if (m != null) {
				m.setAmbientColor(ambientColor);
			}
		}
	}
}

public void getAmbientColor(Color3f ambientColor)
{
	if ((dataBars == null) || (dataBars[0][0] == null)) {
		return;
	}
	JCAppearance app = (JCAppearance)dataBars[0][0].getAppearance();
	JCMaterial m = (JCMaterial)app.getMaterial();
	if (m != null) {
		m.getAmbientColor(ambientColor);
	}
}

public void setShininess(float shininess)
{
	if (dataBars == null) {
		return;
	}
	for (int x=0; x < nXBars; x++) {
		for (int y=0; y < nYBars; y++) {
			if (dataBars[x][y] == null) {
				continue;
			}
			JCAppearance app = (JCAppearance)dataBars[x][y].getAppearance();
			JCMaterial m = (JCMaterial)app.getMaterial();
			if (m != null) {
				m.setShininess(shininess);
			}
		}
	}
}

public float getShininessColor()
{
	float shininess = DEFAULT_SHININESS;
	if ((dataBars != null) && (dataBars[0][0] != null)) {
		JCAppearance app = (JCAppearance)dataBars[0][0].getAppearance();
		JCMaterial m = (JCMaterial)app.getMaterial();
		if (m != null) {
			shininess = m.getShininess();
		}
	}
	return shininess;
}

//  If flag is true, display the bars as wireframes.
public void setTransparent(boolean flag)
{
	if (dataBars == null) {
		return;
	}
	for (int x=0; x < nXBars; x++) {
		for (int y=0; y < nYBars; y++) {
			if (dataBars[x][y] == null) {
				continue;
			}
			JCAppearance app = (JCAppearance)dataBars[x][y].getAppearance();
			PolygonAttributes pa = app.getPolygonAttributes();
			if (pa == null) {
				pa = new PolygonAttributes();
				app.setPolygonAttributes(pa);
			}
			if (flag) {
				pa.setPolygonMode(PolygonAttributes.POLYGON_LINE);
				pa.setPolygonOffsetFactor(0.0f);
				pa.setPolygonOffset(0.0f);
			}
			else {
				pa.setPolygonMode(PolygonAttributes.POLYGON_FILL);
				pa.setBackFaceNormalFlip(true);
				pa.setPolygonOffsetFactor(polygonOffsetFactor);
				pa.setPolygonOffset(polygonOffset);
			}
		}
	}
}

}
