/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCDataPoints.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:06 $  $Locker:  $  Quest Software Inc.

//  JCDataPoints.java

package com.klg.jclass.chart3d.j3d;

import java.awt.Color;
import java.util.Enumeration;
import com.sun.j3d.utils.picking.behaviors.*;
import com.sun.j3d.utils.picking.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.util.ArrayList;
import com.klg.jclass.chart3d.*;
import com.klg.jclass.chart3d.shape3d.*;

public class JCDataPoints extends JCBranchGroup {

private JCChart3dAreaJava3d area = null;
protected double scale = 0.03;
private JCContour contour = null;
private JCContourLevels contourLevels = null;
private ArrayList contourStyles = null;
private JCColoringAttributes colorAttrib = null;

/**
 * Used for Point Data.
 * @param absolute
 * @param data
 * @param foregroundColor
 */
public JCDataPoints(Matrix4d absolute, Chart3dPointData data,
					Color3f foregroundColor)
{
	super();

	Chart3dDataViewJava3d dataView = (Chart3dDataViewJava3d)data.getDataView();
	JCChart3dJava3d chart3d = (JCChart3dJava3d)dataView.getChart3d();
	area = (JCChart3dAreaJava3d)chart3d.getChart3dArea();


	double holeValue = data.getHoleValue();
	for (int series = 0; series < data.getNumSeries(); series++) {
		Chart3dPointSeries pointSeries = data.getSeries(series);
		JCChart3dStyle chartStyle  = pointSeries.getChartStyle();
		for (int point = 0; point < pointSeries.getNumPoints(); point++) {
			Point3d point3d = new Point3d(pointSeries.getPoint(point));
		    if (point3d.z != holeValue) {
                absolute.transform(point3d);
				Data3dPointIndex dataIndex = new Data3dPointIndex(series, point);
			    addChild(new DataPoint(getChartStyleSymbol(chartStyle,
									   new Color3f(chartStyle.getSymbolColor())),
									   (scale * chartStyle.getSymbolSize()) / JCSymbolStyle.DEFAULT_SIZE,
									   point3d, dataIndex));
			}
		}
	}
}

/**
 * Used for Grid Data.
 * @param absolute
 * @param contour
 * @param data
 * @param foregroundColor
 * @param fUseContourColor
 */
public JCDataPoints(Matrix4d absolute, Chart3dGridData data,
					JCContour contour, Color3f foregroundColor,
					boolean fUseContourColor)
{
	super();
	Chart3dDataViewJava3d dataView = (Chart3dDataViewJava3d)data.getDataView();
	JCChart3dJava3d chart3d = (JCChart3dJava3d)dataView.getChart3d();
	area = (JCChart3dAreaJava3d)chart3d.getChart3dArea();

    Color3f color = null;
	this.contour = contour;
	this.contourLevels = contour.getContourLevels();
	this.contourStyles = contour.getContourStyles();
	int numX = data.getNumX();
	int numY = data.getNumY();
	double[] xGrid = data.getXGrid();
	double[] yGrid = data.getYGrid();
	double[][] zValues = data.getZValues();
	double holeValue = data.getHoleValue();
	GridLimits gridLimits = data.getGridLimits();
	JCChart3dStyle chartStyle = data.getChartStyle();

	int level = -1;
	Chart3dGridData zoneData = data.getDataView().getZoneData();
	int index = 0;
	int xOffset = gridLimits.getXOffset();
	int yOffset = gridLimits.getYOffset();
	for (int y = yOffset; y < (yOffset + numY); y++) {
	    for (int x = xOffset; x < (xOffset + numX); x++) {
			if (zValues[x][y] != holeValue) {
                Point3d point = new Point3d(xGrid[x], yGrid[y], zValues[x][y]);
                absolute.transform(point);
				Integer dataIndex = new Integer(index);
				if (contour.isContoured() || contour.isZoned()) {
					if (zoneData.isDataOK()) {
						//  4D data
			        	level = contourLevels.getLevelFromValue(zoneData.getZValue(x, y));
					}
					else {
			        	level = contourLevels.getLevelFromValue(zValues[x][y]);
					}
                    if (fUseContourColor) {
		                color = getContourColor3f(level);
                    }
                    else {
                        color = new Color3f(chartStyle.getSymbolColor());
                    }
			        addChild(new DataPoint(getContourStyleSymbol(level, color),
									    (scale * getContourStyle(level).getSymbolSize()) / JCSymbolStyle.DEFAULT_SIZE,
									    point, dataIndex));
				}
				else {
					if (chartStyle == null) {
						return;
					}
			        addChild(new DataPoint(getChartStyleSymbol(chartStyle,
									        new Color3f(chartStyle.getSymbolColor())),
									        (scale * chartStyle.getSymbolSize()) / JCSymbolStyle.DEFAULT_SIZE,
									        point, dataIndex));
				}
			}
			index++;
			//  MUST increment index even if holeValue is detected.
			//  Otherwise, won't be able to pick the correct data point.
        }
    }
}

//  Returns the color associated with the z parameter.
private Color3f getContourColor3f(int level)
{
	int contourStyleIndex = contour.contourIndex(level);
	JCContourStyle contourStyle =
				     (JCContourStyle)contourStyles.get(contourStyleIndex);
	JCFillStyle fillStyle = contourStyle.getFillStyle();
	Color color = fillStyle.getColor();
	return (new Color3f(color));
}

//  Returns the contourStyle associated with the z parameter.
private JCContourStyle getContourStyle(int level)
{
	int contourStyleIndex = contour.contourIndex(level);
	JCContourStyle contourStyle =
				     (JCContourStyle)contourStyles.get(contourStyleIndex);
    return contourStyle;
}

private Shape3D getSymbol(String shapeName)
{
	Object shapeInstance = area.getShapeInstances().get(shapeName);
	if (shapeInstance == null) {
		try {
	        shapeInstance = Class.forName(shapeName).newInstance();
		}
		catch(Exception e) {
		}
	    if (shapeInstance instanceof JCSymbolShape3d) {
		    area.getShapeInstances().put(shapeName, shapeInstance);
	    }
	}
	return ((JCSymbolShape3d)shapeInstance).getSymbolShape();
}

private Shape3D getChartStyleSymbol(JCChart3dStyle chartStyle, Color3f color)
{
	JCSymbolStyle symbolStyle = chartStyle.getSymbolStyle();
	String shapeName = symbolStyle.getShape();
	Shape3D shape = getSymbol(shapeName);
	Appearance app = createAppearance(color, symbolStyle.getSize());
	shape.setAppearance(app);
	return (shape);
}

private Shape3D getContourStyleSymbol(int level, Color3f color)
{
	int contourStyleIndex = contour.contourIndex(level);
	JCContourStyle contourStyle =
				     (JCContourStyle)contourStyles.get(contourStyleIndex);
	JCSymbolStyle symbolStyle = contourStyle.getSymbolStyle();

	String shapeName = symbolStyle.getShape();
	Shape3D shape = getSymbol(shapeName);
	Appearance app = createAppearance(color, symbolStyle.getSize());
	shape.setAppearance(app);

	return (shape);
}

private Appearance createAppearance(Color3f color, int size)
{
	Appearance appearance = new Appearance();

	JCRenderingAttributes renderAttrib = new JCRenderingAttributes();
	renderAttrib.setIgnoreVertexColors(true);
	appearance.setRenderingAttributes(renderAttrib);

    JCColoringAttributes colorAttrib = new JCColoringAttributes();
    colorAttrib.setColor(color);
	colorAttrib.setShadeModel(ColoringAttributes.NICEST);
    appearance.setColoringAttributes(colorAttrib);

	JCMaterial material = new JCMaterial(color, 64.0f);
	material.setLightingEnable(true);
	appearance.setMaterial(material) ;

	//  Applies only to point primitives.
	PointAttributes pointAttrib = new PointAttributes(size, true);
	appearance.setPointAttributes(pointAttrib);

	return (appearance);
}

public void setScale(double scale)
{
	this.scale = scale;
	Transform3D t3d =  new Transform3D();
	Enumeration enum = getAllChildren();
	while (enum.hasMoreElements()) {
		DataPoint point = (DataPoint)enum.nextElement();
		//  DataPoint extends TransformGroup
		point.getTransform(t3d);
		t3d.setScale(scale);
		point.setTransform(t3d);
	}
}

public double getScale()
{
	return scale;
}

public void setRotation(double angleX, double angleY, double angleZ)
{
	double radianX, radianY, radianZ;
	//  Convert degrees to radians:
	radianX = Math.toRadians(angleX);
	radianY = Math.toRadians(angleY);
	radianZ = Math.toRadians(angleZ);

	Matrix3d rotation = new Matrix3d();
	rotation.rotX((float)radianX);
	Matrix3d rot = new Matrix3d();
	rot.rotY((float)radianY);
	rotation.mul(rot);
	rot.rotZ((float)radianZ);
	rotation.mul(rot);

	Transform3D t3d =  new Transform3D();
	Enumeration enum = getAllChildren();
	while (enum.hasMoreElements()) {
		DataPoint point = (DataPoint)enum.nextElement();
		//  DataPoint extends TransformGroup
		point.getTransform(t3d);
		t3d.setRotation(rotation);
		point.setTransform(t3d);
	}
}

}
