/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCPlotCubeJava3d.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:06 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.j3d;

import java.awt.Color;
import java.awt.Font;
import java.awt.image.*;
import com.sun.j3d.utils.picking.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import javax.swing.*;
import java.util.BitSet;
import com.klg.jclass.chart3d.*;

/**
 * The <code>JCPlotCubeJava3d</code> class creates the cube that surrounds the
 * surface, bar, or scatter plot.
 */
public class JCPlotCubeJava3d extends JCPlotCube {

private boolean fDisplaySolidSurfaces = true;
private boolean fDisplayWireFrame = false;
private boolean ceilingProjectionCreated = false;
private boolean floorProjectionCreated = false;
private Switch objSwitch = null;
private BitSet objSwitchChildMask;
private final int MAX_OBJECT_SWITCH_CHILDREN = 16;
private final int PLOTCUBE_WALLS = 0;
private final int PLOTCUBE_WIREFRAME = 1;
private float defaultTransparency = 0.35f;

private JCCubeSquareArray csaPlotCubeWalls = null;
private Shape3D shPlotCubeWireFrame = null;
private Color wireframeColor = null;
private Color outlineColor = null;
private JCBranchGroup bgPlotCube = null;
private JCBranchGroup bgData = null;
private JCBranchGroup bgPickAxis = null;
private JCBranchGroup bgAnnotation = null;
private PlotCubeTransformGroup plotCubeTG = null;
private JCTransformGroup scaleTG = null;

/**
 * Constructor given a parent chart3d area.  All other values at default.
 *
 * @param chart3dAreaX the parent chart3d area
 */
public JCPlotCubeJava3d(JCChart3dAreaJava3d chart3dAreaX) {
	super(chart3dAreaX);
	foreground = Color.green;
	background = Color.white;
	wireframeColor = Color.red;
	outlineColor = Color.black;
	bgPlotCube = createPlotCube(0.0f, 0.0f, 0.0f);
	JCView3dJava3d view3d = (JCView3dJava3d)chart3dAreaX.getView3d();
	if (view3d != null) {
	    rotate(view3d.getXRotation(), view3d.getYRotation(),
			   view3d.getZRotation());
	}
	else {
		rotate(45.0, 0.0, 45.0);
	}
}

private JCBranchGroup createPlotCube(double xpos, double ypos, double zpos)
{
	Geometry geom = null;
	Color3f foregroundColor3f = new Color3f(foreground);
	Color3f backgroundColor3f = new Color3f(background);

	JCBranchGroup branchGroupPlotCube = new JCBranchGroup();

	// Create a TransformGroup node:
	Transform3D t = new Transform3D();
	t.setTranslation( new Vector3d(xpos, ypos, zpos) );
	plotCubeTG = new PlotCubeTransformGroup((JCChart3dAreaJava3d)area, t);
	plotCubeTG.setCapability(TransformGroup.ENABLE_PICK_REPORTING);

	// Need a separate transform group to allow scaling:
	Transform3D scaleT3D = new Transform3D();
	scaleT3D.setScale(1.0);
	scaleTG = new JCTransformGroup(scaleT3D);

	// Create a switch group to hold the plotcube walls and wireframe.
	// This enables us to select objects to display.  Parameter is
	// number of bits
	objSwitchChildMask = new BitSet(MAX_OBJECT_SWITCH_CHILDREN);
	objSwitch = new Switch( Switch.CHILD_MASK, objSwitchChildMask );
	objSwitch.setCapability( Switch.ALLOW_SWITCH_READ );
	objSwitch.setCapability( Switch.ALLOW_SWITCH_WRITE );
	objSwitch.setCapability( Switch.ALLOW_CHILDREN_READ );
	objSwitch.setCapability( Switch.ALLOW_CHILDREN_WRITE );
	objSwitch.setCapability( Switch.ALLOW_CHILDREN_EXTEND );

	// Add the solid cube:
	csaPlotCubeWalls = new JCCubeSquareArray(backgroundColor3f,
										     new Color3f(outlineColor));
											 //  extends JCBranchGroup
	objSwitch.addChild(csaPlotCubeWalls);

	//  Add a JCBranchGroup to contain the wireframe.
	JCBranchGroup bgPlotCubeWireFrame = new JCBranchGroup();
	geom = new JCCubeLA();

	//  Set appearance for wireframe:
	JCAppearance appearance = new JCAppearance();
	Material m = new Material() ;
	m.setLightingEnable(false) ;
	appearance.setMaterial(m) ;
	JCColoringAttributes colorAttrib = new JCColoringAttributes();
	colorAttrib.setColor(new Color3f(wireframeColor));
	appearance.setColoringAttributes(colorAttrib);

	shPlotCubeWireFrame = new Shape3D(geom,appearance);
	shPlotCubeWireFrame.setCapability(Shape3D.ALLOW_APPEARANCE_READ);
	shPlotCubeWireFrame.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
	shPlotCubeWireFrame.setCapability(Shape3D.ENABLE_PICK_REPORTING);
	PickTool.setCapabilities(shPlotCubeWireFrame, PickTool.INTERSECT_FULL);
	bgPlotCubeWireFrame.addChild(shPlotCubeWireFrame);

	objSwitch.addChild(bgPlotCubeWireFrame);

	//  Add Axes:
	//  Note:  Add or **Detach** a JCBranchGroup to/from a JCBranchGroup
	bgAnnotation = new JCBranchGroup();
	plotCubeTG.addChild(bgAnnotation);

	scaleTG.addChild(objSwitch);

	setSolidWallsVisible(fDisplaySolidSurfaces);
	setWireFrameVisible(fDisplayWireFrame);


 

    bgData = new JCBranchGroup();
    plotCubeTG.addChild(bgData);

	bgPickAxis = new JCBranchGroup();
	plotCubeTG.addChild(bgPickAxis);

	plotCubeTG.addChild(scaleTG);
	branchGroupPlotCube.addChild(plotCubeTG);

	branchGroupPlotCube.compile();
	return branchGroupPlotCube;
}

/**
 * Creates projections of the chart on to the floor and ceiling of the plotcube.
 */
public void createProjections()
{
	if (!hasCeilingProjection()) {
		removeCeilingProjection();
	}
	if (!hasFloorProjection()) {
		removeFloorProjection();
	}

	if (!hasProjections()) {
		return;
	}

	BufferedImage image = null;
	JCTexture2D texture = null;

	JCContour contour = area.getChart3d().getDataView(0).getContour();
	//  Check if we need to recalculate projection images:
	contour.recalcContours();
	if (hasFloorProjection()) {
		image = new BufferedImage(512, 512,
								  BufferedImage.TYPE_INT_ARGB);
		createParallelProjection(image, floor);
		texture = JCTexture2D.createTexture(
									image,
									true,
									SwingConstants.CENTER,
									SwingConstants.CENTER,
									Color.white);
		setWallTexture(JCCubeSquareArray.FACE_FAR, texture, 0);
		setWallOuterFaceVisible(JCCubeSquareArray.FACE_FAR, true);
		floorProjectionCreated = true;
	}
	if (hasCeilingProjection()) {
		image = new BufferedImage(512, 512,
								  BufferedImage.TYPE_INT_ARGB);
		createParallelProjection(image, ceiling);
		texture = JCTexture2D.createTexture(
									image,
									true,
									SwingConstants.CENTER,
									SwingConstants.CENTER,
									Color.white);
		setWallTexture(JCCubeSquareArray.FACE_NEAR, texture, 0);
		setWallOuterFaceVisible(JCCubeSquareArray.FACE_NEAR, true);
		if (getCeiling().isContoured() && !getCeiling().isZoned()) {
		    setWallTransparency(JCCubeSquareArray.FACE_NEAR, defaultTransparency);
		}
		else {
		    setWallTransparency(JCCubeSquareArray.FACE_NEAR, 0.0f);
		}
		//  Flip the FACE_NEAR wall image about the x-axis:
		if (!ceilingProjectionCreated) {
		    flipWallImage(JCCubeSquareArray.FACE_NEAR, JCAxis.AXIS_X);
		}
		ceilingProjectionCreated = true;
	}
	image.flush();
	image = null;
	texture = null;
	contour.destroyContours();
	setWireFrameVisible(true);
}

/**
 * Removes chart projection from the ceiling of the plotcube.
 */
public void removeCeilingProjection()
{
	if (ceilingProjectionCreated) {
	    setWallTexture(JCCubeSquareArray.FACE_NEAR, null, 0);
	    setWallTransparency(JCCubeSquareArray.FACE_NEAR, 0.0f);
	    setWallOuterFaceVisible(JCCubeSquareArray.FACE_NEAR, false);
	    //  Unflip the FACE_NEAR wall:
	    flipWallImage(JCCubeSquareArray.FACE_NEAR, JCAxis.AXIS_X);
	    setWireFrameVisible(false);
		ceilingProjectionCreated = false;
	}
}

/**
 * Removes chart projection from the floor of the plotcube.
 */
public void removeFloorProjection()
{
	if (floorProjectionCreated) {
	    setWallTexture(JCCubeSquareArray.FACE_FAR,  null, 0);
	    setWallTransparency(JCCubeSquareArray.FACE_FAR,  0.0f);
	    setWallOuterFaceVisible(JCCubeSquareArray.FACE_FAR,  false);
	    setWireFrameVisible(false);
		floorProjectionCreated = false;
	}
}

/**
 * Replaces existing floor and ceiling projections.
 */
public void updateProjections()
{
	removeCeilingProjection();
	removeFloorProjection();
	createProjections();
}

protected void addGridLines()
{
	for (int i = 0; i < 6; i++) {
		csaPlotCubeWalls.clearObjects(i);
	}

	Color gridLineColor = ((JCGridLinesJava3d)area.getAxis(JCAxis.AXIS_X).getGridLines()).getLineStyle().getColor();
	((JCGridLinesJava3d)area.getAxis(JCAxis.AXIS_X).getGridLines()).drawGridFace(csaPlotCubeWalls, gridLineColor);
	gridLineColor = ((JCGridLinesJava3d)area.getAxis(JCAxis.AXIS_Y).getGridLines()).getLineStyle().getColor();
	((JCGridLinesJava3d)area.getAxis(JCAxis.AXIS_Y).getGridLines()).drawGridFace(csaPlotCubeWalls, gridLineColor);
	gridLineColor = ((JCGridLinesJava3d)area.getAxis(JCAxis.AXIS_Z).getGridLines()).getLineStyle().getColor();
	((JCGridLinesJava3d)area.getAxis(JCAxis.AXIS_Z).getGridLines()).drawGridFace(csaPlotCubeWalls, gridLineColor);
}

/**
 * Returns the <code>BranchGroup</code> containing the plotcube.
 *
 * @return <code>BranchGroup</code> containing the plotcube
 */
public JCBranchGroup getContent() {
	return bgPlotCube;
}

/**
 * Returns the <code>TransformGroup</code> containing the plotcube.
 *
 * @return the <code>PlotCubeTransformGroup</code> containing the plotcube
 */
public PlotCubeTransformGroup getTransformGroup() {
	return plotCubeTG;
}

//  perspective = eye_plotcube_distance / 2
protected void setPerspective()
{
	JCView3d view = area.getView3d();
	if (view != null) {
	    double perspective = view.getPerspective();
	    double viewPlatformDistance = ((JCChart3dAreaJava3d)area).getChart3dUniverse().getVirtualUniverse().getViewPlatformDistance();
	    double dz = -(perspective * 2.0) + viewPlatformDistance;
	    translate( new Vector3d(0.0, 0.0, dz) );
	}
}

private void setColor(Shape3D shape, Color color)
{
	JCColoringAttributes colorAttrib = null;

	JCAppearance app = (JCAppearance)shape.getAppearance();
	colorAttrib = (JCColoringAttributes)app.getColoringAttributes();

	if (null != colorAttrib) {
		colorAttrib.setColor(new Color3f(color));
	}
	else {
		colorAttrib = new JCColoringAttributes(new Color3f(color), ColoringAttributes.NICEST);
		app.setColoringAttributes(colorAttrib);
	}
}

/**
 * Displays or hides a wireframe outline of the plotcube.
 *
 * @param fVisible <code>true</code> to display the wireframe, <code>false</code> to hide it
 */
public void setWireFrameVisible(boolean fVisible)
{
	fDisplayWireFrame = fVisible;
	if (fDisplayWireFrame) {
		objSwitchChildMask.set(PLOTCUBE_WIREFRAME);
	}
	else {
	    objSwitchChildMask.clear(PLOTCUBE_WIREFRAME);
	}
	objSwitch.setChildMask(objSwitchChildMask);
}

/**
 * Sets the color of the wireframe outline of the plotcube.
 *
 * @param color the desired color of the plotcube wireframe
 */
public void setWireframeColor(Color color) {
	setColor(shPlotCubeWireFrame, color);
}

/**
 * Returns the color of the wireframe outline of the plotcube.
 *
 * @return the color of the plotcube's wireframe
 */
public Color getWireframeColor() {
	return(wireframeColor);
}

/**
 * Displays or hides the solid walls of the plotcube.
 *
 * @param fVisible <code>true</code> to display the solid walls, <code>false</code> to hide them
 */
public void setSolidWallsVisible(boolean fVisible)
{
	fDisplaySolidSurfaces = fVisible;
	if (fDisplaySolidSurfaces) {
		objSwitchChildMask.set(PLOTCUBE_WALLS);
	}
	else {
	    objSwitchChildMask.clear(PLOTCUBE_WALLS);
	}
	objSwitch.setChildMask(objSwitchChildMask);
}

/**
 * The solid walls of the plotcube have both a fill color and an
 * outline color.  This method sets the outline color (not to be
 * confused with the wireframe color).
 *
 * @param color the desired outline color for the solid plotcube walls
 */
public void setOutlineColor(Color color) {
	outlineColor = color;
	for (int face = 0; face < JCCubeSquareArray.NUM_FACES; face++) {
	    JCShape3D wall = csaPlotCubeWalls.getFaceOutline(face);
	    setColor(wall, color);
	}
}

/**
 * Returns the outline color of the solid walls of the plotcube.
 *
 * @return the outline color of the solid plotcube walls
 */
public Color getOutlineColor() {
	return outlineColor;
}

/**
 * Sets the foreground color for the plotcube.
 *
 * @param foreground the desired foreground color of the plotcube
 */
public void setForeground(Color foreground)
{
	super.setForeground(foreground);
}

/**
 * Sets the fill color of the solid walls of the plotcube.
 *
 * @param color the desired fill color for the solid plotcube walls
 */
public void setBackground(Color color) {
	background = color;
	for (int face = 0; face < JCCubeSquareArray.NUM_FACES; face++) {
	    JCShape3D wall = csaPlotCubeWalls.getFace(face);
	    setColor(wall, color);
	}
}
//  getBackground() method exists in base class

public void setWallImage(int face, String filename, double angle) {
	setWallTexture(face, JCTexture2D.createTexture(filename, Color.white), angle);
}

public void setWallTexture(int face, JCTexture2D texture, double angle) {
	csaPlotCubeWalls.applyTexture(face, texture);
    csaPlotCubeWalls.setWallImageRotation(face, angle);
}

public void setWallImage(String filename) {
	setTexture(JCTexture2D.createTexture(filename, Color.white));
}

public void rotateWallImage(int face, double angle) {
    csaPlotCubeWalls.setWallImageRotation(face, angle);
}

public double getWallImageAngle(int face) {
	return csaPlotCubeWalls.getWallImageRotation(face);
}

/**
 * Flips the image 180 degrees about the specified axis.
 * @param face
 * @param axis
 */

public void flipWallImage(int face, int axis) {
	csaPlotCubeWalls.flipWallImage(face, axis);
}

public void setTexture(JCTexture2D texture) {
	for (int face = 0; face < JCCubeSquareArray.NUM_FACES; face++) {
		csaPlotCubeWalls.applyTexture(face, texture);
	}
}

/**
 * Sets the visibility of the exterior face of a plotcube wall.
 *
 * @param face an integer value identifying a particular wall of the plotcube
 * @param flag <code>true</code> to set the exterior face visible, <code>false</code> to hide the exterior face
 */
public void setWallOuterFaceVisible(int face, boolean flag)
{
	JCShape3D wall = csaPlotCubeWalls.getFace(face);
	JCAppearance appearance = (JCAppearance)wall.getAppearance();
	PolygonAttributes pa = appearance.getPolygonAttributes();
	if (flag) {
		//  Display front and back faces.
		pa.setCullFace(PolygonAttributes.CULL_NONE);
	}
	else {
		pa.setCullFace(PolygonAttributes.CULL_BACK);
	}
	pa.setBackFaceNormalFlip(flag);
}

/**
 * Sets the default transparency value for the solid walls of the plotcube.
 *
 * @param value the default transparency value, with 0.0 being fully opaque and 1.0 being fully transparent
 */
public void setDefaultTransparency(float value)
{
	defaultTransparency = value;
}

/**
 * Returns the default transparency value for the solid walls of the plotcube.
 *
 * @return the default transparency value, with 0.0 being fully opaque and 1.0 being fully transparent
 */
public float getDefaultTransparency()
{
	return defaultTransparency;
}

/**
 * Sets the transparency for a specified wall of the plotcube.
 *
 * @param face an integer value identifying a particular wall of the plotcube
 * @param transparency the transparency value, with 0.0 being fully opaque and 1.0 being fully transparent
 */
public void setWallTransparency(int face, float transparency)
{
	JCShape3D wall = csaPlotCubeWalls.getFace(face);
	JCAppearance appearance = (JCAppearance)wall.getAppearance();
	TransparencyAttributes ta = appearance.getTransparencyAttributes();
	ta.setTransparencyMode(TransparencyAttributes.SCREEN_DOOR);
	ta.setTransparency(transparency);
}

//  Make sure we use the TransformGroup that has ENABLE_PICK_REPORTING.
//  Otherwise, the mouse/picking behaviors get messed up.
protected void rotate(double angleX, double angleY, double angleZ)
{
	double radianX, radianY, radianZ;
	//  Convert degrees to radians:
	radianX = Math.toRadians(angleX - 90.0);
	//  We want to be facing the XZ plane when xAngle, yAngle, and zAngle are 0.
	radianY = Math.toRadians(angleY);
	radianZ = Math.toRadians(angleZ);

	Transform3D t3d = new Transform3D();
	plotCubeTG.getTransform(t3d);

	Matrix3d rotation = new Matrix3d();
	rotation.rotX((float)radianX);
	Matrix3d rot = new Matrix3d();
	rot.rotY((float)radianY);
	rotation.mul(rot);
	rot.rotZ((float)radianZ);
	rotation.mul(rot);
	t3d.setRotation(rotation);

	plotCubeTG.setTransform(t3d);
}

protected void translate(Vector3d translation)
{
	Transform3D t3d = new Transform3D();
	plotCubeTG.getTransform(t3d);
	t3d.setTranslation(translation);
	plotCubeTG.setTransform(t3d);
}

protected void getTranslation(Vector3d translation)
{
	Transform3D t3d = new Transform3D();
	plotCubeTG.getTransform(t3d);
	t3d.get(translation);
}

private void scale() {
	Transform3D t3d = new Transform3D();
	scaleTG.getTransform(t3d);
	t3d.setScale( new Vector3d(xNormalizedScale, yNormalizedScale,
						       zNormalizedScale) );
	scaleTG.setTransform(t3d);
}

/**
 * Sets the <code>PlotCube</code>'s <i>x</i> scale (default value is 1.0).
 *
 * @param xScale the new <i>x</i> scale
 */
public void setXScale(double xScale) {
	super.setXScale(xScale);
	scale();
}

/**
 * Sets the <code>PlotCube</code>'s <i>y</i> scale (default value is 1.0).
 *
 * @param yScale the new <i>y</i> scale
 */
public void setYScale(double yScale) {
	super.setYScale(yScale);
	scale();
}

/**
 * Sets the <code>PlotCube</code>'s <i>z</i> scale (default value is 1.0).
 *
 * @param zScale the new <i>z</i> scale
 */
public void setZScale(double zScale) {
	super.setZScale(zScale);
	scale();
}

/**
 * Applies a uniform scaling factor to the plotcube.
 *
 * @param scale the new <i>z</i> scale
 */
public void setScale(double scale) {
	Transform3D t3d = new Transform3D();
	plotCubeTG.getTransform(t3d);
	t3d.setScale(scale);
	plotCubeTG.setTransform(t3d);
}

protected void addDataGroup(BranchGroup bgDataGroup) {
	//  Note:  Unable to add children to object switch once it becomes live.
	bgData.addChild(bgDataGroup);
}

protected void removeAllDataGroups() {
	bgData.removeAllChildren();
}

/**
 * Adds a marker to the chart to show a pick location.
 *
 * @param bgPick a <code>BranchGroup</code> containing the marker
 */
public void addPickAxis(BranchGroup bgPick) {
	bgPickAxis.addChild(bgPick);
}

/**
 * Removes a pick marker from the chart.
 */
public void removePickAxis() {
	bgPickAxis.removeChild(0);
}

/**
 * Adds a <code>BranchGroup</code> containing axis annotation.
 *
 * @param bg a <code>BranchGroup</code> containing the annotation
 */
public void addAnnotationBranchGroup(JCBranchGroup bg)
{
	bgAnnotation.addChild(bg);
}

/**
 * Removes all <code>BranchGroups</code> containing annotation.
 */
public void removeAllAnnotation()
{
	bgAnnotation.removeAllChildren();
}

/**
 * Returns the scaling transform for the plotcube.
 *
 * @return the <code>Transform3D</code> used to scale the plotcube
 */
public Transform3D getPlotCubeScaling() {
	Transform3D t3d = new Transform3D();
	scaleTG.getTransform(t3d);
	return t3d;
}

private TransformGroup createWatermarkText(String text, Color3f color,
										   double scale, Vector3d trans)
{
	Transform3D textTransform = new Transform3D();
	textTransform.setTranslation(trans);
	textTransform.setScale(scale);
	TransformGroup textTG = new TransformGroup(textTransform);
	Font3D f3d = new Font3D(new Font("Arial", Font.BOLD, 2),
						    new FontExtrusion());
	Text3D txt = new Text3D(f3d, text,
						    new Point3f(0.0f, 0.0f, 0.0f),
						    Text3D.ALIGN_CENTER,
						    Text3D.PATH_RIGHT);
	OrientedShape3D sh = new OrientedShape3D();
	sh.setCapability(OrientedShape3D.ALLOW_GEOMETRY_READ);  //  Otherwise, get exception during picking.
	sh.setAlignmentMode(OrientedShape3D.ROTATE_ABOUT_POINT);
	sh.setRotationPoint( new Point3f((float)trans.x, (float)trans.y, (float)trans.z) );
	Appearance app = new Appearance();
	Material mm = new Material();
	mm.setLightingEnable(false);
	app.setMaterial(mm);
	ColoringAttributes colorAttrib = new ColoringAttributes();
	colorAttrib.setColor(color);
	app.setColoringAttributes(colorAttrib);

	TransparencyAttributes transparency =
			new TransparencyAttributes(TransparencyAttributes.NICEST, 0.5f);
	app.setTransparencyAttributes(transparency);

	sh.setGeometry(txt);
	sh.setAppearance(app);
	textTG.addChild(sh);

	return textTG;
}

}
