/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCSolidDataSurface.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:06 $  $Locker:  $  Quest Software Inc.

//  JCSolidDataSurface.java

package com.klg.jclass.chart3d.j3d;

import com.sun.j3d.utils.picking.behaviors.*;
import com.sun.j3d.utils.picking.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.util.ArrayList;
import com.klg.jclass.chart3d.*;

public class JCSolidDataSurface extends JCBranchGroup {

protected final static int SOLID_DATA_SURFACE = 100;
protected final static float DEFAULT_SHININESS = 5.0f;
private JCShape3D solidSurface = null;

private void showSolidSurfaceZoneColors(boolean fZoned, JCShape3D surface) {
	Appearance app = surface.getAppearance();
	JCRenderingAttributes renderAttrib =
						(JCRenderingAttributes)app.getRenderingAttributes();
	renderAttrib.setIgnoreVertexColors(!fZoned);
}

public void setSolidSurfaceShaded(boolean fShaded) {
	JCAppearance app = (JCAppearance)solidSurface.getAppearance();
	JCMaterial m = (JCMaterial)app.getMaterial();
	m.setLightingEnable(fShaded);
}


public void setSolidSurfaceZoned(boolean fZoned) {
	if (solidSurface != null) {
		showSolidSurfaceZoneColors(fZoned, solidSurface);
	}
}

private void setShape3DVisible(JCShape3D shape, boolean fVisible) {
	if (shape == null) {
		return;
	}
	JCAppearance app = (JCAppearance)shape.getAppearance();
	JCRenderingAttributes renderAttrib =
					    (JCRenderingAttributes)app.getRenderingAttributes();
	if (renderAttrib != null) {
	    renderAttrib.setVisible(fVisible);
	}

}

public void setSolidSurfaceVisible(boolean fVisible) {
	setShape3DVisible(solidSurface, fVisible);
}

public void setSpecularColor(Color3f specularColor) {
	if (solidSurface == null) {
		return;
	}
	JCAppearance app = (JCAppearance)solidSurface.getAppearance();
	JCMaterial m = (JCMaterial)app.getMaterial();
	if (m != null) {
		m.setSpecularColor(specularColor);
	}
}

public void getSpecularColor(Color3f specularColor) {
	if (solidSurface == null) {
		return;
	}
	JCAppearance app = (JCAppearance)solidSurface.getAppearance();
	JCMaterial m = (JCMaterial)app.getMaterial();
	if (m != null) {
		m.getSpecularColor(specularColor);
	}
}

public void setEmissiveColor(Color3f emissiveColor) {
	if (solidSurface == null) {
		return;
	}
	JCAppearance app = (JCAppearance)solidSurface.getAppearance();
	JCMaterial m = (JCMaterial)app.getMaterial();
	if (m != null) {
		m.setEmissiveColor(emissiveColor);
	}
}

public void getEmissiveColor(Color3f emissiveColor) {
	if (solidSurface == null) {
		return;
	}
	JCAppearance app = (JCAppearance)solidSurface.getAppearance();
	JCMaterial m = (JCMaterial)app.getMaterial();
	if (m != null) {
		m.getEmissiveColor(emissiveColor);
	}
}

public void setAmbientColor(Color3f ambientColor) {
	if (solidSurface == null) {
		return;
	}
	JCAppearance app = (JCAppearance)solidSurface.getAppearance();
	JCMaterial m = (JCMaterial)app.getMaterial();
	if (m != null) {
		m.setAmbientColor(ambientColor);
	}
}

public void getAmbientColor(Color3f ambientColor) {
	if (solidSurface == null) {
		return;
	}
	JCAppearance app = (JCAppearance)solidSurface.getAppearance();
	JCMaterial m = (JCMaterial)app.getMaterial();
	if (m != null) {
		m.getAmbientColor(ambientColor);
	}
}

public void setShininess(float shininess) {
	if (solidSurface == null) {
		return;
	}
	JCAppearance app = (JCAppearance)solidSurface.getAppearance();
	JCMaterial m = (JCMaterial)app.getMaterial();
	if (m != null) {
		m.setShininess(shininess);
	}
}

public float getShininessColor() {
	float shininess = DEFAULT_SHININESS;
	if (solidSurface != null) {
		JCAppearance app = (JCAppearance)solidSurface.getAppearance();
		JCMaterial m = (JCMaterial)app.getMaterial();
		if (m != null) {
			shininess = m.getShininess();
		}
	}
	return shininess;
}

public JCSolidDataSurface( Matrix4d absolute,
						   Chart3dGridData data,
                           JCContour contour,
						   Color3f foregroundColor) {
	super();

	GeometryArray geometryArray = null;
	if (contour.getZoneMethod() == JCContour.ZONE_CONTOURS) {
		DataSurfaceTA surface = new DataSurfaceTA(absolute, data, contour);
		geometryArray = surface.getGeometryArray();
	}
	else {
		DataSurfaceQA surface = new DataSurfaceQA(absolute, data, foregroundColor, contour);
		geometryArray = surface.getGeometryArray();
	}

	JCAppearance appearance = new JCAppearance();

	PolygonAttributes pa = appearance.getPolygonAttributes();
	if (pa == null) {
		pa = new PolygonAttributes();
		appearance.setPolygonAttributes(pa);
	}
	//  We want to display front and back faces.
	pa.setCullFace(PolygonAttributes.CULL_NONE);
	pa.setBackFaceNormalFlip(true);
	pa.setPolygonOffsetFactor(1.0f);
	pa.setPolygonOffset(1.0f);

	JCRenderingAttributes renderAttrib = new JCRenderingAttributes();
	renderAttrib.setVisible(true);  //  default
	appearance.setRenderingAttributes(renderAttrib);

	JCMaterial material = new JCMaterial(foregroundColor, DEFAULT_SHININESS);
	material.setLightingEnable(false) ;  // default

	JCColoringAttributes colorAttrib = new JCColoringAttributes();
	colorAttrib.setColor(foregroundColor);
	colorAttrib.setShadeModel(ColoringAttributes.NICEST);

	appearance.setMaterial(material) ;
	appearance.setColoringAttributes(colorAttrib);

	solidSurface = new JCShape3D(geometryArray, appearance,
								 SOLID_DATA_SURFACE);
	setSolidSurfaceZoned(true);  // default
	Chart3dDataView dataView = data.getDataView();
	JCChart3dArea area = dataView.getChart3d().getChart3dArea();
	if (area.getSurface().isSolid()) {
		SkirtQA skirt = new SkirtQA(absolute, data, foregroundColor);
		geometryArray = skirt.getGeometryArray();
		solidSurface.addGeometry(geometryArray);
	}

	solidSurface.setCapability(Shape3D.ENABLE_PICK_REPORTING);
	PickTool.setCapabilities(solidSurface, PickTool.INTERSECT_FULL);
	addChild(solidSurface);

	compile();
}

}
