/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCTexture2D.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:06 $  $Locker:  $  Quest Software Inc.

package  com.klg.jclass.chart3d.j3d;

import java.awt.Color;
import java.awt.Rectangle;
import java.awt.Component;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;

import javax.media.j3d.Texture;
import javax.media.j3d.Texture2D;
import javax.media.j3d.ImageComponent2D;

import javax.swing.SwingConstants;

import com.sun.j3d.utils.image.TextureLoader;


public class JCTexture2D extends Texture2D implements SwingConstants {

private int		width;        // width of this texture object
private int		height;       // height of this texture object
private int		imageXOffset; // x locn of the source image
private int		imageYOffset; // y locn of the source image
private int		imageWidth;   // width of the source image
private int		imageHeight;  // height of the source image

/**
 * Constructs a JCTexture2D object from an image file specified
 * by filename.  If Java Advanced Imaging is installed then it is
 * used to load the images, otherwise AWT toolkit it used.  If the
 * source image does not have a width and height equal to a power of
 * two, an enlarged texture will be created for you which meets the
 * power of two criteria.  In this case, the image will be by default
 * centred in the texture, and any texels not covered by the image
 * will have the specified fill color.
 *
 * @param filename - image file used to construct the JCTexture2D object
 * @param fill - color to use fill in the background if the loaded image
 * does not have width and height equal to a power of two
 */
public static JCTexture2D createTexture(String filename, Color fill) {

	return createTexture(filename, true, CENTER, CENTER, fill);
}

/**
 * Constructs a JCTexture2D object from an image file specified
 * by filename.  If Java Advanced Imaging is installed then it is
 * used to load the images, otherwise AWT toolkit it used.  If the
 * source image does not have a width and height equal to a power of
 * two, an enlarged texture will be created for you which meets the
 * power of two criteria.  In this case, the image will be aligned within
 * the texture object according to the hAlignment and vAlignment constants,
 * and any texels not covered by the image will have the specified fill color.
 *
 * @param filename - image file used to construct the JCTexture2D object
 * @param square - a flag to indicate wheteher the texture should have
 * @param hAlignment - horizontal alignment of the image within the
 * texture - one of SwingConstants.LEFT, CENTER, or RIGHT
 * @param vAlignment - vertical alignment of the image within the
 * texture - one of SwingConstants.TOP, CENTER, or BOTTOM
 * @param fill - color to use fill in the background if the loaded image
 * does not have width and height equal to a power of two
 */
public static JCTexture2D createTexture(String filename,
										boolean square,
										int hAlignment,
										int vAlignment,
										Color fill) {


	final Component imageObserver = new Component() {

	};


	TextureLoader loader = new TextureLoader(filename, TextureLoader.Y_UP, imageObserver);

	if (loader.getImage() == null) {
		System.out.println("createTexture() failed for image file " + filename);
		return null;
	}

	return createTexture(loader.getImage().getImage(), square, hAlignment, vAlignment, fill);
}

/**
 * Constructs a JCTexture2D object from the specified image which
 * must be non-null.  If the source image does not have a width and
 * height equal to a power of two, an enlarged texture will be created
 * for you which meets the power of two criteria.  In this case,
 * the image will be aligned within the texture object according to the
 * hAlignment and vAlignment constants, and any texels not covered by
 * the image will have the specified fill color.
 *
 * @param image - non-null image used to construct this JCTexture2D object
 * @param square - a flag to indicate wheteher the texture should have
 * @param hAlignment - horizontal alignment of the image within the
 * texture - one of SwingConstants.LEFT, CENTER, or RIGHT
 * @param vAlignment - vertical alignment of the image within the
 * texture - one of SwingConstants.TOP, CENTER, or BOTTOM
 * @param fill - color to use fill in the background if the loaded image
 * does not have width and height equal to a power of two
 */
public static JCTexture2D createTexture(BufferedImage image,
										boolean square,
										int hAlignment,
										int vAlignment,
										Color fill) {

	Rectangle pad = new Rectangle();

	// Are the dimensions valid?
	if (!isValidTextureImage(image, square)) {
		BufferedImage paddedImage =	createValidTextureImage(
				image,
				square,
				hAlignment,
				vAlignment,
				fill,
				pad);

		image = paddedImage;
	}

	TextureLoader loader = new TextureLoader(image, TextureLoader.Y_UP);
    ImageComponent2D image2D = loader.getImage();

	return new JCTexture2D(image2D, pad.x, pad.y, pad.width, pad.height, Texture.RGBA);
}

/*
 * Constructs a valid sized buffered image which may be used to construct
 * a Texture object.  The returned image will have dimensions which are
 * guaranteed to be a power of two, and if square is true, will have equal
 * width and height.  The bounding rectangle of source, relative to the texture
 * bounds, will be placed into the Rectangle argument imageBounds.
 */
private static BufferedImage createValidTextureImage(BufferedImage source,
													 boolean square,
													 int hAlignment,
													 int vAlignment,
													 Color fill,
													 Rectangle imageBounds) {

	int w = source.getWidth(),
		h = source.getHeight(),
		wTexture = getNextLargestPowerOfTwo(w),
		hTexture = getNextLargestPowerOfTwo(h);

	if (square) {
		wTexture = Math.max(wTexture, hTexture);
		hTexture = wTexture;
	}

	BufferedImage textureImage;
	textureImage = new BufferedImage(wTexture, hTexture, source.getType());

	// Fill the image background with the fill color.

	int ARGB = fill.getRGB();
	for (int x = 0; x < wTexture; x++) {
		for (int y = 0; y < hTexture; y++) {
			textureImage.setRGB(x, y, ARGB);
		}
	}

	// Align the specified image in the textureImage according to the alignment flags.

	switch (hAlignment) {
		case LEFT:
			imageBounds.x = 0;
			break;
		case CENTER:
			imageBounds.x = (wTexture - w) / 2;
			break;
		case RIGHT:
			imageBounds.x = wTexture - w;
			break;
		default:
			throw new IllegalArgumentException(
				"hAlignment must be SwingConstants.LEFT, CENTER or RIGHT");
	}

	switch (vAlignment) {
		case TOP:
			imageBounds.y = 0;
			break;
		case CENTER:
			imageBounds.y = (hTexture - h) / 2;
			break;
		case BOTTOM:
			imageBounds.y = hTexture - h;
			break;
		default:
			throw new IllegalArgumentException(
				"vAlignment must be SwingConstants.TOP, CENTER, or BOTTOM.");
	}

	imageBounds.width  = w;
	imageBounds.height = h;

	// Copy the source image into the area defined by pad.
	int[] pixels = source.getRGB(0, 0, w, h, null, 0, w);
	textureImage.setRGB(imageBounds.x, imageBounds.y, w, h, pixels, 0, w);

	return textureImage;
}

/**
 * Constructs a JCTexture2D object with the specified ImageComponent2D.
 * The imageXOffset, imageYOffset, imageWidth, and imageHeight arguments
 * define the location relative the texture origin, and dimensions of,
 * the source image relative used to construct this JCTexture2D object,
 * and are read-only.  The format argument must be one of Texture.INTENSITY,
 * LUMINANCE, ALPHA, LUMINANCE_ALPHA, RGB, or RGBA. Note that this texture
 * object will have both it's minification and magnification filters set to
 * Texture.NICEST by default.
 */
protected JCTexture2D(ImageComponent2D texture,
					  int imageXOffset,
					  int imageYOffset,
					  int imageWidth,
					  int imageHeight,
					  int format) {

	super(Texture.BASE_LEVEL, format, texture.getWidth(), texture.getHeight());
	setImage(0, texture);   //  the base level mipmap = 0
	setMinFilter(Texture.BASE_LEVEL_LINEAR);
	setMagFilter(Texture.BASE_LEVEL_LINEAR);

	this.width = texture.getWidth();
	this.height = texture.getHeight();
	this.imageXOffset = imageXOffset;
	this.imageYOffset = imageYOffset;
	this.imageWidth = imageWidth;
	this.imageHeight = imageHeight;
}

/**
 * Converts the specified texel x coordinate location to the
 * texture coordinate system, ie returns the equivalent of
 * x / getImageWidth().
 */
public float toAlphaX(int x) {
	return (float)x / width;
}

/**
 * Converts the specified texel y coordinate location to the
 * texture coordinate system, ie returns the equivalent of
 * y / getImageHeight().
 */
public float toAlphaY(int y) {
	return (float)y / height;
}

/**
 * Returns the x offset - measured in pixels relative to the top left
 * corner of this JCTexture2D object -  of the source image's top left
 * corner.
 *
 * @return the x offset of the source image
 */
public int getImageXOffset() {
	return imageXOffset;
}

/**
 * Returns the y offset - measured in pixels relative to the top left
 * corner of this JCTexture2D object -  of the source image's top left
 * corner.
 *
 * @return the y offset of the source image
 */
public int getImageYOffest() {
	return imageYOffset;
}

/**
 * Returns the width in pixels of the source image used to construct
 * this JCTexture2D object.  The functinality of this method is identical
 * to Texture.getWidth(), but note that the Texture.ALLOW_SIZE_READ
 * capability does not have to be set to use this method.
 *
 * @return the width in pixels of the source image
 */
public int getImageWidth() {
	return imageWidth;
}

/**
 * Returns the height in pixels of the source image used to construct
 * this JCTexture2D object.  The functinality of this method is identical
 * to Texture.getHeight(), but note that the Texture.ALLOW_SIZE_READ
 * capability does not have to be set to use this method.
 *
 * @return the height in pixels of the source image
 */
public int getImageHeight() {
	return imageHeight;
}

/*
 * Convenience method which tests whether the specified image
 * has power of two dimensions.  If the square flag is true,
 * checks the above criteria and whether the width and height
 * are equal.
 */
private static boolean isValidTextureImage(RenderedImage source, boolean square) {

	int w = source.getWidth(),
		h = source.getHeight();

	boolean isPowerOfTwo = isPowerOfTwo(w) && isPowerOfTwo(h);

	return isPowerOfTwo && (!square || (square && w == h));
}

/*
 * Convenience method which tests that the specified value
 * is a power of two.
 */
private static boolean isPowerOfTwo(int value) {

	if (value <= 1) {
		return false;
	}

	float remainder = value;

	while (remainder >= 1.0) {
		remainder /= 2.0;
		if (remainder == 1.0) {
			return true;
		}
	}

	return false;

}

/*
 * Convenience method which returns an integer which is
 * a power of two, and which is equal to or the next largest
 * power of two than the specified value.
 */
private static int getNextLargestPowerOfTwo(int value) {

	int powerOfTwo = 2;

	while (powerOfTwo < value) {
		powerOfTwo *= 2;
	}

	return powerOfTwo;
}

}
