/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCView3dJava3d.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:06 $  $Locker:  $  Quest Software Inc.

//  JCView3dJava3d.java

package com.klg.jclass.chart3d.j3d;

import javax.vecmath.*;
import com.klg.jclass.chart3d.*;

public class JCView3dJava3d extends JCView3d {

private JCChart3dAreaJava3d area = null;

JCView3dJava3d(JCChart3dAreaJava3d area)
{
	super();
	this.area = area;
}

JCView3dJava3d(JCChart3dAreaJava3d area, double perspective, double xRotation,
         double yRotation, double zRotation)
{
	super(perspective, xRotation, yRotation, zRotation);
	this.area = area;
}

private void updateRotation()
{
	((JCPlotCubeJava3d)area.getPlotCube()).rotate(xRotation, yRotation, zRotation);
	int previewMethod = area.getViewport().getPreviewMethod();
	if (previewMethod == JCViewport.PREVIEW_FULL) {
	    Transform transform = area.getTransform();
	    if (transform != null) {
	        transform.computeTransformations(area.getChart3d().getGraphics());
	    }
	    area.getAnno().draw3dAnno(area.getGraphics());
	}
	else {
	    ((JCPlotCubeJava3d)area.getPlotCube()).removeAllAnnotation();
	}
}

/**
 * Initializes the plot cube perspective.  Setting the perspective translates the
 * the plotcube towards or away from the viewing platform.  This method may be called
 * during the initialization phase of an application to place the entire plotcube
 * within view.  The default value for perspective is 2.5.  Increasing the perspective
 * pushes the plot cube further away from the view platform.
 *
 * @param perspective the plot cube perspective
 */
public void initPerspective(double perspective)
{
	super.setPerspective(perspective);
	//  Get z-coordinate of plotcube before and after perspective is modified --
	//  need to compute a delta for the fog front and back distances.
	Vector3d trans1 = new Vector3d();
	((JCPlotCubeJava3d)area.getPlotCube()).getTranslation(trans1);
	((JCPlotCubeJava3d)area.getPlotCube()).setPerspective();
	Vector3d trans2 = new Vector3d();
	((JCPlotCubeJava3d)area.getPlotCube()).getTranslation(trans2);
	double delta = trans1.z - trans2.z;
	area.getChart3dUniverse().getVirtualUniverse().adjustFogDistance((float)delta);
}

/**
 * Sets the <i>X</i> rotation angle.
 * @param xRotation the <i>X</i> rotation angle
 */
public void setXRotation(double xRotation)
{
	if (this.xRotation == xRotation) {
		return;
	}
    this.xRotation = xRotation;
	updateRotation();
}

/**
 * Sets the <i>Y</i> rotation angle.
 * @param yRotation the <i>Y</i> rotation angle
 */
public void setYRotation(double yRotation)
{
	if (this.yRotation == yRotation) {
		return;
	}
    this.yRotation = yRotation;
	updateRotation();
}

/**
 * Sets the <i>Z</i> rotation angle.
 * @param zRotation the <i>Z</i> rotation angle
 */
public void setZRotation(double zRotation)
{
	if (this.zRotation == zRotation) {
		return;
	}
    this.zRotation = zRotation;
	updateRotation();
}

}
