/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCViewportJava3d.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:06 $  $Locker:  $  Quest Software Inc.

//  JCViewportJava3d.java

package com.klg.jclass.chart3d.j3d;

import javax.vecmath.*;
import com.klg.jclass.chart3d.*;

public class JCViewportJava3d extends JCViewport {

private JCChart3dAreaJava3d area = null;

public JCViewportJava3d(JCChart3dAreaJava3d area)
{
    super();

    this.area = area;
}

/**
 * Sets the zoom factor.  A value less than one means a viewport smaller than
 * the default.  A value greater than one means a viewport greater than the
 * default (the graph image will be clipped by the <code>JCChart3dArea</code>).
 *
 * @param scale the new scale zoom factor
 */
public void setScale(double scale)
{
    if (this.scale == scale) {
        return;
    }
    this.scale = JCChart3dUtil.clamp(scale, MIN_VIEW_SCALE, MAX_VIEW_SCALE);
    this.normalized = (this.scale           == 1.0 &&
                       this.horizontalShift == 0.0 &&
                       this.verticalShift   == 0.0);
    ((JCPlotCubeJava3d)area.getPlotCube()).setScale(scale);
}

/**
 * Sets the horizontal shift.
 *
 * @param horizontalShift the horizontal shift
 */
public void setHorizontalShift(double horizontalShift)
{
    if (this.horizontalShift == horizontalShift) {
        return;
    }
    this.horizontalShift = horizontalShift;
    this.normalized = (this.scale           == 1.0 &&
                       this.horizontalShift == 0.0 &&
                       this.verticalShift   == 0.0);
    Vector3d shift = new Vector3d(-horizontalShift, -verticalShift, 0.0);
    ((JCPlotCubeJava3d)area.getPlotCube()).translate(shift);
}

/**
 * Sets the vertical shift.
 *
 * @param verticalShift the vertical shift
 */
public void setVerticalShift(double verticalShift)
{
    if (this.verticalShift == verticalShift) {
        return;
    }
    this.verticalShift = verticalShift;
    this.normalized = (this.scale           == 1.0 &&
                       this.horizontalShift == 0.0 &&
                       this.verticalShift   == 0.0);
    Vector3d shift = new Vector3d(-horizontalShift, -verticalShift, 0.0);
    ((JCPlotCubeJava3d)area.getPlotCube()).translate(shift);
}

}
