/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCZonedDataBars.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:06 $  $Locker:  $  Quest Software Inc.

//  JCZonedDataBars.java

package com.klg.jclass.chart3d.j3d;

import java.awt.Color;
import com.sun.j3d.utils.picking.behaviors.*;
import com.sun.j3d.utils.picking.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.util.ArrayList;
import com.klg.jclass.chart3d.*;
import com.klg.jclass.chart3d.JCGridColor;

public class JCZonedDataBars extends JCDataBars {

public JCZonedDataBars(JCBar barFormat, Matrix4d absolute,
					   Chart3dGridData data,
					   boolean fUseForegroundColor,
					   Color3f foregroundColor,
					   boolean fSolid)
{
	super(barFormat, data);
	createDataBars(barFormat, absolute, data, fUseForegroundColor,
					foregroundColor, fSolid);
}

//  Creates a data bar which may be composed of multiple QuadArrays
//  (one QuadArray for each contour level).
protected JCShape3D createDataBar(Matrix4d absolute,
								double x, double y, double zValue,
								double zOrigin,
								Color3f barColor,
								double xWidth, double yWidth,
								Color3f foregroundColor,
								boolean fSolid, int index)
{
	JCBarQA dataBarQA = null;
	ArrayList dataBarArrayList = new ArrayList();
	JCShape3D dataBar = null;
	Point3d barOrigin = null;
	Color3f clr3f = null;
	double zHeight = 0.0;

	JCContourLevels contourLevels = contour.getContourLevels();
	int numLevels = contourLevels.getNumLevelsInternal();
	double[] contourValue = contourLevels.getLevels();
	ArrayList contourStyles = contour.getContourStyles();

	barOrigin = new Point3d(x, y, zOrigin);
	absolute.transform(barOrigin);
	if (zValue >= zOrigin) {
		Point3d barTopCorner = null;
	    for (int i=0; i<numLevels; i++) {
			clr3f = getZoneColor(i);
		    //  Break out of loop AFTER the contour color has been set.
		    if (contourValue[i] >= zValue) {
		    	break;
		    }
			if (contourValue[i] < zOrigin) {
				continue;
			}
		    barTopCorner = new Point3d(x, y, contourValue[i]);
		    absolute.transform(barTopCorner);
		    zHeight = barTopCorner.z - barOrigin.z;
		    dataBarQA = new JCBarQA(barOrigin, xWidth, yWidth, zHeight, clr3f);
		    dataBarArrayList.add(dataBarQA);
		    barOrigin = barTopCorner;
	    }
	    barTopCorner = new Point3d(x, y, zValue);
	    absolute.transform(barTopCorner);
	    zHeight = barTopCorner.z - barOrigin.z;
	    dataBarQA = new JCBarQA(barOrigin, xWidth, yWidth, zHeight, clr3f);
	    dataBarArrayList.add(dataBarQA);
	}
	else {
		// Find the contour level at or immediately above the zOrigin.
		int k=0;
	    for (k=0; k < numLevels; k++) {
			if (contourValue[k] >= zOrigin) {
				break;
			}
		}

		if (k >= numLevels) {
			// Didn't find a contour level.  Start with the highest one.
			k=numLevels-1;
		}

		Point3d barBottomCorner = null;
		if (k == 0) {
			clr3f = getZoneColor(0);
		}
		else {
			for (int i=k-1; i >= 0; i--) {
				clr3f = getZoneColor(i+1);
				//  Break out of loop AFTER the contour color/style has been set.
				if (contourValue[i] <= zValue) {
					break;
				}
				if (contourValue[i] > zOrigin) {
					continue;
				}
				barBottomCorner = new Point3d(x, y, contourValue[i]);
				absolute.transform(barBottomCorner);
				zHeight = barOrigin.z - barBottomCorner.z;
				dataBarQA = new JCBarQA(barBottomCorner, xWidth, yWidth, zHeight, clr3f);
				dataBarArrayList.add(dataBarQA);
				barOrigin = barBottomCorner;
			}
		}
	    barBottomCorner = new Point3d(x, y, zValue);
	    absolute.transform(barBottomCorner);
	    zHeight = barOrigin.z - barBottomCorner.z;
	    dataBarQA = new JCBarQA(barBottomCorner, xWidth, yWidth, zHeight, clr3f);
	    dataBarArrayList.add(dataBarQA);
	}

	//  Create a JCShape3D object from all of the QuadArrays:
	dataBar = new JCShape3D( ((JCBarQA)dataBarArrayList.get(0)).getGeometryArray() );
	int listLength = dataBarArrayList.size();
	for (int i=1; i < listLength; i++) {
		dataBar.addGeometry( ((JCBarQA)dataBarArrayList.get(i)).getGeometryArray() );
	}
	dataBar.setUserData( new Integer(index) );

	JCAppearance appearance = new JCAppearance();
	JCPolygonAttributes pa = new JCPolygonAttributes();
	if (fSolid) {
		pa.setPolygonMode(PolygonAttributes.POLYGON_FILL);
		pa.setBackFaceNormalFlip(true);
		pa.setPolygonOffsetFactor(polygonOffsetFactor);
		pa.setPolygonOffset(polygonOffset);
	}
	else {
		pa.setPolygonMode(PolygonAttributes.POLYGON_LINE);
	}
	appearance.setPolygonAttributes(pa);

	JCRenderingAttributes renderAttrib = new JCRenderingAttributes();
	renderAttrib.setVisible(true);  //  default
	appearance.setRenderingAttributes(renderAttrib);

	JCMaterial material = new JCMaterial(foregroundColor, DEFAULT_SHININESS);
	material.setLightingEnable(false) ;  // default

	JCColoringAttributes colorAttrib = new JCColoringAttributes();
	colorAttrib.setColor(foregroundColor);
	colorAttrib.setShadeModel(ColoringAttributes.SHADE_FLAT);

	appearance.setMaterial(material) ;
	appearance.setColoringAttributes(colorAttrib);

	dataBar.setAppearance(appearance);

	return dataBar;
}

}
