/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCZonedMeshSurface.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:06 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.j3d;

import com.sun.j3d.utils.picking.behaviors.*;
import com.sun.j3d.utils.picking.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.util.ArrayList;
import com.klg.jclass.chart3d.*;

public class JCZonedMeshSurface extends JCBranchGroup {

protected final static int ZONED_MESH_DATA_SURFACE = 102;
protected final static float DEFAULT_LINEWIDTH = 1.0f;
private JCColoringAttributes colorAttrib = null;
private JCShape3D meshLines = null;
private JCShape3D[] contourLines = null;

public JCZonedMeshSurface(Matrix4d absolute,
						  Chart3dGridData data,
                          JCContour contour,
						  Color3f foregroundColor)
{
	super();

	ArrayList contourStyles = contour.getContourStyles();

	DataSurfaceLA surface = new DataSurfaceLA(absolute, data, contour);
	//  Create wire mesh:
	Geometry meshGeom = surface.getMeshGeometry();
	if (meshGeom != null) {
		meshLines = createMeshLines(meshGeom,
									contour,
									foregroundColor);
		Chart3dDataView dataView = data.getDataView();
		JCChart3dArea area = dataView.getChart3d().getChart3dArea();
		// If necessary, create mesh lines for skirt.  Don't create
		// skirt mesh lines if the surface is transparent.
		if (area.getSurface() != null &&
			area.getSurface().isSolid() &&
			!dataView.getElevation().isTransparent()) {
			SkirtMeshLA skirtMeshLines =
								new SkirtMeshLA(absolute, data, foregroundColor);
			Geometry geom = skirtMeshLines.getGeometryArray();
			meshLines.addGeometry(geom);
		}
		meshLines.setCapability(Node.ALLOW_PICKABLE_READ);
		meshLines.setCapability(Node.ALLOW_PICKABLE_WRITE);
		meshLines.setCapability(Shape3D.ENABLE_PICK_REPORTING);
		PickTool.setCapabilities(meshLines, PickTool.INTERSECT_FULL);
		addChild(meshLines);
	}

	//  Create contour lines:
	Geometry[] contourGeom = surface.getContourGeometry();
	if (contourGeom != null) {
		contourLines = new JCShape3D[contourGeom.length];
		for (int i = 0; i < contourGeom.length; i++) {
			if (contourGeom[i] != null) {
		    	contourLines[i] = createContourLines(i, contourGeom[i],
		    									    (JCContourStyle)contourStyles.get(i),
													foregroundColor);
				contourLines[i].setCapability(Node.ALLOW_PICKABLE_READ);
				contourLines[i].setCapability(Node.ALLOW_PICKABLE_WRITE);
	        	contourLines[i].setCapability(Shape3D.ENABLE_PICK_REPORTING);
	        	PickTool.setCapabilities(contourLines[i], PickTool.INTERSECT_FULL);
		    	addChild(contourLines[i]);
			}
		}
	}

	compile();
}

private JCShape3D createMeshLines(Geometry geometry,
								 JCContour contour, //  get line attributes from this
								 Color3f foregroundColor)
{
	JCAppearance appearance = new JCAppearance();

	JCMaterial m = new JCMaterial() ;
	m.setLightingEnable(false) ;
	appearance.setMaterial(m) ;

	JCLineAttributes lineAttrib = new JCLineAttributes();
	lineAttrib.setLineWidth(DEFAULT_LINEWIDTH);
	lineAttrib.setLineAntialiasingEnable(false);
	appearance.setLineAttributes(lineAttrib);

	JCRenderingAttributes renderAttrib = new JCRenderingAttributes();
	renderAttrib.setVisible(false);   //  default
	appearance.setRenderingAttributes(renderAttrib);

	JCColoringAttributes colorAttrib = new JCColoringAttributes();
	colorAttrib.setColor(foregroundColor);
	appearance.setColoringAttributes(colorAttrib);

	JCShape3D shape = new JCShape3D(geometry, appearance,
								    ZONED_MESH_DATA_SURFACE);
	return shape;
}

private JCShape3D createContourLines(int level, Geometry geometry,
								     JCContourStyle contourStyle,
									 Color3f foregroundColor)
{
	JCAppearance appearance = new JCAppearance();
	JCMaterial m = new JCMaterial() ;
	m.setLightingEnable(false) ;
	appearance.setMaterial(m) ;

	JCLineAttributes lineAttrib = new JCLineAttributes();
	lineAttrib.setLineWidth(DEFAULT_LINEWIDTH);
	lineAttrib.setLineAntialiasingEnable(false);
	appearance.setLineAttributes(lineAttrib);

	JCRenderingAttributes renderAttrib = new JCRenderingAttributes();
	renderAttrib.setVisible(false);   //  default
	appearance.setRenderingAttributes(renderAttrib);

	JCColoringAttributes colorAttrib = new JCColoringAttributes();
	colorAttrib.setColor(foregroundColor);
	appearance.setColoringAttributes(colorAttrib);

	JCShape3D shape = new JCShape3D(geometry, appearance,
								    ZONED_MESH_DATA_SURFACE);
	return shape;
}

private void showLineZoneColors(boolean fZoned, JCShape3D lines)
{
	if (lines == null) {
		return;
	}
	JCAppearance app = (JCAppearance)lines.getAppearance();
	JCRenderingAttributes renderAttrib =
						(JCRenderingAttributes)app.getRenderingAttributes();
	if (renderAttrib != null) {
	    renderAttrib.setIgnoreVertexColors(!fZoned);
	}
}

public void setMeshSurfaceZoned(boolean fZoned)
{
	if (meshLines != null) {
		showLineZoneColors(fZoned, meshLines);
	}
}

public void setContourLinesZoned(boolean zoned)
{
	if (contourLines != null) {
		for (int i = 0; i < contourLines.length; i++) {
		    showLineZoneColors(zoned, contourLines[i]);
		}
	}
}

public void setMeshPickable(boolean fPickable) {
	if (meshLines != null) {
		meshLines.setPickable(fPickable);
	}
}

public void setContourLinesPickable(boolean fPickable) {

	if (contourLines != null) {
		for (int i = 0; i < contourLines.length; i++) {
			if (contourLines[i] != null) {
		    	contourLines[i].setPickable(fPickable);
			}
		}
	}
}

private void setShape3DVisible(JCShape3D shape, boolean fVisible)
{
	if (shape == null) {
		return;
	}
	JCAppearance app = (JCAppearance)shape.getAppearance();
	JCRenderingAttributes renderAttrib =
					    (JCRenderingAttributes)app.getRenderingAttributes();
	if (renderAttrib != null) {
	    renderAttrib.setVisible(fVisible);
	}

}

public void setMeshSurfaceVisible(boolean fVisible)
{
	setShape3DVisible(meshLines, fVisible);
}

public void setContourLinesVisible(boolean fVisible)
{
	for (int i = 0; i < contourLines.length; i++) {
		setShape3DVisible(contourLines[i], fVisible);
	}
}

private void setLineWidth(JCShape3D shape, float lineWidth)
{
	if (shape == null) {
		return;
	}
	JCAppearance app = (JCAppearance)shape.getAppearance();
	JCLineAttributes lineAttrib = (JCLineAttributes)app.getLineAttributes();
	if (lineAttrib != null) {
	    lineAttrib.setLineWidth(lineWidth);
	}
}

public void setMeshLineWidth(float lineWidth)
{
	setLineWidth(meshLines, lineWidth);
}

protected void setContourLineWidth(float lineWidth)
{
	for (int i = 0; i < contourLines.length; i++) {
	    setLineWidth(contourLines[i], lineWidth);
	}
}

private void setLinePattern(JCShape3D shape, int linePattern)
{
	if (shape == null) {
		return;
	}
	JCAppearance app = (JCAppearance)shape.getAppearance();
	JCLineAttributes lineAttrib = (JCLineAttributes)app.getLineAttributes();
	if (lineAttrib != null) {
	    lineAttrib.setLinePattern(linePattern);
	}
}

public void setMeshLinePattern(int linePattern)
{
	setLinePattern(meshLines, linePattern);
}

private int getLinePattern(JCShape3D shape)
{
	int linePattern = LineAttributes.PATTERN_SOLID;
	if (shape != null) {
		JCAppearance app = (JCAppearance)shape.getAppearance();
		JCLineAttributes lineAttrib = (JCLineAttributes)app.getLineAttributes();
		if (lineAttrib != null) {
		    linePattern = lineAttrib.getLinePattern();
		}
	}
	return linePattern;
}

public int getMeshLinePattern()
{
	return getLinePattern(meshLines);
}

public int getContourLinePattern(int level)
{
	int linePattern = LineAttributes.PATTERN_SOLID;
	if (level >= 0) {
		linePattern = getLinePattern(contourLines[level]);
	}
	return linePattern;
}

public void setContourLinePattern(int linePattern)
{
	for (int i = 0; i < contourLines.length; i++) {
		setLinePattern(contourLines[i], linePattern);
	}
}

}
