/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: MapPick.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:06 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.j3d;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Dimension;
import javax.vecmath.Point3d;
import javax.vecmath.Point4d;
import javax.media.j3d.*;

import com.klg.jclass.chart3d.*;
import com.klg.jclass.chart3d.Mapping;


/**
 * A class that implements the Mapping interface.  This computes the map and
 * pick functions for the j3d version of Chart3d.
 */
public class MapPick implements Mapping {

protected float tolerance = 1.0f;
protected JCChart3dJava3d chart3d = null;
protected Canvas3D canvas3D = null;
protected BranchGroup branchGroup = null;
protected Chart3dDataView dataView = null;
protected Chart3dData data = null;

public MapPick()
{
}

/**
 * Contructor with data.
 */
public MapPick(Chart3dData data)
{
    this.data = data;
	Chart3dDataView dataView = data.getDataView();
	chart3d = (JCChart3dJava3d)dataView.getChart3d();
	JCChart3dAreaJava3d area = (JCChart3dAreaJava3d)chart3d.getChart3dArea();
	this.canvas3D = area.getChart3dUniverse().getVirtualUniverse().getCanvas3D();
}

/**
 * Contructor with data, Canvas3D, and BranchGroup.
 */
public MapPick(Chart3dData data, Canvas3D canvas3D, BranchGroup branchGroup)
{
	Chart3dDataView dataView = data.getDataView();
	chart3d = (JCChart3dJava3d)dataView.getChart3d();
    this.data = data;
    this.canvas3D = canvas3D;
    this.branchGroup = branchGroup;
}

/**
 * Sets the Canvas3D object required by PickCanvas.
 */
public void setCanvas3D(Canvas3D canvas3D)
{
	this.canvas3D = canvas3D;
}

protected boolean isWithinCanvas3DLimits(int xPixel, int yPixel)
{
	Dimension size = canvas3D.getSize();
	if ((xPixel >= 0) && (xPixel <= size.width)) {
		if ((yPixel >= 0) && (yPixel <= size.height)) {
			return true;
		}
	}
	return false;
}
/**
 * Sets the BranchGroup object required by PickCanvas.
 */
public void setBranchGroup(BranchGroup branchGroup)
{
	this.branchGroup = branchGroup;
}

/**
 * Dummy implementation.  Meant to be overridden in MapPickGrid or MapPickPoint.
 */
public Point3d map(int xPixel, int yPixel)
{
    return(null);
}

/**
 * Performs an unmap operation by transforming the given data point to the
 * position on the screen where it would be drawn.
 * @param point The data point to be unmapped.
 * @return The screen point to which the data point maps
 */
public java.awt.Point unmap(Point3d point)
{
    if (data == null || !data.isDataOK()) {
        return(null);
    }

	return null;
}

/**
 * Dummy implementation.  Meant to be overridden in MapPickGrid or MapPickPoint.
 */
public JCData3dIndex pick(int xPixel, int yPixel)
{
    return(null);
}

/**
 * Dummy implementation.  Meant to be overridden in MapPickGrid or MapPickPoint.
 */
public java.awt.Point unpick(JCData3dIndex index)
{
    return(null);
}

/**
 * Return the internal data object whose type determines the type of MapPick
 * object required.
 */
public Chart3dData getData()
{
    return(data);
}

/**
 * Create a new mapping, depending on the type of input data
 */
public Mapping createMapping(Chart3dData data)
{
    if (data == null) {
        return(null);
    }

    Chart3dDataView dataView = data.getDataView();
    JCChart3dJava3d chart3d = (JCChart3dJava3d)dataView.getChart3d();
    JCChart3dAreaJava3d area = (JCChart3dAreaJava3d)chart3d.getChart3dArea();
    JC3DUniverse vu = area.getChart3dUniverse().getVirtualUniverse();

    if (data instanceof Chart3dGridData) {
        return(new MapPickGrid(data, vu.getCanvas3D(), vu.getSceneBranchGroup()));
    }
    if (data instanceof Chart3dPointData) {
        return(new MapPickPoint(data, vu.getCanvas3D(), vu.getSceneBranchGroup()));
    }
    return(null);
}

public void setTolerance(float tolerance)
{
	this.tolerance = tolerance;
}

}

