/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: MapPickGrid.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:06 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.j3d;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Dimension;
import javax.vecmath.Point2d;
import javax.vecmath.Point3d;
import javax.vecmath.Point4d;
import javax.vecmath.Matrix4d;
import javax.media.j3d.*;
import com.sun.j3d.utils.picking.*;

import com.klg.jclass.chart3d.*;


/**
 * A class that implements the Mapping interface.  This computes the map and
 * pick functions for the j3d version of Chart3d.
 */
public class MapPickGrid extends MapPick {

private boolean fPickClosestDataPoint = true;

/**
 * Default constructor, required by Java Beans.
 */
public MapPickGrid()
{
}

/**
 * Contructor with data.
 */
public MapPickGrid(Chart3dData data)
{
	super(data);
}

/**
 * Contructor with data, Canvas3D, and BranchGroup.
 */
public MapPickGrid(Chart3dData data, Canvas3D canvas3D, BranchGroup branchGroup)
{
	super(data, canvas3D, branchGroup);
}

public void enablePickClosestDataPoint(boolean flag)
{
	fPickClosestDataPoint = flag;
}

/**
 * Performs a map operation by mapping the provided screen pixel
 * information to a value in data space.
 *
 * @param x <i>x</i> value in screen pixels
 * @param y <i>y</i> value in screen pixels
 * @return The data point that maps to the given screen point
 */
public Point3d map(int xPixel, int yPixel)
{
    if (!(data instanceof Chart3dGridData)) {
        return(null);
    }
	if (!isWithinCanvas3DLimits(xPixel, yPixel)) {
		return null;
	}

    Chart3dGridData gridData = (Chart3dGridData)data;
    if (gridData == null || !gridData.isDataOK()) {
        return(null);
    }

	Point3d pt = null;
	dataView = gridData.getDataView();
	if (dataView != null) {
		int chartType = dataView.getChartType();
    	switch(chartType) {
		case JCChart3d.SURFACE:
			pt = mapSurface(xPixel, yPixel);
			break;
		case JCChart3d.BAR:
			pt = mapBars(xPixel, yPixel);
			break;
		case JCChart3d.SCATTER:
			pt = mapPoints(xPixel, yPixel);
			break;
		default:
		}
	}
    return(pt);
}

/**
 * Performs a pick operation by returning the index of the closest point in
 * the data set to the given screen point.
 *
 * @param x <i>x</i> value in screen pixels
 * @param y <i>y</i> value in screen pixels
 */
public JCData3dIndex pick(int xPixel, int yPixel)
{
	JCData3dIndex data3dIndex = null;
    if (!(data instanceof Chart3dGridData)) {
        return(null);
    }
	if (!isWithinCanvas3DLimits(xPixel, yPixel)) {
		return null;
	}

    Chart3dGridData gridData = (Chart3dGridData)data;
    if (gridData == null || !gridData.isDataOK()) {
        return(null);
    }

	dataView = gridData.getDataView();
	if (dataView != null) {
		int chartType = dataView.getChartType();
    	switch(chartType) {
		case JCChart3d.SURFACE:
			data3dIndex = pickSurface(xPixel, yPixel);
			break;
		case JCChart3d.BAR:
			data3dIndex = pickBars(xPixel, yPixel);
			break;
		case JCChart3d.SCATTER:
			data3dIndex = pickPoints(xPixel, yPixel);
			break;
		default:
		}
	}

    return(data3dIndex);
}

/**
 * Performs an unpick operation by returning the screen point that corresponds
 * to the given data index.
 *
 * @param index The data index of the point whose screen point is required.
 */
public java.awt.Point unpick(JCData3dIndex index)
{
    Point pickPoint = new Point(-1, -1);
    if (!(data instanceof Chart3dGridData) ||
        !(index instanceof JCData3dGridIndex))
    {
        return(pickPoint);
    }

    Chart3dGridData      gridData  = (Chart3dGridData)data;
    JCData3dGridIndex    gridIndex = (JCData3dGridIndex)index;
    if (gridData == null || !gridData.isDataOK() || gridIndex == null) {
        return(pickPoint);
    }

	//  Get (x,y,z) from index:
	int xIndex = gridIndex.getX();
	int yIndex = gridIndex.getY();
	Point3d pt = new Point3d(gridData.getX(xIndex), gridData.getY(yIndex),
						     gridData.getZValue(xIndex, yIndex));

	//  Apply scaling:
	JCChart3dAreaJava3d area =
				(JCChart3dAreaJava3d)(dataView.getChart3d().getChart3dArea());
	Matrix4d absolute = area.getTransform().getAbsolute();
	absolute.transform(pt);

	//  Apply rotation:
	Transform3D t3d = new Transform3D();
	JCPlotCubeJava3d plotCube = (JCPlotCubeJava3d)area.getPlotCube();
	TransformGroup currTransform = plotCube.getTransformGroup();
	if (currTransform != null) {
		currTransform.getTransform(t3d);
		t3d.transform(pt);
	}

	Transform3D t3dVWtoIP = new Transform3D();
	canvas3D.getVworldToImagePlate(t3dVWtoIP);
	t3dVWtoIP.transform(pt);

	Point2d pixelLocation = new Point2d();
	canvas3D.getPixelLocationFromImagePlate(pt, pixelLocation);
	pickPoint = new Point((int)(pixelLocation.x + 0.5),
						  (int)(pixelLocation.y + 0.5));
				//  Cast to int will truncate -- add 0.5 to achieve round-off

    return(pickPoint);
}

public Point3d mapSurface(int xPixel, int yPixel)
{
	Point3d pntIntersect = null;
	Point3d pt = null;
	PickCanvas pickCanvas = new PickCanvas(canvas3D, branchGroup);
	pickCanvas.setTolerance(tolerance);
	pickCanvas.setMode(PickCanvas.GEOMETRY_INTERSECT_INFO);
	pickCanvas.setShapeLocation(xPixel, yPixel);
	Point3d eyePos = pickCanvas.getStartPosition();
	PickResult[] pickResult = pickCanvas.pickAllSorted();
	if (pickResult != null) {
		for (int index = 0; index < pickResult.length; index++) {
			Node node = pickResult[index].getNode(PickResult.SHAPE3D);
			if ((node == null) || !(node instanceof JCShape3D)) {
				continue;
			}
			int objectType = ((JCShape3D)node).getObjectType();
			if ( (objectType == JCSolidDataSurface.SOLID_DATA_SURFACE) ||
				 (objectType == JCZonedMeshSurface.ZONED_MESH_DATA_SURFACE) ) {
				//  Note: If the plotcube walls are displayed,
				//  the nearest wall is the closest intersection.
				PickIntersection pi =
					pickResult[index].getClosestIntersection(eyePos);
				//  Get local coordinates:
				pt = pi.getPointCoordinates();
				//  Note -- if fPickClosestDataPoint is true, calling
				//  pi.getClosestVertexCoordinates() won't be
				//  sufficient.  The closest geometric vertex might not be
				//  a data grid vertex.  Call pi.getPointCoordinates(), then
				//  search for the closest data point.

				//  Object returned from mapSurface() is a point in data space.
				JCChart3dAreaJava3d area =
					(JCChart3dAreaJava3d)(dataView.getChart3d().getChart3dArea());
		    	Transform transform = area.getTransform();
			    Matrix4d iAbsolute = transform.getInverseAbsolute();
			    iAbsolute.transform(pt);

				if (fPickClosestDataPoint) {
					Chart3dGridData gd = (Chart3dGridData)data;
				    int xIndex = gd.getXClosest(pt.x);
				    int yIndex = gd.getYClosest(pt.y);
					pt = new Point3d(gd.getX(xIndex), gd.getY(yIndex),
								     gd.getZValue(xIndex, yIndex));
				}
				return(pt);
			}
		}
	}
	return(null);
}

public Point3d mapBars(int xPixel, int yPixel)
{
	Point3d point = null;
    Chart3dGridData gridData = (Chart3dGridData)data;
	PickCanvas pickCanvas = new PickCanvas(canvas3D, branchGroup);
	pickCanvas.setMode(PickTool.GEOMETRY_INTERSECT_INFO);
	pickCanvas.setShapeLocation(xPixel, yPixel);
	pickCanvas.setTolerance(tolerance);
	Point3d eyePos = pickCanvas.getStartPosition();
	PickResult[] pickResult = pickCanvas.pickAllSorted();
	if (pickResult != null) {
		for (int index = 0; index < pickResult.length; index++) {
			Node node = pickResult[index].getNode(PickResult.SHAPE3D);
			if (node != null) {
				Integer userData = (Integer)((Shape3D)node).getUserData();
				if (userData == null) {
					continue;
				}
				//  From the objectID, determine the indices into the
				//  xGrid[] and yGrid[] arrays.
				JCChart3dAreaJava3d area =
					(JCChart3dAreaJava3d)(dataView.getChart3d().getChart3dArea());
				JCBar barFormat = area.getBar();
				int nXBars = 0;
				if (barFormat.getXFormat() == JCBar.HISTOGRAM) {
					nXBars = gridData.getNumX() - 1;
				}
				else {
					nXBars = gridData.getNumX();
				}
				int barIndex = userData.intValue();
				GridLimits gridLimits = gridData.getGridLimits();
				int xIndex = gridLimits.getXOffset() + (barIndex % nXBars);
				int yIndex = gridLimits.getYOffset() + (barIndex / nXBars);
				point = new Point3d(gridData.getX(xIndex),
								    gridData.getY(yIndex),
								    gridData.getZValue(xIndex, yIndex));
				break;
			}
		}
	}
	return(point);
}

public Point3d mapPoints(int xPixel, int yPixel)
{
    Chart3dGridData gridData = (Chart3dGridData)data;
	PickCanvas pickCanvas = new PickCanvas(canvas3D, branchGroup);
	pickCanvas.setMode(PickTool.GEOMETRY_INTERSECT_INFO);
	pickCanvas.setShapeLocation(xPixel, yPixel);
	pickCanvas.setTolerance(tolerance);
	Point3d eyePos = pickCanvas.getStartPosition();
	PickResult[] pickResult = pickCanvas.pickAllSorted();
	if (pickResult != null) {
		for (int index = 0; index < pickResult.length; index++) {
			Node node = pickResult[index].getNode(PickResult.SHAPE3D);
			if (node != null) {
				Integer userData = (Integer)((Shape3D)node).getUserData();
				if (userData == null) {
					continue;
				}
				//  From the objectID, determine the indices into the
				//  xGrid[] and yGrid[] arrays.
				int pointIndex = userData.intValue();
				GridLimits gridLimits = gridData.getGridLimits();
				int xIndex = gridLimits.getXOffset() +
									    (pointIndex % gridData.getNumX());
				int yIndex = gridLimits.getYOffset() +
										(pointIndex / gridData.getNumX());
				Point3d pt = new Point3d(gridData.getX(xIndex),
									     gridData.getY(yIndex),
									     gridData.getZValue(xIndex, yIndex));
				return(pt);
			}
		}
	}
	return(null);
}

public JCData3dIndex pickSurface(int xPixel, int yPixel)
{
	PickCanvas pickCanvas = new PickCanvas(canvas3D, branchGroup);
	pickCanvas.setTolerance(tolerance);
	pickCanvas.setMode(PickCanvas.GEOMETRY_INTERSECT_INFO);

	pickCanvas.setShapeLocation(xPixel, yPixel);
	Point3d eyePos = pickCanvas.getStartPosition();
	PickResult[] pickResult = pickCanvas.pickAllSorted();
	if (pickResult != null) {
		for (int index = 0; index < pickResult.length; index++) {
			Node node = pickResult[index].getNode(PickResult.SHAPE3D);
			if ((node == null) || !(node instanceof JCShape3D)) {
				continue;
			}
			int objectType = ((JCShape3D)node).getObjectType();
			if ( (objectType == JCSolidDataSurface.SOLID_DATA_SURFACE) ||
				 (objectType == JCZonedMeshSurface.ZONED_MESH_DATA_SURFACE) ) {
				//  Note: If the plotcube walls are displayed,
				//  the nearest wall is the closest intersection.
				PickIntersection pi =
					pickResult[index].getClosestIntersection(eyePos);
				Point3d pt = pi.getPointCoordinates();
				//  Convert to data space:
				JCChart3dAreaJava3d area =
					(JCChart3dAreaJava3d)(dataView.getChart3d().getChart3dArea());
		    	Transform transform = area.getTransform();
			    Matrix4d iAbsolute = transform.getInverseAbsolute();
			    iAbsolute.transform(pt);
				int xIndex = ((Chart3dGridData)data).getXClosest(pt.x);
				int yIndex = ((Chart3dGridData)data).getYClosest(pt.y);
				JCData3dGridIndex data3dIndex =
									new JCData3dGridIndex(xIndex, yIndex);
				return(data3dIndex);
			}
		}
	}
	return(null);
}

public JCData3dIndex pickBars(int xPixel, int yPixel)
{
    Chart3dGridData gridData = (Chart3dGridData)data;
	PickCanvas pickCanvas = new PickCanvas(canvas3D, branchGroup);
	pickCanvas.setMode(PickCanvas.GEOMETRY_INTERSECT_INFO);
	pickCanvas.setShapeLocation(xPixel, yPixel);
	pickCanvas.setTolerance(tolerance);
	Point3d eyePos = pickCanvas.getStartPosition();
	PickResult[] pickResult = pickCanvas.pickAllSorted();
	if (pickResult != null) {
		for (int index = 0; index < pickResult.length; index++) {
			Node node = pickResult[index].getNode(PickResult.SHAPE3D);
			if (node != null) {
				Integer userData = (Integer)((Shape3D)node).getUserData();
				if (userData == null) {
					continue;
				}
				//  From the objectID, determine the indices into the
				//  xGrid[] and yGrid[] arrays.
				JCChart3dAreaJava3d area =
					(JCChart3dAreaJava3d)(dataView.getChart3d().getChart3dArea());
				JCBar barFormat = area.getBar();
				int nXBars = 0;
				if (barFormat.getXFormat() == JCBar.HISTOGRAM) {
					nXBars = gridData.getNumX() - 1;
				}
				else {
					nXBars = gridData.getNumX();
				}
				int barIndex = userData.intValue();
				GridLimits gridLimits = gridData.getGridLimits();
				int xIndex = gridLimits.getXOffset() + (barIndex % nXBars);
				int yIndex = gridLimits.getYOffset() + (barIndex / nXBars);
				JCData3dGridIndex data3dIndex =
									new JCData3dGridIndex(xIndex, yIndex);
				return(data3dIndex);
			}
		}
	}
	return(null);
}

public JCData3dIndex pickPoints(int xPixel, int yPixel)
{
    Chart3dGridData gridData = (Chart3dGridData)data;
	PickCanvas pickCanvas = new PickCanvas(canvas3D, branchGroup);
	pickCanvas.setMode(PickCanvas.GEOMETRY_INTERSECT_INFO);
	pickCanvas.setShapeLocation(xPixel, yPixel);
	pickCanvas.setTolerance(tolerance);
	Point3d eyePos = pickCanvas.getStartPosition();
	PickResult[] pickResult = pickCanvas.pickAllSorted();
	if (pickResult != null) {
		for (int index = 0; index < pickResult.length; index++) {
			Node node = pickResult[index].getNode(PickResult.SHAPE3D);
			if (node != null) {
				Integer userData = (Integer)((Shape3D)node).getUserData();
				if (userData == null) {
					continue;
				}
				//  From the objectID, determine the indices into the
				//  xGrid[] and yGrid[] arrays.
				int pointIndex = userData.intValue();
				GridLimits gridLimits = gridData.getGridLimits();
				int xIndex = gridLimits.getXOffset() +
											(pointIndex % gridData.getNumX());
				int yIndex = gridLimits.getYOffset() +
											(pointIndex / gridData.getNumX());
				JCData3dGridIndex data3dIndex =
									new JCData3dGridIndex(xIndex, yIndex);
				return(data3dIndex);
			}
		}
	}
	return(null);
}

}

