/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: MapPickPoint.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:06 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.j3d;

import java.awt.Point;
import java.awt.Rectangle;
import javax.vecmath.Point2d;
import javax.vecmath.Point3d;
import javax.vecmath.Point4d;
import javax.vecmath.Matrix4d;
import javax.media.j3d.*;
import com.sun.j3d.utils.picking.*;

import com.klg.jclass.chart3d.*;


/**
 * A class that implements the Mapping interface.  This computes the map and
 * pick functions for the j3d version of Chart3d.
 */
public class MapPickPoint extends MapPick {

/**
 * Default constructor, required by Java Beans.
 */
public MapPickPoint()
{
}

/**
 * Contructor with data.
 */
public MapPickPoint(Chart3dData data)
{
	super(data);
}

/**
 * Contructor with data, Canvas3D, and BranchGroup.
 */
public MapPickPoint(Chart3dData data, Canvas3D canvas3D, BranchGroup branchGroup)
{
	super(data, canvas3D, branchGroup);
}

/**
 * Performs a map operation by mapping the provided screen pixel
 * information to a value in data space.
 *
 * @param x <i>x</i> value in screen pixels
 * @param y <i>y</i> value in screen pixels
 * @return The data point that maps to the given screen point
 */
public Point3d map(int xPixel, int yPixel)
{

    int series, point;

    if (!(data instanceof Chart3dPointData)) {
        return(null);
    }
    Chart3dPointData pointData = (Chart3dPointData)data;
    if (pointData == null || !pointData.isDataOK()) {
        return(null);
    }

	if (!isWithinCanvas3DLimits(xPixel, yPixel)) {
		return null;
	}

    return mapPoints(xPixel, yPixel);
}

/**
 * Performs a pick operation by returning the index of the closest point in
 * the data set to the given screen point.
 *
 * @param x <i>x</i> value in screen pixels
 * @param y <i>y</i> value in screen pixels
 */
public JCData3dIndex pick(int xPixel, int yPixel)
{
    if (!(data instanceof Chart3dPointData)) {
        return(null);
    }
    Chart3dPointData pointData = (Chart3dPointData)data;
    if (pointData == null || !pointData.isDataOK()) {
        return(null);
    }

	if (!isWithinCanvas3DLimits(xPixel, yPixel)) {
		return null;
	}

    return pickPoints(xPixel, yPixel);
}

/**
 * Performs an unpick operation by returning the screen point that corresponds
 * to the given data index.
 *
 * @param index The data index of the point whose screen point is required.
 */
public java.awt.Point unpick(JCData3dIndex index)
{
    Point pickPoint = new Point(-1, -1);
    if (!(data instanceof Chart3dPointData) ||
        !(index instanceof JCData3dPointIndex))
    {
        return(pickPoint);
    }

    Chart3dPointData   pointData  = (Chart3dPointData)data;
    JCData3dPointIndex pointIndex = (JCData3dPointIndex)index;
    if (pointData == null || !pointData.isDataOK() || pointIndex == null) {
        return(pickPoint);
    }

	int series = pointIndex.getSeries();
	int point = pointIndex.getPoint();
	Point3d pt = new Point3d(pointData.getPoint(series, point));

	//  Apply scaling:
	Chart3dDataView dataView = pointData.getDataView();
	JCChart3dAreaJava3d area =
				(JCChart3dAreaJava3d)(dataView.getChart3d().getChart3dArea());
	Matrix4d absolute = area.getTransform().getAbsolute();
	absolute.transform(pt);

	//  Apply rotation:
	Transform3D t3d = new Transform3D();
	JCPlotCubeJava3d plotCube = (JCPlotCubeJava3d)area.getPlotCube();
	TransformGroup currTransform = plotCube.getTransformGroup();
	if (currTransform != null) {
		currTransform.getTransform(t3d);
		t3d.transform(pt);
	}

	Transform3D t3dVWtoIP = new Transform3D();
	canvas3D.getVworldToImagePlate(t3dVWtoIP);
	t3dVWtoIP.transform(pt);

	Point2d pixelLocation = new Point2d();
	canvas3D.getPixelLocationFromImagePlate(pt, pixelLocation);
	pickPoint = new Point((int)(pixelLocation.x + 0.5),
						  (int)(pixelLocation.y + 0.5));
				//  Cast to int will truncate -- add 0.5 to achieve round-off

	return pickPoint;
}

public Point3d mapPoints(int xPixel, int yPixel)
{
	PickCanvas pickCanvas = new PickCanvas(canvas3D, branchGroup);
	pickCanvas.setMode(PickTool.GEOMETRY_INTERSECT_INFO);
	pickCanvas.setShapeLocation(xPixel, yPixel);
	pickCanvas.setTolerance(tolerance);
	Point3d eyePos = pickCanvas.getStartPosition();
	PickResult[] pickResult = pickCanvas.pickAllSorted();
	if (pickResult != null) {
		for (int index = 0; index < pickResult.length; index++) {
			Node node = pickResult[index].getNode(PickResult.SHAPE3D);
			if (node != null) {
				Data3dPointIndex pointIndex =
							(Data3dPointIndex)((Shape3D)node).getUserData();
				if (pointIndex == null) {
					continue;
				}
    			Chart3dPointData pointData = (Chart3dPointData)data;
				Point3d pt = new Point3d(pointData.getPoint(pointIndex.getSeries(),
															pointIndex.getPoint()));
				return(pt);
			}
		}
	}
	return(null);
}

public JCData3dIndex pickPoints(int xPixel, int yPixel)
{
	PickCanvas pickCanvas = new PickCanvas(canvas3D, branchGroup);
	pickCanvas.setMode(PickCanvas.GEOMETRY_INTERSECT_INFO);
	pickCanvas.setShapeLocation(xPixel, yPixel);
	pickCanvas.setTolerance(tolerance);
	Point3d eyePos = pickCanvas.getStartPosition();
	PickResult[] pickResult = pickCanvas.pickAllSorted();
	if (pickResult != null) {
		for (int index = 0; index < pickResult.length; index++) {
			Node node = pickResult[index].getNode(PickResult.SHAPE3D);
			if (node != null) {
				Data3dPointIndex pointIndex =
							(Data3dPointIndex)((Shape3D)node).getUserData();
				if (pointIndex == null) {
					continue;
				}
				JCData3dPointIndex dataIndex =
					        new JCData3dPointIndex(pointIndex.getSeries(),
												   pointIndex.getPoint());
				Point pt = unpick(dataIndex);
				double deltaX = xPixel - pt.x;
				double deltaY = yPixel - pt.y;
				double distance = Math.sqrt((deltaX * deltaX)+(deltaY * deltaY));
				dataIndex.setDistance(distance);
				return(dataIndex);
			}
		}
	}
	return(null);
}

}

