/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: SkirtQA.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:06 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.j3d;

import java.util.ArrayList;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.klg.jclass.chart3d.Chart3dGridData;
import com.klg.jclass.chart3d.GridLimits;
import com.klg.jclass.chart3d.JCAxis;

public class SkirtQA {

private QuadArray quadArray = null;

public SkirtQA( Matrix4d absolute, Chart3dGridData data,
				Color3f color )
{
	GridLimits gridLimits = data.getGridLimits();
	double[] xGrid = data.getXGrid();
	double[] yGrid = data.getYGrid();
	double[][] zValues = data.getZValues();
	double holeValue = data.getHoleValue();
	int initX = gridLimits.getXOffset();
	int initY = gridLimits.getYOffset();
	int numX = gridLimits.getNumX();
	int numY = gridLimits.getNumY();
	Point3d  verts[][] = new Point3d[numX][numY];

	JCAxis zAxis = data.getDataView().getChart3d().getChart3dArea().getAxis(JCAxis.AXIS_Z);
	Point3d originPnt = new Point3d(0.0, 0.0, zAxis.getMin());
	absolute.transform(originPnt);
	double zOrigin = originPnt.z;

	for (int i = 0; i < numX; i++) {
		for (int j = 0; j < numY; j++) {
			Point3d scaledPnt = new Point3d(xGrid[initX + i],
										    yGrid[initY + j],
										    zValues[initX + i][initY + j]);
			absolute.transform(scaledPnt);
			verts[i][j] = new Point3d(scaledPnt);
		}
	}

    ArrayList pntList = new ArrayList();

	int vertexCount = 0;
	for (int i = 0; i < (numX-1); i++)  //  for each column in the grid
	{
		for (int j = 0; j < (numY-1); j++)  //  for each row in the grid
		{
			try {

				//  Create a skirt quadrilateral from (x,y) to (x+1,y)
				//      - if vertex is at the bottom edge of the surface, and does not contain a hole value
				//      - if vertex is at the top edge of a hole, and the grid cell above it does not contain a hole
				//      - if vertex is at the bottom edge of a hole, and the grid cell below it does not contain a hole
				if ( ( (j==0) && !containsHoleValue(initX+i, initY+j, zValues, holeValue) ) ||
				     ( ((initY+j-1) >= 0) && containsHoleValue(initX+i, initY+j-1, zValues, holeValue) && !containsHoleValue(initX+i, initY+j, zValues, holeValue) ) ||
				     ( ((initY+j-1) >= 0) && containsHoleValue(initX+i, initY+j, zValues, holeValue) && !containsHoleValue(initX+i, initY+j-1, zValues, holeValue) ) ) {
					pntList.add(verts[i][j]);
			       	pntList.add(verts[i+1][j]);
			       	pntList.add( new Point3d(verts[i+1][j].x, verts[i+1][j].y, zOrigin) );
			       	pntList.add( new Point3d(verts[i][j].x, verts[i][j].y, zOrigin) );
				}
				//  Create a skirt quadrilateral from (x,y+1) to (x+1,y+1)
				//  if vertex is at the top edge of the surface, and does not contain a hole value:
				if ( (j==(numY-2)) && !containsHoleValue(initX+i, initY+j, zValues, holeValue) ) {
					pntList.add(verts[i][j+1]);
			       	pntList.add(verts[i+1][j+1]);
			       	pntList.add( new Point3d(verts[i+1][j+1].x, verts[i+1][j+1].y, zOrigin) );
			       	pntList.add( new Point3d(verts[i][j+1].x, verts[i][j+1].y, zOrigin) );
				}

				//  Create a skirt quadrilateral from (x,y) to (x,y+1)
				//      - if vertex is at the left edge of the surface, and does not contain a hole value
				//      - if vertex is at the right edge of a hole, and the grid cell to its right does not contain a hole
				//      - if vertex is at the left edge of a hole, and the grid cell to its left does not contain a hole
				if ( ( (i==0) && !containsHoleValue(initX+i, initY+j, zValues, holeValue) ) ||
				     ( ((initX+i-1) >= 0) && containsHoleValue(initX+i-1, initY+j, zValues, holeValue) && !containsHoleValue(initX+i, initY+j, zValues, holeValue) ) ||
				     ( ((initX+i-1) >= 0) && !containsHoleValue(initX+i-1, initY+j, zValues, holeValue) && containsHoleValue(initX+i, initY+j, zValues, holeValue) ) ) {
					pntList.add(verts[i][j]);
			       	pntList.add(verts[i][j+1]);
			       	pntList.add( new Point3d(verts[i][j+1].x, verts[i][j+1].y, zOrigin) );
			       	pntList.add( new Point3d(verts[i][j].x, verts[i][j].y, zOrigin) );
				}
				//  Create a skirt quadrilateral from (x+1,y) to (x+1,y+1) if
				//  if vertex is at the right edge of the surface, and does not contain a hole value:
				if ( (i == (numX-2)) && !containsHoleValue(initX+i, initY+j, zValues, holeValue) ) {
					pntList.add(verts[i+1][j]);
			       	pntList.add(verts[i+1][j+1]);
			       	pntList.add( new Point3d(verts[i+1][j+1].x, verts[i+1][j+1].y, zOrigin) );
			       	pntList.add( new Point3d(verts[i+1][j].x, verts[i+1][j].y, zOrigin) );
				}

				//  Create surface at bottom of the skirt:
                if (containsHoleValue(initX + i, initY + j,
                                      zValues, holeValue) == false) {
			        pntList.add( new Point3d(verts[i][j].x, verts[i][j].y, zOrigin));
			        pntList.add( new Point3d(verts[i+1][j].x, verts[i+1][j].y, zOrigin));
			        pntList.add( new Point3d(verts[i+1][j+1].x, verts[i+1][j+1].y, zOrigin));
			        pntList.add( new Point3d(verts[i][j+1].x, verts[i][j+1].y, zOrigin));
				}
			}
			catch (ArrayIndexOutOfBoundsException e) {
				System.out.println("Caught an ArrayIndexOutOfBoundsException!");
			}
		}
	}

    int nPoints = pntList.size();
	Point3d pnts[] = new Point3d[nPoints];
    pntList.toArray(pnts);
	Color3f clrs[] = new Color3f[nPoints];
	for (int i = 0; i < nPoints; i++) {
		clrs[i] = new Color3f(color);
	}

	GeometryInfo gi = new GeometryInfo(GeometryInfo.QUAD_ARRAY);
	gi.setCoordinates(pnts);
	gi.setColors(clrs);
	NormalGenerator ng = new NormalGenerator();
	ng.generateNormals(gi);
	quadArray = (QuadArray)gi.getGeometryArray();
}

private boolean containsHoleValue(int x, int y, double[][] zValue,
                                  double holeValue)
{
    return ( (zValue[x][y] == holeValue)     ||
             (zValue[x+1][y] == holeValue)   ||
             (zValue[x+1][y+1] == holeValue) ||
             (zValue[x][y+1] == holeValue) );
}

public GeometryArray getGeometryArray()
{
    return quadArray;
}

}
