/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: SquareQA.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:06 $  $Locker:  $  Quest Software Inc.

//  Java 3D QuadArray definition for a square.

package com.klg.jclass.chart3d.j3d;

import javax.media.j3d.*;
import javax.vecmath.*;
import com.klg.jclass.chart3d.JCAxis;

public class SquareQA extends QuadArray {

public SquareQA(Color3f color)
{
	super(4, GeometryArray.COORDINATES | GeometryArray.TEXTURE_COORDINATE_2 |
		     GeometryArray.COLOR_3 );

	setCapability(GeometryArray.ALLOW_TEXCOORD_WRITE);
	setCapability(GeometryArray.ALLOW_TEXCOORD_READ);

	Point3d verts[] = new Point3d[4];
	Color3f clr[] = new Color3f[4];

//  Quartet of vertices defining the quadrilateral.
	verts[0] = new Point3d( 1.0, 1.0, 0.0);
	verts[1] = new Point3d(-1.0, 1.0, 0.0);
	verts[2] = new Point3d(-1.0,-1.0, 0.0);
	verts[3] = new Point3d( 1.0,-1.0, 0.0);

	for (int i=0; i<4; i++) {
		clr[i] = color;
	}

	setCoordinates(0, verts);
	setColors(0, clr);
	setTextureCoordinates(0,0,1,1);
}


/**
 * Maps this quad's four vertices to the texture map as follows:
 * 		vertex ( 1, 1, 0) > texture(xMax, yMin)
 * 		vertex (-1, 1, 0) > texture(xMin, yMin)
 * 		vertex (-1,-1, 0) > texture(xMin, yMax)
 * 		vertex ( 1,-1, 0) > texture(xMax, yMax)
 */
public void setTextureCoordinates(float xMin, float yMin, float xMax, float yMax)
{
	float[] coords = {
				xMax, yMin,
				xMin, yMin,
				xMin, yMax,
				xMax, yMax
	};

	setTextureCoordinates(0, 0, coords, 0, 4);
}

/**
 * Flip texture coordinates about the specified axis (x or y).
 */
public void flipTextureCoordinates(int axis)
{
	TexCoord2f[] texCoords = new TexCoord2f[4];
	for (int i=0; i<4; i++) {
		texCoords[i] = new TexCoord2f();
	}
	getTextureCoordinates(0, 0, texCoords);
	if (axis == JCAxis.AXIS_X) {
	    TexCoord2f temp0 = new TexCoord2f(texCoords[0]);
	    TexCoord2f temp1 = new TexCoord2f(texCoords[1]);
	    texCoords[0] = texCoords[3];
	    texCoords[1] = texCoords[2];
	    texCoords[2] = temp1;
	    texCoords[3] = temp0;
		setTextureCoordinates(0, 0, texCoords, 0, 4);
	}
	if (axis == JCAxis.AXIS_Y) {
		TexCoord2f temp1 = new TexCoord2f(texCoords[1]);
		TexCoord2f temp2 = new TexCoord2f(texCoords[2]);
	    texCoords[1] = texCoords[0];
	    texCoords[2] = texCoords[3];
	    texCoords[0] = temp1;
	    texCoords[3] = temp2;
		setTextureCoordinates(0, 0, texCoords, 0, 4);
	}
}

public void rotateTextureCoordinates(int n, boolean clockwise)
{
	TexCoord2f[] texCoords = new TexCoord2f[4];
	for (int i=0; i<4; i++) {
		texCoords[i] = new TexCoord2f();
	}
	getTextureCoordinates(0, 0, texCoords);

	for (int i = 0; i < n; i++) {
		TexCoord2f temp0 = new TexCoord2f(texCoords[0]);
		if (clockwise) {
			texCoords[0] = texCoords[1];
			texCoords[1] = texCoords[2];
			texCoords[2] = texCoords[3];
			texCoords[3] = temp0;
		}
		else {
			texCoords[0] = texCoords[3];
			texCoords[3] = texCoords[2];
			texCoords[2] = texCoords[1];
			texCoords[1] = temp0;
		}
	}
	setTextureCoordinates(0, 0, texCoords, 0, 4);
}

}
