/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: EditAction.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:06 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.j3d.actions;

import com.sun.j3d.utils.picking.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.awt.event.InputEvent;
import java.awt.Point;

import com.klg.jclass.chart3d.*;
import com.klg.jclass.chart3d.j3d.*;

/**
 * Handler object for edit actions within JClass Chart 3D Java3D.  When
 * a edit action is started, <code>JCChart3dArea</code> will create an
 * instance of <code>EditAction</code> and call <code>start()</code>,
 * <code>animate()</code> and <code>end()</code> at appropriate points.  A
 * mapping from event to action type is stored the chart's JCActionTable.
 *
 * @see com.klg.jclass.chart3d.JCAction
 * @see com.klg.jclass.chart3d.JCActionTable
 * @see com.klg.jclass.chart3d.j3d.actions.BaseAction
 */
public class EditAction extends BaseAction {

protected JCChart3dAreaJava3d area = null;
protected BranchGroup objRoot = null;
protected Canvas3D canvas3D = null;
protected DataAdjustmentAxis adjustAxis = null;
protected double radiusOfInfluence = 0.0;
protected Vector3d unitZ = null;

public EditAction()
{
	super();
}

/**
 *
 *
 * @param object Initialization parameters
 */
public void initialize(Object object)
{
    super.initialize(object);
    if (object instanceof ActionHandlerParams) {
		ActionHandlerParams params = (ActionHandlerParams)object;
		area = params.getChart3dAreaJava3d();
		canvas3D = params.getCanvas3D();
		objRoot = params.getObjectRoot();
		radiusOfInfluence = getChart3d().getDataView(0).getRadiusOfInfluence();
    }
}

public void start(InputEvent event, int x, int y)
{
    last = new Point(x, y);

	MapPick mapPick = null;
	Chart3dData data = getChart3d().getDataView(0).getElevationData();
	if (data instanceof Chart3dGridData) {
		mapPick = new MapPickGrid(data);
		((MapPickGrid)mapPick).enablePickClosestDataPoint(true);
	}
	else {
		mapPick = new MapPickPoint(data);
	}
	mapPick.setCanvas3D(canvas3D);
	mapPick.setBranchGroup(objRoot);
	mapPick.setTolerance(1.0f);
	Point3d pntIntersect = mapPick.map(x, y);
	if (pntIntersect == null) {
		return;
	}
	//  Point3d returned from mapPickGrid() is in data space.
	Matrix4d absolute = area.getTransform().getAbsolute();
	//  Convert from data space to "local" coordinates:
	absolute.transform(pntIntersect);
	if (adjustAxis == null) {
		adjustAxis = new DataAdjustmentAxis((JCChart3dJava3d)getChart3d(),
											pntIntersect,
											radiusOfInfluence);
	}

	transformGroup = plotCube.getTransformGroup();
	currXform = new Transform3D();
	transformGroup.getTransform(currXform);

	unitZ = new Vector3d(0.0, 0.0, 1.0);
	currXform.transform(unitZ);
}

public void animate(InputEvent event, int x, int y)
{
	if (adjustAxis != null) {
		if (!adjustAxis.isBusy()) {
			double dx = x - last.x;
			double dy = y - last.y;
			Vector3d vec = new Vector3d(dx, -dy, 0.0);
			double dotProduct = unitZ.dot(vec);
			adjustAxis.setZCoord(dotProduct);
			last.x = x;
			last.y = y;
		}
	}
}

public void reanimate(int x, int y) {}

public void end(InputEvent event, int x, int y) {
	//  remove DataAdjustmentAxis object
	if (adjustAxis != null) {
		plotCube.removePickAxis();
		adjustAxis = null;
	}
	transformGroup = null;
	currXform = null;
	unitZ = null;
	area.updatePlotCube();
}

public void cancel() {}

public void setRadiusOfInfluence(double radius)
{
	radiusOfInfluence = radius;
}

public double getRadiusOfInfluence()
{
	return(radiusOfInfluence);
}

}
