/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PickAction.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:06 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.j3d.actions;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.InputEvent;
import javax.vecmath.Matrix4d;



/**
 * Handler object for pick actions within JClass Chart 3D Java3D.  When
 * a pick action is started, <code>JCChart3dArea</code> will create an
 * instance of <code>PickAction</code> and call <code>start()</code>,
 * <code>animate()</code> and <code>end()</code> at appropriate points.  A
 * mapping from event to action type is stored the chart's JCActionTable.
 *
 * @see com.klg.jclass.chart3d.JCAction
 * @see com.klg.jclass.chart3d.JCActionTable
 * @see com.klg.jclass.chart3d.j3d.actions.BaseAction
 */
public class PickAction extends BaseAction {

/**
 * Default constructor.
 */
public PickAction() {}

/**
 * Called by <code>JCChart3dArea</code> when the action is started.
 *
 * @param event The event for the action
 * @param x The x pixel position where customizer is starting
 * @param y The y pixel position where customizer is starting
 */
public void start(InputEvent event, int x, int y)
{
	if (getChart3dArea() == null || !enabled) {
		return;
	}
//  Point point = new Point(getChart3d().convertChartAreaXToX(x),
//                          getChart3d().convertChartAreaYToY(y));
//  getChart3d().sendPick3dEvent(getChart3d().pick(point, null));
    getChart3d().sendPick3dEvent(getChart3d().pick(new Point(x, y), null));
}

/**
 * Called by <code>JCChart3dArea</code> when the action is to be animated.
 *
 * @param event The event for the action
 * @param x The current x position to animate the action
 * @param y The current y position to animate the action
 */
public void animate(InputEvent event, int x, int y) {}

/**
 * Called by <code>JCChart3dArea</code> when the chart area has been refreshed
 * and the animation must be redrawn.
 *
 * @param x The x position to re-animate the action
 * @param y The y position to re-animate the action
 */
public void reanimate(int x, int y) {}

/**
 * Called by <code>JCChart3dArea</code> when the action is over.
 *
 * @param event The event for the action
 * @param x The x position to end the action
 * @param y The y position to end the action
 */
public void end(InputEvent event, int x, int y) {}

/**
 * Called by <code>JCChart3dArea</code> when the action is cancelled
 */
public void cancel() {}

}
