/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ScaleAction.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:06 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.j3d.actions;

import com.sun.j3d.utils.picking.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import com.klg.jclass.chart3d.*;
import com.klg.jclass.chart3d.j3d.*;

/**
 * Handler object for scale actions within JClass Chart 3D Java3D.  When
 * a scale action is started, <code>JCChart3dArea</code> will create an
 * instance of <code>ScaleAction</code> and call <code>start()</code>,
 * <code>animate()</code> and <code>end()</code> at appropriate points.  A
 * mapping from event to action type is stored the chart's JCActionTable.
 *
 * @see com.klg.jclass.chart3d.JCAction
 * @see com.klg.jclass.chart3d.JCActionTable
 * @see com.klg.jclass.chart3d.j3d.actions.BaseAction
 */
public class ScaleAction extends BaseAction {

/**
 * Default constructor.
 */
public ScaleAction() {}

/**
 *
 *
 * @param object Initialization parameters
 */
public void initialize(Object object)
{
    super.initialize(object);
//  Needed to fix mouse behavior when the viewer's orientation is
//  not normal to the XY plane.
    z_factor = .04;
}

/**
 * Called by <code>JCChart3dArea</code> when the action is started.
 *
 * @param event The InputEvent that triggered this action
 * @param x The x pixel position where customizer is starting
 * @param y The y pixel position where customizer is starting
 */
public void start(InputEvent event, int x, int y)
{
    doStart(event, x, y);
}

/**
 * Called by <code>JCChart3dArea</code> when the action is to be animated.
 *
 * @param event The InputEvent that triggered this action
 * @param x The current x position to animate the action
 * @param y The current y position to animate the action
 */
public void animate(InputEvent event, int x, int y)
{
	if (transformGroup == null) {
		return;
	}
	if (last == null) {
		return;
	}
    if (event instanceof KeyEvent) {
        switch (((KeyEvent)event).getKeyCode()) {
        case KeyEvent.VK_PAGE_UP:
            x = last.x;
            y = last.y + 1;
        break;
        case KeyEvent.VK_PAGE_DOWN:
            x = last.x;
            y = last.y - 1;
        break;
        }
    }

	int dx = x - last.x;
	int dy = y - last.y;

	transformGroup.getTransform(currXform);

	//  Fix for mouse behavior when the viewer's orientation is not normal to
	// the XY plane:
	Transform3D viewPlatformT3d = null;

	//  In case constructor without Viewer's TransformGroup was invoked.
	if (null != ViewerTG) {
		// Use this Transform3D to hold Viewer's Transform Group data
		viewPlatformT3d = new Transform3D();

		// Get the T3D of the Viewer's TG
		ViewerTG.getTransform(viewPlatformT3d);

		// Set Translation to origin so multiplications don't impact it
		viewPlatformT3d.setTranslation(new Vector3d(0.0,0.0,0.0));

		// Invert the Viewer's T3D so we can remove it from the objects
		// Transform
		viewPlatformT3d.invert();

		// Multiply the inverted Viewer T3D to factor it out of the objects
		// Transform
		currXform.mul(viewPlatformT3d, currXform);
	}

	// Perform rotations like we were looking normal to the XY plane
	double zValue = dy * z_factor;
	translation.x = 0.0;
	translation.y = 0.0;
	translation.z = zValue;

	transformX.set(translation);

	if (invert) {
		currXform.mul(currXform, transformX);
	}
	else {
		currXform.mul(transformX, currXform);
	}

	if (null != ViewerTG) {
		// Now that the rotations are applied correctly we need to reapply
		// Viewer's T3D. Invert the Viewer's T3D so we can add it back in to
		// the objects Transform
		viewPlatformT3d.invert();

		// Multiply the original Viewer T3D to factor it back in to the
		// objects Transform
		currXform.mul(viewPlatformT3d, currXform);
	}

	transformGroup.setTransform(currXform);

	last.x = x;
	last.y = y;

	currXform.get(translation);
	virtualUniverse.adjustFogDistance(-zValue);
}

/**
 * Called by <code>JCChart3dArea</code> when the chart area has been refreshed
 * and the animation must be redrawn.
 *
 * @param x The x position to re-animate the action
 * @param y The y position to re-animate the action
 */
public void reanimate(int x, int y) {}

/**
 * Called by <code>JCChart3dArea</code> when the action is over.
 *
 * @param event The event for the action
 * @param x The x position to end the action
 * @param y The y position to end the action
 */
public void end(InputEvent event, int x, int y) {
	Transform3D t3d = new Transform3D();
	transformGroup.getTransform(t3d);
	t3d.get(translation);
	double perspective =
	    (virtualUniverse.getViewPlatformDistance() - translation.z) / 2.0;
	chart3dArea.getView3d().setPerspective(perspective);
	if (transformGroup != null) {
		transformGroup.actionEnd();   //  update axis annotation
		transformGroup = null;
	}
    doEnd(event, x, y);
}

/**
 * Called by <code>JCChart3dArea</code> when the action is cancelled
 */
public void cancel() {}

}
