/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Chart3dJava3d.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:07 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.j3d.beans;

import com.klg.jclass.chart3d.j3d.JCChart3dJava3d;
import com.klg.jclass.chart3d.*;
import com.klg.jclass.chart3d.beans.*;
import com.klg.jclass.chart3d.data.*;

import javax.swing.table.TableModel;

import com.klg.jclass.util.legend.JCLegend;

import java.awt.*;
import javax.swing.*;
import java.util.StringTokenizer;

/**
 * This is the bean class for the 2d api version of JChart3d (JCChart3dJava2d).
 */
public class Chart3dJava3d extends JCChart3dJava3d implements java.io.Serializable {


/** Enums for axis placement values.
public final static int AUTOMATIC = JCAxis.AUTOMATIC;
public final static int ORIGIN    = JCAxis.ORIGIN;
public final static int MIN       = JCAxis.MIN;
public final static int MAX       = JCAxis.MAX;
public final static int VALUE_ANCHORED = JCAxis.VALUE_ANCHORED;

/** Enums for annotation method values. */
public final static int VALUE        = JCAxis.ANNOTATION_VALUES;
public final static int VALUE_LABELS = JCAxis.ANNOTATION_DATA_LABELS;
public final static int TIME_LABELS  = JCAxis.ANNOTATION_VALUE_LABELS;




/** Enums for legend anchor values. */
public final static int  NORTHEAST = JCLegend.NORTHEAST;
public final static int  NORTH     = JCLegend.NORTH;
public final static int  NORTHWEST = JCLegend.NORTHWEST;
public final static int  EAST      = JCLegend.EAST;
public final static int  WEST      = JCLegend.WEST;
public final static int  SOUTHEAST = JCLegend.SOUTHEAST;
public final static int  SOUTH     = JCLegend.SOUTH;
public final static int  SOUTHWEST = JCLegend.SOUTHWEST;

/** Enums for legend orientation values. */
public final static int  HORIZONTAL= JCLegend.HORIZONTAL;
public final static int  VERTICAL  = JCLegend.VERTICAL;



private String headerText;
private String footerText;

//Data1
private int data1ChartType;
private boolean data1ElevationMeshed;
private boolean data1ElevationShaded;
private boolean data1ElevationTransparent;
private boolean data1ContouredZoned;
private boolean data1ContourContoured;


//Chart3dArea
private boolean areaOpaque;
private Color areaBackground;
private Color areaForeground;

//Surface
private boolean surfaceSolid;
private boolean surfaceXMeshShowing;
private boolean surfaceYMeshShowing;

//PlotCube
private Color plotCubeBackground;
private Color plotCubeForeground;

private boolean floorContoured;
private boolean floorZoned;
private boolean ceilingContoured;
private boolean ceilingZoned;

//View3D
private double xRotation;
private double yRotation;
private double zRotation;



//X-axis
private int xAxisAnnotationMethod;
private double  xAxisMin;
private double  xAxisMax;
private String xAxisTitleText;
//Y-axis
private int yAxisAnnotationMethod;
private double  yAxisMin;
private double  yAxisMax;
private String yAxisTitleText;
//Z-axis
private int zAxisAnnotationMethod;
private double  zAxisMin;
private double  zAxisMax;
private String zAxisTitleText;
//Legends
private boolean legendVisible;
private int legendAnchor;
private int legendOrientation;
private Font headerFont;
private Font footerFont;

private TableModel elevationSwingDataModel1;
private TableModel zoneSwingDataModel1;

private String zoneDataSource1;
private String elevationDataSource1;

//private TriggerWrapper triggerList;





/**
 * No argument constructor to satisfy the bean spec requirement.
 * Calls super()
 */
public Chart3dJava3d() {
	super();
	headerText = getHeaderText();
	footerText = getFooterText();

	//Data1
	data1ChartType = getData1ChartType();
	data1ElevationMeshed = isData1ElevationMeshed();
	data1ElevationShaded = isData1ElevationShaded();
	data1ElevationTransparent = isData1ElevationTransparent();
	data1ContouredZoned = isData1ContourZoned();
	data1ContourContoured = isData1ContourContoured();
	setPreferredSize(new Dimension(400,400));
	setMinimumSize(new Dimension(100, 100));


	//Chart3dArea
	areaOpaque = isAreaOpaque();
	areaBackground = getAreaBackground();
	areaForeground = getAreaForeground();

	//Surface
	surfaceSolid = isSurfaceSolid();
	surfaceXMeshShowing = isSurfaceXMeshShowing();
	surfaceYMeshShowing = isSurfaceYMeshShowing();

	//PlotCube
	plotCubeBackground = getPlotCubeBackground();
	plotCubeForeground = getPlotCubeForeground();

	floorContoured = isFloorContoured();
	floorZoned = isFloorZoned();
	ceilingContoured = isCeilingContoured();
	ceilingZoned = isCeilingZoned();

	//View3D
	xRotation = getXRotation();
	yRotation = getYRotation();
	zRotation = getZRotation();



	//X-axis
	xAxisAnnotationMethod = getXAxisAnnotationMethod();
	xAxisMin = getXAxisMin();
	xAxisMax = getXAxisMax();
	xAxisTitleText = getXAxisTitleText();
	//Y-axis
	yAxisAnnotationMethod = getYAxisAnnotationMethod();
	yAxisMin = getYAxisMin();
	yAxisMax = getYAxisMax();
	yAxisTitleText = getYAxisTitleText();

	//Z-axis
	zAxisAnnotationMethod = getZAxisAnnotationMethod();
	zAxisMin = getZAxisMin();
	zAxisMax = getZAxisMax();
	zAxisTitleText = getZAxisTitleText();

	//Legends
	legendVisible = isLegendVisible();
	legendAnchor = getLegendAnchor();
	legendOrientation = getLegendOrientation();
	headerFont = getHeaderFont();
	footerFont = getFooterFont();

	elevationSwingDataModel1 = getElevationSwingDataModel1();
	zoneSwingDataModel1 = getZoneSwingDataModel1();
	//setTriggerList(new TriggerWrapper("None","Zoom","None","None","Customize",true));



}//no argument constructor

/**
 *  Gets the values of the properties encapsulated in the TriggerList
 *  property.
 *
public TriggerWrapper getTriggerList() {



	return triggerList;
}

/**
 *  Sets the values of the properties encapsulated in the TriggerList
 *  property.
 *
public void setTriggerList(TriggerWrapper tw) {
	try {
	boolean oldBatched = false;
	oldBatched = isBatched();
	setBatched(true);

	tw.setPropertyValues(this);

	setBatched(oldBatched);
	}
	catch ( Exception e ) {
		e.printStackTrace();
	}
	triggerList = tw;

}


*/


/****************** Data1 ******************/


/**
 * Sets the chart type for the first dataview.
 * Acceptable types are SURFACE, BAR and SCATTER
 *
 * @param   chartType  int representing the desired chart type
 */
public void setData1ChartType(int chartType) {
	this.data1ChartType = chartType;
	getDataView(0).setChartType(this.data1ChartType);
}//setData1ChartType


/**
 * Gets the chart type for the first dataview
 *
 * @return the int which represents the chart type
 */
public int getData1ChartType() {
	return getDataView(0).getChartType();
}//getData1ChartType

/**
 * Sets the Elevation Data1 SwingDataModel property, if a SwingDataModel
 * is to be used.
 * @param a Swing TableModel
 */
public void setElevationSwingDataModel1(TableModel tm) {
	getDataView(0).setElevationDataSource(new JCSwing3dDataSource(tm));
}//setElevationData1SwingModel

/**
 * Retrieves the Elevation DataSource Data1 SwingDataModel, if applicable
 *
 * @return SwingDataModel used to create the datasource.
 */
public TableModel getElevationSwingDataModel1() {
	if (getDataView(0).getElevationDataSource() instanceof JCSwing3dDataSource) {
		return ((JCSwing3dDataSource)getDataView(0).getElevationDataSource()).getTableModel();
	} else {
		return null;
	}
}//getData1SwingModel

/**
 * Sets the Zone Data1 SwingDataModel property, if a SwingDataModel
 * is to be used.
 * @param a Swing TableModel
 */
public void setZoneSwingDataModel1(TableModel tm) {
	getDataView(0).setZoneDataSource(new JCSwing3dDataSource(tm));
}//setZoneData1SwingModel

/**
 * Retrieves the Zone Data1 SwingDataModel, if applicable
 *
 * @return SwingDataModel used to create the datasource.
 */
public TableModel getZoneSwingDataModel1() {
	if (getDataView(0).getZoneDataSource() instanceof JCSwing3dDataSource) {
		return ((JCSwing3dDataSource)getDataView(0).getZoneDataSource()).getTableModel();
	} else {
		return null;
	}
}//getZoneData1SwingModel

/**
 * Sets the Elevation Data1 SwingDataModel property, if a SwingDataModel
 * is to be used.
 * @param a Swing TableModel
 */
public void setElevationDataSource1(String dataString) {
	this.elevationDataSource1 = dataString;


	getDataView(0).setElevationDataSource(getDataSourceFromString(dataString));
}//setElevationDataSource1

/**
 * Retrieves the Elevation DataSource Data1 SwingDataModel, if applicable
 *
 * @return SwingDataModel used to create the datasource.
 */
public String getElevationDataSource1() {
	if(elevationDataSource1 == null) {
		if (getDataView(0).getElevationDataSource() == null) {
			elevationDataSource1 = dataToString((Chart3dGridDataModel)this);
		}
		else {
			elevationDataSource1 = dataToString((Chart3dGridDataModel)getDataView(0).getElevationDataSource());
		}
		//setElevationDataSource1("");
	}
	return elevationDataSource1;

}//getElevationDataSource1

/**
 * Sets the Zone Data1 SwingDataModel property, if a SwingDataModel
 * is to be used.
 * @param a Swing TableModel
 */
public void setZoneDataSource1(String dataString) {
	this.zoneDataSource1 = dataString ;
	getDataView(0).setZoneDataSource(((Chart3dGridDataModel)getDataSourceFromString(dataString)));
}//setZoneDataSource1

/**
 * Retrieves the Zone Data1 SwingDataModel, if applicable
 *
 * @return SwingDataModel used to create the datasource.
 */
public String getZoneDataSource1() {

	if(zoneDataSource1 == null) {
		if (getDataView(0).getZoneDataSource() == null) {
			zoneDataSource1 = dataToString((Chart3dGridDataModel)this);
		}
		else {
			zoneDataSource1 = dataToString((Chart3dGridDataModel)getDataView(0).getZoneDataSource());
		}

	}
	return zoneDataSource1;
}//getZoneDataSource1




/**
 * Convenience method which sets the meshed property for the
 * first dataview's elevation.
 *
 * @param meshed shall mesh lines be drawn?
 */
public void setData1ElevationMeshed(boolean meshed) {
	getDataView(0).getElevation().setMeshed(meshed);
}//setData1ElevationMeshed



/**
 * Convenience method which gets the meshed property for the
 * first dataview's elevation
 *
 * @return boolean are mesh lines drawn?
 */
public boolean isData1ElevationMeshed() {
	return getDataView(0).getElevation().isMeshed();
}//isData1ElevationMeshed

/**
 * Convenience method which sets the shaded property for the
 * first dataview's elevation.
 *
 * @param shaded should the elevation be shaded
 */
public void setData1ElevationShaded(boolean shaded) {
	getDataView(0).getElevation().setShaded(shaded);
}//setData1ElevationShaded



/**
 * Convenience method which gets the shaded property for the
 * first dataview's elevation
 *
 * @return is the elevation shaded?
 */
public boolean isData1ElevationShaded() {
	return getDataView(0).getElevation().isShaded();
}//isData1ElevationShaded

/**
 * Convenience method which sets the transparent property for the
 * first dataview's elevation.
 *
 * @param transparent should the elevation be transparent
 */
public void setData1ElevationTransparent(boolean transparent) {
	getDataView(0).getElevation().setTransparent(transparent);
}//setData1ElevationTransparent



/**
 * Convenience method which gets the transparent property for the
 * first dataview's elevation
 *
 * @return is the elevation transparent?
 */
public boolean isData1ElevationTransparent() {
	return getDataView(0).getElevation().isTransparent();
}//isData1ElevationTransparent

/**
 * Convenience method which sets the zoned property for the
 * first dataview's contour.
 *
 * @param zoned should the contour be zoned
 */
public void setData1ContourZoned(boolean zoned) {
	getDataView(0).getContour().setZoned(zoned);
}//setData1ContourZoned



/**
 * Convenience method which gets the zoned property for the
 * first dataview's contour
 *
 * @return is the contour zoned?
 */
public boolean isData1ContourZoned() {
	return getDataView(0).getContour().isZoned();
}//isData1ContourZoned

/**
 * Convenience method which sets the contoured property for the
 * first dataview's contour.
 *
 * @param contoured should the contour be contoured
 */
public void setData1ContourContoured(boolean contoured) {
	getDataView(0).getContour().setContoured(contoured);
}//setData1ContourContoured



/**
 * Convenience method which gets the contoured property for the
 * first dataview's contour
 *
 * @return is the contour contoured?
 */
public boolean isData1ContourContoured() {
	return getDataView(0).getContour().isContoured();
}//isData1ContourContoured





/****************** ChartArea ******************/




/**
 * Sets the Chart3dArea opaque
 *
 * @param opaque boolean true makes the Chart3dArea opaque
 */
public void setAreaOpaque(boolean opaque) {
	getChart3dArea().setOpaque(opaque);
}//setAreaOpaque


/**
 * Gets whether the Chart3dArea is opaque
 *
 * @return boolean true if the Chart3dArea is opaque
 */
public boolean isAreaOpaque() {
	return getChart3dArea().isOpaque();
}//isAreaOpaque


/**
 * Sets the background color of the Chart3dArea
 *
 * @param   color  the desired background color
 */
public void setAreaBackground(Color color) {
	getChart3dArea().setBackground(color);
}//setAreaBackground


/**
 * Gets the background color of the Chart3dArea
 *
 * @return Color the current background color of the Chart3dArea
 */
public Color getAreaBackground() {
	return getChart3dArea().getBackground();
}//getAreaBackground



/**
 * Sets the foreground color of the Chart3dArea
 *
 * @param   color  the desired foreground color
 */
public void setAreaForeground(Color color) {
	getChart3dArea().setForeground(color);
}//setAreaForeground


/**
 * Gets the foreground color of the Chart3dArea
 *
 * @return color the current foreground color of the Chart3dArea
 */
public Color getAreaForeground() {
	return getChart3dArea().getForeground();
}//getAreaForeground

/****************** Surface ******************/


/**
 * Dictates whether or not the surface of the data is
 * solid
 *
 * @param   solid  boolean to tell if the surface should be
 * solid
 */
public void setSurfaceSolid(boolean solid) {
	getChart3dArea().getSurface().setSolid(solid);
}//setSurfaceSolid


/**
 * Retrieves whether or not the surface of the data is solid
 *
 * @return boolean is the surface of the data solid?
 */
public boolean isSurfaceSolid() {
	return getChart3dArea().getSurface().isSolid();
}//isSurfaceSolid



/**
 * Sets whether or not to show the x Mesh
 *
 * @param   showing  boolean to tell if the x Mesh should be
 * showing
 */
public void setSurfaceXMeshShowing(boolean showing) {
	getChart3dArea().getSurface().setXMeshShowing(showing);
}//setSurfaceXMeshShowing


/**
 * Retrieves whether or not the x Mesh is showing
 *
 * @return  boolean is the X mesh showing?
 */
public boolean isSurfaceXMeshShowing() {
	return getChart3dArea().getSurface().isXMeshShowing();
}//getSurfaceXMeshShowing


/**
 * Sets whether or not to show the y Mesh
 *
 * @param   showing  boolean to tell if the y Mesh should be
 * showing
 */
public void setSurfaceYMeshShowing(boolean showing) {
	getChart3dArea().getSurface().setYMeshShowing(showing);
}//setSurfaceXMeshShowing


/**
 * Retrieves whether or not the y Mesh is showing
 *
 * @return  boolean is the Y mesh showing?
 */
public boolean isSurfaceYMeshShowing() {
	return getChart3dArea().getSurface().isYMeshShowing();
}//getSurfaceYMeshShowing



/****************** PlotCube ******************/


/**
 * Sets the background color of the PlotCube
 *
 * @param   color  the desired background color
 */
public void setPlotCubeBackground(Color color) {
	getChart3dArea().getPlotCube().setBackground(color);
}//setPlotCubeBackground


/**
 * Gets the background color of the PlotCube
 *
 * @return Color the current background color of the PlotCube
 */
public Color getPlotCubeBackground() {
	if (getChart3dArea().getPlotCube().getBackground() == null) {
		return Color.gray;
	}
	return getChart3dArea().getPlotCube().getBackground();
}//getPlotCubeBackground


/**
 * Sets the foreground color of the PlotCube
 *
 * @param   color  the desired foreground color
 */
public void setPlotCubeForeground(Color color) {
	getChart3dArea().getPlotCube().setForeground(color);
}//setPlotCubeForeground


/**
 * Gets the foreground color of the PlotCube
 *
 * @return color the current foreground color of the Cube
 */
public Color getPlotCubeForeground() {
	if (getChart3dArea().getPlotCube().getForeground() == null) {
		return Color.black;
	}

	return getChart3dArea().getPlotCube().getForeground();
}//getPlotCubeForeground


/**
 * Sets whether or not the plot cube's floor is contoured
 *
 * @param   contoured  boolean whether or not contouring is desired
 */
public void setFloorContoured(boolean contoured) {
	getChart3dArea().getPlotCube().getFloor().setContoured(contoured);
}//setFloorContoured


/**
 * Returns whether or not the plotCube floor is contoured
 *
 * @return boolean is the floor contoured
 */
public boolean isFloorContoured() {
	return getChart3dArea().getPlotCube().getFloor().isContoured();
}//isFloorContoured

/**
 * Sets whether or not the plot cube's ceiling is contoured
 *
 * @param   contoured  boolean whether or not contouring is desired
 */
public void setCeilingContoured(boolean contoured) {
	getChart3dArea().getPlotCube().getCeiling().setContoured(contoured);
}//setCeilingContoured


/**
 * Returns whether or not the plotCube ceiling is contoured
 *
 * @return boolean is the floor contoured
 */
public boolean isCeilingContoured() {
	return getChart3dArea().getPlotCube().getCeiling().isContoured();
}//isCeilingContoured


/**
 * Sets whether or not the plot cube's floor is zoned
 *
 * @param   zoned  boolean whether or not zoning is desired
 */
public void setFloorZoned(boolean zoned) {
	getChart3dArea().getPlotCube().getFloor().setZoned(zoned);
}//setFloorZoned


/**
 * Returns whether or not the plotCube floor is zoned
 *
 * @return boolean is the floor zoned
 */
public boolean isFloorZoned() {
	return getChart3dArea().getPlotCube().getFloor().isZoned();
}//isFloorZoned

/**
 * Sets whether or not the plot cube's ceiling is zoned
 *
 * @param   zoned  boolean whether or not zoning is desired
 */
public void setCeilingZoned(boolean zoned) {
	getChart3dArea().getPlotCube().getCeiling().setZoned(zoned);
}//setCeilingZoned


/**
 * Returns whether or not the plotCube ceiling is zoned
 *
 * @return boolean is the floor zoned
 */
public boolean isCeilingZoned() {
	return getChart3dArea().getPlotCube().getCeiling().isZoned();
}//isCeilingZoned

/****************** View3d ******************/


/**
 * Sets the Chart3dArea's View3d XRotation
 *
 * @param   xrotate  double representing degrees of rotation
 */
public void setXRotation(double xrotate) {
	getChart3dArea().getView3d().setXRotation(xrotate);
}//setXRotation


/**
 * Gets the Chart3dArea's View3d XRotation
 *
 * @return double representing degrees of rotation
 */
public double getXRotation() {
	return getChart3dArea().getView3d().getXRotation();
}//getXRotation

/**
 * Sets the Chart3dArea's View3d YRotation
 *
 * @param   yrotate  double representing degrees of rotation
 */
public void setYRotation(double yrotate) {
	getChart3dArea().getView3d().setYRotation(yrotate);
}//setYRotation


/**
 * Gets the Chart3dArea's View3d YRotation
 *
 * @return double representing degrees of rotation
 */
public double getYRotation() {
	return getChart3dArea().getView3d().getYRotation();
}//getYRotation

/**
 * Sets the Chart3dArea's View3d ZRotation
 *
 * @param   zrotate  double representing degrees of rotation
 */
public void setZRotation(double zrotate) {
	getChart3dArea().getView3d().setZRotation(zrotate);
}//setZRotation


/**
 * Gets the Chart3dArea's View3d ZRotation
 *
 * @return double representing degrees of rotation
 */
public double getZRotation() {
	return getChart3dArea().getView3d().getZRotation();
}//getZRotation




/****************** Legend ******************/

/**
 * Convenience method for getting the legend's visible property
 *
 * @return boolean is the legend visible
 * @see <code>com.klg.jclass.util.legend.JCLegend</code>
 */
public boolean isLegendVisible() {
	return this.getLegend().isVisible();
}//isLegendVisible


/**
 * Convenience method for setting the legend's visible property
 *
 * @param   legendVisible boolean
 * @see <code>com.klg.jclass.util.legend.JCLegend</code>
 */
public void setLegendVisible(boolean legendVisible) {
	this.getLegend().setVisible(legendVisible);
}//setlegendVisible


/**
 * Gets the value of the LegendAnchor property, which determines the
 * position of the legend relative to the ChartArea. Valid values include
 * <code>NORTH</code>, <code>SOUTH</code>, <code>EAST</code>, <code>WEST</code>,
 * <code>NORTHWEST</code>, <code>SOUTHWEST</code>, <code>NORTHEAST</code> and
 * <code>SOUTHEAST</code>.
 * @return int value of the LegendAnchor property
 */
public int getLegendAnchor() {
	return getLegend().getAnchor();
}//getLegendAnchor

/**
 * Sets the value of the LegendAnchor property, which determines the
 * position of the legend relative to the ChartArea. Valid values include
 * <code>NORTH</code>, <code>SOUTH</code>, <code>EAST</code>, <code>WEST</code>,
 * <code>NORTHWEST</code>, <code>SOUTHWEST</code>, <code>NORTHEAST</code> and
 * <code>SOUTHEAST</code>.
 * @param int value of the LegendAnchor property
 */
public void setLegendAnchor(int a) {
	try {
	getLegend().setAnchor(a);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}//try/catch
}//setLegendAnchor

/**
 * Gets the value of the LegendOrientation property, which determines
 * how legend information is laid out. Valid values include
 * <code>VERTICAL</code> and <code>HORIZONTAL</code>.
 * @return int the orientation of the legend
 */
public int getLegendOrientation() {
	return getLegend().getOrientation();
}//getLegendOrientation

/**
 * Sets the value of the LegendOrientation property, which determines
 * how legend information is laid out. Valid values include <code>VERTICAL</code>
 * and <code>HORIZONTAL</code>.
 * @param int the desired orientation of the legend
 */
public void setLegendOrientation(int a) {
	try {
	getLegend().setOrientation(a);
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}//try/catch
}//setLegendOrientation

/****************** Header ******************/

/**
 * Gets the value of the HeaderText property; the HeaderText property
 * holds the text that is displayed in the header.
 * @return String the text displayed in the header
 */
public String getHeaderText() {
	JComponent header = getHeader();
	String retStr = null;
	if (header instanceof JLabel) {
		retStr = ((JLabel)header).getText();
	}//if
	if (retStr == null) {
		retStr = new String("");
	}//if
	return retStr;
}//getHeaderText

/**
 * Sets the value of the HeaderText property; the HeaderText property
 * holds the text that is displayed in the header. It also dictates
 * the visible property.
 * @param String the text to be displayed in the header
 */
public void setHeaderText(String t) {
	try {
	JComponent header = getHeader();
	if (!(header instanceof JLabel)) {
		return;
	}//if

	if (t == null || t.length() == 0) {
		header.setVisible(false);
	}
	else {
		header.setVisible(true);
	}//if/else
	((JLabel)header).setText(t);
	update();
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}//try/catch
}//setHeaderText

/**
 * Gets the value of the HeaderFont property, which holds the font
 * that is used to display the header.
 * @return Font the font used in the header
 */
public Font getHeaderFont() {
	JComponent header = getHeader();
	if (header != null) {
		return header.getFont();
	}// if
	return getFont();
}//getHeaderFont

/**
 * Sets the value of the HeaderFont property, which holds the font
 * that is used to display the header.
 * @param Font the desired font for the header
 */
public void setHeaderFont(Font f) {
	try {
	JComponent header = getHeader();
	if (header != null) {
		header.setFont(f);
	}//if

	update();
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}//try/catch
}//setHeaderFont

/****************** Footer ******************/

/**
 * Gets the value of the FooterText property, which holds the text
 * that is displayed in the footer.
 * @return String the text displayed in the footer
 */
public String getFooterText() {
	JComponent footer = getFooter();
	String retStr = null;
	if (footer instanceof JLabel) {
		retStr = ((JLabel)footer).getText();
	}//if
	if (retStr == null) {
		retStr = new String("");
	}//if
	return retStr;
}//getFooterText

/**
 * Sets the value of the FooterText property, which holds the text
 * that is displayed in the footer. It also dictates the visible
 * property.
 * @param String the text for the footer
 */
public void setFooterText(String t) {
	try {
	JComponent footer = getFooter();
	if (!(footer instanceof JLabel)) {
		return;
	}//if
	if (t == null || t.length() == 0)  {
		footer.setVisible(false);
	}
	else {
		footer.setVisible(true);
	}//if/else
	((JLabel)footer).setText(t);
	update();
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}//try/catch
}//setFooterText

/**
 * Gets the value of the FooterFont property, which holds the font that
 * is used to display the footer.
 * @return the Font object used by the footer
 */
public Font getFooterFont() {
	JComponent footer = getFooter();
	if (footer != null) {
		return footer.getFont();
	}//if
	return getFont();
}//getFooterFont

/**
 * Sets the value of the FooterFont property, which holds the font that
 * is used to display the footer.
 * @param Font the desired font for the footer
 */
public void setFooterFont(Font f) {
	try {
	JComponent footer = getFooter();
	if (footer != null) {
		footer.setFont(f);
	}//if

	update();
	}
	catch ( Exception e ) {
		System.out.println( e.getMessage() );
	}//try/catch
}//setFooterFont

/****************** X-axis ******************/

/**
 * Sets whether the x axis is showing.
 * @param showing true - showing
 */
public void setXAxisShowing(boolean showing) {
	getChart3dArea().getAxis(JCAxis.AXIS_X).setShowing(showing);
}//setXAxisShowing

/**
 * Gets whether the x axis is showing.
 * @return boolean true - showing
 */
public boolean isXAxisShowing() {
	return getChart3dArea().getAxis(JCAxis.AXIS_X).isShowing();
}//isXAxisShowing

/**
 * Sets the annotation method for the X Axis.
 * Valid values are ANNO_VALUES, ANNO_DATA_LABELS, ANNO_VALUE_LABELS
 * @param annoMethod int selection of annotation method
 */
public void setXAxisAnnotationMethod(int annoMethod){
	getChart3dArea().getAxis(JCAxis.AXIS_X).setAnnotationMethod(annoMethod);
}//setXAxisAnnotationMethod

/**
 * Returns the annotation method for the X Axis.
 * @return int the annotation method for the x axis.
 */
public int getXAxisAnnotationMethod() {
	return getChart3dArea().getAxis(JCAxis.AXIS_X).getAnnotationMethod();
}//getXAxisAnnotationMethod


//public static final String X_AXIS_GRID_showing = "xAxisGridshowing";

/**
 * Sets the minimum value of the x axis
 * @param min double minimum value of the x axis
 */
public void setXAxisMin(double min) {
	JCAxis axis = getChart3dArea().getAxis(JCAxis.AXIS_X);
	//prevent exception from happening (reset max )
	if (axis.getMax() <= min) {
		axis.setMax(min + 1.0);
	}
	axis.setMin(min);

}//setAxisMin

/**
 * Retrieves the minimum value of the x axis
 * @return double the minimum  value of the x axis
 */
public double getXAxisMin() {
	return getChart3dArea().getAxis(JCAxis.AXIS_X).getMin();
}//getXAxisMin

/**
 * Sets the maximum value of the x axis
 * @param min double maximum value of the x axis
 */
public void setXAxisMax(double max) {
	JCAxis axis = getChart3dArea().getAxis(JCAxis.AXIS_X);
	//prevent exception from happening (reset min )
	if (axis.getMin() >= max) {
		axis.setMin(max - 1.0);
	}
	axis.setMax(max);

}//setAxisMax

/**
 * Retrieves the maximum value of the x axis
 * @return double the maximum  value of the x axis
 */
public double getXAxisMax() {
	return getChart3dArea().getAxis(JCAxis.AXIS_X).getMax();
}//getXAxisMax

/**
 * Sets the Title text for the x axis.
 * @param xText the desired String value of the Title
 */
public void setXAxisTitleText(String xText) {
	getChart3dArea().getAxis(JCAxis.AXIS_X).setTitle(xText);
}//setXTitle

/**
 * Retrieves the Title text for the x axis
 * @return String the text of the x axis title
 */
public String getXAxisTitleText() {
	if (getChart3dArea().getAxis(JCAxis.AXIS_X).getTitle() == null) {
		return "";
	}
	return getChart3dArea().getAxis(JCAxis.AXIS_X).getTitle();
}//getXTitle




/****************** Y-axis ******************/

/**
 * Sets whether the y axis is showing.
 * @param showing true - showing
 */
public void setYAxisShowing(boolean showing) {
	getChart3dArea().getAxis(JCAxis.AXIS_Y).setShowing(showing);
}//setYAxisShowing

/**
 * Gets whether the Y axis is showing.
 * @return boolean true - showing
 */
public boolean isYAxisShowing() {
	return getChart3dArea().getAxis(JCAxis.AXIS_Y).isShowing();
}//isYAxisShowing

/**
 * Sets the annotation method for the Y Axis.
 * Valid values are ANNO_VALUES, ANNO_DATA_LABELS, ANNO_VALUE_LABELS
 * @param annoMethod int selection of annotation method
 */
public void setYAxisAnnotationMethod(int annoMethod){
	getChart3dArea().getAxis(JCAxis.AXIS_Y).setAnnotationMethod(annoMethod);
}//setYAxisAnnotationMethod

/**
 * Returns the annotation method for the Y Axis.
 * @return int the annotation method for the y axis.
 */
public int getYAxisAnnotationMethod() {
	return getChart3dArea().getAxis(JCAxis.AXIS_Y).getAnnotationMethod();
}//getYAxisAnnotationMethod

/**
 * Sets the minimum value of the y axis
 * @param min double minimum value of the y axis
 */
public void setYAxisMin(double min) {
	JCAxis axis = getChart3dArea().getAxis(JCAxis.AXIS_Y);
	//prevent exception from happening (reset max )
	if (axis.getMax() <= min) {

		axis.setMax(min + 1.0);
	}
	axis.setMin(min);

}//setYAxisMin

/**
 * Retrieves the minimum value of the y axis
 * @return double the minimum  value of the y axis
 */
public double getYAxisMin() {
	return getChart3dArea().getAxis(JCAxis.AXIS_Y).getMin();
}//getYAxisMin

/**
 * Sets the maximum value of the y axis
 * @param min double maximum value of the y axis
 */
public void setYAxisMax(double max) {
	JCAxis axis = getChart3dArea().getAxis(JCAxis.AXIS_Y);
	//prevent exception from happening (reset min )
	if (axis.getMin() >= max) {
		axis.setMin(max - 1.0);
	}
	axis.setMax(max);

}//setYAxisMax


/**
 * Retrieves the maximum value of the Y axis
 * @return double the maximum  value of the y axis
 */
public double getYAxisMax() {
	return getChart3dArea().getAxis(JCAxis.AXIS_Y).getMax();
}//getYAxisMax

/**
 * Sets the Title text for the y axis.
 * @param yText the desired String value of the Title
 */
public void setYAxisTitleText(String yText) {
	getChart3dArea().getAxis(JCAxis.AXIS_Y).setTitle(yText);
}//setYTitle

/**
 * Retrieves the Title text for the y axis
 * @return String the text of the y axis title
 */
public String getYAxisTitleText() {
	if (getChart3dArea().getAxis(JCAxis.AXIS_Y).getTitle() == null) {
		return "";
	}

	return getChart3dArea().getAxis(JCAxis.AXIS_Y).getTitle();
}//getYTitle

/****************** Z-axis ******************/

/**
 * Sets whether the Z axis is showing.
 * @param showing true - showing
 */
public void setZAxisShowing(boolean showing) {
	getChart3dArea().getAxis(JCAxis.AXIS_Z).setShowing(showing);
}//setZAxisShowing

/**
 * Gets whether the Z axis is showing.
 * @return boolean true - showing
 */
public boolean isZAxisShowing() {
	return getChart3dArea().getAxis(JCAxis.AXIS_Z).isShowing();
}//isZAxisShowing

/**
 * Sets the annotation method for the Z Axis.
 * Valid values are ANNO_VALUES, ANNO_VALUE_LABELS
 * @param annoMethod int selection of annotation method
 */
public void setZAxisAnnotationMethod(int annoMethod){
	getChart3dArea().getAxis(JCAxis.AXIS_Z).setAnnotationMethod(annoMethod);
}//setZAxisAnnotationMethod

/**
 * Returns the annotation method for the Z Axis.
 * @return int the annotation method for the z axis.
 */
public int getZAxisAnnotationMethod() {
	return getChart3dArea().getAxis(JCAxis.AXIS_Z).getAnnotationMethod();
}//getZAxisAnnotationMethod

/**
 * Sets the minimum value of the z axis
 * @param min double minimum value of the z axis
 */
public void setZAxisMin(double min) {
	JCAxis axis = getChart3dArea().getAxis(JCAxis.AXIS_Z);
	//prevent exception from happening (reset max )
	if (axis.getMax() <= min) {
		axis.setMax(min + 1.0);
	}
	axis.setMin(min);

}//setZAxisMin

/**
 * Retrieves the minimum value of the z axis
 * @return double the minimum  value of the z axis
 */
public double getZAxisMin() {
	return getChart3dArea().getAxis(JCAxis.AXIS_Z).getMin();
}//getZAxisMin

/**
 * Sets the maximum value of the z axis
 * @param min double maximum value of the z axis
 */
public void setZAxisMax(double max) {
	JCAxis axis = getChart3dArea().getAxis(JCAxis.AXIS_Z);
	//prevent exception from happening (reset min )
	if (axis.getMin() >= max) {
		axis.setMin(max - 1.0);
	}
	axis.setMax(max);

}//setZAxisMax

/**
 * Retrieves the maximum value of the Z axis
 * @return double the maximum  value of the z axis
 */
public double getZAxisMax() {
	return getChart3dArea().getAxis(JCAxis.AXIS_Z).getMax();
}//getZAxisMax

/**
 * Sets the Title text for the z axis.
 * @param zText the desired String value of the Title
 */
public void setZAxisTitleText(String zText) {
	getChart3dArea().getAxis(JCAxis.AXIS_Z).setTitle(zText);
}//setZTitle

/**
 * Retrieves the Title text for the z axis
 * @return String the text of the z axis title
 */
public String getZAxisTitleText() {
	if (getChart3dArea().getAxis(JCAxis.AXIS_Z).getTitle() == null) {
		return "";
	}

	return getChart3dArea().getAxis(JCAxis.AXIS_Z).getTitle();
}//getZTitle


/* ********************************* Utility methods *********************/

/**
 * Creates a datasource from the Strings returned by the dataSourceEditor.
 *
 * @param   dataString   String
 * @return    Chart3dDataModel
 */
private Chart3dDataModel getDataSourceFromString(String dataString) {
	if (dataString.indexOf( "," ) < 0) {
		return JCFile3dDataSource.createDataSourceFromFile(dataString);
	}
	double[] xvals = null;
	double[] yvals = null;
	double[][] zvals = null;


	StringTokenizer st = new StringTokenizer(dataString, "\\n");
	int numTokens = st.countTokens();
	if(numTokens < 3) {
		return new JCDefault3dGridDataSource();
	}
	String[] allTokens = new String[numTokens];
	for(int i = 0; i < allTokens.length; i++) {
		allTokens[i] = st.nextToken();
	}
	st = null;
	if (allTokens[0].indexOf("null") < 0) {
		xvals = stringToDoubleArray(allTokens[0]);
	}
	if (allTokens[1].indexOf("null") < 0) {
		yvals = stringToDoubleArray(allTokens[1]);
	}

	if (allTokens[2].indexOf("null") < 0) {
		zvals = new double[numTokens-2][];

		for(int i = 0; i < zvals.length; i++) {
			zvals[i] = stringToDoubleArray(allTokens[i+2]);
		}
	}


	return new JCDefault3dGridDataSource(null, xvals, yvals, zvals);

}


/**
 * Converts a String of comma separated values into
 * an array of doubles
 *
 * @param   String of comma separated doubles
 * @return  an array of doubles
 */
private double[] stringToDoubleArray(String doubles) {
	StringTokenizer st = new StringTokenizer(doubles, ", ");
	int numTokens = st.countTokens();
	double[] tempvals = new double[numTokens];
	for(int i = 0; i < tempvals.length; i++) {
		try {
			tempvals[i] =  Double.parseDouble(st.nextToken().trim());
		}
		catch (NumberFormatException ex) {
			ex.printStackTrace();
		}
	}
	return tempvals;
}


/**
 * Takes a datamodel and turns it into Strings for the
 * DataSourceEditor
 *
 * @param   dm  Chart3dGridDataModel
 * @return String
 */
private String dataToString(Chart3dGridDataModel dm) {
	String datString = "";
	double[] xvals = dm.getXGrid();
	datString += doubleArrayToString(xvals);
	datString += "\\n";
	double[] yvals = dm.getYGrid();
	datString += doubleArrayToString(yvals);
	datString += "\\n";
	double[][] zvals = dm.getZValues();
	for (int i = 0; i < zvals.length-1; i++) {
		datString += doubleArrayToString(zvals[i]);
		datString += "\\n";
	}
	datString += doubleArrayToString(zvals[zvals.length-1]);
	return datString;
}


/**
 * Takes an array of doubles and turns it into
 * an acceptable String for the DataSourceEditor
 * Called by dataToString
 *
 * @param   vals
 * @return   String
 */
private String doubleArrayToString(double[] vals) {
	String doubleString = "";
	for (int i = 0; i < vals.length-1; i++) {
		doubleString += vals[i] + ", ";
	}
	doubleString += vals[vals.length-1] ;
	return doubleString;
}




}//Chart3dY
