/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: BaseShape.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:07 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.chart3d.shape;

import java.awt.Graphics;

/**
 * BaseShape is an abstract base class used for defining custom
 * point styles in Chart 3D.  To create a new shape class, extend
 * BaseShape and provide a resize() method.  The resize() method
 * must populate the x[] and y[] arrays with point values for
 * the shape to be drawn.  The point values are relative to
 * (0,0).  Chart will automatically move the shape to the
 * appropriate location.
 */
abstract public class BaseShape implements JCSymbolShape, java.io.Serializable {

/**
 * Size of the point.  For some symbols, used inside resize() to populate the
 * x[] and y[] arrays
 */
protected int size = 0;

/** Half size of the symbol.  Compute from size. */
int hsize;

/** Array of x values */
protected int x[];

/** Array of y values */
protected int y[];

/**
 * Return a localized name for this shape
 * @return The name of this shape
 */
abstract public String getName();

/**
 * Method used for setting the size of the shape.  For Triangle and Diamond this
 * is use for populating the x[] and y[] arrays based on the provided size.
 * @param sz value to be assigned to size member variable.
 */
public void resize(int size)
{
    this.size = size;
    boolean even = (size % 2 == 0);
    hsize = size / 2;
    if (even) {
        hsize = (size - 1)/2;
    }
}

/**
 * Draws the symbol at the specified coordinates using the x[] and y[] arrays.
 * @param gc graphics context to use for drawing
 * @param xorg x origin point
 * @param yorg y origin point
 */
public void draw(Graphics gc, int xorg, int yorg)
{
	if (x == null || y == null || x.length == 0 || y.length == 0) {
        return;
	}

	int localx[] = new int[x.length];
	int localy[] = new int[y.length];
	for (int i = 0; i < localx.length; i++) {
		localx[i] = x[i] + xorg;
		localy[i] = y[i] + yorg;
	}
	gc.fillPolygon(localx, localy, localx.length);
}

}
