/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Binding.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:08 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource;

import com.klg.jclass.util.JCListenerList;

import java.io.Serializable;
import java.util.Enumeration;

/**
 *
 * Extends ReadOnlyBinding and provides update methods. 
 * Operations can be performed on the row currently in 
 * focus (i.e. getCurrentRowStatus()) or by specifying 
 * a row index (i.e. getRowStatus(rowIndex)).
 *
 */
public class Binding extends ReadOnlyBinding {
	
static final long serialVersionUID = -1517190061360439012L;

/**
 *
 * The event handler instance.
 *
 */
protected EventListener eventListener = new EventListener();
	
/**
 *
 * Root-level constructor.
 * @param	dataModel the DataModel to which this binding 
 *			will belong.
 *
 */
public Binding(DataModel dataModel) {
	super(dataModel);
	this.dataModel.addDataModelListener(eventListener);
}
	
/** 
 * 
 * Constructor for child levels.
 * @param	parent the Binding which is the parent of this level.
 *
 */
public Binding(Binding parent) {
	this(parent.getDataModel());
}
 
/**
 * 
 * Return true if edits are permitted on this column.
 * @param	the column name.
 * @return	true if edits are permitted on this column, false otherwise.
 *
 */
public boolean isColumnEditable(String columnName) {
	try {
		ColumnModel cm = metaDataModel.getColumnObject(columnName);
		return cm != null && 
			!(cm instanceof VirtualColumnModel) && !cm.isReadOnly();
	} catch (DataModelException e) {
		//
	}
	return false;
}

/**
 * 
 * Return true if edits are permitted on this column.
 * @param 	the column index
 * @return	true if edits are permitted on this column, false otherwise.
 *
 */
public boolean isColumnEditable(int columnIndex) {
	return isColumnEditable(getColumnIdentifier(columnIndex));
}
		
/**
 *
 * Delete a range of rows from the table currently in focus.
 * @param	originator the DataModelListener making the delete request.
 * @param	rowIndex the row index at which the deletes will start.
 * @param	number the number of rows to delete
 *
 */
public void deleteRows(DataModelListener originator, int rowIndex, int number) throws DataModelException {
	DataTableModel dataTableModel = getCurrentDataTableModel(originator);

	java.util.Vector v = new java.util.Vector(number);
	for (int i = rowIndex; i < rowIndex + number; i++) {
		v.addElement(new Long(dataTableModel.getRowIdentifier(i)));
	}
	for (int i = 0; i < v.size(); i++) {
		dataTableModel.deleteRow(originator, ((Long)v.elementAt(i)).intValue());
	}
}

/**
 *
 * Delete the specified row.
 * @param	originator the DataModelListener making the delete request.
 * @param	rowIndex the row index at which the deletes will start.
 *
 */
public void deleteRow(DataModelListener originator, int rowIndex) throws DataModelException {
	DataTableModel dataTableModel = getCurrentDataTableModel(originator);
	dataTableModel.deleteRow(originator, dataTableModel.getRowIdentifier(rowIndex));
}

/**
 *
 * Delete the row currently in focus.
 * @param	originator the DataModelListener making the delete request.
 *
 */
public void deleteCurrentRow(DataModelListener originator) throws DataModelException {
	deleteRow(originator, getCurrentRowIndex());	
}
		
/**
 *
 * Add a row to the table currently in focus.
 * @param	originator the DataModelListener making the add request.
 *
 */
public void addRow(DataModelListener originator) throws DataModelException {
	DataTableModel dataTableModel =	getCurrentDataTableModel(originator);
	dataTableModel.addRow(originator);
}
	
/**
 *
 * Cancel edits to the specified row.
 * @param	originator the DataModelListener making the cancel request.
 * @param	rowIndex the row index to be cancelled.
 *
 */
public void cancelRowChanges(DataModelListener originator, int rowIndex) throws DataModelException {
	DataTableModel dataTableModel =	getCurrentDataTableModel(originator);
	dataTableModel.cancelRowChanges(originator, dataTableModel.getRowIdentifier(rowIndex));
}
	
/**
 *
 * Cancel edits to the row currently in focus.
 * @param	originator the DataModelListener making the cancel request.
 *
 */
public void cancelCurrentRowChanges(DataModelListener originator) throws DataModelException {
	cancelRowChanges(originator, getCurrentRowIndex());
}

/**
 *
 * Cancel edits in all levels.
 * @param	originator the DataModelListener making the cancel request.
 *
 */
public void cancelAll(DataModelListener originator) throws DataModelException {
	dataModel.cancelAll(originator);
}
	
/**
 *
 * Commit pending edits on the specified row making them permanent.
 * @param	originator the DataModelListener making the commit request.
 * @param	rowIndex the row index to be committed.
 *
 */
public void commitRow(DataModelListener originator, int rowIndex) throws DataModelException {
	DataTableModel dataTableModel =	getCurrentDataTableModel(originator);
	dataTableModel.commitRow(originator, dataTableModel.getRowIdentifier(rowIndex));
}

/**
 *
 * Commit pending edits on the row currently in focus making them permanent.
 * @param	originator the DataModelListener making the commit request.
 *
 */
public void commitCurrentRow(DataModelListener originator) throws DataModelException {
	commitRow(originator, getCurrentRowIndex());
}

/**
 *
 * Commit pending edits in all levels.
 * @param	originator the DataModelListener making the commit request.
 *
 */
public void commitAll(DataModelListener originator) throws DataModelException {
	dataModel.updateAll(originator);
}
	
/**
 *
 * Set the value in a cell on the specified row.
 * @param	originator the DataModelListener making the set request.
 * @param 	newValue the new Object for the cell.
 * @param	rowIndex the row index.
 * @param 	columnIndex the column index.
 *
 */
public void setCell(DataModelListener originator, int rowIndex, int columnIndex, Object newValue) throws DataModelException {
	String columnName	= metaDataModel.getColumnIdentifier(columnIndex);
	DataTableModel dtm	= getCurrentDataTableModel(originator);
	long bookmark 		= dtm.getRowIdentifier(rowIndex);
    Object oldValue 	= dtm.getResultData(bookmark, columnName);
	
    // return if the string values are the same. If the old_value was
    // null, an empty new string is equivalent. Tests:
    // old == new (true if both null) or
    // old == null && new == "" or
    // old != null && old.equals(new)
    if ((oldValue == newValue) ||
        (oldValue == null && (newValue instanceof String && ((String)newValue).length()==0)) ||
        (oldValue != null && oldValue.equals(newValue))) {
        return;
    }
    getCurrentDataTableModel(originator).updateCell(originator, bookmark, columnName, newValue);
}
	
/**
 *
 * Set the value in the cell on the row currently in focus. 
 * The row must be committed to make the change permanent, or
 * cancelled to restore the original value.
 * @param	originator DataModelListener the listener requesting the update.
 * @param	columnName the string name of the column to be updated.
 * @param 	newValue the Object which is the new value for the cell.
 *
 */
public void setCurrentCell(DataModelListener originator, String columnName, Object newValue) throws DataModelException {
	setCell(originator, getCurrentRowIndex(), metaDataModel.getColumnIndex(columnName), newValue); 
}
	
/**
 *
 * Return true if the specified row is modified.
 * @param	rowIndex the row index.
 * @return 	true if any cell on this row has pending edits, false otherwise.
 *
 */
public boolean isRowModified(int rowIndex) throws DataModelException {
	DataTableModel dataTableModel = getCurrentDataTableModel(eventListener);
	return dataTableModel.getRowStatus(dataTableModel.getRowIdentifier(rowIndex)) != DataTableModel.COMMITTED;
}

/**
 *
 * Return true if row currently in focus is modified.
 * @return 	true if any cell on this row has pending edits, false otherwise.
 *
 */
public boolean isCurrentRowModified() throws DataModelException {
	return isRowModified(getCurrentRowIndex());
}

/**
 *
 * Return the edit status of the specified row.
 * @param	rowIndex the row index.
 * @return 	one of DataTableModel.UPDATED, DataTableModel.INSERTED,
 *			DataTableModel.DELETED, DataTableModel.COMMITTED, 
 *			DataTableModel.NONEXISTENT.
 *
 */
public int getRowStatus(int rowIndex) throws DataModelException {
	DataTableModel dataTableModel = getCurrentDataTableModel(eventListener);
	return dataTableModel.getRowStatus(dataTableModel.getRowIdentifier(rowIndex));
}

/**
 *
 * Return the edit status of the row currently in focus.
 * @return 	one of DataTableModel.UPDATED, DataTableModel.INSERTED,
 *			DataTableModel.DELETED, DataTableModel.COMMITTED, 
 *			DataTableModel.NONEXISTENT.
 *
 */
public int getCurrentRowStatus() throws DataModelException {
	return getRowStatus(getCurrentRowIndex());
}

/**
 *
 * Requery all rows in the table currently in focus.
 * @param	originator the DataModelListener making the requery request.
 *
 */
public void requeryLevel(DataModelListener originator) throws DataModelException {
	DataTableModel dataTableModel = getCurrentDataTableModel(originator);
	if (dataTableModel == null)
        dataModel.requeryAll(originator);
    else {
        long bm = dataTableModel.getParentBookmark();
        if (bm == -1) {
            dataModel.requeryAll(originator);
        }
        else {
            MetaDataModel parentLevel = (MetaDataModel) metaDataModel.getParent();
            DataTableModel parentData = getCurrentDataTableModel(originator);
            parentData.requeryRowAndDetails(originator, bm);
        }
    }
}
	
public void moveToBestRowAfterDelete(DataModelListener originator, DataModelEvent event) throws DataModelException {
    int numRows = event.getTable().getRowCount();
    if (numRows > 0) {
        int rowIndex = event.getRowIndex();
        // if the deleted row is unknown, go to the first record
        // if the deleted row was the last row, go to the previous index
        // otherwise, go to the same index
        dataModel.moveToRow(originator, event.getTable(),
                            event.getTable().getRowIdentifier(rowIndex < 0 ? 0 :
                                                              (rowIndex == numRows ? --rowIndex : rowIndex) ));
    }
    else {
        // if the table has a parent, move to the parent bookmark
        DataTableModel parentTable = (DataTableModel) ((javax.swing.tree.TreeNode)event.getTable()).getParent();
        if (parentTable != null)
            dataModel.moveToRow(originator, parentTable, event.getTable().getParentBookmark());
    } 
}




protected class EventListener implements DataModelListener, Serializable {
	
    final long serialVersionUID = 7619120133353473409L;

	//
	// ReadOnlyListener method implementations.
	// These are passive methods which are invoked after
	// some other component has changed data.
	// See the implementation of the Read-write methods
	// lower down this file

	/**
	 * Listeners must close all expanded views and reset/reread the root node.
	 * The previous pointer to the root node is no longer valid.  Call
	 * DataModel.getDataTableTree().getRoot() for the new root table.
	 * Called when the datasource has been reset.
	 * @see DataModel#requeryAll
	 */
	public void afterReset(DataModelEvent e) {
		Enumeration enum = JCListenerList.elements(listeners);
		while (enum.hasMoreElements()) {
			DataModelListener l = (DataModelListener)enum.nextElement();
			l.initialize(e);
		}
	}
	
	/**
	 * The global cursor has moved to a new row.
	 * Listeners should position their cursor on
	 * the indicated row.  In a master-detail relationship child levels
	 * should refresh themselves to reflect data sets which correspond
	 * to the new parent row by calling DataModel.getCurrentDataTable()
	 * or for field controls, DataModel.getCurrentDataItem().
	 */
	public void afterMoveToCurrentRow(DataModelEvent e) {
		Enumeration enum = JCListenerList.elements(listeners);
		while (enum.hasMoreElements()) {
			DataModelListener l = (DataModelListener)enum.nextElement();
			l.afterMoveToCurrentRow(e);
		}
	}
	
	/**
	 * Re-reads the indicated row and refresh all open children
	 * under this row.
	 */
	public void afterRequeryRowAndDetails(DataModelEvent e) {
		if (isOutDataOrOurAncestors(e)) {
			Enumeration enum = JCListenerList.elements(listeners);
			while (enum.hasMoreElements()) {
				DataModelListener l = (DataModelListener)enum.nextElement();
				l.afterRequeryRowAndDetails(e);
			}
		}
	}
	
	/**
	 * Re-read this table and refresh all open children in
	 * this table.
	 */
	public void afterRequeryTable(DataModelEvent e) {
		if (isOutDataOrOurAncestors(e)) {
			Enumeration enum = JCListenerList.elements(listeners);
			while (enum.hasMoreElements()) {
				DataModelListener l = (DataModelListener)enum.nextElement();
				l.afterRequeryTable(e);
			}
		}
	}
	
	/**
	 * A row has changed, re-read all its cells and its status to reflect the new
	 * values.  If this event is the result of a cell edit call DataModelEvent.getColumn()
	 * to get the name of the column which changed. If getColumn() returns "", re-read
	 * the entire row.
	 * Called when
	 * <ul>
	 * <li>a row is deleted and getShowDeletedRows() == true</li>
	 * <li>a cell was edited</li>
	 * <li>row edits are cancelled and getRowStatus() == UPDATED</li>
	 * <li>row edits are cancelled and getRowStatus() == DELETED and getShowDeletedRows() == true</li>
	 * <li>row is commited and getRowStatus() == UPDATED or INSERTED</li>
	 * <li>row is requeried and getRowStatus() != INSERTED</li>
	 * </ul>
	 */
	public void afterChangeOfRowData(DataModelEvent e) {
		if (isOurData(e)) {
			Enumeration enum = JCListenerList.elements(listeners);
			while (enum.hasMoreElements()) {
				DataModelListener l = (DataModelListener)enum.nextElement();
				l.afterChangeOfRowData(e);
			}
		}
	}
	
	/**
	 * A new row has been added to the datasource. Listeners need to
	 * display the row.  Rows are always added to the end of DataTableModels.
	 */
	public void afterInsertRow(DataModelEvent e) {
		if (isOurData(e)) {
			Enumeration enum = JCListenerList.elements(listeners);
			while (enum.hasMoreElements()) {
				DataModelListener l = (DataModelListener)enum.nextElement();
				l.afterInsertRow(e);
			}
		}
	}
	
	/**
	 * Removes the row from the display.
	 * A row has been physically deleted and needs to be
	 * removed from the display or has been logically deleted but the
	 * showDeletedRows property has been set to false. Called when,
	 * <ul>
	 * <li>a row has been logically deleted and getShowDeletedRows() == false</li>
	 * <li>row changes have been cancelled and getRowStatus() == INSERTED</li>'
	 * <li>a row is committed and getRowStatus() == DELETED and getShowDeletedRows() == true</li>
	 * <li>a row has been requeried and getRowStatus() == INSERTED</li>
	 * </ul>
	 */
	public void afterDeleteRow(DataModelEvent e) {
		if (isOurData(e)) {
			Enumeration enum = JCListenerList.elements(listeners);
			while (enum.hasMoreElements()) {
				DataModelListener l = (DataModelListener)enum.nextElement();
				l.afterDeleteRow(e);
			}
		}
	
	}
	
	/**
	 * Notification that multiple events are coming.
	 * Multiple events will be nested
	 * between beginEvents() and endEvents().
	 * Allows listeners to treat the events as a batch, to,
	 * for example, reduce repaints.
	 */
	public void beginEvents(DataModelEvent e) {
		// DataBinding does nothing here	
	}
	
	/**
	 * Notification that multiple events are complete.
	 * Multiple events will be nested
	 * between beginEvents() and endEvents().
	 * Allows listeners to treat the events as a batch, to,
	 * for example, reduce repaints. Called when
	 * <ul>
	 * <li>DataModel.updateAll is called</li>
	 * <li>DataModel.cancelAll is called</li>
	 * </ul>
	 */
	public void endEvents(DataModelEvent e) {
		if (isOurData(e)) {
			Enumeration enum = JCListenerList.elements(listeners);
			while (enum.hasMoreElements()) {
				DataModelListener l = (DataModelListener)enum.nextElement();
				l.endEvents(e);
			}
		}
	}
	
	/**
	 * For internal use only
	 * Used by the JCData bean.
	 */
	public void initialize(DataModelEvent e) {
		if (isOurData(e)) {
			Enumeration enum = JCListenerList.elements(listeners);
			while (enum.hasMoreElements()) {
				DataModelListener l = (DataModelListener)enum.nextElement();
				l.initialize(e);
			}
		}
	}


	//
	// Read-write implementation of the DataModelListener interface.
	// These methods are invoked before data is changed. This allows
	// read-write componets the opportunity to cancel the proposed
	// data change if it will interfere with their pending changes. 
	//
	
	/**
	 * Called before a row is inserted. Can be cancelled.
	 * If not cancelled will be followed by afterInsertRow().
	 * @see DataTableModel#addRow
	 */
	public void beforeInsertRow(DataModelEvent e) {
		if (isOurData(e)) {
			Enumeration enum = JCListenerList.elements(listeners);
			while (enum.hasMoreElements()) {
				DataModelListener l = (DataModelListener)enum.nextElement();
				l.beforeInsertRow(e);
			}
		}
	}
	
	/**
	 * Called before entire grid is reset. Can be cancelled.
	 * If not cancelled will be followed by afterReset().
	 * This will result from a call to DataModel.requeryAll().
	 * @see DataModel#requeryAll
	 */
	public void beforeReset(DataModelEvent e) {
		if (isOurData(e)) {
			Enumeration enum = JCListenerList.elements(listeners);
			while (enum.hasMoreElements()) {
				DataModelListener l = (DataModelListener)enum.nextElement();
				l.beforeReset(e);
			}
		}
	}
	
	/**
	 * Called when either DataTableModel.requeryRowAndDetails() or
	 * DataTableModel.requeryRow() is called.
	 * If not cancelled will be followed by an,
	 * <ul>
	 * <li>afterRequeryRowAndDetails() call or an</li>
	 * <li>afterDeleteRow() call in the case getRowStatus() == INSERTED or a</li>
	 * <li>afterChangeOfRowData() call in the case getRowStatus() == UPDATED or COMMITTED.</li>
	 * </ul>
	 * @see DataTableModel#requeryRow
	 * @see DataTableModel#requeryRowAndDetails
	 */
	public void beforeRequery(DataModelEvent e) {
		if (isOurData(e)) {
			Enumeration enum = JCListenerList.elements(listeners);
			while (enum.hasMoreElements()) {
				DataModelListener l = (DataModelListener)enum.nextElement();
				l.beforeRequery(e);
			}
		}
	}
	
	/**
	 * Called before a cell is edited. Can be cancelled.
	 * @see DataTableModel#updateCell
	 */
	public void beforeEditCell(DataModelEvent e) {
		if (isOurData(e)) {
			Enumeration enum = JCListenerList.elements(listeners);
			while (enum.hasMoreElements()) {
				DataModelListener l = (DataModelListener)enum.nextElement();
				l.beforeEditCell(e);
			}
		}
	}
	
	/**
	 * Called before a row is [logically] deleted. Can be cancelled.
	 * If not cancelled will be followed by an
	 * afterDeleteRow90 or a afterChangeOfRowData() if the commit policy
	 * is COMMIT_MANUALLY or COMMIT_LEAVING_ANCESTOR.
	 * @see DataTableModel#deleteRow
	 * @see MetaDataModel#getCommitPolicy
	 */
	public void beforeDeleteRow(DataModelEvent e) {
		if (isOurData(e)) {
			Enumeration enum = JCListenerList.elements(listeners);
			while (enum.hasMoreElements()) {
				DataModelListener l = (DataModelListener)enum.nextElement();
				l.beforeDeleteRow(e);
			}
		}
	}
	
	/**
	 * Called before all changes are committed. Can be cancelled.
	 * All modified, deleted and inserted rows at all levels are about
	 * to be committed.  beforeDeleteRow() and afterChangeOfRowData() 
	 * will follow depending on the operations performed on the modified rows
	 * being saved.
	 * Results from a call to DataModel.updateAll.
	 * @see DataModel#updateAll
	 */
	public void beforeCommitAll(DataModelEvent e) {
		Enumeration enum = JCListenerList.elements(listeners);
		while (enum.hasMoreElements()) {
			DataModelListener l = (DataModelListener)enum.nextElement();
			l.beforeCommitAll(e);
		}
	}
	
	/**
	 * Called before all changes are cancelled. Can be cancelled.
	 * afterInsertRow() and afterChangeOfRowData() can follow this
	 * event.
	 * @see DataModel#cancelAll
	 */
	public void beforeCancelAll(DataModelEvent e) {
		Enumeration enum = JCListenerList.elements(listeners);
		while (enum.hasMoreElements()) {
			DataModelListener l = (DataModelListener)enum.nextElement();
			l.beforeCancelAll(e);
		}
	}
	
	/** Called before all edits to a row are undone. Can be cancelled
	 * An afterDeleteRow() or afterChangeOfRowData() will follow.
	 * @see DataTableModel#cancelRowChanges
	 */
	public void beforeCancelRowChanges(DataModelEvent e) {
		if (isOurData(e)) {
			Enumeration enum = JCListenerList.elements(listeners);
			while (enum.hasMoreElements()) {
				DataModelListener l = (DataModelListener)enum.nextElement();
				l.beforeCancelRowChanges(e);
			}
		}
	}
	
	/**
	 * Called when the root-level bookmark for a subtree changes.
	 * When this happens those nodes in the previous subtree which
	 * are not COMMIT_MANUALLY are committed.
	 * Can be cancelled.  If cancelled the cursor moves but the
	 * changes are automatically committed.
	 */
	public void beforeCommitConditional(DataModelEvent e) {
		if (isOurData(e)) {
			Enumeration enum = JCListenerList.elements(listeners);
			while (enum.hasMoreElements()) {
				DataModelListener l = (DataModelListener)enum.nextElement();
				l.beforeCommitConditional(e);
			}
		}
	}
	
	/**
	 * Called before single row is committed to data source.  Can be cancelled in
	 * which case the row edits are not written to the datasource and the rows
	 * status remains modified.  afterDeleteRow() or afterChangeOfRowData() will
	 * follow depending on the status of the row to be committed.
	 * @see DataTableModel#commitRow
	 */
	public void beforeCommitRow(DataModelEvent e) {
		Enumeration enum = JCListenerList.elements(listeners);
		while (enum.hasMoreElements()) {
			DataModelListener l = (DataModelListener)enum.nextElement();
			l.beforeCommitRow(e);
		}
	}
	
	/**
	 * The global cursor will move to a new row.
	 * Can be cancelled.
	 */
	public void beforeMoveToCurrentRow(DataModelEvent e) {
		Enumeration enum = JCListenerList.elements(listeners);
		while (enum.hasMoreElements()) {
			DataModelListener l = (DataModelListener)enum.nextElement();
			l.beforeMoveToCurrentRow(e);
		}
	}
	
	/**
	 * The indicated DataTable will be deleted and flushed from the cache.
	 * Can be cancelled.
	 */
	public void beforeDeleteTable(DataModelEvent e) {
		DataTableModel dataTableModel = null;
		try {
			dataTableModel = getCurrentDataTableModel(this);
		}
		catch (DataModelException dme) {}
		
		if (dataTableModel != null && dataTableModel.tableIsAncestor(e.getTable())) {
	        if (e.isCancelable()) {
	            e.cancelProposedAction();
	        	return;
	        }
	    }
	
		Enumeration enum = JCListenerList.elements(listeners);
		while (enum.hasMoreElements()) {
			DataModelListener l = (DataModelListener)enum.nextElement();
			l.beforeDeleteTable(e);
		}
	}
	
	/**
	 * The current row has been deleted and the originator
	 * of the deletion should now reposition the global
	 * cursor to a new, valid row.
	 */
	public void originatorNavigateRow(DataModelEvent e) {
		if (isOurData(e)) {
			Enumeration enum = JCListenerList.elements(listeners);
			while (enum.hasMoreElements()) {
				DataModelListener l = (DataModelListener)enum.nextElement();
				l.originatorNavigateRow(e);
			}
		}
	}

} // end inner listener class	
}
