/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: BindingModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:08 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource;

import com.klg.jclass.util.JCListenerList;

import java.util.Enumeration;

/**
 *
 * Extends ReadOnlyBindingModel and provides update methods. 
 * Operations can be performed on the row currently in 
 * focus (i.e. getCurrentRowStatus()) or by specifying 
 * a row index (i.e. getRowStatus(rowIndex)).
 * @see ReadOnlyBindingModel
 * @see ReadOnlyBinding
 * @see Binding
 *
 */
public interface BindingModel extends ReadOnlyBindingModel {
 
/**
 * 
 * Return true if edits are permitted on this column.
 * @param	the column name.
 * @return	true if edits are permitted on this column, false otherwise.
 *
 */
public boolean isColumnEditable(String columnName);
	
/**
 * 
 * Return true if edits are permitted on this column.
 * @param 	the column index
 * @return	true if edits are permitted on this column, false otherwise.
 *
 */
public boolean isColumnEditable(int columnIndex);
	
/**
 *
 * Delete a range of rows from the table currently in focus.
 * @param	originator the DataModelListener making the delete request.
 * @param	rowIndex the row index at which the deletes will start.
 * @param	number the number of rows to delete
 *
 */
public void deleteRows(DataModelListener originator, int rowIndex, int number) throws DataModelException;
	
/**
 *
 * Delete the specified row.
 * @param	originator the DataModelListener making the delete request.
 * @param	rowIndex the row index at which the deletes will start.
 *
 */
public void deleteRow(DataModelListener originator, int rowIndex) throws DataModelException;
	
/**
 *
 * Delete the row currently in focus.
 * @param	originator the DataModelListener making the delete request.
 *
 */
public void deleteCurrentRow(DataModelListener originator) throws DataModelException;
	
/**
 *
 * Add a row to the table currently in focus.
 * @param	originator the DataModelListener making the add request.
 *
 */
public void addRow(DataModelListener originator) throws DataModelException;
	
/**
 *
 * Cancel edits to the specified row.
 * @param	originator the DataModelListener making the cancel request.
 * @param	rowIndex the row index to be cancelled.
 *
 */
public void cancelRowChanges(DataModelListener originator, int rowIndex) throws DataModelException;
	
/**
 *
 * Cancel edits to the row currently in focus.
 * @param	originator the DataModelListener making the cancel request.
 *
 */
public void cancelCurrentRowChanges(DataModelListener originator) throws DataModelException;
	
/**
 *
 * Cancel edits in all levels.
 * @param	originator the DataModelListener making the cancel request.
 *
 */
public void cancelAll(DataModelListener originator) throws DataModelException;
	
/**
 *
 * Commit pending edits on the specified row making them permanent.
 * @param	originator the DataModelListener making the commit request.
 * @param	rowIndex the row index to be committed.
 *
 */
public void commitRow(DataModelListener originator, int rowIndex) throws DataModelException;
	
/**
 *
 * Commit pending edits on the row currently in focus making them permanent.
 * @param	originator the DataModelListener making the commit request.
 *
 */
public void commitCurrentRow(DataModelListener originator) throws DataModelException;
	
/**
 *
 * Commit pending edits in all levels.
 * @param	originator the DataModelListener making the commit request.
 *
 */
public void commitAll(DataModelListener originator) throws DataModelException;
	
/**
 *
 * Set the value in a cell on the specified row.
 * @param	originator the DataModelListener making the set request.
 * @param 	newValue the new Object for the cell.
 * @param	rowIndex the row index.
 * @param 	columnIndex the column index.
 *
 */
public void setCell(DataModelListener originator, int rowIndex, int columnIndex, Object newValue) throws DataModelException;
	
/**
 *
 * Set the value in the cell on the row currently in focus. 
 * The row must be committed to make the change permanent, or
 * cancelled to restore the original value.
 * @param	originator DataModelListener the listener requesting the update.
 * @param	columnName the string name of the column to be updated.
 * @param 	newValue the Object which is the new value for the cell.
 *
 */
public void setCurrentCell(DataModelListener originator, String columnName, Object newValue) throws DataModelException;
	
/**
 *
 * Return true if the specified row is modified.
 * @param	rowIndex the row index.
 * @return 	true if any cell on this row has pending edits, false otherwise.
 *
 */
public boolean isRowModified(int rowIndex) throws DataModelException;

/**
 *
 * Return true if row currently in focus is modified.
 * @return 	true if any cell on this row has pending edits, false otherwise.
 *
 */
public boolean isCurrentRowModified() throws DataModelException;
	
/**
 *
 * Return the edit status of the specified row.
 * @param	rowIndex the row index.
 * @return 	one of DataTableModel.UPDATED, DataTableModel.INSERTED,
 *			DataTableModel.DELETED, DataTableModel.COMMITTED, 
 *			DataTableModel.NONEXISTENT.
 *
 */
public int getRowStatus(int rowIndex) throws DataModelException;
	
/**
 *
 * Return the edit status of the row currently in focus.
 * @return 	one of DataTableModel.UPDATED, DataTableModel.INSERTED,
 *			DataTableModel.DELETED, DataTableModel.COMMITTED, 
 *			DataTableModel.NONEXISTENT.
 *
 */
public int getCurrentRowStatus() throws DataModelException;
	
/**
 *
 * Requery all rows in the table currently in focus.
 * @param	originator the DataModelListener making the requery request.
 *
 */
public void requeryLevel(DataModelListener originator) throws DataModelException;
	
public void moveToBestRowAfterDelete(DataModelListener originator, DataModelEvent event) throws DataModelException;
	
	
}
