/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ColumnModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:08 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource;

/**
 * Interface for information about columns.
 *
 * ResultSetMetaData methods are mirrored here, see their
 * documentation for descriptions.
 */

public interface ColumnModel {

public String getCatalogName();

public int getColumnDisplaySize();

public String getColumnLabel();

public String getColumnName();

/**
 * Return one of MetaDataModel types.
 * Maps java.sql.Types types to object types used in this model.
 * @return this column's type as one of the MetaDataModel type constants.
 * @see #getColumnType
 */
public int getMetaColumnType();

/**
 * Is this column's type date/time/timestamp?
 * @return true if this column's type is date/time/timestamp, false otherwise
 */
public boolean isTimeDate();

/**
 * Return the jdbc.Sql.Types type for the column.
 * In the model this type is not used, it is mapped to
 * a MetaDataModel type constant.
 * @return a java.sql.Types type
 * @see #getMetaColumnType
 */
public int getColumnType();

public String getColumnTypeName();

public int getPrecision();

public int getScale();

public String getSchemaName();

public String getTableName();

public boolean isAutoIncrement();

public boolean isCaseSensitive();

public boolean isCurrency();

public boolean isDefinitelyWritable();

public int isNullable();

public boolean isReadOnly();

public boolean isSearchable();

public boolean isSigned();

public boolean isWritable();

public String getColumnClassName();

public String getIdentifier();

/**
 * Set the metaType for this column.
 * Maps the passed in java.sql.Types type to
 * a MetaDataModel type.
 * @param metaType a int which is the MetaDataModel type for the column
 * @see com.klg.jclass.datasource.MetaDataModel
 */
public void setMetaColumnType(int metaType);

/**
 * Set the metaType for this column.
 * Maps the passed in java.sql.Types type to
 * a MetaDataModel type.
 * @param sqlType a int which is the java.sql.Types type for the column
 */
public void setMetaColumnTypeFromSqlType(int type);

/* setters */
public void setCatalogName(String catalog);
public void setColumnDisplaySize(int displayWidth);
public void setColumnLabel(String label);
public void setColumnName(String name);
public void setColumnType(int type);
public void setColumnTypeName(String typeName);
public void setPrecision(int precision);
public void setScale(int scale);
public void setSchemaName(String schema);
public void setTableName(String table);
public void setAutoIncrement(boolean isAutoIncrement);
public void setCaseSensitive(boolean isCaseSensitive);
public void setCurrency(boolean isCurrency);
public void setDefinitelyWritable(boolean isDefinitelyWritable);
public void setNullable(int isNullable);
public void setReadOnly(boolean isReadOnly);
public void setSearchable(boolean isSearchable);
public void setSigned(boolean isSigned);
public void setWritable(boolean isWritable);
public void setColumnClassName(String columnClassName);
public void setIdentifier(String identifier);

}
