/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataModelAdapter.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:08 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource;

import java.io.Serializable;

/**
 * DataModelListener adapter for DataModelEvent.
 * @see DataModelEvent
 */
public class DataModelAdapter implements DataModelListener, Serializable {

static final long serialVersionUID = 3954480812108626L;

/**
 * Called before a row is inserted. Can be cancelled.
 * If not cancelled will be followed by afterInsertRow().
 * @see DataTableModel#addRow
 */
public void beforeInsertRow(DataModelEvent e) {
}

/**
 * Called before entire grid is reset. Can be cancelled.
 * If not cancelled will be followed by afterReset().
 * This will result from a call to DataModel.requeryAll().
 * @see DataModel#requeryAll
 */
public void beforeReset(DataModelEvent e) {
}

/**
 * Called when either DataTableModel.requeryRowAndDetails() or
 * DataTableModel.requeryRow() is called.
 * If not cancelled will be followed by an,
 * <ul>
 * <li>afterRequeryRowAndDetails() call or an</li>
 * <li>afterDeleteRow() call in the case getRowStatus() == INSERTED or a</li>
 * <li>afterChangeOfRowData() call in the case getRowStatus() == UPDATED or COMMITTED.</li>
 * </ul>
 * @see DataTableModel#requeryRow
 * @see DataTableModel#requeryRowAndDetails
 */
public void beforeRequery(DataModelEvent e) {
}

/**
 * Called before a cell is edited. Can be cancelled.
 * @see DataTableModel#updateCell
 */
public void beforeEditCell(DataModelEvent e) {
}

/**
 * Called before a row is [logically] deleted. Can be cancelled.
 * If not cancelled will be followed by an
 * afterDeleteRow90 or a afterChangeOfRowData() if the commit policy
 * is COMMIT_MANUALLY or COMMIT_LEAVING_ANCESTOR.
 * @see DataTableModel#deleteRow
 * @see MetaDataModel#getCommitPolicy
 */
public void beforeDeleteRow(DataModelEvent e) {
}

/**
 * Called before all changes are committed. Can be cancelled.
 * All modified, deleted and inserted rows at all levels are about
 * to be committed.  beforeDeleteRow() and afterChangeOfRowData() 
 * will follow depending on the operations performed on the modified rows
 * being saved.
 * Results from a call to DataModel.updateAll.
 * @see DataModel#updateAll
 */
public void beforeCommitAll(DataModelEvent e) {
}

/**
 * Called before all changes are cancelled. Can be cancelled.
 * afterInsertRow() and afterChangeOfRowData() can follow this
 * event.
 * @see DataModel#cancelAll
 */
public void beforeCancelAll(DataModelEvent e) {
}

/** Called before all edits to a row are undone. Can be cancelled
 * An afterDeleteRow() or afterChangeOfRowData() will follow.
 * @see DataTableModel#cancelRowChanges
 */
public void beforeCancelRowChanges(DataModelEvent e) {
}

/**
 * Called when the root-level bookmark for a subtree changes.
 * When this happens those nodes in the previous subtree which
 * are not COMMIT_MANUALLY are committed.
 * Can be cancelled.  If cancelled the cursor moves but the
 * changes are automatically committed.
 */
public void beforeCommitConditional(DataModelEvent e) {
}

/**
 * Called before single row is committed to data source.  Can be cancelled in
 * which case the row edits are not written to the datasource and the rows
 * status remains modified.  afterDeleteRow() or afterChangeOfRowData() will
 * follow depending on the status of the row to be committed.
 * @see DataTableModel#commitRow
 */
public void beforeCommitRow(DataModelEvent e) {
}

/**
 * The global cursor will move to a new row.
 * Can be cancelled.
 */
public void beforeMoveToCurrentRow(DataModelEvent e) {
}

/**
 * The indicated DataTable will be deleted and flushed from the cache.
 * Can be cancelled.
 */
public void beforeDeleteTable(DataModelEvent e) {
}

/**
 * A row has changed, re-read all its cells and its status to reflect the new
 * values.  If this event is the result of a cell edit call DataModelEvent.getColumn()
 * to get the name of the column which changed. If getColumn() returns "", re-read
 * the entire row.
 * Called when
 * <ul>
 * <li>a row is deleted and getShowDeletedRows() == true</li>
 * <li>a cell was edited</li>
 * <li>row edits are cancelled and getRowStatus() == UPDATED</li>
 * <li>row edits are cancelled and getRowStatus() == DELETED and getShowDeletedRows() == true</li>
 * <li>row is commited and getRowStatus() == UPDATED or INSERTED</li>
 * <li>row is requeried and getRowStatus() != INSERTED</li>
 * </ul>
 */
public void afterChangeOfRowData(DataModelEvent e) {
}

/**
 * A new row has been added to the datasource. Listeners need to
 * display the row.  Rows are always added to the end of DataTableModels.
 */
public void afterInsertRow(DataModelEvent e) {
}

/**
 * Removes the row from the display.
 * A row has been physically deleted and needs to be
 * removed from the display or has been logically deleted but the
 * showDeletedRows property has been set to false. Called when,
 * <ul>
 * <li>a row has been logically deleted and getShowDeletedRows() == false</li>
 * <li>row changes have been cancelled and getRowStatus() == INSERTED</li>'
 * <li>a row is committed and getRowStatus() == DELETED and getShowDeletedRows() == true</li>
 * <li>a row has been requeried and getRowStatus() == INSERTED</li>
 * </ul>
 */
public void afterDeleteRow(DataModelEvent e) {
}

/**
 * Listeners must close all expanded views and reset/reread the root node.
 * The previous pointer to the root node is no longer valid.  Call
 * DataModel.getDataTableTree().getRoot() for the new root table.
 * Called when the datasource has been reset.
 * @see DataModel#requeryAll
 */
public void afterReset(DataModelEvent e) {
}

/**
 * The global cursor has moved to a new row.
 * Listeners should position their cursor on
 * the indicated row.  In a master-detail relationship child levels
 * should refresh themselves to reflect data sets which correspond
 * to the new parent row by calling DataModel.getCurrentDataTable()
 * or for field controls, DataModel.getCurrentDataItem().
 */
public void afterMoveToCurrentRow(DataModelEvent e) {
}

/**
 * Re-reads the indicated row and refresh all open children
 * under this row.
 */
public void afterRequeryRowAndDetails(DataModelEvent e) {
}

/**
 * Re-read this table and refresh all open children in
 * this table.
 */
public void afterRequeryTable(DataModelEvent e) {
}

/**
 * Notification that multiple events are coming.
 * Multiple events will be nested
 * between beginEvents() and endEvents().
 * Allows listeners to treat the events as a batch, to,
 * for example, reduce repaints.
 */
public void beginEvents(DataModelEvent e) {
}

/**
 * Notification that multiple events are complete.
 * Multiple events will be nested
 * between beginEvents() and endEvents().
 * Allows listeners to treat the events as a batch, to,
 * for example, reduce repaints. Called when
 * <ul>
 * <li>DataModel.updateAll is called</li>
 * <li>DataModel.cancelAll is called</li>
 * </ul>
 */
public void endEvents(DataModelEvent e) {
}

/**
 * The current row has been deleted and the originator
 * of the deletion should now reposition the global
 * cursor to a new, valid row.
 */
public void originatorNavigateRow(DataModelEvent e) {
}

/**
 * For internal use only
 * Used by the JCData bean.
 */
public void initialize(DataModelEvent e) {
}

}
