/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataModelException.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:08 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource;

/**
 * Any exceptions from this data model will be wrapped in this
 * DataModelException.  It adds information about the context
 * of the exception (for example, bookmark, columnId).
 *
 * The original exception is wrapped by this class and can be
 * gotten by calling the getWrappedException method.  You can
 * then call original-exception-specific methods on the wrapped
 * exception.  For example, if it were an SQLException you could
 * call DataModelException.getWrappedException().getSQLState();
 */

public class DataModelException extends Exception {

static final long serialVersionUID = 3986904687356485059L;

protected long bookmark;                 // error row
protected String columnID;               // error column
protected DataTableModel dataTableModel; // error data table
protected int action;                    // the action which caused the Exception
protected Exception wrappedException;    // exception being wrapped

/**
 * Create a DataModelException.
 * @param bookmark a long uniquely identifying a row
 * @param columnId a String uniquely identifying a column
 * @param dataTableModel the DataTableModel related to the Exception
 * @param action an int, one of DataModelEvent actions.
 * @param wrappedException an Exception
 * @see DataModelEvent
 */
public DataModelException(long bookmark, String columnID,
                          DataTableModel dataTableModel, int action, Exception wrappedException) {
    super(wrappedException.toString());
    this.bookmark = bookmark;
    this.columnID = columnID;
    this.dataTableModel = dataTableModel;
    this.action = action;
    this.wrappedException = wrappedException;
}

/**
 * Create a DataModelException.
 * @param action an int, one of DataModelEvent actions.
 * @param wrappedException an Exception
 * @see DataModelEvent
 */
public DataModelException(int action, Exception wrappedException) {
    this(-1, "", null, action, wrappedException);
}

/**
 * Return the bookmark for the offending row.
 * @return the bookmark for the offending row, or -1 if not applicable.
 */
public long getBookmark() {
    return bookmark;
}

/**
 * Return the columnId for the offending column.
 * @return the columnId for the offending column, or "" if not applicable.
 */
public String getColumnID() {
    return columnID;
}

/**
 * Return the DataTableModel related to the Exception.
 * @return the DataTableModel in which the Exception occurred, or null
 * if not applicable.
 */
public DataTableModel getDataTableModel() {
    return dataTableModel;
}

/**
 * Return the action which caused the Exception.
 * The action will be one of the DataModelEvents.
 * @return an int which is one of DataModelEvents.
 * @see DataModelEvent
 */
public int getAction() {
    return action;
}

/**
 * Return the Exception that wrapped by this DataModelException.
 * @return the wrapped Exception.
 */
public Exception getWrappedException() {
    return wrappedException;
}
}
