/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataModelListener.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:08 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource;

/**
 * Objects interested in receiving DataModelEvents must implement this
 * interface.  These methods will be called by the data source when 
 * its data has changed.
 * 
 * @see DataModelEvent
 * @see DataTableModel
 */
public interface DataModelListener extends ReadOnlyBindingListener {

/**
 * Called before a row is inserted. Can be cancelled.
 * If not cancelled will be followed by afterInsertRow().
 * @see DataTableModel#addRow
 */
public void beforeInsertRow(DataModelEvent e);

/**
 * Called before entire grid is reset. Can be cancelled.
 * If not cancelled will be followed by afterReset().
 * This will result from a call to DataModel.requeryAll().
 * @see DataModel#requeryAll
 */
public void beforeReset(DataModelEvent e);

/**
 * Called when either DataTableModel.requeryRowAndDetails() or
 * DataTableModel.requeryRow() is called.
 * If not cancelled will be followed by an,
 * <ul>
 * <li>afterRequeryRowAndDetails() call or an</li>
 * <li>afterDeleteRow() call in the case getRowStatus() == INSERTED or a</li>
 * <li>afterChangeOfRowData() call in the case getRowStatus() == UPDATED or COMMITTED.</li>
 * </ul>
 * @see DataTableModel#requeryRow
 * @see DataTableModel#requeryRowAndDetails
 */
public void beforeRequery(DataModelEvent e);

/**
 * Called before a cell is edited. Can be cancelled.
 * @see DataTableModel#updateCell
 */
public void beforeEditCell(DataModelEvent e);

/**
 * Called before a row is [logically] deleted. Can be cancelled.
 * If not cancelled will be followed by an
 * afterDeleteRow90 or a afterChangeOfRowData() if the commit policy
 * is COMMIT_MANUALLY or COMMIT_LEAVING_ANCESTOR.
 * @see DataTableModel#deleteRow
 * @see MetaDataModel#getCommitPolicy
 */
public void beforeDeleteRow(DataModelEvent e);

/**
 * Called before all changes are committed. Can be cancelled.
 * All modified, deleted and inserted rows at all levels are about
 * to be committed.  beforeDeleteRow() and afterChangeOfRowData() 
 * will follow depending on the operations performed on the modified rows
 * being saved.
 * Results from a call to DataModel.updateAll.
 * @see DataModel#updateAll
 */
public void beforeCommitAll(DataModelEvent e);

/**
 * Called before all changes are cancelled. Can be cancelled.
 * afterInsertRow() and afterChangeOfRowData() can follow this
 * event.
 * @see DataModel#cancelAll
 */
public void beforeCancelAll(DataModelEvent e);

/** Called before all edits to a row are undone. Can be cancelled
 * An afterDeleteRow() or afterChangeOfRowData() will follow.
 * @see DataTableModel#cancelRowChanges
 */
public void beforeCancelRowChanges(DataModelEvent e);

/**
 * Called when the root-level bookmark for a subtree changes.
 * When this happens those nodes in the previous subtree which
 * are not COMMIT_MANUALLY are committed.
 * Can be cancelled.  If cancelled the cursor moves but the
 * changes are automatically committed.
 */
public void beforeCommitConditional(DataModelEvent e);

/**
 * Called before single row is committed to data source.  Can be cancelled in
 * which case the row edits are not written to the datasource and the rows
 * status remains modified.  afterDeleteRow() or afterChangeOfRowData() will
 * follow depending on the status of the row to be committed.
 * @see DataTableModel#commitRow
 */
public void beforeCommitRow(DataModelEvent e);

/**
 * The global cursor will move to a new row.
 * Can be cancelled.
 */
public void beforeMoveToCurrentRow(DataModelEvent e);

/**
 * The indicated DataTable will be deleted and flushed from the cache.
 * Can be cancelled.
 */
public void beforeDeleteTable(DataModelEvent e);

/**
 * The current row has been deleted and the originator
 * of the deletion should now reposition the global
 * cursor to a new, valid row.
 */
public void originatorNavigateRow(DataModelEvent e);

}
