/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataSourceTreeNode.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:09 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource;

import java.io.*;
import java.util.*;

import javax.swing.tree.*;

import com.klg.jclass.datasource.*;

/** 
 * This class extends the swing DefaultMutableTreeNode 
 * to provide methods required by JClass DataSource.
 */
public class DataSourceTreeNode extends DefaultMutableTreeNode implements
    Serializable {

static final long serialVersionUID = 7074887604955597029L;

/**
 * Creates a tree node that has no parent and no children, but which
 * allows children.
 */
public DataSourceTreeNode() {
    super();
}

/**
 * Creates a tree node with no parent, no children, but which allows 
 * children, and initializes it with the specified user object.
 * 
 * @param userObject an Object provided by the user that constitutes
 *                   the node's data
 */
public DataSourceTreeNode(Object userObject) {
    super(userObject);
}

/**
 * Creates a tree node with no parent, no children, initialized with
 * the specified user object, and that allows children only if
 * specified.
 * 
 * @param userObject an Object provided by the user that constitutes
 *        the node's data
 * @param allowsChildren if true, the node is allowed to have child
 *        nodes -- otherwise, it is always a leaf node
 */
public DataSourceTreeNode(Object userObject, boolean allowsChildren) {
    super(userObject, allowsChildren);
}

public TreeIterator getIterator() {
    return new ForwardTreeIterator((DataSourceTreeNode)getFirstChild());
}

public TreeIterator getIterator(boolean isForward) {
    if (isForward) {
        return new ForwardTreeIterator((DataSourceTreeNode)getFirstChild());
    }
    else {
        return new ReverseTreeIterator((DataSourceTreeNode)getLastChild());
    }
}

public TreeIterator getIterator(DataSourceTreeNode startNode) {
    return new ForwardTreeIterator(startNode);
}

public TreeIterator getIterator(DataSourceTreeNode startNode, boolean isForward) {
    if (isForward) {
        return new ForwardTreeIterator(startNode);
    }
    else {
        return new ReverseTreeIterator(startNode);
    }
}

public DataSourceTreeNode insert(DataSourceTreeNode before, DataSourceTreeNode data) {
    if ((before == null) || (data == null)) {
        return null;
    }
    insert(data, children.indexOf(before));
    return data;
}

public boolean isChildOf(DataSourceTreeNode possibleParent) {
    DataSourceTreeNode node = this;
    while (node != null) {
        if (node.getParent() == possibleParent) {
            return true;
        }
        node = (DataSourceTreeNode)node.getParent();
    }
    return false;
}

public boolean hasChildren() {
    return getChildCount() > 0;
}

public Vector getChildren() {
    return(Vector) children.clone();
}

public DataSourceTreeNode getPreviousChild(DataSourceTreeNode node) {
    int current = children.indexOf(node);
    if (current < 0) {
        return null;
    }

    current--;
    if (current >= 0) {
        return(DataSourceTreeNode) children.elementAt(current);
    }
    return null;
}

public DataSourceTreeNode getNextChild(DataSourceTreeNode node) {
    int current = children.indexOf(node);
    if (current < 0) {
        return null;
    }

    current++;
    if (current < children.size()) {
        return(DataSourceTreeNode) children.elementAt(current);
    }
    return null;
}

public TreeNode getFirstChild() {
    if (hasChildren()) {
        return super.getFirstChild();
    }
    else {
        return null;
    }
}

public TreeNode getLastChild() {
    if (hasChildren()) {
        return super.getLastChild();
    }
    else {
        return null;
    }
}


public String toString() {
    return getClass().getName() + "@" + Integer.toHexString(hashCode());
//    return ((Object)this).toString();
}

// Serialization support.  
private void writeObject(ObjectOutputStream s) throws IOException {
    s.defaultWriteObject();
//    super.writeObject(s);


}

private void readObject(ObjectInputStream s) 
throws IOException, ClassNotFoundException {
    s.defaultReadObject();
//    super.readObject(s);


}

}

