/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: IdeDataTable.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:09 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource;

import java.util.Hashtable;

/**
 * This class extends BaseDataTable to provide functionality
 * common to IDE implementations.
 */
public abstract class IdeDataTable extends BaseDataTable implements
    IdeDataTableModel, DataModelListener {

static final long serialVersionUID = -7437669651971881789L;

public long bookmarkOfLastRowAdded;
protected Hashtable ideRowIndexToBookmarkMap = new Hashtable();

/**
 * Root-level constructor.
 * Should be called only once to set the data table that will be the
 * root of the data table tree.
 * Subsequent nodes should be created using the constructor below.
 * @param metaData the MetaDataModel which provides the column structure
 * for this data table.
 */
public IdeDataTable(MetaDataModel metaData) {
    super(metaData);
}

/**
 * Constructor to call after root-level BaseDataTable has been created.
 * These tables will be sub-root level nodes in the data table tree.
 * @param metaData the MetaDataModel, which provides the column structure
 * for this data table.
 * @param parentRow an int indicating to which parent bookmark this
 * data table relates.
 */
public IdeDataTable(MetaDataModel metaData, long parentRow) {
    super(metaData, parentRow);
}

public void setValueAt2(long bookmark, String col_id, Object value) throws DataModelException {
    super.setValueAt(bookmark, col_id, value);
}

/**
 * Called by IDE event handler in meta data.
 * @exception DataModelException If row can't be added.
 */
public long addRow2(int newRowIndex) throws DataModelException {
    // duplicate it creating a new internal row
    Object new_row[] = duplicateRow(newRowIndex);

    // add it to the result table
    bookmarkOfLastRowAdded = addInternalRow(new_row);

    // since ide's insert map their index to our bookmark
    ideRowIndexToBookmarkMap.put(new Integer(newRowIndex), new Long(bookmarkOfLastRowAdded));

    // add the row to the updatedRows table
    insertedRows.put(new Long(bookmarkOfLastRowAdded), new_row);

    // update the view
    // originator = this
    fireDataModelEvent(new DataModelEvent(metaData.getDataModel(), this, this, bookmarkOfLastRowAdded, "", DataModelEvent.AFTER_INSERT_ROW));

    // move to the new row, causes listeners to move there, too
    getMetaData().getDataModel().moveToRow(this, this, bookmarkOfLastRowAdded, true);

    return bookmarkOfLastRowAdded;
}

// called in response to an event from jbuilder
public void deleteRow2(long bookmark) {
    Long bm = new Long(bookmark);
    removeRowFromOriginalRowTable(bookmark);
    removeRowFromLogicallyDeletedTable(bookmark);
    removeRowFromInsertedTable(bookmark);
    removeRowFromUpdatedTable(bookmark);
    ideRowIndexToBookmarkMap.clear();
    //originator = this
    try {
        fireDataModelEvent(new DataModelEvent(metaData.getDataModel(), this, this, bookmark, "", DataModelEvent.AFTER_DELETE_ROW));
    }
    catch (DataModelException ex) {
    }
}

public void commitRow2(long bookmark) {
    if (bookmark == ROW_NOT_FOUND) {
        return;
    }
    Long bm = new Long(bookmark);
    removeRowFromUpdatedTable(bookmark);
    removeRowFromLogicallyDeletedTable(bookmark);
    removeRowFromInsertedTable(bookmark);
    ideRowIndexToBookmarkMap.clear();
    // originator = this
    try {
        fireDataModelEvent(new DataModelEvent(metaData.getDataModel(), this, this, bookmark, "", DataModelEvent.AFTER_CHANGE_OF_ROW_DATA));
    }
    catch (DataModelException ex) {
    }
}

/**
 * Converts between an IDE index and our bookmark.
 */
public long getBookmarkFromIdeIndex(int ideIndex) {
    if (ideRowIndexToBookmarkMap.size() > 0) {
        Long bookmark = (Long) ideRowIndexToBookmarkMap.get(new Integer(ideIndex));
        if (bookmark != null && !bookmark.equals(new Long(ROW_NOT_FOUND))) {
            return bookmark.longValue();
        }
    }
    return super.getRowIdentifier(ideIndex);
}

/**
 * Requeries from the IDE data soruce, that is, not from the server.
 */
public void requeryFromIdeDataSource() {
    store.clear();
    logicallyDeletedRows.clear();
    updatedRows.clear();
    insertedRows.clear();
    ideRowIndexToBookmarkMap.clear();
    try {
        copyTable();
    }
    catch (Exception e) {
        System.out.println(e.toString());
    }
}

/**
 * For internal use only.
 * Remove the mapping between ide row indexs and bookmarks.
 */
public void removeRowFromIdeMap(long bookmark) {
    Long bm = new Long(bookmark);
    if (ideRowIndexToBookmarkMap.contains(bm)) {
        ideRowIndexToBookmarkMap.remove(bm);
    }
}

/**
 * Overrides <code>BaseDataTable.requeryRowAndDetails</code>, adds an extra call
 * which refreshes child data sets.
 * Requeries a DataTable row and deletes all children from the cache.
 * Calls <code>requeryRow()</code>.
 * Refetches all rows in this DataTable from the
 * originating data source and deletes all child nodes from the cache.
 *
 * @param originator the DataModelListener which initiated this action
 * @return this data table model
 * @exception DataModelException If requery from server fails.
 * @see com.klg.jclass.datasource.BaseDataTable#requeryRow
 */
public void requeryRowAndDetails(DataModelListener originator, long bookmark) throws DataModelException {
    // give the listeners the opportunity to cancel this requery
    if (!fireDataModelEvent(new DataModelEvent(metaData.getDataModel(), originator, this, bookmark, "", DataModelEvent.BEFORE_REQUERY))) {
        return;
    }
    requeryRow(originator, bookmark);
    removeAllChildren();
    metaData.getDataModel().clearCurrentPath(metaData);
    try {
        ((IdeMetaData)getMetaData()).cleanChildDataSets();
    }
    catch (Exception e) {
        throw new DataModelException(bookmark, "", this, DataModelEvent.AFTER_REQUERY_ROW_AND_DETAILS, e);
    }
    fireDataModelEvent(new DataModelEvent(metaData.getDataModel(), originator, this, bookmark, "", DataModelEvent.AFTER_REQUERY_ROW_AND_DETAILS));
}

//================== DataModelListener interface methods ==================

/**
 * Called before a row is inserted. Can be cancelled.
 * If not cancelled, will be followed by <code>afterInsertRow()</code>.
 * @see DataTableModel#addRow
 */
public void beforeInsertRow(DataModelEvent e) {}

/**
 * Called before entire grid is reset. Can be cancelled.
 * If not cancelled, will be followed by <code>afterReset()</code>.
 * This will result from a call to DataModel.requeryAll().
 * @see DataModel#requeryAll
 */
public void beforeReset(DataModelEvent e) {}

/**
 * Called when either <code>DataTableModel.requeryRowAndDetails()</code> or
 * <code>DataTableModel.requeryRow()</code> is called.
 * If not cancelled will be followed by an
 * <ul>
 * <li><code>afterRequeryRowAndDetails()</code> call or an</li>
 * <li><code>afterDeleteRow()</code> call in the case <code>getRowStatus() == INSERTED</code> or an</li>
 * <li><code>afterChangeOfRowData()</code> call in the case
 * <code>getRowStatus() == UPDATED</code> or <code>COMMITTED</code>.</li>
 * </ul>
 * @see DataTableModel#requeryRow
 * @see DataTableModel#requeryRowAndDetails
 */
public void beforeRequery(DataModelEvent e) {}

/**
 * Called before a cell is edited. Can be cancelled.
 * @see DataTableModel#updateCell
 */
public void beforeEditCell(DataModelEvent e) {}

/**
 * Called before a row is [logically] deleted. Can be cancelled.
 * If not cancelled will be followed by an
 * <code>afterDeleteRow()</code> or an <code>afterChangeOfRowData()</code> if the commit policy
 * is <code>COMMIT_MANUALLY</code> or <code>COMMIT_LEAVING_ANCESTOR</code>.
 * @see DataTableModel#deleteRow
 * @see MetaDataModel#getCommitPolicy
 */
public void beforeDeleteRow(DataModelEvent e) {}

/**
 * Called before all changes are committed. Can be cancelled.
 * All modified, deleted and inserted rows at all levels are about
 * to be committed.  <code>beforeDeleteRow()</code> and <code>afterChangeOfRowData()</code>
 * will follow depending on the operations performed on the modified rows
 * being saved.
 * Results from a call to <code>DataModel.updateAll</code>.
 * @see DataModel#updateAll
 */
public void beforeCommitAll(DataModelEvent e) {}

/**
 * Called before all changes are cancelled. Can be cancelled.
 * <code>afterInsertRow()</code> and <code>afterChangeOfRowData()</code> can follow this
 * event.
 * @see DataModel#cancelAll
 */
public void beforeCancelAll(DataModelEvent e) {}

/** Called before all edits to a row are undone. Can be cancelled.
 * An <code>afterDeleteRow()</code> or <code>afterChangeOfRowData()</code> will follow.
 * @see DataTableModel#cancelRowChanges
 */
public void beforeCancelRowChanges(DataModelEvent e) {}

/**
 * Called when the root-level bookmark for a subtree changes.
 * When this happens those nodes in the previous subtree which
 * are not <code>COMMIT_MANUALLY</code> are committed.
 * Can be cancelled.  If cancelled, the cursor moves but the
 * changes are automatically committed.
 */
public void beforeCommitConditional(DataModelEvent e) {}

/**
 * Called before single row is committed to data source.  Can be cancelled, in
 * which case the row edits are not written to the datasource and the row's
 * status remains modified.  <code>afterDeleteRow()</code>
 * or <code>afterChangeOfRowData()</code> will
 * follow, depending on the status of the row to be committed.
 * @see DataTableModel#commitRow
 */
public void beforeCommitRow(DataModelEvent e) {}

/**
 * The global cursor will move to a new row.
 * Can be cancelled.
 */
public void beforeMoveToCurrentRow(DataModelEvent e) {}

/**
 * The indicated DataTable will be deleted and flushed from the cache.
 * Can be cancelled.
 */
public void beforeDeleteTable(DataModelEvent e) {}

/**
 * A row has changed; re-read all its cells and its status to reflect the new
 * values.  If this event is the result of a cell edit, call <code>DataModelEvent.getColumn()</code>
 * to get the name of the column which changed.
 * If <code>getColumn()</code> returns <code>""</code>, re-read
 * the entire row.
 * Called when
 * <ul>
 * <li>a row is deleted and <code>getShowDeletedRows() == true</code></li>
 * <li>a cell was edited</li>
 * <li>row edits are cancelled and <code>getRowStatus() == UPDATED</code></li>
 * <li>row edits are cancelled and <code>getRowStatus() == DELETED</code>
 * and <code>getShowDeletedRows() == true</code></li>
 * <li>row is commited and <code>getRowStatus() == UPDATED</code> or <code>INSERTED</code></li>
 * <li>row is requeried and <code>getRowStatus() != INSERTED</code></li>
 * </ul>
 */
public void afterChangeOfRowData(DataModelEvent e) {}

/**
 * A new row has been added to the datasource. Listeners need to
 * display the row.  Rows are always added to the end of DataTableModels.
 */
public void afterInsertRow(DataModelEvent e) {}

/**
 * Removes the row from the display.
 * A row has been physically deleted and needs to be
 * removed from the display, or has been logically deleted but the
 * <code>showDeletedRows</code> property has been set to <code>false</code>. Called when
 * <ul>
 * <li>a row has been logically deleted and <code>getShowDeletedRows() == false</code></li>
 * <li>row changes have been cancelled and <code>getRowStatus() == INSERTED</code></li>
 * <li>a row is committed and <code>getRowStatus() == DELETED</code>
 * and <code>getShowDeletedRows() == true</code></li>
 * <li>a row has been requeried and <code>getRowStatus() == INSERTED</code></li>
 * </ul>
 */
public void afterDeleteRow(DataModelEvent e) {}

/**
 * Listeners must close all expanded views and reset/reread the root node.
 * The previous pointer to the root node is no longer valid.  Call
 * <code>DataModel.getDataTableTree().getRoot()</code> for the new root table.
 * Called when the datasource has been reset.
 * @see DataModel#requeryAll
 */
public void afterReset(DataModelEvent e) {}

/**
 * The global cursor has moved to a new row.
 * Listeners should position their cursor on
 * the indicated row.  In a master-detail relationship, child levels
 * should refresh themselves to reflect data sets which correspond
 * to the new parent row by calling <code>DataModel.getCurrentDataTable()</code>
 * or for field controls, <code>DataModel.getCurrentDataItem()</code>.
 */
public void afterMoveToCurrentRow(DataModelEvent e) {}

/**
 * Re-reads the indicated row and refreshes all open children
 * under this row.
 */
public void afterRequeryRowAndDetails(DataModelEvent e) {}

/**
 * Re-read this table and refreshes all open children in
 * this table.
 */
public void afterRequeryTable(DataModelEvent e) {}

/**
 * Notification that multiple events are coming.
 * Multiple events will be nested
 * between <code>beginEvents()</code> and <code>endEvents()</code>.
 * Allows listeners to treat the events as a batch,
 * for example, to reduce repaints.
 */
public void beginEvents(DataModelEvent e) {}

/**
 * Notification that multiple events are complete.
 * Multiple events will be nested
 * between <code>beginEvents()</code> and <code>endEvents()</code>.
 * Allows listeners to treat the events as a batch,
 * for example, to reduce repaints. Called when
 * <ul>
 * <li><code>DataModel.updateAll</code> is called</li>
 * <li><code>DataModel.cancelAll</code> is called</li>
 * </ul>
 */
public void endEvents(DataModelEvent e) {}

/**
 * The current row has been deleted and the originator
 * of the deletion should now reposition the global
 * cursor to a new, valid row.
 */
public void originatorNavigateRow(DataModelEvent e) {}

/**
 * For internal use only.
 * Used by the JCData bean.
 */
public void initialize(DataModelEvent e) {}


}
