/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ReadOnlyBindingModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:09 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource;

import com.klg.jclass.util.JCListenerList;

import java.util.Enumeration;

/**
 *
 * This interface provides a single-level, 2-dimensional view
 * of a set of data. It groups all non-update methods and
 * handles read-only events. This interface exists only to provide
 * a logical separation between read-only and non-read-only
 * methods and event handling.
 * @see BindingModel
 *
 */
public interface ReadOnlyBindingModel {
 
/**
 *
 * Get the table currently in scope.
 * @param 	originator the DataModelListener making this request.
 * @return	the DataTableModel currently in focus for this level.
 *
 */
public DataTableModel getCurrentDataTableModel(DataModelListener originator) throws DataModelException;
/**
 *
 * Get the meta data model for this level.
 * @return	the MetaDataModel for this level.
 *
 */
public MetaDataModel getMetaDataModel();

/**
 *
 * Set the meta data model for this level.
 * @param	the MetaDataModel for this level.
 *
 */
public void setMetaDataModel(MetaDataModel metaDataModel);
	
/**
 *
 * Get the parent Binding of this table.
 * @return	the Binding which is the parent of this level.
 *
 */
public Binding getParent();
	
/**
 *
 * Get the data model containing this level.
 * @return	the DataModel which contains this level.
 *
 */
public DataModel getDataModel();
	
/**
 *
 * Set the data model for this level.
 * @param	dataModel the data model for this level.
 *
 */
public void setDataModel(DataModel dataModel);
	
/**
 *
 * Set the name for this level.
 * @param 	the name for this level.
 *
 */
public void setDescription(String name);
	
/**
 *
 * Get the name for this level.
 * @return 	the name for this level.
 *
 */
public String getDescription();
	
/**
 *
 * Implements DataModel method addDataModelListener.
 * @param	the DataModelListener to be registered.
 *
 */
public void addBindingListener(DataModelListener listener);
	
/**
 *
 * Implements DataModel method removeDataModelListener.
 * @param	the DataModelListener to be removed.
 *
 */
public void removeBindingListener(DataModelListener listener);
	
/**
 *
 * Returns a list of listeners
 * @return	the list of listeners.
 *
 */
public Object getListeners();
	
/**
 *
 * Return the number of rows in the table currently in scope.
 * @return	the number of rows in the table currently in scope.
 *
 */
public int getRowCount();
	
/**
 *
 * Return the number of columns in this table.
 * @return	the number of columns in this table.
 *
 */
public int getColumnCount();
	
/**
 *
 * Move focus to a new row in the current table.
 * @param	originator the DataModelListener requesting the move.
 * @param	rowIndex the index of the row
 *
 */
public boolean moveToRowIndex(DataModelListener originator, int rowIndex) throws DataModelException;
	
/**
 *
 * Return true if data is available from this table.
 * @return	true if this table has data, false otherwise.
 *
 */
public boolean isDataAvailable();
	
/**
 *
 * Return the name of this column.
 * @param	colIndex the int column index.
 * @return	the String name of this column.
 *
 */
public String getColumnIdentifier(int colIndex);
	
/**
 *
 * Return the MetaDataModel type for this column.
 * @param	columnName the name of the column.
 * @return	one of MetaDataModel.TYPE_*.
 * @see 	MetaDataModel
 *
 */
public int getJavaColumnType(String columnName);
	
/**
 *
 * Return the MetaDataModel type for this column.
 * @param	the columnIndex
 * @return	one of MetaDataModel.TYPE_*.
 * @see 	MetaDataModel
 *
 */
public int getJavaColumnType(int columnIndex);
	
/**
 *
 * Get the value in a cell for a specified row.
 * @param	originator the DataModelListener making the commit request.
 * @param	rowIndex the row index.
 * @param	columnIndex the column index.
 * @exception DataModelException If get fails.
 *
 */
public Object getCell(DataModelListener originator, int rowIndex, int columnIndex) throws DataModelException;
	
/**
 *
 * Get the value in a cell for a specified row.
 * @param	originator the DataModelListener making the commit request.
 * @param	rowIndex the row index.
 * @param	columnName the column name.
 * @exception DataModelException If get fails.
 *
 */
public Object getCell(DataModelListener originator, int rowIndex, String columnName) throws DataModelException;
	
/**
 *
 * Get the value in a cell for the row currently in focus.
 * @param 	originator the DataModelListener requesting the value.
 * @param	columnName the String column name for the column.
 * @return	the Object in this cell.
 * @exception DataModelException If get fails.
 *
 */
public Object getCurrentCell(DataModelListener originator, String columnName) 
 throws DataModelException;
	
/**
 *
 * Return the index of the row currently in focus.
 * @return the index of the current row.
 *
 */
public int getCurrentRowIndex();
	
/**
 *
 * Requery the specified row.
 * @param	originator the DataModelListener making the requery request.
 * @param	rowIndex the row index to be requeried.
 *
 */
public void requeryRow(DataModelListener originator, int rowIndex) throws DataModelException;
	
/**
 *
 * Requery the row currently in focus.
 * @param	originator the DataModelListener making the requery request.
 *
 */
public void requeryCurrentRow(DataModelListener originator) throws DataModelException;
	
/**
 *
 * Requery a row and its details.
 * @param	originator the DataModelListener making the requery request.
 * @param	rowIndex the row index to be requeried.
 *
 */
public void requeryRowAndDetails(DataModelListener originator, int rowIndex) throws DataModelException;
	
/**
 *
 * Requery the row currently in focus and its details.
 * @param	originator the DataModelListener making the requery request.
 *
 */
public void requeryCurrentRowAndDetails(DataModelListener originator) throws DataModelException;
	
/**
 *
 * Requery all levels.
 * @param	originator the DataModelListener making the requery request.
 *
 */
public void requeryAll(DataModelListener originator) throws DataModelException ;
	
/**
 *
 * Return the index for this column name.
 * @return the index of this column.
 *
 */
public int getColumnIndex(String columnName);
	
/**
 *
 * Requery all rows in the table currently in focus.
 * @param	originator the DataModelListener making the requery request.
 *
 */
public void requeryLevel(DataModelListener originator) throws DataModelException;
	
/**
 *
 * Is the event intend for this level?
 *
 */
public boolean isOurData(DataModelEvent e);
	
public ColumnModel getColumnObject(String columnName);
		
}
