/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Store.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:09 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource;

import java.io.Serializable;
import java.util.*;

/**
 * Provides storage and retrieval for DataTableData.
 * Maps between bookmarks and row indexes.
 */
public class Store implements StoreModel, Serializable {

static final long serialVersionUID = 6457476356237148099L;

protected Vector idxToBookmarkMap;    // map idxs (ordinal position) to bookmarks
protected Hashtable rowCache;         // the row data

public Store() {
    idxToBookmarkMap = new Vector();
    rowCache = new Hashtable();
}

/**
 * Return the number of rows in this store
 * @return an int which is the number of rows in this store
 */
public int getRowCount() {
    return rowCache.size();
}

/**
 * Given a bookmark, return the row
 * @param bookmark the long which uniquely identifies a row
 * @return the row object for this bookmark
 */
public Object getRow(long bookmark) {
    return rowCache.get(new Long(bookmark));
}

/**
 * Add a row.
 */
public void addRow(long bookmark, Object row) {
    rowCache.put(new Long(bookmark), row);
    idxToBookmarkMap.addElement(new Long(bookmark));
}

/**
 * Insert a row.
 */
public void addRow(long bookmark, Object row, int ordinal) {
    rowCache.put(new Long(bookmark), row);
    idxToBookmarkMap.insertElementAt(new Long(bookmark), ordinal);
}

/**
 * Remove a row.
 * @param bookmark the long bookmark of the row to be removed
 */
public void removeRow(long bookmark) {
    Long bm = new Long(bookmark);
    if (rowCache.containsKey(bm)) {
        rowCache.remove(bm);
    }
    if (idxToBookmarkMap.contains(bm)) {
        idxToBookmarkMap.removeElement(bm);
    }
}

/**
 * Get index.
 * @param bookmark a long which uniquely identifies a row
 * @return its ordinal position
 */
public int getIndex(long bookmark) {
    try {
        return idxToBookmarkMap.indexOf(new Long(bookmark));
    }
    catch (ArrayIndexOutOfBoundsException e) {
        return DataTableModel.ROW_NOT_FOUND;
    }
}

/**
 * Return the bookmark of the row at ordinal.
 * @return the long bookmark of the row at ordinal
 */
public long getBookmark(int ordinal) {
    try {
        return((Long)idxToBookmarkMap.elementAt(ordinal)).longValue();
    }
    catch (ArrayIndexOutOfBoundsException e) {
        return DataTableModel.ROW_NOT_FOUND;
    }
}

/**
 * Clear all rows.
 */
public void clear() {
    idxToBookmarkMap.removeAllElements();
    rowCache.clear();
}

}
