/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TreeIteratorModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:09 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource;

import java.io.Serializable;

/**
 * This is the interface for an iterator which will
 * traverse the list of children for this this node.
 */
public interface TreeIteratorModel extends Serializable {

static final long serialVersionUID = -5073700873886343040L;

/**
 * Return the next child of this node.
 * @return the Object which is the next child of this node
 */
public Object nextElement();

/**
 * Returns boolean: true if this node has more children, false otherwise.
 * @return boolean: true if there are more children, false otherwise
 */
public boolean hasMoreElements();

/**
 * Returns boolean: true if iterator is positioned at the beginning of list,
 * false otherwise.
 * @return boolean: true if positioned at start of list, false otherwise
 */
public boolean atBegin();

/**
 * Returns boolean: true if iterator is positioned at the end of list,
 * false otherwise.
 * @return boolean: true if positioned at end of list, false otherwise
 */
public boolean atEnd();

/**
 * Return the current node.
 * @return the Object on which the iterator is currently positioned
 */
public Object get();

/**
 * Move to the next element in this iterator's list.
 */
public void advance();

/**
 * Move ahead a specified number of elements in this iterator's list.
 * @param advance an int indicating how many elements to skip
 */
public void advance(int offset);

/**
 * Return a copy of the current node.*
 * @return an Object which is a copy of the current node
 */
public Object clone();
}
