/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataSourceBeanInstance.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:09 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.beans;

import com.klg.jclass.datasource.*;

import java.io.Serializable;
import java.util.*;

/** This class contains a list of instances of data source beans
 *	like JCData and JCTreeData.  The list is used for JClass
 *	data-binding at design-time.
 */
public class DataSourceBeanInstance implements Serializable {

static final long serialVersionUID = 1427548816608159370L;

/** The list of instances. */
protected static Vector instances = new Vector();
private static final boolean TRACE = false;

/** Add an instance to the list. */
public static void addInstance(Object bean) {
    if (bean == null) return;

    instances.addElement(bean);
    if (TRACE) {
        System.out.print("DataSourceBean::added an instance ");
        if (bean instanceof DataModel) {
            System.out.print(((DataModel)bean).getModelName());
        }
        System.out.println(": " + instances);
    }
}

/** Remove an instance from the list. */
public static void removeInstance(Object bean) {
    if (bean == null) return;

    instances.removeElement(bean);
    if (TRACE) {
        System.out.print("DataSourceBean::removed an instance ");
        if (bean instanceof DataModel) {
            System.out.print(((DataModel)bean).getModelName());
        }
        System.out.println(": " + instances);
    }
}

/** Gets the first instance matching a model name of a data source bean. */
public static Object getInstance(String model_name) {
    if (model_name == null) return null;

    Object found = null;
    for (Enumeration e=instances.elements(); e.hasMoreElements();) {
        Object obj = e.nextElement();
        if (obj instanceof DataModel) {
            DataModel dm = (DataModel) obj;
            if (dm.getModelName() != null && dm.getModelName().equals(model_name)) {
                found = obj;
                break;
            }
        }
    }
    return found;
}

/** Remove an instance by a model name of a data source bean. */
public static void removeInstance(String model_name) {
    if (model_name == null) return;

    Object obj = getInstance(model_name);
    if (obj != null) {
        removeInstance(obj);
    }
}

/** Gets the list of instances. */
public static Vector getInstances() {
    return instances;
}

}
