/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: IdeJCTreeData.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:09 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.beans;

import com.klg.jclass.datasource.*;

import java.util.Enumeration;

import javax.swing.tree.*;

/**
 *	This bean creates a hierarchical structure of data.
 *	It supports different data source objects.
 *	@see com.klg.jclass.datasource.beans.NodeProperties
 */
abstract public class IdeJCTreeData extends JCTreeData {

static final long serialVersionUID = 4199716935197832460L;

public IdeJCTreeData(int type) {
    super(type);
}

/**	Creates meta data for the tree data bean.
 *	@exception DataModelException If fails to create a meta data.
 */
protected void createMetaData() throws DataModelException {
    setDataSources(getTreeProperties().getRoot(),
                   getTreeProperties().getDataSourceNames(),
                   getTreeProperties().getDataSources());
    super.createMetaData();
}

/** Sets the tree data bean component using a serialization file and
 *	an array of  data source objects.
 * 	Data source objects are specified for IDE data sources.  Supported IDE data
 *	source type are:
 *	<p><ul>
 *	<li>NodeProperties.JBUILDER</li>
 *	<li>NodeProperties.VCAFE</li>
 *	</ul><p>
 *	Data source object can be one of the following:
 *	<p><ul>
 *	<li>borland.jbcl.dataset.DataSet</li>
 *	<li>symantec.itools.db.beans.binding.QueryNavigator</li>
 *	</p>
 *	@param resource_name resource name relative to a class loader
 *						to the tree data bean component
 *	@param cl class loader used to get the resource.
 *	@param names array of names for data source objects
 *	@param data_sources array of data source objects
 *	@see com.klg.jclass.datasource.beans.NodeProperties
 *	@exception DataModelException If meta data cannot be created.
 */
public void setTreeProperties(String resource_name, ClassLoader cl,
                              String[] names, Object[] data_sources)
throws DataModelException {
    if (resource_name == null)  return;

    setTreeProperties(
                     new TreeProperties(resource_name, cl, names, data_sources));
}

/** Helper method called recursively to assign data source objects to the tree. */
void setDataSources(TreeNode node, String[] names, Object[] data_sources) {
    if (node == null || ((DataSourceTreeNode)node).getUserObject() == null || data_sources == null) return;

    NodeProperties data = (NodeProperties) ((DataSourceTreeNode)node).getUserObject();

    for (int i=0; i<data_sources.length; i++) {
        // if both data source name and type match, set data source object.
        if (data.getDataSourceName().equals(names[i])) {
            data.setDataSource(data_sources[i]);
            break;
        }
    }

/*
TreeIterator ti = node.getIterator();
while (ti.hasMoreElements()) {
    TreeNode child = (TreeNode) ti.nextElement();
    setDataSources(child, names, data_sources);
}
*/
    for (Enumeration e = node.children(); e.hasMoreElements();) {
        setDataSources((TreeNode) e.nextElement(), names, data_sources);
    }

}

} // IdeJCTreeData
