/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCData.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:09 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.beans;

import com.klg.jclass.datasource.*;
import com.klg.jclass.util.JCEnvironment;
import com.klg.jclass.util.JCListenerList;
import com.klg.jclass.util.JCTypeConverter;

import javax.swing.*;
import javax.swing.tree.TreeNode;
import java.applet.Applet;
import java.applet.AppletContext;
import java.io.Serializable;
import java.util.Enumeration;
import java.util.Vector;

/**
 *	A component that creates data connectivity with different data source
 *	objects.
 *	@see com.klg.jclass.datasource.beans.NodeProperties
 */
public class JCData extends TreeData implements DataTableModel, Serializable {

static final long serialVersionUID = -2129016695629947676L;

/** The parent applet. */
protected transient Applet applet;

/** Name of the Bean. */
//PR30159
protected String name = "com.klg.jclass.datasource.beans.JCData";

/** Label for bean */
protected String label = "JCData";

protected MetaDataCreator metaDataCreator = null;

/** Data Bean component. */
private NodeProperties comp;

private DataTableModel table;

static JCTypeConverter conv = new JCTypeConverter();

/** Used in <code>getParams</code>. */
private JComponent component = new JLabel();

private boolean isComponentSet = false;

private static String base = "JCData";
private static int nameCounter = 0;
private static int descriptionCounter = 0;
private static final boolean TRACE = false;
private DataModelListener dataModelListener = new DataModelAdapter();

/** Creates an empty data Bean of JDBC type.
 */
public JCData() {
    this(NodeProperties.JDBC);
}

/** Creates a <code>JCData</code> using a data source type.
 *	@param type data source type
 */
public JCData(int type) {
    super();
    if (type == NodeProperties.JDBC) {
        metaDataCreator = com.klg.jclass.datasource.jdbc.MetaDataCreator.getCreator(NodeProperties.JDBC);
    }
    // default description and model name
    descriptionCounter++;
    String description = LocaleBundle.string(LocaleBundle.node) + descriptionCounter;
    String model_name = "JCData" + descriptionCounter;
    comp = new NodeProperties(type, null, description, model_name);
    super.setModelName(model_name);
}

/** Creates a <code>JCData</code> which reads parameters from the applet's HTML file.
 * @param applet the applet whose <code>PARAM</code> tags are to be read
 * @param name if this is not null, only parameters preceded by this name are read
 * @see java.applet.Applet#getParameter
 */
public JCData(Applet applet, String name) {
    this();
    this.applet = applet;
    setName((name == null) ?  (base + nameCounter++) : name);
    if (getClass().getName().equals("com.klg.jclass.datasource.JCData")) {
        getParameters(applet);
    }
}

/** Gets the name of the Bean, as set in the constructor or by <code>setName</code>.
 * .
 * @see #setName
 * @return
 */
public String getName() {
    return name;
}

/**
 * Sets the name of the Bean.
 * @param name
 */
public void setName(String name) {
    this.name = name;
}


/** Reads the parameter values from the HTML page using the specified applet.
 * The values will override those previously set.
 * @see java.applet.Applet#getParameter
 * @param applet
 */
public void getParameters(Applet applet) {
    getParameters(applet, null);
}

/** Reads the parameter values from the file.
 * The values will override those previously set.
 * @param applet if not null and in a browser, its <code>documentBase()</code> is used to construct a
 * complete filename, if necessary
 * @param file if an http protocol is not specified (if a ":" is not present), the current working directory is used
 */
public void getParameters(Applet applet, String file) {
    this.applet = applet;

    if (file == null)
        file = getParam("paramFile");
//FIXOBSOLETE
//	if (file != null)
//		JCTypeConverter.setParamSource(component, JCFile.read(applet, file));

    // Read HTML file if running in a browser, or file if it is not null
    if (file != null || getAppletContext() != null) {
        getParameters();
    }
}

/**
 * Gets the Bean's applet's context. The applet context
 * lets an applet control the applet's environment, which is
 * usually the browser or the applet viewer.
 * @return null if the component is not in an applet or the applet is not in
 * a browser
 * @see java.applet.Applet#getAppletContext
 */
public AppletContext getAppletContext() {
    return getAppletContext(applet);
}


/** Gets the bean's applet's context. The applet context
 * lets an applet control the applet's environment, which is
 * usually the browser or the applet viewer.
 * @return null if the component is not in an applet or the applet is not in
 * a browser
 * @see java.applet.Applet#getAppletContext
 * @param applet
 */
public static AppletContext getAppletContext(Applet applet) {
    if (applet != null) {
        // AWT bug - null exception thrown
        try {
            return applet.getAppletContext();
        }
        catch (Exception e) {
        }
    }
    return null;
}

protected void createMetaData() throws DataModelException {
    metaDataCreator.createMetaData(this, null, comp);
}


/** Sets data bean component information from an existing data bean component.
 *	@exception DataModelException If meta data cannot be created.
 * @param comp
 */
public void setNodeProperties(NodeProperties comp) throws DataModelException {
    // remove the old instance
    if (!getModelName().equals(comp.getModelName())) {
        DataSourceBeanInstance.removeInstance(getModelName());
    }

    this.comp = comp;

    // Clear the trees first.
//	((DataSourceTreeModel)getMetaDataTree()).setRoot(null);
//	((DataSourceTreeModel)getDataTableTree()).setRoot(null);

    setModelName(comp.getModelName());

    createMetaData();
    table = (DataTableModel) getDataTableTree().getRoot();

    // At design time, information may not be set up completely and thus table is null.
    // In this case, we don't want to fire an INITIALIZE event to avoid unnecessary
    // error messages.
    boolean isDesignTime = JCEnvironment.isDesignTime();
    if (!isDesignTime || (isDesignTime && table != null)) {
        isComponentSet = true;
        fireDataModelEvent(
                          new DataModelEvent(dataModelListener, dataModelListener,
                                             (DataTableModel) getDataTableTree().getRoot(),
                                             -1, "", DataModelEvent.INITIALIZE));
    }

    if (DataSourceBeanInstance.getInstance(getModelName()) == null) {
        DataSourceBeanInstance.addInstance(this);
    }

    if (TRACE) {
        System.out.println("set data bean component to: " + comp);
    }
}

/** Gets the data bean component.
 *	@see #setNodeProperties
 * @return
 */
public NodeProperties getNodeProperties() {
    return comp;
}


/** Sets the data bean component using a serialized file.
 *	@param resource_name resource name relative to a class loader
 *						to the data bean component
 *	@param cl class loader used to get the resource
 *	@exception DataModelException If meta data cannot be created.
 */
public void setNodeProperties(String resource_name, ClassLoader cl) throws
DataModelException {
    setNodeProperties(new NodeProperties(resource_name, cl));
}


/** Gets <code>commitPolicy</code> property.
 *	@see #setCommitPolicy
 * @return
 */
public int getCommitPolicy() {
    return comp.getCommitPolicy();
}

/**
 *	Sets the commit policy for this level.  Determines when
 *	changes are permanently written to the originating data source.
 *	Must be one of the following:
 *	<p><ul>
 *	<li><code>MetaDataModel.COMMIT_LEAVING_RECORD</code></li>
 *	<li><code>MetaDataModel.COMMIT_LEAVING_ANCESTOR</code></li>
 *	<li><code>MetaDataModel.COMMIT_MANUALLY</code></li>
 *	</ul><p>
 *	@param commit_policy an int, one of
 * 	<p><ul>
 *	<li><code>MetaDataModel.COMMIT_LEAVING_RECORD</code></li>
 *	<li><code>MetaDataModel.COMMIT_LEAVING_ANCESTOR</code></li>
 *	<li><code>MetaDataModel.COMMIT_MANUALLY</code></li>
 *	</ul><p>
 *	@exception DataModelException If commit policy cannot be set.
 */
public void setCommitPolicy(int commit_policy) throws DataModelException {
    JCDataConverter.checkCommitPolicy(commit_policy);
    if (getMetaDataTree().getRoot() != null) {
        ((MetaDataModel)getMetaDataTree().getRoot()).setCommitPolicy(commit_policy);
    }
    comp.setCommitPolicy(commit_policy);
    comp.fireJCDataEvent(new JCDataEvent(comp));
}

/**
 * Returns the user-assigned description of this instance, or ""
 * if the description has not been set.
 * @return a String which describes this instance
 */
public String getDescription() {
    return(comp.getDescription() == null) ? "" : comp.getDescription();
}

/**
 * Sets the user-assigned description of this instance.
 * @param description a String which describles this instance
 */
public void setDescription(String description) {
    if (getMetaDataTree().getRoot() != null) {
        ((MetaDataModel)getMetaDataTree().getRoot()).setDescription(description);
    }
    comp.setDescription(description);
    comp.fireJCDataEvent(new JCDataEvent(comp));
}

/** Sets the model name of the data Bean.
 * @param s
 */
public void setModelName(String s) {
    // remove old instance
    DataSourceBeanInstance.removeInstance(getModelName());
    super.setModelName(s);
    comp.setModelName(s);
    if (DataSourceBeanInstance.getInstance(s) == null) {
        DataSourceBeanInstance.addInstance(this);
    }
    comp.fireJCDataEvent(new JCDataEvent(comp));
}

/** Gets the <code>about</code> property.
 *	The <code>about</code> property displays contact information for
 * Quest Software.
 * @return
 */
public String getAbout() {
    return LocaleBundle.string(LocaleBundle.about);
}
/** <code>about</code> property is read-only.
 * @param s
 */
public void setAbout(String s) {
}

/**
 * Gets the value of the <code>Version</code> property.
 * The <code>Version</code> property specifies the JClass datasource version number.
 * @return
 */
public String getVersion() {
    return com.klg.jclass.datasource.JCVersion.getVersionString();
}

/** <code>Version</code> property is read-only.
 * @param s
 */
public void setVersion(String s) {
}

/** Adds a data model listener.
 * @param listener
 */
public void addDataModelListener(DataModelListener listener) {
    boolean contains = false;
    Enumeration e = JCListenerList.elements(listeners);
    while (e.hasMoreElements()) {
        if (listener.equals(e.nextElement())) {
            contains = true;
            break;
        }
    }

    if (!contains) super.addDataModelListener(listener);

    // If data bean component is set, then fire a INITIALIZE DataModelEvent.
    // It is necessary because setTreeProperties() may be called before
    // this listener is registered.
    if (!contains && isComponentSet) {
        try {
            listener.initialize(
                               new DataModelEvent(dataModelListener, listener, (DataTableModel) getDataTableTree().getRoot(),
                                                  -1, "", DataModelEvent.INITIALIZE));
        }
        catch (DataModelException ex) {
            ex.printStackTrace();
        }
    }
}


/**
 *	Returns the status of the current row, will be one of
 *	<p><ul>
 *	<li><code>DataTableModel.INSERTED</code></li>
 *	<li><code>DataTableModel.UPDATED</code></li>
 *	<li><code>DataTableModel.DELETED</code></li>
 *	<li><code>DataTableModel.COMMITTED</code></li>
 *	</ul><p>
 *	@see com.klg.jclass.datasource.DataTableModel#getRowStatus
 *	@return an int row status, will be one of
 *	<p><ul>
 *	<li><code>DataTableModel.INSERTED</code></li>
 *	<li><code>DataTableModel.UPDATED</code></li>
 *	<li><code>DataTableModel.DELETED</code></li>
 *	<li><code>DataTableModel.COMMITTED</code></li>
 *	</ul><p>
 *	@exception DataModelException If <code>getCurrentRowStatus()</code> fails.
 *	@see com.klg.jclass.datasource.DataTableModel
 */
public int getCurrentRowStatus() throws DataModelException {
    if (table == null) return DataTableModel.COMMITTED;
    return table.getRowStatus(table.getCurrentBookmark());
}


/////////////////////////////////////////////////////////////////////
//
//	Expose part of the DataTabelModel interface.  Because JCData
//	is single-level, methods to deal with hierarchy are not implemented.
//

/** @see com.klg.jclass.datasource.DataTableModel#getRowStatus
 * @return
 */
public int getRowCount() {
    if (table == null) return -1;
    return table.getRowCount();
}

/** Returns the status of a row, will be one of:	<p><ul>
 *	<li><code>DataTableModel.INSERTED</code></li>
 *	<li><code>DataTableModel.UPDATED</code></li>
 *	<li><code>DataTableModel.DELETED</code></li>
 *	<li><code>DataTableModel.COMMITTED</code></li>
 *	</ul><p>
 *	@param bookmark a long which uniquely identifies this row
 *	@see com.klg.jclass.datasource.DataTableModel#getRowStatus
 *	@return an int row status, will be one of
 *	<p><ul>
 *	<li><code>DataTableModel.INSERTED</code></li>
 *	<li><code>DataTableModel.UPDATED</code></li>
 *	<li><code>DataTableModel.DELETED</code></li>
 *	<li><code>DataTableModel.COMMITTED</code></li>
 *	</ul><p>
 */
public int getRowStatus(long bookmark) {
    if (table == null) return DataTableModel.COMMITTED;
    return table.getRowStatus(bookmark);
}

/**
 *	Returns a long which will uniquely identify this row.
 *	Facilitates sorting: index changes, identifier remains constant.
 *	@param row an int for the original index of this row
 *	@return a unique int to identify this row from now on
 *	@see com.klg.jclass.datasource.DataTableModel#getRowIdentifier
 */
public long getRowIdentifier(int row) {
    if (table == null) return -1;
    return table.getRowIdentifier(row);
}

/**
 * @see com.klg.jclass.datasource.DataTableModel#getResultData
 *	@return an result object
 *	@exception DataModelException If <code>getResultData()</code> fails.
 * @param bookmark
 * @param column_identifier
 */
public Object getResultData(long bookmark,String column_identifier)
throws DataModelException {
    if (table == null) return null;
    return table.getResultData(bookmark, column_identifier);
}

/**
 * @see com.klg.jclass.datasource.DataTableModel#createTable
 *  @param originator the <code>DataModelListener</code> which initiated this action
 *	@return the child <code>DataTableModel</code> of results
 *	@exception DataModelException If <code>createTable()</code> fails.
 * @param bookmark
 * @param node
 */
public DataTableModel createTable(DataModelListener originator, long bookmark, TreeNode node) throws DataModelException {
    if (table == null) return null;
    return table.createTable(originator, bookmark, node);
}

/**	Since <code>JCData</code> has no child, it always returns null.
 *	@param bookmark a long which uniquely identifies a row
 *	@param node the <code>MetaDataModel</code> object
 *	@return null
 *	@see com.klg.jclass.datasource.DataTableModel#getTable
 */
public DataTableModel getTable(long bookmark, TreeNode node) {
    if (table == null) return null;
    return table.getTable(bookmark, node);
}

/**	Returns the meta-data of this data Bean.
 *	@see com.klg.jclass.datasource.DataTableModel#getMetaData
 *	@return the <code>MetaDataModel</code> Object for this table
 */
public MetaDataModel getMetaData() {
    if (table == null) return null;
    return table.getMetaData();
}

/** Since <code>JCData</code> has no parent, it always
 *	returns -1.
 *	@see com.klg.jclass.datasource.DataTableModel#getParentBookmark
 * @return
 */
public long getParentBookmark() {
    return -1;
}

/**
 *	Deletes a row.
 *      @param originator the <code>DataModelListener</code> which initiated this action
 *	@param bookmark a unique long indicating the row to be deleted
 *	@see com.klg.jclass.datasource.DataTableModel#deleteRow
 *	@exception DataModelException If logical delete fails.
 */
public void deleteRow(DataModelListener originator, long bookmark) throws DataModelException {
    if (table == null) return;
    table.deleteRow(originator, bookmark);
}

/** Updates a cell.
 *  @param originator the <code>DataModelListener</code> which initiated this action
 *	@param bookmark a unique long indicating in which row
 *	the change was made
 *	@param column_identifier a unique String indicating in which
 *	column the change occurred
 *	@param value the <code>Object</code> to be placed in the cell
 *	@exception DataModelException If error occurs updating a cell.
 *	@see com.klg.jclass.datasource.DataTableModel#updateCell
 */
public void updateCell(DataModelListener originator, long bookmark, String column_identifier,
                       Object value) throws DataModelException {
    if (table == null) return;
    table.updateCell(originator, bookmark, column_identifier, value);
}

/**
 *	Commits a row.
 *  @param originator the <code>DataModelListener</code> which initiated this action
 *	@param bookmark a unique long indicating which row
 *	to commit to the datasource
 *  @return <code>true</code> if commit succeeds, <code>false</code> if it fails or was cancelled
 *	@exception DataModelException If commit fails.
 *	@see com.klg.jclass.datasource.DataTableModel#commitRow
 */
public boolean commitRow(DataModelListener originator, long bookmark) throws DataModelException {
    if (table == null) return true;
    return table.commitRow(originator, bookmark);
}

/**
 *	Commits all uncommited rows.
 *	@see com.klg.jclass.datasource.DataTableModel#commitAll
 *  @param originator the <code>DataModelListener</code> which initiated this action
 *	@exception DataModelException If any row fails to commit.
 */
public void commitAll(DataModelListener originator) throws DataModelException {
    if (table == null) return;
    table.commitAll(originator);
}

/**
 *	Cancels uncommited row changes.
 *  @param originator the <code>DataModelListener</code> which initiated this action
 *	@param bookmark a unique long indicating which row to uncommit
 *	@see com.klg.jclass.datasource.DataTableModel#cancelRowChanges
 *      @exception DataModelException If changes can't be canceled.
 */
public void cancelRowChanges(DataModelListener originator, long bookmark) throws DataModelException {
    if (table == null) return;
    table.cancelRowChanges(originator, bookmark);
}

/**
 *	Cancels uncommited row changes.
 *      @param originator the <code>DataModelListener</code> which initiated this action
 *	@see com.klg.jclass.datasource.DataTableModel#cancelRowChanges
 *      @exception DataModelException If changes can't be canceled.
 */
public void cancelAllRowChanges(DataModelListener originator) throws DataModelException {
    if (table == null) return;
    table.cancelAllRowChanges(originator);
}

/** Returns a list of this DataTable's ancestor bookmarks.
 *	@see com.klg.jclass.datasource.DataTableModel#getAncestors
 * @param ancestors
 */
public void getAncestors(Vector ancestors) {
    if (table == null) return;
    table.getAncestors(ancestors);
}

/** Since <code>JCData</code has no parent, it always returns -1.
 *	@see com.klg.jclass.datasource.DataTableModel#getRootAncestorBookmark
 * @return
 * @param i
 */
public long getRootAncestorBookmark(long i) {
    if (table == null) return -1;
    return table.getRootAncestorBookmark(i);
}

/**	Re-reads a row from the originating data source.
 *	There is no detail to requery.
 *  @param originator the <code>DataModelListener</code> which initiated this action
 *	@see com.klg.jclass.datasource.DataTableModel#requeryRowAndDetails
 *	@exception DataModelException If requery from server fails.
 * @param bookmark
 */
public void requeryRowAndDetails(DataModelListener originator, long bookmark) throws DataModelException {
    if (table == null) return;
    table.requeryRowAndDetails(originator, bookmark);
}

/** Re-reads a row from the originating data source.
 *  @param originator the <code>DataModelListener</code> which initiated this action
 *	@param bookmark a long which uniquely identifies a row
 *	@exception DataModelException If requery from server fails.
 *	@see com.klg.jclass.datasource.DataTableModel#requeryRow
 */
public void requeryRow(DataModelListener originator, long bookmark) throws DataModelException {
    if (table == null) return;
    table.requeryRow(originator, bookmark);
}


/** Requery the data for the root level <code>DataTable</code>.
 *	@exception DataModelException If requery from server fails.
 *	@see com.klg.jclass.datasource.DataTableModel#requeryLevel
 * @return
 */
public DataTableModel requeryLevel() throws DataModelException {
    if (table == null) return null;
    return table.requeryLevel();
}

/** Gets the cursor of the data table.
 * @return
 */
public int getCursor() {
    if (table == null) return -1;
    return table.getCursor();
}

/** Moves the cursor to the row indicated by the bookmark.
 * @param bookmark
 *	@exception DataModelException If move fails.
 */
public void moveCursorToRow(long bookmark) throws DataModelException {
    if (table == null) return;
    table.moveCursorToRow(bookmark);
}


/**
 * Moves the cursor forward or backward by "n" rows from the current position.
 * Positive numbers move forward, negative backwards.
 * @param rows an int indicating the number of rows to move
 * @exception DataModelException If move fails.
 */
public void relative(int rows) throws DataModelException {
    if (table == null) return;
    table.relative(rows);
}


/**
 *	Moves the cursor to this ordinal position.
 *	Ordinal positions start at 1.
 *	@param ordinal_position an int indicating to which row to move
 *	@exception DataModelException If move fails.
 */
public void absolute(int ordinal_position) throws DataModelException {
    if (table == null) return;
    table.absolute(ordinal_position);
}

/** Moves to the first row.
 * @return
 */
public boolean first() {
    if (table == null) return false;
    return(table.first());
}

/** Moves to the previous row.
 * @return
 */
public boolean previous() {
    if (table == null) return false;
    return(table.previous());
}

/** Moves to the next row.
 * @return
 */
public boolean next() {
    if (table == null) return false;
    return(table.next());
}

/** Moves to the last row.
 * @return
 */
public boolean last() {
    if (table == null) return false;
    return(table.last());
}


/**
 * Positions the cursor before the first row.
 */
public void beforeFirst() {
    if (table == null) return;
    table.beforeFirst();
}

/**
 * Positions the cursor after the last row.
 */
public void afterLast() {
    if (table == null) return;
    table.afterLast();
}

/**
 * Returns <code>true</code> if the cursor is positioned before first row.
 * @return <code>true</code> if cursor is positioned before first row, <code>false</code> otherwise
 */
public boolean isBeforeFirst() {
    if (table == null) return false;
    return table.isBeforeFirst();
}

/**
 * Returns <code>true</code> if the cursor is positioned after the last row.
 * @return <code>true</code> if the cursor is positioned after the last row, <code>false</code> otherwise
 */
public boolean isAfterLast() {
    if (table == null) return false;
    return table.isAfterLast();
}


/**
 *	Returns the bookmark of the current row.
 *	@return the bookmark of the current row
 *	@exception DataModelException If there are no rows or
 *	or an error occurs.
 *	@see com.klg.jclass.datasource.DataTableModel#getCurrentBookmark
 */
public long getCurrentBookmark() throws DataModelException {
    if (table == null) return -1;
    return table.getCurrentBookmark();
}

/** Gets user data.
 *	Returns the Object which was arbitrarily associated with
 *	this table.
 *  @param user an Object, a reference to the user
 *	@see com.klg.jclass.datasource.DataTableModel#getUserData
 * @return
 */
public Object getUserData(Object user) {
    if (table == null) return null;
    return table.getUserData(user);
}

/** Sets user data.
 *	Allows users to augment this table by associating an
 *	arbitrary Object reference with it.
 *  @param user an Object, a reference to the user
 *	@param data an Object, an arbitrary reference to an Object
 *	@see com.klg.jclass.datasource.DataTableModel#setUserData
 */
public void setUserData(Object user, Object data) {
    if (table == null) return;
    table.setUserData(user, data);
}

/**
 *	Since <code>JCData</code> has no child,
 *	it commits all of its changes if <code>commitPolicy</code> does not equal <code>COMMIT_MANUALLY</code>.
 *	@see com.klg.jclass.datasource.DataTableModel#conditionallyCommitYourselfAndChildren
 *	@exception DataModelException If write to data source fails.
 * @param originator
 */
public void conditionallyCommitYourselfAndChildren(DataModelListener originator) throws DataModelException {
    if (table == null) return;
    table.conditionallyCommitYourselfAndChildren(originator);
}

/** Recursively processes pending events from a transaction.
 *  @param originator the <code>DataModelListener</code> which initiated this action
 *	@see com.klg.jclass.datasource.DataTableModel#processEventQueue
 *      @exception DataModelException If changes can't be processed.
 */
public void processEventQueue(DataModelListener originator) throws DataModelException {
    if (table == null) return;
    table.processEventQueue(originator);
}


/**
 *	Recursively clears pending events from a transaction.
 *	Called when an exception occurs. Events are not dispatched
 *	they are simply deleted.
 *	@see com.klg.jclass.datasource.DataTableModel#clearEventQueue
 */
public void clearEventQueue() {
    if (table == null) return;
    table.clearEventQueue();
}


/**
 * Given a bookmark, returns the zero-relative row index in this table.
 * @param bookmark a long which uniquely identifies a row
 * @return rowIndex an int which is the ordinal position of the row
 * in this table
 *	@see com.klg.jclass.datasource.DataTableModel#getRowIndex
 */
public int getRowIndex(long bookmark) {
    if (table == null) return -1;
    return table.getRowIndex(bookmark);
}

/**
 * Deletes the subtree of <code>DataTables</code> rooted with this <code>DataTable</code>
 * as its root.
 * @param originator the <code>DataModelListener</code> which initiated this action
 * @see com.klg.jclass.datasource.DataTableModel#deleteCacheChildren
 * @exception DataModelException If tables can't be deleted.
 */
public void deleteCacheChildren(DataModelListener originator) throws DataModelException {
    if (table == null) return;
    table.deleteCacheChildren(originator);
}

/**
 * For internal use only.
 * Gets internal data.
 * Returns the Object which was arbitrarily associated with
 * this table.
 * @param user an Object, a reference to the user.
 * @return the internal Object associated with this table.
 *	@see com.klg.jclass.datasource.DataTableModel#getInternalData
 */
public Object getInternalData(Object user) {
    if (table == null) return null;
    return table.getInternalData(user);
}

/**
 * For internal use only
 * Sets internal data.
 * Allows component writers to augment this table by associating an
 * arbitrary Object reference with it.
 * @param user an Object, a reference to the user
 * @param data an Object, an arbitrary reference to an Object
 *	@see com.klg.jclass.datasource.DataTableModel#setInternalData
 */
public void setInternalData(Object user, Object data) {
    if (table == null) return;
    table.setInternalData(user, data);
}

/**
 * Moves the ide cursor to this row.
 * @param bookmark a long which uniquely identifies a row
 * @exception DataModelException If move fails.
 * @see com.klg.jclass.datasource.DataTableModel#moveToRow
 */
public void moveToRow(long bookmark) throws DataModelException {
    if (table == null) return;
    table.moveToRow(bookmark);
}

/**
 * Does the subtree rooted at <code>subTreeRootBookmark</code> contain
 * modified nodes?  Modified means the row at <code>subTreeRootBookmark</code> or
 * any of its child nodes has pending insert, update or delete changes.
 * @param subTreeRootBookmark a long, the bookmark of the row which is
 * the root of the subtree to be checked
 * @return <code>true</code> if this subtree contains at least one modified
 * node, <code>false</code> otherwise
 */
public boolean isSubTreeModified(long subTreeRootBookmark) {
    if (table == null) return false;
    return table.isSubTreeModified(subTreeRootBookmark);
}

/**
 * Asks if there are any modifications to this table or any of its child nodes.
 * @return <code>true</code> if there are modifications to this table or any of its
 * child nodes
 */
public boolean isTableOrChildrenModified() {
    if (table == null) return false;
    return table.isTableOrChildrenModified();
}

/**
 * Commits all pending changes in this subtree.
 * Commits this row and all child nodes.
 *
 * @param originator the <code>DataModelListener</code> which initiated this action
 * @param subTreeRootBookmark a long, the bookmark of the row which is
 * the root of the subtree to be checked
 * @exception DataModelException If commit fails.
 */
public void commitSubTree(DataModelListener originator, long subTreeRootBookmark) throws DataModelException {
    if (table == null) return;
    table.commitSubTree(originator, subTreeRootBookmark);
}

/**
 * Asks if this <code>DataTableModel</code> contains any pending inserts, updates or
 * deletes.
 * @return <code>true</code> if there are pending modifications, <code>false</code> otherwise
 */
public boolean isModified() {
    if (table == null) return false;
    return table.isModified();
}

/**
 * Returns a long array of updated row bookmarks.  These are the rows
 * which have updates pending.
 * @return a long[] of updated row bookmarks
 */
public long[] getUpdatedRowBookmarks() {
    if (table == null) return null;
    return table.getUpdatedRowBookmarks();
}

/**
 * Returns a long array of inserted row bookmarks.  These are the rows
 * which are inserted into the data model, but not yet persisted.
 * @return a long[] of inserted row bookmarks
 */
public long[] getInsertedRowBookmarks() {
    if (table == null) return null;
    return table.getInsertedRowBookmarks();
}

/**
 * Returns a long array of logically deleted row bookmarks.  These are the
 * rows which are logically deleted, but not yet physically deleted from the
 * originating data source.
 * @return a long[] of logically deleted row bookmarks
 */
public long[] getDeletedRowBookmarks() {
    if (table == null) return null;
    return table.getDeletedRowBookmarks();
}

/**
 * Given a <code>DataTable</code>, returns whether it is an ancestor of
 * (or equal to) itself.
 * @param table2 a <code>DataTable</code> which may be an ancestor
 * @return <code>true</code> if the table is an ancestor
 */
public boolean tableIsAncestor(DataTableModel table2) {
    if (table == null) return false;
    return table.tableIsAncestor(table2);
}

/** Reads the parameter values from the HTML page using the data Bean's applet.
 * The values will override those previously set.
 * Subclasses may override this method to set their own values - in this case,
 * the method should first call <code>super.getParameters()</code>.
 */
protected void getParameters() {
    JCDataConverter.getParams(this);
}

/** Gets a paramater for this data Bean.
 * @param param
 * @return
 */
String getParam(String param) {
    //FIXOBSOLETE

//	return conv.getParam(applet, component, getName(), param);
    return "";
}

private void readObject(java.io.ObjectInputStream in) throws java.io.IOException {
    try {
        in.defaultReadObject();
        if (!DataSourceBeanInstance.getInstances().contains(this)) {
            DataSourceBeanInstance.addInstance(this);
        }
    }
    catch (Exception e) {
        e.printStackTrace(System.out);
    }
}

/**
 * Adds a row whose status will be <code>INSERTED</code>.
 * Adds a blank row into which the user can input column data.
 * The record is appended to this <code>ResultSet</code>.  The record is saved to datasource
 * when the user commits the surrounding transaction.
 * <P>
 * Fires <code>BEFORE_INSERT_ROW</code> and upon a successful row adds <code>AFTER_INSERT_ROW</code>.
 * <P>
 *
 * @param originator the <code>DataModelListener</code> which initiated this action
 * @exception DataModelException If row fails to be added.
 * @return
 */
public long addRow(DataModelListener originator) throws DataModelException {
    if (table == null) return -1;
    return table.addRow(originator);
}

/**
 * Adds a populated row whose status will be <code>status</code>.
 * Adds a pre-populated row whose status is dictated by the
 * <code>status</code> parameter.
 * <P>
 * Fires <code>BEFORE_INSERT_ROW</code> and upon a successful row adds <code>AFTER_INSERT_ROW</code>.
 * <P>
 * @param row an Object as defined by the interface, but used as an array
 * by this implementation
 * @param status an int, one of:
 * <ul>
 * <li><code>DataTableModel.COMMITTED</code></li>
 * <li><code>DataTableModel.DELETED</code></li>
 * <li><code>DataTableModel.INSERTED</code></li>
 * </ul>
 * @exception DataModelException If row fails to be added.
 * @return
 * @param originator
 */
public long addRow(DataModelListener originator, Object row, int status) throws DataModelException {
    if (table == null) return -1;
    return table.addRow(originator, row, status);
}

/**
 * Adds a populated row whose status will be <code>status</code>.
 * Adds a pre-populated row whose status is dictated by the
 * <code>status</code> parameter.
 * <P>
 * Fires <code>BEFORE_INSERT_ROW</code> and upon a successful row adds <code>AFTER_INSERT_ROW</code>.
 * <P>
 * @param row an Object as defined by the interface, but used as an array
 * by this implementation
 * @param status an int, one of:
 * <ul>
 * <li><code>DataTableModel.COMMITTED</code></li>
 * <li><code>DataTableModel.DELETED</code></li>
 * <li><code>DataTableModel.INSERTED</code></li>
 * </ul>
 * @param ordinal the index position into which the row should be inserted
 *
 * @exception DataModelException If row fails to be added.
 * @return
 * @param originator
 */
public long addRow(DataModelListener originator, Object row, int status, int ordinal) throws DataModelException {
    if (table == null) return -1;
    return table.addRow(originator, row, status, ordinal);
}

/**
 * Adds a populated row whose status will be <code>INSERTED</code>.
 * Adds a pre-populated row.
 * The record is appended to this <code>ResultSet</code>.  The record is saved to datasource
 * when the user commits the surrounding transaction.
 * <P>
 * Fires <code>BEFORE_INSERT_ROW</code> and upon a successful row adds <code>AFTER_INSERT_ROW</code>.
 * <P>
 *
 * @param originator the <code>DataModelListener</code> which initiated this action
 * @param row an Object as defined by the interface, but used as an array
 * by this implementation
 * @exception DataModelException If row fails to be added.
 * @return
 */
public long addRow(DataModelListener originator, Object row) throws DataModelException {
    if (table == null) return -1;
    return table.addRow(originator, row);
}

/**
 * Creates a new (blank) row that can be used as an
 * argument to one of the <code>addRow()</code> methods.
 * @return an Object which is the new row just added
 * @exception DataModelException If create fails.
 */
public Object createNewRow() throws DataModelException {
    if (table == null) return null;
    return table.createNewRow();
}

/**
 * Requery all rows in this table and collapse its child nodes.
 *
 * @param originator the <code>DataModelListener</code> which initiated this action
 * @exception DataModelException If requery fails.
 */
public void requeryTable(DataModelListener originator) throws DataModelException {
    if (table == null) return;
    table.requeryTable(originator);
}

/**
 * Returns the value of the label property
 * @return String
 */
public String getLabel() {
    return label;
}

/**
 * Sets the value of the label property.
  * @param label
 */
public void setLabel(String label) {
    this.label = label;
}


}
