/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCDataConverter.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:09 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.beans;

import com.klg.jclass.datasource.*;

import java.io.Serializable;

import com.klg.jclass.util.JCTypeConverter;

/** Reads <code>JCData</code> parameters from an HTML file or external file. */
public class JCDataConverter implements Serializable {

static final long serialVersionUID = 6339684854667314402L;

public final static int[] commitPolicyValues = {
    MetaDataModel.COMMIT_LEAVING_ANCESTOR,
    MetaDataModel.COMMIT_LEAVING_RECORD,
    MetaDataModel.COMMIT_MANUALLY,
};

public final static String[] commitPolicyStrings = {
    LocaleBundle.commit_leaving_ancestor,
    LocaleBundle.commit_leaving_record,
    LocaleBundle.commit_manually,
};

public final static int[] singleLevelCommitPolicyValues = {
    MetaDataModel.COMMIT_LEAVING_RECORD,
    MetaDataModel.COMMIT_MANUALLY,
};

public final static String[] singleLevelCommitPolicyStrings = {
    LocaleBundle.commit_leaving_record,
    LocaleBundle.commit_manually,
};

public final static int[] javaDataTypeValues = {
    MetaDataModel.TYPE_BIG_DECIMAL,
    MetaDataModel.TYPE_BOOLEAN,
    MetaDataModel.TYPE_BYTE,
    MetaDataModel.TYPE_BYTE_ARRAY,
    MetaDataModel.TYPE_DOUBLE,
    MetaDataModel.TYPE_FLOAT,
    MetaDataModel.TYPE_INTEGER,
    MetaDataModel.TYPE_LONG,
    MetaDataModel.TYPE_OBJECT,
    MetaDataModel.TYPE_SHORT,
    MetaDataModel.TYPE_STRING,
    MetaDataModel.TYPE_SQL_DATE,
    MetaDataModel.TYPE_SQL_TIME,
    MetaDataModel.TYPE_SQL_TIMESTAMP,
    MetaDataModel.TYPE_UTIL_DATE,
};

public final static String[] javaDataTypeStrings = {
    LocaleBundle.type_big_decimal,
    LocaleBundle.type_boolean,
    LocaleBundle.type_byte,
    LocaleBundle.type_byte_array,
    LocaleBundle.type_double,
    LocaleBundle.type_float,
    LocaleBundle.type_integer,
    LocaleBundle.type_long,
    LocaleBundle.type_object,
    LocaleBundle.type_short,
    LocaleBundle.type_string,
    LocaleBundle.type_sql_date,
    LocaleBundle.type_sql_time,
    LocaleBundle.type_sql_timestamp,
    LocaleBundle.type_util_date,
};

public final static String virtualColumnOperationStrings[] = {
    "MIN",
    "MAX",
    "AVERAGE",
    "SUM",
    "DIFFERENCE",
    "PRODUCT",
    "QUOTIENT",
};

public final static int virtualColumnOperationValues[] = {
    BaseVirtualColumn.MIN,
    BaseVirtualColumn.MAX,
    BaseVirtualColumn.AVERAGE,
    BaseVirtualColumn.SUM,
    BaseVirtualColumn.DIFFERENCE,
    BaseVirtualColumn.PRODUCT,
    BaseVirtualColumn.QUOTIENT,
};

static void getParams(JCData comp) {
    JCTypeConverter conv = comp.conv;

    try {
        comp.setNodeProperties(comp.getParam("NodeProperties"), 
                               comp.applet.getClass().getClassLoader());
    }
    catch (DataModelException ex) {
        ex.printStackTrace();
    }
    try {
        comp.setCommitPolicy(toCommitPolicy(conv, comp.getParam("commitPolicy"), 
                                            MetaDataModel.COMMIT_LEAVING_RECORD));
    }
    catch (DataModelException ex) {
        ex.printStackTrace();
    }

    comp.setDescription(comp.getParam("description"));
    comp.setModelName(comp.getParam("modelName"));
}


/** Gets an internationalized array of commit policy Strings.
 * @return
 */
public static String[] getCommitPolicyStrings() {
    String ss[] = new String[commitPolicyStrings.length];
    for (int i=0; i<ss.length; i++) {
        ss[i] = LocaleBundle.string(commitPolicyStrings[i]);
    }
    return ss;
}

/** Converts a String to a commit policy value.
 * @return
 * @param conv
 * @param def
 * @param s
 */
public static int
toCommitPolicy(JCTypeConverter conv, String s, int def) {
    return conv.toEnum(s, "commitPolicy", getCommitPolicyStrings(), 
                       commitPolicyValues, def);
}

/** Checks commit policy enum value.
 * @param v
 */
public static void checkCommitPolicy(int v) {
    JCTypeConverter.checkEnum(v, "commitPolicy", commitPolicyValues);
}


/** Gets an internationalized array of Java data type Strings.
 * @return
 */
public static String[] getJavaDataTypeStrings() {
    String ss[] = new String[javaDataTypeStrings.length];
    for (int i=0; i<ss.length; i++) {
        ss[i] = LocaleBundle.string(javaDataTypeStrings[i]);
    }
    return ss;
}

/** Converts a String to a Java data type value.
 * @return
 * @param conv
 * @param def
 * @param s
 */
public static int
toJavaDataType(JCTypeConverter conv, String s, int def) {
    return conv.toEnum(s, "javaDataType", getJavaDataTypeStrings(), 
                       javaDataTypeValues, def);
}

/** Checks Java data type enum value.
 * @param v
 */
public static void checkJavaDataType(int v) {
    JCTypeConverter.checkEnum(v, "javaDataType", javaDataTypeValues);
}


/** Converts a String to a virtual column model operation.
 * @return
 * @param conv
 * @param def
 * @param s
 */
public static int
toVirtualColumnOperation(JCTypeConverter conv, String s, int def) {
    return conv.toEnum(s, "virtualColumnOperation", virtualColumnOperationStrings, 
                       virtualColumnOperationValues, def);
}

/** Checks a virtual column model operation enum value.
 * @param v
 */
public static void checkVirtualColumnOperation(int v) {
    JCTypeConverter.checkEnum(v, "virtualColumnOperation", virtualColumnOperationValues);
}
}
