/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCTreeData.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:09 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.beans;

import com.klg.jclass.datasource.*;
import com.klg.jclass.util.JCListenerList;
import com.klg.jclass.util.JCTypeConverter;

import javax.swing.*;
import javax.swing.tree.TreeNode;
import java.applet.Applet;
import java.applet.AppletContext;
import java.io.Serializable;
import java.util.Enumeration;

/**
 *	This Bean creates a hierarchical structure of data.  It supports different data source
 *	objects.
 *	@see com.klg.jclass.datasource.beans.NodeProperties
 */
public class JCTreeData extends TreeData implements Serializable {

static final long serialVersionUID = 4891867884806222984L;

/** The parent applet. */
protected transient Applet applet;

/** Name of the Bean. */
//PR30159
protected String name = "com.klg.jclass.datasource.beans.JCTreeData";

protected MetaDataCreator metaDataCreator = null;

/** Tree data Bean component. */
TreeProperties comp;

static JCTypeConverter conv = new JCTypeConverter();

/** Used in <code>getParams</code>. */
private JComponent component = new JLabel();

private boolean isComponentSet = false;
private static String base = "JCTreeData";
private static int nameCounter = 0;
private static int descriptionCounter = 0;
private static final boolean TRACE = false;

private DataModelListener dataModelListener = new DataModelAdapter();

public JCTreeData() {
    this(NodeProperties.JDBC);
}

public JCTreeData(int type) {
    metaDataCreator = com.klg.jclass.datasource.beans.MetaDataCreator.getCreator(type);

    descriptionCounter++;
    String model_name = "TreeData" + descriptionCounter;
    comp = new TreeProperties(type, model_name);
    super.setModelName(model_name);
}

/** Creates a <code>JCTreeData</code> which reads parameters from the applet's HTML file.
 * @param applet the applet whose <code>PARAM</code> tags are to be read
 * @param name if this is not null, only parameters preceded by this name are read
 * @see java.applet.Applet#getParameter
 */
public JCTreeData(Applet applet, String name) {
    this();
    this.applet = applet;
    setName((name == null) ?  (base + nameCounter++) : name);
    if (getClass().getName().equals("com.klg.jclass.datasource.JCTreeData")) {
        getParameters(applet);
    }
}

/** Gets the name of the Bean, as set in the constructor or by <code>setName</code>.
 * If no name has been set, a 0-length String will be returned.
 * @see #setName
 * @return
 */
public String getName() {
    return name != null ? name : "";
}

/**
 * Sets the name of the Bean.
 * @param name Bean name
 */
public void setName(String name) {
    this.name = name;
}

/** Reads the parameter values from the HTML page using the specified applet.
 * The values will override those previously set.
 * @see java.applet.Applet#getParameter
 * @param applet
 */
public void getParameters(Applet applet) {
    getParameters(applet, null);
}

/** Reads the parameter values from the file.
 * The values will override those previously set.
 * @param applet if not null and in a browser, its <code>documentBase()</code> is used to
 * construct a complete filename if necessary
 * @param file if an http protocol is not specified (if a ":" is not present), the current working directory is used
 */
public void getParameters(Applet applet, String file) {
    this.applet = applet;

    if (file == null)
        file = getParam("paramFile");
//FIXOBSOLETE
//	if (file != null)
//		JCTypeConverter.setParamSource(component, JCFile.read(applet, file));

    // Read HTML file if running in a browser, or file if it is not null
    if (file != null || getAppletContext() != null) {
        getParameters();
    }
}

/**
 * Gets the Bean's applet's context. The applet context
 * lets an applet control the applet's environment, which is
 * usually the browser or the applet viewer.
 * @return null if the component is not in an applet or the applet is not in
 * a browser
 * @see java.applet.Applet#getAppletContext
 */
public AppletContext getAppletContext() {
    return getAppletContext(applet);
}


/** Gets the Bean's applet's context. The applet context
 * lets an applet control the applet's environment, which is
 * usually the browser or the applet viewer.
 * @return null if the component is not in an applet or the applet is not in
 * a browser.
 * @see java.applet.Applet#getAppletContext
 * @param applet
 */
public static AppletContext getAppletContext(Applet applet) {
    if (applet != null) {
        // AWT bug - null exception thrown
        try {
            return applet.getAppletContext();
        }
        catch (Exception e) {
        }
    }
    return null;
}

/** Gets the <code>about</code> property.
 *	The <code>about</code> property displays contact information for
 * Quest Software.
 * @return the <code>about</code> propety
 */
public String getAbout() {
    return LocaleBundle.string(LocaleBundle.about);
}
/** <code>about</code> property is read-only.
 * @param s
 */
public void setAbout(String s) {
}

/**
 * Gets the value of the <code>Version</code> property.
 * The <code>Version</code> property specifies the JClass version number.
 * @return
 */
public String getVersion() {
    return com.klg.jclass.datasource.JCVersion.getVersionString();
}

/** <code>Version</code> property is read-only.
 * @param s
 */
public void setVersion(String s) {
}


/** Sets the model name of the tree data Bean.
 * @param s
 */
public void setModelName(String s) {
    // remove old instance
    DataSourceBeanInstance.removeInstance(getModelName());
    super.setModelName(s);
    comp.setModelName(s);
    if (DataSourceBeanInstance.getInstance(s) == null) {
        DataSourceBeanInstance.addInstance(this);
    }
    comp.fireJCDataEvent(new JCDataEvent((Object)comp));
}


/** Sets <code>TreeProperties</code> property.
 *	@exception DataModelException If fails to create a meta data.
 * @param comp
 */
public void setTreeProperties(TreeProperties comp)
throws DataModelException {
    if (TRACE) {
        System.out.println("set JCTreeData component from\n" + this.comp +
                           "\n to \n" + comp);
    }
    // remove the old instance
    if (!getModelName().equals(comp.getModelName())) {
        DataSourceBeanInstance.removeInstance(getModelName());
    }

    this.comp = comp;

    super.setModelName(comp.getModelName());
    createMetaData();

    if (getMetaDataTree().getRoot() != null) {
        isComponentSet = true;
        fireDataModelEvent(new DataModelEvent(this, dataModelListener,
                                              (DataTableModel) getDataTableTree().getRoot(),
                                              -1, "", DataModelEvent.INITIALIZE));
    }
    if (DataSourceBeanInstance.getInstance(getModelName()) == null) {
        DataSourceBeanInstance.addInstance(this);
    }
}

/** Gets <code>TreeProperties</code> property.
 *	@see #setTreeProperties
 * @return
 */
public TreeProperties getTreeProperties() {
    return comp;
}

/** Sets the tree data Bean component using a serialization file.
 *	@param resource_name resource name relative to a class loader
 *						to the tree data Bean component
 *	@param cl class loader used to get the resource
 * 	@exception DataModelException If fails to create a meta data.
 */
public void setTreeProperties(String resource_name, ClassLoader cl)
throws DataModelException {
    setTreeProperties(new TreeProperties(resource_name, cl));
}

/** implements <code>DataModel</code> method <code>addDataModelListener</code>.
 * @param listener
 */
public void addDataModelListener(DataModelListener listener) {
    boolean contains = false;
    Enumeration e = JCListenerList.elements(listeners);
    while (e.hasMoreElements()) {
        if (listener.equals(e.nextElement())) {
            contains = true;
            break;
        }
    }
    if (!contains) super.addDataModelListener(listener);

    // If data Bean component is set, then fire a INITIALIZE DataModelEvent.
    // It is necessary because setTreeProperties() may be called before
    // this listener is registered.
    if (!contains && isComponentSet) {
        try {
            listener.initialize(
                               new DataModelEvent(this, dataModelListener,
                                                  (DataTableModel) getDataTableTree().getRoot(),
                                                  -1, "", DataModelEvent.INITIALIZE));
        }
        catch (DataModelException ex) {
        }
    }
}


/**	Creates meta data for the tree data Bean.
 *	@exception DataModelException If fails to create a meta data.
 */
protected void createMetaData() throws DataModelException {
//FIXOBSOLETE
//	((DataSourceTreeModel)getMetaDataTree()).setRoot(null);
//    ((DataSourceTreeModel)getDataTableTree()).setRoot(null);
    loadModelData(comp.getRoot(), null);
}

/** Reads the parameter values from the HTML page using the tree data Bean's applet.
 * The values will override those previously set.
 * Subclasses may override this method to set their own values; in this case,
 * the method should first call <code>super.getParameters()</code>.
 */
protected void getParameters() {
    JCTreeDataConverter.getParams(this);
}

/** Gets a paramater for this tree data Bean.
 * @return
 * @param param
 */
String getParam(String param) {
//FIXOBSOLETE
//	return conv.getParam(applet, component, getName(), param);
    return null;
}

/** Helper method called recursively that creates meta data.
 *	@exception DataModelException If fails to create a meta data.
 * @param node
 * @param meta_parent
 */
void loadModelData(TreeNode node, MetaDataModel meta_parent) throws DataModelException {
    if (node == null || ((DataSourceTreeNode)node).getUserObject() == null) return;

    NodeProperties data = (NodeProperties) ((DataSourceTreeNode)node).getUserObject();
    MetaDataModel meta = null;
    meta = metaDataCreator.createMetaData(this, meta_parent, data);
/*
TreeIterator ti = node.getIterator();
while (ti.hasMoreElements()) {
    TreeNode child = (TreeNode) ti.nextElement();
    loadModelData(child, meta);
}
*/
    for (Enumeration e = node.children(); e.hasMoreElements();) {
        loadModelData((TreeNode) e.nextElement(), meta);
    }
}

private void readObject(java.io.ObjectInputStream in) throws java.io.IOException {
    try {
        in.defaultReadObject();
        if (!DataSourceBeanInstance.getInstances().contains(this)) {
            DataSourceBeanInstance.addInstance(this);
        }
    }
    catch (Exception e) {
        e.printStackTrace(System.out);
    }
}

}
