/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TreeProperties.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:10 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.beans;

import com.klg.jclass.datasource.*;

import java.beans.Beans;
import java.io.*;
import java.util.Enumeration;

import javax.swing.tree.*;

import com.klg.jclass.datasource.*;
import com.klg.jclass.util.JCEnvironment;

/**
 *	This class encapsulates the data hierarchy for a <code>JCTreeData</code>.  This is a tree
 *	of <code>NodeProperties</code>.
 *	@see com.klg.jclass.datasource.beans.JCTreeData
 *	@see com.klg.jclass.datasource.beans.NodeProperties
 */
public class TreeProperties extends SerializedProperties implements
    JCDataListener {

static final long serialVersionUID = 1589381497924439329L;

/** Data source type of the tree data bean. */
protected int dataSourceType = NodeProperties.JDBC;

/** DataSourceTreeModel for NodeProperties */
private DataSourceTreeModel tree;

private transient Object[] dataSources;
private transient String[] dataSourceNames;
/** data model name used in data-binding editor. */
private String modelName;

/** Serialized version of the a bean component currently being deserialized. */
private static transient String serializedVersion = new String("");
private final static boolean TRACE = false;
private static final String base = "jcdbtree";
private static int counter = 0;

public TreeProperties() {
    this(NodeProperties.JDBC, null);
}

public TreeProperties(int type, String model_name) {
// should be new NodeProperties, with null userObject
    tree = new DataSourceTreeModel(DataSourceTreeModel.NullRoot);
    dataSourceType = type;
    setDefaultSerializationFileBase(base);
    serializationFile = getDefaultSerializationFile(base, counter);
    // update counter
    try {
        String s = serializationFile.substring(serializationFile.indexOf(base)+base.length());
        s = s.substring(0, s.indexOf(LocaleBundle.serialization_extension));
        counter = Integer.parseInt(s);
    }
    catch (Exception ex) {
    }
    this.modelName = model_name;
}

/** Constructs a TreeProperties from a resource. First,
 *	the package name of the caller class is prepended to the resource
 *	name to locate the resource.  If no resource is found, the passed-in
 *	resource name is used.
 *	@param resource_name resource name relative to a class loader
 *						to the bean component. It must have the right extension.
 *	@param caller_class class that uses the serialized bean component.
 *	@param sfile serialization file that is used at design-time in JBuilder to
 *			load a TreeProperties.
 *	@see java.lang.ClassLoader
 *      @exception DataModelException If the class can't be loaded.
 */
public TreeProperties(String resource_name, Class caller_class, String sfile)
throws DataModelException {
    this(resource_name, caller_class, sfile, null, null);
}

/** Constructs a TreeProperties from a resource. First,
 *	the package name of the caller class is prepended to the resource
 *	name to locate the resource.  If no resource is found, the passed-in
 *	resource name is used.
 *	@param resource_name resource name relative to a class loader
 *						to the bean component. It must have the right extension.
 *	@param caller_class class that uses the serialized bean component.
 *	@param sfile serialization file that is used at design-time in JBuilder to
 *			load a TreeProperties.
 *	@see java.lang.ClassLoader
 *      @exception DataModelException If the class can't be loaded.
 */
public TreeProperties(String resource_name, Class caller_class, String sfile,
                      String[] names, Object[] data_sources) throws DataModelException {
    try {
        tree = new DataSourceTreeModel(DataSourceTreeModel.NullRoot);
//FIXOBSOLETE
/*
    if (JCEnvironment.isJBuilder() && JCEnvironment.isDesignTime()) {
        TreeProperties comp = (TreeProperties)
                    TreeProperties.fromFile(sfile);
        setRoot(comp.getRoot());
        setSerializationFile(comp.getSerializationFile());
        setResourceName(comp.getResourceName());
        setModelName(comp.getModelName());
        setDataSourceType(comp.getDataSourceType());
    } else if (resource_name != null) {
*/
        if (resource_name != null) {
            TreeProperties comp = (TreeProperties)
                                  TreeProperties.fromResource(caller_class, resource_name);
            setRoot(comp.getRoot());
            setSerializationFile(comp.getSerializationFile());
            setResourceName(comp.getResourceName());
            setModelName(comp.getModelName());
            setDataSourceType(comp.getDataSourceType());
        }
        else {
            serializationFile = base + counter++ + LocaleBundle.serialization_extension;
        }
        dataSources = data_sources;
        dataSourceNames = names;
    }
    catch (Exception ex) {
        throw new DataModelException(DataModelEvent.INITIALIZE, ex);
    }
}

/** Constructs a TreeProperties from a resource. First,
 *	the package name of the caller class is prepended to the resource
 *	name to locate the resource.  If no resource is found, the passed-in
 *	resource name is used.
 *	@param resource_name resource name relative to a class loader
 *						to the bean component. It must have the right extension.
 *	@param caller_class class that uses the serialized bean component.
 *	@see java.lang.ClassLoader
 *      @exception DataModelException If the class can't be loaded.
 */
public TreeProperties(String resource_name, Class caller_class) throws
DataModelException {
    this(resource_name, caller_class, null, null);
}

/** Constructs a TreeProperties from a resource. First,
 *	the package name of the caller class is prepended to the resource
 *	name to locate the resource.  If no resource is found, the passed-in
 *	resource name is used.
 *	@param resource_name resource name relative to a class loader
 *						to the bean component. It must have the right extension.
 *	@param caller_class class that uses the serialized bean component.
 *	@see java.lang.ClassLoader
 *      @exception DataModelException If the class can't be loaded.
 */
public TreeProperties(String resource_name, Class caller_class,
                      String[] names, Object[] data_sources) throws DataModelException {
    try {
        tree = new DataSourceTreeModel(DataSourceTreeModel.NullRoot);
        if (resource_name != null) {
            TreeProperties comp = (TreeProperties)
                                  TreeProperties.fromResource(caller_class, resource_name);
            setRoot(comp.getRoot());
            setSerializationFile(comp.getSerializationFile());
            setResourceName(comp.getResourceName());
            setModelName(comp.getModelName());
            setDataSourceType(comp.getDataSourceType());
        }
        else {
            serializationFile = base + counter++ + LocaleBundle.serialization_extension;
        }
        dataSources = data_sources;
        dataSourceNames = names;
    }
    catch (Exception ex) {
        throw new DataModelException(DataModelEvent.INITIALIZE, ex);
    }
}

public TreeProperties(String resource_name, ClassLoader cl) throws
DataModelException {
    this(resource_name, cl, null, null);
}

public TreeProperties(String resource_name, ClassLoader cl,
                      String[] names, Object[] data_sources) throws DataModelException {
    try {
        tree = new DataSourceTreeModel(DataSourceTreeModel.NullRoot);
        if (resource_name != null) {
            TreeProperties comp = (TreeProperties)
                                  TreeProperties.fromResource(cl, resource_name);
            setRoot(comp.getRoot());
            setSerializationFile(comp.getSerializationFile());
            setResourceName(comp.getResourceName());
            setModelName(comp.getModelName());
            setDataSourceType(comp.getDataSourceType());
        }
        else {
            serializationFile = base + counter++ + LocaleBundle.serialization_extension;
        }
        dataSources = data_sources;
        dataSourceNames = names;
    }
    catch (Exception ex) {
        throw new DataModelException(DataModelEvent.INITIALIZE, ex);
    }
}

/** Sets the model name. */
public void setModelName(String s) {
    if (isSameString(modelName, s)) return;

    this.modelName = s;
    setChanged();
}

/** Sets the model name. */
public String getModelName() {
    return modelName;
}

/** Sets the data source type. */
public void setDataSourceType(int type) {
    if (dataSourceType == type) return;

    dataSourceType = type;
    setChanged();
}

/** Return the data source type. */
public int getDataSourceType() {
    return dataSourceType;
}

/** Return an array of the names of IDE data source objects. */
public String[] getDataSourceNames() {
    return dataSourceNames;
}

/** Return an array of IDE data source objects. */
public Object[] getDataSources() {
    return dataSources;
}

/** When value(s) of the data bean component is changed. */
public void JCDataValueChanged(JCDataEvent e) {
    // mark component as changed
    setChanged();
    // pass it on
    fireJCDataEvent(new JCDataEvent((Object)this, (NodeProperties) e.getSource()));
}

/** Instantiates a SerializedProperties from a resource.
 *	@see java.beans.Beans#instantiate
 *      @exception ClassNotFoundException If the class can't be loaded.
 *      @exception IOException If an IO exception occurs.
 */
public static SerializedProperties fromResource(ClassLoader cl, String name) throws
ClassNotFoundException, IOException {
    TreeProperties comp = (TreeProperties)
                          SerializedProperties.fromResource(cl, name);
    return comp;
}

//////////////////////////////////////////////////////////////
//
//    DataSourceTreeModel methods
//
/** Gets the root of the tree structure.
 */
public DataSourceTreeNode getRoot() {
    return(DataSourceTreeNode) tree.getRoot();
}

/** Sets the root of the tree structure.
 */
public void setRoot(DataSourceTreeNode  root) {
    tree.setRoot(root);
    setChanged();
}

/** Removes all nodes from the tree.
 */
public void removeAll() {
    tree.setRoot(null);
    setChanged();
}

/** Removes a node from the tree.
 */
public void remove(DataSourceTreeNode node) {
    DataSourceTreeNode parent = (DataSourceTreeNode)node.getParent();
    if (parent == null) {
        removeAll();
    }
    else {
        parent.remove(node);
    }
    setChanged();
}

/** Gets the current version.  */
public static String getVersion() {
    return 	JCVersion.getVersionNumber();
}


/** Gets the serialized version of the bean component currently being de-serialized.  */
public static synchronized String getSerializedVersion() {
    return serializedVersion;
}


/** Helper method to display the tree. */
private void buildTree(StringBuffer s, DataSourceTreeNode node, int level) {
    if (node == null) return;

    s.append("\n\tLEVEL: " + level++);
    s.append("\n");
    s.append(node);
    s.append("\n");
//	s.append(node.getData());
    s.append(((DataSourceTreeNode)node).getUserObject());
    s.append("\n");

    for (Enumeration e = node.children(); e.hasMoreElements();) {
        buildTree(s, (DataSourceTreeNode) e.nextElement(), level);
    }
/*
TreeIterator ti = node.getIterator();
while (ti.hasMoreElements()) {
    TreeNode child = (TreeNode) ti.nextElement();
    buildTree(s, child, level);
}
*/

}

public String toString() {
    String str = super.toString();
    if (TRACE) {
        StringBuffer s = new StringBuffer();
        s.append("\nSerialization file: ");
        s.append(serializationFile);
        s.append("\nModel Name: ");
        s.append(modelName);
        s.append("\n");
        // Build tree
        buildTree(s, getRoot(), 0);
        str = str + new String(s);
    }
    return str;
}

/** Override default write object to write the current version first. */
private void writeObject(java.io.ObjectOutputStream on) throws java.io.IOException {
    if (TRACE) {
        System.out.println("write version: " + getVersion());
    }
    // Write the current version
    on.writeObject(getVersion());
    try {
        on.defaultWriteObject();
    }
    catch (Exception e) {
        e.printStackTrace(System.out);
    }
}

/** Override default read object to read the serialized version first.
 *	Classes that changed between versions can compare the serialized
 *	version and the current version to see if compatibility needs to
 *	be handled.
 */
private void readObject(java.io.ObjectInputStream in) throws java.io.IOException {
    synchronized(serializedVersion) {
        try {
            // Get the version serialized.
            serializedVersion = (String) in.readObject();
            if (TRACE) {
                System.out.println("serialized version: " + serializedVersion);
            }
            in.defaultReadObject();
        }
        catch (Exception e) {
            e.printStackTrace(System.out);
        }
    }
}

}
