/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ConnectionPanel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:10 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.customizer;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.sql.Connection;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;

import com.klg.jclass.datasource.*;
import com.klg.jclass.datasource.jdbc.*;
import com.klg.jclass.datasource.beans.*;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.internal.JClassInfo;
import com.klg.jclass.util.swing.JCAlignLayout;


/*
 * This is the panel on which the user indicates to which data
 * source they would like to connect.  Depending on the type of
 * data source (jdbc or jdbc-odbc) and on driver specifics
 * the amount and type of information required will vary.
 */
public class ConnectionPanel extends DataModelPage implements ActionListener,
    ItemListener {

JCheckBox   useParentConnection,
promptUserForLogin,
autoCommit;
JTextField  hostField,          // tcpip host
portField;              // tcpip port number btw 1025-65535
JTextField  loginField,             // user id text entry field
passwordField,      // password text entry field
dbField;                // database name text entry field
JButton     btnConnect;         //  
JLabel      connectMsg;
JPanel      controlPanel;           // buttons are added to this panel 

JComboBox   urlList;                // show urls read from props file
JComboBox   driverList;
JPanel      serverPanel,            // server panel with border 
loginPanel;         // login panel with border

DataTableConnection dsConnection = new DataTableConnection();

JDBCPage jdbcPage = null;
private static String defaultDelimiter = "#!#*#";
private static String jdbcUrls = "jdbc.urls";
private static String jdbcUrlsDelimiter = "jdbc.urls.delimiter";
private static String jdbcDrivers = "jdbc.drivers";
private static String sunJdbcOdbcDriver = "sun.jdbc.odbc.JdbcOdbcDriver";
private static String msJdbcOdbcDriver = "com.ms.jdbc.odbc.JdbcOdbcDriver";

private static final boolean TRACE = false;

/**
 * Constructor for the ConnectionPanel.  Displays text boxes into
 * which user can enter their connection information.
 */
public ConnectionPanel(JDBCPage jdbcPage) {
    super(LocaleBundle.connection);

    this.jdbcPage = jdbcPage;

    setLayout(new BorderLayout());

    useParentConnection = new JCheckBox(
                                       LocaleBundle.string(LocaleBundle.use_parent_connection));
    autoCommit = new JCheckBox(LocaleBundle.string(LocaleBundle.auto_commit), true);
    useParentConnection.setEnabled(false);
    useParentConnection.addItemListener(this);
    autoCommit.setEnabled(true);
    autoCommit.addItemListener(this);

    // create the server panel
    JLabel lbl_server_name = new JLabel(LocaleBundle.string(LocaleBundle.server_name));
    JLabel lbl_host = new JLabel(LocaleBundle.string(LocaleBundle.host_or_ip));
    hostField = new JTextField("", 30);
    JLabel lbl_port = new JLabel(LocaleBundle.string(LocaleBundle.tcpip_port));
    portField = new JTextField("", 30);
    JLabel lbl_driver = new JLabel(LocaleBundle.string(LocaleBundle.driver));
    urlList = new JComboBox();
    driverList = new JComboBox();
    urlList.setEditable(true);
    driverList.setEditable(true);
    serverPanel = new JPanel();
    serverPanel.setBorder(new TitledBorder(LocaleBundle.string(LocaleBundle.server)));
    JCAlignLayout layout = new JCAlignLayout();
    serverPanel.setLayout(layout);
    serverPanel.add(lbl_server_name);
    serverPanel.add(urlList);
    layout.setResizeWidth(urlList, true);
    urlList.addActionListener(this);
    serverPanel.add(lbl_host);
    serverPanel.add(hostField);
    layout.setResizeWidth(hostField, true);
    hostField.addActionListener(this);
    serverPanel.add(lbl_port);
    serverPanel.add(portField);
    layout.setResizeWidth(portField, true);
    portField.addActionListener(this);
    serverPanel.add(lbl_driver);
    serverPanel.add(driverList);
    layout.setResizeWidth(driverList, true);
    driverList.addActionListener(this);


    try {
        Class.forName(sunJdbcOdbcDriver);
    }
    catch (Exception e) {
    }

    try {
        Class.forName(msJdbcOdbcDriver);
    }
    catch (Exception e) {
    }

    // force finding the jclass.properties file.
    JClassInfo.getProperty("datasource", jdbcDrivers);
    populate();

    // create the login panel
    JLabel lbl_login_name = new JLabel(LocaleBundle.string(LocaleBundle.login_name));
    loginField = new JTextField("", 30);
    JLabel lbl_password = new JLabel(LocaleBundle.string(LocaleBundle.password));
    passwordField = new JPasswordField("", 30);
    JLabel lbl_database = new JLabel(LocaleBundle.string(LocaleBundle.database));
    dbField = new JTextField("", 30);
    JLabel lbl = new JLabel("");
    promptUserForLogin = new JCheckBox(
                                      LocaleBundle.string(LocaleBundle.prompt_user_for_login));
    loginPanel = new JPanel();
    loginPanel.setBorder(new TitledBorder(LocaleBundle.string(LocaleBundle.login)));
    layout = new JCAlignLayout();
    loginPanel.setLayout(layout);
    loginPanel.add(lbl_login_name);
    loginPanel.add(loginField);
    layout.setResizeWidth(loginField, true);
    loginField.addActionListener(this);
    loginPanel.add(lbl_password);
    loginPanel.add(passwordField);
    layout.setResizeWidth(passwordField, true);
    passwordField.addActionListener(this);
    loginPanel.add(lbl_database);
    loginPanel.add(dbField);
    layout.setResizeWidth(dbField, true);
    dbField.addActionListener(this);
    loginPanel.add(lbl);
    promptUserForLogin.addItemListener(this);
    loginPanel.add(promptUserForLogin);

    // Create the control panel
    controlPanel = new JPanel();
    JPanel panel = new JPanel();
    btnConnect = new JButton(LocaleBundle.string(LocaleBundle.connect));
    connectMsg = new JLabel();
    connectMsg.setText(" ");    // SWING needs this for component to be laid out
    controlPanel.setLayout(new BorderLayout());
    panel.add(btnConnect);
    controlPanel.add(panel, BorderLayout.CENTER);
    controlPanel.add(connectMsg, BorderLayout.SOUTH);

    // add the button listeners
    btnConnect.addActionListener(this);

    // add all the panels
    JPanel p = new JPanel();
    JPanel checkbox_panel = new JPanel();
    p.setLayout(new GridLayout(2,1));
    p.add(serverPanel);
    p.add(loginPanel);
    checkbox_panel.setLayout(new GridBagLayout());
    GridBagConstraints c = new GridBagConstraints();
    c.fill = GridBagConstraints.NONE;
    c.anchor = GridBagConstraints.WEST;
    c.weightx = 5;
    c.gridy = 0;
    checkbox_panel.add(useParentConnection, c);
    c.gridy++;
    checkbox_panel.add(autoCommit, c);
    add(checkbox_panel, BorderLayout.NORTH);
    add(p, BorderLayout.CENTER);
    add(controlPanel, BorderLayout.SOUTH);
    com.klg.jclass.datasource.customizer.CustomizerLauncher.setDefaultFont((Component)this, new Font("Dialog", Font.PLAIN, 9));
}

void setUseConnectionPage(boolean v) {
    hostField.setEnabled(v);
    portField.setEnabled(v);
    urlList.setEnabled(v);
    driverList.setEnabled(v);
    loginField.setEnabled(v);
    passwordField.setEnabled(v);
    dbField.setEnabled(v);
    promptUserForLogin.setEnabled(v);
}

public void setUseInHierarchy(boolean v) {
    useParentConnection.setEnabled(v);
    if (v) {
        setUseConnectionPage(useParentConnection.isSelected());
    }
}

private void connectionInfoChanged() {
    // Change the comp and notify observers its change
    try {
        getNodeProperties().setJDBCInfo(getDriver(), getURL(), getUser(), getPassword(), getDatabase());
    }
    catch (Exception e) {
        ErrorDialog.showError(this, e.toString());
    }
}

/** Handle text fields. */
public void actionPerformed(ActionEvent event) {
    if (isInitializing()) return;

    if (event.getSource() instanceof JComboBox) {
        return;
    }

    connectMsg.setText(""); // Clear status message

    if (getNodeProperties() == null) return;

    if (event.getSource() == btnConnect) {
        connectMsg.setText(LocaleBundle.string(LocaleBundle.connecting));
    }

    connectionInfoChanged();
    String msg = "";
    if (event.getSource() instanceof JButton &&
        event.getSource().equals(btnConnect)) {
        Connection c = null;
        try {
            DataTableConnection dtc = getNodeProperties().getConnection();
            c = (dtc == null) ? null : dtc.getConnection();
        }
        catch (Exception e) {
            ErrorDialog.showError(this, e.toString());
        }
        msg = (c == null) ? 
              LocaleBundle.string(LocaleBundle.connect_failed) :
              LocaleBundle.string(LocaleBundle.connect_succeeded);

        if (c != null) {
            // Update property file.
            updatePropertyFile();
        }
        else {
            getNodeProperties().resetConnection();
        }
    }
    getNodeProperties().fireJCDataEvent(new JCDataEvent((Object)getNodeProperties()));
    connectMsg.setText(msg);
}

/** use parent connection is selected/deselected. */
public void itemStateChanged(ItemEvent e) {
    if (isInitializing()) return;

    if (e.getSource() instanceof JCheckBox) {
        JCheckBox cb = (JCheckBox) e.getSource();
        boolean selected = cb.isSelected();
        if (cb.equals(useParentConnection)) {
            // If use parent connection, don't let user enter connection info
            setUseConnectionPage(!selected);
            // Change the comp and notify observers its change
            getNodeProperties().setUseParentConnection(selected);
            connectionInfoChanged();
            getNodeProperties().fireJCDataEvent(new JCDataEvent((Object)getNodeProperties()));
        }
        else if (cb.equals(promptUserForLogin)) {
            // Change the comp and notify observers its change
            getNodeProperties().setPromptUserForLogin(selected);
            connectionInfoChanged();
            getNodeProperties().fireJCDataEvent(new JCDataEvent((Object)getNodeProperties()));
        }
        else if (cb.equals(autoCommit)) {
            // Change the comp and notify observers its change
            getNodeProperties().setAutoCommit(selected);
            connectionInfoChanged();
            getNodeProperties().fireJCDataEvent(new JCDataEvent((Object)getNodeProperties()));
        }
    }
}

/** Displays values in the bean data comp. */
public void setNodeProperties(NodeProperties m) {
    connectMsg.setText(""); // Clear status message

    urlList.setSelectedItem(m.getURL());
    driverList.setSelectedItem(m.getDriver());
    loginField.setText(m.getUser());
    passwordField.setText(m.getPassword());
    dbField.setText(m.getDatabase());
    setUseConnectionPage(!m.getUseParentConnection());
    useParentConnection.setSelected(m.getUseParentConnection());
    autoCommit.setSelected(m.getAutoCommit());
    promptUserForLogin.setSelected(m.getPromptUserForLogin());
}

public String getURL() {
    // create url of the form 
    // jdbc:ff-sybase://<host>:<port>[/database name]
    String url = (String) urlList.getSelectedItem();

    // see if they are including the host field; if they are
    // then they must include the port too.
    if (hostField.getText().length() > 0) {
        url += hostField.getText();
        url += ":";
        url += portField.getText();
    }
    return url;
}

public String getUser() {
    return loginField.getText();
}

public String getDriver() {
    return(String) driverList.getSelectedItem();
}

public String getPassword() {
    return passwordField.getText();
}

public String getDatabase() {
    return dbField.getText();
}

/**
 * Read predefined urls and drivers from the properties file 
 */
void populate() {
    String drivers = JClassInfo.getProperty("datasource", jdbcDrivers);
    String urls = JClassInfo.getProperty("datasource", jdbcUrls);
    String delimiter = JClassInfo.getProperty("datasource", jdbcUrlsDelimiter);

    // If no drivers property, add the default Sun and Microsoft drivers.
    if (drivers == null || drivers.length() == 0) {
        drivers = sunJdbcOdbcDriver + ":" + msJdbcOdbcDriver;
        JClassInfo.setProperty("datasource", jdbcDrivers, drivers);
    }
    drivers += (System.getProperty(jdbcDrivers) == null) ? 
               "" : (":" + System.getProperty(jdbcDrivers));

    // If no delimiter property, default and save it to the property file.
    if (delimiter == null || delimiter.length() == 0) {
        delimiter = defaultDelimiter;
        JClassInfo.setProperty("datasource", jdbcUrlsDelimiter, delimiter);
    }

    StringTokenizer st = null;
    if (urls != null && urls.length() > 0) {
        // Populate URL list
        st = new StringTokenizer(urls, delimiter);
        while (st.hasMoreTokens()) {
            String url = st.nextToken().trim();
            if (url.length() > 0) {
                boolean exists = false;
                for (int i=0; i<urlList.getModel().getSize(); i++) {
                    String s = (String) urlList.getModel().getElementAt(i);
                    if (s.equals(url)) {
                        exists = true;
                        break;
                    }
                }
                if (!exists) urlList.addItem(url);
            }
        }
    }

    if (drivers != null && drivers.length() > 0) {
        // Populate driver list
        st = new StringTokenizer(drivers, ":");
        while (st.hasMoreTokens()) {
            String driver = st.nextToken().trim();
            if (driver.length() > 0) {
                boolean exists = false;
                for (int i=0; i<driverList.getModel().getSize(); i++) {
                    String s = (String) driverList.getModel().getElementAt(i);
                    if (s.equals(driver)) {
                        exists = true;
                        break;
                    }
                }
                if (!exists) driverList.addItem(driver);
            }
        }
    }
}

/** Add URL and driver to the property file. */
void updatePropertyFile() {
    String drivers = JClassInfo.getProperty("datasource", jdbcDrivers);
    String urls = JClassInfo.getProperty("datasource", jdbcUrls);
    String delimiter = JClassInfo.getProperty("datasource", jdbcUrlsDelimiter);

    if (drivers == null) {
        drivers = "";
    }
    if (urls == null) {
        urls = "";
    }
    if (delimiter == null) {
        delimiter = defaultDelimiter;
    }

    // Add driver
    if (getDriver() != null && getDriver().trim().length() > 0) {
        boolean exists = false;
        StringTokenizer st = new StringTokenizer(drivers, ":");
        while (st.hasMoreTokens()) {
            String driver = st.nextToken().trim();
            if (driver.equals(getDriver().trim())) {
                exists = true;
                break;
            }
        }
        if (!exists) {
            if (drivers.length() > 0) {
                drivers += ":";
            }
            drivers += getDriver().trim();
        }
    }

    // Add url
    if (getURL() != null && getURL().trim().length() > 0) {
        boolean exists = false;
        StringTokenizer st = new StringTokenizer(urls, delimiter);
        while (st.hasMoreTokens()) {
            String url = st.nextToken().trim();
            if (url.equals(getURL().trim())) {
                exists = true;
                break;
            }
        }
        if (!exists) {
            if (urls.length() > 0) {
                urls += delimiter;
            }
            urls += getURL().trim();
        }
    }

    if (drivers != null && drivers.trim().length() > 0) {
        JClassInfo.setProperty("datasource", jdbcDrivers, drivers);
    }
    if (urls != null && urls.trim().length() > 0) {
        JClassInfo.setProperty("datasource", jdbcUrls, urls);
    }

    populate();
}


}
