/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CustomizerLauncher.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:10 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.customizer;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;

import javax.swing.*;
import javax.swing.border.TitledBorder;

import com.klg.jclass.datasource.LocaleBundle;

/** Launches a customizer in a dialog so changes to a bean made on the 
 * 	customizer can be displayed.  
 */
public class CustomizerLauncher extends JFrame implements ActionListener {

JButton btnLaunch;
JDialog dialog;
Customizer customizer;


/**	Creates a customizer launcher for a customizer and a bean.
 *	@param title title of the customizer launcher dialog
 *	@param customizer customizer to be launched.  It must be an instance of java.awt.Component.
 *	@param bean bean object used by the customizer
 */
public CustomizerLauncher(String title, Customizer customizer, JComponent bean) {
    this(title, customizer, bean, bean);
}

/**	Creates a customizer launcher for a customizer and a bean.
 *	@param title title of the customizer launcher dialog
 *	@param customizer customizer to be launched.  It must be an instance of java.awt.Component
 *	@param bean bean object used by the customizer
 *	@param display a component used to display the changes made to the bean by the customizer.  It can be the bean itself if the bean is an instanceof java.awt.Component
 */
public CustomizerLauncher(String title, Customizer customizer, Object bean, JComponent display) {
    super(title);
    Beans.setDesignTime(true);
    this.customizer = customizer;
    customizer.setObject(bean);

    getContentPane().setLayout(new BorderLayout());
    btnLaunch = new JButton(LocaleBundle.string(LocaleBundle.launch_customizer));
    if (display != null) {
        getContentPane().add(display, BorderLayout.CENTER);
    }
    getContentPane().add(btnLaunch, BorderLayout.SOUTH);
    btnLaunch.addActionListener(this);
    //FIX before release
    actionPerformed(new ActionEvent(btnLaunch, 0, "quick"));
    addWindowListener(new java.awt.event.WindowAdapter() {
                          public void windowClosing(java.awt.event.WindowEvent e) {
                              System.exit(0);
                          }
                      });
}

/** Launch the customizer. */
public void actionPerformed(ActionEvent e) {
    btnLaunch.setEnabled(false);
    dialog = new JDialog(this, LocaleBundle.string(LocaleBundle.customizer));
    dialog.getContentPane().setLayout(new BorderLayout());
    if (customizer instanceof JComponent) {
        dialog.getContentPane().add((JComponent) customizer, BorderLayout.CENTER);          
    }
    // Handle window close requests
    dialog.addWindowListener(new java.awt.event.WindowAdapter() {
                                 public void windowClosing(java.awt.event.WindowEvent e) {
                                     dialog.dispose(); 
                                     btnLaunch.setEnabled(true);
                                 }
                             });
    // set it to non-modal.  Otherwise, error dialogs show up
    // behind the customizer on Solaris.
    dialog.setModal(false);
    dialog.pack();
    centerDialog(dialog);
    dialog.show();  
}

/** Set the default font for the given component and its children. */
public static void setDefaultFont(Component comp, Font font) {
    if (comp == null || font == null) {
        return;
    }
    if (comp instanceof Container) {
        comp.setFont(font);
        int count = ((Container)comp).getComponentCount();
        for (int i = 0; i < count; i++) {
            setDefaultFont(((Container)comp).getComponent(i), font);
        }
    }
    else {
        comp.setFont(font);
    }
    if (comp instanceof JComponent && ((JComponent)comp).getBorder() instanceof TitledBorder) {
        ((TitledBorder)((JComponent)comp).getBorder()).setTitleFont(font);
    }
    if (comp instanceof JButton) {
        ((JButton)comp).setMargin(new java.awt.Insets(2,2,2,2));
    }
}

/** 
 * Centers the dialog relative to the parent frame or screen.
 */
void centerDialog(Component dialog) {
    int x = 0;
    int y = 0;
    Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
    x = (screenDim.width - dialog.getSize().width) / 2;
    y = (screenDim.height - dialog.getSize().height) / 2;
    dialog.setLocation(x, y);
}

}
