/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataAccessPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:10 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.customizer;

import java.awt.*;
import java.awt.event.*;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.*;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.*;

import com.klg.jclass.beans.EnumEditor;
import com.klg.jclass.datasource.*;
import com.klg.jclass.datasource.beans.*;
import com.klg.jclass.datasource.LocaleBundle;
import com.klg.jclass.datasource.util.*;
import com.klg.jclass.util.swing.JCAlignLayout;

/** This page is used to modify ShowDeletedRows, CommitPolicy, AutoCommit,
 * 	InsertAllowed, UpdateAllowed and DeleteAllowed properties of a data component.
 */
public class DataAccessPage extends DataModelPage implements ItemListener,
    ActionListener {

JCheckBox insertAllowed, updateAllowed, deleteAllowed, showDeletedRows, cacheChildren;
JButton btnAdd, btnModify, btnDelete;
JList tableAccess;
DefaultListModel listModel;
JComboBox table;
CustomizeEnumEditor commitPolicy;
EnumEditor commitPolicyEditor;
boolean useInHierarchy;

static final String delimiter = ";";

public DataAccessPage() {
    this(false);
}

public DataAccessPage(boolean use_in_hierarchy) {
    super(LocaleBundle.string(LocaleBundle.data_access));
    this.useInHierarchy = use_in_hierarchy;
    init();
}

public void init() {
    setLayout(new BorderLayout());

    // Create cache children
    JPanel checkbox_panel = new JPanel();
    cacheChildren = new JCheckBox(
                                 LocaleBundle.string(LocaleBundle.cache_children), false);
    JLabel lbl_commit_policy = new JLabel(LocaleBundle.string(LocaleBundle.commit_policy));
    JLabel lbl = new JLabel("");
    JLabel lbl2 = new JLabel("");
    JLabel lbl3 = new JLabel("");
    JLabel lbl4 = new JLabel("");
    showDeletedRows = new JCheckBox(
                                   LocaleBundle.string(LocaleBundle.show_deleted_rows), true);
    JPanel field_panel = new JPanel();
    JLabel lbl_table = new JLabel(LocaleBundle.string(LocaleBundle.table));
    insertAllowed = new JCheckBox(LocaleBundle.string(LocaleBundle.insert_allowed), true);
    updateAllowed = new JCheckBox(LocaleBundle.string(LocaleBundle.update_allowed), true);
    deleteAllowed = new JCheckBox(LocaleBundle.string(LocaleBundle.delete_allowed), true);
    JPanel btn_panel = new JPanel();
    btnAdd = new JButton(LocaleBundle.string(LocaleBundle.add));
    btnModify = new JButton(LocaleBundle.string(LocaleBundle.modify));
    btnDelete = new JButton(LocaleBundle.string(LocaleBundle.delete));
    JPanel p = new JPanel();
    listModel = new DefaultListModel();
    tableAccess = new JList(listModel);
    JPanel main = new JPanel(); 
    table = new JComboBox();
    JPanel commit_panel = new JPanel();
    commit_panel.setBorder(new EtchedBorder());
    JPanel access_box = new JPanel();
    access_box.setBorder(new TitledBorder(LocaleBundle.string(LocaleBundle.table_access)));

    // Build commit panel
    commit_panel.setLayout(new JCAlignLayout());
    commit_panel.add(lbl_commit_policy);
    if (useInHierarchy) {
        commitPolicyEditor = new CommitPolicyEditor();
    }
    else {
        commitPolicyEditor = new SingleLevelCommitPolicyEditor();
    }
    commit_panel.add(commitPolicy = new CustomizeEnumEditor(commitPolicyEditor));
    commitPolicyEditor.setValue(new Integer(MetaDataModel.COMMIT_LEAVING_RECORD));
    commit_panel.add(lbl);
    commit_panel.add(showDeletedRows);

    // Build table access panel
    field_panel.setLayout(new JCAlignLayout());
    field_panel.add(lbl_table);
    field_panel.add(table);
    field_panel.add(lbl2);
    field_panel.add(insertAllowed);
    field_panel.add(lbl3);
    field_panel.add(updateAllowed);
    field_panel.add(lbl4);
    field_panel.add(deleteAllowed);

    btn_panel.setLayout(new GridBagLayout());
    GridBagConstraints c = new GridBagConstraints();
    c.anchor = GridBagConstraints.WEST;
    c.fill = GridBagConstraints.HORIZONTAL;
    c.insets = new Insets(5, 5, 5, 0);
    btn_panel.add(btnAdd, c);
    btn_panel.add(btnModify, c);
    c.fill = GridBagConstraints.NONE;
    c.weightx = 5;
    btn_panel.add(btnDelete, c);

    p.setLayout(new BorderLayout());
    p.add(field_panel, BorderLayout.CENTER);
    p.add(btn_panel, BorderLayout.SOUTH);
    access_box.setLayout(new BorderLayout());
    access_box.add(new JScrollPane(tableAccess), BorderLayout.CENTER);
    access_box.add(p, BorderLayout.SOUTH);

    main.setLayout(new BorderLayout());
    main.add(commit_panel, BorderLayout.NORTH);
    main.add(access_box, BorderLayout.CENTER);
    checkbox_panel.setLayout(new GridBagLayout());
    c = new GridBagConstraints();
    c.fill = GridBagConstraints.NONE;
    c.anchor = GridBagConstraints.WEST;
    c.weightx = 5;
    checkbox_panel.add(cacheChildren, c);
    add(checkbox_panel, BorderLayout.NORTH);
    add(main, BorderLayout.CENTER);

    cacheChildren.addItemListener(this);
    btnAdd.addActionListener(this);
    btnModify.addActionListener(this);
    btnDelete.addActionListener(this);
    showDeletedRows.addItemListener(this);
    commitPolicy.addItemListener(this);

    tableAccess.addListSelectionListener(new ListSelectionListener() {
                                             public void valueChanged(ListSelectionEvent e) {
                                                 if (isInitializing()) return;

                                                 if (e.getValueIsAdjusting()) {
                                                     return;
                                                 }
                                                 // Table access selected.  Show setting in fields.
                                                 showCurrentTableAccess();
                                             }
                                         });
}

/** Displays values in the bean data comp. */
public void setNodeProperties(NodeProperties comp) {
    showTableAccess();
    cacheChildren.setSelected(comp.getCacheChildren());
    showDeletedRows.setSelected(comp.getShowDeletedRows());
    commitPolicyEditor.setValue(new Integer(comp.getCommitPolicy()));
    commitPolicy.repaint();

    showDeletedRows.setEnabled(comp.getCommitPolicy() != MetaDataModel.COMMIT_LEAVING_RECORD);
}

/** Data source type is changed. */
public void itemStateChanged(ItemEvent e) {
    if (isInitializing()) return;

    NodeProperties comp = getNodeProperties();
    showDeletedRows.setEnabled(comp.getCommitPolicy() != MetaDataModel.COMMIT_LEAVING_RECORD);
    if (e.getSource() instanceof JList) {
        // Table access selected.  Show setting in fields.
        showCurrentTableAccess();
    }
    else {
        comp.setCacheChildren(cacheChildren.isSelected());
        comp.setShowDeletedRows(showDeletedRows.isSelected());
        comp.setCommitPolicy(((Integer)commitPolicyEditor.getValue()).intValue());
        comp.fireJCDataEvent(new JCDataEvent((Object)comp));
    }

    if (comp.getCommitPolicy() == MetaDataModel.COMMIT_LEAVING_RECORD) {
        comp.setShowDeletedRows(true);
        comp.fireJCDataEvent(new JCDataEvent((Object)comp));
    }
}

public void actionPerformed(ActionEvent e) {
    if (isInitializing()) return;

    if (e.getSource() instanceof JButton) {
        JButton btn = (JButton) e.getSource();
        if (btn.equals(btnAdd)) {
            addTableAccess();
        }
        else if (btn.equals(btnModify)) {
            modifyTableAccess();
        }
        else if (btn.equals(btnDelete)) {
            removeTableAccess();
        }
    }
}


void addTableAccess() {
    if (getNodeProperties().getTableAccess().containsKey(table.getSelectedItem())) {
        return;
    }

    setTableAccess((String) table.getSelectedItem(), 
                   insertAllowed.isSelected(),
                   updateAllowed.isSelected(),
                   deleteAllowed.isSelected());
    getNodeProperties().fireJCDataEvent(new JCDataEvent((Object)getNodeProperties()));
}

String getSelectedTable() {
    String selected = null;
    selected = (String) tableAccess.getSelectedValue();
    String table = selected.substring(0, selected.indexOf(delimiter));
    return table;
}


void modifyTableAccess() {
    if (tableAccess.getSelectedIndex() == -1) {
        return;
    }
    String old_table = getSelectedTable();
    getNodeProperties().removeTableAccess(old_table);
    setTableAccess((String) table.getSelectedItem(),
                   insertAllowed.isSelected(),
                   updateAllowed.isSelected(),
                   deleteAllowed.isSelected());
    getNodeProperties().fireJCDataEvent(new JCDataEvent((Object)getNodeProperties()));
}

void removeTableAccess() {
    if (tableAccess.getSelectedIndex() == -1) {
        return;
    }

    String table = getSelectedTable();
    getNodeProperties().removeTableAccess(table);
    getNodeProperties().fireJCDataEvent(new JCDataEvent((Object)getNodeProperties()));
}

void setTableAccess(String table, 
                    boolean is_insert_allowed, 
                    boolean is_update_allowed,
                    boolean is_delete_allowed) {
    Boolean[] access = new Boolean[] { new Boolean(is_insert_allowed), 
        new Boolean(is_update_allowed),
        new Boolean(is_delete_allowed)};
    getNodeProperties().setTableAccess(table, access);
}


/** Load the table access into the page.
 */
void showTableAccess() {
    // Clear fields
    this.table.setSelectedItem("");
    listModel.removeAllElements();
    insertAllowed.setSelected(true);
    updateAllowed.setSelected(true);
    deleteAllowed.setSelected(true);

    // Load tables for JDBC data source type.
    if (getNodeProperties().getDataSourceType() == NodeProperties.JDBC) {
    	Vector v = new Vector();
        if (getNodeProperties().getStatement() != null) {
        	if (getNodeProperties().getStatement().length() > 0) {
        		SqlParser parser = new SqlParser(getNodeProperties().getStatement());
        		for (Enumeration e = parser.getTables().elements(); e.hasMoreElements();) {
        			String t = new String(((SqlStatementTable) e.nextElement()).getAlias());
        			v.addElement(t);
        		}
        	}
        }
    	// overwrite previous model with new one
    	this.table.setModel(new DefaultComboBoxModel(v));
    }
		
    // Load table access
    if (getNodeProperties().getTableAccess().keys() == null) return;
    for (Enumeration e = getNodeProperties().getTableAccess().keys(); e.hasMoreElements();) {
        String table = (String) e.nextElement();
        Boolean[] access = (Boolean[]) getNodeProperties().getTableAccess().get(table);
        String insert = (access[NodeProperties.INSERT_ALLOWED_INDEX].booleanValue()) ? 
                        LocaleBundle.string(LocaleBundle.insert_allowed) : 
                        LocaleBundle.string(LocaleBundle.insert_not_allowed);
        String update = (access[NodeProperties.UPDATE_ALLOWED_INDEX].booleanValue()) ? 
                        LocaleBundle.string(LocaleBundle.update_allowed) : 
                        LocaleBundle.string(LocaleBundle.update_not_allowed);
        String delete = (access[NodeProperties.DELETE_ALLOWED_INDEX].booleanValue()) ? 
                        LocaleBundle.string(LocaleBundle.delete_allowed) : 
                        LocaleBundle.string(LocaleBundle.delete_not_allowed);
        listModel.addElement(table + delimiter + 
                             insert + delimiter + 
                             update + delimiter +
                             delete);
    }

    if (listModel.getSize() > 0) {
        tableAccess.setSelectedIndex(0);
        showCurrentTableAccess();
    }
}

/** Show the access for the selected table.
 */
void showCurrentTableAccess() {
    if (tableAccess.getSelectedIndex() == -1) {
        return;
    }

    String table = getSelectedTable();
    String selected = (String) tableAccess.getSelectedValue();
    String temp = selected.substring(selected.indexOf(delimiter)+1);
    String insert = temp.substring(0, temp.indexOf(delimiter));
    temp = temp.substring(temp.indexOf(delimiter)+1);
    String update = temp.substring(0, temp.indexOf(delimiter));
    String delete = temp.substring(temp.indexOf(delimiter)+1);

    this.table.setSelectedItem(table);
    insertAllowed.setSelected(insert.equals(LocaleBundle.string(LocaleBundle.insert_allowed)));
    updateAllowed.setSelected(update.equals(LocaleBundle.string(LocaleBundle.update_allowed)));
    deleteAllowed.setSelected(delete.equals(LocaleBundle.string(LocaleBundle.delete_allowed)));
}


} // DataAccessPage
