/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataBindingEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:10 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.customizer;

import java.awt.*;
import java.beans.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import com.klg.jclass.datasource.*;
import com.klg.jclass.datasource.beans.DataSourceBeanInstance;
import com.klg.jclass.beans.PanelEditor;
import com.klg.jclass.datasource.util.DataBinding;

/**
 * Editor for JClass data binding property.
 */
public class DataBindingEditor extends PanelEditor implements
    TreeSelectionListener {

protected JTree outliner = null;
protected boolean topLevelOnly = false;
protected boolean allowLeafsOnly = false;

/**
 * Creates a DataBindingEditor
 */
public DataBindingEditor() {
    this(false, false);
}

/**
 * Creates a DataBindingEditor.
 *	@param root_level_only only shows the top level
 *	of all nodes.
 */
public DataBindingEditor(boolean topLevelOnly, boolean allowLeafsOnly) {
    super();
    this.topLevelOnly = topLevelOnly;
    this.allowLeafsOnly = allowLeafsOnly;

    outliner = new JTree(getTree());
    outliner.addTreeSelectionListener(this);
    outliner.setVisible(true);

    setLayout(new BorderLayout());
    add(new JScrollPane(outliner), BorderLayout.CENTER);
}

static Dimension ourSize = new Dimension(300, 300);
public Dimension getPreferredSize() {
    return ourSize;
}

/**
 * This function exists to be over-ridden by subclasses without them having to
 * override the behviours carried out in the addChildren() method below
 * It allows the specific implementation to decide what is considered a
 * child
 */
protected boolean hasChildren(MetaDataModel metaDataModel) {
    return metaDataModel.getChildCount() > 0;
}

protected void addChildren(MutableTreeNode parent,
                           MetaDataModel metaDataModel) {
    Enumeration e = ((TreeNode)metaDataModel).children();
    while (e.hasMoreElements())
        addMetaDataLevel(parent, (MetaDataModel)e.nextElement(), null);
}

private MutableTreeNode getTree() {
    Vector list = DataSourceBeanInstance.getInstances();
    EditorTreeNode root = new EditorTreeNode(LocaleBundle.string(LocaleBundle.available_data_sources));

    for (int i = 0; i < list.size(); i++) {
        DataModel dm = (DataModel)(list.elementAt(i));
        if (dm != null) {
            TreeModel treeModel = (TreeModel)dm.getMetaDataTree();
            if (treeModel != null) {
                MetaDataModel metaDataModel =(MetaDataModel)treeModel.getRoot();
                addMetaDataLevel(root, metaDataModel, dm.getModelName());
            }
        }
    }

    return root;
}

private void addMetaDataLevel(MutableTreeNode folder,
                              MetaDataModel metaDataModel, String modelName) {
    // the end
    if (metaDataModel == null)
        return;

    try {
        // while there are more data controls
        String label = metaDataModel.getDescription();
        if (label == null || label.length() == 0) {
            label = "No Description";
        }
        if (modelName != null)
            label = modelName + DataBinding.SOURCE_DELIMITER + label;
        EditorTreeNode newItem = null;
        if (hasChildren(metaDataModel) && !topLevelOnly) {
            newItem = new EditorTreeNode(label);
        }
        else
            newItem = new EditorTreeNode(label);
        folder.insert(newItem, folder.getChildCount());

        if (hasChildren(metaDataModel) && !topLevelOnly) {
            addChildren(newItem, metaDataModel);
        }
    }
    catch (Exception e) {
        e.printStackTrace();
    }
}

/*
 * TreeSelectionListener implementation
 */
public void valueChanged(TreeSelectionEvent e) {
    support.firePropertyChange("dataBinding", null, getValue());
}


/**
 * Returns the value of the DataBinding property.
 *
 * DataBinding property specifies a string, describing a JCTreeData instance and
 * metadata level of the instance this table is bound with. The format of the property
 * is following:
 *   <JCTreeData name>:<MetaDataModel description>[|<MetaDataModel description>...]
 *
 * @see #setDataBindingHierarchy
 * @see #setValue
 */
public Object getValue() {
    return getDataBindingHierarchy();
}

/**
 * Sets the DataBinding property value to the editor. Highlights the selected level in
 * the data binding hierarchy tree.
 *
 * @see #getValue
 */
public void setValue(Object value) {
    refreshTree();
    setDataBindingHierarchy((String)value);
}

private void refreshTree() {
}

/**
 * Return a string representation of the property for display.
 */
public String getAsText() {
    String s = getDataBindingHierarchy();
    if (s == null || s.equals("")) {
        s = new String(LocaleBundle.string(LocaleBundle.select_a_data_source));
    }
    return s;
}

// Some IDEs like BeanBox have to use paintValue() to display the value as text.
public void paintValue(Graphics gc, Rectangle box) {
    String s = getAsText();
    FontMetrics fm = gc.getFontMetrics();
    int vpad = (box.height - fm.getAscent())/2;
    gc.drawString(s, box.x, box.y+box.height-vpad);
}

/**
 * Returns a Java initialization string that should be used by an IDE to
 * compose a source code line for the property
 */
public String getJavaInitializationString() {
    return "\""+getDataBindingHierarchy()+"\"";
}

protected String getNextPart(TreePath path, int i) {
    return (String)((EditorTreeNode)path.getPathComponent(i)).getUserObject();
}

/**
 * Returns the data binding hierarchy string, starting from a JCTreeData instance name
 * and containing full sequence of MetaDataModel levels starting from the root to the level
 * the component should be bound with.
 *
 * @see #setDataBindingHierarchy
 */
protected String getDataBindingHierarchy() {
    String s = "";
    TreePath path = outliner.getSelectionPath();
    if (path == null) {
        return s;
    }
    TreeNode lastNode = (TreeNode)path.getLastPathComponent();
    if (lastNode == null) {
       return s;
    }
    if (allowLeafsOnly && (!lastNode.isLeaf())) {
       return s;
    }
    for (int i = path.getPathCount() - 1; i > 0; i--) {
        String nextPart = getNextPart(path, i);
        if (s.length() == 0) {
            s = nextPart;
        }
        else {
            s = nextPart + DataBinding.PATH_DELIMITER + s;
        }
    }
    return s;
}

/**
 * Select a tree node for the data binding hierarchy string, starting from a JCTreeData
 * instance name and containing full sequence of MetaDataModel levels starting from the root
 * to the level the table is bound with.
 *
 * @see #getDataBindingHierarchy
 */
protected void setDataBindingHierarchy(String dataBindingHierarchy) {
    TreeModel model = outliner.getModel();
    EditorTreeNode node = (EditorTreeNode)model.getRoot();
    TreePath treePath = null;
    if (dataBindingHierarchy != null && dataBindingHierarchy.length() > 0) {
        // in the following line we prepend the "" to the PATH_SEPARATOR
        // to convert it from a char to a String
        StringTokenizer st = new StringTokenizer(dataBindingHierarchy,
                                                 "" + DataBinding.PATH_DELIMITER);
        Vector path = new Vector(10);
        path.addElement(outliner.getModel().getRoot());
        while (st.hasMoreTokens()) {
            String s = st.nextToken();
            for (int i = 0; i < node.getChildCount(); i++) {
                EditorTreeNode n = (EditorTreeNode)node.getChildAt(i);
                if (s.equals(n.getUserObject())) {
                    node = n;
                    path.addElement(node);
                    break;
                }
            }
        }
        Object[] objects = new Object[path.size()];
        path.copyInto(objects);
        treePath = new TreePath(objects);
        outliner.expandPath(treePath);
        outliner.setSelectionPath(treePath);
    }
}

}
