/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataModelTabs.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:10 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.customizer;

import java.awt.*;
import java.util.*;

import javax.swing.JTabbedPane;

import com.klg.jclass.datasource.beans.NodeProperties;
import com.klg.jclass.util.JCEnvironment;

/**
 * 	The UI portion of the NodeProperties editor.  It observes any changes to the
 *	underlying NodeProperties and changes the current tab accordingly.
 */
public class DataModelTabs extends JTabbedPane {

/** Supported types are JDBC, VCAFE and JBUILDER. */
public static final int DATA_SOURCE_PAGE = 0;
public static final int DATA_ACCESS_PAGE = 1;
public static final int VIRTUAL_COLUMNS_PAGE = 2;
/** max page, i.e. number of pages. */
private static final int MAX_PAGE = VIRTUAL_COLUMNS_PAGE + 1;

public DataModelPage pages[] = new DataModelPage[MAX_PAGE];

/** This UI is often used in a hierarchy. If used in hierarchy, the
 *	data source type is determined by the root of the hierarchy.
 */
boolean useInHierarchy = false;

DesignTimeMetaData meta;
NodeProperties comp;

private static final boolean TRACE = false;

public DataModelTabs(boolean use_in_hierarchy, DesignTimeMetaData meta) {
    super();
    this.useInHierarchy = use_in_hierarchy;
    this.meta = meta;
    init();
}

public DataModelTabs(DesignTimeMetaData meta) {
    this(false, meta);
}

public void init() {
    // Create the tab manager
    DataModelPage page = null;

    page = getDataSourcePage();
    pages[DATA_SOURCE_PAGE] = page;
    addTab(page.getPageName(), page);

    page = new DataAccessPage(useInHierarchy);
    pages[DATA_ACCESS_PAGE] = page;
    addTab(page.getPageName(), page);
    page = new VirtualColumnPage(meta);
    pages[VIRTUAL_COLUMNS_PAGE] = page;
    addTab(page.getPageName(), page);    
//FIXOBSOLETE
/*
if (JCEnvironment.getOS() == JCEnvironment.OS_UNIX) {
    setPreferredSize(new Dimension(400, 640));
}
else {
    setPreferredSize(new Dimension(400, 500));
}
*/
    setPreferredSize(new Dimension(400, 500));
    setSelectedIndex(DATA_SOURCE_PAGE);
    com.klg.jclass.datasource.customizer.CustomizerLauncher.setDefaultFont((Component)this, new Font("Dialog", Font.PLAIN, 9));
}

/** Subclass overrides to provide data source type specific page. */
protected DataModelPage getDataSourcePage() {
    JDBCPage page = new JDBCPage();
    page.setUseInHierarchy(useInHierarchy);
    return page;
}

/** Enables/Disables tabs depending on data source type selected. 
 *	Also changes the current tab corresponding to the selected checkbox. 
 */
public void updateTabs(int type) {
}

public void setNodeProperties(NodeProperties comp) {
    this.comp = comp;
    for (int i=0; i<pages.length; i++) {
        pages[i].init(comp);
    }
}

public NodeProperties getNodeProperties() {
    return comp;
}

}   // DataModelTabs

