/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DriverTablePage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:10 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.customizer;

import java.awt.*;
import java.awt.event.*;

import java.sql.Connection;
import java.util.*;
import java.util.Vector;

import javax.swing.*;
import javax.swing.event.*;

import com.klg.jclass.datasource.*;
import com.klg.jclass.datasource.beans.*;
import com.klg.jclass.datasource.LocaleBundle;
import com.klg.jclass.datasource.util.*;

/** 
 */
public class DriverTablePage extends DataModelPage implements ActionListener,
    ItemListener {


JComboBox driverTable, column;
JList primaryKeys;
JButton btnAdd, btnDelete;
DefaultListModel listModel;

public DriverTablePage() {
    super(LocaleBundle.string(LocaleBundle.driver_table));
    init();
}

public void init() {
    setLayout(new BorderLayout());

    // Build table panel
    JPanel table_panel = new JPanel();
    driverTable = new JComboBox();
    JLabel lbl_table = new JLabel(LocaleBundle.string(LocaleBundle.table));
    table_panel.setLayout(new GridBagLayout());
    GridBagConstraints c = new GridBagConstraints();
    c.insets = new Insets(5, 5, 5, 5);
    c.anchor = GridBagConstraints.WEST;
    c.weightx = 0;
    table_panel.add(lbl_table, c);
    c.fill = GridBagConstraints.HORIZONTAL;
    c.weightx = 5;
    table_panel.add(driverTable, c);

    // Build column panel
    JPanel column_panel = new JPanel();
    JLabel lbl_pkey = new JLabel(LocaleBundle.string(LocaleBundle.primary_key));
    listModel = new DefaultListModel();
    primaryKeys = new JList(listModel);
    JPanel p = new JPanel();
    column = new JComboBox();
    JLabel lbl_column = new JLabel(LocaleBundle.string(LocaleBundle.column_name));
    column_panel.setLayout(new BorderLayout());
    column_panel.add(lbl_pkey, BorderLayout.NORTH);
    column_panel.add(new JScrollPane(primaryKeys), BorderLayout.CENTER);
    p.setLayout(new GridBagLayout());
    c = new GridBagConstraints();
    c.insets = new Insets(5, 5, 5, 5);
    c.anchor = GridBagConstraints.WEST;
    c.weightx = 0;
    p.add(lbl_column, c);
    c.fill = GridBagConstraints.HORIZONTAL;
    c.weightx = 5;
    p.add(column, c);
    column_panel.add(p, BorderLayout.SOUTH);

    // Build button panel
    JPanel btn_panel = new JPanel();
    btnAdd = new JButton(LocaleBundle.string(LocaleBundle.add));
    btnDelete = new JButton(LocaleBundle.string(LocaleBundle.delete));
    btn_panel.setLayout(new GridBagLayout());
    c = new GridBagConstraints();
    c.anchor = GridBagConstraints.WEST;
    c.insets = new Insets(5, 5, 5, 0);
    btn_panel.add(btnAdd, c);
    c.weightx = 5;
    c.fill = GridBagConstraints.NONE;
    btn_panel.add(btnDelete, c);

    add(table_panel, BorderLayout.NORTH);
    add(column_panel, BorderLayout.CENTER);
    add(btn_panel, BorderLayout.SOUTH);


    primaryKeys.addListSelectionListener(new ListSelectionListener() {
                                             public void valueChanged(ListSelectionEvent e) {
                                                 if (isInitializing()) return;

                                                 if (e.getValueIsAdjusting()) {
                                                     return;
                                                 }
                                                 String s = (String) primaryKeys.getSelectedValue();
                                                 column.setSelectedItem(s);
                                             }
                                         });
    driverTable.addItemListener(this);
    column.addItemListener(this);
    btnAdd.addActionListener(this);
    btnDelete.addActionListener(this);
}


public void setNodeProperties(NodeProperties comp) {
    // load tables
    SqlStatement sqlStatement = new SqlStatement(comp.getStatement());
    if (driverTable.getItemCount() > 0) {
        driverTable.removeAllItems();
    }
    String driver_table_qualified_name = null;
    for (Enumeration e = sqlStatement.getTables(); e.hasMoreElements();) {
        SqlStatementTable t = (SqlStatementTable)e.nextElement();
        driverTable.addItem(t.getAlias());
        if (t.getAlias().equals(comp.getDriverTable())) {
            driver_table_qualified_name = t.getQualifiedName();
        }
    }

    // reflect driver table and driver table primary key columns
    loadColumns(driver_table_qualified_name);

    if (comp.getDriverTable() != null) {
        driverTable.setSelectedItem(comp.getDriverTable());
    }
}

void setAllowPrimaryKeysEntered(boolean v) {
    primaryKeys.setEnabled(v);
    column.setEnabled(v);
    btnAdd.setEnabled(v);
    btnDelete.setEnabled(v);
}

void loadColumns(String table_name) {
	
	boolean editsAllowed = true;
	
    if (column.getItemCount() > 0) {
        column.removeAllItems();
    }
    listModel.removeAllElements();

    if (table_name == null) {
        return;
    }

    Connection connection = null;
    try {
        connection = getNodeProperties().getConnection() == null ? 
                     null :
                     getNodeProperties().getConnection().getConnection();

    }
    catch (Exception e) {
        ErrorDialog.showError(this, e.toString());
        return;
    }

    if (connection == null) return;

    Table table = new Table(connection,
                            table_name);

    // Don't let the user type in the primary key if
    // primary key columns can be obtained from the JDBC driver.
	editsAllowed = !(table != null && table.getPrimaryKeys() != null && table.getPrimaryKeys().size() > 0);
    setAllowPrimaryKeysEntered(editsAllowed);

    // Load all columns of the table to the primary key column choice list.
    String[] s = table.getColumnNames();
    for (int i=0; i<s.length; i++) {
        column.addItem(s[i]);
    }

    // Load the primary key.
    Vector v = (table.getPrimaryKeys() != null && 
                table.getPrimaryKeys().size() > 0) ?
               table.getPrimaryKeys() :
               // Can't get the primary key from the JDBC driver
               // Load the primary keys from the data bean component.
               getNodeProperties().getDriverTablePrimaryKeys();
    if (v != null) {
        for (Enumeration e = v.elements(); e.hasMoreElements();) {
            String column = (String) e.nextElement();
            listModel.addElement(column);
        }
    }

    if (listModel.getSize() > 0) {
        primaryKeys.setSelectedIndex(0);
        column.setSelectedItem(primaryKeys.getSelectedValue());
    }
    else {
        // Primary keys must enter manually.
        ErrorDialog.showError(this, 
                              LocaleBundle.string(LocaleBundle.primary_key_of_driver_table_must_be_entered));
	}
	
	// if keys obtained automatically, set them, otherwise
	// user sets them
	if (!editsAllowed) {
		setDriverTablePrimaryKeys();
	}
}

private boolean columnExists(String column) {
    boolean v = false;
    for (int i=0; i<listModel.getSize(); i++) {
        if (listModel.getElementAt(i).equals(column)) {
            v = true;
            break;
        }
    }
    return v;
}

private void addColumn() {
    String s = (String) column.getSelectedItem();
    if (s == null || s.length() == 0) return;

    if (!columnExists(s)) {
        listModel.addElement(s);
        setDriverTablePrimaryKeys();
    }
}

/** Helper method to delete a join. */
private void deleteColumn() {
    int index = primaryKeys.getSelectedIndex();
    if (index > -1) {
        primaryKeys.remove(index);
        setDriverTablePrimaryKeys();
    }
}

private void setDriverTablePrimaryKeys() {
    Vector v = new Vector();
    for (int i=0; i<listModel.getSize(); i++) {
        v.addElement(listModel.getElementAt(i));
    }
    try {
        getNodeProperties().setDriverTablePrimaryKeys(v);
        getNodeProperties().fireJCDataEvent(new JCDataEvent((Object)getNodeProperties()));
    }
    catch (Exception e) {
    }
}


public void actionPerformed(ActionEvent e) {
    if (isInitializing()) return;

    if (e.getSource() instanceof JButton) {
        JButton btn = (JButton) e.getSource();
        if (btn.equals(btnAdd)) {
            addColumn();
        }
        else if (btn.equals(btnDelete)) {
            deleteColumn();
        }
    }
}

/** When a join is selected, change the field content. */
public void itemStateChanged(ItemEvent ev) {
    if (isInitializing()) return;

    if (ev.getStateChange() == ItemEvent.SELECTED) {
        if (ev.getSource().equals(primaryKeys)) {
        }
        else if (ev.getSource().equals(driverTable)) {
            String table_name = (String) driverTable.getSelectedItem();
            try {
                getNodeProperties().setDriverTable(table_name);
                getNodeProperties().fireJCDataEvent(new JCDataEvent((Object)getNodeProperties()));
            }
            catch (Exception e) {
            }
        }
    }
}

} // DriverTablePage
