/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JDBCPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:10 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.customizer;

import java.awt.*;
import java.awt.event.*;
import java.sql.*;
import java.util.*;

import javax.swing.*;

import com.klg.jclass.datasource.*;
import com.klg.jclass.datasource.beans.*;
import com.klg.jclass.datasource.LocaleBundle;
import com.klg.jclass.datasource.jdbc.DataTableConnection;
import com.klg.jclass.datasource.util.*;
import com.klg.jclass.util.swing.JCAlignLayout;


/** This custom editor page contains JDBC data source specific
 *	fields.
 */
public class JDBCPage extends DataModelPage implements Observer, 
    ActionListener {

SqlBuilderPanel sp;
ConnectionPanel cp;
JTabbedPane tab;
Component btnSp, btnDriverTable;
JButton     btnModify, btnRestore, btnClear, btnAddTable, 
btnAddColumn, btnAddJoin, btnJoinToParent;
JTextField designTimeMaxRows;
SqlStatementView view = null;
DriverTablePage driverTablePage = null;

public JDBCPage() {
    super(LocaleBundle.string(LocaleBundle.jdbc));
    setLayout(new GridLayout(1,1));

    // Create SQL statement page
    sp = new SqlBuilderPanel(null);
    view = new SqlStatementView(sp.sqlStatement);
    view.sqlStatement.addObserver(this);

    JPanel button_panel = new JPanel();
    JPanel p1 = new JPanel();
    JPanel p2 = new JPanel();
    btnModify = new JButton(LocaleBundle.string(LocaleBundle.set));
    btnRestore = new JButton(LocaleBundle.string(LocaleBundle.restore));
    btnClear = new JButton(LocaleBundle.string(LocaleBundle.clear));
    btnJoinToParent = new JButton(LocaleBundle.string(LocaleBundle.join_to_parent));
    btnAddTable = new JButton(LocaleBundle.string(LocaleBundle.add_table));
    btnAddColumn = new JButton(LocaleBundle.string(LocaleBundle.add_selected_columns));
    btnAddJoin = new JButton(LocaleBundle.string(LocaleBundle.add_join));
    JPanel sql_panel = new JPanel();
    JPanel design_time_panel = new JPanel();
    JLabel lbl_max = new JLabel(LocaleBundle.string(LocaleBundle.design_time_max_rows));
    designTimeMaxRows = new JTextField("", 5);
    JDBCPageScrollPane sw = new JDBCPageScrollPane(sp);
    JSplitPane w = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
    w.setOneTouchExpandable(true);
    w.add(sw);
    w.add(view);

    button_panel.setLayout(new GridLayout(2, 1));
    p1.setLayout(new GridBagLayout());
    p2.setLayout(new GridBagLayout());
    GridBagConstraints c = new GridBagConstraints();
    c.anchor = GridBagConstraints.WEST;
    c.fill = GridBagConstraints.NONE;
    c.insets = new Insets(0, 5, 5, 0);
    p1.add(btnAddTable, c);
    p1.add(btnAddColumn, c);
    p1.add(btnAddJoin, c);
    c.weightx = 5;
    p1.add(btnJoinToParent, c);
    c.gridy = 2;
    c.weightx = 0;
    p2.add(btnModify, c);
//	p2.add(btnRestore, c);
    c.weightx = 5;
    p2.add(btnClear, c);
    button_panel.add(p1);
    button_panel.add(p2);

    sql_panel.setLayout(new BorderLayout(5, 5));
    sql_panel.add(w, BorderLayout.CENTER);
    sql_panel.add(button_panel, BorderLayout.SOUTH);

    btnModify.addActionListener(this);
    btnRestore.addActionListener(this);
    btnClear.addActionListener(this);
    btnAddTable.addActionListener(this);
    btnAddColumn.addActionListener(this);
    btnAddJoin.addActionListener(this);
    btnJoinToParent.addActionListener(this);

    // Create Connection page
    cp = new ConnectionPanel(this);

    // Driver table page
    driverTablePage = new DriverTablePage();

    // Create tab manager
    tab = new JTabbedPane();
    // Connection page
    tab.addTab(cp.getPageName(), cp);
    // SQL Builder page
    tab.addTab(sp.getPageName(), sql_panel);
    int pos = tab.indexOfComponent(sql_panel);
    btnSp = tab.getComponentAt(pos);
    tab.addTab(driverTablePage.getPageName(), driverTablePage);
    pos = tab.indexOfComponent(driverTablePage);
    btnDriverTable = tab.getComponentAt(pos);
    tab.setEnabledAt(tab.indexOfComponent(btnSp), false);
    tab.setEnabledAt(tab.indexOfComponent(btnDriverTable), false);

    design_time_panel.setLayout(new JCAlignLayout());
    design_time_panel.add(lbl_max);
    design_time_panel.add(designTimeMaxRows);
    designTimeMaxRows.addActionListener(this);
    designTimeMaxRows.addFocusListener(
                                      new FocusAdapter() {
                                          public void focusLost(FocusEvent e) {
                                              if (isInitializing()) return;

                                              setDesignTimeMaxRows();
                                          }
                                      }
                                      );

    setLayout(new BorderLayout());
    add(design_time_panel, BorderLayout.SOUTH);
    add(tab, BorderLayout.CENTER);
}

public void setUseInHierarchy(boolean v) {
    cp.setUseInHierarchy(v);
}

/** Displays values in the bean data comp. */
public void setNodeProperties(NodeProperties comp) {
    if (comp.getDataSourceType() != NodeProperties.JDBC) return;

    // No parent and thus can't use parent connection
    cp.useParentConnection.setEnabled(comp.getParentNodeProperties() != null);
    updateTabs();

    // must call init() instead setNodeProperties().
    // otherwise, isInitializing check is invalid in the sub pages.
    cp.init(comp);
    sp.init(comp);
    driverTablePage.init(comp);

    view.setSqlStatement(sp.sqlStatement);

    designTimeMaxRows.setText(""+com.klg.jclass.datasource.jdbc.MetaData.getDesignTimeMaxRows());
}

void updateTabs() {
    DataTableConnection dtc = null;
    NodeProperties comp = getNodeProperties();
    try {
        dtc = comp.getConnection();
    }
    catch (Exception e) {
    }
    Connection c = null;
    if (dtc != null) {
        c = dtc.getConnection();
    }
    sp.setConnection(c);

    tab.setEnabledAt(tab.indexOfComponent(btnSp), c != null);
    tab.setEnabledAt(tab.indexOfComponent(btnDriverTable), c != null);

    // No parent or no connection and thus can't use join
    btnJoinToParent.setEnabled(c != null && comp.getParentNodeProperties() != null);

    if (c == null) {
        tab.setSelectedIndex(0);    // Switch to Connection page.
    }
}

private void setDesignTimeMaxRows() {
    int max = com.klg.jclass.datasource.jdbc.MetaData.DEFAULT_DESIGN_TIME_MAX_ROWS;
    try {
        max = Integer.valueOf(designTimeMaxRows.getText()).intValue();
    }
    catch (Exception ex) {
    }

    if (com.klg.jclass.datasource.jdbc.MetaData.getDesignTimeMaxRows() != max) {
        com.klg.jclass.datasource.jdbc.MetaData.setDesignTimeMaxRows(max);

        // force a reload of meta data to use the new max row.
        for (Enumeration e=DataSourceBeanInstance.getInstances().elements();
            e.hasMoreElements();) {
            Object inst = e.nextElement();
            if (inst instanceof TreeData) {
                try {
                    ((TreeData)inst).requeryAll(null);
                }
                catch (Exception ex) {
                }
            }
        }
    }
}

void refreshView() {
    // Refresh the statement view from the statement 
    // in the data bean component
    view.setText(getNodeProperties().getStatement());

    // Refresh Sql builder
    sp.setNodeProperties(getNodeProperties());
    view.setSqlStatement(sp.sqlStatement);
}

/** Modify or cancel setting SQL Statement. */
public void actionPerformed(ActionEvent e) {
    if (isInitializing()) return;

    if (e.getSource() instanceof JTextField) {
        setDesignTimeMaxRows();
    }
    else if (e.getSource() instanceof JButton) {
        JButton btn = (JButton) e.getSource();
        if (btn.equals(btnModify)) {
            modify();
        }
        else if (btn.equals(btnRestore)) {
            refreshView();
        }
        else if (btn.equals(btnClear)) {
            clear();
            refreshView();
        }
        else if (btn.equals(btnAddTable)) {
            sp.showTableChooser();
        }
        else if (btn.equals(btnAddColumn)) {
            sp.addColumn();
            modify();
        }
        else if (btn.equals(btnAddJoin)) {
            sp.addJoin();
            modify();
        }
        else if (btn.equals(btnJoinToParent)) {
            joinToParent();
            modify();
        }
    }
}

void clear() {
    // Set the statement of the comp
    try {
        // Update the data bean component.
        getNodeProperties().setStatement("");
        getNodeProperties().setDriverTable(null);
        getNodeProperties().removeAllJoins();
        // Notify change
        getNodeProperties().fireJCDataEvent(new JCDataEvent((Object)getNodeProperties()));
        view.setText(getNodeProperties().getStatement());
    }
    catch (Exception ex) {
        ex.printStackTrace();
    }
}

void modify() {
    // Set the statement of the comp

    // make sure the number of joins equals the number of parameters
    String sql = view.getText();
    SqlParser parser = new SqlParser(sql);
    if (getNodeProperties().getJoins().length != parser.getParameterColumns().size()) {
        ErrorDialog.showError(this, 
                              LocaleBundle.string(LocaleBundle.parameter_count_mismatch));
        joinToParent();
        return;
    }


    try {
        getNodeProperties().setExpertMode(view.sqlStatement.isExpertMode());
        // Update the data bean component.
        getNodeProperties().setStatement(view.getText());

        // default driver table, if none exists
        if (getNodeProperties().getDriverTable() == null ||
            getNodeProperties().getDriverTable().trim().length() == 0) {
            String driver = null;
            for (Enumeration e = view.sqlStatement.getTables(); e.hasMoreElements();) {
                SqlStatementTable t = (SqlStatementTable)e.nextElement();
                driver = t.getAlias();
                break;
            }   
            getNodeProperties().setDriverTable(driver);
        }

        // Notify change
        getNodeProperties().fireJCDataEvent(new JCDataEvent((Object)getNodeProperties()));
    }
    catch (Exception ex) {
        ex.printStackTrace();
    }
}

void joinToParent() {
    // No join to parent if there is no parent.
    if (getNodeProperties().getParentNodeProperties() == null) {
        return;
    }

    // Convert the join into a list of join strings.
    String ss[][] = getNodeProperties().getJoins();
    Vector v = new Vector();
    for (int i=0; i<ss.length; i++) {
        v.addElement(ss[i][0] + " = " + ss[i][1]);
    }
    SqlStatement parent_sql = new SqlStatement(getNodeProperties().getParentNodeProperties().getStatement());
    SqlStatement child_sql = new SqlStatement(view.sqlStatement.getText());
    JoinDialog jdlg = new JoinDialog(null, 
                                     v, 
                                     parent_sql,
                                     getNodeProperties().getParentNodeProperties(),
                                     child_sql,
                                     getNodeProperties(), true, true);
    jdlg.setTitle(LocaleBundle.string(LocaleBundle.join_to_parent));
    jdlg.lblParent.setText(LocaleBundle.string(LocaleBundle.parent));
    jdlg.lblChild.setText(LocaleBundle.string(LocaleBundle.child));
    jdlg.pack();
    jdlg.show();

    // Delete all joins and parameters first
    view.sqlStatement.removeAllParameterColumns();
    getNodeProperties().removeAllJoins();

    if (getNodeProperties().isExpertMode()) {
        // In expert mode, user must enter parameter columns manually.
        ErrorDialog.showWarning(this, 
                                LocaleBundle.string(LocaleBundle.enter_parameter_columns));
    }

    // Add joins back
    for (Enumeration e=jdlg.getJoinStrings().elements(); e.hasMoreElements();) {
        String s = (String) e.nextElement();
        String parent_column = s.substring(0, s.indexOf(" = "));
        String child_column = s.substring(s.indexOf(" = ")+3);
        getNodeProperties().addJoin(parent_column, child_column);

        if (!getNodeProperties().isExpertMode()) {
            // Add parameter to the current level using child column 
            view.sqlStatement.addParameterColumn(child_column);
        }
    }
    view.sqlStatement.notifyObservers(this);
}

public void update(Observable o, Object arg) {
    // if in expert mode, disable some automatic functionalities.
    btnAddTable.setEnabled(!view.sqlStatement.isExpertMode());
    btnAddColumn.setEnabled(!view.sqlStatement.isExpertMode());
    btnAddJoin.setEnabled(!view.sqlStatement.isExpertMode());
    if (getNodeProperties().isExpertMode() != view.sqlStatement.isExpertMode()) {
        modify();
    }
/*
// This is buggy, don't bother
// handle double-clicks on selected column in table
if (o instanceof SqlStatement) {
    modify();
}
*/
}

class JDBCPageScrollPane extends JScrollPane {

    public JDBCPageScrollPane(Component c) {
        super(c);
    }

    public Dimension getMinimumSize() {
        return new Dimension(400, 150);
    }

    public Dimension getPreferredSize() {
        return getMinimumSize();
    }

}

} 
