/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Join.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:11 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.customizer;

import java.io.Serializable;
import java.util.*;

import com.klg.jclass.datasource.util.ColumnMap;


/** This is a model of a join between two database tables. 
 *	A join contains a list of ColumnMaps.
 */
class Join implements Serializable {

// FIX should have aliases of the tables.
Table parent, child;
String parentAlias, childAlias;
Hashtable columnMaps = new Hashtable();

Join(Table parent, String parent_alias, Table child, String child_alias) {
    this.parent = parent;
    this.parentAlias = parent_alias;
    this.child = child;
    this.childAlias = child_alias;
}

String createKey(String s1, String s2) {
    return s1 + " = " + s2;
}

void add(String parent_column, String child_column) {
    if (parent_column == null || child_column == null) return;

    if (!exists(parent_column, child_column)) {
        columnMaps.put(createKey(parent_column, child_column),
                       new ColumnMap(parent_column, child_column));
    }
}

boolean exists(String parent_column, String child_column) {
    if (parent_column == null || child_column == null) return false;

    return 	columnMaps.containsKey(createKey(parent_column, child_column)) ||
    columnMaps.containsKey(createKey(child_column, parent_column));
}

void remove(String parent_column, String child_column) {
    if (parent_column == null || child_column == null) return;

    if (exists(parent_column, child_column)) {
        columnMaps.remove(createKey(parent_column, child_column));
    }
    if (exists(child_column, parent_column)) {
        columnMaps.remove(createKey(child_column, parent_column));
    }
}

/** Get a list of ColumnMap. */
Enumeration elements() {
    return columnMaps.elements();
}

public String toString() {
    StringBuffer sb = new StringBuffer();

    Enumeration e = elements();
    while (e.hasMoreElements()) {
        ColumnMap cm = (ColumnMap) e.nextElement();
        sb.append("[" + cm.getParentColumn() + "=" + cm.getChildColumn() + "]\n");
    }
    return new String(sb);
}

}
