/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JoinDialog.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:11 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.customizer;

import java.awt.*;
import java.awt.event.*;
import java.sql.Connection;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;

import com.klg.jclass.datasource.*;
import com.klg.jclass.datasource.beans.*;
import com.klg.jclass.datasource.LocaleBundle;
import com.klg.jclass.datasource.util.*;

/** A dialog to create/modify joins between two tables.
 *	It can be used to create joins between ONLY two tables, or multiple tables.
 */
class JoinDialog extends JDialog implements ActionListener, ItemListener {

JLabel lblParent, lblChild;
JList joinList;
JButton btnAdd, btnModify, btnDelete, btnAutoJoin,btnOk;
JComboBox parentColumn, childColumn;
JComboBox parentTable, childTable;
DefaultListModel listModel = new DefaultListModel();

Table joinParent, joinChild;
SqlStatement parentSqlStatement, childSqlStatement;
NodeProperties parentComp, childComp;
boolean allowMoreThanTwoTables;
boolean onlyUseParentSelectedColumns;
/** A list of tables used in auto join. */
Vector tables;

private final static String equalJoin = " = ";

/** Constructs a join dialog.
 *	@param frame parent frame of the dialog
 *	@param joins a list of join strings
 *  @param sql sql statement used to get a list of tables.
 *  @param comp NodeProperties that provides a database connection to get 
 *			a list of columns in tables.
 *  to creata a new join model.
 */
JoinDialog(Frame frame, Vector joins, SqlStatement sql, NodeProperties comp) {
    this(frame, joins, sql, comp, sql, comp, false, false);
}

/** Constructs a join dialog.
 *	@param frame parent frame of the dialog
 *	@param joins a list of join strings
 *	@param parentSqlStatement sql statement used to get a list of tables for parent.
 *  @param parentComp NodeProperties that provides a parent database connection to get 
 *			a list of columns in tables.
 *  @param childSqlStatement sql statement used to get a list of tables for child.
 *  @param childComp NodeProperties that provides a child database connection to get 
 *			a list of columns in tables.
 *	@param v indicates if more than 2 tables are allowed.  If not,
 *			joins can only be created between 2 tables.
 *	@param v2 indicates if only selected columns from
 *			the parent's SQL statement is used.
 */
JoinDialog(Frame frame, Vector joins, 
           SqlStatement parentSqlStatement, NodeProperties parentComp,
           SqlStatement childSqlStatement, NodeProperties childComp, 
           boolean v,
           boolean v2) {
    super(frame);
    Vector joinStrings = (joins == null) ? new Vector() : joins;
    this.parentSqlStatement = parentSqlStatement;
    this.parentComp = parentComp;
    this.childSqlStatement = childSqlStatement;
    this.childComp = childComp;
    this.allowMoreThanTwoTables = v;
    this.onlyUseParentSelectedColumns = v2;
    setModal(true);
    setTitle(LocaleBundle.string(LocaleBundle.join));
    init();
    loadJoin(joinStrings);

    addWindowListener(new java.awt.event.WindowAdapter() {
                          public void windowClosing(java.awt.event.WindowEvent e) {
                              setVisible(false);
                          }
                      });
}

public void init() {

    // Create button panel
    getContentPane().setLayout(new BorderLayout());

    JPanel btn_panel = new JPanel();
    btnAdd = new JButton(LocaleBundle.string(LocaleBundle.add));
    btnModify = new JButton(LocaleBundle.string(LocaleBundle.modify));
    btnDelete = new JButton(LocaleBundle.string(LocaleBundle.delete));
    btnAutoJoin = new JButton(LocaleBundle.string(LocaleBundle.auto_join));
    btnOk = new JButton(LocaleBundle.string(LocaleBundle.ok));
    JPanel field_panel = new JPanel();
    JLabel lbl = new JLabel("");
    JLabel lbl_table = new JLabel(LocaleBundle.string(LocaleBundle.table));
    JLabel lbl_column_name = new JLabel(LocaleBundle.string(LocaleBundle.column_name));
    lblParent = new JLabel(LocaleBundle.string(LocaleBundle.primary));
    lblChild = new JLabel(LocaleBundle.string(LocaleBundle.foreign));
    parentTable = new JComboBox();
    parentColumn = new JComboBox();
    childTable = new JComboBox();
    childColumn = new JComboBox();

    joinList = new JList(listModel);
    JPanel p = new JPanel();
    btn_panel.setLayout(new GridBagLayout());
    GridBagConstraints c = new GridBagConstraints();
    c.anchor = GridBagConstraints.NORTH;
    c.fill = GridBagConstraints.BOTH;
    c.gridheight = c.gridwidth = 1;
    c.weightx = c.weighty = 1;
    c.insets = new Insets(5, 5, 5, 0);
    btn_panel.add(btnAdd, c);
    btn_panel.add(btnModify, c);
    btn_panel.add(btnDelete, c);
    btn_panel.add(btnAutoJoin, c);
    c.insets = new Insets(5, 5, 5, 5);
    btn_panel.add(btnOk, c);

    // Create field panel
    field_panel.setLayout(new GridLayout(3, 3, 5, 5));
    field_panel.add(lbl);
    field_panel.add(lbl_table);
    field_panel.add(lbl_column_name);
    field_panel.add(lblParent);
    field_panel.add(parentTable);
    field_panel.add(parentColumn);
    field_panel.add(lblChild);
    field_panel.add(childTable);
    field_panel.add(childColumn);

    p.setLayout(new BorderLayout());
    p.add(field_panel, BorderLayout.CENTER);
    p.add(btn_panel, BorderLayout.SOUTH);

    getContentPane().add(new JScrollPane(joinList), BorderLayout.CENTER);
    getContentPane().add(p, BorderLayout.SOUTH);
    joinList.addListSelectionListener(new ListSelectionListener() {
                                          public void valueChanged(ListSelectionEvent e) {
                                              if (e.getValueIsAdjusting()) {
                                                  return;
                                              }
                                              showSelectedJoin();
                                          }
                                      });
    parentTable.addItemListener(this);
    childTable.addItemListener(this);
    btnAdd.addActionListener(this);
    btnModify.addActionListener(this);
    btnDelete.addActionListener(this);
    btnAutoJoin.addActionListener(this);
    btnOk.addActionListener(this);
}

void loadJoin(Vector joinStrings) {
    // Load table list.
    if (onlyUseParentSelectedColumns) {
        // Special code.
        parentTable.addItem(LocaleBundle.string(LocaleBundle.parent_query));
    }
    else {
        for (Enumeration e = parentSqlStatement.getTables(); e.hasMoreElements();) {
            SqlStatementTable t = (SqlStatementTable)e.nextElement();
            parentTable.addItem(t.getAlias());
        }
    }

    for (Enumeration e = childSqlStatement.getTables(); e.hasMoreElements();) {
        SqlStatementTable t = (SqlStatementTable)e.nextElement();
        childTable.addItem(t.getAlias());
    }

    // Allow setting tables if more than 2 tables are allowed, or
    // join is created the first time.
    parentTable.setEnabled(!onlyUseParentSelectedColumns && 
                           (allowMoreThanTwoTables || joinStrings.size() == 0));
    childTable.setEnabled(allowMoreThanTwoTables || joinStrings.size() == 0);

    for (Enumeration e = joinStrings.elements(); e.hasMoreElements();) {
        String s = (String) e.nextElement();

        // Load existing joins
        ColumnMap cm = getJoinColumns(s);
        addJoin(cm.getParentColumn(), cm.getChildColumn());
    }

    // Select the first join
    if (joinList.getModel().getSize() > 0) {
        joinList.setSelectedIndex(0);
        showSelectedJoin();
    }
    else {
        // Load parent table columns and child table columns
        loadParentColumns();
        loadChildColumns();
    }
}

ColumnMap getJoinColumns(String s) {
    String parent_column, child_column;
    parent_column = new String(s.substring(0, s.indexOf(equalJoin)));
    child_column = new String(s.substring(s.indexOf(equalJoin) + equalJoin.length()));
    return new ColumnMap(parent_column, child_column);
}

void loadParentColumns() {
    if (parentColumn.getItemCount() > 0) {
        parentColumn.removeAllItems();
    }
    if (onlyUseParentSelectedColumns) {
        for (Enumeration e = getParentSelectedColumns().elements(); e.hasMoreElements();) {
            String s = (String) e.nextElement();
            parentColumn.addItem(s);
        }
    }
    else {
        joinParent = getTable((String) parentTable.getSelectedItem(), 
                              parentSqlStatement, parentComp);
        if (joinParent != null) {
            String[] s = joinParent.getColumnNames();
            for (int i=0; i<s.length; i++) {
                parentColumn.addItem(s[i]);
            }
        }
    }
}

/** Get all table columns.  It is used when "*" is in the SQL statement. */
Vector getAllTableColumns(SqlStatement sql, NodeProperties comp) {
    Vector v = new Vector();
    for (Enumeration e = sql.getTables(); e.hasMoreElements();) {
        SqlStatementTable sqlt = (SqlStatementTable) e.nextElement();
        Table t = getTable(sqlt.getQualifiedName(), sql, comp);
        if (t != null) {
            String[] s = t.getColumnNames();
            for (int i=0; i<s.length; i++) {
                v.addElement(s[i]);
            }
        }
    }
    return v;
}

/** When onlyUseParentSelectedColumns is true, get a list of columns
 *	from the parent SQL statement using parent's JDBC connection. */
Vector getParentSelectedColumns() {
    Vector v = new Vector();
    for (Enumeration e = parentSqlStatement.getColumns(); e.hasMoreElements();) {
        SqlStatementColumn c = (SqlStatementColumn) e.nextElement();
        if (c.getAlias().indexOf("*") > -1) {
            if (c.getTableName() == null) {
                // "*" is not qualified and it means all the columns from all tables
                // in the SQL statement.
                v = getAllTableColumns(parentSqlStatement, parentComp);
            }
            else {
                // Add all columns of the table.
                joinParent = getTable(c.getTableName(), parentSqlStatement, parentComp);
                if (joinParent != null) {
                    String[] s = joinParent.getColumnNames();
                    for (int i=0; i<s.length; i++) {
                        v.addElement(s[i]);
                    }
                }
            }
        }
        else {
            v.addElement(c.getAlias());
        }
    }
    return v;
}

void loadChildColumns() {
    if (childColumn.getItemCount() > 0) {
        childColumn.removeAllItems();
    }
    joinChild = getTable((String) childTable.getSelectedItem(), 
                         childSqlStatement, childComp);
    if (joinChild != null) {
        String[] s = joinChild.getColumnNames();
        for (int i=0; i<s.length; i++) {
            childColumn.addItem(s[i]);
        }
    }
}

/**	Gets a table from a SQL statement, given an alias.
 */
public Table getTable(String alias, SqlStatement sqlStatement, NodeProperties comp) {
    // Find table qualified name from alias first.
    String qualified_name = null;
    for (Enumeration e = sqlStatement.getTables(); e.hasMoreElements();) {
        SqlStatementTable t = (SqlStatementTable)e.nextElement();
        if (t.getAlias().equals(alias)) {
            qualified_name = t.getQualifiedName();
            break;
        }
    }

    Table table = null;
    try {
        table = (comp.getConnection() == null) ? 
                null : 
                new Table(comp.getConnection().getConnection(), qualified_name);
    }
    catch (Exception e) {
    }

    return table;
}


Vector getJoinStrings() {
    Vector v = new Vector();
    for (int i=0; i<joinList.getModel().getSize(); i++) {
        v.addElement(joinList.getModel().getElementAt(i));
    }
    return v;
}

void showSelectedJoin() {
    if (joinList.getSelectedIndex() == -1) return;

    String selected = (String) joinList.getSelectedValue();
    ColumnMap cm = getJoinColumns(selected);

    if (onlyUseParentSelectedColumns) {
        loadParentColumns();
        parentColumn.setSelectedItem(cm.getParentColumn());
    }
    else {
        parentTable.setSelectedItem(
                                   cm.getParentColumn().substring(0, cm.getParentColumn().lastIndexOf(".")));
        loadParentColumns();
        parentColumn.setSelectedItem(
                                    cm.getParentColumn().substring(cm.getParentColumn().lastIndexOf(".")+1));
    }

    childTable.setSelectedItem(
                              cm.getChildColumn().substring(0, cm.getChildColumn().lastIndexOf(".")));
    loadChildColumns();
    childColumn.setSelectedItem(
                               cm.getChildColumn().substring(cm.getChildColumn().lastIndexOf(".")+1));
}


public void actionPerformed(ActionEvent e) {
    if (e.getSource() instanceof JButton) {
        JButton btn = (JButton) e.getSource();
        if (btn.equals(btnAdd)) {
            addJoin();
        }
        else if (btn.equals(btnModify)) {
            modifyJoin();
        }
        else if (btn.equals(btnDelete)) {
            deleteJoin();
        }
        else if (btn.equals(btnAutoJoin)) {
            autoJoin();
        }
        else if (btn.equals(btnOk)) {
            setVisible(false);
        }
    }
}

/** When a join is selected, change the field content. */
public void itemStateChanged(ItemEvent ev) {
    if (ev.getStateChange() == ItemEvent.SELECTED) {
        if (ev.getSource().equals(joinList)) {
            showSelectedJoin();
        }
        else if (ev.getSource().equals(this.parentTable)) {
            // Load column for the selected table.
            loadParentColumns();
        }
        else if (ev.getSource().equals(this.childTable)) {
            // Load column for the selected table.
            loadChildColumns();
        }
    }
}

/** Centers the dialog relative to the parent frame. */
public void setVisible(boolean visible) {
    if (visible) {
        int x = getParent().getBounds().x + (getParent().getBounds().width-getBounds().width) / 2;
        int y = getParent().getBounds().y + (getParent().getBounds().height-getBounds().height) / 2;
        if (x >= 0 && y >= 0) {
            setLocation(x, y);
        }
    }
    super.setVisible(visible);
}

/** Helper method to indicate if a join already exists. */
private boolean joinExists(String parent_column, String child_column) {
    boolean v = false;
    for (int i=0; i<joinList.getModel().getSize(); i++) {
        if (joinList.getModel().getElementAt(i).equals(
                                                      parent_column + equalJoin + child_column) ||
            joinList.getModel().getElementAt(i).equals(
                                                      child_column + equalJoin + parent_column)) {
            v = true;
            break;
        }
    }
    return v;
}

private void addJoin(String parent_column, String child_column) {
    if (parent_column.length() == 0 || child_column.length() == 0) return;

    if (!joinExists(parent_column, child_column)) {
        String new_join = parent_column + equalJoin + child_column;
        listModel.addElement(new_join);
        int count = listModel.getSize();

        // Allow setting tables if more than 2 tables are allowed, or
        // join is created the first time.
        parentTable.setEnabled(!onlyUseParentSelectedColumns && 
                               (allowMoreThanTwoTables || count == 0));
        childTable.setEnabled(allowMoreThanTwoTables || count == 0);
    }
}

/** Helper method to add a join. */
private void addJoin() {
    String parent_column = 
    ((onlyUseParentSelectedColumns) ? "" : (parentTable.getSelectedItem() + ".") ) + 
    parentColumn.getSelectedItem();
    String child_column = childTable.getSelectedItem() + "." + childColumn.getSelectedItem();
    addJoin(parent_column, child_column);
}

/** Helper method to modify a join. */
private void modifyJoin() {
    String parent_column = 
    ((onlyUseParentSelectedColumns) ? "" : (parentTable.getSelectedItem() + ".") ) + 
    parentColumn.getSelectedItem();
    String child_column = childTable.getSelectedItem() + "." + childColumn.getSelectedItem();
    if (parent_column.length() == 0 || child_column.length() == 0) return;

    deleteJoin();
    addJoin(parent_column, child_column);
}

/** Helper method to delete a join. */
private void deleteJoin() {
    int index = joinList.getSelectedIndex();
    if (index > -1) {
        joinList.remove(index);

        // Allow setting tables if more than 2 tables are allowed, or
        // join is created the first time.
        int count = listModel.getSize();
        parentTable.setEnabled(!onlyUseParentSelectedColumns && 
                               (allowMoreThanTwoTables || count == 0));
        childTable.setEnabled(allowMoreThanTwoTables || count == 0);
    }
}



/** Helper method to auto join tables based on primary/foreign key combinations. */
private void autoJoin() {
    Connection c = null;
    try {
        c = (parentComp.getConnection() == null) ? 
            null : 
            parentComp.getConnection().getConnection();
    }
    catch (Exception e) {
        ErrorDialog.showError((Frame) getParent(),
                              e + " " +
                              LocaleBundle.string(LocaleBundle.cannot_auto_join) + " " +
                              LocaleBundle.string(LocaleBundle.cannot_establish_connection_for_parent));
        return;
    }

    if (c == null) return;

    Connection current_connection = null;
    try {
        current_connection = (childComp.getConnection() == null) ? 
                             null : 
                             childComp.getConnection().getConnection();
    }
    catch (Exception e) {
        ErrorDialog.showError((Frame) getParent(),
                              e + " " +
                              LocaleBundle.string(LocaleBundle.cannot_auto_join) + " " + 
                              LocaleBundle.string(LocaleBundle.cannot_establish_connection));
        return;
    }
    if (current_connection == null) return;

    // Get all tables at child level
    if (!childSqlStatement.getTables().hasMoreElements() || 
        !parentSqlStatement.getTables().hasMoreElements()) {
        ErrorDialog.showError((Frame) getParent(),
                              LocaleBundle.string(LocaleBundle.cannot_auto_join) + " " +
                              LocaleBundle.string(LocaleBundle.no_table_exists));
        return;
    }

    Enumeration ep = null;
    if (allowMoreThanTwoTables) {
        ep = parentSqlStatement.getTables();
    }
    else {
        // If only allow joins between 2 tables, parent is always the selected table.
        Vector v = new Vector();
        for (Enumeration e = parentSqlStatement.getTables(); e.hasMoreElements();) {
            SqlStatementTable parent_sql_table = (SqlStatementTable) e.nextElement();
            if (parent_sql_table.getAlias() == parentTable.getSelectedItem()) {
                v.addElement(parent_sql_table);
                break;
            }
        }
        ep = v.elements();
    }
    for (; ep.hasMoreElements();) {
        SqlStatementTable parent_sql_table = (SqlStatementTable) ep.nextElement();
        Table parent_table = null;
        try {
            parent_table = new Table(c, parent_sql_table.getQualifiedName());
        }
        catch (Exception e) {
            continue;
        }

        Vector vtemp = getParentSelectedColumns();
        Vector parent_selected_columns = new Vector();
        if (vtemp != null) {
            // convert columns into lower case for comparison
            for (int i=0; i<vtemp.size(); i++) {
                parent_selected_columns.addElement(
                                                  ((String)vtemp.elementAt(i)).toLowerCase());
            }
        }

        // Get the full name of the parent table.  table name returned from
        // getDriverTable() may be without the schema.
        String parent_full_name = parent_table.getFullName();
        String table;
        Enumeration e = null;
        if (allowMoreThanTwoTables) {
            e = childSqlStatement.getTables();
        }
        else {
            // If only allow joins between 2 tables, parent is always the selected table.
            Vector v = new Vector();
            for (Enumeration et = childSqlStatement.getTables(); et.hasMoreElements();) {
                SqlStatementTable child_sql_table = (SqlStatementTable) et.nextElement();
                if (child_sql_table.getAlias() == childTable.getSelectedItem()) {
                    v.addElement(child_sql_table);
                    break;
                }
            }
            e = v.elements();
        }

        for (; e.hasMoreElements();) {
            SqlStatementTable t = (SqlStatementTable)e.nextElement();
            Table current = null;
            try {
                current = new Table(current_connection, t.getQualifiedName());
            }
            catch (Exception ex) {
                continue;
            }

            String current_full_name = current.getFullName();

            // match its foreign keys with the parent primary key.
            for (Enumeration efkeys = current.getForeignKeys().elements();
                efkeys.hasMoreElements();) {
                ColumnMap fk = (ColumnMap) efkeys.nextElement();
                String ptable = fk.getParentColumn().substring(0, 
                                                               fk.getParentColumn().lastIndexOf("."));
                String pcol = fk.getParentColumn().substring(
                                                            fk.getParentColumn().lastIndexOf(".")+1);
                String col = fk.getChildColumn().substring(
                                                          fk.getChildColumn().lastIndexOf(".")+1);


                String p_qualified_column = parent_sql_table.getAlias() + "." + pcol;
                if (ptable.equalsIgnoreCase(parent_full_name)) {
                    if (!onlyUseParentSelectedColumns) {
                        addJoin(p_qualified_column, t.getAlias() + "." + col);  
                    }
                    else {
                        if (parent_selected_columns.contains(p_qualified_column.toLowerCase())) {
                            addJoin(p_qualified_column, t.getAlias() + "." + col);  
                        }
                        else if (parent_selected_columns.contains(pcol.toLowerCase())) {
                            // a parent column may not be qualified by the table name
                            // because it is derived from "*".
                            addJoin(pcol, t.getAlias() + "." + col);    
                        }
                    }
                }
            }

            // match its primary keys with the parent foreign key.
            Enumeration efkeys = parent_table.getForeignKeys().elements();
            if (!efkeys.hasMoreElements()) {
                // warn user if join cannot be accomplished
                ErrorDialog.showWarning(this,
                                        com.klg.jclass.datasource.LocaleBundle.string(
                                                                                     com.klg.jclass.datasource.LocaleBundle.no_foreign_keys));
                return;
            }

            for (; efkeys.hasMoreElements();) {
                ColumnMap fk = (ColumnMap) efkeys.nextElement();
                String ptable = fk.getParentColumn().substring(0, 
                                                               fk.getParentColumn().lastIndexOf("."));
                String pcol = fk.getParentColumn().substring(
                                                            fk.getParentColumn().lastIndexOf(".")+1);
                String col = fk.getChildColumn().substring(
                                                          fk.getChildColumn().lastIndexOf(".")+1);
                String p_qualified_column = parent_sql_table.getAlias() + "." + col;
                if (ptable.equalsIgnoreCase(current_full_name)) {
                    if (!onlyUseParentSelectedColumns) {
                        addJoin(p_qualified_column, t.getAlias() + "." + pcol); 
                    }
                    else {
                        if (parent_selected_columns.contains(p_qualified_column.toLowerCase())) {
                            addJoin(p_qualified_column, t.getAlias() + "." + pcol); 
                        }
                        else if (parent_selected_columns.contains(col.toLowerCase())) {
                            // a parent column may not be qualified by the table name
                            // because it is derived from "*".
                            addJoin(col, t.getAlias() + "." + pcol);    
                        }
                    }
                }
            }
        }
    }
}

}
