/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: NodePropertiesEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:11 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.customizer;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.EtchedBorder;
import javax.swing.tree.*;

import com.klg.jclass.datasource.*;
import com.klg.jclass.datasource.beans.*;
import com.klg.jclass.datasource.beans.SerializedProperties;
import com.klg.jclass.util.*;
import com.klg.jclass.util.internal.JClassInfo;
import com.klg.jclass.util.swing.JCAlignLayout;

/**
 * 	Serializes the data model into a file.  After this custom editor
 *	is closed, the file name is passed to the setter method.
 *	Then, the setter will deserialize the data model.
 */
public class NodePropertiesEditor extends CustomizePage implements
    ActionListener, FocusListener, JCDataListener, SerializationPageParent,
    DesignTimeMetaData {

JTabbedPane tab;
SerializationPage serializationPage;
DataModelTabs dataTabs;

JTextField description, modelName;

private TreeModel tree;
private static Thread showTipsThread;
private static boolean tipsShowed = false;
/** Only return a new instance after this editor is closed. */
private boolean returnNewInstance = false;
private boolean initializing = false;
private static int modelCounter = 0;
private static final int SERIALIZATION_PAGE = 0;
private static final int DATA_PAGE = 1;
private static final boolean TRACE = false;

public NodePropertiesEditor() {
    super(LocaleBundle.string(LocaleBundle.data_model));
    init();

    com.klg.jclass.datasource.customizer.CustomizerLauncher.setDefaultFont((Component)this, new Font("Dialog", Font.PLAIN, 9));

    // Show tips if necessary.
    // Use a thread to wait for the editor to show up
    // before opening the dialog.
    // Otherwise, VCafe 2.5 will hang.
    String s = JClassInfo.getProperty("datasource",
                                      "data_bean_component.customizer.show_tips_at_start");
    if (JCTypeConverter.toBoolean(s, true) && !tipsShowed) {
        if (showTipsThread == null) {
            showTipsThread = new Thread(new threadTipsDialog());
            showTipsThread.start();
        }
    }
}

public void init() {
    setLayout(new BorderLayout());

    // description and model name of the model
    JLabel lbl_description = new JLabel(LocaleBundle.string(LocaleBundle.description));
    JLabel lbl_model_name = new JLabel(LocaleBundle.string(LocaleBundle.model_name));
    description = new JTextField("", 30);
    modelName = new JTextField("", 30);
    tab = new JTabbedPane();
    // Create load/save panel
    tab.addTab(LocaleBundle.string(LocaleBundle.serialization),
               serializationPage = new SerializationPage(this));
    tab.addTab(LocaleBundle.string(LocaleBundle.data_model),
               dataTabs = new DataModelTabs(false, this));
    JPanel p = new JPanel();
    p.setBorder(new EtchedBorder());
    description.addActionListener(this);
    description.addFocusListener(this);
    modelName.addActionListener(this);
    modelName.addFocusListener(this);

    p.setLayout(new JCAlignLayout());
    p.add(lbl_description);
    p.add(description);
    p.add(lbl_model_name);
    p.add(modelName);

    add(p, BorderLayout.NORTH); 
    add(tab, BorderLayout.CENTER);

    setFont(new Font("Dialog", Font.PLAIN, 9));
}


/** Sets the bean to be customized.  It is used if the property editor
 *	is used as a page in a customizer.
 *	@param bean	bean to be customized.
 */
public void setBean(Object bean) {
    super.setBean(bean);
    if (bean instanceof JCData) {
        setValue(((JCData)bean).getNodeProperties());
    }
}

/** Helper method to enable/disable controls for start of editing.
 */
public void checkStartEditing(NodeProperties comp) {
    boolean hasResourceName = comp.isResourceNameSpecified();
    modelName.setEnabled(hasResourceName);
    description.setEnabled(hasResourceName);
    if (hasResourceName != tab.isEnabledAt(DATA_PAGE)) {
        tab.setEnabledAt(DATA_PAGE, hasResourceName);
    }
}

/** When value(s) of the data bean component is changed. */
public void JCDataValueChanged(JCDataEvent e) {
    NodeProperties comp = (NodeProperties) e.getSource();
    checkStartEditing(comp);

    try {
        support.firePropertyChange("NodeProperties", null, comp);
        // Create a meta data tree.
        createMetaDataTree();
        // Assign and reflect NodeProperties in the tab manager.
        description.setText(comp.getDescription());
        modelName.setText(comp.getModelName());
        dataTabs.setNodeProperties(comp);
        serializationPage.setSerializedProperties(comp);
    }
    catch (Exception ex) {
        if (TRACE) {
            ex.printStackTrace();
        }
        if (ex instanceof IOException) {
            ErrorDialog.showError(this,
                                  ex + System.getProperty("line.separator") +
                                  LocaleBundle.string(LocaleBundle.failed_to_serialize));
        }
    }
}

/** Returns a bean component by deserializing the serialization file
 *	if the editor is closed or if JBuilder is used.  
 *	Otherwise, the same instance of the bean 
 *	component is returned.
 *	In this way, a new instance of the bean component is created and thus
 *	guarantees code generation.  JBuilder 2.0 specifically compares the
 *	instances of new and old values.  Even if their contained properties
 *	are changed, no code is generated when the new and old bean component
 *	are the same instance.
 */
public Object getValue() {
    // Only return a new instance after this editor is closed.
//FIXOBSOLETE
//	if (!JCEnvironment.isJBuilder() && !returnNewInstance) {
    if (!returnNewInstance) {
        return target;
    }


    // Save any changes.
    saveToFile();

    // Get a new instance.
    Object new_comp = null;
    try {
        if (TRACE) {
            System.out.println("Editor.getValue()::fromFile: " + 
                               ((SerializedProperties)target).getSerializationFile());
            Thread.currentThread().dumpStack();
        }
        new_comp = SerializedProperties.fromFile(
                                                ((SerializedProperties)target).getSerializationFile());
        returnNewInstance = false;
    }
    catch (Exception ex) {
        if (TRACE) {
            ex.printStackTrace();
        }
        ErrorDialog.showError(null, ex + System.getProperty("line.separator") +
                              LocaleBundle.string(LocaleBundle.failed_to_deserialize));
    }
    return new_comp;
}


/** Sets the new data bean model. */
public void setValue(Object value) {
    if (TRACE) {
        System.out.println("setValue to value: " + value.toString());
    }

    if (value instanceof String) {
        // data model is alredy serialized from data on this editor.
        if (TRACE) {
            System.out.println("Deserialize from file: " + value);
        }
        try {
            NodeProperties new_model = (NodeProperties)
                                       NodeProperties.fromFile((String) value);

            // target MUST be set before firePropertyChange() 
            // so that the new bean data model
            // is used to set the bean.
            Object old_target = target;
            target = new_model;
            if (TRACE) {
                System.out.println("Change model from " + 
                                   System.getProperty("line.separator") + old_target + 
                                   System.getProperty("line.separator") + "to" + 
                                   System.getProperty("line.separator") + new_model);
            }

            // add event listener to this new NodeProperties
            new_model.addJCDataListener(this);

            // In case the file is not the same as the serialized file name for 
            // the serialization file property.  Set the serialization file to 
            // the file name.
            new_model.setSerializationFile((String) value);
            new_model.fireJCDataEvent(new JCDataEvent((Object)new_model));

            support.firePropertyChange("NodeProperties", old_target, target);
        }
        catch (Exception ex) {
            if (TRACE) {
                ex.printStackTrace();
            }
            if (ex instanceof FileNotFoundException) {
                ErrorDialog.showError(this,
                                      ex + System.getProperty("line.separator") +
                                      LocaleBundle.string(LocaleBundle.failed_to_load_file_not_found));
            }
            else {
                ErrorDialog.showError(this,
                                      ex + System.getProperty("line.separator") +
                                      LocaleBundle.string(LocaleBundle.failed_to_deserialize));
            }

            // throw an exception so loadFromFile() can catch it
            // and indicates loading is unsuccessful.
            throw new IllegalArgumentException(ex.toString());
        }
    }
    else if (value instanceof NodeProperties) {
        // Reflects the data model on the editor.
        // firePropertyChange() is not called here because there is no change.

        // Returns a new instance in JBuilder.  Otherwise, no code is
        // generated.
        NodeProperties comp = (NodeProperties) value;
//FIXOBSOLETE
/*
    if (JCEnvironment.isJBuilder()) {
        try {
            if (!comp.isResourceNameSpecified()) {
                // Use a new bean component if the one passed in here
                // is never initialized.  It is to avoid loading in
                // an old default serialization file.
                comp = new NodeProperties();
            } else {
                comp = (NodeProperties) 
                    NodeProperties.fromFile(comp.getSerializationFile());
            }
        } catch (Exception ex) {
            if (TRACE) {
                ex.printStackTrace();
            }
            ErrorDialog.showError(this, 
                        ex + System.getProperty("line.separator") + 
                        LocaleBundle.string(LocaleBundle.failed_to_deserialize));
        }
    } else  if (JCEnvironment.isVisualAge()) {
        // In VisualAge 2.0, setValue() is called after every change.
        // This behaviour screws up the current node selection.
        // This spurious call is ignored.
        if (target != null) {
            return;
        } else {
            comp = (NodeProperties) value;
        }
    } else {
        comp = (NodeProperties) value;
    }
*/

        comp = (NodeProperties) value;
        comp.addJCDataListener(this);

        try {
            target = comp;
            if (TRACE) {
                System.out.println("Show model \n" + comp);
            }
            showNewBeanComponent();
        }
        catch (Exception e) {
            if (TRACE) {
                e.printStackTrace();
            }
            ErrorDialog.showError(this, 
                                  e + System.getProperty("line.separator") + 
                                  LocaleBundle.string(LocaleBundle.failed_to_serialize));
        }
    }
}

public void showNewBeanComponent() {
    initializing = true;

    // Create a meta data tree.
    createMetaDataTree();

    NodeProperties comp = (NodeProperties) target;
    description.setText(comp.getDescription());
    modelName.setText(comp.getModelName());
    serializationPage.setSerializedProperties(comp);
    dataTabs.setNodeProperties(comp);
    initializing = false;
}


/** Returns a string representation of the property for display. */
public String getAsText() {
    return LocaleBundle.string(LocaleBundle.click_to_edit);
}

/** Paints the value. */
public void paintValue(Graphics gc, Rectangle box) {
    String s = getAsText();
    FontMetrics fm = gc.getFontMetrics();
    int vpad = (box.height - fm.getAscent())/2;
    gc.drawString(s, box.x, box.y+box.height-vpad);
}

/** Returns the name of the serialization file. */
public String getJavaInitializationString() {
    NodeProperties comp = (NodeProperties) target;

    // For code-generation-based IDE like VCafe
    // getAsText() is called and the initial value is stored. 
    String file = serializationPage.getResourceName();
    if (file == null) {
        file = new String("");
    }
    else {
        file = file.trim();
    }
    String s = new String("\"" + file.replace('\\', '/') + "\", getClass()");

    // At design-time, JBuilder cannot load the serialization file 
    // based on resource name.  That's why the seriliaztion
    // file name is required.  (PR6044)
//FIXOBSOLETE
//	if (JCEnvironment.isJBuilder()) {
//		s += ", \"" + comp.getSerializationFile().replace('\\', '/') + "\"";
//	}

    // If is IDE data source, pass the data source object name to the setter.
    if (target != null && 
        NodeProperties.isIDEDataSourceType(comp.getDataSourceType())) {
        s = s + ", " + comp.getDataSourceName();
    }
    return new String("new com.klg.jclass.datasource.beans.NodeProperties(" + s + ")"); 
}

public boolean loadFromFile(String file) {
    SerializedProperties comp = (SerializedProperties) target;
    String old_file = comp.getSerializationFile();
    // JDK bug - FilenameFilter.accept() is not called in FileDialog.
    // We'll have to make sure the filename has the right extension
    if (!file.toLowerCase().endsWith(LocaleBundle.serialization_extension)) {
        file += LocaleBundle.serialization_extension;
    }
    try {
        // Update property
        setValue(file);
    }
    catch (Exception ex) {
        if (TRACE) {
            ex.printStackTrace();
        }
        comp.setSerializationFile(old_file);

        return false;
    }
    // serialization file name may be changed.  Notify change if any.
    comp.fireJCDataEvent(new JCDataEvent((Object)comp));

    // show data bean component.
    showNewBeanComponent();

    return true;
}


/** Saves a bean component to a serialization file after
 *	this editor is closed. 
 */
public void saveToFile() {
    if (target == null) return;


    try {
        SerializedProperties comp = (SerializedProperties) target;
        if (comp.isSerializationRequired()) {
            // Warn user if resource name is not entered.
            if (!isClosed() && !comp.isResourceNameSpecified()) {
                ErrorDialog.showError(null,
                                      com.klg.jclass.datasource.LocaleBundle.string(
                                                                                   com.klg.jclass.datasource.LocaleBundle.enter_resource_name_and_save));
            }
            comp.toFile(comp, comp.getSerializationFile());
            returnNewInstance = true;
            // no need to fire property change after the editor is closed.
            // it might also be the reason why VCafe raises EOFException
            // and UTFException.
            //support.firePropertyChange(null, null, target);
        }
    }
    catch (Exception ex) {
        if (TRACE) {
            ex.printStackTrace();
        }
        ErrorDialog.showError(null,
                              ex + System.getProperty("line.separator") +
                              LocaleBundle.string(LocaleBundle.failed_to_serialize));
    }
}

public boolean isClosed() {
    return !description.isShowing();
}

/** Gets a meta data tree.  To do so, a temporary JCData is created
 *	from the serialization file.
 */
public TreeModel getMetaDataTree() {
    return tree;
}

void createMetaDataTree() {
    JCData bean = new JCData();
    try {
        NodeProperties comp = (NodeProperties) target;
        DataSourceBeanInstance.removeInstance(comp.getModelName());
        bean.setNodeProperties(comp);
    }
    catch (Exception ex) {
        // Don't show error because getMetaDataTree() is for internal
        // use of the editor.
        tree = null;
        return;
    }
    tree = bean.getMetaDataTree();
}

public MetaDataModel getMetaData(Object find_data, 
                                 DataSourceTreeNode start_node, DataSourceTreeNode meta_start) {
    if (start_node == null || meta_start == null) return null;

    return(MetaDataModel) meta_start;
}

public boolean foundData(Object find_data, DataSourceTreeNode node) {
    return find_data.equals(((DataSourceTreeNode)node).getUserObject());
}

public DataSourceTreeNode getModelTreeRoot() {
    DataSourceTreeNode node = new DataSourceTreeNode();
    node.setUserObject((NodeProperties) target);
    return node;
}

public void focusGained(FocusEvent e) {
}
public void focusLost(FocusEvent e) {
    if (initializing) return;

    setDescription();
    setModelName();
    ((NodeProperties)target).fireJCDataEvent(new JCDataEvent((Object)target));
}

public void actionPerformed(ActionEvent e) {
    if (initializing) return;

    setDescription();
    setModelName();
    ((NodeProperties)target).fireJCDataEvent(new JCDataEvent((Object)target));
}

void setDescription() {
    NodeProperties comp = (NodeProperties) target;
    comp.setDescription(description.getText());
}

void setModelName() {
    NodeProperties comp = (NodeProperties) target;
    if (modelName.getText() == null || modelName.getText().trim().length() == 0) {
        // default model name
        modelName.setText("JCData" + modelCounter++);
    }
    comp.setModelName(modelName.getText());
}

Frame getFrame() {    
/*
Frame frame = (BWTUtil.getFrame(this) == null) ?
                    new JFrame() : BWTUtil.getFrame(this);
return frame; 
*/
//FIXOBSOLETE
    return new JFrame();
}

class threadTipsDialog implements Runnable {

    public void run() {
        for (;;) {
            if (tipsShowed) {
                break;
            }
            // editor is up.  show tips now.
            if (!isClosed()) {
                showTipsDialog();
                break;
            }
            try {
                showTipsThread.sleep(500);
            }
            catch (Exception ex) {
            }
        }
    }

    private void showTipsDialog() {
        if (tipsShowed) return;

        tipsShowed = true;
        Frame frame = getFrame();
        TipsDialog tipsDialog = new TipsDialog(frame,
                                               LocaleBundle.string(LocaleBundle.title_tips_on_data_bean_customizer));
        String tip = LocaleBundle.string(LocaleBundle.tips_on_data_bean_customizer);
        tipsDialog.setShowTipsAtStart(true);
        tipsDialog.setMessage(tip);
        tipsDialog.setModal(true);
        tipsDialog.pack();
        tipsDialog.setVisible(true);
        boolean v = tipsDialog.getShowTipsAtStart();
        JClassInfo.setProperty("datasource", 
                               "data_bean_component.customizer.show_tips_at_start", 
                               new Boolean(v).toString());
        tipsDialog.dispose();
    }

}

}
