/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: SerializationPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:11 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.customizer;

import java.awt.*;
import java.awt.event.*;
import java.beans.PropertyEditor;
import java.io.*;

import javax.swing.*;
import javax.swing.border.EtchedBorder;
import javax.swing.filechooser.FileFilter;

import com.klg.jclass.datasource.*;
import com.klg.jclass.datasource.beans.*;
import com.klg.jclass.util.swing.*;
import com.klg.jclass.util.JCEnvironment;

/**
 * 	Serializes a bean component into a file.  After the
 *	parent custom editor is closed, the bean component
 *	will be serialized.
 */
public class SerializationPage extends JPanel implements ActionListener {

JButton btnLoad, btnSave;
JTextField resourceName;
SerializedProperties comp;
SerializationPageParent editor;

private boolean initializing = false;

private static final boolean TRACE = false;

public SerializationPage(SerializationPageParent editor) {
    super();
    this.editor = editor;
    init();
}

public void init() {
    setLayout(new BorderLayout());
    resourceName = new JTextField("", 40);
    btnLoad = new JButton(LocaleBundle.string(LocaleBundle.open3));
    btnSave = new JButton(LocaleBundle.string(LocaleBundle.save_as));
    JPanel buttons = new JPanel();
    JLabel lbl_resource = new JLabel(LocaleBundle.string(LocaleBundle.resource_name));
    JPanel p = new JPanel();
    JPanel p2 = new JPanel();
    JPanel g1 = new JPanel();
    g1.setBorder(new EtchedBorder());

    // resource name is assumed to be at the same path of the class that
    // uses the serialized bean.
    resourceName.setEditable(false);

    g1.setLayout(new GridLayout(2, 1));
    p2.setLayout(new JCAlignLayout());
    p2.add(lbl_resource);
    p2.add(resourceName);
    buttons.add(btnLoad);
    buttons.add(btnSave);
    g1.add(p2);
    g1.add(buttons);
    p.setLayout(new GridLayout(1, 1));
    p.add(g1);
    add(p, BorderLayout.NORTH);

    btnLoad.addActionListener(this);
    btnSave.addActionListener(this);
}


/** Sets the bean comp in this page and
 *	Displays resource name of the bean comp in this page.
 */
public void setSerializedProperties(SerializedProperties comp) {
    this.comp = comp;
    initializing = true;
    resourceName.setText(comp.getResourceName());
    initializing = false;
}

/** Gets the bean comp in this page. */
public SerializedProperties getSerializedProperties() {
    return comp;
}

/** Load or save data bean component into a file. */
public void actionPerformed(ActionEvent e) {
    if (initializing) return;

    if (e.getSource() instanceof JButton) {
        JButton btn = (JButton) e.getSource();
        if (btn.equals(btnLoad)) {
            // Load a component from a serialization file.
            loadFromFile();
        }
        else if (btn.equals(btnSave)) {
            // Save a component to a serialization file.
            saveToFile();

			// Reload the data bean component from the serialization
			// file.  This fixes a problem using JCHiGrid, JCData, or
			// JCTreeData in Forte 3.0.
			if (JCEnvironment.isDesignTime()) {
				reloadFromFile();
			}
        }
    }
}

/** Reloads a component from a serialization file. */
protected void reloadFromFile() {
	setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
	try {
		editor.loadFromFile(getSerializedProperties().getSerializationFile());
	}
	catch(Exception e) {
	}
	finally {
		setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	}
}

/** Loads a bean component from a serialization file. */
public void loadFromFile() {
    JFileChooser chooser = new JFileChooser(new File(System.getProperty("user.dir")));
    chooser.setMultiSelectionEnabled(false);
    chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
    FileFilter filter = new

                           JCFileFilter



                                        (LocaleBundle.serialization_extension,
                                         LocaleBundle.string(LocaleBundle.serialization_files));
    chooser.addChoosableFileFilter(filter);
    chooser.setFileFilter(filter);
    String s = getSerializedProperties().getDefaultSerializationFileToOpen();
    if (s != null) {
        File fileToOpen = new File(s);
        chooser.setSelectedFile(fileToOpen);
        chooser.ensureFileIsVisible(fileToOpen);
    }
    int option = chooser.showOpenDialog(this);
    if (option == JFileChooser.APPROVE_OPTION) {
        String serializationFile = chooser.getSelectedFile().getAbsolutePath();
        if (serializationFile != null && serializationFile.length() > 0 &&
            !serializationFile.toLowerCase().endsWith(LocaleBundle.serialization_extension)) {
            serializationFile += LocaleBundle.serialization_extension;
        }
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        try {
            editor.loadFromFile(serializationFile);
        }
        catch (Exception ex) {
        }
        finally {
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        }
    }
}


/** Saves a bean component into a serialization file. */
public void saveToFile() {
    JFileChooser chooser = new JFileChooser(new File(System.getProperty("user.dir")));
    chooser.setMultiSelectionEnabled(false);
    chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
    FileFilter filter = new

                           JCFileFilter



                                        (LocaleBundle.serialization_extension,
                                         LocaleBundle.string(LocaleBundle.serialization_files));
    chooser.addChoosableFileFilter(filter);
    chooser.setFileFilter(filter);
    chooser.setSelectedFile(new File(getSerializedProperties().getSerializationFile()));
    for (boolean keepGoing = true; keepGoing; ) {
        int option = chooser.showSaveDialog(this);
        if (option == JFileChooser.APPROVE_OPTION) {
            String serializationFile = chooser.getSelectedFile().getAbsolutePath();
            if ((new File(serializationFile)).exists()) {
                int selection = JOptionPane.showConfirmDialog(chooser,
                                                              serializationFile + LocaleBundle.string(LocaleBundle.file_exists),
                                                              LocaleBundle.string(LocaleBundle.save_as), JOptionPane.YES_NO_OPTION);
                if (selection != JOptionPane.YES_OPTION) {
                    continue;
                }
            }
            if (serializationFile != null && serializationFile.length() > 0 &&
                !serializationFile.toLowerCase().endsWith(LocaleBundle.serialization_extension)) {
                serializationFile += LocaleBundle.serialization_extension;
            }
            getSerializedProperties().setSerializationFile(serializationFile);
            // Default resource name to the name of the file, without path.
            File temp = new File(serializationFile);
            getSerializedProperties().setResourceName(temp.getName());
            setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            try {
                SerializedProperties.toFile(getSerializedProperties(), serializationFile);
            }
            catch (Exception ex) {
                if (TRACE) {
                    ex.printStackTrace();
                }
                if (ex instanceof FileNotFoundException) {
                    ErrorDialog.showError(this,
                                          ex + System.getProperty("line.separator") +
                                          LocaleBundle.string(LocaleBundle.failed_to_save_file_not_found));
                }
                else {
                    ErrorDialog.showError(this,
                                          ex + System.getProperty("line.separator") +
                                          LocaleBundle.string(LocaleBundle.failed_to_serialize));
                }
            }
            finally {
                setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            }
        }
        keepGoing = false;
    }
}

/** Gets the resource name the user enters. */
public String getResourceName() {
    return resourceName.getText();
}

public Insets getInsets() {
    return new Insets(5,5,5,5);
}

/** If the parent is closed by the user,
 *	parent's saveToFile() will be invoked
 *	to serialize a bean component of the parent.
 */
public void removeNotify() {
    try {
        editor.saveToFile();
    }
    catch (Exception ex) {
        if (TRACE) {
            ex.printStackTrace();
        }
        ErrorDialog.showError(this,
                              ex + System.getProperty("line.separator") +
                              LocaleBundle.string(LocaleBundle.failed_to_serialize));
    }
    super.removeNotify();
}




/** Accept a file if it ends with the specified extension.
 */



}
