/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: SqlBuilderPanel.java,v $ $Revision: 1.2 $
//			  $Date: 2006-03-17 21:13:11 $	$Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.customizer;

import java.awt.*;
import java.awt.event.*;

import java.sql.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;

import com.klg.jclass.datasource.*;
import com.klg.jclass.datasource.beans.*;
import com.klg.jclass.datasource.LocaleBundle;
import com.klg.jclass.datasource.util.*;
import com.klg.jclass.datasource.jdbc.DataTableConnection;

/**
 * This is the Panel which displays one or more tables.  It allows
 * users to drag the tables around, select columns from the tables
 * and make joins between columns in different tables.
 *
 * To drag a table, click down on it and without releasing the mouse
 * button move the table to its desired location. Release the button.
 *
 * To select a column from a table to be included in the list of
 * columns single click on the desired column. Clicking again will
 * unselect it.
 *
 * To form a join, click on a column in one table and drag the cursor
 * to the corresponding column in another table. Release the mouse.
 *
 * There is a "currently selected" table which will be shown in green.
 * Non-selected tables are shown in red.  A selected column in a table
 * will be shown in green, unselected, black.
 */
public class SqlBuilderPanel extends DataModelPage implements
	TableChooserParent, ActionListener, ListSelectionListener, MouseListener,
	MouseMotionListener {

Hashtable tables = new Hashtable(); 				// cache of table objects
Vector tableViews = new Vector();					// cache of a list of table view objects
Vector joinViews = new Vector();					// cache of a list of table view objects

JDesktopPane desktopPane;
// Popup menu
JPopupMenu popup;
JMenu mnAdd, mnDelete;
JMenuItem miAutoJoin, miRefresh;
JMenuItem miAddTable, miAddJoin, miDeleteTable, miDeleteJoin;

Connection connection;
boolean isConnectionChanged = false;
TableChooser tableChooser;

/** Sql statement built by the builder. */
SqlStatement sqlStatement;

/** is moving a table? */
boolean moving = false;

TableView selectedTableView;
JoinView selectedJoinView;

private final static boolean TRACE = false;

public SqlBuilderPanel() {
	this(null);
}

//FIXTODO
// Add DeleteTable button
// Make right-click menu popup when clicking on JInternalFrame titlebar

//FIX - why do I need this to get rid of the Border Insets??
public Insets getInsets() {
	return new Insets(0,0,0,0);
}

public SqlBuilderPanel(Connection connection) {
	super(LocaleBundle.sql_statement);
	this.connection = connection;
	sqlStatement = new SqlStatement();
	setLayout(new BorderLayout(0, 0));
	desktopPane = new JDesktopPane();
	desktopPane.setDesktopManager(new SimpleDesktopManager());
	setBorder(new javax.swing.border.EmptyBorder(0,0,0,0));
	desktopPane.setOpaque(false);
	add(desktopPane);
	// Create popup menu
	createPopupMenu();
	addMouseListener(this);
}

/** Sets the JDBC connection. */
public void setConnection(Connection connection) {
	this.connection = connection;
	isConnectionChanged = true;
}

/** Create popup menu */
protected void createPopupMenu() {
	popup = new JPopupMenu();
	mnAdd = new JMenu(LocaleBundle.string(LocaleBundle.add));
	miAddTable = new JMenuItem(LocaleBundle.string(LocaleBundle.table));
	miAddJoin = new JMenuItem(LocaleBundle.string(LocaleBundle.join));
	mnDelete = new JMenu(LocaleBundle.string(LocaleBundle.delete));
	miDeleteTable = new JMenuItem(LocaleBundle.string(LocaleBundle.table));
	miDeleteJoin = new JMenuItem(LocaleBundle.string(LocaleBundle.join));
	miAutoJoin = new JMenuItem(LocaleBundle.string(LocaleBundle.auto_join));
	miRefresh = new JMenuItem(LocaleBundle.string(LocaleBundle.refresh));

	mnAdd.setActionCommand(LocaleBundle.add);
	miAddTable.setActionCommand(LocaleBundle.add + LocaleBundle.table);
	miAddTable.addActionListener(this);
	miAddJoin.setActionCommand(LocaleBundle.add + LocaleBundle.join);
	miAddJoin.addActionListener(this);
	mnAdd.add(miAddTable);
	mnAdd.add(miAddJoin);


	mnDelete.setActionCommand(LocaleBundle.delete);
	miDeleteTable.setActionCommand(LocaleBundle.delete + LocaleBundle.table);
	miDeleteTable.addActionListener(this);
	miDeleteJoin.setActionCommand(LocaleBundle.delete + LocaleBundle.join);
	miDeleteJoin.addActionListener(this);
	mnDelete.add(miDeleteTable);
	mnDelete.add(miDeleteJoin);

	miAutoJoin.setActionCommand(LocaleBundle.auto_join);
	miAutoJoin.addActionListener(this);

	miRefresh.setActionCommand(LocaleBundle.refresh);
	miRefresh.addActionListener(this);

	popup.add(mnAdd);
	popup.add(mnDelete);
	popup.addSeparator();
	popup.add(miAutoJoin);
	popup.addSeparator();
	popup.add(miRefresh);
	//add(popup);
}

/** Shows the table chooser dialog. Table list is requery if connection is
 *		changed.
 */
public void showTableChooser() {
	if (tableChooser == null) {
		tableChooser = new TableChooser(this);
	}
	else if (isConnectionChanged) {
		// Refresh table list if connection is changed.
		tableChooser.refresh();
	}
	isConnectionChanged = false;
	tableChooser.show();
}

/** Add a table to the SQL builder panel.
 *		@param table_qualified_name table qualified name in the format of [qualifier.]table name
 */
public void addTable(String table_qualified_name) {
	addTable(table_qualified_name, null);
}

/** Add a table to the SQL builder panel.
 *		@param table_qualified_name table qualified name in the format of [qualifier.]table name
 *		@param alias alias of the table.
 */
public void addTable(String table_qualified_name, String alias) {
	if (table_qualified_name == null) return;

	String s = (alias == null) ? table_qualified_name :
			   new String(table_qualified_name + " " + alias);
	SqlStatementTable sst = SqlParser.parseTable(s);
	String table_alias = sst.getAlias();

	// Lookup the table
	Table t = (Table) tables.get(sst.getQualifiedName());
	if (t == null) {
		t = new Table(connection, sst.getQualifier(), sst.getTableName());

		// use schema as qualifier if no qualifier is defined.
		if (sst.getQualifier() == null) {
			sst.setQualifier(t.schema);
		}
		tables.put(sst.getQualifiedName(), t);
	}


	// Add the table to the SQL statement.
	sst = sqlStatement.addTable(sst.getQualifiedName(), sst.getAlias());
	sqlStatement.notifyObservers(this);
	if (sst != null) table_alias = sst.getAlias();

	if (getTableView(t, table_alias) == null) {
		addTableView(t, table_alias);
	}
}


public void addTableView(String table_qualified_name, String alias) {
	if (table_qualified_name == null) return;

	String s = (alias == null) ? table_qualified_name :
			   new String(table_qualified_name + " " + alias);
	SqlStatementTable sst = SqlParser.parseTable(s);
	Table t = (Table) tables.get(sst.getQualifiedName());
	if (t == null) {
		t = new Table(connection, sst.getQualifier(), sst.getTableName());

		// use schema as qualifier if no qualifier is defined.
		if (sst.getQualifier() == null) {
			sst.setQualifier(t.schema);
		}

		tables.put(sst.getQualifiedName(), t);
	}
	if (getTableView(t, alias) == null) {
		addTableView(t, alias);
	}
}

public void addTableView(Table t, String alias) {
	TableView tv = new TableView(t, alias);
	tv.setVisible(true);
	tv.setResizable(true);
//		tv.column_list.addListSelectionListener(this);
	tv.addMouseListener(this);
	tv.column_list.addMouseListener(this);
//		tv.column_list.getList().addMouseListener(this);
	tableViews.addElement(tv);
	tv.pack();
	desktopPane.add(tv);
	desktopPane.revalidate();
	refresh();
}

/** Delete a table from the SQL builder panel.
 *		@param tv TableView of the table to be removed.
 */
public void deleteTable(TableView tv) {
	if (tv == null) return;
	// Remove tv part in the sql statement first.
	sqlStatement.removeTable(tv.getAlias());
	sqlStatement.notifyObservers(this);

	// Remove TableView from the panel.
	tv.column_list.removeListSelectionListener(this);
	tv.removeMouseListener(this);
	tv.column_list.removeMouseListener(this);
	desktopPane.remove(tv);
	desktopPane.revalidate();

	// Delete JoinViews of this TableView
	deleteJoin(tv);

	Table t = tv.table;
	tableViews.removeElement(tv);
	Vector v = getTableViews(t);
	if (!(v != null && v.size() > 0)) {
		// No views for this table.  Remove Table as well.
		tables.remove(t.getFullName());
	}
	tv = null;
	refresh();
}

/** Displays values in the bean data model. */
public void setNodeProperties(NodeProperties m) {
	// Must set expert mode before clear so clear won't
	// refresh the internal text of sql statement.
	sqlStatement.setExpertMode(m.isExpertMode());
	clear();
	sqlStatement.setText(m.getStatement());
	sqlStatement.notifyObservers(this);

	if (m.isExpertMode()) {
		return;
	}

	convertSqlStatement();
	refresh();
}

private void clear() {
	selectedTableView = null;
	selectedJoinView = null;
	desktopPane.removeAll();	// Remove components on the builder panel.
	tables = new Hashtable();
	tableViews = new Vector();
	joinViews = new Vector();
	sqlStatement.clear();
}


/** Converts a SqlStatement into TableViews and JoinViews. */
void convertSqlStatement() {
	// Reverse Engineer the text sql statement in the builder
	for (Enumeration e = sqlStatement.getTables(); e.hasMoreElements();) {
		SqlStatementTable table = (SqlStatementTable) e.nextElement();
		addTableView(table.getQualifiedName(), table.getAlias());
	}
	for (Enumeration e = sqlStatement.getJoins(); e.hasMoreElements();) {
		ColumnMap cm = (ColumnMap) e.nextElement();
		// Find table views for parent and child tables.
		String parent_column = cm.getParentColumn();
		String child_column = cm.getChildColumn();
		String parent_alias = null;
		String child_alias = null;
		if (parent_column.lastIndexOf(".") > -1) {
			parent_alias = parent_column.substring(0, parent_column.lastIndexOf("."));
			parent_column = parent_column.substring(parent_column.lastIndexOf(".")+1);
		}
		if (child_column.lastIndexOf(".") > -1) {
			child_alias = child_column.substring(0, child_column.lastIndexOf("."));
			child_column = child_column.substring(child_column.lastIndexOf(".")+1);
		}
		TableView parent_tv = getTableView(parent_alias, parent_column);
		TableView child_tv = getTableView(child_alias, child_column);
		if (parent_tv != null && child_tv != null) {
			addJoin(parent_tv, parent_column, child_tv, child_column);
		}
	}
}

/**
 * Sets the layout manager for this container.	This method is
 * overridden to prevent the layout mgr from being set.
 */
public final void setLayoutManager(LayoutManager mgr) {
}

/** Columns in TableView is selected/deselected. */
/*
//FIXOBSOLETE
public void actionPerformed(JCActionEvent e) {
if (isInitializing()) return;

if (e.getSource() instanceof JList &&
((JList)e.getSource()).getParent() instanceof TableView) {
	// Columns in TableView selected/deselected
	addColumn((TableView) ((JList)e.getSource()).getParent());
}
}
*/

public void valueChanged(ListSelectionEvent e) {
	if (isInitializing()) return;

	if (e.getSource() instanceof JList &&
		((JList)e.getSource()).getParent() instanceof TableView) {
		// Columns in TableView selected/deselected
//				addColumn((TableView) ((JList)e.getSource()).getParent());
	}
}


void addColumn() {
	addColumn(selectedTableView);
}

void addColumn(TableView tv) {
	if (tv == null) return;

	// Double click on list.  Add selected columns
	int[] selected = tv.column_list.getSelectedIndices();
	if (selected != null) {
		boolean added = false;
		for (int i=0; i<selected.length; i++) {
			Object o = tv.column_list.getModel().getElementAt(selected[i]);
			String s = o.toString();
			// Set column alias to the column name.
			// SqlStatement will ensure alias uniqueness.
			// Don't set alias if the column is "*"
			String alias = (s != null && s.endsWith("*")) ? null : s;
			sqlStatement.addColumn(s, alias, tv.table.getFullName(), tv.getAlias());
			added = true;
		}
		if (added) sqlStatement.notifyObservers(this);
	}
}

/** Handles adding table from TableChooser and popup menu.
 */
public void actionPerformed(ActionEvent e) {
	if (isInitializing()) return;

	//
	//	Popup menus
	//
	if (e.getActionCommand().equals(LocaleBundle.add + LocaleBundle.table)) {
		// Show the table chooser dialog to add table(s).
		showTableChooser();
	}
	else if (e.getActionCommand().equals(LocaleBundle.add + LocaleBundle.join)) {
		addJoin();
	}
	else if (e.getActionCommand().equals(LocaleBundle.delete + LocaleBundle.table)) {
		// Delete table view
		if (selectedTableView != null) {
			deleteTable(selectedTableView);
			repaint();
		}
	}
	else if (e.getActionCommand().equals(LocaleBundle.delete + LocaleBundle.join)) {
		// Delete join
		if (selectedJoinView != null) {
			deleteJoin(selectedJoinView);
			repaint();
		}
	}
	else if (e.getActionCommand().equals(LocaleBundle.refresh)) {
		refresh();
		repaint();
	}
	else if (e.getActionCommand().equals(LocaleBundle.auto_join)) {
		autoJoin();
		repaint();
	}
}


void addJoin() {
	// Show the join dialog
	Vector v = new Vector();
	for (Enumeration en = sqlStatement.getJoins(); en.hasMoreElements();) {
		ColumnMap cm = (ColumnMap) en.nextElement();
		v.addElement(cm.getParentColumn() + " = " + cm.getChildColumn());
	}
	SqlStatement parent_sql = new SqlStatement(sqlStatement.getText());
	SqlStatement child_sql = new SqlStatement(sqlStatement.getText());
	JoinDialog jdlg = new JoinDialog(null, v,
									 parent_sql, getNodeProperties(),
									 child_sql, getNodeProperties(),
									 true, false);
	jdlg.pack();
	jdlg.show();

	// Clean the all joins first
	deleteAllJoins();

	// Add joins back
	addJoinFromStrings(jdlg.getJoinStrings());

	repaint();
}

void addJoinFromStrings(Vector v) {
	for (Enumeration ej = v.elements(); ej.hasMoreElements();) {
		// Add the joins
		String s = (String) ej.nextElement();
		String parent_alias = s.substring(0, s.indexOf("."));
		String parent_column = s.substring(s.indexOf(".")+1, s.indexOf(" = "));
		String child_alias = s.substring(s.indexOf(" = ") + 3, s.lastIndexOf("."));
		String child_column = s.substring(s.lastIndexOf(".")+1);
		TableView ptv = getTableView(parent_alias, parent_column);
		TableView ctv = getTableView(child_alias, child_column);
		if (ptv != null && ctv != null) {
			addJoin(ptv, parent_column, ctv, child_column);
		}
		else {
			// just add the text to the sql statement.
			sqlStatement.addJoin(parent_alias + "." + parent_column,
								 child_alias + "." + child_column);
			sqlStatement.notifyObservers(this);
		}
	}
}

/** Gets a JoinView for a Join. */
JoinView getJoinView(TableView primary, TableView foreign) {
	Enumeration e = joinViews.elements();
	while (e.hasMoreElements()) {
		JoinView jv = (JoinView) e.nextElement();
		if ((jv.parent.equals(primary) &&
			 jv.child.equals(foreign)) ||
			(jv.parent.equals(foreign) &&
			 jv.child.equals(primary))
		   ) {
			return jv;
		}
	}
	return null;

}

/** Gets a JoinView for a Join. */
JoinView getJoinView(Join join) {
	Enumeration e = joinViews.elements();
	while (e.hasMoreElements()) {
		JoinView jv = (JoinView) e.nextElement();
		if ((jv.join.parent.equals(join.parent) &&
			 jv.join.child.equals(join.child)) ||
			(jv.join.parent.equals(join.child) &&
			 jv.join.child.equals(join.parent))
		   ) {
			return jv;
		}
	}
	return null;
}

/** Gets a TableView for a component. */
private TableView getTableView(Component o) {
	Component comp = o;
	while (comp != null) {
		if (comp instanceof TableView) break;
		comp = comp.getParent();
	}
	return(TableView) comp;
}

/** Gets a list of TableView for a Table. */
Vector getTableViews(Table table) {
	Vector v = new Vector();
	Enumeration e = tableViews.elements();
	while (e.hasMoreElements()) {
		TableView tv = (TableView) e.nextElement();
		if (tv.table.equals(table)) {
			v.addElement(tv);
		}
	}
	return v;
}

/** Gets a TableView given the table and its alias.
 */
TableView getTableView(Table table, String alias) {
	for (Enumeration e = getTableViews(table).elements(); e.hasMoreElements();) {
		TableView tv = (TableView) e.nextElement();
		if (tv.getAlias().equals(alias)) return tv;
	}
	return null;
}

/** Gets a TableView given an table alias and a column. */
TableView getTableView(String alias, String column) {
	if (alias == null) {
		// No alias.  Check each table and see if it has the parent column
		// Only the one TableView is returned.
		Enumeration et = tables.elements();
		while (et.hasMoreElements()) {
			Table t = (Table) et.nextElement();
			if (t.containsColumn(column)) {
				Vector v = getTableViews(t);
				if (v != null && v.size() > 0) {
					return(TableView) v.elementAt(0);
				}
			}
		}
	}
	else {
		// Has alias.  Loop through TableView list.
		Enumeration et = tableViews.elements();
		while (et.hasMoreElements()) {
			TableView tv = (TableView) et.nextElement();
			if (tv.getAlias().equals(alias)) {
				return tv;
			}
		}
	}
	return(TableView) null;
}

void deleteAllJoins() {
	for (Enumeration e = joinViews.elements(); e.hasMoreElements();) {
		JoinView jv = (JoinView) e.nextElement();
		deleteJoin(jv);
	}
}


/** Delete a join. */
void deleteJoin(JoinView jv) {
	// Remove joins from the sql statement.
	sqlStatement.removeJoin(jv.parent.getAlias());
	sqlStatement.removeJoin(jv.child.getAlias());
	sqlStatement.notifyObservers(this);

	joinViews.removeElement(jv);
	jv = null;
}

/** Delete all joins of a TableView. */
void deleteJoin(TableView tv) {
	Enumeration e = joinViews.elements();
	while (e.hasMoreElements()) {
		JoinView jv = (JoinView) e.nextElement();
		if (jv.parent.equals(tv) || jv.child.equals(tv)) {
			deleteJoin(jv);
		}
	}
}

/** Create a join between 2 TableViews on a column pair. */
void addJoin(TableView primary, String pcol, TableView foreign, String fcol) {
	if (primary == null || foreign == null) return;

	JoinView jv = getJoinView(primary, foreign);
	if (jv == null) {
		// New join view between these 2 tables.
		Join join = new Join(primary.table, primary.getAlias(),
							 foreign.table, foreign.getAlias());
		join.add(pcol, fcol);
		jv = new JoinView(join, primary, foreign);
		joinViews.addElement(jv);
	}
	else {
		// JoinView already exists.  Add column pair to Join.
		jv.join.add(pcol, fcol);
	}
	sqlStatement.addJoin(primary.getAlias() + "." + pcol,
						 foreign.getAlias() + "." + fcol);
	sqlStatement.notifyObservers(this);
}

/** Create joins between a vector of primary TableViews and a vector of
 *		foreign TableViews on a column pair. */
void addJoin(Vector ptvs, String pcol, Vector ftvs, String fcol) {
	Enumeration e = ptvs.elements();
	while (e.hasMoreElements()) {
		TableView primary = (TableView) e.nextElement();
		Enumeration ef = ftvs.elements();
		while (ef.hasMoreElements()) {
			TableView foreign = (TableView) ef.nextElement();
			addJoin(primary, pcol, foreign, fcol);
		}
	}
}

/** Helper method to auto join tables based on primary/foreign key combinations. */
void autoJoin() {
	Enumeration e = tables.elements();
	while (e.hasMoreElements()) {
		Table current = (Table) e.nextElement();
		Vector current_tvs = getTableViews(current);

		Enumeration efkeys = current.getForeignKeys().elements();
		while (efkeys.hasMoreElements()) {
			ColumnMap fk = (ColumnMap) efkeys.nextElement();
			String ptable = fk.getParentColumn().substring(0,
														   fk.getParentColumn().lastIndexOf("."));
			String pcol = fk.getParentColumn().substring(
														fk.getParentColumn().lastIndexOf(".")+1);
			String col = fk.getChildColumn().substring(
													  fk.getChildColumn().lastIndexOf(".")+1);
			Table primary = (Table) tables.get(ptable);
			if (primary != null) {
				// The referenced primary table exists in the builder
				// Find all views of the table.
				Vector primary_tvs = getTableViews(primary);
				addJoin(primary_tvs, pcol, current_tvs, col);
			}
		}
	}
}

/** reposition all tables and joins. */
void refresh() {
	int offset = 20;
	JComponent parent = desktopPane;
	// Layout all the table views which have joins first.
	Enumeration e = joinViews.elements();
	Vector done = new Vector();
	int max_width = getSize().width;
	int x = getInsets().left, y = getInsets().top;
	int next_row_y = 0;

	// Add the table views that are in joins first.
	while (e.hasMoreElements()) {
		JoinView jv = (JoinView) e.nextElement();
		if (!done.contains(jv.parent)) {
			Dimension d = jv.parent.getPreferredSize();
			Rectangle r = jv.parent.getBounds();
			next_row_y = Math.max(y+r.height+offset, next_row_y);
			if (x+r.width+offset > max_width) {
				// Next row
				x = getInsets().left;
				y = next_row_y;
			}
			jv.parent.setLocation(x, y);
			x += r.width + offset;
			done.addElement(jv.parent);
		}
		if (!done.contains(jv.child)) {
			Rectangle r = jv.child.getBounds();
			next_row_y = Math.max(y+r.height+offset, next_row_y);
			if (x+r.width+offset > max_width) {
				// Next row
				x = getInsets().left;
				y = next_row_y;
			}
			jv.child.setLocation(x, y);
			x += r.width + offset;
			done.addElement(jv.child);
		}
	}

	// Add the rest of the table views.
	Component[] comps = parent.getComponents();
	for (int i=0; i<comps.length; i++) {
		Component c = comps[i];
		if (TRACE) {
			System.out.println("comps["+i+"]: " + c);
		}
		if (c instanceof TableView && !done.contains(c)) {
			Rectangle r = c.getBounds();
			next_row_y = Math.max(y+r.height+offset, next_row_y);
			if (x+r.width+offset > max_width) {
				// Next row
				x = getInsets().left;
				y = next_row_y;
			}
			c.setLocation(x, y);
			x += r.width + offset;
		}
	}
	setSize(getMinimumSize());
	if (getParent() != null) {
		getParent().doLayout();
	}
}

public Dimension getMinimumSize() {
	return getPreferredSize();
}

public Dimension getPreferredSize() {
	JScrollPane scrollpane =
	(JScrollPane) SwingUtilities.getAncestorOfClass(JScrollPane.class,
													this);

	if (scrollpane == null || getComponentCount() == 0) {
		return super.getPreferredSize();
	}
	return getMaxBounds(desktopPane);
}

protected Dimension getMaxBounds(JComponent c) {
	Rectangle r = new Rectangle();
	for (int i=0; i < c.getComponentCount(); i++) {
		r = r.union(c.getComponent(i).getBounds());
	}
	return r.getSize();
}

/*
public Dimension getPreferredSize() {
return getMinimumSize();
}

public Dimension getMinimumSize() {
Component[] comps = getComponents();
Point max_pt = new Point(0,0);
for (int i=0; i<comps.length; i++) {
	Component c = comps[i];

	// Get the bottom right point
	Rectangle r = c.getBounds();
	Point pt = new Point(r.x+r.width, r.y+r.height);
	max_pt.x = Math.max(max_pt.x, r.x+r.width);
	max_pt.y = Math.max(max_pt.y, r.y+r.height);
}
int width = max_pt.x + getInsets().left + getInsets().right,
	height = max_pt.y + getInsets().top + getInsets().bottom;
Dimension d = new Dimension(width, height);
if (getParent() != null && getParent() instanceof java.awt.ScrollPane) {
	Dimension parent = ((java.awt.ScrollPane)getParent()).getViewportSize();
	if (d.width < parent.width && d.height < parent.height) {
		d = parent;
	}
}
return d;

}
*/

public void paint(Graphics gc) {
	super.paint(gc);
	if (moving) {
		return;
	}

	// Draw all the joins last
	gc.setColor(Color.blue);
	Graphics2D gc2 = (Graphics2D)gc;
	gc2.setStroke(new BasicStroke(3f));
	Point from = new Point(), to = new Point();
	Enumeration e = joinViews.elements();
	while (e.hasMoreElements()) {
		JoinView jv = (JoinView) e.nextElement();
		jv.getLocations(from, to);
		gc2.drawLine(from.x, from.y, to.x, to.y);
	}

}

/** Find the JoinView that user clicks on.
 */
public JoinView mouseEventToJoinView(MouseEvent e) {
	Enumeration jvs = joinViews.elements();
	Point current = e.getPoint();
	while (jvs.hasMoreElements()) {
		JoinView jv = (JoinView) jvs.nextElement();
		if (jv.getPolygon().contains(current)) {
			return jv;
		}
	}
	return null;
}

//
//		Implements TableChooserMediator
//
public Connection getConnection() {
	return connection;
}

public void tableSelected(Vector selected) {
	// Add the selected tables
	if (selected == null) return;

	for (Enumeration e = selected.elements(); e.hasMoreElements();) {
		addTable((String) e.nextElement());
	}
	doLayout();
}


//----------------------------------------------------------------
public void mouseClicked(MouseEvent e) {
	if (e.getSource() instanceof JList &&
		((JList)e.getSource()).getParent().getParent().getParent() instanceof TableView) {
		if (e.getClickCount() == 2) {
			// Columns in TableView selected/deselected
			addColumn((TableView) ((JList)e.getSource()).getParent().getParent().getParent());
		}
	}
	else if ((e.getModifiers() & InputEvent.BUTTON1_MASK) == InputEvent.BUTTON1_MASK) {
		// Show join dialog if user clicks on a join line.
		JoinView jv = mouseEventToJoinView(e);
		if (jv != null) {
			// Convert the join into a list of join strings.
			Vector v = new Vector();
			for (Enumeration ej = jv.join.elements(); ej.hasMoreElements();) {
				ColumnMap cm = (ColumnMap) ej.nextElement();
				v.addElement(jv.parent.getAlias() + "." + cm.getParentColumn() + " = " +
							 jv.child.getAlias() + "." + cm.getChildColumn());
			}
			JoinDialog jdlg = new JoinDialog(null,
											 v, sqlStatement,
											 getNodeProperties());
			jdlg.pack();
			jdlg.show();

			// Clean the join between the 2 tables first
			deleteJoin(jv);

			// Add the joins back
			addJoinFromStrings(jdlg.getJoinStrings());
		}
	}
}

/** Set selected table view and join view.	Also, show popup menu. */
public void mousePressed(MouseEvent e) {
/*
if (BWTUtil.instanceOf(e.getSource(), "java.awt.Component")) {
	selectedTableView = getTableView((Component) e.getSource());
}
*/
	if (e.getSource() instanceof Component) {
		selectedTableView = getTableView((Component) e.getSource());
	}

	selectedJoinView = mouseEventToJoinView(e);

	if ((e.getModifiers() & InputEvent.META_MASK) == InputEvent.META_MASK) {
		// Only enable delete table if user clicks on a table.
		miDeleteTable.setEnabled(selectedTableView != null);

		// Only enable delete join if user clicks on a join.
		miDeleteJoin.setEnabled(selectedJoinView != null);

		if (!getNodeProperties().isExpertMode()) {
			popup.show(e.getComponent(), e.getX(), e.getY());
		}
	}
}

/** If a table view is being moved, place it to a new location.
 */
public void mouseReleased(MouseEvent e) {
/*
System.out.println("mouse released "+e.getSource());
if (!(e.getSource() instanceof TableView)) return;
if (moving) {
	moving = false;
	((TableView)e.getSource()).setLocation(
		Math.max(getInsets().left, e.getX()),
		Math.max(getInsets().top, e.getY()));
	((TableView)e.getSource()).validate();
	if (getParent() != null) {
		getParent().doLayout();
		getParent().repaint();
	}
	desktopPane.revalidate();
//				repaint();
}
*/
}

public void mouseEntered(MouseEvent e) {
}
public void mouseExited(MouseEvent e) {
}
/** If mouse drags on a table header, move the table. */
public void mouseDragged(MouseEvent e) {
/*
if (e.getSource() instanceof TableView) {
	moving = true;
}
if (e.getSource() instanceof TableView) {
	moving = true;
	((TableView)e.getSource()).setLocation(
		Math.max(getInsets().left, e.getX()),
		Math.max(getInsets().top, e.getY()));
	((TableView)e.getSource()).validate();
	if (getParent() != null) {
//			  getParent().doLayout();
		getParent().repaint();
	}
}
*/
}
public void mouseMoved(MouseEvent e) {
}
//----------------------------------------------------------------


public Frame getFrame() {
/*
Frame frame = (BWTUtil.getFrame(this) == null) ?
					new JFrame() : BWTUtil.getFrame(this);
return frame;
*/
//FIXOBSOLETE
	return new JFrame();
}

class SimpleDesktopManager extends DefaultDesktopManager {

	public void endDraggingFrame(JComponent f) {
		super.endDraggingFrame(f);
		moving = false;
		repaint();
	}

	public void dragFrame(JComponent f, int x, int y) {
		super.dragFrame(f, x, y);
		moving = true;
	}
}

}
