/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: SqlStatementView.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:11 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.customizer;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.event.DocumentEvent;

import com.klg.jclass.datasource.LocaleBundle;
import com.klg.jclass.datasource.util.SqlStatement;


/** 
 *	Observe a SqlStatement model and reflect any changes of the model.
 *	Also, statement can be manually typed in from the view.
 */
class SqlStatementView extends JPanel implements Observer, ItemListener {

JTextArea statementBox;
JCheckBox expertMode;
SqlStatement sqlStatement;  // a pointer to the sql statement model

/**
 *	SqlStatementView Constructor
 */
SqlStatementView(SqlStatement sql_statement) {
    super();
    setBorder(new TitledBorder(LocaleBundle.string(LocaleBundle.sql_statement)));
    statementBox = new JTextArea("",2,40);
    setLayout(new BorderLayout());
    add(new JScrollPane(statementBox), BorderLayout.CENTER);
    statementBox.setEditable(true);
    expertMode = new JCheckBox(LocaleBundle.string(LocaleBundle.expert_mode), false);
    JPanel p = new JPanel();
    p.setLayout(new BorderLayout());
    expertMode.addItemListener(this);
    add(p, BorderLayout.SOUTH);
    p.add(expertMode, BorderLayout.WEST);

    // set the model for this view
    setSqlStatement(sql_statement);
}   

/** Sets the SqlStatement object to be observed and displayed. */
void setSqlStatement(SqlStatement sql_statement) {
    this.sqlStatement = sql_statement;
    this.sqlStatement.addObserver(this);

    // display sql statement.
    refresh();
}


/** 
 * Clear out the sql box and the various vectors.
 */
void clear() {
    sqlStatement.clear();   
    sqlStatement.notifyObservers(this);
}

void refresh() {
    statementBox.setText(sqlStatement.getText());
    expertMode.setSelected(sqlStatement.isExpertMode());
}


/** SqlStatement model is changed, update view. */
public void update(Observable o, Object arg) {
    if (arg == null || !this.equals(arg)) {
        refresh();
    }
}

/** 
 * Get the text from the text area.
 */
String getText() {
    return statementBox.getText();
}

/** Change the underlining SQL Statement in the model and display the string. */
void setText(String s) {
    if (sqlStatement.getText() == null || !sqlStatement.getText().equals(s)) {
        sqlStatement.setText(s);
        sqlStatement.notifyObservers(this);
    }
}

public void itemStateChanged(ItemEvent e) {
    sqlStatement.setExpertMode(expertMode.isSelected());
    sqlStatement.notifyObservers(this);
}

} // SqlStatementView
