/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Table.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:11 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.customizer;

import java.sql.*;
import java.util.*;



import com.klg.jclass.datasource.util.ColumnMap;


/** This is a model for a database table. It contains a list of columns,
 *	primary keys and foreign keys.
 *	Some JDBC drivers may not support getting columns, primary keys or foreign keys.
 */
class Table {

/** contained columns. */
Hashtable columns = new Hashtable();
Vector fkeys = new Vector();    // foreign key column names
Vector pkey_columns = new Vector(); // primary key column names

/** Table name without the schema part. */
String name;

/** schema name */
String schema;

/** 
 *	Creates a table.
 *	@param connection JDBC connection
 *	@param table table name in the format of [schema.]table
 */
public Table(Connection connection, String table) {
    String schema = null;
    String table_name = table;
    if (table.indexOf(".") > -1) {
        schema = table.substring(0, table.indexOf("."));
        table_name = table.substring(table.indexOf(".")+1);
    }
    init(connection, schema, table_name);
}

/** 
 *	Creates a table.
 *	@param connection JDBC connection
 *	@param schema schema of the table 
 *	@param table table name without schema.
 */
public Table(Connection connection, String schema, String table_name) {
    init(connection, schema, table_name);
}

void init(Connection connection, String schema, String table_name) {
    if (connection == null) return;

    this.schema = schema;
    this.name = table_name;
    try {
        DatabaseMetaData data = connection.getMetaData();
        // Get primary keys
        ResultSet pkeys_rs = null;
        try {
            pkeys_rs = data.getPrimaryKeys(null, schema, table_name);
        }
        catch (Exception e) {
            try {
                pkeys_rs = data.getBestRowIdentifier(null, schema, table_name,
                                                     DatabaseMetaData.bestRowSession, true);
            }
            catch (Exception exc) {
            }
        }
        if (pkeys_rs != null) {
            while (pkeys_rs.next()) {
                String column = pkeys_rs.getString("COLUMN_NAME").trim(); // column names
                pkey_columns.addElement(column);
            }
            pkeys_rs.close();
        }

        // ordinal position of the column in the table
        try {
            ResultSet columns_rs = data.getColumns(null, schema, table_name, "%");
            while (columns_rs.next()) {
                // build column objects for this tables columns_rs
                String column_name = columns_rs.getString(4).trim();    // column name
                if (this.schema == null) {
                    this.schema = columns_rs.getString(2); // schema name
                }
                int type = columns_rs.getShort(5);              // java.sql.Types 
                int width = columns_rs.getShort(7);             // max data width 
                int scale = columns_rs.getShort(9);             // radix 10 or 2 
                int allow_null = columns_rs.getInt(11);         // cardinality
                boolean is_pkey = pkey_columns.contains(column_name);

                // create the column
                Column column = new Column(column_name, type, width, 
                                           scale, allow_null, is_pkey);

                // add it to the table
                columns.put(column_name, column);
            }
            columns_rs.close();
        }
        catch (Exception e) {
        }

        // Get foreign keys
        try {
            ResultSet fkeys_rs = data.getImportedKeys(null, schema, table_name);
            while (fkeys_rs.next()) {
                String fkey_column = fkeys_rs.getString(8).trim();
                String pkey_schema = fkeys_rs.getString(2);
                String pkey_table = ((pkey_schema == null) ? "" : pkey_schema + ".") + 
                                    fkeys_rs.getString(3);
                String pkey_column = fkeys_rs.getString(4);
                //ForeignKey fk = new ForeignKey(fkey_column, pkey_table, pkey_column);
                fkeys.addElement(new ColumnMap(pkey_table + "." + pkey_column, fkey_column));
            }
            fkeys_rs.close();
        }
        catch (Exception e) {
        }
    }
    catch (Exception e) {
    }
}

/** Returns the table name in "schema.name" format. */
String getFullName() {
    return(schema == null) ? name : schema + "." + name;
}

/** Indicates if table has the column.
 *	@param column name of the column to be found.
 */
boolean containsColumn(String column) {
    return columns.containsKey(column);
}

Vector getForeignKeys() {
    return fkeys;
}

Vector getPrimaryKeys() {
    return pkey_columns;
}

/** Gets a sorted list of column names. */
String[] getColumnNames() {



    Vector v = new Vector();


    Enumeration e = columns.elements(); 
    for (int i=0; e.hasMoreElements(); i++) {
        Column c = (Column) e.nextElement();
        if (!v.contains(c.name)) {
            v.addElement(c.name);
        }
    }



    Collections.sort(v);


    String[] s = new String[v.size()];
    v.copyInto(s);
    return s;
}




} // Table
