/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TableChooser.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:11 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.customizer;

import java.awt.*;
import java.awt.event.*;
import java.sql.*;
import java.util.Vector;

import javax.swing.*;

import com.klg.jclass.datasource.*;


/** This is a dialog for choosing table(s). */
public class TableChooser extends JDialog implements ActionListener,
    ItemListener {

/** Cache table names */
Vector tableNames = new Vector();
JCheckBox useSystemTables;
JButton btnClose, btnAdd, btnRefresh;
JList list;
JScrollPane scrollPane;

TableChooserParent parent;

public TableChooser(TableChooserParent parent) {
    super(parent.getFrame(), LocaleBundle.string(LocaleBundle.table_chooser));
    this.parent = parent;
    init();
}

public void init() {
    setModal(true);

    // Build list panel
    GridBagConstraints c = null;
    getContentPane().setLayout(new BorderLayout());

    JPanel list_panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
    useSystemTables = new JCheckBox(
                                   LocaleBundle.string(LocaleBundle.use_system_tables), false);
    JPanel p = new JPanel();
    JPanel panel = new JPanel();
    btnAdd = new JButton(LocaleBundle.string(LocaleBundle.add));
    btnRefresh = new JButton(LocaleBundle.string(LocaleBundle.refresh));
    btnClose = new JButton(LocaleBundle.string(LocaleBundle.close));

    list = new JList();
    scrollPane = new JScrollPane(list);

    useSystemTables.addItemListener(this);
    list_panel.add(useSystemTables);

    panel.setLayout(new BorderLayout());
    panel.add(p, BorderLayout.NORTH);

    btnAdd.addActionListener(this);
    btnRefresh.addActionListener(this);
    btnClose.addActionListener(this);
    p.setLayout(new GridBagLayout());
    c = new GridBagConstraints();
    c.anchor = GridBagConstraints.NORTH;
    c.fill = GridBagConstraints.HORIZONTAL;
    c.gridheight = c.gridwidth = 1;
    c.gridy = 0;
    c.insets = new Insets(5, 5, 5, 5);
    p.add(btnAdd, c);
    c.gridy = 1;
    c.insets = new Insets(0, 5, 5, 5);
    p.add(btnRefresh, c);
    c.gridy = 2;
    c.insets = new Insets(0, 5, 0, 5);
    p.add(btnClose, c);

    getContentPane().add(scrollPane, BorderLayout.CENTER);
    getContentPane().add(list_panel, BorderLayout.SOUTH);
    getContentPane().add(panel, BorderLayout.EAST);
    pack();
    refresh();

    addWindowListener(new java.awt.event.WindowAdapter() {
                          public void windowClosing(java.awt.event.WindowEvent e) {
                              setVisible(false);
                          }
                      });
}

public Dimension getPreferredSize() {
    return new Dimension(300, 240);
}

public Dimension getMinimumSize() {
    return getPreferredSize();
}

/** Centers the dialog relative to the parent frame. */
public void setVisible(boolean visible) {
    if (visible) {
        int x = getParent().getBounds().x + (getParent().getBounds().width-getBounds().width) / 2;
        int y = getParent().getBounds().y + (getParent().getBounds().height-getBounds().height) / 2;
        if (x >= 0 && y >= 0) {
            setLocation(x, y);
        }
    }
    super.setVisible(visible);
}

/** Get a list of tables from the database. */
public void getTables() {
    tableNames.removeAllElements();
    try {
        DatabaseMetaData data = parent.getConnection().getMetaData();

        // determine if system tables should be included
        String types[];
//FIX - it always seems to return SYSTEM tables with the Sybase 6 driver
        if (useSystemTables.isSelected()) {
            types = new String[] { "TABLE", "SYSTEM TABLE"};
        }
        else {
            types = new String[] { "TABLE"};
        }

        // get the table data from the sql server
        ResultSet rs = data.getTables(null, "%", "%", types);

        int count = 0;

        while (rs.next()) {
            String name = rs.getString(2) + "." + rs.getString(3);
            tableNames.addElement(name);

            count++;
        }
        rs.close();
    }
    catch (Exception e) {
        ErrorDialog.showError(this,
                              e + System.getProperty("line.separator") +
                              LocaleBundle.string(LocaleBundle.failed_to_get_a_list_of_tables));
    }
}


/** Refresh the list of tables */
public void refresh() {
    list.removeAll();
    getTables();
    if (tableNames.size() > 0) {
        list.setListData(tableNames);
    }
}

/** Gets the selected tables and send them to the parent.
 */
public void getTableSelected() {
    // Add the selected tables
    int[] selected = list.getSelectedIndices();
    if (selected != null) {
        Vector v = new Vector();
        for (int i=0; i<selected.length; i++) {
            v.addElement((String) list.getModel().getElementAt(selected[i]));
        }
        list.getSelectionModel().clearSelection();
        parent.tableSelected(v);
    }
}

public void actionPerformed(ActionEvent e) {
    if (e.getSource() instanceof JButton) {
        JButton btn = (JButton) e.getSource();
        if (btn.equals(btnAdd)) {
            getTableSelected();
        }
        else if (btn.equals(btnRefresh)) {
            // Refresh the list of tables
            refresh();
        }
        else if (btn.equals(btnClose)) {
            setVisible(false);         // hide the dialog
        }
    }
}

/** Tables are selected.
public void actionPerformed(JCActionEvent e) {
if (e.getSource() instanceof JList) {
    getTableSelected();
}
}
 */

/** Use system table is changed.  Refresh table list.
 */
public void itemStateChanged(ItemEvent e) {
    refresh();
}

}
