/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TableView.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:11 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.customizer;

import java.awt.*;
import java.awt.event.*;
import java.util.*;



import javax.swing.*;

import com.klg.jclass.util.*;
import com.klg.jclass.util.swing.*;

class TableView extends JInternalFrame {

KeyCellRenderer keyCellRenderer; 
Table table;                // table model
String alias;                // table alias name

JList column_list;
Component header;
boolean    expanded = true;    // is the table expanded to show columns?

// Icons for primary key column
static final String pkey_image_string[] = {
    "  ......        ",
    " .XXXXXX.       ",
    " .XX..XXX.      ",
    ".XX. o.XXX.     ",
    ".XX. o.XXX.     ",
    " .XX..XXX.      ",
    "  .XXXXo.       ",
    "    .XXo.       ",
    "   .XXXo.       ",
    "    .oXo.       ",
    "     .Xo.       ",
    "    .XXo.       ",
    "     .oo.       ",
    "       .        ",
    "                ",
    "                "};

transient Image pkey_image = null;
transient Icon pkey_icon = null;

static void addUnique(Vector v, Object object) {
    if (!v.contains(object)) {
        v.addElement(object);
    }
}

TableView(Table table, String alias) {
    this.table = table;    
    this.alias = alias;

    // Create the primary key icon
    JCIconCreator icon = new JCIconCreator(16, 16);
    icon.setColor('X', new Color(235,235,128));
    icon.setColor('.', Color.black);
    icon.setColor('o', new Color(235,235,128).darker());
    icon.setPixels(pkey_image_string);
    pkey_icon = icon.getIcon();
    pkey_image = icon.getIcon().getImage();
    keyCellRenderer = new KeyCellRenderer();
    getContentPane().setLayout(new BorderLayout());

    setTitle(alias);

    // Create column list.
    Vector v = new Vector();
    Vector v_pkeys = new Vector();
    Vector v_temp = new Vector();
    addUnique(v, "*");    // All columns
    Enumeration e = table.columns.elements();
    while (e.hasMoreElements()) {
        Column c = (Column) e.nextElement(); 
        if (c.is_pkey) {
            v_pkeys.addElement(c);
        }
        else {
            v_temp.addElement(c);
        }
    }
    // Sort the vectors
    Comparator comparator = new ColumnComparator();
    Collections.sort(v_pkeys, comparator);
    Collections.sort(v_temp, comparator);

    for (int i=0; i<v_pkeys.size(); i++) {
        addUnique(v, v_pkeys.elementAt(i)); 
        
    }
    for (int i=0; i<v_temp.size(); i++) {
        addUnique(v, v_temp.elementAt(i));
        
    }




    column_list = new JList(v);

    column_list.setCellRenderer(keyCellRenderer);
    column_list.setBackground(SystemColor.control);

    getContentPane().add(column_list, BorderLayout.CENTER);
}

/** Returns alias name of the table. */
String getAlias() {
    return alias;
}


// Display an icon and a string for each object in the list.
class KeyCellRenderer extends JLabel implements ListCellRenderer {

    public KeyCellRenderer() {
        setHorizontalTextPosition(2);
        setOpaque(true);
        setBorder(new javax.swing.border.EmptyBorder(0,5,0,5));
    }

    public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
        if (isSelected) {
            setBackground(list.getSelectionBackground());
            setForeground(list.getSelectionForeground());
        }
        else {
            setBackground(list.getBackground());
            setForeground(list.getForeground());
        }
        String s = "";
        Icon icon = null;
        if (value instanceof Column) {
            Column column = (Column)value;
            s = column.toString();
            if (column.is_pkey) {
                icon = pkey_icon;
            }
        }
        else {
            s = value.toString();
        }
        setText(s);
        setIcon(icon);
        return this;
    }
}


class ColumnComparator implements Comparator {

    public int compare(Object o1, Object o2) {
        return((Column)o1).name.compareTo(((Column)o2).name);
    }

    public boolean equals(Object o) {
        return equals(o);
    }
}

} 
