/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TreeEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:11 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.customizer;

import java.awt.*;
import java.awt.event.*;

import java.io.FileNotFoundException;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import com.klg.jclass.datasource.*;
import com.klg.jclass.datasource.beans.*;

import com.klg.jclass.util.swing.JCAlignLayout;


/**	Abstract implementation to handle graphical interface for a hierarchical property.
 *	Subclass needs to implement data specific operations.
 *	The property to be customized is assumed to be serialized into a file.
 */
abstract public class TreeEditor extends CustomizePage implements
    FocusListener, ActionListener, ListSelectionListener, 
    TreeSelectionListener, SerializationPageParent, DesignTimeMetaData {

protected JButton btnAdd, btnDelete, btnDescription, btnClear;

/** Outliner shows the hierarchical structure of tree data bean. 
 * Each outliner folder node contains user data of EditorTreeNode.  
 * Each tree node carries data of NodeProperties.
 */
protected JTree outliner;
protected JTabbedPane tabManager;
protected JTextField modelName;

private Progress progress = null;
private static int descriptionCounter = 0;
/** Only return a new instance after this editor is closed. */
private boolean returnNewInstance = false;
private static final boolean TRACE = false;

public TreeEditor(String title) {
    this(title, false);
}

public TreeEditor(String title, boolean externalDS) {
    super(title);
    init(externalDS);
}

public void addNotify() {
    if (progress == null) {
//		progress = new Progress();
    }
    super.addNotify();
}

public void init() {
    init(false);
}

public void init(boolean externalDS) {
    setLayout(new BorderLayout());

    // Create outliner panel
    JPanel outliner_panel = new JPanel();
//	outliner_panel.setInsets(new Insets(5,0,0,5));
    outliner_panel.setLayout(new BorderLayout());

    outliner = new JTree();
    outliner.setShowsRootHandles(true);
    outliner.setEditable(true);
    outliner.setModel(null);

    // Create outliner button panel

    // model name of the model
    JPanel outliner_buttonPanel = new JPanel();
    JPanel p1 = new JPanel();
    JPanel p2 = new JPanel();
    btnAdd = new JButton(LocaleBundle.string(LocaleBundle.add3));
    btnDelete = new JButton(LocaleBundle.string(LocaleBundle.delete));
    btnClear = new JButton(LocaleBundle.string(LocaleBundle.clear));
    btnDescription = new JButton(LocaleBundle.string(LocaleBundle.description3));
    add(getNorthComponent(externalDS), BorderLayout.NORTH); 

    modelName.addActionListener(this);
    modelName.addFocusListener(this);
    p1.setLayout(new GridBagLayout());
    p2.setLayout(new GridBagLayout());
    GridBagConstraints c = new GridBagConstraints();
    c.anchor = GridBagConstraints.NORTH;
    c.fill = GridBagConstraints.BOTH;
    c.gridheight = c.gridwidth = 1;
    c.weightx = c.weighty = 1;
    c.insets = new Insets(5, 0, 0, 5);
    p1.add(btnAdd, c);
    p1.add(btnDelete, c);
    c.insets = new Insets(5, 0, 0, 0);
    p1.add(btnClear, c);
    p2.add(btnDescription, c);
    outliner_buttonPanel.setLayout(new GridLayout(2, 1));
    outliner_buttonPanel.add(p1);
    outliner_buttonPanel.add(p2);
    outliner_panel.add(new JScrollPane(outliner), BorderLayout.CENTER);
    outliner_panel.add(outliner_buttonPanel, BorderLayout.SOUTH);

    // Create tab manager panel
    tabManager = createTabs();

    if (tabManager != null) {
        add(tabManager, BorderLayout.EAST);
    }
    add(outliner_panel, BorderLayout.CENTER);

    outliner.addTreeSelectionListener(this);
    btnAdd.addActionListener(this); 
    btnDescription.addActionListener(this);
    btnDelete.addActionListener(this);
    btnClear.addActionListener(this);
}

public Component getNorthComponent(boolean externalDS) {
    JPanel p = new JPanel();
//	p.setBorder(new EtchedBorder(BevelBorder.LOWERED));
    modelName = new JTextField("", 30);
    p.add(new JLabel(LocaleBundle.string(LocaleBundle.model_name)));
    p.setLayout(new JCAlignLayout());
    p.add(modelName);

    return p;
}

public Insets getInsets() {
    return new Insets(5, 5, 5, 0);
}

/** Return a string of the serialization file. This string
 *	is used by getJavaInitializationString() and paintValue().
 */
public String getAsText() {
    // For code-generation-based IDE like VCafe
    // getAsText() is called and the initial value is stored. 
    return LocaleBundle.string(LocaleBundle.click_to_edit);
}

/** Paints the value. */
public void paintValue(Graphics gc, Rectangle box) {
    String s = getAsText();
    FontMetrics fm = gc.getFontMetrics();
    int vpad = (box.height - fm.getAscent())/2;
    gc.drawString(s, box.x, box.y+box.height-vpad);
}


/** Returns the name of the serialization file. */
public String getJavaInitializationString() {
    String file = getResourceName();
    if (file == null) {
        file = new String("");
    }
    else {
        file = file.trim();
    }
    return file.replace('\\', '/');
}

// Expand tree
protected void expandTree() {
    DataSourceTreeNode root = (DataSourceTreeNode)outliner.getModel().getRoot();
    if (root != null) {
        for (Enumeration e = root.breadthFirstEnumeration(); e.hasMoreElements();) {
            DataSourceTreeNode nextNode = (DataSourceTreeNode) e.nextElement();
            TreePath path = new TreePath(nextNode.getPath());
//            outliner.setSelectionPath(path);        
            outliner.expandPath(path);        
        }    
        // select the root, since it will have the connection information
        outliner.setSelectionPath(new TreePath(root.getPath()));        
    }
}

/** Helper method called recursively to show tree in outliner. */
protected void buildTree(EditorTreeNode folder, DataSourceTreeNode node) {
    
    if (node == (DataSourceTreeNode)null) {
        return;
    }
    
    
    if (node.getUserObject() == null) {
        return;
    }
    EditorTreeNode newFolder = new EditorTreeNode(node);
    
//	newFolder.setUserObject(node);

    // Add a new node to outliner
//	folder.insert(newFolder, newFolder.getChildCount() + 1);
    if (outliner.getModel() == null) {
        
        outliner.setModel(new DataSourceTreeModel(newFolder));
    }
    else {
        
        ((DataSourceTreeModel)outliner.getModel()).insertNodeInto(newFolder, folder, folder.getChildCount());
    }
    for (Enumeration e = node.children(); e.hasMoreElements();) {
        buildTree(newFolder, (DataSourceTreeNode) e.nextElement());
    }    
/*
TreeIterator ti = node.getIterator();
while (ti.hasMoreElements()) {
    TreeNode child = (TreeNode) ti.nextElement();
System.out.println("iterating over child " + child);
    showTree(newFolder, child);
}
*/
}

/** Adds a node to the selected outliner folder node. 
 *	A new tree node is created. It is attached node data returned by addNodeData().
 */
EditorTreeNode addNode(String description) {
    EditorTreeNode folder = null;
    if (outliner.getModel() != null) {
        folder = (EditorTreeNode)((outliner.getLastSelectedPathComponent() != null) ?
                                  outliner.getLastSelectedPathComponent() : outliner.getModel().getRoot());
    }
    DataSourceTreeNode treeNode = new DataSourceTreeNode();

    return addNode(folder, treeNode, description);
}

/** Adds a tree node to an outliner folder node. 
 *	@param folder folder in which a new node is added
 *	@param treeNode data in the new node
 *	@return node created
 */
EditorTreeNode addNode(EditorTreeNode folder, DataSourceTreeNode treeNode, String description) {
    DataSourceTreeNode nodeData = null;
    if (folder != null) {
        nodeData = (DataSourceTreeNode)folder.getUserObject();
    }
    addNodeData(nodeData, treeNode, description);
    EditorTreeNode newNode = new EditorTreeNode(treeNode);
    if (folder == null) {
        outliner.setModel(new DataSourceTreeModel(newNode));
        ((DataSourceTreeModel)outliner.getModel()).nodeStructureChanged(folder);
    }
    else {
        ((DataSourceTreeModel)outliner.getModel()).insertNodeInto(newNode, folder, folder.getChildCount());
    }
    TreePath path = new TreePath(newNode.getPath());
    outliner.setSelectionPath(path);
    return newNode;
}

/** Helper method to rename the selected node in the outliner. */
void setNodeDescription() {
    EditorTreeNode node = (EditorTreeNode) outliner.getLastSelectedPathComponent();
    if (node == null) {
        return;        
    };

    String description = node.getDescription();
    if (description == null || description.trim().length() == 0) {
        description = LocaleBundle.string(LocaleBundle.node) + 
                      descriptionCounter++;
        node.setDescription(description);
    }
    outliner.startEditingAtPath(outliner.getSelectionPath());
}

/** Remove a node from the outliner. */
void deleteNode() {
    EditorTreeNode node = (EditorTreeNode) outliner.getLastSelectedPathComponent();
    if (node == null) {
        return;
    }
    removeNodeData((DataSourceTreeNode) node.getUserObject());
    // Delete all children
    if (node.getChildCount() > 0) {
        node.removeAllChildren();
        ((DataSourceTreeModel)outliner.getModel()).nodeStructureChanged(node);
    }
    // look for the next node to be the selected one, after the delete
    EditorTreeNode currentCandidateNode = (EditorTreeNode)node.getNextNode();
    if (currentCandidateNode == null) {
        currentCandidateNode = (EditorTreeNode)node.getPreviousNode();
    }
    // Delete this node from its parent, if this node is not the root
    EditorTreeNode parentNode = (EditorTreeNode)node.getParent();
    if (parentNode == null) {
        outliner.setModel(null);
        clearData();
        nodeSelected(null);
    }
    else {
        ((DataSourceTreeModel)outliner.getModel()).removeNodeFromParent(node);
    }
    if (currentCandidateNode != null) {
        TreePath path = new TreePath(currentCandidateNode.getPath());
        outliner.setSelectionPath(path);
    }
    else {
    }
    ((SerializedProperties)target).fireJCDataEvent(new JCDataEvent((Object)target));
}

void clear() {
    outliner.setModel(null);
    clearData();
    nodeSelected(null);
    ((SerializedProperties)target).fireJCDataEvent(new JCDataEvent((Object)target));
}

/*
 * TreeSelectionListener implementation
 */
public void valueChanged(TreeSelectionEvent e) {
    TreePath path = e.getPath();
    nodeSelected((EditorTreeNode) path.getPathComponent(path.getPathCount()-1));
    outliner.scrollPathToVisible(path);
}

/** Handle add and delete. */
public void actionPerformed(ActionEvent e) {
    if (e.getSource() instanceof JButton) {
        JButton btn = (JButton) e.getSource();
        if (btn.equals(btnAdd)) {
            EditorTreeNode newFolder = null;
            newFolder = addNode(LocaleBundle.string(LocaleBundle.node) + 
                                descriptionCounter++);
            if (newFolder != null) {
                ((SerializedProperties)target).fireJCDataEvent(new JCDataEvent((Object)target));
            }
        }
        else if (btn.equals(btnDescription)) {
            setNodeDescription();
        }
        else if (btn.equals(btnDelete)) {
            deleteNode();
        }
        else if (btn.equals(btnClear)) {
            clear();
        }
    }
    else
        if (e.getSource() instanceof JTextField) {
        if (modelName.equals(e.getSource())) {
            setModelName();
            ((SerializedProperties)target).fireJCDataEvent(new JCDataEvent((Object)target));
        }
    }
}


public void focusGained(FocusEvent e) {
}
public void focusLost(FocusEvent e) {
    setModelName();
    EditorTreeNode current = 
    (EditorTreeNode) outliner.getLastSelectedPathComponent();
    if (current == null) {
        return;
    }
    ((SerializedProperties)target).fireJCDataEvent(new JCDataEvent((Object)target));
}

public void valueChanged(ListSelectionEvent e) {
}

/** Create the tabs.
 */
abstract public JTabbedPane createTabs();

/** Sets the bean to be customized.  It is used if the property editor
 *	is used as a page in a customizer.
 *	Must call super.setBean(bean);
 *	@param bean	bean to be customized.
 */
public void setBean(Object bean) {
    super.setBean(bean);
}

public boolean loadFromFile(String file) {
    SerializedProperties comp = (SerializedProperties) target;
    String old_file = comp.getSerializationFile();
    // JDK bug - FilenameFilter.accept() is not called in FileDialog.
    // We'll have to make sure the filename has the right extension
    if (!file.toLowerCase().endsWith(LocaleBundle.serialization_extension)) {
        file += LocaleBundle.serialization_extension;
    }
    try {
        // Update property
        setValue(file);
    }
    catch (Exception ex) {
        if (TRACE) {
            ex.printStackTrace();
        }
        comp.setSerializationFile(old_file);
        return false;
    }

    // show root
    showNewBeanComponent();

    return true;
}

/** Save a bean component to a serialization file after
 *	this editor is closed. 
 */
public void saveToFile() {
    if (target == null) return;

    try {
        SerializedProperties comp = (SerializedProperties) target;
        if (comp.isSerializationRequired()) {
            // Warn user if resource name is not entered.
            if (!isClosed() && !comp.isResourceNameSpecified()) {
                ErrorDialog.showError(null,
                                      com.klg.jclass.datasource.LocaleBundle.string(
                                                                                   com.klg.jclass.datasource.LocaleBundle.enter_resource_name_and_save));
            }
            comp.toFile(comp, comp.getSerializationFile());
            returnNewInstance = true;
            // no need to fire property change after the editor is closed.
            // it might also be the reason why VCafe raises EOFException
            // and UTFException.
            //support.firePropertyChange(null, null, target);
        }
    }
    catch (Exception ex) {
        if (TRACE) {
            ex.printStackTrace();
        }
        ErrorDialog.showError(null,
                              ex + System.getProperty("line.separator") +
                              LocaleBundle.string(LocaleBundle.failed_to_serialize));
    }
}


public boolean isClosed() {
    return(getParent() != null && getParent() instanceof java.awt.Component) ? 
    !getParent().isShowing() : !this.isShowing();
}

/** Returns a bean component by deserialized the serialization file
 *	if the editor is closed or JBuilder is used.  
 *	Otherwise, the same instance of bean 
 *	component is returned.
 *	In this way, a new instance of bean component is created and thus
 *	guarantee code-generation.  JBuilder 2.0 specifically compares the
 *	instances of new and old values.  Even if their contained properties
 *	are changed, no code is generated when the new and old bean component
 *	are the same instance.
 */
public Object getValue() {
    // Only return a new instance after this editor is closed.
//FIXOBSOLETE
//	if (!JCEnvironment.isJBuilder() && !returnNewInstance) {
    if (!returnNewInstance) {
        return target;
    }

    // Save any changes.
    saveToFile();

    // Get a new instance.
    Object new_comp = null;
    try {
        if (TRACE) {
            System.out.println("TreeEditor.getValue()::fromFile: " + 
                               ((SerializedProperties)target).getSerializationFile());
            Thread.currentThread().dumpStack();
        }
        new_comp = SerializedProperties.fromFile(
                                                ((SerializedProperties)target).getSerializationFile());
        returnNewInstance = false;
    }
    catch (Exception ex) {
        if (TRACE) {
            ex.printStackTrace();
        }
        ErrorDialog.showError(null,
                              ex + System.getProperty("line.separator") +
                              LocaleBundle.string(LocaleBundle.failed_to_deserialize));
    }
    return new_comp;
}


abstract public void setValue(Object value);


/** Gets a description for a node in the tree.  It is used
 *	when a tree is shown in the outliner.
 */
//abstract public String getNodeDescription(TreeNode node);

/** Calls when model name is changed. */
abstract public void setModelName();

/** Sets a description for a node in the tree.  
 */
//abstract public void setNodeDescription(TreeNode node, String description);

/** Creates node data to be added to the node of the tree.
 *	@param parent parent in which new node is created.
 *	@param current current node to be created. 
 *	@param description description of the node.
 */
abstract public void addNodeData(DataSourceTreeNode parent, DataSourceTreeNode current, String description);

/** Removes data from the component.
 *	@param node node in which data to be removed.
 */
abstract public void removeNodeData(DataSourceTreeNode node);

/** Clear data from the component.
 */
abstract public void clearData();

/** Gets the resource name of the bean component. */
abstract public String getResourceName();

/** Helper method to change the editor content when a node is selected. */
abstract public void nodeSelected(EditorTreeNode node);

abstract public void showNewBeanComponent();

protected class Progress implements Runnable {

    JDialog dlg = new JDialog(new JFrame());
    Thread thread = null;

    Progress() {
        // editor is up.  Don't show progress dialog.
        if (!TreeEditor.this.isClosed()) return;

        dlg.setTitle(LocaleBundle.string(LocaleBundle.please_wait));
        dlg.setModal(false);
        JLabel label = new JLabel(LocaleBundle.string(LocaleBundle.loading_property_editor_or_customizer));
        label.setHorizontalAlignment(SwingConstants.CENTER);
        dlg.getContentPane().add(label);
        dlg.pack();
        dlg.setSize(300, 100);
        setVisible(true);
        Thread thread = new Thread(this);
        thread.start();
    }

/** Centers the dialog relative to the parent frame. */
    public void setVisible(boolean v) {
        if (v) {
            centerDialog();
        }
        dlg.setVisible(v);
    }

/** 
 * Centers the dialog relative to the parent frame or screen.
 */
    void centerDialog() {
        int x = 0;
        int y = 0;
        Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
        x = (screenDim.width - dlg.getSize().width) / 2;
        y = (screenDim.height - dlg.getSize().height) / 2;
        dlg.setLocation(x, y);
    }

/** Show a dialog to indicate customizer is loading. */
    public void run() {
        for (;;) {
            try {
                thread.sleep(1000);
            }
            catch (Exception ex) {
            }
            if (!TreeEditor.this.isClosed()) {
                dlg.dispose();
                TreeEditor.this.progress = null;
                break;  
            }
        }
    }

} // Progress 


}
