/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TreePropertiesEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:11 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.customizer;

import java.awt.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.tree.*;

import com.klg.jclass.datasource.*;
import com.klg.jclass.datasource.beans.*;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.internal.JClassInfo;


/**
 * 	<p>Serializes tree data bean component into a file.  After this custom editor
 *	is closed, the name of a serialization file and an array of data source object
 *	names are passed to the setter method.
 *	Then, the setter will deserialize the tree data bean component.</p>
 *	<p>One only data source type is allowed for all the data sources 
 *	in a hierarchy.  The type is determined by the root.</p>
 */
public class TreePropertiesEditor extends TreeEditor implements JCDataListener {

SerializationPage serializationPage;
DataModelTabs dataTabs;

private static Thread showTipsThread;
private static boolean tipsShowed = false;
private boolean initializing = false;
private TreeModel tree;
private static int modelCounter = 0;
private static final int SERIALIZATION_PAGE = 0;
private static final int DATA_PAGE = 1;
private static final boolean TRACE = true;


public TreePropertiesEditor() {
    super(LocaleBundle.string(LocaleBundle.data_model));

    com.klg.jclass.datasource.customizer.CustomizerLauncher.setDefaultFont((Component)this, new Font("Dialog", Font.PLAIN, 9));

    // Show tips if necessary.
    // Use a thread to wait for the editor to show up
    // before opening the dialog.
    // Otherwise, VCafe 2.5 will hang.
    String s = JClassInfo.getProperty("datasource",
                                      "tree_data_bean_component.customizer.show_tips_at_start");
    if (JCTypeConverter.toBoolean(s, true) && !tipsShowed) {
        if (showTipsThread == null) {
            showTipsThread = new Thread(new threadTipsDialog());
            showTipsThread.start();
        }
    }
    checkStartEditing(null);
}

public void init() {
    super.init();
}

public JTabbedPane createTabs() {
    JTabbedPane tab = new JTabbedPane();
    tab.addTab(LocaleBundle.string(LocaleBundle.serialization),
               serializationPage = new SerializationPage(this));
    tab.addTab(LocaleBundle.string(LocaleBundle.data_model),
               dataTabs = new DataModelTabs(true, this));
    return tab;
}

/** Sets the bean to be customized.  It is used if the property editor
 *	is used as a page in a customizer.
 *	@param bean	bean to be customized.
 */
public void setBean(Object bean) {
    super.setBean(bean);
    if (bean instanceof JCTreeData) {
        setValue(((JCTreeData)bean).getTreeProperties());
    }
}

/** Gets a description for a node in the tree.  It is used
 *	when a tree is shown in the outliner.
public String getNodeDescription(TreeNode node) {
return ((NodeProperties)node.getData()).getDescription();
}
 */

/** Sets a description for a node in the tree.  
public void setNodeDescription(TreeNode node, String description) {
((NodeProperties)node.getData()).setDescription(description);
((NodeProperties)node.getData()).fireJCDataEvent(new JCDataEvent((Object)node.getData()));
}
 */


/** Creates node data to be added to the node of the tree.
 *	@param nodeData nodeData in which new node is created.
 *	@param newNode new node to be created. 
 */
public void addNodeData(DataSourceTreeNode nodeData, DataSourceTreeNode newNode, String description) {
    TreeProperties comp = (TreeProperties) target;
    DataSourceTreeNode current = (DataSourceTreeNode)newNode;
    // Warn user if resource name is not entered.
    if (!comp.isResourceNameSpecified()) {
        ErrorDialog.showError(this,
                              com.klg.jclass.datasource.LocaleBundle.string(
                                                                           com.klg.jclass.datasource.LocaleBundle.enter_resource_name_and_save));
    }
    NodeProperties dbc = null;
    if (current.getUserObject() == null) {
        // New node.  Add data bean component as data.
        NodeProperties new_comp = new NodeProperties(comp.getDataSourceType(), "");
        new_comp.addJCDataListener(comp);
        new_comp.setDescription(description);
        current.setUserObject(new_comp);
    }
    dbc = (NodeProperties) current.getUserObject();

    if (nodeData != null) {
        dbc.setDataSourceType(comp.getDataSourceType());
        dbc.setParentNodeProperties((NodeProperties)((DataSourceTreeNode)nodeData).getUserObject());
        dbc.setUseParentConnection(true);
        nodeData.add(current);
    }
    else {
        // Can't use parent connection for root.
        dbc.setUseParentConnection(false);
        comp.setRoot(current);
    }
    
}

/** Removes data from the component.
 *	@param node node in which data to be removed.
 */
public void removeNodeData(DataSourceTreeNode node) {
    TreeProperties comp = (TreeProperties) target;
    comp.remove((DataSourceTreeNode)node);
}

public void clearData() {
    TreeProperties comp = (TreeProperties) target;
    comp.removeAll();
}

/** Helper method to enable/disable controls for start of editing.
 */
public void checkStartEditing(TreeProperties comp) {
    boolean hasResourceName = comp != null && comp.isResourceNameSpecified();
    modelName.setEnabled(hasResourceName);
    btnAdd.setEnabled(hasResourceName);
    if (hasResourceName) {
	hasResourceName = comp.getRoot() != null;
    }
    btnDelete.setEnabled(hasResourceName);
    btnClear.setEnabled(hasResourceName);
 
    tabManager.setEnabledAt(DATA_PAGE, hasResourceName);

}

/** Helper method to change the editor content when a node is selected. */
public void nodeSelected(EditorTreeNode node) {
    boolean isEmpty = (node == null);

    // No tree.  Disable delete, rename and editor tabs
    btnClear.setEnabled(!isEmpty);
    btnDelete.setEnabled(!isEmpty);
    btnDescription.setEnabled(!isEmpty);
    // Can only add to root of tree in TreeProperties if root is not set.
    TreeProperties comp = (TreeProperties) target;
    serializationPage.setSerializedProperties(comp);
    checkStartEditing(comp);

//FIXOBSOLETE NO, it's not - NetBeans.
	if (isEmpty) {
		tabManager.setSelectedIndex(0); // switch to Serialization page.
	} else {
    // data source type is determined the root, thus data source type page
    // should only be enabled if the selected node is the root of the tree.
    boolean isRoot = node.getUserObject().equals(comp.getRoot());

    NodeProperties dbcomp = (NodeProperties) 
                            ((DataSourceTreeNode)node.getUserObject()).getUserObject();
    dataTabs.setNodeProperties(dbcomp);
	}	

}

/** Helper method to change all data source types to the root's.
 */
void setTreeDataSourceType(DataSourceTreeNode node) {
    if (node == null) return;

    NodeProperties data = (NodeProperties)((DataSourceTreeNode)node).getUserObject();
    data.setDataSourceType(((TreeProperties)target).getDataSourceType());
/*
TreeIterator ti = node.getIterator();
while (ti.hasMoreElements()) {
    TreeNode child = (TreeNode) ti.nextElement();
    setTreeDataSourceType(child);
}
*/
    for (Enumeration e = node.children(); e.hasMoreElements();) {
        setTreeDataSourceType((DataSourceTreeNode) e.nextElement());
    }    
}

/** When value(s) of the tree data bean component is changed.
 */
public void JCDataValueChanged(JCDataEvent e) {
    TreeProperties comp = (TreeProperties) e.getSource();
    checkStartEditing(comp);

    try {
        // Fire property change directly from here but not calling 
        // setValue(comp.getSerializationFile()).
        // The reason is that after calling setValue(), data bean 
        // components of the target have different
        // memory location due to serialization.  However, data 
        // bean components in JCOutliner are still the 
        // before-serialization ones.  Calling setValue() makes outliner
        // and actual data bean components of the target out of synch.
        support.firePropertyChange("TreeProperties", null, comp);
    }
    catch (Exception ex) {
        if (TRACE) {
            ex.printStackTrace();
        }
        if (ex instanceof IOException) {
            ErrorDialog.showError(this,
                                  ex + System.getProperty("line.separator") +
                                  LocaleBundle.string(LocaleBundle.failed_to_serialize));
        }
    }

    int dstype = (comp.getRoot() == null ||
                  ((DataSourceTreeNode)comp.getRoot()).getUserObject() == null) ? 
                 NodeProperties.JDBC :
                 ((NodeProperties)((DataSourceTreeNode)comp.getRoot()).getUserObject()).getDataSourceType();
    if (dstype != comp.getDataSourceType()) {
        // root data source type is changed, propagate to the entire tree.
        comp.setDataSourceType(dstype);
        setTreeDataSourceType(comp.getRoot());

        // reload the bean component.
        showNewBeanComponent();
        return;
    }

    // Assign and reflect NodeProperties in the tab manager.
    if (e.getNodeProperties() != null) {
        // Create a meta data tree.
        createMetaDataTree();
        dataTabs.setNodeProperties((NodeProperties) e.getNodeProperties());
    }
    serializationPage.setSerializedProperties(comp);
}


public void setValue(Object value) {

    if (value instanceof String) {
        // data model is already serialized from data on this editor.
        if (TRACE) {
            System.out.println("setValue::Deserialize from file: " + value);
        }
        try {
            TreeProperties new_model = (TreeProperties)
                                       TreeProperties.fromFile((String)value);

            // target MUST be set before firePropertyChange() 
            // so that the new bean data model
            // is used to set the bean.
            Object old_target = target;
            target = new_model;
            if (TRACE) {
                System.out.println("Change model from " + 
                                   System.getProperty("line.separator") + old_target + 
                                   System.getProperty("line.separator") + "to" + 
                                   System.getProperty("line.separator") + new_model);
            }

            // add event listener to this new TreeProperties
            new_model.addJCDataListener(this);

            // In case the file is not the same as the serialized file name for 
            // the serialization file property.  Set the serialization file to 
            // the file name.
            new_model.setSerializationFile((String) value);
            new_model.fireJCDataEvent(new JCDataEvent((Object)new_model));

            support.firePropertyChange("TreeProperties", old_target, target);
        }
        catch (Exception ex) {
            if (TRACE) {
                ex.printStackTrace();
            }
            if (ex instanceof FileNotFoundException) {
                ErrorDialog.showError(this,
                                      ex + System.getProperty("line.separator") +
                                      LocaleBundle.string(LocaleBundle.failed_to_load_file_not_found));
            }
            else {
                ErrorDialog.showError(this,
                                      ex + System.getProperty("line.separator") +
                                      LocaleBundle.string(LocaleBundle.failed_to_deserialize));
            }

            // throw an exception so loadFromFile() can catch it
            // and indicates loading is unsuccessful.
            throw new IllegalArgumentException(ex.toString());
        }
    }
    else if (value instanceof com.klg.jclass.datasource.beans.TreeProperties) {
        // Reflects the component on the editor.
        // firePropertyChange() is not called here because there is no change.
        TreeProperties comp = (TreeProperties) target;
//FIXOBSOLETE
/*
    if (JCEnvironment.isVisualAge()) {
        // In VisualAge 2.0, setValue() is called after every change.
        // This behaviour screws up the current node selection.
        // This spurious call is ignored.
        if (target != null) {
            return;
        } else {
            comp = (TreeProperties) value;
        }
    } else {
        comp = (TreeProperties) value;
    }
*/
        comp = (TreeProperties) value;
        if (comp.getRoot() != null && ((DataSourceTreeNode)comp.getRoot()).getUserObject() != null) {
            comp.setDataSourceType(
                                  ((NodeProperties)((DataSourceTreeNode)comp.getRoot()).getUserObject()).getDataSourceType());
        }
        comp.addJCDataListener(this);

        try {
            target = comp;
            if (TRACE) {
                System.out.println("Show tree component " + comp);
            }
            showNewBeanComponent();
        }
        catch (Exception e) {
            if (TRACE) {
                e.printStackTrace();
            }
            ErrorDialog.showError(this,
                                  e + System.getProperty("line.separator") + 
                                  LocaleBundle.string(LocaleBundle.failed_to_serialize));
        }
    }
}


/** Returns the name of the serialization file. */
public String getJavaInitializationString() {
    TreeProperties comp = (TreeProperties) target;

    String s = new String("\"" + super.getJavaInitializationString() + 
                          "\", getClass()");

    // At design-time, JBuilder cannot load the serialization file 
    // based on resource name.  That's why the seriliaztion
    // file name is required.  (PR6044)
//FIXOBSOLETE
//	if (JCEnvironment.isJBuilder()) {
//		s += ", \"" + comp.getSerializationFile().replace('\\', '/') + "\"";
//	}

    // If there are data sources, append data source names
    Vector v = new Vector();
    getDataSources(v, comp.getRoot());
    if (v.size() > 0) {
        StringBuffer names = new StringBuffer("new String[] {");
        StringBuffer objects = new StringBuffer("new Object[] {");
        for (int i=0; i<v.size(); i++) {
            if (i>0) {
                names.append(", ");
                objects.append(", ");
            }
            names.append("\"" + v.elementAt(i) + "\"");
            objects.append(v.elementAt(i));
        }
        names.append("}");
        objects.append("}");
        s += "," + new String(names) + "," + new String(objects);
    }
    return new String("new com.klg.jclass.datasource.beans.TreeProperties(" + s + ")"); 
}

/** Helper method to get a initialization string to pass back to IDE
 *	for an array of data source objects.
 */
void getDataSources(Vector s, DataSourceTreeNode node) {
    if (node == null || ((DataSourceTreeNode)node).getUserObject() == null) return;

    NodeProperties data = (NodeProperties) ((DataSourceTreeNode)node).getUserObject();
    if (NodeProperties.isIDEDataSourceType(data.getDataSourceType())) {
        s.addElement(data.getDataSourceName());
    }
/*
TreeIterator ti = node.getIterator();
while (ti.hasMoreElements()) {
    TreeNode child = (TreeNode) ti.nextElement();
    getDataSources(s, child);
}
*/
    for (Enumeration e = node.children(); e.hasMoreElements();) {
        getDataSources(s, (DataSourceTreeNode) e.nextElement());
    }    

}

public void showNewBeanComponent() {
    TreeProperties comp = (TreeProperties) target;
    serializationPage.setSerializedProperties(comp);

/*
if (outliner.getModel() == null) {
    return;
}
*/
    // Delete children of root to clear the outliner first.  Then
    // build the outliner from tree.
    initializing = true;

    // Create a meta data tree.
    createMetaDataTree();

    outliner.setModel(null);    
/*
EditorTreeNode root = (EditorTreeNode)outliner.getModel().getRoot();
if (root != null) {
    root.removeAllChildren();
}
*/
    if (target != null) {
        buildTree(null, ((TreeProperties)target).getRoot());
        if (outliner.getModel() != null) {
            ((DataSourceTreeModel)outliner.getModel()).reload();
            expandTree();        
        }
    }
    // Refresh outliner to show the new tree model.
//	outliner.folderChanged((EditorTreeNode)outliner.getModel().getRoot());
    modelName.setText(comp.getModelName());
//FIXOBSOLETE    
/*
EditorTreeNode selectedNode = (outliner.getModel().getRoot().getChildren() != null) ?
    (EditorTreeNode) outliner.getModel().getRoot().getChildren().elementAt(0) : 
    outliner.getModel().getRoot();
// clear selection first to force re-selecting possibly the same node.
outliner.selectNode(null, null);
outliner.selectNode(selectedNode, null);
*/
    initializing = false;
}

public String getResourceName() {
    return serializationPage.getResourceName();
}

/** Gets a meta data tree.  To do so, a temporary JCTreeData is created
 *	from the serialization file.
 */
public TreeModel getMetaDataTree() {
    return tree;
}

void createMetaDataTree() {
    JCTreeData bean = new JCTreeData();
    try {
        TreeProperties comp = (TreeProperties) target;
        DataSourceBeanInstance.removeInstance(comp.getModelName());
        bean.setTreeProperties(comp);
    }
    catch (Exception ex) {
        if (TRACE) {
            ex.printStackTrace();
        }
        tree = null;
        return;
    }
    tree = bean.getMetaDataTree();
}

/** Get a meta data at the same level as the JCTreeDataNode 
 *	in the TreeProperties.
 */
public MetaDataModel getMetaData(Object find_data, 
                                 DataSourceTreeNode start_node, DataSourceTreeNode meta_start) {
    if (start_node == null || meta_start == null) return null;

    if (foundData(find_data, start_node)) {
        return(MetaDataModel) meta_start;
    }

/*
TreeIterator hiti = start_node.getIterator();
TreeIterator meti = meta_start.getIterator();
while (hiti.hasMoreElements() && meti.hasMoreElements()) {
    TreeNode hichild = (TreeNode) hiti.nextElement();
    TreeNode mechild = (TreeNode) meti.nextElement();
    MetaDataModel meta = getMetaData(find_data, hichild, mechild);
    if (meta != null) {
        return meta;
    }
}	
*/

    Enumeration eHi = start_node.children();
    Enumeration eMe = meta_start.children();
    for (; eHi.hasMoreElements() && eMe.hasMoreElements();) {
        MetaDataModel meta = getMetaData(find_data, (DataSourceTreeNode)eHi.nextElement(), (DataSourceTreeNode)eMe.nextElement());
        if (meta != null) {
            return meta;
        }
    }    

    return null;
}

public boolean foundData(Object find_data, DataSourceTreeNode node) {
    return find_data.equals(((DataSourceTreeNode)node).getUserObject());
}

public DataSourceTreeNode getModelTreeRoot() {
    TreeProperties comp = (TreeProperties) target;
    return comp.getRoot();
}

public void setModelName() {
    TreeProperties comp = (TreeProperties) target;
    if (modelName.getText() == null || modelName.getText().trim().length() == 0) {
        // default model name
        modelName.setText("TreeData" + modelCounter++);
    }
    comp.setModelName(modelName.getText());
}

Frame getFrame() {
/*
Frame frame = (BWTUtil.getFrame(this) == null) ?
                    new JFrame() : BWTUtil.getFrame(this);
return frame; 
*/
    return new JFrame();    
}

class threadTipsDialog implements Runnable {

    public void run() {
        for (;;) {
            if (tipsShowed) {
                break;
            }
            // editor is up.  show tips now.
            if (!isClosed()) {
                showTipsDialog();
                break;
            }
            try {
                showTipsThread.sleep(500);
            }
            catch (Exception ex) {
            }
        }
    }

    private void showTipsDialog() {
        if (tipsShowed) return;

        tipsShowed = true;
        Frame frame = getFrame();
        TipsDialog tipsDialog = new TipsDialog(frame,
                                               LocaleBundle.string(LocaleBundle.title_tips_on_tree_customizer));
        String tip = LocaleBundle.string(LocaleBundle.tips_on_tree_customizer);
        tipsDialog.setShowTipsAtStart(true);
        tipsDialog.setMessage(tip);
        tipsDialog.setModal(true);
        tipsDialog.pack();
        tipsDialog.setVisible(true);
        boolean v = tipsDialog.getShowTipsAtStart();
        JClassInfo.setProperty("datasource", 
                               "tree_data_bean_component.customizer.show_tips_at_start", 
                               new Boolean(v).toString());
        tipsDialog.dispose();
    }

}

public static void main(String args[]) {
    java.awt.Frame f = new java.awt.Frame("TreeDataModel Editor");
    java.beans.Beans.setDesignTime(true);
    TreePropertiesEditor editor = new TreePropertiesEditor();
    editor.setValue(new TreeProperties());
    f.setLayout(new java.awt.BorderLayout());
    f.add(editor, java.awt.BorderLayout.CENTER);
    f.pack();
    f.show();
    // Handle window close requests
    f.addWindowListener(new java.awt.event.WindowAdapter() {
                            public void windowClosing(java.awt.event.WindowEvent e) {
                                System.exit(0);
                            }
                        });
}

}
