/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: UnboundPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:11 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.customizer;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.table.*;

import com.klg.jclass.datasource.*;
import com.klg.jclass.datasource.beans.*;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.swing.JCAlignLayout;

/**
 * Configures meta data columns.
 */
public class UnboundPage extends DataModelPage implements ItemListener,
    ActionListener {

DesignTimeMetaData meta;
JButton button = new JButton("Apply");
JTable table = new JTable();
TreeModel tree;
JTextField provider = new JTextField();
JScrollPane scrollPane;
MyTableModel myTableModel = new MyTableModel();

public UnboundPage(DesignTimeMetaData meta) {
    super("Unbound Data"); // LocaleBundle.string(LocaleBundle.virtual_columns));
    
    setLayout(new BorderLayout());
    
    this.meta = meta;
    button.addActionListener(this);   
	

	add(new ColumnPanel(), BorderLayout.CENTER);
	
	JPanel panel = new JPanel();
	panel.add(new JLabel("Provider class name"));
	provider.setPreferredSize(new Dimension(225, 25));
	panel.add(provider);
	
	JPanel p = new JPanel();
	p.setLayout(new BorderLayout());
	p.add(panel,BorderLayout.CENTER);
	JPanel p2 = new JPanel();
	p2.add(button);
	p2.add(new JButton("Cancel"));
	p.add(p2, BorderLayout.SOUTH);
	
	add(p, BorderLayout.SOUTH);
}

/** Displays values in the bean data comp. */
public void setNodeProperties(NodeProperties comp) {

	// set the provider name
	provider.setText(comp.getProviderName());

	// remove old rows
	resetTable();
		
	// tell the table to update itself
	myTableModel.fireTableChanged(new TableModelEvent(myTableModel));
	
	// each row is data about a ColumnModel
	Vector rows = comp.getUnboundColumns();
	int numRows = rows.size();
	
	// put the data in the jtable
	for (int row = 0; row < numRows; row++) {
		ColumnModel c = (ColumnModel) rows.elementAt(row);
		Object cRow[] = new Object[myTableModel.getColumnCount()];
		cRow[0] = c.getColumnName();
		cRow[1] = convertTypeToName(new Integer(c.getMetaColumnType()).intValue());
		cRow[2] = new Boolean(c.isReadOnly());
		myTableModel.addRow(cRow);	
	}
}

// clear the jtable
public void resetTable() {
	// remove old rows
	int cnt = myTableModel.getRowCount();
	System.out.println("cnt = " + cnt);
	for (int i = 0; i < cnt; i++) {
		myTableModel.removeRow(0);
	}
	// tell the table to update itself
	myTableModel.fireTableChanged(new TableModelEvent(myTableModel));
}

public void actionPerformed(ActionEvent e) {
	
	// reset all properties anew each time
	NodeProperties nodeProperties = getNodeProperties();
	nodeProperties.setProviderName(provider.getText());
	nodeProperties.getUnboundColumns().removeAllElements();
	
	// create columns from the model
	for (int row = 0; row < myTableModel.getRowCount(); row++) {
		BaseColumn c = new BaseColumn();

		c.setColumnName((String) myTableModel.getValueAt(row, 0));
		c.setMetaColumnType(convertNameToType((String) myTableModel.getValueAt(row, 1)));
		c.setReadOnly(((Boolean) myTableModel.getValueAt(row, 2)).booleanValue());
		nodeProperties.addUnboundColumn(c);
	}

	// notify
   	nodeProperties.setChanged();
}

public void itemStateChanged(ItemEvent e) {

}

public String convertTypeToName(int type) {
	String name = "TYPE_STRING";
	switch(type) {
        case MetaDataModel.TYPE_BOOLEAN:		name = "TYPE_BOOLEAN"; 		break;
        case MetaDataModel.TYPE_SQL_DATE: 		name = "TYPE_SQL_DATE"; 	break;
		case MetaDataModel.TYPE_DOUBLE:			name = "TYPE_DOUBLE";		break;
		case MetaDataModel.TYPE_FLOAT:			name = "TYPE_FLOAT";		break;
		case MetaDataModel.TYPE_INTEGER:		name = "TYPE_INTEGER";		break;
		case MetaDataModel.TYPE_STRING:			name = "TYPE_STRING";		break;
		case MetaDataModel.TYPE_BIG_DECIMAL:	name = "TYPE_BIG_DECIMAL";	break;
		case MetaDataModel.TYPE_LONG:			name = "TYPE_LONG";			break;
		case MetaDataModel.TYPE_SQL_TIME:		name = "TYPE_SQL_TIME";		break;
		case MetaDataModel.TYPE_SQL_TIMESTAMP: 	name = "TYPE_SQL_TIMESTAMP"; break;
		case MetaDataModel.TYPE_OBJECT:			name = "TYPE_OBJECT";		break;
		case MetaDataModel.TYPE_BYTE:			name = "TYPE_BYTE";			break;
		case MetaDataModel.TYPE_SHORT:			name = "TYPE_SHORT";		break;
		case MetaDataModel.TYPE_BYTE_ARRAY:		name = "TYPE_BYTE_ARRAY";	break;
		case MetaDataModel.TYPE_UTIL_DATE:		name = "TYPE_UTIL_DATE";	break;
        default: 								name = "TYPE_STRING";		break;
    }
	return name;
}

public int convertNameToType(String name) {

    int type = MetaDataModel.TYPE_STRING;

	if (name.equals("TYPE_BOOLEAN")) {
		type =	MetaDataModel.TYPE_BOOLEAN;
    }
    else if (name.equals("TYPE_SQL_DATE")) {
    	type = MetaDataModel.TYPE_SQL_DATE;
    }
	else if (name.equals("TYPE_DOUBLE")) {
		type = MetaDataModel.TYPE_DOUBLE;
    }
    else if (name.equals("TYPE_FLOAT")) {
		type = MetaDataModel.TYPE_FLOAT;
    }
    else if (name.equals("TYPE_INTEGER")) {
		type = MetaDataModel.TYPE_INTEGER;
    }
    else if (name.equals("TYPE_STRING")) {
		type = MetaDataModel.TYPE_STRING;
    }
    else if (name.equals("TYPE_BIG_DECIMAL")) {
		type = MetaDataModel.TYPE_BIG_DECIMAL;
    }
    else if (name.equals("TYPE_LONG")) {
		type = MetaDataModel.TYPE_LONG;
    }
    else if (name.equals("TYPE_SQL_TIME")) {
		type = MetaDataModel.TYPE_SQL_TIME;
    }
    else if (name.equals("TYPE_SQL_TIMESTAMP")) {
		type = MetaDataModel.TYPE_SQL_TIMESTAMP;
	}
    else if (name.equals("TYPE_OBJECT")) {
		type = MetaDataModel.TYPE_OBJECT;
    }
    else if (name.equals("TYPE_BYTE")) {
		type = MetaDataModel.TYPE_BYTE;
    }
    else if (name.equals("TYPE_SHORT")) {
		type = MetaDataModel.TYPE_SHORT;
    }
    else if (name.equals("TYPE_BYTE_ARRAY")) {
		type = MetaDataModel.TYPE_BYTE_ARRAY;
    }
    else if (name.equals("TYPE_UTIL_DATE")) {
		type = MetaDataModel.TYPE_UTIL_DATE;
    }
    else {
		type = MetaDataModel.TYPE_STRING;
    }
    return type;
}

// inner class

public class ColumnPanel extends JPanel {

	int selectedRow = -1;

	public ColumnPanel() {

		setBorder(new TitledBorder("Add/Remove Columns"));

		table = new JTable(myTableModel);
		
		// only allow single selection of rows
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
        // set cell editors/renderers on the type column
        setTypeColumn(table.getColumnModel().getColumn(1));
	
		//Create the scroll pane and add the table to it. 
		scrollPane = new JScrollPane(table);
				
		JPanel buttonPanel 		= new JPanel();
		JButton addButton 		= new JButton("Add Column");
		JButton removeButton	= new JButton("Remove Column");
		 
		buttonPanel.add(addButton);
		buttonPanel.add(removeButton);
	
		setLayout(new BorderLayout());
		add(scrollPane, BorderLayout.CENTER);
		add(buttonPanel, BorderLayout.SOUTH);
		
		// add listeners
		
		// button listeners
		addButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				myTableModel.addRow(new Object[] {"", "TYPE_STRING", new Boolean(false)} );
			}
		});
		removeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (selectedRow > -1 && myTableModel.getRowCount() > 0) {
					myTableModel.removeRow(selectedRow);
					selectedRow = -1;
				}
			}
		});

		// selection listeners
		ListSelectionModel rowSM = table.getSelectionModel();
          rowSM.addListSelectionListener(new ListSelectionListener() {
              public void valueChanged(ListSelectionEvent e) {
                  ListSelectionModel lsm = (ListSelectionModel)e.getSource();
                  if (lsm.isSelectionEmpty()) {
                  	selectedRow = -1;    
                  } else {
					selectedRow = lsm.getMinSelectionIndex();                      
                  }
              }
		});
	}
		
	public void setTypeColumn(TableColumn typeColumn) {
        //Set up the editor for types
        JComboBox comboBox = new JComboBox();
        comboBox.addItem("TYPE_BOOLEAN");
        comboBox.addItem("TYPE_SQL_DATE");
		comboBox.addItem("TYPE_DOUBLE");
		comboBox.addItem("TYPE_FLOAT");
		comboBox.addItem("TYPE_INTEGER");
		comboBox.addItem("TYPE_STRING");
		comboBox.addItem("TYPE_BIG_DECIMAL");
		comboBox.addItem("TYPE_LONG");
		comboBox.addItem("TYPE_SQL_TIME");
		comboBox.addItem("TYPE_SQL_TIMESTAMP");
		comboBox.addItem("TYPE_OBJECT");
		comboBox.addItem("TYPE_BYTE");
		comboBox.addItem("TYPE_SHORT");
		comboBox.addItem("TYPE_BYTE_ARRAY");
		comboBox.addItem("TYPE_UTIL_DATE");
		 
		// set the editor		       
		typeColumn.setCellEditor(new DefaultCellEditor(comboBox));

        // set the renderer
        DefaultTableCellRenderer renderer = new DefaultTableCellRenderer();     
        typeColumn.setCellRenderer(renderer);
    }
		
}

// inner class my table model
class MyTableModel extends DefaultTableModel {
	final String[] columnNames = {"Column Name", 
	                              "Column Type",
	                              "ReadOnly" 
		};
	final Object[][] data = {{}};
	
	public MyTableModel() {
		super();
		setDataVector(data, columnNames);
	}
	
	/*
	 * JTable uses this method to determine the default renderer/
	 * editor for each cell.  If we didn't implement this method,
	 * then the last column would contain text ("true"/"false"),
	 * rather than a check box.
	 */
	public Class getColumnClass(int c) {
	    return getValueAt(0, c).getClass();
	}
	}

} // UnboundPage
