/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: VirtualColumnPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:11 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.customizer;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.event.*;
import javax.swing.tree.*;

import com.klg.jclass.datasource.*;
import com.klg.jclass.datasource.beans.*;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.swing.JCAlignLayout;

/** A page in a custom editor/customizer to modify virtual columns.
 *	Only BaseVirtualColumn is supported.
 */
public class VirtualColumnPage extends DataModelPage implements ItemListener,
    ActionListener {

DefaultListModel virtualModel = new DefaultListModel();
DefaultListModel baseModel = new DefaultListModel();
JList virtualColumns, basedColumns;
JTextField name;
JComboBox type, operation, basedColumnChoices;
JButton btnAddBasedColumn, btnDeleteBasedColumn;
JButton btnAdd, btnDelete, btnClear, btnSet;

DesignTimeMetaData meta;
TreeModel tree;
MetaDataModel mdata;

private int currentColumnIndex;

private static int counter = 0;
private static final boolean TRACE = false;

public VirtualColumnPage(DesignTimeMetaData meta) {
    super(LocaleBundle.string(LocaleBundle.virtual_columns));
    this.meta = meta;
    currentColumnIndex = -1;
    init();
}

public void init() {
    setLayout(new BorderLayout());
    virtualColumns = new JList(virtualModel);
    JLabel lbl_column_name = new JLabel(LocaleBundle.string(LocaleBundle.column_name));
    JLabel lbl_column_type = new JLabel(LocaleBundle.string(LocaleBundle.column_type));
    JLabel lbl_operation = new JLabel(LocaleBundle.string(LocaleBundle.operation));
    name = new JTextField("", 30);
    type = new JComboBox();
    operation = new JComboBox();
    btnAdd= new JButton(LocaleBundle.string(LocaleBundle.add3));
    btnDelete= new JButton(LocaleBundle.string(LocaleBundle.delete));
    btnClear = new JButton(LocaleBundle.string(LocaleBundle.clear));
    btnSet = new JButton(LocaleBundle.string(LocaleBundle.set_or_modify));
    JPanel virtual_panel = new JPanel();
    JPanel virtual_button_panel = new JPanel();
    basedColumns = new JList(baseModel);
    basedColumnChoices = new JComboBox();
    btnAddBasedColumn = new JButton(LocaleBundle.string(LocaleBundle.add));
    btnDeleteBasedColumn = new JButton(LocaleBundle.string(LocaleBundle.delete));
    JPanel button_panel = new JPanel();
    JPanel based = new JPanel();
    JPanel set_panel = new JPanel();
    JPanel field_panel = new JPanel();
    JPanel main_panel = new JPanel();

    virtual_panel.setLayout(new BorderLayout());
    virtual_button_panel.setLayout(new GridBagLayout());
    GridBagConstraints c = new GridBagConstraints();
    c.anchor = GridBagConstraints.WEST;
    c.insets = new Insets(5, 0, 5, 5);
    virtual_button_panel.add(btnAdd, c);
    virtual_button_panel.add(btnDelete, c);
    c.weightx = 5;
    virtual_button_panel.add(btnClear, c);
    virtual_panel.add(new JScrollPane(virtualColumns), BorderLayout.CENTER);
    JPanel box = new JPanel();
    box.setBorder(new TitledBorder(LocaleBundle.string(LocaleBundle.based_columns)));

    // Build based columns box.
    box.setLayout(new BorderLayout());
    button_panel.setLayout(new GridBagLayout());
    c = new GridBagConstraints();
    c.insets = new Insets(0, 5, 5, 5);
    c.anchor = GridBagConstraints.NORTH;
    c.fill = GridBagConstraints.HORIZONTAL;
    c.gridy = 1;
    button_panel.add(btnAddBasedColumn, c);
    c.gridy = 2;
    c.weighty = 5;
    button_panel.add(btnDeleteBasedColumn, c);
    based.setLayout(new BorderLayout());
    based.add(new JScrollPane(basedColumns), BorderLayout.CENTER);
    based.add(basedColumnChoices, BorderLayout.SOUTH);
    box.add(based, BorderLayout.CENTER);
    box.add(button_panel, BorderLayout.EAST);

    // Load choices and lists
    for (int i=0; i<JCDataConverter.virtualColumnOperationStrings.length; i++) {
        operation.addItem(JCDataConverter.virtualColumnOperationStrings[i]);
    }
    for (int i=0; i<JCDataConverter.javaDataTypeStrings.length; i++) {
        type.addItem(JCDataConverter.javaDataTypeStrings[i]);
    }

    JCAlignLayout layout = new JCAlignLayout();
    field_panel.setLayout(layout);
    field_panel.add(lbl_column_name);
    field_panel.add(name);
    layout.setResizeWidth(name, true);
    field_panel.add(lbl_column_type);
    field_panel.add(type);
    layout.setResizeWidth(type, true);
    field_panel.add(lbl_operation);
    field_panel.add(operation);
    layout.setResizeWidth(operation, true);
    set_panel.setLayout(new GridBagLayout());
    c = new GridBagConstraints();
    c.insets = new Insets(5, 5, 5, 5);
    c.anchor = GridBagConstraints.WEST;
    c.weightx = 5;
    set_panel.add(btnSet, c);
    main_panel.setLayout(new BorderLayout());
    main_panel.add(field_panel, BorderLayout.NORTH);
    main_panel.add(box, BorderLayout.CENTER);
    main_panel.add(set_panel, BorderLayout.SOUTH);

    add(main_panel, BorderLayout.CENTER);
    add(virtual_panel, BorderLayout.WEST);
    add(virtual_button_panel, BorderLayout.SOUTH);

    // add listeners
    btnAdd.addActionListener(this);
    btnDelete.addActionListener(this);
    btnClear.addActionListener(this);
    name.addActionListener(this);
    type.addItemListener(this);
    operation.addItemListener(this);
    basedColumnChoices.addItemListener(this);
    btnAddBasedColumn.addActionListener(this);
    btnDeleteBasedColumn.addActionListener(this);
    btnSet.addActionListener(this);

    virtualColumns.addListSelectionListener(new ListSelectionListener() {
                                                public void valueChanged(ListSelectionEvent e) {
                                                    if (isInitializing()) return;

                                                    if (e.getValueIsAdjusting()) {
                                                        return;
                                                    }
                                                    showSelectedVirtualColumn();
                                                    updateAccess();
                                                }
                                            });
    basedColumns.addListSelectionListener(new ListSelectionListener() {
                                              public void valueChanged(ListSelectionEvent e) {
                                                  if (isInitializing()) return;

                                                  updateAccess();
                                              }
                                          });
}

/** Displays values in the bean data comp. */
public void setNodeProperties(NodeProperties comp) {
    loadVirtualColumns();
    tree = null;
    mdata = null;
    tree = meta.getMetaDataTree();
    if (tree != null) {
        mdata = meta.getMetaData(comp,
                                 (DataSourceTreeNode)meta.getModelTreeRoot(),
                                 (DataSourceTreeNode)tree.getRoot());
    }
    loadBasedColumnChoices();
    updateAccess();
    showSelectedVirtualColumn();
}

/** Update access to controls on this page. */
private void updateAccess() {
    String selected_virtual = (String) virtualColumns.getSelectedValue();
    String selected_base = (String) basedColumns.getSelectedValue();
    int size = virtualModel.getSize();
    name.setEnabled(selected_virtual != null);
    type.setEnabled(selected_virtual != null);
    operation.setEnabled(selected_virtual != null);
    basedColumns.setEnabled(selected_virtual != null);
    basedColumnChoices.setEnabled(selected_virtual != null);

    btnAddBasedColumn.setEnabled(selected_virtual != null);
    btnDeleteBasedColumn.setEnabled(selected_virtual != null && selected_base != null);

    btnDelete.setEnabled(size > 0);
    btnClear.setEnabled(size > 0);
}

/** Loads all the virtual columns defined at this level. */
private void loadVirtualColumns() {
    // load virtual column names to the list.
    virtualModel.removeAllElements();
    for (Enumeration e=getNodeProperties().getVirtualColumns().elements();
        e.hasMoreElements();) {
        BaseVirtualColumn col = (BaseVirtualColumn) e.nextElement();
        virtualModel.addElement(col.getColumnName());
    }
    if (virtualModel.getSize() > 0) {
        virtualColumns.setSelectedIndex(Math.max(0, currentColumnIndex));
    }
}

/** Loads all the based column choices for the selected virtual column.
 *	Based column choices are the columns from the meta data and
 *	virtual columns other than the current virtual column.
 */
private void loadBasedColumnChoices() {
    if (basedColumnChoices.getItemCount() > 0) {
        basedColumnChoices.removeAllItems();
    }

    // add based columns
    if (mdata != null) {
        String virtual_col = (String) virtualColumns.getSelectedValue();
        int num_of_cols = 0;
        try {
            num_of_cols = mdata.getColumnCount();
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
        for (int i=0; i<num_of_cols; i++) {
            try {
                if (virtual_col != null &&
                    !mdata.getColumnIdentifier(i).equals(virtual_col)) {
                    basedColumnChoices.addItem(mdata.getColumnIdentifier(i));
                }
            }
            catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }
}

/** Shows the selected virtual column. */
private void showSelectedVirtualColumn() {
    // Clear all fields first
    name.setText("");
    baseModel.removeAllElements();
    String selected_virtual = (String) virtualColumns.getSelectedValue();
    int size = virtualModel.getSize();
    currentColumnIndex = virtualColumns.getSelectedIndex();

    if (selected_virtual == null) return;

    BaseVirtualColumn col = getNodeProperties().getVirtualColumn(selected_virtual);
    if (col == null) return;
    name.setText(col.getColumnName());

    // show column type
    JCTypeConverter conv = new JCTypeConverter();
    int index = -1;
    int default_index = -1;
    for (int i=0; i<JCDataConverter.javaDataTypeValues.length; i++) {
        if (JCDataConverter.javaDataTypeValues[i] == MetaDataModel.TYPE_FLOAT) {
            default_index = i;
        }
        if (col.getColumnType() == JCDataConverter.javaDataTypeValues[i]) {
            index = i;
            break;
        }
    }
    type.setSelectedIndex((index == -1) ? default_index : index);

    // show column operation
    index = -1;
    default_index = -1;
    for (int i=0; i<JCDataConverter.virtualColumnOperationValues.length; i++) {
        if (JCDataConverter.virtualColumnOperationValues[i] == BaseVirtualColumn.SUM) {
            default_index = i;
        }
        if (col.getOperation() == JCDataConverter.virtualColumnOperationValues[i]) {
            index = i;
            break;
        }
    }
    operation.setSelectedIndex((index == -1) ? default_index : index);

    // show based columns
    baseModel.removeAllElements();
    for (int i=0; i<col.getColumns().length; i++) {
        baseModel.addElement(col.getColumns()[i]);
    }

    // load based column choices for this virtual column.
    loadBasedColumnChoices();
}

int getNextCounter() {
    for (Enumeration e=getNodeProperties().getVirtualColumns().elements();
        e.hasMoreElements();) {
        BaseVirtualColumn col = (BaseVirtualColumn) e.nextElement();
        if (col.getColumnName().startsWith("col")) {
            String count = col.getColumnName().substring(new String("col").length());
            int c = 0;
            try {
                c = new Integer(count).intValue();
            }
            catch (Exception ex) {
            }
            counter = Math.max(counter, c);
        }
    }
    return ++counter;
}

/** Adds a virtual column.  Changes are saved right away. */
void addColumn() {
    String default_name = "col" + getNextCounter();
    DescriptionDialog dlg = new DescriptionDialog(new JFrame(),
                                                  LocaleBundle.string(LocaleBundle.column_name));
    dlg.setDescription(default_name);
    dlg.pack();
    dlg.setVisible(true);
    String col_name = (dlg.getDescription() == null  || dlg.getDescription().length() == 0) ?
                      default_name : dlg.getDescription();
    BaseVirtualColumn col = new BaseVirtualColumn(col_name,
                                                  MetaDataModel.TYPE_FLOAT,
                                                  BaseVirtualColumn.SUM,
                                                  new String[0]);
    getNodeProperties().addVirtualColumn(col);
    currentColumnIndex = getNodeProperties().getVirtualColumns().size()-1;
}

/** Deletes a virtual column.  Changes are saved right away. */
void deleteColumn() {
    String selected_virtual = (String) virtualColumns.getSelectedValue();
    if (selected_virtual == null) return;
    BaseVirtualColumn col = getNodeProperties().getVirtualColumn(selected_virtual);
    getNodeProperties().removeVirtualColumn(col);
    currentColumnIndex = getNodeProperties().getVirtualColumns().size()-1;
}

/** Deletes all virtual columns.  Changes are saved right away. */
void deleteAllColumns() {
    getNodeProperties().removeAllVirtualColumns();
}

/** Adds a based column.  Changes are not saved until set/modify button is pressed. */
void addBasedColumn() {
    String col = (String) basedColumnChoices.getSelectedItem();
    if (col != null && col.length() > 0) {
        baseModel.addElement(col);
        currentVirtualColumnChanged();
    }
}

/** Deletes a based column.  Changes are not saved until set/modify button is pressed. */
void deleteBasedColumn() {
    String col = (String) basedColumns.getSelectedValue();
    if (col != null && col.length() > 0) {
        baseModel.removeElement(col);
        currentVirtualColumnChanged();
    }
}

/** When a property of the current virtual column is changed,
 *	rebuild the list of virtual columns of the data bean component.
 *	The order of the columns is preserved.
 */
private void currentVirtualColumnChanged() {
    String col_name = name.getText();
    int col_op = JCDataConverter.virtualColumnOperationValues[operation.getSelectedIndex()];
    int col_type = JCDataConverter.javaDataTypeValues[type.getSelectedIndex()];
    String[] based_columns = new String[baseModel.getSize()];
    baseModel.copyInto(based_columns);
    String selected_virtual = (String) virtualColumns.getSelectedValue();
    BaseVirtualColumn new_col = new BaseVirtualColumn(col_name, col_type, col_op, based_columns);

    BaseVirtualColumn current_col = getNodeProperties().getVirtualColumn(selected_virtual);
    Vector old_list = (Vector) getNodeProperties().getVirtualColumns().clone();
    getNodeProperties().removeAllVirtualColumns();
    for (Enumeration e=old_list.elements();
        e.hasMoreElements();) {
        BaseVirtualColumn col = (BaseVirtualColumn) e.nextElement();
        getNodeProperties().addVirtualColumn(col.equals(current_col) ? new_col : col);
    }
}

public void actionPerformed(ActionEvent e) {
    if (isInitializing()) return;

    if (getNodeProperties() == null) return;

    if (e.getSource() instanceof JButton) {
        JButton btn = (JButton) e.getSource();
        if (btn.equals(btnAdd)) {
            addColumn();
            // Indicates comp is changed.
            getNodeProperties().fireJCDataEvent(new JCDataEvent((Object)getNodeProperties()));
        }
        else if (btn.equals(btnDelete)) {
            deleteColumn();
            // Indicates comp is changed.
            getNodeProperties().fireJCDataEvent(new JCDataEvent((Object)getNodeProperties()));
        }
        else if (btn.equals(btnClear)) {
            deleteAllColumns();
            // Indicates comp is changed.
            getNodeProperties().fireJCDataEvent(new JCDataEvent((Object)getNodeProperties()));
        }
        else if (btn.equals(btnAddBasedColumn)) {
            addBasedColumn();
        }
        else if (btn.equals(btnDeleteBasedColumn)) {
            deleteBasedColumn();
        }
        else if (btn.equals(btnSet)) {
            currentVirtualColumnChanged();
            // Indicates comp is changed.
            getNodeProperties().fireJCDataEvent(new JCDataEvent((Object)getNodeProperties()));
        }
    }
    else if (e.getSource() instanceof JTextField) {
        JTextField text = (JTextField) e.getSource();
        if (text.equals(name)) {
            //virtualColumns.replaceItem(name.getText(), virtualColumns.getSelectedIndex());
        }
    }

    if (TRACE) {
        System.out.println("NodeProperties " +
                           (getNodeProperties().isSerializationRequired() ? "is" : "is not") +
                           " changed because of virtual columns.");
    }
}

public void itemStateChanged(ItemEvent e) {
    if (isInitializing()) return;

    if (getNodeProperties() == null) {
        return;
    }
}


} // VirtualColumnPage
