/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Column.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:11 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.jbuilder;

import com.borland.dx.dataset.Variant;

import com.klg.jclass.datasource.BaseColumn;

import java.sql.Types;


/**
 * JBuilder column information.
 *
 * ResultSetMetaData methods are mirrored here, see their
 * documentation for descriptions.
 */

public class Column extends BaseColumn {
	
static final long serialVersionUID = -4096071228432954457L;

/**
 * Constructor for jclass.datasource.ColumnModel object for JBuilder
 * @param column a com.borland.dx.dataset.Column
 */
public Column(com.borland.dx.dataset.Column column) {
    setColumnName(column.getColumnName());
    int sqlType = column.getSqlType();
    // com.borland's calculated columns have type null, if null
    // determine sql type from data type.
    if (sqlType == Types.NULL) {
        switch(column.getDataType()) {
        case Variant.BIGDECIMAL: sqlType = Types.NUMERIC;   break;
        case Variant.BOOLEAN:    sqlType = Types.BIT;       break;
        case Variant.DATE:       sqlType = Types.DATE;      break;
        case Variant.DOUBLE:     sqlType = Types.DOUBLE;    break;
        case Variant.FLOAT:      sqlType = Types.FLOAT;     break;
        case Variant.INT:        sqlType = Types.INTEGER;   break;
        case Variant.STRING:     sqlType = Types.VARCHAR;   break;
        case Variant.TIMESTAMP:  sqlType = Types.TIMESTAMP; break;
        case Variant.TIME:       sqlType = Types.TIME;      break;
        case Variant.LONG:       sqlType = Types.BIGINT;    break;
        case Variant.SHORT:      sqlType = Types.SMALLINT;  break;
        case Variant.OBJECT:     sqlType = Types.VARCHAR;   break;
        case Variant.BYTE:       sqlType = Types.TINYINT;   break;
        case Variant.BYTE_ARRAY:
        case Variant.INPUTSTREAM:
                                sqlType = Types.LONGVARBINARY; break;
        default:                sqlType = Types.VARCHAR;    break;
        }
    }
    setColumnType(sqlType);
    setTableName(column.getTableName());
    setScale(column.getScale());
    setColumnLabel(column.getCaption());
    setSchemaName(null);
    setPrecision(column.getPrecision());
    setCatalogName(null);
    setColumnTypeName(null);
    setIdentifier(getColumnName());
    setColumnDisplaySize(column.getWidth());
    setNullable(java.sql.ResultSetMetaData.columnNullable);
    setCurrency(column.isCurrency());
    setReadOnly(column.isReadOnly());
    setAutoIncrement(false);
    setCaseSensitive(false);
    setSearchable(column.isSearchable());
    setSigned(false);
    setWritable(true);
    setDefinitelyWritable(true);
    setMetaColumnTypeFromSqlType(getColumnType());
}

}
