/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataTable.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:11 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.jbuilder;

import com.borland.dx.dataset.Variant;

import java.sql.Time;
import java.sql.Timestamp;

import javax.swing.tree.TreeNode;

import java.util.Enumeration;
import java.util.Date;

import com.klg.jclass.datasource.BaseDataTable;
import com.klg.jclass.datasource.DataModelException;
import com.klg.jclass.datasource.DataTableModel;
import com.klg.jclass.datasource.DataModelEvent;
import com.klg.jclass.datasource.MetaDataModel;
import com.klg.jclass.datasource.IdeDataTableModel;
import com.klg.jclass.datasource.IdeDataTable;
import com.klg.jclass.datasource.DataModelListener;
import com.klg.jclass.datasource.LocaleBundle;

import com.borland.dx.dataset.DataSet;
import com.borland.dx.dataset.DataRow;
import com.borland.dx.dataset.DataSetView;
import com.borland.dx.dataset.DataSetException;

import com.borland.dx.dataset.MasterLinkDescriptor;

import java.math.BigDecimal;
import java.io.InputStream;

/**
 * This is the JBuilder implementation for data storage for the
 * JClass Outline Grid datamodel.
 * This data model will request data from instances of
 * this table and will manipulate that data through this interface.
 * That is, rows can then be added, deleted or updated through
 * this DataTable.<P>
 *
 * To allow sorting of rows and columns, all operations access
 * cell data using unique identifiers for rows and columns, rather
 * than their indexes.
 */
public class DataTable extends IdeDataTable {
	
static final long serialVersionUID = 21522420550133001L;

// properties
protected DataSet dataSet;
protected DataSetView dataSetView;
protected boolean eventsOriginallyOn;

public DataTable(MetaData metaData, DataSet dataSet, long parentRow) {
    super(metaData, parentRow);
    this.dataSet = dataSet;
    copyTable();
}

public DataTable(MetaData metaData, DataSet dataSet) {
    super(metaData);
    this.dataSet = dataSet;
    copyTable();
}

public synchronized void copyTable() {
   try {
        turnOffDataModelEvents();
        int rows = getIDERows();
        dataSet.first();
        for (int i = 0; i < rows; i++) {
            DataRow data_row = new DataRow(dataSet);
            dataSet.getDataRow(data_row);
            long bm = addInternalRow(duplicateRow(data_row));
            dataSet.next();
        }
        dataSet.first();
        turnOnDataModelEvents();
   } catch (Exception e) {
        turnOnDataModelEvents();
        System.out.println("getRowIdentifier " + e.toString());
   }
}

/**
 * Duplicate a row.
 * @exception DataModelException If duplicate fails.
 */
public Object[] duplicateRow(int rowIndex) throws DataModelException {
    return new Object[metaData.getColumnCount()];
}

private Object[] duplicateRow(DataRow data_row) throws Exception {
    Object row[] = new Object[metaData.getColumnCount()];
    for (int i = 0; i < metaData.getColumnCount(); i++) {
        row[i] = getCell(data_row, metaData.getColumnIdentifier(i));
    }
    return row;
}

/**
 *	Returns the number of rows in this DataTable.
 * @return the number of rows in this table
 */
private int getIDERows() {
   try {
       return dataSet.getRowCount();
	} catch (Exception e) {
		return 0;
	}
}

// Commenting this out means it goes to the jbuilder ds every time
// otherwise it uses the cached data.
// public Object getCell(long bookmark, String column) throws Exception {
//     moveToRow(bookmark);
//      DataRow dr = new DataRow(dataSet);
//      dataSet.getDataRow(dr);
//      return getCell(dr, column);
// }

/**
 * @exception DataModelException If get fails.
 */
private Object getCell(DataRow data_row, String column_id) throws DataModelException {
	try {
		int type = data_row.getColumn(column_id).getDataType();
		switch(type) {
   		case Variant.BIGDECIMAL: return data_row.getBigDecimal(column_id);
		case Variant.BOOLEAN:    return new Boolean(data_row.getBoolean(column_id));
		case Variant.DATE:       return data_row.getDate(column_id);
		case Variant.DOUBLE:     return new Double(data_row.getDouble(column_id));
		case Variant.FLOAT:      return new Float(data_row.getFloat(column_id));
		case Variant.INT:        return new Integer(data_row.getInt(column_id));
		case Variant.STRING:     return data_row.getString(column_id);
		case Variant.TIMESTAMP:  return data_row.getTimestamp(column_id);
		case Variant.TIME:       return data_row.getTime(column_id);
		case Variant.LONG:       return new Long(data_row.getLong(column_id));
		case Variant.SHORT:      return new Integer(data_row.getShort(column_id));
		case Variant.OBJECT:     return data_row.getObject(column_id);
		case Variant.BYTE:       return new Byte(data_row.getByte(column_id));
		case Variant.BYTE_ARRAY: return data_row.getByteArray(column_id);
		case Variant.INPUTSTREAM:
			return ((MetaData) metaData).createByteArrayFromBinaryStream(data_row.getInputStream(column_id));
		default:
			return data_row.getString(column_id);
		}
	}
	catch (DataSetException dse) {
		throw new DataModelException(DataModelEvent.INITIALIZE, dse);
	}
}

/**
 * Duplicate a row.
 * @exception DataModelException If get fails.
 */
public Object getCell2(long bookmark, String column_id) throws DataModelException {
    moveToRow(bookmark);
	
    DataRow data_row = null;
    try {
		  data_row = new DataRow(dataSet);
	    dataSet.getDataRow(data_row);
	  }
	  catch (DataSetException dse) {
		  throw new DataModelException(DataModelEvent.INITIALIZE, dse);
	  }

    return getCell(data_row, column_id);
}

/**
 * Called by DataTableModel.createTable to create a new table
 * based on the input criteria.
 *
 * @param originator the DataModelListener which initiated this action
 * @param bookmark a long which uniquely identifies a row
 * @param childMetaDataModel a MetaDataModel, this tells this table
 * what structure the child table it is creating should have
 * @exception DataModelException If materializeTable fails.
 */
public DataTableModel materializeTable(DataModelListener originator, long bookmark, TreeNode childMetaDataModel)
   throws DataModelException {
   DataTable data_table = null;
   try {
        turnOffDataModelEvents();
        MetaData child_meta = (MetaData) childMetaDataModel;

        // position the cursor on the proper row
        moveToRow(bookmark);

        // create the new data table
        data_table = new DataTable(child_meta, child_meta.dataSet, bookmark);
        // set the path. This is different than jdbc where a create does not
        // move the cursor
        child_meta.setCurrentDataTable(data_table);
        long childBookmark = data_table.ROW_NOT_FOUND;
        // could be the case this table has no rows
        if (data_table.getRowCount() != 0) {
            data_table.first();
            childBookmark = data_table.getCurrentBookmark();
        }
        child_meta.setCurrentBookmark(childBookmark);
        turnOnDataModelEvents();
        getMetaData().getDataModel().moveToRow(originator, this, bookmark);
   } catch (Exception e) {
        turnOnDataModelEvents();
        throw new DataModelException(DataModelEvent.INITIALIZE, e);
   }
   return data_table;
}

/**
 * Makes "bookmark" the new current row.
 * @param bookmark an int, unique row identifier
 */
public void moveToRow(long bookmark) throws DataModelException {
    // recursively make sure parent data sets are all
    // positioned so ultimately this level's data set
    // pointer is pointing to this data

    // set up the path
    metaData.setCurrentDataTable(this);
    metaData.setCurrentBookmark(bookmark);

    if (getRowStatus(bookmark) == INSERTED) {
        return;
    }

    // turn off events so listeners don't move while we readjust the pointers
    turnOffDataModelEvents();
    positionDataSet();


    int id = store.getIndex(bookmark);
    // move the cursor to it, this causes the meta_data.dataSet
    // to be populated with the detail rows which correspond to
    // the row to which we just moved

    if (dataSet.getRow() != id) {
    	try {
        	if (!dataSet.goToRow(id)) {
            	System.out.println("moveToRow() ERROR row " + bookmark);
        	}
    	}
    	catch (DataSetException dse) {
    		throw new DataModelException(DataModelEvent.INITIALIZE, dse);
    	}
    }
    // turn event back on
    turnOnDataModelEvents();
}

/**
 * Moves the DataSet pointer so that it points to this data.
 * The detail DataSet pointer corresponds to the current row in the
 * parent DataSet.  To ensure it is pointing to this data, recurse
 * up the data tree to this data's ultimate master, then recurse
 * down, setting each DataSet pointer so that ultimately this level's
 * DataSet pointer points to this data.
 * @exception Exception If positionDataSet fails.
 */
void positionDataSet() throws DataModelException {

    MasterLinkDescriptor mld = dataSet.getMasterLink();
    if (mld == null) {
        return;
    }

    DataSet pds = mld.getMasterDataSet();
    DataTable pdt = (DataTable) this.getParent();

    if (pds != null && pdt != null) {
        // keep going up
        pdt.positionDataSet();

        // okay, we are at one level beneath the top, now work our way down
        int id = pdt.store.getIndex(parentRow);

        // set the current data table and row at a level
        pdt.getMetaData().setCurrentDataTable(pdt);
        pdt.getMetaData().setCurrentBookmark(parentRow);

        // only move if we have to, it's expensive
        if (pds.getRow() != id) {
        	try {
            	if (!pds.goToRow(id)) {
                	System.out.println("move failed in positionDataSet");
            	}
        	} 
        	catch (DataSetException dse) {
        		throw new DataModelException(DataModelEvent.INITIALIZE, dse);
        	}
        }
    }
}

/**
 * Adds a row, overrides BaseDataTable.addRow().
 * <P>
 * All we do here is call the ide add row, for this will generate an ide
 * add row event, which will cause us to add a row our data model. Avoids
 * double adds.
 * <P>
 * Adds a blank row into which the user can input column data.
 * The record is appended to this ResultSet.  The record is saved to datasource
 * when the user commits the surrounding transaction.
 *  * @param originator the DataModelListener which initiated this action
 * @exception DataModelException If row fails to be added.
 */
public long addRow(DataModelListener originator) throws DataModelException {
    try {
        // use the first column to check for insert permissions
        com.klg.jclass.datasource.ColumnModel cm = metaData.getColumnObject(metaData.getColumnIdentifier(0));
        if (cm != null && !metaData.getInsertAllowed(cm.getTableName())) {
            Exception e = new Exception(LocaleBundle.string(LocaleBundle.insert_not_allowed));
            throw new DataModelException(DataModelEvent.BEFORE_INSERT_ROW, e);
        }

        // set parent pointers appropriately
        positionDataSet();
        // this inserts the row and move the ide cursor to it
        dataSet.insertRow(false);   // false means append vs. insert
        return bookmarkOfLastRowAdded;
    } catch (Exception e) {
        throw new DataModelException(bookmarkOfLastRowAdded, "", this, DataModelEvent.AFTER_INSERT_ROW, e);
    }
}

/**
 * @exception DataModelException If the row can't be deleted.
 */
public void deleteRow(DataModelListener originator,long bookmark) throws DataModelException {
   try {
       // use the first column to check for delete permissions
       com.klg.jclass.datasource.ColumnModel cm = metaData.getColumnObject(metaData.getColumnIdentifier(0));
       if (cm != null && !metaData.getDeleteAllowed(cm.getTableName())) {
            Exception e = new Exception(LocaleBundle.string(LocaleBundle.delete_not_allowed));
            throw new DataModelException(DataModelEvent.BEFORE_DELETE_ROW, e);
       }

       // give the listeners the opportunity to cancel this delete
       if (!fireDataModelEvent(new DataModelEvent(metaData.getDataModel(), originator, this, bookmark, "", DataModelEvent.BEFORE_DELETE_ROW))) {
           return;
       }
       moveToRow(bookmark);
       dataSet.deleteRow();
       dataSet.saveChanges();
   } catch (Exception e) {
       throw new DataModelException(bookmark, "", this, DataModelEvent.AFTER_DELETE_ROW, e);
   }
}

/**
 * @return true if commit succeeds, false if it fails or was cancelled.
 * @exception DataModelException If the row can't be committed.
 */
public boolean commitRow(DataModelListener originator, long bookmark) throws DataModelException {
   // give the listeners the opportunity to cancel this commit
   if (!fireDataModelEvent(new DataModelEvent(metaData.getDataModel(), originator, this, bookmark, "", DataModelEvent.BEFORE_COMMIT_ROW))) {
       return false;
   }
   int rowIndex = getRowIndex(bookmark);
   // if no changes, return
 	 if (getRowStatus(bookmark) == DataTableModel.COMMITTED || getRowStatus(bookmark) == NONEXISTENT) {
		  return true;
	 }

   int rowStatus = getRowStatus(bookmark);
   Object row = null;
   if (rowStatus == DataTableModel.DELETED) {
       row = logicallyDeletedRows.get(new Long(bookmark));
   }
   else if (rowStatus == DataTableModel.INSERTED) {
       row = insertedRows.get(new Long(bookmark));
   }
   try {
       removeRowFromLogicallyDeletedTable(bookmark);
       // removeRowFromUpdatedTable(bookmark);
       removeRowFromInsertedTable(bookmark);
       turnOffDataModelEvents();
       dataSet.saveChanges();
       turnOnDataModelEvents();
       commitRow2(bookmark);
   } catch (Exception e) {
       turnOnDataModelEvents();
       if (rowStatus == DataTableModel.DELETED) {
	         logicallyDeletedRows.put(new Long(bookmark), row);
       }
       else if (rowStatus == DataTableModel.INSERTED) {
	         insertedRows.put(new Long(bookmark), row);
       }
       fireDataModelEvent(new DataModelEvent(metaData.getDataModel(), originator, this, bookmark, "", DataModelEvent.AFTER_CHANGE_OF_ROW_DATA));
       throw new DataModelException(bookmark, "", this, DataModelEvent.BEFORE_COMMIT_ROW, e);
   }
   return true;
}

/**
 * Saves row changes to originating data source.
 * Called by BaseDataTable.commitRow().
 * @param bookmark a unique long indicating which row
 * to commit to the datasource.
 * @see BaseDataTable#commitRow
 */
public void saveRow(long bookmark) {
  // codes are in commitRow()
}

/**
 * Rolls back the transaction.
 * Request rollback of the transaction surrounding these changes.
 * This method will be called by cancelRowChanges if the commit policy
 * is on a per row basis, or by cancelAllRowChanges if the policy is
 * on multiple rows or manual.
 * The transaction will only be rolled back if there are no other
 * pending changes in the transaction.
 * No explicit "begin" is required since the SQL Standard
 * implicitly starts a "begin" when any change is made.
 */
public void rollbackTransaction() {}

/**
 * Re-reads a row from the originating data source.
 *
 * @param bookmark a long which uniquely identifies a row
 */
public void refreshRow(long bookmark) throws DataModelException {
    moveToRow(bookmark);
	try {
    	DataRow data_row = new DataRow(dataSet);
    	dataSet.getDataRow(data_row);
    	dataSet.refetchRow(data_row);
    	dataSet.updateRow(data_row);
    	removeRowFromIdeMap(bookmark);
	}
	catch (DataSetException dse) {
		throw new DataModelException(DataModelEvent.INITIALIZE, dse);
	}
}


/**
 * For internal use only.
 * Call DataTableModel.updateCell(..) instead.
 * Changes the value of an existing cell.
 * Called by updateCell.
 * @param row_idx an int indicating the row
 * @param col_idx an int indicating the column
 * @see com.klg.jclass.datasource.DataTableModel#updateCell
 */
public void setValueAt(long bookmark, String col_id, Object value) throws DataModelException {
    requestIDEToUpdateCell(bookmark, col_id, value);
    super.setValueAt(bookmark, col_id, value);
}

/**
 * Override BaseDataTable.cancelRowChanges.
 * Cancels uncommited row changes.
 * Restore original cell values for this row.
 *
 * @param originator the DataModelListener which initiated this action
 * @param bookmark a unique long which identifies this row
 * @exception DataModelException If the changes can't be canceled.
 */
public void cancelRowChanges(DataModelListener originator, long bookmark) throws DataModelException {
    // give the listeners the opportunity to cancel this cancel
    if (!fireDataModelEvent(new DataModelEvent(metaData.getDataModel(), originator, this, bookmark, "", DataModelEvent.BEFORE_CANCEL_ROW_CHANGES))) {
       return;
    }
    // request the ide to cancel the row change, then actually handle it in the
    // listener in meta data
    try {
        moveToRow(bookmark);
        // turnOffDataModelEvents();
        dataSet.cancel();
        // turnOnDataModelEvents();
        // removeRowFromIdeMap(bookmark);
        // super.cancelRowChanges(bookmark);
        ((MetaData)metaData).cancelling = false;
    } catch (Exception e) {
        System.out.println(e.toString());
    }
}

// called when the ide does the cancelling
void cancelRowChanges2(long bookmark) throws DataModelException {
    removeRowFromIdeMap(bookmark);
    super.cancelRowChanges(this, bookmark);
}

/**
 * Changes the value of an existing cell.
 * This should only be called by updateCell.
 * An implementation-specific method for setting cell values, it
 * must be implemented in the derived class of BaseDataTable.
 * @param bookmark a long which uniquely identifies a row
 * @param col_id a String indicating the column
 * @exception Exception If set fails.
 */
private void requestIDEToUpdateCell(long bookmark, String column_id, Object value) throws DataModelException {
	if (getRowStatus(bookmark) != DataTableModel.INSERTED &&
		getMetaData().getCurrentBookmark() != bookmark) {
		moveToRow(bookmark);
	}
	// so we do not recurse
	turnOffDataModelEvents();
	try {
		switch(dataSet.getColumn(column_id).getDataType()) {
		case Variant.BIGDECIMAL: dataSet.setBigDecimal(column_id, (BigDecimal) value); break;
		case Variant.BOOLEAN:    dataSet.setBoolean(column_id, ((Boolean)value).booleanValue()); break;
		case Variant.DATE:       dataSet.setDate(column_id, (java.sql.Date) value); break;
		case Variant.DOUBLE:     dataSet.setDouble(column_id, ((Double)value).doubleValue()); break;
		case Variant.FLOAT:      dataSet.setFloat(column_id, ((Float)value).floatValue()); break;
		case Variant.INT:        dataSet.setInt(column_id, ((Integer)value).intValue()); break;
		case Variant.STRING:     dataSet.setString(column_id, value.toString()); break;
		case Variant.TIMESTAMP:  dataSet.setTimestamp(column_id, (Timestamp) value); break;
		case Variant.TIME:       dataSet.setTime(column_id, (Time) value); break;
		case Variant.BYTE:       dataSet.setByte(column_id, ((Byte) value).byteValue()); break;
		case Variant.SHORT:      dataSet.setShort(column_id, ((Integer)value).shortValue()); break;
		case Variant.LONG:       dataSet.setLong(column_id, ((Long) value).longValue()); break;
		case Variant.INPUTSTREAM: dataSet.setInputStream(column_id, new java.io.ByteArrayInputStream((byte[])value)); break;
		case Variant.BYTE_ARRAY: dataSet.setByteArray(column_id, ((String) value).getBytes(), ((String) value).length()); break;
		case Variant.OBJECT:     dataSet.setObject(column_id, value); break;
		default:
		System.out.println("updateCell(): Unsupported data type " +
		dataSet.getColumn(column_id).getDataType());
		dataSet.setString(column_id, value.toString());
		break;
		}
	}
	catch (DataSetException dse) {
		throw new DataModelException(DataModelEvent.INITIALIZE, dse);
	}
	turnOnDataModelEvents();
}

/**
 * For internal use, do not use.
 * Requery the root table.
 * @return this data table model
 */
public DataTableModel requeryLevel() throws DataModelException {
    store.clear();
    logicallyDeletedRows.clear();
    updatedRows.clear();
    insertedRows.clear();
    ideRowIndexToBookmarkMap.clear();
    ((MetaData)getMetaData()).refreshDataSet();
    copyTable();
    ((MetaData)getMetaData()).cleanChildDataSets();
    return this;
}

/**
 * This simply calls dataModel.enableDataModelEvents() but
 * remembers the previous state.  That is, if they were turned
 * off by the user, don't turn them back on.
 */
private void turnOffDataModelEvents() {
    getMetaData().getDataModel().enableDataModelEvents(false);
}
/**
 * If events were originally on turn them back on.
 */
private void turnOnDataModelEvents() {
    getMetaData().getDataModel().enableDataModelEvents(true); // eventsOriginallyOn);
}

}
