/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: MetaData.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:12 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.jbuilder;

import com.borland.dx.dataset.Variant;

import com.borland.jb.util.ErrorResponse;

import java.sql.*;
import java.util.*;
import java.io.*;

import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;

import com.klg.jclass.datasource.*;


import com.borland.dx.dataset.DataSet;
import com.borland.dx.dataset.ReadRow;

import com.borland.dx.dataset.DataChangeEvent;
import com.borland.dx.dataset.AccessListener;
import com.borland.dx.dataset.EditListener;
import com.borland.dx.dataset.AccessEvent;

import com.borland.dx.dataset.NavigationEvent;
import com.borland.dx.dataset.ReadWriteRow;
import com.borland.dx.dataset.DataSetException;
import com.borland.dx.dataset.StorageDataSet;
import com.borland.dx.dataset.ResolverListener;
import com.borland.dx.dataset.ResolverResponse;
import com.borland.dx.dataset.DataChangeListener;
import com.borland.dx.dataset.NavigationListener;

import com.borland.dx.sql.dataset.QueryResolver;

/**
 * This is the JBuilder implmentation for DataControls. DataControls
 * hold the meta data for DataTables.
 *
 * In terms of JClass Outline Grid, DataControls are the nodes of the
 * MetaTree.  The MetaTree describes the hierarchacal relations between
 * DataControls.
 */
public class MetaData extends IdeMetaData implements DataChangeListener,
    AccessListener,
    NavigationListener,
    EditListener,
    DataModelListener {
    	
static final long serialVersionUID = 2096000383498925335L;

protected DataSet dataSet;

/**
 * After first calling the Root constructor, call this to add
 * master-detail relationships.
 * @param dataModel the HiGrid DataModel to which
 * this meta data object will be added.
 * @param parent the MetaData object which will be the master
 * of this detail MetaData object.
 * @param child the Object (DataSet) which contributes its information
 * to form a child MetaData object of parent in the MetaDataTree.
 * @exception DataModelException If the operation fails.
 */
public MetaData(DataModel dataModel, MetaData parent, Object child) throws DataModelException {
    super(dataModel);
    try {
        this.dataSet = (DataSet) child;
        openDataSet();
        dataSet.enableDataSetEvents(false);
//        dataModel.getMetaDataTree().append((TreeNode) parent, (TreeNode) this); - old
	    ((DataSourceTreeNode)parent).add((MutableTreeNode)this);
        createColumnObjects();
        addYourselfAsListener();
        dataSet.enableDataSetEvents(true);
    } catch (Exception e) {
        throw new DataModelException(DataModelEvent.INITIALIZE, e);
    } 
}

/**
 * Root constructor.
 * Call this constructor first, then call the
 * MetaData(HiGrid DataModel, MetaData, DataSet) constructor
 * to build the MetaDataTree hierarchy for subsequent nodes.
 * @param dataModel the HiGridDataModel to which
 * this meta data object will be added as the root.
 * @param root the Object (DataSet) which contributes its information
 * to form the root MetaData object in the MetaDataTree.
 * @exception DataModelException If the operation fails.
 */

public MetaData(DataModel dataModel, Object root) throws DataModelException {
    super(dataModel);
    try {
        this.dataSet = (DataSet) root;
        openDataSet();
        dataSet.enableDataSetEvents(false);
//        dataModel.getMetaDataTree().setRoot((TreeNode) this); - old
	    ((DataSourceTreeModel)dataModel.getMetaDataTree()).setRoot(this);
        createColumnObjects();
//        dataModel.getDataTableTree().setRoot((TreeNode) new DataTable(this, (DataSet)root)); - old
	    ((DataSourceTreeModel)dataModel.getDataTableTree()).setRoot(new DataTable(this, (DataSet)root));
        addYourselfAsListener();
        dataSet.enableDataSetEvents(true);
    } catch (Exception e) {
        throw new DataModelException(DataModelEvent.INITIALIZE, e);
    }
}



/**
 * Changes the query for this data control.
 * For customizer use during design-time.
 */
public void clear() {}

/**
 * Return the native meta data object.
 * This is a DataSet in JBuilder and a RelationView
 * in VisualCafe.
 * @return the native meta data object for this level.
 */
public Object getNativeMetaData() {
   return dataSet;
}

////////////////////////////////////////////////////////////////////////////////
//////                  Borland listeners
////////////////////////////////////////////////////////////////////////////////
public void dataChanged(DataChangeEvent event) {
    if (!getDataModel().getEventsEnabled()) {
        return;
    }
    
    // get the current data table and bookmark for this level
    int row = event.getRowAffected();
    DataTable dt = (DataTable) getCurrentDataTable();
    if (dt == null) {
        return;
    }

    // check for movement in ide parent when we are not
    // in a hierarchical tree ourselves
    if (event.getID() == DataChangeEvent.DATA_CHANGED) {
        try {
            // is this the case of ide-single level binding
            // with a move in the parent ide level?
            if (handledAsSingleLevelBindingMove()) {
                return;
            }
        } catch (Exception e) {
            System.out.println(e.toString());
            try { dataSet.enableDataSetEvents(true); } catch (Exception x) {}
            return;
        }
    }

    long bookmark = DataTableModel.ROW_NOT_FOUND;
    if (event.getID() != event.ROW_ADDED) {
        try {
            bookmark = dt.getBookmarkFromIdeIndex(row);
        } catch (Exception e) {
            return;
        }
        if (bookmark == DataTableModel.ROW_NOT_FOUND) {
            return;
        }
    }

    // showDebug(event, bookmark, row);

    // made it this far so we have a valid data table reference and bookmark
    switch(event.getID()) {
    case DataChangeEvent.ROW_CHANGED:
        if (!inserting) {
            handleIDEUpdateCellRequest(dt, bookmark);
        }
        break;
    case DataChangeEvent.ROW_ADDED:
        if (inserting) {
              handleIDERowAdd(dt, row);
        }
        else {
            // originator = this
            // we come here when,
            //  cursor moves in ide table on inserted row
            turnOffDataModelEvents();
            try {
                dataSet.saveChanges();
                turnOnDataModelEvents();
                ((DataTable)dt).commitRow2(bookmark);
                requeryCurrentRowAndBelow(this, getCurrentDataTable());
            } catch (Exception e) {
                turnOnDataModelEvents();
                System.out.println(e.toString());
            }
        }
        break;
    case DataChangeEvent.ROW_DELETED:
        if (cancelling) {
            // handleIDECancelRow(dt,bookmark);
            requeryCurrentRowAndBelow(this, getCurrentDataTable());
            cancelling = false;
        }
        else {
            // handleIDERowDelete(dt, bookmark);
            requeryCurrentRowAndBelow(this, getCurrentDataTable());
        }
        break;
    case DataChangeEvent.POST_ROW:
        break;
    case DataChangeEvent.ROW_CHANGE_POSTED:
        handleIDERowPost(dt, bookmark);
        break;
    case DataChangeEvent.DATA_CHANGED:
        break;
    default:
        break;
    }
}

/*
void handleIDECancelRow(DataTable dt, long bookmark) {
    dt.cancelRowChanges2(bookmark);
}
*/

void handleIDERowPost(DataTable dt, long bookmark) {
    // Borlands row status variables are
    //  DEFAULT, DELETED, INSERTED, LOADED,
    //  ORIGINAL, UPDATED, PENDING_RESOLVED [internal]
    switch(dt.getRowStatus(bookmark)) {
    case DataTableModel.UPDATED:
        try {
            // original means updated row has been saved
            if ( (dataSet.getStatus() & (com.borland.dx.dataset.RowStatus.ORIGINAL)) == 0) {
                // System.out.println("row was saved");
                turnOffDataModelEvents();
                dataSet.saveChanges();
                turnOnDataModelEvents();
                ((DataTable)dt).commitRow2(bookmark);
            }
        } catch (Exception e) {
            System.out.println(e.toString());
        }
        break;
    }
}

/**
 * Received the row delete event from JBuilder.  Remove the row from
 * our displays.

private void handleIDERowDelete(DataTable dt, long bookmark) {
    if (inserting) {
        return;
    }
    switch(dt.getRowStatus(bookmark)) {
    case dt.DELETED:
    case dt.COMMITTED:
    case dt.UPDATED:    ((DataTable)dt).deleteRow2(bookmark); break;
    case dt.INSERTED:   ((DataTable)dt).cancelChanges(bookmark);break;
    }
}
 */

/**
 * We received an event from the JBuilder data source telling us
 * to add a new row.
 */
private void handleIDERowAdd(DataTable dt, int row) {
    try {
        long newBookmark = ((IdeDataTable)dt).addRow2(row);
    } catch (Exception e) {
        System.out.println(e.toString());
    }
}



public void postRow(DataChangeEvent event) {}

///// Borland AccessListener method
public void accessChange(AccessEvent event) {
	switch(event.getID()) {
	case AccessEvent.OPEN:
		try {
			openDataSet();
		} catch(Exception e) {
            event.appendException(e);
		}
		break;
	case AccessEvent.CLOSE:
        break;
	default:
		break;
	}
}

// Navigation listener
// Any moves in JBuilder components are reflected in our
// data source via this method
public void navigated(NavigationEvent event) {
    DataSet ds = (DataSet) event.getSource();
    int row = dataSet.getRow();
    if (!inserting) {
        navigated(row);
    }
}

///////// EditListeners

// before post of new row to ds
private boolean addingRow = false;
public void adding(DataSet dataSet, ReadWriteRow row) {
    addingRow = true;
    // System.out.println("EditListener: adding row "+ dataSet.getRow());
}
// after new row posted to ds
public void added(DataSet dataSet) {
    // System.out.println("EditListener: row added "+ dataSet.getRow());
    addingRow = false;
}
public void addError(DataSet dataSet, ReadWriteRow row, DataSetException exception, ErrorResponse response) {
    // System.out.println("Exception during row add");
}

// edits to a row are about to be undone
boolean cancelling = false;
public void canceling(DataSet dataSet) {
    // System.out.println("EditListener: cancelling");
    rowToDeleteOrCancel = dataSet.getRow();
    // when the ide data source call cancel, we get a delete row for some
    // reason.  Thus we set this flag to know that we really should not delete
    // the row.  When the JClass DataSource does the cancelling, we need to
    // reset this flag to false since the events are not called above (which
    // is where it is reset when the ide does the cancel.
    cancelling = true;
}
private int rowToDeleteOrCancel = -1;
// before row delete from ds
public void deleting(DataSet dataSet) {
    // System.out.println("EditListener: deleting row " + dataSet.getRow());
    rowToDeleteOrCancel = dataSet.getRow();
}
// row was deleted from ds
public void deleted(DataSet dataSet) {
    // System.out.println("EditListener: row deleted");
}
public void deleteError(DataSet dataSet, DataSetException exception, ErrorResponse response) {
    // System.out.println("EditListener: delete error");
}

public void editError(DataSet dataSet, com.borland.dx.dataset.Column column, com.borland.dx.dataset.Variant variant,
    DataSetException exception, ErrorResponse response) {
    // System.out.println("EditListener: edit error");
}

// before a insert into ds of a new unposted row
private boolean inserting = false;
public void inserting(DataSet dataSet) {
    // System.out.println("EditListener: inserting " + dataSet.getRow());
    inserting = true;
}
public void inserted(DataSet dataSet) {
    // System.out.println("EditListener: inserted" + dataSet.getRow());
    inserting = false;
}
public void modifying(DataSet dataSet) {
    // System.out.println("EditListener: modifying " + dataSet.getRow());
}

// modified row about to be posted to ds
public void updating(DataSet dataSet, ReadWriteRow newRow, ReadRow oldRow) {
    // System.out.println("EditListener: updating " + dataSet.getRow());
}
public void updateError(DataSet dataSet, ReadWriteRow row, DataSetException exception, ErrorResponse response) {
    // System.out.println("EditListener: updateError " + dataSet.getRow());
}
// modified row was posted to ds
public void updated(DataSet dataSet) {
    // System.out.println("EditListener: updated " + dataSet.getRow());
}


/*
 * Some private helpers
 */

/**************************************************************************
private void showDebug(DataChangeEvent event, long bookmark, int row) {
try {
        System.out.println("metaData = " + getDescription());
        System.out.println("ide row = " + row);
        System.out.println("current ide row = " + dataSet.getRow());
        System.out.println("current ide row status = " + dataSet.getStatus());
        System.out.println("bookmark = " + bookmark);
        System.out.print("action = ");
        String debugString = "";
        switch(event.getID()) {
        case DataChangeEvent.ROW_CHANGED:   debugString += "ROW_CHANGED"; break;
        case DataChangeEvent.ROW_ADDED:     debugString += "ROW_ADDED"; break;
        case DataChangeEvent.ROW_DELETED:   if (cancelling) { debugString += "CANCELLING"; } else { debugString +=" DELETE_ROW"; } break;
        case DataChangeEvent.POST_ROW:      debugString += "POST_ROW "; break;
        case DataChangeEvent.ROW_CHANGE_POSTED: debugString += "ROW_CHANGE_POSTED "; break;
        case DataChangeEvent.DATA_CHANGED:  debugString += "DATA_CHANGED"; break;
        }
        System.out.println(debugString);
        System.out.println("========= END OF EVENT ==============");

    } catch (Exception e) {}
}
******************************************************************************/

/**
 * Listens for changes originating from the JBuilder data source.
 */
private void addYourselfAsListener() { // throws DataModelException {
    if (dataSet != null) {
        dataSet.addAccessListener(this);
        dataSet.addDataChangeListener(this);
        dataSet.addNavigationListener(this);
        StorageDataSet sds = dataSet.getStorageDataSet();
        if (sds != null) {
            try {
                sds.addEditListener(this);
            } catch (Exception e) {
                // System.out.println("JBuilder MetaData.addYourselfAsListener: " + e.toString());
            }
        }
    }
}

/**
 * Opens the data set if it is not already open.
 * @exception DataModelException If open fails.
 */
private void openDataSet() throws DataModelException {
   try {
        if (!dataSet.isOpen()) {
            dataSet.open();
        }
        DataTableModel dt = getCurrentDataTable();
        if (dt != null) {
            // originator = this
            requeryCurrentRowAndBelow(this, dt);
        }
   } catch (Exception e) {
       throw new DataModelException(DataModelEvent.INITIALIZE, e);
   }
}

/** Creates a jdbc.db.ColumnModel object for each jbuilder column. */
private void createColumnObjects() {
   try {
       int cols = dataSet.getColumnCount();
       for (int i = 0; i < cols; i++) {
           ColumnModel cm = new com.klg.jclass.datasource.jbuilder.Column(dataSet.getColumn(i));
           addColumn(cm);
       }
   } catch (Exception e) {}
}

/**
 * For internal use only.
 * Call DataSet.refresh, called by DataTableModel.requeryLevel
 * @exception DataModelException If an exception occurs while refreshing the data sets.
 */
public void refreshDataSet() throws DataModelException {
    turnOffDataModelEvents();
    if (dataSet.refreshSupported()) {
    	try {
        	dataSet.refresh();
    	} 
    	catch (DataSetException dse) {
    		turnOnDataModelEvents();
    		throw new DataModelException(DataModelEvent.INITIALIZE, dse);
    	}
    }
    turnOnDataModelEvents();
}

//================== DataModelListener interface methods ==================

/**
 * Called before a row is inserted. Can be cancelled.
 * If not cancelled will be followed by afterInsertRow().
 * @see DataTableModel#addRow
 */
public void beforeInsertRow(DataModelEvent e) {}

/**
 * Called before entire grid is reset. Can be cancelled.
 * If not cancelled will be followed by afterReset().
 * This will result from a call to DataModel.requeryAll().
 * @see DataModel#requeryAll
 */
public void beforeReset(DataModelEvent e) {}

/**
 * Called when either DataTableModel.requeryRowAndDetails() or
 * DataTableModel.requeryRow() is called.
 * If not cancelled will be followed by an,
 * <ul>
 * <li>afterRequeryRowAndDetails() call or an</li>
 * <li>afterDeleteRow() call in the case getRowStatus() == INSERTED or a</li>
 * <li>afterChangeOfRowData() call in the case getRowStatus() == UPDATED or COMMITTED.</li>
 * </ul>
 * @see DataTableModel#requeryRow
 * @see DataTableModel#requeryRowAndDetails
 */
public void beforeRequery(DataModelEvent e) {}

/**
 * Called before a cell is edited. Can be cancelled.
 * @see DataTableModel#updateCell
 */
public void beforeEditCell(DataModelEvent e) {}

/**
 * Called before a row is [logically] deleted. Can be cancelled.
 * If not cancelled will be followed by an
 * afterDeleteRow90 or a afterChangeOfRowData() if the commit policy
 * is COMMIT_MANUALLY or COMMIT_LEAVING_ANCESTOR.
 * @see DataTableModel#deleteRow
 * @see MetaDataModel#getCommitPolicy
 */
public void beforeDeleteRow(DataModelEvent e) {}

/**
 * Called before all changes are committed. Can be cancelled.
 * All modified, deleted and inserted rows at all levels are about
 * to be committed.  beforeDeleteRow() and afterChangeOfRowData() 
 * will follow depending on the operations performed on the modified rows
 * being saved.
 * Results from a call to DataModel.updateAll.
 * @see DataModel#updateAll
 */
public void beforeCommitAll(DataModelEvent e) {}

/**
 * Called before all changes are cancelled. Can be cancelled.
 * afterInsertRow() and afterChangeOfRowData() can follow this
 * event.
 * @see DataModel#cancelAll
 */
public void beforeCancelAll(DataModelEvent e) {}

/** Called before all edits to a row are undone. Can be cancelled
 * An afterDeleteRow() or afterChangeOfRowData() will follow.
 * @see DataTableModel#cancelRowChanges
 */
public void beforeCancelRowChanges(DataModelEvent e) {}

/**
 * Called when the root-level bookmark for a subtree changes.
 * When this happens those nodes in the previous subtree which
 * are not COMMIT_MANUALLY are committed.
 * Can be cancelled.  If cancelled the cursor moves but the
 * changes are automatically committed.
 */
public void beforeCommitConditional(DataModelEvent e) {}

/**
 * Called before single row is committed to data source.  Can be cancelled in
 * which case the row edits are not written to the datasource and the rows
 * status remains modified.  afterDeleteRow() or afterChangeOfRowData() will
 * follow depending on the status of the row to be committed.
 * @see DataTableModel#commitRow
 */
public void beforeCommitRow(DataModelEvent e) {}

/**
 * The global cursor will move to a new row.
 * Can be cancelled.
 */
public void beforeMoveToCurrentRow(DataModelEvent e) {}

/**
 * The indicated DataTable will be deleted and flushed from the cache.
 * Can be cancelled.
 */
public void beforeDeleteTable(DataModelEvent e) {}

/**
 * A row has changed, re-read all its cells and its status to reflect the new
 * values.  If this event is the result of a cell edit call DataModelEvent.getColumn()
 * to get the name of the column which changed. If getColumn() returns "", re-read
 * the entire row.
 * Called when
 * <ul>
 * <li>a row is deleted and getShowDeletedRows() == true</li>
 * <li>a cell was edited</li>
 * <li>row edits are cancelled and getRowStatus() == UPDATED</li>
 * <li>row edits are cancelled and getRowStatus() == DELETED and getShowDeletedRows() == true</li>
 * <li>row is commited and getRowStatus() == UPDATED or INSERTED</li>
 * <li>row is requeried and getRowStatus() != INSERTED</li>
 * </ul>
 */
public void afterChangeOfRowData(DataModelEvent e) {}

/**
 * A new row has been added to the datasource. Listeners need to
 * display the row.  Rows are always added to the end of DataTableModels.
 */
public void afterInsertRow(DataModelEvent e) {}

/**
 * Removes the row from the display.
 * A row has been physically deleted and needs to be
 * removed from the display or has been logically deleted but the
 * showDeletedRows property has been set to false. Called when,
 * <ul>
 * <li>a row has been logically deleted and getShowDeletedRows() == false</li>
 * <li>row changes have been cancelled and getRowStatus() == INSERTED</li>'
 * <li>a row is committed and getRowStatus() == DELETED and getShowDeletedRows() == true</li>
 * <li>a row has been requeried and getRowStatus() == INSERTED</li>
 * </ul>
 */
public void afterDeleteRow(DataModelEvent e) {}

/**
 * Listeners must close all expanded views and reset/reread the root node.
 * The previous pointer to the root node is no longer valid.  Call
 * DataModel.getDataTableTree().getRoot() for the new root table.
 * Called when the datasource has been reset.
 * @see DataModel#requeryAll
 */
public void afterReset(DataModelEvent e) {}

/**
 * The global cursor has moved to a new row.
 * Listeners should position their cursor on
 * the indicated row.  In a master-detail relationship child levels
 * should refresh themselves to reflect data sets which correspond
 * to the new parent row by calling DataModel.getCurrentDataTable()
 * or for field controls, DataModel.getCurrentDataItem().
 */
public void afterMoveToCurrentRow(DataModelEvent e) {}

/**
 * Re-reads the indicated row and refresh all open children
 * under this row.
 */
public void afterRequeryRowAndDetails(DataModelEvent e) {}

/**
 * Re-read this table and refresh all open children in
 * this table.
 */
public void afterRequeryTable(DataModelEvent e) {}

/**
 * Notification that multiple events are coming.
 * Multiple events will be nested
 * between beginEvents() and endEvents().
 * Allows listeners to treat the events as a batch, to,
 * for example, reduce repaints.
 */
public void beginEvents(DataModelEvent e) {}

/**
 * Notification that multiple events are complete.
 * Multiple events will be nested
 * between beginEvents() and endEvents().
 * Allows listeners to treat the events as a batch, to,
 * for example, reduce repaints. Called when
 * <ul>
 * <li>DataModel.updateAll is called</li>
 * <li>DataModel.cancelAll is called</li>
 * </ul>
 */
public void endEvents(DataModelEvent e) {}

/**
 * The current row has been deleted and the originator
 * of the deletion should now reposition the global
 * cursor to a new, valid row.
 */
public void originatorNavigateRow(DataModelEvent e) {}

/**
 * For internal use only
 * Used by the JCData bean.
 */
public void initialize(DataModelEvent e) {}


}
