/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Column.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:12 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.jdbc;

import java.io.Serializable;
import java.sql.ResultSetMetaData;
import java.util.Enumeration;

import com.klg.jclass.datasource.*;
import com.klg.jclass.datasource.util.*;

/**
 * A copy of the ResultSetMetaData for a column.
 * A copy is kept rather than the original ResultSetMetaData so
 * the ResultSet can be explicitly closed, and because with some
 * drivers there is no guarantee it will be preserved.  Moreover,
 * only one copy is kept rather than a ResultSetMetaData object
 * for each level.
 *
 * ResultSetMetaData methods are mirrored here, see their
 * documentation for descriptions.
 *
 * Some drivers do not support all ResultSetMetaData methods.
 * If they do not the following defaults are used:<P>
 * catalog = null<P>
 * displayWidth = 25<P>
 * name = null<P>
 * label = null<P>
 * type = java.sql.Types.OTHER<P>
 * typeName = "OTHER"<P>
 * precision = 5<P>
 * scale = 5<P>
 * schema = null<P>
 * table = null<P>
 * isAutoIncrement = false<P>
 * isCaseSensitive = false<P>
 * isCurrency = false<P>
 * isDefinitelyWritable = true<P>
 * isNullable = ResultSetMetaData.columnNullable<P>
 * isReadOnly = false<P>
 * isSearchable = true<P>
 * isSigned = false<P>
 * isWritable = true<P>
 */

public class Column extends BaseColumn {
	
static final long serialVersionUID = 15861668727816170L;

Column(ResultSetMetaData rsmd, int position) {
    try {
        try {
            setCatalogName(rsmd.getCatalogName(position));
        }
        catch (Exception e) {
            setCatalogName(null);
        }
        try {
            setColumnDisplaySize(rsmd.getColumnDisplaySize(position));
        }
        catch (Exception e) {
            setColumnDisplaySize(25);
        }
        try {
            setColumnName(rsmd.getColumnName(position));
        }
        catch (Exception e) {
            setColumnName(null);
        }
        try {
            setColumnLabel(rsmd.getColumnLabel(position));
        }
        catch (Exception e) {
            setColumnLabel(null);
        }
        try {
            setColumnType(rsmd.getColumnType(position));
        }
        catch (Exception e) {
            setColumnType(java.sql.Types.OTHER);
        }
   		try {
       		setColumnTypeName(rsmd.getColumnTypeName(position));
   		} catch (Exception e) {
       		setColumnTypeName("OTHER");
   		} 
        try {
            setPrecision(rsmd.getPrecision(position));
        }
        catch (Exception e) {
            setPrecision(5);
        }
        try {
            setScale(rsmd.getScale(position));
        }
        catch (Exception e) {
            setScale(5);
        }
        try {
            setSchemaName(rsmd.getSchemaName(position));
        }
        catch (Exception e) {
            setSchemaName(null);
        }
        try {
            setTableName(rsmd.getTableName(position));
        }
        catch (Exception e) {
            setTableName(null);
        }
        try {
            setAutoIncrement(rsmd.isAutoIncrement(position));
        }
        catch (Exception e) {
            setAutoIncrement(false);
        }
        try {
            setCaseSensitive(rsmd.isCaseSensitive(position));
        }
        catch (Exception e) {
            setCaseSensitive(false);
        }
        try {
            setCurrency(rsmd.isCurrency(position));
        }
        catch (Exception e) {
            setCurrency(false);
        }
        try {
            setDefinitelyWritable(rsmd.isDefinitelyWritable(position));
        }
        catch (Exception e) {
            setDefinitelyWritable(true);
        }
        try {
            setNullable(rsmd.isNullable(position));
        }
        catch (Exception e) {
            setNullable(ResultSetMetaData.columnNullable);
        }
        try {
            setReadOnly(rsmd.isReadOnly(position));
        }
        catch (Exception e) {
            setReadOnly(false);
        }
        try {
            setSearchable(rsmd.isSearchable(position));
        }
        catch (Exception e) {
            setSearchable(true);
        }
        try {
            setSigned(rsmd.isSigned(position));
        }
        catch (Exception e) {
            setSigned(false);
        }
        try {
            setWritable(rsmd.isWritable(position));
        }
        catch (Exception e) {
            setWritable(true);
        }
        setMetaColumnTypeFromSqlType(getColumnType());
    }
    catch (Exception e) {
        //
    }
}

/**
 * Determine unique identifier for this column.
 * If the statement uses an alias, use alias. If it
 * uses qualified name, use qualified name.
 * @param parser a SqlParser instance for the query
 */
protected void determineIdentifier(SqlParser sp) {

	boolean stop = false;
   	for (Enumeration e = sp.getColumns().elements(); e.hasMoreElements();) {
		SqlStatementColumn c = (SqlStatementColumn) e.nextElement();
		String name = c.getColumnName();

        // handle case where this is the aliased column
        if (c.getAlias().equals(getColumnName())) {
        	setIdentifier(c.getAlias());
        	setColumnName(c.getColumnName());
        	break;
        }

        // since name can be duplicated, must ensure they match on
        // qualifier as well
        else if (c.getColumnName().equals(getColumnName())) {
            String qualifier = (getCatalogName() != null && !getCatalogName().equals("")) ? (getCatalogName() + ".") : "";
            qualifier += (getSchemaName() != null && !getSchemaName().equals("")) ? getSchemaName() + "." : "";
            qualifier += (getTableName() != null && !getTableName().equals("")) ? getTableName() : "";

            String qualifier2 = (c.getCatalogName() != null && !c.getCatalogName().equals("")) ? (c.getCatalogName() + ".") : "";
            qualifier2 += (c.getSchemaName() != null && !c.getSchemaName().equals("")) ? c.getSchemaName() + "." : "";
            qualifier2 += (c.getTableName() != null && !c.getTableName().equals("")) ? c.getTableName() : "";

           	if (qualifier.equals(qualifier2)) {
            	setIdentifier(c.getAlias());
               	break;
        	}
           	else {
            	for (Enumeration i = sp.getTables().elements(); i.hasMoreElements(); ) {
                	SqlStatementTable t = (SqlStatementTable) i.nextElement();
                   	if (t.getAlias().equals(qualifier) ) {
                    	setIdentifier(c.getAlias());
                    	stop = true;
                    	break;
                   	}
               	}
           	}
       	}
       	if (stop) {
    		break;
       	}
	}
   	// otherwise the default is identifier = column name
}

}
