/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JDBCBinding.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:12 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.jdbc;

import com.klg.jclass.datasource.Binding;
import com.klg.jclass.datasource.DataModelException;
import com.klg.jclass.datasource.DataModel;
import com.klg.jclass.datasource.TreeData;
import com.klg.jclass.util.JCListenerList;

import java.util.Enumeration;

/**
 *
 * A single level.
 *
 */
public class JDBCBinding extends Binding {

static final long serialVersionUID = -920646318559618093L;
	
/**
 *
 * Constructor.
 *
 */
public JDBCBinding(DataModel dataModel) {
	super(dataModel);
}

/**
 *
 * Root-level constructor.
 * @param	connection the DataTableConnection to a data source.
 * @param	query the String query to be used to retrieve data for this level.
 *
 */
public JDBCBinding(DataTableConnection connection, String query) throws DataModelException {
	super(new TreeData());
	metaDataModel 	= new MetaData(dataModel, connection, query);
	metaDataModel.setBinding(this);
	((MetaData)metaDataModel).open();
}
	
/** 
 * 
 * Constructor for child levels. Builds the WHERE clause automatically by appending
 * a WHERE clause to the passed in query and the join columns.
 * @param	connection the DataTableConnection to a data source.
 * @param	query the String query to be used to retrieve data for this level.
 * @param	parent the Binding which is the parent of this level.
 * @param 	an array of column names on which this level joins to its parent. If the column names
 * 			do not match between tables, use the constructor which takes a two-dimensional array
 *			of join column names. For example, here both tables join on CustomerID,
 *			<pre>
 *			JDBCBinding binding2 = new JDBCBinding(c, query2, binding, new String[] { "CustomerID" });
 *			<pre>
 *
 */
public JDBCBinding(DataTableConnection connection, String query, Binding parent, String[] joinColumns) 
		throws DataModelException {
		
	this(connection, query, parent);

	// add the join columns
	for (int i = 0; i < joinColumns.length; i++) {
		addJoinColumn(joinColumns[i], joinColumns[i]);
	}

	// call open
	open();			
}

/** 
 * 
 * Constructor for child levels. Builds the WHERE clause automatically by appending
 * a WHERE clause to the passed in query and the join columns.
 * @param	connection the DataTableConnection to a data source.
 * @param	query the String query to be used to retrieve data for this level.
 * @param	parent the Binding which is the parent of this level.
 * @param 	a two-dimensional array of column names on which this level joins to its parent. 
 *			For example, suppose the table SalesDetails (stor_id, ord_num) joins on
 * 			Sales (store_id, order_number) the constructor would look like this,
 *			<pre>
 *			String joinColumns[][] = { {"stor_id", "store_id"}, {"ord_num", "order_number"} }
 *			JDBCBinding binding2 = new JDBCBinding(c, query2, binding, joinColumns);
 *			<pre>
 * 			
 */
public JDBCBinding(DataTableConnection connection, String query, Binding parent, String[][] joinColumns) 
		throws DataModelException {
	
	this(connection, query, parent);
			
	// add the join columns
	for (int i = 0; i < joinColumns.length; i++) {
		addJoinColumn(joinColumns[i][0], joinColumns[i][1]);
	}

	// call open
	open();
}

/**
 * 
 */
protected JDBCBinding(DataTableConnection connection, String query, Binding parent) 
		throws DataModelException {
	
	super(parent);
			
	this.dataModel 	= parent.getDataModel();
	metaDataModel	= new MetaData(parent.getDataModel(), parent.getMetaDataModel(), connection);
	metaDataModel.setBinding(this);
	dataModel.addDataModelListener(eventListener);
		
	((MetaData)metaDataModel).setStatement(query);
}

/**
 *
 * Get the query for this level.
 * @return	the query String for this level.
 *
 */
public String getQuery() {
	return ((MetaData)metaDataModel).getStatement();
}
	
/**
 *
 * Set the data model for this level.
 * @param	dataModel the data model for this level.
 *
 */
public void setDataModel(DataModel dataModel) {
	this.dataModel = dataModel;
}

/**
 *
 * Add a join column to the list of columns joining this level
 * to its parent.
 * @param	childColumn the name of the column in this level.
 * @param	parentColumn the name of corresponding column in the parent.
 *
 */
public void addJoinColumn(String childColumn, String parentColumn) {
	((MetaData)metaDataModel).joinOnParentColumn(parentColumn, childColumn);
}

/**
 *
 * Open the level.
 *
 */
public void open() throws DataModelException {

	// by this time all the join columns have been set,
	// so add them to the WHERE clause
	StringBuffer whereClause = new StringBuffer(" WHERE ");
	Enumeration e = ((MetaData)metaDataModel).getJoinColumns().elements();
	boolean first = true;
	
	while (e.hasMoreElements()) {
		if (first) {
			first = false;	
		}
		else {
			whereClause.append(" AND ");
		}

		com.klg.jclass.datasource.util.ColumnMap cm = (com.klg.jclass.datasource.util.ColumnMap) e.nextElement();
		whereClause.append(cm.getChildColumn());
		whereClause.append(" = ? ");
	}
	
	// now set the WHERE clause
	((MetaData)metaDataModel).setWhereClause(whereClause.toString());
	
	((MetaData)metaDataModel).open();
	
}

}
