/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: MetaData.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:12 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.jdbc;

import java.beans.Beans;
import java.io.Serializable;
import java.sql.*;

import java.util.Vector;

import java.util.*;




import javax.swing.tree.*;

import com.klg.jclass.datasource.*;
import com.klg.jclass.datasource.util.*;
import com.klg.jclass.datasource.LocaleBundle;
import com.klg.jclass.util.JCEnvironment;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;

/**
 * This MetaData connects to a data source through JDBC.
 * Each instance of this MetaData will have a particular query
 * associated with it.  This MetaData will execute that query
 * and cache the results.
 * <P>
 * When used in the context of the JClass HiGrid, multiple result
 * sets will be cached.  These result sets will be based on the same
 * query but with different parameters. When used in JClass HiGrid,
 * this control will be a node in the MetaTree describing the
 * relationships between SQL queries.
 * <P>
 * Transaction support is built on top of JDBC transactions.
 * To turn on transaction support, "auto commit" must be disabled
 * on the JDBC Connection which is passed into this object. Alternatively,
 * calling setAutoCommit(false) on this MetaData object does the same.
 * Since the same Connection can be shared among one or more MetaData objects
 * calling MetaData.setAutoCommit(false) is redundant on all but the first.
 * <P>
 * Commits and rollbacks happen on a per Connection basis.
 * If all MetaData objects share the same Connection, modifications spanning
 * levels will be committed as group within one transaction.
 * If multiple Connections objects are used however, say for example, one per
 * MetaData object, RootMetaData.rollbackTransaction() will recurse
 * and rollback transaction on all those MetaData objects which have
 * not yet been successfully committed. If they have been committed,
 * they are not rolledback and their changes are permanent.  Those on
 * which a successful commitTransaction has not occurred will be rolled back.
 * <P>
 * The default is to accept the JDBC default which is "auto commit" is
 * "true".  This means that unless the auto commit on the Connection is
 * explicitly set to "false" (by calling setAutoCommit(false) on the
 * Connection object directly or indirectly through MetaData.setAutoCommit(false),
 * each statement will be commit upon completion.
 */
public class MetaData extends BaseMetaData implements Serializable {

static final long serialVersionUID = 4469150227741079237L;

public static final int DEFAULT_DESIGN_TIME_MAX_ROWS = 10;

/**
 * Encapsulates JDBC connection.
 */
protected DataTableConnection ds_connection;

/**
 * The JDBC version of the query.
 */
protected transient PreparedStatement query_statement;

/**
 * Number of colums for this ds.
 */
protected int numColumns = 0;

/**
 * Primary keys.
 * @since JClass 4.0K for Java 2. If you are using JClass 4.0J,
 * which is based on JDK 1.1, this field's or method's return type
 * is based on the corresponding <code>com.sun.java.util</code> class.
 */
protected Hashtable tableToPrimaryKeyMap = new Hashtable();

/**
 * This data control's parent.
 */
protected MetaData parent_control;

/**
 * These are the keys columns
 * used by this control to link to its
 * parent.
 */
protected Vector join_columns = new Vector();

/**
 * Vector of database tables for this level.
 */
protected Vector tables;

/**
 * The query statement is used to populate this DataTable
 */
protected String query;

/**
 * Vector of column indexes for a table.
 * The table is indexed by its own Vector.
 */
protected Vector tableColumnIndexes = new Vector();

/**
 * Returns true if this instance been initialized.
 */
protected boolean open = false;

/**
 * The table which drives a row when there is
 * more than one table for a row.
 */
protected String driverTable = null;


/**
 * Class responsible for saving changes to db
 */
protected PersistorModel persistor = null;

protected SqlParser parser;

protected static int designTimeMaxRows = DEFAULT_DESIGN_TIME_MAX_ROWS;

protected static int NOT_FOUND = -1;

/**
 * MetaData Constructor. Must call setConnection before a query
 * can be made.
 */
MetaData(DataModel dataModel) {
    super(dataModel);
    createDefaultPersistor();
}


/**
 * General purpose MetaData Constructor.
 * Can be used as an alternate to the convenience root meta data
 * constructor to, say, set query parameters on the root.  For example,
 * <pre><code>
 *     MetaData Orders = new MetaData(this, connection);
 *     Orders.setStatement("select * from Orders where OrderID < ?");
 *     Orders.setDescription("Orders");
 *     Orders.setParameter(1, new Integer(OrderIdParameter), java.sql.Types.INTEGER);
 *     Orders.setMaxRows(5);
 *     getMetaDataTree().setRoot(Orders);
 *     getDataTableTree().setRoot(Orders.executeQuery());
 * </pre></code>
 * As this example shows, this constructor does not automatically
 * add this MetaData object to the meta data tree as do the
 * convenience constructors.  It must be added manually.  Since in this
 * example we created the root-level node we must also manually
 * set the root of the data table tree.
 * <P>
 * Your query must include all the primary key columns for each table
 * in the select statement, otherwise update anomalies may occur.
 * <P>
 * @param dataModel the DataModel
 * @param connection a DataTableConnection to a data source
 */
public MetaData(DataModel dataModel, DataTableConnection ds_connection) {
    this(dataModel);

    this.ds_connection = ds_connection;

    tables = new Vector();
}

/**
 * Convenience root constructor which allows max rows to be set.  Useful for design
 * time where only a sampling of rows is required.
 * Sets and executes the query to bootstrap root levels of
 * the MetaDataTree and the DataTableTree.
 * This constructor executes the query and sets the resulting
 * DataTable as the root of the DataTableTree.
 * Call this constructor first, then call the
 * MetaData(DataTableConnection, DataModel, MetaData parent)
 * to build the MetaDataTree hierarchy.
 * Sets "open" to true.
 * <P>
 * Your query must include all the primary key columns for each table
 * in the select statement, otherwise update anomalies may occur.
 * <P>
 *
 * @param connection a DataTableConnection to a data source
 * @param dataModel the DataModel to which
 * this meta data object will be added.
 * @param query the String sql query the results of which will
 * form the root level in the MetaDateTree.
 * @param maxRows an int restricting the number of rows this
 * query can return.
 * @exception DataModelException If the operation fails.
 */
public MetaData(DataModel dataModel, DataTableConnection ds_connection,
                String query, int maxRows) throws DataModelException {
    this(dataModel, ds_connection);
//FIXBIZARRE
//FIXOBSOLETE
//FIX
//System.out.println("dataModel = " + dataModel);
//System.out.println("metadatatree = " + dataModel.getMetaDataTree());
//System.out.println("query = " + query);
    MetaDataModel existingRoot = null;
    if (dataModel.getMetaDataTree().getRoot() != null) {
        DataSourceTreeNode rRoot = (DataSourceTreeNode)dataModel.getMetaDataTree().getRoot();
//    MetaDataModel existingRoot = (MetaDataModel) dataModel.getMetaDataTree().getRoot();
//        existingRoot = (MetaDataModel) dataModel.getMetaDataTree().getRoot();
    }
    if (existingRoot != null) {
        Exception e = new Exception(LocaleBundle.string(LocaleBundle.root_already_set) + existingRoot.getDescription());
        throw new DataModelException(DataModelEvent.INITIALIZE, e);
    }
    ((DataSourceTreeModel)dataModel.getMetaDataTree()).setRoot(this);
//    try {
        setStatement(query);
        this.query = query;

        // force open to get the meta data, this call
        // must come before setting max rows since open()
        // sets max rows to none so no time is wasted
        // transfering results we're not using. This is
        // to fix an oracle bug.
        open();

        if (JCEnvironment.isDesignTime()) {
            setMaxRows(designTimeMaxRows);
        }
        else if (maxRows > -1) {
            setMaxRows(maxRows);
        }
        ((DataSourceTreeModel)dataModel.getDataTableTree()).setRoot(executeQuery());
        open = true;
//    }
//    catch (SQLException e) {
//        open = false;
//        throw new DataModelException(DataModelEvent.INITIALIZE, e);
//    }
}

/**
 * Convenience root constructor.
 * Sets and executes the query to bootstrap root levels of
 * the MetaDataTree and the DataTableTree.
 * This constructor executes the query and sets the resulting
 * DataTable as the root of the DataTableTree.
 * Call this constructor first, then call the
 * MetaData(DataTableConnection, DataModel, MetaData parent)
 * to build the MetaDataTree hierarchy.
 * Sets "open" to true.
 * <P>
 * Your query must include all the primary key columns for each table
 * in the select statement, otherwise update anomalies may occur.
 * <P>
 *
 * @param connection a DataTableConnection to a data source
 * @param dataModel the DataModel to which
 * this meta data object will be added.
 * @param query the String sql query the results of which will
 * form the root level in the MetaDateTree.
 * @exception DataModelException If the operation fails.
 */
public MetaData(DataModel dataModel,
                DataTableConnection ds_connection, String query)
throws DataModelException {
    this(dataModel, ds_connection, query, -1);
}

/**
 * After first calling the Root constructor, call this to add
 * master-detail relationships. After calling setStatement() and
 * joinOnParentColumn(), you must call open().
 *
 * @param dataModel the HiGrid DataModel to which
 * this meta data object will be added.
 * @param parent the MetaData object which will be the master
 * of this detail MetaData object.
 * @param connection a DataTableConnection to a data source
 */
public MetaData(DataModel dataModel, MetaDataModel parent,
                DataTableConnection ds_connection) {
    this(dataModel, ds_connection);
//    dataModel.getMetaDataTree()).append(parent, this);
    ((DataSourceTreeNode)parent).add((MutableTreeNode)this);
}

/**
 * Sets this data control's connection to DataTableConnection.
 * @param connection a DataTableConnection to a data source
 */
public void setConnection(DataTableConnection ds_connection) {
    this.ds_connection = ds_connection;
}

/**
 * Gets this data control's DataTableConnection.
 * @return this data control's DataTableConnection
 */
public DataTableConnection getDataTableConnection() {
    return this.ds_connection;
}

/**
 * Set the SQL statement used to retrieve data.
 * The statement can be a select statement or a call to stored procedure.
 * For example,
 *
 *		setStatement(" select * from products where id = ? ") or
 *		setStatement(" {call sp_product_info(?)} ")
 *
 * Note: the primary keys for each table involved in the select
 * statement must be included in the columns selected.
 * @param query a String, the SQL statement to be executed
 * @exception DataModelException If set fails.
 */
public void setStatement(String query) throws DataModelException {
    this.query = query;
    parser = new SqlParser(query);
    try {
        // clean up in case the garbage collector misses it
        if (query_statement != null) {
            query_statement.close();
        }
        query_statement = ds_connection.getConnection().prepareCall(query);
    }
    catch (SQLException e) {
        try {
            query_statement = ds_connection.getConnection().prepareStatement(query);
        }
        catch (SQLException x) {
            throw new DataModelException(DataModelEvent.INITIALIZE, x);
        }
    }
}

/**
 * Change or add a where clause for the existing query statement.
 * After setting the query statement by calling setStatement() you
 * can call this method to change (or add) the where clause of the
 * existing statement.  This allows you to dynamcially change the
 * selection criteria at run-time.
 * <P>
 * If this is a child level, you must preserve the number and order
 * of the join parameters (i.e. the order in which they were added
 * by calls to the joinOnParentColumn method).
 * <P>
 * For example, if your original query was this:
 * <pre><code>
 *  select * from OrderDetails where OrderID = ?"
 * </code></pre>
 * calling setWhereClause("where OrderID = ? and ProductID = 'AMPB5'") results
 * in this new query:
 * <pre><code>
 *  select * from OrderDetails where OrderID = ? and ProductID = 'AMPB5'
 * </code></pre>
 * @param whereClause a String which is the new where clause
 * @exception DataModelException If the set fails.
 * @see #joinOnParentColumn
 */
public void setWhereClause(String whereClause) throws DataModelException {
    String newQuery = parser.getClause(parser.SELECT, true) + " ";
    newQuery += parser.getClause(parser.FROM, true) + " ";
    setStatement(newQuery + whereClause);
}

/**
 * Return the current "where" clause used for getting data.
 * This is a convenience method.  It calls
 * getParser().getClause(SqlParser.WHERE).
 * @return a String, the current where clause
 * @see SqlParser
 */
public String getWhereClause() {
    return parser == null ? "" : parser.getClause(parser.WHERE);
}

/**
 * Return the current query statement.
 * @return a String, the current query statement for getting data
 * @see #setStatement
 * @see #setWhereClause
 */
public String getStatement() {
    return query;
}

/**
 * Adds a key column which will be used to join to this control's parent.
 * For composite keys (that is, tables are joined on more than one column),
 * call this method once for each key column.
 * @param parentColumnlIdentifier a String indicating the parent column whose
 * values will be used in WHERE clause to retrieve the child level; if the
 * column is aliased, used the alias, otherwise use the name as it appears in
 * the select statement ([catalog.][schema.][table.]column.)
 * @param childColumn a String indicating to which child column the
 * parent column should match, required because parent column names can be
 * different than child column names; if the column is aliased, used the alias,
 * otherwise use the name as it appears in the select statement
 * ([catalog.][schema.][table.]column.)
 */
public void joinOnParentColumn(String parentColumn, String childColumn) {
    join_columns.addElement(new ColumnMap(parentColumn, childColumn));
}

/**
 * Sets a particular parameter in the sql statement.
 * Statement can be a SQL statement or a stored procedure.
 * Sets the parameters by their position in the original statement.
 * @param number an int indicating the position of the parameter,
 * starts at 1.
 * @param value the Object which is to be set
 * @param columnType an int one of java.sql.Types
 * @exception DataModelException If setParameter fails.
 */
public void setParameter(int number, Object value, int type) throws DataModelException {
	try {
		if (value == null) {
	            query_statement.setNull(number, type);
	            // see the java bug parade at
	            //      http://developer.java.sun.com/developer/bugParade/bugs/4038518.html
	            // where this bug is explained. Basically there are conversion
	            // problems between odbc2 which uses 9, 10 and 11 for date,
	            // time and timestamp, whereas odbc3 uses 91, 92, 93.
	            // So on exception we'll do the conversion ourselves.
	            switch (type) {
	            case java.sql.Types.DATE:
	                query_statement.setNull(number, 9); break;
	            case java.sql.Types.TIME:
	                query_statement.setNull(number, 10); break;
	            case java.sql.Types.TIMESTAMP:
	                query_statement.setNull(number, 11); break;
	            default:
	                break;
	            }
	    }
	    else {
	        query_statement.setObject(number, value);
	    }
	}
	catch (java.sql.SQLException e) {
		throw new DataModelException(DataModelEvent.INITIALIZE, e);
	}
}

/**
 * Convenience method: returns the tables associated with this level.
 * There can be one or many tables (related by one-to-one
 * relationships) for a level.
 * @return tables a Vector of tables for this level
 */
public Vector getTables() {
    if (tables.size() != 0) {
        return tables;
    }
    for (int col = 0; col < numColumns; col++) {
        String table = ((ColumnModel) getColumnObjects().elementAt(col)).getTableName();
        if (table != null && !table.equals("") && !tables.contains(table)) {
            tables.addElement(table);
        }
    }
    return tables;
}

/**
 * Convenience method: returns this table's column indexes.
 * Through calls to ResultSetMetaData.getTableName() or by
 * explicitly calling setColumnTableRelationships() each Column
 * object in the columns array has its table name property set.
 * This method returns the ordinal positions of this table's columns
 * in the row.
 * @param table the String name of a table; if aliased, used the alias,
 * otherwise use the table name as it appears in the select statement
 * ([catalog.][schema.]table)
 * @return a Vector of Integers which are the ordinal positions of this
 * table's columns in the row.
 */
public Vector getColumnIndexesForTable(String table) {
    if (tableColumnIndexes.size() != 0) {
        return(Vector) tableColumnIndexes.elementAt(tables.indexOf(table));
    }

    // otherwise build a Vector of Vectors of columns for each table
    for (Enumeration e = tables.elements(); e.hasMoreElements();) {
        String db_table = (String) e.nextElement();
        Vector v = new Vector();
        tableColumnIndexes.addElement(v);
        for (int col = 0; col < numColumns; col++) {
            String t = ((ColumnModel) getColumnObjects().elementAt(col)).getTableName();
            if (t != null && t.equals(db_table)) {
                v.addElement(new Integer(col));
            }
        }
    }

    return(Vector) tableColumnIndexes.elementAt(tables.indexOf(table));
}

/**
 * Internal method, passes parent row to
 * new DataTables.
 * @exception DataModelException If executeQuery fails.
 */
public DataTable executeQuery() throws DataModelException {
    if (!open) {
        open();
    }
    return executeQuery(com.klg.jclass.datasource.DataTableModel.ROOT_BOOKMARK);
}

/**
 * Execute the query.
 * Builds a cached version of the result set data.
 * @param parentRow a long indicating the parent of the returned
 * DataTable
 * @return a DataTable of the new results
 * @exception DataModelException If executeQuery fails
 */
public DataTable executeQuery(long parent_row) throws DataModelException {
    if (ds_connection.getConnection() == null) {
        Exception e = new Exception(LocaleBundle.string(LocaleBundle.null_query_or_connection));
		throw new DataModelException(DataModelEvent.INITIALIZE, e);
    }

    // the result table which will be returned
    DataTable result_table = null;

    // set the max results allowed to be returned
    try {
        query_statement.setMaxRows(JCEnvironment.isDesignTime() ? designTimeMaxRows : getMaxRows());
    }
    catch (java.sql.SQLException sql) {
        // set max not supported, continue anyway
    }

    // Execute the query which builds result set table.

	try {
		ResultSet result_set;
		try {
			result_set = query_statement.executeQuery();
		}
		catch (SQLException e) {
			// if the connection was closed and reopened
			// we need to reset the query, do so here and
			// try the query again, otherwise a real error
			// has occurred
			setStatement(query);
			result_set = query_statement.executeQuery();
		}

	    // create the empty table
	    result_table = new DataTable(numColumns,(MetaData)this, parent_row);

	    // Now populate that table.
	    int rows = 0;
	    while (result_set.next()) {
	        // add this new row to the result table
	        result_table.addInternalRow(createRowFromResultSet(result_set));
	    }

	    // free JDBC resources held by this result set
	    result_set.close();
    }
    catch (java.sql.SQLException sql) {
    	throw new DataModelException(DataModelEvent.INITIALIZE, sql);
    }


    // update a bound component if there is one
    return result_table;
}

/**
 * Adds the column to the internal mapping
 * @param column_name a String which is the column alias or the column name
 * as it appears in the select statement ([catalog.][schema.][table.]column)
 * @see com.klg.jclass.datasource.BaseMetaData#addColumn
 * @exception DataModelException If the mapping fails.
 */
public void addColumnToIdentifiers(String column_name, int position) throws DataModelException {
    if (getColumnMap().containsKey(column_name)) {
        Exception e = new Exception(LocaleBundle.string(LocaleBundle.duplicate_column_found) + column_name);
    	throw new DataModelException(DataModelEvent.INITIALIZE, e);
    }
    // add the column to ident/idx mapping
    getColumnMap().put(column_name, new Integer(position));
}

/**
 * Passed in a row from a result set, returns a row Object[].
 * @param resultSet the ResultSet pointing to the next row to be
 * used to create a row which is an Object[]
 * @return the new row Object[]
 * @exception DataModelException If create row fails.
 */
Object[] createRowFromResultSet(ResultSet result_set) throws DataModelException {
    // create a new row with appropriate number of columns
    Object row[] = new Object[numColumns];
    try {
        // loop on columns, they start a 1 for JDBC
        for (int col = 1; col < numColumns + 1; col++) {
            Object object = null;
            ColumnModel cm = (ColumnModel) getColumnObjects().elementAt(col-1);
            if (cm.getMetaColumnType() == MetaDataModel.TYPE_BYTE_ARRAY) {
                object = createByteArrayFromBinaryStream(result_set.getBinaryStream(col));
            }
            else {
                object = result_set.getObject(col);
            }
            row[col-1] = object;
        }
    }
    catch (SQLException e) {
        throw new DataModelException(DataModelEvent.INITIALIZE, e);
    }
    return row;
}

/**
 * Creates column info by copying it from ResultSetMetaData.
 * @param metaData the ResultSetMetaData for this level
 * @exception DataModelException If the column cannot be created.
 */
void createColumnInfo(ResultSetMetaData rsmd) throws DataModelException {
	try {
		numColumns = rsmd.getColumnCount();
	}
	catch (SQLException e) {
		throw new DataModelException(DataModelEvent.INITIALIZE, e);
	}

    for (int i = 1; i < numColumns + 1; i++) {
        ColumnModel column = new Column(rsmd, i);
        // don't call addColumn here since it both adds the column and
        // sets the identifier which has not been determined yet
        getColumnObjects().addElement(column);
    }
}

public void clear() {
}

/**
 * Sets the primary keys for a table.
 * If both JDBC methods getPrimaryKeys() and getBestRowIdentifier()
 * fail to determine the primary keys for a table, you must
 * explicitly set them by calling this method. An exception will
 * be thrown when the primary keys cannot be determined automatically.
 * @param table a String which is the table alias or table name as it
 * appears in the select statement (([catalog.][schema.]table)
 * @param columns an array of column aliases or names as they appear in the
 * select statement
 */
public void setPrimaryKeys(String table, String columnNames[]) {
    // copy the array into a Vector
    Vector v = new Vector(columnNames.length);
    for (int i = 0; i < columnNames.length; i++) {
        v.addElement(columnNames[i]);
    }
    tableToPrimaryKeyMap.put(table,v);
}

/**
 * Returns a vector of primary keys for this data control.
 * @param table a String representing the fully qualified
 * table name, that is, schema.owner.name.
 * @return a Vector of column names which are the keys in the
 * form <schema>.<table>.<column>
 * @exception DataModelException If a primary key cannot be found.
 */
Vector getPrimaryKeys(String table) throws DataModelException {
    if (table == null) {
        return new Vector(0);
    }
    // see if we already have the keys for this table
    if (tableToPrimaryKeyMap.containsKey(table)) {
        return(Vector) tableToPrimaryKeyMap.get(table);
    }

    // otherwise retrieve them
    Vector keys = new Vector();
    try {
        DatabaseMetaData md = ds_connection.getConnection().getMetaData();
        String schema = null;
        if (table.indexOf(".") > NOT_FOUND) {
            int idx = table.indexOf(".");
            schema = table.substring(0,idx);
            table = table.substring(idx + 1,table.length());
        }
        ResultSet rs = null;
        try {
            rs = md.getPrimaryKeys(null,null,table);
        }
        catch (SQLException e) {
            // method getPrimaryKeys not supported so try getBestRowIdentifier
            rs = md.getBestRowIdentifier(null, null, table, DatabaseMetaData.bestRowSession, true);
        }
        while (rs.next()) {
            String key = schema + "." + table + "." + rs.getString("COLUMN_NAME");
            keys.addElement(key);
        }
        if (keys.size() == 0) {
            // if getPrimaryKeys was supported, but retunred nothing, try getBestRowIdentifier
            rs = md.getBestRowIdentifier(null, null, table, DatabaseMetaData.bestRowSession, true);
            while (rs.next()) {
                String key = schema + "." + table + "." + rs.getString("COLUMN_NAME");
                keys.addElement(key);
            }
        }
        if (keys.size() == 0) {
            String message = LocaleBundle.string(LocaleBundle.no_primary_key);
            message += table;
            throw new DataModelException(DataModelEvent.INITIALIZE, new Exception(message));
        }
    }
    catch (SQLException e) {
		throw new DataModelException(DataModelEvent.INITIALIZE, e);
    }
    tableToPrimaryKeyMap.put(table, keys);
    return keys;
}

/**
 * Called from the exception case in getPrimaryKey.
 * Used when the JDBC call getPrimaryKeys fails.
 * Keys are determined by indexes or by db specific
 * methods for flagging columns as keys.
 * @param table The String name of the table for which primary keys.
 * are to be determined
 * @return keys A Vector of column(s) which comprise the primary key.
 */
private Vector getBestIdentifier(String table) {
    Vector keys = new Vector();

    // otherwise retrieve them
    try {
        DatabaseMetaData md = ds_connection.getConnection().getMetaData();
        String schema = null;
        if (table.indexOf(".") > NOT_FOUND) {
            int idx = table.indexOf(".");
            schema = table.substring(0,idx);
            table = table.substring(idx + 1,table.length());
        }
        ResultSet rs = md.getBestRowIdentifier(null, null, table,
                                               DatabaseMetaData.bestRowSession, true);
        while (rs.next()) {
            String key = schema + "." + table + "." +
                         rs.getString("COLUMN_NAME");
            keys.addElement(key);
        }
    }
    catch (SQLException e) {
        e.printStackTrace();
    }
    return keys;
}

/**
 * Initializes by causing the ResultSetMetaData to be loaded.
 * Does nothing if already open.
 * Performs checks to see that the number of join_columns equals
 * the number of question mark place holders.  Also checks that
 * for unique column names.
 * @exception DataModelException If the operation fails.
 */
public DataTable open() throws DataModelException {
    if (isOpen()) {
        return null;
    }
    // ensure the statement exists
    if (query_statement == null) {
        Exception e = new Exception(LocaleBundle.string(LocaleBundle.statement_not_set) + getDescription());
        throw new DataModelException(DataModelEvent.INITIALIZE, e);
    }
    try {
        if (!parameterCountsMatch()) {
            String message = LocaleBundle.string(LocaleBundle.parameter_count_mismatch);
            message += getDescription();
          	throw new DataModelException(DataModelEvent.INITIALIZE, new Exception(message));
        }
        MetaDataModel md = (MetaDataModel) getParent();
        int cnt = 1;
        // Create a bogus query. This get the ResultSetMetaData
        for (Enumeration e = join_columns.elements(); e.hasMoreElements();) {
            ColumnMap cm = (ColumnMap) e.nextElement();
            ColumnModel model = (ColumnModel) md.getColumnObject(cm.getParentColumn());
            int type = model.getColumnType();
    		Object object = new Object();
    		String dummyValue = "-1";
    		switch (type) {
    			case Types.CHAR:
    			case Types.VARCHAR:
    			case Types.LONGVARCHAR:	object = new String(dummyValue); 	break;
    			case Types.NUMERIC: 	object = new BigDecimal(dummyValue);break;
    			case Types.BIT: 		object = new Boolean(false); 		break;
    			case Types.TINYINT: 	object = new Byte(dummyValue); 		break;
    			case Types.SMALLINT: 	object = new Short(dummyValue);		break;
    			case Types.INTEGER:		object = new Integer(dummyValue);	break;
    			case Types.BIGINT:		object = new BigInteger(dummyValue);break;
    			case Types.REAL:		object = new Float(dummyValue);		break;
    			case Types.DOUBLE:		object = new Double(dummyValue);	break;
    			case Types.BINARY:
    			case Types.VARBINARY:
    			case Types.LONGVARBINARY:object = new Byte(dummyValue);		break;
    			case Types.DATE:		object = new Date(System.currentTimeMillis()); break;
    			case Types.TIME:		object = new Time(System.currentTimeMillis()); break;
    			case Types.TIMESTAMP:	object = new Timestamp(System.currentTimeMillis()); break;
    			default:
    				break;
    		}
    		query_statement.setObject(cnt++, object, type);
        }
        open = true;

		// The bug: if the max rows are set, the driver gets into a deadlock state.
		// This bug is happening in their 4.0/4.1 and 5.0 drivers. March 26, 1999.
		DatabaseMetaData dbmd	= ds_connection.getConnection().getMetaData();
		boolean isJconnect 		= false;
		if (dbmd != null) {
			String driverName = dbmd.getDriverName();
			if (driverName != null) {
				isJconnect = driverName.toLowerCase().indexOf("jconnect") > -1;
			}
		}

        // Execute the query so we can get the result set meta data,
        // set the number of rows to 1 so we don't waste time on returning
        // rows that're not used.
        // set the max results allowed to be returned
		if (!isJconnect) {
			try {
				query_statement.setMaxRows(1);
			} catch (java.sql.SQLException sql) {
				// set max not supported, continue anyway
			}
		}


        // do the query to get the meta data
        ResultSet result_set = query_statement.executeQuery();

        // create the column objects
        if (getColumnObjects().size() == 0) {
            // Get the meta data for the result set.
            // Only need to get it once since all queries
            // to this data control are expected to be the
            // same but with different parameters.
            ResultSetMetaData rsmd = result_set.getMetaData();
            createColumnInfo(rsmd);
            if (!columnTableRelationsSet()) {
                queryColumnTableRelations();
            }

            for (int i = 0; i < numColumns; i++) {
                Column column = (Column) getColumnObjects().elementAt(i);
                // figure out what the identifier should be
                column.determineIdentifier(parser);
                addColumnToIdentifiers(column.getIdentifier(), i);
            }
        }
        // throw away the results and close
        while (result_set.next()) {
            // eat
        }
        result_set.close();

        // reset to no limit
		if (!isJconnect) {
			try {
				query_statement.setMaxRows(0);
			} catch (java.sql.SQLException sql) {
				// set max not supported, continue anyway
			}
		}

        // return a dummy table for backward compatibility
        return new DataTable(columns.size(), this);

    }
    catch (SQLException e) {
        open = false;
        throw new DataModelException(DataModelEvent.INITIALIZE, e);
    }
}

/**
 * Sees if Column objects have their table properties set.
 */
private boolean columnTableRelationsSet() {
    for (int col = 0; col < numColumns; col++) {
        String t = ((ColumnModel) getColumnObjects().elementAt(col)).getTableName();
        if (t != null && !t.equals("")) {
            return true;
        }
    }
    return false;
}

/**
 * If ResultSetMetaData.getTableName() calls were not supported by
 * the driver, i.e., the table property in Column is null, try to
 * figure it out again, this time using getColumns(table).
 */
private void queryColumnTableRelations() {
    SqlParser sp = new SqlParser(query);
    try {
        DatabaseMetaData d = ds_connection.getConnection().getMetaData();
        // for each table ...
        int number_of_tables = sp.getTables().size();
        for (Enumeration e = sp.getTables().elements(); e.hasMoreElements();) {
            SqlStatementTable table = (SqlStatementTable) e.nextElement();
            ResultSet rs = d.getColumns(table.getCatalogName(),table.getSchemaName(),table.getTableName(), null);
            // for each result ...
            while (rs.next()) {
                String column   = rs.getString("COLUMN_NAME");
                int type        = rs.getInt("DATA_TYPE");
                String type_name= rs.getString("TYPE_NAME");
                // for each column ...
                for (Enumeration i = getColumnObjects().elements(); i.hasMoreElements();) {
                    ColumnModel c = (ColumnModel) i.nextElement();
                    if (c.getColumnName().equals(column) &&
                        (number_of_tables == 1 || columnBelongsToTable(table, c, column, type, type_name))) {
                        c.setTableName(table.getTableName());
                        break;
                    }
                }
            }
            rs.close();
        }
    }
    catch (SQLException e) {
        System.out.println(e.toString());
    }
}

/** Test conditions to see if a column belongs to a table. */
private boolean columnBelongsToTable(SqlStatementTable table, ColumnModel c, String column, int type, String type_name) {
    // FIX: we match on type and name, but really there could be
    // duplicate columns with same name and type, but other attributes
    // could be different.
    boolean found = false;
    Enumeration e = parser.getColumnsForTable(table.getAlias()).elements();
    SqlStatementColumn ssc = null;
    while (e.hasMoreElements()) {
        ssc = (SqlStatementColumn) e.nextElement();
        if (ssc.getTableName() == null || ssc.getColumnName().equals(column) || ssc.getColumnName().equals("*")) {
            found = true;
            break;
        }
    }

    // For some reason using msaccess, calls to ResultSetMetaData.getColumnType()
    // and DatabaseMetaData.getColumnType for the same column return
    // different results in the case of a DATETIME column (11 != 93)
    // thus I am adding a check on type name as well if type fails.
    return found &&
    (c.getColumnName().equals(column) || (ssc != null && ssc.getColumnName().equals(column))) &&
    (c.getColumnType() == type || c.getColumnTypeName().equals(type_name)) &&
    (c.getTableName() == null || c.getTableName().equals(""));
}

/**
 * Returns true if the number of join_columns and the number of question
 * mark placeholders are equal.
 */
private boolean parameterCountsMatch() {
    return join_columns.size() == countQuestionMarks();
}

/**
 * Returns the number of question mark placeholders in a query statement.
 */
private int countQuestionMarks() {
    int count = 0, idx = 0;
    while (query.substring(idx,query.length()).indexOf("?") > NOT_FOUND) {
        idx += query.substring(idx, query.length()).indexOf("?");
        idx++; // step one space past the ?
        count++;
    }
    return count;
}

/**
 * Explicitly sets the relationships between tables and columns.
 * If introspection fails to determine the association between
 * tables and their columns (when there is more than one table to
 * a level), or you wish to override the assocations, say to exculde
 * a column in the update statement, use this method.
 * This method must be called for each table.
 * For example, if SalesOrder and Store are joined in a one-to-one
 * relationship for a level, these would be the necessary calls.  In this
 * example the MetaData object is called Orders.
 *   Orders.setColumnTableRelations("SalesOrder", new String[] {"id","store_id","cust_id","ship_via","purchase_order_number","order_date","order_total"});
 *   Orders.setColumnTableRelations("Store", new String[] {"store_store_id","address","phone_number","name"});
 * Or for a single table on a level the call would be,
 *   Customer.setColumnTableRelations("Customer", new String[] {"*"});
 * The "*" means all columns are from the Customer table.
 * @param table a String, the name to which (a subset of the columns belong);
 * use the table name as it appears in the FROM clause; don't use alias
 * @param columns an array of columns belonging to this table; don't use aliases
 * @exception DataModelException If the operation fails.
 */
public void setColumnTableRelations(String table, String columnNames[])
throws DataModelException {
    if (!isOpen()) {
        throw new DataModelException(DataModelEvent.INITIALIZE, createNotOpenException("setColumnTableRelations()"));
    }
    // we already have an array of column objects which have a
    // table name property.  This is normally set by getTableName, but
    // if this does not work we have to set it here manually.
    // Thus, we loop on the columns the user has associated with
    // a table and set the table name for those columns.
    if (((String) columnNames[0]).equals("*")) {
        // all columns in the row belong to "table"
        for (int i = 0; i < getColumnCount(); i++) {
            ((ColumnModel) getColumnObjects().elementAt(i)).setTableName(table);
        }
    }
    else {
        // allow user to override (and exclude columns by setting)
        // the table name to "", and then only setting the associations
        // between those columns the user wants set
        for (int j = 0; j < getColumnObjects().size(); j++) {
            ColumnModel c = (ColumnModel) getColumnObjects().elementAt(j);
            if (c.getTableName() != null && c.getTableName().equals(table)) {
                c.setTableName("");
            }
        }
        // a subset of columns belongs to "table", find and set them
        for (int i = 0; i < columnNames.length; i++) {
            for (int j = 0; j < getColumnObjects().size(); j++) {
                ColumnModel c = (ColumnModel) getColumnObjects().elementAt(j);
                String column_name = columnNames[i];
                if (c.getColumnName().equals(columnNames[i]) || c.getIdentifier().equals(columnNames[i])) {
                    c.setTableName(table);
                    break;
                }
            }
        }
    }
}

/**
 * Returns true if this MetaData object has been initialized.
 * @return true if this MetaData object has been opened,
 * false otherwise.
 * @see #open
 */
public boolean isOpen() {
    return open;
}

/**
 * Creates the exception to be thrown when this instance has
 * not been opened, yet a get method has been called.
 */
private Exception createNotOpenException(String method) {
    String message = LocaleBundle.string(LocaleBundle.meta_data_not_open);
    message += getDescription();
    message += " " + method;
    return new Exception(message);
}

/**
 * Sets the name of the table whose primary key will be used
 * to requery a row when more than one table contributes columns
 * to a row.  The default is to use the first table in the list of tables.
 * The driver table is the table whose primary key is used to
 * requery the row, the other tables join to this table.
 * @param driverTable a String, the table alias or name as it appears in the
 * select statement ([catalog.][schema.]table)
 */
public void setDriverTable(String driverTable) {
    this.driverTable = driverTable;
}

/**
 * Returns the name of the table used to drive requeries when more than
 * one table contributes columns to a row.
 * The driver table is the table whose primary key is used to
 * requery the row, the other tables join to this table.
 * @return a String which is the driver table name.
 */
public String getDriverTable() {
    if (driverTable == null) {
        driverTable = (String) getTables().elementAt(0);
    }
    return driverTable;
}

/**
 * Returns the index of the driver table in the list of tables.
 * Default is to use the first table.
 * The driver table is the table whose primary key is used to
 * requery the row, the other tables join to this table.
 * @return an int which is the index of the driver table in the
 * tables list.
 */
public int getDriverTableIndex() {
    if (getDriverTable() == null) {
        return 0;
    }
    return getTables().indexOf(driverTable);
}

/**
 * Sets the auto commit flag.
 * If the underlying data source supports transactions and this
 * flag is set to false, a transaction will wrap data modifications.
 * The default is to not attempt to wrap modifications in a transaction.
 * @param bool a boolean, false if transactions are to be used,
 * true otherwise.
 * @exception DataModelException If the operation fails.
 */
public void setAutoCommit(boolean value) throws DataModelException {
    try {
        ds_connection.setAutoCommit(value);
    }
    catch (SQLException e) {
        throw new DataModelException(DataModelEvent.INITIALIZE, e);
    }
}

/**
 * Default implementation: returns true.
 * Must be overridden in specific implementations if transactions
 * are supported and enabled.
 * Return true if the transaction capabilities of the underlying
 * data source are to be used when saving modifications
 * (transactions are supported and enabled).
 * If the underlying data source supports transactions and this
 * capability has been enabled, data modifications will be
 * wrapped in a transaction which can be committed or rolled back.
 * @exception DataModelException If the operation fails.
 * @see #commitTransaction
 * @see #rollbackTransaction
 */
public boolean getAutoCommit() throws DataModelException {
    try {
        return ds_connection.getAutoCommit();
    }
    catch (SQLException e) {
        throw new DataModelException(DataModelEvent.INITIALIZE, e);
    }
}

/**
 * Recursivly rolls back the transaction.
 * If the underlying data source supports transactions and
 * transactions have been enabled, calling this method will
 * roll back all pending transactions on the data source,
 * otherwise it does nothing.
 * @exception DataModelException If rollback fails.
 * @see MetaDataModel#getAutoCommit
 */
public void rollbackTransaction() throws DataModelException {
	try {
	    if (!getAutoCommit()) {
	        getDataTableConnection().getConnection().rollback();
	    }
	    for (Enumeration e = children(); e.hasMoreElements();) {
	        ((MetaDataModel) e.nextElement()).rollbackTransaction();
	    }
    }
    catch (SQLException e) {
        throw new DataModelException(DataModelEvent.INITIALIZE, e);
    }

}

/**
 * Recursivly Commits the transaction.
 * If the underlying data source supports transactions and
 * transactions have been enabled, calling this method will
 * commit all pending transactions on the data source,
 * otherwise it does nothing.
 * @exception DataModelException If commit fails.
 * @see MetaDataModel#getAutoCommit
 */
public void commitTransaction() throws DataModelException {
	try {
		if (!getAutoCommit()) {
	        getDataTableConnection().getConnection().commit();
	    }
	    for (Enumeration e = children(); e.hasMoreElements();) {
	        ((MetaDataModel) e.nextElement()).commitTransaction();
	    }
    }
    catch (SQLException e) {
        throw new DataModelException(DataModelEvent.INITIALIZE, e);
    }

}

/**
 * Returns true if an update is allowed on this column.
 * @param column_identifier a String which uniquely identifies a column
 * @return true if updates are allowed (by the data model) on this column
 * @see BaseMetaData#getUpdateAllowed
 * @see BaseMetaData#setUpdateAllowed
 * @exception DataModelException If primary keys not found.
 */
public boolean isUpdateAllowedOnColumn(String column) throws DataModelException {
    for (Enumeration e = getColumnObjects().elements(); e.hasMoreElements();) {
        ColumnModel c = (ColumnModel) e.nextElement();
        if (c.getIdentifier().equals(column)) {
            String table = c.getTableName();
            try {
                getPrimaryKeys(table); // this throws an exception if there are none
            }
            catch (DataModelException x) {
                throw new DataModelException(DataModelEvent.BEFORE_EDIT_CELL, x);
            }
            if (table == null || table.equals("")) {
                return false;
            }
            SqlStatementTable sql_table = parser.getSqlStatementTable(table);
            if (sql_table == null || sql_table.equals("")) {
                return false;
            }
            return getUpdateAllowed(sql_table.getAlias());
        }
    }
    return true;
}

/**
 * Returns the parser of the sql query statement.  From this you can get the
 * SELECT, FROM, and WHERE clauses of the statement.
 * @return the parser of the current sql query statement
 * @see SqlParser
 */
public SqlParser getParser() {
    return parser;
}

/**
 * Adds a column to the list of column objects.
 * Overrides BaseMetaData method of same name.
 * @param columnObject ColumnModel to be added.
 * @see com.klg.jclass.datasource.BaseMetaData#addColumn
 * @exception DataModelException If column add fails.
 */
public void addColumn(ColumnModel column) throws DataModelException {
    if (!isOpen()) {
        throw new DataModelException(DataModelEvent.INITIALIZE, createNotOpenException("addColumn()"));
    }
    try {
        addColumnToIdentifiers(column.getIdentifier(), getColumnMap().size());
    }
    catch (DataModelException e) {
        throw new DataModelException(DataModelEvent.INITIALIZE, e);
    }
    super.addColumn(column);
}

/**
 * Return the native meta data object.
 * This is a DataSet in JBuilder and a RelationView
 * in VisualCafe.
 * @return this.
 */
public Object getNativeMetaData() {
    return this;
}

/**
 * Sets the class which is to be used for saving data modifications
 * back to the originating data base.  This class must implement the
 * PersistorModel interface.  The default behavior is to use the
 * com.klg.jclass.datasource.jdbc.Persistor class.
 * @param className the name of the class on which a
 * Class.forName(className).newInstance will be done
 * @exception DataModelException If loading the class fails.
 */
public void setPersistor(String className) throws DataModelException {
    try {
        persistor = (PersistorModel) Class.forName(className).newInstance();
    }
    catch (ClassNotFoundException e) {
        throw new DataModelException(DataModelEvent.INITIALIZE, e);
    }
	catch (IllegalAccessException e) {
        throw new DataModelException(DataModelEvent.INITIALIZE, e);
    }
	catch (InstantiationException e) {
        throw new DataModelException(DataModelEvent.INITIALIZE, e);
    }

}

/**
 * Return the PersistorModel instance which is to be used to
 * save changes to the data base.
 * @return PersistorModel
 * @see #setPersistor
 */
public PersistorModel getPersistor() {
    return persistor;
}

/**
 *  Sets the maximum number of rows returned from a database
 *	at design-time.  It is used to improve the performance
 *	at design-time.  It has no effect at run-time. (default: 10)
 *	@see java.beans.Beans#isDesignTime
 */
public static void setDesignTimeMaxRows(int max) {
    designTimeMaxRows = max;
}

/**
 *  Gets the maximum number of rows returned from a database
 *	at design-time.
 *	@see #setDesignTimeMaxRows
 */
public static int getDesignTimeMaxRows() {
    return designTimeMaxRows;
}

/**
 * Return the list of join columns.

 */
public Vector getJoinColumns() {
	return join_columns;
}

// creates the default persistor on instantiation.  A call to
// setPersistor replaces this
private void createDefaultPersistor() {
    try {
        setPersistor("com.klg.jclass.datasource.jdbc.Persistor");
    }
    catch (DataModelException e) {
        System.out.println(e.toString());
    }
}

/**
 * Free any lingering jdbc resources.
 */
protected void finalize() {
	if (query_statement != null) {
		try {
			query_statement.close();
		}
		catch (SQLException e) {}
	}
}

}
