/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: MetaDataCreator.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:12 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.jdbc;

import java.awt.Frame;
import java.beans.Beans;
import java.io.Serializable;
import java.util.*;

import com.klg.jclass.datasource.*;
import com.klg.jclass.datasource.beans.*;
import com.klg.jclass.util.JCEnvironment;

/** This class creates a MetaData for JDBC data source type.
 *	@see com.klg.jclass.datasource.beans.NodeProperties
 */
public class MetaDataCreator extends com.klg.jclass.datasource.beans.MetaDataCreator {
	
static final long serialVersionUID = 547088802963888565L;

private static final String sunJdbcOdbcDriver = "sun.jdbc.odbc.JdbcOdbcDriver";
private static final String msJdbcOdbcDriver = "com.ms.jdbc.odbc.JdbcOdbcDriver";

private static Vector dataTableConnections = new Vector();

private final static boolean TRACE = false;

/** Helper method to create a JDBC meta data.
 * @exception DataModelException If the operation failsto create a meta data.
 */
protected MetaDataModel create(DataModel data_model, 
                               MetaDataModel meta_parent, NodeProperties comp) throws DataModelException {
    MetaData jdbc_meta = null;

    DataTableConnection c = MetaDataCreator.getJdbcConnection(meta_parent, comp);
    // Create the meta data
    if (meta_parent == null) {
        // Single level or the root of a tree
        if (comp.getStatement() != null && comp.getStatement().trim().length() > 0) {
            jdbc_meta = new MetaData(data_model, c, comp.getStatement().trim());
        }
        else {
            jdbc_meta = new MetaData(data_model, c);
        }
    }
    else {
        // tree
        jdbc_meta = new MetaData(data_model, (com.klg.jclass.datasource.jdbc.MetaData) meta_parent, c);
        if (jdbc_meta != null) {
            // Setup the meta data
            if (comp.getStatement() != null && comp.getStatement().trim().length() > 0) {
                jdbc_meta.setStatement(comp.getStatement().trim());
            }

            jdbc_meta.setDriverTable(comp.getDriverTable());
            for (int i=0; i<comp.getJoins().length; i++) {
                jdbc_meta.joinOnParentColumn(comp.getJoins()[i][0], comp.getJoins()[i][1]);
            }
            if (!JCEnvironment.isDesignTime() || 
                (comp.getStatement() != null && comp.getStatement().trim().length() > 0)) {
                // open meta data if we are not at design time.  Otherwise, don't open it
                // If statement is empty, don't bother opening meta data to avoid exception.
                jdbc_meta.open();
            }
        }
    }
    if (jdbc_meta != null) {
        if (comp.getDriverTable() != null && comp.getDriverTablePrimaryKeys() != null) {
            String s[] = new String[comp.getDriverTablePrimaryKeys().size()];
            comp.getDriverTablePrimaryKeys().copyInto(s);
            jdbc_meta.setPrimaryKeys(comp.getDriverTable(), s);
        }
    }

    return jdbc_meta;
}


/** Helper method to get a Jdbc connection.  At design-time, a static list
 *	keeps track of all the connections.  Thus, connections can be recycled.
 *      @exception DataModelException If the connection can't be made.
 */
public static DataTableConnection getJdbcConnection(MetaDataModel meta_parent, 
                                                    NodeProperties comp) throws DataModelException {
    // Get connection
    DataTableConnection c = null;
    boolean connected = false;
    if (meta_parent != null && comp.getUseParentConnection()) {
        // Use parent connection
        c = ((com.klg.jclass.datasource.jdbc.MetaData)meta_parent).getDataTableConnection();
        return c;
    }

    // Establish its own connection

    if (JCEnvironment.isDesignTime()) {
        // Re-cycle connections at design-time.
        // Otherwise, a new connection is made for every change.
        ConnectionInfo comp_ci = new ConnectionInfo(comp);
        for (Enumeration e=dataTableConnections.elements(); e.hasMoreElements();) {
            ConnectionInfo ci = (ConnectionInfo) e.nextElement();
            if (ci.equals(comp_ci)) {
                return ci.getConnection();
            }
        }
        if (c == null) {
            c = new DataTableConnection();
            comp_ci.setConnection(c);
            dataTableConnections.addElement(comp_ci);
        }
        else {
            connected = true;
        }
    }
    else {
        // Always create a new connection at run-time.
        c = new DataTableConnection();
    }
    if (!JCEnvironment.isDesignTime() && comp.getPromptUserForLogin()) {
        DataModelException dmex = null;
        Frame frame = new Frame();
        LoginDialog dlg = new LoginDialog(frame);
        dlg.setURL(comp.getURL());
        dlg.setDatabase(comp.getDatabase());
        dlg.setDriver(comp.getDriver());
        for (boolean done = false; !done;) {
            // prompt user for login until successfully connect to
            // the server, or user cancels.
            dlg.pack();
            dlg.show();
            if (dlg.isCancelled()) {
                done = true;
                if (dmex != null) {
                    throw dmex;
                }
            }
            else {
                try {
                    if (dlg.getDriver() != null && dlg.getDriver().trim().length() != 0) {
                        c.loadDriver(dlg.getDriver().trim());
                    }
                    c.connect(comp.getURL(), dlg.getUser(), 
                              dlg.getPassword(), dlg.getDatabase());
                    done = true;
                }
                catch (DataModelException ex) {
                    dmex = ex;
                    ErrorDialog.showError(null, ex.toString());
                }
            }
        }
        dlg.dispose();
    }
    else if (!connected && 
             (!JCEnvironment.isDesignTime() ||
              (comp.getURL() != null && comp.getURL().trim().length() != 0))) {
        if (comp.getDriver() != null && comp.getDriver().trim().length() != 0) {
            c.loadDriver(comp.getDriver().trim());
        }
        c.connect(comp.getURL(), comp.getUser(), 
                  comp.getPassword(), comp.getDatabase());    
    }
    return c;
}

/** Helper method to reset a Jdbc connection.
 */
public static void resetJdbcConnection(NodeProperties comp) {
    if (JCEnvironment.isDesignTime()) {
        ConnectionInfo comp_ci = new ConnectionInfo(comp);
        for (Enumeration e=dataTableConnections.elements(); e.hasMoreElements();) {
            ConnectionInfo ci = (ConnectionInfo) e.nextElement();
            if (ci.equals(comp_ci)) {
                dataTableConnections.removeElement(comp_ci);
            }
        }
    }
}

} // MetaDataCreator

/** Helper class to store connection information. It is used
 *	to recycle database connection at design-time. */
class ConnectionInfo implements Serializable {

String driver, url, user, password, database;
DataTableConnection connection;

ConnectionInfo(NodeProperties comp) {
    this(comp.getDriver(), comp.getURL(), 
         comp.getUser(), comp.getPassword(), comp.getDatabase());
}

ConnectionInfo(String driver, String url, String user, 
               String password, String database) {
    this.driver = driver;
    this.url = url;
    this.user = user;
    this.password = password;
    this.database = database;
}

public boolean equals(Object obj) {
    ConnectionInfo ci = (ConnectionInfo) obj;
    return 	isSameString(driver, ci.driver) &&
    isSameString(url, ci.url) &&
    isSameString(user, ci.user) &&
    isSameString(password, ci.password) &&
    isSameString(database, ci.database);
}

void setConnection(DataTableConnection connection) {
    this.connection = connection;
}

DataTableConnection getConnection() {
    return connection;
}

/** Returns if two strings are equal.  They are considered equal
 *	if both of them are null. 
 */
boolean isSameString(String s1, String s2) {
    return(s1 == null && s2 == null) ||
    (s1 != null && s2 != null && s1.equals(s2));
}

}
