/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ResultSetBinding.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:12 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.jdbc;

import com.klg.jclass.datasource.Binding;
import com.klg.jclass.datasource.DataModelException;
import com.klg.jclass.datasource.DataModelEvent;
import com.klg.jclass.datasource.DataModel;
import com.klg.jclass.datasource.DataTableModel;
import com.klg.jclass.datasource.TreeData;
import com.klg.jclass.datasource.DataSourceTreeModel;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

/**
 *
 * This class provides binding for a single level (non-hierarchical)
 * data source. Its data is provided by a java.sql.ResultSet.
 *
 */
public class ResultSetBinding extends Binding {
	
static final long serialVersionUID = 6356453874717375331L;
	
/**
 * 
 * Create a single-level binding from a java.sql.ResultSet.
 * This binding can then be used as a datasource for jclass
 * chart, table and/or field.
 * @param rs a java.sql.ResultSet
 * 
 */
public ResultSetBinding(ResultSet rs) throws DataModelException {
	super(new TreeData());

	// create the meta data
	metaDataModel = new MetaData(dataModel);
	metaDataModel.setBinding(this);
	
	// set the columns, types and table associations
	int numColumns = 0;
	try {
		ResultSetMetaData rsmd = rs.getMetaData();
		((MetaData)metaDataModel).createColumnInfo(rsmd);
	   	for (int i = 0; i < rsmd.getColumnCount(); i++) {
            Column column = (Column) metaDataModel.getColumnObjects().elementAt(i);
            ((MetaData)metaDataModel).addColumnToIdentifiers(column.getIdentifier(), i);
        }

		
		// create a data table from a result set
		DataTable dtm 	= new DataTable(numColumns, (MetaData) metaDataModel, DataTableModel.ROOT_BOOKMARK);
		
	    // Now populate that table.
	    int rows = 0;
	    while (rs.next()) {
	        // add this new row to the result table
	        dtm.addInternalRow(((MetaData)metaDataModel).createRowFromResultSet(rs));	    		   
	    }
	
		// set the root on the data table tree
		((DataSourceTreeModel)dataModel.getDataTableTree()).setRoot(dtm);

	}
	catch (SQLException e) {
		throw new DataModelException(DataModelEvent.INITIALIZE, e);
	}
}

/**
 * Overridden since updates are not possible using ResultSet.
 *
 * Return true if edits are permitted on this column.
 * @param	the column name.
 * @return	true if edits are permitted on this column, false otherwise.
 *
 */
public boolean isColumnEditable(String columnName) {
	return false;
}

/**
 *
 * Overridden since updates are not possible using ResultSet.
 *
 * Return true if edits are permitted on this column.
 * @param 	the column index
 * @return	true if edits are permitted on this column, false otherwise.
 *
 */
public boolean isColumnEditable(int columnIndex) {
	return false;
}

}
