/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DSdbJCheckbox.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:12 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.swing;

import java.awt.*;
import java.awt.event.*;

import javax.swing.JCheckBox;

import com.klg.jclass.datasource.BindingModel;
import com.klg.jclass.datasource.DataModelException;
import com.klg.jclass.datasource.DataModel;
import com.klg.jclass.datasource.MetaDataModel;

/**
 * This class binds a JCheckbox to a boolean column in the current row of the datasource.
 * Changing the state updates the record in the datasource but does not automatically
 * commit the change.
 */
public class DSdbJCheckbox extends JCheckBox implements FocusListener,
    ItemListener {

protected FieldDataBinding      fieldBinding;

/**
 * No-argument constructor.  Creates a check box with no label
 */
public DSdbJCheckbox() {
    this("");
}

/**
 * Creates a new DSdbJCheckbox component.
 */
public DSdbJCheckbox(String label) {
    super(label);
    fieldBinding = new FieldDataBinding();
    addFocusListener(this);
    addItemListener(this);
}

/**
 * Creates a new DSdbJCheckbox component and binds it to a particular column
 * in a MetaDataModel.
 * @see com.klg.jclass.datasource.beans.NodeProperties#setDataSource
 */
public DSdbJCheckbox(String label, DataModel dataModel, MetaDataModel metaDataModel, String columnName) {
    this(label);
    setDataBinding(dataModel, metaDataModel, columnName);
}

/**
 * Create and binding to a BindingModel.
 */
public DSdbJCheckbox(String label, BindingModel binding, String columnName) {
    this(label);
    setDataBinding(binding, columnName);
}

/**
 * Sets the parameters for binding the component to a particular column
 * in a MetaDataModel. If the MetaDataModel is the root in the DataModel's
 * MetaDataTree, the DataTableModel is automatically set to the root
 * DataTableModel, otherwise, the DataTableModel is set when an appropriate
 * table becomes available.
 */
public void setDataBinding(DataModel dataModel, MetaDataModel metaDataModel, String columnName) {
    fieldBinding.setDataBinding(metaDataModel.getBinding(), columnName);
}

/**
 * Sets the data source to a DataModel and binds the MetaData to the
 * MetaDataModel represented by a "path" of MetaDataModel descriptions
 * separated by '|' (e.g. Orders|Customers).
 */
public void setDataBinding(DataModel dataModel, String metaPath) {
    fieldBinding.setDataBinding(dataModel, metaPath);
}

/**
 * Sets the data source to the path of DataModel, MetaDataModel descriptions
 * and column name.
 */
public void setDataBinding(String dataBinding) {
    fieldBinding.setDataBinding(dataBinding);
}

/**
 * Bind to Binding and column.
 */
public void setDataBinding(BindingModel b, String columnName) {
    fieldBinding.setDataBinding(b, columnName);
}

/**
 * Returns the data binding hierarchy string, starting from a JCTreeData instance name
 * and containing full sequence of MetaDataModel levels starting from the root to the
 * level the table is bound with.
 */
public String getDataBinding() {
    return fieldBinding.getDataBinding();
}

/*
 * FocusListener Implementation.
 *****************************************************************************/

/**
 * Called when the text field gets focus.
 */
public void focusGained(FocusEvent event) {
}

// Semaphore used to prevent reentrance.
private boolean focusRequested = false;

/**
 * Commits any changes when the text field loses focus.
 */
public void focusLost(FocusEvent event) {
    if (focusRequested) {
        return;
    }

    // Critical section of code - must be non-reentrant!
    focusRequested = true;
    fieldBinding.saveItem(new Boolean(isSelected()));

    // End of critical section of code.
    focusRequested = false;
}

/*
 * ItemListener Implementation.
 *****************************************************************************/

/**
 * Called when the checkbox item changes.
 */
public void itemStateChanged(ItemEvent event) {
    fieldBinding.saveItem(new Boolean(isSelected()));
}

/**
 * Returns the preferred size of the component.
 */
public Dimension getPreferredSize() {
    Dimension d = super.getPreferredSize();
    return fieldBinding.isDataAvailable() ? d : new Dimension(70, d.height);
}

// END DSdbJCheckbox

/*
 * DataBinding Inner Class
 *****************************************************************************/

class FieldDataBinding extends com.klg.jclass.datasource.util.FieldDataBinding {

    public FieldDataBinding() {
        super(DSdbJCheckbox.this);
    }

    protected void refreshCell(Object item) {
        if (item instanceof Boolean && ((Boolean)item).booleanValue()) {
            DSdbJCheckbox.this.setSelected(true);
		}
		else if(item instanceof Number && ((Number)item).longValue() > 0) {
            DSdbJCheckbox.this.setSelected(true);
		}
        else {
            DSdbJCheckbox.this.setSelected(false);
		}

    }

    protected boolean saveCell() {
        saveItem(new Boolean(DSdbJCheckbox.this.isSelected()));
        return true;
    }

} // END DataBinding

}
