/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DSdbJComboBox.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:12 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.swing;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

import javax.swing.*;
import javax.swing.event.*;

import com.klg.jclass.cell.*;
import com.klg.jclass.datasource.BindingModel;
import com.klg.jclass.datasource.DataModel;
import com.klg.jclass.datasource.MetaDataModel;
import com.klg.jclass.datasource.DataModelException;
import com.klg.jclass.datasource.util.ExceptionManager;

/**
 * This class binds a JComboBox to a column in the datasource. The list selection
 * responds to row changes in the datasource and can initiate one.
 */
public class DSdbJComboBox extends JComboBox {

ColumnDataBinding columnBinding;
protected JCCellEditorSupport support = new JCCellEditorSupport();

/**
 * Creates a new DSdbJComboBox component.
 */
public DSdbJComboBox() {
//	setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    columnBinding = new ColumnDataBinding();
    addActionListener(this);
}

/**
 * Creates a new DSdbJComboBox component and binds it to a particular column
 * in a MetaDataModel.
 * @see com.klg.jclass.datasource.beans.NodeProperties#setDataSource
 */
public DSdbJComboBox(DataModel dataModel, MetaDataModel metaDataModel, String columnName) {
    this();
    setDataBinding(dataModel, metaDataModel, columnName);
}

/**
 * Create and binding to a Binding.
 */
public DSdbJComboBox(BindingModel binding, String columnName) {
    this();
    setDataBinding(binding, columnName);
}

/**
 * Sets the parameters for binding the component to a particular column
 * in a MetaDataModel. If the MetaDataModel is the root in the DataModel's
 * MetaDataTree, the DataTableModel is automatically set to the root
 * DataTableModel, otherwise, the DataTableModel is set when an appropriate
 * table becomes available.
 */
public void setDataBinding(DataModel dataModel, MetaDataModel metaDataModel, String columnName) {
    columnBinding.setDataBinding(metaDataModel.getBinding(), columnName);
}

/**
 * Sets the data source to a DataModel and binds the MetaData to the
 * MetaDataModel represented by a "path" of MetaDataModel descriptions
 * separated by '|' (e.g. Orders|Customers).
 */
public void setDataBinding(DataModel dataModel, String metaPath) {
    columnBinding.setDataBinding(dataModel, metaPath);
}

/**
 * Sets the data source to the path of DataModel, MetaDataModel descriptions
 * and column name.
 */
public void setDataBinding(String dataBinding) {
    columnBinding.setDataBinding(dataBinding);
}

/**
 * Bind to the specified Binding.
 * @param binding a Binding object.
 * @param columnName the column to which this component will be bound.
 */
public void setDataBinding(BindingModel b, String columnName) {
    columnBinding.setDataBinding(b, columnName);
}

/**
 * Returns the data binding hierarchy string, starting from a JCTreeData instance name
 * and containing full sequence of MetaDataModel levels starting from the root to the
 * level the table is bound with.
 */
public String getDataBinding() {
    return columnBinding.getDataBinding();
}

/*
 * ItemListener Implementation.
 *****************************************************************************/

/**
 * Called when an item's state has been changed.
 */
public void actionPerformed(ActionEvent event) {
	columnBinding.moveToRow(getSelectedIndex());
}

// END DSdbJComboBox

/*
 * DataBinding Inner Class
 *****************************************************************************/

class ColumnDataBinding extends com.klg.jclass.datasource.util.ColumnDataBinding {

    public ColumnDataBinding() {
        super(DSdbJComboBox.this);
    }

    protected void moveToRow(int row) {
        moveToRowIndex(row);
    }

/**
 * Process a change in the data
 */
    protected void tableChanged() {
        readCurrentData();
    }

/**
 * Process a change in the data in the row
 */
    protected void rowDataChanged(int rowIndex) {
        cellDataChanged(rowIndex, column_name);
    }

/**
 * Process a single cell value change.
 */
    protected void cellDataChanged(int rowIndex, String columnName) {
        if (!(columnName.equals(column_name)))
            return;

        readCurrentData();
    }

/**
 * Called when the current row changes.
 */
    protected void currentRowChanged(int rowIndex) {
    	int newRowIndex = binding.getCurrentRowIndex();
        if (getSelectedIndex() != newRowIndex)
            setSelectedIndex(newRowIndex);
    }

/**
 * Checks if all listeners have accepted the move.
 */
    protected boolean beforeCurrentRowChange() {
        return true;
    }

/**
 * Must be called after a row was inserted.
 */
    protected void rowInserted() {
        readCurrentData();
    }

/**
 * Must be called after a row was deleted.
 */
    protected void rowDeleted() {
        readCurrentData();
    }

/**
 * Process a move to row for any metadata level.
 */
    protected void readCurrentData() {
        // reset the list
        if (!isDataAvailable()) {
            setModel(new DefaultComboBoxModel());
            return;
        }

        try {
            ComboBoxModel comboBoxModel = getModel();
            Vector elements = new Vector();
            Object value;
            boolean changed = false;
            for (int i=0; i<getNumRows(); i++) {
                value = binding.getCell(columnBinding, i, column_name);
                changed = changed || value != comboBoxModel.getElementAt(i);
                elements.addElement(value == null ? " " : value.toString());
            }
            if (changed)
                setModel(new DefaultComboBoxModel(elements));
        }
        catch (DataModelException dme) {
            ExceptionManager.getHandler().processException(this.getComponent(), dme);
        }
    }

    protected void setDataAvailable(boolean state) {
        DSdbJComboBox.this.setEnabled(state);
    }

} // END ColumnDataBinding

/**
 * Add a listener to the list that's notified when the editor starts,
 * stops, or cancels editing.
 *
 * @param	l		the CellEditorListener
 */
public void addCellEditorListener(JCCellEditorListener l) {
    support.addCellEditorListener(l);
}

/**
 * Remove a listener from the list that's notified
 *
 * @param	l		the CellEditorListener
 */
public void removeCellEditorListener(JCCellEditorListener l) {
    support.removeCellEditorListener(l);
}


}
