/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DSdbJImage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:13 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.swing;

import java.awt.*;
import java.awt.image.*;
import java.util.*;

import javax.swing.*;

import com.klg.jclass.datasource.BindingModel;
import com.klg.jclass.datasource.DataModelException;
import com.klg.jclass.datasource.DataModel;
import com.klg.jclass.datasource.MetaDataModel;
import com.klg.jclass.datasource.ErrorDialog;

import com.klg.jclass.datasource.util.LocaleBundle;
import com.klg.jclass.util.swing.JCIconCreator;

/**
 * This class binds a component capable of displaying images to a column
 * in the current row of the datasource. The image is not editable
 * using this component.
 */
public class DSdbJImage extends JComponent implements ImageObserver {

FieldDataBinding        fieldBinding;
Image                   image;

protected String nullImageText = "DSdbJImage";  // the text to be displayed in the image initially
protected int textMargin = 2;                   // text margin to display nullImageText

/**
 * Creates a new DSdbImage component.
 */
public DSdbJImage() {
    fieldBinding = new FieldDataBinding();
}

/**
 * Creates a new DSdbImage component and binds it to a particular column
 * in a MetaDataModel.
 * @see com.klg.jclass.datasource.beans.NodeProperties#setDataSource
 */
public DSdbJImage(DataModel dataModel, MetaDataModel metaDataModel, String columnName) {
    super();
    fieldBinding.setDataBinding(dataModel, metaDataModel, columnName);
}

/**
 * Create and binding to a Binding.
 */
public DSdbJImage(BindingModel binding, String columnName) {
    super();
    setDataBinding(binding, columnName);
}

/**
 * Sets the parameters for binding the component to a particular column
 * in a MetaDataModel. If the MetaDataModel is the root in the DataModel's
 * MetaDataTree, the DataTableModel is automatically set to the root
 * DataTableModel, otherwise, the DataTableModel is set when an appropriate
 * table becomes available.
 */
public void setDataBinding(DataModel dataModel, MetaDataModel metaDataModel, String columnName) {
    fieldBinding.setDataBinding(dataModel, metaDataModel, columnName);
}

/**
 * Sets the data source to a DataModel and binds the MetaData to the
 * MetaDataModel represented by a "path" of MetaDataModel descriptions
 * separated by '|' (e.g. Orders|Customers).
 */
public void setDataBinding(DataModel dataModel, String metaPath) {
    fieldBinding.setDataBinding(dataModel, metaPath);
}

/**
 * Sets the data source to the path of DataModel, MetaDataModel descriptions
 * and column name.
 */
public void setDataBinding(String dataBinding) {
    fieldBinding.setDataBinding(dataBinding);
}

/**
 * Bind to Binding and column.
 */
public void setDataBinding(BindingModel b, String columnName) {
    fieldBinding.setDataBinding(b, columnName);
}

/**
 * Returns the data binding hierarchy string, starting from a JCTreeData instance name
 * and containing full sequence of MetaDataModel levels starting from the root to the
 * level the table is bound with.
 */
public String getDataBinding() {
    return fieldBinding.getDataBinding();
}

public void update(Graphics gc) {
    paint(gc);
}

public void paint(Graphics gc) {
    Rectangle r = gc.getClipBounds();
    gc.setColor(getBackground());
    gc.fillRect(r.x, r.y, r.width, r.height);

    if (image != null)
        gc.drawImage(image, 0, 0, this);
    else {
        FontMetrics fm = gc.getFontMetrics();
        gc.setColor(getForeground());
        Dimension d = getImageDimension();
        gc.drawString(nullImageText, textMargin+(r.width-d.width)/2,
                      textMargin+fm.getLeading()+fm.getAscent()+(r.height-d.height)/2);
    }

    super.paint(gc);
}

/**
 * Sets the image for the component. If the object is not an image, it is
 * treated as a byte array and converted to an image using JCIconCreator.
 */
public void setImage(Object item) {
    if (item == null)
        image = null;
    else if (item instanceof Image)
        image = (Image)item;
    else if (item instanceof byte[])
        image = JCIconCreator.getIcon((byte[])item).getImage();
    else {
        ErrorDialog.showError(null, LocaleBundle.string(LocaleBundle.INVALID_IMAGE));
        return;
    }
    repaint();
}

/**
 * Returns the image dimension.
 */
protected Dimension getImageDimension() {
    if (image == null) {
        FontMetrics fm = getFontMetrics(getFont());
        return new Dimension(fm.stringWidth(nullImageText)+textMargin*2,
                             fm.getHeight()+textMargin*2);
    }
    else
        return new Dimension(image.getWidth(this), image.getHeight(this));
}

/**
 * Returns the preferred size of the component. If no image is set, returns
 * text string dimensions.
 */
public Dimension getPreferredSize() {
    return getImageDimension();
}

/**
 * Returns the minimum size of the component.
 *
public Dimension getMinimumSize() {
return getPreferredSize();
}

/**
 * Returns the minimum size of the component.
 *
public Dimension getMaximumSize() {
return getPreferredSize();
}
*/
public boolean imageUpdate(Image image, int flags, int x, int y, int width, int height) {
    return true;
}

// END DSdbImage

/*
 * FieldDataBinding Inner Class
 *****************************************************************************/

class FieldDataBinding extends com.klg.jclass.datasource.util.FieldDataBinding {

    public FieldDataBinding() {
        super(DSdbJImage.this);
    }

    protected void refreshCell(Object item) {
        if (item == null)
            DSdbJImage.this.setImage(null);
        else
            DSdbJImage.this.setImage(item);
    }

    protected boolean saveCell() {
        return true;
    }

} // END FieldDataBinding

}
