/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DSdbJLabel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:13 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.swing;

import java.awt.*;

import javax.swing.JLabel;

import com.klg.jclass.datasource.BindingModel;
import com.klg.jclass.datasource.DataModelException;
import com.klg.jclass.datasource.DataModel;
import com.klg.jclass.datasource.MetaDataModel;

/**
 * This class binds a JLabel to a column in the current row of the datasource.
 */
public class DSdbJLabel extends JLabel {

FieldDataBinding fieldBinding;

/**
 * Creates a new DSdbJLabel component.
 */
public DSdbJLabel() {
    super(nullDatabindingText);
    fieldBinding = new FieldDataBinding();
}

/**
 * Creates a new DSdbJLabel component and binds it to a particular column
 * in a MetaDataModel.
 * @see com.klg.jclass.datasource.beans.NodeProperties#setDataSource
 */
public DSdbJLabel(DataModel dataModel, MetaDataModel metaDataModel, String columnName) {
    this();
    fieldBinding.setDataBinding(dataModel, metaDataModel, columnName);
}

/**
 * Create and bind to a Binding
 */
public DSdbJLabel(BindingModel binding, String columnName) {
    this();
    setDataBinding(binding, columnName);
}

/**
 * Sets the parameters for binding the component to a particular column
 * in a MetaDataModel. If the MetaDataModel is the root in the DataModel's
 * MetaDataTree, the DataTableModel is automatically set to the root
 * DataTableModel, otherwise, the DataTableModel is set when an appropriate
 * table becomes available.
 */
public void setDataBinding(DataModel dataModel, MetaDataModel metaDataModel, String columnName) {
    fieldBinding.setDataBinding(dataModel, metaDataModel, columnName);
}

/**
 * Sets the data source to a DataModel and binds the MetaData to the
 * MetaDataModel represented by a "path" of MetaDataModel descriptions
 * separated by '|' (e.g. Orders|Customers).
 */
public void setDataBinding(DataModel dataModel, String metaPath) {
    fieldBinding.setDataBinding(dataModel, metaPath);
}

/**
 * Sets the data source to the path of DataModel, MetaDataModel descriptions
 * and column name.
 */
public void setDataBinding(String dataBinding) {
    fieldBinding.setDataBinding(dataBinding);
}

/**
 * Bind to Binding and column.
 */
public void setDataBinding(BindingModel b, String columnName) {
    fieldBinding.setDataBinding(b, columnName);
}

/**
 * Returns the data binding hierarchy string, starting from a JCTreeData instance name
 * and containing full sequence of MetaDataModel levels starting from the root to the
 * level the table is bound with.
 */
public String getDataBinding() {
    return fieldBinding.getDataBinding();
}

// the text to be displayed in the label initially
protected final static String nullDatabindingText = "DSdbJLabel";

// text margin to display nullDatabindingText
protected int textMargin = 2;

/**
 * Override to display default text if unbound.
 */
public void paint(Graphics gc) {
    if (fieldBinding.isDataAvailable())
        super.paint(gc);
    else {
        FontMetrics fm = gc.getFontMetrics();
        Rectangle r = gc.getClipBounds();
        Dimension d = getPreferredSize();
        gc.drawString(nullDatabindingText, textMargin+(r.width-d.width)/2,
                      textMargin+fm.getLeading()+fm.getAscent()+(r.height-d.height)/2);

    }
}

/**
 * Returns the preferred size of the component.
 */
public Dimension getPreferredSize() {
    Dimension d = super.getPreferredSize();
    if (!fieldBinding.isDataAvailable()) {
        FontMetrics fm = getFontMetrics(getFont());
        d = new Dimension(fm.stringWidth(nullDatabindingText)+textMargin*2,
                             fm.getHeight()+textMargin*2);
    }
    return d;
}

// END DSdbJLabel

/*
 * DataBinding Inner Class
 *****************************************************************************/

class FieldDataBinding extends com.klg.jclass.datasource.util.FieldDataBinding {

    public FieldDataBinding() {
        super(DSdbJLabel.this);
    }

    protected void refreshCell(Object item) {
        if (item == null || item.toString() == null)
            DSdbJLabel.this.setText("");
        else
            DSdbJLabel.this.setText(item.toString());
    }

    protected boolean saveCell() {
        return true;
    }

} // END DataBinding

}
