/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DSdbJNavigator.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:13 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.swing;

import java.awt.*;
import java.awt.event.*;
import java.io.Serializable;
import java.text.MessageFormat;
import java.util.*;

import javax.swing.*;

import com.klg.jclass.datasource.util.*;
import com.klg.jclass.datasource.util.LocaleBundle;
import com.klg.jclass.datasource.BindingModel;
import com.klg.jclass.datasource.DataModel;
import com.klg.jclass.datasource.MetaDataModel;
import com.klg.jclass.datasource.DataModelException;


import com.klg.jclass.util.*;
import com.klg.jclass.util.swing.*;

/**
 * This component navigates the result set of a data bean and manipulate
 * the data source inserting and deleting records, cancel, save an requery data
 * in the data source.
 */
public class DSdbJNavigator extends JPanel implements ActionListener,
    Serializable {

// The bean properties
protected Color statusBackground = null;
protected Color statusForeground = null;
protected boolean statusVisible = true;
protected Color buttonBackground = null;
protected Color buttonForeground = null;

// Data source navigation and manipulation buttons
protected int buttons = 7;
protected int leftButtons = 4;
protected NavigatorButton first, previous, insert, delete, next, last, menu;

// Label which shows the data binding status and current row index
protected String rowLabel = "";

// Border thickness
protected int border = 1;

// Popup menu
protected NavigatorPopupMenu popupMenu = null;

// Internal data binding support class
protected DSNavigatorDataBinding dataBinding = null;

// Current data source row
// private int currentRow = -1;    // transient 

// Paint image for double buffering
private transient Image buffer = null;

// Last dimensions used for double buffering
private transient Dimension lastDimension = null;

// Debug support. Must be removed before release.
private static String base = "datanavigator";
private static int nameCounter = 0;
private static final boolean TRACE = true;
    	
/**
 * Creates a new DSdbJNavigator component.
 */
public DSdbJNavigator() {
    super();

    statusBackground = new Color(SystemColor.window.getRGB());
    statusForeground = new Color(SystemColor.windowText.getRGB());
    buttonBackground = new Color(SystemColor.control.getRGB());
    buttonForeground = new Color(SystemColor.controlText.getRGB());

    dataBinding = new DSNavigatorDataBinding();
    setName(base + nameCounter++);

    first = initButton(new NavigatorButton(NavigatorButton.FIRST), LocaleBundle.string(LocaleBundle.FIRST_ROW));
    previous = initButton(new NavigatorButton(NavigatorButton.PREVIOUS), LocaleBundle.string(LocaleBundle.PREVIOUS_ROW_TIP));
    insert = initButton(new NavigatorButton(NavigatorButton.INSERT), LocaleBundle.string(LocaleBundle.INSERT_ROW_TIP));
    delete = initButton(new NavigatorButton(NavigatorButton.DELETE), LocaleBundle.string(LocaleBundle.DELETE_ROW_TIP));
    next = initButton(new NavigatorButton(NavigatorButton.NEXT), LocaleBundle.string(LocaleBundle.NEXT_ROW_TIP));
    last = initButton(new NavigatorButton(NavigatorButton.LAST), LocaleBundle.string(LocaleBundle.LAST_ROW_TIP));
    menu = initButton(new NavigatorButton(NavigatorButton.MENU), LocaleBundle.string(LocaleBundle.POPUP_MENU_TIP));

    popupMenu = new NavigatorPopupMenu(this);
    if (popupMenu != null)
        add(popupMenu);

    setBtnBackground(buttonBackground);
    setBtnForeground(buttonForeground);

    reset();
    setLayout(null);
}

/**
 * Create and bind to a Binding.
public DSdbJNavigator(Binding binding) {
    super();
	setDataBinding(binding);
}   	
 */
    	
/**
 * Initializes and adds a button to the component.
 */
protected NavigatorButton initButton(NavigatorButton button, String toolTipText) {
    if (button != null) {
        add(button);
        button.addActionListener(this);
        button.setToolTipText(toolTipText);
    }
    return button;
}

/**
 * Saves current data source row position and updates the component.
 */
public void setCurrentRow() {
    refresh();
}

/**
 * Binds the component to a particular MetaDataModel in the DataModel
 * data source.
 * @param binding 	the Binding to use.
 * @param parameterNotCurrentlyUsed not currently used
 */
public void setDataBinding(BindingModel binding, boolean parameterNotCurrentlyUsed) {
	// The reason for "parameterNotCurrentlyUsed" is to differentiate this
	// method's signature from setDataBinding(String). The problem is introspection
	// is getting confused because it is not checking types of parameters, only the
	// number. This problem causes the "dataBinding" property not to show up in
	// the navigator bean.
    dataBinding.setDataBinding(binding);
}


/*
 * Binds the component to a particular MetaDataModel in the DataModel
 * data source.
 */
public void setDataBinding(DataModel dataModel, MetaDataModel metaDataModel) {
    dataBinding.setDataBinding(dataModel, metaDataModel);
}

/**
 * Binds the component to a particular MetaDataModel in the DataModel
 * data source represented by a "path" of MetaDataModel descriptions
 * separated by '|' (e.g. Orders|Customers).
 */
public void setDataBinding(DataModel dataModel, String metaPath) {
    dataBinding.setDataBinding(dataModel, metaPath);
}

/**
 * Binds the component to a particular MetaDataModel in the DataModel
 * data source represented by a "path" of DataModel instance name and MetaDataModel
 * descriptions separated by '|' (e.g. DataSource:Orders|Customers).
 *
 */
public void setDataBinding(String metaPath) {
    dataBinding.setDataBinding(metaPath);
}

/**
 * Returns the data binding hierarchy string, starting from a JCTreeData
 * instance name and containing full sequence of MetaDataModel level descriptions
 * separated by '|' starting from the root to the level the component is bound with
 * (e.g. DataSource:Orders|Customers).
 */
public String getDataBinding() {
    return dataBinding.getDataBinding();
}

/**
 * Gets StatusBackground property value.
 * @see #setStatusBackground
 */
public Color getStatusBackground() {
    return statusBackground;
}

/**
 * Sets StatusBackground property value.
 * @see #getStatusBackground
 */
public void setStatusBackground(Color color) {
    statusBackground = color;
}

/**
 * Gets StatusForeground property value.
 * @see #setStatusForeground
 */
public Color getStatusForeground() {
    return statusForeground;
}

/**
 * Sets StatusForeground property value.
 * @see #getStatusForeground
 */
public void setStatusForeground(Color color) {
    statusForeground = color;
}

/**
 * Gets StatusVisible property value.
 * @see #setStatusVisible
 */
public boolean getStatusVisible() {
    return statusVisible;
}

/**
 * Sets StatusVisible property value.
 * @see #getStatusVisible
 */
public void setStatusVisible(boolean b) {
    statusVisible = b;
    doLayout();
}

/**
 * Gets ButtonBackground property value.
 * @see #setButtonBackground
 */
public Color getButtonBackground() {
    return buttonBackground;
}

/**
 * Sets ButtonBackground property value.
 * @see #getButtonBackground
 */
public void setButtonBackground(Color color) {
    buttonBackground = color;
    setBtnBackground(buttonBackground);
}

/**
 * Gets ButtonForeground property value.
 * @see #setButtonForeground
 */
public Color getButtonForeground() {
    return buttonForeground;
}

/**
 * Sets ButtonForeground property value.
 * @see #getButtonForeground
 */
public void setButtonForeground(Color color) {
    buttonForeground = color;
    setBtnForeground(buttonForeground);
}

/**
 * Gets FirstVisible property value.
 * @see #setFirstVisible
 */
public boolean getFirstVisible() {
    return first.isVisible();
}

/**
 * Sets FirstVisible property value.
 * @see #getFirstVisible
 */
public void setFirstVisible(boolean b) {
    first.setVisible(b);
    doLayout();
}

/**
 * Gets PreviousVisible property value.
 * @see #setPreviousVisible
 */
public boolean getPreviousVisible() {
    return previous.isVisible();
}

/**
 * Sets PreviousVisible property value.
 * @see #getPreviousVisible
 */
public void setPreviousVisible(boolean b) {
    previous.setVisible(b);
    doLayout();
}

/**
 * Gets CommandVisible property value.
 * @see #setCommandVisible
 */
public boolean getCommandVisible() {
    return menu.isVisible();
}

/**
 * Sets CommandVisible property value.
 * @see #getCommandVisible
 */
public void setCommandVisible(boolean b) {
    menu.setVisible(b);
    doLayout();
}

/**
 * Gets NextVisible property value.
 * @see #setNextVisible
 */
public boolean getNextVisible() {
    return next.isVisible();
}

/**
 * Sets NextVisible property value.
 * @see #getNextVisible
 */
public void setNextVisible(boolean b) {
    next.setVisible(b);
    doLayout();
}

/**
 * Gets LastVisible property value.
 * @see #setLastVisible
 */
public boolean getLastVisible() {
    return last.isVisible();
}

/**
 * Sets LastVisible property value.
 * @see #getLastVisible
 */
public void setLastVisible(boolean b) {
    last.setVisible(b);
    doLayout();
}

/**
 * Gets InsertVisible property value.
 * @see #setInsertVisible
 */
public boolean getInsertVisible() {
    return insert.isVisible();
}

/**
 * Sets InsertVisible property value.
 * @see #getInsertVisible
 */
public void setInsertVisible(boolean b) {
    insert.setVisible(b);
    popupMenu.enableMenuItem(LocaleBundle.INSERT_ROW, b);
    doLayout();
}

/**
 * Gets DeleteVisible property value.
 * @see #setDeleteVisible
 */
public boolean getDeleteVisible() {
    return delete.isVisible();
}

/**
 * Sets DeleteVisible property value.
 * @see #getDeleteVisible
 */
public void setDeleteVisible(boolean b) {
    delete.setVisible(b);
    popupMenu.enableMenuItem(LocaleBundle.DELETE_ROW, b);
    doLayout();
}

/**
 * Returns the preferred width of the component.
 */
protected int preferredWidth() {
    FontMetrics fm = getFontMetrics(getFont());
    int labelWidth = fm.stringWidth(rowLabel);
    labelWidth = statusVisible ? (dataBinding.isEnabled() ? labelWidth+6 : labelWidth*2+6) : 0;
    return 2*border + labelWidth + first.getPreferredSize().width*7;
}

/**
 * Returns the minimum width of the component.
 */
protected int minimumWidth() {
    return 2*border + first.getPreferredSize().width*7 + 1;
}

/**
 * Returns the preferred height of the component.
 */
protected int preferredHeight() {
    FontMetrics fm = getFontMetrics(getFont());
    return border*2 + fm.getHeight();
}

/**
 * Returns the preferred size of the component.
 */
public Dimension getPreferredSize() {
    return new Dimension(preferredWidth(), preferredHeight());
}

/**
 * Returns the minimum size of the component.
 */
public Dimension getMinimumSize() {
    return new Dimension(minimumWidth(), preferredHeight());
}

/**
 * Returns the maximum size of the component.
 */
public Dimension getMaximumSize() {
    return new Dimension(600, preferredHeight());
}

/**
 * Lays out the component.
 */
public void doLayout() {
    Dimension d = getSize();
    if (d == null || d.height <= 0 || d.width <= 0)
        return;

    int height = d.height-border*2;
    int width = d.width-border*2;

    // Button width is the same and fixed.
    int w = first.getPreferredSize().width;
    int x = border;

    leftButtons = 0;
    if (first.isVisible()) {
        first.setBounds(x, border, w, height);
        x += w;
        leftButtons++;
    }
    if (previous.isVisible()) {
        previous.setBounds(x, border, w, height);
        x += w;
        leftButtons++;
    }
    if (delete.isVisible()) {
        delete.setBounds(x, border, w, height);
        x += w;
        leftButtons++;
    }
    if (menu.isVisible()) {
        menu.setBounds(x, border, w, height);
        leftButtons++;
    }

    buttons = leftButtons;

    x = border + width - w;
    if (insert.isVisible()) {
        insert.setBounds(x, border, w, height);
        x -= w;
        buttons++;
    }
    if (last.isVisible()) {
        last.setBounds(x, border, w, height);
        x -= w;
        buttons++;
    }
    if (next.isVisible()) {
        next.setBounds(x, border, w, height);
        buttons++;
    }

    refresh();
}

/**
 * Overwrites default update to avoid the component to be cleared
 * and filled it with the background color that causes flickering.
 */
public void update(Graphics graphics) {
    paint(graphics);
}

/**
 * Paints the component.
 */
public void paint(Graphics gc) {
    Dimension dimension = getSize();
    // Creates a new buffer is it doesn't exist or dimensions were changed.
    if (buffer == null || !dimension.equals(lastDimension)) {
        buffer = createImage(dimension.width, dimension.height);
        lastDimension = dimension;
        super.invalidate(); // To assure buttons repainting after deserialization.
    }
    Graphics graphics = buffer.getGraphics();
    graphics.setClip(0, 0, dimension.width, dimension.height);
    // Paints buttons
    super.paint(graphics);

    // Redraws border
    JCBorder.draw(graphics, JCBorder.ETCHED_IN, border, 0, 0, getSize().width, getSize().height,
                  Color.black, Color.black);

    // Redraws status line
    FontMetrics fm = graphics.getFontMetrics();
    int btn_width = first.getPreferredSize().width;
    int height = getSize().height-border*2;
    int vpad = (height - fm.getAscent())/2;
    int labelWidth = getSize().width - border*2 - btn_width*buttons;
    graphics.setColor(statusBackground);
    graphics.fillRect(border+btn_width*leftButtons, border, labelWidth, height);
    if (statusVisible) {
        String s;
        if (fm.stringWidth(rowLabel) > labelWidth) {
            s = JCWordWrap.wrapText(rowLabel, fm, labelWidth, "\n", true);
            int i = s.indexOf('\n');
            s = i>0 ? s.substring(0, i-1) : "";
        }
        else
            s = rowLabel;
        int x = border + btn_width*leftButtons + (labelWidth - fm.stringWidth(s))/2;
        graphics.setColor(statusForeground);
        graphics.drawString(s, x, height-vpad);
    }

    // Paints the buffer
    gc.drawImage(buffer, 0, 0, this);
}

/**
 * Resets the status line according to the current data binding
 * state, metadata model description, current record index and
 * total number of rows in the result set.
 * Also, enable/disable buttons and menu items according to the
 * current data binding state and row position the result set.
 */
protected void reset() {
	if (dataBinding == null) {
		return;
	}
	
	int currentRow = dataBinding.getCurrentRowIndex();
    boolean enabled = dataBinding.isEnabled();
	boolean isDataAvailable = false;
	if (dataBinding.getBinding() != null) {
		try {
			isDataAvailable = dataBinding.getBinding().getCurrentDataTableModel(dataBinding) != null;
		} 
		catch (DataModelException dme) {
			//
		}
	}
	
    boolean available = enabled && isDataAvailable;
    int total = available ? dataBinding.getNumRows() : 0;
    boolean positioned = available && total>0 && currentRow>=0;

    // Enables buttons
    first.setEnabled(positioned && currentRow > 0);
    previous.setEnabled(positioned && currentRow > 0);
    next.setEnabled(positioned && currentRow < total-1);
    last.setEnabled(positioned && currentRow < total-1);
    insert.setEnabled(available);
    delete.setEnabled(positioned);
    menu.setEnabled(enabled);

    // Enables menu items
    popupMenu.enableMenuItem(LocaleBundle.INSERT_ROW, available);
    popupMenu.enableMenuItem(LocaleBundle.DELETE_ROW, positioned);
    popupMenu.enableMenuItem(LocaleBundle.CANCEL_ROW, positioned);
    popupMenu.enableMenuItem(LocaleBundle.CANCEL_ALL, enabled);
    popupMenu.enableMenuItem(LocaleBundle.REQUERY_ROW_AND, positioned);
    popupMenu.enableMenuItem(LocaleBundle.REQUERY_ALL, enabled);
    popupMenu.enableMenuItem(LocaleBundle.COMMIT_ROW, positioned);
    popupMenu.enableMenuItem(LocaleBundle.COMMIT_ALL, enabled);
    popupMenu.enableMenuItem(LocaleBundle.GO_TO, available);

    // Makes status line
    if (!enabled)
        rowLabel = LocaleBundle.string(LocaleBundle.NOT_BOUND);
    else {
        if (positioned) {
            Object[] args = { new Integer(currentRow+1),new Integer(total)};
            rowLabel = MessageFormat.format(
                                           dataBinding.isRowModified(currentRow) ?
                                           LocaleBundle.string(LocaleBundle.RECORD_OF_TOTAL_STATUS) :
                                           LocaleBundle.string(LocaleBundle.RECORD_OF_TOTAL),
                                           args);
        }
        else if (total > 0)
            rowLabel = total+LocaleBundle.string(LocaleBundle.RECORDS);
        else
            rowLabel = available ? LocaleBundle.string(LocaleBundle.NO_RECORDS) :
                       LocaleBundle.string(LocaleBundle.NOT_AVAILABLE);

        rowLabel = dataBinding.getDescription()+": "+rowLabel;
    }

    /* FOR DEBUGGING PURPOSES
    try {
        rowLabel = "[" + dataBinding.getMetaDataModel().getCurrentBookmark() +  "] " + rowLabel;
    } catch(Exception e) {}
    */
}

/**
 * Resets and repaints component.
 */
public void refresh() {
    reset();
    repaint();
}

/**
 * Sets the layout manager for this container.
 * As layout manager is not needed for the component, the method is
 * overridden to prevent the layout manager from being set.
 */
public final void setLayout(LayoutManager mgr) {
    super.setLayout(null);
}

/**
 * Sets background for all buttons
 */
protected void setBtnBackground(Color color) {
    first.setBackground(buttonBackground);
    previous.setBackground(buttonBackground);
    insert.setBackground(buttonBackground);
    delete.setBackground(buttonBackground);
    next.setBackground(buttonBackground);
    last.setBackground(buttonBackground);
    menu.setBackground(buttonBackground);
}

/**
 * Sets foreground for all buttons
 */
protected void setBtnForeground(Color color) {
    first.setForeground(buttonForeground);
    previous.setForeground(buttonForeground);
    insert.setForeground(buttonForeground);
    delete.setForeground(buttonForeground);
    next.setForeground(buttonForeground);
    last.setForeground(buttonForeground);
    menu.setForeground(buttonForeground);
}

/**
 * Invoked when an action occurs.
 */
public void actionPerformed(ActionEvent event) {
    if (event.getSource() instanceof NavigatorButton) {

    	try {
			if (dataBinding.getBinding() == null 
				|| dataBinding.getBinding().getCurrentDataTableModel(dataBinding) == null) {
    			return;
			}
    	} 
    	catch (DataModelException dme) {
    		return;
    	}

    	int row = dataBinding.getCurrentRowIndex();
        NavigatorButton btn = (NavigatorButton)event.getSource();
        if (btn.equals(first)) {
            row = 0;
        }
        else if (btn.equals(previous)) {
            row--;
        }
        else if (btn.equals(next)) {
            row++;
        }
        else if (btn.equals(last)) {
            row = dataBinding.getNumRows()-1;
        }
        else if (btn.equals(menu)) {
            Point p = btn.getLocation();
            p.y += btn.getHeight();
            showPopupMenu(p.x, p.y);
            return;
        }
        else if (btn.equals(delete)) {
            delete();
            return;
        }
        else if (btn.equals(insert)) {
            insert();
            return;
        }
        dataBinding.moveToRowIndex(row);
    }
    else {
        String command = event.getActionCommand();
        if (command.equals(NavigatorPopupMenu.commands[0]))
            insert();       // Adds a new record in the current table
        else if (command.equals(NavigatorPopupMenu.commands[1]))
            delete();       // Removes the current record in the current table
        else if (command.equals(NavigatorPopupMenu.commands[2]))
            cancelCurrent();    // Cancels the changes made in the current record
        else if (command.equals(NavigatorPopupMenu.commands[3]))
            cancelAll();        // Cancels all the changes made in the data source
        else if (command.equals(NavigatorPopupMenu.commands[4]))
            requeryCurrent();   // Requeries the current row and details from the database
        else if (command.equals(NavigatorPopupMenu.commands[5]))
            requeryAll();       // Requeries the dataset from the database
        else if (command.equals(NavigatorPopupMenu.commands[6]))
            commitCurrent();    // Commits the current row in the database
        else if (command.equals(NavigatorPopupMenu.commands[7]))
            commitAll();        // Commits all changes made in the data source
        else if (command.equals(NavigatorPopupMenu.commands[8]))
            goToRow();          // Opens a dialog to specify the current row position in the data source
    }
}

/**
 * Popups a menu.
 */
protected boolean showPopupMenu(int x, int y) {
    popupMenu.show(this, x, y);
    return true;
}

/*
 * Adds a new record in the current table.
 * The same as the add button.
 */
protected void insert() {
    dataBinding.addRow();
}

/*
 * Removes the current record in the current table.
 * The same as the delete button.
 */
protected void delete() {
    dataBinding.deleteRow();
}

/*
 * Cancels the changes made in the current record.
 */
protected void cancelCurrent() {
    dataBinding.cancelRow();
}

/*
 * Cancels all the changes made in the data source.
 */
protected void cancelAll() {
    dataBinding.cancelAll();
}

/*
 * Requeries the current row and details from the database.
 */
protected void requeryCurrent() {
    dataBinding.requeryRowAndDetails();
}

/*
 * Requeries the dataset from the database.
 */
protected void requeryAll() {
    dataBinding.requeryAll();
}

/*
 * Commits the current row in the database.
 */
protected void commitCurrent() {
    dataBinding.commitRow();
}

/*
 * Commits all changes made in the data source.
 */
protected void commitAll() {
    dataBinding.commitAll();
}

/**
 * Opens a dialog to specify current row index and moves the current
 * row position in the data source
 */
protected void goToRow() {
	int currentRow = dataBinding.getCurrentRowIndex();
    GotoRowDialog dialog = new GotoRowDialog(new JFrame(), currentRow+1);
    dialog.open();
    int row = dialog.getRow()-1;
    int total = dataBinding.getNumRows();
    dataBinding.moveToRowIndex(row<0 ? 0 : (row>=total ? total-1 : row));
}


/**
 * Internal data binding support class.
 */
class DSNavigatorDataBinding extends NavigatorDataBinding {

/**
 * Creates a new data binding support class.
 */
    protected DSNavigatorDataBinding() {
        super(DSdbJNavigator.this);
    }

/**
 * Sets the current row index in the component.
 */
    protected void currentRowChanged(int rowIndex) {
        DSdbJNavigator.this.setCurrentRow();
    }

/**
 * Checks if the move to a new current row position is accepted.
 * Any move is accepted by this class.
 */
    protected boolean beforeCurrentRowChange() {
        return true;
    }

/**
 * Updates the component, because the data table or
 * the current row position was changed.
 */
    protected void tableChanged() {
        DSdbJNavigator.this.refresh();
    }

/**
 * Must be called after row data were changed.
 * Not used by this class.
 */
    protected void rowDataChanged(int rowIndex) {
    }

/**
 * Must be called after a single cell value was changed.
 * Not used by this class.
 */
    protected void cellDataChanged(int row, String columnName) {
    }

/**
 * Is called after a row was inserted.
 * Sets the current row index in the component.
 */
    protected void rowInserted() {
        DSdbJNavigator.this.setCurrentRow();
    }

/**
 * Is called after a row was deleted.
 * Sets the current row index in the component.
 */
    protected void rowDeleted() {
        DSdbJNavigator.this.setCurrentRow();
    }

/**
 * Sets DataModel current row.
 */
    protected boolean moveToRowIndex(int row) {
        return super.moveToRowIndex(row);
    }

/**
 * Retrieves a description of the current metadata model.
 */
    protected String getDescription() {
        return binding.getMetaDataModel().getDescription();
    }

}   // DSNavigatorDataBinding

}  // DSdbJNavigator


/**
 * Internal navigator button class.
 */
class NavigatorButton extends JComponent implements MouseListener, Serializable {

// Button types
static final int FIRST = 0;
static final int PREVIOUS = 1;
static final int INSERT = 2;
static final int DELETE = 3;
static final int NEXT = 4;
static final int LAST = 5;
static final int MENU = 6;

// Button's type
private int type = FIRST;

// Current state of the button
private boolean pressed = false;

// Button's icons for painting
private transient Image enabledIcon = null;
private transient Image disabledIcon = null;

// The list of action listeners
private JCListenerList actionListeners;

// Button's dimensions
private static final int width = 15, height = 15;

// Paint image for double buffering
private transient Image buffer = null;

// Icon image strings for JCImageCreator
private static final String iconStrings[][] = {

    {
        "         ",
        "  .   .  ",
        "  .  ..  ",
        "  . ...  ",
        "  .....  ",
        "  . ...  ",
        "  .  ..  ",
        "  .   .  ",
        "         ",
    },

    {
        "         ",
        "     .   ",
        "    ..   ",
        "   ...   ",
        "  ....   ",
        "   ...   ",
        "    ..   ",
        "     .   ",
        "         ",
    },

    {
        "       . ",
        " .    ...",
        " ..    . ",
        " ... .   ",
        " .....   ",
        " ... .   ",
        " ..  .   ",
        " .   .   ",
        "         ",
    },

    {
        "         ",
        "  .   .  ",
        "  .. ..  ",
        "   ...   ",
        "    .    ",
        "   ...   ",
        "  .. ..  ",
        "  .   .  ",
        "         ",
    },

    {
        "         ",
        "   .     ",
        "   ..    ",
        "   ...   ",
        "   ....  ",
        "   ...   ",
        "   ..    ",
        "   .     ",
        "         ",
    },

    {
        "         ",
        "   .   . ",
        "   ..  . ",
        "   ... . ",
        "   ..... ",
        "   ... . ",
        "   ..  . ",
        "   .   . ",
        "         ",
    },

    {
        " .....   ",
        " . . .   ",
        " .....   ",
        " .  ..   ",
        " .....   ",
        " ..  .   ",
        " .. .....",
        "     ... ",
        "      .  ",
    },

};

/**
 * Creates a new NavigatorButton component.
 */
NavigatorButton(int type) {
    this.type = type;
    super.setEnabled(false);
    addMouseListener(this);
}

/*
 * The component cannot be traversed using Tab or Shift-Tab
 * keyboard focus traversal.
 */
public boolean isFocusTraversable() {
    return false;
}

/**
 * Enables the component.
 */
public void setEnabled(boolean v) {
    super.setEnabled(v);
    repaint();
}

/**
 * Paints the component.
 */
public void paint(Graphics gc) {
    if (buffer == null) {
        Rectangle bounds = getBounds();
        buffer = createImage(bounds.width, bounds.height);
    }
    Graphics graphics = buffer.getGraphics();

    graphics.setColor((pressed) ? getBackground().darker() : getBackground());
    graphics.fillRect(0, 0, getSize().width, getSize().height);

    // Draw border
    JCBorder.drawNormal(graphics, 1, 0, 0, getSize().width,
                        getSize().height, Color.black, Color.black);

    // Draw highlight
    JCBorder.drawNormal(graphics, 1, 1, 1, getSize().width-1, getSize().height-1,
                        (pressed) ? Color.black : Color.white, (pressed) ? Color.white : Color.black);

    // Create arrow icons
    if (enabledIcon == null)
        createIcons();
    Image icon = isEnabled() ? enabledIcon : disabledIcon;
    graphics.drawImage(icon,
                       (getSize().width-icon.getWidth(null))/2,
                       (getSize().height-icon.getHeight(null))/2, null);

    gc.drawImage(buffer, 0, 0, this);
}

/**
 * Sets button's foreground.
 * @see #setBackground
 */
public void setForeground(Color c) {
    super.setForeground(c);
    createIcons();
}

/**
 * Sets button's background.
 * @see #setForeground
 */
public void setBackground(Color c) {
    super.setBackground(c);
    createIcons();
}

public synchronized void addActionListener(ActionListener l) {
    actionListeners = JCListenerList.add(actionListeners, l);
}
public synchronized void removeActionListener(ActionListener l) {
    actionListeners = JCListenerList.remove(actionListeners, l);
}

/**
 * Returns the preferred size of the component.
 */
public Dimension getPreferredSize() {
    return new Dimension(width, height);
}

/**
 * Returns the minimum size of the component.
 */
public Dimension getMinimumSize() {
    return new Dimension(width, height);
}

public void mouseClicked(MouseEvent e) {
}
public void mousePressed(MouseEvent e) {
    if (!isEnabled())
        return;

    pressed = true;
    repaint();
}

public void mouseReleased(MouseEvent e) {
    if (!isEnabled())
        return;

    pressed = false;
    repaint();

    ActionEvent event = new ActionEvent(this, ActionEvent.ACTION_PERFORMED, ""+type, 0);
    for (Enumeration en = JCListenerList.elements(actionListeners);
        en.hasMoreElements();) {
        ActionListener al = (ActionListener) en.nextElement();
        al.actionPerformed(event);
    }
}
public void mouseEntered(MouseEvent e) {
}
public void mouseExited(MouseEvent e) {
}

/**
 * Creates the button icons for painting.
 */
private void createIcons() {
    Color foreground = getForeground();
    Color background = getBackground();
    if (foreground == null || background == null)
        return;

    JCIconCreator icon = new JCIconCreator(9, 9);
    icon.setColor('.', foreground);
    icon.setPixels(iconStrings[type]);
    enabledIcon = icon.getIcon().getImage();
    icon.clear();
    icon.setColor('.', background.darker());
    icon.setPixels(iconStrings[type]);
    disabledIcon = icon.getIcon().getImage();
}

}
