/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DSdbJTextField.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:13 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.swing;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.JTextField;

import com.klg.jclass.datasource.BindingModel;
import com.klg.jclass.datasource.DataModel;
import com.klg.jclass.datasource.MetaDataModel;
import com.klg.jclass.datasource.DataModelException;

/**
 * This class binds a JTextField to a column in the current row of the datasource.
 * Changing the value updates the record in the datasource but does not automatically
 * commit the change. Enter and lost focus initiates the update.
 */
public class DSdbJTextField extends JTextField implements FocusListener,
    KeyListener {

FieldDataBinding        fieldBinding;

/**
 * Creates a new DSdbJTextField component.
 */
public DSdbJTextField() {
    super("");
    fieldBinding = new FieldDataBinding();
    addFocusListener(this);
    addKeyListener(this);
}

/**
 * Creates a new DSdbJTextField component and binds it to a particular column
 * in a MetaDataModel.
 * @see com.klg.jclass.datasource.beans.NodeProperties#setDataSource
 */
public DSdbJTextField(BindingModel b, String columnName) {
    this();
    fieldBinding.setDataBinding(b, columnName);
}

/**
 * Sets the data source to a DataModel and binds the MetaData to the
 * MetaDataModel represented by a "path" of MetaDataModel descriptions
 * separated by '|' (e.g. Orders|Customers).
 */
public void setDataBinding(DataModel dataModel, MetaDataModel metaDataModel, String metaPath) {
    fieldBinding.setDataBinding(dataModel, metaDataModel, metaPath);
}

/**
 * Sets the data source to the path of DataModel, MetaDataModel descriptions
 * and column name.
 */
public void setDataBinding(String dataBinding) {
	fieldBinding.setDataBinding(dataBinding);
}

/**
 * Bind to Binding and column.
 */
public void setDataBinding(BindingModel b, String columnName) {
    fieldBinding.setDataBinding(b, columnName);
}

/**
 * Returns the data binding hierarchy string, starting from a JCTreeData instance name
 * and containing full sequence of MetaDataModel levels starting from the root to the
 * level the table is bound with.
 */
public String getDataBinding() {
    return fieldBinding.getDataBinding();
}

/*
 * FocusListener Implementation.
 *****************************************************************************/

/**
 * Called when the text field gets focus.
 */
public void focusGained(FocusEvent event) {
}

// Semaphore used to prevent reentrance.
private boolean focusRequested = false;

/**
 * Commits any changes when the text field loses focus.
 */
public void focusLost(FocusEvent event) {
    if (focusRequested) {
        return;
    }

    // Critical section of code - must be non-reentrant!
    focusRequested = true;
    fieldBinding.convertAndSaveItem(getText());

    // End of critical section of code.
    focusRequested = false;
}

/*
 * KeyListener Implementation.
 *****************************************************************************/

/**
 * Called when a key is pressed. Saves any changes to the data source.
 */
public void keyPressed(KeyEvent event) {
    if (event.getKeyCode() == KeyEvent.VK_ENTER) {
        fieldBinding.convertAndSaveItem(getText());
    }
}

/**
 * Called when a key is released.
 */
public void keyReleased(KeyEvent event) {
}

/**
 * Called when a key is typed.
 */
public void keyTyped(KeyEvent event) {
}

// the text to be displayed in the label initially
protected final static String nullDatabindingText = "DSdbJTextField";

// text margin to display nullDatabindingText
protected int textMargin = 2;

/**
 * Override to display default text if unbound.
 */
public void paint(Graphics gc) {
    if (fieldBinding.isDataAvailable())
        super.paint(gc);
    else {
        FontMetrics fm = gc.getFontMetrics();
        Rectangle r = gc.getClipBounds();
        Dimension d = getPreferredSize();
        gc.drawString(nullDatabindingText, textMargin+(r.width-d.width)/2,
                      textMargin+fm.getLeading()+fm.getAscent()+(r.height-d.height)/2);

    }
}

/**
 * Returns the preferred size of the component.
 */
public Dimension getPreferredSize() {
    Dimension d = super.getPreferredSize();
    if (!fieldBinding.isDataAvailable()) {
        FontMetrics fm = getFontMetrics(getFont());
        d = new Dimension(fm.stringWidth(nullDatabindingText)+textMargin*2,
                             fm.getHeight()+textMargin*2);
    }
    return d;
}


// END DSdbJTextField

/*
 * DataBinding Inner Class
 *****************************************************************************/

class FieldDataBinding extends com.klg.jclass.datasource.util.FieldDataBinding {

    public FieldDataBinding() {
        super(DSdbJTextField.this);
    }

    protected void refreshCell(Object item) {
        if (item == null || item.toString() == null)
            DSdbJTextField.this.setText("");
        else
            DSdbJTextField.this.setText(item.toString());
    }

    protected boolean saveCell() {
        convertAndSaveItem(DSdbJTextField.this.getText());
        return true;
    }

} // END DataBinding

}
