/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ColumnDataBinding.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:13 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.util;

import java.awt.Component;

import javax.swing.tree.*;
import java.util.*;

import com.klg.jclass.datasource.DataModelEvent;
import com.klg.jclass.datasource.DataModelException;
import com.klg.jclass.datasource.DataModel;
import com.klg.jclass.datasource.MetaDataModel;
import com.klg.jclass.datasource.BindingModel;

/**
 * An extension of DataBinding that further allows binding to
 * a particular column<p>
 * This is an abstract class and is further extended to either
 * (as of this writing) by FieldDataBinding which allows one to
 * bind to the current of the bound column and ColumnDataBinding which
 * allows you to bind to the whole column of the current table of the
 * particular meta-data level.
 */

public abstract class ColumnDataBinding extends DataBinding {

protected int column_type = -1;
protected String column_name = "";

public ColumnDataBinding(Component component) {
    super(component);
}

public ColumnDataBinding(Component component, BindingModel binding, String column_name) {
    super(component);
    setDataBinding(binding, column_name);
}

/**
 * Returns the data binding hierarchy string, starting from a JCTreeData
 * instance name and containing full sequence of MetaDataModel levels
 * starting from the root to the level the table is bound with and further
 * down to the particular column.
 * @see com.klg.jclass.datasource.beans.NodeProperties#setDataSource
 * @see #getDataBinding
 * @see com.klg.jclass.datasource.beans.JCTreeData
 * @see com.klg.jclass.datasource.MetaDataModel
 */
public String getDataBinding() {
	String binding = super.getDataBinding();
	if (!binding.equals("")) {
		binding = binding + PATH_DELIMITER + column_name;
    }
	return binding;
}

/**
 * Sets up parameters to bind the JCDataSource to a MetaDataModel in the
 * given DataModel.
 */
public void setDataBinding(DataModel dataModel, MetaDataModel metaDataModel, String column_name) {
	BindingModel bindingModel = null;
	if (dataModel != null) {
		// if no metaDataModel is specified, assume the root
		bindingModel = metaDataModel == null ?
			((MetaDataModel)dataModel.getMetaDataTree().getRoot()).getBinding() :
				metaDataModel.getBinding();
	}
	setDataBinding(bindingModel, column_name);
}

/**
 * Bind to a level using a Binding.
 * @param 	binding the binding level.
 * @param 	columnName the column name.
 */
public void setDataBinding(BindingModel binding, String column_name) {
    // call routine in super to bind to the data source and meta-data level
    setDataBindingInternalBegin(binding);

    this.column_name = column_name;

	if (this.binding == null) {
        setDataAvailable(false);
        return;
    }

    // figure out the java class we are supporting
    if (column_name != null) {
		column_type = binding.getJavaColumnType(column_name);
    }
    // call routine in super to do the final initialization
    setDataBindingInternalEnd();
}

/**
 * Sets the data source to a DataModel and binds the MetaData and column to
 * the MetaDataModel represented by a "path" of MetaDataModel
 * descriptions separated by '|' (e.g. Orders|Customers|CustomerID).<P>
 *
 * This overrides the super class version of this which binds only
 * to a particular meta-data level
 */
public void setDataBinding(DataModel dataModel, String meta_description) {
    if (dataModel == null) {
    	// unbind
        // setDataBinding(null, null, null);
    }

    // we need to drill down to the appropriate meta-data level
    // and column_name
    TreeModel tree_model = dataModel.getMetaDataTree();
    MetaDataModel metaDataModel = (MetaDataModel) tree_model.getRoot();
    StringTokenizer st = new StringTokenizer(meta_description,
                                             PATH_DELIMITER);
    String column_name = null;
    if (st.hasMoreTokens()) {
        String s = st.nextToken();
        while (st.hasMoreTokens()) {
            // the last token is the column name
            column_name = s = st.nextToken();
            if (st.hasMoreTokens()) {
                // we are not at the last token so we know we are looking
                // for meta-data
                Enumeration e = metaDataModel.children();
                while (e.hasMoreElements()) {
                    MetaDataModel m = (MetaDataModel)e.nextElement();
                    if (s.compareTo(m.getDescription()) == 0) {
                        metaDataModel = m;
                        break;
                    }
                }
            }
        }
    }

    setDataBinding(metaDataModel.getBinding(), column_name);
}

/**
 * Determines whether the current column can be edited.
 * @return true if the column can be edited; false otherwise.
 */
public boolean isColumnEditable() {
    return isDataAvailable() && isColumnEditable(column_name);
}

}
