/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ExceptionManager.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:13 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.util;

import java.awt.*;
import java.io.Serializable;

import com.klg.jclass.datasource.DataModelException;
import com.klg.jclass.util.swing.JCMessageHelper;

/**
 * This class acts as both a repository for the current datasource exception
 * handler and as the default implemention of a handler.  The default
 * implementation puts up a message dialog with the message.
 */
public class ExceptionManager implements ExceptionHandler, Serializable {

static protected ExceptionHandler handler = new ExceptionManager();

/**
 * Return the ExceptionHandler to used when the datasource throws an exception
 */
static public final ExceptionHandler getHandler() {
    return handler;
}

/**
 * Set the ExceptionHandler to use when the datasource throws an exception
 */
static public final void setHandler(ExceptionHandler h) {
    handler = h;
}


protected boolean writeToConsole = false;

public ExceptionManager() {
}

public ExceptionManager(boolean write_to_console) {
    writeToConsole = write_to_console;
}

/**
 * Gets the component's parent frame
 */
private static Frame getFrame(Component comp) {
    Window w = getWindow(comp);
    if (w instanceof Dialog) {
        if (w.getParent() instanceof Window) {
            w = (Window) w.getParent();
        }
    }
    return(w instanceof Frame ? (Frame) w : null);
}

/** Gets the component's parent window (either a Frame or a Dialog). */
private static Window getWindow(Component comp) {
    Component tmp = comp, parent = comp;
    while (true) {
        if (parent == null || parent instanceof Window)
            break;
        parent = tmp;
        if (tmp == null)
            return null;
        tmp = tmp.getParent();
    }
    return(Window) parent;
}

/**
 * This method is invoked by an exception generated by a
 * user action on a databound component.  This is the default implementation
 * and it puts up a dialog window when needed.
 */
public void showException(Component component, String title, String message) {
    if (component != null) {
        // ??? needs localization
        JCMessageHelper.showError(title, message);
    }

    if (component == null || writeToConsole == true) {
        System.err.println(message);
    }
}

/**
 * This method is invoked by an datamodel exception generated by a
 * user action on a databound component.  This is the default implementation
 * and it puts up a dialog window when needed.
 */
public void processException(Component component, DataModelException ex) {
    String message = ex.getMessage();
    if (message == null || message.length() == 0) {
        message = ex.toString();
    }

    showException(component, "DataSource Exception", message);
}

}
