/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: FieldDataBinding.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:13 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.util;

import java.awt.Component;
import java.math.BigDecimal;
import java.sql.*;
import java.sql.*;
import java.util.*;
import java.util.Date;

import com.klg.jclass.datasource.DataModelEvent;
import com.klg.jclass.datasource.DataModelException;
import com.klg.jclass.datasource.DataModelListener;
import com.klg.jclass.datasource.MetaDataModel;
import com.klg.jclass.datasource.BindingModel;

/**
 * An extension of DataBinding that further allows binding to
 * a particular column<p>
 * This is an abstract class and is further extended to either
 * (as of this writing) by FieldDataBinding which allows one to
 * bind to the current of the bound column and ColumnDataBinding which
 * allows you to bind to the whole column of the current table of the
 * particular meta-data level.<p>
 * To complete an implementation of this class tableChanged() must
 * be implemented.
 */
public abstract class FieldDataBinding extends ColumnDataBinding {

public FieldDataBinding(Component component) {
    super(component);
}

/**
 * Process a change in the data
 */
protected void tableChanged() {
    setDataAvailableState();
}

/**
 * Must be called after row data were changed.
 */
protected void rowDataChanged(int rowIndex) {
    refreshCell(readCurrentCell());
}

/**
 * Must be called after a single cell value was changed.
 */
protected void cellDataChanged(int rowIndex, String columnName) {
    if (columnName.equals(column_name))
        refreshCell(readCurrentCell());
}

/**
 * Changes the current row in the component.
 */
protected void currentRowChanged(int rowIndex) {
    setDataAvailableState();
    refreshCell(readCurrentCell());
}

/**
 * Checks if all listeners have accepted the move.
 */
protected boolean beforeCurrentRowChange() {
    return isColumnEditable() ? saveCell() : true;
}

/**
 * Must be called after a row was inserted.
 */
protected void rowInserted() {
}

/**
 * Must be called after a row was deleted.
 */
protected void rowDeleted() {
    toggleDataAvailableState();
}

/**
 * Must be implemented by subclasses in order to update the component's
 * data contents when the current row changes or the record is modified.
 */
protected abstract void refreshCell(Object item);

/**
 * Must be implemented by subclasses in order to save the component's
 * data to the datasource.
 */
protected abstract boolean saveCell();

/**
 * This should be called from implementation of tableChanged to see
 * if we need to change the our data
 */
protected Object readCurrentCell() {
    if (isDataAvailable()) {
		try {
			return binding.getCurrentCell(this, column_name);
		} catch (DataModelException dme) {
			ExceptionManager.getHandler().processException(getComponent(), dme);
		}
    }
	return null;
}

/**
 * This method is invoked by the sub-class to save the data item to the
 * super-class
 */
public void saveItem(Object new_value) {
    if (!isDataAvailable()) {
        return;
    }
	try {
		binding.setCurrentCell(this, column_name, new_value);
	} catch (DataModelException dme) {
		ExceptionManager.getHandler().processException(getComponent(), dme);
	}
}

/**
 * convert the value from a string and save it by invoking saveItem
 */
public void convertAndSaveItem(String value) {
    if (!isDataAvailable()) {
        return;
    }

    Object new_value = (value instanceof String && ((String)value).length()==0) ?
                       null : value;

    try {
        if (new_value != null) {
            switch (column_type) {
            case MetaDataModel.TYPE_BOOLEAN:
                if (value.equalsIgnoreCase("true") || value.equalsIgnoreCase("t"))
                    new_value = new Boolean(true);
                else if (value.equalsIgnoreCase("false") || value.equalsIgnoreCase("f"))
                    new_value = new Boolean(false);
                break;
            case MetaDataModel.TYPE_SQL_DATE:
                new_value = new Date(value.trim());
                break;
            case MetaDataModel.TYPE_DOUBLE:
                if (value.trim().equalsIgnoreCase("infinity"))
                    new_value = new Double(Double.POSITIVE_INFINITY);
                else
                    new_value = Double.valueOf(value.trim());
                break;
            case MetaDataModel.TYPE_FLOAT:
                if (value.trim().equalsIgnoreCase("infinity"))
                    new_value = new Float(Float.POSITIVE_INFINITY);
                else
                    new_value = Float.valueOf(value.trim());
                break;
            case MetaDataModel.TYPE_INTEGER:
                new_value = Integer.valueOf(value.trim());
                break;
            case MetaDataModel.TYPE_STRING:
                new_value = value;
                break;
            case MetaDataModel.TYPE_BIG_DECIMAL:
                new_value = new BigDecimal(value.trim());
                break;
            case MetaDataModel.TYPE_LONG:
                new_value = Long.valueOf(value.trim());
                break;
            case MetaDataModel.TYPE_SQL_TIME:
                new_value = Time.valueOf(value.trim());
                break;
            case MetaDataModel.TYPE_SQL_TIMESTAMP:
                if (value.indexOf(":") < 0)
                    new_value = Timestamp.valueOf(value.trim() + " 00:00:00.0");
                else
                    new_value = Timestamp.valueOf(value.trim());
                break;
            case MetaDataModel.TYPE_OBJECT:
                new_value = value;
                break;
            case MetaDataModel.TYPE_BYTE:
                new_value = Byte.valueOf(value.trim());
                break;
            case MetaDataModel.TYPE_SHORT:
                new_value = Short.valueOf(value.trim());
                break;
            case MetaDataModel.TYPE_BYTE_ARRAY:
                // There is no mechanism to edit this type of data so whatever comes in goes out.
                // Any user-build editor will have to deal with the data-type appropriately.
                new_value = value;
                break;
            case MetaDataModel.TYPE_UTIL_DATE:
                new_value = new Date(value.trim());
                break;
            }
        }
    }
    catch (Exception e) {
        ExceptionManager.getHandler().showException(getComponent(), "Error",
        	LocaleBundle.string(LocaleBundle.INVALID_FORMAT) + value);
        return;
    }

    // do the actual save now that we have converted the value
    try {
        binding.setCurrentCell(this, column_name, new_value);
    }
    catch (DataModelException exception) {
        ExceptionManager.getHandler().processException(getComponent(), exception);
    }
}

/**
 * Disables the component if there is no DataTableModel or there is no
 * current row.
 */
public void setDataAvailable(boolean state) {
    // if there is a component, set it's enabled state to match whether data is available
    if (component != null) {
        component.setEnabled(isColumnEditable());

        //  if there is no data available, force a refresh with null data.
        if (!isDataAvailable())
            refreshCell(null);
    }
}

/**
 * Checks and sets the new data available state.
 */
public void setDataAvailableState() {
    toggleDataAvailableState();
}

/**
 * Checks whether the current bookmark is available and calls
 * toggleDataAvailableState() for a superclass.
 */
protected void toggleDataAvailableState() {
    setDataAvailable(binding.isDataAvailable());
}

}
