/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: SqlStatementTable.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:13 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource.util;

import java.io.Serializable;
import java.util.StringTokenizer;

/**	Describes a table in a SQL statement.  It contains qualifier, alias and name
 *	of the table.  It also indicates if the table is a subquery.  The text
 *	of the subquery is stored.  If alias is not specified, it is 
 *	default to its qualified name.
 */
public class SqlStatementTable implements Serializable {
	
static final long serialVersionUID = 445042217633496325L;

private String alias;
private String table;
private String qualifier;
private String subQuery = null;
private String catalog;
private String schema;
private boolean isSubQuery = false;

/** Constructs a table.
 * @param name table name
 * @param qualifier qualifier of the table, i.e. [qualifier.]table
 * @param alias alias of the table.  If it is not specified, it is
 * default to the qualified name.
 */
public SqlStatementTable(String name, String qualifier, String alias) {
    this.table = name;
    this.qualifier = qualifier;
    this.alias = (alias == null) ? getQualifiedName() : alias;
    parseQualifier();
}

/** Gets the table name without qualifier. */
public String getTableName() {
    return table;
}

/** Gets the table qualifier, i.e. prefix of table name. */
public String getQualifier() {
    return qualifier;
}

/** Sets the table qualifier, i.e. prefix of table name. */
public void setQualifier(String s) {
    String old_qualified_name = getQualifiedName();
    qualifier = s;
    if (alias == null || alias == old_qualified_name) {
        alias = getQualifiedName();
    }
}

/** Gets the qualified table name, i.e. [qualifier].table. */
public String getQualifiedName() {
    return(qualifier == null) ? table : 
    qualifier + "." + table;
}


/** Gets the table alias.  If alias is not defined explicitly,
 *	the table qualified name is used as the alias.
 */
public String getAlias() {
    return alias;
}

/** Sets the table alias explicited defined after the keyword "AS" */
public void setAlias(String s) {
    alias = s;
}

public boolean isSubQuery() {
    return isSubQuery;
}

/** Sets the sub query.
 */
public void setSubQuery(String s) {
    this.subQuery = s;
}

/** Gets the sub query.
 */
public String getSubQuery() {
    return subQuery;
}

/** Return the catalog name, or null if it doesn't exist */
public String getCatalogName() {
    return catalog;
}

/** Return the schema name, or null if it doesn't exist */
public String getSchemaName() {
    return schema;
}

/**
 * Parse a qualifier into it parts which can be
 * catalog.schema. or, schema or null
 */
void parseQualifier() {
    String qualifier = getQualifier();
    if (qualifier == null) {
        return;
    }
    // reverse the tokens
    StringBuffer buffer = new StringBuffer();
    StringTokenizer st = new StringTokenizer(qualifier,".", true);
    while (st.hasMoreTokens()) {
        buffer.insert(0, (String) st.nextToken());
    }
    st = new StringTokenizer(buffer.toString(),".",false);
    if (st.hasMoreTokens()) schema  = (String) st.nextToken();
    if (st.hasMoreTokens()) catalog = (String) st.nextToken();
}

public String toString() {
    return new String("[table name: " + table + 
                      " qualifier: " + qualifier +
                      " alias: " + alias + 
                      " isSubQuery: " + isSubQuery +
                      " subQuery: " + subQuery + "]\n");
}

}
