/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: Field.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:14 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.field;

import com.klg.jclass.field.validate.*;

import com.klg.jclass.util.JCListenerList;
import com.klg.jclass.util.swing.AbstractSpinBox;
import com.klg.jclass.util.swing.JCListModel;
import com.klg.jclass.util.value.JCValueEvent;
import com.klg.jclass.util.value.JCValueListener;
import com.klg.jclass.util.value.JCValueModel;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.undo.UndoableEdit;
import java.awt.*;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.Serializable;
import java.util.Enumeration;
import java.util.Locale;
import java.util.Vector;

/**
 * Note that this class is not a part of the public API and is public
 * as a side-effect of the implementation.<p>
 * This is the "glue" class for JClass Field.  It takes events from the
 * Swing visual components and passes this information on to the generic
 * "validator" that is set on the field.  For clarity we note here that
 * JClass Field "Validators" are actually "validators/formatters/parsers"
 * Also that this class controls the posting of the Validation and Field
 * events.
 *
 *
 *
 */
public class Field extends Object
			       implements Serializable, JCValueListener, ComboBoxModel {

// Enum for State
public static final int VALID     = JCFieldComponent.VALID;
public static final int UNDEREDIT = JCFieldComponent.UNDEREDIT;
public static final int INVALID   = JCFieldComponent.INVALID;

public static final int SHOW_INVALID     = JCInvalidInfo.SHOW_INVALID;
public static final int RESTORE_DEFAULT  = JCInvalidInfo.RESTORE_DEFAULT;
public static final int RESTORE_PREVIOUS = JCInvalidInfo.RESTORE_PREVIOUS;
public static final int CLEAR_FIELD      = JCInvalidInfo.CLEAR_FIELD;

// Enum for case_policy
public static final int AS_IS     = AbstractValidator.AS_IS;
public static final int UPPERCASE = AbstractValidator.UPPERCASE;
public static final int LOWERCASE = AbstractValidator.LOWERCASE;

protected boolean              internalBackgroundChange = false;
protected boolean              internalForegroundChange = false;
protected Color                invalidBackground;
protected Color                invalidForeground;
protected Color                validBackground; // store the initial background color
protected Color                validForeground; // store the inital foregound color

protected boolean              select_on_enter;
protected Object               edit_initial_value;  // for use with cellEditing behaviour
protected Class                edit_class;  // for use with cellEditing behaviour
transient Object               defaultValue;
transient Object               previousValue = null;
protected JCValidator          validator;
protected Locale               current;

transient Object               event_source;

protected Component            vc;

/* input controller */
//protected SpinInterface      spin = null;
/*
protected CellRenderer         renderer = null;
protected CellEditor           editor = null;
*/

protected int                  state;
protected int                  invalidPolicy = SHOW_INVALID;
protected boolean              beepOnInvalid = true;

protected boolean              internal_text_change = false;
protected boolean              has_focus = false;

protected boolean              auto_complete_mode = false;
protected boolean              doing_completion = false;
protected boolean              doing_commit = false;
protected boolean              in_set_selected_item = false;
protected boolean              set_selected_item_called = false;
protected boolean              in_contents_change = false;
protected boolean              upDownPressed = false;

transient JCListenerList       propertyListeners = null;
transient JCListenerList       valueListeners = null;
transient Listeners            listener = new Listeners();
transient EventListenerList    listenerList = new EventListenerList();
protected boolean              okayToBeep = false;
protected boolean              allow_text_change = false;
protected boolean              is_cell_editor = false;
protected boolean              select_all = false;
protected JCValueModel         valueModel = null;
transient TextFieldDocument    document = null;

/**
 * @param vc visual component used to control input;
 * @param validator validator used to validate "value."
 */
public Field(Component vc, JCValidator validator)
{
	super();


 

	this.vc = vc;
	this.validator = validator;

	// get the default value if there is a validator
	if (validator != null) {
		defaultValue = validator.getDefaultValue();
	}

	this.event_source = vc;

	initialize();
}

/**
 * Returns a JTextField given a component. If visual component does
 * not have a JTextField, returns null.
 */
protected JTextField getTextField()
{
	JTextField text = null;

	if (vc instanceof JTextField) {
		text = (JTextField)vc;
		if (text.isEditable() == false) {
			text = null;
		}
	}
	else if (vc instanceof JComboBox) {
		JComboBox combo = (JComboBox)vc;
		if (combo.isEditable() == true) {
			text = (JTextField)combo.getEditor().getEditorComponent();
		}
	}
	else if (vc instanceof AbstractSpinBox) {
		AbstractSpinBox spin = (AbstractSpinBox)vc;
		if (spin.isEditable() == true) {
			text = (JTextField)spin.getEditor().getEditorComponent();
		}
	}

	return(text);
}

private void writeObject(java.io.ObjectOutputStream stream)
    throws java.io.IOException
{
	stream.defaultWriteObject();
}

private void initialize()
{
	
	JTextField text = getTextField();
	boolean isLabel = false;

	if (text == null) {
		isLabel = (vc instanceof JCLabelField);
	}

	if (text != null) {
        document = new TextFieldDocument(new TextFieldContent());
		text.setDocument(document);
	}

	if (text == null && isLabel == false) {
		// Not a supported interface
		throw new ClassCastException("Visual Component not supported by Field");
	}

	invalidBackground = null;
	invalidForeground = null;

	select_on_enter = false;

	// install listeners on the Visual Component and cache initial colors.
	if (text != null) {
		text.addFocusListener(listener);
		text.addKeyListener(listener);
		validBackground = (Color)UIManager.get("TextField.background");
		validForeground = (Color)UIManager.get("TextField.foreground");
	}
	else {
		// no need to add listeners to a JCLabelField
		if (isLabel == false) {
			vc.addFocusListener(listener);
			vc.addKeyListener(listener);
		}
		validBackground = vc.getBackground();
		validForeground = vc.getForeground();
	}

	if (vc instanceof JComboBox && !(vc instanceof JCPopupField)) {
		// install listeners on the Visual Component
		((JComboBox)vc).setModel(this);
	}
}

/**
 * Adds a PropertyChangeListener to the listener list.
 *
 * @param listener  The PropertyChangeListener to be added
 */
public void addPropertyChangeListener(PropertyChangeListener listener)
{
	propertyListeners = JCListenerList.add(propertyListeners, listener);
}

/**
 * Removes a PropertyChangeListener from the listener list.
 * This removes a PropertyChangeListener that was registered
 * for all properties.
 *
 * @param listener  The PropertyChangeListener to be removed
 */
public void removePropertyChangeListener(PropertyChangeListener listener)
{
	propertyListeners = JCListenerList.remove(propertyListeners, listener);
}

/**
 * Adds a listener for changes to the value.
 *
 * @param listener The ValueListener to be added
 */
public void addValueListener(JCValueListener listener)
{
	valueListeners = JCListenerList.add(valueListeners, listener);
}

/**
 * Removes a listener for changes to the value.
 *
 * @param listener The ValueListener to be removed
 */
public void removeValueListener(JCValueListener listener)
{
	valueListeners = JCListenerList.remove(valueListeners, listener);
}

/**
 * Sets a Bean property which wraps the validator, data model, and Field
 * objects properties into one property.
 */
public void setDataProperties(DataProperties data_properties)
{
	this.validator = data_properties.getValidator();

	if (validator != null && vc instanceof JCPopupField) {
		((JCPopupField)vc).setPopupEditor(validator.createPopupComponent());
	}

	if (validator != null) {
		defaultValue = validator.getDefaultValue();
	}

	if (valueModel != null) {
		valueModel.removeValueListener(this);
	}


	if (data_properties.getValueModel() != null) {
		this.valueModel = data_properties.getValueModel();
		this.valueModel.addValueListener(this);
	}

	// Add a value listener to the data model.
	// When the value changes, we need to set the new value in the popup
	if (vc instanceof JCPopupField) {
		valueModel.addValueListener((JCPopupField)vc);
	}

	setValueAndReformat(valueModel.getValue());

	// Set the invalid properties
	setInvalidInfo(data_properties.getInvalidInfo());
}

/**
 * Sets the is_cell_editor property
 * Default is false
 * @param isCellEditor The new value of is_cell_editor
 */
public void setCellEditor(boolean isCellEditor)
{
	this.is_cell_editor = isCellEditor;
}

/**
 * Gets the Bean property which wraps the validator, data model, and Field
 * objects properties into one property.
 */
public DataProperties getDataProperties()
{
	// get the default value if there is a validator
	if (validator != null) {
		defaultValue = validator.getDefaultValue();
	}

	// create the data properties object
 	DataProperties dp = new DataProperties(validator, valueModel,
										   getInvalidInfo());
	return(dp);
}

/**
 * This should be called by the "vc" to inform the component that it has been
 * added.  This is a not ideal, but since there is no "AddListener" we
 * do it this way.
 */
public void addNotify()
{
	
    // Not sure if this is needed, but we'll keep it around in case there are
    // any side effects.
	if (valueModel != null) {
		valueModel.getValue();
	}
	okayToBeep = true;
}

/**
 * Returns the background color used in the visual component if the field
 * is invalid.
 */
protected Color getInvalidBackground()
{
	JTextField text = getTextField();

	if (invalidBackground == null) {
		// Switch to default colors
		if (text != null) {
			return(text.getBackground());
		}
		else {
			return(vc.getBackground());
		}
	}
	return(invalidBackground);
}

/**
 * Sets the background color used in the visual component if the field
 * is invalid.  By default this value is inherited from the background
 * color of the component.
 * @see java.awt.Component#setBackground
 */
protected void setInvalidBackground(Color background)
{
	invalidBackground = background;
	if (state == INVALID) {
		setColors(invalidBackground, invalidForeground);
	}
}

/**
 * Returns the foreground color used in the visual component if the field
 * is invalid.
 */
protected Color getInvalidForeground()
{
	JTextField text = getTextField();

	if (invalidForeground == null) {

		// Switch to default colors
		if (text != null) {
			return(text.getForeground());
		}
		else {
			return(vc.getForeground());
		}
	}
	return(invalidForeground);
}

/**
 * Sets the foreground color used in the visual component if the field
 * is invalid. By default this value is inherited from the foreground
 * color of the component.
 * @see java.awt.Component#setForeground
 */
protected void setInvalidForeground(Color foreground)
{
	invalidForeground = foreground;
	if (state == INVALID) {
		setColors(invalidBackground, invalidForeground);
	}
}

/**
 * Returns the background color used in the visual component if the field
 * is valid. By default, this value is inherited from the background color
 * of the component.
 */
protected Color getValidBackground()
{
	return(validBackground);
}

/**
 * Returns the foreground color used in the visual component if the field
 * is valid. By default, this value is inherited from the foreground color
 * of the component.
 */
protected Color getValidForeground()
{
	return(validForeground);
}

/**
 * Returns the value of beepOnInvalid.
 * @see #setBeepOnInvalid
 */
protected boolean getBeepOnInvalid()
{
	return(beepOnInvalid);
}

/**
 * Sets the value of beepOnInvalid. If beepOnInvalid == true then the field
 * will beep at the user whenever the state is switched to INVALID.
 * @see #setStateInvalid
 */
protected void setBeepOnInvalid(boolean beep)
{
	this.beepOnInvalid = beep;
}

/**
 * Returns the value of invalidPolicy.
 * @see #setInvalidPolicy
 */
protected int getInvalidPolicy()
{
	return(invalidPolicy);
}

/**
 * Sets the invalid policy.  The invalidPolicy governs what happens when
 * a user enters invalid data in a field.
 * <table border>
 * <tr><th>Value</th>    <th>Meaning</th></tr><tr><td>
 * SHOW_INVALID </td><td> Show invalid value in the invalidBackground and
 *                        Foreground colors</td></tr><tr><td>
 * RESTORE_DEFAULT </td><td> Restore the value to the
 *                           defaultValue </td></tr><tr><td>
 * RESTORE_PREVIOUS </td><td> Restore the value to the previous
 *                            valid value </td></tr><tr><td>
 * CLEAR_FIELD </td><td> Clear the field on invalid input </td></tr>
 * </table><p>
 */
protected void setInvalidPolicy(int policy)
{
	if (invalidPolicy == policy) {
		return;
	}

	if (validator == null || valueModel == null) {
		return;
	}

	if (policy != SHOW_INVALID     &&
	    policy != RESTORE_DEFAULT  &&
		policy != RESTORE_PREVIOUS &&
		policy != CLEAR_FIELD)
	{
		throw new IllegalArgumentException("invalid value for invalidPolicy");
	}

	invalidPolicy = policy;
	if (state != INVALID) {
		return;
	}

	// We only have to worry about changing this if we when we switch the
	// current state is INVALID so...
	if (state == INVALID) {
		processFailure();
	}
}

/**
 * Returns the Invalid properties. The Invalid properties are beepOnInvalid,
 * invalidForeground, invalidBackground, and invalidPolicy.
 */
public JCInvalidInfo getInvalidInfo()
{
	Color background = invalidBackground;
	Color foreground = invalidForeground;

	// if invalid colors are null, use default valid colors
	if (invalidForeground == null) {
		foreground = validForeground;
	}

	if (invalidBackground == null) {
		background = validBackground;
	}

	return(new JCInvalidInfo(beepOnInvalid, invalidPolicy,
							 foreground, background));
}

/**
 * Sets the Invalid properties. The Invalid properties are beepOnInvalid,
 * invalidForeground, invalidBackground, and invalidPolicy.
 */
public void setInvalidInfo(JCInvalidInfo info)
{
	setBeepOnInvalid(info.getBeepOnInvalid());
	setInvalidForeground(info.getInvalidForeground());
	setInvalidBackground(info.getInvalidBackground());
	setInvalidPolicy(info.getInvalidPolicy());
}


/**
 * Returns the state of the field.
 * <table border>
 * <tr><th>Value</th>    <th>Meaning</th></tr><tr><td>
 * VALID </td><td> The field is valid </td></tr><tr><td>
 * INVALID </td><td> The field is invalid </td></tr><tr><td>
 * UNDEREDIT </td><td> The field is currently under edit and hence the state is
 * indeterminate </td></tr>
 * </table><p>
 */
public int getState()
{
	return(state);
}

/**
 * Returns a flag indicating the selection status on entering the field.
 * @see #setSelectOnEnter
 */
public boolean getSelectOnEnter()
{
	return(select_on_enter);
}

/**
 * Sets the flag that determines whether of not the Text Component is
 * automatically selected when the component gains focus.  The default value
 * for this is "false."
 */
public void setSelectOnEnter(boolean select)
{
	select_on_enter = select;
}

/**
 * Returns the internal value of the field.
 */
public Object getValue()
{
	if (validator == null || valueModel == null) {
		return(null);
	}
	return(validator.copyValue(valueModel.getValue()));
}

/**
 * Sets the internal value of the field.
 * The big difference between this and setValue() is that setValue ignores
 * new values if they are the same and this method goes through the full
 * formatting as if the value had not changed.  This is useful when one of the
 * components that controls formatting changes.
 */
protected void setValueAndReformat(Object new_value)
{
	
	setValue(new_value, false);
	valueChanged(null);
}

/**
 * Sets the internal value of the field.
 */
public void setClass(Class c)
{
	
	if (valueModel != null) {
		if (valueModel.getValueClass() == c) {
			// we already have a value model that supports the class in
			// question so we simply return
			return;
		}
	}
	// given the value, we get the data type and check the data
	// registry to see if it is a supported type.

	// if the data type is found in the registry, create an instance
	// of the data model and validator and set them.
	if (DataPropertiesRegistry.isRegistered(c)) {
		if (valueModel == null) {
			setValueModel(DataPropertiesRegistry.createValueModel(c));
		}
		if (validator == null) {
			setValidator(DataPropertiesRegistry.createValidator(c));
		}
	}
}

/**
 * Sets the internal value of the field.
 */
public void setValue(Object new_value)
{
	setValue(new_value, true);
}

/**
 * Sets the internal value of the field.
 */
protected void setValue(Object new_value,
						boolean exit_on_detecting_equality)
{
    //
	

	//
	// if there is no data model or validator, we must create them.
	//
	if (valueModel == null || validator == null) {
		
		if (new_value != null) {
			setClass(new_value.getClass());
		}
		else {
			// can't do anything because we have NO information
			return;
		}
	}

	// Data model could still be null.  Be robust - give up!
	if (valueModel == null) {
		return;
	}

	Object current_value = valueModel.getValue();
	// Compare value to internal value
	if (exit_on_detecting_equality &&
		(new_value == current_value ||
		 (validator != null &&
		  validator.compareValues(new_value, current_value))))
    {
		// New value and current value are equal so we return
		// without doing anything

		// Make sure the value is formatted properly if it doesn't have focus.
		internal_text_change = true;
		JTextField text = getTextField();
		if (has_focus == true         &&
			validator.hasEditFormat() &&
			(text != null)            &&
			text.isEditable() == true)
        {
			// Format for edit
			setText(validator.formatForEdit(current_value));
		}
		else {
			// Use normal format
			setText(validator.format(current_value));
		}
		internal_text_change = false;

		// We still need to validate to put the field in the right
		// state. For example, if the state is invalid and the user types
		// a value but then deletes it, the state would always be UNDEREDIT.
		boolean is_valid = validator.validate(current_value);
		if (is_valid == true) {
			setStateValid();
		}
		else {
			setStateInvalid();
		}
		return;
	}

	valueModel.setValue(new_value, exit_on_detecting_equality);
}

/**
 * Used by the Visual Component to see if a call that has been made to its
 * setText() method is permitted.
 */
public boolean getAllowTextChange()
{
	return(allow_text_change);
}

/**
 * Sets the actual text string of the visual component.
 */
protected void setText(String text_string)
{
	
	JTextField text = getTextField();
	if (text == null) {
		if (vc instanceof JCLabelField) {
			allow_text_change = true;
			((JCLabelField)vc).setText(text_string);
			allow_text_change = false;
		}
		else if (vc instanceof JCSpinField) {
			allow_text_change = true;
			((JCSpinField)vc).getRenderer().getComponent((AbstractSpinBox)vc,
														 getValue());
			allow_text_change = false;
		}
		else if (vc instanceof JCPopupField) {
			ListCellRenderer lcr = ((JCPopupField)vc).getRenderer();
			lcr.getListCellRendererComponent(new JList(), text_string, -1,
			                                 true, false);
		}
		return;
	}
	allow_text_change = true;

	// If overstrike is on, we must turn it off because we want to really
	// remove the text instead of replacing it. We must turn it back on after
	// the text is set.
	if (document.getOverstrike() == true) {
		document.setOverstrike(false);
		text.setText(text_string);
		document.setOverstrike(true);
	}
	else {
		text.setText(text_string);
	}

	allow_text_change = false;
}

/**
 * Returns the data model being used by the field.
 */
public JCValueModel getValueModel()
{
	return(valueModel);
}

/**
 * Sets the data model being used by the field.
 */
public void setValueModel(JCValueModel value_model)
{
	
	if (value_model == null) {
		return;
	}

	if (valueModel != null) {
		valueModel.removeValueListener(this);
	}

	// Get the data type so we can search the registry for the correct
	// validator to use
	Class value_class = value_model.getValueClass();

	// If there is a validator already, check to see if the validator
	// supports the data type. If it doesn't, throw an exception.
	if (validator != null) {
		if (validator.isClassSupported(value_class) == false) {
			throw new IllegalArgumentException(
				   "Current validator cannot support the data model");
		}
	}
	else if (DataPropertiesRegistry.isRegistered(value_class)) {
		this.valueModel = value_model;
		// There is no validator so create one
		setValidator(DataPropertiesRegistry.createValidator(value_class));
	}

	this.valueModel = value_model;
	valueModel.addValueListener(this);

	// Add a value listener to the data model.
	// when the value changes, we need to set the new value in the popup
	if (vc instanceof JCPopupField) {
		valueModel.addValueListener((JCPopupField)vc);
	}

	setValueAndReformat(valueModel.getValue());
}

/**
 * Returns the current validator being used by this field of the field.
 */
public JCValidator getValidator()
{
	return(validator);
}

/**
 * Sets the validator being used by the field.
 */
public void setValidator(JCValidator validator)
{
	
	if (validator != null               &&
        (validator != this.validator)   &&
        vc instanceof JCPopupField)
    {
		((JCPopupField)vc).setPopupEditor(validator.createPopupComponent());
	}

	// if there is no data model, just set the validator
	if (valueModel == null) {
		this.validator = validator;
		// get the default value if there is a validator
		if (validator != null) {
			defaultValue = validator.getDefaultValue();
		}
		return;
	}

	Class value_class = valueModel.getValueClass();

	// If the validator does not support the data model, throw exception
	if (validator.isClassSupported(value_class) == false) {
		throw new IllegalArgumentException(
		    "Validator being set does not support the current data model");
	}

	this.validator = validator;

	// get the default value if there is a validator
	if (validator != null) {
		defaultValue = validator.getDefaultValue();
	}

	// Force a validation
	setValueAndReformat(valueModel.getValue());

	if (vc instanceof JCSpinField) {
		((JCSpinField)vc).checkArrowButtons();
	}
}

/**
 * Sets the background and foreground of the component.
 * @param background if background is null, it will use the default background
 * @param foreground if foreground is null, it will use the initial foreground
 */
protected void setColors(Color background, Color foreground)
{
	Component component = null;
	JTextField text = getTextField();

	// Determine if there is a text interface.
	if (text != null) {
		component = text;
	}
	else {
		component = vc;
	}

	// If null, use default background color
	if (background == null) {
		background = validBackground;
	}

	// If null, use default foreground color
	if (foreground == null) {
		foreground = validForeground;
	}

	// Set background and foreground
	internalBackgroundChange = true;
	component.setBackground(background);
	internalBackgroundChange = false;

	internalForegroundChange = true;
	component.setForeground(foreground);
	internalForegroundChange = false;
}

protected boolean restoringPrevious = false;

/**
 * Called either from after a failed parse or after a failed.
 */
protected void processFailure()
{
	
	// We have just experienced a failure in validation so now we must
	// handle it according to the invalidPolicy
	switch (invalidPolicy) {
	case SHOW_INVALID:
		setStateInvalid();
		break;
	case RESTORE_DEFAULT:
		// Set the default value and go through programmatic
		// validation mechanism
		setValue(defaultValue);
		break;
	case RESTORE_PREVIOUS:
		// Restore the current value mechanism to ensure that it is valid
		setValue(previousValue);
		break;
	case CLEAR_FIELD:
		// Clear the field
		setValue(null);
		break;
	}
}

/**
 * Does everything necessary to make the state invalid, including swapping
 * colors.
 */
protected void setStateInvalid()
{
	

	if (validator == null || valueModel == null) {
		return;
	}

	if (state != INVALID) {
		if (okayToBeep && beepOnInvalid) {
		    
			vc.getToolkit().beep();
		}

		// Switch to invalid colors
		setColors(invalidBackground, invalidForeground);

		// Post state change event
		int old_state = state;
		state = INVALID;
		PropertyChangeEvent event =
		    new PropertyChangeEvent(vc, "state", new Integer(old_state),
									new Integer(INVALID));
		Enumeration e = JCListenerList.elements(propertyListeners);
		while (e.hasMoreElements()) {
			PropertyChangeListener listener =
			    (PropertyChangeListener)e.nextElement();
			listener.propertyChange(event);
		}
	}
}

/**
 * Does everything necessary to make the state valid, including swapping
 * colors.
 */
protected void setStateValid()
{
	
	if (state != VALID) {
		// Switch to default colors
		// we check if the invalid colors are set since if they are not then
		// we don't have to change the colors
		setColors(null, null);
		// Post state change event
		int old_state = state;
		PropertyChangeEvent event =
		    new PropertyChangeEvent(vc, "state", new Integer(old_state),
									new Integer(VALID));
		Enumeration e = JCListenerList.elements(propertyListeners);
		while (e.hasMoreElements()) {
			PropertyChangeListener listener =
			    (PropertyChangeListener)e.nextElement();
			listener.propertyChange(event);
		}
		state = VALID;
	}
}

/**
 * Does everything necessary to make the state "under edit," including
 * swapping colors.
 */
protected void setStateUnderEdit()
{
	
	if (state == INVALID) {
		// Switch to default colors
		setColors(null, null);
	}
	if (state != UNDEREDIT) {
		// Post state change event
		int old_state = state;
		PropertyChangeEvent event =
		    new PropertyChangeEvent(vc, "state", new Integer(old_state),
									new Integer(UNDEREDIT));
		Enumeration e = JCListenerList.elements(propertyListeners);
		while (e.hasMoreElements()) {
			PropertyChangeListener listener =
			    (PropertyChangeListener)e.nextElement();
			listener.propertyChange(event);
		}
	}
	state = UNDEREDIT;
}

/**
 * Returns the visual component currently being used.
 */
public Component getComponent()
{
	return(vc);
}

/**
 * Returns the Object that is being used as the source of posted events.
 */
public Object getEventSource()
{
	return(event_source);
}

/**
 * Sets the Object that is being used as the source of posted events.
 */
public void setEventSource(Object o)
{
	this.event_source = o;
}

/**
 * Commits the current field under edit.
 */
public void commitEdit()
{
	commitEdit(null);
}

/**
 * Commits the current field under edit. If a combo box and the popup is
 * visible, make it go away.
 */
public void commitEdit(AWTEvent e)
{
    doCommitEdit(e);

    JComboBox combo = (vc instanceof JComboBox) ? (JComboBox)vc : null;
	if (combo != null) {
        if (combo.isPopupVisible()) {
            combo.hidePopup();
            combo.showPopup();
        }
	    combo.setPopupVisible(false);
        
	}
}

/**
 * Commits the current field under edit.
 */
protected void doCommitEdit(AWTEvent e)
{
	JTextField text = getTextField();

	

	if (validator == null || valueModel == null) {
		return;
	}
    turnOffAutoCompleteMode();
	if (state != UNDEREDIT) {
		// We want to select all anyway if a commit is called
		// if there actually was stuff to commit then we do it later
		// since we need to wait until the text has been written into
		// the component; BUT since we bail here we have to do a separate
		// highlight
		if (text != null && select_on_enter == true) {
			text.setSelectionStart(0);
			text.setSelectionEnd(text.getText().length());
		}
		return;
	}

    doing_commit = true;
	String current_text = null;
	if (text != null) {
		current_text = text.getText();
	}

	Object returned = null;
	boolean parse_failed = false;
	if (text == null || current_text == null || current_text.length() == 0){
		// Short cut so every parseAndValidate() doesn't have to have code to
		// handle null's
		returned = null;
	}
	else {
		try {
			returned = validator.parse(valueModel.getValueClass(),
									   text.getText());
		}
		catch (FieldParseException fpe) {
			parse_failed = true;
		}
	}

	if (parse_failed) {
		// Process the parse failure according to the InvalidPolicy
		// set the previousValue to the current value so that it can be
		// restored if the invalid policy wants it to be
		previousValue = valueModel.getValue();
		processFailure();
        doing_commit = false;
		return;
	}

	// Attempt to set the new value;  note this may still fail because
	// of the validation that occurs inside setValue
	setValue(returned);

	// We need to force reformatting in the case where invalid policy is set
	// to anything BUT SHOW_INVALID. If the field is currently showing a
	// valid value, and a invalid value is entered, the invalid policy
	// may set the value to the same value as the previous value.
	// If both old and new values are the same, the field will not
	// reformat. By calling valueChanged, a reformat will always occur
	if (invalidPolicy != SHOW_INVALID) {
		valueChanged(null);
	}
    doing_commit = false;
}

/**
 * This method is internal. You should not have to call it.
 */
public Component getVisualComponent()
{
    return(vc);
}

/**
 * Invoked before the field's value is changed. This routine invokes the
 * AbstractValidator to check if a particular change is allowed.
 * The event's values can be modified via its setXXX methods.
 */
public void textValueChangeBegin(com.klg.jclass.field.validate.TextEvent e)
{
	if (validator == null || valueModel == null) {
		return;
	}

	

	if (internal_text_change == true) {
		return;
	}

	if (validator != null) {
		// Call the validator to process the keystroke
		validator.changeText(e);
	}

	//	Beep if allowChange is false and the beepOnInvalid property is true
	//  and if addNotify has been called for the component (we don't beep
	//  before add notify to allow the developers to mess around with the
	//  values before it's shown
	if (e.getAllowChange() == false && beepOnInvalid && okayToBeep) {
		
		vc.getToolkit().beep();
	}
	
}

/**
 * Invoked after the field's value is changed. Any changes made to the event
 * are ignored.
 */
public void textValueChangeEnd(com.klg.jclass.field.validate.TextEvent e)
{
	if (internal_text_change == true) {
		return;
	}
}

/**
 * Invoked before the field's cursor is moved.
 * The event's values can be modified via its setXXX methods.
 */
public void textCursorMoveBegin(TextCursorEvent e)
{
	
	if (validator != null) {
		validator.setCursor(e);
	}
}

		//
		// ComboBoxModel interface
		//

/**
 * Returns the number of items in the pick list.
 */
public int getSize()
{
	if (validator == null || validator.getPickList() == null) {
		return(0);
	}
	return(validator.getPickList().getSize());
}

/**
 * Returns the element at a given index in a pick list.
 */
public Object getElementAt(int index)
{
	if (validator == null || validator.getPickList() == null) {
		return(null);
	}
	return(validator.format(validator.getPickList().getElementAt(index)));
}

public void addListDataListener(ListDataListener l)
{
	listenerList.add(ListDataListener.class, l);
}

public void removeListDataListener(ListDataListener l)
{
	listenerList.remove(ListDataListener.class, l);
}

/**
 * Called when the combo box's item list changes (which happens when
 * the picklist changes).
 *
 * @param source The ListModel that changed, typically "this".
 * @param index0 One end of the new interval (ignored in this context)
 * @param index1 The other end of the new interval (ignored in this context)
 * @see EventListenerList
 * @see DefaultListModel
 */
protected void fireContentsChanged(Object source, int index0, int index1)
{
    Object[]        listeners = listenerList.getListenerList();
	ListDataEvent   e         = null;

    in_contents_change = true;
	for (int i = listeners.length - 2; i >= 0; i -= 2) {
	    if (listeners[i] == ListDataListener.class) {
		    if (e == null) {
		        e = new ListDataEvent(source, ListDataEvent.CONTENTS_CHANGED,
                                      index0, index1);
		    }
		    ((ListDataListener)listeners[i+1]).contentsChanged(e);
	    }
	}
    in_contents_change = false;
}

/**
 * Restore the picklist to its original value before AutoRefinement changed
 * it.
 */
protected void restorePickList()
{
    JCComboField combo = (vc instanceof JCComboField) ? (JCComboField)vc : null;
    if (combo != null && combo.isAutoComplete() && combo.isAutoRefinement()) {
        if (validator != null) {
            
            combo.restoreOriginalPickList();
         }
    }
}

/**
 * Set an item from the picklist on the text field
 */
public void setSelectedItem(Object anItem)
{
    
	set_selected_item_called = true;
	if (anItem == null || validator == null) {
		return;
	}
    in_set_selected_item = true;

    boolean string_validator = (validator instanceof JCStringValidator &&
                                !(validator instanceof JCIPAddressValidator));
    boolean has_mask = (string_validator &&
                        ((JCStringValidator)validator).getMask() != null &&
		                ((JCStringValidator)validator).getMask().length() > 0);
    boolean format_item = true;
    if (doing_completion && string_validator && has_mask) {
		format_item = false;
    }
    

	// Need to find the value in the picklist in order to parse properly.
	// The value passed to this method is the display formatted value which may
    // not be a value that can be parse properly by a validator. i.e. $100.
 	ListModel list = validator.getPickList();
    boolean is_equal = false;
	if (list != null) {
		for (int i = 0; i < list.getSize(); i++) {
			Object item = list.getElementAt(i);
            is_equal = format_item ? anItem.equals(validator.format(item))
                                   : anItem.equals(item);
            
 			if (is_equal) {
				anItem = list.getElementAt(i).toString();
				break;
			}
		}
	}
    

    // If nothing in the picklist matches a formatted item, parse the given
    // item and set what we've got.
    if (format_item && !(is_equal && string_validator && has_mask)) {
        Object po = null;
        try {
			if (valueModel != null) {
                po = validator.parse(valueModel.getValueClass(),
                                     (String)anItem);
			}
        }
        catch (FieldParseException fpe) {}
        anItem = po;
        
    }

    setValue(anItem);

    // In AutoComplete mode, we may need to restore the picklist
    if (!doing_completion && !upDownPressed) {
        restorePickList();
    }
    upDownPressed = false;

    // If up/down arrows are pressed, the popup does not getting refreshed
    // properly because the UI expects the model to do it.  However it gives
    // no hooks for doing so.  Calling setPopupVisible() when its already
    // visible causes a refresh to happen and thus solves this problem.  Note
    // that calling fireContentsChanged will also do the trick but it seems
    // to be overkill here.
    if (vc instanceof JComboBox) {
        JComboBox combo = (JComboBox)vc;
        if (combo.isPopupVisible()) {
            combo.setPopupVisible(true);
        }
    }
    in_set_selected_item = false;
    
}

/**
 * Get the currently selected item (and format accordingly)
 */
public Object getSelectedItem()
{
    Object  obj;

	if (validator == null) {
		return("");
	}
    obj = getValue();
    
	if (state == UNDEREDIT && !auto_complete_mode) {
		obj = validator.formatForEdit(obj);
	}
	else {
		obj = validator.format(obj);
	}
    
    return(obj);
}

/**
 * Calculate auto_complete_mode value.
 */
protected void calcAutoCompleteMode(JCComboField combo)
{
    auto_complete_mode = false;
    if (!doing_commit                   &&
        combo != null                   &&
        validator.getPickList() != null &&
        validator instanceof JCStringValidator)
    {
        auto_complete_mode = combo.isAutoComplete();
    }
}

/**
 * Turn off auto_complete_mode.  Remove any highlighting.  Restore the
 * picklist.
 */
protected void turnOffAutoCompleteMode()
{
	JTextField text = getTextField();

    // Remove any remaining selection
    if (auto_complete_mode && text != null) {
        text.setSelectionStart(text.getCaretPosition());
        text.setSelectionEnd(text.getCaretPosition());
    }
    restorePickList();
    auto_complete_mode = false;
    
}

		//
		// ValueListener interface
		//

public void valueChanging(JCValueEvent event)
{
    //
	
    if (event != null) {
        
    }
    else {
        
    }

	if (event != null) {
		// Create new event with visual component as source object
		JCValueEvent new_event = new JCValueEvent(vc, event.getOldValue(),
												  event.getNewValue());

		fireValueChangingEvent(new_event);

		// If user disallowed change, return
		if (new_event.getAllowChange() == false) {
			event.setAllowChange(false);
			// If the change was disallowed, we must reformat the old value
			valueChanged(null);
			return;
		}
		else {
			// Check to see if user changed the new value.  If we are doing
            // auto completion then we don't want to allow the user to change
            // the value.
			if (new_event.getNewValue() != event.getNewValue() &&
                !doing_completion)
            {
				event.setNewValue(new_event.getNewValue());
			}
		}
	}

	// Get the value stored in the data model
	Object current_value = event.getOldValue();
	Object new_value = event.getNewValue();
	boolean is_valid;

	if (validator != null) {
		is_valid = validator.validate(new_value);
		
	}
	else {
		
		is_valid = false;
	}

	if (is_valid) {
		// Set a copy of the new value to the data model
		// NOTE a copy is only made if the value to be copied is mutable.
		// AND the new_value and the current_value are different objects
		if (valueModel != null  &&
            validator != null   &&
			new_value != current_value)
        {
			event.setNewValue(validator.copyValue(new_value));
		}
		setStateValid();
	}
	else {
		// Handle the different invalid policies
		switch (invalidPolicy) {
		case SHOW_INVALID:
			// We allow the set new value to persist but we consider it to
			// be invalid
			setStateInvalid();
			break;
		case RESTORE_DEFAULT:
			// Set the default value and go through programmatic
			// validation mechanism
			new_value = defaultValue;
			break;
		case RESTORE_PREVIOUS:
			// Restore the current value.  Mechanism to ensure that it is valid.
			new_value = event.getOldValue();

			// Check old value for validation
			if (valueModel != null &&
                validator != null &&
                validator.validate(new_value) &&
				event.getNewValue() != current_value)
            {
				new_value = validator.copyValue(new_value);
			}
			else {
				// Validation failed on old value. We set the state to invalid
                // anyway since the previous value was invalid
				setStateInvalid();
			}
			break;
		case CLEAR_FIELD:
			// Clear the field
			boolean returned = validator.validate(null);
			new_value = null;
			if (returned) {
				setStateValid();
			}
			else {
				setStateInvalid();
			}
			break;
		}

		event.setNewValue(new_value);
	}
}

public void valueChanged(JCValueEvent event)
{
    //
	
    if (event != null) {
        
    }
    else {
        
    }

	if (event != null) {
		// Create new event with visual component as source object
		JCValueEvent new_event = new JCValueEvent(vc, event.getOldValue(),
												  event.getNewValue());
		fireValueChangedEvent(new_event);
	}

	// Store previous value in case we ever have to restore in at any
	// arbitrary future date
	Object current_value = null;
	if (event == null) {
		// This was invoked programatically
		current_value = valueModel.getValue();
	}
	else {
		current_value = event.getNewValue();
		previousValue = event.getOldValue();
	}

	//
	// Format the object and display it
	//

	internal_text_change = true;

	JTextField text = getTextField();

	int pos = 0;
	int select_start = 0;
	int select_end = 0;
	int text_length = 0;

	if (text != null) {
		// Save selected region
		select_start = text.getSelectionStart();
		select_end = text.getSelectionEnd();
		// Save text length
		text_length = text.getText().length();
	}

	if (validator != null) {
		if (has_focus == true &&
			validator.hasEditFormat() &&
			(text != null) && text.isEditable() == true)
        {
			// Format for edit
			setText(validator.formatForEdit(current_value));
		}
		else {
			// Use normal format
			setText(validator.format(current_value));
		}
	}

	// Restore cursor position if the length of text hasn't changed
	if (text != null &&
        text_length == text.getText().length() &&
        !auto_complete_mode)
    {
		if (select_start == select_end && select_start == pos) {
			// We have no selection so just restore the cursor position
			text.setCaretPosition(pos);
		}
		else {
			// We have a selection so restore it.  The cursor position will
			// automatically be set by this action
			text.select(select_start, select_end);
		}
	}

	internal_text_change = false;
	if (vc instanceof JCSpinField) {
		((JCSpinField)vc).checkArrowButtons();
	}
}

/**
 * Fires a valueChanging event to the JCValuelisteners.
 */
protected void fireValueChangingEvent(JCValueEvent event)
{
    
	if (event == null || in_contents_change) {
		return;
	}

	Enumeration e = JCListenerList.elements(valueListeners);
    while (e.hasMoreElements()) {
        JCValueListener listener = (JCValueListener)e.nextElement();
        listener.valueChanging(event);
    }
}

/**
 * Fires a valueChanged event to the JCValuelisteners.
 */
protected void fireValueChangedEvent(JCValueEvent event)
{
    
	if (event == null || in_contents_change) {
		return;
	}

	Enumeration e = JCListenerList.elements(valueListeners);
    while (e.hasMoreElements()) {
        JCValueListener listener = (JCValueListener)e.nextElement();
        listener.valueChanged(event);
    }
}

//
// Put all the listeners into an inner class
//

class Listeners extends Object
				implements FocusListener, KeyListener, Serializable
{
    //
	// Focus Listener Interface Implementation
	//

	public void focusGained(FocusEvent e)
	{
		

		if (is_cell_editor == true) {
			return;
		}

		has_focus = true;
		if (validator == null || valueModel == null) {
			return;
		}

		//
		// The rest of focus gain code only applies to editable text components
		//
		JTextField text = getTextField();
		if (text == null) {
			return;
		}
		if (text.isEditable() == false) {
			return;
		}

		// If the state is invalid, then we don't use the edit format,
		// instead we simply let the user edit what s/he left in it.
		if (validator.hasEditFormat() && (state != INVALID)) {
			// If the field has an edit format then switch to it.
			internal_text_change = true;
			setText(validator.formatForEdit(valueModel.getValue()));
			internal_text_change = false;
		}

		// Highlight the text if "select on enter"
		if (select_on_enter == true) {
			text.setSelectionStart(0);
			text.setSelectionEnd(text.getText().length());
		}
	}

	public void focusLost(FocusEvent e) {
		
		if (is_cell_editor == true) {
			return;
		}

		has_focus = false;
		if (validator == null || valueModel == null) {
			return;
		}

		if (state == UNDEREDIT) {
			
			commitEdit(e);
		}
		else if (state != INVALID && validator != null &&
				 validator.hasEditFormat() == true)
        {
			// We set the value to itself to cause it to update in the correct
			// manner
			
			setValue(valueModel.getValue());
		}
        else {
			
			
        }
	}

    //
    // Key Listener Interface Implementation
	//

	/**
	 * Invoked when a key has been typed. This event occurs when a key press
	 * is followed by a key release. (Not true in all cases except the
	 * general.)
	 */
	public void keyTyped(KeyEvent e) {}

	/**
	 * Invoked when a key has been pressed.
	 */
	public void keyPressed(KeyEvent e)
	{
		int kc;
		Object new_obj = null;
		boolean cancelled = false;

		if (validator == null || valueModel == null) {
			return;
		}

		Object internal_value = valueModel.getValue();
		JTextField text = getTextField();

		if (text != null) {
		    
		}

		// Check to see if the popup wants to process key events first
		if (vc instanceof JCPopupField &&
			((JCPopupField)vc).isPopupVisible() == true)
		{
		    if (((JCPopupField)vc).getPopupEditor() == null) {
			    // Editor might be null inside the popped up editor
				return;
			}
			boolean processed =
			    ((JCPopupField)vc).getPopupEditor().processKeyEvent(e);
			if (processed == true) {
			    return;
			}
		}

        JComboBox combo = (vc instanceof JComboBox) ? (JComboBox)vc : null;

		kc = e.getKeyCode();
		switch(kc) {
		case KeyEvent.VK_ESCAPE:
			if (combo != null && combo.isPopupVisible()) {
	            combo.setPopupVisible(false);
				e.consume();
				return;
			}

			//
			// Eliminate edits
			//
			cancelled = true;
			if (state == UNDEREDIT || state == INVALID) {
				// Post state change event
				int old_state = state;
				PropertyChangeEvent event =
                    new PropertyChangeEvent(vc, "state", new Integer(old_state),
										    new Integer(VALID));
				Enumeration enum = JCListenerList.elements(propertyListeners);
				while (enum.hasMoreElements()) {
					PropertyChangeListener listener =
                        (PropertyChangeListener)enum.nextElement();
					listener.propertyChange(event);
				}
				state = VALID;

				new_obj = internal_value;
				e.consume();
			}
			break;
		case KeyEvent.VK_ENTER:
			commitEdit(e);
			e.consume();
			return;
        case KeyEvent.VK_UP:
        case KeyEvent.VK_DOWN:
            upDownPressed = true;
            return;
		default:
			// if the user typed a key we switch to edit mode
			if (state != UNDEREDIT) {
				setStateUnderEdit();
			}
		}

		if (new_obj != null) {
			setValue(new_obj);
		}

		if (text != null && cancelled == true && select_on_enter == true) {
			text.setSelectionStart(0);
			text.setSelectionEnd(text.getText().length());
		}
	}

	/**
	 * Invoked when a key has been released.
	 */
	public void keyReleased(KeyEvent e) {}
} // Listener inner class

/**
 * Document model to be used by Field beans. Adds overstrike functionality
 * required for mask fields.
 */
class TextFieldDocument extends PlainDocument implements CaretListener
{
    protected SelectedItem  selected = null;
    protected boolean       doingRemove = false;
	protected int           currentCursorPosition = 0;
	protected int           lastCursorPosition = 0;

	public TextFieldDocument(Content c)
	{
		super(c);
		JTextField text = getTextField();
		text.addCaretListener(this);
		currentCursorPosition = text.getCaretPosition();
	}

	public void setOverstrike(boolean overstrike)
	{
		((TextFieldContent)getContent()).setOverstrike(overstrike);
	}

	public boolean getOverstrike()
	{
		return(((TextFieldContent)getContent()).getOverstrike());
	}

    /**
     * Given a string, find the longest item in the prefix list which matches it.
     */
    protected String getMatchedPrefix(JCComboField combo, String item_text)
    {
        int             prefix_size     = 0;
        int             prefix_index    = -1;
        String []       prefix_list     = combo.getPrefixList();

        // Find the longest possible match in the prefix list
        if (prefix_list != null) {
            for (int j = 0; j < prefix_list.length; j++) {
                String prefix = prefix_list[j];
                if (prefix == null || prefix.length() == 0) {
                    continue;
                }
                int length = prefix.length();
                if (item_text.startsWith(prefix)) {
                    // We have match. Is it longer than the previous match?
                    if (length > prefix_size) {
                        prefix_size  = length;
                        prefix_index = j;
                    }
                }
            }
        }
        if (prefix_size > 0) {
            // Return matched item
            return(prefix_list[prefix_index]);
        }

        // No match
        return(null);
    }

    /**
     * Given an initial string, this method matches that string with the
     * picklist (matching each string from the beginning) and returns
     * the completion, i.e. the remaining portion of the string.
    */
    protected SelectedItem getPickListCompletion(JCComboField combo,
                                                 String entry,
                                                 String parsed,
                                                 int start_index,
                                                 int cur_position)
    {
        
	    if (validator == null               ||
            validator.getPickList() == null ||
            entry == null                   ||
            start_index < 0)
        {
		    return(null);
	    }

  	    ListModel list = combo.isAutoRefinement() ? combo.getOriginalPickList()
                                                  : validator.getPickList();
        String compare = getOverstrike() ? parsed : entry;
        
	    if (list != null && compare != null) {
		    for (int i = start_index; i < list.getSize(); i++) {
			    Object item      = list.getElementAt(i);
                String item_text = item.toString();
                
                if (item_text != null && item_text.length() > 0) {
                    // Always match if compare string is empty
                    if (compare.length() == 0) {
                        return(new SelectedItem(item, i));
                    }

                    // First check if item begins with a prefix.
                    // Do not use prefixes if there is a mask
                    String prefix = getMatchedPrefix(combo, item_text);
                    if (prefix != null && prefix.length() != 0) {
                        StringBuffer buffer = new StringBuffer(prefix);
                        buffer.append(compare);
                        if (item_text.startsWith(buffer.toString())) {
                            return(new SelectedItem(item, i, prefix.length()));
                        }

                        // If this item matched one of the prefixes then we
                        // don't consider this item for a normal match.
                        continue;
                    }

                    // If prefix failed, check item itself
                    if (item_text.startsWith(compare)) {
                        return(new SelectedItem(item, i));
                    }
                }
		    }
	    }
        return(null);
    }

    /**
     * Match the buffer against the combo's picklist and set the
     * appropriate item as the combobox's selected item.
     */
    protected void doCompletion(JCComboField combo, String buffer,
                                int cur_position)
    {
        

        String parsed = null;
        if (getOverstrike()) {
            // Mask.  Parse the value from the mask
            Object value = null;
            try {
                value = validator.parse(valueModel.getValueClass(), buffer);
            }
            catch (FieldParseException fpe) {}
            parsed = (value == null) ? null : value.toString();
        }

        selected = null;
        JCListModel new_list = null;
        if (combo.isAutoRefinement()) {
            // Do picklist refinement here.  Get all completions.
            new_list = new JCListModel();
            SelectedItem sel_obj = getPickListCompletion(combo, buffer, parsed,
                                                         0, cur_position);
            while (sel_obj != null) {
                if (selected == null) {
                    selected = sel_obj;
                }
                new_list.addElement(sel_obj.item);
                sel_obj = getPickListCompletion(combo, buffer, parsed,
                                                sel_obj.index + 1,
                                                cur_position);
            }

            // Add the current entry at the end (if it doesn't match
            // the current selected object)
            Object new_element = null;
            if (getOverstrike() && parsed != null) {
                new_element = parsed.length() == 0 ? null : parsed;
            }
            else if (buffer != null) {
                new_element = buffer.length() == 0 ? null : buffer;
            }
            if (selected == null ||
                (new_element != null && !new_element.equals(selected.item)))
            {
                new_list.addElement(new_element);
            }

            // If only element is the last item, select it
            int length = new_list.getSize();
            if (selected == null && length > 0) {
                selected = new SelectedItem(new_list.getElementAt(length - 1),
                                            0);
            }
        }
        else {
            // Get the first completion.
            selected = getPickListCompletion(combo, buffer, parsed,
                                             0, cur_position);
        }

        String completion =
            (selected == null) ? null : validator.format(selected.item);
        if (selected != null && completion != null) {
            // We have a match.  Save positions to highlight.
            selected.start_pos =
                getOverstrike() ? cur_position : buffer.length();
            selected.end_pos = completion.length();
            if (selected.prefix_length > 0) {
                selected.end_pos -= selected.prefix_length;
            }
            selected.end_pos = Math.max(selected.start_pos, selected.end_pos);
            selected.parsed = parsed;
            
            
            

            // We set the completed item for the combo box.  This
            // is needed so that the popup will highlight the
            // current autocompletion candidate.  It has a side
            // effect of setting the value.
            doing_completion = true;
            

            if (new_list != null) {
                combo.setTemporaryPickList(new_list);
            }

            // Need to intialize/reconfigure popup or the items in the list
            // may not match the popup size.
            if (combo.isAutoSuggest()) {
                if (combo.isAutoRefinement()) {
                    combo.setPopupVisible(false);
                }
                combo.setPopupVisible(true);
            }

            combo.setSelectedItem(selected.item);
			// In JDK 1.4 and later, if the item is the same as before
			// nothing is done.  We NEED to call setSelectedItem() in order
			// do things like set the append selection string
			if (!set_selected_item_called) {
				setSelectedItem(selected.item);
			}
			set_selected_item_called = false;

            // Refresh the popup (or it won't select the current
            // item -- even though it has been selected above!)
            if (combo.isPopupVisible() || combo.isAutoSuggest()) {
                combo.setPopupVisible(true);
            }

            doing_completion = false;
            
        }
    }

    /**
     * Do autocomplete selection if appropriate.  Note that selected.start_pos
     * and selected.end_pos are set in doCompletion.
     */
    protected void doAutoCompleteSelection(JTextField text, JCComboField combo)
	{
        if (!auto_complete_mode     ||
            text == null            ||
            combo == null           ||
            !combo.isAutoAppend()   ||
            selected == null        ||
            selected.start_pos >= selected.end_pos)
        {
            return;
        }

        // We must be in auto complete mode to be here and we thus know that
        // validator must be a string validator
        JCStringValidator sv = (JCStringValidator)validator;

        String new_text = text.getText();
        if (getOverstrike()) {
            String cur_text   = sv.format(selected.parsed);
            int cur_last_pos  = sv.getLastValidInputCharPosition(cur_text);
            int item_last_pos = sv.getLastValidInputCharPosition(new_text);
            if (cur_last_pos >= item_last_pos) {
                text.getCaret().setDot(sv.getNextCursorPosition(item_last_pos,
                                                                true));
                return;
            }
        }
        if (new_text != null && selected.end_pos <= new_text.length()) {
            text.select(selected.start_pos, selected.end_pos);
        }
    }

	public void insertString(int offs, String str, AttributeSet a)
	    throws BadLocationException
    {
        if (validator == null || valueModel == null || in_contents_change) {
			return;
		}

        //
		
		

		JTextField text = getTextField();
		if (text == null) {
			
			return;
		}

		// Turn on overstrike mode only if there is a mask.
		if (validator instanceof JCStringValidator &&
			((JCStringValidator)validator).getMask() != null &&
			((JCStringValidator)validator).getMask().length() > 0)
        {
			setOverstrike(true);
		}
		boolean allow_change = false;

		Caret caret = text.getCaret();
		int p0 = Math.min(caret.getDot(), caret.getMark());
		int p1 = Math.max(caret.getDot(), caret.getMark());

		int first_cursor = validator.getFirstValidCursorPosition();
		if (p0 < first_cursor) {
			p0 = first_cursor;
		}
		TextEvent te = new TextEvent(text.getText(), p0 , p1, str, false);
        

		// Post begin event
		textValueChangeBegin(te);
		allow_change = te.getAllowChange();

		// If there is text selected, remove it first when inserting text
		String selected_text = text.getSelectedText();
		if (allow_change == true    &&
            selected_text != null   &&
		    selected_text.length() != 0)
        {
			try {
				remove(te.getStartPosition(), selected_text.length());
			}
			catch (BadLocationException e) {}
		}

		// Set insertion point
		caret.setDot(te.getStartPosition());
        if (allow_change == true && !doing_completion) {
            super.insertString(caret.getDot(), te.getText(), a);
        }

	    JCComboField combo =
            (vc instanceof JComboBox) ? (JCComboField)vc : null;
        calcAutoCompleteMode(combo);

		if (auto_complete_mode && allow_change == true) {
            if (!doing_completion) {
                if (in_set_selected_item) {
                    // This was set by the user.  Don't complete.
                    return;
                }

                // The text field now has the string for AutoCompletion.  Note
                // that doCompletion, if it succeeds sets doing_completion to
                // true and calls the combobox's setSelectedItem method.  This
                // will eventually call this routine again resulting in the code
                // below being executed.  Thus we must return now and allow
                // the insertion to happen later. Highlighting, if desired,
                // will also happen at that point.
                String new_text = text.getText();
                if (new_text != null && new_text.length() != 0) {
                    doCompletion(combo, new_text, te.getEndPosition());
                }
                return;
            }

            // Insert the string -- note that doing_completion must be true
            // to get here.
            boolean autoAppend = combo.isAutoAppend() && !doingRemove;
            String insert_text = te.getText();
            if (getOverstrike() && selected != null) {
                insert_text =
                    autoAppend ? selected.item.toString() : selected.parsed;
            }
            if (insert_text != null) {
                if (selected != null &&
                    selected.prefix_length > 0 &&
                    insert_text.length() >= selected.prefix_length)
                {
                    insert_text = insert_text.substring(selected.prefix_length);
                }
                if (getOverstrike() && selected != null) {
                    insert_text = validator.format(insert_text);
                }
                if (!autoAppend && selected != null) {
                    if (getOverstrike()) {
                        // Mask.  Insert formatted string
                        super.insertString(caret.getDot(), insert_text, a);
                        caret.setDot(selected.start_pos);
                    }
                    else if (insert_text.length() >= selected.start_pos) {
                        // Insert string only up to the current cursor
                        String s = insert_text.substring(0, selected.start_pos);
                        super.insertString(caret.getDot(), s, a);
                    }
                }
                else {
                    // Insert string -- If a match was found above, this will
                    // be the entire completed string.
                    super.insertString(caret.getDot(), insert_text, a);
                }
            }

            // Add highlighting and clean up.
            if (autoAppend) {
                doAutoCompleteSelection(text, combo);
            }
            setStateUnderEdit();
		}
	}

	public void remove(int offs, int len) throws BadLocationException
	{
		if (validator == null || valueModel == null || in_contents_change) {
			return;
		}

		
		
		

		JTextField text = getTextField();
		if (text == null) {
			return;
		}

		TextEvent te = new TextEvent(text.getText(), offs , offs + len, "",
                                     true);

		// Post begin event
		textValueChangeBegin(te);

		String ins_text = te.getText();
        int ins_length = (ins_text == null) ? 0 : ins_text.length();
		if (getOverstrike() == true) {
			text.setCaretPosition(te.getStartPosition());
			int text_cur = text.getCaretPosition();

			// workaround. If there are literals in the beginning and we select
			// the entire text, the cursor moves to the next
			// valid position. However, the text passed back from the event
			// expects the cursor to be before the literal.  Therefore, we must
			// remove the literals from the text passed back since they
			// have not been removed.
			int diff = 0;
			if (offs < text_cur &&
			    (ins_text != null && ins_length >=1))
            {
				diff = text_cur - offs;
				ins_text = ins_text.substring(diff);
			}

			// when overstrike is on, move the cursor to the new position.
			super.insertString(text_cur, ins_text, null);
			text.getCaret().setDot((te.getStartPosition() + diff));
		}
		else {
			super.remove(offs, len);
		}

	    JCComboField combo =
            (vc instanceof JComboBox) ? (JCComboField)vc : null;
        calcAutoCompleteMode(combo);

        if (!auto_complete_mode  ||
            !te.getAllowChange() ||
            in_set_selected_item)
        {
            return;
        }

        // We must be in auto complete mode to be here and we thus know that
        // validator must be a string validator
        JCStringValidator sv = (JCStringValidator)validator;

        // Autocomplete if we're not just deleting the selected text or
        // we're not at the end of inputted text.
        int end_of_region = offs + len;
        String new_text   = (text == null) ? null : text.getText();
        int new_length    = (new_text == null) ? 0 : new_text.length();
        int cur_pos       = text.getCaret().getDot();
        int last_pos      = sv.getLastValidInputCharPosition(new_text);
		if ((new_length > 0 || combo.isAutoRefinement())      &&
            !doing_completion                                 &&
            (cur_pos < last_pos || combo.isAutoRefinement())  &&
            (selected != null && !(selected.start_pos == offs &&
                                   selected.end_pos == end_of_region)))
        {
            // The combo box has a string for AutoCompletion.  Note
            // that doCompletion, if it succeeds sets doing_completion to true
            // true and calls the combobox's setSelectedItem method.  This
            // will eventually call insertString to do the insertion.
            doingRemove = true;
            doCompletion(combo, new_text, cur_pos);
            doingRemove = false;
        }
	}

	/**
	 * Fires cursor change events.
	 */
	public void caretUpdate(CaretEvent e)
	{
		lastCursorPosition = currentCursorPosition;
		currentCursorPosition = e.getDot();
		TextCursorEvent ev;
		JTextField text = getTextField();

		
		
		

		ev = new TextCursorEvent(lastCursorPosition, currentCursorPosition);
		textCursorMoveBegin(ev);

        if (getOverstrike() == true                       &&
            ev.getNewPosition() != currentCursorPosition  &&
			text != null                                  &&
			text.getCaret() != null)
        {
			text.getCaret().setDot(ev.getNewPosition());
        }
	}

    public class SelectedItem {
        public int      prefix_length = -1;
        public Object   item = null;
        public String   parsed = null;
        public int      index = -1;
	    public int      start_pos = 0;
	    public int      end_pos = 0;

        SelectedItem(Object item, int index, int prefix_length)
		{
            this.item          = item;
            this.index         = index;
            this.prefix_length = prefix_length;
        }

        SelectedItem(Object item, int index)
		{
            this.item  = item;
            this.index = index;
        }
    }

} // TextFieldDocument


public class TextFieldContent implements AbstractDocument.Content, Serializable
{
	protected boolean      overstrike = false;
	private int            data_length;
	private final char[]   empty = new char[0];
	private char[]         data;
	public transient       Vector marks;

	/**
	 * Creates a new TextFieldContent object.  Initial size defaults to 10.
	 */
	public TextFieldContent()
	{
		this(10);
	}

	/**
	 * Creates a new TextFieldContent object, with the initial
	 * size specified.  If the length is < 1, a size of 1 is used.
	 */
	public TextFieldContent(int length)
	{
		if (length < 1) {
			length = 1;
		}
		data = new char[length];
		data[0] = '\n';
		data_length = 1;
	}

	/**
	 * Returns the length of the content.
	 */
	public int length()
	{
		return(data_length);
	}

	protected void replaceRange(int offset, char[] replArray,
							    int replOffset, int replLength)
    {
		System.arraycopy(replArray, replOffset, data, offset, replLength);
	}

	/**
	 * Inserts a string into the content.
	 */
	public UndoableEdit insertString(int offset, String str)
	    throws BadLocationException
    {
		if (offset >= data_length) {
			throw new BadLocationException("Invalid location", data_length);
		}

		char[] chars = str.toCharArray();
		// if overstrike is false or at inserting at the end of the text,
		// just do the normal insertion of text
		if (overstrike == false || (offset == data_length - 1)) {
			insertText(offset, 0, chars, 0, chars.length);
			if (marks != null) {
				updateMarksForInsert(offset, str.length());
			}
		}
		else {
			replaceRange(offset, chars, 0, chars.length);
		}

		return(null);
	}

	public void insertText(int offset, int length, char[] replArray,
		                   int replOffset, int replLength)
    {
        // Note that length is the number of chars starting at
        // offset that will be replaced by this insert.
		int delta = replLength - length;
		int src = offset + length;
		int nmove = data_length - src;
		int dest = src + delta;
		if ((data_length + delta) >= data.length) {
			// need to grow the array
			int newLength = Math.max(2*data.length, data_length + delta);
			char[] newData = new char[newLength];
			System.arraycopy(data, 0, newData, 0, offset);
			System.arraycopy(replArray, replOffset, newData, offset, replLength);
			System.arraycopy(data, src, newData, dest, nmove);
			data = newData;
		}
		else {
			// move text after insertion point over
			System.arraycopy(data, src, data, dest, nmove);

			// now copy new text into the space
			System.arraycopy(replArray, replOffset, data, offset, replLength);
		}
		data_length = data_length + delta;
	}

	/**
	 * Removes part of the content.  offset + nitems must be < length().
	 *
	 */
	public UndoableEdit remove(int offset, int nitems)
	    throws BadLocationException
    {
		if (offset + nitems >= data_length) {
			throw new BadLocationException("Invalid range", data_length);
		}

		// If overstrike is not on, just delete the text, otherwise,
		// put a blank space in the place of the deleted character.
		if (overstrike == false) {
            // Note that the next line actually deletes characters
            // because nitems chars are getting replaced by nothing
			insertText(offset, nitems, empty, 0, 0);

			if (marks != null) {
				updateMarksForRemove(offset, nitems);
			}
		}
		else {
			// Initialize space array
			char space_array[] = new char[nitems];
			for (int i = 0; i < space_array.length; i++) {
				space_array[i] = ' ';
			}
			replaceRange(offset, space_array, 0, nitems);
		}

		return(null);
	}

	/**
	 * Retrieves a portion of the content.  offset + len must be <= length().
	 */
	public String getString(int offset, int len) throws BadLocationException
	{
		if (offset + len > data_length) {
			throw new BadLocationException("Invalid range", data_length);
		}
		return(new String(data, offset, len));
	}

	/**
	 * Retrieves a portion of the content.  offset + len must be <= length()
	 */
	public void getChars(int offset, int len, Segment chars)
	    throws BadLocationException
	{
		if (offset + len > data_length) {
			throw new BadLocationException("Invalid location", data_length);
		}
		chars.array = data;
		chars.offset = offset;
		chars.count = len;
	}

	/**
	 * Creates a position within the content that will
	 * track change as the content is mutated.
	 */
	public Position createPosition(int offset) throws BadLocationException
	{
		if (marks == null) {
			marks = new Vector();
		}
		return(new ContentPosition(offset, this));
	}

	public void setOverstrike(boolean overstrike)
	{
		this.overstrike = overstrike;
	}

	public boolean getOverstrike()
	{
		return(overstrike);
	}

	// Need to do the same thing as the AbstractDocument.Content class
	synchronized void updateMarksForInsert(int offset, int length)
	{
		if (offset == 0) {
			offset = 1;
		}
		int n = marks.size();
		for (int i = 0; i < n; i++) {
			PositionEntry mark = (PositionEntry) marks.elementAt(i);
			if (mark.unused) {
				marks.removeElementAt(i);
				i -= 1;
				n -= 1;
			}
			else if (mark.offset >= offset) {
				mark.offset += length;
			}
		}
	}

	// need to do the same thing as the AbstractDocument.Content class
	synchronized void updateMarksForRemove(int offset, int length)
	{
		int n = marks.size();
		for (int i = 0; i < n; i++) {
			PositionEntry mark = (PositionEntry) marks.elementAt(i);
			if (mark.unused) {
				// this record is no longer used, get rid of it
				marks.removeElementAt(i);
				i -= 1;
				n -= 1;
			}
			else if (mark.offset >= (offset + length)) {
				mark.offset -= length;
			}
			else if (mark.offset >= offset) {
				mark.offset = offset;
			}
		}
	}
} // TextFieldContent

/**
 * Contains the data for a mark
 */
final class PositionEntry
 {
	int      offset;
	boolean  unused;

	PositionEntry(int offset)
	{
		this.offset = offset;
	}

} // PositionEntry

final class ContentPosition implements Position
 {
	PositionEntry rec;

	ContentPosition(int offset, TextFieldContent tfc)
	{
		rec = new PositionEntry(offset);
		tfc.marks.addElement(rec);
	}

	public int getOffset()
	{
		return(rec.offset);
	}

	protected void finalize() throws Throwable
	{
		rec.unused = true;
	}

	public String toString()
	{
		return(Integer.toString(getOffset()));
	}

} // ContentPosition

}
