/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCComboField.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:14 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.field;

import com.klg.jclass.field.validate.AbstractValidator;
import com.klg.jclass.field.validate.JCValidator;
import com.klg.jclass.util.value.JCValueListener;
import com.klg.jclass.util.value.JCValueModel;

import javax.swing.*;
import javax.swing.plaf.ComboBoxUI;
import java.awt.*;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.util.Locale;

/**
 * <CODE>JCComboField</CODE> has all the capability of <CODE>JComboBox</CODE>,
 * as well as the ability to validate its contents.
 *
 * @see javax.swing.JComboBox
 */
public class JCComboField extends JComboBox implements JCFieldComponent {

protected Field       field = null;
transient JTextField  text;
protected boolean     required = true;
protected ListModel   original_picklist = null;

/**
 * Asks: "Is the <code>AutoComplete</code> mechanism enabled?".
 */
boolean autoComplete = false;

/**
 * In <code>AutoComplete</code> mode, displays a list of suggestions in a
 * drop-down list.
 */
boolean autoSuggest = false;

/**
 * If <code>AutoSuggest</code> is <code>true</code>, refines the list of
 * suggestions as the user types.
 */
boolean autoRefinement = false;

/**
 * In <code>AutoComplete</code> mode, appends the completion at the end of the
 * typed value.
 */
boolean autoAppend = true;

protected String prefixList[] = null;

/**
 * Contains the version number of the JClass product.
 */
public static final String version =
    com.klg.jclass.field.JCVersion.getVersionString();

/**
 * Constructs a <code>JCComboField</code>.
 */
public JCComboField()
{
	// Set the combo box to be editable by default
	setEditable(true);
	text = getTextFieldEditor();
	createField(null);
    field.addListDataListener(this);
}

/**
 * Constructs an empty <code>JCComboField</code> which expects an object of the
 * specified class.
 * @param c
 */
public JCComboField(Class c)
{
	this();
	field.setClass(c);
}

/**
 * Constructs an empty <code>JCComboField</code>, which expects an object of the
 * specified class and which uses the specified validator.
 * @param c
 * @param v
 */
public JCComboField(Class c, JCValidator v)
{
	this();
	field.setValidator(v);
	field.setClass(c);
}

/**
 * Constructs an empty <code>JCComboField</code>, which uses the specified
 * value model to contain its value.
 * @param m The value model
 */
public JCComboField(JCValueModel m)
{
	this();
	field.setValueModel(m);
}

/**
 * Constructs an empty <code>JCComboField</code>, which uses the specified
 * value model to contain its value and which uses the specified validator.
 * @param m The value model
 * @param v The validator
 */
public JCComboField(JCValueModel m, JCValidator v)
{
	this();
	field.setValidator(v);
	field.setValueModel(m);
}


/**
 * Adds a <code>PropertyChangeListener</code> to the listener list.
 *
 * @param listener  the <code>PropertyChangeListener</code> to be added
 */
public void addPropertyChangeListener(PropertyChangeListener listener)
{
	super.addPropertyChangeListener(listener);
	if (field != null) {
		// Pass through to field
		field.addPropertyChangeListener(listener);
	}
}

/**
 * Adds a listener for changes to the value.
 * @param listener The listener to add
 */
public void addValueListener(JCValueListener listener)
{
	field.addValueListener(listener);
}

/**
 * Removes a listener for changes to the value.
 * @param listener The listener to remove
 */
public void removeValueListener(JCValueListener listener)
{
	field.removeValueListener(listener);
}

/**
 * Removes a <code>PropertyChangeListener</code> from the listener list.
 * This removes a <code>PropertyChangeListener</code> that was registered
 * for all properties.
 *
 * @param listener  the <code>PropertyChangeListener</code> to be removed
 */
public void removePropertyChangeListener(PropertyChangeListener listener)
{
	super.removePropertyChangeListener(listener);
	if (field != null) {
		// pass through to field
		field.removePropertyChangeListener(listener);
	}
}

/**
 * Gets the <code>Bean</code> property which wraps the validator, data model,
 * and Field objects properties into one property.
 * @return
 */
public DataProperties getDataProperties()
{
	return(field.getDataProperties());
}

/**
 * Sets the <code>Bean</code> property which wraps the validator, data model,
 * and Field objects properties into one property.
 * @param dp
 */
public void setDataProperties(DataProperties dp)
{
	field.setDataProperties(dp);
}

protected void createField(JCValidator validator)
{
	field = new Field(this, validator);
}

/**
 * Gets the value of the <code>About</code> property.
 * The <code>About</code> property displays contact information for
 * Quest Software in the Bean box.
 * @return
 */
public String getAbout()
{
	return(version);
}

/**
 * Sets the value of the <code>About</code> property.
 * The <code>About</code> property displays contact information for
 * Quest Software in the Bean box.
 * @param s
 */
public synchronized void setAbout(String s) {}

/**
 * Initializes the field object.
 * This method should not need to be called.
 */
public void addNotify()
{
	super.addNotify();
	field.addNotify();
}

/**
 * Sets the background of the combo field.
 * @param background
 */
public void setBackground(Color background)
{
	// If the user tries to change the background, store the background
	// in the field object. Do not change the background immediately because
	// the field might be invalid.

	if (field == null || field.internalBackgroundChange == true) {
		super.setBackground(background);
	}
	else {
		field.validBackground = background;

		// If the field is not invalid, set the color now
		if (field.getState() != Field.INVALID) {
			super.setBackground(background);
		}
	}
}

/**
 * Sets the foreground of the combo field.
 * @param foreground
 */
public void setForeground(Color foreground)
{
	// If the user tries to change the foreground, store the foreground
	// in the field object. Do not change the foreground immediately because
	// the field might be invalid.

	if (field == null || field.internalForegroundChange == true) {
		super.setForeground(foreground);
	}
	else {
		field.validForeground = foreground;

		// If the field is not invalid, set the color now.
		if (field.getState() != Field.INVALID) {
			super.setForeground(foreground);
		}
	}
}

/**
 * Convenience method that returns the <code>TextField</code> Editor Component.
 * @return <code>TextField</code> Editor component
 */
protected JTextField getTextFieldEditor()
{
	return((JTextField)getEditor().getEditorComponent());
}

//
// JCFieldComponent implementation
//

/**
 * Returns the value for the combo field.
 * @return value for the combo field
 */
public Object getValue()
{
	return(field.getValue());
}

/**
 * Sets the value for the combo field.
 * @param value
 */
public void setValue(Object value)
{
	field.setValue(value);
}

/**
 * Returns the data model being used by the field.
 * @see com.klg.jclass.util.value.JCValueModel
 * @return data model being used by the field
 */
public JCValueModel getValueModel()
{
	return(field.getValueModel());
}

/**
 * Sets the data model being used by the field.
 * @see com.klg.jclass.util.value.JCValueModel
 * @param model
 */
public void setValueModel(JCValueModel model)
{
	field.setValueModel(model);
}

/**
 * Returns the validator currently in use.
 * @return validator currently in use
 */
public JCValidator getValidator()
{
	return(field.getValidator());
}

/**
 * Sets the validator to use.
 * @param v
 */
public void setValidator(JCValidator v)
{
	field.setValidator(v);
    original_picklist = (v == null) ? null : v.getPickList();
}

/**
 * Returns the <code>Invalid</code> properties. The <code>Invalid</code>
 * properties are <code>beepOnInvalid</code>, <code>invalidForeground</code>,
 * <code>invalidBackground</code>, <code>defaultValue</code>, and
 * <code>invalidPolicy</code>.
 * @return <code>Invalid</code> properties
 */
public JCInvalidInfo getInvalidInfo()
{
	return(field.getInvalidInfo());
}

/**
 * Sets the <code>Invalid</code> properties. The <code>Invalid</code> properties
 * are <code>beepOnInvalid</code>, <code>invalidForeground</code>,
 * <code>invalidBackground</code>, <code>defaultValue</code>, and
 * <code>invalidPolicy</code>.
 * @param info
 */
public void setInvalidInfo(JCInvalidInfo info)
{
	field.setInvalidInfo(info);
}

/**
 * Returns the state of the component: one of
 * <code>VALID</code>, <code>INVALID</code>, or <code>UNDER_EDIT</code>.
 * @return state of the component
 */
public int getState()
{
	return(field.getState());
}

/**
 * Returns a flag indicating the selection status on entering the field.
 * @see #setSelectOnEnter
 * @return flag indicating the selection status on entering the field
 */
public boolean getSelectOnEnter()
{
	return(field.getSelectOnEnter());
}

/**
 * Sets a flag to determine whether the value in the field is
 * selected when the component gains focus.  The default value is
 * <code>false</code>.
 * @param select
 */
public void setSelectOnEnter(boolean select)
{
	field.setSelectOnEnter(select);
}

/**
 * Commits the current field under edit.
 */
public void commitEdit()
{
	field.commitEdit();
}

/**
 * Returns <code>true</code> if the field is required.
 * @return <code>true</code> if the field is required
 */
public boolean isRequired()
{
	return(required);
}

/**
 * Gets the current pick list.
 * @return
 */
public ListModel getPickList()
{
    JCValidator validator = field.getValidator();
    if (validator == null) {
        return(null);
    }
	return(validator.getPickList());
}

/**
 * Gets the original pick list.
 * @return
 */
protected ListModel getOriginalPickList()
{
    return(original_picklist);
}

/**
 * Sets the pick list (may have been modified through
 * <code>AutoRefinement</code>).
 * @param picklist
 */
public void setPickList(ListModel picklist)
{
    setTemporaryPickList(picklist);
    this.original_picklist = picklist;
}

/**
 * Sets a temporary pick list (for <code>AutoRefinement</code> feature of
 * <code>AutoComplete</code>).
 * @param picklist
 */
protected void setTemporaryPickList(ListModel picklist)
{
    JCValidator validator = field.getValidator();
    if (validator == null) {
        return;
    }
	validator.setPickList(picklist);
    field.fireContentsChanged(field, 0, picklist.getSize());
}

/**
 * Restores picklist to its original (could have been modified through
 * <code>AutoRefinement</code>).
 */
protected void restoreOriginalPickList()
{
    setTemporaryPickList(original_picklist);
}

/**
 * Sets the <code>autoComplete</code> property and its corresponding attributes.
 *
 * @param autoComplete Turns on/off whether auto complete mode is enabled. If
 *      this value is <code>false</code>, the values of the attributes are
 *      ignored.
 * @param autoAppend appends the completion to the partial completed text
 *      in reverse video
 * @param autoSuggest pops up the combo box's popup as a suggestion list upon
 *      typing a character
 * @param autoRefinement If <code>autoSuggest</code> is <code>true</code>,
 *      refines the popup list to include all possible matches as well as the
 *      currently typed text. <br><B>Note:</b>  If <code>AutoSuggest</code> is
 *     <code>false</code>, this attribute will also be set to
 *     <code>false</code>.
 * @param prefixList Sets the <code>PrefixList</code>.  If non-null, the combo
 *      box will ignore the given prefixes when matching items.  The longest
 *      matching prefix is always used. <br><B>Note:</b> Although it is
 *      permitted to use a prefix list without <code>AutoSuggest</code> being
 *      <code>true</code>, the resulting behavior maybe be quite non-intuitive
 *      for the user.
 */
public void setAutoComplete(boolean autoComplete,
                            boolean autoAppend,
                            boolean autoSuggest,
                            boolean autoRefinement,
                            String prefixList[])
{
    if (this.autoComplete != autoComplete && !autoComplete) {
        field.turnOffAutoCompleteMode();
    }
	this.autoComplete   = autoComplete;
    this.autoAppend     = autoAppend;
    this.autoSuggest    = autoSuggest;
    this.autoRefinement = autoSuggest ? autoRefinement : false;
    this.prefixList     = prefixList;
}

/**
 * Returns <code>true</code> if the field has <code>autoComplete</code> turned
 * on.
 * @return <code>true</code> if <code>autoComplete</code> is on
 */
public boolean isAutoComplete()
{
	return(autoComplete);
}

/**
 * Returns <code>true</code> if the field has <code>autoSuggest</code> turned
 * on.
 * @return <code>true</code> if <code>autoSuggest</code> is on
 */
public boolean isAutoSuggest()
{
	return(autoSuggest);
}

/**
 * Returns <code>true</code> if the field has <code>autoAppend</code> turned on.
 * @return <code>true</code> if the field has <code>autoAppend</code> turned on
 */
public boolean isAutoAppend()
{
	return(autoAppend);
}

/**
 * Returns <code>true</code> if the field has <code>autoRefinement</code> turned
 * on.
 * @return <code>true</code> if the field has <code>autoRefinement</code> turned
 *     on
 */
public boolean isAutoRefinement()
{
	return(autoRefinement);
}

/**
 * Returns the value of <code>PrefixList</code>.
 * @see #setAutoComplete
 * @return value of <code>PrefixList</code>
 */
public String[] getPrefixList()
{
	return(prefixList);
}

/**
 * Sets the required property for the current field.
 * @param required
 */
public void setRequired(boolean required)
{
	this.required = required;
}

/**
 * Sets the look and feel for this component.
 * This method is used internally. You should not have to call it.
 * @param ui
 */
public void setUI(ComboBoxUI ui)
{
	super.setUI(ui);
	text = getTextFieldEditor();

	// We must create a new field to add all the listeners to the new editor.
	// Note that we must set the validator after the createField() call
	// because otherwise problems happen (i.e. validator - model mismatches).
	if (field != null) {
		JCValueModel model = field.getValueModel();
		createField(null);
		setValidator(field.getValidator());
		setValueModel(model);
	}
}

/**
 * Updates the look and feel for this component.
 * This method is used internally. You should not have to call it.
 */
public void updateUI()
{
	setUI((ComboBoxUI)UIManager.getUI(this));
}

/**
 * Adds an <code>ActionListener</code>. An action event is generated when the
 * edited item changes.
 * @param l
 */
public void addActionListener(ActionListener l)
{
	text.addActionListener(l);
}

/** Removes an <code>ActionListener</code>.
 * @param l
 */
public void removeActionListener(ActionListener l)
{
	text.removeActionListener(l);
}

/**
 * Sets the list of choices. This method is for internal use only.
 * To set the list, use the <code>setPickList</code> method in the
 * <code>AbstractValidator</code> class.
 * @see com.klg.jclass.field.validate.AbstractValidator#setPickList
 * @param items
 */
public void setItems(String items[])
{
	if (getItemCount() > 0) {
		removeAllItems();
	}
	for (int i = 0; i < items.length; i++) {
		add(items[i]);
	}
}

/**
 * Adds an item to the list. This method is for internal use only.
 * To set the list, use the <code>setPickList</code> method in the
 * <code>AbstractValidator</code> class.
 * @see com.klg.jclass.field.validate.AbstractValidator#setPickList
 * @param item
 */
public void add(String item)
{
	addItem(item);
}

/**
 * Sets the locale being used by the combo field.  Sets it on the validator
 * as well.
 * @param l the new locale to be used
 */
public void setLocale(Locale l)
{
	super.setLocale(l);
	if (field != null) {
		JCValidator v = getValidator();
		if (v instanceof AbstractValidator) {
			AbstractValidator av = (AbstractValidator)v;
			av.setLocale(l);
		}

        // Need to reformat the current value to the new locale
        JCValueModel model = getValueModel();
        if (model != null) {
            field.setValue(model.getValue());
        }
	}
}

}
