/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCFieldComponent.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:14 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.field;

import com.klg.jclass.util.value.JCValueModel;
import com.klg.jclass.util.value.JCValueListener;
import com.klg.jclass.field.validate.JCValidator;

/**
 * List of the public API's that make up a JClass Field Bean.
 */
public interface JCFieldComponent {

// enum for State
public static final int VALID     = 1;
public static final int UNDEREDIT = 2;
public static final int INVALID   = 3;

/**
 * Returns the value for the text field.
 * @return value for the text field
 */
public Object getValue();

/*
 * Sets the value for the text field.
 */
public void setValue(Object value);

/**
 * Returns the data model being used by the field.
 * @see com.klg.jclass.util.value.JCValueModel
 * @return data model used by the field
 */
public JCValueModel getValueModel();

/**
 * Sets the data model being used by the field.
 * @see com.klg.jclass.util.value.JCValueModel
 * @param model
 */
public void setValueModel(JCValueModel model);

/**
 * Returns the validator currently in use.
 * @see com.klg.jclass.field.validate.JCValidator
 * @return the validator currently in use
 */
public JCValidator getValidator();

/**
 * Sets the validator to use.
 * @see com.klg.jclass.field.validate.JCValidator
 * @param validator
 */
public void setValidator(JCValidator validator);

/**
 * Returns the <code>Invalid</code> properties. That is <code>beepOnInvalid</code>,
 * <code>invalidForeground</code>, <code>invalidBackground</code>, <code>defaultValue</code>,
 * and <code>invalidPolicy</code>.
 * @return the <code>Invalid</code> properties
 */
public JCInvalidInfo getInvalidInfo();

/**
 * Sets the <code>Invalid</code> properties. That is <code>beepOnInvalid</code>,
 * <code>invalidForeground</code>, <code>invalidBackground</code>, <code>defaultValue</code>,
 * and <code>invalidPolicy</code>.
 * @param info
 */
public void setInvalidInfo(JCInvalidInfo info);

/**
 * Sets the validator, value model, and invalid properties.
 * This method is convenient for setting the validator and value model 
 * simultaneously.  
 * @see com.klg.jclass.field.validate.JCValidator
 * @see com.klg.jclass.util.value.JCValueModel
 * @see com.klg.jclass.field.JCInvalidInfo
 * @param dp
 */
public void setDataProperties(DataProperties dp);

/**
 * Returns the <code>DataProperties</code> which contains the validator, value model, and
 * invalid proeprties.
 * @see com.klg.jclass.field.validate.JCValidator
 * @see com.klg.jclass.util.value.JCValueModel
 * @see com.klg.jclass.field.JCInvalidInfo
 * @return the <code>DataProperties</code>
 */
public DataProperties getDataProperties();

/**
 * Returns the state of the component; one of
 * <code>VALID</code>, <code>INVALID</code>, or <code>UNDER_EDIT</code>.
 * @return the state of hte component
 */
public int getState();

/**
 * Returns a flag indicating the selection status on entering the field.
 * @see #setSelectOnEnter
 * @return a flag indication the selection status
 */
public boolean getSelectOnEnter();

/**
 * Sets the flag that determines whether of not the Text Component is automatically
 * selected when the component gains focus.  The default value for this is
 * <code>false</code>.
 * @param select
 */
public void setSelectOnEnter(boolean select);

/**
 * Commits the current field under edit.
 */
public void commitEdit();

/**
 * Returns whether this is a required field or not.
 * @return whether this is a required field
 */
public boolean isRequired();

/**
 * Sets whether this is a required field or not.
 * @param required
 */
public void setRequired(boolean required);

/**
 * Adds a listener for changes to the value.
 * @param listener
 */
public void addValueListener(JCValueListener listener);

/**
 * Removes a listener for changes to the value.
 * @param listener
 */
public void removeValueListener(JCValueListener listener);

}
