/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCFormUtil.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:14 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.field;

import java.util.HashSet;
import java.util.Set;
import java.util.Iterator;
import java.awt.Container;
import java.awt.Component;


/**
 * This class contains a collection of methods that look through all the
 * children of a given <CODE>Container</CODE> (usually a <CODE>Form</CODE>) and
 * return different sets of information based on the JClass Field components
 * found in a <CODE>Container</CODE>.
 */
public class JCFormUtil {

/**
 * Adds <code>JCFieldComponent</code> children of the specified parent to the
 * specified set and recursively traverse any container children.
 * @param parent
 * @param set
 */
private static void addFieldChildrenToSet(Container parent, Set set)
{
    Component kids[] = parent.getComponents();
    for (int i = 0; i < kids.length; i++) {
        if (kids[i] instanceof JCFieldComponent) {
            set.add(kids[i]);
        }
        else if (kids[i] instanceof Container) {
            addFieldChildrenToSet((Container)kids[i], set);
        }
    }
}

/**
 * Recursively gets all the <code>JCFieldComponent</code> children of
 * the specified container. All JClass Field components are instances
 * of <code>JCFieldComponent</code>.
 * @param form
 * @return
 */
public static Set getFieldComponents(Container form)
{
    Set set = new HashSet();
    addFieldChildrenToSet(form, set);
    return(set);
}

/**
 * Returns <code>true</code> if all the "required" <code>JCFieldComponent</code>
 * children of the container (<i>isRequired() == true</i>) are valid
 * (<i>getState() == VALID</i>).
 * @param form
 * @return
 */
public static boolean isFieldComponentContainerComplete(Container form)
{
    Set invalid = getInvalidRequiredFieldComponents(form);
    if (invalid == null || invalid.size() == 0) {
        return(true);
    }
    return(false);
}

/**
 * Adds <code>JCFieldComponent</code> children of the specified parent to the
 * specified set and recursively traverses any container children.
 * @param parent
 * @param set
 */
private static void addRequiredFieldChildrenToSet(Container parent, Set set)
{
    Component kids[] = parent.getComponents();
    for (int i = 0; i < kids.length; i++) {
        if (kids[i] instanceof JCFieldComponent) {
            JCFieldComponent field = (JCFieldComponent)kids[i];
            if (field.isRequired()) {
                set.add(kids[i]);
            }
        }
        else if (kids[i] instanceof Container) {
            addRequiredFieldChildrenToSet((Container)kids[i], set);
        }
    }
}

/**
 * Returns the <code>JCFieldComponent</code> children of the form that are
 * considered required (<i>isRequired() == true</i>).
 * @param form
 * @return
 */
public static Set getRequiredFieldComponents(Container form)
{
    Set set = new HashSet();
    addRequiredFieldChildrenToSet(form, set);
    return(set);
}

/**
 * Adds <code>JCFieldComponent</code> children that are required but invalid
 * of the specified parent to the specified set, and recursively
 * traverses any container children.
 * @param parent
 * @param set
 */
private static void addInvalidRequiredFieldChildrenToSet(Container parent,
                                                         Set set)
{
    Component kids[] = parent.getComponents();
    for (int i = 0; i < kids.length; i++) {
        if (kids[i] instanceof JCFieldComponent) {
            JCFieldComponent field = (JCFieldComponent)kids[i];
            if (field.isRequired() == false) {
                continue;
            }
            if (field.getState() == field.UNDEREDIT) {
                field.commitEdit();
            }
            if (field.getState() != field.VALID) {
                set.add(kids[i]);
            }
        }
        else if (kids[i] instanceof Container) {
            addInvalidRequiredFieldChildrenToSet((Container)kids[i], set);
        }
    }
}

/**
 * Returns the <code>JCFieldComponent</code> children that are "required" but
 * invalid, for the specified form. Determines which fields are considered
 * required (<i>isRequired() == true</i>), and of those, which are not valid
 * (<i>getState() != VALID</i>).
 * @return
 * @param form
 */
public static Set getInvalidRequiredFieldComponents(Container form)
{
    Set set = new HashSet();
    addInvalidRequiredFieldChildrenToSet(form, set);
    return(set);
}

/**
 * Sets all the <code>JCFieldComponent</code> children of the specified
 * Container to null.
 * @param form
 */
public static void clearFieldComponents(Container form)
{
    Set fields = getFieldComponents(form);
    Iterator it = fields.iterator();
    while (it.hasNext()) {
        JCFieldComponent field = (JCFieldComponent)it.next();

        JCInvalidInfo invalid_info = field.getInvalidInfo();
        // Turn the beep off so that we don't beep when we clear the field
        invalid_info.setBeepOnInvalid(false);
        field.setInvalidInfo(invalid_info);

        
        field.setValue(null);

        // Turn the beep back on to beep at user entries
        invalid_info.setBeepOnInvalid(true);
        field.setInvalidInfo(invalid_info);
    }
}

/**
 * Resets all the <code>JCFieldComponent</code> children of the specified
 * Container to their default values.
 * @param form
 */
public static void resetFieldComponents(Container form)
{
    Set fields = getFieldComponents(form);
    Iterator it = fields.iterator();
    while (it.hasNext()) {
        JCFieldComponent field = (JCFieldComponent)it.next();

        JCInvalidInfo invalid_info = field.getInvalidInfo();
        // Turn the beep off so that we don't beep when we clear the field
        invalid_info.setBeepOnInvalid(false);
        field.setInvalidInfo(invalid_info);

        
        field.setValue(field.getValidator().getDefaultValue());

        // Turn the beep back on to beep at user entries
        invalid_info.setBeepOnInvalid(true);
        field.setInvalidInfo(invalid_info);
    }
}

}
