/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCInvalidInfo.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:14 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.field;

import java.awt.Color;

/**
 * This class creates an object which holds the invalid info properties.
 */
public class JCInvalidInfo implements java.io.Serializable {

// enum for invalid_policy
/** Use with <code>invalidPolicy</code> to display invalid values. */
public static final int SHOW_INVALID     = 1;
/** Use with <code>invalidPolicy</code> to restore default value after invalid entry. */
public static final int RESTORE_DEFAULT  = 2;
/** Use with <code>invalidPolicy</code> to restore previous value after invalid entry. */
public static final int RESTORE_PREVIOUS = 3;
/** Use with <code>invalidPolicy</code> to clear the field after invalid entry. */
public static final int CLEAR_FIELD      = 4;

protected boolean beepOnInvalid;
protected int invalidPolicy;
protected Color invalidForeground;
protected Color invalidBackground;

/**
 * Creates a default instance where <code>beepOnInvalid == true</code>,
 * <code>policy == SHOW_INVALID</code>, and the rest of the values are null.
 */
public JCInvalidInfo() {
	this(true, SHOW_INVALID, null, null);
}

/**
 * Creates an instance where <code>beepOnInvalid</code> and <code>invalidPolicy</code> are
 * set as specified and the rest of the values are null.
 * @param beep_on_invalid
 * @param invalid_policy
 */
public JCInvalidInfo(boolean beep_on_invalid, int invalid_policy) {
	this(beep_on_invalid, invalid_policy, null, null);
}

/**
 * Creates an instance where <code>invalidPolicy</code> is set as specified,
 * <code>beepOnInvalid == true</code>, and the rest of the values are null.
 * @param invalid_policy
 */
public JCInvalidInfo(int invalid_policy) {
	this(true, invalid_policy, null, null);
}

/**
 * Creates a <code>InvalidInfo</code> object with the specified invalid colors and with
 * <code>invalidPolicy == SHOW_INVALID</code>, <code>beepOnInvalid == true</code>, and
 * <code>defaultValue == null</code>.
 * @param invalid_foreground
 * @param invalid_background
 */
public JCInvalidInfo(Color invalid_foreground, Color invalid_background) {
	this(true, SHOW_INVALID, invalid_foreground, invalid_background);
}

/**
 * Creates a <code>InvalidInfo</code> object with the specified <code>defaultValue</code> and with
 * <code>invalidPolicy == RESTORE_DEFAULT</code>, <code>beepOnInvalid == true</code>, and
 * the invalid colors are null.
 * @param beep_on_invalid
 * @param invalid_background
 * @param invalid_foreground
 * @param invalid_policy
 */
public JCInvalidInfo(boolean beep_on_invalid, int invalid_policy,
					 Color invalid_foreground, Color invalid_background) {
	setBeepOnInvalid(beep_on_invalid);
	setInvalidPolicy(invalid_policy);
	setInvalidForeground(invalid_foreground);
	setInvalidBackground(invalid_background);
}

/**
 * Returns whether the component should beep or not when the user
 * types an invalid entry.
 * @return whether the component should beep or not
 */
public boolean getBeepOnInvalid() {
	return beepOnInvalid;
}

/**
 * Sets whether the component should beep or not when the user
 * types an invalid entry.
 * @param beep_on_invalid
 */
public void setBeepOnInvalid(boolean beep_on_invalid) {
	this.beepOnInvalid = beep_on_invalid;
}

/**
 * Returns the invalid policy. Valid values are:<BR>
 * <code>SHOW_INVALID</code> - displays invalid entry (default)<BR>
 * <code>RESTORE_DEFAULT</code> - restores the default value on invalid entry<BR>
 * <code>RESTORE_PREVIOUS</code> - restores the previous valid entry on invalid entry<BR>
 * <code>CLEAR_FIELD</code> - clears the field
 * @return invalid policy
 */
public int getInvalidPolicy() {
	return invalidPolicy;
}

/**
 * Sets the invalid policy. Valid values are:<BR>
 * <code>SHOW_INVALID</code> - displays invalid entry (default)<BR>
 * <code>RESTORE_DEFAULT</code> - restores the default value on invalid entry<BR>
 * <code>RESTORE_PREVIOUS</code> - restores the previous valid entry on invalid entry<BR>
 * <code>CLEAR_FIELD</code> - clears the field
 * @param invalid_policy
 */
public void setInvalidPolicy(int invalid_policy) {
	if (invalid_policy != SHOW_INVALID &&
		invalid_policy != RESTORE_DEFAULT &&
		invalid_policy != RESTORE_PREVIOUS &&
		invalid_policy != CLEAR_FIELD) {
		throw new IllegalArgumentException("Invalid Policy must be one of SHOW_INVALID, RESTORE_DEFAULT, RESTORE_PREVIOUS, or CLEAR_FIELD");
	}
	this.invalidPolicy = invalid_policy;
}

/**
 * Returns the background color to be used if the invalid policy is
 * <code>SHOW_INVALID</code>. Null means use the normal background color.
 * @return background color to be used if the invalid policy is <code>SHOW_INVALID</code>
 */
public Color getInvalidBackground() {
	return invalidBackground;
}

/**
 * Sets the background color to be used if the invalid policy is
 * <code>SHOW_INVALID</code>. Null means use the normal background color.
 * @param background
 */
public void setInvalidBackground(Color background) {
	this.invalidBackground = background;
}

/**
 * Returns the foreground color to be used if the invalid policy is
 * <code>SHOW_INVALID</code>. Null means use the normal foreground color.
 * @return foreground color to be used if the invalid policy is <code>SHOW_INVALID</code>
 */
public Color getInvalidForeground() {
	return invalidForeground;
}

/**
 * Sets the foreground color to be used if the invalid policy is
 * <code>SHOW_INVALID</code>. Null means use the normal foreground color.
 * @param foreground
 */
public void setInvalidForeground(Color foreground) {
	this.invalidForeground = foreground;
}

}
