/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCLabelField.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:14 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.field;

import com.klg.jclass.field.validate.AbstractValidator;
import com.klg.jclass.field.validate.JCValidator;
import com.klg.jclass.util.value.JCValueListener;
import com.klg.jclass.util.value.JCValueModel;

import javax.swing.*;
import java.awt.*;
import java.beans.PropertyChangeListener;
import java.util.Locale;

/**
 * <CODE>JCLabelField</CODE> has all the capability of <CODE>JLabelField</CODE> as well as the ability to
 * validate its contents.
 */
public class JCLabelField extends JLabel implements JCFieldComponent
{

protected Field    field;
protected boolean  required = true;

/**
 * Contains the version number of the JClass product.
 */
public static final String version = com.klg.jclass.field.JCVersion.getVersionString();

/**
 * Constructs an empty <code>JCLabelField</code>.
 */
public JCLabelField()
{
	createField(null);
	setOpaque(true);
}

/**
 * Constructs an empty <code>JCLabelField</code>, which expects an object of the
 * specified class.
 * @param c
 */
public JCLabelField(Class c)
{
	this();
	field.setClass(c);
}

/**
 * Constructs an empty <code>JCLabelField</code>, which expects an object of the
 * specified class and which uses the specified validator.
 * @param c
 * @param v
 */
public JCLabelField(Class c, JCValidator v)
{
	this();
	field.setValidator(v);
	field.setClass(c);
}

/**
 * Constructs an empty <code>JCLabelField</code>, which uses the specified value model
 * to contain its value.
 * @param m
 */
public JCLabelField(JCValueModel m)
{
	this();
	field.setValueModel(m);
}

/**
 * Constructs an empty <code>JCLabelField</code>, which uses the specified value model
 * to contain its value and which uses the specified validator.
 * @param m
 * @param v
 */
public JCLabelField(JCValueModel m, JCValidator v)
{
	this();
	field.setValidator(v);
	field.setValueModel(m);
}

/**
 * Gets the Bean property which wraps the validator, data model, and <code>Field</code>
 * objects properties into one property.
 * @return
 */
public DataProperties getDataProperties()
{
	return(field.getDataProperties());
}

/**
 * Sets the Bean property which wraps the validator, data model, and <code>Field</code>
 * objects properties into one property.
 * @param dp
 */
public void setDataProperties(DataProperties dp)
{
	field.setDataProperties(dp);
}

/**
 * Adds a <code>PropertyChangeListener</code> to the listener list.
 *
 * @param listener  the <code>PropertyChangeListener</code> to be added
 */
public void addPropertyChangeListener(PropertyChangeListener listener)
{
	super.addPropertyChangeListener(listener);
	if (field != null) {
		// pass through to field
		field.addPropertyChangeListener(listener);
	}
}

/**
 * Removes a <code>PropertyChangeListener</code> from the listener list.
 * This removes a <code>PropertyChangeListener</code> that was registered
 * for all properties.
 *
 * @param listener  the <code>PropertyChangeListener</code> to be removed
 */
public void removePropertyChangeListener(PropertyChangeListener listener)
{
	super.removePropertyChangeListener(listener);
	if (field != null) {
		// pass through to field
		field.removePropertyChangeListener(listener);
	}
}

/**
 * Adds a listener to listen for changes to the value.
 * @param listener
 */
public void addValueListener(JCValueListener listener)
{
	field.addValueListener(listener);
}

/**
 * Removes a listener which listens for changes to the value.
 * @param listener
 */
public void removeValueListener(JCValueListener listener)
{
	field.removeValueListener(listener);
}

/**
 * Initializes the field object.
 * This method should not need to be called.
 */
public void addNotify()
{
	super.addNotify();
	field.addNotify();
}

/**
 * Creates the field object which controls the validation.
 * @param validator
 */
protected void createField(JCValidator validator)
{
	field = new Field(this, null);
}

/**
 * Gets the value of the <code>About</code> property.
 * The <code>About</code> property displays contact information for
 * Quest Software in the Bean box.
 * @return
 */
public String getAbout()
{
	return(version);
}

/**
 * Sets the value of the <code>About</code> property.
 * The <code>About</code> property displays contact information for
 * Quest Software in the Bean box.
 * @param s
 */
public synchronized void setAbout(String s) {}

/**
 * Sets the background of the label field.
 * @param background
 */
public void setBackground(Color background)
{
	// If the user tries to change the background, store the background
	// in the field object. Do not change the background immediately because
	// the field might be invalid.

	if (field == null || field.internalBackgroundChange == true) {
		super.setBackground(background);
	}
	else {
		field.validBackground = background;

		// if the field is not invalid, set the color now
		if (field.getState() != Field.INVALID) {
			super.setBackground(background);
		}
	}
}

/**
 * Sets the foreground of the label field.
 * @param foreground
 */
public void setForeground(Color foreground)
{
	// If the user tries to change the foreground, store the foreground
	// in the field object. Do not change the foreground immediately because
	// the field might be invalid.

	if (field == null || field.internalForegroundChange == true) {
		super.setForeground(foreground);
	}
	else {
		field.validForeground = foreground;

		// if the field is not invalid, set the color now.
		if (field.getState() != Field.INVALID) {
			super.setForeground(foreground);
		}
	}
}
	//
	// Implement methods for JCFieldComponent
	//

/**
 * Returns the value for the label field.
 * @return value for the label field
 */
public Object getValue()
{
	return(field.getValue());
}

/**
 * Sets the value for the label field.
 * @param value
 */
public void setValue(Object value)
{
	field.setValue(value);
}

/**
 * Returns the data model being used by the field.
 * @see com.klg.jclass.util.value.JCValueModel
 * @return data model being used by the field
 */
public JCValueModel getValueModel()
{
	return(field.getValueModel());
}

/**
 * Sets the data model being used by the field.
 * @see com.klg.jclass.util.value.JCValueModel
 * @param model
 */
public void setValueModel(JCValueModel model)
{
	field.setValueModel(model);
}

/**
 * Returns the validator currently in use.
 * @see com.klg.jclass.field.validate.JCValidator
 * @return validator currently in use
 */
public JCValidator getValidator()
{
	return(field.getValidator());
}

/**
 * Sets the validator to use.
 * @see com.klg.jclass.field.validate.JCValidator
 * @param validator
 */
public void setValidator(JCValidator validator) {
	field.setValidator(validator);
}

/**
 * Returns the <code>Invalid</code> properties. The <code>Invalid</code> properties are
 * <code>beepOnInvalid</code>, <code>invalidForeground</code>, <code>invalidBackground</code>,
 * <code>defaultValue</code>, and <code>invalidPolicy</code>.
 * @return the <code>Invalid</code> properties
 */
public JCInvalidInfo getInvalidInfo()
{
	return(field.getInvalidInfo());
}

/**
 * Sets the <code>Invalid</code> properties. The <code>Invalid</code> properties are
 * <code>beepOnInvalid</code>, <code>invalidForeground</code>, <code>invalidBackground</code>,
 * <code>defaultValue</code>, and <code>invalidPolicy</code>.
 * @param info
 */
public void setInvalidInfo(JCInvalidInfo info)
{
	field.setInvalidInfo(info);
}

/**
 * Returns the state of the component:
 * <code>VALID</code> or <code>INVALID</code>.
 * @return <code>VALID</code> or <code>INVALID</code>
 */
public int getState()
{
	return(field.getState());
}

/**
 * Returns a flag indicating the selection status on entering the field.
 * @see #setSelectOnEnter
 * @return a flag indicating the selection status on entering the field
 */
public boolean getSelectOnEnter()
{
	return(false);
}

/**
 * Not used for this component.
 * @param select
 */
public void setSelectOnEnter(boolean select) {}

/**
 * Not used for this component.
 */
public void commitEdit() {}

/**
 * Returns <code>true</code> if the field is required.
 * @return <code>true</code> if the field is required
 */
public boolean isRequired()
{
	return(required);
}

/**
 * Sets the required property for the current field.
 * @param required
 */
public void setRequired(boolean required)
{
	this.required = required;
}

/**
 * Overriden: This method should never be called since it will bypass
 * the validation and formatting. Use the <code>setValue</code> method instead.
 * @see #setValue
 * @param text
 */
public void setText(String text)
{
	if (field != null && field.getAllowTextChange() == true) {
		super.setText(text);
	}
}

/**
 * Sets the locale being used by the label field; sets it on the validator
 * as well.
 * @param l the new locale to be used
 */
public void setLocale(Locale l)
{
	super.setLocale(l);
	if (field != null) {
		JCValidator v = getValidator();
		if (v instanceof AbstractValidator) {
			AbstractValidator av = (AbstractValidator)v;
			av.setLocale(l);
		}

        // Need to reformat the current value to the new locale
        JCValueModel model = getValueModel();
        if (model != null) {
            field.setValue(model.getValue());
        }
	}
}

}
