/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCPopupField.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:14 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.field;

import com.klg.jclass.field.plaf.MetalPopupFieldUI;
import com.klg.jclass.field.plaf.MotifPopupFieldUI;
import com.klg.jclass.field.plaf.WindowsPopupFieldUI;
import com.klg.jclass.util.swing.JCPopupEvent;
import com.klg.jclass.util.swing.JCPopupListener;
import com.klg.jclass.field.validate.JCValidator;
import com.klg.jclass.field.validate.PopupFieldEditor;
import com.klg.jclass.util.value.JCValueEvent;
import com.klg.jclass.util.value.JCValueListener;
import com.klg.jclass.util.value.JCValueModel;

import javax.swing.*;
import javax.swing.plaf.ComboBoxUI;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Calendar;
import java.util.Locale;

/**
 * <CODE>JCPopupField</CODE> is subclassed from <CODE>JCComboField</CODE>.  It
 * uses a combo field to display a monthly calendar when the
 * <CODE>JCDateTimeValidator</CODE> is used.  Only this JClass Field validator
 * implements the popup; however, you can write your own validator to make use
 * of the popup in other ways.
 */
public class JCPopupField extends JCComboField
			              implements JCPopupListener, JCValueListener {

protected PopupFieldEditor popupEditor = null;

/**
 * Constructs an empty <code>JCPopupField</code>.
 */
public JCPopupField() {}

/**
 * Constructs an empty <code>JCPopupField</code>, which expects an object of the
 * specified class.
 * @param c The class
 */
public JCPopupField(Class c)
{
	super(c);
}

/**
 * Constructs an empty <code>JCPopupField</code>, which expects an object of the
 * specified class and which uses the specified validator.
 * @param c The class
 * @param v The validator
 */
public JCPopupField(Class c, JCValidator v)
{
	super(c, v);
}

/**
 * Constructs an empty <code>JCPopupField</code>, which uses the specified value
 * model to contain its value.
 * @param m The value model
 */
public JCPopupField(JCValueModel m)
{
	super(m);
}

/**
 * Constructs an empty <code>JCPopupField</code>, which uses the specified value
 * model to contain its value and which uses the specified validator.
 * @param m The value model
 * @param v The validator
 */
public JCPopupField(JCValueModel m, JCValidator v)
{
	super(m, v);
}

protected ComboBoxUI inferLookAndFeelUI()
{
	ComboBoxUI currentUI = null;
	String lf = UIManager.getLookAndFeel().getID();
    String popupUIString = null;


    // The following mess is required to make this bean work with Visual Cafe.
    // Cafe tries inspects this class in an enviroment that does
    // not include the Windows or Motif plafs.  Creating the UI object this way
    // means Cafe cannot see these plaf objects and doesn't complain. Ironically
    // once the bean has been loaded into Cafe it provides and enviroment that
    // does include the window and motif plafs and infact it uses the Windows
    // plaf in its designer.
	if (lf.equals("Windows")) {
        popupUIString = "com.klg.jclass.field.plaf.WindowsPopupFieldUI";
    }
	else if (lf.equals("Motif")) {
        popupUIString = "com.klg.jclass.field.plaf.MotifPopupFieldUI";
	}
	else {
        popupUIString = "com.klg.jclass.field.plaf.MetalPopupFieldUI";
    }

    try {
        Class popupUIClass = Class.forName(popupUIString);
        Class[] parameterTypes = new Class[] {PopupFieldEditor.class};
        Constructor popupUIConstructor =
            popupUIClass.getConstructor(parameterTypes);
        Object[] initargs = new Object[] {popupEditor};
        currentUI = (ComboBoxUI) popupUIConstructor.newInstance(initargs);
    }
    catch (ExceptionInInitializerError e) {}
    catch (LinkageError e) {}
    catch (ClassNotFoundException e) {}
    catch (InstantiationException e) {}
    catch (IllegalAccessException e) {}
    catch (IllegalArgumentException e) {}
    catch (InvocationTargetException e) {}
    catch (NoSuchMethodException e) {}
    catch (SecurityException e) {}

    // Old method of creating the UI object.
/*
	if (lf.equals("Windows")) {
		currentUI = new WindowsPopupFieldUI(popupEditor);
	}
	else if (lf.equals("Motif")) {
		currentUI = new MotifPopupFieldUI(popupEditor);
	}
	else {
		currentUI = new MetalPopupFieldUI(popupEditor);
	}
*/
	return(currentUI);
}

/**
 * Sets the popup editor.
 * @param popup
 */
public void setPopupEditor(PopupFieldEditor popup)
{
	if (popupEditor != popup) {
		// Remove popup listener
		if (popupEditor != null) {
			popupEditor.removePopupListener(this);
		}

		popupEditor = popup;

		// Add new popup listener
		if (popupEditor != null) {
			popupEditor.addPopupListener(this);
		}


		String lf = UIManager.getLookAndFeel().getID();
		if (lf.equals("Windows")) {
			((WindowsPopupFieldUI)ui).setPopupFieldEditor(popup);
		}
		else if (lf.equals("Motif")) {
			((MotifPopupFieldUI)ui).setPopupFieldEditor(popup);
		}
		else {
			((MetalPopupFieldUI)ui).setPopupFieldEditor(popup);
		}
	}
}

/**
 * Returns the popup editor.
 * @return popup editor
 */
public PopupFieldEditor getPopupEditor()
{
	return(popupEditor);
}

		//
		// JCPopupListener Implementation.
		//
/**
 * This method is used internally. You should not have to call it.
 * @param e
 */
public void commit(JCPopupEvent e)
{
	setValue(e.getNewValue());
}

		//
		// JCValueModel Implementation
		//
/**
 * This method is used internally. You should not have to call it.
 * @param e
 */
public void valueChanging(JCValueEvent e) {}

/**
 * This method is used internally. You should not have to call it.
 * @param e
 */
public void valueChanged(JCValueEvent e)
{
	if (popupEditor != null) {
		Object value = getValue();

		// Set the value of the date time chooser if the value is not null.
		// if null, set the date time chooser to the current date.
		if (value != null) {
			popupEditor.setValue(getValue());
		}
		else {
			popupEditor.setValue(Calendar.getInstance());
		}
	}
}

/**
 * Sets the look and feel for this component.
 * This method is used internally. You should not have to call it.
 * @param ui
 */
public void setUI(ComboBoxUI ui)
{
	super.setUI(inferLookAndFeelUI());
}

/**
 * Updates the look and feel for this component.
 * This method is used internally. You should not have to call it.
 */
public void updateUI()
{
	setUI(null);
}

/**
 * Overridden since this method assumed that a combobox list always exists.
 * Do not call this method directly.
 * @return
 */
public Object getSelectedItem()
{
	if (isEditable() == false) {
		return(getValidator().format(getValue()));
	}
	return(null);
}

/**
 * Sets the locale being used by the popup field
 * chooser, which will set it on its child components.
 * @param l the new locale to be used
 */
public void setLocale(Locale l)
{
	super.setLocale(l);
	if (popupEditor != null) {
		popupEditor.setLocale(l);
	}
}

}
