/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCPromptHelper.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:14 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.field;

import java.awt.*;
import java.awt.event.*;
import javax.swing.JComponent;
import javax.swing.JLabel;

/**
 * This class allows the user to display the tool tip text
 * in a specific prompt label of any container whenever
 * that field component gains focus, and to remove the text when the field
 * loses focus. <br><B>Note:</b> This helper is <i>not</i> restricted to finding
 * <code>JCFieldComponents</code> that are immediate children of the specified container.
 */
public class JCPromptHelper implements ContainerListener, FocusListener {

protected JLabel prompt;

/**
 * Constructs a Prompt Helper on the specified container, with the
 * specified label as the target for the tool tip text of the Field
 * components contained within the container.
 * @param container
 * @param prompt
 */
public JCPromptHelper(Container container, JLabel prompt) {
	this.prompt = prompt;
	container.addContainerListener(this);
}

/**
 * Constructs a prompt helper class on the specified container. <br><b>Note:</b> For
 * this constructor to be useful, the <CODE>setPromptLabel</CODE> method must be
 * called in its future.
 * @param container
 */
public JCPromptHelper(Container container) {
	this(container, null);
}

/**
 * Sets the prompt label to use.  If this is not specified either by
 * invoking it in this method or in the constructor, then this class will not
 * be useful.
 * @param prompt
 */
public void setPromptLabel(JLabel prompt) {
	this.prompt = prompt;
}

/**
 * Returns the prompt label being used.
 * @return prompt label being used
 */
public JLabel getPromptLabel() {
	return prompt;
}

protected void addChild(Component c) {
	c.addFocusListener(this);
	if (c instanceof Container) {
		((Container) c).addContainerListener(this);
		Component kids[] = ((Container) c).getComponents();
		if (kids != null && kids.length != 0) {
			// add all the kiddies too
			for (int i = 0; i < kids.length; i++) {
				addChild(kids[i]);
			}
		}
	}
}

//
// Container Listener implementation
//

/**
 * This method is internal. You should not have to call this method.
 * @param e
 */
public void componentAdded(ContainerEvent e) {
	addChild(e.getChild());
}

protected void removeChild(Component c) {
	c.removeFocusListener(this);
	if (c instanceof Container) {
		((Container) c).removeContainerListener(this);
		Component kids[] = ((Container) c).getComponents();
		if (kids != null && kids.length != 0) {
			// add all the kiddies too
			for (int i = 0; i < kids.length; i++) {
				removeChild(kids[i]);
			}
		}
	}
}

/**
 * This method is internal. You should not have to call this method.
 * @param e
 */
public void componentRemoved(ContainerEvent e) {
	removeChild(e.getChild());
}

//
// Focus Listener implementation
//
/**
 * Sets the tool tip text when the field gains focus.
 * @param e
 */
public void focusGained(FocusEvent e) {
	if (e.getComponent() instanceof JCFieldComponent) {
		// set the prompt label to the tool tip text of the child
		prompt.setText(((JComponent) e.getComponent()).getToolTipText());
	}
	else if (e.getComponent().getParent() instanceof JCFieldComponent) {
		// as is the case in a spin box or a combo box
		prompt.setText(((JComponent) e.getComponent().getParent()).getToolTipText());
	}
}

/**
 * Removes the tool tip text when the field loses focus.
 * @param e
 */
public void focusLost(FocusEvent e) {
	if (e.getComponent() instanceof JCFieldComponent) {
		// clear the prompt label
		prompt.setText("");
	}
	else if (e.getComponent().getParent() instanceof JCFieldComponent) {
		// as is the case in a spin box or a combo box
		prompt.setText("");
	}
}

}
