/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCSpinField.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:14 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.field;

import com.klg.jclass.field.validate.AbstractValidator;
import com.klg.jclass.field.validate.JCValidator;
import com.klg.jclass.util.swing.AbstractSpinBox;
import com.klg.jclass.util.swing.JCSpinBoxEditor;
import com.klg.jclass.util.swing.JCSpinBoxRenderer;
import com.klg.jclass.util.value.JCValueListener;
import com.klg.jclass.util.value.JCValueModel;

import javax.swing.*;
import java.awt.*;
import java.awt.event.FocusEvent;
import java.beans.PropertyChangeListener;
import java.util.Locale;

/**
 * A spin field allows the user to spin through a range of values. The range
 * of values shown in the spin field is determined by one of two mechanisms.
 * The upper and lower limits on the range can be determined by predefining
 * <code>min</code> and <code>max</code> values. The intermediate values are controlled by the
 * increment property. A pick list offers all the possible values that the spin
 * field can contain. This has precedence over the increment method.
 * Only one value at a time is shown in the spin field. Up and down arrow
 * buttons are used to change the value by spinning. The spin field has
 * access to the same validator functions as the text field.
 */
public class JCSpinField extends AbstractSpinBox
			             implements JCFieldComponent
{

protected Field       field;
protected JTextField  text;
protected boolean     required = true;

/**
 * Contains the version number of the JClass product.
 */
public static final String version =
    com.klg.jclass.field.JCVersion.getVersionString();

/**
 * Constructs a <code>JCSpinField</code>.
 */
public JCSpinField()
{
    // Set the combo box to be editable by default
    setEditor(new FieldEditor());
    setRenderer(new FieldRenderer());
    text = (JTextField)editor.getEditorComponent();
    createField(null);
}

/**
 * Constructs an empty <code>JCSpinField</code>, which expects an object of the
 * specified class.
 * @param c
 */
public JCSpinField(Class c)
{
    this();
    field.setClass(c);
}

/**
 * Constructs an empty <code>JCSpinField</code>, which expects an object of the
 * specified class (which uses the specified validator).
 * @param c
 * @param v
 */
public JCSpinField(Class c, JCValidator v)
{
    this();
    field.setValidator(v);
    field.setClass(c);
}

/**
 * Constructs an empty <code>JCSpinField</code>, which uses the specified value model
 * to contain its value.
 * @param m
 */
public JCSpinField(JCValueModel m)
{
    this();
    field.setValueModel(m);
}

/**
 * Constructs an empty <code>JCSpinField</code>, which uses the specified value model
 * to contain its value and which uses the specified validator.
 * @param m
 * @param v
 */
public JCSpinField(JCValueModel m, JCValidator v)
{
    this();
    field.setValidator(v);
    field.setValueModel(m);
}


/**
 * Gets the Bean property which wraps the validator, data model, and <code>Field</code>
 * objects properties into one property.
 * @return
 */
public DataProperties getDataProperties()
{
    return field.getDataProperties();
}

/**
 * Sets the Bean property which wraps the validator, data model, and <code>Field</code>
 * objects properties into one property.
 * @param dp
 */
public void setDataProperties(DataProperties dp)
{
    field.setDataProperties(dp);
}

/**
 * Adds a <code>PropertyChangeListener</code> to the listener list.
 *
 * @param listener  the <code>PropertyChangeListener</code> to be added
 */
public void addPropertyChangeListener(PropertyChangeListener listener)
{
    super.addPropertyChangeListener(listener);
    if (field != null) {
        // pass through to field
        field.addPropertyChangeListener(listener);
    }
}

/**
 * Removes a <code>PropertyChangeListener</code> from the listener list.
 * This removes a <code>PropertyChangeListener</code> that was registered
 * for all properties.
 *
 * @param listener  the <code>PropertyChangeListener</code> to be removed
 */
public void removePropertyChangeListener(PropertyChangeListener listener)
{
    super.removePropertyChangeListener(listener);
    if (field != null) {
        // pass through to field
        field.removePropertyChangeListener(listener);
    }
}

/**
 * Adds a listener for changes to the value.
 * @param listener
 */
public void addValueListener(JCValueListener listener)
{
    field.addValueListener(listener);
}

/**
 * Removes a listener for changes to the value.
 * @param listener
 */
public void removeValueListener(JCValueListener listener)
{
    field.removeValueListener(listener);
}

protected void createField(JCValidator validator)
{
    field = new Field(this, validator);
}

/**
 * Gets the value of the <code>About</code> property.
 * The <code>About</code> property displays contact information for
 * Quest Software in the Bean box.
 * @return
 */
public String getAbout()
{
    return version;
}

/**
 * Sets the value of the <code>About</code> property.
 * The <code>About</code> property displays contact information for
 * Quest Software in the Bean box.
 * @param s
 */
public synchronized void setAbout(String s)
{
}

/**
 * Returns the text field's preferred height. Designed to be overridden.
 * @return text field's preferred height
 */
protected int preferredHeightOfView()
{
    int height;

    if (isEditable()) {
        height = editor.getEditorComponent().getPreferredSize().height;
    }
    else {
        if (getValue() == null) {
            setValue(doSpin(true));
        }
        height = renderer.getComponent(this, getValue()).getPreferredSize().height + renderPanel.getInsets().top + renderPanel.getInsets().bottom;
    }
    return height;
}


/**
 * Determines whether the <code>JCSpinBox</code> field is editable. An editable <code>JCSpinBox</code>
 * allows the user to type into the field or select an item from the list
 * to initialize the field, after which it can be edited. (The editing affects
 * only the field, the list item remains intact.) A non editable <code>JCSpinBox</code>
 * displays the selected item in the field, but the selection cannot be
 * modified.
 *
 * @param aFlag a boolean value, where <code>true</code> indicates that the field is editable
 */
public void setEditable(boolean aFlag)
{
    super.setEditable(aFlag);

    if (getValue() == null) {
        setValue(doSpin(true));
    }
}


/**
 * Initializes the field object.
 * This method should not need to be called.
 */
public void addNotify()
{
    super.addNotify();
    field.addNotify();
}

/**
 * Sets the background of the spin field.
 * @param background
 */
public void setBackground(Color background)
{

    // if the user tries to change the background, store the background
    // in the field object. Do not change the background immediately because
    // the field might be invalid.

    if (field == null || field.internalBackgroundChange == true) {
        super.setBackground(background);
    }
    else {
        field.validBackground = background;

        // if the field is not invalid, set the color now
        if (field.getState() != Field.INVALID) {
            super.setBackground(background);
        }
    }
}

/**
 * Sets the foreground of the spin field.
 * @param foreground
 */
public void setForeground(Color foreground)
{

    // if the user tries to change the foreground, store the foreground
    // in the field object. Do not change the foreground immediately because
    // the field might be invalid.

    if (field == null || field.internalForegroundChange == true) {
        super.setForeground(foreground);
    }
    else {
        field.validForeground = foreground;

        // if the field is not invalid, set the color now.
        if (field.getState() != Field.INVALID) {
            super.setForeground(foreground);
        }
    }
}

protected void checkArrowButtons()
{
    int arrow_info;

    if (field == null || getValidator() == null) {
        setArrowEnabled(upArrow, false);
        setArrowEnabled(downArrow, false);
        return;
    }
    if (field.getState() == field.INVALID || field.getValue() == null) {
        // if the field is invalid or null then we can't calculate spinability
        // off of it so we use the default value
        arrow_info = field.getValidator().calculateSpinability(
            field.getValidator().getDefaultValue());
    }
    else {
        arrow_info = field.getValidator().calculateSpinability(
            field.getValue());
    }

    if (upArrow != null) {
        setArrowEnabled(upArrow, (arrow_info & AbstractValidator.UP) != 0);
    }
    if (downArrow != null) {
        setArrowEnabled(downArrow, (arrow_info & AbstractValidator.DOWN) != 0);
    }

}

/**
 * Performs spinning given the direction.
 * @param spin_up <code>true</code> means to spin up and <code>false</code> means to spin down
 * @return
 */
protected Object doSpin(boolean spin_up)
{
    JCValidator validator = getValidator();
    if (validator == null) {
        return null;
    }

    // commit any current edits before doing the spin
    field.commitEdit();

    // get current field value
    Object field_val = field.getValue();

    // spin from default value if current value is null or invalid
    if (field_val == null || !validator.validate(field_val)) {
        field_val = validator.getDefaultValue();
    }

    // find which subfield to spin
    if (((AbstractValidator)validator).getSpinPolicy() ==
        AbstractValidator.SPIN_SUBFIELD) {
        validator.inferSubField(text.getSelectionStart(),
                                text.getSelectionEnd());
    }

    // call the correct spin method
    Object new_obj;
    if (spin_up == true) {
        new_obj = validator.spinUp(field_val);
    }
    else {
        new_obj = validator.spinDown(field_val);
    }

    // set the state to valid
    if (new_obj != null) {
        field.setStateValid();
    }

    return new_obj;
}

/**
 * Returns the new value generated when spinning up a notch.
 * @return new value generated when spinning up a notch
 */
protected Object spinUp()
{
    return doSpin(true);
}

/**
 * Returns the new value generated when spinning down a notch.
 * @return new value generated when spinning down a notch
 */
protected Object spinDown()
{
    return doSpin(false);
}

protected void processFocusLost(FocusEvent e)
{
}

//
// JCFieldComponent implementation
//

/**
 * Returns the value for the text field.
 * @return value for the text field
 */
public Object getValue()
{
    return field.getValue();
}

/**
 * Sets the value for the text field.
 * @param value
 */
public void setValue(Object value)
{
    field.setValue(value);
}

/**
 * Returns the data model being used by the field.
 * @see com.klg.jclass.util.value.JCValueModel
 * @return data model used by the field
 */
public JCValueModel getValueModel()
{
    if (field != null) {
        return field.getValueModel();
    }
    return null;
}

/**
 * Sets the data model being used by the field.
 * @see com.klg.jclass.util.value.JCValueModel
 * @param model
 */
public void setValueModel(JCValueModel model)
{
    if (field != null) {
        field.setValueModel(model);
    }
}

/**
 * Returns the validator currently in use.
 * @return validator currently in use
 */
public JCValidator getValidator()
{
    return field.getValidator();
}

/**
 * Sets the validator to use.
 * @param v
 */
public void setValidator(JCValidator v)
{
    field.setValidator(v);
}

/**
 * Returns the <code>Invalid</code> properties. The <code>Invalid</code> properties are
 * <code>beepOnInvalid</code>, <code>invalidForeground</code>, <code>invalidBackground</code>,
 * <code>defaultValue</code>, and <code>invalidPolicy</code>.
 * @return <code>Invalid</code> properties
 */
public JCInvalidInfo getInvalidInfo()
{
    return field.getInvalidInfo();
}

/**
 * Sets the <code>Invalid</code> properties. The <code>Invalid</code> properties are
 * <code>beepOnInvalid</code>, <code>invalidForeground</code>, <code>invalidBackground</code>,
 * <code>defaultValue</code>, and <code>invalidPolicy</code>.
 * @param info
 */
public void setInvalidInfo(JCInvalidInfo info)
{
    field.setInvalidInfo(info);
}

/**
 * Returns the state of the component:
 * <code>VALID</code>, <code>INVALID</code>, or <code>UNDER_EDIT</code>.
 * @return <code>VALID</code>, <code>INVALID</code>, or <CODE>UNDER_EDIT</CODE>
 */
public int getState()
{
    return field.getState();
}

/**
 * Returns a flag indicating the selection status on entering the field.
 * @see #setSelectOnEnter
 * @return flag indicating the selection status on entering the field
 */
public boolean getSelectOnEnter()
{
    return field.getSelectOnEnter();
}

/**
 * Sets a flag to determines whether or the value in the field is
 * selected when the component gains focus.  The default value is
 * <code>false</code>.
 * @param select
 */
public void setSelectOnEnter(boolean select)
{
    field.setSelectOnEnter(select);
}

/**
 * Commits the current field under edit.
 */
public void commitEdit()
{
    field.commitEdit();
}

/**
 * Returns <code>true</code> if the field is required.
 * @return <code>true</code> if the field is required
 */
public boolean isRequired()
{
    return required;
}

/**
 * Sets the required property for the current field.
 * @param required
 */
public void setRequired(boolean required)
{
    this.required = required;
}

class FieldEditor extends JTextField implements JCSpinBoxEditor {
    /**
     * Returns the component that should be added to the tree hierarchy for
     * this editor.
     * @return component that should be added to the tree hierarchy
     */
    public Component getEditorComponent()
    {
        return this;
    }

    /** Sets the item that should be edited. Cancels any editing if necessary.
     * @param anObject
     **/
    public void setItem(Object anObject)
    {
        field.setValue(anObject);
    }

    /** Returns the edited item.
     * @return edited item
     **/
    public Object getItem()
    {
        return field.getValue();
    }

} // End of FieldEditor

class FieldRenderer extends JLabel implements JCSpinBoxRenderer {
    public FieldRenderer()
    {
        super();
        setOpaque(true);
    }

    public Component getComponent(AbstractSpinBox spinBox, Object value)
    {
        // format the object and set it on the renderer
        if (field != null && getValidator() != null) {
            setText(getValidator().format(value));
        }
        return this;
    }
} // End of FieldRenderer

/**
 * Sets the locale being used by the spin field; sets it on the validator
 * as well.
 * @param l the new locale to be used
 */
public void setLocale(Locale l)
{
    super.setLocale(l);
    if (field != null) {
        JCValidator v = getValidator();
        if (v instanceof AbstractValidator) {
            AbstractValidator av = (AbstractValidator)v;
            av.setLocale(l);
        }

        // Need to reformat the current value to the new locale
        JCValueModel model = getValueModel();
        if (model != null) {
            field.setValue(model.getValue());
        }
    }
}

}
