/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCTextField.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:14 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.field;

import com.klg.jclass.field.validate.AbstractValidator;
import com.klg.jclass.field.validate.JCValidator;
import com.klg.jclass.util.value.JCValueListener;
import com.klg.jclass.util.value.JCValueModel;

import javax.swing.*;
import java.awt.*;
import java.beans.PropertyChangeListener;
import java.util.Locale;

/**
 * <CODE>JCTextField</CODE> has all the capability of <CODE>JTextField</CODE>,
 * as well as the ability to validate its contents.
 *
 * @see javax.swing.JTextField
 */
public class JCTextField extends JTextField
			             implements JCFieldComponent
{

protected Field    field;
protected boolean  required = true;

/**
 * Contains the version number of the JClass product.
 */
public static final String version =
    com.klg.jclass.field.JCVersion.getVersionString();

/**
 * Constructs an empty <code>JCTextField</code>.
 */
public JCTextField()
{
	this(null, 0);
}

/**
 * Constructs an empty <code>JCTextField</code>, which expects an object of the
 * specified class.
 * @param c the class type the field will expect
 */
public JCTextField(Class c)
{
	this();
	field.setClass(c);
}

/**
 * Constructs an empty <code>JCTextField</code>, which expects an object of the
 * specified class and uses the specified validator.
 * @param c the class type the field will expect
 * @param v the validator that validates input in the field
 */
public JCTextField(Class c, JCValidator v)
{
	this();
	field.setValidator(v);
	field.setClass(c);
}

/**
 * Constructs an empty <code>JCTextField</code>, which uses the specified value
 * model to contain its value.
 * @param m the value model that contains the value
 */
public JCTextField(JCValueModel m)
{
	this();
	field.setValueModel(m);
}

/**
 * Constructs an empty <code>JCTextField</code>, which uses the specified value
 * model to contain its value and uses the specified validator.
 * @param m the value model that contains the value
 * @param v the validator that validates input in the field
 */
public JCTextField(JCValueModel m, JCValidator v)
{
	this();
	field.setValidator(v);
	field.setValueModel(m);
}

/**
 * Constructs a <code>JCTextField</code> initialized with the specified text.
 * @param text the text with which to initialize the field
 */
public JCTextField(String text)
{
	this(text, 0);
}

/**
 * Constructs a <code>JCTextField</code> with the specified number of columns.
 * @param columns the number of columns
 */
public JCTextField(int columns)
{
	this(null, columns);
}

/**
 * Constructs an empty <code>JCTextField</code> with the specified number of
 * columns and initialized with the specified text.
 * @param text the text with which to initialize the field
 * @param columns the number of columns
 */
public JCTextField(String text, int columns)
{
	super(text, columns);
	createField(null);
}

/**
 * Gets the Bean property which wraps the validator, data model, and
 * <code>Field</code> objects properties into one property.
 * @return the <code>DataProperties</code> Bean wrapper
 */
public DataProperties getDataProperties()
{
	return(field.getDataProperties());
}

/**
 * Sets the Bean property which wraps the validator, data model, and
 * <code>Field</code> objects properties into one property.
 * @param dp the new <code>DataProperties</code> value
 */
public void setDataProperties(DataProperties dp)
{
	field.setDataProperties(dp);
}

/**
 * Adds a <code>PropertyChangeListener</code> to the listener list.
 *
 * @param listener  the <code>PropertyChangeListener</code> to be added
 */
public void addPropertyChangeListener(PropertyChangeListener listener)
{
	super.addPropertyChangeListener(listener);
	if (field != null) {
		// pass through to field
		field.addPropertyChangeListener(listener);
	}
}

/**
 * Removes a <code>PropertyChangeListener</code> from the listener list.
 * This removes a <code>PropertyChangeListener</code> that was registered
 * for all properties.
 *
 * @param listener  the <code>PropertyChangeListener</code> to be removed
 */
public void removePropertyChangeListener(PropertyChangeListener listener)
{
	super.removePropertyChangeListener(listener);
	if (field != null) {
		// pass through to field
		field.removePropertyChangeListener(listener);
	}
}

/**
 * Adds a listener to listen for changes to the value.
 * @param listener  the <code>JCValueListener</code> to be added
 */
public void addValueListener(JCValueListener listener)
{
	field.addValueListener(listener);
}

/**
 * Removes a listener which listens for changes to the value.
 * @param listener  the <code>JCValueListener</code> to be removed
 */
public void removeValueListener(JCValueListener listener)
{
	field.removeValueListener(listener);
}

/**
 * Initializes the field object.
 * This method should not need to be called.
 */
public void addNotify()
{
	super.addNotify();
	field.addNotify();
}

/**
 * Creates the field object which controls the validation.
 * @param validator the object which does the validation
 */
protected void createField(JCValidator validator)
{
	field = new Field(this, null);
}

/**
 * Gets the value of the <code>About</code> property. The <code>About</code>
 * property displays contact information for Quest Software in the
 * Bean box.
 * @return the about String
 */
public String getAbout()
{
	return(version);
}

/**
 * Sets the value of the <code>About</code> property. The <code>About</code>
 * property displays contact information for Quest Software in the
 * bean box.
 * @param s the new about String (ignored)
 */
public synchronized void setAbout(String s) {}

/**
 * Sets the background color of the text field.
 * @param background the new background color
 */
public void setBackground(Color background)
{
	// If the user tries to change the background, store the background
	// in the field object. Do not change the background immediately because
	// the field might be invalid.

	if (field == null || field.internalBackgroundChange == true) {
		super.setBackground(background);
	}
	else {
		field.validBackground = background;

		// If the field is not invalid, set the color now
		if (field.getState() != Field.INVALID) {
			super.setBackground(background);
		}
	}
}

/**
 * Sets the foreground color of the text field.
 * @param foreground the new foreground color
 */
public void setForeground(Color foreground)
{
	// If the user tries to change the foreground, store the foreground
	// in the field object. Do not change the foreground immediately because
	// the field might be invalid.

	if (field == null || field.internalForegroundChange == true) {
		super.setForeground(foreground);
	}
	else {
		field.validForeground = foreground;

		// If the field is not invalid, set the color now.
		if (field.getState() != Field.INVALID) {
			super.setForeground(foreground);
		}
	}
}

//
// Implement methods for JCFieldComponent
//

/**
 * Returns the value for the text field.
 * @return the current value of the field
 */
public Object getValue()
{
	return(field.getValue());
}

/**
 * Sets the value for the text field.
 * @param value the new value of the field
 */
public void setValue(Object value)
{
	field.setValue(value);
}

/**
 * Returns the data model being used by the field.
 * @return the current value model for the field
 * @see com.klg.jclass.util.value.JCValueModel
 */
public JCValueModel getValueModel()
{
	return(field.getValueModel());
}

/**
 * Sets the data model being used by the field.
 * @param model the new value model for the field
 * @see com.klg.jclass.util.value.JCValueModel
 */
public void setValueModel(JCValueModel model)
{
	field.setValueModel(model);
}

/**
 * Returns the validator currently in use.
 * @return the current validator for the field
 * @see com.klg.jclass.field.validate.JCValidator
 */
public JCValidator getValidator()
{
	return(field.getValidator());
}

/**
 * Sets the validator to use.
 * @param validator the new validator for the field
 * @see com.klg.jclass.field.validate.JCValidator
 */
public void setValidator(JCValidator validator)
{
	field.setValidator(validator);
}

/**
 * Returns the <code>Invalid</code> properties. The <code>Invalid</code>
 * properties are <code>beepOnInvalid</code>, <code>invalidForeground</code>,
 * <code>invalidBackground</code>, <code>defaultValue</code>, and
 * <code>invalidPolicy</code>.
 * @return The current invalid information object
 */
public JCInvalidInfo getInvalidInfo()
{
	return(field.getInvalidInfo());
}

/**
 * Sets the <code>Invalid</code> properties. The <code>Invalid</code> properties
 * are <code>beepOnInvalid</code>, <code>invalidForeground</code>,
 * <code>invalidBackground</code>, <code>defaultValue</code>, and
 * <code>invalidPolicy</code>.
 * @param info the new <code>Invalid</code> information object
 */
public void setInvalidInfo(JCInvalidInfo info)
{
	field.setInvalidInfo(info);
}

/**
 * Returns the state of the component.
 * @return the current state; will be one of
 * <code>JCFieldComponent.VALID</code>, <code>JCFieldComponent.INVALID</code>,
 * or <code>JCFieldComponent.UNDER_EDIT</code>
 */
public int getState()
{
	return(field.getState());
}

/**
 * Returns a flag indicating the selection status on entering the field.
 * @return whether or not to set the selection on entering the field
 * @see #setSelectOnEnter
 */
public boolean getSelectOnEnter()
{
	return(field.getSelectOnEnter());
}

/**
 * Sets a flag that determines whether or not the value in the field is
 * selected when the component gains focus. The default value is
 * <code>false</code>.
 * @param select if <code>true</code>, set the selection on entering the field
 */
public void setSelectOnEnter(boolean select)
{
	field.setSelectOnEnter(select);
}

/**
 * Commits the current field under edit.
 */
public void commitEdit()
{
	field.commitEdit();
}

/**
 * Returns <code>true</code> if the field is required.
 * @return whether this field is required
 */
public boolean isRequired()
{
	return(required);
}

/**
 * Sets the required property for the current field.
 * @param required if <code>true</code>, this field is required
 */
public void setRequired(boolean required)
{
	this.required = required;
}

/**
 * Overrides <code>setEditable</code> to make sure the background is set to the
 * correct color immediately (otherwise the <code>setBackground()</code> method
 * above will intercept the change and nothing will happen until the user tries
 * to edit the field).
 * @param b if <code>true</code>, the field is editable
 */
public void setEditable(boolean b)
{
	super.setEditable(b);
	if (field != null) {
	    super.setBackground(field.validBackground);
	}
}

/**
 * Sets the locale being used by the text field; sets it on the validator
 * as well.
 * @param l the new locale to be used
 */
public void setLocale(Locale l)
{
	super.setLocale(l);
	if (field != null) {
		JCValidator v = getValidator();
		if (v instanceof AbstractValidator) {
			AbstractValidator av = (AbstractValidator)v;
			av.setLocale(l);
		}

        // Need to reformat the current value to the new locale
        JCValueModel model = getValueModel();
        if (model != null) {
            field.setValue(model.getValue());
        }
	}
}

}
