/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: BigDecimalValidatorEditor.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:14 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.field.beans;

import java.math.BigDecimal;
import com.klg.jclass.field.*;
import com.klg.jclass.field.validate.*;
import com.klg.jclass.util.swing.JCGridLayout;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JComboBox;

public class BigDecimalValidatorEditor extends AbstractValidatorEditor {

protected Number   max;
protected Number   min;
protected Number   increment;
protected String   display_pattern;
protected boolean  allow_null;
protected boolean  currency;
protected Number default_value;
protected JTextField max_field;
protected JTextField min_field;
protected JTextField increment_field;
protected JTextField display_pattern_field;
protected JComboBox allow_null_field;
protected JComboBox currency_field;
protected JTextField default_value_field;

public BigDecimalValidatorEditor() {

	setLayout(new JCGridLayout(7, 2, 5, 5));

	// setup Max property
	add(new JLabel("Max:"));
	add(max_field = new JTextField(TEXTFIELD_SIZE));

	// setup Min property
	add(new JLabel("Min:"));
	add(min_field = new JTextField());

	// setup increment property
	add(new JLabel("Increment:"));
	add(increment_field = new JTextField());

	// setup display pattern
	add(new JLabel("Display Pattern:"));
	add(display_pattern_field = new JTextField());

	// setup allow null property
	add(new JLabel("Allow Null:"));
	add(allow_null_field = new JComboBox(bool_vals));

	// setup allow null property
	add(new JLabel("Currency:"));
	add(currency_field = new JComboBox(bool_vals));

	// setup default value property
	add(new JLabel("Default Value:"));
	add(default_value_field = new JTextField());

}

/**
 * Sets the current Double validator values into the editors.
 */
public void setProperties(JCValidator vp) {
	JCBigDecimalValidator bigdecimal_vp = (JCBigDecimalValidator)vp;

	// get the current values from the validator
	max = (Number)bigdecimal_vp.getMax();
	min = (Number)bigdecimal_vp.getMin();
	increment = bigdecimal_vp.getIncrement();
	display_pattern = bigdecimal_vp.getDisplayPattern();
	allow_null = bigdecimal_vp.getAllowNull();
	currency = bigdecimal_vp.isCurrency();
	default_value = (Number)bigdecimal_vp.getDefaultValue();

	// set the values into the property editor fields
	max_field.setText(max.toString());
	min_field.setText(min.toString());
	increment_field.setText(increment.toString());
	display_pattern_field.setText(display_pattern);

	if (allow_null == true) {
		allow_null_field.setSelectedIndex(1);
	}
	else {
		allow_null_field.setSelectedIndex(0);
	}

	if (currency == true) {
		currency_field.setSelectedIndex(1);
	}
	else {
		currency_field.setSelectedIndex(0);
	}

	if (default_value != null) {
		default_value_field.setText(default_value.toString());
	}
}

/**
 * Creates a new validator from the fields set.
 */
protected ValidatorInfo getValidatorInfo() {

	String text;

	// get the values from the text fields
	text = max_field.getText();
	if (text.length() == 0) {
		max = new BigDecimal(Double.MAX_VALUE);
	}
	else {
		max = new BigDecimal(text);
	}

	// if min field is empty, set it to the default
	text = min_field.getText();
	if (text.length() == 0) {
		min = new BigDecimal(-Double.MAX_VALUE);
	}
	else {
		min = new BigDecimal(text);
	}

	// if increment is null, set it to the default
	text = increment_field.getText();
	if (text.length() == 0) {
		increment = new BigDecimal(1);
	}
	else {
		increment = new BigDecimal(text);
	}

	// if display pattern is null, set it to its default value
    display_pattern = display_pattern_field.getText();
	
	if (allow_null_field.getSelectedIndex() == 0) {
		allow_null = false;
	}
	else {
		allow_null = true;
	}

	if (currency_field.getSelectedIndex() == 0) {
		currency = false;
	}
	else {
		currency = true;
	}

	text = default_value_field.getText();
	if (text.length() == 0) {
		default_value = new BigDecimal(0.0);
	}
	else {
		default_value = new BigDecimal(text);
	}

	return (getInfo());
}

/**
 * Returns a ValidatorInfo object which contains the code generation string
 * and a new instance of the validator.
 */
protected ValidatorInfo getInfo() {
	JCBigDecimalValidator new_vp;

	// create and set the new code generation string
	String init_string = "new com.klg.jclass.field.validate.JCBigDecimalValidator(" + 
	    "null, new java.math.BigDecimal((double)" + min.doubleValue() + 
		"), new java.math.BigDecimal((double)" + max.doubleValue() + 
        "), new java.math.BigDecimal((double)" + increment.doubleValue() + 
        "), \"" + display_pattern + "\", " + allow_null + ", " + currency +
		", false, null, new java.math.BigDecimal((double)" + default_value.doubleValue() + "))";

	// create new validator
	new_vp = new JCBigDecimalValidator(null, min, max, increment, 
									   display_pattern,
									   allow_null, currency, false,
									   null, default_value);

	// create new validator info object
	ValidatorInfo vi = new ValidatorInfo(new_vp, init_string);

	// return the new validator object
	return(vi);
}

}




