/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: DataPropertiesEditor.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:14 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.field.beans;

import com.klg.jclass.beans.AbstractBeanEditor;
import com.klg.jclass.field.*;
import com.klg.jclass.field.validate.*;
import com.klg.jclass.util.swing.JCColumnLayout;
import com.klg.jclass.util.swing.JCBrace;
import com.klg.jclass.util.value.JCValueModel;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

public class DataPropertiesEditor extends AbstractBeanEditor
    implements ActionListener, ChangeListener {

protected JTabbedPane tabPane;
protected TabTypePage typePage;
protected TabFieldPage fieldPage;
protected DataProperties dp = null;
protected AbstractValidatorEditor currentChild = null;
protected JButton commitButton;
public static final String TYPE_HELP = "TypeHelp";
public static final String FIELD_HELP = "FieldHelp";
public static final String VALUE_HELP = "ValueHelp";
public static final String BYTE_VALIDATOR_HELP = "ByteValidatorHelp";
public static final String DOUBLE_VALIDATOR_HELP = "DoubleValidatorHelp";
public static final String FLOAT_VALIDATOR_HELP = "FloatValidatorHelp";
public static final String INTEGER_VALIDATOR_HELP = "IntegerValidatorHelp";
public static final String LONG_VALIDATOR_HELP = "LongValidatorHelp";
public static final String SHORT_VALIDATOR_HELP = "ShortValidatorHelp";
public static final String STRING_VALIDATOR_HELP = "StringValidatorHelp";
public static final String BIGDECIMAL_VALIDATOR_HELP = "BigDecimalValidatorHelp";
public static final String IPADDRESS_VALIDATOR_HELP = "IPAddressValidatorHelp";
public static final String DATETIME_VALIDATOR_HELP = "DateTimeValidatorHelp";
public static final String DATE_VALIDATOR_HELP = "DateValidatorHelp";
public static final String TIME_VALIDATOR_HELP = "TimeValidatorHelp";
public static final String CURRENCY_VALIDATOR_HELP = "CurrencyValidatorHelp";

public DataPropertiesEditor() {

	// call the super to add a html help box
	super("com.klg.jclass.field.beans.resources.LocaleInfo", TYPE_HELP,
		  "DataProperties Editor");

	// setup tab Pane and create tab pages
	tabPane.setPreferredSize(new Dimension(450,410));
	createTabPages();
	tabPane.addChangeListener(this);

	// set the button to be disabled by default
	commitButton.setEnabled(false);
	commitButton.addActionListener(this);
}

/**
 * Set the preferred size for the editor.
 */
public Dimension getPreferredSize() {
	return (new Dimension(450,550));
}

public Component getEditorComponent() {
	JPanel panel = new JPanel();
	panel.setLayout(new JCColumnLayout(SwingConstants.CENTER));

	// add tabPane
	tabPane = new JTabbedPane();

	// add tab pane and button to panel
	panel.add(tabPane);
	panel.add(JCBrace.createVerticalBrace(2));
	panel.add(commitButton = new JButton("Apply"));

	return (panel);
}

/**
 * Creates the tab pages.
 */
protected void createTabPages() {
	// create the tab pages
	typePage = new TabTypePage(this);
	fieldPage = new TabFieldPage();
	tabPane.addTab("Type", typePage);
	tabPane.addTab("Invalid", fieldPage);

}

/**
 * This method will change the help text in the html help pane.
 */
public void stateChanged(ChangeEvent e) {

	// change the help text for the different tabs
	if (tabPane.getSelectedComponent() == typePage) {
		setHelpText(TYPE_HELP);
	}
	else {
		setHelpText(FIELD_HELP);
	}
	tabPane.validate();
}

/**
 * Returns string used for code generation
 */
public String getJavaInitializationString() {
	String init_string = "new com.klg.jclass.field.DataProperties(";

	init_string += typePage.getInitString() + ", ";
	init_string += fieldPage.getInitString() + ")";

	return init_string;
}

public Object getValue() {

	if (typePage.validatorPage == null) {
		return null;
	}

	// get validator
	JCValidator validator = typePage.validatorPage.getNewValidator();

	// get model and set value in model
	JCValueModel model = typePage.getValueModel(typePage.tabValuePage.type);
	if (model != null) {
		model.setValue(typePage.getValue());
	}

	// get field invalidInfo
	JCInvalidInfo info = fieldPage.getInvalidInfo();

	// return new data properties
	return new DataProperties(validator, model, info);
}

protected String getValueAsText() {
	return "";
}

public void setValue(Object o) {

	dp = (DataProperties)o;
	if (dp == null) {
		return;
	}
	JCValidator v = dp.getValidator();
	JCValueModel vm = dp.getValueModel();
	JCInvalidInfo info = dp.getInvalidInfo();

	if (vm == null || info == null) {
		return;
	}

	String class_name = null;
	Class value_class = vm.getValueClass();

	if (value_class != null) {
		class_name = value_class.getName();
	}

	// set the selected list item
	if (class_name != null) {
		typePage.type_list.setSelectedValue(class_name, true);
	}

	// set the datamodel
	if (vm != null) {
		typePage.valueModel = vm;
	}

	// determine the validator type. Remember: JCCalendarValidator is an
	// instance of JCStringValidator so we must compare JCCalendarValidator
	// first
	if (v == null) {
		currentChild = null;
	}
	else if (v instanceof JCTimeValidator) {
		typePage.createValidatorPage(TabTypePage.TIME_VALIDATOR);
	}
	else if (v instanceof JCDateValidator) {
		typePage.createValidatorPage(TabTypePage.DATE_VALIDATOR);
	}
	else if (v instanceof JCDateTimeValidator) {
		typePage.createValidatorPage(TabTypePage.DATETIME_VALIDATOR);
	}
	else if (v instanceof JCIntegerValidator) {
		typePage.createValidatorPage(TabTypePage.INTEGER_VALIDATOR);
	}
	else if (v instanceof JCByteValidator) {
		typePage.createValidatorPage(TabTypePage.BYTE_VALIDATOR);
	}
	else if (v instanceof JCShortValidator) {
		typePage.createValidatorPage(TabTypePage.SHORT_VALIDATOR);
	}
	else if (v instanceof JCLongValidator) {
		typePage.createValidatorPage(TabTypePage.LONG_VALIDATOR);
	}
	else if (v instanceof JCDoubleValidator) {
		typePage.createValidatorPage(TabTypePage.DOUBLE_VALIDATOR);
	}
	else if (v instanceof JCFloatValidator) {
		typePage.createValidatorPage(TabTypePage.FLOAT_VALIDATOR);
	}
	else if (v instanceof JCBigDecimalValidator) {
		typePage.createValidatorPage(TabTypePage.BIGDECIMAL_VALIDATOR);
	}
	else if (v instanceof JCIPAddressValidator) {
		typePage.createValidatorPage(TabTypePage.IPADDRESS_VALIDATOR);
	}
	else if (v instanceof JCStringValidator) {
		typePage.createValidatorPage(TabTypePage.STRING_VALIDATOR);
	}

	// get the properties of the validator and set them in the textfields.
	if (typePage.validatorPage != null) {
		typePage.validatorPage.setProperties(v);
	}

	typePage.setProperties(vm);

	// set the field properties.
	fieldPage.setProperties(info);

}

/**
 * Called when the apply button is pressed. Creates a new instance of
 * the validator and sets it.
 */
public void actionPerformed(ActionEvent e) {

	// fire propertyChange notification
	support.firePropertyChange("", null, null);
}

public static void main(String[] args) {
	javax.swing.JFrame f = new javax.swing.JFrame();
	DataPropertiesEditor dpe = new DataPropertiesEditor();
	f.getContentPane().add(dpe);
	f.pack();
	f.show();
}
}





