/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: DateTimeValidatorEditor.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:14 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.field.beans;

import com.klg.jclass.field.JCPopupField;
import com.klg.jclass.field.validate.*;
import com.klg.jclass.util.swing.JCGridLayout;
import com.klg.jclass.beans.StringListEditor;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.JDialog;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Calendar;

public class DateTimeValidatorEditor extends AbstractValidatorEditor
    implements ActionListener {

protected String   format;
protected String   place_holder_chars;
protected String   edit_formats[];
protected boolean  mask_input;
protected boolean  allow_null;
protected Integer  default_detail;
protected Calendar default_value;
protected int millenniumThreshold;
protected JTextField format_field;
protected JTextField place_holder_field;
protected JTextField millenniumThreshold_field;
protected JButton edit_formats_button;
protected StringListEditor edit_formats_editor;
protected JDialog edit_formats_dialog;
protected JComboBox mask_input_field;
protected JComboBox default_detail_field;
protected JComboBox allow_null_field;
protected JCPopupField default_value_field;

public DateTimeValidatorEditor() {

	setLayout(new JCGridLayout(8, 2, 5, 5));

	// setup Mask property
	add(new JLabel("Format:"));
	add(format_field = new JTextField(TEXTFIELD_SIZE));

	// setup place holder chars property
	add(new JLabel("Place Holder Chars:"));
	add(place_holder_field = new JTextField());

	// setup Edit formats property
	add(new JLabel("Edit Formats:"));
	add(edit_formats_button = new JButton("Click here to edit..."));
	edit_formats_button.addActionListener(this);
	createEditFormatsDialog();

	// setup match picklist property
	add(new JLabel("Mask Input:"));
	add(mask_input_field = new JComboBox(bool_vals));

	// setup default detail property
	add(new JLabel("Default Detail:"));
	String[] items = { "Full", "Long", "Medium", "Short" };
	add(default_detail_field = new JComboBox(items));

	// setup allow null property
	add(new JLabel("Allow Null:"));
	add(allow_null_field = new JComboBox(bool_vals));

    // setup millennium threshold
    add(new JLabel("Millennium Threshold"));
    add(millenniumThreshold_field = new JTextField("" + millenniumThreshold));

	// setup default value property
	add(new JLabel("Default Value:"));
	default_value_field = new JCPopupField(Calendar.class,
										   new JCDateTimeValidator());

	add(default_value_field);
}

/**
 * Sets the current Calendar validator values into the editors.
 */
public void setProperties(JCValidator vp) {
	JCDateTimeValidator dt_vp = (JCDateTimeValidator)vp;

	// get the current values from the validator
	format = dt_vp.getFormat();
	place_holder_chars = dt_vp.getPlaceHolderChars();
	edit_formats = dt_vp.getEditFormats();
	mask_input = dt_vp.getMaskInput();
	default_detail = new Integer(dt_vp.getDefaultDetail());
	allow_null = dt_vp.getAllowNull();
	default_value = (Calendar)dt_vp.getDefaultValue();
    millenniumThreshold = dt_vp.getMilleniumThreshold();

	// set the values into the property editor textfields
	format_field.setText(format);
	place_holder_field.setText(place_holder_chars);

	edit_formats_editor.setValue(edit_formats);

	if (mask_input == true) {
		mask_input_field.setSelectedIndex(1);
	}
	else {
		mask_input_field.setSelectedIndex(0);
	}

	default_detail_field.setSelectedIndex(default_detail.intValue());

	if (allow_null == true) {
		allow_null_field.setSelectedIndex(1);
	}
	else {
		allow_null_field.setSelectedIndex(0);
	}

	default_value_field.setValue(default_value.clone());

    millenniumThreshold_field.setText("" + millenniumThreshold);
}

/**
 * Creates a new validator from the fields set.
 */
protected ValidatorInfo getValidatorInfo() {

	String text;

	// get the values from the text fields
	format = format_field.getText();
	place_holder_chars = place_holder_field.getText();

	edit_formats = convertObjectToStringArray(edit_formats_editor.getValue());

	if (mask_input_field.getSelectedIndex() == 0) {
		mask_input = false;
	}
	else {
		mask_input = true;
	}

	default_detail = new Integer(default_detail_field.getSelectedIndex());

	if (allow_null_field.getSelectedIndex() == 0) {
		allow_null = false;
	}
	else {
		allow_null = true;
	}

	default_value = (Calendar)default_value_field.getValue();
	if (default_value == null) {
		default_value = Calendar.getInstance();
	}

    millenniumThreshold = new Integer(millenniumThreshold_field.getText()).intValue();

	return (getInfo());
}

/**
 * Returns a ValidatorInfo object which contains the code generation string
 * and a new instance of the validator.
 */
protected ValidatorInfo getInfo() {

	JCDateTimeValidator new_vp;
	int year = default_value.get(Calendar.YEAR);
	int month = default_value.get(Calendar.MONTH);
	int date = default_value.get(Calendar.DATE);
	int hour = default_value.get(Calendar.HOUR);
	int minute = default_value.get(Calendar.MINUTE);
	int second = default_value.get(Calendar.SECOND);

	// create and set the new code generation string
	String init_string = ("new com.klg.jclass.field.validate.JCDateTimeValidator(null, \"" +
	    format + "\", \"" + place_holder_chars +
		"\", " + edit_formats_editor.getJavaInitializationString() +
        ", " + mask_input + ", " + default_detail.intValue() +
        ", " + allow_null + ", new java.util.GregorianCalendar(" + year + ", "
		+ month + ", " + date + ", " + hour + ", " + minute + ", " +
		second + "), " + millenniumThreshold + ")");

	// create new validator
	new_vp = new JCDateTimeValidator(null, format, place_holder_chars,
									 edit_formats,
									 mask_input, default_detail.intValue(),
									 allow_null, default_value, millenniumThreshold);

	// create new validator info object
	ValidatorInfo vi = new ValidatorInfo(new_vp, init_string);

	// return the new validator object
	return(vi);

}

/**
 * Shows the edit formats property editor when the button is pressed.
 */
public void actionPerformed(ActionEvent e) {
	edit_formats_dialog.show();

}

/**
 * Creates a dialog for the edit formats editor.
 */
public void createEditFormatsDialog() {

	// create a Modal dialog with the edit formats editor
	edit_formats_dialog = new JDialog(JOptionPane.getFrameForComponent(
							 	      this.getParent()), true);
	edit_formats_editor = new StringListEditor();
	edit_formats_dialog.getContentPane().add(edit_formats_editor);
	edit_formats_dialog.pack();
}

/**
 * Converts the array of Objects to an array of Strings.
 */
public String[] convertObjectToStringArray(Object value) {
	Object[] array = (Object[])value;
	if (array == null) {
		return null;
	}
	String[] string_array = new String[array.length];
	for (int i = 0; i < array.length; i++) {
		string_array[i] = (String)array[i];
	}
	return string_array;
}

}




