/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: LongValidatorEditor.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:14 $ $Locker:  $ Quest Software Inc. 

package com.klg.jclass.field.beans;

import com.klg.jclass.beans.StringListEditor;
import com.klg.jclass.field.*;
import com.klg.jclass.field.validate.*;
import com.klg.jclass.util.swing.JCGridLayout;

import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.ListModel;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JDialog;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

public class LongValidatorEditor extends AbstractValidatorEditor 
    implements ActionListener {

protected Number    max;
protected Number    min;
protected ListModel picklist;
protected boolean   match_picklist;
protected String    display_list[];
protected Number    increment;
protected String    display_pattern;
protected boolean   allow_null;
protected boolean   currency;
protected Number    default_value;
protected JTextField max_field;
protected JTextField min_field;
protected JButton picklist_button;
protected JDialog picklist_dialog;
protected ListModelEditor picklist_editor;
protected JComboBox match_picklist_field;
protected JButton display_list_button;
protected JDialog display_list_dialog;
protected StringListEditor display_list_editor;
protected JTextField increment_field;
protected JTextField display_pattern_field;
protected JComboBox allow_null_field;
protected JComboBox currency_field;
protected JTextField default_value_field;

public LongValidatorEditor() {

	setLayout(new JCGridLayout(10, 2, 5, 5));

	// setup Max property
	add(new JLabel("Max:"));
	add(max_field = new JTextField(TEXTFIELD_SIZE));

	// setup Min property
	add(new JLabel("Min:"));
	add(min_field = new JTextField());

	// setup picklist property
	add(new JLabel("PickList:"));
	add(picklist_button = new JButton("Click here to edit..."));
	picklist_button.addActionListener(this);
	createPickListDialog();

	// setup display list property
	add(new JLabel("Display List:"));
	add(display_list_button = new JButton("Click here to edit..."));
	display_list_button.addActionListener(this);
	createDisplayListDialog();

	// setup increment property
	add(new JLabel("Increment:"));
	add(increment_field = new JTextField());

	// setup display pattern
	add(new JLabel("Display Pattern:"));
	add(display_pattern_field = new JTextField());

	// setup match picklist property
	add(new JLabel("Match PickList:"));
	add(match_picklist_field = new JComboBox(bool_vals));

	// setup allow null property 
	add(new JLabel("Allow Null:"));
	add(allow_null_field = new JComboBox(bool_vals));

	// setup allow null property 
	add(new JLabel("Currency:"));
	add(currency_field = new JComboBox(bool_vals));

	// setup default value property
	add(new JLabel("Default Value:"));
	add(default_value_field = new JTextField());
}


/**
 * Sets the current Int validator values into the editors.
 */
public void setProperties(JCValidator vp) {
	JCLongValidator long_vp = (JCLongValidator)vp;

	// get the current values from the validator
	max = (Number)long_vp.getMax();
	min = (Number)long_vp.getMin();
	picklist = long_vp.getPickList();
	match_picklist = long_vp.getMatchPickList();
	display_list = long_vp.getDisplayList();
	increment = (Number)long_vp.getIncrement();
	display_pattern = long_vp.getDisplayPattern();
	allow_null = long_vp.getAllowNull();
	currency = long_vp.isCurrency();
	default_value = (Number)long_vp.getDefaultValue();

	// set the current values into the property editor textfields
	max_field.setText(max.toString());
	min_field.setText(min.toString());
	picklist_editor.setValue(picklist);
	display_list_editor.setValue(display_list);
	increment_field.setText(increment.toString());
	display_pattern_field.setText(display_pattern);

	if (match_picklist == true) {
		match_picklist_field.setSelectedIndex(1);
	}
	else {
		match_picklist_field.setSelectedIndex(0);
	}

	if (allow_null == true) {
		allow_null_field.setSelectedIndex(1);
	}
	else {
		allow_null_field.setSelectedIndex(0);
	}

	if (currency == true) {
		currency_field.setSelectedIndex(1);
	}
	else {
		currency_field.setSelectedIndex(0);
	}

	if (default_value != null) {
		default_value_field.setText(default_value.toString());
	}
}

/**
 * Creates a new validator from the fields set.
 */
protected ValidatorInfo getValidatorInfo() {

	String text;

	// get the values from the text fields

	// if min or max is null, set them to the default values
	text = max_field.getText();
	if (text.length() == 0) {
		max = new Long(Long.MAX_VALUE);
	}
	else {
		max = new Long(text);
	}

	text = min_field.getText();
	if (text.length() == 0) {
		min = new Long(Long.MIN_VALUE);
	}
	else {
		min = new Long(text);
	}

	picklist = (ListModel) picklist_editor.getValue();

	if (match_picklist_field.getSelectedIndex() == 0) {
		match_picklist = false;
	}
	else {
		match_picklist = true;
	}

	display_list = convertObjectToStringArray(display_list_editor.getValue());

	// if increment is null, set it to its default value
	text = increment_field.getText();
	if (text.length() == 0) {
		increment = new Long(1);
	}
	else {
		increment = new Long(text);
	}

	// if display pattern is null, set it to its default value
	display_pattern = display_pattern_field.getText();


	if (allow_null_field.getSelectedIndex() == 0) {
		allow_null = false;
	}
	else {
		allow_null = true;
	}

	if (currency_field.getSelectedIndex() == 0) {
		currency = false;
	}
	else {
		currency = true;
	}

	text = default_value_field.getText();
	if (text.length() == 0) {
		default_value = new Long(0);
	}
	else {
		default_value = new Long(text);
	}
	return (getInfo());
}

/**
 * Returns a ValidatorInfo object which contains the code generation string
 * and a new instance of the validator.
 */
protected ValidatorInfo getInfo() {

	// create and set the new code generation string
	String init_string = ("new com.klg.jclass.field.validate.JCLongValidator(" + 
	    "null, new Long(\"" + min.longValue() + "\"), new Long(\"" + 
		max.longValue() + 
        "\"), " + picklist_editor.getJavaInitializationString() + 
        ", " + match_picklist + ", " + 
        display_list_editor.getJavaInitializationString() + ", new Long(\"" +
 		increment.longValue() + "\"), \"" + display_pattern + "\", " + 
		allow_null + "," + currency + 
        ", false, null, new Long(\"" + default_value.longValue() + "\"))");

	// create new validator
	JCLongValidator new_vp = new JCLongValidator(null, min, max, picklist, 
												 match_picklist, display_list, 
												 increment, display_pattern,
												 allow_null, currency, 
												 false, null, default_value);

	// create new validator info object
	ValidatorInfo vi = new ValidatorInfo(new_vp, init_string);

	// return the new validator object
	return(vi);
}

/**
 * Shows the picklist/display list property editor when the button is pressed.
 */
public void actionPerformed(ActionEvent e) {
	if (e.getSource() == picklist_button) {
		picklist_dialog.show();
	}
	else {
		display_list_dialog.show();
	}
}

/**
 * Creates a dialog for the picklist editor.
 */
public void createPickListDialog() {

	// create a Modal dialog with the picklist editor
	picklist_dialog = new JDialog(JOptionPane.getFrameForComponent(
	                              this.getParent()), true);
	picklist_editor = new ListModelEditor(Long.class);
	picklist_dialog.getContentPane().add(picklist_editor);
	picklist_dialog.pack();
}

/**
 * Creates a dialog for the display list editor.
 */
public void createDisplayListDialog() {

	// create a Modal dialog with the display list editor
	display_list_dialog = new JDialog(JOptionPane.getFrameForComponent(
	                                  this.getParent()), true);
	display_list_editor = new StringListEditor();
	display_list_dialog.getContentPane().add(display_list_editor);
	display_list_dialog.pack();
}

/**
 * Converts the array of Objects to an array of Strings.
 */
public String[] convertObjectToStringArray(Object value) {
	Object[] array = (Object[])value;
	if (array == null) {
		return null;
	}

	String[] string_array = new String[array.length];
	for (int i = 0; i < array.length; i++) {
		string_array[i] = (String)array[i];
	}
	
	return string_array;
}

}

