/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: StringValidatorEditor.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:15 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.field.beans;

import com.klg.jclass.field.*;
import com.klg.jclass.field.validate.*;
import com.klg.jclass.util.swing.JCGridLayout;

import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.ListModel;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;

import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

public class StringValidatorEditor extends AbstractValidatorEditor
    implements ItemListener, ActionListener{

protected String   mask;
protected ListModel picklist;
protected boolean  match_picklist;
protected Integer  num_match;
protected String   place_holder_chars;
protected String   valid_chars;
protected String   invalid_chars;
protected boolean  allow_null;
protected String   default_value;
protected JTextField mask_field;
protected JComboBox  mask_combo;
protected JTextField num_match_field;
protected JTextField place_holder_field;
protected JTextField invalid_char_field;
protected JTextField valid_char_field;
protected JButton picklist_button;
protected ListModelEditor picklist_editor;
protected JComboBox match_picklist_field;
protected JComboBox allow_null_field;
protected JTextField default_value_field;
protected JDialog picklist_dialog = null;

protected PredefinedMask predefined_masks[] = {
        new PredefinedMask("Custom", null),
        new PredefinedMask("Phone Number", "(@@@) @@@-@@@@"),
        new PredefinedMask("Business Phone Number", "(@@@) @@@-@@@@ x@@@@"),
        new PredefinedMask("Business Phone Number", "(@@@) @@@-@@@@ Ext @@@@"),
        new PredefinedMask("ZIP Code", "@@@@@"),
        new PredefinedMask("Postal Code", "U@U @U@"),
        new PredefinedMask("State", "UU"),
};

public StringValidatorEditor() {

	setLayout(new JCGridLayout(9, 2, 5, 5));

	// setup Num Mask Match property
	add(new JLabel("Num Mask Match:"));
	add(num_match_field = new JTextField(TEXTFIELD_SIZE));

	add(new JLabel("Place Holder Chars:"));
	add(place_holder_field = new JTextField());

	// setup valid chars property
	add(new JLabel("Valid Chars:"));
	add(valid_char_field = new JTextField());

	// setup invalid chars property
	add(new JLabel("Invalid Chars:"));
	add(invalid_char_field = new JTextField());

	// setup picklist property
	add(new JLabel("PickList: "));
	add(picklist_button = new JButton("Click here to edit..."));
	picklist_button.addActionListener(this);
	createPickListDialog();

	// setup match picklist property
	add(new JLabel("Match PickList:"));
	add(match_picklist_field = new JComboBox(bool_vals));
	
	// setup allow null property
	add(new JLabel("Allow Null:"));
	add(allow_null_field = new JComboBox(bool_vals));

	// setup default value property
	add(new JLabel("Default Value:"));
	add(default_value_field = new JTextField());
	
	// setup Mask property
	add(new JLabel("Mask:"));
	JPanel p = new JPanel();
	p.setAlignmentX(0);
	p.setAlignmentY(0);
	p.setLayout(new JCGridLayout(2,1,0,0));
    // build the combobox picklist
	String[] mask_list = new String[predefined_masks.length];
    for (int i = 0; i < predefined_masks.length; i++) {
        mask_list[i] = predefined_masks[i].toString();
    }
	p.add(mask_combo = new JComboBox(mask_list));
	p.add(mask_field = new JTextField());
	mask_combo.addItemListener(this);
	add(p);

}

/**
 * Sets the mask when a mask was selected in the comboBox.
 */
public void itemStateChanged(ItemEvent e) {
	int index = mask_combo.getSelectedIndex();

	// if Custom Mask was selected, clear the text field
	if (index == 0) {
		mask_field.setEditable(true);
		mask_field.setText("");
	}
	else {
		// set the mask in the text field
		mask_field.setText(predefined_masks[index].mask);
		mask_field.setEditable(false);
	}
}

protected class PredefinedMask {
    String name;
    String mask;
    
	public PredefinedMask(String name, String mask) {
        this.name = name;
        this.mask = mask;
    }

    public String toString() {
        if (mask != null) {
            return(name + " [ " + mask + " ]");
        }
        else {
            return(name);
        }
    }
} // PredefinedMask

/**
 * Sets the current String validator values into the editors.
 */
public void	setProperties(JCValidator vp) {
	JCStringValidator string_vp = (JCStringValidator)vp;

	// get the current values from the validator
	mask = string_vp.getMask();
	picklist = string_vp.getPickList();
	match_picklist = string_vp.getMatchPickList();
	num_match = new Integer(string_vp.getNumMaskMatch());
	place_holder_chars = string_vp.getPlaceHolderChars();
	valid_chars = string_vp.getValidChars();
	invalid_chars = string_vp.getInvalidChars();
	allow_null = string_vp.getAllowNull();
	default_value = (String)string_vp.getDefaultValue();

	// set the values into the property editor textfields
	mask_field.setText(mask);

	picklist_editor.setValue(picklist);
	num_match_field.setText(num_match.toString());
	place_holder_field.setText(place_holder_chars);

	valid_char_field.setText(valid_chars);
	invalid_char_field.setText(invalid_chars);

	if (match_picklist == true) {
		match_picklist_field.setSelectedIndex(1);
	}
	else {
		match_picklist_field.setSelectedIndex(0);
	}
	
    if (allow_null == true) {
		allow_null_field.setSelectedIndex(1);
	}
	else {
		allow_null_field.setSelectedIndex(0);
	}

	default_value_field.setText(default_value);
		
}

/**
 * Creates a new validator from the fields set.
 */
protected ValidatorInfo getValidatorInfo() {

	String text;

	// get the values from the text fields
	mask = mask_field.getText();

	// if num_match is null, set it to the default value
	text = num_match_field.getText();
	if (text.length() == 0) {
		num_match = new Integer(-1);
	}
	else {
		num_match = Integer.valueOf(text);
	}

	// if place holder chars, valid chars, or invalid chars  is null, 
	// set it to empty string. This makes it easier for code generation
	place_holder_chars = place_holder_field.getText();
	valid_chars = valid_char_field.getText();
	invalid_chars = invalid_char_field.getText();

	picklist = (ListModel) picklist_editor.getValue();

	if (match_picklist_field.getSelectedIndex() == 0) {
		match_picklist = false;
	}
	else {
		match_picklist = true;
	}

	if (allow_null_field.getSelectedIndex() == 0) {
		allow_null = false;
	}
	else {
		allow_null = true;
	}

	default_value = default_value_field.getText();
	if (default_value == null) {
		default_value = "";
	}

	return (getInfo());
}

/**
 * Returns a ValidatorInfo object which contains the code generation string
 * and a new instance of the validator.
 */
protected ValidatorInfo getInfo() {

	JCStringValidator new_vp;

	// create and set the new code generation string
	String init_string = ("new com.klg.jclass.field.validate.JCStringValidator(null, \"" + 
	    mask + "\", " + num_match.intValue() + ", \"" + place_holder_chars + 
        "\", " + picklist_editor.getJavaInitializationString() + ", " + 
		match_picklist + ", \"" + valid_chars + "\", \"" + 
        invalid_chars + "\", " + allow_null + ", \"" + default_value + "\")");


	// create new validator
	new_vp = new JCStringValidator(null, mask, num_match.intValue(), 
								   place_holder_chars, picklist, 
								   match_picklist, valid_chars, 
								   invalid_chars, allow_null, default_value);

	// create new validator info object
	ValidatorInfo vi = new ValidatorInfo(new_vp, init_string);

	// return the new validator object
	return(vi);

}

/**
 * Shows the edit formats property editor when the button is pressed.
 */
public void actionPerformed(ActionEvent e) {
	picklist_dialog.show();

}

/**
 * Creates a dialog for the picklist editor.
 */
public void createPickListDialog() {

	// create a Modal dialog with the picklist editor
	picklist_dialog = new JDialog(JOptionPane.getFrameForComponent(
	                              this.getParent()), true);
	picklist_editor = new ListModelEditor(String.class);
	picklist_dialog.getContentPane().add(picklist_editor);
	picklist_dialog.pack();
}

}
