/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: TabFieldPage.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:15 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.field.beans;

import com.klg.jclass.field.JCInvalidInfo;
import com.klg.jclass.util.swing.JCAlignLayout;

import javax.swing.*;
import javax.swing.border.LineBorder;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

public class TabFieldPage extends JPanel {

protected JButton invalidBackgroundButton;
protected JButton invalidForegroundButton;
protected JLabel invalidBackgroundLabel;
protected JLabel invalidForegroundLabel;
protected JComboBox beepOnInvalidField;
protected JComboBox invalidPolicyField;

public TabFieldPage() {

	ColorEditorListener color_listener = new ColorEditorListener();

	setLayout(new JCAlignLayout(3,5,10));

	// setup beepOnInvalid property
	String items[] = {"False", "True"};
	add(new JLabel("Beep On Invalid"));
	add(beepOnInvalidField = new JComboBox(items));
	beepOnInvalidField.setSelectedIndex(1);
	add(new JLabel());

	// setup invalidPolicy property
	String policy_items[] = {"Show Invalid", "Restore Default", 
							 "Restore Previous", "Clear Field"};
	add(new JLabel("Invalid Policy"));
	add(invalidPolicyField = new JComboBox(policy_items));
	add(new JLabel());

	//setup invalidBackground property
	add(new JLabel("Invalid Background"));
	add(invalidBackgroundLabel = new JLabel(" "));
	invalidBackgroundLabel.setOpaque(true);
	invalidBackgroundLabel.setBorder(new LineBorder(Color.black));
	add(invalidBackgroundButton = new JButton("..."));
	invalidBackgroundButton.setMargin(new Insets(0,2,0,2));
	invalidBackgroundButton.addActionListener(color_listener);
	invalidBackgroundLabel.setBackground((Color)UIManager.get("TextField.background"));

	// setup invalidForeground property
	add(new JLabel("Invalid Foreground"));
	add(invalidForegroundLabel = new JLabel(" "));
	invalidForegroundLabel.setOpaque(true);
	invalidForegroundLabel.setBorder(new LineBorder(Color.black));
	add(invalidForegroundButton = new JButton("..."));
	invalidForegroundButton.setMargin(new Insets(0,2,0,2));
	invalidForegroundButton.addActionListener(color_listener);
	invalidForegroundLabel.setBackground((Color)UIManager.get("TextField.foreground"));

}

public void setProperties(JCInvalidInfo invalid_info) {
	if (invalid_info == null) {
		return;
	}

	// set beep on invalid property
	if (invalid_info.getBeepOnInvalid() == true) {
		beepOnInvalidField.setSelectedIndex(1);
	}
	else {
		beepOnInvalidField.setSelectedIndex(0);
	}
	
	// set invalid policy field
	invalidPolicyField.setSelectedIndex(invalid_info.getInvalidPolicy() - 1);

	// set invalid background property
	Color color = invalid_info.getInvalidBackground();
	if (color != null) {
		invalidBackgroundLabel.setBackground(color);
	}

	// set invalid foreground property
	color = invalid_info.getInvalidForeground();
	if (color != null) {
		invalidForegroundLabel.setBackground(color);
	}

}

/**
 * Returns JCInvalidInfo object
 */
public JCInvalidInfo getInvalidInfo() {
	boolean beep = true;

	// get the beep on invalid value
	if (beepOnInvalidField.getSelectedIndex() == 1) {
		beep = true;
	}
	else {
		beep = false;
	}

	int policy = invalidPolicyField.getSelectedIndex() + 1;

	Color bg_color = invalidBackgroundLabel.getBackground();
	Color fg_color = invalidForegroundLabel.getBackground();

	return new JCInvalidInfo(beep, policy, fg_color, bg_color);
}	

/**
 * Returns the code generation string for the field object.
 */
public String getInitString() {
	String init_string = "new com.klg.jclass.field.JCInvalidInfo(";
	Color color;
	
	// get the beep on invalid value
	if (beepOnInvalidField.getSelectedIndex() == 1) {
		init_string += "true, ";
	}
	else {
		init_string += "false, ";
	}

	init_string += (invalidPolicyField.getSelectedIndex() + 1);
	
	// get invalid foreground color
	color = invalidForegroundLabel.getBackground();
	init_string += ", new java.awt.Color(" + color.getRed() + ", " + 
        color.getGreen() + ", " + color.getBlue() + ", " + 
        color.getAlpha() + ")";

	// get invalid background color
	color = invalidBackgroundLabel.getBackground();
	init_string += ", new java.awt.Color(" + color.getRed() + ", " + 
		color.getGreen() + ", " + color.getBlue() + ", " + 
		color.getAlpha() + "))";	
	
	
	


	
	return init_string;
}

protected class ColorEditorListener implements ActionListener {
	public void actionPerformed(ActionEvent e) {
		JButton button = (JButton)e.getSource();
		JLabel label;
		Color color;

		// determine which label will change colors
		if (button == invalidBackgroundButton) {
			label = invalidBackgroundLabel;
		}
		else {
			label = invalidForegroundLabel;
		}

		// show the color dialog
		color = JColorChooser.showDialog(button, 
										 "Choose a color", 
										 label.getBackground());
		
		// if color is null, the user did a cancel so no color was selected
		if (color != null) {
			label.setBackground(color);
		}
	}
} // ColorEditorListener
	
}
