/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: TabTypePage.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:15 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.field.beans;

import com.klg.jclass.field.*;
import com.klg.jclass.field.validate.*;
import com.klg.jclass.util.JCIPAddress;
import com.klg.jclass.util.value.JCValueModel;

import javax.swing.*;
import javax.swing.border.LineBorder;
import javax.swing.event.*;

import java.awt.*;
import java.math.BigDecimal;
import java.util.Calendar;

public class TabTypePage extends JPanel 
    implements ListSelectionListener, ChangeListener {

protected static String BYTE_VALIDATOR = "byte";
protected static String DOUBLE_VALIDATOR = "double";
protected static String FLOAT_VALIDATOR = "float";
protected static String INTEGER_VALIDATOR = "integer";
protected static String LONG_VALIDATOR = "long";
protected static String SHORT_VALIDATOR = "short";
protected static String STRING_VALIDATOR = "string";
protected static String BIGDECIMAL_VALIDATOR = "bigdecimal";
protected static String IPADDRESS_VALIDATOR = "ipaddress";
protected static String DATETIME_VALIDATOR = "datetime";
protected static String DATE_VALIDATOR = "date";
protected static String TIME_VALIDATOR = "time";

protected static String[][] list_vals = {
	{ "com.klg.jclass.util.JCIPAddress", IPADDRESS_VALIDATOR },
    { "java.lang.Byte", BYTE_VALIDATOR },
	{ "java.lang.Double", DOUBLE_VALIDATOR },
	{ "java.lang.Float", FLOAT_VALIDATOR },
	{ "java.lang.Integer", INTEGER_VALIDATOR },
   	{ "java.lang.Long", LONG_VALIDATOR }, 
	{ "java.lang.Short", SHORT_VALIDATOR },
	{ "java.lang.String", STRING_VALIDATOR }, 
	{ "java.math.BigDecimal", BIGDECIMAL_VALIDATOR },
	{ "java.util.Calendar", DATETIME_VALIDATOR },
    { "java.util.Date", DATETIME_VALIDATOR },
	{ "java.sql.Date", DATE_VALIDATOR },
	{ "java.sql.Time", TIME_VALIDATOR },
	{ "java.sql.Timestamp", DATETIME_VALIDATOR }
};

protected JList type_list;
protected JTabbedPane tabPane;
protected AbstractValidatorEditor validatorPage = null;
protected JCValueModel valueModel = null;
protected DataPropertiesRegistry registry;
protected DataPropertiesEditor de;
protected TabValuePage tabValuePage;
protected String validator_text = null;

public TabTypePage(DataPropertiesEditor de) {

	this.de = de;

	setLayout(new BorderLayout());

	// add items to list
 	String[] items = new String[list_vals.length];
	for (int i=0; i < list_vals.length; i++) {
		items[i] = list_vals[i][0];
	}
	type_list = new JList(items);
	
	// set outline around list
	type_list.setBorder(new LineBorder(Color.black));
	type_list.addListSelectionListener(this);
	type_list.setVisibleRowCount(15);
	add("West", type_list);

	add("Center", tabPane = new JTabbedPane());
	tabPane.addChangeListener(this);
	createTabPages();
	registry = new DataPropertiesRegistry();
}

public void stateChanged(ChangeEvent e) {
	if (tabPane.getSelectedComponent() == tabValuePage) {
		de.setHelpText(DataPropertiesEditor.TYPE_HELP);
	}
	else {
		if (validator_text != null) {
			de.setHelpText(validator_text);
		}
	}
	tabPane.validate();
}

/**
 * Set the value model value into the editor
 */
public void setProperties(JCValueModel model) {
	valueModel = model;
	if (model != null) {
		int index = model.getClass().getName().indexOf("db");
		if (index > 0) {
			// disable stuff here
			type_list.setEnabled(false);
			tabPane.setEnabledAt(tabPane.indexOfTab("Value"), false);
			tabPane.setSelectedIndex(tabPane.indexOfTab("Validator"));
		}
		else {
			// enable
			type_list.setEnabled(true);
			tabPane.setEnabledAt(tabPane.indexOfTab("Value"), true);
		}
	}

	if (tabValuePage != null) {
		tabValuePage.setProperties(model);
	}
}

protected void createTabPages() {
	// create the tab pages
	tabPane.addTab("Value", new JPanel());
	tabPane.addTab("Validator", new JPanel());

	// disable the validator and invalid pages. Once a type is selected,
	// then these tabs will be enabled.
	tabPane.setEnabledAt(tabPane.indexOfTab("Value"), false);
	tabPane.setEnabledAt(tabPane.indexOfTab("Validator"), false);
}

/** 
 * Called when a supported type in the list is selected.
 */
public void valueChanged(ListSelectionEvent e) {

	int index = type_list.getSelectedIndex();

	// enable commit button
	de.commitButton.setEnabled(true);

	if (index < 0 || index > list_vals.length) {
		return;
	}

	// get new data model if one doesn't exist
    if (valueModel == null || valueModel.getValueClass() == null ||
		(!valueModel.getValueClass().getName().equals(list_vals[index][0]))) {
		valueModel = getValueModel(list_vals[index][0]);
	}
			
	// create validator page.
	createValidatorPage(list_vals[index][1]);

	// enable the other tabs since a type has been selected
	tabPane.setEnabledAt(tabPane.indexOfTab("Value"), true);
	tabPane.setEnabledAt(tabPane.indexOfTab("Validator"), true);

	// selections after index 9 are date and/or time validators 
	// so use calendar popup for editors and not textfields.
	if (index >= 9) {
		tabPane.setComponentAt(tabPane.indexOfTab("Value"), 
							   tabValuePage = new TabValuePage(true));
	}
	else {
		tabPane.setComponentAt(tabPane.indexOfTab("Value"), 
							   tabValuePage = new TabValuePage(false));
	}
	tabValuePage.type = list_vals[index][0];
	if (tabPane.getSelectedIndex() == 1) {
		de.setHelpText(validator_text);
	}
}

public JCValueModel getValueModel(String type) {

	JCValueModel value_model = null;

	// for the databinding case, we always use the same value model.
	// In order to do that, we must pass null
	if (valueModel != null && 
		    valueModel.getClass().getName().indexOf("db") > 0) {
		return null;
	}

	// get the data model
	Class type_class = null;
	try {
		type_class = Class.forName(type);
	}
	catch (ClassNotFoundException cnfe) {
		throw new IllegalArgumentException("Cannot find class \"" + 
										   type + "\"");
	}	
	if (type_class != null &&
		DataPropertiesRegistry.isRegistered(type_class)) {
			value_model = DataPropertiesRegistry.createValueModel(type_class);
	}
	return value_model;
}

/**
 * Creates the validator page and adds it to the tabPane.
 */
protected void createValidatorPage(String val) {

	// get the validator to use
   
	AbstractValidatorEditor currentChild = null;
	JCValidator validator = null;
	// create a new validator if current validator doesn't match the 
	// one needed
	if (val.equals(BYTE_VALIDATOR)) {
		currentChild = new ByteValidatorEditor();
		validator = new JCByteValidator();
		validator_text = DataPropertiesEditor.BYTE_VALIDATOR_HELP;
	}
	else if (val.equals(INTEGER_VALIDATOR)) {
		currentChild = new IntegerValidatorEditor();
		validator = new JCIntegerValidator();
		validator_text = DataPropertiesEditor.INTEGER_VALIDATOR_HELP;
	}
	else if (val.equals(SHORT_VALIDATOR)) {
		currentChild = new ShortValidatorEditor();
		validator = new JCShortValidator();
		validator_text = DataPropertiesEditor.SHORT_VALIDATOR_HELP;
	}
	else if (val.equals(LONG_VALIDATOR)) {
		currentChild = new LongValidatorEditor();
		validator = new JCLongValidator();
		validator_text = DataPropertiesEditor.LONG_VALIDATOR_HELP;
	}
	else if (val.equals(DOUBLE_VALIDATOR)) {
		currentChild = new DoubleValidatorEditor();
		validator = new JCDoubleValidator();
		validator_text = DataPropertiesEditor.DOUBLE_VALIDATOR_HELP;
	}
	else if (val.equals(FLOAT_VALIDATOR)) {
		currentChild = new FloatValidatorEditor();
		validator = new JCFloatValidator();
		validator_text = DataPropertiesEditor.FLOAT_VALIDATOR_HELP;
	}
	else if (val.equals(BIGDECIMAL_VALIDATOR)) {
		currentChild = new BigDecimalValidatorEditor();
		validator = new JCBigDecimalValidator();
		validator_text = DataPropertiesEditor.BIGDECIMAL_VALIDATOR_HELP;
	}
	else if (val.equals(STRING_VALIDATOR)) {
		currentChild = new StringValidatorEditor();
		validator = new JCStringValidator();
		validator_text = DataPropertiesEditor.STRING_VALIDATOR_HELP;
	}
	else if (val.equals(IPADDRESS_VALIDATOR)) {
		currentChild = new IPAddressValidatorEditor();
		validator = new JCIPAddressValidator();
		validator_text = DataPropertiesEditor.IPADDRESS_VALIDATOR_HELP;
	}
	else if (val.equals(DATE_VALIDATOR)) {
		currentChild = new DateValidatorEditor();
		validator = new JCDateValidator();
		validator_text = DataPropertiesEditor.DATE_VALIDATOR_HELP;
	}
	else if (val.equals(TIME_VALIDATOR)) {
		currentChild = new TimeValidatorEditor();
		validator = new JCTimeValidator();
		validator_text = DataPropertiesEditor.TIME_VALIDATOR_HELP;
	}
	else if (val.equals(DATETIME_VALIDATOR)) {
		currentChild = new DateTimeValidatorEditor();
		validator = new JCDateTimeValidator();
		validator_text = DataPropertiesEditor.DATETIME_VALIDATOR_HELP;
	}

	// set the new validator page if needed
	if (currentChild != null) {
		if (validator != null) {
			currentChild.setProperties(validator);
		}
		tabPane.setComponentAt(tabPane.indexOfTab("Validator"), currentChild);
		validatorPage = currentChild;
		// When the tab page is changed, the page does not show up. This
		// is a bug in the tabPane component. Call validate to ensure
		// that it does a relayout and repaint.
		tabPane.validate();
	}
}

/** 
 * Returns the value in the value model
 */
public Object getValue() {

	Object value = null;

	if (tabValuePage.isCalendar == true) {
		value = (((JCPopupField)tabValuePage.value).getValue());
	}
	else { 
		String text = ((JTextField)tabValuePage.value).getText();
		if (text == null || text.length() == 0) {
			return value;
		}

		if (tabValuePage.type.equals("java.lang.Byte")) {
			value = new Byte(text);
		}
		else if (tabValuePage.type.equals("java.lang.Double")) {
			value = new Double(text);
		}
		else if (tabValuePage.type.equals("java.lang.Float")) {
			value = new Float(text);
		}
		else if (tabValuePage.type.equals("java.lang.Integer")) {
			value = new Integer(text);
		}
		else if (tabValuePage.type.equals("java.lang.Long")) {
			value = new Long(text);
		}
		else if (tabValuePage.type.equals("java.lang.Short")) {
			value = new Short(text);
		}
		else if (tabValuePage.type.equals("com.klg.jclass.util.JCIPAddress")) {
			value = new JCIPAddress(text);
		}
		else if (tabValuePage.type.equals("java.lang.String")) {
			value = text;
		}
		else if (tabValuePage.type.equals("java.math.BigDecimal")) {
			value = new BigDecimal(text);
		}
	}
	return value;
}

/**
 * Returns the code generation string for the validator, the data model,
 * and the value and default values.
 * i.e. Format: new JCValidator(), new JCValueModel
 */
public String getInitString() {

	String init_string = "";	
	if (validatorPage == null) {
		return ("null, null");
	}
    init_string = validatorPage.getInitString() + ", ";

	// if databinding, use the same valuemodel. No need to change the model
	if (valueModel.getClass().getName().indexOf("db") > 0) {
		init_string += "null";
		return init_string;
	}

	init_string += "new " + valueModel.getClass().getName() + "(";

	// get the value string
	String value_string;
	String type_string = tabValuePage.type + ".class";

	if (tabValuePage.isCalendar == true) {
		Calendar cal = (Calendar)((JCPopupField)tabValuePage.value).getValue();

		if (cal == null) {
			value_string = "null";
		}
		else {
			int year = cal.get(Calendar.YEAR);
			int month = cal.get(Calendar.MONTH);
			int date = cal.get(Calendar.DATE);
			int hour = cal.get(Calendar.HOUR);
			int minute = cal.get(Calendar.MINUTE);
			int second = cal.get(Calendar.SECOND);
			value_string = "new java.util.GregorianCalendar(" + year + ", " +
				month + ", " + date + ", " + hour + ", " + minute + ", " +
				second + ")";
		}
	}

	else {
		value_string = ((JTextField)tabValuePage.value).getText();
 
		if (value_string == null || value_string.length() == 0) {
			value_string = "null";
		}
		else {
			value_string = "new " + tabValuePage.type + "(\"" + 
				value_string + "\")";
		}
	}

	init_string += type_string + ", " + value_string + ")"; 

	return init_string;

}
};
