/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCSpinFieldCellEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:15 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.field.cell;

import com.klg.jclass.cell.*;
import com.klg.jclass.field.*;
import com.klg.jclass.field.validate.JCValidator;
import com.klg.jclass.util.JCEnvironment;
import java.awt.AWTEvent;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.SystemColor;
import java.awt.event.*;
import javax.swing.FocusManager;
import javax.swing.SwingUtilities;

/**
 * <CODE>JCSpinFieldCellEditor</CODE> provides an editor using a simple 
 * <code>JCSpinField</code> component.
 *
 */
public class JCSpinFieldCellEditor extends JCSpinField 
implements JCCellEditor, KeyListener {

protected JCCellEditorSupport support = new JCCellEditorSupport();
private AWTEvent initialEvent;
private Object data;
/**
 * Creates a new <CODE>JCSpinFieldCellEditor</CODE> object.
 */
public JCSpinFieldCellEditor() {
	super();
	addKeyListener(this);
	setBorder(null);
}

/**
 * Constructs an empty <code>JCSpinFieldCellEditor</code>, which expects an object
 * of the specified class, that uses the specified validator.
 * @param c
 * @param v
 */
public JCSpinFieldCellEditor(Class c, JCValidator v) {
	super(c, v);
	getEditor().getEditorComponent().addKeyListener(this);
	setBorder(null);
}

/**
 * Initializes the <code>JCSpinField</code> component by selecting the appropriate
 * index associated with the data.
 * @param ev
 * @param info
 * @param o
 */
public void initialize(AWTEvent ev, JCCellInfo info, Object o) {

	initialEvent = ev;
	data = o;
	setValue(o);
    setBackground(info.getBackground());
    setForeground(info.getForeground());
    if (info.isEditable()) {
    	setBackground(info.getBackground().brighter());
    }
    field.setCellEditor(true);

}

/**
 * Returns the <code>JCSpinField</code> component.
 * @return <code>JCSpinField</code> component
 */
public Component getComponent() {
	return this;
}

/**
 * Returns the object to be committed.
 * @return object to be committed
 */
public Object getCellEditorValue() {
	commitEdit();
	return getValue();
}

public boolean isModified() {
	Object currentValue = getCellEditorValue();
	if ((data == null) || (data.equals(""))) {
	    if ((currentValue == null) || (currentValue.equals(""))) {
	        return false;
	    }
	    else {
	        return true;
	    }
	}
	return !data.equals(currentValue);


}

/**
 * Called when the edit process is stopped.
 * @return
 */
public boolean stopCellEditing() {
	return true;
}

/**
 * Called when the edit process is cancelled.
 */
public void cancelCellEditing() {
	return;
}

protected JCKeyModifier [] key_modifiers = {
	new JCKeyModifier(KeyEvent.VK_DOWN, JCKeyModifier.ALL),
	new JCKeyModifier(KeyEvent.VK_UP,  JCKeyModifier.ALL)
};

/**
 * This editor reserves no keys for its own use.
 * @return
 */
public JCKeyModifier[] getReservedKeys() {
	return key_modifiers;
}

public void keyTyped(KeyEvent e) {
}

public void keyPressed(KeyEvent e) {
}

public void keyReleased(KeyEvent e) {
	JCCellEditorEvent rval = null;
	if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
		support.fireStopEditing(new JCCellEditorEvent(e));
	}
	else if (e.getKeyCode() == KeyEvent.VK_ENTER) {
		support.fireStopEditing(new JCCellEditorEvent(e));
	}
}

/**
 * Adds a listener to the list that is notified when the editor starts,
 * stops, or cancels editing.
 *
 * @param	l		the <code>JCCellEditorListener</code>
 */  
public void addCellEditorListener(JCCellEditorListener l) {
	support.addCellEditorListener(l);
}

/**
 * Removes a listener from the list that is notified when the editor starts,
 * stops, or cancels editing.
 *
 * @param	l		the <code>JCCellEditorListener</code>
 */  
public void removeCellEditorListener(JCCellEditorListener l) {
	support.removeCellEditorListener(l);
}

}

