/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCTextFieldCellEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:15 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.field.cell;

import com.klg.jclass.cell.*;
import com.klg.jclass.cell.validate.JCValidateEvent;
import com.klg.jclass.cell.validate.JCValidateListener;
import com.klg.jclass.cell.validate.ValidateSupport;
import com.klg.jclass.field.JCTextField;
import com.klg.jclass.field.validate.JCValidator;
import com.klg.jclass.util.JCEnvironment;

import javax.swing.FocusManager;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

/**
 * <CODE>JCTextFieldCellEditor</CODE> provides an editor using a simple
 * <code>JCTextField</code> component.
 *
 */
public class JCTextFieldCellEditor extends JCTextField
			                       implements JCCellTextEditor,
											  JCValidateInterface,
											  KeyListener,
											  MouseListener
{

/**
 * This editor can dynamically alter its reserved key list based upon
 * the key the user has last typed.  Thus, typing navigation keys will
 * cause the cell editor to traverse, but after typing a non-navigation
 * key, the user is placed into edit mode.  At that point, the reserved
 * keys are re-enabled and the traversal keys will traverse within the
 * editor itself.  The only way to get out of edit mode is to press
 * <i>Enter</i> or <i>Escape</i>, or use the mouse to click on another cell.
*/
protected Object               data;
protected int                  caretPosition;
protected AWTEvent             initialEvent;
protected JCCellEditorSupport  support = new JCCellEditorSupport();
protected boolean              selectAll = false;
protected JCKeyModifier[]      reservedKeys = null;
protected ValidateSupport      validate_support = new ValidateSupport();

/**
 * Creates a new <CODE>JCTextFieldCellEditor</CODE> object.
 */
public JCTextFieldCellEditor()
{
	addKeyListener(this);
	addMouseListener(this);
	setDoubleBuffered(true);
	if (reservedKeys == null) {
		reservedKeys = new JCKeyModifier[4];
		reservedKeys[0] = new JCKeyModifier(KeyEvent.VK_LEFT);
		reservedKeys[1] = new JCKeyModifier(KeyEvent.VK_RIGHT);
		reservedKeys[2] = new JCKeyModifier(KeyEvent.VK_HOME);
		reservedKeys[3] = new JCKeyModifier(KeyEvent.VK_END);
	}
	setBorder(null);
}

/**
 * Constructs an empty <code>JCTextFieldCellEditor</code>, which expects an object
 * of the specified class, that uses the specified validator.
 * @param c the class type the editor will expect
 * @param v the validator that validates input in the editor
 */
public JCTextFieldCellEditor(Class c, JCValidator v)
{
	super(c, v);
	addKeyListener(this);
	addMouseListener(this);
	setDoubleBuffered(true);
	if (reservedKeys == null) {
		reservedKeys = new JCKeyModifier[4];
		reservedKeys[0] = new JCKeyModifier(KeyEvent.VK_LEFT);
		reservedKeys[1] = new JCKeyModifier(KeyEvent.VK_RIGHT);
		reservedKeys[2] = new JCKeyModifier(KeyEvent.VK_HOME);
		reservedKeys[3] = new JCKeyModifier(KeyEvent.VK_END);
	}
	setBorder(null);
}

public void initialize(AWTEvent ev, JCCellInfo info,  Object o)
{
	this.initialEvent = ev;

	data = o;
    selectAll = info.getSelectAll();
    setEditable(info.isEditable());
    if (!isEditable()) {
        selectAll = false;
    }

	if(ev instanceof KeyEvent && data != null) {
		caretPosition = data.toString().length();
	}
	else {
        // will be handled by the show() method later on
		caretPosition = -1;
	}

	setValue(o);
    selectAllText();

    setBackground(info.getBackground());
    setForeground(info.getForeground());
    if (info.isEditable()) {
    	setBackground(info.getBackground().brighter());
    }
    setSelectedTextColor(SystemColor.textHighlightText);
    setSelectionColor(SystemColor.textHighlight);
	setFont(info.getFont());
	field.setCellEditor(true);
}

/**
 * Returns whether this editor will select the text on initialization.
 * @return <code>true</code> if text is selected on initialization
 */
public boolean getSelectAll()
{
	return(selectAll);
}

/**
 * Returns the <code>JCTextField</code> component.
 * @return the current <code>JCTextField</code> component
 */
public Component getComponent()
{
	return(this);
}

/**
 * Returns the object to be commited.
 * @return the current cell editor value
 */
public Object getCellEditorValue()
{
    field.commitEdit();
    return(getValue());
}

/**
 * Gets the text for the editor (as a String).
 * @return the current text in the editor
 */
public String getCellEditorText()
{
	return(getText());
}

/**
 * Sets the text for the editor.
 * @param text the new cell editor text value
 */
public void setCellEditorText(String text)
{
    setText(text);
}

/**
 * Returns <code>true</code> if the data has been modified.
 * @return if the data been modified
 */
public boolean isModified()
{
    field.commitEdit();
	if (data == null) {
		return (!(getCellEditorValue() == null));
	}
	return(!data.equals(getCellEditorValue()));
}

protected boolean fireValidated(JCValidateEvent ev)
{
	boolean validated = validate_support.fireValidateEvents(ev);
	if (validated) {
	    setVisible(true);
	}
	return(validated);
}

/**
 * Called when the edit process is stopped.
 * @return whether validation succeeded
 */
public boolean stopCellEditing()
{
	String str = getText();
	JCValidateEvent ev = new JCValidateEvent(this, data, str, true);
	// all values are considered valid but someone registering
	// a ValidateListener can decide otherwise in the valueChangedEnd()
	// event
	return(fireValidated(ev));
}

/**
 * Called when the edit process is cancelled.
 */
public void cancelCellEditing()
{
	setValue(data);
	caretPosition = (data == null) ? 0 : data.toString().length();
    setVisible(true);
}

public Dimension getPreferredSize(Graphics gc, JCCellInfo cellInfo, Object o)
{
	return(getPreferredSize());
}

/**
 * This editor reserves the LEFT and RIGHT arrow keys for its own use.
 * However, after the edit comitted or cancelled, it temporarily
 * "unreserves" these keys.
 * @return a list of reserved keys
 */
public JCKeyModifier[] getReservedKeys()
{
	return(reservedKeys);
}

/**
 * Overrides <CODE>setVisible</CODE> in order to control the caret
 * position.
 * @param b the new value of visible
 */
public void setVisible(boolean b)
{
	super.setVisible(b);
	if (!b) {
		return;
	}

    // This code attempts to position the caret based
    // on where the user clicked the mouse within this cell
    String text = getText();
    if (text == null) {
        return;
    }
    if (caretPosition == -1) {
        caretPosition = text.length();
        if (initialEvent != null && initialEvent instanceof MouseEvent) {
            MouseEvent me = (MouseEvent)initialEvent;
            Component source = (Component)me.getSource();
            Component comp = getComponent();
            Point offset = new Point(0, 0);
            if (comp != source) {
                Container parent = getParent();
                offset.setLocation(comp.getLocation());
                while (parent != null && parent != source) {
                    offset.translate(parent.getLocation().x,
                                     parent.getLocation().y);
                    comp = parent;
                    parent = comp.getParent();
                }
                if (parent == null) {
                    offset.setLocation(0, 0);
                }
            }
            caretPosition = viewToModel(new Point(me.getX() - offset.x,
                                                  me.getY() - offset.y));
        }
    }
    caretPosition = Math.min(caretPosition, text.length());
    select(caretPosition, caretPosition);
    selectAllText();
}

/**
 * Adds a listener to the list that's notified when the editor starts,
 * stops, or cancels editing.
 * @param	l		the <code>CellEditorListener</code>
 */
public void addCellEditorListener(JCCellEditorListener l)
{
	support.addCellEditorListener(l);
}

/**
 * Removes a listener from the list that's notified.
 * @param	l		the <code>CellEditorListener</code>
 */
public void removeCellEditorListener(JCCellEditorListener l)
{
	support.removeCellEditorListener(l);
}

/**
 * Overrides the key event processor.  This allows the parent to pass a key
 * event to this editor, which it might otherwise not get.
 * @param e the event to pass
 */
public void processKeyEvent(KeyEvent e)
{
	if (JCEnvironment.getJavaVersion() >= 140) {
		super.processKeyEvent(e);
	}
	else {
		// In Swing for JDK's prior to 1.4, we have to disable the focus
		// manager in order to get the key processing we want
		FocusManager fm = FocusManager.getCurrentManager();
		FocusManager.disableSwingFocusManager();
		super.processKeyEvent(e);
		fm.setCurrentManager(fm);
	}
}

public void keyPressed(KeyEvent e)
{
    int keyCode = e.getKeyCode();
	switch(keyCode) {
	case KeyEvent.VK_ENTER:
		support.fireStopEditing(new JCCellEditorEvent(e));
		break;
	case KeyEvent.VK_ESCAPE:
		support.fireCancelEditing(new JCCellEditorEvent(e));
		break;
	default:
	    break;
	}
}

public void mouseClicked(MouseEvent e) {}

public void mouseEntered(MouseEvent e)
{
	setCursor(Cursor.getPredefinedCursor(Cursor.TEXT_CURSOR));
}

public void mouseExited(MouseEvent e) {}

public void mousePressed(MouseEvent e) {}

public void mouseReleased(MouseEvent e) {}

public void keyReleased(KeyEvent e) {}

public void keyTyped(KeyEvent e) {}

/**
 * Uses hint from <code>CellInfo</code> to select all the text in the component.
 */
public void selectAllText()
{
    if (getSelectAll() && isDisplayable()) {
        // Setting of the caret position is a hack to ensure that the
        // text is fully scrolled into view
        try {
            setCaretPosition(0);
            setCaretPosition((data == null) ? 0 : data.toString().length());
        }
        catch (Exception e) {
            // checking for non-null peer above doesn't always work
        }

        selectAll();
    }
}

//
// ValidateInterface Implementation
//
/**
 * Adds a new <code>ValidateListener</code> to the list.
 * @param l the listener to be added
 */
public void addValidateListener(JCValidateListener l)
{
	validate_support.addValidateListener(l);
}

/**
 * Removes a <code>ValidateListener</code> from the list.
 * @param l the listener to be removed
 */
public void removeValidateListener(JCValidateListener l)
{
	validate_support.removeValidateListener(l);
}

}



