/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: DSdbPopupField.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:16 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.field.db.datasource;

import com.klg.jclass.field.*;
import java.sql.*;
import com.klg.jclass.datasource.*;

/**
 * This component is a data-bound popup field. A popup field enables
 * end-users to visually select a date/time.
 * <P>The field displays the value at one column of the current record.
 * The data type is set when the component is bound to a data source.
 * <P>
 * <STRONG>Behavior</STRONG>
 * <UL>
 *  <LI>Selection of date and time values is simplified through the use of
 *   a calendar popup to select a day in a chosen month, and spinners for
 *   the selection of year, month and time information.
 *  <LI>Optionally, the date may be typed in the field without using the popup.
 *  <LI>Input can be checked to ensure that it is within a range of values.
 *  <LI>Audible and visual warnings may be given upon receipt of invalid entries.
 *  <LI>Various display and edit formats are possible, as well as locale-specific formats.
 * </UL>
 * <P>
 */

public class DSdbPopupField extends JCPopupField {

protected DataSourceValueModel vm;

/**
 * Creates a <code>DSdbPopupField</code> which can be bound to a data source.
 */
public DSdbPopupField() {
	createField(null);
	vm = new DataSourceValueModel(this);
}

/**
 * Returns the field's value of the <code>dataBinding</code> object property.
 *
 * The <code>dataBinding</code> property specifies a string,
 * describing a <code>JCTreeData</code>
 * instance and metadata level of the instance this table is bound with.
 * The format of the property is as follows:
 * <PRE>
 * &lt;JCTreeData name&gt;:&lt;MetaDataModel description&gt;[|&lt;MetaDataModel description&gt;...]
 * </PRE>
 *
 * @see com.klg.jclass.field.db.datasource.DSdbComboField#setDataBinding
 * @see com.klg.jclass.datasource.TreeData
 * @see com.klg.jclass.datasource.MetaDataModel
 */
public String getDataBinding() {
	return vm.getDataBinding();
}

/**
 * Sets the data binding hierarchy string, starting from a
 * <code>JCTreeData</code> instance name and containing full sequence of
 * <code>MetaDataModel</code> levels starting from the root to the level
 * the table is bound with.
 *
 * @see com.klg.jclass.datasource.beans.NodeProperties#setDataSource
 * @see com.klg.jclass.field.db.datasource.DSdbComboField#setDataBinding
 */
public void setDataBinding(String data_binding) {
	vm.setDataBinding(data_binding);
}

/**
 * Sets the parameters for binding the component to a particular column
 * in a <code>MetaDataModel</code>. If the <code>MetaDataModel</code>
 * is the root in the <code>DataModel</code>'s
 * <code>MetaDataTree</code>, the <code>DataTableModel</code>
 * is automatically set to the root
 * <code>DataTableModel</code>, otherwise, the <code>DataTableModel</code>
 * is set when an appropriate
 * table becomes available.
 */
public void setDataBinding(DataModel data_model,
						   MetaDataModel meta_data_model,
						   String column_name) {
	vm.setDataBinding(data_model, meta_data_model, column_name);
}

/**
 * Binds using a <code>Binding</code>.
 */
public void setDataBinding(BindingModel binding, String column_name) {
	vm.setDataBinding(binding, column_name);
}

}
