/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: DataBinding.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:16 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.field.db.datasource;

import com.klg.jclass.datasource.*;
import com.klg.jclass.datasource.util.FieldDataBinding;

import com.klg.jclass.field.DataPropertiesRegistry;
import com.klg.jclass.field.DataProperties;
import com.klg.jclass.field.JCFieldComponent;
import com.klg.jclass.field.validate.JCValidator;
import com.klg.jclass.field.validate.JCNumberValidator;
import com.klg.jclass.util.value.JCValueModel;

import java.awt.Component;

/**
 * The DataBinding class.
 */
public class DataBinding extends FieldDataBinding {

protected JCFieldComponent fieldComponent;
protected boolean dataSourceUpdating = true;

/**
 * Constructs a data binding object.
 */
public DataBinding(JCFieldComponent c, JCValueModel model) {
	super((Component) c);
	this.fieldComponent = c;
}

/**
 * Tests to see if the data source is updating.
 */
public boolean isDataSourceUpdating() {
	return dataSourceUpdating;
}

/**
 * Retrieves the column model.
 */
public ColumnModel getColumnModel() {
	return binding == null ? null : binding.getColumnObject(column_name);
}

/**
 * Tests to see if the column stores currency values.
 */
public boolean isCurrency() {
	ColumnModel cm = getColumnModel();
	if (cm != null && cm.isCurrency()) {
		return true;
	}
	return false;
}

/**
 * Retrieves the value class for the column.
 */

public Class getValueClass() {
	String className = getValueClassName();
	if (className != null && !className.equals("")) {
		Class c;
		try {
			return Class.forName(className);
		}
		catch (ClassNotFoundException e) {
			return Object.class;
		}
	}
	return Object.class;
}

/**
 * Retrieves the value class name.
 */
public String getValueClassName() {
	ColumnModel cm = getColumnModel();
	if (cm != null) {
		int meta_column_type = cm.getMetaColumnType();
		return convertMetaColumnTypeToClassName(meta_column_type);
	}
	return "";
}

/**
 * Converts a Meta column Type to a Class name.
 */
protected String convertMetaColumnTypeToClassName(int meta_column_type) {
	String class_name = null;

	switch (meta_column_type) {

	case MetaDataModel.TYPE_BOOLEAN:
		class_name = "java.lang.Boolean";
		break;
	case MetaDataModel.TYPE_DOUBLE:
		class_name = "java.lang.Double";
		break;
	case MetaDataModel.TYPE_FLOAT:
		class_name = "java.lang.Float";
		break;
	case MetaDataModel.TYPE_INTEGER:
		class_name = "java.lang.Integer";
		break;
	case MetaDataModel.TYPE_STRING:
		class_name = "java.lang.String";
		break;
	case MetaDataModel.TYPE_BIG_DECIMAL:
		class_name = "java.math.BigDecimal";
		break;
	case MetaDataModel.TYPE_LONG:
		class_name = "java.lang.Long";
		break;
	case MetaDataModel.TYPE_SQL_TIME:
		class_name = "java.sql.Time";
		break;
	case MetaDataModel.TYPE_SQL_TIMESTAMP:
		class_name = "java.sql.Timestamp";
		break;
	case MetaDataModel.TYPE_OBJECT:
		class_name = "java.lang.Object";
		break;
	case MetaDataModel.TYPE_BYTE:
		class_name = "java.lang.Byte";
		break;
	case MetaDataModel.TYPE_SHORT:
		class_name = "java.lang.Short";
		break;
	case MetaDataModel.TYPE_UTIL_DATE:
		class_name = "java.util.Date";
		break;
	}
	return class_name;
}

/**
 * Sets up parameters to bind the JCDataSource to a MetaDataModel in the
 * given DataModel.
 */
public void setDataBinding(DataModel data_model,
						   MetaDataModel meta_data_model,
						   String column_name) {
	super.setDataBinding(data_model, meta_data_model, column_name);
}

/**
 * Binds using a Binding
 */
public void setDataBinding(BindingModel binding, String column_name) {
	super.setDataBinding(binding, column_name);
	JCValidator v = fieldComponent.getValidator();

	if (v == null || !v.isClassSupported(getValueClass())) {
		// we have a bad validator so we need to create a new one
		v = DataPropertiesRegistry.createValidator(getValueClass());
		fieldComponent.setValidator(v);
	}

	if (v instanceof JCNumberValidator) {
		// if it is a JCNumberValidator we need to tell it if
		// we are a currency or not
		((JCNumberValidator) v).setCurrency(isCurrency());
	}
}

/**
 * Implementation of an abstract superclass method that updates the
 * component's  data contents when the current row changes or the
 * record is modified.
 */
protected void refreshCell(Object item) {
  	JCValidator validator = fieldComponent.getValidator();

	// if validator does not support the new databound
	// value, create a new validator that does.
	if (validator != null && item != null &&
		    !validator.isClassSupported(item.getClass())) {

		// create a validator that supports the value
		validator = DataPropertiesRegistry.createValidator(getValueClass());
		if (validator instanceof JCNumberValidator) {
			// if it is a JCNumberValidator we need to tell it if
			// we are a currency or not
			((JCNumberValidator) validator).setCurrency(isCurrency());
		}
		fieldComponent.setValidator(validator);
	}

	dataSourceUpdating = true;
	fieldComponent.getValueModel().setValue(item);
	dataSourceUpdating = false;
}

/**
 * Implementation of an abstract superclasses method that saves the
 * component's data to the datasource.
 */
protected boolean saveCell() {
	return true;
}

}
