/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: DataSourceValueModel.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:16 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.field.db.datasource;

import com.klg.jclass.datasource.*;

import com.klg.jclass.field.JCFieldComponent;

/**
 * The datasource value model.
 */
public class DataSourceValueModel extends AbstractBindingValueModel {

/**
 * Constructs a new datasource value model for the given field component.
 * @param c  a field component
 */
public DataSourceValueModel(JCFieldComponent c) {
	super(c);
}

/* Constructs a new datasource value model.
 * @param c               a field component.
 * @param data_model      the data model.
 * @param meta_data_model the meta data model.
 * @param column_name     column name.
 */

public DataSourceValueModel(JCFieldComponent c,
							DataModel data_model,
							MetaDataModel meta_data_model,
							String column_name) {
	super(c);
	dataBinding.setDataBinding(data_model, meta_data_model, column_name);
}

/**
 * Returns the field's value of the <code>dataBinding</code> object property.
 *
 * The dataBinding property specifies a string
 * describing a <code>JCTreeData</code>
 * instance and metadata level of the instance this table is bound with.
 * The format of the property is as follows:
 * <PRE>
 * &lt;JCTreeData name&gt;:&lt;MetaDataModel description&gt;[|&lt;MetaDataModel description&gt;...]
 * </PRE>
 */
public String getDataBinding() {
	return dataBinding.getDataBinding();
}

/**
 * Sets the data binding hierarchy string,
 * starting from a <code>JCTreeData</code> instance name
 * and containing full sequence of <code>MetaDataModel</code>
 * levels starting from the root to the level
 * the table is bound with.
 */
public void setDataBinding(String data_binding) {
	dataBinding.setDataBinding(data_binding);
}

/**
 * Sets the data binding.
 */
public void setDataBinding(DataModel data_model, MetaDataModel meta_data_model,
						   String column_name) {
	dataBinding.setDataBinding(data_model, meta_data_model, column_name);
}

/**
 * Binds to a data source using a Binding.
 */
public void setDataBinding(BindingModel binding, String column_name) {
	dataBinding.setDataBinding(binding, column_name);
}

}
