/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JBdbPopupField.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:16 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.field.db.jbuilder;

import com.borland.dx.dataset.DataSet;
import com.borland.dx.dataset.DataSetAware;

import com.klg.jclass.field.JCPopupField;

/** 
 * This component is a data-bound  popup box. A popup box enables
 * end-users to visually select a date/time.
 * <P>The field displays the value at one column of the current record.
 * The data type is set when the component is bound to a data source.
 * <P>
 * <STRONG>Behavior</STRONG>
 * <UL>
 *  <LI>Selection of date and time values is simplified through the use of 
 *   a calendar popup to select a day in a chosen month, and spinners for 
 *   the selection of year, month and time information. 
 *  <LI>Optionally, the date may be typed in the field without using the popup. 
 *  <LI>Input can be checked to ensure that it is within a range of values.
 *  <LI>Audible and visual warnings may be given upon receipt of invalid entries. 
 *  <LI>Various display and edit formats are possible, as well as locale-specific formats. 
 * </UL>
 * <P>
 */

public class JBdbPopupField extends JCPopupField implements DataSetAware {

protected JBuilderValueModel vm;

/**
 * Creates a JBdbPopupField which can be bound to a data source. 
 */
public JBdbPopupField() {
	createField(null);
	vm = new JBuilderValueModel(this);
}

/**
 * Returns the DataSet object bound to the table.
 */
public DataSet getDataSet() {
	return vm.getDataSet();
}

/**
 * Sets the DataSet data source to the table.
 */
public void setDataSet(DataSet dataSet) {
    vm.setDataSet(dataSet);
}

/**
 * Returns the name of the column that the field is bound to.
 * @see com.klg.jclass.field.db.jbuilder.JBdbPopupField#setColumnName
 */
public String getColumnName() {
	return vm.getColumnName();
}

/**
 * Sets the column that the field is to be bound to.  You also need to
 * set the dataSet property to complete the task of data binding.
 * @see com.klg.jclass.field.db.jbuilder.JBdbPopupField#setDataSet
 */
public void setColumnName(String cn) {
	vm.setColumnName(cn);
}

}
