/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JBdbSpinField.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:16 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.field.db.jbuilder;

import com.borland.dx.dataset.DataSet;
import com.borland.dx.dataset.DataSetAware;

import com.klg.jclass.field.JCSpinField;

/** 
 * This component is a data-bound spin field. A spin field enables
 * end-users to cycle through a range of values.
 * <P>The field displays the value at one column of the current record.
 * The data type is set when the field is bound to a data source. 
 * <P>
 * <STRONG>Behavior</STRONG>
 * <UL>
 *   <LI>Input can be checked to ensure that it is within a range of values.
 *   <LI>The items in the spin field may be strings, in which case they are associated
 *     with integral values internally.
 *   <LI>Placing the cursor over any portion of the displayed information in the calendar enables spinning on just that
 *     sub-field. <B>(Calendar only)</B>
 *   <LI>The spin increment can be set to non-integral values. 
 *   <LI>Default and custom display formats are possible. 
 *   <LI>Audible and visual warnings may be given upon receipt of invalid entries. 
 *   <LI>Various display and edit formats are possible, as well as locale-specific formats. 
 *   <LI>Items may be matched against a pick list.
 *   <LI>Some characters may be chosen as invalid thereby preventing their use. 
 *   <LI>Alternatively, a presumably shorter list of valid characters may be specified.
 * </UL>
 */

public class JBdbSpinField extends JCSpinField implements DataSetAware {

protected JBuilderValueModel vm;

/**
 * Creates a JBdbSpinField which can be bound to a data source. 
 */
public JBdbSpinField() {
	createField(null);
	vm = new JBuilderValueModel(this);
}

/**
 * Returns the DataSet object bound to the table.
 */
public DataSet getDataSet() {
	return vm.getDataSet();
}

/**
 * Sets the DataSet data source to the table.
 */
public void setDataSet(DataSet dataSet) {
    vm.setDataSet(dataSet);
}

/**
 * Returns the name of the column that the field is bound to.
 * @see com.klg.jclass.field.db.jbuilder.JBdbSpinField#setColumnName
 */
public String getColumnName() {
	return vm.getColumnName();
}

/**
 * Sets the column that the field is to be bound to.  You also need to
 * set the dataSet property to complete the task of data binding.
 * @see com.klg.jclass.field.db.jbuilder.JBdbSpinField#setDataSet
 */
public void setColumnName(String cn) {
	vm.setColumnName(cn);
}

}
