/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JBdbTextField.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:16 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.field.db.jbuilder;

import com.borland.dx.dataset.DataSet;
import com.borland.dx.dataset.DataSetAware;

import com.klg.jclass.field.JCTextField;

/** 
 * This component is a data-bound text field. A text field enables
 * end-users to view and edit textual information. 
 * <P>The field displays the value at one column of the current record.
 * The data type is set when the field is bound to a data source. 
 * <P>
 * <STRONG>Behavior</STRONG>
 * <UL>
 *   <LI>The input field may contain a mask to enforce a specific and complete entry. 
 *   <LI>Invalid characters may be specified and the field prevents their use.
 *   <LI>Alternatively, a list of valid characters may be supplied.
 *   <LI>Many possible formats (including locale-specific formats) can be set for gathering and displaying information.
 *   <LI>Placeholders may be supplied to guide the end-user in the format the field will accept. 
 *   <LI>Input can be checked to ensure that it is within a range of values.  
 *   <LI>Audible and visual warnings may be given upon receipt of invalid entries.
 * </UL>
 */

public class JBdbTextField extends JCTextField implements DataSetAware {

protected JBuilderValueModel vm;

/**
 * Creates a JBdbTextField which can be bound to a data source. 
 */
public JBdbTextField() {
	createField(null);
	vm = new JBuilderValueModel(this);
}

/**
 * Returns the DataSet object bound to the table.
 */
public DataSet getDataSet() {
	return vm.getDataSet();
}

/**
 * Sets the DataSet data source to the table.
 */
public void setDataSet(DataSet dataSet) {
    vm.setDataSet(dataSet);
}

/**
 * Returns the name of the column that the field is bound to.
 * @see com.klg.jclass.field.db.jbuilder.JBdbTextField#setColumnName
 */
public String getColumnName() {
	return vm.getColumnName();
}

/**
 * Sets the column that the field is to be bound to.  You also need to
 * set the dataSet property to complete the task of data binding.
 * @see com.klg.jclass.field.db.jbuilder.JBdbTextField#setDataSet
 */
public void setColumnName(String cn) {
	vm.setColumnName(cn);
}


}
