/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: FieldPopup.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:17 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.field.plaf;

import com.klg.jclass.field.*;
import com.klg.jclass.field.validate.PopupFieldEditor;
import com.klg.jclass.util.swing.JCPopupListener;
import com.klg.jclass.util.swing.JCPopupEvent;


import javax.swing.*;
import javax.swing.plaf.basic.BasicComboPopup;

import java.awt.*;

public class FieldPopup extends BasicComboPopup
                        implements java.io.Serializable
{

protected PopupFieldEditor   popup = null;
protected JCPopupField       pf;
protected PopupListener      popupListener = null;

public FieldPopup(JComboBox comboBox, PopupFieldEditor popup)
{
	super(comboBox);
	
	pf = (JCPopupField)comboBox;
	setPopupFieldEditor(popup);
}

public void setPopupFieldEditor(PopupFieldEditor popup_editor)
{
	
	if (popup_editor == null) {
		return;
	}

	if (popup != null) {
		remove(popup.getEditorComponent());
	}
	add(popup_editor.getEditorComponent());
	this.popup = popup_editor;
    popup.removePopupListener(popupListener);
    popupListener = new PopupListener();
	popup.addPopupListener(popupListener);

    // Call JComponent.setRequestFocusEnabled(false) on all children of the
    // popup editor.  This resolves problems with the lightweight
	// implimentation of the popup. Bug 10046

    JComponent popupComponent = popup.getEditorComponent();
    setRequestFocusEnabledOnTree(popupComponent,false);
}

protected void setRequestFocusEnabledOnTree(Component parent, boolean state)
{
    if (parent == null) {
        return;
    }

    if (parent instanceof JComponent) {
        ((JComponent) parent).setRequestFocusEnabled(state);
    }

    if (parent instanceof Container) {
        Component[] components = ((Container)parent).getComponents();
        for (int i = 0; i < components.length; i++) {
            setRequestFocusEnabledOnTree(components[i],state);
        }
    }
}

/**
 * Called to configure this JPopupMenu (BasicComboPopup is a JPopupMenu).
 */
protected void configurePopup()
{
    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    setBorderPainted(true);
    setBorder(BorderFactory.createLineBorder(Color.black));
    setOpaque(false);
    setDoubleBuffered(true);
    setRequestFocusEnabled(false);
}

/**
 * Implementation of ComboPopup.show().
 */
public void show()
{
    if (popup == null || popup.getEditorComponent() == null) {
		return;
	}

	Object value = pf.getValue();
	if (value != null) {
		popup.setValue(value);
	}

    Dimension popupSize = comboBox.getSize();
    popupSize.setSize(popupSize.width, 200);
    // need this, otherwise first popup will think it has a size of (0, 0)
    JComponent editorComponent = popup.getEditorComponent();
    editorComponent.setSize(editorComponent.getPreferredSize());
    Rectangle popupBounds =
	    computePopupBounds(0, comboBox.getBounds().height,
						   editorComponent.getWidth(),
						   editorComponent.getHeight());
    setLightWeightPopupEnabled(comboBox.isLightWeightPopupEnabled());

    show(comboBox, popupBounds.x, popupBounds.y);
}

protected Rectangle computePopupBounds(int px,int py,int pw,int ph)
{
    Insets insets = getInsets();
    Rectangle absBounds = new Rectangle();
    Rectangle r = new Rectangle(px, py, pw, ph);
    Point p = new Point(0, 0);
    Dimension scrSize = Toolkit.getDefaultToolkit().getScreenSize();

    SwingUtilities.convertPointFromScreen(p, comboBox);
    absBounds.x = p.x;
    absBounds.y = p.y;
    absBounds.width = scrSize.width;
    absBounds.height = scrSize.height;
    if (isRectangleContainingRectangle(absBounds, r)) {
        return(r);
    }
    else {
        // Find a place for it to fit
        if (!isRectangleContainingRectangleFitRight(absBounds, r)) {
            r.x += (absBounds.x + absBounds.width -
                    (r.width + insets.left + insets.right));
        }
        if (!isRectangleContainingRectangleFitLeft(absBounds, r)) {
            r.x = absBounds.x;
        }
        if (!isRectangleContainingRectangleFitDown(absBounds, r)) {
            r.y = -(r.height + insets.top + insets.bottom);
        }
        return(r);
    }
}

public final boolean isRectangleContainingRectangle(Rectangle a,Rectangle b)
{
    return(b.x >= a.x && (b.x + b.width) <= (a.x + a.width) &&
           b.y >= a.y && (b.y + b.height) <= (a.y + a.height));
}

public final boolean isRectangleContainingRectangleFitLeft(Rectangle a,
						                                   Rectangle b)
{
    return(a.x < 0);
}

public final boolean isRectangleContainingRectangleFitRight(Rectangle a,
						                                    Rectangle b)
{
    return((b.x + b.width) < (a.x + a.width));
}

public final boolean isRectangleContainingRectangleFitDown(Rectangle a,
						                                   Rectangle b)
{
    return(b.y >= a.y && (b.y + b.height) <= (a.y + a.height));
}

public void hide()
{
    super.hide();
}

class PopupListener implements JCPopupListener, java.io.Serializable {

    public void commit(JCPopupEvent e)
	{
	    hide();
	}
} // PopupListener

}
