/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCBigDecimalValidator.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:17 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.field.validate;

import java.util.*;
import java.text.*;
import java.math.BigDecimal;

import com.klg.jclass.field.resources.LocaleInfo;


/**
 * This class validates objects of type <code>BigDecimal</code>.
 */
public class JCBigDecimalValidator extends JCNumberValidator {

protected Object   defaultValue = new BigDecimal(0.0);
protected boolean  useFormatting = false;

/**
 * Constructs a <code>JCBigDecimalValidator</code>.
 */
public JCBigDecimalValidator()
{
    super();

    super.setMax(new BigDecimal(Double.MAX_VALUE));
    super.setMin(new BigDecimal(-Double.MAX_VALUE));

    setIncrement(new BigDecimal(1.0));
    supportedClasses = getSupportedClasses();
}

/**
 * Constructs a <code>JCBigDecimalValidator</code> with the values specified.
 * @param allow_null
 * @param currency
 * @param currency_locale
 * @param default_value
 * @param display_pattern
 * @param increment
 * @param locale
 * @param max
 * @param min
 * @param use_intl_currency_locale
 */
public JCBigDecimalValidator(Locale locale, Number min, Number max,
                             Number increment, String display_pattern,
                             boolean allow_null, boolean currency,
                             boolean use_intl_currency_locale,
                             Locale currency_locale, Object default_value)
{
    this();
    setLocale(locale);
    setCurrencyLocale(currency_locale);
    setUseIntlCurrencySymbol(use_intl_currency_locale);
    setMax(getBigDecimal(max));
    setMin(getBigDecimal(min));
    setIncrement(getBigDecimal(increment));
    if (display_pattern != null) {
        setDisplayPattern(display_pattern);
    }
    setAllowNull(allow_null);
    setCurrency(currency);
    setDefaultValue(default_value);
}

/**
 * Creates a list of classes supported by the validator.
 * @return
 */
protected Class[] getSupportedClasses()
{
    Class[] supported_classes = new Class[3];

    supported_classes[0] = Double.class;
    supported_classes[1] = Float.class;
    supported_classes[2] = BigDecimal.class;

    return(supported_classes);
}

/**
 * Given a <code>Number</code> object, this method will return a
 * <code>BigDecimal</code> object.
 * @param num
 * @return
 */
protected BigDecimal getBigDecimal(Object num)
{
    // Make sure there is a number to convert
    if (num == null || !(num instanceof Number)) {
        return(null);
    }

    // if it is a big decimal already, return it
    if (num instanceof BigDecimal) {
        return((BigDecimal)num);
    }

    double val = ((Number)num).doubleValue();
    BigDecimal bd = null;
    try {
        bd = new BigDecimal(val);
    }
    catch (NumberFormatException e) {
        // Happens if val is Infinity or NaN
        bd = new BigDecimal(Double.MAX_VALUE);
    }
    return(bd);
}

protected DecimalFormat createDisplayFormat(Locale l)
{
    return((DecimalFormat)NumberFormat.getNumberInstance(l));
}

protected DecimalFormat createEditFormat(Locale l)
{
    return((DecimalFormat)NumberFormat.getNumberInstance(l));
}

protected void setPatterns()
{
    String np[] = li.getStringArray(LocaleInfo.NUMBER_PATTERNS);
    if (isCurrency) {
        display_pattern = np[LocaleInfo.CURRENCY_PATTERN];
        edit_pattern = "0.##";
    }
    else {
        display_pattern = np[LocaleInfo.DECIMAL_PATTERN];
        edit_pattern = "0.###";
    }
}

/**
 *  Sets the cursor to the specified position.
 * @param e
 */
public void setCursor(TextCursorEvent e)
{
    // We allow all positions in number edits
    e.getNewPosition();
}

protected Object addIncrement(Object current)
{
    if (increment == null || current == null) {
        return(null);
    }

    BigDecimal new_value = getBigDecimal(current).add(getBigDecimal(increment));
    if (new_value.compareTo(getBigDecimal(getMax())) == 1) {
        return(null);
    }

    return(new_value);
}

protected Object subtractIncrement(Object current)
{
    if (increment == null || current == null) {
        return(null);
    }

    BigDecimal new_value =
        getBigDecimal(current).subtract(getBigDecimal(increment));
    if (new_value.compareTo(getBigDecimal(getMin())) == -1) {
        return(null);
    }

    return(new_value);
}

/**
 *  Gets the number by which to spin the values in numeric spin fields.
 * @return number to spin the values in the numeric spin fields
 */
public Number getIncrement()
{
    return((Number)increment);
}

/**
 *  Sets the number by which to spin the values in numeric spin fields.
 * @param increment
 */
public void setIncrement(Number increment)
{
    BigDecimal new_increment = getBigDecimal(increment);
    if (increment == null) {
        return;
    }
    if (new_increment.compareTo(new BigDecimal(0.0)) == -1) {
        throw new IllegalArgumentException("increment <= 0.0");
    }
    this.increment = increment;
}

/**
 *  Checks if the value is in the valid range.
 * @return
 * @param value
 */
public boolean inRange(BigDecimal value)
{
    if (value.compareTo(getBigDecimal(getMin())) == -1 ||
        value.compareTo(getBigDecimal(getMax())) == 1)
    {
        return(false);
    }
    return(true);
}

/**
 *  Checks if the value is in the valid range.
 * @param value
 * @return
 */
public boolean inRange(Object value)
{
    if (value instanceof Number) {
        return(inRange(getBigDecimal(value)));
    }
    else {
        return(false);
    }
}

/**
 * Changes the specified text from the <code>start_position</code> to the
 * <code>end_position</code> by inserting/replacing it by a range specified in
 * <code>new_text</code>.
 * @param e The text event generated by the text change
 */
public void changeText(TextEvent e)
{
    if (e == null) {
        return;
    }
    StringBuffer allowed_new = null;
    StringBuffer working = new StringBuffer(50);

    boolean exponent_char_typed = false;

    // Decimal position of -1 indicates that there is no decimal place
    boolean decimal_typed = false;

    int start = e.getStartPosition();
    int end = e.getEndPosition();

    String oldText = e.getOriginalText();
    if (oldText.indexOf(symbols.getDecimalSeparator()) != -1) {
        decimal_typed = true;
        if (e.isDeletion() == true) {
            // Scan through and see if the decimal character is in the text
            // that is pending delete
            String s = oldText.substring(start, end);
            int decimal_index = s.indexOf(symbols.getDecimalSeparator());

            if (decimal_index != -1) {
                decimal_typed = false;
            }
        }
    }

    if (oldText.indexOf(exponentialChar) != -1) {
        exponent_char_typed = true;
        if (e.isDeletion() == true) {
            // Scan through and see if the exponent character is in the text
            // that is pending delete
            String substring = oldText.substring(start, end);
            if (substring.indexOf(exponentialChar) != -1) {
                exponent_char_typed = false;
            }
        }
    }

    // Figure out if we are at the end or at the beginning
    // We need to know this so that we can
    //  a) Only allow +/-/() symbols at the beginning _or_ the end
    //     of the text
    //  b) We do not allow +/- or decimal signs if they have already
    //     been typed
    //  c) If the +/- signs are not at the begining then they are at the
    //     end and that means that we do not allow any further characters
    //     to be input
    boolean sign_typed = false;
    boolean open_typed = false;
    boolean at_beginning = false;
    boolean at_end = false;

    if (start == 0) {
        at_beginning = true;
    }
    else if (oldText != null && oldText.length() > 1) {
        // Check last position to see if it tells us we are at the end
        char last_char = oldText.charAt(oldText.length() - 1);
        if (last_char == symbols.getMinusSign() &&
            exponent_char_typed == false)
        {
            at_end = true;
        }
        else if (last_char == '+') {
            at_end = true;
        }
        else if (last_char == ')') {
            at_end = true;
        }
    }

    if (oldText.indexOf(symbols.getMinusSign()) != -1) {
        sign_typed = true;
    }
    else if (oldText.indexOf('+') != -1) {
        sign_typed = true;
    }
    else if (oldText.indexOf('(') != -1) {
        sign_typed = true;
        open_typed = true;
    }

    String newText = e.getText();
    if (newText != null && newText.length() != 0) {
        // Call function in super class to strip out specific
        // valid/invalid chars
        allowed_new = checkValidInvalid(newText);
        if (allowed_new != null && allowed_new.length() != 0) {
            // Validate input keys and strip out any that are invalid
            for (int i = 0; i < allowed_new.length(); i++) {
                char c = allowed_new.charAt(i);
                if (at_end) {
                    continue;
                }

                if (Character.isDigit(c) == true) {
                    // allow digits
                    working.append(c);
                }
                else if (decimal_typed == false &&
                         exponent_char_typed == false &&
                         c == symbols.getDecimalSeparator())
                {
                    // Only allow 1 decimal character
                    working.append(c);
                }
                else if (exponent_char_typed == false &&
                         at_beginning == false &&
                         exponentialChar.equalsIgnoreCase(String.valueOf(c)) &&
                         (oldText.indexOf(symbols.getMinusSign()) + 1) != start)
                {
                    // Only allow 1 exponent character
                    working.append(exponentialChar);
                }
                else if (c == symbols.getMinusSign() &&
                         ((sign_typed == false &&
                           exponent_char_typed == false) ||
                          (exponent_char_typed == true &&
                           (oldText.indexOf(exponentialChar) +
                            exponentialChar.length()) == start)))
                {
                    // Allow negative symbol
                    working.append(c);
                }
                else if ((c == '(' || c == ')') &&
                         edit_pattern.indexOf("(") != -1)
                {
                    // Allow ( and ) if we can find it in the edit_pattern
                    // since the parse will fail if it's not

                    // Further more only allow a "(" at position 0
                    // and ")" if a "(" has already been typed
                    if (c == '(' && at_beginning) {
                        working.append(c);
                    }
                    else if (c == ')' && open_typed) {
                        working.append(c);
                    }
                }
            }
        }
        e.setText(working.toString());
        if (working.length() == 0) {
            e.setAllowChange(false);
        }
    }
}

/**
 * Parses the given text and validates it.
 * @param class_type object returned will be of the specified class type
 * @param text text to parse
 * @return
 */
public Object parse(Class class_type, String text)
{
    Object value = null;

    if ((text == null || text.length() == 0) && getAllowNull() == true) {
        return(null);
    }

    if (useFormatting) {
        ParsePosition position = new ParsePosition(0);
        Number num = editFormat.parse(text, position);
        if (position.getIndex() == 0) {
            if (getAllowNull() == false) {
                throw new FieldParseException("null values not allowed");
            }
            return(null);
        }
        else {
            value = new BigDecimal(num.doubleValue());
        }
    }
    else {
        try {
            value = new BigDecimal(text);
        }
        catch(NumberFormatException e) {
            if (getAllowNull() == false) {
                throw new FieldParseException(
                        "cannot create BigDecimal from text");
            }
            return(null);
        }
    }

    // convert value back to original type
    value = convertFromSupported(value, class_type);
    return(value);
}

/**
 * Validates a given object.
 * @return
 * @param obj
 */
public boolean validate(Object obj)
{
    boolean is_valid = super.validate(obj);
    if (is_valid == false) {
        return(false);
    }

    Object value = obj;
    // make sure the class is supported
    if (obj != null && isClassSupported(obj.getClass())) {
        value = convertToSupported(value);
    }

    // check to see if object is in the allowable range
    if (value != null && !inRange(obj)) {
        return(false);
    }

    return(true);
}

/**
 * Override the base class method.
 * Returns a formatted String given an object.
 * @return a formatted String
 * @param obj
 */
public String format(Object obj)
{
    String str = "";
    if (useFormatting) {
        str = super.format(obj);
    }
    else if (obj != null) {
        str = obj.toString();
    }
    return(str);
}

/**
 * Generates a String representation of the given object for use in edit mode.
 * @return
 * @param obj
 */
public String formatForEdit(Object obj)
{
    

    if (obj == null) {
        return ("");
    }

    String str = null;
    if (useFormatting) {
        str = editFormat.format(obj);
    }
    else {
        str = obj.toString();
    }
    return(str);
}

/**
 * Reports whether the validator has a different edit format or not.
 * @return
 */
public boolean hasEditFormat()
{
    return(true);
}

/**
 * Converts the specified type (if possible) to the type supported by
 * the validator.  To reverse this operation use the
 * <code>convertFromSupported()</code> method.
 * @see #convertFromSupported
 * @return
 * @param obj
 */
public Object convertToSupported(Object obj)
{
    if (obj instanceof Number) {
        return(getBigDecimal(obj));
    }
    return(null);
}

/**
 * Converts the supported type to the type supported by
 * the validator. <br><B>Note:</b> The object passed in is not modified, but
 * only used as a reference.
 * @see #convertToSupported
 * @return
 * @param from
 * @param class_type
 */
public Object convertFromSupported(Object from, Class class_type)
{
    if (from == null || class_type == null) {
        return(null);
    }

    String class_name = class_type.getName();
    BigDecimal num = (BigDecimal)from;

    if (class_name.equals("java.lang.Float")) {
        return(new Float(num.floatValue()));
    }
    else if (class_name.equals("java.lang.Double")) {
        return(new Double(num.doubleValue()));
    }
    else if (class_name.equals("java.math.BigDecimal")) {
        return(num);
    }
    return(null);
}

/**
 * Returns the default value for the field.
 * @return default value
 */
public Object getDefaultValue()
{
    return(defaultValue);
}

/**
 * Sets the default value for the field.
 * @param default_value
 */
public void setDefaultValue(Object default_value)
{
    this.defaultValue = default_value;
}

/**
 * Sets the value of the useFormatting property.
 * @param f true if formatting is desired.
 */
public void setUseFormatting(boolean f)
{
    useFormatting = f;
}

/**
 * Returns true if formatting is used.
 */
public boolean getUseFormatting()
{
    return(useFormatting);
}

}
