/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCDateTimeValidator.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:17 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.field.validate;

import com.klg.jclass.field.JCSpinField;
import com.klg.jclass.field.resources.LocaleInfo;
import com.klg.jclass.util.JCListenerList;
import com.klg.jclass.util.calendar.*;
import com.klg.jclass.util.swing.JCColumnLayout;
import com.klg.jclass.util.swing.JCPopupListener;
import com.klg.jclass.util.swing.JCPopupEvent;
import com.klg.jclass.util.value.CalendarValueModel;
import com.klg.jclass.util.value.JCValueModel;


import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.*;
import java.io.Serializable;
import java.text.DateFormatSymbols;
import java.text.MessageFormat;
import java.util.*;

// Note that DateMask is not a mask but a compiled "Format"
class DateMask implements Cloneable, Serializable {
	int      symbol;                      // mask symbol type
	int      field;                       // field used by calendar object
	int      length;                      // length of the subfield
	String   chars;                       // For literal Symbols and representation
	String   strings[] = null;            // For use with enumerated types
	DateMask absolute_replacement = null; // non-null for ambiguous fields
    public Object clone()
	{
		try {
			return(super.clone());
		}
		catch (CloneNotSupportedException e) {
			return(null);
		}
	}
}

class DateFormat implements Cloneable, Serializable {

	DateMask[] elements;
	boolean    size_ambiguous;

	int getField(int i)
	{
	    return(elements[i].field);
	}

	String getString(int i, int str_i)
	{
	    return(elements[i].strings[str_i]);
	}

	String[] getStrings(int i)
	{
	    return(elements[i].strings);
	}

	String getChars(int i)
	{
	    return(elements[i].chars);
	}

	int getSymbol(int i)
	{
	    return(elements[i].symbol);
	}

	int getLength(int i)
	{
	    return(elements[i].length);
	}

	void setLength(int i, int length)
	{
	    elements[i].length = length;
	}

} // DateFormat

/**
 * The <CODE>JCDateTimeValidator</CODE> is used to validate calendar objects.
 */
public class JCDateTimeValidator extends JCStringValidator {

//
// Inner class to bypass new 1.1.4 Calendar behavior
//
protected class WorkingCalendar implements Cloneable, Serializable {

private int fields[] = new int[Calendar.FIELD_COUNT];

public WorkingCalendar()
{
	Calendar cal = Calendar.getInstance(tz, locale);
	create(cal);
}

public WorkingCalendar(Calendar cal)
{
	create(cal);
}

private void create(Calendar cal)
{
	for (int i = 0; i < fields.length; i++) {
		if (cal.isSet(i)) {
			fields[i] = cal.get(i);
		}
		else {
			fields[i] = -1;
		}
	}
}

public void clear()
{
	for (int i = 0; i < fields.length; i++) {
		fields[i] = -1;
	}
}

public int get(int field)
{
	return(fields[field]);
}

public void set(int field, int value)
{
	fields[field] = value;
}

public boolean isSet(int field)
{
	if (fields[field] == -1) {
		return(false);
	}
	return(true);
}

public TimeZone getTimeZone()
{
	return(tz);
}

public void setTimeZone(TimeZone atz)
{
	tz = atz;
}

public Calendar toCalendar()
{
	Calendar cal = Calendar.getInstance(tz, locale);
	/*
	for (int i = 0; i < fields.length; i++) {
		if (isSet(i)) {
			cal.set(i, get(i));
		}
	}
	*/

	if (get(Calendar.HOUR_OF_DAY) == -1 && get(Calendar.HOUR) != -1) {
		setHourOfDayFromHour(this);
	}

	// Create a calendar from the working calendar; zeroing out any
	// fields that were not set; for date purposes we "zero" out to
	// January 1, 1970 because we know that is a date that all systems
	// will handle
	cal.set(get(Calendar.YEAR) == -1 ? 1970 : get(Calendar.YEAR),
			get(Calendar.MONTH) == -1 ? 0 : get(Calendar.MONTH),
			get(Calendar.DATE) == -1 ? 1 : get(Calendar.DATE),
			get(Calendar.HOUR_OF_DAY) == -1 ? 0 : get(Calendar.HOUR_OF_DAY),
			get(Calendar.MINUTE) == -1 ? 0 : get(Calendar.MINUTE),
			get(Calendar.SECOND) == -1 ? 0 : get(Calendar.SECOND));

    
	return(cal);
}

public Object clone()
{
	try {
		return(super.clone());
	}
	catch (CloneNotSupportedException e) {
		return(null);
	}
}

} // Working Calendar

static final int NOVALUE  = -1;
static final int NOTFOUND = -1;
static final int PARSE_ERROR = -1;

// Enum for defaultDetail

/**
 * Detail level of the default format for date/time validators.
 * Example: Fri, Apr 30, 1999 01:33:05 PM EST
 */
static public final int FULL   = 0;
/**
 * Detail level of the default format for date/time validators.
 * Example: Apr 30, 1999 01:34:21 PM EST
 */
static public final int LONG   = 1;
/**
 * Detail level of the default format for date/time validators.
 * Example: Apr 30, 1999 01:35:08 PM
 */
static public final int MEDIUM = 2;
/**
 * Detail level of the default format for date/time validators.
 * Example: 04/30/99 01:35 PM
 */
static public final int SHORT  = 3;

// Mask symbols used in date mask private class
static final int DATE_LITERAL       =  1; /* any literal token, e.g. : */
static final int MERIDIEM           =  2; /* AM or PM, meridiem is latin for noon */
static final int ABBREV_TIMEZONE    =  3; /* local time zone, e.g. EST */
static final int TIMEZONE           =  4; /* local time zone, e.g. Eastern Standard Time */
static final int MERIDIEM_HOUR      =  5; /* 0-12 hour */
static final int ZERO_MERIDIEM_HOUR =  6; /* zero padded 0-12 hour */
static final int MILITARY_HOUR      =  7; /* 0-23 hour */
static final int ZERO_MILITARY_HOUR =  8; /* zero padded 0-23 hour */
static final int MINUTE             =  9; /* standard earth minutes */
static final int ZERO_MINUTE        = 10;				/* zero padded minute */
static final int SECOND             = 11; /* standard earth second */
static final int ZERO_SECOND        = 12; /* zero padded second */
static final int NUM_DAY            = 13; /* day of the month e.g. 1-31 */
static final int ZERO_NUM_DAY       = 14; /* zero padded day of the month e.g. 01-31 */
static final int ABBREV_ALPHA_DAY   = 15; /* abbreviated day of the week e.g. Mon */
static final int ALPHA_DAY          = 16; /* day of the week e.g. Monday */
static final int DAY_OF_YEAR        = 17; /* Julian date: 1-366 */
static final int ZERO_DAY_OF_YEAR   = 18; /* Julian date: 001-366 */
static final int NUM_MONTH          = 19; /* month number, e.g. 1-12 */
static final int ZERO_NUM_MONTH     = 20; /* zero padded month number, e.g. 01-12 */
static final int ABBREV_ALPHA_MONTH = 21; /* abbreviated name of the month, e.g. Nov */
static final int ALPHA_MONTH        = 22; /* name of the month, e.g. November */
static final int SHORT_YEAR         = 23; /* last two digits of the year, e.g. 94 */
static final int LONG_YEAR          = 24; /* all four digits of year, e.g. 1995 */
static final int MILLISECOND        = 25;

// Other symbols
static final int AMSET = -2;
static final int PMSET = -3;

// Symbols for indexing into TimeZoneStrings
static final int ZONE_ID             = 0;
static final int ZONE_STANDARD_LONG  = 1;
static final int ZONE_STANDARD_SHORT = 2;
static final int ZONE_DAYLIGHT_LONG  = 3;
static final int ZONE_DAYLIGHT_SHORT = 4;
static final int ZONE_CITY_NAME      = 5;

static final byte daytable[][] = {
	{31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
	{31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
};

protected Class[]            supportedClasses;
protected boolean            mask_input = false;
protected String             date_format;
protected DateFormat         compiled_format;
protected TimeZone           tz = null;
protected WorkingCalendar    cal = null;
protected DateFormatSymbols  symbols;
protected Vector             edit_formats = new Vector(0);
protected Vector             compiled_edit_formats = new Vector(0);
protected int                increment_field = Calendar.HOUR;
protected int                millenium_threshold = 70;
protected int                default_detail = MEDIUM;
protected int                spin_policy = SPIN_SUBFIELD;
protected String             original_format = null;
protected String             default_format = null;
protected Object             defaultValue = null;

/**
 * Creates a JCDateTimeValidator.
 *
 */
public JCDateTimeValidator()
{
	if (tz == null) {
		tz = TimeZone.getDefault();
	}
	if (cal == null) {
		cal = new WorkingCalendar(Calendar.getInstance(tz, locale));
	}
	if (defaultValue == null) {
		defaultValue = Calendar.getInstance(tz, locale);
	}
 	symbols = new DateFormatSymbols(locale);
	increment = new Integer(1);
	default_format = getDefaultFormat();
	setFormat(default_format);
	setEditFormats(getDefaultEditFormats());
	setSpinPolicy(SPIN_SUBFIELD);
	supportedClasses = getSupportedClasses();
}

/**
 * Creates a <code>JCDateTimeValidator</code> with the values specified.
 * @param allow_null
 * @param default_detail
 * @param default_value
 * @param edit_formats
 * @param format
 * @param locale
 * @param mask_input
 * @param place_holder_chars
 */
public JCDateTimeValidator(Locale locale, String format,
						   String place_holder_chars,
						   String[] edit_formats, boolean mask_input,
						   int default_detail, boolean allow_null,
						   Object default_value)
{
	this();
	setLocale(locale);
	default_format = getDefaultFormat();
	if (format != null && format.length() != 0) {
		try {
			setFormat(format);
		}
		catch (IllegalArgumentException e) {
			setFormat(default_format);
		}
	}
	setPlaceHolderChars(place_holder_chars);
	setEditFormats(edit_formats);
	setMaskInput(mask_input);
	setDefaultDetail(default_detail);
	setAllowNull(allow_null);
	setDefaultValue(default_value);
}

/**
 * Creates a <code>JCDateTimeValidator</code> with the values specified.
 * @param allow_null
 * @param default_detail
 * @param default_value
 * @param edit_formats
 * @param format
 * @param locale
 * @param mask_input
 * @param millenniumThreshold
 * @param place_holder_chars
 */
public JCDateTimeValidator(Locale locale, String format,
						   String place_holder_chars,
						   String[] edit_formats, boolean mask_input,
						   int default_detail, boolean allow_null,
						   Object default_value,int millenniumThreshold)
{
    this(locale,format, place_holder_chars, edit_formats, mask_input,
	     default_detail, allow_null,default_value);

    this.setMilleniumThreshold(millenniumThreshold);
}


/**
 * Creates a list of classes supported by the validator.
 * @return
 */
protected Class[] getSupportedClasses()
{
	Class[] supported_classes = new Class[6];

	supported_classes[0] = Calendar.class;
	supported_classes[1] = java.util.Date.class;
	supported_classes[2] = GregorianCalendar.class;
	supported_classes[3] = java.sql.Date.class;
	supported_classes[4] = java.sql.Time.class;
	supported_classes[5] = java.sql.Timestamp.class;

	return(supported_classes);
}

/**
 * Gets the default <code>Mask</code> for the validator given a specific locale.
 * @return
 * @param l
 */
public String getDefaultFormat(Locale l)
{
	ResourceBundle li =
	    ResourceBundle.getBundle("com.klg.jclass.field.resources.LocaleInfo", l);
	String dtp[] = li.getStringArray(LocaleInfo.DATE_TIME_PATTERNS);

	MessageFormat f = new MessageFormat(dtp[LocaleInfo.DATE_TIME_ORDER]);
	String args[] = new String[2];
	switch (default_detail) {
	case FULL:
		args[0] = dtp[LocaleInfo.FULL_TIME];
		args[1] = dtp[LocaleInfo.FULL_DATE];
		break;
	case LONG:
		args[0] = dtp[LocaleInfo.LONG_TIME];
		args[1] = dtp[LocaleInfo.LONG_DATE];
		break;
	case MEDIUM:
		args[0] = dtp[LocaleInfo.MEDIUM_TIME];
		args[1] = dtp[LocaleInfo.MEDIUM_DATE];
		break;
	case SHORT:
		args[0] = dtp[LocaleInfo.SHORT_TIME];
		args[1] = dtp[LocaleInfo.SHORT_DATE];
		break;
	}

	return(f.format(args));
}

/**
 * Gets the default <code>Mask</code> for the validator.
 * @return
 */
public String getDefaultFormat()
{
	return(getDefaultFormat(locale));
}

/**
 * Returns the detail level of the <I>default</I> format.
 * @see #setDefaultDetail
 * @return detail level of the <i>default</i> format
 */
public int getDefaultDetail()
{
	return(default_detail);
}

/**
 * Specifies the detail level of the <I>default</I> format.
 * Has no effect when the format property has been changed.
 * Valid values are: <code>FULL</code>, <code>LONG</code>, <code>MEDIUM</code> (default), or <code>SHORT</code>.
 * @see #getDefaultDetail
 * @param detail
 */
public void setDefaultDetail(int detail)
{
	if (detail != FULL   &&
	    detail != LONG   &&
		detail != MEDIUM &&
		detail != SHORT)
	{
		throw new IllegalArgumentException("Invalid value for defaultDetail");
	}

	default_detail = detail;

	// Check if the format matches the default format.  If so, the user has
	// not explicitly set a format.  Since the detail is changing, we need
	// to change the default format and use it.
	String format = getFormat();
	if (format != null && format.equals(default_format) == true) {
		default_format = getDefaultFormat();
		setFormat(default_format);
	}
}

private static String [] getStringArrayCopy(Vector v)
{
	String strings[] = new String[v.size()];
	for (int i = 0; i < v.size(); i++) {
		strings[i] = (String) v.elementAt(i);
	}
	return(strings);
}

/**
 * Gets the default <code>EditFormats</code> for the validator.
 * @return
 * @param l
 */
public String[] getDefaultEditFormats(Locale l)
{
	Vector formats = new Vector(0);
	ResourceBundle li;

	if (this.locale == l) {
		li = this.li;
	}
	else {
		li = ResourceBundle.getBundle("com.klg.jclass.field.resources.LocaleInfo", l);
	}

	String dtp[]     = li.getStringArray(LocaleInfo.DATE_TIME_PATTERNS);
	String fulltime  = dtp[LocaleInfo.FULL_TIME];
	String longtime  = dtp[LocaleInfo.LONG_TIME];
	String medtime   = dtp[LocaleInfo.MEDIUM_TIME];
	String shorttime = dtp[LocaleInfo.SHORT_TIME];
	String fulldate  = dtp[LocaleInfo.FULL_DATE];
	String longdate  = dtp[LocaleInfo.LONG_DATE];
	String meddate   = dtp[LocaleInfo.MEDIUM_DATE];
	String shortdate = dtp[LocaleInfo.SHORT_DATE];

	// Build up combinations of the two using the date time order
	// format specified in the locale in conjuction with a
	// MessageFormat object
	MessageFormat f = new MessageFormat(dtp[LocaleInfo.DATE_TIME_ORDER]);

	String dates[] = {fulldate, longdate, meddate, shortdate};
	String times[] = {fulltime, longtime, medtime, shorttime};

	String args[] = new String[2];
	int i, j;
	for (i = 0; i < 4; i++) {
		for (j = 0; j < 4; j++) {
			args[0] = dates[i];
			args[1] = times[j];
			// sprintf time and date together in a locale specific way
			formats.addElement(f.format(args));
		}
	}

	return(getStringArrayCopy(formats));
}

/**
 * Sets/restores the default <code>EditFormats</code> for the validator.
 * @return
 */
public String[] getDefaultEditFormats()
{
	return(getDefaultEditFormats(locale));
}

/**
 * Returns the list of <code>editFormats</code> used to attempt to parse user input.
 * @see #setEditFormats
 * @return list of <code>editFormats</code>
 */
public String[] getEditFormats()
{
	return(getStringArrayCopy(edit_formats));
}

/**
 * Sets the list of <code>editFormats</code> used in the attempt to parse user input.
 * @see #getDefaultEditFormats
 * @see #getEditFormats
 * @see #addEditFormat
 * @param formats
 */
public void setEditFormats(String[] formats)
{
	if (formats == null) {
		return;
	}

	edit_formats = new Vector(formats.length);
	for (int i = 0; i < formats.length; i++) {
		edit_formats.insertElementAt(formats[i], i);
	}

	compiled_edit_formats.setSize(0);
	for (int i = 0; i < formats.length; i++) {
		compiled_edit_formats.addElement(compileFormat(formats[i]));
	}
}

private int getSymbol(int position)
{
    
	for (int count = 0, i = 0; i < compiled_format.elements.length; i++) {
		count += compiled_format.getLength(i);
        
		if (position < count) {
            
			return(compiled_format.getSymbol(i));
		}
	}
	return(NOVALUE);
}

private int getIncrementField(int symbol)
{
	int field = NOVALUE;
	switch (symbol) {
	case NUM_DAY:				/* Day of Month */
	case ZERO_NUM_DAY:
		field = Calendar.DAY_OF_MONTH;
		break;
	case ABBREV_ALPHA_DAY:				/* Day of Week */
	case ALPHA_DAY:
		field = Calendar.DAY_OF_WEEK;
		break;
	case NUM_MONTH:				/* Month */
	case ZERO_NUM_MONTH:
	case ABBREV_ALPHA_MONTH:
	case ALPHA_MONTH:
		field = Calendar.MONTH;
		break;
	case SHORT_YEAR:				/* year */
	case LONG_YEAR:
		field = Calendar.YEAR;
		break;
	case MERIDIEM:		/* AM or PM, meridiem is latin for noon */
		// increment 12 hours to change AM/PM
		field = Calendar.AM_PM;
		break;
		// intentional fall through follows
	case MILITARY_HOUR:
	case ZERO_MILITARY_HOUR:
	    field = Calendar.HOUR_OF_DAY;
		break;
	case MERIDIEM_HOUR:
	case ZERO_MERIDIEM_HOUR:
		field = Calendar.HOUR;
		break;
	case MINUTE:
	case ZERO_MINUTE:
		field = Calendar.MINUTE;
		break;
	case SECOND:
	case ZERO_SECOND:
		field = Calendar.SECOND;
		break;
	case ABBREV_TIMEZONE:	/* local time zone, for us it is EST */
	case TIMEZONE:
		// We don't currently handle spinning time zones
		break;
    case ZERO_DAY_OF_YEAR:
        field = Calendar.DAY_OF_YEAR;
	}

	return(field);
}

/**
 * Infers the sub-field in the currently selected region that will be
 * selected. If there is no selected region, then the selected region is
 * equal to the cursor position.
 * @param selection_start
 * @param selection_end
 */
public void inferSubField(int selection_start, int selection_end)
{
    

	// If we do not have a mask or are not conforming to it then we
	// cannot determine the field
	if (mask_input == false || date_format == null) {
        
		return;
	}

	// Now we try to figure out which field we are in.
	// First try to loop backwards until we hit a valid field
	int position = Math.min(selection_start, date_format.length() - 1);
	int field = NOVALUE;
	for (; field == NOVALUE && position >= 0; position--) {
		field = getIncrementField(getSymbol(position));
	}

	// Now loop forwards if we still haven't found a field
	for (position = Math.max(selection_start, 0);
	     field == NOVALUE && position < date_format.length();
		 position++)
	{
		field = getIncrementField(getSymbol(position));
	}

	if (field != NOVALUE) {
        
		setIncrementField(field);
	}
}

/**
 * Gets the number by which to spin the values in numeric spin fields.
 * @return
 */
public int getIncrement()
{
	if (increment == null) {
		return(0);
	}

	return(((Integer) increment).intValue());
}

/**
 * Sets the number by which to spin the values in numeric spin fields.
 * @param increment
 */
public void setIncrement(int increment)
{
	this.increment = new Integer(increment);
}

/**
 * Returns the <code>incrementField</code> operated on by the currently set increment value.
 * @see java.util.Calendar#set
 * @return <code>incrementField</code> operated on by the currently set increment value
 */
public int getIncrementField()
{
	return(increment_field);
}

/**
 * Sets the value of the <code>incrementField</code> operated on by the increment.
 * @see java.util.Calendar#set
 * @param field
 */
public void setIncrementField(int field)
{
	if (field < 0 && field >= Calendar.FIELD_COUNT) {
		throw new IllegalArgumentException("invalid value for incrementField");
	}

	increment_field = field;
}

/**
 * Returns millenium threshold.
 * @see #setMilleniumThreshold
 * @return millenium threshold
 */
public int getMilleniumThreshold()
{
	return(millenium_threshold);
}

/**
 * Sets the <code>MilleniumThreshold</code>, which is a value used for interpreting 2 digit
 * years. Any 2 digit date less than the threshold is considered to be after the
 * year 2000, while any value greater than or equal to the threshold is
 * considered to be after the year 1900.  The default is 69 so that,
 * for example, '96 --> 1996 and '10 == 2010.
 * @param threshold valid values are 0 through 100
 */
public void setMilleniumThreshold(int threshold)
{
	if (threshold < 0 && threshold >= 100) {
		throw new IllegalArgumentException("invalid value for milleniumThreshold");
	}

	millenium_threshold = threshold;
}

// Implement own field spinning since calendar roll does not work on all fields.
private void spinSubField(Calendar cal, int amount)
{
	int field = increment_field;

	switch (field) {
	case Calendar.ERA:
	case Calendar.ZONE_OFFSET:
	case Calendar.DST_OFFSET:
	case Calendar.WEEK_OF_YEAR:
	case Calendar.WEEK_OF_MONTH:
	case Calendar.DAY_OF_WEEK_IN_MONTH:
		throw new IllegalArgumentException();
	case Calendar.AM_PM:
		if (cal.get(field) == 0) {
			cal.set(field, 1);
			cal.set(cal.HOUR_OF_DAY, cal.get(Calendar.HOUR_OF_DAY) + 12);
		}
		else {
			cal.set(field, 0);
			cal.set(Calendar.HOUR_OF_DAY, cal.get(Calendar.HOUR_OF_DAY) - 12);
		}
		break;
	case Calendar.DAY_OF_YEAR:
		int doy = (cal.get(field) + amount) % daysInYear(cal.get(Calendar.YEAR));
		if (doy == 0) {
			doy = daysInYear(cal.get(Calendar.YEAR));
		}
		cal.set(field, doy);
		break;
	case Calendar.DAY_OF_WEEK:
		int dow = (cal.get(field) + amount) % 7;
		if (dow == 0) {
			dow = 7;
		}
		cal.set(field, dow);
		break;
	case Calendar.HOUR:
		// roll hour of day since the roll function doesn't support rolling
		// the "HOUR"
		//cal.roll(Calendar.HOUR_OF_DAY, amount > 0);

		// In JDK 1.2 and later, roll supports an amount > 1
		cal.roll(Calendar.HOUR_OF_DAY, amount);
		/*
		int hour = cal.get(Calendar.HOUR_OF_DAY);
		if (hour < 12) {
			cal.set(Calendar.AM_PM, 0);
		}
		else if (hour >= 12) {
			hour -= 12;
			cal.set(Calendar.AM_PM, 1);
		}
		if (hour == 0) {
			hour = 12;
		}
		cal.set(Calendar.HOUR, hour);
		*/
		break;
	default:
		// Note that currently we do not support an amount not equal to one
		// since we rely on the Calendar's roll() method where possible
		// cal.roll(field, amount > 0);

		// In JDK 1.2 and later, roll supports an amount > 1
		cal.roll(field, amount);
		break;
	}
}


protected Object addIncrement(Object current)
{
	if (current == null || increment == null) {
        
		return(null);
	}

	int value = ((Integer)increment).intValue();
	if (value == 0) {
		return(null);
	}

    

	Calendar cal = (Calendar) convertToSupported(current);
	cal = (Calendar) cal.clone();
	if (spin_policy == SPIN_SUBFIELD) {
		spinSubField(cal, value);
	}
	else {
		cal.add(increment_field, value);
	}

	return(convertFromSupported(cal, current.getClass()));
}

protected Object subtractIncrement(Object current)
{
	if (current == null || increment == null) {
		return(null);
	}

	int value = ((Integer)increment).intValue();
	if (value == 0) {
		return(null);
	}

    

	Calendar cal = (Calendar) convertToSupported(current);
	cal = (Calendar) cal.clone();
	if (spin_policy == SPIN_SUBFIELD) {
		spinSubField(cal, -value);
	}
	else {
		cal.add(increment_field, -value);
	}
	return(convertFromSupported(cal, current.getClass()));
}

//private void copy_fields(WorkingCalendar target, WorkingCalendar source)
//{
//	for (int i = 0; i < Calendar.FIELD_COUNT; i++) {
//		target.set(i, source.get(i));
//	}
//}

/**
 * Sets the default locale.
 * @see AbstractValidator#setLocale
 * @param locale
 */
public void setLocale(Locale locale)
{
	if (this.locale == locale) {
		return;
	}

	super.setLocale(locale);

	if (tz == null) {
		tz = TimeZone.getDefault();
	}
	// Build new calendar
	cal = new WorkingCalendar(Calendar.getInstance(tz, getLocale()));
//	copy_fields(cal, this.cal);
//	this.cal = cal;

	// Get new symbols
	symbols = new DateFormatSymbols(getLocale());

	// Need to see if we should reset the format
	String format = getFormat();
	if (format == null) {
		return;
	}
	// If this matches the default format, the user has not explicitly set
	// a format.  We need to replace the default format string with a new one
	// for the new locale and use this as the new format
	if (format.equals(default_format) == true) {
		default_format = getDefaultFormat();
		setFormat(default_format);
	}
	else {
		// This seems silly, but we need to recompile the format for the
		// new locale
		setFormat(format);
	}
}

/**
 * Gets the time zone to be used by the validator.
 * @return
 */
public TimeZone getTimeZone()
{
	return(tz);
}

/**
 * Sets the time zone to be used by the validator.
 * @param zone
 */
public void setTimeZone(TimeZone zone)
{
	tz = zone;
	// Handle time zone change on Calendar
	cal = new WorkingCalendar(Calendar.getInstance(tz, locale));
//	copy_fields(cal, this.cal);
//	this.cal = cal;
}

/**
 * Checks if the value is in the valid range.  The default is <code>true</code> for the
 * <code>DateTime</code> validator.  This method is used internally. You should
 * not have to call it.
 * @return
 * @param value
 */
public boolean inRange(Object value)
{
	return(true);
}

/**
 * Adds the specified format to the edit format list.
 * @see #setEditFormats
 * @param edit_format
 */
public void addEditFormat(String edit_format)
{
	edit_formats.addElement(edit_format);
	compiled_edit_formats.addElement(compileFormat(edit_format));
}

/**
 * Converts a date mask to a String mask that can be used with the
 * <code>StringValidator</code>.
 * @return
 */
private	String convertDateFormatToStringMask()
{
    
	StringBuffer mask = new StringBuffer(date_format.length());

	// We also change the form of the Format string so that it useable
	// with mask Input by transmuting ambiguous subfields into biguous ones
	// We also rebuild the date_format from the new compiled_format
	if (compiled_format.size_ambiguous == true) {
		StringBuffer tmp = new StringBuffer(date_format.length() + 10);

		for (int i = 0; i < compiled_format.elements.length; i++) {
			if (compiled_format.elements[i].absolute_replacement == null) {
				// this element is not ambiguous
                
				tmp.append(compiled_format.getChars(i));
				continue;
			}

			compiled_format.elements[i]
				= compiled_format.elements[i].absolute_replacement;
            
			tmp.append(compiled_format.getChars(i));
		}

        
		date_format = tmp.toString();
	}
    

	// loop through compiled format
	for (int i = 0; i < compiled_format.elements.length; i++) {
		if (compiled_format.getSymbol(i) != DATE_LITERAL) {
			String[] strings = compiled_format.getStrings(i);
            int sym = compiled_format.getSymbol(i);
           
			if (strings == null       ||
                sym == MERIDIEM_HOUR  ||
                sym == ZERO_MERIDIEM_HOUR)
			{
				// We know it is a number if it has no strings associated
				// with it and or if it is the MERIDIEM_HOUR which stores the
				// string values for AM and PM

				// Build a numeric field equal to the number of characters
				// in the format
				for (int j = 0; j < compiled_format.getChars(i).length(); j++) {
					mask.append("@");
				}
				compiled_format.setLength(i,
				                          compiled_format.getChars(i).length());
			}
			else {
                int length = 0;
                if (sym == TIMEZONE || sym == ABBREV_TIMEZONE) {
                    String timeZoneID = getTimeZoneString(i);
                    if (timeZoneID != null) {
                        length = timeZoneID.length();
                    }
                    
                }
                else {
                    // Build an alpha field equal to the length of one of
                    // the strings that list the possible values
                    
                    length = strings[0].length();
                }

                if (length == 0) {
                    for (int k = 1; k < strings.length; k++) {
                        length = Math.max(length, strings[k].length());
                        
                   }
                }

				for (int j = 0; j < length; j++) {
					mask.append("A");
				}
				compiled_format.setLength(i, length);
			}
		}
		else {
			// Handle literals
			mask.append(compiled_format.getChars(i));
			compiled_format.setLength(i,
									  compiled_format.getChars(i).length());
		}
	}
    

    
	return(mask.toString());
}

/**
 * Gets the flag that controls whether or not to force the user to input
 * data that exactly matches the format.
 * @see #getFormat
 * @return
 */
public boolean getMaskInput()
{
	return(mask_input);
}

/**
 * Sets the flag that controls whether or not to force the user to input
 * data that exactly matches the format.  When using this flag, be sure
 * that the date format mask is non-ambiguous (for example, 'hh' instead
 * of 'h').
 * @see #setFormat
 * @param mask_input
 */
public void setMaskInput(boolean mask_input)
{
	this.mask_input = mask_input;
	if (mask_input == true) {
		setMask(convertDateFormatToStringMask());
	}
	else {
		setMask("");
	}
}

/**
 * Returns the mask currently being used to display the date/time values.
 * @see #setFormat
 * @return mask currently being used
 */
public String getFormat()
{
	// Return saved format that was set in setFormat(); we know that
	// this will always be set since setFormat() is called in the
	// constructor
	return(original_format);
}


/**
 * Sets the format to use to display the date/time values.
 *
 * The format should be of the form:
 * <table border>
 * <tr><th>Symbol(s)</th>    <th>Meaning</th></tr><tr><td>
 * y	</td><td> Year within the current century (1 or 2 digits)</td></tr><tr><td>
 * yy	</td><td> Year within the current century (2 digits) </td></tr><tr><td>
 * yyyy	</td><td> Year including century (4 digits) </td></tr><tr><td>
 * M	</td><td> numeric month of year (1 or 2 digits)</td></tr><tr><td>
 * MM	</td><td> numeric month of year (2 digits)</td></tr><tr><td>
 * MMM	</td><td> abbreviated month name</td></tr><tr><td>
 * MMMM	</td><td> full month name</td></tr><tr><td>
 * EEEE	</td><td> Day of the Week (full name) </td></tr><tr><td>
 * EE 	</td><td> Day of the Week (abbreviated) </td></tr><tr><td>
 * d	</td><td> numeric day of month (1 or 2 digits)</td></tr><tr><td>
 * dd	</td><td> numeric day of month (2 digits)</td></tr><tr><td>
 * h	</td><td> hour of day (1-12) (1 or 2 digits)</td></tr><tr><td>
 * hh	</td><td> hour of day (1-12) (2 digits)</td></tr><tr><td>
 * H	</td><td> hour of day (0-23) (1 or 2 digits)</td></tr><tr><td>
 * HH	</td><td> hour of day (0-23) (2 digits)</td></tr><tr><td>
 * m	</td><td> minute (1 or 2 digits)</td></tr><tr><td>
 * mm	</td><td> minute (2 digits) </td></tr><tr><td>
 * s	</td><td> second (1 or 2 digits) </td></tr><tr><td>
 * ss	</td><td> second (2 digits) </td></tr><tr><td>
 * a	</td><td> AM/PM representation</td></tr><tr><td>
 * p	</td><td> AM/PM representation</td></tr><tr><td>
 * z    </td><td> time zone abbreviation</td></tr><tr><td>
 * zz   </td><td> time zone abbreviation</td></tr><tr><td>
 * zzzz </td><td> time zone (full name)</td></tr><tr><td>
 * D    </td><td> Day in year (1, 2, or 3 digits)</td></tr><tr><td>
 * DDD  </td><td> Day in year (3 digits)</td></tr><tr><td>
 * \\	</td><td> Next character is a literal</td>
 * </table><p>
 *
 * @param format
 */

public void setFormat(String format)
{
    

	compiled_format = compileFormat(format);
	date_format = format;
	// Save what was set since date_format can be modified if
	// mask_input is true
	original_format = format;

	if (format != null) {
		if (mask_input) {
			// This MUST be called after the call to compileFormat()
			// since it reconstructs the mask from the compiled form
			setMask(convertDateFormatToStringMask());
		}
		else {
			setMask("");
		}
	}
	else {
		setMask("");
	}
}

/**
 * Validates the specified text, and returns a new <code>Object</code> if successful.
 * @param class_type sets which class object the text should be converted to
 * @param text text to be converted to an object
 * @return
 */
public Object parse(Class class_type, String text)
{
	WorkingCalendar newcal;
	boolean built = false;
	Object value = null;

	if (mask_input) {
		Class string_class = null;
		try {
			string_class = Class.forName("java.lang.String");
		}
		catch (ClassNotFoundException e) {
		}

		// Call the string validate method to make sure the validator is
		// adequately masked
		value = super.parse(string_class, text);

		if (value == null) {
			if (getAllowNull() == true) {
				// Handles allow null case. The string validator correctly
				// handles this case so we believe it and simply return
				return(value);
			}
			else {
				throw new FieldParseException("Null text is invalid");
			}
		}
	}

	// Now do the date parsing
	if (text.equalsIgnoreCase("now") || text.equalsIgnoreCase("today")) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(new Date());
		newcal = new WorkingCalendar(cal);
		built = true;
	}
	else if (mask_input == true) {
		// If mask input is true then we need only parse against the "format"
		newcal = new WorkingCalendar();;
		built = parseString(text, compiled_format.elements, newcal);
	}
	else {
		// Parse via the mask list
		newcal = new WorkingCalendar();
		for (int i = 0; i < compiled_edit_formats.size(); i++) {
            
			built = parseString(text,
				((DateFormat) compiled_edit_formats.elementAt(i)).elements,
								newcal);
			if (built) {
				break;
			}
		}
	}

	if (built) {
		value = convertFromSupported(newcal.toCalendar(), class_type);
		return(value);
	}
	else {
		throw new FieldParseException("cannot construct date/time from text");
	}
}

private boolean setDayOfWeek(WorkingCalendar cal, int year, int month, int day)
{
	int weekday = dayOfWeek(year, month, day);
    
	if (cal.isSet(Calendar.DAY_OF_WEEK)) {
		if (weekday != cal.get(Calendar.DAY_OF_WEEK)) {
			// Set date does not match other set fields
            
			return(false);
		}
	}
	else {
		cal.set(Calendar.DAY_OF_WEEK, weekday);
	}
	return(true);
}

/**
 * Creates a new instance of the object with the same value.
 * Since the <code>clone()</code> method is protected in the <code>Object</code> class,
 * we cannot actually clone a generic object without knowing what it is; therefore
 * the validator must provide this routine to do the copying for JClass Field.
 * <br><B>Note:</b> Objects that are inherently immutable do not need to be copied, and
 * hence return the passed in value as the implementation.
 * @return
 * @param value
 */
public Object copyValue(Object value)
{
	if (value == null) {
		return(null);
	}
	Class class_type = value.getClass();
	Calendar new_val = (Calendar)convertToSupported(value);
	if (new_val == value) {
		return new_val.clone();
	}
	return(convertFromSupported(new_val, class_type));
}

/**
 * Validates the given object.
 * @return
 * @param obj
 */
public boolean validate(Object obj)
{
	if (obj == null) {
		return(internal_validate(null));
	}
	else {
		Calendar cal = (Calendar)(convertToSupported(obj));
		return internal_validate(new WorkingCalendar(cal));
	}
}

protected boolean internal_validate(WorkingCalendar cal)
{
    

	if (cal == null) {
		if (allow_null == true) {
            
			return(true);
		}
		else {
            
			return(false);
		}
	}

	// First we fill in all unset fields since we can't properly validate
	// unless we know what they all are.  To that end we get a calendar
	// that is set to right "now"; taking that as our default date and time
	Calendar tmpcal = Calendar.getInstance(tz, locale);
	tmpcal.setTime(new Date());
	WorkingCalendar tmp = new WorkingCalendar(tmpcal);

	int year;
	// If the year is not set then set it to the current year
	if (cal.isSet(Calendar.YEAR)) {
		year = cal.get(Calendar.YEAR);
	}
	else {
		year = tmp.get(Calendar.YEAR);
		cal.set(Calendar.YEAR, year);
	}

	if (cal.isSet(Calendar.WEEK_OF_YEAR) && cal.isSet(Calendar.DAY_OF_WEEK)) {
		// calculate day of the year from this
		// calculate day of the year from this
		int month = cal.get(Calendar.MONTH);
		int thisyear = cal.get(Calendar.YEAR);
		int day_of_year = 0;
		for (int i = 0; i < month; i++) {
			// only loop to the month before the current month
			day_of_year += daysInMonth(i, thisyear);
		}
		day_of_year += cal.get(Calendar.DAY_OF_MONTH);
		if (cal.isSet(Calendar.DAY_OF_YEAR)) {
			if (day_of_year != cal.get(Calendar.DAY_OF_YEAR)) {
				// Calculated date does not match existing one so generate
				// an error
                
				return(false);
			}
		}
		else {
            
			cal.set(Calendar.DAY_OF_YEAR, day_of_year);
		}
	}

	if (cal.isSet(Calendar.DAY_OF_YEAR)) {
		//
		// Calculate the rest of the "Date" fields from the Day of the Year
		//

		// Handle Month
		int month = monthOfYear(year, cal.get(Calendar.DAY_OF_YEAR));
		if (cal.isSet(Calendar.MONTH)) {
			if (month != cal.get(Calendar.MONTH)) {
				// Set month does not match other set fields
                
				return(false);
			}
		}
		else {
			cal.set(Calendar.MONTH, month);
		}

		// Handle Day of Month
		int day = dayOfMonthOfYear(year, cal.get(Calendar.DAY_OF_YEAR));
		if (cal.isSet(Calendar.DAY_OF_MONTH)) {
			if (day != cal.get(Calendar.DAY_OF_MONTH)) {
				// Set date does not match other set fields
                
				return(false);
			}
		}
		else {
			cal.set(Calendar.DAY_OF_MONTH, day);
		}

		// Handle Day of Week
		if (setDayOfWeek(cal, cal.get(Calendar.YEAR), cal.get(Calendar.MONTH),
						 cal.get(Calendar.DAY_OF_MONTH)) == false) {
            
			return(false);
		}

		// The following two fields should not need additional checking
		// since if they have been set they have been checked above

		// Handle Week of Year
		if (!cal.isSet(Calendar.WEEK_OF_YEAR)) {
			cal.set(Calendar.WEEK_OF_YEAR,
					weekOfYear(year, cal.get(Calendar.DAY_OF_YEAR)));
		}
		// Handle Week of Month
		if (!cal.isSet(Calendar.WEEK_OF_MONTH)) {
			cal.set(Calendar.WEEK_OF_MONTH,
				weekOfMonth(year, cal.get(Calendar.MONTH),
							cal.get(Calendar.DAY_OF_MONTH)));
		}
	}
	else if (cal.isSet(Calendar.MONTH)) {
		int month = cal.get(Calendar.MONTH);
		if (cal.isSet(Calendar.DAY_OF_MONTH)) {
			//
			// Calculate the rest of the "Date" fields from the Month
			// and the Day of the Month
			//
			int day = cal.get(Calendar.DAY_OF_MONTH);

			// build up DAY_OF_YEAR which we know is not set since
			// we wouldn't be in this code block if it was
            
			cal.set(Calendar.DAY_OF_YEAR, dayOfYear(year, month, day));

			// Handle Day of Week
			if (setDayOfWeek(cal, year, month, day) == false) {
                
				return(false);
			}

			// Handle Week of Year; We know this has not been set
			cal.set(Calendar.WEEK_OF_YEAR,
					weekOfYear(year, cal.get(Calendar.DAY_OF_YEAR)));

			// Handle Week of Month
			cal.set(Calendar.WEEK_OF_MONTH, weekOfMonth(year, month, day));

		}
		else if (cal.isSet(Calendar.WEEK_OF_MONTH) && cal.isSet(Calendar.DAY_OF_WEEK)) {
			//
			// Calculate the rest of the "Date" fields from the Month
			// and the Week of the Month, and the Day of the Week
			//
			// We don't currently handle this case
			//
			throw new IllegalStateException("Validation from only " +
                                            "WeekOfMonth and DayOfWeek not " +
                                            "currently supported");
		}
	}

	//
	// Rationalize HOUR and HOUR_OF_DAY fields
	//
	if (cal.isSet(Calendar.HOUR) && cal.isSet(Calendar.HOUR_OF_DAY)
		&& cal.isSet(Calendar.AM_PM)) {
		// see if hour matches hour of day
		if ((cal.get(Calendar.HOUR) + (12 * cal.get(Calendar.AM_PM)))
			!= cal.get(Calendar.HOUR_OF_DAY)) {
			// Error the pair do not match
            
			return(false);
		}
	}
	else if (cal.isSet(Calendar.HOUR) && !cal.isSet(Calendar.HOUR_OF_DAY)) {
		if (cal.isSet(Calendar.AM_PM) == false) {
			// assume the beginning of the day
			if (cal.get(Calendar.HOUR) == 12) {
				cal.set(Calendar.AM_PM, Calendar.PM);
			}
			else {
				cal.set(Calendar.AM_PM, Calendar.AM);
			}
		}
		setHourOfDayFromHour(cal);
	}
	else if (cal.isSet(Calendar.HOUR_OF_DAY)) {
		setHourFromHourOfDay(cal);
	}

	// Fill in the "Time" fields.  If any of the time fields are set we
	// set any unset ones to zero.  If they are all unset then we set
	// them to the current time
	if (cal.isSet(Calendar.HOUR_OF_DAY) || cal.isSet(Calendar.MINUTE) ||
		cal.isSet(Calendar.SECOND) || cal.isSet(Calendar.MILLISECOND))
	{
		if (!cal.isSet(Calendar.HOUR_OF_DAY)) {
			// this implies that HOUR and AM_PM are also not properly set
			// because of the rationalization that goes on above
			cal.set(Calendar.HOUR_OF_DAY, 0);
			// this sets HOUR _and_ AM_PM
			setHourFromHourOfDay(cal);
		}
		if (!cal.isSet(Calendar.MINUTE)) {
			cal.set(Calendar.MINUTE, 0);
		}
		if (!cal.isSet(Calendar.SECOND)) {
			cal.set(Calendar.SECOND, 0);
		}
		if (!cal.isSet(Calendar.MILLISECOND)) {
			cal.set(Calendar.MILLISECOND, 0);
		}
	}
	else {
		// all time fields are unset so set them from the tmp (current) time
		cal.set(Calendar.HOUR_OF_DAY, tmp.get(Calendar.HOUR_OF_DAY));
		// this sets HOUR _and_ AM_PM
		setHourFromHourOfDay(cal);
		cal.set(Calendar.MINUTE,      tmp.get(Calendar.MINUTE));
		cal.set(Calendar.SECOND,      tmp.get(Calendar.SECOND));
		cal.set(Calendar.MILLISECOND, tmp.get(Calendar.MILLISECOND));
	}

	//
	//	Sanity check; fill in the rest of fields that have not been filled in
	//
	for (int i = 0; i < Calendar.FIELD_COUNT; i++) {
		if (!cal.isSet(i)) {
			// Fill in the unset fields from the tmp (current time)
			// Calendar that we built earlier
			cal.set(i, tmp.get(i));
		}
	}

	//
	// Now actually perform the VALIDATION
	//
	// We know that all related fields are in sync so we only need to
	// validate some of the to verify the Calendat
	//

	// Validate the month
	int month = cal.get(Calendar.MONTH);
	if (!domain_check(month /* zero based */ + 1, NUM_MONTH)) {
        
		return(false);
	}

	// Implementation Note:
	// We currently do not handle validation if WeekOfYear or WeekOfMonth
	// If we did then we would handle it here

	// Validate Day of Year
	int doy = cal.get(Calendar.DAY_OF_YEAR);
	if (doy < 1 || doy > daysInYear(year)) {
		// Out of valid range
        
		return(false);
	}

	// Validate Day of Month
	int day = cal.get(Calendar.DAY_OF_MONTH);
	if (day < 1 || day > daysInMonth(month, year)) {
		// Out of valid range
        
		return(false);
	}

	// Validate Hour of Day
	if (!domain_check(cal.get(Calendar.HOUR_OF_DAY), MILITARY_HOUR)) {
		// Out of valid range
        
		return(false);
	}

	// Validate Minute
	if (!domain_check(cal.get(Calendar.MINUTE), MINUTE)) {
		// Out of valid range
        
		return(false);
	}

	// Validate Second
	if (!domain_check(cal.get(Calendar.SECOND), SECOND)) {
		// Out of valid range
        
		return(false);
	}

	// Validate Millisecond
	if (!domain_check(cal.get(Calendar.MILLISECOND), MILLISECOND)) {
		// Out of valid range
        
		return(false);
	}

	//
	// If we made it to here then everything is okay
	//
    
	return(true);
}


private void setHourFromHourOfDay(WorkingCalendar cal)
{
	// AM/PM is not set so derive it from HOUR_OF_DAY
	// Note this is a modular division so that if the hour is
	// > 12 we set the field to one if it is less than we set
	// set it to zero.
	int hour = cal.get(Calendar.HOUR_OF_DAY);
	cal.set(Calendar.AM_PM, hour / 12);
	hour %= 12;
	if (hour == 0) {
		hour = 12;
	}
	cal.set(Calendar.HOUR, hour);
}

private void setHourOfDayFromHour(WorkingCalendar cal)
{
	// NOTE cal.get(cal.AM_PM) returns 0 or 1 so in the below statement we are
	// either adding 0 or 12
	int hour_of_day = cal.get(Calendar.HOUR) + (12 * cal.get(Calendar.AM_PM));

	if (hour_of_day == 12) {
		// 12 am + 0 above == 12 so we reset the "hour of day" to 0
		hour_of_day = 0;
	}
	else if (hour_of_day == 24) {
		hour_of_day = 12;
	}
	cal.set(Calendar.HOUR_OF_DAY, hour_of_day);
}

/**
 * Formats an integer to the number of digits specified.
 * @return
 * @param value
 * @param digits
 */
private String formatDigits(int value, int digits)
{
	String valuestr = Integer.toString(value);
	if (valuestr.length() >= digits) {
		return(valuestr);
	}

	// build up leading 0's
	StringBuffer work = new StringBuffer(digits);
	for (int i = 0; i < (digits - valuestr.length()); i++) {
		work.append("0");
	}
	work.append(valuestr);
	return(work.toString());

}

/**
 * Generates a String representation of the given object.
 * @return
 * @param obj
 */
public String format(Object obj)
{
    

	if (obj == null) {
		// if the object is null, we should still format the field with
		// the mask if there is one
		return(super.format(obj));
	}

	if (isClassSupported(obj.getClass())) {
		obj = convertToSupported(obj);
	}
	else {
		return("");
	}

	Calendar cal = (Calendar) obj;
	StringBuffer datestr = new StringBuffer(30);

    
    
    

	// Go through the mask
	int field_value;
	for (int i = 0; i < compiled_format.elements.length; i++) {
		switch(compiled_format.getSymbol(i)) {
		case MERIDIEM_HOUR:	/* 0-12 hour */
			field_value = cal.get(compiled_format.getField(i));
			if (field_value == 0) {
				field_value = 12;
			}
			datestr.append(Integer.toString(field_value));
			break;
		case MILITARY_HOUR: /* 0-23 hour */
		case MINUTE: /* 0-59 minute */
		case SECOND: /* 0-59 second */
		case NUM_DAY:
			field_value = cal.get(compiled_format.getField(i));
			datestr.append(Integer.toString(field_value));
			break;
		case NUM_MONTH:
			field_value = cal.get(compiled_format.getField(i)) + 1;
			datestr.append(Integer.toString(field_value));
			break;
		case DAY_OF_YEAR:
			/* the tm struct day_of_year domain is 0-365 */
			/* But usually we like to use 1-366. So add 1.*/
			field_value = cal.get(compiled_format.getField(i)) + 1;
			datestr.append(Integer.toString(field_value));
			break;
		case LONG_YEAR:
			/* Years since 1900 */
			field_value = cal.get(compiled_format.getField(i));
			datestr.append(Integer.toString(field_value));
			break;
		case ZERO_MERIDIEM_HOUR:
			field_value = cal.get(compiled_format.getField(i));
			if (field_value == 0) {
				field_value = 12;
			}
			datestr.append(formatDigits(field_value, 2));
			break;
		case ZERO_MILITARY_HOUR: /* 0-23 hour */
		case ZERO_MINUTE: /* 0-59 minute */
		case ZERO_SECOND: /* 0-59 second */
		case ZERO_NUM_DAY:
            int curField = compiled_format.getField(i);
            field_value = cal.get(curField);
            
            
			datestr.append(formatDigits(field_value, 2));
			break;
		case ZERO_NUM_MONTH:
			field_value = cal.get(compiled_format.getField(i)) + 1;
			datestr.append(formatDigits(field_value, 2));
			break;
		case SHORT_YEAR:
			field_value = cal.get(compiled_format.getField(i));
			field_value %= 100;
			datestr.append(formatDigits(field_value, 2));
			break;
		case ZERO_DAY_OF_YEAR:
			/* the tm struct day_of_year domain is 0-365 */
			/* But usually we like to use 1-366. So add 1.*/
			field_value = cal.get(compiled_format.getField(i)) + 1;
			datestr.append(formatDigits(field_value, 3));
			break;
		case MERIDIEM: /* AM PM */
		case ABBREV_ALPHA_DAY:
		case ALPHA_DAY:
		case ABBREV_ALPHA_MONTH:
		case ALPHA_MONTH:
			field_value = cal.get(compiled_format.getField(i));
			datestr.append(compiled_format.getString(i, field_value));
            
			break;
		case TIMEZONE:
		case ABBREV_TIMEZONE:
			String timeZoneID = getTimeZoneString(i);
            
			datestr.append(timeZoneID);
			break;
		case DATE_LITERAL:
			datestr.append(compiled_format.getChars(i));
			break;
		}
	}

	if (case_policy != AS_IS) {
		for (int i = 0; i < datestr.length(); i++) {
			 datestr.setCharAt(i, convertCase(datestr.charAt(i)));
		}
	}

	return(datestr.toString());
}

private int getZoneIndex()
{
	String zones[][] = symbols.getZoneStrings();

	for (int i = 0; i < zones.length; i++) {
		if (zones[i][ZONE_ID].equals(tz.getID())) {
			return(i);
		}
	}
	return(-1);
}

private String getTimeZoneString(int index)
{
    String timeZoneID = null;
    int zone_index = getZoneIndex();
    if (zone_index == -1) {
        // Nothing found in the lookup table, just use the original ID
        timeZoneID = tz.getID();
    }
    else {
        timeZoneID = compiled_format.getString(index, zone_index);
    }
    return(timeZoneID);
}

private String[] getZoneStrings(int field)
{
	String zones[][] = symbols.getZoneStrings();
	String strings[] = new String[zones.length];

	for (int i = 0; i < zones.length; i++) {
		strings[i] = zones[i][field];
	}

	return(strings);
}

private String[] getShortZoneStrings()
{
	Calendar newcal = cal.toCalendar();
	if (tz.inDaylightTime(newcal.getTime())) {
		return(getZoneStrings(ZONE_DAYLIGHT_SHORT));
	}
	else {
		return(getZoneStrings(ZONE_STANDARD_SHORT));
	}
}

private String[] getLongZoneStrings()
{
	Calendar newcal = cal.toCalendar();
	if (tz.inDaylightTime(newcal.getTime())) {
		return(getZoneStrings(ZONE_DAYLIGHT_LONG));
	}
	else {
		return(getZoneStrings(ZONE_STANDARD_LONG));
	}
}

//private String[] getZoneCities()
//{
//	return(getZoneStrings(ZONE_CITY_NAME));
//}

private DateFormat compileFormat(String maskString)
{
	Vector output = new Vector();
	boolean ambiguous = false;
	int	i, j;
	int mask_length;

    

	if (maskString == null || maskString.length() == 0) {
        
		return(null);
	}
	mask_length = maskString.length();

	/* Generate the mask structure string */
	i = 0;
	while (i < maskString.length()) {
        
		DateMask datemask = new DateMask();
		switch (maskString.charAt(i)) {
		case 'd':				/* Day of Month */
			j = 1;				/* Count how many d's we got */
			while ((i + j) != mask_length &&
				   j < 2 && (maskString.charAt(i + j) == 'd'))
			{
				j++;
			}

			datemask.chars = maskString.substring(i, i + j);
			i += j;
			datemask.field = Calendar.DAY_OF_MONTH;
			switch (j) {
			case 1:
				/* Day of the month e.g. 1-31 */
				datemask.symbol = NUM_DAY;

				ambiguous = true;
				datemask.absolute_replacement = new DateMask();
				datemask.absolute_replacement.symbol = ZERO_NUM_DAY;
				datemask.absolute_replacement.field = Calendar.DAY_OF_MONTH;
				datemask.absolute_replacement.chars = "dd";
				break;
			case 2:
				/* Zero padded day of the month e.g. 01-31 */
				datemask.symbol = ZERO_NUM_DAY;
				break;
			default:
				/* Error format */
				throw new IllegalArgumentException("error parsing day of month in \"" +
				                                   maskString + "\"");
			}
			break;
		case 'E':				/* Date of Week */
			j = 1;				/* Count how many E's we got */
			while ((i + j) != mask_length &&
				   j < 4 && (maskString.charAt(i + j) == 'E'))
			{
				j++;
			}

			datemask.chars = maskString.substring(i, i + j);
			i += j;
			datemask.field = Calendar.DAY_OF_WEEK;
			switch (j) {
			case 1:
			case 2:
				/* Abbreviated day of the week e.g. Mon */
				datemask.symbol = ABBREV_ALPHA_DAY;
				datemask.strings = symbols.getShortWeekdays();
				break;
			case 3:
			case 4:
				/* Day of the week e.g. Monday */
				datemask.symbol = ALPHA_DAY;
				datemask.strings = symbols.getWeekdays();

				ambiguous = true;
				datemask.absolute_replacement = new DateMask();
				datemask.absolute_replacement.symbol = ABBREV_ALPHA_DAY;
				datemask.absolute_replacement.field = Calendar.DAY_OF_WEEK;
				datemask.absolute_replacement.strings
					= symbols.getShortWeekdays();
				datemask.absolute_replacement.chars = "EE";
				break;
			default:
				/* Error format */
				throw new IllegalArgumentException("error parsing week in \"" +
				                                   maskString + "\"");
			}
			break;
		case 'M':				/* Numeric month of year */
			j = 1;				/* Count how many M's we got */
			while ((i + j) != mask_length &&
				   j < 4 && maskString.charAt(i + j) == 'M')
			{
				j++;
			}

			datemask.chars = maskString.substring(i, i + j);
			i += j;
			datemask.field = Calendar.MONTH;
			switch (j) {
			case 1:
				/* month number, e.g. 1-12 */
				datemask.symbol = NUM_MONTH;

				ambiguous = true;
				datemask.absolute_replacement = new DateMask();
				datemask.absolute_replacement.symbol = ZERO_NUM_MONTH;
				datemask.absolute_replacement.field = Calendar.MONTH;
				datemask.absolute_replacement.chars = "MM";
			break;
			case 2:
				/* zero padded month number, e.g. 01-12 */
				datemask.symbol = ZERO_NUM_MONTH;
				break;
			case 3:
				/* abbreviated name of the month, e.g. Nov */
				datemask.symbol = ABBREV_ALPHA_MONTH;
				datemask.strings = symbols.getShortMonths();
				break;
			case 4:
				/* name of the month, e.g. November */
				datemask.symbol = ALPHA_MONTH;
				datemask.strings = symbols.getMonths();

				ambiguous = true;
				datemask.absolute_replacement = new DateMask();
				datemask.absolute_replacement.symbol = ABBREV_ALPHA_MONTH;
				datemask.absolute_replacement.field = Calendar.MONTH;
				datemask.absolute_replacement.strings = symbols.getShortMonths();
				datemask.absolute_replacement.chars = "MMM";
				break;
			default:
				/* Error format */
				throw new IllegalArgumentException("error parsing month in \"" +
				                                   maskString + "\"");
			}
			break;
		case 'Y':				/* year */
		case 'y':				/* year */
			// Count how many Y's we have
			for (j = 1;
				 (i + j) != mask_length && j < 4 &&
					 ((i + j) < maskString.length())
					 && (maskString.charAt(i + j) == 'Y'
						 || maskString.charAt(i + j) == 'y'); j++);

			datemask.field = Calendar.YEAR;
			datemask.chars = maskString.substring(i, i + j);
			i += j;
			switch (j) {
			case 1:
				ambiguous = true;
				datemask.absolute_replacement = new DateMask();
				datemask.absolute_replacement.symbol = SHORT_YEAR;
				datemask.absolute_replacement.field = Calendar.YEAR;
				datemask.absolute_replacement.chars = "YY";
			case 2:
				/* last two digits of the year, e.g. 94 */
				datemask.symbol = SHORT_YEAR;
				break;
			case 3:
				ambiguous = true;
				datemask.absolute_replacement = new DateMask();
				datemask.absolute_replacement.symbol = LONG_YEAR;
				datemask.absolute_replacement.field = Calendar.YEAR;
				datemask.absolute_replacement.chars = "YYYY";
			case 4:
				/* all four digits of year, e.g. 1995 */
				datemask.symbol = LONG_YEAR;
				break;
			default:
				/* Error format */
				throw new IllegalArgumentException("error parsing year in \"" +
				                                   maskString + "\"");
			}
			break;
		case 'h':				/* MERIDIEM_HOUR */
			j = 1;				/* Count how many h's we got */
			while ((i + j) != mask_length && j < 2 &&
				   (maskString.charAt(i + j) == 'h'))
			{
				j++;
			}

			datemask.field = Calendar.HOUR;
			datemask.strings = symbols.getAmPmStrings();
			datemask.chars = maskString.substring(i, i + j);
			i += j;

			switch (j) {
			case 1:
				/* this is the 0-12 hour */
				datemask.symbol = MERIDIEM_HOUR;

				ambiguous = true;
				datemask.absolute_replacement = new DateMask();
				datemask.absolute_replacement.symbol = ZERO_MERIDIEM_HOUR;
				datemask.absolute_replacement.field = Calendar.HOUR;
				datemask.absolute_replacement.chars = "hh";
			break;
			case 2:
				/* this is a zero padded 0-12 hour */
				datemask.symbol = ZERO_MERIDIEM_HOUR;
				break;
			default:
				/* Error format */
				throw new IllegalArgumentException("error parsing hour in \"" +
				                                   maskString + "\"");
			}
			break;
		case 'H':				/* MILITARY HOUR */
			j = 1;				/* Count how many H's we got */
			while ((i + j) != mask_length && j < 2 &&
				   (maskString.charAt(i + j) == 'H'))
			{
				j++;
			}

			datemask.field = Calendar.HOUR_OF_DAY;
			datemask.chars = maskString.substring(i, i + j);
			i += j;

			switch (j) {
			case 1:
				/* this is the 0-23 hour */
				datemask.symbol = MILITARY_HOUR;

				ambiguous = true;
				datemask.absolute_replacement = new DateMask();
				datemask.absolute_replacement.symbol = ZERO_MILITARY_HOUR;
				datemask.absolute_replacement.field = Calendar.HOUR_OF_DAY;
				datemask.absolute_replacement.chars = "HH";
				break;
			case 2:
				/* this is a zero padded 0-23 hour */
				datemask.symbol = ZERO_MILITARY_HOUR;
				break;
			default:
				/* Error format */
				throw new IllegalArgumentException("error parsing hour in \"" +
				                                   maskString + "\"");
			}
			break;
		case 'm':				/* MINUTE */
			j = 1;				/* Count how many m's we got */
			while ((i + j) != mask_length && j < 2 &&
				   (maskString.charAt(i + j) == 'm'))
			{
				j++;
			}

			datemask.field = Calendar.MINUTE;
			datemask.chars = maskString.substring(i, i + j);
			i += j;

			switch (j) {
			case 1:
				/* standard earth minutes */
				datemask.symbol = MINUTE;

				ambiguous = true;
				datemask.absolute_replacement = new DateMask();
				datemask.absolute_replacement.symbol = ZERO_MINUTE;
				datemask.absolute_replacement.field = Calendar.MINUTE;
				datemask.absolute_replacement.chars = "mm";
				break;
			case 2:
				/* zero padded minute */
				datemask.symbol = ZERO_MINUTE;
				break;
			default:
				/* Error format */
				throw new IllegalArgumentException("error parsing minute in \"" +
				                                   maskString + "\"");
			}
			break;
		case 's':				/* standard earth second */
			j = 1;				/* Count how many s's we got */
			while ((i + j) != mask_length && j < 2 &&
				   ((i + j) < maskString.length()) &&
				   (maskString.charAt(i + j) == 's'))
			{
				j++;
			}

			datemask.field = Calendar.SECOND;
			datemask.chars = maskString.substring(i, i + j);
			i += j;
			switch (j) {
			case 1:
				/* Standard earth second */
				datemask.symbol = SECOND;

				ambiguous = true;
				datemask.absolute_replacement = new DateMask();
				datemask.absolute_replacement.symbol = ZERO_SECOND;
				datemask.absolute_replacement.field = Calendar.SECOND;
				datemask.absolute_replacement.chars = "ss";
				break;
			case 2:
				/* Zero padded second */
				datemask.symbol = ZERO_SECOND;
				break;
			default:
				/* Error format */
				throw new IllegalArgumentException("error parsing second in \"" +
				                                   maskString + "\"");
			}
			break;
		case 'a':
		case 'p':	// MERIDIEM ie AM or PM, meridiem is latin for noon
			datemask.symbol = MERIDIEM;
			datemask.field = Calendar.AM_PM;
			datemask.strings = symbols.getAmPmStrings();
			datemask.chars = "" + maskString.charAt(i);
			i++;
			break;
		case 'z':	/* local time zone, for us it is EST */
			j = 1;				/* Count how many z's we got */
			while ((i + j) != mask_length && j < 4 &&
				   ((i + j) < maskString.length()) &&
				   maskString.charAt(i + j) == 'z')
			{
				j++;
			}

			datemask.symbol = TIMEZONE;
			datemask.field = -1;
			datemask.chars = maskString.substring(i, i + j);
			i += j;
            
			switch (j) {
			case 1:
			case 2:
				/* Abbreviated Time Zone */
				datemask.symbol = ABBREV_TIMEZONE;
				datemask.strings = getShortZoneStrings();
				break;
			case 4:
				/* Full Time Zone */
				datemask.symbol = TIMEZONE;
				datemask.strings = getLongZoneStrings();
				ambiguous = true;
				datemask.absolute_replacement = new DateMask();
				datemask.absolute_replacement.symbol = ABBREV_TIMEZONE;
				datemask.absolute_replacement.field = -1;
				datemask.absolute_replacement.strings
					= getShortZoneStrings();
				datemask.absolute_replacement.chars = "zz";
				break;
			default:
				/* Error format */
				throw new IllegalArgumentException("error parsing timezone " +
                                                   "in \"" + maskString + "\"");
			}
			break;
		case 'D': 	/* the number of year 001-366 */
			j = 1;				/* Count how many D's we got */
			while ((i + j) != mask_length && j < 3 &&
				   (maskString.charAt(i + j) == 'D'))
			{
				j++;
			}

			datemask.field = Calendar.DAY_OF_YEAR;
			datemask.chars = maskString.substring(i, i + j);
			i += j;
			switch (j) {
			case 1:
				/* day of year */
				datemask.symbol = DAY_OF_YEAR;
				ambiguous = true;
				datemask.absolute_replacement = new DateMask();
				datemask.absolute_replacement.symbol = ZERO_DAY_OF_YEAR;
				datemask.absolute_replacement.field = Calendar.DAY_OF_YEAR;
				datemask.absolute_replacement.chars = "DDD";
				break;
			case 3:
				/* zero padded day of year */
				datemask.symbol = ZERO_DAY_OF_YEAR;
				break;
			default:
				/* Error format */
				throw new IllegalArgumentException("error parsing day of year in \"" +
				                                   maskString + "\"");
			}
			break;
		case '\\':				/* any literal token, e.g. :  */
			j = 1;
			i++;
			if (maskString.charAt(i) != '\0') {
				datemask.symbol = DATE_LITERAL;
				datemask.chars = "" + maskString.charAt(i);
				i++;
			}
			break;
		default:	/* any literal token, e.g. : */
			datemask.symbol = DATE_LITERAL;
			datemask.chars = "" + maskString.charAt(i);
			i++;
			break;
		}
		output.addElement(datemask);
	}

	DateMask[] return_array = new DateMask[output.size()];
	output.copyInto(return_array);
	DateFormat format = new DateFormat();
	format.elements = return_array;
	format.size_ambiguous = ambiguous;
	return(format);
}

private static boolean domain_check(int value, int symbol)
{
	switch(symbol) {
	case MERIDIEM_HOUR:
	case ZERO_MERIDIEM_HOUR:
		return (value >= 0 && value <= 12);
	case MILITARY_HOUR:
	case ZERO_MILITARY_HOUR:
		return (value >= 0 && value <= 23);
	case MINUTE:
	case ZERO_MINUTE:
	case SECOND:
	case ZERO_SECOND:
		return (value >= 0 && value <= 59);
	case NUM_DAY:
	case ZERO_NUM_DAY:
		return (value >= 1 && value <= 31);
	case NUM_MONTH:
	case ZERO_NUM_MONTH:
		return (value >= 1 && value <= 12);
	case LONG_YEAR:
	case SHORT_YEAR:
		return (value >= 0 && value <= 9999);
	case DAY_OF_YEAR:
	case ZERO_DAY_OF_YEAR:
		return (value >= 0 && value <= 365);
	case MILLISECOND:
		return (value >= 0 && value <= 999);
	case DATE_LITERAL:
	case MERIDIEM:
	case TIMEZONE:
	case ABBREV_ALPHA_DAY:
	case ALPHA_DAY:
	case ABBREV_ALPHA_MONTH:
	case ALPHA_MONTH:
		break;
	}
	return (false);
}

/**
 * Searches for a pattern in a String array. If only one match is found, returns
 * the index in the String array; otherwise, <code>NOTFOUND</code> is returned.
 * @return index in the String array if only one match is found; otherwise, <code>NOTFOUND</code>
 * @param pattern
 * @param string_array
 */
private int patternSearch(String pattern, String string_array[])
{
    

	if (pattern == null || pattern.length() == 0) {
		return(NOTFOUND);
	}

    boolean found = false;
    int result = NOTFOUND;
	for (int i = 0; i < string_array.length; i++) {
		StringBuffer str = new StringBuffer(string_array[i]);

		/* Truncate string to same length as pattern */
        int pattern_len = pattern.length();
		if (pattern_len < str.length()) {
			str.setLength(pattern_len);
		}

		/* The same as pattern */
		if (str.toString().equalsIgnoreCase(pattern)) {
			if (found) {
				return (NOTFOUND);		/* More than one match */
			}
			else {
				found = true;
				result = i;
				break;
			}
		}
	}

	return(found ? result : NOTFOUND);
}

private int parseEnum(WorkingCalendar date, String input, int j,
					  int symbol, int field, String enums[])
{
    

	if (j >= input.length()) {
		return(PARSE_ERROR);
	}

	int i, k;

	// Calculate the maximum length of the stuff we're comparing
	int max = 0;
	for (i = 0; i < enums.length; i++) {
		max = Math.max(max, enums[i].length());
	}

	// We try for a partial match first and get more complete
	int last_found = NOTFOUND;
	int last_found_length = 0;
	for (k = j + 1; (k - j) <= max; k++) {
		// Look for a match between the input line and the enum string
		int match_length = Math.min(k, input.length());
		i = patternSearch(input.substring(j, match_length), enums);

		if (i != NOTFOUND) {
			last_found = i;
			last_found_length = match_length;
            
		}
	}

	if (last_found == NOTFOUND) {
		return(PARSE_ERROR);
	}

	// If value is being set twice and does not agree with the previously
	// set value then return an error
	if (date.isSet(field) && date.get(field) != i) {
		return(PARSE_ERROR);
	}

	// Set the field
	date.set(field, last_found);
    
    

	return(last_found_length);
}

private int parseDigitField(WorkingCalendar date, String input, int j,
						    int symbol, int field, int max_digits)
{
    

	if (j >= input.length() || !Character.isDigit(input.charAt(j))) {
        
		return(PARSE_ERROR);
	}

	// Count the number of consecutive digits up to the max number of
	// expected digits
	int k;
	for (k = j;
		 k < input.length()                 &&
		 Character.isDigit(input.charAt(k)) &&
		 (k - j) < max_digits;
		 k++);

	int value = Integer.parseInt(input.substring(j, k));

	// Check the range
	if (!domain_check(value, symbol)) {
        
		return(PARSE_ERROR);
	}

	// If value is being set twice and does not agree with the previously
	// set value then return an error
	if (date.isSet(field) && date.get(field) != value) {
        
		return(PARSE_ERROR);
	}

	// Set the field
	date.set(field, value);

	return(k);
}

private int parseTimeZone(WorkingCalendar date, String input, int j)
{
	// Get the locale specific names
	String zones[][] = symbols.getZoneStrings();
	String name = "", zone_id = null;

    

	for (int i = 0; i < zones.length; i++) {
		int name_length;

		// Compare against standard time symbol
		name = zones[i][ZONE_STANDARD_SHORT];
		name_length = Math.min(input.length(), j + name.length());
		if (input.substring(j, name_length).equalsIgnoreCase(name)) {
			zone_id = zones[i][ZONE_ID];
			break;
		}

		// Compare against Daylight saving time long symbol
		name = zones[i][ZONE_DAYLIGHT_LONG];
		name_length = Math.min(input.length(), j + name.length());
		if (input.substring(j, name_length).equalsIgnoreCase(name)) {
			zone_id = zones[i][ZONE_ID];
			break;
		}

		// Compare against Daylight saving time short symbol
		name = zones[i][ZONE_DAYLIGHT_SHORT];
		name_length = Math.min(input.length(), j + name.length());
		if (input.substring(j, name_length).equalsIgnoreCase(name)) {
			zone_id = zones[i][ZONE_ID];
			break;
		}
	}

	if (zone_id == null) {
		// we failed to match within the locale so now we just compare
		// against all possible time zone ids
		String zone_ids[] = tz.getAvailableIDs();
		for (int i = 0; i < zone_ids.length; i++) {
			name = zone_ids[i];
			int name_length = Math.min(input.length(), j + name.length());
			if (input.substring(j, name_length).equalsIgnoreCase(name)) {
				zone_id = zone_ids[i];
				break;
			}
		}
	}

	if (zone_id == null) {
		// Still no match
		return(PARSE_ERROR);
	}

	date.setTimeZone(TimeZone.getTimeZone(zone_id));

	return(j + name.length());
}

private int parseLiteral(String input, int j, char c)
{
    
    

	/* If current char matches the mask, increment both counters */
	/* Otherwise, just skip mask one */
	if ((j < input.length()) && c == input.charAt(j)) {
		j++;
	}
	else if (c != ' ') /* allow spaces in mask to be optional */ {
		return(PARSE_ERROR);
	}

	return(j);
}

private boolean parseString(String input, DateMask[] mask,
							/* To be filled in: */ WorkingCalendar new_date)
{
    

	int i, j;

	if (input == null || input.length() == 0 ||
	    mask == null || mask.length == 0 &&
		new_date != null)
	{
		return(false);
	}

	// Make sure the target time and date fields are clear
	new_date.clear();

	for (i = 0, j = 0; i < mask.length && j < input.length(); i++) {
		/* Skip all the spaces */
		for(;j < input.length() && Character.isSpaceChar(input.charAt(j)); j++);
        
		switch(mask[i].symbol) {
		case MERIDIEM_HOUR:
		case ZERO_MERIDIEM_HOUR:
		case MILITARY_HOUR:
		case ZERO_MILITARY_HOUR:
		case MINUTE:
		case ZERO_MINUTE:
		case SECOND:
		case ZERO_SECOND:
		case NUM_DAY:
		case ZERO_NUM_DAY:
			j = parseDigitField(new_date, input, j,
						        mask[i].symbol, mask[i].field, 2);
			break;
		case NUM_MONTH:
		case ZERO_NUM_MONTH:
			j = parseDigitField(new_date, input, j,
						        mask[i].symbol, mask[i].field, 2);
			if (j == 0) {
				j = PARSE_ERROR;
			}
			if (j != PARSE_ERROR) {
				int month = new_date.get(mask[i].field);
				if (month == 0) {
					// make sure we don't try and set a negative value
					// on the field
					j = PARSE_ERROR;
				}
				else {
					// we need to subract 1 from the value since the Calendar
					// values are zero based
					month -= 1;
					new_date.set(mask[i].field, month);
				}
			}
			break;
		case SHORT_YEAR:
			j = parseDigitField(new_date, input, j,
						   mask[i].symbol, mask[i].field, 2);
			if (j != PARSE_ERROR) {
				// We need to add the century to the two digit date
				int year = new_date.get(mask[i].field);
				if (year < millenium_threshold) {
					year += 2000; // add 2000 for digits considered
					              // to be after the millenium
				}
				else {
					year += 1900; // add 1900 for digits considered to
					              // be before the millenium
				}
				new_date.set(mask[i].field, year);
			}
			break;
		case LONG_YEAR:
			j = parseDigitField(new_date, input, j,
						        mask[i].symbol, mask[i].field, 4);
			break;
		case DAY_OF_YEAR:
		case ZERO_DAY_OF_YEAR:
			j = parseDigitField(new_date, input, j,
						        mask[i].symbol,  mask[i].field, 3);
			break;
		case MERIDIEM:
		case ABBREV_ALPHA_DAY:
		case ALPHA_DAY:
		case ABBREV_ALPHA_MONTH:
		case ALPHA_MONTH:
			j = parseEnum(new_date, input, j,
						  mask[i].symbol, mask[i].field, mask[i].strings);
			break;
		case TIMEZONE:
		case ABBREV_TIMEZONE:
			j = parseTimeZone(new_date, input, j);
			break;
		case DATE_LITERAL:
			j = parseLiteral(input, j, mask[i].chars.charAt(0));
			break;
		}

        

        

        if (j == PARSE_ERROR) {
            
        }
        else {
            
        }

		if (j == PARSE_ERROR) {
			return(false);
		}
	}

	// Skip all the remaining spaces
	for(;j < input.length() && Character.isSpaceChar(input.charAt(j)); j++);

	// If mask can't parse all the input string then the parse fails
	if (j < input.length()) {
        
		return(false);
	}

	// We are satisfied that we have matched the parse string for as
	// far as possible

	return(true);
}


/**
 * Checks whether the given year is a leap year.
 * @param year the year to check (for example, 1997)
 * @return
 */
public static boolean isLeap(int year)
{
	if (year >= 0) {
		if ((year % 4 == 0 && year % 100 != 0) || year % 400 == 0)
			return (true);
		else
			return (false);
	}
	else {
		return (false);
	}
}

/**
 * Calculates the day of year from month, day, and year.
 * @param year the year (for example, 1997)
 * @param month the month (0 to 11)
 * @param day the day of the month (0 to 31)
 * @return
 */
public static int dayOfYear(int year, int month, int day)
{
	int i, leap = isLeap(year) ? 1 : 0;

    

	for (i = 0; i < month; i++) {
		day += daytable[leap][i];
	}
	return(day);
}

/**
 * Calculates the number of days in a given month.
 * @param year the year to use (for example, 1997)
 * @param month the month (0 to 11)
 * @return
 */
public static int daysInMonth(int month, int year)
{
	int leap = isLeap(year) ? 1 : 0;
	return(daytable[leap][month]);
}

/**
 * Calculates the month, given the year and day_of_year; returns month.
 * @param year the year to use (for example, 1997)
 * @param day_of_year the day of the year (1 to 366)
 * @return
 */
public static int monthOfYear(int year, int day_of_year)
{
	int month = 0;
	int leap  = isLeap(year) ? 1 : 0;

	for (; day_of_year > daytable[leap][month]; month++) {
		day_of_year -= daytable[leap][month];
	}

	return(month);
}

/**
 * Calculates the month, day_of_month, given year and day_of_year; returns day.
 * <BR><B>NOTE:</b> YEAR must be the actual year, or rather "tm_year + 1900".
 * @param year
 * @param day_of_year
 * @return
 */
public static int dayOfMonthOfYear(int year, int day_of_year)
{
	int leap = isLeap(year) ? 1 : 0;

	for (int month = 0; day_of_year > daytable[leap][month]; month++) {
		day_of_year -= daytable[leap][month];
	}

	return(day_of_year);
}

/**
 * Returns how many days in a year.
 * <br><B>NOTE:</B> YEAR must be the actual year, or rather "tm_year + 1900".
 * @return days in a year
 * @param year
 */
public static int daysInYear(int year)
{
	if (isLeap(year)) {
		return (366);
	}
	else {
		return (365);
	}
}

/**
 * Calculates the day of week.
 * <BR><B>NOTE:</B> YEAR must be the actual year i.e. 1997
 *
 * <p>While this algorithm appears inelegant, it is guaranteed to
 * work. Other solutions look more elegant but fail to provide a broad
 * enough range of valid day of week calculations.
 * <p>
 * The value returned is a 1 based day of week
 */
private static int YEARMOD = 1997; // Year to start our calculations from
private static int DAYMOD  = 2;    // For Jan 1, 1997 being of Wednesday

/**
 * Returns the 1 based day of week.
 * @return
 * @param year
 * @param month
 * @param day
 */
public static int dayOfWeek(int year, int month, int day)
{
	long total_days = 0;
	int i_year, week_day;

    

	if (year > YEARMOD) {
		total_days = dayOfYear(year, month, day);
		i_year = YEARMOD;

		while (i_year < year) {
			total_days += daysInYear(i_year);
			i_year++;
		}
	}
	else if (year == YEARMOD) {
		total_days = dayOfYear(year, month, day);
	}
	else if (year < 0) {
		// Do not handle negative years
        
		return(NOTFOUND);		/* Error */
	}
	else {
		// Case for year before YEARMOD
		total_days = daysInYear(year)
			- dayOfYear(year, month, day);
		i_year = year + 1;

		while (i_year < YEARMOD) {
			total_days += daysInYear(i_year);
			i_year++;
		}
		week_day = DAYMOD - ((int) (total_days % 7));
		if (week_day < 0) {
			week_day += 7;
		}

        
		// Add 1 because we return a 1 based day of week
		return(week_day + 1);
	}

    
	// + DAYMOD for "Jan 1, YEARMOD" being on the 0 based day DAYMOD
	// + 1 for the return value being 1 based
	return((int) ((total_days + DAYMOD) % 7) + 1);
}

/**
 * Returns the 1 based week of year.
 * @return 1 based week of year
 * @param year year
 * @param day_of_year day
 */
public static int weekOfYear(int year, int day_of_year)
{
	// Figure out which day in the week Jan 1 is
	int weekday = dayOfWeek(year, 0, 0);

	return(((day_of_year + (6 - weekday)) / 7) + 1);
}

/**
 * Returns the 1 based week of month.
 * @return 1 based week of month
 * @param year year
 * @param month month
 * @param day day
 */
public static int weekOfMonth(int year, int month, int day)
{
	// Figure out which day in the week Jan 1 is
	int weekday = dayOfWeek(year, month, 0);

	return(((day + (6 - weekday)) / 7) + 1);
}

/**
 * Converts the specified type (if possible) to the type supported by
 * the validator.  To reverse this operation, use the <code>convertFromSupported()</code>
 * method.
 * @see #convertFromSupported
 * @param obj
 * @return
 */
public Object convertToSupported(Object obj)
{
    return(JCCalendar.convertObjectToCalendar(obj, getLocale()));
}

/**
 * Converts the supported type to the type supported by the validator.
 * <br><B>Note:</B> The object passed in is not modified, but
 * only used as a reference.
 * @see #convertToSupported
 * @return
 * @param from
 * @param class_type
 */
public Object convertFromSupported(Object from, Class class_type)
{
    if (!(from instanceof Calendar)) {
        return(null);
    }
    return(JCCalendar.convertCalendarToObject((Calendar)from, class_type));
}

/**
 * Returns the default value for the field.
 * @return default value for the field
 */
public Object getDefaultValue()
{
	return(defaultValue);
}

/**
 * Sets the default value for the field.
 * @param default_value
 */
public void setDefaultValue(Object default_value)
{
	this.defaultValue = default_value;
}

/**
 * Returns the popup component for this validator.
 * @return
 */
public PopupFieldEditor createPopupComponent()
{
	return(new DateTimePopup());
}

public class DateTimePopup implements PopupFieldEditor, ActionListener
{
	DateTimeChooser  chooser;
	JCListenerList   popupListeners = null;

	public DateTimePopup()
	{
		this(JCDateChooser.DUAL_SPIN, null, null, null);
	}

	public DateTimePopup(int chooserType, String[] months,
	                     String[] shortMonths, String[] days)
	{
		chooser = new DateTimeChooser(chooserType, months, shortMonths, days);
		chooser.addActionListener(this);
	}

	public JComponent getEditorComponent()
	{
		return(getDateTimeChooser());
	}

	public void addPopupListener(JCPopupListener l)
	{
		popupListeners = JCListenerList.add(popupListeners, l);
	}

	public void removePopupListener(JCPopupListener l)
	{
		popupListeners = JCListenerList.remove(popupListeners, l);
	}

	public void setValue(Object value)
	{
		Calendar val = (Calendar)convertToSupported(value);
		chooser.setCalendarModel(new CalendarValueModel(val));
	}

	public boolean processKeyEvent(KeyEvent e)
	{
		boolean processed = true;

		if (e.getKeyCode() == KeyEvent.VK_ENTER) {
			((DayTable)chooser.getDateChooser().getDayComponent()).fireActionEvent();
			return(processed);
		}

		Calendar cal = JCCalendar.copyCalendar(
			chooser.getDateChooser().getValue());

		if (e.getKeyCode() == KeyEvent.VK_UP) {
			cal.add(Calendar.DATE, -1);
		}
		else if (e.getKeyCode() == KeyEvent.VK_DOWN) {
			cal.add(Calendar.DATE, 1);
		}
		else if (e.getKeyCode() == KeyEvent.VK_PAGE_UP) {
			cal.add(Calendar.MONTH, -1);
		}
		else if (e.getKeyCode() == KeyEvent.VK_PAGE_DOWN) {
			cal.add(Calendar.MONTH, 1);
		}
		else {
			processed = false;
		}

		chooser.getDateChooser().setValue(cal);
		return(processed);
	}

	/**
	 * Sets the locale being used by the popup. In turn, sets it on the
	 * chooser, which will set it on its child components.
	 * @param locale the new locale to be used
	 */
	public void setLocale(Locale locale)
	{
		if (chooser != null) {
			chooser.setLocale(locale);
		}
	}
    /**
     * Sets the <code>Font</code> of the popup.  This font gets propagated to
     * the child components
     *
     * @param f the new <code>Font</code> for the popup
     */
    public void setFont(Font f)
    {
        if (chooser != null) {
            chooser.setFont(f);
            chooser.invalidate();
        }
    }

	public void actionPerformed(ActionEvent event)
	{
		// Fire popup select event
		Object value = chooser.getDateChooser().getValue().clone();
		value = convertToSupported(value);
		JCPopupEvent popupEvent = new JCPopupEvent(this, value);
		Enumeration e = JCListenerList.elements(popupListeners);
		while (e.hasMoreElements()) {
			JCPopupListener listener = (JCPopupListener) e.nextElement();
			listener.commit(popupEvent);
		}
	}

    /**
     * Return the date/time chooser component
     * @return The date/time chooser component being used.
     */
    public DateTimeChooser getDateTimeChooser()
    {
        return(chooser);
    }

} // DateTimePopup

// Wraps a DateChooser and a Time Spinner together
public class DateTimeChooser extends CalendarContainer {

	protected JCDateChooser      dateChooser;
	protected CalendarContainer  timeContainer;
	protected boolean            showTimeSpinner = true;

	public DateTimeChooser()
	{
		this(JCDateChooser.DUAL_SPIN, null, null, null);
	}

	public DateTimeChooser(int chooserType, String[] months,
	                       String[] shortMonths, String[] days)
	{
		JCValueModel current_date =
			new CalendarValueModel(Calendar.getInstance());
	    setLayout(new JCColumnLayout(SwingConstants.CENTER));

		// Get the locale from the enclosing validator
		Locale l = JCDateTimeValidator.this.getLocale();
		if (l == null) {
			l = Locale.getDefault();
		}
	    this.setLocale(l);

	    // Setup JCDateChooser
	    dateChooser = new DateChooser(current_date, locale, chooserType,
									  months, shortMonths, days);
	    add(dateChooser);

		// Setup Time spinner container and time spinners
		timeContainer = new CalendarContainer();

		// Setup Hour Spinner
	    TimeSpin hourSpinner = new TimeSpin();
		hourSpinner.setEditable(false);
		JTextField tf = (JTextField)hourSpinner.getEditor().getEditorComponent();
		tf.setHorizontalAlignment(JTextField.CENTER);
		JCTimeValidator hourValidator = new JCTimeValidator();

		// Decide whether to use 0-12 or 0-23 based hour spinner
		boolean use_military_hour = false;
		boolean found = false;
		for (int i = 0; i < compiled_format.elements.length; i++) {
		    switch(compiled_format.getSymbol(i)) {
		    case MERIDIEM_HOUR:	        /* 0-12 hour */
            case ZERO_MERIDIEM_HOUR:	/* Zero padded 0-12 hour */
                use_military_hour = false;
			    found = true;
			    break;
			case MILITARY_HOUR:         /* 0-23 hour */
            case ZERO_MILITARY_HOUR:    /* Zero padded 0-23 hour */
			    use_military_hour = true;
			    found = true;
			    break;
		    }
			if (found) {
				break;
			}
		}
		if (use_military_hour) {
		    hourValidator.setFormat("HH");
		}
		else {
		    hourValidator.setFormat("hh");
		}

		hourValidator.setMaskInput(true);
	    hourSpinner.setValidator(hourValidator);
        if (userFont != null) {
            hourSpinner.setFont(userFont);
        }
		timeContainer.add(hourSpinner);

		// setup Minute Spinner
		TimeSpin minuteSpinner = new TimeSpin();
		minuteSpinner.setEditable(false);
		tf = (JTextField)minuteSpinner.getEditor().getEditorComponent();
		tf.setHorizontalAlignment(JTextField.CENTER);
	    JCTimeValidator minuteValidator = new JCTimeValidator();
		minuteValidator.setFormat("mm");
		minuteValidator.setMaskInput(true);
	    minuteSpinner.setValidator(minuteValidator);
        if (userFont != null) {
            minuteSpinner.setFont(userFont);
        }
		timeContainer.add(minuteSpinner);

		// setup second spinner
		TimeSpin secondSpinner = new TimeSpin();
		secondSpinner.setEditable(false);
		tf = (JTextField)secondSpinner.getEditor().getEditorComponent();
		tf.setHorizontalAlignment(JTextField.CENTER);
	    JCTimeValidator secondValidator = new JCTimeValidator();
		secondValidator.setFormat("ss");
		secondValidator.setMaskInput(true);
	    secondSpinner.setValidator(secondValidator);
        if (userFont != null) {
            secondSpinner.setFont(userFont);
        }
		timeContainer.add(secondSpinner);

		// setup meridian spinner (AM/PM)
		TimeSpin meridiemSpinner = new TimeSpin();
		meridiemSpinner.setEditable(false);
		tf = (JTextField)meridiemSpinner.getEditor().getEditorComponent();
		tf.setHorizontalAlignment(JTextField.CENTER);
	    JCTimeValidator meridiemValidator = new JCTimeValidator();
		meridiemValidator.setFormat("a");
		meridiemValidator.setMaskInput(true);
	    meridiemSpinner.setValidator(meridiemValidator);
        if (userFont != null) {
            meridiemSpinner.setFont(userFont);
        }
		timeContainer.add(meridiemSpinner);

	    add(timeContainer);
		setCalendarModel(current_date);
    }

	/**
	 * Sets whether the time spinner should appear or not.
     * @param showSpinner
	 */
	public void setShowTimeSpinner(boolean showSpinner)
	{
		if (showTimeSpinner == showSpinner) {
			return;
		}
		else if (showSpinner == false) {
			remove(timeContainer);
		}
		else {
			add(timeContainer);
		}
		showTimeSpinner = showSpinner;
		doLayout();
	}

	/**
	 * Returns the value in the data model.
     * @return
	 */
	public Object getValue()
	{
		return(model.getValue());
	}

	/**
	 * Returns the <code>JCDateChooser</code> component.
	 * @see com.klg.jclass.util.calendar.JCDateChooser
     * @return
	 */
	public JCDateChooser getDateChooser()
	{
		return(dateChooser);
	}

	/**
	 * Returns the time spinner component.
	 * @see com.klg.jclass.field.JCSpinField
     * @return
	 */
	public CalendarContainer getTimeSpinnerContainer()
	{
		return(timeContainer);
	}

} // DateTimeChooser

// Time spinner needs to implement CalendarModel
public class TimeSpin extends JCSpinField implements CalendarComponent {

	public void setCalendarModel(JCValueModel model)
	{
		setValueModel(model);
	}

	public boolean isFocusTraversable()
	{
        return(false);
	}

	public void setSpecialDates(JCCalendar special_dates) {}

	/** Sets the locale being used by the calendar component.
     * @param locale
     */
	public void setLocale(Locale locale) {}

	public void addActionListener(ActionListener l) {}

	public void removeActionListener(ActionListener l) {}
} // TimeSpin

public class DateChooser extends JCDateChooser {

    /**
     * Constructs a localized date chooser of a particular type, set to the
     * given date.
     * @param current_date The calendar value model which stores the date
     * @param locale The locale to use
     * @param type The type of chooser; must be one of
     * <code>JCDateChooser.QUICK_SELECT</code>,
     * <code>JCDateChooser.DUAL_SPIN</code>, or
     * <code>JCDateChooser.READ_ONLY</code>.  The chooser cannot be of
     * type <code>JCDateChooser.SPIN_POPDOWN</code> because we cannot have
     * a popdown within a popdown.
     */
	public DateChooser(JCValueModel current_date, Locale locale, int type)
	{
		super(current_date, locale,
              (type == JCDateChooser.SPIN_POPDOWN) ? JCDateChooser.DUAL_SPIN
                                                   : type);
	}

    /**
     * Constructs a localized date chooser of a particular type, set to the
     * given date.
     * @param current_date The calendar value model which stores the date
     * @param locale The locale to use
     * @param type The type of chooser; must be one of
     * <code>JCDateChooser.QUICK_SELECT</code>,
     * <code>JCDateChooser.DUAL_SPIN</code>, or
     * <code>JCDateChooser.READ_ONLY</code>.  The chooser cannot be of
     * type <code>JCDateChooser.SPIN_POPDOWN</code> because we cannot have
     * a popdown within a popdown.
     * @param months A list of month names to use (null means use locale
     * specific names)
     * @param shortMonths A list of short month names to use (null means use
     * locale specific names)
     * @param days A list of day names to use (null means use locale specific
     * names)
     */
	public DateChooser(JCValueModel current_date, Locale locale,
					   int type, String[] months, String[] shortMonths,
					   String[] days)
	{
		super(current_date, locale,
              (type == JCDateChooser.SPIN_POPDOWN) ? JCDateChooser.DUAL_SPIN
                                                   : type,
              months, shortMonths, days);
	}
    /**
     * Sets the date chooser type.
     * @param type The type of chooser; must be one of
     *     <code>JCDateChooser.QUICK_SELECT</code>,
     *     <code>JCDateChooser.DUAL_SPIN,</code>, or
     *     <code>JCDateChooser.READ_ONLY</code>.  The chooser cannot be of
     *     type <code>JCDateChooser.SPIN_POPDOWN</code> because we cannot have
     *     a popdown within a popdown.
     */
    public void setChooserType(int type)
    {
        // Spin popdown not allowed (get a popdown within a popdown)
        if (type == JCDateChooser.SPIN_POPDOWN) {
            type = JCDateChooser.DUAL_SPIN;
        }
        super.setChooserType(type);
    }

	public void actionPerformed(ActionEvent e)
	{
		// Only want to process events from the DayTable
		if (e.getSource() == getDayComponent()) {
			super.actionPerformed(e);
		}
	}
}

}
