/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCDateValidator.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:17 $ $Locker:  $ Quest Software Inc. 

package com.klg.jclass.field.validate;

import java.util.*;
import com.klg.jclass.field.resources.LocaleInfo;

/**
 * The JCDateValidator is used to validate date objects.
 */
public class JCDateValidator extends JCDateTimeValidator {

/**
 * Constructs a JCDateValidator.
 */
public JCDateValidator()
{
    super();
}

/**
 * Constructs a JCDateValidator with the values specified.
 */
public JCDateValidator(Locale locale, String format, String place_holder_chars,
                       String[] edit_formats, boolean mask_input,
                       int default_detail, boolean allow_null,
                       Object default_value)
{

    super(locale, format, place_holder_chars, edit_formats, mask_input,
          default_detail, allow_null, default_value);
}

/**
 * Gets the default EditFormats for the validator.
 */
public String[] getDefaultEditFormats(Locale l)
{
    String dtp[] = li.getStringArray(LocaleInfo.DATE_TIME_PATTERNS);

    String formats[] = new String[4];
    String fulldate  = dtp[LocaleInfo.FULL_DATE];
    String longdate  = dtp[LocaleInfo.LONG_DATE];
    String meddate   = dtp[LocaleInfo.MEDIUM_DATE];
    String shortdate = dtp[LocaleInfo.SHORT_DATE];

    formats[0] = fulldate;
    formats[1] = longdate;
    formats[2] = meddate;
    formats[3] = shortdate;

    return(formats);
}

/**
 * Returns the default format according to the current value of defaultDetail.
 * @see JCDateTimeValidator#setDefaultDetail
 */
public String getDefaultFormat(Locale l)
{
    String bundleName = "com.klg.jclass.field.resources.LocaleInfo";
    ResourceBundle li = ResourceBundle.getBundle(bundleName, l);
    String dtp[] = li.getStringArray(LocaleInfo.DATE_TIME_PATTERNS);
    String mask = null;

    switch (default_detail) {
    case FULL:
        mask = dtp[LocaleInfo.FULL_DATE];
        break;
    case LONG:
        mask = dtp[LocaleInfo.LONG_DATE];
        break;
    case MEDIUM:
        mask = dtp[LocaleInfo.MEDIUM_DATE];
        break;
    case SHORT:
        mask = dtp[LocaleInfo.SHORT_DATE];
        break;
    }

    return(mask);
}

/**
 * Returns the popup component for this validator.
 */
public PopupFieldEditor createPopupComponent()
{
    PopupFieldEditor editor = super.createPopupComponent();
    DateTimeChooser chooser = (DateTimeChooser)editor.getEditorComponent();
    chooser.setShowTimeSpinner(false);
    return(editor);
}

}




