/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCIPAddressValidator.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:17 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.field.validate;

import com.klg.jclass.util.JCIPAddress;

import javax.swing.ListModel;
import java.util.StringTokenizer;
import java.util.Locale;

/**
 * This class validates objects of type IPAddress.
 */
public class JCIPAddressValidator extends JCStringValidator {

protected int                   increment_field = 0;
protected JCIntegerValidator[]  validators = null;

/**
 * Constructs a <code>JCIPAddressValidator</code>.
 */
public JCIPAddressValidator()
{
    // Set mask
    super.setMask("@@@.@@@.@@@.@@@");

    // Set default ip address
    setDefaultValue(new JCIPAddress("0.0.0.0"));

    // Allow for spinning of subfields
    super.setSpinPolicy(SPIN_SUBFIELD);

    setIncrement(new Integer(1));
}

/**
 * Constructs a <code>JCStringValidator</code> with the values specified.
 * @param allow_null
 * @param default_value
 * @param increment
 * @param locale
 * @param match_picklist
 * @param max1
 * @param max2
 * @param max3
 * @param max4
 * @param min1
 * @param min2
 * @param min3
 * @param min4
 * @param picklist
 */
public JCIPAddressValidator(Locale locale, Number min1, Number max1,
                            Number min2, Number max2, Number min3, Number max3,
                            Number min4, Number max4,
                            ListModel picklist, boolean match_picklist,
                            Integer increment,
                            boolean allow_null, Object default_value)
{
    this();
    setLocale(locale);
    JCIntegerValidator[] validators = new JCIntegerValidator[4];

    if (min1 != null || max1 != null) {
        validators[0] = new JCIntegerValidator();
        validators[0].setMin(min1);
        validators[0].setMax(max1);
    }

    if (min2 != null || max2 != null) {
        validators[1] = new JCIntegerValidator();
        validators[1].setMin(min2);
        validators[1].setMax(max2);
    }

    if (min3 != null || max3 != null) {
        validators[2] = new JCIntegerValidator();
        validators[2].setMin(min3);
        validators[2].setMax(max3);
    }

    if (min4 != null || max4 != null) {
        validators[3] = new JCIntegerValidator();
        validators[3].setMin(min4);
        validators[3].setMax(max4);
    }

    setIPValidators(validators);
    setPickList(picklist);
    setMatchPickList(match_picklist);
    setIncrement(increment);
    setAllowNull(allow_null);
    setDefaultValue(default_value);
}

/**
 * Creates a list of classes supported by the validator.
 * @return
 */
protected Class[] getSupportedClasses()
{
    Class[] supported_classes = new Class[1];

    supported_classes[0] = JCIPAddress.class;

    return(supported_classes);
}

/**
 * Pads a String with spaces, given the specified length.
 * @return
 * @param str
 * @param length
 */
private String padString(String str, int length)
{
    String padded_string = str;

    for (int i = 0; i < length; i++) {
        padded_string += " ";
    }

    return(padded_string);
}

/**
 * Formats the <code>JCIPAddress</code>.
 * @param obj
 * @return
 */
public String format(Object obj)
{
    // If the object is null, show the ip mask
    if (obj == null || !isClassSupported(obj.getClass())) {
        return getParsedMask();
    }

    // Convert the value to the supported format
    JCIPAddress value = (JCIPAddress)convertToSupported(obj);

    String ip = value.toString();

    String new_string;
    StringTokenizer tokenizer = new StringTokenizer(ip, ".");

    // Get the first token and pad make sure it is has 3 characters
    String token = tokenizer.nextToken();
    new_string = padString(token, 3 - token.length());

    // Pad the next 3 numbers to ensure they are 3 characters long
    for (int i = 1; i < 4; i++) {
        token = tokenizer.nextToken();
        new_string += "." + padString(token, 3 - token.length());
    }
    return(new_string);
}

/**
 * Parses the text into a <code>JCIPAddress</code> object.
 * @param class_type
 * @param text
 * @return
 */
public Object parse(Class class_type, String text)
{
    StringTokenizer tokenizer = new StringTokenizer(text, ". ");

    // If there are more than 4 tokens, there is something wrong with the
    // ip address so through parse error
    if (tokenizer.countTokens() != 4) {
        throw new FieldParseException("Cannot convert into valid ip address");
    }

    // Build ip address. We create a new string instead of using the text
    // passed in because we need to get rid of all the leading and trailing
    // white spaces for each subfield. The tokenizer does this for us.
    String new_string = tokenizer.nextToken();
    while (tokenizer.hasMoreTokens()) {
        new_string += "." + tokenizer.nextToken();
    }

    // Create a new JCIPAddress object.
    JCIPAddress ip = new JCIPAddress(new_string);
    return(ip);
}

/**
 * Called whenever a user enters a character interactively.
 * @param e
 */
public void changeText(TextEvent e)
{
    String original_text = e.getOriginalText();
    // if the user specifies a '.', go the space after the next dot
    if (e.getText().equals(".")) {
        int start_pos = e.getStartPosition();
        String submask = getParsedMask().substring(start_pos);

        // find the index of the next dot
        int index = submask.indexOf(".");

        // if there is no other dot, just ignore it
        if (index >= 0) {
            String sub_string;
            // If a dot was pushed in a blank field, don't jump to the next
            // subfield.
            if (start_pos >= 0 && start_pos <= 2) {
                sub_string = original_text.substring(0, 3);
            }
            else if (start_pos >= 4 && start_pos <= 6) {
                sub_string = original_text.substring(4, 7);
            }
            else if (start_pos >= 8 && start_pos <= 10) {
                sub_string = original_text.substring(8, 11);
            }
            else {
                sub_string = original_text.substring(12);
            }
            if (sub_string.trim().length() == 0) {
                e.setAllowChange(false);
            }
            else {
                e.setText("");
                e.setStartPosition(index + start_pos);
            }
        }
        else {
            e.setAllowChange(false);
        }
    }
    else {
        // if a dot is not specified, let the StringValidator superclass
        // handle it
        super.changeText(e);
    }
}

/**
 * Since the <code>clone()</code> method is protected in the
 * <code>Object</code> class, we cannot actually clone a generic object without
 * knowing what it is; therefore the validator must provide this routine to do
 * the copying for <code>JCField</code>.
 * @param value
 * @return
 */
public Object copyValue(Object value)
{
    // JCIPAddress' are immutable so we don't need to copy them
    return(value);
}

/**
 * Overrides <code>setMask</code> so the mask cannot change.
 * @param mask
 */
public void setMask(String mask) {}

/**
 * Sets the validators to be used for each subfield of the IP
 * (for example, xxx.xxx.xxx.xxx).
 * You can associate one validator for each subfield. If there are more than
 * four validators specified, only the first four validators will be used.
 * @param validators
 */
public void setIPValidators(JCIntegerValidator[] validators)
{
    this.validators = validators;
}

/**
 * Returns the set of Integer validators used to validate each subfield.
 * @return set of Integer validators used to validate each subfield
 */
public JCIntegerValidator[] getValidators()
{
    return(validators);
}

/**
 * Validates the IP address. <br><B>Note:</b> This does not mean that it will
 * check to see if the IP address is accessible by the network.
 * @param obj
 * @return
 */
public boolean validate(Object obj)
{
    boolean is_valid = true;
    JCIPAddress new_value = (JCIPAddress)convertToSupported(obj);

    // If the new value is null, check to see if null values are valid
    if (new_value == null) {
        return(getAllowNull());
    }

    int[] subfields = new_value.getIPAddress();
    if (subfields == null) {
        return(false);
    }

    for (int i = 0; i < subfields.length; i++) {
        int new_val = subfields[i];

        // Check to see if an IntegerValidator was specified for each
        // part of the ip and check to make sure it's between the max and min
        if (validators != null &&
            i < validators.length &&
            validators[i] != null &&
            (new_val < ((Number)validators[i].getMin()).intValue() ||
             new_val > ((Number)validators[i].getMax()).intValue()))
        {
            is_valid = false;
        }
        // If no validator was specified, make sure the value is in
        // the valid range for an ip. i.e 0-255
        else if (new_val < JCIPAddress.MIN_VALUE ||
            new_val > JCIPAddress.MAX_VALUE) {
            is_valid = false;
        }
    }

    return(is_valid);
}

/**
 * Overrides <code>setSpinPolicy</code>, since we always want
 * <code>SPIN_SUBFIELD</code> for this validator.
 * @param spin_policy
 */
public void setSpinPolicy(int spin_policy) {}

/**
 * Returns the new IP address to spin to.
 * @param ip
 * @param amount
 * @return new ip address to spin to
 */
private JCIPAddress spinSubField(JCIPAddress ip, int amount)
{
    int field = increment_field;
    int[] subfields = ip.getIPAddress();

    // get the current subfield value which is being spun
    int new_val = subfields[field];

    // increment/decrement the value
    new_val += amount;

    if (validators != null &&
        field < validators.length &&
        validators[field] != null)
    {

        // Only spin between the valid range, if the subfield is not in
        // a valid range, return the current ip
        if (new_val < ((Number)validators[field].getMin()).intValue() ||
            (new_val > ((Number)validators[field].getMax()).intValue()))
        {
            return(ip);
        }
        else if (new_val > JCIPAddress.MAX_VALUE ||
                 new_val < JCIPAddress.MIN_VALUE)
        {
            return(ip);
        }
    }

    // Convert new subfield value to a string so we can create a new ip
    subfields[field] = new_val;

    // create the new JCIPAddress object
    JCIPAddress new_ip = new JCIPAddress(subfields);

    return(new_ip);
}

/**
 * Increments the current object by the increment value.
 * @return
 * @param current
 */
protected Object addIncrement(Object current)
{
    if (current == null || increment == null) {
        
        return(null);
    }

    // get increment
    int value = ((Integer)increment).intValue();
    if (value == 0) {
        return(current);
    }

    // spin the spinfield by the increment
    JCIPAddress ip = (JCIPAddress)current;
    if (spin_policy == SPIN_SUBFIELD) {
        ip = spinSubField(ip, value);
    }

    return(ip);
}

/**
 * Decrements the current object by the increment value.
 * @param current
 * @return
 */
protected Object subtractIncrement(Object current)
{
    if (current == null || increment == null) {
        return (null);
    }

    // get the increment
    int value = ((Integer)increment).intValue();
    if (value == 0) {
        return (current);
    }

    // spin the subfield by the increment
    JCIPAddress ip = (JCIPAddress)current;
    if (spin_policy == SPIN_SUBFIELD) {
        ip = spinSubField(ip, -value);
    }

    return(ip);
}

/**
 * Infers the subfield in the currently selected region that will be
 * selected. If there is no selected region, then the selected region is
 * equal to the cursor position.
 * @param selection_start
 * @param selection_end
 */
public void inferSubField(int selection_start, int selection_end)
{

    // Each subfield must have 3 characters in it.

    // selected field falls into the first subfield
    if (selection_start >= 0 && selection_start <= 3) {
        setIncrementField(0);
    }

    // selected field falls into the second subfield
    else if (selection_start >= 4 && selection_start <= 6) {
        setIncrementField(1);
    }

    // selected field falls into the third subfield
    else if (selection_start >= 8 && selection_start <= 10) {
        setIncrementField(2);
    }

    // selected field falls into the fourth subfield
    else if (selection_start >= 12 && selection_start <= 14) {
        setIncrementField(3);
    }
}

/**
 * Sets the increment when spinning a field. The increment must be
 * a <code>Integer</code> object.
 * @param increment
 */
public void setIncrement(Integer increment)
{
    if (increment == null) {
        return;
    }
    this.increment = increment;
}

/**
 * Returns the increment.
 * @return increment
 */
public Integer getIncrement()
{
    return((Integer)increment);
}

/**
 * Returns the <code>incrementField</code> operated on by the currently set
 * increment value.
 * @return <code>incrementField</code> operated on by the currently set
 * increment value
 */
public int getIncrementField()
{
    return(increment_field);
}

/**
 * Sets the value of the <code>incrementField</code> operated on by the increment.
 * @param field
 */
public void setIncrementField(int field)
{
    if (field < 0 && field >= 4) {
        throw new IllegalArgumentException("invalid value for incrementField");
    }

    increment_field = field;
}

/**
 * Converts the specified type (if possible) to the type supported by
 * the validator.  To reverse this operation, use the
 * <code>convertFromSupported()</code> method.
 * @see #convertFromSupported
 * @return
 * @param obj
 */
public Object convertToSupported(Object obj)
{
    if (obj instanceof JCIPAddress) {
        return(obj);
    }
    return(null);
}

/**
 * Converts the supported type to the type supported by
 * the validator. <br><B>Note:</B> The object passed in is not modified, but
 * only used as a reference.
 * @see #convertToSupported
 * @param from
 * @param class_type
 * @return
 */
public Object convertFromSupported(Object from, Class class_type)
{
    if (from == null || class_type == null) {
        return(null);
    }

    String class_name = class_type.getName();

    // if a string, return it.
    if (class_name.equals("com.klg.jclass.util.JCIPAddress")) {
        return(from);
    }

    return(null);
}

/**
 * Returns the default value for the field.
 * @return default value for the field
 */
public Object getDefaultValue()
{
    return(defaultValue);
}

/**
 * Sets the default value for the field.
 * @param default_value
 */
public void setDefaultValue(Object default_value)
{
    defaultValue = default_value;
}

}


