/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCLongValidator.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:17 $ $Locker:  $ Quest Software Inc. 

package com.klg.jclass.field.validate;

import com.klg.jclass.field.resources.LocaleInfo;


import javax.swing.*;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.ParsePosition;
import java.util.Locale;

/**
 * This class validates objects of type long.
 */
public class JCLongValidator extends JCNumberValidator {

protected static final String minstring = Long.toString(Long.MIN_VALUE);
protected static final String maxstring = Long.toString(Long.MAX_VALUE);

protected String  display_list[] = null;
protected Object  defaultValue = new Long(0);

/**
 * Constructs a JCLongValidator.
 */
public JCLongValidator()
{
    super();
    super.setMax(new Long(Long.MAX_VALUE));
    super.setMin(new Long(Long.MIN_VALUE));
    setIncrement(1);
    supportedClasses = getSupportedClasses();
}

/**
 * Constructs a <code>JCLongValidator</code> with the values specified.
 * @param allow_null
 * @param currency
 * @param currency_locale
 * @param default_value
 * @param display_list
 * @param display_pattern
 * @param increment
 * @param locale
 * @param match_picklist
 * @param max
 * @param min
 * @param picklist
 * @param use_intl_currency_locale
 */
public JCLongValidator(Locale locale, Number min, Number max,
                       ListModel picklist,
                       boolean match_picklist, String display_list[],
                       Number increment, String display_pattern,
                       boolean allow_null, boolean currency,
                       boolean use_intl_currency_locale, Locale currency_locale,
                       Object default_value)
{

    super();
    setLocale(locale);
    setCurrencyLocale(currency_locale);
    setUseIntlCurrencySymbol(use_intl_currency_locale);
    setMax(max);
    setMin(min);
    setPickList(picklist);
    setMatchPickList(match_picklist);
    setDisplayList(display_list);
    setIncrement(increment);
    if (display_pattern != null) {
        setDisplayPattern(display_pattern);
    }
    setAllowNull(allow_null);
    setCurrency(currency);
    setDefaultValue(default_value);
    supportedClasses = getSupportedClasses();
}

/**
 * Creates a list of classes supported by the validator.
 * @return
 */
protected Class[] getSupportedClasses()
{
    Class[] supported_classes = new Class[4];

    supported_classes[0] = Byte.class;
    supported_classes[1] = Integer.class;
    supported_classes[2] = Long.class;
    supported_classes[3] = Short.class;

    return(supported_classes);
}

/**
 * Returns the value of <code>DisplayList</code>.
 * @see #setDisplayList
 * @return the value of <code>DisplayList</code>
 */
public String[] getDisplayList()
{
    return(display_list);
}

/**
 * Returns the value of <code>DisplayList</code>. If the user has not set a <code>Displaylist</code>,
 * the validator will construct one from the picklist using the <code>format()</code>
 * method on the items in the picklist.  If the picklist has not been set,
 * then this method will return <code>null</code>.
 * @see #setDisplayList
 * @see #setPickList
 * @return
 */
public String[] getStringList()
{
    if (picklist == null || picklist.getSize() == 0) {
        return(null);
    }

    if (display_list != null && display_list.length == picklist.getSize()) {
        // only return the display list if it matches the size of the
        // picklist exactly
        return(display_list);
    }

    String list[] = new String[picklist.getSize()];

    // Build picklist from as many elements of the display list as we have
    int i = 0;
    if (display_list != null) {
        for (; i < picklist.getSize() && i < display_list.length; i++) {
            list[i] = display_list[i];
        }
    }

    // fill in the rest of the list with formatted values from the picklist
    for (; i < picklist.getSize(); i++) {
        list[i] = format(picklist.getElementAt(i));
    }

    return(list);
}

/**
 * Sets the display list, which specifies a String format for the value in the
 * picklist independent of what the <code>format()</code> method generates.
 * This is particularly useful in the case of Enumerated Database types where
 * <i>1 = Mr,</i> <i>2 = Mrs</i>, etc.  In this case, you would set the
 * <code>DisplayList</code> so that if the internal value was <i>1</i>, then
 * <i>Mr</i> would be displayed.
 * @param list
 */
public void setDisplayList(String list[])
{
    display_list = list;
}

/**
 * Sets the maximum valid value.
 * @param max
 */
public void setMax(long max)
{
    setMax(new Long(max));
}

/**
 * Sets the minimum valid value.
 * @param min
 */
public void setMin(long min)
{
    setMin(new Long(min));
}

protected Object addIncrement(Object current)
{
    if (current == null || increment == null) {
        return(null);
    }

    long new_value = ((Number)current).longValue() +
        ((Number)increment).longValue();

    if (new_value > ((Number)getMax()).longValue()) {
        return(null);
    }

    return(new Long(new_value));
}

protected void setPatterns()
{
    String np[] = li.getStringArray(LocaleInfo.NUMBER_PATTERNS);
    if (isCurrency) {
        display_pattern = np[LocaleInfo.CURRENCY_PATTERN];
    }
    else {
        display_pattern = np[LocaleInfo.DECIMAL_PATTERN];
    }
    edit_pattern = "0";
}

protected DecimalFormat createDisplayFormat(Locale l)
{
    DecimalFormat df =
        (DecimalFormat)NumberFormat.getNumberInstance(Locale.getDefault());
    df.setParseIntegerOnly(true);
    return(df);
}

protected DecimalFormat createEditFormat(Locale l)
{
    DecimalFormat df =
        (DecimalFormat)NumberFormat.getNumberInstance(Locale.getDefault());
    df.setParseIntegerOnly(true);
    return(df);
}

protected Object subtractIncrement(Object current)
{
    if (current == null || increment == null) {
        return(null);
    }

    long new_value = ((Number)current).longValue() -
        ((Number)increment).longValue();

    if (new_value < ((Number)getMin()).longValue()) {
        return(null);
    }

    return(new Long(new_value));

}

/**
 *  Gets the number by which to spin the values in numeric spin fields.
 * @return
 */
public Number getIncrement()
{
    return((Number)increment);
}

/**
 *  Sets the number by which to spin the values in numeric spin fields.
 * @param increment
 */
public void setIncrement(Number increment)
{
    if (increment.longValue() <= 0) {
        throw new IllegalArgumentException("increment <= 0");
    }
    this.increment = increment;
}

/**
 *  Sets the number by which to spin the values in numeric spin fields.
 * @param increment
 */
public void setIncrement(long increment)
{
    setIncrement(new Long(increment));
}

/**
 *  Checks if the value is in the valid range.
 * @param value
 * @return
 */
public boolean inRange(long value)
{
    if (value < ((Number)getMin()).longValue() ||
        value > ((Number)getMax()).longValue())
    {
        return(false);
    }
    return(true);
}

/**
 *  Checks if the value is in the valid range.
 * @param value
 * @return
 */
public boolean inRange(Number value)
{
    return(inRange(value.longValue()));
}

/**
 *  Checks if the value is in the valid range.
 * @param value
 * @return
 */
public boolean inRange(Object value)
{
    if (value instanceof Number) {
        return(inRange(((Number)value).longValue()));
    }
    return(false);
}

/**
 *  Changes the specified text from the <code>start_position</code> to the
 * <code>end_position</code> by inserting/replacing it by specified
 * <code>new_text</code>.
 * @param e
 */
public void changeText(TextEvent e)
{
    StringBuffer allowed_new = null;
    StringBuffer working = new StringBuffer(50);

    // Super handles case switching
    super.changeText(e);

    // Figure out if we are at the end or at the beginning
    // We need to know this so that we can
    //  a) Only allow +/-/() symbols at the beginning _or_ the end
    //     of the text
    //  b) We do not allow +/- or decimal signs if they have already
    //     been typed
    //  c) If the +/- signs are not at the begining then they are at the
    //     end and that means that we do not allow any further characters
    //     to be input
    String oldText = e.getOriginalText();

    boolean exponent_char_typed = false;

    int start = e.getStartPosition();
    int end = e.getEndPosition();
    if (oldText.indexOf(exponentialChar) != -1) {
        exponent_char_typed = true;
        if (e.isDeletion() == true) {
            // Scan through and see if the exponent character is in the text
            // that is pending delete
            String substring = oldText.substring(start, end);
            if (substring.indexOf(exponentialChar) != -1) {
                exponent_char_typed = false;
            }
        }
    }

    boolean sign_typed = false;
    boolean open_typed = false;
    boolean at_beginning = false;
    boolean at_end = false;

    if (start == 0) {
        at_beginning = true;
    }
    else if (oldText != null && oldText.length() > 1) {
        // Check last position to see if it tells us we are at the end
        char last_char = oldText.charAt(oldText.length() - 1);
        if (last_char == symbols.getMinusSign() &&
            exponent_char_typed == false)
        {
            at_end = true;
        }
        else if (last_char == '+') {
            at_end = true;
        }
        else if (last_char == ')') {
            at_end = true;
        }
    }

    if (oldText.indexOf(symbols.getMinusSign()) != -1) {
        sign_typed = true;
    }
    else if (oldText.indexOf('+') != -1) {
        sign_typed = true;
    }
    else if (oldText.indexOf('(') != -1) {
        sign_typed = true;
        open_typed = true;
    }

    String newText = e.getText();
    if (newText != null && newText.length() != 0) {
        // Call function in super class to strip out specific
        // valid/invalid chars
        allowed_new = checkValidInvalid(newText);
        if (allowed_new != null && allowed_new.length() != 0) {
            // Validate input keys and strip out any that are invalid
            for (int i = 0; i < allowed_new.length(); i++) {
                char c = allowed_new.charAt(i);
                if (at_end) {
                    continue;
                }
                if (display_list != null) {
                    // Display list is not null so allow anything through
                    // that wasn't explictly allowed
                    working.append(c);
                }
                else if (Character.isDigit(c) == true) {
                    // Allow digits
                    working.append(c);
                }
                else if (exponent_char_typed == false &&
                         at_beginning == false &&
                         exponentialChar.equalsIgnoreCase(String.valueOf(c)) &&
                         (oldText.indexOf(symbols.getMinusSign()) + 1) != start)
                {
                    // Only allow 1 exponent character
                    working.append(exponentialChar);
                }
                else if (c == symbols.getMinusSign() &&
                         ((sign_typed == false &&
                           exponent_char_typed == false) ||
                          (exponent_char_typed == true &&
                           (oldText.indexOf(exponentialChar) +
                            exponentialChar.length()) == start)))
                {
                    // Allow negative symbol
                    working.append(c);
                }
                else if ((c == '(' || c == ')') &&
                         edit_pattern.indexOf('(') != -1)
                {
                    // Allow ( and ) if we can find it in the edit_pattern
                    // since the parse will fail if it's not

                    // Further more only allow a "(" at position 0
                    // and ")" if a "(" has already been typed
                    if (c == '(' && at_beginning) {
                        working.append(c);
                    }
                    else if (c == ')' && open_typed) {
                        working.append(c);
                    }
                }
            }
        }
        e.setText(working.toString());
        if (working.length() == 0) {
            e.setAllowChange(false);
        }
    }
}

/**
 * Parses the given text and validates it.
 * @return
 * @param class_type object returned will be of the specified class type
 * @param text text to parse
 */
public Object parse(Class class_type, String text)
{
    if (text == null || text.length() == 0) {
        return null;
    }
    // first try to parse from list of strings provided
    Object value = parseFromDisplayList(text);
    if (value != CANT_USE_DISPLAY_LIST) {
        return value;
    }
    else {
        value = null;
    }

    if ((text == null || text.length() == 0) && getAllowNull() == true) {
        return(value);
    }

    // Check if there are too many characters since the format class doesn't
    // seem to behave as documented
    // Note this first check is a ball park check to avoid unnecessary
    // processing
    if (text.length() >= Math.min(minstring.length(), maxstring.length())) {
        // figure out if we are handling the negative case or not by looking
        // for the minusSign or "("
        String touchstone;

        if (text.indexOf(symbols.getMinusSign()) != -1 ||
            text.indexOf('(') != -1) {
            // Negative case
            touchstone = minstring;
        }
        else {
            // Positive case
            touchstone = maxstring;
        }

        // Get rid of all non decimal chars
        StringBuffer tmp = new StringBuffer(text.length());

        for (int i = 0; i < text.length(); i++) {
            char c = text.charAt(i);
            if (Character.isDigit(c)) {
                tmp.append(c);
            }
        }

        // Recheck length
        if (tmp.length() > touchstone.length()) {
            // string is definitely too long; so bail
            throw new FieldParseException("Text too long");
        }
        else if (tmp.length() == touchstone.length()) {
            for (int i = 0; i < tmp.length(); i++) {
                // Compare digit to digit
                if (Long.parseLong(tmp.charAt(i) + "") >
                    Long.parseLong(touchstone.charAt(i) + ""))
                {
                    // number is too big; so bail
                    throw new FieldParseException("Value exceed maximum");
                }
            }
        }
    }

    ParsePosition position = new ParsePosition(0);
    // "position" is filled in by this call to parse()

    Number num = editFormat.parse(text, position);
    if (num != null &&
        (num.longValue() < ((Number)min).longValue() ||
         num.longValue() > ((Number)max).longValue()))
    {
        // text value was too big
        throw new FieldParseException("Value exceed maximum length");
    }
    else if (position.getIndex() == 0) {
        if (getAllowNull() == false) {
            throw new FieldParseException("Value exceeds maximum length");
        }
    }
    else {
        value = new Long(num.longValue());
    }

    // Convert to support type
    value = convertFromSupported(value, class_type);
    return(value);
}

/**
 * Validates a given object.
 * @param value
 * @return
 */
public boolean validate(Object value)
{
    boolean is_valid = super.validate(value);
    if (is_valid == false) {
        return(false);
    }

    // Make sure the class is supported
    if (value != null && isClassSupported(value.getClass())) {
        value = convertToSupported(value);
    }

    // Check to ensure that the object is in the allowable range
    if (value != null && !inRange(value)) {
        return(false);
    }

    return(true);
}

/**
 * Generates a String representation of the given object for use in edit mode.
 * @param obj
 * @return
 */
public String formatForEdit(Object obj)
{
    

    if (obj == null) {
        return("");
    }

    Object ro = formatFromDisplayList(obj);
    if (ro != CANT_USE_DISPLAY_LIST) {
        // Use the format provided by the DisplayList
        return((String)ro);
    }

    return(editFormat.format(obj));
}

/**
 * Reports whether the validator has a different edit format or not.
 * @return
 */
public boolean hasEditFormat()
{
    return(true);
}

/**
 * Returns a formatted String given an object.
 * @return formatted String given an object
 * @param obj
 */
public String format(Object obj)
{
    

    if (obj == null || !isClassSupported(obj.getClass())) {
        return("");
    }

    Object ro = formatFromDisplayList(obj);
    if (ro != CANT_USE_DISPLAY_LIST) {
        // Use the format provided by the DisplayList
        return((String)ro);
    }

    
    return(displayFormat.format(obj));
}

/**
 * Converts the specified type (if possible) to the type supported by
 * the validator.  To reverse this operation, use the
 * <code>convertFromSupported()</code> method.
 * @see #convertFromSupported
 * @return
 * @param obj
 */
public Object convertToSupported(Object obj)
{
    // Don't create a new instance if the object is already a Long object
    if (obj instanceof Long) {
        return(obj);
    }
    if (obj instanceof Number) {
        return(new Long(((Number)obj).longValue()));
    }
    return(null);
}

/**
 * Converts the supported type to the type supported by
 * the validator. <br><B>Note:</B> The object passed in is not modified but
 * only used as a reference.
 * @see #convertToSupported
 * @return
 * @param from
 * @param class_type
 */
public Object convertFromSupported(Object from, Class class_type)
{
    if (from == null || class_type == null) {
        return(null);
    }

    String class_name = class_type.getName();
    Long num = (Long)from;

    if (class_name.equals("java.lang.Long")) {
        return(num);
    }
    if (class_name.equals("java.lang.Byte")) {
        return(new Byte(num.byteValue()));
    }
    if (class_name.equals("java.lang.Short")) {
        return(new Short(num.shortValue()));
    }
    if (class_name.equals("java.lang.Integer")) {
        return(new Integer(num.intValue()));
    }
    return(null);
}

/**
 * Returns the default value for the field.
 * @return default value for the field
 */
public Object getDefaultValue()
{
    return(defaultValue);
}

/**
 * Sets the default value for the field.
 * @param default_value
 */
public void setDefaultValue(Object default_value)
{
    this.defaultValue = default_value;
}

protected Object CANT_USE_DISPLAY_LIST = new Integer(3485934);

protected Object parseFromDisplayList(String text)
{
    if (display_list != null && display_list.length > 0) {
        // Use a string from the DisplayList if possible to format the data
        int i;
        for (i = 0; i < display_list.length; i++) {
            if (text.equalsIgnoreCase(display_list[i])) {
                break;
            }
        }

        if (i != NOT_FOUND && i < picklist.getSize()) {
            return picklist.getElementAt(i);
        }
    }

    return(CANT_USE_DISPLAY_LIST);
}

protected Object formatFromDisplayList(Object obj)
{
    if (display_list != null && display_list.length > 0) {
        // Use a string from the DisplayList if possible to format the data
        int i = getPickListIndex(obj);
        if (i != NOT_FOUND && i < display_list.length) {
            return display_list[i];
        }
    }

    return(CANT_USE_DISPLAY_LIST);
}

}
