/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCStringValidator.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:17 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.field.validate;



import javax.swing.*;
import java.util.Locale;

/**
 * This class validates objects of type String.
 */
public class JCStringValidator extends AbstractValidator {

// Internal constants
static final int NUMBER = 0;
static final int DIGIT = 1;
static final int HEX = 2;
static final int UPPER = 3;
static final int LOWER = 4;
static final int NOCONVERT = 5;
static final int STAR = 6;
static final int ALPHANUMERIC = 7;
static final int LITERAL = 8;

static final String HEXDIGITS = "AaBbCcDdEeFf";
static final String NUMBERDIGITS = "+-,.";

// Internal variables
protected int[]   compiled_mask_sym;
protected char[]  compiled_mask_ch;
protected int     mask_length = 0;
protected int     num_editable_pos = 0;

// Resources
protected String  mask = "";
protected String  place_holder_chars = null;
protected String  mask_chars = "#@HULA*^\\";
protected int     num_mask_match = -1;
protected Object  defaultValue = null;

/**
 * Constructs a <code>JCStringValidator</code>.
 */
public JCStringValidator()
{
	spin_policy = SPIN_WRAP;
	supportedClasses = getSupportedClasses();
}

/**
 * Constructs a <code>JCStringValidator</code> with the values specified.
 * @param allow_null
 * @param default_value
 * @param invalid_chars
 * @param locale
 * @param mask
 * @param match_picklist
 * @param num_match
 * @param picklist
 * @param place_holder_chars
 * @param valid_chars
 */
public JCStringValidator(Locale locale, String mask, int num_match,
						 String place_holder_chars,
						 ListModel picklist, boolean match_picklist,
						 String valid_chars, String invalid_chars,
						 boolean allow_null, Object default_value)
{
	this();
	setLocale(locale);
	setMask(mask);
	setPickList(picklist);
	setMatchPickList(match_picklist);
	setNumMaskMatch(num_match);
	setPlaceHolderChars(place_holder_chars);
	setValidChars(valid_chars);
	setInvalidChars(invalid_chars);
	setAllowNull(allow_null);
	setDefaultValue(default_value);

}

/**
 * Creates a list of classes supported by the validator.
 * @return
 */
protected Class[] getSupportedClasses()
{
	Class[] supported_classes = new Class[2];

	supported_classes[0] = String.class;
	supported_classes[1] = StringBuffer.class;

	return(supported_classes);
}

/**
 * Since the <code>clone()</code> method is protected in the <code>Object</code>
 * class we cannot actually clone a generic object without knowing what it is;
 * therefore, the validator must provide this routine to do the copying for
 * <code>JCField</code>.
 * @return
 * @param value
 */
public Object copyValue(Object value)
{
	// Strings are immutable so we don't need to copy them
	return(value);
}

/**
 * Sets the number of characters to match with the mask from left to
 * right. This number does not include any literals. If the value is negative,
 * the entire mask will be matched.
 * @see #getNumMaskMatch
 * @param num_mask_match
 */
public void setNumMaskMatch(int num_mask_match)
{
	this.num_mask_match = num_mask_match;
}

/**
 * Returns the number of characters to match with the mask.
 * @see #setNumMaskMatch
 * @return number of characters to match with the mask
 */
public int getNumMaskMatch()
{
	return(num_mask_match);
}

/**
 * Sets the mask to be used.<p>
 * #  - Any digit, minus sign, comma, decimal point, or 
 * plus sign.<br>
 * @  - Any digit.<br>
 * H  - Any hexadecimal digit.<br>
 * U  - Any alphabetic character. Lower case characters will be converted
 *      to upper case.<br>
 * L  - Any alphabetic character. Upper case characters will be converted
 *      to lower case.<br>
 * A  - Any alphabetic character. No case conversion.<br>
 * *  - Any character.<br>
 * ^  - Any character or digit. (aA-zZ, 0-9)<br>
 * \\ - The next character that follows is to be treated as a literal, even
 *      if it is one of the above characters.<br>
 * @see #setMaskChars
 * @see #getMask
 * @param mask
 */
public void setMask(String mask)
{
    
	this.mask = mask;
	compileMask();
}

/**
 * Returns the current mask.
 * @see #setMask
 * @return current mask
 */
public String getMask()
{
	return(mask);
}

/**
 * Sets the placeholder String, which specifies the characters to use
 * instead of an empty mask.  If this String is null, or an empty
 * character exists after the number of characters provided, then the field
 * uses a space character.
 * @see #getPlaceHolderChars
 * @param place_holder_chars
 */
public void setPlaceHolderChars(String place_holder_chars)
{
	this.place_holder_chars = place_holder_chars;
}

/**
 * Returns the placeholder String.
 * @see #setPlaceHolderChars
 * @return placeholder String
 */
public String getPlaceHolderChars()
{
	return(place_holder_chars);
}

/**
 * Sets the mask characters to be used.
 * The default is: "#@HULA*^\\".<p>
 * #  - Any digit, minus sign, comma, decimal point, or 
 * plus sign.<br>
 * @  - Any digit.<br>
 * H  - Any hexadecimal digit.<br>
 * U  - Any alphabetic character. Lower case characters will be converted
 *      to upper case.<br>
 * L  - Any alphabetic character. Upper case characters will be converted
 *      to lower case.<br>
 * A  - Any alphabetic character. No case conversion.<br>
 * *  - Any character.<br>
 * ^  - Any character or digit. (aA-zZ, 0-9)<br>
 * \\ - The next character that follows is to be treated as a literal, even
 *      if it is one of the above characters.<br>
 * @see #getMaskChars
 * @param mask_chars
 */
public void setMaskChars(String mask_chars)
{
	if (mask_chars.length() != this.mask_chars.length()) {
		throw new IllegalArgumentException("Missing mask chars");
	}

	this.mask_chars = mask_chars;
	compileMask();
}

/**
 * Returns the mask characters used.
 * @see #setMaskChars
 * @return mask characters used
 * @return mask characters used
 */
public String getMaskChars()
{
	return(mask_chars);
}

/**
 * Returns a String with only literals in the correct position.
 * @see #setMask
 * @return String with only literals in the correct position
 */
public String getParsedMask()
{
	if (mask != null && mask_length != 0) {
		return(String.copyValueOf(compiled_mask_ch));
	}
	else {
		return("");
	}
}

/**
 *  Checks if the value is in the valid range.
 * @return
 * @param value
 */
public boolean inRange(Object value)
{
	return(true);
}

/**
 * This routine compiles the mask into 2 arrays. One array keeps track
 * of the valid characters at certain positions and the other array
 * keeps track of the actual literals that need to be printed.
 */
protected void compileMask()
{
    
	// Check to see if there is a mask
    int string_len = (mask == null) ? 0 : mask.length();
	if (string_len == 0) {
		mask_length = 0;
		return;
	}

	compiled_mask_sym = new int[string_len];
	compiled_mask_ch  = new char[string_len];

	// Reset the number of actual positions a user can type in
	num_editable_pos = 0;

	// Compile the mask
    int size = 0;
	for (int i = 0; i < string_len; i++, size++) {
		int index = mask_chars.indexOf(mask.charAt(i));

		if (index < 0) {
			// The character must be a literal since it was not found in
		    // the mask
			compiled_mask_sym[size] = LITERAL;
			compiled_mask_ch[size] = mask.charAt(i);
		}
		else {
			// The next character after the '\\' must be a literal
			if (mask.charAt(i) == mask_chars.charAt(8)) {
				i += 1;
				if (i < string_len) {
					compiled_mask_sym[size] = LITERAL;
					compiled_mask_ch[size]  = mask.charAt(i);
				}
				else {
					size--;
				}
			}
			else {
				// Index corresponds to the position in the maskChars array
				compiled_mask_sym[size] = index;
				compiled_mask_ch[size] = ' ';
				num_editable_pos += 1;
			}
		}
	}

	// Save the length of the array.
	mask_length = size;
}

/**
 * This function checks to see if the character entered is valid.
 * It compares it to the mask specified.
 * @return
 * @param actual_position
 * @param buffer
 * @param string_position
 */
protected boolean isValidChar(StringBuffer buffer, int string_position, int
							  actual_position)
{
	char value = buffer.charAt(string_position);

	// Ensure that the position is valid
	if (actual_position >= mask_length) {
		return(false);
	}

	switch (compiled_mask_sym[actual_position]) {
	case NUMBER:
		return(Character.isDigit(value) || (NUMBERDIGITS.indexOf(value) >= 0));
	case HEX:
		return(Character.isDigit(value) || (HEXDIGITS.indexOf(value) >= 0));
	case DIGIT:
		return (Character.isDigit(value));
	case UPPER:
		if (Character.isLetter(value)) {
			buffer.setCharAt(string_position, Character.toUpperCase(value));
			return(true);
		}
        else {
			return(false);
		}
	case LOWER:
		if (Character.isLetter(value)) {
			buffer.setCharAt(string_position, Character.toLowerCase(value));
			return(true);
		}
        else {
			return(false);
		}
	case NOCONVERT:
		return(Character.isLetter(value));
	case STAR:
		return(true);
	case LITERAL:
		if (value == compiled_mask_ch[actual_position]) {
			return(true);
		}
		return(false);
	case ALPHANUMERIC:
		return(Character.isLetter(value) || Character.isDigit(value));
	default:
		return(false);
	}
}

/**
 * Gets the next non-literal cursor position.
 * @return
 * @param current_position
 * @param direction
 */
public int getNextCursorPosition(int current_position, boolean direction)
{
	if (mask == null || mask_length == 0) {
		return(current_position);
	}

	if (current_position == mask_length) {
		return(current_position);
	}

	// Search positions greater than current position
	if (direction == true) {
		for (int i = current_position; i < mask_length ; i++) {
			if (compiled_mask_sym[i] != LITERAL) {
				return(i);
			}
		}
	}

	// Search position less than the current position
	else {
		for (int i = current_position; i < mask_length && i >= 0; i--) {
			if (compiled_mask_sym[i] != LITERAL) {
				return(i);
			}
		}
	}
	return(AbstractValidator.NOT_FOUND);
}

/** Adds placeholders. If no placeholders are specified, a space is inserted.
 * @param buffer
 * @param position
 */
private void addPlaceHolder(StringBuffer buffer, int position)
{
	int holder_length = 0;
	if (place_holder_chars != null) {
		holder_length = place_holder_chars.length();
	}

	if (holder_length > 0 && position < holder_length) {
		buffer.append(place_holder_chars.charAt(position));
	}
	else {
		buffer.append(" ");
	}
}

/**
 * Processes all changes in the text field. It will validate the
 * text if a mask is used.
 * @param e
 */
public void changeText(TextEvent e)
{
	

	int           i, j;
	int           position;
	int           next_pos;
	boolean       valid = false;
	String        new_text;
	StringBuffer  delete_buffer = new StringBuffer(50);

	// Super handles case switching
	super.changeText(e);

	// Check to see if a deletion is occuring. If so, do not allow the
    // deletion of a literal, delete the character before it.
	int start_pos = e.getStartPosition();
	int end_pos   = e.getEndPosition();
	if (e.isDeletion() == true             &&
	    (end_pos - start_pos > 0)          &&
	    (mask != null && mask_length != 0) &&
		(start_pos < mask_length && end_pos <= mask_length))
	{
		// CASE 1: Deleting a single character and it is a literal
		if (compiled_mask_sym[start_pos] == LITERAL &&
		    (end_pos - start_pos) == 1)
		{
			next_pos = getNextCursorPosition(start_pos, false);

			// If no position was found, do not perform any deletions
			// since we must be on the first editable position
			if (next_pos == AbstractValidator.NOT_FOUND) {
				e.setStartPosition(end_pos);
				e.setAllowChange(false);
				return;
			}

			// Insert a space for the deleted character unless show mask
			// is set to true.
			addPlaceHolder(delete_buffer, next_pos);

			// Insert all the literals that might have been deleted.
			for (i = next_pos + 1; i <= start_pos; i++) {
				if (compiled_mask_sym[i] == LITERAL) {
					delete_buffer.append(compiled_mask_ch[i]);
				}
				else {
					addPlaceHolder(delete_buffer, i);
				}
			}
			// Set the new starting position
			e.setStartPosition(next_pos);
			e.setText(delete_buffer.toString());
		}
		else if ((end_pos - start_pos) > 1) {
		     // CASE 2: Deleting a block of characters
			StringBuffer text = new StringBuffer(e.getText());
			int text_len = text.length();
			int text_pos = 0;
			// Insert all the literals that might have been deleted.
			for (i = start_pos; i < end_pos; i++) {
				if (compiled_mask_sym[i] == LITERAL) {
					delete_buffer.append(compiled_mask_ch[i]);
				}
				else if (!text.equals("") && text_pos < text_len &&
						 (isValidChar(text, text_pos, i) == true))
				{
					delete_buffer.append(text.charAt(text_pos));
					text_pos += 1;
				}
				else {
					addPlaceHolder(delete_buffer, i);
				}
			}
			e.setStartPosition(start_pos);
			e.setText(delete_buffer.toString());
		}
		else {
			// CASE 3: Deleting a single character that's not a literal
			addPlaceHolder(delete_buffer, start_pos);
			//e.setStartPosition(start_pos);
			e.setText(delete_buffer.toString());
		}
		return;
	}

	new_text = e.getText();
	if (new_text != null && new_text.length() != 0) {
	    StringBuffer working = new StringBuffer(50);

		// Remove all invalid characters
	    StringBuffer allowed_new = checkValidInvalid(new_text);
		if (allowed_new != null && allowed_new.length() != 0) {
			position = start_pos;

			// If there is a mask, validate the text
			if (mask != null && mask_length != 0) {

				for (i = 0; i < allowed_new.length(); i++) {

					valid = isValidChar(allowed_new, i, position);

					// If input was valid, process any literals
					if (valid == true) {
						working.append(allowed_new.charAt(i));
						position += 1;

						if (end_pos - start_pos != 1) {
							// Print out literals which may follow
							for (j = position;
							     (j < mask_length) &&
								 (compiled_mask_sym[j] == LITERAL);
								 j++)
							{
                                char chMask = compiled_mask_ch[j];
								working.append(chMask);
								// If we added a literal and the next
								// input character to check is the
								// same literal, skip it.
								if ((i + 1) < allowed_new.length() &&
									chMask == allowed_new.charAt(i + 1))
								{
									i += 1;
								}
							}
						}

						// Get next valid cursor position
						position = getNextCursorPosition(position, true);
					}
				}
				// Ensure that the end position is 1 greater than the
				// start pos + length of chars.
				if (end_pos - start_pos != 1) {
					e.setEndPosition(e.getStartPosition() + working.length());
				}
			}
			else {
				working.append(allowed_new.toString());
			}
		}

		e.setText(working.toString());
		if (working.length() == 0) {
			e.setAllowChange(false);
		}
	}
}

/**
 * Moves the cursor to a valid cursor position.
 * @param e
 */
public void setCursor(TextCursorEvent e)
{
	int cursor_pos = e.getNewPosition();
	int old_pos = e.getCurrentPosition();

	// Determine whether the cursor movement was forward or backward.
    boolean direction = true;
	if (old_pos > cursor_pos) {
		direction = false;
	}

	// Make sure the cursor is always on a valid position
	int retval = getNextCursorPosition(cursor_pos, direction);
	if (retval == AbstractValidator.NOT_FOUND) {
		retval = getNextCursorPosition(cursor_pos, !direction);
		if (retval == AbstractValidator.NOT_FOUND) {
			e.setNewPosition(e.getCurrentPosition());
			return;
		}
	}

	e.setNewPosition(retval);
}

/**
 * Returns the last valid input character in text.
 *
 * Skips any characters from the end that match the
 * <code>placeHolderChars</code> by counting the length backward.
 * @return last valid input character in text
 * @param text
 */
public int getLastValidInputCharPosition(String text)
{
    int length = 0;
    if (text == null) {
        return(length);
    }

    if ((mask == null || mask_length == 0)) {
        return(text.length());
    }

	for (length = text.length(); length > 0; length--) {
        int pos = length - 1;
		if (place_holder_chars == null ||
			length > place_holder_chars.length())
        {
			// Look for a space or literal
			if (text.charAt(pos) != ' ' && compiled_mask_sym[pos] != LITERAL) {
	            break;
			}
	    }
		else {
			// Look for the place holder char
			if (text.charAt(pos) != place_holder_chars.charAt(pos) &&
	    	    compiled_mask_sym[pos] != LITERAL)
            {
				break;
			}
		}
	}
    return(length);
}

/**
 * Validates the text after the text field loses focus.
 * @param class_type
 * @param text
 * @return
 */
public Object parse(Class class_type, String text)
{
	String internal_string = null;
	Object value;

	// if no text or mask, and allow null is true,
    // then the text is valid
	if (((text == null || text.length() == 0) && getAllowNull() == true) ||
	    (mask == null || mask_length == 0))
	{
		internal_string = text;
	}
	else {
		StringBuffer temp_buffer = new StringBuffer(50);
		int length = getLastValidInputCharPosition(text);

		// strips all literals from the internal value
		for (int i = 0; i < length; i++) {
			// if the character is a literal, do not add it to the
			// internal value
			if (compiled_mask_sym[i] == LITERAL &&
			    compiled_mask_ch[i] == text.charAt(i))
            {
				continue;
			}
			temp_buffer.append(text.charAt(i));
		}
		internal_string = temp_buffer.toString();
	}

	value = internal_string;
	
	// if the class is of type string already, no need to convert.
    // We also do this check because JCDateTimeValidator calls this method.
    // When convertFromSupported gets called, it calls JCDateTimeValidator's
	// method which causes a class cast exception.
	if (value.getClass() != class_type) {
		value = convertFromSupported(value, class_type);
	}
	
	return(value);
}

/**
 * Validates the value in the field.
 * @return
 * @param obj
 */
public boolean validate(Object obj)
{
	Object new_value = convertToSupported(obj);
	boolean is_valid = internalValidate(new_value);
	return(is_valid);
}

/**
 * This does the actual validation for the validate method. We
 * need an internal method because <code>parseAndValidate</code> called any
 * subclass' validate method, which is incorrect. The
 * <code>parseAndValidate</code> method in this class must call the validate
 * method in this class.
 * @return
 * @param obj
 */
protected boolean internalValidate(Object obj)
{
	boolean is_valid = super.validate(obj);

	// If text is null or empty string and allow null is false, return invalid
    String text = (String)obj;
	if ((text == null || text.length() == 0) && getAllowNull() == false) {
		return(false);
	}

	// If there is a mask and there is text, we must validate.
	// In all other cases, we should just return the current value
	// of is_valid
    int mask_pos = 0;
    int last_valid_pos = -1;
	if (is_valid == true && mask != null && mask_length > 0 && text != null) {
		StringBuffer temp_buffer = new StringBuffer(text);
		// Loop through the string and validate
		for (int i = 0; i < temp_buffer.length(); i++) {
			// Skip all literals
			for (int j = mask_pos;
                 j < mask_length && compiled_mask_sym[j] == LITERAL;
                 j++)
            {
				mask_pos += 1;
			}

			// Now validate the character
			if (isValidChar(temp_buffer, i, mask_pos) == false) {
				is_valid = false;
				break;
			}
			mask_pos += 1;

			// This is used to keep track of the last position which has
			// a valid value. It will be used when checking it against
			// the numMatchMask property.
			last_valid_pos = i;
		}

		// After validation, check to see if there are enough characters
		// to match the mask.
		if (num_mask_match >= 0) {
			if (num_mask_match > last_valid_pos + 1) {
				is_valid = false;
			}
			else {
				is_valid = true;
			}
		}
		else if (temp_buffer.length() != num_editable_pos) {
			is_valid = false;
		}
	}
	return(is_valid);
}

/**
 * Formats the text.
 * @return
 * @param obj
 */
public String format(Object obj)
{
	// Convert the object to a string
    boolean class_supported = false;
	if (obj != null && (class_supported = isClassSupported(obj.getClass()))) {
		obj = convertToSupported(obj);
	}

	// If the object is null, create empty string
    String text = null;
	if (obj == null || class_supported == false) {
		text = "";
	}
	else {
		text = (String)obj;
	}

    StringBuffer temp_buffer = new StringBuffer(50);
	if (mask != null && mask_length != 0) {
		// If text empty and there is a mask, show the mask & the place holders
		if (text.length() == 0         &&
            place_holder_chars != null &&
			place_holder_chars.length() != 0)
        {
			temp_buffer.append(place_holder_chars);

			for (int i = place_holder_chars.length(); i < mask_length; i++) {
				if (compiled_mask_sym[i] == LITERAL) {
					temp_buffer.append(compiled_mask_ch[i]);
				}
				else {
					addPlaceHolder(temp_buffer, i);
				}
			}
			return(temp_buffer.toString());
		}

		// Loop through the string and format it
        int mask_pos = 0;
		for (int i = 0; i < text.length() && i < num_editable_pos; i++) {
			if (text.charAt(i) == compiled_mask_ch[mask_pos] &&
				compiled_mask_sym[mask_pos] != LITERAL)
			{
				temp_buffer.append(compiled_mask_ch[mask_pos]);
			}
			else {
				// Skip all literals
				for (int j = mask_pos; compiled_mask_sym[j] == LITERAL; j++) {
					temp_buffer.append(compiled_mask_ch[j]);
					mask_pos += 1;
				}
				// Now add the character
				temp_buffer.append(text.charAt(i));
			}
			mask_pos += 1;
		}

		// If not enough input chars, fill in with place holder and literals
		for (int i = mask_pos; i < mask_length; i++) {
			if (compiled_mask_sym[i] == LITERAL) {
				temp_buffer.append(compiled_mask_ch[i]);
			}
			else {
				addPlaceHolder(temp_buffer, i);
			}
		}
		return(temp_buffer.toString());
	}
	return(text);
}

/**
 * Returns the first valid cursor position in the text field.
 * @return first valid cursor position in the text field
 */
public int getFirstValidCursorPosition()
{
	int position;

	// If there is a mask, search for first non LITERAL position
	if (mask != null && mask_length != 0) {
		if ((position = getNextCursorPosition(0, true)) >= 0) {
			return (position);
		}
		else {
			return(mask_length);
		}
	}
	return(0);
}

/**
 * Generates a String representation of the given object for use in edit mode.
 * @param obj
 * @return
 */
public String formatForEdit(Object obj)
{
	return(null);
}

/**
 * Returns <code>true</code> if the validator has a different edit format.
 * @return <code>true</code> if the validator has a different edit format
 */
public boolean hasEditFormat()
{
	return(false);
}


protected Object addIncrement(Object current)
{
	return(null);
}

protected Object subtractIncrement(Object current)
{
	return(null);
}

/**
 * Converts the specified type (if possible) to the type supported by
 * the validator.  To reverse this operation, use the
 * <code>convertFromSupported()</code> method.
 * @see #convertFromSupported
 * @param obj
 * @return
 */
public Object convertToSupported(Object obj)
{
	if (obj instanceof String) {
		return(obj);
	}
	else if (obj instanceof StringBuffer) {
		return(obj.toString());
	}
	return(null);
}

/**
 * Converts the supported type to the type supported by
 * the validator. <br><B>Note:</b> The object passed in is not modified but
 * only used as a reference.
 * @see #convertToSupported
 * @return
 * @param from
 * @param class_type
 */
public Object convertFromSupported(Object from, Class class_type)
{
	if (from == null || class_type == null) {
		return(null);
	}

	String class_name = class_type.getName();

	// if a string, return it.
	if (class_name.equals("java.lang.String")) {
		return(from);
	}
	// if a string buffer, convert it to a string
	else if (class_name.equals("java.lang.StringBuffer")) {
		return(new StringBuffer((String)from));
	}
	return(null);
}

/**
 * Returns the default value for the field.
 * @return default value for the field
 */
public Object getDefaultValue()
{
	return(defaultValue);
}

/**
 * Sets the default value for the field.
 * @param default_value
 */
public void setDefaultValue(Object default_value)
{
	this.defaultValue = default_value;
}

}
