/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCTimeValidator.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:18 $ $Locker:  $ Quest Software Inc. 

package com.klg.jclass.field.validate;

import java.util.*;
import com.klg.jclass.field.resources.LocaleInfo;

/**
 * The <code>JCTimeValidator</code> is used to validate time objects.
 */
public class JCTimeValidator extends JCDateTimeValidator {

/**
 * Constructs a JCTimeValidator.
 */
public JCTimeValidator()
{
    super();
}

/**
 * Constructs a <code>JCTimeValidator</code> with the values specified.
 * @param default_detail
 * @param allow_null
 * @param default_value
 * @param edit_formats
 * @param format
 * @param locale
 * @param mask_input
 * @param place_holder_chars
 */
public JCTimeValidator(Locale locale, String format, String place_holder_chars,
                       String[] edit_formats, boolean mask_input,
                       int default_detail, boolean allow_null,
                       Object default_value)
{

    super(locale, format, place_holder_chars, edit_formats, mask_input,
          default_detail, allow_null, default_value);
}

/**
 * Gets the default <code>EditFormats</code> for the validator.
 * @param l
 * @return
 */
public String[] getDefaultEditFormats(Locale l)
{
    String dtp[] = li.getStringArray(LocaleInfo.DATE_TIME_PATTERNS);
    String formats[] = new String[4];
    String fulltime = dtp[LocaleInfo.FULL_TIME],
        longtime = dtp[LocaleInfo.LONG_TIME],
        medtime = dtp[LocaleInfo.MEDIUM_TIME],
        shorttime = dtp[LocaleInfo.SHORT_TIME];

    formats[0] = fulltime;
    formats[1] = longtime;
    formats[2] = medtime;
    formats[3] = shorttime;

    return (formats);
}

/**
 * Returns the default format according to the current
 * <code>defaultDetail</code>.
 * @param l
 * @return
 * @see JCDateTimeValidator#setDefaultDetail
 */
public String getDefaultFormat(Locale l)
{
    String bundleName = "com.klg.jclass.field.resources.LocaleInfo";
    ResourceBundle li = ResourceBundle.getBundle(bundleName, l);
    String dtp[] = li.getStringArray(LocaleInfo.DATE_TIME_PATTERNS);
    String mask = null;

    switch (default_detail) {
    case FULL:
        mask = dtp[LocaleInfo.FULL_TIME];
        break;
    case LONG:
        mask = dtp[LocaleInfo.LONG_TIME];
        break;
    case MEDIUM:
        mask = dtp[LocaleInfo.MEDIUM_TIME];
        break;
    case SHORT:
        mask = dtp[LocaleInfo.SHORT_TIME];
        break;
    }

    return(mask);
}


}

