/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCValidator.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:18 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.field.validate;

import javax.swing.ListModel;

/**
 * Interface for writing validators.
 */
public interface JCValidator extends java.io.Serializable {

/** Enum for mask returned by <code>calculateSpinability()</code>. */
public static final int UP   = 1;
/** Enum for mask returned by <code>calculateSpinability()</code>. */
public static final int DOWN = 2;

/**
 * Returns the list of values that the end user can pick from.
 * @return list of values the end user can pick from
 */
public ListModel getPickList();

/**
 * Sets the pick list.
 * @param picklist
 */
public void setPickList(ListModel picklist);

/**
 * Called by a <code>TextChangedListener</code>; gives the validator an
 * opportunity to control the change of text.
 * @param e
 */
public void changeText(TextEvent e);

/**
 * Since the <code>clone()</code> method is protected in the <code>Object</code>
 * class, we cannot actually clone a generic object without knowing what it is;
 * therefore the validator must provide this routine to do the copying for
 * Field.
 * <br><B>Note:</b> Objects that are inherently immutable don't need to be
 * copied and hence just return the passed-in value as the implementation.
 * @return
 * @param value
 */
public Object copyValue(Object value);

/**
 * Since we cannot always rely on an object's <code>equals()</code> method we
 * must define out own compare method.  The default implementation of this
 * in the <code>AbstractValidator</code> class is to call <code>equals()</code>
 * If this is not sufficient, then the subclass can override this method without
 * having to subclass the data object.
 * @param v1
 * @param v2
 * @return
 */
public boolean compareValues(Object v1, Object v2);

/**
 * Sets the cursor to the specified position.
 * See {@link TextCursorEvent} for the methods <code>setCursor()</code>
 * would use to reposition the cursor, or to reset its position if
 * validation fails.
 *
 * @param e the event triggering the repositioning of the cursor
 */
public void setCursor(TextCursorEvent e);

/**
 * Validates the specified text, and returns a new Object if successful.
 * @param c sets which class object the text should be converted to
 * @param text text to be converted to an object
 * @return
 * @throws FieldParseException
 */
public Object parse(Class c, String text) throws FieldParseException;

/**
 * Validates the given object.
 * @param obj
 * @return
 */
public boolean validate(Object obj);

/**
 * Generates a String representation of the given object.
 * @return
 * @param obj
 */
public String format(Object obj);

/**
 * Generates a String representation of the given object for use in edit mode.
 * @return
 * @param obj
 */
public String formatForEdit(Object obj);

/**
 * Reports whether the validator has a different edit format or not.
 * @return
 */
public boolean hasEditFormat();

/**
 * Infers the sub-field we need to work on from the current selected region.
 * If there is no selected region then the selected region is equal to the
 * cursor position and we don't ever need to look at the cursor position.
 * @param selection_start
 * @param selection_end
 */
public void inferSubField(int selection_start, int selection_end);

/**
 * If the validator has the ability to "spin," this gives us the next
 * value.
 * @param current
 * @return
 */
public Object spinUp(Object current);

/**
 * If the validator has the ability to "spin," this gives us the prvious
 * value.
 * @param current
 * @return
 */
public Object spinDown(Object current);

/**
 * Finds out whether or not further spinning is possible.
 * Returns a mask of <code>AbstractValidator.UP</code> and/or
 * <code>AbstractValidator.DOWN</code>.
 * @return
 * @param current_value
 */
public int calculateSpinability(Object current_value);

/**
 * Gets the first valid cursor position.
 * @return
 */
public int getFirstValidCursorPosition();

/**
 * Returns the offset into the <code>PickList</code> of the object.
 * @return offset into the <code>PickList</code> of the object
 * @param obj
 */
public int getPickListIndex(Object obj);

/**
 * Returns <code>true</code> if the object passed in is the native type that the
 * validator is designed to support.
 * @return <code>true</code> if the object passed in is the native type
 * @param c
 */
public boolean isClassSupported(Class c);

/**
 * Returns the default value for the validator.
 * @return
 */
public Object getDefaultValue();

/**
 * Returns the <code>PopupFieldEditor</code>.
 * @return the <code>PopupFieldEditor</code>
 */
public PopupFieldEditor createPopupComponent();

}
