/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: TextEvent.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:18 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.field.validate;

/**
 * TextEvent is posted when text has been changed in a field.
 */
public class TextEvent {

protected String   originalText;
protected int      start;
protected int      end;
protected String   newText;
protected boolean  isDeletion;
protected boolean  allowChange = true;

/**
 * Constructs a TextEvent.
 * @param original_text text before insertion
 * @param start starting position to insert text
 * @param end ending position of text
 * @param new_text new text after insertion
 * @param is_deletion are we deleting?
 */
public TextEvent(String original_text, int start, int end, String new_text,
                 boolean is_deletion)
{
    this.originalText = original_text;
    this.start = start;
    this.end = end;
    this.newText = new_text;
    this.isDeletion = is_deletion;
}

/**
 * Returns the orignal text
 *
 * @return The original text
 */
public String getOriginalText()
{
    return(originalText);
}

/**
 * Gets the starting position of the text to modify.
 *
 * @return The start position
 */
public int getStartPosition()
{
    return(start);
}

/**
 * Sets the starting position of the text to modify.
 *
 * @param v The new starting position
 */
public void setStartPosition(int v)
{
    start = v;
}

/**
 * Gets the ending position of the text to modify.
 *
 * @return The end position
 */
public int getEndPosition()
{
    return(end);
}

/**
 * Sets the ending position of the text to modify.
 *
 * @param v The new end position
 */
public void setEndPosition(int v)
{
    end = v;
}

/**
 * Gets the string to be inserted.
 *
 * @return The insertion string
 */
public String getText()
{
    return(newText);
}

/**
 * Sets the string to be inserted.
 *
 * @param v The new string to be inserted
 */
public void setText(String v)
{
    newText = v;
}

/**
 * Returns true if text is being deleted.
 *
 * @return Is text being deleted?
 */
public boolean isDeletion()
{
    return(isDeletion);
}

/**
 * Determines whether the value change be allowed.
 *
 * @return Are changes allowed?
 */
public boolean getAllowChange()
{
    return(allowChange);
}

/**
 * Determines whether the value change be allowed (default: true).
 *
 * @param v Determines whether changes are allowed.
 */
public void setAllowChange(boolean v)
{
    allowChange = v;
}

/**
 * Override toString() to print more meaningful results
 * @return a string representation of the this object
 */
public String toString()
{
    return("original Text:\"" + originalText + "\" new text:\"" + newText +
           "\" start:" + start + " end:" + end + " isDeletion:" + isDeletion +
           " allowChange:" + allowChange);
}

}
