/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CellPosition.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:18 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.awt.Point;
import java.io.Serializable;
import java.util.Vector;

/**
 * A CellPosition object describes a cell by specifying its rowNode
 * and column index.
 */
public class CellPosition implements Serializable {

static final long serialVersionUID = -3658209448685342823L;

protected RowNode row;
protected CellFormat format;
protected Point point = null;
protected int columnIndex;
protected boolean isIndicator;

/** Default Constructor */
public CellPosition(RowNode row, int column, boolean isIndicator) {
    this();
    this.row = row;
    this.columnIndex = column;
    this.isIndicator = isIndicator;
    setColumnIndex(column);
    // this is usually created for a NullGC and OPERATION_GET_CELL or OPERATION_GET_INDICATOR
    this.point = new Point(0, 0);
}

public CellPosition(RowNode row, int column) {
    this(row, column, false);
}

public CellPosition(int x, int y) {
    this();
    this.point = new Point(x, y);
}

public CellPosition() {
    row = null;
    format = null;
    point = null;
    columnIndex = HiGrid.FIRST_EDITABLE_COLUMN;
}    

/**
 * Sets the column index to the indicated column.
 * Based on the index and indicatorType, the CellFormat is also set.
 * @param column the new column index.
 */

public void setColumnIndex(int column) {
    this.columnIndex = column;
    Vector v = (isIndicator) ? row.getRowFormat().getIndicatorFormats() : row.getRowFormat().getDataFormats();
    if (column >= 0 && column < v.size()) {
        this.format = (CellFormat)v.elementAt(column);
    }
    else {
        this.format = null;
    }
}

/**
 * Compares the passed CellPosition object with itself for logical equality.
 *
 * @param cell the CellPosition object to compare to.
 *
 * @return true if the passed object is logically equal to this one.
 */
public boolean isSameAs(CellPosition cell) {
    return  this.row == cell.row &&
            this.format == cell.format &&
            this.columnIndex == cell.columnIndex &&
            this.isIndicator == cell.isIndicator;
    // don't bother checking point, since it is usually for hit-testing           
}

/**
 * Retrieves the cell's rowNode.
 *
 * @return The rowNode.
 */
public RowNode getRowNode() {
    return row;
}

/**
 * Retrieves the cell's cellFormat.
 *
 * @return The cellFormat.
 */
public CellFormat getCellFormat() {
    return format;
}

/**
 * Retrieves the cell's columnIndex.
 *
 * @return The columnIndex.
 */
public int getColumnIndex() {
    return columnIndex;
}

/**
 * Retrieves if the cell is an indicatorFormat.
 *
 * @return True if the cell is an indicatorFormat, false if dataFormat.
 */
public boolean isIndicator() {
    return this.isIndicator;
}

/**
 * Creates and returns a copy of this object.
 * @return a clone of this instance.
 */
protected Object clone() {
    CellPosition cellCopy = new CellPosition();
    cellCopy.row = this.row;
    cellCopy.columnIndex = this.columnIndex;
    cellCopy.format = this.format;
    cellCopy.isIndicator = this.isIndicator;
    cellCopy.point = new Point(this.point.x, this.point.y);
    return(Object) cellCopy;    
}

/**
 * Returns a string representation of the object.
 * @return a string representation of the object.
 */
public String toString() {
    return ""+row+" CellFormat="+format+" Column="+columnIndex+" Indicator="+isIndicator+" Point="+this.point;
}

}
