/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CellStyle.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:18 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.awt.*;
import java.io.Serializable;

import javax.swing.UIManager;
import javax.swing.plaf.ColorUIResource;
import javax.swing.plaf.FontUIResource;

import com.klg.jclass.cell.*;

/**
 * The <CODE>CellStyle</CODE> class is an implementation of the CellStyleModel
 * interface and provides <CODE>CellRenderer</CODE>
 * and <CODE>CellEditor</CODE> objects formatting information that can
 * be used in drawing.
 */
public class CellStyle implements CellStyleModel, Serializable {

static final long serialVersionUID = -6376210120084557576L;

// Properties
protected Color backgroundColor;
protected Color foregroundColor;
protected Color selectedBackgroundColor;
protected Color selectedForegroundColor;
protected int borderStyle;
protected Insets borderInsets;
protected int verticalAlignment;
protected int horizontalAlignment;
protected int editWidthPolicy;
protected int editHeightPolicy;
protected Insets marginInsets;
protected Font font;
protected boolean allowWidthSizing;
protected boolean selectAll;
protected int clipHints;

public CellStyle() {
    setBackground(new ColorUIResource(UIManager.getColor("Table.background")));
    setForeground(new ColorUIResource(UIManager.getColor("Table.foreground")));
    setSelectedBackground(new ColorUIResource(UIManager.getColor("Table.selectionBackground")));
    setSelectedForeground(new ColorUIResource(UIManager.getColor("Table.selectionForeground")));
    setFont(new FontUIResource(UIManager.getFont("Table.font")));
    setBorderStyle(HiGridBorder.IN);
    setBorderInsets(new Insets(2, 2, 2, 2));
    setMarginInsets(new Insets(2, 2, 2, 2));
    setVerticalAlignment(JCCellInfo.CENTER);
    setHorizontalAlignment(JCCellInfo.LEFT);
    setEditWidthPolicy(HiGrid.EDIT_SIZE_TO_CELL);
    setEditHeightPolicy(HiGrid.EDIT_ENSURE_MINIMUM_SIZE);
    setAllowWidthSizing(true);
    setSelectAll(true);
    setClipHints(JCCellInfo.SHOW_ALL);
}

/**
 * Copies the passed cellStyle's PLAF attributes into this CellStyle.
 *
 * @param cellStyle the CellStyle to initialize this CellStyle with.
 */
public void copyPlafFromCellStyle(CellStyleModel cellStyle) {
    if (getBackground() instanceof ColorUIResource) {
        setBackground(cellStyle.getBackground());
    }
    if (getForeground() instanceof ColorUIResource) {
        setForeground(cellStyle.getForeground());
    }
    if (getSelectedBackground() instanceof ColorUIResource) {
        setSelectedBackground(cellStyle.getSelectedBackground());
    }
    if (getSelectedForeground() instanceof ColorUIResource) {
        setSelectedForeground(cellStyle.getSelectedForeground());
    }
    if (getFont() instanceof FontUIResource) {
        setFont(cellStyle.getFont());
    }

}

/**
 * Copies the passed cellStyle's attributes into this CellStyle.
 *
 * @param cellStyle the CellStyle to initialize this CellStyle with.
 */
public void copyFromCellStyle(CellStyleModel cellStyle) {
    setBackground(cellStyle.getBackground());
    setForeground(cellStyle.getForeground());
    setSelectedBackground(cellStyle.getSelectedBackground());
    setSelectedForeground(cellStyle.getSelectedForeground());
    setBorderStyle(cellStyle.getBorderStyle());
    setBorderInsets((Insets)(cellStyle.getBorderInsets().clone()));
    setMarginInsets((Insets)(cellStyle.getMarginInsets().clone()));
    setVerticalAlignment(cellStyle.getVerticalAlignment());
    setHorizontalAlignment(cellStyle.getHorizontalAlignment());
    setEditWidthPolicy(cellStyle.getEditWidthPolicy());
    setEditHeightPolicy(cellStyle.getEditHeightPolicy());
    setFont(cellStyle.getFont());
    setAllowWidthSizing(cellStyle.getAllowWidthSizing());
    setSelectAll(cellStyle.getSelectAll());
    setClipHints(cellStyle.getClipHints());
}

/**
 * Retrieves the background color for this object.
 *
 * @return The current background color.
 */
public Color getBackground() {
    return backgroundColor;
}

/**
 * Sets the background color for this object.
 *
 * @param color The new background color.
 * @exception IllegalArgumentException If parameter is null.
 */
public void setBackground(Color color) {
    if (color == null) {
        throw new IllegalArgumentException();
    }
    backgroundColor = color;
}

/**
 * Retrieves the foreground color for this object.
 *
 * @return The current foreground color.
 */
public Color getForeground() {
    return foregroundColor;
}

/**
 * Sets the foreground color for this object.
 *
 * @param color The new foreground color.
 * @exception IllegalArgumentException If parameter is null.
 */
public void setForeground(Color color) {
    if (color == null) {
        throw new IllegalArgumentException();
    }
    foregroundColor = color;
}

/**
 * Retrieves the selected background color for this object.
 *
 * @return The current selected background color.
 */
public Color getSelectedBackground() {
    return selectedBackgroundColor;
}

/**
 * Sets the selected background color for this object.
 *
 * @param color The new selected background color.
 * @exception IllegalArgumentException If parameter is null.
 */
public void setSelectedBackground(Color color) {
    if (color == null) {
        throw new IllegalArgumentException();
    }
    selectedBackgroundColor = color;
}

/**
 * Retrieves the selected foreground color for this object.
 *
 * @return The current selected foreground color.
 */
public Color getSelectedForeground() {
    return selectedForegroundColor;
}

/**
 * Sets the selected foreground color for this object.
 *
 * @param color The new selected foreground color.
 * @exception IllegalArgumentException If parameter is null.
 */
public void setSelectedForeground(Color color) {
    if (color == null) {
        throw new IllegalArgumentException();
    }
    selectedForegroundColor = color;
}

/**
 * Retrieves the border style for this object.
 *
 * @return The current border style.
 */
public int getBorderStyle() {
    return borderStyle;
}

/**
 * Sets the border style for this object.
 *
 * @param style The new border style.
 */
public void setBorderStyle(int style) {
    borderStyle = style;
}

/**
 * Retrieves the border insets for this object.
 *
 * @return The current border insets.
 */
public Insets getBorderInsets() {
    return borderInsets;
}

/**
 * Sets the border insets for this object.
 *
 * @param size The new border insets.
 */
public void setBorderInsets(Insets insets) {
    borderInsets = insets;
}

/**
 * Retrieves the vertical alignment for this object.
 *
 * @return The current vertical alignment.
 */
public int getVerticalAlignment() {
    return verticalAlignment;
}

/**
 * Sets the vertical alignment for this object.
 *
 * @param alignment The new vertical alignment.
 */
public void setVerticalAlignment(int alignment) {
    verticalAlignment = alignment;
}

/**
 * Retrieves the horizontal alignment for this object.
 *
 * @return The current horizontal alignment.
 */
public int getHorizontalAlignment() {
    return horizontalAlignment;
}

/**
 * Sets the horizontal alignment for this object.
 *
 * @param alignment The new horizontal alignment.
 */
public void setHorizontalAlignment(int alignment) {
    horizontalAlignment = alignment;
}

/**
 * Retrieves the edit width policy for this object.
 *
 * @return The current edit width policy.
 */
public int getEditWidthPolicy() {
    return editWidthPolicy;
}

/**
 * Sets the edit width policy for this object.
 *
 * @param policy The new edit width policy.
 */
public void setEditWidthPolicy(int policy) {
    editWidthPolicy = policy;
}

/**
 * Retrieves the edit height policy for this object.
 *
 * @return The current edit height policy.
 */
public int getEditHeightPolicy() {
    return editHeightPolicy;
}

/**
 * Sets the edit height policy for this object.
 *
 * @param policy The new edit height policy.
 */
public void setEditHeightPolicy(int policy) {
    editHeightPolicy = policy;
}

/**
 * Retrieves the margin insets for this object.
 *
 * @return The current margin insets.
 */
public Insets getMarginInsets() {
    return marginInsets;
}

/**
 * Sets the margin insets for this object.
 *
 * @param insets The new margin insets.
 */
public void setMarginInsets(Insets insets) {
    marginInsets = insets;
}

/**
 * Retrieves the font for this object.
 *
 * @return The current font.
 */
public Font getFont() {
    return font;
}

/**
 * Sets the font for this object.
 *
 * @param font The new font.
 * @exception IllegalArgumentException If parameter is null.
 */
public void setFont(Font font) {
    if (font == null) {
        throw new IllegalArgumentException();
    }
    this.font = font;
}

/**
 * Retrieves whether width sizing is currently allowed.
 *
 * @return Whether width sizing is currently allowed.
 */
public boolean getAllowWidthSizing() {
    return allowWidthSizing;
}

/**
 * Sets whether width sizing is currently allowed.
 *
 * @param allow Whether to allow width sizing.
 */
public void setAllowWidthSizing(boolean allow) {
    allowWidthSizing = allow;
}

/**
 * Sets whether width sizing is currently allowed
 * for the other header/record pair.
 *
 * @param allow Whether to allow width sizing.
 */
public void setOtherAllowWidthSizing(boolean allow) {
    allowWidthSizing = allow;
}

/**
 * Determines whether the editor should select the contents before editing.
 */
public boolean getSelectAll() {
    return selectAll;
}

/**
 * Determines whether the editor should select the contents before editing.
 */
public void setSelectAll(boolean selectAll) {
    this.selectAll = selectAll;
}

/**
 * Determines whether the renderer should draw some indication if the
 * entire contents of the cell cannot be rendered within the given area.
 */
public int getClipHints() {
    return clipHints;
}

/**
 * Determines whether the renderer should draw some indication if the
 * entire contents of the cell cannot be rendered within the given area.
 */
public void setClipHints(int clipHints) {
    this.clipHints = clipHints;
}

}
