/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CellStyleModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:18 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.awt.*;

/**
 * The <CODE>CellStyleModel</CODE> class provides <CODE>CellRenderer</CODE>
 * and <CODE>CellEditor</CODE> objects formatting information that can
 * be used in drawing.
 */
public interface CellStyleModel extends Cloneable {

/**
 * Retrieves the background color for this object.
 *
 * @return The current background color.
 */
public Color getBackground();

/**
 * Sets the background color for this object.
 *
 * @param color The new background color.
 * @exception IllegalArgumentException If parameter is null.
 */
public void setBackground(Color color);

/**
 * Retrieves the foreground color for this object.
 *
 * @return The current foreground color.
 */
public Color getForeground();

/**
 * Sets the foreground color for this object.
 *
 * @param color The new foreground color.
 * @exception IllegalArgumentException If parameter is null.
 */
public void setForeground(Color color);

/**
 * Retrieves the selected background color for this object.
 *
 * @return The current selected background color.
 */
public Color getSelectedBackground();

/**
 * Sets the selected background color for this object.
 *
 * @param color The new selected background color.
 * @exception IllegalArgumentException If parameter is null.
 */
public void setSelectedBackground(Color color);

/**
 * Retrieves the selected foreground color for this object.
 *
 * @return The current selected foreground color.
 */
public Color getSelectedForeground();

/**
 * Sets the selected foreground color for this object.
 *
 * @param color The new selected foreground color.
 * @exception IllegalArgumentException If parameter is null.
 */
public void setSelectedForeground(Color color);

/**
 * Retrieves the border style for this object.
 *
 * @return The current border style.
 */
public int getBorderStyle();

/**
 * Sets the border style for this object.
 *
 * @param style The new border style.
 */
public void setBorderStyle(int style);

/**
 * Retrieves the border insets for this object.
 *
 * @return The current border insets.
 */
public Insets getBorderInsets();

/**
 * Sets the border insets for this object.
 *
 * @param size The new border insets.
 */
public void setBorderInsets(Insets insets);

/**
 * Retrieves the vertical alignment for this object.
 *
 * @return The current vertical alignment.
 */
public int getVerticalAlignment();

/**
 * Sets the vertical alignment for this object.
 *
 * @param alignment The new vertical alignment.
 */
public void setVerticalAlignment(int alignment);

/**
 * Retrieves the horizontal alignment for this object.
 *
 * @return The current horizontal alignment.
 */
public int getHorizontalAlignment();

/**
 * Sets the horizontal alignment for this object.
 *
 * @param alignment The new horizontal alignment.
 */
public void setHorizontalAlignment(int alignment);

/**
 * Retrieves the edit width policy for this object.
 *
 * @return The current edit width policy.
 */
public int getEditWidthPolicy();

/**
 * Sets the edit width policy for this object.
 *
 * @param policy The new edit width policy.
 */
public void setEditWidthPolicy(int policy);

/**
 * Retrieves the edit height policy for this object.
 *
 * @return The current edit height policy.
 */
public int getEditHeightPolicy();

/**
 * Sets the edit height policy for this object.
 *
 * @param policy The new edit height policy.
 */
public void setEditHeightPolicy(int policy);

/**
 * Retrieves the margin insets for this object.
 *
 * @return The current margin insets.
 */
public Insets getMarginInsets();

/**
 * Sets the margin insets for this object.
 *
 * @param insets The new margin insets.
 */
public void setMarginInsets(Insets insets);

/**
 * Retrieves the font for this object.
 *
 * @return The current font.
 */
public Font getFont();

/**
 * Sets the font for this object.
 *
 * @param font The new font.
 * @exception IllegalArgumentException If parameter is null.
 */
public void setFont(Font font);

/**
 * Retrieves whether width sizing is currently allowed.
 *
 * @return Whether width sizing is currently allowed.
 */
public boolean getAllowWidthSizing();

/**
 * Sets whether width sizing is currently allowed.
 *
 * @param allow Whether to allow width sizing.
 */
public void setAllowWidthSizing(boolean allow);

/**
 * Sets whether width sizing is currently allowed
 * for the other header/record pair.
 *
 * @param allow Whether to allow width sizing.
 */
public void setOtherAllowWidthSizing(boolean allow);

/**
 * Determines whether the editor should select the contents before editing.
 */
public boolean getSelectAll();

/**
 * Determines whether the editor should select the contents before editing.
 */
public void setSelectAll(boolean selectAll);

/**
 * Determines whether the renderer should draw some indication if the
 * entire contents of the cell cannot be rendered within the given area.
 */
public int getClipHints();

/**
 * Determines whether the renderer should draw some indication if the
 * entire contents of the cell cannot be rendered within the given area.
 */
public void setClipHints(int clipHints);

/**
 * Replaces cellStyleModel attributes with the passed one.
 */
public void copyFromCellStyle(CellStyleModel cellStyleModel);

/**
 * Replaces PLAF cellStyleModel attributes with the passed one,
 * only if the PLAF attributes were not already overridden.
 */
public void copyPlafFromCellStyle(CellStyleModel cellStyleModel);

}



