/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Controller.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:18 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.text.JTextComponent;
import javax.swing.FocusManager;
import javax.swing.border.*;
import javax.swing.tree.*;

import com.klg.jclass.cell.*;
import com.klg.jclass.cell.JCCellTextEditor;
import com.klg.jclass.cell.validate.*;
import com.klg.jclass.datasource.*;
import com.klg.jclass.util.JCEnvironment;
import com.klg.jclass.util.JCUtil;


/**
 * The <code>Controller</code> class performs the Controller function in the
 * Model-View-Controller (MVC) paradigm. It handles all mouse and keyboard input
 * and positions the CellEditors in the GridArea.
 */
public class Controller implements Serializable, JCCellEditorListener, KeyListener,
    MouseListener, MouseMotionListener {

static final long serialVersionUID = 4304445072571427191L;

// Resizing constants
public static final int MINIMUM_COLUMN_WIDTH = 8;
public static final int MINIMUM_ROW_HEIGHT = 8;
public static final int SELECT_TOLERANCE = 2;

// States
// Column resizing
protected boolean resizingColumn = false;
protected int columnXResize = 0;
protected int xResizeOffset = -1;
protected int columnIndexResize = -1;

// Row resizing
protected boolean resizingRow = false;
protected int rowYResize = 0;
protected int yResizeOffset = -1;
protected RowNode resizeLevel = null;

// Row/Column resizing
//FIX - this used to be XOR - now a problem if background color of cells is black!
protected JPanel resizer = null;
protected CellPosition resizeCell = null;

// Column moving
protected boolean movingColumn = false;
protected JColumnHeaderLabel columnLabel = null;
protected boolean mouseExited = false;
protected Point columnOrigin = null;

// Column selecting
protected boolean selectingColumn = false;
protected CellPosition selectedCell = null;
protected Point originalMousePress = null;

// Current cell being edited
protected CellPosition currentCell = null;
protected JCCellEditor currentCellEditor = null;
protected boolean inEditMode = false;

// Settable Properties
protected int desiredColumn = 0;

// row selection properties
protected RowNode lastSelectedRowNode = null;
protected Vector selectedRowNodes = null;

// map mouse and key actions
protected Vector actions;
protected int currentAction = HiGridAction.NO_ACTION;

// indicate to listeners that the editor is currently moving to another cell
protected boolean duringTraversal = false;

// implements HiGrid autoEdit property
protected Class nullEditor = null;

protected HiGridFocusAdapter focusListener = null;
private boolean fNewFocusModel = false;

private HiGrid grid;

// Constructor
public Controller(HiGrid grid) {
    this.grid = grid;
    removeAllActions();
    createDefaultActions();
    selectedRowNodes = new Vector(10);
    try {
        nullEditor = Class.forName("com.klg.jclass.higrid.NullCellEditor");
    }
    catch (Exception e) {
        e.printStackTrace();
    }

    fNewFocusModel = (JCEnvironment.getJavaVersion() >= 140);
    String focusListenerString = (fNewFocusModel) ?
                        "com.klg.jclass.higrid.HiGridFocusNew"
                        : "com.klg.jclass.higrid.HiGridFocusOld";
    try {
        Class c = Class.forName(focusListenerString);
        focusListener = (HiGridFocusAdapter)c.newInstance();
        focusListener.setGrid(grid);
    }
    catch (Exception e) {
        System.out.println("FocusListener initialization error:" +
                           e.getMessage());
    }
}

/**
 * Retrieves current GridArea reference from the HiGrid instance
 *
 * @return current GridArea reference from the HiGrid instance
 */
private GridArea getGridArea() {
    return(GridArea)grid.getGridArea();
}

/**
 * Retrieves whether we are currently traversing
 *
 * @return whether we are currently traversing
 */
protected boolean isTraversing() {
    return duringTraversal;
}

/**
 * Retrieves the current edit cell
 *
 * @return The current cell position
 */
protected CellPosition getCurrentCell() {
    return currentCell;
}

/**
 * Sets the current edit cell
 *
 * @param pos The current cell position
 */
protected void setCurrentCell(CellPosition pos) {
    currentCell = pos;
    if (currentCell == null) {
        removeEditor();
    }
}

/**
 * Retrieves the desired column.
 *
 * @return The desired column.
 */
protected int getDesiredColumn() {
    return desiredColumn;
}

/**
 * Sets the desired column.
 *
 * @param column The desired column.
 */
protected void setDesiredColumn(int column) {
    column = Math.max(grid.FIRST_EDITABLE_COLUMN, column);
    desiredColumn = column;
}

/**
 * Determines whether the specified key modifier represents an
 * attempt by an editor to reserve a key that Table reserves
 * for itself.  We reserve Tab and Escape only.
 * We will get the rest if the editor doesn't want them.
 */
protected boolean containerReserved(JCKeyModifier km) {
    if (km.match(KeyEvent.VK_ESCAPE, 0)) {
        return true;
    }
    if (km.match(KeyEvent.VK_LEFT, 0)) {
        return true;
    }
    if (km.match(KeyEvent.VK_RIGHT, 0)) {
        return true;
    }
    if (km.match(KeyEvent.VK_HOME, 0)) {
        return true;
    }
    if (km.match(KeyEvent.VK_END, 0)) {
        return true;
    }
    if (km.match(KeyEvent.VK_TAB, 0)) {
        return true;
    }
    if (km.match(KeyEvent.VK_TAB, InputEvent.SHIFT_MASK)) {
        return true;
    }
    return false;
}

/**
 * Determines whether the specified key and modifier represents a
 * combination that HiGrid should process, or whether it is
 * reserved by a CellEditor.
 */
protected boolean shouldProcessThisKey(int key, int modifiers) {
    if (currentCellEditor == null) {
        return false;
    }
    JCKeyModifier[] keycodes = currentCellEditor.getReservedKeys();
    if (keycodes != null && keycodes.length > 0) {
        for (int i = 0; i < keycodes.length; i++) {
            // The editor can reserve other keys for itself
            if (keycodes[i] == null) {
                continue;
            }
            if (keycodes[i].match(key, modifiers)) {
                // We reserve some keys for ourselves
                if (!inEditMode && containerReserved(keycodes[i])) {
                    continue;
                }
                return false;
            }
        }
    }
    return true;
}

/**
 * Processes the KeyEvent from the user.
 * @param e KeyEvent created by user keyPress.
 */
protected void processKey(KeyEvent e) {
    int key = e.getKeyCode();
    int modifiers = e.getModifiers();
    if (! shouldProcessThisKey(key, modifiers)) {
        return;
    }
    RowTree rowTree = grid.getRowTree();
    boolean changed = false;
    boolean needToContinue = true;
    CellPosition newPosition = null;
    int newDesiredColumn = getDesiredColumn();
    int action = getAction(e);
    setCurrentAction(action);
    switch (action) {
    // these keys will be handled by the cell editor; no movement required
    case HiGridAction.MOVE_UP_ACTION:
        newPosition = goUp();
        break;
    case HiGridAction.MOVE_DOWN_ACTION:
        newPosition = goDown();
        break;
    case HiGridAction.MOVE_LEFT_ACTION:
        newPosition = goLeft();
        if (newPosition != null) {
            newDesiredColumn = getVisibleIndex(newPosition);
        }
        break;
    case HiGridAction.MOVE_RIGHT_ACTION:
        newPosition = goRight();
        if (newPosition != null) {
            newDesiredColumn = getVisibleIndex(newPosition);
        }
        break;
    case HiGridAction.MOVE_PAGE_UP_ACTION:
        if (validateCell(null)) {
            // one page up
            grid.getVerticalScrollbar().scrollPageUp();
            // we've gone up a page, now find the first available record to land on
            newPosition = doTheRecordWalk(rowTree.getTopRowNode(), 0, true);
        }
        if (newPosition != null) {
            newPosition.columnIndex = getActualIndex(newPosition.row.getRowFormat(), newDesiredColumn);
        }
        break;
    case HiGridAction.MOVE_PAGE_DOWN_ACTION:
        if (validateCell(null)) {
            grid.getVerticalScrollbar().scrollPageDown();
            // we've gone down a page, now find the first available record to land on
            newPosition = doTheRecordWalk(rowTree.getTopRowNode(), 0, true);
        }
        if (newPosition != null) {
            newPosition.columnIndex = getActualIndex(newPosition.row.getRowFormat(), newDesiredColumn);
        }
        break;
    case HiGridAction.MOVE_PAGE_LEFT_ACTION:
        if (validateCell(null)) {
            // one page to the left
            grid.getHorizontalScrollbar().scrollPageUp();
        }
        break;
    case HiGridAction.MOVE_PAGE_RIGHT_ACTION:
        if (validateCell(null)) {
            // one page to the right
            grid.getHorizontalScrollbar().scrollPageDown();
        }
        break;
    case HiGridAction.MOVE_HOME_ACTION:
        // the 0 will be overridden in the next statement
        newPosition = new CellPosition(currentCell.row, currentCell.row.getRowFormat().getFirstVisibleCell());
        if (newPosition.columnIndex < 0) {
            needToContinue = false;
            break;
        }
        newDesiredColumn = getVisibleIndex(newPosition);
        // try to show the folder icons, if possible
        // this is safe, since code later on will scroll it back if necessary
        grid.getHorizontalScrollbar().setValue(0);
        break;
    case HiGridAction.MOVE_END_ACTION:
        // the 0 will be overridden in the next statement
        newPosition = new CellPosition(currentCell.row, 0);
        newPosition = getLastCell(newPosition.row);
        newDesiredColumn = getVisibleIndex(newPosition);
        break;
    case HiGridAction.MOVE_GRID_HOME_ACTION:
        // all the way to the first record row
        newPosition = goHome();
        if (newPosition.columnIndex < 0) {
            needToContinue = false;
            break;
        }
        newDesiredColumn = getVisibleIndex(newPosition);
        // try to show the folder icons, if possible
        // this is safe, since code later on will scroll it back if necessary
        grid.getHorizontalScrollbar().setValue(0);
        break;
    case HiGridAction.MOVE_GRID_END_ACTION:
        // all the way to the last row
        // start at the last top-level cell and walk down to the end
        newPosition = goEnd();
        newDesiredColumn = getVisibleIndex(newPosition);
        break;
    case HiGridAction.MOVE_NEXT_ACTION:
        newPosition = goRight();
        if (newPosition == null) {  // at the end of the line; go down
            newPosition = goDown();
            if (newPosition != null) {  // go to the first column
                newPosition = getFirstCell(newPosition.row);
            }
        }
        if (newPosition != null) {
            newDesiredColumn = getVisibleIndex(newPosition);
        }
        break;
    case HiGridAction.MOVE_PREVIOUS_ACTION:
        newPosition = goLeft();
        if (newPosition == null) {  // at the beginning of the line; go up
            newPosition = goUp();
            if (newPosition != null) {  // go to the last column
                newPosition = getLastCell(newPosition.row);
            }
        }
        if (newPosition != null) {
            newDesiredColumn = getVisibleIndex(newPosition);
        }
        break;
    case HiGridAction.OPEN_TABLE_ACTION:
        // open this folder
        if (currentCell.row.getState() == NodeStatus.FOLDER_CLOSED) {
            expandFolder(currentCell.row);
        }
        break;
    case HiGridAction.CLOSE_TABLE_ACTION:
        // close this folder
        if (currentCell.row.getState() == NodeStatus.FOLDER_OPEN) {
            collapseFolder(currentCell.row);
        }
        break;
    case HiGridAction.COLUMN_SELECT_ACTION:
        grid.fireHiGridColumnSelectionEvent(new HiGridColumnSelectionEvent(grid,
            HiGridColumnSelectionEvent.SELECT_COLUMN, currentCell.row, currentCell, e));
        needToContinue = false;
        break;
    case HiGridAction.CANCEL_EDIT_ACTION:
        cancelCellEditing();
        break;
    case HiGridAction.STOP_EDIT_ACTION:
        stopCellEditing();
        break;
    case HiGridAction.IGNORE_ACTION:
        needToContinue = false;
        break;
    case HiGridAction.POPUP_MENU_ACTION:
        showPopupMenu(e);
        needToContinue = false;
        break;
    case HiGridAction.SINGLE_ROW_SELECT_ACTION:
    case HiGridAction.GROUP_ROW_SELECT_ACTION:
        if (grid.isAllowRowSelection()) {
            doRowSelection(currentCell.row, grid.getRowSelectionMode(),
                           action == HiGridAction.GROUP_ROW_SELECT_ACTION);
        }
        needToContinue = false;
        break;
    case HiGridAction.COPY_ACTION:
        performCopyAction();
        needToContinue = false;
        break;
    case HiGridAction.PASTE_ACTION:
        performPasteAction();
        needToContinue = false;
        break;
    case HiGridAction.NO_ACTION:
    default:
        if (!grid.isEditable() || !currentCell.format.isEditable()) {
            // this cell is read-only; don't let the editor see these keys
//            e.consume();
            inEditMode = false;
        }
        // all other keys will cause us to make sure the editor is visible
        makeCellEditorVisible(action);
        // normal input characters handled here
        if (currentCellEditor instanceof NullCellEditor) {
            popupEditor(e);
            acquireFocusForCellEditor();

            // In Java2 1.4 or later, the first KEY_TYPED event is not
            // received by the cell editor.  So for the first input
            // character, we explicitly set the cell editor's text.
            if (JCEnvironment.getJavaVersion() >= 140 &&
                currentCellEditor instanceof JCCellTextEditor) {
                char keyChar = e.getKeyChar();
                if (keyChar != KeyEvent.CHAR_UNDEFINED) {
                    String input = null;
                    if (keyChar == KeyEvent.VK_BACK_SPACE) {
                        input = new String();
                    }
                    else {
                        input = String.valueOf(keyChar);
                    }
                    ((JCCellTextEditor)currentCellEditor).setCellEditorText(input);
                }
            }
        }
        else {
            acquireFocusForCellEditor();
        }

        needToContinue = false;
        break;
    }
    if (needToContinue) {
        // we got here due to a key used for grid navigation purposes
        // eat events that are not meant for the JCCellEditor or the focus manager
        e.consume();
        // recreate the CellPosition object to ensure the CellFormat is correct
        if (newPosition != null) {
            newPosition = new CellPosition(newPosition.row, newPosition.columnIndex);
        }
        // user wants to move and/or validate what has been typed in
        validateAndTraverse(newPosition, newDesiredColumn, e);
    }
    setCurrentAction(HiGridAction.NO_ACTION);
}

/*
  It seems that Internet Explorer V4.01 SP1 has the following bug:
  The PAGE_UP and PAGE_DOWN keys are not sent to the keyPressed()
  method when a cellEditor (a TextField) has the focus.
  However, for some reason, it does call the keyReleased() method.
  So, the lastKeyProcessed variable has been added to handle
  keys that slip through the cracks.
*/

protected int lastKeyProcessed = KeyEvent.VK_UNDEFINED;

/**
 * Sets the HiGridAction (if any) that is being currently processed.
 *
 * @param action the current HiGridAction being processed
 */
protected void setCurrentAction(int action) {
    currentAction = action;
}

/**
 * Retrieves the HiGridAction (if any) that is being currently processed.
 *
 * @return action the current HiGridAction being processed.
 * <code>HiGridAction.NO_ACTION</code> is returned if no action is being processed.
 */
public int getCurrentAction() {
    return currentAction;
}

//////////////////////
// KeyListener methods
public void keyPressed(KeyEvent e) {
    lastKeyProcessed = e.getKeyCode();
    processKey(e);
}

public void keyReleased(KeyEvent e) {
    int key = e.getKeyCode();
    // Internet Explorer only sends us PageUp and PageDown keyRelease
    // events, rather than the keyPressed events
//FIXOBSOLETE
/*
if (key != lastKeyProcessed &&
    ((key == KeyEvent.VK_PAGE_DOWN) || (key == KeyEvent.VK_PAGE_UP)) &&
    JCEnvironment.isBrowserExplorer(grid)) {
    processKey(e);
}
*/
    lastKeyProcessed = KeyEvent.VK_UNDEFINED;
}

// these events are ignored
public void keyTyped(KeyEvent e) {
}

/**
 * Retrieves the visible column index associated with the given cell.
 *
 * @param newPosition the new CellPosition being traversed to.
 * @return the visible column index associated with the given cell.
 */
public int getVisibleIndex(CellPosition newPosition) {
    return newPosition.row.getRowFormat().getVisibleColumnIndex(newPosition.columnIndex);
}

/**
 * Retrieves an actual column index given a desired column.
 * Due to cell visibility, the desired column may not be always possible.
 *
 * @param newRowFormat the new row to move to
 * @param newDesiredColumn the column to move to
 * @return the actual column index that can be moved to.
 */
public int getActualIndex(RowFormat newRowFormat, int newDesiredColumn) {
    int desiredColumn = Math.min(newDesiredColumn, newRowFormat.getVisibleColumnCount() - 1);
    return newRowFormat.getActualColumnIndex(desiredColumn);
}

/**
 * Traverse to the first cell in the current row.
 * @return the new cell to move to.
 */
protected CellPosition goHome() {
    return getFirstCell(null);
}

/**
 * Traverse to the last cell in the current row.
 * @return the new cell to move to.
 */
protected CellPosition goEnd() {
    // start at the last top-level cell and walk down to the end
    CellPosition newPosition = getLastCell(null);
    RowNode row = newPosition.row;
    LastRowWalk lrWalkable = new LastRowWalk(true);
    grid.getRowTree().walk(row, lrWalkable);
    newPosition = getLastCell(lrWalkable.getFoundNode());
    return newPosition;
}

/**
 * Traverse to the same column in the previous row.
 * @return the new cell to move to.
 */
protected CellPosition goUp() {
    // attempted optimization for simple row walking
    RowNode rowNode = currentCell.row;
    RowNode previousSibling = (RowNode)rowNode.getPreviousSibling();
    if ((previousSibling != null) && (previousSibling.getState() == NodeStatus.FOLDER_CLOSED) &&
        (previousSibling.getRowFormat() instanceof RecordFormat)) {
        CellPosition cell = new CellPosition(previousSibling, currentCell.columnIndex);
        return cell;
    }
    CellPosition newCell = doTheRecordWalk(currentCell.row, 1, false);
    if (newCell != null) {
        if (newCell.row == currentCell.row) {
            return null;
        }
        newCell.columnIndex = getActualIndex(newCell.row.getRowFormat(), getDesiredColumn());
    }
    return newCell;
}

/**
 * Traverse to the same column in the next row.
 * @return the new cell to move to.
 */
protected CellPosition goDown() {
    // attempted optimization for simple row walking
    RowNode rowNode = currentCell.row;
    if (rowNode.getState() == NodeStatus.FOLDER_CLOSED) {
        RowNode nextSibling = (RowNode)rowNode.getNextSibling();
        if ((nextSibling != null) && (nextSibling.getRowFormat() instanceof RecordFormat)) {
            CellPosition cell = new CellPosition(nextSibling, currentCell.columnIndex);
            return cell;
        }
    }
    CellPosition newCell = doTheRecordWalk(currentCell.row, 1, true);
    if (newCell != null) {
        if (newCell.row == currentCell.row) {
            return null;
        }
        newCell.columnIndex = getActualIndex(newCell.row.getRowFormat(), getDesiredColumn());
    }
    return newCell;
}

/**
 * Traverse to the previous column in the same row.
 * @return the new cell to move to.
 */
protected CellPosition goLeft() {
    CellPosition newPosition = null;
    int index = currentCell.columnIndex;
    while (index > grid.FIRST_EDITABLE_COLUMN) {
        index--;
        if (((CellFormat)currentCell.row.getRowFormat().getDataFormats().elementAt(index)).isShowing()) {
            newPosition = new CellPosition(currentCell.row, index);
            break;
        }
    }
    return newPosition;
}

/**
 * Traverse to the next column in the same row.
 * @return the new cell to move to.
 */
protected CellPosition goRight() {
    CellPosition newPosition = null;
    int index = currentCell.columnIndex;
    int limit = currentCell.row.getRowFormat().getDataFormats().size() - 1;
    while (index < limit) {
        index++;
        if (((CellFormat)currentCell.row.getRowFormat().getDataFormats().elementAt(index)).isShowing()) {
            newPosition = new CellPosition(currentCell.row, index);
            break;
        }
    }
    return newPosition;
}

/**
 * Traverse to the first row of the table associated with the passed row.
 * @return the new cell to move to.
 */
protected CellPosition goLevelHome(RowNode rowNode) {
    return goLevelUp(rowNode, false);
}

/**
 * Traverse to the first row of the table associated with the current row.
 * @return the new cell to move to.
 */
protected CellPosition goLevelHome() {
    return goLevelHome(currentCell.row);
}

/**
 * Traverse to the previous row of the table associated with the current row.
 * @return the new cell to move to.
 */
protected CellPosition goLevelUp() {
    return goLevelUp(currentCell.row, true);
}

/**
 * Traverse to the previous row of the table associated with the passed row.
 * @return the new cell to move to.
 */
protected CellPosition goLevelUp(RowNode rowNode, boolean upOneOnly) {
    RowNode finalRowNode = rowNode;
    FormatNode currentFormatNode = rowNode.getFormatNode();
    while (true) {
        rowNode = (RowNode)rowNode.getPreviousSibling();
        if (rowNode == null) {
            break;
        }
        if (rowNode.getRowFormat() instanceof RecordFormat) {
            if (rowNode.getFormatNode() == currentFormatNode) {
                finalRowNode = rowNode;
                if (upOneOnly) {
                    break;
                }
            }
            else {
                break;
            }
        }
    }
    return new CellPosition(finalRowNode,
                            getActualIndex(finalRowNode.getRowFormat(), getDesiredColumn()));
}

/**
 * Traverse to the last row of the table associated with the passed row.
 * @return the new cell to move to.
 */
protected CellPosition goLevelEnd(RowNode rowNode) {
    return goLevelDown(rowNode, false);
}

/**
 * Traverse to the previous row of the table associated with the current row.
 * @return the new cell to move to.
 */
protected CellPosition goLevelEnd() {
    return goLevelDown(currentCell.row, false);
}

/**
 * Traverse to the next row of the table associated with the current row.
 * @return the new cell to move to.
 */
protected CellPosition goLevelDown() {
    return goLevelDown(currentCell.row, true);
}

/**
 * Traverse to the next row of the table associated with the passed row.
 * @return the new cell to move to.
 */
protected CellPosition goLevelDown(RowNode rowNode, boolean downOneOnly) {
    RowNode finalRowNode = rowNode;
    FormatNode currentFormatNode = rowNode.getFormatNode();
    while (true) {
        rowNode = (RowNode)rowNode.getNextSibling();
        if (rowNode == null) {
            break;
        }
        if (rowNode.getRowFormat() instanceof RecordFormat) {
            if (rowNode.getFormatNode() == currentFormatNode) {
                finalRowNode = rowNode;
                if (downOneOnly) {
                    break;
                }
            }
            else {
                break;
            }
        }
    }
    return new CellPosition(finalRowNode,
                            getActualIndex(finalRowNode.getRowFormat(), getDesiredColumn()));
}

/**
 * Traverse to the parent row of the current row.
 * @return the new cell to move to.
 */
protected CellPosition goLevelParent() {
    RowNode finalRowNode = currentCell.row;
    RowNode parent = (RowNode) finalRowNode.getParent();
    if (parent != null && (RowNode) grid.getRowTree().getRootNode() != parent) {
        finalRowNode = parent;
    }
    return new CellPosition(finalRowNode,
                            getActualIndex(finalRowNode.getRowFormat(), getDesiredColumn()));
}

/**
 * Checks whether the passed source object refers to the current CellEditor.
 * @return true if the source refers to the current CellEditor.
 */
protected boolean isSourceEditorComponent(Object source) {
    return((currentCellEditor != null) &&
           ((Component)source == getFirstTraversable(currentCellEditor.getComponent())));
}

////////////////////////
// JCCellEditorListener methods
/**
 * Tells the listeners that the editor has ended editing.
 */
public void editingStopped(JCCellEditorEvent e) {
    if (currentCellEditor.isModified()) {
        if (commitAndConvert()) {
            inEditMode = false;
            handleAutoEdit();
        }
    }
    else {
        currentCellEditor.cancelCellEditing();
        inEditMode = false;
        handleAutoEdit();
    }
}

/**
 * Tells the listeners that the editor has cancelled editing.
 */
public void editingCanceled(JCCellEditorEvent e) {
    currentCellEditor.cancelCellEditing();
    inEditMode = false;
    handleAutoEdit();
}

/**
 * Tells the current CellEditor to cancel editing.
 */
public void cancelCellEditing() {
    // try to avoid calling cancelCellEditing() twice
    if (currentCellEditor != null && grid.isAutoEdit()) {
        currentCellEditor.cancelCellEditing();
    }
    inEditMode = false;
    handleAutoEdit();
}

/**
 * Tells the current CellEditor to (save changes and) stop editing.
 */
public void stopCellEditing() {
    if (currentCellEditor != null) {
        // commitAndConvert will call the cell editor's stopCellEditing()
        if (commitAndConvert()) {
            inEditMode = false;
            handleAutoEdit();
        }
    }
}

/**
 * Pops up the appropriate CellEditor if HiGrid is in autoEdit mode.
 */
protected void handleAutoEdit() {
    if (!grid.isAutoEdit()) {
        popupEditor(null);
    }
}

////////////////////////
// MouseListener methods
public void mouseClicked(MouseEvent e) {
    // ignore events for the cell editor
    if (isSourceEditorComponent(e.getSource())) {
        return;
    }
}

public void mouseEntered(MouseEvent e) {
    // ignore events for the cell editor
    if (isSourceEditorComponent(e.getSource())) {
        return;
    }

    if (grid.isTrackCursor()) {
        setOurCursor(e);
    }
    else {
        getGridArea().setCursor(grid.getDefaultCursor());
    }
    mouseExited = false;
    if (columnLabel != null) {
        columnLabel.setVisible(true);
    }
}

public void mouseExited(MouseEvent e) {
    // ignore events for the cell editor
    if (isSourceEditorComponent(e.getSource())) {
        return;
    }

    getGridArea().setCursor(Cursor.getDefaultCursor());
    mouseExited = true;
    if (columnLabel != null) {
        columnLabel.setVisible(false);
    }
}

public void mousePressed(MouseEvent e) {
    Object source = e.getSource();
    if (source instanceof NullCellEditor) {
        popupEditor(e);
        acquireFocusForCellEditor();
        return;
    }
    if (isSourceEditorComponent(source)) {
        inEditMode = true;
        return;
    }

    // bug in VM!
    if (source != getGridArea()) {
        // this event isn't really for us
        return;
    }
    RowTree rowTree = grid.getRowTree();
    GridArea gridArea = getGridArea();

    int x = e.getX();
    int y = e.getY();

    // take the horizontal scrollbar's position into account
    x += grid.getHorizontalScrollbar().getValue();

    originalMousePress = new Point(x, y);

    NullGraphics nullgc = getCellFromXY(new Point(x,y));
    RowNode rowNode = null;
    if (nullgc.cell != null) {
        rowNode = nullgc.cell.row;
    }

    if (!duringRowResizing(nullgc) && !duringColumnResizing(nullgc)) {
        boolean actionPerformed = true;
        int action = getAction(e);
        setCurrentAction(action);
        switch (action) {
        case HiGridAction.COLUMN_SELECT_ACTION:
            if (nullgc.cell != null && !nullgc.cell.isIndicator && nullgc.cell.format != null
                && rowNode != null && rowNode.getRowFormat() instanceof HeaderFormat) {
                selectingColumn = true;
                selectedCell = (CellPosition)nullgc.cell.clone();
                columnOrigin = grid.getPointFromCellPosition(selectedCell);
                columnOrigin.x -= x;
            }
            else {
                actionPerformed = false;
            }
            break;
        case HiGridAction.POPUP_MENU_ACTION:
            showPopupMenu(e);
            break;
        case HiGridAction.SINGLE_ROW_SELECT_ACTION:
        case HiGridAction.GROUP_ROW_SELECT_ACTION:
            if (grid.isAllowRowSelection() && rowNode != null && rowNode.getRowFormat() instanceof RecordFormat) {
                doRowSelection(rowNode, grid.getRowSelectionMode(), action == HiGridAction.GROUP_ROW_SELECT_ACTION);
            }
            break;
        case HiGridAction.COPY_ACTION:
            performCopyAction();
            break;
        case HiGridAction.PASTE_ACTION:
            performPasteAction();
            break;
        default:
            actionPerformed = false;
            break;
        }

        // we accomplished a special action, so no point in checking the rest
        if (actionPerformed) {
            setCurrentAction(HiGridAction.NO_ACTION);
            return;
        }
    }

    // all actions below assume left mouse button is pressed
    if (!SwingUtilities.isLeftMouseButton(e)) {
        return;
    }

    if (duringRowResizing(nullgc)) {
        if (grid.fireHiGridResizeCellEvent(new HiGridResizeCellEvent(grid,
                HiGridResizeCellEvent.BEFORE_RESIZE_ROW, rowNode))) {
            resizingRow = true;
            resizeLevel = rowNode;
            yResizeOffset = nullgc.rowYOffset;
            rowYResize = y + yResizeOffset;
            resizer = new JPanel();
            resizer.setVisible(false);
            resizer.setBackground(Color.black);
            getGridArea().add(resizer, 0);
            resizer.setBounds(0, rowYResize, grid.getSize().width, 1);
            resizer.setVisible(true);
        }
    }

    else if (duringColumnResizing(nullgc)) {
        if (grid.fireHiGridResizeCellEvent(new HiGridResizeCellEvent(grid,
                HiGridResizeCellEvent.BEFORE_RESIZE_COLUMN, rowNode, nullgc.resizeFormat.getName()))) {
            resizingColumn = true;
            columnXResize = nullgc.columnX;
            columnIndexResize = nullgc.columnIndex;
            resizeCell = new CellPosition(rowNode, columnIndexResize, nullgc.cell.isIndicator);
            resizeCell.format = nullgc.resizeFormat;
            xResizeOffset = x - columnXResize;
            resizer = new JPanel();
            resizer.setVisible(false);
            resizer.setBackground(Color.black);
            getGridArea().add(resizer, 0);
            resizer.setBounds(columnXResize - grid.getHorizontalScrollbar().getValue(),
                              0, 1, grid.getSize().height);
            resizer.setVisible(true);
        }
    }

    // clicked on a folder icon
    else if (nullgc.overFolderIcon) {
        switch (rowNode.getState()) {
        case NodeStatus.FOLDER_CLOSED:
            expandFolder(rowNode);
            break;
        case NodeStatus.FOLDER_OPEN:
            collapseFolder(rowNode);
            break;
        case NodeStatus.FOLDER_NONE:
            break;
        }
    }

    else if (nullgc.cell.format != null) {
        // clicked on an EditStatus cell
        if (grid.lookupTable.getCellRendererInstance(nullgc.cell.format.getCellRenderer())
            instanceof HiGridEditStatusRenderer) {
            // let's see if the edit status requires us to do something for this row
            if (rowNode != null && rowNode.getDataTableModel() != null) {
                if (rowNode.getDataTableModel().getRowStatus(rowNode.getBookmark()) != DataTableModel.COMMITTED) {
                    try {
                        rowNode.getDataTableModel().commitRow(grid.getDataModelListener(), rowNode.getBookmark());
                    }
                    catch (Exception exception) {
                        grid.fireHiGridErrorEvent(new HiGridErrorEvent(grid, exception));
                    }
                    resetRowSelection();
                    // refreshing the data will be done by HiGrid's DataModelListener
                }
            }
        }
        // clicked on an arbitrary RecordFormat cell
        else if (rowNode != null && rowNode.getRowFormat() instanceof RecordFormat) {
            validateAndTraverse(nullgc.cell,
                                rowNode.getRowFormat().getVisibleColumnIndex(nullgc.cell.columnIndex), e);
        }
    }
    else {
        // clicked somewhere else in gridArea; try to complete edit
        commitAndConvert();
    }
}

public void mouseReleased(MouseEvent e) {
    // ignore events for the cell editor
    if (isSourceEditorComponent(e.getSource())) {
        return;
    }

    int x = e.getX();
    // take the horizontal scrollbar's position into account
    x += grid.getHorizontalScrollbar().getValue();

    if (resizingColumn) {
        resizer.setVisible(false);
        getGridArea().remove(resizer);
        int oldWidth = resizeCell.format.getWidth();
        int newWidth = oldWidth + x - columnXResize - xResizeOffset;
        if (oldWidth != newWidth) {
            // prevent columns that are too narrow
            int minSize = MINIMUM_COLUMN_WIDTH;
            // prevent folder icons from being sized smaller than they are wide
            if (resizeCell.isIndicator && resizeCell.columnIndex == 0) {
                minSize = grid.folderIconStyle.getMaximumWidth();
            }
            newWidth = Math.max(newWidth, minSize);
            // holding the Shift key down will cause all columns to size to the same width
            if (e.isShiftDown() && !resizeCell.isIndicator) {
                resizeCell.row.getRowFormat().setCellWidths(newWidth);
            }
            else {
                // this changes the cell format for the sized row
                resizeCell.format.setWidth(newWidth);
            }
            grid.getRowTree().updateGridArea();
        }
        if (oldWidth != newWidth) {
            grid.fireHiGridResizeCellEvent(new HiGridResizeCellEvent(grid,
                HiGridResizeCellEvent.AFTER_RESIZE_COLUMN, resizeCell.row, resizeCell.format.getName()));
            grid.refresh();
        }
        // clear resize state
        resizeCell = null;
        resizingColumn = false;
    }
    else if (resizingRow) {
        int y = e.getY();
        resizer.setVisible(false);
        getGridArea().remove(resizer);
        int oldHeight = resizeLevel.getRowFormat().getHeight();
        int newHeight = oldHeight + y - rowYResize + yResizeOffset;
        if (oldHeight != newHeight) {
            // prevent rows that are too narrow
            newHeight = Math.max(newHeight, MINIMUM_ROW_HEIGHT);
            // this changes the cell format for the header
            resizeLevel.getRowFormat().setHeight(newHeight);
            int difference = (newHeight - oldHeight) * resizeLevel.getFormatNode().getNumberOfScrollableRows();
            RowTree rowTree = grid.getRowTree();
            rowTree.setScrollableRowsHeight(difference + rowTree.getScrollableRowsHeight());
            rowTree.updateGridArea();
        }
        resizingRow = false;
        if (oldHeight != newHeight) {
            grid.fireHiGridResizeCellEvent(new HiGridResizeCellEvent(grid,
                HiGridResizeCellEvent.AFTER_RESIZE_ROW, resizeLevel));
            grid.refresh();
        }
    }
    else if (movingColumn) {
        if (mouseExited) {
            mouseExited = false;
        }
        else {
            NullGraphics nullgc = getCellFromXY(new Point(x, columnOrigin.y));
            exchangeColumns(selectedCell, nullgc.cell);
            grid.fireHiGridMoveCellEvent(new HiGridMoveCellEvent(grid,
                HiGridMoveCellEvent.AFTER_MOVE_COLUMN, selectedCell.row, selectedCell.getCellFormat().getName()));
        }
        grid.getGridArea().remove(columnLabel);
        columnLabel = null;
        movingColumn = false;
        grid.refresh();
    }
    else if (selectingColumn) {
        grid.fireHiGridColumnSelectionEvent(new HiGridColumnSelectionEvent(grid,
            HiGridColumnSelectionEvent.SELECT_COLUMN, selectedCell.row, selectedCell, e));
        selectingColumn = false;
    }
    else {
        return;
    }

    // clear all possible states
    resizingColumn = false;
    resizingRow = false;
    movingColumn = false;
    selectingColumn = false;

    acquireFocusForCellEditor();
}

//////////////////////////////
// MouseMotionListener methods
public void mouseDragged(MouseEvent e) {
    if (resizingColumn) {
        resizer.setLocation(e.getX() - xResizeOffset, 0);
    }
    else if (resizingRow) {
        resizer.setLocation(0, e.getY() + yResizeOffset);
    }
    else if (movingColumn) {
        positionColumnLabel(e);
    }
    else if (selectingColumn) {
        if ((Math.abs(originalMousePress.x - e.getX()) > SELECT_TOLERANCE) ||
            (Math.abs(originalMousePress.y - e.getY()) > SELECT_TOLERANCE)) {
            selectingColumn = false;
            if (grid.fireHiGridMoveCellEvent(new HiGridMoveCellEvent(grid,
                    HiGridMoveCellEvent.BEFORE_MOVE_COLUMN, selectedCell.row, selectedCell.getCellFormat().getName()))) {
                movingColumn = true;
                createColumnLabel();
                grid.getGridArea().add(columnLabel, 0);
                positionColumnLabel(e);
            }
        }
    }
    else {
        setOurCursor(e);
    }
}

public void mouseMoved(MouseEvent e) {
    if (resizingColumn || resizingRow || movingColumn) {
        return;
    }
    setOurCursor(e);
}

// Helper methods

/**
 * Based on the passed event, sets the appropriate cursor for HiGrid.
 */
protected void setOurCursor(MouseEvent e) {
    // ignore events for the cell editor
    if (isSourceEditorComponent(e.getSource())) {
        return;
    }

    // don't set cursor if not supposed to
    if (!grid.isTrackCursor()) {
        return;
    }

    int x = e.getX();
    int y = e.getY();

    // take the horizontal scrollbar's position into account
    x += grid.getHorizontalScrollbar().getValue();

    // find out which row the mouse is on
    NullGraphics nullgc = getCellFromXY(new Point(x,y));
    GridArea gridArea = getGridArea();
    if (resizingRow || duringRowResizing(nullgc)) {
        gridArea.setCursor(Cursor.getPredefinedCursor(Cursor.N_RESIZE_CURSOR));
    }
    else if (resizingColumn || duringColumnResizing(nullgc)) {
        gridArea.setCursor(Cursor.getPredefinedCursor(Cursor.W_RESIZE_CURSOR));
    }
    else {
        gridArea.setCursor(grid.getDefaultCursor());
    }
}

/**
 * Creates the moving column header label for run-time user
 * placement of columns.
 */
protected void createColumnLabel() {
    columnLabel = new JColumnHeaderLabel();
    CellFormat selectedFormat = selectedCell.getCellFormat();
    columnLabel.setText(selectedFormat.getText());
    columnLabel.setBackground(selectedFormat.getBackground());
    columnLabel.setForeground(Color.black);
    columnLabel.setBounds(selectedFormat.getTotalArea());
    columnLabel.setBorder(new EmptyBorder(4, 4, 4, 4));
    columnLabel.setFont(selectedFormat.getFont());
}

/**
 * Positions the moving column header label horizontally based on
 * user input, but keeps it from moving vertically.
 */
protected void positionColumnLabel(MouseEvent e) {
    columnLabel.setLocation(columnOrigin.x+e.getX(), columnOrigin.y);
}

/**
 * Returns true if the mouse is over a row divider.
 */
protected boolean duringRowResizing(NullGraphics nullgc) {
    return nullgc.overRowDivider &&
    ((RowFormat)nullgc.cell.row.getRowFormat()).getAllowHeightSizing();
}

/**
 * Returns true if the mouse is over a column divider.
 */
protected boolean duringColumnResizing(NullGraphics nullgc) {
    if (!nullgc.overColumnDivider) {
        return false;
    }
    Vector cellFormats = (nullgc.cell.isIndicator) ?
                         nullgc.cell.row.getRowFormat().getIndicatorFormats() :
                         nullgc.cell.row.getRowFormat().getDataFormats();
    return((CellFormat)cellFormats.elementAt(nullgc.columnIndex)).getAllowWidthSizing();
}

/**
 * Helper method to exchange two columns within a given table.
 */
protected void exchangeColumns(Vector dataFormats, int oldColumn, int newColumn) {
    Object o = dataFormats.elementAt(oldColumn);
    dataFormats.removeElementAt(oldColumn);
    dataFormats.insertElementAt(o, newColumn);
}

/**
 * Exchanges two columns within a given table.
 */
protected void exchangeColumns(CellPosition oldCell, CellPosition newCell) {
    if ((oldCell == null || newCell == null) ||
        (oldCell.isIndicator() || newCell.isIndicator()) ||
        (oldCell.getRowNode() != newCell.getRowNode()) ||
        (newCell.getCellFormat() == null) ||
        (oldCell.getColumnIndex() == newCell.getColumnIndex())) {
        return;
    }
    FormatNode formatNode = oldCell.getRowNode().getFormatNode();
    int oldColumn = oldCell.getColumnIndex();
    int newColumn = newCell.getColumnIndex();
    exchangeColumns(formatNode.getHeaderFormat().getDataFormats(), oldColumn, newColumn);
    exchangeColumns(formatNode.getRecordFormat().getDataFormats(), oldColumn, newColumn);
    if ((currentCell.getRowNode().getRowFormat() == formatNode.getRecordFormat()) &&
        (currentCell.columnIndex == oldColumn)) {
        currentCell.columnIndex = newColumn;
        setDesiredColumn(newColumn);
        positionEditor(true);
    }
}

/**
 * Expands the passed rowNode.
 * @param rowNode the RowNode to expand.
 * @return true if the expansion was not denied
 */
protected boolean expandFolder(RowNode rowNode) {
    if ((rowNode == null) || (rowNode.getState() != NodeStatus.FOLDER_CLOSED)) {
        return false;
    }
    boolean proceed = false;
    proceed = grid.fireHiGridExpansionEvent(new HiGridExpansionEvent(grid,
        HiGridExpansionEvent.BEFORE_EXPAND_ROW, rowNode));
    if (proceed) {
        // this could take a while, so
        GridArea gridArea = getGridArea();
        Cursor oldCursor = gridArea.getCursor();
        gridArea.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        resetRowSelection();
        grid.refreshRowAndBelow(rowNode, NodeStatus.FOLDER_CLOSED);
        grid.fireHiGridExpansionEvent(new HiGridExpansionEvent(grid,
            HiGridExpansionEvent.AFTER_EXPAND_ROW, rowNode));
        gridArea.setCursor(oldCursor);
    }
    return proceed;
}

/**
 * Collapses the passed rowNode.
 * @param rowNode the RowNode to expand.
 * @return true if the collapse was not denied
 */
protected boolean collapseFolder(RowNode rowNode) {
    if ((rowNode == null) || (rowNode.getState() != NodeStatus.FOLDER_OPEN)) {
        return false;
    }
    boolean proceed = false;
    proceed = grid.fireHiGridExpansionEvent(new HiGridExpansionEvent(grid,
        HiGridExpansionEvent.BEFORE_COLLAPSE_ROW, rowNode));
    if (proceed && currentCell.row.isChildOf(rowNode)) {
        CellPosition cell = new CellPosition(rowNode, getActualIndex(rowNode.getRowFormat(), getDesiredColumn()));
        if (!validateAndTraverse(cell, getDesiredColumn(), null)) {
            proceed = false;
        }
    }
    if (proceed) {
        resetRowSelection();
        grid.refreshRowAndBelow(rowNode, NodeStatus.FOLDER_OPEN);
        grid.fireHiGridExpansionEvent(new HiGridExpansionEvent(grid,
            HiGridExpansionEvent.AFTER_COLLAPSE_ROW, rowNode));
    }
    return proceed;
}

/**
 * Shows or hides the current CellEditor
 * @param true to show, false to hide
 */
protected void showEditor(boolean show) {
    if (grid.getRowTree().getRootNode() == null || !(grid.getRowTree().getRootNode().getChildCount() > 0)  ||
        currentCellEditor == null || currentCellEditor.getComponent() == null) {
        return;
    }

 
    if (grid.isEditorHidden()) {
        show = false;
    }
    currentCellEditor.getComponent().setVisible(show);
}

/**
 * Initializes the current CellEditor
 */
protected void initializeEditor() {
    initializeEditor(null);
}

/**
 * Initializes the current CellEditor
 * @param initialEvent the initial MouseEvent or KeyEvent
 */
protected void initializeEditor(AWTEvent initialEvent) {
    // sanity check
    if (grid.getDataModel() == null || currentCell == null) {
        return;
    }
    // get data value for this cell from the data source
    Object o = null;
    try {
        o = currentCell.row.getDataTableModel().getResultData(currentCell.row.getBookmark(), currentCell.format.getName());
    }
    catch (Exception e) {
        grid.fireHiGridErrorEvent(new HiGridErrorEvent(grid, e));
    }
    if (initialEvent instanceof MouseEvent) {
        Point p = grid.getPointFromCellPosition(currentCell);
        MouseEvent m = (MouseEvent)initialEvent;
        // create a MouseEvent with the point translated properly
        initialEvent = new MouseEvent(currentCellEditor.getComponent(),
                                      m.getID(),
                                      m.getWhen(),
                                      m.getModifiers(),
                                      m.getX() - p.x,
                                      m.getY() - p.y,
                                      m.getClickCount(),
                                      m.isPopupTrigger());

    }
    currentCellEditor.initialize(initialEvent, currentCell.format, o);
    inEditMode = false;
}

/**
 * Removes the current CellEditor from the GridArea
 */
protected void removeEditor() {
    if (currentCellEditor != null) {
        currentCellEditor.cancelCellEditing();
        getGridArea().remove(currentCellEditor.getComponent());
        currentCellEditor.removeCellEditorListener(this);
        if (currentCellEditor instanceof JCValidateInterface) {
            ((JCValidateInterface)currentCellEditor).removeValidateListener(grid);
        }
        Component editorComponent = currentCellEditor.getComponent();
        if (editorComponent != null) {
            Component focusComponent = getFirstTraversable(editorComponent);
            if (focusComponent != null) {
                focusComponent.removeKeyListener(this);
                focusComponent.removeMouseListener(this);
                focusComponent.removeFocusListener(focusListener);
            }
        }
        currentCellEditor = null;
    }
}

/**
 * Adds the current CellEditor to the GridArea
 */
protected void addEditor() {
    Component editorComponent = currentCellEditor.getComponent();
    if (editorComponent != null) {
        currentCellEditor.addCellEditorListener(this);
        if (currentCellEditor instanceof JCValidateInterface) {
            ((JCValidateInterface)currentCellEditor).addValidateListener(grid);
        }
        Component focusComponent = getFirstTraversable(editorComponent);
        if (focusComponent != null) {
            focusComponent.addKeyListener(this);
            focusComponent.addFocusListener(focusListener);

            if (fNewFocusModel) {
                focusComponent.setFocusTraversalKeysEnabled(false);
            }

            focusComponent.addMouseListener(this);
        }
    }
}

/**
 * Creates a CellEditor for the current cell
 */
protected void createEditor() {
    createEditor(null);
}

/**
 * Creates a CellEditor for the current cell
 * @param initialEvent the initial MouseEvent or KeyEvent
 */
protected void createEditor(AWTEvent initialEvent) {
    // sanity check
    if (grid.getDataModel() == null || currentCell == null) {
        return;
    }
    // make current editor disappear
    showEditor(false);
    removeEditor();
    currentCellEditor = grid.lookupTable.getCellEditorInstance(
        ((grid.isAutoEdit() || initialEvent != null) ? currentCell.format.getCellEditor() : nullEditor));
    addEditor();
    initializeEditor(initialEvent);
    showEditor(false);
}

/**
 * Positions the current CellEditor at the current cell
 */
protected void positionEditor() {
    positionEditor(false);
}

/**
 * Positions the current CellEditor at the current cell
 * @param wantFocus if true, also tries to acquire keyboard focus for the editor
 */
protected void positionEditor(boolean wantFocus) {
    if (grid.getRowTree().getRootNode() == null || !(grid.getRowTree().getRootNode().getChildCount() > 0)) {
        return;
    }
    if (currentCellEditor == null) {
        createEditor();
    }
    if (currentCellEditor != null && currentCell != null) {
        if (currentCell.format != null && currentCell.row != null) {
            Component editorComponent = currentCellEditor.getComponent();
            if (editorComponent == null) {
                return;
            }
            // the next remove/add portion is to handle Z-order properly
            getGridArea().remove(currentCellEditor.getComponent());
            NullGraphics nullgc = getXYFromCell(currentCell);
            if (! nullgc.finishedOperation) {
                return;
            }
            // if a component resize has happened, we need to reset the current cell's (x,y) point
            currentCell = nullgc.cell;
            getGridArea().add(editorComponent, 0);
            // place the current JCCellEditor appropriately
            Dimension editorSize =
            new Dimension(currentCell.format.getTotalArea().width,
                          currentCell.row.getHeight());
            getEditorSize(editorComponent, currentCell.format, editorSize);
            editorComponent.setBounds(
                                     currentCell.point.x - grid.getHorizontalScrollbar().getValue(),
                                     currentCell.point.y, editorSize.width, editorSize.height);
            if (wantFocus) {
                acquireFocusForCellEditor();
            }
        }
    }
}




/**
 * Attempts to acquire keyboard focus for the current CellEditor
 */
protected void acquireFocusForCellEditor() {
    if (currentCellEditor == null) {
        return;
    }

    if (!fNewFocusModel) {
        // this check should probably be done for other complex components
        // FIX - perhaps this should be done for all container components that
        // have embedded editors
        if ((currentCellEditor instanceof JComboBox) ||
            (currentCellEditor instanceof JCCellEditor)) {
                FocusManager.disableSwingFocusManager();
        }
        else {
            FocusManager.setCurrentManager(FocusManager.getCurrentManager());
        }
    }

    // pass focus to container components that have embedded editors
    Component focusComponent =
            getFirstTraversable(currentCellEditor.getComponent());

    if (focusComponent != null &&



        (!focusComponent.hasFocus()

        || focusComponent instanceof JTextComponent
        || focusComponent instanceof NullCellEditor) ) {
        // switch the focus to another component temporarily
        // this is needed to force some sort of reinitialization in the cell
        // editor if it isn't present, then keys no longer appear in the
        // editor when typed!
        getGridArea().getFocusManager().requestFocus();
        getGridArea().getFocusManager().requestFocus(focusComponent);
    }
}

/**
 * Gets the CellEditor size based on the EditWidth and EditHeight policies.
 * @param editComponent the editor's component
 * @param format the format for the cell that the editor will be placed at
 * @param cellSize the eventual size of the editor
 */
protected void getEditorSize(Component editComponent, CellFormat format,
                             Dimension cellSize) {
    switch (format.getEditWidthPolicy()) {
    case HiGrid.EDIT_SIZE_TO_CELL:
        break;
    case HiGrid.EDIT_ENSURE_MINIMUM_SIZE:
        cellSize.width = Math.max(editComponent.getMinimumSize().width,
                                  cellSize.width);
        break;
    case HiGrid.EDIT_ENSURE_PREFERRED_SIZE:
        cellSize.width = Math.max(editComponent.getPreferredSize().width,
                                  cellSize.width);
        break;
    }
    switch (format.getEditHeightPolicy()) {
    case HiGrid.EDIT_SIZE_TO_CELL:
        break;
    case HiGrid.EDIT_ENSURE_MINIMUM_SIZE:
        cellSize.height = Math.max(editComponent.getMinimumSize().height,
                                   cellSize.height);
        break;
    case HiGrid.EDIT_ENSURE_PREFERRED_SIZE:
        cellSize.height = Math.max(editComponent.getPreferredSize().height,
                                   cellSize.height);
        break;
    }
}

/**
 * Retrieves the first traversable component of the CellEditor
 * @param editorComponent the CellEditor's component
 */
protected Component getFirstTraversable(Component editorComponent) {
    // pass focus to container components that have embedded editors
    if (editorComponent != null) {
        if (editorComponent instanceof Container) {
            if (editorComponent.isFocusTraversable()) {
                return editorComponent;
            }
            else {
                Component children[] = ((Container) editorComponent).getComponents();
                for (int p = 0; p < children.length; p++) {
                    if (children[p].isFocusTraversable()) {
                        return children[p];
                    }
                }
                // if none of them volunteer, pick the first one!
                if (children.length > 0) {
                    return children[0];
                }
            }
        }
        else {
            return editorComponent;
        }
    }
    return null;
}

/**
 * Finds a default place to put the current cell when
 * it is uninitialized
 */
protected void positionCurrentCell() {
    if (currentCell == null || currentCell.row == null) {
        CellPosition newPosition = getFirstCell(null);
        if (newPosition == null) {
            return;
        }
        setDesiredColumn(newPosition.columnIndex);
        moveCurrentCellInternal(newPosition, newPosition.columnIndex, false, true);
    }
}

/**
 * Reposition the current cell.
 *
 * @param rowNode The row node of the current cell.
 * @param showEditor if true, display the editor
 * @param moveEditor if true, move the editor to this location
 */
protected void repositionCurrentCell(RowNode rowNode, boolean showEditor, boolean moveEditor) {
    if (rowNode != null) {
        CellPosition cell = new CellPosition(rowNode, rowNode.getRowFormat().getFirstVisibleCell());
        moveCurrentCellInternal(cell, cell.columnIndex, showEditor, moveEditor);
        createEditor();
        positionEditor();
        if (showEditor) {
            showEditor(true);
        }
    }
}

/**
 * Returns the last available editable position in the grid
 * @param row The row to look on; if null start at the first row of the grid
 */
protected CellPosition getLastCell(RowNode row) {
    int column = -1;

    if (row == null) {
        // we want to iterate backwards from the end, until we hit an editable column
        // start with the last row, and walk backwards 0 rows until we hit a RecordFormat
        row = ((RowNode)grid.getRowTree().getRootNode().getLastChild());
        CellPosition lastCell = doTheRecordWalk(row, 0, false);
        row = lastCell.row;
    }

    column = row.getRowFormat().getLastVisibleCell();
    if (column < grid.FIRST_EDITABLE_COLUMN) {
        return null;
    }

    return new CellPosition(row, column);
}

/**
 * Returns the first available editable position in the grid
 * @param row The row to look on; if null start at the first row of the grid
 */
protected CellPosition getFirstCell(RowNode row) {
    int column = -1;

    if (row == null) {
        // iterate over the rows until we find a valid place to put an editor
        TreeIterator ti = grid.getRowTree().getRootNode().getIterator();
        while (ti.hasMoreElements()) {
            row = (RowNode)ti.get();
            column = row.getRowFormat().getFirstVisibleCell();
            if (column >= grid.FIRST_EDITABLE_COLUMN) {
                break;
            }
            ti.nextElement();
        }
    }
    else {
        column = row.getRowFormat().getFirstVisibleCell();
    }

    if (column < grid.FIRST_EDITABLE_COLUMN) {
        return null;
    }
    return new CellPosition(row, column);
}

/**
 * Tries to complete any existing CellEditor edits, validate the result
 * and store the result in the dataModel.
 */
protected boolean commitAndConvert() {
    if (currentCellEditor == null || currentCell == null ||
        // this row may have already been deleted, discard any changes
        (currentCell.row.getDataTableModel().getRowStatus(currentCell.row.getBookmark()) == DataTableModel.NONEXISTENT) ||
        (currentCell.row.getDataTableModel().getRowStatus(currentCell.row.getBookmark()) == DataTableModel.DELETED)) {
        return true;
    }
    if (!grid.isEditable() || !currentCell.format.isEditable()) {
        currentCellEditor.cancelCellEditing();
        return true;
    }
    // allow the JCCellEditor to validate the cell
    if (! currentCellEditor.stopCellEditing()) {
        // commit failed; don't need to move anywhere
        return false;
    }
    Object o = currentCellEditor.getCellEditorValue();
    String columnId = currentCell.format.getName();
    boolean status = true;
    // update cell in dataModel
    try {
        currentCell.row.getDataTableModel().updateCell(grid.getDataModelListener(), currentCell.row.getBookmark(), columnId, o);
    }
    catch (Exception e) {
        grid.fireHiGridErrorEvent(new HiGridErrorEvent(grid, e));
        status = false;
    }
    return status;
}

/**
 * Computes the number of pixels needed to scroll horizontally to
 * make the passed cellPosition visible.
 * @param cellPosition the CellPosition to scroll into view
 */
protected int horizontalCellAdjustment(CellPosition cellPosition) {
    if (cellPosition != null) {
        GridArea gridArea = getGridArea();
        GridScrollbar horizontalScrollbar = grid.getHorizontalScrollbar();
        int cellWidth = cellPosition.format.getTotalArea().width;
        int gridAreaWidth = gridArea.getSize().width;
        // is it past the left edge of the visible window?
        if (cellPosition.point.x < horizontalScrollbar.getValue()) {
            if (cellPosition.point.x + cellWidth < gridAreaWidth) {
                return 0 - horizontalScrollbar.getValue();
            }
            return cellPosition.point.x - horizontalScrollbar.getValue();
        }
        // does this cell even fit into the visible grid area?
        if (cellWidth >= gridAreaWidth) {
            return 0;
        }
        // is it past the right edge of the visible window?
        if (cellPosition.point.x + cellWidth > gridAreaWidth + horizontalScrollbar.getValue()) {
            return cellPosition.point.x + cellWidth - (gridAreaWidth + horizontalScrollbar.getValue());
        }
    }
    return 0;
}

/**
 * Scrolls both vertically and horizontally, if necessary, to scroll
 * the current CellEditor into view.
 * @param action the action that initiated the scroll
 */
protected boolean scrollEditorIntoView(int action) {
    if (currentCell == null || getGridArea().getSize().height <= 0) {
        // no need to scroll if we're just being initialized
        return false;
    }
    boolean needRepaint = false;
    RowTree rowTree = grid.getRowTree();
    // first, see if it is in view vertically

    NullGraphics nullgc = getXYFromCell(currentCell);
    CellPosition cell = nullgc.cell;
    if (! nullgc.finishedOperation && (action != HiGridAction.MOVE_DOWN_ACTION)) {
        // make this row the top row, unless we are using cursor down
        rowTree.setTopRow(currentCell.row);
        nullgc = getXYFromCell(currentCell);
        cell = nullgc.cell;
        needRepaint = true;
    }
    // partially visible vertically, need to set this one as the last
    // row, and walk backwards to determine where to place the first row
    else if (! nullgc.fullyVisible) {
        RowNode rowNode = rowTree.getTopRowNodeFromLastFullyVisibleRow(currentCell.row);
        rowTree.setTopRow(rowNode == null ? currentCell.row : rowNode);
        needRepaint = true;
    }
    // now, make the editor visible horizontally
    int adjustment = horizontalCellAdjustment(cell);
    if (adjustment != 0) {
        GridScrollbar horizontalScrollbar = grid.getHorizontalScrollbar();
        horizontalScrollbar.setValue(adjustment + horizontalScrollbar.getValue());
    }
    return needRepaint;
}

/**
 * Scrolls HiGrid so that the current CellEditor is in view.
 */
public void makeCellEditorVisible() {
    makeCellEditorVisible(KeyEvent.VK_UNDEFINED);
}

/**
 * Scrolls HiGrid so that the current CellEditor is in view.
 * @param action Action that caused traversal
 */
protected void makeCellEditorVisible(int action) {
    if (!isTraversing()) {
        if (scrollEditorIntoView(action)) {
            grid.refresh();
            acquireFocusForCellEditor();
            inEditMode = true;
        }
    }
}

/**
 * Moves the current cell to the new row and column.
 * @param newPosition the new cell to move to
 * @param newDesiredColumn the default column to move to when moving vertically
 * Due to cell visibility, the desired column may not be always possible.
 * @return true if validation and movement succeeds
 */
public boolean moveCurrentCell(CellPosition newPosition, int newDesiredColumn) {
    if (newPosition == null) {
        return false;
    }
    if (!(newPosition.getRowNode().getRowFormat() instanceof RecordFormat)) {
        return false;
    }
    newPosition.setColumnIndex(getActualIndex(newPosition.getRowNode().getRowFormat(), newDesiredColumn));
    if (newPosition.getCellFormat() == null) {
        return false;
    }
    return validateAndTraverse(newPosition, newPosition.columnIndex, null);
}

/**
 * Moves the current cell to the new row and column.
 * @param newPosition the new cell to move to
 * @param newDesiredColumn the default column to move to when moving vertically
 * @param refresh if true refresh editStatus
 * @param moveInDataModel if true, also move current row in dataModel
 * Due to cell visibility, the desired column may not be always possible.
 * @return true if validation and movement succeeds
 */
public boolean moveCurrentCell(CellPosition newPosition, int newDesiredColumn, boolean refresh, boolean moveInDataModel) {
    /*
     * This method remains to ensure backward compatibility. It is currently
     * functionally equivalent to the two-argument method above.
     * The orginal versions were internal methods made public, now renamed to
     * moveCurrentCellInternal() and unchanged.
     */
    return moveCurrentCell(newPosition, newDesiredColumn);
}

/**
 * Moves the current cell to the new row and column.
 * @param newPosition the new cell to move to
 * @param newDesiredColumn the default column to move to when moving vertically
 */
protected boolean moveCurrentCellInternal(CellPosition newPosition, int newDesiredColumn) {
    return moveCurrentCellInternal(newPosition, newDesiredColumn, true, true);
}

/**
 * Moves the current cell to the new row and column.
 * @param newPosition the new cell to move to
 * @param newDesiredColumn the default column to move to when moving vertically
 * @param refresh if true refresh editStatus
 * @param moveInDataModel if true, also move current row in dataModel
 */
protected boolean moveCurrentCellInternal(CellPosition newPosition, int newDesiredColumn, boolean refresh, boolean moveInDataModel) {
    boolean changedRow = (currentCell == null) ? true : (currentCell.row != newPosition.row);
    CellPosition oldPosition = null;
    if (currentCell != null) {
        oldPosition = (CellPosition)currentCell.clone();
    }
    boolean moveSucceeded = true;

    // IDE support
    boolean ideStrangeness = false;
    boolean repaintAll = false;
    DataTableModel newDataTable = newPosition.row.getDataTableModel();
    long newBookmark = newPosition.row.getBookmark();
    // translate the current bookmark to an absolute row index
    int rowIndex = newDataTable.getRowIndex(newBookmark);
    DataTableModel parentDataTable = (DataTableModel)((DataSourceTreeNode)newDataTable).getParent();
    long parentBookmark = newDataTable.getParentBookmark();

    if (changedRow) {
        if (moveInDataModel) {
            try {
                moveSucceeded = grid.getDataModel().moveToRow(grid.getDataModelListener(), newDataTable, newBookmark);
            }
            catch (Exception e) {
                grid.fireHiGridErrorEvent(new HiGridErrorEvent(grid, e));
                moveSucceeded = false;
            }
        }
        if (moveSucceeded) {
            // IDE support
            DataModel dataModel = newDataTable.getMetaData().getDataModel();
            DataSourceTreeNode node = (DataSourceTreeNode)newDataTable;
            DataSourceTreeNode root = (DataSourceTreeNode)dataModel.getDataTableTree().getRoot();
            // has a RESET happened during our move (and thus our bookmark
            // and perhaps our table has vanished)?
            ideStrangeness =
            (node.getParent() == null) && (node != root) ||
            (((DataTableModel)root).getRowIndex(((DataTableModel)node).getRootAncestorBookmark(newBookmark)) == DataTableModel.ROW_NOT_FOUND) ||
            (newBookmark != dataModel.getCurrentGlobalBookmark());
            if (ideStrangeness) {
                RowNode rowNode;
                try {
                    // both dest + source are root
                    if (node == root && oldPosition.row.getDataTableModel() == root) {
                        // after reset, going to another row in root table
                        // There is a null global table in the bean, so check for it here
                        if (dataModel.getCurrentGlobalTable() != null) {
                            rowNode = grid.getRowTree().findRecordRowNode(null, dataModel.getCurrentGlobalTable().getRowIdentifier(rowIndex));
                        }
                        else {
                            rowNode = null;
                        }
                    }
                    // source is root, dest is details
                    else if (oldPosition.row.getDataTableModel() == root) {
                        // after reset, going to details and details disappearing
                        rowNode = grid.getRowTree().findFirstRowNode(null, (DataTableModel)root);
                        if (rowNode != null) {
                            newDesiredColumn = Math.max(grid.FIRST_EDITABLE_COLUMN, rowNode.getRowFormat().getFirstVisibleCell());
                        }
                        else {
                            newDesiredColumn = grid.FIRST_EDITABLE_COLUMN;
                        }
                    }
                    // source + dest are details
                    else if (node != root) {
                        // after a requery row and details, staying inside details
                        rowNode = grid.getRowTree().findRecordRowNode(null,
                                                                      parentDataTable.createTable(grid.getDataModelListener(), parentBookmark, (DataSourceTreeNode)newDataTable.getMetaData()).getRowIdentifier(rowIndex));
                    }
                    // dest is root, source is details
                    // if so, then normal operation and we shouldn't be in this block of code
                    // should be impossible!
                    else {
                        rowNode = null;
                    }
                    newPosition = new CellPosition(rowNode, newDesiredColumn);
                }
                catch (Exception e) {
                    // not sure what to do with an exception here
                }
            }
            // did the current position change during our move to yet another spot?
            // if so, we may have repainted a row and its edit status might have been painted as current
            // to prevent two of these from appearing, just repaint everything
//            if (oldPosition != null && oldPosition.row.getBookmark() != currentCell.row.getBookmark()) {
            if (newPosition.row.getBookmark() != newBookmark) {
                try {
                    grid.getDataModel().moveToRow(grid.getDataModelListener(), newPosition.row.getDataTableModel(), newPosition.row.getBookmark());
                }
                catch (Exception e) {
                    grid.fireHiGridErrorEvent(new HiGridErrorEvent(grid, e));
                }
                repaintAll = true;
            }
            currentCell = newPosition;
            grid.refreshEditStatus(oldPosition, refresh);
        }
    }
    if (moveSucceeded) {
        currentCell = newPosition;
        setDesiredColumn(newDesiredColumn);
        grid.refreshEditStatus(currentCell, refresh);
        if (getSelectedRows().size() != 0) {
            resetRowSelection();
        }
    }
    else {
        currentCell = oldPosition;
    }
    if (repaintAll) {
        grid.refresh();
    }
    if (moveSucceeded) {
        grid.fireHiGridTraverseEvent(new HiGridTraverseEvent(grid,
            HiGridTraverseEvent.AFTER_TRAVERSE, oldPosition, newPosition));
    }
    return moveSucceeded;
}

/**
 * Tries to complete any outstanding edits and validate the passed cellPosition
 * @param newCellPosition the new cell to move to
 * @return true if validation succeeds
 */
protected boolean validateCell(CellPosition newCellPosition) {
    return validateCell(newCellPosition, true);
}

/**
 * Tries to complete any outstanding edits and validate the passed cellPosition
 * @param newCellPosition the new cell to move to
 * @param refresh if true refresh editStatus or current cell
 * @return true if validation succeeds
 */
protected boolean validateCell(CellPosition newCellPosition, boolean refresh) {
    if (currentCellEditor != null && currentCellEditor.isModified()) {
        if (!commitAndConvert()) {
            return false;
        }
        if (refresh) {
            if (newCellPosition == null) {  // we haven't moved, but we have modified this cell
                grid.refreshEditStatus(currentCell, true);
            }
            else {
                // need to refresh current cell here, since its value has changed and we are moving
                grid.refreshCell(currentCell);
            }
        }
    }
    return true;
}

/**
 * Traverses to the new cell and displays a CellEditor
 * @param newPosition the new cell to move to
 * @param newDesiredColumn the default column to move to when moving vertically
 * @param initialEvent the initial MouseEvent or KeyEvent
 * @return true if traversal successful
 */
protected boolean traverseToCell(CellPosition newCellPosition,
                                 int newDesiredColumn, AWTEvent initialEvent) {
    duringTraversal = true;
    if (newCellPosition != null) {
        // this doesn't seem to work very well, or at all!
        if (!moveCurrentCellInternal(newCellPosition, newDesiredColumn)) {
            return false;
        }
        showEditor(false);
        // find out how many rows/pixels we have moved
        RowTree rowTree = grid.getRowTree();
        RowNode oldTopRowNode = rowTree.getTopRowNode();
        int oldValue = grid.getVerticalScrollbar().getValue();
        int action = getAction(initialEvent);
        boolean changed = scrollEditorIntoView(action);
        if (initialEvent instanceof KeyEvent) {
            popupEditor(action != HiGridAction.NO_ACTION ? null : initialEvent);
        }
        else {
            // got here via mouse click
            popupEditor(initialEvent);
        }
        // for Solaris compatibility, must requestfocus here again
        acquireFocusForCellEditor();
    }
    duringTraversal = false;
    return true;
}

/**
 * Pops up the CellEditor for the current cell
 * @param initialEvent the initial MouseEvent or KeyEvent
 */
protected void popupEditor(AWTEvent initialEvent) {
    createEditor(initialEvent);
    positionEditor(true);
    showEditor(true);
}

/**
 * Validates any changes in the CellEditor and traverses to the new cell.
 * @param newPosition the new cell to move to
 * @param newDesiredColumn the default column to move to when moving vertically
 * @param initialEvent the initial MouseEvent or KeyEvent
 * @return true if traversal successful
 */
protected boolean validateAndTraverse(CellPosition newCellPosition, int newDesiredColumn, AWTEvent initialEvent) {
    if (!validateCell(newCellPosition)) {
        return false;
    }
    return traverseToCell(newCellPosition, newDesiredColumn, initialEvent);
}

/**
 * Determines the cell that corresponds to the given point.
 * @param point a point on the grid
 * @return a NullGraphics object that contains the corresponding cell
 */
protected NullGraphics getCellFromXY(Point point) {
    NullGraphics nullgc = new NullGraphics(NullGraphics.OPERATION_HITTEST, grid, null, point);
    grid.getRowTree().displayRuntimeTree(nullgc);
    return nullgc;
}

/**
 * Determines the point that corresponds to the given cell.
 * @param cellPosition a cell in the grid
 * @return a NullGraphics object that contains the corresponding point
 */
protected NullGraphics getXYFromCell(CellPosition cellPosition) {
    cellPosition = new CellPosition(cellPosition.row, cellPosition.columnIndex,
                                    cellPosition.isIndicator);
    int operation = (cellPosition.isIndicator) ? NullGraphics.OPERATION_GET_INDICATOR
                    : NullGraphics.OPERATION_GET_CELL;
    NullGraphics nullgc = new NullGraphics(operation, grid, cellPosition, new Point(-1, -1));
    grid.getRowTree().displayRuntimeTree(nullgc);
    return nullgc;
}

/**
 * Walks the RowTree to a CellPosition based on the passed parameters.
 * @param startNode the initial row to walk from
 * @param count the number of units to walk
 * @param downward if true walk downward vertically
 */
protected CellPosition doTheRecordWalk(RowNode startNode, int count, boolean downward) {
    RecordWalk walkable = new RecordWalk(count, downward);
    grid.getRowTree().walk(startNode, walkable);
    RowNode foundNode = walkable.getFoundNode();
    if (foundNode == null) {
        if (startNode.getRowFormat() instanceof RecordFormat) {
            foundNode = startNode;
        }
    }
    if (foundNode != null) {
        return new CellPosition(foundNode,
                                Math.max(foundNode.getRowFormat().getFirstVisibleCell(),
                                         Math.min(getDesiredColumn(), foundNode.getRowFormat().getLastVisibleCell())));
    }
    return null;
}

/**
 * Helper method to show the popup menu
 *
 * @param event The mouse event.
 */
protected void showPopupMenu(InputEvent e) {
    if (!grid.isAllowPopupMenu()) {
        return;
    }

    /*
     * Solaris workaround -
     * Two right-clicks in a row will cause an exception in the Motif menu peer.
     * The call to createPopupMenu can be removed if the bug in Solaris is fixed.
     */
//    getGridArea().createPopupMenu();
    int x, y;
    if (e instanceof MouseEvent) {
        x = ((MouseEvent)e).getX();
        y = ((MouseEvent)e).getY();
    }
    else if (e instanceof KeyEvent) {
        Point p = grid.getPointFromCellPosition(currentCell);
        x = p.x;
        y = p.y + currentCell.getCellFormat().getTotalArea().height;
    }
    else {
        return;
    }
    EditPopupMenu popup = grid.getEditPopupMenu();
    if (popup != null) {
        popup.show(x, y);
        getGridArea().getFocusManager().requestFocus(popup);
    }
}

/**
 * Executes a copy action using the system clipboard.
 * The currently selected rows are copied to the clipboard.
 */
public void performCopyAction() {
    performCopyAction(getSelectedRows());
}

/**
 * Executes a copy action using the system clipboard.
 *
 * @param rows The rows to copy to the clipboard.
 */
public void performCopyAction(Vector rows) {
    StringRowFlavor selection = new StringRowFlavor(grid, rows);
    Toolkit.getDefaultToolkit().getSystemClipboard().setContents(selection, selection);
}

/**
 * Executes a paste action using the system clipboard.
 */
public void performPasteAction() {
    performPasteAction(grid.getCurrentCellPosition());
}

/**
 * Executes a paste action using the system clipboard.
 *
 * @param cellPosition the cell position to insert before
 */
public void performPasteAction(CellPosition cellPosition) {
    if (cellPosition == null) {
        return;
    }
    Transferable tfr = Toolkit.getDefaultToolkit().getSystemClipboard().getContents(this);
    if (tfr != null) {
        try {
            DataFlavor flavors[] = tfr.getTransferDataFlavors();
            for (int i = 0; i < flavors.length; i++ ) {
                if (flavors[i].getHumanPresentableName().equals("Unicode String")) {
                    StringTokenizer st = new StringTokenizer((String)tfr.getTransferData(flavors[i]), "\n", false);
                    Vector cells = new Vector();
                    Vector row;
                    while (st.hasMoreElements()) {
                        String line = (String)st.nextElement();
                        if (line != null) {
                            StringTokenizer tabs = new StringTokenizer(line, "\t", false);
                            row = new Vector();
                            while (tabs.hasMoreElements()) {
                                row.addElement((String)tabs.nextToken());
                            }
                            cells.addElement(row);
                        }
                    }
                    pasteData(cells, cellPosition.getRowNode());
                    return;
                }
            }
        } catch (IOException ioe) {
            //FIX should be localized strings
            System.out.println("Error pasting data: " + ioe.getMessage());
        } catch (UnsupportedFlavorException ufe) {
            System.out.println("Error pasting data: " + ufe.getMessage());
        }
    }
}

/**
 *  Pastes a vector of rows before the given cellPosition.
 *
 * @param rows a vector of row object vectors
 * @param insertBefore the row to insert/paste the rows before
 */
public void pasteData(Vector rows, RowNode insertBefore) {
    DataTableModel dataTable = insertBefore.getDataTableModel();
    if (dataTable == null) {
        return;
    }
    // change commit policy to insert all rows as not comitted
    MetaDataModel metaData = dataTable.getMetaData();
    if (metaData == null) {
        return;
    }
    int oldPolicy = metaData.getCommitPolicy();
    try {
        metaData.setCommitPolicy(MetaDataModel.COMMIT_MANUALLY);
    }
    catch (DataModelException dme) {
        grid.fireHiGridErrorEvent(new HiGridErrorEvent(grid, dme));
    }
    long bookmark = insertBefore.getBookmark();
    for (Enumeration e = rows.elements(); e.hasMoreElements();) {
        Vector v = (Vector)e.nextElement();
        try {
            Object[] newRow = (Object[])dataTable.createNewRow();
            if (newRow.length != v.size()) {
                throw new HiGridException("Number of columns does not match "+newRow.length+" "+v.size()+" - paste aborted ");
            }
            for (int i = 0; i < newRow.length; i++) {
                newRow[i] = v.elementAt(i);
            }
            dataTable.addRow(grid.getDataModelListener(), newRow, DataTableModel.INSERTED,
                             dataTable.getRowIndex(bookmark));
        }
        catch (Exception ex) {
            grid.fireHiGridErrorEvent(new HiGridErrorEvent(grid, ex));
        }
    }
    try {
        metaData.setCommitPolicy(oldPolicy);
    }
    catch (DataModelException dme) {
        grid.fireHiGridErrorEvent(new HiGridErrorEvent(grid, dme));
    }
    resetRowSelection();
}

/**
 * Returns the Vector of selected row items.
 *
 * @return The Vector of RowNodes.
 */
public Vector getSelectedRows() {
    return selectedRowNodes;
}

/**
 * Sets the Vector of selected row items.
 *
 * @param rowNodes The Vector of RowNodes.
 */
protected void setSelectedRows(Vector rowNodes) {
    selectedRowNodes = rowNodes;
}

/**
 * Helper method to determine if selection mode allows the operation to continue.
 *
 * @param selectionMode The row selection mode.
 * @param referenceRowNode The reference row node.
 * @param rowNode The row node to check.
 *
 * @return True if the row node can be selected according to the selection mode.
 */
protected boolean canSelectRowNode(int selectionMode, RowNode referenceRowNode, RowNode rowNode) {
    switch (selectionMode) {
    case HiGrid.ROW_SELECT_IN_SAME_TABLE:
        // only if the row nodes are in the same data table
        if (referenceRowNode != null &&
            referenceRowNode.getDataTableModel() != rowNode.getDataTableModel()) {
            return false;
        }
        break;
    case HiGrid.ROW_SELECT_IN_SAME_LEVEL:
        // only if the row nodes are at the same level
        if (referenceRowNode != null &&
            referenceRowNode.getFormatNode() != rowNode.getFormatNode()) {
            return false;
        }
        break;
    case HiGrid.ROW_SELECT_ANY:
    default:
        // any record row node is ok
        break;
    }
    return true;
}

/**
 * Do row select/deselect based on mouse input
 *
 * @param rowNode The record row node where the mouse click occurred.
 * @param selectionMode The row selection mode.
 * @param groupSelection True if a group of rows from the last single
 * row selected should be selected.
 * @param refresh True if the rows are to be repainted.
 */
protected boolean doRowSelection(RowNode rowNode, int selectionMode,
                                 boolean groupSelection, boolean refresh) {
    if (rowNode == null || !(rowNode.getRowFormat() instanceof RecordFormat)) {
        return false;
    }

    // determine if selection mode allows the operation to continue
    if (!canSelectRowNode(selectionMode, lastSelectedRowNode, rowNode)) {
        return false;
    }

    if (groupSelection) {
        // always from the last selected to current
        if (lastSelectedRowNode == null) {
            return false;
        }
        RowNode savedLastRowNode = lastSelectedRowNode;
        if (!resetRowSelection()) {
            return false;
        }
        lastSelectedRowNode = savedLastRowNode;
        RowSelectWalk walkable = new RowSelectWalk(grid, selectionMode, lastSelectedRowNode, rowNode);
        grid.getRowTree().walk((RowNode)grid.getRowTree().getRootNode().getFirstChild(), walkable);
        if (!selectRows(walkable.getSelectedRows(), true)) {
            return false;
        }
        setSelectedRows(walkable.getSelectedRows());
    }
    else {
        // do single row select/deselect
        if (!selectRows(rowNode, !rowNode.isSelected())) {
            return false;
        }
        if (rowNode.isSelected()) {
            selectedRowNodes.addElement(rowNode);
            lastSelectedRowNode = rowNode;
        }
        else {
            selectedRowNodes.removeElement(rowNode);
        }
    }

    if (refresh) {
        if (groupSelection) {
            grid.refresh();
        }
        else {
            grid.refreshRowWithoutEditor(rowNode);
        }
    }
    return true;
}

/**
 * Do row select/deselect based on mouse input
 *
 * @param rowNode The record row node where the mouse click occurred.
 * @param selectionMode The row selection mode.
 * @param groupSelection True if a group of rows from the last single
 * row selected should be selected.
 */
protected boolean doRowSelection(RowNode rowNode, int selectionMode, boolean groupSelection) {
    return doRowSelection(rowNode, selectionMode, groupSelection, true);
}

/**
 * Do row select based on the selection mode.
 *
 * @param rowNode The record row node where the mouse click occurred.
 * @param selectionMode The row selection mode.
 * @return true if all selection operations succeeded
 */
protected boolean doRowSelection(RowNode rowNode, int selectionMode) {
    if (rowNode == null || !(rowNode.getRowFormat() instanceof RecordFormat)) {
        return false;
    }

    // determine if selection mode allows the operation to continue
    if (!canSelectRowNode(selectionMode, lastSelectedRowNode, rowNode)) {
        return false;
    }

    if (!resetRowSelection()) {
        return false;
    }

    if (selectionMode == HiGrid.ROW_SELECT_ANY) {
        CellPosition newPosition = goHome();
        RowNode newRowNode = (newPosition == null) ? null : newPosition.row;
        if (!doRowSelection(newRowNode, HiGrid.ROW_SELECT_ANY, false)) {
            return false;
        }
        newPosition = goEnd();
        newRowNode = (newPosition == null) ? null : newPosition.row;
        if (!doRowSelection(newRowNode, HiGrid.ROW_SELECT_ANY, true)) {
            return false;
        }
    }

    else if (selectionMode == HiGrid.ROW_SELECT_IN_SAME_TABLE) {
        CellPosition newPosition = goLevelHome();
        RowNode newRowNode = (newPosition == null) ? null : newPosition.row;
        if (!doRowSelection(newRowNode, HiGrid.ROW_SELECT_IN_SAME_TABLE, false)) {
            return false;
        }
        newPosition = goLevelEnd();
        newRowNode = (newPosition == null) ? null : newPosition.row;
        if (!doRowSelection(newRowNode, HiGrid.ROW_SELECT_IN_SAME_TABLE, true)) {
            return false;
        }
    }

    else if (selectionMode == HiGrid.ROW_SELECT_IN_SAME_LEVEL) {
        LevelSelectWalk walkable = new LevelSelectWalk(grid, rowNode);
        grid.getRowTree().walk((RowNode)grid.getRowTree().getRootNode().getFirstChild(), walkable);
        if (selectRows(walkable.getSelectedRows(), true)) {
            setSelectedRows(walkable.getSelectedRows());
            lastSelectedRowNode = walkable.first();
        }
    }

    // FIX this may not be optimal any more!
    grid.refresh();
    return true;
}

/**
 * Reset all row selection.
 */
protected boolean resetRowSelection() {
    return resetRowSelection(true);
}

/**
 * Reset all row selection.
 * @param refresh True if the rows are to be refreshed.
 */
protected boolean resetRowSelection(boolean refresh) {
    if (selectedRowNodes.size() == 0) {
        return true;
    }
    Vector v = (Vector)selectedRowNodes.clone();
    if (!selectRows(v, false)) {
        return false;
    }
    RowNode rowNode = (RowNode)(selectedRowNodes.elementAt(0));
    boolean minimalRefresh = (selectedRowNodes.size() == 1);
    selectedRowNodes.removeAllElements();
    lastSelectedRowNode = null;
    if (refresh) {
        if (minimalRefresh) {
            grid.refreshRowWithoutEditor(rowNode);
        }
        else {
            grid.refresh();
        }
    }
    return true;
}

/**
 * (De-)Select the passed RowNode.
 * The selection can be denied in the BEFORE_SELECT_ROW event.
 * @param rowNode A RowNode to select
 * @param state true for Select, false for Deselect
 * @return true if the selection was permitted, or empty
 */
protected boolean selectRows(RowNode rowNode, boolean state) {
    Vector vector = new Vector(1);
    vector.addElement(rowNode);
    return selectRows(vector, state);
}

/**
 * (De-)Select the passed Vector of rows.
 * The selection can be denied in the BEFORE_SELECT_ROW event.
 * @param rows A Vector of RowNodes to select
 * @param state true for Select, false for Deselect
 * @return true if the selection was permitted, or empty
 */
protected boolean selectRows(Vector rows, boolean state) {
    if (rows.size() == 0) {
        return true;
    }
    if (!grid.fireHiGridRowSelectionEvent(new HiGridRowSelectionEvent(grid,
        HiGridRowSelectionEvent.BEFORE_SELECT_ROW, state, rows))) {
        return false;
    }
    for (Enumeration e = rows.elements(); e.hasMoreElements();) {
        RowNode rowNode = (RowNode) e.nextElement();
        rowNode.setSelected(state);
    }
    grid.fireHiGridRowSelectionEvent(new HiGridRowSelectionEvent(grid,
        HiGridRowSelectionEvent.AFTER_SELECT_ROW, state, rows));
    return true;
}

// Internal class to support column moving
protected class JColumnHeaderLabel extends JLabel {

    protected void paintComponent(Graphics g) {
        g.setXORMode(Color.gray);
        g.setColor(getBackground());



        ((Graphics2D)g).fill(g.getClipBounds());

        g.setPaintMode();
        super.paintComponent(g);
    }
}

////////////////////////
// ActionHandling

/**
 * Adds an <CODE>ActionInitiator</CODE>/action pair to table.
 * @see #addAction(HiGridAction)
 */
public void addAction(ActionInitiator initiator, int action) {
    addAction(new HiGridAction(initiator, action));
}

/**
 * Adds a <CODE>HiGridAction</CODE> to table. A <CODE>HiGridAction</CODE>
 * maps an event to a particular action on HiGrid.
 */
public void addAction(HiGridAction action) {
    if (actions == null) {
        actions = new Vector();
    }
    actions.addElement(action);
}

/**
 * Returns the action mapped to the given event. If no action
 * exists, <CODE>HiGridAction.NO_ACTION</CODE> is returned.
 */
public int getAction(AWTEvent event) {
    if (actions != null) {
        for (int i = 0; i < actions.size(); i++) {
            HiGridAction action = (HiGridAction)actions.elementAt(i);
            if (action.isMatch(grid, event)) {
                return action.getAction();
            }
        }
    }
    return HiGridAction.NO_ACTION;
}

/**
 * Returns the specified KeyActionInitiator, If no matching ActionInitiator
 * exists, null is returned.
public KeyActionInitiator getKeyActionInitiator(int keycode, int modifier) {
    if (actions != null) {
        for (int i = 0; i < actions.size(); i++) {
            HiGridAction action = (HiGridAction)actions.elementAt(i);
            if (action.getInitiator() instanceof KeyActionInitiator) {
                KeyActionInitiator initiator = (KeyActionInitiator)action.getInitiator();
                if (initiator.keycode == keycode && initiator.modifier == modifier) {
                    return initiator;
                }
            }
        }
    }
    return null;
}
 */

/**
 * Returns the specified MouseActionInitiator, If no matching ActionInitiator
 * exists, null is returned.
 */
public MouseActionInitiator getMouseActionInitiator(int button_mask, int modifier) {
    if (actions != null) {
        for (int i = 0; i < actions.size(); i++) {
            HiGridAction action = (HiGridAction)actions.elementAt(i);
            if (action.getInitiator() instanceof MouseActionInitiator) {
                MouseActionInitiator initiator = (MouseActionInitiator)action.getInitiator();
                if (initiator.button_mask == button_mask && initiator.modifier == modifier) {
                    return initiator;
                }
            }
        }
    }
    return null;
}

/**
 * Removes an action from HiGrid.
 */
public void removeAction(HiGridAction action) {
    removeAction(action.getInitiator(), action.getAction());
}

/**
 * Removes an <CODE>ActionInitiator</CODE>/action pair from HiGrid.
 */
public void removeAction(ActionInitiator initiator, int action) {
    if (actions != null) {
        for (int i = 0; i < actions.size(); i++) {
            HiGridAction a = (HiGridAction)actions.elementAt(i);
            if (a.getInitiator() == initiator && a.action == action) {
                actions.removeElement(a);
            }
        }
        if (actions.size() == 0) {
            actions = null;
        }
    }
}

/**
 * Removes any <CODE>ActionInitiator</CODE> set for the given action type.
 */
public void removeAction(int action) {
    if (actions != null) {
        for (int i = 0; i < actions.size(); i++) {
            HiGridAction a = (HiGridAction)actions.elementAt(i);
            if (a.action == action) {
                actions.removeElement(a);
            }
        }
        if (actions.size() == 0) {
            actions = null;
        }
    }
}

/**
 * Removes all key actions from HiGrid.
 */
public void removeAllKeyActions() {
    if (actions != null) {
        for (int i = 0; i < actions.size(); i++) {
            HiGridAction a = (HiGridAction)actions.elementAt(i);
            if (a.getInitiator() instanceof KeyActionInitiator) {
                actions.removeElement(a);
            }
        }
        if (actions.size() == 0) {
            actions = null;
        }
    }
}

/**
 * Removes all mouse actions from HiGrid.
 */
public void removeAllMouseActions() {
    if (actions != null) {
        for (int i = 0; i < actions.size(); i++) {
            HiGridAction a = (HiGridAction)actions.elementAt(i);
            if (a.getInitiator() instanceof MouseActionInitiator) {
                actions.removeElement(a);
            }
        }
        if (actions.size() == 0) {
            actions = null;
        }
    }
}

/**
 * Removes all actions from HiGrid.
 */
public void removeAllActions() {
    if (actions != null) {
        actions.removeAllElements();
        actions = null;
    }
}

/**
 * Returns an enumeration of the HiGridActions
 *
 * @return An enumeration of all current HiGridActions
 */
public Enumeration getAllActions() {
    return actions.elements();
}

/**
 * Re-creates all standard actions. Call <CODE>removeAllActions()</CODE> first
 * if you want to start anew.
 */
public void createDefaultActions() {
    addAction(new KeyActionInitiator(KeyEvent.VK_LEFT, ActionInitiator.NO_MODIFIER_MASK),
                                     HiGridAction.MOVE_LEFT_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_RIGHT, ActionInitiator.NO_MODIFIER_MASK),
                                     HiGridAction.MOVE_RIGHT_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_UP, ActionInitiator.NO_MODIFIER_MASK),
                                     HiGridAction.MOVE_UP_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_DOWN, ActionInitiator.NO_MODIFIER_MASK),
                                     HiGridAction.MOVE_DOWN_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_PAGE_UP, ActionInitiator.NO_MODIFIER_MASK),
                                     HiGridAction.MOVE_PAGE_UP_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_PAGE_DOWN, ActionInitiator.NO_MODIFIER_MASK),
                                     HiGridAction.MOVE_PAGE_DOWN_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_PAGE_UP, InputEvent.ALT_MASK),
                                     HiGridAction.MOVE_PAGE_LEFT_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_PAGE_DOWN, InputEvent.ALT_MASK),
                                     HiGridAction.MOVE_PAGE_RIGHT_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_HOME, ActionInitiator.NO_MODIFIER_MASK),
                                     HiGridAction.MOVE_HOME_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_END, ActionInitiator.NO_MODIFIER_MASK),
                                     HiGridAction.MOVE_END_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_HOME, InputEvent.CTRL_MASK),
                                     HiGridAction.MOVE_GRID_HOME_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_END, InputEvent.CTRL_MASK),
                                     HiGridAction.MOVE_GRID_END_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_TAB, ActionInitiator.NO_MODIFIER_MASK),
                                     HiGridAction.MOVE_NEXT_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_TAB, InputEvent.SHIFT_MASK),
                                     HiGridAction.MOVE_PREVIOUS_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_ADD, InputEvent.CTRL_MASK),
                                     HiGridAction.OPEN_TABLE_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_SUBTRACT, InputEvent.CTRL_MASK),
                                     HiGridAction.CLOSE_TABLE_ACTION);

    // FIX temporary for testing
/*
    addAction(new KeyActionInitiator(KeyEvent.VK_A, InputEvent.CTRL_MASK),
                                     HiGridAction.COPY_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_B, InputEvent.CTRL_MASK),
                                     HiGridAction.PASTE_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_P, InputEvent.CTRL_MASK),
                                     HiGridAction.POPUP_MENU_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_S, InputEvent.CTRL_MASK),
                                     HiGridAction.COLUMN_SELECT_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_S, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK),
                                     HiGridAction.COLUMN_SELECT_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_Z, InputEvent.CTRL_MASK),
                                     HiGridAction.CANCEL_EDIT_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_Q, InputEvent.CTRL_MASK),
                                     HiGridAction.STOP_EDIT_ACTION);
*/
    addAction(new MouseActionInitiator(MouseEvent.BUTTON1_MASK, ActionInitiator.ANY_MODIFIER_MASK),
                                       HiGridAction.COLUMN_SELECT_ACTION);
    addAction(new MouseActionInitiator(MouseEvent.BUTTON3_MASK, ActionInitiator.ANY_MODIFIER_MASK),
                                       HiGridAction.POPUP_MENU_ACTION);
    addAction(new MouseActionInitiator(MouseEvent.BUTTON1_MASK, InputEvent.CTRL_MASK),
                                       HiGridAction.SINGLE_ROW_SELECT_ACTION);
    addAction(new MouseActionInitiator(MouseEvent.BUTTON1_MASK, InputEvent.SHIFT_MASK),
                                       HiGridAction.GROUP_ROW_SELECT_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_ENTER, ActionInitiator.ANY_MODIFIER_MASK),
                                     HiGridAction.IGNORE_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_ESCAPE, ActionInitiator.ANY_MODIFIER_MASK),
                                     HiGridAction.IGNORE_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_CONTROL, ActionInitiator.ANY_MODIFIER_MASK),
                                     HiGridAction.IGNORE_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_SHIFT, ActionInitiator.ANY_MODIFIER_MASK),
                                     HiGridAction.IGNORE_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_ALT, ActionInitiator.ANY_MODIFIER_MASK),
                                     HiGridAction.IGNORE_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_CAPS_LOCK, ActionInitiator.ANY_MODIFIER_MASK),
                                     HiGridAction.IGNORE_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_NUM_LOCK, ActionInitiator.ANY_MODIFIER_MASK),
                                     HiGridAction.IGNORE_ACTION);
    addAction(new KeyActionInitiator(KeyEvent.VK_SCROLL_LOCK, ActionInitiator.ANY_MODIFIER_MASK),
                                     HiGridAction.IGNORE_ACTION);
}

}
