/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DefaultEditStatusRenderer.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:19 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.awt.*;

import com.klg.jclass.cell.*;
import com.klg.jclass.cell.renderers.*;
import com.klg.jclass.datasource.*;
import com.klg.jclass.util.swing.JCIconCreator;

/**
 * The <CODE>DefaultEditStatusRenderer</CODE> class defines a 
 * <CODE>JCCellRenderer</CODE> 
 * object that paints {@link EditStatus} objects in a HiGrid edit status cell.
 */
public class DefaultEditStatusRenderer extends DefaultImageRenderer 
implements HiGridEditStatusRenderer {

static final long serialVersionUID = -2909248749782807724L;

transient private Image icons[] = null;

// Icons for Edit Status
private static final String updateCurrentIconString[] = {
    "             ",
    "        ...  ",
    "        .  . ",
    "       . ..  ",
    "       .  .  ",
    "      .  .   ",
    "      .  .   ",
    "     .  .    ",
    "     .  .    ",
    "     ...     ",
    "     ..      ",
    " . . .       ",
    "             ",
};

private static final String updatePendingIconString[] = {
    "             ",
    "        ggg  ",
    "        g  g ",
    "       g gg  ",
    "       g  g  ",
    "      g  g   ",
    "      g  g   ",
    "     g  g    ",
    "     g  g    ",
    "     ggg     ",
    "     gg      ",
    " g g g       ",
    "             ",
};

private static final String deleteCurrentIconString[] = {
    "...         .",
    "....      .. ",
    " ....    ..  ",
    "   ...  ..   ",
    "    .....    ",
    "     ...     ",
    "     ...     ",
    "    .....    ",
    "   ...  ..   ",
    " ....    ..  ",
    "....      .. ",
    "...         .",
    "             ",
};

private static final String deletePendingIconString[] = {
    "ggg         g",
    "gggg      gg ",
    " gggg    gg  ",
    "   ggg  gg   ",
    "    ggggg    ",
    "     ggg     ",
    "     ggg     ",
    "    ggggg    ",
    "   ggg  gg   ",
    " gggg    gg  ",
    "gggg      gg ",
    "ggg         g",
    "             ",
};

private static final String currentIconString[] = {
    "             ",
    "    .        ",
    "    ..       ",
    "    ...      ",
    "    ....     ",
    "    .....    ",
    "    ......   ",
    "    .....    ",
    "    ....     ",
    "    ...      ",
    "    ..       ",
    "    .        ",
    "             ",
};

/**
 * Default constructor
 */
public DefaultEditStatusRenderer() {
    recreateIcons();
}

/**
 * Allows lazy creation of icons and permits
 * cut and paste serialization.
 */
protected void recreateIcons() {
    if (icons == null) {
        icons = new Image[EditStatus.EDIT_COUNT];

        JCIconCreator icon = new JCIconCreator(13, 13);
        icon.setColor('.', Color.black);
        icon.setColor('r', new Color(255,0,0).darker());
        icon.setColor('g', Color.gray);

        icon.setPixels(updateCurrentIconString);
        setIcon(icon.getIcon().getImage(), EditStatus.EDIT_UPDATE_CURRENT);
        icon.clear();

        icon.setPixels(updatePendingIconString);
        setIcon(icon.getIcon().getImage(), EditStatus.EDIT_UPDATE_PENDING);
        icon.clear();

        icon.setPixels(deleteCurrentIconString);
        setIcon(icon.getIcon().getImage(), EditStatus.EDIT_DELETE_CURRENT);
        icon.clear();

        icon.setPixels(deletePendingIconString);
        setIcon(icon.getIcon().getImage(), EditStatus.EDIT_DELETE_PENDING);
        icon.clear();

        icon.setPixels(currentIconString);
        setIcon(icon.getIcon().getImage(), EditStatus.EDIT_CURRENT);
    }
}

/**
 * Retrieves the image for this object.
 *
 * @param type The type of icon to get.
 * @return The current icon of this type.
 */
public Image getIcon(int type) {
    recreateIcons();
    return icons[type];
}

/**
 * Sets the icon images for this object.
 *
 * @param type The type of icon to set.
 * @param icon The new icon of this type.
 */
public void setIcon(Image icon, int type) {
    recreateIcons();
    icons[type] = icon;
}

/**
 * Used to draw the cell. Note that the size of the cell comes from 
 * <CODE>JCCellInfo.getDrawingArea()</CODE>.  When the renderer's draw method is
 * called, the gc is translated to the upper-left corner of the drawing area 
 * for the cell.  The "drawing area" of the cell is the region <EM>inside</EM> 
 * the borders and the margins.
 * 
 * @param gc Graphics context to use for drawing.
 * @param cellFormat Cell property information.
 * @param o Object to be rendered.
 * @param selected Boolean value indicating whether the cell is selected.
 */
public void draw(Graphics gc, JCCellInfo cellFormat, Object o, boolean selected) {
    draw(gc, cellFormat, (EditStatus)o, selected);
}

/**
 * Used to draw the cell. Note that the size of the cell comes from 
 * <CODE>JCCellInfo.getDrawingArea()</CODE>.  When the renderer's draw method is
 * called, the gc is translated to the upper-left corner of the drawing area 
 * for the cell.  The "drawing area" of the cell is the region <EM>inside</EM> 
 * the borders and the margins.
 * 
 * @param gc Graphics context to use for drawing.
 * @param cellFormat Cell property information.
 * @param editStatus Information about the object to be rendered.
 * @param selected Boolean value indicating whether the cell is selected.
 */
public void draw(Graphics gc, JCCellInfo cellFormat, EditStatus editStatus, boolean selected) {
    // set all of the necessary display attributes from cellFormat
    Rectangle rect = cellFormat.getDrawingArea();
    Color background = selected ? cellFormat.getSelectedBackground() : cellFormat.getBackground();
    gc.setColor(background);
    gc.fillRect(rect.x, rect.y, rect.width, rect.height);

    if (editStatus == null) {
        return;
    }

    int imageIndex;

    // convert it to one of our states
    switch (editStatus.getStatus()) {
    case DataTableModel.UPDATED:
        imageIndex = editStatus.isCurrentRow() ? EditStatus.EDIT_UPDATE_CURRENT : EditStatus.EDIT_UPDATE_PENDING;
        break;
    case DataTableModel.INSERTED:
        imageIndex = editStatus.isCurrentRow() ? EditStatus.EDIT_UPDATE_CURRENT : EditStatus.EDIT_UPDATE_PENDING;
        break;
    case DataTableModel.DELETED:
        imageIndex = editStatus.isCurrentRow() ? EditStatus.EDIT_DELETE_CURRENT : EditStatus.EDIT_DELETE_PENDING;
        break;
    case DataTableModel.COMMITTED:
        imageIndex = editStatus.isCurrentRow() ? EditStatus.EDIT_CURRENT : -1;
        break;
    default:
        imageIndex = -1;
    }

    if (imageIndex < 0) {
// DEBUG only
/*
String s = Long.toString(editStatus.getBookmark());
JCLightCellRenderer stringRenderer = new JCStringCellRenderer();
stringRenderer.draw(gc, cellFormat, s , selected);
*/
        return;
    }

    // draw the status icon inside the cell
    super.draw(gc, cellFormat, getIcon(imageIndex), selected, null);
}

/**
 * Returns the preferred size of the cell's drawing area.  The "drawing area" 
 * of the cell is the region <EM>inside</EM> the borders and the margins.
 * 
 * @param JCCellInfo Cell property information.
 * @return <code>Dimension</code> instance containing preferrred height and
 * width.
 */
public Dimension getPreferredSize(Graphics gc, JCCellInfo JCCellInfo, Object o) {
    Rectangle rect = JCCellInfo.getDrawingArea();
    return new Dimension(rect.width, rect.height);
}

}
