/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DefaultErrorHandler.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:19 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.io.Serializable;

import com.klg.jclass.util.swing.JCMessageHelper;

/**
 * <code>DefaultErrorHandler</code> is the default error handler for HiGrid Error events.
 */

public class DefaultErrorHandler implements HiGridErrorHandler, Serializable {

static final long serialVersionUID = -2814928107046785138L;

private HiGrid grid = null;
private boolean showErrorDialog = true;
private String errorDialogTitle = null;

/**
 * Constructor
 */
public DefaultErrorHandler(HiGrid grid) {
    this.grid = grid;
}

/**
 * Sets the show error dialog indication.  Default is show.
 *
 * @param show True if to show error dialog.
 */
public void setShowErrorDialog(boolean show) {
    showErrorDialog = show;
}

/**
 * Sets the title to the error dialog.
 *
 * @param title The title.
 */
public void setErrorDialogTitle(String title) {
    errorDialogTitle = title;
}

/**
 * Invoked when a HiGrid Error event has occurred.
 */
public void processError(HiGridErrorEvent event) {
    Exception exception = event.getException();
    // show the error dialog
    if (showErrorDialog) {
        String title = (errorDialogTitle != null) ? errorDialogTitle
                       : LocaleBundle.string(LocaleBundle.defaultErrorDialogTitle);
        String message = exception.getMessage();
        if (message == null || message.length() == 0) {
            message = exception.toString();
        }
        JCMessageHelper.showError(title, message);
    }
}

}
