/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DefaultSortable.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:19 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.io.Serializable;
import java.util.*;



/**
 * <code>DefaultSortable</code> is the default HiGrid implementation of
 * the Comparator interface. It sorts all items based on their data type.
 * Any unrecognized types are sorted based on their toString() value.
 * @since JClass 4.0K for Java 2. If you are using JClass 4.0J, 
 * which is based on JDK 1.1, the Comparator implementation 
 * is based on <code>com.sun.java.util.collections.Comparator</code>.
 */
public class DefaultSortable implements Serializable, Comparator {

static final long serialVersionUID = 6191367176391554164L;

public static final int LESS_THAN    = -1;
public static final int EQUAL        = 0;
public static final int GREATER_THAN = 1;

/**
 * Compares its two arguments for order. 
 *
 * @return a negative integer, zero, or a positive integer as the first argument
 * is less than, equal to, or greater than the second.
 */
public int compare(Object a, Object b) {
    if (a == null || b == null) {
        return EQUAL;
    }

    if (a instanceof String) {
        if (b instanceof String) {
            return((String)a).compareTo((String)b);
        }
        else {
            return a.toString().compareTo(b.toString());
        }
    }
    else if (a instanceof Double) {
        if (b instanceof Double) {
            double doubleA = ((Double)a).doubleValue();
            double doubleB = ((Double)b).doubleValue();
            if (doubleA < doubleB)
                return LESS_THAN;
            else if (doubleA == doubleB)
                return EQUAL;
            return GREATER_THAN;
        }
        else {
            return a.toString().compareTo(b.toString());
        }
    }
    else if (a instanceof Float) {
        if (b instanceof Float) {
            float floatA = ((Float)a).floatValue();
            float floatB = ((Float)b).floatValue();
            if (floatA < floatB)
                return LESS_THAN;
            else if (floatA == floatB)
                return EQUAL;
            return GREATER_THAN;
        }
        else {
            return a.toString().compareTo(b.toString());
        }
    }
    else if (a instanceof java.math.BigDecimal) {
        if (b instanceof java.math.BigDecimal) {
            return((java.math.BigDecimal)a).compareTo((java.math.BigDecimal)b);
        }
        else {
            return a.toString().compareTo(b.toString());
        }
    }
    else if (a instanceof Integer) {
        if (b instanceof Integer) {
            int intA = ((Integer)a).intValue();
            int intB = ((Integer)b).intValue();
            if (intA < intB)
                return LESS_THAN;
            else if (intA == intB)
                return EQUAL;
            return GREATER_THAN;
        }
        else {
            return a.toString().compareTo(b.toString());
        }
    }
    else if (a instanceof Long) {
        if (b instanceof Long) {
            long longA = ((Long)a).longValue();
            long longB = ((Long)b).longValue();
            if (longA < longB)
                return LESS_THAN;
            else if (longA == longB)
                return EQUAL;
            return GREATER_THAN;
        }
        else {
            return a.toString().compareTo(b.toString());
        }
    }
    else if (a instanceof Short) {
        if (b instanceof Short) {
            short shortA = ((Short)a).shortValue();
            short shortB = ((Short)b).shortValue();
            if (shortA < shortB)
                return LESS_THAN;
            else if (shortA == shortB)
                return EQUAL;
            return GREATER_THAN;
        }
        else {
            return a.toString().compareTo(b.toString());
        }
    }
    else if (a instanceof Byte) {
        if (b instanceof Byte) {
            byte byteA = ((Byte)a).byteValue();
            byte byteB = ((Byte)b).byteValue();
            if (byteA < byteB)
                return LESS_THAN;
            else if (byteA == byteB)
                return EQUAL;
            return GREATER_THAN;
        }
        else {
            return a.toString().compareTo(b.toString());
        }
    }
    else if (a instanceof Boolean) {
        if (b instanceof Boolean) {
            boolean booleanA = ((Boolean)a).booleanValue();
            boolean booleanB = ((Boolean)b).booleanValue();
            if (booleanA && booleanB)
                return EQUAL;
            else if (!booleanA && booleanB)
                return LESS_THAN;
            return GREATER_THAN;
        }
        else {
            return a.toString().compareTo(b.toString());
        }
    }
    else if (a instanceof java.sql.Date) {
        if (b instanceof java.sql.Date) {
            long dateA = ((java.sql.Date)a).getTime();
            long dateB = ((java.sql.Date)b).getTime();
            if (dateA < dateB)
                return LESS_THAN;
            else if (dateA == dateB)
                return EQUAL;
            return GREATER_THAN;
        }
        else {
            return a.toString().compareTo(b.toString());
        }
    }
    else if (a instanceof java.util.Date) {
        if (b instanceof java.util.Date) {
            long dateA = ((java.util.Date)a).getTime();
            long dateB = ((java.util.Date)b).getTime();
            if (dateA < dateB)
                return LESS_THAN;
            else if (dateA == dateB)
                return EQUAL;
            return GREATER_THAN;
        }
        else {
            return a.toString().compareTo(b.toString());
        }
    }
    else if (a instanceof java.sql.Time) {
        if (b instanceof java.sql.Time) {
            long dateA = ((java.sql.Time)a).getTime();
            long dateB = ((java.sql.Time)b).getTime();
            if (dateA < dateB)
                return LESS_THAN;
            else if (dateA == dateB)
                return EQUAL;
            return GREATER_THAN;
        }
        else {
            return a.toString().compareTo(b.toString());
        }
    }
    else if (a instanceof java.sql.Timestamp) {
        if (b instanceof java.sql.Timestamp) {
            long dateA = ((java.sql.Timestamp)a).getTime();
            long dateB = ((java.sql.Timestamp)b).getTime();
            if (dateA < dateB)
                return LESS_THAN;
            else if (dateA == dateB)
                return EQUAL;
            return GREATER_THAN;
        }
        else {
            return a.toString().compareTo(b.toString());
        }
    }
/*
    // Calendar
    else if (a instanceof Calendar) {
        if (b instanceof Calendar) {
            Calendar ac = (Calendar)a;
            Calendar bc = (Calendar)b;
            if (ac.equals(bc)) return EQUAL;
            if (ac.before(bc))
                return LESS_THAN;
            else
                return GREATER_THAN;
        }
        else {
            return a.toString().compareTo(b.toString());
        }
    }
*/
    // everything else
    else {
        return a.toString().compareTo(b.toString());
    }
}

}
