/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: EditPopupMenu.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:19 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.tree.*;

import com.klg.jclass.datasource.*;
import com.klg.jclass.util.swing.*;

/**
 * The EditPopupMenu class is the default popup menu for runtime editing
 * of HiGrid.
 */
public class EditPopupMenu extends JPopupMenu implements ActionListener {

static final long serialVersionUID = 8019952542645451744L;

/** default short popup menu list */
public final static int DEFAULT_SHORT_POPUPMENU_LIST    = 0;
/** default long popup menu list */
public final static int DEFAULT_LONG_POPUPMENU_LIST     = 1;

/** List of enums for the default edit popup menu. */
public final static int POPUP_INSERT                        = 0;
public final static int POPUP_APPEND                        = 1;
public final static int POPUP_DELETE                        = 2;
public final static int POPUP_DELETE_CURRENT                = 3;
public final static int POPUP_DELETE_SELECTED               = 4;
public final static int POPUP_CANCEL                        = 5;
public final static int POPUP_CANCEL_CURRENT                = 6;
public final static int POPUP_CANCEL_SELECTED               = 7;
public final static int POPUP_CANCEL_ALL                    = 8;
public final static int POPUP_REQUERY                       = 9;
public final static int POPUP_REQUERY_RECORD                = 10;
public final static int POPUP_REQUERY_RECORD_AND_DETAILS    = 11;
public final static int POPUP_REQUERY_SELECTED              = 12;
public final static int POPUP_REQUERY_SELECTED_AND_DETAILS  = 13;
public final static int POPUP_REQUERY_TABLE                 = 14;
public final static int POPUP_REQUERY_ALL                   = 15;
public final static int POPUP_UPDATE                        = 16;
public final static int POPUP_UPDATE_CURRENT                = 17;
public final static int POPUP_UPDATE_SELECTED               = 18;
public final static int POPUP_UPDATE_ALL                    = 19;
public final static int POPUP_SELECT                        = 20;
public final static int POPUP_SELECT_CURRENT                = 21;
public final static int POPUP_SELECT_ALL_IN_SAME_LEVEL      = 22;
public final static int POPUP_SELECT_ALL_IN_SAME_TABLE      = 23;
public final static int POPUP_SELECT_ALL                    = 24;
public final static int POPUP_MOVE_TO_RECORD                = 25;
public final static int POPUP_MOVE_TO_RECORD_FIRST          = 26;
public final static int POPUP_MOVE_TO_RECORD_PREVIOUS       = 27;
public final static int POPUP_MOVE_TO_RECORD_NEXT           = 28;
public final static int POPUP_MOVE_TO_RECORD_LAST           = 29;
public final static int POPUP_MOVE_TO_TABLE_RECORD          = 30;
public final static int POPUP_MOVE_TO_TABLE_RECORD_FIRST    = 31;
public final static int POPUP_MOVE_TO_TABLE_RECORD_PREVIOUS = 32;
public final static int POPUP_MOVE_TO_TABLE_RECORD_NEXT     = 33;
public final static int POPUP_MOVE_TO_TABLE_RECORD_LAST     = 34;
public final static int POPUP_MOVE_TO_PARENT                = 35;
public final static int POPUP_COLLAPSE_PARENT               = 36;
public final static int POPUP_PRINT                         = 37;
public final static int POPUP_PRINT_AS_DISPLAYED            = 38;
public final static int POPUP_PRINT_AS_EXPANDED             = 39;
public final static int POPUP_PRINT_EXPAND_ALL              = 40;
public final static int POPUP_PRINT_PREVIEW                 = 41;
public final static int POPUP_PRINT_PREVIEW_AS_DISPLAYED    = 42;
public final static int POPUP_PRINT_PREVIEW_AS_EXPANDED     = 43;
public final static int POPUP_PRINT_PREVIEW_EXPAND_ALL      = 44;
public final static int POPUP_TIPS                          = 45;
public final static int POPUP_TIPS_KEYBOARD                 = 46;
public final static int POPUP_TIPS_MOUSE                    = 47;
public final static int POPUP_TIPS_MENU                     = 48;

// convenience indices
protected final static int NO_RECORDS_START_INDEX = POPUP_MOVE_TO_RECORD;
protected final static int NO_RECORDS_END_INDEX = POPUP_TIPS_MENU+1;

// Note: MUST be in the SAME sequence as the list of enums (indices) above
protected String[] commands = new String[] {
    LocaleBundle.insert,  
    LocaleBundle.append,  
    LocaleBundle.delete, LocaleBundle.deleteCurrent, 
    LocaleBundle.deleteSelected, 
    LocaleBundle.cancel, LocaleBundle.cancelCurrent, 
    LocaleBundle.cancelSelected, LocaleBundle.cancelAll,
    LocaleBundle.requery, LocaleBundle.requeryRecord, 
    LocaleBundle.requeryRecordAndDetails, LocaleBundle.requerySelected,
    LocaleBundle.requerySelectedAndDetails,
    LocaleBundle.requeryTable, LocaleBundle.requeryAll,
    LocaleBundle.update, LocaleBundle.updateCurrent, 
    LocaleBundle.updateSelected, LocaleBundle.updateAll,
    LocaleBundle.select, LocaleBundle.selectCurrent, 
    LocaleBundle.selectAllSameLevel, LocaleBundle.selectAllSameTable, 
    LocaleBundle.selectAll,
    LocaleBundle.moveToGridRecord, LocaleBundle.moveToGridRecordFirst,
    LocaleBundle.moveToGridRecordPrevious, LocaleBundle.moveToGridRecordNext,
    LocaleBundle.moveToGridRecordLast,
    LocaleBundle.moveToTableRecord, LocaleBundle.moveToTableRecordFirst,
    LocaleBundle.moveToTableRecordPrevious, LocaleBundle.moveToTableRecordNext,
    LocaleBundle.moveToTableRecordLast,
    LocaleBundle.moveToParent, LocaleBundle.collapseParent,
    LocaleBundle.print,
    LocaleBundle.printAsDisplayed, LocaleBundle.printAsExpanded,
    LocaleBundle.printExpandAll,
    LocaleBundle.printPreview,
    LocaleBundle.printPreviewAsDisplayed, LocaleBundle.printPreviewAsExpanded,
    LocaleBundle.printPreviewExpandAll,
    LocaleBundle.tips, LocaleBundle.tipsKeyboard, LocaleBundle.tipsMouse,
    LocaleBundle.tipsMenu,
};

// list of tables that can be inserted based on current cell's rowNode
protected Vector insertList;

protected HiGrid grid;

/**
 * Default Constructor
 */
public EditPopupMenu(HiGrid grid) {
    super();
    this.grid = grid;

    // debug check
    if (!((String)LocaleBundle.tipsMenu).equals(getCommand(POPUP_TIPS_MENU))) {
        System.out.println("HiGrid : Error in EditPopupMenu command set.");
    }

    // setup the default popup menu list
    setDefaultMenuList(DEFAULT_LONG_POPUPMENU_LIST);        
}

/*
 * Remove references for proper garbage collection.
 */
public void dispose() {
    setInvoker(null);
    grid = null;
    insertList = null;
    removeAll();
}

////////////////////////
// ActionListener methods
public void actionPerformed(ActionEvent event) {
    performAction(event);
}

/** 
 * Sets the popup menu to the indicated default list.
 *
 * @param list The default list indication.
 */
public void setDefaultMenuList(int list) {
    PopupMenuList popupMenuList = new PopupMenuList((JPopupMenu) this, this);
    popupMenuList.setMenu(list);
}

/**
 * Create a menu component from the default popup menu list.
 * The default label and command text are used.
 *
 * @param index The popup menu enum value.
 *
 * @return The created menu.
 */
public JMenuItem createMenu(int index) {
    String command = getCommand(index);
    if (command == null) {
        return null;
    }
    JMenu menu = new JMenu(LocaleBundle.string(command));
    if (menu != null) {
        menu.setActionCommand(command);
    }
    return menu;
}

/**
 * Create a menu item component from the default popup menu list.
 * The default label and command text are used and HiGrid
 * will be the action listener to this menu item.
 *
 * @param index The popup menu item enum value.
 *
 * @return The created menu item.
 */
public JMenuItem createMenuItem(int index) {
    String command = getCommand(index);
    if (command == null) {
        return null;
    }
    JMenuItem mi = new JMenuItem(LocaleBundle.string(command));
    if (mi != null) {
        mi.setActionCommand(command);
        mi.addActionListener(this);
    }
    return mi;
}

/**
 * Retrieves the specified command string.
 *
 * @param index The index for the command string.
 *
 * @return The command string.
 */
public String getCommand(int index) {
    if (index >=0 && index < commands.length) {
        return commands[index];
    }
    return null;
}

/**
 * Retrieves the position of the specified command from the subElements.
 *
 * @param subElements The submenu to search.
 * @param command The command id.
 *
 * @return The index of the command id.
 */
protected int findMenuIndex(MenuElement[] subElements, int command) {
    // check if insert menu item is present
    int menuIndex = (-1);
    int count = subElements.length;
    String stringCommand = getCommand(command);
    for (int i = 0; i < count; i++) {
        if (((JMenuItem)subElements[i]).getText().equals(stringCommand)) {
            menuIndex = i;
            break;
        }
    }
    return menuIndex;
}

/**
 * Creates a submenu based on the passed command (currently insert/append)
 *
 * @param insertMenu The submenu to modify.
 * @param command The command string.
 */
protected void buildDynamicSubMenu(JMenuItem insertMenu, String command) {
    insertMenu.removeAll();
    for (Enumeration e = insertList.elements(); e.hasMoreElements();) {
        FormatNode formatNode = (FormatNode) e.nextElement();
        JMenuItem mi = new JMenuItem(formatNode.getName());
        if (mi != null) {
            mi.setActionCommand(command + " " + formatNode.toString());
            mi.addActionListener(this);
            insertMenu.add(mi);
        }
    }
}

/**
 * Show the popup menu.
 *
 * @param x The x position of the cursor.
 * @param y The y position of the cursor.
 */
public void show(int x, int y) {
    MenuElement[] subElements = getSubElements();

    // check if menu items are present
    int insertMenuIndex = findMenuIndex(subElements, POPUP_INSERT);
    int appendMenuIndex = findMenuIndex(subElements, POPUP_APPEND);

    // add current available levels to insert
    CellPosition cell = grid.getCurrentCell();
    if (cell != null && cell.row != null) {
        insertList = grid.getRowTree().getNodeLevelList(cell.row);
    }
    else if (cell == null && grid.getDataModel() != null) {
        insertList = grid.getRowTree().getNodeLevelList(null);
    }
    if (insertList != null) {
        if (insertMenuIndex != (-1)) {
            JMenuItem insertMenu = (JMenuItem) ((JMenuItem)subElements[insertMenuIndex]);
            buildDynamicSubMenu(insertMenu, LocaleBundle.insert);
        }
        if (appendMenuIndex != (-1)) {
            JMenuItem appendMenu = (JMenuItem) ((JMenuItem)subElements[appendMenuIndex]);
            buildDynamicSubMenu(appendMenu, LocaleBundle.append);
        }
    }

    // enable/disable all edit menu items
    boolean editable = grid.isEditable();
    if (insertList != null) {
        enableMenuItem((JPopupMenu)this, getCommand(POPUP_INSERT), editable);
        enableMenuItem((JPopupMenu)this, getCommand(POPUP_APPEND), editable);
    }
    if (cell == null) {
        for (int i = NO_RECORDS_START_INDEX; i < NO_RECORDS_END_INDEX; i++) {
            enableMenuItem((JPopupMenu)this, getCommand(i), false);
        }
        enableMenuItem((JPopupMenu)this, getCommand(POPUP_DELETE), false);
        enableMenuItem((JPopupMenu)this, getCommand(POPUP_CANCEL), false);
        enableMenuItem((JPopupMenu)this, getCommand(POPUP_UPDATE), false);
        enableMenuItem((JPopupMenu)this, getCommand(POPUP_SELECT), false);
        enableMenuItem((JPopupMenu)this, getCommand(POPUP_REQUERY), false);
    }
    else {
        for (int i = NO_RECORDS_START_INDEX; i < NO_RECORDS_END_INDEX; i++) {
            enableMenuItem((JPopupMenu)this, getCommand(i), true);
        }
        enableMenuItem((JPopupMenu)this, getCommand(POPUP_REQUERY), true);

        // enable/disable all edit menu items
        enableMenuItem((JPopupMenu)this, getCommand(POPUP_DELETE), editable);
        enableMenuItem((JPopupMenu)this, getCommand(POPUP_CANCEL), editable);
        enableMenuItem((JPopupMenu)this, getCommand(POPUP_UPDATE), editable);

        // enable/disable all menu items for selected rows and select menu items
        boolean selection = !grid.getController().selectedRowNodes.isEmpty();
        enableMenuItem((JPopupMenu)this, getCommand(POPUP_DELETE_SELECTED), selection);
        enableMenuItem((JPopupMenu)this, getCommand(POPUP_CANCEL_SELECTED), selection);
        enableMenuItem((JPopupMenu)this, getCommand(POPUP_REQUERY_SELECTED), selection);
        enableMenuItem((JPopupMenu)this, getCommand(POPUP_REQUERY_SELECTED_AND_DETAILS), selection);
        enableMenuItem((JPopupMenu)this, getCommand(POPUP_UPDATE_SELECTED), selection);
        enableMenuItem((JPopupMenu)this, getCommand(POPUP_SELECT), grid.isAllowRowSelection());
        switch (grid.getRowSelectionMode()) {
        case HiGrid.ROW_SELECT_IN_SAME_TABLE:
            // only if the row nodes are in the same data table
            enableMenuItem((JPopupMenu)this, getCommand(POPUP_SELECT_ALL), false);
            enableMenuItem((JPopupMenu)this, getCommand(POPUP_SELECT_ALL_IN_SAME_LEVEL), false);
            break;
        case HiGrid.ROW_SELECT_IN_SAME_LEVEL:
            // only if the row nodes are at the same level
            enableMenuItem((JPopupMenu)this, getCommand(POPUP_SELECT_ALL), false);
            enableMenuItem((JPopupMenu)this, getCommand(POPUP_SELECT_ALL_IN_SAME_LEVEL), true);
            break;
        case HiGrid.ROW_SELECT_ANY:
        default:
            // any record row node is ok
            enableMenuItem((JPopupMenu)this, getCommand(POPUP_SELECT_ALL), true);
            enableMenuItem((JPopupMenu)this, getCommand(POPUP_SELECT_ALL_IN_SAME_LEVEL), true);
            break;
        }
    }

    // need this, otherwise first popup will think it has a size of (0, 0)
    this.getComponent().setSize(this.getComponent().getPreferredSize());
    Point p = new Point(x, y);
    SwingUtilities.convertPointToScreen(p, grid.getGridArea());
    Rectangle popupBounds = computePopupBounds(p.x, p.y, this.getWidth(), this.getHeight());
    p.setLocation(popupBounds.x, popupBounds.y);
    SwingUtilities.convertPointFromScreen(p, grid.getGridArea());
    // show the popup menu
    super.show(grid.getGridArea(), p.x, p.y);
}

protected Rectangle computePopupBounds(int x, int y, int width, int height) {
    Dimension scrSize = Toolkit.getDefaultToolkit().getScreenSize();
    Rectangle screenBounds = new Rectangle(0, 0, scrSize.width, scrSize.height);
    Point p = new Point(x, y);
    Rectangle popupBounds = new Rectangle(p.x, p.y, width, height);
    if (isRectangleContainingRectangle(screenBounds, popupBounds) ) {
        return popupBounds;
    }
    else {
        // find a place for it to fit
        if (! isRectangleContainingRectangleFitRight(screenBounds, popupBounds)) {
            popupBounds.x = (screenBounds.width - (popupBounds.width) );
        }
        if (! isRectangleContainingRectangleFitLeft(screenBounds, popupBounds)) {
            popupBounds.x = screenBounds.x;
        }
        if (! isRectangleContainingRectangleFitDown(screenBounds, popupBounds)) {
            popupBounds.y = popupBounds.y - popupBounds.height;
        }
        if (! isRectangleContainingRectangleFitUp(screenBounds, popupBounds)) {
            popupBounds.y = screenBounds.y;
        }
        return popupBounds;
    }
}

private final boolean isRectangleContainingRectangle(Rectangle a, Rectangle b) {
    return (b.x >= a.x && (b.x + b.width) <= (a.x + a.width) &&
            b.y >= a.y && (b.y + b.height) <= (a.y + a.height));
}

private final boolean isRectangleContainingRectangleFitLeft(Rectangle a, Rectangle b) {
    return (a.x < b.x);
}
    
private final boolean isRectangleContainingRectangleFitRight(Rectangle a, Rectangle b) {
    return ((b.x + b.width) < (a.x + a.width));
}
    
private final boolean isRectangleContainingRectangleFitDown(Rectangle a, Rectangle b) {
    return (b.y + b.height) <= (a.y + a.height);
}

private final boolean isRectangleContainingRectangleFitUp(Rectangle a, Rectangle b) {
    return (a.y < b.y);
}

/**
 * Enable or disable a menu item in the popup menu.
 *
 * @param menu The menu level.
 * @param command The command string.
 * @param enable True if enable else false.
 * 
 * @return True if menu item is found and is enabled or is disabled.
 */
public boolean enableMenuItem(Object menu, String command, boolean enable) {
    MenuElement[] subElements;
    if (menu instanceof JPopupMenu) {
        subElements = ((JPopupMenu)menu).getSubElements();
    }
    else {
        subElements = (((JMenu)menu).getPopupMenu()).getSubElements();
    }
    int count = subElements.length;
    for (int i = 0; i < count; i++) {
        MenuElement me = subElements[i];
        if (me instanceof JMenu) {
            if (enableMenuItem(me, command, enable)) {
                return true;
            }
        }
        else if (me instanceof JMenuItem) {
            JMenuItem mi = (JMenuItem)me;
            String itemCommand = mi.getActionCommand();
            if (itemCommand.equals(command)) {
                mi.setEnabled(enable);  
                return true;
            }
        }
    }
    return false;
}

/**
 * Handles a popup menu action event.
 *
 * @param event The action event.
 */
public void performAction(ActionEvent event) {
    performCommand(event.getActionCommand());
}

/**
 * Execute one of the popup menu's commands.
 *
 * @param command The POPUP_ command to execute.
 */
public void performCommand(int command) {
    performCommand(getCommand(command));
}

/**
 * Traverse to a new cell in the grid.
 *
 * @param newPosition New cell position to traverse to.
 * @param columnIndex New desired column index.
 */
private void traverse(CellPosition newPosition, int columnIndex) {
    grid.getController().validateAndTraverse(newPosition, columnIndex, null);
}

/**
 *  Display the HiGrid tips window.
 *
 * @param title The frame window's title.
 * @param text The HTML text to display in the window.
 */
protected void showTips(String title, String text) {
//    JCMessageHelper.showInformation(
//        LocaleBundle.string(title), LocaleBundle.string(text));
    JDialog dlg = new JDialog();
    dlg.setTitle(LocaleBundle.string(title));
    dlg.setModal(false);
    JCHTMLPane textArea = new JCHTMLPane();
    textArea.setText(LocaleBundle.string(text));
//    JTextArea textArea = new JTextArea(LocaleBundle.string(text));
    JScrollPane scrollPane = new JScrollPane(textArea);
    dlg.getContentPane().add(scrollPane);
    dlg.pack();
    dlg.setSize(600, 400);
    dlg.setVisible(true);
}

/**
 * Execute one of the popup menu's commands.
 *
 * @param command The command to execute.
 */
public void performCommand(String command) {
    if (command == null) {
        return;
    }
    CellPosition cell = grid.getCurrentCell();
    DataTableModel dataTable = null;
    long bookmark = 0;
    DataModel dataModel = grid.getDataModel();
    if (cell != null && cell.row != null) {
        dataTable = cell.row.getDataTableModel();
        bookmark = cell.row.getBookmark();
    }
    else if (cell == null && dataModel != null) {
        dataTable = (DataTableModel) dataModel.getDataTableTree().getRoot();
    }
    if (dataTable == null || dataModel == null) {
        return;
    }

    boolean actionPerformed = true;
    boolean resetSelectedRows = false;
    Enumeration en = null;
    Vector cloneSelected = null;
    Controller controller = grid.getController();
    try {
        if (command.equals(getCommand(POPUP_DELETE_CURRENT))) {
            dataTable.deleteRow(grid.getDataModelListener(), bookmark);
            resetSelectedRows = true;
        }

        else if (command.equals(getCommand(POPUP_DELETE_SELECTED))) {
            boolean batched = grid.isBatched();
            grid.setBatched(true);
            cloneSelected = (Vector) controller.selectedRowNodes.clone();
            controller.resetRowSelection(false);
            try {
                for (en = cloneSelected.elements(); en.hasMoreElements();) {
                    RowNode rowNode = (RowNode) en.nextElement();
                    if (rowNode != null) {
                        DataTableModel dataTableSelected = rowNode.getDataTableModel();
                        if (dataTableSelected != null) {
                            dataTableSelected.deleteRow(grid.getDataModelListener(), rowNode.getBookmark());
                        }
                    }
                }
            }
            catch (Exception e) {
                grid.fireHiGridErrorEvent(new HiGridErrorEvent(grid, e));
            }
            grid.setBatched(batched);
        }

        else if (command.equals(getCommand(POPUP_CANCEL_CURRENT))) {
            controller.cancelCellEditing();
            dataTable.cancelRowChanges(grid.getDataModelListener(), bookmark);
            resetSelectedRows = true;
        }

        else if (command.equals(getCommand(POPUP_CANCEL_SELECTED))) {
            bookmark = cell.row.getBookmark();
            cloneSelected = (Vector) controller.selectedRowNodes.clone();
            controller.resetRowSelection(false);
            for (en = cloneSelected.elements(); en.hasMoreElements();) {
                RowNode rowNode = (RowNode) en.nextElement();
                if (rowNode != null) {
                    DataTableModel dataTableSelected = rowNode.getDataTableModel();
                    if (dataTableSelected != null) {
                        dataTableSelected.cancelRowChanges(grid.getDataModelListener(), rowNode.getBookmark());
                        if (bookmark == rowNode.getBookmark()) {
                            controller.cancelCellEditing();
                        }
                    }
                }
            }
        }

        else if (command.equals(getCommand(POPUP_CANCEL_ALL))) {
            controller.resetRowSelection(false);
            controller.cancelCellEditing();
            dataModel.cancelAll(grid.getDataModelListener());
        }

        else if (command.equals(getCommand(POPUP_REQUERY_RECORD))) {
            dataTable.requeryRow(grid.getDataModelListener(), bookmark);
            resetSelectedRows = true;
        }

        else if (command.equals(getCommand(POPUP_REQUERY_SELECTED))) {
            cloneSelected = (Vector) controller.selectedRowNodes.clone();
            controller.resetRowSelection(false);
            for (en = cloneSelected.elements(); en.hasMoreElements();) {
                RowNode rowNode = (RowNode) en.nextElement();
                if (rowNode != null) {
                    DataTableModel dataTableSelected = rowNode.getDataTableModel();
                    if (dataTableSelected != null) {
                        dataTableSelected.requeryRow(grid.getDataModelListener(), rowNode.getBookmark());
                    }
                }
            }
        }

        else if (command.equals(getCommand(POPUP_REQUERY_RECORD_AND_DETAILS))) {
            dataTable.requeryRowAndDetails(grid.getDataModelListener(), bookmark);
            resetSelectedRows = true;
        }

        else if (command.equals(getCommand(POPUP_REQUERY_SELECTED_AND_DETAILS))) {
            cloneSelected = (Vector) controller.selectedRowNodes.clone();
            controller.resetRowSelection(false);
            for (en = cloneSelected.elements(); en.hasMoreElements();) {
                RowNode rowNode = (RowNode) en.nextElement();
                if (rowNode != null) {
                    DataTableModel dataTableSelected = rowNode.getDataTableModel();
                    if (dataTableSelected != null) {
                        dataTableSelected.requeryRowAndDetails(grid.getDataModelListener(), rowNode.getBookmark());
                    }
                }
            }
        }

        else if (command.equals(getCommand(POPUP_REQUERY_TABLE))) {
            dataTable.requeryTable(grid.getDataModelListener());
            resetSelectedRows = true;
        }

        else if (command.equals(getCommand(POPUP_REQUERY_ALL))) {
            dataModel.requeryAll(grid.getDataModelListener());
            resetSelectedRows = true;
        }

        else if (command.equals(getCommand(POPUP_UPDATE_CURRENT))) {
            dataTable.commitRow(grid.getDataModelListener(), bookmark);
            resetSelectedRows = true;
        }

        else if (command.equals(getCommand(POPUP_UPDATE_SELECTED))) {
            for (en = controller.selectedRowNodes.elements(); en.hasMoreElements();) {
                RowNode rowNode = (RowNode) en.nextElement();
                if (rowNode != null) {
                    DataTableModel dataTableSelected = rowNode.getDataTableModel();
                    if (dataTableSelected != null) {
                        dataTableSelected.commitRow(grid.getDataModelListener(), rowNode.getBookmark());
                    }
                }
            }
            resetSelectedRows = true;
        }

        else if (command.equals(getCommand(POPUP_UPDATE_ALL))) {
            controller.resetRowSelection(false);
            dataModel.updateAll(grid.getDataModelListener());
        }

        else if (command.equals(getCommand(POPUP_SELECT_CURRENT))) {
            controller.resetRowSelection();
            controller.doRowSelection(cell.row, grid.getRowSelectionMode(), false);
        }

        else if (command.equals(getCommand(POPUP_SELECT_ALL_IN_SAME_LEVEL))) {
            controller.doRowSelection(cell.row, HiGrid.ROW_SELECT_IN_SAME_LEVEL);
        }

        else if (command.equals(getCommand(POPUP_SELECT_ALL_IN_SAME_TABLE))) {
            controller.doRowSelection(cell.row, HiGrid.ROW_SELECT_IN_SAME_TABLE);
        }

        else if (command.equals(getCommand(POPUP_SELECT_ALL))) {
            controller.doRowSelection(cell.row, HiGrid.ROW_SELECT_ANY);
        }

        else if (command.equals(getCommand(POPUP_MOVE_TO_RECORD_FIRST))) {
            CellPosition newPosition = controller.goHome();
            traverse(newPosition, controller.getDesiredColumn());
            resetSelectedRows = true;
        }

        else if (command.equals(getCommand(POPUP_MOVE_TO_RECORD_PREVIOUS))) {
            CellPosition newPosition = controller.goUp();
            traverse(newPosition, controller.getDesiredColumn());
            resetSelectedRows = true;
        }

        else if (command.equals(getCommand(POPUP_MOVE_TO_RECORD_NEXT))) {
            CellPosition newPosition = controller.goDown();
            traverse(newPosition, controller.getDesiredColumn());
            resetSelectedRows = true;
        }

        else if (command.equals(getCommand(POPUP_MOVE_TO_RECORD_LAST))) {
            CellPosition newPosition = controller.goEnd();
            traverse(newPosition, controller.getDesiredColumn());
            resetSelectedRows = true;
        }

        else if (command.equals(getCommand(POPUP_MOVE_TO_TABLE_RECORD_FIRST))) {
            CellPosition newPosition = controller.goLevelHome();
            traverse(newPosition, controller.getDesiredColumn());
            resetSelectedRows = true;
        }

        else if (command.equals(getCommand(POPUP_MOVE_TO_TABLE_RECORD_PREVIOUS))) {
            CellPosition newPosition = controller.goLevelUp();
            traverse(newPosition, controller.getDesiredColumn());
            resetSelectedRows = true;
        }

        else if (command.equals(getCommand(POPUP_MOVE_TO_TABLE_RECORD_NEXT))) {
            CellPosition newPosition = controller.goLevelDown();
            traverse(newPosition, controller.getDesiredColumn());
            resetSelectedRows = true;
        }

        else if (command.equals(getCommand(POPUP_MOVE_TO_TABLE_RECORD_LAST))) {
            CellPosition newPosition = controller.goLevelEnd();
            traverse(newPosition, controller.getDesiredColumn());
            resetSelectedRows = true;
        }

        else if (command.equals(getCommand(POPUP_MOVE_TO_PARENT))) {
            CellPosition newPosition = controller.goLevelParent();
            if (newPosition != null && cell.row != newPosition.row) {
                traverse(newPosition, controller.getDesiredColumn());
            }
            resetSelectedRows = true;
        }

        else if (command.equals(getCommand(POPUP_COLLAPSE_PARENT))) {
            CellPosition newPosition = controller.goLevelParent();
            if (newPosition != null && cell.row != newPosition.row) {
                RowNode rowNode = newPosition.row;
                if (!grid.fireHiGridExpansionEvent(new HiGridExpansionEvent(grid,
                                                                            HiGridExpansionEvent.BEFORE_COLLAPSE_ROW, rowNode))) {
                    return;
                }
                traverse(newPosition, controller.getDesiredColumn());
                if (rowNode.getState() == NodeStatus.FOLDER_OPEN) {
                    grid.refreshRowAndBelow(rowNode, NodeStatus.FOLDER_OPEN);
                }
            }
            resetSelectedRows = true;
        }

        else if (command.equals(getCommand(POPUP_PRINT_AS_DISPLAYED))) {
            controller.resetRowSelection();
            grid.setPrintFormat(HiGrid.PRINT_AS_DISPLAYED);
            grid.print();
        }

        else if (command.equals(getCommand(POPUP_PRINT_AS_EXPANDED))) {
            controller.resetRowSelection();
            grid.setPrintFormat(HiGrid.PRINT_AS_EXPANDED);
            grid.print();
        }

        else if (command.equals(getCommand(POPUP_PRINT_EXPAND_ALL))) {
            controller.resetRowSelection();
            grid.setPrintFormat(HiGrid.PRINT_EXPAND_ALL);
            grid.print();
        }

        else if (command.equals(getCommand(POPUP_PRINT_PREVIEW_AS_DISPLAYED))) {
            controller.resetRowSelection();
            grid.setPrintFormat(HiGrid.PRINT_AS_DISPLAYED);
            grid.printPreview();
        }

        else if (command.equals(getCommand(POPUP_PRINT_PREVIEW_AS_EXPANDED))) {
            controller.resetRowSelection();
            grid.setPrintFormat(HiGrid.PRINT_AS_EXPANDED);
            grid.printPreview();
        }
        else if (command.equals(getCommand(POPUP_PRINT_PREVIEW_EXPAND_ALL))) {
            controller.resetRowSelection();
            grid.setPrintFormat(HiGrid.PRINT_EXPAND_ALL);
            grid.printPreview();
        }
        else if (command.equals(getCommand(POPUP_TIPS_KEYBOARD))) {
            showTips(LocaleBundle.tipsKeyboard, LocaleBundle.tipsKeyboardText);
        }
        else if (command.equals(getCommand(POPUP_TIPS_MOUSE))) {
            showTips(LocaleBundle.tipsMouse, LocaleBundle.tipsMouseText);
        }
        else if (command.equals(getCommand(POPUP_TIPS_MENU))) {
            showTips(LocaleBundle.tipsMenu, LocaleBundle.tipsMenuText);
        }
        else {
            actionPerformed = false;
        }

        // handle inserting a new row at the formatNode's level
        if (!actionPerformed && insertList != null) {
            for (en = insertList.elements(); en.hasMoreElements();) {
                FormatNode formatNode = (FormatNode) en.nextElement();
                String s = formatNode.toString();
                boolean isInsert = command.equals(LocaleBundle.insert + " " + s);
                boolean isAppend = command.equals(LocaleBundle.append + " " + s);
                if (isInsert || isAppend) {
                    actionPerformed = true;
                    DataTableModel dataTableInsert = null;
                    // get the DataTableModel to insert row
                    boolean isSubTable = (cell != null && cell.row.getFormatNode() != formatNode);
                    if (!isSubTable) {
                        dataTableInsert = dataTable;
                    }
                    else {
                        RowTree rt = grid.getRowTree();
                        if (cell.row.getState() == NodeStatus.FOLDER_CLOSED) {
                            if (!grid.fireHiGridExpansionEvent(new HiGridExpansionEvent(grid,
                                                                                        HiGridExpansionEvent.BEFORE_EXPAND_ROW, cell.row))) {
                                return;
                            }
                            rt.openFolder(cell.row);
                            DataModelListener dml = grid.getDataModelListener();
                            if (dml instanceof DefaultDataModelListener) {
                                ((DefaultDataModelListener)dml).setLastOpenedRowNode(cell.row);
                            }
                        }
                        dataTableInsert = rt.getDataTableModel(formatNode, cell.row);
                        // if no DataTableModel in row tree, 
                        // may be a node with no record rows
                        if (dataTableInsert == null) {
                            dataTableInsert = dataTable.getTable(bookmark, (TreeNode)formatNode.getRecordFormat().getMetaData());
                        }
                    }
                    // insert row in found DataTableModel, before indicated row
                    if (dataTableInsert != null) {
                        int rowIndex;
                        if (isSubTable) {
                            rowIndex = isInsert ? 0 : -1;
                        }
                        else {
                            rowIndex = (cell == null) ? 0 : ((isInsert) ? dataTableInsert.getRowIndex(cell.row.getBookmark())
                                                                        : dataTableInsert.getRowIndex(cell.row.getBookmark())+1);

                        }
                        dataTableInsert.addRow(grid.getDataModelListener(), 
                                               dataTableInsert.createNewRow(),
                                               DataTableModel.INSERTED,
                                               rowIndex);
                    }
                    resetSelectedRows = true;
                    break;
                }
            }
        }
        if (resetSelectedRows) {
            controller.resetRowSelection();
        }
    }
    catch (Exception e) {
        grid.fireHiGridErrorEvent(new HiGridErrorEvent(grid, e));
    }
}

}
