/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: EditStatus.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:19 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.io.Serializable;

/**
 * The <code>EditStatus</code> class packages together some status information
 * for display by a HiGridEditStatusRenderer.
 */
public class EditStatus implements Serializable {

static final long serialVersionUID = -5157735527208117590L;

// Enums
/**
 * Row is marked as modified, and is the current row.
 */
public static final int EDIT_UPDATE_CURRENT = 0;
/**
 * Row is marked as modified, and is not the current row.
 */
public static final int EDIT_UPDATE_PENDING = 1;
/**
 * Row is marked as deleted, and is the current row.
 */
public static final int EDIT_DELETE_CURRENT = 2;
/**
 * Row is marked as deleted, and is not the current row.
 */
public static final int EDIT_DELETE_PENDING = 3;
/**
 * Row is up to date, and is the current row.
 */
public static final int EDIT_CURRENT = 4;
/**
 * Total number of states possible.
 */
public static final int EDIT_COUNT = 5;

private HiGrid grid;
private int status;
private boolean currentRow;
private long bookmark;

/**
 * Default Constructor
 */
public EditStatus(HiGrid grid, int status, boolean currentRow, long bookmark) {
    this.grid = grid;
    this.status = status;
    this.currentRow = currentRow;
    this.bookmark = bookmark;
}

/**
 * Retrieves the instance of HiGrid.
 * @return the instance of HiGrid
 */
public HiGrid getGrid() {
    return grid;
}

/**
 * Retrieves the EDIT_ status for this object.
 * @return the EDIT_ status for this object.
 */
public int getStatus() {
    return status;
}

/**
 * Retrieves whether this EditStatus is the current row in the dataModel.
 * @return whether this EditStatus is the current row in the dataModel.
 */
public boolean isCurrentRow() {
    return currentRow;
}

/**
 * Retrieves the bookmark for this row.
 * @return the bookmark for this row.
 */
public long getBookmark() {
    return bookmark;
}

}
