/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: FocusManagerNew.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:19 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.awt.Component;
import java.awt.Window;
import java.awt.KeyboardFocusManager;

import com.klg.jclass.util.JCUtil;


/**
 * A FocusManager object for JDK 1.4 and later
 */
public class FocusManagerNew extends FocusManager {

/**
 * Create the FocusManagerNew for the given gridArea with no parameters.  One must
 * call setGrid() after creation.
 */
FocusManagerNew()
{
	super();
}

/**
 * Create the FocusManger for the given gridArea
 * @param gridArea The gridArea in which this focus manager is used.
 */
FocusManagerNew(GridArea gridArea)
{
	super(gridArea);
}


/**
 * Set the gridArea that uses this focus manager
 *
 * @param gridArea The new gridArea that uses this focus manager
 */
public void setGridArea(GridArea gridArea)
{
	super.setGridArea(gridArea);
	if (gridArea != null) {
	    gridArea.setFocusTraversalKeysEnabled(false);
		gridArea.setFocusCycleRoot(true);
	}
}


/**
 * Gets the passed in component to request focus.
 * @param c The component who will request the focus.
 *
 * @return Whether the request will fail or most likely succeed (it only
 *     fails under an extraordinary event such as a disposal of the
 *     Component's peer).
 */
public boolean requestFocus(Component c)
{
	
	if (c == null) {
		return(requestFocus());
	}
	else {
	    return(c.requestFocusInWindow());
	}
}

/**
 * Gets gridArea to request focus.
 *
 * @return Whether the request will fail or most likely succeed (it only
 *     fails under an extraordinary event such as a disposal of the
 *     Component's peer).
 */
public boolean requestFocus()
{
	
	boolean returnValue = false;
    if (gridArea != null) {
		returnValue = gridArea.requestFocusInWindow();
	}
	
	return(returnValue);
}

/**
 * Can this component receive the focus
 *
 * @param c The component whose focusability is in question
 * @return Whether the component can receive focus
 */
public boolean isFocusTraversable(Component c)
{
	 if (c == null) {
		return(false);
	 }
	 return(c.isFocusable());
}

/**
 * Return the child <code>Component</code> of the specified
 * <code>Component</code> that is the focus owner, if any.
 *
 * @param c the root of the <code>Component</code> hierarchy to search for the
 *     focus owner
 * @return the focus owner, or <code>null</code> if there is no focus owner, or
 *     if the focus owner is not <code>c</code>, or a descendant of
 *     <code>c</code>
 */
public Component findFocusOwner(Component c)
{
	if (c == null) {
		return(null);
	}

	Component focusOwner =
	    KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusOwner();
	

	// Verify focusOwner is a descendant of c
	for (Component temp = focusOwner;
	     temp != null;
	     temp = (temp instanceof Window) ? null : temp.getParent())
	{
	    if (temp == c) {
		    return(focusOwner);
	    }
	}

	return(null);
}

}
