/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: FolderIconStyle.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:19 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.awt.*;
import java.io.Serializable;

import com.klg.jclass.util.swing.JCIconCreator;

/**
 * The <code>FolderIconStyle</code> class defines the folder icon style to be used.
 */
public class FolderIconStyle implements Serializable {

static final long serialVersionUID = 6876423810461928052L;

/**
 * User defined folder icon style.
 */
public static final int FOLDER_ICON_STYLE_USER_DEFINED = 0;
/**
 * Shortcut folder icon style.
 */
public static final int FOLDER_ICON_STYLE_SHORTCUT = 1;
/**
 * Folder folder icon style.
 */
public static final int FOLDER_ICON_STYLE_FOLDER = 2;
/**
 * Triangle folder icon style.
 */
public static final int FOLDER_ICON_STYLE_TRIANGLE = 3;
/**
 * Small line 3D folder icon style.
 */
public static final int FOLDER_ICON_STYLE_SMALL_LINE_3D = 4;
/**
 * Medium line 3D folder icon style.
 */
public static final int FOLDER_ICON_STYLE_MEDIUM_LINE_3D = 5;
/**
 * Large line 3D folder icon style.
 */
public static final int FOLDER_ICON_STYLE_LARGE_LINE_3D = 6;
/**
 * Turner folder icon style.
 */
public static final int FOLDER_ICON_STYLE_TURNER = 7;

protected static final int FIRST_FOLDER_ICON_STYLE = 1;
protected static final int LAST_FOLDER_ICON_STYLE = 7;
protected static final int DEFAULT_FOLDER_ICON_STYLE = 1;

// default folder icon style
private static final String shortcutOpenIcon[] = {
    ".........",
    ".       .",
    ".       .",
    ".       .",
    ". ..... .",
    ".       .",
    ".       .",
    ".       .",
    ".........",
};

private static final String shortcutClosedIcon[] = {
    ".........",
    ".       .",
    ".   .   .",
    ".   .   .",
    ". ..... .",
    ".   .   .",
    ".   .   .",
    ".       .",
    ".........",
};

private HiGrid grid = null;
private int style;
transient private Image folderIcons[] = null;

/**
 * Constructor
 */
public FolderIconStyle(HiGrid grid) {
    this.grid = grid;
    recreateIcons();
}

/**
 * Allows lazy creation of icons and permits
 * cut and paste serialization.
 */
protected void recreateIcons() {
    if (folderIcons == null) {
        folderIcons = new Image[NodeStatus.FOLDER_COUNT];
        setStyle(DEFAULT_FOLDER_ICON_STYLE);
    }
}

/**
 * Computes the maximum height of all of the icons.
 *
 * @return The maximum height of all of the icons.
 */
protected int getMaximumHeight() {
    int height = 0;
    for (int i = 0; i < NodeStatus.FOLDER_COUNT; i++) {
        Image image = getFolderIcon(i);
        if (image != null) {
            height = Math.max(height, image.getHeight(null));
        }
    }
    return height;
}

/**
 * Computes the maximum width of all of the icons.
 *
 * @return The maximum width of all of the icons.
 */
protected int getMaximumWidth() {
    int width = 0;
    for (int i = 0; i < NodeStatus.FOLDER_COUNT; i++) {
        Image image = getFolderIcon(i);
        if (image != null) {
            width = Math.max(width, image.getWidth(null));
        }
    }
    return width;
}

/**
 * Retrieves the folder style index.
 *
 * @return The folder style index.
 */
protected int getStyle() {
    return style;
}

/**
 * Ensure that the entire node will be displayed.
 */
protected void validateGridNodeWidth() {
    int maximumWidth = getMaximumWidth();
    if (maximumWidth > grid.getNodeWidth()) {
        grid.setNodeWidth(maximumWidth);
    }
}

/**
 * Set the folder icons given the style index.
 *
 * @param style The style index.
 */
protected void setStyle(int style) {
    if (style < FIRST_FOLDER_ICON_STYLE || style > LAST_FOLDER_ICON_STYLE) {
        style = DEFAULT_FOLDER_ICON_STYLE;
    }
    JCIconCreator icon = null;
    if (style == DEFAULT_FOLDER_ICON_STYLE) {
        icon = new JCIconCreator(9, 9);
        icon.setColor('X', new Color(235,235,128));
        icon.setColor('.', Color.black);
        icon.setPixels(shortcutOpenIcon);
        setIcon(icon.getIcon().getImage(), NodeStatus.FOLDER_OPEN);
        icon.clear();
        icon.setPixels(shortcutClosedIcon);
        setIcon(icon.getIcon().getImage(), NodeStatus.FOLDER_CLOSED);
        setIcon(null, NodeStatus.FOLDER_NONE);
    }
    else {
        FolderIcons icons = new FolderIcons(grid, this);
        icons.setStyle(style);
    }
    this.style = style;
    validateGridNodeWidth();
}

/**
 * Retrieves the current image for the specified folder state.
 *
 * @param type The type of icon to get.
 * @return The current icon of this type.
 */
public Image getFolderIcon(int type) {
    recreateIcons();
    return folderIcons[type];
}

/**
 * Sets the current image for the specified folder state.
 *
 * @param type The type of icon to set.
 * @param icon The new icon of this type.
 */
public void setFolderIcon(Image icon, int type) {
    recreateIcons();
    folderIcons[type] = icon;
    style = FOLDER_ICON_STYLE_USER_DEFINED;
    validateGridNodeWidth();
}

/**
 * Sets the current image for the specified folder state.
 * This is for supplied folder styles.
 *
 * @param type The type of icon to set.
 * @param icon The new icon of this type.
 */
protected void setIcon(Image icon, int type) {
    recreateIcons();
    folderIcons[type] = icon;
}

}
