/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: HeaderTipWalk.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:19 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import javax.swing.tree.*;

/**
 * The HeaderTipWalk class is used as a container of information
 * during recursive searches of the RowTree.  It is used
 * to determine whether a headerTip should be displayed.
 */

public class HeaderTipWalk implements Walkable {

static final long serialVersionUID = 3803023194486635795L;

private RowNode startRowNode;
private RowNode topRowNode;
private HeaderFormat startHeaderFormat;
private RecordFormat startRecordFormat;
private int rowTipDisplay;
private boolean done;
private boolean headerVisible;
private boolean recordsContiguous;

public HeaderTipWalk(RowNode topRowNode, RowNode startRowNode, int rowTipDisplay) {
    this.topRowNode = topRowNode;
    this.startRowNode = startRowNode;
    this.rowTipDisplay = rowTipDisplay;
    startHeaderFormat = startRowNode.getFormatNode().getHeaderFormat();
    startRecordFormat = (RecordFormat)startRowNode.getRowFormat();
    done = false;
    headerVisible = false;
    recordsContiguous = true;
}

public boolean getDownward() {
    return false;
}

public boolean done() {
    return done;
}

public void next(RowNode rowNode) {
    if (rowNode != null && rowNode.getParent() != null) {
        if ((rowNode.getRowFormat() instanceof HeaderFormat) &&
            rowNode.getRowFormat().equals(startHeaderFormat)) {
            headerVisible = true;
        }
        if ((rowNode.getRowFormat() instanceof RecordFormat) &&
            !(rowNode.getRowFormat().equals(startRecordFormat))) {
            if (!headerVisible) {
                recordsContiguous = false;
            }
        }
    }
}

public void previous(RowNode rowNode) {
}

public boolean find(RowNode rowNode) {
    if (rowNode == topRowNode) {
        done = true;
    }
    return done;
}

public boolean shouldDisplay() {
    boolean display;
    switch (rowTipDisplay) {
    case HiGrid.HEADER_TIP_DISPLAY_LIKE_REPEAT_HEADER:
        display = !headerVisible || !recordsContiguous;
        break;
    case HiGrid.HEADER_TIP_DISPLAY_MINIMAL:
        display = !headerVisible;
        break;
    case HiGrid.HEADER_TIP_DISPLAY_ALWAYS:
    default:
        display = true;
        break;
    }
    return display;
}

}
