/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: HiGrid.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:19 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.awt.*;
import java.awt.event.*;
import java.util.*;



import java.awt.print.*;


import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.tree.*;

import com.klg.jclass.util.*;
import com.klg.jclass.cell.*;
import com.klg.jclass.cell.validate.*;
import com.klg.jclass.datasource.*;
import com.klg.jclass.higrid.beans.JCHiGridExternalDS;

/**
 * The core class for JClass HiGrid.
 * <P>
 * JClass HiGrid is an outline-grid that employs JClass
 * DataSource for managing, displaying and updating master-detail relational
 * data.
 *
 */
public class HiGrid extends JComponent implements ComponentListener,
JCValidateListener {

static final long serialVersionUID = 2547033485726164720L;

// Enums

/** Row Selection Mode: All record rows are selectable. */
public static final int ROW_SELECT_ANY             = 0;
/** Row Selection Mode: All record rows at the same format level are selectable. */
public static final int ROW_SELECT_IN_SAME_LEVEL   = 1;
/** Row Selection Mode: All record rows in the same data table are selectable. */
public static final int ROW_SELECT_IN_SAME_TABLE   = 2;

/** Row Selection Option: Select the current row. */
public static final int SELECT_ROWS_CURRENT                 = 0;
/** Row Selection Option: Select the table rows in the same level. */
public static final int SELECT_ROWS_ALL_IN_SAME_LEVEL       = 1;
/** Row Selection Option: Select the current table rows. */
public static final int SELECT_ROWS_ALL_IN_SAME_TABLE       = 2;
/** Row Selection Option: Select all rows. */
public static final int SELECT_ROWS_ALL                     = 3;

/** Row Deletion Option: Delete the current row. */
public static final int DELETE_ROWS_CURRENT     = 0;
/** Row Deletion Option: Delete the selected rows. */
public static final int DELETE_ROWS_SELECTED    = 1;

/** Cancel Row Changes Option: Cancel changes for the current row. */
public static final int CANCEL_ROWS_CURRENT     = 0;
/** Cancel Row Changes Option: Cancel changes for the selected rows. */
public static final int CANCEL_ROWS_SELECTED    = 1;
/** Cancel Row Changes Option: Cancel changes for all rows. */
public static final int CANCEL_ROWS_ALL         = 2;

/** Update Row Changes Option: Commit the changes for the current row . */
public static final int UPDATE_ROWS_CURRENT     = 0;
/** Update Row Changes Option: Commit the changes for the selected rows. */
public static final int UPDATE_ROWS_SELECTED    = 1;
/** Update Row Changes Option: Commit the changes for all rows. */
public static final int UPDATE_ROWS_ALL         = 2;

/** Requery Row Option: Requery the current row. */
public static final int REQUERY_ROWS_RECORD                 = 0;
/** Requery Row Option: Requery the current row and its details . */
public static final int REQUERY_ROWS_RECORD_AND_DETAILS     = 1;
/** Requery Row Option: Requery the selected rows. */
public static final int REQUERY_ROWS_SELECTED               = 2;
/** Requery Row Option: Requery the selected rows and the details of each selected row. */
public static final int REQUERY_ROWS_SELECTED_AND_DETAILS   = 3;
/** Requery Row Option: Requery all rows. */
public static final int REQUERY_ROWS_ALL                    = 4;

/** Move To Row Option: Move to first row of HiGrid. */
public static final int MOVE_TO_ROW_HIGRID_FIRST        = 0;
/** Move To Row Option: Move to previous row of HiGrid. */
public static final int MOVE_TO_ROW_HIGRID_PREVIOUS     = 1;
/** Move To Row Option: Move to next row of HiGrid. */
public static final int MOVE_TO_ROW_HIGRID_NEXT         = 2;
/** Move To Row Option: Move to last row of HiGrid. */
public static final int MOVE_TO_ROW_HIGRID_LAST         = 3;
/** Move To Row Option: Move to first row of the current table. */
public static final int MOVE_TO_ROW_TABLE_FIRST         = 4;
/** Move To Row Option: Move to previous row of the current table. */
public static final int MOVE_TO_ROW_TABLE_PREVIOUS      = 5;
/** Move To Row Option: Move to next row of the current table. */
public static final int MOVE_TO_ROW_TABLE_NEXT          = 6;
/** Move To Row Option: Move to last row of the current table. */
public static final int MOVE_TO_ROW_TABLE_LAST          = 7;
/** Move To Row Option: Move to parent of the current row. */
public static final int MOVE_TO_ROW_PARENT              = 8;

/** Print the rows as displayed. */
public static final int PRINT_AS_DISPLAYED         = 0;
/** Print the rows expanding all currently expanded format levels. */
public static final int PRINT_AS_EXPANDED          = 1;
/** Print the rows expanding all levels. */
public static final int PRINT_EXPAND_ALL      = 2;

/** Fit the cell editor to the cell size. */
public static final int EDIT_SIZE_TO_CELL = 0;
/** Use the cell editor's minimum size. */
public static final int EDIT_ENSURE_MINIMUM_SIZE = 1;
/** Use the cell editor's preferred size. */
public static final int EDIT_ENSURE_PREFERRED_SIZE = 2;

/** Display the header tip only if the header is not visible in the grid. */
public static final int HEADER_TIP_DISPLAY_MINIMAL = 0;
/** Display the header tip only if the header is not visible above similar contiguous records.
    This acts as a dynamic repeat header. */
public static final int HEADER_TIP_DISPLAY_LIKE_REPEAT_HEADER = 1;
/** Always display the header tip for all records. */
public static final int HEADER_TIP_DISPLAY_ALWAYS = 2;

// initial width of edit status column
public static final int INITIAL_NODE_WIDTH = 20;
public static final int INITIAL_EDIT_STATUS_WIDTH = 20;
public static final int INITIAL_LEVEL_INDENT = 25;
public static final int INITIAL_EXTRA_WIDTH = 10;

protected static final int NODE_INDICATOR_CELL = -10;
protected static final int EDITSTATUS_INDICATOR_CELL = -11;
protected static final int NUMBER_OF_INDICATOR_CELLS = 2;
protected static final int FIRST_EDITABLE_COLUMN = 0;

// States
private boolean firstLayout = true;

// Controller (Input Handler)
protected Controller controller = null;

// LookupTable (for renderers and editors)
protected LookupTable lookupTable = null;

// Properties
private JPanel horizontalPanel;
private JPanel verticalPanel;
private JPanel cornerPanel;
private GridScrollbar horizontalScrollbar;
private GridScrollbar verticalScrollbar;
private FormatTree formatTree = null;
private RowTree rowTree = null;
protected FolderIconStyle folderIconStyle = null;

// Settable Properties
private DataModel dataModel = null;
private GridArea gridArea = null;
protected DataModelListener dataModelListener = null;
protected boolean autoEdit = false;
protected boolean batched = false;
protected boolean beep = true;
protected boolean connectionsVisible = true;
protected Cursor defaultCursor = null;
protected boolean editable = true;
protected boolean editorHidden = false;
protected int extraWidth = 0;
protected boolean headerTipVisible;
protected int headerTipDisplay;
protected boolean rowHeightResizingAll = false;
protected boolean sortIconsVisible = true;
protected boolean trackCursor = true;
protected int verticalScrollbarConstraints = SwingConstants.EAST;
protected int horizontalScrollbarConstraints = SwingConstants.SOUTH;
protected Border editorBorder = null;
protected CellStyleModel headerCellStyle = null;
protected CellStyleModel recordCellStyle = null;
protected CellStyleModel footerCellStyle = null;
protected CellStyleModel beforeDetailsCellStyle = null;
protected CellStyleModel afterDetailsCellStyle = null;
protected boolean scrollOnMoveEvent	= true;

// Drawable properties
private boolean levelIndented = true;
private int nodeWidth = INITIAL_NODE_WIDTH;
private int editStatusWidth = INITIAL_EDIT_STATUS_WIDTH;

// Handlers
protected HiGridErrorHandler errorHandler = null;
protected HiGridColumnSelectionListener columnSelectionHandler = null;

// Event listeners
protected JCListenerList hiGridColumnSelectionListeners = null;
protected JCListenerList hiGridErrorListeners = null;
protected JCListenerList hiGridExpansionListeners = null;
protected JCListenerList hiGridFormatNodeListeners = null;
protected JCListenerList hiGridMoveCellListeners = null;
protected JCListenerList hiGridPrintListeners = null;
protected JCListenerList hiGridRepaintListeners = null;
protected JCListenerList hiGridResizeCellListeners = null;
protected JCListenerList hiGridRowSelectionListeners = null;
protected JCListenerList hiGridSortTableListeners = null;
protected JCListenerList hiGridTraverseListeners = null;
protected JCListenerList hiGridUpdateListeners = null;
protected JCListenerList hiGridValidateListeners = null;

// Callbacks
protected AutoTraverseCallback autoTraverseCallback = null;

// popup menu property
protected boolean allowPopupMenu = true;
protected EditPopupMenu popupMenu = null;

// selection properties
protected boolean allowRowSelection = true;
protected int rowSelectionMode = ROW_SELECT_ANY;

// printing properties
protected boolean printing = false;
protected PrintGrid printGrid = null;
protected boolean printFoldersAndConnections = false;
protected int printFormat = PRINT_AS_DISPLAYED;
protected int printMetaID[] = null;

// sorting properties
protected boolean allowSorting = true;
protected Comparator defaultSortable = null;
protected Comparator sortable = null;

// bean design time flag
protected transient boolean adviseClearAllFormats;

private boolean fNewFocusModel = false;

// Constructor
public HiGrid() {
	fNewFocusModel = (JCEnvironment.getJavaVersion() >= 140);

    // setup the error handler
    setErrorHandler(new DefaultErrorHandler(this));

    // setup the column selection handler
    setColumnSelectionHandler(new DefaultColumnSelectionHandler(this));

    // setup the controller (input handler)
    setController(new Controller(this));

    // setup the dataModelListener
    setDataModelListener(new DefaultDataModelListener(this));

    // create the lookup tables for the renderers and editors
    setLookupTable(new LookupTable(this));

    // setup the PrintGrid



    setPrintGrid(new PrintGrid12(this));


    // setup default sortable
    defaultSortable = new DefaultSortable();
    sortable = defaultSortable;

    // create the contained components
    setEditPopupMenu(new EditPopupMenu(this));
    verticalScrollbar = new GridScrollbar(this, GridScrollbar.VERTICAL);
    horizontalPanel = new JPanel();
    horizontalPanel.setLayout(new BorderLayout());
    verticalPanel = new JPanel();
    verticalPanel.setLayout(new BorderLayout());
    horizontalScrollbar = new GridScrollbar(this, GridScrollbar.HORIZONTAL);
    cornerPanel = new JPanel();
    int cornerWidth = verticalScrollbar.getComponent().getPreferredSize().width;
    cornerPanel.setPreferredSize(new Dimension(cornerWidth, cornerWidth));
    setGridArea(new GridArea(this));

    // setup listeners
    addComponentListener(this);

    // initialize default cellStyles
    setHeaderCellStyle(new DefaultHeaderCellStyle());
    setRecordCellStyle(new DefaultRecordCellStyle());
    setFooterCellStyle(new DefaultFooterCellStyle());
    setBeforeDetailsCellStyle(new DefaultBeforeDetailsCellStyle());
    setAfterDetailsCellStyle(new DefaultAfterDetailsCellStyle());

    // create and set default editor border
    setEditorBorder(getDefaultEditorBorder());

    // make header tips visible by default
    setHeaderTipVisible(true);
    setHeaderTipDisplay(HiGrid.HEADER_TIP_DISPLAY_MINIMAL);

    // amount of extra space at end of HiGrid
    setExtraWidth(INITIAL_EXTRA_WIDTH);

    // setup the design time tree
    setFormatTree(new FormatTree(this));

    // setup the run time tree
    setRowTree(new RowTree(this));

    // setup the default folder icon style
    folderIconStyle = new FolderIconStyle(this);
    folderIconStyle.setStyle(FolderIconStyle.DEFAULT_FOLDER_ICON_STYLE);

    // init vertical scrollbar
    verticalScrollbar.initValues();

    // init horizontal scrollbar
    horizontalScrollbar.initValues();

    // set cursor used by HiGrid
    setDefaultCursor(Cursor.DEFAULT_CURSOR);

    // this minimizes redraws of the GridArea
    setDoubleBuffered(true);
    setOpaque(true);

    // install UI based on JTable
    UIManager.getDefaults().put("HiGridUI", "com.klg.jclass.higrid.HiGridUI");
    setUI(HiGridUI.createUI(this));


 
 }

/**
 * Makes this Component displayable.
 */
public void addNotify() {


 

    super.addNotify();
}

/**
 * Returns the UI for HiGrid.
 */
public HiGridUI getUI() {
	return (HiGridUI)ui;
}

/**
 * Sets the UI for HiGrid.
 */
public void setUI(ComponentUI newUI) {
	super.setUI(newUI);
}

/**
 * Updates the UI for HiGrid.
 */
public void updateUI() {
	setUI((HiGridUI)UIManager.getUI(this));
}

/**
 * Returns the UIClass ID for HiGrid.
 */
public String getUIClassID() {
	return "HiGridUI";
}

/**
 * Adds the specified mouse listener to receive mouse events from this component.
 */
// delegate to the gridArea for mouse and key events
public void addMouseListener(MouseListener l) {
    if (gridArea != null) {
        gridArea.addMouseListener(l);
    }
}

/**
 * Removes the specified mouse listener so that it no longer receives mouse events from this component.
 */
public void removeMouseListener(MouseListener l) {
    if (gridArea != null) {
        gridArea.removeMouseListener(l);
    }
}

/**
 * Adds the specified mouse motion listener to receive mouse motion events from this component.
 */
public void addMouseMotionListener(MouseMotionListener l) {
    if (gridArea != null) {
        gridArea.addMouseMotionListener(l);
    }
}

/**
 * Removes the specified mouse motion listener so that it no longer receives mouse motion events from this component.
 */
public void removeMouseMotionListener(MouseMotionListener l) {
    if (gridArea != null) {
        gridArea.removeMouseMotionListener(l);
    }
}

/**
 * Adds the specified key listener to receive key events from this component.
 */
public void addKeyListener(KeyListener l) {
    if (gridArea != null) {
		if (fNewFocusModel) {
			gridArea.setFocusTraversalKeysEnabled(false);
		}
        gridArea.addKeyListener(l);
    }
}

/**
 * Removes the specified key listener so that it no longer receives key events from this component.
 */
public void removeKeyListener(KeyListener l) {
    if (gridArea != null) {
        gridArea.removeKeyListener(l);
    }
}

/**
 * Gets the grid area.
 */
public JComponent getGridArea() {
    return gridArea;
}

/**
 * Sets the <code>GridArea</code> object for this grid.
 */
public void setGridArea(GridArea gridArea) {
    removeMouseListener(this.controller);
    removeMouseMotionListener(this.controller);
    removeKeyListener(this.controller);
    if (this.gridArea != null) {
        remove(this.gridArea);
        this.gridArea.removeAll();
        this.gridArea.setToolTipText(null);
        this.gridArea.toolTip.dispose();
        this.gridArea.toolTip = null;
        this.gridArea.dispose();
    }
    this.gridArea = gridArea;
    addMouseListener(this.controller);
    addMouseMotionListener(this.controller);
    addKeyListener(this.controller);
    if (gridArea != null) {
        setLayout(null);
        layoutGrid();
    }
}

/**
 * Gets the <code>Controller</code> object for this grid.
 */
public Controller getController() {
    return controller;
}

/**
 * Sets the controller for the grid.
 */
public void setController(Controller controller) {
    removeMouseListener(this.controller);
    removeMouseMotionListener(this.controller);
    removeKeyListener(this.controller);
    if (this.controller != null) {
        this.controller.setCurrentCell(null);
        this.controller.removeAllActions();
    }
    this.controller = controller;
    addMouseListener(this.controller);
    addMouseMotionListener(this.controller);
    addKeyListener(this.controller);
}

/**
 * Gets the <code>DataModelListener</code> object for this grid.
 */
public DataModelListener getDataModelListener() {
    return dataModelListener;
}

/**
 * Sets the <code>DataModelListener</code> object for this grid.
 */
protected void setDataModelListener(DataModelListener listener) {
//FIX should remove prior one and re-add this one, if necessary
    this.dataModelListener = listener;
}

/**
 * Gets the vertical scrollbar for this component.
 */
public GridScrollbar getVerticalScrollbar() {
    return verticalScrollbar;
}

public GridScrollbar getHorizontalScrollbar() {
    return horizontalScrollbar;
}

/**
 * Resets the scrollbar constraints.
 */
protected void resetScrollbarConstraints() {
    remove(verticalPanel);
    remove(horizontalPanel);
    verticalPanel.removeAll();
    horizontalPanel.removeAll();
    add(verticalPanel, convertConstraints(verticalScrollbarConstraints));
    verticalPanel.add(verticalScrollbar.getComponent());
    add(horizontalPanel, convertConstraints(horizontalScrollbarConstraints));
    horizontalPanel.add(horizontalScrollbar.getComponent());
    horizontalPanel.add(cornerPanel, convertConstraints(verticalScrollbarConstraints));
}

/**
 * Retrieves the lookup table.
 *
 * @return The Lookup Table for Cell Editors and Renderers.
 */
public LookupTable getLookupTable() {
    return lookupTable;
}

/**
 * Sets the lookup table.
 *
 * @param lookupTable The Lookup Table for Cell Editors and Renderers.
 */
public void setLookupTable(LookupTable lookupTable) {
    if (this.lookupTable != null) {
//FIX
//        this.lookupTable.dispose();
    }
    this.lookupTable = lookupTable;
}

/**
 * Retrieves the format tree.
 *
 * @return The format tree.
 */
public FormatTree getFormatTree() {
    return formatTree;
}

/**
 * Sets the format tree.
 *
 * @param formatTree The format tree.
 */
public void setFormatTree(FormatTree formatTree) {
    if (this.formatTree != null) {
//FIX
//        this.formatTree.dispose();
    }
    this.formatTree = formatTree;
}

/**
 * Retrieves the row tree.
 *
 * @return The row tree.
 */
public RowTree getRowTree() {
    return rowTree;
}

/**
 * Sets the row tree.
 *
 * @param rowTree The row tree.
 */
public void setRowTree(RowTree rowTree) {
    if (this.rowTree != null) {
//FIX
//        this.rowTree.dispose();
    }
    this.rowTree = rowTree;
}

/**
 * Retrieves the error handler.
 *
 * @return The error handler.
 */
public HiGridErrorHandler getErrorHandler() {
    return errorHandler;
}

/**
 * Sets the error handler. Removes the previous error handler
 * as a HiGridErrorListener, and adds this one as a HiGridErrorListener.
 *
 * @param handler The new error handler.
 */
public void setErrorHandler(HiGridErrorHandler handler) {
    removeHiGridErrorListener(this.errorHandler);
    this.errorHandler = handler;
    addHiGridErrorListener(this.errorHandler);
}

/**
 * Retrieves the column selection handler.
 *
 * @return The column selection handler.
 */
public HiGridColumnSelectionListener getColumnSelectionHandler() {
    return columnSelectionHandler;
}

/**
 * Sets the column selection handler. Removes the previous handler
 * as a HiGridColumnSelectionListener, and adds this one as a
 * HiGridColumnSelectionListener.
 *
 * @param handler The new column selection handler.
 */
public void setColumnSelectionHandler(HiGridColumnSelectionListener handler) {
    removeHiGridColumnSelectionListener(this.columnSelectionHandler);
    this.columnSelectionHandler = handler;
    addHiGridColumnSelectionListener(this.columnSelectionHandler);
}

/**
 * Returns the HiGrid AutoTraverseCallback.
 * @return The HiGrid AutoTraverseCallback.
 */
public AutoTraverseCallback getAutoTraverseCallback() {
    return autoTraverseCallback;
}

/**
 * Sets the HiGrid AutoTraverseCallback.
 * @param The HiGrid AutoTraverseCallback.
 */
public void setAutoTraverseCallback(AutoTraverseCallback callback) {
    this.autoTraverseCallback = callback;
}

/**
 * Returns the default HiGrid sortable.
 * @return The default HiGrid sortable.
 * @since JClass 4.0K for Java 2. If you are using JClass 4.0J,
 * which is based on JDK 1.1, this field's or method's return type
 * is based on the corresponding <code>com.sun.java.util.collections.Comparator</code> class.
 */
public Comparator getDefaultSortable() {
    return defaultSortable;
}

/**
 * Returns the current cellStyle corresponding to the RowFormat type.
 * @param rowFormatType a value in the range from FormatNode.FIRST_FORMAT
 * to FormatNode.LAST_FORMAT.
 * @return The current cellStyle for the given RowFormat type
 */
public CellStyleModel getCellStyle(int rowFormatType) {
    CellStyleModel cellStyle = null;
    switch(rowFormatType) {
    case FormatNode.HEADER_FORMAT:
        cellStyle = getHeaderCellStyle();
        break;
    case FormatNode.RECORD_FORMAT:
        cellStyle = getRecordCellStyle();
        break;
    case FormatNode.FOOTER_FORMAT:
        cellStyle = getFooterCellStyle();
        break;
    case FormatNode.BEFORE_DETAILS_FORMAT:
        cellStyle = getBeforeDetailsCellStyle();
        break;
    case FormatNode.AFTER_DETAILS_FORMAT:
        cellStyle = getAfterDetailsCellStyle();
        break;
    default:
        throw new IllegalArgumentException();
    }
    return cellStyle;
}

/**
 * Returns the current header cellStyle.
 * @return The current header cellStyle.
 */
public CellStyleModel getHeaderCellStyle() {
    return headerCellStyle;
}

/**
 * Sets the current header cellStyle.
 * @param cellStyle The current header cellStyle.
 */
protected void setHeaderCellStyle(CellStyleModel cellStyle) {
    this.headerCellStyle = cellStyle;
}

/**
 * Returns the current record cellStyle.
 * @return The current record cellStyle.
 */
public CellStyleModel getRecordCellStyle() {
    return recordCellStyle;
}

/**
 * Sets the current record cellStyle.
 * @param cellStyle The current record cellStyle.
 */
protected void setRecordCellStyle(CellStyleModel cellStyle) {
    this.recordCellStyle = cellStyle;
}

/**
 * Returns the current footer cellStyle.
 * @return The current footer cellStyle.
 */
public CellStyleModel getFooterCellStyle() {
    return footerCellStyle;
}

/**
 * Sets the current footer cellStyle.
 * @param cellStyle The current footer cellStyle.
 */
protected void setFooterCellStyle(CellStyleModel cellStyle) {
    this.footerCellStyle = cellStyle;
}

/**
 * Returns the current before details cellStyle.
 * @return The current before details cellStyle.
 */
public CellStyleModel getBeforeDetailsCellStyle() {
    return beforeDetailsCellStyle;
}

/**
 * Sets the current before details cellStyle.
 * @param cellStyle The current before details cellStyle.
 */
protected void setBeforeDetailsCellStyle(CellStyleModel cellStyle) {
    this.beforeDetailsCellStyle = cellStyle;
}

/**
 * Returns the current after details cellStyle.
 * @return The current after details cellStyle.
 */
public CellStyleModel getAfterDetailsCellStyle() {
    return afterDetailsCellStyle;
}

/**
 * Sets the current after details cellStyle.
 * @param cellStyle The current after details cellStyle.
 */
protected void setAfterDetailsCellStyle(CellStyleModel cellStyle) {
    this.afterDetailsCellStyle = cellStyle;
}

/**
 * Returns the HiGrid sortable.
 * @return The HiGrid sortable.
 * @since JClass 4.0K for Java 2. If you are using JClass 4.0J,
 * which is based on JDK 1.1, this field's or method's return type
 * is based on the corresponding <code>com.sun.java.util.collections.Comparator</code> class.
 */
public Comparator getSortable() {
    return sortable;
}

/**
 * Sets the HiGrid sortable.
 * @param sortable The HiGrid sortable.
 * @since JClass 4.0K for Java 2. If you are using JClass 4.0J,
 * which is based on JDK 1.1, this method's parameter must be
 * a <code>com.sun.java.util.collections.Comparator</code>.
 */
public void setSortable(Comparator sortable) {
    this.sortable = sortable;
}

/**
 * Retrieves the vertical scrollbar display mode.
 *
 * @return The display mode.
 */
public int getVerticalScrollbarDisplay() {
    return verticalScrollbar.getDisplay();
}

/**
 * Sets the vertical scrollbar display mode.
 *
 * @param display The display mode.
 */
public void setVerticalScrollbarDisplay(int display) {
    verticalScrollbar.setDisplay(display);
    refresh();
}

/**
 * Retrieves the horizontal scrollbar display mode.
 *
 * @return The display mode.
 */
public int getHorizontalScrollbarDisplay() {
    return horizontalScrollbar.getDisplay();
}

/**
 * Sets the horizontal scrollbar display mode.
 *
 * @param display The display mode.
 */
public void setHorizontalScrollbarDisplay(int display) {
    horizontalScrollbar.setDisplay(display);
    refresh();
}

/**
 * Retrieves the horizontal scrollbar constraints.
 *
 * @return The Swing Constants constraints.
 */
public int getHorizontalScrollbarConstraints() {
    return horizontalScrollbarConstraints;
}

/**
 * Sets the horizontal scrollbar constraints.
 *
 * @param constraints The SwingConstants constraints.
 */
public void setHorizontalScrollbarConstraints(int constraints) {
    switch (constraints) {
    case SwingConstants.NORTH:
    case SwingConstants.SOUTH:
        horizontalScrollbarConstraints = constraints;
        resetScrollbarConstraints();
        break;
    }
}

/**
 * Retrieves the vertical scrollbar constraints.
 *
 * @return The Swing Constants constraints.
 */
public int getVerticalScrollbarConstraints() {
    return verticalScrollbarConstraints;
}

/**
 * Sets the vertical scrollbar constraints.
 *
 * @param constraints The SwingConstants constraints.
 */
public void setVerticalScrollbarConstraints(int constraints) {
    switch (constraints) {
    case SwingConstants.EAST:
    case SwingConstants.WEST:
        verticalScrollbarConstraints = constraints;
        resetScrollbarConstraints();
        break;
    }
}

/**
 * Retrieves the String version of the SwingConstants constraints.
 *
 * @param constraints The SwingConstants constraints.
 *
 * @return The String constraints.
 */
protected String convertConstraints(int constraints) {
    String s = null;
    switch (constraints) {
    case SwingConstants.EAST:
        s = BorderLayout.EAST;
        break;
    case SwingConstants.WEST:
        s = BorderLayout.WEST;
        break;
    case SwingConstants.NORTH:
        s = BorderLayout.NORTH;
        break;
    case SwingConstants.SOUTH:
        s = BorderLayout.SOUTH;
        break;
    }
    return s;
}

/**
 * Retrieves the common border used for all cell editors.
 *
 * @return The border used by all cell editors.
 */
public Border getEditorBorder() {
    return editorBorder;
}

/**
 * Sets the common border used for all cell editors.
 *
 * @param border The border used by all cell editors.
 */
public void setEditorBorder(Border border) {
    editorBorder = border;
}

/**
 * Retrieves the default common border used for all cell editors.
 *
 * @return The default border used by all cell editors.
 */
public Border getDefaultEditorBorder() {
    return createDefaultEditorBorder(Color.green, 2, new Insets(2, 2, 2, 2));
}

/**
 * Creates a default common border used for all cell editors.
 *
 * @param color the color of the border
 * @param lineWidth the width of the border
 * @param insets the editor insets
 * @return The default border used by all cell editors.
 */
public Border createDefaultEditorBorder(Color color, int lineWidth, Insets insets) {
    Border compoundBorder = new CompoundBorder(
        new LineBorder(color, lineWidth),
//        new BevelBorder(BevelBorder.RAISED, color, color.darker()),
        new EmptyBorder(insets));
    return compoundBorder;
}

/**
 * Retrieves whether a cell editor is automatically popped
 * up when a cell is traversed to via the keyboard.
 *
 * @return The current autoEdit value.
 */
public boolean isAutoEdit() {
    return autoEdit;
}

/**
 * Sets whether a cell editor is automatically popped
 * up when a cell is traversed to via the keyboard.
 *
 * @param autoEdit The new autoEdit value.
 */
public void setAutoEdit(boolean autoEdit) {
    this.autoEdit = autoEdit;
}

/**
 * Retrieves the current row node (the row that the cell editor
 * is currently on.)
 *
 * @return The current row node.
 */
public RowNode getCurrentRowNode() {
    CellPosition cell = controller.getCurrentCell();
    if (cell != null) {
        return cell.row;
    }
    return null;
}

/**
 * Get the row node given the bookmark.
 *
 * @param bookmark. The bookmark of the row node.
 */
public RowNode getRowNode(long bookmark) {
    return rowTree.findRecordRowNode(null, bookmark);
}

/**
 * Converts an index to a RowNode
 *
 * @param index The index to convert.
 *
 * @return The RowNode in the index'th position in the RowTree.
 */
public RowNode getRowNode(int index) {
    return rowTree.getRowNode(index);
}

/**
 * Converts a RowNode to its index in the RowTree
 *
 * @param rowNode The row to convert.
 *
 * @return The index of the RowNode in the RowTree.
 */
public int  getRowIndex(RowNode rowNode) {
    return rowTree.getRowIndex(rowNode);
}

/**
 * Returns all of the RowNodes that exist in the given dataTable,
 * in the order that they visually appear.
 *
 * @return The array of RowNodes
 */
public RowNode[] getTableRowNodes(RowNode rowNode) {
    if (rowNode == null) {
        return null;
    }
    FormatNode formatNode = rowNode.getFormatNode();
    DataTableModel table = rowNode.getDataTableModel();
    if (table == null) {
        return null;
    }
    int rowCount = table.getRowCount();
    if (rowCount == 0) {
        return null;
    }
    RowNode[] rows = new RowNode[rowCount];
    // move to the first record of this table
    CellPosition firstPosition = controller.goLevelHome(rowNode);
    if (firstPosition == null) {
        return null;
    }
    rowNode = firstPosition.row;
    if (rowNode == null) {
        return null;
    }
    int i = 0;
    rows[i++] = rowNode;
    // going forward, add one row at a time until we hit the end of the table
    while (rowNode != null) {
        rowNode = (RowNode)rowNode.getNextSibling();
        if (rowNode == null) {
            break;
        }
        if (rowNode.getRowFormat() instanceof RecordFormat) {
            if (rowNode.getFormatNode() == formatNode) {
                rows[i++] = rowNode;
            }
            else {
                break;
            }
        }
    }
    return rows;
}

/**
 * Moves the cell editor to a new row and column.
 *
 * @param rowNode The row to move the editor to (must be a RecordFormat)
 * @param column The column index (0 based)
 *
 * @return True if the move was successful.
 */
public boolean moveCellEditor(RowNode rowNode, int column) {
    if (rowNode.getRowFormat() instanceof RecordFormat) {
        CellPosition cell = new CellPosition(rowNode, column);
        if (controller.validateAndTraverse(cell, column, null)) {
            return true;
        }
    }
    return false;
}

/**
 * Retrieves the topmost visible RowNode.
 *
 * @return The topmost visible RowNode.
 */
public RowNode getTopRowNode() {
    return rowTree.getTopRowNode();
}

/**
 * Sets the topmost visible row to the passed RowNode.
 *
 * @param rowNode The new topmost visible row.
 */
public void setTopRowNode(RowNode rowNode) {
    rowTree.setTopRow(rowNode);
    refresh();
}

/**
 * Sets the topmost visible row to the passed bookmark.
 *
 * @param bookmark The bookmark of the row.
 */
public void setTopRowBookmark(long bookmark) {
    setTopRowNode(rowTree.findRecordRowNode(null, bookmark));
}

/**
 * Retrieves the topmost visible row.
 *
 * @return The topmost visible row.
 */
public int getTopRowIndex() {
    return getVerticalScrollbar().getValue();
}

/**
 * Sets the topmost visible row to RowNode indexed
 * by the passed value.
 *
 * @param index The new topmost visible row based on this index.
 */
public void setTopRowIndex(int index) {
    rowTree.setTopRow(index);
    refresh();
}

/**
 * Returns the value of rowHeightResizingAll.
 */
public boolean isRowHeightResizingAll() {
    return(rowHeightResizingAll);
}

/**
 * Sets the value of rowHeightResizingAll. If true then the user
 * is allowed to resize a row anywhere on the row.  If false,
 * the row can be resized only in the EditStatus column.
 */
public void setRowHeightResizingAll(boolean rowHeightResizingAll) {
    this.rowHeightResizingAll = rowHeightResizingAll;
}

/**
 * Returns the number of extra pixels to draw at the rightmost
 * end of HiGrid.
 */
public int getExtraWidth() {
    return(extraWidth);
}

/**
 * Sets the number of extra pixels to draw at the rightmost
 * end of HiGrid.
 * @param width the new additional width
 */
public void setExtraWidth(int width) {
    this.extraWidth = width;
}

/**
 * Returns the value of beepOnInvalid.
 */
public boolean isBeepOnInvalid() {
    return(beep);
}

/**
 * Sets the value of beepOnInvalid. If beepOnInvalid == true then the field
 * will beep at the user whenever the state is switched to INVALID.
 * @param beep whether to beep when cell becomes invalid
 */
public void setBeepOnInvalid(boolean beep) {
    this.beep = beep;
}

/**
 * Retrieves whether the header tooltip is visible.
 *
 * @return True if the header tooltip is visible.
 */
public boolean isHeaderTipVisible() {
    return headerTipVisible;
}

/**
 * Sets whether the header tooltip is visible.
 *
 * @param visible True if the header tooltip is visible.
 */
public void setHeaderTipVisible(boolean visible) {
    headerTipVisible = visible;
    getGridArea().setToolTipText(visible ? "1" : null);
}

/**
 * Retrieves when the header tooltip will be displayed.
 *
 * @return when the header tooltip will be displayed.
 */
public int getHeaderTipDisplay() {
    return headerTipDisplay;
}

/**
 * Sets when the header tooltip will be displayed.
 *
 * @param display when the header tooltip will be displayed.
 */
public void setHeaderTipDisplay(int display) {
    headerTipDisplay = display;
}

/**
 * Retrieves whether the row tip is visible.
 *
 * @return True if row tip is visible.
 */
public boolean isRowtipVisible() {
    return verticalScrollbar.isRowtipVisible();
}

/**
 * Sets whether the row tip is visible.
 *
 * @param visible True if row tip is visible.
 */
public void setRowtipVisible(boolean visible) {
    verticalScrollbar.setRowtipVisible(visible);
}

/**
 * Retrieves whether the editor is hidden.
 *
 * @return True if editor is hidden.
 */
public boolean isEditorHidden() {
    return editorHidden;
}

/**
 * Sets whether the editor is hidden.
 *
 * @param hidden True if editor is hidden.
 */
public void setEditorHidden(boolean hidden) {
    editorHidden = hidden;
}

/**
 * Returns whether the connection lines are visible.
 */
public boolean isConnectionsVisible() {
    return connectionsVisible;
}

/**
 * Sets whether the connection lines are visible.
 *
 * @param connections Whether the connection lines are visible.
 */
public void setConnectionsVisible(boolean connections) {
    connectionsVisible = connections;
    refresh();
}

/**
 * Returns whether the sort icons are visible.
 */
public boolean isSortIconsVisible() {
    return sortIconsVisible;
}

/**
 * Sets whether the sort icons are visible.
 *
 * @param sortIcons Whether the sort icons are visible.
 */
public void setSortIconsVisible(boolean sortIcons) {
    sortIconsVisible = sortIcons;
    refresh();
}

/**
 * Returns whether to synchronize scrolling with
 * DataModel row moved events.
 *
 * @see HiGrid#setScrollOnMoveEvent
 */
public boolean isScrollOnMoveEvent() {
    return scrollOnMoveEvent;
}

/**
 * Sets whether to synchronize scrolling upon receipt
 * of a current row change event.
 * If the current row is changed in the DataModel, HiGrid
 * will, by default, scroll to the new row.
 *
 * @param scrollOnMoveEvent boolean true means scroll to make
 * the editor visible; false means don't automatically scroll.
 */
 public void setScrollOnMoveEvent(boolean scrollOnMoveEvent) {
 	this.scrollOnMoveEvent = scrollOnMoveEvent;
 }

/**
 * Retrieves the datasource model for this object.
 *
 * @return The current datasource model.
 */
public DataModel getDataModel() {
    return dataModel;
}

/**
 * Sets the datasource model for the higrid.
 * This will result in (re-)creation of the design-time
 * FormatTree and the run-time RowTree, followed
 * by a repaint of the grid.
 *
 * @param dataModel The new datasource model.
 */
public void setDataModel(DataModel dataModel) {
    // clean up any references to previous dataModels
    removeOldReferences();

    if (dataModel == null || dataModel.getMetaDataTree() == null ||
        dataModel.getMetaDataTree().getRoot() == null) {
        dataModel = null;
    }
    boolean addListener = (this.dataModel == null);
    if (dataModel == null || dataModel instanceof HiGridData) {
        if (this.dataModel != null) {
            this.dataModel.removeDataModelListener(getDataModelListener());
            this.dataModel.clearDataTableTreeInternalData(this);
            addListener = true;
        }
        this.dataModel = dataModel;
    }
    else if (dataModel instanceof DataModel) {
        // Remove listener if the current data model is not the passed in data model.
        if (this.dataModel != null) {
            if (this.dataModel instanceof HiGridData) {
                if (!((HiGridData)this.dataModel).dataModel.equals(dataModel)) {
                    this.dataModel.removeDataModelListener(getDataModelListener());
                    this.dataModel.clearDataTableTreeInternalData(this);
                    addListener = true;
                }
            }
            else if (!this.dataModel.equals(dataModel)) {
                this.dataModel.removeDataModelListener(getDataModelListener());
                this.dataModel.clearDataTableTreeInternalData(this);
                addListener = true;
            }
        }
        this.dataModel = new HiGridData((DataModel) dataModel);
    }

    if (this.dataModel != null && addListener) {
        this.dataModel.addDataModelListener(getDataModelListener());
    }

    // FIX this may not be optimal
    // we need it for now to properly draw the grid the first time
//FIXOBSOLETE
//    formatTree.setRoot(null);
    if (this.dataModel != null) {
        formatTree.createFormatTreeFromDataModel(this.dataModel);
    }
    resetRunTimeGrid();
    createDefaultTrees();
    refresh();
}

/**
 * Remove references to previous DataModels
 * so they can be garbage collected.
 */
protected void removeOldReferences() {
    // Swing limitations require GridScrollbar to maintain
    // a reference to the last top row node. This needs to
    // be removed when a new data model is set.
    if (horizontalScrollbar != null) {
        horizontalScrollbar.removeOldReferences();
    }
    if (verticalScrollbar != null) {
        verticalScrollbar.removeOldReferences();
    }
}

/**
 * Retrieves the folder icon style index.
 *
 * @return The folder icon style index.
 */
public int getFolderIconStyle() {
    return folderIconStyle.getStyle();
}

/**
 * Set the folder icon given the style index.
 *
 * @param style The folder icon style index.
 */
public void setFolderIconStyle(int style) {
    folderIconStyle.setStyle(style);
    refresh();
}

/**
 * Retrieves the folder icon image for the given folder state.
 *
 * @param type The type of icon to get.
 * @return The current icon of this type.
 */
public Image getFolderIcon(int type) {
    return folderIconStyle.getFolderIcon(type);
}

/**
 * Sets the folder icon image for the given folder state.
 *
 * @param type The type of icon to set.
 * @param icon The new icon of this type.
 */
public void setFolderIcon(Image icon, int type) {
    folderIconStyle.setFolderIcon(icon, type);
    refresh();
}

/**
 * Retrieves whether subsequent property changes are batched.
 * For batch operation, no grid refreshing is performed
 * until batched is set to false.
 *
 * @return The batched mode.
 */
public boolean isBatched() {
    return batched;
}

/**
 * Sets whether subsequent property changes are batched.
 * For batch operation, no grid refreshing is performed
 * until batch is set to false.
 *
 * @return batched The batched mode.
 */
public void setBatched(boolean batched) {
    this.batched = batched;
    // turning off batch will trigger a refresh
    if (!batched) {
        controller.initializeEditor();
        refresh();
    }
}

/**
 * Retrieves whether levels are indented.
 *
 * @return If levels are indented .
 */
public boolean isLevelIndented() {
    return levelIndented;
}

/**
 * Sets whether levels are indented.
 *
 * @param indent True if levels are to be indented by the NodeWidth.
 */
public void setLevelIndented(boolean indent) {
    levelIndented = indent;
    recalculateRowFormats(NODE_INDICATOR_CELL);
    refresh();
}

/**
 * Retrieves the node width in pixels for all levels.
 *
 * @return The current node width.
 */
public int getNodeWidth() {
    return nodeWidth;
}

/**
 * Sets the node width in pixels for all levels.
 *
 * @param width The new node width.
 */
public void setNodeWidth(int width) {
    nodeWidth = width;
    recalculateRowFormats(NODE_INDICATOR_CELL);
    refresh();
}

/**
 * Retrieves the edit status width in pixels for all levels.
 *
 * @return The current width.
 */
public int getEditStatusWidth() {
    return editStatusWidth;
}

/**
 * Sets the edit status width in pixels for all levels.
 *
 * @param width The new width.
 */
public void setEditStatusWidth(int width) {
    editStatusWidth = width;
    recalculateRowFormats(EDITSTATUS_INDICATOR_CELL);
    refresh();
}

/**
 * Retrieves whether input is allowed for this object.
 *
 * @return Whether input is currently allowed.
 */
public boolean isEditable() {
    return editable;
}

/**
 * Sets whether input is currently allowed.
 *
 * @param isEditable Whether to allow input.
 */
public void setEditable(boolean editable) {
    this.editable = editable;
}

/**
 * Reset the sort indications.
 *
 * @param rowNode The row node. If null, reset the entire grid.
 * @param checkChildren True if the row node's children has be
 *        checked as well.
 */
public void resetSortIndications(RowNode rowNode, boolean checkChildren) {
    if (rowNode == null) {
        TreeIterator ti = ((RowNode)rowTree.getRoot()).getIterator();
        while (ti.hasMoreElements()) {
            rowNode = (RowNode)ti.get();
            resetSortIndicationsInternal(rowNode, checkChildren);
            ti.nextElement();
        }
    }
    else {
        resetSortIndicationsInternal(rowNode, checkChildren);
    }
}

/**
 * Reset possible sort indications for the given row
 * and maybe its children as well.
 *
 * @param row The row bookmark.
 * @param checkChildren True if the row node's children has be
 * checked as well.
 */
protected void resetSortIndications(long row, boolean checkChildren) {
    RowNode rowNode = getRowTree().findRecordRowNode(null, row);
    resetSortIndications(rowNode, checkChildren);
}

/**
 * Helper recursive method to reset the sort indications.
 *
 * @param rowNode The row node.
 * @param checkChildren True if the row node's children has be
 *        checked as well.
 */
protected void resetSortIndicationsInternal(RowNode rowNode, boolean checkChildren) {
    if (rowNode == null || !(rowNode.getRowFormat() instanceof RecordFormat)) {
        return;
    }
    HiGridInternalData internalData = (HiGridInternalData)rowNode.getDataTableModel().getInternalData(this);
    if (internalData != null) {
        internalData.setSortData(null);
    }
    if (checkChildren) {
        TreeIterator ti = rowNode.getIterator();
        while (ti.hasMoreElements()) {
            RowNode rn = (RowNode)ti.get();
            resetSortIndicationsInternal(rn, checkChildren);
            ti.nextElement();
        }
    }
}

/**
 * Sort the indicated data table by the specified column.
 *
 * @param rowNode A row node of the data table.
 * @param columnID The column to be sorted.
 * @param direction The sorting direction.
 * @param makeDefault Make this the new default sort column and direction
 */
public void sortDataTable(RowNode rowNode, String columnID, int direction, boolean makeDefault) {
    FormatNode node = (FormatNode) rowNode.getFormatNode();
    if (makeDefault) {
        node.setDefaultSortData(new SortData(columnID, direction));
    }

    Cursor oldCursor = gridArea.getCursor();
    gridArea.setCursor(new Cursor(Cursor.WAIT_CURSOR));

    SortGrid sortGrid = new SortGrid(this, false);
    sortGrid.sortColumn(node, rowNode, new SortData(columnID, direction));

    gridArea.setCursor(oldCursor);
    refreshChildrenRows(rowNode);
}


/**
 * Sort the data table(s) by the specified cellPosition.
 *
 * @param cell The cell position.
 * @param multipleTableSort If true, sort all expanded data tables
 */
public void sortDataTable(CellPosition cell, boolean multipleTableSort) {
    Cursor oldCursor = gridArea.getCursor();
    gridArea.setCursor(new Cursor(Cursor.WAIT_CURSOR));

    SortGrid sortGrid = new SortGrid(this, multipleTableSort);
    sortGrid.sortColumn(cell, !multipleTableSort);

    gridArea.setCursor(oldCursor);
    refreshChildrenRows(multipleTableSort ? null : cell.row);
}

/**
 * Retrieves the data from the indicated cell.
 *
 * @param cell The cell to extract data from.
 *
 * @return The data associated with the given cell.
 */
public Object getCellData(CellPosition cell) {
    Object data = null;
    try {
        RowNode rowNode = cell.getRowNode();
        String name = cell.getCellFormat().getName();
        DataTableModel dataTable = rowNode.getDataTableModel();
        if (dataTable != null) {
            try {
                data = dataTable.getResultData(rowNode.getBookmark(), name);
            }
            catch (DataModelException dme) {
                return null;
            }
        }
    }
    catch (NullPointerException npe) {
        return null;
    }
    return data;
}

/**
 * Retrieves the cell that the cell editor is on.
 *
 * @return The current cell position.
 */
public CellPosition getCurrentCellPosition() {
    // return a copy of the cell's position, to prevent changes
    return(CellPosition)controller.getCurrentCell().clone();
}

/**
 * Retrieves the cell from the specified point.
 *
 * @param point The x,y pixel co-ordinate.  This method does not correct
 * for any horizontal scroll offset.  To do that, add
 * getHorizontalScrollbar().getValue() to the Point's x coordinate.
 * Alternatively, call getHorizontalScrollableCellPositionFromPoint()
 * to have the horizontal scroll offset applied automatically.
 *
 * @return the cell from the specified point, or null if no cell at that point.
 */
public CellPosition getCellPositionFromPoint(Point point) {
    NullGraphics nullgc = controller.getCellFromXY(point);
    if (nullgc == null ||
    	nullgc.cell == null ||
    	nullgc.cell.format == null) {
        return null;
    }
    return nullgc.cell;
}

/**
 * Retrieves the point from the specified cell.
 *
 * @param cell The cell to locate.
 *
 * @return the top-left x,y co-ordinate of the cell, or null if not found.
 * The return value is not corrected for any horizontal scroll offset.  To
 * do that, subtract getHorizontalScrollbar().getValue() from the Point's
 * x coordinate.  Alternatively, call
 * getPointFromHorizontalScrollableCellPosition() to have the horizontal
 * scroll offset subtracted automatically.
 */
public Point getPointFromCellPosition(CellPosition cell) {
    if (cell == null) {
        return null;
    }
    NullGraphics nullgc = controller.getXYFromCell(cell);
    if (nullgc == null || nullgc.cell == null) {
        return null;
    }
    // Don't correct for horizontalScrollbar value.  This method is
    // used in a couple of Controller methods where the horizontal scroll
    // offset is already taken into account.
    return (nullgc.cell.point);
}

/**
 * Retrieves the cell from the specified point.
 *
 * @param point The x,y pixel co-ordinate.  If the grid is scrolled
 * horizontally, this method will add the horizontal scroll offset
 * to the Point's x coordinate.
 *
 * @return the cell from the specified point, or null if no cell at that point.
 */
public CellPosition getHorizontalScrollableCellPositionFromPoint(Point point) {
    Point pnt = new Point(point);
    if (horizontalScrollbar != null && horizontalScrollbar.isVisible()) {
        pnt.translate(horizontalScrollbar.getValue(), 0);
    }
    return getCellPositionFromPoint(pnt);
}

/**
 * Retrieves the point from the specified cell.
 *
 * @param cell The cell to locate.
 *
 * @return the top-left x,y co-ordinate of the cell, or null if not found.
 * If the grid is scrolled horizontally, this method will apply a correction
 * for the horizontal scroll offset.
 */
public Point getPointFromHorizontalScrollableCellPosition(CellPosition cell) {
	Point pnt = getPointFromCellPosition(cell);
	if (pnt != null) {
		if (horizontalScrollbar != null && horizontalScrollbar.isVisible()) {
			pnt.translate(-horizontalScrollbar.getValue(), 0);
		}
	}
	return pnt;
}

/**
 * Retrieves the cell that the cell editor is on.
 *
 * @return The current cell position.
 */
protected CellPosition getCurrentCell() {
    return controller.getCurrentCell();
}

/**
 * Retrieves whether the context menu is allowed to be displayed.
 *
 * @return True if popup menu is allowed to be displayed.
 */
public boolean isAllowPopupMenu() {
    return allowPopupMenu;
}

/**
 * Sets whether the popup context menu is allowed to be displayed.
 *
 * @param allow True if popup menu is allowed to be displayed.
 */
public void setAllowPopupMenu(boolean allow) {
    allowPopupMenu = allow;
}

/**
 * Retrieves the edit popup menu.
 *
 * @return The edit popup menu.
 */
public EditPopupMenu getEditPopupMenu() {
    return popupMenu;
}

/**
 * Sets the edit popup menu.
 *
 * @param popupMenu The edit popup menu.
 */
public void setEditPopupMenu(EditPopupMenu popupMenu) {
    if (this.popupMenu != popupMenu) {
        if (this.popupMenu != null) {
            this.popupMenu.dispose();
        }
    }
    this.popupMenu = popupMenu;
}

/**
 * Retrieves whether user sorting is supported.
 *
 * @return True if user sorting is supported.
 */
public boolean isAllowSorting() {
    return allowSorting;
}

/**
 * Sets whether user sorting is supported.
 *
 * @param allow True if user sorting is to be supported.
 */
public void setAllowSorting(boolean allow) {
    allowSorting = allow;
}

/**
 * Gets the grid's <CODE>TrackCursor</CODE> value to determine whether the
 * mouse pointer is being tracked as the user moves the mouse.
 * @see HiGrid#setTrackCursor
 */
public boolean isTrackCursor() {
	return trackCursor;
}

/**
 * Sets whether the mouse pointer should be tracked as a user moves the mouse
 * across the grid. When true, <CODE>HiGrid</CODE> tracks the mouse pointer
 * and changes its shape to reflect the interactions available to the user at
 * the location. When set false, the grid uses the application-defined pointer.
 * (Default: <CODE>true</CODE).
 *
 * @param trackCursor whether the mouse pointer should be tracked.
 */
public void setTrackCursor(boolean trackCursor) {
	this.trackCursor = trackCursor;
    setCursor(getDefaultCursor());
}

/**
 * Sets the default global cursor used by HiGrid.
 *
 * @param defaultCursor The default global cursor used by HiGrid.
 * @see #setTrackCursor
 */
public void setDefaultCursor(Cursor defaultCursor) {
    this.defaultCursor = defaultCursor;
}

/**
 * Retrieves the default global cursor used by HiGrid.
 *
 * @return The default global cursor used by HiGrid.
 */
public Cursor getDefaultCursor() {
    return defaultCursor;
}

/**
 * Sets the global cursor when cursor tracking is off.
 * @param type the java.awt.Cursor type (e.g Cursor.WAIT_CURSOR)
 * @see #setTrackCursor
 */
public void setDefaultCursor(int type) {
	setDefaultCursor(Cursor.getPredefinedCursor(type));
}

/**
 * This routine will normally be called by a child to display
 * the grid in a sub-window.
 *
 * @param gc The Graphics Context to use.
 */
protected void draw(Graphics gc) {
    gc.setColor(this.getBackground());
    Rectangle rect = gc.getClipBounds();
    gc.fillRect(rect.x, rect.y, rect.width, rect.height);
    rowTree.displayRuntimeTree(gc);
    if (getDataModel() == null && JCEnvironment.isDesignTime()) {
        Font font = getFont();
        if (font != null) {
            gc.setFont(font);
            gc.setColor(this.getForeground());
            FontMetrics fm = gc.getFontMetrics(font);
            String beanText = LocaleBundle.string((this instanceof JCHiGridExternalDS) ? LocaleBundle.JCHiGridExternalDS : LocaleBundle.JCHiGrid);
            gc.drawString(beanText, 0, fm.getHeight());
        }
    }
}

/**
 * Ensures that we are ready to display the grid.
 */
public void createDefaultTrees() {
    if (dataModel == null) {
        return;
    }
    if (formatTree.getRoot() == null) {
        formatTree.createFormatTreeFromDataModel(dataModel);
        rowTree.computeTotalWidth();
    }
    if (rowTree.getRoot() == null) {
        rowTree.createRuntimeTree(formatTree);
    }
    if (rowTree.getTopRowNode() == null) {
        rowTree.setTopRowNode((RowNode)rowTree.getRootNode().getFirstChild());
    }
    controller.positionCurrentCell();
}

/**
 * Recreates the FormatTree from the current dataModel.
 * This method should be called whenever a drastic change
 * has been made at run-time to the design-time information,
 * such as adding a column to a table.
 * Must be followed by a call to resetRunTimeGrid() to
 * properly display the grid at runtime.
 */
public void resetDesignTimeGrid() {
    formatTree.createFormatTreeFromDataModel(dataModel);
}

/**
 * Removes all cached runtime grid information.
 * Destroys and recreates the RowTree.
 * This routine needs to be called after any changes
 * are made to the design-time information,
 * and after all calls to resetDesignTimeGrid().
 * After calling this method, the grid should be redrawn
 * by a call to refresh() or setBatched(false).
 */
public void resetRunTimeGrid() {
    controller.showEditor(false);
    controller.setCurrentCell(null);
    rowTree.setTopRowNode(null);
//FIXOBSOLETE
//    rowTree.setRoot(null);
    formatTree.resetRuntimeInfo();
    if (getDataModel() != null) {
		// Clearing the internal data fixes PR 26591.
    	getDataModel().clearDataTableTreeInternalData(this);
	}
    rowTree.createRuntimeTree(formatTree);
    if (getDataModel() != null && rowTree.getRoot() != null &&
        rowTree.getRootNode().getChildCount() > 0) {
        // set current row to the current bookmark
        RowNode rowNode = null;
        if (!JCEnvironment.isDesignTime()) {
            rowNode = rowTree.findRecordRowNode(null, dataModel.getCurrentGlobalBookmark());
            if (rowNode == null) {
                DataTableModel dataTable = dataModel.getCurrentGlobalTable();
                if (dataTable != null) {
                    Vector ancestors = new Vector();
                    dataTable.getAncestors(ancestors);
                    for (int index = (ancestors.size()-1); index >= 0; index--) {
                        rowNode = rowTree.findRecordRowNode(null, ((Long)ancestors.elementAt(index)).longValue());
                        if (rowNode != null) {
                            break;
                        }
                    }
                }
            }
        }
        if (rowNode == null) {
            rowTree.setTopRow(0);
            controller.positionCurrentCell();
            controller.createEditor();
            controller.positionEditor();
        }
        else {
            if (rowNode == rowTree.getFirstRecordRowNode()) {
                rowTree.setTopRow(0);
            }
            else {
                rowTree.setTopRow(rowNode);
            }
            controller.repositionCurrentCell(rowNode, false, true);
        }
    }
    horizontalScrollbar.setValue(0);
}

/**
 * Sets the background color for the entire grid.
 *
 * @param color The new background color.
 */
public void setBackground(Color color) {
    // override in order to set associated corner panel
    cornerPanel.setBackground(color);
    super.setBackground(color);
}

protected void widthChanged(Vector v, int index) {
    if (index < v.size()) {
        ((CellFormat)(v.elementAt(index))).widthChanged();
    }
}

/**
 * Calculates the width
 */
protected void calculateWidth(FormatNode node, int index) {
    // find out where this index really is
    index = getIndicatorIndex(node, index);
    if (index >= 0) {
        widthChanged(node.getRecordFormat().getIndicatorFormats(), index);
        widthChanged(node.getHeaderFormat().getIndicatorFormats(), index);
        widthChanged(node.getFooterFormat().getIndicatorFormats(), index);
        widthChanged(node.getBeforeDetailsFormat().getIndicatorFormats(), index);
        widthChanged(node.getAfterDetailsFormat().getIndicatorFormats(), index);
    }
}

/**
 * Visits all the nodes of the <code>FormatTree</code>.
 */
protected void walkFormatTree(FormatNode node, int index) {
    // recursively walk formatTree
    if (node == null) {
        return;
    }
    calculateWidth(node, index);
    TreeIterator ti = node.getIterator();
    while (ti.hasMoreElements()) {
        node = (FormatNode)ti.get();
        walkFormatTree(node, index);
        ti.nextElement();
    }
}

/**
 * Recalculates the <code>RowFormat</code>s.
 */
protected void recalculateRowFormats(int index) {
    if (formatTree == null || formatTree.getRoot() == null) {
        return;
    }
    walkFormatTree((FormatNode)formatTree.getRoot(), index);
}

protected int getIndicatorIndex(FormatNode formatNode, int cellType) {
    if (formatNode.getRecordFormat() == null) {
        return -1;
    }
    else {
        return getIndicatorIndex(formatNode.getRecordFormat().getIndicatorFormats(), cellType);
    }
}

/**
 * Gets the indicator index based on its cell type.
 */
protected int getIndicatorIndex(Vector indicatorFormats, int cellType) {
    if (indicatorFormats != null) {
        if (cellType == NODE_INDICATOR_CELL) {
            for (int i = 0; i < indicatorFormats.size(); i++) {
                if (indicatorFormats.elementAt(i) instanceof NodeCellFormat) {
                    return i;
                }
            }
        }
        else if (cellType == EDITSTATUS_INDICATOR_CELL) {
            for (int i = 0; i < indicatorFormats.size(); i++) {
                if (indicatorFormats.elementAt(i) instanceof EditStatusCellFormat) {
                    return i;
                }
            }
        }
    }
    return -1;
}

/**
 * Returns a <code>CellPosition</code> for the given type of cell.
 */
protected CellPosition makeCellPosition(RowNode rowNode, int cellType) {
    int index = getIndicatorIndex(rowNode.getFormatNode(), cellType);
    if (index < 0) {
        return null;
    }
    else {
        return new CellPosition(rowNode, index, true);
    }
}

/**
 * Refreshes the edit status cell.
 */
protected void refreshEditStatus(RowNode rowNode) {
    CellPosition cell = makeCellPosition(rowNode, EDITSTATUS_INDICATOR_CELL);
    if (cell != null) {
        refreshCell(cell);
    }
}

/**
 * Refreshes the edit status cell.
 */
protected void refreshEditStatus(CellPosition cell, boolean refresh) {
    if (refresh && cell != null) {
        refreshEditStatus(cell.row);
    }
}

/**
 * Refreshes the cell.
 */
protected void refreshCell(CellPosition cell) {
    NullGraphics nullgc = controller.getXYFromCell(cell);
    if (! nullgc.finishedOperation) {
        return;
    }
    cell = nullgc.cell;
    Rectangle rect = cell.format.getTotalArea();
    refreshGridArea(cell.point.x, cell.point.y, rect.width, rect.height);
}

/**
 * Refreshes rows.
 */
protected void refreshRowsWithoutEditor(int y, int height) {
    Dimension dimension = gridArea.getSize();
    refreshGridArea(0, y, Math.max(rowTree.getGridAreaWidth(), dimension.width), height);
}

/**
 * Refreshes rows.
 */
protected void refreshRows(int y, int height) {
    controller.showEditor(false);
    refreshRowsWithoutEditor(y, height);
    controller.positionEditor();
    controller.showEditor(true);
}

/**
 * Expands a folder given a <code>RowNode</code>.
 */
public boolean expandFolder(RowNode rowNode) {
    return controller.expandFolder(rowNode);
}

/**
 * Collapses a folder given a <code>RowNode</code>.
 */
public boolean collapseFolder(RowNode rowNode) {
    return controller.collapseFolder(rowNode);
}

/**
 * Refreshes rows that depend on <code>RowNode</code>.
 */
protected void refreshRowAndBelow(RowNode rowNode, int currentState) {
    int topRow = verticalScrollbar.getValue();
    if (currentState == NodeStatus.FOLDER_OPEN) {
        rowTree.closeFolder(rowNode);
    }
    else if (currentState == NodeStatus.FOLDER_CLOSED) {
        rowTree.openFolder(rowNode);
    }
    else {
        // should never happen
        throw new HiGridException("Bad current state: "+currentState+" for rowNode "+rowNode);
    }
    // opening/closing a folder caused a vertical scroll and this changes everything
    if (topRow != verticalScrollbar.getValue()) {
        refresh();
    }
    else {
        NullGraphics nullgc = controller.getXYFromCell(makeCellPosition(rowNode, NODE_INDICATOR_CELL));
        if (nullgc.finishedOperation) {
            refreshRows(nullgc.cell.point.y, gridArea.getSize().height - nullgc.cell.point.y);
        }
    }
}

protected void refreshRow(RowNode rowNode) {
    NullGraphics nullgc = controller.getXYFromCell(makeCellPosition(rowNode, NODE_INDICATOR_CELL));
    if (nullgc.finishedOperation) {
        refreshRows(nullgc.cell.point.y, rowNode.getHeight());
    }
}

/**
 * Refreshes rows.
 */
protected void refreshRowWithoutEditor(RowNode rowNode) {
    NullGraphics nullgc = controller.getXYFromCell(makeCellPosition(rowNode, NODE_INDICATOR_CELL));
    if (nullgc.finishedOperation) {
        refreshRowsWithoutEditor(nullgc.cell.point.y, rowNode.getHeight());
    }
}

/**
 * Refreshes the children of <code>RowNode</code>.
 */
protected void refreshChildrenRows(RowNode rowNode) {
    if (rowNode == null) {
        refresh();
        return;
    }
    RowNode parentNode = (RowNode)rowNode.getParent();
    if (parentNode == null || parentNode.getParent() == null) {
        refresh();
        return;
    }
    NullGraphics nullgc = controller.getXYFromCell(makeCellPosition(parentNode, NODE_INDICATOR_CELL));
    if (nullgc.finishedOperation) {
        int firstRowOffset = nullgc.cell.point.y + parentNode.getHeight();
//FIX need an optimization here:
// if the cell editor is not part of the subtree being sorted, it shouldn't be positioned
// and thus refreshRowsWithoutEditor() should be called
        refreshRows(firstRowOffset, gridArea.getSize().height - firstRowOffset);
    }
    else {
        // parent node is not visible, must be beyond the top
        refresh();
    }
}

protected void refresh(RowNode oldTopRowNode, RowNode newTopRowNode, boolean downward) {
    if (oldTopRowNode != newTopRowNode) {
        PixelCountWalk walkable = new PixelCountWalk(newTopRowNode, gridArea.getSize().height, downward);
        rowTree.walk(oldTopRowNode, walkable);
        refresh(downward ? walkable.getPixelCount() : -walkable.getPixelCount());
    }
}

/**
 * Refreshes an area given specified by <code>pixels</code>.
 */
protected void refresh(int pixels) {
    if (!isBatched()) {
        controller.showEditor(false);
        rowTree.updateGridState();
        layoutGrid();
        refreshGridArea(pixels);
        controller.showEditor(true);
    }
}

/**
 * Redraw the entire grid.  No drawing will be performed
 * if batched == true.
 */
public void refresh() {
    refresh(true, true);
}

/**
 * Refreshes the grid area if the passed-parameter is true.
 */
protected void refresh(boolean rebuildDoubleBuffer) {
    refresh(rebuildDoubleBuffer, true);
}

/**
 * Refreshes the grid area if the passed-parameters are true.
 */
protected void refresh(boolean rebuildDoubleBuffer, boolean positionEditor) {
    if (!isBatched()) {
        controller.showEditor(false);
        if (rebuildDoubleBuffer) {
            rowTree.updateGridState();
            layoutGrid();
            refreshGridArea();
        }
        else {
            gridArea.repaint();
        }
        if (positionEditor) {
            controller.positionEditor();
        }
        controller.showEditor(true);
    }
}

/**
 * Refreshes the grid area.
 */
protected void refreshGridArea() {
    gridArea.refresh();
    gridArea.repaint();
}

/**
 * Refreshes a portion of the grid area.
 */
protected void refreshGridArea(int pixels) {
    gridArea.refresh(pixels);
    gridArea.repaint();
}

/**
 * Refreshes the grid area based on the coordinates and dimension.
 */
protected void refreshGridArea(int x, int y, int width, int height) {
    gridArea.refresh(x, y, width, height);
    // take the horizontal scrollbar's position into account
    x -= getHorizontalScrollbar().getValue();
    gridArea.repaint(x, y, width, height);
}

/**
 * Implement ComponentListener interface.
 */
public void componentShown(ComponentEvent event) {
}

/**
 * Implement ComponentListener interface.
 */
public void componentHidden(ComponentEvent event) {
}

/**
 * Implement ComponentListener interface.
 */
public void componentMoved(ComponentEvent event) {
}

/**
 * Implement ComponentListener interface.
 */
public void componentResized(ComponentEvent event) {
    layoutGrid();
    rowTree.setGridAreaRows();
    layoutGrid();
    refreshGridArea();
    controller.positionEditor();
    controller.showEditor(true);
}

/**
 * Performs a layout for the grid.
 */
protected void layoutGrid() {
    // do the normal layout
    super.doLayout();
    int preferredWidth = verticalScrollbar.getComponent().getPreferredSize().width;
    if (dataModel == null) {
        if (verticalScrollbar.isVisible()) {
            verticalScrollbar.setVisible(false);
            super.doLayout();
        }
        if (horizontalPanel.isVisible()) {
            horizontalPanel.setVisible(false);
            super.doLayout();
        }
        return;
    }

    // ensure number of grid area rows are calculated
    if (rowTree.getGridAreaRows() == 0) {
        rowTree.updateGridState();
    }

    // check if we need to show the scrollbars
    boolean checkHSB = !(horizontalScrollbar.getDisplay() == GridScrollbar.DISPLAY_NEVER);
    boolean checkVSB = !(verticalScrollbar.getDisplay() == GridScrollbar.DISPLAY_NEVER);
    boolean alwaysHSB = horizontalScrollbar.getDisplay() == GridScrollbar.DISPLAY_ALWAYS;
    boolean alwaysVSB = verticalScrollbar.getDisplay() == GridScrollbar.DISPLAY_ALWAYS;;
    boolean showHSB = false;
    boolean showVSB = false;
    boolean enableHSB = true;
    boolean enableVSB = true;
    int displayWidth = gridArea.getSize().width;
    int displayHeight = gridArea.getSize().height;
    int prevDisplayWidth = displayWidth;
    int prevDisplayHeight = displayHeight;

    if (checkHSB) {
        enableHSB = rowTree.getGridAreaWidth() > displayWidth;
        if (enableHSB || alwaysHSB) {
            showHSB = true;
            if (!horizontalPanel.isVisible()) {
                displayHeight = Math.max(0, displayHeight - preferredWidth);
            }
        }
        else if (horizontalPanel.isVisible()) {
            showHSB = false;
            displayHeight = displayHeight + preferredWidth;
        }
    }
    if (checkVSB) {
        enableVSB = rowTree.getScrollableRowsHeight() > displayHeight;
        if (enableVSB  || alwaysVSB) {
            showVSB = true;
            if (!verticalScrollbar.isVisible()) {
                displayWidth = Math.max(0, displayWidth - preferredWidth);
            }
        }
        else if (verticalScrollbar.isVisible()) {
            showVSB = false;
            displayWidth = displayWidth + preferredWidth;
        }
    }
    if (checkHSB && prevDisplayWidth != displayWidth) {
        if (rowTree.getGridAreaWidth() > displayWidth || alwaysHSB) {
            if (!showHSB) {
                showHSB = true;
                if (!horizontalPanel.isVisible()) {
                    displayHeight = Math.max(0, displayHeight - preferredWidth);
                }
            }
        }
        else if (horizontalPanel.isVisible()) {
            if (showHSB) {
                showHSB = false;
                displayHeight = displayHeight + preferredWidth;
            }
        }
    }
    if (checkVSB && prevDisplayHeight != displayHeight) {
        if (rowTree.getScrollableRowsHeight() > displayHeight || alwaysVSB) {
            if (!showVSB) {
                showVSB = true;
                if (!verticalScrollbar.isVisible()) {
                    displayWidth = Math.max(0, displayWidth - preferredWidth);
                }
            }
        }
        else if (verticalScrollbar.isVisible()) {
            if (showVSB) {
                showVSB = false;
                displayWidth = displayWidth + preferredWidth;
            }
        }
    }
    if (!alwaysHSB && !alwaysVSB && showHSB && showVSB &&
        horizontalPanel.isVisible() && verticalScrollbar.isVisible() &&
        rowTree.getGridAreaWidth() <= (displayWidth + preferredWidth) &&
        rowTree.getScrollableRowsHeight() <= (displayHeight + preferredWidth)) {
        showHSB = false;
        showVSB = false;
    }

    // do re-layout if there is a change in state for the scrollbars
    boolean redoLayout = firstLayout;
    if (firstLayout) {
        firstLayout = false;
    }
    if (showHSB != horizontalPanel.isVisible()) {
        horizontalPanel.setVisible(showHSB);
        if (!showHSB) {
            horizontalScrollbar.setValue(0);
        }
        redoLayout = true;
    }
    if (showVSB != verticalScrollbar.isVisible()) {
        verticalScrollbar.setVisible(showVSB);
        if (!showVSB) {
            rowTree.setTopRow(0);
        }
        redoLayout = true;
    }
    // redoLayout has to be done after vertical scrollbar changes for
    // horizontal scrollbar to size properly
    if (showHSB) {
        horizontalPanel.setSize(new Dimension(getSize().width, preferredWidth));
        horizontalScrollbar.adjustValues(rowTree.getGridAreaWidth(), gridArea.getSize().width);
        horizontalScrollbar.setSize(new Dimension(gridArea.getSize().width, preferredWidth));
    }
    if (redoLayout) {
        super.doLayout();
    }
    if (showVSB) {
        verticalScrollbar.adjustValues(rowTree.getTotalNumberOfScrollableRows(), rowTree.getGridAreaRows());
        verticalScrollbar.setSize(new Dimension(preferredWidth, gridArea.getSize().height));
    }
    if (redoLayout) {
        super.doLayout();
    }
    if (!enableVSB) {
        verticalScrollbar.setValues(0, 0, 0, 0);
    }
    verticalScrollbar.getComponent().setEnabled(enableVSB);
    if (!enableHSB) {
        horizontalScrollbar.setValues(0, 0, 0, 0);
    }
    horizontalScrollbar.getComponent().setEnabled(enableHSB);
    cornerPanel.setVisible(verticalScrollbar.isVisible() && horizontalScrollbar.isVisible());
}

/**
 * Sets the layout manager for this container.  This method is
 * overridden to prevent the layout mgr from being set.
 */

public final void setLayout(LayoutManager mgr) {
    super.setLayout(new BorderLayout());
    add(gridArea, "Center");
    resetScrollbarConstraints();
}

/**
 * Retrieves whether user row selection is allowed.
 *
 * @return True if user row selection is allowed.
 */
public boolean isAllowRowSelection() {
    return allowRowSelection;
}

/**
 * Sets whether user row selection is allowed.
 *
 * @param allow True if user row selection is allowed.
 */
public void setAllowRowSelection(boolean allow) {
    if (allowRowSelection && !allow) {
        resetRowSelection();
    }
    allowRowSelection = allow;
}

/**
 * Retrieves the row selection mode.
 *
 * @return The row selection mode.
 */
public int getRowSelectionMode() {
    return rowSelectionMode;
}

/**
 * Sets the row selection mode.
 *
 * @param selectionMode The row selection mode.
 */
public void setRowSelectionMode(int selectionMode) {
    rowSelectionMode = selectionMode;
}

/**
 * Select row(s) according to the specified option.
 *
 * @param selectOption The Row Selection Option
 */
public void selectRows(int selectOption) {
    if (popupMenu == null) {
        return;
    }
    String command = null;
    switch (selectOption) {
    case SELECT_ROWS_ALL:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_SELECT_ALL);
        break;
    case SELECT_ROWS_ALL_IN_SAME_TABLE:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_SELECT_ALL_IN_SAME_TABLE);
        break;
    case SELECT_ROWS_ALL_IN_SAME_LEVEL:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_SELECT_ALL_IN_SAME_LEVEL);
        break;
    case SELECT_ROWS_CURRENT:
    default:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_SELECT_CURRENT);
        break;
    }
    popupMenu.performCommand(command);
}

/**
 * Toggle the selected status of the indicated rowNode.
 *
 * @param rowNode. The row node to select/deselect.
 * @param allowMultipleSelect.  True if more than one row can be selected.
 * @param refresh. True if the rows are to be repainted.
 */
public void toggleRowSelection(RowNode rowNode, boolean allowMultipleSelections, boolean refresh) {
    if (!allowMultipleSelections) {
        resetRowSelection(refresh);
    }
    controller.doRowSelection(rowNode, rowSelectionMode, false, refresh);
}

/**
 * Toggle the selected status of the indicated rowNode.
 *
 * @param rowNode. The row node to select/deselect.
 * @param allowMultipleSelect.  True if more than one row can be selected.
 */
public void toggleRowSelection(RowNode rowNode, boolean allowMultipleSelections) {
    toggleRowSelection(rowNode, allowMultipleSelections, true);
}

/**
 * Reset all selected rows.
 */
public void resetRowSelection() {
    resetRowSelection(true);
}

/**
 * Reset all selected rows.
 *
 * @param refresh. True if the rows are to be repainted.
 */
public void resetRowSelection(boolean refresh) {
    controller.resetRowSelection(refresh);
}

/**
 * Insert a row in the same table as the indicated rowNode.
 *
 * @param rowNode. The row node.
 */
public void insertRow(RowNode rowNode) {
    if (!isEditable()) {
        return;
    }
    if (rowNode != null) {
        DataTableModel dataTable = rowNode.getDataTableModel();
        if (dataTable != null) {
            try {
                dataTable.addRow(getDataModelListener());
            }
            catch (Exception e) {
                fireHiGridErrorEvent(new HiGridErrorEvent(this, e));
            }
            resetRowSelection();
        }
    }
}

/**
 * Delete row(s) according to the specified option.
 *
 * @param deleteOption. The Row Deletion Option:
 */
public void deleteRows(int deleteOption) {
    if (!isEditable()) {
        return;
    }
    if (popupMenu == null) {
        return;
    }
    String command = null;
    switch (deleteOption) {
    case DELETE_ROWS_SELECTED:
        if (controller.selectedRowNodes.isEmpty()) {
            return;
        }
        command = popupMenu.getCommand(EditPopupMenu.POPUP_DELETE_SELECTED);
        break;
    case DELETE_ROWS_CURRENT:
    default:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_DELETE_CURRENT);
        break;
    }
    popupMenu.performCommand(command);
}

/**
 * Delete the indicated rowNode.
 *
 * @param rowNode. The row node.
 */
public void deleteRow(RowNode rowNode) {
    if (!isEditable()) {
        return;
    }
    if (rowNode != null) {
        DataTableModel dataTable = rowNode.getDataTableModel();
        if (dataTable != null) {
            try {
                dataTable.deleteRow(getDataModelListener(), rowNode.getBookmark());
            }
            catch (Exception e) {
                fireHiGridErrorEvent(new HiGridErrorEvent(this, e));
            }
            resetRowSelection();
        }
    }
}

/**
 * Cancel changes for row(s) according to the specified option.
 *
 * @param cancelOption. The row cancel changes option.
 */
public void cancelRows(int cancelOption) {
    if (!isEditable()) {
        return;
    }
    if (popupMenu == null) {
        return;
    }
    String command = null;
    switch (cancelOption) {
    case CANCEL_ROWS_ALL:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_CANCEL_ALL);
        break;
    case CANCEL_ROWS_SELECTED:
        if (controller.selectedRowNodes.isEmpty()) {
            return;
        }
        command = popupMenu.getCommand(EditPopupMenu.POPUP_CANCEL_SELECTED);
        break;
    case CANCEL_ROWS_CURRENT:
    default:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_CANCEL_CURRENT);
        break;
    }
    popupMenu.performCommand(command);
}

/**
 * Cancel row changes for the indicated rowNode.
 *
 * @param rowNode. The row node.
 */
public void cancelRow(RowNode rowNode) {
    if (!isEditable()) {
        return;
    }
    if (rowNode != null) {
        DataTableModel dataTable = rowNode.getDataTableModel();
        if (dataTable != null) {
            try {
                dataTable.cancelRowChanges(getDataModelListener(), rowNode.getBookmark());
            }
            catch (Exception e) {
                fireHiGridErrorEvent(new HiGridErrorEvent(this, e));
            }
            resetRowSelection();
        }
    }
}

/**
 * Update changes for row(s) according to the specified option.
 *
 * @param updateOption. The row update changes option.
 */
public void updateRows(int updateOption) {
    if (!isEditable()) {
        return;
    }
    if (popupMenu == null) {
        return;
    }
    String command = null;
    switch (updateOption) {
    case UPDATE_ROWS_ALL:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_UPDATE_ALL);
        break;
    case UPDATE_ROWS_SELECTED:
        if (controller.selectedRowNodes.isEmpty()) {
            return;
        }
        command = popupMenu.getCommand(EditPopupMenu.POPUP_UPDATE_SELECTED);
        break;
    case UPDATE_ROWS_CURRENT:
    default:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_UPDATE_CURRENT);
        break;
    }
    popupMenu.performCommand(command);
}

/**
 * Update row changes for the indicated rowNode.
 *
 * @param rowNode. The row node.
 */
public void updateRow(RowNode rowNode) {
    if (!isEditable()) {
        return;
    }
    if (rowNode != null) {
        DataTableModel dataTable = rowNode.getDataTableModel();
        if (dataTable != null) {
            try {
                dataTable.commitRow(getDataModelListener(), rowNode.getBookmark());
            }
            catch (Exception e) {
                fireHiGridErrorEvent(new HiGridErrorEvent(this, e));
            }
            resetRowSelection();
        }
    }
}

/**
 * Requery the row(s) according to the specified option.
 *
 * @param requeryOption. The row requery option.
 */
public void requeryRows(int requeryOption) {
    if (popupMenu == null) {
        return;
    }
    String command = null;
    switch (requeryOption) {
    case REQUERY_ROWS_ALL:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_REQUERY_ALL);
        break;
    case REQUERY_ROWS_SELECTED:
        if (controller.selectedRowNodes.isEmpty()) {
            return;
        }
        command = popupMenu.getCommand(EditPopupMenu.POPUP_REQUERY_SELECTED);
        break;
    case REQUERY_ROWS_SELECTED_AND_DETAILS:
        if (controller.selectedRowNodes.isEmpty()) {
            return;
        }
        command = popupMenu.getCommand(EditPopupMenu.POPUP_REQUERY_SELECTED_AND_DETAILS);
        break;
    case REQUERY_ROWS_RECORD_AND_DETAILS:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_REQUERY_RECORD_AND_DETAILS);
        break;
    case REQUERY_ROWS_RECORD:
    default:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_REQUERY_RECORD);
        break;
    }
    popupMenu.performCommand(command);
}

/**
 * Requery the indicated rowNode.
 *
 * @param rowNode. The row node.
 */
public void requeryRow(RowNode rowNode) {
    if (rowNode != null) {
        DataTableModel dataTable = rowNode.getDataTableModel();
        if (dataTable != null) {
            try {
                dataTable.requeryRow(getDataModelListener(), rowNode.getBookmark());
            }
            catch (Exception e) {
                fireHiGridErrorEvent(new HiGridErrorEvent(this, e));
            }
            resetRowSelection();
        }
    }
}

/**
 * Move to row according to the specified option.
 *
 * @param moveToRowOption. The Move To Row Option:
 */
public void moveToRow(int moveToRowOption) {
    if (popupMenu == null) {
        return;
    }
    String command = null;
    switch (moveToRowOption) {
    case MOVE_TO_ROW_HIGRID_FIRST:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_MOVE_TO_RECORD_FIRST);
        break;
    case MOVE_TO_ROW_HIGRID_PREVIOUS:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_MOVE_TO_RECORD_PREVIOUS);
        break;
    case MOVE_TO_ROW_HIGRID_NEXT:
    default:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_MOVE_TO_RECORD_NEXT);
        break;
    case MOVE_TO_ROW_HIGRID_LAST:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_MOVE_TO_RECORD_LAST);
        break;
    case MOVE_TO_ROW_TABLE_FIRST:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_MOVE_TO_TABLE_RECORD_FIRST);
        break;
    case MOVE_TO_ROW_TABLE_PREVIOUS:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_MOVE_TO_TABLE_RECORD_PREVIOUS);
        break;
    case MOVE_TO_ROW_TABLE_NEXT:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_MOVE_TO_TABLE_RECORD_NEXT);
        break;
    case MOVE_TO_ROW_TABLE_LAST:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_MOVE_TO_TABLE_RECORD_LAST);
        break;
    case MOVE_TO_ROW_PARENT:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_MOVE_TO_PARENT);
        break;
    }
    popupMenu.performCommand(command);
}

/**
 * Move to row specified by the bookmark.
 *
 * @param bookmark. The bookmark of the row.
 */
public void moveToRow(long bookmark) {
    moveToRow(getRowTree().findRecordRowNode(null, bookmark));
}

/**
 * Move to row specified by the bookmark.
 *
 * @param bookmark. The bookmark of the row.
 */
public void moveToRow(RowNode rowNode) {
    if (rowNode == null) {
        return;
    }
    int columnIndex = rowNode.getRowFormat().getFirstVisibleCell();
    CellPosition newPosition = new CellPosition(rowNode, columnIndex);
    controller.validateAndTraverse(newPosition, columnIndex, null);
}

/**
 * Returns an enumeration of the selected row items.
 *
 * @return An enumeration of row objects.
 */
public Enumeration getSelectedRows() {
    return controller.getSelectedRows().elements();
}

/**
 * Gets the <code>Frame</code> containing the <code>Component</code>.
 */
static protected Frame getFrame(Component comp) {
    while ((comp = comp.getParent()) != null) {
        if (comp instanceof Frame) {
            return(Frame) comp;
        }
    }
    return null;
}

/**
 * Prints the grid.
 *
 * @param printFormat. The print format.
 */
public void print(int printFormat) {
    if (popupMenu == null) {
        return;
    }
    String command = null;
    switch (printFormat) {
    case PRINT_AS_EXPANDED:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_PRINT_AS_EXPANDED);
        break;
    case PRINT_EXPAND_ALL:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_PRINT_EXPAND_ALL);
        break;
    case PRINT_AS_DISPLAYED:
    default:
        command = popupMenu.getCommand(EditPopupMenu.POPUP_PRINT_AS_DISPLAYED);
        break;
    }
    popupMenu.performCommand(command);
}

/**
 * Prints the grid without prompting the user in the default page orientation.
 */
public void print() {

    print(null, false, null);



}

/**
 * Prints the grid with the specified pageFormat.
 *
 * @param pageFormat describes the page orientation for all pages to be printed.
 */

public void print(PageFormat pageFormat) {
    print(null, false, pageFormat);
}


/**
 * Prints the grid with an optional user dialog.
 * If the correct number of pages in the print dialog are desired, then
 * job.setPageable(grid.getPrintGrid());
 * should be called prior to this method.
 *
 * @param displayPrintDialog true if the user should be prompted with the print dialog.
 */

public void print(boolean displayPrintDialog) {
    print(null, displayPrintDialog, null);
}


/**
 * Prints the grid.
 * If the correct number of pages in the print dialog are desired, then
 * job.setPageable(grid.getPrintGrid());
 * should be called prior to this method.
 *
 * @param job The printjob.
 * @param displayPrintDialog true if the user should be prompted with the print dialog.
 * @param pageFormat describes the page orientation for all pages to be printed.
 */

public void print(PrinterJob job, boolean displayPrintDialog, PageFormat pageFormat) {
    if (job == null) {
        // get a PrinterJob
        job = PrinterJob.getPrinterJob();
    }
    if (displayPrintDialog) {
        if (! job.printDialog()) {
            return;
        }
    }
    if (pageFormat == null) {
        pageFormat = job.defaultPage();
    }
    getPrintGrid().initialize(pageFormat);
    setPrinting(true);
    job.setPrintable(getPrintGrid(), pageFormat);
    try {
        job.print();
    }
    catch (Exception e) {
        // do nothing
    }
    finally {
        setPrinting(false);
    }
}




/**
 * Print preview the grid.
 */
public void printPreview() {
    Frame frame = getFrame(this);
    boolean useOwnFrame = false;
    if (frame == null) {
        useOwnFrame = true;
        frame = new Frame();
    }
    PrintPreview preview = new PrintPreview(frame, LocaleBundle.string(LocaleBundle.printPreview), this);
    if (preview != null) {
        preview.show();
    }
    if (useOwnFrame) {
        frame.dispose();
    }
}

/**
 * Returns the printing status.
 *
 * @return True if printing.
 */
public boolean isPrinting() {
    return printing;
}

/**
 * Sets the printing status.
 *
 * @param printing The printing status.
 */
public void setPrinting(boolean printing) {
    this.printing = printing;
}

/**
 * Determines if printing subsequent column pages.
 */
protected boolean isPrintingSubsequentColumnPages() {
    if (isPrinting() && getPrintColumnPageIndex() != 0) {
        return true;
    }
    return false;
}

/**
 * Determines if showing or printing folders and connections.
 */
protected boolean isDrawingFoldersAndConnections() {
    if (!isPrinting() || isPrintFoldersAndConnections()) {
        return true;
    }
    return false;
}

/**
 * Sets the <code>PrintGrid</code> object for this grid.
 */
protected void setPrintGrid(PrintGrid grid) {
    this.printGrid = grid;
}

/**
 * Returns the print grid object.
 */

public PrintGrid12 getPrintGrid() {
    return (PrintGrid12)printGrid;
}





/**
 * Returns the printing row page index value.
 */
public int getPrintRowPageIndex() {
    return(getPrintGrid() == null) ? 0 : getPrintGrid().getRowPageIndex();
}

/**
 * Returns the printing column page index value.
 */
public int getPrintColumnPageIndex() {
    return(getPrintGrid() == null) ? 0 : getPrintGrid().getColumnPageIndex();
}

/**
 * Returns the print folder nodes and connection lines indication.
 */
public boolean isPrintFoldersAndConnections() {
    return printFoldersAndConnections;
}

/**
 * Sets the print folder nodes and connection lines indication.
 *
 * @param printConnections The print folder nodes and connection lines indication
 */
public void setPrintFoldersAndConnections(boolean printConnections) {
    printFoldersAndConnections = printConnections;
}

/**
 * Returns the print format.
 */
public int getPrintFormat() {
    return printFormat;
}

/**
 * Sets the print format.
 *
 * @param printFormat The print format.
 */
public void setPrintFormat(int printFormat) {
    this.printFormat = printFormat;
}

/**
 * Adds the specified listener to receive HiGridColumnSelectionEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void addHiGridColumnSelectionListener(HiGridColumnSelectionListener listener) {
    hiGridColumnSelectionListeners = JCListenerList.add(hiGridColumnSelectionListeners, listener);
}

/**
 * Removes the specified listener so it no longer receives HiGridColumnSelectionEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void removeHiGridColumnSelectionListener(HiGridColumnSelectionListener listener) {
    hiGridColumnSelectionListeners = JCListenerList.remove(hiGridColumnSelectionListeners, listener);
}

/**
 * Notify registered listeners of a grid ColumnSelection event.
 *
 * @param event The HiGridColumnSelectionEvent.
 *
 * @return true if the event was accepted by all listeners, or false
 * as soon as any listener cancels the change.
 */
protected boolean fireHiGridColumnSelectionEvent(HiGridColumnSelectionEvent event) {
    Enumeration e = JCListenerList.elements(hiGridColumnSelectionListeners);
    while (e.hasMoreElements()) {
        HiGridColumnSelectionListener listener = (HiGridColumnSelectionListener)e.nextElement();
        switch (event.getEventType()) {
        case HiGridColumnSelectionEvent.SELECT_COLUMN:
            listener.selectColumn(event);
            break;
        default:
            break;
        }
        if (event.getCancelled() && event.isCancelable()) {
            return false;
        }
    }
    return true;
}

/**
 * Adds the specified listener to receive HiGridErrorEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void addHiGridErrorListener(HiGridErrorListener listener) {
    hiGridErrorListeners = JCListenerList.add(hiGridErrorListeners, listener);
}

/**
 * Removes the specified listener so it no longer receives HiGridErrorEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void removeHiGridErrorListener(HiGridErrorListener listener) {
    hiGridErrorListeners = JCListenerList.remove(hiGridErrorListeners, listener);
}

/**
 * Notify registered listeners of a grid Error event.
 *
 * @param event The HiGridErrorEvent.
 *
 * @return true if the event was accepted by all listeners, or false
 * as soon as any listener cancels the change.
 */
protected boolean fireHiGridErrorEvent(HiGridErrorEvent event) {
    Enumeration e = JCListenerList.elements(hiGridErrorListeners);
    while (e.hasMoreElements()) {
        HiGridErrorListener listener = (HiGridErrorListener)e.nextElement();
        listener.processError(event);
    }
    return true;
}

/**
 * Adds the specified listener to receive HiGridExpansionEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void addHiGridExpansionListener(HiGridExpansionListener listener) {
    hiGridExpansionListeners = JCListenerList.add(hiGridExpansionListeners, listener);
}

/**
 * Removes the specified listener so it no longer receives HiGridExpansionEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void removeHiGridExpansionListener(HiGridExpansionListener listener) {
    hiGridExpansionListeners = JCListenerList.remove(hiGridExpansionListeners, listener);
}

/**
 * Notify registered listeners of a grid Expansion event.
 *
 * @param event The HiGridExpansionEvent.
 *
 * @return true if the event was accepted by all listeners, or false
 * as soon as any listener cancels the change.
 */
protected boolean fireHiGridExpansionEvent(HiGridExpansionEvent event) {
    Enumeration e = JCListenerList.elements(hiGridExpansionListeners);
    while (e.hasMoreElements()) {
        HiGridExpansionListener listener = (HiGridExpansionListener)e.nextElement();
        switch (event.getEventType()) {
        case HiGridExpansionEvent.BEFORE_EXPAND_ROW:
            listener.beforeExpandRow(event);
            break;
        case HiGridExpansionEvent.BEFORE_COLLAPSE_ROW:
            listener.beforeCollapseRow(event);
            break;
        case HiGridExpansionEvent.AFTER_COLLAPSE_ROW:
            listener.afterCollapseRow(event);
            break;
        case HiGridExpansionEvent.AFTER_EXPAND_ROW:
            listener.afterExpandRow(event);
            break;
        default:
            break;
        }
        if (event.getCancelled() && event.isCancelable()) {
            return false;
        }
    }
    return true;
}

/**
 * Adds the specified listener to receive HiGridFormatNodeEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void addHiGridFormatNodeListener(HiGridFormatNodeListener listener) {
    hiGridFormatNodeListeners = JCListenerList.add(hiGridFormatNodeListeners, listener);
}

/**
 * Removes the specified listener so it no longer receives HiGridFormatNodeEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void removeHiGridFormatNodeListener(HiGridFormatNodeListener listener) {
    hiGridFormatNodeListeners = JCListenerList.remove(hiGridFormatNodeListeners, listener);
}

/**
 * Notify registered listeners of a grid FormatNode event.
 *
 * @param event The HiGridFormatNodeEvent.
 *
 * @return true if the event was accepted by all listeners, or false
 * as soon as any listener cancels the change.
 */
protected boolean fireHiGridFormatNodeEvent(HiGridFormatNodeEvent event) {
    Enumeration e = JCListenerList.elements(hiGridFormatNodeListeners);
    while (e.hasMoreElements()) {
        HiGridFormatNodeListener listener = (HiGridFormatNodeListener)e.nextElement();
        switch (event.getEventType()) {
        case HiGridFormatNodeEvent.BEFORE_CREATE_FORMAT_NODE_CONTENTS:
            listener.beforeCreateFormatNodeContents(event);
            break;
        case HiGridFormatNodeEvent.AFTER_CREATE_FORMAT_NODE_CONTENTS:
            listener.afterCreateFormatNodeContents(event);
            break;
        default:
            break;
        }
        if (event.getCancelled() && event.isCancelable()) {
            return false;
        }
    }
    return true;
}

/**
 * Adds the specified listener to receive HiGridMoveCellEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void addHiGridMoveCellListener(HiGridMoveCellListener listener) {
    hiGridMoveCellListeners = JCListenerList.add(hiGridMoveCellListeners, listener);
}

/**
 * Removes the specified listener so it no longer receives HiGridMoveCellEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void removeHiGridMoveCellListener(HiGridMoveCellListener listener) {
    hiGridMoveCellListeners = JCListenerList.remove(hiGridMoveCellListeners, listener);
}

/**
 * Notify registered listeners of a grid MoveCell event.
 *
 * @param event The HiGridMoveCellEvent.
 *
 * @return true if the event was accepted by all listeners, or false
 * as soon as any listener cancels the change.
 */
protected boolean fireHiGridMoveCellEvent(HiGridMoveCellEvent event) {
    Enumeration e = JCListenerList.elements(hiGridMoveCellListeners);
    while (e.hasMoreElements()) {
        HiGridMoveCellListener listener = (HiGridMoveCellListener)e.nextElement();
        switch (event.getEventType()) {
        case HiGridMoveCellEvent.BEFORE_MOVE_COLUMN:
            listener.beforeMoveColumn(event);
            break;
        case HiGridMoveCellEvent.AFTER_MOVE_COLUMN:
            listener.afterMoveColumn(event);
            break;
        default:
            break;
        }
        if (event.getCancelled() && event.isCancelable()) {
            return false;
        }
    }
    return true;
}

/**
 * Adds the specified listener to receive HiGridPrintEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void addHiGridPrintListener(HiGridPrintListener listener) {
    hiGridPrintListeners = JCListenerList.add(hiGridPrintListeners, listener);
}

/**
 * Removes the specified listener so it no longer receives HiGridPrintEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void removeHiGridPrintListener(HiGridPrintListener listener) {
    hiGridPrintListeners = JCListenerList.remove(hiGridPrintListeners, listener);
}

/**
 * Notify registered listeners of a grid print event.
 *
 * @param event The HiGridPrintEvent.
 */
protected void fireHiGridPrintEvent(HiGridPrintEvent event) {
    Enumeration e = JCListenerList.elements(hiGridPrintListeners);
    if (!e.hasMoreElements() && event.getEventType() == HiGridPrintEvent.PRINT_FOOTER) {
        if (getPrintGrid() != null) {
            getPrintGrid().printPageFooter(event);
        }
    }
    while (e.hasMoreElements()) {
        HiGridPrintListener listener = (HiGridPrintListener)e.nextElement();
        switch (event.getEventType()) {
        case HiGridPrintEvent.PRINT_HEADER:
            listener.printPageHeader(event);
            break;
        case HiGridPrintEvent.PRINT_FOOTER:
            listener.printPageFooter(event);
            break;
        case HiGridPrintEvent.PRINT_END:
            listener.printEnd(event);
            break;
        default:
            break;
        }
    }
}

/**
 * Adds the specified listener to receive HiGridRepaintEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void addHiGridRepaintListener(HiGridRepaintListener listener) {
    hiGridRepaintListeners = JCListenerList.add(hiGridRepaintListeners, listener);
}

/**
 * Removes the specified listener so it no longer receives HiGridRepaintEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void removeHiGridRepaintListener(HiGridRepaintListener listener) {
    hiGridRepaintListeners = JCListenerList.remove(hiGridRepaintListeners, listener);
}

/**
 * Notify registered listeners of a grid Repaint event.
 *
 * @param event The HiGridRepaintEvent.
 *
 * @return true if the event was accepted by all listeners, or false
 * as soon as any listener cancels the change.
 */
protected boolean fireHiGridRepaintEvent(HiGridRepaintEvent event) {
    Enumeration e = JCListenerList.elements(hiGridRepaintListeners);
    while (e.hasMoreElements()) {
        HiGridRepaintListener listener = (HiGridRepaintListener)e.nextElement();
        switch (event.getEventType()) {
        case HiGridRepaintEvent.BEFORE_REPAINT_ROW:
            listener.beforeRepaintRow(event);
            break;
        default:
            break;
        }
    }
    return true;
}

/**
 * Adds the specified listener to receive HiGridResizeCellEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void addHiGridResizeCellListener(HiGridResizeCellListener listener) {
    hiGridResizeCellListeners = JCListenerList.add(hiGridResizeCellListeners, listener);
}

/**
 * Removes the specified listener so it no longer receives HiGridResizeCellEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void removeHiGridResizeCellListener(HiGridResizeCellListener listener) {
    hiGridResizeCellListeners = JCListenerList.remove(hiGridResizeCellListeners, listener);
}

/**
 * Notify registered listeners of a grid ResizeCell event.
 *
 * @param event The HiGridResizeCellEvent.
 *
 * @return true if the event was accepted by all listeners, or false
 * as soon as any listener cancels the change.
 */
protected boolean fireHiGridResizeCellEvent(HiGridResizeCellEvent event) {
    Enumeration e = JCListenerList.elements(hiGridResizeCellListeners);
    while (e.hasMoreElements()) {
        HiGridResizeCellListener listener = (HiGridResizeCellListener)e.nextElement();
        switch (event.getEventType()) {
        case HiGridResizeCellEvent.BEFORE_RESIZE_ROW:
            listener.beforeResizeRow(event);
            break;
        case HiGridResizeCellEvent.BEFORE_RESIZE_COLUMN:
            listener.beforeResizeColumn(event);
            break;
        case HiGridResizeCellEvent.AFTER_RESIZE_ROW:
            listener.afterResizeRow(event);
            break;
        case HiGridResizeCellEvent.AFTER_RESIZE_COLUMN:
            listener.afterResizeColumn(event);
            break;
        default:
            break;
        }
        if (event.getCancelled() && event.isCancelable()) {
            return false;
        }
    }
    return true;
}

/**
 * Adds the specified listener to receive HiGridRowSelectionEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void addHiGridRowSelectionListener(HiGridRowSelectionListener listener) {
    hiGridRowSelectionListeners = JCListenerList.add(hiGridRowSelectionListeners, listener);
}

/**
 * Removes the specified listener so it no longer receives HiGridRowSelectionEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void removeHiGridRowSelectionListener(HiGridRowSelectionListener listener) {
    hiGridRowSelectionListeners = JCListenerList.remove(hiGridRowSelectionListeners, listener);
}

/**
 * Notify registered listeners of a grid RowSelection event.
 *
 * @param event The HiGridRowSelectionEvent.
 *
 * @return true if the event was accepted by all listeners, or false
 * as soon as any listener cancels the change.
 */
protected boolean fireHiGridRowSelectionEvent(HiGridRowSelectionEvent event) {
    Enumeration e = JCListenerList.elements(hiGridRowSelectionListeners);
    while (e.hasMoreElements()) {
        HiGridRowSelectionListener listener = (HiGridRowSelectionListener)e.nextElement();
        switch (event.getEventType()) {
        case HiGridRowSelectionEvent.BEFORE_SELECT_ROW:
            listener.beforeSelectRow(event);
            break;
        case HiGridRowSelectionEvent.AFTER_SELECT_ROW:
            listener.afterSelectRow(event);
            break;
        default:
            break;
        }
        if (event.getCancelled() && event.isCancelable()) {
            return false;
        }
    }
    return true;
}

/**
 * Adds the specified listener to receive HiGridSortTableEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void addHiGridSortTableListener(HiGridSortTableListener listener) {
    hiGridSortTableListeners = JCListenerList.add(hiGridSortTableListeners, listener);
}

/**
 * Removes the specified listener so it no longer receives HiGridSortTableEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void removeHiGridSortTableListener(HiGridSortTableListener listener) {
    hiGridSortTableListeners = JCListenerList.remove(hiGridSortTableListeners, listener);
}

/**
 * Notify registered listeners of a grid SortTable event.
 *
 * @param event The HiGridSortTableEvent.
 *
 * @return true if the event was accepted by all listeners, or false
 * as soon as any listener cancels the change.
 */
protected boolean fireHiGridSortTableEvent(HiGridSortTableEvent event) {
    Enumeration e = JCListenerList.elements(hiGridSortTableListeners);
    while (e.hasMoreElements()) {
        HiGridSortTableListener listener = (HiGridSortTableListener)e.nextElement();
        switch (event.getEventType()) {
        case HiGridSortTableEvent.BEFORE_SORT_TABLE:
            listener.beforeSortTable(event);
            break;
        case HiGridSortTableEvent.AFTER_SORT_TABLE:
            listener.afterSortTable(event);
            break;
        default:
            break;
        }
        if (event.getCancelled() && event.isCancelable()) {
            return false;
        }
    }
    return true;
}

/**
 * Adds the specified listener to receive HiGridTraverseEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void addHiGridTraverseListener(HiGridTraverseListener listener) {
    hiGridTraverseListeners = JCListenerList.add(hiGridTraverseListeners, listener);
}

/**
 * Removes the specified listener so it no longer receives HiGridTraverseEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void removeHiGridTraverseListener(HiGridTraverseListener listener) {
    hiGridTraverseListeners = JCListenerList.remove(hiGridTraverseListeners, listener);
}

/**
 * Notify registered listeners of a grid Traverse event.
 *
 * @param event The HiGridTraverseEvent.
 *
 * @return true if the event was accepted by all listeners, or false
 * as soon as any listener cancels the change.
 */
protected boolean fireHiGridTraverseEvent(HiGridTraverseEvent event) {
    Enumeration e = JCListenerList.elements(hiGridTraverseListeners);
    while (e.hasMoreElements()) {
        HiGridTraverseListener listener = (HiGridTraverseListener)e.nextElement();
        switch (event.getEventType()) {
        case HiGridTraverseEvent.AFTER_TRAVERSE:
            listener.afterTraverse(event);
            break;
        default:
            break;
        }
        if (event.getCancelled() && event.isCancelable()) {
            return false;
        }
    }
    return true;
}

/**
 * Adds the specified listener to receive HiGridUpdateEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void addHiGridUpdateListener(HiGridUpdateListener listener) {
    hiGridUpdateListeners = JCListenerList.add(hiGridUpdateListeners, listener);
}

/**
 * Removes the specified listener so it no longer receives HiGridUpdateEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void removeHiGridUpdateListener(HiGridUpdateListener listener) {
    hiGridUpdateListeners = JCListenerList.remove(hiGridUpdateListeners, listener);
}

/**
 * Notify registered listeners of a grid Update event.
 *
 * @param event The HiGridUpdateEvent.
 *
 * @return true if the event was accepted by all listeners, or false
 * as soon as any listener cancels the change.
 */
protected boolean fireHiGridUpdateEvent(HiGridUpdateEvent event) {
    Enumeration e = JCListenerList.elements(hiGridUpdateListeners);
    while (e.hasMoreElements()) {
        HiGridUpdateListener listener = (HiGridUpdateListener)e.nextElement();
        switch (event.getEventType()) {
        case HiGridUpdateEvent.AFTER_RESET_FORMAT_DATA:
            listener.afterResetFormatData(event);
            break;
        case HiGridUpdateEvent.AFTER_RESET_HIGRID_DATA:
            listener.afterResetHiGridData(event);
            break;
        case HiGridUpdateEvent.AFTER_CREATE_ROW:
            listener.afterCreateRow(event);
            break;
        default:
            break;
        }
        if (event.getCancelled() && event.isCancelable()) {
            return false;
        }
    }
    return true;
}

/*
Validation scheme:
Developers need to register with HiGrid as cell editor validation
listeners.  HiGrid itself is a validation listener on the current
cell editor (HiGrid is added and removed as a listener by the
Controller, whenever the user moves from cell to cell)
and passes a wrapped (enhanced with additional contextual information)
event to the developer.  This allows the developer to only have to add
himself as a listener on all cell editors once.
*/

/**
 * Adds the specified listener to receive HiGridValidateEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void addHiGridValidateListener(HiGridValidateListener listener) {
    hiGridValidateListeners = JCListenerList.add(hiGridValidateListeners, listener);
}

/**
 * Removes the specified listener so it no longer receives HiGridValidateEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void removeHiGridValidateListener(HiGridValidateListener listener) {
    hiGridValidateListeners = JCListenerList.remove(hiGridValidateListeners, listener);
}

protected HiGridValidateEvent makeValidateEvent(int eventType, JCValidateEvent event) {
    RowNode row = null;
    String column = "";
    CellPosition cell = getCurrentCell();
    if (cell != null) {
        row = cell.row;
        column = cell.format.getName();
    }
    try {
        return new HiGridValidateEvent(this,
                                       HiGridValidateEvent.VALUE_CHANGED_BEGIN, event, row, column);
    }
    catch (DataModelException ex) {
        // should not happen
        ex.printStackTrace();
    }
    return null;
}

//////////////////////
// ValidateListener methods
/**
 * Invoked just before the internal value of the field is changed.
 */
public void valueChangedBegin(JCValidateEvent event) {
    Enumeration e = JCListenerList.elements(hiGridValidateListeners);
    while (e.hasMoreElements()) {
        HiGridValidateListener listener = (HiGridValidateListener)e.nextElement();
        HiGridValidateEvent validateEvent = makeValidateEvent(
                                                             HiGridValidateEvent.VALUE_CHANGED_BEGIN, event);
        listener.valueChangedBegin(validateEvent);
        // return early if any of the listeners reject the change
        if (!validateEvent.getValidateEvent().isValid()) {
            return;
        }
    }
}

/**
 * Invoked just after the internal value of the field has been changed.
 */
public void valueChangedEnd(JCValidateEvent event) {
    Enumeration e = JCListenerList.elements(hiGridValidateListeners);
    while (e.hasMoreElements()) {
        HiGridValidateListener listener = (HiGridValidateListener)e.nextElement();
        HiGridValidateEvent validateEvent = makeValidateEvent(
                                                             HiGridValidateEvent.VALUE_CHANGED_END, event);
        listener.valueChangedEnd(validateEvent);
    }
}

/**
 * Invoked if the state is changed to INVALID.
 */
public void stateIsInvalid(JCValidateEvent event) {
    Enumeration e = JCListenerList.elements(hiGridValidateListeners);
    while (e.hasMoreElements()) {
        HiGridValidateListener listener = (HiGridValidateListener)e.nextElement();
        HiGridValidateEvent validateEvent = makeValidateEvent(
                                                             HiGridValidateEvent.STATE_IS_INVALID, event);
        listener.stateIsInvalid(validateEvent);
    }
    if (isBeepOnInvalid() && event.getBeep() && !event.isValid()) {
        getToolkit().beep();
    }
}

/**
 * Returns whether this component can be traversed using
 * Tab or Shift-Tab keyboard focus traversal. .
 */
public boolean isFocusTraversable() {
    return true;
}

/**
 * Gets the minimum allowable size for the grid.
 */
// Note: getPreferredSize() of HiGrid is based on getPreferredSize() of its
// children, namely the GridArea and the GridScrollbars
public Dimension getMinimumSize() {
    return getPreferredSize();
}

protected void finalize() throws Throwable {
    //FIX
//System.out.println("Finalizing HiGrid "+ this);
}

/**
 * Clears all memory references to enable HiGrid to be garbage collected.
 */
public void dispose() {
    if (getDataModel() != null) {
        setDataModel(null);
        setDataModelListener(null);
    }
    setController(null);
    setGridArea(null);
    setEditPopupMenu(null);

	// remove the link to the ui
	ui = null;

//FIX
// these seem not to be necessary
/*
    removeAll();
    setLookupTable(null);
    removeComponentListener(this);
    setRowTree(null);
    setFormatTree(null);
    setPrintGrid(null);
    setColumnSelectionHandler(null);
    setErrorHandler(null);

    sortable = null;
    defaultSortable = null;
    folderIconStyle = null;
    verticalScrollbar.setAdjustable(null);
    verticalScrollbar = null;
    horizontalScrollbar.setAdjustable(null);
    horizontalScrollbar = null;

    hiGridColumnSelectionListeners = null;
    hiGridErrorListeners = null;
    hiGridExpansionListeners = null;
    hiGridMoveCellListeners = null;
    hiGridPrintListeners = null;
    hiGridRepaintListeners = null;
    hiGridResizeCellListeners = null;
    hiGridRowSelectionListeners = null;
    hiGridSortTableListeners = null;
    hiGridTraverseListeners = null;
    hiGridUpdateListeners = null;
    hiGridValidateListeners = null;
*/

}

/**
 * Test routine to generate an error.
 */
public void generateError() {
    try {
        if (getDataModel() != null) {
            getCurrentRowNode().getDataTableModel().getResultData(getCurrentRowNode().getBookmark(), "Unknown Field");
        }
    }
    catch (Exception e) {
        fireHiGridErrorEvent(new HiGridErrorEvent(this, e));
    }
}

protected void printStackTrace() {
    
}

/**
 * Adds an item unless it is already there.
 */
static protected void addUnique(Vector v, Object object) {
    if (!v.contains(object)) {
        v.addElement(object);
    }
}

}
