/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: HiGridData.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:19 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.io.Serializable;

import javax.swing.tree.*;

import com.klg.jclass.datasource.*;

/**
 * HiGridData includes support for the footer,
 * before details, and after details records.
 */
public class HiGridData implements DataModel, Serializable {

static final long serialVersionUID = 1222248980540293497L;

public static final int FOOTER          = 0;
public static final int BEFORE_DETAILS  = 1;
public static final int AFTER_DETAILS   = 2;
public static final int SUMMARY_ROWS    = 3;


// the datasource data model
protected DataModel dataModel;

// the summary meta data trees
private DataSourceTreeModel footerMetaTree;
private DataSourceTreeModel beforeDetailsMetaTree;
private DataSourceTreeModel afterDetailsMetaTree;

/**
 * Constructors.
 */
protected HiGridData() {
    // instantiate the tree implementations
//    footerMetaTree = new DataSourceTreeModel(new DataSourceTreeNode());
//    beforeDetailsMetaTree = new DataSourceTreeModel(new DataSourceTreeNode());
//    afterDetailsMetaTree = new DataSourceTreeModel(new DataSourceTreeNode());
    footerMetaTree = new DataSourceTreeModel(DataSourceTreeModel.NullRoot);
    beforeDetailsMetaTree = new DataSourceTreeModel(DataSourceTreeModel.NullRoot);
    afterDetailsMetaTree = new DataSourceTreeModel(DataSourceTreeModel.NullRoot);
}

public HiGridData(DataModel dataModel) {
    this();
    this.dataModel = dataModel;
}

/////////////////////////////////////////////////////////////////////////////
// implementation of the DataModel interface and TreeData object

/**
 * Return a handle to the MetaTree.
 * @return a MetaDataTableTreeModel which can be traversed to determine the
 * hierarchy of the data.
 */
public TreeModel getMetaDataTree() {
    return dataModel.getMetaDataTree();
}

/**
 * Return a handle to the DataTree.
 * @return a DataTableTreeModel which can be traversed to the hierarchy
 * of DataTables
 */
public TreeModel getDataTableTree() {
    return dataModel.getDataTableTree();
}

/**
 * Register a listener for this Data Model.
 * @see com.klg.jclass.datasource.DataModelListener
 */
public void addDataModelListener(DataModelListener listener) {
    dataModel.addDataModelListener(listener);
}

/**
 * Remove a listener of this Data Model.
 * @see com.klg.jclass.datasource.DataModelListener
 */
public void removeDataModelListener(DataModelListener listener) {
    dataModel.removeDataModelListener(listener);
}

/**
 * Called by observers to tell this data model to which row it
 * should move.  Depending on the commit policy,
 * moving the internal cursor may automatically commit changes
 * to the originating data source.
 *
 * @param originator the DataModelListener which initiated this action
 * @param data_table the DataTable to which the cursor should be moved
 * @param bookmark a long uniquely identify to which row the cursor
 * should move
 * @return true if move succeeds, false otherwise
 * @exception DataModelException If move to row fails.
 */
public boolean moveToRow(DataModelListener originator, DataTableModel data_table, long bookmark) throws DataModelException {
    return dataModel.moveToRow(originator, data_table, bookmark);
}

/**
 * For internal use only.
 * Called by observers to tell this data model to which row it
 * should move.  Depending on the commit policy,
 * moving the internal cursor may automatically commit changes
 * to the originating data source.
 *
 * @param originator the DataModelListener which initiated this action
 * @param  data_table the DataTable to which the cursor should be moved
 * @param  bookmark a long uniquely identify to which row the cursor
 *         should move
 * @param moveIdeCursor a boolean indicating if this move should also cause
 * the ide's cursor to move
 * @return true if move succeeded, false otherwise
 * @exception DataModelException If move to row fails.
 */
public boolean moveToRow(DataModelListener originator,
                         DataTableModel data_table, long bookmark, boolean moveIdeCursor) throws DataModelException {
    return dataModel.moveToRow(originator, data_table, bookmark, moveIdeCursor);
}

/**
 * Write all pending changes in all DataTables to the data source.
 *
 * @param originator the DataModelListener which initiated this action
 * @exception DataModelException If update fails.
 */
public void updateAll(DataModelListener originator) throws DataModelException {
    dataModel.updateAll(originator);
}

/**
 * Cancel all pending changes in all DataTables.
 *
 * @param originator the DataModelListener which initiated this action
 * @exception DataModelException If an exception occurs while canceling changes.
 */
public void cancelAll(DataModelListener originator) throws DataModelException {
    dataModel.cancelAll(originator);
}

/**
 * Requery all DataTables.
 *
 * @param originator the DataModelListener which initiated this action
 * @exception DataModelException If requery fails.
 */
public void requeryAll(DataModelListener originator) throws DataModelException {
    dataModel.requeryAll(originator);
}

/**
 * Return a list of listeners on this DataModel.
 * @return an Object which is a list of listeners
 */
public Object getListeners() {
    return dataModel.getListeners();
}

/**
 * Pass event on to listeners of this DataModel.
 * @param event a DataModelEvent
 */
public boolean fireDataModelEvent(DataModelEvent event) {
    return dataModel.fireDataModelEvent(event);
}

/**
 * Pass event on to listeners of this DataModel.
 * @param event a DataModelEvent
 */
public boolean fireDataModelEvent(DataModelEvent event, DataModelListener listener) {
    return dataModel.fireDataModelEvent(event, listener);
}

/**
 * Enable or disable data model events.  When disabled, events are not
 * dispatched to registered listeners.  This is useful for making mass changes
 * without having listeners repainting on each change.  That is,
 * set enabled to false, make the changes, then set back true.
 * <P>
 * This is different from the DataModelEvent
 * BEGIN_EVENTS and END_EVENTS events where events are still sent
 * but the listener having received the BEGIN_EVENTS event knows it may choose
 * to disregard the events until the END_EVENTS event is received.  They are
 * given the choice.
 * <P>
 * Temporarily disabling events is more efficient than using the BEGIN/END_EVENTS
 * method since the events are not dispatched to all listeners.
 * @param enable a boolean, true if events are to be dispatched to listeners,
 * false otherwise.
 * @see com.klg.jclass.datasource.DataModelEvent
 * @see #getEventsEnabled
 */
public void enableDataModelEvents(boolean value) {
    dataModel.enableDataModelEvents(value);
}

/**
 * Return true if events are enabled, false otherwise.
 * @return true if events are enabled, false otherwise.
 * @see #enableDataModelEvents
 */
public boolean getEventsEnabled() {
    return dataModel.getEventsEnabled();
}

/**
 * Set this model's name.
 * This is used to identify this model so it must be unique.
 * @param modelName a String which is to be this model's name
 */
public void setModelName(String modelName) {
    dataModel.setModelName(modelName);
}

/**
 * Get this model's name.
 * If no name is set, the default name of an instance of this
 * class will be the concatenation of this class's name
 * and a numeric digit. For example TreeData1.
 * @return a String which is the name of this model.
 */
public String getModelName() {
    return dataModel.getModelName();
}

/**
 * Return the DataTableModel currently in focus.
 * @return the DataTableModel which has the global
 * cursor's focus
 * @see #moveToRow
 * @see #getCurrentGlobalBookmark
 */
public DataTableModel getCurrentGlobalTable() {
    return dataModel.getCurrentGlobalTable();
}

/**
 * Return the bookmark of the row which has the
 * global cursor's focus.
 * @return the bookmark of the row on which the
 * global cursor is located
 * @see #moveToRow
 * @see #getCurrentGlobalTable
 */
public long getCurrentGlobalBookmark() {
    return dataModel.getCurrentGlobalBookmark();
}

/**
 * Are there any pending modifications in any rows in this entire
 * data model?
 * @return true if there is at least one change pending, false otherwise
 */
public boolean isModified() {
    return dataModel.isModified();
}

/**
 * Returns the DataTable at the specified MetaData level based on
 * the current bookmark.  Bookmark refers to what we're calling
 * the "bookmark path" ie the bookmarks for each parent MetaData
 * level down to the specified level.
 * <P>
 * Will create the table if it does not exist.  In this case it
 * will be the child of the current row in the parent, and so on.
 *
 * @param originator the DataModelListener which initiated this action
 * @param metaDataModel the MetaDataModel for which the current
 * DataTableModel should be returned
 * @return the DataTableModel currently in focus for this level
 * @exception DataModelException If get fails.
 */
public DataTableModel getCurrentDataTable(DataModelListener originator, MetaDataModel myLevel) throws
DataModelException {
    return dataModel.getCurrentDataTable(originator, myLevel);
}


/**
 * Returns the data item at the specified MetaData level and column for
 * the current row.
 * Equivalent to calling DataTableModel.getResultData() with the
 * current bookmark at the MetaData level.
 * <P>
 * Will create the table if it does not exist.  In this case it
 * will be the child of the current row in the parent, and so on.
 *
 * @param originator the DataModelListener which initiated this action
 * @param metaDataModel the MetaDataModel from which the column value
 * is to be retrieved
 * @param column_identifier a String which is the column from which
 * a value is to be retrieved
 * @exception DataModelException If get fails.
 */
public Object getCurrentDataItem(DataModelListener originator, MetaDataModel myLevel, String column) throws
DataModelException {
    return dataModel.getCurrentDataItem(originator, myLevel, column);
}

/**
 * Get a MetaDataModel instance by name, or null if not found.
 * @param metaDataName a String which is the name of the MetaDataModel object
 * @return the MetaDataModel for which getDescription() == metaDataName, or
 * null if not found.
 * @see MetaDataModel#setDescription
 * @see MetaDataModel#getDescription
 */
public MetaDataModel getMetaData(String metaDataName) {
    return dataModel.getMetaData(metaDataName);
}

/**
 * Clear the current path of all children of this MetaDataModel.
 * @param parentMeta the MetaDataModel whose children are to be
 * cleared.
 */
public void clearCurrentPath(MetaDataModel parentMeta) {
    dataModel.clearCurrentPath(parentMeta);
}

/////////////////////////////////////////////////////////////////////////////

/**
 * Retrieves the footer meta tree.
 *
 * @return The footer meta tree.
 */
public TreeModel getFooterMetaTree() {
    return(TreeModel) footerMetaTree;
}

/**
 * Retrieves the before details meta tree.
 *
 * @return The before details meta tree.
 */
public TreeModel getBeforeDetailsMetaTree() {
    return(TreeModel) beforeDetailsMetaTree;
}

/**
 * Retrieves the after details meta tree.
 *
 * @return The after details meta tree.
 */
public TreeModel getAfterDetailsMetaTree() {
    return(TreeModel) afterDetailsMetaTree;
}

/**
 * Create the footer meta tree with the same hierarchy as
 * the meta tree for the record type.
 */
protected void createFooterMetaTree() {
    footerMetaTree = createSummaryMetaTree(footerMetaTree, FOOTER);
}

/**
 * Create the before details meta tree with the same hierarchy as
 * the meta tree for the record type.
 */
protected void createBeforeDetailsMetaTree() {
    beforeDetailsMetaTree = createSummaryMetaTree(beforeDetailsMetaTree, BEFORE_DETAILS);
}

/**
 * Create the after details meta tree with the same hierarchy as
 * the meta tree for the record type.
 */
protected void createAfterDetailsMetaTree() {
    afterDetailsMetaTree = createSummaryMetaTree(afterDetailsMetaTree, AFTER_DETAILS);
}

/**
 * Create the summary meta trees with the same hierarchy as
 * the meta tree for the record type.
 * It assumes the record type meta data tree is already built.
 */
protected void createSummaryMetaTree() {
    createFooterMetaTree();
    createBeforeDetailsMetaTree();
    createAfterDetailsMetaTree();
}

/**
 * Create a summary meta tree with the same hierarchy as
 * the meta tree for the record type.
 *
 * @param summaryTree The summary meta tree.
 * @param summary The summary type.
 *
 * @return The new summary meta tree.
 */
protected DataSourceTreeModel createSummaryMetaTree(DataSourceTreeModel summaryTree, int summary) {
    // a new tree always
    if (summaryTree.getRoot() != null) {
        summaryTree = new DataSourceTreeModel(DataSourceTreeModel.NullRoot);
    }

    // get the record type meta tree
    TreeModel metaTree = getMetaDataTree();
    if (metaTree == null || metaTree.getRoot() == null) {
        return summaryTree;
    }

    // create a summary meta data tree following the same
    // hierarchy as the record type meta tree
    MetaDataModel rootMetaData = (MetaDataModel) metaTree.getRoot();
    MetaDataModel rootSummary = createSummaryMetaData(summary, rootMetaData.getMetaID());
    summaryTree.setRoot(rootSummary);
    createSummaryMetaTreeFromNode(summaryTree, rootMetaData, rootSummary, summary);

    return summaryTree;
}

/**
 * Recursive helper routine to create the summary meta tree.
 *
 * @param summaryTree The summary meta tree.
 * @param rootMetaData The record type root meta data.
 * @param rootSummary The summary root meta data.
 * @param summary The summary type.
 */
protected void createSummaryMetaTreeFromNode(DataSourceTreeModel summaryTree, MetaDataModel rootMetaData, MetaDataModel rootSummary, int summary) {
    TreeIterator i = ((DataSourceTreeNode) rootMetaData).getIterator();
    while (i.hasMoreElements()) {
        MetaDataModel metaData = (MetaDataModel)i.get();
        MetaDataModel summaryMeta = createSummaryMetaData(summary, metaData.getMetaID());
//FIX        summaryTree.append((DataSourceTreeNode) rootSummary, summaryMeta);
        ((DataSourceTreeNode)rootSummary).add((DataSourceTreeNode)summaryMeta);
        createSummaryMetaTreeFromNode(summaryTree, metaData, summaryMeta, summary);
        i.nextElement();
    }
}

/**
 * Create a new summary meta data object.
 *
 * @param summary The summary type.
 * @param metaID The meta id.
 *
 * @return The new summary meta data object.
 */
protected MetaDataModel createSummaryMetaData(int summary, int metaID) {
    MetaDataModel mdm = null;
    switch (summary) {
    case FOOTER:
        mdm = (MetaDataModel) new FooterMetaData();
        break;
    case BEFORE_DETAILS:
        mdm = (MetaDataModel) new DetailsMetaData();
        break;
    case AFTER_DETAILS:
    default:
        mdm = (MetaDataModel) new DetailsMetaData();
        break;
    }
    if (mdm != null) {
        ((SummaryMetaData)mdm).setMetaID(metaID);
    }
    return mdm;
}

/**
 * Set the root for the summary data model.
 *
 * @param grid The HiGrid instance.
 * @param recordData The data table model for the record type.
 */
protected void setSummaryDataRoot(HiGrid grid, DataTableModel recordData) {
    if (recordData == null) {
        return;
    }

    // create the root summary data table
    DataTableModel rootData = (DataTableModel) new SummaryDataTable((MetaDataModel) footerMetaTree.getRoot(),
                                                                    (MetaDataModel) beforeDetailsMetaTree.getRoot(),
                                                                    (MetaDataModel) afterDetailsMetaTree.getRoot());
    // update the tree
    HiGridInternalData internalData = (HiGridInternalData)recordData.getInternalData(grid);
    if (internalData == null) {
        internalData = new HiGridInternalData();
        recordData.setInternalData(grid, internalData);
    }
    internalData.setDataTable(rootData);
}

/**
 * Create the summary data table for the given data table model.
 *
 * @param grid The HiGrid instance.
 * @param recordData The data table model for the record type.
 * @param node The format node of the level.
 */
protected void createSummaryDataTable(HiGrid grid, DataTableModel recordData, FormatNode node) {
    if (recordData == null || node == null) {
        return;
    }

    // create the root summary data table
    DataTableModel summaryData = (DataTableModel) new SummaryDataTable((MetaDataModel) node.getFooterFormat().getMetaData(),
                                                                       (MetaDataModel) node.getBeforeDetailsFormat().getMetaData(),
                                                                       (MetaDataModel) node.getAfterDetailsFormat().getMetaData());
    // update the tree
    HiGridInternalData internalData = (HiGridInternalData)summaryData.getInternalData(grid);
    if (internalData == null) {
        internalData = new HiGridInternalData();
        recordData.setInternalData(grid, internalData);
    }
    internalData.setDataTable(summaryData);
}

/**
 * Reset the summary data tables of all cached data.
 *
 * @param grid The HiGrid instance.
 * @param clearCachedData True if the cached data are to be removed.
 */
protected void resetSummaryDataTables(HiGrid grid, boolean clearCachedData) {
    DataTableModel rootTable = (DataTableModel) getDataTableTree().getRoot();
    resetSummaryDataTables(grid, rootTable, clearCachedData);
}

/**
 * Helper method to reset the summary data tables.
 *
 * @param grid The HiGrid instance.
 * @param rootTable The root record table.
 * @param clearCachedData True if the cached data are to be removed.
 */
protected void resetSummaryDataTables(HiGrid grid, DataTableModel rootTable, boolean clearCachedData) {
    if (rootTable != null) {
        HiGridInternalData internalData = (HiGridInternalData)rootTable.getInternalData(grid);
        SummaryDataTable dataTable = null;
        if (internalData != null) {
            dataTable = (SummaryDataTable)internalData.getDataTable();
        }
        if (dataTable != null) {
            dataTable.resetSummaryDataTables(clearCachedData);
        }
        TreeIterator ti = ((DataSourceTreeNode)rootTable).getIterator();
        while (ti.hasMoreElements()) {
            DataTableModel recordTable = (DataTableModel)ti.get();
            resetSummaryDataTables(grid, recordTable, clearCachedData);
            ti.nextElement();
        }
    }
}

/**
 * Clears all user data.
 * Clears all Objects which were arbitrarily associated with
 * the entire DataTableTree.
 * @param user an Object, a reference to the user.
 */
public void clearDataTableTreeUserData(Object user) {
    dataModel.clearDataTableTreeUserData(user);
}

/**
 * For internal use only.
 * Clears all internal data.
 * Clears all Objects which were arbitrarily associated with
 * the entire DataTableTree.
 * @param user an Object, a reference to the user.
 */
public void clearDataTableTreeInternalData(Object user) {
    dataModel.clearDataTableTreeInternalData(user);
}

}
