/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: HiGridEvent.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:19 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.util.EventObject;

/**
 *  <code>HiGridEvent</code> is the base class for most HiGrid events.
 */

public abstract class HiGridEvent extends EventObject {

static final long serialVersionUID = -8001231998534705054L;

// Properties
protected HiGrid grid = null;
protected int eventType;
protected RowNode rowNode = null;
protected String column;
protected boolean cancelled = false;

/**
 * Constructor
 */
public HiGridEvent(HiGrid grid, int eventType, RowNode rowNode, String column) {
    super(grid);
    this.eventType = eventType;
    this.rowNode = rowNode;
    this.column = column;
    this.grid = grid;
}

/**
 * Returns a string indicating which column is affected, or "" in the case where
 * the column is not applicable.
 * @return the column name of the column
 */
public String getColumn() {
    return column == null ? "" : column;
}

/**
 * Retrieves the type of this event.
 *
 * @return The type of event.
 */
public int getEventType() {
    return eventType;
}

/**
 * Retrieves the RowNode associated with this event.
 *
 * @return The RowNode associated with this event.
 */
public RowNode getRowNode() {
    return rowNode;
}

/**
 * Cancels the proposed action.
 * Call this method to veto the pending action.
 * The default is to accept the change (false).
 * If isCancelable() returns false the cancel request is ignored.
 * @see #isCancelable
 * @see #getCancelled
 */
public void cancelProposedAction() {
    if (isCancelable()) {
        cancelled = true;
    }
}

/**
 * Indicates if proposed action was cancelled by a listener.
 * @return a boolean: true if cancelled.
 * @see #cancelProposedAction()
 */
public boolean getCancelled() {
    return cancelled;
}

/**
 * Return true if this event can be cancelled. Cancelling an event
 * cancels the operation which generated the event.
 * @return true if this event can be cancelled.
 */
public boolean isCancelable() {
    // subclasses should override
    return false;
}

}
