/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: HiGridFocusNew.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:19 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.awt.Component;
import java.awt.event.FocusEvent;
import com.klg.jclass.util.JCUtil;


public class HiGridFocusNew extends HiGridFocusAdapter {

// Semaphore used to prevent reentrance to the focusLost() method
public static boolean gridAreaFocusRequested = false;

// Is the current focus change temporary?
protected boolean temporaryFocusLost = false;

boolean focusTraversalKeysEnabledSave;

/**
 * Constructor
 */
public HiGridFocusNew() {
	super();
}

/**
 * Constructor
 */
public HiGridFocusNew(HiGrid grid) {
	super(grid);
}

public void focusGained(FocusEvent e) {
//	System.out.println("HiGridFocusNew.focusGained():  e=" + e);
	if (temporaryFocusLost) {
		temporaryFocusLost = false;
		return;
	}

	getGridArea().setFocusTraversalPolicy(
		new HiGridFocusTraversalPolicy(getGridArea(), e.getComponent()));
}

public void focusLost(FocusEvent e) {
//	System.out.println("HiGridFocusNew.focusLost():   e=" + e);
	// Ignore events if re-entrant or event is only a temporary loss of focus
	temporaryFocusLost = e.isTemporary();
	if (gridAreaFocusRequested || temporaryFocusLost) {
		return;
	}

	// Critical section of code - must be non-reentrant!
	gridAreaFocusRequested = true;

	Component eventComponent = e.getComponent();

	// Check if the new focus owner is outside of grid
	Component oppositeComponent = e.getOppositeComponent();
	boolean outsideGridArea = (getGridArea() != oppositeComponent &&
	                        !getGridArea().isAncestorOf(oppositeComponent));
	

	getGridArea().setFocusTraversalPolicy(
			new HiGridFocusTraversalPolicy(getGridArea(), null));

	// Remove EditTraverseHandlerNew as focus listener if it is no longer the
	// current edit component.  Ideally, this should be done in
	// setCellEditor(null), but it doesn't sequence the focus requests correctly
	// and therefore doesn't work.  This will remove the listener as late as
	// possible.
	// CRG - This focus event sequencing may actually work in JDK1.4 but we
	// still remove it here because we want the pre and post JDK mechanisms to
	// be as similar as possible.
	if (outsideGridArea) {
	    
		e.getComponent().removeFocusListener(this);
		e.getComponent().setFocusTraversalKeysEnabled(focusTraversalKeysEnabledSave);
	}

	// End of critical section of code.
	gridAreaFocusRequested = false;
	
}

}
