/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: HiGridPrintEvent.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:19 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.awt.Graphics;

import java.awt.print.PageFormat;

import java.util.EventObject;

/**
 * Event object for <code>HiGridPrintEvent</code>, a subclass of
 * <code>HiGridEvent</code> containing information about the particular
 * type of event.
 *
 * The event posted for each page during printing.
 */
public class HiGridPrintEvent extends EventObject {

static final long serialVersionUID = -7415088550638858357L;

/*
 * Print the header part of the page.
 */
public static final int PRINT_HEADER    = 2001;
/*
 * Print the footer part of the page.
 */
public static final int PRINT_FOOTER    = 2002;
/*
 * Print the end part of the last page.
 */
public static final int PRINT_END       = 2003;

protected int eventType;
protected PrintGrid printGrid;
protected Graphics gc;
protected int pageIndex;
protected int numPages;

protected PageFormat pageFormat;


/**
 * Default constructor
 */



public HiGridPrintEvent(HiGrid grid, int eventType, PrintGrid printGrid,
                        Graphics gc, PageFormat pageFormat, int pageIndex, int numPages) {

    super(grid);
    this.eventType = eventType;
    this.printGrid = printGrid;
    this.gc = gc;
    this.pageIndex = pageIndex;
    this.numPages = numPages;
}

/**
 * Retrieves the type of this event.
 *
 * @return The type of event.
 */
public int getEventType() {
    return eventType;
}

/**
 * Gets the print grid.
 */
public PrintGrid getPrintGrid() {
    return printGrid;
}

/**
 * Gets the graphics for the current page.
 */
public Graphics getGraphics() {
    return gc;
}

/**
 * Gets the page format.
 */

public PageFormat getPageFormat() {
    return pageFormat;
}


/**
 * Gets the page index.
 */
public int getPageIndex() {
    return pageIndex;
}

/**
 * Gets the number of pages.
 */
public int getNumPages() {
    return numPages;
}

}
