/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: HiGridRowSelectionEvent.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:19 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.util.EventObject;
import java.util.Vector;

/**
 * Event object for <code>HiGridRowSelectionEvent</code>, a subclass of
 * <code>HiGridEvent</code> containing information about the particular
 * type of event.
 */
public class HiGridRowSelectionEvent extends HiGridEvent {

static final long serialVersionUID = 7536446993729196705L;

/**
 * The row node is being selected/de-selected.  A listener to this event
 * can abort the action.
 */
public static final int BEFORE_SELECT_ROW = 1;
/**
 * The row node has been selected/de-selected.
 */
public static final int AFTER_SELECT_ROW = 2;

protected boolean select;
protected Vector selectedRows = null;

/**
 * Default constructor
 */
public HiGridRowSelectionEvent(HiGrid grid, int eventType, boolean select, Vector selectedRows) {
    super(grid, eventType, null, "");
    this.select = select;
    this.selectedRows = selectedRows;
}

/**
 * Return true if this event can be cancelled. Cancelling an event
 * cancels the operation which generated the event.
 * @return true if this event can be cancelled.
 */
public boolean isCancelable() {
    switch (getEventType()) {
    case BEFORE_SELECT_ROW:
        return true;
    }
    return false;
}

/**
 * Returns true if this is a Selection, false if a DeSelection
 */
public boolean getSelectState() {
    return this.select;
}

/**
 * Returns the Vector of newly selected rows.
 */
public Vector getNewRows() {
    return this.selectedRows;
}

/**
 * Returns the Vector of currently selected rows.
 */
public Vector getSelectedRows() {
    return grid.getController().getSelectedRows();
}

}
