/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: HiGridSortTableEvent.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:19 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.util.EventObject;

import com.klg.jclass.datasource.DataTableModel;

/**
 * Event object for <code>HiGridSortTableEvent</code>, a subclass of
 * <code>HiGridEvent</code> containing information about the particular
 * type of event.
 */
public class HiGridSortTableEvent extends HiGridEvent {

static final long serialVersionUID = 2815827159337485885L;

/**
 * The table is being sorted.  A listener to this event
 * can abort the action.
 */
public static final int BEFORE_SORT_TABLE = 1;
/**
 * The table has been sorted.
 */
public static final int AFTER_SORT_TABLE = 2;

/**
 * Default constructor
 */
public HiGridSortTableEvent(HiGrid grid, int eventType, RowNode rowNode,
                            String column) {
    super(grid, eventType, rowNode, column);
}

/**
 * Return true if this event can be cancelled. Cancelling an event
 * cancels the operation which generated the event.
 * @return true if this event can be cancelled.
 */
public boolean isCancelable() {
    switch (getEventType()) {
    case BEFORE_SORT_TABLE:
        return true;
    }
    return false;
}
/**
 * Gets the dataTable that is being sorted.
 *
 * @return the dataTable that is being sorted
 */
public DataTableModel getDataTable() {
    if (rowNode != null) {
        return rowNode.getDataTableModel();
    }
    else {
        return null;
    }
}

}
