/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: HiGridUI.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:20 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.awt.Color;
import java.awt.Insets;

import javax.swing.JComponent;
import javax.swing.UIManager;
import javax.swing.LookAndFeel;
import javax.swing.border.BevelBorder;
import javax.swing.border.SoftBevelBorder;
import javax.swing.plaf.ColorUIResource;
import javax.swing.plaf.ComponentUI;

/**
 * <CODE>HiGridUI</CODE> controls PLAF changes for HiGrid. Affected
 * properties are: header background, foreground, font and border; record
 * background, foreground, font and border; selection color.
 * Five special <CODE>CellStyleModel</CODE>
 * objects are modified for PLAF: <code>DefaultHeaderCellStyle</code>,
 * <code>DefaultRecordCellStyle</code>, <code>DefaultFooterCellStyle</code>,
 * <code>DefaultBeforeDetailsCellStyle</code> and
 * <code>DefaultAfterDetailsCellStyle</code>.
 * Modifying these two styles or basing another style on these, will cause
 * all PLAF changes to continue for properties that weren't 
 * modified. <CODE>HiGridUI</CODE> uses UIResource objects in order to
 * distinguish between PLAF-set properties (which change when the plaf 
 * changes) and user-set properties (which don't change).
 * @see HiGrid#getHeaderCellStyle
 * @see HiGrid#getRecordCellStyle
 * @see HiGrid#getFooterCellStyle
 * @see HiGrid#getBeforeDetailsCellStyle
 * @see HiGrid#getAfterDetailsCellStyle
 */
public class HiGridUI extends ComponentUI {

/**
 * Installs the default settings for the current look and feel.
 */
public void installUI(JComponent c) {
    c.setOpaque(true);
    LookAndFeel.installColorsAndFont(c,
                                     "Table.background",
                                     "Table.foreground",
                                     "Table.font");
    installDefaults(c);
}

/**
 * Changes the border, font and colors (background, foreground, focus and 
 * selection) for HiGrid given the current look and feel.
 */
protected void installDefaults(JComponent c) {
    HiGrid grid = (HiGrid)c;

    // get the current LookAndFeel
    String name = UIManager.getLookAndFeel().getName();

    // background
    Color background = grid.getBackground();
    if (background == null || background instanceof ColorUIResource) {
        grid.setBackground(new ColorUIResource(UIManager.getColor("Panel.background")));
    }

    // initialize default cellStyles
    grid.setHeaderCellStyle(new DefaultHeaderCellStyle());
    grid.setRecordCellStyle(new DefaultRecordCellStyle());
    grid.setFooterCellStyle(new DefaultFooterCellStyle());
    grid.setBeforeDetailsCellStyle(new DefaultBeforeDetailsCellStyle());
    grid.setAfterDetailsCellStyle(new DefaultAfterDetailsCellStyle());

    // reset the running grid
    grid.getFormatTree().resetPlaf();

// FIX this looks ugly; let's skip it for now
/*
    // focus cell
    grid.setEditorBorder(grid.getDefaultEditorBorder(
        new ColorUIResource(UIManager.getColor("Table.focusCellForeground")), 2, new Insets(2, 2, 2, 2)));
*/        
}

/**
 * Returns a <CODE>HiGridUI</CODE> instance.
 */
public static ComponentUI createUI(JComponent c) {
    return new HiGridUI();
}

public void uninstallUI(JComponent c) {
}

}
