/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: NodeCellFormat.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:20 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.awt.Insets;

import com.klg.jclass.cell.JCCellInfo;

/**
 * The <code>NodeStatusCellFormat</code> class manages certain
 * properties as class properties: BorderInsets, MarginInsets, width and showing
 */

public class NodeCellFormat extends IndicatorCellFormat {

static final long serialVersionUID = 7663113766506785548L;

/** NAME is the String name for all cells of type NodeCellFormat */
public static final String NAME = "NodeCell";

transient private HiGrid grid = null;
protected static boolean globalShowing = true;
protected static Insets globalBorderInsets = new Insets(2, 2, 2, 2);
protected static Insets globalMarginInsets = new Insets(2, 2, 2, 2);
protected static int globalWidth = HiGrid.INITIAL_NODE_WIDTH;

public NodeCellFormat(HiGrid grid) {
    this.grid = grid;
    if (grid != null) {
        copyFromCellStyle(grid.getHeaderCellStyle());
    }
    setHorizontalAlignment(JCCellInfo.CENTER);
    setVerticalAlignment(JCCellInfo.CENTER);
    setSelectedBackground(getBackground());
    setSelectedForeground(getForeground());
    setName(NAME);
}

/**
 * Retrieves the width of this object.
 *
 * @return The current width.
 */
public int getWidth() {
    if (grid == null) {
        return globalWidth;
    }
    return grid.getNodeWidth();
}

/**
 * Sets the width of this object.
 *
 * @param width The new width.
 */
public void setWidth(int width) {
    this.globalWidth = width;
    if (grid != null) {
        grid.setNodeWidth(globalWidth);
        drawingAreaValid = false;
        totalAreaValid = false;
        preferredTotalAreaValid = false;
    }
}

/**
 * Retrieves whether this object is showing.
 *
 * @return Whether object is currently showing.
 */
public boolean isShowing() {
    return globalShowing;
}

/**
 * Sets whether this object is showing.
 *
 * @param showing Whether object is to be shown.
 */
public void setShowing(boolean showing) {
    this.globalShowing = showing;
}

/**
 * Retrieves the border insets for this object.
 *
 * @return The current border insets.
 */
public Insets getBorderInsets() {
    return globalBorderInsets;
}

/**
 * Sets the border insets for this object.
 *
 * @param size The new border insets.
 */
public void setBorderInsets(Insets insets) {
    globalBorderInsets = insets;
    drawingAreaValid = false;
    totalAreaValid = false;
    preferredTotalAreaValid = false;
    translationOffsetValid = false;
}

/**
 * Retrieves the margin insets for this object.
 *
 * @return The current margin insets.
 */
public Insets getMarginInsets() {
    return globalMarginInsets;
}

/**
 * Sets the margin insets for this object.
 *
 * @param insets The new margin insets.
 */
public void setMarginInsets(Insets insets) {
    globalMarginInsets = insets;
    drawingAreaValid = false;
    totalAreaValid = false;
    preferredTotalAreaValid = false;
    translationOffsetValid = false;
}

}
