/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: NullCellEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:20 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.FocusManager;

import com.klg.jclass.cell.*;
import com.klg.jclass.cell.validate.*;
import com.klg.jclass.util.JCEnvironment;

/**
 * <CODE>NullEditor</CODE> provides a simple component
 * that only draws a focus-like rectangle around the current cell.
 * Its sole purpose is to indicate where an actual editor should be placed.
 */
public class NullCellEditor extends JComponent implements JCCellEditor,
	JCValidateInterface, KeyListener {

static final long serialVersionUID = 6001913893769434632L;

protected Object data;
protected JCCellEditorSupport support = new JCCellEditorSupport();
protected JCKeyModifier[] reservedKeys = null;
protected ValidateSupport validate_support = new ValidateSupport();
private boolean fNewFocusModel = false;

/**
 * Default constructor
 */
public NullCellEditor() {
	addKeyListener(this);
    boolean fNewFocusModel =
				(JCEnvironment.getJavaVersion() >= 140);
	if (fNewFocusModel) {
		setFocusTraversalKeysEnabled(false);
	}
	setDoubleBuffered(true);
    setOpaque(false);
/*
	if (reservedKeys == null) {
		reservedKeys = new JCKeyModifier[4];
		reservedKeys[0] = new JCKeyModifier(KeyEvent.VK_LEFT);
		reservedKeys[1] = new JCKeyModifier(KeyEvent.VK_RIGHT);
		reservedKeys[2] = new JCKeyModifier(KeyEvent.VK_HOME);
		reservedKeys[3] = new JCKeyModifier(KeyEvent.VK_END);
	}
*/
}


public void initialize(AWTEvent ev, JCCellInfo info,  Object o) {
    data = o;
}

/**
 * Returns the edit component.
 */
public Component getComponent() {
	return this;
}

/**
 * Returns the object to be commited.
 */
public Object getCellEditorValue() {
    return data;
}

/**
 * Returns true if the data has been modified.
 */
public boolean isModified() {
    return false;
}

protected boolean fireValidated(JCValidateEvent ev) {
	boolean validated = validate_support.fireValidateEvents(ev);
	if (validated) {
	    setVisible(true);
	}
	return validated;
}

/**
 * Called when the edit process is stopped
 */
public boolean stopCellEditing() {
    return true;
}

/**
 * Called when the edit process is cancelled.
 */
public void cancelCellEditing() {
}

public Dimension getPreferredSize(Graphics gc, JCCellInfo cellInfo, Object o) {
	return getPreferredSize();
}

/**
 * No keys are reserved.
 */
public JCKeyModifier[] getReservedKeys() {
	return reservedKeys;
}

/**
 * Add a listener to the list that's notified when the editor starts,
 * stops, or cancels editing.
 *
 * @param	l		the CellEditorListener
 */
public void addCellEditorListener(JCCellEditorListener l) {
	support.addCellEditorListener(l);
}

/**
 * Remove a listener from the list that's notified
 *
 * @param	l		the CellEditorListener
 */
public void removeCellEditorListener(JCCellEditorListener l) {
	support.removeCellEditorListener(l);
}


//
// In Swing, we have to disable the focus manager in order
// to get the key processing we want
//
public void processKeyEvent(KeyEvent e) {
	FocusManager fm = null;
	if (!fNewFocusModel) {
		fm = FocusManager.getCurrentManager();
		FocusManager.disableSwingFocusManager();
	}
	super.processKeyEvent(e);

	if (!fNewFocusModel) {
		fm.setCurrentManager(fm);
	}
}

public void keyPressed(KeyEvent e) {
/*
    int keyCode = e.getKeyCode();
	switch(keyCode) {
	case KeyEvent.VK_ENTER:
		support.fireStopEditing(new JCCellEditorEvent(e));
		break;
	case KeyEvent.VK_ESCAPE:
		support.fireCancelEditing(new JCCellEditorEvent(e));
		break;
	default:
	    break;
	}
*/
}

public void keyReleased(KeyEvent e) {
}

public void keyTyped(KeyEvent e) {
}


//
// ValidateInterface Implementation
//
/**
 * Add a new ValidateListener to the list.
 */
public void addValidateListener(JCValidateListener l) {
	validate_support.addValidateListener(l);
}

/**
 * Remove a ValidateListener from the list.
 */
public void removeValidateListener(JCValidateListener l) {
	validate_support.removeValidateListener(l);
}

public boolean isFocusTraversable() {
    return true;
}

}

