/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PopupMenuItem.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:20 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.io.Serializable;

/**
 * PopupMenuItem holds the info for a popup menu item.
 */
class PopupMenuItem implements Serializable {

static final long serialVersionUID = 3221770226077431939L;

// must not be in the same range as the default edit popup menu enums
// in EditPopupMenu
public final static int POPUP_MENU              = 1000;
public final static int POPUP_MENUITEM          = 1001;
public final static int POPUP_END_OF_MENUITEMS  = 1002;
public final static int POPUP_SEPARATOR         = 1003;

private int type;
private String command;
private String label;
private boolean visible;
private boolean enabled;

/**
 * Constructors
 */
public PopupMenuItem() {
    this(POPUP_SEPARATOR, "", "", true, true);
}

public PopupMenuItem(int type, String command, String label, boolean visible, boolean enabled) {
    this.type = type;
    if (command == null) {
        command = new String("");
    }
    this.command = new String(command);
    if (label == null) {
        label = new String("");
    }
    this.label = new String(label);
    this.visible = visible;
    this.enabled = enabled;
}

/**
 * Retrieves the menu item type.
 *
 * @return The menu item type.
 */
public int getType() {
    return type;
}

/**
 * Sets the menu item type.
 *
 * @param type The menu item type.
 */
public void setType(int type) {
    this.type = type;
}

/**
 * Retrieves the menu item command string.
 *
 * @return The menu item command string.
 */
public String getCommand() {
    return command;
}

/**
 * Sets the menu item command string.
 *
 * @param command The menu item command string.
 */
public void setCommand(String command) {
    if (command == null) {
        command = new String("");
    }
    this.command = command;
}

/**
 * Retrieves the menu item label string.
 *
 * @return The menu item label string.
 */
public String getLabel() {
    return label;
}

/**
 * Sets the menu item label string.
 *
 * @param label The menu item label string.
 */
public void setLabel(String label) {
    if (label == null) {
        label = new String("");
    }
    this.label = label;
}

/**
 * Retrieves the visible indicator for the menu item.
 *
 * @return The visibile indicator.
 */
public boolean isVisible() {
    return visible;
}

/**
 * Sets the visible indicator for the menu item.
 *
 * @param visible The visible indicator.
 */
public void setVisible(boolean visible) {
    this.visible = visible;
}

/**
 * Retrieves the enabled indicator for the menu item.
 *
 * @return The enabled indicator.
 */
public boolean isEnabled() {
    return enabled;
}

/**
 * Sets the enabled indicator for the menu item.
 *
 * @param enabled The enabled indicator.
 */
public void setEnabled(boolean enabled) {
    this.enabled = enabled;
}

}
