/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PopupMenuList.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:20 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.awt.event.ActionListener;
import java.io.Serializable;
import java.util.*;

import javax.swing.*;

/**
 * Implements the default popup menu item list.
 * A helper class to create an initial menu set.
 */

class PopupMenuList implements Serializable {

static final long serialVersionUID = 3328579489424610079L;

final static int MAX_MENU_LEVELS        = 3;
final static int MAX_MENU_LEVELS_INDEX  = 2;

private JPopupMenu popupMenu;
private ActionListener listener;

/**
 * Constructor
 */
PopupMenuList(JPopupMenu popupMenu, ActionListener listener) {
    this.popupMenu = popupMenu;
    this.listener = listener;
}

/**
 * Setup the default popup menu according to the list option.
 *
 * @param list Option for the short or long menu list.
 */
protected void setMenu(int list) {
    Vector menuList = getPopupMenuList(list);

    JPopupMenu menu[] = new JPopupMenu[MAX_MENU_LEVELS];
    boolean visible[] = new boolean[MAX_MENU_LEVELS];
    int level = 0;
    popupMenu.removeAll();
    popupMenu.removeNotify();
    menu[level] = popupMenu;
    visible[level] = true;
    for (Enumeration e = menuList.elements(); e.hasMoreElements();) {
        PopupMenuItem item = (PopupMenuItem) e.nextElement();
        if (item == null) {
            continue;
        }
        switch (item.getType()) {
        case PopupMenuItem.POPUP_MENU:
            int prevLevel = level;
            level++;
            level = Math.min(MAX_MENU_LEVELS_INDEX, level);
            visible[level] = item.isVisible();
            if (visible[level]) {
                JMenu m = new JMenu(item.getLabel());
                if (m != null) {
                    m.setActionCommand(item.getCommand());
                    if (!item.isEnabled()) {
                        m.setEnabled(false);
                    }
                    menu[prevLevel].add(m);
                    menu[level] = m.getPopupMenu();
                }
                else {
                    visible[level] = false;
                }
            }
            break;
        case PopupMenuItem.POPUP_MENUITEM:
            if (visible[level] && item.isVisible()) {
                JMenuItem mi = new JMenuItem(item.getLabel());
                if (mi != null) {
                    mi.setActionCommand(item.getCommand());
                    mi.addActionListener(listener);
                    if (!item.isEnabled()) {
                        mi.setEnabled(false);
                    }
                    menu[level].add(mi);
                }
            }
            break;
        case PopupMenuItem.POPUP_END_OF_MENUITEMS:
            level--;
            level = Math.max(0, level);
            break;
        case PopupMenuItem.POPUP_SEPARATOR:
            menu[level].add(new JSeparator());
            break;
        default:
            break;
        }
    }
}

/**
 * Returns the default popup menu list.
 *
 * @param list Option for the short or long menu list.
 * 
 * @return The popup menu list.
 */
public Vector getPopupMenuList(int list) {
    int menuPart1[] = new int[] {
        PopupMenuItem.POPUP_MENU, EditPopupMenu.POPUP_INSERT, 
        PopupMenuItem.POPUP_END_OF_MENUITEMS,
        PopupMenuItem.POPUP_MENU, EditPopupMenu.POPUP_APPEND, 
        PopupMenuItem.POPUP_END_OF_MENUITEMS,
        PopupMenuItem.POPUP_MENU, EditPopupMenu.POPUP_DELETE,
        PopupMenuItem.POPUP_MENUITEM, 
        EditPopupMenu.POPUP_DELETE_CURRENT,
        EditPopupMenu.POPUP_DELETE_SELECTED,
        PopupMenuItem.POPUP_END_OF_MENUITEMS,
        PopupMenuItem.POPUP_MENU, EditPopupMenu.POPUP_CANCEL,
        PopupMenuItem.POPUP_MENUITEM, 
        EditPopupMenu.POPUP_CANCEL_CURRENT,
        EditPopupMenu.POPUP_CANCEL_SELECTED,
        EditPopupMenu.POPUP_CANCEL_ALL,
        PopupMenuItem.POPUP_END_OF_MENUITEMS,
        PopupMenuItem.POPUP_MENU, EditPopupMenu.POPUP_REQUERY,
        PopupMenuItem.POPUP_MENUITEM, 
        EditPopupMenu.POPUP_REQUERY_RECORD,
        EditPopupMenu.POPUP_REQUERY_RECORD_AND_DETAILS,
        EditPopupMenu.POPUP_REQUERY_SELECTED,
        EditPopupMenu.POPUP_REQUERY_SELECTED_AND_DETAILS,
        EditPopupMenu.POPUP_REQUERY_TABLE,
        EditPopupMenu.POPUP_REQUERY_ALL,
        PopupMenuItem.POPUP_END_OF_MENUITEMS,
        PopupMenuItem.POPUP_MENU, EditPopupMenu.POPUP_UPDATE,
        PopupMenuItem.POPUP_MENUITEM, 
        EditPopupMenu.POPUP_UPDATE_CURRENT,
        EditPopupMenu.POPUP_UPDATE_SELECTED,
        EditPopupMenu.POPUP_UPDATE_ALL,
        PopupMenuItem.POPUP_END_OF_MENUITEMS,
        PopupMenuItem.POPUP_MENU, EditPopupMenu.POPUP_SELECT,
        PopupMenuItem.POPUP_MENUITEM, 
        EditPopupMenu.POPUP_SELECT_CURRENT,
        EditPopupMenu.POPUP_SELECT_ALL_IN_SAME_LEVEL,
        EditPopupMenu.POPUP_SELECT_ALL_IN_SAME_TABLE,
        EditPopupMenu.POPUP_SELECT_ALL,
        PopupMenuItem.POPUP_END_OF_MENUITEMS,
        PopupMenuItem.POPUP_SEPARATOR,
        PopupMenuItem.POPUP_MENU, EditPopupMenu.POPUP_PRINT,
        PopupMenuItem.POPUP_MENUITEM, 
        EditPopupMenu.POPUP_PRINT_AS_DISPLAYED,
        EditPopupMenu.POPUP_PRINT_AS_EXPANDED,
        EditPopupMenu.POPUP_PRINT_EXPAND_ALL,
        PopupMenuItem.POPUP_END_OF_MENUITEMS,
        PopupMenuItem.POPUP_MENU, EditPopupMenu.POPUP_PRINT_PREVIEW,
        PopupMenuItem.POPUP_MENUITEM, 
        EditPopupMenu.POPUP_PRINT_PREVIEW_AS_DISPLAYED,
        EditPopupMenu.POPUP_PRINT_PREVIEW_AS_EXPANDED,
        EditPopupMenu.POPUP_PRINT_PREVIEW_EXPAND_ALL,
        PopupMenuItem.POPUP_END_OF_MENUITEMS,
    };

    int menuPart2[] = new int[] {
        PopupMenuItem.POPUP_SEPARATOR,
        PopupMenuItem.POPUP_MENU, EditPopupMenu.POPUP_MOVE_TO_RECORD, 
        PopupMenuItem.POPUP_MENUITEM, 
        EditPopupMenu.POPUP_MOVE_TO_RECORD_FIRST,
        EditPopupMenu.POPUP_MOVE_TO_RECORD_PREVIOUS,
        EditPopupMenu.POPUP_MOVE_TO_RECORD_NEXT,
        EditPopupMenu.POPUP_MOVE_TO_RECORD_LAST,
        PopupMenuItem.POPUP_END_OF_MENUITEMS,
        PopupMenuItem.POPUP_MENU, EditPopupMenu.POPUP_MOVE_TO_TABLE_RECORD, 
        PopupMenuItem.POPUP_MENUITEM, 
        EditPopupMenu.POPUP_MOVE_TO_TABLE_RECORD_FIRST,
        EditPopupMenu.POPUP_MOVE_TO_TABLE_RECORD_PREVIOUS,
        EditPopupMenu.POPUP_MOVE_TO_TABLE_RECORD_NEXT,
        EditPopupMenu.POPUP_MOVE_TO_TABLE_RECORD_LAST,
        PopupMenuItem.POPUP_END_OF_MENUITEMS,
        PopupMenuItem.POPUP_MENUITEM, 
        EditPopupMenu.POPUP_MOVE_TO_PARENT,
        EditPopupMenu.POPUP_COLLAPSE_PARENT,
        PopupMenuItem.POPUP_MENU, EditPopupMenu.POPUP_TIPS,
        PopupMenuItem.POPUP_MENUITEM, 
        EditPopupMenu.POPUP_TIPS_KEYBOARD,
        EditPopupMenu.POPUP_TIPS_MOUSE,
        EditPopupMenu.POPUP_TIPS_MENU,
        PopupMenuItem.POPUP_END_OF_MENUITEMS,
    };

    Vector menuList = new Vector();
    boolean menuIndication = true;    
    int menuLength = menuPart1.length;
    if (list == EditPopupMenu.DEFAULT_LONG_POPUPMENU_LIST) {
        menuLength += menuPart2.length;
    }
    // create the popup menu list (part 1 - the short version)
    for (int i = 0; i < menuLength; i++) {
        int menuitem = (i < menuPart1.length) ? menuPart1[i] : menuPart2[i-menuPart1.length];
        switch (menuitem) {
        case PopupMenuItem.POPUP_MENU:
            menuIndication = true;
            break;
        case PopupMenuItem.POPUP_MENUITEM:
            menuIndication = false;
            break;
        case PopupMenuItem.POPUP_END_OF_MENUITEMS:
            menuList.addElement((Object) new PopupMenuItem(PopupMenuItem.POPUP_END_OF_MENUITEMS,
                                                           null, null, true, true));
            break;
        case PopupMenuItem.POPUP_SEPARATOR:
            menuList.addElement((Object) new PopupMenuItem());
            break;
        default:
            String command = ((EditPopupMenu)popupMenu).getCommand(menuitem);
            if (command != null) {
                int type = menuIndication ? PopupMenuItem.POPUP_MENU : PopupMenuItem.POPUP_MENUITEM;
                menuList.addElement((Object) new PopupMenuItem(type,
                                                               command, 
                                                               LocaleBundle.string(command),
                                                               true, true));
            }
            break;
        }
    }

    // always end with a POPUP_END_OF_MENUITEMS item
    menuList.addElement((Object) new PopupMenuItem(PopupMenuItem.POPUP_END_OF_MENUITEMS,
                                                   null, null, true, true));
    return menuList;
}

}
