/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PrePrintWalk.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:20 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;


import java.awt.print.PageFormat;

import java.util.Vector;

/**
 * The PrePrintWalk class performs the calculation for printing the rows.
 */

class PrePrintWalk implements Walkable {

static final long serialVersionUID = 2182520562385955747L;

private PrintGrid printGrid;
private HiGrid grid;
private int rowPageCount;
private int printableHeight;
private int printableWidth;
private int height;
private int width;
private Vector rowNodeList;
private boolean stop;




public PrePrintWalk(PrintGrid12 printGrid, HiGrid grid, PageFormat pageFormat) {

    this.printGrid = printGrid;
    this.grid = grid;
    height = 0;
    width = 0;
    rowPageCount = 0;
    rowNodeList = null;
    if (grid == null || printGrid == null) {
        stop = true;
        return;
    }



    printableHeight = (int)pageFormat.getImageableHeight();
    printableWidth = (int)pageFormat.getImageableWidth();

    stop = false;
}

public boolean getDownward() {
    return true;
}

public boolean done() {
    return stop;
}

public void next(RowNode rowNode) {
    width = Math.max(width, rowNode.getRowFormat().getTotalPrintWidth(grid, rowNode.getFormatNode(), printableWidth));
    int rowHeight = rowNode.getHeight();
    if ((height + rowHeight) > printableHeight) {
        rowPageCount++;
        height = rowHeight;
        if (rowNodeList != null) {
            rowNodeList.addElement((Object) rowNode);
        }
    }
    else {
        height += rowHeight;
    }
    if (rowNodeList == null) {
        rowNodeList = new Vector(10);
        rowNodeList.addElement((Object) rowNode);
    }
}

public void previous(RowNode rowNode) {
}

public boolean find(RowNode rowNode) {
    return stop;
}

/**
 * Determine the number of pages a row needs.
 */
public int getColumnPages() {
    int pages = 0;
    if (printableWidth != 0) {
        pages = width / printableWidth;
        if ((width % printableWidth) > 0) {
            pages++;
        }
    }
    return pages;
}

/**
 * Determine the number of pages the set of rows needs.
 */
public int getRowPages() {
    int pages = rowPageCount;
    if (height > 0) {
        pages++;
    }
    return pages;
}

/**
 * Determine the total number of pages.
 */
public int getPrintPages() {
    return getRowPages() * getColumnPages();
}

/**
 * Returns the list of top row nodes for each print page.
 */
public Vector getRowNodeList() {
    return rowNodeList;
}

}
