/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PrintPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:20 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.awt.*;

import javax.swing.*;

/**
 * PrintPage displays a print page for Print Preview usage.
 */
class PrintPage extends JPanel {

static final long serialVersionUID = 947507678212036162L;

protected transient Image page;
protected Dimension preferredDimension;

/**
 * Constructs a Dimension and initializes it to the specified preferred
 * dimension. 
 */
PrintPage(Dimension d) {
    preferredDimension = d;
}

/**
 * Sets the page image.
 */
public void setImage(Image page) {
    if (this.page != null) {
        this.page.flush();
    }
    this.page = page;
    repaint();
}

/**
 * Gets the preferred size of the page.
 */
public Dimension getPreferredSize() {
    return preferredDimension;
}

/**
 * Paints the table image centered on the screen.
 */
public void paintComponent(Graphics gc) {
    Dimension d = this.getSize();

    gc.setColor(Color.gray);
    gc.fillRect(0, 0, d.width, d.height);

    if (page != null) {
        Dimension di = new Dimension(page.getWidth(this), page.getHeight(this));
        Point p = new Point((d.width - di.width)/2,(d.height - di.height)/2);
        gc.setColor(Color.black);
        gc.drawRect(p.x - 2, p.y - 2, di.width + 2, di.height + 2);
        gc.setColor(Color.white);
        gc.fillRect(p.x - 1, p.y - 1, di.width + 1, di.height + 1);
        gc.drawImage(page, p.x, p.y, this);
    }
}   

protected void finalize() throws Throwable {
    if (this.page != null) {
        this.page.flush();
    }
}

}
