/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PrintPreview.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:20 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.awt.*;
import java.awt.event.*;



import java.awt.print.*;


import javax.swing.*;

/**
 * The PrintPreview is a frame to display print pages. It allows
 * users to flip through the pages and send selected pages or
 * all the pages to the printer.
 */
public class PrintPreview extends JDialog implements WindowListener,
ActionListener {

static final long serialVersionUID = -3450555077594570366L;

transient protected JButton firstButton;
transient protected JButton prevButton;
transient protected JButton nextButton;
transient protected JButton lastButton;
transient protected JButton printButton;
transient protected JButton printAllButton;
transient protected JButton pageFormatButton;
transient protected JButton closeButton;
transient protected JLabel status;
transient protected JScrollPane pane;




transient protected PrinterJob job;
transient protected PageFormat pageFormat;
transient protected PrintGrid12 printGrid;

transient protected HiGrid grid;
transient protected PrintPage printPage;
transient protected int currentPage = 0;

/**
 * Gets the number of pages in this job.
 *
 * @return The number of pages in this job.
 */
public int getNumPages() {
    return printGrid.getTotalNumPages();
}    

/**
 * Gets the width of a page.
 *
 * @return The page width.
 */
public int getPageWidth() {
    return printGrid.getPageWidth();
}

/**
 * Gets the height of a page.
 *
 * @return The page height.
 */
public int getPageHeight() {
    return printGrid.getPageHeight();
}


/**
 * Creates a new PrintPreview frame.
 * @param owner the frame that will own this dialog.
 * @param title the title of this dialog.
 * @param grid the HiGrid parent instance.
 */
public PrintPreview(Frame owner, String title, HiGrid grid) {
    super(owner, title);
    setModal(true);
    this.grid = grid;




    job = PrinterJob.getPrinterJob();

    if (job == null) {
        return;
    }

    printGrid = grid.getPrintGrid();

    grid.setPrinting(true);

    pageFormat = job.defaultPage();
    printGrid.initialize(pageFormat);




    //FIX localize this!!!

    job.setJobName("HiGrid Print");
    job.setPrintable(printGrid, pageFormat);
// this allows correct number of pages to be computed
    job.setPageable(printGrid);


    getContentPane().setLayout(new BorderLayout());
    setBackground(Color.lightGray);

    JPanel buttons = new JPanel();
    buttons.setLayout(new FlowLayout(FlowLayout.LEFT,1,1));
    buttons.add(firstButton = new JButton(LocaleBundle.string(LocaleBundle.printPreviewFirst)));
    firstButton.addActionListener(this);
    buttons.add(prevButton = new JButton(LocaleBundle.string(LocaleBundle.printPreviewPrevious)));
    prevButton.addActionListener(this);
    buttons.add(nextButton = new JButton(LocaleBundle.string(LocaleBundle.printPreviewNext)));
    nextButton.addActionListener(this);
    buttons.add(lastButton = new JButton(LocaleBundle.string(LocaleBundle.printPreviewLast)));
    lastButton.addActionListener(this);
    buttons.add(printButton = new JButton(LocaleBundle.string(LocaleBundle.printPreviewPrint)));
    printButton.addActionListener(this);
    buttons.add(printAllButton = new JButton(LocaleBundle.string(LocaleBundle.printPreviewPrintAll)));
    printAllButton.addActionListener(this);

    buttons.add(pageFormatButton = new JButton(LocaleBundle.string(LocaleBundle.printPreviewPageFormat)));
    pageFormatButton.addActionListener(this);

    buttons.add(closeButton = new JButton(LocaleBundle.string(LocaleBundle.printPreviewClose)));
    closeButton.addActionListener(this);
    pane = new JScrollPane();
    createPage();
    getContentPane().add(buttons, BorderLayout.NORTH);
    getContentPane().add(pane, BorderLayout.CENTER);
    getContentPane().add(status = new JLabel("", JLabel.LEFT), BorderLayout.SOUTH);

    addWindowListener(this);

    setSize(650,500);
    showPage(0);
}

/**
 * Creates an image of the page to preview.
 */
public void createPage() {
    if (printPage != null) {
        pane.remove(printPage);
    }
    printPage = new PrintPage(new Dimension(getPageWidth(), getPageHeight()));
    pane.add(printPage);
    pane.setViewportView(printPage);
    printPage.setSize(printPage.getPreferredSize());
    pane.setSize(new Dimension((int)(getPageWidth() * 1.05), (int)(getPageHeight() * 1.05)));
    getContentPane().doLayout();
}

/**
 * Shows the first page of the set.
 */
public void setVisible(boolean show) {
    if (show) {
        if (job == null || printGrid == null) {
            grid.setPrinting(false);
        }
        else {
            showPage(0);
        }
    }
    super.setVisible(show);
}

/**
 * Generates a new HiGrid page image and posts it to the
 * PrintPage panel for display.
 */
public void showPage(int page) {
    if (job == null) {
        grid.setPrinting(false);
        return;
    }

    currentPage = page;
    Image image = grid.getGridArea().createImage(getPageWidth(), getPageHeight());
    Graphics gc = image.getGraphics();
    printGrid.print(gc,

                    printGrid.getPageFormat(), 

                    page);
    printPage.setImage(image);
    gc.dispose();

    status.setText(" " + LocaleBundle.string(LocaleBundle.printTextPage) + " " + (currentPage+1) + " " + 
                   LocaleBundle.string(LocaleBundle.printTextOf) + " " + getNumPages() + ".");
}

/**
 * Cleanup before closing PrintPreview window.
 */
public void close() {
    grid.setPrinting(false);


    setVisible(false);
    dispose();
}

/**
 * ActionListener method.
 */
public void actionPerformed(ActionEvent e) {
    if (e.getSource() instanceof JButton) {
        JButton button = (JButton) e.getSource();
        if (button.equals(firstButton)) {
            showPage(0);
        }
        else if (button.equals(prevButton)) {
            if (currentPage > 0) {
                showPage(currentPage - 1);
            }
        }
        else if (button.equals(nextButton)) {
            if (currentPage < getNumPages()-1) {
                showPage(currentPage + 1);
            }
        }
        else if (button.equals(lastButton)) {
            showPage(getNumPages()-1);
        }
        else if (button.equals(printButton)) {
            if (job != null) {



            try {
                // allow user to select which pages for printing
                if (job.printDialog()) {
                    job.print();
                }
            }
            catch (PrinterException xe) {
                System.out.println(xe);
            }

            }
        }
        else if (button.equals(printAllButton)) {
            if (job != null) {



            try {
                job.print();
            }
            catch (PrinterException pe) {
                System.out.println(pe);
            }

            }
        }
        else if (button.equals(pageFormatButton)) {

            pageFormat = job.pageDialog(pageFormat);
            job.setPrintable(printGrid, pageFormat);
            job.setPageable(printGrid);
            printGrid.initialize(pageFormat);
            createPage();
            showPage(0);

        }
        else if (button.equals(closeButton)) {
            close();
        }
    }
}

/**
 * WindowListener methods.
 */
public void windowClosing(WindowEvent e) {
    close();
}
public void windowActivated(WindowEvent e) {
}
public void windowClosed(WindowEvent e) {
}
public void windowDeactivated(WindowEvent e) {
}
public void windowDeiconified(WindowEvent e) {
}
public void windowIconified(WindowEvent e) {
}
public void windowOpened(WindowEvent e) {
}

}
