/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PrintWalk.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:20 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.awt.Graphics;

import java.awt.print.PageFormat;


/**
 * The PrintWalk class is used to print the rows.
 */

class PrintWalk implements Walkable {

static final long serialVersionUID = 320638883999410839L;

private PrintGrid printGrid;
private HiGrid grid;
private Graphics gc;
private int printableHeight;
private int printableWidth;
private int height;
private int currentPage;
private RowNode lastRowNode;
private boolean firstRowDrawn;
private boolean stop;




public PrintWalk(PrintGrid12 printGrid, HiGrid grid, PageFormat pageFormat) {

    this.printGrid = printGrid;
    this.grid = grid;
    height = 0;
    currentPage = 0;
    lastRowNode = null;
    firstRowDrawn = false;
    if (grid == null || printGrid == null) {
        stop = true;
        return;
    }



    printableHeight = (int)pageFormat.getImageableHeight();
    printableWidth = (int)pageFormat.getImageableWidth();

    stop = false;
}

public boolean getDownward() {
    return true;
}

public boolean done() {
    return stop;
}

public void next(RowNode rowNode) {
    int rowHeight = rowNode.getHeight();
    if (firstRowDrawn && (height + rowHeight) > printableHeight) {
        lastRowNode = rowNode;
        stop = true;
    }
    else {
        height += rowHeight;
        rowNode.draw(gc, grid);
        firstRowDrawn = true;
    }
}

public void previous(RowNode rowNode) {
}

public boolean find(RowNode rowNode) {
    return stop;
}

/**
 * Initialization for each page.
 *
 * @param gc The graphics context.
 * @param page The page number
 */
protected void init(Graphics gc, int page) {
    this.gc = gc;
    currentPage = page;
    lastRowNode = null;
    firstRowDrawn = false;
    height = 0;
    stop = false;
}

/**
 * Returns the last row node.
 *
 * @return The last row node.
 */
public RowNode getLastRowNode() {
    return lastRowNode;
}

/**
 * Returns the height value.
 *
 * @return The height value.
 */
public int getHeight() {
    return height;
}

}
