/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: SortData.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:20 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.io.Serializable;

/**
 * <CODE>SortData</CODE> holds the sorting info for a column.
 */
public class SortData implements Serializable {

static final long serialVersionUID = 8860060724529410315L;

protected String columnName;
protected int direction;
protected int previousDirection;

/**
 * Constructors
 */
public SortData(String columnName, int direction) {
    this.columnName = columnName;
    this.direction = direction;
    this.previousDirection = SortGrid.UNKNOWN;
}

public SortData(String columnName) {
    this.columnName = columnName;
    direction = SortGrid.ASCENDING;
}

public SortData(SortData sortData) {
    this.columnName = sortData.columnName;
    this.direction = sortData.direction;
}

/**
 * Compare the SortData to determine equality.
 *
 * @param sortData The SortData to compare with.
 * @return True if equal.
 */
public boolean equals(SortData sortData) {
    if (columnName.equals(sortData.columnName) &&
        direction == sortData.direction) {
        return true;
    }
    return false;
}

/**
 * Retrieves the column name.
 */
public String getColumnName() {
    return columnName;
}

/**
 * Retrieves the direction value.
 */
public int getDirection() {
    return direction;
}

/**
 * Retrieves the previous direction value.
 */
public int getPreviousDirection() {
    return previousDirection;
}

/**
 * Set direction.
 */
protected void setDirection(int direction) {
	previousDirection = this.direction;
	this.direction = direction;
}

/**
 * Get direction to the next stage.
 */
protected int getNextDirection() {
    if (direction == SortGrid.ASCENDING) {
        return SortGrid.DESCENDING;
    }
    else {
        return SortGrid.ASCENDING;
    }
}

/**
 * Set direction to the next stage.
 */
protected void setNextDirection() {
    setDirection(getNextDirection());
}

}
