/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: SortLabelCellRenderer.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:20 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.awt.*;

import com.klg.jclass.cell.*;
import com.klg.jclass.cell.renderers.JCStringCellRenderer;
import com.klg.jclass.util.swing.JCIconCreator;

/**
 * A cell renderer to draw a label with the ascending/descending
 * icon indication.
 */
public class SortLabelCellRenderer extends JCStringCellRenderer implements
    HiGridHeaderRenderer {

static final long serialVersionUID = -8549022746709463592L;

public static final int ICON_ARROW = 1;
public static final int ICON_TRIANGLE = 2;
public static final int ICON_NONE = 3;

/**
 * Sets the ascending and descending sorting images to one of the predefined ICON_* types.
 *
 * @param type the image type to use (one of the ICON_* types)
 */
public static void setImages(int type) {
    switch (type) {
    case ICON_TRIANGLE:
        // override setting of color for this image
        sortColor = Color.white;
        lightColor = Color.white;
        darkColor = Color.black;
        makeImage(type);
        break;
    default:
        setImages(type, DEFAULT_SORT_COLOR);
        break;
    }
}

/**
 * Sets the ascending and descending sorting images to one of the predefined ICON_* types.
 *
 * @param type the image type to use (one of the ICON_* types)
 * @param color the color to use within the image
 */
public static void setImages(int type, Color color) {
    setSortColor(color);
    makeImage(type);
}

/**
 * Retrieves the sort label ascending image.
 */
public static Image getAscendingImage() {
    return ascendingImage;
}

/**
 * Sets the sort label ascending image.
 *
 * @param image the image to use
 */
public static void setAscendingImage(Image image) {
    ascendingImage = image;
}

/**
 * Retrieves the sort label descending image.
 */
public static Image getDescendingImage() {
    return descendingImage;
}

/**
 * Sets the sort label descending image.
 *
 * @param image the image to use
 */
public static void setDescendingImage(Image image) {
    descendingImage = image;
}

/**
 * Swaps the ascending and descending images.
 */
public static void swapImages() {
    Image image = ascendingImage;
    ascendingImage = descendingImage;
    descendingImage = image;
}

protected final static String[] arrowDescendingPixels = {
    ".............",
    ".............",
    "...ssss......",
    "..s...ss.....",
    ".s....sss....",
    ".s.... ss....",
    ".s.....sss...",
    ".......sss...",
    "....sssssssss",
    ".....sssssss.",
    "......sssss..",
    ".......sss...",
    "........s....",
};

protected final static String[] arrowAscendingPixels = {
    ".............",
    ".............",
    "........s....",
    ".......sss...",
    "......sssss..",
    ".....sssssss.",
    "....sssssssss",
    ".......sss...",
    ".s.....sss...",
    ".s.... ss....",
    ".s....sss....",
    "..s...ss.....",
    "...ssss......",
};

protected final static String[] triangleAscendingPixels = {
    ".............",
    ".............",
    ".............",
    ".............",
    ".............",
    "......d......",
    ".....d.l.....",
    "....d...l....",
    "...d.....l...",
    "..dllllllll..",
    ".............",
    ".............",
    "............."
};

protected final static String[] triangleDescendingPixels = {
    ".............",
    ".............",
    ".............",
    ".............",
    ".............",
    "..ddddddddd..",
    "...d.....l...",
    "....d...l....",
    ".....d.l.....",
    "......l......",
    ".............",
    ".............",
    "............."
};

protected final static String[] noneAscendingPixels = {
    "",
};

protected final static String[] noneDescendingPixels = {
    "",
};

protected static transient JCIconCreator creator = null;
protected static transient Image ascendingImage = null;
protected static transient Image descendingImage = null;


protected static int CLIP_ARROW_SPACING = 2;
protected static Color DEFAULT_SORT_COLOR = Color.blue;
protected static Color sortColor = null;
protected static Color lightColor = null;
protected static Color darkColor = null;

/* Constructor */
public SortLabelCellRenderer() {
    setImages(ICON_NONE);
    setImages(ICON_TRIANGLE);
    setImages(ICON_ARROW);
}

/**
 * Sets up the various sorting colors used in the icons
 *
 * @param color the basic color of the image
 */
protected static void setSortColor(Color color) {
    if (color == null) {
        color = DEFAULT_SORT_COLOR;
    }
    sortColor = color;
    lightColor = brighter(sortColor);
    darkColor = darker(sortColor);
}


/**
 * Creates the actual images from the passed String arrays.
 *
 * @param ascending the String representation of the ascending sort icon
 * @param descending the String representation of the descending sort icon
 */
protected static void makeImages(String[] ascending, String[] descending) {
    creator.setSize(ascending.length, ascending.length);
    creator.setPixels(ascending);
    ascendingImage = creator.getIcon().getImage();
    creator.clear();

    creator.setSize(descending.length, descending.length);
    creator.setPixels(descending);
    descendingImage = creator.getIcon().getImage();
}

/**
 * Sets the ascending and descending sorting images to one of the predefined ICON_* types.
 *
 * @param type the image type to use (one of the ICON_* types)
 */
protected static void makeImage(int type) {
    if (creator == null) {
        creator = new JCIconCreator();
    }
    creator.setColor('s', sortColor);
    creator.setColor('l', lightColor);
    creator.setColor('d', darkColor);
    switch (type) {
    case ICON_TRIANGLE:
        makeImages(triangleAscendingPixels, triangleDescendingPixels);
        break;
    case ICON_NONE:
        makeImages(noneAscendingPixels, noneDescendingPixels);
        break;
    default:
    case ICON_ARROW:
        makeImages(arrowAscendingPixels, arrowDescendingPixels);
        break;
    }
}

//////////////////////////////////////
// HiGridHeaderRenderer implementation

/**
 * Used to draw the cell. Note that the size of the cell comes from
 * <CODE>JCCellInfo.getDrawingArea()</CODE>.  When the renderer's draw method is
 * called, the gc is translated to the upper-left corner of the drawing area
 * for the cell.  The "drawing area" of the cell is the region <EM>inside</EM>
 * the borders and the margins.
 *
 * @param gc Graphics context to use for drawing.
 * @param cellInfo Cell property information
 * @param o Object to be rendered
 * @param selected Boolean value indicating whether the cell is selected
 */
public void draw(Graphics gc, JCCellInfo cellInfo, Object o, boolean selected) {
    super.draw(gc, cellInfo, ((SortLabel)o).getLabel(), selected);
    int direction = ((SortLabel)o).getDirection();
    if (direction != SortGrid.UNKNOWN) {
        Image imageToDraw = (direction == SortGrid.ASCENDING) ? ascendingImage : descendingImage;
        Rectangle cellSize = cellInfo.getDrawingArea();
        int align = cellInfo.getHorizontalAlignment();
        int x = 0;
        if (align == JCCellInfo.CENTER || align == JCCellInfo.LEFT) {
            x = cellSize.width - imageToDraw.getWidth(null) - CLIP_ARROW_SPACING;
        }
        int y = 0;
        gc.setColor(cellInfo.getBackground());
        gc.fillRect(x, y, imageToDraw.getWidth(null), imageToDraw.getHeight(null));
        gc.drawImage(imageToDraw, x, y, cellInfo.getBackground(), null);
        Dimension ps = getPreferredSize(gc, cellInfo, o);
        if ((ps.width > cellSize.width || ps.height > cellSize.height) &&
            cellInfo.getClipHints() != JCCellInfo.SHOW_NONE) {
            gc.setColor(Color.black);
            utilities.drawClipArrows(gc, cellInfo, ps,
                                     Utilities.DEFAULT_CLIP_ARROW_SIZE,
                                     drawClipArrowOutline);
        }
    }
}

/**
 * Returns the preferred size of the cell's drawing area.  The "drawing area"
 * of the cell is the region <EM>inside</EM> the borders and the margins.
 *
 * @param gc Graphics context for calculating size
 * @param cellInfo Cell property information
 * @param o Object to be rendered
 * @return java.awt.Dimension instance containing preferrred height and
 * width.
 */
public Dimension getPreferredSize(Graphics gc, JCCellInfo cellInfo, Object o) {
    if (o != null && o instanceof SortLabel) {
        Dimension dim = super.getPreferredSize(gc, cellInfo, ((SortLabel)o).getLabel());
        int width = ascendingImage.getWidth(null);
        if (width > 0) {
            dim.width += width + CLIP_ARROW_SPACING;
        }
        return dim;
    }
    else {
        return super.getPreferredSize(gc, cellInfo, o);
    }
}

///////////////////////////////////
// Brighter/darker implementation
private static final Color WHITE_BRIGHTER = new Color(200, 200, 200);
private static final Color WHITE_DARKER = new Color(140, 140, 140);
private static final Color BLACK_BRIGHTER = new Color(125, 125, 125);
private static final Color BLACK_DARKER = new Color(75, 75, 75);
private static final double FACTOR = 0.5;

/**
 * Calculates the color even when it is saturated, for example,
 * when it is black or white.
 */
protected static Color brighter(Color color) {
    if (color.equals(Color.white))
        return WHITE_BRIGHTER;
    else if (color.equals(Color.black))
        return BLACK_BRIGHTER;
    else {
        // instead of calling Color.brighter() I do my own calculation
        // for a lighter color.  This calc brightens up dark colors very
        // quickly while lightening up bright colors only a bit.
        int red = color.getRed();
        red += (int) ((255 - red) * FACTOR);
        int blue = color.getBlue();
        blue += (int) ((255 - blue) * FACTOR);
        int green = color.getGreen();
        green += (int) ((255 - green) * FACTOR);

        return new Color(Math.min(red, 255),
                         Math.min(green, 255),
                         Math.min(blue, 255));
    }
}

/**
 * Calculates the color even when it is saturated, for example,
 * when it is black or white.
 */
protected static Color darker(Color color) {
    if (color.equals(Color.white))
        return WHITE_DARKER;
    else if (color.equals(Color.black))
        return BLACK_DARKER;
    else
        return color.darker();
}

}
