/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: SummaryData.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:20 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.io.Serializable;
import java.math.BigDecimal;

import com.klg.jclass.datasource.MetaDataModel;

/**
 * The cached data for a summary column.
 */

class SummaryData implements Serializable {

static final long serialVersionUID = 3367486869277639193L;

private boolean useCachedValue;
private Object value;

/**
 * SummaryData Constructor.
 *
 * @param value The summary data.
 */
public SummaryData(Object value) {
    this.value = value;
    useCachedValue = true;
}

/**
 * Retrieves use cached value indication.
 *
 * @return The use cached value indication.
 */
protected boolean getUseCachedValue() {
    return useCachedValue;
}

/**
 * Sets use cached value indication.
 *
 * @param The use cached value indication.
 */
protected void setUseCachedValue(boolean useCachedValue) {
    this.useCachedValue = useCachedValue;
}

/**
 * Retrieves the cached value.
 *
 * @return The cached value.
 */
protected Object getValue() {
    return value;
}

/**
 * Sets the cached value.
 *
 * @param The cached value.
 */
protected void setValue(Object value) {
    this.value = value;
}

/**
 * Determines if the given value is the same as the one stored.
 *
 * @param object The given value as an object.
 * @param type The data type of the value.
 *
 * @return True if the values are the same.
 */
public boolean equals(Object object, int type) {
    switch (type) {
    case MetaDataModel.TYPE_DOUBLE:
        return(((Double)object).doubleValue() == ((Double)value).doubleValue());
    case MetaDataModel.TYPE_FLOAT:
        return(((Float)object).floatValue() == ((Float)value).floatValue());
    case MetaDataModel.TYPE_BIG_DECIMAL:
        return(((BigDecimal)object).doubleValue() == ((BigDecimal)value).doubleValue());
    case MetaDataModel.TYPE_LONG:
        return(((Long)object).longValue() == ((Long)value).longValue());
    case MetaDataModel.TYPE_INTEGER:
        return(((Integer)object).intValue() == ((Integer)value).intValue());
    case MetaDataModel.TYPE_BYTE:
        return(((Byte)object).intValue() == ((Byte)value).intValue());
    case MetaDataModel.TYPE_SHORT:
        return(((Short)object).intValue() == ((Short)value).intValue());
    case MetaDataModel.TYPE_STRING:
        return(((String)object).compareTo((String)object) == 0);
    default:        
        break;
    }
    return false;
}

}
