/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: SummaryRefreshWalk.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:20 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.util.*;

import javax.swing.tree.*;

import com.klg.jclass.datasource.DataTableModel;

/**
 * The SummaryRefreshWalk class is used to refresh the summary rows
 * in the visible grid area.
 */
class SummaryRefreshWalk implements Walkable {

static final long serialVersionUID = 3128056133926266838L;

private HiGrid grid;
private int height;
private boolean found;

/**
 * Constructor
 */
public SummaryRefreshWalk(HiGrid grid, int height) {
    this.grid = grid;
    this.height = height;
    found = false;
}

public boolean getDownward() {
    return true;
}

public boolean done() {
    return found;
}

public void next(RowNode rowNode) {
    if (rowNode != null && height >= 0) {
        height -= rowNode.getHeight();
        refreshSummaryRow(rowNode);
    }
}

public void previous(RowNode rowNode) {
}

public boolean find(RowNode rowNode) {
    if (height < 0) {
        found = true;
    }
    return found;
}

private void refreshSummaryRow(RowNode rowNode) {
    RowFormat rowFormat = rowNode.getRowFormat();
    if (rowFormat instanceof FooterFormat ||
        rowFormat instanceof BeforeDetailsFormat ||
        rowFormat instanceof AfterDetailsFormat) {
        if (isSummaryDataChanged(grid, rowNode)) {
            grid.refreshRow(rowNode);
        }
    }
}

private boolean isSummaryDataChanged(HiGrid grid, RowNode rowNode) {
    DataTableModel dataTable = rowNode.getDataTableModel();
    HiGridInternalData internalData = (HiGridInternalData)dataTable.getInternalData(grid);
    SummaryDataTable summaryDataTable = (SummaryDataTable) internalData.getDataTable();
    RowFormat rowFormat = rowNode.getRowFormat();
    Vector cells = rowFormat.getDataFormats();

    // loop over all of the columns for this row object
    for (Enumeration e = cells.elements(); e.hasMoreElements();) {
        CellFormat cellFormat = (CellFormat) e.nextElement();
        String name = cellFormat.getName();

        // assume summary row format
        if (summaryDataTable.isResultDataChanged(grid, dataTable, rowFormat, rowNode, name)) {
            return true;
        }
    }
    return false;
}

}
