/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TableNodeRenderer.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:20 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.awt.*;

import com.klg.jclass.cell.*;
import com.klg.jclass.cell.renderers.*;

/**
 * An alternative to <code>DefaultNodeRenderer</code> when no indentation is wanted.
 */
public class TableNodeRenderer extends JCImageCellRenderer implements
    HiGridNodeRenderer {

static final long serialVersionUID = 6400676461571545997L;

/**
 * Used to draw the cell. Note that the size of the cell comes from
 * <CODE>JCCellInfo.getDrawingArea()</CODE>.  When the renderer's draw method is
 * called, the gc is translated to the upper-left corner of the drawing area
 * for the cell.  The "drawing area" of the cell is the region <EM>inside</EM>
 * the borders and the margins.
 *
 * @param gc Graphics context to use for drawing.
 * @param JCCellInfo Cell property information
 * @param o Object to be rendered
 * @param selected Boolean value indicating whether the cell is selected
 */
public void draw(Graphics gc, JCCellInfo cellFormat, Object o, boolean selected) {
    draw(gc, cellFormat, (NodeStatus)o, selected);
}

/**
 * Used to draw the cell. Note that the size of the cell comes from
 * <CODE>JCCellInfo.getDrawingArea()</CODE>.  When the renderer's draw method is
 * called, the gc is translated to the upper-left corner of the drawing area
 * for the cell.  The "drawing area" of the cell is the region <EM>inside</EM>
 * the borders and the margins.
 *
 * @param gc Graphics context to use for drawing.
 * @param JCCellInfo Cell property information
 * @param nodeStatus Information about the object to be rendered
 * @param selected Boolean value indicating whether the cell is selected
 */
public void draw(Graphics gc, JCCellInfo cellFormat, NodeStatus nodeStatus, boolean selected) {
    Point p = ((CellFormat)cellFormat).getTranslationOffset();
    gc.translate(-p.x, -p.y);
    Rectangle totalArea = ((CellFormat)cellFormat).getTotalArea();
    gc.setClip(totalArea);
    RowFormat.prepareDrawingArea(gc, (CellFormat)cellFormat, totalArea, selected);
    gc.translate(p.x, p.y);
    gc.setClip(cellFormat.getDrawingArea());
    if (nodeStatus == null || nodeStatus.getImage() == null) {
        return;
    }

    Point offset = getAlignmentOffset(cellFormat, nodeStatus.getImage());
    gc.setFont(cellFormat.getFont());
    gc.setColor(selected ? cellFormat.getSelectedForeground() :
                cellFormat.getForeground());

    gc.drawImage(nodeStatus.getImage(), offset.x, offset.y, cellFormat.getBackground(), this);
}

}
