/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Walkable.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:20 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid;

import java.io.Serializable;

/**
 * The Walkable interface should be implemented by objects that
 * wish to traverse HiGrid's RowTree.  The walk() method within
 * RowTree class takes as arguments an initial rowNode to start
 * walking from and a Walkable object that controls when the
 * walking is to finish.  You can use your Walkable object to
 * store state information that you can examine after the
 * walk has completed.
 */

public interface Walkable extends Serializable {

static final long serialVersionUID = 2625126985591452597L;

/**
 * Return true if walking downward, false if walking upward.
 */
boolean getDownward();

/**
 * Return true if the walk has met its terminating condition.
 * If this method never returns true, an infinite loop will result
 * when RowTree.walk() is invoked.
 */
boolean done();

/**
 * next() is invoked when the walk() routine moves to the next RowNode.
 *
 * @param rowNode The rowNode that the walk() routine has traversed to.
 */
void next(RowNode rowNode);

/**
 * previous() is invoked when the walk() routine moves to the previous RowNode.
 *
 * @param rowNode The rowNode that the walk() routine has traversed to.
 */
void previous(RowNode rowNode);

/**
 * find() is invoked by the walk() routine to test whether the passed
 * rowNode meets the walk termination criteria.
 *
 * @param rowNode The rowNode that the walk() routine has traversed to
 *                and is checking for termination.
 */
boolean find(RowNode rowNode);
}
