/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ExternalGridProperties.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:20 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid.beans;

import com.klg.jclass.higrid.*;
import com.klg.jclass.datasource.*;
import com.klg.jclass.datasource.beans.*;

/**
 *	This class encapsulates the data and format hierarchy for a <code>HiDataExternalDS</code>.
 *	@see com.klg.jclass.higrid.beans.JCHiGrid
 */
public class ExternalGridProperties extends GridProperties {

static final long serialVersionUID = 8295898196648158105L;

private final static String base = "jchigriddb";
private final static boolean TRACE = false;
private static int counter = 0;

public ExternalGridProperties() {
    this(NodeProperties.JDBC, true);
}

public ExternalGridProperties(boolean externalDS) {
    super(NodeProperties.JDBC, true);
}

public ExternalGridProperties(int type, boolean externalDS) {
    super(type, true);
}

/** Constructs a ExternalGridProperties from a resource.
 *	@param resource_name resource name relative to a class loader
 *						to the bean component. It must have the right extension.
 *	@param cl class loader to use.
 *  @exception DataModelException If the class can't be loaded.
 */
public ExternalGridProperties(String resource_name, ClassLoader cl) throws
DataModelException {
    this(resource_name, cl, null, null);
}

/** Constructs a ExternalGridProperties from a resource. First,
 *	the package name of the caller class is prepended to the resource
 *	name to locate the resource.  If no resource is found, the passed-in
 *	resource name is used.
 *	@param resource_name resource name relative to a class loader
 *						to the bean component. It must have the right extension.
 *	@param caller_class class that uses the serialized bean component.
 *	@param sfile serialization file that is used at design-time in JBuilder to
 *			load a ExternalGridProperties.
 *	@see java.lang.ClassLoader
 *      @exception DataModelException If the class can't be loaded.
 */
public ExternalGridProperties(String resource_name, Class caller_class, String sfile) throws
DataModelException {
    this(resource_name, caller_class, null, null);
}

/** Constructs a ExternalGridProperties from a resource. First,
 *	the package name of the caller class is prepended to the resource
 *	name to locate the resource.  If no resource is found, the passed-in
 *	resource name is used.
 *	@param resource_name resource name relative to a class loader
 *						to the bean component. It must have the right extension.
 *	@param caller_class class that uses the serialized bean component.
 *	@param sfile serialization file that is used at design-time in JBuilder to
 *			load a ExternalGridProperties.
 *	@see java.lang.ClassLoader
 *      @exception DataModelException If the class can't be loaded.
 */
public ExternalGridProperties(String resource_name, Class caller_class, String sfile,
                              String[] names, Object[] data_sources) throws DataModelException {
    super(resource_name, caller_class, sfile, names, data_sources, true);
}

/** Constructs a ExternalGridProperties from a resource. First,
 *	the package name of the caller class is prepended to the resource
 *	name to locate the resource.  If no resource is found, the passed-in
 *	resource name is used.
 *	@param resource_name resource name relative to a class loader
 *						to the bean component. It must have the right extension.
 *	@param caller_class class that uses the serialized bean component.
 *	@see java.lang.ClassLoader
 *      @exception DataModelException If the class can't be loaded.
 */
public ExternalGridProperties(String resource_name, Class caller_class) throws
DataModelException {
    this(resource_name, caller_class, null, null);
}

/** Constructs a ExternalGridProperties from a resource. First,
 *	the package name of the caller class is prepended to the resource
 *	name to locate the resource.  If no resource is found, the passed-in
 *	resource name is used.
 *	@param resource_name resource name relative to a class loader
 *						to the bean component. It must have the right extension.
 *	@param caller_class class that uses the serialized bean component.
 *	@see java.lang.ClassLoader
 *      @exception DataModelException If the class can't be loaded.
 */
public ExternalGridProperties(String resource_name, Class caller_class,
                              String[] names, Object[] data_sources) throws DataModelException {
    super(resource_name, caller_class, names, data_sources, true);
}

/** Constructs a ExternalGridProperties from a resource.
 *	@param resource_name resource name relative to a class loader
 *						to the bean component. It must have the right extension.
 *	@param cl class loader to use.
 *  @exception DataModelException If the class can't be loaded.
 */
public ExternalGridProperties(String resource_name, ClassLoader cl,
                              String[] names, Object[] data_sources) throws DataModelException {
    super(resource_name, cl, names, data_sources, true);
}

protected String getBase() {
    return base;
}

protected int getCounter() {
    return counter;
}

protected void setCounter(int counter) {
    this.counter = counter;
}

}
