/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: GridProperties.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:20 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid.beans;

import java.beans.Beans;
import java.io.IOException;
import java.util.Enumeration;

import javax.swing.tree.*;

import com.klg.jclass.higrid.LocaleBundle;

import com.klg.jclass.datasource.beans.NodeProperties;
import com.klg.jclass.datasource.beans.SerializedProperties;
import com.klg.jclass.datasource.DataSourceTreeModel;
import com.klg.jclass.datasource.DataModelException;
import com.klg.jclass.datasource.DataSourceTreeNode;
import com.klg.jclass.datasource.TreeIterator;
import com.klg.jclass.datasource.DataModelEvent;
import com.klg.jclass.datasource.ErrorDialog;
import com.klg.jclass.util.*;

/**
 *	This class encapsulates the data and format hierarchy for a <code>HiJCData</code>.  This is a tree
 *	of <code>JCHiGridNode</code>.  It observes changes of each <code>JCHiGridNode</code> and then
 *	tell its own observers of change.
 *  This is meant for internal use only by HiGrid
 *  and is not meant to be used by applets or applications.
 *	@see com.klg.jclass.higrid.beans.JCHiGridNode
 */
public class GridProperties extends SerializedProperties implements
    JCHiGridListener {

static final long serialVersionUID = -8818261247530958236L;

/** Data source type of the higrid bean. */
protected int dataSourceType = NodeProperties.JDBC;

/** Tree for JCHiGridNode */
private DataSourceTreeModel tree;

private transient Object[] dataSources;
private transient String[] dataSourceNames;
/** data model name used in data-binding editor. */
private String modelName;

/** For external data source. */
private String dataBinding;

/** Event listeners */
private transient JCListenerList JCHiGridListeners = null;
private transient boolean isFiringJCHiGridEvent = false;

/** Serialized version of the a bean component currently being deserialized. */
private static transient String serializedVersion = new String("");

private final static String base = "jchigrid";
private final static boolean TRACE = false;
private static int counter = 0;
private static int counterModelName = 1;

protected boolean externalDS;

/**
 *  Default Constructor.
 */
public GridProperties() {
    this(NodeProperties.JDBC, false);
}

/**
 * Constructor that is passed a flag to indicate whether an
 * internal or external datasource is required.
 *	@param externalDS indicates internal or external datasource
 */
public GridProperties(boolean externalDS) {
    this(NodeProperties.JDBC, externalDS);
}

/**
 * Constructor that is passed a dataSource type
 * and a flag to indicate whether an
 * internal or external datasource is required.
 *	@param type the dataSource type
 *	@param externalDS indicates internal or external datasource
 */
public GridProperties(int type, boolean externalDS) {
    this.dataSourceType = type;
    this.externalDS = externalDS;
    tree = new DataSourceTreeModel(DataSourceTreeModel.NullRoot);
    setDefaultSerializationFileBase(getBase());
    serializationFile = getDefaultSerializationFile(getBase(), getCounter());
    // update counter
    try {
        String s = serializationFile.substring(serializationFile.indexOf(getBase())+getBase().length());
        s = s.substring(0, s.indexOf(LocaleBundle.serialization_extension));
        setCounter(Integer.parseInt(s));
    }
    catch (Exception ex) {
    }
    this.modelName = "HiGridData" + counterModelName++;
}

/** Constructs a GridProperties from a resource.
 *	@param resource_name resource name relative to a class loader
 *						to the bean component. It must have the right extension.
 *	@param cl class loader to use.
 *  @exception DataModelException If the class can't be loaded.
 */
public GridProperties(String resource_name, ClassLoader cl) throws
DataModelException {
    this(resource_name, cl, null, null);
}

/** Constructs a GridProperties from a resource. First,
 *	the package name of the caller class is prepended to the resource
 *	name to locate the resource.  If no resource is found, the passed-in
 *	resource name is used.
 *	@param resource_name resource name relative to a class loader
 *						to the bean component. It must have the right extension.
 *	@param caller_class class that uses the serialized bean component.
 *	@param sfile serialization file that is used at design-time in JBuilder to
 *			load a GridProperties.
 *	@see java.lang.ClassLoader
 *  @exception DataModelException If the class can't be loaded.
 */
public GridProperties(String resource_name, Class caller_class, String sfile) throws
DataModelException {
/* previous code
this(resource_name, caller_class, null, null);
*/
    this(resource_name, caller_class, sfile, null, null);
}

/** Constructs a GridProperties from a resource. First,
 *	the package name of the caller class is prepended to the resource
 *	name to locate the resource.  If no resource is found, the passed-in
 *	resource name is used.
 *	@param resource_name resource name relative to a class loader
 *						to the bean component. It must have the right extension.
 *	@param caller_class class that uses the serialized bean component.
 *	@param sfile serialization file that is used at design-time in JBuilder to
 *			load a GridProperties.
 *	@see java.lang.ClassLoader
 *  @exception DataModelException If the class can't be loaded.
 */
public GridProperties(String resource_name, Class caller_class, String sfile,
                      String[] names, Object[] data_sources) throws DataModelException {
    this(resource_name, caller_class, sfile, names, data_sources, false);
}

/** Constructs a GridProperties from a resource. First,
 *	the package name of the caller class is prepended to the resource
 *	name to locate the resource.  If no resource is found, the passed-in
 *	resource name is used.
 *	@param resource_name resource name relative to a class loader
 *						to the bean component. It must have the right extension.
 *	@param caller_class class that uses the serialized bean component.
 *	@param sfile serialization file that is used at design-time in JBuilder to
 *			load a GridProperties.
 *	@param caller_class class that uses the serialized bean component.
 *	@see java.lang.ClassLoader
 *  @exception DataModelException If the class can't be loaded.
 */
public GridProperties(String resource_name, Class caller_class, String sfile,
                      String[] names, Object[] data_sources, boolean externalDS) throws DataModelException {
    try {
//        tree = new DataSourceTreeModel(new DataSourceTreeNode());
        tree = new DataSourceTreeModel(DataSourceTreeModel.NullRoot);
//FIXOBSOLETE
/*
    if (JCEnvironment.isJBuilder() && JCEnvironment.isDesignTime()) {
        GridProperties comp = (GridProperties)
                        GridProperties.fromFile(sfile);
        // copy properties
        copyProperties(comp);
    } else if (resource_name != null) {
*/
        if (resource_name != null) {
            GridProperties comp = (GridProperties)
                                  GridProperties.fromResource(caller_class, resource_name);
            // copy properties
            copyProperties(comp);
        }
        else {
            int c = getCounter();
            serializationFile = getBase() + c++ + LocaleBundle.serialization_extension;
            setCounter(c);
            resourceName = serializationFile;
            this.externalDS = externalDS;
        }
        dataSources = data_sources;
        dataSourceNames = names;
        if (this.externalDS != externalDS) {
            ErrorDialog.showError(null,
                                  LocaleBundle.string(LocaleBundle.wrongDataSourceType));
        }
    }
    catch (Exception ex) {
        throw new DataModelException(DataModelEvent.INITIALIZE, ex);
    }
}

/** Constructs a GridProperties from a resource. First,
 *	the package name of the caller class is prepended to the resource
 *	name to locate the resource.  If no resource is found, the passed-in
 *	resource name is used.
 *	@param resource_name resource name relative to a class loader
 *						to the bean component. It must have the right extension.
 *	@param caller_class class that uses the serialized bean component.
 *	@see java.lang.ClassLoader
 *      @exception DataModelException If the class can't be loaded.
 */
public GridProperties(String resource_name, Class caller_class) throws
DataModelException {
    this(resource_name, caller_class, null, null);
}

/** Constructs a GridProperties from a resource. First,
 *	the package name of the caller class is prepended to the resource
 *	name to locate the resource.  If no resource is found, the passed-in
 *	resource name is used.
 *	@param resource_name resource name relative to a class loader
 *						to the bean component. It must have the right extension.
 *	@param caller_class class that uses the serialized bean component.
 *	@see java.lang.ClassLoader
 *
 *      @exception DataModelException If the class can't be loaded.
 */
public GridProperties(String resource_name, Class caller_class,
                      String[] names, Object[] data_sources) throws DataModelException {
    this(resource_name, caller_class, names, data_sources, false);
}

/** Constructs a GridProperties from a resource.
 *	@param resource_name resource name relative to a class loader
 *						to the bean component. It must have the right extension.
 *  @exception DataModelException If the class can't be loaded.
 */
public GridProperties(String resource_name, Class caller_class,
                      String[] names, Object[] data_sources, boolean externalDS) throws DataModelException {
    try {
//        tree = new DataSourceTreeModel(new DataSourceTreeNode());
        tree = new DataSourceTreeModel(DataSourceTreeModel.NullRoot);
        if (resource_name != null) {
            GridProperties comp = (GridProperties)
                                  GridProperties.fromResource(caller_class, resource_name);
            // copy properties
            copyProperties(comp);
        }
        else {
            int c = getCounter();
            serializationFile = getBase() + c++ + LocaleBundle.serialization_extension;
            setCounter(c);
            resourceName = serializationFile;
            this.externalDS = externalDS;
        }
        dataSources = data_sources;
        dataSourceNames = names;
        if (this.externalDS != externalDS) {
            ErrorDialog.showError(null,
                                  LocaleBundle.string(LocaleBundle.wrongDataSourceType));
        }
    }
    catch (Exception ex) {
        throw new DataModelException(DataModelEvent.INITIALIZE, ex);
    }
}

/** Constructs a GridProperties from a resource.
 *	@param resource_name resource name relative to a class loader
 *						to the bean component. It must have the right extension.
 *  @exception DataModelException If the class can't be loaded.
 */
public GridProperties(String resource_name, ClassLoader cl,
                      String[] names, Object[] data_sources) throws DataModelException {
    this(resource_name, cl, names, data_sources, false);
}

/** Constructs a GridProperties from a resource.
 *	@param resource_name resource name relative to a class loader
 *						to the bean component. It must have the right extension.
 *  @exception DataModelException If the class can't be loaded.
 */
public GridProperties(String resource_name, ClassLoader cl,
                      String[] names, Object[] data_sources, boolean externalDS) throws DataModelException {
    try {
//        tree = new DataSourceTreeModel(new DataSourceTreeNode());
        tree = new DataSourceTreeModel(DataSourceTreeModel.NullRoot);
        if (resource_name != null) {
            GridProperties comp = (GridProperties)
                                  GridProperties.fromResource(cl, resource_name);
            // copy properties
            copyProperties(comp);
        }
        else {
            int c = getCounter();
            serializationFile = getBase() + c++ + LocaleBundle.serialization_extension;
            setCounter(c);
            resourceName = serializationFile;
            this.externalDS = externalDS;
        }
        dataSources = data_sources;
        dataSourceNames = names;
        if (this.externalDS != externalDS) {
            ErrorDialog.showError(null,
                                  LocaleBundle.string(LocaleBundle.wrongDataSourceType));
        }
    }
    catch (Exception ex) {
        throw new DataModelException(DataModelEvent.INITIALIZE, ex);
    }
}

private void copyProperties(GridProperties comp) {
    setRoot(comp.getRoot());
    setSerializationFile(comp.getSerializationFile());
    setResourceName(comp.getResourceName());
    setDataSourceType(   comp.getDataSourceType(   ));
    setModelName(comp.getModelName());
    setDataBinding(comp.getDataBinding());
    setExternalDS(comp.isExternalDS());
}

/**
 * Gets the base part of the default serialization file name.
 */
protected String getBase() {
    return base;
}

/**
 *Gets the variable part of the default serialization file name.
 */
protected int getCounter() {
    return counter;
}

/**
 * Sets the variable part of the default serialization file name.
 */
protected void setCounter(int counter) {
    this.counter = counter;
}

/**
 * Sets the data source type.
 */
public void setDataSourceType(int type) {
    if (dataSourceType == type) return;

    dataSourceType = type;
    setChanged();
}

/**
 * Returns the data source type.
 */
public int getDataSourceType() {
    return dataSourceType;
}


/**
 * Returns an array of the names of IDE data source objects.
 */
public String[] getDataSourceNames() {
    return dataSourceNames;
}

/**
 * Returns an array of IDE data source objects.
 */
public Object[] getDataSources() {
    return dataSources;
}

/**
 * Sets the model name.
 */
public void setModelName(String s) {
    if (isSameString(modelName, s)) return;

    this.modelName = s;
    setChanged();
}

/**
 * Gets the model name.
 */
public String getModelName() {
    return modelName;
}

/**
 * Sets the external datasource indication.
 */
public void setExternalDS(boolean externalDS) {
    this.externalDS = externalDS;
}

/**
 * Gets the external datasource indication.
 */
public boolean isExternalDS() {
    return externalDS;
}

/** Sets the external data binding string. */
public void setDataBinding(String s) {
    if (isSameString(dataBinding, s)) return;

    this.dataBinding = s;
    setChanged();
}

/**
 * Gets the external data binding string.
 */
public String getDataBinding() {
    return dataBinding;
}

/** Instantiates a SerializedProperties from a resource.
 *	@see java.beans.Beans#instantiate
 *      @exception ClassNotFoundException If the class can't be found.
 *      @exception IOException If an IO exception occurs.
 */
public static SerializedProperties fromResource(ClassLoader cl, String name) throws
ClassNotFoundException, IOException {
    GridProperties comp = (GridProperties)
                          SerializedProperties.fromResource(cl, name);
    return comp;
}

/**
 * Gets the current version.
 */
public static String getVersion() {
    return 	com.klg.jclass.higrid.JCVersion.getVersionNumber();
}

/**
 * Gets the serialized version of the bean component currently being de-serialized.
 */
public static synchronized String getSerializedVersion() {
    return serializedVersion;
}

//////////////////////////////////////////////////////////////
//
//	DataSourceTreeModel methods
//
/** Gets the root of the tree structure.
 */
public DataSourceTreeNode getRoot() {
    return(DataSourceTreeNode) tree.getRoot();
}

/**
 * Sets the root of the tree structure.
 */
public void setRoot(DataSourceTreeNode  root) {
    tree.setRoot(root);
    setChanged();
}

/**
 * Removes all nodes from the tree.
 */
public void removeAll() {
    tree.setRoot(null);
    setChanged();
}

/**
 * Removes a node from the tree.
 */
public void remove(DataSourceTreeNode node) {
    DataSourceTreeNode parent = (DataSourceTreeNode)node.getParent();
    if (parent == null) {
        removeAll();
    }
    else {
        parent.remove(node);
    }
    setChanged();
}

/**
 * Sets the component value changed indication.
 */
protected void setChanged() {
    if (TRACE) {
        System.out.println("GridProperties changed.");
        Thread.currentThread().dumpStack();
    }
    super.setChanged();
}

/**
 * Adds the specified listener to receive JCHiGridEvent objects.
 *
 * @param listener A listener.
 */
public synchronized void addJCHiGridListener(JCHiGridListener listener) {
    JCHiGridListeners = JCListenerList.add(JCHiGridListeners, listener);
}

/**
 * Removes the specified listener so it no longer
 * receives <code>JCHiGridEvent</code> objects.
 *
 * @param listener A listener.
 */
public synchronized void removeJCHiGridListener(JCHiGridListener listener) {
    JCHiGridListeners = JCListenerList.remove(JCHiGridListeners, listener);
}

/**
 * Informs registered listeners of a <code>JCHiGridEvent</code>.
 *
 * @param event The JCHiGridEvent.
 */
public void fireJCHiGridEvent(JCHiGridEvent event) {
    boolean checkInFire = (event.getValue() != JCHiGridEvent.HIGRID_BEAN_FIELDS_UPDATE);
    if (checkInFire) {
        if (isFiringJCHiGridEvent) {
            // avoid recursive calls
            return;
        }
        isFiringJCHiGridEvent = true;
    }
    Enumeration e = JCListenerList.elements(JCHiGridListeners);
    while (e.hasMoreElements()) {
        JCHiGridListener listener = (JCHiGridListener)e.nextElement();
        listener.JCHiGridValueChanged(event);
    }
    if (checkInFire) {
        isFiringJCHiGridEvent = false;
    }
}

/** Helper method to display the tree. */
private void buildTree(StringBuffer s, DataSourceTreeNode node, int level) {
    if (node == null) return;

    s.append("\n\tLEVEL: " + level++);
    s.append("\n");
    s.append(node);
    s.append("\n");
    s.append(node.getUserObject());
    s.append("\n");

    TreeIterator ti = node.getIterator();
    while (ti.hasMoreElements()) {
        DataSourceTreeNode child = (DataSourceTreeNode) ti.nextElement();
        buildTree(s, child, level);
    }
}

/**
 * Overrides the generic <code>toString()</code> method.
 */
public String toString() {
    String str = super.toString();
    String ln = System.getProperty("line.separator");
    if (TRACE) {
        StringBuffer s = new StringBuffer();
        s.append(ln + "Serialization file: ");
        s.append(serializationFile);
        s.append(ln);
        // Build tree
        buildTree(s, getRoot(), 0);
        str = str + new String(s);
    }
    return str;
}


/**
 * Overrides the default write object to write the current version first.
 * @exception java.io.IOException If an IO exception occurs.
 */
private void writeObject(java.io.ObjectOutputStream on) throws java.io.IOException {
    if (TRACE) {
        System.out.println("write version: " + getVersion());
    }
    // Write the current version
    on.writeObject(getVersion());
    try {
        on.defaultWriteObject();
    }
    catch (Exception e) {
        e.printStackTrace(System.out);
    }
}

/** Overrides the default read object to read the serialized version first.
 *	Classes that changed between versions can compare the serialized
 *	version and the current version to see if compatibility needs to
 *	be handled.
 */
private void readObject(java.io.ObjectInputStream in) throws java.io.IOException {
    synchronized(serializedVersion) {
        try {
            // Get the version serialized.
            serializedVersion = (String) in.readObject();
            if (TRACE) {
                System.out.println("serialized version: " + serializedVersion);
            }
            in.defaultReadObject();
        }
        catch (Exception e) {
            e.printStackTrace(System.out);
        }
    }
}

/**
 * When value(s) of the <code>JCHiGridNode</code> is changed.
 * @see com.klg.jclass.higrid.beans.JCHiGridNode
 */
public void JCHiGridValueChanged(JCHiGridEvent e) {
    // mark component as changed
    setChanged();

    if (e.getJCHiGridNode() != null &&
        e.getValue() == JCHiGridEvent.DATA_BEAN_COMPONENT_VALUE) {
        // data bean component has changed, need to do this
        // just in case columns will be different
        ((JCHiGridNode) e.getJCHiGridNode()).setDoRetrieveAllFields(true);
    }

    // pass it on
    fireJCHiGridEvent(new JCHiGridEvent((Object)this, e.getValue(),
                                        e.getJCHiGridNode(), e.getNodeProperties()));
}

}
