/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCHiGrid.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:20 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid.beans;

import com.klg.jclass.datasource.*;
import com.klg.jclass.datasource.beans.DataSourceBeanInstance;
import com.klg.jclass.datasource.beans.MetaDataCreator;
import com.klg.jclass.datasource.beans.NodeProperties;
import com.klg.jclass.datasource.util.DataBinding;
import com.klg.jclass.higrid.*;
import com.klg.jclass.higrid.LocaleBundle;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.swing.JCMessageHelper;

import javax.swing.*;
import java.applet.Applet;
import java.applet.AppletContext;
import java.util.Vector;

/**
 * A Bean wrapper for HiGrid. This is meant only for internal use by HiGrid
 * and is not meant to be used by applets or applications.
 */
public class JCHiGrid extends HiGrid {

static final long serialVersionUID = -7927463458656822966L;

/** The parent applet. */
protected transient Applet applet;

/** Creator to construct a meta data. */
protected MetaDataCreator metaDataCreator = null;

/** Name of the Bean. */
//PR30159 need to specify the name.
protected String name = "com.klg.jclass.higrid.beans.JCHiGrid";

/** External DS indication. */
protected boolean externalDS;

/** Bean component. */
protected GridProperties comp;

/** Data model of the higrid Bean. */
protected HiGridData tree;


/** label property for the bean */
protected String label = "JCHiGrid";

protected static JCTypeConverter conv = new JCTypeConverter();

/** Used in <code>getParams</code>. */
private JComponent component = new JLabel();

private static String base = "JCHiGrid";
private static int nameCounter = 0;
private static final boolean TRACE = false;

/** Creates a <code>JCHiGrid</code> with the default JDBC data source type
 * and uses an internal datasource.
 */
public JCHiGrid() {
    this(NodeProperties.JDBC, false);
}

/** Creates a <code>JCHiGrid</code> with the indicated data source type that
 * uses an internal or external datasource.
 * @param type the datasource type
 * @param externalDS indicates if it uses an external datasource or not
 */
public JCHiGrid(int type, boolean externalDS) {
    super();

    this.externalDS = externalDS;
    metaDataCreator = com.klg.jclass.datasource.beans.MetaDataCreator.getCreator(type);
    comp = createGridProperties(type, externalDS);
    tree = new HiGridData(new TreeData());
    tree.setModelName(comp.getModelName());
}

/** Creates a <code>JCHiGrid</code>, which reads parameters from the applet's HTML file.
 * @param applet the applet whose <code>PARAM</code> tags are to be read
 * @param name if this is not null, only parameters preceded by this name are read
 * @see java.applet.Applet#getParameter
 */
public JCHiGrid(Applet applet, String name) {
    this();
    this.applet = applet;
    setName((name == null) ?  (base + nameCounter++) : name);
    if (getClass().getName().equals("com.klg.jclass.higrid.JCHiGrid")) {
        getParameters(applet);
    }
}

/** Creates a <code>GridProperties</code> or a <code>ExternalGridProperties</code>.
 * @param type the datasource type
 * @param externalDS indicates if it uses an external datasource or not
 * @return
 */
protected GridProperties createGridProperties(int type, boolean externalDS) {
    GridProperties comp = null;
    if (externalDS) {
        comp =  new ExternalGridProperties(type, externalDS);
    }
    else {
        comp = new GridProperties(type, externalDS);
    }
    return comp;
}

/** Creates a <code>GridProperties</code> or a <code>ExternalGridProperties</code>.
 *	@param resource_name resource name relative to a class loader 
 *						to the bean component (it must have extension ".ser")
 *	@param cl class loader to use
 *  @exception DataModelException If the class can't be loaded.
 * @return
 */
protected GridProperties createGridProperties(String resource_name, ClassLoader cl) 
throws DataModelException {
    GridProperties comp = null;
    if (externalDS) {
        comp =   new ExternalGridProperties(resource_name, cl);
    }
    else {
        comp =  new GridProperties(resource_name, cl);
    }
    return comp;
}

/** Gets the name of the Bean, as set in the constructor or by <code>setName</code>.
 * If no name has been set, a 0-length String will be returned.
 * @see #setName
 * @return
 */
public String getName() {
    return name != null ? name : "";
}

/**
 * Sets the name of the Bean.
 * @param name
 */
public void setName(String name) {
    this.name = name;
}

/**
 * Gets the <code>externalDS</code> indication.
 * @return
 */
public boolean isExternalDS() {
    return externalDS;
}

/** Reads the parameter values from the HTML page using the specified applet.
 * The values will override those previously set.
 * @param applet if not null and in a browser, its <code>documentBase()</code> is used to
 * construct a complete filename, if necessary
 * @see java.applet.Applet#getParameter
 */
public void getParameters(Applet applet) {
    getParameters(applet, null);
}

/** Reads the parameter values from the file.
 * The values will override those previously set.
 * @param applet if not null and in a browser, its <code>documentBase()</code> is used to
 * construct a complete filename, if necessary
 * @param file if an http protocol is not specified (if a ":" is not present), the current working directory is used
 */
public void getParameters(Applet applet, String file) {
    this.applet = applet;

    if (file == null)
        file = getParam("paramFile");
//FIXOBSOLETE
/*
if (file != null)
    JCTypeConverter.setParamSource(component, JCFile.read(applet, file));
*/
    // Read HTML file if running in a browser, or file if it is not null
    if (file != null || getAppletContext() != null) {
        getParameters();
    }
}

/**
 * Gets the context for the Bean's applet. The applet context
 * lets an applet control the applet's environment, which is
 * usually the browser or the applet viewer.
 * @return null if the component is not in an applet or the applet is not in
 * a browser
 * @see java.applet.Applet#getAppletContext
 */
public AppletContext getAppletContext() {
    return getAppletContext(applet);
}


/** Gets the context for the Bean's applet. The applet context
 * lets an applet control the applet's environment which is
 * usually the browser or the applet viewer.
 * @param applet if not null and in a browser, its <code>documentBase()</code> is used to
 * construct a complete filename if necessary
 * @return null if the component is not in an applet or the applet is not in
 * a browser.
 * @see java.applet.Applet#getAppletContext
 */
public static AppletContext getAppletContext(Applet applet) {
    if (applet != null) {
        // AWT bug - null exception thrown
        try {
            return applet.getAppletContext();
        }
        catch (Exception e) {
        }
    }
    return null;
}

/** Gets the <code>About</code> property.
 *	The <code>About</code> property displays contact information for
 * Quest Software.
 * @return
 */
public String getAbout() {
    return LocaleBundle.string(LocaleBundle.about);
}
/** <code>About</code> property is read-only.
 * @param s
 */
public void setAbout(String s) {
}

/**
 * Gets the value of the <code>Version</code> property.
 * The <code>Version</code> property specifies the version number.
 * @return
 */
public String getVersion() {
    return com.klg.jclass.higrid.JCVersion.getVersionString();
}

/** <code>Version</code> property is read-only.
 * @param s
 */
public void setVersion(String s) {
}

/** Helper method called recursively to create meta data. 
 *	@exception DataModelException If fails to create a meta data.
 * @param node
 * @param meta_parent
 */
protected void loadModelData(DataSourceTreeNode node, MetaDataModel meta_parent) throws DataModelException {
    if (node == null || node.getUserObject() == null) return;

    NodeProperties data = ((JCHiGridNode) node.getUserObject()).getNodeProperties();

    MetaDataModel meta = null;
    meta = metaDataCreator.createMetaData(tree, meta_parent, data);

    TreeIterator ti = node.getIterator();
    while (ti.hasMoreElements()) {
        DataSourceTreeNode child = (DataSourceTreeNode) ti.nextElement();
        loadModelData(child, meta);
    }
}

/** Helper method to load format into a node in JClass HiGrid.
 * @param node
 * @param fn
 */
protected void loadFormat(DataSourceTreeNode node, FormatNode fn) {
    if (node == null || node.getUserObject() == null || fn == null) return;

    JCHiGridNode on = (JCHiGridNode) node.getUserObject();

    // no data format in JCHiGridNode.  It means no format is 
    // set up.  Thus, don't need to load format.
    if (on.getRowFormat(JCHiGridNode.SECTION_RECORD).getDataFormats().size() == 0) {
        return;
    }

    // auto expand
    fn.setAutoTraverse(on.getAutoTraverse());

    // default sort data
    SortData sortData = on.getDefaultSortData();
    if (sortData != null) {
        String name = sortData.getColumnName();
        if (name.equals("") || sortData.getDirection() == SortGrid.UNKNOWN) {
            sortData = null;
        }
    }
    fn.setDefaultSortData(sortData);

//FIXOBSOLETE
/*
// Set indicator formats' grid to this instance.
for (int i=0; i<JCHiGridNode.SECTION_MAX; i++) {
    on.getRowFormat(i).setIndicatorFormatsHiGrid(this);
}
*/
    // set meta data (record and header)
    on.getRowFormat(JCHiGridNode.SECTION_RECORD).setMetaData(
                                                            fn.getRecordFormat().getMetaData());
    on.getRowFormat(JCHiGridNode.SECTION_HEADER).setMetaData(
                                                            fn.getHeaderFormat().getMetaData());

    // set summary meta data
    SummaryMetaData metaData = (SummaryMetaData) fn.getBeforeDetailsFormat().getMetaData();
    Vector columns = on.getSummaryColumns(JCHiGridNode.SECTION_BEFORE_DETAILS);
    if (!columns.isEmpty()) {
        metaData.setupSumaryColumns(columns);
    }
    on.getRowFormat(JCHiGridNode.SECTION_BEFORE_DETAILS).setMetaData(metaData);

    metaData = (SummaryMetaData) fn.getAfterDetailsFormat().getMetaData();
    columns = on.getSummaryColumns(JCHiGridNode.SECTION_AFTER_DETAILS);
    if (!columns.isEmpty()) {
        metaData.setupSumaryColumns(columns);
    }
    on.getRowFormat(JCHiGridNode.SECTION_AFTER_DETAILS).setMetaData(metaData);

    metaData = (SummaryMetaData) fn.getFooterFormat().getMetaData();
    columns = on.getSummaryColumns(JCHiGridNode.SECTION_FOOTER);
    if (!columns.isEmpty()) {
        metaData.setupSumaryColumns(columns);
    }
    on.getRowFormat(JCHiGridNode.SECTION_FOOTER).setMetaData(metaData);

    // modify format node
    fn.setHeaderFormat((HeaderFormat) 
                       on.getRowFormat(JCHiGridNode.SECTION_HEADER));
    fn.setBeforeDetailsFormat((BeforeDetailsFormat) 
                              on.getRowFormat(JCHiGridNode.SECTION_BEFORE_DETAILS));
    fn.setRecordFormat((RecordFormat) 
                       on.getRowFormat(JCHiGridNode.SECTION_RECORD));
    fn.setAfterDetailsFormat((AfterDetailsFormat) 
                             on.getRowFormat(JCHiGridNode.SECTION_AFTER_DETAILS));
    fn.setFooterFormat((FooterFormat) 
                       on.getRowFormat(JCHiGridNode.SECTION_FOOTER));

    TreeIterator ti = node.getIterator();
    TreeIterator tifn = fn.getIterator();
    while (ti.hasMoreElements() && tifn.hasMoreElements()) {
        DataSourceTreeNode child = (DataSourceTreeNode) ti.nextElement();
        FormatNode childfn = (FormatNode) tifn.nextElement();
        loadFormat(child, childfn);
    }   
}

protected void formatGrid() {
    // Don't format if no meta data tree root.
    if (getDataModel() == null || getDataModel().getMetaDataTree() == null ||
        getDataModel().getMetaDataTree().getRoot() == null) return;

    // recursively load format.
    adviseClearAllFormats = false;
    setBatched(true);
    loadFormat(comp.getRoot(), (FormatNode) getFormatTree().getRoot());
    resetRunTimeGrid();
    createDefaultTrees();
    setBatched(false);
    if (adviseClearAllFormats) {
        JCMessageHelper.showError(
                                 LocaleBundle.string(LocaleBundle.adviseClearAllFormatsTitle), 
                                 LocaleBundle.string(LocaleBundle.adviseClearAllFormatsMessage));
    }
}

/** Sets the <code>GridProperties</code> property.
 *  @param comp the <code>GridProperties</code>
 *	@exception DataModelException If fails to create a meta data.
 */
public void setGridProperties(GridProperties comp) throws DataModelException {
    if (TRACE) {
        System.out.println("set GridProperties from\n" + this.comp + 
                           "\n to \n" + comp);
    }

    // remove the old instance
    if (externalDS) {
        // remove the internal instance, in case this JCHiGrid uses
        // internal data source before.
        DataSourceBeanInstance.removeInstance(tree);
    }
    else {
        if (comp.getModelName() != null && tree.getModelName() != null &&
            !tree.getModelName().equals(comp.getModelName())) {
            DataSourceBeanInstance.removeInstance(tree.getModelName());
        }
    }

    this.comp = comp;

    if (!externalDS) {
        tree.setModelName(comp.getModelName());
        createMetaData();
        setDataModel(tree);
    }
    else {
        DataModel dm = null;
        int index = -1;
        if (comp.getDataBinding() != null) {
            index = comp.getDataBinding().indexOf(DataBinding.SOURCE_DELIMITER);
        }
        if (index > -1) {
            dm = (DataModel) DataSourceBeanInstance.getInstance(
                                                               comp.getDataBinding().substring(0, index));
        }
        setDataModel(dm);
    }

//FIX we need our own data model listener here to handle the initialize() method
/*
if (tree.getMetaDataTree().getRoot() != null) {
    fireHiGridEvent(new HiGridEvent(this, 
                        new DataModelEvent(this, getDataModelListener(),  
                                (DataTableModel) tree.getDataTableTree().getRoot(), 
                                -1, "", DataModelEvent.INITIALIZE)));
}
*/    
    if (tree.getMetaDataTree().getRoot() != null && getDataModel() != null) {
        getDataModel().fireDataModelEvent(new DataModelEvent(this, getDataModelListener(),  
                                                             (DataTableModel) tree.getDataTableTree().getRoot(), 
                                                             -1, "", DataModelEvent.INITIALIZE));
    }
    if (!externalDS) {
        if (DataSourceBeanInstance.getInstance(tree.getModelName()) == null) {
            DataSourceBeanInstance.addInstance(tree);
        }
    }
    formatGrid();
}

/** Gets the <code>GridProperties</code> property.
 *	@see #setGridProperties
 * @return
 */
public GridProperties getGridProperties() {
//FIXOBSOLETE
/*
if (JCEnvironment.isDesignTime() && JCEnvironment.isVisualCafe()) {
    // Because GridProperties becomes non-serializable 
    // after data model of HiGrid is set, VCafe 2.5 
    // has problem handling it.  Thus,
    // a "clean" and simple GridProperties is returned
    // to VCafe 2.5.
    removeHiGridReferences(comp.getRoot());
} 
*/
    return comp;
}

/** Sets the <code>GridProperties</code> property.
 *	@param resource_name resource name relative to a class loader 
 *						to the HiGrid Bean component
 *	@param cl class loader used to get the resource
 * 	@exception DataModelException If fails to create a meta data.
 */
public void setGridProperties(String resource_name, 
                              ClassLoader cl) throws DataModelException {
    setGridProperties(createGridProperties(resource_name, cl));
}


/**	Creates meta data for the tree data Bean.
 *	@exception DataModelException If fails to create a meta data.
 */
protected void createMetaData() throws DataModelException {
//FIXOBSOLETE
//    tree.getMetaDataTree().setRoot(null);
//	tree.getDataTableTree().setRoot(null);
    loadModelData(comp.getRoot(), null);
}

/** Reads the parameter values from the HTML page using the data Bean's applet.
 * The values will override those previously set.
 * Subclasses may override this method to set their own values - in this case,
 * the method should first call <code>super.getParameters()</code>.
 */
protected void getParameters() {
    JCHiGridConverter.getParams(this);
}

/** Gets a parameter for this data Bean.
 * @return
 * @param param
 */
protected String getParam(String param) {
//FIXOBSOLETE
//	return conv.getParam(applet, component, getName(), param);
    return "";
}

//FIXOBSOLETE
private void removeHiGridReferences(DataSourceTreeNode node) {
/*
if (node == null) return;

JCHiGridNode hn = (JCHiGridNode) node.getUserObject();
if (hn != null) {
    for (int i=0; i<JCHiGridNode.SECTION_MAX; i++) {
        Vector v = hn.getRowFormat(i).getIndicatorFormats();
        Vector vdata = hn.getRowFormat(i).getDataFormats();
        if (v != null) {
            for (Enumeration e = v.elements(); e.hasMoreElements(); ) {
                CellFormat cell = (CellFormat) e.nextElement();
                cell.setGrid(null);
            }
        }
        if (vdata != null) {
            for (Enumeration e = vdata.elements(); e.hasMoreElements(); ) {
                CellFormat cell = (CellFormat) e.nextElement();
                cell.setGrid(null);
            }
        }
    }
}
TreeIterator ti = node.getIterator();
while (ti.hasMoreElements()) {
    DataSourceTreeNode child = (DataSourceTreeNode) ti.nextElement();
    removeHiGridReferences(child);
}
*/
}

class JCHiGridDataModelListener extends DataModelAdapter {

    /** Handles the <code>initialize()</code> <code>DataModelEvent</code>.
     * This overrides the <code>dataModelCreated()</code> implementation
     * in HiGrid.
     * @param e
     */
    public void initialize(DataModelEvent e) {
        if (e.getSource() instanceof DataModel) {
            setDataModel((DataModel) e.getSource());
            formatGrid();
        }
    }

//FIX - what happens to the rest of the events?
// they are currently "thrown away".  Should they be passed on?

}

/**
 *  Returns the value of the label property.
  * @return String
 */
public String getLabel() {
    return label;
}

/**
 * Sets the label property.
  * @param label
 */
public void setLabel(String label) {
    this.label = label;
}

}
