/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCHiGridConverter.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:21 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid.beans;

import java.io.Serializable;

import com.klg.jclass.higrid.HiGrid;
import com.klg.jclass.higrid.RowFormat;
import com.klg.jclass.higrid.SummaryColumn;
import com.klg.jclass.higrid.HiGridBorder;
import com.klg.jclass.higrid.FormatNode;
import com.klg.jclass.higrid.LocaleBundle;
import com.klg.jclass.higrid.FolderIconStyle;
import com.klg.jclass.higrid.SortGrid;
import com.klg.jclass.higrid.GridScrollbar;

import com.klg.jclass.cell.JCCellInfo;
import com.klg.jclass.datasource.*;
import com.klg.jclass.util.JCTypeConverter;

/** Reads <CODE>JCHiGrid</CODE> parameters from an HTML file or external file. */
public class JCHiGridConverter implements Serializable { 

static final long serialVersionUID = -8116690312484391294L;

/**
 * Row selection mode value that corresponds to the list of String names.
 * @see #getRowSelectionModeStrings
 */
public final static int[] rowSelectionModeValues = {
    HiGrid.ROW_SELECT_ANY,
    HiGrid.ROW_SELECT_IN_SAME_LEVEL,
    HiGrid.ROW_SELECT_IN_SAME_TABLE,
};

public final static String[] rowSelectionModeStrings = {
    LocaleBundle.ROW_SELECT_ANY,
    LocaleBundle.ROW_SELECT_IN_SAME_LEVEL,
    LocaleBundle.ROW_SELECT_IN_SAME_TABLE,
};

/**
 * Print format value that corresponds to the list of String names.
 * @see #getPrintFormatStrings
 */
public final static int[] printFormatValues = {
    HiGrid.PRINT_AS_DISPLAYED,
    HiGrid.PRINT_AS_EXPANDED,
    HiGrid.PRINT_EXPAND_ALL,
};

public final static String[] printFormatStrings = {
    LocaleBundle.PRINT_AS_DISPLAYED,
    LocaleBundle.PRINT_AS_EXPANDED,
    LocaleBundle.PRINT_EXPAND_ALL,
};


/**
 * Auto show value that corresponds to the list of String names.
 * @see #getAutoTraverseStrings
 */
public final static int[] autoTraverseValues = {
    FormatNode.AUTO_TRAVERSE_NONE,
    FormatNode.AUTO_TRAVERSE_EXPAND,
    FormatNode.AUTO_TRAVERSE_DETECT_CHILDREN,
    FormatNode.AUTO_TRAVERSE_CALLBACK,
};

public final static String[] autoTraverseStrings = {
    LocaleBundle.AUTO_TRAVERSE_NONE,
    LocaleBundle.AUTO_TRAVERSE_EXPAND,
    LocaleBundle.AUTO_TRAVERSE_DETECT_CHILDREN,
    LocaleBundle.AUTO_TRAVERSE_CALLBACK,
};


/**
 * Border value that corresponds to the list of String names.
 * @see #getRowFormatBorderStyleStrings
 */
public final static int[] rowFormatBorderStyleValues = {
    HiGridBorder.NONE,
    HiGridBorder.ETCHED_IN, HiGridBorder.ETCHED_OUT, 
    HiGridBorder.IN, HiGridBorder.OUT, 
    HiGridBorder.PLAIN, 
    HiGridBorder.FRAME_IN, HiGridBorder.FRAME_OUT,
    HiGridBorder.CONTROL_IN, HiGridBorder.CONTROL_OUT,
    RowFormat.MIXED_BORDER_STYLES, 
};

/** String names of the border styles. */
// Note: Assume mixed border style is the last one.
public final static String[] rowFormatBorderStyleStrings = {
    LocaleBundle.NONE,
    LocaleBundle.ETCHED_IN, LocaleBundle.ETCHED_OUT, 
    LocaleBundle.IN, LocaleBundle.OUT, 
    LocaleBundle.PLAIN, 
    LocaleBundle.FRAME_IN, LocaleBundle.FRAME_OUT,
    LocaleBundle.CONTROL_IN, LocaleBundle.CONTROL_OUT,
    LocaleBundle.MIXED_BORDER_STYLES, 
};

/**
 * Cell horizontal alignment value that corresponds to the list of String names.
 * @see #getCellHorizontalAlignmentStrings
 */
public final static int[] cellHorizontalAlignmentValues = {
    JCCellInfo.LEFT, JCCellInfo.CENTER, JCCellInfo.RIGHT,
};

/** String names of the cell horizontal alignment. */
public final static String[] cellHorizontalAlignmentStrings = {
    LocaleBundle.LEFT, LocaleBundle.CENTER, LocaleBundle.RIGHT,
};

/**
 * Cell vertical alignment value that corresponds to the list of String names.
 * @see #getCellVerticalAlignmentStrings
 */
public final static int[] cellVerticalAlignmentValues = {
    JCCellInfo.TOP, JCCellInfo.CENTER, JCCellInfo.BOTTOM,
};

/** String names of the cell vertical alignment. */
public final static String[] cellVerticalAlignmentStrings = {
    LocaleBundle.TOP, LocaleBundle.CENTER, LocaleBundle.BOTTOM,
};

/**
 * <code>MetaDataModel</code> data type values that corresponds to the list of String names.
 * @see #getMetaDataModelTypeStrings
 */
public final static int[] metaDataModelTypeValues = {
    MetaDataModel.TYPE_BOOLEAN, MetaDataModel.TYPE_SQL_DATE,
    MetaDataModel.TYPE_DOUBLE, MetaDataModel.TYPE_FLOAT,
    MetaDataModel.TYPE_INTEGER, MetaDataModel.TYPE_STRING,
    MetaDataModel.TYPE_BIG_DECIMAL, MetaDataModel.TYPE_LONG,
    MetaDataModel.TYPE_SQL_TIME, MetaDataModel.TYPE_SQL_TIMESTAMP,
    MetaDataModel.TYPE_OBJECT, MetaDataModel.TYPE_BYTE,
    MetaDataModel.TYPE_SHORT, MetaDataModel.TYPE_BYTE_ARRAY,
    MetaDataModel.TYPE_UTIL_DATE,
};

/** String names of the <code>MetaDataModel</code> data type. */
public final static String[] metaDataModelTypeStrings = {
    LocaleBundle.TYPE_BOOLEAN, LocaleBundle.TYPE_SQL_DATE,
    LocaleBundle.TYPE_DOUBLE, LocaleBundle.TYPE_FLOAT,
    LocaleBundle.TYPE_INTEGER, LocaleBundle.TYPE_STRING,
    LocaleBundle.TYPE_BIG_DECIMAL, LocaleBundle.TYPE_LONG,
    LocaleBundle.TYPE_SQL_TIME, LocaleBundle.TYPE_SQL_TIMESTAMP,
    LocaleBundle.TYPE_OBJECT, LocaleBundle.TYPE_BYTE,
    LocaleBundle.TYPE_SHORT, LocaleBundle.TYPE_BYTE_ARRAY,
    LocaleBundle.TYPE_UTIL_DATE,
};

/**
 * Cell edit policy value that corresponds to the list of String names.
 * @see #getCellEditPolicyStrings
 */
public final static int[] cellEditPolicyValues = {
    HiGrid.EDIT_SIZE_TO_CELL, 
    HiGrid.EDIT_ENSURE_MINIMUM_SIZE, 
    HiGrid.EDIT_ENSURE_PREFERRED_SIZE,
};

/** String names of the cell edit policy. */
public final static String[] cellEditPolicyStrings = {
    LocaleBundle.EDIT_SIZE_TO_CELL, 
    LocaleBundle.EDIT_ENSURE_MINIMUM_SIZE, 
    LocaleBundle.EDIT_ENSURE_PREFERRED_SIZE,
};

/**
 * Summary column type value that corresponds to the list of String names.
 * @see #getSummaryColumnTypeStrings
 */
public final static int[] summaryColumnTypeValues = {
    SummaryColumn.COLUMN_TYPE_LABEL, 
    SummaryColumn.COLUMN_TYPE_DATASOURCE, 
    SummaryColumn.COLUMN_TYPE_AGGREGATE, 
    SummaryColumn.COLUMN_TYPE_UNBOUND, 
};

/** String names of the summary column types. */
public final static String[] summaryColumnTypeStrings = {
    LocaleBundle.COLUMN_TYPE_LABEL, 
    LocaleBundle.COLUMN_TYPE_DATASOURCE, 
    LocaleBundle.COLUMN_TYPE_AGGREGATE, 
    LocaleBundle.COLUMN_TYPE_UNBOUND, 
};

/**
 * Aggregate type value that corresponds to the list of String names.
 * @see #getAggregateTypeStrings
 */
public final static int[] aggregateTypeValues = {
    SummaryColumn.AGGREGATE_TYPE_NONE, 
    SummaryColumn.AGGREGATE_TYPE_COUNT, 
    SummaryColumn.AGGREGATE_TYPE_SUM, 
    SummaryColumn.AGGREGATE_TYPE_AVERAGE, 
    SummaryColumn.AGGREGATE_TYPE_MIN, 
    SummaryColumn.AGGREGATE_TYPE_MAX, 
    SummaryColumn.AGGREGATE_TYPE_FIRST, 
    SummaryColumn.AGGREGATE_TYPE_LAST, 
};

/** String names of the aggregate types. */
public final static String[] aggregateTypeStrings = {
    LocaleBundle.AGGREGATE_TYPE_NONE, 
    LocaleBundle.AGGREGATE_TYPE_COUNT, 
    LocaleBundle.AGGREGATE_TYPE_SUM, 
    LocaleBundle.AGGREGATE_TYPE_AVERAGE, 
    LocaleBundle.AGGREGATE_TYPE_MIN, 
    LocaleBundle.AGGREGATE_TYPE_MAX, 
    LocaleBundle.AGGREGATE_TYPE_FIRST, 
    LocaleBundle.AGGREGATE_TYPE_LAST, 
}; 


/**
 * Grid scrollbar display values that corresponds to the list of String names.
 * @see #getGridScrollbarDisplayStrings
 */
public final static int[] gridScrollbarDisplayValues = {
    GridScrollbar.DISPLAY_AS_NEEDED,
    GridScrollbar.DISPLAY_ALWAYS,
    GridScrollbar.DISPLAY_NEVER,
};

public final static String[] gridScrollbarDisplayStrings = {
    LocaleBundle.DISPLAY_AS_NEEDED,
    LocaleBundle.DISPLAY_ALWAYS,
    LocaleBundle.DISPLAY_NEVER,
}; 

/**
 * Folder icon style values that corresponds to the list of String names.
 * @see #getFolderIconStyleStrings
 */
public final static int[] folderIconStyleValues = {
    FolderIconStyle.FOLDER_ICON_STYLE_SHORTCUT,
    FolderIconStyle.FOLDER_ICON_STYLE_FOLDER,
    FolderIconStyle.FOLDER_ICON_STYLE_TRIANGLE,
    FolderIconStyle.FOLDER_ICON_STYLE_SMALL_LINE_3D,
    FolderIconStyle.FOLDER_ICON_STYLE_MEDIUM_LINE_3D,
    FolderIconStyle.FOLDER_ICON_STYLE_LARGE_LINE_3D,
};

public final static String[] folderIconStyleStrings = {
    LocaleBundle.FOLDER_ICON_STYLE_SHORTCUT,
    LocaleBundle.FOLDER_ICON_STYLE_FOLDER,
    LocaleBundle.FOLDER_ICON_STYLE_TRIANGLE,
    LocaleBundle.FOLDER_ICON_STYLE_SMALL_LINE_3D,
    LocaleBundle.FOLDER_ICON_STYLE_MEDIUM_LINE_3D,
    LocaleBundle.FOLDER_ICON_STYLE_LARGE_LINE_3D,
};

/**
 * Sort data direction value that corresponds to the list of String names.
 * @see #getSortDataDirectionStrings
 */
public final static int[] sortDataDirectionValues = {
    SortGrid.UNKNOWN, 
    SortGrid.ASCENDING, 
    SortGrid.DESCENDING,
};

/** String names of the sort data direction. */
public final static String[] sortDataDirectionStrings = {
    LocaleBundle.sortDataDirectionNone, 
    LocaleBundle.sortDataDirectionAscending, 
    LocaleBundle.sortDataDirectionDescending,
};

/**
 * Cell clip hints value that corresponds to the list of String names.
 * @see #getCellClipHintsStrings
 */
public final static int[] cellClipHintsValues = {
    JCCellInfo.SHOW_NONE, 
    JCCellInfo.SHOW_HORIZONTAL, 
    JCCellInfo.SHOW_VERTICAL, 
    JCCellInfo.SHOW_ALL, 
};

/** String names of the cell clip hints. */
public final static String[] cellClipHintsStrings = {
    LocaleBundle.clipHintsShowNone, 
    LocaleBundle.clipHintsShowHorizontal, 
    LocaleBundle.clipHintsShowVertical,
    LocaleBundle.clipHintsShowAll,
};

protected static void getParams(JCHiGrid comp) {
    JCTypeConverter conv = comp.conv;

    try {
        comp.setGridProperties(comp.getParam("GridProperties"), 
                               comp.applet.getClass().getClassLoader());
    }
    catch (DataModelException ex) {
        ex.printStackTrace();
    }
    comp.setAllowPopupMenu(conv.toBoolean(comp.getParam("allowPopupMenu"), true));
    comp.setAllowRowSelection(conv.toBoolean(comp.getParam("allowRowSelection"), true));
    comp.setAllowSorting(conv.toBoolean(comp.getParam("allowSorting"), true));
    comp.setBatched(conv.toBoolean(comp.getParam("batched"), false));
    comp.setBeepOnInvalid(conv.toBoolean(comp.getParam("beepOnInvalid"), false));
    comp.setConnectionsVisible(conv.toBoolean(comp.getParam("connectionsVisible"), true));
    comp.setEditStatusWidth(conv.toInt(comp.getParam("editStatusWidth"), 20));
    comp.setEditable(conv.toBoolean(comp.getParam("editable"), true));
    comp.setFolderIconStyle(
                           toFolderIconStyle(conv,
                                             comp.getParam("folderIconStyle"), 
                                             FolderIconStyle.FOLDER_ICON_STYLE_SHORTCUT));
    comp.setHorizontalScrollbarDisplay(
                                      toGridScrollbarDisplay(conv, comp.getParam("horizontalScrollbarDisplay"), 
                                                             GridScrollbar.DISPLAY_AS_NEEDED));
    comp.setLevelIndented(conv.toBoolean(comp.getParam("levelIndented"), true));
    comp.setNodeWidth(conv.toInt(comp.getParam("nodeWidth"), HiGrid.INITIAL_NODE_WIDTH));
    comp.setPrintFoldersAndConnections(
                                      conv.toBoolean(comp.getParam("printFoldersAndConnections"), false));
    comp.setPrintFormat(
                       toPrintFormat(conv,
                                     comp.getParam("printFormat"), 
                                     HiGrid.PRINT_AS_DISPLAYED));
    comp.setRowSelectionMode(
                            toRowSelectionMode(conv,
                                               comp.getParam("rowSelectionMode"), 
                                               HiGrid.ROW_SELECT_ANY));
    comp.setRowtipVisible(conv.toBoolean(comp.getParam("rowtipVisible"), true));
    comp.setSortIconsVisible(conv.toBoolean(comp.getParam("sortIconsVisible"), true));
    comp.setVerticalScrollbarDisplay(
                                    toGridScrollbarDisplay(conv,
                                                           comp.getParam("verticalScrollbarDisplay"), 
                                                           GridScrollbar.DISPLAY_AS_NEEDED));
}

/** Gets an internationalized array of row selection mode Strings.
 * @return
 */
public static String[] getRowSelectionModeStrings() {
    String ss[] = new String[rowSelectionModeStrings.length];
    for (int i=0; i<ss.length; i++) {
        ss[i] = LocaleBundle.string(rowSelectionModeStrings[i]);
    }
    return ss;
}

/** Converts a String to a row selection mode value.
 * @param conv
 * @param def
 * @param s
 * @return
 */
public static int
toRowSelectionMode(JCTypeConverter conv, String s, int def) {
    return conv.toEnum(s, "rowSelectionMode", getRowSelectionModeStrings(), 
                       rowSelectionModeValues, def);
}

/** Checks row selection mode enum value.
 * @param v
 */
public static void checkRowSelectionMode(int v) {
    JCTypeConverter.checkEnum(v, "rowSelectionMode", rowSelectionModeValues);
}


/** Gets an internationalized array of print format Strings.
 * @return
 */
public static String[] getPrintFormatStrings() {
    String ss[] = new String[printFormatStrings.length];
    for (int i=0; i<ss.length; i++) {
        ss[i] = LocaleBundle.string(printFormatStrings[i]);
    }
    return ss;
}

/** Converts a String to a print format value.
 * @param conv
 * @param def
 * @param s
 * @return
 */
public static int
toPrintFormat(JCTypeConverter conv, String s, int def) {
    return conv.toEnum(s, "printFormat", getPrintFormatStrings(), 
                       printFormatValues, def);
}

/** Checks print format enum value.
 * @param v
 */
public static void checkPrintFormat(int v) {
    JCTypeConverter.checkEnum(v, "printFormat", printFormatValues);
}

/** Gets an internationalized array of auto show Strings.
 * @return
 */
public static String[] getAutoTraverseStrings() {
    String ss[] = new String[autoTraverseStrings.length];
    for (int i=0; i<ss.length; i++) {
        ss[i] = LocaleBundle.string(autoTraverseStrings[i]);
    }
    return ss;
}

/** Converts a String to an auto show value.
 * @return
 * @param conv
 * @param def
 * @param s
 */
public static int
toAutoTraverse(JCTypeConverter conv, String s, int def) {
    return conv.toEnum(s, "autoTraverse", getAutoTraverseStrings(), 
                       autoTraverseValues, def);
}

/** Checks auto show enum value.
 * @param v
 */
public static void checkAutoTraverse(int v) {
    JCTypeConverter.checkEnum(v, "autoTraverse", autoTraverseValues);
}

/** Gets an internationalized array of row format border style Strings.
 * @return
 */
public static String[] getRowFormatBorderStyleStrings() {
    String ss[] = new String[rowFormatBorderStyleStrings.length];
    for (int i=0; i<ss.length; i++) {
        ss[i] = LocaleBundle.string(rowFormatBorderStyleStrings[i]);
    }
    return ss;
}

/** Converts a String to a row format border style value.
 * @param conv
 * @param def
 * @param s
 * @return
 */
public static int
toRowFormatBorderStyle(JCTypeConverter conv, String s, int def) {
    return conv.toEnum(s, "rowFormatBorderStyle", getRowFormatBorderStyleStrings(), 
                       rowFormatBorderStyleValues, def);
}

/** Checks row format border style enum value.
 * @param v
 */
public static void checkRowFormatBorderStyle(int v) {
    JCTypeConverter.checkEnum(v, "rowFormatBorderStyle", rowFormatBorderStyleValues);
}

/** Gets an internationalized array of cell horizontal alignment Strings.
 * @return
 */
public static String[] getCellHorizontalAlignmentStrings() {
    String ss[] = new String[cellHorizontalAlignmentStrings.length];
    for (int i=0; i<ss.length; i++) {
        ss[i] = LocaleBundle.string(cellHorizontalAlignmentStrings[i]);
    }
    return ss;
}

/** Converts a String to a cell horizontal alignment value.
 * @param conv
 * @param def
 * @param s
 * @return
 */
public static int toCellHorizontalAlignment(JCTypeConverter conv, String s, int def) {
    return conv.toEnum(s, "cellHorizontalAlignment", getCellHorizontalAlignmentStrings(), 
                       cellHorizontalAlignmentValues, def);
}

/** Checks cell horizontal alignment enum value.
 * @param v
 */
public static void checkCellHorizontalAlignment(int v) {
    JCTypeConverter.checkEnum(v, "cellHorizontalAlignment", cellHorizontalAlignmentValues);
}

/** Gets an internationalized array of cell vertical alignment Strings.
 * @return
 */
public static String[] getCellVerticalAlignmentStrings() {
    String ss[] = new String[cellVerticalAlignmentStrings.length];
    for (int i=0; i<ss.length; i++) {
        ss[i] = LocaleBundle.string(cellVerticalAlignmentStrings[i]);
    }
    return ss;
}

/** Converts a String to a cell vertical alignment value.
 * @param conv
 * @param def
 * @param s
 * @return
 */
public static int toCellVerticalAlignment(JCTypeConverter conv, String s, int def) {
    return conv.toEnum(s, "cellVerticalAlignment", getCellVerticalAlignmentStrings(), 
                       cellVerticalAlignmentValues, def);
}

/** Checks cell vertical alignment enum value.
 * @param v
 */
public static void checkCellVerticalAlignment(int v) {
    JCTypeConverter.checkEnum(v, "cellVerticalAlignment", cellVerticalAlignmentValues);
}

/** Gets an internationalized array of meta data value type Strings.
 * @return
 */
public static String[] getMetaDataModelTypeStrings() {
    String ss[] = new String[metaDataModelTypeStrings.length];
    for (int i=0; i<ss.length; i++) {
        ss[i] = LocaleBundle.string(metaDataModelTypeStrings[i]);
    }
    return ss;
}

/** Converts a String to a meta data value type.
 * @param conv
 * @param def
 * @param s
 * @return
 */
public static int toMetaDataModelType(JCTypeConverter conv, String s, int def) {
    return conv.toEnum(s, "metaDataModelType", getMetaDataModelTypeStrings(), 
                       metaDataModelTypeValues, def);
}

/** Checks meta data value type enum value.
 * @param v
 */
public static void checkMetaDataModelType(int v) {
    JCTypeConverter.checkEnum(v, "metaDataModelType", metaDataModelTypeValues);
}

/** Gets an internationalized array of cell edit policy Strings.
 * @return
 */
public static String[] getCellEditPolicyStrings() {
    String ss[] = new String[cellEditPolicyStrings.length];
    for (int i=0; i<ss.length; i++) {
        ss[i] = LocaleBundle.string(cellEditPolicyStrings[i]);
    }
    return ss;
}

/** Converts a String to a cell edit policy value.
 * @param conv
 * @param def
 * @param s
 * @return
 */
public static int toCellEditPolicy(JCTypeConverter conv, String s, int def) {
    return conv.toEnum(s, "cellEditPolicy", getCellEditPolicyStrings(), 
                       cellEditPolicyValues, def);
}

/** Checks cell edit policy enum value.
 * @param v
 */
public static void checkCellEditPolicy(int v) {
    JCTypeConverter.checkEnum(v, "cellEditPolicy", cellEditPolicyValues);
}

/** Gets an internationalized array of summary column type Strings.
 * @return
 */
public static String[] getSummaryColumnTypeStrings() {
    String ss[] = new String[summaryColumnTypeStrings.length];
    for (int i=0; i<ss.length; i++) {
        ss[i] = LocaleBundle.string(summaryColumnTypeStrings[i]);
    }
    return ss;
}

/** Converts a String to a summary column type value.
 * @param conv
 * @param def
 * @param s
 * @return
 */
public static int toSummaryColumnType(JCTypeConverter conv, String s, int def) {
    return conv.toEnum(s, "summaryColumnType", getSummaryColumnTypeStrings(), 
                       summaryColumnTypeValues, def);
}

/** Checks summary column type enum value.
 * @param v
 */
public static void checkSummaryColumnType(int v) {
    JCTypeConverter.checkEnum(v, "summaryColumnType", summaryColumnTypeValues);
}

/** Gets an internationalized array of aggregate type Strings.
 * @return
 */
public static String[] getAggregateTypeStrings() {
    String ss[] = new String[aggregateTypeStrings.length];
    for (int i=0; i<ss.length; i++) {
        ss[i] = LocaleBundle.string(aggregateTypeStrings[i]);
    }
    return ss;
}

/** Converts a String to a aggregate type value.
 * @param conv
 * @param def
 * @param s
 * @return
 */
public static int toAggregateType(JCTypeConverter conv, String s, int def) {
    return conv.toEnum(s, "aggregateType", getAggregateTypeStrings(), 
                       aggregateTypeValues, def);
}

/** Checks aggregate type enum value.
 * @param v
 */
public static void checkAggregateType(int v) {
    JCTypeConverter.checkEnum(v, "aggregateType", aggregateTypeValues);
}

/** Gets an internationalized array of grid scrollbar display Strings.
 * @return
 */
public static String[] getGridScrollbarDisplayStrings() {
    String ss[] = new String[gridScrollbarDisplayStrings.length];
    for (int i=0; i<ss.length; i++) {
        ss[i] = LocaleBundle.string(gridScrollbarDisplayStrings[i]);
    }
    return ss;
}

/** Converts a String to a grid scrollbar display value.
 * @param conv
 * @param def
 * @param s
 * @return
 */
public static int toGridScrollbarDisplay(JCTypeConverter conv, String s, int def) {
    return conv.toEnum(s, "gridScrollbarDisplay", getGridScrollbarDisplayStrings(), 
                       gridScrollbarDisplayValues, def);
}

/** Checks grid scrollbar display enum value.
 * @param v
 */
public static void checkGridScrollbarDisplay(int v) {
    JCTypeConverter.checkEnum(v, "gridScrollbarDisplay", gridScrollbarDisplayValues);
}

/** Gets an internationalized array of grid scrollbar display Strings.
 * @return
 */
public static String[] getFolderIconStyleStrings() {
    String ss[] = new String[folderIconStyleStrings.length];
    for (int i=0; i<ss.length; i++) {
        ss[i] = LocaleBundle.string(folderIconStyleStrings[i]);
    }
    return ss;
}

/** Converts a String to a grid scrollbar display value.
 * @param conv
 * @param def
 * @param s
 * @return
 */
public static int toFolderIconStyle(JCTypeConverter conv, String s, int def) {
    return conv.toEnum(s, "folderIconStyle", getFolderIconStyleStrings(), 
                       folderIconStyleValues, def);
}

/** Checks grid scrollbar display enum value.
 * @param v
 */
public static void checkFolderIconStyle(int v) {
    JCTypeConverter.checkEnum(v, "folderIconStyle", folderIconStyleValues);
}

/** Gets an internationalized array of sort data direction Strings.
 * @return
 */
public static String[] getSortDataDirectionStrings() {
    String ss[] = new String[sortDataDirectionStrings.length];
    for (int i=0; i<ss.length; i++) {
        ss[i] = LocaleBundle.string(sortDataDirectionStrings[i]);
    }
    return ss;
}

/** Converts a String to a sort data direction value.
 * @param conv
 * @param def
 * @param s
 * @return
 */
public static int toSortDataDirection(JCTypeConverter conv, String s, int def) {
    return conv.toEnum(s, "sortDataDirection", getSortDataDirectionStrings(), 
                       sortDataDirectionValues, def);
}

/** Checks sort data direction enum value.
 * @param v
 */
public static void checkSortDataDirection(int v) {
    JCTypeConverter.checkEnum(v, "sortDataDirection", sortDataDirectionValues);
}

/** Gets an internationalized array of cell clip hints Strings.
 * @return
 */
public static String[] getCellClipHintsStrings() {
    String ss[] = new String[cellClipHintsStrings.length];
    for (int i=0; i<ss.length; i++) {
        ss[i] = LocaleBundle.string(cellClipHintsStrings[i]);
    }
    return ss;
}

/** Converts a String to a cell clip hints value.
 * @param conv
 * @param def
 * @param s
 * @return
 */
public static int toCellClipHints(JCTypeConverter conv, String s, int def) {
    return conv.toEnum(s, "cellClipHints", getCellClipHintsStrings(), 
                       cellClipHintsValues, def);
}

/** Checks cell clip hints enum value.
 * @param v
 */
public static void checkCellClipHints(int v) {
    JCTypeConverter.checkEnum(v, "cellClipHints", cellClipHintsValues);
}

}
