/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCHiGridNode.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:21 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid.beans;

import java.awt.*;
import java.io.Serializable;
import java.util.*;

import com.klg.jclass.datasource.*;
import com.klg.jclass.datasource.beans.*;
import com.klg.jclass.datasource.customizer.NodePropertiesWrapper;
import com.klg.jclass.higrid.*;
import com.klg.jclass.util.JCListenerList;

/** 
 *	This class encapsulates information about data and format of a node in <code>JCHiGrid</code>.
 */
public class JCHiGridNode implements JCDataListener, NodePropertiesWrapper,
    Serializable {

static final long serialVersionUID = 5262368671112968621L;

public static final int SECTION_HEADER = 0;
public static final int SECTION_BEFORE_DETAILS = 1;
public static final int SECTION_RECORD = 2;
public static final int SECTION_AFTER_DETAILS = 3;
public static final int SECTION_FOOTER = 4;
public static final int SECTION_MAX = SECTION_FOOTER + 1;

private NodeProperties dataComp;
private GridProperties higridComp;

private Vector sections;

private Vector summaryColumns;

private int autoTraverse;

private SortData defaultSortData;

/** Used when no cell exists in a row format.
 */
private Vector emptyRowFormats;

/** Event listeners. */
private transient JCListenerList JCHiGridListeners = null;
private transient boolean isFiringJCHiGridEvent = false;

private transient boolean doRetrieveAllFields = false;

private static final boolean TRACE = false;

/*
private JCHiGrid bean = null;
                                    
public JCHiGridNode(Object bean, GridProperties comp) {
this(bean, comp, comp.getDataSourceType());
}

public JCHiGridNode(Object bean, GridProperties comp, int type) {
this.bean = (JCHiGrid)bean;
higridComp = comp;
dataComp = new NodeProperties(type, null);
dataComp.addJCDataListener(this);
initFormat();
}
*/

public JCHiGridNode(GridProperties comp) {
    this(comp, comp.getDataSourceType());
}

/** Creates a <code>JCHiGridNode</code> with the data source type.
 * @param comp
 * @param type
 */
public JCHiGridNode(GridProperties comp, int type) {
    higridComp = comp;
    dataComp = new NodeProperties(type, null);
    dataComp.addJCDataListener(this);
    initFormat();
}

/** Initializes format.
 */
public void initFormat() {
    sections = new Vector(SECTION_MAX);
    sections.addElement(new HeaderFormat());
    sections.addElement(new BeforeDetailsFormat());
    sections.addElement(new RecordFormat());
    sections.addElement(new AfterDetailsFormat());
    sections.addElement(new FooterFormat());

//    getRowFormat(SECTION_BEFORE_DETAILS).createIndicatorFormats(bean);
//    getRowFormat(SECTION_AFTER_DETAILS).createIndicatorFormats(bean);
//    getRowFormat(SECTION_FOOTER).createIndicatorFormats(bean);
    getRowFormat(SECTION_BEFORE_DETAILS).createIndicatorFormats(null);
    getRowFormat(SECTION_AFTER_DETAILS).createIndicatorFormats(null);
    getRowFormat(SECTION_FOOTER).createIndicatorFormats(null);

    summaryColumns = new Vector(SECTION_MAX);
    emptyRowFormats = new Vector(SECTION_MAX);
    for (int i = 0; i < SECTION_MAX; i++) {
        summaryColumns.addElement(new Vector(10));
        emptyRowFormats.addElement(new EmptyRowFormat());
    }
}

public NodeProperties getNodeProperties() {
    return dataComp;
}

public GridProperties getGridProperties() {
    return higridComp;
}

public boolean isSectionShowing(int section) {
    return getRowFormat(section).isShowing();
}

public void setSectionShowing(int section, boolean v) {
    if (getRowFormat(section).isShowing() == v) return;

    getRowFormat(section).setShowing(v);
    setChanged();
}

////////////////////////////////////////////////////////////////
//
// Section level properties
//

public void setHeight(int section, int height) {
    if (getRowFormat(section).getHeight() == height) return;

    getRowFormat(section).setHeight(height);
    setChanged();
}
public int getHeight(int section) {
    return getRowFormat(section).getHeight();
}

public void setAllowHeightResizing(int section, boolean v) {
    if (getRowFormat(section).getAllowHeightSizing() == v) return;

    getRowFormat(section).setAllowHeightSizing(v);
    setChanged();
}
public boolean getAllowHeightResizing(int section) {
    return getRowFormat(section).getAllowHeightSizing();
}

public void setBorderStyle(int section, int style) {
    if (getRowFormat(section).getBorderStyle() == style ||
        style == RowFormat.MIXED_BORDER_STYLES) return;

    ((EmptyRowFormat)emptyRowFormats.elementAt(section)).borderStyle = style;
    getRowFormat(section).setBorderStyle(style);
    setChanged();
}
public int getBorderStyle(int section) {
    // even when no cell formats exists, we still
    // want to return the user selected style.
    return(getRowFormat(section).getDataFormats().size() != 0) ?
    getRowFormat(section).getBorderStyle() :
    ((EmptyRowFormat)emptyRowFormats.elementAt(section)).borderStyle;
}

public void setBorderInsets(int section, Insets i) {
    if (getRowFormat(section).getBorderInsets().equals(i)) return;

    ((EmptyRowFormat)emptyRowFormats.elementAt(section)).borderInsets = new Insets(i.top, i.left, i.bottom, i.right);
    getRowFormat(section).setBorderInsets(i);
    setChanged();
}

public Insets getBorderInsets(int section) {
    // even when no cell formats exists, we still
    // want to return the user selected insets
    return(getRowFormat(section).getDataFormats().size() != 0) ?
    getRowFormat(section).getBorderInsets() :
    ((EmptyRowFormat)emptyRowFormats.elementAt(section)).borderInsets;   
}

public void setForeground(int section, Color color) {
    Color prevColor = getRowFormat(section).getForeground();
    if (prevColor != null && prevColor.equals(color)) {
        return;
    }
    if (color == null) {
        color = Color.black;
    }
    ((EmptyRowFormat)emptyRowFormats.elementAt(section)).foreground = color;
    getRowFormat(section).setForeground(color);
    setChanged();
}

public Color getForeground(int section) {
    // even when no cell formats exists, we still
    // want to return the user selected foreground color
    return(getRowFormat(section).getDataFormats().size() != 0) ?
    getRowFormat(section).getForeground() :
    ((EmptyRowFormat)emptyRowFormats.elementAt(section)).foreground; 
}

public void setBackground(int section, Color color) {
    Color prevColor = getRowFormat(section).getBackground();
    if (prevColor != null && prevColor.equals(color)) {
        return;
    }
    if (color == null) {
        color = Color.lightGray;
    }
    ((EmptyRowFormat)emptyRowFormats.elementAt(section)).background = color;
    getRowFormat(section).setBackground(color);
    setChanged();
}

public Color getBackground(int section) {
    // even when no cell formats exists, we still
    // want to return the user selected background color
    return(getRowFormat(section).getDataFormats().size() != 0) ?
    getRowFormat(section).getBackground() :
    ((EmptyRowFormat)emptyRowFormats.elementAt(section)).background; 
}

public void setFont(int section, String name, int style, int size, FontMetrics fm) {
    String prevFontName = getRowFormat(section).getFontName();
    int prevFontStyle = getRowFormat(section).getFontStyle();
    int prevFontSize = getRowFormat(section).getFontSize();
    if (prevFontName != null && prevFontName.equals(name) &&
        prevFontStyle != RowFormat.MIXED_STYLE && prevFontStyle == style &&
        prevFontSize != RowFormat.MIXED_SIZE && prevFontSize == size) {
        return;
    }
    ((EmptyRowFormat)emptyRowFormats.elementAt(section)).fontName = 
    (name == null) ? new String("serif") : name;
    ((EmptyRowFormat)emptyRowFormats.elementAt(section)).fontStyle = 
    (style == RowFormat.MIXED_STYLE) ? Font.PLAIN : style;
    ((EmptyRowFormat)emptyRowFormats.elementAt(section)).fontSize = 
    (size == RowFormat.MIXED_SIZE) ? 12 : size;
    getRowFormat(section).setFont(name, style, size, fm);
    setChanged();
}

// even when no cell formats exists, we still
// want to return the user selected font values
public String getFontName(int section) {
    return(getRowFormat(section).getDataFormats().size() != 0) ?
    getRowFormat(section).getFontName() :
    ((EmptyRowFormat)emptyRowFormats.elementAt(section)).fontName;   
}

public int getFontStyle(int section) {
    return(getRowFormat(section).getDataFormats().size() != 0) ?
    getRowFormat(section).getFontStyle() :
    ((EmptyRowFormat)emptyRowFormats.elementAt(section)).fontStyle;  
}

public int getFontSize(int section) {
    return(getRowFormat(section).getDataFormats().size() != 0) ?
    getRowFormat(section).getFontSize() :
    ((EmptyRowFormat)emptyRowFormats.elementAt(section)).fontSize;   
}

public void setAutoTraverse(int v) {
    if (autoTraverse == v) {
        return;
    }
    autoTraverse = v;
    setChanged();
}

public int getAutoTraverse() {
    return autoTraverse;
}

public void setDefaultSortData(SortData sortData) {
    if (defaultSortData == null && sortData == null) {
        return;
    }
    if (defaultSortData != null && sortData != null &&
        defaultSortData.equals(sortData)) {
        return;
    }
    defaultSortData = new SortData(sortData);;
    setChanged();
}

public SortData getDefaultSortData() {
    if (defaultSortData == null) {
        return null;
    }
    return defaultSortData;
}

public void setRepeatHeader(boolean v) {
    HeaderFormat header = (HeaderFormat) getRowFormat(SECTION_HEADER);
    if (header.isRepeatHeader() == v) return;

    header.setRepeatHeader(v);
    setChanged();
}

public boolean isRepeatHeader() {
    return((HeaderFormat) getRowFormat(SECTION_HEADER)).isRepeatHeader();
}

public RowFormat getRowFormat(int section) {
    return(RowFormat) sections.elementAt(section);
}

public void setRowFormat(int section, RowFormat format) {
    sections.setElementAt(format, section);
    setChanged();
}

public Vector getSummaryColumns(int section) {
    return(Vector) summaryColumns.elementAt(section);
}

public void setSummaryColumns(int section, Vector columns) {
    summaryColumns.setElementAt(columns, section);
    setChanged();
}

/** Exposes <code>setChanged()</code>. It is used when a <code>CellFormat</code> in
 *	a <code>RowFormat</code> is changed.
 */
public void setChanged() {
    if (higridComp != null) {
        higridComp.setChanged();
    }
}

/** Compares all properties of <code>JCHiGridNode</code> to check equality.
 *	It is done in this way because of serialization.  After serialization,
 *	a <code>JCHiGridNode</code> is stored in different memory space than
 *	the <code>JCHiGridNode</code> before serialization.
 * @return
 * @param o
 */
public boolean equals(Object o) {
    if (!(o instanceof JCHiGridNode)) return false;

    JCHiGridNode node = (JCHiGridNode) o;
    boolean v = node.getNodeProperties().equals(this.getNodeProperties());
    if (!v) return v;

    // Check level level properties.
    for (int i=0; i<SECTION_MAX; i++) {
        v = this.getHeight(i) == node.getHeight(i) &&
            this.getAllowHeightResizing(i) == node.getAllowHeightResizing(i) &&
            this.getBorderStyle(i) == node.getBorderStyle(i) &&
            this.getBorderInsets(i).equals(node.getBorderInsets(i));
        if (!v) break;
    }
    if (!v) return v;

    v = this.isRepeatHeader() == node.isRepeatHeader();

    // assume cell level properties are the same. FIX

    return v;
}

public boolean getDoRetrieveAllFields() {
    return doRetrieveAllFields;
}

public void setDoRetrieveAllFields(boolean value) {
    doRetrieveAllFields = value;
}

/**
 * Adds the specified listener to receive <code>JCHiGridEvent</code> objects.
 *
 * @param listener a listener
 */
public synchronized void addJCHiGridListener(JCHiGridListener listener) {
    JCHiGridListeners = JCListenerList.add(JCHiGridListeners, listener);
}

/**
 * Removes the specified listener so it no longer receives <code>JCHiGridEvent</code> objects.
 *
 * @param listener a listener
 */
public synchronized void removeJCHiGridListener(JCHiGridListener listener) {
    JCHiGridListeners = JCListenerList.remove(JCHiGridListeners, listener);
}

/**
 * Informs registered listeners of a <code>JCHiGridEvent</code>.
 *
 * @param event the <code>JCHiGridEvent</code>
 */
public void fireJCHiGridEvent(JCHiGridEvent event) {
    if (isFiringJCHiGridEvent) {
        // avoid recursive calls
        return;
    }
    isFiringJCHiGridEvent = true;
    // do the GridProperties listener first
    Enumeration e = JCListenerList.elements(JCHiGridListeners);
    while (e.hasMoreElements()) {
        JCHiGridListener listener = (JCHiGridListener)e.nextElement();
        if (listener instanceof GridProperties) {
            listener.JCHiGridValueChanged(event);
            break;
        }
    }
    e = JCListenerList.elements(JCHiGridListeners);
    while (e.hasMoreElements()) {
        JCHiGridListener listener = (JCHiGridListener)e.nextElement();
        if (!(listener instanceof GridProperties)) {
            listener.JCHiGridValueChanged(event);
        }
    }
    isFiringJCHiGridEvent = false;
}

/** When value(s) of the data Bean component is changed.
 * @param e
 */
public void JCDataValueChanged(JCDataEvent e) {
    // mark component as changed
    setChanged();
    // pass it on as a JCHiGridEvent
    fireJCHiGridEvent(new JCHiGridEvent((Object)this, 
                                        JCHiGridEvent.DATA_BEAN_COMPONENT_VALUE, this, (NodeProperties) e.getSource()));
}

}

/** Helper class for empty row format.  
 *	Used when no cell exists in a row format.
 */
class EmptyRowFormat implements Serializable {

static final long serialVersionUID = 2533449451599629360L;

int borderStyle = HiGridBorder.ETCHED_IN;
Insets borderInsets = new Insets(2,2,2,2);
Color foreground = Color.black;
Color background = Color.lightGray;
String fontName = new String("serif");
int fontStyle = Font.PLAIN;
int fontSize = 12;

EmptyRowFormat() {
}

}
