/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AbstractSectionPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:21 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid.customizer;

import java.awt.*;

import javax.swing.JTabbedPane;

import com.klg.jclass.higrid.*;
import com.klg.jclass.higrid.beans.JCHiGridNode;

/**
 * 	The UI portion of each section page in GridProperties editor.  
 *	Sections are header, before details, details, after details and footer.
 *	Each section has a Level and a Cell page.
 */
public class AbstractSectionPage extends FormatPage {

private static final boolean TRACE = false;

public static final int LEVEL_PAGE = 0;
public static final int CELL_PAGE = 1;
public static final int INDICATOR_PAGE = 2;
public static final int MAX_PAGE = INDICATOR_PAGE+1;

JTabbedPane tabManager;

FormatPage pages[] = {
    new BaseLevelPage(JCHiGridNode.SECTION_HEADER),
    new BaseCellPage(JCHiGridNode.SECTION_HEADER),
    new BaseEditStatusPage(JCHiGridNode.SECTION_HEADER),
};
static String pageNames[] = {
    LocaleBundle.level,
    LocaleBundle.cell,
    LocaleBundle.editStatusIndicator,
};

int section = JCHiGridNode.SECTION_HEADER;

public AbstractSectionPage(String title, int section) {
    super(title);
    this.section = section;
    init();
}

public void init() {
    setLayout(new GridLayout(1, 1));
    // Create the tab manager
    if (tabManager == null) {
        tabManager = new JTabbedPane();
        for (int i=0; i<MAX_PAGE; i++) {
            tabManager.addTab(LocaleBundle.string(pageNames[i]), pages[i]);
        }
    }
    add(tabManager);
    com.klg.jclass.datasource.customizer.CustomizerLauncher.setDefaultFont((Component)this, new Font("Dialog", Font.PLAIN, 9));
}

public void setEditor(GridPropertiesEditor editor) {
    super.setEditor(editor);
    if (tabManager != null) {
        BaseCellPage page = (BaseCellPage) tabManager.getComponentAt(CELL_PAGE);
        page.setEditor(editor);
    }
}

public void setJCHiGridNode(JCHiGridNode node) {
    if (tabManager != null) {
        for (int i=0; i<MAX_PAGE; i++) {
            FormatPage page = (FormatPage) tabManager.getComponentAt(i);
            if (page instanceof BaseLevelPage) {
                ((BaseLevelPage)page).setSection(section);
            }
            else if (page instanceof BaseCellPage) {
                ((BaseCellPage)page).setSection(section);
            }
            else if (page instanceof BaseEditStatusPage) {
                ((BaseEditStatusPage)page).setSection(section);
            }
            page.init(node);
        }
    }
}

public void showValue() {
}

public void setSection(int section) {
    this.section = section;
}

}   // AbstractSectionPage

