/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: BaseCellPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:21 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid.customizer;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;

import com.klg.jclass.datasource.MetaDataModel;
import com.klg.jclass.higrid.*;
import com.klg.jclass.higrid.beans.JCHiGridListener;
import com.klg.jclass.higrid.beans.JCHiGridNode;
import com.klg.jclass.higrid.beans.JCHiGridEvent;


/** It is a page contains properties at the column level.  These properties apply to
 *	one column at the level.  When a <code>JCHiGridNode</code> is set,
 *	showValue() is called to display property values.
 *	When a property value is changed, valueChanged() is called.
 */
public class BaseCellPage extends FormatPage implements ActionListener,
    JCHiGridListener {

int section;

/** Move column up */
private static final int UP = 0;
/** Move column down */
private static final int DOWN = 1;

// common properties
JList columns;
DefaultListModel columnsModel = new DefaultListModel();
JButton buttonAdd, buttonDelete, buttonClear;
JButton up, down;
CellTabs cellTabManager;

public BaseCellPage(int section) {
    super(LocaleBundle.string(LocaleBundle.cell));
    this.section = section;
    init();
}

private boolean isSummaryCellPage() {
    return(section == JCHiGridNode.SECTION_FOOTER ||
           section == JCHiGridNode.SECTION_BEFORE_DETAILS ||
           section == JCHiGridNode.SECTION_AFTER_DETAILS); 
}

/** initialize UI.
 */
public void init() {
    setLayout(new BorderLayout());

    JPanel columnsPanel = new JPanel();
    columns = new JList(columnsModel);
    columns.addListSelectionListener(new ListSelectionListener() {
                                         public void valueChanged(ListSelectionEvent e) {
                                             if (isInitializing()) return;

                                             if (e.getValueIsAdjusting()) {
                                                 return;
                                             }
                                             int index = e.getFirstIndex();
                                             while (index <= e.getLastIndex()) {
                                                 if (columns.isSelectedIndex(index)) {
                                                     updateUpDownButtons();
                                                     cellTabManager.setSelectedColumn(getSelectedCell(), getSelectedSummaryColumn());
                                                     break;
                                                 }
                                                 index++;
                                             }
                                         }
                                     });
    JPanel orderButtonsPanel = new JPanel();
    JPanel p = new JPanel();
    JPanel cellButtonsPanel = new JPanel();
    buttonAdd = new JButton(LocaleBundle.string(LocaleBundle.cellAdd));
    buttonDelete = new JButton(LocaleBundle.string(LocaleBundle.cellDelete));
    buttonClear = new JButton(LocaleBundle.string(LocaleBundle.cellClear));
    // column list
    columnsPanel.setLayout(new BorderLayout());

    up = new javax.swing.plaf.basic.BasicArrowButton(SwingConstants.NORTH);
    down = new javax.swing.plaf.basic.BasicArrowButton(SwingConstants.SOUTH);
    up.addActionListener(this);
    down.addActionListener(this);

    orderButtonsPanel.setLayout(new GridBagLayout());
    GridBagConstraints c = new GridBagConstraints();
    c.fill = GridBagConstraints.NONE;
    c.insets = new Insets(5, 5, 5, 5);
    p.setLayout(new GridLayout(2, 1, 5, 5));
    p.add(up);
    p.add(down);
    orderButtonsPanel.add(p, c);
    columnsPanel.add(new JScrollPane(columns), BorderLayout.CENTER);
    columnsPanel.add(orderButtonsPanel, BorderLayout.EAST);

    cellButtonsPanel.setLayout(new GridBagLayout());
    GridBagConstraints c2 = new GridBagConstraints();
    c2.anchor = GridBagConstraints.NORTH;
    c2.fill = GridBagConstraints.BOTH;
    c2.gridheight = c2.gridwidth = 1;
    c2.weightx = c2.weighty = 1;
    c2.insets = new Insets(5, 0, 0, 5);
    cellButtonsPanel.add(buttonAdd, c2);
    cellButtonsPanel.add(buttonDelete, c2);
    c2.insets = new Insets(5, 0, 0, 0);
    cellButtonsPanel.add(buttonClear, c2);
    columnsPanel.add(cellButtonsPanel, BorderLayout.SOUTH);
    buttonAdd.addActionListener(this);
    buttonDelete.addActionListener(this);
    buttonClear.addActionListener(this);

    setEnabledCellButtons();

    // cell tabs
    cellTabManager = new CellTabs(section);
//FIX - customizer needs to be wider to accomodate both these panels!
    add(columnsPanel, BorderLayout.WEST);
    add(cellTabManager, BorderLayout.EAST);
}

/**
 * Sets the GridPropertiesEditor.
 */
public void setEditor(GridPropertiesEditor editor) {
    super.setEditor(editor);
    if (cellTabManager != null) {
        cellTabManager.setEditor(editor);
    }
}

public void setSection(int section) {
    this.section = section;
    if (cellTabManager != null) {
        cellTabManager.setSection(section);
    }
}

/** Overrides to perform operations necessary after value is changed on this page.
 */
public void valueChanged() {
}

/** Overrides to refresh values displayed on this page after the underlined
 *	<code>JCHiGridNode</code> is changed.
 */
public void showValue() {
    if (section == JCHiGridNode.SECTION_RECORD) {
        showRecordValue();
    }
    String old_selected_column = (String) columns.getSelectedValue();
    columnsModel.removeAllElements();
    RowFormat format = getJCHiGridNode().getRowFormat(section);
    Vector v = format.getDataFormats();

    if (v != null) {
        for (Enumeration e=v.elements(); e.hasMoreElements();) {
            CellFormat cell = (CellFormat) e.nextElement();
            columnsModel.addElement(isSummaryCellPage() ? cell.getText() : cell.getName());
        }
    }

    int size = columnsModel.getSize();
    String[] s = new String[size];
    columnsModel.copyInto(s);
    if (size > 0) {
        int index = 0;
        if (old_selected_column != null && old_selected_column.length() > 0) {
            for (int i=0; i<s.length; i++) {
                if (s[i].equals(old_selected_column)) {
                    index = i;
                    break;
                }
            }
        }
        columns.setSelectedIndex(index);
    }
    else {
        cellTabManager.disableAllTabs();
    }

    setEnabledCellButtons();
    updateUpDownButtons();
}

void showRecordValue() {
    JCHiGridNode node = getJCHiGridNode();
    if (node.getRowFormat(JCHiGridNode.SECTION_RECORD).getDataFormats().size() == 0) {
        editor.resetRecordAndHeader(node, false, true);
        // If record columns are loaded, then notify observers.
        // There are cases where no columns are loaded.  This check is
        // to avoid infinite loop.
        if (node.getRowFormat(JCHiGridNode.SECTION_RECORD).getDataFormats().size() > 0) {
            node.getGridProperties().fireJCHiGridEvent(new JCHiGridEvent(node.getGridProperties(), node));
        }
    }
    else {
        if (node.getDoRetrieveAllFields()) {
            node.setDoRetrieveAllFields(false);
            editor.retrieveAllFields();
            node.getGridProperties().fireJCHiGridEvent(new JCHiGridEvent(node.getGridProperties(), 
                                                                         JCHiGridEvent.HIGRID_BEAN_FIELDS_UPDATE, node, 
                                                                         node.getNodeProperties()));
        }
    }
}

void updateUpDownButtons() {
    int size = columnsModel.getSize();
    // Can't move column in header section, or there is no column to move.
    // For up, can't move up if selected item is the first item
    // For down, can't move up if selected item is the last item
    up.setEnabled(section != JCHiGridNode.SECTION_HEADER &&
                  size > 0 &&
                  columns.getSelectedIndex() != 0);
    down.setEnabled(section != JCHiGridNode.SECTION_HEADER &&
                    size > 0 &&
                    columns.getSelectedIndex() != size-1);
}

/**
 * Retrieves the cell format of the selected column.
 */
CellFormat getSelectedCell() {
    CellFormat selectedCell = null;
    int index = (-1);
    int size = columnsModel.getSize();
    String columnName = (String) columns.getSelectedValue();
    if (size > 0) {
        index = columns.getSelectedIndex();
    }
    if (index == (-1)) {
        return selectedCell;
    }
    RowFormat format = getJCHiGridNode().getRowFormat(section);
    Vector v = format.getDataFormats();
    if (v == null) {
        return selectedCell;
    }

    int countIndex = 0;
    for (Enumeration e = v.elements(); e.hasMoreElements();) {
        CellFormat cell = (CellFormat) e.nextElement();
        if (isSummaryCellPage()) {
            if (countIndex == index) {
                selectedCell = cell;
                break;
            }
            countIndex++;
        }
        else {
            if (columnName.equals(cell.getName())) {
                selectedCell = cell;
                break;
            }
        }
    }
    return selectedCell;
}

/**
 * Retrieves the summary column for the selected column.
 */
SummaryColumn getSelectedSummaryColumn() {
    if (!isSummaryCellPage()) {
        return null;
    }

    SummaryColumn selectedSummaryColumn = null;
    int index = (-1);
    int size = columnsModel.getSize();
    if (size > 0) {
        index = columns.getSelectedIndex();
    }
    if (index == (-1)) {
        return selectedSummaryColumn;
    }
    Vector v = getJCHiGridNode().getSummaryColumns(section);
    if (v == null) {
        return selectedSummaryColumn;
    }

    int countIndex = 0;
    for (Enumeration e = v.elements(); e.hasMoreElements();) {
        SummaryColumn summaryColumn = (SummaryColumn) e.nextElement();
        if (countIndex == index) {
            selectedSummaryColumn = summaryColumn;
            break;
        }
        countIndex++;
    }
    return selectedSummaryColumn;
}

/** Loads a <code>JCHiGridNode</code> into the page.
 */
public void setJCHiGridNode(JCHiGridNode node) {
    showValue();
    cellTabManager.setJCHiGridNode(node, getSelectedCell(), getSelectedSummaryColumn());
    node.removeJCHiGridListener(this);
    node.addJCHiGridListener(this);
}

/** Calls showValue() to determine if changes are made.
 */
public void actionPerformed(ActionEvent e) {
    if (isInitializing()) return;

    if (e.getSource() instanceof JButton) {
        JButton button = (JButton) e.getSource();
        if (button.equals(buttonAdd)) {
            addSummaryColumn();
            cellTabManager.setSelectedColumn(getSelectedCell(), getSelectedSummaryColumn());
        }
        else if (button.equals(buttonDelete)) {
            if (isSummaryCellPage()) {
                deleteSummaryColumn();
                cellTabManager.setSelectedColumn(getSelectedCell(), getSelectedSummaryColumn());
            }
        }
        else if (button.equals(buttonClear)) {
            clearSummaryColumns();
            cellTabManager.setSelectedColumn(getSelectedCell(), getSelectedSummaryColumn());
        }
        else if (button.equals(up)) {
            moveColumn(UP);
        }
        else if (button.equals(down)) {
            moveColumn(DOWN);
        }
    }
//	valueChanged();
}

/**
 * Observe any changes.
 */
public void JCHiGridValueChanged(JCHiGridEvent e) {
    int size = columnsModel.getSize();
    String selected = (String) columns.getSelectedValue();
    if (isSummaryCellPage() && size > 0) {
        // Update summary column cell name if changed.
        CellFormat cell = getSelectedCell();
        String cellText = cell.getText();
        if (!cellText.equals(selected)) {
            columnsModel.setElementAt(cellText, columns.getSelectedIndex());
        }
    }
}

/** Move a column/cell format of the current row format.
 */
void moveColumn(int direction) {
    if (section == JCHiGridNode.SECTION_RECORD) {
        // Also, move header column
        moveColumn(direction, (String) columns.getSelectedValue(), 
                   getJCHiGridNode().getRowFormat(JCHiGridNode.SECTION_HEADER));
    }

    if (isSummaryCellPage()) {
        RowFormat format = getJCHiGridNode().getRowFormat(section);
        Vector v = (Vector) format.getDataFormats();
        JCHiGridNode node = getJCHiGridNode();
        if (node == null) {
            return;
        }
        Vector summaryColumns = node.getSummaryColumns(section);
        if (v != null && summaryColumns != null) {
            moveColumn(direction, columns.getSelectedIndex(), format, v, summaryColumns);
        }
    }
    else {
        String selected = (String) columns.getSelectedValue();
        moveColumn(direction, selected,
                   getJCHiGridNode().getRowFormat(section));
    }
    getJCHiGridNode().setRowFormat(section, getJCHiGridNode().getRowFormat(section));
    getJCHiGridNode().getGridProperties().fireJCHiGridEvent(new JCHiGridEvent(getJCHiGridNode().getGridProperties(), getJCHiGridNode()));
}

/** Move a column/cell format in a row format. */
void moveColumn(int direction, String column_name, RowFormat format) {
    Vector v = (Vector) format.getDataFormats();
    if (v == null) return;

    // Find the current column.
    int index = 0;
    for (Enumeration e=v.elements(); e.hasMoreElements();) {
        CellFormat cell = (CellFormat) e.nextElement();
        if (cell.getName().equals(column_name)) {
            break;
        }
        index++;
    }

    moveColumn(direction, index, format, v, null);
}

void moveColumn(int direction, int index, RowFormat format, Vector cellVector, Vector summaryVector) {
    Vector new_order = null;
    Vector new_order2 = null;
    // Move up
    if (direction == UP && index > 0 && index < cellVector.size()) {
        new_order = new Vector();
        if (summaryVector != null) {
            new_order2 = new Vector();
        }
        for (int i=0; i<cellVector.size(); i++) {
            if (i == index-1) {
                new_order.addElement(cellVector.elementAt(i+1));
                new_order.addElement(cellVector.elementAt(i));
                if (new_order2 != null) {
                    new_order2.addElement(summaryVector.elementAt(i+1));
                    new_order2.addElement(summaryVector.elementAt(i));
                }
                i++;
            }
            else {
                new_order.addElement(cellVector.elementAt(i));
                if (new_order2 != null) {
                    new_order2.addElement(summaryVector.elementAt(i));
                }
            }
        }
    }
    // Move down
    else if (direction == DOWN && index >= 0 && index < cellVector.size()-1) {
        new_order = new Vector();
        if (summaryVector != null) {
            new_order2 = new Vector();
        }
        for (int i=0; i<cellVector.size(); i++) {
            if (i == index) {
                new_order.addElement(cellVector.elementAt(i+1));
                new_order.addElement(cellVector.elementAt(i));
                if (new_order2 != null) {
                    new_order2.addElement(summaryVector.elementAt(i+1));
                    new_order2.addElement(summaryVector.elementAt(i));
                }
                i++;
            }
            else {
                new_order.addElement(cellVector.elementAt(i));
                if (new_order2 != null) {
                    new_order2.addElement(summaryVector.elementAt(i));
                }
            }
        }
    }
    if (new_order != null) {
        format.setDataFormats(new_order);
    }
    if (new_order2 != null) {
        JCHiGridNode node = getJCHiGridNode();
        if (node != null) {
            node.setSummaryColumns(section, new_order2);
        }
    }
}

void setEnabledCellButtons() {
    buttonAdd.setEnabled(isSummaryCellPage());
    int size = columnsModel.getSize();
    boolean enable = (size > 0 && isSummaryCellPage());
    buttonDelete.setEnabled(enable);
    buttonClear.setEnabled(enable);
}

/**
 * Add a summary column.
 */
void addSummaryColumn() {
    JCHiGridNode node = getJCHiGridNode();
    if (node == null) {
        return;
    }
    RowFormat rowFormat = node.getRowFormat(section);
    if (rowFormat == null) {
        return;
    }
    Vector summaryColumns = node.getSummaryColumns(section);
    if (summaryColumns == null) {
        return;
    }

    // create a default label summary column
    String name = new String("column");
    SummaryColumn summaryColumn = new SummaryColumn(name);
    summaryColumns.addElement(summaryColumn);

    // create a corresponding cell format
    CellFormat cell = new CellFormat();
    cell.setParent(rowFormat);
    try {
        cell.setName(summaryColumn.getIdentifier());
        cell.setText(name);
        cell.setType(summaryColumn.getDataType());
        cell.setCellRenderer(Class.forName(LookupTable.getDefaultCellRendererName(cell.getType())));
    }
    catch (Exception ex) {
        ex.printStackTrace();
    }
    cell.setEditable(false);
    cell.setDefaultSize(this, false);
    Vector cells = rowFormat.getDataFormats();
    if (cells == null) {
        cells = new Vector(10);
    }
    cells.addElement(cell);
    rowFormat.setDataFormats(cells);

    int index = columnsModel.getSize();
    columnsModel.addElement(cell.getText());
    columns.setSelectedIndex(index);

    updateOnColumnListChanged(index);
}

/**
 * Delete a summary column.
 */
void deleteSummaryColumn() {
    JCHiGridNode node = getJCHiGridNode();
    if (node == null) {
        return;
    }
    RowFormat rowFormat = node.getRowFormat(section);
    if (rowFormat == null) {
        return;
    }
    Vector summaryColumns = node.getSummaryColumns(section);
    if (summaryColumns == null) {
        return;
    }
    int selectedIndex = columns.getSelectedIndex();
    if (selectedIndex < 0) {
        return;
    }

    summaryColumns.removeElementAt(selectedIndex);
    Vector cells = rowFormat.getDataFormats();
    if (cells != null) {
        cells.removeElementAt(selectedIndex);
        rowFormat.setDataFormats(cells);
    }
    columns.remove(selectedIndex);
    int count = columnsModel.getSize();
    if (count > 0) {
        selectedIndex = Math.min((count-1), selectedIndex);
        columns.setSelectedIndex(selectedIndex);
    }
    else {
        selectedIndex = (-1);
    }

    updateOnColumnListChanged(selectedIndex);
}

/**
 * Clear all summary columns.
 */
void clearSummaryColumns() {
    JCHiGridNode node = getJCHiGridNode();
    if (node == null) {
        return;
    }
    RowFormat rowFormat = node.getRowFormat(section);
    if (rowFormat == null) {
        return;
    }
    Vector summaryColumns = node.getSummaryColumns(section);
    if (summaryColumns == null) {
        return;
    }

    node.setSummaryColumns(section, new Vector(10));
    rowFormat.setDataFormats(new Vector(10));
    columns.removeAll();

    updateOnColumnListChanged(-1);
}

/**
 * Update on column list changed.
 *
 * @param index The new selected index.
 */
void updateOnColumnListChanged(int index) {
    setEnabledCellButtons();
    updateUpDownButtons();
    cellTabManager.setSelectedColumn(getSelectedCell(), getSelectedSummaryColumn());
    getJCHiGridNode().setRowFormat(section, getJCHiGridNode().getRowFormat(section));
    getJCHiGridNode().getGridProperties().fireJCHiGridEvent(new JCHiGridEvent(getJCHiGridNode().getGridProperties(), getJCHiGridNode()));
}

}
