/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: BaseEditStatusPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:21 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid.customizer;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.TitledBorder;

import com.klg.jclass.higrid.*;
import com.klg.jclass.higrid.beans.*;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.swing.JCGridLayout;

/** It is a page contains properties for an edit status indicator format.
 *	When a <code>JCHiGridNode</code> is set,
 *	showValue() is called to display property values.
 *	When a property value is changed, valueChanged() is called.
 */
public class BaseEditStatusPage extends FormatPage implements ActionListener,
    ItemListener {

protected int section;
EditStatusCellFormat editStatusCell;

// common properties
ColorEditor cellBackgroundColor;
JTextField borderInsets;
JComboBox borderStyle;

public BaseEditStatusPage(int section) {
    super(LocaleBundle.string(LocaleBundle.editStatusIndicator));
    init();
    this.section = section;
}


/** initialize UI.
 */
public void init() {
    setLayout(new BorderLayout());
    JPanel panel1 = new JPanel();
    panel1.setBorder(new TitledBorder(LocaleBundle.string(LocaleBundle.border)));
    borderStyle = new JComboBox();
    borderInsets = new JTextField("", 15);
    JLabel lbl_border_style = new JLabel(LocaleBundle.string(LocaleBundle.style));
    JLabel lbl_border_insets = new JLabel(LocaleBundle.string(LocaleBundle.insets));
    FocusListener fl = new FocusAdapter() {
        public void focusLost(FocusEvent e) {
            cellBackgroundColor.checkColorSource(e.getSource());
            valueChanged();
        }
    };
    cellBackgroundColor = new ColorEditor(this, this, fl,
                                          LocaleBundle.string(LocaleBundle.backgroundColor), false);
    add(cellBackgroundColor, BorderLayout.NORTH);

    panel1.setLayout(new JCGridLayout(JCGridLayout.VARIABLE, 2, 5, 5));
    String choicesBS[] = JCHiGridConverter.getRowFormatBorderStyleStrings();
    for (int i = 0; i < (choicesBS.length-1); i++) { // exclude mixed style
        borderStyle.addItem(choicesBS[i]);
    }
    borderStyle.addItemListener(this);
    borderInsets.addActionListener(this);
    borderInsets.addFocusListener(fl);
    panel1.add(lbl_border_style);
    panel1.add(borderStyle);
    panel1.add(lbl_border_insets);
    panel1.add(borderInsets);
    add(panel1, BorderLayout.CENTER);
}


public void setSection(int section) {
    this.section = section;
}

/** Overrides to perform operations necessary after value is changed on this page.
 */
public void valueChanged() {
    if (editStatusCell == null) {
        return;
    }
    boolean changed = false;

    // set background color
    Color color = cellBackgroundColor.getColor();
    if (color != null && !color.equals(editStatusCell.getBackground())) {
        editStatusCell.setBackground(color);
        changed = true;
    }

    // set border insets
    String s = borderInsets.getText();
    StringTokenizer st = new StringTokenizer(s, ",");
    int insets[] = { 2, 2, 2, 2};
    boolean convertStatus = true;
    for (int i=0; i<4; i++) {
        if (st.hasMoreTokens()) {
            try {
                insets[i] = new Integer(st.nextToken()).intValue();
            }
            catch (Exception e) {
                convertStatus = false;
            }
        }
    }
    if (convertStatus) {
        editStatusCell.setBorderInsets(new Insets(insets[0], insets[1], 
                                                  insets[2], insets[3]));
        changed = true;
    }

    // set border style
    s = (String) borderStyle.getSelectedItem();
    JCTypeConverter conv = new JCTypeConverter();
    int style = JCHiGridConverter.toRowFormatBorderStyle(conv, s, HiGridBorder.ETCHED_IN);
    if (editStatusCell.getBorderStyle() != style) {
        editStatusCell.setBorderStyle(style);
        changed = true;
    }

    // notify observers of changes, if any.
    if (changed) {
        getJCHiGridNode().setChanged();
        getJCHiGridNode().getGridProperties().fireJCHiGridEvent(new JCHiGridEvent(getJCHiGridNode().getGridProperties(), getJCHiGridNode()));
    }
}

/** Overrides to refresh values displayed on this page after the underlined
 *	<code>JCHiGridNode</code> is changed.
 */
public void showValue() {
    if (editStatusCell == null) {
        return;
    }

    cellBackgroundColor.selectColor(editStatusCell.getBackground());

    Insets i = editStatusCell.getBorderInsets();
    borderInsets.setText(new String(i.top + "," + i.left + "," + i.bottom + "," + i.right));

    int style = editStatusCell.getBorderStyle();
    int index = -1;
    int defaultBorder = 0;
    for (int j = 0; j < (JCHiGridConverter.rowFormatBorderStyleValues.length-1); j++) {
        if (JCHiGridConverter.rowFormatBorderStyleValues[j] == HiGridBorder.ETCHED_IN) {
            defaultBorder = JCHiGridConverter.rowFormatBorderStyleValues[j];
        }
        if (style == JCHiGridConverter.rowFormatBorderStyleValues[j]) {
            index = j;
            break;
        }
    }
    if (index == -1) {
        index = defaultBorder;
    }
    borderStyle.setSelectedIndex(index);
}

/** Loads a <code>JCHiGridNode</code> into the page.
 */
public void setJCHiGridNode(JCHiGridNode node) {
    editStatusCell = getEditStatusCellFormat(node);
    showValue();
}

/** Calls showValue() to determine if changes are made.
 */
public void actionPerformed(ActionEvent e) {
    if (isInitializing()) return;

    cellBackgroundColor.checkColorSource(e.getSource());
    valueChanged();
}

/** Calls showValue() to determine if changes are made.
 */
public void itemStateChanged(ItemEvent e) {
    if (isInitializing()) return;

    cellBackgroundColor.checkColorSource(e.getSource());
    valueChanged();
}

/**
 * Retrieves the edit status cell format.
 */
EditStatusCellFormat getEditStatusCellFormat(JCHiGridNode node) {
    EditStatusCellFormat esCell = null;
    if (node == null) {
        return esCell;
    }
    Vector v = node.getRowFormat(section).getIndicatorFormats();
    if (v == null) {
        return esCell;
    }

    for (Enumeration e = v.elements(); e.hasMoreElements();) {
        CellFormat cell = (CellFormat) e.nextElement();
        if (cell instanceof EditStatusCellFormat) {
            esCell = (EditStatusCellFormat) cell;
            break;
        }
    }
    return esCell;
}


}
