/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: BaseLevelPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:21 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid.customizer;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;

import com.klg.jclass.higrid.*;
import com.klg.jclass.higrid.beans.JCHiGridNode;
import com.klg.jclass.higrid.beans.JCHiGridEvent;
import com.klg.jclass.higrid.beans.JCHiGridConverter;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.swing.*;

/** It is a page contains properties at the section level.  These properties apply to
 *	all columns at the level.  When a <code>JCHiGridNode</code> is set, 
 *	showValue() is called to display property values.
 *	When a property value is changed, valueChanged() is called.
 */
public class BaseLevelPage extends FormatPage implements ActionListener,
    ItemListener {

protected int section;

// common properties
JTextField height, borderInsets;
JCheckBox allowHeightSizing;
JComboBox borderStyle;
JComboBox autoTraverse;
JCheckBox repeatHeader;
ColorEditor levelForegroundColor;
ColorEditor levelBackgroundColor;
FontEditor levelFontEditor;

public BaseLevelPage(int section) {
    super(LocaleBundle.string(LocaleBundle.level));
    init();
    this.section = section;
}


/** initialize UI.
 */
public void init() {
    setLayout(new BorderLayout());

    JPanel fontPanel = new JPanel();
    fontPanel.setBorder(new TitledBorder(LocaleBundle.string(LocaleBundle.font)));
    levelFontEditor = new FontEditor(this, true);
    fontPanel.add(levelFontEditor);
//    ((FlowLayout)(levelFontEditor.getLayout())).setAlignment(FlowLayout.LEFT);
    JLabel lbl_height = new JLabel(LocaleBundle.string(LocaleBundle.height));
    JLabel lbl = new JLabel("");
    JLabel lbl2 = new JLabel("");
    JLabel lbl3 = new JLabel("");
    JLabel lbl_autoTraverse = new JLabel(LocaleBundle.string(LocaleBundle.autoTraverse));
    height = new JTextField("", 15);
    allowHeightSizing = new JCheckBox(LocaleBundle.string(LocaleBundle.allow_height_resizing), true);
    JPanel panel1 = new JPanel();
    JPanel panel2 = new JPanel();
    borderInsets = new JTextField("", 15);
    borderStyle = new JComboBox();
    JLabel lbl_style = new JLabel(LocaleBundle.string(LocaleBundle.style));
    JLabel lbl_insets = new JLabel(LocaleBundle.string(LocaleBundle.insets));
    autoTraverse = new JComboBox();
    String choicesAS[] = JCHiGridConverter.getAutoTraverseStrings();
    for (int i = 0; i < choicesAS.length; i++) {
        autoTraverse.addItem(choicesAS[i]);
    }
    autoTraverse.addItemListener(this);
    repeatHeader = new JCheckBox(LocaleBundle.string(LocaleBundle.repeat_header), false);

    JPanel panel3 = new JPanel();
    JPanel panel4 = new JPanel();
    JPanel border_grp = new JPanel();
    border_grp.setBorder(new TitledBorder(LocaleBundle.string(LocaleBundle.border)));
    panel1.setLayout(new JCGridLayout(JCGridLayout.VARIABLE, 2, 5, 5));
    //FIX font panel not positioned nicely
    panel1.add(lbl);
    panel1.add(fontPanel);
    panel1.add(lbl_height);
    panel1.add(height);
    panel1.add(lbl2);
    panel1.add(allowHeightSizing);
    panel1.add(lbl_autoTraverse);
    panel1.add(autoTraverse);
    panel1.add(lbl3);
    panel1.add(repeatHeader);

    height.addActionListener(this);
    allowHeightSizing.addItemListener(this);

    panel2.setLayout(new BorderLayout());

    border_grp.setLayout(new JCGridLayout(JCGridLayout.VARIABLE, 2, 5, 5));
    border_grp.add(lbl_style);
    border_grp.add(borderStyle);
    border_grp.add(lbl_insets);
    border_grp.add(borderInsets);

    String ss[] = JCHiGridConverter.getRowFormatBorderStyleStrings();
    for (int i=0; i<ss.length; i++) {
        borderStyle.addItem(ss[i]);
    }
    borderInsets.addActionListener(this);
    allowHeightSizing.addItemListener(this);
    borderStyle.addItemListener(this);  

    FocusListener fl = new FocusAdapter() {
        public void focusLost(FocusEvent e) {
            levelForegroundColor.checkColorSource(e.getSource());
            levelBackgroundColor.checkColorSource(e.getSource());
            valueChanged();
        }
    };

    // level colors panels
    levelForegroundColor = new ColorEditor(this, this, fl,
                                           LocaleBundle.string(LocaleBundle.foregroundColor), true);
    levelBackgroundColor = new ColorEditor(this, this, fl,
                                           LocaleBundle.string(LocaleBundle.backgroundColor), true);

    repeatHeader.addItemListener(this);

    panel2.add(border_grp, BorderLayout.NORTH);
    panel3.setLayout(new BorderLayout());
    panel2.add(panel3, BorderLayout.CENTER);
    panel3.add(levelForegroundColor, BorderLayout.NORTH);
    panel4.setLayout(new BorderLayout());
    panel4.add(levelBackgroundColor, BorderLayout.NORTH);
    panel3.add(panel4, BorderLayout.CENTER);

    add(panel1, BorderLayout.CENTER);
    add(panel2, BorderLayout.NORTH);

    height.addFocusListener(fl);
    borderInsets.addFocusListener(fl);
}


public void setSection(int section) {
    this.section = section;
}

/** Overrides to perform operations necessary after value is changed on this page.
 */
public void valueChanged() {
    // set height
    String s = height.getText();
    int height = 0;
    try {
        height = new Integer(s).intValue();
    }
    catch (Exception e) {
    }
    getJCHiGridNode().setHeight(section, height);

    // set allow height resizing
    getJCHiGridNode().setAllowHeightResizing(section, allowHeightSizing.isSelected());

    // set border insets
    s = borderInsets.getText();
    StringTokenizer st = new StringTokenizer(s, ",");
    int insets[] = { 2, 2, 2, 2};
    for (int i=0; i<4; i++) {
        if (st.hasMoreTokens()) {
            try {
                insets[i] = new Integer(st.nextToken()).intValue();
            }
            catch (Exception e) {
            }
        }
    }
    getJCHiGridNode().setBorderInsets(section, 
                                      new Insets(insets[0], 
                                                 insets[1],
                                                 insets[2],
                                                 insets[3]));

    // set border style
    s = (String) borderStyle.getSelectedItem();
    JCTypeConverter conv = new JCTypeConverter();
    int style = JCHiGridConverter.toRowFormatBorderStyle(conv, s, com.klg.jclass.cell.Utilities.ETCHED_IN);
    getJCHiGridNode().setBorderStyle(section, style);

    // set data formats' colors and fonts
    Color color = levelForegroundColor.getColor();
    if (color != null) {
        getJCHiGridNode().setForeground(section, color);
    }
    color = levelBackgroundColor.getColor();
    if (color != null) {
        getJCHiGridNode().setBackground(section, color);
    }
    getJCHiGridNode().setFont(section, levelFontEditor.getFontName(), 
                              levelFontEditor.getFontStyle(), levelFontEditor.getFontSize(),
                              getFontMetrics(new Font(levelFontEditor.getFontName(),
                                                      levelFontEditor.getFontStyle(),
                                                      levelFontEditor.getFontSize())));

    if (section == JCHiGridNode.SECTION_RECORD) {
        // set auto show
        s = (String) autoTraverse.getSelectedItem();
        getJCHiGridNode().setAutoTraverse(JCHiGridConverter.toAutoTraverse(conv, s, FormatNode.AUTO_TRAVERSE_NONE));
    }

    if (section == JCHiGridNode.SECTION_HEADER) {
        // set repeat header
        getJCHiGridNode().setRepeatHeader(repeatHeader.isSelected());
    }

    // notify observers of changes, if any.
    getJCHiGridNode().getGridProperties().fireJCHiGridEvent(new JCHiGridEvent(getJCHiGridNode().getGridProperties(),
                                                                              getJCHiGridNode()));
}

/** Overrides to refresh values displayed on this page after the underlined
 *	<code>JCHiGridNode</code> is changed.
 */
public void showValue() {
    height.setText(new Integer(getJCHiGridNode().getHeight(section)).toString());
    allowHeightSizing.setSelected(getJCHiGridNode().getAllowHeightResizing(section));

    Insets i = getJCHiGridNode().getBorderInsets(section);
    borderInsets.setText(new String(i.top + "," + i.left + "," + i.bottom + "," + i.right));
    int style = getJCHiGridNode().getBorderStyle(section);
    int index = -1;
    int default_border = 0;
    for (int j=0; j<JCHiGridConverter.rowFormatBorderStyleValues.length; j++) {
        if (JCHiGridConverter.rowFormatBorderStyleValues[j] == com.klg.jclass.cell.Utilities.ETCHED_IN) {
            default_border = j;
        }
        if (style == JCHiGridConverter.rowFormatBorderStyleValues[j]) {
            index = j;
            break;
        }
    }
    if (index == -1) {
        index = default_border;
    }
    borderStyle.setSelectedIndex(index);

    if (section == JCHiGridNode.SECTION_RECORD) {
        // record specific
        index = 0;
        int show = getJCHiGridNode().getAutoTraverse();
        int default_show = 0;
        for (int j=0; j<JCHiGridConverter.autoTraverseValues.length; j++) {
            if (JCHiGridConverter.autoTraverseValues[j] == FormatNode.AUTO_TRAVERSE_NONE) {
                default_show = j;
            }
            if (show == JCHiGridConverter.autoTraverseValues[j]) {
                index = j;
                break;
            }
        }
        if (index == -1) {
            index = default_show;
        }
        autoTraverse.setSelectedIndex(index);
    }
    else if (section == JCHiGridNode.SECTION_HEADER) {
        // header specific
        repeatHeader.setSelected(getJCHiGridNode().isRepeatHeader());
    }

    levelForegroundColor.selectColor(getJCHiGridNode().getForeground(section));
    levelBackgroundColor.selectColor(getJCHiGridNode().getBackground(section));
    levelFontEditor.selectFont(getJCHiGridNode().getFontName(section),
                               getJCHiGridNode().getFontStyle(section),
                               getJCHiGridNode().getFontSize(section));

    // only enable type-specific properties in their proper sections
    autoTraverse.setEnabled(section == JCHiGridNode.SECTION_RECORD);
    repeatHeader.setEnabled(section == JCHiGridNode.SECTION_HEADER);
}


/** Loads a <code>JCHiGridNode</code> into the page.
 */
public void setJCHiGridNode(JCHiGridNode node) {
    showValue();
}

/** Calls valueChanged() to determine if changes are made.
 */
public void actionPerformed(ActionEvent e) {
    if (isInitializing()) return;

    checkColorEditors(e.getSource());
    valueChanged();
}

/** Calls valueChanged() to determine if changes are made.
 */
public void itemStateChanged(ItemEvent e) {
    if (isInitializing()) return;

    System.out.println("ItemEvent is " + e);
// ignore combobox deselects
    if (e.getStateChange() == ItemEvent.DESELECTED) {
        return;
    }

    checkColorEditors(e.getSource());
    valueChanged();
}

void checkColorEditors(Object source) {
    if (!levelForegroundColor.checkColorSource(source) &&
        !levelBackgroundColor.checkColorSource(source)) {
    }
}

}
