/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CellColorPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:21 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid.customizer;

import java.awt.*;
import java.awt.event.*;

import javax.swing.JPanel;

import com.klg.jclass.higrid.*;
import com.klg.jclass.higrid.beans.*;

/**
 * This is the color properties page for the specified cell.
 * When a <code>JCHiGridNode</code> is set, showValue() is called
 * to display property values.
 * When a property value is changed, valueChanged() is called.
 */
public class CellColorPage extends CellFormatPage implements ActionListener,
    ItemListener {

// common properties
ColorEditor cellForegroundColor;
ColorEditor cellBackgroundColor;
ColorEditor cellSelectedForegroundColor;
ColorEditor cellSelectedBackgroundColor;

/**
 * Constructors
 */
public CellColorPage() {
    super(LocaleBundle.string(LocaleBundle.color));
    init();
}

/**
 * Initialize the UI.
 */
public void init() {
    FocusListener fl = new FocusAdapter() {
        public void focusLost(FocusEvent e) {
            cellForegroundColor.checkColorSource(e.getSource());
            cellBackgroundColor.checkColorSource(e.getSource());
            cellSelectedForegroundColor.checkColorSource(e.getSource());
            cellSelectedForegroundColor.checkColorSource(e.getSource());
            valueChanged();
        }
    };
    cellForegroundColor = new ColorEditor(this, this, fl,
                                          LocaleBundle.string(LocaleBundle.foregroundColor), false);
    cellBackgroundColor = new ColorEditor(this, this, fl,
                                          LocaleBundle.string(LocaleBundle.backgroundColor), false);

    JPanel panel1 = new JPanel(new BorderLayout());
    JPanel panel2 = new JPanel(new BorderLayout());
    JPanel panel3 = new JPanel(new BorderLayout());
    cellSelectedForegroundColor = new ColorEditor(this, this, fl,
                                                  LocaleBundle.string(LocaleBundle.selectedForegroundColor), false);
    cellSelectedBackgroundColor = new ColorEditor(this, this, fl,
                                                  LocaleBundle.string(LocaleBundle.selectedBackgroundColor), false);
    panel1.add(cellForegroundColor, BorderLayout.NORTH);
    panel1.add(cellBackgroundColor, BorderLayout.SOUTH);
    panel2.add(cellSelectedForegroundColor, BorderLayout.NORTH);
    panel2.add(cellSelectedBackgroundColor, BorderLayout.SOUTH);
    panel3.add(panel1, BorderLayout.NORTH);
    panel3.add(panel2, BorderLayout.SOUTH);
    setLayout(new BorderLayout());
    add(panel3, BorderLayout.NORTH);
}

/** Overrides to perform operations necessary after value is changed on this page.
 */
public void valueChanged() {
    if (cellFormat == null) {
        return;
    }

    boolean changed = false;
    Color color = cellForegroundColor.getColor();
    if (color != null && !color.equals(cellFormat.getForeground())) {
        cellFormat.setForeground(color);
        changed = true;
    }
    color = cellBackgroundColor.getColor();
    if (color != null && !color.equals(cellFormat.getBackground())) {
        cellFormat.setBackground(color);
        changed = true;
    }
    if (section == JCHiGridNode.SECTION_RECORD) {
        color = cellSelectedForegroundColor.getColor();
        if (color != null && !color.equals(cellFormat.getSelectedForeground())) {
            cellFormat.setSelectedForeground(color);
            changed = true;
        }
        color = cellSelectedBackgroundColor.getColor();
        if (color != null && !color.equals(cellFormat.getSelectedBackground())) {
            cellFormat.setSelectedBackground(color);
            changed = true;
        }
    }

    if (changed) {
        // notify observers of changes, if any.
        getJCHiGridNode().setChanged();
        getJCHiGridNode().getGridProperties().fireJCHiGridEvent(new JCHiGridEvent(getJCHiGridNode().getGridProperties(), getJCHiGridNode()));
    }
}

/** Overrides to refresh values displayed on this page after the underlined
 *	<code>JCHiGridNode</code> is changed.
 */
public void showValue() {
    if (cellFormat == null) {
        return;
    }
    cellForegroundColor.selectColor(cellFormat.getForeground());
    cellBackgroundColor.selectColor(cellFormat.getBackground());
    cellSelectedForegroundColor.setEnabled(section == JCHiGridNode.SECTION_RECORD);
    cellSelectedBackgroundColor.setEnabled(section == JCHiGridNode.SECTION_RECORD);
    if (section == JCHiGridNode.SECTION_RECORD) {
        cellSelectedForegroundColor.selectColor(cellFormat.getSelectedForeground());
        cellSelectedBackgroundColor.selectColor(cellFormat.getSelectedBackground());
    }
}

/** Loads a <code>JCHiGridNode</code> into the page.
 */
public void setJCHiGridNode(JCHiGridNode node, CellFormat cellFormat, SummaryColumn summaryColumn) {
    showValue();
}

/**
 * Show values for the new selected column.
 */
public void setSelectedColumn(CellFormat cellFormat, SummaryColumn summaryColumn) {
    super.setSelectedColumn(cellFormat, summaryColumn);
    showValue();
}

/** Calls showValue() to determine if changes are made.
 */
public void actionPerformed(ActionEvent e) {
    if (isInitializing()) return;

    checkColorEditors(e.getSource());
    valueChanged();
}

/** Calls showValue() to determine if changes are made.
 */
public void itemStateChanged(ItemEvent e) {
    if (isInitializing()) return;

    checkColorEditors(e.getSource());
    valueChanged();
}

void checkColorEditors(Object source) {
    if (!cellForegroundColor.checkColorSource(source) &&
        !cellBackgroundColor.checkColorSource(source)) {
        if (section == JCHiGridNode.SECTION_RECORD) {
            if (!cellSelectedForegroundColor.checkColorSource(source) && 
                !cellSelectedBackgroundColor.checkColorSource(source)) {
            }
        }
    }
}

}
