/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CellEditPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:21 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid.customizer;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

import javax.swing.*;
import javax.swing.border.*;

import com.klg.jclass.datasource.*;
import com.klg.jclass.higrid.*;
import com.klg.jclass.higrid.beans.*;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.swing.*;
import com.klg.jclass.higrid.LocaleBundle;

/**
 * This is the edit and renderer properties page for the specified cell.
 * When a <code>JCHiGridNode</code> is set, showValue() is called
 * to display property values.
 * When a property value is changed, valueChanged() is called.
 */
public class CellEditPage extends CellFormatPage implements ActionListener,
    ItemListener {

// common properties
static Vector cellRendererClasses = null;
static Vector cellEditorClasses = null;

JComboBox valueType;
JCheckBox editable;
JCheckBox selectAll;
JComboBox editWidthPolicy;
JComboBox editHeightPolicy;
JRadioButton selectCellRenderer;
JComboBox JCCellRenderer;
JRadioButton selectCustomCellRenderer;
JTextField customCellRenderer;
JRadioButton selectCellEditor;
JComboBox cellEditor;
JRadioButton selectCustomCellEditor;
JTextField customCellEditor;

/**
 * Constructors
 */
public CellEditPage() {
    super(LocaleBundle.string(LocaleBundle.edit));
    initEditorRendererClasses();
    init();
}

/**
 * Setup the editor and renderer classes vectors.
 */
void initEditorRendererClasses() {
    String s = null;
    if (cellRendererClasses == null) {
        cellRendererClasses = new Vector();                        
        for (int i = 0; i < JCHiGridConverter.metaDataModelTypeValues.length; i++) {
            int type = JCHiGridConverter.metaDataModelTypeValues[i];
            s = LookupTable.getDefaultCellRendererName(type);
            insertClass(cellRendererClasses, s);
            s = LookupTable.getAlternateCellRendererName(type);
            insertClass(cellRendererClasses, s);
        }
        for (int j = 0; j < 100; j++) {
            s = LookupTable.getRemainingCellRendererName(j);
            if (s == null) {
                break;
            }
            insertClass(cellRendererClasses, s);
        }           
    }

    if (cellEditorClasses == null) {
        cellEditorClasses = new Vector();                        
        for (int i = 0; i < JCHiGridConverter.metaDataModelTypeValues.length; i++) {
            int type = JCHiGridConverter.metaDataModelTypeValues[i];
            s = LookupTable.getDefaultCellEditorName(type);
            insertClass(cellEditorClasses, s);
            s = LookupTable.getAlternateCellEditorName(type);
            insertClass(cellEditorClasses, s);
        }           
        for (int j = 0; j < 100; j++) {
            s = LookupTable.getRemainingCellEditorName(j);
            if (s == null) {
                break;
            }
            insertClass(cellEditorClasses, s);
        }           
    }
}

void insertClass(Vector classes, String s) {
    if (s == null) {
        return;
    }
    int index = (-1);
    int size = classes.size();
    for (int i = 0; i < size; i++) {
        int result = s.compareTo((String)classes.elementAt(i));
        if (result == 0) {
            index = i;
            break;
        }
        else if (result < 0) {
            index = i;
            classes.insertElementAt(s, i);
            break;
        }
    }
    if (index == (-1)) {
        classes.addElement(s);
    }
}

/**
 * Initialize the UI.
 */
public void init() {
    setLayout(new BorderLayout());

    FocusListener fl = new FocusAdapter() {
        public void focusLost(FocusEvent e) {
            if (isInitializing()) return;

            valueChanged();
        }
    };

    JPanel group1 = new JPanel();
    valueType = new JComboBox();
    editable = new JCheckBox(LocaleBundle.string(LocaleBundle.editable), true);
    selectAll = new JCheckBox(LocaleBundle.string(LocaleBundle.editorSelectAll), true);
    editWidthPolicy = new JComboBox();
    editHeightPolicy = new JComboBox();
    JLabel lbl1 = new JLabel("");
    JLabel lbl2 = new JLabel("");
    JLabel lbl_value_type = new JLabel(LocaleBundle.string(LocaleBundle.valueType));
    JLabel lbl_edit_width = new JLabel(LocaleBundle.string(LocaleBundle.editWidthPolicy));
    JLabel lbl_edit_height = new JLabel(LocaleBundle.string(LocaleBundle.editHeightPolicy));
    JPanel panel2 = new JPanel();

    // first panel - miscellaneous properties
    group1.setLayout(new JCAlignLayout());

    String choicesVT[] = JCHiGridConverter.getMetaDataModelTypeStrings();
    for (int i = 0; i < choicesVT.length; i++) {
        valueType.addItem(choicesVT[i]);
    }
    valueType.addItemListener(this);
    editable.addItemListener(this);
    selectAll.addItemListener(this);
    String choicesCEP[] = JCHiGridConverter.getCellEditPolicyStrings();
    for (int i = 0; i < choicesCEP.length; i++) {
        editWidthPolicy.addItem(choicesCEP[i]);
    }
    editWidthPolicy.addItemListener(this);
    for (int i = 0; i < choicesCEP.length; i++) {
        editHeightPolicy.addItem(choicesCEP[i]);
    }
    editHeightPolicy.addItemListener(this);

    group1.add(lbl_value_type);
    group1.add(valueType);
    group1.add(lbl1);
    group1.add(editable);
    group1.add(lbl2);
    group1.add(selectAll);
    group1.add(lbl_edit_width);
    group1.add(editWidthPolicy);
    group1.add(lbl_edit_height);
    group1.add(editHeightPolicy);
    add(group1, BorderLayout.NORTH);

    panel2.setLayout(new BorderLayout());
    add(panel2, BorderLayout.CENTER);

    // second panel - renderer
    JPanel group2 = new JPanel();
    group2.setBorder(new TitledBorder(LocaleBundle.string(LocaleBundle.renderer)));
    group2.setLayout(new JCGridLayout(JCGridLayout.VARIABLE, 2, 5, 5));

    ButtonGroup cbgroup1 = new ButtonGroup();
    selectCellRenderer = new JRadioButton("", true);
    JCCellRenderer = new JComboBox();
    selectCustomCellRenderer = new JRadioButton("", false);
    customCellRenderer = new JTextField("", 30);;
    cbgroup1.add(selectCellRenderer);
    cbgroup1.add(selectCustomCellRenderer);
    JPanel panel3 = new JPanel();
    ButtonGroup cbGroup2 = new ButtonGroup();
    selectCellEditor = new JRadioButton("", true);
    cellEditor = new JComboBox();
    selectCustomCellEditor = new JRadioButton("", false);
    customCellEditor = new JTextField("", 30);;
    cbGroup2.add(selectCellEditor);
    cbGroup2.add(selectCustomCellEditor);

    // third panel - editor
    JPanel group3 = new JPanel();
    group3.setBorder(new TitledBorder(LocaleBundle.string(LocaleBundle.editor)));
    group3.setLayout(new JCGridLayout(JCGridLayout.VARIABLE, 2, 5, 5));
    selectCellRenderer.addItemListener(this);
    int size = cellRendererClasses.size();
    for (int i = 0; i < size; i++) {
        JCCellRenderer.addItem((String)cellRendererClasses.elementAt(i));
    }
    JCCellRenderer.addItemListener(this);
    selectCustomCellRenderer.addItemListener(this);
    customCellRenderer.addActionListener(this);
    customCellRenderer.addFocusListener(fl);

    group2.add(selectCellRenderer);
    group2.add(JCCellRenderer);
    group2.add(selectCustomCellRenderer);
    group2.add(customCellRenderer);
    panel2.add(group2, BorderLayout.NORTH);

    panel3.setLayout(new BorderLayout());
    panel2.add(panel3, BorderLayout.CENTER);

    selectCellEditor.addItemListener(this);
    size = cellEditorClasses.size();
    for (int i = 0; i < size; i++) {
        cellEditor.addItem((String)cellEditorClasses.elementAt(i));
    }
    cellEditor.addItemListener(this);
    selectCustomCellEditor.addItemListener(this);
    customCellEditor.addActionListener(this);
    customCellEditor.addFocusListener(fl);

    group3.add(selectCellEditor);
    group3.add(cellEditor);
    group3.add(selectCustomCellEditor);
    group3.add(customCellEditor);
    panel3.add(group3, BorderLayout.NORTH);
}

/** Overrides to perform operations necessary after value is changed on this page.
 */
public void valueChanged() {
    if (cellFormat == null) {
        return;
    }

    String s = (String) valueType.getSelectedItem();
    JCTypeConverter conv = new JCTypeConverter();
    int type = JCHiGridConverter.toMetaDataModelType(conv, s, MetaDataModel.TYPE_OBJECT);
    cellFormat.setType(type);

    cellFormat.setEditable(editable.isSelected());
    cellFormat.setSelectAll(selectAll.isSelected());

    s = (String) editWidthPolicy.getSelectedItem();
    int policy = JCHiGridConverter.toCellEditPolicy(conv, s, HiGrid.EDIT_SIZE_TO_CELL);
    cellFormat.setEditWidthPolicy(policy);

    s = (String) editHeightPolicy.getSelectedItem();
    policy = JCHiGridConverter.toCellEditPolicy(conv, s, HiGrid.EDIT_ENSURE_MINIMUM_SIZE);
    cellFormat.setEditHeightPolicy(policy);

    if (selectCellRenderer.isSelected()) {
        s = new String((String) JCCellRenderer.getSelectedItem());
    }
    else {
        s = customCellRenderer.getText();
    }
    if (s == null || s.equals("")) {
        ErrorDialog.showError(this, 
                              LocaleBundle.string(LocaleBundle.noRendererClassSupplied));
    }
    else {
        try {
            cellFormat.setCellRenderer(Class.forName(s));
        }
        catch (Exception ex) {
            ex.printStackTrace();
            ErrorDialog.showError(this, ex.toString());
        }
    }

    if (selectCellEditor.isSelected()) {
        s = new String((String) cellEditor.getSelectedItem());
    }
    else {
        s = customCellEditor.getText();
    }
    if (s == null || s.equals("")) {
        ErrorDialog.showError(this, 
                              LocaleBundle.string(LocaleBundle.noEditorClassSupplied));
    }
    else {
        try {
            cellFormat.setCellEditor(Class.forName(s));
        }
        catch (Exception ex) {
            ex.printStackTrace();
            ErrorDialog.showError(this, ex.toString());
        }
    }

    // Always notify observers.
    getJCHiGridNode().setChanged();
    getJCHiGridNode().getGridProperties().fireJCHiGridEvent(new JCHiGridEvent(getJCHiGridNode().getGridProperties(), getJCHiGridNode()));
}

/** Overrides to refresh values displayed on this page after the underlined
 *	<code>JCHiGridNode</code> is changed.
 */
public void showValue() {
    if (cellFormat == null) {
        return;
    }

    int type = cellFormat.getType();
    int index = -1;
    for (int j = 0; j < JCHiGridConverter.metaDataModelTypeStrings.length; j++) {
        if (type == JCHiGridConverter.metaDataModelTypeValues[j]) {
            index = j;
            break;
        }
    }
    index = Math.max(0, index);
    valueType.setSelectedIndex(index);
    editable.setSelected(cellFormat.isEditable());
    selectAll.setSelected(cellFormat.getSelectAll());

    int policy = cellFormat.getEditWidthPolicy();
    index = -1;
    for (int j = 0; j < JCHiGridConverter.cellEditPolicyStrings.length; j++) {
        if (policy == JCHiGridConverter.cellEditPolicyValues[j]) {
            index = j;
            break;
        }
    }
    index = Math.max(0, index);
    editWidthPolicy.setSelectedIndex(index);

    policy = cellFormat.getEditHeightPolicy();
    index = -1;
    for (int j = 0; j < JCHiGridConverter.cellEditPolicyStrings.length; j++) {
        if (policy == JCHiGridConverter.cellEditPolicyValues[j]) {
            index = j;
            break;
        }
    }
    index = Math.max(0, index);
    editHeightPolicy.setSelectedIndex(index);

    Class renderer = cellFormat.getCellRenderer();
    if (renderer != null) {
        String rendererName = renderer.getName();
        index = -1;
        int size = cellRendererClasses.size();
        for (int j = 0; j < size; j++) {
            if (rendererName.equals((String)cellRendererClasses.elementAt(j))) {
                index = j;
                break;
            }
        }
        if (index != (-1)) {
            JCCellRenderer.setSelectedIndex(index);
            selectCellRenderer.setSelected(true);
        }
        else {
            customCellRenderer.setText(rendererName);
            selectCustomCellRenderer.setSelected(true);
        }
    }

    Class editor = cellFormat.getCellEditor();
    if (editor != null) {
        String editorName = editor.getName();
        index = -1;
        int size = cellEditorClasses.size();
        for (int j = 0; j < size; j++) {
            if (editorName.equals((String)cellEditorClasses.elementAt(j))) {
                index = j;
                break;
            }
        }
        if (index != (-1)) {
            cellEditor.setSelectedIndex(index);
            selectCellEditor.setSelected(true);
        }
        else {
            selectCustomCellEditor.setSelected(true);
            customCellEditor.setText(editorName);
        }
    }
}

/** Loads a <code>JCHiGridNode</code> into the page.
 */
public void setJCHiGridNode(JCHiGridNode node, CellFormat cellFormat, SummaryColumn summaryColumn) {
    showValue();
}

/**
 * Show values for the new selected column.
 */
public void setSelectedColumn(CellFormat cellFormat, SummaryColumn summaryColumn) {
    super.setSelectedColumn(cellFormat, summaryColumn);
    customCellEditor.setText("");
    customCellRenderer.setText("");
    showValue();
}

/** Calls showValue() to determine if changes are made.
 */
public void actionPerformed(ActionEvent e) {
    if (isInitializing()) return;

    valueChanged();
}

/** Calls showValue() to determine if changes are made.
 */
public void itemStateChanged(ItemEvent e) {
    if (isInitializing()) return;

    valueChanged();
}

}
