/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CellFontPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:21 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid.customizer;

import java.awt.*;
import java.awt.event.*;

import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import com.klg.jclass.higrid.*;
import com.klg.jclass.higrid.beans.*;

/**
 * This is the font properties page for the specified cell.
 * When a <code>JCHiGridNode</code> is set, showValue() is called
 * to display property values.
 * When a property value is changed, valueChanged() is called.
 */
public class CellFontPage extends CellFormatPage implements ItemListener {

// common properties
FontEditor cellFontEditor;

/**
 * Constructors
 */
public CellFontPage() {
    super(LocaleBundle.string(LocaleBundle.font));
    init();
}

/**
 * Initialize the UI.
 */
public void init() {
    setLayout(new BorderLayout());
    JPanel panel = new JPanel();
    panel.setBorder(new TitledBorder(LocaleBundle.string(LocaleBundle.font)));
    cellFontEditor = new FontEditor(this);
    panel.add(cellFontEditor);
    add(panel, BorderLayout.NORTH);
}

/** Overrides to perform operations necessary after value is changed on this page.
 */
public void valueChanged() {
    if (cellFormat == null) {
        return;
    }

    Font font = cellFontEditor.getSelectedFont();
    if (font != null && !font.equals(cellFormat.getFont())) {
        cellFormat.setFont(getFontMetrics(font), font);
        getJCHiGridNode().setChanged();
        getJCHiGridNode().getGridProperties().fireJCHiGridEvent(new JCHiGridEvent(getJCHiGridNode().getGridProperties(), getJCHiGridNode()));
    }
}

/** Overrides to refresh values displayed on this page after the underlined
 *	<code>JCHiGridNode</code> is changed.
 */
public void showValue() {
    if (cellFormat == null) {
        return;
    }
    cellFontEditor.selectFont(cellFormat.getFont());
}

/** Loads a <code>JCHiGridNode</code> into the page.
 */
public void setJCHiGridNode(JCHiGridNode node, CellFormat cellFormat, SummaryColumn summaryColumn) {
    showValue();
}

/**
 * Show values for the new selected column.
 */
public void setSelectedColumn(CellFormat cellFormat, SummaryColumn summaryColumn) {
    super.setSelectedColumn(cellFormat, summaryColumn);
    showValue();
}

/** Calls showValue() to determine if changes are made.
 */
public void itemStateChanged(ItemEvent e) {
    if (isInitializing()) return;

    valueChanged();
}

}
