/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CellGeneralPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:21 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid.customizer;

import java.awt.*;
import java.awt.event.*;
import java.util.StringTokenizer;

import javax.swing.*;

import com.klg.jclass.cell.JCCellInfo;
import com.klg.jclass.higrid.*;
import com.klg.jclass.higrid.beans.*;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.swing.JCGridLayout;

/**
 * This is the general properties page for the specified cell.
 * When a <code>JCHiGridNode</code> is set, showValue() is called
 * to display property values.
 * When a property value is changed, valueChanged() is called.
 */
public class CellGeneralPage extends CellFormatPage implements ActionListener,
    ItemListener {

// common properties
JTextField text;
JTextField width;
JCheckBox showing;
JCheckBox allowWidthSizing;
JComboBox horizontalAlignment;
JComboBox verticalAlignment;
JTextField marginInsets;
JComboBox borderStyle;
JTextField borderInsets;
JComboBox clipHints;

boolean useTextField;
boolean summaryCellPage;

/**
 * Constructors
 */
public CellGeneralPage() {
    super(LocaleBundle.string(LocaleBundle.general));
    init();
}

/**
 * Initialize the UI.
 */
public void init() {
    setLayout(new JCGridLayout(JCGridLayout.VARIABLE, 2, 5, 5));

    FocusListener fl = new FocusAdapter() {
        public void focusLost(FocusEvent e) {
            if (isInitializing()) return;

            valueChanged();
        }
    };

    if (section == JCHiGridNode.SECTION_RECORD) {
        useTextField = false;
    }
    else {
        useTextField = true;
    }

    if (section == JCHiGridNode.SECTION_FOOTER ||
        section == JCHiGridNode.SECTION_BEFORE_DETAILS ||
        section == JCHiGridNode.SECTION_AFTER_DETAILS) {
        summaryCellPage = true;
    }
    else {
        summaryCellPage = false;
    }

    text = new JTextField("", 15);
    width = new JTextField("", 15);
    showing = new JCheckBox(LocaleBundle.string(LocaleBundle.cellFormatShowing), true);
    allowWidthSizing = new JCheckBox(LocaleBundle.string(LocaleBundle.allowWidthSizing), true);
    horizontalAlignment = new JComboBox();
    verticalAlignment = new JComboBox();
    marginInsets = new JTextField("", 15);
    borderStyle = new JComboBox();
    borderInsets = new JTextField("", 15);
    clipHints = new JComboBox();
    JLabel lbl = new JLabel("");
    JLabel lbl2 = new JLabel("");
    JLabel lbl_header = new JLabel(LocaleBundle.string(LocaleBundle.headerText));
    JLabel lbl_width = new JLabel(LocaleBundle.string(LocaleBundle.width));
    JLabel lbl_halign = new JLabel(LocaleBundle.string(LocaleBundle.horizontalAlignment));
    JLabel lbl_valign = new JLabel(LocaleBundle.string(LocaleBundle.verticalAlignment));
    JLabel lbl_margin = new JLabel(LocaleBundle.string(LocaleBundle.marginInsets));
    JLabel lbl_border_style = new JLabel(LocaleBundle.string(LocaleBundle.border_style));
    JLabel lbl_border_insets = new JLabel(LocaleBundle.string(LocaleBundle.border_insets));
    JLabel lbl_clip_hints = new JLabel(LocaleBundle.string(LocaleBundle.clipHints));

    if (useTextField) {
        text.addActionListener(this);
        text.addFocusListener(fl);
    }
    width.addActionListener(this);
    width.addFocusListener(fl);
    showing.addItemListener(this);
    allowWidthSizing.addItemListener(this);
    String choicesHA[] = JCHiGridConverter.getCellHorizontalAlignmentStrings();
    for (int i = 0; i < choicesHA.length; i++) {
        horizontalAlignment.addItem(choicesHA[i]);
    }
    horizontalAlignment.addItemListener(this);
    String choicesVA[] = JCHiGridConverter.getCellVerticalAlignmentStrings();
    for (int i = 0; i < choicesVA.length; i++) {
        verticalAlignment.addItem(choicesVA[i]);
    }
    verticalAlignment.addItemListener(this);
    marginInsets.addActionListener(this);
    marginInsets.addFocusListener(fl);
    String choicesBS[] = JCHiGridConverter.getRowFormatBorderStyleStrings();
    for (int i = 0; i < (choicesBS.length-1); i++) { // exclude mixed style
        borderStyle.addItem(choicesBS[i]);
    }
    borderStyle.addItemListener(this);
    borderInsets.addActionListener(this);
    borderInsets.addFocusListener(fl);
    String choicesCH[] = JCHiGridConverter.getCellClipHintsStrings();
    for (int i = 0; i < choicesCH.length; i++) {
        clipHints.addItem(choicesCH[i]);
    }
    clipHints.addItemListener(this);

    add(lbl);
    add(showing);
    if (useTextField) {
        add(lbl_header);
        add(text);
    }
    add(lbl_width);
    add(width);
    add(lbl2);
    add(allowWidthSizing);
    add(lbl_halign);
    add(horizontalAlignment);
    add(lbl_valign);
    add(verticalAlignment);
    add(lbl_margin);
    add(marginInsets);
    add(lbl_border_style);
    add(borderStyle);
    add(lbl_border_insets);
    add(borderInsets);
    add(lbl_clip_hints);
    add(clipHints);
}

/** Overrides to perform operations necessary after value is changed on this page.
 */
public void valueChanged() {
    if (cellFormat == null) {
        return;
    }

    boolean changed = false;

    String s = null;
    if (useTextField) {
        s = text.getText();
        if (s.equals("")) {
            s = null;
        }
        changed = !(new String((s==null) ? "": s).equals(
                                                        new String((cellFormat.getText()==null) ? "" :
                                                                   cellFormat.getText()) )
                   );
        cellFormat.setText(s);
        if (summaryCellPage && summaryColumn != null && 
            summaryColumn.getColumnType() == SummaryColumn.COLUMN_TYPE_LABEL) {
            summaryColumn.setName(s);
            summaryColumn.setIdentifier();
            cellFormat.setName(summaryColumn.getIdentifier());
        }
    }

    boolean is_showing = showing.isSelected();
    boolean allow_resize = allowWidthSizing.isSelected();

    if (cellFormat.isShowing() != is_showing) {
        changed = true;
        cellFormat.setShowing(is_showing);
        CellFormat other = getOtherCellFormat();
        if (other != null) {
            other.setShowing(is_showing);
        }
    }

    if (cellFormat.getAllowWidthSizing() != allow_resize) {
        changed = true;
        cellFormat.setAllowWidthSizing(allow_resize);
        CellFormat other = getOtherCellFormat();
        if (other != null) {
            other.setAllowWidthSizing(allow_resize);
        }
    }

    boolean convertStatus = true;
    s = width.getText();
    int widthValue = 0;
    try {
        widthValue = new Integer(s).intValue();
    }
    catch (Exception e) {
        convertStatus = false;
    }
    if (convertStatus) {
        changed = true;
        cellFormat.setWidth(widthValue);
        CellFormat other = getOtherCellFormat();
        if (other != null) {
            other.setWidth(widthValue);
        }
    }

    convertStatus = true;
    s = marginInsets.getText();
    StringTokenizer st = new StringTokenizer(s, ",");
    int insets[] = { 2, 2, 2, 2};
    for (int i=0; i<4; i++) {
        if (st.hasMoreTokens()) {
            try {
                insets[i] = new Integer(st.nextToken()).intValue();
            }
            catch (Exception e) {
                convertStatus = false;
            }
        }
    }
    if (convertStatus) {
        cellFormat.setMarginInsets(new Insets(insets[0], 
                                              insets[1], 
                                              insets[2], 
                                              insets[3]));
        changed = true;
    }

    s = (String) horizontalAlignment.getSelectedItem();
    JCTypeConverter conv = new JCTypeConverter();
    int alignment = JCHiGridConverter.toCellHorizontalAlignment(conv, s, JCCellInfo.LEFT);
    if (cellFormat.getHorizontalAlignment() != alignment) {
        changed = true;
    }
    cellFormat.setHorizontalAlignment(alignment);

    s = (String) verticalAlignment.getSelectedItem();
    alignment = JCHiGridConverter.toCellVerticalAlignment(conv, s, JCCellInfo.CENTER);
    if (cellFormat.getVerticalAlignment() != alignment) {
        changed = true;
    }
    cellFormat.setVerticalAlignment(alignment);

    convertStatus = true;
    s = borderInsets.getText();
    st = new StringTokenizer(s, ",");
    int insetsBorder[] = { 2, 2, 2, 2};
    for (int i=0; i<4; i++) {
        if (st.hasMoreTokens()) {
            try {
                insetsBorder[i] = new Integer(st.nextToken()).intValue();
            }
            catch (Exception e) {
                convertStatus = false;
            }
        }
    }
    if (convertStatus) {
        cellFormat.setBorderInsets(new Insets(insetsBorder[0], 
                                              insetsBorder[1], 
                                              insetsBorder[2], 
                                              insetsBorder[3]));
        changed = true;
    }

    s = (String) borderStyle.getSelectedItem();
    conv = new JCTypeConverter();
    int style = JCHiGridConverter.toRowFormatBorderStyle(conv, s, HiGridBorder.ETCHED_IN);
    if (cellFormat.getBorderStyle() != style) {
        changed = true;
    }
    cellFormat.setBorderStyle(style);

    s = (String) clipHints.getSelectedItem();
    int hint = JCHiGridConverter.toCellClipHints(conv, s, JCCellInfo.SHOW_ALL);
    if (cellFormat.getClipHints() != hint) {
        changed = true;
    }
    cellFormat.setClipHints(hint);

    if (changed) {
        // notify observers of changes, if any.
        getJCHiGridNode().setChanged();
        getJCHiGridNode().getGridProperties().fireJCHiGridEvent(new JCHiGridEvent(getJCHiGridNode().getGridProperties(), getJCHiGridNode()));
    }
}

/** Overrides to refresh values displayed on this page after the underlined
 *	<code>JCHiGridNode</code> is changed.
 */
public void showValue() {
    if (cellFormat == null) {
        return;
    }
    if (useTextField) {
        String s = cellFormat.getText();
        if (s == null) {
            s = new String("");
        }
        text.setText(new String(s));
    }
    showing.setSelected(cellFormat.isShowing());
    allowWidthSizing.setSelected(cellFormat.getAllowWidthSizing());
    width.setText(new Integer(cellFormat.getWidth()).toString());
    Insets i = cellFormat.getMarginInsets();
    marginInsets.setText(new String(i.top + "," + i.left + "," + i.bottom + "," + i.right));
    int alignment = cellFormat.getHorizontalAlignment();
    int index = -1;
    for (int j = 0; j < JCHiGridConverter.cellHorizontalAlignmentStrings.length; j++) {
        if (alignment == JCHiGridConverter.cellHorizontalAlignmentValues[j]) {
            index = j;
            break;
        }
    }
    index = Math.max(0, index);
    horizontalAlignment.setSelectedIndex(index);
    alignment = cellFormat.getVerticalAlignment();
    index = -1;
    for (int j = 0; j < JCHiGridConverter.cellVerticalAlignmentStrings.length; j++) {
        if (alignment == JCHiGridConverter.cellVerticalAlignmentValues[j]) {
            index = j;
            break;
        }
    }
    index = Math.max(0, index);
    verticalAlignment.setSelectedIndex(index);
    Insets i2 = cellFormat.getBorderInsets();
    borderInsets.setText(new String(i2.top + "," + i2.left + "," + i2.bottom + "," + i2.right));
    int style = cellFormat.getBorderStyle();
    index = -1;
    int defaultBorder = 0;
    for (int j = 0; j < (JCHiGridConverter.rowFormatBorderStyleValues.length-1); j++) {
        if (JCHiGridConverter.rowFormatBorderStyleValues[j] == HiGridBorder.ETCHED_IN) {
            defaultBorder = JCHiGridConverter.rowFormatBorderStyleValues[j];
        }
        if (style == JCHiGridConverter.rowFormatBorderStyleValues[j]) {
            index = j;
            break;
        }
    }
    if (index == -1) {
        index = defaultBorder;
    }
    borderStyle.setSelectedIndex(index);
    int hint = cellFormat.getClipHints();
    index = -1;
    for (int j = 0; j < JCHiGridConverter.cellClipHintsStrings.length; j++) {
        if (hint == JCHiGridConverter.cellClipHintsValues[j]) {
            index = j;
            break;
        }
    }
    index = Math.max(0, index);
    clipHints.setSelectedIndex(index);
}

/** Loads a <code>JCHiGridNode</code> into the page.
 */
public void setJCHiGridNode(JCHiGridNode node, CellFormat cellFormat, SummaryColumn summaryColumn) {
    showValue();
}

/**
 * Show values for the new selected column.
 */
public void setSelectedColumn(CellFormat cellFormat, SummaryColumn summaryColumn) {
    super.setSelectedColumn(cellFormat, summaryColumn);
    showValue();
}

/** Calls showValue() to determine if changes are made.
 */
public void actionPerformed(ActionEvent e) {
    if (isInitializing()) return;

    valueChanged();
}

/** Calls showValue() to determine if changes are made.
 */
public void itemStateChanged(ItemEvent e) {
    if (isInitializing()) return;

    valueChanged();
}

/** 
 * Gets the other cell format in a header/record pair
 * if necessary.
 */
CellFormat getOtherCellFormat() {
    if (summaryCellPage) {
        // no need for summary sections
        return null;
    }
    if (cellFormat.getOtherCellFormat() != null) {
        // no need if higrid is taking care it
        return null;
    }
    CellFormat other = null;
    JCHiGridNode node = getJCHiGridNode();
    int index = node.getRowFormat(section).getDataFormats().indexOf(cellFormat);
    RowFormat rowFormat = null;
    if (section == JCHiGridNode.SECTION_HEADER) {
        rowFormat = node.getRowFormat(JCHiGridNode.SECTION_RECORD);
    }
    else if (section == JCHiGridNode.SECTION_RECORD) {
        rowFormat = node.getRowFormat(JCHiGridNode.SECTION_HEADER);
    }
    else {
        return null;
    }
    if (index != (-1) && rowFormat != null) {
        other = ((CellFormat)rowFormat.getDataFormats().elementAt(index));
    }
    return other;
}

}
