/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CellSummaryColumnPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:21 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid.customizer;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.tree.*;

import com.klg.jclass.datasource.*;
import com.klg.jclass.higrid.*;
import com.klg.jclass.higrid.beans.*;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.swing.JCAlignLayout;
import com.klg.jclass.higrid.LocaleBundle;

/**
 * This is the summary column properties page for the specified cell.
 * When a <code>JCHiGridNode</code> is set, showValue() is called
 * to display property values.
 * When a property value is changed, valueChanged() is called.
 */
public class CellSummaryColumnPage extends CellFormatPage implements
    ActionListener, ItemListener, ChooserDialogParent {

// common properties
JComboBox columnType;
JTextField datasourceTable;
JButton datasourceTableButton;
JTextField datasourceColumn;
JButton datasourceColumnButton;
JComboBox aggregateType;
JComboBox dataType;
JTextField unboundClass;
JRadioButton selectCellRenderer;
JComboBox JCCellRenderer;
JRadioButton selectCustomCellRenderer;
JTextField customCellRenderer;

JLabel labelColumnType;
JLabel labelDatasourceTable;
JLabel labelDatasourceColumn;
JLabel labelAggregateType;
JLabel labelDataType;
JLabel labelUnboundClass;
ButtonGroup cbGroup1;

transient Frame frame;
transient boolean useOwnFrame;
transient boolean tableChooser;
transient boolean tableChanged;
transient Vector tableVector;
transient Vector tableValueVector;
transient int tableIndex;
transient boolean columnChanged;
transient Vector columnVector;
transient int columnIndex;

/**
 * Constructors
 */
public CellSummaryColumnPage() {
    super(LocaleBundle.string(LocaleBundle.cellSummaryColumn));
    init();
}

/**
 * Initialize the UI.
 */
public void init() {
    setLayout(new BorderLayout());

    FocusListener fl = new FocusAdapter() {
        public void focusLost(FocusEvent e) {
            if (isInitializing()) return;

            valueChanged();
        }
    };

    String stringNone = new String(LocaleBundle.string(LocaleBundle.choiceStringNone));
    JPanel panel1 = new JPanel();
    columnType = new JComboBox();
    JPanel p1 = new JPanel(new BorderLayout(2, 2));
    datasourceTable = new JTextField(stringNone, 16);
    datasourceTableButton = new JButton("...");
    JPanel p2 = new JPanel(new BorderLayout(2, 2));
    datasourceColumn = new JTextField(stringNone, 16);
    datasourceColumnButton = new JButton("...");
    aggregateType = new JComboBox();
    dataType = new JComboBox();
    unboundClass = new JTextField("", 20);
    labelColumnType = new JLabel(LocaleBundle.string(LocaleBundle.summaryColumnType));
    labelDatasourceTable = new JLabel(LocaleBundle.string(LocaleBundle.datasourceTable));
    labelDatasourceColumn = new JLabel(LocaleBundle.string(LocaleBundle.datasourceColumn));
    labelAggregateType = new JLabel(LocaleBundle.string(LocaleBundle.aggregateType));
    labelDataType = new JLabel(LocaleBundle.string(LocaleBundle.dataType));
    labelUnboundClass = new JLabel(LocaleBundle.string(LocaleBundle.unboundClass));
    JPanel panel2 = new JPanel();
    cbGroup1 = new ButtonGroup();
    selectCellRenderer = new JRadioButton("", true);
    JCCellRenderer = new JComboBox();
    selectCustomCellRenderer = new JRadioButton("", false);
    customCellRenderer = new JTextField("", 30);
    cbGroup1.add(selectCellRenderer);
    cbGroup1.add(selectCustomCellRenderer);

    JPanel group1 = new JPanel();
    group1.setBorder(new TitledBorder(LocaleBundle.string(LocaleBundle.renderer)));
    group1.setLayout(new JCAlignLayout());

    // first panel
    panel1.setLayout(new JCAlignLayout());

    String choicesCT[] = JCHiGridConverter.getSummaryColumnTypeStrings();
    for (int i = 0; i < choicesCT.length; i++) {
        columnType.addItem(choicesCT[i]);
    }
    columnType.addItemListener(this);

    datasourceTable.setEditable(false);
    datasourceTableButton.addActionListener(this);
    p1.add(datasourceTable, BorderLayout.CENTER);
    p1.add(datasourceTableButton, BorderLayout.EAST);

    datasourceColumn.setEditable(false);
    datasourceColumnButton.addActionListener(this);
    p2.add(datasourceColumn, BorderLayout.CENTER);
    p2.add(datasourceColumnButton, BorderLayout.EAST);

    String choicesAT[] = JCHiGridConverter.getAggregateTypeStrings();
    for (int i = 0; i < choicesAT.length; i++) {
        aggregateType.addItem(choicesAT[i]);
    }
    aggregateType.addItemListener(this);

    String choicesDT[] = JCHiGridConverter.getMetaDataModelTypeStrings();
    for (int i = 0; i < choicesDT.length; i++) {
        dataType.addItem(choicesDT[i]);
    }
    dataType.addItemListener(this);

    unboundClass.addActionListener(this);
    unboundClass.addFocusListener(fl);

    panel1.add(labelColumnType);
    panel1.add(columnType);
    panel1.add(labelDatasourceTable);
    panel1.add(p1);
    panel1.add(labelDatasourceColumn);
    panel1.add(p2);
    panel1.add(labelAggregateType);
    panel1.add(aggregateType);
    panel1.add(labelDataType);
    panel1.add(dataType);
    panel1.add(labelUnboundClass);
    panel1.add(unboundClass);
    add(panel1, BorderLayout.NORTH);

    // second panel
    panel2.setLayout(new BorderLayout());

    selectCellRenderer.addItemListener(this);

    if (CellEditPage.cellRendererClasses != null) {
        int size = CellEditPage.cellRendererClasses.size();
        for (int i = 0; i < size; i++) {
            JCCellRenderer.addItem((String)CellEditPage.cellRendererClasses.elementAt(i));
        }
    }
    JCCellRenderer.addItemListener(this);
    selectCustomCellRenderer.addItemListener(this);
    customCellRenderer.addActionListener(this);
    customCellRenderer.addFocusListener(fl);

    group1.add(selectCellRenderer);
    group1.add(JCCellRenderer);
    group1.add(selectCustomCellRenderer);
    group1.add(customCellRenderer);
    panel2.add(group1, BorderLayout.NORTH);
    add(panel2, BorderLayout.CENTER);

    disableAll();
}

/**
 * Disable all controls.
 */
void disableAll() {
    labelColumnType.setEnabled(false);
    columnType.setEnabled(false);
    labelDatasourceTable.setEnabled(false);
    datasourceTableButton.setEnabled(false);
    labelDatasourceColumn.setEnabled(false);
    datasourceColumnButton.setEnabled(false);
    labelAggregateType.setEnabled(false);
    aggregateType.setEnabled(false);
    labelDataType.setEnabled(false);
    dataType.setEnabled(false);
    labelUnboundClass.setEnabled(false);
    unboundClass.setEnabled(false);
    selectCellRenderer.setEnabled(false);
    JCCellRenderer.setEnabled(false);
    selectCustomCellRenderer.setEnabled(false);
    customCellRenderer.setEnabled(false);
}

/** Overrides to perform operations necessary after value is changed on this page.
 */
public void valueChanged() {
    if (cellFormat == null || summaryColumn == null) {
        disableAll();
        return;
    }

    boolean changed = false;

    // column type change sets certain default values in the other fields
    // invoke showValue() to enable/disable the other fields
    String s = (String) columnType.getSelectedItem();
    JCTypeConverter conv = new JCTypeConverter();
    int type = JCHiGridConverter.toSummaryColumnType(conv, s, SummaryColumn.COLUMN_TYPE_LABEL);
    int prevColumnType = summaryColumn.getColumnType();
    if (prevColumnType != type) {
        changed = true;
        switch (type) {
        default:            
        case SummaryColumn.COLUMN_TYPE_LABEL:
            summaryColumn.init(0, 
                               cellFormat.getText(), 
                               SummaryColumn.COLUMN_TYPE_LABEL, 
                               SummaryColumn.AGGREGATE_TYPE_NONE, 
                               MetaDataModel.TYPE_STRING);
            break;
        case SummaryColumn.COLUMN_TYPE_DATASOURCE:            
            summaryColumn.init(summaryColumn.getTable(), 
                               (prevColumnType == SummaryColumn.COLUMN_TYPE_AGGREGATE) ?
                               summaryColumn.getName() : "", 
                               SummaryColumn.COLUMN_TYPE_DATASOURCE, 
                               SummaryColumn.AGGREGATE_TYPE_NONE, 
                               summaryColumn.getDataType());
            break;
        case SummaryColumn.COLUMN_TYPE_AGGREGATE:            
            summaryColumn.init(summaryColumn.getTable(), 
                               (prevColumnType == SummaryColumn.COLUMN_TYPE_DATASOURCE) ?
                               summaryColumn.getName() : "", 
                               SummaryColumn.COLUMN_TYPE_AGGREGATE, 
                               summaryColumn.getDefaultAggregateType(summaryColumn.getDataType()), 
                               summaryColumn.getDataType());
            break;
        case SummaryColumn.COLUMN_TYPE_UNBOUND:            
            summaryColumn.init(summaryColumn.getTable(), 
                               "", 
                               SummaryColumn.COLUMN_TYPE_UNBOUND, 
                               SummaryColumn.AGGREGATE_TYPE_NONE, 
                               summaryColumn.getDataType());
            break;
        }
        updateDefaultDataType();
        updateColumnIdentifier();
        showValue();
        // notify observers of changes, if any.
        getJCHiGridNode().setChanged();
        getJCHiGridNode().getGridProperties().fireJCHiGridEvent(new JCHiGridEvent(getJCHiGridNode().getGridProperties(), getJCHiGridNode()));
        return;
    }

    // datasource table
    if (tableChanged) {
        tableChanged = false;
        changed = true;
        if (tableIndex < 0) {
            summaryColumn.setTable(0);
        }
        else {
            summaryColumn.setTable(((Integer)tableValueVector.elementAt(tableIndex)).intValue());
        }
        summaryColumn.setIdentifier();
        updateDefaultDataType();
        updateColumnIdentifier();
    }

    // datasource column
    if (columnChanged) {
        columnChanged = false;
        changed = true;
        if (columnIndex < 0) {
            summaryColumn.setName("");
        }
        else {
            summaryColumn.setName((String)columnVector.elementAt(columnIndex));
        }
        summaryColumn.setIdentifier();
        updateDefaultDataType();
        updateColumnIdentifier();
    }

    // aggregate type
    s = (String) aggregateType.getSelectedItem();
    int aggregate = JCHiGridConverter.toAggregateType(conv, s, SummaryColumn.AGGREGATE_TYPE_NONE);
    if (summaryColumn.getAggregateType() != aggregate) {
        changed = true;
        summaryColumn.setAggregateType(aggregate);
        summaryColumn.setIdentifier();
        updateDefaultDataType();
        updateColumnIdentifier();
    }

    // data type
    s = (String) dataType.getSelectedItem();
    int data = JCHiGridConverter.toMetaDataModelType(conv, s, MetaDataModel.TYPE_OBJECT);
    if (summaryColumn.getDataType() != data) {
        changed = true;
        summaryColumn.setDataType(data);
    }

    // unbound class 
    if (type == SummaryColumn.COLUMN_TYPE_UNBOUND) {
        s = unboundClass.getText();
        if (!s.equals(summaryColumn.getName())) {
            changed = true;
            summaryColumn.setName(s);
            summaryColumn.setIdentifier();
            updateDefaultDataType();
            updateColumnIdentifier();
        }
    }

    // cell renderer
    if (selectCellRenderer.isSelected()) {
        s = new String((String) JCCellRenderer.getSelectedItem());
    }
    else {
        s = customCellRenderer.getText();
    }
    if (s == null || s.equals("")) {
        ErrorDialog.showError(this, 
                              LocaleBundle.string(LocaleBundle.noRendererClassSupplied));
    }
    else {
        try {
            String name = cellFormat.getCellRenderer().getName();
            if (!s.equals(name)) {
                changed = true;
                cellFormat.setCellRenderer(Class.forName(s));
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
            ErrorDialog.showError(this, ex.toString());
        }
    }

    if (changed) {
        // notify observers of changes, if any.
        getJCHiGridNode().setChanged();
        getJCHiGridNode().getGridProperties().fireJCHiGridEvent(new JCHiGridEvent(getJCHiGridNode().getGridProperties(), getJCHiGridNode()));
    }
}

/** Overrides to refresh values displayed on this page after the underlined
 *	<code>JCHiGridNode</code> is changed.
 */
public void showValue() {
    if (cellFormat == null || summaryColumn == null) {
        disableAll();
        return;
    }

    // Column Type (always enabled)
    if (!columnType.isEnabled()) {
        columnType.setEnabled(true);
        labelColumnType.setEnabled(true);
    }
    int type = summaryColumn.getColumnType();
    int index = -1;
    for (int j = 0; j < JCHiGridConverter.summaryColumnTypeStrings.length; j++) {
        if (type == JCHiGridConverter.summaryColumnTypeValues[j]) {
            index = j;
            break;
        }
    }
    index = Math.max(0, index);
    columnType.setSelectedIndex(index);

    // DataSource Table (disabled only if LABEL)
    if (type == SummaryColumn.COLUMN_TYPE_LABEL) {
        if (datasourceTableButton.isEnabled()) {
            datasourceTableButton.setEnabled(false);    
            labelDatasourceTable.setEnabled(false);    
        }
        datasourceTable.setText(LocaleBundle.string(LocaleBundle.choiceStringNone));
    }
    else {
        if (!datasourceTableButton.isEnabled()) {
            datasourceTableButton.setEnabled(true);
            labelDatasourceTable.setEnabled(true);    
        }
        datasourceTable.setText(getMetaDataDescription(summaryColumn.getTable()));
    }

    // DataSource Column (disabled only if LABEL or UNBOUNDED)
    if (type == SummaryColumn.COLUMN_TYPE_LABEL || 
        type == SummaryColumn.COLUMN_TYPE_UNBOUND) {
        if (datasourceColumnButton.isEnabled()) {
            datasourceColumnButton.setEnabled(false);    
            labelDatasourceColumn.setEnabled(false);    
        }
        datasourceColumn.setText(LocaleBundle.string(LocaleBundle.choiceStringNone));
    }
    else {
        if (!datasourceColumnButton.isEnabled()) {
            datasourceColumnButton.setEnabled(true);
            labelDatasourceColumn.setEnabled(true);    
        }
        String s = summaryColumn.getName();
        if (s == null || s.length() == 0) {
            s = new String(LocaleBundle.string(LocaleBundle.choiceStringNone));
        }
        datasourceColumn.setText(s);
    }

    // Aggregate Type (enabled only if AGGREGATE)
    int aggregate = -1;
    if (type == SummaryColumn.COLUMN_TYPE_AGGREGATE) {
        if (!aggregateType.isEnabled()) {
            aggregateType.setEnabled(true);
            labelAggregateType.setEnabled(true);
        }
        aggregate = summaryColumn.getAggregateType();
    }
    else {
        if (aggregateType.isEnabled()) {
            aggregateType.setEnabled(false);    
            labelAggregateType.setEnabled(false);
        }
        aggregate = SummaryColumn.AGGREGATE_TYPE_NONE;
    }
    index = -1;
    for (int j = 0; j < JCHiGridConverter.aggregateTypeStrings.length; j++) {
        if (aggregate == JCHiGridConverter.aggregateTypeValues[j]) {
            index = j;
            break;
        }
    }
    index = Math.max(0, index);
    aggregateType.setSelectedIndex(index);

    // Data Type (enabled only if UNBOUNDED)
    int data = -1;
    if (type == SummaryColumn.COLUMN_TYPE_UNBOUND) {
        if (!dataType.isEnabled()) {
            dataType.setEnabled(true);
            labelDataType.setEnabled(true);    
        }
    }
    else {
        if (dataType.isEnabled()) {
            dataType.setEnabled(false);    
            labelDataType.setEnabled(false);    
        }
    }
    data = summaryColumn.getDataType();
    setDataType(data);

    // unbound class (enabled only if UNBOUNDED)
    if (type == SummaryColumn.COLUMN_TYPE_UNBOUND) {
        if (!unboundClass.isEnabled()) {
            unboundClass.setEnabled(true);
            labelUnboundClass.setEnabled(true);
        }
        String name = summaryColumn.getName();
        unboundClass.setText(summaryColumn.getName() == null ? "" : name);
    }
    else {
        if (unboundClass.isEnabled()) {
            unboundClass.setEnabled(false);    
            labelUnboundClass.setEnabled(false);
        }
        unboundClass.setText("");
    }

    // cell renderer (always enabled)
    if (!selectCellRenderer.isEnabled()) {
        selectCellRenderer.setEnabled(true);
        JCCellRenderer.setEnabled(true);
        selectCustomCellRenderer.setEnabled(true);
        customCellRenderer.setEnabled(true);
    }
    Class renderer = cellFormat.getCellRenderer();
    if (renderer != null && CellEditPage.cellRendererClasses != null) {
        String rendererName = renderer.getName();
        index = -1;
        int size = CellEditPage.cellRendererClasses.size();
        for (int j = 0; j < size; j++) {
            if (rendererName.equals((String)CellEditPage.cellRendererClasses.elementAt(j))) {
                index = j;
                break;
            }
        }
        if (index != (-1)) {
            JCCellRenderer.setSelectedIndex(index);
            selectCellRenderer.setSelected(true);
        }
        else {
            customCellRenderer.setText(rendererName);
            selectCustomCellRenderer.setSelected(true);
        }
    }
}

void setDataType(int data) {
    int index = -1;
    for (int j = 0; j < JCHiGridConverter.metaDataModelTypeStrings.length; j++) {
        if (data == JCHiGridConverter.metaDataModelTypeValues[j]) {
            index = j;
            break;
        }
    }
    index = Math.max(0, index);
    dataType.setSelectedIndex(index);
}

/** Loads a <code>JCHiGridNode</code> into the page.
 */
public void setJCHiGridNode(JCHiGridNode node, CellFormat cellFormat, SummaryColumn summaryColumn) {
    showValue();
}

/**
 * Show values for the new selected column.
 */
public void setSelectedColumn(CellFormat cellFormat, SummaryColumn summaryColumn) {
    super.setSelectedColumn(cellFormat, summaryColumn);
    customCellRenderer.setText("");
    showValue();
}

/** Calls showValue() to determine if changes are made.
 */
public void actionPerformed(ActionEvent e) {
    if (isInitializing()) return;

    if (e.getSource() instanceof JButton) {
        JButton button = (JButton) e.getSource();
        if (button.equals(datasourceTableButton)) {
            getLevelTable();
        }
        else if (button.equals(datasourceColumnButton)) {
            getColumnField();
        }
    }
    valueChanged();
}

/** Calls showValue() to determine if changes are made.
 */
public void itemStateChanged(ItemEvent e) {
    if (isInitializing()) return;

    valueChanged();
}

void getLevelTable() {
    if (editor == null) {
        return;
    }
    tableChooser = true;
    ChooserDialog chooser = new ChooserDialog((ChooserDialogParent) this);
    chooser.show();
    if (useOwnFrame) {
        useOwnFrame = false;
        frame.dispose();
    }
}

void getColumnField() {
    if (editor == null) {
        return;
    }
    tableChooser = false;
    ChooserDialog chooser = new ChooserDialog((ChooserDialogParent) this);
    chooser.show();
    if (useOwnFrame) {
        useOwnFrame = false;
        frame.dispose();
    }
}

/** Gets the dialog title
 */
public String getTitle() {
    if (tableChooser) {
        return LocaleBundle.string(LocaleBundle.tableChooser);
    }
    return LocaleBundle.string(LocaleBundle.columnChooser);
}

/** Gets the choice list,
 */
public Vector getChoiceList() {
    Vector v = new Vector();
    TreeModel metaTree = editor.getMetaDataTree();
    if (metaTree == null) {
        return v;
    }
    MetaDataModel root = (MetaDataModel) metaTree.getRoot();
    if (root == null) {
        return v;
    }

    if (tableChooser) {
        tableValueVector = new Vector();
        addTable(v, tableValueVector, root);
        createTableList(v, tableValueVector, root);
        tableVector = v;
    }
    else {
        if (summaryColumn != null) {
            int metaID = summaryColumn.getTable();
            createColumnList(v, metaID, root);
            columnVector = v;
        }
    }
    return v;
}

void createTableList(Vector list, Vector tableValueVector, MetaDataModel root) {
    TreeIterator ti = ((DataSourceTreeNode) root).getIterator();
    while (ti.hasMoreElements()) {
        // get the meta data 
        MetaDataModel metaData = (MetaDataModel) ti.get();

        // create a selection
        addTable(list, tableValueVector, metaData);

        createTableList(list, tableValueVector, metaData);
        ti.nextElement();
    }
}

void addTable(Vector list, Vector tableValueVector, MetaDataModel metaData) {
    String table = new String(metaData.getDescription());
    if (table == null) {
        table = new String(" ");
    }
    list.addElement((Object)table);
    tableValueVector.addElement((Object) new Integer(metaData.getMetaID()));
}

void createColumnList(Vector list, int metaID, MetaDataModel root) {
    MetaDataModel metaTable = getMetaTable(metaID, root);
    if (metaTable != null) {
        int cols = 0;
        try {
            cols = metaTable.getColumnCount();
            for (int i = 0; i < cols; i++) {
                list.addElement((Object)metaTable.getColumnIdentifier(i));
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}

MetaDataModel getMetaTable(int metaID, MetaDataModel root) {
    if (root.getMetaID() == metaID) {
        return root;
    }
    TreeIterator ti = ((DataSourceTreeNode) root).getIterator();
    while (ti.hasMoreElements()) {
        MetaDataModel metaData = getMetaTable(metaID, (MetaDataModel) ti.get());
        if (metaData != null) {
            return metaData;
        }
        ti.nextElement();
    }
    return null;
}

/**	Choice selected in ChooserDialog is returned.
 */
public void choiceSelected(int index) {
    if (tableChooser) {
        tableIndex = index;
        tableChanged = true;
    }
    else {
        columnIndex = index;
        columnChanged = true;
    }
    if (tableChooser) {
        if (index < 0) {
            datasourceTable.setText(LocaleBundle.string(LocaleBundle.choiceStringNone));
        }
        else {
            datasourceTable.setText((String)tableVector.elementAt(index));
        }
    }
    else {
        if (index < 0) {
            datasourceColumn.setText(LocaleBundle.string(LocaleBundle.choiceStringNone));
        }
        else {
            datasourceColumn.setText((String)columnVector.elementAt(index));
        }
    }
}

String getMetaDataDescription(int metaID) {
    String s = new String(LocaleBundle.string(LocaleBundle.choiceStringNone));
    if (editor == null) {
        return s;
    }
    TreeModel metaTree = editor.getMetaDataTree();
    if (metaTree == null) {
        return s;
    }
    MetaDataModel root = (MetaDataModel) metaTree.getRoot();
    if (root == null) {
        return s;
    }
    MetaDataModel metaData = getMetaTable(metaID, root);
    if (metaData != null) {
        s = new String(metaData.getDescription());
    }
    return s;
}

void updateDefaultDataType() {
    if (summaryColumn.getColumnType() == SummaryColumn.COLUMN_TYPE_LABEL ||
        summaryColumn.getColumnType() == SummaryColumn.COLUMN_TYPE_UNBOUND) {
        return;
    }

    if (summaryColumn.getColumnType() == SummaryColumn.COLUMN_TYPE_AGGREGATE &&
        summaryColumn.getAggregateType() == SummaryColumn.AGGREGATE_TYPE_COUNT) {
        summaryColumn.setDataType(MetaDataModel.TYPE_INTEGER);
        setDataType(MetaDataModel.TYPE_INTEGER);
        return;
    }

    if (editor == null) {
        return;
    }
    TreeModel metaTree = editor.getMetaDataTree();
    if (metaTree == null) {
        return;
    }
    MetaDataModel root = (MetaDataModel) metaTree.getRoot();
    if (root == null) {
        return;
    }
    MetaDataModel metaData = getMetaTable(summaryColumn.getTable(), root);
    if (metaData != null) {
        try {
            int type = metaData.getJavaColumnType(summaryColumn.getName());
            summaryColumn.setDataType(type);
            setDataType(type);
        }
        catch (Exception e) {
            return;
        }
    }
}

void updateColumnIdentifier() {
    if (cellFormat == null || summaryColumn == null) {
        return;
    }
    cellFormat.setName(summaryColumn.getIdentifier());
}

}
