/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CellTabs.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:21 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid.customizer;

import java.awt.Insets;

import javax.swing.JTabbedPane;

import com.klg.jclass.higrid.*;
import com.klg.jclass.higrid.beans.JCHiGridNode;

/**
 * 	The tab manager for the cell tabs.
 */
public class CellTabs extends JTabbedPane {

private static final boolean TRACE = false;

public static final int CELL_GENERAL_PAGE = 0;
public static final int CELL_FONT_PAGE = 1;
public static final int CELL_COLOR_PAGE = 2;
public static final int CELL_EDIT_PAGE = 3;
public static final int CELL_SUMMARY_COLUMN_PAGE = 4;
public static final int NUM_OF_PAGES = CELL_SUMMARY_COLUMN_PAGE+1;

static CellFormatPage[] pages;

static int[] pagesHeader = {
    CELL_GENERAL_PAGE,
    CELL_FONT_PAGE,
    CELL_COLOR_PAGE,
};
static int[] pagesRecord = {
    CELL_GENERAL_PAGE,
    CELL_FONT_PAGE,
    CELL_COLOR_PAGE,
    CELL_EDIT_PAGE,
};
static int[] pagesSummary = {
    CELL_GENERAL_PAGE,
    CELL_FONT_PAGE,
    CELL_COLOR_PAGE,
    CELL_SUMMARY_COLUMN_PAGE,
};

JCHiGridNode node;
int section;

public CellTabs(int section) {
    super();
    this.section = section;
    init();
}

public void init() {
    // Create the tab manager
    if (pages == null) {
        pages = new CellFormatPage[NUM_OF_PAGES];
        pages[CELL_GENERAL_PAGE] = new CellGeneralPage();
        pages[CELL_FONT_PAGE] = new CellFontPage();
        pages[CELL_COLOR_PAGE] = new CellColorPage();
        pages[CELL_EDIT_PAGE] = new CellEditPage();
        pages[CELL_SUMMARY_COLUMN_PAGE] = new CellSummaryColumnPage();

        for (int i = 0; i < pages.length; i++) {
            try {
                CellFormatPage page = pages[i];
                addTab(page.getPageName(), page);
                setEnabledAt(i, false);
            }
            catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

	if (getTabCount() > 0) {
    	setSelectedIndex(CELL_GENERAL_PAGE);
	}
}

/**
 * Sets the GridPropertiesEditor.
 */
public void setEditor(GridPropertiesEditor editor) {
    for (int i = 0; i < pages.length; i++) {
        pages[i].setEditor(editor);
    }
}

public void setJCHiGridNode(JCHiGridNode node, CellFormat cellFormat, SummaryColumn summaryColumn) {
    this.node = node;
    CellFormatPage[] current_pages = getCurrentPages();
    for (int i = 0; i < current_pages.length; i++) {
        current_pages[i].init(node, cellFormat, summaryColumn);
    }
}

public void setSelectedColumn(CellFormat cellFormat, SummaryColumn summaryColumn) {
    if (cellFormat == null) {
        disableAllTabs();
    }
    else {
        enableTabs();
    }
    CellFormatPage[] current_pages = getCurrentPages();
    boolean currentPageInitializing;
    for (int i = 0; i < current_pages.length; i++) {
        currentPageInitializing = current_pages[i].isInitializing();
        current_pages[i].setInitializing(true);
        current_pages[i].setSelectedColumn(cellFormat, summaryColumn);
        current_pages[i].setInitializing(currentPageInitializing);
    }
}

public void setSection(int section) {
    if (this.section == section) return;

    this.section = section;
    updateTabs();
}

private int[] getCurrentPageIndices() {
    int[] current_indices = null;
    switch (section) {
    case JCHiGridNode.SECTION_HEADER:
        current_indices = pagesHeader;
        break;
    case JCHiGridNode.SECTION_RECORD:
        current_indices = pagesRecord;
        break;
    default: // Summary records
        current_indices = pagesSummary;
        break;
    }
    return current_indices;
}

private CellFormatPage[] getCurrentPages() {
    int[] current_indices = getCurrentPageIndices();

    int size = current_indices.length;
    CellFormatPage[] current_pages = new CellFormatPage[size];
    for (int i=0; i<current_indices.length; i++) {
        current_pages[i] = pages[current_indices[i]];
    }
    return current_pages;
}

private void updateTabs() {
    CellFormatPage[] current_pages = getCurrentPages();
    for (int i = 0; i < current_pages.length; i++) {
        try {
            CellFormatPage page = current_pages[i];
            page.init(section);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }
    enableTabs();
	if (getTabCount() > 0) {
	    setSelectedIndex(CELL_GENERAL_PAGE);
	}
}

private void enableTabs() {
	if (getTabCount() > 0) {
	    for (int i = 0; i < pages.length; i++) {
	        setEnabledAt(i, isTabUsed(i));
	        if (getComponentAt(i) != null) {
	            getComponentAt(i).setEnabled(isTabUsed(i));
	        }
	    }
	}
}

public void disableAllTabs() {
	if (getTabCount() > 0) {
	    for (int i = 0; i < pages.length; i++) {
	        setEnabledAt(i, false);
	        if (getComponentAt(i) != null) {
	            getComponentAt(i).setEnabled(false);
	        }
	    }
	}
}

private boolean isTabUsed(int index) {
    boolean using = false;
    int[] current_indices = getCurrentPageIndices();
    for (int i = 0; i < current_indices.length; i++) {
        if (current_indices[i] == index) {
            using = true;
            break;
        }
    }
    return using;
}

}

