/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ColorEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:21 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid.customizer;

import java.awt.*;
import java.awt.event.*;
import java.util.StringTokenizer;

import javax.swing.*;
import javax.swing.border.*;

import com.klg.jclass.higrid.LocaleBundle;
import com.klg.jclass.util.swing.*;

/**
 *	Editor for colors.
 */
public class ColorEditor extends JPanel {

// check codes if moving custom colorName to elsewhere
static String colorNames[] = {
    "  ", "Black", "Blue", "Cyan", "DarkGray", "Gray", "Green", "LightGray",
    "LightBlue", "Magenta", "Orange", "Pink", "Red", "Yellow", "White"
};

static int colorRed[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
static int colorGreen[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
static int colorBlue[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
static boolean colorSet = false;

JRadioButton selectCannedColor;
JRadioButton selectCustomColor;
JComboBox cannedColor;
JTextField customColor;

ItemListener itemListener;

boolean levelColor;

/**
 *	Constructs a color editor.
 */
public ColorEditor(ItemListener itemListener, ActionListener actionListener, 
                   FocusListener focusListener, String label, boolean levelColor) {
    super(new GridLayout(1, 1));

    this.itemListener = itemListener;

    if (!colorSet) {
        setCannedColors();
        colorSet = true;
    }

    this.levelColor = levelColor;

    JPanel group = new JPanel();
    group.setBorder(new TitledBorder(label));
    group.setLayout(new JCGridLayout(JCGridLayout.VARIABLE, 3, 5, 5));

    ButtonGroup cbGroup = new ButtonGroup();
    selectCannedColor = new JRadioButton("", true);
    cannedColor = new JComboBox();
    selectCustomColor = new JRadioButton("", false);
    customColor = new JTextField("", 10);
    cbGroup.add(selectCannedColor);
    cbGroup.add(selectCustomColor);

    selectCannedColor.addItemListener(itemListener);
    for (int i = 0; i < colorNames.length; i++) {
        cannedColor.addItem(colorNames[i]);
    }
    cannedColor.addItemListener(itemListener);
    selectCustomColor.addItemListener(itemListener);
    customColor.addActionListener(actionListener);
    if (focusListener != null) {
        customColor.addFocusListener(focusListener);
    }

    group.add(selectCannedColor);
    group.add(cannedColor);
    group.add(new JLabel());
    group.add(selectCustomColor);
    group.add(customColor);
    group.add(new JLabel(LocaleBundle.string(LocaleBundle.colorEditorRGB)));
    add(group);
}

public ColorEditor(ItemListener itemListener, ActionListener actionListener, 
                   String label, boolean levelColor) {
    this(itemListener, actionListener, null, label, levelColor);
}

public ColorEditor(ItemListener itemListener, ActionListener actionListener, 
                   String label) {
    this(itemListener, actionListener, label, false);
}

public void setEnabled(boolean v) {
    super.setEnabled(v);
    selectCannedColor.setEnabled(v);
    selectCustomColor.setEnabled(v);
    cannedColor.setEnabled(v);
    customColor.setEnabled(v);
}

/**
 * Get the canned color.
 */
Color getCannedColor(String colorName) {
    Color color = null;
    if (colorName.equals("Black")) {
        color = Color.black;
    }
    else if (colorName.equals("Blue")) {
        color = Color.blue;
    }
    else if (colorName.equals("Cyan")) {
        color = Color.cyan;
    }
    else if (colorName.equals("DarkGray")) {
        color = Color.darkGray;
    }
    else if (colorName.equals("Gray")) {
        color = Color.gray;
    }
    else if (colorName.equals("Green")) {
        color = Color.green;
    }
    else if (colorName.equals("LightBlue")) {
        color = JCSwingTypeConverter.lightBlue;
    }
    else if (colorName.equals("LightGray")) {
        color = Color.lightGray;
    }
    else if (colorName.equals("Magenta")) {
        color = Color.magenta;
    }
    else if (colorName.equals("Orange")) {
        color = Color.orange;
    }
    else if (colorName.equals("Pink")) {
        color = Color.pink;
    }
    else if (colorName.equals("Red")) {
        color = Color.red;
    }
    else if (colorName.equals("White")) {
        color = Color.white;
    }
    else if (colorName.equals("Yellow")) {
        color = Color.yellow;
    }
    else {
        // custom
        color = Color.black;
    }
    return color;
}

/**
 * Sets the canned RGB colors.
 */
void setCannedColors() {
    Color color = null;
    for (int i = 0; i < colorNames.length; i++) {
        String s = colorNames[i];
        color = getCannedColor(s);
        if (color != null) {
            colorRed[i] = color.getRed();
            colorGreen[i] = color.getGreen();
            colorBlue[i] = color.getBlue();
        }
    }
}

/**
 * Select the color.
 */
void selectColor(Color color) {
    if (color == null) {
        if (levelColor) {
            customColor.setText(new String(LocaleBundle.string(LocaleBundle.mixedCellColors)));
            if (cannedColor.getSelectedIndex() != 0) {
                cannedColor.setSelectedIndex(0);
            }
            if (!selectCustomColor.isSelected()) {
                selectCustomColor.setSelected(true);    
            }
        }
        return;
    }
    int red = color.getRed();
    int green = color.getGreen();
    int blue = color.getBlue();

    customColor.setText(new String(red + "," + green + "," + blue));

    int index = -1;
    // i == 0 is the custom color
    for (int i = 1; i < colorNames.length; i++) {
        if (colorRed[i] == red && colorGreen[i] == green && colorBlue[i] == blue) {
            index = i;
            break;
        }
    }
    index = Math.max(0, index);
    if (cannedColor.getSelectedIndex() != index) {
        cannedColor.setSelectedIndex(index);
    }
    if (index == 0) {
        if (!selectCustomColor.isSelected()) {
            selectCustomColor.setSelected(true);    
        }
    }
    else {
        if (!selectCannedColor.isSelected()) {
            selectCannedColor.setSelected(true);
        }
    }
}

/**
 * Get the color selected.
 */
Color getColor() {
    Color color = null;
    String cannedSelection = (String) cannedColor.getSelectedItem();
    if (selectCustomColor.isSelected() || cannedSelection.equals(colorNames[0])) {
        boolean convertStatus = true;
        String s = customColor.getText();
        if (levelColor && s.equals(LocaleBundle.string(LocaleBundle.mixedCellColors))) {
            return null;
        }
        StringTokenizer st = new StringTokenizer(s, ",");
        int colors[] = { 0, 0, 0};
        for (int i = 0; i < 3; i++) {
            if (st.hasMoreTokens()) {
                try {
                    colors[i] = new Integer(st.nextToken()).intValue();
                }
                catch (Exception e) {
                    convertStatus = false;
                }
            }
        }
        if (convertStatus) {
            color = new Color(colors[0], colors[1], colors[2]);
        }
        if (cannedColor.getSelectedIndex() != 0) {
            cannedColor.setSelectedIndex(0);
        }
    }
    else {
        color = getCannedColor(cannedSelection);
        selectColor(color);
    }

    return color; 
}

/**
 * Check if source is one of the color selection control.
 * If so, set the corresponding checkbox.
 */
boolean checkColorSource(Object source) {
    if (source.equals(cannedColor)) {
        if (!selectCannedColor.isSelected()) {
            selectCannedColor.setSelected(true);
        }
        return true;
    }
    else if (source.equals(customColor)) {
        if (!selectCustomColor.isSelected()) {
            selectCustomColor.setSelected(true);
        }
        return true;
    }
    return false;
}

}
