/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CustomizeTabs.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:21 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid.customizer;

import java.awt.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.tree.*;
import javax.swing.event.*;

import com.klg.jclass.datasource.*;
import com.klg.jclass.datasource.beans.*;
import com.klg.jclass.datasource.customizer.*;
import com.klg.jclass.util.JCEnvironment;
import com.klg.jclass.higrid.*;
import com.klg.jclass.higrid.beans.*;

/**
 * 	The UI portion of GridProperties editor.  It observes any changes to the
 *	underlining GridProperties and changes the current tab accordingly.
 */
public class CustomizeTabs extends JTabbedPane implements ChangeListener {

private static final boolean TRACE = false;

public static final int SERIALIZATION_PAGE = 0;
public static final int DATA_PAGE = 1;
public static final int FORMAT_PAGE = 2;

DataModelTabs dataTabs;
FormatTabs formatTabs;
SerializationPage serializationPage;
GridPropertiesEditor editor;
JCHiGridNode node;


public CustomizeTabs(GridPropertiesEditor editor) {
    super();
    this.editor = editor;
    init(editor);
}

public void init(GridPropertiesEditor editor) {
    dataTabs = editor.getDataModelTabs();
    serializationPage = new SerializationPage(editor);

    addTab(com.klg.jclass.datasource.LocaleBundle.string(com.klg.jclass.datasource.LocaleBundle.serialization), serializationPage);
    addTab(com.klg.jclass.datasource.LocaleBundle.string(com.klg.jclass.datasource.LocaleBundle.data), dataTabs);
    addTab(com.klg.jclass.datasource.LocaleBundle.string(com.klg.jclass.datasource.LocaleBundle.format), formatTabs);
    addChangeListener(this);
//FIXOBSOLETE
/*
if (JCEnvironment.getOS() == JCEnvironment.OS_UNIX) {
    setPreferredSize(new Dimension(450, 640));
} else {
    setPreferredSize(new Dimension(450, 500));
}
*/
    setSelectedIndex(SERIALIZATION_PAGE);
    setEnabledAt(DATA_PAGE, false);
    setEnabledAt(FORMAT_PAGE, false);
}

public Dimension getPreferredSize() {
    if (formatTabs != null) {
        return formatTabs.getPreferredSize();
    }
    else {
        return new Dimension(490,500);
    }
}

public void setJCHiGridNode(JCHiGridNode node) {
    setJCHiGridNode(node, false);
}

public void setJCHiGridNode(JCHiGridNode node, boolean updateNodeProperties) {
    this.node = node;

    if (formatTabs != null) {
        formatTabs.setJCHiGridNode(node);
    }

    if (updateNodeProperties || 
        !node.getNodeProperties().equals(dataTabs.getNodeProperties())) {
        if (TRACE) {
            System.out.println("Data is changed in: " + node.getNodeProperties());
        }
        dataTabs.setNodeProperties(node.getNodeProperties());
    }
}

public void stateChanged(ChangeEvent e) {
    tabChanged(getSelectedIndex());
}

private void tabChanged(int tab) {
    // Since construction of format tab is deferred to when 
    // the format tab button is clicked, the page will be
    // created if it has not been.
    if (formatTabs == null && tab == FORMAT_PAGE) {
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        formatTabs = new FormatTabs(editor);
        setComponentAt(FORMAT_PAGE, formatTabs);
        if (node != null) {
            formatTabs.setJCHiGridNode(node);
        }
        setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }
}

}   
