/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: FontEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:21 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid.customizer;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import com.klg.jclass.higrid.LocaleBundle;
import com.klg.jclass.util.JCTypeConverter;

/** Editor for fonts. */
public class FontEditor extends JPanel {

static final int MIXED_STYLE = (-1); 
static final int MIXED_SIZE = (-1); 

JComboBox familyChoser;
JComboBox styleChoser;
JComboBox sizeChoser;

private String fonts[];
private String[] style_names = { "plain", "bold", "italic", "bold and italic"};
private int[] styles = { Font.PLAIN, Font.BOLD, Font.ITALIC, Font.BOLD | Font.ITALIC};
private int[] point_sizes = { 3, 5, 8, 10, 12, 14, 18, 24, 28, 30, 32, 36, 48};

boolean levelFont;

/** Constructs a font editor.  */
public FontEditor(ItemListener listener) {
    this(listener, false);
}

public FontEditor(ItemListener listener, boolean levelFont) {
    super(new FlowLayout());



    fonts = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();

    this.levelFont = levelFont;
    familyChoser = new JComboBox();
    styleChoser = new JComboBox();
    sizeChoser = new JComboBox();
    if (levelFont) {
        familyChoser.addItem(LocaleBundle.string(LocaleBundle.mixedCellFonts));
        styleChoser.addItem(LocaleBundle.string(LocaleBundle.mixedCellFonts));
        sizeChoser.addItem(LocaleBundle.string(LocaleBundle.mixedCellFonts));
    }
    for (int i = 0; i < fonts.length; i++) {
        familyChoser.addItem(fonts[i]);
    }
    familyChoser.addItemListener(listener);
    add(familyChoser);

    for (int i = 0; i < style_names.length; i++) {
        styleChoser.addItem(style_names[i]);
    }
    styleChoser.addItemListener(listener);
    add(styleChoser);

    for (int i = 0; i < point_sizes.length; i++) {
        sizeChoser.addItem("" + point_sizes[i]);
    }
    sizeChoser.addItemListener(listener);
    add(sizeChoser);
}

/**
 * Set the font info in the font editor.
 */
void selectFontName(String name) {
    int index = levelFont ? 0 : 1;
    if (levelFont && name == null) {
        index = 0;
    }
    else {
        for (int i = 0; i < fonts.length; i++) {
            if (name.equals(fonts[i])) {
                index = levelFont ? (i+1) : i;
                break;
            }
        }
    }
}

void selectFontStyle(int style) {
    int index = levelFont ? 0 : 1;
    if (levelFont && style == MIXED_STYLE) {
        index = 0;
    }
    else {
        for (int i = 0; i < styles.length; i++) {
            if (style == styles[i]) {
                index = levelFont ? (i+1) : i;
                break;
            }
        }
    }
    styleChoser.setSelectedIndex(index);
}

void selectFontSize(int size) {
    int index = levelFont ? 0 : 4;
    if (levelFont && size == MIXED_SIZE) {
        index = 0;
    }
    else {
        for (int i = 0; i < point_sizes.length; i++) {
            if (size == point_sizes[i]) {
                index = levelFont ? (i+1) : i;
                break;
            }
        }
    }
    sizeChoser.setSelectedIndex(index);
}

void selectFont(Font font) {
    if (font == null) {
        return;
    }
    selectFontName(font.getName());
    selectFontStyle(font.getStyle());
    selectFontSize(font.getSize());
}

void selectFont(String name, int style, int size) {
    selectFontName(name);
    selectFontStyle(style);
    selectFontSize(size);
}

/**
 * Get the font info from the GUI.
 */
Font getSelectedFont() {
    String name = getFontName();
    int style = getFontStyle();
    int size = getFontSize();
    return(new Font(name, style, size)); 
}

String getFontName() {
    String name = (String) familyChoser.getSelectedItem();
    if (name == null) {
        name = "Serif";
    }
    else if (name.equals(LocaleBundle.string(LocaleBundle.mixedCellFonts))) {
        name = null;
    }
    return name;
}

int getFontStyle() {
    JCTypeConverter conv = new JCTypeConverter();
    String styleName = (String) styleChoser.getSelectedItem();
    int style = MIXED_STYLE;
    if (!styleName.equals(LocaleBundle.string(LocaleBundle.mixedCellFonts))) {
        style = conv.toEnum(styleName, "cellFontStyle", style_names, styles, Font.PLAIN);
    }
    return style;
}

int getFontSize() {
    JCTypeConverter conv = new JCTypeConverter();
    String sizeName = (String) sizeChoser.getSelectedItem();
    int size = MIXED_SIZE;
    if (!sizeName.equals(LocaleBundle.string(LocaleBundle.mixedCellFonts))) {
        size = 12;
        try {
            size = new Integer(sizeName).intValue();
        }
        catch (Exception e) {
        }
    }
    return size;
}

}

