/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: FormatTabs.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:21 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid.customizer;

import java.awt.*;

import javax.swing.*;
import javax.swing.event.*;

import com.klg.jclass.higrid.*;
import com.klg.jclass.higrid.beans.*;

/**
 * 	The UI portion of to display and modify <code>JCHiGridNode</code>.
 *	It observes any changes to the underlining <code>JCHiGridNode</code>.
 */
public class FormatTabs extends JTabbedPane implements ChangeListener,
    JCHiGridListener {

// Note: same order as pageNames
public static final int SECTION_PAGE = 0;
public static final int RECORD_PAGE = 1;
public static final int HEADER_PAGE = 2;
public static final int FOOTER_PAGE = 3;
public static final int BEFORE_DETAILS_PAGE = 4;
public static final int AFTER_DETAILS_PAGE = 5;
/** max page, i.e. number of pages. */
private static final int MAX_PAGE = AFTER_DETAILS_PAGE + 1;

static String pageNames[] = {
    LocaleBundle.sections,
    LocaleBundle.record,
    LocaleBundle.header,
    LocaleBundle.footer,
    LocaleBundle.before_details,
    LocaleBundle.after_details,
};

static int formatSections[] = {
    -1, // not applicable
    JCHiGridNode.SECTION_RECORD,
    JCHiGridNode.SECTION_HEADER,
    JCHiGridNode.SECTION_FOOTER,
    JCHiGridNode.SECTION_BEFORE_DETAILS,
    JCHiGridNode.SECTION_AFTER_DETAILS,
};

JCHiGridNode node;

SectionsPage sectionsPage = new SectionsPage();
AbstractSectionPage page = new AbstractSectionPage("", RECORD_PAGE);


private static final boolean TRACE = false;

public FormatTabs(GridPropertiesEditor editor) {
    super();
    init(editor);
}


public void init(GridPropertiesEditor editor) {
    // Create the tab manager
    addTab(LocaleBundle.string(pageNames[0]), sectionsPage);
    for (int i=1; i<MAX_PAGE; i++) {
        // create a dummy panel to contain
        // the actual page
        JPanel panel = new JPanel();
        panel.setLayout(new GridLayout(1, 1));
        addTab(LocaleBundle.string(pageNames[i]), panel);
    }

    setSelectedIndex(SECTION_PAGE);
    addChangeListener(this);
    sectionsPage.setEditor(editor);
    page.setEditor(editor);
    com.klg.jclass.datasource.customizer.CustomizerLauncher.setDefaultFont((Component)this, new Font("Dialog", Font.PLAIN, 9));
}

//FIX should return preferred size based on children!
public Dimension getPreferredSize() {
    return new Dimension(490,500);
}

/** Sets enabled of a page. */
public void setPageEnabled(int page, boolean v) {
    setEnabledAt(page, v);
}

/** Enable/Disable tabs depending on section visible flags.
 *	Also change the current tab corresponding to the selected checkbox.
 */
public void updateTabs() {
    setPageEnabled(HEADER_PAGE, node.isSectionShowing(JCHiGridNode.SECTION_HEADER));
    setPageEnabled(BEFORE_DETAILS_PAGE, node.isSectionShowing(JCHiGridNode.SECTION_BEFORE_DETAILS));
    setPageEnabled(RECORD_PAGE, node.isSectionShowing(JCHiGridNode.SECTION_RECORD));
    setPageEnabled(AFTER_DETAILS_PAGE, node.isSectionShowing(JCHiGridNode.SECTION_AFTER_DETAILS));
    setPageEnabled(FOOTER_PAGE, node.isSectionShowing(JCHiGridNode.SECTION_FOOTER));

    // Only change tab if current tab is disabled.
    if (!isEnabledAt(getSelectedIndex())) {
        setCurrentTab(SECTION_PAGE);
    }
}

public int getCurrentTab() {
    return getSelectedIndex();
}

public void setCurrentTab(int tab) {
    setSelectedIndex(tab);
}

public void setJCHiGridNode(JCHiGridNode node) {
    this.node = node;
    if (getCurrentTab() == SECTION_PAGE) {
        sectionsPage.init(node);
    }
    else {
        tabChanged(getCurrentTab());
    }
    node.removeJCHiGridListener(this);
    node.addJCHiGridListener(this);
    updateTabs();
}

/** When any property is changed, update tab. */
public void JCHiGridValueChanged(JCHiGridEvent e) {
    updateTabs();
}

public void stateChanged(ChangeEvent e) {
    tabChanged(getSelectedIndex());
}

private void tabChanged(int tab) {
    // There is only one page created. In BWT, a page is
    // shared among several tabs.  In Swing, when 
    // a section tab button is clicked, the page will be
    // added to the dummy panel.
    if (tab != SECTION_PAGE) {
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        ((JPanel)getSelectedComponent()).add(page);
        page.setSection(formatSections[tab]);
        page.init(node);
        setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }
}

}   // FormatTabs
