/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: GridPropertiesEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:21 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid.customizer;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.tree.*;

import com.klg.jclass.cell.JCCellInfo;
import com.klg.jclass.datasource.*;
import com.klg.jclass.datasource.customizer.*;
import com.klg.jclass.datasource.beans.*;
import com.klg.jclass.datasource.util.DataBinding;
import com.klg.jclass.higrid.*;
import com.klg.jclass.higrid.beans.*;
import com.klg.jclass.higrid.LocaleBundle;
import com.klg.jclass.util.*;
import com.klg.jclass.util.internal.JClassInfo;

/**
 * 	<p>Serializes tree data bean component into a file.  After this custom editor
 *	is closed, the name of a serialization file and an array of data source object
 *	names are passed to the setter method.
 *	Then, the setter will deserialize the tree data bean component.</p>
 *	<p>One only data source type is allowed for all the data sources
 *	in a hierarchy.  The type is determined by the root.</p>
 */
public class GridPropertiesEditor extends TreeEditor implements ItemListener,
    JCHiGridListener, JCDataListener {

JTextField dataBinding;
JButton btnDataBinding;
JPanel card;
ButtonGroup sourceGroup;

private static Thread showTipsThread;
private static boolean tipsShowed = false;
private boolean initializing = false;
private boolean externalDS;
/** meta data tree.  It is created after a bean component change. */
protected TreeModel tree;
private static int modelCounter = 0;
private static int tempFileCounter = 0;
private static final boolean TRACE = false;

private static final String NONE = LocaleBundle.string(LocaleBundle.NONE);

/** data source type of the root of a hierarchy.
 *	It is the data source for all data sources in the hierarchy.
 */
int rootDataSourceType = NodeProperties.JDBC;

public GridPropertiesEditor() {
    this(false);
}

public GridPropertiesEditor(boolean externalDS) {
    super(LocaleBundle.string(LocaleBundle.data_model), externalDS);
    this.externalDS = externalDS;
    com.klg.jclass.datasource.customizer.CustomizerLauncher.setDefaultFont((Component)this, new Font("Dialog", Font.PLAIN, 9));

    // Show tips if necessary.
    // Use a thread to wait for the editor to show up
    // before opening the dialog.
    // Otherwise, VCafe 2.5 will hang.
    String s = JClassInfo.getProperty("higrid",
                                      "higrid_bean_component.customizer.show_tips_at_start");
    if (JCTypeConverter.toBoolean(s, true) && !tipsShowed) {
        if (showTipsThread == null) {
            showTipsThread = new Thread(new ThreadTipsDialog());
            showTipsThread.start();
        }
    }
}

public Component getNorthComponent(boolean external_DS) {
    // Create internal panel
    JPanel inp = new JPanel();
    modelName = new JTextField("", 30);
    inp.setLayout(new FlowLayout(FlowLayout.LEFT));
    inp.add(new JLabel(com.klg.jclass.datasource.LocaleBundle.string(
                                                                    com.klg.jclass.datasource.LocaleBundle.model_name)));
    inp.add(modelName);

    // Create external panel
    JPanel exp = new JPanel();
    exp.setLayout(new FlowLayout(FlowLayout.LEFT));
    dataBinding = new JTextField(NONE, 30);
    dataBinding.setEnabled(false);
    btnDataBinding = new JButton("...");
    exp.add(new JLabel(com.klg.jclass.datasource.LocaleBundle.string(
                                                                    com.klg.jclass.datasource.LocaleBundle.model_name)));
    exp.add(dataBinding);
    exp.add(btnDataBinding);

    JPanel source_panel = new JPanel();
    String datasource = LocaleBundle.string(external_DS ? LocaleBundle.external : LocaleBundle.internal);
    source_panel.setBorder(new TitledBorder(datasource));
    source_panel.setLayout(new BorderLayout());
    source_panel.add(external_DS ? exp : inp, BorderLayout.CENTER);
    setSourceEnabled(false);
    btnDataBinding.addActionListener(new ActionListener() {
                                         public void actionPerformed(ActionEvent e) {
                                             showDataBindingEditor();
                                         }
                                     });
    return source_panel;
}

public JTabbedPane createTabs() {
    return new CustomizeTabs(this);
}


/** Sets the bean to be customized.  It is used if the property editor
 *	is used as a page in a customizer.
 *	@param bean	bean to be customized.
 */
public void setBean(Object bean) {
    super.setBean(bean);
    if (bean instanceof JCHiGrid) {
        setValue(((JCHiGrid)bean).getGridProperties());
    }
}

/** Gets the external datasource indication */
protected boolean isExternalDS() {
    return externalDS;
}

/** Gets a description for a node in the tree.  It is used
 *	when a tree is shown in the outliner.
 */
/*
public String getNodeDescription(DataSourceTreeNode node) {
return ((JCHiGridNode)node.getUserObject()).getNodeProperties().getDescription();
}
*/
/** Sets a description for a node in the tree.
 */
/*
public void setNodeDescription(DataSourceTreeNode node, String description) {
((JCHiGridNode)node.getUserObject()).getNodeProperties().setDescription(description);
((JCHiGridNode)node.getUserObject()).getNodeProperties().fireJCDataEvent(new JCDataEvent((Object)((JCHiGridNode)node.getUserObject()).getNodeProperties()));
}
*/

/** Creates node data to be added to the node of the tree.
 *	@param parent parent in which new node is created.
 *	@param current current node to be created.
 */
public void addNodeData(DataSourceTreeNode parent, DataSourceTreeNode current, String description) {
    GridProperties comp = (GridProperties) target;
    // Warn user if resource name is not entered.
    if (!comp.isResourceNameSpecified()) {
        ErrorDialog.showError(this,
                              com.klg.jclass.datasource.LocaleBundle.string(
                                                                           com.klg.jclass.datasource.LocaleBundle.enter_resource_name_and_save));
    }

    JCHiGridNode ogbn = null;
//    if (current.getUserObject().equals(current)) {
    if (current.getUserObject() == null) {
        // New node.  Add data bean component as data.
        JCHiGridNode new_node = createJCHiGridNode(comp);
        new_node.getNodeProperties().setDescription(description);
        current.setUserObject(new_node);
    }
    ogbn = (JCHiGridNode) current.getUserObject();

    if (parent != null) {
        ogbn.getNodeProperties().setDataSourceType(rootDataSourceType);
        ogbn.getNodeProperties().setParentNodeProperties(
                                                        ((JCHiGridNode)parent.getUserObject()).getNodeProperties());
        ogbn.getNodeProperties().setUseParentConnection(true);
        parent.add(current);
    }
    else {
        // Can't use parent connection for root.
        ogbn.getNodeProperties().setUseParentConnection(false);
        comp.setRoot(current);
    }

    // switch to data model tab.
    tabManager.setSelectedIndex(CustomizeTabs.DATA_PAGE);
}

/** Removes data from the component.
 *	@param node node in which data to be removed.
 */
public void removeNodeData(DataSourceTreeNode node) {
    GridProperties comp = (GridProperties) target;
    comp.remove(node);
}

public void clearData() {
    GridProperties comp = (GridProperties) target;
    comp.removeAll();
}

void setSourceEnabled(boolean v) {
    modelName.setEnabled(v && !externalDS);
    btnDataBinding.setEnabled(v && externalDS);
}

/** Helper method to enable/disable controls for start of editing.
 */
public void checkStartEditing(JCHiGridNode node) {
/*
if (node == null) {
    // ensure no actual node is selected
    EditorTreeNode folder = null;
    if (outliner.getModel() != null) {
        folder = (EditorTreeNode)((outliner.getLastSelectedPathComponent() != null) ?
        outliner.getLastSelectedPathComponent() : outliner.getModel().getRoot());
    }
//    	EditorTreeNode folder = (EditorTreeNode)
//    		((outliner.getSelectedNode() != null) ?
//    		outliner.getSelectedNode() : outliner.getRootNode());
    if (folder != null) {
        node = (JCHiGridNode) (folder.getUserObject() == null ? null :
            ((DataSourceTreeNode) folder.getUserObject()).getUserObject());
    }
}
*/
    GridProperties comp = (GridProperties) target;
	boolean hasResourceName = comp.isResourceNameSpecified();
	if (!hasResourceName) {
		SerializedProperties serializedProps = ((CustomizeTabs)tabManager).serializationPage.getSerializedProperties();
    	String resourceName = (serializedProps == null) ? null : serializedProps.getResourceName();
    	hasResourceName = (resourceName != null);
    	if (resourceName != null) {
    		comp.setResourceName(resourceName);
    		comp.setSerializationFile(serializedProps.getSerializationFile());
    		// Sets the full path to the serialization file.  Need to do this to
    		// ensure that the serialization file gets updated properly when the
    		// user exits the editor.
		}
	}

    setSourceEnabled(hasResourceName);
    btnAdd.setEnabled(hasResourceName && !externalDS);
    btnDelete.setEnabled(node != null && hasResourceName && !externalDS);
    btnDescription.setEnabled(node != null && hasResourceName && !externalDS);
    btnClear.setEnabled(hasResourceName && !externalDS);
    tabManager.setEnabledAt(CustomizeTabs.DATA_PAGE, hasResourceName && !externalDS);

    boolean valid_meta_data = false;
    TreeModel tree = getMetaDataTree();
    if (tree != null && node != null) {
        MetaDataModel mdata = getMetaData(node,
                                          (DataSourceTreeNode) comp.getRoot(),
                                          (DataSourceTreeNode) tree.getRoot());
        if (mdata != null) {
            // get columns from meta data.
            int num_of_cols = 0;
            try {
                num_of_cols = mdata.getColumnCount();
            }
            catch (Exception ex) {
                ex.printStackTrace();
            }
            if (num_of_cols > 0) {
                valid_meta_data = true;
            }
        }
    }

    // format customization is only available if meta data tree
    // can be created.
    if (((CustomizeTabs)tabManager).formatTabs != null) {
        ((CustomizeTabs)tabManager).formatTabs.setEnabled(valid_meta_data);
    }

    tabManager.setEnabledAt(CustomizeTabs.FORMAT_PAGE, valid_meta_data);

    int select = CustomizeTabs.SERIALIZATION_PAGE;
    if (tabManager.isEnabledAt(tabManager.getSelectedIndex())) {
        select = tabManager.getSelectedIndex();
    }
    if (valid_meta_data && externalDS) {
        select = CustomizeTabs.FORMAT_PAGE;
    }

    tabManager.setSelectedIndex(select);

}

/** Helper method to change the editor content when a node is selected. */
public void nodeSelected(EditorTreeNode node) {
    boolean isEmpty = (node == null) && !externalDS;
//	boolean is_root_folder = outliner.getModel().getRoot().equals(node);

    CustomizeTabs tabs = (CustomizeTabs) tabManager;

    // Selected root.  Disable delete, rename and editor tabs
    if (tabs.formatTabs != null) {
        tabs.setEnabledAt(CustomizeTabs.SERIALIZATION_PAGE, isEmpty);
    }

    btnClear.setEnabled(!isEmpty);
    btnDelete.setEnabled(!isEmpty);
    btnDescription.setEnabled(!isEmpty);

    // Can only add to root of tree in GridProperties if root is not set.
    GridProperties comp = (GridProperties) target;
    tabs.serializationPage.setSerializedProperties(comp);
    if (node != null) {
        JCHiGridNode hgbn = (JCHiGridNode) (node.getUserObject() == null ? null :
                                            ((DataSourceTreeNode) node.getUserObject()).getUserObject());
        if (hgbn != null) {
            tabs.setJCHiGridNode(hgbn);
        }
/*
    if (is_root_folder) {
        // switch to data model tab.
        tabs.setSelectedIndex(CustomizeTabs.SERIALIZATION_PAGE);
        // switch to DataSourceType page.
        tabs.dataTabs.setSelectedIndex(DataModelTabs.DATA_SOURCE_PAGE);
    } else {
        if (hgbn != null) {
            tabs.setJCHiGridNode(hgbn);
        }
    }
*/
        checkStartEditing(hgbn);
    }

}

protected DataModelTabs getDataModelTabs() {
    return new DataModelTabs(true, this);
}

/** Helper method to change all data source types to the root's.
 */
void setTreeDataSourceType(DataSourceTreeNode node) {
    if (node == null) return;

    NodeProperties data = ((JCHiGridNode)node.getUserObject()).getNodeProperties();
    data.setDataSourceType(rootDataSourceType);
    TreeIterator ti = node.getIterator();
    while (ti.hasMoreElements()) {
        DataSourceTreeNode child = (DataSourceTreeNode) ti.nextElement();
        setTreeDataSourceType(child);
    }
}

/** Value(s) of data bean component is changed.
 */
public void JCDataValueChanged(JCDataEvent e) {
    if (TRACE) {
        System.out.println("update:: NodeProperties is updated.");
    }
    GridProperties comp = (GridProperties) e.getSource();
    // pass it on as a JCHiGridEvent
    comp.fireJCHiGridEvent(new JCHiGridEvent((Object)comp,
                                             JCHiGridEvent.DATA_BEAN_COMPONENT_VALUE, null, null));
}

/** When value(s) of higrid bean component is changed.
 *  Also all data bean events are passed on to here.
 */
public void JCHiGridValueChanged(JCHiGridEvent e) {
    if (TRACE) {
        System.out.println("update:: GridProperties is updated.");
    }
    GridProperties comp = (GridProperties) e.getSource();

    try {
        // Fire property change directly from here but not calling
        // setValue(comp.getSerializationFile()).
        // The reason is that after calling setValue(), data bean
        // components of the target have different
        // memory location due to serialization.  However, data
        // bean components in JCOutliner are still the
        // before-serialization ones.  Calling setValue() makes outliner
        // and actual data bean components of the target out of synch.
        support.firePropertyChange(comp.isExternalDS() ? "ExternalGridProperties" :
                                   "GridProperties", null, comp);
    }
    catch (Exception ex) {
        if (TRACE) {
            ex.printStackTrace();
        }
        if (ex instanceof IOException) {
            ErrorDialog.showError(this,
                                  ex + System.getProperty("line.separator") +
                                  LocaleBundle.string(LocaleBundle.failed_to_serialize));
        }
    }
    int dstype = (comp.getRoot() == null ||
                  comp.getRoot().getUserObject() == null) ?
                 NodeProperties.JDBC :
                 ((JCHiGridNode)comp.getRoot().getUserObject()).
                 getNodeProperties().getDataSourceType();
    if (dstype != rootDataSourceType) {
        // root data source type is changed, propagate to the entire tree.
        rootDataSourceType = dstype;
        setTreeDataSourceType(comp.getRoot());

        // reload the bean component.
        showNewBeanComponent();
        return;
    }

    // Assign and reflect JCHiGridNode in the tab manager.
    if (e.getJCHiGridNode() != null) {
        JCHiGridNode node = e.getJCHiGridNode();
        if (TRACE) {
            System.out.println("update:: GridProperties is updated by " +
                               "a JCHiGridNode: " + node);
        }
        // Create a meta data tree only when data model is changed.
        if (e.getNodeProperties() != null) {
            createMetaDataTree();
        }
        checkStartEditing(node);
        ((CustomizeTabs)tabManager).setJCHiGridNode(node, (e.getNodeProperties() != null));
    }
    else if (outliner.getModel() == null) {
        checkStartEditing(null);
    }
    ((CustomizeTabs)tabManager).serializationPage.setSerializedProperties(comp);
}

/** Returns a bean component by deserialized the serialization file
 *	if the editor is closed.  Otherwise, the same instance of bean
 *	component is returned.
 *	In this way, a new instance of bean component is created and thus
 *	guarantee code-generation.  JBuilder 2.0 specifically compares the
 *	instances of new and old values.  Even if their contained properties
 *	are changed, no code is generated when the new and old bean component
 *	are the same instance.
 */
public Object getValue() {
    GridProperties new_comp = (GridProperties) super.getValue();
//FIXOBSOLETE
/*
if (JCEnvironment.isVisualCafe()) {
    // Because GridProperties becomes non-serializable
    // after data model of HiGrid is set, VCafe 2.5
    // has problem handling it.  Thus,
    // a "clean" and simple GridProperties is returned
    // to VCafe 2.5.
    removeHiGridReferences(new_comp.getRoot());
}
*/
    return new_comp;
}


public void setValue(Object value) {
    if (TRACE) {
        System.out.println("setValue::value: " + value);
        Thread.currentThread().dumpStack();
    }
    if (value instanceof String) {
        // data model is already serialized from data on this editor.
        try {
            GridProperties new_model = (GridProperties)
                                       GridProperties.fromFile((String) value);
            if (this.externalDS != new_model.isExternalDS()) {
                ErrorDialog.showError(this,
                                      LocaleBundle.string(LocaleBundle.wrongDataSourceType));
                return;
            }

            // target MUST be set before firePropertyChange()
            // so that the new bean data model
            // is used to set the bean.
            Object old_target = target;
            target = new_model;
            if (TRACE) {
                System.out.println("Change model from " +
                                   System.getProperty("line.separator") + old_target +
                                   System.getProperty("line.separator") + "to" +
                                   System.getProperty("line.separator") + new_model);
            }

            // add event listeners to this new GridProperties
            addBeanEventListeners(new_model);

            // In case the file is not the same as the serialized file name for
            // the serialization file property.  Set the serialization file to
            // the file name.
            new_model.setSerializationFile((String) value);
//FIX not sure if we need the next line; it seems to duplicate the following line's effect
//			new_model.fireJCHiGridEvent(new JCHiGridEvent((Object)new_model));

            support.firePropertyChange("GridProperties", old_target, target);
        }
        catch (Exception ex) {
            if (TRACE) {
                ex.printStackTrace();
            }
            if (ex instanceof FileNotFoundException) {
                ErrorDialog.showError(this,
                                      ex + System.getProperty("line.separator") +
                                      LocaleBundle.string(LocaleBundle.failed_to_load_file_not_found));
            }
            else {
                ErrorDialog.showError(this,
                                      ex + System.getProperty("line.separator") +
                                      LocaleBundle.string(LocaleBundle.failed_to_deserialize));
            }

            // throw an exception so loadFromFile() can catch it
            // and indicates loading is unsuccessful.
            throw new IllegalArgumentException(ex.toString());
        }
    }
    else if (value instanceof GridProperties) {
        // Reflects the component on the editor.
        // firePropertyChange() is not called here because there is no change.
        GridProperties comp = null;
        GridProperties hvalue = (GridProperties) value;
//FIXOBSOLETE
/*
    if (JCEnvironment.isVisualCafe()) {
        try {
            comp = (GridProperties)
                    GridProperties.fromFile(hvalue.getSerializationFile());
        } catch (Exception ex) {
            if (TRACE) {
                ex.printStackTrace();
            }
            if (hvalue != null) {
                comp = createGridProperties(hvalue.getDataSourceType(), hvalue.isExternalDS());
                comp.setSerializationFile(hvalue.getSerializationFile());
                comp.setResourceName(hvalue.getResourceName());
                comp.setModelName(hvalue.getModelName());
                comp.setRoot(hvalue.getRoot());
            } else {
                comp = createGridProperties(externalDS);
            }
        }
    } else if (JCEnvironment.isVisualAge()) {
        // In VisualAge 2.0, setValue() is called after every change.
        // This behaviour screws up the current node selection.
        // This spurious call is ignored.
        if (target != null) {
            return;
        } else {
            comp = hvalue;
        }
    } else {
        comp = hvalue;
    }
*/
        comp = hvalue;

        if (comp.getRoot() != null && comp.getRoot().getUserObject() != null) {
            rootDataSourceType = ((JCHiGridNode)comp.getRoot().getUserObject()).
                                 getNodeProperties().getDataSourceType();
        }
        addBeanEventListeners(comp);

        try {
            target = comp;
            if (TRACE) {
                System.out.println("Show tree component " + comp);
            }
            showNewBeanComponent();
        }
        catch (Exception ex) {
            if (TRACE) {
                ex.printStackTrace();
            }
            ErrorDialog.showError(this,
                                  ex + System.getProperty("line.separator") +
                                  LocaleBundle.string(LocaleBundle.failed_to_serialize));
        }
    }
}

/** Creates a GridProperties or one of its superclasses. */
protected GridProperties createGridProperties(int type, boolean externalDS) {
    GridProperties comp = null;
    if (externalDS) {
        comp = (GridProperties) new ExternalGridProperties(type, externalDS);
    }
    else {
        comp = new GridProperties(type, externalDS);
    }
    return comp;
}

protected GridProperties createGridProperties(boolean externalDS) {
    GridProperties comp = null;
    if (externalDS) {
        comp = (GridProperties) new ExternalGridProperties(externalDS);
    }
    else {
        comp = new GridProperties(externalDS);
    }
    return comp;
}

/** Returns the name of the serialization file. */
public String getJavaInitializationString() {
    GridProperties comp = (GridProperties) target;

    // Escape all path separators in the filename.
    String s = new String("\"" + super.getJavaInitializationString() +
                          "\", getClass()");


    // At design-time, JBuilder cannot load the serialization file
    // based on resource name.  That's why the serialization
    // file name is required.  (PR6044)
//FIXOBSOLETE
/*
if (JCEnvironment.isJBuilder()) {
    s += ", \"" + comp.getSerializationFile().replace('\\', '/') + "\"";
}
*/

    // If there are data sources, append data source names
    Vector v = new Vector();
    getDataSources(v, comp.getRoot());
    if (v.size() > 0) {
        StringBuffer names = new StringBuffer("new String[] {");
        StringBuffer objects = new StringBuffer("new Object[] {");
        for (int i=0; i<v.size(); i++) {
            if (i>0) {
                names.append(", ");
                objects.append(", ");
            }
            names.append("\"" + v.elementAt(i) + "\"");
            objects.append(v.elementAt(i));
        }
        names.append("}");
        objects.append("}");
        s += "," + new String(names) + "," + new String(objects);
    }
    if (externalDS) {
        return new String("new com.klg.jclass.higrid.beans.ExternalGridProperties(" + s + ")");
    }
    return new String("new com.klg.jclass.higrid.beans.GridProperties(" + s + ")");
}

/** Helper method to get a initialization string to pass back to IDE
 *	for an array of data source objects.
 */
void getDataSources(Vector s, DataSourceTreeNode node) {
    if (node == null || node.getUserObject() == null) return;

    NodeProperties data = (NodeProperties)
                          ((JCHiGridNode) node.getUserObject()).getNodeProperties();
    if (NodeProperties.isIDEDataSourceType(data.getDataSourceType())) {
        s.addElement(data.getDataSourceName());
    }

    TreeIterator ti = node.getIterator();
    while (ti.hasMoreElements()) {
        DataSourceTreeNode child = (DataSourceTreeNode) ti.nextElement();
        getDataSources(s, child);
    }
}
//FIXOBSOLETE
void removeHiGridReferences(DataSourceTreeNode node) {
/*
if (node == null) return;

JCHiGridNode hn = (JCHiGridNode) node.getUserObject();
if (hn != null) {
    for (int i=0; i<JCHiGridNode.SECTION_MAX; i++) {
        Vector v = hn.getRowFormat(i).getIndicatorFormats();
        Vector vdata = hn.getRowFormat(i).getDataFormats();
        if (v != null) {
            for (Enumeration e = v.elements(); e.hasMoreElements(); ) {
                CellFormat cell = (CellFormat) e.nextElement();
                cell.setGrid(null);
            }
        }
        if (vdata != null) {
            for (Enumeration e = vdata.elements(); e.hasMoreElements(); ) {
                CellFormat cell = (CellFormat) e.nextElement();
                cell.setGrid(null);
            }
        }
    }
}
TreeIterator ti = node.getIterator();
while (ti.hasMoreElements()) {
    DataSourceTreeNode child = (DataSourceTreeNode) ti.nextElement();
    removeHiGridReferences(child);
}
*/
}

public void showNewBeanComponent() {
    initializing = true;

    // Create a meta data tree.
    createMetaDataTree();
    GridProperties comp = (GridProperties) target;
    if (externalDS) {
        synchronize();
        if (comp.getDataBinding() == null || comp.getDataBinding().length() == 0) {
            dataBinding.setText(NONE);
        }
        else {
            dataBinding.setText(comp.getDataBinding());
        }
    }
    else {
        modelName.setText(comp.getModelName());
    }

    createTree();

    JCHiGridNode node = null;
    EditorTreeNode editorNode = (EditorTreeNode)outliner.getLastSelectedPathComponent();
    if (editorNode != null) {
        DataSourceTreeNode treeNode = (DataSourceTreeNode)editorNode.getUserObject();
        node = (JCHiGridNode)treeNode.getUserObject();
    }
    comp.fireJCHiGridEvent(new JCHiGridEvent((Object)comp, node));
}

void createTree() {
/*
// Delete children of root to clear the outliner first.  Then
// build the outliner from tree.
// need to do proper re-creation of tree as before
EditTreeNode root = (EditorTreeNode)outliner.getModel().getRoot();
((EditorTreeNode)outliner.getModel().getRoot()).setChildren(null);

showTree((EditorTreeNode)outliner.getModel().getRoot(), comp.getRoot());

// Refresh outliner to show the new tree model.
outliner.folderChanged((EditorTreeNode)outliner.getModel().getRoot());

// Clear up reference to HiGrid in NodeCellFormat.  Reference to
// HiGrid makes bean component not serializable! (need to find out why)
removeHiGridReferences(comp.getRoot());
JCOutlinerNode select = (outliner.getRootNode().getChildren() != null) ?
                (JCOutlinerNode) outliner.getRootNode().getChildren().elementAt(0) :
                outliner.getRootNode();
// clear selection first to force re-selecting possibly the same node.
outliner.selectNode(null, null);
outliner.selectNode(select, null);
*/
    GridProperties comp = (GridProperties) target;
    outliner.setModel(null);
    buildTree(null, comp.getRoot());
    if (outliner.getModel() != null) {
        ((DefaultTreeModel)outliner.getModel()).reload();
        expandTree();
    }
    modelName.setText(comp.getModelName());

    initializing = false;
}

public String getResourceName() {
    return((CustomizeTabs)tabManager).serializationPage.getResourceName();
}


/** Gets a meta data tree.
 */
public TreeModel getMetaDataTree() {
    return tree;
}

/** Creates a meta data tree.
 */
void createMetaDataTree() {
    if (externalDS) {
        createExternalMetaDataTree();
        synchronize();
    }
    else {
        createInternalMetaDataTree();
    }
    if (TRACE) {
        System.out.println("created meta data tree: " + tree);
    }
}

void createExternalMetaDataTree() {
    String db = ((GridProperties)target).getDataBinding();

    DataModel dm = null;
    int index = -1;
    if (db != null) {
        index = db.indexOf(DataBinding.SOURCE_DELIMITER);
    }
    if (index > -1) {
        dm = (DataModel) DataSourceBeanInstance.getInstance(db.substring(0, index));
    }
    if (dm == null) {
        tree = null;
    }
    else {
        tree = dm.getMetaDataTree();
    }
}

void createInternalMetaDataTree() {
    HiGridData hdata = new HiGridData(new TreeData());
    try {
        GridProperties comp = (GridProperties) target;
        loadModelData(hdata, comp.getRoot(), null);
    }
    catch (DataModelException ex) {
        if (TRACE) {
            ex.printStackTrace();
        }
        tree = null;
    }
    tree = hdata.getMetaDataTree();
}

/** Helper method called recursively to create meta data.
 *	@exception DataModelException If fails to create a meta data.
 */
void loadModelData(HiGridData hdata, DataSourceTreeNode node, MetaDataModel meta_parent) throws DataModelException {
    if (node == null || node.getUserObject() == null) return;

    NodeProperties data = ((JCHiGridNode) node.getUserObject()).getNodeProperties();

    MetaDataModel meta = null;
    meta = getCreator().createMetaData(hdata, meta_parent, data);

    TreeIterator ti = node.getIterator();
    while (ti.hasMoreElements()) {
        DataSourceTreeNode child = (DataSourceTreeNode) ti.nextElement();
        loadModelData(hdata, child, meta);
    }
}

protected com.klg.jclass.datasource.beans.MetaDataCreator getCreator() {
    return com.klg.jclass.datasource.beans.MetaDataCreator.getCreator(rootDataSourceType);
}

/** Reset header and record columns at all levels.
 */
public void retrieveAllFields() {
    // Create a higrid bean on the fly to get a meta data tree.
    TreeModel tree = getMetaDataTree();
    if (tree == null) return;

    GridProperties comp = (GridProperties) target;
    resetRecordAndHeader((DataSourceTreeNode) comp.getRoot(), (DataSourceTreeNode) tree.getRoot(), false, false);
}

/** Clear format at all levels and reset header and record columns at all levels.
 */
public void clearAllFormat() {
    // Create a higrid bean on the fly to get a meta data tree.
    TreeModel tree = getMetaDataTree();
    if (tree == null) return;

    GridProperties comp = (GridProperties) target;
    resetRecordAndHeader((DataSourceTreeNode) comp.getRoot(), (DataSourceTreeNode) tree.getRoot(), true, true);
}

/** Reset header and record columns at a level identified by the JCHiGridNode.
 */
public void resetRecordAndHeader(JCHiGridNode hdata, boolean clear_other_formats, boolean reset) {
    GridProperties comp = (GridProperties) target;
    // Create a higrid bean on the fly to get a meta data tree.
    TreeModel tree = getMetaDataTree();
    if (tree == null) return;

    // Find meta data at the same level as the higrid bean node.
    MetaDataModel mdata = getMetaData(hdata, (DataSourceTreeNode) comp.getRoot(), (DataSourceTreeNode) tree.getRoot());
    if (mdata != null) {
        if (reset) {
            resetRecordAndHeader(hdata, mdata, clear_other_formats);
        }
        else {
            updateRecordAndHeader(hdata, mdata);
        }
    }
}

/** Reset header and record columns at a level identified by the JCHiGridNode.
 */
public void resetRecordAndHeader(JCHiGridNode hdata, MetaDataModel mdata, boolean clear_other_formats) {
    hdata.setRowFormat(JCHiGridNode.SECTION_RECORD, new RecordFormat());
    hdata.setRowFormat(JCHiGridNode.SECTION_HEADER, new HeaderFormat());
    if (clear_other_formats) {
        hdata.setRowFormat(JCHiGridNode.SECTION_BEFORE_DETAILS, new BeforeDetailsFormat());
        hdata.setRowFormat(JCHiGridNode.SECTION_AFTER_DETAILS, new AfterDetailsFormat());
        hdata.setRowFormat(JCHiGridNode.SECTION_FOOTER, new FooterFormat());
        createIndicatorFormats(hdata.getRowFormat(JCHiGridNode.SECTION_BEFORE_DETAILS), (JCHiGrid)bean);
        createIndicatorFormats(hdata.getRowFormat(JCHiGridNode.SECTION_AFTER_DETAILS), (JCHiGrid)bean);
        createIndicatorFormats(hdata.getRowFormat(JCHiGridNode.SECTION_FOOTER), (JCHiGrid)bean);
    }

    // set columns from meta data.
    int num_of_cols = 0;
    try {
        num_of_cols = mdata.getColumnCount();
    }
    catch (Exception ex) {
        ex.printStackTrace();
    }
    Vector record_cells = new Vector(num_of_cols);
    Vector header_cells = new Vector(num_of_cols);
    for (int i=0; i<num_of_cols; i++) {
        addRecordAndHeader(record_cells, header_cells, hdata, mdata, i);
    }
    hdata.getRowFormat(JCHiGridNode.SECTION_RECORD).setDataFormats(record_cells);
    hdata.getRowFormat(JCHiGridNode.SECTION_HEADER).setDataFormats(header_cells);
    createIndicatorFormats(hdata.getRowFormat(JCHiGridNode.SECTION_HEADER), (JCHiGrid)bean);
    createIndicatorFormats(hdata.getRowFormat(JCHiGridNode.SECTION_RECORD), (JCHiGrid)bean);
}

// This code is implicitly tied to EditStatusCellFormat and NodeCellFormat constructors.
// Unfortunately, when this object is invoked as a property editor
// and not a customizer, then the bean is null and the above CellFormats are not
// initialized properly.  Thus, we must duplicate code from the constructors here!
void createIndicatorFormats(RowFormat rowFormat, JCHiGrid bean) {
    rowFormat.createIndicatorFormats(bean);
    if (bean == null) {
        for (Enumeration e = rowFormat.getIndicatorFormats().elements(); e.hasMoreElements(); ) {
            CellFormat cellFormat = (CellFormat) e.nextElement();
            cellFormat.copyFromCellStyle(getDefaultHeaderCellStyle());
            cellFormat.setSelectedBackground(cellFormat.getBackground());
            cellFormat.setSelectedForeground(cellFormat.getForeground());
            cellFormat.setHorizontalAlignment(JCCellInfo.CENTER);
            cellFormat.setVerticalAlignment(JCCellInfo.CENTER);
        }
    }
}

/** Helper method called recursively to reset header and record columns.
 */
void resetRecordAndHeader(DataSourceTreeNode higrid_node, DataSourceTreeNode meta_node, boolean clear_other_formats, boolean reset) {
    if (higrid_node == null || meta_node == null) return;

    // Clear and reset record and header format.
    if (reset) {
        resetRecordAndHeader((JCHiGridNode) higrid_node.getUserObject(),
                             (MetaDataModel) meta_node, clear_other_formats);
    }
    else {
        updateRecordAndHeader((JCHiGridNode) higrid_node.getUserObject(),
                              (MetaDataModel) meta_node);
    }

    TreeIterator hiti = higrid_node.getIterator();
    TreeIterator meti = meta_node.getIterator();
    while (hiti.hasMoreElements() && meti.hasMoreElements()) {
        DataSourceTreeNode hichild = (DataSourceTreeNode) hiti.nextElement();
        DataSourceTreeNode mechild = (DataSourceTreeNode) meti.nextElement();
        resetRecordAndHeader(hichild, mechild, clear_other_formats, reset);
    }
}

/** Update header and record columns at a level identified by the JCHiGridNode.
 */
public void updateRecordAndHeader(JCHiGridNode hdata, MetaDataModel mdata) {
    Vector record_cells = hdata.getRowFormat(JCHiGridNode.SECTION_RECORD).getDataFormats();
    // may have to do a reset instead
    if (record_cells.size() == 0) {
        resetRecordAndHeader(hdata, mdata, false);
        return;
    }

    // get columns from meta data.
    int num_of_cols = 0;
    try {
        num_of_cols = mdata.getColumnCount();
    }
    catch (Exception ex) {
        ex.printStackTrace();
    }
    Vector header_cells = hdata.getRowFormat(JCHiGridNode.SECTION_HEADER).getDataFormats();
    Vector new_record_cells = new Vector(num_of_cols);
    Vector new_header_cells = new Vector(num_of_cols);
    String[] meta_column_names = new String[num_of_cols];
    try {
        for (int i=0; i<num_of_cols; i++) {
            meta_column_names[i] = new String(mdata.getColumnIdentifier(i));
        }
    }
    catch (Exception ex) {
        ex.printStackTrace();
    }

    // keep the cells matched by name in the same order as the cellFormat
    Enumeration enumHeader = header_cells.elements();
    for (Enumeration enumRecord = record_cells.elements(); enumRecord.hasMoreElements();) {
        CellFormat record_cell = (CellFormat) enumRecord.nextElement();
        CellFormat header_cell = (CellFormat) enumHeader.nextElement();
        String name = record_cell.getName();
        for (int i=0; i<num_of_cols; i++) {
            if (name.equals(meta_column_names[i])) {
                meta_column_names[i] = null;
                new_record_cells.addElement(record_cell);
                new_header_cells.addElement(header_cell);
                break;
            }
        }
    }

    // add the rest of the meta data columns
    for (int i=0; i<num_of_cols; i++) {
        if (meta_column_names[i] != null) {
            addRecordAndHeader(new_record_cells, new_header_cells, hdata, mdata, i);
        }
    }

    hdata.getRowFormat(JCHiGridNode.SECTION_RECORD).setDataFormats(new_record_cells);
    hdata.getRowFormat(JCHiGridNode.SECTION_HEADER).setDataFormats(new_header_cells);
}

/** Helper method to add new record and header cells.
 */
void addRecordAndHeader(Vector record_cells, Vector header_cells, JCHiGridNode hdata, MetaDataModel mdata, int index) {
    CellFormat header_cell = new CellFormat();
    header_cell.copyFromCellStyle(getDefaultHeaderCellStyle());
    CellFormat record_cell = new CellFormat();
    record_cell.copyFromCellStyle(getDefaultRecordCellStyle());
    header_cell.setParent(hdata.getRowFormat(JCHiGridNode.SECTION_HEADER));
    record_cell.setParent(hdata.getRowFormat(JCHiGridNode.SECTION_RECORD));
    try {
        header_cell.setName(mdata.getColumnIdentifier(index));
        header_cell.setText(header_cell.getName());
        header_cell.setCellRenderer(LookupTable.getDefaultHeaderRenderer());
        record_cell.setName(header_cell.getName());
        record_cell.setType(mdata.getJavaColumnType(record_cell.getName()));
        record_cell.setCellRenderer(Class.forName(LookupTable.getCellRendererName(record_cell.getType())));
        record_cell.setCellEditor(Class.forName(LookupTable.getCellEditorName(record_cell.getType())));
        ColumnModel cm = mdata.getColumnObject(record_cell.getName());
        record_cell.setEditable(!((cm != null) && (cm.isReadOnly())));
    }
    catch (Exception ex) {
        ex.printStackTrace();
    }
    header_cell.setDefaultSize(this, true);
    record_cell.setDefaultSize(this, true);
    record_cells.addElement(record_cell);
    header_cells.addElement(header_cell);
}

CellStyleModel defaultHeaderCellStyle = null;
CellStyleModel getDefaultHeaderCellStyle() {
    if (bean != null) {
        defaultHeaderCellStyle = ((JCHiGrid)bean).getHeaderCellStyle();
    }
    else if (defaultHeaderCellStyle == null) {
        defaultHeaderCellStyle = new DefaultHeaderCellStyle();
    }
    return defaultHeaderCellStyle;
}

CellStyleModel defaultRecordCellStyle = null;
CellStyleModel getDefaultRecordCellStyle() {
    if (bean != null) {
        defaultRecordCellStyle = ((JCHiGrid)bean).getRecordCellStyle();
    }
    else if (defaultRecordCellStyle == null) {
        defaultRecordCellStyle = new DefaultRecordCellStyle();
    }
    return defaultRecordCellStyle;
}

/** Get a meta data at the same level as the JCHiGridNode in the GridProperties.
 */
public MetaDataModel getMetaData(Object find_data,
                                 DataSourceTreeNode start_node, DataSourceTreeNode meta_start) {
    if (TRACE) {
        System.out.println("start_node: " + start_node);
        System.out.println("meta_start: " + meta_start);
    }

    if (start_node == null || meta_start == null) return null;

    if (foundData(find_data, start_node)) {
        return(MetaDataModel) meta_start;
    }

    TreeIterator hiti = start_node.getIterator();
    TreeIterator meti = meta_start.getIterator();
    while (hiti.hasMoreElements() && meti.hasMoreElements()) {
        DataSourceTreeNode hichild = (DataSourceTreeNode) hiti.nextElement();
        DataSourceTreeNode mechild = (DataSourceTreeNode) meti.nextElement();
        MetaDataModel meta = getMetaData(find_data, hichild, mechild);
        if (meta != null) {
            return meta;
        }
    }
    return null;
}

public boolean foundData(Object find_data, DataSourceTreeNode node) {
    if (find_data instanceof NodeProperties) {
        NodeProperties data_comp = ((JCHiGridNode)
                                    node.getUserObject()).getNodeProperties();
        if (TRACE) {
            System.out.println("data_comp: " + data_comp);
        }
        return find_data.equals(data_comp);
    }
    else if (find_data instanceof JCHiGridNode) {
        return find_data.equals(node.getUserObject());
    }
    return false;
}

public DataSourceTreeNode getModelTreeRoot() {
    return((GridProperties)target).getRoot();
}

public void setModelName() {
    GridProperties comp = (GridProperties) target;
    if (modelName.getText() == null || modelName.getText().trim().length() == 0) {
        // default model name
        modelName.setText("HiGridData" + modelCounter++);
    }
    comp.setModelName(modelName.getText());
}

public void itemStateChanged(ItemEvent e) {
    if (initializing || e.getStateChange() == ItemEvent.DESELECTED) return;

    showNewBeanComponent();
}

private void showDataBindingEditor() {
    DataBindingDialog d = new DataBindingDialog(null, true, false);
    d.getEditor().setValue(dataBinding.getText());
    d.pack();
    d.setVisible(true);

    if (!d.isCancelled()) {
        // set data binding
        String db = (String) d.getEditor().getValue();
        if (db == null || db.length() == 0) {
            db = NONE;
        }
        dataBinding.setText(db);
        GridProperties comp = (GridProperties) target;
        comp.setDataBinding(db);
        showNewBeanComponent();
    }
}

private void synchronize() {
    // copy top level properties to new GridProperties
    GridProperties comp = (GridProperties) target;
    GridProperties new_comp = createGridProperties(externalDS);
    new_comp.setSerializationFile(comp.getSerializationFile());
    new_comp.setResourceName(comp.getResourceName());
    new_comp.setDataSourceType(comp.getDataSourceType());
    new_comp.setModelName(comp.getModelName());
    new_comp.setDataBinding(comp.getDataBinding());

    if (tree != null && tree.getRoot() != null) {
        DataSourceTreeNode root = new DataSourceTreeNode();
        JCHiGridNode node = createJCHiGridNode(new_comp);
        root.setUserObject(node);
        new_comp.setRoot(root);
        synchronize(new_comp, (DataSourceTreeNode) tree.getRoot(),
                    root, ((GridProperties)target).getRoot());
    }

    if (TRACE) {
        System.out.println("synchronize::new comp: " + new_comp);
    }

    addBeanEventListeners(new_comp);
    target = new_comp;
}

/** Add bean event listeners */
private void addBeanEventListeners(GridProperties comp) {
    comp.addJCHiGridListener(this);
    comp.addJCDataListener(this);
}

private void synchronize(GridProperties comp, DataSourceTreeNode meta, DataSourceTreeNode new_node, DataSourceTreeNode old_node) {
    if (meta == null) return;

    if (old_node != null) {
        new_node.setUserObject(old_node.getUserObject());
    }
    else {
        JCHiGridNode hgbn = createJCHiGridNode(comp);
        // set the description for display
        new_node.setUserObject(hgbn);
    }
    JCHiGridNode hgbn = (JCHiGridNode) new_node.getUserObject();
    hgbn.getNodeProperties().setDescription(((MetaDataModel)meta).getDescription());

    TreeIterator ti = meta.getIterator();
    TreeIterator oti = (old_node == null) ? null : old_node.getIterator();
    while (ti.hasMoreElements()) {
        DataSourceTreeNode child = (DataSourceTreeNode) ti.nextElement();
        DataSourceTreeNode nchild = new DataSourceTreeNode();
        DataSourceTreeNode ochild = null;
        if (oti != null && oti.hasMoreElements()) {
            ochild = (DataSourceTreeNode) oti.nextElement();
        }
        new_node.add(nchild);
        synchronize(comp, child, nchild, ochild);
    }
}

/** Create new JCHiGridNode and do initialization for a new node */
private JCHiGridNode createJCHiGridNode(GridProperties comp) {
    JCHiGridNode node = new JCHiGridNode(comp);
    node.addJCHiGridListener(comp);
    return node;
}

/** A thread to show tips dialog.
 *
 */
class ThreadTipsDialog implements Runnable {

    public void run() {
        for (;;) {
            if (tipsShowed) {
                break;
            }
            // editor is up.  show tips now.
            if (!isClosed()) {
                showTipsDialog();
                break;
            }
            try {
                showTipsThread.sleep(500);
            }
            catch (Exception ex) {
            }
        }
    }

    private void showTipsDialog() {
        if (tipsShowed) return;

        tipsShowed = true;
//	Frame frame = getFrame();

        TipsDialog tipsDialog = new TipsDialog(null,
                                               com.klg.jclass.higrid.LocaleBundle.string(com.klg.jclass.higrid.LocaleBundle.title_tips_on_customizer));
        String tip = com.klg.jclass.higrid.LocaleBundle.string(com.klg.jclass.higrid.LocaleBundle.tips_on_customizer) +
                     com.klg.jclass.higrid.LocaleBundle.string(com.klg.jclass.higrid.LocaleBundle.tips_on_customizer2);
        tipsDialog.setShowTipsAtStart(true);
        tipsDialog.setMessage(tip);
        tipsDialog.setModal(true);
        tipsDialog.pack();
        tipsDialog.setVisible(true);
        boolean v = tipsDialog.getShowTipsAtStart();
        JClassInfo.setProperty("higrid",
                               "higrid_bean_component.customizer.show_tips_at_start",
                               new Boolean(v).toString());
        tipsDialog.dispose();
    }

}

public static void main(String args[]) {
    JFrame f = new JFrame("GridPropertiesEditor");
    java.beans.Beans.setDesignTime(true);
    GridPropertiesEditor editor = new GridPropertiesEditor();
    editor.setValue(new GridProperties(editor.isExternalDS()));
    f.getContentPane().setLayout(new java.awt.BorderLayout());
    f.getContentPane().add(editor, java.awt.BorderLayout.CENTER);
    f.pack();
    f.show();
    // Handle window close requests
    f.addWindowListener(new java.awt.event.WindowAdapter() {
                            public void windowClosing(java.awt.event.WindowEvent e) {
                                System.exit(0);
                            }
                        });
}


}
