/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: SectionsPage.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:21 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.higrid.customizer;


import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.tree.*;

import com.klg.jclass.datasource.*;
import com.klg.jclass.higrid.*;
import com.klg.jclass.higrid.beans.*;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.swing.JCGridLayout;
import com.klg.jclass.higrid.LocaleBundle;

/** Displays and modify the showing properties of all sections.
 */
public class SectionsPage extends FormatPage implements ActionListener,
    ItemListener, ChooserDialogParent {

JButton btnRetrieveFields,
btnClearFormat;
JButton btnRetrieveAllFields,
btnClearAllFormat;
JCheckBox[] showing;
JTextField sortDataColumn;
JButton sortDataColumnButton;
JComboBox sortDataDirection;

ColorEditor nodeIndicatorForegroundColor;

static String[] labels = { LocaleBundle.header,
    LocaleBundle.footer,
    LocaleBundle.before_details,
    LocaleBundle.after_details
};

static int[] showSections = {JCHiGridNode.SECTION_HEADER,
    JCHiGridNode.SECTION_FOOTER,
    JCHiGridNode.SECTION_BEFORE_DETAILS,
    JCHiGridNode.SECTION_AFTER_DETAILS
};

static int NUM_SHOWING = JCHiGridNode.SECTION_MAX - 1;

// for ChooserDialog
transient Frame frame;
transient boolean useOwnFrame;
transient Vector columnVector;
transient int columnIndex;
transient boolean sortDataChanged;

transient boolean firstTimeFlag;
transient boolean showingCheckboxes;

public SectionsPage() {
    super(LocaleBundle.string(LocaleBundle.sections));
    firstTimeFlag = true;
    init();
}

public void init() {
    setLayout(new GridLayout(1, 1));


    JPanel grp = new JPanel();
    grp.setBorder(new TitledBorder(LocaleBundle.string(LocaleBundle.show)));
    grp.setLayout(new GridBagLayout());
    GridBagConstraints c = new GridBagConstraints();
    c.fill = GridBagConstraints.NONE;
    c.anchor = GridBagConstraints.WEST;
    c.weightx = 5;
    showing = new JCheckBox[NUM_SHOWING];
    for (int i=0; i<NUM_SHOWING; i++) {
        c.gridy = i;
        c.insets = new Insets(0, 5, (i+1==NUM_SHOWING) ? 5 : 0, 0);
        showing[i] = new JCheckBox(LocaleBundle.string(labels[i]));
        grp.add(showing[i], c);
        showing[i].addItemListener(this);
    }

    FocusListener fl = new FocusAdapter() {
        public void focusLost(FocusEvent e) {
            if (isInitializing()) return;

            nodeIndicatorForegroundColor.checkColorSource(e.getSource());
            valueChanged();
        }
    };

    ItemListener il = new ItemListener() {
        public void itemStateChanged(ItemEvent e) {
            if (isInitializing()) return;

            nodeIndicatorForegroundColor.checkColorSource(e.getSource());
            valueChanged();
        }
    };

    nodeIndicatorForegroundColor = new ColorEditor(il, this, fl,
                                                   LocaleBundle.string(LocaleBundle.nodeIndicatorFgColor), false);

    String stringNone = new String(LocaleBundle.string(LocaleBundle.choiceStringNone));
    sortDataColumn = new JTextField(stringNone, 16);
    sortDataColumnButton = new JButton("...");
    sortDataDirection = new JComboBox();
    JPanel panel3 = new JPanel(new BorderLayout(2, 2));
    JLabel lbl_sort_data_column = 
    new JLabel(LocaleBundle.string(LocaleBundle.sortDataColumn));
    JLabel lbl_sort_data_dir = 
    new JLabel(LocaleBundle.string(LocaleBundle.sortDataDirection));
    JPanel panel1 = new JPanel(new BorderLayout());
    JPanel panel2 = new JPanel(new BorderLayout());
    JPanel panel4 = new JPanel(new BorderLayout());
    btnRetrieveFields = new JButton(LocaleBundle.string(LocaleBundle.retrieve_fields));
    btnClearFormat = new JButton(LocaleBundle.string(LocaleBundle.clear_format));
    btnRetrieveAllFields = new JButton(LocaleBundle.string(LocaleBundle.retrieve_fields));
    btnClearAllFormat = new JButton(LocaleBundle.string(LocaleBundle.clear_format));
    JPanel p = new JPanel();
    JPanel content = new JPanel();
    JPanel grpSortData = new JPanel();
    grpSortData.setBorder(new TitledBorder(LocaleBundle.string(LocaleBundle.defaultSortData)));
    JPanel current_level_grp = new JPanel();
    current_level_grp.setBorder(new TitledBorder(LocaleBundle.string(LocaleBundle.current_level)));
    JPanel all_levels_grp = new JPanel();
    all_levels_grp.setBorder(new TitledBorder(LocaleBundle.string(LocaleBundle.all_levels)));
    sortDataColumn.setEditable(false);
    sortDataColumnButton.addActionListener(this);
    String choicesSDD[] = JCHiGridConverter.getSortDataDirectionStrings();
    for (int i = 0; i < choicesSDD.length; i++) {
        sortDataDirection.addItem(choicesSDD[i]);
    }
    sortDataDirection.addItemListener(this);

    grpSortData.setLayout(new JCGridLayout(JCGridLayout.VARIABLE, 2, 5, 5));
    panel3.add(sortDataColumn, BorderLayout.CENTER);
    panel3.add(sortDataColumnButton, BorderLayout.EAST);
    grpSortData.add(lbl_sort_data_column);
    grpSortData.add(panel3);
    grpSortData.add(lbl_sort_data_dir);
    grpSortData.add(sortDataDirection);
    panel1.add(grp, BorderLayout.NORTH);
    panel2.add(nodeIndicatorForegroundColor, BorderLayout.NORTH);
    panel4.add(panel2, BorderLayout.NORTH);
    panel4.add(grpSortData, BorderLayout.SOUTH);
    panel1.add(panel4, BorderLayout.SOUTH);

    btnRetrieveFields.addActionListener(this);
    btnClearFormat.addActionListener(this);
    btnRetrieveAllFields.addActionListener(this);
    btnClearAllFormat.addActionListener(this);

    current_level_grp.setLayout(new FlowLayout());
    all_levels_grp.setLayout(new FlowLayout());
    current_level_grp.add(btnRetrieveFields);
    current_level_grp.add(btnClearFormat);
    all_levels_grp.add(btnRetrieveAllFields);
    all_levels_grp.add(btnClearAllFormat);
    p.setLayout(new GridLayout(2, 1));
    p.add(current_level_grp);
    p.add(all_levels_grp);
    content.setLayout(new BorderLayout());
    content.add(panel1, BorderLayout.NORTH);
    content.add(p, BorderLayout.SOUTH);
    add(content);
}

/** Shows what sections are showing.
 */
public void setJCHiGridNode(JCHiGridNode node) {
    if (firstTime(node)) {
        // need actual meta data info
        clearFormat();
    }
    showValue();
}

private boolean firstTime(JCHiGridNode node) {
    if (firstTimeFlag) {
        firstTimeFlag = false;
        if (node == null) {
            return true;
        }
        NodeCellFormat nodeCell = null;
        Vector v = node.getRowFormat(JCHiGridNode.SECTION_RECORD).getIndicatorFormats();
        if (v == null) {
            return true;
        }
        for (Enumeration e = v.elements(); e.hasMoreElements();) {
            CellFormat cell = (CellFormat) e.nextElement();
            if (cell instanceof NodeCellFormat) {
                return false;
            }
        }
        return true;
    }
    return false;
}

/** Clear format at current level.
 */
void clearFormat() {
    editor.resetRecordAndHeader(getJCHiGridNode(), true, true);
    // Notify changes at this level.
    // Tabs will be refreshed
    getJCHiGridNode().getGridProperties().fireJCHiGridEvent(new JCHiGridEvent(getJCHiGridNode().getGridProperties(), getJCHiGridNode()));
}

/** Retrieve fields at current level.
 */
void retrieveFields() {
    editor.resetRecordAndHeader(getJCHiGridNode(), false, false);
    getJCHiGridNode().setChanged();
    // Notify changes at this level.
    // Tabs will be refreshed
    getJCHiGridNode().getGridProperties().fireJCHiGridEvent(new JCHiGridEvent(getJCHiGridNode().getGridProperties(), getJCHiGridNode()));
}

/** Clear format at all levels.
 */
void clearAllFormat() {
    editor.clearAllFormat();
    // Notify changes for the GridProperties.
    // Tabs will be refreshed to show the current level.
    getJCHiGridNode().getGridProperties().fireJCHiGridEvent(new JCHiGridEvent(getJCHiGridNode().getGridProperties(), getJCHiGridNode()));
}

/** Retrieve fields at all levels.
 */
void retrieveAllFields() {
    editor.retrieveAllFields();
    getJCHiGridNode().setChanged();
    // Notify changes for the GridProperties.
    // Tabs will be refreshed to show the current level.
    getJCHiGridNode().getGridProperties().fireJCHiGridEvent(new JCHiGridEvent(getJCHiGridNode().getGridProperties(), getJCHiGridNode()));
}

/** Overrides to perform operations necessary after value is changed on this page.
 */
public void valueChanged() {
    JCHiGridNode node = getJCHiGridNode();

    // set section showing
    for (int i=0; i<NUM_SHOWING; i++) {
        node.setSectionShowing(showSections[i], showing[i].isSelected());
    }

    // set node indicator foreground color
    Color color = nodeIndicatorForegroundColor.getColor();
    if (!color.equals(getNodeIndicatorForegroundColor())) {
        setNodeIndicatorForegroundColor(color);
        getJCHiGridNode().setChanged();
    }

    // sortData column and direction
    String s = (String) sortDataDirection.getSelectedItem();
    JCTypeConverter conv = new JCTypeConverter();
    int dir = JCHiGridConverter.toSortDataDirection(conv, s, SortGrid.UNKNOWN);
    if (node.getDefaultSortData() != null &&
        node.getDefaultSortData().getDirection() != dir) {
        sortDataChanged = true;
    }
    if (sortDataChanged) {
        sortDataChanged = false;
        SortData sortData = null;
        if (columnIndex < 1 || columnVector == null) {
            sortData = new SortData("", SortGrid.UNKNOWN);
        }
        else {
            sortData = new SortData((String)columnVector.elementAt(columnIndex), dir);
        }
        node.setDefaultSortData(sortData);
    }

    // When a section is selected/de-selected, notify observers to make changes
    // to the higrid bean node.
    getJCHiGridNode().getGridProperties().fireJCHiGridEvent(new JCHiGridEvent(getJCHiGridNode().getGridProperties(), getJCHiGridNode()));
}

/** Overrides to refresh values displayed on this page after the underlined
 *	<code>JCHiGridNode</code> is changed.
 */
public void showValue() {
    JCHiGridNode node = getJCHiGridNode();
    showingCheckboxes = true;
    for (int i=0; i<NUM_SHOWING; i++) {
        showing[i].setSelected(node.isSectionShowing(showSections[i]));
    }
    showingCheckboxes = false;

    Color color = getNodeIndicatorForegroundColor();
    if (color != null) {
        nodeIndicatorForegroundColor.selectColor(color);
    }

    SortData sortData = node.getDefaultSortData();
    if (sortData == null) {
        sortData = new SortData("", SortGrid.UNKNOWN);
        node.setDefaultSortData(sortData);
    }
    String columnName = sortData.getColumnName();
    boolean hasColumnName = true;
    if (columnName == null || columnName.length() == 0) {
        columnName = new String(LocaleBundle.string(LocaleBundle.choiceStringNone));
        hasColumnName = false;
    }
    sortDataColumn.setText(columnName);
    getChoiceList();
    columnIndex = columnVector.indexOf(columnName);
    int dir = sortData.getDirection();
    int index = -1;
    if (columnName.length() > 0) {
        for (int j = 0; j < JCHiGridConverter.sortDataDirectionStrings.length; j++) {
            if (dir == JCHiGridConverter.sortDataDirectionValues[j]) {
                index = j;
                break;
            }
        }
    }
    index = Math.max(0, index);
    sortDataDirection.setSelectedIndex(index);
    sortDataDirection.setEnabled(hasColumnName);
}

public void itemStateChanged(ItemEvent e) {
    if (isInitializing()) {
        return;
    }
    if (showingCheckboxes) {
        return;
    }
//FIXOBSOLETE
// Q: this line causes the format tree to be recreated; is it necessary?
// A: it is used to enable the Footer tab when the checkbox is clicked
//    Unfortunately, it is also called when these tabs are created.	
    valueChanged();
}

public void actionPerformed(ActionEvent e) {
    if (isInitializing()) return;

    if (e.getSource() instanceof JButton) {
        JButton btn = (JButton) e.getSource();
        if (btn.equals(btnRetrieveFields)) {
            retrieveFields();
        }
        else if (btn.equals(btnClearFormat)) {
            clearFormat();
        }
        else if (btn.equals(btnRetrieveAllFields)) {
            retrieveAllFields();
        }
        else if (btn.equals(btnClearAllFormat)) {
            clearAllFormat();
        }
        else if (btn.equals(sortDataColumnButton)) {
            getSortDataColumnField();
        }
    }
    nodeIndicatorForegroundColor.checkColorSource(e.getSource());
    valueChanged();
}

void setNodeIndicatorForegroundColor(Color color) {
    JCHiGridNode node = getJCHiGridNode();
    if (node == null || color == null) {
        return;
    }
    if (color.equals(getNodeIndicatorForegroundColor())) {
        return;
    }
    NodeCellFormat nodeCell = null;
    for (int i = 0; i < JCHiGridNode.SECTION_MAX; i++) {
        Vector v = node.getRowFormat(i).getIndicatorFormats();
        if (v == null) {
            continue;
        }
        for (Enumeration e = v.elements(); e.hasMoreElements();) {
            CellFormat cell = (CellFormat) e.nextElement();
            if (cell instanceof NodeCellFormat) {
                nodeCell = (NodeCellFormat) cell;
                nodeCell.setForeground(color);
                break;
            }
        }
    }
}

Color getNodeIndicatorForegroundColor() {
    Color color = null;
    JCHiGridNode node = getJCHiGridNode();
    if (node == null) {
        return color;
    }
    NodeCellFormat nodeCell = null;
    // assume all section have the same color
    Vector v = node.getRowFormat(JCHiGridNode.SECTION_RECORD).getIndicatorFormats();
    if (v == null) {
        return color;
    }
    for (Enumeration e = v.elements(); e.hasMoreElements();) {
        CellFormat cell = (CellFormat) e.nextElement();
        if (cell instanceof NodeCellFormat) {
            nodeCell = (NodeCellFormat) cell;
            color = nodeCell.getForeground();
            break;
        }
    }
    return color;
}

void getSortDataColumnField() {
    if (editor == null) {
        return;
    }
    ChooserDialog chooser = new ChooserDialog((ChooserDialogParent) this);
    chooser.show();
    if (useOwnFrame) {
        useOwnFrame = false;
        frame.dispose();
    }
}

/** ChooserDialogParent implementation*/
public Frame getFrame() {
//FIXOBSOLETE
    return null;
/*    
frame = BWTUtil.getFrame(this);
if (frame == null) {
    useOwnFrame = true;
    frame = new Frame();
}
else {
    useOwnFrame = false;
}
return frame;
*/
}

/** Gets the dialog title
 */
public String getTitle() {
    return LocaleBundle.string(LocaleBundle.columnChooser);
}

/** Gets the choice list,
 */
public Vector getChoiceList() {
    RowFormat format = getJCHiGridNode().getRowFormat(JCHiGridNode.SECTION_RECORD);
    Vector v = (Vector) format.getDataFormats();

    columnVector = new Vector();
    if (v != null) {
        columnVector.addElement((Object) LocaleBundle.string(LocaleBundle.sortDataColumnNone));
        for (Enumeration e=v.elements(); e.hasMoreElements();) {
            CellFormat cell = (CellFormat) e.nextElement();
            String name = new String(cell.getName());
            columnVector.addElement((Object) name);
        }
    }
    return columnVector;
}

/**	Choice selected in ChooserDialog is returned.
 */
public void choiceSelected(int index) {
    columnIndex = index;
    sortDataChanged = true;
    if (index < 0) {
        return;
    }
    sortDataColumn.setText((index == 0) ? 
                           LocaleBundle.string(LocaleBundle.choiceStringNone) : 
                           (String)columnVector.elementAt(index));
}

}
