/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: EPSImage.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:22 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page;

import java.io.BufferedReader;
import java.awt.Image;
import java.awt.Graphics;
import java.awt.image.ImageProducer;
import java.awt.image.ImageObserver;

/**
 * EPSImage encapsulates the input of an EPS image, plus display attributes.
 */
public class EPSImage extends Image {

/** The dimensions at which the EPS is to be displayed */
protected JCUnit.Dimension  size;
/** The file or buffer stream which contains the data of the image */
protected BufferedReader    inputSource;


////////////// Image methods ////////////////////////////////////////

/**
 * Determines the width of the image. If the width is not yet known,
 * this method returns <code>-1</code> and the specified
 * <code>ImageObserver</code> object is notified later.
 * @param     observer   an object waiting for the image to be loaded.
 * @return    the width of this image, or <code>-1</code>
 *                   if the width is not yet known.
 * @see       java.awt.Image#getHeight
 * @see       java.awt.image.ImageObserver
 */
public int getWidth(ImageObserver observer) {
	int width = -1;
	if (size != null) {
		JCUnit.Measure m = size.getWidth();
		width = (int) m.getAs(JCUnit.POINTS);
	}
	return width;
}

/**
 * Determines the height of the image. If the height is not yet known,
 * this method returns <code>-1</code> and the specified
 * <code>ImageObserver</code> object is notified later.
 * @param     observer   an object waiting for the image to be loaded.
 * @return    the height of this image, or <code>-1</code>
 *                   if the height is not yet known.
 * @see       java.awt.Image#getWidth
 * @see       java.awt.image.ImageObserver
 */
public int getHeight(ImageObserver observer) {
	int height = -1;
	if (size != null) {
		JCUnit.Measure m = size.getWidth();
		height = (int) m.getAs(JCUnit.POINTS);
	}
	return height;
}

/**
 * Gets the object that produces the pixels for the image.
 * This method is called by the image filtering classes and by
 * methods that perform image conversion and scaling.
 * @return     the image producer that produces the pixels
 *                                  for this image.
 * @see        java.awt.image.ImageProducer
 */
public ImageProducer getSource() {
	return null;
}

/**
 * Creates a graphics context for drawing to an off-screen image.
 * This method can only be called for off-screen images.
 * @return  a graphics context to draw to the off-screen image.
 * @see     java.awt.Graphics
 * @see     java.awt.Component#createImage(int, int)
 */
public Graphics getGraphics() {
	return null;
}

/**
 * Gets a property of this image by name.
 * <p>
 * Individual property names are defined by the various image
 * formats. If a property is not defined for a particular image, this
 * method returns the <code>UndefinedProperty</code> object.
 * <p>
 * If the properties for this image are not yet known, this method
 * returns <code>null</code>, and the <code>ImageObserver</code>
 * object is notified later.
 * <p>
 * The property name <code>"comment"</code> should be used to store
 * an optional comment which can be presented to the application as a
 * description of the image, its source, or its author.
 * @param       name   a property name.
 * @param       observer   an object waiting for this image to be loaded.
 * @return      the value of the named property.
 * @see         java.awt.image.ImageObserver
 * @see         java.awt.Image#UndefinedProperty
 */
public Object getProperty(String name, ImageObserver observer) {
	return Image.UndefinedProperty;
}

/**
 * Creates a scaled version of this image.
 * A new <code>Image</code> object is returned which by default renders
 * the image at the specified <code>width</code> and
 * <code>height</code>.  The new <code>Image</code> object
 * may be loaded asynchronously even if the original source image
 * has already been loaded completely.  If either the <code>width</code>
 * or <code>height</code> is a negative number then a value is
 * substituted to maintain the aspect ratio of the image's original
 * dimensions.
 * @param width the width to which to scale the image.
 * @param height the height to which to scale the image.
 * @param hints flags to indicate the type of algorithm to use
 * for image resampling.
 * @return     a scaled version of the image.
 * @see        java.awt.Image#SCALE_DEFAULT
 * @see        java.awt.Image#SCALE_FAST
 * @see        java.awt.Image#SCALE_SMOOTH
 * @see        java.awt.Image#SCALE_REPLICATE
 * @see        java.awt.Image#SCALE_AREA_AVERAGING
 * @since      JDK1.1
 */
public Image getScaledInstance(int width, int height, int hints) {
	return this;
}

/**
 * Flushes all resources used by this Image object.  This
 * includes any pixel data that is cached for rendering to
 * the screen as well as any system resources that are used
 * to store data or pixels for the image.  The image is reset to
 * a state similar to when it was first created so that if it is
 * again rendered, the image data will have to be recreated or
 * fetched again from its source.
 */
public void flush() {
	//
}

/////////////////// End of awt.Image methods /////////////////////////


/**
 * Get the display size of the EPS image
 * @return The output size of the image
 */
public JCUnit.Dimension getSize() {
	return size;
}

/**
 * Specify the dimensions of the image
 * @param size The output size of the image
 */
public void setSize(JCUnit.Dimension size) {

	this.size = size;
}

/**
 * Return the input source for this image
 */
public BufferedReader getInputSource() {
	return inputSource;
}

/**
 * Create an EPS image object given the input source
 */
public EPSImage(BufferedReader input) {

	// Assign default values
	inputSource = input;
	size = new JCUnit.Dimension(JCUnit.INTERNAL, 1.0, 1.0);
}

}
