/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: FontBase.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:22 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page;

import java.awt.*;
import java.awt.font.FontRenderContext;
import java.awt.font.GlyphVector;
import java.io.IOException;
import java.io.InputStream;
import java.text.CharacterIterator;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;


/**
 *
 * Extends Font to provide common code for subclasses.
 *
 */
public abstract class FontBase extends Font {

// constants
protected static int DEFAULT_FONT_SIZE 		= 12;
protected static char REPLACEMENT_CHARACTER	='_';


/**
 * Constructor
 * @param name the String name of the font
 * @param style the style for this font
 * @param size the size for this font
 */
public FontBase(String name, int style, int size) {

    super(name, style, size);

}

/**
 * Load fonts from "fonts.jar" file located in
 *
 * 	com/klg/jclass/page/adobe/fonts.jar or
 * 	com/klg/jclass/page/pcl/fonts.jar
 *
 */
private static void loadFonts(FontParser parser, String file, String fileExtension,
	Hashtable fonts)
		throws IOException {

	String lcFileExtension = fileExtension.toLowerCase();
    // get an input stream from the class loader
    InputStream is		= FontBase.class.getResourceAsStream(file);
    if (is == null) {
        throw new RuntimeException("Font archive not found: " + file);
    }

    // break out the individual afm/tfm files
    ZipInputStream zis  = new ZipInputStream(is);
    ZipEntry entry 		= null;

    while ( (entry = zis.getNextEntry()) != null) {
        if (entry.getName().toLowerCase().endsWith(lcFileExtension)) {
            Vector v = parser.parse(zis);
            // because each tfm file can contain multiple fonts ...
            for (Enumeration e = v.elements(); e.hasMoreElements(); ) {
                Font font = (Font) e.nextElement();
                fonts.put(font.getName().trim(), font);

            }
            zis.closeEntry();
        }
    }
    zis.close();
}

/************** Overridden Font methods ********/

/**
 * Overrides Font method.
 */
public static Font getFont(String nm) {
	return decode(nm);
}


/**
 * Overrides Font method.
 */
public static Font getFont(String nm, Font font) {
	return decode(nm);
}

/************ End overridden Font methods **************/


/**
 * Return an array of fonts. If fonts are not already loaded,
 * loadFonts() is called.
 */
protected static Font[] getFonts(FontParser parser, String file, String ext,
		Hashtable fonts, boolean loaded){
	if (!loaded) {
		try {
			loadFonts(parser, file, ext, fonts);
		}
		catch (Exception e) {
			System.out.println(e.toString());
		}
	}
	Font a[] = new Font[fonts.size()];
	int cnt = 0;
	for (Enumeration e = fonts.elements(); e.hasMoreElements(); ) {
		a[cnt++] = (Font) e.nextElement();
	}
 	return a;
}

/**
 *
 * Given a font, build the decode string so the
 * corresponding device-dependant font can be decoded
 *
 */
public static String getDecodeString(String name, int style, int size) {
	// pagelayout will pass in a generic awt font
	// from which we have to deduce the appropriate
	// afm/tfm/pcl font

	// strings are of the format
	// 		fontname-style-ptsize
	// 		fontname-ptsize
	// 		fontname-size
	// 		fontname

	StringBuffer str 	= new StringBuffer(name);
	String strStyle 	= getStyleString(style);


	if (!strStyle.equals("")) {
		str.append(strStyle);
	}

	// include size
	str.append("-");
	str.append(size);

	return str.toString();
}

/**
 *
 * Given a style constant return its string.
 * Returns -bolditalic, -bold, -italic, or ""
 */
protected static String getStyleString(int style) {
	String strStyle = "";
	if (style == (Font.BOLD | Font.ITALIC)) {
		strStyle = "-BoldItalic";
	}
	else if (style == Font.BOLD) {
		strStyle = "-Bold";
	}
	else if (style == Font.ITALIC) {
		strStyle = "-Italic";
	}
	return strStyle;
}

/**
 * Correctly processes a call to deriveFont() for the current JDK level
 */
public static Font deriveFont(Font f, int style, float size) {

	return(f.deriveFont(style, size));

}

/**
 * Correctly processes a call to deriveFont() for the current JDK level
 */
public static Font deriveFont(Font f, int style) {

	return(f.deriveFont(style));

}

/**
 * Correctly processes a call to deriveFont() for the current JDK level
 */
public static Font deriveFont(Font f, float size) {

	return(f.deriveFont(size));

}

/**
 * Reinterprets getSize2D depending on the Java version
 */
public static float getSize2D(Font f) {

	return(f.getSize2D());

}

/**
 * Correctly remaps a call to createGlyphVector() for the current JDK level
 */
public static GlyphVector createGlyphVector(Font f, FontRenderContext frc,
											String text) {

	return(f.createGlyphVector(frc, text));

}

/**
 * Correctly remaps a call to createGlyphVector() for the current JDK level
 */
public static GlyphVector createGlyphVector(Font f, FontRenderContext frc,
											char[] chars) {

	return(f.createGlyphVector(frc, chars));

}

/**
 * Correctly remaps a call to createGlyphVector() for the current JDK level
 */
public static GlyphVector createGlyphVector(Font f, FontRenderContext frc,
											CharacterIterator iter) {

	return(f.createGlyphVector(frc, iter));

}

//
// This section contains necessary methods which java.awt.Font implements
// in Java2, but are not present in Java1.1.
//

} // end FontBase class
