/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCDrawStyle.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:22 $ $Locker:  $  Quest Software Inc.
package com.klg.jclass.page;



import java.awt.*;
import java.awt.geom.GeneralPath;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

/**
 * <code>JCDrawStyle</code> creates styles for drawn objects, specifying line and fill
 * attributes (for example, color and width).
 */
public class JCDrawStyle implements Cloneable {

/** The unique name of the style. */
protected String            name;

/** Foreground colour for drawing graphics. */
protected Color             foreground;
/** FUTURE: Background colour for drawing graphics. */
// protected Color             background;
/** Foreground colour for filling graphics. */
protected Color             fillForeground;
/** FUTURE: Background colour for filling graphics. */
// protected Color             fillBackground;

/** Method for determining which parts of a complex polygon are "inside". */
protected int               fillRule;
/** FUTURE: Pattern to be used for fills. */
// protected int               fillPattern;

/** Width of a drawn line. */
protected JCUnit.Measure    lineWidth;
/** The type of line to draw. */
protected int               lineType;
/** The spacing between lines in a multi-line style. */
protected JCUnit.Measure    lineSpacing;
/** The length of dashes and spaces in a broken line style. */
protected JCUnit.Measure    dashLength;

/** Flag identifying if the style may be modified. */
protected boolean           immutable;

/** Counts of the number of instances of the class. */
protected static int        styleCount = 0;
/** A list containing references to all named style instances. */



protected static TreeMap    instanceCollection = new TreeMap();


/*
 * Fill rules for determining the "inside" of a polygon.
 */
/** Uses the even/odd rule to determine the inside of a closed figure. */
public static final int FILL_RULE_EVEN_ODD = GeneralPath.WIND_EVEN_ODD;
/** Uses the non-zero winding rule to determine the inside of a closed figure. */
public static final int FILL_RULE_NON_ZERO_WINDING = GeneralPath.WIND_NON_ZERO;

/*
 * Line type (appearance) for line drawing.
 */
/** Don't draw the line. */
public static final int LINE_TYPE_NONE = 1;
/** Draws a single unornamented line. */
public static final int LINE_TYPE_PLAIN = 2;
/** Draws a single unornamented line. */
public static final int LINE_TYPE_SINGLE = 2;
/** Draws a dashed/dotted line. */
public static final int LINE_TYPE_BROKEN = 3;
/** Draws a pair of parallel lines. */
public static final int LINE_TYPE_DOUBLE = 4;
// /** Draws a pair of parallel dashed/dotted lines. */
// public static final int LINE_TYPE_BROKEN_DOUBLE = 5;
// /** Draws three parallel lines. */
// public static final int LINE_TYPE_TRIPLE = 6;
// /** Draws a line using a fill pattern instead of a solid colour. */
// public static final int LINE_TYPE_FILLED = 7;

/*
 * Vertical position of embedded objects on a line.
 */
/** The bottom of the object aligns to the text baseline. */
public static final int POSITION_ON_BASELINE = 1;
/** The top of the object aligns to the text baseline. */
public static final int POSITION_BELOW_BASELINE = 2;
/** The center of the object aligns to the text baseline. */
public static final int CENTER_ON_BASELINE = 3;
/** The center of the object aligns to the center of the text ascender
	(space used above the baseline). */
public static final int CENTER_ABOVE_BASELINE = 4;
/** The center of the object aligns to the center of the text
	(between the top of the ascender and bottom of the descender). */
public static final int CENTER_IN_LINE = 5;
/** The top of the object is aligned to the top of the text ascender. */
public static final int ALIGN_TO_TOP = 6;
/** The bottom of the object is aligned to the bottom of the text descender. */
public static final int ALIGN_TO_BOTTOM = 7;

/** The name of the style which draws a solid line of minimum width. */
public static final String DEFAULT_LINE = "default line";
/** The name of the style that draws a solid 1 point line. */
public static final String DEFAULT_1PT_LINE = "default 1pt line";
/** The name of the style which draws a solid line 2 points in width. */
public static final String DEFAULT_2PT_LINE = "default 2pt line";

protected static final String UNAMED_STYLE_PREFIX = "jcpldrawstyle";
protected Map immutableDrawStyles = new Hashtable();

// These constants make the following pre-defined styles easier to read
private static final JCUnit.Measure DEFAULT_WIDTH = null;
private static final JCUnit.Measure DEFAULT_SPACING = null;
private static final JCUnit.Measure DEFAULT_DASH = null;

/** A drawing style that does not paint drawn lines.  The value of the <code>name</code>
 * property for this style is <code>default blank</code>.
 */
public static final JCDrawStyle BLANK =
			new JCDrawStyle("default blank", null,
							FILL_RULE_EVEN_ODD, LINE_TYPE_SINGLE,
							DEFAULT_WIDTH, DEFAULT_SPACING, DEFAULT_DASH, true);

/** A default style for thin black lines. The value of the <code>name</code> property for
 * this style is <code>default line</code>.
 */
public static final JCDrawStyle LINE =
			new JCDrawStyle("default line", Color.black,
							FILL_RULE_EVEN_ODD, LINE_TYPE_SINGLE,
							DEFAULT_WIDTH, DEFAULT_SPACING, DEFAULT_DASH, true);

/** A default style for thin black double lines. The value of the <code>name</code> property
 * for this style is <code>default double line</code>.
 */
public static final JCDrawStyle LINE_DOUBLE =
			new JCDrawStyle("default double line", Color.black,
							FILL_RULE_EVEN_ODD, LINE_TYPE_DOUBLE,
							DEFAULT_WIDTH,
							new JCUnit.Measure(JCUnit.POINTS, 1.0),
							DEFAULT_DASH, true);

/** A default style for thick (1 point) black lines. The value of the <code>name</code>
 * property for this style is <code>default 1pt line</code>.*/
public static final JCDrawStyle LINE_1POINT =
			new JCDrawStyle("default 1pt line", Color.black,
							FILL_RULE_EVEN_ODD, LINE_TYPE_SINGLE,
							new JCUnit.Measure(JCUnit.POINTS, 1.0),
							DEFAULT_SPACING, DEFAULT_DASH, true);

/** A default style for very thick (2 point) black lines. The value of the <code>name</code>
 * property for this style is <code>default 2pt line</code>
 */
public static final JCDrawStyle LINE_2POINT =
			new JCDrawStyle("default 2pt line", Color.black,
							FILL_RULE_EVEN_ODD, LINE_TYPE_SINGLE,
							new JCUnit.Measure(JCUnit.POINTS, 2.0),
							DEFAULT_SPACING, DEFAULT_DASH, true);

/** A default style for thin dashed lines. The value of the <code>name</code> property for
 * this style is <code>dashed line</code>.
 */
public static final JCDrawStyle LINE_DASHED =
			new JCDrawStyle("dashed line", Color.black,
							FILL_RULE_EVEN_ODD, LINE_TYPE_BROKEN,
							DEFAULT_WIDTH, DEFAULT_SPACING,
							new JCUnit.Measure(JCUnit.POINTS, 9.0), true);


/**
 * Creates a named draw style with the given attributes.
 * @param name the name to assign to the style
 * @param foreground the colour to paint drawn objects
 * @param fill_rule the rule used to determine the inside of figures
 * @param line_type the type of line to be drawn
 * @param line_width the width of a drawn line
 * @param line_spacing the distance between double lines
 * @param dash_length the length of dashes and spaces in a broken line
 * @param immutable indicates if this style is to be made immutable
 */
public JCDrawStyle(String name,
				   Color foreground, int fill_rule, int line_type,
				   JCUnit.Measure line_width, JCUnit.Measure line_spacing,
				   JCUnit.Measure dash_length, boolean immutable) {

	this.name = name;

	// Use the set methods so we can error-check the inputs.
	setForegroundColor(foreground);
	setFillRule(fill_rule);
	setLineWidth(line_width);
	setLineSpacing(line_spacing);
	setDashLength(dash_length);
	setLineType(line_type);

	this.immutable = immutable;

	if (!name.regionMatches(0, UNAMED_STYLE_PREFIX,
							0, UNAMED_STYLE_PREFIX.length())) {
		// store da ting
		// ??? Should we check for name conflicts first?
		instanceCollection.put(name, this);
	}

}

/**
 * Creates a draw style with default name and values.
 */
public JCDrawStyle() {

	// Assign default values
	// default name
	this(UNAMED_STYLE_PREFIX + (++styleCount), Color.black,
		 FILL_RULE_EVEN_ODD, LINE_TYPE_SINGLE,
		 new JCUnit.Measure(JCUnit.INTERNAL, 0.0),
		 DEFAULT_SPACING, DEFAULT_DASH, false);
}

/**
 * Creates a draw style with the given name and default values.
 * @param name the name to give the new draw style
 */
public JCDrawStyle(String name) {
	this();
	setName(name);
}

/**
 * Gets the name of the style.
 * @return the name used to describe the style
 */
public String getName() {
	return name;
}

/**
 * Sets a name for the style.
 * @param name the new name for the style
 */
public void setName(String name) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable objects");
	}

	// Find a style (if any) having the same name
	if (instanceCollection.containsKey(this.name)) {
		// remove this mapping (discard previous version of this style)
		instanceCollection.remove(this.name);
	}

	this.name = name;

	if (!name.regionMatches(0, UNAMED_STYLE_PREFIX,
							0, UNAMED_STYLE_PREFIX.length())) {
		// store da ting
		instanceCollection.put(name, this);
	}
}

/**
 * Gets the color in which lines are to be drawn.
 * @return the line foreground currently assigned to this style
 */
public Color getForegroundColor() {
	return foreground;
}

/**
 * Sets the color in which lines are to be drawn.
 * @param color the foreground color to assign to the graphics
 */
public void setForegroundColor(Color color) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable objects");
	}

	this.foreground = color;
}

/**
 * Gets the color which is to be drawn between lines.
 * @return the line background currently assigned to this style
 */
/* FUTURE
public Color getBackgroundColor() {
	return background;
}
*/

/**
 * Sets the color in which line backgrounds are to be drawn.
 * @param color the background color to assign to the graphics
 */
/* FUTURE
public void setBackgroundColor(Color color) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable objects");
	}

	this.background = color;
}
*/

/**
 * Gets the color in which figures are to be filled.
 * @return the fill foreground currently assigned to this style
 */
public Color getFillForegroundColor() {
	return fillForeground;
}

/**
 * Sets the color in which fills are to be drawn.
 * @param color the foreground color to assign to fill areas
 */
public void setFillForegroundColor(Color color) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable objects");
	}

	this.fillForeground = color;
}

/**
 * Gets the color which is to be used to fill figures.
 * @return the fill background currently assigned to this style
 */
/* FUTURE
public Color getFillBackgroundColor() {
	return fillBackground;
}
*/

/**
 * Sets the color in which fill backgrounds are to be drawn.
 * @param color the background color to assign to fill areas
 */
/* FUTURE
public void setFillBackgroundColor(Color color) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable objects");
	}

	this.fillBackground = color;
}
*/

/**
 * Gets the method used to determine the "inside" and "outside" of a polygon.
 * @return the rule used to determine the "inside" of a polygon for fills
 */
public int getFillRule() {
	return fillRule;
}

/**
 * Sets the method used to determine the "inside" and "outside" of a polygon.
 * @param fill_rule the rule used to determine "inside" for filling a polygon
 */
public void setFillRule(int fill_rule) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable objects");
	}

	if (fill_rule != FILL_RULE_NON_ZERO_WINDING &&
		fill_rule != FILL_RULE_EVEN_ODD) {
		throw new IllegalArgumentException("Unrecognized fill rule: "
										   + fill_rule);
	}

	this.fillRule = fill_rule;
}

/**
 * Gets the pattern used to fill a region.
 * @return the pattern used to draw a fill
 */
/*
public int getFillPattern() {
	return fillPattern;
}
*/

/**
 * Sets the pattern used to fill a region.
 * @param pattern the pattern used to draw a fill
 */
/*
public void setFillPattern(int fill_pattern) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable objects");
	}

	if (fill_pattern <= 0 || fill_pattern > FILL_PATTERN_LAST) {
		throw new IllegalArgumentException("Unrecognized fill pattern");
	}

	this.fillPattern = fill_pattern;
}
*/

/**
 * Gets the width of drawn lines.
 * @return the width used to draw individual lines
 */
public JCUnit.Measure getLineWidth() {
	return lineWidth;
}

/**
 * Sets the width of drawn lines.
 * @param width the width to draw individual lines
 */
public void setLineWidth(JCUnit.Measure width) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable objects");
	}

	if (width == null) {
		this.lineWidth = new JCUnit.Measure(JCUnit.POINTS, 0.1);
	}
	else {
		if (width.distance < 0.0) {
			throw new IllegalArgumentException("Line width must be >= 0");
		}

		this.lineWidth = (JCUnit.Measure) width.clone();
	}
}

/**
 * Gets the type used for drawing lines.
 * @return the type of line pattern to use for drawing lines
 */
public int getLineType() {
	return lineType;
}

/**
 * Sets the type of line this style will draw.
 * @param line_type the type of line that this style will use
 */
public void setLineType(int line_type) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable objects");
	}

	if (line_type <= 0 || line_type > LINE_TYPE_DOUBLE) {
		throw new IllegalArgumentException("Unrecognized line type");
	}

	this.lineType = line_type;

	// If line type is broken, we set dashes = 10x line width or 2 points
	if (line_type == LINE_TYPE_BROKEN && dashLength.isZero()) {
		if (lineWidth.isZero()) {
			dashLength = new JCUnit.Measure(JCUnit.POINTS, 2.0);
		}
		else {
			dashLength.assign(lineWidth);
			dashLength.multiply(10.0);
		}
	}
}

/**
 * Gets the spacing between drawn lines in a multi-line style.
 * @return the width used to separate multiple parallel lines
 */
public JCUnit.Measure getLineSpacing() {
	return lineSpacing;
}

/**
 * Gets the spacing between drawn lines in a multi-line style.
 * @param spacing the width used to separate multiple parallel lines
 */
public void setLineSpacing(JCUnit.Measure spacing) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable objects");
	}

	if (spacing == null) {
		this.lineSpacing = new JCUnit.Measure();
	}
	else {
		if (spacing.distance <= 0.0) {
			throw new IllegalArgumentException("Line spacing must be > 0");
		}

		this.lineSpacing = (JCUnit.Measure) spacing.clone();
	}
}

/**
 * Gets the total width of a line.
 * @return the width required to draw the (possibly multiple) line
 */
public JCUnit.Measure getDrawnWidth() {

	double  width;

	switch (lineType) {

		case LINE_TYPE_DOUBLE:
			width = 2 * lineWidth.internal()
				+ lineSpacing.internal();
			return(new JCUnit.Measure(JCUnit.INTERNAL, width));

		/*
		case LINE_TYPE_TRIPLE:
			width = 3 * lineWidth.internal()
				+ 2 * lineSpacing.internal();
			return(new JCUnit.Measure(JCUnit.INTERNAL, width));
		*/

		default:
			return((JCUnit.Measure) lineWidth.clone());
	}
}

/**
 * Gets the length of dashes and space between dashes in a broken line style.
 * @return the length of a dash in a drawn line
 */
public JCUnit.Measure getDashLength() {
	return dashLength;
}

/**
 * Sets the length of dashes and space between dashes in a broken line style.
 * @param length the length of a dash in a drawn line
 */
public void setDashLength(JCUnit.Measure length) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable objects");
	}

	if (length == null) {
		this.dashLength = new JCUnit.Measure(JCUnit.POINTS, 0.0);
	}
	else {
		if (length.distance <= 0.0) {
			throw new IllegalArgumentException("Dash length must be > 0");
		}

		this.dashLength = (JCUnit.Measure) length.clone();
	}
}

/**
 * Determines if the style can be modified.
 * @return <code>true</code>/<code>false</code>, depending whether or not the style can be modified
 */
public boolean isMutable() {
	return(!immutable);
}

/**
 * Sets the style so that it can no longer be modified.
 */
public void makeImmutable() {

	// Mark the style unchangeable
	immutable = true;
}

/**
 * Sets the style so that it can no longer be modified, storing
 * the style in the name lookup table.
 */
public void makeImmutableAndStore() {

	// Ensure the name is unique in the space of immutable styles
	if (instanceCollection.containsKey(name)) {
		throw new DuplicateNameException("Style name already exists: " + name);
	}

	// Add the style to the style map
	instanceCollection.put(name, this);

	// Mark the style unchangeable
	immutable = true;
}

/**
 * Sets the style so that it can no longer be modified and stores its
 * name in the lookup map, overriding any existing style with the same name.
 */
public void makeImmutableAndOverride() {

	// Find a style (if any) having the same name
	if (instanceCollection.containsKey(name)) {
		// remove the mapping for the old version of this style
		instanceCollection.remove(name);
	}

	// Add the style to the style map
	instanceCollection.put(name, this);

	// Mark the style unchangeable
	immutable = true;
}

/**
 * Returns a style which is a copy of this one.
 * @return a new style with all significant fields copied
 */
public Object clone() {

	JCDrawStyle     new_style;

	// Clone the style
	try {
		new_style = (JCDrawStyle) super.clone();
	}
	catch (CloneNotSupportedException e) {
		return(null);
	}

	// Copy the name of the old style
	new_style.name = new String(name);
	// make sure to clone the Measure objects.  Otherwise, the two
	// object will be sharing the same Measure objects.
	new_style.lineWidth = (JCUnit.Measure) lineWidth.clone();
	new_style.lineSpacing = (JCUnit.Measure) lineSpacing.clone();
	new_style.dashLength = (JCUnit.Measure) dashLength.clone();
	
	new_style.immutable = false;

	// return the newly created style copy
	return(new_style);
}

/**
 * Compares another draw style to this one and returns <code>true</code> if the two
 * draw styles are equivalent.
 * @param style the draw style to compare to this one
 * @return <code>true</code> if the draw styles result in approximately "equivalent" borders
 */
public boolean isEqualBorder(JCDrawStyle style) {

	Color   colour;

	// If the other border is null, they are unequal
	if (style == null) {
		return(false);
	}

	// if the line styles are different so are the borders
	if (lineType != style.getLineType()) {
		return(false);
	}

	// the borders are also different if they are different thicknesses
	if (!lineWidth.equals(style.getLineWidth())) {
		return(false);
	}

	// if the colours of the lines differ, the borders are not equal
	colour = style.getForegroundColor();
	if ((foreground == null || colour == null) && foreground != colour) {
		return(false);
	}
	if (foreground != null && !foreground.equals(colour)) {
		return(false);
	}

	// if the line type is double and spacing is different, they differ
	if (lineType == LINE_TYPE_DOUBLE &&
		!lineSpacing.equals(style.getLineSpacing())) {
		return(false);
	}

	// if the line type is broken (dashed) and the dash length is different
	// then the border styles differ
	if (lineType == LINE_TYPE_BROKEN &&
		!dashLength.equals(style.getDashLength())) {
		return(false);
	}

	// otherwise we consider them equal
	return(true);
}

/**
 * Provides an iterator over all immutable styles.
 * @return an iterator the style class's underlying <code>SortedMap</code>
 */
public static Iterator getStyles() {
	return(instanceCollection.entrySet().iterator());
}

/**
 * Returns the entire style Map.
 * @return a Map interface to the styles
 */
public static Map getStyleList() {
	return(instanceCollection);
}

/**
 * Returns the style which matches the given name.
 * @param name the name of an existing style
 * @return the style object which has that name, or null if not found
 */
public static JCDrawStyle stringToStyle(String name) {

	// Find the name in the Collection of instances
	if (instanceCollection.containsKey(name)) {
		return((JCDrawStyle) instanceCollection.get(name));
	}

	return null;
}

/**
 * Creates a hash that generates the same value depending on the
 * property values.
 * @return
 */
public int hashCode() {
    int result;
    result = (foreground != null ? foreground.hashCode() : 0);
    result = 29 * result + (fillForeground != null ? fillForeground.hashCode() : 0);
    result = 29 * result + fillRule;
    result = 29 * result + (lineWidth != null ? lineWidth.hashCode() : 0);
    result = 29 * result + lineType;
    result = 29 * result + (lineSpacing != null ? lineSpacing.hashCode() : 0);
    result = 29 * result + (dashLength != null ? dashLength.hashCode() : 0);
    return result;
}

/**
 * Compares this text style to the passed-in style.
 * Text styles with the same properties are considered equal regardless
 * of their name.
 * @param o
 * @return
 */
public boolean equals(Object o) {
    if (this == o) {
        return true;
    }
    if (!(o instanceof JCDrawStyle)) {
        return false;
    }

    final JCDrawStyle jcDrawStyle = (JCDrawStyle) o;

    if (fillRule != jcDrawStyle.fillRule) {
        return false;
    }
    if (lineType != jcDrawStyle.lineType) {
        return false;
    }
    if (dashLength != null ? !dashLength.equals(jcDrawStyle.dashLength) :
                             jcDrawStyle.dashLength != null) {
        return false;
    }
    if (fillForeground != null ? !fillForeground.equals(jcDrawStyle.fillForeground) :
                                 jcDrawStyle.fillForeground != null) {
        return false;
    }
    if (foreground != null ? !foreground.equals(jcDrawStyle.foreground) :
                             jcDrawStyle.foreground != null) {
        return false;
    }
    if (lineSpacing != null ? !lineSpacing.equals(jcDrawStyle.lineSpacing) :
                              jcDrawStyle.lineSpacing != null) {
        return false;
    }
    if (lineWidth != null ? !lineWidth.equals(jcDrawStyle.lineWidth) :
                             jcDrawStyle.lineWidth != null) {
        return false;
    }

    return true;
}

/**
 * Returns this object if it is already immutable; otherwise, sees if a style
 * with the same properties already exists in the immutable cache and returns
 * it.  If there is no cached style then we clone this style, make it
 * immutable, store it in the cache, and return it.
 * @return
 */
public JCDrawStyle getImmutable() {
	if (immutable == true) {
		
		return this;
	}

	JCDrawStyle style = (JCDrawStyle) immutableDrawStyles.get(this);
	if (style == null) {
		style = (JCDrawStyle) this.clone();
		
		// only immutable styles get stuffed in the cache
		style.makeImmutable();
		// add style to map using itself as a key
		immutableDrawStyles.put(style, style);
	}
	else {
		
	}
	return style;
}

}
