/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCFlow.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:22 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page;

import java.awt.Component;
import java.awt.Image;

import javax.swing.Icon;

import java.util.Enumeration;
import java.util.NoSuchElementException;


import com.klg.jclass.util.JCListenerList;

import com.klg.jclass.page.render.Render;
import com.klg.jclass.page.render.MacroRender;
import com.klg.jclass.page.render.TableMarker;






import java.awt.Graphics2D;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;


/**
 * JCFlow controls the flow of rendered content from
 * frame to frame, page to page, and section to section.
 */
public class JCFlow {

/** The document to which this flow belongs. */
protected JCDocument        document;

/** The page currently receiving the contents of the flow. */
protected JCPage            currentPage;
/** The active frame on the page currently receiving the contents of the flow. */
protected JCFrame           currentFrame;
/** User-controllable page numbering (not absolute number). */
protected int               currentPageNumber;
/** The number of the current section in the document. */
protected int               currentSectionNumber;
/** The number of the first page in the document -- used to start counting
    at numbers other than one. */
protected int               startingPageNumber;

/** The style applied to text being rendered. */
protected JCTextStyle       currentTextStyle;
/** The style applied to graphics being rendered. */
protected JCDrawStyle       currentDrawStyle;

/** An iterator over the list of flow frames on the current page. */
protected List              flowFrameList;
/** The index in the flow frame list of the next frame. */
protected int               nextFrameIndex;
/** The list of large objects which are to be floated into the flow. */
protected Vector            floatList;
/** A list of all items rendered to this flow, in order. */
// protected Vector            reflowList;

/** The number of pages in the document (running count). */
protected int               documentPageCount;
/** The number of pages in the current section (running count). */
protected int               sectionPageCount;
/** The total number of pages in the document (final count). */
protected int               documentTotalPages;
/** The total number of pages in the current section (final count). */
protected int               sectionTotalPages;

/** The listeners for flow events. */
protected JCListenerList	flowListeners = null;

/** Indicates if flow has ended. */
protected int           	flowStatus;
/** Indicates if current section has ended. */
protected boolean          	sectionComplete = false;

/** Count of user-added, non-flow pages that are waiting to be added
    into the full page count. */
protected int               deferredPageCount = 0;

/*
 * Flow status (completion levels) enumeration.
 */
public static final int		FLOW_INIT = 0;
public static final int		FLOW_ACTIVE = 1;
public static final int		FLOW_COMPLETE = 2;


/**
 * Creates a flow for the given document.
 * @param doc a document for which to initialize the flow
 */
public JCFlow(JCDocument doc) {

	this(doc, null, null);
}

/**
 * Creates a flow for the given document.
 * @param doc a document for which to initialize the flow
 * @param fl a <code>FlowListener</code> to notify of flow events
 */
public JCFlow(JCDocument doc, JCFlowListener fl) {

	this(doc, fl, null);
}

/**
 * Creates a flow for the document using the given page templates.
 * @param doc a document for which to initialize the flow
 * @param templates a list of page templates to use in the document
 */
public JCFlow(JCDocument doc, List templates) {

	this(doc, null, templates);
}

/**
 * Creates a flow for the document using the given page templates and
 * registers the given <code>FlowListener</code> for <code>JCFlow</code> events.
 * @param doc a document for which to initialize the flow
 * @param fl a <code>FlowListener</code> to notify of flow events
 * @param templates a list of page templates to use in the document
 * @see JCFlowEvent
 * @see JCFlowListener
 */
public JCFlow(JCDocument doc, JCFlowListener fl, List templates) {

	JCPage      first = null;

	floatList = new Vector();
    startingPageNumber = doc.getStartingPageNumber();
    currentPageNumber = startingPageNumber - 1;
	currentSectionNumber = 1;
	documentPageCount = 0;
	documentTotalPages = -1;
	sectionPageCount = 0;
	sectionTotalPages = -1;
	document = doc;
	flowStatus = FLOW_INIT;

	// if templates are provided, add them to the document
	if (templates != null) {
		doc.setTemplates(templates);
	}

	// If the document doesn't contain any page templates
	if (document.getTemplates().size() == 0) {
		// NOTE:  Create a default page template
		createDefaultTemplate(document);
	}

	// Get an initial page template from the document
	first = document.findFirstTemplate();

	// Get the default text and drawing styles
	currentTextStyle = JCTextStyle.stringToStyle("default text");
	currentDrawStyle = JCDrawStyle.stringToStyle("default line");

	// register the FlowListener so the first PageStart event can be received
	if (fl != null) {
		flowListeners = JCListenerList.add(flowListeners, fl);
	}

	

	// Record this flow at the document
	doc.setFlow(this);

	// Create the first page (starts the flow)
	startPage(first);

	// Loop until a valid frame can be initialised
	while (startFrame() == false) {
		startPage();
	}

	// set flow status to active
	flowStatus = FLOW_ACTIVE;
}

/**
 * Indicates if document flow has ended.
 * @return <code>true</code> if flow has ended
 */
public boolean isFlowComplete() {
	return(flowStatus == FLOW_COMPLETE);
}

/**
 * Indicates if document flow is active.
 * @return <code>true</code> if flow has ended
 */
public boolean isFlowActive() {
	return(flowStatus == FLOW_ACTIVE);
}

/**
 * Indicates if current section has ended.
 * @return <code>true</code> if flow has ended
 */
public boolean isSectionComplete() {
	return sectionComplete;
}

/**
 * Gets the total number of pages in the current section.
 * @return the total page numbers of the current section
 */
public int getSectionTotalPages() {
	return sectionTotalPages;
}

/**
 * Gets the page number of current section.
 * @return the current page number of the current section
 */
public int getSectionPageNumber() {
	return sectionPageCount;
}

/**
 * Gets the total number of pages in the document.
 * @return the total number to pages in the document
 */
public int getDocumentTotalPages() {
	return documentTotalPages;
}

/**
 * Gets the current page number.
 * @return the value to be used to number the current page
 */
public int getCurrentPageNumber() {
	return currentPageNumber;
}

/**
 * Assigns the current, user-defined page number (not the absolute page number).
 * @param number the number of the current page
 */
public void setCurrentPageNumber(int number) {

	currentPageNumber = number;

	// if there is a current page, update it
	if (currentPage != null) {
		currentPage.setPageNumber(number);

		// Check & update macros
		currentPage.updateLayoutStatus(this, currentPage);
	}
}

/**
 * Gets the current section number.
 * @return the number assigned to the current section
 */
public int getCurrentSectionNumber() {
	return currentSectionNumber;
}

/**
 * Assigns a number to the current section.
 * @param number the number of the current section
 */
public void setCurrentSectionNumber(int number) {

	currentSectionNumber = number;

	// if there is a current page, update it
	if (currentPage != null) {
		currentPage.setPageNumber(number);

		// Check & update macros
		currentPage.updateLayoutStatus(this, currentPage);
	}
}

/**
 * Gets the current page receiving printed data.
 * @return the page of the document currently being flowed to
 */
public JCPage getCurrentPage() {
	return currentPage;
}

/**
 * Gets the frame of the current page receiving data.
 * @return the frame to which the document is currently flowing
 */
public JCFrame getCurrentFrame() {
	return currentFrame;
}

/**
 * Gets the text style being applied to rendered text.
 * @return the current text style
 */
public JCTextStyle getCurrentTextStyle() {
	return currentTextStyle;
}

/**
 * Sets the text style in which text is to be rendered.
 * @param style a text style for formatting printed text
 */
public void setCurrentTextStyle(JCTextStyle style) {

	currentTextStyle = style;
}

/**
 * Gets the draw style being applied to rendered graphics.
 * @return the current draw style
 */
public JCDrawStyle getCurrentDrawStyle() {
	return currentDrawStyle;
}

/**
 * Sets the draw style in which graphical objects are to be rendered.
 * @param style a draw style for formatting printed graphics
 */
public void setCurrentDrawStyle(JCDrawStyle style) {

	currentDrawStyle = style;
}

/**
 * Gets a list of the template pages in the flow.
 * @return a name-keyed map of the page templates
 */
public List getTemplates() {
	return document.getTemplates();
}

/**
 * Prints a line of text to the flow at the current insertion point.
 * @param text String to flow to the document
 */
public void print(String text) {

	try {
		currentFrame.print(currentTextStyle, text);
	}
	catch (EndOfFrameException e) {
		nextFlowFrame(e.getCurrentLine(), e.isFirstLine(), e.getFloatList());
	}
}

/**
 * Prints a text macro.
 * @param macro the macro to add to the flow
 */
public void print(TextMacro macro) {

	Graphics2D  gc;
	MacroRender render;

	// Evaluate the macro in the current context
	macro.evaluate(this, currentPage);

	// Create a render object for this macro
	gc = (Graphics2D) document.getPrinterGraphics();
	render = new MacroRender(gc, macro, currentTextStyle);

	try {
		// Print the macro to the current frame
		currentFrame.print(currentTextStyle, render);
	}
	catch (EndOfFrameException e) {
		nextFlowFrame(e.getCurrentLine(), e.isFirstLine(), e.getFloatList());

		// if the macro didn't fit on the first page, re-evaluate it on the new
		// FUTURE:  Re-compute the position of the macro
		render.evaluateMacro(this, currentPage);
	}
}

/**
 * Prints a table to the flow (beginning at the insertion point).
 * @param table the table to paste into the flow
 */
public void print(JCPageTable table) {

	int                 last_row, last_column, current_row, current_column;
	Vector              savedFloatList;
	JCFlowInfo          info;

	// before adding table marker, go to new line
	// otherwise, HTML tables will appear above text rather than below it
	info = currentFrame.getFlowInfo();
    if (info.currentLine.size() != 0) {
        currentFrame.doLineBreak(info.currentTextStyle, false);
    }

	// Store a marker for the table at the current location in the flow
	TableMarker marker = new TableMarker(table, currentTextStyle);
	currentFrame.getRenderList().add(marker);

	// Store the size of the table and our start point
	last_row = table.getRowCount();
	last_column = table.getColumnCount();

	current_row = 0;
	current_column = 0;

	// NOTE: we disable the floating object list until after the table

	// Replace the frame's float list with an empty float list
	savedFloatList = info.floatList;
	info.floatList = floatList;

    // print whole table to the flow
    print(table, current_row, current_column, last_row, last_column);

	// NOTE:  We must re-add the floating objects to the new frame

	// Replace the empty float list with the old frame's float list
	info = currentFrame.getFlowInfo();
	info.floatList = savedFloatList;
}

/**
 * Begins the process of printing the specified sub-table to the flow
 * (beginning at the insertion point).
 * @param table the table to paste into the flow
 * @param last_row
 * @param last_column
 * @param current_row
 * @param current_column
 */
protected void print(JCPageTable table, int current_row, int current_column,
                                 			int last_row, int last_column) {
	print(table, current_row, current_column, last_row, last_column, false);
}

/**
 * Prints the specified sub-table to the flow (beginning at the insertion
 * point).
 * @param table the table to paste into the flow
 * @param newFlow <code>true</code> if a new flow frame has just begun; <code>false</code> if in the
 *               current flow frame
 * @param last_row
 * @param last_column
 * @param current_row
 * @param current_column
 */
protected void print(JCPageTable table, int current_row, int current_column,
                           	int last_row, int last_column, boolean newFlow) {

    boolean             done = false;

    // Loop, printing sections of the table until it is completed
    // if number of rows is zero, still perform the loop as
    // empty tables are handled within the frame code to ensure
    // headers are drawn if present
    while (!done && (current_row < last_row || last_row == 0)
	   && current_column < last_column) {

	try {
	    // if overflow method specified new flow frame for each new section
	    // of columns, make sure new sections of columns (other than first
	    // section) gets put in a new flow frame.
	    if (table.getOverflowMode() ==
		JCPageTable.OVERFLOW_WRAP_COLUMNS_NEXT &&
		!newFlow && current_column > 0 && current_row == 0) {
		newColumn();
	    }

	    currentFrame.print(table, current_row, current_column,
			       last_row - 1, last_column - 1);
	    done = true;
	}
	catch (EndOfFrameException e) {
	    JCPageTable tmp_table = e.getTable();

            nextFlowFrame(e.getCurrentLine(), e.isFirstLine(),
			  e.getFloatList());
	    // note that we have entered a new flow frame
	    newFlow = true;
            // if we had an out-of-space error printing the table, resume
            if (table == tmp_table) {
                current_row = e.getTableStartRow();
                current_column = e.getTableStartColumn();
                int sub_last_column = e.getTableEndColumn();
                //still some remaining rows in this part
                if (current_row < last_row && sub_last_column != -1 &&
		    sub_last_column < last_column) {
                    print(table, current_row, current_column,
			  last_row, sub_last_column + 1, true);
		    // successful print, reset newFlow to indicate we're
		    // still on the same page
		    newFlow = false;
                    current_row = 0;
                    current_column = sub_last_column + 1;
                }
            }
            else {
                current_row = last_row;
                current_column = last_column;
            }
	    // fix for PR19023
	    e = null;
		}
    }
}


/**
 * Moves to the next tab position on the current line.
 */
public void tab() {

	currentFrame.tab(currentTextStyle);
}

// Image float routines add images into the flow as independent objects
// (each floated object will appear on its own line)

/**
 * Adds an image to the document so it is added in the next space it fits,
 * subject to the existence of other floating objects that may precede it.
 * This form of the <code>floatImage()</code> method assumes a default size based on
 * the pixel size of the image, and takes the horizontal alignment from the
 * current text style.
 * @param image the Image to be floated in the document
 */
public void floatImage(Image image) {

	// Float the image in the current frame
	currentFrame.floatImage(image);
}

/**
 * Adds an image to the document so it is added in the next space it fits,
 * subject to the existence of other floating objects that may precede it.
 * This form of the <code>floatImage()</code> method assumes a default size based on
 * the pixel size of the image.
 * @param image the Image to be floated in the document
 * @param alignment the horizontal alignment of the image in the frame
 */
public void floatImage(Image image, int alignment) {

	// Float the image in the current frame
	currentFrame.floatImage(image, alignment);
}

/**
 * Adds an image to the document so it is added in the next space it fits,
 * subject to the existence of other floating objects that may precede it.
 * This form of the <code>floatImage()</code> method uses the current text style to
 * specify the horizontal aligment of the image in the frame
 * @param image the Image to be floated in the document
 * @param size the dimensions of the image as it is to appear on the page
 */
public void floatImage(Image image, JCUnit.Dimension size) {

	// Float the image in the current frame
	currentFrame.floatImage(image, size);
}

/**
 * Adds an image to the document so it is added in the next space it fits,
 * subject to the existence of other floating objects that may precede it.
 * @param image the Image to be floated in the document
 * @param alignment the horizontal alignment of the image in the frame
 * @param size the dimensions of the image as it is to appear on the page
 */
public void floatImage(Image image, int alignment, JCUnit.Dimension size) {

	// Float the image in the current frame
	currentFrame.floatImage(image, alignment, size);
}

/**
 * Adds an icon to the document so it is added in the next space it fits,
 * subject to the existence of other floating objects that may precede it.
 * This form of the <code>floatIcon()</code> method assumes a default size based on
 * the pixel size of the icon, and takes the horizontal alignment from the
 * current text style.
 * <br>This method only accepts instances of the <code>ImageIcon</code> class.
 * @param icon the Icon to be floated in the document
 */
public void floatIcon(Icon icon) {

	// Float the icon in the document at the current frame
	currentFrame.floatIcon(icon);
}

/**
 * Adds an icon to the document so it is added in the next space it fits,
 * subject to the existence of other floating objects that may precede it.
 * This form of the <code>floatIcon()</code> method assumes a default size based on
 * the pixel size of the icon.
 * <br>This method only accepts instances of the <code>ImageIcon</code> class.
 * @param icon the Icon to be floated in the document
 * @param alignment the horizontal alignment of the icon in the frame
 */
public void floatIcon(Icon icon, int alignment) {

	// Float the icon in the document at the current frame
	currentFrame.floatIcon(icon, alignment);
}

/**
 * Adds an icon to the document so it is added in the next space it fits,
 * subject to the existence of other floating objects that may precede it.
 * This form of the <code>floatIcon()</code> method uses the current text style to
 * specify the horizontal aligment of the icon in the frame.
 * <br>This method only accepts instances of the <code>ImageIcon</code> class.
 * @param icon the Icon to be floated in the document
 * @param size the dimensions of the icon as it is to appear on the page
 */
public void floatIcon(Icon icon, JCUnit.Dimension size) {

	// Float the icon in the document at the current frame
	currentFrame.floatIcon(icon, size);
}


/**
 * Adds an icon to the document so it is added in the next space it fits,
 * subject to the existence of other floating objects that may precede it.
 * <br>This method only accepts instances of the <code>ImageIcon</code> class.
 * @param icon the Icon to be floated in the document
 * @param alignment the horizontal alignment of the icon in the frame
 * @param size the dimensions of the icon as it is to appear on the page
 */
public void floatIcon(Icon icon, int alignment, JCUnit.Dimension size) {

	// Float the icon in the document at the current frame
	currentFrame.floatIcon(icon, alignment, size);
}

/**
 * Adds a component's drawn image to the document so it is added in the next
 * space it fits (subject to the existence of other floating objects that may
 * precede this).  This form of the <code>floatComponent()</code> method assumes a default
 * size based on the pixel size of the component from its current geometry,
 * and takes the horizontal alignment from the current text style.
 * @param c the Component to be floated in the document
 */
public void floatComponent(Component c) {

	// Float the visual of the component in the current frame
	currentFrame.floatComponent(c);
}

/**
 * Adds a component's drawn image to the document so it is added in the next
 * space it fits (subject to the existence of other floating objects that may
 * precede this).  This form of the <code>floatComponent()</code> method assumes a default
 * size based on the pixel size of the component from its current geometry.
 * @param c the Component to be floated in the document
 * @param alignment the horizontal alignment of the component in the frame
 */
public void floatComponent(Component c, int alignment) {

	// Float the visual of the component in the current frame
	currentFrame.floatComponent(c, alignment);
}

/**
 * Adds a component's drawn image to the document so it is added in the next
 * space it fits, subject to the existence of other floating objects that
 * may precede it. This form of the <code>floatComponent()</code> method takes the
 * horizontal alignment from the current text style.
 * @param c the Component to be floated in the document
 * @param size the dimensions of the component as it is to appear on the page
 */
public void floatComponent(Component c, JCUnit.Dimension size) {

	// Float the visual of the component in the current frame
	currentFrame.floatComponent(c, size);
}

/**
 * Adds a component's drawn image to the document so it is added in the next
 * space it fits, subject to the existence of other floating objects that
 * may precede it.
 * @param c the Component to be floated in the document
 * @param alignment the horizontal alignment of the component in the frame
 * @param size the dimensions of the component as it is to appear on the page
 */
public void floatComponent(Component c, int alignment, JCUnit.Dimension size) {

	// Float the visual of the component in the current frame
	currentFrame.floatComponent(c, alignment, size);
}

/**
 * Adds an Encapsulated PostScript (EPS) image to the document so it is added
 * in the next space it fits, subject to the existence of other floating
 * objects that may precede it. This form of the <code>floatEPS()</code> method takes
 * the horizontal alignment from the current text style and assumes the
 * <code>EPSImage</code> object has a size set.
 * @param eps the EPS image to be floated in the document
 */
public void floatEPS(EPSImage eps) {

	// Float the EPS image in the current frame
	currentFrame.floatEPS(eps);
}

/**
 * Adds an Encapsulated PostScript (EPS) image to the document so it is added
 * in the next space it fits, subject to the existence of other floating
 * objects that may precede it. This form of the <code>floatEPS()</code> method assumes
 * the <code>EPSImage</code> object has a size set.
 * @param eps the EPS image to be floated in the document
 * @param alignment the horizontal alignment of the image in the frame
 */
public void floatEPS(EPSImage eps, int alignment) {

	// Float the EPS image in the current frame
	currentFrame.floatEPS(eps, alignment);
}

/**
 * Adds an Encapsulated PostScript (EPS) image to the document so it is added
 * in the next space it fits, subject to the existence of other floating
 * objects that may precede it.  This form of the <code>floatEPS()</code> method takes
 * the horizontal alignment from the current text style.
 * @param eps the Image to be floated in the document
 * @param size the dimensions of the image as it is to appear on the page
 */
public void floatEPS(EPSImage eps, JCUnit.Dimension size) {

	// Float the EPS image in the current frame
	currentFrame.floatEPS(eps, size);
}

/**
 * Adds an Encapsulated PostScript (EPS) image to the document so it is added
 * in the next space it fits, subject to the existence of other floating
 * objects that may precede it.
 * @param eps the EPS image to be floated in the document
 * @param alignment the horizontal alignment of the image in the frame
 * @param size the dimensions of the image as it is to appear on the page
 */
public void floatEPS(EPSImage eps, int alignment, JCUnit.Dimension size) {

	// Float the EPS image in the current frame
	currentFrame.floatEPS(eps, alignment, size);
}

// Image embed functions add the image to the objects composing the current
// line.  Embedding works best for importing small images onto lines of text.

/**
 * Adds an image to the document as part of the current line. The image may
 * wrap if there isn't enough horizontal space.  The height of the current
 * line will be adjusted to accomodate the image.  This form of the
 * <code>embedImage()</code> method assumes a default size based on the pixel size of the
 * image, and centers the image vertically on the current line.
 * @param image the Image to be embedded in the current line
 */
public void embedImage(Image image) {

	// Embed the image in the current frame (add to current line)
	try {
		currentFrame.embedImage(currentTextStyle, image);
	}
	catch (EndOfFrameException e) {
		nextFlowFrame(e.getCurrentLine(), e.isFirstLine(), e.getFloatList());
	}
}

/**
 * Adds an image to the document as part of the current line. The image may
 * wrap if there isn't enough horizontal space.  The height of the current
 * line will be adjusted to accomodate the image.  This form of the
 * <code>embedImage()</code> method assumes a default size based on the pixel size of the
 * image.
 * @param image the Image to be embedded in the current line
 * @param alignment the vertical alignment of the image on the current line
 */
public void embedImage(Image image, int alignment) {

	// Embed the image in the current frame (add to current line)
	try {
		currentFrame.embedImage(currentTextStyle, image, alignment);
	}
	catch (EndOfFrameException e) {
		nextFlowFrame(e.getCurrentLine(), e.isFirstLine(), e.getFloatList());
	}
}

/**
 * Adds an image to the document as part of the current line. The image may
 * wrap if there isn't enough horizontal space.  The height of the current
 * line will be adjusted to accomodate the image.  This form of the
 * <code>embedImage()</code> method centers the image vertically on the current line.
 * @param image the Image to be embedded in the current line
 * @param size the dimensions of the image as it is to appear on the page
 */
public void embedImage(Image image, JCUnit.Dimension size) {

	// Embed the image in the current frame (add to current line)
	try {
		currentFrame.embedImage(currentTextStyle, image, size);
	}
	catch (EndOfFrameException e) {
		nextFlowFrame(e.getCurrentLine(), e.isFirstLine(), e.getFloatList());
	}
}

/**
 * Adds an image to the document as part of the current line. The image may
 * wrap if there isn't enough horizontal space.  The height of the current
 * line will be adjusted to accomodate the image.
 * @param image the Image to be embedded in the current line
 * @param alignment the vertical alignment of the image on the current line
 * @param size the dimensions of the image as it is to appear on the page
 */
public void embedImage(Image image, int alignment, JCUnit.Dimension size) {

	// Embed the image in the current frame (add to current line)
	try {
		currentFrame.embedImage(currentTextStyle, image, alignment, size);
	}
	catch (EndOfFrameException e) {
		nextFlowFrame(e.getCurrentLine(), e.isFirstLine(), e.getFloatList());
	}
}

/**
 * Adds an icon to the document as part of the current line. The image may
 * wrap if there isn't enough horizontal space.  The height of the current
 * line will be adjusted to accomodate the icon.  This form of the
 * <code>embedIcon()</code> method assumes a default size based on the pixel size of the
 * icon, and centers the icon vertically on the current line.
 * <br>This method only accepts instances of the <code>ImageIcon</code> class.
 * @param icon the Icon to be embedded in the current line
 */
public void embedIcon(Icon icon) {

	// Embed the icon at the current insertion point
	try {
		currentFrame.embedIcon(currentTextStyle, icon);
	}
	catch (EndOfFrameException e) {
		nextFlowFrame(e.getCurrentLine(), e.isFirstLine(), e.getFloatList());
	}
}

/**
 * Adds an icon to the document as part of the current line. The image may
 * wrap if there isn't enough horizontal space.  The height of the current
 * line will be adjusted to accomodate the icon.  This form of the
 * <code>embedIcon()</code> method assumes a default size based on the pixel size of the
 * icon.
 * <br>This method only accepts instances of the <code>ImageIcon</code> class.
 * @param icon the Icon to be embedded in the current line
 * @param alignment the vertical alignment of the icon on the current line
 */
public void embedIcon(Icon icon, int alignment) {

	// Embed the icon at the current insertion point
	try {
		currentFrame.embedIcon(currentTextStyle, icon, alignment);
	}
	catch (EndOfFrameException e) {
		nextFlowFrame(e.getCurrentLine(), e.isFirstLine(), e.getFloatList());
	}
}

/**
 * Adds an icon to the document as part of the current line. The image may
 * wrap if there isn't enough horizontal space.  The height of the current
 * line will be adjusted to accomodate the icon.  This form of the
 * <code>embedIcon()</code> method centers the icon vertically on the current line.
 * <br>This method only accepts instances of the <code>ImageIcon</code> class.
 * @param icon the Icon to be embedded in the current line
 * @param size the dimensions of the icon as it is to appear on the page
 */
public void embedIcon(Icon icon, JCUnit.Dimension size) {

	// Embed the icon at the current insertion point
	try {
		currentFrame.embedIcon(currentTextStyle, icon, size);
	}
	catch (EndOfFrameException e) {
		nextFlowFrame(e.getCurrentLine(), e.isFirstLine(), e.getFloatList());
	}
}

/**
 * Adds an icon to the document as part of the current line. The image may
 * wrap if there isn't enough horizontal space.  The height of the current
 * line will be adjusted to accomodate the icon.
 * <br>This method only accepts instances of the <code>ImageIcon</code> class.
 * @param icon the Icon to be embedded in the current line
 * @param alignment the vertical alignment of the icon on the current line
 * @param size the dimensions of the icon as it is to appear on the page
 */
public void embedIcon(Icon icon, int alignment, JCUnit.Dimension size) {

	// Embed the icon at the current insertion point
	try {
		currentFrame.embedIcon(currentTextStyle, icon, alignment, size);
	}
	catch (EndOfFrameException e) {
		nextFlowFrame(e.getCurrentLine(), e.isFirstLine(), e.getFloatList());
	}
}

/**
 * Adds a component's drawn image to the document as part of the current line.
 * The component image may wrap if there isn't enough horizontal space.
 * The height of the current line will be adjusted to accomodate the component
 * drawing.  This form of the <code>embedComponent()</code> method assumes a default size
 * based on the pixel size of the component from its current geometry, and
 * centers the component vertically on the current line.
 * @param c the Component to be embedded in the current line
 */
public void embedComponent(Component c) {

	// Embed the drawn component in the current frame (add to current line)
	try {
		currentFrame.embedComponent(currentTextStyle, c);
	}
	catch (EndOfFrameException e) {
		nextFlowFrame(e.getCurrentLine(), e.isFirstLine(), e.getFloatList());
	}
}

/**
 * Adds a component's drawn image to the document as part of the current line.
 * The component image may wrap if there isn't enough horizontal space.
 * The height of the current line will be adjusted to accomodate the component
 * drawing.  This form of the <code>embedComponent()</code> method assumes a default size
 * based on the pixel size of the component from its current geometry.
 * @param c the Component to be embedded in the current line
 * @param alignment the vertical alignment of the component on the current line
 */
public void embedComponent(Component c, int alignment) {

	// Embed the drawn component in the current frame (add to current line)
	try {
		currentFrame.embedComponent(currentTextStyle, c, alignment);
	}
	catch (EndOfFrameException e) {
		nextFlowFrame(e.getCurrentLine(), e.isFirstLine(), e.getFloatList());
	}
}

/**
 * Adds a component's drawn image to the document as part of the current line.
 * The component image may wrap if there isn't enough horizontal space.
 * The height of the current line will be adjusted to accomodate the component
 * drawing.  This form of the <code>embedComponent()</code> method centers the component
 * vertically on the current line.
 * @param c the Component to be embedded in the current line
 * @param size the dimensions of the component as it is to appear on the page
 */
public void embedComponent(Component c, JCUnit.Dimension size) {

	// Embed the drawn component in the current frame (add to current line)
	try {
		currentFrame.embedComponent(currentTextStyle, c, size);
	}
	catch (EndOfFrameException e) {
		nextFlowFrame(e.getCurrentLine(), e.isFirstLine(), e.getFloatList());
	}
}


/**
 * Adds a component's drawn image to the document as part of the current line.
 * The component image may wrap if there isn't enough horizontal space.
 * The height of the current line will be adjusted to accomodate the component
 * drawing.
 * @param c the Component to be embedded in the current line
 * @param alignment the vertical alignment of the component on the current line
 * @param size the dimensions of the component as it is to appear on the page
 */
public void embedComponent(Component c, int alignment, JCUnit.Dimension size) {

	// Embed the drawn component in the current frame (add to current line)
	try {
		currentFrame.embedComponent(currentTextStyle, c, alignment, size);
	}
	catch (EndOfFrameException e) {
		nextFlowFrame(e.getCurrentLine(), e.isFirstLine(), e.getFloatList());
	}
}


/**
 * Adds an Encapsulated PostScript image to the document as part of the current
 * line. The EPS image may wrap if there isn't enough horizontal space.  The
 * height of the current line will be adjusted to accomodate the image.  This
 * form of the <code>embedEPS()</code> method centers the eps image vertically on the
 * current line and assumes a size has been provided to the eps image object.
 * @param eps the EPS image to be embedded in the current line
 */
public void embedEPS(EPSImage eps) {

	// Embed the EPS image in the current frame (add to current line)
	try {
		currentFrame.embedEPS(currentTextStyle, eps);
	}
	catch (EndOfFrameException e) {
		nextFlowFrame(e.getCurrentLine(), e.isFirstLine(), e.getFloatList());
	}
}


/**
 * Adds an Encapsulated PostScript image to the document as part of the current
 * line. The EPS image may wrap if there isn't enough horizontal space.  The
 * height of the current line will be adjusted to accomodate the image.  This
 * form of the <code>embedEPS()</code> method assumes a size has been provided to the eps
 * image object.
 * @param eps the EPS image to be embedded in the current line
 * @param alignment the vertical alignment of the image on the current line
 */
public void embedEPS(EPSImage eps, int alignment) {

	// Embed the EPS image in the current frame (add to current line)
	try {
		currentFrame.embedEPS(currentTextStyle, eps, alignment);
	}
	catch (EndOfFrameException e) {
		nextFlowFrame(e.getCurrentLine(), e.isFirstLine(), e.getFloatList());
	}
}

/**
 * Adds an Encapsulated PostScript image to the document as part of the current
 * line. The EPS image may wrap if there isn't enough horizontal space.  The
 * height of the current line will be adjusted to accomodate the image.  This
 * form of the <code>embedEPS()</code> method centers the eps image vertically on the
 * current line.
 * @param eps the Image to be embedded in the current line
 * @param size the dimensions of the image as it is to appear on the page
 */
public void embedEPS(EPSImage eps, JCUnit.Dimension size) {

	// Embed the EPS image in the current frame (add to current line)
	try {
		currentFrame.embedEPS(currentTextStyle, eps, size);
	}
	catch (EndOfFrameException e) {
		nextFlowFrame(e.getCurrentLine(), e.isFirstLine(), e.getFloatList());
	}
}

/**
 * Adds an Encapsulated PostScript image to the document as part of the current
 * line. The EPS image may wrap if there isn't enough horizontal space.  The
 * height of the current line will be adjusted to accomodate the image.
 * @param eps the EPS image to be embedded in the current line
 * @param alignment the vertical alignment of the image on the current line
 * @param size the dimensions of the image as it is to appear on the page
 */
public void embedEPS(EPSImage eps, int alignment, JCUnit.Dimension size) {

	// Embed the EPS image in the current frame (add to current line)
	try {
		currentFrame.embedEPS(currentTextStyle, eps, alignment, size);
	}
	catch (EndOfFrameException e) {
		nextFlowFrame(e.getCurrentLine(), e.isFirstLine(), e.getFloatList());
	}
}

/**
 * Draws a horizontal line across the column/frame below the current line.
 * @param percent_width the percentage of the width of the column that the line
 *     covers
 */
public void hRule(double percent_width) {

	boolean     done = false;

	// if the hRule causes a frame flow, try again.
	while (!done) {
		try {
			currentFrame.hRule(currentTextStyle, currentDrawStyle,
							   percent_width);
			done = true;
		}
		catch (EndOfFrameException e) {
			nextFlowFrame(e.getCurrentLine(), e.isFirstLine(),
						  e.getFloatList());
		}
	}
}

/**
 * Draws a horizontal line across the column/frame below the current line.
 * @param rule_length the length of the line
 */
public void hRule(JCUnit.Measure rule_length) {

	boolean     done = false;

	// if the hRule causes a frame flow, try again.
	while (!done) {
		try {
			currentFrame.hRule(currentTextStyle, currentDrawStyle, rule_length);
			done = true;
		}
		catch (EndOfFrameException e) {
			nextFlowFrame(e.getCurrentLine(), e.isFirstLine(),
						  e.getFloatList());
		}
	}
}

/**
 * Advances the text flow to a new line.
 */
public void newLine() {

	currentFrame.newLine(currentTextStyle);
}

/**
 * Advances the text flow to a new line, starting a new paragraph.
 */
public void newParagraph() {

	currentFrame.newParagraph(currentTextStyle);
}

/**
 * Advances the text flow to a new column.
 */
public void newColumn() {

	try {
		// New column, as public, will correctly paste the current line
		currentFrame.newColumn(currentTextStyle);
	}
	catch (EndOfFrameException e) {
		nextFlowFrame(e.getCurrentLine(), e.isFirstLine(), e.getFloatList());
	}
}

/**
 * Advances the text flow to a new frame.
 */
public void newFrame() {

	int         i, columns;
	JCFlowInfo  pending;

	// Use newLine to paste any current text in the old frame
	currentFrame.doLineBreak(currentTextStyle, true);

	// Try to float objects into this frame if there are more columns
	columns = currentFrame.getColumnCount() - currentFrame.getCurrentColumn();
	for (i = 1; i < columns; i++) {
		// Use flow's version to avoid having to catch exceptions
		newColumn();
	}

	// get the info structure from this frame in case there are floats to place
	pending = currentFrame.getFlowInfo();

	// Get next frame, passing it the current flow information
	advanceFrame(pending.firstLine, pending.floatList);
}

/**
 * Advances the text flow to a new page.
 */
public void newPage() {

	JCPage          templatePage;

	endSection();

	templatePage = currentPage.getFlowPageTemplate();
	newFlowedPage(templatePage);
}

/**
 * Advances the text flow to a new Section.
 */
public void newSection() {

	/*
	// At this point we can go back through the pages of this section and
	// update any macros which require the count of pages in the section
	*/

	// Paste the current line and any floating objects in the previous section
	endSection();

	// update section count, section page total, and reset section page count
	currentSectionNumber++;
	sectionTotalPages = sectionPageCount;
	sectionPageCount = 0;

	// add first page in new section to document; section page count is incremented
	JCPage templatePage = currentPage.getFlowSectionTemplate();
	newFlowedPage(templatePage);

	sectionComplete = true;

	// resolve any section page counts by updating macros of all pages in the section
	List page_list = document.getPages();
	for (int i = documentPageCount - sectionTotalPages - 1; i < documentPageCount - 1; i++) {
		JCPage page = (JCPage) page_list.get(i);
		page.updateLayoutStatus(this, page);
	}

	// reset the count of the number of pages in the section
	sectionTotalPages = -1;
	sectionComplete = false;
}

/**
 * Completes the document so it can be printed.
 */
public void endFlow() {

	int             i, columns;
	JCFlowInfo      pending;

	

	// Use newLine to paste any current text in the current frame
	newLine();

	// Try to paste floating objects into this frame if there are more columns
	columns = currentFrame.getColumnCount() - currentFrame.getCurrentColumn();
	for (i = 1; i < columns; i++) {
		// Use flow's version to avoid having to catch exceptions
		newColumn();
	}

	// Empty the float list before returning at the new page
	pending = currentFrame.getFlowInfo();
	while (pending.getFloatList().size() > 0) {

		// NOTE:  We need to trap infinite page generation errors
		advanceFrame(pending.firstLine, pending.floatList);

		// Go through the columns of the frame so we can float to all
		for (i = 1; i < currentFrame.getColumnCount(); i++) {
			// Use flow's version to avoid having to catch exceptions
			newColumn();
		}
		pending = currentFrame.getFlowInfo();
	}

	// End the last frame we've used and its page
	endFrame();
	endPage(null);

	// section is completed, update total number of pages in current section
	sectionTotalPages = sectionPageCount;
	sectionComplete = true;

	// flow is completed, update total number of pages in document
	flowStatus = FLOW_COMPLETE;
	documentTotalPages = documentPageCount + startingPageNumber - 1;

	// finish all unfinished macros for every page of the document
	Iterator page_iterator = document.getPages().iterator();
	while (page_iterator.hasNext()) {
		JCPage page = (JCPage) page_iterator.next();
		page.updateLayoutStatus(this, page);
	}
}


/**
 * Adds a listener for flow events.
 * @see JCFlowEvent
 * @see JCFlowListener
 * @param listener
 */
public void addFlowListener(JCFlowListener listener) {
	flowListeners = JCListenerList.add(flowListeners, listener);
}


/**
 * Removes a listener for flow events.
 * @param listener
 */
public void removeFlowListener(JCFlowListener listener) {
	flowListeners = JCListenerList.remove(flowListeners, listener);
}


/**
 * Finishes the current frame.
 */
private void endSection() {

	int             i, columns;
	JCFlowInfo      pending;

	// Use newLine to paste any current text in the old frame
	newLine();

	// Try to paste floating objects into this frame if there are more columns
	columns = currentFrame.getColumnCount() - currentFrame.getCurrentColumn();
	for (i = 1; i < columns; i++) {
		// Use flow's version to avoid having to catch exceptions
		newColumn();
	}

	// Empty the float list before returning at the new page
	pending = currentFrame.getFlowInfo();
	while (pending.getFloatList().size() > 0) {

		// NOTE:  We need to trap infinite page generation errors
		advanceFrame(pending.firstLine, pending.floatList);

		// Go through the columns of the frame so we can float to all
		for (i = 1; i < currentFrame.getColumnCount(); i++) {
			// Use flow's version to avoid having to catch exceptions
			newColumn();
		}
		pending = currentFrame.getFlowInfo();
	}

	// Prepare to go to a new page after the last frame we've written to
	endFrame();
}

/**
 * Adds a new page to the flow after flushing the current data and floats.
 * @param templatePage the page template to copy
 */
private void newFlowedPage(JCPage templatePage) {

	// Create the selected new page
	startPage(templatePage);

	// Get a flow frame on the new page, otherwise get another new page
	while (startFrame() == false) {
		startPage();
	}
}

/**
 * Moves to the next frame in the flow, processing floating objects and carrying
 * forward all uncompleted text.
 * @param currentLine
 * @param first
 * @param floatList
 */
private void nextFlowFrame(List currentLine, boolean first, List floatList) {

	boolean     done = false;

	

	while (!done) {

		// Get the next frame of the document
		advanceFrame(first, floatList);

		// If there are elements on the current line, paste them
		if (currentLine != null) {

			// Print current and remaining text to the current frame
			try {
				// This needs to be atomic so text isn't lost if the first fails
				currentFrame.setCurrentLine(currentLine);
				done = true;
			}
			catch (EndOfFrameException e) {
				first = e.isFirstLine();
				floatList = e.getFloatList();
				currentLine = e.getCurrentLine();
			}
		}
		else {
			done = true;
		}
	}
}

/**
 * Ends the current frame and finds the next one in the flow.
 */
private void advanceFrame() {

	int             i, columns;

	endFrame();
	while (startFrame() == false) {
		startPage();
	}
}

/**
 * Ends the current frame and finds the next one in the flow.
 * @param first <code>True</code> if the line is the first one of a paragraph
 * @param floatList a List of objects to be floated into the document
 */
private void advanceFrame(boolean first, List floatList) {

	int             i, columns;

	endFrame();
	while (startFrame(first, floatList) == false) {
		startPage();
	}
}

/**
 * Computes the next frame on the page.
 * @return the next frame in the page's flow frame list
 */
private JCFrame nextFrame() {

	if (nextFrameIndex == flowFrameList.size()) {
		return(null);
	}
	return((JCFrame) flowFrameList.get(nextFrameIndex));
}

/**
 * Moves to the next frame in the frame list.
 */
private void incrementFrame() {
	nextFrameIndex++;
}

/**
 * Gets the next frame on the page.
 * @param first <code>True</code> if the line is the first one of a paragraph
 * @param floatList a List of objects to be floated into the document
 * @return
 */
private boolean startFrame(boolean first, List floatList) {

	


	currentFrame = nextFrame();
	if (currentFrame == null) {
		return false;
	}
	incrementFrame();

	

	// Give the new frame a style so it can do linefeeds
	currentFrame.getFlowInfo().currentTextStyle = currentTextStyle;

	// initialise the new frame's column and line extents and insertion point
	currentFrame.initialiseInfo(first);

	JCFlowEvent event = new JCFlowEvent(this, JCFlowEvent.FRAME_BEGIN,
										currentFrame);
	Enumeration e = JCListenerList.elements(flowListeners);

	for (; e.hasMoreElements(); ) {
		JCFlowListener listener = (JCFlowListener) e.nextElement();
		listener.frameBegin(event);
	}

	// paste floating images
	currentFrame.addFloats(floatList);

	return true;
}

/**
 * Gets the next frame on the page.
 * @return
 */
private boolean startFrame() {

	

	currentFrame = nextFrame();
	if (currentFrame == null) {

	return false;
	}
	incrementFrame();


	

	// Give the new frame a style so it can do linefeeds
	currentFrame.getFlowInfo().currentTextStyle = currentTextStyle;

	// initialise the new frame's column and line extents and insertion point
	currentFrame.initialiseInfo();

	JCFlowEvent event = new JCFlowEvent(this, JCFlowEvent.FRAME_BEGIN,
										currentFrame);
	Enumeration e = JCListenerList.elements(flowListeners);
	for (; e.hasMoreElements(); ) {
		JCFlowListener listener = (JCFlowListener) e.nextElement();
		listener.frameBegin(event);
	}

	return true;
}



/**
 * Finishes the current frame.
 */
private void endFrame() {


	

	JCFlowEvent event = new JCFlowEvent(this, JCFlowEvent.FRAME_END,
										currentFrame, nextFrame());

	Enumeration e = JCListenerList.elements(flowListeners);
	for (; e.hasMoreElements(); ) {
		JCFlowListener listener = (JCFlowListener) e.nextElement();
		listener.frameEnd(event);
	}

	// check for completeness
	currentFrame.complete();

}

/**
 * Convenience method to fire the <code>FRAME_COMPLETE event</code>.
 * @param frame instance of the frame completed
 */
protected void fireFrameCompleted(JCFrame frame) {
	JCFlowEvent event = new JCFlowEvent(this, JCFlowEvent.FRAME_COMPLETE,
		currentFrame);
	Enumeration e = JCListenerList.elements(flowListeners);
	for (; e.hasMoreElements(); ) {
		JCFlowListener listener = (JCFlowListener) e.nextElement();
		listener.frameComplete(event);
	}
}



/**
 * Gets the next page from the given template.
 * @param templatePage page template to copy for the new page
 */
private void startPage(JCPage templatePage) {

	// complete the previous (current) page
	endPage(templatePage);

	currentPage = new JCPage(Integer.toString(currentPageNumber + 1), document,
							 templatePage);
	// Page numbers are incremented by the updatePageNumbers() method, which
	// is called by document.addPage().  It also sets the page number on the
	// current page.
	document.addPage(currentPage, true, true);

	
	

	// Update any macros on the new page
	
	currentPage.updateLayoutStatus(this, currentPage);
	

	// Get the flow frame list of the new page
	flowFrameList = currentPage.getFlowFrameList();
	nextFrameIndex = 0;

	JCFlowEvent event = new JCFlowEvent(this, JCFlowEvent.PAGE_BEGIN,
										currentPage);
	Enumeration e = JCListenerList.elements(flowListeners);
	for (; e.hasMoreElements(); ) {
		JCFlowListener listener = (JCFlowListener) e.nextElement();
		listener.pageBegin(event);
	}

	// regenerate flowFrameList in case the user modified it
	flowFrameList = currentPage.getFlowFrameList();
}

/**
 * Gets the next page from the default next page template.
 */
private void startPage() {

	JCPage      templatePage = currentPage.getFlowPageTemplate();

	// complete the previous (current) page
	endPage(templatePage);

	currentPage = new JCPage(Integer.toString(currentPageNumber + 1), document,
							 templatePage);
	// Page numbers are incremented by the updatePageNumbers() method, which
	// is called by document.addPage().  It also sets the page number on the
	// current page.
	document.addPage(currentPage, true, true);

	
	

	// Update any macros on the new page
	currentPage.updateLayoutStatus(this, currentPage);

	// Get the flow frame list of the new page
	flowFrameList = currentPage.getFlowFrameList();
	nextFrameIndex = 0;

	JCFlowEvent event = new JCFlowEvent(this, JCFlowEvent.PAGE_BEGIN,
										currentPage);
	Enumeration e = JCListenerList.elements(flowListeners);
	for (; e.hasMoreElements(); ) {
		JCFlowListener listener = (JCFlowListener) e.nextElement();
		listener.pageBegin(event);
	}

	// regenerate flowFrameList in case the user modified it
	flowFrameList = currentPage.getFlowFrameList();
}

/**
 * Updates the page numbers being tracked by the flow, including the
 * current page number and the page counts for the section and the
 * document.  Also sets the page number on the new page (which
 * should be the newly added or created current page) if provided.
 * This method is currently called by <code>JCDocument.addPage()</code> so that
 * all added pages affect the page number counts.  It is also called
 * by <code>JCFlow.endPage()</code> in the case that any user added pages have
 * been deferred.
 * @param newPage the newly created added page (may be null if method is
 *          called from a method that does not create a page but only
 *          wishes to add in deferred pages to the page count)
 */
protected void updatePageNumbers(JCPage newPage)
{
    int newPageCount = 0;

    // if new page exists, make sure it gets added in
    if (newPage != null) {
        newPageCount = 1;
    }

    // make sure any deferred pages get added into the count
    newPageCount += deferredPageCount;
    deferredPageCount = 0;

    // update page counts
    currentPageNumber += newPageCount;
    sectionPageCount += newPageCount;
    documentPageCount += newPageCount;

    // make new page with proper page number
    if (newPage != null) {
        newPage.setPageNumber(documentPageCount);
    }
}

/**
 * Tracks the number of pages added but not yet part of the page count.  This is
 * used specifically to track pages added by the user -- that is, pages that
 * are not part of the flow -- so that they get included in the page count.
 * They can't be added to the page count right away because the current
 * flow page needs to have access to the correct current page number.
 * @param newPage the newly created/added page that is to be deferred from page
 *          counting for the moment; if non-null, its page number will
 *          be set with the correct page number
 */
protected void deferCountingNewPage(JCPage newPage)
{
    deferredPageCount++;
    // make new page with proper page number
    if (newPage != null) {
        newPage.setPageNumber(documentPageCount + deferredPageCount);
    }
}

/**
 * Completes the current page.
 * @param next_template
 */
private void endPage(JCPage next_template) {

	Iterator    frames;
	JCFrame     frame;

	// If there is no current page, exit
	if (currentPage == null) {
		return;
	}

	

	// Complete the frames on the page
	frames = currentPage.getFrameList().iterator();
	while (frames.hasNext()) {
		frame = (JCFrame) frames.next();
		frame.complete();
	}

	// Send the PAGE_END event
	JCFlowEvent event = new JCFlowEvent(this, JCFlowEvent.PAGE_END,
										currentPage, next_template);
	Enumeration e = JCListenerList.elements(flowListeners);
	for (; e.hasMoreElements(); ) {
		JCFlowListener listener = (JCFlowListener) e.nextElement();
		listener.pageEnd(event);
	}

	// If there are no uncompleted macros on this page
	currentPage.calcLayoutStatus();
	if (currentPage.getLayoutStatus() == Render.LAYOUT_COMPLETE) {

		// Pass a PAGE_COMPLETE event to any flow listeners
		event = new JCFlowEvent(this, JCFlowEvent.PAGE_COMPLETE, currentPage);
		e = JCListenerList.elements(flowListeners);
		for (; e.hasMoreElements(); ) {
			JCFlowListener listener = (JCFlowListener) e.nextElement();
			listener.pageComplete(event);
		}

		// Check if the page should be output (ASAP printing mode)
		document.checkOutputPage(currentPage);
	}

    // if any pages were deferred from being added to the page count,
    // now's the time to add them
    if (deferredPageCount > 0) {
        updatePageNumbers(null);
    }
}

/**
 * Creates a default page template.
 * @param doc the document for which the template is created
 */
private void createDefaultTemplate(JCDocument doc) {

	JCPage      page;
	JCFrame     frame;
	List        frameList;

	// Create a default page template
	page = new JCPage("default", doc);
	page.setPageType(JCPage.PAGE_TYPE_TEMPLATE);
	doc.getTemplates().add(page);

	// Set the size and flow info for the page
	page.setSize(new JCUnit.Dimension(JCUnit.INCHES, 8.5, 11));
	page.setFlowPageTemplate(page);
	page.setFlowSectionTemplate(page);
	frameList = page.getFrameList();

	// Create frames and add them to the page
	frame = new JCFrame("header", doc);
	frame.setLocation(new JCUnit.Point(JCUnit.INCHES, 0.5, 0.5));
	frame.setSize(new JCUnit.Dimension(JCUnit.INCHES, 7.5, 0.25));
	frameList.add(frame);

	frame = new JCFrame("footer", doc);
	frame.setLocation(new JCUnit.Point(JCUnit.INCHES, 0.5, 10.25));
	frame.setSize(new JCUnit.Dimension(JCUnit.INCHES, 7.5, 0.25));
	frameList.add(frame);

	frame = new JCFrame("body", doc);
	frame.setLocation(new JCUnit.Point(JCUnit.INCHES, 0.5, 1.0));
	frame.setSize(new JCUnit.Dimension(JCUnit.INCHES, 7.5, 9.0));
	frameList.add(frame);

	// Put the body frame in the flow list
	frameList = page.getFlowFrameList();
	frameList.add(frame);
}

}

