/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCFlowEvent.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:22 $ $Locker:  $  Quest Software Inc. 
package com.klg.jclass.page;

/**
 * A <code>JCFlow</code> event occurs when entering or exiting a new frame or page as a
 * result of document flow, and also when a frame or page is marked as
 * complete by the resolution of embedded macros.
 */
public class JCFlowEvent {

public static final int PAGE_BEGIN = 1;
public static final int PAGE_END = 2;
public static final int PAGE_COMPLETE = 3;
public static final int FRAME_BEGIN = 4;
public static final int FRAME_END = 5;
public static final int FRAME_COMPLETE = 6;

/** The flow in which the event occurs. */
protected JCFlow        source;
/** The identifier indicating the event that has occurred. */
protected int           eventId;
/** The frame or page in which the event has occurred. */
protected PageArea      currentPageArea;
/** The next frame or page in which the event has occurred. */
protected PageArea      nextPageArea;
/** The name of the next frame or page template. */
protected String        nextAreaName;

public JCFlowEvent(JCFlow source, int event_id, PageArea current_page_area) {

	if (event_id != PAGE_BEGIN && event_id != FRAME_BEGIN &&
		   event_id != PAGE_COMPLETE && event_id != FRAME_COMPLETE) {
		throw new IllegalArgumentException("invalid value for event_id");
	}

	this.source = source;
	this.eventId = event_id;
	this.currentPageArea = current_page_area;
	this.nextPageArea = null;
	this.nextAreaName = null;
}

public JCFlowEvent(JCFlow source, int event_id, PageArea current_page_area,
				   PageArea next_page_area) {

	if (event_id != PAGE_BEGIN && event_id != PAGE_END &&
		event_id != FRAME_BEGIN && event_id != FRAME_END &&
		event_id != PAGE_COMPLETE && event_id != FRAME_COMPLETE) {
		throw new IllegalArgumentException("invalid value for event_id");
	}

	this.source = source;
	this.eventId = event_id;
	this.currentPageArea = current_page_area;
	this.nextPageArea = next_page_area;

	if (next_page_area != null) {
		this.nextAreaName = next_page_area.getName();
	}
	else {
		this.nextAreaName = null;
	}
}

public JCFlow getSource() {
	return source;
}

public PageArea getCurrentPageArea() {
	return currentPageArea;
}

public PageArea getNextPageArea() {
	return nextPageArea;
}

public String getNextElementName() {
	return nextAreaName;
}

public void setNextPageArea(PageArea new_next) {
	if (eventId != PAGE_BEGIN && eventId != FRAME_BEGIN) {
		throw new IllegalArgumentException("cannot set next page area in a begin call");
	}
	nextPageArea = new_next;
}		

}
