/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCFlowInfo.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:22 $ $Locker:  $  Quest Software Inc.
package com.klg.jclass.page;




import java.util.List;
import java.util.Vector;


import com.klg.jclass.page.render.SymbolRender;

/**
 * <code>JCFlowInfo</code> contains current information on the state of the the document flow.
 */
public class JCFlowInfo {

/** The list of floating objects currently awaiting placement. */
public Vector           floatList;
/** The list of objects in the current (last) line which must be placed. */
public Vector           currentLine;
/** The style which has been most recently applied to this frame. */
public JCTextStyle      currentTextStyle;

/** The left edge of the current column. */
public JCUnit.Measure   leftOfColumn;
/** The right edge of the current column. */
public JCUnit.Measure   rightOfColumn;
/** The left edge of the current line of text (accounting for indentation). */
public JCUnit.Measure   lineBegin;
/** The right edge of the current line of text (accounting for indentation). */
public JCUnit.Measure   lineEnd;
/** The current insertion point in the frame. */
public JCUnit.Point     insertion;
/** The current column of a multi-column frame which is receiving input. */
public int              currentColumn;
/** Indicates if the current line is the first line of a paragraph. */
public boolean          firstLine;

/** The maximal ascender of all objects in the current line. */
public JCUnit.Measure   ascender;
/** The maximal descender of all objects in the current line. */
public JCUnit.Measure   descender;
/** The amount of additional whitespace to leave between lines of text. */
public JCUnit.Measure   leading;
/** The height (ascender and descender) of the current line. */
public JCUnit.Measure   height;
/** The amount of spacing to advance to the next line of text. */
public JCUnit.Measure   lineSpacing;

/** The maximal ascender of all embedded objects in the current line. */
public JCUnit.Measure   embedAscent;
/** The maximal descender of all embedded objects in the current line. */
public JCUnit.Measure   embedDescent;

/** The tab object which is controlling current text layout. */
public JCTab            currentTab;
/** The index of the first item in the current line affected by the tab. */
public int              tabIndex;
/** The symbol representing the current tab. */
public SymbolRender     tabSymbol;
/** A flag to indicate the presence of a tab on this line. */
public boolean          tabUsed;

/**
 * Creates a flow info object with no contents.
 */
public JCFlowInfo() {

	currentLine = new Vector();
	floatList = new Vector();
	currentTextStyle = null;

	leftOfColumn = new JCUnit.Measure();
	rightOfColumn = new JCUnit.Measure();
	lineBegin = new JCUnit.Measure(JCUnit.INTERNAL, -1.0);
	lineEnd = new JCUnit.Measure(JCUnit.INTERNAL, -1.0);
	insertion = new JCUnit.Point();

	currentColumn = 0;
	firstLine = true;

	ascender = new JCUnit.Measure();
	descender = new JCUnit.Measure();
	leading = new JCUnit.Measure();
	height = new JCUnit.Measure();
	lineSpacing = new JCUnit.Measure();

	embedAscent = new JCUnit.Measure();
	embedDescent = new JCUnit.Measure();

	currentTab = null;
	tabIndex = -1;
	tabUsed = false;
}

/**
 * Creates a flow info object with the given current and float lists.
 * @param currents the list of current floats in the frame
 * @param floats the list of floating objects in the frame
 */
public JCFlowInfo(Vector currents, Vector floats) {

	currentLine = currents;
	floatList = floats;
	currentTextStyle = null;

	leftOfColumn = new JCUnit.Measure();
	rightOfColumn = new JCUnit.Measure();
	lineBegin = new JCUnit.Measure(JCUnit.INTERNAL, -1.0);
	lineEnd = new JCUnit.Measure(JCUnit.INTERNAL, -1.0);
	insertion = new JCUnit.Point();

	currentColumn = 0;
	firstLine = true;

	ascender = new JCUnit.Measure();
	descender = new JCUnit.Measure();
	leading = new JCUnit.Measure();
	height = new JCUnit.Measure();
	lineSpacing = new JCUnit.Measure();

	embedAscent = new JCUnit.Measure();
	embedDescent = new JCUnit.Measure();

	currentTab = null;
	tabIndex = -1;
	tabUsed = false;
}

/**
 * Gets the list of unplaced floating objects.
 * @return all unplaced floating objects given to the current frame
 */
public List getFloatList() {
	return (List) floatList;
}

/**
 * Gets the current line (placed, but unpasted objects).
 * @return the list of objects making up the current line
 */
public List getCurrent() {
	return (List) currentLine;
}

}
