/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCFlowListener.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:22 $ $Locker:  $  Quest Software Inc. 
package com.klg.jclass.page;

/**
 * Listens to events during the flow of a document.<p>
 * The general order of calls is:<br>
 * <code>pageBegin()</code> -- a new page has been created, but the frames have not been
 *                created and flow has not yet begun.<br>
 * <code>frameBegin()</code> -- a new frame has been created.  Text will be flowed to
 *                 the frame following this call.<br>
 * <code>frameEnd()</code> -- flow is being transferred to another frame.<br>
 * <code>frameComplete()</code> -- flow is complete in this frame.  All macros in the
 *                    frame have been evaluated.<br>
 * <code>pageEnd()</code> -- flow is being transferred to another page.<br>
 * <code>pageComplete()</code> -- flow is complete for this page.  All macros in the page
 *                   have been evaluated.<br>
 */
public interface JCFlowListener {

/**
 * Invoked before the flow to a frame begins.
 * @param event
 */
public void frameBegin(JCFlowEvent event);

/**
 * Invoked when the flow is transfered to another frame.
 * @param event
 */
public void frameEnd(JCFlowEvent event);

/**
 * Invoked when the flow to a frame is complete.
 * @param event
 */
public void frameComplete(JCFlowEvent event);

/**
 * Invoked before the flow to a page begins.
 * @param event
 */
public void pageBegin(JCFlowEvent event);

/**
 * Invoked when the flow is transferred to another page.
 * @param event
 */
public void pageEnd(JCFlowEvent event);

/**
 * Invoked when the flow to a page is complete.
 * @param event
 */
public void pageComplete(JCFlowEvent event);

}
