/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCFrame.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:22 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page;

import java.awt.Component;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Image;
import java.awt.Dimension;
import java.awt.Graphics;

import java.util.StringTokenizer;

import javax.swing.JComponent;
import javax.swing.Icon;
import javax.swing.ImageIcon;



import com.klg.jclass.page.render.Embedable;
import com.klg.jclass.page.render.Floatable;
import com.klg.jclass.page.render.Render;
import com.klg.jclass.page.render.ArcRender;
import com.klg.jclass.page.render.BoxRender;
import com.klg.jclass.page.render.HRuleMarker;
import com.klg.jclass.page.render.ImageMarker;
import com.klg.jclass.page.render.ImageRender;
import com.klg.jclass.page.render.LineRender;
import com.klg.jclass.page.render.MacroRender;
import com.klg.jclass.page.render.RoundRectRender;
import com.klg.jclass.page.render.Splitable;
import com.klg.jclass.page.render.SymbolRender;
import com.klg.jclass.page.render.StringRender;
import com.klg.jclass.page.render.TableMarker;
import com.klg.jclass.util.ServerRenderable;
import com.klg.jclass.util.swing.JCSwingUtilities;





import java.awt.Graphics2D;

import java.awt.font.FontRenderContext;
import java.awt.font.GlyphVector;
import java.awt.font.LineMetrics;

import java.awt.geom.Rectangle2D;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;


/**
 * A <code>JCFrame</code> represents a part of a page (or another <code>JCFrame</code>) into which
 * the page's content is drawn.
 */
public class JCFrame extends PageArea implements Cloneable, Floatable {

/** A large value used internatlly to mark a frame as having a flexible size. */
protected static JCUnit.Measure FLEXIBLE_SIZE =
                        new JCUnit.Measure(JCUnit.INTERNAL, 10000);

/** Specifies the border style, thickness, color, etc. */
protected JCDrawStyle       border;
/** The number of columns of text which are laid out in the frame. */
protected int               columnCount;
/** The amount of space to allow for a gutter between columns. */
protected JCUnit.Measure    columnSpace;
/** A derived quantity determined by frame width, column count, and separation. */
protected JCUnit.Measure    columnWidth;
/** The amount of space to be left for margins on each side of the frame. */
protected JCUnit.Margins    margin;
/** A list of all the graphical objects which are rendered in this frame. */
protected List              renderList;

/** If different from the frame size, the rectangular region to display. */
protected JCUnit.Margins    clipMask;    // used mostly for table cells
/** This variable tracks the furthest position drawn to. */
protected JCUnit.Measure    yExtent;

/** Flow information contains current state and pending additions. */
protected JCFlowInfo        info;
/** Determines the horizontal alignment of this frame as a floating object. */
protected int               floatAlignment;

/** Stores the page on which this frame will be printed. */
protected JCPage            page;

/** The standard name for a header frame on a page. */
public static final String  HEADER = "header";
/** The standard name for a footer frame on a page. */
public static final String  FOOTER = "footer";


/**
 * Creates a new zero-sized frame.
 */
public JCFrame() {
    // Create a frame without a name - note that this may cause problems
    super();

    // Assign default values
    columnCount = 1;
    columnSpace = new JCUnit.Measure(JCUnit.INTERNAL, 0.25);
    margin = new JCUnit.Margins();
    border = JCDrawStyle.stringToStyle("default blank");

    // Column width is 0 since the frame has no given width
    columnWidth = new JCUnit.Measure();

    // Create the frame's flow information
    info = new JCFlowInfo();

    renderList = new Vector();
    yExtent = new JCUnit.Measure();
    clipMask = null;
}

/**
 * Creates a new blank frame.
 * @param doc the document to which the frame will belong
 */
public JCFrame(JCDocument doc) {

    // Create a frame without a name - note that this may cause problems
    super(doc);

    // Assign default values
    columnCount = 1;
    columnSpace = new JCUnit.Measure(JCUnit.INTERNAL, 0.25);
    margin = new JCUnit.Margins();
    border = JCDrawStyle.stringToStyle("default blank");

    // Column width is 0 since the frame has no given width
    columnWidth = new JCUnit.Measure();

    // Create the frame's flow information
    info = new JCFlowInfo();

    renderList = new Vector();
    yExtent = new JCUnit.Measure();
    clipMask = null;
}

/**
 * Creates a new named frame.
 * @param name the name of the frame
 */
public JCFrame(String name) {

    super(name);

    // Assign default values
    columnCount = 1;
    columnSpace = new JCUnit.Measure(JCUnit.INTERNAL, 0.25);
    margin = new JCUnit.Margins();
    border = JCDrawStyle.stringToStyle("default blank");

    // Column width is 0 since the frame has no given width
    columnWidth = new JCUnit.Measure();

    // Create the frame's flow information
    info = new JCFlowInfo();

    renderList = new Vector();
    yExtent = new JCUnit.Measure();
    clipMask = null;
}

/**
 * Creates a new named frame.
 * @param doc the document to which this frame belongs
 * @param name the name of the frame
 */
public JCFrame(String name, JCDocument doc) {

    super(name, doc);

    // Assign default values
    columnCount = 1;
    columnSpace = new JCUnit.Measure(JCUnit.INTERNAL, 0.25);
    margin = new JCUnit.Margins();
    border = JCDrawStyle.stringToStyle("default blank");

    // Column width is 0 since the frame has no given width
    columnWidth = new JCUnit.Measure();

    // Create the frame's flow information
    info = new JCFlowInfo();

    renderList = new Vector();
    yExtent = new JCUnit.Measure();
    clipMask = null;
}

/**
 * Creates a new frame at a given position.
 * @param name the name of the frame
 * @param location the origin of the frame on the page
 * @param size the dimensions of the frame
 */
public JCFrame(String name, JCUnit.Point location, JCUnit.Dimension size) {

    super(name, location, size);

    // Assign default values
    columnCount = 1;
    columnSpace = new JCUnit.Measure(JCUnit.INTERNAL, 0.25);

    // Margins are initialised to zero
    margin = new JCUnit.Margins();
    border = JCDrawStyle.stringToStyle("default blank");

    // compute column width
    columnWidth = new JCUnit.Measure();
    computeColumnWidth();

    // Create the frame's flow information
    info = new JCFlowInfo();

    renderList = new Vector();
    yExtent = new JCUnit.Measure();
    clipMask = null;
}

/**
 * Creates a new frame at a given position.
 * @param doc the document to which this frame will belong
 * @param name the name of the frame
 * @param location the origin of the frame on the page
 * @param size the dimensions of the frame
 */
public JCFrame(String name, JCDocument doc, JCUnit.Point location,
               JCUnit.Dimension size) {
    super(name, doc, location, size);

    // Assign default values
    columnCount = 1;
    columnSpace = new JCUnit.Measure(JCUnit.INTERNAL, 0.25);

    // Margins are initialised to zero
    margin = new JCUnit.Margins();
    border = JCDrawStyle.stringToStyle("default blank");

    // compute column width
    columnWidth = new JCUnit.Measure();
    computeColumnWidth();


    // Create the frame's flow information
    info = new JCFlowInfo();

    renderList = new Vector();
    yExtent = new JCUnit.Measure();
    clipMask = null;

}


/**
 * Gets the horizontal floating alignment of this frame within its parent frame.
 * @return alignment of this frame to its parent
 */
public int getAlignment() {
    return floatAlignment;
}


/**
 * Sets the horizontal alignment of this frame as a floating object in another
 * frame.
 * @param alignment alignment of this frame within another
 */
public void setAlignment(int alignment) {

    if (alignment <= 0 || alignment > JCTextStyle.ALIGNMENT_JUSTIFY) {
        throw new IllegalArgumentException("Unrecognised horizontal alignment");
    }

    floatAlignment = alignment;
}


/**
 * Gets the style which determines how the frame's border is drawn.
 * @return draw style object which defines the frame's border
 */
public JCDrawStyle getBorder() {
    return border;
}


/**
 * Sets the style object which determines how the frame's border is drawn.
 * @param border drawing style object to define the frame's border
 */
public void setBorder(JCDrawStyle border) {

    if (border == null) {
        throw new IllegalArgumentException("border style must be non-null");
    }

    // Make sure the style is immutable
    border = border.getImmutable();

    this.border = border;
}

/**
 * Gets the mask which controls the portion of the frame which is drawn.
 * @return the clip mask to apply when drawing the cell
 */
public JCUnit.Margins getClipMask() {
    return clipMask;
}

/**
 * Sets the mask which controls the portion of the frame which is drawn.
 * @param clip the clip mask to apply when drawing the cell
 */
public void setClipMask(JCUnit.Margins clip) {

    JCUnit.Measure  base, sum, max;
    JCUnit.Margins  newMask;

    // Make a copy of the provided mask
    newMask = (JCUnit.Margins) clip.clone();

    // Constrain the mask within the boundaries of the frame
    base = newMask.getLeft();
    sum = newMask.getRight();
    sum.add(base);
    if (base.isNegative()) {
        newMask.setLeft(new JCUnit.Measure());
        base = newMask.getLeft();
    }
    max = size.getWidth();
    if (sum.greaterThan(max)) {
        max.subtract(base);
        newMask.setRight(max);
    } else {
        newMask.setRight(sum);
    }

    base = newMask.getTop();
    sum = newMask.getBottom();
    sum.add(base);
    if (base.isNegative()) {
        newMask.setTop(new JCUnit.Measure());
        base = newMask.getTop();
    }
    max = size.getHeight();
    if (sum.greaterThan(max)) {
        max.subtract(base);
        newMask.setBottom(max);
    } else {
        newMask.setBottom(sum);
    }

    // set the new mask on the frame
    this.clipMask = newMask;
}

/**
 * Gets the number of columns of text to display in the frame.
 * @return number of columns in the frame
 */
public int getColumnCount() {
    return columnCount;
}

/**
 * Specifies the number of columns of text to display in the frame.
 * @param columnCount number of columns of text to create
 */
public void setColumnCount(int columnCount) {

    // Verify the columnCount is > 0
    if (columnCount < 1) {
        throw new IllegalArgumentException("Number of columns must be >= 1");
    }

    // if the column count changes, recompute the column width
    if (this.columnCount != columnCount) {

        this.columnCount = columnCount;
        computeColumnWidth();
    }
}

/**
 * Gets the width of columns of text in this frame.
 * @return width of text columns in this frame
 */
public JCUnit.Measure getColumnWidth() {
    return columnWidth;
}

/**
 * Gets the space to leave between columns of text.
 * @return white space between columns in the frame
 */
public JCUnit.Measure getColumnSpace() {
    return columnSpace;
}

/**
 * Specifies the space to leave between columns of text.
 * @param columnSpace white space between columns in the frame
 */
public void setColumnSpace(JCUnit.Measure columnSpace) {

    // Verify the columnSpace is >= 0
    if (columnSpace.isNegative()) {
        throw new IllegalArgumentException("Column spacing must be >= 0");
    }

    // if the column space changes, recompute the column width
    if (this.columnSpace.internal() != columnSpace.internal()) {

        this.columnSpace = columnSpace;
        computeColumnWidth();
    }
}

/**
 * Gets the margin spacing defined for the perimeter of the frame.
 * @return white space surrounding the contents of the frame
 */
public JCUnit.Margins getMargins() {
    return((JCUnit.Margins) margin.clone());
}

/**
 * Specifies the margin spacing for the perimeter of the frame.
 * @param margins white space surrounding the contents of the frame
 */
public void setMargins(JCUnit.Margins margins) {

    boolean         recompute;
    JCUnit.Measure  width, height;

    // all margins must be non-negative
    if (margins.top < 0.0 || margins.bottom < 0.0
        || margins.left < 0.0 || margins.right < 0.0) {
        throw new IllegalArgumentException("Margin width must be >= 0");
    }

    // Check that margins leave non-zero space in the frame
    width = size.getWidth();
    width.subtract(margins.getLeft());
    width.subtract(margins.getRight());
    if (width.isNegative()) {
        throw new IllegalArgumentException("Margins can't be wider than the frame");
    }

    height = size.getHeight();
    height.subtract(margins.getTop());
    height.subtract(margins.getBottom());
    if (height.isNegative()) {
        throw new IllegalArgumentException("Margins can't be taller than the frame");
    }

    // if the side margins change, recompute the column width
    recompute = ( !this.margin.getLeft().equals(margins.getLeft())
                  || !this.margin.getRight().equals(margins.getRight()) );

    this.margin = (JCUnit.Margins) margins.clone();
    if (recompute) {
        computeColumnWidth();
    }
}

/**
 * Sets the page on which this frame (and its descendants) appears.
 * @param page the page on which this frame is being placed
 */
public void setPage(JCPage page) {

    int         i, count;
    JCFrame     frame;

    // Set the page for this frame
    this.page = page;

    // Set the page for all included frames
    count = frameList.size();
    for (i = 0; i < count; i++) {
        frame = (JCFrame) frameList.get(i);
        frame.setPage(page);
    }
}

/**
 * Changes the size of the frame.
 * @param size new size of the frame
 */
public void setSize(JCUnit.Dimension size) {

    JCUnit.Margins  newMask;
    JCUnit.Measure  base, sum;

    // check for non-negative size
    if (size.width < 0.0 || size.height < 0.0) {
        throw new IllegalArgumentException("Dimensions must be non-negative");
    }

    // set the new size and recompute the column width
    this.size = (JCUnit.Dimension) size.clone();
    computeColumnWidth();

    // if the frame has a clipMask, recompute the clipMask based on the size
    if (clipMask != null) {

        
        
        

        // Constrain the mask within the new boundaries of the frame
        base = clipMask.getLeft();
        sum = clipMask.getRight();
        if (base.isNegative()) {
            sum.add(base);
            clipMask.setLeft(new JCUnit.Measure());
        }
        if (sum.greaterThan(size.getWidth())) {
            clipMask.setRight(size.getWidth());
        }

        base = clipMask.getTop();
        sum = clipMask.getBottom();
        if (base.isNegative()) {
            sum.add(base);
            clipMask.setTop(new JCUnit.Measure());
        }
        if (sum.greaterThan(size.getWidth())) {
            clipMask.setBottom(size.getWidth());
        }

        
        
        
    }
}

/**
 * Gets the flow information object.
 * @return the information on the current state of flowing to this frame
 * @deprecated this method is superceded by <code>getFlowInfo()</code>
 */
public JCFlowInfo getInfo() {
    return(info);
}

// print methods

/**
 * Flows text into this frame.
 * @param style the style to apply to the new text
 * @param text the text to be flowed
 * @throws EndOfFrameException
 */
public void print(JCTextStyle style, String text) throws EndOfFrameException
{
    Graphics2D          gc;
    StringRender        render;

    // Make sure we've got an immutable style
    style = style.getImmutable();
    info.currentTextStyle = style;

    // Create a render object for the text given the style
    gc = (Graphics2D) document.getPrinterGraphics();
    render = new StringRender(gc, text, style);

    // Flow the text object to the frame
    flowPrint(gc, render, false);
}

/**
 * Flows a text macro into this frame.
 * @param style the style to apply to the macro
 * @param macro the macro to be added to the flow
 * @throws EndOfFrameException
 */
public void print(JCTextStyle style, TextMacro macro)
            throws EndOfFrameException {

    Graphics2D          gc;
    MacroRender         render;

    // Make sure we've got an immutable style
    style = style.getImmutable();
    info.currentTextStyle = style;

    // evaluate the macro in the current context (this page)
    macro.evaluate(null, page);

    // Create a render object for the text given the style
    gc = (Graphics2D) document.getPrinterGraphics();

    // Construct the macro as a render object
    render = new MacroRender(gc, macro, style);

    // Flow the text object to the frame
    flowPrint(gc, render, false);
}

/**
 * Flows a rendered text macro into this frame.
 * @param style the style to apply to the macro
 * @param macro a render object encapsulating the macro to be added to the flow
 * @throws EndOfFrameException
 */
public void print(JCTextStyle style, MacroRender macro)
            throws EndOfFrameException {
    Graphics2D          gc;
    MacroRender         render;

    // Make sure we've got an immutable style
    style = style.getImmutable();
    info.currentTextStyle = style;

    // Create a render object for the text given the style
    gc = (Graphics2D) document.getPrinterGraphics();

    // Flow the text object to the frame
    flowPrint(gc, macro, false);

}


/**
 * Prints text into this frame at a given position without flowing.
 * @param style the style to apply to the new text
 * @param text the text to be placed in the frame
 * @param location the origin point for the text
 */
public void print(JCTextStyle style, String text, JCUnit.Point location) {

    Graphics2D          gc;
    StringRender        render;

    // Make sure we've got an immutable style
    style = style.getImmutable();
    info.currentTextStyle = style;

    // Create a render object for the text given the style
    gc = (Graphics2D) document.getPrinterGraphics();
    render = new StringRender(gc, text, style);

    // Set the location of the render object and add to the frame's render list
    render.setLocation(location);
    renderList.add(render);

}


/**
 * Flows text, which may have embedded newline characters,
 * into this frame such that the newlines are performed.
 * @param style the style to apply to the new text
 * @param text the text to be printed in the frame
 * @throws EndOfFrameException
 */
protected void printWithNewlines(JCTextStyle style, String text)
               throws EndOfFrameException {

    // Check to make sure the text object isn't null, if it is,
    // we don't need to tokenize it.
    if (text == null)
        return;

    // Break the input into a sequence of strings based on newlines
    StringTokenizer st = new StringTokenizer(text, "\n");

    // Traverse the string, printing lines and performing newlines between
    while (st.hasMoreTokens()) {
        print(style, st.nextToken());
        newLine(style);
    }
}

/**
 * Draws a table in the current frame.
 * @param table the table to draw
 * @throws EndOfFrameException
 */
public void print(JCPageTable table) throws EndOfFrameException {

    

    // add a marker object for the table to the flow
    renderList.add(new TableMarker(table, info.currentTextStyle));

    // start printing the table in the upper-left corner
    print(table, 0, 0);
}

/**
 * Draws a table in the current frame starting in the given position.
 * @param table the table to draw
 * @param firstRow the row of the table to start drawing from
 * @param firstColumn the first column of the table to be output in the start row
 * @throws EndOfFrameException
 */
public void print(JCPageTable table, int firstRow, int firstColumn)
                                              throws EndOfFrameException {
    int                 last_row, last_column;

    // Get the size of the table
    last_row = table.getRowCount() - 1;
    last_column = table.getColumnCount() - 1;

    try {
        print(table, firstRow, firstColumn, last_row, last_column);
    } catch (EndOfFrameException e) {
        // Pass on the end-of-frame exception
        throw e;
    }
}

/**
 * Draws a table in the current frame starting in the given position.
 * @param table the table to draw
 * @param firstRow the row of the table to start drawing from
 * @param firstColumn the first column of the table to be output in the start row
 * @param last_row the row of the table to stop drawing at
 * @param last_column the last column of the table to be output
 * @throws EndOfFrameException
 */
public void print(JCPageTable table, int firstRow, int firstColumn,
                                        int last_row, int last_column)
                                              throws EndOfFrameException {

    int                 start_column, end_column, start_row, end_row;
    int                 header_rows;
    int                 overflow_mode;
    JCPageTable         header_table;
    JCUnit.Measure      width, height, header_height, frame_height;
    JCUnit.Measure      vspace, actual_height, actual_width;
    JCUnit.Measure      available_height, remaining_height;
    JCUnit.Measure      new_y;

    

    // Make sure the frame has the layout constraints initialised
    if (!isInfoInitialised()) {
        initialiseInfo();
    }

    // add a marker object for the table to the flow
    renderList.add(new TableMarker(table, info.currentTextStyle, firstRow,
                                   firstColumn));

    // make sure table is ready to print (fix frame heights, etc.)
    table.completeLayout();

    // Check for tables with no rows
    if (last_row == -1) {

        // check if the null table has headers
        if (table.hasHeaders()) {
            header_table = table.getHeaders();
            header_rows = header_table.getRowCount() - 1;

            // if there are headers try printing them directly
            if (header_rows != -1) {
                print(header_table, 0, firstColumn);
            }
        }

        // otherwise ignore tables with no rows
        return;
    }

    // Compute the dimensions of the table
    width = table.getColumnRangeWidth(firstColumn, last_column);
    height = table.getRowRangeHeight(firstRow, last_row);
    

    // If the table has headers, compute their height
    if (table.hasHeaders()) {
        header_table = table.getHeaders();
        header_rows = header_table.getRowCount() - 1;
        header_height = header_table.getRowRangeHeight(0, header_rows);
    }
    else {
        header_height = new JCUnit.Measure();
    }

    // Make sure we have a text style
    if (info.currentTextStyle == null) {
        info.currentTextStyle = JCTextStyle.DEFAULT_TEXT;
    }

    // if there is anything in the current line, go to a new line
    if (info.currentLine.size() != 0) {
        doLineBreak(info.currentTextStyle, false);
    }

    // determine the amount of space remaining in this frame
    frame_height = size.getHeight();
    vspace = (JCUnit.Measure) frame_height.clone();
    vspace.subtract(info.insertion.getY());
    vspace.subtract(margin.getBottom());

    // Initially we try to fit the entire (remaining) table in the width
    start_column = firstColumn;
    end_column = last_column;
    actual_width = (JCUnit.Measure) width.clone();
    overflow_mode = table.getOverflowMode();

    // NOTE:  Currently there is no overflow option that keeps rows together
    //        at the expense of interleaving the columns.  (All the columns
    //        of one set of rows are printed before starting new rows.)
    //        Overflow currently prints all the rows of a column, then
    //        procedes to succeeding columns

    // if the table is too wide and it's not being clipped (wraps or overflows)
    if (columnWidth.lessThan(width)
        && overflow_mode != JCPageTable.OVERFLOW_CLIP_COLUMNS) {

        // determine a range of columns which fit
        end_column = table.getColumnSet(start_column, columnWidth,
                                        actual_width);
    }
    


    start_row = firstRow;
    remaining_height = (JCUnit.Measure) height.clone();
    actual_height = (JCUnit.Measure) height.clone();

    available_height = (JCUnit.Measure) vspace.clone();
    available_height.subtract(header_height);

    // loop through the rows in this set of columns
    while (remaining_height.distance > 0.0) {

        // determine a set of rows that fit in the available space
        
        end_row = table.getRowSet(start_row, available_height, actual_height);
        
        

        // print the resulting section unless it's a single row that doesn't
        // fit into a frame that's already partly full
        JCUnit.Measure start_space = margin.getTop();
        start_space.add(getFirstLineOffset(info.currentTextStyle));

        if (actual_height.greaterThan(available_height) &&
            (start_row + 1 < end_row ||
             info.insertion.getY().greaterThan(start_space))) {

            
            // Go to the next column in this frame and try there
            try {
                newColumn(info.currentTextStyle, false);
            }
            catch (EndOfFrameException e) {

                // update the frame/flow info for the current position
                if (start_row == firstRow)
                    e.setTableFlow(table, start_row, start_column);
                else
                    e.setTableFlow(table, start_row, start_column, end_column);

                // Pass on the end-of-frame exception
                throw e;
            }

            // if newColumn() succeeded, update the space parameters
            vspace = (JCUnit.Measure) frame_height.clone();
            vspace.subtract(info.insertion.getY());
            vspace.subtract(margin.getBottom());

            available_height = (JCUnit.Measure) vspace.clone();
            available_height.subtract(header_height);

            // try again in the new column
            continue;
        }

        // draw the section of the table
        table.drawTablePart(this, start_row, start_column, end_row, end_column,
                            header_height, actual_height, actual_width);

        // NOTE:  At this point we would add tear-off pages (side pages)
        //        to continue the table horizontally.

        // move the insertion point after the table
        new_y = info.insertion.getY();
        new_y.add(actual_height);
        new_y.add(header_height);

        // Add space needed to offset the top of text below the table
        new_y.add(getFirstLineOffset(info.currentTextStyle));
        info.insertion.setY(new_y);

        // We now want to consider the next set of rows in the table
        start_row = end_row + 1;

        // if there are more rows in the table then they didn't fit
        if (start_row <= last_row) {
            remaining_height = table.getRowRangeHeight(start_row, last_row);

            // Go to the next column in this frame and try there
            try {
                newColumn(info.currentTextStyle, false);
            }
            catch (EndOfFrameException e) {

                // update the frame/flow info for the current position
                if (start_row == firstRow)
                    e.setTableFlow(table, start_row, start_column);
                else
                    e.setTableFlow(table, start_row, start_column, end_column);

                // Pass on the end-of-frame exception
                throw e;
            }

            // if newColumn() succeeded, update the space parameters
            vspace = (JCUnit.Measure) frame_height.clone();
            vspace.subtract(info.insertion.getY());
            vspace.subtract(margin.getBottom());
        }

        // here all the rows are done; check for wrapped columns
        else if (end_column < last_column
                 && (overflow_mode == JCPageTable.OVERFLOW_WRAP_COLUMNS)) {

            // get the next range of columns
            start_column = end_column + 1;
            end_column = table.getColumnSet(start_column, columnWidth,
                                            actual_width);

            // update the amount of space still needed
            start_row = firstRow;

            remaining_height = height;

            // compute the amount of space left in the frame
            vspace = (JCUnit.Measure) frame_height.clone();
            vspace.subtract(info.insertion.getY());
            vspace.subtract(margin.getBottom());
        }
        // all the rows are done, but wrapping new columns to a new
        // frame column or flow frame rather than to the existing frame
        else if (end_column < last_column
                 && (overflow_mode == JCPageTable.OVERFLOW_WRAP_COLUMNS_NEXT)) {

            // get the next range of columns
            start_column = end_column + 1;
            end_column = table.getColumnSet(start_column, columnWidth,
                                            actual_width);

            // update the amount of space still needed
            start_row = firstRow;

            remaining_height = height;

            // Go to the next column in this frame and try there
            try {
                newColumn(info.currentTextStyle, false);
            }
            catch (EndOfFrameException e) {

                   // update the frame/flow info for the current position
                   if (start_row == firstRow)
                       e.setTableFlow(table, start_row, start_column);
                   else
                       e.setTableFlow(table, start_row, start_column, end_column);

                // Pass on the end-of-frame exception
                throw e;
            }

            // if newColumn() succeeded, update the space parameters
            vspace = (JCUnit.Measure) frame_height.clone();
            vspace.subtract(info.insertion.getY());
            vspace.subtract(margin.getBottom());
        }
        // otherwise we're done
        else {
            remaining_height.distance = 0.0;
        }

        // update the amount of space remaining in the frame
        available_height = (JCUnit.Measure) vspace.clone();
        available_height.subtract(header_height);
    }
}

/**
 * Advances the insertion point to the next tab position.
 * @param style the style which contains the list of tab stops to use
 */
public void tab(JCTextStyle style) {

    JCTab           tab, selectedTab = null;
    Iterator        tabs;
    SymbolRender    tabSymbol;
    JCUnit.Point    tab_location;
    JCUnit.Measure  current_x, current_y, position = null;

    // Make sure we've got an immutable style
    style = style.getImmutable();
    info.currentTextStyle = style;

    // Make sure the frame has the layout constraints initialised
    if (!isInfoInitialised()) {
        initialiseInfo();
    }

    // NOTE:  This is a bit of a hack to avoid problems with infoInitialise
    // If there are no elements on the current line and it's the first
    // line of a frame, make sure the line bounds are correct.
    if (info.currentLine.size() == 0 && info.firstLine) {
        
        computeLineBounds(style, info.firstLine);
        info.insertion.setX(info.lineBegin);
    }

    // Find the next applicable tab
    current_x = info.insertion.getX();
    
    tabs = style.getTabs().iterator();

    while (tabs.hasNext()) {

        // Get a tab and compute its position in the column
        tab = (JCTab) tabs.next();
        position = (JCUnit.Measure) tab.getPosition().clone();
        
        position.add(info.leftOfColumn);
        

        // if the tab is past the current point choose it
        // (We must move to the next tab if we are directly on a tab,
        //  otherwise tab() tab() will not work as expected)
        if (position.greaterThan(current_x)) {
            
            selectedTab = tab;
            break;
        }
    }

    // If there were no more tabs on this line or the selected tab is past
    // the end of the usable portion of this line, move to the next line
    if (selectedTab == null || position.greaterThan(info.lineEnd)) {
        

        // The insertion point will be at the beginning of the new line,
        // but no tab there will have any effect, so we note the tab (as a
        // symbol in the render list) but otherwise ignore it
        tabSymbol = new SymbolRender(SymbolRender.TAB_SYMBOL, style);
        tabSymbol.setLeaderFill(JCTab.TAB_FILL_NONE);
        info.currentLine.add(tabSymbol);

        // Position the tab on the baseline of the current line
        current_y = info.insertion.getY();
        current_y.add(info.ascender);
        tab_location = (JCUnit.Point) info.insertion.clone();
        tab_location.setY(current_y);
        tabSymbol.setLocation(tab_location);

        // Move to the beginning of the next line
        doLineBreak(style, false);
        return;
    }

    // move the insertion point to the tab location
    info.insertion.setX(position);

    // Store the tab we are using to align text
    info.currentTab = selectedTab;
    info.tabUsed = true;

    // The next item in the line will be the first controlled by the tab
    info.tabIndex = info.currentLine.size() + 1;

    // Create a tab symbol
    tabSymbol = new SymbolRender(SymbolRender.TAB_SYMBOL, style);

    // Position the tab on the baseline of the current line
    current_y = info.insertion.getY();
    current_y.add(info.ascender);
    tab_location = (JCUnit.Point) info.insertion.clone();
    tab_location.setY(current_y);
    tabSymbol.setLocation(tab_location);

    // Set the position of the symbol and add to the current line
    tabSymbol.setLeaderStart(current_x);
    tabSymbol.setLeaderEnd(info.insertion.getX());
    tabSymbol.setLeaderFill(selectedTab.getTabFill());
    tabSymbol.setAlignment(selectedTab.getTabAlignment());
    info.currentLine.add(tabSymbol);
    info.tabSymbol = tabSymbol;
}

/**
 * Determines what the next tab position is - the one that the insertion
 * point would traverse to if <code>frame.tab()</code> were called at this point.
 * @param style the style which contains the list of tab stops to use
 * @return the tab location which is next after the current insertion point
 */
public JCTab nextTab(JCTextStyle style) {

    Iterator        tabs;
    JCTab           tab, selected = null;
    JCUnit.Measure  current_x, position;

    // Make sure we've got an immutable style
    style = style.getImmutable();

    // Make sure the frame has the layout constraints initialised
    if (!isInfoInitialised()) {
        initialiseInfo();
    }

    // Determine the current position in the flow
    current_x = info.insertion.getX();

    // Find the next applicable tab
    tabs = style.getTabs().iterator();

    while (tabs.hasNext()) {
        // Find the next tab and compute its location
        tab = (JCTab) tabs.next();
        position = tab.getPosition();
        position.add(info.leftOfColumn);

        if (position.greaterThan(current_x)) {
            selected = tab;
            break;
        }
    }

    // return the next tab (or null if none were found on this line)
    return(selected);
}

// Image float routines add images into the flow as independent objects
// (each floated object will appear on its own line)

/**
 * Adds an image to the document so it is added in the next space it fits
 * (subject to the existence of other floating objects that may precede this).
 * This form of the <code>floatImage()</code> method assumes a default size based on
 * the pixel size of the image and centres the image in the frame.
 * @param image the Image to be floated in the document
 */
public void floatImage(Image image) {

    JCUnit.Dimension    size;
    Icon                temp_icon;
    ImageRender         render;

    

    // Compute a default size from the image assuming 1 pixel = 1 point
    temp_icon = new ImageIcon(image);
    size = new JCUnit.Dimension(JCUnit.POINTS, temp_icon.getIconWidth(),
                                temp_icon.getIconHeight());

    // Create a render object (centre-aligned)
    render = new ImageRender((Object) image, size,
                             JCTextStyle.ALIGNMENT_CENTER);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render));

    // add the render object to the float list
    addFloat(render);

}


/**
 * Adds an image to the document so it is added in the next space it fits
 * (subject to the existence of other floating objects that may precede this).
 * This form of the <code>floatImage()</code> method assumes a default size based on
 * the pixel size of the image.
 * @param image the Image to be floated in the document
 * @param alignment the horizontal alignment of the image in the frame
 */
public void floatImage(Image image, int alignment) {

    JCUnit.Dimension    size;
    Icon                temp_icon;
    ImageRender         render;

    

    // Compute a default size from the image assuming 1 pixel = 1 point
    temp_icon = new ImageIcon(image);
    size = new JCUnit.Dimension(JCUnit.POINTS, temp_icon.getIconWidth(),
                                temp_icon.getIconHeight());

    // Create a render object
    render = new ImageRender((Object) image, size, alignment);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render));

    // add the render object to the float list
    addFloat(render);
}

/**
 * Adds an image to the document so it is added in the next space it fits
 * (subject to the existence of other floating objects that may precede this).
 * This form of the <code>floatImage()</code> method centres the image in the frame.
 * @param image the Image to be floated in the document
 * @param size the dimensions of the image as it is to appear on the page
 */
public void floatImage(Image image, JCUnit.Dimension size) {

    ImageRender         render;

    

    // Create a render object (centre-aligned)
    render = new ImageRender((Object) image, size,
                             JCTextStyle.ALIGNMENT_CENTER);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render));

    // add the render object to the float list
    addFloat(render);
}

/**
 * Adds an image to the document so it is added in the next space it fits
 * (subject to the existence of other floating objects that may precede this).
 * @param image the Image to be floated in the document
 * @param alignment the horizontal alignment of the image in the frame
 * @param size the dimensions of the image as it is to appear on the page
 */
public void floatImage(Image image, int alignment, JCUnit.Dimension size) {

    ImageRender         render;

    

    // Create a render object
    render = new ImageRender((Object) image, size, alignment);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render));

    // add the render object to the float list
    addFloat(render);
}



/**
 * Adds an icon to the document so it is added in the next space it fits
 * (subject to the existence of other floating objects that may precede this).
 * This form of the <code>floatIcon()</code> method assumes a default size based on
 * the pixel size of the icon and centres the icon in the frame.
 * <br>This method only accepts instances of the <code>ImageIcon</code> class.
 * @param icon the Icon to be floated in the document
 */
public void floatIcon(Icon icon) {

    JCUnit.Dimension    size;
    ImageRender         render;

    // Compute a default size from the icon assuming 1 pixel = 1 point
    size = new JCUnit.Dimension(JCUnit.POINTS, icon.getIconWidth(),
                                icon.getIconHeight());

    // Create a render object (centre-aligned)
    render = new ImageRender((Object) icon, size, JCTextStyle.ALIGNMENT_CENTER);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render));

    // add the render object to the float list
    addFloat(render);
}

/**
 * Adds an icon to the document so it is added in the next space it fits
 * (subject to the existence of other floating objects that may precede this).
 * This form of the <code>floatIcon()</code> method assumes a default size based on
 * the pixel size of the icon.
 * <br>This method only accepts instances of the <code>ImageIcon</code> class.
 * @param icon the Icon to be floated in the document
 * @param alignment the horizontal alignment of the icon in the frame
 */
public void floatIcon(Icon icon, int alignment) {

    JCUnit.Dimension    size;
    ImageRender         render;

    // Compute a default size from the icon assuming 1 pixel = 1 point
    size = new JCUnit.Dimension(JCUnit.POINTS, icon.getIconWidth(),
                                icon.getIconHeight());

    // Create a render object
    render = new ImageRender((Object) icon, size, alignment);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render));

    // add the render object to the float list
    addFloat(render);
}

/**
 * Adds an icon to the document so it is added in the next space it fits
 * (subject to the existence of other floating objects that may precede this).
 * This form of the <code>floatIcon()</code> method centres the icon in the frame.
 * <br>This method only accepts instances of the <code>ImageIcon</code> class.
 * @param icon the Icon to be floated in the document
 * @param size the dimensions of the icon as it is to appear on the page
 */
public void floatIcon(Icon icon, JCUnit.Dimension size) {

    ImageRender         render;

    // Create a render object (centre-aligned)
    render = new ImageRender((Object) icon, size, JCTextStyle.ALIGNMENT_CENTER);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render));

    // add the render object to the float list
    addFloat(render);
}

/**
 * Adds an icon to the document so it is added in the next space it fits
 * (subject to the existence of other floating objects that may precede this).
 * <br>This method only accepts instances of the <code>ImageIcon</code> class.
 * @param icon the Icon to be floated in the document
 * @param alignment the horizontal alignment of the icon in the frame
 * @param size the dimensions of the icon as it is to appear on the page
 */
public void floatIcon(Icon icon, int alignment, JCUnit.Dimension size) {

    ImageRender         render;

    // Create a render object
    render = new ImageRender((Object) icon, size, alignment);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render));

    // add the render object to the float list
    addFloat(render);
}

/**
 * Adds a component's drawn image to the document so it is added in the next
 * space it fits (subject to the existence of other floating objects that may
 * precede this).  This form of the <code>floatComponent()</code> method assumes a default
 * size based on the pixel size of the component from its current geometry
 * and centres the component's image.
 * @param c the Component to be floated in the document
 */
public void floatComponent(Component c) {

    JCUnit.Dimension    size;
    ImageRender         render;
    Graphics2D            gc;

    // Compute a default size from the component assuming 1 pixel = 1 point
    gc = (Graphics2D) document.getPrinterGraphics();
    size = sizeComponent(c, gc, 0, 0);

    // Create a render object (centre-aligned)
    render = new ImageRender((Object) c, size, JCTextStyle.ALIGNMENT_CENTER);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render));

    // add the render object to the float list
    addFloat(render);
}

/**
 * Adds a component's drawn image to the document so it is added in the next
 * space it fits (subject to the existence of other floating objects that may
 * precede this).  This form of the <code>floatComponent()</code> method assumes a default
 * size based on the pixel size of the component from its current geometry.
 * @param c the Component to be floated in the document
 * @param alignment the horizontal alignment of the component in the frame
 */
public void floatComponent(Component c, int alignment) {

    JCUnit.Dimension    size;
    ImageRender         render;
    Graphics2D            gc;

    // Compute a default size from the component assuming 1 pixel = 1 point
    gc = (Graphics2D) document.getPrinterGraphics();
    size = sizeComponent(c, gc, 0, 0);

    // Create a render object
    render = new ImageRender((Object) c, size, alignment);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render));

    // add the render object to the float list
    addFloat(render);
}

/**
 * Adds a component's drawn image to the document so it is added in the next
 * space it fits (subject to the existence of other floating objects that may
 * precede this).  This form of the <code>floatComponent()</code> method centres the
 * component in the frame.
 * @param c the Component to be floated in the document
 * @param size the dimensions of the component as it is to appear on the page
 */
public void floatComponent(Component c, JCUnit.Dimension size) {

    ImageRender         render;
    Graphics2D            gc;

    gc = (Graphics2D) document.getPrinterGraphics();
    sizeComponent(c, gc,
        (int) JCUnit.getAsPoints(size.units, size.width),
        (int) JCUnit.getAsPoints(size.units, size.height));
    // Create a render object (centre-aligned)
    render = new ImageRender((Object) c, size, JCTextStyle.ALIGNMENT_CENTER);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render));

    // add the render object to the float list
    addFloat(render);
}

/**
 * Adds a component's drawn image to the document so it is added in the next
 * space it fits (subject to the existence of other floating objects that may
 * precede this).
 * @param c the component to be floated in the document
 * @param alignment the horizontal alignment of the component in the frame
 * @param size the dimensions of the component as it is to appear on the page
 */
public void floatComponent(Component c, int alignment, JCUnit.Dimension size) {

    ImageRender         render;
    Graphics2D            gc;

    gc = (Graphics2D) document.getPrinterGraphics();
    sizeComponent(c, gc,
        (int) JCUnit.getAsPoints(size.units, size.width),
        (int) JCUnit.getAsPoints(size.units, size.height));

    // Create a render object
    render = new ImageRender((Object) c, size, alignment);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render));

    // add the render object to the float list
    addFloat(render);
}

/**
 * Adds an Encapsulated PostScript (EPS) image to the document so it is added
 * in the next space it fits (subject to the existence of other floating
 * objects that may precede this).  This form of the <code>floatEPS()</code> method centres
 * the image and assumes the <code>EPSImage</code> object has a size set.
 * @param eps the EPS image to be floated in the document
 */
public void floatEPS(EPSImage eps) {

    JCUnit.Dimension    size;
    ImageRender         render;
    double              width, height;

    // Compute a default size from the image assuming 1 pixel = 1 point
    size = eps.getSize();
    width = JCUnit.getAsPoints(size.units, size.width);
    height = JCUnit.getAsPoints(size.units, size.height);
    size = new JCUnit.Dimension(JCUnit.POINTS, width, height);

    // Create a render object (centre-aligned)
    render = new ImageRender((Object) eps, size, JCTextStyle.ALIGNMENT_CENTER);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render));

    // add the render object to the float list
    addFloat(render);
}

/**
 * Adds an Encapsulated PostScript (EPS) image to the document so it is added
 * in the next space it fits (subject to the existence of other floating
 * objects that may precede this).  This form of the <code>floatEPS()</code> method assumes
 * the <code>EPSImage</code> object has a size set.
 * @param eps the EPS image to be floated in the document
 * @param alignment the horizontal alignment of the image in the frame
 */
public void floatEPS(EPSImage eps, int alignment) {

    JCUnit.Dimension    size;
    ImageRender         render;
    double              width, height;

    // Compute a default size from the image assuming 1 pixel = 1 point
    size = eps.getSize();
    width = JCUnit.getAsPoints(size.units, size.width);
    height = JCUnit.getAsPoints(size.units, size.height);
    size = new JCUnit.Dimension(JCUnit.POINTS, width, height);

    // Create a render object
    render = new ImageRender((Object) eps, size, alignment);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render));

    // add the render object to the float list
    addFloat(render);
}

/**
 * Adds an Encapsulated PostScript (EPS) image to the document so it is added
 * in the next space it fits (subject to the existence of other floating
 * objects that may precede this).  This form of the <code>floatEPS()</code> method centres
 * the image.
 * @param eps the Image to be floated in the document
 * @param size the dimensions of the image as it is to appear on the page
 */
public void floatEPS(EPSImage eps, JCUnit.Dimension size) {

    ImageRender         render;

    // Create a render object (centre-aligned)
    render = new ImageRender((Object) eps, size, JCTextStyle.ALIGNMENT_CENTER);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render));

    // add the render object to the float list
    addFloat(render);
}

/**
 * Adds an Encapsulated PostScript (EPS) image to the document so it is added
 * in the next space it fits (subject to the existence of other floating
 * objects that may precede this).
 * @param eps the EPS image to be floated in the document
 * @param alignment the horizontal alignment of the image in the frame
 * @param size the dimensions of the image as it is to appear on the page
 */
public void floatEPS(EPSImage eps, int alignment, JCUnit.Dimension size) {

    ImageRender         render;

    // Create a render object
    render = new ImageRender((Object) eps, size, alignment);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render));

    // add the render object to the float list
    addFloat(render);
}

/**
 * Adds a floating object to this frame; pastes it immediately, if possible.
 * @param render a render object encapsulating the floating object
 */
protected void addFloat(ImageRender render) {

    // Add the object at the end of the list of floating objects
    info.floatList.add(render);

    // if there is no data on the current line (we are at a new line)
    // then try to paste floating objects
    if (info.currentLine == null || info.currentLine.size() == 0) {
        
        pasteFloats();
    }
}

// Image embed functions add the image to the objects composing the current
// line.  (This works best for mixing small images with text)

/**
 * Adds an image to the document as part of the current line (the image may
 * wrap if there isn't enough horizontal space).  The height of the current
 * line will be adjusted to accomodate the image.  This form of the
 * <code>embedImage()</code> method assumes a default size based on the pixel size of the
 * image, and centres the image vertically on the current line.
 * @param image the Image to be embedded in the current line
 * @param style
 * @throws EndOfFrameException
 */
public void embedImage(JCTextStyle style, Image image)
                       throws EndOfFrameException {

    Icon                temp_icon;
    Graphics2D          gc;
    ImageRender         render;
    JCUnit.Dimension    size;

    // Make sure we have a style for every frame that has something in it
    info.currentTextStyle = style;

    // Compute a default size from the image assuming 1 pixel = 1 point
    temp_icon = new ImageIcon(image);
    size = new JCUnit.Dimension(JCUnit.POINTS, temp_icon.getIconWidth(),
                                temp_icon.getIconHeight());

    // Create a render object (centre-aligned)
    gc = (Graphics2D) document.getPrinterGraphics();
    render = new ImageRender((Object) image, size);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render, true));

    // add the image to the current line (checking for fit, etc.)
    flowPrint(gc, render, false);
}

/**
 * Adds an image to the document as part of the current line (the image may
 * wrap if there isn't enough horizontal space).  The height of the current
 * line will be adjusted to accomodate the image.  This form of the
 * <code>embedImage()</code> method assumes a default size based on the pixel size of the
 * image.
 * @param image the Image to be embedded in the current line
 * @param alignment the vertical alignment of the image on the current line
 * @param style
 * @throws EndOfFrameException
 */
public void embedImage(JCTextStyle style, Image image, int alignment)
                       throws EndOfFrameException {

    Icon                temp_icon;
    Graphics2D          gc;
    ImageRender         render;
    JCUnit.Dimension    size;

    // Make sure we have a style for every frame that has something in it
    info.currentTextStyle = style;

    // Compute a default size from the image assuming 1 pixel = 1 point
    temp_icon = new ImageIcon(image);
    size = new JCUnit.Dimension(JCUnit.POINTS, temp_icon.getIconWidth(),
                                temp_icon.getIconHeight());

    // Create a render object
    gc = (Graphics2D) document.getPrinterGraphics();
    render = new ImageRender((Object) image, size);
    render.setVerticalAlignment(alignment);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render, true));

    // add the image to the current line (checking for fit, etc.)
    flowPrint(gc, render, false);
}

/**
 * Adds an image to the document as part of the current line (the image may
 * wrap if there isn't enough horizontal space).  The height of the current
 * line will be adjusted to accomodate the image.  This form of the
 * <code>embedImage()</code> method centres the image vertically on the current line.
 * @param image the Image to be embedded in the current line
 * @param size the dimensions of the image as it is to appear on the page
 * @param style
 * @throws EndOfFrameException
 */
public void embedImage(JCTextStyle style, Image image, JCUnit.Dimension size)
                       throws EndOfFrameException {

    Graphics2D          gc;
    ImageRender         render;

    // Make sure we have a style for every frame that has something in it
    info.currentTextStyle = style;

    // Create a render object (centre-aligned)
    gc = (Graphics2D) document.getPrinterGraphics();
    render = new ImageRender((Object) image, size);
    render.setScaleImage(!document.getPrinter().isImageScalable());

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render, true));

    // add the image to the current line (checking for fit, etc.)
    flowPrint(gc, render, false);
}

/**
 * Adds an image to the document as part of the current line (the image may
 * wrap if there isn't enough horizontal space).  The height of the current
 * line will be adjusted to accomodate the image.
 * @param image the Image to be embedded in the current line
 * @param alignment the vertical alignment of the image on the current line
 * @param size the dimensions of the image as it is to appear on the page
 * @param style
 * @throws EndOfFrameException
 */
public void embedImage(JCTextStyle style, Image image, int alignment,
                       JCUnit.Dimension size) throws EndOfFrameException {

    Graphics2D          gc;
    ImageRender         render;

    // Make sure we have a style for every frame that has something in it
    info.currentTextStyle = style;

    // Create a render object
    gc = (Graphics2D) document.getPrinterGraphics();
    render = new ImageRender((Object) image, size);
    render.setScaleImage(!document.getPrinter().isImageScalable());
    render.setVerticalAlignment(alignment);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render, true));

    // add the image to the current line (checking for fit, etc.)
    flowPrint(gc, render, false);
}


/**
 * Adds an icon to the document as part of the current line (the icon may
 * wrap if there isn't enough horizontal space).  The height of the current
 * line will be adjusted to accomodate the icon.  This form of the
 * <code>embedIcon()</code> method assumes a default size based on the pixel size of the
 * icon, and centres the icon vertically on the current line.
 * <br>This method only accepts instances of the <code>ImageIcon</code> class.
 * @param icon the Icon to be embedded in the current line
 * @param style
 * @throws EndOfFrameException
 */
public void embedIcon(JCTextStyle style, Icon icon) throws EndOfFrameException {

    Graphics2D          gc;
    ImageRender         render;
    JCUnit.Dimension    size;

    // Make sure we have a style for every frame that has something in it
    info.currentTextStyle = style;

    // Compute a default size from the icon assuming 1 pixel = 1 point
    size = new JCUnit.Dimension(JCUnit.POINTS, icon.getIconWidth(),
                                icon.getIconHeight());

    // Create a render object (centre-aligned)
    gc = (Graphics2D) document.getPrinterGraphics();
    render = new ImageRender((Object) icon, size);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render, true));

    // add the image to the current line (checking for fit, etc.)
    flowPrint(gc, render, false);
}

/**
 * Adds an icon to the document as part of the current line (the icon may
 * wrap if there isn't enough horizontal space).  The height of the current
 * line will be adjusted to accomodate the icon.  This form of the
 * <code>embedIcon()</code> method assumes a default size based on the pixel size of the
 * icon.
 * <br>This method only accepts instances of the <code>ImageIcon</code> class.
 * @param icon the Icon to be embedded in the current line
 * @param alignment the vertical alignment of the icon on the current line
 * @param style
 * @throws EndOfFrameException
 */
public void embedIcon(JCTextStyle style, Icon icon, int alignment)
                      throws EndOfFrameException {

    Graphics2D          gc;
    ImageRender         render;
    JCUnit.Dimension    size;

    // Make sure we have a style for every frame that has something in it
    info.currentTextStyle = style;

    // Compute a default size from the icon assuming 1 pixel = 1 point
    size = new JCUnit.Dimension(JCUnit.POINTS, icon.getIconWidth(),
                                icon.getIconHeight());

    // Create a render object
    gc = (Graphics2D) document.getPrinterGraphics();
    render = new ImageRender((Object) icon, size);
    render.setVerticalAlignment(alignment);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render, true));

    // add the image to the current line (checking for fit, etc.)
    flowPrint(gc, render, false);
}

/**
 * Adds an icon to the document as part of the current line (the icon may
 * wrap if there isn't enough horizontal space).  The height of the current
 * line will be adjusted to accomodate the icon.  This form of the
 * <code>embedIcon()</code> method centres the icon vertically on the current line.
 * <br>This method only accepts instances of the <code>ImageIcon</code> class.
 * @param icon the Icon to be embedded in the current line
 * @param size the dimensions of the icon as it is to appear on the page
 * @param style
 * @throws EndOfFrameException
 */
public void embedIcon(JCTextStyle style, Icon icon, JCUnit.Dimension size)
                      throws EndOfFrameException {

    Graphics2D          gc;
    ImageRender         render;

    // Make sure we have a style for every frame that has something in it
    info.currentTextStyle = style;

    // Create a render object (centre-aligned)
    gc = (Graphics2D) document.getPrinterGraphics();
    render = new ImageRender((Object) icon, size);
    render.setScaleImage(!document.getPrinter().isImageScalable());

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render, true));

    // add the image to the current line (checking for fit, etc.)
    flowPrint(gc, render, false);
}

/**
 * Adds an icon to the document as part of the current line (the icon may
 * wrap if there isn't enough horizontal space).  The height of the current
 * line will be adjusted to accomodate the icon.
 * <br>This method only accepts instances of the <code>ImageIcon</code> class.
 * @param icon the Icon to be embedded in the current line
 * @param alignment the vertical alignment of the icon on the current line
 * @param size the dimensions of the icon as it is to appear on the page
 * @param style
 * @throws EndOfFrameException
 */
public void embedIcon(JCTextStyle style, Icon icon, int alignment,
                      JCUnit.Dimension size) throws EndOfFrameException {

    Graphics2D          gc;
    ImageRender         render;

    // Make sure we have a style for every frame that has something in it
    info.currentTextStyle = style;

    // Create a render object
    gc = (Graphics2D) document.getPrinterGraphics();
    render = new ImageRender((Object) icon, size);
    render.setScaleImage(!document.getPrinter().isImageScalable());
    render.setVerticalAlignment(alignment);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render, true));

    // add the image to the current line (checking for fit, etc.)
    flowPrint(gc, render, false);
}

/**
 * Adds a component's drawn image to the document as part of the current line
 * (the component's image may wrap if there isn't enough horizontal space).
 * The height of the current line will be adjusted to accomodate the component
 * drawing.  This form of the <code>embedComponent()</code> method assumes a default size
 * based on the pixel size of the component from its current geometry, and
 * centres the component vertically on the current line.
 * @param c the Component to be embedded in the current line
 * @param style
 * @throws EndOfFrameException
 */
public void embedComponent(JCTextStyle style, Component c)
                           throws EndOfFrameException {

    Graphics2D          gc;
    ImageRender         render;
    JCUnit.Dimension    size;

    // Make sure we have a style for every frame that has something in it
    info.currentTextStyle = style;

    gc = (Graphics2D) document.getPrinterGraphics();
    size = sizeComponent(c, gc, 0, 0);

    // Create a render object (entre-aligned)
    render = new ImageRender((Object) c, size);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render, true));

    // add the image to the current line (checking for fit, etc.)
    flowPrint(gc, render, false);
}

/**
 * Adds a component's drawn image to the document as part of the current line
 * (the component image may wrap if there isn't enough horizontal space).
 * The height of the current line will be adjusted to accomodate the component
 * drawing.  This form of the <code>embedComponent()</code> method assumes a default size
 * based on the pixel size of the component from its current geometry.
 * @param c the component to be embedded in the current line
 * @param alignment the vertical alignment of the component on the current line
 * @param style
 * @throws EndOfFrameException
 */
public void embedComponent(JCTextStyle style, Component c, int alignment)
                           throws EndOfFrameException {

    Graphics2D          gc;
    ImageRender         render;
    JCUnit.Dimension    size;

    // Make sure we have a style for every frame that has something in it
    info.currentTextStyle = style;

    gc = (Graphics2D) document.getPrinterGraphics();
    size = sizeComponent(c, gc, 0, 0);

    // Create a render object (centre-aligned)

    render = new ImageRender((Object) c, size);
    render.setVerticalAlignment(alignment);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render, true));

    // add the image to the current line (checking for fit, etc.)
    flowPrint(gc, render, false);

}

/**
 * Adds a component's drawn image to the document as part of the current line
 * (the component image may wrap if there isn't enough horizontal space).
 * The height of the current line will be adjusted to accomodate the component
 * drawing.  This form of the <code>embedComponent()</code> method centres the component
 * vertically on the current line.
 * @param c the component to be embedded in the current line
 * @param size the dimensions of the component as it is to appear on the page
 * @param style
 * @throws EndOfFrameException
 */
public void embedComponent(JCTextStyle style, Component c,
                           JCUnit.Dimension size) throws EndOfFrameException {

    Graphics2D          gc;
    ImageRender         render;

    // Make sure we have a style for every frame that has something in it
    info.currentTextStyle = style;

    // Create a render object (centre-aligned)
    gc = (Graphics2D) document.getPrinterGraphics();
    sizeComponent(c, gc,
        (int) JCUnit.getAsPoints(size.units, size.width),
        (int) JCUnit.getAsPoints(size.units, size.height));
    render = new ImageRender((Object) c, size);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render, true));

    // add the image to the current line (checking for fit, etc.)
    flowPrint(gc, render, false);
}

/**
 * Adds a component's drawn image to the document as part of the current line
 * (the component image may wrap if there isn't enough horizontal space).
 * The height of the current line will be adjusted to accomodate the component
 * drawing.
 * @param c the component to be embedded in the current line
 * @param alignment the vertical alignment of the component on the current line
 * @param size the dimensions of the component as it is to appear on the page
 * @param style
 * @throws EndOfFrameException
 */
public void embedComponent(JCTextStyle style, Component c, int alignment,
                           JCUnit.Dimension size) throws EndOfFrameException {

    Graphics2D          gc;
    ImageRender         render;

    // Make sure we have a style for every frame that has something in it
    info.currentTextStyle = style;

    // Create a render object
    gc = (Graphics2D) document.getPrinterGraphics();
    sizeComponent(c, gc,
        (int) JCUnit.getAsPoints(size.units, size.width),
        (int) JCUnit.getAsPoints(size.units, size.height));
    render = new ImageRender((Object) c, size);
    render.setVerticalAlignment(alignment);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render, true));

    // add the image to the current line (checking for fit, etc.)
    flowPrint(gc, render, false);
}

/**
 * Sets up the size of a component that has been added to the document.
 * If the component has a size, it is kept. If it doesn't, either
 * the defaults are used (width, height) specified in this method's
 * parameters or, if they are also zero, the component's
 * preferred size is used.
 * @return
 * @param c
 * @param gc
 * @param height
 * @param width
 */
protected JCUnit.Dimension sizeComponent(Component c,
                                    Graphics gc, int width, int height)
{
    

    
    int comp_width = c.getWidth();
    int comp_height = c.getHeight();


    // if chart size not set, try chart's preferred size
    if (comp_width == 0 || comp_height == 0) {
        if (!c.isShowing() && c instanceof ServerRenderable) {
            ServerRenderable  sr = (ServerRenderable) c;
            sr.setGraphics(gc);
        }
        Dimension dim;
		// Get preferred size of JComponent objects from our own
		// method that ensures JLabels and JButtons use the correct
		// printer font metrics instead of the screen font metrics
		// Note: this will have the unhappy side effect of ignoring
		// user set preferred sized.  The workaround for that case
		// is to have the user call setSize(), not setPreferredSize()
		if (c instanceof JComponent) {
			dim = JCSwingUtilities.getPreferredSizeFromGraphics(
											(JComponent)c, gc, false);
		} else {
        	dim = c.getPreferredSize();
		}
        if (width == 0) {
            comp_width = dim.width;
        } else {
            comp_width = width;
        }
        if (height == 0) {
            comp_height = dim.height;
        } else {
            comp_height = height;
        }
        c.setSize(comp_width, comp_height);
    }
    // Compute a default size from the component assuming 1 pixel = 1 point
    return new JCUnit.Dimension(JCUnit.POINTS, comp_width, comp_height);
}

/**
 * Adds an Encapsulated PostScript image to the document as part of the current
 * line (the eps image may wrap if there isn't enough horizontal space).  The
 * height of the current line will be adjusted to accomodate the image.  This
 * form of the <code>embedEPS()</code> method centres the eps image vertically on the
 * current line and assumes a size has been provided to the eps image object.
 * @param eps the EPS image to be embedded in the current line
 * @param style
 * @throws EndOfFrameException
 */
public void embedEPS(JCTextStyle style, EPSImage eps)
                     throws EndOfFrameException {

    double              width, height;
    Graphics2D          gc;
    ImageRender         render;
    JCUnit.Dimension    size;

    // Make sure we have a style for every frame that has something in it
    info.currentTextStyle = style;

    // Compute a default size from the image assuming 1 pixel = 1 point
    size = eps.getSize();
    width = JCUnit.getAsPoints(size.units, size.width);
    height = JCUnit.getAsPoints(size.units, size.height);
    size = new JCUnit.Dimension(JCUnit.POINTS, width, height);

    // Create a render object (centre-aligned)
    gc = (Graphics2D) document.getPrinterGraphics();
    render = new ImageRender((Object) eps, size);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render, true));

    // add the image to the current line (checking for fit, etc.)
    flowPrint(gc, render, false);
}

/**
 * Adds an Encapsulated PostScript image to the document as part of the current
 * line (the eps image may wrap if there isn't enough horizontal space).  The
 * height of the current line will be adjusted to accomodate the image.  This
 * form of the <code>embedEPS()</code> method assumes a size has been provided to the eps
 * image object.
 * @param eps the EPS image to be embedded in the current line
 * @param alignment the vertical alignment of the image on the current line
 * @param style
 * @throws EndOfFrameException
 */
public void embedEPS(JCTextStyle style, EPSImage eps, int alignment)
                     throws EndOfFrameException {

    double              width, height;
    Graphics2D          gc;
    ImageRender         render;
    JCUnit.Dimension    size;

    // Make sure we have a style for every frame that has something in it
    info.currentTextStyle = style;

    // Compute a default size from the image assuming 1 pixel = 1 point
    size = eps.getSize();
    width = JCUnit.getAsPoints(size.units, size.width);
    height = JCUnit.getAsPoints(size.units, size.height);
    size = new JCUnit.Dimension(JCUnit.POINTS, width, height);

    // Create a render object
    gc = (Graphics2D) document.getPrinterGraphics();
    render = new ImageRender((Object) eps, size);
    render.setVerticalAlignment(alignment);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render, true));

    // add the image to the current line (checking for fit, etc.)
    flowPrint(gc, render, false);
}

/**
 * Adds an Encapsulated PostScript image to the document as part of the current
 * line (the eps image may wrap if there isn't enough horizontal space).  The
 * height of the current line will be adjusted to accomodate the image.  This
 * form of the <code>embedEPS()</code> method centres the eps image vertically on the
 * current line.
 * @param eps the Image to be embedded in the current line
 * @param size the dimensions of the image as it is to appear on the page
 * @param style
 * @throws EndOfFrameException
 */
public void embedEPS(JCTextStyle style, EPSImage eps, JCUnit.Dimension size)
                     throws EndOfFrameException {

    Graphics2D          gc;
    ImageRender         render;

    // Make sure we have a style for every frame that has something in it
    info.currentTextStyle = style;

    // Create a render object (centre-aligned)
    gc = (Graphics2D) document.getPrinterGraphics();
    render = new ImageRender((Object) eps, size);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render, true));

    // add the image to the current line (checking for fit, etc.)
    flowPrint(gc, render, false);
}

/**
 * Add an Encapsulated PostScript image to the document as part of the current
 * line (the eps image may wrap if there isn't enough horizontal space).  The
 * height of the current line will be adjusted to accomodate the image.
 * @param eps the EPS image to be embedded in the current line
 * @param alignment the vertical alignment of the image on the current line
 * @param size the dimensions of the image as it is to appear on the page
 * @param style
 * @throws EndOfFrameException
 */
public void embedEPS(JCTextStyle style, EPSImage eps, int alignment,
                     JCUnit.Dimension size) throws EndOfFrameException {

    Graphics2D          gc;
    ImageRender         render;

    // Make sure we have a style for every frame that has something in it
    info.currentTextStyle = style;

    // Create a render object
    gc = (Graphics2D) document.getPrinterGraphics();
    render = new ImageRender((Object) eps, size);
    render.setVerticalAlignment(alignment);

    // Store marker object for this image in the render list
    renderList.add(new ImageMarker(render, true));

    // add the image to the current line (checking for fit, etc.)
    flowPrint(gc, render, false);
}

// Image paste functions place an image at a specific location, irrespective
// of the other contents of its frame.  (the image will still be clipped)

/**
 * Pastes an image at the given location in the frame.  This form of the
 * <code>pasteImage()</code> method assumes a default size based on the pixel size of the
 * image.
 * @param image the Image to be pasted into this frame
 * @param origin the point at which to place the image
 */
public void pasteImage(Image image, JCUnit.Point origin) {

    JCUnit.Dimension    size;
    Icon                temp_icon;
    ImageRender         render;

    // Compute a default size from the image assuming 1 pixel = 1 point
    temp_icon = new ImageIcon(image);
    size = new JCUnit.Dimension(JCUnit.POINTS, temp_icon.getIconWidth(),
                                temp_icon.getIconHeight());

    // Create a render object (centre-aligned)
    render = new ImageRender((Object) image, size,
                             JCTextStyle.ALIGNMENT_CENTER);

    // Set the position of the image
    render.setLocation(origin);

    // add the image object to the render list
    renderList.add(render);
}

/**
 * Pastes an image at the given location in the frame.
 * @param image the Image to be pasted into this frame
 * @param size the dimensions of the image as it is to appear on the page
 * @param origin the point at which to place the image
 */
public void pasteImage(Image image, JCUnit.Dimension size,
                       JCUnit.Point origin) {

    ImageRender         render;

    // Create a render object (centre-aligned)
    render = new ImageRender((Object) image, size,
                             JCTextStyle.ALIGNMENT_CENTER);

    // Set the position of the image
    render.setLocation(origin);

    // add the image object to the render list
    renderList.add(render);
}

/**
 * Pastes an icon at the given location in the frame.  This form of the
 * <code>pasteIcon()</code> method assumes a default size based on the pixel size of the
 * icon.
 * @param icon the Icon to be pasted into this frame
 * @param origin the point at which to place the icon
 */
public void pasteIcon(Icon icon, JCUnit.Point origin) {


    JCUnit.Dimension    size;
    ImageRender         render;

    // Compute a default size from the image assuming 1 pixel = 1 point
    size = new JCUnit.Dimension(JCUnit.POINTS, icon.getIconWidth(),
                                icon.getIconHeight());

    // Create a render object (centre-aligned)
    render = new ImageRender((Object) icon, size, JCTextStyle.ALIGNMENT_CENTER);

    // Set the position of the image
    render.setLocation(origin);

    // add the image object to the render list
    renderList.add(render);
}

/**
 * Pastes an icon at the given location in the frame.
 * @param icon the Icon to be pasted into this frame
 * @param size the dimensions of the icon as it is to appear on the page
 * @param origin the point at which to place the icon
 */
public void pasteIcon(Icon icon, JCUnit.Dimension size, JCUnit.Point origin) {

    ImageRender         render;

    // Create a render object (centre-aligned)
    render = new ImageRender((Object) icon, size, JCTextStyle.ALIGNMENT_CENTER);

    // Set the position of the image
    render.setLocation(origin);

    // add the image object to the render list
    renderList.add(render);
}

/**
 * Pastes the drawn image of a component at the given location in the frame.
 * This form of the <code>pasteComponent()</code> method assumes a default size based on
 * the pixel size of the component from its current geometry.
 * @param c the component to be pasted into this frame
 * @param origin the point at which to place the component
 */
public void pasteComponent(Component c, JCUnit.Point origin) {

    JCUnit.Dimension    size;
    ImageRender         render;
    Graphics2D            gc;

    // first ensure the component has a size
    gc = (Graphics2D) document.getPrinterGraphics();
    sizeComponent(c, gc, 0, 0);
    // Compute a default size from the component assuming 1 pixel = 1 point
    

    
    size = new JCUnit.Dimension(JCUnit.POINTS, c.getWidth(), c.getHeight());


    // Create a render object (centre-aligned)
    render = new ImageRender((Object) c, size, JCTextStyle.ALIGNMENT_CENTER);

    // Set the position of the image
    render.setLocation(origin);

    // add the image object to the render list
    renderList.add(render);
}

/**
 * Pastes the drawn image of a component at the given location in the frame.
 * @param c the component to be pasted into this frame
 * @param size the dimensions of the component as it is to appear on the page
 * @param origin the point at which to place the component
 */
public void pasteComponent(Component c, JCUnit.Dimension size,
                           JCUnit.Point origin) {

    ImageRender         render;
    Graphics2D            gc;

    // first ensure the component has a size
    gc = (Graphics2D) document.getPrinterGraphics();
    sizeComponent(c, gc, 
        (int) JCUnit.getAsPoints(size.units, size.width),
        (int) JCUnit.getAsPoints(size.units, size.height));

    // Create a render object
    render = new ImageRender((Object) c, size, JCTextStyle.ALIGNMENT_CENTER);

    // Set the position of the image
    render.setLocation(origin);

    // add the image object to the render list
    renderList.add(render);
}

/**
 * Pastes an Encapsulated PostScript image at the given location in the frame.
 * This form of the <code>pasteEPS()</code> method assumes a size has been provided to the
 * eps image object.
 * @param eps the <code>EPSImage</code> to be pasted into this frame
 * @param origin the point at which to place the eps image
 */
public void pasteEPS(EPSImage eps, JCUnit.Point origin) {

    JCUnit.Dimension    size;
    ImageRender         render;
    double              width, height;

    // Compute a default size from the eps image assuming 1 pixel = 1 point
    size = eps.getSize();
    width = JCUnit.getAsPoints(size.units, size.width);
    height = JCUnit.getAsPoints(size.units, size.height);
    size = new JCUnit.Dimension(JCUnit.POINTS, width, height);

    // Create a render object (centre-aligned)
    render = new ImageRender((Object) eps, size, JCTextStyle.ALIGNMENT_CENTER);

    // Set the position of the image
    render.setLocation(origin);

    // add the image object to the render list
    renderList.add(render);
}

/**
 * Pastes an Encapsulated PostScript image at the given location in the frame.
 * @param eps the <code>EPSImage</code> to be pasted into this frame
 * @param size the dimensions of the eps image as it is to appear on the page
 * @param origin the point at which to place the eps image
 */
public void pasteEPS(EPSImage eps, JCUnit.Dimension size, JCUnit.Point origin) {

    ImageRender         render;

    // Create a render object
    render = new ImageRender((Object) eps, size, JCTextStyle.ALIGNMENT_CENTER);

    // Set the position of the image
    render.setLocation(origin);

    // add the image object to the render list
    renderList.add(render);
}

/**
 * Draws a horizontal line on a new line in the current column.
 * @param style a text style defining the column positioning (indents, spacing)
 * @param dstyle a drawing style which defines the rule's appearance
 * @param percent_width the percentage of the width of the column the rule is wide
 * @throws EndOfFrameException
 */
public void hRule(JCTextStyle style, JCDrawStyle dstyle,
                  double percent_width) throws EndOfFrameException {

    JCUnit.Measure  length;

    // Make sure we've got an immutable style
    style = style.getImmutable();
    dstyle = dstyle.getImmutable();

    // Compute the length of the line
    length = (JCUnit.Measure) info.rightOfColumn.clone();
    length.subtract(info.leftOfColumn);
    length.multiply(percent_width / 100.0);

    hRule(style, dstyle, length);
}

/**
 * Draws a horizontal line on a new line in the current column.
 * @param style a text style defining the column positioning (indents, spacing)
 * @param dstyle a drawing style which defines the rule's appearance
 * @param rule_length the length of the rule
 * @throws EndOfFrameException
 */
public void hRule(JCTextStyle style, JCDrawStyle dstyle,
                  JCUnit.Measure rule_length) throws EndOfFrameException
{
    JCUnit.Measure  x0, x1, y, end_y;
    JCUnit.Measure  line_width, bottom;
    JCUnit.Point    p1, p2;

    renderList.add(new HRuleMarker(style, dstyle, rule_length));
    // Make sure we've got an immutable style
    style = style.getImmutable();
    dstyle = dstyle.getImmutable();

    // Make sure the frame has the layout constraints initialised
    if (!isInfoInitialised()) {
        initialiseInfo();
    }

    // End the current line
    if (info.currentLine.size() != 0) {
        doLineBreak(style, false);
    }

    // Compute the start and end coordinates of the line
    switch (style.getAlignment()) {

    case JCTextStyle.ALIGNMENT_RIGHT:
        x1 = (JCUnit.Measure) info.rightOfColumn.clone();
        x0 = (JCUnit.Measure) info.rightOfColumn.clone();
        x0.subtract(rule_length);
        if (x0.lessThan(info.leftOfColumn)) {
            x0 = (JCUnit.Measure) info.leftOfColumn.clone();
        }
        break;

    case JCTextStyle.ALIGNMENT_CENTER:
    case JCTextStyle.ALIGNMENT_JUSTIFY:
        x0 = (JCUnit.Measure) info.leftOfColumn.clone();
        x0.add(info.rightOfColumn);
        x0.subtract(rule_length);
        x0.divide(2.0);
        x1 = (JCUnit.Measure) x0.clone();
        x1.add(rule_length);
        if (x0.lessThan(info.leftOfColumn)) {
            x0 = (JCUnit.Measure) info.leftOfColumn.clone();
        }
        if (x1.greaterThan(info.rightOfColumn)) {
            x1 = (JCUnit.Measure) info.rightOfColumn.clone();
        }
        break;

    case JCTextStyle.ALIGNMENT_NONE:
    case JCTextStyle.ALIGNMENT_LEFT:
        x0 = (JCUnit.Measure) info.leftOfColumn.clone();
        x1 = (JCUnit.Measure) x0.clone();
        x1.add(rule_length);
        if (x1.greaterThan(info.rightOfColumn)) {
            x1 = (JCUnit.Measure) info.rightOfColumn.clone();
        }
        break;

    default:
        throw new IllegalArgumentException("Invalid text alignment");
    }
    y = info.insertion.getY();

    // Make sure the line will fit
    line_width = dstyle.getDrawnWidth();
    end_y = (JCUnit.Measure) line_width.clone();
    end_y.multiply(2.0);
    end_y.add(y);

    bottom = this.size.getHeight();
    bottom.subtract(this.margin.getBottom());

    // if there is insufficient space, move to the next column
    if (end_y.greaterThan(bottom)) {
        newColumn(style, false);
        y = info.insertion.getY();
    }

    // compute the vertical position of the line
    line_width.multiply(1.5);
    y.add(line_width);

    // draw the line
    p1 = new JCUnit.Point(x0.units, x0.distance, 0.0);
    p1.setY(y);
    p2 = new JCUnit.Point(x1.units, x1.distance, 0.0);
    p2.setY(y);
    drawLine(dstyle, p1, p2);

    // Move to the start of the next line
    JCUnit.Measure pad_spacing = new JCUnit.Measure(JCUnit.POINTS, 2.0);
    if (line_width.lessThan(pad_spacing))
        y.add(pad_spacing);
    else
        y.add(line_width);
    info.insertion.setY(y);
}

/**
 * Advances to a new line (hard line break).
 * @param style the current style which determines the required line spacing
 */
public void newLine(JCTextStyle style) {

    SymbolRender    newline_symbol;
    JCUnit.Measure  new_x;

    // Create a newline symbol and add to the current line
    newline_symbol = new SymbolRender(SymbolRender.NEWLINE_SYMBOL, style);

    // Locate the symbol on the line
    newline_symbol.setLocation(info.insertion);

    // Add the symbol to the current line
    info.currentLine.add(newline_symbol);

    // Paste the current line and advance to the next one
    doLineBreak(style, false);
}

/**
 * Advances to a new line (starting a new paragraph).
 * @param style the current style which determines the required line spacing
 */
public void newParagraph(JCTextStyle style) {

    SymbolRender    newline_symbol;
    JCUnit.Measure  new_x;

    // Create a newline symbol and add to the current line
    newline_symbol = new SymbolRender(SymbolRender.NEWPARAGRAPH_SYMBOL, style);

    // Locate the symbol on the line
    newline_symbol.setLocation(info.insertion);

    // Add the symbol to the current line
    info.currentLine.add(newline_symbol);

    // Paste the current line and advance to the next one
    doLineBreak(style, true);
}

/**
 * Advances to a new line (pasting down the current line).
 * @deprecated Use <code>newLine</code>(<code>JCTextStyle</code>) or
 * <code>newParagraph</code>(<code>JCTextStyle</code>) instead
 * @param style the current style which determines the required line spacing
 * @param paragraph <code>true</code> if the next line begins a new paragraph
 */
protected void newLine(JCTextStyle style, boolean paragraph) {

    // Call the method that should have been called in the first place
    if (paragraph) {
        newParagraph(style);
    }
    else {
        newLine(style);
    }
}

/**
 * Advances to a new line (pasting down the current line).
 * @param style the current style which determines the required line spacing
 * @param paragraph <code>true</code> if the next line begins a new paragraph
 */
protected void doLineBreak(JCTextStyle style, boolean paragraph) {
    //PR22514
    doLineBreak(style, paragraph, false);
}

/**
 * Advances to a new line (pasting down the current line).
 * @param style the current style which determines the required line spacing
 * @param paragraph <code>true</code> if the next line begins a new paragraph
 * @param endOfFrame <code>true</code> if this is the last line of the frame
 */
protected void doLineBreak(JCTextStyle style, boolean paragraph, boolean endOfFrame) {
    int             i, j, count, space_count;
    int             alignment;
    Iterator        lineList;
    Render          render, item;
    JCUnit.Point    position;
    JCUnit.Measure  new_x, new_y;
    JCUnit.Measure  space, current_space;

    // Make sure the frame has been initialised
    if (!isInfoInitialised()) {
        initialiseInfo();
    }

    // Make sure we've got an immutable style
    style = style.getImmutable();

    // Get the paragraph alignment specified by the style
    alignment = style.getAlignment();

    // We don't obey the alignment if there are tabs in the line
    if (info.tabUsed) {
        alignment = JCTextStyle.ALIGNMENT_NONE;
    }

    

    // if alignment requires repositioning, trim end of line whitespace
    count = info.currentLine.size();
    if (count > 0 && (alignment == JCTextStyle.ALIGNMENT_CENTER
                      || alignment == JCTextStyle.ALIGNMENT_RIGHT
                      || alignment == JCTextStyle.ALIGNMENT_JUSTIFY)) {

        // Get the last item on the line and check if it is a string
        item = (Render) info.currentLine.get(count - 1);

        if (item instanceof StringRender) {

            // Trim trailing whitespace and update the insertion point
            ((StringRender) item).trimTrailingWhitespace();
            new_x = item.getLocation().getX();
            new_x.add(item.getSize().getWidth());
            info.insertion.setX(new_x);
        }
    }

    // Compute the amount of space on the line
    space = (JCUnit.Measure) info.rightOfColumn.clone();
    space.subtract(info.insertion.getX());
    space.subtract(style.getRightIndent());

    // We don't do any adjustments if there is no extra space
    if (space.isZero() || space.isNegative()) {
        alignment = JCTextStyle.ALIGNMENT_NONE;
    }

    // We don't justify the last line of a paragraph
    if (paragraph && alignment == JCTextStyle.ALIGNMENT_JUSTIFY) {
        alignment = JCTextStyle.ALIGNMENT_NONE;
    }

    // Adjust the line depending on the alignment
    switch (alignment) {

    // For centre and right alignment we shift the items on the line
    case JCTextStyle.ALIGNMENT_RIGHT:
    case JCTextStyle.ALIGNMENT_CENTER:

        
        // Centre-aligning moves by half the available space
        if (alignment == JCTextStyle.ALIGNMENT_CENTER) {
            space.divide(2.0);
        }

        // Loop through the line and move all the objects
        for (i = 0; i < count; i++) {

            // Get the position of this item
            item = (Render) info.currentLine.get(i);
            position = item.getLocation();

            // Adjust the horizontal location by the required space
            new_x = position.getX();
            new_x.add(space);
            position.setX(new_x);
        }
        break;

    // To justify (align both sides) the line, we need to split the text...
    case JCTextStyle.ALIGNMENT_JUSTIFY:

        
        
        space_count = 0;

        // Loop through the line, splitting text into words
        for (i = count - 1; i >= 0; i--) {

            // Get the next item and check its type
            item = (Render) info.currentLine.get(i);

            // Count all individual visual objects on the line
            if (item instanceof StringRender) {
                
                

                // NOTE:  This should probably use isAtomic() and the Splitable
                //        interface; i.e. splitIntoWords() should be added
                // Break the text into words
                List wordList = ((StringRender) item).splitIntoWords();
                space_count += wordList.size();

                // Copy the words into the current line
                for (j = 0; j < wordList.size(); j++) {
                    info.currentLine.insertElementAt(wordList.get(j), i + 1);

                    
                    
                }

                // Delete the original text from the line
                info.currentLine.remove(i);
            }
            else if (!(item instanceof SymbolRender)) {
                space_count++;
            }
        }

        

        // Update the (possibly changed) number of objects on the line
        count = info.currentLine.size();

        // Don't do anything if the line has no graphical objects
        if (space_count == 0) {
            
            break;
        }

        // Check for the special case of only one object on the line (centre it)
        if (space_count == 1) {
            
            space.divide(2.0);

            // Find the graphical render object
            for (i = 0; i < count; i++) {

                // Get the next render object
                item = (Render) info.currentLine.get(i);

                // if it's printable, center it
                if (!(item instanceof SymbolRender)) {

                    // Adjust the horizontal location by the required space
                    position = item.getLocation();
                    new_x = position.getX();
                    new_x.add(space);
                    position.setX(new_x);
                }
            }
            break;
        }

        // Determine the amount of space per object
        space.divide(space_count - 1);
        current_space = (JCUnit.Measure) space.clone();
        

        // Loop through the line, adjusting objects
        boolean first_text = true;
        for (i = 0; i < count; i++) {

            // Get the next render object
            item = (Render) info.currentLine.get(i);

            // if it's printable, adjust its position
            if (!(item instanceof SymbolRender)) {

                // We skip the first object on the line
                if (first_text) {
                    first_text = false;
                    continue;
                }

                
                

                // Adjust the horizontal location by the required space
                position = item.getLocation();
                new_x = position.getX();
                new_x.add(current_space);
                
                position.setX(new_x);

                // Update the current space amount
                current_space.add(space);
            }
        }
        
        break;

    // No adjustments are needed for left or no alignment
    case JCTextStyle.ALIGNMENT_NONE:
    case JCTextStyle.ALIGNMENT_LEFT:
        
        break;
    }

    
    // Paste down each element on the current line
    lineList = info.currentLine.iterator();
    while (lineList.hasNext()) {

        // Add the element to the frame's render list
        render = (Render) lineList.next();
        renderList.add(render);
    }

    

    // Clear the current line and move to the next
    info.currentLine.removeAllElements();
    computeLineBounds(style, paragraph);
    info.insertion.setX(info.lineBegin);

    // If there is an embedded image requiring additional space, honour it
    /*
    if (info.embedDescent.greaterThan(info.descender)) {
        info.lineSpacing.subtract(info.descender);
        info.lineSpacing.add(info.embedDescent);
    }
    */

    // Determine the amount to advance to the next line
    

    // PR 25362 -- line/paragraph spacing as percentage of point size
    // should only be applied if not on the last line of the frame
    // The last line of the frame should start from a zero offset
    // since there are no more lines coming after and the size of
    // the frame will be calculated based on the new insertion point.
    if (!endOfFrame || info.floatList.size() > 0) {
    	new_y = new JCUnit.Measure(JCUnit.POINTS, style.getPointSize());
    	// adjust the line spacing based on depending on
		// whether it's ending a paragraph or not
    	if (paragraph) {
        	new_y.multiply(style.getParagraphSpacing() - 1);
        	
    	}
    	else {
        	new_y.multiply(style.getLineSpacing() - 1);
        	
    	}
    } else {
        new_y = new JCUnit.Measure(JCUnit.POINTS, 0);
    }

    // If there are no contents on the line, lineSpacing will be zero
    if (info.lineSpacing.isZero()) {

        // Use the point size of the font instead to size the line
        double point_size = style.getPointSize();
        new_y.add(new JCUnit.Measure(JCUnit.POINTS, point_size));

        // Compute leading based on the font information
        Graphics2D gc = (Graphics2D) document.getPrinterGraphics();
        FontMetrics fm = gc.getFontMetrics(style.getFont());

        point_size -= (fm.getAscent() + fm.getDescent());

        // Since AWT screws up the size of the font, check for negatives
        if (point_size <= 0) {
            point_size = 1;
        }
        info.leading = new JCUnit.Measure(JCUnit.POINTS, point_size);
    }
    else {
        new_y.add(info.lineSpacing); // affected by line items
    }
    //PR22514 don't add info.leading if endOfFrame.  On the other hand,
	// if endOfFrame but there are still items in the float list,
	// add the info.leading as a buffer between the line and the floated
	// images.
    if (!endOfFrame || info.floatList.size() > 0) {
        new_y.add(info.leading);     // computed from the text height
    }
    

    // Advance the insertion point by this amount
    new_y.add(info.insertion.getY());

    info.insertion.setY(new_y);

    // Set the initial line spacing for the next line
    info.lineSpacing = new JCUnit.Measure(JCUnit.POINTS);
    info.ascender = new JCUnit.Measure(JCUnit.POINTS);
    info.descender = new JCUnit.Measure(JCUnit.POINTS);
    info.leading = new JCUnit.Measure(JCUnit.POINTS);
    info.height = new JCUnit.Measure(JCUnit.POINTS);
    info.embedAscent = new JCUnit.Measure(JCUnit.POINTS);
    info.embedDescent = new JCUnit.Measure(JCUnit.POINTS);

    // paste floating objects (may have floated an object in the last line)
    pasteFloats();

    // Unset any current tab
    info.currentTab = null;
    info.tabSymbol = null;
    info.tabIndex = -1;
    info.tabUsed = false;
}

/**
 * Adjusts insertion.x and <code>lineEnd</code> for the new line.
 * @param style the style that specifies the indentation
 * @param paragraph indicates if this line begins a paragraph
 */
private void computeLineBounds(JCTextStyle style, boolean paragraph) {

    
    

    
    
    

    // Compute the start and end points of the line
    info.lineBegin.assign(info.leftOfColumn);
    info.lineEnd.assign(info.rightOfColumn);

    // Compute the right edge of the printable area
    info.lineEnd.subtract(style.getRightIndent());

    // If we're starting a new paragraph we need paragraph indentation
    if (paragraph) {
        info.lineBegin.add(style.getParagraphIndent());
        info.firstLine = true;
    }
    // otherwise the paragraph is continuing so we use body indentation
    else {
        info.lineBegin.add(style.getLeftIndent());
        info.firstLine = false;
    }

    // If there is no printable area on the line, ignore the indentation
    if (!info.lineBegin.lessThan(info.lineEnd)) {

        
        info.lineBegin.assign(info.leftOfColumn);
        info.lineEnd.assign(info.rightOfColumn);
    }

    
}

/**
 * Moves the insertion point to the top of the next column.
 * @param style the current style which determines the required line spacing
 * @throws EndOfFrameException
 */
public void newColumn(JCTextStyle style) throws EndOfFrameException {

    newColumn(style, true);
}

/**
 * Moves the insertion point to the top of the next column.
 * @param style the current style which determines the required line spacing
 * @param newLine
 * @throws EndOfFrameException
 */
private void newColumn(JCTextStyle style, boolean newLine)
             throws EndOfFrameException {

    int             i, count;
    Render          item;
    JCUnit.Point    item_location;
    JCUnit.Measure  new_x, new_y, baseline;

    // if we have to flush the current line before continuing, do so
    if (newLine) {
        newLine(style);
    }

    // Compute the baseline of the line
    baseline = info.insertion.getY();

    // translate the position of the current line
    count = info.currentLine.size();
    for (i = 0; i < count; i++) {
        item = (Render) info.currentLine.get(i);

        item_location = item.getLocation();
        new_x = item_location.getX();
        new_x.subtract(info.leftOfColumn);
        new_y = item_location.getY();
        new_y.subtract(baseline);

        item_location.setX(new_x);
        item_location.setY(new_y);

        // Also adjust tab leader start and end
        if (item instanceof SymbolRender) {
            SymbolRender tabItem = (SymbolRender) item;

            if (tabItem.getType() == SymbolRender.TAB_SYMBOL) {
                tabItem.getLeaderStart().subtract(info.leftOfColumn);
                tabItem.getLeaderEnd().subtract(info.leftOfColumn);
            }
        }
    }

    // If we're already in the last column, throw an exception
    if (info.currentColumn + 1 == columnCount) {

        
        EndOfFrameException e = new EndOfFrameException(info,
                                                        "Frame end reached");
        // Clear the current line of the old frame
        info.currentLine.clear();
        throw e;
    }

    // Store the old insertion point
    new_x = info.insertion.getX();
    new_x.subtract(info.leftOfColumn);

    // Move to the next column;
    info.currentColumn++;
    

    // Compute the positions of the column edges
    info.leftOfColumn.assign(columnWidth);
    info.leftOfColumn.add(columnSpace);
    info.leftOfColumn.multiply(info.currentColumn);
    info.leftOfColumn.add(margin.getLeft());

    info.rightOfColumn.assign(info.leftOfColumn);
    info.rightOfColumn.add(columnWidth);

    // Compute the beginning and ending of the useable part of the line
    computeLineBounds(style, info.firstLine);
    info.insertion.setX(info.lineBegin);

    new_y = getFirstLineOffset(style);
    new_y.add(margin.getTop());

    // Set the new insertion point
    new_x.add(info.leftOfColumn);
    info.insertion.setX(new_x);
    info.insertion.setY(new_y);

    // paste floating objects
    pasteFloats();

    // Recompute the new baseline of the line
    baseline = info.insertion.getY();

    // translate to the new position of the current line
    count = info.currentLine.size();
    for (i = 0; i < count; i++) {
        item = (Render) info.currentLine.get(i);

        item_location = item.getLocation();
        new_x = item_location.getX();
        new_x.add(info.leftOfColumn);
        new_y = item_location.getY();
        new_y.add(baseline);

        item_location.setX(new_x);
        item_location.setY(new_y);

        // new position of tab leader start and end
        if (item instanceof SymbolRender) {
            SymbolRender tabItem = (SymbolRender) item;

            if (tabItem.getType() == SymbolRender.TAB_SYMBOL) {
                tabItem.getLeaderStart().add(info.leftOfColumn);
                tabItem.getLeaderEnd().add(info.leftOfColumn);
            }
        }
    }
}

/**
 * Calculates the extra top offset required for the first line of a frame.
 * @param style the current style of text being printed
 * @return the amount of offset to add to the first line of text
 */
public JCUnit.Measure getFirstLineOffset(JCTextStyle style) {

    double          space;
    Graphics2D      gc;
    FontMetrics     fm;

    gc = (Graphics2D) document.getPrinterGraphics();
    fm = gc.getFontMetrics(style.getFont());
    space = style.getPointSize() - fm.getAscent() - fm.getDescent();

    // Since AWT screws up the size of the font, check for negatives
    if (space <= 0) {
        space = 1;
    }
    // space = fm.getHeight() - fm.getAscent() - fm.getDescent();

    
    
    return(new JCUnit.Measure(JCUnit.POINTS, space));
}

/**
 * Returns the point in the frame at which text is being flowed.
 * @return the current insertion point for text and flowed images
 */
public JCUnit.Point getInsertionPoint() {

    return(info.insertion);
}

/**
 * Sets a new location for inserting flowed objects.
 * @param point the new insertion point
 */
public void setInsertionPoint(JCUnit.Point point) {

    if (point == null) {
        return;
    }

    // if we haven't initialized, do it now
    // otherwise, this set will be overwritten by defaults
    if (!isInfoInitialised()) {
        initialiseInfo();
    }

    // if the insertion point is being moved to a different line and the
    // current line already has some content, then we need to paste down
    // the elements in the current line.

    if (!point.getY().equals(info.insertion.getY()) &&
        info.currentLine.size() > 0) {

        // use the default text style if none set
        JCTextStyle tempStyle = info.currentTextStyle;
        if (tempStyle == null) {
            tempStyle = JCTextStyle.DEFAULT_TEXT;
        }

        
        doLineBreak(tempStyle, false);
    }

    // Set the new text insertion point
    
    info.insertion = (JCUnit.Point) point.clone();

    // We do not do paragraph indentation after explicit setting of the
    // insertion point; since the user is specifying the start point, the
    // user can be responsible for computing paragraph insertion as desired
    info.firstLine = false;
}

/**
 * Gets the size of the largest ascender among objects flowed to the current line.
 * @return the line's tallest ascender
 */
public JCUnit.Measure getCurrentAscender() {
    return info.ascender;
}

/**
 * Gets the largest descender among objects flowed to the current line.
 * @return the magnitude of the line's largest descender
 */
public JCUnit.Measure getCurrentDescender() {
    return info.descender;
}

/**
 * Gets the baseline of the current line as adjusted for the ascender height.
 * @return the frame position of the baseline of text in the current line
 */
public JCUnit.Measure getCurrentBaseline() {

    JCUnit.Measure  position;

    position = info.insertion.getY();
    position.add(info.ascender);

    return(position);
}

/**
 * Gets the column position within the frame.
 * @return number of the current column
 */
public int getCurrentColumn() {
    return info.currentColumn;
}

/**
 * Returns the flow structure, which controls the flow in this frame.
 * @return the attributes of the flow in this frame
 */
public JCFlowInfo getFlowInfo() {
    return info;
}

/**
 * Adds a list of objects to the floating objects which are pending.
 * @param floatObjectList a list of objects which float in the flow (for example, images)
 */
public void addFloats(List floatObjectList) {
    info.floatList.addAll(floatObjectList);
}

/**
 * Copies a previous current line to this frame's current line.
 * @param oldCurrentList a list of objects which were the old current line
 * @throws EndOfFrameException
 */
public void setCurrentLine(List oldCurrentList) throws EndOfFrameException {

    Graphics2D          gc;

    // Reflow the elements in the list to the current frame
    gc = (Graphics2D) document.getPrinterGraphics();
    reflow(gc, oldCurrentList);

    // Remove pasted elements from the current list
    oldCurrentList.clear();
}

// Line-drawing graphics functions for simple geometric figures

/**
 * Draws a circle given a centre point and radius.
 * @param style the drawing style used to specify the appearance of the figure
 * @param center the origin of the circle
 * @param radius a measure specifying the radius of the circle
 */
public void drawCircle(JCDrawStyle style, JCUnit.Point center,
                       JCUnit.Measure radius) {

    ArcRender       circle;
    JCUnit.Measure  max_y;

    // Make sure we've got an immutable style
    style = style.getImmutable();

    // sanity-check the input
    if (radius.isNegative()) {
        throw new IllegalArgumentException("Radius must be positive");
    }

    // Create a circle and add it to the render list
    circle = new ArcRender(center, radius, 0, 360, style);
    renderList.add(circle);

    // Track the bounds of the frame
    max_y = style.getDrawnWidth();
    max_y.divide(2.0);
    max_y.add(center.getY());
    max_y.add(radius);

    if (max_y.greaterThan(yExtent)) {
        yExtent.assign(max_y);
    }
}

/**
 * Draws a filled circle given a centre point and radius.
 * @param style the drawing style used to specify the appearance of the figure
 * @param center the origin of the circle
 * @param radius a measure specifying the radius of the circle
 */
public void fillCircle(JCDrawStyle style, JCUnit.Point center,
                       JCUnit.Measure radius) {

    ArcRender       circle;
    JCUnit.Measure  max_y;

    // Make sure we've got an immutable style
    style = style.getImmutable();

    // sanity-check the input
    if (radius.isNegative()) {
        throw new IllegalArgumentException("Radius must be positive");
    }

    // Create a circle and add it to the render list
    circle = new ArcRender(center, radius, 0, 360, style);
    circle.setFilled(true);
    renderList.add(circle);

    // Track the bounds of the frame
    max_y = style.getDrawnWidth();
    max_y.divide(2.0);
    max_y.add(center.getY());
    max_y.add(radius);

    if (max_y.greaterThan(yExtent)) {
        yExtent.assign(max_y);
    }
}

/**
 * Draws a rectangle given a start point and size.
 * @param style the drawing style used to specify the appearance of the figure
 * @param origin the location of the rectangle
 * @param size the dimensions of the rectangle
 */
public void drawRectangle(JCDrawStyle style, JCUnit.Point origin,
                          JCUnit.Dimension size) {

    BoxRender       rectangle;
    JCUnit.Measure  max_y;

    // Make sure we've got an immutable style
    style = style.getImmutable();

    // Create a box and add it to the render list
    rectangle = new BoxRender(origin, size, style);
    renderList.add(rectangle);

    // Track the bounds of the frame
    max_y = style.getDrawnWidth();
    max_y.divide(2.0);
    max_y.add(origin.getY());
    max_y.add(size.getHeight());

    if (max_y.greaterThan(yExtent)) {
        yExtent.assign(max_y);
    }
}

/**
 * Draws a rectangle given a start point and end point.
 * @param style the drawing style used to specify the appearance of the figure
 * @param startPoint the first point of the rectangle
 * @param endPoint the diagonal corner of the rectangle
 */
public void drawRectangle(JCDrawStyle style, JCUnit.Point startPoint,
                          JCUnit.Point endPoint) {

    BoxRender       rectangle;
    JCUnit.Measure  max_y, y1, y2;

    // Make sure we've got an immutable style
    style = style.getImmutable();

    // Create a box and add it to the render list
    rectangle = new BoxRender(startPoint, endPoint, style);
    renderList.add(rectangle);

    // Track the bounds of the frame
    y1 = startPoint.getY();
    y2 = endPoint.getY();

    max_y = style.getDrawnWidth();
    max_y.divide(2.0);
    if (y1.greaterThan(y2)) {
        max_y.add(y1);
    }
    else {
        max_y.add(y2);
    }

    if (max_y.greaterThan(yExtent)) {
        yExtent.assign(max_y);
    }
}

/**
 * Draws a filled rectangle given a start point and size.
 * @param style the drawing style used to specify the appearance of the figure
 * @param origin the location of the rectangle
 * @param size the dimensions of the rectangle
 */
public void fillRectangle(JCDrawStyle style, JCUnit.Point origin,
                          JCUnit.Dimension size) {

    BoxRender       rectangle;
    JCUnit.Measure  max_y;

    // Make sure we've got an immutable style
    style = style.getImmutable();

    // Create a box and add it to the render list
    rectangle = new BoxRender(origin, size, style);
    rectangle.setFilled(true);
    renderList.add(rectangle);

    // Track the bounds of the frame
    max_y = style.getDrawnWidth();
    max_y.divide(2.0);
    max_y.add(origin.getY());
    max_y.add(size.getHeight());

    if (max_y.greaterThan(yExtent)) {
        yExtent.assign(max_y);
    }
}

/**
 * Draws a filled rectangle given a start point and end point.
 * @param style the drawing style used to specify the appearance of the figure
 * @param startPoint the first point of the rectangle
 * @param endPoint the diagonal corner of the rectangle
 */
public void fillRectangle(JCDrawStyle style, JCUnit.Point startPoint,
                          JCUnit.Point endPoint) {

    BoxRender       rectangle;
    JCUnit.Measure  max_y, y1, y2;

    // Make sure we've got an immutable style
    style = style.getImmutable();

    // Create a box and add it to the render list
    rectangle = new BoxRender(startPoint, endPoint, style);
    rectangle.setFilled(true);
    renderList.add(rectangle);

    // Track the bounds of the frame
    y1 = startPoint.getY();
    y2 = endPoint.getY();

    max_y = style.getDrawnWidth();
    max_y.divide(2.0);
    if (y1.greaterThan(y2)) {
        max_y.add(y1);
    }
    else {
        max_y.add(y2);
    }

    if (max_y.greaterThan(yExtent)) {
        yExtent.assign(max_y);
    }
}

/**
 * Draws a line between two given points.
 * @param style the drawing style used to specify the appearance of the figure
 * @param startPoint the first point of the line
 * @param endPoint the end point of the line
 */
public void drawLine(JCDrawStyle style, JCUnit.Point startPoint,
                     JCUnit.Point endPoint) {

    LineRender      line;
    JCUnit.Measure  max_y, y1, y2;

    // Make sure we've got an immutable style
    style = style.getImmutable();

    // Create a box and add it to the render list
    line = new LineRender(startPoint, endPoint, style);
    renderList.add(line);

    // Track the bounds of the frame
    y1 = startPoint.getY();
    y2 = endPoint.getY();

    max_y = style.getDrawnWidth();
    max_y.divide(2.0);
    if (y1.greaterThan(y2)) {
        max_y.add(y1);
    }
    else {
        max_y.add(y2);
    }

    if (max_y.greaterThan(yExtent)) {
        yExtent.assign(max_y);
    }
}

/**
 * Draws a line given a list of points.
 * @param style the drawing style used to specify the appearance of the figure
 * @param pointList a list of points from which to create the line
 */
public void drawLine(JCDrawStyle style, List pointList) {

    int             i, npoints;
    LineRender      line;
    JCUnit.Measure  max_y, y1, y2;
    JCUnit.Point    point;
    JCUnit.Point[]  points;

    // Make sure we've got an immutable style
    style = style.getImmutable();

    // get the number of points in the line and create an array for the points
    npoints = pointList.size();
    points = new JCUnit.Point[npoints];
    max_y = new JCUnit.Measure();

    for (i = 0; i < npoints; i++) {

        // Get each point of the list and add it to the array
        point = (JCUnit.Point) pointList.get(i);
        points[i] = point;

        // accumulate the maximum point
        y1 = point.getY();
        if (y1.greaterThan(max_y)) {
            max_y = y1;
        }
    }

    // Create a line render and add it to the frame's render list
    line = new LineRender(points, style);
    renderList.add(line);

    // Add the line width to the extent
    y2 = style.getDrawnWidth();
    y2.divide(2.0);
    max_y.add(y2);

    // Update the greatest vertical extent of the frame
    if (max_y.greaterThan(yExtent)) {
        yExtent.assign(max_y);
    }
}

/**
 * Draws a line given an array of points.
 * @param style the drawing style used to specify the appearance of the figure
 * @param points an array of points from which to create the line
 */
public void drawLine(JCDrawStyle style, JCUnit.Point[] points) {

    int             i, npoints;
    LineRender      line;
    JCUnit.Measure  max_y, y1, y2;

    // Make sure we've got an immutable style
    style = style.getImmutable();

    // get the number of points in the line
    npoints = points.length;
    max_y = new JCUnit.Measure();

    // Scan the points in the array
    for (i = 0; i < npoints; i++) {

        // accumulate the maximum point
        y1 = points[i].getY();
        if (y1.greaterThan(max_y)) {
            max_y = y1;
        }
    }

    // Create a line render and add it to the frame's render list
    line = new LineRender(points, style);
    renderList.add(line);

    // Add the line width to the extent
    y2 = style.getDrawnWidth();
    y2.divide(2.0);
    max_y.add(y2);

    // Update the greatest vertical extent of the frame
    if (max_y.greaterThan(yExtent)) {
        yExtent.assign(max_y);
    }
}

/**
 * Draws a polygon given a list of points (the sequence of points will
 * automatically be closed).
 * @param style the drawing style used to specify the appearance of the figure
 * @param pointList a list of points from which to create the polygon
 */
public void drawPolygon(JCDrawStyle style, List pointList) {

    int             i, npoints, extraPoints = 0;
    boolean         close = false;
    LineRender      line;
    JCUnit.Measure  max_y, y1, y2;
    JCUnit.Point    point, first, last, second = null;
    JCUnit.Point[]  points;

    // Make sure we've got an immutable style
    style = style.getImmutable();

    // get the number of points in the line
    npoints = pointList.size();

    // if the first and last points differ, add a point to close the line
    first = (JCUnit.Point) pointList.get(0);
    // also re-add the second point to ensure close polygon connection works
    if (npoints > 1)
        second = (JCUnit.Point) pointList.get(1);
    last = (JCUnit.Point) pointList.get(npoints - 1);
    if (!first.equals(last)) {
        close = true;
        if (second != null) {
            extraPoints = 2;
        } else {
            extraPoints = 1;
        }
    } else {
        if (second != null) {
            extraPoints = 1;
        }
    }

    // create an array for the points
    points = new JCUnit.Point[npoints + extraPoints];
    max_y = new JCUnit.Measure();

    for (i = 0; i < npoints; i++) {
        // Get each point of the list and add it to the array
        point = (JCUnit.Point) pointList.get(i);
        points[i] = point;

        // accumulate the maximum point
        y1 = point.getY();
        if (y1.greaterThan(max_y)) {
            max_y = y1;
        }
    }

    // Copy the first point as the last point to close the line
    // Also re-add the second point to ensure close polygon connection works
    // (kind of a hack -- but since we're converting polygons to a series
    // of lines, we need to do this for Java to close the polygon correctly)
    if (close) {
        points[npoints] = (JCUnit.Point) first.clone();
        if (second != null) {
            points[npoints + 1] = (JCUnit.Point) second.clone();
        }
    } else {
        if (second != null) {
            points[npoints] = (JCUnit.Point) second.clone();
        }
    }

    // Create a line render and add it to the frame's render list
    line = new LineRender(points, style);
    renderList.add(line);

    // Add the line width to the extent
    y2 = style.getDrawnWidth();
    y2.divide(2.0);
    max_y.add(y2);

    // Update the greatest vertical extent of the frame
    if (max_y.greaterThan(yExtent)) {
        yExtent.assign(max_y);
    }
}

/**
 * Draws a polygon given an array of points (the sequence of points will
 * automatically be closed).
 * @param style the drawing style used to specify the appearance of the figure
 * @param points an array of points from which to create the polygon
 */
public void drawPolygon(JCDrawStyle style, JCUnit.Point[] points) {

    int             i, npoints;
    boolean         close = false;
    LineRender      line;
    JCUnit.Measure  max_y, y1, y2;
    JCUnit.Point[]  new_points = null;

    // Make sure we've got an immutable style
    style = style.getImmutable();

    // get the number of points in the line
    npoints = points.length;
    max_y = new JCUnit.Measure();

    // if the first and last points differ, add a point to close the line
    if (!points[0].equals(points[npoints - 1])) {
        npoints++;
        close = true;

        // Copy the points to a longer array
        new_points = new JCUnit.Point[npoints];
        new_points[npoints - 1] = (JCUnit.Point) points[0].clone();
    }

    // Scan the points in the array
    for (i = 0; i < npoints - 1; i++) {

        // if we need to close the line, copy the point
        if (close) {
            new_points[i] = points[i];
        }

        // accumulate the maximum point
        y1 = points[i].getY();
        if (y1.greaterThan(max_y)) {
            max_y = y1;
        }
    }

    // if we're closing the line, replace the point array
    if (close) {
        points = new_points;
    }

    // Create a line render and add it to the frame's render list
    line = new LineRender(points, style);
    renderList.add(line);

    // Add the line width to the extent
    y2 = style.getDrawnWidth();
    y2.divide(2.0);
    max_y.add(y2);

    // Update the greatest vertical extent of the frame
    if (max_y.greaterThan(yExtent)) {
        yExtent.assign(max_y);
    }
}

/**
 * Draws a filled polygon given a list of points (the sequence of points
 * will automatically be closed).
 * @param style the drawing style used to specify the appearance of the figure
 * @param pointList a list of points from which to create the polygon
 */
public void fillPolygon(JCDrawStyle style, List pointList) {

    int             i, npoints;
    boolean         close = false;
    LineRender      line;
    JCUnit.Measure  max_y, y1, y2;
    JCUnit.Point    point, first, last;
    JCUnit.Point[]  points;

    // Make sure we've got an immutable style
    style = style.getImmutable();

    // get the number of points in the line
    npoints = pointList.size();

    // if the first and last points differ, add a point to close the line
    first = (JCUnit.Point) pointList.get(0);
    last = (JCUnit.Point) pointList.get(npoints - 1);
    if (!first.equals(last)) {
        npoints++;
        close = true;
    }

    // create an array for the points
    points = new JCUnit.Point[npoints];
    max_y = new JCUnit.Measure();

    for (i = 0; i < npoints - 1; i++) {

        // Get each point of the list and add it to the array
        point = (JCUnit.Point) pointList.get(i);
        points[i] = point;

        // accumulate the maximum point
        y1 = point.getY();
        if (y1.greaterThan(max_y)) {
            max_y = y1;
        }
    }

    // Copy the first point as the last point to close the line
    if (close) {
        points[npoints - 1] = (JCUnit.Point) first.clone();
    }

    // Create a line render and add it to the frame's render list
    line = new LineRender(points, style);
    line.setFilled(true);
    renderList.add(line);

    // Add the line width to the extent
    y2 = style.getDrawnWidth();
    y2.divide(2.0);
    max_y.add(y2);

    // Update the greatest vertical extent of the frame
    if (max_y.greaterThan(yExtent)) {
        yExtent.assign(max_y);
    }
}

/**
 * Draws a filled polygon given an array of points (the sequence of points
 * will automatically be closed).
 * @param style the drawing style used to specify the appearance of the figure
 * @param points an array of points from which to create the polygon
 */
public void fillPolygon(JCDrawStyle style, JCUnit.Point[] points) {

    int             i, npoints;
    boolean         close = false;
    LineRender      line;
    JCUnit.Measure  max_y, y1, y2;
    JCUnit.Point[]  new_points = null;

    // Make sure we've got an immutable style
    style = style.getImmutable();

    // get the number of points in the line
    npoints = points.length;
    max_y = new JCUnit.Measure();

    // if the first and last points differ, add a point to close the line
    if (!points[0].equals(points[npoints - 1])) {
        npoints++;
        close = true;

        // Copy the points to a longer array
        new_points = new JCUnit.Point[npoints];
        new_points[npoints - 1] = (JCUnit.Point) points[0].clone();
    }

    // Scan the points in the array
    for (i = 0; i < npoints - 1; i++) {

        // if we need to close the line, copy the point
        if (close) {
            new_points[i] = points[i];
        }

        // accumulate the maximum point
        y1 = points[i].getY();
        if (y1.greaterThan(max_y)) {
            max_y = y1;
        }
    }

    // if we're closing the line, replace the point array
    if (close) {
        points = new_points;
    }

    // Create a line render and add it to the frame's render list
    line = new LineRender(points, style);
    line.setFilled(true);
    renderList.add(line);

    // Add the line width to the extent
    y2 = style.getDrawnWidth();
    y2.divide(2.0);
    max_y.add(y2);

    // Update the greatest vertical extent of the frame
    if (max_y.greaterThan(yExtent)) {
        yExtent.assign(max_y);
    }
}

/**
 * Draws a rounded-corner rectangle given a start point, size, and corner radius.
 * @param style the drawing style used to specify the appearance of the figure
 * @param origin the location of the rectangle
 * @param size the dimensions of the rectangle
 * @param r the radius of the corner arcs
 */
public void drawRoundedRectangle(JCDrawStyle style, JCUnit.Point origin,
                                 JCUnit.Dimension size, JCUnit.Measure r) {

    RoundRectRender     rectangle;
    JCUnit.Measure      max_y;

    // Make sure we've got an immutable style
    style = style.getImmutable();

    // The radius should not be negative
    if (r.isNegative()) {
        throw new IllegalArgumentException("Radius must be positive");
    }

    // Create a rectangle with round corners and add it to the render list
    rectangle = new RoundRectRender(origin, size, r, style);
    renderList.add(rectangle);

    // Track the bounds of the frame
    max_y = style.getDrawnWidth();
    max_y.divide(2.0);
    max_y.add(origin.getY());
    max_y.add(size.getHeight());

    if (max_y.greaterThan(yExtent)) {
        yExtent.assign(max_y);
    }
}

/**
 * Draws a rounded-corner rectangle given a start point, end point, and arc radius.
 * @param style the drawing style used to specify the appearance of the figure
 * @param startPoint the first point of the rectangle
 * @param endPoint the diagonal corner of the rectangle
 * @param r the radius of the corner arcs
 */
public void drawRoundedRectangle(JCDrawStyle style, JCUnit.Point startPoint,
                                 JCUnit.Point endPoint, JCUnit.Measure r) {

    RoundRectRender     rectangle;
    JCUnit.Measure      max_y, y1, y2;

    // Make sure we've got an immutable style
    style = style.getImmutable();

    // The radius should not be negative
    if (r.isNegative()) {
        throw new IllegalArgumentException("Radius must be positive");
    }

    // Create a rectangle with round corners and add it to the render list
    rectangle = new RoundRectRender(startPoint, endPoint, r, style);
    renderList.add(rectangle);

    // Track the bounds of the frame
    y1 = startPoint.getY();
    y2 = endPoint.getY();

    max_y = style.getDrawnWidth();
    max_y.divide(2.0);
    if (y1.greaterThan(y2)) {
        max_y.add(y1);
    }
    else {
        max_y.add(y2);
    }

    if (max_y.greaterThan(yExtent)) {
        yExtent.assign(max_y);
    }
}

/**
 * Draws a filled rounded rectangle given a start point, size, and corner radius.
 * @param style the drawing style used to specify the appearance of the figure
 * @param origin the location of the rectangle
 * @param size the dimensions of the rectangle
 * @param r
 */
public void fillRoundedRectangle(JCDrawStyle style, JCUnit.Point origin,
                                 JCUnit.Dimension size, JCUnit.Measure r) {

    RoundRectRender     rectangle;
    JCUnit.Measure      max_y;

    // Make sure we've got an immutable style
    style = style.getImmutable();

    // The radius should not be negative
    if (r.isNegative()) {
        throw new IllegalArgumentException("Radius must be positive");
    }

    // Create a rectangle with round corners and add it to the render list
    rectangle = new RoundRectRender(origin, size, r, style);
    rectangle.setFilled(true);
    renderList.add(rectangle);

    // Track the bounds of the frame
    max_y = style.getDrawnWidth();
    max_y.divide(2.0);
    max_y.add(origin.getY());
    max_y.add(size.getHeight());

    if (max_y.greaterThan(yExtent)) {
        yExtent.assign(max_y);
    }
}

/**
 * Draws a filled rounded rectangle given a start point, end point, and arc radius.
 * @param style the drawing style used to specify the appearance of the figure
 * @param startPoint the first point of the rectangle
 * @param endPoint the diagonal corner of the rectangle
 * @param r the radius of the corner arcs
 */
public void fillRoundedRectangle(JCDrawStyle style, JCUnit.Point startPoint,
                                 JCUnit.Point endPoint, JCUnit.Measure r) {

    RoundRectRender     rectangle;
    JCUnit.Measure      max_y, y1, y2;

    // Make sure we've got an immutable style
    style = style.getImmutable();

    // The radius should not be negative
    if (r.isNegative()) {
        throw new IllegalArgumentException("Radius must be positive");
    }

    // Create a rectangle with round corners and add it to the render list
    rectangle = new RoundRectRender(startPoint, endPoint, r, style);
    rectangle.setFilled(true);
    renderList.add(rectangle);

    // Track the bounds of the frame
    y1 = startPoint.getY();
    y2 = endPoint.getY();

    max_y = style.getDrawnWidth();
    max_y.divide(2.0);
    if (y1.greaterThan(y2)) {
        max_y.add(y1);
    }
    else {
        max_y.add(y2);
    }

    if (max_y.greaterThan(yExtent)) {
        yExtent.assign(max_y);
    }
}


/**
 * Completes any pending actions on the frame (for example, current line).
 */
public void complete() {
    // INSETS:  We may need to push this action to subframes
    

    // Paste down the current line
    if (info.currentLine.size() > 0) {
        // use the default text style if none set
        JCTextStyle tempStyle = info.currentTextStyle;
        if (tempStyle == null) {
            tempStyle = JCTextStyle.DEFAULT_TEXT;
        }

        
        doLineBreak(tempStyle, false, true);
    }

    

    // Process macros that can be evaluated
    calcLayoutStatus();
    if (layoutStatus < Render.LAYOUT_COMPLETE) {
        // Find uncompleted macros and try to evaluate them
        updateLayoutStatus(document.getFlow(), page);
    }  else {
        document.getFlow().fireFrameCompleted(this);
    }
}

/**
 * Checks the render objects on the page to determine the layout status.
 */
public void calcLayoutStatus() {

    int         render_status, worst_status = Render.LAYOUT_COMPLETE;
    Iterator    renders;
    Render      item;

    // Check the layout status of included frames
    super.calcLayoutStatus();

    // Check the layout status of the render objects on the page
    renders = renderList.iterator();
    while(renders.hasNext()) {
        item = (Render) renders.next();

        // compare the layout status of each render object with the worst so far
        render_status = item.getLayoutStatus();
        if (Render.isLayoutLower(worst_status, render_status)) {
            worst_status = render_status;
        }
    }

    // Compare the status of render objects with the status of subframes
    if (Render.isLayoutLower(layoutStatus, worst_status)) {
        layoutStatus = worst_status;
    }
}

/**
 * Attempts to evaluate unresolved macros in this frame according to the
 * given information (page and flow determine the current context).
 * @param flow the document flow in which the macro appears
 * @param page the page on which the macro is placed
 */
public void updateLayoutStatus(JCFlow flow, JCPage page) {

    int         render_status, worst_status = Render.LAYOUT_COMPLETE;
    int         index, new_line = -1, count;
    Render      item;

    // Don't update this frame if it is already complete
    if (layoutStatus == Render.LAYOUT_COMPLETE) {
        return;
    }

    

    // Use the parent method to update contained frames
    super.updateLayoutStatus(flow, page);

    // Scan render objects in the frame and update any that need it
    count = renderList.size();
    
    for (index = 0; index < count; index++) {
        item = (Render) renderList.get(index);
        

        // update any macro render objects which require it
        if (item instanceof MacroRender) {
            JCUnit.Measure  old_width, new_width;
            MacroRender     macro = (MacroRender) item;

            
            // If we have an uncompleted macro, try to resolve it
            if (macro.getStatus() != Render.LAYOUT_COMPLETE) {
                

                old_width = macro.getSize().getWidth();
                macro.evaluateMacro(flow, page);

                // FUTURE:  Adjust position of text on successful resolution
                new_width = macro.getSize().getWidth();
                if (!new_width.equals(old_width)) {
                    new_width.subtract(old_width);
                    macroAdjustLine(macro, new_width, new_line);
                }
            }
        }
        else if (item instanceof SymbolRender) {
            SymbolRender symbol = (SymbolRender) item;

            

            // if this is a newline, note its position
            if (symbol.getType() == SymbolRender.NEWPARAGRAPH_SYMBOL ||
                symbol.getType() == SymbolRender.NEWLINE_SYMBOL) {
                new_line = index;
            }
            //JLA:  Next 4 lines fix Bunka Flow bugs 003 001 and 003)
            //these items have no status, so don't check them, just call them complete
            item.setLayoutStatus(Render.LAYOUT_COMPLETE);
        } else {
            item.setLayoutStatus(Render.LAYOUT_COMPLETE);
        }
        

        // compare the layout status of each render object with the worst so far
        render_status = item.getLayoutStatus();
        if (Render.isLayoutLower(worst_status, render_status)) {
            worst_status = render_status;
        }
    }

    // Compare the status of this frame's contents with those of subframes
    if (Render.isLayoutLower(layoutStatus, worst_status)) {
        layoutStatus = worst_status;
    }
}

/**
 * Adjusts the elements on a line given that a macro on the line has changed.
 * @param macro the macro requiring the line's adjustment
 * @param delta the change in width of the macro text
 * @param new_line the end of the previous line
 */
private void macroAdjustLine(MacroRender macro, JCUnit.Measure delta,
                             int new_line) {

    int             index, line_end = -1, count, object_count = 0;
    int             macro_index = -1;
    int             alignment = JCTextStyle.ALIGNMENT_LEFT;
    int             tab_alignment = JCTab.TAB_ALIGNMENT_LEFT;
    boolean         before = true, tabs = false;
    boolean         tab_begin = false, tab_end = false;
    Render          item;
    SymbolRender    symbol;
    JCTextStyle     style;
    JCUnit.Point    position;
    JCUnit.Measure  new_x;

    // Scan the line, searching for the end of line and checking the alignment
    count = renderList.size();
    for (index = new_line + 1; index < count && line_end == -1; index++) {
        item = (Render) renderList.get(index);

        if (item instanceof SymbolRender) {
            symbol = (SymbolRender) item;

            

            // if this is a newline, note its position
            if (symbol.getType() == SymbolRender.NEWPARAGRAPH_SYMBOL ||
                symbol.getType() == SymbolRender.NEWLINE_SYMBOL) {
                line_end = index;
            }

            // if this is a tab, track the alignment that applies to the macro
            else if (symbol.getType() == SymbolRender.TAB_SYMBOL) {

                tabs = true;

                // Keep the most recent alignment until we find the macro
                if (before) {
                    tab_alignment = symbol.getAlignment();

                    // we begin re-aligning from the tab
                    new_line = index;
                    tab_begin = true;
                }

                // stop if we find a tab after the macro
                else {
                    line_end = index;
                    tab_end = true;
                }

            }
        }
        else {
            if ((Render) macro == item) {
                before = false;
                macro_index = index;

                // if the macro is decimal-aligned then (for now at least)
                // it (and it alone) is effectively right-aligned.
                if (tab_alignment == JCTab.TAB_ALIGNMENT_DECIMAL) {
                    tab_alignment = JCTab.TAB_ALIGNMENT_RIGHT;
                    line_end = index + 1;
                }
            }
            else {

                // If the macro doesn't immediately follow a decimal tab
                // then we don't have to decimal-align the macro
                if (tab_alignment == JCTab.TAB_ALIGNMENT_DECIMAL &&
                    index == new_line + 1) {

                    tab_alignment = JCTab.TAB_ALIGNMENT_LEFT;
                    new_line = index;
                    tab_begin = false;
                }
            }

            // count non-symbol objects on the line in case we have to justify
            object_count++;
        }
    }


    // Convert the tab alignment to an equivalent paragraph alignment
    if (tabs) {
        if (tab_alignment == JCTab.TAB_ALIGNMENT_CENTER) {
            alignment = JCTextStyle.ALIGNMENT_CENTER;
        }
        else if (tab_alignment == JCTab.TAB_ALIGNMENT_RIGHT) {
            alignment = JCTextStyle.ALIGNMENT_RIGHT;
        }
        else {
            alignment = JCTextStyle.ALIGNMENT_LEFT;
        }
    }

    // otherwise we're aligning the line based on the text style
    else {

        // Find the alignment applied at the end of the line
        if (line_end == -1) {
            for (index = count - 1; index > new_line; index--) {
                item = (Render) renderList.get(index);

                if (item instanceof StringRender
                    || item instanceof MacroRender) {

                    if (item instanceof StringRender) {
                        style = ((StringRender) item).getStyle();
                    }
                    else {
                        style = ((MacroRender) item).getStyle();
                    }
                    alignment = style.getAlignment();
                    index = new_line;
                }
            }
            line_end = count;
        }
        else {
            style = ((SymbolRender) renderList.get(line_end)).getStyle();
            alignment = style.getAlignment();
        }
    }

    // Use CENTER for JUSTIFY if there is just one object
    if (alignment == JCTextStyle.ALIGNMENT_JUSTIFY && object_count == 1) {
        alignment = JCTextStyle.ALIGNMENT_CENTER;
    }

    // Depending on the alignment of the line, adjust the positions...
    before = true;
    switch (alignment) {

        case JCTextStyle.ALIGNMENT_LEFT:

            // Scan the line, adjust positions following the macro
            for (index = new_line + 1; index < line_end; index++) {
                item = (Render) renderList.get(index);

                // Initially just scan until we find the object
                if (before) {
                    if ((Render) macro == item) {
                        before = false;
                    }
                }

                // after we've found the object adjust all positions
                else {
                    position = item.getLocation();
                    new_x = position.getX();
                    new_x.add(delta);
                    position.setX(new_x);
                }
            }

            // Adjust the leader of any tab that follows
            if (tab_end) {
                symbol = (SymbolRender) renderList.get(line_end);
                new_x = symbol.getLeaderStart();
                new_x.add(delta);
                symbol.setLeaderStart(new_x);
            }
            break;

        case JCTextStyle.ALIGNMENT_RIGHT:

            // Scan the line, adjust positions preceding the macro
            index = new_line + 1;
            while (before && index < line_end) {
                item = (Render) renderList.get(index);

                // scan, adjusting positions, until we find the object
                if (before) {
                    position = item.getLocation();
                    new_x = position.getX();
                    new_x.subtract(delta);
                    position.setX(new_x);

                    if ((Render) macro == item) {
                        before = false;
                    }
                }

                index++;
            }

            // Adjust the leader of a tab that precedes the text block
            if (tab_begin) {
                symbol = (SymbolRender) renderList.get(new_line);
                new_x = symbol.getLeaderEnd();
                new_x.subtract(delta);
                symbol.setLeaderEnd(new_x);
            }
            break;

        case JCTextStyle.ALIGNMENT_CENTER:

            delta.divide(2.0);

            // Scan the line, adjust positions following the macro
            for (index = new_line + 1; index < line_end; index++) {
                item = (Render) renderList.get(index);

                // scan, moving right, until we find the object
                if (before) {
                    position = item.getLocation();
                    new_x = position.getX();
                    new_x.subtract(delta);
                    position.setX(new_x);

                    if ((Render) macro == item) {
                        before = false;
                    }
                }

                // after we've found the object move positions left
                else {
                    position = item.getLocation();
                    new_x = position.getX();
                    new_x.add(delta);
                    position.setX(new_x);
                }
            }

            // Adjust the leader of a tab that precedes the text block
            if (tab_begin) {
                symbol = (SymbolRender) renderList.get(new_line);
                new_x = symbol.getLeaderEnd();
                new_x.subtract(delta);
                symbol.setLeaderEnd(new_x);
            }

            // Adjust the leader of any tab that follows
            if (tab_end) {
                symbol = (SymbolRender) renderList.get(line_end);
                new_x = symbol.getLeaderStart();
                new_x.add(delta);
                symbol.setLeaderStart(new_x);
            }
            break;

        case JCTextStyle.ALIGNMENT_JUSTIFY:

            JCUnit.Measure add_space = new JCUnit.Measure(JCUnit.POINTS);
            JCUnit.Measure step_space = (JCUnit.Measure) delta.clone();
            step_space.divide(object_count - 1);

            // Scan the line, adjust positions following the macro
            for (index = new_line + 1; index < line_end; index++) {
                item = (Render) renderList.get(index);

                // Scan across the line, moving each item
                position = item.getLocation();
                new_x = position.getX();
                new_x.subtract(add_space);
                position.setX(new_x);

                // When we find the object we change the sense of adding space
                if (before && ((Render) macro == item)) {
                    before = false;
                    add_space.subtract(delta);
                }

                // Adjust the spacing adjustment for the next object
                add_space.add(step_space);
            }

            // NOTE:  This case does not correspond to any tab alignments,
            //        so we don't have to worry about tab leader adjustments

            break;

        default:
            break;
    }

}

/**
 * Gets the list of render objects which have been placed in this frame.
 * @return a list of render objects pasted in this frame
 */
protected List getRenderList() {
    return renderList;
}

/**
 * Returns a frame which is a deep copy of this one (copies included frames).
 * @return a new frame which is the same as this
 */
public Object clone() {

    Iterator    frames, renders;
    JCFrame     subframe, new_frame;
    Render        render;

    

    // if the frame has pending text, paste it first
    if (info.currentLine.size() > 0) {
        doLineBreak(info.currentTextStyle, false);
    }

    // Clone the frame
    try {
        new_frame = (JCFrame) super.clone();
    }
    catch (CloneNotSupportedException e) {
        return(null);
    }

    // Create a new FlowInfo object for the new frame
    new_frame.info = new JCFlowInfo();

    
    // Copy any frames contained within this one
    new_frame.frameList = new Vector();
    frames = this.frameList.iterator();
    while (frames.hasNext()) {
        subframe = (JCFrame) frames.next();
        
        new_frame.frameList.add((JCFrame) subframe.clone());
    }

    
    // Copy the list of render objects
    new_frame.renderList = new Vector();
    renders = this.renderList.iterator();
    while (renders.hasNext()) {
        render = (Render) renders.next();
        if (render instanceof StringRender) {
            
        }
        else {
            
        }
        new_frame.renderList.add(render.clone());
        
        
    }

    
    
    return((Object) new_frame);
}

/**
 * Draws the frame and its contents.
 * @param gc the graphics into which to draw the frame
 */
public void draw(Graphics2D gc) {

    int                 i, count;
    double                x, y, width, height;
    Iterator            frames;
    Graphics2D          gc1;
    Render              item;
    JCFrame             frame;
    JCDrawStyle         style;
    JCUnit.Point        frame_location;
    JCUnit.Dimension    frame_size;
    JCUnit.Margins      clip_mask;

    
    


    // This Graphics represents a frame
    if (gc instanceof Graphics2DBase) {
        ((Graphics2DBase) gc).setFrameRoot();

        // Establish the frame's context
        ((Graphics2DBase) gc).updateContext();
    }

    // if there is a background colour set, paint the frame with it
    if (color != null) {
        

        // Create a style for filling in the area's background
        style = new JCDrawStyle();
        style.setFillForegroundColor(color);
        style.makeImmutable();

        // Create a background object and draw it
        if (background == null) {
            background = new BoxRender(new JCUnit.Point(), size, style);
            ((BoxRender) background).setFilled(true);
        }
        background.draw(gc);
    }

    // If there is a coloured border style, draw borders
    if (border != null && border.getForegroundColor() != null) {
        double              border_width, border_height, line_width;
        BoxRender           border_rect;
        JCUnit.Point        border_origin;
        JCUnit.Dimension    border_size;


        // Compute the amount to offset the border into the frame
        line_width = border.getDrawnWidth().getAs(JCUnit.POINTS);
        // special case -- line width of zero means thinnest line,
        // so we have to make room for it on the page

        if (line_width <= 0.0) {



            line_width = 1.0;
        }

        // Compute the size of the frame and adjust for the border width
        double frame_width = size.getWidth().getAs(JCUnit.POINTS);
        double frame_height = size.getHeight().getAs(JCUnit.POINTS);
        double half_line_width = line_width / 2.0;
        double half_frame_width = frame_width / 2.0;
        double half_frame_height = frame_height / 2.0;

        double box_width;
        if (line_width >= frame_width) {
            box_width = frame_width;
        } else if (line_width > half_frame_width) {
            box_width = frame_width - half_line_width;
        } else {
            box_width = frame_width - line_width;
        }
        double box_height;
        if (line_width >= frame_height) {
            box_height = frame_height;
        } else if (line_width > half_frame_height) {
            box_height = frame_height - half_line_width;
        } else {
            box_height = frame_height - line_width;
        }
        JCUnit.Dimension box_size = new JCUnit.Dimension(JCUnit.POINTS,
                                                    box_width, box_height);
        double box_x;
        if (line_width >= frame_width) {
            box_x = 0;
        } else if (line_width > half_frame_width) {
            box_x = Math.floor(half_line_width);
        } else {
            box_x = half_line_width;
        }
        double box_y;
        if (line_width >= frame_height) {
            box_y = 0;
        } else if (line_width > half_frame_height) {
            box_y = Math.floor(half_line_width);
        } else {
            box_y = half_line_width;
        }
        border_origin = new JCUnit.Point(JCUnit.POINTS, box_x, box_y);

        border_rect = new BoxRender(border_origin, box_size, border);

        border_rect.draw(gc);
    }

    
    count = renderList.size();

    // Draw the contents of the frame
    for (i = 0; i < count; i++) {

        // get and draw each render object
        item = (Render) renderList.get(i);
        
        item.draw(gc);
    }

    // Markup output ignores all included frames (at least for now)
    if (gc instanceof Markup) {
        
        return;
    }

    // Loop through all contained frames and draw them
    
    frames = frameList.iterator();

    double clip_x, clip_y, clip_width, clip_height;
    while(frames.hasNext()) {

        frame = (JCFrame) frames.next();
        

        double frame_width = size.getWidth().getAs(JCUnit.POINTS);
        double frame_height = size.getHeight().getAs(JCUnit.POINTS);
        // Create the gc for the new frame
        frame_location = frame.getLocation();
        x = frame_location.getX().getAs(JCUnit.POINTS);
        y = frame_location.getY().getAs(JCUnit.POINTS);
        frame_size = frame.getSize();
        width = frame_size.getWidth().getAs(JCUnit.POINTS);
        height = frame_size.getHeight().getAs(JCUnit.POINTS);
        // if subframe not in visible portion of frame, don't draw it
        if ((x > frame_width || x + width < 0.0) ||
            (y > frame_height || y + height < 0.0)) {
            continue;
        }

        // create the clip rect in doubles
        Rectangle2D.Double r2d = new Rectangle2D.Double(x, y, width, height);
        Rectangle2D.Double frame_r2d =
                new Rectangle2D.Double(0.0, 0.0, frame_width, frame_height);
        Rectangle2D.Double clipRect = clipToFrame(frame_r2d, r2d);

        // graphics api does not have a create method which takes doubles, so
        // we create a gc then set the clip on it, then translate in doubles
        /*
         * DO NOT change the way this frame's Graphics is created!!
         */
        gc1 = (Graphics2D) gc.create();

        /*
         * We mark this Graphics as being a "frame" for purposes of determining
         * if it can be drawn (nested) inside another frame.
         */
        if (gc1 instanceof Graphics2DBase) {
            ((Graphics2DBase) gc1).setFrameRoot();
        }

        // set it on the gc
        gc1.setClip(clipRect);

        // finally, translate it
        gc1.translate(x,y);

        // translate the frame rectangle -- still using it for clipping
        frame_r2d.x -= x;
        frame_r2d.y -= y;

        // Set the clip mask for the new gc
        clip_mask = frame.getClipMask();
        if (clip_mask != null) {
            x         = clip_mask.getLeft().getAs(JCUnit.POINTS);
            y         = clip_mask.getTop().getAs(JCUnit.POINTS);
            width     = clip_mask.getRight().getAs(JCUnit.POINTS);
            height     = clip_mask.getBottom().getAs(JCUnit.POINTS);
            r2d = new Rectangle2D.Double(x, y, width, height);
            clipRect = clipToFrame(frame_r2d, r2d);
            gc1.setClip(clipRect);
        }

        // draw the frame and dispose of the gc
        frame.draw(gc1);
        gc1.dispose();
    }

    
}

private Rectangle2D.Double clipToFrame(Rectangle2D.Double frameRect,
                                        Rectangle2D.Double newRect)
{
    // derive clip rectangle from intersection of main frame
    // and this subframe.
    // Note: I've tried using Rectangle2D's intersect() method but
    // it returned some strange results
    Rectangle2D.Double clipRect = new Rectangle2D.Double(0.0, 0.0, 0.0, 0.0);
    if (newRect.x < frameRect.x) {
        clipRect.x = frameRect.x;
        clipRect.width = newRect.width + newRect.x - frameRect.x;
    } else {
        clipRect.x = newRect.x;
        clipRect.width = newRect.width;
    }
    if (newRect.y < frameRect.y) {
        clipRect.y = frameRect.y;
        clipRect.height = newRect.height + newRect.y - frameRect.y;
    } else {
        clipRect.y = newRect.y;
        clipRect.height = newRect.height;
    }
    if (clipRect.x + clipRect.width > frameRect.x + frameRect.width) {
        clipRect.width = frameRect.x + frameRect.width - clipRect.x;
    }
    if (clipRect.y + clipRect.height > frameRect.height) {
        clipRect.height = frameRect.y + frameRect.height - clipRect.y;
    }
    return clipRect;
}

/**
 * Computes the width of columns in the frame from its width, the number of
 * columns, and the spacing between them.
 */
private void computeColumnWidth() {

    JCUnit.Measure  width, gutter;

    // Get the width between the frame margins
    width = size.getWidth();
    width.subtract(margin.getLeft());
    width.subtract(margin.getRight());

    // Error-check width
    if (width.isNegative()) {
        throw new IllegalArgumentException("Printable space would be < 0");
    }

    // If there are multiple columns, apportion the space
    if (columnCount > 1) {

        // Compute the amount of space wasted between columns
        gutter = (JCUnit.Measure) columnSpace.clone();
        gutter.multiply(columnCount - 1);

        // Compute the remaining space per column
        width.subtract(gutter);
        width.divide(columnCount);

        // Error-check width
        if (width.isNegative()) {
            throw new IllegalArgumentException("Column widths would be < 0");
        }
    }

    columnWidth.assign(width);
}

/**
 * Takes a given sequence of render objects and regenerate the flow.
 * @param gc the Graphics context in which the drawing is to be done
 * @param itemList a List of render items to be flowed to this frame
 * @throws EndOfFrameException
 */
private void reflow(Graphics2D gc, List itemList) throws EndOfFrameException {

    int         i, count;
    Render      item;

    

    // loop through the render items in the list
    count = itemList.size();
    for (i = 0; i < count; i++) {
        item = (Render) itemList.get(i);

        // Pass symbol render object on by performing the action
        if (item instanceof SymbolRender) {
            int type = ((SymbolRender) item).getType();
            info.currentTextStyle = ((SymbolRender) item).getStyle();

            // Perform an analogous tab action
            if (type == SymbolRender.TAB_SYMBOL) {
                tab(info.currentTextStyle);
            }

            // Insert a new line where required
            else if (type == SymbolRender.NEWPARAGRAPH_SYMBOL ||
                     type == SymbolRender.NEWLINE_SYMBOL) {
                newLine(info.currentTextStyle);
            }
        }

        // otherwise we flow the pending object to the current flow
        else {

            // if this is a string, make sure we have the current style
            if (item instanceof StringRender) {
                info.currentTextStyle = ((StringRender) item).getStyle();
            }

            // MACRO: if this is a text macro, we must re-evaluate it
            if (item instanceof MacroRender) {
                // NOTE: we don't have flow information available here
                ((MacroRender) item).evaluateMacro(null, page);
            }

            // flow the object
            try {
                flowPrint(gc, item, false);
            }

            // if the frame gets full
            catch (EndOfFrameException e) {

                // add pending objects to the line
                i++;
                while (i < count) {
                    e.currentLine.add(itemList.get(i++));
                }

                // pass the exception onward
                throw e;
            }
        }
    }
}

/**
 * Determines if the frame's info data has been initialized yet.
 * @return <code>false</code> if the frame's local flow info has not been defined
 */
private boolean isInfoInitialised() {
    return(info.lineEnd.distance != -1.0);
}

/**
 * Computes the info fields necessary to print to the frame.
 */
protected void initialiseInfo() {
    initialiseInfo(true);
}

/**
 * Computes the <code>FlowInfo</code> parameters for the frame.
 * @param paragraph indicates if the first line begins a new paragraph
 */
protected void initialiseInfo(boolean paragraph) {

    JCTextStyle     style;
    JCUnit.Measure  insertion_x, insertion_y;

    

    // Generate a style for use in computing frame positions
    if (info.currentTextStyle == null) {
        style = JCTextStyle.DEFAULT_TEXT;
    }
    else {
        style = info.currentTextStyle;
    }

    // Start in the first column and define its extents
    info.leftOfColumn.assign(margin.getLeft());
    info.rightOfColumn.assign(info.leftOfColumn);
    info.rightOfColumn.add(columnWidth);

    // compute the insertion point and line extents
    info.insertion = new JCUnit.Point(info.leftOfColumn.units);
    computeLineBounds(style, paragraph);
    info.insertion.setX(info.lineBegin);

    insertion_y = getFirstLineOffset(style);
    insertion_y.add(margin.getTop());
    info.insertion.setY(insertion_y);
    
}

/**
 * Given a render object, adds it to the formatted stream in the current frame.
 * @param gc the target Graphics context for formatting and outputting
 * @param item a render object to append at the current insertion point
 * @param force <code>true</code> if the given object is to be pasted even if it doesn't fit
 * @throws EndOfFrameException
 */
private void flowPrint(Graphics2D gc, Render item, boolean force)
                       throws EndOfFrameException {

    int                 i, tab_align;
    double              item_ascent = 0, item_descent = 0;
    double              item_spacing, baseline_offset;
    Render              fitItem, remainItem, move_item;
    JCTextStyle         style;
    JCUnit.Point        position, temp_position;
    JCUnit.Measure      width, trim_width, bottom, new_x, new_y, delta_x;
    JCUnit.Measure      left_space, right_space, total_space, need_space;

    if (!isInfoInitialised()) {
        initialiseInfo();
    }

    
    
    
    
    
    
    

    

    // Pre-compute the width
    width = item.getSize().getWidth();

    // Compute the bottom of the printable area of the frame
    bottom = this.size.getHeight();
    bottom.subtract(this.margin.getBottom());

    // Find a style to use for line wrapping
    if (item instanceof StringRender) {
        style = ((StringRender) item).getStyle();

        // Recompute the width ignoring trailing space
        // (We don't want to line-break for trailing spaces,
        //  we don't even want them carrying over to the following line)
        trim_width = ((StringRender) item).getTrimWidth(gc);
    }
    else {
        style = info.currentTextStyle;

        if (style == null) {
            // NOTE:  we should have a context-provided style!!
            style = JCTextStyle.DEFAULT_TEXT;
        }

        // The trimmed width is the same as the width
        trim_width = width;
    }

    // NOTE:  This is a bit of a hack to avoid problems with infoInitialise
    // If there are no elements on the current line and it's the first
    // line of a frame, make sure the line bounds are correct.
    if (info.currentLine.size() == 0 && info.firstLine) {
        
        computeLineBounds(style, info.firstLine);
        info.insertion.setX(info.lineBegin);

        
    }

    // deal with tabs
    left_space = new JCUnit.Measure();
    if (info.currentTab != null) {
        tab_align = info.currentTab.getTabAlignment();

        switch(tab_align) {

        case JCTab.TAB_ALIGNMENT_LEFT:
            
            break;

        case JCTab.TAB_ALIGNMENT_RIGHT:
        case JCTab.TAB_ALIGNMENT_CENTER:
            left_space.assign(info.tabSymbol.getLeaderEnd());
            left_space.subtract(info.tabSymbol.getLeaderStart());

            // if there is no remaining space left, forget the tab
            if (left_space.isZero()) {
                tab_align = JCTab.TAB_ALIGNMENT_LEFT;
            }

            
            

            break;

        // deal with decimal tabs (warning: messy!)
        case JCTab.TAB_ALIGNMENT_DECIMAL:
            

            // First start with the actual amount of space
            left_space.assign(info.tabSymbol.getLeaderEnd());
            left_space.subtract(info.tabSymbol.getLeaderStart());

            // if the item has a decimal compute the space left of it
            if (item instanceof StringRender) {
                if (((StringRender) item).hasDecimal()) {
                    JCUnit.Measure  decimal;
                    decimal = ((StringRender) item).getDecimalPosition();

                    // reduce the space available to the amount of space that
                    // will actually be used to display the string
                    if (left_space.greaterThan(decimal)) {
                        left_space = decimal;
                    }
                }
            }

            // NOTE:  for now we force decimal tabs to be honoured
            force = true;
            break;
        }
    }

    // otherwise we are not tabbing, simply laying out text (same as left tab)
    else {
        tab_align = JCTab.TAB_ALIGNMENT_LEFT;
    }

    // Compute the amount of space on the remainder of the line
    right_space = (JCUnit.Measure) info.lineEnd.clone();
    right_space.subtract(info.insertion.getX());
    total_space = (JCUnit.Measure) right_space.clone();
    total_space.add(left_space);

    // Check if zero or negative.  Because of the JCUnit.areProbablyEqual
    // check, the distance value in total_space could be slightly negative.
    // This can cause a negative index to be computed when the flow code
    // tries to split the string below which causes a
    // StringIndexOutOfBoundsException.  See PR20860.
    if (total_space.isZero() || total_space.isNegative()) {
        total_space.distance = 0.0;
    }

    /*
     * If the object fits in the remaining space of the current line or the
     * force flag indicates that the object is to be placed regardless.
     */
    
    if (!trim_width.greaterThan(total_space) || force) {

        JCUnit.Measure  style_size, leading;

        // Get the adequate minimum size for the current line
        style_size = new JCUnit.Measure(JCUnit.POINTS, style.getPointSize());

        // if the line is smaller, or there is nothing on it, use the minimum
        if (info.lineSpacing.lessThan(style_size) ||
            info.currentLine.size() == 0) {

            info.lineSpacing = style_size;
        }

        // Compute the vertical space requirements of the render object
        if (item instanceof StringRender || item instanceof MacroRender) {

            Font        font = style.getFont();
            FontMetrics fm   = gc.getFontMetrics(font);
            String      text;

            // Cast the render object as appropriate to extract the text
            // NOTE:  Perhaps we should have a text interface of some sort
            if (item instanceof MacroRender) {
                text = ((MacroRender) item).getText();
            }
            else {
                text = ((StringRender) item).getText();
            }

            // For basic text use the font's ascent and descent
            if (style.getBaselineOffset() == JCTextStyle.OFFSET_NONE) {

                item_ascent = fm.getAscent();
                item_descent = fm.getDescent();
                
                
                

                // If the text is underlined add any additional space
                if (style.getUnderlining() == JCTextStyle.LINEMODE_UNDERLINE) {

                    double  alt_descent;

                    // Derive the line metrics appropriately
                    

                    
                    LineMetrics lm = fm.getLineMetrics(text, gc);


                    alt_descent = lm.getUnderlineOffset();
                    alt_descent += lm.getUnderlineThickness();
                    if (alt_descent > item_descent) {
                        item_descent = alt_descent;
                    }
                }

                baseline_offset = 0.0;
            }

            // For subscript or superscript use the actual character extents
            else {

                Font                ss_font;
                FontRenderContext   frc;
                GlyphVector         gv_x, gv_text;
                Rectangle2D            lb_x, lb_text;
                FontMetrics         ffm;

                // get dimension information for a subscript font
                ss_font = FontBase.deriveFont(font, (float)
                        (FontBase.getSize2D(font) * style.getSubscriptRatio()));

                // bit of trickery here, we get the font metrics
                // which then converts the java font to the native
                // font (ps/pdf/pcl or awt) which we then use to
                // get the glyph vector
                ffm     = gc.getFontMetrics(ss_font);
                ss_font    = ffm.getFont();

                frc     = gc.getFontRenderContext();

                
                gv_x    = FontBase.createGlyphVector(ss_font, frc, "x");
                gv_text = FontBase.createGlyphVector(ss_font, frc, text);


                


                lb_x    = gv_x.getLogicalBounds();

                lb_text = gv_text.getLogicalBounds();

                // compute the ascender and descender
                // NOTE:  the ascender is -y (top of text height above baseline)
                //        and the descender is (height + y) => height - ascender
                double  ss_x_height  = - lb_x.getY();
                double  ss_ascender  = - lb_text.getY();
                double  ss_descender = lb_text.getHeight() + lb_text.getY();

                // The baseline of a superscript is the x-height of the
                // superscripted text above the baseline of normal text.
                // The baseline of a subscript is the difference between
                // the x-height of the subscripted text and its maximum
                // ascent below the baseline of normal text.  This creates
                // an area where the 'body' (or x-height) of super and
                // subscripted text could theoretically co-exist but the
                // the superscript descenders and subscript asceneders
                // would overlap.

                // If this is a superscript
                if (style.getBaselineOffset()
                                        == JCTextStyle.OFFSET_SUPERSCRIPT) {
                    item_ascent = ss_x_height + ss_ascender;
                    baseline_offset = ss_x_height;
                }

                // otherwise, for subscripts
                else {
                    item_ascent = ffm.getAscent();
                    //PR22749 offset should be constant, and
                    //based on the x-height of the font.
                    baseline_offset = ss_x_height - ffm.getAscent();
                }

                // in both cases the descender is adjusted by the baseline
                item_descent = ss_descender - baseline_offset;

                // NOTE:  item_descent could be negative (meaning that it lies
                //        _above_ the normal text baseline)

                // Make sure the line size tracks the base font
                if (fm.getAscent() > info.ascender.getAs(JCUnit.POINTS)) {
                    info.ascender =
                            new JCUnit.Measure(JCUnit.POINTS, fm.getAscent());
                }
                if (fm.getDescent() > info.descender.getAs(JCUnit.POINTS)) {
                    info.descender =
                            new JCUnit.Measure(JCUnit.POINTS, fm.getDescent());
                }
            }

            // if the line height is greater than the line spacing
            item_spacing = fm.getLeading();

            
            
            
            
            

            // NOTE:  Since FontMetrics.getLeading() doesn't work, we try
            //        to estimate the leading based on the font size
            item_spacing = style.getPointSize() - fm.getAscent()
                           - fm.getDescent();
            
            leading = new JCUnit.Measure(JCUnit.POINTS, item_spacing);
            if (leading.greaterThan(info.leading)) {
                info.leading = leading;
            }

            if (item_ascent + item_descent + item_spacing
                > info.lineSpacing.getAs(JCUnit.POINTS)) {

                // If this is baseline text, increase the overall line spacing
                if (style.getBaselineOffset() == JCTextStyle.OFFSET_NONE) {
                    info.lineSpacing = new JCUnit.Measure(JCUnit.POINTS,
                                     item_ascent + item_descent + item_spacing);
                    
                }
            }

            // NOTE:  We need to potentially recompute the height of the
            //        line if a change to the ascender or descender will
            //        change the position of an embedded image

            // Check the effective descender we need to this point
            double old_descent = info.descender.getAs(JCUnit.POINTS);
            double embed_descent = info.embedDescent.getAs(JCUnit.POINTS);

            // Take the larger of the new and old descender
            if (item_descent > old_descent) {
                info.descender = new JCUnit.Measure(JCUnit.POINTS,
                                                    item_descent);

                // NOTE:  recompute the embedded descender in the new line!
            }
            else {
                if (old_descent > embed_descent) {
                    item_descent = old_descent;
                }
                else {
                    item_descent = embed_descent;
                }
            }

            // Check the effective ascender we need to this point
            double old_ascent = info.ascender.getAs(JCUnit.POINTS);
            double embed_ascent = info.embedAscent.getAs(JCUnit.POINTS);

            // Check the new ascender against the line so far and adjust at need
            if (item_ascent > old_ascent) {
                info.ascender = new JCUnit.Measure(JCUnit.POINTS, item_ascent);

                if (item_ascent > embed_ascent) {
                    

                    // add vertical space above the line, adjusting images
                    pushLineDown(item_ascent - old_ascent,
                                 item_descent - old_descent, true);
                }
                // NOTE:  recompute the embedded ascender in the new line!
            }
            else {
                // Push down by the maximum amount of space requested
                if (old_ascent > embed_ascent) {
                    item_ascent = old_ascent;
                }
                else {
                    item_ascent = embed_ascent;
                }
            }

            
            
            
            

            // if the line height is greater than the existing height
            if (item_ascent + item_descent + item_spacing
                > info.height.getAs(JCUnit.POINTS)) {

                

                // recompute the height of the line
                info.height.assign(info.ascender);
                info.height.add(info.descender);
                info.height.add(new JCUnit.Measure(JCUnit.POINTS,item_spacing));
            }

            // NOTE:  Because we are computing the distance down to the
            //        baseline of the text from the top of the line, we
            //        have to _subtract_ the baseline offset from the
            //        ascender height to find the actual text baseline

            // set the vertical position of the text
            position = item.getLocation();
            new_y = new JCUnit.Measure(JCUnit.POINTS,
                                       item_ascent - baseline_offset);
            new_y.add(info.insertion.getY());
            position.setY(new_y);
        }

        // Compute ascent & descent for embedded images and graphics
        else {

            int    v_align;
            double center_line;
            double item_height = item.getSize().getHeight().getAs(JCUnit.POINTS);

            // If the item has special support for line embedding
            if (item instanceof Embedable) {
                v_align = ((Embedable) item).getVerticalAlignment();
            }
            else {
                v_align = JCDrawStyle.CENTER_ON_BASELINE;
            }

            // Based on the alignment compute the effective ascender/descender
            item_ascent = computeAscent(item_height, v_align);
            item_descent = item_height - item_ascent;

            // in the cases where the image is fixed to the baseline
            if (!isVariable(v_align)) {

                // Take the larger of the new and old descender
                double old_descent = info.descender.getAs(JCUnit.POINTS);
                if (item_descent > old_descent) {
                    info.descender = new JCUnit.Measure(JCUnit.POINTS,
                                                        item_descent);
                }

                // Check the ascender against the line so far and adjust at need
                double old_ascent = info.ascender.getAs(JCUnit.POINTS);
                if (item_ascent > old_ascent) {
                    info.ascender = new JCUnit.Measure(JCUnit.POINTS,
                                                       item_ascent);

                    // add vertical space above the line, adjusting images
                    pushLineDown(item_ascent - old_ascent,
                                 item_descent - old_descent, true);
                }

                // Compute the position of the top of the object
                new_y = new JCUnit.Measure(JCUnit.POINTS, - item_ascent);
                if (info.ascender.greaterThan(info.embedAscent)) {
                    new_y.add(info.ascender);
                }
                else {
                    new_y.add(info.embedAscent);
                }
            }

            // otherwise the image can vary in vertical position
            else {         // (isVariable(v_align) == true)


                // Check the size against any image space so far
                double old_ascent  = info.embedAscent.getAs(JCUnit.POINTS);
                double old_descent = info.embedDescent.getAs(JCUnit.POINTS);

                // Take the larger of the image ascenders
                if (item_ascent > old_ascent) {
                    info.embedAscent = new JCUnit.Measure(JCUnit.POINTS,
                                                          item_ascent);

                    if (info.embedAscent.greaterThan(info.ascender)) {

                        // Get the old maximum line ascent
                        double line_ascent = info.ascender.getAs(JCUnit.POINTS);
                        old_ascent = info.ascender.getAs(JCUnit.POINTS);
                        if (line_ascent > old_ascent) {
                            old_ascent = line_ascent;
                        }

                        // Add vertical space above the line,
                        // _without_ adjusting other images
                        pushLineDown(item_ascent - old_ascent, 0.0, false);
                    }
                }

                // Take the larger of the image descenders
                if (item_descent > old_descent) {
                    info.embedDescent = new JCUnit.Measure(JCUnit.POINTS,
                                                           item_descent);
                }

                // Compute the position of the top of the object
                new_y = new JCUnit.Measure(JCUnit.POINTS, - item_ascent);
                if (info.ascender.greaterThan(info.embedAscent)) {
                    new_y.add(info.ascender);
                }
                else {
                    new_y.add(info.embedAscent);
                }
            }

            // set the vertical position of the embedded object
            position = item.getLocation();
            new_y.add(info.insertion.getY());
            position.setY(new_y);
        }

        

        // recompute the height of the line
        if (info.ascender.greaterThan(info.embedAscent)) {
            info.height.assign(info.ascender);
            
        }
        else {
            info.height.assign(info.embedAscent);
            
        }
        if (info.descender.greaterThan(info.embedDescent)) {
            info.height.add(info.descender);
            
        }
        else {
            info.height.add(info.embedDescent);
            
        }
        

        // NOTE:  This may not be entirely accurate since the embedded
        //        object ascender and descender may change!
        // Make sure the line size tracks the largest object
        if (info.height.greaterThan(info.lineSpacing)) {
            info.lineSpacing.assign(info.height);
        }

        // Locate the render object horizontally on the line
        position = item.getLocation();
        if (tab_align == JCTab.TAB_ALIGNMENT_LEFT) {

            // Just locate at the next horizontal position
            new_x = info.insertion.getX();
            position.setX(new_x);

            // Adjust the postion to follow the object
            new_x.add(width);
        }

        // otherwise we have to compute how much space before and after is used
        else if (tab_align == JCTab.TAB_ALIGNMENT_CENTER) {

            

            need_space = (JCUnit.Measure) width.clone();
            need_space.divide(2.0);
            
            

            // if there isn't enough space to the left for the object
            if (need_space.greaterThan(left_space)) {
                delta_x = left_space;

                // Compute the amount of space needed on the right
                need_space.multiply(2.0);
                need_space.subtract(left_space);

                // We've run out of space in the tab
                info.currentTab = null;
            }
            // if there isn't enough space to the right of the object
            else if (need_space.greaterThan(right_space)) {
                delta_x = (JCUnit.Measure) width.clone();
                delta_x.subtract(right_space);

                need_space = right_space;
            }
            // otherwise there is enough space to locate it correctly
            else {
                delta_x = need_space;
            }

            // Adjust the position of preceding elements
            for (i = info.tabIndex; i < info.currentLine.size(); i++) {
                move_item = (Render) info.currentLine.get(i);
                temp_position = move_item.getLocation();
                new_x = temp_position.getX();
                new_x.subtract(delta_x);
                temp_position.setX(new_x);
            }

            // Also adjust the tab leader space
            new_x = info.tabSymbol.getLeaderEnd();
            new_x.subtract(delta_x);
            info.tabSymbol.setLeaderEnd(new_x);

            // Because this shifts the line left, it places the render object
            // left of the nominal insertion point
            new_x = info.insertion.getX();
            new_x.subtract(delta_x);
            position.setX(new_x);
            

            // If the tab is now to be ignored, unset the tab index
            if (info.currentTab == null) {
                info.tabIndex = -1;
            }

            // Adjust the postion to follow the object
            new_x.add(width);
            
        }

        else if (tab_align == JCTab.TAB_ALIGNMENT_RIGHT) {

            

            need_space = new JCUnit.Measure();

            // if there isn't enough space to the left for the object
            if (width.greaterThan(left_space)) {
                delta_x = left_space;

                // Compute the amount of space needed on the right
                need_space.assign(width);
                need_space.subtract(left_space);

                // We've run out of space in the tab
                info.currentTab = null;
            }
            // otherwise there is enough space to locate it correctly
            else {
                delta_x = width;
            }

            
            

            // Adjust the position of preceding elements
            for (i = info.tabIndex; i < info.currentLine.size(); i++) {
                move_item = (Render) info.currentLine.get(i);
                temp_position = move_item.getLocation();
                new_x = temp_position.getX();
                new_x.subtract(delta_x);
                temp_position.setX(new_x);
            }

            // Also adjust the tab leader space
            new_x = info.tabSymbol.getLeaderEnd();
            new_x.subtract(delta_x);
            info.tabSymbol.setLeaderEnd(new_x);

            // Because this shifts the line left, it places the render object
            // left of the nominal insertion point
            new_x = info.insertion.getX();
            new_x.subtract(delta_x);
            position.setX(new_x);
            

            // If the tab is now to be ignored, unset the tab index
            if (info.currentTab == null) {
                info.tabIndex = -1;
            }

            // Adjust the postion to follow the object
            new_x.add(width);
            
        }
        else {    // (tab_align == JCTab.TAB_ALIGNMENT_DECIMAL)
            boolean     align_decimal = false;

            // If this is a string render scan for a decimal
            if (item instanceof StringRender) {
                align_decimal = ((StringRender) item).hasDecimal();
            }
            need_space = new JCUnit.Measure();

            // if there is a decimal we try to align at the decimal point
            if (align_decimal) {
                JCUnit.Measure  decimal_location;

                decimal_location = ((StringRender) item).getDecimalPosition();

                // if there isn't enough space to the left for the number part
                if (decimal_location.greaterThan(left_space)) {
                    delta_x = left_space;
                }
                // otherwise there is enough space to locate it correctly
                else {
                    delta_x = decimal_location;
                }
            }

            // otherwise we have to right-align to the tab
            else {

                // if there isn't enough space to the left for the object
                if (width.greaterThan(left_space)) {
                    delta_x = left_space;
                }
                // otherwise there is enough space to locate it correctly
                else {
                    delta_x = width;
                }
            }

            // Compute the amount of space needed on the right
            need_space.assign(width);
            need_space.subtract(delta_x);

            // Adjust the tab leader space
            new_x = info.tabSymbol.getLeaderEnd();
            new_x.subtract(delta_x);
            info.tabSymbol.setLeaderEnd(new_x);

            // this places the render object left of the nominal insertion point
            new_x = info.insertion.getX();
            new_x.subtract(delta_x);
            position.setX(new_x);
            

            // We've used the tab, henceforth we ignore it
            info.currentTab = null;
            info.tabIndex = -1;

            // Adjust the postion to follow the object
            new_x.add(width);
        }

        

        //  Add the render object to the current line
        info.currentLine.add(item);
        info.insertion.setX(new_x);
        

        // Compute the (new) bottom of the current line
        new_y = info.insertion.getY();
        new_y.add(info.height);

        // wrap the current line if it's too tall to fit and we're
        // not forcing the item to fit on this page
//DAM
//need some way to determine I'm at the TOP of a column
        if (new_y.greaterThan(bottom)) {
            // This can throw an EndOfFrameException, which we propogate up
            
            newColumn(style, false);
        }

        // if printing an object has left us past the end of the line
        if (new_x.greaterThan(info.lineEnd)) {
            // advance to a new line
            doLineBreak(style, false);
        }
    }
    // otherwise the render object doesn't fit on this line (&& force == false)
    else {

        // if the object can't split...
        if (item.isAtomic()) {

            // if we're not already at the start of a line, move to the next
            if (info.insertion.getX().greaterThan(info.lineBegin)) {
                doLineBreak(style, false);

                // try printing again on the next line - this could throw an
                // exception, but that is permissible (pass it upwards)
                flowPrint(gc, item, false);
            }

            // otherwise the object is too long for a line, paste it anyway
            else {

                // Force the printing of the object on the current line
                flowPrint(gc, item, true);
            }
        }

        // Otherwise the item can split into parts
        else {

            // Determine the sub-part which fits on this line
            
            fitItem = ((Splitable) item).splitBefore(gc, total_space);
            
            remainItem = ((Splitable) item).splitAfter(gc, total_space);

            // if nothing fit, see if anything at all will fit on a line
            if (fitItem.getSize().width == 0.0) {

                Render          temp_item, force_item;
                JCUnit.Measure  line_space;

                // compute the total space on a full line between margins
                line_space = (JCUnit.Measure) info.lineEnd.clone();
                line_space.subtract(info.lineBegin);

                // Determine the sub-part which fits on a full line
                if (width.greaterThan(line_space)) {
                    temp_item = ((Splitable) item).splitBefore(gc, line_space);
                }
                else {
                    // The item will fit on the next line
                    temp_item = item;
                }

                // If nothing fits on a _full_ line or if this is already a
                // blank line (perhaps the first line of a paragraph)
                if (temp_item.getSize().width == 0.0 ||
                    info.currentLine.size() == 0) {

                    // then we have to split a word
                    
                    force_item = ((Splitable) item).forceSplit(gc, total_space);
                    // if forced fit item fits in the space we have
                    // or we're at the beginning of a line, commit
                    // the force.  Otherwise, short circuit the force
                    // in the hopes that we can better force a split
                    // on the next line
                    if ((force_item.getSize().width <=
                            total_space.getAs(JCUnit.POINTS)) ||
                                        (info.currentLine.size() == 0)) {
                        fitItem = force_item;
                        force = true;
                        remainItem = ((Splitable) item).splitAfter(gc, total_space);
                    }
                }
            }

            // As long as something fit, try to paste it
            if (fitItem.getSize().width > 0.0) {

                
                

                try {
                    
                    flowPrint(gc, fitItem, force);
                }
                // If it didn't fit (vertically) and generated an exception
                catch (EndOfFrameException e) {

                    int     lastIndex;
                    List    temp_line;
                    Render  lastItem;

                    // Add the pending data to the exception and pass it up

                    // Get the last item of the line (so we preserve location)
                    temp_line = e.getCurrentLine();
                    lastIndex = temp_line.size() - 1;
                    lastItem = (Render) temp_line.get(lastIndex);

                    // NOTE:  We put the unsplit object on the end of the line
                    //        in place of the part that was computed to fit,
                    //        so no data is left hanging
                    ((StringRender) lastItem).unsplit(gc, remainItem);

                    throw e;
                }
            }

            // Move to the next line to continue printing
            // if we're not already at the start of a line
            if (info.insertion.getX().greaterThan(info.lineBegin)) {
                doLineBreak(style, false);
            }
            if (remainItem.getSize().width > 0.0) {
                flowPrint(gc, remainItem, false);
            }
        }
    }

    // Make sure we have a style for every frame that has something in it
    if (info.currentTextStyle == null || item instanceof StringRender) {
        info.currentTextStyle = style;
    }
    
}

private void pushLineDown(double add_ascent, double delta_descent,
                          boolean adjust_variable) {

    int             i, count, v_align;
    double          item_ascent, item_descent, item_height;
    Render          item;
    JCUnit.Point    position;
    JCUnit.Measure  new_y;

    
    
    count = info.currentLine.size();

    // if we are adjusting the vertical position of embedded objects
    if (adjust_variable) {
        double  max_ascent, max_descent, old_ascent, delta;

        // Store the maximal ascender and descender of the line
        max_ascent = 0;
        max_descent = 0;

        // loop through the line, checking the position of embedded objects
        for (i = 0; i < count; i++) {

            // Get an object from the current line, ignoring non-embedded
            item = (Render) info.currentLine.get(i);
            if (!(item instanceof Embedable)) {
                continue;
            }

            // get the alignment of the object
            v_align = ((Embedable) item).getVerticalAlignment();

            // Also ignore embedded objects fixed to the text baseline
            if (!isVariable(v_align)) {
                continue;
            }

            // Compute the item's new ascender and descender
            item_height = item.getSize().getHeight().getAs(JCUnit.POINTS);
            item_ascent = computeAscent(item_height, v_align);
            item_descent = item_height - item_ascent;

            // Compute the maximum ascender and descender across the line
            if (item_ascent > max_ascent) {
                max_ascent = item_ascent;
            }
            if (item_descent > max_descent) {
                max_descent = item_descent;
            }
        }

        // compute the amount the maximum embedded ascender has shifted
        old_ascent = info.embedAscent.getAs(JCUnit.POINTS);
        delta = old_ascent - max_ascent;
        info.embedAscent = new JCUnit.Measure(JCUnit.POINTS, max_ascent);

        // Set the current maximal descender
        info.embedDescent = new JCUnit.Measure(JCUnit.POINTS, max_descent);

        // Adjust the amount the line is to be shifted by
        add_ascent -= delta;
    }

    // Loop through the line adjusting the position of the items
    for (i = 0; i < count; i++) {
        item = (Render) info.currentLine.get(i);

        // Recompute the positions of all embedded variable objects (if needed)
        if (item instanceof Embedable && adjust_variable) {

            v_align = ((Embedable) item).getVerticalAlignment();
            if (isVariable(v_align)) {

                // Compute the item's new ascender and descender
                item_height = item.getSize().getHeight().getAs(JCUnit.POINTS);
                item_ascent = computeAscent(item_height, v_align);
                item_descent = item_height - item_ascent;

                // Compute the new position of the embedded object
                new_y = info.insertion.getY();
                if (info.ascender.greaterThan(info.embedAscent)) {
                    new_y.add(info.ascender);
                }
                else {
                    new_y.add(info.embedAscent);
                }
                new_y.subtract(new JCUnit.Measure(JCUnit.POINTS, item_ascent));

                // Move the embedded object to its new location
                position = item.getLocation();
                position.setY(new_y);

                // Procede with the next element of the current line
                continue;
            }
        }

        
        

        // Add the adjustment to the current object's location
        position = item.getLocation();
        new_y = position.getY();
        new_y.add(new JCUnit.Measure(JCUnit.POINTS, add_ascent));
        position.setY(new_y);
    }
}

private boolean isVariable(int v_align) {

    switch (v_align) {

    case JCDrawStyle.CENTER_ABOVE_BASELINE:
    case JCDrawStyle.CENTER_IN_LINE:
    case JCDrawStyle.ALIGN_TO_TOP:
    case JCDrawStyle.ALIGN_TO_BOTTOM:
        return(true);

    case JCDrawStyle.POSITION_ON_BASELINE:
    case JCDrawStyle.POSITION_BELOW_BASELINE:
    case JCDrawStyle.CENTER_ON_BASELINE:
    default:
        return(false);
    }
}

private double computeAscent(double item_height, int v_align) {

    double  item_ascent, center_line;

    switch (v_align) {

    case JCDrawStyle.POSITION_ON_BASELINE:

        item_ascent = item_height;
        break;

    case JCDrawStyle.POSITION_BELOW_BASELINE:

        item_ascent = 0;
        break;

    case JCDrawStyle.CENTER_ON_BASELINE:

        item_ascent = item_height / 2.0;
        break;

    case JCDrawStyle.CENTER_ABOVE_BASELINE:

        center_line = info.ascender.getAs(JCUnit.POINTS) / 2.0;
        item_ascent = center_line + item_height / 2.0;
        break;

    case JCDrawStyle.CENTER_IN_LINE:

        center_line = (info.ascender.getAs(JCUnit.POINTS)
                       - info.descender.getAs(JCUnit.POINTS)) / 2.0;
        item_ascent = center_line + item_height / 2.0;
        break;

    case JCDrawStyle.ALIGN_TO_TOP:

        item_ascent = info.ascender.getAs(JCUnit.POINTS);
        break;

    case JCDrawStyle.ALIGN_TO_BOTTOM:

        item_ascent = item_height - info.descender.getAs(JCUnit.POINTS);
        break;

    default:
        item_ascent = -1;
    }

    return(item_ascent);
}

private void pasteFloats() {

    Floatable           floatItem;
    JCUnit.Measure      availSpace, whitespace, new_y, new_x;
    JCUnit.Dimension    itemSize;
    JCUnit.Point        position;

    // Return if there are no floating objects
    if (info.floatList.size() == 0) {
        return;
    }

    

    // Make sure we have a style to use for computing space between objects
    if (info.currentTextStyle == null) {
        info.currentTextStyle = JCTextStyle.DEFAULT_TEXT;
    }

    // initialise the column margins, insertion point, etc. if we need to
    if (!isInfoInitialised()) {
        initialiseInfo();
    }

    // Get the first object in the float list and its size
    floatItem = (Floatable) info.floatList.firstElement();
    itemSize = floatItem.getSize();

    

    // Compute the available space
    availSpace = this.size.getHeight();
    availSpace.subtract(this.margin.getBottom());

    // at the top of the frame add minimal whitespace
    if (info.insertion.getY().distance == 0.0) {
        info.insertion.setY(getFirstLineOffset(info.currentTextStyle));
    }

    

    availSpace.subtract(info.insertion.getY());

    

    // If there is enough space to float the object into the frame
    while (itemSize.getHeight().lessThan(availSpace)) {

        // Get the location reference of the floating object
        position = floatItem.getLocation();

        // Allow for various alignments
        switch (floatItem.getAlignment()) {

            case JCTextStyle.ALIGNMENT_NONE:
            case JCTextStyle.ALIGNMENT_LEFT:
                
                position.setX(info.leftOfColumn);
                break;

            case JCTextStyle.ALIGNMENT_RIGHT:
                
                new_x = (JCUnit.Measure) info.rightOfColumn.clone();
                new_x.subtract(itemSize.getWidth());
                position.setX(new_x);
                break;

            case JCTextStyle.ALIGNMENT_CENTER:
            case JCTextStyle.ALIGNMENT_JUSTIFY:
                
                new_x = (JCUnit.Measure) info.rightOfColumn.clone();
                new_x.add(info.leftOfColumn);
                new_x.subtract(itemSize.getWidth());
                new_x.divide(2.0);
                position.setX(new_x);
                break;

            default:
                // we don't recognise the style, so throw an error
                throw new IllegalArgumentException("invalid alignment");
        }
        new_y = info.insertion.getY();
        position.setY(new_y);

        

        // Remove the item from the float list and add to the render list
        info.floatList.remove(0);
        renderList.add(floatItem);

        // Compute the amount of space remaining & adjust the insertion point
        whitespace = getFirstLineOffset(info.currentTextStyle);
        whitespace.add(itemSize.getHeight());

        availSpace.subtract(whitespace);
        new_y.add(whitespace);
        info.insertion.setY(new_y);

        // If there are no more objects exit the loop.
        if (info.floatList.size() == 0) {
            break;
        }

        

        // Get the next object and its size
        floatItem = (Floatable) info.floatList.firstElement();
        itemSize = floatItem.getSize();

        
    }
}

}

