/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCPageTableFromJCTable.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:22 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page;

import com.klg.jclass.table.*;

import javax.swing.*;
import java.awt.*;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;
/* JDK12_END */

/**
 *
 * Creates a <code>JCPageTable</code> from a JClass <code>JCTable</code>.
 *
 */
public class JCPageTableFromJCTable {
	
protected static int defaultColumnWidth = 75;

protected static int[][] alignment_map = {
	{SwingConstants.CENTER, JCTextStyle.ALIGNMENT_CENTER},
	{SwingConstants.LEFT, JCTextStyle.ALIGNMENT_LEFT},
	{SwingConstants.RIGHT, JCTextStyle.ALIGNMENT_RIGHT},
	{SwingConstants.LEADING, JCTextStyle.ALIGNMENT_NONE},
	{SwingConstants.TRAILING, JCTextStyle.ALIGNMENT_NONE},
};

/**
 *
 * Creates a <code>JCPageTable</code> from a JClass <code>JCTable</code>.
 * @return
 * @param doc
 * @param jcTable
 * @param populate
 *
 */
public static JCPageTable createTable(JCDocument doc, JCTable jcTable,
									  boolean populate) {
	// get the data model from the JCTable
	TableDataModel tableDataModel = jcTable.getDataSource();

	return createTable(doc, tableDataModel, jcTable, populate);
}

/**
 * Creates a <code>JCPageTable</code> from a JClass <code>TableDataModel</code>.
 * @return
 * @param doc
 * @param populate
 * @param tableDataModel
 */
public static JCPageTable createTable(JCDocument doc,
                                      TableDataModel tableDataModel,
									  boolean populate) {
	return createTable(doc, tableDataModel, null, populate);
}

/**
 * Creates a <code>JCPageTable</code> from a JClass <code>JCTable</code>.
 * @return
 * @param doc
 * @param jcTable
 * @param populate
 * @param tableDataModel
 */
protected static JCPageTable createTable(JCDocument doc,
                                         TableDataModel tableDataModel,
										 JCTable jcTable, boolean populate) {
	JCPageTable table = null;

	// build up columns
	int num_columns = 0;


	Vector columns;
	// create an extra column for the row label, if displayed
	if(jcTable == null) {
        num_columns =  tableDataModel.getNumColumns();
        int offset = 0;
        for (int i = 0; i < tableDataModel.getNumRows(); i++){
            if (tableDataModel.getTableRowLabel(i) != null) {
                offset = 1;

            }
        }

		columns = new Vector(num_columns + offset);
        if(offset > 0){
		    columns.add(new JCUnit.Measure(JCUnit.POINTS, defaultColumnWidth));
        }
	}
	else if(jcTable != null && jcTable.isRowLabelDisplay()) {
        num_columns = jcTable.getNumColumns();
		columns = new Vector(num_columns + 1);
		// add the row label as a column
		int colWidth = jcTable.getColumnPixelWidth(JCTableEnum.LABEL);
		// ensure column has width if jctable not yet layed out
		if (jcTable.getPixelWidth(JCTableEnum.LABEL) != JCTableEnum.AS_IS &&
															colWidth == 0) {
			colWidth = defaultColumnWidth;
		}
		columns.add(new JCUnit.Measure(JCUnit.POINTS, colWidth));
	}
	else {
        num_columns = jcTable.getNumColumns();

		columns = new Vector(num_columns);
	}

	// add the columns
	for(int c = 0; c < num_columns; c++) {
		// add the column and set its width
		int colWidth = defaultColumnWidth;
		if(jcTable != null) {
            if(jcTable.isColumnHidden(c)) {

                continue;
            }
            else {
                colWidth = jcTable.getColumnPixelWidth(c);
                // ensure column has width if jctable not yet layed out
                if (jcTable.getPixelWidth(c) != JCTableEnum.AS_IS &&
                        colWidth == 0){
                    colWidth = defaultColumnWidth;
                }
            }
		}

		columns.add(new JCUnit.Measure(JCUnit.POINTS, colWidth));
	}

	// create the table
	table = new JCPageTable(doc, columns);


	table.setAlignment(JCTextStyle.ALIGNMENT_LEFT);
	if ((jcTable == null ||
			(jcTable != null && jcTable.isColumnLabelDisplay()))) {
		table.createHeaders();
	}

	if(populate) {
        boolean apply = jcTable != null;
		populateTable(table, tableDataModel, jcTable, apply);
	}

	// handle spans
	if(jcTable != null) {
		Collection spans = jcTable.getSpannedRanges();
		if(spans != null) {
			// adjust ranges to account for row labels
			int offset = jcTable.isRowLabelDisplay() ? 1 : 0;
			Iterator it = spans.iterator();
			while(it.hasNext()) {
				JCCellRange cr = (JCCellRange)it.next();
				table.spanCells(cr.start_row,
								cr.start_column + offset,
								cr.end_row - cr.start_row + 1,
								cr.end_column - cr.start_column + 1);
			}
		}
	}

	return table;
}

/**
 * Populates an empty <code>JPageTable</code> from the specified
 * <code>JCTable</code>.
 * @param jcTable
 * @param table
 */
public static void populateTable(JCPageTable table, JCTable jcTable) {
	populateTable(table, jcTable.getDataSource(), jcTable, true);
}

/**
 * Populates an empty <code>JCPageTable</code> from the specified <code>TableDataModel</code>.
 * @param table
 * @param tableDataModel
 */
public static void populateTable(JCPageTable table,
                                 TableDataModel tableDataModel) {
	populateTable(table, tableDataModel, null, false);
}


/**
 * Populates an empty <code>JPageTable</code> from the specified
 * <code>JCTable</code>.
 * @param table
 * @param jcTable
 */
public static void populateTable(JCPageTable table, JCTable jcTable,
                                 boolean applyJCTableStyles) {
    populateTable(table, jcTable.getDataSource(), jcTable,
            applyJCTableStyles);
}



/**
 * Converts the <code>SwingConstants</code> alignment to a <code>JCTextStyle</code> alignment.
 * @return
 * @param alignment
 */
protected static int convertSwingToTextStyleAlignment(int alignment) {
	int new_alignment = JCTextStyle.ALIGNMENT_LEFT;

	for (int i = 0; i < alignment_map.length; i++) {
		if (alignment_map[i][0] == alignment) {
			new_alignment = alignment_map[i][1];
		}
	}
	return new_alignment;
}

/**
 * Given a <code>CellStyleModel</code>, sets <code>JCTextStyle</code> attributes.
 * @param styleModel
 * @param currentStyle
 * @return
 */
protected static JCTextStyle setTextStyle(CellStyleModel styleModel,
										  JCTextStyle currentStyle) {
	if(styleModel == null || currentStyle == null) {
		return null;
	}

	int alignment = styleModel.getHorizontalAlignment();
	alignment = convertSwingToTextStyleAlignment(alignment);
	if(styleModel.getFont() != currentStyle.getFont() ||
	   alignment != currentStyle.getAlignment() ||
	   styleModel.getForeground() != currentStyle.getColor()) {
		// something has chnaged, so clone the current style
		currentStyle = (JCTextStyle)currentStyle.clone();
		// change the font if necessary
		if(styleModel.getFont() != currentStyle.getFont()) {
			currentStyle.setFont(styleModel.getFont());
		}
		if(alignment != currentStyle.getAlignment()) {
			currentStyle.setAlignment(alignment);
		}
		if(styleModel.getForeground() != currentStyle.getColor()) {
			currentStyle.setColor(styleModel.getForeground());
		}
		// make style immutable
		currentStyle = currentStyle.getImmutable();
	}

	return currentStyle;
}

protected static JCDrawStyle setDrawStyle(CellStyleModel styleModel,
										  JCDrawStyle currentDrawStyle) {
	if(styleModel == null || currentDrawStyle == null) {
		return null;
	}

	// get the border model for the style
	CellBorderModel cbm = styleModel.getCellBorder();

	// consider the style as having a border unless it's border type
	// is an instance of JCCellBorder with BORDER_NONE
	boolean hasBorder = true;
	if(cbm instanceof JCCellBorder) {
		hasBorder = !(((JCCellBorder)cbm).getBorderType() == JCTableEnum.BORDER_NONE);
	}

	// if the style has a border other than JCCellBorder with BORDER_NONE,
	// consider it a line type of LINE_TYPE_SINGLE
	if(hasBorder && 
	   currentDrawStyle.getLineType() != JCDrawStyle.LINE_TYPE_SINGLE) {
		currentDrawStyle = (JCDrawStyle)currentDrawStyle.clone();
		currentDrawStyle.setLineType(JCDrawStyle.LINE_TYPE_SINGLE);
		currentDrawStyle.setLineWidth(new JCUnit.Measure(JCUnit.POINTS, 1));
		currentDrawStyle = currentDrawStyle.getImmutable();
	}
	// if there is no border, use line type LINE_TYPE_NONE
	else if(!hasBorder && 
			currentDrawStyle.getLineType() != JCDrawStyle.LINE_TYPE_NONE) {
		currentDrawStyle = (JCDrawStyle)currentDrawStyle.clone();
		currentDrawStyle.setLineType(JCDrawStyle.LINE_TYPE_NONE);
		currentDrawStyle.setLineWidth(new JCUnit.Measure(JCUnit.POINTS, 0));
		currentDrawStyle = currentDrawStyle.getImmutable();
	}
	return currentDrawStyle;
}
/**
 * Populates an empty <code>JCPageTable</code> from the specified <code>TableDataModel</code>.
 * @param table
 * @param jcTable
 * @param tableDataModel
 */
protected static void populateTable(JCPageTable table,
                                    TableDataModel tableDataModel,
									JCTable jcTable) {
    populateTable(table, tableDataModel, jcTable, true);
}

/**
 * Populates an empty <code>JCPageTable</code> from the specified
 * <code>TableDataModel</code>.
 * @param jcTable
 * @param table
 * @param tableDataModel
 */
protected static void populateTable(JCPageTable table,
                                    TableDataModel tableDataModel,
									JCTable jcTable,
                                    boolean applyJCTableStyles) {
	JCTextStyle defaultStyle = JCTextStyle.stringToStyle("default text");
	JCTextStyle currentStyle = defaultStyle;

	JCDrawStyle defaultDrawStyle = JCDrawStyle.stringToStyle("default line");
	JCDrawStyle currentDrawStyle = defaultDrawStyle;

	JCPageTable header = table.getHeaders();
	// create the header
	if(header == null &&
	   (jcTable == null || (jcTable != null
            && jcTable.isColumnLabelDisplay()))) {
		header = table.createHeaders();
	}

	try {
		//Font currentFont = null;
		//Font oldFont = currentStyle.getFont();

		int offset = 0;
		if(jcTable == null) {

            for (int i = 0; i < tableDataModel.getNumRows(); i++){
                if (tableDataModel.getTableRowLabel(i) != null) {
                    offset = 1;
                }
            }
        }
        else if(jcTable != null && jcTable.isRowLabelDisplay()) {
			offset = 1;
		}

		int num_columns = 0;
        if (jcTable == null) {
            num_columns = tableDataModel.getNumColumns();
        }
        else {
            num_columns = jcTable.getNumColumns();

        }
		if(header != null) {
			// go through the columns, shifting the cells by one if
			// row labels are displayed
            int cCount = 0;
            int numberColumns = num_columns;
			for(int i = 0; i < numberColumns; i++) {
                boolean decrement = false;
                if(jcTable != null && jcTable.isColumnHidden(i)) {
                    cCount++;
                    decrement = true;

                }
				// set the string to appear in the header

				JCFrame frame = header.getCellFrame(0, i + offset);
                Object object = null;
                if (jcTable == null){
				    object = tableDataModel.getTableColumnLabel(i);


				// format the cell if we have a table
                }
                else  {
                   object = jcTable.getDataView().
                                getTableColumnLabel(cCount);
                    if(applyJCTableStyles){

					// get the style for the cell
                        CellStyleModel csm = jcTable.getCellStyle(
                                JCTableEnum.LABEL, cCount);
                        JCPageTable.Cell cell = header.getCell(0, i + offset);

                        // set the foreground, font and alignment
                        currentStyle = setTextStyle(csm, cell.getStyle());
                        // set the background and border
                        currentDrawStyle = setDrawStyle(csm, currentDrawStyle);

                        cell.setBackgroundColor(csm.getBackground());
                        cell.setBottomBorderStyle(currentDrawStyle);
                        cell.setRightBorderStyle(currentDrawStyle);

                        table.setColumnBorder(currentDrawStyle);
                    }

				}

                if (object instanceof Image) {
                    frame.embedImage(currentStyle, (Image) object);
                } else if (object instanceof Icon) {
                    frame.embedIcon(currentStyle, (Icon) object);
                } else {
					String s = object != null ? object.toString() : "";
					frame.printWithNewlines(currentStyle, s);
				}
                cCount++;
                if (decrement){
                    numberColumns--;
                }
			}
		}


		int num_rows = 0;
        if (jcTable == null) {
            num_rows = tableDataModel.getNumRows();
        }
        else {
            num_rows = jcTable.getNumRows();
        }
        int rCount = 0;
		for (int rowIdx = 0; rowIdx < num_rows; rowIdx++) {
            if (jcTable != null){
                if (jcTable.isRowHidden(rowIdx)) {

                    rCount++;
                    num_rows--;
                }
            }
            int cCount = 0;
            int numberColumns = num_columns;
			// go through the columns accounting for the row label if present
			for (int colIdx = -offset; colIdx < numberColumns; colIdx++) {
                boolean decrement = false;
                if (jcTable != null){
                    if (jcTable.isColumnHidden(colIdx)){

                        cCount++;
                        decrement = true;
                    }
                }
				JCFrame frame = table.getCellFrame(rowIdx, colIdx + offset);

				Object object;
				if(colIdx == -1) {
                    if (jcTable == null){
					    object = tableDataModel.getTableRowLabel(rowIdx);
                    }
                    else {
                        object = jcTable.getDataView().
                                getTableRowLabel(rCount);
                    }
				}
				else {
                    if (jcTable == null){
                        object = tableDataModel.getTableDataItem(
                                rowIdx, colIdx);
                    }
                    else {
                        object = jcTable.getDataView().getTableDataItem(
                                rCount, cCount);
                    }
				}

				// format the cell if we have a table
				if(jcTable != null) {
                    if (applyJCTableStyles){
					// get the style for the cell
                        CellStyleModel csm = jcTable.getCellStyle(rCount,
                                cCount);
                        JCPageTable.Cell cell = table.getCell(
                                rowIdx, colIdx + offset);

                        // set the foreground, font and alignment
                        currentStyle = setTextStyle(csm, cell.getStyle());
                        // set the background and border
                        currentDrawStyle = setDrawStyle(csm, currentDrawStyle);

                        cell.setBackgroundColor(csm.getBackground());
                        cell.setBottomBorderStyle(currentDrawStyle);
                        cell.setRightBorderStyle(currentDrawStyle);
                    }
                }

                if (object instanceof Image) {
                    frame.embedImage(currentStyle, (Image) object);
                } else if (object instanceof Icon) {
                    frame.embedIcon(currentStyle, (Icon) object);
                } else {
					String s = object != null ? object.toString() : "";
					frame.printWithNewlines(currentStyle, s);

				}
                cCount++;
                if (decrement){
                    numberColumns--;
                }
			}
            rCount++;
		}
	}
	catch (EndOfFrameException e) {}
}


}

