/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCPageTableFromJDBC.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:22 $ $Locker:  $  Quest Software Inc.
package com.klg.jclass.page;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;

import com.klg.jclass.page.JCDocument;
import com.klg.jclass.page.JCFrame;
import com.klg.jclass.page.JCFlow;
import com.klg.jclass.page.JCPage;
import com.klg.jclass.page.JCDrawStyle;
import com.klg.jclass.page.JCTextStyle;
import com.klg.jclass.page.JCUnit;
import com.klg.jclass.page.JCPageTable;
import com.klg.jclass.page.EndOfFrameException;

import com.klg.jclass.util.swing.JCExitFrame;

import javax.swing.JOptionPane;




import java.util.Vector;


/**
 *
 * Creates a <code>JCPageTable</code> from a JDBC ResultSet.
 *
 */
public class JCPageTableFromJDBC {

public static JCTextStyle defStyle = null;
	
/**
 * Creates a <code>JCPageTable</code> from a JDBC <code>ResultSet</code>
 * and populates it with the data from that result set.
 * @param doc
 * @param resultSet
 * @return
 */
public static JCPageTable createTable(JCDocument doc, ResultSet resultSet) {
	return createTable(doc, resultSet, true);
}

/**
 * Creates a <code>JCPageTable</code> from a JDBC <code>ResultSet</code>.
 * If populate is true, then the table will be populated with the
 * contents of the result set.
 * @param doc
 * @param resultSet
 * @param populate
 * @return
 */
public static JCPageTable createTable(JCDocument doc, ResultSet resultSet,
															boolean populate) {
	JCPageTable table = null;
	ResultSetMetaData rsmd;
		// get the result set meta data
		try {
			rsmd = resultSet.getMetaData();
			
			// build up columns
			Vector columns = new Vector(rsmd.getColumnCount());								
			for (int i = 0; i < rsmd.getColumnCount(); i++) {
				
				// add the column and set its width
				int colWidth	= rsmd.getColumnDisplaySize(i + 1);
				colWidth		= Math.max(colWidth, 75); // 75 = TableColumn default
				JCUnit.Measure width= new JCUnit.Measure(JCUnit.POINTS, colWidth);
				columns.add(width);				
			}
	
			// create the table
			table = new JCPageTable(doc, columns);
			table.setAlignment(JCTextStyle.ALIGNMENT_CENTER);
			table.createHeaders();
			
		} catch (SQLException sqle) {
			JOptionPane.showMessageDialog(null, 
				sqle.toString(), 
				"SQL error", 
				JOptionPane.ERROR_MESSAGE);
			return table;
		}	
		if (populate) {
			populateTable(table, resultSet);
		}
	return table;
}

/**
 * Populates an existing <code>JCPageTable</code> from a JDBC <code>ResultSet</code>.
 * @param table
 * @param resultSet
 */
public static void populateTable(JCPageTable table, ResultSet resultSet) {

	ResultSetMetaData rsmd;
	try {
		// get the result set meta data
		try {
			rsmd = resultSet.getMetaData();
			
			// create the table header
			JCPageTable header = table.createHeaders();
			for (int i = 0; i < rsmd.getColumnCount(); i++) {
                JCTextStyle style = deriveCellStyle(header.getCell(0, i));
			 	
				// set the string to appear in the header
				JCFrame frame 	= header.getCellFrame(0, i);
			 	frame.printWithNewlines(style, rsmd.getColumnLabel(i + 1));
			}
			
			// add rows
			int rowIdx = 0;
			
			// for each row ...
			while (resultSet.next()) {
				
				// for each column ...
				for (int colIdx = 0; colIdx < rsmd.getColumnCount(); colIdx++) {
                    JCTextStyle style = deriveCellStyle(table.getCell(rowIdx, colIdx));
					JCFrame frame = table.getCellFrame(rowIdx, colIdx);
					String string = resultSet.getString(colIdx + 1);			
					frame.printWithNewlines(style, string);
				}
				rowIdx++;
			}
			
		} catch (SQLException sqle) {
			JOptionPane.showMessageDialog(null, 
				sqle.toString(), 
				"SQL error", 
				JOptionPane.ERROR_MESSAGE);
		}	
	}
	catch (EndOfFrameException e) {}
}

protected static JCTextStyle deriveCellStyle(JCPageTable.Cell cell)
{
    JCTextStyle useStyle = null;
    if (defStyle == null) {
        defStyle = JCTextStyle.stringToStyle("default text");
    }

    if (cell != null) {
        useStyle = cell.getStyle();
    }

    if (useStyle == null)
        useStyle = defStyle;

    return useStyle;
}
	
} // end JTable2JCPageTable
