/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCPageTableFromJTable.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:22 $ $Locker:  $  Quest Software Inc.
package com.klg.jclass.page;

import com.klg.jclass.page.JCDocument;
import com.klg.jclass.page.JCFrame;
import com.klg.jclass.page.JCPage;
import com.klg.jclass.page.JCPrinter;
import com.klg.jclass.page.JCDrawStyle;
import com.klg.jclass.page.JCTextStyle;
import com.klg.jclass.page.JCUnit;
import com.klg.jclass.page.JCPageTable;
import com.klg.jclass.page.EndOfFrameException;

import java.awt.Component;
import java.awt.Container;
import java.awt.Font;
import java.awt.Image;
import javax.swing.table.TableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.JTableHeader;
import javax.swing.Icon;
import javax.swing.JTable;
import javax.swing.JLabel;
import javax.swing.SwingConstants;




import java.util.Vector;


/**
 * Creates a <code>JCPageTable</code> from a swing <code>JTable</code>.
 */

public class JCPageTableFromJTable {
	
protected static int defaultColumnWidth = 75;

protected static int[][] alignment_map = {
	{SwingConstants.CENTER, JCTextStyle.ALIGNMENT_CENTER},
	{SwingConstants.LEFT, JCTextStyle.ALIGNMENT_LEFT},
	{SwingConstants.RIGHT, JCTextStyle.ALIGNMENT_RIGHT},
	{SwingConstants.LEADING, JCTextStyle.ALIGNMENT_NONE},
	{SwingConstants.TRAILING, JCTextStyle.ALIGNMENT_NONE},
};

/**
 * Creates a <code>JCPageTable</code> from a swing <code>JTable</code>.
 * The <code>JCPageTable</code> will have the same number of columns and the
 * same column widths as the list of columns in the <code>JTable</code> view.
 * @param doc      current <code>JCDocument</code> instance
 * @param jTable   <code>JTable</code> instance being converted to <code>JCPageTable</code>
 * @param populate if <code>true</code>, the created <code>JCPageTable</code> is immediately populated
 *                 with the contents and certain style elements (alignment,
 *                 font) of the <code>JTable</code>; if <code>false</code>, a
 *                 <code>JCPageTable</code> is created but no data is placed within it
 * @return
 */
public static JCPageTable createTable(JCDocument doc, JTable jTable,
									  boolean populate) {
	// get the data model from the JTable
	TableModel tableModel = jTable.getModel();

	return createTable(doc, tableModel, jTable, populate);
}

/**
 * Creates a <code>JCPageTable</code> from a swing <code>TableModel</code>.
 * The <code>JCPageTable</code> will have the same number of columns as the
 * <code>TableModel</code> and will use the default column size.
 * @param doc        current <code>JCDocument</code> instance
 * @param tableModel <code>TableModel</code> instance being converted to <code>JCPageTable</code>
 * @param populate   if <code>true</code>, the created <code>JCPageTable</code> is immediately
 *                   populated with the contents of the <code>TableModel</code>; if <code>false</code>,
 *                   a <code>JCPageTable</code> is created but no data is placed within it
 * @return
 */
public static JCPageTable createTable(JCDocument doc, TableModel tableModel,
									  						boolean populate) {
	return createTable(doc, tableModel, null, populate);
}

/**
 * Creates a <code>JCPageTable</code> from either a swing <code>TableModel</code> or
 * a swing <code>JTable</code>. If the <code>JTable</code> is non-null, the number of
 * columns and column widths are determined from the <code>JTable</code> view.  If
 * <code>JTable</code> is null, the number of columns are determined from the
 * <code>TableModel</code> and the default column width is used.
 * @param doc        current <code>JCDocument</code> instance
 * @param tableModel <code>TableModel</code> instance being converted to <code>JCPageTable</code>
 * @param jTable     <code>JTable</code> instance being converted to <code>JCPageTable</code>
 * @param populate if <code>true</code>, the created <code>JCPageTable</code> is immediately
 *                 populated with the contents of either the <code>JTable</code> or the
 *                 <code>TableModel</code> (if the <code>JTable</code> is non-null, certain
 *                 style elements (alignment, font) are also copied); if <code>false</code>,
 *                 a <code>JCPageTable</code> is created but no data is placed within it
 * @return
 */
protected static JCPageTable createTable(JCDocument doc, TableModel tableModel,
										 JTable jTable, boolean populate) {
	JCPageTable table = null;
			
	// Figure out the correct number of columns depending on whether
	// a JTable was specified or if we're just working from a TableModel
	TableColumnModel tcm = null; 
	int numColumns = 0;
	if (jTable != null) {
		tcm = jTable.getColumnModel();
		numColumns = jTable.getColumnCount();
	} else if (tableModel != null) {
		numColumns = tableModel.getColumnCount();
	}
	
	// build up columns
	Vector columns = new Vector(numColumns);

	// size columns in PageTable -- use existing TableColumnModel from
	// JTable if one exists
	for (int i = 0; i < numColumns; i++) {
		// add the column and set its width
		int colWidth = defaultColumnWidth; // JTable default
		if (tcm != null) {
			TableColumn tc 	= tcm.getColumn(i);
			colWidth		= tc.getPreferredWidth();
		}
		JCUnit.Measure width = new JCUnit.Measure(JCUnit.POINTS, colWidth);
		columns.add(width);				
	}

	// create the table
	table = new JCPageTable(doc, columns);
	// align to center of page
	table.setAlignment(JCTextStyle.ALIGNMENT_CENTER);
	// create table header
	table.createHeaders();
	// populate table if requested
	if (populate) {
		populateTable(table, tableModel, jTable, true);
	}
	return table;
}

/**
 * Populates an empty <code>JCPageTable</code> from the view of the specified
 * <code>JTable</code>, copying certain style elements (alignment, font).
 * @param table  <code>JCPageTable</code> instance to populate
 * @param jTable <code>JTable</code> instance to populate from
 */
public static void populateTable(JCPageTable table, JTable jTable) {
	populateTable(table, jTable.getModel(), jTable, true);
}

/**
 * Populates an empty <code>JCPageTable</code> from the view of the specified <code>JTable</code>.
 * @param table  <code>JCPageTable</code> instance to populate
 * @param jTable <code>JTable</code> instance to populate from
 * @param applyJTableStyles <code>true</code> if certain style elements (alignment, font)
 *                          should be applied to the <code>JCPageTable</code> cells
 *                          from the <code>JTable</code> cells
 */
public static void populateTable(JCPageTable table, JTable jTable,
												boolean applyJTableStyles) {
	populateTable(table, jTable.getModel(), jTable, applyJTableStyles);
}

/**
 * Populates an empty <code>JCPageTable</code> from the specified <code>TableModel</code>.
 * @param table      <code>JCPageTable</code> instance to populate
 * @param tableModel <code>TableModel</code> instance to populate from
 */
public static void populateTable(JCPageTable table, TableModel tableModel) {
	populateTable(table, tableModel, null, false); 
}

/**
 * Converts the <code>SwingConstants</code> alignment to a <code>JCTextStyle</code> alignment.
 * @return
 * @param alignment
 */
protected static int convertSwingToTextStyleAlignment(int alignment) {
	int new_alignment = JCTextStyle.ALIGNMENT_LEFT;

	for (int i = 0; i < alignment_map.length; i++) {
		if (alignment_map[i][0] == alignment) {
			new_alignment = alignment_map[i][1];
		}
	}
	return new_alignment;
}

/**
 * Searches for a <code>JLabel</code> and returns it. If no <code>JLabel</code> is found,
 * null is returned.
 * @return
 * @param comp
 */
protected static JLabel getLabelComponent(Component comp) {
	JLabel label = null;

	if (comp instanceof JLabel) {
		label = (JLabel)comp;
	}
	else if (comp instanceof Container) {
		Component[] children = ((Container)comp).getComponents();
		if (children == null) {
			return null;
		}
		for (int i = 0; i < children.length; i++) {
			label = getLabelComponent(children[i]);
			if (label != null) {
				break;
			}
		}
	}
	return label;			
}

/**
 * Given a renderer, the font and alignment will be set in the style.
 * @param colIdx
 * @param currentStyle
 * @param jTable
 * @param object
 * @param renderer
 * @param rowIdx
 * @return
 */
protected static JCTextStyle setFontAndAlignment(TableCellRenderer renderer,
	JCTextStyle currentStyle, JTable jTable, Object object, 
	int rowIdx, int colIdx) {

	if (renderer == null || currentStyle == null) {
		return null;
	}

	Component c = renderer.getTableCellRendererComponent(jTable, object, 
												 false, false, rowIdx, colIdx);
					
	if (c != null) {
		
		// if a JLabel exists in the renderer, get it 
		JLabel label = getLabelComponent(c);
		
		// get the current alignment
		int alignment= currentStyle.getAlignment();
		
		// see if the label has its own alignment
		if (label != null) {
			// get the label alignment
			alignment= label.getHorizontalAlignment();
			
			// convert alignment value to JCTextStyle alignment value
			alignment = convertSwingToTextStyleAlignment(alignment);

		}

		// if font or alignment changed, create the new style
		if (c.getFont() != currentStyle.getFont() || currentStyle.getAlignment() != alignment) {
			
			// something has changed, so clone the current style
			currentStyle = (JCTextStyle) currentStyle.clone();
			
			// change font if necessary
			if (c.getFont() != currentStyle.getFont()) {
				currentStyle.setFont(c.getFont());
			}
			
			// change alignment if necessary
			if (currentStyle.getAlignment() != alignment) { 
				currentStyle.setAlignment(alignment);
			}
			
			// make style immutable
			currentStyle = currentStyle.getImmutable();
	
		}
	}

	return currentStyle;
}

/**
 * Populates an empty <code>JCPageTable</code> from the specified <code>TableModel</code>
 * or <code>JTable</code>. If both a <code>JTable</code> and <code>TableModel</code> are
 * specified, the table will be populated as per the 'view' of the <code>JTable</code>,
 * although data is obtained by querying the <code>TableModel</code>. If the <code>JTable</code>
 * is non-null, certain style elements (alignment, font) will be copied to the <code>JCPageTable</code>.
 * No longer used.  Provided for backwards compatibility.
 * @param table  <code>JCPageTable</code> instance to populate
 * @param tableModel <code>TableModel</code> instance to populate from
 * @param jTable <code>JTable</code> instance to populate from
 */
protected static void populateTable(JCPageTable table, TableModel tableModel,
															JTable jTable) {
	populateTable(table, tableModel, jTable, true);
}

/**
 * Populates an empty <code>JCPageTable</code> from the specified <code>TableModel</code>
 * or <code>JTable</code>. If both a <code>JTable</code> and <code>TableModel</code> are specified,
 * the table will be populated as per the 'view' of the <code>JTable</code>, although data is
 * obtained by querying the <code>TableModel</code>.  If the <code>JTable</code> is non-null
 * and <code>applyJTableStyles</code> is <code>true</code>, certain style elements (alignment, font)
 * will be copied to the <code>JCPageTable</code>.
 * @param table  <code>JCPageTable</code> instance to populate
 * @param tableModel <code>TableModel</code> instance to populate from
 * @param jTable <code>JTable</code> instance to populate from
 * @param applyJTableStyles <code>true</code> if certain style elements (alignment, font)
 *                          should be applied to the <code>JCPageTable</code> cells
 *                          from the <code>JTable</code> cells
 */
protected static void populateTable(JCPageTable table, TableModel tableModel,
									JTable jTable, boolean applyJTableStyles) {

	JCTextStyle defaultStyle = JCTextStyle.stringToStyle("default text");
	JCTextStyle currentStyle = defaultStyle;
	JCTextStyle newStyle = null;

	TableColumnModel tcm = null;
	TableColumn tc = null;

	// determine number of columns in table depending on whether we're
	// populating the table from a JTable or a TableModel
	int numColumns = 0;
	if (jTable != null) {
		tcm = jTable.getColumnModel();
		numColumns = jTable.getColumnCount();
	} else if (tableModel != null) {
		numColumns = tableModel.getColumnCount();
	}

	// ensure that headers have been created
	JCPageTable header = table.getHeaders();
	if (header == null) {
		header = table.createHeaders();
	}

	try {
		TableCellRenderer renderer = null;

		// Populate the header
		for (int i = 0; i < numColumns; i++) {
			JCFrame frame 	= header.getCellFrame(0, i);
			Object object = "";

			// Determine cell contents from TableColumnModel -- index is of view
			if (tcm != null) {
				tc = tcm.getColumn(i);
				object = tc.getHeaderValue();
			// Determine cell contents from TableModel -- index is of data model
			} else if (tableModel != null) {
				object = tableModel.getColumnName(i);
			}

			// Determine cell styles
			newStyle = null;
			// if we want to apply styles from the jtable, look at its
			// renderers
			if (jTable != null && applyJTableStyles) {
				if (tcm != null) {
					tc = tcm.getColumn(i);
					renderer = tc.getHeaderRenderer();
				} 
				if (renderer == null) {
					JTableHeader jTableHeader = jTable.getTableHeader();
					if (jTableHeader != null) {
						renderer = jTableHeader.getDefaultRenderer();
					}
				}
				if (renderer != null) {
                    JCTextStyle style =
                        table.getHeaders().getCell(0, i).getStyle();
					newStyle = setFontAndAlignment(renderer, style, jTable,
															 object, -1, i);
				}
			}
			// catch all -- use existing style set on JCPageTable cell
			if (newStyle == null) {
				newStyle = table.getHeaders().getCell(0, i).getStyle();
				if (newStyle == null) {
					newStyle = defaultStyle;
				}
			}
			currentStyle = newStyle;

			// Populate cell with contents
            if (object instanceof Image) {
                frame.embedImage(currentStyle, (Image) object);
            } else if (object instanceof Icon) {
                frame.embedIcon(currentStyle, (Icon) object);
            } else {
				String s = object != null ? object.toString() : "";
		 		frame.printWithNewlines(currentStyle, s);
			}
		}
		
		// Populate rest of table

		// for each row ...
		for (int rowIdx = 0; rowIdx < tableModel.getRowCount(); rowIdx++) {

			// for each column ...
			for (int colIdx = 0; colIdx < numColumns; colIdx++) {

                // Determine index to data in TableModel.  If not populating
                // from a JTable, this index is the same as the loop index.
                // If populating from a JTable, the loop index is the
                // view column index and must be converted to a data model
                // index.
                int dataColumn = colIdx;
                if (jTable != null) {
                    dataColumn = jTable.convertColumnIndexToModel(colIdx);
                }
				
				// get cell for population
				JCFrame frame = table.getCellFrame(rowIdx, colIdx);

				// get object to populate cell with
				Object object = tableModel.getValueAt(rowIdx, dataColumn);

				// determine correct cell style
				newStyle = null;
				// if we want to apply styles from the jtable, look at its
				// renderers
				if (jTable != null && applyJTableStyles) {
					if (tcm != null) {
						tc = tcm.getColumn(colIdx);
						renderer = tc.getCellRenderer();
					}
					if (renderer == null) {
						renderer = jTable.getDefaultRenderer(
											jTable.getColumnClass(colIdx));
					}
					if (renderer != null) {
                        JCTextStyle style =
                            table.getCell(rowIdx, colIdx).getStyle();
						newStyle = setFontAndAlignment(renderer, 
							style, jTable, object, rowIdx, colIdx);
					} 
				}
				// catch all -- use existing style set on JCPageTable cell
				if (newStyle == null) {
					newStyle = table.getCell(rowIdx, colIdx).getStyle();
					if (newStyle == null) {
						newStyle = defaultStyle;
					}
				}
				currentStyle = newStyle;

				// Populate cell with contents
            	if (object instanceof Image) {
                	frame.embedImage(currentStyle, (Image) object);
            	} else if (object instanceof Icon) {
                	frame.embedIcon(currentStyle, (Icon) object);
            	} else {
					String string = object != null ? object.toString() : "";
					frame.printWithNewlines(currentStyle, string);
				}
			}
		}
	}
	catch (EndOfFrameException e) {}
}
	
}
