/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCPageTemplate.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:23 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page;

import java.io.File;
import java.io.Reader;
import java.io.StringReader;
import java.io.IOException;
import java.io.InputStream;

import java.awt.Color;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.NamedNodeMap;




import java.util.ArrayList;
import java.util.Iterator;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Vector;


import com.klg.jclass.util.swing.JCSwingTypeConverter;
import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.xml.JCXMLErrorPrinter;
import com.klg.jclass.util.xml.JCXMLDTDResolver;

/**
 * This class contains the methods and schema necessary to convert an
 * XML-described page template into a template <code>JCPage</code>.
 */
public class JCPageTemplate {

/**
 * Loads and verifies a system of page templates from an XML file.
 * @param doc the document to store the templates in
 * @param xmlfile the file from which to read the XML source
 * @throws IOException
 */
static public void importTemplates(JCDocument doc, File xmlfile)
				   throws SAXException, IOException, ParserConfigurationException {

	importTemplates(doc, JCXMLDTDResolver.createInputSource(xmlfile));
}

/**
 * Loads and verifies a system of page templates from an XML file.
 * @param doc the document to store the templates in
 * @param reader the source from which to read the XML page template
 * @throws IOException
 */
static public void importTemplates(JCDocument doc, Reader reader)
				   throws SAXException, IOException, ParserConfigurationException {

	InputSource in = new InputSource(reader);
	in.setSystemId("file:/");
	importTemplates(doc, in);
}

/**
 * Loads and verifies a system of page templates from an XML file.
 * @param doc the document to store the templates in
 * @param input the <code>InputSource</code> from which to read the XML page template
 * @throws IOException
 */
static public void importTemplates(JCDocument doc, InputSource input)
				   throws SAXException, IOException, ParserConfigurationException {

	List templates = loadTemplates(input);
	doc.setTemplates(templates);
}

/**
 * Loads and verifies a system of page templates from an XML file.
 * @param xmlfile the file from which to read the XML source
 * @return a List containing the template <code>JCPages</code> constructed from the XML
 * @throws IOException
 */
static public List loadTemplates(File xmlfile)
				   throws SAXException, IOException, ParserConfigurationException {

	return loadTemplates(JCXMLDTDResolver.createInputSource(xmlfile));
}

/**
 * Loads and verifies a system of page templates from an XML stream.
 * @param reader the source from which to read the XML page template
 * @return a List containing the template <code>JCPages</code> constructed from the XML
 * @throws IOException
 */
static public List loadTemplates(Reader reader)
				   throws SAXException, IOException, ParserConfigurationException {

	InputSource in = new InputSource(reader);
	in.setSystemId("file:/");
	return loadTemplates(in);
}

/**
 * Gets the name attribute of the current node.
 * @param node the node whose name is to be retrieved
 * @return the name of the node
 */
static protected String getNameAttribute(Node node) {
	NamedNodeMap attributes = node.getAttributes();
	Node name_node = attributes.getNamedItem("NAME");
	if (name_node == null) {
		return null;
	}
	return name_node.getNodeValue();
}

/**
 * Returns the color; null means transparent, no color.
 * @param node the node whose color is to be retrieved
 * @return the color of the node
 */
static protected Color getColorAttribute(Node node) {
	NamedNodeMap attributes = node.getAttributes();
	Node color_node = attributes.getNamedItem("COLOR");
	if (color_node == null) {
		return null;
	}
	return JCSwingTypeConverter.toColor(color_node.getNodeValue());
}

/**
 * Returns whether the template is considered a starting point or not.
 * @param node the node whose first marker attributed is to be checked
 * @return <code>true</code> if the node is marked to be the first page used
 */
static protected boolean getFirstAttribute(Node node) {
	NamedNodeMap attributes = node.getAttributes();
	Node first_node = attributes.getNamedItem("FIRST");
	if (first_node == null) {
		return false;
	}
	return JCTypeConverter.toBoolean(first_node.getNodeValue(), false);
}

/**
 * Returns the orientation method for this node.  If a method was not 
 * specified, then the default "automatic" applies.
 * @param node the node whose orientation method is to be retrieved
 * @return the orientatiuon method used by this node
 */
static protected int getOrientationAttribute(Node node) {
	NamedNodeMap attributes = node.getAttributes();
	Node orientation_node = attributes.getNamedItem("ORIENTATION");
	if (orientation_node == null) {
		return JCPage.ORIENTATION_AUTOMATIC;
	}
	String orientation_text = orientation_node.getNodeValue();
	int orientation = JCPage.ORIENTATION_AUTOMATIC;
	if (orientation_text.equals("portrait")) {
		orientation = JCPage.ORIENTATION_PORTRAIT;
	}
	else if (orientation_text.equals("landscape")) {
		orientation = JCPage.ORIENTATION_LANDSCAPE;
	}

	return orientation;
}

/**
 * Returns the <code>JCUnit</code> specified for this node.  If a unit was not specified this should
 * still be passed a valid node with "inches" as the default value
 * @param node the node whose units are to be retrieved
 * @return the units used to measure the node
 */
static protected JCUnit getUnitAttribute(Node node) {
	NamedNodeMap attributes = node.getAttributes();
	Node unit_node = attributes.getNamedItem("UNIT");
	// don't check for null because the DTD should always handle providing
	// a node for us
	String unit_text = unit_node.getNodeValue();
	JCUnit unit = null;
	if (unit_text.equals("inches")) {
		unit = JCUnit.INCHES;
	}
	else if (unit_text.equals("points")) {
		unit = JCUnit.POINTS;
	}
	else if (unit_text.equals("cm")) {
		unit = JCUnit.CM;
	}
	else if (unit_text.equals("cms")) {
		unit = JCUnit.CM;
	}
	else if (unit_text.equals("centimeters")) {
		unit = JCUnit.CM;
	}
	else if (unit_text.equals("centimetres")) {
		unit = JCUnit.CM;
	}

	return unit;
}

static protected JCUnit.Point getLocation(Node parent_node, JCUnit unit) {
	NodeList location_node_list = ((Element)parent_node).
										getElementsByTagName("LOCATION");
	Node node = location_node_list.item(0);

	NamedNodeMap attributes = node.getAttributes();
	double x, y;

	Node x_node = attributes.getNamedItem("X");
	if (x_node == null) {
		x = 0.0;
	}
	else {
		try {
			x = Double.valueOf(x_node.getNodeValue()).doubleValue();
		}
		catch (NumberFormatException nfe) {
			// ??? throw error ???
			x = 0.0;
		}
	}

	Node y_node = attributes.getNamedItem("Y");
	if (y_node == null) {
		y = 0.0;
	}
	else {
		try {
			y = Double.valueOf(y_node.getNodeValue()).doubleValue();
		}
		catch (NumberFormatException nfe) {
			// ??? throw error ???
			y = 0.0;
		}
	}

	return new JCUnit.Point(unit, x, y);
}

static protected JCUnit.Dimension getSize(Node parent_node, JCUnit unit) {
	NodeList size_node_list = ((Element)parent_node).
										getElementsByTagName("SIZE");
	Node node = size_node_list.item(0);

	NamedNodeMap attributes = node.getAttributes();
	double width, height;

	Node width_node = attributes.getNamedItem("WIDTH");
	if (width_node == null) {
		width = 0.0;
	}
	else {
		try {
			width = Double.valueOf(width_node.getNodeValue()).doubleValue();
		}
		catch (NumberFormatException nfe) {
			// ??? throw error ???
			width = 0.0;
		}
	}

	Node height_node = attributes.getNamedItem("HEIGHT");
	if (height_node == null) {
		height = 0.0;
	}
	else {
		try {
			height = Double.valueOf(height_node.getNodeValue()).doubleValue();
		}
		catch (NumberFormatException nfe) {
			// ??? throw error ???
			height = 0.0;
		}
	}

	return new JCUnit.Dimension(unit, width, height);
}

static protected ColumnDefinition getColumnSetup(Node parent_node, JCUnit unit) {

	NodeList column_node_list = ((Element)parent_node).
										getElementsByTagName("COLUMN");
	Node node = column_node_list.item(0);

	int             count = 1;
	JCUnit.Measure  spacing = new JCUnit.Measure(JCUnit.INTERNAL, 0.25);

	if (node != null) {
		NamedNodeMap attributes = node.getAttributes();

		Node count_node = attributes.getNamedItem("COUNT");
		if (count_node != null) {
			try {
				count = Integer.valueOf(count_node.getNodeValue()).intValue();
			}
			catch (NumberFormatException nfe) {
				// ??? throw error ???
				count = 1;
			}
		}

		Node spacing_node = attributes.getNamedItem("SPACING");
		if (spacing_node != null) {
			try {
				double space =
					Double.valueOf(spacing_node.getNodeValue()).doubleValue();
				spacing = new JCUnit.Measure(unit, space);
			}
			catch (NumberFormatException nfe) {
				// ??? throw error ???
				spacing = new JCUnit.Measure(JCUnit.INTERNAL, 0.25);
			}
		}
	}

	return(new ColumnDefinition(count, spacing));
}

/**
 * Returns the border type specified for this node.
 * @return border type specified for this node
 * @param node
 */
static protected int getBorderType(Node node) {

	NamedNodeMap attributes = node.getAttributes();
	Node type_node = attributes.getNamedItem("TYPE");

	// don't check for null because the DTD should always handle providing
	// a node for us

	String type_text = type_node.getNodeValue();
	int type = JCDrawStyle.LINE_TYPE_NONE;

	if (type_text.equals("blank")) {
		type = JCDrawStyle.LINE_TYPE_NONE;
	}
	else if (type_text.equals("broken")) {
		type = JCDrawStyle.LINE_TYPE_BROKEN;
	}
	else if (type_text.equals("dashed")) {
		type = JCDrawStyle.LINE_TYPE_BROKEN;
	}
	else if (type_text.equals("double")) {
		type = JCDrawStyle.LINE_TYPE_DOUBLE;
	}
	else if (type_text.equals("none")) {
		type = JCDrawStyle.LINE_TYPE_NONE;
	}
	else if (type_text.equals("plain")) {
		type = JCDrawStyle.LINE_TYPE_PLAIN;
	}
	else if (type_text.equals("regular")) {
		type = JCDrawStyle.LINE_TYPE_PLAIN;
	}
	else if (type_text.equals("single")) {
		type = JCDrawStyle.LINE_TYPE_PLAIN;
	}

	return type;
}

static protected JCUnit.Measure getBorderThickness(Node node, JCUnit units) {

	NamedNodeMap attributes = node.getAttributes();
	Node width_node = attributes.getNamedItem("THICKNESS");
	JCUnit.Measure thickness = new JCUnit.Measure();

	if (width_node == null) {
		thickness = new JCUnit.Measure(JCUnit.POINTS, 0.1);
	}
	else {
		try {
			double width = Double.valueOf(width_node.getNodeValue()).doubleValue();
			thickness = new JCUnit.Measure(units, width);
		}
		catch (NumberFormatException nfe) {
			// ??? throw error ???
			thickness = new JCUnit.Measure(JCUnit.POINTS, 0.1);
		}
	}

	return(thickness);
}

static protected BorderDefinition getBorderSetup(Node parent_node, JCUnit unit) {

	NodeList border_node_list = ((Element)parent_node).
										getElementsByTagName("BORDER");
	Node node = border_node_list.item(0);
	int             style = JCDrawStyle.LINE_TYPE_NONE;
	Color           color = Color.black;
	JCUnit.Measure  thickness = new JCUnit.Measure(JCUnit.POINTS, 0.1);

	if (node != null) {

		style = getBorderType(node);
		color = getColorAttribute(node);
		if (color == null) {
			color = Color.black;
		}
		thickness = getBorderThickness(node, unit);
	}

	return(new BorderDefinition(style, color, thickness));
}

static protected JCFrame getFrameFromListByName(List frames, String name) {
	JCFrame frame = null;
	// lookup proper frame object by name
	for (int i = 0; i < frames.size(); i++) {
		if (((JCFrame)frames.get(i)).getName().equals(name)) {
			frame = (JCFrame) frames.get(i);
			break;
		}
	}
	return frame;
}

static protected double getMarginSide(NamedNodeMap attributes, String location)
{

	Node   node;
	double width;

	node = attributes.getNamedItem(location);
	if (node == null) {
		width = 0.0;
	}
	else {
		try {
			width = Double.valueOf(node.getNodeValue()).doubleValue();
		}
		catch (NumberFormatException nfe) {
			// ??? throw error ???
			width = 0.0;
		}
	}
	return(width);
}

static protected JCUnit.Margins getMargins(Node parent_node, JCUnit unit) {

	NodeList margin_node_list = ((Element)parent_node).
										getElementsByTagName("MARGIN");
	Node node = margin_node_list.item(0);
	double          width;
	JCUnit.Margins  margins = new JCUnit.Margins(unit);

	if (node != null) {
		NamedNodeMap attributes = node.getAttributes();

		width = getMarginSide(attributes, "TOP");
		margins.setTop(new JCUnit.Measure(unit, width));

		width = getMarginSide(attributes, "LEFT");
		margins.setLeft(new JCUnit.Measure(unit, width));

		width = getMarginSide(attributes, "RIGHT");
		margins.setRight(new JCUnit.Measure(unit, width));

		width = getMarginSide(attributes, "BOTTOM");
		margins.setBottom(new JCUnit.Measure(unit, width));
	}

	return(margins);
}


static protected JCPage getTemplateByName(List templates, String name) {
	for (int i = 0; i < templates.size(); i++) {
		if (((JCPage) templates.get(i)).getName().equals(name)) {
			return (JCPage) templates.get(i);
		}
	}
	return null;
}

/**
 * Loads and verifies a system of page templates from an XML stream.
 * @param input the source from which to read an XML-formatted page template
 * @return a List containing the template <code>JCPages</code> constructed from the XML
 * @throws IOException
 */
static public List loadTemplates(InputSource input)
			   throws SAXException, IOException, ParserConfigurationException {

	Document xml = null;
	List templates = new ArrayList();
	try {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setValidating(true);

		DocumentBuilder builder = factory.newDocumentBuilder();

		//
		// Configure the parser
		//
		builder.setErrorHandler(new JCXMLErrorPrinter());
		builder.setEntityResolver(
			new JCXMLDTDResolver("JCPageTemplate.dtd", JCPageTemplate.class));

		xml = builder.parse(input);

	}
	catch (ParserConfigurationException e) {
		e.printStackTrace(System.err);
		throw e;
	} catch (SAXParseException e) {
		// already reported by error printer
		e.printStackTrace(System.err);
		throw e;
	}
	catch (SAXException e) {
		e.printStackTrace (System.err);
		throw e;
	}
	catch (IOException e) {
		e.printStackTrace (System.err);
		throw e;
	}

	if (xml == null) {
		// ??? error message
		return null;
	}

	// Temporary variables used often
	Color color;
	String name;
	int orientation;
	boolean first;
	JCUnit.Point location;
	JCUnit.Dimension size;

	//
	// First Pass
	// create page templates
	//

	NodeList page_node_list = xml.getElementsByTagName("PAGE");
	for (int i = 0; i < page_node_list.getLength(); i++) {
		Node page_node = page_node_list.item(i);

		//
		// Handle attributes
		//

		// name
		name = getNameAttribute(page_node);

		// unit
		JCUnit page_unit = getUnitAttribute(page_node);

		// orientation
		orientation = getOrientationAttribute(page_node);

		// color
		color = getColorAttribute(page_node);

		// first
		first = getFirstAttribute(page_node);

		//
		// Handle location
		//
		location = getLocation(page_node, page_unit);

		//
		// Handle Size
		//
		size = getSize(page_node, page_unit);
		

		//
		// Create the page object
		//
		JCPage page = new JCPage(name, location, size);
		//System.err.println("Created page name = " + name);
		page.setColor(color);
		page.setFirstTemplate(first);
		page.setOrientation(orientation);
		page.setPageType(JCPage.PAGE_TYPE_TEMPLATE);

		//
		// Handle Frames
		//
		List frames = new ArrayList(10);
		NodeList frame_node_list = ((Element)page_node).
										getElementsByTagName("FRAME");
		for (int j = 0; j < frame_node_list.getLength(); j++) {
			Node frame_node = frame_node_list.item(j);

			//
			// Handle attributes
			//

			// name
			name = getNameAttribute(frame_node);

			// unit
			JCUnit frame_unit = getUnitAttribute(frame_node);

			// color
			color = getColorAttribute(frame_node);

			//
			// handle location
			//
			location = getLocation(frame_node, frame_unit);

			//
			// handle size
			//
			size = getSize(frame_node, frame_unit);

			//
			// Border appearance
			//
			BorderDefinition bd = getBorderSetup(frame_node, frame_unit);

			//
			// Check for multiple columns
			//
			ColumnDefinition cd = getColumnSetup(frame_node, frame_unit);

			//
			// Margins around frame
			//
			JCUnit.Margins margin = getMargins(frame_node, frame_unit);

			JCFrame frame = new JCFrame(name, location, size);
			frame.setColor(color);

			frame.setColumnCount(cd.getCount());
			frame.setColumnSpace(cd.getSpacing());

			frame.setMargins(margin);

			JCDrawStyle border = (JCDrawStyle) JCDrawStyle.BLANK.clone();
			border.setLineWidth(bd.getThickness());
			border.setForegroundColor(bd.getColor());
			border.setLineType(bd.getType());

			frame.setBorder(border);

			frames.add(frame);
		}
		page.setFrameList(frames);

		//
		// handle flowframes
		//
		List flowframes = new ArrayList();
		NodeList flow_frame_node_list = ((Element)page_node).
										getElementsByTagName("FLOWFRAME");
		for (int j = 0; j < flow_frame_node_list.getLength(); j++) {
			Node flow_frame_node = flow_frame_node_list.item(j);

			name = getNameAttribute(flow_frame_node);

			JCFrame flowframe = getFrameFromListByName(frames, name);
			//System.err.println("**** flowframe = " + flowframe.getName());
			if (flowframe == null) {
				// ???
				System.err.println("Blowed up real good");
			}
			flowframes.add(flowframe);
		}
		page.setFlowFrameList(flowframes);

		//
		// handle flow page
		//
		// shouldn't need the following call
		NodeList flow_page_node_list = ((Element)page_node).
										getElementsByTagName("FLOWPAGE");
		Node flow_page_node = flow_page_node_list.item(0);
		name = getNameAttribute(flow_page_node);

		// create temporary page
		JCPage flow_page = new JCPage(name);
		flow_page.setPageType(JCPage.PAGE_TYPE_TEMPLATE);
		page.setFlowPageTemplate(flow_page);

		//
		// handle flow section
		//
		NodeList flow_section_node_list = ((Element)page_node).
										getElementsByTagName("FLOWSECTION");
		Node flow_section_node = flow_section_node_list.item(0);
		name = getNameAttribute(flow_section_node);

		// create temporary page
		JCPage flow_section = new JCPage(name);
		flow_section.setPageType(JCPage.PAGE_TYPE_TEMPLATE);
		page.setFlowSectionTemplate(flow_section);

		//
		// add page to page map
		//
		templates.add(page);
	}

	//
	// Second Pass
	// Validate flow page and flow section references
	//
	for (int j = 0; j < templates.size(); j++) {
		JCPage page, real_page, temp_page;
		page = (JCPage) templates.get(j);
		// replace temp flow page with real flow page
		temp_page = page.getFlowPageTemplate();
		real_page = getTemplateByName(templates, temp_page.getName());

		// If a template of the specified name was found, set it as the flow
		// page template, otherwise use this page as the flow page template
		if (real_page != null) {
			page.setFlowPageTemplate(real_page);
		}
		else {
			page.setFlowPageTemplate(page);
		}

		// replace temp flow section with real flow section
		temp_page = page.getFlowSectionTemplate();
		real_page = getTemplateByName(templates, temp_page.getName());

		// If a template of the specified name was found, set it as the flow
		// section template, otherwise use this page as the section template
		if (real_page != null) {
			page.setFlowSectionTemplate(real_page);
		}
		else {
			page.setFlowSectionTemplate(page);
		}
	}

	return templates;
}

/**
 * Loads page templates from a file given as the first argument.
 * @param args
 */
public static void main(String args[]) {
	try {
		JCPageTemplate.loadTemplates(new File(args[0]));
	}
	catch (Exception e) {
		System.err.println("Threw excetopnm = " + e);
		e.printStackTrace();
	}
}

/**
 * This class stores/encapsulates the attributes of a page/frame/table border.
 */
static class BorderDefinition {

	int             type;
	Color           colour;
	JCUnit.Measure  line_width;

	public BorderDefinition(int type, Color color, JCUnit.Measure width) {
		this.type = type;
		this.colour = color;
		this.line_width = width;
	}

	public int getType() {
		return(type);
	}

	public Color getColor() {
		return(colour);
	}

	public JCUnit.Measure getThickness() {
		return(line_width);
	}
}

/**
 * This class stores the attributes which define the layout of a multi-column
 * frame.
 */
static class ColumnDefinition {

	int             count;
	JCUnit.Measure  spacing;

	public ColumnDefinition(int count, JCUnit.Measure spacing) {
		this.count = count;
		this.spacing = spacing;
	}

	public int getCount() {
		return(count);
	}

	public JCUnit.Measure getSpacing() {
		return(spacing);
	}
}

}
