/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCPrintEvent.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:23 $ $Locker:  $  Quest Software Inc.
package com.klg.jclass.page;

import java.util.EventObject;

/**
 * Encapsulates notification of printing events.
 */
public class JCPrintEvent extends EventObject {

public static final int BEGIN_PAGE 		= 1;
public static final int END_PAGE		= 2;
public static final int OPEN_DOCUMENT	= 3;
public static final int CLOSE_DOCUMENT	= 4;

/** The identifier indicating the event that has occurred. */
protected int           eventId;

/** The page number related to this event. */
protected int 			pageNumber 		= -1;


/**
 * Encapsulates notification of printing events.
 * @param source the <code>JCPrinter</code> sending this event
 * @param eventId the type of event being sent
 */
public JCPrintEvent(JCPrinter source, int eventId) {
	this(source, eventId, -1);
}

/**
 * Encapsulates notification of printing events.
 * @param source the <code>JCPrinter</code> sending this event
 * @param eventId the type of event being sent
 * @param pageNumber the page number related to this event
 */
public JCPrintEvent(JCPrinter source, int eventId, int pageNumber) {

	super(source);

	switch (eventId) {
	case BEGIN_PAGE:
	case END_PAGE:
	case OPEN_DOCUMENT:
	case CLOSE_DOCUMENT:
		break;
	default:
		throw new IllegalArgumentException("invalid value for event_id");
	}

	this.eventId 	= eventId;
	this.pageNumber = pageNumber;
}

/**
 * Returns the page number, or -1 if not applicable.
 * @return the page number related to this event
 */
public int getPageNumber() {
	return pageNumber;
}

/**
 * Returns the <code>eventId</code>.
 * @return the type of event
 */
public int getEventId() {
	return eventId;
}

}
