/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCPrinter.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:23 $ $Locker:  $  Quest Software Inc.
package com.klg.jclass.page;


import com.klg.jclass.util.JCListenerList;

import javax.swing.tree.TreeModel;
import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.util.Enumeration;
import java.util.List;


/**
 * Deprecated, not for public usage.
 *
 * Interface methods for document handling.
 *
 */
public abstract class JCPrinter {

/** The graphics which represent the output drawable for this printer. */
protected Graphics2D    graphics;
/** Current output stream on which "printed" graphics are written. */
protected OutputStream  os;
/** Are multiple copies interleaved (as opposed to each page copied in turn). */
protected boolean       collate         = true; // FIX default?
/** Are pages output last-page-first. */
protected boolean       reverseOrder    = false;
/** The number of copies of each page printed. */
protected int           numCopies       = 1;
/** The last page which will be printed (-1 indicates last available). */
protected int           endPage         = -1;
/** The first page which will be printed (zero-based). */
protected int           startPage       = 0;
/** The number of pages which have been printed. */
protected int           pageCount       = 0;
/** The listeners for flow events. */
protected JCListenerList printListeners = null;
/** Flags if the document is open for printing. */
protected boolean       isDocOpen       = false;
/** The orientation of the current page printing. */
protected int           orientation     = JCPage.ORIENTATION_AUTOMATIC;
/** <code>True</code> if output should be compressed. */
protected boolean       compressed      = true;
/** The Bookmark Tree used for this output -- usually a <code>DefaultTreeModel</code> class. */
protected TreeModel     bookmarkTree    = null;
/** The visibility of the Bookmark Tree. */
protected boolean       bookmarkTreeVisible = false;
/** The number of the next image created on a page. */
public int              imageNumber;
/* Draw cache for java components.  This is cleared during closeDocument(). */
//protected Hashtable     componentDrawCache = new Hashtable();

/**
 *
 * Constructor
 *
 */
public JCPrinter() {
    //
}

/**
 * Creates default styles, any other one-time initialization.
 * Builds font families from available fonts.
 */
public abstract void setup();

/**
 * Returns <code>true</code> if printer does own image scaling (as in PDF, PostScript and
 * PCL) or <code>false</code> if we need to tell the java Image class to scale itself.
 */
public abstract boolean isImageScalable();

/**
 * Any required initialization and event notification.
 * @param pageSize a rectangle describing the size and location of the page
 */
public void openDocument(Rectangle2D pageSize) {

    // Note that the document is open
    isDocOpen = true;
    imageNumber = 0;

    // Send a start-of-printing event to interested listeners
    JCPrintEvent event = new JCPrintEvent(this, JCPrintEvent.OPEN_DOCUMENT);
    Enumeration e = JCListenerList.elements(printListeners);
    for (; e.hasMoreElements();) {
        JCPrintListener listener = (JCPrintListener) e.nextElement();
        listener.openDocument(event);
    }
}

/**
 * Document completion (for example, PDF cross-ref table and trailer)
 * and event notification.
 * @param pageSize a rectangle describing the size and location of the page
 */
public void closeDocument(Rectangle2D pageSize) {

    // Note that the document is closed
    isDocOpen = false;

    //componentDrawCache.clear();

    // Send an end-of-printing event to interested listeners
    JCPrintEvent event = new JCPrintEvent(this, JCPrintEvent.CLOSE_DOCUMENT);
    Enumeration e = JCListenerList.elements(printListeners);
    for (; e.hasMoreElements();) {
        JCPrintListener listener = (JCPrintListener) e.nextElement();
        listener.closeDocument(event);
    }
}

/**
 * Starts a page, since there may be printer-specific actions to take
 * (for example, PS Document Structuring Conventions comments, page orientation...).
 * Does event notification, too.
 * @param pageSize a rectangle describing the size and location of the page
 * @param pageNumber the number of the new page
 */
public void beginPage(Rectangle2D pageSize, int pageNumber) {
    JCPrintEvent event = new JCPrintEvent(this, JCPrintEvent.BEGIN_PAGE, pageNumber);
    Enumeration e = JCListenerList.elements(printListeners);
    for (; e.hasMoreElements();) {
        JCPrintListener listener = (JCPrintListener) e.nextElement();
        listener.beginPage(event);
    }
}

/**
 * Ends a page (PDF Page bounding box...) and sends event notification.
 * @param pageSize a rectangle describing the size and location of the page
 */
public void endPage(Rectangle2D pageSize) {
    JCPrintEvent event = new JCPrintEvent(this, JCPrintEvent.END_PAGE);
    Enumeration e = JCListenerList.elements(printListeners);
    for (; e.hasMoreElements();) {
        JCPrintListener listener = (JCPrintListener) e.nextElement();
        listener.endPage(event);
    }
}

/**
 * Checks if the document is open
 * @return <code>True</code> if the document has been opened for printing
 */
public boolean isDocumentOpen() {
    return (isDocOpen);
}

/**
 * Sets the value of the <code>Compressed</code> property, which determines whether
 * the content of pages will be compressed. <br><B>Note:</b> Even when set to
 * <code>true</code>, the functionality of this property depends upon the existence
 * of compression in the output format.
 * @param compressed <code>true</code> if compression should be performed; <code>false</code> otherwise
 */
public void setCompressed(boolean compressed) {
    this.compressed = compressed;
}

/**
 * Gets the value of the <code>Compressed</code> property, which determines whether
 * the content of pages will be compressed. <br><B>Note:</b> Even when set to
 * <code>true</code>, the functionality of this property depends upon the existence
 * of compression in the output format.
 * @return <code>true</code> if compression will be performed; <code>false</code> otherwise
 */
public boolean isCompressed() {
    return compressed;
}

/**
 * Sets the collate flag.
 * @param collate <code>True</code> means that the complete document is output
 * for each copy
 */
public void setCollate(boolean collate) {
    this.collate = collate;
}

/**
 * Gets the collate flag.
 * @return <code>True</code> means that the complete document is output
 *  for each copy
 */
public boolean getCollate() {
    return collate;
}

/**
 * Sets number of copies.
 * @param numCopies integer indicating the number of copies to be printed
 */
public void setNumCopies(int numCopies) {

    if (numCopies < 1) {
        throw new IllegalArgumentException("Number of Copies must be >= 1.");
    }

    this.numCopies = numCopies;
}

/**
 * Gets number of copies.
 * @return numCopies integer indicating the number of copies to be printed
 */
public int getNumCopies() {
    return numCopies;
}

/**
 * Sets end page in the range to print.
 * @param endPage an int the last page to print
 */
public void setOutputPageEnd(int endPage) {
    this.endPage = endPage;
}

/**
 * Gets end page in the range to print.
 * @return endPage an int the last page to print
 */
public int getOutputPageEnd() {
    return endPage;
}

/**
 * Sets the first page in the range to print.
 * @param startPage an int, the first page in the range to print
 */
public void setOutputPageStart(int startPage) {

    if (startPage < 0) {
        throw new IllegalArgumentException("Start page must be > 0.");
    }

    this.startPage = startPage;
}

/**
 * Gets the first page in the range to print.
 * @return startPage an int, the first page in the range to print
 */
public int getOutputPageStart() {
    return startPage;
}

/**
 * Sets output stream for PostScript, PDF and PCL printers.
 * @param os the stream to which output will be written
 */
public void setOutputStream(OutputStream os) {

    if (os == null) {
        throw new IllegalArgumentException("Output Stream cannot be <null>.");
    }

    this.os = os;
    Graphics2DBase g2db = (Graphics2DBase) getGraphics();
    g2db.setOutputStream(os);
}

/**
 * Gets output stream.
 * @return the stream to which output will be written
 */
public OutputStream getOutputStream() {
    return os;
}

/**
 * Sets reverse order
 * @param reverseOrder <code>True</code> if pages are to be printed
 * in reverse order
 */
public void setReverseOrder(boolean reverseOrder) {
    this.reverseOrder = reverseOrder;
}

/**
 * Gets reverse order
 * @return <code>True</code> if pages are to be printed in reverse order,
 *  <code>false</code> otherwise
 */
public boolean getReverseOrder() {
    return reverseOrder;
}

/**
 * Gets a list of available font families.
 * @return a list of font families supported by this printer
 */
public List getFontFamilies() {
    return null;
}

///**
// * Sets a Bookmark Tree for output.  A Bookmark Tree is a hierarchical
// * collection of text and destination pairs that is used to create a table of
// * contents-like structure outside of the document that links to locations
// * inside the document.  In PDF output, this is the tree of links that
// * can be found inside the "Bookmarks" tab in the Acrobat PDF viewer.
// * If this property is set to null (the default), no Bookmark Tree wil
// * be created.
// * <p>
// * For more information on the contents of a Bookmark Tree see
// * <code>JCDocument.setBookmarkTree</code>.
// * @param bookmarkTree the Bookmark Tree to output
// */
//public void setBookmarkTree(TreeModel bookmarkTree) {
//    this.bookmarkTree = bookmarkTree;
//}
//
///**
// * Gets the Bookmark Tree that will be output.  A Bookmark Tree is a
// * hierarchical collection of text and destination pairs that is used to create
// * a table of contents-like structure outside of the document that links to
// * locations inside the document.  In PDF output, this is the tree of links that
// * can be found inside the "Bookmarks" tab in the Acrobat PDF viewer.
// * If this property is set to null (the default), no Bookmark Tree wil
// * be created.
// * <p>
// * For more information on the contents of a Bookmark Tree see
// * <code>JCDocument.setBookmarkTree</code>.
// * @return the Bookmark Tree to output
// */
//public TreeModel getBookmarkTree() {
//    return bookmarkTree;
//}
//
///**
// * Sets the state of the Bookmark Tree.   In PDF output, a value of
// * <code>true</code> means that the Bookmark tab will automatically expand when the
// * document is opened by the viewer.  A value of <code>false</code> (the default)
// * means that the Bookmark tab will be existent, but not expanded until
// * the user chooses to expand it.
// * @param bookmarkTreeVisible the desired visibility of the Bookmark tree
// */
//public void setBookmarkTreeVisible(boolean bookmarkTreeVisible) {
//    this.bookmarkTreeVisible = bookmarkTreeVisible;
//}
//

/**
 * Not supported in PageLayout. Exists for compatibility.
 * Gets the state of the Bookmark Tree.   In PDF output, a value of
 * <code>true</code> means that the Bookmark tab will automatically expand when the
 * document is opened by the viewer.  A value of <code>false</code> (the default)
 * means that the Bookmark tab will be existent, but not expanded until
 * the user chooses to expand it.
 * @return the visibility of the Bookmark tree
 */
public boolean isBookmarkTreeVisible() {
    return bookmarkTreeVisible;
}

/**
 * Returns <code>True</code> if this printer supports hyperlinks.  Should be overridden
 * by printer subclasses that wish to support hyperlinks.
 * @return <code>True</code> if printer supports hyperlinks; <code>false</code> otherwise
 */
public boolean supportsHyperlinks() {
    return false;
}

///**
// * Adds hyperlink at specified location.  If printer does not support
// * hyperlinks, this method does nothing.
// * @param rect     <code>Rectangle2D.Double</code> object encompassing region of active
// *                 hyperlink
// * @param linkInfo <code>HyperlinkInfo</code> object specifying destination of
// *                 hyperlink
// */
//public void addHyperlink(Rectangle2D.Double rect, HyperlinkInfo linkInfo) {
//}

///**
// *  Associates a location in the document with a name for future reference
// *  by a hyperlink.  If printer does not support hyperlinks,
// *  this method does nothing.
// *  @param location <code>Point2D.Double</CODE> object specifying location on the
// *                  current page that should be associated with the provided
// *                  name
// *  @param name     name to associate with the provided location
// */
//public void addHyperlinkLocation(Point2D.Double location, String name) {
//}

/**
 * Gets the graphics context used by this printer.
 */
public Graphics getGraphics() {
    return graphics;
}

/**
 *  Sets the orientation of the page to be printed.  Used only for temporary
 *  storage;  this is  not a property of the printer.  Valid values are
 *  <code>JCPage.AUTOMATIC</code> (to be determined by the page size), <code>JCPage.PORTRAIT</code>,
 *  or <code>JCPage.LANDSCAPE</code>.
 *  @param orient the orientation of this page
 */
protected void setOrientation(int orient) {
    orientation = orient;
}

/**
 *  Gets the page orientation.  Used only for temporary storage;  this is
 *  not a property of the printer.  Valid values are <code>JCPage.AUTOMATIC</code>
 *  (to be determined by the page size), <code>JCPage.PORTRAIT</code>, or
 *  <code>JCPage.LANDSCAPE</code>.
 *  @return the orientation of the page being printed.
 */
protected int getOrientation() {
    return orientation;
}

/**
 * Is this a landscape page? The method returns <code>true</code> if the width
 * is greater than the height.
 */
protected boolean isLandscape(double width, double height) {
    // take the abs value to handle cases where axis is negative
    if (orientation == JCPage.ORIENTATION_AUTOMATIC) {
        return Math.abs(width) > Math.abs(height);
    }
    else if (orientation == JCPage.ORIENTATION_LANDSCAPE) {
        return true;
    }
    else {
        return false;
    }
}

/**
 * Writes the given String to the output stream, adjusting as necessary.
 * Write a carriage return character after the end of the string,
 * just to keep things nicely readable.
 * @param string the commands or data to be written to the output stream
 */
protected void output(String string) {
    try {
        try {
            os.write(string.getBytes("US-ASCII"));
        } catch (UnsupportedEncodingException uee) {
            os.write(string.getBytes());
        }
        os.write('\n');
    } catch (IOException e) {
        throw new PrintException(e.toString());
    }
}

/**
 * Write the given bytes to the output stream.  Do *not* write
 * a carriage return at the end of the bytes.  Leave that up to the
 * caller.
 */
protected void output(byte[] bytes) {
    try {
        os.write(bytes);
    } catch (IOException e) {
        throw new PrintException(e.toString());
    }
}

/**
 * Write the specified bytes to the output stream.  Do *not* write
 * a carriage return at the end of the bytes.  Leave that up to the
 * caller.
 */
protected void output(byte[] bytes, int off, int len) {
    try {
        os.write(bytes, off, len);
    } catch (IOException e) {
        throw new PrintException(e.toString());
    }
}

/**
 * Write the specified byte to the output stream.  Do *not* write
 * a carriage return after the bytes.  Leave that up to the
 * caller.
 */
protected void output(byte oneByte) {
    try {
        os.write(oneByte);
    } catch (IOException e) {
        throw new PrintException(e.toString());
    }
}

/**
 * Adds a listener for print events.
 * @see JCPrintEvent
 * @see JCPrintListener
 */
public void addPrintListener(JCPrintListener listener) {
    printListeners = JCListenerList.add(printListeners, listener);
}

/**
 * Removes a listener for print events.
 */
public void removePrintListener(JCPrintListener listener) {
    printListeners = JCListenerList.remove(printListeners, listener);
}


}
