/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCTableStyle.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:23 $ $Locker:  $  Quest Software Inc.
package com.klg.jclass.page;

import java.awt.Color;
import java.util.Enumeration;

/**
 * <code>JCTableStyle</code> encapsulates table styles.
 * New styles can be created and existing ones can be modified
 * using this class.
 */
public class JCTableStyle implements Cloneable {

// Style properties

// /** The name of this style. */
protected 					String name;
/** The bottom border is drawn at the bottom of a table, or parts of it. */
protected JCDrawStyle       bottomBorder;
/** The top border is drawn at the top of a table, above the header table. */
protected JCDrawStyle       topBorder;
/** The left border is drawn at the left of a table, or parts of it. */
protected JCDrawStyle       leftBorder;
/** The right border is drawn at the right of a table, or parts of it. */
protected JCDrawStyle       rightBorder;
/** The header border is drawn between the header rows and the table. */
protected JCDrawStyle       headerBorder;
/** The column border is the default border drawn between table columns. */
protected JCDrawStyle       columnBorder;
/** The row border is the default border drawn between table rows. */
protected JCDrawStyle       rowBorder;
/** The color used as a default background for table elements. */
protected Color             background;
/** Determines if the the top border of first row is shown. */
protected boolean           showFirstRowTopBorder;
/** First column special? Not currently implemented. */
protected boolean           firstColumnSpecial;
/** The text style. */
protected JCTextStyle       textStyle;
/** Alternating color object. */
protected JCAlternate       alternate;
/** Styles for the header. */
protected JCTableStyle      headerStyle;


// Properties for allowing enumeration of this class

private JCTableStyle prev;
private JCTableStyle next;
private static int upperBound = 0;
private static JCTableStyle first = null;
private static JCTableStyle last = null;

// Predefined JCPageTable Styles

// header styles. These are used internally and should not be used
// to specify a style on a JCPageTable.

/** Internal use only, Black Header. */
private final static JCTableStyle INTERNAL_USE_HEADER1 = new JCTableStyle(
    "BlackHeader",          // style name
    null,		    // topBorder
    null,		    // bottomBorder
    null,		    // leftBorder
    null,		    // rightBorder
    null,		    // headerBorder
    null,		    // rowBorder
    null,		    // columnBorder
    true,                   // show first row border
    false,                  // first column special
    null,                   // inherit header style
    JCTextStyle.WHITE_BOLD, // text
    Color.black,            // background color
    true		    // internal use
);

/** Internal use only, Gray Header. */
private final static JCTableStyle INTERNAL_USE_HEADER2 = new JCTableStyle(
    "GrayHeader",		// style name
    null,				// topBorder
    null,				// bottomBorder
    null,				// leftBorder
    null,				// rightBorder
    null,				// headerBorder
    null,				// rowBorder
    null,				// columnBorder
    true,               // show first row border
    false,              // first column special
    null,               // inherit header style
    JCTextStyle.WHITE_BOLD, // text
    Color.gray,			// background color
    true				// internal use
);

/** Internal use only, Null header -- inherit all. */
private final static JCTableStyle INTERNAL_USE_HEADER3 = new JCTableStyle(
    "Null Header",		// style name
    null,				// topBorder
    null,				// bottomBorder
    null,				// leftBorder
    null,				// rightBorder
    null,				// headerBorder
    null,				// rowBorder
    null,				// columnBorder
    true,               // show first row border
    false,              // first column special
    null,               // inherit header style
    JCTextStyle.DEFAULT_TEXT, // text
    null,			    // background color
    true				// internal use
);

// Body styles

/**
 * STYLE_DEFAULT -- the default table style; a simple grid with
 * borders around all cells.  The value of the <code>name</code> property for this
 * style is <code>default</code>.
 */
public final static JCTableStyle STYLE_DEFAULT = new JCTableStyle(
    "default",			// style name
    JCDrawStyle.LINE,	// topBorder
    JCDrawStyle.LINE,	// bottomBorder
    JCDrawStyle.LINE,	// leftBorder
    JCDrawStyle.LINE,	// rightBorder
    JCDrawStyle.LINE,	// headerBorder
    JCDrawStyle.LINE,	// rowBorder
    JCDrawStyle.LINE,	// columnBorder
    true,               // show first row border
    false,              // first column special
    null,               // inherit header style
    JCTextStyle.DEFAULT_TEXT, // text
    null				// background color
);



/** Style 0 -- tabular data only with no borders around any cells. The value of the
 * <code>name</code> property for this style is <code>Style 0</code>.
 */
public final static JCTableStyle STYLE_0 = new JCTableStyle(
    "Style 0",		// style name
    JCDrawStyle.BLANK,	// topBorder
    JCDrawStyle.BLANK,	// bottomBorder
    JCDrawStyle.BLANK,	// leftBorder
    JCDrawStyle.BLANK,	// rightBorder
    JCDrawStyle.BLANK,	// headerBorder
    JCDrawStyle.BLANK,	// rowBorder
    JCDrawStyle.BLANK,	// columnBorder
    true,               // show first row border
    false,              // first column special
    null,               // inherit header style
    JCTextStyle.DEFAULT_TEXT, // text
    null				// background color
);

/**
 * Style 1 -- tabular data with no borders except a thick line
 * header border.  The value of the <code>name</code> property for this
 * style is <code>Style 1</code>.
 */
public final static JCTableStyle STYLE_1 = new JCTableStyle(
    "Style 1",			// style name
    JCDrawStyle.BLANK,	        // topBorder
    JCDrawStyle.BLANK,	        // bottomBorder
    JCDrawStyle.BLANK,		// leftBorder
    JCDrawStyle.BLANK,		// rightBorder
    JCDrawStyle.LINE_2POINT,    // headerBorder
    JCDrawStyle.BLANK,          // rowBorder
    JCDrawStyle.BLANK,		// columnBorder
    false,                      // show first row border
    true,                       // first column special
    null,                       // inherit header style
    JCTextStyle.DEFAULT_TEXT,   // text
    null						// background color
);

/**
 * Style 2 -- tabular data with thick top and bottom borders
 * and a header border.  The value of the <code>name</code> property
 * for this style is <code>Style 2</code>.
 */
public final static JCTableStyle STYLE_2 = new JCTableStyle(
    "Style 2",			// style name
    JCDrawStyle.LINE_2POINT,	// topBorder
    JCDrawStyle.LINE_2POINT,	// bottomBorder
    JCDrawStyle.BLANK,		// leftBorder
    JCDrawStyle.BLANK,		// rightBorder
    JCDrawStyle.LINE,		// headerBorder
    JCDrawStyle.BLANK,		// rowBorder
    JCDrawStyle.BLANK,		// columnBorder
    true,                       // show first row border
    false,                      // first column special
    null,                       // inherit header style
    JCTextStyle.DEFAULT_TEXT,   // text
    null		        // background color
);

/**
 * Style 3 -- thick top, bottom and header borders with row
 * borders. The value of the <code>name</code> property for this
 * style is <code>Style 3</CODE>.
 */
public final static JCTableStyle STYLE_3 = new JCTableStyle(
    "Style 3",			// style name
    JCDrawStyle.LINE_2POINT,	// topBorder
    JCDrawStyle.LINE_2POINT,	// bottomBorder
    JCDrawStyle.BLANK,		// leftBorder
    JCDrawStyle.BLANK,		// rightBorder
    JCDrawStyle.LINE_2POINT,    // headerBorder
    JCDrawStyle.LINE,           // rowBorder
    JCDrawStyle.BLANK,		// columnBorder
    true,                      	// show first row border
    false,                      // first column special
    null,                       // inherit header style
    JCTextStyle.DEFAULT_TEXT,   // text
    null			// background color
);

/**
 * Style 4 -- tabular data only with white labels on a
 * black header. The value of the <code>name</code> property
 * for this style is <code>Style 4</code>.
 */
public final static JCTableStyle STYLE_4 = new JCTableStyle(
    "Style 4",			// style name
    JCDrawStyle.BLANK,	        // topBorder
    JCDrawStyle.BLANK,	        // bottomBorder
    JCDrawStyle.BLANK,		// leftBorder
    JCDrawStyle.BLANK,		// rightBorder
    JCDrawStyle.LINE_2POINT,    // headerBorder
    JCDrawStyle.BLANK,          // rowBorder
    JCDrawStyle.BLANK,		// columnBorder
    false,                      // show first row border
    false,                      // first column special
    JCTableStyle.INTERNAL_USE_HEADER1,  // header style
    JCTextStyle.WHITE_BOLD,     // text
    null			// background color
);

/**
 * Style 5 -- tabular data with thick bottom, left, right borders
 * and a black header. The value of the <code>name</code> property
 * for this style is <code>Style 5</code>.
 */
public final static JCTableStyle STYLE_5 = new JCTableStyle(
    "Style 5",		        // style name
    JCDrawStyle.BLANK,          // topBorder
    JCDrawStyle.LINE_2POINT,    // bottomBorder
    JCDrawStyle.LINE_2POINT,    // leftBorder
    JCDrawStyle.LINE_2POINT,    // rightBorder
    JCDrawStyle.LINE_2POINT,    // headerBorder
    JCDrawStyle.BLANK,          // rowBorder
    JCDrawStyle.BLANK,		// columnBorder
    false,                      // show first row border
    false,                      // first column special
    JCTableStyle.INTERNAL_USE_HEADER1,  // header style
    JCTextStyle.WHITE_BOLD,     // text
    null			// background color
);

/**
 * Style 6 -- thick bottom, left, and right borders, a black header and
 * thin row and column lines. The value of the <code>name</code> property
 * for this style is <code>Style 6</code>.
 */
public final static JCTableStyle STYLE_6 = new JCTableStyle(
    "Style 6",			// style name
    JCDrawStyle.BLANK,	        // topBorder
    JCDrawStyle.LINE_2POINT,    // bottomBorder
    JCDrawStyle.LINE_2POINT,    // leftBorder
    JCDrawStyle.LINE_2POINT,    // rightBorder
    JCDrawStyle.LINE_2POINT,    // headerBorder
    JCDrawStyle.LINE,           // rowBorder
    JCDrawStyle.LINE,		// columnBorder
    true,                      	// show first row border
    false,                      // first column special
    JCTableStyle.INTERNAL_USE_HEADER1,  // header style
    JCTextStyle.WHITE_BOLD,     // text
    null			// background color
);

/**
 * Style 7 -- tabular data with thick bottom, left, right borders
 * and a gray header. The value of the <code>name</code> property for
 * this style is <code>Style 7</code>.
 */
public final static JCTableStyle STYLE_7 = new JCTableStyle(
    "Style 7",			// style name
    JCDrawStyle.LINE_2POINT,    // topBorder
    JCDrawStyle.LINE_2POINT,    // bottomBorder
    JCDrawStyle.LINE_2POINT,    // leftBorder
    JCDrawStyle.LINE_2POINT,    // rightBorder
    JCDrawStyle.LINE_2POINT,    // headerBorder
    JCDrawStyle.BLANK,          // rowBorder
    JCDrawStyle.BLANK,		// columnBorder
    false,                      // show first row border
    false,                      // first column special
    JCTableStyle.INTERNAL_USE_HEADER2,   // header style
    JCTextStyle.WHITE_BOLD,     // text
    null			// background color
);

/**
 * Style 8 -- thick bottom, left, and right borders, a gray header and
 * thin row and column lines. The value of the <code>name</code> property
 * for this style is <code>Style 8</code>.
 */
public final static JCTableStyle STYLE_8 = new JCTableStyle(
    "Style 8",		        // style name
    JCDrawStyle.LINE_2POINT,    // topBorder
    JCDrawStyle.LINE_2POINT,    // bottomBorder
    JCDrawStyle.LINE_2POINT,    // leftBorder
    JCDrawStyle.LINE_2POINT,    // rightBorder
    JCDrawStyle.LINE_2POINT,    // headerBorder
    JCDrawStyle.LINE,           // rowBorder
    JCDrawStyle.LINE,		// columnBorder
    true,                      	// show first row border
    false,                      // first column special
    JCTableStyle.INTERNAL_USE_HEADER2,   // header style
    JCTextStyle.WHITE_BOLD,     // text
    null                        // background color
);

/**
 * Style 9 -- tabular data, gray header, thick top and bottom borders.
 *  The value of the <code>name</code> property for this style is <code>Style 9</code>.
 */
public final static JCTableStyle STYLE_9 = new JCTableStyle(
    "Style 9",			// style name
    JCDrawStyle.LINE_2POINT,	// topBorder
    JCDrawStyle.LINE_2POINT,	// bottomBorder
    JCDrawStyle.BLANK,		// leftBorder
    JCDrawStyle.BLANK,		// rightBorder
    JCDrawStyle.LINE,		// headerBorder
    JCDrawStyle.BLANK,		// rowBorder
    JCDrawStyle.BLANK,		// columnBorder
    true,                       // show first row border
    false,                      // first column special
    JCTableStyle.INTERNAL_USE_HEADER2,   // header style
    JCTextStyle.WHITE_BOLD,     // text
    null			// background color
);

/**
 * Style 10 -- a grid with thick top, bottom and side borders.
 *  The value of the <code>name</code> property for this style is <code>Style 10</code>.
 */
public final static JCTableStyle STYLE_10 = new JCTableStyle(
    "Style 10",                 // style name
    JCDrawStyle.LINE_DOUBLE,	// topBorder
    JCDrawStyle.LINE_DOUBLE,	// bottomBorder
    JCDrawStyle.LINE_DOUBLE,	// leftBorder
    JCDrawStyle.LINE_DOUBLE,	// rightBorder
    JCDrawStyle.LINE,	        // headerBorder
    JCDrawStyle.LINE,	        // rowBorder
    JCDrawStyle.LINE,	        // columnBorder
    true,                       // show first row border
    false,                      // first column special
    null,                       // inherit header style
    JCTextStyle.DEFAULT_TEXT,   // text
    null		        // background color
);

/**
 * Style 11 -- a grid with no top, bottom or side borders.
 *  The value of the <code>name</code> property for this style is <code>Style 11</code>.
 */
public final static JCTableStyle STYLE_11 = new JCTableStyle(
    "Style 11",                 // style name
    JCDrawStyle.BLANK,	        // topBorder
    JCDrawStyle.BLANK,	        // bottomBorder
    JCDrawStyle.BLANK,	        // leftBorder
    JCDrawStyle.BLANK,	        // rightBorder
    JCDrawStyle.LINE,	        // headerBorder
    JCDrawStyle.LINE,	        // rowBorder
    JCDrawStyle.LINE,	        // columnBorder
    true,                       // show first row border
    false,                      // first column special
    null,                       // inherit header style
    JCTextStyle.DEFAULT_TEXT,   // text
    null		        // background color
);

/**
 * Style 12 -- all gray with thick top, bottom, side and header borders.
 *  The value of the <code>name</code> property for this style is <code>Style 12</code>.
 */
public final static JCTableStyle STYLE_12 = new JCTableStyle(
    "Style 12", 		// style name
    JCDrawStyle.LINE_2POINT,    // topBorder
    JCDrawStyle.LINE_2POINT,    // bottomBorder
    JCDrawStyle.LINE_2POINT,    // leftBorder
    JCDrawStyle.LINE_2POINT,    // rightBorder
    JCDrawStyle.LINE_2POINT,    // headerBorder
    JCDrawStyle.BLANK,          // rowBorder
    JCDrawStyle.BLANK,		// columnBorder
    false,                      // show first row border
    false,                      // first column special
    JCTableStyle.INTERNAL_USE_HEADER2,   // header style
    JCTextStyle.WHITE_BOLD,     // text
    Color.lightGray            	// background color
);

/**
 * Style 13 -- thick bottom, left, and right borders, a black header and
 * thin column lines.
 *  The value of the <code>name</code> property for this style is <code>Style 13</code>
 */
public final static JCTableStyle STYLE_13 = new JCTableStyle(
    "Style 13",			// style name
    JCDrawStyle.BLANK,	        // topBorder
    JCDrawStyle.LINE_2POINT,    // bottomBorder
    JCDrawStyle.LINE_2POINT,    // leftBorder
    JCDrawStyle.LINE_2POINT,    // rightBorder
    JCDrawStyle.LINE_2POINT,    // headerBorder
    JCDrawStyle.LINE,          	// rowBorder
    JCDrawStyle.BLANK,		// columnBorder
    true,                      	// show first row border
    false,                      // first column special
    JCTableStyle.INTERNAL_USE_HEADER1,  // header style
    JCTextStyle.WHITE_BOLD,     // text
    null			// background color
);

/**
 * Style 14 -- all white with a thin top, bottom, and side borders with a
 * thick header border.
 *  The value of the <code>name</code> property for this style is <code>Style 14</code>.
 */
public final static JCTableStyle STYLE_14 = new JCTableStyle(
    "Style 14",			// style name
    JCDrawStyle.LINE,	        // topBorder
    JCDrawStyle.LINE,    	// bottomBorder
    JCDrawStyle.LINE,    	// leftBorder
    JCDrawStyle.LINE,    	// rightBorder
    JCDrawStyle.LINE_2POINT,    // headerBorder
    JCDrawStyle.LINE,          	// rowBorder
    JCDrawStyle.BLANK,		// columnBorder
    true,                      	// show first row border
    false,                      // first column special
    null,  			// header style
    JCTextStyle.DEFAULT_TEXT,	// text
    null			// background color
);

/**
 * Style 15 -- all white with thin top, bottom, column and side
 * borders.
 *  The value of the <code>name</code> property for this style is <code>Style 15</code>.
 */
public final static JCTableStyle STYLE_15 = new JCTableStyle(
    "Style 15",			// style name
    JCDrawStyle.LINE,	        // topBorder
    JCDrawStyle.LINE,    	// bottomBorder
    JCDrawStyle.LINE,    	// leftBorder
    JCDrawStyle.LINE,    	// rightBorder
    JCDrawStyle.LINE,    	// headerBorder
    JCDrawStyle.BLANK,         	// rowBorder
    JCDrawStyle.LINE,		// columnBorder
    true,                      	// show first row border
    false,                      // first column special
    null,  			// header style
    JCTextStyle.DEFAULT_TEXT,	// text
    null			// background color
);

/**
 * Style 16 -- list-style table with alternating light- and dark-gray
 * rows.
 *  The value of the <code>name</code> property for this style is <code>Style 16</code>.
 */
public final static JCTableStyle STYLE_16 = new JCTableStyle(
    "Style 16",			// style name
    JCDrawStyle.LINE,	        // topBorder
    JCDrawStyle.LINE,    	// bottomBorder
    JCDrawStyle.LINE,    	// leftBorder
    JCDrawStyle.LINE,    	// rightBorder
    JCDrawStyle.LINE_2POINT,    // headerBorder
    JCDrawStyle.LINE,          	// rowBorder
    JCDrawStyle.BLANK,		// columnBorder
    true,                      	// show first row border
    false,                      // first column special
    JCTableStyle.INTERNAL_USE_HEADER2, //header style
    JCTextStyle.WHITE_BOLD,     // text
    null,			// background color
    JCAlternate.ROW		// JCAlternate object
);

/**
 * Style 17 -- alternating light- and dark-gray columns in header
 * and body.
 *  The value of the <code>name</code> property for this style is <code>Style 17</code>.
 */
public final static JCTableStyle STYLE_17 = new JCTableStyle(
    "Style 17",			// style name
    JCDrawStyle.LINE,	        // topBorder
    JCDrawStyle.LINE,    	// bottomBorder
    JCDrawStyle.LINE,    	// leftBorder
    JCDrawStyle.LINE,    	// rightBorder
    JCDrawStyle.LINE_2POINT,    // headerBorder
    JCDrawStyle.LINE,          	// rowBorder
    JCDrawStyle.BLANK,		// columnBorder
    true,                      	// show first row border
    false,                      // first column special
    null,  			// header style
    JCTextStyle.DEFAULT_TEXT,	// text
    null,			// background color
    JCAlternate.COLUMN		// JCAlternate object
);


/**
 * Creates a table style using the given properties.
 * @param topBorder the <code>JCDrawStyle</code> for the top border of the table
 * @param bottomBorder the <code>JCDrawStyle</code> for the bottom border of the table
 * @param leftBorder the <code>JCDrawStyle</code> for the left border of the table
 * @param rightBorder the <code>JCDrawStyle</code> for the right border of the table
 * @param headerBorder the <code>JCDrawStyle</code> for the rows between the header and table
 * @param rowBorder the <code>JCDrawStyle</code> for row borders
 * @param columnBorder the <code>JCDrawStyle</code> for column borders
 * @param background the Color for the tables background
 * @param name
 * @param showFirstRowTopBorder
 * @param firstColumnSpecial
 * @param headerStyle
 * @param textStyle
 */
public JCTableStyle(String name, JCDrawStyle topBorder, JCDrawStyle bottomBorder,
            JCDrawStyle leftBorder, JCDrawStyle rightBorder,
            JCDrawStyle headerBorder, JCDrawStyle rowBorder,
            JCDrawStyle columnBorder, boolean showFirstRowTopBorder,
            boolean firstColumnSpecial, JCTableStyle headerStyle,
           	JCTextStyle textStyle, Color background) {

	// call this to add this style to the enumeration
    this(name);

    // set the properties
	this.topBorder 		        = topBorder;
    this.bottomBorder	        = bottomBorder;
    this.leftBorder		        = leftBorder;
    this.rightBorder	        = rightBorder;
    this.headerBorder	        = headerBorder;
    this.rowBorder		        = rowBorder;
    this.columnBorder	        = columnBorder;
    this.showFirstRowTopBorder  = showFirstRowTopBorder;
    this.firstColumnSpecial     = firstColumnSpecial;
    this.background 	        = background;
    this.textStyle              = textStyle;
    this.headerStyle			= headerStyle;
}

/**
 * Creates a table style using the given properties.
 * @param topBorder the <code>JCDrawStyle</code> for the top border of the table
 * @param bottomBorder the <code>JCDrawStyle</code> for the bottom border of the table
 * @param leftBorder the <code>JCDrawStyle</code> for the left border of the table
 * @param rightBorder the <code>JCDrawStyle</code> for the right border of the table
 * @param headerBorder the <code>JCDrawStyle</code> for the rows between the header and table
 * @param rowBorder the <code>JCDrawStyle</code> for row borders
 * @param columnBorder the <code>JCDrawStyle</code> for column borders
 * @param background the Color for the tables background
 * @param alternate	the alternating color object
 * @param textStyle
 * @param headerStyle
 * @param firstColumnSpecial
 * @param showFirstRowTopBorder
 * @param name
 */
public JCTableStyle(String name, JCDrawStyle topBorder, JCDrawStyle bottomBorder,
            JCDrawStyle leftBorder, JCDrawStyle rightBorder,
            JCDrawStyle headerBorder, JCDrawStyle rowBorder,
            JCDrawStyle columnBorder, boolean showFirstRowTopBorder,
            boolean firstColumnSpecial, JCTableStyle headerStyle,
           	JCTextStyle textStyle, Color background, JCAlternate alternate) {

	// call this to add this style to the enumeration
    this(name, topBorder, bottomBorder, leftBorder, rightBorder,
            headerBorder, rowBorder, columnBorder, showFirstRowTopBorder,
            firstColumnSpecial, headerStyle, textStyle, background);

    this.alternate = alternate;
}

/**
 * Creates a table style using the given properties.
 * @param topBorder the <code>JCDrawStyle</code> for the top border of the table
 * @param bottomBorder the <code>JCDrawStyle</code> for the bottom border of the table
 * @param leftBorder the <code>JCDrawStyle</code> for the left border of the table
 * @param rightBorder the <code>JCDrawStyle</code> for the right border of the table
 * @param headerBorder the <code>JCDrawStyle</code> for the rows between the header and table
 * @param rowBorder the <code>JCDrawStyle</code> for row borders
 * @param columnBorder the <code>JCDrawStyle</code> for column borders
 * @param background the Color for the tables background
 * @param internalUse the internal use boolean flag
 * @param name
 * @param showFirstRowTopBorder
 * @param firstColumnSpecial
 * @param headerStyle
 * @param textStyle
 */
protected JCTableStyle(String name, JCDrawStyle topBorder, JCDrawStyle bottomBorder,
            JCDrawStyle leftBorder, JCDrawStyle rightBorder,
            JCDrawStyle headerBorder, JCDrawStyle rowBorder,
            JCDrawStyle columnBorder, boolean showFirstRowTopBorder,
            boolean firstColumnSpecial, JCTableStyle headerStyle,
           	JCTextStyle textStyle, Color background, boolean internalUse) {

	// don't call this(name) so this style does not get added to the list
	this.name 			= name;

    // set the properties
	this.topBorder 		        = topBorder;
    this.bottomBorder	        = bottomBorder;
    this.leftBorder		        = leftBorder;
    this.rightBorder	        = rightBorder;
    this.headerBorder	        = headerBorder;
    this.rowBorder		        = rowBorder;
    this.columnBorder	        = columnBorder;
    this.showFirstRowTopBorder  = showFirstRowTopBorder;
    this.firstColumnSpecial     = firstColumnSpecial;
    this.background 	        = background;
    this.textStyle              = textStyle;
    this.headerStyle			= headerStyle;

}

/**
 * A do-it-yourself table style. The new style will be added to the
 * enumeration.
 * @param name the name of the style
 */
public JCTableStyle(String name) {
    this.name = name;

    // make it enumerable
    if (first == null) {
        first = this;
    }
    if (last != null) {
        this.prev = last;
        last.next = this;
    }
    last = this;
}


// Setters and getters

/**
 * Returns the color used as a default background for table elements.
 * @return the background color
 */
public Color getBackground() {
    return background;
}

/**
 * Sets the color used as a default background for table elements.
 * @param background the Color for the background of the table
 */
public void setBackground(Color background) {
    this.background = background;
}

/**
 * Returns the bottom border which is drawn at the bottom of a table, or parts of it.
 * @return the bottomBorder
 */
public JCDrawStyle getBottomBorder() {
    return bottomBorder;
}

/**
 * Sets the bottom border which is drawn at the bottom of a table, or parts of it.
 * @param bottomBorder the <code>JCDrawStyle</code> for the <code>bottomBorder</code>
 */
public void setBottomBorder(JCDrawStyle bottomBorder) {
    this.bottomBorder = bottomBorder;
}

/**
 * Returns the column border which is the default border drawn between
 * table columns.
 * @return the <code>columnBorder</code>
 */
public JCDrawStyle getColumnBorder() {
    return columnBorder;
}

/**
 * Sets the column border which is the default border drawn between table
 * columns.
 * @param columnBorder the <code>JCDrawStyle</code> for the <code>columnBorder</code>
 */
public void setColumnBorder(JCDrawStyle columnBorder) {
    this.columnBorder = columnBorder;
}

/**
 * Returns the header border which is drawn between the header rows
 * and the table.
 * @return the <code>headerBorder</code>
 */
public JCDrawStyle getHeaderBorder() {
    return headerBorder;
}

/**
 * Sets the header border which is drawn between the header rows and the table.
 * @param headerBorder the <code>JCDrawStyle</code> for the header border
 */
public void setHeaderBorder(JCDrawStyle headerBorder) {
    this.headerBorder = headerBorder;
}

/**
 * Returns the left border which is drawn at the left of a table,
 * or parts of it.
 * @return the leftBorder
 */
public JCDrawStyle getLeftBorder() {
    return leftBorder;
}

/**
 * Gets the left border which is drawn at the left of a table, or parts of it.
 * @param leftBorder the <code>JCDrawStyle</code> for the <code>leftBorder</code>
 */
public void setLeftBorder(JCDrawStyle leftBorder) {
    this.leftBorder = leftBorder;
}

/**
 * Returns the name of this style.
 * @return the name of this style
 */
public String getName() {
    return name;
}

/**
 * Sets the name of this style.
 * @param name the String name of this style
 */
public void setName(String name) {
    this.name = name;
}

/**
 * Returns the right border which is drawn at the right of a table, or
 * parts of it.
 * @return the <code>rightBorder</code>
 */
public JCDrawStyle getRightBorder() {
    return rightBorder;
}

/**
 * Sets the right border which is drawn at the right of a table,
 * or parts of it.
 * @param rightBorder the <code>JCDrawStyle</code> for the right border
 */
public void setRightBorder(JCDrawStyle rightBorder) {
    this.rightBorder = rightBorder;
}

/**
 * Returns the row border which is the default border drawn between table rows.
 * @return the <code>rowBorder</code>
 */
public JCDrawStyle getRowBorder() {
    return rowBorder;
}

/**
 * Sets the row border which is the default border drawn between table rows.
 * @param rowBorder the <code>JCDrawStyle</code> for the row border
 */
public void setRowBorder(JCDrawStyle rowBorder) {
    this.rowBorder = rowBorder;
}

/**
 * Returns the top border which is drawn at the top of a table, above
 * the header table.
 * @return the <code>topBorder</code>
 */
public JCDrawStyle getTopBorder() {
    return topBorder;
}

/**
 * Sets the top border which is drawn at the top of a table, above
 * the header table.
 * @param topBorder the <code>JCDrawStyle</code> for the top border
 */
public void setTopBorder(JCDrawStyle topBorder) {
    this.topBorder = topBorder;
}

/**
 * Returns the header style for this table.
 * @return the <code>JCTableStyle</code> for this table's header
 */
public JCTableStyle getHeaderStyle() {
    // if no header style exists, inherit it from this style
    if (headerStyle == null) {
		headerStyle	= (JCTableStyle) JCTableStyle.INTERNAL_USE_HEADER3.clone();
    }

    return headerStyle;
}

/**
 * Sets the header style for this table.
 * @param headerStyle the <code>JCTableStyle</code> which encapsulates style information
 * for the header of this table.
 */
public void setHeaderStyle(JCTableStyle headerStyle) {
	this.headerStyle = headerStyle;
}

/**
 * Returns a deep-copy of an instance of this class.
 * @return a copy of an instance of this class
 */
public Object clone() {

	JCTableStyle newStyle;

	// Clone the primitives of this style
	try {
		newStyle = (JCTableStyle) super.clone();
	}
	catch (CloneNotSupportedException e) {
		return(null);
	}

    // clone the sub-objects

    newStyle.bottomBorder   = bottomBorder  == null ? null : (JCDrawStyle) bottomBorder.clone();
    newStyle.topBorder      = topBorder     == null ? null : (JCDrawStyle) topBorder.clone();
    newStyle.leftBorder     = leftBorder    == null ? null : (JCDrawStyle) leftBorder.clone();
    newStyle.rightBorder    = rightBorder   == null ? null : (JCDrawStyle) rightBorder.clone();
    newStyle.headerBorder   = headerBorder  == null ? null : (JCDrawStyle) headerBorder.clone();
    newStyle.columnBorder   = columnBorder  == null ? null : (JCDrawStyle) columnBorder.clone();
    newStyle.rowBorder      = rowBorder     == null ? null : (JCDrawStyle) rowBorder.clone();
    newStyle.textStyle      = textStyle     == null ? null : (JCTextStyle) textStyle.clone();
    newStyle.headerStyle    = headerStyle   == null ? null : (JCTableStyle) headerStyle.clone();

    return newStyle;
}

/**
 * Returns the <code>JCAlternate</code> object which controls alternating colors for rows
 * or columns.
 * @return the <code>JCAlternate</code> object controlling alternating colors
 */
public JCAlternate getAlternate() {
	return alternate;
}

/**
 * Sets the <code>JCAlternate</code> object which controls alternating colors for rows
 * or columns.
 * @param alternate the <code>JCAlternate</code> object controlling alternating colors
 */
public void setAlternate(JCAlternate alternate) {
	this.alternate = alternate;
}

/**
 * Returns the boolean which controls whether the top border of the first
 * row is shown or not.
 * @return a boolean which controls whether the top border of the first
 * row is shown or not; if <code>true</code>, it is shown
 */
public boolean getShowFirstRowTopBorder() {
    return showFirstRowTopBorder;
}

/**
 * Sets the boolean which controls whether the top border of the first
 * row is shown or not.
 * @param b a boolean which controls whether the top border of the first
 * row is shown or not; if <code>true</code>, it is shown
 */
public void setShowFirstRowTopBorder(boolean b) {
    showFirstRowTopBorder = b;
}

/**
 * Not implemented
public boolean getFirstColumnSpecial() {
    return firstColumnSpecial;
}
 */

/**
 * Not implemented
public void setFirstColumnSpecial(boolean b) {
    firstColumnSpecial = b;
}
 */

/**
 * Returns the <code>JCTextStyle</code> for this table.
 * @return the <code>JCTextStyle</code> for this table
 */
public JCTextStyle getTextStyle() {
    return textStyle;
}

/**
 * Sets the <code>JCTextStyle</code> for this table.
 * @param textStyle the <code>JCTextStyle</code> for this table
 */
public void setTextStyle(JCTextStyle textStyle) {
    this.textStyle = textStyle;
}

/////////////////// Methods to make this class enumerable

//
// A template for creating enums which are ordered, enumerable,
// printable, usable as an index, and usable in a loop.
// See http://www.javaworld.com/javaworld/jw-07-1997/jw-07-enumerated_p.html
//
public static Enumeration elements() {
  return new Enumeration() {
    private JCTableStyle curr = first;
    public boolean hasMoreElements() {
      return curr != null;
    }
    public Object nextElement() {
      JCTableStyle c = curr;
      curr = curr.next();
      return c;
    }
  };
}
/** Returns the String representation of this class.
 * @return Sting representation of this class
 */
public String toString() {return name; }
/** Returns the number of enumerable styles.
 * @return number of enumerable styles
 */
public static int size() { return upperBound; }
/** Returns the first enumerable style.
 * @return first enumerable style
 */
public static JCTableStyle first() { return first; }
/** Returns the last enumerable style.
 * @return last enumerable style
 */
public static JCTableStyle last() { return last; }
/** Returns the style which precedes this style in the enumeration.
 * @return style which preceds this style in the enumeration
 */
public JCTableStyle prev() { return this.prev; }
/** Returns the style which suceeds this style in the enumeration.
 * @return style which suceeds this styls in the enumeration
 */
public JCTableStyle next() { return this.next; }

} // End JCTableStyle
