/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCTextStyle.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:23 $ $Locker:  $  Quest Software Inc.
package com.klg.jclass.page;

import java.awt.*;
import java.util.*;
import java.util.List;

/**
 * <code>JCTextStyle</code> specifies all attributes controlling
 * the appearance of text in JClass PageLayout.
 */
public class JCTextStyle implements Cloneable {

/*
 * Underlining mode enumeration (can extend for StrikeThrough, OverLine).
 */
/** No underlining is done. */
public static final int LINEMODE_NONE = 1;
/** A single underline is drawn. */
public static final int LINEMODE_UNDERLINE = 2;

/*
 * Baseline offset (vertical position) enumeration.
 */
/** The text is placed on the text baseline. */
public static final int OFFSET_NONE = 1;
/** The text is placed below the text baseline as a subscript. */
public static final int OFFSET_SUBSCRIPT = 2;
/** The text is placed above the text baseline as a superscript. */
public static final int OFFSET_SUPERSCRIPT = 3;

/*
 * Paragraph alignment enumeration.
 */
/** No horizontal text alignment is performed. */
public static final int ALIGNMENT_NONE = 1;
/** Text is aligned to the left margin of the current column. */
public static final int ALIGNMENT_LEFT = 2;
/** Text is aligned to the right margin of the current column. */
public static final int ALIGNMENT_RIGHT = 3;
/** Text is aligned to the center of the current column. */
public static final int ALIGNMENT_CENTER = 4;
/** Text is aligned to both the left and right margins of the current column. */
public static final int ALIGNMENT_JUSTIFY = 5;

/** The unique name of the style. */
protected String            name;

/** Color the text is to be drawn in. */
protected Color             color;
/** Underlining mode (enumerated type). */
protected int               underline;

/** The particular font to use to render the text. */
protected Font              font;
/** The font family to which this font belongs. */
protected String            fontFamily;
/** The style (Regular, Bold, Italic) to be used in drawing the text. */
protected int               fontStyle;

/** The point size of the standard text described by this style. */
protected double            pointSize;
/** The point size reduction factor to super-/subscript size. */
protected double            subscriptRatio;
/** Text position relative to the baseline (for super-/subscript). */
protected int               baselineOffset;

/** Horizontal alignment of text to the frame/column edge. */
protected int               alignment;
/** Amount of white space to leave as indentation from the left margin. */
protected JCUnit.Measure    indentLeft;
/** Amount of white space to leave as indentation from the right margin. */
protected JCUnit.Measure    indentRight;
/** The indentation from the left margin for the first line of a paragraph.*/
protected JCUnit.Measure    indentParagraph;

/** Spacing to leave between lines of text (multiple of line height). */
protected double            lineSpacing;
/** Spacing to leave between paragraphs (multiple of line height). */
protected double            paragraphSpacing;

/** All the tabs (in order) maintained by the style. */
protected List	           tabList;

/** Flag identifying if the style may be modified. */
protected boolean           immutable = false;

/** Count of the number of instances of the class. */
protected static int        styleCount = 0;
/** A list containing references to all named style instances. */



protected static TreeMap    instanceCollection = new TreeMap();


protected Map immutableTextStyles = new Hashtable();

protected static final String UNAMED_STYLE_PREFIX = "jcplstyle";

/** Normal (upright, medium weight) text style.
 *  The value of the <code>name</code> property for this style is <code>Normal</code>.
 */
public static final JCTextStyle NORMAL = new JCTextStyle("Normal",
    new JCUnit.Measure(JCUnit.INTERNAL), new JCUnit.Measure(JCUnit.INTERNAL),
	new JCUnit.Measure(JCUnit.INTERNAL), Color.black, null,
    "TimesRoman", Font.PLAIN, 10.0, LINEMODE_NONE, ALIGNMENT_LEFT,
    0.75, OFFSET_NONE, 1.0, 1.0, true);

/** Plain (== Normal) text style.
 *  The value of the <code>name</code> property for this style is <code>Plain</code>.
 */
public static final JCTextStyle PLAIN = new JCTextStyle("Plain",
    new JCUnit.Measure(JCUnit.INTERNAL), new JCUnit.Measure(JCUnit.INTERNAL),
	new JCUnit.Measure(JCUnit.INTERNAL), Color.black, null,
    "TimesRoman", Font.PLAIN, 10.0, LINEMODE_NONE, ALIGNMENT_LEFT,
    0.75, OFFSET_NONE, 1.0, 1.0, true);

/** Bold text style.  The value of the <code>name</code> property for this style is <code>Bold</code>.*/
public static final JCTextStyle BOLD = new JCTextStyle("Bold",
    new JCUnit.Measure(JCUnit.INTERNAL), new JCUnit.Measure(JCUnit.INTERNAL),
	new JCUnit.Measure(JCUnit.INTERNAL), Color.black, null,
    "TimesRoman", Font.BOLD, 10.0, LINEMODE_NONE, ALIGNMENT_LEFT,
    0.75, OFFSET_NONE, 1.0, 1.0, true);

/** Bold/italic text style.
 *  The value of the <code>name</code> property for this style is <code>Bold Italic</code>.
 */
public static final JCTextStyle BOLD_ITALIC = new JCTextStyle("Bold Italic",
    new JCUnit.Measure(JCUnit.INTERNAL), new JCUnit.Measure(JCUnit.INTERNAL),
	new JCUnit.Measure(JCUnit.INTERNAL), Color.black, null,
    "TimesRoman", Font.BOLD | Font.ITALIC, 10.0, LINEMODE_NONE, ALIGNMENT_LEFT,
    0.75, OFFSET_NONE, 1.0, 1.0, true);

/** Italic text style.  The value of the <code>name</code> property for this style is <code>Italic</code>.*/
public static final JCTextStyle ITALIC = new JCTextStyle("Italic",
    new JCUnit.Measure(JCUnit.INTERNAL), new JCUnit.Measure(JCUnit.INTERNAL),
	new JCUnit.Measure(JCUnit.INTERNAL), Color.black, null,
    "TimesRoman", Font.ITALIC, 10.0, LINEMODE_NONE, ALIGNMENT_LEFT,
    0.75, OFFSET_NONE, 1.0, 1.0, true);

/** A text style indented from the column margins.
 *  The value of the <code>name</code> property for this style is <code>Indented</code>.
 */
public static final JCTextStyle INDENTED = new JCTextStyle("Indented",
    new JCUnit.Measure(JCUnit.INCHES, 0.25),
    new JCUnit.Measure(JCUnit.INCHES, 0.25),
    new JCUnit.Measure(JCUnit.INCHES, 0.25), Color.black, null,
    "TimesRoman", Font.PLAIN, 10.0, LINEMODE_NONE, ALIGNMENT_LEFT,
    0.75, OFFSET_NONE, 1.0, 1.0, true);

/** A text style designed for printing code samples.
 *  The value of the <code>name</code> property for this style is <code>Code</code>.
 */
public static final JCTextStyle CODE = new JCTextStyle("Code",
    new JCUnit.Measure(JCUnit.INTERNAL), new JCUnit.Measure(JCUnit.INTERNAL),
	new JCUnit.Measure(JCUnit.INTERNAL), Color.black, null,
    "Courier", Font.PLAIN, 10.0, LINEMODE_NONE, ALIGNMENT_LEFT,
    0.75, OFFSET_NONE, 1.0, 1.0, true);

/** An indented text style designed for printing code samples.
 *  The value of the <code>name</code> property for this style is <code>CodeIndented</code>.
 */
public static final JCTextStyle CODE_INDENTED = new JCTextStyle("CodeIndented",
    new JCUnit.Measure(JCUnit.INCHES, 0.25),
    new JCUnit.Measure(JCUnit.INCHES, 0.25),
    new JCUnit.Measure(JCUnit.INCHES, 0.25), Color.black, null,
    "Courier", Font.PLAIN, 10.0, LINEMODE_NONE, ALIGNMENT_LEFT,
    0.75, OFFSET_NONE, 1.0, 1.0, true);

/** A heading text style.
 *  The value of the <code>name</code> property for this style is <code>Heading</code>.
 */
public static final JCTextStyle HEADING = new JCTextStyle("Heading",
    new JCUnit.Measure(JCUnit.INTERNAL), new JCUnit.Measure(JCUnit.INTERNAL),
	new JCUnit.Measure(JCUnit.INTERNAL), Color.black, null,
    "Helvetica", Font.PLAIN, 10.0, LINEMODE_NONE, ALIGNMENT_LEFT,
    0.75, OFFSET_NONE, 1.0, 1.0, true);

/** A bold heading text style.
 *  The value of the <code>name</code> property for this style is <code>HeadingBold</code>.
 */
public static final JCTextStyle HEADING_BOLD = new JCTextStyle("HeadingBold",
    new JCUnit.Measure(JCUnit.INTERNAL), new JCUnit.Measure(JCUnit.INTERNAL),
	new JCUnit.Measure(JCUnit.INTERNAL), Color.black, null,
    "Helvetica", Font.BOLD, 10.0, LINEMODE_NONE, ALIGNMENT_LEFT,
    0.75, OFFSET_NONE, 1.0, 1.0, true);

/** A major (top-level) heading text style.
 *  The value of the <code>name</code> property for this style is <code>H1</code>.
 */
public static final JCTextStyle HEADING1 = new JCTextStyle("H1",
    new JCUnit.Measure(JCUnit.INTERNAL), new JCUnit.Measure(JCUnit.INTERNAL),
	new JCUnit.Measure(JCUnit.INTERNAL), Color.black, null,
    "Helvetica", Font.BOLD, 18.0, LINEMODE_NONE, ALIGNMENT_LEFT,
    0.75, OFFSET_NONE, 1.0, 1.0, true);

/** A secondary (second-level) heading text style.
 *  The value of the <code>name</code> property for this style is <code>H2</code>.
 */
public static final JCTextStyle HEADING2 = new JCTextStyle("H2",
    new JCUnit.Measure(JCUnit.INTERNAL), new JCUnit.Measure(JCUnit.INTERNAL),
	new JCUnit.Measure(JCUnit.INTERNAL), Color.black, null,
    "Helvetica", Font.PLAIN, 18.0, LINEMODE_NONE, ALIGNMENT_LEFT,
    0.75, OFFSET_NONE, 1.0, 1.0, true);

/** Level three heading text style.
 *  The value of the <code>name</code> property for this style is <code>H3</code>.
 */
public static final JCTextStyle HEADING3 = new JCTextStyle("H3",
    new JCUnit.Measure(JCUnit.INTERNAL), new JCUnit.Measure(JCUnit.INTERNAL),
	new JCUnit.Measure(JCUnit.INTERNAL), Color.black, null,
    "Helvetica", Font.BOLD, 16.0, LINEMODE_NONE, ALIGNMENT_LEFT,
    0.75, OFFSET_NONE, 1.0, 1.0, true);

/** Level four heading text style.
 *  The value of the <code>name</code> property for this style is <code>H4</code>.
 */
public static final JCTextStyle HEADING4 = new JCTextStyle("H4",
    new JCUnit.Measure(JCUnit.INTERNAL), new JCUnit.Measure(JCUnit.INTERNAL),
	new JCUnit.Measure(JCUnit.INTERNAL), Color.black, null,
    "Helvetica", Font.PLAIN, 16.0, LINEMODE_NONE, ALIGNMENT_LEFT,
    0.75, OFFSET_NONE, 1.0, 1.0, true);

/** Level five heading text style.
 *  The value of the <code>name</code> property for this style is <code>H5</code>.
 */
public static final JCTextStyle HEADING5 = new JCTextStyle("H5",
    new JCUnit.Measure(JCUnit.INTERNAL), new JCUnit.Measure(JCUnit.INTERNAL),
	new JCUnit.Measure(JCUnit.INTERNAL), Color.black, null,
    "Helvetica", Font.BOLD, 14.0, LINEMODE_NONE, ALIGNMENT_LEFT,
    0.75, OFFSET_NONE, 1.0, 1.0, true);

/** Level six heading text style.
 *  The value of the <code>name</code> property for this style is <code>H6</code>.
 */
public static final JCTextStyle HEADING6 = new JCTextStyle("H6",
    new JCUnit.Measure(JCUnit.INTERNAL), new JCUnit.Measure(JCUnit.INTERNAL),
	new JCUnit.Measure(JCUnit.INTERNAL), Color.black, null,
    "Helvetica", Font.PLAIN, 14.0, LINEMODE_NONE, ALIGNMENT_LEFT,
    0.75, OFFSET_NONE, 1.0, 1.0, true);

/** Level seven heading text style.
 *  The value of the <code>name</code> property for this style is <code>H7</code>.
 */
public static final JCTextStyle HEADING7 = new JCTextStyle("H7",
    new JCUnit.Measure(JCUnit.INTERNAL), new JCUnit.Measure(JCUnit.INTERNAL),
	new JCUnit.Measure(JCUnit.INTERNAL), Color.black, null,
    "Helvetica", Font.BOLD, 12.0, LINEMODE_NONE, ALIGNMENT_LEFT,
    0.75, OFFSET_NONE, 1.0, 1.0, true);

/** White, Bold text style.
 *  The value of the <code>name</code> property for this style is <code>WhiteBold</code>.
 */
public static final JCTextStyle WHITE_BOLD = new JCTextStyle("WhiteBold",
    new JCUnit.Measure(JCUnit.INTERNAL), new JCUnit.Measure(JCUnit.INTERNAL),
	new JCUnit.Measure(JCUnit.INTERNAL), Color.white, null,
    "TimesRoman", Font.BOLD, 12.0, LINEMODE_NONE, ALIGNMENT_LEFT,
    0.75, OFFSET_NONE, 1.0, 1.0, true);

	// old style defaults
/** A default (standard, unornamented) text style.
 *  The value of the <code>name</code> property for this style is <code>default text</code>.
 */
public static final JCTextStyle DEFAULT_TEXT = new JCTextStyle("default text");

/** A default text style for headers.
 *  The value of the <code>name</code> property for this style is <code>default header</code>.
 */
public static final JCTextStyle DEFAULT_HEADER = new JCTextStyle("default header",
    new JCUnit.Measure(JCUnit.INTERNAL), new JCUnit.Measure(JCUnit.INTERNAL),
	new JCUnit.Measure(JCUnit.INTERNAL), Color.black, null,
    "TimesRoman", Font.BOLD, 14.0, LINEMODE_NONE, ALIGNMENT_CENTER,
    0.75, OFFSET_NONE, 1.0, 1.0, true);


/**
 * Creates a text style with the given name and values.
 * @param name the name by which the new style will be known
 * @param indent_left the distance from the left margin to indent text
 * @param indent_right the distance from the right margin to indent text
 * @param indent_paragraph the left indent for the first line of a paragraph
 * @param color the color to draw the text
 * @param tablist a list of tabs defined for the style
 * @param font_family the font family used to draw text
 * @param font_style the style (bold, italic) used to draw the text
 * @param point_size the size of text (in points)
 * @param underline the style of underlining to use
 * @param alignment the horizontal alignment of text in a column
 * @param subscript_ratio the ratio of the size of subscripts to normal text
 * @param baseline_offset the position of the text (super/subscript)
 * @param line_spacing the distance between lines of text
 * @param paragraph_spacing the distance between lines starting a paragraph
 * @param immutable indicates if the style may be changed or not
 */
public JCTextStyle(String name,
				   JCUnit.Measure indent_left, JCUnit.Measure indent_right,
				   JCUnit.Measure indent_paragraph, Color color, List tablist,
				   String font_family, int font_style, double point_size,
				   int underline, int alignment,
				   double subscript_ratio, int baseline_offset,
				   double line_spacing, double paragraph_spacing,
				   boolean immutable) {

	this.name = name;
	this.indentLeft = indent_left;
	this.indentRight = indent_right;
	this.indentParagraph = indent_paragraph;
	this.color = color;
	this.tabList = tablist;
	if (this.tabList == null) {
		this.tabList = new Vector();
	}
	this.fontFamily = font_family;
	this.fontStyle = font_style;
	this.pointSize = point_size;
	// Generate font from family, style & size
	// JDK11 compatibility method:
	// font = Font.decode(fontFamily).deriveFont(fontStyle, (float) pointSize);
	font = FontBase.deriveFont(Font.decode(fontFamily), fontStyle,
							   (float) pointSize);
	this.underline = underline;
	this.alignment = alignment;
	this.subscriptRatio = subscript_ratio;
	this.baselineOffset = baseline_offset;
	this.lineSpacing = line_spacing;
	this.paragraphSpacing = paragraph_spacing;

	this.immutable = immutable;

	if (!name.regionMatches(0, UNAMED_STYLE_PREFIX,
							0, UNAMED_STYLE_PREFIX.length())) {
		// store da ting
		// ??? Shouldn't we make sure the name is unique?
		instanceCollection.put(name, this);
	}
}

/**
 * Creates a text style with a default name and values.
 */
public JCTextStyle() {
	// Assign default values
	// default name
	this(UNAMED_STYLE_PREFIX + (++styleCount),
		 new JCUnit.Measure(JCUnit.INTERNAL),
		 new JCUnit.Measure(JCUnit.INTERNAL),
		 new JCUnit.Measure(JCUnit.INTERNAL),
		 Color.black, null, "TimesRoman", Font.PLAIN, 12.0,
		 LINEMODE_NONE, ALIGNMENT_LEFT, 0.75, OFFSET_NONE, 1.0, 1.0, false);
}

/**
 * Creates a text style with the given name and default values.
 * @param name the name to give the new text style
 */
public JCTextStyle(String name) {
	this();
	setName(name);
}

/**
 * Gets the name of the style.
 * @return the name used to refer to the style
 */
public String getName() {
	return name;
}

/**
 * Sets a name for the style.
 * @param name the new name for the style
 */
public void setName(String name) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable styles");
	}

	// Find a style (if any) having the same name
	if (instanceCollection.containsKey(this.name)) {
		// remove this mapping (discard previous version of this style)
		instanceCollection.remove(this.name);
	}

	this.name = name;

	if (!name.regionMatches(0, UNAMED_STYLE_PREFIX,
							0, UNAMED_STYLE_PREFIX.length())) {
		// store da ting
		instanceCollection.put(name, this);
	}
}

/**
 * Gets the color which is used to paint the text.
 * @return color assigned to the text
 */
public Color getColor() {
	return color;
}

/**
 * Sets the color to be used to paint the text.
 * @param color color object to assign to the text
 */
public void setColor(Color color) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable styles");
	}

	this.color = color;
}

/**
 * Gets the underlining mode this style is applying.
 * @return type of underlining that will be performed
 */
public int getUnderlining() {
	return underline;
}

/**
 * Sets the underlining mode this style is applying.
 * @param underline type of underlining to be performed
 */
public void setUnderlining(int underline) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable styles");
	}

	if (underline <= 0 || underline > LINEMODE_UNDERLINE) {
		throw new IllegalArgumentException("Unrecognised Line type");
	}

	this.underline = underline;
}

/**
 * Gets the style's defined point size.
 * @return the point size being applied to text using this style
 */
public double getPointSize() {
	return pointSize;
}

/**
 * Sets the point size of text using this style.
 * @param point_size the point size to be applied to text using this style
 */
public void setPointSize(double point_size) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable styles");
	}

	// Can't have 0 point text
	if (point_size < 1) {
		throw new IllegalArgumentException("Point size must be >= 1");
	}

	// Adjust the font for the new size
	this.pointSize = point_size;

	// JDK11 compatibility method:
	font = FontBase.deriveFont(font, fontStyle, (float) point_size);
}

/**
 * Gets the font which is used for rendering text.
 * @return the font the style will use to lay out text
 */
public Font getFont() {
	return font;
}

/**
 * Sets the font which is used for rendering text.
 * @param font the font to use for displaying text
 */
public void setFont(Font font) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable styles");
	}

	// user passed in font with size and style set
	this.font = font;

	// update the font family, style and point size for the new font
	this.fontFamily = font.getFamily();
	this.fontStyle = font.getStyle();
	this.pointSize = (double) FontBase.getSize2D(font);
}

/**
 * Gets the font family to which this the font belongs.
 * @return the font family the font is taken from
 */
public String getFontFamily() {
	return fontFamily;
}

/**
 * Sets the font family from which fonts and styles are selected.
 * @param font_family the font family that contains the desired font
 */
public void setFontFamily(String font_family) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable styles");
	}

	// Given the name of a font family, get the particular font
	this.fontFamily = font_family;

	// JDK11 compatibility method:
	this.font = FontBase.deriveFont(Font.decode(font_family), fontStyle,
									(float) pointSize);
}

/**
 * Gets the writing style (italic, bold) of the font.
 * @return style the font is drawn in
 */
public int getFontStyle() {
	return fontStyle;
}

/**
 * Sets the writing style (italic, bold) of the font.
 * @param font_style style (angle, weight) used to draw the font
 * Valid arguments are <i>java.awt.Font.PLAIN</i>, <i>java.awt.Font.BOLD</i>,
 * <i>java.awt.Font.ITALIC</i>, and <i>java.awt.Font.BOLD</i> | <i>java.awt.Font.ITALIC></i>
 */
public void setFontStyle(int font_style) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable styles");
	}

	// Adjust the font for the new style
	this.fontStyle = font_style;
	font = FontBase.deriveFont(font, font_style);
}

/**
 * Gets the offset from the normal baseline of the text.
 * @return position of the text relative to the normal text baseline
 */
public int getBaselineOffset() {
	return baselineOffset;
}

/**
 * Sets the position of the text relative to the normal baseline.
 * @param baseline_offset position of text (superscript/subscript) relative to
 * 						 the normal baseline of the text
 */
public void setBaselineOffset(int baseline_offset) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable styles");
	}

	if (baseline_offset <= 0 || baseline_offset > OFFSET_SUPERSCRIPT) {
		throw new IllegalArgumentException("Unrecognised baseline offset");
	}

	this.baselineOffset = baseline_offset;
}

/**
 * Gets the text alignment mode within a paragraph. Valid values are:
 * <code>JCTextStyle.ALIGNMENT_NONE</code>, <code>JCTextStyle.ALIGNMENT_LEFT</code>,
 * <code>JCTextStyle.ALIGNMENT_RIGHT</code>, <code>JCTextStyle.ALIGNMENT_CENTER</code>,
 * and <code>JCTextStyle.ALIGNMENT_JUSTIFY</code>.
 * @return alignment of lines of text in a paragraph
 */
public int getAlignment() {
	return alignment;
}

/**
 * Sets the text alignment mode within a paragraph. Valid values are:
 * <code>JCTextStyle.ALIGNMENT_NONE</code>, <code>JCTextStyle.ALIGNMENT_LEFT</code>,
 * <code>JCTextStyle.ALIGNMENT_RIGHT</code>, <code>JCTextStyle.ALIGNMENT_CENTER</code>,
 * and <code>JCTextStyle.ALIGNMENT_JUSTIFY</code>.
 * @param alignment alignment of lines of text in a paragraph
 */
public void setAlignment(int alignment) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable styles");
	}

	if (alignment <= 0 || alignment > ALIGNMENT_JUSTIFY) {
		throw new IllegalArgumentException("Unrecognised paragraph alignment");
	}

	this.alignment = alignment;
}

/**
 * Gets the amount of space to move text in from the left margin.
 * @return text indentation space on the left
 */
public JCUnit.Measure getLeftIndent() {
	return indentLeft;
}

/**
 * Specifies the amount of space between the left margin and the start of text.
 * @param indent_width space to indent text on the left side
 */
public void setLeftIndent(JCUnit.Measure indent_width) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable styles");
	}

	// Verify the indentation is >= 0
	if (indent_width.distance < 0.0) {
		throw new IllegalArgumentException("Indentation must be >= 0");
	}

	this.indentLeft = indent_width;
}

/**
 * Gets the amount of space to move text in from the right margin.
 * @return text indentation space on the right
 */
public JCUnit.Measure getRightIndent() {
	return indentRight;
}

/**
 * Specifies the amount of space between the right margin and the start of text.
 * @param indent_width space to indent text on the right side
 */
public void setRightIndent(JCUnit.Measure indent_width) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable styles");
	}

	// Verify the indentation is >= 0
	if (indent_width.distance < 0.0) {
		throw new IllegalArgumentException("Indentation must be >= 0");
	}

	this.indentRight = indent_width;
}

/**
 * Gets the amount of space to indent the first line of a paragraph.
 * @return text indentation space on the left of the first paragraph line
 */
public JCUnit.Measure getParagraphIndent() {
	return indentParagraph;
}

/**
 * Specifies the amount of space before the first line of text in a paragraph.
 * @param indent_width space to indent text for the beginning of a paragraph
 */
public void setParagraphIndent(JCUnit.Measure indent_width) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable styles");
	}

	// Verify the indentation is >= 0
	if (indent_width.distance < 0.0) {
		throw new IllegalArgumentException("Indentation must be >= 0");
	}

	this.indentParagraph = indent_width;
}

/**
 * Gets the line spacing as a factor of the text height.
 * @return number of times to advance the line based on the text height
 */
public double getLineSpacing() {
	return lineSpacing;
}

/**
 * Sets the line spacing (top of line to top) as a multiple of text height.
 * @param spacing the multiple of the text height to advance between lines
 */
public void setLineSpacing(double spacing) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable styles");
	}

	// Verify the spacing > 0
	if (spacing <= 0.0) {
		throw new IllegalArgumentException("Line spacing must be > 0");
	}

	this.lineSpacing = spacing;
}

/**
 * Gets the paragraph spacing as a factor of the text height.
 * @return factor to advance the between paragraphs based on the text height
 */
public double getParagraphSpacing() {
	return paragraphSpacing;
}

/**
 * Sets the paragraph spacing (line spacing betweeen paragraphs).
 * @param spacing text height multiplier for inter-paragraph line spacing
 */
public void setParagraphSpacing(double spacing) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable styles");
	}

	// Verify the spacing > 0
	if (spacing <= 0.0) {
		throw new IllegalArgumentException("Paragraph spacing must be > 0");
	}

	this.paragraphSpacing = spacing;
}

/**
 * Gets the list of tabs from the style.
 * @return the style's list of tabs
 */
public List getTabs() {
	return tabList;
}

/**
 * Sets the given tab as a tab stop in this style.
 * @param tab the tab object to add to this style's list of tabs
 */
public void addTab(JCTab tab) {

	int             index = 0;
	boolean         found = false;
	JCTab           old_tab;
	JCUnit.Measure  position;

	position = tab.getPosition();

	// Find the position for the tab in the list and insert it
	while (index < tabList.size() && !found) {
		old_tab = (JCTab) tabList.get(index);
		if (position.lessThan(old_tab.getPosition())) {
			tabList.add(index, tab);
			found = true;
		}
		index++;
	}

	// If the tab didn't fit before any existing tab, add it to the end
	if (!found) {
		tabList.add(tab);
	}
}

/**
 * Sets a tab stop in this style.
 * @param alignment the behavior of flowed objects aligning to the new tab
 * @param position the horizontal location of the tab across the line
 */
public void addTab(int alignment, JCUnit.Measure position) {

	int         index = 0;
	boolean     found = false;
	JCTab       tab, old_tab;

	// Create a tab at the given location
	tab = new JCTab(position, alignment);

	// Find the position for the tab in the list and insert it
	while (index < tabList.size() && !found) {
		old_tab = (JCTab) tabList.get(index);
		if (position.lessThan(old_tab.getPosition())) {
			tabList.add(index, tab);
			found = true;
		}
		index++;
	}

	// If the tab didn't fit before any existing tab, add it to the end
	if (!found) {
		tabList.add(tab);
	}
}

/**
 * Sets a list of tabs with a common alignment.
 * @param alignment the alignment of flowed objects to the tab locations
 * @param positions a List of positions for the tabs
 */
public void setTabs(int alignment, List positions) {

	int     i;

	// Clear any existing tabs
	tabList.clear();

	// Loop through the list of positions setting a tab at each
	for (i = 0; i < positions.size(); i++) {
		addTab(alignment, (JCUnit.Measure) positions.get(i));
	}
}

/**
 * Sets a list of regularly-spaced tabs with a common alignment.
 * @param alignment the alignment of flowed objects to the tab locations
 * @param spacing the spacing between tabs
 * @param count the number of tabs to be added
 */
public void setTabs(int alignment, JCUnit.Measure spacing, int count) {

	// Clear any existing tabs
	tabList.clear();

	// Set the tabs starting at "spacing" and separated by "spacing"
	setTabs(alignment, spacing, spacing, count);
}

/**
 * Sets a list of regularly-spaced tabs with a common alignment.
 * @param alignment the alignment of flowed objects to the tab locations
 * @param first the location of the first tab on the line
 * @param spacing the spacing between tabs
 * @param count the number of tabs to be added
 */
public void setTabs(int alignment, JCUnit.Measure first,
				    JCUnit.Measure spacing, int count) {

	int             i;
	JCUnit.Measure  position;

	// Clear any existing tabs
	tabList.clear();

	// set the position of the first tab
	position = (JCUnit.Measure) first.clone();

	// Loop through the list of positions setting a tab at each
	for (i = 0; i < count; i++) {
		addTab(alignment, position);
		position.add(spacing);
	}
}

/**
 * Removes all tabs from this style.
 */
public void deleteTabs() {
	tabList.clear();
}

/**
 * Removes the tab at a given position.
 * @param position the location of the tab which is to be cleared
 */
public void deleteTab(JCUnit.Measure position) {

	int         index = 0;
	boolean     found = false;
	JCTab       old_tab;

	// Find the position for the tab in the list and delete it
	while (index < tabList.size() && !found) {
		old_tab = (JCTab) tabList.get(index);
		if (position.equals(old_tab.getPosition())) {
			tabList.remove(index);
			found = true;
		}
		index++;
	}

}

/**
 * Gets the ratio of sub-/superscript text to normal text.
 * @return factor used to compute sub-/superscript text size from the font size
 */
public double getSubscriptRatio() {
	return subscriptRatio;
}

/**
 * Sets the ratio of sub-/superscript text to normal text.
 * @param subscript_ratio the scaling factor to derive subscript text size
 */
public void setSubscriptRatio(double subscript_ratio) {

	// Attributes can't be set on immutable objects
	if (immutable) {
		throw new IllegalArgumentException("Set only on mutable styles");
	}

	// The subscript ratio must be in reasonable bounds (0.01 < ratio < 10)
	if (subscript_ratio < 0.01 || subscript_ratio > 10.0) {
		throw new IllegalArgumentException("Subscript ratio must be between 0.01 and 10");
	}

	this.subscriptRatio = subscript_ratio;
}

/**
 * Determines if the style can be modified.
 * @return <code>true</code> or <code>false</code>, depending whether or not the style can be modified
 */
public boolean isMutable() {
	return(!immutable);
}

/**
 * Sets the style to immutable so it can no longer be modified.
 */
public void makeImmutable() {
	// Mark the style unchangeable
	immutable = true;
}

/**
 * Returns a style which is a copy of this one.
 * @return a new style with all significant fields copied
 */
public Object clone() {

	Iterator        tabs;
	JCTab           old_tab, new_tab;
	JCTextStyle     new_style;

	// Clone the style
	try {
		new_style = (JCTextStyle) super.clone();
	}
	catch (CloneNotSupportedException e) {
		return(null);
	}

	// Clone the indentations
	new_style.indentLeft = (JCUnit.Measure) indentLeft.clone();
	new_style.indentRight = (JCUnit.Measure) indentRight.clone();
	new_style.indentParagraph = (JCUnit.Measure) indentParagraph.clone();

	// Clone the list of tabs
	tabs = tabList.iterator();
	new_style.tabList = new Vector();
	while (tabs.hasNext()) {
		old_tab = (JCTab) tabs.next();
		new_tab = (JCTab) old_tab.clone();
		new_style.tabList.add(new_tab);
	}

	// copy the name of the style
	new_style.name = new String("Copy of " + name);
	new_style.immutable = false;

	// return the newly created style copy
	return(new_style);
}

/**
 * Provides an iterator over all immutable styles.
 * @return an iterator over the style class's underlying <code>SortedMap</code>
 */
public static Iterator getStyles() {
	return(instanceCollection.entrySet().iterator());
}

/**
 * Returns the entire style Map.
 * @return a Map interface to the styles
 */
public static Map getStyleList() {
	return(instanceCollection);
}

/**
 * Returns the style which matches the given name.
 * @param name the name of an existing style
 * @return the style object which has that name
 */
public static JCTextStyle stringToStyle(String name) {

	// Find the name in the Collection of instances
	if (instanceCollection.containsKey(name)) {
		return((JCTextStyle) instanceCollection.get(name));
	}

	// If no style was found return null
	return(null);
}

/**
 * Creates a hash that generates the same value depending on the
 * property values.
 * @return
 */
public int hashCode() {
	// add the hash codes of all the relavent properties together
	int hashcode = color.hashCode();
	hashcode += underline;
	hashcode += fontFamily.hashCode();
	hashcode += fontStyle;
	hashcode += pointSize;
	hashcode += subscriptRatio;
	hashcode += baselineOffset;
	hashcode += alignment;
	hashcode += indentLeft.hashCode();
	hashcode += indentRight.hashCode();
	hashcode += indentParagraph.hashCode();
	hashcode += lineSpacing;
	hashcode += paragraphSpacing;

	hashcode += tabList.size();
	for (int i = 0; i < this.tabList.size(); i++) {
		hashcode += this.tabList.get(i).hashCode();
	}

	/*
	if (name.equals("Normal")) {
		System.err.println("returning style = " + getFontStyle() + " hashcode = " + hashcode);
		System.err.println("\tcolor " + color.hashCode());
		System.err.println("\tunderline " + underline);
		System.err.println("\tfontfamily " + fontFamily.hashCode());
		System.err.println("\tfont style " + fontStyle);
		System.err.println("\tpoint size " + pointSize);
		System.err.println("\tsubscript ratio " + subscriptRatio);
		System.err.println("\tbaseline offset " + baselineOffset);
		System.err.println("\talignment " + alignment);
		System.err.println("\tindent left " + indentLeft.hashCode());
		System.err.println("\tindent right " + indentRight.hashCode());
		System.err.println("\tindent pargraph " + indentParagraph.hashCode());
		System.err.println("\t line spacing " + lineSpacing);
		System.err.println("\t para spacing " + paragraphSpacing);

	}
	*/
	return hashcode;
}

/**
 * Text styles with the same properties are considered equal regardless
 * of their names.
 * @return
 * @param o
 */
public boolean equals(Object o) {
	if (o == null) {
		return false;
	}
	if (o == this) {
		return true;
	}
	if (!(o instanceof JCTextStyle)) {
		return false;
	}

	JCTextStyle other = (JCTextStyle) o;
	if (this.color.equals(other.color) &&
		this.underline == other.underline &&
		this.fontFamily.equals(other.fontFamily) &&
		this.fontStyle == other.fontStyle &&
		this.pointSize == other.pointSize &&
		this.subscriptRatio == other.subscriptRatio &&
		this.baselineOffset == other.baselineOffset &&
		this.alignment == other.alignment &&
		this.indentLeft.equals(other.indentLeft) &&
		this.indentRight.equals(other.indentRight) &&
		this.indentParagraph.equals(other.indentParagraph) &&
		this.lineSpacing == other.lineSpacing &&
		this.paragraphSpacing == other.paragraphSpacing) {

		// easy comparing done. Now compare tabd stops
		if (this.tabList.size() != other.tabList.size()) {
			return false;
		}

		for (int i = 0; i < this.tabList.size(); i++) {
			JCTab this_tab = (JCTab) this.tabList.get(i);
			JCTab other_tab = (JCTab) other.tabList.get(i);
			if (!this_tab.equals(other_tab)) {
				// if any tab is different then we return false
				return false;
			}
		}

		return true;
	}

	return false;
}

/**
 * Returns this object if it is already immutable; otherwise, checks to see if a style
 * with the same properties already exists in the immutable cache and returns
 * it.  If there is no cached style, then this style is cloned, making it
 * immutable, stored in the cache, and returned.
 * @return
 */
public JCTextStyle getImmutable() {
	if (immutable == true) {
		// System.err.println("Style " + name + " is already Immutable");
		return this;
	}

	JCTextStyle style = (JCTextStyle) immutableTextStyles.get(this);
	if (style == null) {
		style = (JCTextStyle) this.clone();
		// System.err.println("Cloning new style from " + name);
		// only immutable styles get stuffed in the cache
		style.makeImmutable();
		// add style to map using itself as a key
		immutableTextStyles.put(style, style);
	}
	else {
		// System.err.println("returning style " + name + " from cache");
	}
	return style;
}

}
