/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: LineMetricsBase.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:23 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page;

import java.awt.FontMetrics;




import java.awt.font.LineMetrics;


/**
 *
 * Base implementation of LineMetrics to be extended.
 *
 */
public class LineMetricsBase extends LineMetrics {

protected FontMetrics fontMetrics;
protected String string;
	
/**
 *
 * Constructor
 *
 */
public LineMetricsBase(FontMetrics fontMetrics, String string) {
	this.fontMetrics 	= fontMetrics;
	this.string			= string;
}
	
/**
 * Returns the number of characters in the <code>Font</code>.
 * @return the number of characters in the <code>Font</code>.
 */
public  int getNumChars() {
	return string.length();
}
	
/**
 * Returns the ascent of the <code>Font</code>.  The ascent
 * of a <code>Font</code> is the distance from the baseline
 * to the ascender line.  The ascent usually represents the
 * the height of the capital letters of the <code>Font</code>.
 * @return the ascent of the <code>Font</code>.
 */
public  float getAscent() {
	return fontMetrics.getAscent();
}

/**
 * Returns the descent of the <code>Font</code>.  The descent
 * of a <code>Font</code> is the distance from the baseline 
 * to the descender line.  The characters of the 
 * <code>Font</code> usually do not extend below the 
 * descender line.
 * @return the descent of the <code>Font</code>.
 */
public  float getDescent() {
	return fontMetrics.getDescent();
}

/**
 * Returns the leading of the <code>Font</code>. The
 * leading of a <code>Font</code> is the recommended 
 * distance from the bottom of the descender line to the
 * top of next line.
 * @return the leading of the <code>Font</code>.
 */
public  float getLeading() {
	return fontMetrics.getLeading();
}

/**
 * Returns the height of the <code>Font</code>.  The
 * height is equal to the sum of the ascent, the
 * descent and the leading.
 * @return the height of the <code>Font</code>.
 */
public  float getHeight() {
	return fontMetrics.getHeight();
}

/**
 * Returns the current baseline index used for
 * laying out the text of the <code>Font</code>.
 * @return the baseline index of the <code>Font</code>.
 */
public  int getBaselineIndex() {
	return 0;
}
	
/**
 * Returns the baseline offsets as defined in the
 * <code>Font</code>, relative to y=0.
 * @return the baseline offsets of the <code>Font</code>.
 */
public  float[] getBaselineOffsets() {
	float array[] = {'0'};
	return array;
}

/**
 * Returns the position of the strike through line 
 * relative to the baseline.
 * @return the position of the strike through.
 */
public  float getStrikethroughOffset() {
	return -(getAscent() / 2);
}

/**
 * Returns the thickness of the strike through line.
 * @return the thickness of the strike through line.         
 */
public  float getStrikethroughThickness() {
	return fontMetrics.getFont().getSize() / 12.0f;
}
	
/**
 * Returns the position of the underline relative to
 * the descender.   
 * @return the position of the underline.
 */
public float getUnderlineOffset() {
	return 0;
}

/**
 * Returns the thickness of the underline.
 * @return the thickness of the underline.
 */
public float getUnderlineThickness() {
	return 0;
}

} // end class
