/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: Markup.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:23 $ $Locker:  $  Quest Software Inc. 

package com.klg.jclass.page;

import java.awt.Color;
import java.awt.Font;
import java.awt.Image;

import java.awt.image.ImageObserver;

/**
 * Denotes a Graphics object which permits the output of markup-style
 * operations.
 */
public interface Markup {

/**
 * Start a table in the output
 * @param columns Number of columns in the table
 */
public void startTable(int columns);

// NOTE:  Do we need a startTable() method which allows setting column widths?

/**
 * Start a new table row
 */
public void startRow();

/**
 * Start a table cell
 */
public void startCell();

/**
 * Start a table cell
 */
public void startCell(String hAlignment, boolean isHeaderCell);

/**
 * Start a spanned cell of the table
 * @param rows the number of rows the span overlays
 * @param columns the number of columns overlain by the spanned cell
 */
public void startCell(int rows, int columns);

/**
 * Span a cell of the table
 * @param rows the number of rows the span overlays
 * @param columns the number of columns overlain by the spanned cell
 */
public void startCell(String hAlignment, boolean isHeaderCell,
					  int rows, int columns);

/** 
 * End a cell
 */
// NOTE:  Should this be implicit in the next start{Row|Cell} action?
public void endCell();

/** 
 * End a cell
 */
// NOTE:  Should this be implicit in the next start{Row|Cell} action?
public void endCell(boolean isHeaderCell);

/**
 * End a row
 */
// NOTE:  Should this be implicit in the next startRow action?
public void endRow();

/**
 * End a table
 */
public void endTable();

/**
 * Mark an advance to the next line
 */
public void newLine();

/**
 * Mark an advance to a new paragraph
 */
public void newParagraph();

/**
 * Mark an advance to a new paragraph
 * @param hAlignment Horizontal alignment (left | center | right) to the margins
 */
public void newParagraph(String hAlignment);

/**
 * Begin a section (HTML "division") of the body
 * @param hAlignment Horizontal alignment of the content of this section
 */
public void newSection(String hAlignment);

/**
 * Draw a simple horizontal rule
 */
public void hRule();

/**
 * Draw a horizontal rule
 * @param percent The percentage width of the document to span with the rule
 */
public void hRule(double percent);

/**
 * Draw a horizontal rule
 * @param percent The percentage width of the document to span with the rule
 */
public void hRule(double percent, String hAlignment, int thickness);

/**
 * Draw a horizontal rule
 * @param pointsWidth The width of the rule in points
 */
public void hRule(int pointsWidth);

/**
 * Draw a horizontal rule
 * @param pointsWidth The width of the rule in points
 */
public void hRule(int pointsWidth, String hAlignment, int thickness);

/**
 * Add an image to the output
 * @param image The image to paint
 * @param width The width of the image in pixels
 * @param height The height of the image in pixels
 * @param bgcolor The background colour for the image
 * @param observer An ImageObserver to monitor loading of the image
 * @param horizontal The horizontal alignment of the image in the frame
 * @param vertical The vertical alignment of the image in the line
 */
public boolean outputImage(Image image, int width, int height, Color bgcolor,
						   ImageObserver observer, String horizontal,
						   String vertical);

}
