/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: PageArea.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:23 $ $Locker:  $  Quest Software Inc. 
package com.klg.jclass.page;

import java.awt.Color;

import java.util.NoSuchElementException;



import com.klg.jclass.page.render.Render;
import com.klg.jclass.page.render.BoxRender;
import com.klg.jclass.page.render.LineRender;




import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;

import java.util.Vector;
import java.util.List;
import java.util.Iterator;


/**
 * PageArea is a class containing appearance- and layout-related
 * methods common to Page and JCFrame.
 */
public class PageArea {

/** The name of this object */
protected String            name;
/** The document to which this object belongs */
protected JCDocument        document;
/** The location of this object relative to its parent object */
protected JCUnit.Point      location;
/** The width and height of this object */
protected JCUnit.Dimension  size;
/** The background colour of this object */
protected Color             color;
/** A render object representing the background paint of this object */
protected Render            background;
/** The list of frames included in this object */
protected List              frameList;
/** An indication of what types of macros remain unresolved in this object */
protected int               layoutStatus;

/**
 * Create an empty page area
 */
public PageArea() {

	layoutStatus = Render.LAYOUT_PASS_0;
	frameList = new Vector();

	location = new JCUnit.Point();
	size = new JCUnit.Dimension();

	background = null;
	color = null;
	name = null;

	document = null;
}

/**
 * Create a named page area
 * @param name The name to use for this page area
 */
public PageArea(String name) {
	this();
	this.name = name;
}

/**
 * Create an empty page area belonging to a particular document
 * @param doc The document to which this page area will belong
 */
public PageArea(JCDocument doc) {
	this();
	this.document = doc;
}

/**
 * Create a named page area belonging to a particular document
 * @param name The name to use for this page area
 * @param doc The document to which this page area will belong
 */
public PageArea(String name, JCDocument doc) {
	this();
	this.name = name;
	this.document = doc;
}

/**
 * Create a named page area with the given position and size
 * @param name The name to use for this page area
 * @param location The origin of the page area with respect to its parent
 * @param size The width and height of the page area
 */
public PageArea(String name, JCUnit.Point location, JCUnit.Dimension size) {
	this();
	this.name = name;
	this.location = location;
	this.size = size;
}

/**
 * Create a named page area with the given position and size in a document
 * @param name The name to use for this page area
 * @param doc The document to which this page area will belong
 * @param location The origin of the page area with respect to its parent
 * @param size The width and height of the page area
 */
public PageArea(String name, JCDocument doc, JCUnit.Point location,
				JCUnit.Dimension size) {
	this();

	this.name = name;
	this.document = doc;

	this.location = location;
	this.size = size;
}

/**
 * Get the name by which this object is known to other objects
 * @return Internal name of the object
 */
public String getName() {
	return name;
}

/**
 * Return the document to which this object belongs
 * @return this object's document
 */
public JCDocument getDocument() {
	return(document);
}

/**
 * Set the document to which this object belongs
 * @param doc The document to which this is being assigned
 */
public void setDocument(JCDocument doc) {

	int         i, count;
	JCFrame     frame;

	// Set the document for this object
	document = doc;

	// Set the document for all included frames
	count = frameList.size();
	for (i = 0; i < count; i++) {
		frame = (JCFrame) frameList.get(i);
		frame.setDocument(doc);
	}
}

/**
 * Get the origin (relative to the parent) of the object
 * @return Position of the object's origin
 */
public JCUnit.Point getLocation() {
	return location;
}

/**
 * Set the origin (relative to the parent) of the object
 */
public void setLocation(JCUnit.Point new_location) {
	location = new_location;
}

/**
 * Get the width and height of the object
 * @return Dimensions of the object
 */
public JCUnit.Dimension getSize() {
	return size;
}

/**
 * Set the width and height of the object
 * @param newSize Dimensions to resize the object to
 */
public void setSize(JCUnit.Dimension new_size) {
	size = new_size;
}

/**
 * Get the color which is used to paint the background of the object
 * @return Color assigned to the background
 */
public Color getColor() {
	return color;
}

/**
 * Set the color which is used to paint the background of the object
 * @param colour The new colour to use as the background
 */
public void setColor(Color color) {

	
	

	// The user can pass "null" to unset the background colour (transparent)
	if (color == null) {
		this.color = null;
		
		return;
	}

	if (!color.equals(this.color)) {
		
		this.color = new Color(color.getRGB());
	}
	
	

	
	
}

/**
 * Get the flag which indicates if more passes are necessary to resolve
 * any macros or print-time variables contained in the area
 * @return Status of the most incomplete object in the area
 */
public int getLayoutStatus() {
	return layoutStatus;
}

/**
 * Change the layout status flag to reflect manual changes to the
 * completeness of any macros contained in the area
 * @param new_status Status to reflect new level of page completeness
 */
public void setLayoutStatus(int new_status) {
	layoutStatus = new_status;
}

/**
 * Set the list of frame which are included in (overlaid on) this object
 */

/**
 * Performs a lookup of the given name in the frame list
 * @return Frame which matches the given name
 */
public JCFrame stringToFrame(String name) {

	Iterator frames;
	JCFrame frame;

	frames = frameList.iterator();
	while(frames.hasNext()) {
		frame = (JCFrame) frames.next();
		if (frame.getName().equalsIgnoreCase(name)) {
			return(frame);
		}
	}
	throw new NoSuchElementException("Frame not Found: " + name);
}

/**
 * Set the name by which this object is known to other objects
 */
protected void setName() {
}

/**
 * Get the background render object representing this object's background
 * @return The render object that fills in the background color
 */
protected Render getBackground() {
	return background;
}

/**
 * Set the list of frames which are included in (overlaid on) this object
 */
public void setFrameList(List frames) {
	this.frameList = frames;
}

/**
 * Get the list of frames which are included in (overlaid on) this object
 * @return Set of frames this object contains
 */
public List getFrameList() {
	return frameList;
}

/**
 * Process the list of frames on the area to determine the current status
 */
protected void calcLayoutStatus() {

	int			frame_status, worst_status = Render.LAYOUT_COMPLETE;
	Iterator	frames;
	JCFrame		frame;

	frames = frameList.iterator();
	while(frames.hasNext()) {
		frame = (JCFrame) frames.next();

		frame.calcLayoutStatus();
		frame_status = frame.getLayoutStatus();

		if (Render.isLayoutLower(worst_status, frame_status)) {
			worst_status = frame_status;
		}
	}

	layoutStatus = worst_status;
}

/**
 * Attempt to resolve macros in this area given the flow and the current page
 * @param flow the document's current flow
 * @param page the page on which the macros appear
 */
public void updateLayoutStatus(JCFlow flow, JCPage page) {

	// FUTURE:  Update and reposition existing macros
	int			frame_status, worst_status = Render.LAYOUT_COMPLETE;
	Iterator    frames;
	JCFrame     frame;

	// Don't update this object if it is already complete
	if (layoutStatus == Render.LAYOUT_COMPLETE) {
		return;
	}

	

	// Scan all frames included in this object and update them
	frames = frameList.iterator();
	while(frames.hasNext()) {
		frame = (JCFrame) frames.next();
		frame.updateLayoutStatus(flow, page);

		// Check the status of the updated frame and track the lowest level
		frame_status = frame.getLayoutStatus();
		if (Render.isLayoutLower(worst_status, frame_status)) {
			worst_status = frame_status;
		}
	}

	// The overall layout status is that of the worst-prepared frame
	layoutStatus = worst_status;
}

/**
 * Draw the area and everything contained in it
 * @param graphics The graphics object to use for drawing functions
 */
public void draw(Graphics2D gc) {
	// ??? This has become too complicated and needed to be split into
	//     JCPage.draw() and JCFrame.draw().
	return;
}

private static int NUM_LINES = 11;
private static double[][] WATERMARK_POINTS = {
	{ 0.23061515, 0.37929364, 0.41679985, 0.26812136 }, // (0, -2) -> (0, 2)
	{ 0.23061515, 0.37929364, 0.30255015, 0.45122864 }, // (0, -2) -> (2, -2)
	{ 0.41679985, 0.26812136, 0.48873485, 0.34005636 }, // (0, 2) -> (2, 2) 
	{ 0.3237075,  0.3237075,  0.37765875, 0.37765875 }, // (0, 0) -> (1.5, 0)

	{ 0.52470235, 0.37602386, 0.37448514, 0.52316364 }, // (3, 2) -> (4, -2)
	{ 0.37448514, 0.52316364, 0.59663735, 0.44795886 }, // (4, -2) -> (5, 2)

	{ 0.41045264, 0.55913113, 0.63260485, 0.48392636 }, // (5, -2) -> (6, 2)
	{ 0.63260485, 0.48392636, 0.48238765, 0.63106614 }, // (6, 2) -> (7, -2)
	{ 0.52152875, 0.52152875, 0.55749625, 0.55749625 }, // (5.5, 0) -> (6.5, 0)

	{ 0.70453985, 0.55586136, 0.51835515, 0.66703364 }, // (8, -2) -> (8, 2)
	{ 0.51835515, 0.66703364, 0.59029015, 0.73896864 }  // (8, -2) -> (10, -2)
	};

/**
 * Draw a watermark that reads "EVAL" on the given graphics
 */
protected void drawEvalWatermark(Graphics2D gc) {

	JCUnit.Point point1, point2;
	LineRender   watermark_line;

	// Create a GC to draw the background in
	Graphics2D gc1 = (Graphics2D) gc.create();

	if (gc1 instanceof Graphics2DBase) {
		((Graphics2DBase) gc1).setFrameRoot();
	}

	// Set the clip rect on the new graphics
	double w = size.getWidth().getAs(JCUnit.POINTS);
	double h = size.getHeight().getAs(JCUnit.POINTS);
	Rectangle2D.Double rect = new Rectangle2D.Double(0.0, 0.0, w, h);
	gc1.setClip(rect);

	// Create a style for drawing the watermark
	JCDrawStyle style = new JCDrawStyle();
	style.setForegroundColor(Color.lightGray);
	style.setLineWidth(new JCUnit.Measure(JCUnit.POINTS, 4.0));
	style.makeImmutable();

	// draw the watermark
	for (int i = 0; i < NUM_LINES; i++) {
		point1 = new JCUnit.Point(JCUnit.POINTS,
								  WATERMARK_POINTS[i][0] * w,
								  WATERMARK_POINTS[i][1] * h);
		point2 = new JCUnit.Point(JCUnit.POINTS,
								  WATERMARK_POINTS[i][2] * w,
								  WATERMARK_POINTS[i][3] * h);
		watermark_line = new LineRender(point1, point2, style);
		watermark_line.draw(gc1);
	}

	gc1.dispose();
}

}
