/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PageTableVectorDataSource.java,v $ $Revision: 1.2 $
//  		  $Date: 2006-03-17 21:13:23 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.page;

import com.klg.jclass.table.JCTableEnum;
import com.klg.jclass.table.JCTableDataEvent;
import com.klg.jclass.table.data.JCEditableVectorDataSource;




import java.util.Vector;


/**
 * PageTableVectorDataSource is a JClass PageLayout-specialised instance of
 * JCEditableVectorDataSource.  In addition to the inherited behavior
 * it stores the table so that table attributes can be retrieved and has
 * an implementation of getNumRows() that works better for JClass PageLayout.
 */
public class PageTableVectorDataSource extends JCEditableVectorDataSource {

/** The table to which this data source belongs */
protected JCPageTable	table;

/**
 * Creates a new PageTableVectorDataSource.
 * @param table The table the DataSource is associated with
 */
public PageTableVectorDataSource(JCPageTable table) {
	this.table = table;
}

/**
 * Creates a new PageTableVectorDataSource.
 * @param table The table the DataSource is associated with
 * @param cols The number of columns in the table
 */
public PageTableVectorDataSource(JCPageTable table, int cols) {

	this.table = table;
	columns = cols;
}

/**
 * Gets the table associated with this data source.
 * @return The table this data source is associated with
 */
public JCPageTable getTable() {
	return(table);
}

/**
 * Overrides getTableDataItem in order to provide inheritance
 * within a JClass PageLayout table.
 * @param row The row of the data to get an element from
 * @param column The column of the data to get an element from
 * @return The object stored at, or inherited by, the specified cell
 */
public Object getTableDataItem(int row, int column) {

	Object value = null;

	// Return null if the requested cell is out of range
	// (We don't throw an exception because other TableDataModels don't)
	if (row < 0 || row >= rows || column < 0 || column >= columns) {
		return(null);
	}

	// Otherwise the cell is in range but may not be defined

	// If the cell is defined, get its current value
	if (row < cells.size() && cells.elementAt(row) != null &&
		column < ((Vector) cells.elementAt(row)).size()) {

		value = ((Vector) cells.elementAt(row)).elementAt(column);
	}

	// If the value is null (it is either not defined or set to null)
	// find the correct value to inherit
	if (value == null) {

		// Determine whether row or column values are highest priority
		int dominance = table.getRowColumnDominance();

		// Depending on whether the table is row-dominant or column, get
		// the value that this cell inherits
		if (dominance == JCPageTable.ROW_DOMINANCE) {

			// get the row's default value; if unset, use the column's
			value = getTableRowLabel(row);
			if (value == null) {
				value = getTableColumnLabel(column);
			}
		}
		else { // (dominance == COLUMN_DOMINANCE)

			// get the column's default value; if unset, use the row's
			value = getTableColumnLabel(column);
			if (value == null) {
				value = getTableRowLabel(row);
			}
		}
	}

	// Return the (possibly still null) value
	return(value);
}

/**
 * Private method set cell values.
 */
private void setCellValue(int row, int column, Object value) {
	Object cell_value;

	int i, j, ii, jj, length,
		i1 = 0, i2 = rows - 1, j1 = 0, j2 = columns - 1;

	if(row >= 0 && column >= 0) {
		i1 = i2 = row;
		j1 = j2 = column;
	}
	else {
		if(row == JCTableEnum.ALLCELLS) {
			row = JCTableEnum.ALL;
		}

		if(column == JCTableEnum.ALLCELLS) {
			column = JCTableEnum.ALL;
		}

		if(row == JCTableEnum.ALL) {
			if(column != JCTableEnum.ALL) {
				j1 = j2 = column;
			}
			i1 = 0; i2 = rows - 1;
		}

		if(column == JCTableEnum.ALL) {
			if(row != JCTableEnum.ALL) {
				i1 = i2 = row;
			}
			j1 = 0; j2 = columns - 1;
		}

		if(row != JCTableEnum.ALL && column != JCTableEnum.ALL) {
			i1 = i2 = row;
			j1 = j2 = column;
		}

		if(i1 < 0 || j1 < 0) {
			return;
		}
	}

	for(i = i1; i <= i2; i++) {
		for(j = j1; j <= j2; j++) {
			cell_value = isValidCell(row, column) ?
					((Vector)cells.elementAt(row)).elementAt(column) : null;
			if(cell_value != null) {
				if(cell_value.equals(value)) {
					continue;
				}
				if(isValidCell(i,j)) {
					setElementAt(((Vector)cells.elementAt(i)), j, null);
				}
			}

			if(cells.size() < i+1 || cells.elementAt(i) == null) {
				setElementAt(cells, i, createVector(j+1));
			}
			setElementAt(((Vector)cells.elementAt(i)), j, value);

			setDataChanged(i, j, 1, 0, JCTableDataEvent.CHANGE_VALUE);
		}

		// if we are setting the last cell in row to null, discard excess values
		if (value == null && isValidCell(i, j) && j2 == cells.size() - 1) {
			strip(((Vector)cells.elementAt(i)));
			strip(cells);
		}

		// update rows to be the last row that now has data
		if (i2 >= rows) {
			rows = i2 + 1;
		}
	}
}

} // End PageTableVectorDataSource

