/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: TrueTypeFontProperties.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:23 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page;

public class TrueTypeFontProperties
{

/* Possible Embed Rules Values */
/** Do not embed the font in the generated document file. */
public static final int DO_NOT_EMBED = 0;
/** Embed only the necessary glyphs in the generated document file. */
public static final int EMBED_NEEDED = 1;
/** Embed the entire font in the generated document file. */
public static final int EMBED_ENTIRE_FONT = 2;

/** Rules for embedding the font in the generated document file.  One of
    DO_NOT_EMBED, EMBED_NEEDED or EMBED_ENTIRE_FONT.
 */
protected int embeddingRules = DO_NOT_EMBED;

/* Possible Character Range Values.  AUTO_DETECT is default and recommended
   value.  UNICODE encompasses the most characters, but will likely result
   in bigger PDF documents in the case where a the majority of the document's
   text lies in the 0-255 range.
 */
/** Automatically detects from document text when characters are in the
    ANSI and UNICODE ranges.
 */
public static final int AUTO_DETECT = 0;
/** All characters in document text lie in the ANSI range, that is character
    codes between 0 and 255.  Any characters references outside of the ANSI
    range will likely not draw correctly.
 */
public static final int ANSI = 1;
/** Characters in document text lie in the UNICODE range, that is
    character codes between 0 and 65535.  This setting encompasses the
    greatest number of characters.
 */
public static final int UNICODE = 2;

/** Rules for outputting document text int he resulding PDF file.  One of
    AUTO_DETECT, ANSI or UNICODE.
 */
protected int characterRange = AUTO_DETECT;

/** Determines whether output (e.g. a PDF File) should contain a
    glyph-to-unicode map.  The existence of this map allows cut-and-paste
    of non-Ansi text.
 */
protected boolean includeUnicodeMap = true;

/**
 * Default constructor.
 */
public TrueTypeFontProperties()
{
}

/**
 * Default constructor.
 */
public TrueTypeFontProperties(int characterRange, int embeddingRules,
									boolean includeMap)
{
	this.characterRange = characterRange;
	this.embeddingRules = embeddingRules;
	this.includeUnicodeMap = includeMap;
}

/**
 * Get the CharacterRange value, which specifies the range that the characters
 * that make up document text will occupy.  Possible values include
 * AUTO_DETECT (which leaves the detection up to ServerReport), ANSI
 * (for character codes in the range 0-255) and UNICODE (for character codes
 * in the range 0-65536).
 * @return the characterRange value, one of AUTO_DETECT, ANSI or UNICODE.
 */
public int getCharacterRange()
{
	return characterRange;
}

/**
 * Set the CharacterRange value, which specifies the range that the characters
 * that make up document text will occupy.  Possible values include
 * AUTO_DETECT (which leaves the detection up to ServerReport), ANSI
 * (for character codes in the range 0-255) and UNICODE (for character codes
 * in the range 0-65536).
 * @param characterRange the characterRange value, one of AUTO_DETECT, ANSI
 *                       or UNICODE.
 */
public void setCharacterRange(int characterRange)
{
	if (characterRange != AUTO_DETECT &&
			characterRange != ANSI &&
			characterRange != UNICODE) {
		throw new IllegalArgumentException("Character Range must be one of AUTO_DETECT, ANSI or UNICODE.");
	}
	this.characterRange = characterRange;
}

/**
 * Get the rules for embedding a related true type font within the
 * generated document (e.g. PDF) file.
 * @return the embedding rules value, one of DO_NOT_EMBED, EMBED_NEEDED
 * or EMBED_ENTIRE_FONT.
 */
public int getEmbeddingRules()
{
	return embeddingRules;
}

/**
 * Set the rules for embedding a related true type font within the
 * generated document (e.g. PDF) file.
 * @param embeddingRules the embedding rules value, one of
 * DO_NOT_EMBED, EMBED_NEEDED or EMBED_ENTIRE_FONT.
 */
public void setEmbeddingRules(int embeddingRules)
{
	if (embeddingRules != DO_NOT_EMBED &&
			embeddingRules != EMBED_NEEDED &&
			embeddingRules != EMBED_ENTIRE_FONT) {
		throw new IllegalArgumentException("Embedding Rules must be one of DO_NOT_EMBED, EMBED_NEEDED, or EMBED_ENTIRE_FONT.");
	}
	this.embeddingRules = embeddingRules;
}

/**
 * Get the value of the IncludeUnicodeMap property, which determines
 * whether output (e.g. a PDF File) should contain a glyph-to-unicode
 * map.  The existence of this map allows cut-and-paste of non-Ansi text.
 * @return true if output should contain a glyph-to-unicode map; false
 *         otherwise.
 */
public boolean getIncludeUnicodeMap()
{
	return includeUnicodeMap;
}

/**
 * Set the value of the IncludeUnicodeMap property, which determines
 * whether output (e.g. a PDF File) should contain a glyph-to-unicode
 * map.  The existence of this map allows cut-and-paste of non-Ansi text.
 * @param includeMap true if output should contain a glyph-to-unicode map;
 *                   false otherwise.
 */
public void setIncludeUnicodeMap(boolean includeMap)
{
	this.includeUnicodeMap = includeMap;
}


}
