/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCAdobeFontMap.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:23 $ $Locker:  $  Quest Software Inc.
package com.klg.jclass.page.adobe;

import java.util.ListResourceBundle;
import java.util.ResourceBundle;

/**
 * This resource bundle is used to map between a font name
 * and an alias (the actual font to use). It's required
 * because the the "Times" font, for example, can appear as,
 * <ul>
 * <li>Times</li>
 * <li>Times Roman</li>
 * <li>Times New Roman</li>
 * <li>CG Times</li>
 * </ul>
 * and so on. There is no standard.
 * <P>
 * The font names (resource keys) must not have spaces. This is
 * because <code>PropertyResourceBundle</code> cannot handle them. Replace
 * spaces with underscores in the keys. Use a dash between
 * the font name and its style. Leave the spaces in
 * the alias names. Here are the default mappings.
 * <pre>
 * 	{"Times",                       "Times-Roman"},
 *	{"Times-Bold",                  "Times-Bold"},
 *	{"Times-Italic",                "Times-Italic"},
 *	{"Times-BoldItalic",            "Times-BoldItalic"},
 *
 *	{"TimesRoman",                  "Times-Roman"},
 *	{"TimesRoman-Bold",             "Times-Bold"},
 *	{"TimesRoman-Italic",           "Times-Italic"},
 *	{"TimesRoman-BoldItalic",       "Times-BoldItalic"},
 *
 *	{"Times_New_Roman",             "Times-Roman"},
 *	{"Times_New_Roman-Bold",        "Times-Bold"},
 *	{"Times_New_Roman-Italic",      "Times-Italic"},
 *	{"Times_New_Roman-BoldItalic",  "Times-BoldItalic"},
 *
 *
 *	{"Helvetica",                   "Helvetica"},
 *	{"Helvetica-Bold",              "Helvetica-Bold"},
 *	{"Helvetica-Italic",            "Helvetica-Oblique"},
 *	{"Helvetica-BoldItalic",        "Helvetica-BoldOblique"},
 *
 *	{"Arial",                       "Helvetica"},
 *	{"Arial-Bold",                  "Helvetica-Bold"},
 *	{"Arial-Italic",                "Helvetica-Oblique"},
 *	{"Arial-BoldItalic",            "Helvetica-BoldOblique"},
 *
 *	{"Courier",                     "Courier"},
 *	{"Courier-Bold",                "Courier-Bold"},
 *	{"Courier-Italic",              "Courier-Oblique"},
 *	{"Courier-BoldItalic",          "Courier-BoldOblique"},
 *
 *	{"Courier_New",                 "Courier"},
 *	{"Courier_New-Bold",            "Courier-Bold"},
 *	{"Courier_New-Italic",          "Courier-Oblique"},
 *	{"Courier_New-BoldItalic",      "Courier-BoldOblique"},
 *
 *	{"Dialog",                      "Helvetica"},
 *	{"Dialog-Bold",                 "Helvetica-Bold"},
 *	{"Dialog-Italic",               "Helvetica-Oblique"},
 *	{"Dialog-BoldItalic",           "Helvetica-BoldOblique"},
 *
 *	{"DialogInput",                 "Courier"},
 *	{"DialogInput-Bold",            "Courier-Bold"},
 *	{"DialogInput-Italic",          "Courier-Oblique"},
 *	{"DialogInput-BoldItalic",      "Courier-BoldOblique"},
 *
 *	{"Serif",                       "Times-Roman"},
 *	{"Serif-Bold",                  "Times-Bold"},
 *	{"Serif-Italic",                "Times-Italic"},
 *	{"Serif-BoldItalic",            "Times-BoldItalic"},
 *
 *	{"SansSerif",                   "Helvetica"},
 *	{"SansSerif-Bold",              "Helvetica-Bold"},
 *	{"SansSerif-Italic",            "Helvetica-Oblique"},
 *	{"SansSerif-BoldItalic",        "Helvetica-BoldOblique"},
 *
 * </pre>
 * <P>
 * Keys are case-sensitive. Styles must appear with these
 * capitalizations: Bold, BoldItalic, or Italic. If the
 * font is plain, do not specify a style.
 * <P>
 * Here are additional resources on adobe fonts:<br>
 *
 *		ftp://ftp.adobe.com/pub/adobe/type/win/all/afmfiles/<br>
 *		http://www.adobe.com/supportservice/custsupport/SOLUTIONS/957a.htm<br>
 *		http://www.adobe.com/supportservice/custsupport/SOLUTIONS/407e.htm<br>
 *		http://www.adobe.com/supportservice/custsupport/SOLUTIONS/9762.htm<br>
 *
 */
public class JCAdobeFontMap extends ListResourceBundle {
public Object[][] getContents() {
	return contents;
}

// spaces in key font names must be replaced with underscores
static final Object[][] contents = {

	{"Times", 						"Times-Roman"},
	{"Times-Bold", 					"Times-Bold"},
	{"Times-Italic", 				"Times-Italic"},
	{"Times-BoldItalic", 			"Times-BoldItalic"},

	{"TimesRoman", 					"Times-Roman"},
	{"TimesRoman-Bold", 			"Times-Bold"},
	{"TimesRoman-Italic", 			"Times-Italic"},
	{"TimesRoman-BoldItalic", 		"Times-BoldItalic"},

	{"Times_New_Roman", 			"Times-Roman"},
	{"Times_New_Roman-Bold", 		"Times-Bold"},
	{"Times_New_Roman-Italic", 		"Times-Italic"},
	{"Times_New_Roman-BoldItalic", 	"Times-BoldItalic"},


	{"Helvetica",					"Helvetica"},
	{"Helvetica-Bold",				"Helvetica-Bold"},
	{"Helvetica-Italic",			"Helvetica-Oblique"},
	{"Helvetica-BoldItalic",		"Helvetica-BoldOblique"},

	{"Arial",						"Helvetica"},
	{"Arial-Bold",					"Helvetica-Bold"},
	{"Arial-Italic",				"Helvetica-Oblique"},
	{"Arial-BoldItalic",			"Helvetica-BoldOblique"},

	{"Courier",						"Courier"},
	{"Courier-Bold",				"Courier-Bold"},
	{"Courier-Italic",				"Courier-Oblique"},
	{"Courier-BoldItalic",			"Courier-BoldOblique"},

	{"Courier_New",					"Courier"},
	{"Courier_New-Bold",			"Courier-Bold"},
	{"Courier_New-Italic",			"Courier-Oblique"},
	{"Courier_New-BoldItalic",		"Courier-BoldOblique"},

	{"Dialog",						"Helvetica"},
	{"Dialog-Bold",					"Helvetica-Bold"},
	{"Dialog-Italic",				"Helvetica-Oblique"},
	{"Dialog-BoldItalic",			"Helvetica-BoldOblique"},

	{"DialogInput",					"Courier"},
	{"DialogInput-Bold",			"Courier-Bold"},
	{"DialogInput-Italic",			"Courier-Oblique"},
	{"DialogInput-BoldItalic",		"Courier-BoldOblique"},

	{"Serif", 						"Times-Roman"},
	{"Serif-Bold", 					"Times-Bold"},
	{"Serif-Italic", 				"Times-Italic"},
	{"Serif-BoldItalic", 			"Times-BoldItalic"},

	{"SansSerif",					"Helvetica"},
	{"SansSerif-Bold",				"Helvetica-Bold"},
	{"SansSerif-Italic",			"Helvetica-Oblique"},
	{"SansSerif-BoldItalic",		"Helvetica-BoldOblique"},


};

}
