/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: AcroFontEntry.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:23 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page.adobe.pdf;

import java.util.SortedSet;
import java.util.Hashtable;

import com.klg.jclass.page.adobe.postscript.FontPS;

/**
 * Class AcroFontEntry defines the attributes necessary to maintain a list
 * of font resources used on a page and in the pdf document.
 */
class AcroFontEntry{
	/** Font class that backs this AcroFontEntry. */
    protected FontPS font;

	/** Name of font. */
    protected String name;

	/** Abbreviated name as used within pdf file.  e.g. F1 */
    protected String abbrev;

	/** Number of pdf object that contains the descriptor for usage of the
		ansi portions of this font.
     */
    protected int number;

	/** Number of pdf object that contains the descriptor for usage of the
		unicode portions of this font.
     */
    protected int unicodeNumber;

	/** True if this font is backed by a TrueType font. */
    protected boolean usingTTF;

	/** Derived from the TrueTypeFontProperties class for this font.
        One of AUTO_DETECT, ANSI or UNICODE.
     */
    protected int expectedCharacterRange;

	/** True if this document will be using the ansi portions of the font.
  		If auto detect is on, this value will be set if ansi characters
		are encountered.  If auto detect is off, this value will be
		set according to the value of expectedCharacterRange.
     */
    protected boolean usingAnsi;

	/** True if this document will be using the unicode portions of the font.
  		If auto detect is on, this value will be set if above-ansi (character
		codes > 255) characters are encountered.  If auto detect is off, this
        value will be set according to the value of expectedCharacterRange.
        In this latter case, if the value of this property is true, it refers
		to the full range of character codes from 0-65535.
     */
    protected boolean usingUnicode;

	/** Ordered set of ansi characters used from this font in the document. */
    protected SortedSet ansiCharactersUsed;

	/** Ordered set of unicode characters used from this font in the document.
  		If auto detect is on, this property will contain only above-ansi
		(character codes > 255) characters that are encountered within
		document text.  If auto detect is off, this property will contain
        the full range of unicode characters (0-65535) that are encountered
		within the document.  Currently, the values stored in this
		variable are the glyph codes that correspond to unicode values,
        not the unicode values themselves.
	 */
    protected SortedSet unicodeCharactersUsed;

	/** Hashtable of widths for unicode characters used from this font.
        Key is width;  value if number of times width has been referenced.
        This is used to prevent large width tables that result from
        fonts where most characters have the same width.
     */
    protected Hashtable unicodeWidths;

	/** Derived from the TrueTypeFontProperties class for this font.
        One of DO_NOT_EMBED, EMBED_NEEDED or EMBED_ENTIRE_FONT.
     */
    protected int embeddingRules;

    /** True if a ToUnicode map should be included for a unicode CID font.
        (Such an entity allows cutting and pasting of unicode font info.)
        False otherwise.
     */
    protected boolean includeUnicodeMap;
}
