/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: Graphics2DPDF.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:23 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page.adobe.pdf;

import com.klg.jclass.page.Graphics2DBase;
import com.klg.jclass.page.TrueTypeFontProperties;
import com.klg.jclass.page.adobe.postscript.CharMetricPS;
import com.klg.jclass.page.adobe.postscript.FontPS;
import com.klg.jclass.page.adobe.postscript.PairKern;
import com.klg.jclass.page.adobe.postscript.FontMetricsPS;
import com.klg.jclass.page.FontLibrary;
import com.klg.jclass.page.EPSImage;

import java.awt.*;
import java.awt.geom.*;
import java.awt.image.ImageObserver;
import java.awt.image.PixelGrabber;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Hashtable;
import java.util.Set;
import java.util.zip.DeflaterOutputStream;

/**
 * Overrides Graphics2DBase to provide the java interface
 * to pdf drawing.
 */

public class Graphics2DPDF extends Graphics2DBase {

protected JCPDFPrinter printer;
/** Need to keep a separate transform just for pdf output because
 pdf angles are the opposite of java angles. */
protected AffineTransform pdfTransform = new AffineTransform();
protected final static char hd[] = {
    '0', '1', '2', '3', '4', '5', '6', '7',
    '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};

protected Color currentLineColor = null;
protected boolean lineColorChanged = true;
protected Color currentFillColor = null;
protected boolean fillColorChanged = true;
protected Paint currentPaint = null;
protected static Hashtable convertedFonts = new Hashtable();
protected int patternIndex = 0;

/**
 *
 * Constructor
 *
 */
public Graphics2DPDF(OutputStream os, JCPDFPrinter printer) {
    super(os);
    this.printer = printer;
}

/**
 * Overrides g2d method.
 */
public void drawString(String string, float x, float y) {

    // shouldn't attempt to draw null or empty strings
    if (string == null || string.equals("")) {
        return;
    }

    // Make sure we are in the correct state
    updateContext();

    // adjust to java origin
    y = -y;

    CharMetricPS currentChar, previousChar = null;

    outputCurrentPaint("rg");

    // initialize the buffer
    initializeOutString(x, y);

    // set the state to text
    printer.streamInfo.textPresent = true;

    // start the text segment
    output("BT");


    // set the text location
    StringBuffer location = new StringBuffer();
    location.append(x);
    location.append(" ");
    location.append(y);
    location.append(" Td");
    output(location.toString());

    // get the font
    FontPS font = (FontPS) getFont();

    // now add the string to the buffer
    String fontName = getFont().getName();

    // get the printer's font entry structure for this font -- we'll
    // be using some of its information and perhaps adding some of our own
    AcroFontEntry fontEntry = printer.getFontEntry(fontName);

    // determine if we're in autodetect mode for this font's character range
    boolean autoDetectCharacterRange = false;
    if (fontEntry.usingTTF) {
        autoDetectCharacterRange =
            (fontEntry.expectedCharacterRange ==
            TrueTypeFontProperties.AUTO_DETECT);
    }

    char stringItem;
    int charCode;
    Set characterList = null;
    boolean writingUnicodeAsHex = false;
    boolean above255 = false;
    boolean previousAbove255 = false;
    int glyphCode = -1;

    for (int i = 0; i < string.length(); i++) {
        stringItem = string.charAt(i);

        // special case -- intercept unicode for Euro character
        // when in ANSI or AUTO_DETECT output mode since the character
        // exists within PDF's WinAnsiEncoding encoding
        // (still need to use Unicode value to retrieve metrics and
        // the like however, since that's how it is stored in the TTF file)
        if (stringItem == (char) 0x20ac &&
            (fontEntry.expectedCharacterRange ==
            TrueTypeFontProperties.ANSI ||
            fontEntry.expectedCharacterRange ==
            TrueTypeFontProperties.AUTO_DETECT)) {
            charCode = 128;

            // special case -- if using ANSI mode but character is above
            // 255, substitute space character -- want to intercept it here
            // so substituted character gets added into character list instead
            // of the unsupported character
        }
        else if ((int) stringItem > 255 &&
            fontEntry.expectedCharacterRange ==
            TrueTypeFontProperties.ANSI) {
            System.err.println("Warning: expecting single-byte character, " +
                               "found multi-byte character 0x" +
                               Integer.toHexString((int) stringItem));
            stringItem = ' ';
            charCode = (int) stringItem;
        }
        else {
            charCode = (int) stringItem;
        }

        // true if the character has a value above decimal 255, in other
        // words, above the upper limit of the ansi character set
        above255 = charCode > 255;

        // special case -- ttf unicode require glyph code, not character code
        // make sure glyph exists in font and if not, use the backup character
        if (fontEntry.usingTTF &&
            ((autoDetectCharacterRange && above255) ||
            (!autoDetectCharacterRange && fontEntry.usingUnicode))) {
            glyphCode = font.ttfParser.unicodeToGlyphCode(
                (int) stringItem);
            // if character doesn't exist,
            // use glyph code zero (missing char in PDF)
            if (glyphCode < 0) {
                glyphCode = 0;
            }
        }

        currentChar = font.getCharMetric(stringItem);

        // if we're at least on the seond character
        if (previousChar != null) {
            // see if there is kern data for this combination of characters
            PairKern kern = null;
            if (fontEntry.usingTTF) {
                kern = previousChar.getKernByUnicode(currentChar.getCode());
            }
            else {
                kern = previousChar.getKernByName(currentChar.getName());
            }

            // set to true if kerning takes us outside of the string
            boolean outsideString = false;

            if (kern != null && kern.xAmount != 0) {
                // flush the existing string
                endTextSegment(writingUnicodeAsHex);

                // if we're in auto detect mode for the character range
                // and the string has changed from ansi to unicode, stay
                // outside of the string once the kerning has been written
                if (autoDetectCharacterRange) {
                    outsideString = (previousAbove255 != above255);
                }

                // adjust position by kerning distance
                addKerning(kern.xAmount, writingUnicodeAsHex, !outsideString);
            }

            // if we're in auto detect mode for the character range and
            // if string has changed from ansi to unicode, write out
            // existing string before new character is written
            if (autoDetectCharacterRange && previousAbove255 != above255) {
                printOutString(writingUnicodeAsHex, !outsideString);
                out.text.delete(0, out.text.length());
            }
        }

        // if this is the first character in string, initialize settings
        // and write out the font command before printing
        // if we're in auto detect mode for the character range and
        // string has changed from ansi to unicode or vice versa, change
        // appropriate settings and write out the font command for the
        // new ansi/unicode version of the font
        if (previousChar == null ||
            (autoDetectCharacterRange && previousAbove255 != above255)) {
            // if in auto detect mode, settings depend on whether
            // this new character is in ansi range or above it in unicode
            if (autoDetectCharacterRange) {
                writingUnicodeAsHex = above255;
                if (above255) {
                    fontEntry.usingUnicode = true;
                }
                else {
                    fontEntry.usingAnsi = true;
                }
                loadFont(fontName, above255);
                if (above255) {
                    characterList = fontEntry.unicodeCharactersUsed;
                }
                else {
                    characterList = fontEntry.ansiCharactersUsed;
                }
                // if not in autodetect mode, settings depend on
                // value of unicode/ansi booleans in the font entry, which
                // have been set by the printer class
            }
            else {
                writingUnicodeAsHex = fontEntry.usingUnicode;
                loadFont(fontName, fontEntry.usingUnicode);
                if (fontEntry.usingUnicode) {
                    characterList = fontEntry.unicodeCharactersUsed;
                }
                else {
                    characterList = fontEntry.ansiCharactersUsed;
                }
            }
        }

        // add the current char to the buffer and
        // to the list of characters referenced from a font
        // if using unicode and TrueType, it's the glyph code that
        // should be added, not the character code
        if (writingUnicodeAsHex && fontEntry.usingTTF) {

            // if glyph is not already in character list,
            // add it and add its width to the list of unicode widths
            Integer charToAdd = new Integer(glyphCode);
            if (!characterList.contains(charToAdd)) {
                characterList.add(charToAdd);

                Integer widthKey = new Integer(currentChar.getWx());
                Integer widthCount =
                    (Integer) fontEntry.unicodeWidths.get(widthKey);
                if (widthCount == null) {
                    fontEntry.unicodeWidths.put(widthKey, new Integer(1));
                }
                else {
                    fontEntry.unicodeWidths.put(widthKey,
                                                new Integer(widthCount.intValue() + 1));
                }
            }
            appendToOutString(glyphCode, currentChar.getWx(),
                              font, writingUnicodeAsHex);
        }
        else {
            characterList.add(new Integer(charCode));
            appendToOutString(charCode, currentChar.getWx(),
                              font, writingUnicodeAsHex);
        }
        previousChar = currentChar;
        previousAbove255 = above255;
    }

    // print out the existing string
    printOutString(writingUnicodeAsHex, true);

    // end the text segment
    output("ET");

    // Add an underline if required
    if (underline) {

        // correct the drawing line with if necessary
        double underlineWidth =
            FontMetricsPS.asPoints(
                font.getUnderlineThickness(), font.getSize());
        setStroke(new BasicStroke((float) underlineWidth));

        // adjust the baseline of the underlining and draw
        // and convert back to java origin since drawLine expects it
        y = (float) (-y - FontMetricsPS.asPoints(
            font.getUnderlinePosition(), font.getSize()));
        drawLinePW(x, y, out.x, y);
    }

}

/**
 * Override the Graphics2D translate method
 * @param x the distance to translate in x
 * @param y the distance to translate in y
 */
public void translate(int x, int y) {
    translate((double) x, (double) y);
}

/**
 * Override the Graphics2D translate method
 * @param x the distance to translate in x
 * @param y the distance to translate in y
 */
public void translate(double x, double y) {

    // update the current transformation matrix
    transform.translate(x, y);
    pdfTransform.translate(x, y);

    // Adjust the clip in the opposite direction so it doesn't move
    AffineTransform translateTransform =
        AffineTransform.getTranslateInstance(-x, -y);
    clip = translateTransform.createTransformedShape(clip);

    // if this is the current context, output the translation
    if (isContextCurrent()) {
        output("1 0 0 1 " + truncateDecimalDigits(x) + " " +
               truncateDecimalDigits(-y) + " cm");
    }
}

/**
 * Override the Graphics2D scale method
 * @param x the factor to scale in x
 * @param y the factor to scale in y
 */
public void scale(double x, double y) {

    // update the current transformation matrix
    transform.scale(x, y);
    pdfTransform.scale(x, y);

    // Adjust the clip in the opposite direction so it doesn't move
    AffineTransform scaleTransform =
        AffineTransform.getScaleInstance(1.0 / x, 1.0 / y);
    clip = scaleTransform.createTransformedShape(clip);

    // if this is the current context, output the translation
    if (isContextCurrent()) {
        output(truncateDecimalDigits(x) + " 0 0 " +
               truncateDecimalDigits(y) + " 0 0 cm");
    }
}

/**
 * Override the Graphics2D rotate method.
 * Only implement right angle rotation (90/180/270 degrees)
 * (that's all that chart uses) for now.
 * @param theta the angle to rotate, in radians.
 */
public void rotate(double theta) {


    // Adjust the clip in the opposite direction so it doesn't move
    AffineTransform rotateTransform = AffineTransform.getRotateInstance(-theta);
    clip = rotateTransform.createTransformedShape(clip);

    // update the current transformation matrix
    transform.rotate(theta);

    // For some reason, java has angles backwards.  Flip.
    theta = (2 * Math.PI) - theta;

    // update the pdf-specific transformation matrix
    pdfTransform.rotate(theta);

    // if this is the current context, output the translation
    if (isContextCurrent()) {
        output(truncateDecimalDigits(Math.cos(theta)) + " " +
               truncateDecimalDigits(Math.sin(theta)) + " " +
               truncateDecimalDigits(-1 * Math.sin(theta)) + " " +
               truncateDecimalDigits(Math.cos(theta)) + " " +
               " 0 0 cm");
    }
}

/**
 * Override the Graphics2D rotate method.
 * Implement in terms of existing translate and rotate methods.
 * @param theta the angle to rotate, in radians.
 * @param x the distance to translate in x
 * @param y the distance to translate in y
 */
public void rotate(double theta, double x, double y) {
    translate(x, y);
    rotate(theta);
    translate(-x, -y);
}

/**
 * Override the Graphics2D clip method
 * @param x the left side of the clipping rectangle
 * @param y the top of the clipping rectangle
 * @param width the width of the clipping rectangle
 * @param height the height of the clipping rectangle
 */
public void clipRect(int x, int y, int width, int height) {
    clipRect((double) x, (double) y, (double) width, (double) height);
}

/**
 * Override the Graphics2D clip method
 * @param x the left side of the clipping rectangle
 * @param y the top of the clipping rectangle
 * @param width the width of the clipping rectangle
 * @param height the height of the clipping rectangle
 */
public void clipRect(double x, double y, double width, double height) {

    Rectangle2D.Double desiredClip =
        new Rectangle2D.Double(x, y, width, height);

    // if there is already a clip rectangle set, intersect with the new clip
    if (clipSet) {
        Shape intersectionShape = null;
        // find the intersection of the desired clip and the current clip
        if (clip instanceof Rectangle2D) {
            intersectionShape = new Rectangle2D.Double();
            Rectangle2D.intersect((Rectangle2D) clip,
                                  desiredClip,
                                  (Rectangle2D) intersectionShape);
        }
        else {
            Area intersectionArea = new Area(clip);
            intersectionArea.intersect(new Area(desiredClip));
            intersectionShape = new GeneralPath();
            ((GeneralPath) intersectionShape).append(intersectionArea, false);
        }
        clip = intersectionShape;
    }

    // otherwise (no existing clip) use the new clip
    else {
        clip = desiredClip;
        clipSet = true;
    }


    // if this is the current context, output the new clip
    if (isContextCurrent()) {
        outputClip();
    }
}

//////////////////// Some Graphics (not 2D) methods /////////

/**
 * Overrides Graphics2DBase method.
 */
public void setPaint(Paint paint)
{
    currentPaint = paint;
    if (paint == null || paint instanceof Color) {
        setColor((Color)paint);
    }
    else if (paint instanceof GradientPaint) {
        // we don't support GradientPaint at the moment,
        // but we can at least behave nicely until we do
        setColor(((GradientPaint)paint).getColor1());
    }
}

public Paint getPaint() {
    return ((currentPaint != null) ? currentPaint : getColor());
}

public void setColor(Color clr) {
    super.setColor(clr);
    currentPaint = clr;
}

/**
 *
 * Overrides Graphics method
 *
 */
public void drawPolyline(int[] xPoints, int[] yPoints, int nPoints) {

    // Make sure we are in the correct state
    updateContext();

    // set the color
    outputCurrentPaint("RG");

    // set the line width
    setLineStroke();

    // first point
    output(xPoints[0] + " " + (-yPoints[0]) + " m");

    // the rest
    for (int i = 1; i < nPoints; i++) {
        output(xPoints[i] + " " + (-yPoints[i]) + " l");
    }
    output("S");
}


/**
 *
 * Overrides Graphics method
 *
 */
public void drawPolygon(int[] xPoints, int[] yPoints, int nPoints) {
    printPolygon(xPoints, yPoints, nPoints, "S");
}

/**
 *
 * Overrides Graphics method
 *
 */
public void fillPolygon(int[] xPoints, int[] yPoints, int nPoints) {
    printPolygon(xPoints, yPoints, nPoints, "f");
}

/**
 *
 * Overrides Graphics method
 *
 */
public void setFont(Font font) {

    // Make sure we are in the correct state
//DAM is this even needed?  don't think so
    updateContext();

    // if an awt font was passed in, get its ps equivalent
    if (!(font instanceof FontPS)) {
        if (convertedFonts.containsKey(font)) {
            font = (FontPS) convertedFonts.get(font);
        }
        else {


            Font font2 = font;

            try {
                font = FontPS.getNativeFont(font2);
            } catch (NullPointerException e) {
                addFont(font2);
                font = FontPS.getNativeFont(font2);
            }
            convertedFonts.put(font2, font);
        }
    }

    // only set it if we must
    if (currentFont != null &&
        (currentFont.equals(font) &&
        currentFont.getStyle() == font.getStyle() &&
        currentFont.getSize() == font.getSize())) {
        return;
    }

    this.currentFont = font;
    fontChanged = true;
    String name = font.getName();

    // already exists in document?
    Object o = printer.fontList.get(name);
    if (o == null) {
        // didn't exist so add it

        printer.addFontToList((FontPS) font);
    }
    printer.getFontAbbreviation(name);
}

/**
 *
 * Overrides Graphics method
 *
 */
public FontMetrics getFontMetrics(Font font) {

    if (!(font instanceof FontPS)) {
        // if an awt font was passed in, get its ps equavalent
        if (convertedFonts.containsKey(font)) {
            font = (FontPS) convertedFonts.get(font);
        }
        else {
            if (!printer.fontList.containsKey(font.getName())) {

//DAM why is set font called here?  This means that the font will
//    be set whenever this method is called, which is not a good idea.
                setFont(font);

            }
            Font font2 = font;

            font = FontPS.getNativeFont(font2);
            convertedFonts.put(font2, font);
        }

    }
    return new FontMetricsPS((FontPS) font);
}

public void addFont(Font font) {
    if (FontLibrary.availableFonts[FontLibrary.PDF].
        containsKey(font.getName())){
        FontPS.loadFont(font.getName());
    }
}

/**
 *
 * Overrides Graphics method
 *
 */
public boolean drawImage(Image img, int x, int y, int width, int height,
                         Color bgcolor, ImageObserver observer) {

    // don't draw null images
    if (img == null) {
        return false;
    }

    // Make sure we are in the correct state
    updateContext();

    if (img instanceof EPSImage) {
        // eps images are not supported in pdf, so draw a rectangle instead
        Color c = getColor();
        setColor(Color.lightGray);
        fillRect(x, y, width, height);
        setColor(c);
        return true;
    }
    else {
        return drawJavaImage(img, x, y, width, height,
                             bgcolor, observer, true, false, null);
    }
}

////////////////////////// Device-specific methods /////////////////////

/**
 * Override parent method to do some pdf-specific handling.
 */
public Graphics create() {
    Graphics newGC = super.create();

    if (newGC instanceof Graphics2DPDF) {
        Graphics2DPDF gcpdf = (Graphics2DPDF) newGC;
        // replace the transformation if this is a frame, copy it otherwise
        if (this.isFrameGraphics) {
            gcpdf.pdfTransform = new AffineTransform();
        }
        else {
            gcpdf.pdfTransform = new AffineTransform(this.pdfTransform);
        }
    }

    return newGC;
}

/**
 * Override parent method to do some pdf-specific handling.
 */
public void setTransform(AffineTransform transform) {
    super.setTransform(transform);
    this.pdfTransform = new AffineTransform(transform);
}

/**
 * Create a new Graphics class that is the same as the existing graphics
 * class.  This is different than create() since it creates an independent
 * graphics class that is unrelated to previous graphics classes yet
 * uses the same output stream.
 */
public Graphics newGraphics() {
    return new Graphics2DPDF(getOutputStream(), printer);
}

/**
 * Enter a graphics context
 */
public void startContext() {
    //check to make sure we have a document to print.
    if (printer.isDocOpen()) {

        // Create the new local context for printing
        output("q");

        // Set the transformation matrix
        double[] matrix = new double[6];
        pdfTransform.getMatrix(matrix);
        output("" + truncateDecimalDigits(matrix[0]) + " " +
               truncateDecimalDigits(matrix[1]) + " " +
               truncateDecimalDigits(matrix[2]) + " " +
               truncateDecimalDigits(matrix[3]) + " " +
               truncateDecimalDigits(matrix[4]) + " " +
               truncateDecimalDigits(-1 * matrix[5]) + " cm");

        outputClip();

        // Colours must be respecified
        lineColorChanged = true;
        fillColorChanged = true;

        // line stroke must be respecified, otherwise the one from the
        // parent will be used and it could be different
        strokeChanged = true;
    }
}

/**
 * Output the clip -- pdf commands depend on shape of clip.
 */
public void outputClip() {
    if (clip == null)
        return;

    // output rectangular clip
    if (clip instanceof Rectangle2D) {
        Rectangle2D bounds = (Rectangle2D) clip;
        output(truncateDecimalDigits(bounds.getX()) + " " +
               truncateDecimalDigits(-bounds.getY()) + " " +
               truncateDecimalDigits(bounds.getWidth()) + " " +
               truncateDecimalDigits(-bounds.getHeight()) + " re W n");

        // output polygon clip
    }
    else if (clip instanceof GeneralPath) {
        PathIterator path = clip.getPathIterator(null);
        double[] segment = new double[6];
        int segtype = 0;
        while (!path.isDone()) {
            segtype = path.currentSegment(segment);
            switch (segtype) {
                case PathIterator.SEG_MOVETO:
                    output(truncateDecimalDigits(segment[0]) + " " +
                           truncateDecimalDigits(-segment[1]) + " m");
                    break;

                case PathIterator.SEG_LINETO:
                    output(truncateDecimalDigits(segment[0]) + " " +
                           truncateDecimalDigits(-segment[1]) + " l");
                    break;

                case PathIterator.SEG_CLOSE:
                    output("h");
                    break;

                case PathIterator.SEG_QUADTO:
                case PathIterator.SEG_CUBICTO:
                    break;
            }
            path.next();
        }
        output("W n");

        // otherwise, output rectangular bounds of current clip
    }
    else {
        Rectangle2D bounds = clip.getBounds2D();
        output(truncateDecimalDigits(bounds.getX()) + " " +
               truncateDecimalDigits(-bounds.getY()) + " " +
               truncateDecimalDigits(bounds.getWidth()) + " " +
               truncateDecimalDigits(-bounds.getHeight()) + " re W n");
    }
}

/**
 * Exit a graphics context
 */
public void endContext() {
    output("Q");

    // Colours must be respecified
    lineColorChanged = true;
    fillColorChanged = true;
}

/**
 * Default implementation of a fillShape method.  All curves are
 * turned into lines and the provided shape is reduced to a polygon.
 */
public void fillShape(Shape s) {
    // NOTE:  We use an identity transform since we've already
    //        transformed our context.
    PathIterator path;


    path = s.getPathIterator(new AffineTransform());

    String strokeOrFill = "f";

    // Make sure we are in the correct state
    updateContext();

    // set the color
    //String s = strokeOrFill.equals("S") ? "RG" : "rg";
    outputCurrentPaint("rg");

    // set the filling rule
    windingRule = ((GeneralPath) s).getWindingRule();

    // handle winding rule
    // fill, set winding rule
    if (windingRule == GeneralPath.WIND_NON_ZERO) {
        strokeOrFill = "f*";
    }
    else {
        strokeOrFill = "f";
    }

    // first point of shape section, defined by move
    float firstX = 0.0f, firstY = 0.0f;

    // current point
    float currentX = 0.0f, currentY = 0.0f;

    // true if the shape has been closed off, false otherwise
    boolean closed = true;

    // draw the path
    while (!path.isDone()) {
        float pts[] = new float[6];
        switch (path.currentSegment(pts)) {
            // move -- output PDF move command, save new current point, save
            //      start point for shape section, set closed boolean to false
            case PathIterator.SEG_MOVETO:
                output(truncateDecimalDigits(pts[0]) + " " +
                       truncateDecimalDigits(-pts[1]) + " m");
                currentX = pts[0];
                currentY = pts[1];
                firstX = pts[0];
                firstY = pts[1];
                closed = false;
                break;
                // straight line -- output PDF line command, save new current point
            case PathIterator.SEG_LINETO:
                output(truncateDecimalDigits(pts[0]) + " " +
                       truncateDecimalDigits(-pts[1]) + " l");
                currentX = pts[0];
                currentY = pts[1];
                break;
                // cubic bezier curve -- output PDF curve command, save new
                //      current point
            case PathIterator.SEG_CUBICTO:
                output(truncateDecimalDigits(pts[0]) + " " +
                       truncateDecimalDigits(-pts[1]) + " " +
                       truncateDecimalDigits(pts[2]) + " " +
                       truncateDecimalDigits(-pts[3]) + " " +
                       truncateDecimalDigits(pts[4]) + " " +
                       truncateDecimalDigits(-pts[5]) + " c");
                currentX = pts[4];
                currentY = pts[5];
                break;
                // quadratic bezier curve -- convert to cubic bezier, output
                //      PDF curve command, save new current point
            case PathIterator.SEG_QUADTO:
                output(
                    truncateDecimalDigits(
                        cvtQuadraticCPToFirstCubicCP(pts[0], currentX)) + " " +
                    truncateDecimalDigits(
                        -cvtQuadraticCPToFirstCubicCP(pts[1], currentY)) + " " +
                    truncateDecimalDigits(
                        cvtQuadraticCPToSecondCubicCP(pts[0], pts[2])) + " " +
                    truncateDecimalDigits(
                        -cvtQuadraticCPToSecondCubicCP(pts[1], pts[3])) + " " +
                    truncateDecimalDigits(pts[2]) + " " +
                    truncateDecimalDigits(-pts[3]) + " c");
                currentX = pts[2];
                currentY = pts[3];
                break;
                // close shape section -- output PDF close shape command,
                //      save new current point (which is set to be the
                //      first point of the section), set closed boolean to true
            case PathIterator.SEG_CLOSE:
                currentX = firstX;
                currentY = firstY;
                output("h");
                closed = true;
                break;
            default:
                break;
        }
        // next command in path
        path.next();
    }

    // if shape was not explicitly closed, close it
    // in both cases, send the appropriate fill command
    if (!closed) {
        output("h " + strokeOrFill);
    }
    else {
        output(strokeOrFill);
    }
}

/**
 * Convert the control point of a quadratic Bezier curve to the first
 * control point of a cubic bezier curve.
 * @param quadCP the control point of the quadratic Bezier curve
 * @param quadInit the initial point of the quadratic Bezier curve
 */
private float cvtQuadraticCPToFirstCubicCP(float quadCP, float quadInit) {
    return ((2.0f * (quadCP - quadInit)) / 3.0f) + quadInit;
}

/**
 * Convert the control point of a quadratic Bezier curve to the second
 * control point of a cubic bezier curve.
 * @param quadCP the control point of the quadratic Bezier curve
 * @param quadFinal the final point of the quadratic Bezier curve
 */
private float cvtQuadraticCPToSecondCubicCP(float quadCP, float quadFinal) {
    return ((quadFinal - quadCP) / 3.0f) + quadCP;
}

/**
 *
 * Device-specific methods
 *
 */
public void drawArcPW(double x, double y, double width, double height,
                      double startAngle, double arcAngle, int type) {

    double rad, x0, y0;

    // Make sure we are in the correct state
    updateContext();

    // set the line width
    setLineStroke();

    // set the color
    outputCurrentPaint("RG");

    // Compute the semi-major axes
    double semiMajorX = width / 2;
    double semiMajorY = height / 2;

    // Compute the centre of the arc
    x = x + semiMajorX;
    y = -(y + semiMajorY);

    // Find the start point of the arc and move there
    // NOTE: we compute radians because 1.1 doesn't provide a conversion
    if (arcAngle < 0) {
        rad = (startAngle + arcAngle) / 180.0 * Math.PI;
    }
    else {
        rad = startAngle / 180.0 * Math.PI;
    }
    x0 = x + semiMajorX * Math.cos(rad);
    y0 = y + semiMajorY * Math.sin(rad);
    output(truncateDecimalDigits(x0) + " " + truncateDecimalDigits(y0) + " m");

    // Construct the arc's path
    traceArcPath(x, y, semiMajorX, semiMajorY, startAngle, arcAngle);

    // draw the arc
    output("S");
}

/**
 *
 * Device-specific methods
 *
 */
public void fillArcPW(double x, double y, double width, double height,
                      double startAngle, double arcAngle, int type) {

    double rad, x0, y0;

    // Make sure we are in the correct state
    updateContext();

    // set the color
    outputCurrentPaint("rg");

    // Compute the semi-major axes
    double semiMajorX = width / 2;
    double semiMajorY = height / 2;

    // Compute the centre of the arc
    x = x + semiMajorX;
    y = -(y + semiMajorY);

    // Move to the origin and draw to the start point of the arc
    // NOTE: we compute radians because 1.1 doesn't provide a conversion
    if (arcAngle < 0) {
        rad = (startAngle + arcAngle) / 180.0 * Math.PI;
    }
    else {
        rad = startAngle / 180.0 * Math.PI;
    }
    x0 = x + semiMajorX * Math.cos(rad);
    y0 = y + semiMajorY * Math.sin(rad);
    output(truncateDecimalDigits(x) + " " + truncateDecimalDigits(y) + " m " +
           truncateDecimalDigits(x0) + " " + truncateDecimalDigits(y0) + " l");

    // Construct the arc's path
    traceArcPath(x, y, semiMajorX, semiMajorY, startAngle, arcAngle);

    // close the area swept by the arc and fill it
    output("h f");
}

/**
 *
 * Device-specific methods
 *
 */
public void setClipPW(double x, double y, double width, double height) {

    // if this context is current, we must pop it to grow the clip region
    boolean current = isContextCurrent();
    if (current) {
        endContext();
    }


    // Since this path is independent of our old path,
    // we must manually clip it to our parent's clip
    if (parent != null && ((Graphics2DPDF) parent).clipSet) {

        Shape intersectionShape = null;
        Shape transformedParentClip = null;
        Shape parentClip = parent.getClip();

        try {
            // to get the real parent clip, we first have to
            // put it through the inverse of the transform
            // being used by this Graphics object
            AffineTransform inverseTransform;

            // If parent is frame, the transform owned by this graphics
            // class is independent of its parent.  Therefore, we can just
            // reverse the transformation used by this graphics class to
            // find the parent clip relative to the space being used by
            // this graphics class.
            if (((Graphics2DBase) parent).isFrameRoot()) {

                inverseTransform = pdfTransform.createInverse();

                // If parent is not a frame, the transform owned by this
                // graphics class contains the parent transform.  Therefore,
                // we must separate out the bit of the transform that applies only
                // to this class before we can form an inverse transformation
                // that will be used to find the parent clip relative to the
                // space being used by this graphics class.
            }
            else {
                AffineTransform parentTransform =
                    (AffineTransform) ((Graphics2D) parent).getTransform().clone();
                if (parentTransform.isIdentity()) {
                    inverseTransform = pdfTransform.createInverse();
                }
                else {
                    inverseTransform = parentTransform.createInverse();
                    inverseTransform.concatenate(pdfTransform);
                    inverseTransform = inverseTransform.createInverse();
                }
            }

            // now that we have the transformation from the parent's space,
            // we just need to apply it to the parent's clip to bring it
            // in line with the space being used by this graphics class
            transformedParentClip =
                inverseTransform.createTransformedShape(parentClip);
        } catch (NoninvertibleTransformException nte) {
            // Can't invert?  Just use the parent clip.
            transformedParentClip = parentClip;
        }

        Rectangle2D.Double desiredClip =
            new Rectangle2D.Double(x, y, width, height);

        // find the intersection of the parent clip transformed to
        // this space and the requested new clip
        if (transformedParentClip instanceof Rectangle2D) {
            intersectionShape = new Rectangle2D.Double();
            Rectangle2D.intersect((Rectangle2D) transformedParentClip,
                                  desiredClip,
                                  (Rectangle2D) intersectionShape);
        }
        else {
            Area intersectionArea = new Area(transformedParentClip);
            intersectionArea.intersect(new Area(desiredClip));
            intersectionShape = new GeneralPath();
            ((GeneralPath) intersectionShape).append(intersectionArea, false);
        }

        // new clip rectangle is intersection of desired clip and parent clip
        clip = intersectionShape;

    }
    else {
        // Set the new clipping rectangle
        clip = new Rectangle2D.Double(x, y, width, height);
    }

    clipSet = true;

    // if it was the current context, re-enter it
    if (current) {
        startContext();
    }
}

/**
 *
 * Device-specific methods
 *
 */
public void fillRectPW(double x, double y, double width, double height) {
    printRect(x, y, width, height, "f");
}

/**
 *
 * Device-specific methods
 *
 */
public void drawRectPW(double x, double y, double width, double height) {
    printRect(x, y, width, height, "S");
}

/**
 *
 * Device-specific methods
 *
 */
public void drawLinePW(double x1, double y1, double x2, double y2) {

    // Make sure we are in the correct state
    updateContext();

    // set the color
    outputCurrentPaint("RG");

    // set the line width
    setLineStroke();

    output(truncateDecimalDigits(x1) + " " +
           truncateDecimalDigits(-y1) + " m " +
           truncateDecimalDigits(x2) + " " +
           truncateDecimalDigits(-y2) + " l S");
}


/**
 *
 * Device-specific methods
 *
 */
public void drawRoundRectPW(double x, double y, double width,
                            double height, double arcWidth, double arcHeight) {
    drawRRect(x, y, width, height, arcWidth, arcHeight, "S");
}

/**
 *
 * Device-specific methods
 *
 */
public void fillRoundRectPW(double x, double y, double width,
                            double height, double arcWidth, double arcHeight) {
    drawRRect(x, y, width, height, arcWidth, arcHeight, "f");

}

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
////////////////////              Miscellaneous               ////////////
//////////////////// Some non-Graphics2D non Graphics methods ////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

/**
 *
 * Append a character to the output buffer.  *
 *
 */
protected void appendToOutString(int code, double width,
                                 FontPS font, boolean writingUnicodeAsHex) {
    String specials = "\\()";
    char escape_char = '\\';
    //int intCode = (int) code;

    // Note: the length of the output buffer can be wider
    // than the number of characters when we have to
    // convert char's to octal or write as hex

    // writing out multi-byte unicode as hex data
    if (writingUnicodeAsHex) {

        // write out high byte
        int partCode = (code >> 8);
        if (partCode <= 15)
            out.text.append("0");
        out.text.append(Integer.toHexString(partCode));

        // write out low byte
        partCode = (code & 0x00ff);
        if (partCode <= 15)
            out.text.append("0");
        out.text.append(Integer.toHexString(partCode));

        // outside the realm of alphabetic characters ...
    }
    else if (code < 32 || code > 126) {

        // single-byte character -- write out as octal
        if (code <= 255) {
            out.text.append(octal3((char) (code & 0x00ff)));

            // multi-byte character
            // intercept unicode value for Euro character -- PDF has its
            // own value for it
        }
        else if (code == 0x20ac) {
            out.text.append(octal3((char) 0x80));

            // multi-byte character
            // Ideally, this should never happen as we don't support
            // multi-byte non-hex characters other than the euro.
            // Any characters > 255 should have been intercepted and
            // substituted in the calling method.  But just in case
            // an above-ansi character is encountered, use the 'space'
            // character since the 'space' character metrics are substituted
            // for unsupported characters.
        }
        else {
            System.err.println("Warning: expecting single-byte character, " +
                               "found multi-byte character 0x" + Integer.toHexString(code));
            out.text.append(' ');

        }

        // single-byte character data
        // inside the realm of alphabetic characters
    }
    else {

        // if it's a special char, escape it
        if (specials.indexOf((char) code) != -1) {
            out.text.append(escape_char);
        }
        out.text.append((char) code);
    }
    out.width += FontMetricsPS.asPoints(width, getFont().getSize());
}

/**
 *
 * Flush the buffer to the file.
 * @param writingHex true if the format for writing the character string is
 *                   hex data, which is enclosed in angle brackets;  false if
 *                   the format is character data, which is enclosed in
 *                   standard curved brackets.
 * @param close      close if the output string needs to be closed off with
 *                   a new close bracket; false otherwise.
 *
 */
protected void printOutString(boolean writingHex, boolean close) {
    // terminate and output the buffer
    int length = out.text.length() + 7;
    StringBuffer buf = new StringBuffer(length);
    if (writingHex) {
        buf.append("[<");
        buf.append(out.text.toString());
        if (close) {
            buf.append(">");
        }
    }
    else {
        buf.append("[(");
        buf.append(out.text.toString());
        if (close) {
            buf.append(")");
        }
    }
    buf.append("]TJ");
    output(buf.toString());

    // reset the buffer
    out.x += out.width;
    out.width = 0;
}

/**
 *
 * End the text segment
 * @param writingHex true if the format for writing the character string is
 *                   hex data, which is enclosed in angle brackets;  false if
 *                   the format is character data, which is enclosed in
 *                   standard curved brackets.
 */
protected void endTextSegment(boolean writingHex) {
    if (writingHex)
        out.text.append(">");
    else
        out.text.append(")");
}

/**
 *
 * do kerning
 * @param writingHex true if the format for writing the character string is
 *                   hex data, which is enclosed in angle brackets;  false if
 *                   the format is character data, which is enclosed in
 *                   standard curved brackets.
 * @param open       true if a new open bracket should be output after the
 *                   kerning information; false otherwise.
 *
 */
protected void addKerning(int kerning, boolean writingHex, boolean open) {
    out.text.append((-kerning));
    if (open) {
        if (writingHex) {
            out.text.append("<");
        }
        else {
            out.text.append("(");
        }
    }
}

protected void outputColor(String strokeOrFill, Color c) {

    if (c == null) {
        return;
    }

    // Compare the current colour with the last stroke/fill colour
    if (strokeOrFill.equals("RG")) {
        if (c.equals(currentLineColor) && !lineColorChanged) {
            return;
        }
        else {
            currentLineColor = c;
            lineColorChanged = false;
        }
    }
    else {
        if (c.equals(currentFillColor) && !fillColorChanged) {
            return;
        }
        else {
            currentFillColor = c;
            fillColorChanged = false;
        }
    }

    // pdf colors are btw 0-1
    double r = c.getRed()   / 255.0;
    double g = c.getGreen() / 255.0;
    double b = c.getBlue()  / 255.0;

    // according to the pdf spec color values
    // should have no more than 4 digits

    // stroke uses RG fill uses rg
    output(truncateDecimalDigits(r) + " " +
           truncateDecimalDigits(g) + " " +
           truncateDecimalDigits(b) + " " + strokeOrFill);

    colorChanged = false;
}

protected void outputTexture(TexturePaint pattern) {
    String patternName = "/P" + patternIndex;
    patternIndex++;
    output("/Pattern cs " + patternName + " scn");
    printer.addPattern(patternName, pattern.getImage());
    outputPatternObjectDictionaries(pattern.getImage());
    // Respecify the fill color the next time outputColor() is called.
    fillColorChanged = true;
}

protected void outputCurrentPaint(String strokeOrFill) {

    Paint paint = getPaint();
    if (strokeOrFill.equals("RG") || paint == null) {
        outputColor(strokeOrFill, getColor());
    }
    else {
        if (paint instanceof Color) {
            outputColor(strokeOrFill, (Color)paint);
        }
        else if (paint instanceof TexturePaint) {
            outputTexture((TexturePaint)paint);
        }
    }
}

protected void outputCurrentColor(String strokeOrFill) {
    outputColor(strokeOrFill, getColor());
}

/**
 *
 * Some common code for polygons, called by both fillPolygon and
 * drawPolygon.
 *
 */
protected void printPolygon(int[] xPoints, int[] yPoints, int nPoints,
                            String strokeOrFill) {

    // Make sure we are in the correct state
    updateContext();

    // set the color
    String s = strokeOrFill.equals("S") ? "RG" : "rg";
    outputCurrentPaint(s);

    output(xPoints[0] + " " + (-yPoints[0]) + " m");

    for (int i = 1; i < nPoints; i++) {
        int x = xPoints[i];
        int y = -yPoints[i];
        output(x + " " + y + " l");
    }

    // handle winding rule
    if (strokeOrFill.equals("f")) {
        // fill, set winding rule
        if (getWindingRule() == GeneralPath.WIND_NON_ZERO) {
            strokeOrFill = "f*";
        }
    }

    output("h " + strokeOrFill);
}

/**
 *
 * Some common code for drawing round rectangles, called by both
 * fillRoundRect and drawRoundRect.
 *
 */
protected void drawRRect(double x, double y, double width, double height,
                         double arcWidth, double arcHeight, String strokeOrFill) {

    // Make sure we are in the correct state
    updateContext();

    // adjust to java origin
    y = -y;

    // set the color
    String s = strokeOrFill.equals("S") ? "RG" : "rg";
    outputCurrentPaint(s);

    // set the line width
    setLineStroke();

    // compute the control point offsets
    double cpConstX = arcWidth * 0.448;
    double cpConstY = arcHeight * 0.448;

    // Move to the start of the first arc
    output(truncateDecimalDigits(x + arcWidth) + " " +
           truncateDecimalDigits(y) + " m");

    // Draw the first arc
    output(truncateDecimalDigits(x + cpConstX) + " " +
           truncateDecimalDigits(y) + " " + truncateDecimalDigits(x) + " " +
           truncateDecimalDigits(y - cpConstY) + " " +
           truncateDecimalDigits(x) + " " +
           truncateDecimalDigits(y - arcHeight) + " c");

    // Draw the left side
    if (height > arcHeight * 2) {
        output(truncateDecimalDigits(x) + " " +
               truncateDecimalDigits(y - height + arcHeight) + " l");
    }

    // Draw the lower-left arc
    output(truncateDecimalDigits(x) + " " +
           truncateDecimalDigits(y - height + cpConstY) + " " +
           truncateDecimalDigits(x + cpConstX) + " " +
           truncateDecimalDigits(y - height) + " " +
           truncateDecimalDigits(x + arcWidth) + " " +
           truncateDecimalDigits(y - height) + " c");

    // Draw the bottom
    if (width > arcWidth * 2) {
        output(truncateDecimalDigits(x + width - arcWidth) + " " +
               truncateDecimalDigits(y - height) + " l");
    }

    // Draw the lower-right arc
    output(truncateDecimalDigits(x + width - cpConstX) + " " +
           truncateDecimalDigits(y - height) + " " +
           truncateDecimalDigits(x + width) + " " +
           truncateDecimalDigits(y - height + cpConstY) + " " +
           truncateDecimalDigits(x + width) + " " +
           truncateDecimalDigits(y - height + arcHeight) + " c");

    // Draw the right side
    if (height > arcHeight * 2) {
        output(truncateDecimalDigits(x + width) + " " +
               truncateDecimalDigits(y - arcHeight) + " l");
    }

    // Draw the last arc and close the path
    output(truncateDecimalDigits(x + width) + " " +
           truncateDecimalDigits(y - cpConstY) + " " +
           truncateDecimalDigits(x + width - cpConstX) + " " +
           truncateDecimalDigits(y) + " " +
           truncateDecimalDigits(x + width - arcWidth) + " " +
           truncateDecimalDigits(y) + " c h");

    // adjust the bounds
    // adjustBounds(x, (int) (y + PAGE_MAX_Y), width, height);

    output(strokeOrFill);
}

/**
 *
 * output the line stroke if it has changed
 *
 */
protected void setLineStroke() {

    if (strokeChanged) {

        // set the line width
        float width = ((BasicStroke) currentStroke).getLineWidth();
        output(truncateDecimalDigits(width) + " w");

        // set the dash-ness
        float dashArray[] = ((BasicStroke) currentStroke).getDashArray();
        float dashPhase = ((BasicStroke) currentStroke).getDashPhase();

        if (dashArray != null) {
            output("[ ");
            for (int i = 0; i < dashArray.length; i++) {
                output(truncateDecimalDigits(dashArray[i]) + " ");
            }
            output("] ");

            output(String.valueOf(dashPhase).toString());

            output(" d");

            dashMode = true;
        }
        else if (dashMode) {
            // restore solid line, the default
            output("[] 0 d");
            dashMode = false;
        }

    }
    strokeChanged = false;
}

/**
 *
 * Load font
 *
 */
protected void loadFont(String fontName, boolean unicode) {

    // get the font's abbreviation

    String abbrev = printer.getFontAbbreviation(fontName);
    if (abbrev != null) {
        printer.streamInfo.fontList.put(
            (unicode ? fontName + "-Unicode" : fontName), fontName);

        // select the font and size
        output("/" + abbrev +
               (unicode ? "U" : "") + " " + getFont().getSize() + " Tf");
    }
    else {
        throw new IllegalArgumentException("Font not found " + fontName);
    }
}

/**
 *
 * Construct the arc's path
 *
 */
public void traceArcPath(double x, double y, double semiMajorX,
                         double semiMajorY, double startAngle,
                         double arcAngle) {

    double theta, rad, endAngle, cp_factor, angle;
    double x0 = 0, x1 = 0, x2 = 0, x3 = 0, y0 = 0, y1 = 0, y2 = 0, y3 = 0;

    // If arcAngle is negative, swap the endpoints
    if (arcAngle < 0) {
        startAngle += arcAngle;
        arcAngle = -arcAngle;
    }

    // If arc sweep < 0.001 degree, draw quick fake (straight line)
    if (Math.abs(arcAngle) < 0.001) {
        rad = (startAngle + arcAngle) / 180.0 * Math.PI;
        x0 = x + semiMajorX * Math.cos(rad);
        y0 = y + semiMajorY * Math.sin(rad);
        output(truncateDecimalDigits(x0) + " " +
               truncateDecimalDigits(-y0) + " l");
        return;
    }

    // If arc is degenerate (straight line), draw it
    if (semiMajorX <= 0) {
        rad = (startAngle + arcAngle) / 180.0 * Math.PI;
        y0 = y + semiMajorY * Math.sin(rad);
        output(truncateDecimalDigits(x) + " " +
               truncateDecimalDigits(-y0) + " l");
        return;
    }
    else if (semiMajorY <= 0) {
        rad = (startAngle + arcAngle) / 180.0 * Math.PI;
        x0 = x + semiMajorX * Math.cos(rad);
        output(truncateDecimalDigits(x0) + " " +
               truncateDecimalDigits(-y) + " l");
        return;
    }

    // From this point on, we don't have to worry about negative angles
    // or division by zero for the axes.

    // Pre-compute the offset constants for 90-degree arcs
    double x_cp_offset = 0.5522848 * semiMajorX;
    double y_cp_offset = 0.5522848 * semiMajorY;

    // Compute the square of the ratios of the axes; these are necessary
    // to adjust the slope of tangent lines to the ellipse
    double axis_ratio = semiMajorX / semiMajorY;
    double x_ratio_square = axis_ratio;
    double y_ratio_square = 1.0 / x_ratio_square;

    // Compute the start point and end angle
    // NOTE: we compute radians because 1.1 doesn't provide a conversion
    rad = startAngle / 180.0 * Math.PI;
    x0 = x + semiMajorX * Math.cos(rad);
    y0 = y + semiMajorY * Math.sin(rad);

    // Compute the first cardinal point in the range
    theta = ((int) startAngle / 90) * 90;

    // If there is a partial (less than quarter) arc at the start of the range
    if (Math.abs(theta - startAngle) > 0.001) {

        // Adjust theta to the next cardinal point after the start
        if (startAngle > 0) {
            theta += 90;
        }

        // If the initial arc extends to (or past) a cardinal point, stop it
        endAngle = startAngle + arcAngle;
        if (endAngle > theta) {
            endAngle = theta;
        }

        // Draw an arc from startAngle to endAngle (less than 1/4 circle)
        // NOTE: we compute radians because 1.1 doesn't provide a conversion
        rad = endAngle / 180.0 * Math.PI;
        x3 = x + semiMajorX * Math.cos(rad);
        y3 = y + semiMajorY * Math.sin(rad);

        // Determine the actual arc swept
        angle = endAngle - startAngle;

        // Compute the bezier curve constant for this angle
        cp_factor = computeControlPointFactor(angle);

        // Compute the control points
        x1 = x0 - cp_factor * x_ratio_square * (y0 - y);
        y1 = y0 + cp_factor * y_ratio_square * (x0 - x);
        x2 = x3 + cp_factor * x_ratio_square * (y3 - y);
        y2 = y3 - cp_factor * y_ratio_square * (x3 - x);

        // Draw the arc segment
        drawSegment(x1, y1, x2, y2, x3, y3);

        // Advance to the start of the next arc segment
        x0 = x3;
        y0 = y3;
    }

    // While we can draw quarter circles within the range of the arc
    endAngle = startAngle + arcAngle;
    while (theta + 90 <= endAngle) {

        int modulus = ((int) theta) % 360;

        // Generate the control point offsets of the curve
        if (modulus == 0) {
            x1 = 0;
            y1 = y_cp_offset;
            x2 = x_cp_offset;
            y2 = 0;
        }
        else if (modulus == 90 || modulus == -270) {
            x1 = -x_cp_offset;
            y1 = 0;
            x2 = 0;
            y2 = y_cp_offset;
        }
        else if (modulus == 180 || modulus == -180) {
            x1 = 0;
            y1 = -y_cp_offset;
            x2 = -x_cp_offset;
            y2 = 0;
        }
        else if (modulus == 270 || modulus == -90) {
            x1 = x_cp_offset;
            y1 = 0;
            x2 = 0;
            y2 = -y_cp_offset;
        }

        // Compute the endpoint of this segment
        theta += 90;

        // NOTE: we compute radians because 1.1 doesn't provide a conversion
        rad = theta / 180.0 * Math.PI;
        x3 = x + semiMajorX * Math.cos(rad);
        y3 = y + semiMajorY * Math.sin(rad);

        // Compute the control points
        x1 += x0;
        y1 += y0;
        x2 += x3;
        y2 += y3;

        // Draw the curve for this segment
        drawSegment(x1, y1, x2, y2, x3, y3);

        // Advance to the start of the next segment
        x0 = x3;
        y0 = y3;
    }

    // If there is a segment of arc left to be drawn
    if (theta < endAngle && Math.abs(theta - endAngle) > 0.001) {

        // Draw an arc from theta to endAngle (less than 1/4 circle)
        // NOTE: we compute radians because 1.1 doesn't provide a conversion
        rad = endAngle / 180.0 * Math.PI;
        x3 = x + semiMajorX * Math.cos(rad);
        y3 = y + semiMajorY * Math.sin(rad);

        // Determine the actual arc swept
        angle = endAngle - theta;

        // Compute the bezier curve constant for this angle
        cp_factor = computeControlPointFactor(angle);

        // Compute the control points
        x1 = x0 - cp_factor * x_ratio_square * (y0 - y);
        y1 = y0 + cp_factor * y_ratio_square * (x0 - x);
        x2 = x3 + cp_factor * x_ratio_square * (y3 - y);
        y2 = y3 - cp_factor * y_ratio_square * (x3 - x);

        // Draw the arc segment
        drawSegment(x1, y1, x2, y2, x3, y3);

        // Advance to the start of the next arc segment
        x0 = x3;
        y0 = y3;
    }

}

protected double computeControlPointFactor(double angle) {
    double rad, factor;

    // NOTE: we compute radians because 1.1 doesn't provide a conversion
    rad = (angle / 2.0) / 180.0 * Math.PI;

    // r * (4/3) * (1 - cos(theta/2)) / sin(theta/2)
    factor = (4.0 / 3.0) * (1.0 - Math.cos(rad)) / Math.sin(rad);

    return (factor);
}

protected void drawSegment(double x1, double y1, double x2, double y2,
                           double x3, double y3) {

    output(truncateDecimalDigits(x1) + " " + truncateDecimalDigits(y1) + " " +
           truncateDecimalDigits(x2) + " " + truncateDecimalDigits(y2) + " " +
           truncateDecimalDigits(x3) + " " + truncateDecimalDigits(y3) + " c");
}

protected void printRect(double x, double y, double width, double height,
                         String strokeOrFill) {

    // Make sure we are in the correct state
    updateContext();

    // adjust to java origin
    y = -y;

    // set the color
    String s = strokeOrFill.equals("S") ? "RG" : "rg";
    outputCurrentPaint(s);

    // set the line width
    setLineStroke();

    output(truncateDecimalDigits(x) + " " + truncateDecimalDigits(y) + " " +
           truncateDecimalDigits(width) + " " + truncateDecimalDigits(-height) +
           " re " + strokeOrFill);
}

/**
 *
 * Overrides Graphics2DBase to add length
 *
 */
protected void output(String string) {
    printer.output(string);
}

/**
 * Outputs the Image XObject dictionaries associated with a pattern.
 * @param img The Image used for the pattern.
 */
protected void outputPatternObjectDictionaries(BufferedImage img) {

    BufferedImage imgCopy = img.getSubimage(0, 0, img.getWidth(), img.getHeight());
    Int maskObjectNumber = new Int();
    drawJavaImage(imgCopy, 0, 0, 0, 0, null, null,
                  false, true, maskObjectNumber);
    drawJavaImage(img, 0, 0, 0, 0, null, null,
                  false, false, maskObjectNumber);
}

/**
 *
 * Draw normalized jpeg/gif image, that is, pixels which encode rgb info.
 * Note that since we allow pdf to do its own scaling, the width
 * and height parameters represent the DESIRED size of the image, not its
 * actual size.
 *
 */
protected boolean drawJavaImage(Image img, int x, int y, int width, int height,
                                Color bgcolor, ImageObserver observer,
                                boolean writeImagePrintCommand,
                                boolean isMask,
                                Int maskObjectNumber) {

    long streamStart = 0;
    int imgWidth = img.getWidth(null);
    int imgHeight = img.getHeight(null);

    // adjust for java origin
    y = -y - height;
    int imageOutNumber = -1;
    boolean cachedImage = false;
    if (printer.imageCache.containsKey(img)) {
        imageOutNumber = ((Integer) printer.imageCache.get(img)).intValue();
        cachedImage = true;
    }
    else {
        imageOutNumber = printer.imageNumber;
    }

    if (cachedImage) {
        if (writeImagePrintCommand) {
            // Write the image printing commands to the current contents stream
            outputImageDraw(imageOutNumber, x, y, width, height);
        }

        printer.streamInfo.imagePresent = true;
        Integer newInt = new Integer(imageOutNumber);
        if (!printer.streamInfo.imageList.contains(newInt)) {
            printer.streamInfo.imageList.add(newInt);
            printer.streamInfo.xObjectList.add(
                printer.docImageList.get(imageOutNumber));
        }
        return true;
    }

    // retrieve the image pixel data
    int pixels[] = new int[imgWidth * imgHeight];

    PixelGrabber pg =
        new PixelGrabber(img, 0, 0, imgWidth, imgHeight, pixels, 0, imgWidth);
    try {
        pg.grabPixels();
    } catch (InterruptedException e) {
        System.err.println("interrupted waiting for pixels!");
        return false;
    }
    if ((pg.getStatus() & ImageObserver.ABORT) != 0) {
        System.err.println("image fetch aborted or errored");
        return false;
    }

    if (writeImagePrintCommand) {
        // Write the image printing commands to the current contents stream
        // Make sure to do this AFTER the pixel grab, since we don't
        // want to write a reference to an image that can't be loaded
        outputImageDraw(imageOutNumber, x, y, width, height);
    }

    Integer imgObjNum = new Integer(imageOutNumber);
    printer.imageCache.put(img, imgObjNum);

    int imageObjectNumber = ++printer.objectNumber;
    Integer xObjNum = new Integer(imageObjectNumber);

    printer.streamInfo.imageList.add(imgObjNum);
    printer.docImageList.add(xObjNum);
    printer.streamInfo.xObjectList.add(xObjNum);

    if (isMask && maskObjectNumber != null) {
        maskObjectNumber.value = imageObjectNumber;
    }

    // save current page stream, create new image stream
    printer.switchToNewStream();

    // Output the Image XObject dictionary
    printer.storeObjectReference(imageObjectNumber, printer.getCurrentOffset());
    printer.streamInfo.imagePresent = true;
    output(imageObjectNumber + " 0 obj\n<<");
    output("/Type /XObject\n/Subtype /Image");

    output("/Name /Im" + printer.imageNumber++);

    // Image dimensions
    output("/Width " + imgWidth + "\n/Height " + imgHeight);
    output("/BitsPerComponent 8");

    String colorSpace = (isMask) ? "/DeviceGray" : "/DeviceRGB";
    output("/ColorSpace " + colorSpace);

    output("/Filter [ /ASCII85Decode /FlateDecode ]");
    output("/Length " + (imageObjectNumber + 1) + " 0 R");
    if (!isMask && maskObjectNumber != null) {
        output("/SMask " + maskObjectNumber.value + " 0 R");
    }
    output("/Resources " + (imageObjectNumber + 2) + " 0 R\n>>");

    // Start the image data stream
    output("stream");
    streamStart = printer.getCurrentOffset();

    // Create a stream to store compressed image data, and a compression filter
    ByteArrayOutputStream byteOS = new ByteArrayOutputStream();
    DeflaterOutputStream compressor =
        new DeflaterOutputStream(new Ascii85OutputStream(byteOS));

    int offset = 0;

    // array to hold a line of pixel data
    int bytesPerRow = (isMask) ? imgWidth : imgWidth * 3;
    byte[] scanLine = new byte[bytesPerRow];

    int h = pg.getHeight();
    int w = pg.getWidth();
    for (int i = 0; i < h; i++) {
        offset = 0;

        // Copy a row of pixel values to the array
        for (int j = 0; j < w; j++) {
            int n = pixels[i * w + j];

            if (isMask) {
                int alpha = (n >> 24) & 0xFF;
                scanLine[offset++] = (alpha == 0) ? 0 : (byte)0xFF;
            }
            else {
                // convert integer colour value into bytes
                scanLine[offset++] = (byte) ((n & 0xFF0000) >> 16); // red
                scanLine[offset++] = (byte) ((n & 0xFF00) >> 8);    // green
                scanLine[offset++] = (byte) (n & 0xFF);             // blue
            }
        }

        // Write the row of pixel values to the compressed buffer
        try {
            compressor.write(scanLine);
        } catch (IOException e) {
        }

    }

    // End the compressed buffer
    try {
        compressor.finish();
        compressor.flush();
    } catch (IOException e) {
    }

    // Copy the deflated data to the output
    int bufferSize = byteOS.size();
    byte[] compressedData = byteOS.toByteArray();
    offset = 0;

    while (offset + 72 < bufferSize) {
        printer.output(compressedData, offset, 72);
        offset += 72;
        // force a carriage return
        printer.output((byte) '\n');
    }
    printer.output(compressedData, offset, bufferSize - offset);
    // force a carriage return
    printer.output((byte) '\n');

    // End the image stream and object
    int streamLength = (int) (printer.getCurrentOffset() - streamStart - 1);
    output("endstream\nendobj");

    // Store the length object for the image data stream
    printer.storeObjectReference(++printer.objectNumber,
                                 printer.getCurrentOffset());
    output(printer.objectNumber + " 0 obj\n" + streamLength + " \nendobj");

    // write resources dictionary for this image object
    printer.storeObjectReference(++printer.objectNumber,
                                 printer.getCurrentOffset());
    output(printer.objectNumber + " 0 obj\n<<");
    printer.writeResourcesDictionary();
    output(">>\nendobj");

    // return to page stream
    printer.switchBackToOldStream();

    return true;
}

protected void outputImageDraw(int imageOutNumber, double x, double y,
                               double width, double height) {
    output("q");
    output(width + " 0 0 " + height + " " + x + " " + y + " cm");
    output("0 0 1 1 re W n");
    output("/Im" + imageOutNumber + " Do");
    output("Q");
}

protected String octal3(char code) {

    StringBuffer sb = new StringBuffer(Integer.toOctalString(code));

    int length = sb.length();
    for (int i = length; i < 3; i++) {
        sb.insert(0, 0);
    }
    sb.insert(0, '\\');

    return sb.toString();
}

/**
 * Wrapper for an int value.
 */
protected class Int {
    int value = -1;
}

}
