/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCPDFPrinter.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:24 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page.adobe.pdf;

import com.klg.jclass.util.SeqRangeCollector;
import com.klg.jclass.page.TrueTypeFontProperties;
import com.klg.jclass.page.PrintException;
import com.klg.jclass.page.JCPrinter;
import com.klg.jclass.page.adobe.postscript.FontPS;
import com.klg.jclass.page.adobe.postscript.CharMetricPS;
import com.klg.jclass.page.FontLibrary;
import com.klg.jclass.page.Graphics2DBase;
import com.klg.jclass.page.FontParser;

import javax.swing.tree.DefaultMutableTreeNode;
import java.util.*;
import java.util.List;
import java.util.zip.DeflaterOutputStream;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Point2D;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.OutputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;

/**
 *
 * Interface methods for document handling
 *
 */
public class JCPDFPrinter extends JCPrinter {

/** The number to be allocated to the next object created */
protected int objectNumber;
/** The object reference number of the root Pages object */
protected int rootPagesReference;
/** The total number of pages output in this PDF document */
protected int totalPageCount = 0;
/** The object reference number of the current Pages object */
protected int pagesReference;
/** The object reference number of the current Page object */
protected int currentPageObjectNumber = -1;
/** The object reference number of the standard Font encoding */
protected int encodingObj;
/** The object reference number of the document Graphics State Dictionary */
protected int graphicsDictionaryObj;
/** A List of the Page objects in the current Pages object */
protected Vector pageList = new Vector();
/** A List of the Pages objects in the document */
protected Vector pagesList = new Vector();
/** A List of contents streams on the current Page */
protected Vector contentsList = new Vector();
/** A List of annotations on the current Page */
protected Vector annotationList = new Vector();
/** A Set of Fonts which have been used in the document */
protected Hashtable fontList = new Hashtable();
/** The bounding box of the current page */
protected Rectangle2D bounds = new Rectangle2D.Double();
/** Track the size and orientation of the last page printed */
protected Rectangle2D oldPageSize = null;
/** Track the size of the current page being printed */
protected Rectangle2D currentPageSize = null;
/** Records if the document has been opened. */
protected boolean docOpen = false;
/** List of available images */
protected WeakHashMap imageCache = new WeakHashMap();
/** List of document Image objects */
protected Vector docImageList = new Vector();
/** Determines whether to use PDF Line Stroke Adjustment mode or not.*/
protected boolean useAutoStrokeAdjustment = true;
/** Object number for java component. */
protected int drawJavaComponentObjectNumber = -1;
/** Object number for java component's length object. */
protected int drawJavaComponentLengthObjectNumber = -1;
/** Object number for java component's resource dictionary. */
protected int drawJavaComponentResourcesObjectNumber = -1;
/** True if drawing java component. */
protected boolean drawingJavaComponent = false;

/* list of rectangles representing bounding boxes for pdf annotations
 (used for hyperlinks).  The items correspond to the items in the
 same position in the hyperlinkInfoList variable. */
/** Internal use only - do not use */
protected Vector annotationRectangles = new Vector();

/* list of HyperlinkInfo objects representing hyperlinks being added
 to the document.  The items correspond to the items in the
 same position in the annotationRectangles variable. */
/** Internal use only - do not use */
protected Vector hyperlinkInfoList = new Vector();

/* table of action object numbers keyed by link destination String. */
/** Internal use - do not use */
protected Hashtable hyperlinkObjects  = new Hashtable();

/* Table of internal locations, keyed by location name, sorted
 by key in ascending order. Value is a Point2D.Double instance.
 */
/** Internal use only - do not use */
protected TreeMap annoInternalLocations = new TreeMap();

/* Table of pages for internal locations, keyed by location name.
 Value is integer representing page object number.
 */
/** Internal use only - do not use */
protected Hashtable annoInternalPages = new Hashtable();

/** Information for current page or xobject stream. */
protected StreamInfo streamInfo = new StreamInfo();
/** Stack of StreamInfo objects.  Used to save page stream while writing
 XObject stream. */
protected Stack streamStack = new Stack();
/** List of XObject Reference Lists for current page. */
protected Vector pageXObjectReferences = new Vector();
/** List of XObject Reference Streams for current page. */
protected Vector pageXObjectStreams = new Vector();

/**
 * Construct a PDF printer given an OutputStream. Akin to calling the 5-parameter
 * constructor with these values:
 * <pre>
 *      JCPrinter p = new JCPDFPrinter(
 *              someOutputStream,                       // the output stream
 *              new com.klg.jclass.page.adobe.postscript.AFMParser(), // the parser
 *              "/com/klg/jclass/page/adobe/fonts.jar", // jar location
 *              ".afm",                                 // file extension
 *              "com.klg.jclass.page.adobe.JCAdobeFontMap");
 * </pre>
 * <P>
 * <b>Note:</b> For large documents and for optimal performance, wrap
 * the output stream in a <code>BufferedOutputStream</code> large enough
 * to hold the generated output. For example:
 * <pre>
 *      BufferedOutputStream bos    = new BufferedOutputStream(os, 2048);
 *      JCPrinter printer           = new JCPDFPrinter(bos, ...);
 * </pre>
 * would create a buffer large enough to hold a document of 2Kb.
 *
 * @param os The OutputStream to which the PDF is written
 * @see com.klg.jclass.page.adobe.postscript.AFMParser
 */
public JCPDFPrinter(OutputStream os) {
    this.os = os;

    // make sure to save stream in current StreamInfo object
    streamInfo.os = os;

    // force fonts to be loaded
    FontLibrary.initFontLibrary();

    // Load default fonts
    FontPS.getAllFonts();
}

/**
 * Allows detailed specification of font creation. Only .afm files may
 * be added using this constructor.
 * <P>
 * <b>Note:</b> For large documents and for optimal performance, wrap
 * the output stream in a <code>BufferedOutputStream</code> large enough
 * to hold the generated output. For example:
 * <pre>
 *      BufferedOutputStream bos    = new BufferedOutputStream(os, 2048);
 *      JCPrinter printer           = new JCPDFPrinter(bos, ...);
 * </pre>
 * would create a buffer large enough to hold a document of 2Kb.
 *
 * @param os            the <code>OutputStream</code> to which formatted output 
 *                      should be sent
 * @param parser        the <code>FontParser</code> which creates fonts from afm
 *                      files. Currently, only instances of 
 * com.klg.jclass.adobe.postscript.AFMParser are supported.
 * @param jarLocation   the location of the jar containing afm files relative
 *                      to the class loader, including the name of the jar file
 * @param fileExtension the parser will turn files contained in "jarLocation"
 *                      ending with this extension into fonts
 * @param fontMapFileName the user fontmap file; a file containing mappings
 *                      between font names and their aliases
 * @see com.klg.jclass.page.adobe.postscript.AFMParser#AFMParser
 *
 */
public JCPDFPrinter(OutputStream os, FontParser parser, String jarLocation,
                    String fileExtension, String fontMapFileName) {
    this.os = os;

    // force fonts to be loaded
    FontLibrary.initFontLibrary();

    // force fonts to be loaded
    FontPS.getAllFonts(parser, jarLocation, fileExtension, fontMapFileName);

}


/*
* Sets output stream for PostScript, PDF and PCL printers.
* @param OutputStream the stream to which output will be written
*/
public void setOutputStream(OutputStream os) {
    super.setOutputStream(os);

    // make sure to save stream in current StreamInfo object
    streamInfo.os = os;
}

/**
 * Set the value of the AutoStrokeAdjustment property.  This property
 * determines whether output PDF uses Line Stroke Adjustment mode or not.
 * (The default is true.)  Using Line Stroke Adjustment will ensure that
 * lines defined to be the same size get drawn as the same number of
 * pixels when renderered to the screen, but the resulting changes in
 * the position of lines may mean that some lines (such as those used
 * by table borders) no longer connect exactly.  See PDF reference
 * documentation (available from Adobe) for more information.
 */
public void setAutoStrokeAdjustment(boolean useASA) {
    useAutoStrokeAdjustment = useASA;
}

/**
 * Get the value of the AutoStrokeAdjustment property.  This property
 * determines whether output PDF uses Line Stroke Adjustment mode or not.
 * (The default is true.)  Using Line Stroke Adjustment will ensure that
 * lines defined to be the same size get drawn as the same number of
 * pixels when renderered to the screen, but the resulting changes in
 * the position of lines may mean that some lines (such as those used
 * by table borders) no longer connect exactly.  See PDF reference
 * documentation (available from Adobe) for more information.
 */
public boolean getAutoStrokeAdjustment() {
    return useAutoStrokeAdjustment;
}

/**
 * Returns whether or not openDocument has already
 * been called.
 * @return docOpen
 */
public boolean isDocOpen() {
    return docOpen;
}

/**
 * Creates default styles, any other one-time initialisation
 * Builds font families from available fonts.
 */
public void setup() {
    // no PDF-specific initialisation required
}

/**
 * Returns true since PDF can scale images itself.
 */
public boolean isImageScalable() {
    return true;
}

/**
 * Initialise the document (document header, initialise lists).
 * @param pageSize The size of a default/first page
 */
public void openDocument(Rectangle2D pageSize) {

    super.openDocument(pageSize);

    // clean up from previous runs
    //fix for PR11222
    //these values weren't being reset
    totalPageCount = 0;
    //fix for PR11222

    streamInfo.openDocument();

    pageList.clear();
    pagesList.clear();
    contentsList.clear();

    // clear hyperlink related storage
    annoInternalLocations.clear();
    annoInternalPages.clear();
    annotationRectangles.clear();
    hyperlinkInfoList.clear();
    hyperlinkObjects.clear();
    annotationList.clear();

    imageCache.clear();
    docImageList.clear();
    // Create PDF document header
    output("%PDF-1.4");
    byte[] codeLine = new byte[6];
    codeLine[0] = (byte) '%';
    codeLine[1] = (byte) '\342';
    codeLine[2] = (byte) '\343';
    codeLine[3] = (byte) '\317';
    codeLine[4] = (byte) '\323';
    codeLine[5] = (byte) '\n';
    output(codeLine);

    objectNumber = 0;
    rootPagesReference = ++objectNumber;
    pagesReference = ++objectNumber;

    // Output the standard Type 1 font encoding
    writeFontEncoding();
    encodingObj = objectNumber;

    // Output the graphics state dictionary for the document
    writeDocGraphicsDictionary();
    graphicsDictionaryObj = objectNumber;

    docOpen = true;
}

/**
 * Completes the document (pages tree, cross-references, trailer).
 * @param pageSize The size of the default/last page
 */
public void closeDocument(Rectangle2D pageSize) {

    int catalogRef, infoRef;
    long start_xref;

    // Write the pages structure
    writePagesTree(pageSize);

    // Write name tree if needed for internal hyperlink locations
    Graphics2DPDF gcpdf = (Graphics2DPDF) getGraphics();
    boolean needNameTree = !annoInternalLocations.isEmpty();
    int nameTreeObjectNumber = -1;
    int nameDictionaryObjectNumber = -1;
    if (needNameTree) {
        // Name Tree -- first create objects detailing location on page
        //           -- store object number of created object for next step
        Hashtable destinationObjects = new Hashtable();
        Iterator locationIterator = annoInternalLocations.keySet().iterator();
        while (locationIterator.hasNext()) {
            String locationName = (String) locationIterator.next();
            Point2D.Double location =
                (Point2D.Double) annoInternalLocations.get(locationName);
            Integer pageObject = (Integer) annoInternalPages.get(locationName);

            storeObjectReference(++objectNumber, getCurrentOffset());
            destinationObjects.put(locationName, new Integer(objectNumber));
            output(objectNumber + " 0 obj");
            output("[ " + pageObject.intValue() + " 0 R /XYZ " +
                   gcpdf.truncateDecimalDigits(location.getX()) + " " +
                   gcpdf.truncateDecimalDigits(location.getY()) + " 0 ]");
            output("endobj");
        }

        // Write out Name Tree, associating internal hyperlink location
        // names with the page location objects created in the previous step
        nameTreeObjectNumber = ++objectNumber;
        storeObjectReference(nameTreeObjectNumber, getCurrentOffset());
        output(nameTreeObjectNumber + " 0 obj\n<<");
        output("/Names [");
        locationIterator = annoInternalLocations.keySet().iterator();
        while (locationIterator.hasNext()) {
            String locationName = (String) locationIterator.next();
            Integer destObject = (Integer) destinationObjects.get(locationName);
            output("(" + locationName + ") " +
                   destObject.intValue() + " 0 R");
        }
        output("]");
        output(">> \nendobj");

        // Write out Name Dictionary referring to Name Tree
        nameDictionaryObjectNumber = ++objectNumber;
        storeObjectReference(nameDictionaryObjectNumber, getCurrentOffset());
        output(nameDictionaryObjectNumber + " 0 obj\n<<");
        output("/Dests " + nameTreeObjectNumber + " 0 R");
        output(">> \nendobj");
    }

    // clear annotaion location table -- no longer needed
    annoInternalLocations.clear();
    annoInternalPages.clear();

    // output bookmark tree if one exists
    boolean haveOutlines = false;
    int outlineDictionaryObjectNumber = -1;
    if (bookmarkTree != null) {
        haveOutlines = true;
        outlineDictionaryObjectNumber = ++objectNumber;
        outputBookmarkTree(
            (DefaultMutableTreeNode) bookmarkTree.getRoot(),
            outlineDictionaryObjectNumber);
    }

    // Write the Font objects
    Enumeration efont = fontList.elements();
    while (efont.hasMoreElements()) {
        AcroFontEntry fe = (AcroFontEntry) efont.nextElement();
        writeFontDescription(fe);
    }

    // Write the Catalog object
    storeObjectReference(++objectNumber, getCurrentOffset());
    catalogRef = objectNumber;

    output(objectNumber + " 0 obj\n<<");
    output("/Type /Catalog");
    output("/Pages " + rootPagesReference + " 0 R");
    if (needNameTree) {
        output("/Names " + nameDictionaryObjectNumber + " 0 R");
    }
    if (haveOutlines) {
        output("/Outlines " + outlineDictionaryObjectNumber + " 0 R");
        if (isBookmarkTreeVisible()) {
            output("/PageMode /UseOutlines");
        }
    }
    output(">> \nendobj");

    // Write the info object
    storeObjectReference(++objectNumber, getCurrentOffset());
    infoRef = objectNumber;
    output(objectNumber + " 0 obj\n<<");
    output("/Creator (Quest Software's PageLayout 6.3.0)");

    // Compose and write the current (file creation) time
    GregorianCalendar calendar = new GregorianCalendar();
    StringBuffer date = new StringBuffer("/CreationDate (D:");
    date.append(calendar.get(Calendar.YEAR));
    date.append(calendar.get(Calendar.MONTH));
    date.append(calendar.get(Calendar.DAY_OF_MONTH));
    date.append(calendar.get(Calendar.HOUR));
    date.append(calendar.get(Calendar.MINUTE));
    date.append(calendar.get(Calendar.SECOND));
    date.append(") ");
    output(date.toString());
    output(">> \nendobj");

    // Write the cross-reference table
    start_xref = streamInfo.fileOffset;
    output("xref");
    output("0 " + (objectNumber + 1) + "\n0000000000 65535 f ");

    Collections.sort(streamInfo.referenceList);
    for (int i = 0; i < objectNumber; i++) {
        AcroReference reference =
            (AcroReference) streamInfo.referenceList.elementAt(i);
        // format is XXXXXXXXXX XXXXX n
        output(pad10(reference.offset) + " 00000 n ");
    }

    // Write the PDF trailer
    output("trailer\n<<\n/Size " + (objectNumber + 1));
    output("/Root " + catalogRef + " 0 R");
    output("/Info " + infoRef + " 0 R");
    output(">>");
    output("startxref\n" + start_xref + "\n%%EOF");

    super.closeDocument(pageSize);

    // Discard the graphics used to build the document
    graphics.dispose();
    graphics = null;

    fontList.clear();

    docOpen = false;
}


/**
/**
 * Internal use method (noop). Do not use.
 */
protected void outputBookmarkTree(DefaultMutableTreeNode rootNode,
                              int rootObjectNumber) {


}




/**
 * Starts a page; create the correct initial transformation and initialise
 * the page resources lists.
 * @param pageSize the dimensions of this page
 * @param pageNumber the number of this page in the document
 */
public void beginPage(Rectangle2D pageSize, int pageNumber) {

    super.beginPage(pageSize, pageNumber);

    // Check if this page's attributes are changed since the last page
    if (pageChanged(pageSize)) {

        // Write the Pages object for the last set of pages
        writePagesObject(pagesReference, pageList, rootPagesReference,
                         oldPageSize);

        // Start a new Pages object
        pageList.clear();
        pagesList.add(new Integer(pagesReference));
        pagesReference = ++objectNumber;

        oldPageSize = (Rectangle2D) pageSize.clone();
    }

    // reset stream lists
    streamInfo.beginPage();

    // reset lists
    contentsList.clear();
    annotationList.clear();
    pageXObjectReferences.clear();
    pageXObjectStreams.clear();

    // get object number to use for current page
    currentPageObjectNumber = ++objectNumber;
    // store current page size
    currentPageSize = pageSize;

    // Start a new contents stream
    beginPageStream();

    // start a graphics stack
    // output("q");

    // adjust the origin of the page and the clip mask
    output("q");

    // ensure that the document Graphics State is set right off the bat
    output("/GSDOC gs");

    StringBuffer origin = new StringBuffer("1 0 0 1 ");
    origin.append(pageSize.getX());
    origin.append(" ");
    origin.append(pageSize.getHeight() + pageSize.getY());
    origin.append(" cm");
    output(origin.toString());

    StringBuffer clip = new StringBuffer("0 0 ");
    clip.append(pageSize.getWidth());
    clip.append(" ");
    clip.append(-pageSize.getHeight());
    clip.append(" re W n");
    output(clip.toString());

    // Reset the bounds for this page
    bounds.setRect(pageSize.getX(),
                   pageSize.getY(),
                   pageSize.getWidth() + pageSize.getX(),
                   pageSize.getHeight() + pageSize.getY());
}

/**
 * Complete the page (terminate the contents stream, write out the Page
 * object including all resources used on the page.
 * @param pageSize the dimensions of the page
 */
public void endPage(Rectangle2D pageSize) {

    int count = 0;

    // Terminate the page contents stream
    output("Q");
    endStream();

    // Write the Pattern objects
    Iterator iter = streamInfo.patternList.iterator();
    while (iter.hasNext()) {
        writePatternDescription((PatternInfo)iter.next());
    }




    annotationRectangles.clear();
    hyperlinkInfoList.clear();

    /*
    * Create the font object for any fonts which are used but don't
    * already have descriptions stored.
    */
    createNecessaryFontObjects(streamInfo.fontList);

    // Add the page object to the page and reference list
    storeObjectReference(currentPageObjectNumber, getCurrentOffset());
    pageList.add(new Integer(currentPageObjectNumber));

    // Print out the Page object
    output(currentPageObjectNumber + " 0 obj\n<<\n/Type /Page");

    // Output the Crop Box and reset the printed area bounds
    String box = "/CropBox [ "
        + bounds.getX() + " "
        + bounds.getY() + " "
        + bounds.getWidth() + " "
        + bounds.getHeight() + " ] ";
    output(box);

    /*
    * Revised (4.0.1) Pages tree construction:
    *       The tree is still not "optimal"; it is no longer a completely
    *       flat tree, but a run of similar pages are grouped under a
    *       single Pages object (flat, no further tree structure).
    *
    *       This leads to two notable cases that could be optimised but
    *       currently are not:
    *       1)  A document with only one page type/size will have one Pages
    *           object beneath the root Pages object, and all Page objects
    *           will belong to that Pages object.
    *       2)  A single Page object that differs from the preceding and
    *           following Page objects will have a parent Pages object that
    *           has only one Page object.
    *
    *       There is still no provision for tree balancing or optimisation.
    *       (A truly correct tree would need to be built in advance.)
    *       This could be achieved by assigning Page objects to Pages objects
    *       (subject to restrictions on similar pages, see above) up to a
    *       total of c.10 Page objects.  The Page objects having been assigned
    *       to a Pages object, they could be written out immediately (useful
    *       for the "flush on output" case).  The Pages objects could be
    *       stored until the complete job is finished, and then the tree
    *       could be balanced and optimised and all Pages objects written out.
    *
    */

    output("/Parent " + pagesReference + " 0 R");

    // Resources dictionary for current page
    output("/Resources <<");
    writeResourcesDictionary();
    output(">>");

    // Contents array for the Page
    count = contentsList.size();
    if (count > 0) {
        output("/Contents [");
        for (int i = 0; i < count; i++) {
            int objectNum = ((Integer) contentsList.elementAt(i)).intValue();
            output(objectNum + " 0 R");
        }
        output("]");
    }

    // Annotations array for the Page
    count = annotationList.size();
    if (count > 0) {
        output("/Annots [");
        for (int i = 0; i < count; i++) {
            int objectNum = ((Integer) annotationList.elementAt(i)).intValue();
            output(objectNum + " 0 R");
        }
        output("]");
    }

    // End the page object
    output(">>\nendobj");

    // write out the x object streams that have accumulated for this page
    // shift offsets for these x object stream to match the actual file offset
    for (int i = 0; i < pageXObjectStreams.size(); i++) {
        ByteArrayOutputStream byteOS =
            (ByteArrayOutputStream) pageXObjectStreams.elementAt(i);
        Vector streamReferenceList =
            (Vector) pageXObjectReferences.elementAt(i);

        long currentFileOffset = streamInfo.fileOffset;
        int streamSize = byteOS.size();
        try {
            byteOS.writeTo(os);
            streamInfo.fileOffset += streamSize;

            for (int j = 0; j < streamReferenceList.size(); j++) {
                AcroReference reference =
                    (AcroReference) streamReferenceList.elementAt(j);
                reference.offset += currentFileOffset;
                streamInfo.referenceList.addElement(reference);
            }
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }
    }

    // Force the colour and line width to be set again on the next page
    // dps->old_width = -1.0;
    // dps->old_fill_colour = NULL;
    // dps->old_stroke_colour = NULL;

    super.endPage(pageSize);
}

/**
 * Write a Resources Dictionary based on the current StreamInfo.
 */
protected void writeResourcesDictionary() {

    // Resource dictionary element XObject list
    int count = streamInfo.xObjectList.size();
    if (count > 0) {
        output("/XObject <<");
        for (int i = 0; i < count; i++) {
            int refNum =
                ((Integer) streamInfo.xObjectList.elementAt(i)).intValue();
            int imNum =
                ((Integer) streamInfo.imageList.elementAt(i)).intValue();
            String xObj = "/Im" + imNum + " " + refNum + " " + "0 R";
            output(xObj);
        }
        output(">>");
    }

    // Resource dictionary element ColorSpace list
    count = streamInfo.colorSpaceList.size();
    if (count > 0) {
        output("/ColorSpace <<");
        for (int i = 0; i < count; i++) {
            int refNm =
                ((Integer) streamInfo.colorSpaceList.elementAt(i)).intValue();
            String cs = "/CS" + i + " " + refNm + " " + " 0 R";
            output(cs);
        }
        output(">>");
    }

    // Resource dictionary element Font list
    writeResourcesFontList(streamInfo.fontList);

    // Resource dictionary element Pattern list
    writeResourcesPatternList();

    // Resource dictionary element Graphics State list
    // For the moment, we only have the global document Graphics State
    output("/ExtGState << /GSDOC " + graphicsDictionaryObj + " 0 R >>");

    // Resource dictionary element ProcSet
    StringBuffer dict = new StringBuffer("/ProcSet [ /PDF ");
    if (streamInfo.textPresent) {
        dict.append(" /Text ");
    }
    if (streamInfo.imagePresent) {
        dict.append(" /ImageC ");
    }
    if (streamInfo.maskPresent) {
        dict.append(" /ImageB ");
    }
    if (streamInfo.indexImage) {
        dict.append(" /ImageI ");
    }
    dict.append("]");
    output(dict.toString());

}

/**
 * Given a List of font names, created the necessary font entry
 * objects and store them in the master font list.
 */
protected void createNecessaryFontObjects(Hashtable thisFontList) {
    /*
    * Create the font object for any fonts which are used but don't
    * already have descriptions stored.
    */
    Enumeration e = thisFontList.elements();
    while (e.hasMoreElements()) {
        String fontName = (String) e.nextElement();

        // Look up the font in the font list
        AcroFontEntry fe = (AcroFontEntry) fontList.get(fontName);

        if (fe.usingAnsi && fe.number == -1) {
            // Assign the Font Entry a number, but don't write the
            // font object until the end of the document.  (This
            // lets us utilize knowledge of which characters in the
            // font have been used.)
            fe.number = ++objectNumber;
        }
        if (fe.usingUnicode && fe.unicodeNumber == -1) {
            fe.unicodeNumber = ++objectNumber;
        }
    }
}

/**
 * Write the Font list entry of a Resources dictionary.
 */
protected void writeResourcesFontList(Hashtable thisFontList) {
    if (thisFontList.size() > 0) {
        output("/Font <<");

        // Write the list of font objects used on this page
        Enumeration e2 = thisFontList.keys();
        while (e2.hasMoreElements()) {
            String fontKey = (String) e2.nextElement();
            String fontName = (String) thisFontList.get(fontKey);

            // Look up the font in the font list
            AcroFontEntry fe = (AcroFontEntry) fontList.get(fontName);
            String entry = "";
            if (!fontKey.endsWith("-Unicode") && fe.usingAnsi) {
                entry = "/" + fe.abbrev + " " + fe.number + " " + "0 R";
                output(entry);
            }
            if (fontKey.endsWith("-Unicode") && fe.usingUnicode) {
                entry = "/" + fe.abbrev + "U " + fe.unicodeNumber + " " + "0 R";
                output(entry);
            }
        }
        output(">>");
    }
}

protected void writeResourcesPatternList() {
    if (streamInfo.patternList.size() > 0) {
        output("/Pattern <<");
        // Write the list of pattern objects used on this page
        Iterator iter = streamInfo.patternList.iterator();
        while (iter.hasNext()) {
            PatternInfo pattern = (PatternInfo)iter.next();
            output(pattern.patternName + " " +
                   pattern.patternObjectNumber + " 0 R");
        }
        output(">>");
    }
}

/**
 * Gets the graphics context used by this printer.
 * @return A graphics context that outputs to this printer
 */
public Graphics getGraphics() {
    if (graphics == null) {
        graphics = new Graphics2DPDF(os, this);

        // Mark this graphics as a root frame
        ((Graphics2DBase) graphics).setFrameRoot();
    }
    return (graphics);
}

/**
 * Returns true if this printer supports hyperlinks.
 * @return false - Hyperlinks are not supported.
 */
public boolean supportsHyperlinks() {
    return false;
}




////////// NON interface methods /////////////

/**
 * Returns the number of the XObject associated with the specified image,
 * or -1 if the imageNumber parameter is invalid.
 */
private int getXObjectNumber(int imageNumber) {
    int index = streamInfo.imageList.indexOf(new Integer(imageNumber));
    if (index < 0 || index >= streamInfo.xObjectList.size()) {
        return -1;
    }
    return ((Integer)streamInfo.xObjectList.elementAt(index)).intValue();
}

protected void writePatternDescription(PatternInfo patternInfo) {
    BufferedImage img = patternInfo.img;
    int right = img.getWidth();
    int top = img.getHeight();
    double yScale = 11.;  // empirically determined scaling factor
    double xScale = yScale * (double)right / (double)top;
    Integer objNum = (Integer)imageCache.get(img);
    if (objNum != null) {
        int imageObjectNumber = objNum.intValue();
        if (imageObjectNumber >= 0) {
            String imageName = "/Im" + imageObjectNumber;
            int xObjRefNum = getXObjectNumber(imageObjectNumber);
            if (xObjRefNum >= 0) {
                String extraEntries =
                        "/Type /Pattern\n" +
                        "/PatternType 1\n" +
                        "/Name " + patternInfo.patternName + "\n" +
                        "/PaintType 1\n" +
                        "/TilingType 1\n" +
                        "/Matrix [ " + xScale + " 0 0 " +
                                   yScale + " 0 0 ]\n" +
                        "/BBox [ 0 0 " + right + " " + top + " ]\n" +
                        "/Resources <<\n" +
                        "/ProcSet [ /PDF /Text /ImageC ]\n" +
                        "/XObject << " + imageName + " " +
                                     xObjRefNum + " 0 R >>\n" +
                        ">>\n" +
                        "/XStep 1\n" +
                        "/YStep 1";

                String streamCommands = "q " + imageName + " Do Q";
                //  Save graphics state, paint image, restore graphics state.
                patternInfo.patternObjectNumber = ++objectNumber;
                if (compressed) {
                    writeCompressedStreamObject(
                            patternInfo.patternObjectNumber,
                            streamCommands.getBytes(), extraEntries);
                }
                else {
                    writeUncompressedStreamObject(
                            patternInfo.patternObjectNumber,
                            streamCommands, extraEntries);
                }
            }
        }
    }
}

/**
 * Writes the font description to the output.
 * @param fontEntry A FontEntry object describing a font used in the document
 */
protected void writeFontDescription(AcroFontEntry fontEntry) {
    int first_char = 256, last_char = -1;
    int charwidths[] = new int[256];
    int ansiFontDescriptorObjectNumber = -1;
    int unicodeFontDescriptorObjectNumber = -1;
    CharMetricPS charData = null;

    // Make sure font has an associated number
    if (fontEntry.usingAnsi && fontEntry.number == -1) {
        fontEntry.number = ++objectNumber;
    }
    if (fontEntry.usingUnicode && fontEntry.unicodeNumber == -1) {
        fontEntry.unicodeNumber = ++objectNumber;
    }

    // embedded fonts have modified name, preceded by a random string of
    // characters so that there is no confusion with fonts stored on
    // the user's system
    String fontNameInDescriptor = "";
    if (fontEntry.embeddingRules != TrueTypeFontProperties.DO_NOT_EMBED) {
        Random random = new Random();
        fontNameInDescriptor = "";
        for (int i = 0; i < 6; i++) {
            fontNameInDescriptor +=
                (char) (random.nextInt(26) + 65);
        }
        fontNameInDescriptor += '+';
    }
    // make sure spaces in name are removed for TTF
    if (fontEntry.usingTTF) {
        StringBuffer newFontName = new StringBuffer();
        char[] fontNameChars = fontEntry.name.toCharArray();
        for (int i = 0; i < fontNameChars.length; i++) {
            if (fontNameChars[i] != ' ') {
                newFontName.append(fontNameChars[i]);
            }
        }
        fontNameInDescriptor += newFontName.toString();
        // make sure spaces in name are replaced with underscores for Type 1 fonts
    }
    else {
        fontNameInDescriptor += fontEntry.name.replace(' ', '_');
    }

    // Get the font info from the font object
    // instance = (fontEntry.font.instanceList.elementAt(0);
    // info = klgpf_fontPS_getFontInfo(instance);
    FontPS font = fontEntry.font;

    // normal, non-Unicode, good old ansi character sets
    if (fontEntry.usingAnsi) {

        // Store reference of this font object
        storeObjectReference(fontEntry.number, getCurrentOffset());
        // Begin writing the font object
        output(fontEntry.number + " 0 obj\n<<");

        if (fontEntry.usingTTF) {
            output("/Type /Font\n/Subtype /TrueType");
        }
        else {
            output("/Type /Font\n/Subtype /Type1");
        }

        String name = "/Name /" + fontEntry.abbrev
            + "\n/BaseFont /" +
            fontNameInDescriptor;
        //fontEntry.name;
        output(name);

        if (!fontEntry.usingTTF) {
            output("/Encoding " + encodingObj + " 0 R");
        }
        else {
            // use straight WinAnsi Encoding for TrueType fonts
            // since any non-ansi characters can be specified through unicode
            output("/Encoding /WinAnsiEncoding");
        }

        if (!fontEntry.usingTTF) {
            /*
            * May not need the rest of the font definition/description;
            * check font name against list of 14 base fonts
            */
            if (isBaseFont(fontEntry.name)) {
                // End the object and exit
                output(">>\nendobj");
                return;
            }
        }

        ansiFontDescriptorObjectNumber = ++objectNumber;

        // Write a reference to a font descriptor to be created next
        output("/FontDescriptor " + ansiFontDescriptorObjectNumber + " 0 R");

        // Initialise the table of character widths with .notdef (space) width
        charData = font.getCharMetric(' ');
        int spaceWx = charData.getWx();
        for (int i = 0; i < 256; i++) {
            charwidths[i] = spaceWx;
        }

        // monospaced font -- only write out first character in width
        // array and use MissingWidth property of FontDescriptor to handle
        // the rest
        // (it'd be great if we didn't have to write out the width array
        // at all, but PDF requires it)
        if (font.isFixedPitch()) {
            first_char =
                ((Integer) fontEntry.ansiCharactersUsed.first()).intValue();
            last_char = first_char;
            charData = font.getCharByCode(first_char);
            charwidths[first_char] = charData.getWx();

            // variable width font -- populate the width array with widths of
            // the characters used from this font in this document
        }
        else {
            // Find first and last character
            first_char =
                ((Integer) fontEntry.ansiCharactersUsed.first()).intValue();
            last_char =
                ((Integer) fontEntry.ansiCharactersUsed.last()).intValue();
            if (last_char > 255)
                last_char = 255;

            // Fill the character width array with values of the characters
            // used in this document
            Iterator it = fontEntry.ansiCharactersUsed.iterator();
            while (it.hasNext()) {
                int charCode = ((Integer) it.next()).intValue();
                if (charCode >= 0 && charCode <= 255) {
                    // special case -- if WinAnsiEncoding character code
                    // for Euro found in TrueType's ansi section, make sure
                    // to get its width using the actual unicode value
                    if (charCode == 128 && fontEntry.usingTTF) {
                        charData = font.getCharByCode(0x20ac);
                        // standard case -- retrieve character width
                    }
                    else {
                        charData = font.getCharByCode(charCode);
                    }
                    charwidths[charCode] = charData.getWx();
                }
            }
        }

        // Output the font metrics from the font object
        output("/FirstChar " + first_char);
        output("/LastChar " + last_char);

        output("/Widths " + (objectNumber + 1) + " 0 R");
        output(">>\nendobj");

        // Output the font character widths array
        storeObjectReference(++objectNumber, getCurrentOffset());
        output(objectNumber + " 0 obj");

        output("[");
        int count = 0;
        StringBuffer sb = new StringBuffer();
        for (int i = first_char; i <= last_char; i++) {
            sb.append(charwidths[i]);

            if (++count % 16 == 0) {
                output(sb.toString());
                sb.delete(0, sb.length());
            }
            else {
                sb.append(" ");
            }
        }
        output(sb.toString());
        output("]");

        output("endobj");
    }

    // Unicode character sets
    if (fontEntry.usingTTF && fontEntry.usingUnicode) {
        // Store reference of this font object
        storeObjectReference(fontEntry.unicodeNumber, getCurrentOffset());
        // Begin writing the font object
        output(fontEntry.unicodeNumber + " 0 obj\n<<");

        output("/Type /Font\n/Subtype /Type0");
        output("/Name /" + fontEntry.abbrev + "U");
        output("/BaseFont /" + fontNameInDescriptor);
        output("/Encoding /Identity-H");

        int descendantFontObjectNumber = ++objectNumber;
        output("/DescendantFonts [ " + descendantFontObjectNumber + " 0 R ]");

        int toUnicodeObjectNumber = -1;
        if (fontEntry.includeUnicodeMap) {
            toUnicodeObjectNumber = ++objectNumber;
            output("/ToUnicode " + toUnicodeObjectNumber + " 0 R");
        }

        // ToDo: ToUnicode Field
        output(">>\nendobj");

        storeObjectReference(descendantFontObjectNumber, getCurrentOffset());
        output(descendantFontObjectNumber + " 0 obj\n<<");

        output("/Type /Font\n/Subtype /CIDFontType2");
        output("/BaseFont /" + fontNameInDescriptor);

        // Write a reference to a font descriptor to be created next
        int cidSysInfoObjectNumber = ++objectNumber;
        unicodeFontDescriptorObjectNumber = ++objectNumber;
        output("/CIDSystemInfo " + cidSysInfoObjectNumber + " 0 R");
        output("/FontDescriptor " + unicodeFontDescriptorObjectNumber + " 0 R");

        // monospaced font -- use width of first character (since it is the
        // same as the width of all other characters) and write it out as
        // the default width -- no need for actual width array
        if (font.isFixedPitch()) {
            int oneChar =
                ((Integer) fontEntry.unicodeCharactersUsed.first()).intValue();
            output("/DW " +
                   font.ttfParser.getGlyphWidthByGlyphCode(oneChar));

            // variable width font -- write out array of widths of glyphs
            // used in document
        }
        else {

            // Even in a variable width font, there are sometimes many
            // widths that are the same.  If hashtable is considerably
            // smaller than the character list, that means there are a
            // number of repeated widths, so check stored hashtable of
            // widths for most used width and use that as a default width.
            int mostUsedWidth = -1;
            if (fontEntry.unicodeWidths.size() <=
                (fontEntry.unicodeCharactersUsed.size() / 2)) {
                int numTimesUsed = -1;
                Enumeration widthKeys = fontEntry.unicodeWidths.keys();
                while (widthKeys.hasMoreElements()) {
                    Integer uniWidth = (Integer) widthKeys.nextElement();
                    Integer widthCount = (Integer)
                        fontEntry.unicodeWidths.get(uniWidth);
                    if (widthCount.intValue() > numTimesUsed) {
                        mostUsedWidth = uniWidth.intValue();
                        numTimesUsed = widthCount.intValue();
                    }
                }
            }

            // use most used width as default width
            if (mostUsedWidth >= 0)
                output("/DW " + mostUsedWidth);

            // begin output widths of glyphs used in this document
            output("/W [");

            // get list of glyphs used in this document
            Iterator it = fontEntry.unicodeCharactersUsed.iterator();

            Integer startChar = null;
            Integer thisChar = null;
            Integer endChar = null;
            Integer startWidthChar = null;
            int thisWidth = -1;
            int startWidth = -1;
            boolean flushMetrics = false;

            // loop through list of glyphs, collecting ranges that
            // are continguous.  Within these continguous ranges, find
            // sub-ranges where the width is the same for each glyph.

            // while there's still characters in the list or to be written out
            while (it.hasNext() || startChar != null) {

                // still characters left in the list
                if (it.hasNext()) {
                    thisChar = (Integer) it.next();
                    thisWidth =
                        font.ttfParser.getGlyphWidthByGlyphCode(
                            thisChar.intValue());

                    // if width is the same as the most used width, don't
                    // bother with the character since it will be covered
                    // by the default width setting
                    if (mostUsedWidth >= 0 && thisWidth == mostUsedWidth) {
                        if (startChar == null) {
                            thisWidth = -1;
                            continue;
                        }
                        else {
                            thisChar = null;
                            flushMetrics = true;
                        }
                    }
                    else {

                        // start new range with the current character
                        if (startChar == null) {
                            startChar = thisChar;
                            endChar = thisChar;
                            startWidthChar = thisChar;
                            startWidth = thisWidth;

                            // extend current range if the current character is one
                            // greater than the end of the current range
                        }
                        else if (thisChar.intValue() ==
                            endChar.intValue() + 1) {
                            // width is still the same -- extend the current
                            // range
                            if (thisWidth == startWidth) {
                                endChar = thisChar;
                            }
                            else {
                                // width is different -- if there's a
                                // contiguous range with the same width for
                                // at least 3 characters write it out
                                if (endChar.intValue()
                                    - startWidthChar.intValue() >= 2) {
                                    flushMetrics = true;

                                    // width is different -- it was only the
                                    // same for a small number of glyphs (1 or 2)
                                    // so just put these glyphs in the different
                                    // width range -- also start a new range for
                                    // same widths
                                }
                                else {
                                    endChar = thisChar;
                                    startWidthChar = thisChar;
                                    startWidth = thisWidth;
                                }
                            }

                            // current character is too far outside current range,
                            // so write out the metrics of the range we have
                        }
                        else {
                            flushMetrics = true;
                        }

                    }

                    // no characters left in list, so just write out the metrics
                    // of the current range
                }
                else {
                    thisChar = null;
                    flushMetrics = true;
                }

                // write out metrics of current range
                if (flushMetrics) {
                    // if we ended on a range containing a suitable number
                    // of same widths, write them out
                    if (endChar.intValue()
                        - startWidthChar.intValue() >= 2) {

                        // but first, check if there are any different widths
                        // at the start of the range and write those out
                        if (startChar.intValue() < startWidthChar.intValue()) {
                            writeCharRangeDiffWidth(font,
                                                    startChar.intValue(),
                                                    startWidthChar.intValue() - 1);
                        }
                        writeCharRangeSameWidth(font,
                                                startWidthChar.intValue(),
                                                endChar.intValue());

                        // range contains different widths, write it out
                    }
                    else {
                        writeCharRangeDiffWidth(font,
                                                startChar.intValue(),
                                                endChar.intValue());
                    }
                    flushMetrics = false;

                    // begin new range with current character
                    startChar = thisChar;
                    startWidthChar = thisChar;
                    startWidth = thisWidth;
                    endChar = startChar;
                }
            }
            output("]");
        }
        output(">>\nendobj");

        // Output the CID System Info
        storeObjectReference(cidSysInfoObjectNumber, getCurrentOffset());
        output(cidSysInfoObjectNumber + " 0 obj\n<<");

        output("/Type /CIDSystemInfo");
        output("/Registry (Adobe)");

        output("/Ordering (Identity)");
        output("/Supplement 0");

        output(">>\nendobj");

        // Output the ToUnicode CMap if one has been requested
        if (fontEntry.includeUnicodeMap) {
            writeToUnicodeCMap(toUnicodeObjectNumber, fontEntry, font);
        }
    }

    // Output the font description dictionary for ansi font
    if (ansiFontDescriptorObjectNumber >= 0) {
        writeFontDescriptor(ansiFontDescriptorObjectNumber, fontEntry, font,
                            fontNameInDescriptor);
    }

    // Output the font description dictionary for unicode font
    if (unicodeFontDescriptorObjectNumber >= 0) {
        writeFontDescriptor(unicodeFontDescriptorObjectNumber, fontEntry, font,
                            fontNameInDescriptor);
    }

    // Now, embed font if requested
    if (fontEntry.embeddingRules !=
        TrueTypeFontProperties.DO_NOT_EMBED) {

        ByteArrayOutputStream fontByteOS = new ByteArrayOutputStream();

        if (fontEntry.embeddingRules ==
            TrueTypeFontProperties.EMBED_ENTIRE_FONT) {

            font.ttfParser.writeFont(fontByteOS);

        }
        else if (fontEntry.embeddingRules ==
            TrueTypeFontProperties.EMBED_NEEDED) {

            font.ttfParser.writeReducedFont(fontByteOS,
                                            fontEntry.usingAnsi ?
                                            fontEntry.ansiCharactersUsed : null,
                                            fontEntry.usingUnicode ?
                                            fontEntry.unicodeCharactersUsed : null);
        }

        int embedObjectNumber = ++objectNumber;
        writeCompressedStreamObject(embedObjectNumber, fontByteOS,
                                    "/Length1 " + fontByteOS.size());
    }
}

/**
 * Write the actual PDF Font Descriptor to the output.
 * @param fontDescriptorObjectNumber object number of this font descriptor
 * @param fontEntry                  AcroFontEntry object encapsulating
 *                                   font information for PDF output
 * @param font                       FontPS class representing actual font
 * @param fontNameInDescriptor       name to use for the font in the
 *                                   descriptor.  If font is embedded, this
 *                                   will be different than the usual name
 *                                   of the font to preserve uniqueness.
 */
protected void writeFontDescriptor(int fontDescriptorObjectNumber,
                                   AcroFontEntry fontEntry, FontPS font, String fontNameInDescriptor) {

    CharMetricPS charData = null;
    int bitmask = 0;

    storeObjectReference(fontDescriptorObjectNumber, getCurrentOffset());
    output(fontDescriptorObjectNumber + " 0 obj\n<<");

    output("/Type /FontDescriptor");
    output("/Ascent " + font.getAscender());
    output("/CapHeight " + font.getCapHeight());
    output("/Descent " + font.getDescender());

    // ansi monospaced font -- write out width for all characters (use
    // width of first character, since all characters are the same width)
    if (fontEntry.usingAnsi && font.isFixedPitch()) {
        int oneChar =
            ((Integer) fontEntry.ansiCharactersUsed.first()).intValue();
        charData = font.getCharByCode(oneChar);
        output("/MissingWidth " + charData.getWx());
    }

    // Try to determine font flags
    // Fixed width vs. proportional font
    if (font.isFixedPitch()) {
        bitmask |= 0x01;
    }
    // Serif font -- can only get this info from TTF font
    if (font.isSerif()) {
        bitmask |= 0x02;
    }
    // Symbolic fonts -- if one of the known symbol fonts or not using
    // Adobe standard character set
    if (font.getName().equals("Symbol") ||
        font.getName().equals("ZapfDingbats") ||
        !(font.getEncodingScheme().equals("AdobeStandardEncoding") ||
        font.getEncodingScheme().equals("Unicode"))) {
        bitmask |= 0x04;
    }
    else {
        // All the other PS and TTF fonts we support use standard encodings
        bitmask |= 0x20;
    }
    // Script font -- Unsure how to find this information
    if (false) {
        bitmask |= 0x08;
    }
    // Italic
    if (Math.abs(font.getItalicAngle()) > 4) {
        bitmask |= 0x40;
    }
    // All-cap font -- Unsure how to find this information
    if (false) {
        bitmask |= 0x10000;
    }
    // Small-cap font -- Unsure how to find this information
    if (false) {
        bitmask |= 0x20000;
    }
    // Bold small text sizes -- Unsure how to find this information
    if (true) {
        bitmask |= 0x40000;
    }

    output("/Flags " + bitmask);

    String fontBox = "/FontBBox [ "
        + font.getFontBBox().llx + " "
        + font.getFontBBox().lly + " "
        + font.getFontBBox().urx + " "
        + font.getFontBBox().ury + " ]";
    output(fontBox);

    output("/FontName /" + fontNameInDescriptor);
    output("/ItalicAngle " + (int) font.getItalicAngle());


    double width = (double) font.getStdVW();
    // if stem width not specifed, try to calculate it
    // (Stem width is used by the PDF Viewer to approximate the
    // requested font when said font cannot be found.  Hence, it's
    // important to have some value for it.)
    if (width <= 0.0) {
        //
        // Compute a vertical stem width - use an approximate value based
        // of the width of the exclamation mark.
        // (Perhaps a couple of units smaller than exclam width?)
        //
        charData = font.getCharMetric('!');

        width = charData.getCharBBox().urx - charData.getCharBBox().llx;
        if (font.getItalicAngle() != 0) {
            width -= (charData.getCharBBox().ury -
                charData.getCharBBox().lly) *
                Math.tan(font.getItalicAngle());
        }
        width -= 4;
    }
    output("/StemV " + (int) width);

    // if embedding TrueType font, need to have a pointer to it here
    if (fontEntry.embeddingRules !=
        TrueTypeFontProperties.DO_NOT_EMBED) {
        output("/FontFile2 " + (objectNumber + 1) + " 0 R");
    }

    output(">>\nendobj");
}

/**
 * Write a ToUnicode CMap as a stream object.
 */
protected void writeToUnicodeCMap(int toUnicodeObjectNumber,
                                  AcroFontEntry fontEntry, FontPS font) {
    // write out the ToUnicode CMap to a StringBuffer for temporary storage
    StringBuffer toUnicodeCMap = new StringBuffer();
    toUnicodeCMap.append("/CIDInit /ProcSet findresource begin\n");
    toUnicodeCMap.append("12 dict begin\n");
    toUnicodeCMap.append("begincmap\n");
    toUnicodeCMap.append("/CIDSystemInfo\n");
    toUnicodeCMap.append("<< /Registry (Adobe)\n");
    toUnicodeCMap.append("/Ordering (UCS)\n");
    toUnicodeCMap.append("/Supplement 0\n");
    toUnicodeCMap.append(">> def\n");
    toUnicodeCMap.append("/CMapName /Adobe-Identity-UCS def\n");
    toUnicodeCMap.append("/CMapType 2 def\n");

    // code space range
    toUnicodeCMap.append("1 begincodespacerange\n");
    toUnicodeCMap.append("<0001> <FFFF>\n");
    toUnicodeCMap.append("endcodespacerange\n");

    Vector bfRangeLines = new Vector();  // lines for 'bfrange' command
    Vector bfCharLines = new Vector();   // lines for 'bfchar' command

    // go through list of glyphs, collecting ranges if any exist,
    // to ensure smallest possible CMap
    SeqRangeCollector glyphRangeCollector = new SeqRangeCollector();
    // for some reason I can't quite fathom, the CMap table won't
    // work if a glyph range crosses the boundary from 13ff to 1400
    // -- presumably true for all ff-00 boundaries, so make sure to
    // break the range when these are encountered
    glyphRangeCollector.setRangeBreak(
        SeqRangeCollector.RANGE_BREAK_HEX_BOUNDARY);
    Iterator unicodeIter = fontEntry.unicodeCharactersUsed.iterator();
    while (unicodeIter.hasNext()) {
        int glyphCode = ((Integer) unicodeIter.next()).intValue();
        glyphRangeCollector.addItem(glyphCode);
    }
    glyphRangeCollector.done();

    List glyphRanges = glyphRangeCollector.getRanges();
    for (int i = 0; i < glyphRanges.size(); i++) {
        SeqRangeCollector.Range glyphRange =
            (SeqRangeCollector.Range) glyphRanges.get(i);
        // handle sequential ranges of at least 3 glyphs in a row
        if (glyphRange.end - glyphRange.start > 1) {
            // run through the range to see if the corresponding unicode
            // values are also sequential
            SeqRangeCollector unicodeRangeCollector = new SeqRangeCollector();
            for (int j = glyphRange.start; j <= glyphRange.end; j++) {
                List unicodes = font.ttfParser.glyphCodeToUnicodes(j);
                if (unicodes == null || unicodes.size() == 0) {
                    continue;
                }
                int currentUnicode = ((Integer) unicodes.get(0)).intValue();
                unicodeRangeCollector.addItem(currentUnicode);
            }
            unicodeRangeCollector.done();

            // match sequential ranges of unicode values to the
            // sequential range of glyph codes that spawned them
            List unicodeRanges = unicodeRangeCollector.getRanges();
            int glyphRangeCount = 0;
            for (int j = 0; j < unicodeRanges.size(); j++) {
                SeqRangeCollector.Range unicodeRange =
                    (SeqRangeCollector.Range) unicodeRanges.get(j);
                // at least 3 sequential unicode values in a range
                // use sequential form of line
                if (unicodeRange.end - unicodeRange.start > 1) {
                    bfRangeLines.add("<" +
                                     formatHexString(
                                         glyphRange.start + glyphRangeCount, 4) + "> <" +
                                     formatHexString(
                                         glyphRange.start + glyphRangeCount +
                                         unicodeRange.size - 1, 4) + "> <" +
                                     formatHexString(unicodeRange.start, 4) + ">\n");
                    glyphRangeCount += unicodeRange.size;
                    // less than 3 sequential unicode values in a range
                    // use array format of line
                }
                else {
                    StringBuffer line = new StringBuffer();
                    line.append("<");
                    line.append(
                        formatHexString(glyphRange.start + glyphRangeCount, 4));
                    line.append("> ");
                    StringBuffer subline = new StringBuffer();

                    // Keep moving through list of unicode ranges
                    // collating short ranges into array line.
                    // When we find one that is at least 3 characters long,
                    // we finish up here let the loop take us back to
                    // where such long ranges are handled.
                    int unicodeCount = 0;
                    SeqRangeCollector.Range subUnicodeRange = unicodeRange;
                    while (j < unicodeRanges.size() &&
                        subUnicodeRange.end - subUnicodeRange.start <= 1) {
                        for (int k = subUnicodeRange.start;
                             k <= subUnicodeRange.end; k++) {
                            subline.append("<");
                            subline.append(
                                formatHexString(k, 4));
                            subline.append(">");
                            subline.append(" ");
                            unicodeCount++;
                        }
                        j++;
                        if (j < unicodeRanges.size()) {
                            subUnicodeRange =
                                (SeqRangeCollector.Range) unicodeRanges.get(j);
                        }
                    }
                    j--;

                    // build array-style line
                    line.append("<");
                    line.append(
                        formatHexString(
                            glyphRange.start + glyphRangeCount +
                            unicodeCount - 1, 4));
                    glyphRangeCount += unicodeCount;
                    line.append("> ");
                    line.append("[ ");
                    // break up array line over multiple lines if it is
                    // too long -- note this algorith depends on array line
                    // containing 6-character entities followed by a space
                    while (subline.length() > 49) {
                        line.append(subline.substring(0, 49));
                        line.append("\n");
                        line.append("                ");
                        subline = new StringBuffer(subline.substring(49));
                    }
                    line.append(subline.toString());
                    line.append("]\n");
                    bfRangeLines.add(line.toString());
                }
            }
            // ranges of only 1 or 2 glyphs -- consider them standalone glyphs,
            // which are specified with the 'bfchar' command
        }
        else {
            for (int j = glyphRange.start; j <= glyphRange.end; j++) {
                List unicodes = font.ttfParser.glyphCodeToUnicodes(j);
                if (unicodes == null || unicodes.size() == 0) {
                    continue;
                }
                bfCharLines.add("<" + formatHexString(j, 4) + "> <" +
                                formatHexString(
                                    ((Integer) unicodes.get(0)).intValue(), 4) +
                                ">\n");
            }
        }
    }

    // write the range character mappings (the 'bfrange' command)
    // can only write a hundred items per bunch, says the spec
    int hundreds = 0;
    while (hundreds * 100 < bfRangeLines.size()) {
        int numberToWriteNow = Math.min(
            bfRangeLines.size() - (hundreds * 100), 100);
        toUnicodeCMap.append("" + numberToWriteNow + " beginbfrange\n");
        for (int i = 0; i < numberToWriteNow; i++) {
            toUnicodeCMap.append((String) bfRangeLines.get(
                (hundreds * 100) + i));
        }
        toUnicodeCMap.append("endbfrange\n");

        hundreds++;
    }

    // write the individual character mappings (the 'bfchar' command)
    // can only write a hundred items per bunch, says the spec
    hundreds = 0;
    while (hundreds * 100 < bfCharLines.size()) {
        int numberToWriteNow = Math.min(
            bfCharLines.size() - (hundreds * 100), 100);
        toUnicodeCMap.append("" + numberToWriteNow + " beginbfchar\n");
        for (int i = 0; i < numberToWriteNow; i++) {
            toUnicodeCMap.append((String) bfCharLines.get(
                (hundreds * 100) + i));
        }
        toUnicodeCMap.append("endbfchar\n");

        hundreds++;
    }

    toUnicodeCMap.append("endcmap\n");
    toUnicodeCMap.append("CMapName currentdict /CMap defineresource pop\n");
    toUnicodeCMap.append("end\n");
    toUnicodeCMap.append("end");

    if (compressed) {
        byte[] cmap_bytes = null;
        // bytes must be retrieved as ascii
        try {
            cmap_bytes = toUnicodeCMap.toString().getBytes("US-ASCII");
        } catch (UnsupportedEncodingException uee) {
            cmap_bytes = toUnicodeCMap.toString().getBytes();
        }
        writeCompressedStreamObject(toUnicodeObjectNumber, cmap_bytes, null);
    }
    else {
        writeUncompressedStreamObject(toUnicodeObjectNumber,
                                      toUnicodeCMap.toString(), null);
    }
}

/**
 * Format an integer as a hex string, padding the beginning of the
 * string with zeroes until it is the requested number of characters.
 */
public String formatHexString(int value, int numChars) {
    StringBuffer sb = new StringBuffer();
    sb.append(Integer.toHexString(value));
    while (sb.length() < numChars) {
        sb.insert(0, '0');
    }
    return sb.toString();
}

/**
 * Write a compressed stream object to the output.  Compression is
 * done using Flate, and the resulting stream is then put through
 * the Ascii85 Encoder.
 * @param streamObjectNumber number reference of the stream object
 * @param byteArrayOS        contents of the stream as a ByteArrayOutputStream
 * @param extraEntries       extra entries (other than Filter and Length)
 *                           for the descriptor that precedes the stream
 */
protected void writeCompressedStreamObject(int streamObjectNumber,
                                           ByteArrayOutputStream byteArrayOS, String extraEntries) {
    writeCompressedStreamObject(streamObjectNumber,
                                byteArrayOS.toByteArray(), extraEntries);
}

/**
 * Write a compressed stream object to the output.  Compression is
 * done using Flate, and the resulting stream is then put through
 * the Ascii85 Encoder.
 * @param streamObjectNumber number reference of the stream object
 * @param byteArray          contents of the stream as an array of bytes
 * @param extraEntries       extra entries (other than Filter and Length)
 *                           for the descriptor that precedes the stream
 */
protected void writeCompressedStreamObject(int streamObjectNumber,
                                           byte[] byteArray, String extraEntries) {

    int lengthObjectNumber = ++objectNumber;
    writeCompressedStreamHeader(streamObjectNumber,
                                lengthObjectNumber, extraEntries);
    int streamLength = writeCompressedStreamBody(byteArray);
    writeCompressedStreamFooter(lengthObjectNumber, streamLength);
}

/**
 * Write the beginning of a compressed stream object to the output, including
 * the filter specifications and the stream command.  Compression will be
 * done using Flate, and the resulting stream will also put through the
 * Ascii85 Encoder.
 * @param streamObjectNumber       number reference of the stream object
 * @param streamLengthObjectNumber number reference of the stream length object
 * @param extraEntries       extra entries (other than Filter and Length)
 *                           for the descriptor that precedes the stream
 */
protected void writeCompressedStreamHeader(int streamObjectNumber,
                                           int streamLengthObjectNumber, String extraEntries) {

    storeObjectReference(streamObjectNumber, getCurrentOffset());
    output(streamObjectNumber + " 0 obj\n<<");

    output("/Filter [ /ASCII85Decode /FlateDecode ]");

    output("/Length " + (streamLengthObjectNumber) + " 0 R");
    if (extraEntries != null)
        output(extraEntries);
    output(">>");
    output("stream");
}

/**
 * Write the body (i.e. the actual stream portion) of a compressed stream
 * object to the output.  Compression is done using Flate, and the
 * resulting stream is then put through the Ascii85 Encoder.
 * @param byteArray array of bytes to write to stream
 * @return length of stream written to output
 */
protected int writeCompressedStreamBody(byte[] byteArray) {

    ByteArrayOutputStream byteOS = new ByteArrayOutputStream();
    DeflaterOutputStream compressor = new DeflaterOutputStream(
        new Ascii85OutputStream(byteOS));
    try {
        compressor.write(byteArray);
        ;
    } catch (IOException ioe) {
        ioe.printStackTrace();
    }

    try {
        compressor.finish();
        compressor.flush();
    } catch (IOException ioe) {
        ioe.printStackTrace();
    }

    // Start the image data stream
    long streamStart = getCurrentOffset();

    byte[] filteredBytes = byteOS.toByteArray();
    int bufferSize = filteredBytes.length;
    int offset = 0;

    // output stream in 72-byte chunks
    while (offset + 72 < bufferSize) {
        output(filteredBytes, offset, 72);
        offset += 72;

        // force a carriage return
        output((byte) '\n');
    }
    // output rest of buffer
    output(filteredBytes, offset, bufferSize - offset);
    // force a carriage return
    output((byte) '\n');

    return (int) (getCurrentOffset() - streamStart - 1);
}

/**
 * Write the tail end of a compressed stream object to the output, including
 * the endstream/endjob commands and an object containing the stream length.
 * @param streamLengthObjectNumber number reference of the stream length object
 * @param streamLength             length of the preceding stream
 */
protected void writeCompressedStreamFooter(int streamLengthObjectNumber,
                                           int streamLength) {
    // End the stream and object
    output("endstream\nendobj");

    // Store the length object for the image data stream
    storeObjectReference(streamLengthObjectNumber, getCurrentOffset());
    output(streamLengthObjectNumber + " 0 obj\n" + streamLength + " \nendobj");
}


/**
 * Write uncompressed (text) stream object to the output.
 * @param streamObjectNumber number reference of the stream object
 * @param contents          contents of the stream
 * @param extraEntries       extra entries (other than Filter and Length)
 *                           for the descriptor that precedes the stream
 */
protected void writeUncompressedStreamObject(int streamObjectNumber,
                                             String contents, String extraEntries) {

    int lengthObjectNumber = ++objectNumber;
    writeUncompressedStreamHeader(streamObjectNumber, lengthObjectNumber,
                                  extraEntries);
    // Start the image data stream
    long streamStart = getCurrentOffset();

    output(contents);

    // End the stream and object
    int streamLength = (int) (getCurrentOffset() - streamStart - 1);
    writeUncompressedStreamFooter(lengthObjectNumber, streamLength);
}

/**
 * Write the beginning of an uncompressed stream object to the output, including
 * the stream command.
 * @param streamObjectNumber       number reference of the stream object
 * @param streamLengthObjectNumber number reference of the stream length object
 * @param extraEntries       extra entries (other than Filter and Length)
 *                           for the descriptor that precedes the stream
 */
protected void writeUncompressedStreamHeader(int streamObjectNumber,
                                             int streamLengthObjectNumber, String extraEntries) {

    storeObjectReference(streamObjectNumber, getCurrentOffset());
    output(streamObjectNumber + " 0 obj\n<<");

    output("/Length " + (streamLengthObjectNumber) + " 0 R");
    if (extraEntries != null)
        output(extraEntries);
    output(">>");
    output("stream");
}

/**
 * Write the tail end of an uncompressed stream object to the output, including
 * the endstream/endjob commands and an object containing the stream length.
 * @param streamLengthObjectNumber number reference of the stream length object
 * @param streamLength             length of the preceding stream
 */
protected void writeUncompressedStreamFooter(int streamLengthObjectNumber,
                                             int streamLength) {
    // End the stream and object
    output("endstream\nendobj");

    // Store the length object for the image data stream
    storeObjectReference(streamLengthObjectNumber, getCurrentOffset());
    output(streamLengthObjectNumber + " 0 obj\n" + streamLength + " \nendobj");
}


/**
 * Write out character widths for a CID font descriptor in the
 * format "startchar endchar width".  All characters in the range
 * are expected to have the same width.
 */
protected void writeCharRangeSameWidth(FontPS font, int start, int end) {
    StringBuffer sb = new StringBuffer();

    sb.append("    " + start);
    sb.append(" " + end);

    int width = font.ttfParser.getGlyphWidthByGlyphCode(start);
    sb.append(" " + width);

    output(sb.toString());
}

/**
 * Write out character widths for a CID font descriptor in the
 * format "startchar [width-1 width-2 ... width-n]".
 */
protected void writeCharRangeDiffWidth(FontPS font, int start, int end) {
    StringBuffer sb = new StringBuffer();

    sb.append("    " + start + " [");
    int bracketPos = sb.toString().indexOf('[');
    int count = 0;

    for (int i = start; i <= end; i++) {

        sb.append(" " +
                  font.ttfParser.getGlyphWidthByGlyphCode(i));

        // break line before it gets too long
        // but don't break if we're at the last value
        if (++count % 12 == 0 && i != end) {
            output(sb.toString());
            sb.delete(0, sb.length());
            for (int j = 0; j <= bracketPos; j++) {
                sb.append(' ');
            }
        }
    }
    sb.append(" ]");

    output(sb.toString());
}

/**
 * Writes the encoding vector of the standard (ISO Latin1) character set.
 */
protected void writeFontEncoding() {

    // Open the font encoding object
    storeObjectReference(++objectNumber, getCurrentOffset());
    output(objectNumber + " 0 obj\n<<");

    output("/Type /Encoding \n/BaseEncoding /WinAnsiEncoding");
    output("/Differences [");
    output("39 /quoteright 45 /minus 96 /quoteleft");
    output("127 /.notdef");
    output("129 /.notdef /.notdef /.notdef /.notdef");
    output("   /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef");
    output("   /.notdef /.notdef /.notdef /.notdef /.notdef");
    output("144 /dotlessi /grave /acute /circumflex /tilde /macron /breve");
    output("   /dotaccent /dieresis");
    output("153 /.notdef");
    output("154 /ring /cedilla");
    output("156 /.notdef");
    output("157 /hungarumlaut /ogonek /caron ]");

    output(">>\nendobj");
}

/**
 * Writes the encoding vector of the standard (ISO Latin1) character set.
 */
protected void writeDocGraphicsDictionary() {

    // Open the font encoding object
    storeObjectReference(++objectNumber, getCurrentOffset());
    output(objectNumber + " 0 obj\n<<");

    output("/Type /ExtGState");
    if (useAutoStrokeAdjustment) {
        output("/SA true");
    }
    else {
        output("/SA false");
    }
    output(">>\nendobj");
}

/**
 * Completes and write the pages tree.  First we must complete
 * and output the last Pages object that owns Page objects, then
 * we write out the root Pages object.
 * NOTE:  This is a somewhat simplistic tree; all Page objects are
 *        owned by a Pages object one level down from the root.
 *        For now there is no tree balancing or optimisation done.
 * @param pageSize The size of the last Page object
 */
protected void writePagesTree(Rectangle2D pageSize) {

    // Write out the open (last) Pages object
    writePagesObject(pagesReference, pageList, rootPagesReference, pageSize);
    pagesList.add(new Integer(pagesReference));

    // Write out the root Pages object
    writePagesObject(rootPagesReference, pagesList, 0, null);
}

/**
 * Writes out a Pages object.
 * @param referenceNumber The Pages object's reference number
 * @param childList The List of Page/Pages objects below this Pages object
 * @param parent The reference number of the parent Pages object
 * @param pageSize The size of the Page objects under this Pages object
 */
protected void writePagesObject(int referenceNumber, Vector childList,
                                int parent, Rectangle2D pageSize) {

    int i, count, page_ref;

    // Store the reference for the Pages object
    storeObjectReference(referenceNumber, getCurrentOffset());

    output(referenceNumber + " 0 obj");
    output("<<\n/Type /Pages\n/Kids [");

    // Output the page reference list
    count = childList.size();
    for (i = 0; i < count; i++) {
        page_ref = ((Integer) childList.elementAt(i)).intValue();
        output(page_ref + " 0 R");
    }

    /*
    * NOTE:  This will need to change if the Pages tree is constructed
    *        in a more optimal fashion; for now it assumes that the number
    *        of children of a Pages object is the number of Page objects
    *        depending from that root.  The Root Pages object is handled
    *        as a special case that has a count of all pages in the document
    */
    if (parent != 0) {
        // Use the number of children as the number of pages except for the Root
        output("]\n/Count " + count);
        totalPageCount += count;
    }
    else {
        output("]\n/Count " + totalPageCount);
    }

    // Output the parent for all but the topmost Pages object
    if (parent != 0) {
        output("/Parent " + parent + " 0 R");
    }

    // Output the MediaBox for the pages of the document
    if (pageSize != null) {
        String box = "/MediaBox [ "
            + pageSize.getX() + " "
            + pageSize.getY() + " "
            + (pageSize.getWidth() + pageSize.getX()) + " "
            + (pageSize.getHeight() + pageSize.getY()) + " ]";
        output(box);
    }

    // Terminate the object
    output(">>\nendobj");
}

/**
 * Determines if the page has changed, which would require a new Pages
 * object to be started.
 * @param pageSize the current page's size
 * @return True if the current page's size is different from the last
 */
protected boolean pageChanged(Rectangle2D pageSize) {

    // If there is no previous page, store the current page size
    if (oldPageSize == null) {
        oldPageSize = (Rectangle2D) pageSize.clone();
        return (false);
    }

    return (!oldPageSize.equals(pageSize));
}

/**
 * Determines if a given font name represents one of the 3 standard (built-in)
 * PDF font families.
 * @param name the name of a font to check
 * @return True if the font is a standard PDF font
 */
public boolean isBaseFont(String name) {

    String family, style = null;

    if (name.indexOf('-') != -1) {
        style = name.substring(name.indexOf('-') + 1, name.length());
        family = name.substring(0, name.indexOf('-'));
        // style[0] = '-';
        // style++;
    }
    else {
        family = name;
    }

    if (family.equals("Courier") || family.equals("Helvetica")) {
        if (style == null || style.equals("Bold") ||
            style.equals("Oblique") ||
            style.equals("BoldOblique")) {
            return (true);
        }
    }
    else if (family.equals("Times")) {
        if (style.equals("Roman") ||
            style.equals("Bold") ||
            style.equals("Italic") ||
            style.equals("BoldItalic")) {
            return (true);
        }
    }
    else if (family.equals("Symbol") ||
        family.equals("ZapfDingbats")) {
        return (true);
    }

    return (false);

}

/**
 *  Switch output to a buffered stream which will be collected
 *  and written out following the end of the current page stream.
 *  This method takes into account the lists of font and xobject
 *  resources that are needed for each stream.
 */
protected void switchToNewStream() {
    // save current StreamInfo on stack
    streamStack.push(streamInfo);

    // crate new output stream
    os = new ByteArrayOutputStream();

    streamInfo = new StreamInfo();

    // save new output stream in new StreamInfo
    streamInfo.os = os;
}

/**
 * Switch back to the previously used stream.
 */
protected void switchBackToOldStream() {
    // save current list of references on current page
    pageXObjectReferences.addElement(streamInfo.referenceList);
    // save new output stream on current page and in new StreamInfo
    pageXObjectStreams.addElement(os);

    // revert to old StreamInfo and output stream
    streamInfo = (StreamInfo) streamStack.pop();
    os = streamInfo.os;
}

/**
 * Stores the cross-reference table information about an object (number, offset).
 * @param objNumber The number of the object
 * @param offset The offset from the start of the file, in bytes
 */
protected void storeObjectReference(int objNumber, long offset) {
    AcroReference reference = new AcroReference();
    reference.number = objNumber;
    reference.offset = offset;

    streamInfo.referenceList.add(reference);
}

/**
 * Gets the current offset into the output stream.
 * @return The number of bytes currently written to the output
 */
protected long getCurrentOffset() {
    return streamInfo.fileOffset;
}

/**
 * Gets the start (initial offset) of the current contents stream.
 * @return The position (offset) of the start of the contents stream
 */
protected long getStreamStart() {
    return (streamInfo.contentsStart);
}

/**
 * Marks the start of a contents stream (at the current offset).
 */
protected void setStreamStart() {
    streamInfo.contentsStart = streamInfo.fileOffset;
}

/**
 * Zero-pad the given number (file offset) as a 10-digit decimal number
 * @param offset The number to be padded to 10 digits
 * @return The string representation of the number including pad digits
 */
protected String pad10(long offset) {
    StringBuffer sb = new StringBuffer(Long.toString(offset));
    int length = sb.length();
    for (int i = length; i < 10; i++) {
        sb.insert(0, 0);
    }
    return sb.toString();
}

/**
 * Begin a page stream.  If compression turned on, collect output to later write
 * as a compressed stream.  endStream() should be called when all desired
 * input has been written to the stream via the output() method.
 */
protected void beginPageStream() {
    // regardless of whether the stream will be collected,
    // create a new stream
    streamInfo.objectNumber = ++objectNumber;
    streamInfo.lengthObjectNumber = ++objectNumber;

    contentsList.add(new Integer(streamInfo.objectNumber));

    // if collecting stream for compression, write out the appropriate header
    // and start a StringBuffer for the collection
    if (!streamInfo.collectStream && compressed) {
        writeCompressedStreamHeader(streamInfo.objectNumber,
                                    streamInfo.lengthObjectNumber, null);


        streamInfo.collectStream = true;
        streamInfo.collectedStream = new ByteArrayOutputStream();


        // if not collecting stream for compression, write out the appropriate
        // header and note the start of the stream
    }
    else {
        writeUncompressedStreamHeader(streamInfo.objectNumber,
                                      streamInfo.lengthObjectNumber, null);
        setStreamStart();
    }
}

/**
 * Begin a non-page stream.  In this case, the stream header must have
 * already been written to the output.  This stream begins by writing
 * the word "stream".  If compression turned on, collect output to later write
 * as a compressed stream.  endStream() should be called when all desired
 * input has been written to the stream via the output() method.
 */
protected void beginNonPageStream() {
    // begin stream
    output("stream");

    // if collecting stream for compression,
    // start a StringBuffer for the collection
    if (!streamInfo.collectStream && compressed) {
        streamInfo.collectStream = true;
        streamInfo.collectedStream = new ByteArrayOutputStream();
        // if not collecting stream for compression,
        // note the start of the stream
    }
    else {
        setStreamStart();
    }
}

/**
 * End stream.  If compression turned on, write out collected output
 * as a compressed stream.  beginPageStream() or beginNonPageStream()
 * must have been called previously.
 */
protected void endStream() {
    // if was collecting stream for compression, write out compressed
    // stream and appropriate footers
    if (streamInfo.collectStream && compressed) {
        streamInfo.collectStream = false;

        int streamLength =
            writeCompressedStreamBody(
                streamInfo.collectedStream.toByteArray());
        writeCompressedStreamFooter(streamInfo.lengthObjectNumber,
                                    streamLength);

        streamInfo.collectedStream = null;

        // if was not collecting stream for compression, write out appropriate
        // footers for stream
    }
    else {
        int streamLength =
            (int) (streamInfo.fileOffset - streamInfo.contentsStart - 1);
        writeUncompressedStreamFooter(streamInfo.lengthObjectNumber,
                                      streamLength);
    }
}

/**
 * Write the given string to the output, tracking the number of bytes written
 * and adding a new-line at the end of the string.
 * @param string The string to be output
 */
protected void output(String string) {

    if (!isDocumentOpen()) {
        return;
    }

    // not collecting stream for later output -- just let it go out
    if (!streamInfo.collectStream) {
        streamInfo.fileOffset += string.length() + 1;
        super.output(string);
        // collecting stream for later -- add to local buffer as bytes
    }
    else {
        try {
            // bytes must be retrieves as ascii, if possible
            try {
                streamInfo.collectedStream.write(string.getBytes("US-ASCII"));
            } catch (UnsupportedEncodingException uee) {
                streamInfo.collectedStream.write(string.getBytes());
            }
            streamInfo.collectedStream.write((byte) '\n');
        } catch (IOException ioe) {
            throw new PrintException(ioe.toString());
        }
    }
}

/**
 * Write the given byte array to the output, tracking the number of bytes
 * written.  Do *not* add a new line to the end of the byte array.
 * @param bytes the byte array to be output
 */
protected void output(byte[] bytes) {
    if (!isDocumentOpen()) {
        return;
    }

    // not collecting stream for later output -- just let it go out
    if (!streamInfo.collectStream) {
        streamInfo.fileOffset += bytes.length;
        super.output(bytes);
        // collecting stream for later output
    }
    else {
        try {
            streamInfo.collectedStream.write(bytes);
        } catch (IOException ioe) {
            throw new PrintException(ioe.toString());
        }
    }
}

/**
 * Write the specified subsection of the given byte array to the output,
 * tracking the number of bytes written.  Do *not* add a new line to the
 * end of the byte array.
 * @param bytes the byte array to be output
 * @param off offset at which subsection to be written begins
 * @param len length of subsection to be written
 */
protected void output(byte[] bytes, int off, int len) {
    if (!isDocumentOpen()) {
        return;
    }

    // not collecting stream for later output -- just let it go out
    if (!streamInfo.collectStream) {
        streamInfo.fileOffset += len;
        super.output(bytes, off, len);
        // collecting stream for later output
    }
    else {
        streamInfo.collectedStream.write(bytes, off, len);
    }
}

/**
 * Write the specified byte to the output, tracking the number of bytes
 * written.  Do *not* add a new line after the byte is written.
 * @param oneByte the byte to output
 */
protected void output(byte oneByte) {
    if (!isDocumentOpen()) {
        return;
    }

    // not collecting stream for later output -- just let it go out
    if (!streamInfo.collectStream) {
        streamInfo.fileOffset++;
        super.output(oneByte);
        // collecting stream for later output
    }
    else {
        streamInfo.collectedStream.write(oneByte);
    }
}

/**
 * Gets the abbreviation for a given font name.
 * @param fontName The name of a font
 * @return The abbreviation assigned this font on this page
 */
protected String getFontAbbreviation(String fontName) {

    AcroFontEntry fe = (AcroFontEntry) fontList.get(fontName);
    return fe.abbrev;
}

/**
 * Gets the AcroFontEntry class associated with the requested font
 * @param fontName The name of a font
 * @return The AcroFontEntry class associated with the requested font
 */
protected AcroFontEntry getFontEntry(String fontName) {

    AcroFontEntry fe = (AcroFontEntry) fontList.get(fontName);
    return fe;
}

/**
 * Adds a font to the document's list of fonts.
 * @param font The font to be stored in the list
 */
protected void addFontToList(FontPS font) {

    // create an abbreviation for it
    String abbrev = "F" + (fontList.size() + 1);
    AcroFontEntry entry = new AcroFontEntry();
    entry.font = font;
    entry.name = font.getName();

    if (font.usingTTF) {
        entry.usingTTF = true;
        TrueTypeFontProperties ttfProps =
            FontLibrary.lookupTrueTypeFontProperties(entry.name);
        entry.expectedCharacterRange = ttfProps.getCharacterRange();
        if (entry.expectedCharacterRange == TrueTypeFontProperties.ANSI) {
            entry.usingAnsi = true;
            entry.usingUnicode = false;
        }
        else if (entry.expectedCharacterRange ==
            TrueTypeFontProperties.UNICODE) {
            entry.usingAnsi = false;
            entry.usingUnicode = true;
        }
        else if (entry.expectedCharacterRange ==
            TrueTypeFontProperties.AUTO_DETECT) {
            entry.usingAnsi = false;
            entry.usingUnicode = false;
            // this shold never happen
        }
        else {
            entry.usingAnsi = false;
            entry.usingUnicode = false;
        }
        entry.embeddingRules = ttfProps.getEmbeddingRules();
        entry.ansiCharactersUsed = new TreeSet();
        entry.unicodeCharactersUsed = new TreeSet();
        entry.unicodeWidths = new Hashtable();
        entry.includeUnicodeMap = ttfProps.getIncludeUnicodeMap();
    }
    else {
        entry.usingTTF = false;
        entry.expectedCharacterRange = TrueTypeFontProperties.ANSI;
        entry.usingAnsi = true;
        entry.usingUnicode = false;
        entry.embeddingRules = TrueTypeFontProperties.DO_NOT_EMBED;
        entry.ansiCharactersUsed = new TreeSet();
        entry.unicodeCharactersUsed = null;
        entry.unicodeWidths = null;
        entry.includeUnicodeMap = false;
    }

    entry.abbrev = abbrev;
    entry.number = -1;
    entry.unicodeNumber = -1;

    fontList.put(entry.name, entry);
}

/**
 * Returns true if this printer will generate compressed output
 */
public boolean isCompressed(){
    return super.isCompressed();
}

/**
 * Sets whether or not this printer will generate compressed output. Default
 * is true. Only one level of compression is available - it's either on or off.
 * @param compress When true, causes PDF output to be compressed.
 */
public void setCompressed(boolean compress){
    super.setCompressed(compress);
}

protected void addPattern(String patternName, BufferedImage img) {
    PatternInfo patternInfo =
        new PatternInfo(patternName, img);
    streamInfo.patternList.add(patternInfo);
}

///// Inner classes ///////////////////////////////////////

/**
 * Contains pattern-related information.
 */
protected class PatternInfo {
    protected String patternName;
    protected int patternObjectNumber;
    protected int imageObjectNumber;
    protected BufferedImage img;

    protected PatternInfo() {
    }

    protected PatternInfo(String patternName, BufferedImage img) {
        this.patternName = patternName;
        this.img = img;
    }
}

/**
 * Class AcroReference defines a PDF cross-reference marker
 */
protected class AcroReference implements Comparable {
    protected int number;
    protected long offset;

    /**
     * Create a null reference
     */
    public AcroReference() {
    }

    /**
     * Creates a reference for the numbered object at the offset location.
     * @param number The object number whose reference is being stored
     * @param offset The offset into the output stream of the object's start
     */
    public AcroReference(int number, long offset) {
        this.number = number;
        this.offset = offset;
    }

    /**
     * Compares two references and determine which occurs previous in order.
     * @param o The AcroReference object to compare to this one
     * @return An integer indicating the relative order of the two references
     */
    public int compareTo(Object o) {
        AcroReference ar = (AcroReference) o;
        return (this.number - ar.number);
    }
}

/**
 * Information related to PDF 'stream' object which can be used for
 * current page body but also for form and image XObjects.
 */
public class StreamInfo {
    /** Output stream */
    protected OutputStream os;
    /** List of Images and Component ID #'s referenced by this stream. */
    protected Vector imageList;
    /** A List of XObjects (images and components) object numbers
     referenced by this stream. */
    protected Vector xObjectList;
    /** A Set of Fonts which have been used by this stream */
    protected Hashtable fontList;
    /** A List of Patterns referenced by this stream. */
    protected Vector patternList;
    /** Current byte offset into the file or stream */
    protected long fileOffset;
    /** The byte offset of the start of a contents stream */
    protected long contentsStart;
    /** A List of object references and offsets */
    protected Vector referenceList;
    /** object number of current stream */
    protected int objectNumber;
    /** object number of collected stream length */
    protected int lengthObjectNumber;
    /** Records if the current page has any Text objects */
    protected boolean textPresent;
    /** Records if the current page has any Image objects */
    protected boolean imagePresent;
    /** Records if the current page has any Mask objects */
    protected boolean maskPresent;
    /** Records if the current page has any Index Image objects */
    protected boolean indexImage;
    /** A List of ColorSpaces for the current Page */
    protected Vector colorSpaceList;
    /** true if currently collecting output for later use in compressed stream*/
    protected boolean collectStream;
    /** contents of current collected stream */
    //protected StringBuffer collectedStream;
    protected ByteArrayOutputStream collectedStream;

    /** Constructor. */
    public StreamInfo() {
        os = null;
        imageList = new Vector();
        xObjectList = new Vector();
        fontList = new Hashtable();
        fileOffset = 0;
        contentsStart = 0;
        referenceList = new Vector();
        objectNumber = 0;
        lengthObjectNumber = 0;
        colorSpaceList = new Vector();
        patternList = new Vector();

        textPresent = false;
        imagePresent = false;
        maskPresent = false;
        indexImage = false;

        collectStream = false;
        collectedStream = null;
    }

    /**
     * Reset necessary properties during open document operation.
     */
    public void openDocument() {
        fileOffset = 0;
        xObjectList.clear();
        fontList.clear();
        referenceList.clear();
        colorSpaceList.clear();
        patternList.clear();
    }

    /**
     * Reset necessary properties during begin page operation.
     */
    public void beginPage() {
        xObjectList.clear();
        imageList.clear();
        fontList.clear();
        colorSpaceList.clear();
        patternList.clear();

        imagePresent = false;
        indexImage = false;
        maskPresent = false;
        textPresent = false;
    }

}

}
