/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: FontMetricsPS.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:24 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page.adobe.postscript;

import java.awt.Font;
import java.awt.Graphics;
import java.awt.FontMetrics;
import java.util.Enumeration;
import java.text.CharacterIterator;

import com.klg.jclass.page.JCUnit;




import java.awt.font.LineMetrics;


/**
 * An extension of FontMetrics to add PostScript
 * specific font/character information. Sample usage,
 * <pre>
 *		FontPS.loadFonts(); // read from afm files
 *		FontPS font = (FontPS) FontPS.getFont("Palatino-Bold");
 *		FontMetricsPS metrics = new FontMetricsPS(font);
 *		String s = "Hello my name is Joe Lyle.";
 *		double sw = metrics.stringWidth(JCUnit.POINTS, 12, s);
 *		double cw = metrics.charWidth(JCUnit.INCHES, 18, 'W');
 * </pre>
 */



public class FontMetricsPS extends FontMetrics {


public static int AFM_DIVISOR = 1000;

/**
 * Create a new FontMetricPS
 * @param font the FontPS for which metric information is to be gotten
 */
public FontMetricsPS(FontPS font) {
    super(font);
	if ( !(font instanceof FontPS) ) {
		throw new java.lang.IllegalArgumentException("font must be instance of FontPS");
	}
}

/**
 * Overrides FontMetrics method.
 * Return the ascent in points.
 * @see #getAscender
 */
public int getAscent() {
	return (int) getAscender(JCUnit.POINTS, font.getSize());
}

/**
 * Overrides FontMetrics method.
 * Return the descent in points.
 * @see #getDescender
 */
public int getDescent() {
	return (int) - getDescender(JCUnit.POINTS, font.getSize());
}

/**
 * Overrides FontMetrics method.
 * @see #getAscender
 */
public int getMaxAscent() {
	return getAscent(); // is there a max is ps?
}

/**
 * Overrides FontMetrics method.
 * @see #getDescender
 */
public int getMaxDescent() {
	return getDescent(); // is there a min in ps?
}

/**
 * Overrides FontMetrics method.
 */
public int charWidth(char ch) {
	return (int) (preciseCharWidth(ch) + 0.5);
}

/**
 * This more precise method is used by both charWidth()
 * and charsWidth() to prevent cumulative round-off error
 * when determining the width of a string.
 */
protected double preciseCharWidth(char ch) {
	CharMetricPS metric = ((FontPS)font).getCharMetric(ch);
	int w = metric == null ? 0 : metric.getWx();
	return asPoints(w, font.getSize());
}

/**
 * Override FontMetrics method.
 */
public int charsWidth(char chars[], int offset, int length) {
    double width = 0.0;
	for (int i = offset; i < length; i++) {
    	width += preciseCharWidth(chars[i]);
	}
	return (int)(width + 0.5);
}

/**
 * Override FontMetrics method.
 */
public LineMetrics getLineMetrics( String string, Graphics context) {
    return (LineMetrics) new LineMetricsPS(this, string);}

/**
 * Override FontMetrics method.
 */
public LineMetrics getLineMetrics( String string,
                                        int beginIndex, int limit,
                                        Graphics context) {
    return (LineMetrics) new LineMetricsPS(this, string.substring(beginIndex, limit));
}

/**
 * Override FontMetrics method.
 */
public LineMetrics getLineMetrics(char [] chars,
                                        int beginIndex, int limit,
                                        Graphics context) {
	return (LineMetrics) new LineMetricsPS(this, new String(chars, beginIndex, limit));    
}

/**
 * Override FontMetrics method.
 */
public LineMetrics getLineMetrics(CharacterIterator iter,
                                        int beginIndex, int limit,
                                        Graphics context) {
    // make a string
    StringBuffer string = new StringBuffer();
 	for(char c = iter.first(); c != CharacterIterator.DONE; c = iter.next()) {
         string.append(c);
    }
	// Java 1.1 doesn't have StringBuffer.substring, so we convert to String
	String newString = string.toString().substring(beginIndex, limit);
	return((LineMetrics) new LineMetricsPS(this, newString));
}

//////////////// Some FontPS only methods //////////////////////


/**
 * Return the height of this font in the indicated units where height is
 * ascender - descender.
 * @param toUnits the JCUnit in which the result should be returned
 * @param pointSize the int pointSize for scaling this font
 * @return the size of this font in the indicated units
 */
public double getHeight(JCUnit units, int pointSize) {
	int asc = ((FontPS)font).getAscender();
    int dsc = ((FontPS)font).getDescender();
	double heightInPoints = asPoints((asc - dsc), pointSize);
	return convertFromPoints(units, heightInPoints);
}

/**
 * Return the scaled xHeight for this font in the indicated units.
 * @param toUnits the JCUnit in which the result should be returned
 * @param pointSize the int pointSize for scaling this font
 * @return the xHeight of this font in the indicated units
 */
public double getXHeight(JCUnit units, int pointSize) {
	double xHeightInPoints = asPoints(((FontPS)font).getXHeight(),pointSize);
	return convertFromPoints(units, xHeightInPoints);
}

/**
 * Return the scaled ascender for this font in the indicated units.
 * @param toUnits the JCUnit in which the result should be returned
 * @param pointSize the int pointSize for scaling this font
 * @return the ascender of this font in the indicated units
 */
public double getAscender(JCUnit units, int pointSize) {
	double ascenderInPoints = asPoints(((FontPS)font).getAscender(),pointSize);
	return convertFromPoints(units, ascenderInPoints);
}

/**
 * Return the scaled descender for this font in the indicated units.
 * @param toUnits the JCUnit in which the result should be returned
 * @param pointSize the int pointSize for scaling this font
 * @return the descender of this font in the indicated units
 */
public double getDescender(JCUnit units, int pointSize) {
	double descenderInPoints =
		asPoints(((FontPS)font).getDescender(), pointSize);
	return convertFromPoints(units, descenderInPoints);
}

/**
 * Return an afm size converted to point size.
 */
public static double asPoints(double afmSize, int pointSize) {
	return (double) (afmSize * pointSize) / AFM_DIVISOR;
}

/**
 * Given 'value' in points, convert it to 'units'.
 * @param toUnits the JCUnit in which the result should be returned
 * @param value the original value in 'points'
 * @return value converted to 'units'
 */
protected static double convertFromPoints(JCUnit units, double value) {
	double converted = 0;
	if (units == JCUnit.POINTS) {
    	converted = value;
	}
	else if (units == JCUnit.INCHES) {
		converted = JCUnit.getAsInches(JCUnit.POINTS, value);
	}
	else if (units == JCUnit.CM) {
		converted = JCUnit.getAsCentimeters(JCUnit.POINTS, value);
	}
	else {
		throw new
			java.lang.IllegalArgumentException("Unknown units " + units);
	}
	return converted;
}

/**
 * Returns the width of this character scaled to the indicated point
 * size in the indicated units.
 * @param toUnits the JCUnit in which the result should be returned
 * @param pointSize the int pointSize for scaling this font
 * @return the width of this char in the indicated units
 */
public double charWidth(JCUnit units, int pointSize, char ch) {
	CharMetricPS metric = ((FontPS)font).getCharMetric(ch);
	double charWidth = 0.0;
	if (metric != null ) {
		charWidth = asPoints(metric.getWx(), pointSize);
	}
	return convertFromPoints(units, charWidth);
}

/**
 * Returns the total advance width for showing the specified String
 * in this Font taking into account kerning.
 * @param toUnits the JCUnit in which the result should be returned
 * @param pointSize the int pointSize for scaling this font
 * @return the width of this string in the indicated units
 */
public double stringWidth(JCUnit units, int pointSize, String string) {
    double width = 0.0;
	CharMetricPS currentChar, previousChar = null;
	for (int i = 0; i < string.length(); i++) {
    	currentChar 	= ((FontPS)font).getCharMetric(string.charAt(i));
		width 			+= currentChar.getWx();
		// see if there is kern data for this combination of characters
		if (previousChar != null) {
			PairKern kern = previousChar.getKernByName(currentChar.getName());
			if (kern != null) {
				width += kern.xAmount;
			}
		}
		previousChar = currentChar;
	}
	double widthInPoints = asPoints(width, pointSize);
	return convertFromPoints(units, widthInPoints);
}

/**
 * Returns the leading space for the first character of this String.
 * @param toUnits the JCUnit in which the result should be returned
 * @param pointSize the int pointSize for scaling this font
 * @return the width of the leading space in the indicated units
 */
public double getLeadingSpace(JCUnit units, int pointSize, String string) {
	CharMetricPS metric = ((FontPS)font).getCharMetric(string.charAt(0));
	double width = 0.0;
	if (metric != null) {
		if (metric.getCharBBox().llx < 0) {
			width = - metric.getCharBBox().llx;
			width = asPoints(width, pointSize);
		}
	}
	return convertFromPoints(units, width);
}

/**
 * Returns the trailing space for the first character of this String.
 * @param toUnits the JCUnit in which the result should be returned
 * @param pointSize the int pointSize for scaling this font
 * @return the width of the trailing space in the indicated units
 */
public double getTrailingSpace(JCUnit units, int pointSize, String string) {
	int end = string.length() - 1;
	CharMetricPS metric = ((FontPS)font).getCharMetric(string.charAt(end));
	double width = 0.0;
	if (metric != null) {
		if (metric.getCharBBox().llx < 0) {
			width = - metric.getCharBBox().llx;
			width = asPoints(width, pointSize);
		}
	}
	return convertFromPoints(units, width);
}

}
