/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: FontPS.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:24 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page.adobe.postscript;



import com.klg.jclass.page.FontBase;
import com.klg.jclass.page.FontFileLookup;
import com.klg.jclass.page.FontLibrary;
import com.klg.jclass.page.FontParser;
import com.klg.jclass.page.JCUnit;
import com.klg.jclass.page.TrueTypeFontProperties;

import java.awt.*;
import java.awt.font.FontRenderContext;
import java.awt.font.GlyphVector;
import java.awt.geom.Rectangle2D;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.Vector;



/**
 * Extends Font to provide specific information about a font that is
 * culled from backing AFM or TTF files. Sample
 * usage:
 * <pre>
 *		FontPS.loadFonts();
 *
 *		// get a single font by name
 *		FontPS aFont = (FontPS) FontPS.getFont("Palatino-Bold");
 *		System.out.println(aFont.getXHeight());
 *
 *		// get all fonts
 *		Font fonts[] = FontPS.getAllFonts();
 *		for (int i = 0; i < fonts.length; i++) {
 *			FontPS font = (FontPS) fonts[i];
 *			FontMetricsPS metrics = new FontMetricsPS(font);
 *			String s = "hello my name is joe Lyle.";
 *			double sw = metrics.stringWidth(JCUnit.POINTS, 12, s);
 *		}
 * </pre>
 */
public class FontPS extends FontBase implements Cloneable {

public final static int UNICODE_NOT_FOUND_CHARACTER = '\u0020';
public final static int UNICODE_NOT_FOUND_GLYPH = 0;

// properties
protected String    afmVersion;
protected String    fullName;
protected String    familyName;
protected String    weight;
protected float     italicAngle;
protected boolean   isFixedPitch;
protected boolean   isSerif;
protected BoundBox  fontBBox;
protected int       underlinePosition;
protected int       underlineThickness;
protected String    version;
protected String    notice;
protected String    encodingScheme;
protected boolean   encodingFontSpecific = false;
protected int       capHeight;
protected int       xHeight;
protected int       ascender;
protected int       descender;
protected int       fullHeight      = 1000;
protected int       stdHW;
protected int       stdVW;
public static Hashtable fonts       = new Hashtable();
protected Hashtable trackKerning    = new Hashtable();
protected Hashtable charMetrics     = new Hashtable();
public CharMetricPS charMetricList[];// we maintain a hashtable and an
                                        // array of charMetric objects because
                                        // we need to lookup on both name
                                        // and position
protected Hashtable unicodeCharMetrics = new Hashtable();
protected FontMetricsPS psFontMetrics = null;

protected final static String userPropertiesName = "user";
protected static String defaultJarLocation =
        "/com/klg/jclass/page/adobe/fonts.jar";
protected static String defaultFontMap =
        "com.klg.jclass.page.adobe.JCAdobeFontMap";
protected static String defaultFileExtension = ".afm";
protected static boolean loaded = false;
protected static ResourceBundle userFontBundle;
protected static ResourceBundle defaultFontBundle;

public boolean usingTTF = false;

// Marker for a user font loaded via this class     
private static FontFileLookup fakeUserFontLookup = new FontFileLookup();
// Marker for a stock font loaded via this class     
private static FontFileLookup fakeStockFontLookup = new FontFileLookup();

    
/**
 * Parser class that mediates interactions between PageLayout code
 * and the underlying TrueType font parsing code.
 */
public TTFParser ttfParser = null;

// character mappings for iso latin fonts
protected static String char_names_iso_latin[] = {
    "space", "space", "space", "space", "space", "space",           // 0-5
    "space", "space", "space", "space", "space", "space",           // 6-11
    "space", "space", "space", "space", "space", "space",           // 12-17
    "space", "space", "space", "space", "space", "space", "space",  // 18-24
    "space", "space", "space", "space", "space", "space", "space",  // 25-31
    "space", "exclam", "quotedbl", "numbersign", "dollar",          // 32-36
    "percent", "ampersand", "quoteright", "parenleft", "parenright",// 37-41
    "asterisk", "plus", "comma", "minus", "period", "slash", "zero",// 42-48
    "one", "two", "three", "four", "five", "six", "seven", "eight", // 49-56
    "nine", "colon", "semicolon", "less", "equal", "greater",       // 57-62
    "question", "at", "A", "B", "C", "D", "E", "F", "G", "H", "I",  // 63-73
    "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V",// 74-86
    "W", "X", "Y", "Z", "bracketleft", "backslash", "bracketright", // 87-93
    "asciicircum", "underscore", "quoteleft", "a", "b", "c", "d",   // 94-100
    "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q",// 101-113
    "r", "s", "t", "u", "v", "w", "x", "y", "z", "braceleft", "bar",// 114-124
    "braceright", "asciitilde", "space", "Euro", "space", "space",  // 125-130
    "space", "space", "space", "space", "space", "space", "space",  // 131-137
    "space", "space", "space", "space", "space", "space", "dotlessi",// 138-144
    "grave", "acute", "circumflex", "tilde", "macron", "breve",     // 145-150
    "dotaccent", "dieresis", "space", "ring", "cedilla", "space",   // 151-156
    "hungarumlaut", "ogonek", "caron", "space", "exclamdown", "cent",// 157-162
    "sterling", "currency", "yen", "brokenbar", "section",          // 163-167
    "dieresis", "copyright", "ordfeminine", "guillemotleft",        // 168-171
    "logicalnot", "hyphen", "registered", "macron", "degree",       // 172-176
    "plusminus", "twosuperior", "threesuperior", "acute", "mu",     // 177-181
    "paragraph", "periodcentered", "cedilla", "onesuperior",        // 182-185
    "ordmasculine", "guillemotright", "onequarter", "onehalf",      // 186-189
    "threequarters", "questiondown", "Agrave", "Aacute",            // 190-193
    "Acircumflex", "Atilde", "Adieresis", "Aring", "AE", "Ccedilla",// 194-199
    "Egrave", "Eacute", "Ecircumflex", "Edieresis", "Igrave",       // 200-204
    "Iacute", "Icircumflex", "Idieresis", "Eth", "Ntilde", "Ograve",// 205-210
    "Oacute", "Ocircumflex", "Otilde", "Odieresis", "multiply",     // 211-215
    "Oslash", "Ugrave", "Uacute", "Ucircumflex", "Udieresis",       // 216-220
    "Yacute", "Thorn", "germandbls", "agrave", "aacute",            // 221-225
    "acircumflex", "atilde", "adieresis", "aring", "ae", "ccedilla",// 226-231
    "egrave", "eacute", "ecircumflex", "edieresis", "igrave",       // 232-236
    "iacute", "icircumflex", "idieresis", "eth", "ntilde", "ograve",// 237-242
    "oacute", "ocircumflex", "otilde", "odieresis", "divide",       // 243-247
    "oslash", "ugrave", "uacute", "ucircumflex", "udieresis",       // 248-252
    "yacute", "thorn", "ydieresis"									// 253-255
};

/**
 * Constructor
 * @param name the String name of the font
 * @param style the style for this font
 * @param size the size for this font
 */
public FontPS(String name, int style, int size) {
    super(name, style, size);
    fontBBox = new BoundBox();
}

/**
 * Get local font metrics.  Create class if it hasn't been created yet.
 * This ensures only one font metrics class is created per font.
 */
protected FontMetricsPS getLocalFontMetrics() {
    if (psFontMetrics == null) {
        psFontMetrics = new FontMetricsPS(this);
    }
    return psFontMetrics;
}

/**
 *
 * Make a copy of this font including its properties.
 *
 */
protected Object clone() {
    try {
        FontPS clonedFont = (FontPS) super.clone();
        // ensure cloning clears out cached font metrics class
        clonedFont.psFontMetrics = null;
        return clonedFont;
    } catch (CloneNotSupportedException e) {
        // can't happen
        throw new InternalError(e.toString());
    }
}

/************** Overridden Font methods ********/

/**
 * Overrides Font method.
 * Gets the specified font using the name passed in.
 *
 * @param str string of the form
 * <pre>
 *		font name-style-size
 *		font name-style
 *		font name-size
 *		font name
 * </pre>
 * @return the Font or null if not found
 */
public static Font decode(String str) {

    // PENDING the only reason I create an awt font is so it will parse
    // the string for size and style. Should write a decode method.

    // Use our own decodeToFont() instead of the Font.decode() since we
    // can handle a case where the font name has a dash in it
    Font f = decodeToFont(str);

    // get the key name in this format name-style, for example, Times-Bold
    String key = f.getName();
    String style = getStyleString(f.getStyle());

    // if style is already part of font name, remove it from the key
    // so that we can re-add it in our own format.  (Style is sometimes
    // part of font name like "Arial Bold".  We need keys of the form
    // "Arial-Bold".)
    String styleInName = style.replace('-', ' ');
    if (styleInName.length() > 0 && key.endsWith(styleInName)) {
        key = key.substring(0, key.indexOf(styleInName));
    }

    // add style to key
    key += getStyleString(f.getStyle());

    // no longer replacing spaces with underscores in key -- this is
    // taken care of in the lookup method

    // actual font name
    String alias = FontLibrary.lookupFontNameByUserAlias(key, FontLibrary.PDF);

    // if font not yet loaded, load it
    if (!fonts.containsKey(alias)) {
        loadFont(alias);
    }

    // now retrieve the font
    FontPS font = (FontPS) fonts.get(alias);

    // font not found -- use TimesRoman
    if (font == null) {
        System.err.println("Warning: font " + alias + " not loaded, " +
                           "substituting Times-Roman");

        alias = "Times-Roman";
        if (!fonts.containsKey(alias)) {
            loadFont(alias);
        }
        font = (FontPS) fonts.get(alias);
    }

    // and make it into the correct style
    return font.deriveFont(f.getStyle(), f.getSize());
}

/**
 * Load file containing desired font and parse it for the necessary font
 * metrics information.
 */
public static void loadFont(String fontName) {

    // look up font name in list of available fonts to find the location
    // of the file that contains it
    FontFileLookup flook =
        (FontFileLookup) FontLibrary.availableFonts[FontLibrary.PDF].
            get(fontName);

    if (flook == null)
        return;

    // open file containing desired font and parse it
    InputStream inStream = null;
    try {
        if (flook.fileType == FontFileLookup.TTF_FILE) {
            TTFParser parser = new TTFParser();
            TrueTypeFontProperties ttfProps =
                FontLibrary.lookupTrueTypeFontProperties(fontName);

            Vector v = parser.parse(flook, fontName,
                                    // TODO
                                    // Always keep font file around for
                                    // now.  Ideally, this should be
                                    // dependent on whether the font
                                    // is being embedded or not, but
                                    // the current code won't reload
                                    // font if the embed setting
                                    // has changed.  FIX THIS LATER
                                    true);
            //ttfProps.getEmbeddingRules() !=
            //TrueTypeFontProperties.DO_NOT_EMBED);

            if (v.size() > 0) {
                FontPS font = (FontPS) v.firstElement();

                // add font to list of parsed fonts
                fonts.put(font.getName().trim(), font);
            }
        }
        else if (flook.fileType == FontFileLookup.AFM_FILE) {
        	inStream = flook.openInputStream();
            AFMParser parser = new AFMParser();

            Vector v = parser.parse(inStream);
            Font font = (Font) v.firstElement();

            // add font to list of parsed fonts
            fonts.put(font.getName().trim(), font);

    		// close input stream if it exists
    		if (inStream != null) {
        		try {
            		inStream.close();
        		} catch (java.io.IOException ie) {
            		ie.printStackTrace();
        		}
    		}
        }
    } catch (java.io.IOException ie) {
        ie.printStackTrace();
    }

}

/**
 * Decodes a string representation of a font and uses that information
 * to create a Font object.  This is a modification of Sun's Font.decode()
 * method that fixes the problem they have with font names that contain
 * dashes.
 */
public static Font decodeToFont(String str) {
    String fontName = str;
    String fontSizeStr;
    int index;
    int fontSize = 12;
    int fontStyle = Font.PLAIN;

    if (str == null) {
        return new Font("dialog", fontStyle, fontSize);
    }

    int i = str.indexOf('-');
    if (i >= 0) {
        // name may be in upper/lower causing incorrect style matching
        String lcstr = str.toLowerCase();
        index = lcstr.indexOf("bold-italic");
        if (index != -1) {
            fontStyle = Font.BOLD | Font.ITALIC;
        }
        if (index == -1) {
            index = lcstr.indexOf("bolditalic");
            if (index != -1) {
                fontStyle = Font.BOLD | Font.ITALIC;
            }
        }
        if (index == -1) {
            index = lcstr.indexOf("bold");
            if (index != -1) {
                fontStyle = Font.BOLD;
            }
        }
        if (index == -1) {
            index = lcstr.indexOf("italic");
            if (index != -1) {
                fontStyle = Font.ITALIC;
            }
        }
        int lastindex = str.lastIndexOf("-");
        // if we found a style, the font name exists from the beginning
        // of the string to the dash before the style
        if (index != -1) {
            if (index > 0)
                fontName = str.substring(0, index - 1);
        }
        else {
            // if no style, the font name exists from the beginning
            // of the string to this dash
            // note: we know that the dash exists, otherwise we wouldn't
            // have got this far
            fontName = str.substring(0, lastindex);
        }
        if (lastindex != -1) {
            str = str.substring(lastindex + 1);
        }
        try {
            fontSize = Integer.valueOf(str).intValue();
        } catch (NumberFormatException e) {
        }
    }
    else if (i == -1) {

        fontStyle = Font.PLAIN;
        fontSize = 12;

        str = str.toLowerCase();
        index = str.indexOf("bolditalic");
        if (index != -1) {
            fontStyle = Font.BOLD | Font.ITALIC;
        }
        if (index == -1) {
            index = str.indexOf("bold italic");
            if (index != -1) {
                fontStyle = Font.BOLD | Font.ITALIC;
            }
        }
        if (index == -1) {
            index = str.indexOf("bold");

            if (index != -1) {
                fontStyle = Font.BOLD;
            }
        }
        if (index == -1) {
            index = str.indexOf("italic");
            if (index != -1)
                fontStyle = Font.ITALIC;
        }

        if (index != -1) {  // found a style
            fontName = fontName.substring(0, index);
            fontName = fontName.trim();
        }
        index = str.lastIndexOf(" ");
        if (index != -1) {
            fontSizeStr = str.substring(index);
            fontSizeStr = fontSizeStr.trim();
            try {
                fontSize = Integer.valueOf(fontSizeStr).intValue();
            } catch (NumberFormatException e) {
            }
        }
    }
    return new Font(fontName, fontStyle, fontSize);
}

/**
 * Overrides Font.getFamily().
 */
public String getFamily() {
    return getFamilyName();
}


/**
 * Overrides Font method.
 */
public String getFontName() {
    return getFullName();
}

/**
 * Overrides Font method.
 */
public boolean isPlain() {
    return !(isBold() || isItalic());
}

/**
 * Overrides Font method.
 */
public boolean isBold() {
    return getFontName().toLowerCase().indexOf("bold") > -1;
}

/**
 * Overrides Font method.
 */
public boolean isItalic() {
    // italic fonts have angles
    // which are not zero
    return getItalicAngle() != 0;
}

public boolean isTrueType() {
    return usingTTF;
}

/**
 * Override Font method.
 */
public Font deriveFont(int style, float size) {
    FontPS fps = (FontPS) clone();
    fps.style = style;
    fps.size = (int) size;
    return fps;
}

/**
 * Overrides Font method.
 * All all the attributes of this font instance the same
 * as obj's?
 * @return true if attributes are the same values, false otherwise.
 */
public boolean equals(Object obj) {
    if (obj instanceof FontPS) {
        FontPS font = (FontPS) obj;
        return
            getAfmVersion() == font.getAfmVersion() &&
            ((getFullName() == null && font.getFullName() == null) ||
            getFullName().equals(font.getFullName())) &&
            ((getFamilyName() == null && font.getFamilyName() == null) ||
            getFamilyName().equals(font.getFamilyName())) &&
            getWeight() == font.getWeight() &&
            getItalicAngle() == font.getItalicAngle() &&
            isFixedPitch() == font.isFixedPitch() &&
            isSerif() == font.isSerif() &&
            getUnderlinePosition() == font.getUnderlinePosition() &&
            getUnderlineThickness() == font.getUnderlineThickness() &&
            getVersion() == font.getVersion() &&
            getNotice() == font.getNotice() &&
            getEncodingScheme() == font.getEncodingScheme() &&
            getCapHeight() == font.getCapHeight() &&
            getXHeight() == font.getXHeight() &&
            getAscender() == font.getAscender() &&
            getDescender() == font.getDescender() &&
            getStdHW() == font.getStdHW() &&
            getStdVW() == font.getStdVW() &&
            ((getName() == null && font.getName() == null) ||
            getName().equals(font.getName())) &&
            getTrackKerning() == font.getTrackKerning();
    }
    return false;
}

/**
 * Overrides Font method returns true, for now.
 */
public boolean canDisplay(char c) {
    return true;
}

/**
 * Overrides Font method
 * Returns the italic angle of this font.
 */
public float getItalicAngle() {
    return italicAngle;
}

/**
 * Overrides Font method
 *
 */
public GlyphVector createGlyphVector(FontRenderContext frc, String string) {
    FontMetricsPS metrics = new FontMetricsPS(this);
    return new GlyphVectorPS(getLocalFontMetrics(), frc, string);
}

/**
 * Overrides Font method
 *
 */
public Rectangle2D getStringBounds(String string, FontRenderContext frc) {


    Rectangle r = new Rectangle(0, 0, 0, 0);

    if (string != null && string.length() > 0) {
        FontMetricsPS m = getLocalFontMetrics();
        int size = m.getFont().getSize();
        double width = m.stringWidth(JCUnit.POINTS, size, string);
        double x = m.getLeadingSpace(JCUnit.POINTS, size, string) +
            m.getTrailingSpace(JCUnit.POINTS, size, string);
        double y = -m.getAscender(JCUnit.POINTS, size);
        double height = m.getHeight(JCUnit.POINTS, size);
        r.setRect(x, y, width, height);
    }
    return r;
}

/************ End overridden Font methods **************/

////////////// Accessors/Setters ///////////////

/**
 * Set the AFM format specification used to create
 * this file.
 * @param afmVersion the String AFM version
 */
public void setAfmVersion(String afmVersion) {
    this.afmVersion = afmVersion;
}

/**
 * Get the AFM format specification used to create
 * this file.
 * @return the String AFM version
 */
public String getAfmVersion() {
    return afmVersion;
}

/**
 * Set the full name of the font, for example,
 * ITC Garamond-Light.
 * @param fullName the String fullName
 */
public void setFullName(String fullName) {
    this.fullName = fullName;
}

/**
 * Get the full name of the font, for example,
 * ITC Garamond-Light.
 * @return the full name of the font
 */
public String getFullName() {
    return fullName;
}

/**
 * Set the typeface family to which this font
 * belongs.
 * @param familyName the familyName as a String
 */
public void setFamilyName(String familyName) {
    this.familyName = familyName;
}

/**
 * Get the typeface family to which this font
 * belongs.
 * @return the typeface family to which this font
 * belongs
 */
public String getFamilyName() {
    return familyName;
}

/**
 * Set the weight of the font, for example,
 * Roman, Bold, Light.
 * @param weight the weight of the font
 */
public void setWeight(String weight) {
    this.weight = weight;
}

/**
 * Get the weight of the font, for example,
 * Roman, Bold, Light.  For a TrueType font, this will be an empty string.
 * @return the weight of the font
 */
public String getWeight() {
    return weight;
}

/**
 * Set the italicAngle for this font.
 * Angle (in degrees counter-clockwise with the
 * vertical) of the dominant vertical strokes of the
 * font, zero for non italic fonts.
 * @param italicAngle the degree of slant
 */
public void setItalicAngle(float italicAngle) {
    this.italicAngle = italicAngle;
}

/**
 * Is this a monospace font?
 * @param isFixedPitch - true is monospace, false is
 * proportional
 */
public void setFixedPitch(boolean isFixedPitch) {
    this.isFixedPitch = isFixedPitch;
}

/**
 * Is this a monospace font?
 * @return fixedPitch - true is monospace, false is
 * proportional
 */
public boolean isFixedPitch() {

    return isFixedPitch;
}

/**
 * Is this a serif font?  A value of false indicates either no serifs
 * or unsure whether the font has serifs.
 * @param isSerif - true if font has serifs, false if it doesn't or
 * not sure.
 */
public void setSerif(boolean isSerif) {
    this.isSerif = isSerif;
}

/**
 * Is this a serif font?
 * @return true if font has serifs, false if it doesn't or not sure.
 */
public boolean isSerif() {

    return isSerif;
}

/**
 * Set the four numbers giving the lower left corner
 * and the upper right corner of the font bounding
 * box.
 * @param fontBBox the bounding box for this font
 */
public void setFontBBox(BoundBox fontBBox) {
    this.fontBBox = fontBBox;
}

/**
 * Get the four numbers giving the lower left corner
 * and the upper right corner of the font bounding
 * box.
 * @return the BoundBox for this font
 */
public BoundBox getFontBBox() {
    return fontBBox;
}

/**
 * Set the distance from the baseline for centering
 * underline strokes.
 * @param underlinePosition an int indictating the
 * underlinePosisition
 */
public void setUnderlinePosition(int underlinePosition) {
    this.underlinePosition = underlinePosition;
}

/**
 * Get the distance from the baseline for centering
 * underline strokes.
 * @return underlinePosition an int indictating the
 * underlinePosisition
 */
public int getUnderlinePosition() {
    return underlinePosition;
}

/**
 * Set the stroke width for underlining.
 * @param underlineThickness an int for underlineThickness
 */
public void setUnderlineThickness(int underlineThickness) {
    this.underlineThickness = underlineThickness;
}

/**
 * Get the stroke width for underlining.
 * @return underlineThickness an int for underlineThickness
 */
public int getUnderlineThickness() {
    return underlineThickness;
}

/**
 * Set the font program version identifier.
 * @param version String - program version
 */
public void setVersion(String version) {
    this.version = version;
}

/**
 * Get the font program version identifier.
 * @return version String - program version
 */
public String getVersion() {
    return version;
}

/**
 * Set the copyright or trademark.
 * @param notice String - copyright
 */
public void setNotice(String notice) {
    this.notice = notice;
}

/**
 * Get the copyright or trademark.
 * @return copyright
 */
public String getNotice() {
    return notice;
}

/**
 * Set the default encoding vector.
 * @param encodingScheme the default encoding vector such as
 * "AdobeStandardEncoding"
 */
public void setEncodingScheme(String encodingScheme) {
    this.encodingScheme = encodingScheme;

    // to speed things up, store common encodings in booleans
    // for quick checking later on
    if (encodingScheme.equals("FontSpecific")) {
        encodingFontSpecific = true;
    }
}

/**
 * Get the default encoding vector.
 * @return the default encoding vector such as
 * "AdobeStandardEncoding"
 */
public String getEncodingScheme() {
    return encodingScheme;
}

/**
 * Set the y value of the capital H.
 * @param capHeight an int height
 */
public void setCapHeight(int capHeight) {
    this.capHeight = capHeight;
}

/**
 * Get the y value of the capital H.
 * @return the int height
 */
public int getCapHeight() {
    return capHeight;
}

/**
 * Set the y value of the top of lowercase x.
 * @param xHeight an int, the height
 */
public void setXHeight(int xHeight) {
    this.xHeight = xHeight;
}

/**
 * Get the y value of the top of lowercase x.
 * @return xHeight an int
 */
public int getXHeight() {
    return xHeight;
}

/**
 * Set the y value for the top of the lowercase
 * "d".
 * @param ascender the height of the "d" char
 */
public void setAscender(int ascender) {
    this.ascender = ascender;
}

/**
 * Get the y value for the top of the lowercase
 * "d".
 * @return the height of the "d" char
 */
public int getAscender() {
    return ascender;
}

/**
 * Set the y value for the bottom of the lowercase
 * "p" char.
 * @param descender an int the depth
 */
public void setDescender(int descender) {
    this.descender = descender;
}

/**
 * Get the y value for the bottom of the lowercase
 * "p" char.
 * @return an int, the depth
 */
public int getDescender() {
    return descender;
}

/**
 * Set the full height of a standard character in the font (including leading),
 * against which the ascender and descender are compared.  This is only
 * used for TrueType fonts.
 * @param fh full height of standard character in font, including ascender,
 * descender and leading.
 */
public void setFullHeight(int fh) {
    this.fullHeight = fh;
}

/**
 * Fet the full height of a standard character in the font (including leading),
 * against which the ascender and descender are compared.  This is only
 * used for TrueType fonts.
 * @return full height of standard character in font, including ascender,
 * descender and leading.
 */
public int getFullHeight() {
    return fullHeight;
}

/**
 * Set the dominant width of horizontal stems.
 * @param stdHW an int height
 */
public void setStdHW(int stdHW) {
    this.stdHW = stdHW;
}

/**
 * Get the dominant width of horizontal stems.
 * @return an int, the height
 */
public int getStdHW() {
    return stdHW;
}

/**
 * Set the dominant width of horizontal stems.
 * @param stdVW an int, the width
 */
public void setStdVW(int stdVW) {
    this.stdVW = stdVW;
}

/**
 * Get the dominant width of horizontal stems.
 * @return an int, the width
 */
public int getStdVW() {
    return stdVW;
}

/**
 * Set the name of the font. Sets the 'name'
 * property in the super class.
 *
 * @param name the String name of the font
 */
public void setFontName(String name) {
    super.name = name;
}

public Hashtable getTrackKerning() {
    return trackKerning;
}

/**
 * Get the character metrics hashtable for Type 1 fonts.  This should
 * never be called by a font backed by a TrueType file.
 */
public Hashtable getCharMetrics() {
    return charMetrics;
}

/**
 * Get the character metrics hashtable for Unicode-based fonts.  This should
 * only be called by a font backed by a TrueType file.
 */
public Hashtable getUnicodeCharMetrics() {
    return unicodeCharMetrics;
}

/**
 * Get the character metric information for this character.
 * Look it up by code if it is "FontSpecific", by name otherwise.
 * @param ch the char to lookup
 * @return the corresponding CharMetricPS
 */
public CharMetricPS getCharMetric(char ch) {

    CharMetricPS metric;

    if (usingTTF) {
        // lookup char by code
        metric = getCharByCode((int) ch);
    }
    else if (encodingFontSpecific) {

        // lookup char by code
        metric = getCharByCode((int) ch);
    }
    else {
        // intercept euro character -- map to euro character in pdf mapping
        if ((int) ch == 0x20ac) {
            ch = (char) 128;
            // lookup by name
            metric = getCharByName(char_names_iso_latin[ch]);
            // if euro character not found in metrics, use metrics for
            // the zero character -- they should be nearly the same
            if (metric == null) {
                metric = getCharByName("zero");
            }
            // character outside ansi range -- fake with default metrics
        }
        else if ((int) ch > 255) {
            String warning = "Warning: character 0x" +
                Integer.toHexString((int)ch) +
                " oustide ISO-Latin1 range -- substituting metrics";
            System.err.println(warning);
            metric = null;
            // character inside ansi range -- lookup metrics by name
        }
        else {
            metric = getCharByName(char_names_iso_latin[ch]);
        }
    }

    // if the metrics are not found, substitute the metrics for
    // the space character -- it'll be wrong, but at least it won't crash
    if (metric == null) {
        if (usingTTF) {
            metric = getCharByCode(UNICODE_NOT_FOUND_CHARACTER);
        }
        else {
            metric = getCharByName("space");
        }
    }
    return metric;
}

/**
 * Find a character's metrics by its code.
 * @param code the int code
 * @return the corresponding CharMetricPS
 */
public CharMetricPS getCharByCode(int code)
{
    CharMetricPS metric = null;

    // TrueType Fonts -- characters are referenced by unicode values
    if (usingTTF) {
        // see if we've already created the metric class
        metric = (CharMetricPS) unicodeCharMetrics.get(new Integer(code));
        if (metric == null) {
            // it hasn't been created yet, so go to it, and save the result
            metric = ttfParser.createCharMetric(code);
            // if metric does not exist within font, copy the metrics from
            // the 'missing' glyph used by fonts for just such circumstances
            if (metric == null) {
                metric = ttfParser.createCharMetricFromGlyph(
                    UNICODE_NOT_FOUND_GLYPH);
                // if the metrics exist, use them for this character
                if (metric != null) {
                    metric.setCode(code);
                    // if the metrics for the special 'missing' character don't
                    // exist either, just give up and use blank ones
                }
                else {
                    metric = ttfParser.createEmptyCharMetric(code);
                }
            }
            unicodeCharMetrics.put(new Integer(code), metric);
        }
        // Type 1 Fonts -- characters are referenced by postscript name
    }
    else {
        Enumeration e = getCharMetrics().elements();
        while (e.hasMoreElements()) {
            CharMetricPS m = (CharMetricPS) e.nextElement();
            if (m.getCode() == code) {
                metric = m;
                break;
            }
        }
    }

    // if not found return space
    return metric == null ? getCharByName("space") : metric;
}

/**
 * Find a character's metrics by its name.
 * This should never be called by a Unicode font such as a TrueType font.
 * @param name the String name of the char
 * @return the corresponding CharMetricPS
 */
public CharMetricPS getCharByName(String name) {
    return (CharMetricPS) getCharMetrics().get(name);
}

/**
 *
 * Load fonts from the specified jar file.
 * This constructor will load the fonts specified in the user-supplied
 * jar file. These will be added to the default fonts supported by
 * PageLayout.
 * @param parser the FontParser instance to use to parse the afm files
 * @param jarLocation the location of the jar file relative to this class
 * @param fileExtension the the file extension of the afm files (normally .afm)
 * @param userFontMap a text resource containing mappings of user font names
 *  (as used by you)
 * to font file font names (as contained within an actual font file)
 */
public static Font[] getAllFonts(FontParser parser, String jarLocation,
                                 String fileExtension, String userFontMap) {

    // save state so that loadAllFonts can be repeated
    boolean wasLoaded = loaded;
    
    // get the stock fonts - same code used to be duplicated here 
    getAllFonts();
    // undo some of the settings by the previous call
    getFontBundles(defaultFontMap, userFontMap);

    // restore to value before loading stock fonts, then load user fonts
    loaded = wasLoaded;
    
    Hashtable userFonts = new Hashtable();
    // load the user-defined mappings - returns fonts loaded from jar
    Font f[] = getFonts(parser,
                        jarLocation,
                        fileExtension,
                        userFonts,
                        loaded);
    
    // Tell FontLibrary about these fonts. Since they're already loaded,
    // the FontFileLookup will not be needed to load these fonts,
    //  but FontLibrary needs to know that the font is available.
    // As these fonts can't be loaded by FontLibrary yet, store their references
    // here, together with stock fonts - this is legacy behaviour
    for (int i=0;i<f.length;i++){
            FontLibrary.availableFonts[FontLibrary.PDF].put(
                f[i].getName(),fakeUserFontLookup);
            fonts.put(f[i].getName(),f[i]);
    }
    // assuming font type is AFM - javadoc says this is the only supported 
    // parser
    FontLibrary.addFontNameMap(userFontBundle, FontLibrary.AFM);
    loaded = true;
    return f;
}

/**
 *
 * Default font loading. Read fonts from the "fonts.jar" file,
 *	"/com/klg/jclass/page/adobe/fonts.jar"
 * which is relative to the classpath.
 * File which end with the extension ".afm" are parsed.
 * This maps between fonts and their aliases (the real font to
 * use).
 *
 */
public static Font[] getAllFonts() {
    AFMParser parser = new AFMParser();

    // user did not supply a font map so use default for both
    getFontBundles(defaultFontMap, defaultFontMap);

    // user has not specified a font bundle so use default as both
    Font f[] = getFonts(parser,
                        defaultJarLocation,
                        defaultFileExtension,
                        fonts,
                        loaded);

    // Tell FontLibrary about these fonts - since they're already loaded,
    // the FontFileLookup will not be needed to load these fonts, but FontLibrary
    // needs to know that the font is available
    if (FontLibrary.availableStockFonts[FontLibrary.PDF]==null){
        FontLibrary.availableStockFonts[FontLibrary.PDF] = new Hashtable();
    }
    for (int i = 0; i < f.length; i++) {
        FontLibrary.availableStockFonts[FontLibrary.PDF].put(
            f[i].getName(),fakeStockFontLookup);
    }
    FontLibrary.stockFontNameMapBundles[FontLibrary.PDF] = defaultFontBundle;
    loaded = true;
    return f;
}


/**
 * PageLayout passes in awt Fonts, so we need to
 * get the equivalent native (ps or pcl) font.
 */
public static Font getNativeFont(Font font) {

    // Use decode to get a FontPS equivalent font
    String nm = font.getName();
    int st = font.getStyle();
    int sz = font.getSize();
    Font f = decode(getDecodeString(nm, st, sz));

    return (f);


}

/**
 * Load both the default font map resoruce bundle and the
 * user assigned font map resource bundle.
 *
 * @param defaultBundleName the fully qualified default fontmap bundle
 *			name.
 * @param userBundleName the fully qualified default fontmap bundle name,
 * can be a class or a text file -- for example, specifying
 * "com.klg.jclass.page.UserBundle" will cause the loading mechanism
 * to look first for the class "com.klg.jclass.page.UserBundle.class"
 * and if that's not found then the file "UserBundle.properties" in the
 * com/klg/jclass/page directory.
 */
protected static void getFontBundles(String defaultBundleName, String userBundleName) {
    try {
        userFontBundle = ResourceBundle.getBundle(userBundleName);
    } catch (MissingResourceException e) {
        // okay, we'll only use the default
    }
    try {
        defaultFontBundle = ResourceBundle.getBundle(defaultBundleName);
    } catch (MissingResourceException e2) {
        System.out.println(e2.toString());
    }
}

} // end FontPS class
