/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: Graphics2DPS.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:24 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page.adobe.postscript;

import java.io.BufferedReader;
import java.io.IOException;
import java.awt.Image;
import java.awt.image.PixelGrabber;
import java.awt.Shape;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Rectangle;
import java.awt.Graphics;
import java.awt.Polygon;
import java.awt.image.ImageObserver;
import java.util.StringTokenizer;

import java.io.OutputStream;

import com.klg.jclass.page.Graphics2DBase;
import com.klg.jclass.page.EPSImage;




import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.Stroke;

import java.awt.geom.*;


/**
 * Overrides Graphics2DBase to provide the java interface
 * to postscript drawing.
 */

public class Graphics2DPS extends Graphics2DBase {

// keep track of whether font has been written out yet in current frame
boolean fontWritten = false;

// hex number mapping
protected final static char hd[] = {'0', '1', '2', '3', '4', '5', '6', '7',
                                      '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
/** how far to read ahead while looking for an EPS's bounding box information */
protected int readAheadAmount = 1000;

protected JCPostScriptPrinter printer = null;

/**
 *
 * Constructor
 *
 */
public Graphics2DPS(OutputStream os) {
	super(os);
}

/**
 *
 * Constructor
 *
 */
public Graphics2DPS(OutputStream os, JCPostScriptPrinter printer) {
	super(os);
	this.printer = printer;
}

/**
 * Overrides g2d method.
 */
public void drawString(String string, float x, float y) {

    // shouldn't attempt to draw null or empty strings
    if (string == null || string.equals("")) {
        return;
    }

	// Make sure we are in the correct state
	updateContext();

	// adjust to java origin
	y = -y;

    CharMetricPS currentChar, previousChar = null;

	// adjust color if necessary
	outputCurrentColor();

	// get the font
	FontPS font = (FontPS) getFont();

	if (!fontWritten)
		outputFont(font);

	// initialize the buffer
	initializeOutString(x, y);

	// now add the string to the buffer
    for (int i = 0; i < string.length(); i++) {
        currentChar     = font.getCharMetric(string.charAt(i));

        // see if there is kern data for this combination of characters
        if (previousChar != null) {
            PairKern kern = previousChar.getKernByName(currentChar.getName());
            if (kern != null && kern.xAmount != 0) {
				// flush the existing string
				printOutString();

				// adjust position by kerning distance
    			out.x += FontMetricsPS.asPoints(kern.xAmount, font.getSize());
            }
        }

		// add the current char to the buffer
		double xWidth = FontMetricsPS.asPoints(currentChar.getWx(), font.getSize());
		appendToOutString(string.charAt(i), xWidth);
        previousChar = currentChar;
    }

	// print out the existing string
	printOutString();

    // Add an underline if required
    if (underline) {

        // correct the drawing line with if necessary
        double underlineWidth = FontMetricsPS.asPoints(font.getUnderlineThickness(), font.getSize());
		setStroke(new BasicStroke((float) underlineWidth));

        // adjust the baseline of the underlining and draw
		// convert back to java coordinates for since drawLine expects them
        y = (float) (-y - FontMetricsPS.asPoints(font.getUnderlinePosition(), font.getSize()));
		drawLinePW(x, y, out.x, y);
    }
}

/**
 * Override the Graphics2D translate method
 * @param x the X co-ordinate of the translation
 * @param y the Y co-ordinate of the translation
 */
public void translate(int x, int y) {
	translate((double) x, (double) y);
}

/**
 * Override the Graphics2D translate method
 * @param x the X co-ordinate of the translation
 * @param y the Y co-ordinate of the translation
 */
public void translate(double x, double y) {
    Rectangle2D clip = (Rectangle2D)getClip();
	double clipX, clipY, clipWidth, clipHeight;

	// Accumulate the translation to the current transformation matrix
	transform.translate(x, y);

	// Adjust the clip so it doesn't move
	clipX = clip.getX() - x;
	clipY = clip.getY() - y;
	clipWidth = clip.getWidth();
	clipHeight = clip.getHeight();
	clip.setRect(clipX, clipY, clipWidth, clipHeight);
	this.clipSet = true;

	// if this is the current context, output the translation motion
	if (isContextCurrent()) {
		y = -y;
		output(truncateDecimalDigits(x) + " " + truncateDecimalDigits(y)
			   + " translate");
	}
}

/**
 * Override the Graphics2D scale method
 * @param x the X factor of the scale
 * @param y the Y factor of the scale
 */
public void scale(double x, double y) {
    Rectangle2D clip = (Rectangle2D)getClip();

	double clipX, clipY, clipWidth, clipHeight;

	// Accumulate the scaling to the current transformation matrix
	transform.scale(x, y);

	// Adjust the clip so it doesn't move
	clipX = clip.getX() / x;
	clipY = clip.getY() / y;
	clipWidth = clip.getWidth() / x;
	clipHeight = clip.getHeight() / y;
	clip.setRect(clipX, clipY, clipWidth, clipHeight);
	this.clipSet = true;

	// if this is the current context, output the scale command
	if (isContextCurrent()) {
	    output(truncateDecimalDigits(x) + " "
			   + truncateDecimalDigits(y) + " scale");
	}
}

/**
 * Override the Graphics2D rotate method.
 * Only implement right angle rotation (90/180/270 degrees)
 * (that's all that chart uses) for now.
 * @param theta the angle to rotate, in radians.
 */
public void rotate(double theta) {
    Rectangle2D clip = (Rectangle2D)getClip();

	// update the current transformation matrix
	transform.rotate(theta);

	//DAM -- can I somehow work this for all angles?? Should be doable.

	// Adjust the clip so it doesn't move
	// Size of clip only changes in 90 and 270 degree case
	if (theta == Math.PI/2.0 ||
			theta == 3 * Math.PI/2.0) {
		double clipX, clipY, clipWidth, clipHeight;
		clipX = clip.getY();
		clipY = clip.getX();
		clipWidth = clip.getHeight();
		clipHeight = clip.getWidth();
		clip.setRect(clipX, clipY, clipWidth, clipHeight);
		this.clipSet = true;
	}

	// For some reason, java has angles backwards.  Flip.
    theta = (2*Math.PI) - theta;

	// Convert from radians to degrees
	theta = Math.toDegrees(theta);

	// if this is the current context, output the translation
	if (isContextCurrent()) {
	    output(truncateDecimalDigits(theta) + " "
			   + " rotate");
	}
}

/**
 * Override the Graphics2D rotate method.
 * Implement in terms of existing translate and rotate methods.
 * @param theta the angle to rotate, in radians.
 * @param x the distance to translate in x
 * @param y the distance to translate in y
 */
public void rotate(double theta, double x, double y) {
	translate(x, y);
	rotate(theta);
	translate(-x, -y);
}


/**
 * Override the Graphics2D clip method
 * @param x the X co-ordinate of the clip origin
 * @param y the Y co-ordinate of the clip origin
 * @param width the width of the clip window
 * @param height the height of the clip window
 */
public void clipRect(int x, int y, int width, int height) {
	clipRect((double) x, (double) y, (double) width, (double) height);
}

/**
 * Override the Graphics2D clip method
 * @param x the X co-ordinate of the clip origin
 * @param y the Y co-ordinate of the clip origin
 * @param width the width of the clip window
 * @param height the height of the clip window
 */
public void clipRect(double x, double y, double width, double height) {
    Rectangle2D clip = (Rectangle2D)getClip();

	// if there is a clip rectangle set already, intersect them
	if (clipSet) {

		// Store the intersection of the new and old rectangles
		Rectangle2D intersectionRect = new Rectangle2D.Double();
		Rectangle2D.intersect(clip, new Rectangle2D.Double(x, y, width, height),
							  intersectionRect);
		clip = intersectionRect;
	}

	// otherwise (no existing clip) use the given clip
	else {
		clip.setRect(x, y, width, height);
		this.clipSet = true;
	}

	// If the context is current, output the clip window change
	if (isContextCurrent()) {
	    output(truncateDecimalDigits(clip.getX()) + " "
			   + truncateDecimalDigits(-clip.getY()) + " "
			   + truncateDecimalDigits(clip.getWidth()) + " "
			   + truncateDecimalDigits(-clip.getHeight()) + " rc");
	}
}

//////////////////// Some Graphics (not 2D) methods /////////

/**
 *
 * Overrides Graphics method
 *
 */
public void drawPolyline(int[] xPoints, int[] yPoints, int nPoints) {

	// Make sure we are in the correct state
	updateContext();

	// set the color
	outputCurrentColor();

	// set the line width
	setLineStroke();

	// first point
	output("newpath " + xPoints[0] + " " + (-yPoints[0]) + " moveto");

	// the rest
	for (int i = 1; i < nPoints; i++) {
		output(xPoints[i] + " " + (-yPoints[i]) + " lineto");
	}
	output("stroke");
}

/**
 *
 * Overrides Graphics method
 *
 */
public void drawPolygon(int[] xPoints, int[] yPoints, int nPoints) {
	printPolygon(xPoints, yPoints, nPoints, "stroke");
}

/**
 *
 * Overrides Graphics method
 *
 */
public void fillPolygon(int[] xPoints, int[] yPoints, int nPoints) {
	printPolygon(xPoints, yPoints, nPoints, "fill");
}

/**
 *
 * Overrides Graphics method
 *
 */
public boolean drawImage(Image img, int x, int y, int width, int height,
		Color bgcolor, ImageObserver observer) {

	// Make sure we are in the correct state
	updateContext();

	if (img instanceof EPSImage) {
		return drawEPSImage((EPSImage)img, x, y, width, height, bgcolor, observer);
	}
	else {
		return drawJavaImage(img, x, y, width, height, bgcolor, observer);
	}
}

/**
 *
 * Overrides Graphics method
 *
 */
public  void setFont(Font font) {

	// Make sure we are in the correct state
	updateContext();

    if (!(font instanceof FontPS)) {
		// if an awt font was passed in, get its ps equivalent
		font = FontPS.getNativeFont(font);
    }

	// only set it if we must
	if (currentFont != null && fontWritten &&
			(currentFont.equals(font) &&
			currentFont.getStyle() == font.getStyle() &&
			currentFont.getSize() == font.getSize())) {
		return;
	}
	this.currentFont	= font;
	outputFont(font);
}

/**
 * Write out the PS code for setting a font.
 */
protected void outputFont(Font font)
{
	String name			= font.getName();
	int size			= font.getSize();

	// if it's not Symbol nor ZapfDingbats see if we need to load it
	if (!name.equals("Symbol") && !name.equals("ZapfDingbats")) {

		// if not already loaded ...
		if (!loadedFonts.contains(name)) {

			// add the font to the list of loaded fonts
			loadedFonts.put(name, name);

			// load the font and create an ISO-Latin version
			output("/" + name + "-ISOLatin1 /" + name + " iso");
		}

		output("/" + name + "-ISOLatin1 " + size + " sf");
	}
	else {
		output("/" + name + size + " sf");
	}
	fontWritten = true;
}

public  FontMetrics getFontMetrics(Font font) {
	if (!(font instanceof FontPS)) {
		// if an awt font was passed in, get its ps equivalent
		font = FontPS.getNativeFont(font);
    }
	return new FontMetricsPS((FontPS) font);
}

////////////////////////// Device-specific methods /////////////////////

/**
 * Enter a graphics
 */
public void startContext() {
    Rectangle2D clip = (Rectangle2D)getClip();

	// Create a local context for the new coordinates
	output("gsave % frame begin");

    // Set the transformation matrix
    double[] matrix = new double[6];
    transform.getMatrix(matrix);
    output("[" + truncateDecimalDigits(matrix[0]) + " " +
                truncateDecimalDigits(matrix[1]) + " " +
                truncateDecimalDigits(matrix[2]) + " " +
                truncateDecimalDigits(matrix[3]) + " " +
                truncateDecimalDigits(matrix[4]) + " " +
                truncateDecimalDigits(-1 * matrix[5]) + "] concat");

	output(truncateDecimalDigits(clip.getX()) + " " +
		   truncateDecimalDigits(-clip.getY()) + " " +
		   truncateDecimalDigits(clip.getWidth()) + " " +
		   truncateDecimalDigits(-clip.getHeight()) + " rc");

    // line stroke must be respecified, otherwise the one from the
    // parent will be used and it could be different
    strokeChanged = true;
}

/**
 * Exit a graphics
 */
public void endContext() {
	// Restore the previous local context
	output("grestore % frame end");
	fontWritten = false;
}

/**
 *
 * Device-specific methods
 *
 */
public void drawArcPW(double x, double y, double width, double height,
                             double startAngle, double arcAngle, int type) {

	// Make sure we are in the correct state
	updateContext();

	// set the line width
	setLineStroke();

	// set the color
	outputCurrentColor();

	// Note: the java origin is not the center of the arc's
	// circle as it is in postscript, it is the center of the
	// rectangle which bounds the arc

	output("newpath");
	outputEllipticArc(x, y, width/2, height/2, startAngle, arcAngle);
	output("stroke");

}

/**
 *
 * Device-specific methods
 *
 */
public void fillArcPW(double x, double y, double width, double height,
                             double startAngle, double arcAngle, int type) {

	// Make sure we are in the correct state
	updateContext();

	// set the color
	outputCurrentColor();

	output("newpath");

	// move to java origin
	// Note: the java origin is not the center of the arc's
	// circle as it is in postscript, it is the center of the
	// rectangle which bounds the arc
	output(	truncateDecimalDigits(x + width/2) + " " +
			truncateDecimalDigits(-(y + height/2)) + " moveto");

	outputEllipticArc(x, y, width/2, height/2, startAngle, arcAngle);
	output("closepath fill");
}

protected void outputEllipticArc(double x, double y,
								 double semiMajorX, double semiMajorY,
								 double startAngle, double arcSweep) {

	// Choose the correct keyword based on the direction swept
	String arcWord = (arcSweep >= 0) ? " arc" : " arcn";
	double endAngle = startAngle + arcSweep;

	// if the arc is circular, output the simple case
	if (semiMajorX == semiMajorY) {
		output( truncateDecimalDigits(x + semiMajorX) + " " +
				truncateDecimalDigits(-(y + semiMajorY)) + " " +
				truncateDecimalDigits(semiMajorX) + " " +
				truncateDecimalDigits(startAngle) + " " +
				truncateDecimalDigits(endAngle) + arcWord);
	}

	// otherwise the arc is elliptical, so we have to scale the axes
	else {

		// Move to the center of the arc and scale it
		output(truncateDecimalDigits(x + semiMajorX) + " " +
			   truncateDecimalDigits(-(y + semiMajorY)) + " translate");
		output(truncateDecimalDigits(semiMajorX) + " " +
			   truncateDecimalDigits(semiMajorY) + " scale");

		// Draw the arc
		output("0 0 1 " + truncateDecimalDigits(startAngle) + " " +
			   truncateDecimalDigits(endAngle) + arcWord);

		// Restore the old position
		output(truncateDecimalDigits(1.0 / semiMajorX) + " " +
			   truncateDecimalDigits(1.0 / semiMajorY) + " scale");
		output(truncateDecimalDigits(-(x + semiMajorX)) + " " +
			   truncateDecimalDigits(y + semiMajorY) + " translate");
	}
}

/**
 *
 * Device-specific methods
 *
 */
public  void setClipPW(double x, double y, double width, double height) {
    Rectangle2D clip = (Rectangle2D)getClip();

	// If this is the current context, exit the graphics
	boolean current = isContextCurrent();
	if (current) {
		endContext();
	}

	// Since we are an independent context, we must manually clip to our parent
	if (parent != null && ((Graphics2DPS) parent).clipSet) {

		Rectangle2D intersectionRect = new Rectangle2D.Double();
		Rectangle2D parentRect = (Rectangle2D) parent.getClip();




        try {
            // to get the real parent clip, we first have to
            // put it through the inverse of the transform
            // being used by this Graphics object
            AffineTransform inverseTransform;

            // If parent is frame, the transform owned by this graphics
            // class is independent of its parent.  Therefore, we can just
            // reverse the transformation used by this graphics class to
            // find the parent clip relative to the space being used by
            // this graphics class.
            if (((Graphics2DBase)parent).isFrameRoot()) {

                inverseTransform = transform.createInverse();

            // If parent is not a frame, the transform owned by this
            // graphics class contains the parent transform.  Therefore,
            // we must separate out the bit of the transform that applies only
            // to this class before we can form an inverse transformation
            // that will be used to find the parent clip relative to the
            // space being used by this graphics class.
            } else {
                AffineTransform parentTransform =
                    (AffineTransform) ((Graphics2D)parent).getTransform().clone();
                if (parentTransform.isIdentity()) {
                    inverseTransform = transform.createInverse();
                } else {
                    inverseTransform = parentTransform.createInverse();
                    inverseTransform.concatenate(transform);
                }
            }

            // now that we have the transformation from the parent's space,
            // we just need to apply it to the parent's clip to bring it
            // in line with the space being used by this graphics class

            Shape transformedParentShape =
                    inverseTransform.createTransformedShape(parentRect);

            // find the intersection of the parent clip transformed to
            // this space and the requested new clip
            Rectangle2D.intersect(transformedParentShape.getBounds2D(),
                              new Rectangle2D.Double(x, y, width, height),
                              intersectionRect);

        } catch (NoninvertibleTransformException nte) {
            // Can't invert?  Just go with a straight intersection.
            Rectangle2D.intersect(parentRect,
                              new Rectangle2D.Double(x, y, width, height),
                              intersectionRect);
        }


		// new clip rectangle is intersection of desired clip and parent clip
		x = intersectionRect.getX();
		y = intersectionRect.getY();
		width = intersectionRect.getWidth();
		height = intersectionRect.getHeight();
	}

	// set the new clipping rect
	clip.setRect(x, y, width, height);
	this.clipSet = true;

	// Re-enter the graphics if we've exited it.
	if (current) {
		startContext();
	}
}

/**
 *
 * Device-specific methods
 *
 */
public void fillRectPW(double x, double y, double width, double height) {

	// Make sure we are in the correct state
	updateContext();

	//set the color
	outputCurrentColor();

	output(truncateDecimalDigits(x) + " "
		   + truncateDecimalDigits(-y) + " "
		   + truncateDecimalDigits(width) + " "
		   + truncateDecimalDigits(-height) + " rf");
}

/**
 *
 * Device-specific methods
 *
 */
public void drawRectPW(double x, double y, double width, double height) {

	// Make sure we are in the correct state
	updateContext();

	// set the color
	outputCurrentColor();

	// set the line width
	setLineStroke();

	output(truncateDecimalDigits(x) + " "
		   + truncateDecimalDigits(-y) + " "
		   + truncateDecimalDigits(width) + " "
		   + truncateDecimalDigits(-height) + " rs");
}

/**
 *
 * Device-specific methods
 *
 */
public void drawLinePW(double x1, double y1, double x2, double y2) {

	// Make sure we are in the correct state
	updateContext();

	// set the color
	outputCurrentColor();

	// set the line width
	setLineStroke();

	output(truncateDecimalDigits(x1) + " "
		   + truncateDecimalDigits(-y1) + " "
		   + truncateDecimalDigits(x2) + " "
		   + truncateDecimalDigits(-y2) + " dla");
}

/**
 *
 * Device-specific methods
 *
 */
public void drawRoundRectPW(double x, double y, double width,
		double height, double arcWidth, double arcHeight) {

	drawRRect(x, -y, width, -height, arcWidth, arcHeight, "stroke");
}

/**
 *
 * Device-specific methods
 *
 */
public void fillRoundRectPW(double x, double y, double width,
		double height, double arcWidth, double arcHeight) {

	drawRRect(x, -y, width, -height, arcWidth, arcHeight, "fill");
}

/**
 * The fonts have to be reloaded, so we need to dump the font list
 */
public void flushLoadedFonts() {
	loadedFonts.clear();
}

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
////////////////////              Miscellaneous               ////////////
//////////////////// Some non-Graphics2D non Graphics methods ////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

/**
 *
 * Output raw eps image data
 *
 */
public void epsfOutputFile(BufferedReader reader) {

	try {

		// read it in and spit it out
		String line;
		while ((line = reader.readLine()) != null &&
			   !line.equals("%%EOF")) {
			output(line);
		}
	} catch (IOException e) {
		System.out.println(e.toString());
	}
}

/**
 *
 * Draw an eps image
 *
 */
public boolean drawEPSImage(EPSImage img, int x, int y, int width, int height,
							Color bgcolor, ImageObserver observer) {

	// Initialise the graphics state for image output
	output("PFBeginEpsf");

	// adjust to java origin
	y 		= -y;
	height 	= -height;

	// translate the origin
	output(x + " " + (y + height) + " translate");

	// get dimensions
	Rectangle2D r2d = getEPSBBox(img.getInputSource());
	double llx = r2d.getX();
	double lly = r2d.getY();
	double urx = r2d.getWidth();
	double ury = r2d.getHeight();

	// scale
	output(truncateDecimalDigits(width/(urx - llx)) + " "
		   + truncateDecimalDigits(-height/(ury - lly)) + " scale");

	// translate again
	output(truncateDecimalDigits(-llx) + " "
		   + truncateDecimalDigits(-lly) + " translate");

	// now print out the eps image
	output("%%BeginDocument");
	epsfOutputFile(img.getInputSource());

	// finish
	output("%%EndDocument");
	output("PFEndEpsf");

	return true;
}


/**
 *
 * Append a character to the output buffer.  *
 *
 */
protected void appendToOutString(char code, double width) {
	String specials 	= "\\()";
	char escape_char	= '\\';

	// the length of the output buffer can be wider
	// than the number of characters when we have to
	// convert char's to octal

    // outside the realm of alphabetic characters ...
    if ((int) code < 32 || (int) code > 126) {
        // single-byte character -- write out as octal
        if (code <= 255) {
            out.text.append(octal3(code));

        // multi-byte character
		// intercept Euro character, which is specially located at decimal
		//   200 if the user is using the special Euro encoding vector
		//   from the printer class
		// if this encoding is not set, we still need to intercept the char
		//   because the metrics are retrieved in FontPS for the purposes of
		//   PDF output.  In this case, substitute the "currency" character
		//   at code 164 (decimal)
        } else if (code == 0x20ac) {
			if (printer != null && printer.getCharacterEncoding() ==
					JCPostScriptPrinter.ENCODING_ISO_LATIN_1_EURO) {
            	out.text.append(octal3((char) 128));
			} else {
            	out.text.append(octal3((char) 164));
			}

        // multi-byte character
        // ideally, this should never happen as we don't support
        // multi-byte characters at all
		// since the 'space' character metrics are substituted
		// for unsupported characters, the 'space' character should
		// be subsititued during printing as well
        } else {
            System.err.println("Warning: expecting single-byte character, " +
                "found multi-byte character 0x" + Integer.toHexString(code));
            out.text.append(octal3(' '));
        }
    }
	else {
		// if it's a special char, escape it
		if (specials.indexOf(code) != -1) {
			out.text.append(escape_char);
		}
		out.text.append(code);
	}
	out.width += width;
}

/**
 * Convert a character to a 3-digit octal representation
 * @param code The character to convert
 * @return A string containing the octal triplet
 */
protected String octal3(char code) {

	// Convert the number to octal
	StringBuffer sb = new StringBuffer(Integer.toOctalString(code));

	// pad to 3 digits
	int length = sb.length();
	for (int i = length; i < 3; i++) {
		sb.insert(0, 0);
	}
	sb.insert(0, '\\');

	return(sb.toString());
}

/**
 *
 * Flush the buffer to the file.
 *
 */
protected void printOutString() {
	// output to stnd out for now
	StringBuffer string = new StringBuffer( "(");
	string.append(out.text.toString());
	string.append(") ");
	string.append(truncateDecimalDigits(out.x));
	string.append(" ");
	string.append(truncateDecimalDigits(out.y));
	string.append(" st");

	// write 'er out
	output(string.toString());

	// reset the buffer
	out.x 		+= out.width; // running total
	out.width 	= 0;


	
	out.text.delete(0, out.text.length());
	
}

protected void outputCurrentColor() {

	// Don't output the colour unless we need to
	if (!colorChanged) {
		return;
	}

	// set the color
	Color c = getColor();
	// postscript colors are btw 0-1
	String s = 	truncateDecimalDigits(c.getRed()/255.0) + " " +
				truncateDecimalDigits(c.getGreen()/255.0) + " " +
				truncateDecimalDigits(c.getBlue()/255.0) + " setrgbcolor";
	output(s);

	colorChanged = false;
}

/**
 *
 * Some common code for polygons
 *
 */
protected void printPolygon(int[] xPoints, int[] yPoints, int nPoints,
		String strokeOrFill) {

	// Make sure we are in the correct state
	updateContext();

	// set the color
	outputCurrentColor();

	output("newpath " + xPoints[0] + " " + (-yPoints[0]) + " moveto");

	for (int i = 1; i < nPoints; i++) {
		int x = xPoints[i];
		int y = (int) (-yPoints[i]);
		output(x + " " + y + " lineto");
	}

	// handle winding rule
	if (strokeOrFill.equals("fill")) {
		// fill, set winding rule
		if (getWindingRule() != GeneralPath.WIND_NON_ZERO) {
			strokeOrFill = "eofill";
		}
	}
	output("closepath " + strokeOrFill);
}

/**
 *
 * Some common code for drawing round rectangles.
 *
 */
protected void drawRRect(double x, double y, double width,
		double height, double arcWidth, double arcHeight, String strokeOrFill) {

	double scaleFactor = 1.0;

	// Make sure we are in the correct state
	updateContext();

	// set the color
	outputCurrentColor();

	// set the line width
	setLineStroke();

	// if the arcs are non-circular, establish differential scaling
	if (arcHeight != arcWidth) {
		scaleFactor = arcHeight / arcWidth;
		output("1 " + truncateDecimalDigits(scaleFactor) + " scale");
		y /= scaleFactor;
		height /= scaleFactor;
	}

	output("newpath");
	output(truncateDecimalDigits(x + arcWidth) + " "
		   + truncateDecimalDigits(y) + " moveto");

	output( truncateDecimalDigits(x + width) 	+ " " +
			truncateDecimalDigits(y) 			+ " " +
			truncateDecimalDigits(x + width) 	+ " " +
		 	truncateDecimalDigits(y + height) 	+ " " +
			truncateDecimalDigits(arcWidth)		+ " arct");

	output( truncateDecimalDigits(x + width) 	+ " " +
			truncateDecimalDigits(y + height)	+ " " +
			truncateDecimalDigits(x)		 	+ " " +
		 	truncateDecimalDigits(y + height) 	+ " " +
			truncateDecimalDigits(arcWidth)		+ " arct");

	output( truncateDecimalDigits(x)		 	+ " " +
			truncateDecimalDigits(y + height)	+ " " +
			truncateDecimalDigits(x)		 	+ " " +
		 	truncateDecimalDigits(y) 			+ " " +
			truncateDecimalDigits(arcWidth)		+ " arct");

	output( truncateDecimalDigits(x)		 	+ " " +
			truncateDecimalDigits(y)			+ " " +
			truncateDecimalDigits(x + width) 	+ " " +
		 	truncateDecimalDigits(y) 			+ " " +
			truncateDecimalDigits(arcWidth)		+ " arct");

	// finish it off
	output("closepath " + strokeOrFill);

	// If we have differential scaling, restore the original scale
	if (arcHeight != arcWidth) {
		output("1 " + truncateDecimalDigits(1.0 / scaleFactor) + " scale");
	}
}

// output the line stroke if it has changed
protected void setLineStroke() {

	if (strokeChanged) {

		// set line width
		float width = ((BasicStroke)currentStroke).getLineWidth();
		output(truncateDecimalDigits(width) + " setlinewidth");

		// set the dash-ness
		float dashArray[] 	= ((BasicStroke)currentStroke).getDashArray();
		float dashPhase 	= ((BasicStroke)currentStroke).getDashPhase();

		if (dashArray != null) {

			output("[");
			for (int i = 0; i < dashArray.length; i++) {
				output(" " + truncateDecimalDigits(dashArray[i]) + " ");
			}
			output("] ");

			output(String.valueOf(dashPhase).toString());

			output(" setdash");

			dashMode = true;
		}
		else if (dashMode) {
			// restore solid line, the default
			output("[] 0 setdash");
			dashMode = false;
		}


	}
	strokeChanged = false;
}


protected void emitColorImageProlog(int xdim) {
    output("% Color picture");

    output("% define string to hold a scanline's worth of data");
    output("/pix " + xdim*3);
    output(" string def");

    output("% define space for color conversions");
    output("/grays " + xdim);
    output(" string def  % space for gray scale line");
    output("/npixls 0 def");
    output("/rgbindx 0 def");

    output("% define 'colorimage' if it isn't defined");
    output("%   ('colortogray' and 'mergeprocs' come from xwd2ps");
    output("%     via xgrab)");
    output("/colorimage where   % do we know about 'colorimage'?");
    output("{ pop }           % yes: pop off the 'dict' returned");
    output("{                 % no:  define one");
    output("/colortogray {  % define an RGB->I function");
    output("/rgbdata exch store    % call input 'rgbdata'");
    output("rgbdata length 3 idiv");
    output("/npixls exch store");
    output("/rgbindx 0 store");
    output("0 1 npixls 1 sub {");
    output("grays exch");
    output("rgbdata rgbindx       get 20 mul    % Red");
    output("rgbdata rgbindx 1 add get 32 mul    % Green");
    output("rgbdata rgbindx 2 add get 12 mul    % Blue");
    output("add add 64 idiv      % I = .5G + .31R + .18B");
    output("put");
    output("/rgbindx rgbindx 3 add store");
    output("} for");
    output("grays 0 npixls getinterval");
    output("} bind def");
    output("");
    output("% Utility procedure for colorimage operator.");
    output("% This procedure takes two procedures off the");
    output("% stack and merges them into a single procedure.");
    output("");
    output("/mergeprocs { % def");
    output("dup length");
    output("3 -1 roll");
    output("dup");
    output("length");
    output("dup");
    output("5 1 roll");
    output("3 -1 roll");
    output("add");
    output("array cvx");
    output("dup");
    output("3 -1 roll");
    output("0 exch");
    output("putinterval");
    output("dup");
    output("4 2 roll");
    output("putinterval");
    output("} bind def");
    output("");
    output("/colorimage { % def");
    output("pop pop     % remove 'false 3' operands");
    output("{colortogray} mergeprocs");
    output("image");
    output("} bind def");
    output("} ifelse          % end of 'false' case");

}

/**
 *
 * Draw normalized jpeg/gif image, that is, pixels which encode rgb info.
 * Note that since we allow postscript to do its own scaling, the width
 * and height parameters represent the DESIRED size of the image, not its
 * actual size.
 *
 */
protected boolean drawJavaImage(Image img, int x, int y, int width, int height,
		Color bgcolor, ImageObserver observer) {

    Rectangle2D clip = (Rectangle2D)getClip();
    output("%doImage");
    int imgWidth = img.getWidth(null);
    int imgHeight = img.getHeight(null);
    int clipWidth = (int)clip.getWidth();
    int clipHeight = (int)clip.getHeight();

	// special case -- if we aren't scaling the image at all
	// but the clip is less than the size of the image, truncate
	// the image to the clip size to save space in postscript file
	if (width == imgWidth && height == imgHeight) {
		if (clipWidth < width || clipHeight < height) {
			imgWidth = Math.min(clipWidth, imgWidth);
			imgHeight = Math.min(clipHeight, imgHeight);
			width = imgWidth;
			height = imgHeight;
		}
	}

	int[] pixels = new int[imgWidth * imgHeight];

	// retrieve the image pixel data
	PixelGrabber pg =
		new PixelGrabber(img, 0, 0, imgWidth, imgHeight, pixels, 0, imgWidth);
	try {
		pg.grabPixels();
	} catch (InterruptedException e) {
		System.err.println("interrupted waiting for pixels!");
		return false;
	}
	if ((pg.getStatus() & ImageObserver.ABORT) != 0) {
		System.err.println("image fetch aborted or errored");
		return false;
	}


	output("gsave");

	output("% build a temporary dictionary");
	output("20 dict begin");
	emitColorImageProlog(pg.getWidth());

	output("% lower left corner");
	y = (int) -y;
	output(x + " " + y + " translate");

	// compute image size. First of all, if width or height is 0, image is 1:1.
	if (width == 0 || height == 0) {
		height	= pg.getHeight();
		width 	= pg.getWidth();
    }

    output("% size of image");
    output(width + " " + height + " scale");

    output(pg.getWidth() + " " + pg.getHeight());
    output(" 8");

    output("[" + pg.getWidth() + " 0 0 -" + pg.getHeight() + " 0 " + "0" + "]");

    output("{currentfile pix readhexstring pop}");
    output("false 3 colorimage");
    output("");

    int offset = 0, sleepyet=0;;
    // array to hold a line of pixel data
	int charsPerRow = 12 * 6;
    char[] scanLine = new char[charsPerRow + 1];

	int h = pg.getHeight();
	int w = pg.getWidth();
	for (int i = 0; i < h; i++) {
		offset = 0;
		++sleepyet;
		for (int j = 0; j < w; j++) {
			int n = pixels[i * w + j];

			// put hex chars into string
			scanLine[offset++] = hd[(n & 0xF00000) >> 20];	// red
			scanLine[offset++] = hd[(n & 0xF0000)  >> 16];
			scanLine[offset++] = hd[(n & 0xF000)   >> 12];	// green
			scanLine[offset++] = hd[(n & 0xF00)    >>  8];
			scanLine[offset++] = hd[(n & 0xF0)     >>  4];	// blue
			scanLine[offset++] = hd[(n & 0xF)           ];

			if (offset >= charsPerRow) {
				String s = String.copyValueOf(scanLine, 0, offset);
				output(s);
				if (sleepyet > 5) {
					try {
						// let the screen update occasionally!
						Thread.sleep(15);
					} catch (java.lang.InterruptedException ex) {
						// yeah, so?
					}
					sleepyet = 0;
				}
				offset = 0;
			}
		}

		// print partial rows
		if (offset != 0) {
			String s = String.copyValueOf(scanLine, 0, offset);
			output(s);
		}
	}

    output("");
    output("end");
    output("grestore");

    return true;
}

/**
 *
 * Read the bounding box from an EPS image.
 *
 */
protected Rectangle2D getEPSBBox(BufferedReader reader) {

	// read it in and spit it out
	String bbox = "%%BoundingBox:";
	String line;

	// mark so we can rewind
	try {
		reader.mark(readAheadAmount);

		/*
		// Read the first line of the file and store it.
		line = reader.readLine();
		headers.append(line);

		// Try to rewind the file; if we can't we have to save the headers
		reader.reset();
		line = reader.readLine();
		if (!line.equals(headers.toString())) {
			saveHeaders = true;
			headers.append(line);
		}
		else {
			saveHeaders = false;
			headers.delete(0, headers.length());
		}
		*/

		while ((line = reader.readLine()) != null) {

			/*
			// Store the line we've read if we can't rewind the file
			if (saveHeaders) {
				headers.append(line);
			}
			*/

			// if the line contains the bounding box string and
			// doesn't include placeholder "(atend)" get the bounds
			if (line.indexOf(bbox) > -1 && line.indexOf("(atend)") == -1) {
				int start = line.indexOf(bbox) + bbox.length();
				StringTokenizer st =
					new StringTokenizer(line.substring(start + 1, line.length()));
				double llx = Double.valueOf(st.nextToken()).doubleValue();
				double lly = Double.valueOf(st.nextToken()).doubleValue();
				double urx = Double.valueOf(st.nextToken()).doubleValue();
				double ury = Double.valueOf(st.nextToken()).doubleValue();

				// reset to the mark and return the bounding box
				reader.reset();
				return new Rectangle2D.Double(llx, lly, urx, ury);
			}
		}
	} catch (IOException e) {
		System.out.println(e.toString());
		e.printStackTrace();
	}

	throw new RuntimeException("Bounding box not set for EPS image");
}

/**
 * Overrides create() method.  Set fontWritten to false.  New frames
 * must always have fonts written out.
 */
public Graphics create()
{
	Graphics newGC = super.create();
	((Graphics2DPS)newGC).fontWritten = false;
	return newGC;
}

} // end class
