/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/


// RCSID -- $RCSfile: JCEPSPrinter.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:24 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page.adobe.postscript;

import java.util.Date;

import com.klg.jclass.page.JCPrinter;
import com.klg.jclass.page.FontParser;
import com.klg.jclass.page.Graphics2DBase;

import java.io.OutputStream;
import java.awt.Graphics;
import java.util.Enumeration;




import java.awt.geom.Rectangle2D;


/**
 *
 * Implements the <code>PrinterDriver</code> interface.
 *
 */
public class JCEPSPrinter extends JCPrinter {

static String std_EPS_headers =
			"%!PS-Adobe-3.0 EPSF-3.0\n" 							+
			"%%Creator: PageLayout 6.x";

static String std_EPS_prologue_begin =
			"%%EndComments\n" 										+
			"%%BeginProlog\n" 										+
			"%\n" 													+
			"% This document produced by PageLayout 6.x\n" 			+
			"%\n" 													+
			"% PageLayout is a trademark of Quest Software Inc.\n" 		+
			"% Copyright (c) 2003 by Quest Software Inc.\n"				+
			"% All rights reserved.\n" 								+
			"%\n" 													+
			"%%BeginResource: procset KlgPfglobals\n" 				+
			"	/PFLevel1\n" 										+
			"		/languagelevel where {\n" 						+
			"			pop languagelevel\n" 						+
			"		} {\n" 											+
			"			1\n" 										+
			"		} ifelse\n" 									+
			"		2 lt def\n" 									+
			"	/PFBeginEPS {\n" 									+
			"		/klgeps save def\n"	 							+
			"		KlgPfDict begin\n" 								+
			"	} def\n" 											+
			"	/PFEndEPS {\n" 										+
			"		end\n" 											+
			"		klgeps restore\n"	 							+
			"		showpage\n"										+
			"	} def\n" 											+
			"%%EndResource\n" 										+
			"%%BeginResource: procset KlgPfDict\n" 					+
			"	/KlgPfDict 100 dict dup begin\n" 					+
			"		/bd {\n" 										+
			"			bind def\n" 								+
			"		} bind def\n" 									+
			"		/dla {\n" 										+
			"			newpath 4 2 roll moveto lineto stroke\n" 	+
			"		} bd\n" 										+
			"		/st {\n" 										+
			"			moveto show\n" 								+
			"		} bd\n" 										+
			"		/PFrect {\n" 									+
			"			newpath\n" 									+
			"			4 2 roll moveto\n" 							+
			"			1 index 0 rlineto\n" 						+
			"			0 exch rlineto\n" 							+
			"			neg 0 rlineto\n" 							+
			"			closepath\n" 								+
			"		} bd\n" 										+
			"		/rc {\n" 										+
			"			PFLevel1 {\n" 								+
			"				PFrect\n" 								+
			"				clip\n" 								+
			"				newpath\n" 								+
			"			} {\n" 										+
			"				rectclip\n" 							+
			"			} ifelse\n" 								+
			"		} bd\n" 										+
			"		/rf {\n" 										+
			"			PFLevel1 {\n" 								+
			"				gsave\n" 								+
			"				PFrect\n" 								+
			"				fill\n" 								+
			"				grestore\n" 							+
			"			} {\n" 										+
			"				rectfill\n" 							+
			"			} ifelse\n" 								+
			"		} bd\n" 										+
			"		/rs {\n" 										+
			"			PFLevel1 {\n" 								+
			"				gsave\n" 								+
			"				PFrect\n" 								+
			"				stroke\n" 								+
			"				grestore\n" 							+
			"			} {\n" 										+
			"				rectstroke\n" 							+
			"			} ifelse\n" 								+
			"		} bd\n" 										+
			"		/sf {\n" 										+
			"			PFLevel1 {\n" 								+
			"				exch findfont exch scalefont setfont\n"	+
			"			} {\n" 										+
			"				selectfont\n" 							+
			"			} ifelse\n" 								+
			"		} bd\n" 										+
			"		/iso {		% iso-fontname fontname |- -\n" 			+
			"			findfont\n" 										+
			"			dup length dict begin\n" 							+
			"				{1 index /FID ne {def} {pop pop} ifelse} forall\n"+
			"				/Encoding ISOLatin1Encoding def\n" 				+
			"				currentdict\n" 									+
			"			end\n" 												+
			"			definefont pop\n" 									+
			"		} bd\n" 												+
			"	end def\n" 													+
			"%%EndResource";

static String std_EPS_prologue_end = "%%EndProlog";

/**
 * Creates an EPS printer given an <code>OutputStream</code>.
 * @param os an <code>OutputStream</code> to write the PostScript code to
 */
public JCEPSPrinter(OutputStream os) {

	setOutputStream(os);

	// force fonts to be loaded
	FontPS.getAllFonts();
}

/**
 *
 * Constructor.
 * Allows detailed specification of font creation. The constructor which takes
 * one argument is akin to calling this constructor with these parameters:
 * <pre>
 *		JCPrinter p = new JCEPSPrinter(
 *			someOutputStream,						// the output stream
 *			new com.klg.jclass.page.adobe.postscript.AFMParser(), // the parser
 *			"/com/klg/jclass/page/adobe/fonts.jar", // jar location
 *			".afm",									// file extension
 *			"com.klg.jclass.page.adobe.JCAdobeFontMap");
 * </pre>
 *
 *
 * @param os 				the <code>OutputStream</code> to which formatted output should
 *							be sent
 * @param parser 			the <code>FontParser</code> which creates fonts from afm files
 * @param jarLocation 		the location of the jar containing afm files
 * 							relative to the class loader including the name of
 *							the jar file
 * @param fileExtension		the parser will turn all files contained in
 *							"jarLocation" which end with this extension into
 *							fonts
 * @param fontMapFileName	the user fontmap file; a file containing mappings
 *							between font names and their aliases
 *
 */
public JCEPSPrinter(OutputStream os, FontParser parser,
					String jarLocation, String fileExtension,
					String fontMapFileName) {

	this.os = os;

	// force fonts to be loaded
	FontPS.getAllFonts(parser, jarLocation, fileExtension,fontMapFileName);
}

/**
 * Creates default styles, any other one-time initialization.
 * Builds font families from available fonts.
 */
public void setup() {
	// no EPS-specific actions
}

/**
 * Returns <code>true</code> since EPS can scale images itself.
 * @return <code>true</code> since EPS can scale images itself
 */
public boolean isImageScalable()
{
    return true;
}

/**
 * Initializes the document (EPS document headers, etc.)
 * @param pageSize the size of the default/first page of the document
 */
public void openDocument(Rectangle2D pageSize) {

	super.openDocument(pageSize);

	// print out the post script header
	output(std_EPS_headers);

	// print out date/time
	output("%%CreationDate: " + new Date().toString());

	// print out the bounding box of the EPS
	output("%%BoundingBox: " + ((int) pageSize.getX()) + " " +
		   ((int) pageSize.getY()) + " " + ((int) pageSize.getWidth()) + " " +
		   ((int) pageSize.getHeight()));

	// print out start prologue
	output(std_EPS_prologue_begin);

	// print out end prologue
	output(std_EPS_prologue_end);
	output("PFBeginEPS");

	// get the page dimensions in points
	double x 		= pageSize.getX();
	double y 		= pageSize.getY();
	double width 	= pageSize.getWidth();
	double height	= pageSize.getHeight();

	// Transform the co-ordinate system for drawing
	output(x + " " + (height + y) + " translate");
	output("0.0 0.0 " + width + " " + (-height) + " rc");
}

/**
 * Completes the document (removes the dictionary and draw).
 * @param pageSize the size of the default/last page of the document
 */
public void closeDocument(Rectangle2D pageSize) {
	output("PFEndEPS");
	output("%%EOF");

	super.closeDocument(pageSize);
}

/**
 * Defines <code>beginPage()</code> to be a no-op for EPS.
 * @param pageSize the dimensions of the new page
 * @param pageNumber
 */
public void beginPage(Rectangle2D pageSize, int pageNumber) {
	super.beginPage(pageSize, pageNumber);
}

/**
 * Defines <code>endPage()</code> to be a no-op for EPS.
 * @param pageSize the dimensions of the current page
 */
public void endPage(Rectangle2D pageSize) {
	super.endPage(pageSize);
}

/**
 * Gets a Graphics that draws to this printer.
 * @return a Graphics object instantiated for this printer
 */
public Graphics getGraphics() {
	if (graphics == null) {
		graphics = new Graphics2DPS(os);
	}
	return (Graphics) graphics;
}

} // end class
