/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCPostScriptPrinter.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:24 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page.adobe.postscript;

import java.util.Date;

import com.klg.jclass.page.JCPrinter;
import com.klg.jclass.page.FontParser;
import com.klg.jclass.page.Graphics2DBase;

import java.io.OutputStream;
import java.awt.Graphics;
import java.util.Enumeration;




import java.awt.geom.Rectangle2D;


/**
 *
 * Implements the <code>PrinterDriver</code> interface.
 *
 */
public class JCPostScriptPrinter extends JCPrinter {

public final static int ENCODING_ISO_LATIN_1 = 0;
public final static int ENCODING_ISO_LATIN_1_EURO = 1;

static String std_PS_headers =
			"%!PS-Adobe-3.0\n" 										+
			"%%PageOrder: Ascend\n" 								+
			"%%DocumentData: Clean7Bit\n" 							+
			"%%Pages: (atend)\n" 									+
			"%%BoundingBox: (atend)\n" 								+
			"%%Creator: PageLayout 6.x";

static String std_PS_prologue_begin =
			"%%EndComments\n" 										+
			"%%BeginProlog\n" 										+
			"%\n" 													+
			"% This document produced by PageLayout 6.x\n" 	+
			"%\n" 													+
			"% PageLayout is a trademark of Quest Software Inc.\n" 		+
			"% Copyright (c) 2003 by Quest Software Inc.\n"				+
			"% All rights reserved.\n" 								+
			"%\n" 													+
			"%%BeginResource: procset KlgPfglobals\n" 				+
			"	/PFLevel1\n" 										+
			"		/languagelevel where {\n" 						+
			"			pop languagelevel\n" 						+
			"		} {\n" 											+
			"			1\n" 										+
			"		} ifelse\n" 									+
			"		2 lt def\n" 									+
			"	/PFBeginPage {\n" 									+
			"		/pgsave save def\n" 							+
			"		KlgPfDict begin\n" 								+
			"	} def\n" 											+
			"	/PFEndPage {\n" 									+
			"		end\n" 											+
			"		pgsave restore\n" 								+
			"		showpage\n" 									+
			"	} def\n" 											+
			"	/PfOutputError {\n" 								+
			"		dup = flush\n" 									+
			"		showpage\n" 									+
			"		/Helvetica findfont 12 scalefont setfont\n" 	+
			"		72 200 moveto show\n" 							+
			"		showpage\n" 									+
			"		quit\n" 										+
			"	} def\n" 											+
			"	/PFEpsfError {\n" 										+
			"		(Command X must not be called from an EPS file)\n" 	+
			"		(X) search pop dup length index 4 putinterval\n" 	+
			"		dup length index 3 putinterval\n" 					+
			"		4 1 roll pop pop pop\n" 							+
			"		PFOutputError\n" 								+
			"	} def\n" 											+
			"	/PFBeginEpsf {\n" 									+
			"		end\n" 											+
			"		/Epsfsave save def\n" 							+
			"		/dict_count countdictstack def\n" 				+
			"		/op_count count 1 sub def\n" 					+
			"		userdict begin\n" 								+
			"		/showpage {} def\n" 							+
			"		/banddevice {(banddevice) PFEpsfError} def\n"	+
			"		/clear {(clear) PFEpsfError} def\n" 			+
			"		/cleardictstack {(cleardictstack) PFEpsfError} def\n"	+
			"		/copypage {(copypage) PFEpsfError} def\n" 				+
			"		/erasepage {(erasepage) PFEpsfError} def\n" 			+
			"		/exitserver {(exitserver) PFEpsfError} def\n" 			+
			"		/framedevice {(framedevice) PFEpsfError} def\n" 		+
			"		/grestoreall {(grestoreall) PFEpsfError} def\n" 		+
			"		/initclip {(initclip) PFEpsfError} def\n" 				+
			"		/initgraphics {(initgraphics) PFEpsfError} def\n" 		+
			"		/initmatrix {(initmatrix) PFEpsfError} def\n" 			+
			"		/quit {(quit) PFEpsfError} def\n" 						+
			"		/renderbands {(renderbands) PFEpsfError} def\n" 		+
			"		/setglobal {(setglobal) PFEpsfError} def\n" 			+
			"		/setpagedevice {(setpagedevice) PFEpsfError} def\n" 	+
			"		/setshared {(setshared) PFEpsfError} def\n" 			+
			"		/startjob {(startjob) PFEpsfError} def\n" 				+
			"		/lettertray {(lettertray) PFEpsfError} def\n" 			+
			"		/letter {(letter) PFEpsfError} def\n" 					+
			"		/lettersmall {(lettersmall) PFEpsfError} def\n" 		+
			"		/11x17tray {(11x17tray) PFEpsfError} def\n" 			+
			"		/11x17 {(11x17) PFEpsfError} def\n" 					+
			"		/ledgertray {(ledgertray) PFEpsfError} def\n" 			+
			"		/ledger {(ledger) PFEpsfError} def\n" 					+
			"		/legaltray {(legaltray) PFEpsfError} def\n" 			+
			"		/legal {(legal) PFEpsfError} def\n" 					+
			"		/statementtray {(statementtray) PFEpsfError} def\n" 	+
			"		/statement {(statement) PFEpsfError} def\n" 			+
			"		/executivetray {(executivetray) PFEpsfError} def\n" 	+
			"		/executive {(executive) PFEpsfError} def\n" 			+
			"		/a3tray {(a3tray) PFEpsfError} def\n" 					+
			"		/a3 {(a3) PFEpsfError} def\n" 							+
			"		/a4tray {(a4tray) PFEpsfError} def\n" 					+
			"		/a4 {(a4) PFEpsfError} def\n" 							+
			"		/a4small {(a4small) PFEpsfError} def\n" 				+
			"		/b4tray {(b4tray) PFEpsfError} def\n" 					+
			"		/b4 {(b4) PFEpsfError} def\n" 							+
			"		/b5tray {(b5tray) PFEpsfError} def\n" 					+
			"		/b5 {(b5) PFEpsfError} def\n" 							+
			"		0 setgray\n" 											+
			"		0 setlinecap\n" 										+
			"		1 setlinewidth\n" 										+
			"		0 setlinejoin\n" 										+
			"		10 setmiterlimit\n" 									+
			"		[ ] 0 setdash\n" 										+
			"		newpath\n" 												+
			"		PFLevel1 not {\n" 										+
			"			false setoverprint false setstrokeadjust\n" 		+
			"		} if\n" 												+
			"	} bind def\n" 												+
			"	/PFEndEpsf {\n" 											+
			"		count op_count sub { pop } repeat\n" 					+
			"		countdictstack dict_count sub { end } repeat\n" 		+
			"		Epsfsave restore\n" 									+
			"		KlgPfDict begin\n" 										+
			"	} bind def\n" 												+
			"%%EndResource\n" 												+
			"%%BeginResource: procset KlgPfDict\n" 					+
			"	/KlgPfDict 100 dict dup begin\n" 					+
			"		/bd {\n" 										+
			"			bind def\n" 								+
			"		} bind def\n" 									+
			"		/dla {\n" 										+
			"			newpath 4 2 roll moveto lineto stroke\n" 	+
			"		} bd\n" 										+
			"		/st {\n" 										+
			"			moveto show\n" 								+
			"		} bd\n" 										+
			"		/PFrect {\n" 									+
			"			newpath\n" 									+
			"			4 2 roll moveto\n" 							+
			"			1 index 0 rlineto\n" 						+
			"			0 exch rlineto\n" 							+
			"			neg 0 rlineto\n" 							+
			"			closepath\n" 								+
			"		} bd\n" 										+
			"		/rc {\n" 										+
			"			PFLevel1 {\n" 								+
			"				PFrect\n" 								+
			"				clip\n" 								+
			"				newpath\n" 								+
			"			} {\n" 										+
			"				rectclip\n" 							+
			"			} ifelse\n" 								+
			"		} bd\n" 										+
			"		/rf {\n" 										+
			"			PFLevel1 {\n" 								+
			"				gsave\n" 								+
			"				PFrect\n" 								+
			"				fill\n" 								+
			"				grestore\n" 							+
			"			} {\n" 										+
			"				rectfill\n" 							+
			"			} ifelse\n" 								+
			"		} bd\n" 										+
			"		/rs {\n" 										+
			"			PFLevel1 {\n" 								+
			"				gsave\n" 								+
			"				PFrect\n" 								+
			"				stroke\n" 								+
			"				grestore\n" 							+
			"			} {\n" 										+
			"				rectstroke\n" 							+
			"			} ifelse\n" 								+
			"		} bd\n" 										+
			"		/sf {\n" 										+
			"			PFLevel1 {\n" 								+
			"				exch findfont exch scalefont setfont\n"	+
			"			} {\n" 										+
			"				selectfont\n" 							+
			"			} ifelse\n" 								+
			"		} bd\n" 										+
			"		/iso {		% iso-fontname fontname |- -\n" 			+
			"			findfont\n" 										+
			"			dup length dict begin\n" 							+
			"				{1 index /FID ne {def} {pop pop} ifelse} forall";

// standard character encoding
static String std_PS_prologue_encoding_isolatin =
			"				/Encoding ISOLatin1Encoding def";

// custom character encoding including Euro character at decimal 128
static String std_PS_prologue_encoding_euro =
			"                /Encoding [\n" +
			"                  /.notdef /.notdef /.notdef /.notdef /.notdef\n" +
			"                  /.notdef /.notdef /.notdef /.notdef /.notdef\n" +
			"                  /.notdef /.notdef /.notdef /.notdef /.notdef\n" +
			"                  /.notdef /.notdef /.notdef /.notdef /.notdef\n" +
			"                  /.notdef /.notdef /.notdef /.notdef /.notdef\n" +
			"                  /.notdef /.notdef /.notdef /.notdef /.notdef\n" +
			"                  /.notdef /.notdef /space /exclam /quotedbl\n" +
			"                  /numbersign /dollar /percent /ampersand\n" +
			"                  /quoteright /parenleft /parenright /asterisk\n" +
			"                  /plus /comma /minus /period /slash /zero\n" +
			"                  /one /two /three /four /five /six /seven\n" +
			"                  /eight /nine /colon /semicolon /less /equal\n" +
			"                  /greater /question /at /A /B /C /D /E /F /G\n" +
			"                  /H /I /J /K /L /M /N /O /P /Q /R /S /T /U /V\n" +
			"                  /W /X /Y /Z /bracketleft /backslash\n" +
			"                  /bracketright /asciicircum /underscore\n" +
			"                  /quoteleft /a /b /c /d /e /f /g /h /i /j /k\n" +
			"                  /l /m /n /o /p /q /r /s /t /u /v /w /x /y /z\n" +
			"                  /braceleft /bar /braceright /asciitilde\n" +
			"                  /.notdef /Euro /.notdef /.notdef /.notdef\n" +
			"                  /.notdef /.notdef /.notdef /.notdef /.notdef\n" +
			"                  /.notdef /.notdef /.notdef /.notdef /.notdef\n" +
			"                  /.notdef /.notdef /dotlessi /grave /acute\n" +
			"                  /circumflex /tilde /macron /breve /dotaccent\n" +
			"                  /dieresis /.notdef /ring /cedilla /.notdef\n" +
			"                  /hungarumlaut /ogonek /caron /space\n" +
			"                  /exclamdown /cent /sterling /currency /yen\n" +
			"                  /brokenbar /section /dieresis /copyright\n" +
			"                  /ordfeminine /guillemotleft /logicalnot\n" +
			"                  /hyphen /registered /macron /degree\n" +
			"                  /plusminus /twosuperior /threesuperior\n" +
			"                  /acute /mu /paragraph /periodcentered\n" +
			"                  /cedilla /onesuperior /ordmasculine\n" +
			"                  /guillemotright /onequarter /onehalf\n" +
			"                  /threequarters /questiondown /Agrave\n" +
			"                  /Aacute /Acircumflex /Atilde /Adieresis\n" +
			"                  /Aring /AE /Ccedilla /Egrave /Eacute\n" +
			"                  /Ecircumflex /Edieresis /Igrave /Iacute\n" +
			"                  /Icircumflex /Idieresis /Eth /Ntilde\n" +
			"                  /Ograve /Oacute /Ocircumflex /Otilde\n" +
			"                  /Odieresis /multiply /Oslash /Ugrave\n" +
			"                  /Uacute /Ucircumflex /Udieresis /Yacute\n" +
			"                  /Thorn /germandbls /agrave /aacute\n" +
			"                  /acircumflex /atilde /adieresis /aring\n" +
			"                  /ae /ccedilla /egrave /eacute /ecircumflex\n" +
			"                  /edieresis /igrave /iacute /icircumflex\n" +
			"                  /idieresis /eth /ntilde /ograve /oacute\n" +
			"                  /ocircumflex /otilde /odieresis /divide\n" +
			"                  /oslash /ugrave /uacute /ucircumflex\n" +
			"                  /udieresis /yacute /thorn /ydieresis\n" +
			"                ] def";

static String std_PS_prologue_after_encoding =
			"				currentdict\n" 									+
			"			end\n" 												+
			"			definefont pop\n" 									+
			"		} bd\n" 												+
			"	end def\n" 													+
			"%%EndResource";

static String std_PS_prologue_end = "%%EndProlog";

protected int characterEncoding = ENCODING_ISO_LATIN_1;

/**
 * Creates a PostScript printer given an <code>OutputStream</code>.
 * <P>
 * <b>Note:</b> For large documents and for optimal performance, wrap
 * the output stream in a <code>BufferedOutputStream</code>. For example:
 * <pre>
 * 		BufferedOutputStream bos 	= new BufferedOutputStream(os, 2048);
 * 		JCPrinter printer 			= new JCPostScriptPrinter(bos);
 * </pre>
 *
 * @param os an <code>OutputStream</code> to write the PostScript code to
 */
public JCPostScriptPrinter(OutputStream os) {

	setOutputStream(os);

	// force fonts to be loaded
	FontPS.getAllFonts();
}

/**
 *
 * Constructor.
 * Allows detailed specification of font creation. The constructor which takes
 * one argument is akin to calling this constructor with these parameters:
 * <pre>
 *		JCPrinter p = new JCPostScriptPrinter(
 *				someOutputStream,						// the output stream
 *				new com.klg.jclass.page.adobe.postscript.AFMParser(), // the parser
 *				"/com/klg/jclass/page/adobe/fonts.jar", // jar location
 *				".afm",									// file extension
 *				"com.klg.jclass.page.adobe.JCAdobeFontMap");
 * </pre>
 *
 * <P>
 * <b>Note:</b> For large documents and for optimal performance, wrap
 * the output stream in a <code>BufferedOutputStream</code>. For example:
 * <pre>
 * 		BufferedOutputStream bos 	= new BufferedOutputStream(os, 2048);
 * 		JCPrinter printer 			= new JCPostScriptPrinter(bos, ...);
 * </pre>
 *
 * @param os 			the <code>OutputStream</code> to which formatted output should be sent
 * @param parser 		the <code>FontParser</code> which creates fonts from afm files
 * @param jarLocation 	the location of the jar containing afm files relative
 *						to the class loader including the name of the jar file
 * @param fileExtension	the parser will turn files contained in "jarLocation"
 *						ending with this extension into fonts
 * @param fontMapFileName the user fontmap file; a file containing mappings
 *						between font names and their aliases
 *
 *
 */
public JCPostScriptPrinter(OutputStream os, FontParser parser,
						   String jarLocation, String fileExtension,
						   String fontMapFileName) {

	this.os = os;

	// force fonts to be loaded
	FontPS.getAllFonts(parser, jarLocation, fileExtension,fontMapFileName);
}

/**
 * Creates default styles, any other one-time initialization.
 * Builds font families from available fonts.
 */
public void setup() {
	// no PS-specific actions
}

/**
 * Returns <code>true</code> since PostScript can scale images itself.
 * @return <code>true</code> since PostScript can scale images itself
 */
public boolean isImageScalable()
{
	return true;
}

/**
 * Sets the character encoding vector to use for PostScript output.
 * Possible values are <code>ENCODING_ISO_LATIN_1</code> (self-explanatory) or
 * <code>ENCODING_ISO_LATIN_1_EURO</code> (the ISO Latin 1 set with the Euro
 * character added at decimal 128).
 * @param encoding the character encoding vector to use
 */
public void setCharacterEncoding(int encoding)
{
	if (encoding != ENCODING_ISO_LATIN_1 &&
		encoding != ENCODING_ISO_LATIN_1_EURO) {
		throw new IllegalArgumentException("PostScript Character Encoding"+
				" must be one of ENCODING_ISO_LATIN_1 or" +
				" ENCODING_ISO_LATIN_1_EURO");
	}
	characterEncoding = encoding;
}

/**
 * Gets the character encoding vector being used for PostScript output.
 * Possible values are <code>ENCODING_ISO_LATIN_1</code> (self-explanatory) or
 * <code>ENCODING_ISO_LATIN_1_EURO</code> (the ISO Latin 1 set with the Euro
 * character added at decimal 128).
 * @return the character encoding vector being used
 */
public int getCharacterEncoding()
{
	return characterEncoding;
}

/**
 * Initializes the document (PostScript document headers, etc.)
 * @param pageSize the size of the default/first page of the document
 */
public void openDocument(Rectangle2D pageSize) {

	super.openDocument(pageSize);

	// print out the post script header
	output(std_PS_headers);

	// print out date/time
	output("%%CreationDate: " + new Date().toString());

	// print out start prologue
	output(std_PS_prologue_begin);

	// is user wants to use Euro character, must have character encoding
	// properly set (sadly, it adds about 4k to the output file, which
 	// is why we don't just do it automatically)
	if (characterEncoding == ENCODING_ISO_LATIN_1_EURO) {
		output(std_PS_prologue_encoding_euro);
	} else {
		output(std_PS_prologue_encoding_isolatin);
	}

	output(std_PS_prologue_after_encoding);

	// print out end prologue
	output(std_PS_prologue_end);
}

/**
 * Completes the document (output the DSC trailer).
 * @param pageSize the size of the default/last page of the document
 */
public void closeDocument(Rectangle2D pageSize) {
	output("%%Trailer");

	output("%%Pages: " + pageCount);
	output("%%EOF");
	((Graphics2DPS)graphics).setFinishedOutput(true);

	super.closeDocument(pageSize);

	// Discard the graphics used to draw the document
	graphics.dispose();
}

/**
 * Outputs the PostScript code necessary to start a page.
 * (Document Structuring Conventions comments, page orientation, etc.)
 * @param pageSize the dimensions of the new page
 * @param pageNumber
 */
public void beginPage(Rectangle2D pageSize, int pageNumber) {

	super.beginPage(pageSize, pageNumber);

	// Initialise the set of loaded fonts (reload per page)
	((Graphics2DPS) this.getGraphics()).flushLoadedFonts();

	// get the page dimensions in points
	double x 		= pageSize.getX();
	double y 		= pageSize.getY();
	double width 	= pageSize.getWidth();
	double height	= pageSize.getHeight();

	// Print out DSC comments
	output("%%Page: " + pageNumber + " " + (++pageCount));

	output("%%BeginPageSetup");
	output("PFBeginPage");

	output("%%EndPageSetup");

	// handle landscape, auto detect by determining if
	// the width is wider than the height
	if (isLandscape(width, height)) {
		output("90 rotate");
		y = - y;
	}
	else {
		// put the y origin at the top left corner
		y = height + y;
	}

	output(x + " " + y + " translate");
	output("0.0 0.0 " + width + " " + (-height) + " rc");
}

/**
 * End the page; outputs the appropriate DSC comments, etc.
 * @param pageSize the dimensions of the current page
 */
public void endPage(Rectangle2D pageSize) {

	/* Print out DSC comments */
	output("%%PageTrailer");
	output("PFEndPage");

	/* Force the colour and line width to be set again on the next page.
	old_width = -1.0;
	old_colour = NULL;
	*/

	super.endPage(pageSize);
}

/**
 * Gets a <code>Graphics</code> that draws to this printer.
 * @return a <code>Graphics</code> object instantiated for this printer
 */
public Graphics getGraphics() {
	if (graphics == null) {
		graphics = new Graphics2DPS(os, this);

		// Mark the graphics as a root frame
		((Graphics2DBase) graphics).setFrameRoot();
	}
	return (Graphics) graphics;
}

} // end class
