/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: TTFParser.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:24 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page.adobe.postscript;

import java.io.IOException;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.OutputStream;
import java.util.SortedSet;
import java.awt.Font;

import com.klg.jclass.page.FontParser;
import com.klg.jclass.page.FontLibrary;
import com.klg.jclass.page.FontFileLookup;
//import com.klg.jclass.page.adobe.postscript.FontPS;
//import com.klg.jclass.page.adobe.postscript.CharMetricPS;
//import com.klg.jclass.page.adobe.postscript.PairKern;
import com.klg.jclass.page.ttf.*;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;
import java.util.List;

/**
 * This class is a bit different from the usual font file parser in that
 * it defers the parsing of TTF files to a third party, then references
 * that third party's image of the TTF file when prompted by the customer.
 */
public class TTFParser implements FontParser {

/** Stores font file read from file system.  This should only be non-null
    during the initial parsing of the font and if the font should be kept
    for embedding.
 */
public FontFileReader fontFileReader = null;

/**
 * Stores parsed TrueType file information such as metrics, unicode maps, etc.
 */
public TTFFile ttfFile = null;

/**
 * Constructor
 * Call parse() to parse the file.
 */
public TTFParser()
{
}

/**
 * Constructor
 * Calls parse() for you.
 *
 * @param ttfFileName a String file name.
 * @exception FileNotFoundException If a file is not found.
 * @exception IOException If there is an error.
 */
public TTFParser(String ttfFileName) throws FileNotFoundException, IOException
{
    FileInputStream fis = new FileInputStream(ttfFileName);
    parse(fis);
}

/**
 * Parse the file and build the in-memory image.
 *
 * @param inputStream the input stream containing the font file
 * @return the FontPS object created by reading this file.
 * @exception IOException If there is an error.
 */
public Vector parse(InputStream inputStream) throws IOException
{
	return parse(inputStream, (String) null);
}

/**
 * Parse the file and build the in-memory image.
 *
 * @param inputStream the input stream containing the font file
 * @param fontName name of font that is residing in input stream.  Necessary
 *                 for .ttc streams that contain multiple fonts.
 * @return the FontPS object created by reading this file.
 * @exception IOException If there is an error.
 */
public Vector parse(InputStream inputStream, String fontName)
												throws IOException
{
	return parse(inputStream, fontName, -1, false);
}

/**
 * Parse the file and build the in-memory image.
 * This method is preferred over the the other parse()
 * methods because it takes a FontFileLookup object, which can
 * be used to give more informative error messages.
 *
 * @param fontFile  FontFileLookup object encapsulating location of font file.
 * @param fontName name of font that is residing in input stream.  Necessary
 *                 for .ttc streams that contain multiple fonts.
 * @param keepFontFile true if the underling FontFileReader object should
 *                     be kept after the parsing is done (for things like
 *                     embedding the font).  False if it should be discarded.
 * @return a Vector of FontPS objects created by reading this file.
 * @exception IOException If there is an error.
 */
public Vector parse(FontFileLookup fontFile, String fontName,
                                boolean keepFontFile) throws IOException
{
    InputStream inputStream = fontFile.openInputStream();
    int streamLength = (int) fontFile.getFileLength();

    ttfFile = new TTFFile();
    ttfFile.setFontFileName(fontFile.toString());
    FontFileReader reader = new FontFileReader(inputStream, streamLength);
    // FIX ME: If keepFontFile is false the first time the font is
    // referenced, but changes to true later, the font file won't be
    // reloaded.  For now, keepFontFile is ignored and always considered
    // to be true.
    //if (keepFontFile)
        fontFileReader = reader;

    ttfFile.readFont(reader, fontName);

    if (!FontLibrary.getSuppressErrorMessages()) {
        if (ttfFile.getStatus() != TTFFile.PARSE_OK) {
            System.err.println(ttfFile.getStatusMessage());
        }
    }

    // Fatal error - do not return a font
    if (ttfFile.getStatus() == TTFFile.PARSE_FATAL) {
        return new Vector();
    }

    return populateFont(ttfFile);
}

/**
 * Parse the file and build the in-memory image.
 *
 * @param inputStream  the input stream containing the font file
 * @param fontName name of font that is residing in input stream.  Necessary
 *                 for .ttc streams that contain multiple fonts.
 * @param streamLength the length of the stream being read.
 * @param keepFontFile true if the underling FontFileReader object should
 *                     be kept after the parsing is done (for things like
 *                     embedding the font).  False if it should be discarded.
 * @return a Vector of FontPS objects created by reading this file.
 * @exception IOException If there is an error.
 */
public Vector parse(InputStream inputStream, String fontName,
							int streamLength, boolean keepFontFile)
												throws IOException
{
    if (inputStream == null) {
        throw new NullPointerException("parse(): Input stream is null");
    }

    ttfFile = new TTFFile();
    FontFileReader reader = new FontFileReader(inputStream, streamLength);
	// FIX ME: If keepFontFile is false the first time the font is
	// referenced, but changes to true later, the font file won't be
	// reloaded.  For now, keepFontFile is ignored and always considered
	// to be true.
	//if (keepFontFile)
		fontFileReader = reader;
    ttfFile.readFont(reader, fontName);

    if (!FontLibrary.getSuppressErrorMessages()) {
        if (ttfFile.getStatus() != TTFFile.PARSE_OK) {
            System.err.println(ttfFile.getStatusMessage());
        }
    }

    // Fatal error - do not return a font
    if (ttfFile.getStatus() == TTFFile.PARSE_FATAL) {
        return new Vector();
    }

    return populateFont(ttfFile);
}

protected Vector populateFont(TTFFile ttfFile)
{
    // the default font size for fonts retrieved using Java's
    // getAllFonts method is a point size of one
    FontPS font = new FontPS("uninitialized", Font.PLAIN, 1);
	font.ttfParser = this;

	font.usingTTF = true;

	// Get the values required by FontPS from the global font information
	// in the TTF File

	font.setFontName(ttfFile.getFontName());
	font.setFullName(ttfFile.getFullName());
	font.setFamilyName(ttfFile.getFamilyName());
	font.setNotice(ttfFile.getNotice());
	font.setItalicAngle((float)ttfFile.getItalicAngleValue());
	font.setFixedPitch(ttfFile.isFixedPitch());
	font.setSerif(ttfFile.isSerif());
	font.setUnderlinePosition(ttfFile.getUnderlinePosition());
	font.setUnderlineThickness(ttfFile.getUnderlineThickness());
	font.setVersion(Double.toString(ttfFile.getVersion()));

	// Encoding Scheme for TTF is Unicode
	font.setEncodingScheme("Unicode");

	// weight not used for TTF
	font.setWeight("");

	// vertical/horizontal stem width not used by TTF
	font.setStdHW(0);
	font.setStdVW(0);

	int[] bbox = ttfFile.getFontBBox();
	font.getFontBBox().llx = bbox[0];
	font.getFontBBox().lly = bbox[1];
	font.getFontBBox().urx = bbox[2];
	font.getFontBBox().ury = bbox[3];

	font.setCapHeight(ttfFile.getCapHeight());
	font.setXHeight(ttfFile.getXHeight());
	font.setDescender(ttfFile.getLowerCaseDescent());
	font.setAscender(ttfFile.getLowerCaseAscent());

	// TrueType doesn't seem to have a notion of the full height of
	// a character, including leading -- calculate it here
	// (With Type 1 fonts, it's a simple value of 1000.)

	int fullHeight = (int) ((font.getAscender() - font.getDescender()) * 1.15);
	font.setFullHeight(fullHeight);

    // even though we only read one font from a ttf file, we return it in
    // a vector for compatiblity with pcl fonts which can have
    // multiple fonts per tfm file
    Vector v = new Vector(1);
    v.addElement(font);
    return v;
}

/**
 * Parse a TTF file for the font names it contains.
 * This method is preferred over the the other parseFontNames()
 * method because it takes a FontFileLookup object, which can
 * be used to give more informative error messages.
 *
 * @param ttfFile  FontFileLookup object encapsulating location of font file.
 * @return a List of font names contained within the specified TTF file.
 * @exception IOException If there is an error.
 */
public List parseFontNames(FontFileLookup ttfFile)
                                                throws IOException
{
    InputStream inputStream = ttfFile.openInputStream();
    int streamLength = (int) ttfFile.getFileLength();

    TTFFile ttfParser = new TTFFile();
    ttfParser.setFontFileName(ttfFile.toString());
    FontFileReader reader = new FontFileReader(inputStream, streamLength);
    List fontNames = ttfParser.readFontNames(reader);

    if (!FontLibrary.getSuppressErrorMessages()) {
        if (ttfParser.getStatus() != TTFFile.PARSE_OK) {
            System.err.println(ttfParser.getStatusMessage());
        }
    }

    inputStream.close();

    return fontNames;
}

/**
 * Parse a TTF file for the font names it contains.
 *
 * @param inputStream  the input stream containing the file
 * @param streamLength the length of the stream being read
 * @return a List of font names contained within the specified TTF file.
 * @exception IOException If there is an error.
 */
public List parseFontNames(InputStream inputStream, int streamLength)
												throws IOException
{
    TTFFile ttfParser = new TTFFile();
    FontFileReader reader = new FontFileReader(inputStream, streamLength);
    List fontNames = ttfParser.readFontNames(reader);

    if (!FontLibrary.getSuppressErrorMessages()) {
        if (ttfParser.getStatus() != TTFFile.PARSE_OK) {
            System.err.println(ttfParser.getStatusMessage());
        }
    }

	return fontNames;
}

/**
 * Parse a TTF file for all of the font names it contains.
 * This method is preferred over the the other parseAllFontNames()
 * method because it takes a FontFileLookup object, which can
 * be used to give more informative error messages.
 *
 * @param ttfFile  FontFileLookup object encapsulating location of font file.
 * @return a List of TTFFontNames objects that list the
 *         various font names used to refer to the font contained
 *         within the specified TTF file.
 * @exception IOException If there is an error.
 */
public List parseAllFontNames(FontFileLookup ttfFile) throws IOException
{
    InputStream inputStream = ttfFile.openInputStream();
    int streamLength = (int) ttfFile.getFileLength();

    TTFFile ttfParser = new TTFFile();
    ttfParser.setFontFileName(ttfFile.toString());
    FontFileReader reader = new FontFileReader(inputStream, streamLength);
    List fontNames = ttfParser.readAllFontNames(reader);

    if (!FontLibrary.getSuppressErrorMessages()) {
        if (ttfParser.getStatus() != TTFFile.PARSE_OK) {
            System.err.println(ttfParser.getStatusMessage());
        }
    }

    inputStream.close();

    return fontNames;
}

/**
 * Parse a TTF file for all of the font names it contains.
 *
 * @param inputStream  the input stream containing the file
 * @param streamLength the length of the stream being read
 * @return a List of TTFFontNames objects that list the
 *         various font names used to refer to the font contained
 *         within the specified TTF file.
 * @exception IOException If there is an error.
 */
public List parseAllFontNames(InputStream inputStream, int streamLength)
    throws IOException {
    TTFFile ttfParser = new TTFFile();
    FontFileReader reader = new FontFileReader(inputStream, streamLength);
    List fontNames = ttfParser.readAllFontNames(reader);

    if (!FontLibrary.getSuppressErrorMessages()) {
        if (ttfParser.getStatus() != TTFFile.PARSE_OK) {
            System.err.println(ttfParser.getStatusMessage());
        }
    }

    return fontNames;
}
    
/**
 * Create the CharMetricPS class for the passed character code by
 * inspecting the TTFFile class for correct character metrics, including
 * kerning.  Note: this method may only be called after parse() has
 * been successfully executed.  The returned CharMetricPS class will
 * may be null, if the glyph for the requested character does not exist
 * within the font file.
 */
public CharMetricPS createCharMetric(int code)
{
	// first, find the metrics for the glyph associated with this unicode value
	TTFMtxEntry ttfMtx = ttfFile.getGlyphMetricsByUnicode(code);

	// if no metrics exist, then the glyph is not in this font
	// return null metrics and let the calling class decide what to do
	if (ttfMtx == null) {
		return null;
	}

	return createCharMetric(code, ttfMtx);
}

/**
 * Create the CharMetricPS class for the passed glyph code by
 * inspecting the TTFFile class for correct glyph metrics, including
 * kerning.  Note: this method may only be called after parse() has
 * been successfully executed.  The returned CharMetricPS class will
 * may be null, if the glyph for the requested character does not exist
 * within the font file.  The code member of the CharMetrics class
 * will be set with the glyph code, but it should be noted that
 * the correct use of this field is for the character code.  The calling
 * class should be sure to set this field appropriately.
 */
public CharMetricPS createCharMetricFromGlyph(int glyph)
{
	TTFMtxEntry ttfMtx = ttfFile.getGlyphMetricsByGlyphCode(glyph);
	// if no metrics exist, then the glyph is not in this font
	// return null metrics and let the calling class decide what to do
	if (ttfMtx == null) {
		return null;
	}

	return createCharMetric(glyph, ttfMtx);
}

/**
 * Create the CharMetricPS class for the passed character or glyph code from
 * the passed metrics.
 */
public CharMetricPS createCharMetric(int code, TTFMtxEntry ttfMtx)
{
	// initialize the metric class for this character
	CharMetricPS metric = new CharMetricPS();
	metric.setCode(code);

	// if metrics found, populate our new CharMetricPS class with them
	metric.setWx(ttfFile.get_ttf_funit(ttfMtx.getWx()));
	metric.setWy(0);
	int[] bbox = ttfMtx.getBBox();
	metric.getCharBBox().llx = ttfFile.get_ttf_funit(bbox[0]);
	metric.getCharBBox().lly = ttfFile.get_ttf_funit(bbox[1]);
	metric.getCharBBox().urx = ttfFile.get_ttf_funit(bbox[2]);
	metric.getCharBBox().ury = ttfFile.get_ttf_funit(bbox[3]);
	metric.setName(ttfMtx.getName());

	// look for kerning information for this character
	// Kerning information in TTFFile is stored in a hashtable
	// indexed by glyph-code of the first character in the kerning pair.
	// Each value in the hashtable is another hashtable that is indexed
	// by the glyph-code of the second glyph in the kerning pair.  The
	// values in this hashtable are Integer objects holding the kerning
	// x value.

	// first, get the glyph-code indexed kerning table
	Hashtable ttfKerningTable = ttfFile.getKerning();

	// convert the unicode value of this character to a glyph code
	// this will be the first glyph in the desired pair
	int thisGlyphCode = ttfFile.unicodeToGlyphCode(code);

	// if glyph code exists ...
	if (thisGlyphCode != -1) {
		// get the glyph-code indexed kerning subtable for the glyph
		// in question
		Hashtable ttfKerningSubtable = (Hashtable)
				ttfKerningTable.get( new Integer(thisGlyphCode));

		if (ttfKerningSubtable != null) {
			// for each entry in the kerning subtable
			for (Enumeration en = ttfKerningSubtable.keys();
										en.hasMoreElements(); ) {
				// 2nd glyph in pair
				Integer glyphObj = (Integer) en.nextElement();
				// Actual kerning x value for the pair of glyphs
				Integer kernObj = (Integer)
						ttfKerningSubtable.get(glyphObj);

				// convert 2nd glyph to list of corresponding unicode values
				// (a glyph may be used for more than one unicode character)
				List unicodeChars = ttfFile.
					glyphCodeToUnicodes(glyphObj.intValue());

				// Finally, for each unicode value corresponding to
				// the 2nd glyph, add a PairKern object representing
				// the unicode pair and the kerning value to the
				// CharMetricPS class we've created for the first
				// unicode value
				for (int i = 0; i < unicodeChars.size(); i++) {
					PairKern uniPairKern = new PairKern();
					uniPairKern.code1 = code;
					Integer unicodeChar = (Integer) unicodeChars.get(i);
					uniPairKern.code2 = unicodeChar.intValue();
					uniPairKern.xAmount = kernObj.intValue();
					uniPairKern.yAmount = 0;
					metric.getUnicodeKernPairs().put(
										unicodeChar, uniPairKern);
				}
			}
		}
	}
	return metric;
}

/**
 * Write the full font to the given output stream.
 * @param os OutputStream to write font to.
 */
public void writeFont(OutputStream os)
{
	if (fontFileReader != null) {
		//ToDo: in the case of a ttc file, should only write out the
		//subset of the ttc that relates to this font
		try {
			byte[] byteArray = fontFileReader.getBytes(0,  fontFileReader.getFileSize());
			os.write(byteArray);
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
	}
}

/**
 * Write a reduced version of the font (one that contains only the
 * glyphs that are used in this document, minus some table not used
 * by PDF) to the given output stream.
 * @param os OutputStream to write font to.
 * @param ansiCharactersUsed SortedSet containing all ansi characters
 *                           used in the document from this font
 * @param unicodeCharactersUsed SortedSet containing all unicode characters
 *                           used in the document from this font
 */
public void writeReducedFont(OutputStream os,
		SortedSet ansiCharactersUsed, SortedSet unicodeCharactersUsed)
{
	if (fontFileReader != null && ttfFile != null) {
		try {
			ttfFile.writeReducedFont(os, fontFileReader,
						ansiCharactersUsed, unicodeCharactersUsed);
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
	}
}

/**
 * Create empty character metrics for the given character code.
 */
public CharMetricPS createEmptyCharMetric(int code)
{
	// initialize the metric class for this character
	CharMetricPS metric = new CharMetricPS();
	metric.setCode(code);
	metric.setWx(0);
	metric.setWy(0);
	metric.getCharBBox().llx = 0;
	metric.getCharBBox().lly = 0;
	metric.getCharBBox().urx = 0;
	metric.getCharBBox().ury = 0;

	return metric;
}

/**
 * Get glyph code corresponding to given unicode value.  May return -1
 * if unicode value has no corresponding glyph in this font.
 */
public int unicodeToGlyphCode(int unicodeValue) {
	return ttfFile.unicodeToGlyphCode(unicodeValue);
}

/**
 * Given glyph code, return list of unicode values it represents.
 */
public List glyphCodeToUnicodes(int glyphCode) {
	return ttfFile.glyphCodeToUnicodes(glyphCode);
}

/**
 * Get the width of a glyph specified by the unicode character value.
 */
public int getGlyphWidthByUnicode(int unicodeValue)
{
	if (ttfFile != null) {
		return ttfFile.getGlyphWidthByUnicode(unicodeValue);
	} else {
		return 0;
	}
}

/**
 * Get the width of a glyph specified by the glyph number.
 */
public int getGlyphWidthByGlyphCode(int glyphCode)
{
	if (ttfFile != null) {
		return ttfFile.getGlyphWidthByGlyphCode(glyphCode);
	} else {
		return 0;
	}
}

}
