/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: Font11.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:11:46 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page.awt;

import java.awt.Font;
import java.awt.Rectangle;
import java.applet.Applet;
import java.awt.FontMetrics;

import java.util.StringTokenizer;
import java.util.Enumeration;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import java.util.zip.ZipFile;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.InputStream;

import java.text.CharacterIterator;
import com.klg.jclass.page.FontBase;



 
import java.awt.font.FontRenderContext;
import java.awt.font.GlyphVector;
import java.awt.geom.Rectangle2D;

import java.util.Hashtable;
import java.util.Vector;



/**
 *
 * Extends Font to provide common code for subclasses.
 *
 */
public class Font11 extends FontBase {

Font font;

/**
 * Constructor
 * @param name the String name of the font
 * @param style the style for this font
 * @param size the size for this font
 */
public Font11(String name, int style, int size, Font font) {
	super(name, style, size);
	this.font = font;
}

/**
 * Creates a new font with the specified name, style and point size.
 * @param name the font name
 * @param style the constant style used
 * @param size the point size of the font
 * @see Toolkit#getFontList
 * @since JDK1.0
 */
//public Font11(String name, int style, int size, Font font) {
//	super(name, style, size);
//	this.font = font;
//}

/**
 * Gets the platform specific family name of the font. Use the 
 * <code>getName</code> method to get the logical name of the font.
 * @return    a string, the platform specific family name.
 * @see       java.awt.Font#getName
 * @since     JDK1.0
 */
public String getFamily() {
	return font.getFamily();
}

/**
 * Gets the logical name of the font.
 * @return    a string, the logical name of the font.
 * @see #getFamily
 * @since     JDK1.0
 */
public String getName() {
	return font.getName();
}

/**
 * Gets the style of the font.
 * @return the style of this font.
 * @see #isPlain
 * @see #isBold
 * @see #isItalic
 * @since JDK1.0
 */
public int getStyle() {
	return font.getStyle();
}

/**
 * Gets the point size of the font.
 * @return the point size of this font.
 * @since JDK1.0
 */
public int getSize() {
	return font.getSize();
}

/**
 * Indicates whether the font's style is plain.
 * @return     <code>true</code> if the font is neither 
 *                bold nor italic; <code>false</code> otherwise.
 * @see        java.awt.Font#getStyle
 * @since      JDK1.0
 */
public boolean isPlain() {
	return font.isPlain();
}

/**
 * Indicates whether the font's style is bold.
 * @return    <code>true</code> if the font is bold; 
 *            <code>false</code> otherwise.
 * @see       java.awt.Font#getStyle
 * @since     JDK1.0
 */
public boolean isBold() {
	return font.isBold();
}

/**
 * Indicates whether the font's style is italic.
 * @return    <code>true</code> if the font is italic; 
 *            <code>false</code> otherwise.
 * @see       java.awt.Font#getStyle
 * @since     JDK1.0
 */
public boolean isItalic() {
	return font.isItalic();
}

/**
 * Gets a font from the system properties list.
 * @param nm the property name
 * @see       java.awt.Font#getFont(java.lang.String, java.awt.Font)
 * @since     JDK1.0
 */
public static Font getFont(String nm) {
	return Font.getFont(nm);
}

/**
 * Gets the specified font using the name passed in.
 * @param str the name
 * @since JDK1.1
 */
public static Font decode(String str) {
	return Font.decode(str);
}

/**
 * Gets the specified font from the system properties list.
 * The first argument is treated as the name of a system property to 
 * be obtained as if by the method <code>System.getProperty</code>.  
 * The string value of this property is then interpreted as a font. 
 * <p>
 * The property value should be one of the following forms: 
 * <ul>
 * <li><em>fontname-style-pointsize</em>
 * <li><em>fontname-pointsize</em>
 * <li><em>fontname-style</em>
 * <li><em>fontname</em>
 * </ul>
 * where <i>style</i> is one of the three strings 
 * <code>"BOLD"</code>, <code>"BOLDITALIC"</code>, or 
 * <code>"ITALIC"</code>, and point size is a decimal 
 * representation of the point size. 
 * <p>
 * The default style is <code>PLAIN</code>. The default point size 
 * is 12. 
 * <p>
 * If the specified property is not found, the <code>font</code> 
 * argument is returned instead. 
 * @param nm the property name
 * @param font a default font to return if property <code>nm</code> 
 *             is not defined
 * @return    the <code>Font</code> value of the property.
 * @since     JDK1.0
 */
public static Font getFont(String nm, Font font) {
	return Font.getFont(nm, font);
}

public String toString() {
	return "Font11 " + super.toString();
}

public static GlyphVector createGlyphVector(FontMetrics metrics, FontRenderContext frc, String str) {
	return new GlyphVector11(metrics, frc, str);
}

}
