/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: FontMetrics11.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:11:46 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page.awt;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.text.CharacterIterator;
import com.klg.jclass.page.FontMetricsBase;




import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.awt.font.FontRenderContext;
import java.awt.font.GlyphVector;


/**
 * The FontMetricsBase class supplies FontMetrics methods which are missing
 * in the JDK1.1 version of FontMetrics.
 */

public class FontMetrics11 extends FontMetricsBase {
	
FontMetrics fontMetrics;

public FontMetrics11(Font f, FontMetrics fontMetrics) {
	super(new Font11(f.getName(), f.getStyle(), f.getSize(), f));
	this.fontMetrics = fontMetrics;
}


//////////////////////////////////////////////////////////////////////
///////////////// FontMetrics 1.1 methods ////////////////////////////
//////////////////////////////////////////////////////////////////////


/**
 * Determines the <em>standard leading</em> of the font described by 
 * this font metric. The standard leading (interline spacing) is the 
 * logical amount of space to be reserved between the descent of one 
 * line of text and the ascent of the next line. The height metric is 
 * calculated to include this extra space. 
 * @return    the standard leading of the font.
 * @see       java.awt.FontMetrics#getHeight
 * @see       java.awt.FontMetrics#getAscent
 * @see       java.awt.FontMetrics#getDescent
 * @since     JDK1.0
 */
public int getLeading() {
	return fontMetrics.getLeading();
}

/**
 * Determines the <em>font ascent</em> of the font described by this 
 * font metric. The font ascent is the distance from the font's 
 * baseline to the top of most alphanumeric characters. Some 
 * characters in the font may extend above the font ascent line. 
 * @return     the font ascent of the font.
 * @see        java.awt.FontMetrics#getMaxAscent
 * @since      JDK1.0
 */
public int getAscent() {
	return fontMetrics.getAscent();
}

/**
 * Determines the <em>font descent</em> of the font described by this 
 * font metric. The font descent is the distance from the font's 
 * baseline to the bottom of most alphanumeric characters with 
 * descenders. Some characters in the font may extend below the font 
 * descent line. 
 * @return     the font descent of the font.
 * @see        java.awt.FontMetrics#getMaxDescent
 * @since      JDK1.0
 */
public int getDescent() {
	return fontMetrics.getDescent();
}

/**
 * Gets the standard height of a line of text in this font.  This
 * is the distance between the baseline of adjacent lines of text.
 * It is the sum of the leading + ascent + descent.  There is no
 * guarantee that lines of text spaced at this distance will be
 * disjoint; such lines may overlap if some characters overshoot
 * either the standard ascent or the standard descent metric.
 * @return    the standard height of the font.
 * @see       java.awt.FontMetrics#getLeading
 * @see       java.awt.FontMetrics#getAscent
 * @see       java.awt.FontMetrics#getDescent
 * @since     JDK1.0
 */
public int getHeight() {
	return fontMetrics.getHeight();
}

/**
 * Determines the maximum ascent of the font described by this font 
 * metric. No character extends further above the font's baseline 
 * than this height. 
 * @return    the maximum ascent of any character in the font.
 * @see       java.awt.FontMetrics#getAscent
 * @since     JDK1.0
 */
public int getMaxAscent() {
	return fontMetrics.getMaxAscent();
}

/**
 * Determines the maximum descent of the font described by this font 
 * metric. No character extends further below the font's baseline 
 * than this height. 
 * @return    the maximum descent of any character in the font.
 * @see       java.awt.FontMetrics#getDescent
 * @since     JDK1.0
 */
public int getMaxDescent() {
	return fontMetrics.getMaxDescent();
}

/**
 * For backward compatibility only.
 * @see #getMaxDescent
 * @deprecated As of JDK version 1.1.1,
 * replaced by <code>getMaxDescent()</code>.
 */
public int getMaxDecent() {
	return fontMetrics.getMaxDescent();
}

/**
 * Gets the maximum advance width of any character in this Font. 
 * The advance width is the amount by which the current point is
 * moved from one character to the next in a line of text.
 * @return    the maximum advance width of any character 
 *            in the font, or <code>-1</code> if the 
 *            maximum advance width is not known.
 * @since     JDK1.0
 */
public int getMaxAdvance() {
	return fontMetrics.getMaxAdvance();
}

/** 
 * Returns the advance width of the specified character in this Font.
 * The advance width is the amount by which the current point is
 * moved from one character to the next in a line of text.
 * @param ch the character to be measured
 * @return    the advance width of the specified <code>char</code> 
 *                 in the font described by this font metric.
 * @see       java.awt.FontMetrics#charsWidth
 * @see       java.awt.FontMetrics#stringWidth
 * @since     JDK1.0
 */
public int charWidth(int ch) {
	return fontMetrics.charWidth(ch);
}

/** 
 * Returns the advance width of the specified character in this Font.
 * The advance width is the amount by which the current point is
 * moved from one character to the next in a line of text.
 * @param ch the character to be measured
 * @return     the advance width of the specified <code>char</code> >
 *                  in the font described by this font metric.
 * @see        java.awt.FontMetrics#charsWidth
 * @see        java.awt.FontMetrics#stringWidth
 * @since      JDK1.0
 */
public int charWidth(char ch) {
	return fontMetrics.charWidth(ch);
}

/** 
 * Returns the total advance width for showing the specified String
 * in this Font.
 * The advance width is the amount by which the current point is
 * moved from one character to the next in a line of text.
 * @param str the String to be measured
 * @return    the advance width of the specified string 
 *                  in the font described by this font metric.
 * @see       java.awt.FontMetrics#bytesWidth
 * @see       java.awt.FontMetrics#charsWidth
 * @since     JDK1.0
 */
public int stringWidth(String str) {
	return fontMetrics.stringWidth(str);
}

/** 
 * Returns the total advance width for showing the specified array
 * of characters in this Font.
 * The advance width is the amount by which the current point is
 * moved from one character to the next in a line of text.
 * @param data the array of characters to be measured
 * @param off the start offset of the characters in the array
 * @param len the number of characters to be measured from the array
 * @return    the advance width of the subarray of the specified 
 *               <code>char</code> array in the font described by 
 *               this font metric.
 * @see       java.awt.FontMetrics#charWidth(int)
 * @see       java.awt.FontMetrics#charWidth(char)
 * @see       java.awt.FontMetrics#bytesWidth
 * @see       java.awt.FontMetrics#stringWidth
 * @since     JDK1.0
 */
public int charsWidth(char data[], int off, int len) {
	return fontMetrics.charsWidth(data, off, len);
}

/** 
 * Returns the total advance width for showing the specified array
 * of bytes in this Font.
 * The advance width is the amount by which the current point is
 * moved from one character to the next in a line of text.
 * @param data the array of bytes to be measured
 * @param off the start offset of the bytes in the array
 * @param len the number of bytes to be measured from the array
 * @return    the advance width of the subarray of the specified 
 *               <code>byte</code> array in the font described by 
 *               this font metric.
 * @see       java.awt.FontMetrics#charsWidth
 * @see       java.awt.FontMetrics#stringWidth
 * @since     JDK1.0
 */
public int bytesWidth(byte data[], int off, int len) {
	return fontMetrics.bytesWidth(data, off, len);
}

/**
 * Gets the advance widths of the first 256 characters in the Font.
 * The advance width is the amount by which the current point is
 * moved from one character to the next in a line of text.
 * @return    an array giving the advance widths of the 
 *                 characters in the font 
 *                 described by this font metric.
 * @since     JDK1.0
 */
public int[] getWidths() {
	return fontMetrics.getWidths();
}

/** 
 * Returns a representation of this <code>FontMetric</code> 
 * object's values as a string.
 * @return    a string representation of this font metric.
 * @since     JDK1.0.
 */
public String toString() {
	return "Graphics11FontMetrics " + fontMetrics.toString();
}


//////////////////////////////////////////////////////////////////////
///////////////// FontMetrics 1.2 methods ////////////////////////////
//////////////////////////////////////////////////////////////////////


public Rectangle2D getStringBounds(String str, Graphics gc) {

	FontRenderContext frc;

	if (gc instanceof Graphics2D) {
		frc = ((Graphics2D) gc).getFontRenderContext();
	}
	else {
		frc = new FontRenderContext(null, false, false);
	}

	GlyphVector gv = new GlyphVector11(this, frc, str);

	return gv.getLogicalBounds();
}

}
