/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCAWT11Printer.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:11:46 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page.awt;

import com.klg.jclass.page.JCDocument;
import com.klg.jclass.page.JCPrinter;
import com.klg.jclass.page.awt.Graphics2D11;

import java.util.Stack;
import java.util.Date;
import java.io.OutputStream;
import java.awt.PrintJob;
import java.awt.Graphics;





import java.awt.geom.Rectangle2D;
import java.awt.Graphics2D;


import java.awt.Toolkit;
import java.awt.Frame;

/**
 *
 * Implements the <code>PrinterDriver</code> interface
 * for jdk 1.1 compatible printing.
 *
 */
public class JCAWT11Printer extends JCPrinter {

/* The 1.1 printing model works like this. From
 * a PrintJob we get a graphics context which is
 * good for printing one page. Dispose flushes the
 * page and sets up a new gc for the next page. To
 * end call PrintJob.end.
 */

protected PrintJob printJob;
protected JCDocument document;
public boolean showDialog 		= true;
protected boolean first 		= true;

/**
 *
 * Constructor. Uses 8.5 x 11 default page.
 *
 */
public JCAWT11Printer() throws PrintJobCancelledException {
	super();
	Frame frame = new Frame();

	// SOLARIS 1.1.8 fix:  we need to call addNotify() on the frame to avoid
	//   (PR#14065)        a NullPointerException in getPrintJob().
	


	printJob = Toolkit.getDefaultToolkit().getPrintJob(frame, "JClass PageLayout", null);

	if (printJob == null) {
		throw new PrintJobCancelledException();
	}

	graphics = new Graphics2D11(printJob.getGraphics());
	((Graphics2D11) graphics).setFrameRoot();
}

/**
 * Creates default styles, any other one-time initialization.
 * Builds font families from available fonts.
 */
public void setup() {
	// not implemented
}

/**
 * Returns <code>false</code> since images sent to AWT11 Printer are pre-scaled.
 * @return <code>false</code> since images sent to AWT11 Printer are pre-scaled
 */
public boolean isImageScalable()
{
    return false;
}

/**
 * Stores the document being printed in this object so callbacks to
 * the document can be made by the printer thread.
 * @param document
 */
public void registerDocument(JCDocument document) {
	this.document = document;
}

/**
 * Initialization.
 * @param pageSize
 */
public void openDocument(Rectangle2D pageSize) {
	super.openDocument(pageSize);
}

/**
 *
 * @deprecated Use document.print() instead.
 * @param document
 */
public void print(JCDocument document) {
	document.print(this);
}

/**
 * Document completion (for example, PDF cross-reference table and trailer).
 * @param pageSize
 */
public synchronized void closeDocument(Rectangle2D pageSize) {
	if (printJob != null) {
		printJob.end();
	}

	super.closeDocument(pageSize);
}

/**
 * PS Document Structuring Conventions comments, page orientation, etc.
 * @param pageSize
 * @param pageNumber
 */
public void beginPage(Rectangle2D pageSize, int pageNumber) {

	super.beginPage(pageSize, pageNumber);

	// there is a new graphics context for each page
	// in the 11 printing model, get it
	if (!first) {
		Graphics2D11 g11 = new Graphics2D11(printJob.getGraphics());
		g11.setFrameRoot();
		setGraphics(g11);
	}
	first = false;
}

/**
 * Page bounding box.
 * @param pageSize
 */
public void endPage(Rectangle2D pageSize) {
	Graphics g = getGraphics();

	// calling dispose on the jdk1.1 print
	// graphics really means print the page.
	if (g != null) {
		g.dispose();
	}

	super.endPage(pageSize);
}


/**
 * Prints a range.
 * @param document
 * @param pageIndex
 */
protected void printPage(JCDocument document, int pageIndex) {
//	// document.print(this, pageIndex, pageIndex);
}

/**
 * Is the print dialog to be shown when
 * printing is requested?
 * <code>true</code> means show it.
 * @deprecated there is no option -- jdk1.1 always requires
 * the print dialog; setting this does nothing
 * @param showDialog
 */
public void setShowDialog(boolean showDialog) {
	this.showDialog = showDialog;
}

/**
 * Is the print dialog to be shown when
 * printing is requested?
 * <code>true</code> means show it.
 * @deprecated there is no option -- jdk1.1 always requires
 * the print dialog
 * @return <code>true</code> if the print dialog is to be shown
 */
public boolean getShowDialog() {
	return showDialog;
}

/**
 *
 * Cancels the pending job and kills the waiting thread.
 *
 */
public void cancel() {
	// abort = true;

	// if (printerJob != null) {
	// 	printerJob.cancel();
	// 	printerThread.wakeUp();
	// }

}

public void setGraphics(Graphics2D graphics) {
	this.graphics = graphics;
}

public static class PrintJobCancelledException extends Exception {

	public PrintJobCancelledException() {
		super("User elected not to print");
	}

} // end PrintJobCancelled inner class

} // end class
