/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCAWTPreviewer.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:11:46 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page.awt;

import com.klg.jclass.page.JCDocument;
import com.klg.jclass.page.JCPage;
import com.klg.jclass.page.JCPrinter;
import com.klg.jclass.page.adobe.postscript.JCPostScriptPrinter;
import com.klg.jclass.page.adobe.pdf.JCPDFPrinter;
import com.klg.jclass.page.pcl.JCPCLPrinter;

import java.io.FileOutputStream;
import java.io.OutputStream;
import java.io.File;
import java.io.IOException;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowListener;
import java.awt.event.WindowEvent;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Toolkit;





import java.awt.geom.Rectangle2D;
import java.awt.print.PageFormat;
import java.awt.print.Paper;
import java.awt.print.PrinterException;


import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JFrame;

/**
 * The <code>JCAWTPreviewer</code> is a frame to display print pages. It allows
 * users to flip through the pages and send the current page or
 * all the pages to the printer.
 */
public class JCAWTPreviewer extends JDialog implements WindowListener,
	ActionListener {

protected JButton firstButton;
protected JButton prevButton;
protected JButton nextButton;
protected JButton lastButton;
protected JButton printButton;
protected JButton closeButton;

protected JScrollPane pane;

protected JLabel status;

int currentPage = 0;

protected JCAWTScreenPrinter printer;
protected JCDocument document;
protected JCPrintPage printPage;

/**
 *
 * Creates a new <code>PrintPreview</code> frame.
 *
 * @param title the title of the frame
 * @param frame the frame in which to show the previewer
 * @param document the laid out document to preview
 *
 */
public JCAWTPreviewer(String title, JFrame frame, JCDocument document) {
	this(title, frame, document, new JCAWTScreenPrinter());
}

/**
 *
 * Creates a new <code>PrintPreview</code> frame.
 *
 * @param title the title of the frame
 * @param frame the frame in which to show the previewer
 * @param document the laid out document to preview
 * @param printer the <code>JCAWTScreenPrinter</code> whose component will be
 * be used to draw on in this previewer
 */
public JCAWTPreviewer(String title, JFrame frame, JCDocument document, JCPrinter printer) {

	super(frame, title);

	if (!(printer instanceof JCAWTScreenPrinter)) {
		throw new RuntimeException("Printer must be instance of JCAWTScreenPrinter");
	}

	this.printer 	= (JCAWTScreenPrinter) printer;
	this.document 	= document;
	this.printPage 	= (JCPrintPage) this.printer.getComponent();

	// add some space for header, scroll bars, etc
	int request_height = getPageHeight() + 133;
	int request_width = getPageWidth() + 88;

	// Restrict the maximum size of the preview window to the screen size
	Dimension screen_size = Toolkit.getDefaultToolkit().getScreenSize();
	if (request_height > screen_size.height) {
		request_height = screen_size.height;
	}
	if (request_width > screen_size.width) {
		request_width = screen_size.width;
	}
	setSize(request_width, request_height);

	setModal( true );

	// Properties prop = new Properties();
	// job = frame.getToolkit().getPrintJob(frame, "Higrid PrintPreview", prop);

	getContentPane().setLayout(new BorderLayout());
	setBackground(Color.lightGray);

	JPanel buttons = new JPanel();
	buttons.setLayout(new FlowLayout(FlowLayout.LEFT,1,1));
	buttons.add(firstButton = new JButton("First"));
	firstButton.addActionListener(this);
	buttons.add(prevButton = new JButton("Previous"));
	prevButton.addActionListener(this);
	buttons.add(nextButton = new JButton("Next"));
	nextButton.addActionListener(this);
	buttons.add(lastButton = new JButton("Last"));
	lastButton.addActionListener(this);

	buttons.add(printButton = new JButton("Print"));
	printButton.addActionListener(this);

	buttons.add(closeButton = new JButton("Close"));
	closeButton.addActionListener(this);

	Dimension d = new Dimension(
		(int)(getPageWidth() * 1.05),
		(int)(getPageHeight() * 1.05));

	pane = new JScrollPane();
	pane.setSize(d);

	getContentPane().add(buttons, BorderLayout.NORTH);
	getContentPane().add(pane, BorderLayout.CENTER);
	getContentPane().add(status = new JLabel("", JLabel.LEFT), BorderLayout.SOUTH);

	// set the first page
	createPage(d);

	addWindowListener(this);

}

/**
 * Creates an image of the page to preview.
 * @param d
 */
public void createPage(Dimension d) {
	pane.setViewportView(printPage);
	printPage.setPreferredSize(d);
	printPage.setSize(d);

	// pane.setSize(d);
	printPage.setDocument(document);

	// show this
	showPage(0);

}

/**
 * Generates a new page image and posts it to the
 * <code>PrintPage</code> component for display.
 * @param pageIndex
 */
public void showPage(int pageIndex) {
	if (pageIndex < getNumPages() && pageIndex > -1) {
		currentPage = pageIndex;
		printPage.setPage(pageIndex);
		status.setText((currentPage + 1) + " of " + getNumPages() + ".");
	}
	else {
		throw new RuntimeException("Page index out of range " + pageIndex);
	}
}

/**
 * Cleanup before closing <code>PrintPreview</code> window.
 */
void close() {
	setVisible(false);
	dispose();
}

/**
 * Disposes of the PageLayout objects that are being held.
 */
public void dispose() {

	// Delete the Pagelayout objects we're linked to
	document = null;
	printer = null;
	printPage.dispose();
	printPage = null;

	// Clear the contents of the ScrollPane
	pane.setViewportView(null);

	// Call our parent's dispose method
	super.dispose();
}

/**
 * Returns the number of pages in this document.
 * @return the number of pages in this document
 */
protected int getNumPages() {
	return document.getPages().size();
}

/**
 * <code>ActionListener</code> method.
 * @param e
 */
public void actionPerformed(ActionEvent e) {
	if (e.getSource() instanceof JButton) {
		JButton button = (JButton) e.getSource();
		if (button.equals(firstButton)) {
			showPage(0);
		}
		else if (button.equals(prevButton)) {
			if(currentPage > 0) {
				showPage(currentPage - 1);
			}
		}
		else if(button.equals(nextButton)) {
			if(currentPage < getNumPages() - 1) {
				showPage(currentPage + 1);
			}
		}
		else if(button.equals(lastButton)) {
			showPage(getNumPages() - 1);
		}
		else if(button.equals(printButton)) {
			if(document != null) {
				if (document.getPrinter() instanceof JCAWTScreenPrinter) {
					// figure out page dimensions from the document
					JCPage page 	= document.getFlow().getCurrentPage();
					Rectangle2D r2d	= page.getPageSize();

					
					// now create the priner
					JCAWTPrinter awtPrinter;

					try {
						awtPrinter = new JCAWTPrinter(document.getTemplates());
					} catch (PrinterException px) {
						// error in printer
						return;
					} catch (JCAWTPrinter.PrinterJobCancelledException x) {
						// user cancelled print job
						return;
					}


					awtPrinter.getGraphics();
					awtPrinter.print(document);
				}

				
				else if (document.getPrinter() instanceof JCAWTPrinter) {


				


					document.print();
				}
				else {
					// let the user select the file to write to
					// (a.k.a. print to file)
					JFileChooser chooser = new JFileChooser();
					int returnVal = chooser.showSaveDialog(this);

					// if they didn't cancel ...
					if(returnVal == JFileChooser.APPROVE_OPTION) {

						// get the handle to the file they selected
						File file = chooser.getSelectedFile();

						// set the output stream to the file they chose
						OutputStream os 	= null;
						OutputStream oldOs	= null;
						try {
							os = new FileOutputStream(file);

							// preserve their existing os
							oldOs = document.getPrinter().getOutputStream();

							// temporarily set the new one
							document.getPrinter().setOutputStream(os);

							// then print to that file
							document.print();

							// free resources
							os.flush();
							os.close();

							// restore their output stream
							document.getPrinter().setOutputStream(oldOs);

						} catch (IOException ioe) {
							// restore their output stream
							document.getPrinter().setOutputStream(oldOs);

							// pop up error message
							JOptionPane.showMessageDialog(null,
												ioe.toString(), "File error",
												JOptionPane.ERROR_MESSAGE);
							return;
						}
					}
				}
			}
		}
		else if(button.equals(closeButton)) {
			close();
		}
	}
}


/**
 * Gets the width of a page.
 *
 * @return the page width
 */
public int getPageWidth() {
	return((int) document.getFlow().getCurrentPage().getPageSize().getWidth());
}

/**
 * Gets the height of a page.
 *
 * @return the page height
 */
public int getPageHeight() {
	return((int) document.getFlow().getCurrentPage().getPageSize().getHeight());
}

/**
 *
 * Returns the screen printer for this previewer.
 * @return screen printer for this previewer
 */
 public JCPrinter getPrinter() {
 	return printer;
 }

/**
 * <code>WindowListener</code> methods.
 * @param e
 */
public void windowActivated(WindowEvent e) {}
public void windowClosed(WindowEvent e) {}

public void windowClosing(WindowEvent e) {
	close();
}

public void windowDeactivated(WindowEvent e) {}
public void windowDeiconified(WindowEvent e) {}
public void windowIconified(WindowEvent e) {}
public void windowOpened(WindowEvent e) {}

}
