/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCAWTPrinter.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:11:46 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page.awt;

import com.klg.jclass.page.JCDocument;
import com.klg.jclass.page.JCPrinter;
import com.klg.jclass.page.JCPage;
import com.klg.jclass.page.JCFlow;
import com.klg.jclass.util.JCEnvironment;

import java.util.List;
import java.util.Stack;
import java.util.Date;
import java.io.OutputStream;
import java.awt.print.PageFormat;
import java.awt.print.Paper;
import java.awt.print.Printable;
import java.awt.print.Pageable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterGraphics;
import java.awt.print.PrinterJob;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;

/**
 *
 * Implements PrinterDriver interface.
 *
 */
public class JCAWTPrinter extends JCPrinter
{

protected PrinterThread     printerThread;
protected Thread            thread;
protected PrinterJob        printerJob;
protected boolean           actuallyPrinting;
protected JCDocument        document;
protected boolean           showDialog 	= true;
protected Stack             clipStack 		= new Stack();
protected PageFormat        pageFormat	= new PageFormat();
protected boolean           abort			= false; // stop printer thread
protected int               firstToPrint = -1;
protected int               lastToPrint = -1;
protected PrinterException  threadException = null; // exc from printer thread

/** First template page in document, used to set up correct PageFormat
    properties for initial page.  Only used if template list passed in
	in constructor. */
protected JCPage			firstPage = null;

/** True if the PageFormat should be updated with information from the
    passed page template (for the first page) or the document (for other
    pages).
 */
protected boolean			updatePageFormat = false;

/**
 * Constructor -- uses 8.5 x 11 default page.
 * The Print Dialog will be popped up so the user can
 * indicate which printer to use for metrics and printing.
 * This should only be used for standard 8.5 x 11 documents.
 * Documents with special page requirements such as landscape
 * orientation or other page sizes should use the contructor
 * that takes a list of page templates.
 */
public JCAWTPrinter()
    throws PrinterJobCancelledException, PrinterException
{
	this(true);
}

/**
 * Constructor -- uses 8.5 x 11 default page.
 * This should only be used for standard 8.5 x 11 documents.
 * Documents with special page requirements such as landscape
 * orientation or other page sizes should use the contructor
 * that takes a list of page templates.
 * @param showDialog a boolean; true means show the dialog,
 * false means do not show the dialog -- use the default printer.
 */
public JCAWTPrinter(boolean showDialog)
    throws PrinterJobCancelledException, PrinterException
{

	this.showDialog = showDialog;

	Paper paper = new Paper();
	// NOTE:  We offset the imageable area by (10, 10) from the nominal
	//        origin because (for 1.2.2 at least) the JDK seems to move
	//        the page fractionally down and to the right.  This may need
	//        to be removed at some point in the future (1.3?).
	paper.setImageableArea(0, 0, 612, 792);
	pageFormat.setPaper(paper);

	// get the graphics which forces user to choose
	// printer whose metrics we'll use to do layout
	Graphics2D tempGraphics = (Graphics2D) getGraphics();
	if (tempGraphics == null) {
		if (threadException != null) {
			throw threadException;
		}
		else {
			throw new PrinterJobCancelledException();
		}
	}
}

/**
 * Constructor with template list.
 * The Print Dialog will be popped up so the user can
 * indicate which printer to use for metrics and printing.
 * This constructor will handle documents with special page requirements
 * such as landscape orientation or page sizes other than 8.5 by 11
 * by copying such information to the default PageFormat object used
 * in printing.
 * @param templates a List of page templates, the same one that will
 *                  later be passed to the JCDocument constructor.
 */
public JCAWTPrinter(List templates)
    throws PrinterJobCancelledException, PrinterException
{
	this(templates, true);
}

/**
 * Constructor with template list.
 * This constructor will handle documents with special page requirements
 * such as landscape orientation or page sizes other than 8.5 by 11
 * by copying such information to the default PageFormat object used
 * in printing.
 * @param templates a List of page templates, the same one that will
 *                  later be passed to the JCDocument constructor.
 * @param showDialog a boolean; true means show the dialog,
 * false means do not show the dialog -- use the default printer.
 */
public JCAWTPrinter(List templates, boolean showDialog)
    throws PrinterJobCancelledException, PrinterException
{
	this ((PrinterJob) null, (PageFormat) null, (List) templates, showDialog);
}

/**
 * Constructor taking specialized PageFormat class.
 * The Print Dialog will be popped up so the user can
 * indicate which printer to use for metrics and printing.
 *
 * This constructor should only be used when the passed PageFormat object
 * contains all formatting information neeeded for printing the
 * document.  Properties set on the document, such as page
 * orientation and page size, will not be copied to this
 * PageFormat object.  If this is not desired, use the constructor
 * that takes both a PageFormat object and a List of page templates instead.
 * @param pageFormat the PageFormat object to use for page
 * information.
 */
public JCAWTPrinter(PageFormat pageFormat)
    throws PrinterJobCancelledException, PrinterException
{
	this(pageFormat, true);
}

/**
 * Constructor taking specialized PageFormat class.
 * This constructor should only be used when the passed PageFormat object
 * contains all formatting information neeeded for printing the
 * document.  Properties set on the document, such as page
 * orientation and page size, will not be copied to this
 * PageFormat object.  If this is not desired, use the constructor
 * that takes both a PageFormat object and a List of page templates instead.
 * @param pageFormat the PageFormat object to use for page
 * information.
 * @param showDialog a boolean; true means show the dialog,
 * false means do not show the dialog -- use the default printer.
 */
public JCAWTPrinter(PageFormat pageFormat, boolean showDialog)
    throws PrinterJobCancelledException, PrinterException
{
	this((PrinterJob) null, pageFormat, (List) null, showDialog);
}

/**
 * Constructor taking specialized PageFormat class and list of page templates.
 * The Print Dialog will be popped up so the user can
 * indicate which printer to use for metrics and printing.
 * Page information such as page orientation and page size will
 * be copied from the document settings to the passed PageFormat object
 * during printing.
 * @param pageFormat the PageFormat object to use for page
 * information.
 * @param templates a List of page templates, the same one that will
 *                  later be passed to the JCDocument constructor.
 */
public JCAWTPrinter(PageFormat pageFormat, List templates)
    throws PrinterJobCancelledException, PrinterException
{
	this(pageFormat, templates, true);
}

/**
 * Constructor taking specialized PageFormat class and list of page templates.
 * Page information such as page orientation and page size will
 * be copied from the document settings to the passed PageFormat object
 * during printing.
 * @param pageFormat the PageFormat object to use for page
 * information.
 * @param templates a List of page templates, the same one that will
 *                  later be passed to the JCDocument constructor.
 * @param showDialog a boolean; true means show the dialog,
 * false means do not show the dialog -- use the default printer.
 */
public JCAWTPrinter(PageFormat pageFormat, List templates, boolean showDialog)
    throws PrinterJobCancelledException, PrinterException
{
	this((PrinterJob) null, pageFormat, templates, showDialog);
}

/**
 * Constructor taking specific PrinterJob and PageFormat.
 * The Print Dialog will be popped up so the user can
 * indicate which printer to use for metrics and printing.
 *
 * This constructor should only be used when the passed PageFormat object
 * contains all formatting information neeeded for printing the
 * document.  Properties set on the document, such as page
 * orientation and page size, will not be copied to this
 * PageFormat object.  If this is not desired, use the constructor
 * that takes both a PageFormat object and a List of page templates instead.
 * @param printerJob the PrinterJob whose information will be used
 * for printing
 * @param pageFormat the PageFormat object to use for page
 * information.
 */
public JCAWTPrinter(PrinterJob printerJob, PageFormat pageFormat)
    throws PrinterJobCancelledException, PrinterException
{
	this( printerJob, pageFormat, true );
}

/**
 * Constructor taking specific PrinterJob and PageFormat.
 * This constructor should only be used when the passed PageFormat object
 * contains all formatting information neeeded for printing the
 * document.  Properties set on the document, such as page
 * orientation and page size, will not be copied to this
 * PageFormat object.  If this is not desired, use the constructor
 * that takes both a PageFormat object and a List of page templates instead.
 * @param printerJob the PrinterJob whose information will be used
 * for printing
 * @param pageFormat the PageFormat object to use for page
 * information.
 * @param showDialog a boolean; true means show the dialog,
 * false means do not show the dialog -- use the default printer.
 *
 */
public JCAWTPrinter(PrinterJob printerJob, PageFormat pageFormat,
                    boolean showDialog)
    throws PrinterJobCancelledException, PrinterException
{
	this (printerJob, pageFormat, (List) null, showDialog);
}

/**
 * Constructor taking specific PrinterJob and PageFormat as well
 * as a list of page templates.
 * The Print Dialog will be popped up so the user can
 * indicate which printer to use for metrics and printing.
 * Page information such as page orientation and page size will
 * be copied from the document settings to the passed PageFormat object
 * during printing.
 * @param printerJob the PrinterJob whose information will be used
 * for printing
 * @param pageFormat the PageFormat object to use for page
 * information.
 * @param templates a List of page templates, the same one that will
 *                  later be passed to the JCDocument constructor.
 * @param showDialog a boolean; true means show the dialog,
 * false means do not show the dialog -- use the default printer.
 *
 */
public JCAWTPrinter(PrinterJob printerJob, PageFormat pageFormat,
                    List templates)
    throws PrinterJobCancelledException, PrinterException 
{
	this(printerJob, pageFormat, templates, true);
}

/**
 * Constructor taking specific PrinterJob and PageFormat as well
 * as a list of page templates.
 * Page information such as page orientation and page size will
 * be copied from the document settings to the passed PageFormat object
 * during printing.
 * @param printerJob the PrinterJob whose information will be used
 * for printing
 * @param pageFormat the PageFormat object to use for page
 * information.
 * @param templates a List of page templates, the same one that will
 *                  later be passed to the JCDocument constructor.
 * @param showDialog a boolean; true means show the dialog,
 * false means do not show the dialog -- use the default printer.
 *
 */
public JCAWTPrinter(PrinterJob printerJob, PageFormat pageFormat,
                    List templates, boolean showDialog)
    throws PrinterJobCancelledException, PrinterException
{
	this.showDialog = showDialog;

	if (printerJob != null) {
		this.printerJob = printerJob;
	}

	if (pageFormat != null) {
		this.pageFormat = pageFormat;
	}

	if (templates != null) {

		// if list of templates passed in, note that PageFormat should
		// be updated
		updatePageFormat = true;

		// save first template page -- needed when setting up PageFormat
		// for first page
		if (templates.size() > 0) {
			JCPage templatePage = (JCPage) templates.get(0);
			firstPage = templatePage;
		}
	}

	// Get the graphics which forces user to choose
	// printer whose metrics we'll use to do layout
	Graphics2D tempGraphics = (Graphics2D) getGraphics();
	if (tempGraphics == null) {
		if (threadException != null) {
			throw threadException;
		}
		else {
			throw new PrinterJobCancelledException();
		}
	}
}

/**
 * Creates default styles, any other one-time initialisation
 * Builds font families from available fonts.
 */
public void setup()
{
	// not implemented
}

/**
 * Returns false since images sent to AWT Printer are pre-scaled.
 */
public boolean isImageScalable()
{
    return(false);
}

/**
 * Store the document being printed in this object so callbacks to
 * the document can be made by the printer thread
 * This is for internal use only.
 */
public void registerDocument(JCDocument document) {
	this.document = document;
}

/**
 * Store the first and last page to be printed so that when printing
 * actually starts, we know what we're supposed to print.
 * This is for internal use only.
 */
public void registerFirstLast(int first, int last)
{
	this.firstToPrint = first;
	this.lastToPrint = last;
}

/**
 * Any required initialisation (e.g. PostScript document headers, etc.)
 */
public synchronized void openDocument(Rectangle2D pageSize)
{
	// Called below because each page is treated as a doc in this printer
	// super.openDocument(pageSize);

	// If printer thread does not yet exist, create it by getting the
	// printer graphics object
	if (printerThread == null) {
		getGraphics();
	}

	if (!actuallyPrinting) {
		actuallyPrinting 	= true;

		// wake up the printer thread
   		printerThread.wakeUp();

   		// and ut the main thread to sleep so both threads are not
   		// trying to modify the gc at the same time
   		try {
    		wait();
   		}
   		catch (InterruptedException e) {}
	}
}

/**
 * Convenience method with bogus boolean to overload the method and
 * call super, since VisualAge doesn't understand calling the superclass
 * from an inner class.
 */
public void openDocument(Rectangle2D pageSize, boolean superclass)
{
	super.openDocument(pageSize);
}

/**
 * Called by the print thread after it has finished
 * printing the document. It wakes up the main thread
 * so it can resume its activities.
 */
protected synchronized void wakeUpMainThread()
{
	notifyAll();
}

/**
 * Called by the print thread after it has finished printing
 * (but before waking up the main thread) to clear out
 * the variable that would impede another thread being created.
 */
protected synchronized void clearThread()
{
	printerThread = null;
	graphics = null;
}

/**
 *
 * @deprecated Use document.print() instead.
 *
 */
public void print(JCDocument document)
{
	this.document       = document;
	actuallyPrinting 	= true;

	// wake up the printer thread
    printerThread.wakeUp();
}

/**
 * document completion (e.g. PDF cross-ref table and trailer)
 */
public synchronized void closeDocument(Rectangle2D pageSize)
{
	// called below because each page is treated as a doc in this printer
	// super.closeDocument(pageSize);
}
/**
 * Convenience method with bogus boolean to overload the method and
 * call super, since VisualAge doesn't understand calling the superclass
 * from an inner class.
 */
public void closeDocument(Rectangle2D pageSize, boolean superclass)
{
	super.closeDocument(pageSize);
}

/**
 * PS Document Structuring Conventions comments, page orientation...
 */
public void beginPage(Rectangle2D pageSize, int pageNumber)
{
	// called below because each page is treated as a doc in this printer
	// super.beginPage(pageSize, pageNumber);
}

/**
 * Page bounding box...
 */
public void endPage(Rectangle2D pageSize)
{
	// called below because each page is treated as a doc in this printer
	// super.endPage(pageSize);
}

protected void initPrinterJob()
{
	// Use the already created job, if any
	if (printerJob == null) {
		printerJob = PrinterJob.getPrinterJob();
	}

	// Start a print job on another thread so we can
	// hijack the printer graphics and suspend the thread
	// while we draw on it.
	printerThread = new PrinterThread(printerJob);

	// if page templates were passed in, check first page for
	// PageFormat attributes such as orientation
	if (updatePageFormat && firstPage != null) {

		updatePageFormat(firstPage);

		// don't hang on to first template page, no longer needed
		firstPage = null;
	}

	// Set printable on printer
	printerJob.setPrintable(printerThread, pageFormat);
}

/**
 * Update the current PageFormat object with properties from the
 * current page being printed.  Currently, only orientation and
 * paper size are set on the PageFormat object.
 * @param page JCPage object (perhaps a template page, perhaps a real
 *             document page) to query for PageFormat properties
 */
protected void updatePageFormat(JCPage page)
{
	Paper paper = pageFormat.getPaper();
	if (paper == null) {
		paper = new Paper();
	}

	setOrientation(page.getOrientation());
	Rectangle2D pageSize = page.getPageSize();
	if (isLandscape(pageSize.getWidth(), pageSize.getHeight())) {
		pageFormat.setOrientation(PageFormat.LANDSCAPE);
		// in JCPage, location and size is relative to orientation,
		// in Paper, it is independent, so flip width/height and x/y
	    paper.setImageableArea(pageSize.getY(), pageSize.getX(), 
							pageSize.getHeight(), pageSize.getWidth());
	    paper.setSize(pageSize.getHeight(), pageSize.getWidth());
	} else {
		pageFormat.setOrientation(PageFormat.PORTRAIT);
	    paper.setImageableArea(pageSize.getX(), pageSize.getY(), 
							pageSize.getWidth(), pageSize.getHeight());
	    paper.setSize(pageSize.getWidth(), pageSize.getHeight());
	}

	pageFormat.setPaper(paper);
}


/**
 *
 * Retruns null if user cancel this print job.
 *
 */
public Graphics getGraphics()
{
	if (graphics == null) {
	    initPrinterJob();

		// true means user accepted the print job
		if (showDialog) {
			if (!printerJob.printDialog()) {
				return null;
			}
		}

		thread = new Thread(printerThread, "PageLayout print thread");
		thread.start();

		// now we wait until java's call-back printing calls the print
		// method below which passes in the coveted graphics object
		while (graphics == null && !abort) {
			try {
				Thread.currentThread().sleep(100);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}
	return(graphics);
}

/**
 * Print a range
 */
protected void printPage(JCDocument document, int pageIndex)
{
	// Called here because each page is treated as a doc in this printer
	super.beginPage(null, pageIndex);

	// print the page
	JCPage page = (JCPage) document.getPages().get(pageIndex);
	page.draw((Graphics2D) getGraphics());

	// called here because each page is treated as a doc in this printer
	super.endPage(null);
}

protected void setGraphics(Graphics2D g)
{
	this.graphics = g;
	// PR#10344 - Origin is misplaced down and right by approximately 13pt
	if (JCEnvironment.getJavaVersion() < 130) {
	    this.graphics.translate(-13,-13);
	}
}

/**
 * Is the print dialog to be shown when
 * printing is requested?
 * True means show it.
 * @deprecated Not implemented -- its intended functionality has been
 * superceded by the constructors for this class which take
 * the showDialog parameter.
 */
public void setShowDialog(boolean showDialog)
{
	// this.showDialog = showDialog;
}

/**
 * Is the print dialog to be shown when
 * printing is requested?
 * True means show it.
 */
public boolean getShowDialog()
{
	return(showDialog);
}

/**
 *
 * Cancel the pending job and kill the waiting thread.
 *
 */
public void cancel()
{
	abort = true;

	if (printerJob != null) {
		printerJob.cancel();
		if (printerThread != null) {
			printerThread.wakeUp();
		}
	}

}

/**
 *
 * Cancel the pending job and note the exception that caused the printer
 * thread to fail.
 *
 */
public void cancelFromPrinterThread(PrinterException e)
{
	abort = true;
	threadException = e;

	if (printerJob != null) {
		printerJob.cancel();
	}

}

/**
 *
 * To get proper metrics for the printer we need to get
 * its graphics. But, you cannot get a printer's graphics
 * unless you actually print something. Classic chicken and
 * egg problem. That is, we don't know what to print until
 * we have used the metrics to figure out what will fit on a
 * page. So, we start a print job to get the graphics context.
 * Once we have it we immediately suspend the printing thread
 * so we now have the luxury of using the gc. Once we have
 * rendered all the document's pages using said gc we awake
 * the printing thread to print those pages.
 *
 */
protected class PrinterThread implements Runnable, Printable  //, Pageable
{
	protected          PrinterJob 	pj;
	protected int      copies, first, last, pageCount;
	protected boolean  collate, reverse;
	protected int      currentPage;
	protected int	   lastPageIndex = -1;
	protected int	   pageIncr = 1;
	protected int      numPages;
	// if this is the first time the print method is being called
	protected boolean  firstPass = true;

	public PrinterThread(PrinterJob pj)
	{
		this.pj = pj;
	}

	/**
	 *
	 * This will be run on the "PageLayout print thread" thread.
	 * It will start the print job. Java2 printing uses a callback
	 * method of printing. The callback is made to the "print" method
	 * below. The coveted Graphics object is passed in.
	 *
	 */
	public void run()
	{
		try {
			firstPass = true;
			pj.print();
			// Only wake up the main thread after we've done printing
			JCAWTPrinter.this.clearThread();
			JCAWTPrinter.this.wakeUpMainThread();
		}
		catch (PrinterException e) {
			// Pass printer exception along to AWTPrinter class, who will
			// throw it once thread has finished
			JCAWTPrinter.this.clearThread();
			JCAWTPrinter.this.cancelFromPrinterThread(e);
		}
	}

	/**
	 *
	 * Called by the Java2 printing mechanism.
	 *
	 */
	public synchronized int print(Graphics g, PageFormat pf, int pageIndex)
	    throws PrinterException
	{
		int pageExists = Printable.PAGE_EXISTS;

		// Special case -- in some JVM's, ignore the first call to this
		// method and save the graphics class from the SECOND call, which
		// is the one that contains the actual Printer graphics class
		if (JCEnvironment.isWorking(
				JCEnvironment.PRINTING_USE_2ND_CALL) ==
									JCEnvironment.WORKING_YES)
		{
			if (firstPass) {
				firstPass = false;
				lastPageIndex = pageIndex;
				return(pageExists);
			}
		}

		// set the global graphics
		setGraphics((Graphics2D) g);

		try {
			// we now have the printer's graphics context
			// go to sleep and let PageLayout draw on it
			// this puts the "PageLayout print thread" thread
			// to sleep until pw is actually ready to print
			if (!actuallyPrinting) {
				wait();

				if (abort) {
					return Printable.NO_SUCH_PAGE;
				}

				// This cannot be put in openDocument method because this
				// printer treats each page as a document.
				JCAWTPrinter.this.openDocument(null, true);
			}
		}
		catch (InterruptedException e) {
			//
		}

		if (!reverse) {
			if (copies == 1) {
				currentPage = first + pageIndex;
			}
			else if (collate) {
				currentPage = first + (pageIndex % pageCount);
			}
			else {
				currentPage = first + (pageIndex / copies);
			}
			pageIncr = 1;
		}
		else {
	        //PR22704 still use first, because in reverse order it is
	        //the higher number
			if (copies == 1) {
				currentPage = first  - pageIndex;
			}
			else if (collate) {
				currentPage = first - (pageIndex % pageCount);
			}
			else {
				currentPage = first - (pageIndex / copies);
			}
			pageIncr = -1;
		}

		if (pageIndex < numPages) {

			printPage(document, currentPage);

			// decide whether the next page should be checked for
			// PageFormat properties
			boolean doNextPageCheck = 
				updatePageFormat && (pageIndex + 1 < numPages);

			// Special case -- in some JVM's, method is called twice per page,
			// so invalidate the page check for the first call
			if (doNextPageCheck &&
					JCEnvironment.isWorking(
						JCEnvironment.PRINTING_USE_2ND_CALL) ==
											JCEnvironment.WORKING_YES &&
					lastPageIndex != pageIndex) {

				doNextPageCheck = false;
			}

			// check next page and set PageFormat properties (such as
			// orientation) accordingly

			if (doNextPageCheck) {
				JCPage page = (JCPage) document.getPages().get(currentPage +
																	 pageIncr);
				updatePageFormat(page);
			}
		}
		else {
			// finished printing the document
			pageExists 			= Printable.NO_SUCH_PAGE;
			actuallyPrinting 	= false;

			// send the close document event
			JCAWTPrinter.this.closeDocument(null, true);

			// wake up the main thread only after this method has exited
			// otherwise, a calling program could exit before all pages
			// have been sent to the printer
		}

		lastPageIndex = pageIndex;
		return pageExists;
	}

	// When thread - main - calls this method
	// it becomes the object owner and thus has the
	// right to call notifyAll to wake up the printer
	// thread
	public synchronized void wakeUp()
	{
		// when this thread is woken, first set up the variables we'll need
		copies = getNumCopies();
		if (copies <= 0) {
			copies = 1;
		}
		collate = getCollate();
		reverse = getReverseOrder();

		first = firstToPrint;
		last = lastToPrint;
		pageCount = document.getPages().size();

        //PR22704 need to check for different things if order is reversed
		//for example in a reverse order first >= last is correct.
		if (!reverse) {
            if (first < 0) {
	        	first = 0;
			}
		    if (last == -1 || last >= pageCount) {
			    last = pageCount - 1;
		    }
		    if (first > last) {
			    first = last;
		    }
		    pageCount = last - first + 1;
		    numPages = pageCount * copies;
		}
		else {
		    if (last < 0) {
			    last = 0;
		    }
		    if (first == -1 || first >= pageCount) {
			    first = pageCount -1;
		    }
		    if (last > first) {
			    last = first;
		    }
		    pageCount = (first - last) + 1;
		    numPages = pageCount * copies;
		}

		currentPage = -1;
		notifyAll();
	}

} // end PrinterThread inner class

public static class PrinterJobCancelledException extends Exception
{

	public PrinterJobCancelledException()
	{
		super("User elected not to print");
	}

} // end PrinterJobCancelled inner class

} // end class
