/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCPrintPage.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:11:46 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page.awt;

import java.awt.*;
import javax.swing.*;

 
import java.awt.geom.Rectangle2D;





import com.klg.jclass.page.JCDocument;
import com.klg.jclass.page.JCPage;

/**
 * <code>PrintPage</code> displays a print page.
 */
public class JCPrintPage extends JComponent {

protected JCDocument document;
protected int page = -1;
protected Dimension preferredDimension;
protected Color bgColor = Color.white;
protected boolean requiresRepaint = true;
protected Image image;
protected JCAWTScreenPrinter printer;
	



/**
 * Constructor. Assumes 8.5 x 11 page.
 * @param printer
 */
public JCPrintPage(JCAWTScreenPrinter printer) {
	this.printer = printer;
	this.preferredDimension = new Dimension(612, 792);
}

/**
 * Constructs a Dimension and initializes it to the specified preferred
 * dimension.
 * @param printer
 * @param preferredDimension
 */
public JCPrintPage(JCAWTScreenPrinter printer, Dimension preferredDimension) {
	this.printer = printer;
	this.preferredDimension = preferredDimension;
}

/**
 * Sets the background color of the page;
 * default is white.
 * @param bgColor
 */
public void setBackground(Color bgColor) {
	this.bgColor = bgColor;
}

/**
 * Sets the document to be printed.
 * @param document
 */
public void setDocument(JCDocument document) {
	requiresRepaint = true;
	this.document = document;
}

/**
 * Sets the page to be printed.
 * @param page
 */
public void setPage(int page) {
	this.page = page;
	requiresRepaint = true;
	repaint();
}
	
/**
 * Gets the preferred size of the page.
 * @return
 */
public Dimension getPreferredSize() {
	return preferredDimension;
}

/**
 * Sets the preferred size of the page.
 * @param preferredDimension
 */
public void setPreferredSize(Dimension preferredDimension) {
	this.preferredDimension = preferredDimension;
}
	
/**
 * Paints the table image centered on the screen.
 * @param gc
 */
public void paintComponent(Graphics gc) {
	Dimension d = this.getSize();
	
	// draw the current page
	if (document != null && page != -1) {
		
		// if the page has not changed, blit its image on the screen,
		// otherwise, recreate the page and make an image of it
		if (requiresRepaint) {			
				
			// get the page so we can get its dimensions
			JCPage p = (JCPage) document.getPages().get(page); 

			// get its dimensions
			Rectangle2D pageSize = p.getPageSize();

			int w = (int) pageSize.getWidth();
			int h = (int) pageSize.getHeight();
			
			image = printer.getComponent().createImage(w, h);
			Graphics g = image.getGraphics();
			
			// draw the blank page
			g.setColor(bgColor);
			g.fillRect(0, 0, w, h);		
	
			// set its graphics to the gc passed in by java
			
			printer.setGraphics((Graphics2D) g);
			
			
			

			
			// get document to print on that gc
			document.print(printer, page, page);
	
			// reset the graphics
			printer.setGraphics(null);
						
			// Changed draw starting position from (10,10) to (0,0)
			gc.drawImage(image, 0, 0, printer.getComponent());
			
			requiresRepaint = false;
		}
		else if (image != null) {
			// Changed draw starting position from (10,10) to (0,0)
			gc.drawImage(image, 0, 0, printer.getComponent());
		}
	}
}	

/**
 * Makes sure the graphics are disposed, and cleans up the rest.
 */
public void dispose() {

	// Drop the document and clean up the image
	document = null;
	image.flush();
	image = null;

	

}

}
