/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: Graphics2DHTML.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:24 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page.html;

import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.awt.image.ImageObserver;
import java.util.StringTokenizer;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.io.IOException;

import com.klg.jclass.page.EPSImage;
import com.klg.jclass.page.FontBase;
import com.klg.jclass.page.Graphics2DBase;
import com.klg.jclass.page.JCTextStyle;
import com.klg.jclass.page.Markup;

import com.klg.jclass.page.awt.FontMetrics11;

import com.klg.jclass.util.swing.encode.JPEGEncoder;
import com.klg.jclass.util.swing.encode.EncoderException;


/**
 * Overrides Graphics2DBase to provide an API for generating HTML documents
 */
public class Graphics2DHTML extends Graphics2DBase implements Markup {


/** The default font from which all fonts are derived */
protected Font				defaultFont = JCTextStyle.DEFAULT_TEXT.getFont();
/** The point size of text being drawn to the document */
protected int				currentPointSize = defaultFont.getSize();
/** A font which represents the current font in the HTML stream */
protected Font				drawingFont = defaultFont;
/** The default colour of text in the document */
protected Color				defaultColour = JCTextStyle.DEFAULT_TEXT.getColor();
/** The current colour of text being added to the HTML document */
protected Color				currentColour = defaultColour;

/** The name of a directory to which the image files are dumped */
protected String			directoryName = null;

/** Flag if the text currently being drawn is in a fixed-width font */
protected boolean			isCurrentMonospace = false;
/** Flag if the text currently being drawn is underlined */
protected boolean			isCurrentUnderline = false;
/** Flag if the text currently being drawn is positioned as a subscript */
protected boolean			isCurrentSubscript = false;
/** Flag if the text currently being drawn is positioned as a superscript */
protected boolean			isCurrentSuperscript = false;

/** Note if the current context is enclosed by a <div> tag */
protected boolean			inDivision = false;

/** This array defines the mapping from point size to HTML text size */
protected static final int[]		pointMapping = {
								1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 3, 3,
								4, 4, 4, 5, 5, 5, 5, 5, 6, 6, 6, 6, 6, 6,
								6, 6, 6, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7 };

/** Identifies an attribute which specifies the text colour */
protected static final int		ATTRIBUTE_COLOUR = 1;
/** Identifies an attribute which specifies the text size */
protected static final int		ATTRIBUTE_SIZE = 2;
/** Identifies an attribute which specifies the use of fixed-width text */
protected static final int		ATTRIBUTE_MONOSPACE = 4;
/** Identifies an attribute which specifies the drawing of text in bold */
protected static final int		ATTRIBUTE_BOLD = 8;
/** Identifies an attribute which specifies the drawing of text in italic */
protected static final int		ATTRIBUTE_ITALIC = 16;
/** Identifies an attribute which specifies the drawing of text underlined */
protected static final int		ATTRIBUTE_UNDERLINE = 32;
/** Identifies an attribute which specifies the text be drawn as a subscript */
protected static final int		ATTRIBUTE_SUBSCRIPT = 64;
/** Identifies an attribute which specifies the text is a superscript */
protected static final int		ATTRIBUTE_SUPERSCRIPT = 128;

/** This array contains the list of all possible text attributes */
protected static final int[]	attributeSet =
								{ ATTRIBUTE_COLOUR, ATTRIBUTE_SIZE,
								  ATTRIBUTE_MONOSPACE, ATTRIBUTE_BOLD,
								  ATTRIBUTE_ITALIC, ATTRIBUTE_UNDERLINE,
								  ATTRIBUTE_SUBSCRIPT, ATTRIBUTE_SUPERSCRIPT };

/** This array lists the attributes currently applied to document text */
protected int[]				appliedAttributes = new int[attributeSet.length];
/** The number of attributes currently in effect on the document */
protected int				appliedAttributeCount = 0;

/** The HTML names for the characters in the ISO-Latin-1 character set */
protected static final String[]	charNames = {
		"&#000;", "&#001;", "&#002;", "&#003;", "&#004;", "&#005;", "&#006;",
		"&#007;", "&#008;", "&#009;", "&#010;", "&#011;", "&#012;", "&#013;",
		"&#014;", "&#015;", "&#016;", "&#017;", "&#018;", "&#019;", "&#020;",
		"&#021;", "&#022;", "&#023;", "&#024;", "&#025;", "&#026;", "&#027;",
		"&#028;", "&#029;", "&#030;", "&#031;", " ", "!", "\"", "#", "$", "%",
		"&amp;", "'", "(", ")", "*", "+", ",", "-", ".", "/", "0", "1", "2",
		"3", "4", "5", "6", "7", "8", "9", ":", ";", "&lt;", "=", "&gt;", "?",
		"@", "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M",
		"N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z", "[",
		"\\", "]", "^", "_", "`", "a", "b", "c", "d", "e", "f", "g", "h", "i",
		"j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w",
		"x", "y", "z", "{", "|", "}", "~", "&#127;", "&#128;", "&#129;",
		"&#130;", "&#131;", "&#132;", "&#133;", "&#134;", "&#135;", "&#136;",
		"&#137;", "&#138;", "&#139;", "&#140;", "&#141;", "&#142;", "&#143;",
		"&#144;", "&#145;", "&#146;", "&#147;", "&#148;", "&#149;", "&#150;",
		"&#151;", "&#152;", "&#153;", "&#154;", "&#155;", "&#156;", "&#157;",
		"&#158;", "&#159;", "&nbsp;", "&iexcl;", "&cent;", "&pound;",
		"&curren;", "&yen;", "&brvbar;", "&sect", "&uml;", "&copy;", "&ordf;",
		"&laquo;", "&not;", "&shy;", "&reg;", "&macr;", "&deg;", "&plusmn;",
		"&sup2;", "&sup3;", "&acute;", "&micro;", "&para;", "&middot;",
		"&cedil;", "&sup1;", "&ordm;", "&raquo;", "&frac14;", "&frac12;",
		"&frac34;", "&iquest;", "&Agrave;", "&Aacute;", "&Acirc;", "&Atilde;",
		"&Auml;", "&Aring;", "&AElig;", "&Ccedil;", "&Egrave;", "&Eacute;",
		"&Ecirc;", "&Euml;", "&Igrave;", "&Iacute;", "&Icirc;", "&Iuml;",
		"&ETH;", "&Ntilde;", "&Ograve;", "&Oacute;", "&Ocirc;", "&Otilde;",
		"&Ouml;", "&times;", "&Oslash;", "&Ugrave;", "&Uacute;", "&Ucirc;",
		"&Uuml;", "&Yacute;", "&THORN;", "&szlig;", "&agrave;", "&aacute;",
		"&acirc;", "&atilde;", "&auml;", "&aring;", "&aelig;", "&ccedil;",
		"&egrave;", "&eacute;", "&ecirc;", "&euml;", "&igrave;", "&iacute;",
		"&icirc;", "&iuml;", "&eth;", "&ntilde;", "&ograve;", "&oacute;",
		"&ocirc;", "&otilde;", "&ouml;", "&divide;", "&oslash;", "&ugrave;",
		"&uacute;", "&ucirc;", "&uuml;", "&yacute;", "&thorn;", "&yuml;" };


/**
 * Create an HTML implementation of the Graphics2D API given an OutputStream
 * @param os The OutputStream to which the document will be written
 */
public Graphics2DHTML(OutputStream os) {
	super(os);
}

/**
 * Create an HTML implementation of the Graphics2D API given an OutputStream
 * @param os The OutputStream to which the document will be written
 * @param directory The name of a directory to write image files to
 */
public Graphics2DHTML(OutputStream os, String directory) {

	super(os);
	if (directory != null) {
		directoryName = new String(directory);
	}
}

/**
 * Overrides g2d method.
 */
public void drawString(String string, float x, float y) {

	// Make sure we have defined the correct style to output the text in
	outputTextStyle(getFont(), getColor(), underline, subscript, false);

	// Write the string to the output
	output(escapeCharacters(string));
}

/**
 * Override the Graphics2D translate method
 * @param x the X co-ordinate of the translation
 * @param y the Y co-ordinate of the translation
 */
public void translate(int x, int y) {
	translate((double) x, (double) y);
}

/**
 * Override the Graphics2D translate method
 * @param x the X co-ordinate of the translation
 * @param y the Y co-ordinate of the translation
 */
public void translate(double x, double y) {
    Rectangle2D clip = (Rectangle2D)getClip();

	double clipX, clipY, clipWidth, clipHeight;

	// Accumulate the translation to the current transformation matrix
	transform.translate(x, y);

	// Adjust the clip so it doesn't move
	clipX = clip.getX() - x;
	clipY = clip.getY() - y;
	clipWidth = clip.getWidth();
	clipHeight = clip.getHeight();
	clip.setRect(clipX, clipY, clipWidth, clipHeight);
}

/**
 * Override the Graphics2D scale method
 * @param x the X factor of the scale
 * @param y the Y factor of the scale
 */
public void scale(double x, double y) {
    Rectangle2D clip = (Rectangle2D)getClip();

	double clipX, clipY, clipWidth, clipHeight;

	// Accumulate the scaling to the current transformation matrix
	transform.scale(x, y);

	// Adjust the clip so it doesn't move
	clipX = clip.getX() / x;
	clipY = clip.getY() / y;
	clipWidth = clip.getWidth() / x;
	clipHeight = clip.getHeight() / y;
	clip.setRect(clipX, clipY, clipWidth, clipHeight);
}

/**
 * Override the Graphics2D clip method
 * @param x the X co-ordinate of the clip origin
 * @param y the Y co-ordinate of the clip origin
 * @param width the width of the clip window
 * @param height the height of the clip window
 */
public void clipRect(int x, int y, int width, int height) {
	clipRect((double) x, (double) y, (double) width, (double) height);
}

/**
 * Override the Graphics2D clip method
 * @param x the X co-ordinate of the clip origin
 * @param y the Y co-ordinate of the clip origin
 * @param width the width of the clip window
 * @param height the height of the clip window
 */
public void clipRect(double x, double y, double width, double height) {
    Rectangle2D clip = (Rectangle2D)getClip();

	// if there is a clip rectangle set already, intersect them
	if (clipSet) {

		// Store the intersection of the new and old rectangles
		Rectangle2D intersectionRect = new Rectangle2D.Double();
		Rectangle2D.intersect(clip, new Rectangle2D.Double(x, y, width, height),
							  intersectionRect);
		clip = intersectionRect;
	}

	// otherwise (no existing clip) use the given clip
	else {
		clip.setRect(x, y, width, height);
	}
}

//////////////////// Some Graphics (not 2D) methods /////////

/**
 * Overrides Graphics method
 */
public void drawPolyline(int[] xPoints, int[] yPoints, int nPoints) {
	// This does nothing in HTML
}

/**
 * Overrides Graphics method
 */
public void drawPolygon(int[] xPoints, int[] yPoints, int nPoints) {
	// This does nothing in HTML
}

/**
 * Overrides Graphics method
 */
public void fillPolygon(int[] xPoints, int[] yPoints, int nPoints) {
	// This does nothing in HTML
}

/**
 * Overrides Graphics method
 */
public boolean drawImage(Image img, int x, int y, int width, int height,
		Color bgcolor, ImageObserver observer) {

	if (img instanceof EPSImage) {
		return drawEPSImage((EPSImage)img, x, y, width, height, bgcolor, observer);
	}
	else {
		return drawJavaImage(img, x, y, width, height, bgcolor, observer);
	}
}

/**
 * Overrides Graphics method
 */
public void setFont(Font font) {

	// Make sure we are in the correct state
	updateContext();

	// only set it if we must
	if (currentFont != null && currentFont.equals(font)) {
		return;
	}

	this.currentFont	= font;
	String name			= font.getName();
	int size			= font.getSize();
}

public FontMetrics getFontMetrics(Font font) {

	FontMetrics fm = java.awt.Toolkit.getDefaultToolkit().getFontMetrics(font);
	return(new FontMetrics11(font, fm));
	// return new FontMetrics(font);
}

////////////////////////// Markup interface methods ////////////////////

/**
 * Start a table in the output
 * @param columns Number of columns in the table
 */
public void startTable(int columns) {

	// Since the inheritance of attributes inside a table seems to be a bit
	// idiosyncratic (not the same for all), we close all text attributes.
	if (appliedAttributeCount > 0) {
		removeAttributes(0);
	}

	// TODO:  Need to set table attributes.
	output("<table>");
}

/**
 * Start a new table row
 */
public void startRow() {

	output("\t<tr>");
}

/**
 * Start a table cell
 */
public void startCell() {

	startCell("left", false, 1, 1);
}

/**
 * Start a table cell
 * @param hAlignment The horizontal alignment of the cell contents
 * @param isHeaderCel Indicates if this cell is to be marked as a header cell
 */
public void startCell(String hAlignment, boolean isHeaderCell) {

	startCell(hAlignment, isHeaderCell, 1, 1);
}

/**
 * Start a spanned cell of the table
 * @param rows the number of rows the span overlays
 * @param columns the number of columns overlain by the spanned cell
 */
public void startCell(int rows, int columns) {
	startCell("left", false, rows, columns);
}

/**
 * Start a spanned cell of the table
 * @param hAlignment The horizontal alignment of the cell contents
 * @param isHeaderCel Indicates if this cell is to be marked as a header cell
 * @param rows the number of rows the span overlays
 * @param columns the number of columns overlain by the spanned cell
 */
public void startCell(String hAlignment, boolean isHeaderCell,
					  int rows, int columns) {

	StringBuffer command = new StringBuffer();

	if (isHeaderCell) {
		command.append("\t\t<th");
	}
	else {
		command.append("\t\t<td");
	}

	if (hAlignment != null && !hAlignment.equals("left")) {
		command.append(" align=" + hAlignment);
	}

	if (rows > 1) {
		command.append(" rowspan=\"" + rows + "\"");
	}
	if (columns > 1) {
		command.append(" colspan=\"" + columns + "\"");
	}

	command.append(">");
	output(command.toString());
}

/**
 * End a cell
 */
// NOTE:  Should this be implicit in the next start{Row|Cell} action?
public void endCell() {
	endCell(false);
}

/**
 * End a cell
 * @param isHeader True if the cell belongs to the headers of the table
 */
public void endCell(boolean isHeaderCell) {

	// Clean up any attributes set in this cell.
	if (appliedAttributeCount > 0) {
		removeAttributes(0);
	}

	if (isHeaderCell) {
		output("\t\t</th>");
	}
	else {
		output("\t\t</td>");
	}
}

/**
 * End a row
 */
// NOTE:  Should this be implicit in the next startRow action?
public void endRow() {

	output("\t</tr>");
}

/**
 * End a table
 */
public void endTable() {

	output("</table>");
}

/**
 * Mark an advance to the next line
 */
public void newLine() {

	output("<br>");
}

/**
 * Mark an advance to a new paragraph
 */
public void newParagraph() {

	output("<p>");
}

/**
 * Mark an advance to a new paragraph
 * @param hAlignment The alignment of lines of text in this paragraph
 */
public void newParagraph(String hAlignment) {

	if (hAlignment != null && !hAlignment.equals("left")) {
		output("<p align=" + hAlignment + ">");
	}
	else {
		output("<p>");
	}
}

/**
 * Begin a new document division with the given text alignment
 * @param hAlignment The alignment of lines of text in this division
 */
public void newSection(String hAlignment) {

	// Close all text attributes
	if (appliedAttributeCount > 0) {
		removeAttributes(0);
	}

	// If a division is current, end it
	if (inDivision) {
		output("</div>");
	}

	output("<div align=" + hAlignment + ">");
	inDivision = true;
}

/**
 * End any current document division
 */
public void endSection() {

	// Close all text attributes
	if (appliedAttributeCount > 0) {
		removeAttributes(0);
	}

	// If a division is current, end it
	if (inDivision) {
		output("</div>");
		inDivision = false;
	}
}

/**
 * Draw a horizontal rule
 */
public void hRule() {

	output("<hr>");
}

/**
 * Draw a horizontal rule
 * @param percent The percentage width of the document to span with the rule
 */
public void hRule(double percent) {

	output("<hr width=" + percent + "%>");
}

/**
 * Draw a horizontal rule with the given appearance
 * @param percent The percentage width of the document to span with the rule
 * @param hAlignment The alignment of the hRule to the margins
 * @param thickness The thickness of the line used to draw the hRule
 */
public void hRule(double percent, String hAlignment, int thickness) {

	output("<hr width=" + percent + "% align=" + hAlignment + " size=" +
		   thickness + ">");
}

/**
 * Draw a horizontal rule
 * @param width The width of the document to span with the rule
 */
public void hRule(int pixelWidth) {

	output("<hr width=" + pixelWidth + ">");
}

/**
 * Draw a horizontal rule
 * @param width The width of the document to span with the rule
 * @param hAlignment The alignment of the hRule to the margins
 * @param thickness The thickness of the line used to draw the hRule
 */
public void hRule(int pixelWidth, String hAlignment, int thickness) {

	output("<hr width=" + pixelWidth + "align=" + hAlignment + " size=" +
		   thickness + ">");
}

/**
 * Emit image with markup alignment
 */
public boolean outputImage(Image img, int width, int height, Color bgcolor,
							  ImageObserver obs, String hAlign, String vAlign) {

	int		oHeight, oWidth;
	Image	outImage = null;

	// Get the original size of the image
	oHeight = img.getHeight(obs);
	oWidth = img.getWidth(obs);

	// compute image size. First of all, if width or height is 0, image is 1:1.
	if (height == 0 && width == 0) {
		height = oHeight;
		width = oWidth;
		outImage = img;
	}
	// if the height is unspecified, rescale based on width
	else if (height == 0) {
		height = oHeight * width / oWidth;
	}
	// if the width is unspecified, rescale based on height
	else if (width == 0) {
		width = oWidth * height / oHeight;
	}
	// If the specified dimensions are the existing dimenions, image is 1:1
	else if (height == oHeight && width == oWidth) {
		outImage = img;
	}
	// otherwise both height and width are used to rescale

	// if the original image isn't being used, compute the rescaled image
	if (outImage == null) {
		outImage = img.getScaledInstance(width, height, Image.SCALE_DEFAULT);

		while (outImage.getHeight(obs) != height ||
			   outImage.getWidth(obs) != width) {
			try {
				Thread.sleep(5);
			}
			catch (InterruptedException ie) {}
		}
	}

	// Store the image in a file and get the file's name
	String filename = createImageFile(outImage);

	// Initialise the image command
	StringBuffer imgTag = new StringBuffer("<img src=\"" + filename + "\"");

	// There is currently no way to provide a meaningful alt description

	// Add the alignment
	if (hAlign != null) {
		// imgTag.append(" align=" + hAlign);
		output("<p align=" + hAlign + ">");
	}
	else if (vAlign != null) {
		imgTag.append(" align=" + vAlign);
	}

	// Add the dimensions of the image
	imgTag.append(" width=" + width + " height=" + height + ">");

	// Write out the image tag
	output(imgTag.toString());

	// If we specified the horizontal alignment, unset it
	if (hAlign != null) {
		output("</p>");
	}

	return true;
}

////////////////////////// Device-specific methods /////////////////////

/**
 * Enter a graphics
 */
public void startContext() {

	// This is a null method for HTML, since we never really switch contexts
}

/**
 * Exit a graphics
 */
public void endContext() {

	// This is a null method for HTML, since we never really switch contexts
}

/**
 * Get the name of the directory used for image files
 * @return The name of the directory
 */
public String getDirectory() {
	return(directoryName);
}

/**
 * Set the name of the directory used for image files
 * @param name The name of the directory
 */
public void setDirectory(String dirName) {

	if (dirName != null) {
		directoryName = new String(dirName);
	}
	else {
		directoryName = null;
	}
}

/**
 * Device-specific arc-drawing method
 */
public void drawArcPW(double x, double y, double width, double height,
					  double startAngle, double arcAngle, int type) {

	// This is a null method since HTML doesn't support line-drawing
}

/**
 * Device-specific method for drawing a filled arc
 */
public void fillArcPW(double x, double y, double width, double height,
					  double startAngle, double arcAngle, int type) {

	// This is a null method since HTML doesn't support line-drawing
}

/**
 * Device-specific method for setting the clip window
 */
public  void setClipPW(double x, double y, double width, double height) {

	// This is a null method since HTML doesn't support clipping
}

/**
 * Device-specific draw method for filled rectangles
 */
public void fillRectPW(double x, double y, double width, double height) {

	// This is a null method since HTML doesn't support line-drawing
}

/**
 * Device-specific draw method for rectangles
 */
public void drawRectPW(double x, double y, double width, double height) {

	// This is a null method since HTML doesn't support line-drawing
}

/**
 * Device-specific draw line method
 */
public void drawLinePW(double x1, double y1, double x2, double y2) {

	// This is a null method since HTML doesn't support line-drawing
}

/**
 * Device-specific method for a rectangle with round corners
 */
public void drawRoundRectPW(double x, double y, double width,
							double height, double arcWidth, double arcHeight) {

	// This is a null method since HTML doesn't support line-drawing
}

/**
 * Device-specific filled rounded-corner rectangle
 */
public void fillRoundRectPW(double x, double y, double width,
							double height, double arcWidth, double arcHeight) {

	// This is a null method since HTML doesn't support line-drawing
}

/**
 * The fonts have to be reloaded, so we need to dump the font list
 */
public void flushLoadedFonts() {
	// NOTE: Do we need this method?
	loadedFonts.clear();
}

/**
 * Overridden to be sure returned object is an instance of Rectangle2D
 * @return super.getClip(), if it returns a Rectangle2D. Otherwise,
 *         it's getBounds2D().
 */
public Shape getClip(){
    Shape result;
    if (super.getClip() instanceof Rectangle2D){
        result = super.getClip();
    }
    else{
        result = super.getClip().getBounds2D();
    }
    return result;
}

//////////// Miscellaneous non-Graphics2D, non Graphics methods //////////

/**
 * Draw a blank box to signify an eps image
 */
public boolean drawEPSImage(EPSImage img, int x, int y, int width, int height,
							Color bgcolor, ImageObserver observer) {

	// NOTE: We use a single-cell table coloured the background colour to
	// represent the missing EPS image

	// TODO: Use a one-pixel grey gif tiled into the appropriate space

	return true;
}


/**
 * Given a String, return a copy of the String with special characters
 * replaced with their escape sequences.
 */
protected String escapeCharacters(String inString) {

	int		runStart = -1;
	String	specials = "&<>";

	StringBuffer outString = new StringBuffer();

	// Ideally we'd like to be able to call a method like C's strpBrk()
	// so we could find groups of characters to copy in one operation

	for (int i = 0; i < inString.length(); i++) {
		char code = inString.charAt(i);

		// the length of the output buffer can be wider than the number of
		// characters in the string when we have to convert chars to names

		// outside the realm of printable characters
		if ((int) code < 32 || (int) code > 126) {

			// Output any run of printable characters accumulated to this point
			if (runStart > -1) {
				outString.append(inString.substring(runStart, i));
				runStart = -1;
			}

			// inside ansi range -- find the character name and use that
			if ((int) code <= 255) {
				// append the name of the non-printing character to the string
				outString.append(charNames[code]);
			// outside ansi range -- use unicode value
			} else {
				StringBuffer codeSB = new StringBuffer();
				codeSB.append(Integer.toHexString((int) code));
				while (codeSB.length() < 4) {
					codeSB.insert(0, '0');
				}
				codeSB.insert(0, 'x');
				codeSB.insert(0, '#');
				codeSB.insert(0, '&');
				codeSB.append(';');
				outString.append(codeSB.toString());
			}
		}

		// also if it's a special char, we need to replace it
		else if (specials.indexOf(code) != -1) {

			// Output any run of printable characters accumulated to this point
			if (runStart > -1) {
				outString.append(inString.substring(runStart, i));
				runStart = -1;
			}

			// append the name of the special character to the string
			outString.append(charNames[code]);
		}

		// otherwise the character is a normal printable character
		else {

			// If we're not in a run, start one.
			if (runStart == -1) {
				runStart = i;
			}
		}
	}

	// if the entire string consisted of printable characters
	if (runStart == 0) {
		// return the original string
		return(inString);
	}

	// otherwise if there is a final run of printable characters
	if (runStart > -1) {
		// Output any final run of printable characters
		outString.append(inString.substring(runStart));
	}

	// Return the converted string
	return(outString.toString());
}

protected String getColorString(Color c) {

	// convert the color value to a string
	String value = "#" + hexCode(c.getRed(), 2) + hexCode(c.getGreen(), 2) +
				   hexCode(c.getBlue(), 2);

	return(value);
}

/**
 * Get gex code for supplied integer.
 */
private String hexCode(int r) {
	return hexCode(r, -1);
}

/**
 * Get hex code for supplied integer.  If digits is greated than 0,
 * pad resulting code with leading zeroes to specified number of digits.
 */
private String hexCode(int r, int digits) {

	int				current = 0;
	int[]			codes = new int[8];
	StringBuffer	sb = new StringBuffer();
	char[]			hex = {'0', '1', '2', '3', '4', '5', '6', '7',
						   '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};

	// Extract the hex codes from the int (back to front)
	while (r > 0) {
		codes[current++] = r % 16;
		r /= 16;
	}

	// pad with leading zeroes
	if (digits > current) {
		for (int i = 0; i < (digits-current); i++) {
			sb.append('0');
		}
	}

	// Convert to hex digits, reversing the order
	current--;
	for (; current >= 0; current--) {
		sb.append(hex[codes[current]]);
	}

	// Convert the hex number to a String and return
	return(sb.toString());
}

/**
 * Set the text style attributes for any following drawString() actions
 * @param font The font in which to output the text
 * @param color The colour the text is printed in
 * @param isUnderline True if the text is to be underlined
 * @param isSubscript True if the text is positioned on the line as a subscript
 * @param isSuperscript True if the text is in the superscript position
 */
private void outputTextStyle(Font font, Color color, boolean isUnderline,
							 boolean isSubscript, boolean isSuperscript) {

	int index = 0;

	// Check for already-applied attributes which persist in the current mode
	while (index < appliedAttributeCount) {

		// if an attribute has changed, remove all nested attributes
		if (!isAttributeConstant(appliedAttributes[index], font, color,
								 isUnderline, isSubscript, isSuperscript)) {

			removeAttributes(index);
		}
		index++;
	}

	// Apply any attributes that need to be (re-)applied
	for (index = 0; index < attributeSet.length; index++) {

		// if an attribute needs to be set, set it
		if (!isAttributeConstant(attributeSet[index], font, color, isUnderline,
								 isSubscript, isSuperscript)) {

			applyAttribute(attributeSet[index], font, color, isUnderline,
						   isSubscript, isSuperscript);
		}
	}
}

// Open a given attribute tag
private void applyAttribute(int attributeID, Font font, Color colour,
							boolean isUnderline, boolean isSubscript,
							boolean isSuperscript) {

	switch (attributeID) {

	case ATTRIBUTE_COLOUR:

		// Don't set a colour which agrees with the default
		// (then we don't have to make sure we unset it)
		if (colour == null || colour.equals(defaultColour)) {
			return;
		}

		output("<font color=" + getColorString(colour) + ">");
		currentColour = colour;
		break;

	case ATTRIBUTE_SIZE:

		// Don't set a size which agrees with the default
		// (then we don't have to make sure we unset it)
		if (font.getSize() == currentPointSize) {
			return;
		}

		int base = pointMapping[currentPointSize];
		int size = pointMapping[font.getSize()];

		// if both point sizes map to the same size increment, forget it
		if (base == size) {
			return;
		}

		int sizeDelta;
		String posNeg;

		// determine if the new size is larger or smaller, and by how much
		if (base > size) {
			posNeg = "-";
			sizeDelta = base - size;
		}
		else {  // size > base
			posNeg = "+";
			sizeDelta = size - base;
		}

		// the maximum deviation from the base size is 4
		if (sizeDelta > 4) {
			sizeDelta = 4;
		}

		output("<font size=" + posNeg + sizeDelta + ">");
		currentPointSize = font.getSize();
		break;

	case ATTRIBUTE_MONOSPACE:
		output("<tt>");
		isCurrentMonospace = true;
		break;

	case ATTRIBUTE_ITALIC:
		output("<i>");
		drawingFont = FontBase.deriveFont(drawingFont,
										  drawingFont.getStyle() | Font.ITALIC);
		break;

	case ATTRIBUTE_BOLD:
		output("<b>");
		drawingFont = FontBase.deriveFont(drawingFont,
										  drawingFont.getStyle() | Font.BOLD);
		break;

	case ATTRIBUTE_UNDERLINE:
		output("<u>");
		isCurrentUnderline = true;
		break;

	case ATTRIBUTE_SUBSCRIPT:
		output("<sub>");
		isCurrentSubscript = true;
		break;

	case ATTRIBUTE_SUPERSCRIPT:
		output("<sup>");
		isCurrentSuperscript = true;
		break;

	default:
		throw new IllegalArgumentException("Unrecognised attribute type");
	}

	// Add the attribute to the list of applied attributes
	appliedAttributes[appliedAttributeCount++] = attributeID;
}

// Close a given attribute tag
private void removeAttribute(int attributeID) {

	switch (attributeID) {

	case ATTRIBUTE_COLOUR:
		output("</font>");
		currentColour = defaultColour;
		break;

	case ATTRIBUTE_SIZE:
		output("</font>");
		currentPointSize = defaultFont.getSize();
		break;

	case ATTRIBUTE_MONOSPACE:
		output("</tt>");
		isCurrentMonospace = false;
		break;

	case ATTRIBUTE_ITALIC:
		output("</i>");
		drawingFont = FontBase.deriveFont(drawingFont,
									  drawingFont.getStyle() & ~Font.ITALIC);
		break;

	case ATTRIBUTE_BOLD:
		output("</b>");
		drawingFont = FontBase.deriveFont(drawingFont,
										  drawingFont.getStyle() & ~Font.BOLD);
		break;

	case ATTRIBUTE_UNDERLINE:
		output("</u>");
		isCurrentUnderline = false;
		break;

	case ATTRIBUTE_SUBSCRIPT:
		output("</sub>");
		isCurrentSubscript = false;
		break;

	case ATTRIBUTE_SUPERSCRIPT:
		output("</sup>");
		isCurrentSuperscript = false;
		break;

	default:
		throw new IllegalArgumentException("Unrecognised attribute type" +
										   attributeID);
	}

	// Remove the attribute from the list of applied attributes
	appliedAttributeCount--;
}

// Close the tag applying this attribute, and all attributes nested inside it
private void removeAttributes(int removeFrom) {

	while (appliedAttributeCount > removeFrom) {
		removeAttribute(appliedAttributes[appliedAttributeCount - 1]);
	}
}

// Determine if the specified attribute will be the same as its current
// setting given the new text style data
private boolean isAttributeConstant(int attributeID, Font font, Color colour,
									boolean isUnderline, boolean isSubscript,
									boolean isSuperscript) {

	switch (attributeID) {

	case ATTRIBUTE_COLOUR:

		// We count no given colour as same as current colour
		if (colour == null || colour.equals(currentColour)) {
			return(true);
		}
		return(false);

	case ATTRIBUTE_SIZE:

		if (font.getSize() != currentPointSize &&
			pointMapping[font.getSize()] != pointMapping[currentPointSize]) {
			return(false);
		}
		return(true);

	case ATTRIBUTE_MONOSPACE:

		boolean isNewMonospace = false;
		if (font.getName() != defaultFont.getName() &&
			font.getName().indexOf("Courier") != -1) {
			isNewMonospace = true;
		}

		return(isNewMonospace == isCurrentMonospace);

	case ATTRIBUTE_ITALIC:

		if (font.isItalic() != drawingFont.isItalic()) {
			return(false);
		}
		return(true);

	case ATTRIBUTE_BOLD:

		if (font.isBold() != drawingFont.isBold()) {
			return(false);
		}
		return(true);

	case ATTRIBUTE_UNDERLINE:

		if (isUnderline != isCurrentUnderline) {
			return(false);
		}
		return(true);

	case ATTRIBUTE_SUBSCRIPT:

		if (isSubscript != isCurrentSubscript) {
			return(false);
		}
		return(true);

	case ATTRIBUTE_SUPERSCRIPT:

		if (isSuperscript != isCurrentSuperscript) {
			return(false);
		}
		return(true);

	default:
		throw new IllegalArgumentException("Unrecognised attribute type");
	}
}

/**
 * Close all style/division/etc. tags which are currently open
 */
public void closeOpenTags() {

	// Close all text attributes
	if (appliedAttributeCount > 0) {
		removeAttributes(0);
	}

	// If a division is current, end it
	if (inDivision) {
		output("</div>");
	}

}

/**
 * Some common code for polygons
 */
/*
protected void printPolygon(int[] xPoints, int[] yPoints, int nPoints,
							String strokeOrFill) {

	// This is a null method since HTML doesn't support line-drawing
}
*/

/**
 * Some common code for drawing round rectangles.
 */
/*
protected void drawRRect(double x, double y, double width,
						 double height, double arcWidth, double arcHeight,
						 String strokeOrFill) {

	// This is a null method since HTML doesn't support line-drawing
}
*/

// output the line stroke if it has changed
protected void setLineStroke() {

	// This is a null method since HTML doesn't support line-drawing
}

// Use this JPEGEncoder to encode the images as JPEG images
private JPEGEncoder jpegWriter = new JPEGEncoder();

/**
 * Draw normalized jpeg/gif image, that is, pixels which encode rgb info.
 * Note that since the image is pre-scaled, the width and height parameters
 * represent the actual scaled size of the image.
 */
protected boolean drawJavaImage(Image img, int x, int y, int width, int height,
								Color bgcolor, ImageObserver observer) {


	int		oHeight, oWidth;
	Image	outImage = null;

	// Get the original size of the image
	oHeight = img.getHeight(observer);
	oWidth = img.getWidth(observer);

	// compute image size. First of all, if width or height is 0, image is 1:1.
	if (height == 0 && width == 0) {
		height = oHeight;
		width = oWidth;
		outImage = img;
	}
	// if the height is unspecified, rescale based on width
	else if (height == 0) {
		height = oHeight * width / oWidth;
	}
	// if the width is unspecified, rescale based on height
	else if (width == 0) {
		width = oWidth * height / oHeight;
	}
	// If the specified dimensions are the existing dimenions, image is 1:1
	else if (height == oHeight && width == oWidth) {
		outImage = img;
	}
	// otherwise both height and width are used to rescale

	// if the original image isn't being used, compute the rescaled image
	if (outImage == null) {
		outImage = img.getScaledInstance(width, height, Image.SCALE_DEFAULT);

		while (outImage.getHeight(observer) != height ||
			   outImage.getWidth(observer) != width) {
			try {
				Thread.sleep(5);
			}
			catch (InterruptedException ie) {}
		}
	}

	// Store the image in a file and get the file's name
	String filename = createImageFile(outImage);

	// Initialise the image command
	StringBuffer imgTag = new StringBuffer("<img src=\"" + filename + "\"");

	// There is currently no way to provide a meaningful alt description
	// nor to interpolate an alignment

	// Add the dimensions of the image
	imgTag.append(" width=" + width + " height=" + height + ">");

	// Write out the image tag
	output(imgTag.toString());

	return true;
}

// Create a file which holds the image and return the file's name
private String createImageFile(Image image) {

	File				imageFile;
	FileOutputStream	img_out;
	String				urlName;

	// Check the printer for a directory to store the image in
	if (directoryName == null) {
		return(null);
	}

	// Generate a filename for the image and create the file
	String filename = "plimg" + System.currentTimeMillis() + ".jpg";
	imageFile = new File(directoryName, filename);

	
	try {
		if (!imageFile.createNewFile()) {
			return(null);
		}
	}
	catch (IOException ioe) {
		return(null);
	}
	



	// Create the output stream and store the image
	try {
		img_out = new FileOutputStream(imageFile);
		jpegWriter.saveImage(image, img_out);
	}
	catch (IOException ioe) {
		imageFile.delete();
		return(null);
	}
	catch (EncoderException ee) {
		imageFile.delete();
		return(null);
	}

	// Close the output stream
	try {
		img_out.close();
	}
	catch (IOException ioe) {}

	
	try {
		urlName = imageFile.toURL().getFile();
	}
	catch (IOException ioe) {
		try {
			urlName = makeURL(imageFile.getCanonicalPath());
		}
		catch (IOException ioe2) {
			imageFile.delete();
			return(null);
		}
	}
	



	// Return the pathname as a URL
	return(urlName);
}

private String makeURL(String path) {

	// if we have a unix-like pathname, return it
	if (path.charAt(0) == '/' && File.separatorChar == '/') {
		return(path);
	}

	StringBuffer outName = new StringBuffer();

	// if we have a DOS-like pathname, munge the drive letter
	if (Character.isLetter(path.charAt(0)) && path.charAt(1) == ':') {
		outName.append("/");
		outName.append(path.charAt(0));
		outName.append("|/");

		path = path.substring(2);
	}

	// for other path types, make sure the path is rooted.
	else {
		if (path.charAt(0) != File.separatorChar) {
			outName.append("/");
		}
	}

	// Convert the path to unix/html-style
	StringTokenizer st = new StringTokenizer(path.substring(2),
											 File.separator, true);

	while (st.hasMoreTokens()) {
		String element = st.nextToken();
		if (element.equals(File.separator)) {
			outName.append("/");
		}
		else {
			outName.append(element);
		}
	}

	// return the URLified name
	return(outName.toString());
}

} // end class
