/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCHTMLPrinter.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:24 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page.html;

import java.util.Date;

import com.klg.jclass.page.JCPrinter;
import com.klg.jclass.page.FontParser;
import com.klg.jclass.page.Graphics2DBase;

import java.io.OutputStream;
import java.awt.Graphics;
import java.util.Enumeration;




import java.awt.geom.Rectangle2D;


/**
 *
 * Implements the <code>PrinterDriver</code> interface.
 *
 */
public class JCHTMLPrinter extends JCPrinter {

/** The name of a directory to which image files can be written. */
protected String	directoryName = null;

static String std_HTML_headers =
			"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">\n" +
			"<html>\n" +
			"<head>\n" +
			"\t<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n" +
			"\t<meta name=\"GENERATOR\" content=\"Quest Software JClass PageLayout" +
			com.klg.jclass.util.Version.VERSION + "\">";

/**
 * Creates an HTML printer given an <code>OutputStream</code>.
 * @param os an <code>OutputStream</code> to write the HTML code to
 */
public JCHTMLPrinter(OutputStream os) {

	setOutputStream(os);

	// force fonts to be loaded
	// ??? FontPS.getAllFonts();
}

/**
 * Creates an HTML printer given an <code>OutputStream</code> and an image directory.
 * @param os an <code>OutputStream</code> to write the HTML code to
 * @param dir the name of a directory to use for images
 */
public JCHTMLPrinter(OutputStream os, String dir) {

	if (dir != null)  {
		directoryName = new String(dir);
	}

	setOutputStream(os);
}

/**
 *
 * Constructor.
 * Allows detailed specification of font creation. The constructor which takes
 * one argument is akin to calling this constructor with these parameters:
 * <pre>
 *		JCPrinter p = new JCHTMLPrinter(
 *				someOutputStream,						// the output stream
 *				new com.klg.jclass.page.adobe.postscript.AFMParser(), // the parser
 *				"/com/klg/jclass/page/adobe/fonts.jar", // jar location
 *				".afm",									// file extension
 *				"com.klg.jclass.page.adobe.JCAdobeFontMap");
 * </pre>
 *
 *
 * @param os 			the <code>OutputStream</code> to which formatted output should be sent
 * @param parser 		the <code>FontParser</code> which creates fonts from afm files
 * @param jarLocation 	the location of the jar containing afm files relative
 *						to the class loader including the name of the jar file
 * @param fileExtension	the parser will turn files contained in "jarLocation"
 *						ending with this extension into fonts
 * @param fontMapFileName the user fontmap file; a file containing mappings
 *						between font names and their aliases
 *
 *
 */
public JCHTMLPrinter(OutputStream os, FontParser parser,
						   String jarLocation, String fileExtension,
						   String fontMapFileName) {

	this.os = os;

	// force fonts to be loaded
	// ??? FontPS.getAllFonts(parser, jarLocation, fileExtension,fontMapFileName);
}

/**
 * Creates default styles, any other one-time initialization.
 * Builds font families from available fonts.
 */
public void setup() {
	// ??? no HTML-specific actions
}

/**
 * Returns <code>false</code> since images sent to HTML printer are pre-scaled.
 * @return <code>false</code> since images sent to HTML printer are pre-scaled
 */
public boolean isImageScalable()
{
    return false;
}

/**
 * Initializes the document (standard document headers, etc.).
 * @param pageSize the notional size of a page of the document
 */
public void openDocument(Rectangle2D pageSize) {

	super.openDocument(pageSize);

	// print out the post script header
	output(std_HTML_headers);

	// print out date/time
	output("\t<title>PageLayout" +
			com.klg.jclass.util.Version.VERSION +
			" document dated " + new Date().toString() +
		   "</title>");

	// end the title section
	output("</head>");

	// Start the body of the document
	StringBuffer bodyTag = new StringBuffer("<body");

	// ??? get the first page of the printed range of the document
	// Get the background colour and output it

	// Close the body tag and output
	bodyTag.append(">");
	output(bodyTag.toString());
}

/**
 * Completes the document.
 * @param pageSize the notional size of a page of the document
 */
public void closeDocument(Rectangle2D pageSize) {
	((Graphics2DHTML) graphics).closeOpenTags();
	output("</body>\n</html>");

	super.closeDocument(pageSize);
}

/**
 * Outputs the code necessary to start a page (NO-OP in HTML).
 * @param pageSize the notional size of a page of the document
 * @param pageNumber
 */
public void beginPage(Rectangle2D pageSize, int pageNumber) {
	super.beginPage(pageSize, pageNumber);
}

/**
 * Ends the page (NO-OP in HTML).
 * @param pageSize the notional size of a page of the document
 */
public void endPage(Rectangle2D pageSize) {
	super.endPage(pageSize);
}

/**
 * Gets a Graphics that draws to this printer.
 * @return a Graphics object instantiated for this printer
 */
public Graphics getGraphics() {
	if (graphics == null) {
		graphics = new Graphics2DHTML(os, directoryName);
	}
	return (Graphics) graphics;
}

//---  HTML Printer specific methods ---

/**
 * Gets the image directory name.
 * @return the name of the image directory
 */
public String getImageDirectory() {
	return(directoryName);
}

/**
 * Sets the image directory name.
 * @param dir the name of the image directory
 */
public void setImageDirectory(String dir) {

	// Store the given name
	if (dir != null)  {
		directoryName = new String(dir);
	}
	else {
		directoryName = null;
	}

	// if the graphics has already been created, pass the name on
	if (graphics != null) {
		((Graphics2DHTML) graphics).setDirectory(dir);
	}
}

} // end class
