/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCPCLFontMap.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:25 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page.pcl;

import java.util.ListResourceBundle;
import java.util.ResourceBundle;

/**
 * This resource bundle is used to map between a font name 
 * and an alias (the actual font to use). It's required 
 * because the the "Times" font can appear as,
 * <ul>
 * <li>Times</li>
 * <li>Times Roman</li>
 * <li>Times New Roman</li>
 * <li>CG Times</li>
 * </ul>
 * and so on. There is no standard. 
 * <P>
 * The font names (resource keys) must not have spaces. This
 * because the <code>PropertyResourceBundle</code> cannot handle them. Replace
 * spaces with underscores in the font names. Use a dash between
 * a font name and its style. Leave the spaces in
 * the alias names. Here is a example illustrating
 * a mapping between some Windows NT fonts and PCL fonts.
 * <pre>
 *	{"Times", 						"M Times New"},
 *	{"Times-Bold", 					"M Times New Bd"},
 *	{"Times-Italic", 				"M Times New It"},
 *	{"Times-BoldItalic", 			"M Times New BdIt"},
 *
 *	{"TimesRoman", 					"M Times New"},
 *	{"TimesRoman-Bold", 			"M Times New Bd"},
 *	{"TimesRoman-Italic", 			"M Times New It"},
 *	{"TimesRoman-BoldItalic", 		"M Times New BdIt"},
 *
 *	{"Times_New_Roman", 			"M Times New"},
 *	{"Times_New_Roman-Bold", 		"M Times New Bd"},
 *	{"Times_New_Roman-Italic", 		"M Times New It"},
 *	{"Times_New_Roman-BoldItalic", 	"M Times New BdIt"},
 *
 *	{"Helvetica",					"M Arial"}, 
 *	{"Helvetica-Bold",				"M Arial Bd"}, 
 *	{"Helvetica-Italic",			"M Arial It"}, 
 *	{"Helvetica-BoldItalic",		"M Arial BdIt"}, 
 *
 *	{"Arial",						"M Arial"}, 
 *	{"Arial-Bold",					"M Arial Bd"}, 
 *	{"Arial-Italic",				"M Arial It"}, 
 *	{"Arial-BoldItalic",			"M Arial BdIt"}, 
 *
 *	{"Courier",						"Courier"}, 
 *	{"Courier-Bold",				"Courier       Bd"}, 
 *	{"Courier-Italic",				"Courier       It"}, 
 *	{"Courier-BoldItalic",			"Courier     BdIt"}, 
 *
 *	{"Courier_New",					"Courier"}, 
 *	{"Courier_New-Bold",			"Courier       Bd"}, 
 *	{"Courier_New-Italic",			"Courier       It"}, 
 *	{"Courier_New-BoldItalic",		"Courier     BdIt"}, 
 *		
 *	{"Serif", 						"M Times New"},
 *	{"Serif-Bold", 					"M Times New Bd"},
 *	{"Serif-Italic", 				"M Times New It"},
 *	{"Serif-BoldItalic", 			"M Times New BdIt"},
 *		
 *	{"SansSerif",					"M Arial"}, 
 *	{"SansSerif-Bold",				"M Arial Bd"}, 
 *	{"SansSerif-Italic",			"M Arial It"}, 
 *	{"SansSerif-BoldItalic",		"M Arial BdIt"}, 
 *		
 *	{"Dialog",						"M Arial"}, 
 *	{"Dialog-Bold",					"M Arial Bd"}, 
 *	{"Dialog-Italic",				"M Arial It"}, 
 *	{"Dialog-BoldItalic",			"M Arial BdIt"}, 
 *
 *	{"DialogInput",					"Courier"}, 
 *	{"DialogInput-Bold",			"Courier       Bd"}, 
 *	{"DialogInput-Italic",			"Courier       It"}, 
 *	{"DialogInput-BoldItalic",		"Courier     BdIt"}, 
 * </pre>
 *
 */ 


public class JCPCLFontMap extends ListResourceBundle {
public Object[][] getContents() {
	return contents;
}

// spaces in key font names must be replaced with underscores
static final Object[][] contents = {
	
	{"Times", 						"M Times New"},
	{"Times-Bold", 					"M Times New Bd"},
	{"Times-Italic", 				"M Times New It"},
	{"Times-BoldItalic", 			"M Times New BdIt"},

	{"TimesRoman", 					"M Times New"},
	{"TimesRoman-Bold", 			"M Times New Bd"},
	{"TimesRoman-Italic", 			"M Times New It"},
	{"TimesRoman-BoldItalic", 		"M Times New BdIt"},

	{"Times_New_Roman", 			"M Times New"},
	{"Times_New_Roman-Bold", 		"M Times New Bd"},
	{"Times_New_Roman-Italic", 		"M Times New It"},
	{"Times_New_Roman-BoldItalic", 	"M Times New BdIt"},

	{"Helvetica",					"M Arial"}, 
	{"Helvetica-Bold",				"M Arial Bd"}, 
	{"Helvetica-Italic",			"M Arial It"}, 
	{"Helvetica-BoldItalic",		"M Arial BdIt"}, 

	{"Arial",						"M Arial"}, 
	{"Arial-Bold",					"M Arial Bd"}, 
	{"Arial-Italic",				"M Arial It"}, 
	{"Arial-BoldItalic",			"M Arial BdIt"}, 

	{"Courier",						"Courier"}, 
	{"Courier-Bold",				"Courier       Bd"}, 
	{"Courier-Italic",				"Courier       It"}, 
	{"Courier-BoldItalic",			"Courier     BdIt"}, 

	{"Courier_New",					"Courier"}, 
	{"Courier_New-Bold",			"Courier       Bd"}, 
	{"Courier_New-Italic",			"Courier       It"}, 
	{"Courier_New-BoldItalic",		"Courier     BdIt"}, 
		
	{"Serif", 						"M Times New"},
	{"Serif-Bold", 					"M Times New Bd"},
	{"Serif-Italic", 				"M Times New It"},
	{"Serif-BoldItalic", 			"M Times New BdIt"},
		
	{"SansSerif",					"M Arial"}, 
	{"SansSerif-Bold",				"M Arial Bd"}, 
	{"SansSerif-Italic",			"M Arial It"}, 
	{"SansSerif-BoldItalic",		"M Arial BdIt"}, 
		
	{"Dialog",						"M Arial"}, 
	{"Dialog-Bold",					"M Arial Bd"}, 
	{"Dialog-Italic",				"M Arial It"}, 
	{"Dialog-BoldItalic",			"M Arial BdIt"}, 

	{"DialogInput",					"Courier"}, 
	{"DialogInput-Bold",			"Courier       Bd"}, 
	{"DialogInput-Italic",			"Courier       It"}, 
	{"DialogInput-BoldItalic",		"Courier     BdIt"}, 

};

}
