/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: TypeFacePCL.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:25 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page.pcl;

import java.util.StringTokenizer;

/**
 * A PCL typeface encompasses an indexed list of character glyphs with their
 * associated metrics, font metrics, per-character and font-wide kerning
 * information, font selection and definition information, and a set of symbol
 * sets (font encodings).
 */
public class TypeFacePCL {

protected GeneralInfoPCL general		= new GeneralInfoPCL();
protected PanosePCL panose				= new PanosePCL();
protected MetricsPCL typefaceMetrics	= new MetricsPCL();
protected SymbolPCL symbol 				= new SymbolPCL();
protected CharMetricsPCL charMetrics	= new CharMetricsPCL();
protected KernPCL kerning				= new KernPCL();
protected String familyName;
protected int style						= -1;

public static final int PCL_TYPEFACE_FLAGS_NONE 			= 0x0;
public static final int PCL_TYPEFACE_FLAGS_NORMAL 			= 0x1;
public static final int PCL_TYPEFACE_FLAGS_BOLD 			= 0x2;
public static final int PCL_TYPEFACE_FLAGS_ITALIC 			= 0x4;
public static final int PCL_TYPEFACE_FLAGS_CONDENSED 		= 0x8;
public static final int PCL_TYPEFACE_FLAGS_SINGLE_NAME 		= 0x10;
public static final int PCL_TYPEFACE_FLAGS_COMPOUND_NAME	= 0x20;
public static final int PCL_TYPEFACE_FLAGS_FIXEDWIDTH 		= 0x40;
public static final int PCL_TYPEFACE_FLAGS_INVALID 			= 0xff;



public CharacterPCL getCharacter(int index) {
	return (CharacterPCL) charMetrics.chars.elementAt(index);
}

public String getName() {
	return general.typeface;
}

public String getFamily() {
	if (familyName == null) {
		parseFamilyName(general.typeface);
    }
	return familyName;
}

public int getStyle() {
	if (familyName == null) {
		parseFamilyName(general.typeface);
    }
	return style;
}

// Permit the TFM parser to update style information if it needs to
public void setStyle(int new_style) {
	this.style = new_style;
}

public void parseFamilyName(String typefaceName) {
	int		i, pos, start_token, token_count = 0;
	String	tokens[];
	String	last_token, temp_name;

	/* Count the tokens on the line */
	StringTokenizer tokenizer = new StringTokenizer(typefaceName);
    tokens = new String[tokenizer.countTokens()];
	while (tokenizer.hasMoreTokens()) {
		tokens[token_count++] = tokenizer.nextToken();
	}

	/* Return an error if no names were found */
	if (token_count == 0) {
		style = PCL_TYPEFACE_FLAGS_INVALID;
		familyName = null;
		return;
	}
	else {
    	style = 0;
	}

	/* Assume a single name is a font family name */
	if (token_count == 1) {
		style = PCL_TYPEFACE_FLAGS_SINGLE_NAME;
		familyName = tokens[0];
		return;
	}

	/* If there are two or more names, look for type flags */
	else {
		last_token = tokens[token_count - 1];
		pos = 0;
		while (pos != last_token.length() && style != PCL_TYPEFACE_FLAGS_INVALID) {

			switch(last_token.charAt(pos)) {
				case 'B':

					if (last_token.charAt(pos + 1) == 'd') {
						style |= PCL_TYPEFACE_FLAGS_BOLD;
						pos += 2;
					}
					else {
						style = PCL_TYPEFACE_FLAGS_INVALID;
						continue;
					}
					break;

				case 'C':

					if (last_token.charAt(pos + 1) == 'd') {
						style |= PCL_TYPEFACE_FLAGS_CONDENSED;
						pos += 2;
					}
					else {
						style = PCL_TYPEFACE_FLAGS_INVALID;
						continue;
					}
					break;

				case 'H':

					if (last_token.substring(pos + 1, pos + 3).equals("lb")) {
						style |= PCL_TYPEFACE_FLAGS_BOLD;
						pos += 3;
					}
					else {
						style = PCL_TYPEFACE_FLAGS_INVALID;
						continue;
					}
					break;

				case 'I':

					if (last_token.charAt(pos + 1) == 't') {
						style |= PCL_TYPEFACE_FLAGS_ITALIC;
						pos += 2;
					}
					else {
						style = PCL_TYPEFACE_FLAGS_INVALID;
						continue;
					}
					break;

				case 'K':

					if (last_token.substring(pos + 1, pos + 4).equals("rsv")) {
						style |= PCL_TYPEFACE_FLAGS_ITALIC;
						pos += 4;
					}
					else {
						style = PCL_TYPEFACE_FLAGS_INVALID;
						continue;
					}
					break;

				case 'M':

					if (last_token.charAt(pos + 1) == 'd') {
						style |= PCL_TYPEFACE_FLAGS_NORMAL;
						pos += 2;
					}
					else {
						style = PCL_TYPEFACE_FLAGS_INVALID;
						continue;
					}
					break;

				case 'X':

					if (last_token.charAt(pos + 1) == 'b') {
						style |= PCL_TYPEFACE_FLAGS_BOLD;
						pos += 2;
					}
					else {
						style = PCL_TYPEFACE_FLAGS_INVALID;
						continue;
					}
					break;

				default:

					style = PCL_TYPEFACE_FLAGS_INVALID;
					continue;

			} /* switch */
		}

		/* If the last token was codes, discard it */
		if (style != PCL_TYPEFACE_FLAGS_INVALID) {
			token_count--;
		}
	}

	/*
	 * Reset the flags of "invalid" fonts, since they may simply have
	 * two-word names and no font style codes
	 */
	if (style == PCL_TYPEFACE_FLAGS_INVALID) {
		style = 0;
	}
	start_token = 0;

	/* If there are two or more name fields, check for a foundry code */
	if (token_count >= 2 && tokens[0].length() < 3) {
		String s = tokens[0];
		boolean isFoundry = false;

		/* first field is a foundry if it is one or two capital letters */
    	for (int j = 0; j < tokens[0].length(); j++) {
			if ( Character.isUpperCase(s.charAt(j)) ) {
				isFoundry = true;
			}
		}
		if (isFoundry) {
			start_token = 1;
		}

	}

	/* If the typeface is neither bold nor italic, assume normal */
	if ((style & (PCL_TYPEFACE_FLAGS_BOLD |
							PCL_TYPEFACE_FLAGS_ITALIC)) == 0) {
		style |= PCL_TYPEFACE_FLAGS_NORMAL;
	}

	/* If there are more than one word in the name */
	if (token_count - start_token > 1) {
		style |= PCL_TYPEFACE_FLAGS_COMPOUND_NAME;

		/* assemble the tokens and return the name */
        String compound = "";
		for (int j = start_token; j < tokens.length; j++) {
			compound += tokens[j];
			compound += " ";
		}
		familyName = compound;
	}
	else {
		/* Name is a single token; return it */
		familyName = tokens[start_token];
	}
}

/**
 * Return the typeface metrics for this font.
 */
public MetricsPCL getTypefaceMetrics() {
	return typefaceMetrics;
}

}
