/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: ArcRender.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:26 $ $Locker:  $  Quest Software Inc. 
package com.klg.jclass.page.render;

import java.awt.Color;



import com.klg.jclass.page.JCDrawStyle;
import com.klg.jclass.page.JCUnit;




import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;


/**
 * ArcRender describes a circular arc object
 */
public class ArcRender extends Render implements Cloneable {

/** The origin (centre) of the arc */
protected JCUnit.Point      center;
/** The radius of the circle */
protected JCUnit.Measure    radius;

/** The initial angle (Cartesian) from which the angle starts */
protected double            theta1;
/** The end angle to which the arc is drawn */
protected double            theta2;

/** The draw style which specifies drawing parameters */
protected JCDrawStyle       style;
/** Indicates if the arc is to be filled or not */
protected boolean           fill;

/**
 * Create an Arc render object
 */
public ArcRender() {

	// This creates a degenerate circle of zero radius
	bounds = new JCUnit.Dimension(JCUnit.POINTS);
	origin = new JCUnit.Point(JCUnit.POINTS);
	radius = new JCUnit.Measure(JCUnit.POINTS);
	center = new JCUnit.Point(JCUnit.POINTS);
	theta1 = 0;
	theta2 = 360;
	style = null;
}

/**
 * Create an arc render object given centre, radius and start/end angles
 * @param center the center of the arc
 * @param r the radius at which the arc is to be drawn
 * @param startAngle the angle (cartesian) at which the arc starts
 * @param endAngle the angle (cartesian) at which the arc ends
 * @param style the draw style to use for painting the arc
 */
public ArcRender(JCUnit.Point center, JCUnit.Measure r, double startAngle,
				 double endAngle, JCDrawStyle style) {

	JCUnit.Measure  x, y;

	// Set the arc parameters
	this.center = (JCUnit.Point) center.clone();
	this.radius = (JCUnit.Measure) r.clone();
	this.theta1 = startAngle;
	this.theta2 = endAngle;
	this.style = style;

	// Compute the new origin and bounds of the arc 
	x = center.getX();
	x.subtract(r);
	y = center.getY();
	y.subtract(r);

	this.origin = new JCUnit.Point(x.units, x.distance, y.distance);

	// Compute the bounds of a square in which the arc is inscribed
	x.assign(r);
	x.multiply(2.0);
	this.bounds = new JCUnit.Dimension(x.units, x.distance, x.distance);
}

/**
 * Get the centre of the arc
 * @return the point which is the centre of the arc
 */
public JCUnit.Point getCenter() {

	return(center);
}

/**
 * Set the origin (centre) of the arc
 * @param point The location of the arc's center with respect to the frame
 */
public void setCenter(JCUnit.Point point) {

	JCUnit.Measure  x, y;

	// Set the origin
	this.center = point;

	// Compute the new origin of the arc (circle) bounds
	x = center.getX();
	x.subtract(radius);
	y = center.getY();
	y.subtract(radius);

	// Set the new origin
	this.origin.setX(x);
	this.origin.setY(y);

	// The size (bounds) doesn't change unless the radius changes
}

/**
 * Get the radius of the arc
 * @return The radius of the arc
 */
public JCUnit.Measure getRadius() {

	return(radius);
}

/**
 * Set the radius of the arc
 * @param r The radius of the arc
 */
public void setRadius(JCUnit.Measure r) {

	JCUnit.Measure  x, y;

	// Set the radius
	this.radius.assign(r);

	// Compute the new origin of the arc/circle bounds
	x = center.getX();
	x.subtract(r);
	y = center.getY();
	y.subtract(r);

	// Set the origin
	this.origin.setX(x);
	this.origin.setY(y);

	// Compute the bounds of a square in which the arc is inscribed
	x.assign(r);
	x.multiply(2.0);
	this.bounds.setWidth(x);
	this.bounds.setHeight(x);
}

/**
 * Get the start angle of the arc
 * @return the (Cartesian) angle at which the arc begins
 */
public double getStartAngle() {
	return(theta1);
}

/**
 * Set the start angle of the arc
 * @param theta The angle from which the arc begins
 */
public void setStartAngle(double theta) {

	theta1 = theta;
}

/**
 * Get the end angle of the arc
 * @return the (Cartesian) angle at which the arc ends
 */
public double getEndAngle() {
	return(theta1);
}

/**
 * Set the end angle of the arc
 * @param theta The angle at which the arc ends
 */
public void setEndAngle(double theta) {

	theta2 = theta;
}

/**
 * Get the style used to draw this arc
 * @return the draw style assigned to this arc
 */
public JCDrawStyle getStyle() {
	return style;
}

/**
 * Set the style used to draw this arc
 * @param style The draw style used to create this arc
 */
public void setStyle(JCDrawStyle style) {

	this.style = style;
}

/**
 * Check if the figure described by the arc is to be filled
 * @return True if the arc is to be filled
 */
public boolean isFilled() {
	return fill;
}

/**
 * Set whether this arc is to be filled
 * @param filled True if the arc is to be used as the outline of a fill area
 */
public void setFilled(boolean filled) {

	this.fill = filled;
}

/**
 * Returns an indication if this object can be split into components
 * @returns True indicating the arc can't be split
 */
public boolean isAtomic() {
	return(true);
}

/**
 * Draw the arc described by this object
 * @param gc The graphics context which will draw this object
 */
public void draw(Graphics2D gc) {

	// NOTE: Rather than compute a bezier equivalent to the arc in order to
	//       use the Graphics2D draw(Shape) method, we just use the
	//       Graphics.drawArc() method.  (Despite the loss of accuracy)
	int                 x, y, width, height;
	int                 line_type;
	double              dash, spacing, line_width;
	Color               colour;
	BasicStroke         stroke;

	// Determine the colour to use to draw/fill the figure
	if (fill) {
		colour = style.getFillForegroundColor();

		// if a fill colour is not given, use the draw colour
		if (colour == null) {
			colour = style.getForegroundColor();
		}
	}
	else {
		colour = style.getForegroundColor();
	}

	// if there is no colour set, don't draw anything
	if (colour == null) {
		return;
	}

	// Set the colour to use
	gc.setColor(colour);

	// Create a stroke to handle line width, etc.
	line_width = style.getLineWidth().getAs(JCUnit.POINTS);

	line_type = style.getLineType();
	if (line_type == JCDrawStyle.LINE_TYPE_BROKEN) {
		float[]     dashes = new float[2];

		// Get the dash length and create the dash sequence array
		dash = style.getDashLength().getAs(JCUnit.POINTS);
		dashes[0] = (float) dash; dashes[1] = (float) dash;

		// Create a dashed stroke
		stroke = new BasicStroke((float) line_width, BasicStroke.CAP_SQUARE,
								 BasicStroke.JOIN_MITER, (float) 10.0, dashes,
								 (float) 0.0);
	}
	else if (line_type == JCDrawStyle.LINE_TYPE_DOUBLE) {
		spacing = style.getLineSpacing().getAs(JCUnit.POINTS);

		// Draw a line the thickness of the double line
		line_width = line_width * 2 + spacing;
		stroke = new BasicStroke((float) line_width, BasicStroke.CAP_SQUARE,
								 BasicStroke.JOIN_MITER);
	}
	else if (line_type == JCDrawStyle.LINE_TYPE_PLAIN ||
			 line_type == JCDrawStyle.LINE_TYPE_SINGLE) {
		// Create the standard stroke
		if (theta2 >= 360.0) {
			// bad idea to use cap square if we're drawing a completer circle
			// as java will make the join quite ugly.  Use cap but in this
			// case.
			stroke = new BasicStroke((float) line_width, BasicStroke.CAP_BUTT,
								 				BasicStroke.JOIN_MITER);
		} else {
			stroke = new BasicStroke((float) line_width, BasicStroke.CAP_SQUARE,
								 				BasicStroke.JOIN_MITER);
		}
	} 
	else {
		// line type = JCDrawStyle.LINE_TYPE_NONE
		// so don't draw it
		return;
	}

	// Set the stroke style
	gc.setStroke(stroke);

	// convert the dimensions of the arc's bounding box to points
	x = (int) JCUnit.getAsPoints(origin.units, origin.x);
	y = (int) JCUnit.getAsPoints(origin.units, origin.y);
	width = (int) JCUnit.getAsPoints(bounds.units, bounds.width);
	height = (int) JCUnit.getAsPoints(bounds.units, bounds.height);

	// Use the given graphics to draw the arc
	if (fill) {
		gc.fillArc(x, y, width, height, (int) theta1, (int) theta2);
	}
	else {
		gc.drawArc(x, y, width, height, (int) theta1, (int) theta2);
	}
}

/**
 * Create an arc which is a copy of this one
 * @return a clone of this arc
 */
public Object clone() {

	ArcRender   new_arc;

	// Clone the arc
	new_arc = (ArcRender) super.clone();

	// Copy dependent objects
	new_arc.radius = (JCUnit.Measure) this.radius.clone();
	new_arc.center = (JCUnit.Point) this.center.clone();

	return((Object) new_arc);
}

}
