/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: BoxRender.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:26 $ $Locker:  $  Quest Software Inc. 
package com.klg.jclass.page.render;

import java.awt.Color;



import com.klg.jclass.page.JCDrawStyle;
import com.klg.jclass.page.JCUnit;




import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;


/**
 * BoxRender describes a rectangular drawn object
 */
public class BoxRender extends Render implements Cloneable {

/** The lower left (min_x, min_y) corner of the box */
protected JCUnit.Point      lowerLeft;
/** The upper right (max_x, max_y) corner of the box */
protected JCUnit.Point      upperRight;

/** The draw style which specifies drawing parameters */
protected JCDrawStyle       style;
/** Indicates if the box is to be filled or not */
protected boolean           fill;

/**
 * Create a Box render object
 */
public BoxRender() {

	// Create a degenerate box
	bounds = new JCUnit.Dimension(JCUnit.POINTS);
	origin = new JCUnit.Point(JCUnit.POINTS);
	style = null;
}

/**
 * Create a box render object given opposite points and a style
 * @param point1 a corner of the box to draw
 * @param point2 the diagonally opposite corner of the box
 * @param style the draw style to use for painting the box
 */
public BoxRender(JCUnit.Point point1, JCUnit.Point point2, JCDrawStyle style) {

	double      x_min, y_min, x_max, y_max;

	this.style = style;

	double		x1int = point1.xInt(), y1int = point1.yInt();
	double		x2int = point2.xInt(), y2int = point2.yInt();

	// if the points are lower left and upper right, use them
	if ((x1int <= x2int && y1int <= y2int) ||
		(x1int >= x2int && y1int >= y2int)) {

		JCUnit.Measure width, height;

		// if they are in the right order
		if (x1int <= x2int && y1int <= y2int) {

			// Clone point1 as the origin
			origin = (JCUnit.Point) point1.clone();
			lowerLeft = (JCUnit.Point) point1.clone();
			upperRight = (JCUnit.Point) point2.clone();
		}
		// otherwise we need to reverse the order
		else {

			// Clone point2 as the origin
			origin = (JCUnit.Point) point2.clone();
			lowerLeft = (JCUnit.Point) point2.clone();
			upperRight = (JCUnit.Point) point1.clone();
		}

		// Compute the width and height
		width = upperRight.getX();
		width.subtract(origin.getX());
		height = upperRight.getY();
		height.subtract(origin.getY());

		// Create the bounds of the rectangle
		bounds = new JCUnit.Dimension(upperRight.units, width.distance,
									  height.distance);
	}

	// otherwise the points are upper-left and lower-right, so we need to
	// re-group them
	else {

		// Convert both points to the same units if necessary
		if (point1.units != point2.units) {
			point2.x = JCUnit.getAs(point1.units, point2.units, point2.x);
			point2.y = JCUnit.getAs(point1.units, point2.units, point2.y);
			point2.units = point1.units;
		}

		// compute the lowerLeft and upperRight corners...
		if (x1int >= x2int) {
			x_min = point2.x;
			x_max = point1.x;
		}
		else {
			x_min = point1.x;
			x_max = point2.x;
		}

		if (y1int >= y2int) {
			y_min = point2.y;
			y_max = point1.y;
		}
		else {
			y_min = point1.y;
			y_max = point2.y;
		}

		// Compute the corners
		lowerLeft = new JCUnit.Point(point1.units, x_min, y_min);
		upperRight = new JCUnit.Point(point1.units, x_max, y_max);

		// Compute the location and size
		origin = new JCUnit.Point(point1.units, x_min, y_min);
		bounds = new JCUnit.Dimension(point1.units, x_max - x_min,
									  y_max - y_min);
	}
}

/**
 * Create a box render object given a location, size and a style
 * @param location the point at which this box begins
 * @param size the size of the box
 * @param style the draw style to use for painting the box
 */
public BoxRender(JCUnit.Point location, JCUnit.Dimension size,
				 JCDrawStyle style) {
				 	
	this.style = null;

	this.style = style;

	// copy points and compute bounds
	origin = (JCUnit.Point) location.clone();
	bounds = (JCUnit.Dimension) size.clone();

	lowerLeft = (JCUnit.Point) location.clone();

	// Compute the position of the upper right corner
	JCUnit.Measure urx = origin.getX();
	urx.add(bounds.getWidth());
	JCUnit.Measure ury = origin.getY();
	ury.add(bounds.getHeight());

	upperRight = new JCUnit.Point(origin.units, urx.distance, ury.distance);
}

/**
 * Create a list of points describing the perimeter
 * @return An array containing the points of the box's perimeter in order
 */
public JCUnit.Point[] getPoints() {

	JCUnit.Point    points[] = new JCUnit.Point[4];

	// NOTE:  we do not close the figure
	points[0] = (JCUnit.Point) lowerLeft.clone();
	points[1] = (JCUnit.Point) lowerLeft.clone();
	points[1].setX(upperRight.getX());
	points[2] = (JCUnit.Point) upperRight.clone();
	points[3] = (JCUnit.Point) upperRight.clone();
	points[3].setX(lowerLeft.getX());

	return points;
}

/**
 * Get the lower left (min, min) corner of the box
 * @return the point which is the lower-left corner
 */
public JCUnit.Point getLowerLeft() {

	return(lowerLeft);
}

/**
 * Get the upper right (max, max) corner of the box
 * @return the point which is the upper-right corner
 */
public JCUnit.Point getUpperRight() {

	return(upperRight);
}

/**
 * Set the origin of the box
 * @param point The location of the box with respect to its frame
 */
public void setLocation(JCUnit.Point point) {

	// Set the origin and lower-left corner
	origin = (JCUnit.Point) point.clone();
	lowerLeft = (JCUnit.Point) point.clone();

	// Compute the position of the upper right corner
	JCUnit.Measure urx = origin.getX();
	urx.add(bounds.getWidth());
	JCUnit.Measure ury = origin.getY();
	ury.add(bounds.getHeight());

	// Set the upper-right corner
	upperRight = new JCUnit.Point(origin.units, urx.distance, ury.distance);
}

/**
 * Set the bounds of the box
 * @param size The size of the box
 */
public void setSize(JCUnit.Dimension size) {

	// Set the origin
	this.bounds = size;

	// Compute the position of the upper right corner
	JCUnit.Measure urx = origin.getX();
	urx.add(bounds.getWidth());
	JCUnit.Measure ury = origin.getY();
	ury.add(bounds.getHeight());

	// Set the upper-right corner
	upperRight = new JCUnit.Point(origin.units, urx.distance, ury.distance);
}

/**
 * Get the style used to draw this box
 * @return the draw style assigned to this box
 */
public JCDrawStyle getStyle() {
	return style;
}

/**
 * Set the style used to draw this box
 * @param style The draw style used to create this box
 */
public void setStyle(JCDrawStyle style) {

	this.style = style;
}

/**
 * Check if the box is to be filled
 * @return True if the box is to be filled
 */
public boolean isFilled() {
	return fill;
}

/**
 * Set whether this box is to be filled
 * @param filled True if the box is to be drawn filled
 */
public void setFilled(boolean filled) {

	this.fill = filled;
}

/**
 * Returns an indication if this object can be split into components
 * @returns True indicating the box can't be split
 */
public boolean isAtomic() {
	return(true);
}

/**
 * Draw the box described by this object
 * @param gc The graphics context which will draw this object
 */
public void draw(Graphics2D gc) {

	int                 line_type;
	double              x, y, width, height;
	double              dash, spacing, line_width;
	Rectangle2D.Double  rect;
	Color               colour;
	BasicStroke         stroke;

	// Determine the colour to use to draw/fill the figure
	if (fill) {
		colour = style.getFillForegroundColor();

		// if a fill colour is not given, use the draw colour
		if (colour == null) {
			colour = style.getForegroundColor();
		}
	}
	else {
		colour = style.getForegroundColor();
	}

	// if there is no colour set, don't draw anything
	if (colour == null) {
		return;
	}

	// Set the colour to use
	gc.setColor(colour);

	// Create a stroke to handle line width, etc.
	line_width = style.getLineWidth().getAs(JCUnit.POINTS);
	
	// convert the dimensions of the box to points
	x = JCUnit.getAsPoints(origin.units, origin.x);
	y = JCUnit.getAsPoints(origin.units, origin.y);
	width = JCUnit.getAsPoints(bounds.units, bounds.width);
	height = JCUnit.getAsPoints(bounds.units, bounds.height);

	line_type = style.getLineType();
	if (line_type == JCDrawStyle.LINE_TYPE_BROKEN) {
		float[]     dashes = new float[2];

		// Get the dash length and create the dash sequence array
		dash = style.getDashLength().getAs(JCUnit.POINTS);
		dashes[0] = (float) dash; dashes[1] = (float) dash;

		// Create a dashed stroke
		stroke = new BasicStroke((float) line_width, BasicStroke.CAP_SQUARE,
								 BasicStroke.JOIN_MITER, (float) 10.0, dashes,
								 (float) 0.0);
		
	}
	else if (line_type == JCDrawStyle.LINE_TYPE_DOUBLE) {
		spacing = style.getLineSpacing().getAs(JCUnit.POINTS);

		
		stroke = new BasicStroke((float) line_width, BasicStroke.CAP_SQUARE,
								 BasicStroke.JOIN_MITER);
		
		//Following fixes Bunka bug DWS 001 001 double line style not honoured.
		gc.setStroke(stroke);
		
		//If line type is double, we need to create an inner rectangle
		//First calculate the size of the internal rectangle
		double x2 = x + spacing;
		double y2 = y + spacing;	
		double width2 = width - (2*spacing);
		double height2 = height - (2*spacing);
		
		// Create a new rectangle
		rect = new Rectangle2D.Double(x2, y2, width2, height2);
		//draw the inner rectangle
		gc.draw(rect);
		

	

	}
	else if (line_type == JCDrawStyle.LINE_TYPE_PLAIN ||
			 line_type == JCDrawStyle.LINE_TYPE_SINGLE) {
		// Create the standard stroke
		stroke = new BasicStroke((float) line_width, BasicStroke.CAP_SQUARE,
								 BasicStroke.JOIN_MITER);
								

	

	} 
	else {
		// line type = JCDrawStyle.LINE_TYPE_NONE
		// so don't draw it
		return;
	}

	// Set the stroke style
	gc.setStroke(stroke);

	

	// Create a new rectangle
	rect = new Rectangle2D.Double(x, y, width, height);

	// Use the given graphics to draw the box
	if (fill) {
		gc.fill(rect);
	}
	else {
		gc.draw(rect);
	}
}

/**
 * Create a box which is a copy of this one
 * @return a clone of this box
 */
public Object clone() {

	BoxRender   new_box;

	// Clone the box
	new_box = (BoxRender) super.clone();

	// Copy dependent objects
	new_box.lowerLeft = (JCUnit.Point) this.lowerLeft.clone();
	new_box.upperRight = (JCUnit.Point) this.upperRight.clone();

	return((Object) new_box);
}

}
