/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: HRuleMarker.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:26 $ $Locker:  $  Quest Software Inc. 

package com.klg.jclass.page.render;

import com.klg.jclass.page.JCDrawStyle;
import com.klg.jclass.page.JCTextStyle;
import com.klg.jclass.page.JCUnit;
import com.klg.jclass.page.Markup;




import java.awt.Graphics2D;


/**
 * HRuleMarker describes a marker for the position of an hRule in the flow
 */
public class HRuleMarker extends Render implements Cloneable, FlowMarker {

/** The length of the rule in units */
public JCUnit.Measure	length = null;
/** The percentage width of the frame the rule covers */
public double			percent = 1.0;

/** The style which describes the text attributes at the hRule's location */
protected JCTextStyle		style;
/** The style which describes the line attributes of the rule */
protected JCDrawStyle		dStyle;

/** The alignment of the table in the column/frame */
protected int				alignment = JCTextStyle.ALIGNMENT_CENTER;

/**
 * Create an hRule marker object
 */
public HRuleMarker() {

}

/**
 * Create an hRule marker object
 */
public HRuleMarker(JCTextStyle style, JCDrawStyle dStyle,
				   double percentWidth) {

	this.style = style;
	this.dStyle = dStyle;
	alignment = style.getAlignment();

	if (percentWidth < 0.0 || percentWidth > 1.0) {
		throw new IllegalArgumentException("Width fraction must be between 0.0 and 1.0");
	}
	percent = percentWidth;
}

/**
 * Create an hRule marker object
 */
public HRuleMarker(JCTextStyle style, JCDrawStyle dStyle,
				   JCUnit.Measure rule_length) {

	this.style = style;
	this.dStyle = dStyle;
	alignment = style.getAlignment();

	length = (JCUnit.Measure) rule_length.clone();
}

/**
 * Get the length of this hRule
 * @return The length of this hRule
 */
public JCUnit.Measure getLength() {
	return(length);
}

/**
 * Set the length of this hRule
 * @param length The length of this hRule
 */
public void setLength(JCUnit.Measure new_length) {

	length = (JCUnit.Measure) new_length.clone();
}

/**
 * Get the percentage of overall width of this hRule
 * @return The percentage of overall width of this hRule
 */
public double getPercentWidth() {
	return(percent);
}

/**
 * Set the percentage of overall width of this hRule
 * @param percentWidth The percentage of overall width of this hRule
 */
public void setPercentWidth(double percentWidth) {

	if (percentWidth < 0.0 || percentWidth > 1.0) {
		throw new IllegalArgumentException("Width fraction must be between 0.0 and 1.0");
	}

	percent = percentWidth;
}

/**
 * Get the alignment of the marked table
 * @return The alignment the table this marker represents has
 */
public int getAlignment() {
	return(alignment);
}

/**
 * Set the alignment of a table
 * @param alignment The alignment to use for this marker's table
 */
/*
public void setAlignment(int alignment) {
	// ??? Error-checking
	this.alignment = alignment
}
*/

/**
 * Get the text style which controls line spacing, etc.
 * @return The hRule's text style
 */
public JCTextStyle getTextStyle() {

	return(style);
}

/**
 * Set the style which controls text-mode attributes like line spacing
 * @param style The new text style
 */
public void setTextStyle(JCTextStyle newStyle) {

	style = newStyle;
}

/**
 * Get the draw style which controls line appearance
 * @return The hRule's drawing style
 */
public JCDrawStyle getDrawStyle() {

	return(dStyle);
}

/**
 * Set the style which controls drawing-mode attributes like line width
 * @param style The new drawing style
 */
public void setDrawStyle(JCDrawStyle newStyle) {

	dStyle = newStyle;
}

/**
 * Returns an indication if this object can be split into components
 * @returns True indicating that a marker object can't be split
 */
public boolean isAtomic() {
	return(true);
}

/**
 * The hRule will only be drawn to devices which support the Markup interface
 * @param gc The graphics context which will draw this object
 */
public void draw(Graphics2D gc) {

	// If this context doesn't support Markup, we don't do anything
	if (!(gc instanceof Markup)) {
		return;
	}

	// Access the Markup methods of the gc
	Markup mo = (Markup) gc;

	// Output an hRule to the document
	if (length != null) {
		int points = (int) length.getAs(JCUnit.POINTS);
		mo.hRule(points);
	}
	else { // percent of width
		mo.hRule(percent);
	}
}

/**
 * Create a Marker object which is a copy of this one
 * @return a clone of this table marker
 */
public Object clone() {

	HRuleMarker		new_marker;

	// Clone the symbol object
	new_marker = (HRuleMarker) super.clone();

	// Copy dependent objects

	return((Object) new_marker);
}

}
