/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: ImageMarker.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:26 $ $Locker:  $  Quest Software Inc. 

package com.klg.jclass.page.render;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Rectangle;

import java.awt.image.ImageObserver;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.CellRendererPane;

import com.klg.jclass.page.EPSImage;
import com.klg.jclass.page.JCTextStyle;
import com.klg.jclass.page.JCDrawStyle;
import com.klg.jclass.page.JCUnit;
import com.klg.jclass.page.Markup;

import com.klg.jclass.util.ServerRenderable;




import java.awt.Graphics2D;


/**
 * ImageMarker describes a marker for a image's insertion in the document flow
 */
public class ImageMarker extends Render implements Cloneable, FlowMarker,
												   ImageObserver {

/** The image which is being inserted in the flow */
protected ImageRender		image;
/** True if the image is embedded in a line of content */
protected boolean			embedded = false;

/** The style which describes the text attributes at the image's location */
// protected JCTextStyle		style;

/** An id number used by the MediaTracker to watch this image */
private int                 imageID;
/** This component is used by MediaTracker to check the loading of images */
private static ImComponent  imageComponent = new ImComponent();
/** MediaTracker waits for images to load before printing them */
private static MediaTracker tracker = new MediaTracker(imageComponent);
/** Used to assign a unique ID to each image in MediaTracker */
private static int          nextImageID = 0;

/**
 * Create an image marker object
 */
public ImageMarker() {

	image = null;
}

/**
 * Create an image marker object
 * @param image The image which is being added to the flow
 */
public ImageMarker(ImageRender image) {

	this.image = image;
}

/**
 * Create an image marker object
 * @param image The image which is being added to the flow
 * @param embedded True if the image is embedded in a line of text
 */
public ImageMarker(ImageRender image, boolean embedded) {

	this.image = image;
	this.embedded = embedded;
}

/**
 * Get the image referred to by this marker
 * @return The image associated with this marker
 */
public ImageRender getImage() {
	return image;
}

/**
 * Set the image referred to by this marker
 * @param image The image which is added to the flow by this marker
 */
public void setImage(ImageRender image) {

	this.image = image;
}

/**
 * Return whether or not the image is embedded in a line
 */
public boolean isEmbedded() {
	return(embedded);
}

/**
 * Returns an indication if this object can be split into components
 * @returns True indicating that a marker object can't be split
 */
public boolean isAtomic() {
	return(true);
}

/**
 * The image will only be drawn to devices which support the Markup interface
 * @param gc The graphics context which will draw this object
 */
public void draw(Graphics2D gc) {

	Object	content = image.getImage();
	Image	drawImage = null;
	String	horizontal, vertical;

	// If this context doesn't support Markup, we don't do anything
	if (!(gc instanceof Markup)) {
		return;
	}

	// Access the MarkupOutput methods of the gc
	Markup mo = (Markup) gc;

	// Get the specified size of the image
	JCUnit.Dimension imgBounds = image.getSize();
	int width = (int) JCUnit.getAsPoints(imgBounds.units, imgBounds.width);
	int height = (int) JCUnit.getAsPoints(imgBounds.units, imgBounds.height);

	// If the image is a component, paint an Image from it
	if (content instanceof Component) {

		Component c = (Component) content;

		// Get the size of the component
		

		
		int imgWidth = c.getWidth();
		int imgHeight = c.getHeight();
		

		// if component size not set, try component's preferred size
		if (imgWidth == 0 || imgHeight == 0) {

			// Set a graphics on ServerRenderable components so they can
			// get font metrics, etc. for layout computations
			if (!c.isShowing() && c instanceof ServerRenderable) {
				((ServerRenderable) c).setGraphics(gc);
			}

			// Get the width/height as necessary from the preferred size
			Dimension dim = c.getPreferredSize();
			if (imgWidth == 0) {
				imgWidth = dim.width;
			}
			if (imgHeight == 0) {
				imgHeight = dim.height;
			}
		}

		// if both dimensions are unspecified, use the component's
		if (height == 0 && width == 0) {
			height = imgHeight;
			width = imgWidth;
		}

		// if one dimension is unset, scale the image proportionally
		else if (height == 0) {
			height = imgHeight * width / imgWidth;
		}
		else if (width == 0) {
			width = imgWidth * height / imgHeight;
		}

		// if the image isn't the correct size, rescale it
		if (height != imgHeight || width != imgWidth) {
			// NOTE:  If this doesn't work for ServerRenderable objects, it
			//        may be because setGraphics() needs to be called and hasn't
			c.setSize(width, height);
		}

		// Create an image from the component
		drawImage = c.createImage(width, height);

		if (drawImage == null) {
			return;
		}

		// Get the image graphics and set its clip
		Graphics imgGC = drawImage.getGraphics();
		imgGC.setClip(new Rectangle(0, 0, width, height));

		// Draw the component into the image
		if (!c.isShowing() && c instanceof ServerRenderable) {
			// ServerRenderable objects draw directly
			((ServerRenderable) c).serverPaint(imgGC);
		} else {
			// Otherwise use a CellRendererPane to draw the component
			CellRendererPane cellRP = new CellRendererPane();
			Frame blankFrame = new Frame();

			cellRP.paintComponent(imgGC, c, blankFrame, 0, 0,
								  width, height, true);
		}
	}

	// If this is an Image or ImageIcon object, use the graphic's drawImage()
	if (content instanceof Image || content instanceof Icon) {

		// get the original (unscaled) image
		if (content instanceof Image) {
			drawImage = (Image) content;
		}
		else if (content instanceof ImageIcon) {
			drawImage = ((ImageIcon) content).getImage();
		}
		else {
			// Of Icons, we only know how to deal with ImageIcons
			throw new IllegalArgumentException("Unknown Icon type");
		}

		// Determine the pixel dimensions of the provided image
		int imgWidth = drawImage.getWidth(this);
		int imgHeight = drawImage.getHeight(this);

		// if the sizes don't match, rescale the image
		if (imgWidth != width || imgHeight != height) {
			drawImage = drawImage.getScaledInstance(width, height,
													Image.SCALE_DEFAULT);
		}

		// Set MediaTracker to watch the image until it's complete
		tracker.addImage(drawImage, imageID);

		try {
			tracker.waitForID(imageID);
		}
		catch (InterruptedException e) {}
	}
	else if (content instanceof EPSImage) {
		// ??? We currently do nothing for EPS in HTML output
	}

	else {
		throw new IllegalArgumentException("Bad image type");
	}

	if (embedded) {
		vertical = getVAlign(image.getVerticalAlignment());
		horizontal = null;
	}
	else {
		vertical = null;
		horizontal = getHAlign(image.getAlignment());
	}

	// if we have an image we can draw, do it.
	if (drawImage != null) {
		mo.outputImage(drawImage, width, height, Color.white, this,
					   horizontal, vertical);

		// if the image was an Image or Icon, remove it from MediaTracker
		if ((content instanceof Image) || (content instanceof Icon)) {
			tracker.removeImage(drawImage);
		}
	}
}

// Convert a horizontal alignment value to a designator
private String getHAlign(int align) {

	switch (align) {

	case JCTextStyle.ALIGNMENT_LEFT:
		return("left");

	case JCTextStyle.ALIGNMENT_RIGHT:
		return("right");

	case JCTextStyle.ALIGNMENT_NONE:
	case JCTextStyle.ALIGNMENT_CENTER:
	case JCTextStyle.ALIGNMENT_JUSTIFY:

	default:
		return("center");
	}
}

// Convert a vertical alignment value to a designator
private String getVAlign(int align) {

	switch (align) {

	case JCDrawStyle.POSITION_ON_BASELINE:
	case JCDrawStyle.ALIGN_TO_BOTTOM:
		return("bottom");

	case JCDrawStyle.POSITION_BELOW_BASELINE:
	case JCDrawStyle.ALIGN_TO_TOP:
		return("top");

	case JCDrawStyle.CENTER_ON_BASELINE:
	case JCDrawStyle.CENTER_ABOVE_BASELINE:
	case JCDrawStyle.CENTER_IN_LINE:

	default:
		return("middle");
	}
}

/**
 * This method satisfies the ImageObserver interface and is used to get
 * asynchronous notification of the known information about the image
 * @param image The image being updated
 * @param flags Flags indicating what information is now available
 * @param x X location of the image
 * @param y Y location of the image
 * @param width The width of the image
 * @param height The height of the image
 * @return True if more information is needed, otherwise false
 */
public boolean imageUpdate(Image img, int flags, int x, int y, int width,
						   int height) {

	// wait until at least the dimensions are available
	if ((flags & ImageObserver.WIDTH) == 0 ||
		(flags & ImageObserver.HEIGHT) == 0) {
		return(true);
	}

	// Use the width if we've been waiting for it
	if (bounds.getWidth().isZero()) {
		bounds.setWidth(new JCUnit.Measure(JCUnit.POINTS, (double) width));
	}

	// Use the height if we've been waiting for it
	if (bounds.getHeight().isZero()) {
		bounds.setHeight(new JCUnit.Measure(JCUnit.POINTS, (double) height));
	}

	// if we're drawing the image we want it to be completed before proceding
	if ((flags & ImageObserver.ALLBITS) == 0) {
		return(true);
	}

	// We're happy now
	return(false);
}

/**
 * Create a Marker object which is a copy of this one
 * @return a clone of this image marker
 */
public Object clone() {

	ImageMarker		new_marker;

	// Clone the symbol object
	new_marker = (ImageMarker) super.clone();

	// Copy dependent objects

	return((Object) new_marker);
}


protected static class ImComponent extends Component {

	protected ImComponent() {
		super();
	}
}

}
