/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: PageNumberMacro.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:26 $ $Locker:  $  Quest Software Inc. 
package com.klg.jclass.page.render;



import com.klg.jclass.page.TextMacro;
import com.klg.jclass.page.JCFlow;
import com.klg.jclass.page.JCPage;

/**
 * PageNumberMacro evaluates to the number of the current page
 */
public class PageNumberMacro implements TextMacro {

/** The text to which this macro has been evaluated */
protected String    text;
/** The status of the result of the last evaluation */
protected int       status;

/**
 * Create a macro instance
 */
public PageNumberMacro() {}

/**
 * Compute the current page number based on the flow and page
 * @param flow The flow within which we are evaluating this macro
 * @param page the page the macro appears upon
 * @return The status of the evaluation
 */
public int evaluate(JCFlow flow, JCPage page) {

	

	// We need either the page or the flow in order to get the page number
	if (page == null && flow == null) {
		text = null;
		status = TextMacro.MACRO_NOT_YET_EVALUATED;
	}

	// if we have a flow, get the information from that
	else if (flow != null) {
		text = Integer.toString(flow.getCurrentPageNumber());
		status = TextMacro.MACRO_EVALUATED;

	}

	// If the page is a document page, use its number
	else if (page.getPageType() == JCPage.PAGE_TYPE_DOCUMENT) {
		text = Integer.toString(page.getPageNumber());
		status = TextMacro.MACRO_EVALUATED;
	}

	// otherwise we have only a non-document page, which we can't use
	else {
		text = null;
		status = TextMacro.MACRO_NOT_YET_EVALUATED;
	}

	
	return(status);
}

/**
 * Return the page number as determined by the last evaluation
 * @return The page number
 */
public String getText() {

	// if there is no current value for the macro, return placeholder text
	if (text == null) {
		return("-1");
	}

	return(text);
}

/**
 * Return the status of the last evaluation of the macro
 * @return An indication of the success of the last evaluation
 */
public int getStatus() {
	return(status);
}

}
