/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: RoundRectRender.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:26 $ $Locker:  $  Quest Software Inc. 
package com.klg.jclass.page.render;

import java.awt.Color;

import com.klg.jclass.page.JCDrawStyle;
import com.klg.jclass.page.JCUnit;




import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;


/**
 * RoundRectRender describes a rectangular object with circular-arc corners
 */
public class RoundRectRender extends Render implements Cloneable {

/** The radius of the arc use do round the corners */
protected JCUnit.Measure    radius;

/** The lower left (min_x, min_y) corner of the rectangle */
protected JCUnit.Point      lowerLeft;
/** The upper right (max_x, max_y) corner of the rectangle */
protected JCUnit.Point      upperRight;

/** The draw style which specifies drawing parameters */
protected JCDrawStyle       style;
/** Indicates if the rectangle is to be filled or not */
protected boolean           fill;

/**
 * Create a rounded-corner rectangle render object
 */
public RoundRectRender() {

	// Create a degenerate rectangle
	bounds = new JCUnit.Dimension(JCUnit.POINTS);
	origin = new JCUnit.Point(JCUnit.POINTS);
	radius = new JCUnit.Measure(JCUnit.POINTS);
	style = null;
}

/**
 * Create a rounded-corner rectangle render object given the diagonal points
 * and a style
 * @param point1 a corner of the rectangle to draw
 * @param point2 the diagonally opposite corner of the rectangle
 * @param style the draw style to use for painting the rectangle
 */
public RoundRectRender(JCUnit.Point point1, JCUnit.Point point2,
                       JCDrawStyle style) {

	double      x_min, y_min, x_max, y_max;

	this.style = style;

	// Make sure the corners are specified in the same units
	if (point1.units != point2.units) {
		point2.x = JCUnit.getAs(point1.units, point2.units, point2.x);
		point2.y = JCUnit.getAs(point1.units, point2.units, point2.y);
		point2.units = point1.units;
	}

	// compute the lowerLeft and upperRight corners...
	if (point1.x > point2.x) {
		x_min = point2.x;
		x_max = point1.x;
	}
	else {
		x_min = point1.x;
		x_max = point2.x;
	}

	if (point1.y > point2.y) {
		y_min = point2.y;
		y_max = point1.y;
	}
	else {
		y_min = point1.y;
		y_max = point2.y;
	}

	// Compute the corners
	lowerLeft = new JCUnit.Point(point1.units, x_min, y_min);
	upperRight = new JCUnit.Point(point1.units, x_max, y_max);

	// Compute the location and size
	origin = new JCUnit.Point(point1.units, x_min, y_min);
	bounds = new JCUnit.Dimension(point1.units, x_max - x_min, y_max - y_min);

	// Since the radius wasn't given, set it to zero
	radius = new JCUnit.Measure();
}

/**
 * Create a rounded-corner rectangle render object given the diagonal points,
 * the radius of the corner arc, and a style
 * @param point1 a corner of the rectangle to draw
 * @param point2 the diagonally opposite corner of the rectangle
 * @param radius the radius to be used to draw the corners
 * @param style the draw style to use for painting the rectangle
 */
public RoundRectRender(JCUnit.Point point1, JCUnit.Point point2,
                       JCUnit.Measure radius, JCDrawStyle style) {

	double      x_min, y_min, x_max, y_max;

	this.style = style;

	// Make sure the corners are specified in the same units
	if (point1.units != point2.units) {
		point2.x = JCUnit.getAs(point1.units, point2.units, point2.x);
		point2.y = JCUnit.getAs(point1.units, point2.units, point2.y);
		point2.units = point1.units;
	}

	// compute the lowerLeft and upperRight corners...
	if (point1.x > point2.x) {
		x_min = point2.x;
		x_max = point1.x;
	}
	else {
		x_min = point1.x;
		x_max = point2.x;
	}

	if (point1.y > point2.y) {
		y_min = point2.y;
		y_max = point1.y;
	}
	else {
		y_min = point1.y;
		y_max = point2.y;
	}

	// Compute the corners
	lowerLeft = new JCUnit.Point(point1.units, x_min, y_min);
	upperRight = new JCUnit.Point(point1.units, x_max, y_max);

	// Compute the location and size
	origin = new JCUnit.Point(point1.units, x_min, y_min);
	bounds = new JCUnit.Dimension(point1.units, x_max - x_min, y_max - y_min);
	this.radius = (JCUnit.Measure) radius.clone();
}

/**
 * Create a rounded-corner rectangle render object from location, size and style
 * @param location the point at which this rectangle begins
 * @param size the size of the rectangle
 * @param style the draw style to use for painting the rectangle
 */
public RoundRectRender(JCUnit.Point location, JCUnit.Dimension size,
				 JCDrawStyle style) {

	this.style = style;

	// copy points and compute bounds
	origin = (JCUnit.Point) location.clone();
	bounds = (JCUnit.Dimension) size.clone();
	radius = new JCUnit.Measure();

	lowerLeft = (JCUnit.Point) location.clone();

    // Compute the position of the upper right corner
    JCUnit.Measure urx = origin.getX();
    urx.add(bounds.getWidth());
    JCUnit.Measure ury = origin.getY();
    ury.add(bounds.getHeight());

    upperRight = new JCUnit.Point(origin.units, urx.distance, ury.distance);
}

/**
 * Create a rounded-corner rectangle render object given the location, size,
 * radius of the corners and style
 * @param location the point at which this rectangle begins
 * @param size the size of the rectangle
 * @param radius the radius of the arcs at the corners
 * @param style the draw style to use for painting the rectangle
 */
public RoundRectRender(JCUnit.Point location, JCUnit.Dimension size,
					   JCUnit.Measure radius, JCDrawStyle style) {

	this.style = style;

	// copy points and compute bounds
	origin = (JCUnit.Point) location.clone();
	bounds = (JCUnit.Dimension) size.clone();
	this.radius = (JCUnit.Measure) radius.clone();

	lowerLeft = (JCUnit.Point) location.clone();

    // Compute the position of the upper right corner
    JCUnit.Measure urx = origin.getX();
    urx.add(bounds.getWidth());
    JCUnit.Measure ury = origin.getY();
    ury.add(bounds.getHeight());

    upperRight = new JCUnit.Point(origin.units, urx.distance, ury.distance);
}

/**
 * Create a list of points describing the perimeter
 * @return An array containing the points of the rectangle's perimeter in order
 */
public JCUnit.Point[] getPoints() {

	JCUnit.Point    points[] = new JCUnit.Point[4];

	// NOTE:  we do not close the figure
	points[0] = (JCUnit.Point) lowerLeft.clone();
	points[1] = (JCUnit.Point) lowerLeft.clone();
	points[1].setX(upperRight.getX());
	points[2] = (JCUnit.Point) upperRight.clone();
	points[3] = (JCUnit.Point) upperRight.clone();
	points[3].setX(lowerLeft.getX());

	return points;
}

/**
 * Get the lower left (min, min) corner of the rectangle
 * @return the point which is the lower-left corner
 */
public JCUnit.Point getLowerLeft() {

	return(lowerLeft);
}

/**
 * Get the upper right (max, max) corner of the rectangle
 * @return the point which is the upper-right corner
 */
public JCUnit.Point getUpperRight() {

	return(upperRight);
}

/**
 * Set the origin of the rectangle
 * @param point The location of the rectangle with respect to its frame
 */
public void setLocation(JCUnit.Point point) {

	// Set the origin
	this.origin = point;

	// Compute the new extents of the rectangle
	lowerLeft = (JCUnit.Point) origin.clone();

    // Compute the position of the upper right corner
    JCUnit.Measure urx = origin.getX();
    urx.add(bounds.getWidth());
    JCUnit.Measure ury = origin.getY();
    ury.add(bounds.getHeight());

    upperRight = new JCUnit.Point(origin.units, urx.distance, ury.distance);
}

/**
 * Set the bounds of the rectangle
 * @param size The size of the rectangle
 */
public void setSize(JCUnit.Dimension size) {

	// Set the origin
	this.bounds = size;

	// Compute the new extents of the rectangle
    JCUnit.Measure urx = origin.getX();
    urx.add(bounds.getWidth());
    JCUnit.Measure ury = origin.getY();
    ury.add(bounds.getHeight());

    upperRight = new JCUnit.Point(origin.units, urx.distance, ury.distance);
}

/**
 * Get the radius of the corner arc
 * @return the measure which gives the radius of the arc at the corners
 */
public JCUnit.Measure getCornerRadius() {
	return radius;
}

/**
 * Set the radius of the corner arc
 * @param style The measure which gives the radius of the arc at the corners
 */
public void setCornerRadius(JCUnit.Measure radius) {

	// Sanity-check the radius
	if (radius.distance < 0.0) {
		throw new IllegalArgumentException("Corner radius must be non-negative");
	}

	this.radius = radius;
}

/**
 * Get the style used to draw this rectangle
 * @return the draw style assigned to this rectangle
 */
public JCDrawStyle getStyle() {
	return style;
}

/**
 * Set the style used to draw this rectangle
 * @param style The draw style used to create this rectangle
 */
public void setStyle(JCDrawStyle style) {

	this.style = style;
}

/**
 * Check if the rectangle is to be filled
 * @return True if the rectangle is to be filled
 */
public boolean isFilled() {
	return fill;
}

/**
 * Set whether this rectangle is to be filled
 * @param filled True if the rectangle is to be filled
 */
public void setFilled(boolean filled) {

	this.fill = filled;
}

/**
 * Returns an indication if this object can be split into components
 * @returns True indicating the rectangle can't be split
 */
public boolean isAtomic() {
	return(true);
}

/**
 * Draw the rectangle described by this object
 * @param gc The graphics context which will draw this object
 */
public void draw(Graphics2D gc) {

	int                         line_type;
	double                      radius_size;
	double                      dash, spacing, line_width;
	Color                       colour;
	BasicStroke                 stroke;
	RoundRectangle2D.Double     rect;

	// Determine the colour to use to draw/fill the figure
	if (fill) {
		colour = style.getFillForegroundColor();

		// if a fill colour is not given, use the draw colour
		if (colour == null) {
			colour = style.getForegroundColor();
		}
	}
	else {
		colour = style.getForegroundColor();
	}

	// if there is no colour set, don't draw anything
	if (colour == null) {
		return;
	}

	// Set the colour to use
	gc.setColor(colour);

	// Create a stroke to handle line width, etc.
	line_width = style.getLineWidth().getAs(JCUnit.POINTS);

	line_type = style.getLineType();
	if (line_type == JCDrawStyle.LINE_TYPE_BROKEN) {
		float[]     dashes = new float[2];

		// Get the dash length and create the dash sequence array
		dash = style.getDashLength().getAs(JCUnit.POINTS);
		dashes[0] = (float) dash; dashes[1] = (float) dash;

		// Create a dashed stroke
		stroke = new BasicStroke((float) line_width, BasicStroke.CAP_SQUARE,
								 BasicStroke.JOIN_MITER, (float) 10.0, dashes,
								 (float) 0.0);
	}
	else if (line_type == JCDrawStyle.LINE_TYPE_DOUBLE) {
		spacing = style.getLineSpacing().getAs(JCUnit.POINTS);

		// Draw a line the thickness of the double line
		line_width = line_width * 2 + spacing;
		stroke = new BasicStroke((float) line_width, BasicStroke.CAP_SQUARE,
								 BasicStroke.JOIN_MITER);
	}
	else if (line_type == JCDrawStyle.LINE_TYPE_PLAIN ||
			 line_type == JCDrawStyle.LINE_TYPE_SINGLE) {
		// Create the standard stroke
		stroke = new BasicStroke((float) line_width, BasicStroke.CAP_SQUARE,
								 BasicStroke.JOIN_MITER);
	} 
	else {
		// line type = JCDrawStyle.LINE_TYPE_NONE
		// so don't draw it
		return;
	}

	// Set the stroke style
	gc.setStroke(stroke);

	radius_size = radius.getAs(JCUnit.POINTS);

	// Create a new rounded rectangle (translate to points)
	rect = new RoundRectangle2D.Double(
							JCUnit.getAsPoints(origin.units, origin.x),
							JCUnit.getAsPoints(origin.units, origin.y),
							JCUnit.getAsPoints(bounds.units, bounds.width),
							JCUnit.getAsPoints(bounds.units, bounds.height),
							radius_size, radius_size);

	// Use the given graphics to draw the rectangle
	if (fill) {
		gc.fill(rect);
	}
	else {
		gc.draw(rect);
	}
}

/**
 * Create a rectangle with rounded corners which is a copy of this one
 * @return a clone of this rounded rectangle
 */
public Object clone() {

	RoundRectRender     new_round_rect;

	// Clone the rounded rectangle
	new_round_rect = (RoundRectRender) super.clone();

	// Copy dependent objects
	new_round_rect.radius = (JCUnit.Measure) this.radius.clone();
	new_round_rect.lowerLeft = (JCUnit.Point) this.lowerLeft.clone();
	new_round_rect.upperRight = (JCUnit.Point) this.upperRight.clone();

	return((Object) new_round_rect);
}

}
