/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: SectionPageNumberMacro.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:26 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.page.render;

import com.klg.jclass.page.TextMacro;
import com.klg.jclass.page.JCFlow;
import com.klg.jclass.page.JCPage;



/**
 * SectionPageNumberMacro evaluates to the number of the current page in the current section
 */
public class SectionPageNumberMacro implements TextMacro {

/** The text to which this macro has been evaluated */
protected String    text;
/** The status of the result of the last evaluation */
protected int       status;

/**
 * Create a macro instance
 */
public SectionPageNumberMacro() {}

/**
 * Determine the section number (as a string) based on flow and page
 * @param flow The document's flow
 * @param page The current page when we are evaluating the macro
 * @return Whether or not the evaluation succeeded
 */
public int evaluate(JCFlow flow, JCPage page) {

	

	// We need the flow in order to get the section number
	if (flow == null) {
		text = null;
		status = TextMacro.MACRO_NOT_YET_EVALUATED;
	}

	// otherwise we get the information from the flow
	else {
		text = Integer.toString(flow.getSectionPageNumber());
		status = TextMacro.MACRO_EVALUATED;
	}

	
	return(status);
}

/**
 * Return the text that was the result of the most recent evaluation
 * @return The current text value of this macro
 */
public String getText() {

	// If there is no evaluated value for the macro return placeholder text
	if (text == null) {
		return("0");
	}

	return(text);
}

/**
 * Return the status of the last evaluation
 * @return The indication of the success of the last macro evaluation
 */
public int getStatus() {
	return(status);
}

}
