/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: StringRender.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:27 $ $Locker:  $  Quest Software Inc. 
package com.klg.jclass.page.render;

import java.text.BreakIterator;

import java.awt.FontMetrics;

import java.awt.Font;



import com.klg.jclass.page.FontBase;
import com.klg.jclass.page.FontMetricsBase;
import com.klg.jclass.page.JCFrame;
import com.klg.jclass.page.JCTextStyle;
import com.klg.jclass.page.JCUnit;




import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;

import java.awt.font.TextAttribute;
import java.text.AttributedString;

import java.util.List;
import java.util.Vector;


/**
 * StringRender describes a text object
 */
public class StringRender extends Render implements Splitable, Cloneable {

/** The text that this render object is responsible for displaying */
protected String            text;

/** The full text of the String that is being rendered */
protected String            fullText;
/** Start index in the full text of the rendered text */
protected int               textStart;
/** End index in the full text of the rendered text */
protected int               textEnd;

/** The text style which describes the text appearance */
protected JCTextStyle       style;
/** The Graphics context most recently used to lay out the string */
// We can test this to see how much recomputation is necessary
protected Graphics2D        saved_gc;

/** Indicate what macros appear in the text of this render object */
protected int               macroFlags;
/** Optional space that may be necessary before the first character */
protected JCUnit.Measure    leadingSpace;
/** Optional space that may be necessary after the last character */
protected JCUnit.Measure    trailingSpace;

/** Store the position at which the text was broken in the last split */
protected int               splitIndex;
/** Store the distance to which the last split was measured */
protected JCUnit.Measure    splitDistance;

/**
 * Create a String render object
 */
public StringRender() {

	// Create a null render object
	style = null;
	text = null;
	fullText = null;
	textStart = -1;
	textEnd = -1;

	origin = new JCUnit.Point(JCUnit.POINTS);
	bounds = new JCUnit.Dimension(JCUnit.POINTS);
}

/**
 * Create a string render object given a string and a style
 * @param gc The graphics context which will draw this object
 * @param text the string for this render object to display
 * @param style the text style to use for formatting the text
 */
public StringRender(Graphics2D gc, String text, JCTextStyle style) {

	FontMetrics     fm;
	Rectangle2D     extent;

	this.style = style;

	this.text = text;
	this.fullText = text;
	this.textStart = 0;
	this.textEnd = text.length();

	// Get the font metrics from the graphics
	saved_gc = gc;
	fm = getFontMetrics(gc, style);

	// compute the size of the new string
	// JDK11 compatibility method:
	extent = FontMetricsBase.getStringBounds(fm, text, gc);
	bounds = new JCUnit.Dimension(JCUnit.POINTS, extent.getWidth(),
								  extent.getHeight());
	origin = new JCUnit.Point(JCUnit.POINTS);
}

/**
 * Create a string render object given a string, endpoints and a style
 * @param gc The graphics context which will draw this object
 * @param text the string from which to extract a substring to display
 * @param first the first character of the substring
 * @param last the last character of the substring
 * @param style the text style to use for formatting the text
 */
public StringRender(Graphics2D gc, String text, int first, int last,
					JCTextStyle style) {

	FontMetrics     fm;
	Rectangle2D     extent;

	this.style = style;

	this.text = text.substring(first, last);
	this.fullText = text;
	this.textStart = first;
	this.textEnd = last;

	// Get the font metrics from the graphics
	saved_gc = gc;
	fm = getFontMetrics(gc, style);

	// compute the size of the string
	// JDK11 compatibility method:
	extent = FontMetricsBase.getStringBounds(fm, this.text, gc);
	bounds = new JCUnit.Dimension(JCUnit.POINTS, extent.getWidth(),
								  extent.getHeight());
	origin = new JCUnit.Point(JCUnit.POINTS);
}

/**
 * Get the text that the render object is displaying
 * @return The text that is under the control of this render object 
 */
public String getText() {
	return text;
}

/**
 * Set the text that the render object is displaying
 * @param gc The graphics context which will draw this object
 * @param text Text to display with this object
 */
public void setText(Graphics2D gc, String text) {

	FontMetrics     fm;
	Rectangle2D     extent;

	this.text = text;
	this.fullText = text;
	this.textStart = 0;
	this.textEnd = text.length();

	// Get the Graphics and FontMetrics for the current style
	saved_gc = gc;
	fm = getFontMetrics(gc, style);

	// compute the size of the string
	// JDK11 compatibility method:
	extent = FontMetricsBase.getStringBounds(fm, text, gc);
	bounds = new JCUnit.Dimension(JCUnit.POINTS, extent.getWidth(),
								  extent.getHeight());
}

/**
 * Set the text that the render object is displaying
 * @param gc The graphics context which will draw this object
 * @param text Full text from which to display a substring
 * @param first First character in the string to display
 * @param last First character after the string to display
 */
public void setText(Graphics2D gc, String text, int first, int last) {

	FontMetrics     fm;
	Rectangle2D     extent;

	this.fullText = text;
	this.text = text.substring(first, last);
	this.textStart = first;
	this.textEnd = last;

	// Get the Graphics and FontMetrics for the current style
	saved_gc = gc;
	fm = getFontMetrics(gc, style);

	// compute the size of the string
	// JDK11 compatibility method:
	extent = FontMetricsBase.getStringBounds(fm, text, gc);
	bounds = new JCUnit.Dimension(JCUnit.POINTS, extent.getWidth(),
								  extent.getHeight());
}

/**
 * Get the full text that includes what this object is displaying
 * @return The full text this belongs to
 */
public String getFullText() {
	return fullText;
}

/**
 * Set the full text that includes what this object is displaying
 * @param text Full text of which a substring is being displayed
 */
public void setFullText(String original) {

	int     location;

	fullText = original;

	// If text is set and exists in fullText, set the endpoints
	location = fullText.indexOf(text);
	if (location != -1) {

		// NOTE: if the string appears more than once use the first occurrence
		textStart = location;
		textEnd = location + text.length();
	}
}

/**
 * Get the position of the first character of this string in the full text
 * @return The index of this (sub)string in the full text
 */
public int getStartIndex() {
	return(textStart);
}

/**
 * Get the position of the last character of this string in the full text
 * @return The end index of this (sub)string in the full text
 */
public int getEndIndex() {
	return(textEnd);
}

/**
 * Get the style used to generate this string
 * @return the text style assigned to this text
 */
public JCTextStyle getStyle() {
	return style;
}

/**
 * Set the style used to generate this string
 * @param gc The graphics context which will draw this object
 * @param style The text style used to format this text
 */
public void setStyle(Graphics2D gc, JCTextStyle style) {

	Rectangle2D     extent;
	FontMetrics     fm;

	this.style = style;

	// Recompute string data based on the new style
	saved_gc = gc;
	fm = getFontMetrics(gc, style);
	// JDK11 compatibility method:
	extent = FontMetricsBase.getStringBounds(fm, text, gc);
	bounds = new JCUnit.Dimension(JCUnit.POINTS, extent.getWidth(),
								  extent.getHeight());
}

/**
 * Get the flags which indicate what macros appear in this text
 * @return The record of which flags appear in this text
 */
public int getMacroFlags() {
	return macroFlags;
}

/**
 * Get the leading space the string requires when not following text
 * @return Leading space requirements of the text
 */
protected JCUnit.Measure getLeadingSpace() {
	// NOTE:  we should compute leading space based on the first character
	return leadingSpace;
}

/**
 * Get the trailing space the string requires when not preceding text
 * @return Trailing space requirements of the text
 */
protected JCUnit.Measure getTrailingSpace() {
	// NOTE:  we should compute trailing space based on the last character
	return trailingSpace;
}

/**
 * Returns an indication if this object can be split into components
 * @returns False indicating that a text object can be split
 */
public boolean isAtomic() {
	return(false);
}

/**
 * Get the width of the string not including trailing white space
 * @param gc The graphics context which will draw this object
 * @param style The text style used to format this text
 */
public JCUnit.Measure getTrimWidth(Graphics2D gc) {

	int             end_index;
	Rectangle2D     extent;
	FontMetrics     fm;

	// Get a font metrics object for the current style and graphics
	saved_gc = gc;
	fm = getFontMetrics(gc, style);

	// Find the last non-whitespace character in the string
	end_index = text.length();
	while ((end_index > 0) &&
		   Character.isWhitespace(text.charAt(end_index - 1))) {
		
		end_index--;
	}

	// Compute and return the trimmed string's length
	// JDK11 compatibility method:
	extent = FontMetricsBase.getStringBounds(fm, text.substring(0, end_index),
											 gc);
	return(new JCUnit.Measure(JCUnit.POINTS, extent.getWidth()));
}

/**
 * Returns the portion of the object before the mark
 * @param gc The graphics context which will draw this object
 * @param mark A distance across the object up to which to split
 * @return A text render object containing the first part of the string
 */
public Render splitBefore(Graphics2D gc, JCUnit.Measure mark) {

	int             breakIndex, oldIndex, lastChar = 0;
	double          distance;
	BreakIterator   splitNodes;
	FontMetrics     fm;
	Rectangle2D     extent;
	StringRender    portion;

	
	

	// Determine the distance we have to fill
	distance = mark.getAs(JCUnit.POINTS);
	
	

	// Take an initial guess as to the split index
	breakIndex = (int) ((distance / bounds.getWidth().getAs(JCUnit.POINTS))
						* text.length());

	// Create an iterator across the string
	splitNodes = BreakIterator.getLineInstance();
	splitNodes.setText(text);

	// Get the FontMetrics from the Graphics?
	saved_gc = gc;
	fm = getFontMetrics(gc, style);

	// Compute the size of the substring
	// JDK11 compatibility method:
	extent = FontMetricsBase.getStringBounds(fm, text.substring(0, breakIndex),
											 gc);
	

	// Find the last break location before the end of the available space
	if (extent.getWidth() <= distance) {

		// If we're on a word, find its beginning
		while ((breakIndex > 0) &&
			   !Character.isWhitespace(text.charAt(breakIndex - 1))) {
			breakIndex--;
		}

		// Move forward until we fill the space
		oldIndex = breakIndex;
		while (extent.getWidth() < distance) {
			oldIndex = breakIndex;
			try {
				lastChar = breakIndex = splitNodes.following(oldIndex);
			} catch (IllegalArgumentException iae) {
				lastChar = breakIndex = oldIndex;
				break;
			}

			// Ignore trailing whitespace in size computation
			while ((lastChar > 0) &&
				   Character.isWhitespace(text.charAt(lastChar - 1))) {
				lastChar--;
			}

			// JDK11 compatibility method:
			extent = FontMetricsBase.getStringBounds(fm,
											text.substring(0, lastChar), gc);
			
		}

		// Use the last break position before we went past the end
		breakIndex = oldIndex;
		
	}
	else {

		// Move backwards until we move within the available space
		while (extent.getWidth() > distance) {
			

			
			breakIndex = splitNodes.preceding(breakIndex);
			

			// Ignore trailing whitespace in size computation
			while ((breakIndex > 0) &&
				   Character.isWhitespace(text.charAt(breakIndex - 1))) {
				breakIndex--;
			}
			// JDK11 compatibility method:
			extent = FontMetricsBase.getStringBounds(fm,
											text.substring(0, breakIndex), gc);
			
		}
	}

	// Store the position at which we split the string
	splitIndex = breakIndex;
	splitDistance = mark;

	// Remove any trailing whitespace
	lastChar = breakIndex;
	while ((lastChar > 0) &&
		   Character.isWhitespace(text.charAt(lastChar - 1))) {
		lastChar--;
	}

	

	// create a render object containing the split portion of the string
	portion = new StringRender(gc, text, 0, lastChar, this.style);

	// if the string was all whitespace which is deleted, set a non-zero length
	if (lastChar == 0 && breakIndex != 0) {
		JCUnit.Dimension size = portion.getSize();
		size.setWidth(new JCUnit.Measure(JCUnit.POINTS, 0.01));
		portion.setSize(size);
	}

	return(portion);
}

/**
 * Returns the portion of the object after the mark
 * @param gc The graphics context which will draw this object
 * @param mark A distance across the object from which to split
 * @return A text render object containing the second part of the string
 */
public Render splitAfter(Graphics2D gc, JCUnit.Measure mark) {

	int             firstChar;
	Rectangle2D     extent;
    FontMetrics     fm;

	

	// First check if there isn't an existing split to this position
	if (!mark.equals(splitDistance)) {

		// Create a split at this position
		
		this.splitBefore(gc, mark);
	}

	// Split off the remainder
	// NOTE:  we don't create a new render object; we just modify this one!
	fullText = text;
	textStart = splitIndex;
	textEnd = text.length();

	// Eat any leading whitespace
	while ((splitIndex < textEnd) &&
		   Character.isWhitespace(text.charAt(splitIndex))) {
		splitIndex++;
	}

	// Create the new substring
	text = text.substring(splitIndex);

	// Get the font metrics from the Graphics
	fm = getFontMetrics(gc, style);

	// Compute the extents of this string
	// JDK11 compatibility method:
	extent = FontMetricsBase.getStringBounds(fm, text, gc);
	bounds = new JCUnit.Dimension(JCUnit.POINTS, extent.getWidth(),
								  extent.getHeight());

	// Return this object with the modified string
	return(this);
}

/**
 * Returns the portion of the object before the mark (ignoring words)
 * @param gc The graphics context which will draw this object
 * @param mark A distance across the object up to which to split
 * @return A text render object containing the first part of the string
 */
public Render forceSplit(Graphics2D gc, JCUnit.Measure mark) {

	int             breakIndex;
	double          distance;
	FontMetrics     fm;
	Rectangle2D     extent;
	StringRender    portion;

	
	

	// Determine the distance we have to fill
	distance = mark.getAs(JCUnit.POINTS);
	

	// Take an initial guess as to the split index
	breakIndex = (int) ((distance / bounds.getWidth().getAs(JCUnit.POINTS))
						* text.length());

	// Get the FontMetrics from the Graphics?
	saved_gc = gc;
	fm = getFontMetrics(gc, style);

	// Compute the size of the substring
	// JDK11 compatibility method:
	extent = FontMetricsBase.getStringBounds(fm, text.substring(0, breakIndex),
											 gc);
	

	// Find the last break location before the end of the available space
	if (extent.getWidth() <= distance) {

		// Scan forward until we pass the string length
		while (breakIndex < text.length() && extent.getWidth() <= distance) {
			breakIndex++;
			// JDK11 compatibility method:
			extent = FontMetricsBase.getStringBounds(fm,
											text.substring(0, breakIndex), gc);
		}

		// back up one character
		breakIndex--;
	}
	else {

		// Scan backward until we pass the string length
		while (breakIndex > 0 && extent.getWidth() > distance) {
			breakIndex--;
			// JDK11 compatibility method:
			extent = FontMetricsBase.getStringBounds(fm,
											text.substring(0, breakIndex), gc);
		}
	}

	// We _must_ split at least one character off the string
	if (breakIndex == 0) {
		breakIndex++;
	}

	// Store the position at which we split the string
	splitIndex = breakIndex;
	splitDistance = mark;

	

	// create a render object containing the split portion of the string
	portion = new StringRender(gc, text, 0, breakIndex, this.style);

	return(portion);
}

/**
 * Joins two render objects which had been split apart
 * @param gc The graphics context which will draw this object
 * @param following The render object which follows this (was once joined to)
 */
public void unsplit(Graphics2D gc, Render following) {

	int             i, first, last;
	FontMetrics     fm;
	Rectangle2D     extent;
	// If "following" is not a string this will throw an exception
	StringRender    tail = (StringRender) following;

	// if the strings do not occur in the expected order, it is an error
	first = tail.getStartIndex();
	if (first < textEnd) {
		throw new IllegalArgumentException("Strings do not adjoin");
	}

	// If anything other than whitespace separates the strings, it is an error
	for (i = textEnd; i < first; i++) {
		if (!Character.isWhitespace(fullText.charAt(i))) {
			throw new IllegalArgumentException("Strings do not adjoin");
		}
	}

	// Create the substring joining the strings
	text = fullText.substring(textStart, tail.getEndIndex());

	// Get the font metrics from the Graphics
	fm = getFontMetrics(gc, style);

	// Compute the extents of this string
	// JDK11 compatibility method:
	extent = FontMetricsBase.getStringBounds(fm, text, gc);
	bounds = new JCUnit.Dimension(JCUnit.POINTS, extent.getWidth(),
								  extent.getHeight());
}

/**
 * Remove any whitespace characters a the end of the string (not changing
 * the stored full text) and recompute the size of the string
 */
public void trimTrailingWhitespace() {

	int             last_char = text.length() - 1;
	FontMetrics     fm;
	Rectangle2D     extent;

	// Back up as long as the end character is white space
	while (last_char >= 0 && Character.isWhitespace(text.charAt(last_char))) {
		textEnd--;
		last_char--;
	}

	// If the entire string is whitespace, return an infinitesimal string
	if (last_char < 0) {
		textEnd = 0;
		text = fullText.substring(textStart, textEnd);
		bounds.setWidth(new JCUnit.Measure(JCUnit.POINTS, 0.05));
		return;
	}

	// If the end of the text changed, recompute the string
	if (last_char < text.length() - 1) {
		text = fullText.substring(textStart, textEnd);
		fm = getFontMetrics(saved_gc, style);

		// compute the size of the new string
		// JDK11 compatibility method:
		extent = FontMetricsBase.getStringBounds(fm, text, saved_gc);
		bounds = new JCUnit.Dimension(JCUnit.POINTS, extent.getWidth(),
									  extent.getHeight());
	}
}

/**
 * Convert this text object into a sequence of text objects containing
 * individual words
 * @return the list of objects constructed to replace this one
 */
public List splitIntoWords() {

	int             i, word_last, last;
	boolean         on_word = false;
	Vector          wordList = new Vector();
	FontMetrics     fm;
	Rectangle2D     extent;
	Render          render;
	JCUnit.Measure  new_x;
	JCUnit.Point    new_position;

	// Get the FontMetrics from the Graphics
	fm = getFontMetrics(saved_gc, style);

	// Scan through the string from back to front
	word_last = last = text.length();
	for (i = last - 1; i >= 0; i--) {
		if (Character.isWhitespace(text.charAt(i))) {

			// If we've just found the beginning of a word
			if (on_word) {

				// Get the width of the string preceding the word
				// JDK11 compatibility method:
				extent = FontMetricsBase.getStringBounds(fm,
											text.substring(0, i + 1), saved_gc);

				// Compute the start point of this string
				new_x = origin.getX();
				new_x.add(new JCUnit.Measure(JCUnit.POINTS, extent.getWidth()));
				new_position = (JCUnit.Point) origin.clone();
				new_position.setX(new_x);

				// Create this word as an individual object
				render = new StringRender(saved_gc, text, i + 1, word_last,
										  style);
				render.setLocation(new_position);
				wordList.add(render);

				// We are no longer tracking a word
				on_word = false;
			}
		}

		// otherwise we have a character which might form part of a word
		else {
			// If we're not already on a word
			if (!on_word) {

				// We're now on a word; note its ending position
				on_word = true;
				word_last = i + 1;
			}
		}
	}

	// if we finished the string on a word, create it
	if (on_word) {
		new_position = (JCUnit.Point) origin.clone();

		// Create this word as an individual object
		render = new StringRender(saved_gc, text, 0, word_last, style);
		render.setLocation(new_position);
		wordList.add(render);
	}

	return(wordList);
}

/**
 * Determine if the text of this string has a decimal (period) in it
 * @return true if there is a decimal in the displayed text
 */
public boolean hasDecimal() {
	return(text.indexOf(".") != -1);
}

/**
 * Get the position of a decimal character in the text of this string
 * @return the position (middle) of the decimal for alignment purposes
 */
public JCUnit.Measure getDecimalPosition() {

	int             decimal_index;
	FontMetrics     fm;
	Rectangle2D     extent;
	JCUnit.Measure  decimal;

	// find the decimal point, return -1 if there isn't one
	decimal_index = text.indexOf(".");
	if (decimal_index == -1) {
		return(new JCUnit.Measure(JCUnit.POINTS, -1));
	}

	// Get the FontMetrics from the Graphics
	fm = getFontMetrics(saved_gc, style);

	// Average the position before and after the decimal
	// JDK11 compatibility method:
	extent = FontMetricsBase.getStringBounds(fm,
								text.substring(0, decimal_index), saved_gc);
	decimal = new JCUnit.Measure(JCUnit.POINTS, extent.getWidth());
	// JDK11 compatibility method:
	extent = FontMetricsBase.getStringBounds(fm,
								text.substring(0, decimal_index + 1), saved_gc);
	decimal.add(new JCUnit.Measure(JCUnit.POINTS, extent.getWidth()));
	decimal.divide(2.0);

	return(decimal);
}

/**
 * Draw the text described by this object
 * @param gc The graphics context which will draw this object
 */
public void draw(Graphics2D gc) {

	int                 last, offset;
	float               x, y, point_size;
	Font                font;
	AttributedString    attrib;

	// ignore unset or zero-length strings
	if (text == null || text.length() == 0) {
		
		return;
	}

	
	

	// Compute the location of the string in points
	x = (float) JCUnit.getAsPoints(origin.units, origin.x);
	y = (float) JCUnit.getAsPoints(origin.units, origin.y);

	// Convert the text into an attributed string
	attrib = new AttributedString(text);

	last = text.length();
	font = style.getFont();

	// Get the font attributes
	offset = style.getBaselineOffset();
	point_size = (float) style.getPointSize();

	// handle super- and sub- script
	if (offset == JCTextStyle.OFFSET_SUBSCRIPT ||
		offset == JCTextStyle.OFFSET_SUPERSCRIPT) {

		point_size *= (float) style.getSubscriptRatio();
	}

	// handle font and size together
	// JDK11 compatibility method:
	font = FontBase.deriveFont(font, point_size);
	attrib.addAttribute(TextAttribute.FONT, font, 0, last);

	// handle underlining
	if (style.getUnderlining() == JCTextStyle.LINEMODE_UNDERLINE) {
		attrib.addAttribute(TextAttribute.UNDERLINE, TextAttribute.UNDERLINE_ON,
							0, last);
	}

	// draw the string
	gc.setColor(style.getColor());
	gc.drawString(attrib.getIterator(), x, y);
}

/**
 * Create a string render object which is a copy of this one
 * @return a clone of this string
 */
public Object clone() {

	StringRender    new_string;

	// Clone the string
	new_string = (StringRender) super.clone();

	// Copy dependent objects
	new_string.text = new String(text);
	new_string.fullText = new String(fullText);

	return((Object) new_string);
}

/**
 * Get the font metrics object appropriate to the font style and attributes
 * @param gc The graphics for which the text is being sized
 * @param style The JCTextStyle controlling the appearance of the text
 * @return The appropriate FontMetrics
 */
private FontMetrics getFontMetrics(Graphics2D gc, JCTextStyle style) {

	double          point_size;
	FontMetrics     fm;
	Font            font, subscript_font;

	// Get the base font from the style
	font = style.getFont();

	// If the font is a sub- or superscript
	if (style.getBaselineOffset() == JCTextStyle.OFFSET_SUBSCRIPT ||
		style.getBaselineOffset() == JCTextStyle.OFFSET_SUPERSCRIPT) {

		// Get a font scaled to the correct size
		// JDK11 compatibility method:
		point_size = FontBase.getSize2D(font) * style.getSubscriptRatio();
		// JDK11 compatibility method:
		subscript_font = FontBase.deriveFont(font, (float) point_size);

		// Get the metrics from this font
		fm = gc.getFontMetrics(subscript_font);
	}

	// otherwise just use the metrics of the base font
	else {
		fm = gc.getFontMetrics(font);
	}

	return(fm);
}

public String toString()
{
	return "StringRender: " + text;
}

}
