/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: TableCellRender.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:27 $ $Locker:  $  Quest Software Inc. 
package com.klg.jclass.page.render;

import java.awt.Color;
import java.awt.Component;
import java.awt.Rectangle;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Insets;

import javax.swing.CellRendererPane;

import com.klg.jclass.page.JCPageTable;
import com.klg.jclass.page.PageTableCellInfo;
import com.klg.jclass.page.JCTextStyle;
import com.klg.jclass.page.JCUnit;
import com.klg.jclass.page.Markup;

import com.klg.jclass.cell.JCCellInfo;
import com.klg.jclass.cell.JCCellRenderer;
import com.klg.jclass.cell.JCComponentCellRenderer;
import com.klg.jclass.cell.JCLightCellRenderer;
import com.klg.jclass.cell.EditorRendererRegistry;

import com.klg.jclass.table.TableDataModel;

import com.klg.jclass.util.formulae.ExpressionVariable;
import com.klg.jclass.util.formulae.MathScalar;




import java.awt.Graphics2D;


/**
 * TableCellRender wraps up the data necessary to render a single cell
 * from a TableData
 */
public class TableCellRender extends Render implements Cloneable {

/** The TableData which contains the cell to be displayed */
protected TableDataModel	tableData;
/** The row location of the cell in the table data */
protected int				row;
/** The column location of the cell in the table data */
protected int				column;
/** The JCPageTable which contains the data */
protected JCPageTable		table;

/** A CellRendererPane in which to draw the cell for output */
protected static CellRendererPane	cellRendererPane = new CellRendererPane();

/** The horizontal alignment of the image within the frame */
protected int               horizontalAlignment;
/** The vertical alignment of the image within its line */
protected int               verticalAlignment;
/** Indicates if the image background is to be filled or not */
protected boolean           fillBackground;

/**
 * Create a TableCell render object
 */
public TableCellRender() {

}

/**
 * Create a table cell render object for a given table and cell
 * @param table The PageTable from which to extract the cell
 * @param row The cell's row in the TableDataModel
 * @param column The cell's column position in the TableDataModel
 */
public TableCellRender(JCPageTable table, int row, int column) {

	this(table, row, column, null);
}

/**
 * Create a table cell render object for a given table and cell
 * @param table The PageTable from which to extract the cell
 * @param row The cell's row in the TableDataModel
 * @param column The cell's column position in the TableDataModel
 * @param size The size of space the table cell is to be fit into
 */
public TableCellRender(JCPageTable table, int row, int column,
					   JCUnit.Dimension size) {

	// create a basic render object
	super();

	// Make sure the cell position is valid
	this.tableData = table.getTableData();
	if (row < 0 || row > tableData.getNumRows() ||
		column < 0 || column > tableData.getNumColumns()) {
		throw new ArrayIndexOutOfBoundsException("Cell (" + row + ", " + column
												 + ") is out of bounds");
	}

	// set the table data, cell location and size
	this.row = row;
	this.column = column;
	this.table = table;

	if (size != null) {
		this.bounds.assign(size);
	}
}

/**
 * Get the table data for this object
 * @return The TableDataModel this render object references
 */
public TableDataModel getTableData() {
	return tableData;
}

/**
 * Set the table data to retrieve the cell from
 * @param data The TableDataModel this object is to refer to
 */
public void setTableData(TableDataModel data) {

	// Make sure the cell position is still valid
	if (row < 0 || row > tableData.getNumRows() ||
		column < 0 || column > tableData.getNumColumns()) {
		throw new ArrayIndexOutOfBoundsException("Cell (" + row + ", " + column
												 + ") is out of bounds");
	}

	this.tableData = data;
}

/**
 * Retrieve the row of the table data the cell is in
 * @return The row position of the cell
 */
public int getRow() {
	return(row);
}

/**
 * Retrieve the column of the table data the cell is in
 * @return The column position of the cell
 */
public int getColumn() {
	return(column);
}

/**
 * Set the cell in the table data to which this object refers
 * @param row The row position of the cell in the table 
 * @param column The column position of the cell in the table 
 */
public void setCellPosition(int row, int column) {

	// Make sure the cell is valid
	if (row < 0 || row > tableData.getNumRows() ||
		column < 0 || column > tableData.getNumColumns()) {
		throw new ArrayIndexOutOfBoundsException("Cell (" + row + ", " + column
												 + ") is out of bounds");
	}

	// Store the cell position
	this.row = row;
	this.column = column;
}

/**
 * Returns an indication if this object can be split into components
 * @returns True indicating the line can't be split
 */
public boolean isAtomic() {
	return(true);
}

/**
 * Draw the image described by this object
 * @param gc The graphics context which will draw this object
 */
public void draw(Graphics2D gc) {

	Object contents = tableData.getTableDataItem(row, column);
	// Check if the tableData contains data for this cell
	// Also, do not allow Markup gc's to draw this render object
	// otherwise, the contents will show up twice -- once from this
	// object (in the wrong place) and once from the Markup objects
	if (contents == null || gc instanceof Markup) {
		return;
	}

	// Set the cell's position context in case variables need evaluating
	ExpressionVariable.setVariable(table, "currentRow", new MathScalar(row));
	ExpressionVariable.setVariable(table, "currentColumn", new MathScalar(column));

	// Get a renderer for the cell contents
	Class type = contents.getClass();
	JCCellRenderer cr = EditorRendererRegistry.getCentralRegistry().getCellRendererInstance(type, null);

	// Create a fake JCTableCellInfo object
	JCCellInfo cellInfo = new PageTableCellInfo(table, row, column);

	// Determine the area the cell is drawn into
	int x = (int) JCUnit.getAsPoints(origin.units, origin.x);
	int y = (int) JCUnit.getAsPoints(origin.units, origin.y);
	int width = (int) JCUnit.getAsPoints(bounds.units, bounds.width);
	int height = (int) JCUnit.getAsPoints(bounds.units, bounds.height);

	//clip the rectangle
	Rectangle clipBounds = gc.getClipBounds();
	gc.clipRect(x, y, width, height);

	// Call the cell renderer's draw method
	if (cr instanceof JCLightCellRenderer) {
		//JCStringCellRenderer does this strange thing where it calculuates
		//the baseline from fontheight - 4 rather than using the ascent.
		//We want to use the ascent, so figure out the difference between
		//the two methods and add it into our drawing area rectangle
		gc.setFont(((PageTableCellInfo)cellInfo).getFont());
		FontMetrics fm = gc.getFontMetrics();
		int diff = fm.getAscent() - (fm.getHeight() - 4);
		y += diff;

		//lets give ourselves an extra pixel in from the edge so that we
		//don't get clobbered by the border
		x++;
		y++;
		width--;
		height--;
		
		

		
		gc.translate(x, y);

		((PageTableCellInfo)cellInfo).setDrawingArea(new Rectangle(0,0,width,height));
		((JCLightCellRenderer) cr).draw(gc, cellInfo, contents, false);
		

		
		gc.translate(-x, -y);

	}

	else if (cr instanceof JCComponentCellRenderer) {
		JCComponentCellRenderer jccr = (JCComponentCellRenderer) cr;
		Component comp = jccr.getRendererComponent(cellInfo, contents, false);

		// Paint the component, using a generic Frame as the Container
		cellRendererPane.paintComponent(gc, comp, new java.awt.Frame(), x, y,
										width, height, true);
	}

	/// Reset the clip



	gc.setClip(clipBounds);

}

/**
 * Create a cell render which is a copy of this one
 * @return a clone of this cell render
 */
public Object clone() {

	TableCellRender     new_cell;

	// Clone and return the table cell
	new_cell = (TableCellRender) super.clone();
	return((Object) new_cell);
}

}
