/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: TableMarker.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:27 $ $Locker:  $  Quest Software Inc. 

package com.klg.jclass.page.render;

import java.awt.Point;

import com.klg.jclass.page.JCPageTable;
import com.klg.jclass.page.JCTextStyle;
import com.klg.jclass.page.Markup;

import com.klg.jclass.table.TableDataModel;




import java.util.Iterator;

import java.awt.Graphics2D;


/**
 * TableMarker describes a marker for a table's insertion in the document flow
 */
public class TableMarker extends Render implements Cloneable, FlowMarker {

/** The table which is being inserted in the flow */
protected JCPageTable		table;

/** The first row of the table being printed in this cell group */
protected int				firstRow;
/** The first column of the table being printed in this cell group */
protected int				firstColumn;

/** The style which describes the text attributes at the table's location */
protected JCTextStyle		style;

/** The alignment of the table in the column/frame */
protected int				alignment;

/**
 * Create a table marker object
 */
public TableMarker() {

	this(null, null, -1, -1);
}

/**
 * Create a table marker object
 * @param table The table which is being added to the flow
 */
public TableMarker(JCPageTable table) {

	this(table, null, -1, -1);
}

/**
 * Create a table marker object
 * @param table The table which is being added to the flow
 * @param style The style which provides local layout context for the table
 */
public TableMarker(JCPageTable table, JCTextStyle style) {

	this(table, style, -1, -1);
}

/**
 * Create a table marker object
 * @param table The table which is being added to the flow
 * @param row The first row of the table being drawn
 * @param column The first column of the table being drawn
 */
public TableMarker(JCPageTable table, int row, int column) {

	this(table, null, -1, -1);
}

/**
 * Create a table marker object
 * @param table The table which is being added to the flow
 * @param style The style which provides local layout context for the table
 * @param row The first row of the table being drawn
 * @param column The first column of the table being drawn
 */
public TableMarker(JCPageTable table, JCTextStyle style, int row, int column) {

	this.table = table;
	this.style = style;
	firstRow = row;
	firstColumn = column;
}

/**
 * Get the table referred to by this marker
 * @return The table associated with this marker
 */
public JCPageTable getTable() {
	return table;
}

/**
 * Set the table referred to by this marker
 * @param table The table which is added to the flow by this marker
 */
public void setTable(JCPageTable table) {

	this.table = table;
}

/**
 * Get the alignment of the marked table
 * @return The alignment the table this marker represents has
 */
public int getAlignment() {
	return(table.getAlignment());
}

/**
 * Set the alignment of a table
 * @param alignment The alignment to use for this marker's table
 */
/*
public void setAlignment(int alignment) {
	table.setAlignment(alignment);
}
*/

/**
 * Returns an indication if this object can be split into components
 * @returns True indicating that a marker object can't be split
 */
public boolean isAtomic() {
	return(true);
}

/**
 * The table will only be drawn to devices which support the Markup interface
 * @param gc The graphics context which will draw this object
 */
public void draw(Graphics2D gc) {

	// If this context doesn't support Markup, we don't do anything
	if (!(gc instanceof Markup)) {
		return;
	}

	// For the initial case of HTML output, we only output the whole
	// table, as initiated by its global placement
	if (firstRow != -1 || firstColumn != -1) {
		return;
	}

	// Access the Markup methods of the gc
	Markup mo = (Markup) gc;

	// Initialise a table with the correct number of columns
	mo.startTable(table.getColumnCount());

	// if the table has headers, output them
	if (table.hasHeaders()) {
		JCPageTable headerTable = table.getHeaders();

		outputRows(mo, headerTable);
	}

	// Output the rows of the table
	outputRows(mo, table);

	// Finish the table
	mo.endTable();
}

/** Output rows of a table */
private void outputRows(Markup mo, JCPageTable tbl) {

	int					r = -1, c = -1;
	Iterator			rli, cli;
	JCPageTable.Row		row;
	JCPageTable.Cell	cell;
	TableDataModel		tdm = tbl.getTableData();

	// Get an iterator over the rows in the table
	rli = tbl.getRowList().iterator();

	// iterate over the rows of the table
	while (rli.hasNext()) {

		// Keep count of the position in the table
		r++;
		c = -1;

		// Start a table row
		mo.startRow();

		// Get the next row of the table
		row = (JCPageTable.Row) rli.next();

		// if the row has no cells, go on to the next row
		if (row == null) {
			mo.endRow();
			continue;
		}

		// Get an iterator over the cells in the row
		cli = row.getCellList().iterator();

		// iterate over the cells in the row
		while (cli.hasNext()) {
			c++;

			// Get a cell from the current row
			cell = (JCPageTable.Cell) cli.next();

			// if the cell exists and is not spanned, start it normally;
			// if the cell does not exist we still need to mark a place for it
			if (cell == null || !cell.isSpanned()) {

				mo.startCell("left", tbl.isHeader());
			}

			// otherwise, the cell is part of a span
			else {

				// Get the origin of the span
				Point origin = cell.getSpanOrigin();

				// if the cell is not the span origin, ignore it
				if (origin.y != r && origin.x != c) {
					continue;
				}

				// start a spanned cell
				mo.startCell("left", tbl.isHeader(), cell.getSpanRows(),
							 cell.getSpanColumns());
			}

			// If the cell has a contents frame, draw it
			if (cell != null && cell.getFrameRaw() != null) {
				cell.getFrameRaw().draw(((Graphics2D) mo));
			}

			// otherwise the cell has no contents, check for tableData
			else if (tdm != null) {

				// Get the contents of the tableData at this cell
				Object contents = tdm.getTableDataItem(r, c);

				// if there is nothing in the cell, end it and continue
				if (contents == null) {
					mo.endCell(tbl.isHeader());
					continue;
				}

				// Draw the cell contents...
				// ???
				((Graphics2D) mo).drawString(contents.toString(),
											 (float) 0.0, (float) 0.0);
			}

			// End the cell
			mo.endCell(tbl.isHeader());
		}

		// End the row
		mo.endRow();
	}
}

/**
 * Create a Marker object which is a copy of this one
 * @return a clone of this table marker
 */
public Object clone() {

	TableMarker		new_marker;

	// Clone the symbol object
	new_marker = (TableMarker) super.clone();

	// Copy dependent objects

	return((Object) new_marker);
}

}
