/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: BaseWizard.java,v $Revision: 1.2 $
//          $Date: 2006-03-17 21:13:27 $ $Locker:  $ Quest Software Inc.
package com.klg.jclass.swing;

import com.klg.jclass.util.JCListenerList;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Base class for common code for JCSplitWizard and JCWizard.
 */
public abstract class BaseWizard extends JPanel {
/** Layout used. */
protected CardLayout cardLayout;

/**  Handle for the container which has the cardlayout.*/
protected Container cardOwner;

/** List of wizard listeners registered with the Bean. */
transient protected JCListenerList listeners;


/**
 * No argument constructor for the baseWizard,
 * called by subclasses.
  */
public BaseWizard() {
    cardLayout = new CardLayout();
}

/**
 * Provided for Beans property editors - has no effect.
 * @param s
 */
public void setAbout(String s) {}

/**
 * Returns vendor information.
 * @return vendor information
 */
public String getAbout() {
    return(JCVersion.getVersionString());
}

/**
 * Returns true if the given component is the last page.
  * @param comp
 * @return  whether or not the given component is the last page.
 */
public boolean isLastPage(Component comp) {


    if (comp == getComponent(getComponentCount() - 1)) {
        return true;
    }
    return false;
}

/**
 * Moves to the wizard page with the specified name.
 * @param name name of the page when it was added to the <code>JCWizard</code>
 * @see #first
 * @see #last
 */
public void show(String name) {
    cardLayout.show(cardOwner, name);
}


/**
 * Should return the last page.  Implemented by subclasses.
  * @return the last page.
 */
protected abstract Component getCurrentPage();

/**
 * Moves to the first page in the Wizard.
 * @see #last
 * @see #show
 */
public void first() {
    cardLayout.first(cardOwner);
}

/**
 * Moves to the last page in the Wizard.
 * @see #first
 * @see #show
 */
public void last() {
    cardLayout.last(cardOwner);
}

/**
 * Advances to the next page.
 * @see #previous
 * @see #finish
 * @see #cancel
 * @return whether or not the move to the next page was successful.
 */
public abstract boolean next();

/**
 * Moves to the previous page.
 * @return whether or not the action was successful
 * @see #next
 * @see #finish
 * @see #cancel
 */
public abstract boolean previous();

/**
 * Invokes the registered "cancel" action.
 * @see #next
 * @see #previous
 * @see #finish
 * @return whether or not the action was successful
 */
public abstract boolean cancel();

/**
 * Invokes the registered "help" action.
 * @see #next
 * @see #previous
 * @see #finish
 * @see #cancel
 */
public abstract void help();


/**
 * Invokes the registered "finish" action
 * @return whether or not the action was successful
 * @see #next
 * @see #previous
 * @see #cancel
  */
public abstract boolean finish();


/**
 * Adds a new <code>WizardListener</code> to the list.
 * @param l
 */
public void addWizardListener(JCWizardListener l)
{
    listeners = JCListenerList.add(listeners, l);
}

/**
 * Removes a <code>ValidateListener</code> from the list.
 * @param l
 */
public void removeWizardListener(JCWizardListener l)
{
    listeners = JCListenerList.remove(listeners, l);
}





/**
 * A listener on the "next" button that is implemented as an anonymous
 * inner class; it simply invokes the containing classes <code>next()</code>
 * method.
 * @see #next
 */
transient protected ActionListener nextListener = new ActionListener()
{
    public void actionPerformed(ActionEvent e)
    {
        next();
    }
};
  /**
 * A listener on the "previous" button that is implemented as an anonymous
 * inner class; it simply invokes the containing classes <code>previous()</code>
 * method.
 * @see #previous
 */
transient protected ActionListener previousListener = new ActionListener()
{
    public void actionPerformed(ActionEvent e)
    {
        previous();
    }
};
    /**
 * A listener on the "finish" button that is implemented as an anonymous
 * inner class; it simply invokes the containing classes <code>finish()</code>
 * method.
 * @see #finish
 */
transient protected ActionListener finishListener = new ActionListener()
{
    public void actionPerformed(ActionEvent ev)
    {
        finish();
    }
};
    /**
 * A listener on the "cancel" button that is implemented as an anonymous
 * inner class; it simply invokes the containing classes <code>cancel()</code>
 * method.
 * @see #cancel
 */
transient protected ActionListener cancelListener = new ActionListener()
{
    public void actionPerformed(ActionEvent ev)
    {
        cancel();
    }
};

/**
 * A listener on the "help" button that is implemented as an anonymous
 * inner class; it simply invokes the containing classes <code>help()</code>
 * method.
 * @see #help
 */
transient protected ActionListener helpListener = new ActionListener()
{
    public void actionPerformed(ActionEvent ev)
    {
        help();
    }
};
}
