/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

package com.klg.jclass.swing;

import com.klg.jclass.util.swing.JCAction;
import com.klg.jclass.util.swing.JCBrace;
import com.klg.jclass.util.swing.JCSpring;
import com.klg.jclass.util.JCLocaleManager;

import javax.swing.*;
import java.awt.event.ActionEvent;

/**
 * BaseWizardPage  is the common ancestor for SplitWizardPage and
 * JCWizardPage, containing common code.
 */
public abstract class BaseWizardPage extends JPanel
        implements java.io.Serializable {

/** Indicates that no buttons are to be added to the page */
static public final int NO_BUTTONS = 0x00;
/** Indicates that the Previous button should be added to the page*/
static public final int PREVIOUS   = 0x01;
/** Indicates that the Next button should be added to the page */
static public final int NEXT       = 0x02;
/** Indicates thate the Finish button should be added to the page */
static public final int FINISH     = 0x04;
/** Indicates that the Cancel button should be added to the page */
static public final int CANCEL     = 0x08;
/** Indicates that the Help button should be added to the page */
static public final int HELP       = 0x10;
/** Indicates that all buttons should be added to the page */
static public final int ALL = PREVIOUS | NEXT | FINISH | CANCEL | HELP;
protected static final int GROUP_SPACING = 5;
protected static final int MARGIN = 11;
protected JButton      previousButton;
protected JButton      nextButton;
protected JButton      finishButton;
protected JButton      cancelButton;
protected JButton      helpButton;
protected NextAction     nextAction;
protected PreviousAction previousAction;
protected CancelAction   cancelAction;
protected HelpAction     helpAction;
protected JPanel       buttonBox;
protected JSeparator   separator;


/**
 * creates the bottom button box with the specified buttons.
 * This method is for internal use, but is protected for the
 * convenience of subclassers.
  * @param button_flags  The static ints from this class that
 * indicate buttons.
 */
protected void createButtonBox(int button_flags, boolean old) {


    buttonBox = new JPanel();
    buttonBox.setLayout(new BoxLayout(buttonBox, BoxLayout.X_AXIS));

    buttonBox.setName("buttonbox");

    //add(bottomBox, BorderLayout.SOUTH);
    if (button_flags == 0) {
        // No buttons to add :-(
        return;
    }
    if (old){
    // Margin on left hand side
        buttonBox.add(JCBrace.createHorizontalBrace(MARGIN));
    // Add spring to cause right justification of buttons

        buttonBox.add(JCSpring.createHorizontalSpring());
    }



    if ((button_flags & PREVIOUS) != 0) {
        previousAction = new PreviousAction();
        previousButton = previousAction.createButton();
        buttonBox.add(previousButton);
    }
    if ((button_flags & NEXT) != 0) {
        nextAction = new NextAction();
        nextButton = nextAction.createButton();
        buttonBox.add(nextButton);
    }
    if ((button_flags & FINISH) != 0) {
        buttonBox.add(JCBrace.createHorizontalBrace(GROUP_SPACING));
        // PENDING internationalization
        finishButton = new JButton("Finish");
        buttonBox.add(finishButton);
    }
        buttonBox.add(JCSpring.createHorizontalSpring());
    if ((button_flags & CANCEL) != 0) {
        buttonBox.add(JCBrace.createHorizontalBrace(GROUP_SPACING));
        cancelAction = new CancelAction();
        cancelButton = cancelAction.createButton();
        buttonBox.add(cancelButton);
    }
    if ((button_flags & HELP) != 0) {
        buttonBox.add(JCBrace.createHorizontalBrace(GROUP_SPACING));
        helpAction = new HelpAction();
        helpButton = helpAction.createButton();
        buttonBox.add(helpButton);
    }
    // Margin on right hand side
    buttonBox.add(JCBrace.createHorizontalBrace(MARGIN));



}

/**
 * Gets the nextButton
  * @return nextButton
 */
public JButton getNextButton() {
    return(nextButton);
}

/**
 * Gets the previousButton
  * @return previousButton
 */
public JButton getPreviousButton() {
    return(previousButton);
}

/**
 * gets the finishButton
 * @return finishButton
 */
public JButton getFinishButton() {
    return(finishButton);
}

/**
 *  Gets the cancelButton
  * @return  cancelButton
 */
public JButton getCancelButton() {
    return(cancelButton);
}

/**
 * Gets the helpButton
  * @return helpButton
 */
public JButton getHelpButton() {
    return(helpButton);
}

 /**
 * Provided for Beans property editors - has no effect.
 * @param s
 */
public void setAbout(String s) {}

    /**
 * Returns vendor information.
 * @return vendor information
 */
public String getAbout() {
    return(JCVersion.getVersionString());
}

/**
 * Class to handle <code>Next</code> actions.
 */
protected class NextAction extends JCAction implements java.io.Serializable {
    public NextAction() {
        super(JCLocaleManager.ACTION_NEXT);
    }

    public void actionPerformed(ActionEvent e) {}
}

/**
 * Class to handle <code>Previous</code> actions.
 */
protected class PreviousAction extends JCAction implements java.io.Serializable {
    public PreviousAction() {
        super(JCLocaleManager.ACTION_PREVIOUS);
    }

    public void actionPerformed(ActionEvent e) {}
}

/**
 * Class to handle <code>Cancel</code> actions.
 */
protected class CancelAction extends JCAction implements java.io.Serializable {
    public CancelAction()
    {
        super(JCLocaleManager.ACTION_CANCEL);
    }

    public void actionPerformed(ActionEvent e) {}
}

/**
 * Class to handle <code>Help</code> actions.
 */
protected class HelpAction extends JCAction implements java.io.Serializable {
    public HelpAction()
    {
        super(JCLocaleManager.ACTION_HELP);
    }

    public void actionPerformed(ActionEvent e) {}
}






}
