/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: DateTimeChooser.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:27 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.swing;

import com.klg.jclass.util.calendar.DateTimeEditor;
import com.klg.jclass.util.calendar.JCDateChooser;

import java.util.Calendar;
import java.util.Locale;

/**
 *  Extends DateTimeEditor to add a popup calendar editor.
 */
public class DateTimeChooser extends DateTimeEditor {

/** The date/time popup which uses this component to edit. */
protected JCPopupCalendarEditor  popupEditor = null;

/**
 * Create a DateTimeChooser with default values and no associated popupEditor.
 */
public DateTimeChooser()
{
    this(null, JCDateChooser.DUAL_SPIN, true, false, null, null,
         null, null, null);
}

/**
 * Create a DateTimeChooser with default values.
 * @param popupEditor The date/time popup which uses this component to edit.
 */
public DateTimeChooser(JCPopupCalendarEditor popupEditor)
{
    this(popupEditor, JCDateChooser.DUAL_SPIN, true, false, null, null,
         null, null, null);
}

/**
 * Create a DateTimeChooser that specifies the type of chooser, whether or
 * not to show a time component, the locale, and an intial date/time.
 * @param popupEditor The date/time popup which uses this component to edit.
 * @param chooserType The type of date chooser.  Must be one of
 * JCDateChooser.QUICK_SELECT, JCDateChooser.DUAL_SPIN,
 * JCDateChooser.SPIN_POPDOWN, or JCDateChooser.READ_ONLY.  Note that
 * specifying JCDateChooser.SPIN_POPDOWN will cause JCDateChooser.DUAL_SPIN to
 * be used because we cannot have a popdown within a popdown.
 * @param showTimeComponent Determines whether the time component is shown or
 * not.
 * @param showApplyButton If true, an apply button is shown.
 * @param l The locale to use.
 * @param c The intial date and time.
 */
public DateTimeChooser(JCPopupCalendarEditor popupEditor, int chooserType,
                       boolean showTimeComponent, boolean showApplyButton,
                       Locale l, Calendar c)
{
    this(popupEditor, chooserType, showTimeComponent, showApplyButton, l, c,
         null, null, null);
}

/**
 * Create a DateTimeChooser that specifies the type of chooser, whether or
 * not to show a time component, the locale, an intial date/time, and gives a
 * list of month, shortMonth, and day strings to use for the calendar.
 * @param popupEditor The date/time popup which uses this component to edit.
 * @param chooserType The type of date chooser.  Must be one of
 * JCDateChooser.QUICK_SELECT, JCDateChooser.DUAL_SPIN,
 * JCDateChooser.SPIN_POPDOWN, or JCDateChooser.READ_ONLY.  Note that
 * specifying JCDateChooser.SPIN_POPDOWN will cause JCDateChooser.DUAL_SPIN to
 * be used because we cannot have a popdown within a popdown.
 * @param showTimeComponent Determines whether the time component is shown or
 * not.
 * @param showApplyButton If true, an apply button is shown.
 * @param l The locale to use
 * @param c The intial date and time.
 * @param months The month strings to use for the calendar.  Note that this
 * array must be the correct size for the locale (or longer).
 * @param shortMonths The shortMonth strings to use for the calendar.  Note that
 * this array must be the correct size for the locale (or longer).
 * @param days The day strings to use for the calendar.  Note that this
 * array must be the correct size for the locale (or longer).
 */
public DateTimeChooser(JCPopupCalendarEditor popupEditor, int chooserType,
                       boolean showTimeComponent, boolean showApplyButton,
                       Locale l, Calendar c, String[] months,
                       String[] shortMonths, String[] days)
{
    super(chooserType, showTimeComponent, showApplyButton, l, c,
          months, shortMonths, days);
    this.popupEditor = popupEditor;
}

/**
 * Fire a popup event to all listeners of the popup editor.
 */
protected void firePopupEvent()
{
    if (popupEditor != null) {
        popupEditor.firePopupEvent();
    }
}

}
