/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: DateTimePopup.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:27 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.swing;

import com.klg.jclass.util.JCListenerList;
import com.klg.jclass.util.calendar.JCDateChooser;
import com.klg.jclass.util.calendar.JCCalendar;
import com.klg.jclass.util.swing.JCPopupListener;
import com.klg.jclass.util.swing.JCPopupEvent;
import com.klg.jclass.util.value.CalendarValueModel;
import com.klg.jclass.util.value.JCValueEvent;
import com.klg.jclass.util.value.JCValueListener;
import com.klg.jclass.util.value.JCValueModel;

import javax.swing.*;
import java.awt.*;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.Enumeration;

/**
 * Class that functions as the popup editor for JCPopupCalendar.
 */
public class DateTimePopup implements JCPopupCalendarEditor,
                                      JCValueListener
{

/** The popup's parent.  Normally a JCPopupCalendar. */
protected Component        parent = null;
/** A date/time chooser component to do the editing. */
protected DateTimeChooser  chooser = null;
/** A list of listeners to tell about value changes. */
protected JCListenerList   popupListeners = null;
/** Value model used to store the editor's value. */
protected JCValueModel     model = null;

/**
 * A no arguments constructor.  Create a DateTimePopup with default values.
 */
public DateTimePopup()
{
    this(null, JCDateChooser.DUAL_SPIN, true, false, null, null,
         null, null, null);
}

/**
 * A constructor with a given parent. For all other values, the DateTimePopup
 * will be created with default values.
 * @param parent The popup's parent.
 */
public DateTimePopup(Component parent)
{
    this(parent, JCDateChooser.DUAL_SPIN, true, false, null, null,
         null, null, null);
}

/**
 * Create a DateTimePopup with a given date/time but otherwise with default
 * values.
 * @param parent The popup's parent.
 * @param c The date/time stored in a Calendar object
*/
public DateTimePopup(Component parent, Calendar c)
{
    this(parent, JCDateChooser.DUAL_SPIN, true, false, null, c,
         null, null, null);
}

/**
 * Create a DateTimePopup of the given chooserType.  Also specify whether a time
 * component should be shown, the locale, and the date/time to use.
 * @param parent The popup's parent
 * @param chooserType The type of date chooser.  Must be one of
 * JCDateChooser.QUICK_SELECT, JCDateChooser.DUAL_SPIN,
 * JCDateChooser.SPIN_POPDOWN, or JCDateChooser.READ_ONLY.  Note that
 * specifying JCDateChooser.SPIN_POPDOWN will cause JCDateChooser.DUAL_SPIN to
 * be used because we cannot have a popdown within a popdown.
 * @param showTimeComponent Determines whether the time component is shown or
 * not.
 * @param showApplyButton Determines whether the apply button is shown or not.
 * @param l The locale to use
 * @param c The date/time stored in a Calendar object
 */
public DateTimePopup(Component parent, int chooserType,
                     boolean showTimeComponent, boolean showApplyButton,
                     Locale l, Calendar c)
{
    this(parent, chooserType, showTimeComponent, showApplyButton, l, c,
         null, null, null);
}

/**
 * Create a DateTimePopup of the given chooserType.  Also specify whether a time
 * component should be shown, the locale, the date/time to use, and a list of
 * month, shortMonth, and day strings to use for the calendar.
 * @param parent The popup's parent
 * @param chooserType The type of date chooser.  Must be one of
 * JCDateChooser.QUICK_SELECT, JCDateChooser.DUAL_SPIN,
 * JCDateChooser.SPIN_POPDOWN, or JCDateChooser.READ_ONLY.  Note that
 * specifying JCDateChooser.SPIN_POPDOWN will cause JCDateChooser.DUAL_SPIN to
 * be used because we cannot have a popdown within a popdown.
 * @param showTimeComponent Determines whether the time component is shown or
 * not.
 * @param showApplyButton Determines whether the apply button is shown or not.
 * @param l The locale to use
 * @param c The date/time stored in a Calendar object
 * @param months The month strings to use for the calendar.  Note that this
 * array must be the correct size for the locale (or longer).
 * @param shortMonths The shortMonth strings to use for the calendar.  Note that
 * this array must be the correct size for the locale (or longer).
 * @param days The day strings to use for the calendar.  Note that this
 * array must be the correct size for the locale (or longer).
 */
public DateTimePopup(Component parent, int chooserType,
                     boolean showTimeComponent, boolean showApplyButton,
                     Locale l, Calendar c, String[] months,
                     String[] shortMonths, String[] days)
{
    this.parent = parent;
    chooser = new DateTimeChooser(this, chooserType, showTimeComponent,
                                  showApplyButton, l, c, months,
                                  shortMonths, days);
    model = new CalendarValueModel((Calendar)chooser.getValue());
    model.addValueListener(this);
}

/**
 * Returns the DateTimeChooser component that does the date/time editing.
 * @return The component which does the date/time editing for this popup.
 */
public DateTimeChooser getDateTimeChooser()
{
    return(chooser);
}

//
// Implement CalendarPopupEditor interface
//

/**
 * Returns the component in the popup that does the date/time editing.  Note
 * this is the same as getDateTimeChooser() but this method is needed for the
 * CalendarPopupEditor interface.
 * @return The component which does the date/time editing for this popup.
 * @see #getDateTimeChooser
 */
public JComponent getEditorComponent()
{
    return(getDateTimeChooser());
}

/**
 * Add a JCPopupListener to the popup to inform when the value in the
 * popup component has been committed.
 * @param l The popup listener to add
 */
public void addPopupListener(JCPopupListener l)
{
    popupListeners = JCListenerList.add(popupListeners, l);
}

/**
 * Removes a JCPopupListener.
 * @param l The popup listener to remove
 */
public void removePopupListener(JCPopupListener l)
{
    popupListeners = JCListenerList.remove(popupListeners, l);
}

/**
 * Fire a popup event to all listeners.
 */
public void firePopupEvent()
{
    Object value = chooser.getDateChooser().getValue().clone();
    JCPopupEvent popupEvent = new JCPopupEvent(this, value);
    Enumeration e = JCListenerList.elements(popupListeners);
    while (e.hasMoreElements()) {
        JCPopupListener listener = (JCPopupListener)e.nextElement();
        listener.commit(popupEvent);
    }
}

/**
 * Sets the value in the popup editor.
 * @param value The value to set on the popup editor.   This value can be of
 * type Calendar or Date.
 */
public void setValue(Object value)
{
    Calendar val = JCCalendar.convertObjectToCalendar(value,
                                                      chooser.getLocale());
    if (model != null) {
        model.removeValueListener(this);
    }
    model = new CalendarValueModel(val);
    model.addValueListener(this);
    chooser.setCalendarModel(model);
}

/**
 * Sets the minimum date. The calendar cannot change to before this date.
 * @param minDate The new minimum date.  This value can be of type Calendar or
 * Date.
 */
public void setMinimumDate(Object minDate)
{
    Calendar min = JCCalendar.convertObjectToCalendar(minDate,
                                                      chooser.getLocale());
    chooser.setMinimumDate(min);
}

/**
 * Sets the maximum date. The calendar cannot change to after this date.
 * @param maxDate The new maximum date.  This value can be of type Calendar or
 * Date.
 */
public void setMaximumDate(Object maxDate)
{
    Calendar max = JCCalendar.convertObjectToCalendar(maxDate,
                                                      chooser.getLocale());
    chooser.setMaximumDate(max);
}

/**
 * Sets the locale being used by the popup. In turn, this object sets the
 * locale on the DateTimeChooser, which will set it on its child components.
 * @param locale The new locale to be used
 */
public void setLocale(Locale locale)
{
    if (chooser != null) {
        chooser.setLocale(locale);
    }
}

/**
 * Sets the <code>Font</code> of the popup.  This font gets propagated to the
 * child components
 *
 * @param f the new <code>Font</code> for the popup
 */
public void setFont(Font f)
{
    if (chooser != null) {
        chooser.setFont(f);
        chooser.invalidate();
    }
}

/**
 * Sets whether the time component should appear or not.
 * @param showTimeComponent If true, the time component will appear
 */
public void setShowTimeComponent(boolean showTimeComponent)
{
    if (chooser != null) {
        chooser.setShowTimeComponent(showTimeComponent);
    }
}

/**
 * Returns whether the time component should appear or not.
 * @return true, if the time component should appear
 */
public boolean getShowTimeComponent()
{
    if (chooser != null) {
        return(chooser.getShowTimeComponent());
    }
    return(true);
}

/**
 * Set whether an apply button be shown in the popup.  If an apply button is
 * shown, clicking on the button applies the value to the model and hides the
 * popup.
 * @param showApplyButton If true, an apply button is shown in the popup.
 */
public void setShowApplyButton(boolean showApplyButton)
{
    if (chooser != null) {
        chooser.setShowApplyButton(showApplyButton);
    }
}

/**
 * Get whether an apply button be shown in the popup.  If an apply button is
 * shown, clicking on the button applies the value to the model and hides the
 * popup.
 * @return true, if the apply button will appear.
 */
public boolean getShowApplyButton()
{
    if (chooser != null) {
        return(chooser.getShowApplyButton());
    }
    return(false);
}

/**
 * Set whether the popup will popdown when the user selects a day on the
 * day table.
 * @param hidePopupOnDayTableClick If true, the popup will popdown when the
 * user selects a day on the day table.
 */
public void setHidePopupOnDayTableClick(boolean hidePopupOnDayTableClick)
{
    if (chooser != null) {
        chooser.setHidePopupOnDayTableClick(hidePopupOnDayTableClick);
    }
}

/**
 * Get whether the popup will popdown when the user selects a day on the
 * day table.
 * @return true, if the popup will popdown when the user selects a day on the
 * day table.
 */
public boolean getHidePopupOnDayTableClick()
{
    if (chooser != null) {
        return(chooser.getHidePopupOnDayTableClick());
    }
    return(false);
}

//
// JCValueListener Implementation
//

/**
 * The value is changing.  A listener may further change the
 * <code>new_value</code> or abort the change altogether.  In the case of an
 * abort (initiatied by calling <code>event.setAllowChange(false)</code>), the
 * <code>valueChanged</code> method will never be called.
 * @param event The event generated by the value changing
 */
public void valueChanging(JCValueEvent event) {}

/**
 * The method informs a listener that the value has been changed.
 * @param event The event generated by the value changed
 */
public void valueChanged(JCValueEvent event)
{
    if (parent instanceof JCPopupCalendar) {
        JCPopupCalendar popupCalendar = (JCPopupCalendar)parent;
        Calendar c = (Calendar)model.getValue();
        Date d = (Date)JCCalendar.convertCalendarToObject(c, Date.class);
        popupCalendar.updateComboBoxModel(d);
        popupCalendar.updateComboBoxEditor(d);
        popupCalendar.updateComboBoxRenderer();
    }
}

}
