/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCDefaultListSelectionModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:28 $  $Locker:  $  Quest Software Inc.
package com.klg.jclass.swing;

import javax.swing.*;

/**
 * Extend the DefaultListSelectionModel to fix a bug in Swing which forgets to
 * update the anchor and lead when rows are added or deleted.
 */
public class JCDefaultListSelectionModel extends DefaultListSelectionModel
{

/**
 * Insert length indices beginning before/after index.  This is typically
 * called to sync the selection model with a corresponding change
 * in the data model.
 */
public void insertIndexInterval(int index, int length, boolean before)
{
    super.insertIndexInterval(index, length, before);

    int anchor   = super.getAnchorSelectionIndex();
    int lead     = super.getLeadSelectionIndex();
    int min      = Math.min(anchor, lead);
    int max      = Math.max(anchor, lead);
    int minIndex = (before) ? index : index + 1;

    if (length <= 0 || max < minIndex) {
        // Zero range or index interval inserted beyond anchor & lead interval
        return;
    }

    if (minIndex <= min) {
        // Index interval inserted below anchor & lead interval
        anchor += length;
        lead   += length;
    }
    else {
        // Index interval inside anchor & lead interval.
        if (anchor < index) {
            if (lead >= index) {
                lead += length;
            }
        }
        else {
            anchor += length;
        }
    }
    super.setAnchorSelectionIndex(anchor);
    super.setLeadSelectionIndex(lead);

}

/**
 * Remove the indices in the interval index0,index1 (inclusive) from
 * the selection model.  This is typically called to sync the selection
 * model width a corresponding change in the data model.
 */
public void removeIndexInterval(int index0, int index1)
{
    super.removeIndexInterval(index0, index1);

    int anchor   = super.getAnchorSelectionIndex();
    int lead     = super.getLeadSelectionIndex();
    int min      = Math.min(anchor, lead);
    int max      = Math.max(anchor, lead);
    int minIndex = Math.min(index0, index1);
    int maxIndex = Math.max(index0, index1);
    int length   = maxIndex - minIndex + 1;

    if (length <= 0 || max < minIndex) {
        // Zero range or Index range beyond anchor & lead interval
        return;
    }

    if (maxIndex < min) {
        // Index interval below anchor & lead interval
        anchor -= length;
        lead   -= length;
    }
    else if (min >= minIndex) {
        if (max <= maxIndex) {
            // Anchor & lead interval entirely with index interval.
            anchor = -1;
            lead   = -1;
        }
        else {
            // Max outside index interval.  Min goes to minIndex;
            if (max == anchor) {
                anchor -= length;
                lead    = minIndex;
            }
            else {
                anchor  = minIndex;
                lead   -= length;
            }
        }
    }
    else {  // min < minIndex.  Min stays the same.
        if (max <= maxIndex) {
            // Max inside index interval
            if (max == anchor) {
                anchor = minIndex - 1;
            }
            else {
                lead = minIndex - 1;
            }
        }
        else {
            if (max == anchor) {
                anchor -= length;
            }
            else {
                lead -= length;
            }
        }
    }
    super.setAnchorSelectionIndex(anchor);
    super.setLeadSelectionIndex(lead);
}

}

