/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCMDIFrame.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:28 $  $Locker:  $  Quest Software Inc.
package com.klg.jclass.swing;

import com.klg.jclass.swing.util.ShrinkWrapLayout;

import javax.swing.*;
import java.awt.*;

/**
 * Extends Swing's basic internal frame class to make it possible to associate a
 * toolbar with an internal frame, as well as with the menu bar that is
 * supported by the basic <code>JInternalFrame</code>.
 */
public class JCMDIFrame extends JInternalFrame {
//Fix for PR10703
/** The menubar associated with this frame. */
protected JMenuBar mDIMenubar = null;
/** The toolbar associated with this frame. */
protected JToolBar mDIToolbar = null;
//end Fix
/** Substitute for content pane to make maximization easier. */
protected Container container = null;

public JCMDIFrame()
{
    super();
    // We call the super of getContentPane since we override it below
    // to return the container we are just creating here
    super.getContentPane().setLayout(new ShrinkWrapLayout());
    super.getContentPane().add(container = new JPanel(new ShrinkWrapLayout()));
}

public JCMDIFrame(String title)
{
    super(title);
    // We call the super of getContentPane since we override it below
    // to return the container we are just creating here
    super.getContentPane().setLayout(new ShrinkWrapLayout());
    super.getContentPane().add(container = new JPanel(new ShrinkWrapLayout()));
}

public JCMDIFrame(String title, boolean resizable)
{
    super(title, resizable);
    // We call the super of getContentPane since we override it below
    // to return the container we are just creating here
    super.getContentPane().setLayout(new ShrinkWrapLayout());
    super.getContentPane().add(container = new JPanel(new ShrinkWrapLayout()));
}

public JCMDIFrame(String title, boolean resizable, boolean closable)
{
    super(title, resizable, closable);
    // We call the super of getContentPane since we override it below
    // to return the container we are just creating here
    super.getContentPane().setLayout(new ShrinkWrapLayout());
    super.getContentPane().add(container = new JPanel(new ShrinkWrapLayout()));
}

public JCMDIFrame(String title, boolean resizable,
                  boolean closable, boolean maximizable)
{
    super(title, resizable, closable, maximizable);
    // We call the super of getContentPane since we override it below
    // to return the container we are just creating here
    super.getContentPane().setLayout(new ShrinkWrapLayout());
    super.getContentPane().add(container = new JPanel(new ShrinkWrapLayout()));
}

public JCMDIFrame(String title, boolean resizable,
                  boolean closable, boolean maximizable, boolean iconifiable)
{
    super(title, resizable, closable, maximizable, iconifiable);
    // We call the super of getContentPane since we override it below
    // to return the container we are just creating here
    super.getContentPane().setLayout(new ShrinkWrapLayout());
    super.getContentPane().add(container = new JPanel(new ShrinkWrapLayout()));
}

/**
 * Provided for Beans property editors - has no effect.
 * @param s
 */
public void setAbout(String s) {}

/**
 * Returns vendor information.
 * @return vendor information
 */
public String getAbout()
{
    return(JCVersion.getVersionString());
}

/**
 * Sets the menu bar associated with this frame.<p> If the parent of this frame
 * is a <code>JCMDIPane</code>, then when this frame becomes active, its menu
 * bar will become the containing frame's menu bar.
 * @see #getMDIMenuBar
 * @see #setMDIToolBar
 * @see #getMDIToolBar
 * @param menubar
 */
public void setMDIMenuBar(JMenuBar menubar)
{
    //fix for PR10703
    this.mDIMenubar = menubar;
}

/**
 * Gets the menu bar associated with this frame.<p>  If the parent of this
 * frame is a <code>JCMDIPane</code>, then when this frame becomes active, its
 * menu bar will become the containing frame's menu bar.
 * @see #setMDIMenuBar
 * @see #setMDIToolBar
 * @see #getMDIToolBar
 * @return
 */
public JMenuBar getMDIMenuBar()
{
    //fix for PR10703
    return(mDIMenubar);
}

/**
 * Sets the toolbar associated with this frame.<p>
 * If the parent of this frame is a <code>JCMDIPane</code>, then when this
 * frame becomes active, its toolbar will become the containing frame's.
 * @see #getMDIToolBar
 * @see #setMDIMenuBar
 * @see #getMDIMenuBar
 * @param toolbar
 */
public void setMDIToolBar(JToolBar toolbar)
{
    //fix for PR10703
    this.mDIToolbar = toolbar;
}

/**
 * Sets the toolbar associated with this frame.<p>
 * If the parent of this frame is a <code>JCMDIPane</code>, then when this
 * frame becomes active, its toolbar will become the containing frame's.
 * @see #setMDIToolBar
 * @see #setMDIMenuBar
 * @see #getMDIMenuBar
 * @return
 */
public JToolBar getMDIToolBar()
{
    //fix for PR10703
    return(mDIToolbar);
}

/**
 * Overrides <code>getContentPane</code> to provide a container one level
 * removed; this allows the frame to be <i>maximized</i> by reparenting its
 * children to a different parent.
 * @return
 */
public Container getContentPane()
{
    return(container);
}


/**
 * <i>Internal use only</i><p>.
 * Method used by <code>JCMDIPane</code> when unmaximizing.
 * @param container
 */
public void restoreContainer(Container container)
{
    if (container != this.container) {
        throw new IllegalArgumentException("Trying to restore the " +
                                           "wrong container");
    }
    super.getContentPane().add(container);
}




}
