/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCMDIPane.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:28 $ $Locker:  $ Quest Software Inc.

// Potential Future Features
//   Arranged icons should wrap at desktop pane width
//   Activation mgr to transfer activation and focus when one window closes

package com.klg.jclass.swing;

import com.klg.jclass.swing.util.*;

import com.klg.jclass.util.JCLocaleManager;
import com.klg.jclass.util.JCUtil;
import com.klg.jclass.util.JCEnvironment;
import com.klg.jclass.util.swing.JCAction;
import com.klg.jclass.util.swing.JCIconCreator;

import javax.swing.*;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ContainerEvent;
import java.awt.event.ContainerListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;
import java.beans.VetoableChangeListener;
import java.util.Vector;

/**
 * Replaces <code>JDesktopPane</code> as an internal frame container with the
 * following features:<p>
 * <ul>
 * <li>Removes the internal frame's header on maximization and moves controls
 *     into menu bar area.
 * <li>Installs "windows" menu on the parent menu bar, if one exists.
 * <li>Windows menu contains functionality to tile and cascade internal frames,
 *     as well as providing a selectable list of the existant internal frames.
 * <li>If the active frame is maximized, its title is appended to the containing
 *     frame's title.
 * <li>Wireframe "Drag" capability for fast resizing and window dragging.
 * <li>If the pane is in <code>JScrollPane</code>, then the pane will resize
 *     itself so that it is big enough to accommodate the internal frames as
 *     positioned.
 * <li> If the internal frame children are instances of <code>JCMDIFrame</code>,
 *     then it also manages toolbar and menu bar switching as frames become
 *     active and are opened and closed.
 * </ul>
 * <code>JCMDIPane</code> works best with a <code>JCMDIFrame</code>, but it can
 * be used with standard <code>JInternal</code> frames as long as there is only
 * one child of the <code>contentPane</code>. The child can be a container as
 * well, so that this is not a functional restriction.
 */
public class JCMDIPane extends JDesktopPane {

/** Constant for use with <code>setFrameManipulation(int mode)</code>. */
public static final int WIREFRAME = 1;
/** Constant for use with <code>setFrameManipulation(int mode)</code>. */
public static final int DEFAULT   = 2;

/** Constant for use with <code>setInitialLayout(int layout)</code>. */
public static final int TILE_HORIZONTAL = 1;
/** Constant for use with <code>setInitialLayout(int layout)</code>. */
public static final int TILE_VERTICAL   = 2;
/** Constant for use with <code>setInitialLayout(int layout)</code>. */
public static final int CASCADE         = 3;

protected static JCLocaleManager li = JCLocaleManager.getDefault();

protected int              initialLayout = 0;
protected JMenuBar         menubar = null;
protected JToolBar         toolbar = null;
protected Listener         listener = new Listener();
protected FrameListener    frameListener = new FrameListener();
protected ActionListener   menuListener = new WindowsMenuListener();
protected boolean          maximized = false;
protected LayoutButtonBox  layoutButtonBox = null;
protected String           savedTitle = null;
protected boolean          titleIsSaved = false;
protected JCAction         maxMinAction = null;
protected JMenuItem        maxMinItem = null;
protected JCAction         cascadeAction = null;
protected JCAction         tileHorizAction = null;
protected JCAction         tileVertAction = null;
protected JCAction         arrangeIconsAction = null;
protected int              frameManipulationStyle = WIREFRAME;
protected JToolBar         lastToolBarSetToRoot = null;
protected Icon             selectedIcon;
protected Icon             nonSelectedIcon;
protected JInternalFrame   activeFrame = null;
protected JPanel           maximizedPanel = new JPanel(new ShrinkWrapLayout());
protected JInternalFrame   maximizedFrame = null;
protected JInternalFrame   maximizingFrame = null;
protected boolean          considerIconsWhenTiling = false;

/**
 * We track the current menu bar because we cannot rely on Swing to actually
 * change it when requested; therefore, we cache the menu bar that we believe
 * we are working with.
 */
protected JMenuBar currentMenuBar = null;

/**
 * This is the frame that is being activated.  We keep track of it so that
 * we know it is not really being deleted.  This is done because of the way
 * <code>JInternalFrames</code> are made active; they are removed and then
 * added back at the top of the drawing order.
 */
protected JInternalFrame activatingFrame = null;

private static final String selectedIconPixels[] = {
	"             ",
	"             ",
    "         bbb ",
    "        bbb  ",
    "       bbb   ",
    "bbbb  bbb    ",
    " bbb bbb     ",
    "  bbbbb      ",
    "   bbb       ",
    "    b        ",
 	"             ",
  	"             ",
    "             ",
};

private static final String nonSelectedIconPixels[] = {
    "             ",
    "             ",
    "             ",
    "             ",
    "             ",
    "             ",
    "             ",
    "             ",
    "             ",
    "             ",
    "             ",
    "             ",
    "             ",
};

public JCMDIPane()
{
    setMinimumSize(new Dimension(300, 200));
    this.addContainerListener(listener);

    // Set the default icons
    setSelectedIcon(makeImage(selectedIconPixels));
    setNonSelectedIcon(makeImage(nonSelectedIconPixels));
};

/**
 * Provided for Beans property editors - has no effect.
 * @param s
 */
public void setAbout(String s) {}

/**
 * Returns vendor information.
 * @return vendor information
 */
public String getAbout()
{
    return(JCVersion.getVersionString());
}

/**
 * See <code>setFrameManipulationStyle</code>.
 * @see #setFrameManipulationStyle
 * @return The current frame manipulation style
 */
public int getFrameManipulationStyle()
{
    return(frameManipulationStyle);
}

/**
 * Sets the frame manipulation style.  Valid styles are:
 * <ul><li><code>WIREFRAME</code> - drag and resize as a wireframe (DEFAULT)
 * <li><code>DEFAULT</code> - default style specified by the PLAF you are
 * using</ul>
 * @param style The new frame manipulation style
 */
public void setFrameManipulationStyle(int style)
{
    if (style < WIREFRAME || style > DEFAULT) {
        throw new IllegalArgumentException("invalid parameter: " + style);
    }

    if (style == frameManipulationStyle) {
        return;
    }
    frameManipulationStyle = style;
}

/**
 * Sets the drag style of the JCMDIPane.  Note the dragging style is overridden
 * by JCMDIPane's frame manipulation style mechanism.  Setting the dragging
 * style to <code>LIVE_DRAG_MODE</code> is equivalent to setting the
 * frameManipulation style to <code>DEFAULT</code>.  Setting the dragging
 * style to <code>OUTLINE_DRAG_MODE</code> is equivalent to setting the
 * frameManipulation style to <code>WIREFRAME</code>.
 * @param dragMode The new dragging style
 * @see #setFrameManipulationStyle
 */
public void setDragMode(int dragMode)
{
    if (JCEnvironment.getJavaVersion() >= 130) {
        super.setDragMode(dragMode);
    }
    if (dragMode == LIVE_DRAG_MODE) {
        setFrameManipulationStyle(DEFAULT);
    }
    else {
        setFrameManipulationStyle(WIREFRAME);
    }
}

/**
 * Gets the drag style of the JCMDIPane.  Note the dragging style is overridden
 * by JCMDIPane's frame manipulation style mechanism.  Setting the dragging
 * style to <code>LIVE_DRAG_MODE</code> is equivalent to setting the
 * frameManipulation style to <code>DEFAULT</code>.  Setting the dragging
 * style to <code>OUTLINE_DRAG_MODE</code> is equivalent to setting the
 * frameManipulation style to <code>WIREFRAME</code>.
 * @return The current dragging style
 * @see #setFrameManipulationStyle
 */
public int getDragMode()
{
    if (JCEnvironment.getJavaVersion() >= 130) {
        return(super.getDragMode());
    }
    else {
        if (frameManipulationStyle == DEFAULT) {
            return(LIVE_DRAG_MODE);
        }
        else {
            return(OUTLINE_DRAG_MODE);
        }
    }
}

/**
 * Set whether icons should be considered when tiling.  If true, tiling happens
 * above (and not over) the icon display.  If false, the frames will be tiled
 * over the icons.  The default is false.
 * @return Whether icons should be considered when tiling
 */
public boolean isConsiderIconsWhenTiling()
{
    return(considerIconsWhenTiling);
}

/**
 * Get whether icons should be considered when tiling.  If true, tiling happens
 * above (and not over) the icon display.  If false, the frames will be tiled
 * over the icons.  The default is false.
 * @param considerIconsWhenTiling The new value of the considerIconsWhenTiling
 * property.
 */
public void setConsiderIconsWhenTiling(boolean considerIconsWhenTiling)
{
    this.considerIconsWhenTiling = considerIconsWhenTiling;
}

/**
 * Returns the toolbar used if there are no internal frames.
 * @return toolbar used if there are no internal frames
 */
public JToolBar getMDIToolBar()
{
    return(toolbar);
}

/**
 * Method used to rationalize the Menu and Toolbar settings. Chiefly
 * called from places such as <code>addNotify</code> when the state of the menu
 * and toolbars may not be accurate due to the previously unrealized state
 * of the components.
 */
protected void checkIfMenuOrToolBarsNeeded()
{
    
    JToolBar toolbar = null;
    JMenuBar menubar = null;

    if (isDisplayable()) {
        Component[] children = getComponents();
        // Loop through children until we find a JCMDIFrame or icon that
        // with a non-null menubar or toolbar
        for (int i = 0; i < children.length; i++) {
            JCMDIFrame frame = null;
            if (children[i] instanceof JCMDIFrame) {
                frame = (JCMDIFrame)children[i];
            }
            else if (children[i] instanceof JInternalFrame.JDesktopIcon) {
              JInternalFrame.JDesktopIcon icon =
                    (JInternalFrame.JDesktopIcon)children[i];
                JInternalFrame iFrame = icon.getInternalFrame();
                if (iFrame instanceof JCMDIFrame) {
                    frame = (JCMDIFrame)iFrame;
                }
            }
            if (frame != null && frame.isSelected()) {
                // We found an internal frame so we exit
                menubar = frame.getMDIMenuBar();
                toolbar = frame.getMDIToolBar();
                break;
            }
        }

        // If no selected frame found, or the selected frame has no menubar
        // or toolbar, then use the default ones.
        if (menubar == null) {
            menubar = this.menubar;
        }
        if (toolbar == null) {
            toolbar = this.toolbar;
        }
        setMenuAndToolBarsToRootPane(null, menubar, toolbar);
    }
    
}

/**
 * Returns the toolbar used for the specified frame, if the frame is
 * a <code>JCMDIFrame</code> with a non-null <code>MDIToolBar</code>. Otherwise,
 * the <code>MDIToolBar</code> for the this pane is returned.
 * @param frame The frame from which to get the <code>MDIToolBar</code>
 * @return toolbar used for the specified frame
 */
public JToolBar getMDIToolBar(JInternalFrame frame)
{
    if (frame != null && frame instanceof JCMDIFrame) {
        JToolBar tb = ((JCMDIFrame)frame).getMDIToolBar();
        if (tb != null) {
            return(tb);
        }
    }
    return(toolbar);
}

/**
 * Sets the toolbar to use if there are no internal frames.
 * @param toolbar The new toolbar
 */
public void setMDIToolBar(JToolBar toolbar)
{
    this.toolbar = toolbar;
    checkIfMenuOrToolBarsNeeded();
}

/**
 * Returns the menubar used if there are no internal frames.
 * @return menubar used
 */
public JMenuBar getMDIMenuBar()
{
    return(menubar);
}

/**
 * Returns the menubar used for the specified frame, if the frame is
 * a <code>JCMDIFrame</code> with a non-null <code>MDIMenuBar</code>. Otherwise,
 * the <code>MDIMenuBar</code> for the this pane is returned.
 * @return menubar used for the specified frame
 * @param frame  The frame from which to get the <code>MDIMenuBar</code>
 */
public JMenuBar getMDIMenuBar(JInternalFrame frame)
{
    if (frame != null && frame instanceof JCMDIFrame) {
        JMenuBar mb = ((JCMDIFrame)frame).getMDIMenuBar();
        if (mb != null) {
            return(mb);
        }
    }
    return(menubar);
}

/**
 * Sets the menubar to use if there are no internal frames.
 * @param menubar The new menubar
 */
public void setMDIMenuBar(JMenuBar menubar)
{
    this.menubar = menubar;
    checkIfMenuOrToolBarsNeeded();
}

/**
 * Override that ensures the layout cannot be modified.
 */
public void setLayout() {}

public void addNotify()
{

 

    
    // If we already have a child then attempt to activate it
    selectNextFrame();
    super.addNotify();

    // See if the master toolbar of menubar needs swapping
    checkIfMenuOrToolBarsNeeded();

    // Update the windows menu in the master toolbar
    updateWindowsMenu();
    
}

/**
 * Returns the topmost frame.
 * @return The topmost frame
 */
public JInternalFrame getTopFrame()
{
    JInternalFrame[] list = getAllFramesInLayer(DEFAULT_LAYER.intValue());
    return(list.length > 0 ? list[0] : null);
}

/**
 * If this pane has an ancestor that is a scrollpane or it has no children,
 * then it returns the default preferred size. If it has children and no
 * scrollpane for an ancestor, then it returns a size big enough to show
 * all its children in their current locations.
 * @return
 */
public Dimension getPreferredSize()
{
    JScrollPane scrollpane =
        (JScrollPane)SwingUtilities.getAncestorOfClass(JScrollPane.class, this);

    int componentCount = getComponentCount();
    if (scrollpane == null || componentCount == 0) {
        return(getMinimumSize());
    }
    Rectangle r = new Rectangle();
    for (int i = 0; i < componentCount; i++) {
        r = r.union(getComponent(i).getBounds());
    }
    
    return(r.getSize());
}

public Dimension getMaximumSize()
{
    
    return(new Dimension(100, 100));
}

protected void updateWindowsMenu()
{
    
    JMenu windows_menu = getWindowsMenuFromMenuBar();
    if (windows_menu != null) {
        buildWindowsMenu(windows_menu);
    }
}

protected JMenuBar getJMenuBar()
{
    if (currentMenuBar != null) {
        // We have taken control and don't care what actually is there anymore
        // although we hope it matches this cached member here
        return(currentMenuBar);
    }
    JMenuBar menubar = null;
    
    // get frame specific menu bar or if null

    JRootPane p = getRootPane();
    if (p == null) {
        
        return(null);
    }
    menubar = p.getJMenuBar();

    currentMenuBar = menubar;
    
    return(menubar);
}

public JMenu getWindowsMenuFromMenuBar()
{
    JMenuBar mb = getJMenuBar();
    if (mb == null) {
        return(null);
    }

    // Look for an existing JCMDIWindowMenu
    MenuElement items[] = mb.getSubElements();
    for (int i = 0; i < items.length; i++) {
        if (items[i] instanceof JCMDIWindowMenu) {
            // Found a JCMDIWindowMenu so we return it
            return((JMenu)items[i]);
        }
    }

    // We did not find a JCMDIWindowMenu so we create one
    JMenu wmenu = new JCMDIWindowMenu();
    // Add newly created window menu to the parent
    // FEATURE REQUEST: Look for a help menu and add it before the help menu
    mb.add(wmenu);
    return(wmenu);
}

protected void buildWindowsMenu(JMenu windows_menu)
{
    

    if (windows_menu.getMenuComponentCount() != 0) {
        // The windows menu has children so remove them all
        windows_menu.removeAll();
    }

    // Get window count.  If window count is 0 then we disable all options
    int window_count = getOpenInternalFrameCount();
    JMenuItem item;
    cascadeAction = new CascadeAction();

    item = cascadeAction.createMenuItem();
    windows_menu.add(item);

    if (window_count == 0) {
        item.setEnabled(false);
    }

    maxMinAction = new MaxMinAction();
    item = maxMinAction.createMenuItem();
    if (maximized) {
        item.setText(li.getString(li.ACTION_UNDOMAX));
    }
    else {
        item.setText(li.getString(li.ACTION_MAXIMIZE));
    }
    maxMinItem = item;
    windows_menu.add(item);

    // Disable if activeFrame is not maximizable
    JInternalFrame f =
        (activatingFrame == null) ? activeFrame : activatingFrame;
    if (window_count == 0 || (f != null && !f.isMaximizable())) {
        item.setEnabled(false);
    }

    tileHorizAction = new TileHAction();
    item = tileHorizAction.createMenuItem();
    windows_menu.add(item);
    if (window_count == 0) {
        item.setEnabled(false);
    }

    tileVertAction = new TileVAction();
    item = tileVertAction.createMenuItem();
    windows_menu.add(item);
    if (window_count == 0) {
        item.setEnabled(false);
    }

    arrangeIconsAction = new ArrangeAction();
    item = arrangeIconsAction.createMenuItem();
    windows_menu.add(item);
    if (getIconifiedFrameCount() == 0) {
        item.setEnabled(false);
    }

    windows_menu.add(new JSeparator());

    Component c[] = getComponents();
    int cnt = 0;
    for (int i = 0; i < c.length; i++) {
        JInternalFrame frame = null;
        if (c[i] instanceof JInternalFrame) {
            frame = (JInternalFrame)c[i];
        }
        else if (c[i] instanceof JInternalFrame.JDesktopIcon) {
            frame = ((JInternalFrame.JDesktopIcon)c[i]).getInternalFrame();
            if (frame.getParent() != null) {
                // Don't add the desktop pane version if the frame is visible
                // this fixes a problem where there is an overlap between when
                // the frame is restored and the icon has not been removed.
                frame = null;
            }
        }

        if (frame != null && frame.isVisible()) {
            
            // Build a menu item with the title of the frame
            // prepended by the window # with the Mnemonic set as the
            // window number
            char mnemonic = Integer.toString(i).charAt(0);
            WindowMenuItem mitem = new WindowMenuItem("" + cnt++ + " ",
                                                      mnemonic,
                                                      frame);

            // If this is the first, it is the selected, so add selected icon
            if (cnt == 1) {
                mitem.setIcon(getSelectedIcon());
            }
            // Otherwise add an empty place holder
            else {
                mitem.setIcon(getNonSelectedIcon());
            }

            mitem.addActionListener(menuListener);
            windows_menu.add(mitem);
        }
    }
}

protected void moveChildren(Container source, Container target)
{
    Component child;

    // Make a copy of the child list that we can go through since the
    // remove all will cause our array to be mushed
    if (source instanceof JCMDIFrame) {
        child = ((JCMDIFrame)source).getContentPane();
    }
    else {
        if (source instanceof RootPaneContainer) {
            source = ((RootPaneContainer)source).getContentPane();
        }

        if (source.getComponentCount() < 1) {
            // No children to move
            return;
        }
        child = source.getComponent(0);
    }


    if (target instanceof JCMDIFrame) {
        ((JCMDIFrame)target).restoreContainer((Container)child);
    }
    else {
        if (target instanceof RootPaneContainer) {
            target = ((RootPaneContainer)target).getContentPane();
        }
        target.add(child);
    }

    /*
    Object copy[] = new Object[source.getComponentCount()];
    System.arraycopy(source.getComponents(), 0,
                     copy, 0, copy.length);

    // Remove children from the source
    source.removeAll();

    // Add children to target
    for (int i = 0; i < copy.length; i++) {
        target.add((Component) copy[i]);
    }
    */
    // Make sure that the container re-laysout it's children
    target.invalidate();
    target.validate();
}

protected void maximizeFrame(JInternalFrame frame)
{
    
    if (maximizingFrame != null) {
        
        return;
    }
    activeFrame = frame;

    if (maximizedFrame == frame) {
        // Frame to maximize is already maximized
        
        return;
    }

    maximizingFrame = frame;

    // Unmaximize previous frame
    if (maximizedFrame != null) {
        
        try {
            // Turn off maximim property of frame
            maximizedFrame.setMaximum(false);
        }
        catch (PropertyVetoException pve) {
            
        }
    }

    // Set this frame to be selected so its menu bar will be used in the
    // parent frame.
    if (frame != null) {
        
        try {
            frame.setSelected(true);
        }
        catch (PropertyVetoException pe) {}
    }

    // Get the title bar of the containing frame
    Frame parentFrame = (Frame)SwingUtilities.getAncestorOfClass(Frame.class,
                                                                 this);
    if (parentFrame != null && titleIsSaved == false) {
        savedTitle = parentFrame.getTitle();
        titleIsSaved = true;
    }

    if (maximizedFrame != null && maximizedPanel.getComponentCount() != 0) {
        // Move children back to original frame
        
        moveChildren(maximizedPanel, maximizedFrame);
    }

    if (frame != null) {
        // Move the children from the specified frame to the maximized panel
        
        moveChildren(frame, maximizedPanel);

        if (maximizedPanel.getParent() == null) {
            Container parent = getParent();
            if (parent != null             &&
                parent.getParent() != null &&
                parent.getParent() instanceof JScrollPane)
            {
                JScrollPane p = (JScrollPane)parent.getParent();
                // Freeze the scrollbars so that they do not show any
                // frames that might be sticking out beyond the range of
                // the main frame and so that they do not obstruct the
                // maximized panel
                p.setHorizontalScrollBarPolicy(p.HORIZONTAL_SCROLLBAR_NEVER);
                p.setVerticalScrollBarPolicy(p.VERTICAL_SCROLLBAR_NEVER);
                p.revalidate();
            }
            // Maximized panel has not been added yet so we do so
            add(maximizedPanel);
        }

        // Make sure the maximized panel is at the top layer
        maximizedPanel.setBounds(0, 0, getVisibleSize().width,
                                 getVisibleSize().height);
        moveToFront(maximizedPanel);
    }
    else if (maximizedPanel.getParent() != null) {
        // Frame is null so we want to remove the maximized panel
        remove(maximizedPanel);
    }

    if (frame == null) {
        
    }
    else {
        
    }
    maximizedFrame = frame;

    if (titleIsSaved == true) {
        String title;
        if (savedTitle == null) {
            title = "";
        }
        else {
            title = savedTitle;
        }
        if (maximizedFrame != null) {
            title = title + " - " + maximizedFrame.getTitle();
        }
        parentFrame.setTitle(title);
    }

    checkIfMenuOrToolBarsNeeded();
    updateIconsInMenuBar(frame);
    if (maximizedFrame != null) {
        maximizedFrame.grabFocus();
    }
    maximizingFrame = null;
}

protected void unmaximizeFrame()
{
    if (maximizedFrame == null) {
        // No frame maximized
        return;
    }

    
    if (maximizedPanel.getParent() != null) {
        Container parent = getParent();
        if (parent != null             &&
            parent.getParent() != null &&
            parent.getParent() instanceof JScrollPane)
        {
            JScrollPane p = (JScrollPane)parent.getParent();
            // Free the scrollbars to do whatever they want
            p.setHorizontalScrollBarPolicy(p.HORIZONTAL_SCROLLBAR_AS_NEEDED);
            p.setVerticalScrollBarPolicy(p.VERTICAL_SCROLLBAR_AS_NEEDED);
            p.revalidate();
        }
        // Maximized panel needs removing
        remove(maximizedPanel);
    }
    // Move the children from the maximized panel to the specified frame
    moveChildren(maximizedPanel, maximizedFrame);
    if (maximizedFrame != null) {
        maximizedFrame.grabFocus();
    }
    maximizedFrame = null;
    // Get the title bar of the containing frame so that we can restore the
    // title
    Frame parent_frame =
        (Frame)SwingUtilities.getAncestorOfClass(Frame.class, this);
    if (parent_frame != null && titleIsSaved == true) {
        parent_frame.setTitle(savedTitle);
    }
}

/**
 * Method to unmaximize an MDI frame.
 */
public void unmaximize()
{
    maximized = false;
    if (maximizedFrame != null) {
        try {
            // Turn off maximum property of frame
            maximizedFrame.setMaximum(false);
        }
        catch (PropertyVetoException pve) {
            
        }
    }

    unmaximizeFrame();
    if (maxMinItem != null) {
        maxMinItem.setText(li.getString(li.ACTION_MAXIMIZE));
    }

    updateIconsInMenuBar(null);

    repaint();
}

protected void updateIconsInMenuBar(JInternalFrame frame)
{
    LayoutButtonBox layout_box = getLayoutButtonBox();
    if (frame == null || maximized == false) {
        // Remove the layout from its parents
        Container parent = layout_box.getParent();
        if (parent != null) {
            parent.remove(layout_box);
            parent.repaint();
        }
        return;
    }

    JMenuBar mb = getJMenuBar();
    if (mb == null) {
        // bail
        return;
    }

    layout_box.setMaximizedFrame(frame);
    if (layout_box.getParent() != mb) {
        // If the parent of the layout box is not the current menu bar then
        // switch parents
        mb.add(layout_box);
        mb.validate();
        mb.repaint();
    }
}

/**
 * Causes this container to lay out its components.
 */
public void doLayout()
{
    
    if (maximized) {
        maximizedPanel.setBounds(0, 0, getVisibleSize().width,
                                 getVisibleSize().height);
    }

    // Loop through and make up some bounds for kids that don't have them
    Component kids[] = getComponents();
    
    for (int i = 0; i < kids.length; i++) {
        
        if (kids[i] instanceof JInternalFrame) {
            //JInternalFrame frame = (JInternalFrame)kids[i];
            //Rectangle r = frame.getBounds();
            //if (r.x == 0 && r.y == 0 && r.width == 0 && r.height == 0) {
            // bounds are unset so we set them
            //cascadeFrame(frame);
            //}
        }
    }
    super.doLayout();
}

public void paint(Graphics g)
{
    super.paint(g);

    switch (initialLayout) {
    case TILE_HORIZONTAL:
        this.tileWindowsHorizontally();
        break;
    case TILE_VERTICAL:
        this.tileWindowsVertically();
        break;
    case CASCADE:
        this.cascadeWindows();
        break;
    default:
    }
    initialLayout = 0;
}

public void setMaximized(boolean m)
{
    if (maximized == m) {
        return;
    }

    if (m == true) {
        if (activeFrame != null) {
            maximize(activeFrame);
        }
        else {
            maximized = true;
        }
    }
    else {
        unmaximize();
    }
}

public boolean isMaximized()
{
    return(maximized);
}

/**
 * Method to maximize a particular frame in MDI style.
 * @param frame
 */
public void maximize(JInternalFrame frame)
{
    
    if (frame != null && frame.isMaximizable() && frame.isVisible()) {
        maximized = true;
        maximizeFrame(frame);
    }
}

/**
 * Allows the layout of the <code>MDIFrame</code> windows to be set before the
 * <code>MDIPane</code> window has been displayed. This has no effect after the
 * <code>MDIPane</code> has been displayed for the first time.
 * @param layout
 */
public void setInitialLayout(int layout)
{
    initialLayout = layout;
}

protected void setTitleBar()
{
    // Get the title bar of the containing frame
    Frame parentFrame =
        (Frame)SwingUtilities.getAncestorOfClass(Frame.class, this);

    if (parentFrame == null) {
        // We can't find a parent frame so we bail
        return;
    }

    String title;
    if (maximized == true) {
        // Append the title of the maximized window to what is currently there
        savedTitle = parentFrame.getTitle();
        if (savedTitle == null) {
            title = "";
        }
        else {
            title = savedTitle;
        }
        title = title + " - " + maximizedFrame.getTitle();
    }
    else {
        // Restore the title to the it's previous state (minus the
        // unmaximized tile)
        title = savedTitle;
    }

    parentFrame.setTitle(title);
}

/**
 * Returns the <code>LayoutButtonBox</code>.
 * @return the <code>LayoutButtonBox</code>
 */
protected LayoutButtonBox getLayoutButtonBox()
{
    if (layoutButtonBox == null) {
        // Create the layout box which manages and contains the icons
        // that we stick on the menu bar when the frame is maximized
        layoutButtonBox = new LayoutButtonBox(new LayoutListener());
    }
    return(layoutButtonBox);
}

/**
 * Returns all non-iconified, and non Palette layer
 * <code>JInternalFrames</code> currently displayed in the desktop.
 * @return all non-iconified, and non Palette layer
 * <code>JInternalFrames</code> currently displayed in the desktop
 */
public JInternalFrame[] getAllNonIconifiedAndNonPaletteFrames()
{
    Vector vResults = new Vector(getComponentCount());
    for (int i = 0; i < getComponentCount(); i++) {

        // Only interested in internal frames
        if (getComponent(i) instanceof JInternalFrame) {
            JInternalFrame frame = (JInternalFrame)getComponent(i);

            // Those that are visible
            if (!frame.isVisible()) {
                continue;
            }
            // And those that are not Palette layer
            if (frame.getLayer() != JLayeredPane.PALETTE_LAYER.intValue()) {
                vResults.addElement(frame);
            }
        }
    }

    JInternalFrame[] results = new JInternalFrame[vResults.size()];
    vResults.copyInto(results);
    return(results);
}

/**
 * Returns all iconified <code>JInternalFrames</code> currently displayed in the
 * desktop.
 * @return all iconified <code>JInternalFrames</code> currently displayed in the
 * desktop
 */
public JInternalFrame.JDesktopIcon[] getAllIconifiedFrames()
{
    Vector vResults = new Vector(getComponentCount());

    for (int i = 0; i < getComponentCount(); i++) {
        Component c = getComponent(i);
        if (c instanceof JInternalFrame.JDesktopIcon) {
            JInternalFrame.JDesktopIcon icon = (JInternalFrame.JDesktopIcon)c;
            JInternalFrame frame = icon.getInternalFrame();
            if (frame != null && frame.isVisible()) {
                vResults.addElement(c);
            }
        }
    }

    JInternalFrame.JDesktopIcon[] results =
        new JInternalFrame.JDesktopIcon[vResults.size()];
    vResults.copyInto(results);
    return(results);
}

protected int      cascadeX = -1;
protected int      cascadeY = -1;
protected int      cascadeXStart = 0;
protected int      cascadeYStart = 0;
protected int      cascadeXIncrement = 20;
protected int      cascadeYIncrement = 25;
protected int      cascadeStartIncrement = 100;
protected int      cascadeTolerance = 50;

private Dimension  cascadeSize = new Dimension(0, 0);
private int        fudge2 = 0; // 1
private int        fudge1 = 0; // 3

protected Dimension calcCascadeSize()
{
    cascadeSize.width  = (int)(0.8 * getSize().width);
    cascadeSize.height = (int)(0.8 * getSize().height);
    return(cascadeSize);
}

protected void cascadeReset()
{
    cascadeX = -1;
    cascadeY = -1;
    cascadeXStart = 0;
    cascadeYStart = 0;
}

protected void calcNewCascadePosition(Dimension frameSize)
{
    // If first time, set position to top left
    if (cascadeX < 0 || cascadeY < 0) {
        cascadeX = cascadeXStart;
        cascadeY = cascadeYStart;
        return;
    }
    int windowWidth  = getVisibleSize().width;
    int windowHeight = getVisibleSize().height;
    cascadeX += cascadeXIncrement;
    cascadeY += cascadeYIncrement;

    // Check if window fits.  If not adjust start position.
    if (cascadeY > windowHeight - cascadeTolerance ||
        cascadeX > windowWidth - cascadeTolerance)
    {
        cascadeXStart += cascadeStartIncrement;
        if (cascadeXStart > windowWidth - cascadeTolerance) {
            cascadeXStart = 0;
            cascadeYStart += cascadeStartIncrement;
            if (cascadeYStart > windowHeight - cascadeTolerance) {
                cascadeYStart = 0;
            }
        }
        cascadeX = cascadeXStart;
        cascadeY = cascadeYStart;
    }
}

protected void cascadeFrame(JInternalFrame frame)
{
    if (!isDisplayable()) {
        // Dont layout the frame if the window is not displayable
        return;
    }
    Dimension size = calcCascadeSize();
    
    calcNewCascadePosition(size);
    if (frame.isResizable()) {
        frame.setBounds(cascadeX, cascadeY, size.width, size.height);
    }
    else {
        frame.setLocation(cascadeX, cascadeY);
    }
}

protected void cascadeFrameByPreferredSize(JInternalFrame frame)
{
    Dimension size = frame.getPreferredSize();
    
    calcNewCascadePosition(size);
    if (frame.isResizable()) {
        frame.setBounds(cascadeX, cascadeY, size.width, size.height);
    }
    else {
        frame.setLocation(cascadeX, cascadeY);
    }
}

protected void resizingScrollPane()
{
    JScrollPane scrollpane =
        (JScrollPane)SwingUtilities.getAncestorOfClass(JScrollPane.class, this);
    //scrollpane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    //scrollpane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
    if (scrollpane != null) {
        // If we have a scrollpane for a parent anywhere along the
        // way then we force a layout
        this.revalidate();
    }
}

protected void sizingToScrollPane()
{
    JScrollPane scrollpane =
        (JScrollPane)SwingUtilities.getAncestorOfClass(JScrollPane.class,
                                                       this);
    //scrollpane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    //scrollpane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
    if (scrollpane != null) {
        // If we have a scrollpane for a parent anywhere along the
        // way then we force a layout
        this.revalidate();
    }
}

/**
 * Arranges non-iconified panes in cascade form.
 */
public void cascadeWindows()
{
    JInternalFrame[] frames = getAllNonIconifiedAndNonPaletteFrames();
    if (frames.length == 0) {
        return;
    }

    moveIconsToBack(getAllIconifiedFrames());

    // Reset cascade
    cascadeReset();

    // Cascade frames
    for (int i = frames.length - 1; i >= 0; i--) {
        cascadeFrame(frames[i]);
    }
    sizingToScrollPane();
}

/**
 * Gets the size of the visible area, which is the size of this pane.
 * @return The size of the visible area
 */
protected Dimension getVisibleSize()
{
    Container parent = getParent();
    if (parent != null && parent.getParent() instanceof JScrollPane) {
        // If the parent is in a scrollpane then for some strange reason
        // we have to fudge sizes to avoid growing scrollbars
        fudge1 = 3;
        fudge2 = 1;
        return(parent.getParent().getSize());
    }
    fudge1 = 0;
    fudge2 = 0;
    return(getSize());
}

/**
 * Move icons to back so they don't appear in front of cascaded or tiled
 * windows
 * @param icons The list of desktop icons
 */
protected void moveIconsToBack(JInternalFrame.JDesktopIcon[] icons)
{
    if (icons != null) {
        for (int i = 0; i < icons.length; i++) {
            moveToBack(icons[i]);
        }
    }
}

/**
 * Get the current height used by all the icons.
 * @param icons The list of desktop icons
 * @return The height of the icon "display"
 */
protected int getIconsHeight(JInternalFrame.JDesktopIcon[] icons)
{
    int height = getVisibleSize().height;
    int y = height;
    for (int i = 0; i < icons.length; i++) {
        Point p = icons[i].getLocation();
        y = Math.min(y, (int)p.getY());
    }
    return(height - y);
}

/**
 * Tiles the frames horizontally.
 */
public void tileWindowsHorizontally()
{
    JInternalFrame[] frames = getAllNonIconifiedAndNonPaletteFrames();
    int nFrames = frames.length;
    if (nFrames == 0) {
        return;
    }
    

    int windowWidth  = getVisibleSize().width - fudge1;
    int windowHeight = getVisibleSize().height - fudge2;

    // Possibly adjust windowHeight if we a tiling over icons
    JInternalFrame.JDesktopIcon[] icons = getAllIconifiedFrames();
    if (considerIconsWhenTiling) {
        arrangeIcons();
        // If the icons height is bigger than the window then we punt and tile
        // over the icons anyways
        int iconsHeight = getIconsHeight(icons);
        if (windowHeight > iconsHeight) {
            windowHeight -= iconsHeight;
        }
    }

    // Don't want icons to be on top of tiled windows
    moveIconsToBack(icons);

    int frameHeight = windowHeight / nFrames;
    int fatterSliceCount = windowHeight % nFrames;
    
    int y = 0;
    for (int i = 0; i < nFrames; i++) {
        // If the number of frames did not equally divide into the height of
        // the window, add one to the height of the first "remainder" number
        // of frames
        int curHeight = (i < fatterSliceCount) ? frameHeight + 1 : frameHeight;
        if (frames[i].isResizable()) {
            frames[i].setBounds(0, y, windowWidth, curHeight);
        }
        else {
            frames[i].setLocation(0, y);
        }
        y += curHeight;
    }

    repaint();
    sizingToScrollPane();
}

/**
 * Tiles the frames vertically.
 */
public void tileWindowsVertically()
{
    JInternalFrame[] frames = getAllNonIconifiedAndNonPaletteFrames();
    int nFrames = frames.length;
    if (nFrames == 0) {
        return;
    }
    

    int windowWidth  = getVisibleSize().width - fudge2;
    int windowHeight = getVisibleSize().height - fudge1;

    // Possibly adjust windowHeight if we a tiling over icons
    JInternalFrame.JDesktopIcon[] icons = getAllIconifiedFrames();
    if (considerIconsWhenTiling) {
        arrangeIcons();
        // If the icons height is bigger than the window then we punt and tile
        // over the icons anyways
        int iconsHeight = getIconsHeight(icons);
        if (windowHeight > iconsHeight) {
            windowHeight -= iconsHeight;
        }
    }

    // Don't want icons to be on top of tiled windows
    moveIconsToBack(icons);

    int frameWidth = windowWidth / nFrames;
    int fatterSliceCount = windowWidth % nFrames;
    
    int x = 0;
    for (int i = 0; i < nFrames; i++) {
        // If the number of frames did not equally divide into the width of
        // the window, add one to the width of the first "remainder" number
        // of frames
        int curWidth = (i < fatterSliceCount) ? frameWidth + 1 : frameWidth;
        if (frames[i].isResizable()) {
            frames[i].setBounds(x, 0, curWidth, windowHeight);
        }
        else {
            frames[i].setLocation(x, 0);
        }
        x += curWidth;
    }
    sizingToScrollPane();
    repaint();
}

/**
 * Arranges "iconified" panes along the bottom.
 */
public void arrangeIcons()
{
    JInternalFrame.JDesktopIcon[] icons = getAllIconifiedFrames();
    if (icons.length == 0) {
        return;
    }
    int width  = getVisibleSize().width;
    int height = getVisibleSize().height;

    int x = 0;
    int y = height;
    int maxHeight = 0;
    for (int i = 0; i < icons.length; i++) {
        int w = icons[i].getWidth();
        int h = icons[i].getHeight();

        // If not enought available space on this row and there has at least
        // been one icon on this row, then move to new row
        if (x + w > width && maxHeight > 0) {
            x = 0;
            y -= maxHeight;
            maxHeight = 0;
        }
        maxHeight = Math.max(maxHeight, h);

        icons[i].setBounds(x, y - h, w, h);
        x += w;
    }
    repaint();
}

/**
 * Returns the count of internal frames that are not closed.
 * @return internal frames that are not closed
 */
protected int getOpenInternalFrameCount()
{
    Component c[] = getComponents();
    int count = 0;
    for (int i = 0; i < c.length; i++) {
        if (c[i] instanceof JInternalFrame) {
            JInternalFrame frame = (JInternalFrame)c[i];
            if (!frame.isClosed() &&
                frame.isVisible() &&
                frame.getLayer() != JLayeredPane.PALETTE_LAYER.intValue())
            {
                count++;
            }
        }
    }
    return(count);
}

/**
 * Returns the count of the iconified frames.
 * @return count of the iconified frames
 */
protected int getIconifiedFrameCount()
{
    Component c[] = getComponents();
    int count = 0;
    for (int i = 0; i < c.length; i++) {
        if (c[i] instanceof JInternalFrame.JDesktopIcon) {
            JInternalFrame.JDesktopIcon icon =
                (JInternalFrame.JDesktopIcon)c[i];
            JInternalFrame frame = icon.getInternalFrame();
            if (frame != null && frame.isVisible()) {
                count++;
            }
        }
    }
    return(count);
}

/**
 * No frame is selected, or the selected frame is being or has been removed;
 * therefore, we need to select a frame.
 */
protected void selectNextFrame()
{
    
    if (activatingFrame != null) {
        // We don't attempt to select another frame since we are in the
        // middle of activating one
        
        return;
    }

    // Select another window to be active
    // WARNING This may need to be remedied in the future if the default
    // desktop implements this behaviour
    Component children[] = getComponents();
    JInternalFrame next_frame = null;
    if (children.length != 0) {
        // Loop through children until we find a JInternalFrame
        for (int i = 0; i < children.length; i++) {
            if (children[i] instanceof JInternalFrame) {
                JInternalFrame frame = (JInternalFrame)children[i];
                if (frame.isMaximizable() && frame.isVisible()) {
                    next_frame = (JInternalFrame)children[i];
                    break;
                }
            }
        }
    }
    else {
        this.activateFrame(null);
    }
    if (next_frame != null) {
        
        try {
            next_frame.setSelected(true);
        }
        catch (PropertyVetoException e) {
            // PENDING - should we do something special if user vetoes
            // frame selection
        }
    }
    else {
        // Clear the menu bars
        
        setMenuAndToolBarsToRootPane(null, menubar, toolbar);
    }
}

protected void setMenuAndToolBarsToRootPane(JInternalFrame frame,
                                            JMenuBar menubar,
                                            JToolBar toolbar)
{
    
    if (menubar == null && toolbar == null) {
        return;
    }

    JRootPane rootpane = getRootPane();
    if (rootpane == null) {
        return;
    }

    if (menubar != null) {
        currentMenuBar = menubar;
        
        updateWindowsMenu();
        updateIconsInMenuBar(frame);
        rootpane.setJMenuBar(menubar);
        
        //updateWindowsMenu();
    }
    else {
        
    }
    if (toolbar != null && toolbar != lastToolBarSetToRoot) {
        // Remove old toolbar first
        if (lastToolBarSetToRoot != null) {
            Container ancestorRoot =
                SwingUtilities.getAncestorOfClass(JFrame.class, rootpane);
            Container ancestorLast =
                SwingUtilities.getAncestorOfClass(JFrame.class,
                                                  lastToolBarSetToRoot);
            if (ancestorRoot == ancestorLast) {
                lastToolBarSetToRoot.getParent().remove(lastToolBarSetToRoot);
            }
        }
        rootpane.getContentPane().add(toolbar, "North");
        lastToolBarSetToRoot = toolbar;
    }
    rootpane.invalidate();
    rootpane.validate();
    rootpane.repaint();
    
}

/**
 * Makes this frame the active frame.
 * @param iframe The frame to activate
 */
public void activateFrame(JInternalFrame iframe)
{
    
    if (activeFrame == iframe) {
        
        return;
    }
    activatingFrame = iframe;

    if (iframe instanceof JCMDIFrame) {
        // If toolbar is associated then update the toolbar
        // if menubar is associated then update the "Windows" list
        
        JCMDIFrame frame = (JCMDIFrame)iframe;
        // Set menu and tool bars if possible
        setMenuAndToolBarsToRootPane(frame, getMDIMenuBar(frame),
                                     getMDIToolBar(frame));
    }
    if (maximized) {
        updateIconsInMenuBar(iframe);
    }
    updateWindowsMenu();
    activeFrame = iframe;
    // Frame has finished activation so we can reset activatingFrame
    activatingFrame = null;
}

/**
 * Deactivates a frame.
 * @param iframe The frame to deactivate
 */
public void deactivateFrame(JInternalFrame iframe)
{
    activeFrame = null;
}

/**
 * Copies any children back.
 * @param frame The frame to be closed
 */
public void closeFrame(JInternalFrame frame)
{
    if (frame == null) {
        return;
    }
    

    int open_frame_count = getOpenInternalFrameCount();
    if (frame != maximizedFrame) {
        JInternalFrame f =
            (activatingFrame == null) ? activeFrame : activatingFrame;
        if (open_frame_count == 0 && f == null) {
            // Switch toolbar and menu bar to default
            checkIfMenuOrToolBarsNeeded();
        }
        return;
    }

    if (open_frame_count < 1) {
        // If there are only one internal frame we move it's children
        // back into it by maximizing a "null" frame
        
        maximizeFrame(null);
    }
    else {
        // We switch the active frame to the first sibling internal
        // frame that we find
        
        Component c[] = getComponents();
        for (int i = 0; i < c.length; i++) {
            if (c[i] != frame && c[i] instanceof JInternalFrame) {
                try {
                    // Select other frame
                    JInternalFrame oframe = (JInternalFrame)c[i];
                    if (!oframe.isVisible()) {
                        continue;
                    }
                    //activateFrame(oframe);
                    oframe.setSelected(true);
                    // This will set the target to be another frame
                    updateIconsInMenuBar(oframe);
                    if (oframe.isMaximizable()) {
                        maximizeFrame(oframe);
                    }
                    break;
                }
                catch (java.beans.PropertyVetoException pve) {
                    // Do nothing if some wanker vetoes selection
                    
                }
            }
        }
    }
    repaint();
}

/**
 * Sets the icon which is to appear beside the selected window item
 * in the Windows menu. Set to null if you do not want an icon to appear.
 * @param selectedIcon the icon to use
 */
public void setSelectedIcon(Icon selectedIcon)
{
    this.selectedIcon = selectedIcon;
}

/**
 * Gets the icon which is to appear beside the selected window item
 * in the Windows menu. The default icon is a checkmark.
 * @return the icon to use
 */
public Icon getSelectedIcon()
{
    return(selectedIcon);
}

/**
 * Sets the icon which is to appear beside the non-selected window items
 * in the Windows menu. Set to null if you do not want an icon to appear.
 * @param nonSelectedIcon the icon to use
 */
public void setNonSelectedIcon(Icon nonSelectedIcon)
{
    this.nonSelectedIcon = nonSelectedIcon;
}

/**
 * Gets the icon which is to appear beside the non-selected window item
 * in the Windows menu. The default is an empty icon as a place holder
 * in the menu.
 * @return the icon to use
 */
public Icon getNonSelectedIcon()
{
    return(nonSelectedIcon);
}

/**
 * Creates an icon from a String array.
 * @param pixels the String representation of the image
 * @return image the Icon icon
 */
protected static Icon makeImage(String[] pixels)
{
    JCIconCreator creator = new JCIconCreator(13, 13);
    creator.setColor('g', Color.green);
    creator.setColor('l', Color.blue);
    creator.setColor('X', Color.lightGray);
    creator.setColor('w', Color.white);
    creator.setColor('b', Color.black);
    creator.setColor('r', new Color(220, 50, 0));

    creator.setPixels(pixels);
    ImageIcon image = creator.getIcon();
    creator.clear();
    return(image);
}

/**
 * Listener class to listen to components added to the MDI container
 * and to add listeners to the components.
 */
class Listener implements ContainerListener, java.io.Serializable
{
    //
    // ContainerListener Interface Implementaion
    //
    public void componentAdded(ContainerEvent e)
    {
        
        // If component is a JInternalFrame then add it to the
        // Windows menu list
        if (e.getChild() instanceof JInternalFrame) {
            JInternalFrame f = (JInternalFrame)e.getChild();
            f.addInternalFrameListener(frameListener);
            f.addVetoableChangeListener(frameListener);
            DesktopManager dm = f.getDesktopPane().getDesktopManager();
            if (!(dm instanceof MDIDesktopManager)) {
                f.getDesktopPane().setDesktopManager(new MDIDesktopManager(dm));
            }
            Rectangle r = f.getBounds();
            
            if (r.x == 0 && r.y == 0 && r.width == 0 && r.height == 0) {
                // Bounds are unset so we set them
                cascadeFrameByPreferredSize(f);
            }
            if (isDisplayable() && f.isVisible()) {
                try {
                    f.setSelected(true);
                }
                catch (PropertyVetoException pve) {}
            }
        }
        updateWindowsMenu();
    }

    public void componentRemoved(ContainerEvent e)
    {
        

        if (e.getChild() instanceof JInternalFrame) {
            closeFrame((JInternalFrame)e.getChild());
            selectNextFrame();
        }
        updateWindowsMenu();
    }
}

protected class FrameListener implements InternalFrameListener,
                                         VetoableChangeListener,
                                         java.io.Serializable
{
    //
    // InternalFrameListener Interface Implementaion
    //
    public void internalFrameOpened(InternalFrameEvent e)
    {
        
        updateWindowsMenu();
    }

    public void internalFrameClosing(InternalFrameEvent e) {}

    public void internalFrameClosed(InternalFrameEvent e)
    {
        
        updateWindowsMenu();
    }

    public void internalFrameIconified(InternalFrameEvent e) {}

    // Frames are removed and re-added when activated, except when
    // they are currently iconified. Thus we cannot rely on activate
    // to bring a frame to the front from an iconified state.
    // Thus, explicitly move it to the front when deiconified.
    public void internalFrameDeiconified(InternalFrameEvent e)
    {
        ((JInternalFrame)e.getSource()).moveToFront();
    }

    public void internalFrameActivated(InternalFrameEvent e)
    {
        
        JInternalFrame f = (JInternalFrame)e.getSource();
        if (f != null && !f.isVisible()) {
            return;
        }
        activateFrame(f);
    }

    public void internalFrameDeactivated(InternalFrameEvent e)
    {
        
        //deactivateFrame((JInternalFrame) e.getSource());
    }

    public void vetoableChange(PropertyChangeEvent e)
        throws PropertyVetoException
    {
        
        if (e.getPropertyName().equals("maximum")) {
            Object o = e.getSource();
            if (o instanceof JInternalFrame) {
                maximize((JInternalFrame)o);
                throw new PropertyVetoException("MDI Override", e);
            }
        }
    }
}

/**
 * </code>WindowsMenuListener</code> listens for selections in the window list
 * menu, then brings the selected window to the front.
 */
protected class WindowsMenuListener implements ActionListener,
                                               java.io.Serializable
{
    public void actionPerformed(ActionEvent e)
    {
        

        JInternalFrame frame =
            (JInternalFrame)((UserMenuItem)e.getSource()).getUserData();
        if (maximized && frame.isMaximizable()) {
            try {
                frame.setSelected(true);
            }
            catch (java.beans.PropertyVetoException pex) {
                // Do nothing if some wanker vetoes the selection
                
            }
            maximizeFrame(frame);
        }
        else {
            // Post a to-front event
            if (frame.isIcon()) {
                try {
                    frame.setIcon(false);
                }
                catch (java.beans.PropertyVetoException pve) {
                    // Do nothing if some wanker vetoes the deiconification
                    
                }
            }
            try {
                frame.setSelected(true);
            }
            catch (java.beans.PropertyVetoException ex) {
                // Do nothing if some wanker vetoes the selection
                
            }
        }
    }
}

protected class MaxMinAction extends JCAction implements java.io.Serializable
{
    public MaxMinAction()
    {
        super(li.getString(li.ACTION_MAXIMIZE),
              li.getMnemonic(li.ACTION_MAXIMIZE_MNEMONIC), null);
    }

    public void actionPerformed(ActionEvent e)
    {
        
        if (maximized == true) {
            unmaximize();
        }
        else {
            if (activeFrame != null) {
                maximize(activeFrame);
            }
        }
    }
}

protected class CascadeAction extends JCAction implements java.io.Serializable
{
    public CascadeAction()
    {
        super(li.getString(li.ACTION_CASCADE),
              li.getMnemonic(li.ACTION_CASCADE_MNEMONIC), null);
    }

    public void actionPerformed(ActionEvent e)
    {
        if (maximized == true) {
            unmaximize();
        }
        cascadeWindows();
    }
}

protected class TileHAction extends JCAction implements java.io.Serializable
{
    public TileHAction()
    {
        super(li.getString(li.ACTION_TILEH),
              li.getMnemonic(li.ACTION_TILEH_MNEMONIC), null);
    }

    public void actionPerformed(ActionEvent e)
    {
        if (maximized == true) {
            unmaximize();
        }
        tileWindowsHorizontally();
    }
}

protected class TileVAction extends JCAction implements java.io.Serializable
{
    public TileVAction()
    {
        super(li.getString(li.ACTION_TILEV),
              li.getMnemonic(li.ACTION_TILEV_MNEMONIC), null);
    }

    public void actionPerformed(ActionEvent e)
    {
        if (maximized == true) {
            unmaximize();
        }
        tileWindowsVertically();
    }
}

protected class ArrangeAction extends JCAction implements java.io.Serializable
{
    public ArrangeAction()
    {
        super(li.getString(li.ACTION_ARRANGE),
              li.getMnemonic(li.ACTION_ARRANGE_MNEMONIC),
              null);
    }

    public void actionPerformed(ActionEvent e)
    {
        arrangeIcons();
    }
}

/**
 * <code>LayoutListener</code> listens for requests to manage the internal
 * frames.
 */
protected class LayoutListener implements ActionListener, java.io.Serializable
{
    public void actionPerformed(ActionEvent e)
    {
        JInternalFrame frame = maximizedFrame;
        //
        // Handle Requests from buttons
        //
        LayoutButton b = (LayoutButton)e.getSource();
        switch (b.getControlType()) {
        case LayoutButton.CLOSE:
            
            if (frame != null && frame.isClosable()) {
                int closable = frame.getDefaultCloseOperation();
                if (closable != JInternalFrame.DO_NOTHING_ON_CLOSE) {
                    try {
                        
                        frame.setClosed(true);
                    }
                    catch (java.beans.PropertyVetoException pve) {
                        // Do nothing if some wanker vetoes closing
                        
                    }
                }
            }
            break;
        case LayoutButton.MINIMIZE:
            
            unmaximize();
            break;
        case LayoutButton.ICONIFY:
            
            unmaximize();
            if (frame != null && frame.isIconifiable()) {
                try {
                    frame.setIcon(true);
                }
                catch (java.beans.PropertyVetoException pve) {
                    // Do nothing if some wanker vetoes the iconification
                    
                }
            }
            break;
        }
    }
}

/**
 * This inner class wraps a PLAF-specific desktop manager and overrides its
 * basic window dragging and resizing behavior to optionally force
 * the use of wireframes (for speed).  It also overrides the maximization
 * behavior to provide maximal use of screen real estate and traps
 * activation and deactivation of frames to handle tool and menu bar
 * switching.
 */
public class MDIDesktopManager implements DesktopManager,
                                          java.io.Serializable
{
    protected DesktopManager manager;

    public MDIDesktopManager(DesktopManager manager)
    {
        // This is the manager that handles everything that
        // we are not overriding
        setDesktopManager(manager);
    }

    /**
     * Returns the desktop manager that this class wraps.
     * @return desktop manager
     */
    public DesktopManager getDesktopManager()
    {
        return(manager);
    }

    /**
     * Sets the desktop manager that this class wraps.
     * @param manager
     */
    public void setDesktopManager(DesktopManager manager)
    {
        this.manager = manager;
    }

    //
    // All the pass through methods
    //
    public void openFrame(JInternalFrame f)
    {
        
        manager.openFrame(f);
    }

    public void closeFrame(JInternalFrame f)
    {
        JCMDIPane.this.closeFrame(f);
        manager.closeFrame(f);
    }

    public void maximizeFrame(JInternalFrame f)
    {
        // Intercept maximize request here
        
        //maximize(f);
    }

    public void minimizeFrame(JInternalFrame f)
    {
        manager.minimizeFrame(f);
    }

    public void iconifyFrame(JInternalFrame f)
    {
        manager.iconifyFrame(f);
    }

    public void deiconifyFrame(JInternalFrame f)
    {
        manager.deiconifyFrame(f);
    }

    public void activateFrame(JInternalFrame f)
    {
        
        if (f != null && !f.isVisible()) {
            return;
        }
        JCMDIPane.this.activateFrame(f);
        manager.activateFrame(f);
        if (maximized == true  &&
            f != null          &&
            f.isMaximizable()  &&
            f != maximizingFrame)
        {
            try {
                f.setMaximum(true);
            }
            catch (java.beans.PropertyVetoException pex) {}
        }
    }

    public void deactivateFrame(JInternalFrame f)
    {
        JCMDIPane.this.deactivateFrame(f);
        manager.deactivateFrame(f);
    }

    //
    // Override default behaviour to handle MDI maximization
    //

    //
    // Override default sizing and dragging behaviour
    //
    final int INVALID_X = -999;
    private Rectangle rect = new Rectangle(INVALID_X, 0, 0, 0);
    boolean dragging = false;

    /**
     * Draws temporary line in all of parent's children while dragging.
     * @param clear if <code>true</code>, the previously-drawn line is erased
     * (and the remaining arguments are ignored)
     * @param clear
     * @param f
     * @param h
     * @param w
     * @param x
     * @param y
     */
    private void drawRect(boolean clear, JComponent f,
                          int x, int y, int w, int h)
    {
        // Drawing the line again at the same position erases it
        if (!clear) {
            rect.setBounds(x, y, w, h);
        }
        else if (rect.x == INVALID_X) {
            return;
        }

        JComponent desktop;
        if (f instanceof JInternalFrame) {
            desktop = ((JInternalFrame)f).getDesktopPane();
        }
        else if (f instanceof JInternalFrame.JDesktopIcon) {
            desktop = ((JInternalFrame.JDesktopIcon)f).getDesktopPane();
        }
        else {
            
            return;
        }

        Graphics gc = desktop.getGraphics();
        gc.setXORMode(desktop.getBackground());
        // Subtract one to make sure that the wireframe is within the
        // region that get's triggered by the repaint manager
        // The drawing is now off by one but this is not noticable and
        // the alternative is having a ghost line remaining when the
        // frame is made smaller.
        gc.drawRect(rect.x, rect.y, rect.width - 1, rect.height - 1);
        // Make sure the clearing doesn't get done twice
        gc.dispose();
    }

    private void setBounds(JComponent f, int x, int y, int w, int h)
    {
        drawRect(true, f, 0, 0, 0, 0);
        drawRect(false, f, x, y, w, h);
    }

    public void beginDraggingFrame(JComponent f)
    {
        if (frameManipulationStyle != WIREFRAME) {
            manager.beginDraggingFrame(f);
        }
    }

    public void dragFrame(JComponent f, int x, int y)
    {
        if (frameManipulationStyle != WIREFRAME) {
            manager.dragFrame(f, x, y);
        }
        else {
            dragging = true;
            setBounds(f, x, y, f.getWidth(), f.getHeight());
        }
    }

    public void endDraggingFrame(JComponent f)
    {
        if (frameManipulationStyle != WIREFRAME) {
            manager.endDraggingFrame(f);
        }
        else if (dragging) {
            drawRect(true, f, 0, 0, 0, 0);
            dragging = false;
            setBoundsForFrame(f, rect.x, rect.y, rect.width, rect.height);
        }
        resizingScrollPane();
    }

    public void beginResizingFrame(JComponent f, int direction)
    {
        if (frameManipulationStyle != WIREFRAME) {
            manager.beginResizingFrame(f, direction);
        }
    }

    public void resizeFrame(JComponent f, int x, int y,
                            int width, int height)
    {
        if (frameManipulationStyle != WIREFRAME) {
            manager.resizeFrame(f, x, y, width, height);
        }
        else {
            dragging = true;
            setBounds(f, x, y, width, height);
        }
    }

    public void endResizingFrame(JComponent f)
    {
        if (frameManipulationStyle != WIREFRAME) {
            manager.endResizingFrame(f);
        }
        else if (dragging) {
            
            drawRect(true, f, 0, 0, 0, 0);
            dragging = false;
            setBoundsForFrame(f, rect.x, rect.y, rect.width, rect.height);
        }
    }

    public void setBoundsForFrame(JComponent f, int x, int y,
                                  int width, int height)
    {
        manager.setBoundsForFrame(f, x, y, width, height);
        // invalidate rect.x so no wire frames are drawn later
        rect.x = INVALID_X;

        if (frameManipulationStyle == WIREFRAME) {
            return;
        }
        JScrollPane scrollpane =
            (JScrollPane)SwingUtilities.getAncestorOfClass(JScrollPane.class,
                                                           f);
        if (scrollpane != null) {
            // if we have a scrollpane for a parent anywhere along the
            // way then we force a layout
            f.revalidate();
        }
    }
} // end of MDIDesktopManager

}
