/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCMultiSelectList.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:28 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.swing;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.plaf.ComponentUI;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.Serializable;
import java.util.Vector;


/**
 * <code>JCMultiSelectList</code> matches the API for <code>JList</code>, except that
 * two lists instead of one appear in the components GUI. There are
 * buttons between the two lists that move items back and forth.
 */
public class JCMultiSelectList extends JComponent implements Serializable {

// Button types
protected static final String SELECT_ITEM 	= ">";
protected static final String SELECT_ALL 	= ">>";
protected static final String DESELECT_ITEM = "<";
protected static final String DESELECT_ALL	= "<<";

// Internal components
protected JList            mainList = null;
protected JList            sourceList = null;
protected JList            selectionList = null;
protected JScrollPane      sourcePane = null;
protected JScrollPane      selectionPane = null;
protected SelectionButton  selectItemButton = null;
protected SelectionButton  selectAllButton = null;
protected SelectionButton  deselectItemButton = null;
protected SelectionButton  deselectAllButton = null;
protected JPanel           buttonPanel = null;
protected EventHandler     eventHandler = new EventHandler();
protected Dimension        defaultListSize = new Dimension(100, 200);
protected FontMetrics      lastFontMetrics = null;

private static final String uiClassID = "MultiSelectListUI";

/**
 * Constructs a <code>JCMultiSelectList</code> that displays the elements in
 * the specified non-null model.
 * All <code>JCMultiSelectList</code> constructors delegate to this one.
 * @param dataModel
 */
public JCMultiSelectList(ListModel dataModel)
{
    this();
    mainList.setModel(dataModel);
    init();
}

/**
 * Constructs a <code>JCMultiSelectList</code> that displays the elements in
 * the specified array.
 * This constructor just delegates to the <code>ListModel</code> constructor.
 * @param listData
 */
public JCMultiSelectList(Object[] listData)
{
    this();
    mainList.setListData(listData);
    init();
}

/**
 * Constructs a <code>JCMultiSelectList</code> that displays the elements in
 * the specified Vector.
 * This constructor just delegates to the <code>ListModel</code> constructor.
 * @since JClass 4.0K for Java 2. If you are using the J version,
 * which is based on JDK 1.1, this method's parameter must be
 * a <code>com.sun.java.util.collections.Vector</code>.
 * @param listData
 */
public JCMultiSelectList(Vector listData)
{
    this();
    mainList.setListData(listData);
    init();
}

/**
 * Creates a new <code>JCMultiSelectList</code> component with an empty model.
 */
public JCMultiSelectList()
{
    this(4);
}

/**
 * Creates a new <code>JCMultiSelectList</code> component with an empty model
 * and the specified horizontal gap between components.
 * @param hgap
 */
public JCMultiSelectList(int hgap)
{
    super();

    mainList = new JList();
    mainList.addListSelectionListener(eventHandler);

    sourceList = new JList();
    sourceList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
    sourceList.addListSelectionListener(eventHandler);
    sourceList.addMouseListener(
        new MouseAdapter() {
            public void mouseClicked(MouseEvent e)
            {
                if (e.getClickCount() == 2) {
                    selectItem();
                }
            }
        }
    );

    selectionList = new JList();
    selectionList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
    selectionList.addListSelectionListener(eventHandler);
    selectionList.addMouseListener(
        new MouseAdapter() {
            public void mouseClicked(MouseEvent e)
            {
                if (e.getClickCount() == 2) {
                    deselectItem();
                }
            }
        }
    );

    // Needs localization!
    buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout());
    selectItemButton = initButton(new SelectionButton(SELECT_ITEM),
                                  "Add selected");
    selectAllButton = initButton(new SelectionButton(SELECT_ALL), "Add all");
    deselectItemButton = initButton(new SelectionButton(DESELECT_ITEM),
                                    "Remove selected");
    deselectAllButton = initButton(new SelectionButton(DESELECT_ALL),
                                   "Remove all");

    Dimension dimButton = getButtonSize();
    buttonPanel.setPreferredSize(new Dimension(dimButton.width,
                                               4 * dimButton.height));

    setLayout(new JCFixedCenterLayout(hgap));

    sourcePane = new JScrollPane(sourceList);
    sourcePane.setPreferredSize(defaultListSize);
    selectionPane = new JScrollPane(selectionList);
    selectionPane.setPreferredSize(defaultListSize);

    add(sourcePane, "West");
    add(selectionPane, "East");
    add(buttonPanel, "Center");

//	add(sourcePane, SwingConstants.WEST);
//	add(selectionPane, SwingConstants.EAST);
//	add(buttonPanel, SwingConstants.CENTER);
    setUI(new MultiSelectListUI());
}

/**
 * Updates the look and feel.
 */
public void updateUI()
{
    setUI(new MultiSelectListUI());
}

/**
 * Returns the name of the <code>UIFactory</code> for this component.
 * @return <code>UIFactory</code> name
 */
public String getUIClassID()
{
    return(uiClassID);
}

/**
 * Provided for Beans property editors - has no effect.
 * @param s
 */
public void setAbout(String s) {}

/**
 * Returns vendor information.
 * @return vendor information
 */
public String getAbout()
{
    return(JCVersion.getVersionString());
}

/**
 * Returns the cell size based on an estimate of the cell's width and height.
 * @return cell size
 */
public Object getPrototypeCellValue()
{
    return(mainList.getPrototypeCellValue());
}

/**
 * Sets the cell size based on an estimate of the cell's width and height.
 * @param prototypeCellValue
 */
public void setPrototypeCellValue(Object prototypeCellValue)
{
    mainList.setPrototypeCellValue(prototypeCellValue);
    sourceList.setPrototypeCellValue(prototypeCellValue);
    selectionList.setPrototypeCellValue(prototypeCellValue);
}

/**
 * Returns the fixed cell width.
 * @return fixed cell width
 */
public int getFixedCellWidth()
{
    return(mainList.getFixedCellWidth());
}

/**
 * Sets the fixed cell width.
 * @param width fixed cell width
 */
public void setFixedCellWidth(int width)
{
    mainList.setFixedCellWidth(width);
    sourceList.setFixedCellWidth(width);
    selectionList.setFixedCellWidth(width);
}

/**
 * Returns the fixed cell height.
 * @return fixed cell height
 */
public int getFixedCellHeight()
{
    return(mainList.getFixedCellHeight());
}

/**
 * Sets the fixed cell height.
 * @param height fixed cell height
 */
public void setFixedCellHeight(int height)
{
    mainList.setFixedCellHeight(height);
    sourceList.setFixedCellHeight(height);
    selectionList.setFixedCellHeight(height);
}

/**
 * Returns the renderer that's used to paint each cell.
 * @return rendered used to paint each cell
 */
public ListCellRenderer getCellRenderer()
{
    return(mainList.getCellRenderer());
}

/**
 * Sets the renderer that's used to paint each cell.
 * @param cellRenderer rendered used to paint each cell
 */
public void setCellRenderer(ListCellRenderer cellRenderer)
{
    mainList.setCellRenderer(cellRenderer);
    sourceList.setCellRenderer(cellRenderer);
    selectionList.setCellRenderer(cellRenderer);
}

/**
 * Returns the foreground color for selected cells.
 * @return foreground color for selected cells
 */
public Color getSelectionForeground()
{
    return(mainList.getSelectionForeground());
}

/**
 * Sets the foreground color for selected cells.
 * @param selectionForeground foreground color
 */
public void setSelectionForeground(Color selectionForeground)
{
    mainList.setSelectionForeground(selectionForeground);
    sourceList.setSelectionForeground(selectionForeground);
    selectionList.setSelectionForeground(selectionForeground);
}

/**
 * Returns the background color for selected cells.
 * @return background color for selected cells
 */
public Color getSelectionBackground()
{
    return(mainList.getSelectionBackground());
}

/**
 * Sets the background color for selected cells.
 * @param selectionBackground background color
 */
public void setSelectionBackground(Color selectionBackground)
{
    mainList.setSelectionBackground(selectionBackground);
    sourceList.setSelectionBackground(selectionBackground);
    selectionList.setSelectionBackground(selectionBackground);
}

//================== ListModel interface methods ==================

/**
 * Returns the list's data model.
 * @return list's data model
 */
public ListModel getModel()
{
    return(mainList.getModel());
}

/**
 * Sets the list's data model.
 * @param model
 */
public void setModel(ListModel model)
{
    mainList.setModel(model);
    updateView();
}

/**
 * Constructs the list's data model from an array of Objects.
 * @param listData
 */
public void setListData(final Object[] listData)
{
    mainList.setListData(listData);
    updateView();
}

/**
 * Constructs the list's data model from a Vector.
 * @param listData
 */
public void setListData(final Vector listData)
{
    mainList.setListData(listData);
    updateView();
}

//================== ListSelectionModel interface methods ==================

/**
 * Returns the current selection model.
 * @return current selection model
 */
public ListSelectionModel getSelectionModel()
{
    return(mainList.getSelectionModel());
}

/**
 * Sets the selection model.
 *
public void setSelectionModel(ListSelectionModel selectionModel)
{
     mainList.setSelectionModel(selectionModel);
}
*/

/**
 * Adds a listener to the <code>ListSelectionListeners</code> list.
 * @param listener
 */
public void addListSelectionListener(ListSelectionListener listener)
{
    listenerList.add(ListSelectionListener.class, listener);
}

/**
 * Removes a listener from the the <code>ListSelectionListeners</code> list.
 * @param listener
 */
public void removeListSelectionListener(ListSelectionListener listener)
{
    listenerList.remove(ListSelectionListener.class, listener);
}

/**
 * Forwards the given notification event to all registered
 * listeners.
 * @param firstIndex
 * @param isAdjusting
 * @param lastIndex
 */
protected void fireSelectionValueChanged(int firstIndex,
                                         int lastIndex, boolean isAdjusting)
{
    // Guaranteed to return a non-null array
    Object[] listeners = listenerList.getListenerList();
    ListSelectionEvent e = null;

    // Process the listeners last to first, notifying
    // those that are interested in this event
    for (int i = listeners.length - 2; i >= 0; i -= 2) {
        if (listeners[i] == ListSelectionListener.class) {
            if (e == null) {
                e = new ListSelectionEvent(this, firstIndex,
                                           lastIndex, isAdjusting);
            }
            ((ListSelectionListener)listeners[i + 1]).valueChanged(e);
        }
    }
}

/**
 * Returns the <code>selectionMode</code> property value.
 * @return <code>selectionMode</code> property value
 */
public int getSelectionMode()
{
    return(mainList.getSelectionMode());
}

/**
 * Returns the first index argument from the most recent interval selection.
 * @return first index argument from the most recent interval selection
 */
public int getAnchorSelectionIndex()
{
    return(getSelectionModel().getAnchorSelectionIndex());
}

/**
 * Returns the second index argument from the most recent interval selection.
 * @return second index argument from the most recent interval selection
 */
public int getLeadSelectionIndex()
{
    return(mainList.getLeadSelectionIndex());
}

/**
 * Returns the minimal selected cell index.
 * @return minimal selected cell index
 */
public int getMinSelectionIndex()
{
    return(mainList.getMinSelectionIndex());
}

/**
 * Returns the maximal selected cell index.
 * @return maximal selected cell index
 */
public int getMaxSelectionIndex()
{
    return(mainList.getMaxSelectionIndex());
}

/**
 * Returns <code>true</code> if the specified index is selected.
 * @return <code>true</code> if the specified index is selected
 * @param index specified index
 */
public boolean isSelectedIndex(int index)
{
    return(mainList.isSelectedIndex(index));
}

/**
 * Returns <code>true</code> if nothing is selected.
 * @return <code>true </code> if nothing is selected
 */
public boolean isSelectionEmpty()
{
    return(mainList.isSelectionEmpty());
}

/**
 * Clears the selection.
 */
public void clearSelection()
{
    mainList.clearSelection();
    updateView();
}

/**
 * Selects the specified interval.
 * @param anchor
 * @param lead
 */
public void setSelectionInterval(int anchor, int lead)
{
    mainList.setSelectionInterval(anchor, lead);
    updateView();
}

/**
 * Adds the specified interval to the current selection.
 * @param anchor
 * @param lead
 */
public void addSelectionInterval(int anchor, int lead)
{
    mainList.addSelectionInterval(anchor, lead);
    updateView();
}

/**
 * Removes the specified interval to the current selection.
 * @param index0
 * @param index1
 */
public void removeSelectionInterval(int index0, int index1)
{
    mainList.removeSelectionInterval(index0, index1);
    updateView();
}

/**
 * Returns the first selected cell index.
 * @return first selected cell index
 */
public int getSelectedIndex()
{
    return(mainList.getSelectedIndex());
}

/**
 * Selects a cell by the index.
 * @param index
 */
public void setSelectedIndex(int index)
{
    mainList.setSelectedIndex(index);
    updateView();
}

/**
 * Returns the selected indices array.
 * @return selected indices array
 */
public int[] getSelectedIndices()
{
    return(mainList.getSelectedIndices());
}

/**
 * Selects the specified cells.
 * @param indices
 */
public void setSelectedIndices(int[] indices)
{
    mainList.setSelectedIndices(indices);
    updateView();
}

/**
 * Returns an array of the selected cell values.
 * @return array of the selected cell values
 */
public Object[] getSelectedValues()
{
    return(mainList.getSelectedValues());
}

/**
 * Returns the first selected cell value.
 * @return first selected cell value
 */
public Object getSelectedValue()
{
    return(mainList.getSelectedValue());
}

/**
 * Selects the specified object.
 * @param anObject
 * @param shouldScroll
 */
public void setSelectedValue(Object anObject, boolean shouldScroll)
{
    mainList.setSelectedValue(anObject, shouldScroll);
    updateView();
}

/**
 * Returns a String representation of the object.
 * @return String representation of the object
 */
protected String paramString()
{
    return(super.paramString() + ", the list is: " + mainList);
}

/**
 * Initializes and adds a button to the component.
 * @param button
 * @param toolTipText
 * @return
 */
protected SelectionButton initButton(SelectionButton button, String toolTipText)
{
    if (button != null) {
        buttonPanel.add(button);
        button.addActionListener(eventHandler);
        button.setToolTipText(toolTipText);
    }
    return(button);
}

/**
 * Initializes the component after it is created.
 */
protected void init()
{
    updateView();
}

/**
 * Initializes and adds a button to the component.
 */
protected void updateView()
{
    ListModel mainModel = mainList.getModel();
    int[] selected = mainList.getSelectedIndices();

    Vector sourceVector = new Vector(mainModel.getSize() - selected.length);
    Vector selectionVector = new Vector(selected.length);
    for (int i = 0; i < mainModel.getSize(); i++) {
        if (mainList.isSelectedIndex(i)) {
            selectionVector.addElement(mainModel.getElementAt(i));
        }
        else {
            sourceVector.addElement(mainModel.getElementAt(i));
        }
    }
    sourceList.setListData(sourceVector);
    selectionList.setListData(selectionVector);
    updateButtons();
}

/*
* Enables (or disables) the buttons.
*/
public void updateButtons()
{
    selectItemButton.setEnabled(!sourceList.isSelectionEmpty());
    selectAllButton.setEnabled(sourceList.getModel().getSize() > 0);
    deselectItemButton.setEnabled(!selectionList.isSelectionEmpty());
    deselectAllButton.setEnabled(selectionList.getModel().getSize() > 0);
}

/**
 * Returns the nominal size of the component.
 * @return nominal size of the component
 */
public static final Dimension getButtonSize()
{
    return(new Dimension(32, 32));
}

/**
 * Returns the preferred size of the component.
 * @return preferred size of component
 */
public Dimension getPreferredSize()
{
    Dimension dimSource = sourcePane.getPreferredSize();
    Dimension dimSelection = selectionPane.getPreferredSize();

    Dimension dimButtons = buttonPanel.getPreferredSize();
    int w = dimButtons.width + 2 * getListPreferredWidth();
    int h = Math.max(dimButtons.height, Math.max(dimSource.height,
                                                 dimSelection.height));
    return(new Dimension(w, h));
}

/**
 * Returns the minimum size of the component.
 * @return minimum size of component
 */
public Dimension getMinimumSize()
{
    Dimension dimSource = sourceList.getMinimumSize();
    Dimension dimSelection = selectionList.getMinimumSize();
    Dimension dimButton = buttonPanel.getMinimumSize();
    int w = dimButton.width + dimSource.width + dimSelection.width;
    int h = Math.max(dimButton.height, Math.max(dimSource.height,
                                                dimSelection.height));
    return(new Dimension(w, h));
}

/**
 * Returns the maximum size of the component.
 * @return maximum size of component
 */
public Dimension getMaximumSize()
{
    Dimension dimSource = sourceList.getMaximumSize();
    Dimension dimSelection = selectionList.getMaximumSize();
    Dimension dimButton = buttonPanel.getMaximumSize();
    int w = dimButton.width + dimSource.width + dimSelection.width;
    int h = Math.max(dimButton.height, Math.max(dimSource.height,
                                                dimSelection.height));
    return(new Dimension(w, h));
}

public void paint(Graphics g)
{
    lastFontMetrics = g.getFontMetrics();
    super.paint(g);
}

/**
 * Returns the preferred width of the list.
 * @return preferred width of the list
 */
public int getListPreferredWidth()
{
    ListModel mainModel = mainList.getModel();
    int maxWidth = 0;
    if (lastFontMetrics != null) {
        for (int i = 0; i < mainModel.getSize(); i++) {
            Object item = mainModel.getElementAt(i);
            if (item instanceof String) {
                Math.max(lastFontMetrics.stringWidth((String)item), maxWidth);
            }
        }
    }
    return(maxWidth == 0 ? defaultListSize.width : maxWidth);
}




/**
 * Moves the items selected in the left list to the right list.
 */
protected void selectItem()
{
    // Only respond to user input if enabled
    if (isEnabled()) {
        int[] indices = mainList.getSelectedIndices();
        Object[] values = sourceList.getSelectedValues();

        int newIndices[] = new int[indices.length + values.length];
        System.arraycopy(indices, 0, newIndices, 0, indices.length);

        ListModel mainModel = mainList.getModel();
        for (int i = 0, last = indices.length; i < mainModel.getSize(); i++) {
            Object value = mainModel.getElementAt(i);
            for (int j = 0; j < values.length; j++) {
                if (value == values[j]) {
                    newIndices[last++] = i;
                }
            }
        }

        // Move the selected items
        doUpdate(sourceList, values, newIndices);
    }
}

/**
 * Moves all items from the left list to the right list.
 */
protected void selectAll()
{
    addSelectionInterval(0, mainList.getModel().getSize() - 1);
    updateView();
}

/**
 * Moves the items selected in the right list to the left list.
 */
protected void deselectItem()
{
    // Only respond to user input if enabled
    if (isEnabled()) {
        int[] indices = mainList.getSelectedIndices();
        Object[] values = selectionList.getSelectedValues();

        int newIndices[] = new int[indices.length - values.length];

        ListModel mainModel = mainList.getModel();
        for (int i = 0, last = 0; i < indices.length; i++) {
            Object value = mainModel.getElementAt(indices[i]);
            boolean found = false;
            for (int j = 0; j < values.length; j++) {
                if (value == values[j]) {
                    found = true;
                    break;
                }
            }
            if (!found) {
                newIndices[last++] = indices[i];
            }
        }

        // Move the selected items
        doUpdate(selectionList, values, newIndices);
    }
}

/**
 * Move the selected items from one list and
 * also selects the next item in the list.
 * @param list the list which is to be changed
 * @param values the selected values
 * @param newIndices the indeces of the selected items
 */
protected void doUpdate(JList list, Object values[], int newIndices[])
{
    if (values == null || values.length < 1) {
        return;
    }

    // Order is important. This must come before the updateView since
    // it recreates the selection/source lists.
    int nextIndex = getIndexFromItem(list, values[values.length - 1]);

    // Account for the number of items being removed
    nextIndex -= (values.length - 1);

    // Recreate the lists based on the selected items
    mainList.setSelectedIndices(newIndices);
    updateView();

    // Ready the next item in the list for selection
    if (nextIndex < list.getModel().getSize()) {
        list.setSelectedIndex(nextIndex);
    }
    else {
        // Select last item
        list.setSelectedIndex(list.getModel().getSize() - 1);
    }
}

/**
 * Given an Object, finds its index in the list.
 * @param list the <code>JList</code> to be searched
 * @param item the Object to search for
 * @return the item's index, or -1 if not found
 */
public int getIndexFromItem(JList list, Object item)
{
    ListModel lm = list.getModel();
    for (int i = 0; i < lm.getSize(); i++) {
        Object o = lm.getElementAt(i);
        if (o.equals(item)) {
            return(i);
        }
    }
    return(-1);
}

/**
 * Moves all items from the right list to the left list.
 */
protected void deselectAll()
{
    mainList.clearSelection();
    updateView();
}

/**
 * Enables, disables this component and its subcomponents.
 * @param b boolean: <code>true</code> means enable, <code>false</code> means
 * disable
 */
public void setEnabled(boolean b)
{

    // Disable top level
    super.setEnabled(b);

    // And the two lists
    sourceList.setEnabled(b);
    selectionList.setEnabled(b);

    // Global disable means disabling all buttons, but when enabling
    // only do those that make sense given the state of the lists
    if (b) {
        updateButtons();
    }
    else {
        disableButtons();
    }
}

/**
 * Disables all the buttons.
 */
protected void disableButtons()
{
    selectItemButton.setEnabled(false);
    selectAllButton.setEnabled(false);
    deselectItemButton.setEnabled(false);
    deselectAllButton.setEnabled(false);
}

/**
 * Internal event handler class.
 */
private class EventHandler implements ActionListener,
                                      ListSelectionListener,
                                      Serializable
{
    //================== ActionListener interface methods ==================

    /**
     * Invoked when an action occurs.
     * @param event
     */
    public void actionPerformed(ActionEvent event)
    {
        if (event.getSource() instanceof SelectionButton) {
            SelectionButton btn = (SelectionButton)event.getSource();
            if (btn.equals(selectItemButton)) {
                JCMultiSelectList.this.selectItem();
            }
            else if (btn.equals(selectAllButton)) {
                JCMultiSelectList.this.selectAll();
            }
            else if (btn.equals(deselectItemButton)) {
                JCMultiSelectList.this.deselectItem();
            }
            else if (btn.equals(deselectAllButton)) {
                JCMultiSelectList.this.deselectAll();
            }
        }
    }

    //================== ListSelectionListener interface methods ==================

    /*
     * Called whenever the value of the selection changes.
     */
    public void valueChanged(ListSelectionEvent e)
    {
        if (e.getSource() == JCMultiSelectList.this.mainList) {
            JCMultiSelectList.this.fireSelectionValueChanged(e.getFirstIndex(),
                                                             e.getLastIndex(),
                                                             e.getValueIsAdjusting());
        }
        JCMultiSelectList.this.updateButtons();
    }

} // EventHandler


/**
 * Internal selection button class.
 */
protected class SelectionButton extends JButton implements Serializable {

    /**
     * Creates a new <code>SelectionButton</code> component.
     * @param text
     */
    SelectionButton(String text)
    {
        super(text);
        this.setEnabled(false);
        setMargin(new Insets(2, 2, 2, 2));
    }

    /**
     * Returns the preferred size of the component.
     * @return preferred size of the component
     */
    public Dimension getPreferredSize()
    {
        return(getButtonSize());
    }

    /**
     * Returns the minimum size of the component.
     * @return minimum size of the component
     */
    public Dimension getMinimumSize()
    {
        return(getPreferredSize());
    }

    /**
     * Returns the minimum size of the component.
     * @return minimum size of the component
     */
    public Dimension getMaximumSize()
    {
        return(getPreferredSize());
    }

} // SelectionButton


/**
 * Internal layout.
 */
protected class JCFixedCenterLayout implements LayoutManager2, Serializable {

    protected int        hgap;
    protected Component  west;
    protected Component  east;
    protected Component  center;

    /**
     * Constructs a new layout with no gap between components.
     */
    public JCFixedCenterLayout()
    {
        this(0);
    }

    /**
     * Constructs a layout with the specified gaps between components.
     * @param hgap
     */
    public JCFixedCenterLayout(int hgap)
    {
        this.hgap = hgap;
    }

    /**
     * Returns the horizontal gap between components.
     * @return horizontal gap
     */
    public int getHgap()
    {
        return hgap;
    }

    /**
     * Sets the horizontal gap between components.
     * @param hgap
     */
    public void setHgap(int hgap)
    {
        this.hgap = hgap;
    }

    /**
     * Adds the specified component to the layout.
     * @param comp
     * @param constraints
     */
    public void addLayoutComponent(Component comp, Object constraints)
    {
        synchronized (comp.getTreeLock()) {
            if ((constraints == null) || (constraints instanceof String)) {
                addLayoutComponent((String)constraints, comp);
            }
            else {
                throw new IllegalArgumentException("Cannot add to layout: " +
                                                   "constraint must be a " +
                                                   "string or null");
            }
        }
    }

    /**
     * We are forced to support it by <code>LayoutManager</code>.
     * @param name
     * @param comp
     */
    public void addLayoutComponent(String name, Component comp)
    {
        synchronized (comp.getTreeLock()) {
            if (name == null || BorderLayout.CENTER.equals(name)) {
                center = comp;
            }
            else if (BorderLayout.EAST.equals(name)) {
                east = comp;
            }
            else if (BorderLayout.WEST.equals(name)) {
                west = comp;
            }
            else {
                throw new IllegalArgumentException("cannot add to layout: " +
                                                   "unknown constraint: " +
                                                   name);
            }
        }
    }

    /**
     * Removes the specified component from this layout.
     * @param component
     */
    public void removeLayoutComponent(Component component)
    {
        synchronized (component.getTreeLock()) {
            if (component == center) {
                center = null;
            }
            else if (component == east) {
                east = null;
            }
            else if (component == west) {
                west = null;
            }
        }
    }

    /**
     * Determines the minimum size of the target.
     * @param target
     * @return
     */
    public Dimension minimumLayoutSize(Container target)
    {
        synchronized (target.getTreeLock()) {
            Dimension d = new Dimension(0, 0);

            addMinimumSize(d, east);
            addMinimumSize(d, west);
            addMinimumSize(d, center);

            Insets insets = target.getInsets();
            d.width += insets.left + insets.right;
            d.height += insets.top + insets.bottom;

            return(d);
        }
    }

    /**
     * Determines the preferred size of the target.
     * @return
     * @param target
     */
    public Dimension preferredLayoutSize(Container target)
    {
        synchronized (target.getTreeLock()) {
            Dimension d = new Dimension(0, 0);

            addPreferredSize(d, east);
            addPreferredSize(d, west);
            addPreferredSize(d, center);

            Insets insets = target.getInsets();
            d.width += insets.left + insets.right;
            d.height += insets.top + insets.bottom;

            return(d);
        }
    }

    /**
     * Determines the maximum size of the target.
     * @param target
     * @return
     */
    public Dimension maximumLayoutSize(Container target)
    {
        return(new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE));
    }

    /**
     * Returns the alignment along the <i>X</i> axis.
     * @return alignment along <i>X</i> axis
     * @param parent
     */
    public float getLayoutAlignmentX(Container parent)
    {
        return(0.5f);
    }

    /**
     * Returns the alignment along the <i>Y</i> axis.
     * @return alignment along <i>Y</i> axis
     * @param parent
     */
    public float getLayoutAlignmentY(Container parent)
    {
        return(0.5f);
    }

    /**
     * Invalidates the layout, indicating that if the layout manager
     * has cached information it should be discarded.
     * @param target
     */
    public void invalidateLayout(Container target) {}

    /**
     * Lays out the target argument using this layout.
     * @param target
     */
    public void layoutContainer(Container target)
    {
        synchronized (target.getTreeLock()) {
            Insets insets = target.getInsets();
            int top = insets.top;
            //	int bottom = target.getHeight() - insets.bottom;
            int bottom = target.getBounds().height - insets.bottom;
            int left = insets.left;
            //	int right = target.getWidth() - insets.right;
            int right = target.getBounds().width - insets.right;

            int leftCenter = (right - left) / 2;
            int rightCenter = leftCenter;

            if (center != null) {
                Dimension d = center.getPreferredSize();
                leftCenter = (right - left - d.width) / 2;
                rightCenter = leftCenter + d.width;
                center.setBounds(leftCenter, top, d.width, bottom - top);
            }
            if (west != null) {
                west.setBounds(left, top, leftCenter - left - hgap,
                               bottom - top);
            }
            if (east != null) {
                east.setBounds(rightCenter + hgap, top,
                               right - rightCenter - 2 * hgap, bottom - top);
            }
        }
    }

    private void addMinimumSize(Dimension d, Component c)
    {
        if (c != null) {
            addSize(d, c.getMinimumSize());
        }
    }

    private void addPreferredSize(Dimension d, Component c)
    {
        if (c != null) {
            addSize(d, c.getPreferredSize());
        }
    }

    private void addSize(Dimension d, Dimension size)
    {
        d.width += size.width + hgap;
        d.height = Math.max(size.height, d.height);
    }

    /**
     * Returns a String representation of the state of this layout.
     * @return String representation of the state of this layout
     */
    public String toString()
    {
        return(getClass().getName() + "[hgap=" + hgap + "]");
    }

} // JCFixedCenterLayout

// dummy class to handle Look and Feel and proper serialization
protected class MultiSelectListUI extends ComponentUI {
}

}
