/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCPopupCalendar.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:28 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.swing;

import com.klg.jclass.swing.plaf.*;

import com.klg.jclass.util.JCEnvironment;
import com.klg.jclass.util.JCListenerList;
import com.klg.jclass.util.calendar.JCDateChooser;
import com.klg.jclass.util.swing.JCCheckBoxList;
import com.klg.jclass.util.swing.JCExitFrame;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.plaf.ComboBoxUI;
import javax.swing.text.DateFormatter;
import javax.swing.text.DefaultFormatterFactory;
import java.awt.*;
import java.awt.event.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.Serializable;
import java.text.DateFormat;
import java.text.ParseException;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

/**
 * <CODE>JCPopupCalendar</CODE> is subclassed from <CODE>JCComboBox</CODE>.
 * It uses a model with a single Date object in conjunction with a calendar
 * popup.
 *
 * <b>Note:</b> This component can only be used with JDK 1.4 or later.
 */
public class JCPopupCalendar extends JComboBox
			                 implements KeyListener,
                                        PropertyChangeListener,
                                        DocumentListener,
                                        FocusListener
{

/** Valid value for <code>CalendarType</code>. */
public static final int DATE_TIME = 1;

/** Valid value for <code>CalendarType</code>. */
public static final int DATE = 2;


/** The type of calendar - either Date_TIME or DATE. */
protected int                    calendarType = DATE_TIME;

/** The current popup editor. */
protected JCPopupCalendarEditor  popupEditor = null;

/**
 * If true, and the user presses up or down arrow while in the editor, the
 * calendar popup appears.
 */
protected boolean                showPopupOnUpDownArrow = true;

/** The current model. */
protected DefaultComboBoxModel   model = null;

/** An object which stores the minimum value and handles comparisons. */
private Bounds                   minimum = new Bounds();

/** An object which stores the maximum value and handles comparisons. */
private Bounds                   maximum = new Bounds();

private boolean                  editorChanged = false;
private Border                   editorUIBorder;

/**
 * Constructs a <code>JCPopupCalendar</code> of type DATE_TIME with the current
 * date and time and the default locale.
 */
public JCPopupCalendar()
{
    this(DATE_TIME, new Date(), Locale.getDefault());
}

/**
 * Constructs a <code>JCPopupCalendar</code> of the supplied calendarType with
 * the current date and the default locale.
 * @param calendarType The calendar type of the JCPopupCalendar.  Must be one
 * of JCPopupCalendar.DATE_TIME or JCPopupCalendar.DATE.
 */
public JCPopupCalendar(int calendarType)
{
    this(calendarType, new Date(), Locale.getDefault());
}

/**
 * Constructs a <code>JCPopupCalendar</code> of type DATE_TIME with the supplied
 * date and the default locale.
 * @param d The date to use for the calendar.  If null, the current date and
 * time is used.
 */
public JCPopupCalendar(Date d)
{
    this(DATE_TIME, d, Locale.getDefault());
}

/**
 * Constructs a <code>JCPopupCalendar</code> of the supplied calendarType with
 * the supplied date and the default locale.
 * @param calendarType The calendar type of the JCPopupCalendar.  Must be one
 * of JCPopupCalendar.DATE_TIME or JCPopupCalendar.DATE.
 * @param d The date to use for the calendar.  If null, the current date and
 * time is used.
 */
public JCPopupCalendar(int calendarType, Date d)
{
    this(calendarType, d, Locale.getDefault());
}

/**
 * Constructs a <code>JCPopupCalendar</code> of type DATE_TIME with the supplied
 * date and locale.
 * @param d The date to use for the calendar.  If null, the current date and
 * time is used.
 * @param l The locale to use. If null, the default locale is used.
 */
public JCPopupCalendar(Date d, Locale l)
{
    this(DATE_TIME, d, l);
}

/**
 * Constructs a <code>JCPopupCalendar</code> with the supplied date and locale.
 * @param calendarType The calendar type of the JCPopupCalendar.  Must be one
 * of JCPopupCalendar.DATE_TIME or JCPopupCalendar.DATE.
 * @param d The date to use for the calendar.  If null, the current date and
 * time is used.
 * @param l The locale to use If null, the default locale is used.
 */
public JCPopupCalendar(int calendarType, Date d, Locale l)
{
    if (JCEnvironment.getJavaVersion() < 140) {
        throw new IllegalStateException("JCPopupCalendar is incompatible " +
                                        "with JDKs prior to 1.4.0");
    }
    setCalendarType(calendarType);
    if (l == null) {
        l = Locale.getDefault();
    }
    setLocale(l);
    Calendar c = Calendar.getInstance(getLocale());
    if (d == null) {
        d = new Date();
    }
    c.setTime(d);
    setPopupEditor(new DateTimePopup(this, JCDateChooser.DUAL_SPIN,
                                     calendarType == DATE_TIME, false,
                                     getLocale(), c));
    model = new DefaultComboBoxModel(new Object[]{d});
    setModel(model);
    setEditor(new DateEditor(this, getCalendarType(), d, getLocale()));
    setRenderer(new DateRenderer(this, getCalendarType(), d, getLocale()));
    addKeyListener(this);
}

/**
 * Return the appropriate ComboBoxUI object based on the Look and Feel.
 * @return A ComboBoxUI based on the L&F
 */
protected ComboBoxUI inferLookAndFeelUI()
{
	ComboBoxUI currentUI = null;
	String lf = UIManager.getLookAndFeel().getID();
	if (lf.equals("Windows")) {
		currentUI = new WindowsCalendarPopupUI(null);
	}
	else if (lf.equals("Motif")) {
        currentUI = new MotifCalendarPopupUI(null);
	}
	else {
        currentUI = new MetalCalendarPopupUI(null);
	}
    if (currentUI instanceof CalendarPopupUI) {
        CalendarPopupUI cui = (CalendarPopupUI)currentUI;
        Component e = cui.getEditor().getEditorComponent();
        if (e instanceof JComponent) {
            editorUIBorder = ((JComponent)e).getBorder();
        }
    }
	return(currentUI);
}

/**
 * Sets the popup editor.
 * @param popup The new popup editor
 */
public void setPopupEditor(JCPopupCalendarEditor popup)
{
	if (popupEditor != popup) {

		popupEditor = popup;
        if (popupEditor != null) {
            popupEditor.setLocale(getLocale());
            popupEditor.setMinimumDate(minimum.getValue());
            popupEditor.setMaximumDate(maximum.getValue());
        }

		if (JCEnvironment.isWindows()) {
			((WindowsCalendarPopupUI)ui).setPopupCalendarEditor(popup);
		}
		else if (JCEnvironment.isMotif()) {
			((MotifCalendarPopupUI)ui).setPopupCalendarEditor(popup);
		}
		else { // Metal
			((MetalCalendarPopupUI)ui).setPopupCalendarEditor(popup);
		}
	}
}

/**
 * Returns the popup editor.
 * @return The current popup editor
 */
public JCPopupCalendarEditor getPopupEditor()
{
	return(popupEditor);
}

/**
 * Sets the value on the model as a Date object.
 * @param d The new date value.
 */
public void setValue(Date d)
{
    setValue(d, true);
}

/**
 * Sets the value on the model as a Date object.
 * @param d The new date value.
 * @param updatePopupEditor If true, pass the new value on to the popup editor.
 */
protected void setValue(Date d, boolean updatePopupEditor)
{
    if (d == null) {
        // Set to current time
        d = new Date();
    }
    updateComboBoxModel(d);
    updateComboBoxEditor(d);
    updateComboBoxRenderer();
    if (updatePopupEditor && popupEditor != null) {
        popupEditor.setValue(d);
    }
}

/**
 * Return the model's current value (stored as a Date object).
 * @return The current value
 */
public Date getValue()
{
    Date d = (Date)model.getElementAt(0);
    return((Date)d.clone());
}

/**
 * Returns the minimum date. The popup calendar's date value cannot change
 * to before this minimum date.
 * @return The current minimum date.
 */
public Date getMinimumDate()
{
    Calendar min = minimum.getValue();
    Date d = null;
    if (min != null) {
        d = min.getTime();
    }
    return(d);
}

/**
 * Sets the minimum date. The popup calendar's date value cannot change
 * to before this minimum date.
 * @param minDate The new minimum date.
 */
public void setMinimumDate(Date minDate)
{
    Calendar min = null;
    if (minDate != null) {
        min = Calendar.getInstance();
        min.setTime(minDate);
    }
    minimum.setValue(min);
    if (popupEditor != null) {
        popupEditor.setMinimumDate(min);
    }
}

/**
 * Returns the maximum date. The popup calendar's date value cannot change
 * to after this maximum date.
 * @return The current maximum date.
 */
public Date getMaximumDate()
{
    Calendar max = maximum.getValue();
    Date d = null;
    if (max != null) {
        d = max.getTime();
    }
    return(d);
}

/**
 * Sets the maximum date. The popup calendar's date value cannot change
 * to after this maximum date.
 * @param maxDate The new maximum date.
 */
public void setMaximumDate(Date maxDate)
{
    Calendar max = null;
    if (maxDate != null) {
        max = Calendar.getInstance();
        max.setTime(maxDate);
    }
    maximum.setValue(max);
    if (popupEditor != null) {
        popupEditor.setMaximumDate(max);
    }
}

/**
 * Get the calendar type.  Will be one of JCPopupCalendar.DATE_TIME which means
 * both the date and the time are shown or JCPopupCalendar.DATE which means only
 * the date is shown.
 * @return The current calendar type
 */
public int getCalendarType()
{
    return(calendarType);
}

/**
 * Set the calendar type.  Must be one of JCPopupCalendar.DATE_TIME which means
 * both the date and the time are shown or JCPopupCalendar.DATE which means only
 * the date is shown.
 * @param calendarType The new calendarType. Must be one of
 * JCPopupCalendar.DATE_TIME or JCPopupCalendar.DATE
 */
public void setCalendarType(int calendarType)
{
    if (calendarType != DATE_TIME && calendarType != DATE) {
        throw new IllegalArgumentException("Bad calendar type.  Must be one " +
                                           "of JCPopupCalendar.DATE_TIME or " +
                                           "JCPopupCalendar.DATE");
    }
    if (this.calendarType == calendarType) {
        return;
    }
    this.calendarType = calendarType;

    if (popupEditor != null) {
        popupEditor.setShowTimeComponent(calendarType == DATE_TIME);
    }

    ComboBoxEditor e = getEditor();
    if (e instanceof DateEditor) {
        DateEditor de = (DateEditor)e;
        de.setCalendarType(calendarType);
    }
    ListCellRenderer r = getRenderer();
    if (r instanceof DateRenderer) {
        DateRenderer dr = (DateRenderer)r;
        dr.setCalendarType(calendarType);
    }
}

/**
 * Get whether the popup should appear when the user presses up or down arrow
 * while in the editor.
 * @return The current value of isShowPopupOnUpDownArrow.
 */
public boolean getShowPopupOnUpDownArrow()
{
    return(showPopupOnUpDownArrow);
}

/**
 * Set whether the popup should appear when the user presses up or down arrow
 * while in the editor.
 * @param showPopupOnUpDownArrow If true, and the user presses up or down arrow
 * while in the editor, the calendar popup appears.
 */
public void setShowPopupOnUpDownArrow(boolean showPopupOnUpDownArrow)
{
    this.showPopupOnUpDownArrow = showPopupOnUpDownArrow;
}

/**
 * Get whether the popup will popdown when the user selects a day on the
 * day table.
 * @return The current value of hidePopupOnDayTableClick.
 */
public boolean getHidePopupOnDayTableClick()
{
    if (popupEditor != null) {
        return(popupEditor.getHidePopupOnDayTableClick());
    }
    return(false);
}

/**
 * Set whether the popup will popdown when the user selects a day on the
 * day table.
 * @param hidePopupOnDayTableClick The new value of hidePopupOnDayTableClick. If
 * true, the popup will popdown when the user selects a day on the day table.
 */
public void setHidePopupOnDayTableClick(boolean hidePopupOnDayTableClick)
{
    if (popupEditor != null) {
        popupEditor.setHidePopupOnDayTableClick(hidePopupOnDayTableClick);
    }
}

/**
 * Get whether an apply button be shown in the popup.  If an apply button is
 * shown, clicking on the button applies the value to the model and hides the
 * popup.
 * @return The current value of showApplyButton.
 */
public boolean getShowApplyButton()
{
    if (popupEditor != null) {
        return(popupEditor.getShowApplyButton());
    }
    return(false);
}

/**
 * Set whether an apply button be shown in the popup.  If an apply button is
 * shown, clicking on the button applies the value to the model and hides the
 * popup.
 * @param showApplyButton The new value of showApplyButton.  If true, an apply
 * button is shown in the popup.
 */
public void setShowApplyButton(boolean showApplyButton)
{
    if (popupEditor != null) {
        popupEditor.setShowApplyButton(showApplyButton);
    }
}

//
// KeyListener Implementation
//

/**
 * Invoked when a key has been typed.
 * See the class description for {@link KeyEvent} for a definition of
 * a key typed event.
 * @param e The event generated by the key being typed
 */
public void keyTyped(KeyEvent e) {}

/**
 * Invoked when a key has been pressed.
 * See the class description for {@link KeyEvent} for a definition of
 * a key pressed event.
 * @param e The event generated by the keyPress
 */
public void keyPressed(KeyEvent e)
{
    editorChanged = false;
    int keyCode = e.getKeyCode();
    
    Component editor = getEditor().getEditorComponent();
    if (editor != null && e.getSource() == editor &&
        (keyCode == KeyEvent.VK_UP || keyCode == KeyEvent.VK_DOWN))
    {
        editorChanged = true;
        if (showPopupOnUpDownArrow && !isPopupVisible()) {
            showPopup();
        }
    }
    if (keyCode == KeyEvent.VK_ESCAPE && isPopupVisible()) {
        ComboBoxUI ui = getUI();
        if (ui instanceof CalendarPopupUI) {
            CalendarPopup popup = ((CalendarPopupUI)ui).getPopup();
            Date originalDate = popup.getOriginalDate();
            if (originalDate != null) {
                setValue(originalDate);
                hidePopup();
            }
        }
    }
}

/**
 * Invoked when a key has been released.
 * See the class description for {@link KeyEvent} for a definition of
 * a key released event.
 * @param e The event generated by the keyRelease
 */
public void keyReleased(KeyEvent e) {}

//
// PropertyChangeListener Implementation
//

/**
 * This method gets called when a bound property is changed.
 * @param e A PropertyChangeEvent object describing the event source and the
 * property that has changed.
 */
public void propertyChange(PropertyChangeEvent e)
{
    
    if ("value".equals(e.getPropertyName())) {
        Date d = extractCurrentEditorValue();
        if (d != null) {
            updatePopupEditor(d);
            updateComboBoxModel(d);
        }
    }
}

//
// FocusListener Implementation
//

/**
 * Invoked when a component gains the keyboard focus.
 */
public void focusGained(FocusEvent e) {}

/**
 * Invoked when a component loses the keyboard focus.  Sometimes the editor's
 * value gets reset when the focus changes.  Make sure it reflects the current
 * model's value.
 */
public void focusLost(FocusEvent e)
{
    
    Date dateObject = (Date)model.getElementAt(0);
    updateComboBoxEditor(dateObject);
}

//
// DocumentListener Implementation
//

/**
 * Gives notification that there was an insert into the document.  The
 * range given by the DocumentEvent bounds the freshly inserted region.
 *
 * @param event The document event
 */
public void insertUpdate(DocumentEvent event)
{
    
    if (editorChanged) {
        Date d = extractCurrentEditorValue();
        
        if (d != null) {
            updatePopupEditor(d);
            updateComboBoxModel(d);
        }
    }
    editorChanged = false;
}

/**
 * Gives notification that a portion of the document has been
 * removed.  The range is given in terms of what the view last
 * saw (that is, before updating sticky positions).
 *
 * @param e The document event
 */
public void removeUpdate(DocumentEvent e) {}

/**
 * Gives notification that an attribute or set of attributes changed.
 *
 * @param e The document event
 */
public void changedUpdate(DocumentEvent e) {}

/**
 * This returns the currently being edited value (possibly before a commit takes
 * place) of the combo box editor.  This may or may not be equal to the value
 * in the formatted text field.
 * @return The current editor value
 */
protected Date extractCurrentEditorValue()
{
    ComboBoxEditor curEditor = getEditor();
    Date d = null;
    if (curEditor != null) {
        Component editorComponent = curEditor.getEditorComponent();
        if (editorComponent instanceof JFormattedTextField) {
            JFormattedTextField ftf = (JFormattedTextField)editorComponent;
            DateFormatter df = (DateFormatter)ftf.getFormatter();
            Object o = null;
            try {
                o = df.stringToValue(ftf.getText());
            }
            catch (ParseException e) {}
            if (o instanceof Date) {
                d = (Date)o;
            }
        }
    }
    return(d);
}

/**
 * Update the combo box editor with a new date.
 * @param d The new date
 */
protected void updateComboBoxEditor(Date d)
{
    if (!isEditable()) {
        return;
    }
    ComboBoxEditor curEditor = getEditor();
    if (curEditor != null && d != null) {
        curEditor.setItem(d);
    }
}

/**
 * Update the combo box renderer (this gets done when the popup calendar).
 */
protected void updateComboBoxRenderer()
{
    if (isEditable()) {
        return;
    }
    repaint();
}

/**
 * Update the combo box model with a new date.
 * @param d The new date
 */
protected void updateComboBoxModel(Date d)
{
    if (model != null && d != null) {
        Date dateObject = (Date)model.getElementAt(0);
        dateObject.setTime(d.getTime());
    }
}

/**
 * Update the popup editor with a new date.
 * @param d The new date
 */
protected void updatePopupEditor(Date d)
{
    if (popupEditor != null && d != null) {
        popupEditor.setValue(d);
    }
}

/**
 * Sets the look and feel for this component.
 * @param ui The new Look and Feel object.
 */
public void setUI(ComboBoxUI ui)
{
    super.setUI(inferLookAndFeelUI());
}

/**
 * Resets the UI property to a value from the current look and feel.  Note that
 * if you are using a popup editor that is not the default, you will
 * need to recreate your popup editor (with the new L&F) and reset it on popup
 * calendar.
 */
public void updateUI()
{
	setUI(null);
    if (popupEditor != null) {
        Font userFont = null;
        int chooserType = JCDateChooser.DUAL_SPIN;
        JComponent jComp = popupEditor.getEditorComponent();
        if (jComp instanceof DateTimeChooser) {
            DateTimeChooser chooser = (DateTimeChooser)jComp;
            userFont = chooser.getUserFont();
            chooserType = chooser.getChooserType();
        }
        Calendar c = Calendar.getInstance(getLocale());
        c.setTime(getValue());
        DateTimePopup newDTPopup = new DateTimePopup(this, chooserType,
                                                     calendarType == DATE_TIME,
                                                     getShowApplyButton(),
                                                     getLocale(), c);

        // If the old popupEditor has a list of listeners, make sure they get
        // passed on to the new popupEditor.
        JCListenerList list = null;
        if (popupEditor instanceof DateTimePopup) {
            DateTimePopup oldDTPopup = (DateTimePopup)popupEditor;
            list = oldDTPopup.popupListeners;
            oldDTPopup.popupListeners = null;
        }
        newDTPopup.popupListeners = list;

        // Pass any user set fonts to the new popupEditor
        if (userFont != null) {
            newDTPopup.setFont(userFont);
        }

        setPopupEditor(newDTPopup);
    }
}

/**
 * Sets the locale being used by the popup calendar, which will set it on the
 * popup editor and its child components, the editor, and the renderer.
 * @param l The new locale to be used.
 */
public void setLocale(Locale l)
{
	super.setLocale(l);
	if (popupEditor != null) {
		popupEditor.setLocale(l);
	}
    ComboBoxEditor e = getEditor();
    if (e instanceof DateEditor) {
        DateEditor de = (DateEditor)e;
        de.setLocale(l);
    }
    ListCellRenderer r = getRenderer();
    if (r instanceof DateRenderer) {
        DateRenderer dr = (DateRenderer)r;
        dr.setLocale(l);
    }
}

/**
 * Sets the editor used to paint and edit the selected item in the
 * <code>JCPopupCalendar</code>.  The editor is used only if the
 * receiving <code>JCPopupCalendar</code> is editable. If not editable,
 * the popup calendar uses the renderer to paint the selected item.
 *
 * @param anEditor The <code>ComboBoxEditor</code> that displays the current
 * value
 * @see #setRenderer
 */
public void setEditor(ComboBoxEditor anEditor)
{
    ComboBoxEditor oldEditor = getEditor();
    if (oldEditor != null) {
        Component editorComponent = oldEditor.getEditorComponent();
        editorComponent.removeKeyListener(this);
        editorComponent.removePropertyChangeListener(this);
        editorComponent.removeFocusListener(this);
        if (editorComponent instanceof JTextField) {
            JTextField tf = (JTextField)editorComponent;
            tf.getDocument().removeDocumentListener(this);
        }
    }
    super.setEditor(anEditor);
    ComboBoxEditor newEditor = getEditor();
    if (newEditor != null) {
        Component editorComponent = newEditor.getEditorComponent();
        editorComponent.addKeyListener(this);
        editorComponent.addPropertyChangeListener(this);
        editorComponent.addFocusListener(this);
        if (editorComponent instanceof JTextField) {
            JTextField tf = (JTextField)editorComponent;
            tf.getDocument().addDocumentListener(this);
        }
    }
}

/**
 * Given a calendar type and a locale, return the default date format that
 * JCPopupCalendar uses.
 * @param type A calendar type.  Must be one of JCPopupCalendar.DATE_TIME or
 * JCPopupCalendar.DATE.
 * @param l The locale.  If null, the default locale is used.
 * @return A DateFormat object based on the type and locale.
 */
public static DateFormat getDefaultDateFormat(int type, Locale l)
{
    if (type != DATE_TIME && type != DATE) {
        throw new IllegalArgumentException("Bad calendar type.  Must be one " +
                                           "of JCPopupCalendar.DATE_TIME or " +
                                           "JCPopupCalendar.DATE");
    }
    if (l == null) {
        l = Locale.getDefault();
    }
    DateFormat df = null;
    if (type == DATE) {
        df = DateFormat.getDateInstance(DateFormat.MEDIUM, l);
    }
    // type == DATE_TIME
    else {
        df = DateFormat.getDateTimeInstance(DateFormat.MEDIUM,
                                            DateFormat.MEDIUM,
                                            l);
    }
    return(df);
}

/**
 * Use for testing to provide a simple popup calendar within a frame.
 * @param args Command line args (ignored)
 */
public static void main(String[] args)
{
//    String tags[] = {"popupCalendar"};
//    
//    

    if (args.length > 0) {
        if (args[0].equals("windows")) {
            try {
                String s = "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
                UIManager.setLookAndFeel(s);
            }
            catch (Exception e) {}
        }
        if (args[0].equals("motif")) {
            try {
                String s = "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
                UIManager.setLookAndFeel(s);
            }
            catch (Exception e) {}
        }
    }
    JCExitFrame frame = new JCExitFrame("JCPopupCalendar");
    Locale locale = new Locale("fr", "FR");
    final JCPopupCalendar t = new JCPopupCalendar(DATE_TIME, new Date(),
                                                  locale);
    t.setEditable(true);
    Component editor = t.getEditor().getEditorComponent();
    ((JTextField)editor).setColumns(20);

    // Create minimum date: today's date but 10 years ago
    Calendar min = Calendar.getInstance();
    int year = min.get(Calendar.YEAR);
    min.set(Calendar.YEAR, year - 2);
    t.setMinimumDate(min.getTime());

    // Create maximum date: today's date but 10 years from now
    Calendar max = Calendar.getInstance();
    max.set(Calendar.YEAR, year + 2);
    t.setMaximumDate(max.getTime());

    // Create a control to allow changing of the popup calendar's locale
    final String types[] = {"English (US)", "French", "Spanish", "Russian"};
    final Locale locales[] = {new Locale("en", "US"), new Locale("fr", "FR"),
                              new Locale("es", "ES"), new Locale("ru", "RU")};
    final JCCheckBoxList cb = new JCCheckBoxList(types);
    cb.setSelectedIndex(1);
    cb.addListSelectionListener(new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent e)
        {
            t.setLocale(locales[cb.getSelectedIndex()]);
        }
    });

    // Create a control to allow changing of the popup calendar's type
    JButton b = new JButton("Switch types");
    b.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e)
        {
            t.setCalendarType(3 - t.getCalendarType());
        }
    });

    Container c = frame.getContentPane();
    c.setLayout(new BorderLayout());
    c.add(t, BorderLayout.NORTH);
    c.add(cb, BorderLayout.CENTER);
    c.add(b, BorderLayout.SOUTH);
    frame.pack();
    frame.show();
}

/**
 * The default editor for the JCPopupCalendar.
 */
public class DateEditor implements ComboBoxEditor, Serializable
{
    /** The parent popup calendar. */
    protected JCPopupCalendar      popupCalendar;

    /** The editor component. */
    protected JFormattedTextField  editor;

    /** The type of calendar - either Date_TIME or DATE. */
    protected int                  calendarType = DATE_TIME;

    /**
     * Create a DateEditor given its parent, the type of calendar, and an
     * initial date.
     * @param popupCalendar The parent popup calendar
     * @param calendarType The type of calendar.  Must be one of
     * JCPopupCalendar.DATE_TIME or JCPopupCalendar.DATE
     * @param d The initial date for the editor
     */
    public DateEditor(JCPopupCalendar popupCalendar, int calendarType, Date d)
    {
        this(popupCalendar, calendarType, d, null);
    }

    /**
     * Create a DateEditor given its parent, the type of calendar, an
     * initial date, and a locale.
     * @param popupCalendar The parent popup calendar
     * @param calendarType The type of calendar.  Must be one of
     * JCPopupCalendar.DATE_TIME or JCPopupCalendar.DATE
     * @param d The initial date for the editor
     * @param l The locale to use
     */
    public DateEditor(JCPopupCalendar popupCalendar, int calendarType, Date d,
                      Locale l)
    {
        this.popupCalendar = popupCalendar;
        this.calendarType = calendarType;
        editor = new JFormattedTextField();
        if (d == null) {
            d = new Date();
        }
        editor.setValue(d);
        if (l == null) {
            l = Locale.getDefault();
        }
        setLocale(l);
        editor.setFocusLostBehavior(JFormattedTextField.COMMIT_OR_REVERT);
        editor.setBorder(editorUIBorder);
    }

    /**
     * Return the editor's JCPopupCalendar parent.
     * @return The parent
     */
    public JCPopupCalendar getPopupCalendar()
    {
        return(popupCalendar);
    }

    /**
     * Get the calendar type for the editor.  Will be one of
     * JCPopupCalendar.DATE_TIME which means both the date and the time are
     * shown or JCPopupCalendar.DATE which means only the date is shown.
     * @return The current calendar type
     */
    public int getCalendarType()
    {
        return(calendarType);
    }

    /**
     * Set the calendarType on the editor.  This method will also update the
     * format of the editor to the default one for the new calendarType.
     * @param calendarType The new calendarType.  Must be one of
     * JCPopupCalendar.DATE_TIME or JCPopupCalendar.DATE
     */
    public void setCalendarType(int calendarType)
    {
        if (calendarType != DATE_TIME && calendarType != DATE) {
            throw new IllegalArgumentException("Bad calendar type.  " +
                                               "Must be one of " +
                                               "JCPopupCalendar.DATE_TIME or " +
                                               "JCPopupCalendar.DATE");
        }
        if (this.calendarType == calendarType) {
            return;
        }
        this.calendarType = calendarType;
        setFormat(JCPopupCalendar.getDefaultDateFormat(calendarType,
                                                       editor.getLocale()));
    }

    /**
     * Set the locale on the editor.  This method will also update the format
     * of the editor to the default one for the new locale.
     * @param locale The new locale
     */
    public void setLocale(Locale locale)
    {
        if (locale == null) {
            locale = Locale.getDefault();
        }
        editor.setLocale(locale);
        if (popupCalendar != null) {
            setFormat(JCPopupCalendar.getDefaultDateFormat(calendarType,
                                                           editor.getLocale()));
        }
    }

    /**
     * The format to use for the editor.
     * @param df The new format
     */
    public void setFormat(DateFormat df)
    {
        DateFormatter formatter = new DateFormatter(df);
        formatter.setValueClass(Date.class);
        formatter.setMinimum(minimum);
        formatter.setMaximum(maximum);
        DefaultFormatterFactory factory =
            new DefaultFormatterFactory(formatter);
        editor.setFormatterFactory(factory);
    }

    //
    // ComboBoxEditor implementation
    //

    /**
     * Return the component that should be added to the tree hierarchy for
     * this editor.
     * @return The editing component which is JFormattedTextField
     */
    public Component getEditorComponent()
    {
        return(editor);
    }

    /**
     * Set the item that should be edited. Cancel any editing if necessary.
     * @param anObject The new item to be edited.  It must be null or of type
     * Date
     **/
    public void setItem(Object anObject)
    {
        if (anObject == null) {
            editor.setValue(new Date());
        }
        else if (anObject instanceof Date) {
            editor.setValue(anObject);
        }
        else {
            throw new IllegalArgumentException("Object must be of type Date");
        }
    }

    /**
     * Return the edited item which will be of type Date.
     * @return The edited item
     **/
    public Object getItem()
    {
        return(editor.getValue());
    }

    /**
     * Ask the editor to start editing and to select everything.
     **/
    public void selectAll()
    {
        editor.selectAll();
        editor.requestFocus();
    }

    /**
     * Add an ActionListener. An action event is generated when the edited item
     * changes
     * @param l The action listener to be added
     **/
    public void addActionListener(ActionListener l)
    {
        editor.addActionListener(l);
    }

    /**
     * Remove an ActionListener.
     * @param l The action listener to be removed
     **/
    public void removeActionListener(ActionListener l)
    {
        editor.removeActionListener(l);
    }
}

/**
 * The default renderer for the JCPopupCalendar.
 */
public class DateRenderer extends DefaultListCellRenderer
{
    /** The parent popup calendar. */
    protected JCPopupCalendar  popupCalendar;

    /** The type of calendar - either Date_TIME or DATE. */
    protected int              calendarType = DATE_TIME;

    /** The format object to format dates. */
    protected DateFormat       format;

    /**
     * Create a DateRenderer given its parent, the type of calendar, and an
     * initial date.
     * @param popupCalendar The parent popup calendar
     * @param calendarType The type of calendar.  Must be one of
     * JCPopupCalendar.DATE_TIME or JCPopupCalendar.DATE
     * @param d The initial date for the editor
     */
    public DateRenderer(JCPopupCalendar popupCalendar, int calendarType, Date d)
    {
        this(popupCalendar, calendarType, d, null);
    }

    /**
     * Create a DateRenderer given its parent, the type of calendar, an
     * initial date, and a locale.
     * @param popupCalendar The parent popup calendar.
     * @param calendarType The type of calendar.  Must be one of
     * JCPopupCalendar.DATE_TIME or JCPopupCalendar.DATE
     * @param d The initial date for the editor.
     * @param l The locale to use.
     */
    public DateRenderer(JCPopupCalendar popupCalendar, int calendarType, Date d,
                        Locale l)
    {
        super();
        this.popupCalendar = popupCalendar;
        this.calendarType = calendarType;
        if (d == null) {
            d = new Date();
        }
        if (l == null) {
            l = Locale.getDefault();
        }
        setLocale(l);
    }

    /**
     * Return the renderer's JCPopupCalendar parent.
     * @return The parent
     */
    public JCPopupCalendar getPopupCalendar()
    {
        return(popupCalendar);
    }

    /**
     * Get the calendar type for the renderer.  Will be one of
     * JCPopupCalendar.DATE_TIME which means both the date and the time are
     * shown or JCPopupCalendar.DATE which means only the date is shown.
     * @return The current calendar type
     */
    public int getCalendarType()
    {
        return(calendarType);
    }

    /**
     * Set the calendarType on the renderer.  This method will also update the
     * format of the renderer to the default one for the new calendarType.
     * @param calendarType The new calendarType.  Must be one of
     * JCPopupCalendar.DATE_TIME or JCPopupCalendar.DATE
     */
    public void setCalendarType(int calendarType)
    {
        if (calendarType != DATE_TIME && calendarType != DATE) {
            throw new IllegalArgumentException("Bad calendar type.  " +
                                               "Must be one of " +
                                               "JCPopupCalendar.DATE_TIME or " +
                                               "JCPopupCalendar.DATE");
        }
        if (this.calendarType == calendarType) {
            return;
        }
        this.calendarType = calendarType;
        setFormat(JCPopupCalendar.getDefaultDateFormat(calendarType,
                                                       getLocale()));
    }

    /**
     * Set the locale on the renderer.  This method will also update the format
     * of the renderer to the default one for the new locale.
     * @param locale The new locale
     */
    public void setLocale(Locale locale)
    {
        if (locale == null) {
            locale = Locale.getDefault();
        }
        super.setLocale(locale);
        if (popupCalendar != null) {
            setFormat(JCPopupCalendar.getDefaultDateFormat(calendarType,
                                                           getLocale()));
        }
    }

    /**
     * Return the format used by the renderer.
     * @return The current formatter
     */
    public DateFormat getFormat()
    {
        return(format);
    }

    /**
     * Set the format to use for the renderer.
     * @param df The new format
     */
    public void setFormat(DateFormat df)
    {
        if (df == null) {
            throw new IllegalArgumentException("Format cannot be null");
        }
        this.format = df;
        Date d = (Date)popupCalendar.getModel().getElementAt(0);
        setText(format.format(d));
        if (popupCalendar.isShowing()) {
            popupCalendar.repaint();
        }
    }


    /**
     * Return a component that has been configured to display the specified
     * value. That component's <code>paint</code> method is then called to
     * "render" the cell.  If it is necessary to compute the dimensions
     * of a list because the list cells do not have a fixed size, this method
     * is called to generate a component on which <code>getPreferredSize</code>
     * can be invoked.
     *
     * @param list The JList we're painting.
     * @param value The value returned by list.getModel().getElementAt(index).
     * @param index The cells index.
     * @param isSelected True if the specified cell was selected.
     * @param cellHasFocus True if the specified cell has the focus.
     * @return A component whose paint() method will render the specified value.
     *
     * @see JList
     * @see ListSelectionModel
     * @see ListModel
     */
    public Component getListCellRendererComponent(JList list, Object value,
                                                  int index, boolean isSelected,
                                                  boolean cellHasFocus)
    {
        String s = null;
        if (value instanceof Date) {
            s = format.format((Date)value);
        }
        else if (!(value instanceof Icon)) {
            s = (value == null) ? "" : value.toString();
        }
        Object o = (s == null) ? value : s;
        return(super.getListCellRendererComponent(list, o, index,
                                                  isSelected, cellHasFocus));
    }
}

/**
 * This class is allows the DateEditor to make sure its value falls between
 * any minimum or maximum specified on the popup calendar.
 */
protected class Bounds implements Comparable, Serializable
{

/** The value used for this bounds.  Either the minimum or maximum value. */
protected Calendar value = null;

/**
 * The no args constructor.  Value is null which implies no minimum or maximum.
 */
public Bounds() {}

/**
 * Constructor which specifies a value as minimum or maximum.
 * @param value The minimum or maximum value.
 */
public Bounds(Calendar value)
{
    this.value = value;
}

/**
 * Get the minimum or maximum value.
 * @return The current minimum or maximum value.
 */
public Calendar getValue()
{
    return(value);
}

/**
 * Sets a new minimum or maximum value.
 * @param value The new minimum or maximum value.
 */
public void setValue(Calendar value)
{
    this.value = value;
}

/**
 * Compares this object with the specified object for order.  Returns a
 * negative integer, zero, or a positive integer as this object is less
 * than, equal to, or greater than the specified object.
 * @param o The Object to be compared.
 * @return A negative integer, zero, or a positive integer as this object
 * is less than, equal to, or greater than the specified object.
 * @throws ClassCastException if the object passed in is not a Date object.
 */
public int compareTo(Object o)
{
    Date d = (Date)o;
    if (value == null || d == null) {
        return(0);
    }

    Calendar c = Calendar.getInstance();
    c.setTime(d);

    // The following logic works whether this object is used as a min or a max.
    // min - returnValue of  1 ==> o is invalid
    //                      -1 ==> o is valid
    //     - value before c ==>  o is valid
    //
    // max - returnValue of  1 ==> o is valid
    //                      -1 ==> o is invalid
    //     - value before c ==> o is invalid
    int returnValue = 1;
    if (value.before(c)){
        returnValue = -1;
    }
    return(returnValue);
}

} // Bounds

}
