/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCPopupCalendarEditor.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:28 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.swing;

import com.klg.jclass.util.swing.JCPopupListener;

import javax.swing.*;
import java.awt.*;
import java.util.Locale;

/**
 *  Interface for writing calendar popup editors.
 */
public interface JCPopupCalendarEditor extends java.io.Serializable {

/**
 * Returns the component in the popup calendar editor.
 * @return The component used to do the editing.
 */
public JComponent getEditorComponent();

/**
 * Add a JCPopupListener to the popup to inform when the value in the
 * popup component has been committed.
 * @param l The popup listener to add
 */
public void addPopupListener(JCPopupListener l);

/**
 * Removes a JCPopupListener.
 * @param l The popup listener to remove
 */
public void removePopupListener(JCPopupListener l);

/**
 * Fire a popup event to all listeners.
 */
public void firePopupEvent();

/**
 * Sets the value in the popup editor.
 * @param value The value to set on the popup editor.   This value can be of
 * type Calendar or Date.
 */
public void setValue(Object value);

/**
 * Sets the minimum date. The calendar cannot change to before this date.
 * @param minDate The new minimum date.  This value can be of type Calendar or
 * Date.
 */
public void setMinimumDate(Object minDate);

/**
 * Sets the maximum date. The calendar cannot change to after this date.
 * @param maxDate The new maximum date.  This value can be of type Calendar or
 * Date.
 */
public void setMaximumDate(Object maxDate);

/**
 * Sets the locale being used by the popup.
 * @param locale The new locale to be used
 */
public void setLocale(Locale locale);

/**
 * Sets the <code>Font</code> of the popup.  This font gets propagated to the
 * child components.
 *
 * @param f The new <code>Font</code> for the popup.
 */
public void setFont(Font f);

/**
 * Gets whether the time component should appear or not.
 * @return true, if the time component will appear in the popup.
 */
public boolean getShowTimeComponent();

/**
 * Sets whether the time component should appear or not.
 * @param showTimeComponent If true, the time component will appear.
 */
public void setShowTimeComponent(boolean showTimeComponent);

/**
 * Get whether an apply button be shown in the popup.  If an apply button is
 * shown, clicking on the button applies the value to the model and hides the
 * popup.
 * @return true, if the apply button will appear.
 */
public boolean getShowApplyButton();

/**
 * Set whether an apply button be shown in the popup.  If an apply button is
 * shown, clicking on the button applies the value to the model and hides the
 * popup.
 * @param showApplyButton If true, an apply button is shown in the popup.
 */
public void setShowApplyButton(boolean showApplyButton);

/**
 * Get whether the popup will popdown when the user selects a day on the
 * day table.
 * @return true, if the popup will popdown when the user selects a day on the
 * day table.
 */
public boolean getHidePopupOnDayTableClick();

/**
 * Set whether the popup will popdown when the user selects a day on the
 * day table.
 * @param hidePopupOnDayTableClick If true, the popup will popdown when the
 * user selects a day on the day table.
 */
public void setHidePopupOnDayTableClick(boolean hidePopupOnDayTableClick);

}
