/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCSplashScreen.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:28 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.swing;

import javax.swing.*;
import java.awt.*;
import java.net.URL;

/**
 * Uses an Icon or the location of an image to create an application's
 * introductory screen.
 */
public class JCSplashScreen {

protected Window  dialog = null;
protected Frame   f = null;
protected JLabel  label = null;

/**
 * Creates a splash screen given the location of the image.
 * The image location is the package path of the image and must be in
 * the classpath. For example, if an image was located in
 * <i>/test/examples/image.gif</i>, and the classpath specified contains
 * <i>/test</i>, the constructor should be passed <i>"/examples/image.gif"</i>.
 * @param image_location
 */
public JCSplashScreen(String image_location)
{

    // Get the URL given the image location
    URL url = getClass().getResource(image_location);

    if (url == null) {
        throw new IllegalArgumentException("Image could not be found: " +
                                           image_location);
    }
    // Convert URL to Image icon
    Icon image = new ImageIcon(url);
    initialize(image);
    image = null;
}

/**
 * Creates a splash screen given an Icon image.
 * @param image
 */
public JCSplashScreen(Icon image)
{
    initialize(image);
}

/**
 * Creates the splash screen window and configures it.
 * @param image
 */
protected void initialize(Icon image)
{
    // If no image, return
    if (image == null) {
        throw new IllegalArgumentException("Image specified is invalid.");
    }

    // Create dialog window
    f = new Frame();
    dialog = new Window(f);
    JLabel label = new JLabel(image);
    image = null;
    dialog.add(label);
    dialog.pack();

    // Position splash screen
    Dimension s = dialog.getSize();
    Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
    int x = (screen.width - s.width) / 2;
    if (x < 0) {
        x = 0;
    }

    int y = (screen.height - s.height) / 2;
    if (y < 0) {
        y = 0;
    }

    dialog.setLocation(x, y);
}

/**
 * This method will show or hide the splash screen.  Once the splash
 * screen is hidden, the splash screen window will be disposed. This means
 * the splash screen cannot become visible again.
 * @param show
 */
public void setVisible(boolean show)
{
    if (show == true && dialog != null && f != null && !dialog.isVisible()) {
        dialog.setVisible(true);
    }
    else {
        if (dialog != null) {
            dialog.dispose();
        }
        if (f != null) {
            f.dispose();
        }
        label = null;
        dialog = null;
        f = null;
    }
}

}
