/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCSplitWizard.java,v $Revision: 1.6.2.1 $
//          $Date: 2006-03-17 21:13:28 $ $Locker:  $ Quest Software Inc.
package com.klg.jclass.swing;

import com.klg.jclass.util.JCListenerList;

import javax.swing.*;
import java.awt.*;
import java.util.Enumeration;

/**
 * This class creates a wizard with two panes, and a bottom button
 * panel.  The left panel can optionally contain an automatically
 * generated Steps list or, like the right panel, any component of
 * the developer's choosing.
 */
public class JCSplitWizard extends BaseWizard {


/** The SplitWizardPage that fills this panel **/
protected SplitWizardPage splitWizardPage;
/** The right panel which holds developer content **/
protected JPanel rightPanel;

    
/**
 * Default constructor which creates a SplitWizard, with 
 * standard buttons, and no stepslist.
 */
public JCSplitWizard(){
    this(BaseWizardPage.ALL, null);
}


/**
 * Constructor needs to know which buttons to display,
 * and the progress list title.
  * @param buttons which buttons to use - integer values
 * from BaseWizardPage
 * @param stepsTitle  The title to use for an automatically generated
 * progressList displayed in the left panel.  If null, no list is created.
 */
public JCSplitWizard(int buttons, String stepsTitle){
    setLayout(new GridLayout(1,1));
    splitWizardPage = new SplitWizardPage(buttons, stepsTitle);

    JButton b;
    b = splitWizardPage.getNextButton();
    if (b != null) {
        b.addActionListener(nextListener);
    }
    b = splitWizardPage.getPreviousButton();
    if (b != null) {
        b.addActionListener(previousListener);
        b.setEnabled(false);
    }

    b = splitWizardPage.getFinishButton();
    if (b != null) {
        b.addActionListener(finishListener);
        b.setEnabled(false);
    }
    b = splitWizardPage.getCancelButton();
    if (b != null) {
        b.addActionListener(cancelListener);
    }
    b = splitWizardPage.getHelpButton();
    if (b != null) {
        b.addActionListener(helpListener);
    }
    rightPanel = splitWizardPage.getRightPanel();
    rightPanel.setLayout(cardLayout);
    cardOwner = rightPanel;
    add(splitWizardPage);

}

//convenience methods.

public JButton getNextButton() {
    return splitWizardPage.getNextButton();
}

public JButton getPreviousButton() {
    return splitWizardPage.getPreviousButton();
}

public JButton getFinishButton() {
    return splitWizardPage.getFinishButton();
}

public JButton getCancelButton() {
    return splitWizardPage.getCancelButton();
}

public JButton getHelpButton() {
    return splitWizardPage.getHelpButton();
}

public JPanel getLeftPanel() {
    return splitWizardPage.getLeftPanel();
}

/**
 * Moves to the first page in the Wizard.
 * @see #last
 * @see #show
 */
public void first() {
    super.first();

    splitWizardPage.setRightPanelTitle(getCurrentPage().getName());
    splitWizardPage.updateStepSelection(getCurrentPage().getName());
    if (splitWizardPage.getPreviousButton() != null) {
        splitWizardPage.getPreviousButton().setEnabled(false);
    }
    if (splitWizardPage.getFinishButton() != null){
        splitWizardPage.getFinishButton().setEnabled(false);
    }
}


/**
 * Advances to the next page.
 * @see #previous
 * @see #finish
 * @see #cancel
 * @return
 */
public boolean next() {
    boolean is_last_page = false;
    Component current_page = null;
    Component next_page = null;

    int ncomponents = rightPanel.getComponentCount();
    for (int i = 0; i < ncomponents; i++) {
        Component comp = rightPanel.getComponent(i);
        if (comp.isVisible()) {
            current_page = comp;

            if (i == ncomponents - 1) {
                is_last_page = true;
                next_page = rightPanel.getComponent(0);
            }
            else {
                next_page = rightPanel.getComponent(i + 1);
            }
            break;
        }
    }

    JCWizardEvent event = new JCWizardEvent(this, current_page, next_page,
                                            is_last_page, !is_last_page);

    // In the preceding constructor, by default, we want
    // to prevent wraparound to first card from the
    // last card so we set "allow_change" to be the
    // opposite of "is_last_page"

    //
    // Post nextBegin event
    //
    if (event != null) {
        Enumeration e = JCListenerList.elements(listeners);
        while (e.hasMoreElements()) {
            JCWizardListener listener = (JCWizardListener)e.nextElement();
            listener.nextBegin(event);
        }
    }

    if (event.getAllowChange() == false) {
        return(false);
    }

    //
    // Advance to the next page unless a new page has been specified
    //
    if (next_page != event.getNewPage()) {
        cardLayout.show(rightPanel, event.getNewPage().getName());
    }
    else {
        cardLayout.next(rightPanel);
    }
     updatePage();


    //
    // Post nextComplete event
    //
    if (event != null) {
        Enumeration e = JCListenerList.elements(listeners);
        while (e.hasMoreElements()) {
            JCWizardListener listener = (JCWizardListener)e.nextElement();
            listener.nextComplete(event);
        }
    }

    return(true);



}

/**
 * Moves to the previous page.
 * @return  whether the move to the previous page was successful.
 * @see #next
 * @see #finish
 * @see #cancel
 */
public boolean previous() {
    boolean is_first_page = false;
    Component current_page = null;
    Component previous_page = null;

    int ncomponents = rightPanel.getComponentCount();
    for (int i = 0; i < ncomponents; i++) {
        Component comp = rightPanel.getComponent(i);
        if (comp.isVisible()) {
            current_page = comp;

            if (i == 0) {
                previous_page = rightPanel.getComponent(ncomponents - 1);
                is_first_page = true;
            }
            else {
                previous_page = rightPanel.getComponent(i - 1);
            }
            break;
        }
    }

    JCWizardEvent event = new JCWizardEvent(this, current_page, previous_page,
                                            isLastPage(current_page),
                                         !is_first_page);
    // In the preceding constructor, by default, we want
    // to prevent wraparound to the last card from the
    // first card so we set "allow_change" to be the
    // opposite of "is_first_page"

    //
    // post previousBegin event
    //
    if (event != null) {
        Enumeration e = JCListenerList.elements(listeners);
        while (e.hasMoreElements()) {
            JCWizardListener listener = (JCWizardListener)e.nextElement();
            listener.previousBegin(event);
        }
    }

    if (event.getAllowChange() == false) {
        return(false);
    }

    //
    // Advance to previous page unless a new page has been specified
    //
    if (previous_page != event.getNewPage()) {
        cardLayout.show(rightPanel, event.getNewPage().getName());
    }
    else {
        cardLayout.previous(rightPanel);
    }

    updatePage();

    //
    // Post previousComplete event
    //
    if (event != null) {
        Enumeration e = JCListenerList.elements(listeners);
        while (e.hasMoreElements()) {
            JCWizardListener listener = (JCWizardListener)e.nextElement();
            listener.previousComplete(event);
        }
    }



    return true;
}

/**
 * updates splicWizard page items like current list item, if any
 * and enabling/disabling next and previous buttons depending upn
 * which page is visible.
  */
protected void updatePage() {
    splitWizardPage.updateStepSelection(getCurrentPage().getName());
    JButton b = splitWizardPage.getPreviousButton();
    Component c = getCurrentPage();
    if (b != null){
        b.setEnabled(c !=
                rightPanel.getComponent(0));
    }
    b = splitWizardPage.getNextButton();
    if (b!= null){
        b.setEnabled(c !=
            rightPanel.getComponent(rightPanel.getComponentCount()
               - 1) );
    }
    b = splitWizardPage.getFinishButton();
    if (b!= null){
        b.setEnabled(c == rightPanel.getComponent(
                rightPanel.getComponentCount() -1));
    }
}

/**
 * Invokes the registered "cancel" action.
 * @see #next
 * @see #previous
 * @see #finish
 * @return whether the cancel succeeded or failed.
 */
public boolean cancel() {

    Component comp = getCurrentPage();


    JCWizardEvent event = new JCWizardEvent(this, comp, null,
                                            isLastPage(comp), true);

    //
    // Post Canceled event
    //
    if (event != null) {
        Enumeration e = JCListenerList.elements(listeners);
        while (e.hasMoreElements()) {
            JCWizardListener listener = (JCWizardListener)e.nextElement();
            listener.canceled(event);
        }
    }

    return(event.getAllowChange());

}

/**
 * Invokes the registered "help" action.
 * @see #next
 * @see #previous
 * @see #finish
 * @see #cancel
 */
public void help() {

    Component comp = getCurrentPage();



    JCWizardEvent event = new JCWizardEvent(this, comp, null,
                                            isLastPage(comp), true);
    //
    // Post Help event
    //
    if (event != null) {
        Enumeration e = JCListenerList.elements(listeners);
        while (e.hasMoreElements()) {
            JCWizardListener listener = (JCWizardListener)e.nextElement();
            listener.help(event);
        }
    }
}

/**
 * Invokes the registered "finish" action
 * @see #next
 * @see #previous
 * @see #cancel
 */
public boolean finish() {

    Component comp = getCurrentPage();

    JCWizardEvent event = new JCWizardEvent(this, comp, null,
            isLastPage(comp), true);

    //
    // Post finished event
    //
    if (event != null) {
        Enumeration e = JCListenerList.elements(listeners);
        while (e.hasMoreElements()) {
            JCWizardListener listener = (JCWizardListener)e.nextElement();
            listener.finished(event);
        }
    }

    return(event.getAllowChange());

}



/**
 * Retireves the current visible page.
 * @return current visible page
 */
public Component getCurrentPage()
{
    int ncomponents = rightPanel.getComponentCount();
    for (int i = 0; i < ncomponents; i++) {
        Component comp = rightPanel.getComponent(i);
        if (comp.isVisible()) {
            return(comp);
        }
    }
    return(null);
}

/**
 * Adds a page to the rightPanel.
 * @param page  Component which will populate the right panel
 * @param title Title for the right panel, which will appear
 * above the right panel, and in the progresslist, if used.
 */
public void addPage(Component page, String title){

    rightPanel.add(page, title);
    if (title != null && !title.equals(page.getName())){
        page.setName(title);
    }
    if (rightPanel.getComponentCount() == 1){
        splitWizardPage.setRightPanelTitle(title);
    }
    splitWizardPage.addToNavList(title);
    cardLayout.layoutContainer(rightPanel);


}

}
