/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCTreeTable.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:28 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.swing;

import com.klg.jclass.cell.EditorRendererRegistry;
import com.klg.jclass.cell.JCCellEditor;
import com.klg.jclass.cell.JCCellRenderer;
import com.klg.jclass.cell.JCCellInfo;
import com.klg.jclass.cell.swing.JCCellEditorToJTable;
import com.klg.jclass.cell.swing.JCCellRendererToJTable;

import com.klg.jclass.util.JCEnvironment;
import com.klg.jclass.util.swing.JCBrace;
import com.klg.jclass.util.swing.JCElasticLayout;
import com.klg.jclass.util.swing.JCSortableTable;
import com.klg.jclass.util.swing.JCTableColumn;
import com.klg.jclass.util.treetable.EmptyTreeTableModel;
import com.klg.jclass.util.treetable.JCTreeIconRenderer;
import com.klg.jclass.util.treetable.JCTreeTableModel;
import com.klg.jclass.util.treetable.TreeTableSupport;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.event.CellEditorListener;
import javax.swing.event.MouseInputListener;
import javax.swing.event.TreeExpansionListener;
import javax.swing.event.TreeWillExpandListener;
import javax.swing.plaf.TableUI;
import javax.swing.plaf.basic.BasicTableUI;
import javax.swing.table.*;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Enumeration;
import java.util.EventObject;

 /**
  * A component that gives a visual representation of the hierarchical
  * nature of its structure.
  * <P>
  * <B>See</B>
  * <A HREF="http://www.javasoft.com/products/jdk/1.3/docs/api/index.html">
  * The Java 2 API</A>
  * to compare this component to <code>JTree</code> and <code>JTable</code>.
  */
public class JCTreeTable extends JCSortableTable {

/** Valid value for view. */
public static final int TREE  = TreeTableSupport.TREE;
/** Valid value for view. */
public static final int TABLE = TreeTableSupport.TABLE;

/**
 * Valid value for <code>SwitchPolicy</code>. View is not interactively
 * switchable.
 */
public static final int SWITCH_VIEW_NEVER = 1;
/**
 * Valid value for <code>SwitchPolicy</code>. Switch view to table any time a
 * sort is invoked and whenever the switch icon is clicked.
 */
public static final int SWITCH_VIEW_TO_TABLE_ON_SORT = 2;
/**
 * Valid (default) value for <code>SwitchPolicy</code>.  Switch only when the
 * switch icon is clicked on.
 */
public static final int SWITCH_VIEW_ON_ICON_ONLY = 3;
/**
 * Valid value for <code>SwitchPolicy</code>. Do not even show the switch icon.
 * Thus the view is also not interactively switchable.
 */
public static final int SWITCH_BUTTON_DONT_SHOW = 4;

/** Valid value for <code>ShowNodeLines</code>. */
public static final int NODE_LINES_SHOW = TreeTableSupport.NODE_LINES_SHOW;
/** Valid value for <code>ShowNodeLines</code>. */
public static final int NODE_LINES_HIDE = TreeTableSupport.NODE_LINES_HIDE;
/** Valid (default) value for <code>ShowNodeLines</code>. */
public static final int NODE_LINES_USE_PLAF = TreeTableSupport.NODE_LINES_USE_PLAF;

protected static final int           ICON_SIZE = 16;
protected int                        switchPolicy = SWITCH_VIEW_ON_ICON_ONLY;
protected TreeTableSupport           support;
private static final Border          emptyBorder = new EmptyBorder(0, 0, 0, 0);
protected JCTreeTableSelectionModel  treeSelectionModel;
protected boolean                    sortable = true;
protected boolean                    expandInTreeColumnOnly = false;

/**
 * Null constructor.  An <code>EmptyTreeTableModel</code> will be used.
 */
public JCTreeTable()
{
	this(null);
}

/**
 * Constructor which uses the passed in model.  Use an intercellspacing of 0.
 *
 * @param model the <code>JCTreeTableModel</code> to be used
 */
public JCTreeTable(JCTreeTableModel model)
{
	this(model, new Dimension(0, 0));
}

/**
 * Constructor which uses the passed in model and a preset intercellspacing.
 *
 * @param model the <code>JCTreeTableModel</code> to be used
 * @param intercellSpacing the horizontal and vertical space between cells
 */
public JCTreeTable(JCTreeTableModel model, Dimension intercellSpacing)
{
    // Install the tree editor renderer and editor.
    setDefaultRenderer(TreePath.class, new Renderer());
    setDefaultEditor(TreePath.class, new Editor());

    support = new TreeTableSupport(this);

    setIntercellSpacing(intercellSpacing);

    // If the model is null create a simple default model
    if (model == null) {
        model = new EmptyTreeTableModel(EmptyTreeTableModel.makeDefaultTree());
    }

    // Pass the data model into the treetable support class
    support.setTreeTableModel(model);
    setTreeTableSelectionModel(new DefaultTreeTableSelectionModel());
    setModel(support);

    setUI(new TreeTableUI());

    if (support.getMaxIconHeight() > getRowHeight()) {
        setRowHeight(support.getMaxIconHeight());
    }
}

public void setUI(TableUI ui)
{
	if (support != null) {
		support.inferLookAndFeel();
	}
	super.setUI(ui);
}

public void updateUI()
{
	setUI(new TreeTableUI());
}

public Dimension getPreferredSize()
{
    // Use a nice, compact size for design-time
    if (JCEnvironment.isDesignTime()) {
        expandRow(0);
        return(new Dimension(200, 200));
    }
    else {
        return(super.getPreferredSize());
    }
}

/**
 * Provided for Beans property editors - has no effect.
 * @param s ignored
 */
public void setAbout(String s) {}

/**
 * Returns vendor information.
 * @return the version String
 */
public String getAbout()
{
    return(JCVersion.getVersionString());
}

/**
* Gets whether the treetable is sortable or not.
* @return sortable the boolean value
*/
public boolean isSortable()
{
	return(this.sortable);
}

/**
 * Sets whether the treetable is sortable or not.
 * @param sortable the boolean value
 */
public void setSortable(boolean sortable)
{
	// Keep the TreeTableSupport value parallel.
	support.setSortable(sortable);
	this.sortable = sortable;
}

/**
* Sets the <code>switchPolicy</code> variable which determines whether or
* not to allow view switching.
*
* @param policy the new switch policy; valid values are
*     <code>SWITCH_VIEW_NEVER</code>,
 *    <code>SWITCH_VIEW_TO_TABLE_ON_SORT</code>,
*     <code>SWITCH_VIEW_ON_ICON_ONLY</code>, or
 *    <code>SWITCH_BUTTON_DONT_SHOW</code>
*/
public void setSwitchPolicy(int policy)
{
	if (policy < SWITCH_VIEW_NEVER || policy > SWITCH_BUTTON_DONT_SHOW) {
		throw new IllegalArgumentException("Invalid switch policy.  Must be" +
		                                   "one of: SWITCH_VIEW_NEVER, " +
										   "SWITCH_VIEW_TO_TABLE_ON_SORT, " +
										   "SWITCH_VIEW_ON_ICON_ONLY, or " +
										   "SWITCH_BUTTON_DONT_SHOW");
	}
	switchPolicy = policy;

    // Enumerate through the columns.  Find the switch renderer and
    // redo the layout.
    Enumeration e = getColumnModel().getColumns();
    while (e.hasMoreElements()) {
        TableColumn column = (TableColumn)e.nextElement();
        TableCellRenderer renderer = column.getHeaderRenderer();
        if (renderer instanceof SwitchRenderer) {
            SwitchRenderer switchRenderer = (SwitchRenderer)renderer;
            switchRenderer.setSwitchPolicy(switchPolicy);
            boolean left = getComponentOrientation().isLeftToRight();
            switchRenderer.reLayout(left);
            getTableHeader().repaint();
        }
    }
}

/**
* Returns the current value of the <code>switchPolicy</code> variable.
*
* @return   policy  the int which indicates the policy; will be one of
*     <code>SWITCH_VIEW_NEVER</code>,
 *    <code>SWITCH_VIEW_TO_TABLE_ON_SORT</code>,
*     <code>SWITCH_VIEW_ON_ICON_ONLY</code>, or
 *    <code>SWITCH_BUTTON_DONT_SHOW</code>
*/
public int getSwitchPolicy()
{
	return(this.switchPolicy);
}

/**
 * Sets whether we are viewing this as a tree or a table.
 * @param view Valid values are <code>TREE</code> and <code>TABLE</code>
 */
public void setView(int view)
{
	// Enumerate through the columns
	Enumeration e = getColumnModel().getColumns();
	while (e.hasMoreElements()) {
		TableColumn column = (TableColumn)e.nextElement();
		TableCellRenderer renderer = column.getHeaderRenderer();
		if (renderer instanceof SwitchRenderer) {
			((SwitchRenderer)renderer).setView(view);
		}
	}
	support.setView(view);
}

/**
 * Returns the current view.
 * @return The current view; will be one of <code>TREE</code> or
 *         <code>TABLE</code>
 * @see #setView
 */
public int getView()
{
    if (support != null) {
	return(support.getView());
    }
    return JCTreeTable.TREE;
}

/**
 * Allows you override the PLAF specified behavior of drawing lines.
 * Particularly useful in the Metal LookAndFeel. <P>
 *
 * @param lines the new line drawing style; must be one of
 *     <code>NODE_LINES_USE_PLAF</code> (the default),
 *     <code>NODE_LINES_SHOW</code>,
 *     or <code>NODE_LINES_HIDE</code>
 */
public void setShowNodeLines(int lines)
{
	support.setShowNodeLines(lines);
}

/**
 * Returns the state of <code>ShowNodeLines</code>.
 * @return the current line drawing style; will be one of
 *     <code>NODE_LINES_USE_PLAF</code> (the default),
 *     <code>NODE_LINES_SHOW</code>,
 *     or <code>NODE_LINES_HIDE</code>
 * @see #setShowNodeLines
 */
public int getShowNodeLines()
{
	return(support.getShowNodeLines());
}
 /**
  * Gets the value of the <code>expandInTreeColumnOnly</code> property. If
  * <code>true</code>, the only way to expand or collapse the tree is to click
  * or double click in the tree column. If <code>false</code> (the default
  * value), clicking anywhere in the row (where an editor is not present) will
  * expand/collapse the tree.
  * @return current value of the <code>expandInTreeColumnOnly</code> property
  */
 public boolean isExpandInTreeColumnOnly()
 {
     return(expandInTreeColumnOnly);
 }

 /**
  * Sets the value of the <code>expandInTreeColumnOnly</code> property. If
  * <code>true</code>, the only way to expand or collapse the tree is to click
  * or double click in the tree column.  If <code>false</code> (the default
  * value), clicking anywhere in the row (where an editor is not present) will
  * expand/collapse the tree.
  * @param expandInTreeColumnOnly the new value of the
  *                               <code>expandInTreeColumnOnly</code> property
  */
 public void setExpandInTreeColumnOnly(boolean expandInTreeColumnOnly)
 {
     this.expandInTreeColumnOnly = expandInTreeColumnOnly;
 }

/**
 * Returns the actual tree column index. It is usually 0, but it may have been
 * moved by the user.
 * @return the current column, which displays the tree when the view is
 *         <code>TREE</code>
 */
public int getTreeColumn()
{
    TableColumnModel model = getColumnModel();
    int count = model.getColumnCount();
    for (int i = 0; i < count; i++) {
        if (model.getColumn(i) instanceof SwitchTableColumn) {
            return(i);
        }
    }
    // Something has gone wrong (assume the first column)
    return(0);
}

/**
 * Overridden to return the appropriate data renderer for the first column if
 * treetable is in tree display mode.
 *
 * @param row the row of the cell to render, where 0 is the first
 * @param column the column of the cell to render, where 0 is the first
 * @return the current cell renderer for this row and column
 */
public TableCellRenderer getCellRenderer(int row, int column)
{
    TableCellRenderer renderer = null;
    if (support.getView() == support.TREE && column == getTreeColumn()) {
        renderer = getDefaultRenderer(TreePath.class);
	}
    else {
        renderer = super.getCellRenderer(row, column);
    }
	return(renderer);
}

/**
 * Overridden to return the appropriate data editor for the first column if
 * treetable is in tree display mode.
 *
 * @param row the row of the cell to edit, where 0 is the first
 * @param column the column of the cell to edit, where 0 is the first
 * @return the current cell editor for this row and column
 */
public TableCellEditor getCellEditor(int row, int column)
{
    TableCellEditor editor = null;
    if (support.getView() == support.TREE && column == getTreeColumn()) {
        editor = getDefaultEditor(TreePath.class);
    }
    else {
        editor = super.getCellEditor(row, column);
	}
	return(editor);
}

/**
 * Adds a listener for <code>TreeExpansion</code> events.
 *
 * @param l a <code>TreeExpansionListener</code> that will be notified when
 *            a tree node is expanded or collapsed (a "negative
 *            expansion")
 */
public void addTreeExpansionListener(TreeExpansionListener l)
{
	support.addTreeExpansionListener(l);
}

/**
 * Removes a listener for <code>TreeExpansion</code> events.
 *
 * @param l the <code>TreeExpansionListener</code> to remove
 */
public void removeTreeExpansionListener(TreeExpansionListener l)
{
	support.removeTreeExpansionListener(l);
}

/**
 * Adds a listener for <code>TreeWillExpand</code> events.
 *
 * @param l a <code>TreeWillExpandListener</code> that will be notified when
 *            a tree node will be expanded or collapsed (a "negative
 *            expansion")
 */
public void addTreeWillExpandListener(TreeWillExpandListener l)
{
	support.addTreeWillExpandListener(l);
}

/**
 * Removes a listener for <code>TreeWillExpand</code> events.
 *
 * @param l the <code>TreeWillExpandListener</code> to remove
 */
public void removeTreeWillExpandListener(TreeWillExpandListener l)
{
	support.removeTreeWillExpandListener(l);

}
/**
 * Sets the icon renderer.  If null, no icon will be drawn.
 * @param renderer the new icon renderer
 */
public void setTreeIconRenderer(JCTreeIconRenderer renderer)
{
	support.setTreeIconRenderer(renderer);
}

/**
 * Returns the icon renderer being used.  If null, no icons are being
 * drawn.
 * @return the current icon renderer
 */
public JCTreeIconRenderer getTreeIconRenderer()
{
	return(support.getTreeIconRenderer());
}

/**
 * Ensures that the node in the specified row is collapsed.
 * <p> If <code>row</code> is &lt; 0 or &gt;= getRowCount, this
 * will have no effect.
 *
 * @param row  an int specifying a display row, where 0 is the
 *             first row in the display
 */
public void collapseRow(int row)
{
	support.collapseRow(row);
}

/**
 * Ensures that the node identified by the specified path is
 * collapsed and viewable.
 *
 * @param path  the <code>TreePath</code> identifying a node
 */
public void collapsePath(TreePath path)
{
	support.collapsePath(path);
}

/**
 * Ensures that the node in the specified row is expanded and
 * viewable. <p> If <code>row</code> is &lt; 0 or &gt;= getRowCount, this
 * will have no effect.
 *
 * @param row  an int specifying a display row, where 0 is the
 *             first row in the display
 */
public void expandRow(int row)
{
	support.expandRow(row);
}

/**
 * Ensures that the node identified by the specified path is
 * expanded and viewable.
 *
 * @param path  the <code>TreePath</code> identifying a node
 */
public void expandPath(TreePath path)
{
	support.expandPath(path);
}

/**
 * Ensures that the node identified by path is currently viewable.
 *
 * @param path  the <code>TreePath</code> to make visible
 */
public void makeVisible(TreePath path)
{
	if (path != null) {
	    TreePath parent_path = path.getParentPath();

		if (parent_path != null) {
			// Expanding the parent causes the node to become visible; so
			// it is not necessary to expand the node as well
			expandPath(parent_path);
	    }
	}
}

/**
 * Returns the path to the node that is closest to <i>X</i>,<i>Y</i>.  If
 * no nodes are currently viewable, or there is no model, returns
 * null; otherwise it always returns a valid path. To test if
 * the node is exactly at <i>X</i>, <i>Y</i>, get the node's bounds and
 * test <i>X</i>, <i>Y</i> against that.
 *
 * @param x an int giving the number of pixels horizontally from
 *          the left edge of the display area, minus any left margin
 * @param y an int giving the number of pixels vertically from
 *          the top of the display area, minus any top margin
 * @return  the <code>TreePath</code> for the node closest to that location;
 *          null if nothing is viewable or there is no model
 *
 * @see #getPathForLocation
 * @see #getPathBounds
 */
public TreePath getClosestPathForLocation(int x, int y)
{
	int row = rowAtPoint(new Point(x, y));
	return(support.getPathForRow(row));
}

/**
 * Returns the row to the node that is closest to <i>X</i>,<i>Y</i>. If no nodes
 * are viewable or there is no model, returns -1. Otherwise,
 * it always returns a valid row.  To test if the returned object is
 * exactly at <i>X</i>, <i>Y</i>, get the bounds for the node at the returned
 * row and test <i>X</i>, <i>Y</i> against that.
 *
 * @param x an int giving the number of pixels horizontally from
 *          the left edge of the display area, minus any left margin
 * @param y an int giving the number of pixels vertically from
 *          the top of the display area, minus any top margin
 * @return the row closest to the location, -1 if nothing is
 *         viewable or there is no model
 *
 * @see #getRowForLocation
 * @see #getRowBounds
 */
public int getClosestRowForLocation(int x, int y)
{
	return(rowAtPoint(new Point(x, y)));
}

/**
 * Returns the path to the element that is currently being edited.
 *
 * @return  the <code>TreePath</code> for the node being edited
 */
public TreePath getEditingPath()
{
	int edit_row = getEditingRow();
	if (edit_row == -1) {
		return(null);
	}
	return(support.getPathForRow(edit_row));
}

/**
 * Returns an Enumeration of the descendants of <code>path</code> that
 * are currently expanded. If <code>path</code> is not currently
 * expanded, this will return null. If you expand/collapse nodes while
 * iterating over the returned Enumeration, this may not return all
 * the expanded paths, or may return paths that are no longer expanded.
 * @param parent the path to the parent whose expanded descendants are desired
 * @return an Enumeration of the expanded descendants of the parent node
 */
public Enumeration getExpandedDescendants(TreePath parent)
{
	return(support.getExpandedDescendants(parent));
}

/**
 * Returns the <code>JCTreeTableModel</code> that is providing the data.
 *
 * @return the <code>TreeModel</code> that is providing the data
 */
public JCTreeTableModel getTreeTableModel()
{
	return(support.getTreeTableModel());
}

/**
 * Sets the <code>TreeModel</code> that will provide the data.
 *
 * @param model the <code>TreeModel</code> that is to provide the data
 */
public void setTreeTableModel(JCTreeTableModel model)
{
	support.setTreeTableModel(model);
	// Trick JTable into actually changing the model
    setModel(new DefaultTableModel());
    // Now set it to what we really wanted
    setModel(support);
}

/**
 * <i>This method not functional in this release</i><p>
 * Returns the <code>Rectangle</code> that the specified node will be drawn
 * into. Returns null if any component in the path is hidden
 * (under a collapsed parent).
 * <p>
 * <b>Note:</b>
 * This method returns a valid rectangle, even if the specified
 * node is not currently displayed.
 *
 * @param path the <code>TreePath</code> identifying the node
 * @return the <code>Rectangle</code> the node is drawn in, or null
 */
public Rectangle getPathBounds(TreePath path)
{
	// PENDING Requires ability to map a given path to a particular row
	// also figure how to get this info from the table
	return(null);
}

/**
 * Returns the path for the node at the specified location.
 *
 * @param x an int giving the number of pixels horizontally from
 *          the left edge of the display area, minus any left margin
 * @param y an int giving the number of pixels vertically from
 *          the top of the display area, minus any top margin
 * @return  the <code>TreePath</code> for the node at that location
 */
public TreePath getPathForLocation(int x, int y)
{
	return(getClosestPathForLocation(x, y));
}

/**
 * Returns the row for the specified location.
 *
 * @param x an int giving the number of pixels horizontally from
 *          the left edge of the display area, minus any left margin
 * @param y an int giving the number of pixels vertically from
 *          the top of the display area, minus any top margin
 * @return the row corresponding to the location, or -1 if the
 *         location is not within the bounds of a displayed cell
 * @see #getClosestRowForLocation
 */
public int getRowForLocation(int x, int y)
{
	return(getClosestRowForLocation(x, y));
}

/**
 * <i>This method is not functional in this release</i><p>
 * Returns the <code>Rectangle</code> that the node at the specified row is
 * drawn in.
 *
 * @param row  the row to be drawn, where 0 is the first row in the
 *             display
 * @return the <code>Rectangle</code> the node is drawn in
 */
public Rectangle getRowBounds(int row)
{
	// PENDING Requires ability to map a given path to a particular row
	// also figure how to get this info from the table
	return(null);
}

/**
 * Returns the row that displays the node identified by the specified
 * path.
 *
 * @param path  the <code>TreePath</code> identifying a node
 * @return an int specifying the display row, where 0 is the first
 *         row in the display, or -1 if any of the elements in path
 *         are hidden under a collapsed parent.
 */
public int getRowForPath(TreePath path)
{
	return(support.getRowForPath(path));
}

/**
 * Returns the path that is displayed at the table row specified.
 *
 * @param row  the row whose path is returned
 * @return the <code>TreePath</code> displayed at the specified row
 */
public TreePath getPathForRow(int row)
{
	return(support.getPathForRow(row));
}

/**
 * Returns <code>true</code> if the tree scrolls to show previously hidden
 * children.
 * @return <code>true</code> when a node is expanded, and as many of the
 *         descendants as possible are scrolled to be visible
 */
public boolean getScrollsOnExpand()
{
	return(support.getScrollsOnExpand());
}

/**
 * Determines whether or not as many of
 * the descendants are scrolled to be inside the viewport as
 * possible when a node is expanded. The default is <code>true</code>.
 * @param newValue if set to <code>true</code>, when a node is expanded as many
 *        of the descendants as possible are scrolled to be visible
 */
public void setScrollsOnExpand(boolean newValue)
{
	support.setScrollsOnExpand(newValue);
}

/**
 * Returns <code>true</code> if the root node of the tree is displayed.
 *
 * @return <code>true</code> if the root node of the tree is displayed
 * @see #setRootVisible
 */
public boolean isRootVisible()
{
	return(support.isRootVisible());
}

/**
 * Determines whether or not the root node from the <code>TreeModel</code> is
 * visible.
 *
 * @param rootVisible <code>true</code> if the root node of the tree is to be
 *                    displayed
 * @see #isRootVisible
 */
public void setRootVisible(boolean rootVisible)
{
	support.setRootVisible(rootVisible);
}

/**
 * Returns <code>true</code> if handles for the root nodes are displayed.
 *
 * @return <code>true</code> if root handles are displayed
 * @see #setShowsRootHandles
 */
public boolean getShowsRootHandles()
{
	return(support.getShowsRootHandles());
}

/**
 * Determines whether the handles for the root nodes are to be displayed.
 *
 * @param newValue <code>true</code> if root handles are to be displayed
 * @see #getShowsRootHandles
 */
public void setShowsRootHandles(boolean newValue)
{
	support.setShowsRootHandles(newValue);
}

/**
 * Returns the rows for the visible specified paths.
 * @param paths an array of paths for which the corresponding rows are required
 * @return a list of row indicies corresponding to the paths
 */
public int[] getRowsForPaths(TreePath paths[])
{
	return(support.getRowsForPaths(paths));
}

/**
 * Adds the node identified by the specified <code>TreePath</code> to the
 * current selection. If any component of the path isn't viewable, it is made
 * viewable.
 * @param path the <code>TreePath</code> specifying the node to add to the
 * selection
 */
public void addSelectionPath(TreePath path)
{
	makeVisible(path);
	treeSelectionModel.addSelectionPath(path);
}

/**
 * Adds each path in the array of paths to the current selection. If any
 * component of any of the paths isn't viewable, it is made viewable.
 * @param paths An array of <code>TreePath</code> objects that specifies the
 *              nodes to add to selection
 */
public void addSelectionPaths(TreePath[] paths)
{
	if (paths != null) {
	    for (int i = paths.length - 1; i >= 0; i--) {
			// Make sure all paths are visible to ensure proper row mapping
			makeVisible(paths[i]);
		}
	}
	treeSelectionModel.addSelectionPaths(paths);
}

/**
 * Selects the node identified by the specified path. If any component
 * of the path is hidden (under a collapsed node), it is exposed
 * (made viewable).
 * @param path the <code>TreePath</code> specifying the node to select
 */
public void setSelectionPath(TreePath path)
{
	makeVisible(path);
	treeSelectionModel.setSelectionPath(path);
}

/**
 * Select the specified paths.
 * @param paths an array of <code>TreePath</code> objects that specifies the
 *              nodes to select
 */
public void setSelectionPaths(TreePath paths[])
{
	if (paths != null) {
	    for (int i = paths.length - 1; i >= 0; i--) {
			// make sure all paths are visible to ensure proper row mapping
			makeVisible(paths[i]);
		}
	}
	treeSelectionModel.setSelectionPaths(paths);
}

/**
 * Returns <code>true</code> if the item identified by the path is currently
 * selected.
 * @param path the path to the item whose selection status is required
 * @return asks: "is the item that the path identifies currently selected?"
 */
public boolean isPathSelected(TreePath path)
{
	if (treeSelectionModel != null) {
		return(treeSelectionModel.isPathSelected(path));
	}
	return(false);
}

/**
 * Returns the path to the first selected node.
 *
 * @return the <code>TreePath</code> for the first selected node, or null if
 *         nothing is currently selected
 */
public TreePath getSelectionPath()
{
	//return treeSelectionModel.getSelectionPath();
	int row = getSelectedRow();
	if (row < 0) {
		return(null);
	}
	return(support.getPathForRow(row));
}

/**
 * Returns the paths of all selected values.
 *
 * @return an array of <code>TreePath</code> objects indicating the selected
 *         nodes, or null if nothing is currently selected.
 */
public TreePath[] getSelectionPaths()
{
	//return treeSelectionModel.getSelectionPaths();
	int rows[] = getSelectedRows();
	if (rows == null || rows.length == 0) {
		return(null);
	}
	TreePath paths[] = new TreePath[rows.length];

	for (int i = 0; i < rows.length; i++) {
		paths[i] = support.getPathForRow(rows[i]);
	}
	return(paths);
}

//
// Tree Selection Support/Mapping
//

/**
 * Sets the tree's selection model. When a null value is specified,
 * an empty <code>selectionModel</code> is used, which does not allow
 * selections.
 *
 * @param treeSelectionModel the <code>TreeSelectionModel</code> to use, or null
 * to disable selections
 * @see TreeSelectionModel
 */
public void setTreeTableSelectionModel(JCTreeTableSelectionModel treeSelectionModel)
{
	if (treeSelectionModel == null) {
		treeSelectionModel = EmptyTreeSelectionModel.getSharedInstance();
	}
	TreeSelectionModel old = this.treeSelectionModel;

	this.treeSelectionModel = treeSelectionModel;
	this.treeSelectionModel.setRowMapper(support);
	setSelectionModel(this.treeSelectionModel.getListSelectionModel());
	firePropertyChange("treeSelectionModel", old, this.selectionModel);
}

/**
 * Returns the model for selections. This should always return a
 * non-null value. If you do not want to allow anything to be selected,
 * set the selection model to null, which forces an empty
 * selection model to be used.
 *
 * @return the current <code>TreeSelectionModel</code> in use
 * @see #setSelectionModel
 */
public JCTreeTableSelectionModel getTreeSelectionModel()
{
	return(treeSelectionModel);
}

/**
 * Asks: "Is the point over the icon that controls whether the view is
 * <code>TREE</code> or <code>TABLE</code>?".
 * @param p the point to be tested
 * @return true asks: is the input point over the "switch views" icon?
 */
protected boolean isPointOverSwitch(Point p)
{
	// Get the x position of the column we are over
	Enumeration e = getColumnModel().getColumns();
    boolean left = getComponentOrientation().isLeftToRight();

	int x = left ? 0 : getWidth();
	while (e.hasMoreElements()) {
		TableColumn column = (TableColumn)e.nextElement();
		int width = column.getWidth();
        boolean inColumn = left ? ((x + width) > p.x) : ((x - width) < p.x);
		if (inColumn) {
			// We are in the right column
            TableCellRenderer renderer = column.getHeaderRenderer();
            if (renderer instanceof SwitchRenderer) {
                SwitchRenderer switchRenderer = (SwitchRenderer)renderer;
                boolean isSwitchLeft = switchRenderer.isCurrentLayoutLeft();
                Insets insets = switchRenderer.getInsets();
                int sbWidth = switchRenderer.getSwitchButton().getSize().width;

                // Calculate difference between point and the edge of the
                // renderer to which the switch is adjacent.  Note that the
                // treetable can be a different orientation than the renderer.
                int leftEdge  = left ? x : x - width;
                int rightEdge = left ? x + width : x;
                int diff = isSwitchLeft ? p.x - leftEdge : rightEdge - p.x;

                // Get the size of the switch.  If right oriented, make sure the
                // right brace (size 1) is accounted for.
                int size = isSwitchLeft ? sbWidth + insets.left
                                        : sbWidth + insets.right + 1;

                // If the difference between the point and the edge is less
                // than the size of the switch, we've hit the switch.
                if (diff <= size) {
                    return(true);
                }
            }
			return(false);
		}
		x = left ? x + width : x - width;
	}
	return(false);
}

/**
 * Switches views between <code>TREE</code> and <code>TABLE</code>.
 */
protected void toggleView()
{
	if (getView() == TREE) {
		setView(TABLE);
	}
	else {
		setView(TREE);
	}
}

/**
 * Adds a <code>MouseListener</code> to the table header.
 * @return the new table header mouse listener
 */
protected MouseAdapter createTableHeaderMouseListener()
{
    return(new TableHeaderMouseAdapter());
}

/**
 * <code>MouseAdapter</code> for the table header.
 */
protected class TableHeaderMouseAdapter extends JCSortableTable.TableHeaderMouseListener
{
    /**
     * Invoked when a mouse button has been clicked on a component.
     * @param e the event generated by the mouse click
     */
    public void mouseClicked(MouseEvent e)
    {
        // If either the mouse press or mouse release is a popup trigger then
        // do not do the switch or the sort.  Note that whether it happens on
        // mouse press or release is system dependent.
        if (!isPopupTrigger) {
            int column = columnAtPoint(e.getPoint());
            boolean switch_clicked = false;
            if (switchPolicy == SWITCH_VIEW_ON_ICON_ONLY ||
                switchPolicy == SWITCH_VIEW_TO_TABLE_ON_SORT)
            {
                switch_clicked = isPointOverSwitch(e.getPoint());
            }

            if (switch_clicked) {
                toggleView();
            }
            else if (isSortable() &&
                     switchPolicy == SWITCH_VIEW_TO_TABLE_ON_SORT &&
                     getView() == TREE)
            {
                // Switch to table view from tree view on sort
                setView(TABLE);
                sort(column);
            }
            else if (isSortable()) {
                sort(column);
            }
        }
    }
}


 

protected class TreeTableUI extends BasicTableUI
{
    /**
	 * Creates a mouse input listener (for example, a handler).
	 * @return the new tree table mouse input handler
	 */
	protected MouseInputListener createMouseInputListener()
	{
		return(new TreeTableMouseInputHandler());
	}

	protected class TreeTableMouseInputHandler extends MouseInputHandler
    {
		protected boolean mousePressedInControlBox = false;

		/**
		 * Handles a mouse pressed event.
		 * @param e the event generated by the mouse being pressed
		 */
		public void mousePressed(MouseEvent e)
		{
			int row    = rowAtPoint(e.getPoint());
			int column = columnAtPoint(e.getPoint());

			// Translate the point into the coordinates of the cell clicked on
			Point p = new Point(e.getPoint());
			if (!support.processClickInTreeColumn(e, row, column, p,
			                                      getRowHeight()))
			{
				mousePressedInControlBox = false;
				super.mousePressed(e);
			}
			else {
				mousePressedInControlBox = true;
                removeEditor();
			}
		}

		/**
		 * Handles a mouse dragged event.
		 * @param e the event generated by the mouse being dragged
		 */
		public void mouseDragged(MouseEvent e)
		{
			// Don't call the super if the original click happened in
			// a control box
			if (!mousePressedInControlBox) {
				super.mouseDragged(e);
			}
		}

        /**
         * Handles a mouse released event.
         * @param e the event generated by the mouse being released
         */
        public void mouseReleased(MouseEvent e)
        {
            if (JCEnvironment.getJavaVersion() < 140 ||
                !mousePressedInControlBox)
            {
                super.mouseReleased(e);
            }
        }
	} // TreeTableMouseInputHandler
} // TreeTableUI

 /**
  * Iterate through the columns to get the renderer so that we can reset the
  * horizontal alignment on JCCellRenderers.
  */
 protected void reLayoutColumnRenderers()
 {
     super.reLayoutColumnRenderers();
     TableCellRenderer renderer = getDefaultRenderer(TreePath.class);
     if (renderer instanceof Renderer) {
         TableCellRenderer r = ((Renderer)renderer).renderer;
         if (r instanceof JCCellRendererToJTable) {
             boolean isLeft = getComponentOrientation().isLeftToRight();
             int align = isLeft ? JCCellInfo.LEFT : JCCellInfo.RIGHT;
             ((JCCellRendererToJTable)r).setHorizontalAlignment(align);
         }
     }
}

/**
 * Inner class to render the node hierarchy and wrap the defined PLAF
 * renderer behavior.
 */
protected class TreeRendererEditor extends JPanel
{
	protected Insets    insets = new Insets(0, 0, 0, 0);
	protected TreePath  path = null;
	protected Object    value;
	protected Class     value_class;

	/**
	 * Gets the current insets (which are set to be 0).
	 * @return the current insets
	 */
	public Insets getInsets()
	{
		return(insets);
	}

	/**
	 * Override method at which to layout the true renderer.
	 */
	public void doLayout()
	{
		if (getComponentCount() == 0 || path == null) {
			
			return;
		}
		Component c = getComponent(0);
		Dimension s = getSize();
        boolean isLeft =
            JCTreeTable.this.getComponentOrientation().isLeftToRight();
        int rendererIndent = support.getRendererIndent(path, value,
                                                       value_class);
		int x = isLeft ? rendererIndent : 0;
		c.setBounds(x, 0, s.width - rendererIndent, s.height);
	}

	/**
	 * Override paint to paint the hierarchy.
	 * @param gc The graphics object used for painting
	 */
	public void paint(Graphics gc)
	{
		Dimension size = getSize();
		support.prepareCellForRender(gc, size.width, size.height);
		super.paint(gc);
		// Clear the background
		support.renderHierarchy(gc, new Rectangle(new Point(0, 0), size),
								path, value, value_class);
	}

	/**
	 * We set the opaque property to be always <code>false</code> for renderers
     * and editors that extend this class.
	 * @return false
	 */
	public boolean isOpaque()
	{
		return(false);
	}
}

protected class Renderer extends TreeRendererEditor
                         implements TableCellRenderer
{
    protected TableCellRenderer renderer;

	public Renderer() {}

	public Component getTableCellRendererComponent(JTable table,
												   Object valueObject,
												   boolean isSelected,
												   boolean hasFocus,
												   int row, int column)
	{
		if (table != null) {
            ComponentOrientation o = JCTreeTable.this.getComponentOrientation();
			if (renderer == null) {
                TableColumn tableColumn =
                    table.getColumnModel().getColumn(column);
				if (tableColumn != null) {
				    TableCellRenderer cellRenderer =
                        tableColumn.getCellRenderer();
					if (cellRenderer != null) {
						renderer = cellRenderer;
					}
				}
                if ((renderer == null) &&
                    (tableColumn instanceof JCTableColumn))
                {
                    JCTableColumn jcTableColumn = (JCTableColumn)tableColumn;
		            EditorRendererRegistry registry =
				        EditorRendererRegistry.getCentralRegistry();
		            Class c = support.getTreeColumnClass();
		            String type = jcTableColumn.getRegistryType();
                    JCCellRenderer jcRenderer =
                        registry.getCellRendererInstance(c, type);
                    if (jcRenderer != null) {
                        JCCellRendererToJTable jcCellRen =
                            new JCCellRendererToJTable(jcRenderer);
                        if (!o.isLeftToRight()) {
                            jcCellRen.setHorizontalAlignment(JCCellInfo.RIGHT);
                        }
                        renderer = jcCellRen;
                    }
                }
                if (renderer == null) {
                    renderer = getDefaultRenderer(support.getTreeColumnClass());
                }
			}

            if (valueObject instanceof TreePath) {
    			path = (TreePath) valueObject;
    		    value = support.getTreeColumnValue(path.getLastPathComponent());
            }
            else {
                path = null;
                value = valueObject;
            }
            if (value != null) {
                value_class = value.getClass();
            }
            else {
                value_class = null;
            }
			Component rc = renderer.getTableCellRendererComponent(table, value,
                                                                  isSelected,
													              hasFocus,
                                                                  row, column);
            JCTreeTable.setComponentOrientation(rc, o, false);
			if (rc.getParent() != this) {
                removeAll();
				add(rc);
			}
		}
		return(this);
	}

	public Dimension getPreferredSize()
	{
		if (getComponentCount() == 0) {
			return(super.getPreferredSize());
		}
		Component c = getComponent(0);
        Dimension dim = c.getPreferredSize();
        return(new Dimension(dim.width + support.getRendererIndent(path, value,
                                                                   value_class),
		                     dim.height));
    }
}

protected class Editor extends TreeRendererEditor implements TableCellEditor
{
	protected TableCellEditor editor;

	public Editor() {}

    protected TableCellEditor getEditor()
	{
        return(getEditor(JCTreeTable.this));
    }

	protected TableCellEditor getEditor(JTable table)
	{
        // This is only used for the tree in the first column of the model so a
        // column number of 0 can be assumed.
		if (editor == null) {
            TableColumn tableColumn = getColumnModel().getColumn(0);
            editor = tableColumn.getCellEditor();
            if ((editor == null) && (tableColumn instanceof JCTableColumn)) {
		        EditorRendererRegistry registry =
				    EditorRendererRegistry.getCentralRegistry();
		        Class c = support.getTreeColumnClass();
		        String type = ((JCTableColumn)tableColumn).getRegistryType();
                JCCellEditor jcEditor = registry.getCellEditorInstance(c, type);
                if (jcEditor != null) {
                    editor = new JCCellEditorToJTable(jcEditor);
                }
            }
            if (editor == null) {
                editor = getDefaultEditor(support.getTreeColumnClass());
            }
		}
		return(editor);
	}

	public Component getTableCellEditorComponent(JTable table,
												 Object value,
												 boolean isSelected,
												 int row, int column)
	{
		path = (TreePath) value;
		Object obj = support.getTreeColumnValue(path.getLastPathComponent());
		Component ec = getEditor(table).getTableCellEditorComponent(table, obj,
					                                                isSelected,
														            row,
                                                                    column);
        ComponentOrientation o = JCTreeTable.this.getComponentOrientation();
        JCTreeTable.setComponentOrientation(ec, o, false);
		if (ec.getParent() != this) {
            removeAll();
			add(ec);
		}
		return(this);
	}

	public Object getCellEditorValue()
	{
		return(getEditor().getCellEditorValue());
	}

	public boolean isCellEditable(EventObject anEvent)
	{
		return(getEditor().isCellEditable(anEvent));
	}

	public boolean shouldSelectCell(EventObject anEvent)
	{
		return(getEditor().shouldSelectCell(anEvent));
	}

	public boolean stopCellEditing()
	{
		return(getEditor().stopCellEditing());
	}

	public void cancelCellEditing()
	{
		getEditor().cancelCellEditing();
	}

	public void addCellEditorListener(CellEditorListener l)
	{
		getEditor().addCellEditorListener(l);
	}

	public void removeCellEditorListener(CellEditorListener l)
	{
		getEditor().removeCellEditorListener(l);
	}
}

/**
 * Creates the <code>TableColumn</code> needed.  This method was created to
 * allow subclasses to override, to easily change the default Column created.
 * @param column the column index for which the new table column is to be
 *    created
 * @return the new sortable table column
 */
public TableColumn createSortableTableColumn(int column)
{
	if (column == 0) {
		// ??? should be if column == tree column
		return(new SwitchTableColumn(column));
	}
	else {
		return(super.createSortableTableColumn(column));
	}
}

/**
 * Overridden version of <code>TableColumn</code> that returns
 * <code>SwitchRenderer</code> instead of the normal default for the tree
 * column.
 */
protected class SwitchTableColumn extends JCTableColumn
{
    public SwitchTableColumn(int column)
    {
        super(column);
        setHeaderRenderer(createDefaultHeaderRenderer());
    }

	/**
	 * Overridden to provide a renderer that reserves a location on
	 * the right for a sort icon, if one exists.
	 * @return A header renderer that renderers using a
     *         <code>SwitchRenderer</code>; this allows the user to
     *         interactively switch between <code>TREE</code> and
     *         <code>TABLE</code> mode
	 */
    protected TableCellRenderer createDefaultHeaderRenderer()
	{
        // The following is done because this method may be called from
        // JCTreeTable's or subclass constructor in which case
        // JCTreeTable.this has not yet been set.
        boolean isLeft = ComponentOrientation.UNKNOWN.isLeftToRight();
        int swPolicy = JCTreeTable.SWITCH_VIEW_ON_ICON_ONLY;
        if (JCTreeTable.this != null) {
            isLeft = JCTreeTable.this.getComponentOrientation().isLeftToRight();
            swPolicy = switchPolicy;
        }
		return(new SwitchRenderer(swPolicy, isLeft));
    }
} // SwitchTableColunm

/**
 * Renderer used to draw the tree&lt;--&gt;table switch icon in the tree column
 * header.
 */
protected class SwitchRenderer extends JCSortableTable.SortableHeaderRenderer
							   implements TableCellRenderer,
									      JCSortableTable.SortableColumnHeader
{
    protected int          switchPolicy = SWITCH_VIEW_ON_ICON_ONLY;
	protected JButton      switchButton = null;
	protected Icon         icon = null;
    protected boolean      initialized = false;

	/**
	 * Constructor. Creates button, separator, and label, and sets up the
     * layout.
	 */
	public SwitchRenderer()
	{
		this(SWITCH_VIEW_ON_ICON_ONLY, true);
	}

    /**
     * Constructor that specifies switchPolicy.
     * @param switchPolicy The switchPolicy to use.
     */
    public SwitchRenderer(int switchPolicy)
    {
        this(switchPolicy, true);
    }

    /**
     * Constructor that specifies switchPolicy and whether the renderer is
     * oriented leftToRight.
     * @param switchPolicy The switchPolicy to use.
     * @param orientationLeft If true, the renderer is oriented leftToRight.
     */
	public SwitchRenderer(int switchPolicy, boolean orientationLeft)
	{
        super(orientationLeft);

        setSwitchPolicy(switchPolicy);
        int view = (JCTreeTable.this != null) ? getView() : TREE;
        setSwitchIcon(new SwitchIcon(orientationLeft, view));
        initialized = true;
        reLayout(orientationLeft);
	}

    /**
     * Sets the view for this renderer.  Valid values are <code>TREE</code> or
     * <code>TABLE</code>.
     * @param view The new view for this renderer; must be one of
     *             <code>TREE</code> or <code>TABLE</code>.
     */
    public void setView(int view)
    {
        if (icon instanceof SwitchIcon) {
            ((SwitchIcon)icon).setView(view);
        }
    }

    /**
     * Sets the <code>switchPolicy</code> variable which determines whether or
     * not to allow view switching.
     * @param switchPolicy The new switchPolicy
     */
    public void setSwitchPolicy(int switchPolicy)
    {
        this.switchPolicy = switchPolicy;
    }

    /**
     * Set the icon to switch between <code>TREE</code> and <code>TABLE</code>
     * views.
     * @param icon The new switch icon. Must not be null.
     */
    public void setSwitchIcon(Icon icon)
    {
        if (icon == null) {
            throw new IllegalArgumentException("Icon cannot be null.");
        }
        this.icon = icon;
        if (switchButton != null) {
            switchButton.setIcon(icon);
        }
    }

    /**
     * Return the switch button.
     * @return The switch button.
     */
    protected JButton getSwitchButton()
    {
        return(switchButton);
    }

    /**
     * Set properties on the header renderer component based on the table
     * header.
     * @param table The table whose header is used to set the properties.
     */
    protected void setProperties(JTable table)
    {
        if (table == null) {
            return;
        }
        super.setProperties(table);

        JTableHeader header = table.getTableHeader();
        if (header != null && icon instanceof SwitchIcon) {
            SwitchIcon switchIcon = (SwitchIcon)icon;
            switchIcon.setBackground(header.getBackground());
            switchIcon.setForeground(header.getForeground());
        }
    }

    /**
     * Remove all components and add the components back in based on their
     * availability and the table's component orientation.
     * @param orientationLeft If true, layout in left to right orientation.
     */
	public void reLayout(boolean orientationLeft)
	{
        if (!initialized) {
            return;
        }

		removeAll();

        // Reset orientations on the icon and the label if necessary.
        if (icon instanceof SwitchIcon &&
            orientationLeft != ((SwitchIcon)icon).isOrientationLeft())
        {
            int view = (JCTreeTable.this != null) ? getView() : TREE;
            icon = new SwitchIcon(orientationLeft, view);
            switchButton = null;
        }
        if (orientationLeft != currentLayoutLeft) {
            ComponentOrientation o =
                orientationLeft ? ComponentOrientation.LEFT_TO_RIGHT
                                : ComponentOrientation.RIGHT_TO_LEFT;
            JCTreeTable.setComponentOrientation(label, o, true);
        }

		if (switchPolicy == SWITCH_BUTTON_DONT_SHOW) {
			switchButton = null;
		}
        else if (switchButton == null) {
            switchButton = new JButton(icon);
            switchButton.setBorder(emptyBorder);
        }

        // Layout according to orienation
        if (orientationLeft) {
            if (switchButton != null) {
                add(switchButton);
                add(new JSeparator(SwingConstants.VERTICAL));
            }
            add(label, JCElasticLayout.HORIZONTALLY_ELASTIC_CONSTRAINT);
        }
        if (sortIcon != null) {
            add(sortButton);
            add(brace);
        }
        if (!orientationLeft) {
            add(label, JCElasticLayout.HORIZONTALLY_ELASTIC_CONSTRAINT);
            if (switchButton != null) {
                add(new JSeparator(SwingConstants.VERTICAL));
                add(switchButton);
                add(JCBrace.createHorizontalBrace(1));
            }
        }
        currentLayoutLeft = orientationLeft;
	}
}

/**
 * The icon that renderers itself differently whether the view is
 * <code>TREE</code> or <code>TABLE</code>.
 */
protected static class SwitchIcon implements Icon, java.io.Serializable
{
    protected boolean  orientationLeft = true;
	protected int      view            = TREE;
	protected Color    background      = null;
	protected Color    foreground      = Color.black;

	/**
	 * Constructor.
	 */
	public SwitchIcon() {}

    /**
     * Constructor that specifies the orientation.
     * @param orientationLeft If true, the orientation is left to right.
     * @param view The view of the icon, either <code>TREE</code> or
     * <code>TABLE</code>.
     */
    public SwitchIcon(boolean orientationLeft, int view)
    {
        setOrientationLeft(orientationLeft);
        setView(view);
    }

    /**
     * Return whether the orientation is left to right or not.
     * @return true, if the orientation is left to right, false if the
     * orientation is right to left.
     */
    public boolean isOrientationLeft()
    {
        return(orientationLeft);
    }

    /**
     *
     * @param orientationLeft
     */
    public void setOrientationLeft(boolean orientationLeft)
    {
        this.orientationLeft = orientationLeft;
    }


	/**
	 * Sets the icon view.  Valid values are <code>TREE</code> or
     * <code>TABLE</code>.
	 * @param view The new icon view; must be one of <code>TREE</code> or
     *             <code>TABLE</code>
	 */
	public void setView(int view)
	{
        if (view != TREE && view != TABLE) {
            throw new IllegalArgumentException("View must be one of " +
                                               "JCTreeTable.TREE or " +
                                               "JCTreeTable.Table");
        }
		this.view = view;
	}

	/**
	 * Gets the current icon view.
	 * @return The current icon view; will be one of <code>TREE</code> or
     *         <code>TABLE</code>
	 */
	public int getView()
	{
		return(view);
	}

	/**
	 * Gets the width of the icon.
	 * @return the current width of the icon
	 */
	public int getIconWidth()
	{
		return(ICON_SIZE);
	}

	/**
	 * Gets the height of the icon.
	 * @return the current height of the icon
	 */
	public int getIconHeight()
	{
		return(ICON_SIZE);
	}

	/**
	 * Gets the current background color of the icon.
	 * @return the current background color of the icon
	 */
	public Color getBackground()
	{
		return(background);
	}

	/**
	 * Sets the new background color of the icon.
	 * @param background the new background color of the icon
	 */
	public void setBackground(Color background)
	{
		this.background = background;
	}

	/**
	 * Gets the current foreground color of the icon.
	 * @return the current foreground color of the icon
	 */
	public Color getForeground()
	{
		return(foreground);
	}

	/**
	 * Sets the new foreground color of the icon.
	 * @param foreground the new foreground color of the icon
	 */
	public void setForeground(Color foreground)
	{
		this.foreground = (foreground == null) ? Color.black : foreground;
	}

	/**
	 * Draws the icon.
	 * @param c a component used to aid in drawing icon; currently unused
	 * @param g the graphics object used to draw the icon
	 * @param x the <i>X</i> position at which to draw the icon
	 * @param y the <i>Y</i> position at which to draw the icon
	 */
	public void paintIcon(Component c, Graphics g, int x, int y)
	{
		if (background != null) {
		    g.setColor(background);
			g.fillRect(x, y, getIconWidth(), getIconHeight());
		}
		g.setColor(foreground);
		if (view == TABLE) {
			// Draw an icon of a table
			for (int i = 0; i < ICON_SIZE - 1; i += 3) {
				for (int j = 0; j < ICON_SIZE; j+=4) {
					g.drawLine(x + j, y + i + 1, x + j + 2,  y + i + 1);
					g.drawLine(x + j, y + i + 1 + 1, x + j + 2,  y + i + 1 + 1);
				}
			}
		}
		else {
			// Draw an icon of a tree, based on component orientation

			// Draw first node not indented
            int size = ICON_SIZE - 2;
			int y0 = y + 1;
            int x1 = orientationLeft ? x : x + size;
            int x2 = orientationLeft ? x + 4 + 2 : x + size - (4 + 2);
			g.drawLine(x1, y0, x2, y0);
			g.drawLine(x1, y0 + 1, x2, y0 + 1);

			// Draw second node indented
			y0 += 3;
            x1 = orientationLeft ? x + 3 : x + size - 3;
			g.drawLine(x1, y0, x2, y0);
			g.drawLine(x1, y0 + 1, x2, y0 + 1);

			// Draw third node indented
			y0 += 3;
			g.drawLine(x1, y0, x2, y0);
			g.drawLine(x1, y0 + 1, x2, y0 + 1);

			// Draw fourth node not indented
			y0 += 3;
            x1 = orientationLeft ? x : x + size;
			g.drawLine(x1, y0, x2, y0);
			g.drawLine(x1, y0 + 1, x2, y0 + 1);

			for (int i = 0; i < ICON_SIZE - 1; i += 3) {
				for (int j = 8; j < ICON_SIZE; j += 4) {
                    x1 = orientationLeft ? x + j : x + size - j;
                    x2 = orientationLeft ? x + j + 2 : x + size - (j + 2);
                    y0 = y + i + 1;
					g.drawLine(x1, y0, x2,  y0);
					g.drawLine(x1, y0 + 1, x2, y0 + 1);
				}
			}
		}
	}
} // end of SwitchIcon

/**
 * <code>EmptySelectionModel</code> is a <code>TreeSelectionModel</code> that
 * does not allow anything to be selected.  This class has a static method that
 * returns a static instance; this can be shared, since the model is immutable.
 */
protected static class EmptyTreeSelectionModel extends DefaultTreeTableSelectionModel
{
	protected static EmptyTreeSelectionModel emptyModel =
        new EmptyTreeSelectionModel();

	/**
	 * Returns a shared instance of an empty selection model.
	 * @return a shared instance of an empty tree selection model
	 */
	static public EmptyTreeSelectionModel getSharedInstance() {
		return(emptyModel);
	}

	/**
	 * A null implementation that selects nothing.
	 * @param paths ignored
	 */
	public void setSelectionPaths(TreePath[] paths) {}

	/**
	 * A null implementation that adds nothing.
	 * @param paths ignored
	 */
	public void addSelectionPaths(TreePath[] paths) {}

	/**
	 * A null implementation that removes nothing.
	 * @param paths ignored
	 */
	public void removeSelectionPaths(TreePath[] paths) {}

} // EmptyTreeSelectionModel

}
