/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCWizard.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:28 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.swing;

import com.klg.jclass.util.JCListenerList;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ContainerEvent;
import java.awt.event.ContainerListener;
import java.io.Serializable;
import java.util.Enumeration;

/**
 * Class to create and manage a <i>Wizard</i> style framework.  Create and add
 * <code>JCWizardPages</code> and add <code>WizardListener</code>'s for all
 * your Wizard fun.
 */
public class JCWizard extends BaseWizard implements Serializable {

/**
 * Creates a <code>JCWizard</code> component.
 */
public JCWizard()
{
    cardLayout = new CardLayout();
        setLayout(cardLayout);
        cardOwner = this;
        addContainerListener(containerListener);
}


/**
 * Advances to the next page.
 * @see #previous
 * @see #finish
 * @see #cancel
 * @return whether or not the action was successful
 */
public boolean next()
{
    boolean is_last_page = false;
    Component current_page = null;
    Component next_page = null;

    int ncomponents = getComponentCount();
    for (int i = 0; i < ncomponents; i++) {
        Component comp = getComponent(i);
        if (comp.isVisible()) {
            current_page = comp;
            if (i == ncomponents - 1) {
                is_last_page = true;
                next_page = getComponent(0);
            }
            else {
                next_page = getComponent(i + 1);
            }
            break;
        }
    }

    JCWizardEvent event = new JCWizardEvent(this, current_page, next_page,
                                            is_last_page, !is_last_page);

    // In the preceding constructor, by default, we want
    // to prevent wraparound to first card from the
    // last card so we set "allow_change" to be the
    // opposite of "is_last_page"

    //
    // Post nextBegin event
    //
    if (event != null) {
        Enumeration e = JCListenerList.elements(listeners);
        while (e.hasMoreElements()) {
            JCWizardListener listener = (JCWizardListener)e.nextElement();
            listener.nextBegin(event);
        }
    }

    if (event.getAllowChange() == false) {
        return(false);
    }

    //
    // Advance to the next page unless a new page has been specified
    //
    if (next_page != event.getNewPage()) {
        cardLayout.show(this, event.getNewPage().getName());
    }
    else {
        cardLayout.next(this);
    }

    //
    // Post nextComplete event
    //
    if (event != null) {
        Enumeration e = JCListenerList.elements(listeners);
        while (e.hasMoreElements()) {
            JCWizardListener listener = (JCWizardListener)e.nextElement();
            listener.nextComplete(event);
        }
    }

    return(true);
}

/**
 * Moves to the previous page.
 * @return whether or not the action was successful
 * @see #next
 * @see #finish
 * @see #cancel
 */
public boolean previous()
{

    boolean is_first_page = false;
    Component current_page = null;
    Component previous_page = null;

    int ncomponents = getComponentCount();
    for (int i = 0; i < ncomponents; i++) {
        Component comp = getComponent(i);
        if (comp.isVisible()) {
            current_page = comp;

            if (i == 0) {
                previous_page = getComponent(ncomponents - 1);
                is_first_page = true;
            }
            else {
                previous_page = getComponent(i - 1);
            }
            break;
        }
    }

    JCWizardEvent event = new JCWizardEvent(this, current_page, previous_page,
                                            isLastPage(current_page),
                                         !is_first_page);
    // In the preceding constructor, by default, we want
    // to prevent wraparound to the last card from the
    // first card so we set "allow_change" to be the
    // opposite of "is_first_page"

    //
    // post previousBegin event
    //
    if (event != null) {
        Enumeration e = JCListenerList.elements(listeners);
        while (e.hasMoreElements()) {
            JCWizardListener listener = (JCWizardListener)e.nextElement();
            listener.previousBegin(event);
        }
    }

    if (event.getAllowChange() == false) {
        return(false);
    }

    //
    // Advance to previous page unless a new page has been specified
    //
    if (previous_page != event.getNewPage()) {
        cardLayout.show(this, event.getNewPage().getName());
    }
    else {
        cardLayout.previous(this);
    }

    //
    // Post previousComplete event
    //
    if (event != null) {
        Enumeration e = JCListenerList.elements(listeners);
        while (e.hasMoreElements()) {
            JCWizardListener listener = (JCWizardListener)e.nextElement();
            listener.previousComplete(event);
        }
    }
    return(true);
}

/**
 * Invokes the registered "cancel" action.
 * @see #next
 * @see #previous
 * @see #finish
 * @return whether or not the action was successful
 */
public boolean cancel()
{

    Component comp = getCurrentPage();


    JCWizardEvent event = new JCWizardEvent(this, comp, null,
                                            isLastPage(comp), true);

    //
    // Post Canceled event
    //
    if (event != null) {
        Enumeration e = JCListenerList.elements(listeners);
        while (e.hasMoreElements()) {
            JCWizardListener listener = (JCWizardListener)e.nextElement();
            listener.canceled(event);
        }
    }

    return(event.getAllowChange());
}

/**
 * Invokes the registered "help" action.
 * @see #next
 * @see #previous
 * @see #finish
 * @see #cancel
 */
public void help()
{

    Component comp = getCurrentPage();



    JCWizardEvent event = new JCWizardEvent(this, comp, null,
                                            isLastPage(comp), true);
    //
    // Post Help event
    //
    if (event != null) {
        Enumeration e = JCListenerList.elements(listeners);
        while (e.hasMoreElements()) {
            JCWizardListener listener = (JCWizardListener)e.nextElement();
            listener.help(event);
        }
    }
}


/**
 * Invokes the registered "finish" action.
 * @see #next
 * @see #previous
 * @see #cancel
 * @return whether or not the action was completed
 */
public boolean finish() {

    Component comp = getCurrentPage();


    JCWizardEvent event = new JCWizardEvent(this, comp, null,
            isLastPage(comp), true);

    //
    // Post finished event
    //
    if (event != null) {
        Enumeration e = JCListenerList.elements(listeners);
        while (e.hasMoreElements()) {
            JCWizardListener listener = (JCWizardListener)e.nextElement();
            listener.finished(event);
        }
    }

    return(event.getAllowChange());
}


/**
 * Retrieves the current visible page.
 * @return current visible page
 */
protected Component getCurrentPage()
{
    int ncomponents = getComponentCount();
    for (int i = 0; i < ncomponents; i++) {
        Component comp = getComponent(i);
        if (comp.isVisible()) {
            return(comp);
        }
    }
    return(null);
}




/**
 * Adds a page child.
 * @return the component added.
 * @param page
 */
public Component add(Component page)
{
    add(page, page.getName());
    return(page);
}



/**
 * Container listner that listens for new pages that are added, and adds
 * listeners to the buttons of the children so that the container knows
 * when to post the proper "Wizard" events.
 */
transient protected ContainerListener containerListener =
    new ContainerListener()
    {
        public void componentAdded(ContainerEvent e)
        {
            if (e.getChild() instanceof JCWizardPage) {
                JCWizardPage wp = (JCWizardPage)e.getChild();
                JButton b;
                b = wp.getNextButton();
                if (b != null) {
                    b.addActionListener(nextListener);
                }
                b = wp.getPreviousButton();
                if (b != null) {
                    b.addActionListener(previousListener);
                }
                b = wp.getFinishButton();
                if (b != null) {
                    b.addActionListener(finishListener);
                }
                b = wp.getCancelButton();
                if (b != null) {
                    b.addActionListener(cancelListener);
                }
                b = wp.getHelpButton();
                if (b != null) {
                    b.addActionListener(helpListener);
                }
            }
        }

        public void componentRemoved(ContainerEvent e)
        {
            if (e.getChild() instanceof JCWizardPage) {
                JCWizardPage wp = (JCWizardPage)e.getChild();
                JButton b;
                b = wp.getNextButton();
                if (b != null) {
                    b.removeActionListener(nextListener);
                }
                b = wp.getPreviousButton();
                if (b != null) {
                    b.removeActionListener(previousListener);
                }
                b = wp.getFinishButton();
                if (b != null) {
                    b.removeActionListener(finishListener);
                }
                b = wp.getCancelButton();
                if (b != null) {
                    b.removeActionListener(cancelListener);
                }
                b = wp.getHelpButton();
                if (b != null) {
                    b.removeActionListener(helpListener);
                }
            }
        }
    };


}
