/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: SplitWizardPage.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:28 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.swing;

import com.klg.jclass.util.swing.JCBox;
import com.klg.jclass.util.swing.JCGridLayout;

import javax.swing.*;
import java.awt.*;
import java.io.Serializable;
import java.util.Hashtable;

/**
 * Class to create and manage a Wizard style framework for you.
 */
class SplitWizardPage
        extends BaseWizardPage implements Serializable {


protected JPanel leftPanel, rightPanel;



protected boolean navList;

protected DefaultListModel navigationList;

protected String listTitle;
protected JLabel label;

protected JList stepsList;

protected Hashtable stepsName;




/**
 * Creates a WizardPage with only the selected buttons.
  * @param button_flags flags are final static constant inst from
 * superclass.
 * @param stepsTitle the title of the progresslist. Null if no
 * list is to be generated.
 */
protected SplitWizardPage(int button_flags,
                          String stepsTitle) {

    GridBagLayout gbl = new GridBagLayout();
    GridBagConstraints gbc = new GridBagConstraints();
    setLayout(gbl);

    gbc.gridx = 0;
    gbc.gridy = 0;
    gbc.fill = GridBagConstraints.VERTICAL;
    gbc.gridwidth= 1;
    gbc.ipadx = 0;
    gbc.ipady = 0;
    gbc.insets = new Insets(12, 12, 0, 0);

    leftPanel = new JPanel(new JCGridLayout(1,1, 6, 6));


    add(leftPanel, gbc);
    JPanel rightHolder = new JPanel(new BorderLayout());
    JCBox labelBox = JCBox.createVerticalBox();

    label = new JLabel();
    labelBox.add(label);
    labelBox.add(new JSeparator());
    rightHolder.add(labelBox, BorderLayout.NORTH);
    rightPanel = new JPanel();

    rightHolder.add(rightPanel, BorderLayout.CENTER);
    gbc = new GridBagConstraints();
    gbc.fill = GridBagConstraints.BOTH;
    gbc.gridx = 1;
    gbc.gridy = 0;
    gbc.weightx = 1;
    gbc.weighty = 1;
    gbc.gridwidth = 2;
    gbc.ipadx = 0;
    gbc.ipady = 0;
    gbc.insets = new Insets(12,6, 0, 11);


    add(rightHolder, gbc);



    JSeparator jSep = new JSeparator();
    gbc = new GridBagConstraints();
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.gridheight = GridBagConstraints.RELATIVE;
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.weightx = 1;
    gbc.weighty = 0;
    gbc.gridx = 0;
    gbc.gridy = 1;
    gbc.ipadx = 0;
    gbc.ipady = 5;



    add(jSep, gbc);
    gbc = new GridBagConstraints();
    gbc.weightx = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = GridBagConstraints.REMAINDER;
    gbc.gridx = 0;
    gbc.gridy = 2;
    gbc.ipadx = 0;
    gbc.ipady = 0;
    gbc.insets = new Insets(0, 12, 11,0);


    gbc.anchor = GridBagConstraints.SOUTHWEST;
    gbc.weighty = 0;
    gbc.fill = GridBagConstraints.HORIZONTAL;

    JPanel jp = new JPanel(new BorderLayout());



    add(jp, gbc);
    gbc = new GridBagConstraints();
    gbc.anchor = GridBagConstraints.SOUTHEAST;
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.gridx = 1;
    gbc.gridy =2;

    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.ipadx = 0;
    gbc.ipady = 0;
    gbc.insets = new Insets(0, 0, 11, 11);

    createButtonBox(button_flags, false);
    gbl.setConstraints(buttonBox, gbc);
    add(buttonBox);

    //if stepsTitle is not new, adds stepslist to rightHandPanel.
    if (stepsTitle != null){
        addProgressList(stepsTitle);


    }
    else {
        stepsList = null;
        stepsName = null;
    }

}

/**
 * Adds a progress list with the selected title. Only called if
 * a stepsTitle is provided to the constructor.
  * @param title
 */
protected  void addProgressList(String title){
    navList = true;
    navigationList = new DefaultListModel();
    stepsName = new Hashtable();
    listTitle = title;

    leftPanel.setLayout(new JCGridLayout(3, 1, 6,6));
    leftPanel.setOpaque(true);
    leftPanel.setBackground(Color.white);
    JLabel label =  new JLabel(listTitle);
    label.setOpaque(false);
    leftPanel.add(label);
    leftPanel.add(new JSeparator());
    stepsList = new JList(navigationList);
    stepsList.setEnabled(false);

    stepsList.setSelectedIndex(0);
    JScrollPane scrollPane = new JScrollPane(stepsList,
            JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
            JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    scrollPane.setBorder(null);
    leftPanel.add(scrollPane);





}

/**
 * if  a navigation list is being generated, this method will update the
 * selected item to reflect the current page.
  * @param name
 */
protected void updateStepSelection(String name) {
    if (stepsList != null){
        stepsList.setSelectedValue(stepsName.get(name), true);
    }
    setRightPanelTitle(name);



}

/**
 *  Gets the left Panel
  * @return
 */
public JPanel getLeftPanel() {
    return leftPanel;
}

protected JPanel getRightPanel(){
    return rightPanel;
}


public DefaultListModel getNavigationList() {
    return navigationList;
}


/**
 * Sets the title.
  * @param title
 */
public void setRightPanelTitle(String title) {
    label.setText(title);

}

public void addToNavList(String title){
    if (navList){
        String navName = (navigationList.size()+1) + ". "
                    +title;
        navigationList.addElement(navName);
        stepsName.put(title, navName);

        leftPanel.setMinimumSize(leftPanel.getPreferredSize());
        leftPanel.setMaximumSize(leftPanel.getPreferredSize());
        stepsList.setSelectedIndex(0);




    }


}
}
