/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: DefaultLegendPopulatorRenderer.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:29 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.swing.gauge;

import com.klg.jclass.util.legend.JCLegend;
import com.klg.jclass.util.legend.JCLegendItem;
import com.klg.jclass.util.legend.JCLegendPopulator;
import com.klg.jclass.util.legend.JCLegendRenderer;

import java.awt.*;
import java.util.Enumeration;
import java.util.List;
import java.util.Vector;

/**
 * This is the default implmentation of both <code>JCLegendPopulator</code> and
 * <code>JCLegendRenderer</code>. These interfaces, respectively, control the
 * creation and display characteristics of the <code>JCLegendItem<code>s.
 *
 * @see com.klg.jclass.util.legend.JCLegend#setLegendPopulator
 * @see com.klg.jclass.util.legend.JCLegend#setLegendRenderer
 */
public class DefaultLegendPopulatorRenderer implements JCLegendRenderer,
		                                               JCLegendPopulator,
                                                       java.io.Serializable
{

static final long serialVersionUID = 6132225179285554488L;

/**
 * The <code>Vector</code> of ranges which are used to create corresponding
 * <code>JCLegendItems</code>
 */
protected Vector ranges;

/**
 * The <code>JCLegend</code> for which this class will populate and render
 * <code>JCLegendItems</code>
 */
protected JCLegend legend;

/** The outline color to use when drawing this legend item's symbol */
protected Color outlineColor = Color.black;

/**  */
protected String legendTitle;

/**
 * Creates a Populator and Renderer for a <code>JCLegend</code>.
 *
 * @param legend The <code>JCLegend</code> for which this class will populate
 *     and render <code>JCLegendItems</code>
 * @param ranges The <code>Vector</code> of ranges which are used to create
 *      corresponding <code>JCLegendItems</code>
 */
public DefaultLegendPopulatorRenderer(JCLegend legend, Vector ranges)
{
	this.ranges = ranges;
	this.legend	= legend;
}

///////////// Implement the JCLegendPopulator interface

/**
 * Creates legend items for each range listed in the ranges list.
 * This uses the range name and range foreground color to create the
 * <code>JCLegendItem</code>s.
 *
 * @param fm FontMetrics Information about the font in which legend text will
 *     be written
 */
public List getLegendItems(FontMetrics fm)
{
    Vector listOfItemLists = new Vector();
    Vector items = new Vector(ranges.size());

    int symbolSize = legend.getSymbolSize();
    int height = fm.getHeight();

    // add a title legend item
    if (legendTitle != null) {
        JCLegendItem titleItem = createLegendItem(legendTitle, symbolSize,
                                                  fm.stringWidth(legendTitle),
                                                  height, null);
        items.addElement(titleItem);
    }

    // Now add an item for each range
    Enumeration e = ranges.elements();
    while (e.hasMoreElements()) {
        // Get the range information
        JCRange range = (JCRange)e.nextElement();
        String name = range.getRangeName();
        int width = fm.stringWidth(name);

        // Create the legend item
        JCLegendItem i = createLegendItem(name, symbolSize, width, height,
                                          range.getForeground());

        // Finally add it
        items.addElement(i);
    }
    listOfItemLists.addElement(items);
    return(listOfItemLists);
}

/**
 * Creates a legend item.
 *
 * @param name The name of the item
 * @param symbolSize The symbol size
 * @param width The width of the item
 * @param height The height of the item
 * @param c The color of the item
 */
protected JCLegendItem createLegendItem(String name, int symbolSize,
                                        int width, int height, Color c)
{
    // And create a JCLegendItem from it
    JCLegendItem li = new JCLegendItem();
    li.symbolDim = new Dimension(symbolSize, symbolSize);
    li.textDim = new Dimension(width, height);
    li.contents = name;
    li.drawType = JCLegend.BOX;
    li.itemInfo = c;
    return(li);
}

/**
 * Since the title is just another <code>JCLegendItem</code> we need
 * a way to identify it as the title. This method indicates
 * if an item is a title. In this case a null <code>itemInfo</code> indicates
 * that this is a title.
 *
 * @param item The item to check
 * @return true If this item is the legend title
 */
public boolean isTitleItem(JCLegendItem item)
{
    return item.itemInfo == null;
}

///////////// implement the JCLegendRenderer interface

/**
 * For implementing custom legend item draws.  For use with CUSTOM_ALL
 * draw type.
 *
 * @param gc graphics object used to draw the legend
 * @param useFont the font being used in this legend
 * @param thisItem the <code>JCLegendItem</code> object representing the
 *     legend item to draw
 */
public void drawLegendItem(Graphics gc, Font useFont, JCLegendItem thisItem)
{
    // no implementation necessary
}

/**
 * For implementing custom legend item symbol draws.  For use with
 * <code>CUSTOM_SYMBOL</code> draw type.
 *
 * @param gc graphics object used to draw the legend
 * @param useFont the font being used in this legend
 * @param thisItem the <code>JCLegendItem</code> object representing the
 *     legend item to draw
 */
public void drawLegendItemSymbol(Graphics gc, Font useFont,
                                 JCLegendItem thisItem)
{
    // no implementation necessary
}

/**
 * Used for implementing custom text draws for a legend item.  This method
 * is called when a legend item's contents field contains an object other
 * than a String object.
 *
 * @param gc Graphics object used to draw the legend
 * @param useFont the font being used in this legend
 * @param thisItem the <code>JCLegendItem</code> object representing the
 *     legend item to draw.
 */
public void drawLegendItemText(Graphics gc, Font useFont,
                               JCLegendItem thisItem)
{
    // no implementation necessary
}

/**
 * Gets the outline color to use when drawing this legend item's symbol. If
 * null, the legend will use its own foreground color.
 *
 * @param thisItem the <code>JCLegendItem</code> object representing the
 *     legend item to draw.
 */
public Color getOutlineColor(JCLegendItem thisItem)
{
    return(outlineColor);
}

/**
 * Sets the outline color to use when drawing this legend item's symbol. If
 * null, legend will use its own foreground color. The default is black.
 *
 * @param outlineColor
 */
public void setOutlineColor(Color outlineColor)
{
    this.outlineColor = outlineColor;
}

/**
 * Sets the provided <code>Graphics</code> object with the appropriate settings
 * to use when drawing this legend item's symbol. In this case it sets
 * the <code>gc</code> color to the range's foreground color.
 *
 * @param gc
 * @param thisItem
 */
public void setFillGraphics(Graphics gc, JCLegendItem thisItem)
{
    gc.setColor((Color)thisItem.itemInfo);
}

/**
 * Returns this legend's title.
 *
 * @return the title
 */
public String getLegendTitle()
{
    return legendTitle;
}

/**
 * Sets the legend's title.
 *
 * @param legendTitle The legend's title
 */
public void setLegendTitle(String legendTitle)
{
    this.legendTitle = legendTitle;
}

}
