/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: GaugeConstraint.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:29 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.swing.gauge;

/**
 * A <code>GaugeConstraint</code> is one that provides layout
 * information for a component that needs to be placed on
 * a circular or linear gauge. It provides information on the
 * radial or linear extent of the component.
 */
public abstract class GaugeConstraint implements java.io.Serializable {

static final long serialVersionUID = 8309996236706274034L;

/**
 * For circular scales, the ratio of the radius away from the center at
 * which to place this label.  For linear scales, the ratio of the width
 * or height away from the top left of the scale.
 */
protected double extent;

/**
 * The gauge, upon whose gauge area this constraint is based.
 */
protected JCGauge gauge;

/**
 * Constructor for <code>GaugeConstraint</code> using a <code>JCGauge</code>
 * as the component.
 *
 * @param gauge The associated gauge
 * @param extent For circular scales, the ratio of the radius away from the
 *     center at which to place a label.  For linear scales, the ratio of the
 *     width or height away from the top left of the scale.
 */
public GaugeConstraint(JCGauge gauge, double extent)
{
	this.gauge		= gauge;
    this.extent		= extent;
}

/**
 * Returns the extent at which this component will be placed.
 *
 * @return The extent at which this component will be placed
 */
public double getExtent()
{
	return(extent);
}

/**
 * Sets the extent at which this component will be placed.
 *
 * @param extent The extent at which to place this component
 */
public void setExtent(double extent)
{
	this.extent = extent;
}

/**
 * Returns the <code>JCGauge</code> on which this constraint will be used
 *
 * @return The gauge on which this constraint will be used
 */
public JCGauge getGauge()
{
	return(gauge);
}

}
