/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCCircularIndicator.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:29 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.swing.gauge;

import java.awt.*;

/**
 * A indicator points to a given value on its scale.
 * There may be multiple indicators associated with a scale.
 *
 * @see com.klg.jclass.swing.gauge.JCIndicatorStyle
 */
public class JCCircularIndicator extends JCAbstractIndicator {

static final long serialVersionUID = 6509202418618173656L;

/**
 * Creates a indicator.
 *
 * @param scale the associated circular scale
 */
public JCCircularIndicator(JCCircularScale scale)
{
    super(scale);
}

/**
 * Creates a indicator.
 *
 * @param foreground the indicator's color
 * @param indicatorWidth the indicator's width in pixels
 * @param scale the associated circular scale
 * @param visible a Boolean that controls the visibility of the indicator
 * @param inner_extent the inner extent of the indicator expressed as a ratio
 *      of its value to the radius of the associated circular scale
 * @param outer_extent the outer extent of the indicator expressed as a ratio
 *      of its value to the radius of the associated circular scale
 * @param indicatorStyle indicator styles are kept in
 *     <code>JCIndicatorStyle</code>.; user-defined indicator styles  may be
 *     passed to <code>JCIndicatorStyle</code>'s constructor
 * @param value the indicator's value; the indicator points to this value on its
 *     associated scale
 */
public JCCircularIndicator(Color						foreground,
                           double						indicatorWidth,
                           JCCircularScale				scale,
                           boolean						visible,
                           double						inner_extent,
                           double						outer_extent,
                           JCIndicatorStyle			    indicatorStyle,
                           double						value)
{
    super(foreground, indicatorWidth, scale, visible,
          inner_extent, outer_extent, indicatorStyle, value);
}

/**
 * Returns the circular scale associated with this indicator.
 *
 * @return the circular scale associated with this indicator
 */
protected JCCircularScale getCircularScale()
{
	return((JCCircularScale)getScale());
}

/**
 * This routine draws the circular indicator object on the screen.
 *
 * @param g
 */
public void paint(Graphics g)
{
    JCCircularScale cscale = getCircularScale();

    // Do we need to draw anything?
    if (!isVisible()                        ||
        cscale == null                      ||
        cscale.getMin() >= cscale.getMax()  ||
        inner_extent >= outer_extent        ||
        indicatorWidth <= 0)
    {
        return;
    }

    if (indicatorStyle == JCIndicatorStyle.CIRCLE) {
        GaugeUtil.drawCircleForCircularScale(g, cscale, getForeground(), value,
                                             inner_extent, outer_extent);
    }
    else {
        GaugeUtil.drawCircularPolygon(g, cscale, getForeground(),
                                      inner_extent, outer_extent,
                                      indicatorWidth, value, reversed, false,
                                      indicatorStyle);
    }
}

} // JCLinearIndicator
