/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCCircularNeedle.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:29 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.swing.gauge;

import java.awt.*;

/**
 * A needle points to values on its scale.
 * There may be multiple needles associated with a scale.
 *
 * @see com.klg.jclass.swing.gauge.JCIndicatorStyle
 */
public class JCCircularNeedle extends JCAbstractNeedle
{

static final long serialVersionUID = -1480029503570557918L;

/**
 * Creates a needle.
 *
 * @param scale The associated circular scale
 */
public JCCircularNeedle(JCCircularScale scale)
{
    super(scale);
}

/**
 * Creates a needle.
 *
 * @param foreground the needle's color
 * @param needleWidth the needle's width in pixels
 * @param scale the associated circular scale
 * @param visible a Boolean that controls the visibility of the needle
 * @param inner_extent the needle's inner extent, expressed as a ratio of its length to
 *     the relevant dimension (radius, width/length) of the associated circular scale
 * @param outer_extent the needle's outer extent, expressed as a ratio of its length to
 *     the relevant dimension (radius, width/length) of the associated circular scale
 * @param needleStyle needle styles are kept in <code>JCIndicatorStyle</code>;
 *     user-defined needle styles  may be passed to
 *     <code>JCIndicatorStyle</code>'s constructor
 * @param value the needle's value; the needle points to this value on its
 *     associated scale
 */
public JCCircularNeedle(Color						foreground,
                        double						needleWidth,
                        JCCircularScale				scale,
                        boolean						visible,
                        double						inner_extent,
                        double						outer_extent,
                        JCIndicatorStyle			needleStyle,
                        double						value)
{
    super(foreground, needleWidth, scale, visible, inner_extent,
          outer_extent, needleStyle, value);
}

/**
 * Creates a needle.
 *
 * @param foreground the needle's color
 * @param needleWidth the needle's width in pixels
 * @param scale the associated circular scale
 * @param interactionType the interaction type;
 * <code>CLICK</code>, <code>CLICK_DRAG</code>, or <code>NONE</code>
 * @param visible a Boolean that controls the visibility of the needle
 * @param length the needle's length, expressed as a ratio of its length to
 *     the relevant dimension (radius, width/length) of the associated scale
 * @param needleStyle needle styles are kept in <code>JCIndicatorStyle</code>;
 *     user-defined needle styles  may be passed to
 *     <code>JCIndicatorStyle</code>'s constructor
 * @param value the needle's value; the needle points to this value on its
 *     associated scale
 * @deprecated Beginning with JClass 5, the <code>length</code> parameter has
 *  been replaced with <code>inner_extent</code> and <code>outer_extent</code>.
 */
public JCCircularNeedle(Color						foreground,
                        double						needleWidth,
                        JCCircularScale				scale,
                        InteractionType				interactionType,
                        boolean						visible,
                        double						length,
                        JCIndicatorStyle			needleStyle,
                        double						value)
{
    super(foreground, needleWidth, scale, visible, 0.0, length,
	      needleStyle, value);
    setInteractionType(interactionType);
}

/**
 * Returns the circular scale associated with this needle.
 *
 * @return the circular scale associated with this needle
 */
protected JCCircularScale getCircularScale()
{
	return((JCCircularScale)getScale());
}

/**
 * This routine draws the circular needle object on the screen.
 *
 * @param g
 */
public void paint(Graphics g)
{
    JCCircularScale cscale = getCircularScale();

    // Do we need to draw anything?
    if (!isVisible()                        ||
        cscale == null                      ||
        cscale.getMin() >= cscale.getMax()  ||
        inner_extent >= outer_extent        ||
        indicatorWidth <= 0)
    {
        return;
    }

    if (indicatorStyle == JCIndicatorStyle.CIRCLE) {
        GaugeUtil.drawCircleForCircularScale(g, cscale, getForeground(), value,
                                             inner_extent, outer_extent);
    }
    else {
        GaugeUtil.drawCircularPolygon(g, cscale, getForeground(),
                                      inner_extent, outer_extent,
                                      indicatorWidth, value, reversed, false,
                                      indicatorStyle);
    }
}

}
