/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCGaugeArea.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:29 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.swing.gauge;

import javax.swing.*;
import java.awt.*;

/**
 * This is the sub-portion of the overall gauge real estate onto which the
 * actual linear/circular portion of the gauge is drawn. Headers, footers,
 * and legends are not drawn in this area; scales, ticks and labels are.
 */
public class JCGaugeArea extends JComponent implements java.io.Serializable
{

static final long serialVersionUID = 2667771882493917459L;

/**
 * The gauge to be placed in this gauge area.
 */
protected JCGauge gauge;

/**
 * Asks: "Is this a circular or linear gauge?".
 */
protected boolean isCircularGauge;

/**
 * Creates a gauge area for a gauge.
 *
 * @param gauge the <code>JCGauge</code> whose scale will be drawn on this area
 */
protected JCGaugeArea(JCGauge gauge)
{
    isCircularGauge = (gauge instanceof JCCircularGauge);
    if (isCircularGauge) {
	    setLayout(new RadialLayout());
    }
    else {
	    setLayout(new LinearLayout());
    }
	setOpaque(false);

	this.gauge = gauge;
}

/**
 * Overrides add to pass in a constraint.
 * @param c The component to be added
 * @return The component argument
 */
public Component add(Component c)
{
    if (isCircularGauge) {
	    add(c, new RadialConstraint(gauge, Double.MAX_VALUE, Double.MAX_VALUE));
    }
    else {
	    add(c, new LinearConstraint(gauge, Double.MAX_VALUE, Double.MAX_VALUE));
    }
    return(c);
}

/**
 * Overrides add to pass in constraint.
 * @param c The component to be added
 * @param index The index specifing the drawing order; components added last
 *     are drawn first
 * @return The component argument
 */
public Component add(Component c, int index)
{
    if (isCircularGauge) {
	    add(c, new RadialConstraint(gauge, Double.MAX_VALUE, Double.MAX_VALUE),
            index);
    }
    else {
	    add(c, new LinearConstraint(gauge, Double.MAX_VALUE, Double.MAX_VALUE),
            index);
    }
    return(c);
}

/**
 * Is responsible for rendering the <code>JCChart3dArea</code> contents.
 * @param gc graphics context used for drawing
 */
public void paintComponent(Graphics gc)
{
    if (gc == null || !isVisible()) {
        return;
    }

	// Draw own opaque backgrounds.  Shouldn't this be
	// done by the superclass?  That code appears to be in the UI, which
	// is null for this component it seems.
	if (isOpaque()) {
        if (!gc.getColor().equals(getBackground())) {
            gc.setColor(getBackground());
        }
		Rectangle r = getBounds();
        gc.fillRect(0, 0, r.width, r.height);
	}
}

}
