/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCGaugePickEvent.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:29 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.swing.gauge;

import java.util.EventObject;
import java.awt.Point;
import java.awt.Component;

/**
 * Represents a pick event in <code>JCGauge</code>.  A pick event occurs when a
 * <code>JCPickListener</code> is installed on a gauge and the mouse button is
 * pressedover a <code>JCGauge</code> object.
 */
public class JCGaugePickEvent extends EventObject
{

/**
 * The gauge associated with this event.
 */
protected JCGauge gauge;

/**
 * The component at the click location.
 */
protected Component comp;

/**
 * The scale value closest to this click.
 */
protected double value;

/**
 * The (<i>x</i>, <i>y</i>) point of the click.
 */
protected Point point;

/**
 * Constructor.
 *
 * @param gauge the gauge associated with this event
 * @param comp the component at the click location
 * @param value the scale value at the pick location
 * @param point the <i>x</i>, <i>y</i> coordinate of the pick
 */
public JCGaugePickEvent(JCGauge gauge, Component comp, double value,
                        Point point)
{
	super(gauge);

    this.gauge	= gauge;
    this.comp	= comp;
    this.value	= value;
    this.point	= point;
}

/**
 * Returns the gauge associated with this event.
 *
 * @return the gauge associated with this event
 */
public JCGauge getGauge()
{
	return(gauge);
}

/**
 * The component associated with this event.
 * @return
 */
public Component getComponent()
{
	return(comp);
}

/**
 * The value associated with this event.
 * The value is found by transforming
 * the (<i>x</i>, <i>y</i>) point of the click
 * to the corresponding scale value.
 * @return
 */
public double getValue()
{
	return(value);
}

/**
 * Returns the (<i>x</i>, <i>y</i>) point of the click.
 * @return
 */
public Point getPoint()
{
	return(point);
}

/**
 * Returns the point where the mouse was clicked and the associated scale value
 * as a String.
 *
 * @return The point where the mouse was clicked and the associated scale
 *     value as a String
 */
public String toString()
{
	return(new String("value = " + value + " point = " + point));
}

}
