/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCIndicator.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:29 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.swing.gauge;

/**
 * <code>JCIndicator</code> interface for indicators.  Those wishing to use
 * their own indicators should implement this interface.
 */
public interface JCIndicator
{

/**
 * Returns the <code>JCIndicatorStyle</code> for this indicator.
 *
 * @return the <code>JCIndicatorStyle</code> of this indicator
 */
public JCIndicatorStyle getIndicatorStyle();

/**
 * Returns the indicator width in pixels.
 *
 * @return the indicator width in pixels
 */
public double getIndicatorWidth();

/**
 * Returns the inner extent of this indicator as a ratio of the
 * scale's width, height, or radius (depending on the scale's type and
 * orientation). For example, in the circular case, 0.75 means draw out
 * to 75% of the scale's radius.
 *
 * @return The <code>inner_exent</code> of this indicator as a ratio of the
 *         scale's width, height, or radius (depending on the scale's type and
 *         orientation)
 */
public double getInnerExtent();

/**
 * Returns the outer extent of this indicator as a ratio of the
 * scale's width, height, or radius (depending on the scale's type and
 * orientation). For example, in the circular case, 0.75 means draw out
 * to 75% of the scale's radius.
 *
 * @return The <code>outer_exent</code> of this indicator as a ratio of the
 *         scale's width, height, or radius (depending on the scale's type and
 *         orientation)
 */
public double getOuterExtent();

/**
 * Returns the scale value at which this indicator is currently
 * pointing.
 *
 * @return the current value pointed to by this indicator
 */
public double getValue();

/**
 * Returns the scale associated with this indicator.
 *
 * @return the scale for the <code>JCAbstractIndicator</code> component
 */
public JCScale getScale();

/**
 * Asks :"Is the indicator reversed?".
 *
 * @return whether the indicator is reversed or not
 */
public boolean isReversed();

/**
 * Sets the <code>JCIndicatorStyle</code> for this indicator.
 *
 * @param indicatorStyle the <code>JCIndicatorStyle</code> for this indicator
 */
public void setIndicatorStyle(JCIndicatorStyle indicatorStyle);

/**
 * Sets the indicator width in pixels.
 *
 * @param indicatorWidth the indicator width in pixels
 */
public void setIndicatorWidth(double indicatorWidth);

/**
 * Sets the inner extent of this indicator as a ratio of the
 * scale's width, height, or radius (depending on the scale's type and
 * orientation). For example, in the circular case, 0.75 means draw out
 * to 75% of the scale's radius.
 *
 * @param inner_extent the inner extent of this indicator as a ratio of the
 *        scale's width, height, or radius (depending on the scale's type and
 *        orientation)
 */
public void setInnerExtent(double inner_extent);

/**
 * Sets the outer extent of this indicator as a ratio of the
 * scale's width, height, or radius (depending on the scale's type and
 * orientation). For example, in the circular case, 0.75 means draw out
 * to 75% of the scale's radius.
 *
 * @param outer_extent the outer extent of this indicator as a ratio of the
 *        scale's width, height, or radius (depending on the scale's type and
 *        orientation)
 */
public void setOuterExtent(double outer_extent);

/**
 * Sets the reversed property of the indicator.  If the indicator is
 * reversed it draws in the opposite direction (for example, for linear
 * scales it points left or up instead of right or down, for circular
 * scales it points towards the center instead of away from the center).
 *
 * @param reversed is the indicator's direction reversed
 */
public void setReversed(boolean reversed);

/**
 * Sets the scale value to which this indicator should point.
 *
 * @param value the value at which to point
 */
public void setValue(double value);

}
