/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCIndicatorStyle.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:29 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.swing.gauge;

/**
 * Provides constants for the built-in indicator styles for a <code>JCIndicator</code>
 * and provides a constructor for defining a user-defined indicator style.
 * Alternatively, custom indicator styles may be defined by subclassing.
 */
public class JCIndicatorStyle extends JCPolygon implements java.io.Serializable {

static final long serialVersionUID = 2928285666017554237L;

/**
 * A pointer indicator using these points.
 * <P>
 * <pre>
 * new int[] { 0,	0,	90,	100,	90,	0}
 * new int[] { -5, 5,	5,	0,		-5,	-5}
 *
 * --------------*
 * |               \
 * |                *
 * |               /
 * --------------*
 * </pre>
 *
 */
public static final JCIndicatorStyle POINTER =
    new JCIndicatorStyle(
        new int[]{0, 0, 90, 100, 90, 0},
        new int[]{-5, 5, 5, 0, -5, -5},
        6);

/**
 * Arrow polygon using these points.
 * <P>
 * <pre>
 * new int[] { 0,	0,	100,	100,	120,	100,	100,	0},
 * new int[] { 5, -5,	-5,		-10,	0,		10,		5,		5},
 *
 *
 * |\
 * --------* \
 * |          *
 * --------* /
 * |/
 * </pre>
 *
 */
public static final JCIndicatorStyle ARROW =
    new JCIndicatorStyle(
        new int[]{0, 0, 100, 100, 120, 100, 100, 0},
        new int[]{5, -5, -5, -10, 0, 10, 5, 5},
        8);

/**
 * Triangle using ponits.
 * <P>
 * <pre>
 * new int[] {0,	100,	0},
 * new int[] {-7,	0,		7},
 *
 * *
 * | \
 * |  *
 * | /
 * *
 *
 * </pre>
 *
 */
public static final JCIndicatorStyle TRIANGLE =
    new JCIndicatorStyle(
        new int[]{0, 100, 0},
        new int[]{-7, 0, 7},
        3);

/** Rectangle */
public static final JCIndicatorStyle RECTANGLE =
    new JCIndicatorStyle(
        new int[]{0, 0, 10, 10},
        new int[]{5, -5, -5, 5},
        4);

/** Circle */
public static final JCIndicatorStyle CIRCLE = new JCIndicatorStyle();

/**
 * A tailed pointer indicator (tail extends beyond center). It uses
 * these points.
 * <P>
 * <pre>
 * new int[] { -20,		-20,	90,	100,	90,	-20}
 * new int[] { -5,		5,		5,	0,		-5,	-5}
 *
 *
 * --------------*
 * |               \
 * |   x(Center)    *
 * |               /
 * --------------*
 * </pre>
 *
 */
public static final JCIndicatorStyle TAILED_POINTER =
    new JCIndicatorStyle(
        new int[]{-20, -20, 90, 100, 90, -20},
        new int[]{-5, 5, 5, 0, -5, -5},
        6);

/**
 * Tailed Arrow polygon (tail extends beyond center).
 * <P>
 * <pre>
 * new int[] { -20,	-20,	100,	100,	120,	100,	100,	-20},
 * new int[] { 5, -5,	-5,		-10,	0,		10,		5,		5},
 *
 *
 * |\
 * ---------------------* \
 * |  x(Center)            *
 * ---------------------* /
 * |/
 * </pre>
 *
 */
public static final JCIndicatorStyle TAILED_ARROW =
    new JCIndicatorStyle(
        new int[]{-20, -20, 100, 100, 120, 100, 100, -20},
        new int[]{5, -5, -5, -10, 0, 10, 5, 5},
        8);

/**
 * Construct a indicator using the points of a polygon.
 * For a user-defined indicator.
 *
 * @param xpoints the <i>x</i> coordinates for the polygon
 * @param ypoints the <i>y</i> coordinates for the polygon
 * @param numPoints the number of points in this polygon
 */
public JCIndicatorStyle(int xpoints[], int ypoints[], int numPoints)
{
    super(xpoints, ypoints, numPoints);
}

/** Default constuctor */
public JCIndicatorStyle()
{
    super();
}

}
