/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCLinearGauge.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:29 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.swing.gauge;

import com.klg.jclass.util.ServerRenderable;
import com.klg.jclass.util.ServerRenderer;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.util.Enumeration;
import java.util.Vector;

/**
 * <code>JCLinearGauge</code>.
 */
public class JCLinearGauge extends JCGauge
                           implements ServerRenderable
{

/**
 * The list of labels for this gauge.
 *
 * @see com.klg.jclass.swing.gauge.JCLabelGenerator
 */
protected Vector 		labels = new Vector();

/** Temporary Graphics object used to draw without showing to screen.*/
private Graphics        prevSetGraphics = null;

/**
 * No-argument constructor for <code>JCLinearGauge</code>.
 * Creates a default scale, needle, and tick marks.
 *
 * @see #setScale(JCLinearScale)
 * @see #addIndicator(JCAbstractIndicator)
 * @see #addNeedle(JCAbstractNeedle)
 * @see com.klg.jclass.swing.gauge.JCAbstractScale#addRange(JCRange)
 * @see #addTick(JCTick)
 * @see #getScale()
 * @see #removeIndicator(JCAbstractIndicator)
 * @see #removeNeedle(JCAbstractNeedle)
 * @see com.klg.jclass.swing.gauge.JCAbstractScale#removeRange(JCRange)
 * @see #removeTick(JCTick)
 */
public JCLinearGauge()
{
	// create a scale for a full circle
    JCLinearScale lscale = new JCLinearScale(this);
	setScale(lscale, false);

	// create a needle
    JCLinearNeedle needle = new JCLinearNeedle(lscale);
    addNeedle(needle);

    // default ticks
	JCLinearTick tick = new JCLinearTick(lscale);
    addTick(tick);

    // Now add scale to gauge area (so its drawn first)
    gaugeArea.add(lscale);

    // set the size
    setPreferredSize(new Dimension(200, 100));
}

/**
 * Constructs a <code>JCLinearGauge</code> with no default objects except
 * possibly a linear scale.
 *
 * Use the following methods to add needles, indicators, ticks, and ranges.
 * @see #addIndicator(JCAbstractIndicator)
 * @see #addNeedle(JCAbstractNeedle)
 * @see com.klg.jclass.swing.gauge.JCAbstractScale#addRange(JCRange)
 * @see #addTick(JCTick)
 * @see #getScale()
 * @see #removeIndicator(JCAbstractIndicator)
 * @see #removeNeedle(JCAbstractNeedle)
 * @see com.klg.jclass.swing.gauge.JCAbstractScale#removeRange(JCRange)
 * @see #removeTick(JCTick)
 * @param createScale
 */
public JCLinearGauge(boolean createScale)
{
	if (createScale) {
	    // create a linear scale
        JCLinearScale lscale = new JCLinearScale(this);
	    setScale(lscale);
    }
}

/**
 * Paints the gauge.
 *
 * @param g The <code>Graphics</code> with which to paint
 */
public void paintComponent(Graphics g)
{
    // Draw a rectangle with gauge background
    Rectangle bounds = getBounds();
    if ( isOpaque() ) {
        g.setColor(getBackground());
        g.fillRect(bounds.x, bounds.y, bounds.width, bounds.height);
    }

    // Draw a background image if there is one
    if(image != null) {
        drawImage(g, image, getBounds(), getBackground());
    }

    // force the center to reposition itself
	gaugeArea.doLayout();

    // continue with normal painting
    super.paintComponent(g);


 
}

/**
 * Returns the linear gauge's linear scale (appropriately cast).
 *
 * @return the linear gauge's linear scale
 */
public JCLinearScale getLinearScale()
{
    return((JCLinearScale)getScale());
}

/**
 * Adds a linear scale to the linear gauge.
 *
 * @param scale the linear scale to be added
 * @param addToGaugeArea Add the scale to the <code>JCGaugeArea</code> parent?
 */
public void setScale(JCLinearScale scale, boolean addToGaugeArea)
{
    // Out with the old
	if (this.scale != null) {
    	gaugeArea.remove((Component)this.scale);
    }
    // In with the new
	this.scale = scale;
    if (addToGaugeArea == true) {
        gaugeArea.add(scale);
    }
    setIndicatorScales(scale);
    setNeedleScales(scale);
}

/**
 * Adds a linear scale to the linear gauge.
 *
 * @param scale the linear scale to be added
 */
public void setScale(JCLinearScale scale)
{
    setScale(scale, true);
}

/**
 * Adds a label to the gauge.
 *
 * @param label the <code>JComponent</code> to be added as a label
 * @param constraint the <code>LinearConstraint</code> specifying where to
 *     place the label
 */
public void addLabel(JComponent label, LinearConstraint constraint)
{
	labels.addElement(label);
    getGaugeArea().add(label, constraint, 0);
}

/**
 * Adds a label to the gauge.
 *
 * @param label the <code>JComponent</code> to be added as a label
 * @param constraint the <code>LinearConstraint</code> specifying where to
 *     place the label
 * @param index the <i>z</i>-order index
 */
public void addLabel(JComponent label, LinearConstraint constraint, int index)
{
	labels.addElement(label);
    getGaugeArea().add(label, constraint, index);
}

/**
 * Removes a label.
 *
 * @param label the <code>JComponent</code> to be removed
 */
public void removeLabel(JComponent label)
{
	if (labels.contains(label)) {
    	labels.removeElement(label);
	    getGaugeArea().remove(label);
    }
}

/**
 * Sends pick events to listeners and moves the closest
 * needle with a <code>CLICK</code> interaction enabled to the value indicated
 * by the mouse click.
 *
 * @param event a <code>MouseEvent</code>
 */
public void mouseClicked(MouseEvent event)
{
	// propagate event to listeners
	super.mouseClicked(event);

    JCGaugePickEvent pe = pick(event.getPoint());

    if (pe != null) {
        // see if there are any needles interested in snap movements
        // and move the closest one
        JCLinearNeedle closest;

         // if there is such a needle, move it
        closest = getClosestNeedle(pe, JCAbstractNeedle.InteractionType.CLICK);
        if (closest != null) {
            closest.setValue(pe.getValue());
            redraw();
        }
	}
}

/**
 * Drags the closest needle.
 *
 * @param event a <code>MouseEvent</code>
 */
public void mouseDragged(MouseEvent event)
{
	// propagate event to listeners
	super.mouseClicked(event);

	JCGaugePickEvent pe = pick(event.getPoint());

    if (pe != null) {
        // if we are not already in the process of dragging a needle ...
        // @see mouseReleased where it is nulled out when released

        if (draggedNeedle == null) {
            // find the closest dragable needle
            draggedNeedle =
                getClosestNeedle(pe, JCAbstractNeedle.InteractionType.DRAG);
        }

        // if there was such a needle, move it
        if (draggedNeedle != null) {
			draggedNeedle.setValue(pe.getValue());
        	redraw();
    	}
    }
}

/**
 * Returns the closest needle to the clicked/dragged point.
 * Returns closest needle of type
 * <code>CLICK</code>/<code>DRAG</code>/<code>CLICK_DRAG</code>,
 * or if there are none of these, the closest needle of type <code>NONE</code>.
 *
 * @param pe the pick event with the click inforamation
 * @param interactionType the needle must be of this
 *     <code>JCAbstractNeedle.InteractionType</code>
 * @return the closest needle of the indicated type
 */
public JCLinearNeedle getClosestNeedle(
    JCGaugePickEvent                    pe,
    JCAbstractNeedle.InteractionType    interactionType)
{
    JCLinearNeedle closestNeedle = null;

    // get the position of the clicked point
    int pointPosition = valueToPosition(pe.getValue());

    // find needle to drag/click
    // by comparing the linear distance between the point and each of the
    // needles and choosing the needle which is closest to the point.  Note
    // that for horizontal scales only the horizontal distance is used and
    // for vertical scales on the vertical distance is used.
    Enumeration e = needles.elements();
    while(e.hasMoreElements()) {

        JCLinearNeedle needle = (JCLinearNeedle) e.nextElement();

        // Needle must be of the interactive sort
        JCAbstractNeedle.InteractionType it = needle.getInteractionType();
        if (it == JCAbstractNeedle.InteractionType.CLICK_DRAG ||
        	it == interactionType)
        {
            if (closestNeedle == null) {
                closestNeedle = needle;
            }
            else {
                int newPosition = valueToPosition(needle.getValue());
                int oldPosition = valueToPosition(closestNeedle.getValue());
                int newDistance = Math.abs(pointPosition - newPosition);
                int oldDistance	= Math.abs(pointPosition - oldPosition);

                // If the old distance was father away, select the new,
                // closest needle
                if (oldDistance > newDistance) {
                    closestNeedle = needle;
                }
            }
        }
    }
    return(closestNeedle);
}

/**
 * A convenience method that returns the position corresponding to a linear
 * scale value.
 * @param value a circular scale value.
 * @return The position based on the value.
 */
protected int valueToPosition(double value)
{
    int                         position;
    JCLinearScale               lscale      = getLinearScale();
    JCLinearScale.Orientation   orientation = lscale.getOrientation();
    Rectangle                   bounds      = lscale.getScaleSize();

    // Get the position
    if (orientation == JCLinearScale.Orientation.VERTICAL) {
        position = GaugeUtil.valueToPosition(value, lscale.getMin(),
                                             lscale.getMax(), bounds.y,
                                             bounds.y + bounds.height);
    }
    else {
        position = GaugeUtil.valueToPosition(value, lscale.getMin(),
                                             lscale.getMax(), bounds.x,
                                             bounds.x + bounds.width);
    }
    return(position);
}

/**
* Draws an image as the gauge background.
* @param bounds
* @param color
* @param image
* @param g the Graphics on which to draw
*/
protected void drawImage(Graphics g, Image image, Rectangle bounds, Color color)
{
	// create an image observer
    java.awt.image.ImageObserver observer = new javax.swing.JLabel();

    // draw the image
    if (super.getScaleImage()) {
	    g.drawImage(image, bounds.x, bounds.y, bounds.width, bounds.height,
	                color, observer);
    }
    else {
        g.drawImage(image, bounds.x, bounds.y, color, observer);
    }
}

/**
 * Call this instead of paint so that the <code>repaintEnabled</code> flag can
 * be checked. No repaint will occur if that flag is <code>false</code>.
 */
public void redraw()
{
    if (getRepaintEnabled()) {
        repaint();
    }
}

//ServerRenderable methods
/**
 * Overrides parent <code>getGraphics()</code>. If another Graphics object has
 * been set to be the current one, will return that. Otherwise, call the
 * superclass' <code>getGraphics()</code> method.
 * @return the Graphics object to use to draw this chart
 */
public Graphics getGraphics()
{
    if (prevSetGraphics == null) {
        return(super.getGraphics());
    }
    return(prevSetGraphics);
}

/**
 * Provides a way for the circular gauge to draw to a custom Graphics object.
 * Call with a null to specify use of the default Graphics object used
 * to draw to the screen.
 * @param g Graphics object to use for drawing
 */
public void setGraphics(Graphics g)
{
    prevSetGraphics = g;
    getLegend().setGraphics(g);
}

/**
 * Internal use only.  Don't use.
 */
public Image serverSnapshot()
{
	return(ServerRenderer.snapshot(this));
}

/**
 * Internal use only.  Don't use.
 */
public void serverPaint(Graphics igc)
{
    ServerRenderer.paintContainer(igc, this);
}

}
