/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCLinearIndicator.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:29 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.swing.gauge;

import java.awt.*;

/**
 * An indicator points to values on its scale.
 * There may be multiple indicators associated with a scale.
 *
 * @see com.klg.jclass.swing.gauge.JCIndicatorStyle
 */
public class JCLinearIndicator extends JCAbstractIndicator {

static final long serialVersionUID = -1403769126069742224L;

/**
 * Creates an indicator.
 *
 * @param scale the associated linear scale
 */
public JCLinearIndicator(JCLinearScale scale)
{
    super(scale);
}

/**
 * Creates an indicator.
 *
 * @param foreground the indicator's color
 * @param indicatorWidth the indicator's width in pixels
 * @param scale the associated linear scale
 * @param visible a boolean that controls the visibility of the indicator
 * @param inner_extent the indicator's <code>inner_extent</code>, expressed as
 *      a ratio of its length to the width or height of the associated linear
 *      scale
 * @param outer_extent the indicator's <code>outer_extent</code>, expressed as
 *      a ratio of its length to the width or height of the associated linear
 *      scale
 * @param indicatorStyle indicator styles are kept in
 *      <code>JCIndicatorStyle</code>; user-defined indicator styles  may be
 *      passed to <code>JCIndicatorStyle</code>'s constructor
 * @param value the indicator's value; the indicator points to this value on its
 *     associated scale
 */
public JCLinearIndicator(Color						foreground,
                         double						indicatorWidth,
                         JCLinearScale				scale,
                         boolean					visible,
                         double						inner_extent,
                         double						outer_extent,
                         JCIndicatorStyle			indicatorStyle,
                         double						value)
{
    super(foreground, indicatorWidth, scale, visible,
          inner_extent, outer_extent, indicatorStyle, value);
}

/**
 * Returns the linear scale associated with this indicator.

 * @return the linear scale associated with this indicator
 */
protected JCLinearScale getLinearScale()
{
	return((JCLinearScale)getScale());
}

/**
 * This routine draws the linear indicator object on the screen.
 *
 * @param g The graphics object with which to draw
 */
public void paint(Graphics g)
{
    JCLinearScale   lscale = getLinearScale();

    // Do we need to draw anything?
    if (!isVisible()                        ||
        lscale.getMin() >= lscale.getMax()  ||
        inner_extent >= outer_extent        ||
        indicatorWidth <= 0)
    {
        return;
    }

    if (indicatorStyle == JCIndicatorStyle.CIRCLE) {
        GaugeUtil.drawCircleForLinearScale(g, lscale, getForeground(), value,
                                           inner_extent, outer_extent);
    }
    else {
        GaugeUtil.drawLinearPolygon(g, lscale, getForeground(),
                                    inner_extent, outer_extent,
                                    indicatorWidth, value, reversed, false,
                                    indicatorStyle);
    }
}

} // JCLinearIndicator
