/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCLinearNeedle.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:29 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.swing.gauge;

import java.awt.*;

/**
 * A needle points to values on its scale.
 * There may be multiple needles associated with a scale.
 *
 * @see com.klg.jclass.swing.gauge.JCIndicatorStyle
 */
public class JCLinearNeedle extends JCAbstractNeedle {

static final long serialVersionUID = 783053371828851844L;

/**
 * Creates a needle.
 *
 * @param scale the associated linear scale
 */
public JCLinearNeedle(JCLinearScale scale)
{
    super(scale);
}

/**
 * Creates a needle.
 *
 * @param foreground the needle's color
 * @param needleWidth the needle's width in pixels
 * @param scale the associated circular scale
 * @param interactionType the needle's interaction type
 * @param visible Aa boolean that controls the visibility of the needle
 * @param inner_extent the needle's inner extent, expressed as a ratio
 *      of its length in pixels to the width/height
 *      of the associated linear scale
 * @param outer_extent the needle's outer extent, expressed as a ratio
 *      of its length in pixels to the width/height
 *      of the associated linear scale
 * @param needleStyle needle styles are kept in <code>JCIndicatorStyle</code>;
 *      user-defined needle styles  may be passed to
 *      <code>JCIndicatorStyle</code>'s constructor
 * @param value the needle's value; the needle points to this value on its
 *      associated scale
 */
public JCLinearNeedle(Color				foreground,
                      double		    needleWidth,
                      JCLinearScale		scale,
                      InteractionType   interactionType,
                      boolean			visible,
                      double			inner_extent,
                      double			outer_extent,
                      JCIndicatorStyle	needleStyle,
                      double			value)
{
    super(foreground, needleWidth, scale, visible, inner_extent,
          outer_extent, needleStyle, value);
    setInteractionType(interactionType);
}

/**
 * Creates a needle.
 *
 * @param foreground the needle's color
 * @param needleWidth the needle's width in pixels
 * @param scale the associated circular scale
 * @param interactionType the needle's interaction type.
 * @param visible a boolean that controls the visibility of the needle
 * @param length the needle's length, expressed as a ratio
 *      of its length in pixels to the width/height
 *      of the associated linear scale
 * @param needleStyle needle styles are kept in <code>JCIndicatorStyle</code>;
 *      user-defined needle styles  may be passed to
 *      <code>JCIndicatorStyle</code>'s constructor
 * @param value the needle's value; the needle points to this value on its
 *      associated scale
 */
public JCLinearNeedle(Color						foreground,
                      double					needleWidth,
                      JCLinearScale				scale,
                      InteractionType           interactionType,
                      boolean					visible,
                      double					length,
                      JCIndicatorStyle			needleStyle,
                      double					value)
{
    super(foreground, needleWidth, scale, visible, 0.0, length,
	      needleStyle, value);
    setInteractionType(interactionType);
}

/**
 * Returns the linear scale associated with this needle.
 *
 * @return the linear scale associated with this needle
 */
protected JCLinearScale getLinearScale()
{
	return((JCLinearScale)getScale());
}


/**
 * This routine draws the linear needle object on the screen.
 *
 * @param g
 */
public void paint(Graphics g)
{
    JCLinearScale   lscale = getLinearScale();

    // Do we need to draw anything?
    if (!isVisible()                        ||
        lscale.getMin() >= lscale.getMax()  ||
        inner_extent >= outer_extent        ||
        indicatorWidth <= 0)
    {
        return;
    }

    if (indicatorStyle == JCIndicatorStyle.CIRCLE) {
        GaugeUtil.drawCircleForLinearScale(g, lscale, getForeground(), value,
                                           inner_extent, outer_extent);
    }
    else {
        GaugeUtil.drawLinearPolygon(g, lscale, getForeground(),
                                    inner_extent, outer_extent,
                                    indicatorWidth, value, reversed, false,
                                    indicatorStyle);
    }
}

}
