/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCLinearScale.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:29 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.swing.gauge;

import java.awt.*;
import java.util.Enumeration;


 


 

/**
 * <code>JCLinearScale</code> provides a graduated scale drawn in a linear
 * fashion.  It is oriented either horizontally or vertically.
 */
public class JCLinearScale extends JCAbstractScale {

/**
 * The orientation of this scale.
 */
protected Orientation orientation = Orientation.HORIZONTAL;

/**
 * Asks: "Does <code>zoomFactor</code> have effect in the minimum extent
 * direction?".
 */
protected boolean useZoomFactorForMin = true;

/**
 * Asks: "Does <code>zoomFactor</code> have effect in the maximum extent
 * direction?".
 */
protected boolean useZoomFactorForMax = true;

/**
 * Creates a scale given its parent gauge.
 *
 * @param gauge the <code>JCLinearGauge</code> associated with this scale
 */
public JCLinearScale(JCLinearGauge gauge)
{
	super(gauge);

	// set the layout
    setLayout(new LinearLayout());

    setForeground(Color.white);
    GaugeUtil.createLegendPopulatorRenderer(gauge, this);
}

/**
 * Creates a graduated linear scale.
 * @param orientation
 * @param gauge the <code>JCLinearGauge</code> associated with this scale
 * @param direction a <code>JCAbstractScale.Direction</code> enum indicating
 *     <code>FORWARD</code> or <code>BACKWARD</code>
 * @param min the minumum value a scale can have
 * @param max the maximum value a scale can have
 * @param foreground the color of this scale's foreground
 */
public JCLinearScale (JCLinearGauge   gauge,
                      Direction 	  direction,
                      double		  min,
                      double		  max,
                      Orientation     orientation,
                      Color		      foreground)
{
	super(gauge, direction, min, max, foreground);

	// set the layout
    setLayout(new LinearLayout());

    this.orientation = orientation;

    GaugeUtil.createLegendPopulatorRenderer(gauge, this);
}

/**
 * Returns the gauge associated with this scale.
 *
 * @return the gauge associated with this scale
 */
public JCLinearGauge getLinearGauge()
{
	return((JCLinearGauge)getGauge());
}

/**
 * This is called by our widget parent when it wants to know our
 * preferred size.
 *
 * @return the <code>Dimension</code> (size) we'd like to be based on our
 *     width & height
 */
public Dimension getPreferredSize()
{
	return(new Dimension(getParent().getBounds().width,
                         getParent().getBounds().height));
}

/**
 * Overrides add to pass in a linear constraint.
 * @return
 * @param c the component to be added
 */
public Component add(Component c)
{
	add(c, new LinearConstraint(getGauge(), Double.MAX_VALUE,
                                Double.MAX_VALUE));
    return(c);
}

/**
 * Overrides add to pass in a linear constraint.
 * @return
 * @param c the component to be added
 * @param index the order this component has in the list; components added
 *     last are drawn first
 */
public Component add(Component c, int index)
{
	add(c, new LinearConstraint(getGauge(), Double.MAX_VALUE, Double.MAX_VALUE),
        index);
    return(c);
}

/**
 * Calculates the rectangle that defines the position and dimensions of
 * the linear scale.
 *
 * @return a rectangle that contains the position and dimensions of the
 *      linear scale
 */
public Rectangle getScaleSize()
{
    // Compute the combined insets of the scale and the scale's parent
    // (which happens to be the gaugeArea)
    JCGaugeArea gaugeArea   = getGauge().getGaugeArea();
	Insets parent_insets    = gaugeArea.getInsets();
	Insets insets	        = getInsets();
	int w		            = gaugeArea.getBounds().width;
	int h		            = gaugeArea.getBounds().height;
    int left                = Math.max(insets.left,   parent_insets.left);
    int right               = Math.max(insets.right,  parent_insets.right);
    int top                 = Math.max(insets.top,    parent_insets.top);
    int bottom              = Math.max(insets.bottom, parent_insets.bottom);
    int x                   = left;
    int y                   = top;

	w -= left + right;
	h -= top  + bottom;
    w = Math.max(w, 0);
    h = Math.max(h, 0);

    // Adjust for zoomFactor
    if (useZoomFactorForMin || useZoomFactorForMax) {
        if (orientation == JCLinearScale.Orientation.VERTICAL) {
            double width_factor = w - (w * zoomFactor);
            if (useZoomFactorForMin && useZoomFactorForMax) {
                x += (int)(width_factor / 2.0);
            }
            else if (useZoomFactorForMin) {
                x += (int)(width_factor);
            }
            w *= zoomFactor;
        }
        else {
            double height_factor = h - (h * zoomFactor);
            if (useZoomFactorForMin && useZoomFactorForMax) {
                y += (int)(height_factor / 2.0);
            }
            else if (useZoomFactorForMin) {
                y += (int)(height_factor);
            }
            h *= zoomFactor;
        }
    }

	return(new Rectangle(x, y, w, h));
}

/**
 * Draws the background of this scale.
 * @param g the <code>Graphics</code> object onto which we will draw our background
 */
public void paintComponent(Graphics g)
{
    Rectangle bounds = getScaleSize();
	super.paintComponent(g);


 

    // start painting
    Graphics gc = g.create();

    // get the adjusted x, y, w, h
	int x = bounds.x;
    int y = bounds.y;
    int w = bounds.width;
    int h = bounds.height;

    // Draw an image if there is one
    if (image != null) {
        drawImage(gc, image, bounds);
    }
    else {
        // draw the rectangle
        gc.fillRect(x, y, w, h);
    }


 


 


 





 

    // clean up
    gc.dispose();

}


 


 

/**
 * Given a screen position in pixels, returns the closest scale value.
 *
 * @param p the point representing the <i>x</i>,<i> y</i> coordinates of the
 *     click
 * @return the value closest to the point
 */
public double pick(Point p)
{
	double     value = min;
    Rectangle  bounds = getScaleSize();

    // Get the value
    if (orientation == JCLinearScale.Orientation.VERTICAL) {
        value = GaugeUtil.positionToValue(p.y, min, max, bounds.y,
                                          bounds.y + bounds.height);
    }
    else {
        value = GaugeUtil.positionToValue(p.x, min, max, bounds.x,
                                          bounds.x + bounds.width);
    }

    // Adjust value according to direction
    if (direction.equals(JCAbstractScale.Direction.BACKWARD) ^
        (orientation == JCLinearScale.Orientation.VERTICAL))
    {
		value  = max - value + min;
	}
    value = GaugeUtil.clamp(value, min, max);

	// Finally, return the value
    return(value);
}

/**
 * Returns the <code>JCLinearScale.Orientation</code> for this linear scale,
 * <code>HORIZONTAL</code> or <code>VERTICAL</code>, giving the orientation
 * in which scale values are rendered.
 *
 * @return the <code>JCLinearScale.Orientation</code> for this scale
 */
public Orientation getOrientation()
{
	return(this.orientation);
}

/**
 * Sets the <code>JCLinearScale.Orientation</code> for this linear scale,
 * <code>HORIZONTAL</code> or <code>VERTICAL</code>, giving the orientation
 * in which scale values are rendered.
 *
 * @param orientation the <code>JCLinearScale.Orientation</code>
 */
public void setOrientation(Orientation orientation)
{
	this.orientation = orientation;
    redraw();
}

/**
 * Gets the value of <code>useZoomFactorForMin</code>, which tells whether the
 * <code>ZoomFactor</code> is in effect in the minimum extent direction or not.
 *
 * @return the current <code>useZoomFactorForMin</code> value
 */
public boolean getUseZoomFactorForMin()
{
    return(this.useZoomFactorForMin);
}

/**
 * Sets the <code>useZoomFactorForMin</code> value, which tells whether the
 * <code>ZoomFactor</code> is in effect in the minimum extent direction or not.
 *
 * @param useZoomFactorForMin the new <code>useZoomFactorForMin</code> value
 */
public void setUseZoomFactorForMin(boolean useZoomFactorForMin)
{
    this.useZoomFactorForMin = useZoomFactorForMin;
}

/**
 * Gets the value of <code>useZoomFactorForMax</code>, which tells whether the
 * <code>ZoomFactor</code> is in effect in the maximum extent direction or not.
 *
 * @return the current <code>useZoomFactorForMax</code> value
 */
public boolean getUseZoomFactorForMax()
{
    return(this.useZoomFactorForMax);
}

/**
 * Sets the <code>useZoomFactorForMax</code> value, which tells whether the
 * <code>ZoomFactor</code> is in effect in the maximum extent direction or not.
 *
 * @param useZoomFactorForMax the new <code>useZoomFactorForMax</code> value
 */
public void setUseZoomFactorForMax(boolean useZoomFactorForMax)
{
    this.useZoomFactorForMax = useZoomFactorForMax;
}

/**
 * Orientation enumeration.
 */
public static class Orientation {

/** The Horizontal orientation. */
public static final Orientation HORIZONTAL = new Orientation();

/** The Vertical orientation. */
public static final Orientation VERTICAL = new Orientation();

/**
 * Private constructor so we only allow the expected Orientations.
 */
private Orientation() {}

}

/**
* Draws an image as the scale.
* @param bounds
 * @param image
* @param g the Graphics on which to draw
*/
protected void drawImage(Graphics g, Image image, Rectangle bounds)
{
	// create an image observer
    java.awt.image.ImageObserver observer = new javax.swing.JLabel();

    // draw the image
    if (super.getScaleImage()) {
	    g.drawImage(image, bounds.x, bounds.y, bounds.width, bounds.height,
	                getForeground(), observer);
    }
    else {
        g.drawImage(image, bounds.x, bounds.y, getForeground(), observer);
    }
}

/**
 * Call this instead of paint so that the <code>repaintEnabled</code> flag can
 * be checked. No repaint will occur if that flag is <code>false</code>.
 */
public void redraw()
{
    if (gauge.getRepaintEnabled()) {
        Enumeration e = gauge.getTicks().elements();
        while(e.hasMoreElements()) {
            JCLinearTick tick = (JCLinearTick) e.nextElement();
            tick.redraw(true);
        }
        repaint();
    }
}


 

}
