/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCLinearTick.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:30 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.swing.gauge;

import com.klg.jclass.util.JCNumberUtil;

import javax.swing.*;
import java.awt.*;

/**
 * The class for tick marks that are drawn on a <code>JCLinearScale</code>.
 */
public class JCLinearTick extends JCAbstractTick {

/**
 * Creates a <code>JCLinearTick</code> that spans its linear scale. Other
 * tick properties have default values.
 *
 * @param scale the <code>JCLinearScale</code> for which these ticks are
 *     drawn
 */
public JCLinearTick(JCLinearScale scale)
{
	super(scale, scale.getMin(), scale.getMax());

    // set the layout
	setLayout(new LinearLayout());
}

/**
 * Creates a linear tick object.
 *
 * @param scale the associated linear scale
 * @param automatic whether to automatically calculate the tick start, stop,
 *      and increment values from the scale's min and max, or to use the
 *      provided values
 * @param startValue where the tick marks are to begin
 * @param stopValue where the tick marks are to end
 * @param incrementValue the tick interval
 * @param precisionUseDefault sets whether tick labels use a default
 *      precision, or use <code>precision</code>
 * @param precision The format of the tick label; positive values denote the
 *      number of places after the decimal point (for example, 3 means multiples
 *      of 0.001), negative values indicate the positive number of zeros to be
 *      used before the decimal place (for example, -3 means numbering will be in
 *      multiples of 1000). Overridden if <code>precisionUseDefault</code> is
 *      set to <code>true</code>.
 * @param width each tick's width in pixels
 * @param drawLabels whether to draw labels or not
 * @param visible Are the ticks visible?
 * @param labelExtent each label's extent expressed as a ratio of its distance
 *      from the top or left of the scale to the width or height of the scale
 * @param innerExtent the tick's <code>inner_extent</code>, expressed as a ratio
 *      of its distance from the top or left to the width or height of the scale
 * @param outerExtent the tick's <code>outer_extent</code>, expressed as a ratio
 *      of its distance from the top or left to the width or height of the scale
 * @param foreground the tick's color
 * @param style the type of tick; valid values are <code>DIAMOND</code>,
 *      <code>TRIANGLE</code>, <code>REVERSE_TRIANGLE</code>,
 *     <code>RECTANGLE</code>, <code>LINE</code>, and <code>CIRCLE</code>.
 * @param style The <code>JCTickStyle</code> that defines the shape of the
 *     tick. Valid values are <code>DIAMOND</code>, <code>TRIANGLE</code>,
 *     <code>REVERSE_TRIANGLE</code>, <code>RECTANGLE</code>,
 *     <code>LINE</code>, and <code>CIRCLE</code>.
 * You can also create your own <code>JCTickStyle</code>.
 * @param font the font used to draw tick labels
 * @param fontColor the color of the font for tick labels
 */
public JCLinearTick(JCLinearScale 	    scale,
                    boolean				automatic,
                    double				startValue,
                    double				stopValue,
                    double				incrementValue,
                    boolean				precisionUseDefault,
                    int					precision,
                    double				width,
                    boolean				drawLabels,
                    boolean				visible,
                    double				labelExtent,
                    double				innerExtent,
                    double				outerExtent,
                    Color				foreground,
                    JCTickStyle			style,
                    Font				font,
                    Color				fontColor)
{
	super(scale,
          automatic,
          startValue,
          stopValue,
          incrementValue,
          precisionUseDefault,
          precision,
          width,
          drawLabels,
          visible,
          labelExtent,
          innerExtent,
          outerExtent,
          foreground,
          style,
          font,
          fontColor);

	// set the layout
	setLayout(new LinearLayout());
}

/**
 * Returns the linear scale associated with this tick object.

 * @return the linear scale associated with this tick object
 */
protected JCLinearScale getLinearScale()
{
	return((JCLinearScale)getScale());
}

/**
 * This is called by our widget parent when it wants to know our
 * preferred size.
 *
 * @return the <code>Dimension</code> (size) we'd like to be based on our
 *      associated linear scale
 */

public Dimension getPreferredSize()
{
	return(getLinearScale().getPreferredSize());
}

/**
 * Overridden to preserve the layout of labels on a resize.
 * Moves and resizes this component.
 *
 * @param x  the new horizontal location
 * @param y  the new vertical location
 * @param w  the new width
 * @param h  the new height
 * @see java.awt.Component#setBounds
 */
public void reshape(int x, int y, int w, int h)
{
    super.reshape(x, y, w, h);
    redrawLabels();
}

/**
 * If <code>automatic</code>, this method is called to determine and
 * set the start/stop and increment values for laying out ticks.
 * The goal is to have as many ticks/labels as will fit without
 * the ticks or labels overlapping.
 *
 * @param g the graphics used to get font metric information.
 */
public void calcLinearTicks(Graphics g)
{
    JCLinearScale               lscale      = getLinearScale();
    JCLinearScale.Orientation   orientation = lscale.getOrientation();
    Rectangle                   bounds      = lscale.getScaleSize();
    int width =
        (orientation == JCLinearScale.Orientation.HORIZONTAL) ? bounds.width
                                                              : bounds.height;
    calcTicks(g, width);
}

/**
 *  Layout and redraw labels.
 */
public void redrawLabels()
{
    boolean                     reversed;
    double                      value_used;
    JCLinearScale               lscale      = getLinearScale();
    JCLinearScale.Orientation   orientation = lscale.getOrientation();
    JCAbstractScale.Direction   direction   = lscale.getDirection();

    // if we are drawing labels ...
    if (drawLabels) {

        // remove all the old labels
        removeAll();

        // get info from CircularScale
        double lin_min = lscale.getMin();
        double lin_max = lscale.getMax();

        // calculate the ticks
        if (automatic) {
        	Graphics g = getGraphics();
        	if (g != null) {
            	calcLinearTicks(g);
        	}
        }

        reversed = (direction.equals(JCAbstractScale.Direction.BACKWARD) ^
                    orientation == JCLinearScale.Orientation.VERTICAL);

        // loop and position the labels
        double value = startValue;
        while (value <= stopValue && value <= lin_max && incrementValue > 0) {
            if (value < lin_min) {
                value += incrementValue;
                continue;
            }

            value_used = (reversed == true) ? lin_max - value + lin_min : value;
			double position = 0.0;
			if (lin_min < lin_max) {
                position = 100.0 * ((value_used - lin_min) /
			                        Math.abs(lin_max - lin_min));
			}


            // Finally, do the label
            JComponent label = null;

            LinearConstraint constraint =
                new LinearConstraint(lscale.getLinearGauge(), getLabelExtent(),
				                     position);

            // Allow the user to define a label
            if (labelGenerator != null) {
                label = labelGenerator.makeLabel(this, value, constraint);
            }
            else { // Otherwise create a JLabel for them
				// format the value according to precision
				String sValue = JCNumberUtil.format(value, precision);
            	label = new JLabel(sValue);

	            // set the labels attributes"
    	    	label.setFont(getFont());
        	    label.setForeground(fontColor);
			}

            label.setSize(label.getPreferredSize());

            // and finally, add it to the layout
            add(label, constraint);

            value += incrementValue;
        }
    }
}

/**
 * Main tick draw routine.
 *
 * @param g
 */
public void paintComponent(Graphics g)
{
    double value, value_used;
    double inner_extent, outer_extent;
    boolean scale_reversed;

    JCLinearScale lscale = getLinearScale();
    JCLinearScale.Orientation orientation = lscale.getOrientation();
    JCAbstractScale.Direction direction = lscale.getDirection();
    Rectangle bounds = lscale.getScaleSize();

	super.paintComponent(g);

	// Calculate the ticks, this can potentially change start, stop
	// and increment values, so do it first
	if (automatic) {
		calcLinearTicks(g);
	}

	// Get info from Linear Scale
	double lin_min = lscale.getMin();
	double lin_max = lscale.getMax();

    // Do we need to draw anything?
    if (!isVisible()                    ||
        lin_min >= lin_max              ||
        innerExtent >= outerExtent      ||
        startValue > stopValue          ||
        incrementValue <= 0.0           ||
        tickWidth <= 0)
    {
        return;
    }

    scale_reversed = (direction.equals(JCAbstractScale.Direction.BACKWARD) ^
                      orientation == JCLinearScale.Orientation.VERTICAL);

    //	Determine the inner and outer extents (in pixels).
    if (orientation == JCLinearScale.Orientation.VERTICAL) {
	    inner_extent = innerExtent * bounds.width;
	    outer_extent = outerExtent * bounds.width;
    }
    else {
	    inner_extent = innerExtent * bounds.height;
	    outer_extent = outerExtent * bounds.height;
    }

	value = startValue;
	while (value <= stopValue && value <= lin_max) {
		if (value < lin_min) {
			value += incrementValue;
			continue;
		}

		// if there is something to draw ...
		if (drawTicks) {

			// draw a simple line
			if (tickStyle == JCTickStyle.LINE) {
                value_used = scale_reversed ? lin_max - value + lin_min : value;

                // Get the tick position
                int pos;
                if (orientation == JCLinearScale.Orientation.VERTICAL) {
                    pos = GaugeUtil.valueToPosition(value_used, lin_min,
                                                    lin_max, bounds.y,
                                                    bounds.y + bounds.height);
                }
                else {
                    pos = GaugeUtil.valueToPosition(value_used, lin_min,
                                                    lin_max, bounds.x,
                                                    bounds.x + bounds.width);
                }
                int x_inner, y_inner, x_outer, y_outer;
               if (orientation == JCLinearScale.Orientation.VERTICAL) {
				    x_inner = bounds.x + (int)inner_extent;
				    y_inner = pos;
                    x_outer = bounds.x + (int)outer_extent;
                    y_outer = pos;
                }
                else {
				    x_inner = pos;
				    y_inner = bounds.y + (int)inner_extent;
                    x_outer = pos;
                    y_outer = bounds.y + (int)outer_extent;
                }
                Graphics gc = g.create();
                gc.setColor(tickColor);
				gc.drawLine(x_inner, y_inner, x_outer, y_outer);
                gc.dispose();
			}
            // A circle
            else if (tickStyle == JCTickStyle.CIRCLE) {
                GaugeUtil.drawCircleForLinearScale(g, lscale, tickColor,
                                                   value, innerExtent,
                                                   outerExtent);
             }
			// Or a polygon
			else {
                GaugeUtil.drawLinearPolygon(g, lscale, tickColor, innerExtent,
                                            outerExtent, tickWidth, value,
                                            reversed, true, tickStyle);
			}
		}

		value += incrementValue;
	}

}

/**
 * Call this instead of paint so that the <code>repaintEnabled</code> flag can
 * be checked. No repaint will occur if that flag is <code>false</code>.
 *
 * @param redrawLabels <code>true</code> means reposition and redraw the labels;
 *     otherwise, just repaint the component
 */
public void redraw(boolean redrawLabels)
{
    if (getLinearScale().getGauge().getRepaintEnabled()) {
        if (redrawLabels) {
            redrawLabels();
        }

        revalidate();

        // revalidate is supposed to call repaint at some point, but doesn't
        // always, so .. ("Graphic Java 2", p 144)
        repaint();
    }
}

}
