/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCNeedle.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:30 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.swing.gauge;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * The interface for needles.  Those wishing to use their own needles should
 * implement this interface.
 *
 * A needle points to a current value on its associated scale.  The end user may
 * choose to interact with a needle.  There may be multiple needles
 * associated with a scale.
 *
 */
public interface JCNeedle extends JCIndicator {

/**
 * Returns the interation type for this needle.
 *
 * The <code>JCNeedle.InteractionType</code> for this
 *      needle, one of <ul> <li><code>InteractionType.NONE</code></li>
 *      <li><code>InteractionType.DRAG</code></li>
 *      <li><code>InteractionType.CLICK</code>
 *      </li> <li><code>InteractionType.CLICK_DRAG</code></li> </ul>
 *
 * @return The needle whose interaction type is being requested
 */
public JCAbstractNeedle.InteractionType getInteractionType();

/**
 * Returns the length of this needle as a ratio of the scale's radius, width,
 * or height (depending on the scale's type and orientation). Note that the
 * length is really the outer extent.
 *
 * @return the length of this needle as a ratio
 */
public double getLength();

/**
 * Returns the <code>JCIndicatorStyle</code> for this needle.
 *
 * @return the <code>JCIndicatorStyle</code> of this needle
 */
public JCIndicatorStyle getNeedleStyle();

/**
 * Returns the width for this needle.
 *
 * @return the width of this needle
 */
public double getNeedleWidth();

/**
 * Returns a <code>sendEvents</code> flag; <code>true</code> means events will
 * be sent when the needle's value changes, <code>false</code> means don't send
 * the events.
 *
 * @return whether or not needle change events should be sent
 */
public boolean getSendEvents();

/** Removes listener interested in needle movements.
 *
 * @param l the listener to be removed
 */
public void removeChangeListener(ChangeListener l);

/**
 * Broadcasts the 'needle has been moved' event to interested listeners.
 *
 * @param ev the <code>ChangeEvent</code> of interest
 */
public void sendChangeEvent(ChangeEvent ev);

/**
 * Sets the interation type for this needle.
 *
 * The <code>JCNeedle.InteractionType</code> for this
 *     needle, one of <ul> <li><code>InteractionType.NONE</code></li>
 *     <li><code>InteractionType.DRAG</code></li>
 *     <li><code>InteractionType.CLICK</code>
 *     </li> <li><code>InteractionType.CLICK_DRAG</code></li> </ul>
 *
 * @param interactionType the <code>JCNeedle.InteractionType</code> for
 *     this needle
 */
public void setInteractionType(JCAbstractNeedle.InteractionType interactionType);

/**
 * Sets the length of this needle as a ratio of the
 * scale's radius, width or height (depending on the scale's type and
 * orientation). For example, for a circular scale, a length of 0.75 means
 * draw the needle from the center out to 75% of the scale's radius.
 *
 * @param length the needle's length expressed as a ratio
 */
public void setLength(double length);

/**
 * Sets the <code>JCIndicatorStyle</code> for this needle.
 *
 * @param needleStyle the <code>JCIndicatorStyle</code> for this needle
 */
public void setNeedleStyle(JCIndicatorStyle needleStyle);

/**
 * Sets the width of needle (this is actually stored in the inherited
 * <code>indicatorWidth</code> property).
 *
 * @param needleWidth the width of this needle
 */
public void setNeedleWidth(double needleWidth);

/**
 * Sets the <code>sendEvents</code> flag; <code>true</code> means events will
 * be sent when the needle's value changes, <code>false</code> means don't
 * send the events.
 *
 * @param sendEvents <code>true</code> means send needle's value change
 *     events to listeners
 */
public void setSendEvents(boolean sendEvents);

}
