/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCPolygon.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:30 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.swing.gauge;

import java.awt.Polygon;
import java.awt.Rectangle;

/**
 * Provides an extension of the <code>Polygon</code> class that stores a
 * bounding box for easy access.  This is also done so that the bounding box
 * will only have to be calculated once.  Both <code>JCIndicatorStyle</code> and
 * <code>JCTickStyle</code> subclass from this abstract class.
 */
public abstract class JCPolygon extends Polygon
                                implements java.io.Serializable
{

static final long serialVersionUID = -2201114721090353435L;

/**
 * <code>min_x</code>, <code>min_y</code> are <i>x</i> and <i>y</i>,
 * and <code>max_x</code> and <code>max_y</code> are width and height.
 */
protected Rectangle boundingBox;

/**
 * Constructs an indicator using the points of a polygon.
 * For a user-defined indicator.
 *
 * @param xpoints the <i>x</i> coordinates for the polygon
 * @param ypoints the <i>y</i> coordinates for the polygon
 * @param numPoints the number of points in this polygon
 */
public JCPolygon(int xpoints[], int ypoints[], int numPoints)
{
	super(xpoints, ypoints, numPoints);
}

/** */
public JCPolygon()
{
	super();
}

/**
 * Returns the upper-left-most point of the polygon as
 * <i>x</i>, <i>y</i> in the Rectangle, and the lower-right-most point
 * as width and height.
 *
 * The rectangle is defined to be: <i>x</i> = <i>min_x</i>,
 * <i>y</i> = <i>min_y</i>, <i>width</i> = <i>max_x</i> and
 * <i>height</i> = <i>max_y</i>.
 *
 * @return return the upper-left-most point of the polygon as <i>x</i>,
 *     <i>y</i> in the Rectangle, and the lower-right-most point as width and
 *     height
 */
public Rectangle getExtrema()
{
	// Determine the bounding box if not already determined
	if (boundingBox == null) {
        Rectangle bounds = this.getBounds();
		boundingBox = new Rectangle(bounds.x, bounds.y,
		                            bounds.x + bounds.width,
                                    bounds.y + bounds.height);
	}
	return(boundingBox);
}

}
