/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCRange.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:30 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.swing.gauge;

import java.awt.Color;

/**
 * The interface for range objects.  Those wishing to use their own
 * ranges should implement this interface.
 *
 * @since Version 5.0.0
 */
public interface JCRange {

/**
 * The foreground color of the range.
 *
 * @return the foreground color of the range
 */
public Color getForeground();

/**
 * Specifies where the inner boundary of the range is.  Values are ratios based
 * on the scale's radius, width, or height (depending on the scale's type and
 * orientation). For example, for circular scale, an inner extent of 0.8 means
 * start drawing this range 80% of the radius' size away from the center.
 *
 * @return the inner extent
 */
public double getInnerExtent();

/**
 * Returns where the outer boundary of the range is.  Values are ratios based
 * on the scale's radius, width, or height (depending on the scale's type and
 * orientation). For example, for circular scale, an outer extent of 0.8
 * means stop drawing this range 80% of the radius' size away from the center.
 *
 * @return the outer extent
 */
public double getOuterExtent();

/**
 * Gets the name of this range. This is the name which will appear in the
 * legend. If no name is supplied, a name of the form "Range &#60;x&#62;" will
 * be automatically generated.
 *
 * @return the range's name
 */
public String getRangeName();

/**
 * Returns the scale associated with this range.
 *
 * @return the scale for the <code>JCRange</code> component
 */
public JCScale getScale();

/**
 * Returns the scale value (not angle or position) at which to begin the range.
 *
 *
 * @return the range's start value
 */
public double getStartValue();

/**
 * Returns the scale value (not angle or position) at which to stop the range.
 *
 * @return the range's stop value
 */
public double getStopValue();

/**
 * Sets where the inner boundary of the range is. Values are ratios based on
 * the scale's radius, width, or height (depending on the scale's type and
 * orientation). For example, for horizontal linear scale, an inner extent of
 * 0.9 means start drawing this range 90% of the scale's height away from the
 * top of the scale.
 *
 * @param innerExtent The inner extent of the range
 */
public void setInnerExtent(double innerExtent);

/**
 * Sets where the outer boundary of the range is. Values are ratios based on
 * the scale's radius, width, or height (depending on the scale's type and
 * orientation). For example, for vertical linear scale, an outer extent of 0.9
 * means stop drawing this range 90% of the scale's width away from the left of
 * the scale.
 *
 * @param outerExtent the outer extent of the range
 */
public void setOuterExtent(double outerExtent);

/**
 * Sets the range's name. This is the name which will appear in the legend.
 *
 * @param rangeName the name to appear in the legend
 */
public void setRangeName(String rangeName);

/**
 * Sets the scale value (not angle or position) at which to begin the range.
 *
 * @param startValue the start value for the range
 */
public void setStartValue(double startValue);

/**
 * Sets the scale value (not angle or position) at which to stop the range.
 *
 * @param stopValue the stop value for the range
 */
public void setStopValue(double stopValue);

}

