/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCScale.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:30 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.swing.gauge;


import javax.swing.border.Border;
import java.awt.*;
import java.util.Vector;

/**
 * <code>JCScale</code> is the interface for circular and linear scales.
 * Those wishing to use their own type of
 * scale should implement this interface.
 *
 * A <code>JCScale</code> represents a graduated scale.
 * A scale has a minimum value, a maximum value, and a direction.
 * It also is associated with other objects like <code>JCTick</code>,
 * <code>JCIndicator</code>, <code>JCNeedle</code>, and <code>JCRange</code>.
 * These associated objects use the scale to get information that
 * they need to render themselves.
 *
 */
public interface JCScale {

/**
 * Adds a range object to a scale.
 *
 * @param range the <code>Range</code> object to be added
 */
public void addRange(JCRange range);

/**
 * Adds a range object to a scale.
 *
 * @param range the <code>JCRange</code> to be added to the scale
 * @param index the <i>z</i>-order index
 */
public void addRange(JCRange range, int index);

/**
 * Adds a tick object to a scale.
 *
 * @param tick the <code>JCTick</code> to be added to the scale
 */
public void addTick(JCTick tick);

/**
 * Adds a tick object to a scale.
 *
 * @param tick the <code>JCTick</code> to be added to the scale
 * @param index the <i>z</i>-order index
 */
public void addTick(JCTick tick, int index);

/**
 * Returns the <code>JCAbstractScale.Direction</code> for this scale,
 * <code>FORWARD</code> or <code>BACKWARD</code>, giving the direction in
 * which scale values increase.
 *
 * @return the <code>JCAbstractScale.Direction</code> for this scale
 */
public JCAbstractScale.Direction    getDirection();

/**
 * Returns the <code>zoomFactor</code> for this scale.
 * @deprecated As of JClass version 4.5.1;
 * replaced by <code>getZoomFactor()</code>.
 *
 * @return The <code>zoomFactor</code> of this scale
 */
public double getExtent();

/**
 * Returns the gauge associated with this scale.
 *
 * @return The gauge associated with this scale
 */
public JCGauge getGauge();

/**
 * Returns the <code>zoomFactor</code> for this scale.  The scale is normally
 * sized so it fits exactly within the component.  This may not always be what
 * the user wants, since ticks, labels, and other components may need to be
 * drawn outside the scale.  Setting <code>zoomFactor</code> allows the scale
 * to be smaller so things outside the scale can be seen.
 *
 * @return The <code>zoomFactor</code> of this scale
 */
public double getZoomFactor();

/**
 * Returns the maximum value for this scale.
 *
 * @return the maximum value for this scale
 */
public double getMax();

/**
 * Returns the minimum value for this scale.
 *
 * @return the minimum value for this scale
 */
public double getMin();

/**
 * Returns the <code>Vector</code> of tick objects for this scale.
 *
 * @return a <code>Vector</code> of ranges
 */
public Vector getRanges();

/**
 * Returns the <code>Vector</code> of tick objects for this scale.
 *
 * @return a <code>Vector</code> of ticks
 */
public Vector getTicks();

/**
 * Returns <code>true</code> if the value is within the scale's minimum and
 * maximum.
 *
 * @param value the value to be tested
 * @return <code>true</code> if the value is within the scale's minimum
 *     and maximum
 */
public boolean inBounds(double value);

/**
 * Given a screen position in pixels, returns the closest scale value.
 *
 * @param p the point representing the <i>x</i>,<i> y</i> coordinates of
 *     the click
 * @return the scale value closest to the <code>Point</code>
 */
public double pick(Point p);

/**
 * Removes a range from a scale.
 *
 * @param range the <code>JCRange</code> to be removed
 */
public void removeRange(JCRange range);

/**
 * Removes a tick series from a scale.
 *
 * @param tick the tick object to be removed
 */
public void removeTick(JCTick tick);

/**
 * Sets a border on the scale.
 *
 * @param border the border for the scale
 */
public void setBorder(Border border);

/**
 * Sets the <code>JCAbstractScale.Direction</code> for this scale,
 * <code>FORWARD</code> or <code>BACKWARD</code>, giving the direction in
 * which scale values increase.
 *
 * @param direction the <code>JCAbstractScale.Direction</code>
 */
public void setDirection(JCAbstractScale.Direction direction);

/**
 * Sets the <code>zoomFactor</code> for this scale.
 * @deprecated As of JClass version 4.5.1;
 * replaced by <code>setZoomFactor()</code>.
 *
 * @param extent the new zoom factor
 */
public void setExtent(double extent);

/**
 * Sets the zoom factor for this scale.  The scale is normally
 * sized so it fits exactly within the component.  This may not always be what
 * the user wants, since ticks, labels, and other components may need to be
 * drawn outside the scale.
 *
 * @param extent the new zoom factor
 */
public void setZoomFactor(double extent);

/**
 * Sets the maximum value for this scale.
 *
 * @param max the maxiumum value for this scale
 */
public void setMax(double max);

/**
 * Sets the minimum value for this scale.
 *
 * @param min the minimum value for this scale
 */
public void setMin(double min);

}
