/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: LinearConstraint.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:30 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.swing.gauge;

/**
 * A <code>LinearConstraint</code> is one that provides layout information for
 * a component that needs to be placed on a linear gauge. It provides
 * information to the layout by giving a distance either horizontally or
 * vertically (depending on the orientation of the gauge's linear scale) from
 * top-left corner of the scale and a position from the top or left of the
 * scale.
 */
public class LinearConstraint extends GaugeConstraint
                              implements java.io.Serializable
{

static final long serialVersionUID = -6188065325759569415L;

/**
 * The position at which to place a label on the scale. The position is
 * specified as a percent from the width of the scale (horizontal orientation)
 * or the height of the scale (vertical orientation).
 */
protected double position;

/**
 * Constructor for <code>LinearConstraint</code> using a <code>JComponent</code>
 * as the component.
 *
 * @param extent The distance from the top-left of the rectangle.  The distance
 *      is vertical for horizontal scales and horizontal for vertical scales and
 *      is specified as a ratio of this distance in pixels to the height or
 *      width of the scale.
 * @param position The position (normally corresponding to a particular scale
 *      value) at which this component will be placed (at an "extent" distance
 *      from the top or left of the scale). The position is specified as a
 *      percent from the width of the scale (horizontal orientation) or the
 *      height of the scale (vertical orientation).  Percentage specified as
 *      an integer.
 */
public LinearConstraint(JCGauge gauge, double extent, int position)
{
	super(gauge, extent);
    this.position = (double)position;
}

/**
 * Constructor for <code>LinearConstraint</code> using a <code>JComponent</code>
 * as the component.
 *
 * @param extent The distance from the top-left of the rectangle.  The distance
 *      is vertical for horizontal scales and horizontal for vertical scales and
 *      is specified as a ratio of this distance in pixels to the height or
 *      width of the scale.
 * @param position The position (normally corresponding to a particular scale
 *      value) at which this component will be placed (at an "extent" distance
 *      from the top or left of the scale). The position is specified as a
 *      percent from the width of the scale (horizontal orientation) or the
 *      height of the scale (vertical orientation).  Percentage specified as an
 *      double.
 */
public LinearConstraint(JCGauge gauge, double extent, double position)
{
	super(gauge, extent);
    this.position = position;
}

/**
 * Returns the position (normally corresponding to a particular scale value)
 * at which this component will be placed on the scale as an integer
 * @return The position at which this component will be placed
 */
public int getPosition()
{
	return((int)position);
}

/**
 * Sets the position (normally corresponding to a particular scale value) at
 * which this component will be placed as an integer.
 * @param position The position at which this component will be placed
 */
public void setPosition(int position)
{
	this.position = (double)position;
}
/**
 * Returns the position (normally corresponding to a particular scale value)
 * at which this component will be placed on the scale as a double.
 * @return The position at which this component will be placed
 */
public double getPositionAsDouble()
{
	return(position);
}

/**
 * Sets the position (normally corresponding to a particular scale value) at
 * which this component will be placed as a double.
 * @param position The position at which this component will be placed
 */
public void setPositionAsDouble(double position)
{
	this.position = position;
}

/**
 * Return a string version of this constraint
 * @return A string version of this constraint
 */
public String toString()
{
	return("LinearConstraint: extent = " + extent + " position = " + position);
}

}
