/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: LinearLayout.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:30 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.swing.gauge;

import java.awt.*;

/**
 * LinearLayout -- lays out <code>LinearConstraint</code> components.
 */
public class LinearLayout extends GaugeLayout implements java.io.Serializable
{

static final long serialVersionUID = 3406266177708900157L;

/**
 * Lays out the container linearly. Each component encapsulates the position
 * and extent at which it would like to be placed in the rectangle corresponding
 * to the gauge's linear scale.
 *
 * @param target The container on which a LinearLayout has been set
 */
public void layoutContainer(Container target)
{
    // If we have anything to draw
    if (components.size() > 0) {
        int ncomponents = target.getComponentCount();

        LinearConstraint lConst = ((LinearConstraint)constraints.elementAt(0));
        if (lConst == null) {
            return;
        }
        JCLinearScale lscale = (JCLinearScale)lConst.getGauge().getScale();

        JCLinearScale.Orientation orientation = lscale.getOrientation();
        Rectangle bounds = lscale.getScaleSize();

        // Layout the components
        for (int i = 0 ; i < ncomponents ; i++) {
       	    Component comp = target.getComponent(i);
            int index = components.indexOf(comp);
            lConst = (LinearConstraint) constraints.elementAt(index);
	        double labelExtent = lConst.getExtent();

            double x = 0;
	        double y = 0;
	        int w = comp.getPreferredSize().width;
	        int h = comp.getPreferredSize().height;

	        // Double.MAX_VALUE means draw size of container, otherwise comp
	        // size get the position at which this label will be draw
	        if (labelExtent != Double.MAX_VALUE) {
	            // Get the precalulated position
		        double position = lConst.getPositionAsDouble() / 100.0;

	    	    // Calculate label position
                if (orientation == JCLinearScale.Orientation.VERTICAL) {
		            x = (double)bounds.x + labelExtent * bounds.width;
                    y = (double)bounds.y + bounds.height * position;
                }
                else {
                    x = (double)bounds.x + bounds.width * position;
		            y = (double)bounds.y + labelExtent * bounds.height;
                }

		        // Center the component on this point
                Dimension size = comp.getSize();
                x = x - (double)size.width / 2.0;
	            y = y - (double)size.height / 2.0;

	        }
	    	comp.setBounds((int)x, (int)y, w, h);
	    }
	}
}

}
