/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: RadialLayout.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:30 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.swing.gauge;

import java.awt.*;

/**
 * RadialLayout -- lays out <code>RadialConstraint</code> components.
 */
public class RadialLayout extends GaugeLayout implements java.io.Serializable
{

static final long serialVersionUID = -7628614352061334896L;

/**
 * Lays out the container radially. Each component encapsulates the angle and
 * extent at which it would like to be placed in the circle.
 *
 * @param target The container on which a RadialLayout has been set
 */
public void layoutContainer(Container target)
{
	// If we have anything to draw
	if (components.size() > 0) {
	    int ncomponents 	= target.getComponentCount();

        RadialConstraint rConst = (RadialConstraint)constraints.elementAt(0);
        if (rConst == null) {
            return;
        }
	    JCCircularScale cscale = (JCCircularScale)rConst.getGauge().getScale();
		double radius = cscale.getRadius();

	    // Layout the components
	    for (int i = 0 ; i < ncomponents ; i++) {
	    	Component comp = target.getComponent(i);
            int index = components.indexOf(comp);
	    	rConst = (RadialConstraint)constraints.elementAt(index);
	 		double labelExtent = radius * rConst.getExtent();

	        int x = 0;
	        int y = 0;
			int w = comp.getPreferredSize().width;
	        int h = comp.getPreferredSize().height;

	        // Double.MAX_VALUE means draw size of container, otherwise comp size
	        // get the position at which this label will be draw
	        if (rConst.getExtent() != Double.MAX_VALUE) {
				// Massage the angle
				double angle = rConst.getAngle();

                // In the case of the scale, add the start angle to
                // the angle. This makes positioning independent of
                // direction and start angle in that code.
                if (target instanceof JCCircularTick) {
                	angle += cscale.getStartAngle();
                }

	    	    // Get the radians
		        double r = GaugeUtil.toRadians(angle);
		        x 	     = (int)radius + (int)(labelExtent * Math.cos(r));
	    	    y 		 = (int)radius - (int)(labelExtent * Math.sin(r));

	            // Translate for the real origin
                Rectangle arcBounds =
                    ((JCCircularGauge)rConst.getGauge()).getArcBounds();
                x += arcBounds.x;
                y += arcBounds.y;

		        // Center the component on this point
                Dimension size = comp.getSize();
                x = x - size.width / 2;
	        	y = y - size.height / 2;

	        }
	    	comp.setBounds(x, y, w, h);
	    }
	}
}

}
