/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCCircularGaugeBean.java,v $ $Revision: 1.11
//          $ $Date: 2006-03-17 21:13:31 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.swing.gauge.beans;

import com.klg.jclass.swing.gauge.*;

import java.awt.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

/**
 * A Bean which exposes gauge properties suitable for editing in
 * an Integrated Development Environment (IDE).
 */
public class JCCircularGaugeBean extends JCCircularGauge
                                 implements java.io.Serializable
{

static final long serialVersionUID = -2626452859468404350L;

// interactions

/**
 * Neither clicking nor dragging affects the needle.
 */
public static final int INTERACTION_NONE = 0;

/**
 * The needle follows a mouse drag action.
 */
public static final int INTERACTION_DRAG = 1;

/**
 * The needle snaps to a mouse click.
 */
public static final int INTERACTION_CLICK = 2;

/**
 * The needle responds to both mouse click and mouse drag actions.
 */
public static final int INTERACTION_CLICK_DRAG = 3;

/** Interaction type to enum mapping. */
protected JCAbstractNeedle.InteractionType interactionTypes[] =
	new JCAbstractNeedle.InteractionType[4];

// Tick styles

/**
 * Circle tick style.
 */
public static final int TICK_CIRCLE = 0;

/**
 * Diamond tick style.
 */
public static final int TICK_DIAMOND = 1;

/**
 * Line tick style.
 */
public static final int TICK_LINE = 2;

/**
 * Rectangle tick style.
 */
public static final int TICK_RECTANGLE = 3;

/**
 * Reverse Triangle tick style.
 */
public static final int TICK_REVERSE_TRIANGLE = 4;

/**
 * Triangle tick style.
 */
public static final int TICK_TRIANGLE = 5;


/** <code>JCTickStyle</code> to enum mapping. */
protected JCTickStyle tickStyles[] = new JCTickStyle[6];

// Needle Styles

/**
 * Rectangle needle style.
 */
public static final int NEEDLE_RECTANGLE = 0;

/**
 * Circle needle style.
 */
public static final int NEEDLE_CIRCLE = 1;

/**
 * A pointer needle using these points.
 */
public static final int NEEDLE_POINTER = 2;

/**
 * A tailed pointer needle (tail extends beyond center).
 */
public static final int NEEDLE_TAILED_POINTER = 3;

/**
 * Arrow polygon using these points.
 */
public static final int NEEDLE_ARROW = 4;

/**
 * Tailed Arrow polygon needle style.
 */
public static final int NEEDLE_TAILED_ARROW = 5;

/**
 * Triangle needle style.
 */
public static final int NEEDLE_TRIANGLE = 6;


// Map enums to needle styles
protected JCIndicatorStyle needleStyles[] = new JCIndicatorStyle[7];

// Gauge types

/**
 * The scale is a full circle.
 */
public static final int TYPE_FULL_CIRCLE = 0;

/**
 * The scale is a semicircle oriented so that
 * its diameter is horizontal and its
 * circumference is above its base.
 */
public static final int TYPE_TOP_HALF_CIRCLE = 1;

/**
 * The scale is a semicircle oriented so that
 * its diameter is horizontal and its
 * circumference is below its base.
 */
public static final int TYPE_BOTTOM_HALF_CIRCLE = 2;

/**
 * The scale is a semicircle oriented so that
 * its diameter is vertical and its
 * circumference is to the left of its base.
 */
public static final int TYPE_LEFT_HALF_CIRCLE = 3;

/**
 * The scale is a semicircle oriented so that
 * its diameter is vertical and its
 * circumference is to the right of its base.
 */
public static final int TYPE_RIGHT_HALF_CIRCLE = 4;

/**
 * A quadrant whose arc lies northeast.
 */
public static final int TYPE_UPPER_RIGHT_QUARTER_CIRCLE = 5;

/**
 * A quadrant whose arc lies southeast.
 */
public static final int TYPE_LOWER_RIGHT_QUARTER_CIRCLE	= 6;

/**
 * A quadrant whose arc lies northwest.
 */
public static final int TYPE_UPPER_LEFT_QUARTER_CIRCLE = 7;

/**
 * A quadrant whose arc lies southwest.
 */
public static final int TYPE_LOWER_LEFT_QUARTER_CIRCLE = 8;

// map enums to gauge types
protected JCCircularGauge.GaugeType gaugeTypes[] =
    new JCCircularGauge.GaugeType[9];

/**
 * Scale values increase in a clockwise direction.
 * For example, a scale whose start angle is 0&#176;
 * and whose stop angle is 180&#176;, and whose max
 * and min values are 0 and 100, places its zeroth tick mark
 * at 180&#176; and its 100th tick mark at 0&#176;.
 *
 * @see com.klg.jclass.swing.gauge.JCTick
 */
public static final int DIRECTION_CLOCKWISE = 0;

/**
 * Scale values increase in a clockwise direction.
 * For example, a scale whose start angle is 0&#176;
 * and whose stop angle is 180&#176;, and whose max
 * and min values are 0 and 100, places its zeroth tick mark
 * at 180&#176; and its 100th tick mark at 0&#176;.
 *
 * @see com.klg.jclass.swing.gauge.JCTick
 */
public static final int DIRECTION_COUNTERCLOCKWISE = 1;

/**
 * Scale values increase in a counterclockwise direction.
 * For example, a scale whose start angle is 0&#176;
 * and whose stop angle is 180&#176;, and whose max
 * and min values are 0 and 100, places its zeroth tick mark
 * at 0&#176; and its 100th tick mark at 180&#176;.
 *
 * @see com.klg.jclass.swing.gauge.JCTick
 */
public static final int DIRECTION_FORWARD = 2;

/**
 * Scale values increase in a counterclockwise direction.
 * For example, a scale whose start angle is 0&#176;
 * and whose stop angle is 180&#176;, and whose max
 * and min values are 0 and 100, places its zeroth tick mark
 * at 0&#176; and its 100th tick mark at 180&#176;.
 *
 * @see com.klg.jclass.swing.gauge.JCTick
 */
public static final int DIRECTION_BACKWARD = 3;


// map enums to directions
protected JCAbstractScale.Direction circularDirections[] = new JCAbstractScale.Direction[4];

/**
 * Property listener list.
 */
private PropertyChangeSupport support = new PropertyChangeSupport(this);

/**
 * <code>JCCircularGaugeBean</code> no arg constructor.
 */
public JCCircularGaugeBean()
{
	// Establish a mapping between int enums and interaction enums
	interactionTypes[0] = JCAbstractNeedle.InteractionType.NONE;
	interactionTypes[1] = JCAbstractNeedle.InteractionType.DRAG;
	interactionTypes[2] = JCAbstractNeedle.InteractionType.CLICK;
	interactionTypes[3] = JCAbstractNeedle.InteractionType.CLICK_DRAG;

    // Mapping between JCTickStyles and enum
    tickStyles[0] = JCTickStyle.CIRCLE;
    tickStyles[1] = JCTickStyle.DIAMOND;
    tickStyles[2] = JCTickStyle.LINE;
    tickStyles[3] = JCTickStyle.RECTANGLE;
    tickStyles[4] = JCTickStyle.REVERSE_TRIANGLE;
    tickStyles[5] = JCTickStyle.TRIANGLE;

    // Mapping between JCNeedleStyles and enums
    needleStyles[0] = JCIndicatorStyle.RECTANGLE;
    needleStyles[1] = JCIndicatorStyle.CIRCLE;
	needleStyles[2] = JCIndicatorStyle.POINTER;
	needleStyles[3] = JCIndicatorStyle.TAILED_POINTER;
	needleStyles[4] = JCIndicatorStyle.ARROW;
    needleStyles[5] = JCIndicatorStyle.TAILED_ARROW;
    needleStyles[6] = JCIndicatorStyle.TRIANGLE;

    // Gauge types
    gaugeTypes[0] = JCCircularGauge.GaugeType.FULL_CIRCLE;
    gaugeTypes[1] = JCCircularGauge.GaugeType.TOP_HALF_CIRCLE;
    gaugeTypes[2] = JCCircularGauge.GaugeType.BOTTOM_HALF_CIRCLE;
    gaugeTypes[3] = JCCircularGauge.GaugeType.LEFT_HALF_CIRCLE;
    gaugeTypes[4] = JCCircularGauge.GaugeType.RIGHT_HALF_CIRCLE;
    gaugeTypes[5] = JCCircularGauge.GaugeType.UPPER_RIGHT_QUARTER_CIRCLE;
    gaugeTypes[6] = JCCircularGauge.GaugeType.LOWER_RIGHT_QUARTER_CIRCLE;
    gaugeTypes[7] = JCCircularGauge.GaugeType.UPPER_LEFT_QUARTER_CIRCLE;
    gaugeTypes[8] = JCCircularGauge.GaugeType.LOWER_LEFT_QUARTER_CIRCLE;

    // Directions
    circularDirections[0] = JCAbstractScale.Direction.CLOCKWISE;
    circularDirections[1] = JCAbstractScale.Direction.COUNTERCLOCKWISE;
    circularDirections[2] = JCAbstractScale.Direction.FORWARD;
    circularDirections[3] = JCAbstractScale.Direction.BACKWARD;
}

/**
 * Adds a property change listener for needle value changes.
 * @param l
 */
public void addPropertyChangeListener(PropertyChangeListener l)
{
    support.addPropertyChangeListener(l);
}

/**
 * Removes a property change listener for needle value changes.
 * @param l
 */
public void removePropertyChangeListener(PropertyChangeListener l)
{
    support.removePropertyChangeListener(l);
}

/**
 * Gets the radius of the center. The value is a decimal fraction of the
 * scale's radius.
 *
 * @return the radius as a fraction of the scale's radius
 */
public double getCenterRadius()
{
	return(getCenter().getRadius());
}

/**
 * Sets the radius of the disk. The <code>radius</code> parameter is a decimal
 * fraction of the scale's radius.
 *
 * @param radius the radius of the center as a proportion of the circular
 *     scale's radius
 */
public void setCenterRadius(double radius)
{
	getCenter().setRadius(radius);
}

/**
 * Gets the center's color.
 *
 * @return the center's color
 */
public Color getCenterColor()
{
	return(getCenter().getForeground());
}

/**
 * Sets the center's color.
 *
 * @param color the center's color
 */
public void setCenterColor(Color color)
{
	getCenter().setForeground(color);
}

/**
 * Returns the length of the needle as a ratio.
 *
 * @return the length of the needle in pixels
 */
public double getNeedleLength()
{
	JCNeedle needle = (JCNeedle) getNeedles().firstElement();
	return(needle.getLength());
}

/**
 * Sets the length of the needle as a ratio.
 *
 * @param length the length in pixels
 */
public void setNeedleLength(double length)
{
	JCNeedle needle = (JCNeedle) getNeedles().firstElement();
	needle.setLength(length);
}


/**
 * Returns the inner extent of the needle as a ratio.
 *
 * @return the inner extent of the needle in pixels
 */
public double getNeedleInnerExtent()
{
	JCNeedle needle = (JCNeedle) getNeedles().firstElement();
	return(needle.getInnerExtent());
}

/**
 * Sets the inner extent of the needle as a ratio.
 *
 * @param innerExtent extent the needle in pixels
 */
public void setNeedleInnerExtent(double innerExtent)
{
	JCNeedle needle = (JCNeedle) getNeedles().firstElement();
	needle.setInnerExtent(innerExtent);
}

/**
 * Returns the outer extent of the needle as a ratio.
 *
 * @return the outer extent of the needle in pixels
 */
public double getNeedleOuterExtent()
{
	JCNeedle needle = (JCNeedle) getNeedles().firstElement();
	return(needle.getOuterExtent());
}

/**
 * Sets the outer extent of the needle as a ratio.
 *
 * @param outerExtent the outer extent in pixels
 */
public void setNeedleOuterExtent(double outerExtent)
{
	JCNeedle needle = (JCNeedle) getNeedles().firstElement();
	needle.setOuterExtent(outerExtent);
}

/**
 * Returns the width of the needle in pixels.
 *
 * @return the width of the needle in pixels
 */
public double getNeedleWidth()
{
	JCNeedle needle = (JCNeedle) getNeedles().firstElement();
	return(needle.getNeedleWidth());
}

/**
 * Sets the width of the needle in pixels.
 *
 * @param width the width in pixels
 */
public void setNeedleWidth(double width)
{
	JCNeedle needle = (JCNeedle) getNeedles().firstElement();
	needle.setNeedleWidth(width);
}

/**
 * Returns the scale value at which the needle is currently pointing.
 *
 * @return the scale value at which the needle is currently pointing.
 */
public double getNeedleValue()
{
	JCNeedle needle = (JCNeedle) getNeedles().firstElement();
	return(needle.getValue());
}

/**
 * Moves the needle to point at this scale value. Fires the
 * <code>needleValue</code> property change event.
 *
 * @param value the value at which to point
 */
public void setNeedleValue(double value)
{
	JCNeedle needle = (JCNeedle) getNeedles().firstElement();
    Double oldValue = new Double(needle.getValue());
	needle.setValue(value);
    support.firePropertyChange("needleValue", oldValue, new Double(value));
}

/**
 * Returns the interation type for this needle.
 *
 * @return the <code>JCAbstractNeedle.InteractionType</code> for this needle
 */
public int getNeedleInteractionType()
{
	JCNeedle needle = (JCNeedle) getNeedles().firstElement();
	return(getInteractionTypeIndex(needle.getInteractionType()));
}

/**
 * Sets the interation type for this needle.
 *
 * @param type the <code>JCAbstractNeedle.InteractionType</code> for this
 *     needle; one of: <ul> <li><code>InteractionType.NONE</code></li>
 *     <li><code>InteractionType.DRAG</code></li>
 *     <li><code>InteractionType.CLICK</code>
 *     </li> <li><code>InteractionType.CLICK_DRAG</code></li> </ul>
 */
public void setNeedleInteractionType(int type)
{
	JCNeedle needle = (JCNeedle) getNeedles().firstElement();
	needle.setInteractionType(interactionTypes[type]);
}

/**
 * Returns the <code>JCIndicatorStyle</code> for this needle.
 *
 * @return the <code>JCIndicatorStyle</code> of this needle
 */
public int getNeedleStyle()
{
	JCNeedle needle = (JCNeedle) getNeedles().firstElement();
	return(getNeedleStyleIndex(needle.getNeedleStyle()));
}

/**
 * Sets the <code>JCIndicatorStyle</code> for this needle.
 *
 * @param style the <code>JCIndicatorStyle</code> for this needle
 */
public void setNeedleStyle(int style)
{
	JCNeedle needle = (JCNeedle) getNeedles().firstElement();
	needle.setNeedleStyle(needleStyles[style]);
}

/**
 * Returns the color of the needle.
 *
 * @return the color of the needle
 */
public Color getNeedleColor()
{
	JCCircularNeedle needle = (JCCircularNeedle) getNeedles().firstElement();
	return needle.getForeground();
}

/**
 * Sets the color of the needle.
 *
 * @param color the color of the needle
 */
public void setNeedleColor(Color color)
{
	JCCircularNeedle needle = (JCCircularNeedle) getNeedles().firstElement();
	needle.setForeground(color);
}

/**
 * Gets the scale's color.
 *
 * @return color the scale's color
 */
public Color getScaleColor()
{
	return(((JCCircularScale)getScale()).getForeground());
}

/**
 * Sets the scale's color.
 *
 * @param color the scale color
 */
public void setScaleColor(Color color)
{
	((JCCircularScale)getScale()).setForeground(color);
}

/**
 * Gets the start angle of the scale.
 *
 * @return the angle at which the scale starts; angles
 * increase counterclockwise from 3 o'clock -- 0 degrees
 */
public double getStartAngle()
{
	return(((JCCircularScale)getScale()).getStartAngle());
}

/**
 * Sets the start angle of the scale.
 *
 * @param angle the angle at which the scale starts; angles
 * increase counterclockwise from 3 o'clock -- 0 degrees
 */
public void setStartAngle(double angle)
{
	((JCCircularScale)getScale()).setStartAngle(angle);
}

/**
 * Gets the stop angle of the scale.
 *
 * @return the angle at which the scale terminates; angles
 * increase counterclockwise from 3 o'clock -- 0 degrees
 */
public double getStopAngle()
{
	return(((JCCircularScale)getScale()).getStopAngle());
}

/**
 * Sets the stop angle of the scale.
 *
 * @param angle the angle at which the scale terminates; angles
 * increase counterclockwise from 3 o'clock -- 0 degrees
 */
public void setStopAngle(double angle)
{
	((JCCircularScale)getScale()).setStopAngle(angle);
}

/**
 * Returns the extent to which the scale's background will
 * be drawn from the center. This is a fraction of the scale's radius.
 *
 * @return the extent
 */
public double getScaleExtent()
{
	return(getScale().getZoomFactor());
}

/**
 * Sets the extent to which the scale's background will
 * be drawn from the center. This is a fraction of the scale's
 * radius.
 *
 * @param extent the extent
 */
public void setScaleExtent(double extent)
{
	getScale().setZoomFactor(extent);
}

/**
 * Returns the value which will appear at the scale's <code>startAngle</code>.
 *
 * @return the value which will appear at the scale's <code>startAngle</code>
 */
public double getScaleMin()
{
	return(getScale().getMin());
}

/**
 * Sets the value which will appear at the scale's <code>startAngle</code>.
 *
 * @param min the value which will appear at the scale's <code>startAngle</code>
 */
public void setScaleMin(double min)
{
    getScale().setMin(min);
    repaint();
}

/**
 * Returns the value which will appear at the scale's <code>stopAngle</code>.
 *
 * @return the value which will appear at the scale's <code>stopAngle</code>
 */
public double getScaleMax()
{
    return(getScale().getMax());
}

/**
 * Sets the value which will appear at the scale's <code>stopAngle</code>.
 *
 * @param max the value which will appear at the scale's <code>stopAngle</code>
 */
public void setScaleMax(double max)
{
    getScale().setMax(max);
}

/**
 * Paints the complete background defined by the <code>GaugeType</code>,
 * not just area swept by scale. For instance, you can have a
 * scale defined from 0&#176; to 270&#176; for a <code>FULL_CIRCLE</code>, yet
 * have the full circle background painted, not just from 0&#176; to 270&#176;.
 * The default is <code>false</code>.
 *
 * @return <code>true</code> means paint all the background; <code>false</code>
 *     means just the area spanned by scale's sweep angle
 */
public boolean getPaintCompleteBackground()
{
    return(((JCCircularScale)getScale()).getPaintCompleteBackground());
}

/**
 * If <code>true</code>, paints the complete background for this
 * <code>GaugeType</code>, not just area between the scale's start and stop
 * angles. For instance, a scale defined from 0&#176; to 270&#176; and having
 * gauge type <code>FULL_CIRCLE</code> has the full circular background painted,
 * not just the part from 0&#176; to 270&#176;. The default is
 * <code>false</code>.
 *
 * @param b <code>true</code> means paint all of the background
 *     consistent with the gauge type
 */
public void setPaintCompleteBackground(boolean b)
{
    ((JCCircularScale)getScale()).setPaintCompleteBackground(b);
}

/**
 * Returns the <code>automatic</code> boolean for this tick object.
 *
 * @return the automatic setting for this tick; <code>true</code> means ticks
 *     and increments are dynamically determined by the gauge; otherwise,
 *     <code>startValue</code>, <code>stopValue</code>, and
 *     <code>incrementValue</code> are used to lay out ticks
 */
public boolean getAutoTickGeneration()
{
    JCTick tick = (JCTick)getTicks().firstElement();
    return(tick.getAutomatic());
}

/**
 * Sets the <code>automatic</code> boolean for this tick object.
 *
 * @param b the automatic setting for this tick: <code>true</code> means ticks
 *     and increments are dynamically determined by the gauge; otherwise,
 *     <code>startValue</code>, <code>stopValue</code>, and
 *     <code>incrementValue</code> are used to lay out ticks
 */
public void setAutoTickGeneration(boolean b)
{
    JCTick tick = (JCTick)getTicks().firstElement();
    tick.setAutomatic(b);
}

/**
 * In non-automatic mode, uses this value as the start value.
 *
 * @return the <code>startValue</code>
 */
public double getTickStartValue()
{
    JCTick tick = (JCTick)getTicks().firstElement();
    return(tick.getStartValue());
}

/**
 * In non-automatic mode, uses this value as the start value.
 *
 * @param value the <code>startValue</code>
 */
public void setTickStartValue(double value)
{
    JCTick tick = (JCTick)getTicks().firstElement();
    tick.setStartValue(value);
}

/**
 * In non-automatic mode, uses this value as the stop value.
 *
 * @return the <code>stopValue</code>
 */
public double getTickStopValue()
{
    JCTick tick = (JCTick)getTicks().firstElement();
    return(tick.getStopValue());
}

/**
 * In non-automatic mode, uses this value as the stop value.
 *
 * @param value the <code>stopValue</code>
 */
public void setTickStopValue(double value)
{
    JCTick tick = (JCTick)getTicks().firstElement();
    tick.setStopValue(value);
}

/**
 * In non-automatic mode, uses this value as the increment value; that
 * is, places ticks at every <code>incrementValue</code> along the scale.
 *
 * @return the spacing between adjacent tick marks
 */
public double getTickIncrement()
{
    JCTick tick = (JCTick)getTicks().firstElement();
    return(tick.getIncrementValue());
}

/**
 * In non-automatic mode, uses this value as the increment value; that
 * is, places ticks at every <code>incrementValue</code> along the scale.
 *
 * @param value the spacing between adjacent tick marks
 */
public void setTickIncrement(double value)
{
    JCTick tick = (JCTick)getTicks().firstElement();
    tick.setIncrementValue(value);
}

/**
 * Returns the boolean controlling the drawing of labels. <code>true</code>
 * means draw the labels, <code>false</code> means hide the labels.
 *
 * @return the boolean controlling the drawing of labels
 */
public boolean getDrawTickLabels()
{
    JCTick tick = (JCTick)getTicks().firstElement();
    return(tick.getDrawLabels());
}

/**
 * Sets the boolean controlling the drawing of labels. <code>true</code>
 * means draw the labels, <code>false</code> means hide the labels.
 *
 * @param b the boolean controlling the drawing of labels
 */
public void setDrawTickLabels(boolean b)
{
    JCTick tick = (JCTick)getTicks().firstElement();
    tick.setDrawLabels(b);
}

/**
 * Returns the boolean controlling the drawing of ticks. <code>True</code>
 * means draw the ticks, <code>false</code> means hide the ticks.
 *
 * @return the boolean controlling the drawing of ticks
 */
public boolean getDrawTickMarks()
{
    JCTick tick = (JCTick)getTicks().firstElement();
    return(tick.getDrawTicks());
}

/**
 * Sets the boolean controlling the drawing of ticks. <code>true</code>
 * means draw the ticks, <code>false</code> means hide the ticks.
 *
 * @param b the boolean controlling the drawing of ticks
 */
public void setDrawTickMarks(boolean b)
{
    JCTick tick = (JCTick)getTicks().firstElement();
    tick.setDrawTicks(b);
}

/**
 * Returns the width in pixels of the tick mark.
 *
 * @return the width of the tick to be drawn
 */
public double getTickWidth()
{
    JCTick tick = (JCTick)getTicks().firstElement();
    return(tick.getTickWidth());
}

/**
 * Sets the width in pixels of the tick mark.
 *
 * @param width the width of the tick to be drawn in pixels
 */
public void setTickWidth(double width)
{
    JCTick tick = (JCTick)getTicks().firstElement();
    tick.setTickWidth(width);
}

/**
 * Returns the percentage of the scale's radius (as a decimal) at
 * at which to begin drawing ticks radially outward.
 *
 * @return the <code>innerExtent</code> of the tick mark
 */
public double getTickInnerExtent()
{
    JCTick tick = (JCTick)getTicks().firstElement();
    return(tick.getInnerExtent());
}

/**
 * Sets the percentage of the scale's radius (as a decimal) at
 * at which to begin drawing ticks radially outward.
 *
 * @param extent the inner extent of the tick mark
 */
public void setTickInnerExtent(double extent)
{
    JCTick tick = (JCTick)getTicks().firstElement();
    tick.setInnerExtent(extent);
}

/**
 * Returns the percentage of the scale's radius (as a decimal) at
 * which to stop drawing the ticks.
 *
 * @return the <code>outerExtent</code> of the tick mark
 */
public double getTickOuterExtent()
{
    JCTick tick = (JCTick)getTicks().firstElement();
    return(tick.getOuterExtent());
}

/**
 * Sets the percentage of the scale's radius (as a decimal) at
 * which to stop drawing the ticks.
 *
 * @param extent the <code>outerExtent</code> of the tick mark
 */
public void setTickOuterExtent(double extent)
{
    JCTick tick = (JCTick)getTicks().firstElement();
    tick.setOuterExtent(extent);
}

/** Returns the <code>JCTickStyle</code> controlling the shape of the tick.
 *
 * @return the tick style
 * @see com.klg.jclass.swing.gauge.JCTickStyle
 */
public int getTickStyle()
{
    JCTick tick = (JCTick)getTicks().firstElement();
    return(getTickStyleIndex(tick.getTickStyle()));
}

/**
 * Sets the <code>JCTickStyle</code> controlling the shape of the tick.
 * Users may define their own tick style, if desired.
 *
 * @param style one of the built-in tick styles, or a user-supplied style
 * @see com.klg.jclass.swing.gauge.JCTickStyle
 */
public void setTickStyle(int style)
{
    JCTick tick = (JCTick)getTicks().firstElement();
    tick.setTickStyle(tickStyles[style]);
}

/**
 * Returns the position, expressed as a decimal fraction of the scale's radius,
 * where tick labels are placed.
 *
 * @return the extent at which labels will be drawn
 */
public double getTickLabelExtent()
{
    JCTick tick = (JCTick)getTicks().firstElement();
    return(tick.getLabelExtent());
}

/**
 * Sets the percentage, expressed as a decimal fraction of the scale's radius,
 * at which to place the center of the tick label objects.
 *
 * @param extent the extent at which labels will be drawn
 */
public void setTickLabelExtent(double extent)
{
    JCTick tick = (JCTick)getTicks().firstElement();
    tick.setLabelExtent(extent);
}

/**
 * Returns the font in which ticks are to be drawn.
 *
 * @return  the font in which ticks are to be drawn
 */
public Font getTickFont()
{
    JCTick tick = (JCTick)getTicks().firstElement();
    return(((JCCircularTick)tick).getFont());
}

/**
 * Sets the font in which ticks are to be drawn.
 *
 * @param  font the font in which ticks are to be drawn
 */
public void setTickFont(Font font)
{
    JCTick tick = (JCTick)getTicks().firstElement();
    ((JCCircularTick)tick).setFont(font);
}

/**
 * Returns the color in which tick labels will be drawn.
 *
 * @return the color in which tick labels will be drawn
 */
public Color getTickFontColor()
{
    JCTick tick = (JCTick)getTicks().firstElement();
    return(tick.getFontColor());
}

/**
 * Sets the color in which tick labels will be drawn.
 *
 * @param color the color in which tick labels will be drawn
 */
public void setTickFontColor(Color color)
{
    JCTick tick = (JCTick)getTicks().firstElement();
    tick.setFontColor(color);
}

/**
 * Returns the color in which tick marks will be drawn.
 *
 * @return the color in which tick marks will be drawn
 */
public Color getTickColor()
{
    JCTick tick = (JCTick)getTicks().firstElement();
    return(tick.getTickColor());
}

/**
 * Sets the color in which tick marks will be drawn.
 *
 * @param color the color in which tick marks will be drawn
 */
public void setTickColor(Color color)
{
    JCTick tick = (JCTick)getTicks().firstElement();
    tick.setTickColor(color);
}

/**
 * If <code>true</code>, uses gauge-determined precision for tick labels;
 * otherwise, uses <code>setPrecision</code> to label tick marks.
 *
 * @return a boolean indicating whether or not a label's precision is
 *     calculated by the gauge or set using <code>setPrecision</code>
 */
public boolean getPrecisionUseDefault()
{
    JCTick tick = (JCTick)getTicks().firstElement();
    return(tick.getPrecisionUseDefault());
}

/**
 * If <code>true</code>, uses gauge-determined precision for tick labels;
 * otherwise, uses <code>setPrecision</code> to label tick marks.
 *
 * @param b a boolean indicating whether or not a label's precision is
 *     calculated by the gauge or set using <code>setPrecision</code>
 */
public void setPrecisionUseDefault(boolean b)
{
    JCTick tick = (JCTick)getTicks().firstElement();
    tick.setPrecisionUseDefault(b);
}

/**
 * Returns the format for numbers in a tick label: positive values denote the
 * number of places after the decimal point (for example, 3 means multiples
 * of 0.001); negative values indicate the positive number of zeros are
 * used before the decimal place (for example, -3 means numbering will be in
 * multiples of 1000). Overridden if <code>precisionUseDefault</code> is set to
 * <code>true</code>.
 *
 * @return the precision
 */
public int getPrecision()
{
    JCTick tick = (JCTick)getTicks().firstElement();
    return(tick.getPrecision());
}

/**
 * Sets the format for numbers in a tick label: positive values denote the
 * number of places after the decimal point (for example, 3 means multiples
 * of 0.001); negative values indicate the positive number of zeros are
 * used before the decimal place (for example, -3 means numbering will be in
 * multiples of 1000). Overridden if <code>precisionUseDefault</code> is set to
 * <code>true</code>.
 *
 * @param precision an integer
 */
public void setPrecision(int precision)
{
    JCTick tick = (JCTick)getTicks().firstElement();
    tick.setPrecision(precision);
}


/**
 * Returns the <code>JCCircularGauge</code> type for this gauge.
 *
 * @return the <code>JCCircularGaugeBean</code> type of this gauge
 */
public int getType()
{
    return(getGaugeTypeIndex(getGaugeType()));
}

/**
 * Sets the <code>JCCircularGaugeBean</code> type for this gauge.
 *
 * @param type the <code>JCCircularGaugeBean</code> type for this gauge
 */
public void setType(int type)
{
    setGaugeType(gaugeTypes[type]);
}

/**
 * Returns the direction enum for this gauge.
 *
 * @return the direction enum for this gauge
 */
public int getDirection()
{
    return(getDirectionIndex(getScale().getDirection()));
}

/**
 * Sets the direction enum for this gauge.
 *
 * @param direction the direction enum for this gauge
 */
public void setDirection(int direction)
{
    getScale().setDirection(circularDirections[direction]);
}

/**
 * Gets the zoom factor for this scale.
 * @return zoom factor
 */
public double getZoomFactor()
{
    return(getScale().getZoomFactor());
}

/**
 * Sets the zoom factor for this scale.
 * @param zoomFactor zoom factor
 */
public void setZoomFactor(double zoomFactor)
{
    getScale().setZoomFactor(zoomFactor);
}

// utility methods

/**
 * Returns the enum index for a needle interaction type.
 * @param type the type to lookup
 * @return the enum for this type
 */
public int getInteractionTypeIndex(JCAbstractNeedle.InteractionType type)
{
    for (int i = 0; i < interactionTypes.length; i++) {
        if (interactionTypes[i] == type) {
            return(i);
        }
    }
    return(0);
}

/**
 * Returns the enum for a given <code>JCTickStyle</code>.
 * @param style for which the index is sought
 * @return the index of the tick style
 */
public int getTickStyleIndex(JCTickStyle style)
{
    for (int i = 0; i < tickStyles.length; i++) {
        if (tickStyles[i] == style) {
            return(i);
        }
    }
    return(0);
}

/**
 * Returns the enum for a given <code>JCIndicatorStyle</code>.
 * @param style for which the index is sought
 * @return the index of the needle style
 */
public int getNeedleStyleIndex(JCIndicatorStyle style)
{
    for (int i = 0; i < needleStyles.length; i++) {
        if (needleStyles[i] == style) {
            return(i);
        }
    }
    return(0);
}

/**
 * Returns the enum for a given <code>JCCircularGauge.GaugeType</code>.
 * @param type for which the index is sought
 * @return the index of the <code>JCCircularGauge.GaugeType</code>
 */
public int getGaugeTypeIndex(JCCircularGauge.GaugeType type)
{
    for (int i = 0; i < gaugeTypes.length; i++) {
        if (gaugeTypes[i] == type) {
            return(i);
        }
    }
    return(0);
}

/**
 * Returns the enum for a given <code>JCCircularScale.Direction</code>.
 * @param direction for which the index is sought
 * @return the index of the <code>JCCircularScale.Direction</code>
 */
public int getDirectionIndex(JCAbstractScale.Direction direction)
{
    for (int i = 0; i < circularDirections.length; i++) {
        if (circularDirections[i] == direction) {
            return(i);
        }
    }
    return(0);
}

}
