/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: CalendarPopup.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:31 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.swing.plaf;

import com.klg.jclass.swing.JCPopupCalendar;
import com.klg.jclass.swing.JCPopupCalendarEditor;

import com.klg.jclass.util.swing.JCPopupEvent;
import com.klg.jclass.util.swing.JCPopupListener;

import javax.swing.*;
import javax.swing.plaf.basic.BasicComboPopup;
import java.awt.*;
import java.util.Date;

public class CalendarPopup extends BasicComboPopup
                           implements java.io.Serializable
{

protected JCPopupCalendarEditor  popup = null;
protected JCPopupCalendar        popupCalendar;
protected PopupListener          popupListener = new PopupListener();
protected Date                   originalDate = null;

public CalendarPopup(JComboBox comboBox, JCPopupCalendarEditor popup)
{
	super(comboBox);
	
	popupCalendar = (JCPopupCalendar)comboBox;
	setPopupCalendarEditor(popup);
}

public void setPopupCalendarEditor(JCPopupCalendarEditor popupEditor)
{
	
	if (popupEditor == null) {
		return;
	}

	if (popup != null) {
        popup.removePopupListener(popupListener);
		remove(popup.getEditorComponent());
	}
	popup = popupEditor;
	popup.addPopupListener(popupListener);
    JComponent popupComponent = popup.getEditorComponent();
    add(popupComponent);

    // Call Component.setFocusable(false) on all children of the popup editor.
    setFocusableOnTree(popupComponent, false);
}

protected void setFocusableOnTree(Component parent, boolean state)
{
    if (parent == null) {
        return;
    }

    parent.setFocusable(false);
    if (parent instanceof Container) {
        Component[] components = ((Container)parent).getComponents();
        for (int i = 0; i < components.length; i++) {
            setFocusableOnTree(components[i], state);
        }
    }
}

/**
 * Called to configure this JPopupMenu (BasicComboPopup is a JPopupMenu).
 */
protected void configurePopup()
{
    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    setBorderPainted(true);
    setBorder(BorderFactory.createLineBorder(Color.black));
    setOpaque(false);
    setDoubleBuffered(true);
    setRequestFocusEnabled(false);
}

/**
 * Implementation of ComboPopup.show().
 */
public void show()
{
    if (popup == null || popup.getEditorComponent() == null) {
        return;
    }

    originalDate = popupCalendar.getValue();
    if (originalDate != null) {
        popup.setValue(originalDate);
    }

   JComponent editorComponent = popup.getEditorComponent();
   Rectangle popupBounds = computePopupBounds(0,
                                              popupCalendar.getBounds().height,
                                              editorComponent.getWidth(),
                                              editorComponent.getHeight());
    setLightWeightPopupEnabled(popupCalendar.isLightWeightPopupEnabled());

    show(popupCalendar, popupBounds.x, popupBounds.y);
}

/**
 * Return the date when the popup was originally popped up.
 * @return The original date, when the editing started.
 */
public Date getOriginalDate()
{
    return(originalDate);
}

protected Rectangle computePopupBounds(int px, int py, int pw, int ph)
{
    Insets insets = getInsets();
    Rectangle absBounds = new Rectangle();
    Rectangle r = new Rectangle(px, py, pw, ph);
    Point p = new Point(0, 0);
    Dimension scrSize = Toolkit.getDefaultToolkit().getScreenSize();

    SwingUtilities.convertPointFromScreen(p, comboBox);
    absBounds.x = p.x;
    absBounds.y = p.y;
    absBounds.width = scrSize.width;
    absBounds.height = scrSize.height;
    if (isRectangleContainingRectangle(absBounds, r)) {
        return(r);
    }
    else {
        // Find a place for it to fit
        if (!isRectangleContainingRectangleFitRight(absBounds, r)) {
            r.x += (absBounds.x + absBounds.width -
                   (r.width + insets.left + insets.right));
        }
        if (!isRectangleContainingRectangleFitLeft(absBounds, r)) {
            r.x = absBounds.x;
        }
        if (!isRectangleContainingRectangleFitDown(absBounds, r)) {
            r.y = -(r.height + insets.top + insets.bottom);
        }
        return(r);
    }
}

public final boolean isRectangleContainingRectangle(Rectangle a, Rectangle b)
{
    return(b.x >= a.x && (b.x + b.width) <= (a.x + a.width) &&
           b.y >= a.y && (b.y + b.height) <= (a.y + a.height));
}

public final boolean isRectangleContainingRectangleFitLeft(Rectangle a,
						                                   Rectangle b)
{
    return(a.x < 0);
}

public final boolean isRectangleContainingRectangleFitRight(Rectangle a,
						                                    Rectangle b)
{
    return((b.x + b.width) < (a.x + a.width));
}

public final boolean isRectangleContainingRectangleFitDown(Rectangle a,
						                                   Rectangle b)
{
    return(b.y >= a.y && (b.y + b.height) <= (a.y + a.height));
}

public void hide()
{
    super.hide();
    if (!popupListener.inCommit && popup != null) {
        popup.firePopupEvent();
    }
}

class PopupListener implements JCPopupListener, java.io.Serializable
{
    boolean inCommit = false;

    public void commit(JCPopupEvent e)
	{
        inCommit = true;
	    hide();
        inCommit = false;
	}
} // PopupListener

}
