/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CellArea.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:32 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;



import javax.swing.*;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

/**
 * A <CODE>CellArea</CODE> is a transparent component that manages
 * a specific scrollable or non-scrollable region within JCTable.
 * There will typically be 1 to 9 Area objects in a JCTable --
 * depending on the existance of row labels, column labels, and
 * frozen rows/columns. It also acts as the container
 * for editors and components in cells.
 */
public class CellArea extends JComponent {

public final static int             CELL = 0;
public final static int             FROZENROW = 1;
public final static int             COLUMNLABEL = 2;
public final static int             FROZENCOLUMN = 3;
public final static int             FROZENCELL = 4;
public final static int             FROZENCOLUMNLABEL = 5;
public final static int             ROWLABEL = 6;
public final static int             FROZENROWLABEL = 7;

protected int                       standard_offset_x; // x origin adjustor based on the type of area
                                                       // and relative placement
protected int                       standard_offset_y; // y origin adjustor based on the type of area
                                                       // and relative placement

protected Dimension                 cell_size;         // the total dimensions of the contained cells

protected int                       start_row;         // the start row of the managed area
protected int                       start_column;      // the start column of the managed area
protected int                       end_row;           // the end row of the managed area
protected int                       end_column;        // the end column of the managed area

protected int                       type;              // the type of area

/**
 * Creates a new CellArea.
 */
protected CellArea(int type)
{
	this.type = type;

	this.cell_size = new Dimension(0, 0);

	setCellRange(JCTableEnum.NOVALUE, JCTableEnum.NOVALUE,
				 JCTableEnum.NOVALUE, JCTableEnum.NOVALUE);

	standard_offset_x = 0;
	standard_offset_y = 0;

	// Note: this needs to be an exact replica of what is in JCTable.
	// We will pass these events up to JCTable.  This is needed
	// to bypass cases where events do not automatically propogate
	// up to the parent table from the CellArea.  (That happens when
	// event listeners are attached to the cell area by other classes,
	// like cell editors.)
    enableEvents(AWTEvent.MOUSE_EVENT_MASK        |
                 AWTEvent.MOUSE_MOTION_EVENT_MASK |
				 AWTEvent.KEY_EVENT_MASK);
}

// Event handling -- need to pass on events up to the parent table
// where the actual handling is done.
protected void processMouseEvent(MouseEvent e)
{
	MouseEvent e2 = translateMouseEventToParent(e);
    super.processMouseEvent(e);
	((JCTable)getParent()).processMouseEvent(e2);
}

protected void processMouseMotionEvent(MouseEvent e)
{
	MouseEvent e2 = translateMouseEventToParent(e);
	((JCTable)getParent()).processMouseMotionEvent(e2);
    super.processMouseMotionEvent(e);
}

protected MouseEvent translateMouseEventToParent(MouseEvent e)
{
	int x = e.getX() + getLocation().x;
	int y = e.getY() + getLocation().y;
	MouseEvent e2 = new MouseEvent(getParent(), e.getID(), e.getWhen(),
					               e.getModifiers(), x, y, e.getClickCount(),
								   e.isPopupTrigger());
	return(e2);
}

protected void processKeyEvent(KeyEvent e)
{
	
	KeyEvent e2 = new KeyEvent(getParent(), e.getID(), e.getWhen(),
				               e.getModifiers(), e.getKeyCode(),
							   e.getKeyChar());
	((JCTable)getParent()).processKeyEvent(e2);
    super.processKeyEvent(e);
}

/* DEBUGGING
public void paintComponent(Graphics gc)
{
	Rectangle r = getBounds();
	//gc.setColor(Color.blue);
	//gc.drawRect(0, 0, r.width - 1, r.height - 1);

	java.awt.Font f = new java.awt.Font("Lucida Console",
                                        java.awt.Font.PLAIN, 9);
	gc.setFont(f);

	java.awt.FontMetrics fm = gc.getFontMetrics();
	int width = fm.stringWidth(getTypeString());
	int height = fm.getHeight();

	gc.setColor(Color.black);
	gc.fillRect(3, 3, width + 2, 13);
	gc.setColor(new Color(255, 255, 204));
	gc.drawString(getTypeString(), 4, 13);
}
 */

/**
 * Returns the range of cells contained by the Area.
 * @see #setCellRange
 */
public JCCellRange getCellRange()
{
	return(new JCCellRange(start_row, start_column, end_row, end_column));
}

/**
 * Sets the range of cells that the Area manages.
 */
public void setCellRange(JCCellRange range)
{
	setCellRange(range.start_row, range.start_column,
	             range.end_row, range.end_column);
}

/**
 * Sets the range of cells that the area manages.
 */
public void setCellRange(int start_row, int start_column,
			             int end_row, int end_column)
{
	this.start_row    = start_row;
	this.start_column = start_column;
	this.end_row      = end_row;
	this.end_column   = end_column;
}

/**
 * Returns the total dimensions of the contained cells.
 */
public Dimension getCellSize()
{
	return(cell_size);
}

/**
 * Sets the total dimensions of the contained cells for use in
 * preferred size calculations.
 */
public void setCellSize(Dimension d)
{
	cell_size.height = d.height;
	cell_size.width  = d.width;
}

/**
 * Sets the total dimensions of the contained cells.
 */
public void setCellSize(int width, int height)
{
	cell_size.height = height;
	cell_size.width  = width;
}

/**
 * Returns true if the given cell is contained in the area.
 */
public boolean containsCell(int row, int column)
{
	return(start_row != JCTableEnum.NOVALUE &&
		   start_row <= row                 &&
		   end_row >= row                   &&
		   start_column <= column           &&
		   end_column >= column);
}

/**
 * Returns true if the bounds of the area intersects the given rectangle.
 */
public boolean intersects(Rectangle r)
{
	return(!((r.x + r.width <= getX())    ||
	         (r.y + r.height <= getY())   ||
		     (r.x >= getX() + getWidth()) ||
		     (r.y >= getY() + getHeight())));
}

/**
 * Returns the minimum size of the area.
 */
public Dimension getMinimumSize()
{
	switch(type) {
	case CELL:
		return(new Dimension(1, 1));
	case FROZENROW:
	case COLUMNLABEL:
		return(new Dimension(1, getPreferredSize().height));
	case FROZENCOLUMN:
	case ROWLABEL:
		return(new Dimension(getPreferredSize().width, 1));
	default:
		return(getPreferredSize());
	}
}

/**
 * Returns the preferred size of the cell area. The preferred
 * cell is the space required to display all cells mapped to
 * the cell area.
 */
public Dimension getPreferredSize()
{
	return(cell_size);
}

/**
 * Returns the x offset of the area bounds to standard layout.
 */
public int getStandardOffsetX()
{
	return(standard_offset_x);
}

/**
 * Returns the y offset of the area bounds to standard layout.
 */
public int getStandardOffsetY()
{
	return(standard_offset_y);
}

/**
 * Sets the offset of the area bounds to standard layout.
 */
public void setStandardOffset(int x, int y)
{
	standard_offset_x = x;
	standard_offset_y = y;
}

/**
 * Sets the x offset of the area bounds to standard layout.
 */
public void setStandardOffsetX(int x)
{
	standard_offset_x = x;
}

/**
 * Sets the y offset of the area bounds to standard layout.
 */
public void setStandardOffsetY(int y)
{
	standard_offset_y = y;
}

/**
 * Returns the type of area.
 * @see #setType
 */
public int getType()
{
	return(type);
}

/**
 * Sets the type for the area.
 * Valid types are <CODE>Area.CELL</CODE>, <CODE>CellArea.FROZENROW</CODE>,
 * <CODE>CellArea.COLUMNLABEL</CODE>, <CODE>CellArea.FROZENCOLUMN</CODE>,
 * <CODE>CellArea.FROZENCELL</CODE>, <CODE>CellArea.FROZENCOLUMNLABEL</CODE>,
 * <CODE>CellArea.ROWLABEL</CODE>, <CODE>CellArea.FROZENROWLABEL</CODE>
 */
public void setType(int type)
{
	this.type = type;
}

/**
 * Returns a string representation of the Area type.
 */
public String getTypeString()
{
	switch(type) {
	case CELL:
		return("CELL");
	case FROZENROW:
		return("FROZEN_ROW");
	case COLUMNLABEL:
		return("COLUMN_LABEL");
	case FROZENCOLUMN:
		return("FROZEN_COLUMN");
	case FROZENCELL:
		return("FROZEN_CELL");
	case FROZENCOLUMNLABEL:
		return("FROZEN_COLUMN_LABEL");
	case ROWLABEL:
		return("ROW_LABEL");
	case FROZENROWLABEL:
		return("FROZEN_ROW_LABEL");
	default:
		return("UNKNOWN");
	}
}

/**
 * Returns true if this Area scrolls horizontally.
 */
public boolean isHorizontalScrollable()
{
	return(type == COLUMNLABEL || type == FROZENROW || type == CELL);
}

/**
 * Returns true if this Area scrolls vertically.
 */
public boolean isVerticalScrollable()
{
	return(type == ROWLABEL || type == FROZENCOLUMN || type == CELL);
}

/**
 * Returns the start row for the range of cells contained in the area.
 */
public int getStartRow()
{
	return(start_row);
}

/**
 * Returns the start column for the range of cells contained in the area.
 */
public int getStartColumn()
{
	return(start_column);
}

/**
 * Returns the end row for the range of cells contained in the area.
 */
public int getEndRow()
{
	return(end_row);
}

/**
 * Returns the end column for the range of cells contained in the area.
 */
public int getEndColumn()
{
	return(end_column);
}

public boolean isFocusTraversable()
{
	return(false);
}

/**
 * Returns a string representation of the Area.
 */
public String toString()
{
	return("CellArea type=" + getTypeString() + ", " +
           "r" + start_row + "c" + start_column +
           ":r" + end_row + "c" + end_column +
           ", [x" + getX() + " y" + getY() +
           " w" + getWidth() + " h" + getHeight() + "]" +
           ", visible " + isVisible());
}

}
