/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CellAreaHandler.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:32 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import com.klg.jclass.cell.JCCellRenderer;
import com.klg.jclass.cell.JCComponentCellRenderer;
import com.klg.jclass.cell.JCLightCellRenderer;

import java.awt.*;
import java.util.Vector;

/**
 * <CODE>CellAreaHandler</CODE> controls the creation and removal of
 * <CODE>CellAreas</CODE> within the table.
 */
public class CellAreaHandler implements java.io.Serializable {

protected JCTable              table;

protected CellArea             cellArea;
protected CellArea             frozenRowArea;
protected CellArea             columnLabelArea;
protected CellArea             frozenColumnArea;
protected CellArea             frozenCellArea;
protected CellArea             frozenColumnLabelArea;
protected CellArea             rowLabelArea;
protected CellArea             frozenRowLabelArea;

protected Vector               areas = new Vector();

protected CellAreaRenderer     cellAreaRenderer;
protected TableCellInfoModel   cellInfo;
protected Point                cellOrigin = new Point(0,0);
protected boolean              isStandardLayout = true;

protected Rectangle            rowLabelRect = null;
protected Rectangle            columnLabelRect = null;
protected Rectangle            cellRect = null;

protected boolean              DRAGGING = false;
protected int                  drag_row = JCTableEnum.NOVALUE;
protected int                  drag_column = JCTableEnum.NOVALUE;
protected int                  drop_row = JCTableEnum.NOVALUE;
protected int                  drop_column = JCTableEnum.NOVALUE;
protected int[]                drag_columns;
protected int[]                drag_rows;

/**
 * Creates a new controller for Areas.
 */
public CellAreaHandler(JCTable table)
{
    this.table = table;
    cellAreaRenderer = new CellAreaRenderer(this);
    cellInfo = new JCTableCellInfo();
}

/**
 * Returns the AreaRenderer.
 */
public CellAreaRenderer getCellAreaRenderer()
{
    return(cellAreaRenderer);
}

/**
 * Returns the table controlled by the AreaHandler.
 */
public JCTable getTable()
{
    return(table);
}

/**
 * Creates a new CellArea of the specified type. Valid types are:
 * <UL>
 * <LI><CODE>CellArea.CELL</CODE></LI>
 * <LI><CODE>CellArea.FROZENROW</CODE></LI>
 * <LI><CODE>CellArea.COLUMNLABEL</CODE></LI>
 * <LI><CODE>CellArea.FROZENCOLUMN</CODE></LI>
 * <LI><CODE>CellArea.FROZENCELL</CODE></LI>
 * <LI><CODE>CellArea.FROZENCOLUMNLABEL</CODE></LI>
 * <LI><CODE>CellArea.ROWLABEL</CODE></LI>
 * <LI><CODE>CellArea.FROZENROWLABEL</CODE></LI>
 * </UL>
 * <P>Only one CellArea can exist of any type. If one already exists, it
 * is returned rather than creating a new one.
 */
public CellArea createCellArea(int type)
{
    switch (type) {
    case CellArea.CELL:
        if (cellArea == null) {
            cellArea = new CellArea(type);
        }
        return(cellArea);
    case CellArea.COLUMNLABEL:
        if (columnLabelArea == null) {
            columnLabelArea = new CellArea(type);
        }
        return(columnLabelArea);
    case CellArea.FROZENCELL:
        if (frozenCellArea == null) {
            frozenCellArea = new CellArea(type);
        }
        return(frozenCellArea);
    case CellArea.FROZENCOLUMN:
        if (frozenColumnArea == null) {
            frozenColumnArea = new CellArea(type);
        }
        return(frozenColumnArea);
    case CellArea.FROZENCOLUMNLABEL:
        if (frozenColumnLabelArea == null) {
            frozenColumnLabelArea = new CellArea(type);
        }
        return(frozenColumnLabelArea);
    case CellArea.FROZENROW:
        if (frozenRowArea == null) {
            frozenRowArea = new CellArea(type);
        }
        return(frozenRowArea);
    case CellArea.FROZENROWLABEL:
        if (frozenRowLabelArea == null) {
            frozenRowLabelArea = new CellArea(type);
        }
        return(frozenRowLabelArea);
    case CellArea.ROWLABEL:
        if (rowLabelArea == null) {
            rowLabelArea = new CellArea(type);
        }
        return(rowLabelArea);
    }
    return(null);
}

/**
 * Find the Area of the specified type
 */
public CellArea findByType(int type)
{
    switch (type) {
    case CellArea.CELL:
        return(cellArea);
    case CellArea.COLUMNLABEL:
        return(columnLabelArea);
    case CellArea.FROZENCELL:
        return(frozenCellArea);
    case CellArea.FROZENCOLUMN:
        return(frozenColumnArea);
    case CellArea.FROZENCOLUMNLABEL:
        return(frozenColumnLabelArea);
    case CellArea.FROZENROW:
        return(frozenRowArea);
    case CellArea.FROZENROWLABEL:
        return(frozenRowLabelArea);
    case CellArea.ROWLABEL:
        return(rowLabelArea);
    }
    return(null);
}

/**
 * Finds the Area by row/column
 */
public CellArea find(int row, int column)
{
    for (int i = 0; i < areas.size(); i++) {
        CellArea area = (CellArea)areas.elementAt(i);
        if (area.containsCell(row, column)) {
            return(area);
        }
    }
    return(null);
}

/**
 * Returns a Vector containing the areas of the table.
 */
public Vector getCellAreas()
{
    return(areas);
}

/**
 * Update the CellArea to which the type refers.  If the CellArea is needed,
 * get it reading for viewing.  Otherwise, make sure its removed from view.
 * @param type The type of the CellArea to update.
 * @param isNeeded Is the CellArea needed based on the current table paramters?
 */
protected void updateCellArea(int type, boolean isNeeded)
{
    CellArea area = null;
    if (isNeeded) {
        area = createCellArea(type);
        if (!areas.contains(area)) {
            areas.addElement(area);
        }
        area.setVisible(true);
        table.add(area);
        setAreaCellRange(area);
    }
    else {
        area = findByType(type);
        if (area != null) {
            areas.removeElement(area);
            table.remove(area);
            area.setVisible(false);
        }
    }
}

/**
 * Ensures that all required Areas exist.
 */
public void createRequiredCellAreas()
{
    int rows           = table.getNumRows();
    int columns        = table.getNumColumns();
    int frozen_rows    = table.getFrozenRows();
    int frozen_columns = table.getFrozenColumns();

    // Examine the number of frozen and non-frozen rows and columns to determine
    // which areas are required. If an area is not required, a check is made to
    // determine whether the area already exists, and if it does, it is hidden.

    // Scrolling cells
    updateCellArea(CellArea.CELL, columns > 0 && rows > 0);

    // Frozen cells
    updateCellArea(CellArea.FROZENCELL, frozen_rows > 0 && frozen_columns > 0);

    // Frozen rows
    updateCellArea(CellArea.FROZENROW, frozen_rows > 0 && rows > 0 &&
                                       columns > 0);

    // Frozen columns
    updateCellArea(CellArea.FROZENCOLUMN, frozen_columns > 0 && rows > 0 &&
                                          columns > 0);

    // Frozen row labels
    updateCellArea(CellArea.FROZENROWLABEL, frozen_rows > 0 && rows > 0 &&
                                            table.isRowLabelDisplay());

    // Scrolling row labels
    updateCellArea(CellArea.ROWLABEL, rows > 0 && table.isRowLabelDisplay());

    // Frozen column labels
    updateCellArea(CellArea.FROZENCOLUMNLABEL, frozen_columns > 0 &&
                                               columns > 0        &&
                                               table.isColumnLabelDisplay());

    // Scrolling column labels
    updateCellArea(CellArea.COLUMNLABEL, columns > 0 &&
                                         table.isColumnLabelDisplay());
}

/**
 * Sets the range of cells associated with the Area. The range of cells
 * associated with an Area is dependent on the existence of a cell in that area
 * and not on whether the area is visible or the size of the cells that the
 * area would contain.
 */
public void setAreaCellRange(CellArea area)
{
    int type = area.getType();

    int start_row    = JCTableEnum.NOVALUE;
    int start_column = JCTableEnum.NOVALUE;
    int end_row      = JCTableEnum.NOVALUE;
    int end_column   = JCTableEnum.NOVALUE;

    int rows           = table.getNumRows();
    int columns        = table.getNumColumns();
    int frozen_rows    = table.getFrozenRows();
    int frozen_columns = table.getFrozenColumns();

    switch (type) {
    case CellArea.CELL:
        if (rows > 0) {
            start_row = frozen_rows;
            end_row = rows - 1;
        }
        if (columns > 0) {
            start_column = frozen_columns;
            end_column = columns - 1;
        }
        break;
    case CellArea.FROZENCELL:
        if (frozen_rows > 0 && frozen_columns > 0) {
            start_row = 0;
            start_column = 0;
            end_row = Math.min(frozen_rows, rows) - 1;
            end_column = Math.min(frozen_columns, columns) - 1;
        }
        break;
    case CellArea.FROZENCOLUMN:
        if (rows > 0 && columns > 0 && frozen_columns > 0) {
            start_row = frozen_rows;
            start_column = 0;
            end_row = rows - 1;
            end_column = Math.min(frozen_columns, columns) - 1;
        }
        break;
    case CellArea.FROZENROW:
        if (rows > 0 && columns > 0 && frozen_rows > 0) {
            start_row = 0;
            start_column = frozen_columns;
            end_row = Math.min(frozen_rows, rows) - 1;
            end_column = columns - 1;
        }
        break;
    case CellArea.FROZENROWLABEL:
        if (frozen_rows > 0) {
            start_row = 0;
            start_column = JCTableEnum.LABEL;
            end_row = Math.min(frozen_rows, rows) - 1;
            end_column = JCTableEnum.LABEL;
        }
        break;
    case CellArea.ROWLABEL:
        if (rows > 0) {
            start_row = frozen_rows;
            start_column = JCTableEnum.LABEL;
            end_row = rows - 1;
            end_column = JCTableEnum.LABEL;
        }
        break;
    case CellArea.FROZENCOLUMNLABEL:
        if (frozen_columns > 0) {
            start_row = JCTableEnum.LABEL;
            start_column = 0;
            end_row = JCTableEnum.LABEL;
            end_column = Math.min(frozen_columns, columns) - 1;
        }
        break;
    case CellArea.COLUMNLABEL:
        if (columns > 0) {
            start_row = JCTableEnum.LABEL;
            start_column = Math.min(frozen_columns, columns - 1);
            end_row = JCTableEnum.LABEL;
            end_column = columns - 1;
        }
        break;
    }

    area.setCellRange(start_row, start_column, end_row, end_column);
    Rectangle r = getBounds(start_row, start_column, end_row, end_column);

    // This is kind of a cheap fix for the case where there are no rows and
    // columns but frozen rows and columns.  The proper way to do things
    // would be to fix table to allow a null or blank CELL area.
    // Unfortuneately, the assumption of there always being a valid CELL
    // area appears to be pretty ingrained in a lot of the code.
    // Instead, I'll just pretend there's a CELL area, but I'll never draw it.
    // Then, I'll force the width or height (or both) to zero depending
    // on whether its rows or columns (or both) that are frozen.

    if (type == CellArea.CELL) {
        if (rows > 0 && frozen_rows >= rows &&
            columns > 0 && frozen_columns >= columns)
        {
            area.setCellSize(0, 0);
        }
        else if (rows > 0 && frozen_rows >= rows) {
            area.setCellSize(r.width, 0);
        }
        else if (columns > 0 && frozen_columns >= columns) {
            area.setCellSize(0, r.height);
        }
        else {
            area.setCellSize(r.width, r.height);
        }
    }
    else if (type == CellArea.COLUMNLABEL &&
             (columns > 0 && frozen_columns >= columns))
    {
        area.setCellSize(0, r.height);
    }
    else if (type == CellArea.ROWLABEL &&
             (rows > 0 && frozen_rows >= rows))
    {
        area.setCellSize(r.width, 0);
    }
    else {
        area.setCellSize(r.width, r.height);
    }
}

/**
 * Returns the scrollbar cell origin.
 */
public Point getCellOrigin()
{
    return(cellOrigin);
}

/**
 * Sets the scrollable cell orgin.
 */
public void setCellOrigin(Point p)
{
    setCellOrigin(p.x, p.y);
}

/**
 * Sets the scrollable cell origin based on an x and y value.
 */
public void setCellOrigin(int x, int y)
{
    this.cellOrigin.x = x;
    this.cellOrigin.y = y;
    table.layoutAreas();
}

/**
 * Sets the x coordinate for the cell origin.
 */
public void setCellOriginX(int x)
{
    this.cellOrigin.x = x;
    table.layoutAreas();
}

/**
 * Sets the y coordinate for the cell origin.
 */
public void setCellOriginY(int y)
{
    this.cellOrigin.y = y;
    table.layoutAreas();
}

/**
 * Returns true if the table is in standard layout (label, frozen, cell)
 */
public boolean isStandardLayout()
{
    return(isStandardLayout);
}

/**
 * Set true if table is in standard layout (label, frozen, cell)
 */
public void setStandardLayout(boolean value)
{
    this.isStandardLayout = value;
}

/**
 * Returns the TableCellInfoModel object that acts as an intermediary between
 * CellStyleModel objects and CellEditors/CellRenderers.
 */
public TableCellInfoModel getTableCellInfo()
{
    return(cellInfo);
}

/**
 * Sets the TableCellInfoModel object that acts as an intermediary between
 * CellStyleModel objects and CellEditors/CellRenderers.
 */
public void setTableCellInfo(TableCellInfoModel cellInfo)
{
    this.cellInfo = cellInfo;
}

/**
 * Returns the row containing the given coordinates.
 */
public int getRow(int x, int y)
{
    CellArea area = contains(x, y);
    if (area == null) {
        return(JCTableEnum.NOVALUE);
    }

    // Adjust for the position of the CellArea
    x += -area.getX() + area.getStandardOffsetX();
    y += -area.getY() + area.getStandardOffsetY();

    // Adjust the coordinates by the type of area in order to determine the cell
    // that contains the given point
    if (area.isHorizontalScrollable()) {
        x -= cellOrigin.x;
    }
    if (area.isVerticalScrollable()) {
        y -= cellOrigin.y;
    }

    return(table.getCellLayout().getRow(x, y));
}

/**
 * Returns the column containing the given coordinates.
 */
public int getColumn(int x, int y)
{
    CellArea area = contains(x, y);
    if (area == null) {
        return(JCTableEnum.NOVALUE);
    }

    // Adjust for the position of the CellArea
    x += -area.getX() + area.getStandardOffsetX();
    y += -area.getY() + area.getStandardOffsetY();

    // Adjust the coordinates by the type of area in order to determine the cell
    // that contains the given point
    if (area.isHorizontalScrollable()) {
        x -= cellOrigin.x;
    }
    if (area.isVerticalScrollable()) {
        y -= cellOrigin.y;
    }

    return(table.getCellLayout().getColumn(x, y));
}

/**
 * Returns the CellArea that contains the given coordintates.
 */
protected CellArea contains(int x, int y)
{
    CellArea hitArea;
    for (int i = 0; i < areas.size(); i++) {
        hitArea = (CellArea)areas.elementAt(i);
        if (hitArea.getBounds().contains(x, y)) {
            return(hitArea);
        }
    }
    return(null);
}

/**
 * Returns the bounds of the cell accounting for the position of the CellArea
 * that manages the cell.
 */
public Rectangle getBounds(int row, int column)
{
    return(getBounds(row, column, new Rectangle()));
}

/**
 * Returns the bounds of the cell accounting for the position of the
 * CellArea the manages the cell. This method uses the given rectangle
 * rather than creating a new one.
 */
public Rectangle getBounds(int row, int column, Rectangle r)
{
    if (!DRAGGING) {
        r = table.getCellLayout().getBounds(row, column, r);
    }
    else {
        r = getDragBounds(row, column);
    }

    if (table.spanHandler != null) {
        JCCellRange cr = table.spanHandler.getSpannedRange(row, column);
        if (cr != null) {
            // Cell is spanned and is the controlling cell
            if (table.spanHandler.isControllingCell(cr, row, column)) {
                // Adjust the height of the bounding rectangle to cover all
                // spanned rows. If the number of spanned rows exceeds the
                // actual number of rows, use the actual instead.
                int actual_end = Math.min(cr.end_row, table.getNumRows() - 1);
                for (int sr = row + 1; sr <= actual_end; sr++) {
                    r.height += getBounds(sr, column).height;
                }
                // Adjust the width of the bounding rectangle to cover all
                // spanned columns. If the number of spanned columns exceeds
                // the actual number of columns, use the actual instead.
                int actual_cols = Math.min(cr.end_column,
                                           table.getNumColumns() - 1);
                for (int sc = column + 1; sc <= actual_cols; sc++) {
                    r.width += getBounds(row, sc).width;
                }
            }
        }
    }

    // Adjust for the relative position of the CellArea
    CellArea area = find(row, column);
    if (area == null) {
        return(r);
    }
    // Adjust for the position of the CellArea
    r.x += area.getX() - area.getStandardOffsetX();
    r.y += area.getY() - area.getStandardOffsetY();

    return(r);
}

/**
 * Returns the bounds of a cell range.
 */
public Rectangle getBounds(int start_row, int start_column, int end_row,
                           int end_column)
{
    if (!DRAGGING) {
        Rectangle r = new Rectangle();

        r = table.getCellLayout().getBounds(start_row, start_column,
                                            end_row, end_column, r);

        // Adjust for the relative position of the CellArea
        CellArea area = find(start_row, start_column);
        if (area == null) {
            return(r);
        }
        // Adjust for the position of the CellArea
        r.x += area.getX() - area.getStandardOffsetX();
        r.y += area.getY() - area.getStandardOffsetY();

        return(r);
    }
    else {
        Rectangle r = new Rectangle();

        r = getBounds(start_row, start_column, r);
        int x = r.x;
        int y = r.y;

        r = getBounds(end_row, end_column, r);
        r.width = r.x + r.width - x;
        r.height = r.y + r.height - y;
        r.x = x;
        r.y = y;

        return(r);
    }
}

/**
 * Returns the bounds of a cell during a drag process.
 */
public Rectangle getDragBounds(int row, int column)
{
    // Column drag
    if (drag_row == JCTableEnum.LABEL && column != JCTableEnum.LABEL) {
        if (drag_columns != null) {
            if (column == drag_columns[column]) {
                return(table.getCellLayout().getBounds(row, column));
            }
            else {
                Rectangle bounds = table.getCellLayout().getBounds(row, column);
                bounds.x = getTransposedX(column, drag_column, drop_column);
                return(bounds);
            }
        }
        else {
            return(table.getCellLayout().getBounds(row, column));
        }
    }
    // Row drag
    else if (drag_column == JCTableEnum.LABEL && row != JCTableEnum.LABEL) {
        if (drag_rows != null) {
            if (row == drag_rows[row]) {
                return(table.getCellLayout().getBounds(row, column));
            }
            else {
                Rectangle bounds = table.getCellLayout().getBounds(row, column);
                bounds.y = getTransposedY(row, drag_row, drop_row);
                return(bounds);
            }
        }
        else {
            return(table.getCellLayout().getBounds(row, column));
        }
    }
    else {
        return(table.getCellLayout().getBounds(row, column));
    }
}

/**
 * Returns true if the full content of the cell is not displayed because of
 * the size of the cell.
 */
public boolean isContentClipped(int row, int column)
{
    Object data = table.getDataView().getObject(row, column);
    JCCellRenderer renderer = table.getDataView().getCellRenderer(row, column,
                                                                  data);
    cellInfo.initialize(table, data, row, column);

    Dimension prefSize = null;
    if (renderer instanceof JCLightCellRenderer) {
        JCLightCellRenderer cr = (JCLightCellRenderer)renderer;
        prefSize = cr.getPreferredSize(table.getGraphics(),
                                       cellInfo, data);
    }
    else {
        JCComponentCellRenderer cr = (JCComponentCellRenderer)renderer;
        Component c = cr.getRendererComponent(cellInfo, data,
                                              table.isSelected(row, column));
        prefSize = c.getPreferredSize();
    }
    Rectangle bounds = getBounds(row, column);
    return(prefSize.width > bounds.width || prefSize.height > bounds.height);
}

/**
 * Returns the cell range of visible, scrollable cells.
 */
public JCCellRange getVisibleCells()
{
    CellArea area = findByType(CellArea.CELL);

    if (area == null) {
        return(new JCCellRange());
    }
    int x = area.getX();
    int y = area.getY();
    int width = area.getWidth();
    int height = area.getHeight();

    int sr = getRow(x + 1, y + 1);
    int sc = getColumn(x + 1, y + 1);
    int er = getRow(x + width - 1, y + height - 1);
    int ec = getColumn(x + width - 1, y + height - 1);

    if (er == JCTableEnum.NOVALUE) {
        er = area.getEndRow();
    }

    if (ec == JCTableEnum.NOVALUE) {
        ec = area.getEndColumn();
    }

    return(new JCCellRange(sr, sc, er, ec));
}

/**
 * Remaps rows/columns during the drag process and forces appropriate cell
 * repainting. This simulates the drag process without modifying the
 * internal state of table. When the process is complete (on mouse release),
 * the internal modifications are made through dragRow or dragColumn.
 */
protected void performDrag(int drag_row, int drag_column, int drop_row,
                           int drop_column)
{
    DRAGGING = true;

    if (this.drag_row == drag_row && this.drag_column == drag_column &&
        this.drop_row == drop_row && this.drop_column == drop_column) {
        return;
    }

    if (drag_row == JCTableEnum.LABEL) {
        if (drag_column != drop_column) {
            int columns = table.getNumColumns();
            drag_columns = new int[columns];
            for (int c = 0; c < columns; c++) {
                drag_columns[c] = c;
            }
            drag_columns = shiftArray(drag_columns, drag_column, drop_column);
        }
        else {
            drag_columns = null;
        }
        drag_rows = null;

        this.drag_row = drag_row;
        this.drag_column = drag_column;
        this.drop_row = drop_row;
        this.drop_column = drop_column;

        table.repaint();
    }
    else if (drag_column == JCTableEnum.LABEL) {
        if (drag_row != drop_row) {
            int rows = table.getNumRows();
            drag_rows = new int[rows];
            for (int r = 0; r < rows; r++) {
                drag_rows[r] = r;
            }
            drag_rows = shiftArray(drag_rows, drag_row, drop_row);
        }
        else {
            drag_rows = null;
        }
        drag_columns = null;

        this.drag_row = drag_row;
        this.drag_column = drag_column;
        this.drop_row = drop_row;
        this.drop_column = drop_column;

        table.repaint();
    }
    else {
        this.drag_row = drag_row;
        this.drag_column = drag_column;
        this.drop_row = drop_row;
        this.drop_column = drop_column;

        table.repaint();
    }
}

///**
// * Returns the new position of v relative to a drag and drop pair.
// */
//private int getTransposition(int v, int d1, int d2)
//{
//    if (d1 < d2) {
//        if (v >= d1 && v < d2 - 1) {
//            return v + 1;
//        }
//        else if (v == d2 - 1) {
//            return d1;
//        }
//    }
//    else {
//        if (v > d2 && v <= d1) {
//            return v - 1;
//        }
//        else if (v == d2) {
//            return d1;
//        }
//    }
//    return v;
//}

/**
 * Resets drag variables when the drag process is cancelled.
 */
protected void cancelDrag()
{
    DRAGGING    = false;
    drag_row    = JCTableEnum.NOVALUE;
    drag_column = JCTableEnum.NOVALUE;
    drop_row    = JCTableEnum.NOVALUE;
    drop_column = JCTableEnum.NOVALUE;
}

/**
 * Resets drag variables at the end of the drag process.
 */
protected void endDrag()
{
    DRAGGING    = false;
    drag_row    = JCTableEnum.NOVALUE;
    drag_column = JCTableEnum.NOVALUE;
    drop_row    = JCTableEnum.NOVALUE;
    drop_column = JCTableEnum.NOVALUE;
}

/**
 * Gets the y position of a row during a row drag.
 */
protected int getTransposedY(int row, int drag_row, int drop_row)
{
    CellLayoutModel clm = table.getCellLayout();
    if (drag_row < drop_row) {
        if (row < drag_row) {
            return clm.getBounds(row, JCTableEnum.LABEL).y;
        }
        else if (row == drag_row) {
            if (drop_row >= table.getNumRows()) {
                Rectangle last = clm.getBounds(table.getNumRows() - 1,
                                               JCTableEnum.LABEL);
                Rectangle bounds = clm.getBounds(row, JCTableEnum.LABEL);
                return last.y + last.height - bounds.height;
            }
            else {
                return clm.getBounds(drop_row, JCTableEnum.LABEL).y -
                       clm.getBounds(row, JCTableEnum.LABEL).height;
            }
        }
        else if (row < drop_row) {
            return clm.getBounds(row, JCTableEnum.LABEL).y -
                clm.getBounds(drag_row, JCTableEnum.LABEL).height;
        }
    }
    else if (drag_row > drop_row) {
        if (row < drop_row) {
            return clm.getBounds(row, JCTableEnum.LABEL).y;
        }
        else if (row == drag_row) {
            return clm.getBounds(drop_row, JCTableEnum.LABEL).y;
        }
        else if (row >= drop_row) {
            return clm.getBounds(row, JCTableEnum.LABEL).y +
                   clm.getBounds(drag_row, JCTableEnum.LABEL).height;
        }
    }
    return clm.getBounds(row, JCTableEnum.LABEL).y;
}

/**
 * Gets the x position of a column during a column drag.
 */
protected int getTransposedX(int column, int drag_column, int drop_column)
{
    CellLayoutModel clm = table.getCellLayout();
    if (drag_column < drop_column) {
        if (column < drag_column) {
            return clm.getBounds(JCTableEnum.LABEL, column).x;
        }
        else if (column == drag_column) {
            if (drop_column >= table.getNumColumns()) {
                Rectangle last = clm.getBounds(JCTableEnum.LABEL,
                                               table.getNumColumns() - 1);
                Rectangle bounds = clm.getBounds(JCTableEnum.LABEL, column);
                return last.x + last.width - bounds.width;
            }
            else {
                return clm.getBounds(JCTableEnum.LABEL, drop_column).x -
                       clm.getBounds(JCTableEnum.LABEL, column).width;
            }
        }
        else if (column < drop_column) {
            return clm.getBounds(JCTableEnum.LABEL, column).x -
                   clm.getBounds(JCTableEnum.LABEL, drag_column).width;
        }
    }
    else if (drag_column > drop_column) {
        if (column < drop_column) {
            return clm.getBounds(JCTableEnum.LABEL, column).x;
        }
        else if (column == drag_column) {
            return clm.getBounds(JCTableEnum.LABEL, drop_column).x;
        }
        else if (column >= drop_column) {
            return clm.getBounds(JCTableEnum.LABEL, column).x +
                   clm.getBounds(JCTableEnum.LABEL, drag_column).width;
        }
    }
    return clm.getBounds(JCTableEnum.LABEL, column).x;
}

/**
 * Repositions a row or column to a new location in the mapped array.
 */
protected int[] shiftArray(int[] array, int item1, int item2)
{
    int size = array.length;
    int[] temp = new int[size];

    // handle case where destination is the last item
    if (item2 == JCTableEnum.MAXINT) {
        for (int i = 0; i < item1; i++) {
            temp[i] = array[i];
        }
        for (int i = item1 + 1; i < size; i++) {
            temp[i - 1] = array[i];
        }
        temp[size - 1] = array[item1];
    }
    // handle case where item1 is less than item2
    else if (item1 < item2) {
        for (int i = 0; i < size; i++) {
            temp[i] = array[i];
        }
        for (int i = item1 + 1; i < item2; i++) {
            temp[i - 1] = array[i];
        }
        temp[item2 - 1] = array[item1];
    }
    // handle case where item1 is greater than item2
    else if (item1 > item2) {
        for (int i = 0; i < size; i++) {
            temp[i] = array[i];
        }
        for (int i = item2; i < item1; i++) {
            temp[i + 1] = array[i];
        }
        temp[item2] = array[item1];
    }
    return temp;
}

/**
 * Returns the scrollable cell area.
 */
public CellArea getCellArea()
{
    return(cellArea);
}

/**
 * Determines whether additional sides are needed for the thin border of
 * the specified cell.  (Thin borders traditionally draw just the top
 * and the left but there are some cases where we need to draw the bottom
 * and the right hand sides too.)
 * @param row the cell's row
 * @param column the cell's column
 * @return integer containing flags JCCellBorder.BORDERSIDE_THIN_BOTTOM
 * and JCCellBorder.BORDERSIDE_THIN_RIGHT if either of those are needed.
 */
protected int determineThinSides(int row, int column)
{
    CellArea area = table.getCellAreaHandler().find(row, column);
    if (area == null) {
        // handle potential data race condition
        return 0;
    }
    return(determineThinSides(row, column,
                              area.getEndRow(), area.getEndColumn()));
}

/**
 * Determines whether additional sides are needed for the thin border of
 * the specified cell.  (Thin borders traditionally draw just the top
 * and the left but there are some cases where we need to draw the bottom
 * and the right hand sides too.)
 * @param row the cell's row
 * @param column the cell's column
 * @param end_row the end row where we need bottom sides
 * @param end_column the end column where we need right sides
 * @return integer containing flags JCCellBorder.BORDERSIDE_THIN_BOTTOM
 * and JCCellBorder.BORDERSIDE_THIN_RIGHT if either of those are needed.
 */
protected int determineThinSides(int row, int column,
                                 int end_row, int end_column)
{
    int thinSides = 0;
    TableCellInfoModel cellInfo = getTableCellInfo();
    CellBorderModel cellBorder = cellInfo.getCellBorder();
    if (cellBorder != null) {
        if (cellBorder instanceof JCCellBorder) {
            JCCellBorder cb = (JCCellBorder)cellBorder;
            if (cb.borderType == JCTableEnum.BORDER_THIN) {

                // If cell on extreme right side, force right border
                int frCol = table.getFrozenColumns();
                int frColPl = table.getFrozenColumnPlacement();
                if (frCol <= 0 ||
                    (column >= frCol &&
                     frColPl == JCTableEnum.PLACE_LEFT) ||
                    (column < frCol &&
                     frColPl == JCTableEnum.PLACE_RIGHT))
                {
                    if (column == end_column) {
                        thinSides |= JCCellBorder.BORDERSIDE_THIN_RIGHT;
                    }
                }

                // If cell on extreme bottom side, force bottom border
                int frRow = table.getFrozenRows();
                int frRowPl = table.getFrozenRowPlacement();
                if (frRow <= 0 ||
                    (row >= frRow &&
                     frRowPl == JCTableEnum.PLACE_TOP) ||
                    (row < frRow &&
                     frRowPl == JCTableEnum.PLACE_BOTTOM))
                {
                    if (row == end_row) {
                        thinSides |= JCCellBorder.BORDERSIDE_THIN_BOTTOM;
                    }
                }
            }
        }
    }
    return(thinSides);
}

}
