/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CellAreaRenderer.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:32 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import com.klg.jclass.cell.JCCellRenderer;
import com.klg.jclass.cell.JCComponentCellRenderer;
import com.klg.jclass.cell.JCLightCellRenderer;

import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.io.Serializable;


 

/**
 * <CODE>CellAreaRenderer</CODE> controls the painting of
 * <CODE>CellAreas</CODE> within the table.
 */
public class CellAreaRenderer implements Serializable {

protected JCTable                table;
protected CellAreaHandler        cellAreaHandler;
protected Color                  borderColor;
protected Color                  borderShadowColor;
protected Color                  borderHighlightColor;
transient protected BasicStroke  dashedStroke = null;

public CellAreaRenderer(CellAreaHandler cellAreaHandler)
{
	this.cellAreaHandler = cellAreaHandler;
	this.table = cellAreaHandler.getTable();
}

/**
 * Paints the entire visible region.
 */
public void paintComponent(Graphics gc)
{
    drawFrameBorder(gc);

    Rectangle clipBounds = gc.getClipBounds();

    for (int i = 0; i < cellAreaHandler.areas.size(); i++) {
        CellArea area = (CellArea)cellAreaHandler.areas.elementAt(i);
        if (area.isVisible() && area.intersects(clipBounds)) {
            paintArea(gc, area);
        }
    }

    // If the EditWidthPolicy is preferred size and the editor is active,
    // repaint the editor here so it will be the last cell painted. This is
    // necessary because it can overlap other cells and if it is not painted
    // last subsequent cells will obscure it.
    if (table.getEditWidthPolicy() == JCTableEnum.EDIT_ENSURE_PREFERRED_SIZE) {
        Component editor = table.getEditingComponent();
        if (editor != null) {
            editor.repaint();
        }
    }


 


 


 
}


 

/**
 * Paints all visible cells in a cell area.
 */
protected void paintArea(Graphics gc, CellArea area)
{
	if (!area.isVisible()) {
		return;
	}

	// Clip the graphics object by the bounds of the CellArea
	Rectangle clip = gc.getClipBounds();
	Rectangle bounds = area.getBounds();

	// No width or height?  Then don't draw!  Otherwise, it gets messy.
    if (bounds.width <= 0 || bounds.height <= 0) {
        return;
    }

	gc.clipRect(bounds.x, bounds.y, bounds.width, bounds.height);

	// Determine the paintable rectangle
	Rectangle r = gc.getClipBounds();

	int realER = area.getEndRow();
	int realEC = area.getEndColumn();

    int sr = cellAreaHandler.getRow(r.x + 1, r.y + 1);
    int sc = cellAreaHandler.getColumn(r.x + 1, r.y + 1);
    int er = cellAreaHandler.getRow(r.x + r.width - 1, r.y + r.height - 1);
    int ec = cellAreaHandler.getColumn(r.x + r.width - 1, r.y + r.height - 1);

	if (sr == JCTableEnum.NOVALUE || sc == JCTableEnum.NOVALUE) {
		// Must reset clip before returning
		gc.setClip(clip);
		return;
	}

	if (er == JCTableEnum.NOVALUE || er == JCTableEnum.MAXINT) {
		er = realER;
	}
	if (ec == JCTableEnum.NOVALUE || ec == JCTableEnum.MAXINT) {
		ec = realEC;
	}

	// Adjust the rectangle by the type of area in order to determine the cell
	// range that contains the entire paintable rectangle
	if (area.getType() == CellArea.CELL) {
		gc.translate(cellAreaHandler.cellOrigin.x,
		             cellAreaHandler.cellOrigin.y);
	}
	else if (area.getType() == CellArea.FROZENCOLUMN ||
	         area.getType() == CellArea.ROWLABEL)
	{
		gc.translate(0, cellAreaHandler.cellOrigin.y);
	}
	else if (area.getType() == CellArea.FROZENROW ||
	         area.getType() == CellArea.COLUMNLABEL)
	{
		gc.translate(cellAreaHandler.cellOrigin.x, 0);
	}

	paintRange(gc, sr, sc, er, ec);

	// Make sure the drag row is repainted (since it may originate from a
    // non-visible area of the table
	if (cellAreaHandler.DRAGGING) {
		if (cellAreaHandler.drag_row != JCTableEnum.NOVALUE &&
		    cellAreaHandler.drag_row != JCTableEnum.LABEL   &&
		    (cellAreaHandler.drag_row < sr || cellAreaHandler.drag_row > er))
		{
			paintRange(gc, cellAreaHandler.drag_row, sc,
			           cellAreaHandler.drag_row, ec);
		}
		else if (cellAreaHandler.drag_column != JCTableEnum.NOVALUE &&
				 cellAreaHandler.drag_column != JCTableEnum.LABEL   &&
				 (cellAreaHandler.drag_column < sc ||
				  cellAreaHandler.drag_column > ec))
		{
			paintRange(gc, sr, cellAreaHandler.drag_column, er,
			           cellAreaHandler.drag_column);
		}
	}

	if (area.getType() == CellArea.CELL) {
		gc.translate(-cellAreaHandler.cellOrigin.x,
		             -cellAreaHandler.cellOrigin.y);
	}
	else if (area.getType() == CellArea.FROZENCOLUMN ||
	         area.getType() == CellArea.ROWLABEL)
	{
		gc.translate(0, -cellAreaHandler.cellOrigin.y);
	}
	else if (area.getType() == CellArea.FROZENROW ||
	         area.getType() == CellArea.COLUMNLABEL)
	{
		gc.translate(-cellAreaHandler.cellOrigin.x, 0);
	}

	gc.setClip(clip);
}


/**
 * Paints all cells in the range that are in the visible region.
 */
protected void paintRange(Graphics gc, int start_row, int start_column,
						  int end_row, int end_column)
{
	if (gc == null) {
		return;
	}

	if (table.paintListeners != null) {
		table.fireJCPaintEvent(new JCPaintEvent(table,
                                                JCPaintEvent.BEFORE_PAINT,
												start_row, start_column,
												end_row, end_column));
	}

	Rectangle rect = new Rectangle();
	for (int r = start_row; r <= end_row; r++) {
		for (int c = start_column; c <= end_column; c++) {
			paintCell(gc, r, c, false, rect);
		}
	}

	if (table.paintListeners != null) {
		table.fireJCPaintEvent(new JCPaintEvent(table, JCPaintEvent.AFTER_PAINT,
												start_row, start_column,
												end_row, end_column));
	}
}

/**
 * Paints the cell if it is in the visible region and notifies any JCPaintListeners.
 */
protected void paintCell(Graphics gc, int row, int column)
{
	paintCell(gc, row, column, true, new Rectangle());
}

/**
 * Paints the cell if it is in the visible region.
 */
protected void paintCell(Graphics gc, int row, int column, boolean notify)
{
	paintCell(gc, row, column, notify, new Rectangle());
}

/**
 * Paints the cell if it is in the visible region re-using the given
 * rectangle for cell bounds manipulation.
 */
protected void paintCell(Graphics gc, int row, int column,
                         boolean notify, Rectangle r)
{
	// Not visible? return
	if (table.isRowHidden(row) || table.isColumnHidden(column)) {
		return;
	}

	Rectangle clip = gc.getClipBounds();

	r = cellAreaHandler.getBounds(row, column, r);
	if (r == null) {
		return;
	}

	// Save these values for drawing the focus rect below
	int fx = r.x;
	int fy = r.y;
	int fw = r.width;
	int fh = r.height;

	// If the following,
	// 		a) a span exists
	// 		b) this cell is covered by the span
	//		c) this cell is not the control cell (upper-left cell in span)
	// don't paint the cell because the control cell will
	// be painted into its area. Return.
	JCCellRange spannedRange = null;
	if (table.spanHandler != null) {
		spannedRange = table.spanHandler.getSpannedRange(row, column);
		if (spannedRange != null &&
		    !table.spanHandler.isControllingCell(spannedRange, row, column))
		{
			// Force the controlling cell to be repainted if it isn't exposed
			Rectangle b =
			    table.getCellLayout().getBounds(spannedRange.start_row,
			                                    spannedRange.start_column);
			CellArea area = cellAreaHandler.find(spannedRange.start_row,
			                                     spannedRange.start_column);
			if (area != null) {
				b.x += area.getX() - area.getStandardOffsetX();
				b.y += area.getY() - area.getStandardOffsetY();
				if(b.x < clip.x || b.y < clip.y) {
					paintCell(gc, spannedRange.start_row,
					          spannedRange.start_column);
				}
			}
			return;
		}
	}

    // We are now committed to displaying this cell, so retrieve its data.
	Object data = table.getDataView().getObject(row, column);

    // Notify listeners interested in data retrieval events.
	if (table.cellDisplayListeners != null) {
		JCCellDisplayEvent ev = new JCCellDisplayEvent(table, row,
		                                               column, data);
		table.fireJCCellDisplayEvent(ev);
		data = ev.getDisplayData();
	}

	if (notify) {
		table.fireJCPaintEvent(new JCPaintEvent(table,
								                JCPaintEvent.BEFORE_PAINT,
												row, column, row, column));
	}

	TableCellInfoModel cellInfo = cellAreaHandler.getTableCellInfo();
	cellInfo.initialize(table, data, row, column);

	JCCellRenderer renderer = table.getDataView().getCellRenderer(row, column,
                                                                  data);
	boolean selected = table.isSelected(row, column);

	int focusIndicator = table.getFocusIndicator();

	// Special case -- thin border in JCCellBorder
	// needs to know cases where it needs to draw more than just
	// the top and left hand side of the border (edge cases)
	int thinSides = 0;
    if (spannedRange == null) {
        thinSides = cellAreaHandler.determineThinSides(row, column);
    }
	else {
        thinSides = cellAreaHandler.determineThinSides(spannedRange.end_row,
												       spannedRange.end_column);
    }

	// Draw the background
	if (table.isCurrentCell(row, column)              &&
	    focusIndicator == JCTableEnum.FOCUS_HIGHLIGHT &&
		table.getFocusColor() != null)
	{
		gc.setColor(table.getFocusColor());
	}
	else if (selected) {
		gc.setColor(cellInfo.getSelectedBackground());
	}
	else {
		gc.setColor(cellInfo.getBackground());
	}
	gc.fillRect(r.x, r.y, r.width, r.height);

	CellBorderModel cellBorder = cellInfo.getCellBorder();
	if (cellBorder != null) {
		int mode = cellInfo.getCellStyle().getCellBorderColorMode();
		Color plainColor = cellInfo.getCellStyle().getCellBorderColor();
		if ((plainColor == null && mode == JCTableEnum.USE_CELL_BORDER_COLOR) ||
		    mode == JCTableEnum.BASE_ON_BACKGROUND)
		{
			if (selected) {
				plainColor = cellInfo.getSelectedBackground();
			}
			else {
				plainColor = cellInfo.getBackground();
			}
		}
		else if (mode == JCTableEnum.BASE_ON_FOREGROUND) {
			if (selected) {
				plainColor = cellInfo.getSelectedForeground();
			}
			else {
				plainColor = cellInfo.getForeground();
			}
		}
		if (plainColor != borderColor) {
			if (plainColor.equals(Color.white)) {
				borderHighlightColor = TableCellInfoModel.WHITE_BRIGHTER;
				borderShadowColor = TableCellInfoModel.WHITE_DARKER;
			}
			else if (plainColor.equals(Color.black)) {
				borderHighlightColor = TableCellInfoModel.BLACK_BRIGHTER;
				borderShadowColor = TableCellInfoModel.BLACK_DARKER;
			}
			else {
				borderHighlightColor = plainColor.brighter();
				borderShadowColor = plainColor.darker();
			}
			borderColor = plainColor;
		}

		cellBorder.drawBackground(gc, table.getCellBorderWidth(),
				                  cellInfo.getCellBorderSides() | thinSides,
				                  r.x, r.y, r.width, r.height,
				                  borderHighlightColor, borderShadowColor,
								  borderColor);
	}

	Component component = table.getComponent(row, column);

	if (component != null) {
		CellArea area = table.getCellAreaHandler().find(row, column);
		if (area != component.getParent()) {
			area.add(component);
		}
		if (!component.isVisible()) {
			component.setVisible(true);
		}
		// Difference between location and translated results
		// for cell (0,0) yield valid offsets.  Note that this
		// is really reverse-engineering what is coded in
		// TableLayout.

		Rectangle rorig = new Rectangle();
        int startRow;
        int startColumn;

		// If dragging replaces start row/column with dragged row/column,
		// must calculate from that row/column instead
        if (cellAreaHandler.DRAGGING && cellAreaHandler.drag_rows != null) {
            startRow = cellAreaHandler.drag_rows[area.getStartRow()];
        }
        else {
            startRow = area.getStartRow();
        }

        if (cellAreaHandler.DRAGGING && cellAreaHandler.drag_columns != null) {
            startColumn = cellAreaHandler.drag_columns[area.getStartColumn()];
        }
        else {
            startColumn = area.getStartColumn();
        }

        rorig = cellAreaHandler.getBounds(startRow, startColumn, rorig);

		// Adjust the coordinates by the type of area
		int x = r.x;
		int y = r.y;
		if (area.isHorizontalScrollable()) {
			x -= area.getStandardOffsetX() - cellAreaHandler.cellOrigin.x;
			rorig.x -= area.getStandardOffsetX();
		}
		if (area.isVerticalScrollable()) {
			y -= area.getStandardOffsetY() - cellAreaHandler.cellOrigin.y;
			rorig.y -= area.getStandardOffsetY();
		}
		// Calculate offsets that account for frame border
		// width and row and column label offsets, among
		// other things
		int xoffset = rorig.x;
		int yoffset = rorig.y;

		int bw = table.getComponentBorderWidth();
		component.setBounds(x + bw - xoffset,
							y + bw - yoffset,
							r.width - 2 * bw, r.height - 2 * bw);
		component.validate();

		forcePaintComponent(component, gc, r.x, r.y, r.width, r.height);
	}
	else {

		// Draw the foreground
		Rectangle da = cellInfo.getDrawingArea();

		r.x += da.x;
		r.y += da.y;
		r.width = da.width;
		r.height = da.height;

		// Clip the rectangle
		gc.clipRect(r.x, r.y, r.width, r.height);

		if (renderer instanceof JCLightCellRenderer) {
			gc.translate(r.x, r.y);
			// Try and catch errors in custom renderers
			try {
				((JCLightCellRenderer)renderer).draw(gc, cellInfo, data, selected);
			}
			catch (Exception e) {
				// Swallow all errors unconditionally.
			}
			gc.translate(-r.x, -r.y);
		}
		else if (renderer instanceof JCComponentCellRenderer) {
			JCComponentCellRenderer cr = (JCComponentCellRenderer)renderer;
			Component comp = cr.getRendererComponent(cellInfo, data, selected);
			// Try and catch errors in custom renderers
			try {
				table.cellRendererPane.paintComponent(gc, comp, table,
													  r.x, r.y, r.width,
													  r.height, true);
			}
			catch (Exception e) {
				// Swallow all errors unconditionally.
			}
		}

		if (notify) {
			table.fireJCPaintEvent(new JCPaintEvent(table,
			                                        JCPaintEvent.AFTER_PAINT,
													row, column, row, column));
		}
		gc.setClip(clip);

	}

	// special case -- thin border, need to expand the focus rect
	// in certain cases
	if ((thinSides & JCCellBorder.BORDERSIDE_THIN_RIGHT) == 0) {
		fw++;
	}
	if ((thinSides & JCCellBorder.BORDERSIDE_THIN_BOTTOM) == 0) {
		fh++;
	}
	// Draw the focus rectangle
	if (table.isCurrentCell(row, column)) {
		if (focusIndicator == JCTableEnum.FOCUS_RECTANGLE) {
			drawTable3FocusRectangle(gc, fx, fy, fw, fh);
		}
		else if (focusIndicator == JCTableEnum.FOCUS_THIN_RECTANGLE) {
			drawThinFocusRectangle(gc, fx, fy, fw, fh);
		}
		else if (focusIndicator == JCTableEnum.FOCUS_DASHED_RECTANGLE) {
			drawDashedFocusRectangle(gc, fx, fy, fw, fh);
		}
	}
}

/**
 * Allows subclasses to paint components in cells. This is not required for
 * normal table painting, but is required by the CellAreaRenderer subclasss
 * used for printing in order to position components properly on the page.
 */
protected void forcePaintComponent(Component component, Graphics gc,
                                   int x, int y, int width, int height)
{
	// the normal repaint mechanism will handle this --- JCPrintTable overrides this to
	// force repaint of the component
}

/**
 * Draws a border around the cell and label regions.
 */
public void drawFrameBorder(Graphics gc)
{
	Color bg = table.getBackground();

	CellBorderModel cellBorder = table.getFrameBorder();
	if (cellBorder == null) {
		return;
	}
	int frame_width = table.getFrameBorderWidth();
	int dfw         = 2 * frame_width;

	// Row label area
	if (cellAreaHandler.rowLabelRect != null) {
		cellBorder.drawBackground(gc, frame_width, JCTableEnum.BORDERSIDE_ALL,
								  cellAreaHandler.rowLabelRect.x - frame_width,
								  cellAreaHandler.rowLabelRect.y - frame_width,
								  cellAreaHandler.rowLabelRect.width + dfw,
								  cellAreaHandler.rowLabelRect.height + dfw,
								  bg.brighter(), bg.darker(), bg);
	}

	// Column label area
	if (cellAreaHandler.columnLabelRect != null) {
		cellBorder.drawBackground(gc, frame_width, JCTableEnum.BORDERSIDE_ALL,
								  cellAreaHandler.columnLabelRect.x - frame_width,
								  cellAreaHandler.columnLabelRect.y - frame_width,
								  cellAreaHandler.columnLabelRect.width + dfw,
								  cellAreaHandler.columnLabelRect.height + dfw,
								  bg.brighter(), bg.darker(), bg);
	}

	// Cell area
	if (cellAreaHandler.cellRect != null) {
		cellBorder.drawBackground(gc, frame_width, JCTableEnum.BORDERSIDE_ALL,
								  cellAreaHandler.cellRect.x - frame_width,
								  cellAreaHandler.cellRect.y - frame_width,
								  cellAreaHandler.cellRect.width + dfw,
								  cellAreaHandler.cellRect.height + dfw,
								  bg.brighter(), bg.darker(), bg);
	}
}

/**
 * Draws the Table3-style focus rectangle.
 */
public void drawTable3FocusRectangle(Graphics gc, int x, int y,
                                     int width, int height)
{
	Color focusColor = table.getFocusColor();
	if (focusColor == null) {
		focusColor = cellAreaHandler.getTableCellInfo().getForeground().darker();
	}
	gc.setColor(focusColor);
	gc.drawRect(x + 1, y + 1, width - 3, height - 3);
	gc.drawRect(x + 2, y + 2, width - 5, height - 5);
}

/**
 * Draws a thin focus rectangle.
 */
public void drawThinFocusRectangle(Graphics gc, int x, int y,
                                   int width, int height)
{
	Color focusColor = table.getFocusColor();
	if(focusColor == null) {
		focusColor = cellAreaHandler.getTableCellInfo().getForeground().darker();
	}
	gc.setColor(focusColor);
	gc.drawRect(x + 1, y + 1, width - 3, height - 3);
}

/**
 * Draws a dashed focus rectangle.
 */
public void drawDashedFocusRectangle(Graphics gc, int x, int y,
                                     int width, int height)
{




	Color focusColor = table.getFocusColor();
	if(focusColor == null) {
		focusColor = cellAreaHandler.getTableCellInfo().getForeground().darker();
	}

	Graphics2D g2 = (Graphics2D)gc;
	g2.setColor(focusColor);

	Stroke stroke = g2.getStroke();

    if (dashedStroke == null) {
        float dash[] = { 1.0f, 3.0f };
        dashedStroke = new BasicStroke(1.0f, BasicStroke.CAP_BUTT,
		                               BasicStroke.JOIN_MITER, 1.0f,
									   dash, 0.0f);
	}
	g2.setStroke(dashedStroke);
	Rectangle2D rect = new Rectangle2D.Float(x + 1, y + 1,
	                                         width - 3, height - 3);
	g2.draw(rect);

	g2.setStroke(stroke);

}

/**
 * Draws a border around selected regions.
 */
public void drawSelectionBorder(Graphics gc) {
	/*
	Vector selections = table.getSelectedCells();
	if(selections != null && selections.size() > 0) {

		// setup 2d line drawing
		Graphics2D g2 = (Graphics2D)gc;
		g2.setColor(Color.black);
		Stroke stroke = g2.getStroke();

        if (dashedStroke == null) {
            float dash[] = { 1.0f, 3.0f };
            dashedStroke = new BasicStroke(1.0f, BasicStroke.CAP_BUTT,
			                               BasicStroke.JOIN_MITER,
										   1.0f, dash, 0.0f);
        }
		g2.setStroke(dashedStroke);

		for(int i = 0; i < selections.size(); i++) {
			JCCellRange cr = (JCCellRange)selections.elementAt(i);
			int sr = Math.min(cr.start_row, cr.end_row);
			if(sr == JCTableEnum.LABEL) {
				sr = 0;
			}
			int sc = Math.min(cr.start_column, cr.end_column);
			if(sc == JCTableEnum.LABEL) {
				sc = 0;
			}
			int er = Math.max(cr.start_row, cr.end_row);
			if(er == JCTableEnum.MAXINT) {
				er = table.getNumRows() - 1;
			}
			int ec = Math.max(cr.start_column, cr.end_column);
			if(ec == JCTableEnum.MAXINT) {
				ec = table.getNumColumns() - 1;
			}
			Rectangle r = cellAreaHandler.getBounds(sr, sc, er, ec);
			g2.draw(new Rectangle2D.Float(r.x, r.y, r.width - 1, r.height - 1));
		}

		g2.setStroke(stroke);
	}
	*/
}

/**
 * Notifies table that the given cell range requires repaint.
 */
public void repaintRange(JCCellRange cr)
{
	repaintRange(cr.start_row, cr.start_column, cr.end_row, cr.end_column);
}

/**
 * Notifies table that the given cell range requires repaint.
 */
public void repaintRange(int start_row, int start_column,
                         int end_row, int end_column)
{
	if (end_row == JCTableEnum.MAXINT) {
		end_row = table.getNumRows() - 1;
	}
	if (end_column == JCTableEnum.MAXINT) {
		end_column = table.getNumColumns() - 1;
	}

	// adjust the range ensure that all spanned cells are included in the range
	if (table.spanHandler != null && table.resize_interactive) {
		JCCellRange cr = table.spanHandler.getCompleteRange(start_row,
		                                                    start_column,
                                                            end_row,
															end_column);
		start_row    = cr.start_row;
		start_column = cr.start_column;
		end_row      = cr.end_row;
		end_column   = cr.end_column;
	}

	Rectangle startRect, endRect;
	int sr, sc, er, ec;
	int x1, x2, y1, y2;

	// go through each cell area and determine if the requested cell range falls in
	// the area
	for (int a = 0; a < cellAreaHandler.areas.size(); a++) {
		CellArea area = (CellArea)cellAreaHandler.areas.elementAt(a);
		// If the range intersects, determine the intersecting cells and
		// invoke repaint
		if (start_row <= area.end_row       &&
		    end_row >= area.start_row       &&
		    start_column <= area.end_column &&
			end_column >= area.start_column)
		{
			sr = Math.max(area.start_row, start_row);
			sc = Math.max(area.start_column, start_column);
			er = Math.min(area.end_row, end_row);
			ec = Math.min(area.end_column, end_column);

			startRect = cellAreaHandler.getBounds(sr, sc);
			if(startRect == null) {
				startRect = new Rectangle(0, 0, 0, 0);
			}
			endRect = cellAreaHandler.getBounds(er, ec);

			// determine the union of the two rectangles
			x1 = Math.min(startRect.x, endRect.x);
			y1 = Math.min(startRect.y, endRect.y);
			x2 = Math.max(startRect.x + startRect.width,
			              endRect.x + endRect.width) - x1;
			y2 = Math.max(startRect.y + startRect.height,
			              endRect.y + endRect.height) - y1;

			// Adjust the coordinates by the type of area in order to determine
			// the cell that contains the given point
			if (area.isHorizontalScrollable()) {
				x1 += cellAreaHandler.cellOrigin.x;
			}
			if (area.isVerticalScrollable()) {
				y1 += cellAreaHandler.cellOrigin.y;
			}

			table.repaint(x1, y1, x2, y2);
		}
	}
}

/**
 * Notifies table that the given cell requires repaint.
 */
public void repaintCell(int row, int column)
{
	repaintCell(row, column, false);
}

/**
 * Notifies table that the given cell requires repaint.
 */
public void repaintCell(int row, int column, boolean useStore)
{
	// Check if the cell is part of a span
	if (table.spanHandler != null) {
		JCCellRange cr = table.spanHandler.getSpannedRange(row, column);
		if (cr != null) {
			repaintRange(cr);
			return;
		}
	}

	Rectangle r = cellAreaHandler.getBounds(row, column);

	// Adjust the coordinates by the type of area in order to determine the cell
	// that contains the given point
	CellArea area = cellAreaHandler.find(row, column);
	if (area == null) {
		// CellAreaHandler.find() may return null if one sets the datasource
		// and row/column sizes are out of sync.
		return;
	}
	if (area.isHorizontalScrollable()) {
		r.x += cellAreaHandler.cellOrigin.x;
	}
	if (area.isVerticalScrollable()) {
		r.y += cellAreaHandler.cellOrigin.y;
	}

	if (useStore && table.store_image_enabled && table.store_image != null) {
		Graphics gc = table.store_image.getGraphics();
		int dx = 0;
		int dy = 0;
		// Adjust the rectangle by the type of area in order to determine the
		// cell range that contains the entire paintable rectangle
		if (area.getType() == CellArea.CELL) {
			dx = table.cellAreaHandler.cellOrigin.x;
			dy = table.cellAreaHandler.cellOrigin.y;
		}
		else if (area.getType() == CellArea.FROZENCOLUMN ||
		         area.getType() == CellArea.ROWLABEL)
		{
			dy = table.cellAreaHandler.cellOrigin.y;
		}
		else if(area.getType() == CellArea.FROZENROW ||
		        area.getType() == CellArea.COLUMNLABEL)
		{
			dx = table.cellAreaHandler.cellOrigin.x;
		}

		gc.translate(dx, dy);
		paintCell(gc, row, column, false);
		gc.translate(-dx, -dy);
	}

	table.repaint(r.x, r.y, r.width, r.height);
}

}
