/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CellLayoutModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:32 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import java.awt.Rectangle;

/**
 * Defines the requirements for an object responsible for cell dimensions.
 */
public interface CellLayoutModel {
	
/**
 * Returns the JCTable.
 */
public JCTable getTable();
	
/**
 * Sets the JCTable.
 */
public void setTable(JCTable table);

/**
 * Returns the bounds of the given cell or label.
 */
public Rectangle getBounds(int row, int column);

/**
 * Gets the bounds of the given cell or label using the given Rectangle.
 */
public Rectangle getBounds(int row, int column, Rectangle r);

/**
 * Returns the height of a row.
 */
public int getHeight(int row);

/**
 * Sets the height of a row.
 */
public void setHeight(int row, int value);

/**
 * Sets the height of a sequence of rows.
 */
public void setHeight(int start_row, int end_row, int value);

/**
 * Returns the width of a column.
 */
public int getWidth(int column);

/**
 * Sets the width of a column.
 */
public void setWidth(int column, int value);

/**
 * Sets the height of a sequence of columns.
 */
public void setWidth(int start_column, int end_column, int value);

/**
 * Determines the column that contains the given (x,y) coordinate.
 */
public int getColumn(int x, int y);

/**
 * Determines the row that contains the given (x,y) coordinate.
 */
public int getRow(int x, int y);

/**
 * Returns the start position of a column.
 */
public int getColumnPosition(int column);

/**
 * Returns the start position of a row.
 */
public int getRowPosition(int row);

/**
 * Gets the bounds of a cell range using the given Rectangle.
 */
public Rectangle getBounds(int start_row, int start_column, int end_row, 
					       int end_column, Rectangle r);

/**
 * Returns the bounds of a cell range.
 */
public Rectangle getBounds(int start_row, int start_column, int end_row,
                           int end_column);

/**
 * Moves a set of rows to a position above the destination. To move
 * rows to the end, use <CODE>JCTableEnum.MAXINT</CODE>.
 * @param start_row the first row
 * @param num_rows the number of rows to move
 * @param destination the row before which the moved rows are inserted
 */
public void moveRows(int start_row, int num_rows, int destination);
	
/**
 * Moves a set of columns to a position left of the destination. To move
 * columns to the end, use <CODE>JCTableEnum.MAXINT</CODE>.
 * @param start_column the first column
 * @param num_columns the number of columns to move
 * @param destination the column before which the moved columns are inserted
 */
public void moveColumns(int start_column, int num_columns, int destination);

/**
 * Inserts the given number of rows.
 * @param start_row the first row
 * @param num_rows the number of rows to insert
 */
public void addRows(int start_row, int num_rows);

/**
 * Inserts the given number of columns.
 * @param start_column the first column
 * @param num_columns the number of columns to insert
 */
public void addColumns(int start_column, int num_columns);

/**
 * Deletes the given number of rows.
 * @param start_row the first row
 * @param num_rows the number of rows to delete
 */
public void deleteRows(int start_row, int num_rows);

/**
 * Deletes the given number of columns.
 * @param start_column the first column
 * @param num_columns the number of columns to delete
 */
public void deleteColumns(int start_column, int num_columns);
	
/**
 * Remaps heights after the row map changes.
 */
public void remapRows(int current_map[], int new_map[]);

/**
 * Remaps widths after the column map changes.
 */
public void remapColumns(int current_map[], int new_map[]);

/**
 * Remove cached height information.
 */
public void resetRows();

/**
 * Remove cached width information.
 */
public void resetColumns();
}



