/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CellRangeFlavor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:32 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import java.awt.datatransfer.*;
import java.util.Vector;

/**
 * <CODE>CellRangeFlavor</CODE> is a <CODE>Transferable</CODE>
 * object that passes cell data through a Vector of cells.
 */
public class CellRangeFlavor implements Transferable, ClipboardOwner {

protected JCTable        table;
protected JCCellRange    cr;
protected DataFlavor     flavor;
protected Vector         cells;

/**
 * Create a new CellRangeFlavor given a range of cells from table and
 * creates a copy of the data in vector format.
 */
public CellRangeFlavor(JCTable table, JCCellRange cr)
{
    this.table = table;
    this.cr = cr;
    this.flavor = new DataFlavor(JCCellRange.class, "CellRange");

    cells = new Vector();
    if (cr.end_row == JCTableEnum.MAXINT) {
        cr.end_row = table.getNumRows() - 1;
    }
    if (cr.end_column == JCTableEnum.MAXINT) {
        cr.end_column = table.getNumColumns() - 1;
    }

    DataViewModel dvm = table.getDataView();
    for (int r = cr.start_row; r <= cr.end_row; r++) {
        Vector row = new Vector();
        for (int c = cr.start_column; c <= cr.end_column; c++) {
            row.addElement(dvm.getObject(r, c));
        }
        cells.addElement(row);
    }
}

/**
 * Returns the CellRange data flavor.
 */
public DataFlavor[] getTransferDataFlavors()
{
    return(new DataFlavor[]{flavor});
}

/**
 * Returns true if the DataFlavor matche the CellRange flavor.
 */
public boolean isDataFlavorSupported(DataFlavor f)
{
    return(f.equals(flavor));
}

/**
 * Returns the vector of cells in the given DataFlavor format.
 */
public Object getTransferData(DataFlavor f) throws UnsupportedFlavorException
{
    if (f.equals(flavor))
        return(cells);
    else {
        throw new UnsupportedFlavorException(f);
    }
}

/**
 * Destroys the copy of cells when notified that this object is no longer
 * the owner of the contents of the clipboard.
 */
public void lostOwnership(Clipboard clipboard, Transferable contents)
{
    cells = null;
}

}
