/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CellStyleModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:33 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import com.klg.jclass.cell.JCCellEditor;
import com.klg.jclass.cell.JCCellRenderer;

import java.awt.*;
import java.beans.PropertyChangeListener;

/**
 * The <CODE>CellStyleModel</CODE> class provides <CODE>CellRenderer</CODE>
 * and <CODE>CellEditor</CODE> objects formatting information that can
 * be used in drawing.
 */
public interface CellStyleModel extends Cloneable {

/** Alignment value */
public static final int LEFT = JCTableEnum.LEFT;
/** Alignment value */
public static final int CENTER = JCTableEnum.CENTER;
/** Alignment value */
public static final int RIGHT = JCTableEnum.RIGHT;
/** Alignment value */
public static final int TOP = JCTableEnum.TOP;
/** Alignment value */
public static final int BOTTOM = JCTableEnum.BOTTOM;

/** clipHints value */
public static final int SHOW_NONE = 0;
/** clipHints value */
public static final int SHOW_HORIZONTAL = 1;
/** clipHints value */
public static final int SHOW_VERTICAL = 2;
/** clipHints value */
public static final int SHOW_ALL = 3;

/**
 * Returns the parent style. If there is no parent, use null.
 */
public CellStyleModel getParentStyle();

/**
 * Sets the parent style.
 */
public void setParentStyle(CellStyleModel value);

/**
 * Returns the background color of the cell.
 */
public Color getBackground();

/**
 * Sets the background color of the cell.
 */
public void setBackground(Color value);

/**
 * Returns the CellBorder object for the cell.
 */
public CellBorderModel getCellBorder();

/**
 * Sets the CellBorder object for the cell.
 */
public void setCellBorder(CellBorderModel value);

/**
 * Returns the CellBorder color for the cell.
 */
public Color getCellBorderColor();

/**
 * Sets the CellBorder color for the cell.
 */
public void setCellBorderColor(Color value);

/**
 * Returns the mode used to determine how the cell border color is chosen.
 */
public int getCellBorderColorMode();

/**
 * Sets the mode used to determine cell border colors.
 * <PRE><UL>
 *<LI>JCTableEnum.USE_CELL_BORDER_COLOR  use the CellBorderColor
 *<LI>JCTableEnum.BASE_ON_BACKGROUND     determine color by the cell background
 *<LI>JCTableEnum.BASE_ON_FOREGROUND     determine color by the cell foreground
 *</UL></PRE>	 
 */
public void setCellBorderColorMode(int value);

/**
 * Returns the border sides value for the cell.
 */
public int getCellBorderSides();

/**
 * Sets the border sides for the cell. Valid values are
 * <CODE>BORDERSIDE_NONE</CODE>, <CODE>BORDERSIDE_ALL</CODE> and combinations of
 * <CODE>BORDERSIDE_LEFT</CODE>,  <CODE>BORDERSIDE_RIGHT</CODE>,
 * <CODE>BORDERSIDE_TOP</CODE> and <CODE>BORDERSIDE_BOTTOM</CODE>.
 */
public void setCellBorderSides(int value);

/**
 * Gets the cell's editor. If null, table maps the editor based on the
 * type of data.
 */
public JCCellEditor getCellEditor();

/**
 * Sets the editor for a cell. If null, table maps the editor based on
 * the type of data.
 */
public void setCellEditor(JCCellEditor editor);

/**
 * Gets the cell's renderer. If null, table maps the renderer based on the
 * type of data.
 */
public JCCellRenderer getCellRenderer();

/**
 * Sets the renderer for a cell. If null, table maps the renderer based on
 * the type of data.
 */
public void setCellRenderer(JCCellRenderer renderer);

/**
 * Determines whether the renderer should draw some indication if the
 * entire contents of the cell cannot be rendered within the given area.
 */
public int getClipHints();

/**
 * Sets whether the renderer should draw some indication if the entire
 * contents of the cell cannot be rendered within the given area.
 * Valid values are <CODE>SHOW_NONE</CODE>, <CODE>SHOW_HORIZONTAL</CODE>,
 * <CODE>SHOW_VERTICAL</CODE>, and <CODE>SHOW_ALL</CODE>
 */
public void setClipHints(int value);

/**
 * Returns the cell's associated data type.
 * @see #setDataType 
 */
public Class getDataType();

/**
 * Sets the cell's associated data type. The data type is used to
 * determine the renderer/editor in the case where the data source
 * returns a null value.
 */
public void setDataType(Class value);

/**
 * Returns true if the cell is editable.
 */
public boolean isEditable();

/**
 * Sets whether the cell is editable.
 */
public void setEditable(boolean value);

/**
 * Returns the font of the cell.
 */
public Font getFont();

/**
 * Sets the font used for drawing text within the cell.
 */
public void setFont(Font value);

/**
 * Returns the foreground color of the cell.  
 */
public Color getForeground();

/**
 * Sets the foreground color of the cell.
 */
public void setForeground(Color value);

/**
 * Returns the horizontal alignment for the cell.
 * Valid return values are <CODE>LEFT</CODE>, <CODE>CENTER</CODE>, 
 * <CODE>RIGHT</CODE>. 
 */
public int getHorizontalAlignment();

/**
 * Sets the horizontal alignment for the cell.
 */
public void setHorizontalAlignment(int value);

/**
 * Returns the background repeat flag.
 */
public int getRepeatBackground();

/**
 * Sets whether the background color uses the
 * <CODE>BackgroundRepeatColors</CODE> for rows or columns. Valid values are
 * <CODE>JCTableEnum.REPEAT_NONE</CODE>, <CODE>JCTableEnum.REPEAT_ROW</CODE> or
 * <CODE>JCTableEnum.REPEAT_COLUMN</CODE>.
 */
public void setRepeatBackground(int value);

/**
 * Gets the table's <CODE>RepeatBackgroundColors</CODE> value.
 * @see #setRepeatBackgroundColors
 */
public Color[] getRepeatBackgroundColors();

/**
 * Sets an array of colors to be used when <CODE>Background</CODE> value is
 * <CODE>JCTableEnum.REPEAT_ROW</CODE> or
 * <CODE>JCTableEnum.REPEAT_COLUMN</CODE>.
 */
public void setRepeatBackgroundColors(Color[] values);

/**
 * Returns the foreground repeat flag.
 */
public int getRepeatForeground();

/**
 * Sets whether the foreground color uses the
 * <CODE>ForegroundRepeatColors</CODE> for rows or columns. Valid values are
 * <CODE>JCTableEnum.REPEAT_NONE</CODE>, <CODE>JCTableEnum.REPEAT_ROW</CODE> or
 * <CODE>JCTableEnum.REPEAT_COLUMN</CODE>.
 */
public void setRepeatForeground(int value);

/**
 * Gets the table's <CODE>RepeatForegroundColors</CODE> value.
 * @see #setRepeatForegroundColors
 */
public Color[] getRepeatForegroundColors();

/**
 * Sets an array of colors to be used when <CODE>Foreground</CODE> value is
 * <CODE>JCTableEnum.REPEAT_ROW</CODE> or
 * <CODE>JCTableEnum.REPEAT_COLUMN</CODE>.
 */
public void setRepeatForegroundColors(Color[] values);

/**
 * Returns true if the cell is traversable.
 */
public boolean isTraversable();

/**
 * Sets whether the cell is traversable.
 */
public void setTraversable(boolean value);

/**
 * Returns the vertical alignment for the cell.
 * Valid return values are <CODE>TOP</CODE>, <CODE>CENTER</CODE>, 
 * <CODE>BOTTOM</CODE>. 
 */
public int getVerticalAlignment();

/**
 * Sets the vertical alignment for the cell.
 */
public void setVerticalAlignment(int value);

/**
 * Add a <CODE>PropertyChangeListener</CODE> for CellStyle property changes.
 * <CODE>CellStyleChangeSupport</CODE> is a utility class that can be 
 * used to manage listeners and fire <CODE>PropertyChange</CODE> events.
 */
public void addPropertyChangeListener(PropertyChangeListener listener);

/**
 * Remove a <CODE>PropertyChangeListener</CODE> for CellStyle property changes.
 */
public void removePropertyChangeListener(PropertyChangeListener listener);

/**
 * Returns a copy of the style object.
 */
public Object clone();

}
