/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ComponentHandlerNew.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:33 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import com.klg.jclass.cell.JCCellEditor;

import com.klg.jclass.util.JCUtil;

import java.awt.*;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

/**
 * <CODE>ComponentHandler</CODE> controls the placement of components within
 * table and ensures that traverse from a component to table or table to
 * component flows across and down the cells or to the left and up for reverse
 * traversal.
 *
 * @since JClass 6.0 for JDK 1.4 and later
 */
public class ComponentHandlerNew extends ComponentHandler {

boolean focusTraversalKeysEnabledSave;

/**
 * Creates a new ComponentHandler instance. <CODE>setTable</CODE> must be called
 * to connect the handler to table.
 */
public ComponentHandlerNew()
{
	super();
}

/**
 * Creates new ComponentHandler instance linked to table.
 *
 * @param table The table for this component handler
 */
public ComponentHandlerNew(JCTable table)
{
	super(table);
}

/**
 * Add key and focus listeners to component if it is focusable, otherwise add
 * them the first child which is focusable
 *
 * @param c The component to which to add the listeners
 * @param keyListener The key listener to add
 * @param focusListener  The focusListener to add
 */
protected void addListeners(Component c, KeyListener keyListener,
							FocusListener focusListener)
{
	
	addKeyAndFocusListeners(c, keyListener, focusListener);
}

/**
 * Adds a <CODE>KeyListener</CODE> to the component when table is managing
 * component focus.  Cancels any edit and cause a traversal to the underlying
 * cell.
 *
 * @param event The focus event which caused foucs to be gained.
 */
public void focusGained(FocusEvent event)
{
	
	table.getEditTraverseHandler().commit(true);

	JCCellEditor ed = table.getEditTraverseHandler().getCellEditor();
	
	Component component = event.getComponent();
	focusTraversalKeysEnabledSave = component.getFocusTraversalKeysEnabled();

    if (updateTable(component)) {
	    table.setFocusTraversalPolicy(new TableFocusTraversalPolicy(table,
		                                                            component));
		component.setFocusTraversalKeysEnabled(false);
    }
}

/**
 * Removes TraverseHandler as a key listener on the component when it loses focus.
 *
 * @param event The focus event which caused foucs to be lost.
 */
public void focusLost(FocusEvent event)
{
	

	Component component = event.getComponent();
	component.setFocusTraversalKeysEnabled(focusTraversalKeysEnabledSave);
	table.setFocusTraversalPolicy(new TableFocusTraversalPolicy(table, null));
}

/**
 * Tracks TAB/SHIFT-TAB and keypad arrow keys to control navigation when a
 * component has focus.
 *
 * @param event The event from the key press
 */
public void keyPressed(KeyEvent event)
{
	

    int direction = getDirectionFromKeyEvent(event);
    if (direction != JCTableEnum.NOVALUE) {
		event.consume();

		// Handle CTRL-TAB and CTRL-SHIFT-TAB.  We need to set the focus
		// outside of table to either the next or previous component in the
		// focus cycle above us.
		if (controlDown) {
			table.getFocusManager().handleControlTab(shiftDown, false);
			return;
		}

		JCCellPosition cp = getTablePosition(event.getComponent());
		if (cp != null) {
		    traverseNext(cp, direction);
		}
	}
}

}
