/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ComponentHandlerOld.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:33 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;



import javax.swing.FocusManager;
import javax.swing.*;
import java.awt.*;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

/**
 * <CODE>ComponentHandler</CODE> controls the placement of components within
 * table and ensures that traverse from a component to table or table to
 * component flows across and down the cells or to the left and up for reverse
 * traversal.
 */
public class ComponentHandlerOld extends ComponentHandler {

protected FocusManager fm;

/**
 * Creates a new ComponentHandler instance. <CODE>setTable</CODE> must be called
 * to connect the handler to table.
 */
public ComponentHandlerOld()
{
	super();
}

/**
 * Creates new ComponentHandler instance linked to table.
 *
 * @param table The table for this component handler
 */
public ComponentHandlerOld(JCTable table)
{
	super(table);
}

/**
 * Add key and focus listeners to component if it is focusable, otherwise add
 * them the first child which is focusable
 *
 * @param c The component to which to add the listeners
 * @param keyListener The key listener to add
 * @param focusListener  The focusListener to add
 */
protected void addListeners(Component c, KeyListener keyListener,
							FocusListener focusListener)
{
	
	addKeyAndFocusListeners(c, keyListener, focusListener);

	// This is added so that tabbing out of the component does not tab you
	// you out of the entire table.  Fix for bug 11275.
	if (c instanceof JComponent) {
		((JComponent)c).setNextFocusableComponent(table);
	}
}

/**
 * Adds a <CODE>KeyListener</CODE> to the component when table is managing
 * component focus.  Cancels any edit and cause a traversal to the underlying
 * cell.
 *
 * @param event The focus event which caused foucs to be gained.
 */
public void focusGained(FocusEvent event)
{
	
	// Save the focus manager so that we can disable it while a component in
	// ths table has focus
	fm = FocusManager.getCurrentManager();

	// Disable the focus manager in order to capture TAB/shift-TAB. With it
    // enabled, we can set the next focusable component but have no way to
    // determine whether the new "current" cell in table is to the left or
    // right. When disabled, we can pick up these keys and figure out were to go
    // next. The manager will be reset when the component loses focus -- or when
    // a TAB/shift-TAB is caught by the key listener. This isn't an ideal
    // solution, but shouldn't cause any problems since you have to click
    // somewhere else to transfer focus, an action that will generate a lost
    // focus on the component and cause the manager to be re-enabled.
	FocusManager.disableSwingFocusManager();
	

	table.getEditTraverseHandler().commit(true);

	Component component = event.getComponent();
    updateTable(component);
}

/**
 * Removes TraverseHandler as a key listener on the component when it loses focus.
 *
 * @param event The focus event which caused foucs to be lost.
 */
public void focusLost(FocusEvent event)
{
	
	// Restore the focus manager
	fm.setCurrentManager(fm);
}

/**
 * Tracks TAB/SHIFT-TAB and keypad arrow keys to control navigation when a
 * component has focus.
 *
 * @param event The event from the key press
 */
public void keyPressed(KeyEvent event)
{
	

    int direction = getDirectionFromKeyEvent(event);
    if (direction != JCTableEnum.NOVALUE) {
		event.consume();

		// Restore the focus manager in case this call occurs before focusLost
	    
		fm.setCurrentManager(fm);
		JCCellPosition cp = getTablePosition(event.getComponent());
		if (cp != null) {
		    traverseNext(cp, direction);
		}
	}
}

}
