/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ComponentModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:33 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import java.awt.Component;

/**
 * <CODE>ComponentModel</CODE> defines the requirements of a class that controls
 * the storage, placement and traversal of components in cells.
 */
public interface ComponentModel extends SeriesModel {

/**
 * Returns the JCTable.
 *
 * @return The table for this component model
 */
public JCTable getTable();

/**
 * Sets the JCTable.
 *
 * @param table The new table for this component model
 */
public void setTable(JCTable table);

/**
 * Forces all components to be added to their container whether they have been
 * exposed or not. This is required for Look And Feel support.
 */
public void addComponentsToParent();

/**
 * Returns the JCCellPosition that contains the given component.
 *
 * @param c The component whose position we want to find
 *
 * @return An object which contains the cell position of the component
 */
public JCCellPosition findPosition(Component c);

/**
 * Returns true if one or more components have not been added to table yet.
 *
 * @return Whether one or more components have not been added to table yet.
 * @see #setParentSetRequired
 */
public boolean isParentSetRequired();

/**
 * Flags that one or more components do not have a parent yet. The appropriate
 * container for components will not be present before table is realized.
 *
 * @param value The new value of parentSetRequired which tells us whether one
 *     or more components do not have a parent yet.
 */
public void setParentSetRequired(boolean value);

/**
 * Remove components when a span is added. Any components not contained
 * in the span will re-added in CellAreaRenderer.
 */
public void removeCellComponents();

/**
 * Reconfigure components such that components in visible cells are visible
 * and components in invisible cells are invisible.
 */
public void reconfigureComponents();

}
