/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataViewModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:33 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import com.klg.jclass.cell.JCCellRenderer;
import com.klg.jclass.cell.JCCellEditor;

import java.util.Hashtable;

/**
 * The <CODE>DataViewModel</CODE> interface defines the methods for classes
 * that can serve as an  intermediary between the <CODE>JCTable</CODE> and the
 * data source (<CODE>TableDataModel</CODE>).
 * @see TableDataModel
 */
public interface DataViewModel extends TableDataModel, JCTableDataListener {

/**
 * Returns the JCTable controlled by the DataViewModel.
 * @return The current table controlled by this DataView.
 */
public JCTable getTable();

/**
 * Sets the JCTable controlled by the DataViewModel.
 * @param table The new table for this DataView
 */
public void setTable(JCTable table);

/**
 * Returns the cell renderer for a row/column.
 * @param row The row index
 * @param column The column index
 * @return The cell renderer for the given row and column.
 */
public JCCellRenderer getCellRenderer(int row, int column);

/**
 * Returns the cell renderer for a row/column given the data for the cell.
 * @param row The row index
 * @param column The column index
 * @param o The data in the cell indexed by row and column
 * @return The cell renderer based on the data in the cell given the row and
 *    column indicies
 */
public JCCellRenderer getCellRenderer(int row, int column, Object o);

/**
 * Returns the cell renderer given an object.
 * @param o The object the cell renderer needs to render
 * @return The cell renderer based on the object.
 */
public JCCellRenderer getCellRenderer(Object o);

/**
 * Returns a Hashtable of object to renderer mappings.
 * @return The current renderer cache
 */
public Hashtable getRenderersTable();

/**
 * Sets the Hashtable of object to renderer mappings.
 * @param ht The new renderer cache
 */
public void setRenderersTable(Hashtable ht);

/**
 * Returns the cell editor for a row/column.
 * @param row The row index
 * @param column The column index
 * @return The cell editor for the given row and column
 */
public JCCellEditor getCellEditor(int row, int column);

/**
 * Returns the cell editor given an object.
 * @param o The object for which a cell editor is required
 * @return The cell editor based on the given object
 */
public JCCellEditor getCellEditor(Object o);

/**
 * Returns a Hashtable of object to editor mappings.
 * @return The current editor cache
 */
public Hashtable getEditorsTable();

/**
 * Sets the Hashtable for object to editor mappings.
 * @param ht The new eidtor cache
 */
public void setEditorsTable(Hashtable ht);

/**
 * Returns the bound data source.
 * @return The current bound data source
 */
public TableDataModel getDataSource();

/**
 * Sets the bound data source.
 * @param tableData The new bound data source
 */
public void setDataSource(TableDataModel tableData);

/**
 * Returns true if the data source implements EditableTableData.
 * @return Is the data source editable?
 */
public boolean isEditable();

/**
 * Returns the object given a row or column. This convenience method handles
 * both cells and labels.
 * @param row The row index
 * @param column The column index
 * @return The data object based on the given row and column
 */
public Object getObject(int row, int column);

/**
 * Sets the cell value for editable data sources.
 * @param o The new cell value
 * @param row The row index
 * @param column The column index
 * @return The status of the edit.  Note that if the datasource is not
 *    editable, no edit is done and true is returned.
 */
public boolean setTableDataItem(Object o, int row, int column);

}




