/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: EditTraverseHandler.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:33 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import com.klg.jclass.cell.JCCellEditor;

import java.awt.AWTEvent;
import java.awt.Component;

/**
 * <CODE>EditTraverseHandler</CODE> controls the cell traversal and editor
 * display/commit/cancel logic. It also fires <CODE>TraverseCell</CODE> and
 * <CODE>EditCell</CODE> events as required.
 */
public interface EditTraverseHandler {

public static final int LEFT  = 1;
public static final int RIGHT = 2;
public static final int UP    = 3;
public static final int DOWN  = 4;

public static final int HORIZONTAL = 0;
public static final int VERTICAL   = 1;

/**
 * Returns the current cell editor.
 *
 * @return The current cell editor.  Returns <CODE>null</CODE> if none.
 */
public JCCellEditor getCellEditor();

/**
 * Sets the cell editor.
 *
 * @param ed The new cell editor
 */
public void setCellEditor(JCCellEditor ed);

/**
 * Get the row of the current cell being edited.
 *
 * @return The current edit row
 */
public int getEditRow();

/**
 * Get the column of the current cell being edited.
 *
 * @return The current edit column
 */
public int getEditColumn();

/**
 * Traverse to the specified cell.
 *
 * @param row The row to which to traverse
 * @param column The column to which to traverse
 * @param select Should the cell be selected
 * @param event The event that caused this traverse (null if none)
 * @param traverse_type The type of traversal (right, left, down, up, home, ...)
 * @param show_editor Display the component, if called programatically
 *
 * @return false if the cell is not traversable
 */
public boolean traverse(int row, int column, boolean select, AWTEvent event,
						int traverse_type, boolean show_editor);

/**
 * Traverses to the top-left cell.
 */
public void traverseInitial();

/**
 * Check to see if editor component needs reparenting.  Reparenting is
 * needed if the CellArea objects have changed boundaries around the
 * cell the editor is in.
 */
public void checkAndReparentEditor();

/**
 * Sets the width of the edit component.
 *
 * @param value The new width
 */
public void setWidth(int value);

/**
 * Sets the height of the edit component.
 *
 * @param value The new height
 */
public void setHeight(int value);

/**
 * Repositions the editor.
 */
public void repositionEditor();

/**
 * Move the component.
 *
 * @param direction The direction of movement, horizontal or vertical
 * @param value The amount in pixels of movement
 */
public void move(int direction, int value);

/**
 * Cancels the current edit on the given table, if an edit is in progress
 * @param hide determines whether the editor is hidden after the commit.
 *
 * @return Whether cancel succeeded
 */
public boolean cancel(boolean hide);

/**
 * Commits the current edit on the table.
 * @param hide determines whether the editor is hidden after the commit.
 *
 * @return Whether commit succeeded
 */
public boolean commit(boolean hide);

/**
 * Set focus on the component if it is focusable or to the first focusable child.
 */
public void setFocusOnEditor();

/**
 * Return the component if it is focusable, otherwise the first kid which
 * is focusable.  If no focusable kids, return the component (we need to use
 * something).  Note that this method does not recurse into the children's
 * children.
 *
 * @param c The component whose focus or children's focus is being investigated
 *
 * @return The component or child component which is focusable.  Note that if
 *     no children are focusable, the original component is returned whether
 *     it is focusable or not
 */
public Component getFocusableChild(Component c);

}
