/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: EditTraverseHandlerNew.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:33 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;


import com.klg.jclass.util.JCUtil;

import java.awt.*;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyListener;

/**
 * <CODE>EditTraverseHandlerNew</CODE> controls the cell traversal and editor
 * display/commit/cancel logic for JDK's 1.4 and later. It also fires
 * <CODE>TraverseCell</CODE> and <CODE>EditCell</CODE> events as required.
 */
public class EditTraverseHandlerNew extends EditTraverseHandlerAbstract {

// Semaphore used to prevent reentrance to the focusLost() method
public static boolean tableFocusRequested = false;

// Is the current focus change temporary?
protected boolean temporaryFocusLost = false;

boolean focusTraversalKeysEnabledSave;

/**
 * Constructor that requires a table
 *
 * @param table The table for which this EditTraverseHandler is used.
 */
public EditTraverseHandlerNew(JCTable table)
{
	super(table);
}

/**
 * Add key and focus listeners to component if it is focusable, otherwise add
 * them the first child which is focusable
 *
 * @param c The component to which to add the listeners
 * @param keyListener The key listener to add
 * @param focusListener  The focusListener to add
 */
protected void addListeners(Component c, KeyListener keyListener,
							FocusListener focusListener)
{
	
	addKeyAndFocusListeners(c, keyListener, focusListener);

	focusTraversalKeysEnabledSave = c.getFocusTraversalKeysEnabled();
	c.setFocusTraversalKeysEnabled(false);
}

/**
 * Reset the old focus owner for this focus period
 *
 * @param event The focus event which caused focus to be gained
 */
public void focusGained(FocusEvent event)
{
	
	if (temporaryFocusLost) {
		temporaryFocusLost = false;
		return;
	}

	table.setFocusTraversalPolicy(new TableFocusTraversalPolicy(table, component));
}

/**
 * Sets the table as a focused component when the editor loses permanent focus.
 *
 * @param event The focus event which caused focus to be lost
 */
public void focusLost(FocusEvent event)
{
	

	// Ignore events if re-entrant or event is only a temporary loss of focus
	temporaryFocusLost = event.isTemporary();
	if (tableFocusRequested || temporaryFocusLost) {
		return;
	}

	// Critical section of code - must be non-reentrant!
	tableFocusRequested = true;

	Component eventComponent = event.getComponent();
	

	// Check if the new focus owner is outside of table
	Component oppositeComponent = event.getOppositeComponent();
	boolean outsideTable = (table != oppositeComponent &&
	                        !table.isAncestorOf(oppositeComponent));
	

	table.setFocusTraversalPolicy(new TableFocusTraversalPolicy(table, null));

	// Remove EditTraverseHandlerNew as focus listener if it is no longer the
	// current edit component.  Ideally, this should be done in
	// setCellEditor(null), but it doesn't sequence the focus requests correctly
	// and therefore doesn't work.  This will remove the listener as late as
	// possible.
	// CRG - This focus event sequencing may actually work in JDK1.4 but we
	// still remove it here because we want the pre and post JDK mechanisms to
	// be as similar as possible.
	if (outsideTable || event.getComponent() != component) {
	    
		event.getComponent().removeFocusListener(this);
		event.getComponent().setFocusTraversalKeysEnabled(focusTraversalKeysEnabledSave);
	}

	// End of critical section of code.
	tableFocusRequested = false;
	
}

}
