/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: EditTraverseHandlerOld.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:33 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;



import javax.swing.*;
import java.awt.*;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyListener;

/**
 * <CODE>EditTraverseHandlerOld</CODE> controls the cell traversal and editor
 * display/commit/cancel logic for JDK's prior to 1.4. It also fires
 * <CODE>TraverseCell</CODE> and <CODE>EditCell</CODE> events as required.
 */
public class EditTraverseHandlerOld extends EditTraverseHandlerAbstract {

/**
 * Constructor that requires a table
 *
 * @param table The table for which this EditTraverseHandler is used.
 */
public EditTraverseHandlerOld(JCTable table)
{
	super(table);
}

/**
 * Add key and focus listeners to component if it is focusable, otherwise add
 * them the first child which is focusable
 *
 * @param c The component to which to add the listeners
 * @param keyListener The key listener to add
 * @param focusListener  The focusListener to add
 */
protected void addListeners(Component c, KeyListener keyListener,
							FocusListener focusListener)
{
	
	addKeyAndFocusListeners(c, keyListener, focusListener);

	// This is added so that tabbing out of the component does not tab you
	// you out of the entire table.  Fix for bug 11275.
	if (c instanceof JComponent) {
		((JComponent)c).setNextFocusableComponent(table);
	}
}


// Semaphore used to prevent reentrance to the focusLost() method
public static boolean tableFocusRequested = false;

// Is the current focus change temporary?
protected boolean temporaryFocusLost = false;

/**
 * Reset the old focus owner for this focus period
 *
 * @param event The focus event which caused focus to be gained
 */
public void focusGained(FocusEvent event)
{
	
	if (temporaryFocusLost) {
		temporaryFocusLost = false;
		return;
	}
	table.oldFocusOwner = null;
}

/**
 * Sets the table as a focused component when the editor loses permanent focus.
 *
 * @param event The focus event which caused focus to be lost
 */
public void focusLost(FocusEvent event)
{
	

	// Ignore events if re-entrant or event is only a temporary loss of focus
	temporaryFocusLost = event.isTemporary();
	if (tableFocusRequested || temporaryFocusLost) {
		return;
	}

	// Critical section of code - must be non-reentrant!
	tableFocusRequested = true;

	

	// When a key press or a mouse click inside table causes a focus change,
	// table will end up calling requestFocus().  At that time, the
	// oldFocusOwner is stored and if we get here, this previous owner will be
	// the current editor. Thus if we get here and the oldFocusOwner is null,
	// the user must have clicked outside of table.  In this case, we don't want
	// table to request focus (as the user explicitly has asked for the focus to
	// be shifted outside of table).  Otherwise, we want table to request focus
	// so we know where it is.
	boolean outsideTable = false;
	if (table.oldFocusOwner == null) {
		outsideTable = true;
	}
	else {
	    table.getFocusManager().requestFocus();
	}
	table.oldFocusOwner = null;
	

	// Remove EditTraverseHandlerOld as focus listener if it is no longer the
	// current edit component.  Ideally, this should be done in
	// setCellEditor(null), but it doesn't sequence the focus requests correctly
	// and therefore doesn't work.  This will remove the listener as late as
	// possible.
	if (outsideTable || event.getComponent() != component) {
	    
		event.getComponent().removeFocusListener(this);
	}

	// End of critical section of code.
	tableFocusRequested = false;
	
}

}
