/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: FocusManagerNew.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:33 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;


import com.klg.jclass.util.JCUtil;

import java.awt.*;

/**
 * A FocusManager object for JDK 1.4 and later
 */
public class FocusManagerNew extends FocusManager {

/**
 * Create the FocusManagerNew for the given table with no parameters.  One must
 * call setTable() after creation.
 */
FocusManagerNew()
{
	super();
}

/**
 * Create the FocusManger for the given table
 * @param table The table in which this focus manager is used.
 */
FocusManagerNew(JCTable table)
{
	super(table);
}


/**
 * Set the table that uses this focus manager
 *
 * @param table The new table that uses this focus manager
 */
public void setTable(JCTable table)
{
	super.setTable(table);
	if (table != null) {
	    table.setFocusTraversalKeysEnabled(false);
		table.setFocusCycleRoot(true);
	    table.setFocusTraversalPolicy(new TableFocusTraversalPolicy(table,
                                                                    null));
	}
}

/**
 * Gets the passed in component to request focus.
 * @param c The component who will request the focus.
 *
 * @return Whether the request will fail or most likely succeed (it only
 *     fails under an extraordinary event such as a disposal of the
 *     Component's peer).
 */
public boolean requestFocus(Component c)
{
	
	if (c == null) {
		return(requestFocus());
	}
	else {
	    return(c.requestFocusInWindow());
	}
}

/**
 * Gets table to request focus.
 *
 * @return Whether the request will fail or most likely succeed (it only
 *     fails under an extraordinary event such as a disposal of the
 *     Component's peer).
 */
public boolean requestFocus()
{
	
	boolean returnValue = false;
    if (table != null) {
		returnValue = table.requestFocusInWindow();
	}
	
	return(returnValue);
}

/**
 * Creates the default edit handler for JDKs 1.4 or later
 *
 * @return The newly created edit handler for a given model
 */
public EditTraverseHandler createDefaultEditTraverseHandler()
{
	return(new EditTraverseHandlerNew(table));
}

/**
 * Creates the default component handler.
 *
 * @return The newly created component handler for pre JDK1.4
 */
public ComponentModel createDefaultComponentModel()
{
	return(createDefaultComponentModel("com.klg.jclass.table.ComponentHandlerNew"));
}

/**
 * Can this component receive the focus
 *
 * @param c The component whose focusability is in question
 * @return Whether the component can receive focus
 */
public boolean isFocusTraversable(Component c)
{
	 if (c == null) {
		return(false);
	 }
	 return(c.isFocusable());
}

/**
 * Return the child <code>Component</code> of the specified
 * <code>Component</code> that is the focus owner, if any.
 *
 * @param c the root of the <code>Component</code> hierarchy to search for the
 *     focus owner
 * @return the focus owner, or <code>null</code> if there is no focus owner, or
 *     if the focus owner is not <code>c</code>, or a descendant of
 *     <code>c</code>
 */
public Component findFocusOwner(Component c)
{
	if (c == null) {
		return(null);
	}

	Component focusOwner =
	    KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusOwner();
	

	// Verify focusOwner is a descendant of c
	for (Component temp = focusOwner;
	     temp != null;
	     temp = (temp instanceof Window) ? null : temp.getParent())
	{
	    if (temp == c) {
		    return(focusOwner);
	    }
	}

	return(null);
}

/**
 * The user has hit CTRL-TAB or CTRL-SHIFT-TAB.  We need to go up one cycle
 * and tab out of table to the next or previous component.
 *
 * @param isShift True if the shift key has been hit.
 * @param doCommit Do a commit edit before traversing.  This is needed the user
 *    is editing a cell when he/she hits CTRL-TAB or CTRL-SHIFT-TAB.
 */
public void handleControlTab(boolean isShift, boolean doCommit)
{
	if (table == null) {
		return;
	}

	KeyboardFocusManager kfm =
	    KeyboardFocusManager.getCurrentKeyboardFocusManager();
	if (doCommit && editTraverseHandler != null) {
	    editTraverseHandler.commit(true);
	}
	if (isShift) {
		kfm.upFocusCycle(table);
		kfm.focusPreviousComponent(table);
	}
	else {
		kfm.upFocusCycle(table);
		kfm.focusNextComponent(table);
	}
}

/**
 * In JDK 1.4 or later, we need to make sure that when table has the focus
 * that we are within table's current focus cycle rather than any focus cycle
 * of a container that table happens to be in.
 */
public void transferFocusCycle()
{
	KeyboardFocusManager kfm =
	    KeyboardFocusManager.getCurrentKeyboardFocusManager();
	if (kfm.getCurrentFocusCycleRoot() != table) {
	    table.transferFocusDownCycle();
	}
}

}
