/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: FocusModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:33 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import java.awt.Component;

/**
 * Since in JDK1.4, a different focus mechanism is used, an object implementing
 * the <CODE>FocusManager</CODE> interface is required to use the correct
 * mechanism for the current environment.
 */
public interface FocusModel {

/**
 * Return the table for this focus manager
 *
 * @return The table that uses this focus manager
 */
public JCTable getTable();

/**
 * Set the table that uses this focus manager
 *
 * @param table The new table that uses this focus manager
 */
public void setTable(JCTable table);

/**
 * Gets the passed in component to request focus.
 * @param c The component who will request the focus.
 *
 * @return Whether the request will fail or most likely succeed (it only
 *     fails under an extraordinary event such as a disposal of the
 *     Component's peer).
 */
public boolean requestFocus(Component c);

/**
 * Gets table to request focus.

 * @return Whether the request will fail or most likely succeed (it only
 *     fails under an extraordinary event such as a disposal of the
 *     Component's peer).
 */
public boolean requestFocus();

/**
 * Returns the edit handler for the table.
 *
 * @return The current edit handler for the table
 */
public EditTraverseHandler getEditTraverseHandler();

/**
 * Sets a new edit handler for the table
 *
 * @param editTraverseHandler The new edit handler
 */
public void setEditTraverseHandler(EditTraverseHandler editTraverseHandler);

/**
 * Returns the current component handler.
 *
 * @return The component handler for the table
 */
public ComponentModel getComponentModel();

/**
 * Sets a new component handler for the table
 *
 * @param componentHandler The new compnnent handler for the table
 */
public void setComponentModel(ComponentModel componentHandler);

/**
 * Creates the default Component handler.
 *
 * @return The newly created component handler for a given model
 */
public ComponentModel createDefaultComponentModel();

/**
 * Can this component receive the focus
 *
 * @param c The component whose focusability is in question
 * @return Whether the component can receive focus
 */
public boolean isFocusTraversable(Component c);

/**
 * Return the child <code>Component</code> of the specified
 * <code>Component</code> that is the focus owner, if any.
 *
 * @param c the root of the <code>Component</code> hierarchy to search for the
 *     focus owner
 * @return the focus owner, or <code>null</code> if there is no focus owner, or
 *     if the focus owner is not <code>c</code>, or a descendant of
 *     <code>c</code>
 */
public Component findFocusOwner(Component c);

/**
 * The user has hit CTRL-TAB or CTRL-SHIFT-TAB.  We need to go up one cycle
 * and tab out of table to the next or previous component.  Nothing is done
 * in JDK's prior to 1.4.
 *
 * @param isShift True if the shift key has been hit.
 * @param doCommit Do a commit edit before traversing.  This is needed the user
 *    is editing a cell when he/she hits CTRL-TAB or CTRL-SHIFT-TAB.
 */
public void handleControlTab(boolean isShift, boolean doCommit);

/**
 * In JDK 1.4 or later, we need to make sure that when table has the focus
 * that we are within table's current focus cycle rather than any focus cycle
 * of a container that table happens to be in.
 */
public void transferFocusCycle();

}
