/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCCellPosition.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:33 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import java.io.Serializable;

/**
 * <CODE>JCCellPosition</CODE> is used to set the attributes or 
 * values of a cell, label or range.<P>
 * The following special row/column values specify other parts of 
 * the table:
 * <pre>
 *         ALL         all cells and labels in a row or column
 *         LABEL       a row or column label
 *         ALLCELLS    all cells (but not labels) in a row or column
 * </pre>
 * If both row and column are set to <CODE>ALL</CODE>, the entire table is 
 * specified (all rows, columns, and labels). The following are some 
 * examples of usage:
 * <pre>
 *          1 2         cell (1, 2)
 *          ALL 1       all cells and the label of column 1
 *          2 ALL       all cells and the label of row 2
 *          ALL LABEL   all row labels
 * </pre>
 */
public class JCCellPosition implements Serializable {

/** Row number, starting at 0. */
public int row = JCTableEnum.NOVALUE;

/** Column number, starting at 0. */
public int column = JCTableEnum.NOVALUE;

/**
 * Creates a new <code>JCCellPosition</code> instance with row and column set to
 * <CODE>JCTableEnum.NOVALUE</CODE>
 */
public JCCellPosition() {}

/**
 * Creates a new <code>JCCellPosition</code> instance for the given row/column.
 * @param r The row index
 * @param c The column index
 */
public JCCellPosition(int r, int c)
{
    row = r;
    column = c;
}

/**
 * Returns <code>true</code> if another <code>JCCellPosition</code> equals this
 * one.
 * @return <code>true</code> if another <code>JCCellPosition</code> equals this
 * one
 * @param p The JCCellPosition object to compare against.
 */
public boolean equals(JCCellPosition p)
{
    return(p.row == row && p.column == column);
}

/**
 * Returns <code>true</code> if a row/column pair matches this
 * <code>JCCellPosition</code>.
 * @return <code>true</code> if a row/column pair matches this
 * <code>JCCellPosition</code>
 * @param r The row index
 * @param c The column index
 */
public boolean equals(int r, int c)
{
    return(r == row && c == column);
}

/**
 * Changes the row/column values for a <code>JCCellPosition</code> object.
 * @param r The row index
 * @param c The column index
 */
public void reshape(int r, int c)
{
    row = r;
    column = c;
}

/**
 * Return a string representation of this JCCellPosition object.
 * @return A string representation of this object.
 */
public String toString()
{
    return("R" + row + "C" + column);
}

}

