/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCCellRange.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:33 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import java.awt.Rectangle;
import java.io.Serializable;

/**
 * <CODE>JCCellRange</CODE> is used to specify a range of cells or labels.
 */
public class JCCellRange implements Serializable {

/** Row of the first cell in range (top-left corner). */
public int start_row;

/** Column of the first cell in range (top-left corner). */
public int start_column;

/** Row of the last cell in range (bottom-right corner). */
public int end_row;

/** Column of the last cell in range (top-left corner). */
public int end_column;

/**
 * Creates an instance with all cells set to <CODE>JCTableEnum.NOVALUE</CODE>.
 */
public JCCellRange()
{
    reshape(JCTableEnum.NOVALUE, JCTableEnum.NOVALUE, JCTableEnum.NOVALUE,
            JCTableEnum.NOVALUE);
}

/**
 * Creates an instance with the specified start and end cells.
 * @param c1
 * @param c2
 * @param r1
 * @param r2
 */
public JCCellRange(int r1, int c1, int r2, int c2)
{
    reshape(r1, c1, r2, c2);
}

/**
 * Creates a <code>JCCellRange</code> based on a rectangle.
 * @param r
 */
public JCCellRange(Rectangle r)
{
    reshape(r.y, r.x, r.y + r.height, r.x + r.width);
}

/**
 * Returns the minimum of two values.
 * @return minimum of two values
 * @param a
 * @param b
 */
private final static int min(int a, int b)
{
    return (a < b) ? a : b;
}

/**
 * Returns the maximum of two values.
 * @return maximum of two values
 * @param a
 * @param b
 */
private final static int max(int a, int b)
{
    return (a > b) ? a : b;
}

/**
 * Reshapes the range to the specified start and end cells.
 * @param c1
 * @param c2
 * @param r1
 * @param r2
 * @return
 */
public final JCCellRange reshape(int r1, int c1, int r2, int c2)
{
    start_row = r1;
    start_column = c1;
    end_row = r2;
    end_column = c2;
    return this;
}

/**
 * Reshapes the range to the specified single cell.
 * @return
 * @param r
 * @param c
 */
public final JCCellRange reshape(int r, int c)
{
    start_row = end_row = r;
    start_column = end_column = c;
    return this;
}

/**
 * Reshapes the range to the specified range.
 * @return
 * @param r
 */
public final JCCellRange reshape(JCCellRange r)
{
    start_row = r.start_row;
    start_column = r.start_column;
    end_row = r.end_row;
    end_column = r.end_column;
    return this;
}

/**
 * Reshapes the range to the specified rectangle.
 * @param r
 * @return
 */
public final JCCellRange reshape(java.awt.Rectangle r)
{
    start_row = r.y;
    start_column = r.x;
    end_row = r.y + r.height;
    end_column = r.x + r.width;
    return this;
}

/**
 * Returns <code>true</code> if the specified range is equal to this range.
 * @return <code>true</code> if the specified range is equal to this range
 * @param r
 */
public final boolean equals(JCCellRange r)
{
    return(r.start_row == start_row &&
           r.start_column == start_column &&
           r.end_row == end_row &&
           r.end_column == end_column);
}

/**
 * Checks whether the two ranges intersect.
 * @param r
 * @return
 */
public final boolean intersects(JCCellRange r)
{
    return(!((r.start_column + r.end_column < start_column) ||
           (r.start_row + r.end_row < start_row) ||
           (r.start_column >= start_column + end_column) ||
           (r.start_row >= start_row + end_row)));
}

/**
 * Checks to see whether the two ranges overlap.
 * @return
 * @param r
 */
public final boolean overlaps(JCCellRange r)
{
    return(end_row >= r.start_row &&
           start_row <= r.end_row &&
           end_column >= r.start_column &&
           start_column <= r.end_column);
}

/**
 * Computes the intersection of two ranges.
 * @param r
 * @return
 */
public final JCCellRange intersection(JCCellRange r)
{
    return new JCCellRange(max(start_row, r.start_row),
                           max(start_column, r.start_column),
                           min(end_row, r.end_row),
                           min(end_column, r.end_column));
}

public final void intersection(JCCellRange r, JCCellRange inter)
{
    inter.reshape(max(start_row, r.start_row),
                  max(start_column, r.start_column),
                  min(end_row, r.end_row),
                  min(end_column, r.end_column));
}

/**
 * Returns <code>true</code> if the cell is inside this range.
 * @return <code>true</code> if the cell is inside this range
 * @param row
 * @param col
 */
public final boolean inside(int row, int col)
{
    return(start_row != JCTableEnum.NOVALUE &&
           (start_row < end_row ? start_row : end_row) <= row &&
           (start_row > end_row ? start_row : end_row) >= row &&
           (start_column < end_column ? start_column : end_column) <= col &&
           (start_column > end_column ? start_column : end_column) >= col);
}

/**
 * Returns <code>true</code> if the row is inside this range.
 * @param row
 * @return <code>true</code> if the row is inside this range
 */
public final boolean inRowRange(int row)
{
    return(start_row != JCTableEnum.NOVALUE &&
           row >= start_row && row <= end_row);
}

/**
 * Returns <code>true</code> if this range is not empty and the row is inside
 * it.
 * @return <code>true</code> if this range is not empty and the row is inside it
 * @param row
 */
public final boolean inRowRange2(int row)
{
    return(min(start_row, end_row) <= row &&
           (start_row > end_row ? start_row : end_row) >= row);
}

/**
 * Returns <code>true</code> if this range is not empty and the column is inside
 * it.
 * @return <code>true</code> if this range is not empty and the column is inside
 * it
 * @param col
 */
public final boolean inColumnRange(int col)
{
    return(start_row != JCTableEnum.NOVALUE &&
           col >= start_column && col <= end_column);
}

/**
 * Returns <code>true</code> if the column is inside this range.
 * @return <code>true</code> if the column is inside this range
 * @param col
 */
public final boolean inColumnRange2(int col)
{
    return(min(start_column, end_column) <= col &&
           max(start_column, end_column) >= col);
}

public String toString()
{
    return("R" + start_row + "C" + start_column + ":R" + end_row +
           ":C" + end_column);
}

}

