/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCCellStyle.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:33 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import com.klg.jclass.cell.JCCellEditor;
import com.klg.jclass.cell.JCCellRenderer;

import javax.swing.plaf.ColorUIResource;
import java.awt.*;
import java.beans.PropertyChangeListener;

/**
 * The <CODE>JCCellStyle</CODE> class provides a default implementation of
 * the <CODE>CellStyleModel</CODE> interface.
 */
public class JCCellStyle implements java.io.Serializable, CellStyleModel {

protected CellStyleModel   parent_style;

protected Color            background;
protected CellBorderModel  border;
protected int              border_sides;
protected Color            cell_border_color;
protected int              cell_border_color_mode;
protected int              clip_hints;
protected Class            data_type;
protected boolean          editable;
protected JCCellEditor     editor;
protected Color            foreground;
protected Font             font;
protected int              horizontal_alignment;
protected JCCellRenderer   renderer;
protected int              repeat_background;
protected Color[]          repeat_background_colors;
protected int              repeat_foreground;
protected Color[]          repeat_foreground_colors;
protected boolean          traversable;
protected int              vertical_alignment;
	
protected boolean          border_sides_set = false;
protected boolean          cell_border_color_mode_set = false;
protected boolean          clip_hints_set = false;
protected boolean          data_type_set = false;
protected boolean          editable_set = false;
protected boolean          editor_set = false;
protected boolean          horizontal_alignment_set = false;
protected boolean          renderer_set = false;
protected boolean          repeat_background_set = false;
protected boolean          repeat_foreground_set = false;
protected boolean          traversable_set = false;
protected boolean          vertical_alignment_set = false;


protected CellStyleChangeSupport changeSupport;

/**
 * Creates a new <code>JCCellStyle</code> instance and sets default values for
 * all properties.
 */
public JCCellStyle()
{
    background = SystemColor.control;
    border = new JCCellBorder(JCTableEnum.BORDER_IN);
    border_sides = JCTableEnum.BORDERSIDE_ALL;
    cell_border_color = new ColorUIResource(Color.lightGray);
    cell_border_color_mode = JCTableEnum.BASE_ON_BACKGROUND;
    clip_hints = JCTableEnum.SHOW_ALL;
    data_type = null;
    editable = true;
    editor = null;
    foreground = SystemColor.controlText;
    font = new Font("Dialog", Font.PLAIN, 12);
    horizontal_alignment = JCTableEnum.LEFT;
    renderer = null;
    repeat_background = JCTableEnum.REPEAT_NONE;
    repeat_foreground = JCTableEnum.REPEAT_NONE;
    traversable = true;
    vertical_alignment = JCTableEnum.TOP;

    changeSupport = new CellStyleChangeSupport(this);
}

/**
 * Creates a new <code>JCCellStyle</code> instance with the given style as a
 * parent. All requests for property values will be forwarded to the parent
 * unless overriden in this style.
 * @param style
 */
public JCCellStyle(CellStyleModel style)
{
    parent_style = style;

    changeSupport = new CellStyleChangeSupport(this);
}

/**
 * Returns the parent style.
 * @return parent style
 */
public CellStyleModel getParentStyle()
{
    return parent_style;
}

/**
 * Sets the parent style.
 * @param parent
 */
public void setParentStyle(CellStyleModel parent)
{
    if (parent == this) {
        return;
    }
    CellStyleModel old_parent_style = parent_style;
    this.parent_style = parent;
    changeSupport.firePropertyChange(CellStyleChangeSupport.STYLE_PARENT,
                                     old_parent_style, parent_style);
}

/**
 * Returns the background color of the cell.
 * @return background color of the cell
 */
public Color getBackground()
{
    if (background == null && parent_style != null) {
        return parent_style.getBackground();
    }
    return background;
}

/**
 * Sets the background color of the cell.
 * @param value
 */
public void setBackground(Color value)
{
    Color old_background = background;
    this.background = value;
    changeSupport.firePropertyChange(CellStyleChangeSupport.STYLE_BACKGROUND,
                                     old_background, background);
}

/**
 * Returns the <code>CellBorderModel</code> object for the cell.
 * @return <code>CellBorderModel</code> object for the cell
 */
public CellBorderModel getCellBorder()
{
    if (border == null && parent_style != null) {
        return parent_style.getCellBorder();
    }
    return border;
}

/**
 * Sets the <code>CellBorder</code> object for the cell.
 * @param value
 */
public void setCellBorder(CellBorderModel value)
{
    CellBorderModel old_border = border;
    this.border = value;
    changeSupport.firePropertyChange(CellStyleChangeSupport.STYLE_BORDER,
                                     old_border, border);
}

/**
 * Returns the <code>CellBorder</code> color for the cell.
 * @return <code>CellBorder</code> color for the cell
 */
public Color getCellBorderColor()
{
    if (cell_border_color == null && parent_style != null) {
        return parent_style.getCellBorderColor();
    }
    return cell_border_color;
}

/**
 * Sets the <code>CellBorder</code> color for the cell.
 * @param value
 */
public void setCellBorderColor(Color value)
{
    Color old_cell_border_color = cell_border_color;
    this.cell_border_color = value;
    changeSupport.firePropertyChange(CellStyleChangeSupport.STYLE_CELL_BORDER_COLOR,
                                     old_cell_border_color, cell_border_color);
}

/**
 * Returns the mode used to determine how the cell border color is chosen.
 * @return mode used to determine how the cell border color is chosen
 */
public int getCellBorderColorMode()
{
    if (!cell_border_color_mode_set && parent_style != null) {
        return parent_style.getCellBorderColorMode();
    }

    return cell_border_color_mode;
}

/**
 * Sets the mode used to determine cell border colors.
 * <UL>
 *<LI><code>JCTableEnum.USE_CELL_BORDER_COLOR</code> - use the
 * <code>CellBorderColor</code>
 *<LI><code>JCTableEnum.BASE_ON_BACKGROUND</code> - determine color by the cell
 * background
 *<LI><code>JCTableEnum.BASE_ON_FOREGROUND</code> - determine color by the cell
 * foreground
 *</UL></PRE>
 * @param value
 */
public void setCellBorderColorMode(int value)
{
    int old_cell_border_color_mode = cell_border_color_mode;
    this.cell_border_color_mode = value;
    this.cell_border_color_mode_set = true;
    changeSupport.firePropertyChange(CellStyleChangeSupport.STYLE_CELL_BORDER_COLOR,
                                     new Integer(old_cell_border_color_mode),
                                     new Integer(cell_border_color_mode));
}

/**
 * Returns the border sides value for the cell.
 * @return border sides value for the cell
 */
public int getCellBorderSides()
{
    if (!border_sides_set && parent_style != null) {
        return parent_style.getCellBorderSides();
    }
    return border_sides;
}

/**
 * Sets the border sides for the cell. Valid values are
 * <CODE>BORDERSIDE_NONE</CODE>, <CODE>BORDERSIDE_ALL</CODE>, and combinations
 * of <CODE>BORDERSIDE_LEFT</CODE>, <CODE>BORDERSIDE_RIGHT</CODE>,
 * <CODE>BORDERSIDE_TOP</CODE>, and <CODE>BORDERSIDE_BOTTOM</CODE>.
 * @param value
 */
public void setCellBorderSides(int value)
{
    int old_border_sides = border_sides;
    this.border_sides = value;
    border_sides_set = true;
    changeSupport.firePropertyChange(CellStyleChangeSupport.STYLE_BORDER_SIDES,
                                     new Integer(old_border_sides),
                                     new Integer(border_sides));
}

/**
 * Gets the cell's editor. If null, JClass LiveTable maps the editor based on
 * the type of data.
 * @return
 */
public JCCellEditor getCellEditor()
{
    if (!editor_set && parent_style != null) {
        return parent_style.getCellEditor();
    }
    return editor;
}

/**
 * Sets the editor for a cell. If null, JClass LiveTable maps the editor based
 * on the type of data.
 * @param editor
 */
public void setCellEditor(JCCellEditor editor)
{
    JCCellEditor old_editor = editor;
    this.editor = editor;
    editor_set = true;
    changeSupport.firePropertyChange(CellStyleChangeSupport.STYLE_CELL_EDITOR,
                                     old_editor, editor);
}

/**
 * Gets the cell's renderer. If null, JClass LiveTable maps the renderer based
 * on the type of data.
 * @return
 */
public JCCellRenderer getCellRenderer()
{
    if (!renderer_set && parent_style != null) {
        return parent_style.getCellRenderer();
    }
    return renderer;
}

/**
 * Sets the renderer for a cell. If null, JClass LiveTable maps the renderer
 * based on the type of data.
 * @param renderer
 */
public void setCellRenderer(JCCellRenderer renderer)
{
    JCCellRenderer old_renderer = renderer;
    this.renderer = renderer;
    renderer_set = true;
    changeSupport.firePropertyChange(CellStyleChangeSupport.STYLE_CELL_RENDERER,
                                     old_renderer, renderer);
}

/**
 * Determines whether the renderer should draw some indication if the
 * entire contents of the cell cannot be rendered within the given area.
 * @return
 */
public int getClipHints()
{
    if (!clip_hints_set && parent_style != null) {
        return parent_style.getClipHints();
    }
    return clip_hints;
}

/**
 * Sets whether the renderer should draw some indication if the entire
 * contents of the cell cannot be rendered within the given area.
 * Valid values are <CODE>SHOW_NONE</CODE>, <CODE>SHOW_HORIZONTAL</CODE>,
 * <CODE>SHOW_VERTICAL</CODE>, and <CODE>SHOW_ALL</CODE>
 * @param value
 */
public void setClipHints(int value)
{
    int old_clip_hints = clip_hints;
    this.clip_hints = value;
    clip_hints_set = true;
    changeSupport.firePropertyChange(CellStyleChangeSupport.STYLE_CLIP_HINTS,
                                     new Integer(old_clip_hints),
                                     new Integer(clip_hints));
}

/**
 * Returns the cell's associated data type.
 * @see #setDataType
 * @return cell's associated data type
 */
public Class getDataType()
{
    if (!data_type_set && parent_style != null) {
        return parent_style.getDataType();
    }
    return data_type;
}

/**
 * Sets the cell's associated data type. The data type is used to
 * determine the renderer/editor in the case where the data source
 * returns a null value.
 * @param value
 */
public void setDataType(Class value)
{
    Class old_data_type = data_type;
    this.data_type = value;
    data_type_set = true;
    changeSupport.firePropertyChange(CellStyleChangeSupport.STYLE_DATA_TYPE,
                                     old_data_type, data_type);
}

/**
 * Returns <code>true</code> if the cell is editable.
 * @return <code>true</code> if cell is editable
 */
public boolean isEditable()
{
    if (!editable_set && parent_style != null) {
        return parent_style.isEditable();
    }
    return editable;
}

/**
 * Sets whether the cell is editable.
 * @param value
 */
public void setEditable(boolean value)
{
    boolean old_editable = editable;
    this.editable = value;
    editable_set = true;
    changeSupport.firePropertyChange(CellStyleChangeSupport.STYLE_EDITABLE,
                                     new Boolean(old_editable),
                                     new Boolean(editable));
}

/**
 * Returns the font of the cell.
 * @return font of the cell
 */
public Font getFont()
{
    if (font == null && parent_style != null) {
        return parent_style.getFont();
    }
    return font;
}

/**
 * Sets the font used for drawing text within the cell.
 * @param value
 */
public void setFont(Font value)
{
    Font old_font = font;
    this.font = value;
    changeSupport.firePropertyChange(CellStyleChangeSupport.STYLE_FONT,
                                     old_font, font);
}

/**
 * Returns the foreground color of the cell.
 * @return foreground color of the cell
 */
public Color getForeground()
{
    if (foreground == null && parent_style != null) {
        return parent_style.getForeground();
    }
    return foreground;
}

/**
 * Sets the foreground color of the cell.
 * @param value
 */
public void setForeground(Color value)
{
    Color old_foreground = foreground;
    this.foreground = value;
    changeSupport.firePropertyChange(CellStyleChangeSupport.STYLE_FOREGROUND,
                                     old_foreground, foreground);
}

/**
 * Returns the horizontal alignment for the cell.
 * Valid return values are <CODE>LEFT</CODE>, <CODE>CENTER</CODE>, and
 * <CODE>RIGHT</CODE>.
 * @return horizontal alignment for the cell
 */
public int getHorizontalAlignment()
{
    if (!horizontal_alignment_set && parent_style != null) {
        return parent_style.getHorizontalAlignment();
    }
    return horizontal_alignment;
}

/**
 * Sets the horizontal alignment for the cell.
 * @param value
 */
public void setHorizontalAlignment(int value)
{
    int old_horizontal_alignment = horizontal_alignment;
    this.horizontal_alignment = value;
    horizontal_alignment_set = true;
    changeSupport.firePropertyChange(CellStyleChangeSupport.STYLE_HORIZONTAL_ALIGNMENT,
                                     new Integer(old_horizontal_alignment),
                                     new Integer(horizontal_alignment));
}

/**
 * Returns the background repeat flag.
 * @return background repreat flag
 */
public int getRepeatBackground()
{
    if (!repeat_background_set && parent_style != null) {
        return parent_style.getRepeatBackground();
    }
    return repeat_background;
}

/**
 * Sets whether the background color uses the
 * <CODE>BackgroundRepeatColors</CODE>
 * for rows or columns. Valid values are <CODE>JCTableEnum.REPEAT_NONE</CODE>,
 * <CODE>JCTableEnum.REPEAT_ROW</CODE>, or
 * <CODE>JCTableEnum.REPEAT_COLUMN</CODE>.
 * @param value
 */
public void setRepeatBackground(int value)
{
    int old_repeat_background = repeat_background;
    this.repeat_background = value;
    repeat_background_set = true;
    changeSupport.firePropertyChange(CellStyleChangeSupport.STYLE_REPEAT_BACKGROUND,
                                     new Integer(old_repeat_background),
                                     new Integer(repeat_background));
}

/**
 * Gets the table's <CODE>RepeatBackgroundColors</CODE> value.
 * @see #setRepeatBackgroundColors
 * @return
 */
public Color[] getRepeatBackgroundColors()
{
    if (repeat_background_colors == null && parent_style != null) {
        return parent_style.getRepeatBackgroundColors();
    }
    return repeat_background_colors;
}

/**
 * Sets an array of colors to be used when <CODE>Background</CODE> value is
 * <CODE>JCTableEnum.REPEAT_ROW</CODE> or
 * <CODE>JCTableEnum.REPEAT_COLUMN</CODE>.
 * @param values
 */
public void setRepeatBackgroundColors(Color[] values)
{
    repeat_background_colors = values;
}

/**
 * Returns the foreground repeat flag.
 * @return foreground repeat flag
 */
public int getRepeatForeground()
{
    if (!repeat_foreground_set && parent_style != null) {
        return(parent_style.getRepeatForeground());
    }
    return(repeat_foreground);
}

/**
 * Sets whether the foreground color uses the
 * <CODE>ForegroundRepeatColors</CODE>
 * for rows or columns. Valid values are <CODE>JCTableEnum.REPEAT_NONE</CODE>,
 * <CODE>JCTableEnum.REPEAT_ROW</CODE>, or
 * <CODE>JCTableEnum.REPEAT_COLUMN</CODE>.
 * @param value The new repeatForeground value
 */
public void setRepeatForeground(int value)
{
    int old_repeat_foreground = repeat_foreground;
    this.repeat_foreground = value;
    repeat_foreground_set = true;
    changeSupport.firePropertyChange(CellStyleChangeSupport.STYLE_REPEAT_FOREGROUND,
                                     new Integer(old_repeat_foreground),
                                     new Integer(repeat_foreground));
}

/**
 * Gets the table's <CODE>RepeatForegroundColors</CODE> value.
 * @see #setRepeatForegroundColors
 * @return
 */
public Color[] getRepeatForegroundColors()
{
    if (repeat_foreground_colors == null && parent_style != null) {
        return(parent_style.getRepeatForegroundColors());
    }
    return(repeat_foreground_colors);
}

/**
 * Sets an array of colors to be used when <CODE>Foreground</CODE> value is
 * <CODE>JCTableEnum.REPEAT_ROW</CODE> or
 * <CODE>JCTableEnum.REPEAT_COLUMN</CODE>.
 * @param values
 */
public void setRepeatForegroundColors(Color[] values)
{
    repeat_foreground_colors = values;
}

/**
 * Returns <code>true</code> if the cell is traversable.
 * @return <code>true</code> if cell is traversable
 */
public boolean isTraversable()
{
    if (!traversable_set && parent_style != null) {
        return(parent_style.isTraversable());
    }
    return(traversable);
}

/**
 * Sets whether the cell is traversable.
 * @param value
 */
public void setTraversable(boolean value)
{
    boolean old_traversable = traversable;
    this.traversable = value;
    traversable_set = true;
    changeSupport.firePropertyChange(CellStyleChangeSupport.STYLE_TRAVERSABLE,
                                     new Boolean(old_traversable),
                                     new Boolean(traversable));
}

/**
 * Returns the vertical alignment for the cell.
 * Valid return values are <CODE>TOP</CODE>, <CODE>CENTER</CODE>, and
 * <CODE>BOTTOM</CODE>.
 * @return vertical alignment for the cell; one of <code>TOP</code>,
 * <code>CENTER</code>, or <code>BOTTOM</code>
 */
public int getVerticalAlignment()
{
    if (!vertical_alignment_set && parent_style != null) {
        return parent_style.getVerticalAlignment();
    }
    return vertical_alignment;
}

/**
 * Sets the vertical alignment for the cell.
 * @param value
 */
public void setVerticalAlignment(int value)
{
    int old_vertical_alignment = vertical_alignment;
    this.vertical_alignment = value;
    vertical_alignment_set = true;
    changeSupport.firePropertyChange(CellStyleChangeSupport.STYLE_VERTICAL_ALIGNMENT,
                                     new Integer(old_vertical_alignment),
                                     new Integer(vertical_alignment));
}

public Object clone()
{
    JCCellStyle style = new JCCellStyle();
    copyInto(style);
    return style;
}

public void copyInto(JCCellStyle style)
{
    style.parent_style = this.parent_style;
    style.background = this.background;
    style.border = this.border;
    style.border_sides = this.border_sides;
    style.cell_border_color = this.cell_border_color;
    style.cell_border_color_mode = this.cell_border_color_mode;
    style.clip_hints = this.clip_hints;
    style.data_type = this.data_type;
    style.editable = this.editable;
    style.editor = this.editor;
    style.foreground = this.foreground;
    style.font = this.font;
    style.horizontal_alignment = this.horizontal_alignment;
    style.renderer = this.renderer;
    style.repeat_background = this.repeat_background;

    if (this.repeat_background_colors != null) {
        int length = this.repeat_background_colors.length;
        style.repeat_background_colors = new Color[length];
        for (int c = 0; c < length; c++) {
            style.repeat_background_colors[c] =
                this.repeat_background_colors[c];
        }
    }

    style.repeat_foreground = this.repeat_foreground;

    if (this.repeat_foreground_colors != null) {
        int length = this.repeat_foreground_colors.length;
        style.repeat_foreground_colors = new Color[length];
        for (int c = 0; c < length; c++) {
            style.repeat_foreground_colors[c] =
                this.repeat_foreground_colors[c];
        }
    }

    style.traversable = this.traversable;
    style.vertical_alignment = this.vertical_alignment;

    style.border_sides_set = this.border_sides_set;
    style.cell_border_color_mode_set = this.cell_border_color_mode_set;
    style.clip_hints_set = this.clip_hints_set;
    style.data_type_set = this.data_type_set;
    style.editable_set = this.editable_set;
    style.editor_set = this.editor_set;
    style.horizontal_alignment_set = this.horizontal_alignment_set;
    style.renderer_set = this.renderer_set;
    style.repeat_background_set = this.repeat_background_set;
    style.repeat_foreground_set = this.repeat_foreground_set;
    style.traversable_set = this.traversable_set;
    style.vertical_alignment_set = this.vertical_alignment_set;
}

/**
 * Adds a <CODE>PropertyChangeListener</CODE> for <code>CellStyle</code>
 * property changes.
 * <CODE>CellStyleChangeSupport</CODE> is a utility class that can be
 * used to manage listeners and fire <CODE>PropertyChange</CODE> events.
 * @param l The property change listener to add
 */
public void addPropertyChangeListener(PropertyChangeListener l)
{
    changeSupport.addPropertyChangeListener(l);
}

/**
 * Removes a <CODE>PropertyChangeListener</CODE> for <code>CellStyle</code>
 * property changes.
 * @param l The property change listener to remove
 */
public void removePropertyChangeListener(PropertyChangeListener l)
{
    changeSupport.removePropertyChangeListener(l);
}

}
