/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCEditCellEvent.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:33 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import java.awt.*;
import java.util.EventObject;

/**
 * The event posted before and after editing a cell.
 * @see JCEditCellAdapter
 * @see JCEditCellListener
 */
public class JCEditCellEvent extends EventObject {

/** Types of <code>JCEnterCellEvents</code>. */
public static final int BEFORE_EDIT_CELL = 1;
public static final int EDIT_CELL = 2;
public static final int AFTER_EDIT_CELL = 3;

protected int           type;
protected int           row;
protected int           column;
protected Component     editComponent = null;
protected boolean       cancelled = false;

public JCEditCellEvent(Object source, int type, int row, int column)
{
    super(source);
    this.type = type;
    this.row = row;
    this.column = column;
}

/**
 * Returns the row of the cell being traversed to.
 * @return row of the cell being traversed to
 */
public int getRow()
{
    return(row);
}

/**
 * Returns the column of the cell being traversed to.
 * @return the column of the cell being traversed to
 */
public int getColumn()
{
    return(column);
}

/**
 * Returns the editing component.
 * @return editing component
 */
public Component getEditingComponent()
{
    return(editComponent);
}

/**
 * Gets the event type.
 * @return event type
 */
public int getType()
{
    return(type);
}

/**
 * Gets the <CODE>Cancelled</CODE> value.
 * @see #setCancelled
 * @return <code>Cancelled</code> value
 */
public boolean isCancelled()
{
    return(cancelled);
}

/**
 * Determines whether to allow an interactive resize (default is
 * <code>true</code>).
 * @param value
 */
public void setCancelled(boolean value)
{
    if (type != BEFORE_EDIT_CELL) {
        throw new ImmutableEventException("setCancelled is only available " +
                                          "in beforeEditCell");
    }
    this.cancelled = value;
}

/**
 * Returns a String representation of the event.
 * @return String representation of the event
 */
public String toString()
{
    return("EDIT [" +
           (type == BEFORE_EDIT_CELL ? "beforeEditCell: "
                                     : (type == EDIT_CELL ? "editCell: "
                                                          : "afterEditCell: ")) +
           " cancelled=" + isCancelled() + " R" + row + "C" + column + "]");
}

}
