/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCPaintEvent.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:33 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import java.util.EventObject;

/**
 * The event posted before and after a portion of the table is 
 * painted (whenever the table redraws or modifies an onscreen area).
 * @see JCPaintListener
 */
public class JCPaintEvent extends EventObject {

public static final int BEFORE_PAINT = 0;
public static final int AFTER_PAINT = 1;

protected int           type;
protected int           start_row, start_column, end_row, end_column;

public JCPaintEvent(Object source, int type,
                    int start_row, int start_column,
                    int end_row, int end_column)
{
    super(source);
    this.type = type;
    this.start_row = start_row;
    this.start_column = start_column;
    this.end_row = end_row;
    this.end_column = end_column;
}

/**
 * Gets the event type.
 * @return
 */
public int getType()
{
    return(type);
}

/**
 * Gets the row of the first cell in the range (top-left corner).
 * @return
 */
public int getStartRow()
{
    return(start_row);
}

/**
 * Gets the column of the first cell in the range (top-left corner).
 * @return
 */
public int getStartColumn()
{
    return(start_column);
}

/**
 * Gets the row of the last cell in the range (top-left corner).
 * @return
 */
public int getEndRow()
{
    return(end_row);
}

/**
 * Gets the column of the last cell in the range (top-left corner).
 * @return
 */
public int getEndColumn()
{
    return(end_column);
}

/**
 * Returns a <code>JCCellRange</code> containing the painted area.
 * <br><B>Note:</b> This method requires the creation of a
 * <code>JCCellRange</code> object. It will be more efficient
 * to call the individual get methods.
 * @return <code>JCCellRange</code> containing the painted area
 */
public JCCellRange getCellRange()
{
    return(new JCCellRange(start_row, start_column, end_row, end_column));
}

/**
 * Returns a String representation of the event.
 * @return String representation of the event
 */
public String toString()
{
    return("PAINT [" +
           (type == BEFORE_PAINT ? "beforePaint: " : "afterPaint:  ") +
           "R" + start_row + "C" + start_column + ":R" + end_row +
           "C" + end_column + "]");
}

}


