/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCPrintEvent.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:33 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Insets;
import java.util.EventObject;

/**
 * The event posted for each page during printing.
 * @see JCPrintListener
 */
public class JCPrintEvent extends EventObject {

public static final int PRINT_HEADER = 1;
public static final int PRINT_BODY = 2;
public static final int PRINT_FOOTER = 3;

protected Graphics     gc;
protected int          page;
protected int          numPages;
protected int          type;
protected JCPrintTable printTable;

public JCPrintEvent(Object source, Graphics gc, int page, int numPages,
                    int type, JCPrintTable printTable)
{
    super(source);
    this.gc = gc;
    this.page = page;
    this.numPages = numPages;
    this.type = type;
    this.printTable = printTable;
}

/**
 * Gets the graphics for the current page.
 * @return
 */
public Graphics getGraphics()
{
    return(gc);
}

/**
 * Gets the units for margins. Valid unit are
 * <code>JCPrintTable.MARGIN_IN_INCHES</code> and
 * <code>JCPrintTable.MARGIN_IN_PIXELS</code>.
 * @return
 */
public int getMarginUnits()
{
    return(printTable.getMarginUnits());
}

/**
 * Gets the number of horizontal pages required to print all of the columns
 * in the table.
 * @return
 */
public int getNumHorizontalPages()
{
    return(printTable.getNumHorizontalPages());
}

/**
 * Gets the number of pages required to print the table.
 * @return
 */
public int getNumPages()
{
    return(numPages);
}

/**
 * Gets the number of vertical pages required to print all of the rows
 * in the table. Use <CODE>getNumPages</CODE> to obtain the total number
 * of pages required to print the entire table.
 * @return
 */
public int getNumVerticalPages()
{
    return(printTable.getNumVerticalPages());
}

/**
 * Gets the page number.
 * @return
 */
public int getPage()
{
    return(page);
}

/**
 * Gets the page dimensions.
 * @return
 */
public Dimension getPageDimensions()
{
    return(printTable.getPageDimensions());
}

/**
 * Gets the page margins.
 * @return
 */
public Insets getPageMargins()
{
    return(printTable.getPageMargins());
}

/**
 * Gets the page resolution.
 * @return
 */
public int getPageResolution()
{
    return(printTable.getPageResolution());
}

/**
 * Gets the dimension required to print the table on the current page.
 * @return
 */
public Dimension getTableDimensions()
{
    return(printTable.getTableDimensions(page - 1));
}

/**
 * Gets the event type.
 * @return
 */
public int getType()
{
    return(type);
}

}
