/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCResizeCellAdapter.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:33 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

/**
 * The adapter which receives <CODE>JCResizeCell</CODE> events.
 * The methods in this class are empty; this class is provided as a
 * convenience for easily creating listeners by extending this class
 * and overriding only the methods of interest.
 * @see JCResizeCellEvent
 */
public abstract class JCResizeCellAdapter implements JCResizeCellListener
{

/**
 * Invoked before the row and/or column is resized. A
 * <code>JCResizeCellEvent</code> isn't generated until the mouse moves. The
 * event's <CODE>setCancelled</CODE> method can be called to prevent the resize.
 * @see JCResizeCellEvent#setCancelled
 * @param e The resize cell event
 */    
public void beforeResizeCell(JCResizeCellEvent e) {}

/**
 * Invoked after the mouse is released. The event's <CODE>setCancelled</CODE>
 * method can be called to prevent the resize and the new width or height can
 * be changed by calling <code>setNewColumnWidth</code> or
 * <code>setNewRowHeight</code>. The width or height is not updated until all
 * listeners have received the event.
 * @see JCResizeCellEvent#setCancelled
 * @see JCResizeCellEvent#setNewRowHeight
 * @see JCResizeCellEvent#setNewColumnWidth
 * @param e The resize cell event
 */
public void resizeCell(JCResizeCellEvent e) {}

/**
 * Invoked after the row and/or column is resized and all listeners have been
 * notified of the new value through <CODE>resizeCell</CODE>. This method is
 * called after the new values has been set. If a listener cancelled the resize
 * in the <CODE>beforeResizeCell</CODE> call, <code>resizeCellEnd</code> will
 * be called with the cancelled flag set.
 * @see #resizeCell
 * @see JCResizeCellEvent#isCancelled
 * @param e The resize cell event
 */    
public void afterResizeCell(JCResizeCellEvent e) {}

}
