/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCResizeCellEvent.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import java.util.EventObject;

/**
 * The event posted as a user resizes a row and/or column.
 * @see Table#addResizeCellListener
 * @see JCResizeCellAdapter
 * @see JCResizeCellListener
 */
public class JCResizeCellEvent extends EventObject
{

public static final int BEFORE_RESIZE = 1;
public static final int RESIZE        = 2;
public static final int RESIZE_DRAG   = 3;
public static final int AFTER_RESIZE  = 4;

protected int           type;
protected int           row;
protected int           column;
protected int           current_row_height;
protected int           current_column_width;
protected int           new_row_height;
protected int           new_column_width;
protected boolean       cancelled = false;

public JCResizeCellEvent(Object source, int type, int row, int column, 
		                 int current_width, int current_height)
{
	super(source);
	this.type = type;
	this.row = row;
	this.column = column;
	
	this.current_row_height = current_height;
	this.current_column_width = current_width;
	
	this.new_row_height = JCTableEnum.NOVALUE;
	this.new_column_width = JCTableEnum.NOVALUE;
}

/**
 * Gets the row being resized.
 * @return <code>JCTblEnum.NOVALUE</code> if only a column is being resized
 */
public int getRow()
{
	return(row);
}

/**
 * Gets the column being resized.
 * @return <code>JCTableEnum.NOVALUE</code> if only a row is being resized
 */
public int getColumn()
{
	return(column);
}

/**
 * Gets the current row height. 
 * @return <code>JCTableEnum.NOVALUE</code> if only a column is being resized
 */
public int getCurrentRowHeight()
{
	return(current_row_height);
}

/**
 * Gets the new row height. 
 * @return <code>JCTableEnum.NOVALUE</code> if only a column is being resized
 */
public int getNewRowHeight()
{
	return(new_row_height);
}

/**
 * Sets the new row height. 
 * This has no effect if only a column is being resized.
 * @param value The new row height
 */
public void setNewRowHeight(int value)
{
	if (type == BEFORE_RESIZE || type == AFTER_RESIZE) {
		throw new ImmutableEventException("setNewRowHeight is only available " +
                                          "in resizeCell/resizeCellDragged");
	}
	new_row_height = value;
}

/**
 * Gets the current column width. 
 * @return <code>JCTableEnum.NOVALUE</code> if only a row is being resized
 */
public int getCurrentColumnWidth()
{
	return current_column_width;
}

/**
 * Gets the new column width. 
 * @return <code>JCTableEnum.NOVALUE</code> if only a row is being resized
 */
public int getNewColumnWidth()
{
	return new_column_width;
}

/**
 * Sets the new column width. 
 * This has no effect if only a row is being resized.
 * @param value The new column width.
 */
public void setNewColumnWidth(int value)
{
	if(type == BEFORE_RESIZE || type == AFTER_RESIZE) {
		throw new ImmutableEventException("setNewColumnWidth is only " +
                                          "available in " +
                                          "resizeCell/resizeCellDragged");
	}
	new_column_width = value;
}

/**
 * Gets the <CODE>Cancelled</CODE> value.
 * @see #setCancelled
 * @return The current <CODE>Cancelled</CODE> value.
 */
public boolean isCancelled()
{
	return(cancelled);
}

/**
 * Determines whether to allow an interactive resize (default is
 * <code>true</code>).
 * @param value The new <CODE>Cancelled</CODE> value
 */
public void setCancelled(boolean value)
{
	if(type == AFTER_RESIZE) {
		throw new ImmutableEventException("setCancelled is not available " +
                                          "in afterResize");
	}
	this.cancelled = value;
}

/**
 * Gets the event type.
 * @return The event type
 */
public int getType()
{
	return(type);
}

/**
 * Returns a String representation of the event.
 * @return String representation of the event
 */
public String toString()
{
	String s = "RESIZE [";
	if(type == BEFORE_RESIZE) {
		s += "beforeResize ";
	}
	else if(type == RESIZE) {
		s += "resize ";
	}
	else if(type == RESIZE_DRAG) {
		s += "resizeDrag ";
	}
	else if(type == AFTER_RESIZE) {
		s += "afterResize ";
	}
	return(s + "R" + row + "C" + column + ", cancelled=" + isCancelled() + " " +
		   current_column_width + "x" + current_row_height + " to " +
		   new_column_width + "x" + new_row_height + "]");
}

}
