/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCScrollEvent.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import java.awt.Adjustable;
import java.awt.event.AdjustmentEvent;
import java.util.EventObject;

/**
 * The event posted before and after the table is scrolled.
 * @see Table#addScrollListener
 * @see JCScrollAdapter
 * @see JCScrollListener
 */
public class JCScrollEvent extends EventObject {

public static final int SCROLL = 1;
public static final int AFTER_SCROLL = 2;

protected Adjustable       adjustable;
protected int              type;
protected int              value;
protected AdjustmentEvent  event;
protected int              direction;

public JCScrollEvent(Object source, int type, Adjustable adjustable,
                     int direction, int value, AdjustmentEvent event)
{
    super(source);
    this.type = type;
    this.adjustable = adjustable;
    this.direction = direction;
    this.value = value;
    this.event = event;
}

/**
 * Gets the affected adjustable object.
 * @return
 */
public Adjustable getAdjustable()
{
    return(adjustable);
}

/**
 * Gets the scrolling direction. One of <CODE>Adjustable.HORIZONTAL</CODE>
 * or <CODE>Adjustable.VERTICAL</CODE>.
 * @return
 */
public int getDirection()
{
    return(direction);
}

/**
 * Gets the event which initiated the action.
 * @return
 */
public AdjustmentEvent getEvent()
{
    return(event);
}

/**
 * Gets the event type.
 * @return
 */
public int getType()
{
    return(type);
}

/**
 * Gets the scrollbar's current value.
 * @return
 */
public int getValue()
{
    return(value);
}

/**
 * Sets the scrollbar's current value.
 * @param value
 */
public void setValue(int value)
{
    if (type == AFTER_SCROLL) {
        throw new ImmutableEventException("setValue is only available in " +
                                          "scroll");
    }
    this.value = value;
}

/**
 * Returns a String representation of the event.
 * @return String representation of the event
 */
public String toString()
{
    return("SCROLL [" + (type == SCROLL ? "scroll: " : "after:  ") +
           "value: " + value + "]");
}

}
