/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCSelectEvent.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import java.util.EventObject;

/**
 * The event posted as a user selects cells or cells are
 * selected programmatically.
 */
public class JCSelectEvent extends EventObject {

public static final int BEFORE_SELECTION = 1;
public static final int SELECTION = 2;
public static final int AFTER_SELECTION = 3;

public static final int SELECT = SelectionModel.SELECT;
public static final int EXTEND = SelectionModel.EXTEND;
public static final int ADD = SelectionModel.ADD;
public static final int END = SelectionModel.END;

public static final int DESELECT = SelectionModel.DESELECT;

protected int           type;
protected int           start_row;
protected int           start_column;
protected int           end_row;
protected int           end_column;
protected int           action;
protected boolean       cancelled = false;

public JCSelectEvent(Object source, int type,
                     int start_row, int start_column,
                     int end_row, int end_column, int action)
{
    super(source);
    this.type = type;
    this.start_row = start_row;
    this.start_column = start_column;
    this.end_row = end_row;
    this.end_column = end_column;
    this.action = action;
}

/**
 * Returns the type of event. Valid types are <CODE>EVENT_BEGIN</CODE> or
 * <CODE>EVENT_END</CODE>.
 * @return type of event; one of <code>EVENT_BEGIN</CODE> or
 * <code>EVENT_END</code>
 */
public int getType()
{
    return(type);
}

/**
 * Returns the start row of the selected or deselected cell range.
 * @return start row of the selected or deselected cell range
 */
public int getStartRow()
{
    return(start_row);
}

/**
 * Returns the start column of the selected or deselected cell range.
 * @return start column of the selected or deselected cell range
 */
public int getStartColumn()
{
    return(start_column);
}

/**
 * Returns the end row of the selected or deselected cell range.
 * @return end row of the selected or deselected cell range
 */
public int getEndRow()
{
    return(end_row);
}

/**
 * Returns the end column of the selected or deselected cell range.
 * @return end column of the selected or deselected cell range
 */
public int getEndColumn()
{
    return(end_column);
}

/**
 * Returns <code>true</code> if any listener has rejected the selection.
 * @return <code>true</code> if any listener has rejected the selection
 */
public boolean isCancelled()
{
    return(cancelled);
}

/**
 * Sets whether the selection (or unselection) should be cancelled. Default is
 * <code>false</code>.
 * @param value
 */
public void setCancelled(boolean value)
{
    if (type == AFTER_SELECTION) {
        throw new ImmutableEventException("setCancelled is not available " +
                                          "in afterSelect");
    }
    this.cancelled = value;
}

/**
 * Returns the type of action where valid values are: <CODE>SELECT</CODE>,
 * <CODE>ADD</CODE>, <CODE>EXTEND</CODE>, <CODE>DESELECT</CODE>, and
 * <CODE>END</CODE>.
 * @return type of action; one of <CODE>SELECT</CODE>,
 * <CODE>ADD</CODE>, <CODE>EXTEND</CODE>, <CODE>DESELECT</CODE>, or
 * <CODE>END</CODE>
 */
public int getAction()
{
    return(action);
}

/**
 * Returns a String representation of the action.
 * @return String representation of the action
 */
public String getActionString()
{
    switch (action) {
    case SELECT:
        return("SELECT");
    case ADD:
        return("ADD");
    case EXTEND:
        return("EXTEND");
    case DESELECT:
        return("DESELECT");
    case END:
        return("END");
    }
    return("");
}

/**
 * Returns a String representation of the event.
 * @return String representation of the event
 */
public String toString()
{
    return("SELECT [" +
           (type == BEFORE_SELECTION ? "beforeSelect: "
                                     : (type == SELECTION ? "select: "
                                                          : "afterSelect: ")) +
           "R" + start_row + "C" + start_column + ":R" + end_row + ":C" +
           end_column + ", action:" + getActionString() + "]");
}

}
