/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCSeries.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import java.io.Serializable;
import java.util.Collection;
import java.util.List;
import java.util.Vector;

/**
 * <CODE>SeriesModel</CODE> defines the interface required by classes that can
 * store row/column based properties (for example: editors, renderers, styles).
 */
public class JCSeries extends Vector implements SeriesModel, Serializable {

/** The index of the last found cell. */
protected int last_index = JCTableEnum.NOTFOUND;

/**
 * Creates an empty series with no default.
 */
public JCSeries()
{
    super();
}

/**
 * Create a new series with the specified default.
 * @param value
 */
public JCSeries(Object value)
{
    super();
    setDefault(value);
}

/**
 * Gets the default member value. The default value is always equivalent to the
 * (<i>ALL</i>, <i>ALL</i>) cell position.
 * @return
 */
public Object getDefault()
{
    return getValue(JCTableEnum.ALL, JCTableEnum.ALL);
}

/**
 * Creates or sets the default member to the specified value. The default value
 * is always equivalent to the (<i>ALL</i>, <i>ALL</i>) cell position.
 * @param value
 */
public void setDefault(Object value)
{
    last_index = find(JCTableEnum.ALL, JCTableEnum.ALL);
    if (last_index == JCTableEnum.NOTFOUND) {
        SeriesValue sv = new SeriesValue(JCTableEnum.ALL, JCTableEnum.ALL,
                                         value);
        insertElementAt(sv, 0);
    }
    else {
        ((SeriesValue)elementAt(last_index)).value = value;
    }
}

/**
 * Returns a Vector of unique objects used in <code>JCTable</code>.
 * @return Vecotor of unique objects used in <code>JCTable</code>
 */
public Collection getUniqueValues()
{
    Vector v = new Vector();

    // Search from end of list
    for (int i = size() - 1; i >= 0; i--) {
        Object value = ((SeriesValue)elementAt(i)).value;
        if (!v.contains(value)) {
            v.addElement(value);
        }
    }

    return v;
}

/**
 * Returns a Vector of <code>JCCellRanges</code> that contain the given object.
 * @param value
 * @return Vector of <code>JCCellRanges</code>
 */
public Collection getCellRanges(Object value)
{
    Vector v = new Vector();

    int sr = JCTableEnum.NOVALUE;
    int sc = JCTableEnum.NOVALUE;
    int er = JCTableEnum.NOVALUE;
    int ec = JCTableEnum.NOVALUE;

    for (int i = 0; i < size(); i++) {
        SeriesValue sv = (SeriesValue)elementAt(i);
        if (sv.value == value) {
            if (sv.row == JCTableEnum.ALL) {
                sr = JCTableEnum.LABEL;
                er = JCTableEnum.MAXINT;
            }
            else if (sv.row == JCTableEnum.ALLCELLS) {
                sr = 0;
                er = JCTableEnum.MAXINT;
            }
            else {
                sr = sv.row;
                er = sv.row;
            }
            if (sv.column == JCTableEnum.ALL) {
                sc = JCTableEnum.LABEL;
                ec = JCTableEnum.MAXINT;
            }
            else if (sv.column == JCTableEnum.ALLCELLS) {
                sc = 0;
                ec = JCTableEnum.MAXINT;
            }
            else {
                sc = sv.column;
                ec = sv.column;
            }
            v.addElement(new CellRangeValue(sr, sc, er, ec, sv.value));
        }
    }
    v = compressRanges(v);
    return v;
}

/**
 * Returns an iterator for an ordered set of series values.
 * @return iterator for an ordered set of series values
 */
public List getSeriesValues()
{
    Vector v = new Vector();

    int sr = JCTableEnum.NOVALUE;
    int sc = JCTableEnum.NOVALUE;
    int er = JCTableEnum.NOVALUE;
    int ec = JCTableEnum.NOVALUE;

    for (int i = 0; i < size(); i++) {
        SeriesValue sv = (SeriesValue)elementAt(i);
        if (sv.row == JCTableEnum.ALL) {
            sr = JCTableEnum.LABEL;
            er = JCTableEnum.MAXINT;
        }
        else if (sv.row == JCTableEnum.ALLCELLS) {
            sr = 0;
            er = JCTableEnum.MAXINT;
        }
        else {
            sr = sv.row;
            er = sv.row;
        }
        if (sv.column == JCTableEnum.ALL) {
            sc = JCTableEnum.LABEL;
            ec = JCTableEnum.MAXINT;
        }
        else if (sv.column == JCTableEnum.ALLCELLS) {
            sc = 0;
            ec = JCTableEnum.MAXINT;
        }
        else {
            sc = sv.column;
            ec = sv.column;
        }
        v.addElement(new CellRangeValue(sr, sc, er, ec, sv.value));
    }
    return compressRanges(v);
}

/**
 * A recursive method that combines adjacent <code>JCCellRanges</code> in
 * a Vector of <code>JCCellRanges</code>.
 * @param input
 * @return
 */
private Vector compressRanges(Vector input)
{
    if (input.size() == 1) {
        return input;
    }

    CellRangeValue cr1, cr2;
    for (int i = input.size() - 1; i > 0; i--) {
        cr2 = (CellRangeValue)input.elementAt(i);
        for (int j = i; j > 0; j--) {
            cr1 = (CellRangeValue)input.elementAt(j - 1);
            if (isCombinable(cr1, cr2)) {
                addRange(cr1, cr2);
                input.removeElementAt(i);
                return compressRanges(input);
            }
        }
    }
    return input;
}

/**
 * Returns <code>true</code> if the addition of two cell ranges makes a valid
 * range that contains no extra cells.
 * @return <code>true</code> if the addition of two cell ranges make a valid
 * range
 * @param cr1
 * @param cr2
 */
private boolean isCombinable(CellRangeValue cr1, CellRangeValue cr2)
{
    if (cr1.value != cr2.value) {
        return(false);
    }
    if (cr1.start_row == cr2.start_row &&
        cr1.end_row == cr2.end_row &&
        (cr1.end_column + 1 == cr2.start_column ||
         cr2.end_column + 1 == cr1.start_column))
    {
        return(true);
    }
    else if (cr1.start_column == cr2.start_column &&
             cr1.end_column == cr2.end_column &&
             (cr1.end_row + 1 == cr2.start_row ||
              cr2.end_row + 1 == cr1.start_row))
    {
        return(true);
    }
    return(false);
}

/**
 * Makes the first cell range a combination of itself with the second.
 * @param cr1
 * @param cr2
 */
private void addRange(CellRangeValue cr1, CellRangeValue cr2)
{
    cr1.start_row = Math.min(cr1.start_row, cr2.start_row);
    cr1.start_column = Math.min(cr1.start_column, cr2.start_column);
    cr1.end_row = Math.max(cr1.end_row, cr2.end_row);
    cr1.end_column = Math.max(cr1.end_column, cr2.end_column);
}

/**
 * Returns the value at the specified cell, or within the cell's row or column.
 * @return null if the series is empty
 * @param column
 * @param row
 */
public Object getValue(int row, int column)
{
    int pos = find(row, column);
    return((pos == JCTableEnum.NOTFOUND) ? null
                                         : ((SeriesValue)elementAt(pos)).value);
}

/**
 * Sets the specified cell to the value.
 * @return <code>true</code> if the series was changed after adding the value
 * @param column
 * @param row
 * @param value
 */
public boolean setValue(int row, int column, Object value)
{
    if (row == JCTableEnum.LABEL && column == JCTableEnum.LABEL) {
        boolean b1 = setValue(JCTableEnum.ALL, JCTableEnum.LABEL, value);
        boolean b2 = setValue(JCTableEnum.LABEL, JCTableEnum.ALL, value);
        return b1 & b2;
    }

    if (row == JCTableEnum.ALL && column == JCTableEnum.ALL) {
        removeAllElements();
        last_index = JCTableEnum.NOTFOUND;
    }
    else if (row == JCTableEnum.ALLCELLS && column == JCTableEnum.ALLCELLS) {
        for (int e = size() - 1; e >= 0; e--) {
            SeriesValue sv = (SeriesValue)elementAt(e);
            if ((sv.column == JCTableEnum.ALLCELLS ||
                 sv.column > JCTableEnum.LABEL) &&
                (sv.row == JCTableEnum.ALLCELLS ||
                 sv.row > JCTableEnum.LABEL))
            {
                removeElementAt(e);
            }
        }
        last_index = JCTableEnum.NOTFOUND;
    }
    else if (row == JCTableEnum.ALL) {
        for (int e = size() - 1; e >= 0; e--) {
            SeriesValue sv = (SeriesValue)elementAt(e);
            if (sv.column == column) {
                removeElementAt(e);
            }
        }
        last_index = JCTableEnum.NOTFOUND;
    }
    else if (row == JCTableEnum.ALLCELLS) {
        for (int e = size() - 1; e >= 0; e--) {
            SeriesValue sv = (SeriesValue)elementAt(e);
            if (sv.row != JCTableEnum.LABEL && sv.column == column) {
                removeElementAt(e);
            }
        }
        last_index = JCTableEnum.NOTFOUND;
    }
    else if (column == JCTableEnum.ALL) {
        for (int e = size() - 1; e >= 0; e--) {
            SeriesValue sv = (SeriesValue)elementAt(e);
            if (sv.row == row) {
                removeElementAt(e);
            }
        }
        last_index = JCTableEnum.NOTFOUND;
    }
    else if (column == JCTableEnum.ALLCELLS) {
        for (int e = size() - 1; e >= 0; e--) {
            SeriesValue sv = (SeriesValue)elementAt(e);
            if (sv.row == row && sv.column != JCTableEnum.LABEL) {
                removeElementAt(e);
            }
        }
        last_index = JCTableEnum.NOTFOUND;
    }

    int i = findExact(row, column);

    /*
     * If context already exists, or element is to be removed,
     * move it to end of series.
     */
    if (i >= 0 && i < size()) {
        SeriesValue v = (SeriesValue)elementAt(i);
        /* If this is not the last element, shift the series to make room. */
        if (i < size() - 1) {
            Object temp = elementAt(i);
            removeElementAt(i);
            addElement(temp);
        }
        /* Otherwise check if value has changed. */
        else if ((v.eqValue(value))) {
            return(false);
        }

        /* Set element value. */
        SeriesValue v2 = (SeriesValue)elementAt(size() - 1);
        v2.value = value;
    }
    /* Add item to end of list. */
    else {
        addElement(new SeriesValue(row, column, value));
    }
    last_index = JCTableEnum.NOTFOUND;

    return(true);
}

/**
 * Sets a range of cells to the value.
 * @return <code>true</code> if the series was changed after adding the value
 * @param end_column
 * @param end_row
 * @param start_column
 * @param start_row
 * @param value
 */
public boolean setValue(int start_row, int start_column,
                        int end_row, int end_column, Object value)
{
    int sr = start_row;
    int er = end_row;
    if (sr == JCTableEnum.LABEL && er == JCTableEnum.MAXINT) {
        sr = er = JCTableEnum.ALL;
    }
    else if (sr == 0 && er == JCTableEnum.MAXINT) {
        sr = er = JCTableEnum.ALLCELLS;
    }

    int sc = start_column;
    int ec = end_column;
    if (sc == JCTableEnum.LABEL && ec == JCTableEnum.MAXINT) {
        sc = ec = JCTableEnum.ALL;
    }
    else if (sc == 0 && ec == JCTableEnum.MAXINT) {
        sc = ec = JCTableEnum.ALLCELLS;
    }

    // we're going to disallow the LABEL,LABEL case during setting if it
    // is the start point of the range but not the end point.  Not only does
    // it not exist as a cell, the setValue() method interprets it as meaning
    // all labels everywhere!  However, if the user has specified LABEL
    // at both ends of the range, that is exactly the behaviour he wants.
    boolean disallowLabelLabel = false;
    if (sr == JCTableEnum.LABEL && sc == JCTableEnum.LABEL &&
        er != JCTableEnum.LABEL && ec != JCTableEnum.LABEL)
    {
        disallowLabelLabel = true;
    }

    for (int r = sr; r <= er; r++) {
        for (int c = sc; c <= ec; c++) {
            if (!(r == JCTableEnum.LABEL &&
                  c == JCTableEnum.LABEL &&
                  disallowLabelLabel))
            {
                setValue(r, c, value);
            }
        }
    }
    return(true);
}

/**
 * Sets the cell range to the value.
 * @return <code>true</code> if the series was changed after adding the value
 * @param cr
 * @param value
 */
public boolean setValue(JCCellRange cr, Object value)
{
    return(setValue(cr.start_row, cr.start_column,
                    cr.end_row, cr.end_column, value));
}

/**
 * Returns <code>true</code> if specified item's cells are equal.
 * @return <code>true</code> if specified item's cells are equal
 * @param column
 * @param row
 * @param value
 */
private static boolean eqCell(Object value, int row, int column)
{
    return(value != null &&
           ((SeriesValue)value).row == row &&
           ((SeriesValue)value).column == column);
}

/**
 * Returns <code>true</code> if the specified item's cells are equal or in the
 * same row/column.
 * @return <code>true</code> if the specified item's cells are equal or in the
 * same row/column
 * @param column
 * @param i
 * @param row
 */
private boolean eqCell(int i, int row, int column)
{
    SeriesValue v = (SeriesValue)elementAt(i);
    return((v.row == row ||
            v.row == JCTableEnum.ALL ||
            (row >= 0 && v.row == JCTableEnum.ALLCELLS)) &&
           (v.column == column || v.column == JCTableEnum.ALL ||
            (column >= 0 && v.column == JCTableEnum.ALLCELLS)));
}

/**
 * Searches for the specified cell and returns its index within the list.
 * @return <code>JCTableEnum.NOTFOUND</code> if the cell was not found
 * @param column
 * @param row
 */
private int findExact(int row, int column)
{
    int i;

    /* Check cached index. */
    if ((i = last_index) != JCTableEnum.NOTFOUND &&
        eqCell(elementAt(i), row, column))
    {
        return(i);
    }
    else {
        for (i = size() - 1; i >= 0; i--) {
            if (eqCell(elementAt(i), row, column)) {
                return(i);
            }
        }
    }
    return(JCTableEnum.NOTFOUND);
}

/**
 * Removes the item if the cell was found.
 * @param column
 * @param row
 */
protected void remove(int row, int column)
{
    int pos = findExact(row, column);
    if (pos != JCTableEnum.NOTFOUND) {
        removeAt(pos);
    }
}

/**
 * Removes the item at <code>pos</code>.
 * @param pos
 */
private void removeAt(int pos)
{
    if (pos < size()) {
        removeElementAt(pos);
        last_index = JCTableEnum.NOTFOUND;
    }
}

/**
 * Searches for the specified cell or closest match and returns its index
 * within the list.
 * @return <code>JCTableEnum.NOTFOUND</code> if the cell was not found
 * @param column
 * @param row
 */
private int find(int row, int column)
{
    int i = last_index;

    /* Check cached index. */
    if (i != JCTableEnum.NOTFOUND && i < size()) {
        SeriesValue v = (SeriesValue)elementAt(i);
        if (v.row == row && v.column == column) {
            return(i);
        }
    }

    /* Search from end of list. */
    for (i = size() - 1; i >= 0; i--) {
        if (eqCell(i, row, column)) {
            last_index = i;
            return(i);
        }
    }

    return(last_index = JCTableEnum.NOTFOUND);
}

/**
 * Returns <code>true</code> if the specified object is one of the values.
 * @return <code>true</code> if the specified object is one of the values
 * @param value
 */
private boolean containsValue(Object value)
{
    for (int i = 0; i < size(); i++) {
        if (((SeriesValue)elementAt(i)).value.equals(value)) {
            return(true);
        }
    }
    return(false);
}

/**
 * Returns the value at the specified index.
 * @return null if the index is illegal
 * @param pos
 */
private Object getValue(int pos)
{
    if (pos >= size()) {
        return(null);
    }
    return(((SeriesValue)elementAt(pos)).value);
}

/**
 * Adjusts the series after moving columns.
 * @param destination
 * @param num_columns
 * @param start_column
 */
public void moveColumns(int start_column, int num_columns, int destination)
{
    last_index = JCTableEnum.NOTFOUND;

    for (int i = 0; i < size(); i++) {
        SeriesValue s = (SeriesValue)elementAt(i);
        s.column = adjust(s.column, start_column, num_columns, destination);
    }
}

/**
 * Adjusts the series after moving rows.
 * @param destination
 * @param num_rows
 * @param start_row
 */
public void moveRows(int start_row, int num_rows, int destination)
{
    last_index = JCTableEnum.NOTFOUND;

    for (int i = 0; i < size(); i++) {
        SeriesValue s = (SeriesValue)elementAt(i);
        s.row = adjust(s.row, start_row, num_rows, destination);
    }
}

/**
 * Reorders columns after the column mapping changes.
 * @param new_positions
 */
public void remapColumns(int new_positions[])
{
    last_index = JCTableEnum.NOTFOUND;

    for (int i = 0; i < size(); i++) {
        SeriesValue s = (SeriesValue)elementAt(i);
        if (s.column >= 0 && s.column < new_positions.length) {
            s.column = new_positions[s.column];
        }
    }
}

/**
 * Reorders rows after the row mapping changes.
 * @param new_positions
 */
public void remapRows(int new_positions[])
{
    last_index = JCTableEnum.NOTFOUND;

    for (int i = 0; i < size(); i++) {
        SeriesValue s = (SeriesValue)elementAt(i);
        if (s.row >= 0 && s.row < new_positions.length) {
            s.row = new_positions[s.row];
        }
    }
}

/**
 * Adjusts the series after adding or deleting columns.
 * @param start_column the start position
 * @param num_columns the number of columns
 * @param direction use <CODE>+1</CODE> for insert and <CODE>-1</CODE>
 * for delete
 */
public void shiftColumn(int start_column, int num_columns, int direction)
{
    last_index = JCTableEnum.NOTFOUND;

    for (int i = size() - 1; i >= 0; i--) {
        SeriesValue s = (SeriesValue)elementAt(i);
        if (direction > 0) {
            if (s.column >= start_column) {
                s.column += num_columns;
            }
        }
        else {
            /* Erase the value if it is in a deleted column. */
            if (s.column >= start_column &&
                s.column < start_column + num_columns)
            {
                removeElementAt(i);
            }
            else if (s.column >= start_column + num_columns) {
                s.column -= num_columns;
            }
        }
    }
}

/**
 * Adjusts the series after adding or deleting rows.
 * @param start_row the start position
 * @param num_rows the number of rows
 * @param direction use <CODE>+1</CODE> for insert and <CODE>-1</CODE>
 * for delete
 */
public void shiftRow(int start_row, int num_rows, int direction)
{
    last_index = JCTableEnum.NOTFOUND;

    for (int i = size() - 1; i >= 0; i--) {
        SeriesValue s = (SeriesValue)elementAt(i);
        if (direction > 0) {
            if (s.row >= start_row) {
                s.row += num_rows;
            }
        }
        else {
            /* Erase value if in a deleted row. */
            if (s.row >= start_row && s.row < start_row + num_rows) {
                removeElementAt(i);
            }
            else if (s.row >= start_row + num_rows) {
                s.row -= num_rows;
            }
        }
    }
}

/**
 * Adjusts the series based on two columns swapping position.
 * @param column1
 * @param column2
 */
public void swapColumns(int column1, int column2)
{
    last_index = JCTableEnum.NOTFOUND;

    for (int i = 0; i < size(); i++) {
        SeriesValue s = (SeriesValue)elementAt(i);
        s.column = swap(s.column, column1, column2);
    }
}

/**
 * Adjusts the series based on two rows swapping position.
 * @param row1
 * @param row2
 */
public void swapRows(int row1, int row2)
{
    last_index = JCTableEnum.NOTFOUND;

    for (int i = 0; i < size(); i++) {
        SeriesValue s = (SeriesValue)elementAt(i);
        s.row = swap(s.row, row1, row2);
    }
}

/**
 * Calculates a new row or column position after some rows or columns
 * have been moved.
 * @return the original value, shifted
 * @param amount
 * @param destination
 * @param source
 * @param value
 */
private int adjust(int value, int source, int amount, int destination)
{
    if (source > destination) {
        if (value >= destination && value < source) {
            value += amount;
        }
        else if (value >= source && value < source + amount) {
            value -= source - destination;
        }
    }
    else {
        if (value >= source + amount && value < destination) {
            value -= amount;
        }
        else if (value >= source && value < source + amount) {
            value += destination - source - amount;
        }
    }
    return(value);
}

/**
 * Calculates a new row or column position after some rows or columns
 * have been swapped.
 * @return the original value, shifted
 * @param position1
 * @param position2
 * @param value
 */
private int swap(int value, int position1, int position2)
{
    if (value == position1) {
        return(position2);
    }
    else if (value == position2) {
        return(position1);
    }
    else {
        return(value);
    }
}

class SeriesValue implements Serializable {

    int row;
    int column;
    Object value;

    /**
     * Creates a new series value.
     * @param c
     * @param r
     * @param v
     */
    SeriesValue(int r, int c, Object v)
    {
        row = r;
        column = c;
        value = v;
    }

    /**
     * Returns <code>true</code> if the object is equal to this
     * <CODE>SeriesValue</CODE>.
     * @return <code>true</code> if the object is equal to this
     * <CODE>SeriesValue</CODE>
     * @param v
     */
    boolean eqValue(Object v)
    {
        if (value != null) {
            return (v != null) ? value.equals(v) : false;
        }
        return(v == null);
    }

    /**
     * Returns a String representation of the <code>JCSeriesValue</code> object.
     * @return String representation of the <code>JCSeriesValue</code> object
     */
    public String toString()
    {
        return("R" + row + "C" + column + " value=" + value);
    }
}

}
