/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCTable.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import com.klg.jclass.cell.JCCellEditor;
import com.klg.jclass.cell.JCCellRenderer;
import com.klg.jclass.cell.JCCellTextEditor;
import com.klg.jclass.table.resources.LocaleBundle;

import com.klg.jclass.util.JCEnvironment;
import com.klg.jclass.util.JCListenerList;
import com.klg.jclass.util.swing.GotoRowDialog;
import com.klg.jclass.util.swing.JCMessageHelper;

import javax.swing.*;
import javax.swing.plaf.ColorUIResource;
import javax.swing.plaf.ComponentUI;
import java.awt.*;
import java.awt.event.*;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.util.*;
import java.util.List;


 

/**
 * The core JClass LiveTable class; <code>JCTable</code> is the base
 * class for all the Beans, and contains the majority
 * of the programmatic API for JClass LiveTable.
 */
public class JCTable extends JComponent
			         implements ActionListener,  // Popup menu events handling
								FocusListener
{

protected AdjustmentHandler     adjustmentHandler;
protected CellAreaHandler       cellAreaHandler;
protected CellLayoutModel       cellLayout;
protected CellRendererPane      cellRendererPane;
protected SeriesModel           cellStyles;
protected RowColumnSeriesModel  columnWidths;
protected DataViewModel         dataView;
protected KeyInputHandler       keyHandler;
protected MouseInputHandler     mouseHandler;
protected RowColumnSeriesModel  rowHeights;
protected SelectionModel        selectionHandler;
protected SpanHandler           spanHandler;
protected TableChangeHandler    tableChangeHandler;
protected TableLayout           tableLayout;
protected SeriesModel           userData;

protected Vector                moveables;

protected JScrollBar            hsb;
protected JScrollBar            vsb;

// Internal properties
protected int                   allow_cell_resize = JCTableEnum.RESIZE_ALL;
protected int                   allow_resize_by = JCTableEnum.RESIZE_BY_ALL;
protected boolean               auto_edit = false;
protected int                   auto_scroll = JCTableEnum.AUTO_SCROLL_NONE;
protected int                   cell_border_width = 1;
protected boolean               column_label_display = true;
protected int                   column_label_placement = JCTableEnum.PLACE_TOP;
protected int                   column_label_offset = 0;
protected int                   component_border_width = 0;
protected CellStyleModel        default_label_style = null;
protected CellStyleModel        default_cell_style = null;
protected int                   edit_height_policy = JCTableEnum.EDIT_SIZE_TO_CELL;
protected int                   edit_width_policy = JCTableEnum.EDIT_SIZE_TO_CELL;
protected Color                 focus_color = new ColorUIResource(Color.black);
protected FocusModel            focusManager = null;
protected int                   focus_indicator = JCTableEnum.FOCUS_RECTANGLE;
protected CellBorderModel       frame_border = null;
protected int                   frame_border_width = 0;
protected int                   frozen_columns;
protected int                   frozen_column_placement = JCTableEnum.PLACE_LEFT;
protected int                   frozen_rows;
protected int                   frozen_row_placement = JCTableEnum.PLACE_TOP;
protected int                   hsb_attach = JCTableEnum.SIZE_TO_CELLS;
protected int                   hsb_display = JCTableEnum.SCROLLBAR_AS_NEEDED;
protected int                   hsb_offset = 0;
protected int                   hsb_position = JCTableEnum.POSITION_BY_CELLS;
protected int                   hsb_track = JCTableEnum.TRACK_LIVE;
protected int                   hsb_track_row = JCTableEnum.TRACK_COLUMN_NUMBER;
protected boolean               ignore_container_size = false;
protected int                   jump_mode = JCTableEnum.JUMP_NONE;
protected int                   left_column = 0;
protected int                   margin_height = 1;
protected int                   margin_width = 2;
protected int                   min_cell_visibility = 100;
protected Object                mouseWheelListener = null;
protected boolean               popupMenuEnabled = false;
protected boolean               repaint_enabled = true;
protected boolean               resize_even = false;
protected boolean               resize_interactive = true;
protected boolean               row_label_display = true;
protected int                   row_label_placement = JCTableEnum.PLACE_LEFT;
protected int                   row_label_offset = 0;
protected boolean               select_include_labels = true;
protected Color                 selected_background = null;
protected int                   selected_background_mode = JCTableEnum.USE_CELL_FOREGROUND;
protected Color                 selected_foreground = null;
protected int                   selected_foreground_mode = JCTableEnum.USE_CELL_BACKGROUND;
protected boolean               sort_series = true;
protected boolean               store_image_enabled = false;
protected int                   top_row = 0;
protected Color                 track_background;
protected Color                 track_foreground;
protected Dimension             track_size = new Dimension(50, 20);
protected boolean               track_cursor = true;
protected boolean               traverse_cycle = true;
protected int                   variable_estimate_count = JCTableEnum.ALL;
protected int                   visible_columns = JCTableEnum.NOVALUE;
protected int                   visible_rows = JCTableEnum.NOVALUE;
protected int                   vsb_attach = JCTableEnum.SIZE_TO_CELLS;
protected int                   vsb_display = JCTableEnum.SCROLLBAR_AS_NEEDED;
protected int                   vsb_offset = 0;
protected int                   vsb_position = JCTableEnum.POSITION_BY_CELLS;
protected int                   vsb_track = JCTableEnum.TRACK_LIVE;
protected int                   vsb_track_column = JCTableEnum.TRACK_ROW_NUMBER;

protected int                   current_row = JCTableEnum.NOVALUE;
protected int                   current_column = JCTableEnum.NOVALUE;

// Listeners
protected JCListenerList        cellDisplayListeners = null;
protected JCListenerList        editCellListeners = null;
protected JCListenerList        paintListeners = null;
protected JCListenerList        resizeListeners = null;
protected JCListenerList        resizeMotionListeners = null;
protected JCListenerList        scrollListeners = null;
protected JCListenerList        selectListeners = null;
protected JCListenerList        sortListeners = null;
protected JCListenerList        traverseListeners = null;

// Internal variables
protected Vector                actions;
protected Component             oldFocusOwner = null;
protected boolean               scrolling = false;
protected boolean               set_initial_left = false;
protected boolean               set_initial_top = false;
protected boolean               set_left_column = false;
protected boolean               set_top_row = false;
protected int                   sort_column = JCTableEnum.NOVALUE;
protected int                   sort_direction = JCTableEnum.NOVALUE;
protected transient Image       store_image = null;
protected transient Point       store_origin = new Point(0, 0);

private boolean                 needs_recalc = false;
private boolean                 needs_relayout = false;
private boolean                 needs_repaint = false;
protected boolean               needs_initial_traverse = true;
protected boolean               needs_initial_set_left_check = true;
protected boolean               needs_initial_set_top_check = true;

protected TablePopupMenu        popupMenu = null;

protected boolean				pointer_traverse_forces_edit = true;


 

public JCTable()
{
	setDoubleBuffered(true);
	setLayout(null);

	String focusManagerString =
	    (JCEnvironment.getJavaVersion() >= 140) ? "com.klg.jclass.table.FocusManagerNew"
										        : "com.klg.jclass.table.FocusManagerOld";
	try {
	    Class c = Class.forName(focusManagerString);
	    focusManager = (FocusModel)c.newInstance();
	    focusManager.setTable(this);
	}
	catch (Exception e) {
		System.out.println("FocusManager initialization error:" +
		                   e.getMessage());
	}

	cellRendererPane = new CellRendererPane();
	add(cellRendererPane);

	tableChangeHandler = new TableChangeHandler(this);

	createDefaultDataView();

	adjustmentHandler   = new AdjustmentHandler(this);
	cellAreaHandler     = new CellAreaHandler(this);
	keyHandler          = new KeyInputHandler(this);
	mouseHandler        = new MouseInputHandler(this);
	tableLayout         = new TableLayout(cellAreaHandler);
	addFocusListener(this);
	addTableMouseWheelListener(null);

	hsb = new JScrollBar(JScrollBar.HORIZONTAL);
	hsb.setUnitIncrement(20);
	hsb.setBlockIncrement(100);
	hsb.addAdjustmentListener(adjustmentHandler);
	add(hsb);

	vsb = new JScrollBar(JScrollBar.VERTICAL);
	vsb.setUnitIncrement(21);
	vsb.setBlockIncrement(200);
	vsb.addAdjustmentListener(adjustmentHandler);
	add(vsb);

	enableEvents(AWTEvent.MOUSE_EVENT_MASK         |
		         AWTEvent.MOUSE_MOTION_EVENT_MASK  |
				 AWTEvent.KEY_EVENT_MASK);

	setFont(new Font("DialogInput", Font.PLAIN, 12));

	createDefaultColumnWidthSeries();
	createDefaultRowHeightSeries();

	UIManager.getDefaults().put("JCTableUI", "com.klg.jclass.table.JCTableUI");
	setUI(JCTableUI.createUI(this));
}

public void addNotify()
{
	super.addNotify();


 


 

	CellSizeHandler.resetDimensions(this);

    // so we are not relaying out twice on startup
    setRecalcRequired(false);


 

}

/**
 * Does passed in class have the given method name?  If so, returns it.
 * @param c the class in which the method is to be found
 * @param methodString the name of the method
 * @return the <code>Method</code> object from <code>JCTable</code>, based on
 *     the input String; if there is no such method, return null
 */
protected Method findMethod(Class c, String methodString)
{
	Method returnMethod = null;
    if (c != null && methodString != null) {
		Method[] methods = c.getMethods();
		for (int i = 0; methods != null && i < methods.length; i++) {
			if (methods[i] != null &&
				methods[i].getName().equals(methodString))
			{
				returnMethod = methods[i];
				break;
			}
		}
    }
	return(returnMethod);
}

/**
 * Adds the given <code>MouseWheelListener</code> to the table.
 * <br><B>Note:</b> This will replace the current
 * <code>MouseWheelListener</code>. This method will do nothing if the
 * underlying JDK is pre 1.4.
 * @param listener The new listener; this must be of type
 *     <code>MouseWheelListener</code>. If the value passed in is null, this
 *     method will add a new <code>TableMouseWheelListener</code> (which is the
 *     default listener).
 */
public void addTableMouseWheelListener(Object listener)
{
	if (mouseWheelListener != null) {
		removeTableMouseWheelListener();
	}

	Class tableClass = getClass();
	Method method = findMethod(tableClass, "addMouseWheelListener");
	if (method == null) { // Must be pre JDK 1.4
		return;
	}

	if (listener == null) {
		try {
			String s = "com.klg.jclass.table.TableMouseWheelListener";
			Class c = Class.forName(s);

			// Find the constructor with exactly one parameter = JCTable
			// If found, invoke constructor to create new mouseWheelListener
			Constructor[] constructors = c.getConstructors();
			if (constructors != null && constructors.length > 0) {
				Constructor constructor = null;
				for (int i = 0; i < constructors.length; i++) {
					Constructor cons = constructors[i];
					Class[] params = cons.getParameterTypes();
					if (params != null && params.length == 1) {
						Class paramClass = params[0];
						boolean found = false;
						while(tableClass != null && !found) {
							if (paramClass.equals(tableClass)) {
								found = true;
							}
							else {
								tableClass = tableClass.getSuperclass();
							}
						}
						if (found) {
						    constructor = cons;
							break;
						}
					}
				}
				if (constructor != null) {
				    mouseWheelListener =
				        constructor.newInstance(new Object[] {this});
				}
			}
		}
		catch (Exception e) {
			e.printStackTrace();
			return;
		}
	}
	else {
		mouseWheelListener = listener;
	}
	if (mouseWheelListener == null) {
		return;
	}

	try {
		method.invoke(this, new Object[] {mouseWheelListener});
	}
	catch (Exception e) {
		e.printStackTrace();
	}
}

/**
 * Removes the current mouse wheel listener.  This method can be called to
 * remove the default mouse wheel listener that is installed on startup if
 * the underlying JDK is 1.4 or later (this method does nothing otherwise).
 */
public void removeTableMouseWheelListener()
{
	if (mouseWheelListener == null) {
		return;
	}
	Method method = findMethod(getClass(), "removeMouseWheelListener");
	if (method == null) { // Must be pre JDK 1.4
		return;
	}

	try {
		method.invoke(this, new Object[] {mouseWheelListener});
	}
	catch (Exception e) {
		e.printStackTrace();
	}
	mouseWheelListener = null;
}

public Dimension getMinimumSize()
{
	if (tableLayout != null) {
		return(tableLayout.getMinimumSize());
	}
	else {
		return(new Dimension(50,50));
	}
}

public Dimension getPreferredSize()
{
	if (tableLayout != null) {
		return(tableLayout.getPreferredSize());
	}
	else {
		return(new Dimension(50, 50));
	}
}

public void layoutAreas()
{
	tableLayout.layoutAreas();
}

public void doLayout()
{
	if (isDisplayable()) {
		requestChange(TableChangeHandler.TABLE_LAYOUT);
	}
}

/**
 * Returns the UI for <code>JCTable</code>.
 * @return the UI for <code>JCTable</code>
 */
public JCTableUI getUI()
{
	return((JCTableUI)ui);
}

/**
 * Sets the UI for <code>JCTable</code>.
 * @param newUI the new UI for <code>JCTable</code>
 */
public void setUI(ComponentUI newUI)
{
	super.setUI(newUI);
}

/**
 * Updates the UI for <code>JCTable</code>.
 */
public void updateUI()
{
	setUI(UIManager.getUI(this));
}

/**
 * Returns the UIClass ID for <code>JCTable</code>.
 * @return the UIClass ID for <code>JCTable</code>
 */
public String getUIClassID()
{
	return("JCTableUI");
}

//
// Methods: CellAreaHandler, CellLayout, CellStyle, Component, DataSource,
//          DataView, EditHandler, KeyInputHandler, MouseInputHandler,
//          TableLayout, UserData
// ----------------------------------------------------------------------------
//

/**
 * Returns the <code>CellArea</code> handler, which manages the creation and removal of
 * <code>CellAreas</code> within the table.
 * @return the <code>CellArea</code> handler
 */
public CellAreaHandler getCellAreaHandler()
{
	return(cellAreaHandler);
}

/**
 * Creates a mapping between a cell type (for example, class) and an editing
 * object.
 * @param cellType the class type of the object in a cell
 * @param editor an editor (specified by class) specifically suited to edit
 *     objects of the given class
 */
public void setCellEditor(Class cellType, Class editor)
{
	setCellEditorMapping(cellType, editor);
}

/**
 * Creates a mapping between a cell type and an instance of an
 * editing object.
 * @param cellType the class type of the object in a cell
 * @param editor an editor (specified by an object which implements the
 *     <code>JCCellEditor</code> interface) specifically suited to edit objects
 *     of the given class
 */
public void setCellEditor(Class cellType, JCCellEditor editor)
{
	setCellEditorMapping(cellType, editor);
}

/**
 * Creates a mapping between a cell type and an editor object or
 * instance of an editing object.
 * @param cellType the class type of the object in a cell
 * @param editor an editor object specifically suited to edit objects
 *     of the given class
 */
protected void setCellEditorMapping(Class cellType, Object editor)
{
	if (cellType == null) {
        return;
	}

    Hashtable ht = getDataView().getEditorsTable();

    // Handle remove case
    if (editor == null) {
        ht.remove(cellType);
        return;
    }
	else {
        // Add to hash table, or replace existing mapping.
        ht.put(cellType, editor);
    }
    // Put the table back into the data view
    getDataView().setEditorsTable(ht);
}

/**
 * Returns the <code>CellLayout</code> manager.
 * @return the table's <code>CellLayout</code> manager
 */
public CellLayoutModel getCellLayout()
{
	if (cellLayout == null) {
		createDefaultCellLayout();
	}
	return(cellLayout);
}

/**
 * Creates the default <code>CellLayoutModel</code> manager.
 */
protected void createDefaultCellLayout()
{
	try {
		Class c = Class.forName("com.klg.jclass.table.JCCellLayout");
		CellLayoutModel cellLayout = (CellLayoutModel)c.newInstance();
		cellLayout.setTable(this);
		setCellLayout(cellLayout);
	}
	catch (Exception e) {
		System.out.println("TableCellLayout initialization error: " +
                           e.getMessage());
	}
}

/**
 * Sets the <code>CellLayout</code> manager.
 * @param cellLayout the new <code>CellLayout</code> manager
 */
public void setCellLayout(CellLayoutModel cellLayout)
{
	this.cellLayout = cellLayout;
	cellLayout.setTable(this);
}

/**
 * Creates a mapping between a cell type and an renderering object.
 * @param cellType the class type of the object in a cell
 * @param renderer a renderer (specified by class) specifically suited to render
 *     objects of the given class
 */
public void setCellRenderer(Class cellType, Class renderer)
{
	setCellRendererMapping(cellType, renderer);
}

/**
 * Creates a mapping between a cell type and an instance of a
 * renderering object.
 * @param cellType the class type of the object in a cell
 * @param renderer a renderer (specified by an object which implements the
 *     <code>JCCellRenderer</code> interface) specifically suited to render
 *     objects of the given class
 */
public void setCellRenderer(Class cellType, JCCellRenderer renderer)
{
	setCellRendererMapping(cellType, renderer);
}

/**
 * Creates a mapping between a cell type and an renderer object or
 * instance of an renderering object.
 * @param cellType the class type of the object in a cell
 * @param renderer a renderer specifically suited to render objects of the
 *     given class
 */
protected void setCellRendererMapping(Class cellType, Object renderer)
{
	if (cellType == null) {
        return;
	}

    Hashtable ht = getDataView().getRenderersTable();

    // Handle remove case
    if (renderer == null) {
        ht.remove(cellType);
        return;
    }
	else {
        // Add to hash table, or replace existing mapping.
        ht.put(cellType, renderer);
    }
    // Put the table back into the data view
    getDataView().setRenderersTable(ht);
}

/**
 * Adds to the moveables collection.
 *
 * @param newMoveable the new moveable to be added
 * @param oldMoveable the old moveable to be replaced
 */
protected void addToMoveables(Object newMoveable, Object oldMoveable)
{
	if (moveables == null) {
		moveables = new Vector();
	}
	if (oldMoveable != null) {
	    moveables.removeElement(oldMoveable);
	}
	moveables.addElement(newMoveable);
}

/**
 * Gets the cell's <CODE>CellStyleModel</CODE> value. The
 * <code>CellStyleModel</code> object returned here may not be unique to the
 * given row and column.
 * @param row the row of the cell whose <code>cellStyleModel</code> is required
 * @param column the column of the cell whose <code>cellStyleModel</code> is
 *     required
 * @return a <code>CellStyleModel</code> object for the given row and column
 * @see #setCellStyle
 */
public CellStyleModel getCellStyle(int row, int column)
{
	if (cellStyles != null) {
		return((CellStyleModel)(cellStyles.getValue(row, column)));
	}
	return(createCellStyle());
}

/**
 * Gets a unique instance of the cell's <CODE>CellStyleModel</CODE> value.
 * Because style instances can be applied to a range of cells,
 * <CODE>getCellStyle</CODE> may not return an <CODE>CellStyleModel</CODE>
 * object that is only used by the given cell. As a result, changing a
 * property in the style may affect more than one cell when table display
 * is updated.
 *<P>Calling <CODE>getUniqueCellStyle</CODE> returns a clone of the cell
 * style instance used for the cell. This instance is detached from the
 * cell and must be set back using <CODE>setCellStyle</CODE> to change
 * the cell.
 *<P> <code>JCTableEnum.ALL</code> is an invalid value for the row and column
 * parameters. Any call with the <code>JCTableEnum.ALL</code> parameter value
 * causes it to be converted into <code>JCTableEnum.ALLCELLS</code> value.
 * @param row the row of the cell for which a unique cell style is required
 * @param column the column of the cell for which a unique cell style is
 *     required
 * @return A unique <code>CellStyleModel</code> object for the given row and
 *     column
 */
public CellStyleModel getUniqueCellStyle(int row, int column)
{
	CellStyleModel csm =
	    getCellStyle(row == JCTableEnum.ALL ? JCTableEnum.ALLCELLS : row,
                     column == JCTableEnum.ALL ? JCTableEnum.ALLCELLS : column);
	return((CellStyleModel)csm.clone());
}

/**
 * Returns a unique Collection of <code>CellStyleModel</code> instances used in table.
 * @return a Collection of unique instances of <code>CellStyleModel</code> objects
 */
public Collection getCellStyles()
{
    if (cellStyles != null) {
        return(cellStyles.getUniqueValues());
    }
    return(null);
}

/**
 * Returns a Collection of <code>JCCellRanges</code> that use the given
 * <code>CellStyleModel</code>.
 * @param style the <code>CellStyleModel</code> whose ranges are required
 * @return a Collection of <CODE>JCCellRanges</CODE> objects that use
 *     the given <code>CellStyleModel</code>
 */
public Collection getCellStyleRanges(CellStyleModel style)
{
	if (cellStyles != null) {
		return(cellStyles.getCellRanges(style));
	}
	return(null);
}

/**
 * Returns a list of <code>CellStyle</code> series values.
 * @return a list of <code>CellStyle</code> series values
 */
public List getCellStyleValues()
{
	if (cellStyles != null) {
		return(cellStyles.getSeriesValues());
	}
	return(null);
}

/**
 * Creates a new cell style object.
 * @return a new <code>CellStyleModel</code> object
 */
protected CellStyleModel createCellStyle()
{
	try {
		Class c = Class.forName("com.klg.jclass.table.JCCellStyle");
		return((CellStyleModel)c.newInstance());
	}
	catch(Exception e) {
		System.out.println("Unable to create a new CellStyle object: " +
		                   e.getMessage());
	}
	return(null);
}

/**
 * Returns the default label style, if set.
 * @return the current default label style
 */
public CellStyleModel getDefaultLabelStyle()
{
	return(default_label_style);
}

/**
 * Returns the default cell style, if set.
 * @return the current default cell style
 */
public CellStyleModel getDefaultCellStyle()
{
	return(default_cell_style);
}

/**
 * Creates the default <code>CellStyleModel</code> series object.
 */
protected void createDefaultCellStyleSeries()
{
	try {
		SeriesModel oldCellStyles = cellStyles;
		Class c = Class.forName("com.klg.jclass.table.JCSeries");
		cellStyles = (SeriesModel)c.newInstance();

		addToMoveables(cellStyles, oldCellStyles);
	}
	catch(Exception e) {
		System.out.println("CellStyleModels series initialization error: " +
                           e.getMessage());
	}
}

/**
 * Sets the style for a cell (or list of cells).
 * @param row the row index of the cell
 * @param column the column index of the cell
 * @param style the new style for the given cell
 */
public void setCellStyle(int row, int column, CellStyleModel style)
{
	if (cellStyles == null) {
		createDefaultCellStyleSeries();
	}
    cellStyles.setValue(row, column, style);

	// Remove TableChangeHandler as a listener in case the same
	// instance of the CellStyleModel is being applied to another cell
	style.removePropertyChangeListener(tableChangeHandler);

	// Add TableChangeHandler as a listener for CellStyle changes
	style.addPropertyChangeListener(tableChangeHandler);

	if (row < JCTableEnum.LABEL || column < JCTableEnum.LABEL) {
		requestChange(TableChangeHandler.TABLE_REPAINT);
	}
	else {
		requestChange(TableChangeHandler.CELL_REPAINT, row, column);
	}
}

/**
 * Sets the style for a range of cells.
 * @param cr the cell range
 * @param style the new style for the given cell range
 */
public void setCellStyle(JCCellRange cr, CellStyleModel style)
{
	if (cellStyles == null) {
		createDefaultCellStyleSeries();
	}
	cellStyles.setValue(cr, style);

	// Remove TableChangeHandler as a listener in case the same
	// instance of the CellStyleModel is being applied to another cell
	style.removePropertyChangeListener(tableChangeHandler);

	// Add TableChangeHandler as a listener for CellStyle changes
	style.addPropertyChangeListener(tableChangeHandler);

	requestChange(TableChangeHandler.CELL_REPAINT, cr);
}

/**
 * If possible, creates a clone of the cell style container stored
 * in this table object and returns them to the caller.  Use
 * this method in conjunction with <code>setCellStyles()</code> if you want
 * to save the current state of the styles, make a temporary
 * change, then reverse the temporary change.
 * @return a clone of the cell styles for this table
 * @throws CloneNotSupportedException
 */
public SeriesModel cloneCellStyles() throws CloneNotSupportedException
{
	SeriesModel sm = null;
	// Possibly use introspection to find a public clone() method?
	if (cellStyles instanceof Vector) {
		sm = (SeriesModel)((Vector)cellStyles).clone();
	}
	return(sm);
}

/**
 * Replaces the cell style container with the one provided.
 * Use this method in conjunction with <code>cloneCellStyles()</code> for
 * making and reversing temporary style changes in a
 * convenient way.
 *<p>
 * Calling this method with null essentially resets the
 * cell styles to initial state.  The default cell and
 * label styles are used.
 *<p>
 * It is recommended that the default cell and label styles
 * not be changed.  For most uses, this is fine.  However,
 * subclasses of JCTable may be tempted to set <code>default_label_style</code>
 * and <code>default_cell_style</code> to new values.  Unfortunately, we
 * cannot detect these new defaults EXCEPT when this call
 * is given a null.
 *<p>
 * <B>Note:</b> This method will force a table repaint, but
 * not a relayout.  In cases in which a relayout is
 * required, the caller should force it by calling
 * <pre>
 *   table.setRepaintEnabled(false); // disable repaint
 *   table.setCellStyles(oldStyles); // restore
 *   table.setRelayoutRequired(true); // force relayout
 *   table.setRepaintEnabled(true); // will force relayout and repaint.
 * </pre>
 * @param sm the new cell style container; if null, the cell styles are
 *     set to their default values
 */
public void setCellStyles(SeriesModel sm)
{
	// Should probably disable all property change listeners
	// on the old styles, then re-enable them on the new styles.
	// The only impact is possible unnecessary updates (if user
	// changes property on style not in current SeriesModel).
	// Disabling is not currently done because there is no
	// way for table to programmatically remove TableChangeHandler
	// from each style without hard coding SeriesValue.

	if (sm == null) {
		// Need to create new cell style immediately in order
		// to guarantee that all cells have a style associated
		// with them.
		createDefaultCellStyleSeries();

		// Add default label style
		if (default_label_style == null) {
			// Not a likely case, but could happen
			default_label_style = createCellStyle();
		}
		setCellStyle(JCTableEnum.LABEL, JCTableEnum.LABEL,
					 default_label_style);

		// Add default cell style
		if (default_cell_style == null) {
			// Not a likely case, but could happen
			default_cell_style = createCellStyle();
		}
		setCellStyle(JCTableEnum.ALLCELLS, JCTableEnum.ALLCELLS,
					 default_cell_style);
	}
	else {
		// Remove current style from moveable list
		if (cellStyles != null && moveables != null) {
			moveables.removeElement(cellStyles);
		}

		// Assign style.  Assume that the defaults don't
		// need to change.  We cannot reverse-engineer these
		// defaults anyways because series works like a stack.
		cellStyles = sm;

		// Add new style to moveable list
		if (moveables != null) {
			moveables.addElement(cellStyles);
		}
	}
	requestChange(TableChangeHandler.TABLE_REPAINT);
}

/**
 * Creates the default column width series.
 */
protected void createDefaultColumnWidthSeries()
{
	try {
		Class c = Class.forName("com.klg.jclass.table.JCRowColumnSeries");
		columnWidths = (RowColumnSeriesModel)c.newInstance();
		columnWidths.setDefault(new CellSize(10));
	}
	catch(Exception e) {
		System.out.println("ColumnWidth series initialization error: " +
                           e.getMessage());
	}
}

/**
 * Returns a list of column width series values.
 * @return the current list of column width values
 */
public List getColumnWidthValues()
{
	if (columnWidths != null) {
		return(columnWidths.getSeriesValues());
	}
	return(null);
}

/**
 * Returns the component handler which manages components in cells.
 * <br><B>Note:</b> This may be a different handler depending on the JRE
 * (specifically 1.3 vs 1.4).
 * @return the table's component handler
 */
protected ComponentModel getComponentModel()
{
	return(focusManager.getComponentModel());
}

/**
 * Returns the table data source.
 * @return the table's current data source
 * @see #setDataSource
 */
public TableDataModel getDataSource()
{
	if (dataView == null) {
		createDefaultDataSource();
	}
	return(dataView.getDataSource());
}

/**
 * Creates the default data source with 10 rows and 5 columns.
 */
protected void createDefaultDataSource()
{
	try {
		Class c = Class.forName("com.klg.jclass.table.data.JCVectorDataSource");
		TableDataModel tableData = (TableDataModel)c.newInstance();
		setDataSource(tableData);
	}
	catch(Exception e) {
		System.out.println("DataSource initialization error: " +
                           e.getMessage());
	}
}

/**
 * Sets the table data source.
 * @param tableData the new data source for this table
 */
public void setDataSource(TableDataModel tableData)
{
	if (dataView == null) {
		createDefaultDataView();
	}
    dataView.setDataSource(tableData);
}

/**
 * Returns the data view which controls access to data and table notification
 * when data changes.
 * @return the current data view
 */
public DataViewModel getDataView()
{
	if (dataView == null) {
		createDefaultDataView();
	}
	return(dataView);
}

/**
 * 	Creates the default data view manager.
 */
protected void createDefaultDataView()
{
	try {
		Class c = Class.forName("com.klg.jclass.table.TableDataView");
		DataViewModel dataView = (DataViewModel)c.newInstance();
		dataView.setTable(this);
		setDataView(dataView);
	}
	catch(Exception e) {
		System.out.println("View initialization error: " + e.getMessage());
	}
}

/**
 * Sets a new data view manager.
 * @param dataView the new data view
 */
public void setDataView(DataViewModel dataView)
{
	this.dataView = dataView;
}

/**
 * Returns the edit handler for the table, which manages editors within cells.
 * @return the table's edit traverse handler; this could be different
 *     depending on the JRE (specifically 1.3 vs 1.4)
 */
public EditTraverseHandler getEditTraverseHandler()
{
	return(focusManager.getEditTraverseHandler());
}

/**
 * Returns the key input manager for the table.
 * @return the key input manager for the table
 */
public KeyInputHandler getKeyInputHandler()
{
	return(keyHandler);
}

/**
 * Returns the mouse input manager for the table.
 * @return the mouse input manager for the table
 */
public MouseInputHandler getMouseInputHandler()
{
	return(mouseHandler);
}

/**
 * Creates the default row height series.
 */
protected void createDefaultRowHeightSeries()
{
	try {
		Class c = Class.forName("com.klg.jclass.table.JCRowColumnSeries");
		rowHeights = (RowColumnSeriesModel)c.newInstance();
		rowHeights.setDefault(new CellSize(1));
	}
	catch(Exception e) {
		System.out.println("RowHeight series initialization error: " +
		                   e.getMessage());
	}
}

/**
 * Returns a list of row height series values.
 * @return the current list of row height values
 */
public List getRowHeightValues()
{
	if (rowHeights != null) {
		return(rowHeights.getSeriesValues());
	}
	return(null);
}

/**
 * Returns the <code>TableLayout</code> manager.
 * @return the table's layout manager
 */
public TableLayout getTableLayout()
{
	return(tableLayout);
}

/**
 * Gets the cell's <CODE>UserData</CODE> value.
 * @param row the row index of the cell
 * @param column the column index of the cell
 * @return the user data object for the cell
 * @see #setUserData
 */
public Object getUserData(int row, int column)
{
	if (userData == null)
	{
		return(null);
	}
	return(userData.getValue(row, column));
}

/**
 * Creates the default <code>UserData</code> series object.
 */
protected void createDefaultUserDataSeries()
{
	try {
		SeriesModel oldUserData = userData;
		Class c = Class.forName("com.klg.jclass.table.JCSeries");
		userData = (SeriesModel)c.newInstance();

		addToMoveables(userData, oldUserData);
	}
	catch(Exception e) {
		System.out.println("UserData series initialization error: " +
                           e.getMessage());
	}
}

/**
 * Sets the user data for a cell.
 * @param row the row index of the cell
 * @param column the column index of the cell
 * @param value the user data object for the cell
 */
public void setUserData(int row, int column, Object value)
{
	if (userData == null) {
		createDefaultUserDataSeries();
	}
    userData.setValue(row, column, value);
}

/**
 * Sets the user data for a range of cells.
 * @param cr the cell range
 * @param value the user data object for the cell range
 */
public void setUserData(JCCellRange cr, Object value)
{
	if (userData == null) {
		createDefaultUserDataSeries();
	}
	userData.setValue(cr, value);
}


//
//
// Properties
// ----------------------------------------------------------------------------
//

/**
 * Gets the table's <CODE>AllowCellResize</CODE> value.  This determines how
 * an end-user can interactively resize rows/columns.
 * @return the current <CODE>AllowCellResize</CODE> value; one of
 * <code>JCTableEnum.RESIZE_ALL</code> (default),
 * <code>JCTableEnum.RESIZE_NONE</code>, <code>JCTableEnum.RESIZE_COLUMN</code>,
 * or <code>JCTableEnum.RESIZE_ROW</code>
 * @see #setAllowCellResize
 */
public int getAllowCellResize()
{
	return(allow_cell_resize);
}

/**
 * Sets how an end-user can interactively resize rows/columns.
 * Valid values are:
 * <UL>
 * <LI><code>JCTableEnum.RESIZE_ALL</code> (default)
 * <LI><code>JCTableEnum.RESIZE_NONE</code>
 * <LI><code>JCTableEnum.RESIZE_COLUMN</code>
 * <LI><code>JCTableEnum.RESIZE_ROW</code>
 * </UL>
 * @param value the new <CODE>AllowCellResize</CODE> value; must be one of
 * <code>JCTableEnum.RESIZE_ALL</code> (default),
 * <code>JCTableEnum.RESIZE_NONE</code>, <code>JCTableEnum.RESIZE_COLUMN</code>,
 * or <code>JCTableEnum.RESIZE_ROW</code>
 */
public void setAllowCellResize(int value)
{
	if (value != JCTableEnum.RESIZE_ALL    &&
	    value != JCTableEnum.RESIZE_NONE   &&
	    value != JCTableEnum.RESIZE_COLUMN &&
	    value != JCTableEnum.RESIZE_ROW)
	{
		throw new IllegalArgumentException("Parameter value must be one of " +
		                                   "JCTableEnum.RESIZE_ALL, " +
										   "JCTableEnum.RESIZE_NONE, " +
										   "JCTableEnum.RESIZE_COLUMN, " +
										   "JCTableEnum.RESIZE_ROW");
	}
	allow_cell_resize = value;
}

/**
 * Gets the table's <CODE>AllowResizeBy</CODE> value to determine whether
 * row height and column width can be resized by labels, cells, or both.
 * @return The current table's <CODE>AllowResizeBy</CODE> value; one of
 *     <code>JCTableEnum.RESIZE_BY_LABELS</code>,
 *     <code>JCTableEnum.RESIZE_BY_CELLS</code>, or
 *     <code>JCTableEnum.RESIZE_BY_ALL</code>
 * @see #setAllowResizeBy
 */
public int getAllowResizeBy()
{
	return(allow_resize_by);
}

/**
 * If set to <CODE>JCTableEnum.RESIZE_BY_LABELS</CODE>, a user can only
 * resize the column widths and row heights by resizing the respective
 * row or column's label. Use <CODE>JCTableEnum.RESIZE_BY_CELLS</CODE>
 * to allow cell resize but not label resize, and
 * <CODE>JCTableEnum.RESIZE_BY_ALL</CODE> to allow label or cell
 * resize.
 * @param value the new <CODE>AllowResizeBy</CODE> value; one of
 *     <code>JCTableEnum.RESIZE_BY_LABELS</code>,
 *     <code>JCTableEnum.RESIZE_BY_CELLS</code>, or
 *     <code>JCTableEnum.RESIZE_BY_ALL</code>
 */
public void setAllowResizeBy(int value)
{
	if (value != JCTableEnum.RESIZE_BY_LABELS &&
	    value != JCTableEnum.RESIZE_BY_CELLS  &&
	    value != JCTableEnum.RESIZE_BY_ALL)
	{
		throw new IllegalArgumentException("Parameter value must be one of " +
		                                   "JCTableEnum.RESIZE_BY_LABELS, " +
										   "JCTableEnum.RESIZE_BY_CELLS, " +
										   "JCTableEnum.RESIZE_BY_ALL");
	}
    allow_resize_by = value;
}

/**
 * Gets the table's <CODE>AutoEdit</CODE> value.
 * @return the table's current <CODE>AutoEdit</CODE> value
 * @see #setAutoEdit
 */
public boolean isAutoEdit()
{
	return(auto_edit);
}

/**
 * If <code>true</code>, the table automatically displays an editor when a cell
 * is entered.
 * @param value the new <CODE>AutoEdit</CODE> value
 */
public void setAutoEdit(boolean value)
{
	this.auto_edit = value;
}

/**
 * Gets the table's <CODE>AutoScroll</CODE> value.  This controls how the table
 * scrolls when the user selects cells or drags the mouse past the bounds of
 * the table.
 * @return the table's current <CODE>AutoScroll</CODE> value; one of
 * <code>JCTblEnum.AUTO_SCROLL_NONE</code> (default),
 * <code>JCTblEnum.AUTO_SCROLL_ROW</code>,
 * <code>JCTblEnum.AUTO_SCROLL_COLUMN</code>, or
 * <code>JCTblEnum.AUTO_SCROLL_BOTH</code>
 * @see #setAutoScroll
 */
public int getAutoScroll()
{
	return(auto_scroll);
}

/**
 * Sets how the table scrolls when the user selects cells or drags the mouse
 * past the bounds of the table. Valid values are:
 * <UL>
 * <LI><code>JCTblEnum.AUTO_SCROLL_NONE</code> (default)
 * <LI><code>JCTblEnum.AUTO_SCROLL_ROW</code>
 * <LI><code>JCTblEnum.AUTO_SCROLL_COLUMN</code>
 * <LI><code>JCTblEnum.AUTO_SCROLL_BOTH</code>
 * </UL>
 * @param value the new <CODE>AutoScroll</CODE> value; one of
 * <code>JCTblEnum.AUTO_SCROLL_NONE</code> (default),
 * <code>JCTblEnum.AUTO_SCROLL_ROW</code>,
 * <code>JCTblEnum.AUTO_SCROLL_COLUMN</code>, or
 * <code>JCTblEnum.AUTO_SCROLL_BOTH</code>
 */
public void setAutoScroll(int value)
{
	if (value != JCTableEnum.AUTO_SCROLL_NONE    &&
	    value != JCTableEnum.AUTO_SCROLL_ROW   &&
	    value != JCTableEnum.AUTO_SCROLL_COLUMN &&
	    value != JCTableEnum.AUTO_SCROLL_BOTH)
	{
		throw new IllegalArgumentException("Parameter value must be one of " +
		                                   "JCTableEnum.AUTO_SCROLL_NONE, " +
										   "JCTableEnum.AUTO_SCROLL_ROW, " +
										   "JCTableEnum.AUTO_SCROLL_COLUMN, " +
										   "JCTableEnum.AUTO_SCROLL_BOTH");
	}
	auto_scroll = value;
}

/**
 * Gets the table's <CODE>CellBorderWidth</CODE> value.
 * @return the current <CODE>CellBorderWidth</CODE> value
 * @see #setCellBorderWidth
 */
public int getCellBorderWidth()
{
	return(cell_border_width);
}

/**
 * Sets the width of the borders surrounding each cell and label (default is
 * <code>1</code>).
 * @param value the new cell border width value
 */
public void setCellBorderWidth(int value)
{
    cell_border_width = value;
    requestChange(TableChangeHandler.TABLE_RESET);
}

/**
 * Returns the <CODE>CharHeight</CODE> value for a row.
 * @param row the row whose height is to be returned
 * @return the current <CODE>CharHeight</CODE> value for the specified row
 * @see #setCharHeight
 */
public int getCharHeight(int row)
{
	CellSize cellSize = (CellSize)rowHeights.getValue(row);
	return(cellSize.getCharValue());
}

/**
 * Sets the number of lines of text that a row can display (default is
 * <code>1</code>).  Multiline cell values that do not fit in a cell or label
 * are clipped. This property controls the height only if
 * <CODE>PixelHeight</CODE> is <CODE>JCTableEnum.NOVALUE</CODE>. <P>This
 * property works best with lists that use fixed-width fonts.<P>When a user
 * interactively resizes a row, this property is overriden by the pixel heights.
 * @param row row whose height is to be set; this may be a specified row,
 *     <CODE>JCTableEnum.LABEL</CODE> (for column labels),
 *     <CODE>JCTableEnum.ALLCELLS</CODE> (for all cells), or
 *     <CODE>JCTableEnum.ALL</CODE> (for all rows).
 * @param value the new <CODE>CharHeight</CODE> value for the specified row
 */
public void setCharHeight(int row, int value)
{
	// Get the original height.
	CellSize cellSize = (CellSize) ((CellSize)rowHeights.getValue(row)).clone();

	// Fire the BEFORE_RESIZE and RESIZE events.
	JCResizeCellEvent event = doResizeEvents(row, 				   // row
								             JCTableEnum.NOVALUE,  // column
								             JCTableEnum.NOVALUE,  // oldWidth
								             cellSize.char_value,  // oldHeight
								             JCTableEnum.NOVALUE,  // new width
								             value);			   // new height

	if (!event.isCancelled()){
		// Do the resize.
		cellSize.setCharValue(value);
		rowHeights.setValue(row, cellSize);
		setRowHeightChanged(row);
	}

	// Fire the AFTER_RESIZE event.
	fireJCResizeCellEvent(event);
}

/**
 * Returns the <CODE>CharWidth</CODE> value for a column.
 * @param column the column whose width is to be returned
 * @return the current <CODE>CharWidth</CODE> value for the specified column
 * @see #setCharWidth
 */
public int getCharWidth(int column)
{
	CellSize cellSize = (CellSize)columnWidths.getValue(column);
	return(cellSize.getCharValue());
}

/**
 * Sets the number of characters of text that a column can display
 * (default is 10). Cell values that do not fit in a cell or label are clipped.
 * This property controls the width only if <CODE>PixelWidth</CODE> is
 * <CODE>JCTableEnum.NOVALUE</CODE>. <P>This property works best with lists that
 * use fixed-width fonts.<P>When a user interactively resizes a column, this
 * property is overriden by the pixel widths.
 * @param column column whose width is to be set; this may be a specified column,
 *     <CODE>JCTableEnum.LABEL</CODE> (for row labels),
 *     <CODE>JCTableEnum.ALLCELLS</CODE> (for all cells), or
 *     <CODE>JCTableEnum.ALL</CODE> (for all columns)
 * @param value the new <CODE>CharWidth</CODE> value for the specified column
 */
public void setCharWidth(int column, int value)
{

	// Get the original width.
	CellSize cellSize =
        (CellSize)((CellSize)columnWidths.getValue(column)).clone();

	// Fire the BEFORE_RESIZE and RESIZE events.
	JCResizeCellEvent event = doResizeEvents(JCTableEnum.NOVALUE,  // row
								             column, 			   // column
								             cellSize.char_value,  // oldWidth
								             JCTableEnum.NOVALUE,  // oldHeight
								             value, 			   // new width
								             JCTableEnum.NOVALUE); // new height

	// If not cancelled, do the resize.
	if (!event.isCancelled()) {
		cellSize.setCharValue(value);
		columnWidths.setValue(column, cellSize);
		setColumnWidthChanged(column);
	}

	// Fire the AFTER_RESIZE event
	fireJCResizeCellEvent(event);
}

/**
 * Returns <code>true</code> if the column is hidden.
 * @param column the column index
 * @return Is the specified column hidden?
 * @see #setColumnHidden
 */
public boolean isColumnHidden(int column)
{
	CellSize cellSize = (CellSize)columnWidths.getValue(column);
	return(cellSize.isHidden());
}

/**
 * Hides or shows a given column.
 * @param column the column index
 * @param value if <code>true</code>, hide the specified column; if
 * <code>false</code>, it is not hidden
 */
public void setColumnHidden(int column, boolean value)
{
	CellSize cellSize =
        (CellSize)((CellSize)columnWidths.getValue(column)).clone();
	cellSize.setHidden(value);
	columnWidths.setValue(column, cellSize);

	// if there's a component in this column, make sure to hide it, too
	if (hasComponents()) {
		Component comp;
		int numRows = getNumRows();
		for (int i = 0; i < numRows; i++) {
			if ((comp = getComponent(i, column)) != null) {
				comp.setVisible(false);
			}
		}
	}

	setColumnWidthChanged(column);
}

/**
 * Gets the table's <CODE>PopupMenuEnabled</CODE> value.
 * @return whether the popup menu is enabled
 * @see #setPopupMenuEnabled
 */
public boolean isPopupMenuEnabled()
{
	return(popupMenuEnabled);
}

/**
 * Sets the table's <CODE>PopupMenuEnabled</CODE> value.
 * @param b if <code>true</code>, the popup menu is now enabled; if
 * <code>false</code>, it is disabled
 * @see #isPopupMenuEnabled
 */
public void setPopupMenuEnabled(boolean b)
{
	popupMenuEnabled = b;
	if (popupMenuEnabled && popupMenu == null) {
        popupMenu = createPopupMenu();
        add(popupMenu);
    }
}

/**
 * Gets the table's <CODE>ColumnLabelDisplay</CODE> value.
 * @return whether the column labels are displayed
 * @see #setColumnLabelDisplay
 */
public boolean isColumnLabelDisplay()
{
	return(column_label_display);
}

/**
 * Determines whether column labels are displayed. Default is <CODE>true</CODE>.
 * @param value if <code>true</code>, column labels are displayed
 */
public void setColumnLabelDisplay(boolean value)
{
	column_label_display = value;
	requestChange(TableChangeHandler.TABLE_RESET);
}

/**
 * Gets the table's <CODE>ColumnLabelOffset</CODE> value.
 * @return the spacing between the column labels and the cell area in pixels
 * @see #setColumnLabelOffset
 */
public int getColumnLabelOffset()
{
	return(column_label_offset);
}

/**
 * Sets the spacing between the column labels and the cell area in pixels
 * (default is 0). If a negative value is specified, the labels will overlap
 * the cell area.
 * @param value the new column label offset
 */
public void setColumnLabelOffset(int value)
{
	column_label_offset = value;
	if (column_label_display) {
		requestChange(TableChangeHandler.TABLE_LAYOUT |
					  TableChangeHandler.TABLE_REPAINT);
	}
}

/**
 * Gets the table's <CODE>ColumnLabelPlacement</CODE> value.
 * @return the current position of the column labels relative to the cells;
 * one of <code>JCTableEnum.PLACE_TOP</CODE> (default) or
 *     <code>JCTableEnum.PLACE_BOTTOM</code>
 * @see #setColumnLabelPlacement
 */
public int getColumnLabelPlacement()
{
	return(column_label_placement);
}

/**
 * Sets the position of the column labels relative to the cells. Valid values
 * are: <UL><LI><CODE>JCTableEnum.PLACE_TOP</CODE> (default)</LI>
 * <LI><CODE>JCTableEnum.PLACE_BOTTOM</CODE></LI></UL>
 * @param value the new column label placement value; one of
 * <code>JCTableEnum.PLACE_TOP</CODE> (default) or
 * <code>JCTableEnum.PLACE_BOTTOM</code>
 */
public void setColumnLabelPlacement(int value)
{
	if (value != JCTableEnum.PLACE_TOP &&
	    value != JCTableEnum.PLACE_BOTTOM)
	{
		throw new IllegalArgumentException("Parameter value must be one of " +
		                                   "JCTableEnum.PLACE_TOP, " +
										   "JCTableEnum.PLACE_BOTTOM");
	}

	column_label_placement = value;
	if (column_label_display) {
		requestChange(TableChangeHandler.TABLE_LAYOUT |
					  TableChangeHandler.TABLE_REPAINT);
	}
}

/**
 * Gets the Component located at the given cell.
 * @param row the row index of the cell
 * @param column the column index of the cell
 * @return the component, if any, at the given cell
 */
public Component getComponent(int row, int column)
{
	ComponentModel componentHandler = getComponentModel();
	if (componentHandler != null) {
		return((Component)componentHandler.getValue(row, column));
	}
	return(null);
}

/**
 * Returns <code>true</code> if the cell contains a component.
 * @param row the row index of the cell
 * @param column the column index of the cell
 * @return whether the cell at (row, column) contains a component
 */
public boolean hasComponent(int row, int column)
{
	return(getComponent(row, column) != null);
}

/**
 * Sets a Component for the contents of the given cell.
 * @param row the row index of the cell
 * @param column the column index of the cell
 * @param component the new component for a cell
 */
public void setComponent(int row, int column, Component component)
{
	ComponentModel componentHandler = getComponentModel();
	if (componentHandler == null) {
		componentHandler = focusManager.createDefaultComponentModel();
	}
	componentHandler.setValue(row, column, component);
}

/**
 * Returns the spacing between the border of a cell and a cell component.
 * @return the current component border width
 * @see #setComponentBorderWidth
 */
public int getComponentBorderWidth()
{
	return(component_border_width);
}

/**
 * Sets the spacing between the border of a cell and a cell component.
 * By default, the spacing is zero, resulting in components that exactly
 * fit into the cell.
 * @param value the new component border width
 */
public void setComponentBorderWidth(int value)
{
	this.component_border_width = value;
}

/**
 * Sets a <code>JComponent</code> to span a range of cells. This is a
 * convenience method which creates a span and then adds the component to it.
 * @param cr the cell range which this component will span
 * @param component the component which will span the range
 */
public void setComponent(JCCellRange cr, Component component)
{
	ComponentModel componentHandler = getComponentModel();
	if (componentHandler == null) {
		componentHandler = focusManager.createDefaultComponentModel();
	}
    // Add the span
    addSpannedRange(cr);
	setComponent(cr.start_row, cr.start_column, component);
}

/**
 * Sets the global cursor when cursor tracking is off.
 * @param type the java.awt.Cursor type (for example,
 * <code>Cursor.WAIT_CURSOR</code>)
 * @see #setTrackCursor
 */
public void setCursor(int type)
{
	setCursor(Cursor.getPredefinedCursor(type));
}

/**
 * Returns the vertical sizing policy for cell editors.
 * @return the current vertical sizing policy for cell editors; one of
 * <code>JCTableEnum.EDIT_SIZE_TO_CELL</code>,
 * <code>JCTableEnum.EDIT_ENSURE_MINIMUM_SIZE</code>, or
 * <code>JCTableEnum.EDIT_ENSURE_PREFERRED_SIZE</code>
 * @see #setEditHeightPolicy
 */
public int getEditHeightPolicy()
{
	return(edit_height_policy);
}

/**
 * Specified how high the editing component of a cell should be when the
 * user clicks on it. Valid values are:
 * <UL>
 * <LI><CODE>JCTableEnum.EDIT_SIZE_TO_CELL</CODE></LI>
 * <LI><CODE>JCTableEnum.EDIT_ENSURE_MINIMUM_SIZE</CODE></LI>
 * <LI><CODE>JCTableEnum.EDIT_ENSURE_PREFERRED_SIZE</CODE></LI>
 * </UL>
 * @param value the new vertical sizing policy for cell editors; one of
 * <code>JCTableEnum.EDIT_SIZE_TO_CELL</code>,
 * <code>JCTableEnum.EDIT_ENSURE_MINIMUM_SIZE</code>, or
 * <code>JCTableEnum.EDIT_ENSURE_PREFERRED_SIZE</code>
 */
public void setEditHeightPolicy(int value)
{
	if (value != JCTableEnum.EDIT_SIZE_TO_CELL        &&
	    value != JCTableEnum.EDIT_ENSURE_MINIMUM_SIZE &&
	    value != JCTableEnum.EDIT_ENSURE_PREFERRED_SIZE)
	{
		throw new IllegalArgumentException("Parameter value must be one of " +
		                                   "JCTableEnum.EDIT_SIZE_TO_CELL, " +
		                                   "JCTableEnum.EDIT_ENSURE_MINIMUM_SIZE, " +
										   "JCTableEnum.EDIT_ENSURE_PREFERRED_SIZE");
	}
	this.edit_height_policy = value;
}

/**
 * Returns the horizontal sizing policy for cell editors.
 * @return the current horizontal sizing policy for cell editors; one of
 * <code>JCTableEnum.EDIT_SIZE_TO_CELL</code>,
 * <code>JCTableEnum.EDIT_ENSURE_MINIMUM_SIZE</code>,
 * or <code>JCTableEnum.EDIT_ENSURE_PREFERRED_SIZE</code>
 * @see #setEditWidthPolicy
 */
public int getEditWidthPolicy()
{
	return(edit_width_policy);
}

/**
 * Specifies how wide the editing component of a cell should be when the
 * user clicks on it. Valid values are:
 * <UL>
 * <LI><CODE>JCTableEnum.EDIT_SIZE_TO_CELL</CODE></LI>
 * <LI><CODE>JCTableEnum.EDIT_ENSURE_MINIMUM_SIZE</CODE></LI>
 * <LI><CODE>JCTableEnum.EDIT_ENSURE_PREFERRED_SIZE</CODE></LI>
 * </UL>
 * @param value the new horizontal sizing policy for cell editors; one of
 * <code>JCTableEnum.EDIT_SIZE_TO_CELL</code>,
 * <code>JCTableEnum.EDIT_ENSURE_MINIMUM_SIZE</code>,
 *     or <code>JCTableEnum.EDIT_ENSURE_PREFERRED_SIZE</code>
 */
public void setEditWidthPolicy(int value)
{
	if (value != JCTableEnum.EDIT_SIZE_TO_CELL        &&
	    value != JCTableEnum.EDIT_ENSURE_MINIMUM_SIZE &&
	    value != JCTableEnum.EDIT_ENSURE_PREFERRED_SIZE)
	{
		throw new IllegalArgumentException("Parameter value must be one of " +
		                                   "JCTableEnum.EDIT_SIZE_TO_CELL, " +
		                                   "JCTableEnum.EDIT_ENSURE_MINIMUM_SIZE, " +
										   "JCTableEnum.EDIT_ENSURE_PREFERRED_SIZE");
	}
	this.edit_width_policy = value;
}

/**
 * Returns the focus color.
 * @return the current focus color
 * @see #setFocusColor
 */
public Color getFocusColor()
{
	return(focus_color);
}

/**
 * Sets the focus color.
 * @param value the new focus color
 */
public void setFocusColor(Color value)
{
	this.focus_color = value;
	requestChange(TableChangeHandler.CELL_REPAINT, current_row, current_column);
}

/**
 * Returns the current <code>FocusManager</code>.
 * @see #setFocusManager
 * @return the current <code>FocusManager</code>
 */
public FocusModel getFocusManager()
{
	return(focusManager);
}

/**
 * Sets a new <code>FocusManager</code>.
 * @param focusManager the new focus manager; null values are ignored
 */
public void setFocusManager(FocusModel focusManager)
{
	if (focusManager != null) {
		this.focusManager = focusManager;
	}
}

/**
 * Returns the focus indicator type.
 * @return the current focus indicator type; one of
 * <code>JCTableEnum.FOCUS_NONE</code>,
 * <code>JCTableEnum.FOCUS_HIGHLIGHT</code>,
 * <code>JCTableEnum.FOCUS_RECTANGLE</code>,
 * <code>JCTableEnum.FOCUS_THIN_RECTANGLE</code>, or
 * <code>JCTableEnum.FOCUS_DASHED_RECTANGLE</code>
 * @see Table#setFocusIndicator
 */
public int getFocusIndicator()
{
	return(focus_indicator);
}

/**
 * Sets the focus indicator. Valid indicators are:
 * <CODE>JCTableEnum.FOCUS_NONE</CODE>,
 * <CODE>JCTableEnum.FOCUS_HIGHLIGHT</CODE>,
 * <CODE>JCTableEnum.FOCUS_RECTANGLE</CODE>,
 * <CODE>JCTableEnum.FOCUS_THIN_RECTANGLE</CODE>, and
 * <CODE>JCTableEnum.FOCUS_DASHED_RECTANGLE</CODE>.
 * @param value the new focus indicator type; one of
 * <code>JCTableEnum.FOCUS_NONE</code>,
 * <code>JCTableEnum.FOCUS_HIGHLIGHT</code>,
 * <code>JCTableEnum.FOCUS_RECTANGLE</code>,
 * <code>JCTableEnum.FOCUS_THIN_RECTANGLE</code>, or
 * <code>JCTableEnum.FOCUS_DASHED_RECTANGLE</code>
 */
public void setFocusIndicator(int value)
{
	if (value != JCTableEnum.FOCUS_NONE           &&
	    value != JCTableEnum.FOCUS_HIGHLIGHT      &&
	    value != JCTableEnum.FOCUS_RECTANGLE      &&
	    value != JCTableEnum.FOCUS_THIN_RECTANGLE &&
	    value != JCTableEnum.FOCUS_DASHED_RECTANGLE)
	{
		throw new IllegalArgumentException("Parameter value must be one of " +
		                                   "JCTableEnum.FOCUS_NONE, " +
		                                   "JCTableEnum.FOCUS_HIGHLIGHT, " +
		                                   "JCTableEnum.FOCUS_RECTANGLE, " +
		                                   "JCTableEnum.FOCUS_THIN_RECTANGLE, " +
										   "JCTableEnum.FOCUS_DASHED_RECTANGLE");
	}
	this.focus_indicator = value;
	requestChange(TableChangeHandler.CELL_REPAINT, current_row, current_column);
}

/**
 * Gets the table's <CODE>FrameBorderWidth</CODE> value as a
 * <code>CellBorderModel</code>.
 * @return the current <CODE>FrameBorderWidth</CODE>
 * <code>CellBorderModel</code> value
 * @see #setFrameBorderWidth
 */
public CellBorderModel getFrameBorder()
{
	return(frame_border);
}

/**
 * Sets the <code>CellBorder</code> model of the borders surrounding cell and
 * label areas (default is null).
 * @param value the new <CODE>FrameBorderWidth</CODE> value specified by
 *    <code>CellBorderModel</code>
 */
public void setFrameBorder(CellBorderModel value)
{
    frame_border = value;
    requestChange(TableChangeHandler.TABLE_REPAINT);
}

/**
 * Gets the table's <CODE>FrameBorderWidth</CODE> value.
 * @return the current <CODE>FrameBorderWidth</CODE> value
 * @see #setFrameBorderWidth
 */
public int getFrameBorderWidth()
{
	return(frame_border_width);
}

/**
 * Sets the width of the borders surrounding cell and label areas (default is 0).
 * @param value the new <CODE>FrameBorderWidth</CODE> value
 */
public void setFrameBorderWidth(int value)
{
    frame_border_width = value;
    requestChange(TableChangeHandler.TABLE_RESET);
}

/**
 * Gets the table's <CODE>FrozenColumnPlacement</CODE> value.
 * @return the current <CODE>FrozenColumnPlacement</CODE> value; one of
 * <code>JCTableEnum.PLACE_LEFT</code> or <code>JCTableEnum.PLACE_RIGHT</code>
 * @see #setFrozenColumnPlacement
 */
public int getFrozenColumnPlacement()
{
	return(frozen_column_placement);
}

/**
 * Sets the location of all frozen columns within the component display.
 * Changing the placement of frozen columns does not change the location
 * of the columns in the table's internal <code>CellValue</code>s. One of
 * <code>JCTableEnum.PLACE_LEFT</code> or
 * <code>JCTableEnum.PLACE_RIGHT</code>.
 * @param value the new <CODE>FrozenColumnPlacement</CODE> value; one of
 * <code>JCTableEnum.PLACE_LEFT</code> or <code>JCTableEnum.PLACE_RIGHT</code>
 */
public void setFrozenColumnPlacement(int value)
{
	if (value != JCTableEnum.PLACE_LEFT &&
	    value != JCTableEnum.PLACE_RIGHT)
	{
		throw new IllegalArgumentException("Parameter value must be one of " +
		                                   "JCTableEnum.PLACE_LEFT, " +
										   "JCTableEnum.PLACE_RIGHT");
	}

    frozen_column_placement = value;
	requestChange(TableChangeHandler.TABLE_RESET);
}

/**
 * Gets the table's <CODE>FrozenColumns</CODE> value.
 * @return the current <CODE>FrozenColumns</CODE> value
 * @see #setFrozenColumns
 */
public int getFrozenColumns()
{
	return(frozen_columns);
}

/**
 * Sets the number of columns from the start of the table that are not
 * horizontally scrollable (default is 0). Placement of the frozen columns
 * on the screen is specified by <CODE>FrozenColumnPlacement</CODE>.
 * @param value the new <CODE>FrozenColumns</CODE> value
 */
public void setFrozenColumns(int value)
{
    frozen_columns = value;
	requestChange(TableChangeHandler.TABLE_RESET);
	getEditTraverseHandler().checkAndReparentEditor();
}

/**
 * Gets the table's <CODE>FrozenRowPlacement</CODE> value.
 * @return the current frozen row placement value; one of
 * <code>JCTableEnum.PLACE_TOP</code> or <code>JCTableEnum.PLACE_BOTTOM</code>
 * @see #setFrozenRowPlacement
 */
public int getFrozenRowPlacement()
{
	return(frozen_row_placement);
}

/**
 * Sets the location of all frozen rows. Changing the placement of
 * frozen rows does not change the location of the rows in the table's
 * internal CellValues. Valid values are:
 * <UL><LI><CODE>JCTableEnum.PLACE_TOP</CODE></LI>
 * <LI><CODE>JCTableEnum.PLACE_BOTTOM</CODE></LI></UL>.
 * @param value the new frozen row placement value; one of
 * <code>JCTableEnum.PLACE_TOP</code> or <code>JCTableEnum.PLACE_BOTTOM</code>
 */
public void setFrozenRowPlacement(int value)
{
	if (value != JCTableEnum.PLACE_TOP &&
	    value != JCTableEnum.PLACE_BOTTOM)
	{
		throw new IllegalArgumentException("Parameter value must be one of " +
		                                   "JCTableEnum.PLACE_TOP, " +
										   "JCTableEnum.PLACE_BOTTOM");
	}

    frozen_row_placement = value;
	requestChange(TableChangeHandler.TABLE_RESET);
}

/**
 * Gets the table's <CODE>FrozenRows</CODE> value.
 * @return the current <CODE>FrozenRows</CODE> value
 * @see #setFrozenRows
 */
public int getFrozenRows()
{
	return(frozen_rows);
}

/**
 * Sets the number of rows from the start of the table that are not
 * vertically scrollable (default is 0). Placement of the frozen rows
 * on the screen is specified by <CODE>FrozenRowPlacement</CODE>.
 * @param value the new <CODE>FrozenRows</CODE> value
 */
public void setFrozenRows(int value)
{
    frozen_rows = value;
	requestChange(TableChangeHandler.TABLE_RESET);
	getEditTraverseHandler().checkAndReparentEditor();
}

/**
 * Gets the horizontal scrollbar.
 * @return the current horizontal scrollbar component
 */
public JScrollBar getHorizSB()
{
	return(hsb);
}

/**
 * Sets the horizontal scrollbar.
 * @param value the new horizontal scrollbar component
 */
public void setHorizSB(JScrollBar value)
{
	if (hsb != null) {
	    hsb.removeAdjustmentListener(adjustmentHandler);
		remove(hsb);
		hsb = null;
	}
	hsb = value;
	hsb.addAdjustmentListener(adjustmentHandler);
	add(hsb);
	requestChange(TableChangeHandler.TABLE_RESET);
}

/**
 * Gets the tables <CODE>HorizSBAttachment</CODE> value.
 * @return the current <CODE>HorizSBAttachment</CODE> value; one of
 * <code>JCTableEnum.SIZE_TO_CELLS</code> or
 * <code>JCTableEnum.SIZE_TO_TABLE</code>
 * @see #setHorizSBAttachment
 */
public int getHorizSBAttachment()
{
	return(hsb_attach);
}

/**
 * Sets how the end of the horizontal scrollbar is attached to the table.
 * When set to <CODE>JCTableEnum.SIZE_TO_CELLS</CODE> (default), the
 * scrollbar ends at the edge of the visible cells. <P>
 * When set to <CODE>JCTableEnum.SIZE_TO_TABLE</CODE>, the scrollbar ends
 * at the edge of the table. This setting should be used in conjunction
 * with <CODE>HorizSBPosition</CODE>.
 * @param value the new <CODE>HorizSBAttachment</CODE> value; one of
 * <code>JCTableEnum.SIZE_TO_CELLS</code> or <code>JCTableEnum.SIZE_TO_TABLE</code>
 */
public void setHorizSBAttachment(int value)
{
	if (value != JCTableEnum.SIZE_TO_CELLS &&
	    value != JCTableEnum.SIZE_TO_TABLE)
	{
		throw new IllegalArgumentException("Parameter value must be one of " +
		                                   "JCTableEnum.SIZE_TO_CELLS, " +
										   "JCTableEnum.SIZE_TO_TABLE");
	}
	hsb_attach = value;
	requestChange(TableChangeHandler.TABLE_LAYOUT);
	if (hsb != null) {
		hsb.invalidate();
		hsb.validate();
	}
}

/**
 * Gets the table's <CODE>HorizSBDisplay</CODE> value to determine when to
 * display scrollbars.
 * @return the current <CODE>HorizSBDisplay</CODE> value; one of
 * <code>JCTableEnum.SCROLLBAR_ALWAYS</code>,
 * <code>JCTableEnum.SCROLLBAR_NEVER</code>, or
 * <code>JCTableEnum.SCROLLBAR_AS_NEEDED</code>
 * @see #setHorizSBDisplay
 */
public int getHorizSBDisplay()
{
	return(hsb_display);
}

/**
 * Sets when to display the horizontal scrollbar. Valid values are:
 * <PRE>
 * JCTableEnum.SCROLLBAR_ALWAYS        Always displayed
 * JCTableEnum.SCROLLBAR_NEVER         Never displayed
 * JCTableEnum.SCROLLBAR_AS_NEEDED     Displayed only when table
 *                                     area is larger than frame
 * </PRE>
 * @param value The new <CODE>HorizSBDisplay</CODE> value; one of
 * <code>JCTableEnum.SCROLLBAR_ALWAYS</code>,
 * <code>JCTableEnum.SCROLLBAR_NEVER</code>, or
 * <code>JCTableEnum.SCROLLBAR_AS_NEEDED</code>
 */
public void setHorizSBDisplay(int value)
{
	if (value != JCTableEnum.SCROLLBAR_ALWAYS &&
	    value != JCTableEnum.SCROLLBAR_NEVER  &&
	    value != JCTableEnum.SCROLLBAR_AS_NEEDED)
	{
		throw new IllegalArgumentException("Parameter value must be one of " +
		                                   "JCTableEnum.SCROLLBAR_ALWAYS, " +
		                                   "JCTableEnum.SCROLLBAR_NEVER, " +
										   "JCTableEnum.SCROLLBAR_AS_NEEDED");
	}
	hsb_display = value;
	requestChange(TableChangeHandler.TABLE_LAYOUT);
}

/**
 * Gets the table's <CODE>HorizSBOffset</CODE> value.
 * @return the current <CODE>HorizSBOffset</CODE> value
 * @see #setHorizSBOffset
 */
public int getHorizSBOffset()
{
	return(hsb_offset);
}

/**
 * Sets the distance (in pixels) between the horizontal scrollbar and
 * the table (default is 0).
 * @param value the new <CODE>HorizSBOffset</CODE> value
 */
public void setHorizSBOffset(int value)
{
	hsb_offset = value;
	if (hsb_offset < 0) {
		hsb_offset = 0;
	}
	requestChange(TableChangeHandler.TABLE_LAYOUT);
}

/**
 * Gets the table's <CODE>HorizSBPosition</CODE> value.
 * @return the current <CODE>HorizSBPosition</CODE> value; one of
 * <code>JCTableEnum.POSITION_BY_CELLS</code> or
 * <code>JCTableEnum.POSITION_AT_SIDE</code>
 * @see #setHorizSBPosition
 */
public int getHorizSBPosition()
{
	return(hsb_position);
}

/**
 * Sets how the horizontal scrollbar is attached to the table. When set
 * to <CODE>JCTableEnum.POSITION_BY_CELLS</CODE> (default), the scrollbar
 * is attached to the cell/labels. <P> When set to
 * <CODE>JCTableEnum.POSITION_AT_SIDE</CODE>, the scrollbar is attached to the
 * edge of the table within its container.
 * @param value the new <CODE>HorizSBPosition</CODE> value; one of
 * <code>JCTableEnum.POSITION_BY_CELLS</code> or
 * <code>JCTableEnum.POSITION_AT_SIDE</code>
 * @see #setHorizSBPosition
 */
public void setHorizSBPosition(int value)
{
	if (value != JCTableEnum.POSITION_BY_CELLS &&
	    value != JCTableEnum.POSITION_AT_SIDE)
	{
		throw new IllegalArgumentException("Parameter value must be one of " +
		                                   "JCTableEnum.POSITION_BY_CELLS, " +
										   "JCTableEnum.POSITION_AT_SIDE");
	}
	hsb_position = value;
	requestChange(TableChangeHandler.TABLE_LAYOUT);
}

/**
 * Gets the table's <CODE>HorizSBTrack</CODE> value.
 * @return the current <CODE>HorizSBTrack</CODE> value; one of
 * <code>JCTableEnum.TRACK_LIVE</code>,
 * <code>JCTableEnum.TRACK_COLUMN_NUMBER</code>, or
 * <code>JCTableEnum.TRACK_ROW</code>
 * @see #setHorizSBTrack
 */
public int getHorizSBTrack()
{
	return(hsb_track);
}

/**
 * Sets how the horizontal scrollbar works during tracking. When set to
 * <CODE>JCTableEnum.TRACK_LIVE</CODE>, table redisplays during scrollbar
 * tracking. When set to <CODE>JCTableEnum.TRACK_COLUMN_NUMBER</CODE>, table
 * redisplays after tracking completes and displays the number of the left
 * column during the track.
 * Use <CODE>JCTableEnum.TRACK_ROW</CODE> and <CODE>setHorizSBTrackRow</CODE>
 * to specify a row number in order to use data from the column to indicate
 * the left-most column.
 * @param value the new <CODE>HorizSBTrack</CODE> value; one of
 * <code>JCTableEnum.TRACK_LIVE</code>,
 * <code>JCTableEnum.TRACK_COLUMN_NUMBER</code>, or
 * <code>JCTableEnum.TRACK_ROW</code>
 */
public void setHorizSBTrack(int value)
{
	if (value != JCTableEnum.TRACK_LIVE          &&
	    value != JCTableEnum.TRACK_COLUMN_NUMBER &&
	    value != JCTableEnum.TRACK_ROW)
	{
		throw new IllegalArgumentException("Parameter value must be one of " +
		                                   "JCTableEnum.TRACK_LIVE, " +
		                                   "JCTableEnum.TRACK_COLUMN_NUMBER, " +
										   "JCTableEnum.TRACK_ROW");
	}
	this.hsb_track = value;
}

/**
 * Gets the row number that determines the text to display when the
 * track indicator is displayed.
 * @return the current row number to do the tracking
 * @see #setHorizSBTrackRow
 */
public int getHorizSBTrackRow()
{
	return(hsb_track_row);
}

/**
 * Sets the row number that determines the text to display when the
 * track indicator is displayed. Use <CODE>JCTableEnum.LABEL</CODE>
 * to display the data from the column label or a valid row number
 * to display cell data.
 * @param value the new row number to do the tracking
 */
public void setHorizSBTrackRow(int value)
{
	this.hsb_track_row = value;
}

/**
 * Gets the table's <code>IgnoreContainerSize</code> value.
 * <p><B>Note:</b> <code>JScrollPane</code> contains table correctly without
 * using this logic -- so this is not implemented anywhere.
 *  @return the current <code>IgnoreContainerSize</code> value
 * @see #setIgnoreContainerSize
 *
 */
public boolean isIgnoreContainerSize()
{
	return(ignore_container_size);
}

/**
 * Determines whether the size of the table is determined by its
 * container.  This property should be set to <code>true</code> when the table
 * is placed in a scrollpane so that table will take the
 * maximum amount of space required to display all cells.
 * @param value the new <code>IgnoreContainerSize</code> value
 */
public void setIgnoreContainerSize(boolean value)
{
	this.ignore_container_size = value;
	requestChange(TableChangeHandler.TABLE_LAYOUT);
}

/**
 * Gets the table's <CODE>JumpScroll</CODE> value.
 * @return the current <CODE>JumpScroll</CODE> value; one of
 * <code>JCTableEnum.JUMP_NONE</code>, <code>JCTableEnum.JUMP_HORIZONTAL</code>,
 * <code>JCTableEnum.JUMP_VERTICAL</code>, or <code>JCTableEnum.JUMP_ALL</code>
 * @see #setJumpScroll
 */
public int getJumpScroll()
{
	return(jump_mode);
}

/**
 * Sets the scrolling behavior of each scrollbar. Scrollbars can either scroll
 * smoothly or "jump" scroll in whole row/column increments. When set to
 * <CODE>JCTableEnum.JUMP_NONE</CODE> (default), both scrollbars will scroll
 * smoothly. To enable jump scroll for the horizontal, vertical or both
 * scrollbars, use <CODE>JCTableEnum.JUMP_HORIZONTAL</CODE>,
 * <CODE>JCTableEnum.JUMP_VERTICAL</CODE>, or <CODE>JCTableEnum.JUMP_ALL</CODE>
 * respectively.
 * @param value the new <CODE>JumpScroll</CODE> value; one of
 * <code>JCTableEnum.JUMP_NONE</code>, <code>JCTableEnum.JUMP_HORIZONTAL</code>,
 * <code>JCTableEnum.JUMP_VERTICAL</code>, or <code>JCTableEnum.JUMP_ALL</code>
 */
public void setJumpScroll(int value)
{
	if (value != JCTableEnum.JUMP_NONE       &&
	    value != JCTableEnum.JUMP_HORIZONTAL        &&
	    value != JCTableEnum.JUMP_VERTICAL &&
	    value != JCTableEnum.JUMP_ALL)
	{
		throw new IllegalArgumentException("Parameter value must be one of " +
		                                   "JCTableEnum.JUMP_NONE, " +
		                                   "JCTableEnum.JUMP_HORIZONTAL, " +
		                                   "JCTableEnum.JUMP_VERTICAL, " +
										   "JCTableEnum.JUMP_ALL");
	}
	this.jump_mode = value;
}

/**
 * Gets the table's <CODE>LeftColumn</CODE> value.
 * @return the current <CODE>LeftColumn</CODE> value
 * @see #setLeftColumn
 */
public int getLeftColumn()
{
	if (left_column < frozen_columns) {
		left_column = frozen_columns;
	}
	return left_column;
}

/**
 * Sets the non-frozen column at least partially visible at the left side
 * of the window. This value is updated as a user scrolls through a table.<P>
 * When set, the table scrolls to display the column at the left side of the
 * window. This value is always larger or equal to the number of frozen columns.
 * @param value the new <CODE>LeftColumn</CODE> value
 */
public void setLeftColumn(int value)
{
	left_column = Math.min(value, getDataView().getNumColumns() - 1);
	left_column = Math.max(left_column, frozen_columns);
	set_left_column = true;
	if (needs_initial_set_left_check) {
		set_initial_left = true;
	}
	requestChange(TableChangeHandler.TABLE_LAYOUT |
	              TableChangeHandler.TABLE_REPAINT);
}

/**
 * Gets the table's <CODE>MarginHeight</CODE> value.
 * @return the current <CODE>MarginHeight</CODE> value
 * @see #setMarginHeight
 */
public int getMarginHeight()
{
	return(margin_height);
}

/**
 * Sets the distance (in pixels) between the inside edge
 * of the cell border and the top and bottom edge of the cell's contents
 * (default it 2).
 * @param value the new <CODE>MarginHeight</CODE> value
 */
public void setMarginHeight(int value)
{
	// Fire the BEFORE_RESIZE and RESIZE events.
	JCResizeCellEvent event = doResizeEvents(JCTableEnum.NOVALUE,  // row
								             JCTableEnum.NOVALUE,  // column
								             JCTableEnum.NOVALUE,  // oldWidth
								             margin_height, 	   // oldHeight
								             JCTableEnum.NOVALUE,  // new width
								             value);			   // new height

	if (!event.isCancelled()) {
	    margin_height = value;
		requestChange(TableChangeHandler.TABLE_RESET);
	}

	// Fire the AFTER_RESIZE event
	fireJCResizeCellEvent(event);
}

/**
 * Gets the table's <CODE>MarginWidth</CODE> value.
 * @return the current <CODE>MarginWidth</CODE> value
 * @see #setMarginWidth
 */
public int getMarginWidth()
{
	return(margin_width);
}

/**
 * Sets the distance (in pixels) between the
 * inside edge of the cell border and the left/right edge of the cell's
 * contents (default is 3). This property affects the entire table.
 * @param value the new <CODE>MarginWidth</CODE> value
 */
public void setMarginWidth(int value)
{
	// Fire the BEFORE_RESIZE and RESIZE events.
	JCResizeCellEvent event = doResizeEvents(JCTableEnum.NOVALUE,  // row
								             JCTableEnum.NOVALUE,  // column
								             margin_width, 		   // oldWidth
								             JCTableEnum.NOVALUE,  // oldHeight
								             value, 			   // new width
								             JCTableEnum.NOVALUE); // new height

	if (!event.isCancelled()) {
	    margin_width = value;
		requestChange(TableChangeHandler.TABLE_RESET);
	}

	// Fire the AFTER_RESIZE event
	this.fireJCResizeCellEvent(event);
}

/**
 * Returns the <CODE>MaxHeight</CODE> value for a row.
 * @param row the row whose maximum height is to be returned
 * @return the current <CODE>MaxHeight</CODE> value for a row
 * @see #setMaxHeight
 */
public int getMaxHeight(int row)
{
	CellSize cellSize = (CellSize)rowHeights.getValue(row);
	return(cellSize.getMaxValue());
}

/**
 * Sets the maximmum number of pixels for a row.
 * @param row row  whose maximum height is to be set; this may be a specified
 * row, <CODE>JCTableEnum.LABEL</CODE> (for column labels),
 * <CODE>JCTableEnum.ALLCELLS</CODE> (for all cells), or
 * <CODE>JCTableEnum.ALL</CODE> (for all rows)
 * @param value the new <CODE>MaxHeight</CODE> value for a row
 */
public void setMaxHeight(int row, int value)
{
	CellSize cellSize = (CellSize) ((CellSize)rowHeights.getValue(row)).clone();
	cellSize.setMaxValue(value);
	rowHeights.setValue(row, cellSize);
	setRowHeightChanged(row);
}

/**
 * Returns the <CODE>MaxWidth</CODE> value for a column.
 * @param column the column whose maximum width is to be returned
 * @return the current <CODE>MaxWidth</CODE> value for a column
 * @see #setMaxWidth
 */
public int getMaxWidth(int column)
{
	CellSize cellSize = (CellSize)columnWidths.getValue(column);
	return(cellSize.getMaxValue());
}

/**
 * Sets the maximmum number of pixels for a column.
 * @param column column whose maximum width is to be set; this may be a
 * specified column, <CODE>JCTableEnum.LABEL</CODE> (for row labels),
 * <CODE>JCTableEnum.ALLCELLS</CODE> (for all cells), or
 * <CODE>JCTableEnum.ALL</CODE> (for all columns)
 * @param value the new <CODE>MaxWidth</CODE> value for a column
 */
public void setMaxWidth(int column, int value)
{
	CellSize cellSize =
        (CellSize)((CellSize)columnWidths.getValue(column)).clone();
	cellSize.setMaxValue(value);
	columnWidths.setValue(column, cellSize);
	setColumnWidthChanged(column);
}

/**
 * Gets the table's <CODE>MinCellVisibility</CODE> value.
 * @return the current <CODE>MinCellVisibility</CODE> value
 * @see #setMinCellVisibility
 */
public int getMinCellVisibility()
{
	return(min_cell_visibility);
}

/**
 * Sets the minimum amount of a cell that is made visible when edited. When the
 * table scrolls to edit a non-visible cell, this property determines the
 * percentage of the cell that is scrolled into view. <P>When set to 100, the
 * cell is made completely visible.<P>When set to 10, only 10% of the cell is
 * made visible.
 * @param value the new <CODE>MinCellVisibility</CODE> value
 */
public void setMinCellVisibility(int value)
{
	this.min_cell_visibility = value;
}

/**
 * Returns the <CODE>MinHeight</CODE> value for a row.
 * @param row the row whose minimum height is to be returned
 * @return the current <CODE>MinHeight</CODE> value for a row
 * @see #setMinHeight
 */
public int getMinHeight(int row)
{
	CellSize cellSize = (CellSize)rowHeights.getValue(row);
	return(cellSize.getMinValue());
}

/**
 * Sets the minimum number of pixels for a row.
 * @param row row whose minimum height is to be set; this may be a specified
 * row, <CODE>JCTableEnum.LABEL</CODE> (for column labels),
 * <CODE>JCTableEnum.ALLCELLS</CODE> (for all cells), or
 * <CODE>JCTableEnum.ALL</CODE> (for all rows)
 * @param value the new <CODE>MinHeight</CODE> value for a row
 */
public void setMinHeight(int row, int value)
{
	CellSize cellSize = (CellSize) ((CellSize)rowHeights.getValue(row)).clone();
	cellSize.setMinValue(value);
	rowHeights.setValue(row, cellSize);
	setRowHeightChanged(row);
}

/**
 * Returns the <CODE>MinWidth</CODE> value for a column.
 * @param column the column whose minimum width is to be returned
 * @return the current <CODE>MinWidth</CODE> value for a column
 * @see #setMinWidth
 */
public int getMinWidth(int column)
{
	CellSize cellSize = (CellSize)columnWidths.getValue(column);
	return(cellSize.getMinValue());
}

/**
 * Sets the minimum number of pixels for a column.
 * @param column column whose minimum width is to be set; this may be a
 * specified column, <CODE>JCTableEnum.LABEL</CODE> (for row labels),
 * <CODE>JCTableEnum.ALLCELLS</CODE> (for all cells), or
 * <CODE>JCTableEnum.ALL</CODE> (for all columns)
 * @param value the new <CODE>MinWidth</CODE> value for a column
 */
public void setMinWidth(int column, int value)
{
	CellSize cellSize =
        (CellSize)((CellSize)columnWidths.getValue(column)).clone();
	cellSize.setMinValue(value);
	columnWidths.setValue(column, cellSize);
	setColumnWidthChanged(column);
}


/**
 * Returns the <CODE>PixelHeight</CODE> value for a row, previous set using
 * <CODE>setPixelHeight</CODE>.
 * @param row the row whose height is to be returned
 * @return the current <CODE>PixelHeight</CODE> value for a row
 * @see #setPixelHeight
 */
public int getPixelHeight(int row)
{
	CellSize cellSize = (CellSize)rowHeights.getValue(row);
	return(cellSize.getPixelValue());
}

/**
 * Sets the height of a row in pixels. Use <CODE>JCTableEnum.NOVALUE</CODE> to
 * control height by character value.<P>
 * The following special values are valid:
 * <pre>
 *JCTableEnum.NOVALUE            Use character setting to control
 *                               the size of a row
 *JCTableEnum.AS_IS              Do not change. Use to convert from
 *                               VARIABLE to fixed size
 *JCTableEnum.VARIABLE           Set to the highest value in the row
 *JCTableEnum.VARIABLE_ESTIMATE  Set to the highest value in the
 *                               first <CODE>VariableCount</CODE> cells
 * </pre>
 *<P>No pixel setting can violate the minimum and maximum values for a row.
 * @param row the row whose height is being set
 * @param value the new <CODE>PixelHeight</CODE> value for a row
 */
public void setPixelHeight(int row, int value)
{
	CellSize cellSize = (CellSize) ((CellSize)rowHeights.getValue(row)).clone();

	// Fire the BEFORE_RESIZE and RESIZE events.
	JCResizeCellEvent event = doResizeEvents(row, 				   // row
								             JCTableEnum.NOVALUE,  // column
								             JCTableEnum.NOVALUE,  // oldWidth
								             cellSize.pixel_value, // oldHeight
								             JCTableEnum.NOVALUE,  // new width
								             value);			   // new height

	if (!event.isCancelled()) {
		cellSize.setPixelValue(value);
		rowHeights.setValue(row, cellSize);
		setRowHeightChanged(row);
	}

	// Fire the AFTER_RESIZE event
	this.fireJCResizeCellEvent(event);
}

/**
 * Returns the <CODE>PixelWidth</CODE> value for a column, previously set using
 * <CODE>setPixelWidth</CODE>.
 * @param column the column whose width is to be returned
 * @return the current <CODE>PixelWidth</CODE> value for a column
 * @see #setPixelWidth
 */
public int getPixelWidth(int column)
{
	CellSize cellSize = (CellSize)columnWidths.getValue(column);
	return(cellSize.getPixelValue());
}

/**
 * Sets the width of a column in pixels. Use <CODE>JCTableEnum.NOVALUE</CODE>
 * to control width by character value.<P>
 * The following special values are valid:
 * <pre>
 *JCTableEnum.NOVALUE            Use character setting to control
 *                               the size of a column
 *JCTableEnum.AS_IS              Do not change. Use to convert from
 *                               VARIABLE to fixed size
 *JCTableEnum.VARIABLE           Set to the highest value in the column
 *JCTableEnum.VARIABLE_ESTIMATE  Set to the highest value in the
 *                               first <CODE>VariableCount</CODE> cells
 * </pre>
 *<P>No pixel setting can violate the minimum and maximum values for a column.
 * @param column the column whose width is being set
 * @param value the new <CODE>PixelWidth</CODE> value for a column
 */
public void setPixelWidth(int column, int value)
{
	CellSize cellSize = (CellSize) ((CellSize)columnWidths.getValue(column)).clone();

	// Fire the BEFORE_RESIZE and RESIZE events.
    JCResizeCellEvent event =
        doResizeEvents(JCTableEnum.NOVALUE,	        // row
                       column, 				        // column
                       cellSize.getPixelValue(), 	// oldWidth
                       JCTableEnum.NOVALUE, 	    // oldHeight
                       value, 					    // new width
                       JCTableEnum.NOVALUE);	    // new height

	// If not cancelled, do the resize.
	if (!event.isCancelled()) {
		cellSize.setPixelValue(value);
		columnWidths.setValue(column, cellSize);
		setColumnWidthChanged(column);
	}

	// Fire the AFTER_RESIZE event
	this.fireJCResizeCellEvent(event);
}

/**
 * Gets the table's <CODE>RepaintEnabled</CODE> value.
 * @return the current <CODE>RepaintEnabled</CODE> value
 * @see #setRepaintEnabled
 */
public boolean isRepaintEnabled()
{
	return(repaint_enabled);
}

/**
 * Sets whether the table should be redrawn and recomputed whenever one of its
 * properties is set (default is <CODE>true</CODE>). If <CODE>false</CODE>,
 * setting a property will have no effect until <CODE>RepaintEnabled</CODE> is
 * switched to <code>true</code>.
 * @param value the new <CODE>RepaintEnabled</CODE> value
 */
public void setRepaintEnabled(boolean value)
{
	repaint_enabled = value;
	if (repaint_enabled && (needs_repaint || needs_recalc || needs_relayout)) {
		requestChange(TableChangeHandler.TABLE_RESET);
	}
}

/**
 * Gets the table's <CODE>ResizeEven</CODE> value to determine whether all
 * column widths or row heights are resized to the same value.
 * @return the current <CODE>ResizeEven</CODE> value
 * @see #setResizeEven
 */
public boolean isResizeEven()
{
	return(resize_even);
}

/**
 * If set to <CODE>true</CODE>, when a column width or row height is changed
 * interactively, all columns or rows are resized to the same value.  Labels
 * are not affected.
 * @param value the new <CODE>ResizeEven</CODE> value
 */
public void setResizeEven(boolean value)
{
    resize_even = value;
}

/**
 * Gets the table's <CODE>ResizeInteractive</CODE> value to determine
 * whether table repaints during the resize of a column or row.
 * @return the current <CODE>ResizeInteractive</CODE> value
 * @see #setResizeInteractive
 */
public boolean isResizeInteractive()
{
	return(resize_interactive);
}

/**
 * If set to <CODE>true</CODE>, when a column width or row height is changed
 * interactively, table is repainted to reflect the new value during
 * the drag event.
 * @param value the new <CODE>ResizeInteractive</CODE> value
 */
public void setResizeInteractive(boolean value)
{
	this.resize_interactive = value;
}

/**
 * Returns <code>true</code> if the row is hidden.
 * @param row the row index
 * @return whether the specified row is hidden
 * @see #setRowHidden
 */
public boolean isRowHidden(int row)
{
	CellSize cellSize = (CellSize)rowHeights.getValue(row);
	return(cellSize.isHidden());
}

/**
 * Hides or shows a given row.
 * @param row the row index
 * @param value if <code>true</code>, hide the specified row; if
 * <code>false</code>, it is not hidden
 */
public void setRowHidden(int row, boolean value)
{
	CellSize cellSize = (CellSize) ((CellSize)rowHeights.getValue(row)).clone();
	cellSize.setHidden(value);
	rowHeights.setValue(row, cellSize);

    // if there's a component in this column, make sure to hide it, too
    if (hasComponents()) {
        Component comp;
        int numColumns = getNumColumns();
        for (int i = 0; i < numColumns; i++) {
            if ((comp = getComponent(i, row)) != null) {
                comp.setVisible(false);
            }
        }
    }

	setRowHeightChanged(row);
}

/**
 * Gets the table's <CODE>RowLabelDisplay</CODE> value.
 * @return the current <CODE>RowLabelDisplay</CODE> value
 * @see #setRowLabelDisplay
 */
public boolean isRowLabelDisplay()
{
	return(row_label_display);
}

/**
 * Determines whether row labels are displayed. Default is <CODE>true</CODE>.
 * @param value the new <CODE>RowLabelDisplay</CODE> value
 */
public void setRowLabelDisplay(boolean value)
{
	row_label_display = value;
	requestChange(TableChangeHandler.TABLE_RESET);
}

/**
 * Gets the table's <CODE>RowLabelOffset</CODE> value.
 * @return the current <CODE>RowLabelOffset</CODE> value which specifies
 *     the spacing between the row labels and the cell area in pixels
 * @see #setRowLabelOffset
 */
public int getRowLabelOffset()
{
	return(row_label_offset);
}

/**
 * Sets the spacing between the row labels and the cell area in pixels
 * (default is 0). If a negative value is specified, the labels will overlap
 * the cell area.
 * @param value the new <CODE>RowLabelOffset</CODE> value
 */
public void setRowLabelOffset(int value)
{
	row_label_offset = value;
	if (row_label_display) {
		requestChange(TableChangeHandler.TABLE_LAYOUT);
	}
}

/**
 * Gets the table's <CODE>RowLabelPlacement</CODE> value.
 * @return the current <CODE>RowLabelPlacement</CODE> value; one of
 * <code>JCTableEnum.PLACE_LEFT</code> or <code>JCTableEnum.PLACE_RIGHT</code>
 * @see #setRowLabelPlacement
 */
public int getRowLabelPlacement()
{
	return(row_label_placement);
}

/**
 * Sets the position of the row labels relative to the cells. Valid values are:
 * <UL><LI><CODE>JCTableEnum.PLACE_LEFT</CODE> (default)</LI>
 * <LI><CODE>JCTableEnum.PLACE_RIGHT</CODE></LI></UL>
 * @param value the current <CODE>RowLabelPlacement</CODE> value; one of
 * <code>JCTableEnum.PLACE_LEFT</code> or <code>JCTableEnum.PLACE_RIGHT</code>
 */
public void setRowLabelPlacement(int value)
{
	if (value != JCTableEnum.PLACE_LEFT &&
	    value != JCTableEnum.PLACE_RIGHT)
	{
		throw new IllegalArgumentException("Parameter value must be one of " +
		                                   "JCTableEnum.PLACE_LEFT, " +
										   "JCTableEnum.PLACE_RIGHT");
	}

	row_label_placement = value;
	if (row_label_display) {
		requestChange(TableChangeHandler.TABLE_LAYOUT);
	}
}

/**
 * Gets the table's <CODE>SelectedBackground</CODE> value.
 * @return the current <CODE>SelectedBackground</CODE> value
 * @see #setSelectedBackground
 */
public Color getSelectedBackground()
{
	return(selected_background);
}

/**
 * Sets the background color of user-selected cells (default is the cell's
 * foreground color). Use <code>SelectedBackgroundMode</code> to determine how
 * the selected background is determined.
 * @param value the new <CODE>SelectedBackground</CODE> value
 * @see #setSelectedBackgroundMode
 */
public void setSelectedBackground(Color value)
{
	selected_background = value;
    requestChange(TableChangeHandler.TABLE_REPAINT);
}

/**
 * Gets the table's <CODE>SelectedBackgroundMode</CODE> value.
 * @return the current <CODE>SelectedBackgroundMode</CODE> value; one of
 * <code>JCTableEnum.USE_SELECTED_BACKGROUND</code>,
 * <code>JCTableEnum.USE_CELL_BACKGROUND</code>, or
 * <code>JCTableEnum.USE_CELL_FOREGROUND</code>
 * @see #setSelectedBackgroundMode
 */
public int getSelectedBackgroundMode()
{
	return(selected_background_mode);
}

/**
 * Sets the table's <CODE>SelectedBackgroundMode</CODE>.
 * If the mode is <CODE>JCTableEnum.USE_SELECTED_BACKGROUND</CODE>,
 * the selected background color is determined by the
 * <CODE>SelectedBackground</CODE> property. In order to keep the
 * selected background identical to the cell background, use
 * <CODE>JCTableEnum.USE_CELL_BACKGROUND</CODE>. To inverse
 * the background/foreground colors, use
 * <CODE>JCTableEnum.USE_CELL_FOREGROUND</CODE>.
 * @param value the new <CODE>SelectedBackgroundMode</CODE> value; one of
 * <code>JCTableEnum.USE_SELECTED_BACKGROUND</code>,
 * <code>JCTableEnum.USE_CELL_BACKGROUND</code>, or
 * <code>JCTableEnum.USE_CELL_FOREGROUND</code>
 */
public void setSelectedBackgroundMode(int value)
{
	if (value != JCTableEnum.USE_SELECTED_BACKGROUND &&
	    value != JCTableEnum.USE_CELL_BACKGROUND     &&
	    value != JCTableEnum.USE_CELL_FOREGROUND)
	{
		throw new IllegalArgumentException("Parameter value must be one of " +
		                                   "JCTableEnum.USE_SELECTED_BACKGROUND, " +
		                                   "JCTableEnum.USE_CELL_BACKGROUND, " +
										   "JCTableEnum.USE_CELL_FOREGROUND");
	}
	this.selected_background_mode = value;
}

/**
 * Gets the table's <CODE>SelectedForeground</CODE> value.
 * @return the current <CODE>SelectedForeground</CODE> value
 * @see #setSelectedForeground
 */
public Color getSelectedForeground()
{
	return(selected_foreground);
}

/**
 * Sets the foreground color of user-selected cells (default is the cell's
 * background color). Use <code>SelectedForegroundMode</code> to determine
 * how the selected foreground is determined.
 * @param value the new <CODE>SelectedForeground</CODE> value
 * @see #setSelectedForegroundMode
 */
public void setSelectedForeground(Color value)
{
    selected_foreground = value;
    requestChange(TableChangeHandler.TABLE_REPAINT);
}

/**
 * Gets the table's <CODE>SelectedForegroundMode</CODE> value.
 * @return the current <CODE>SelectedForegroundMode</CODE> value; one of
 * <code>JCTableEnum.USE_SELECTED_FOREGROUND</code>,
 * <code>JCTableEnum.USE_CELL_BACKGROUND</code>, or
 * <code>JCTableEnum.USE_CELL_FOREGROUND</code>
 * @see #setSelectedForegroundMode
 */
public int getSelectedForegroundMode()
{
	return(selected_foreground_mode);
}

/**
 * Sets the table's <CODE>SelectedForegroundMode</CODE>.
 * If the mode is <CODE>JCTableEnum.USE_SELECTED_FOREGROUND</CODE>,
 * the selected foreground color is determined by the
 * <CODE>SelectedForeground</CODE> property. In order to keep the
 * selected foreground identical to the cell foreground, use
 * <CODE>JCTableEnum.USE_CELL_FOREGROUND</CODE>. To inverse
 * the background/foreground colors, use
 * <CODE>JCTableEnum.USE_CELL_BACKGROUND</CODE>.
 * @param value the new <CODE>SelectedForegroundMode</CODE> value; one of
 * <code>JCTableEnum.USE_SELECTED_FOREGROUND</code>,
 * <code>JCTableEnum.USE_CELL_BACKGROUND</code>, or
 * <code>JCTableEnum.USE_CELL_FOREGROUND</code>
 */
public void setSelectedForegroundMode(int value)
{
	if (value != JCTableEnum.USE_SELECTED_FOREGROUND &&
	    value != JCTableEnum.USE_CELL_FOREGROUND     &&
	    value != JCTableEnum.USE_CELL_BACKGROUND)
	{
		throw new IllegalArgumentException("Parameter value must be one of " +
		                                   "JCTableEnum.USE_SELECTED_FOREGROUND, " +
		                                   "JCTableEnum.USE_CELL_FOREGROUND, " +
										   "JCTableEnum.USE_CELL_BACKGROUND");
	}
	selected_foreground_mode = value;
	requestChange(TableChangeHandler.TABLE_REPAINT);
}

/**
 * Gets the table's <CODE>SelectIncludeLabels</CODE> value.
 * @return the current <CODE>SelectIncludeLabels</CODE> value
 * @see #setSelectIncludeLabels
 */
public boolean getSelectIncludeLabels()
{
	return(select_include_labels);
}

/**
 * Sets the selection behavior for row and column labels. When
 * <CODE>SelectIncludeLabels</CODE> is set, full row or column selection
 * does not display labels as selected.
 * @param value the new <CODE>SelectIncludeLabels</CODE> value
 */
public void setSelectIncludeLabels(boolean value)
{
	select_include_labels = value;
	if (getSelectedCells() != null) {
		requestChange(TableChangeHandler.TABLE_REPAINT);
	}
}

/**
 * Gets the table's <CODE>SelectionPolicy</CODE> value.
 * @return the current <CODE>SelectionPolicy</CODE> value; one of
 * <code>JCTableEnum.SELECT_NONE</code>, <code>JCTableEnum.SELECT_SINGLE</code>,
 * <code>JCTableEnum.SELECT_RANGE</code>, or
 * <code>JCTableEnum.SELECT_MULTIRANGE</code>
 * @see #setSelectionPolicy
 */
public int getSelectionPolicy()
{
	if (selectionHandler != null) {
		return(selectionHandler.getSelectionPolicy());
	}
	else {
		return(JCTableEnum.SELECT_NONE);
	}
}

/**
 * Sets the selection behavior allowed on the table, both by end-users
 * and by the application. Valid values are:
 * <PRE>
 * JCTableEnum.SELECT_NONE        No selection is allowed
 * JCTableEnum.SELECT_SINGLE      Selection of only one cell allowed
 * JCTableEnum.SELECT_RANGE       Selection of only one range of cells allowed
 * JCTableEnum.SELECT_MULTIRANGE  Selection of multiple ranges of cells allowed
 * </PRE>
 * When <CODE>SelectionPolicy</CODE> is set to
 * <CODE>JCTableEnum.SELECT_NONE</CODE>, <CODE>SELECT</CODE> events are not
 * posted as a user edits or attempts to select cells. Setting this property
 * clears the selected cell list.
 * @param value the new <CODE>SelectionPolicy</CODE> value; one of
 * <code>JCTableEnum.SELECT_NONE</code>, <code>JCTableEnum.SELECT_SINGLE</code>,
 * <code>JCTableEnum.SELECT_RANGE</code>, or
 * <code>JCTableEnum.SELECT_MULTIRANGE</code>
 */
public void setSelectionPolicy(int value)
{
	if (value != JCTableEnum.SELECT_NONE   &&
	    value != JCTableEnum.SELECT_SINGLE &&
	    value != JCTableEnum.SELECT_RANGE  &&
	    value != JCTableEnum.SELECT_MULTIRANGE)
	{
		throw new IllegalArgumentException("Parameter value must be one of " +
		                                   "JCTableEnum.SELECT_NONE, " +
		                                   "JCTableEnum.SELECT_SINGLE, " +
		                                   "JCTableEnum.SELECT_RANGE, " +
										   "JCTableEnum.SELECT_MULTIRANGE");
	}
	if (selectionHandler == null) {
		createDefaultSelectionModel();
	}
	selectionHandler.setSelectionPolicy(value);
}

/**
 * Returns <code>true</code> if series data (styles, cell dimensions, and so on)
 * are maintained in sync with data when it is sorted.
 * @return whether series data is kept in sync with the data
 * @see #setSeriesDataSorted
 */
public boolean isSeriesDataSorted()
{
	return(sort_series);
}

/**
 * Sets whether series data (styles, cell dimensions, and so on) are maintained
 * in sync with data when it is sorted.
 * @param value if <code>true</code>, series data is kept in sync with the data
 */
public void setSeriesDataSorted(boolean value)
{
	this.sort_series = value;
}

/**
 * Returns <code>true</code> if an image of table is maintained offscreen.
 * Disabling <CODE>StoreImage</CODE> will affect scroll
 * performance.
 * @return whether an image of table is maintained offscreen
 */
public boolean isStoreImageEnabled()
{
	return(store_image_enabled);
}

/**
 * Determines whether an image of table is maintained offscreen.
 * This is independent of <code>JComponent</code>'s double buffering and
 * is used to optimize scroll performance.
 * @param value if <code>true</code>, an image of the table is maintained
 * offscreen
 */
public void setStoreImageEnabled(boolean value)
{
	this.store_image_enabled = value;
	requestChange(TableChangeHandler.TABLE_REPAINT);
}

/**
 * Gets the table's <CODE>TopRow</CODE> value.
 * @return the current <CODE>TopRow</CODE> value
 * @see #setTopRow
 */
public int getTopRow()
{
	if (top_row < frozen_rows) {
		top_row = frozen_rows;
	}
	return(top_row);
}

/**
 * Sets the non-frozen row at least partially visible at the top of the window.
 * This value is updated as a user scrolls through the table. When set, the
 * table scrolls to display the row at the top of the window.
 * @param value the new <CODE>TopRow</CODE> value
 */
public void setTopRow(int value)
{
	top_row = Math.min(value, getDataView().getNumRows() - 1);
	top_row = Math.max(top_row, frozen_rows);
	set_top_row = true;
	if (needs_initial_set_top_check) {
		set_initial_top = true;
	}
	requestChange(TableChangeHandler.TABLE_LAYOUT |
	              TableChangeHandler.TABLE_REPAINT);
}

/**
 * Returns the background color of the track component.
 * @return the current background color of the track component
 * @see #setTrackBackground
 */
public Color getTrackBackground()
{
	return(track_background);
}

/**
 * Sets the background color of the component that displays horizontal
 * or vertical scrollbar tracking info.
 * @param value the new background color of the track component
 */
public void setTrackBackground(Color value)
{
	this.track_background = value;
}

/**
 * Returns the foreground color of the track component.
 * @return the current foreground color of the track component
 * @see #setTrackForeground
 */
public Color getTrackForeground()
{
	return(track_foreground);
}

/**
 * Sets the foreground color of the component that display horizontal
 * or vertical scrollbar tracking info.
 * @param value the new foreground color of the track component
 */
public void setTrackForeground(Color value)
{
	this.track_foreground = value;
}

/**
 * Returns the dimensions of the component that tracks horizontal
 * or vertical scrollbar info.
 * @return the current dimensions of the component that tracks horizontal
 *     or vertical scrollbar info
 * @see #setTrackSize
 */
public Dimension getTrackSize()
{
	return(track_size);
}

/**
 * Sets the dimensions of the component that tracks horizontal
 * or vertical scrollbar info.
 * @param value the new dimensions of the component that tracks horizontal
 *     or vertical scrollbar info
 */
public void setTrackSize(Dimension value)
{
	this.track_size = value;
}

/**
 * Gets the table's <CODE>TrackCursor</CODE> value to determine whether the
 * mouse pointer is being tracked as the user moves the mouse.
 * @return whether the mouse pointer is being tracked as the user moves the
 *     mouse
 * @see JCTable#setTrackCursor
 */
public boolean isTrackCursor()
{
	return(track_cursor);
}

/**
 * Sets whether the mouse pointer should be tracked as a user moves the mouse
 * across the table. When <code>true</code>, the <CODE>JCTable</CODE> tracks
 * the mouse pointer and changes its shape to reflect the interactions available
 * to the user at the location. When <code>false</code>, the table uses the
 * application-defined pointer. Default is <CODE>true</CODE>.
 * @param value if <code>true</code>, the mouse pointer will be tracked as the
 *     user moves the mouse
 */
public void setTrackCursor(boolean value)
{
	this.track_cursor = value;
	setCursor(Cursor.DEFAULT_CURSOR);
}

/**
 * Returns the <CODE>TraverseCycle</CODE> value for the table.
 * @return whether the table traverses to the opposite side when the
 *     left, top, right, or bottom cell is reached
 * @see #setTraverseCycle
 */
public boolean isTraverseCycle()
{
    return(traverse_cycle);
}

/**
 * Determines whether table traverses to the opposite side when the
 * left, top, right, or bottom cell is reached.  Default is <CODE>true</CODE>.
 * @param value the new traverse cycle value
 */
public void setTraverseCycle(boolean value)
{
    this.traverse_cycle = value;
}

/**
 * Returns the number of cells to use in estimating variable pixel calculations.
 * <P>The <code>VariableEstimateCount</code> variable is used when the
 * <code>setPixel[Height | Width](VARIABLE_ESTIMATE)</code> method has been
 * called. When that method is called with <code>VARIABLE_ESTIMATE</code>, it
 * iterates over the <code>variableEstimateCount</code> number of rows in the
 * table to determine the highest/widest cell contents in those rows. The method
 * then uses this local maxima to set all row heights/widths for the table; that
 * is, a given subset of table rows are sampled to estimate the height/width.
 * <P>The default is to sample all rows, which is functionally equivalent to
 * calling <code>setPixel[Height | Width](VARIABLE)</code>.
 * <P>Use <code>JCTableEnum.ALL</code> for all cells and labels (default),
 * <code>JCTableEnum.ALLCELLS</code> for all cells,
 * <code>JCTableEnum.LABEL</code> for labels only, or any other number to
 * indicate the last row/column number to use (including the label).
 * @return the current variable estimate count
 * @see #setVariableEstimateCount
 */
public int getVariableEstimateCount()
{
	return(variable_estimate_count);
}

/**
 * Sets the number of cells to use in estimating variable pixel calculations.
 * <P>The VariableEstimateCount variable is used when the <code>setPixel[Height
 * | Width](VARIABLE_ESTIMATE)</code> method has been called. When that method
 * is called with <code>VARIABLE_ESTIMATE</code>, it iterates over the
 * <code>variableEstimateCount</code> number of rows in the table to determine
 * the highest/widest cell contents in those rows. The method then uses this
 * local maxima to set all row heights/widths for the table, that is, a given
 * subset of table rows are sampled to estimate the height/width.
 * <P>The default is to sample all rows, which is functionally equivalent to
 * calling <code>setPixel[Height | Width](VARIABLE)</code>.
 * <P>Use <code>JCTableEnum.ALL</code> for all cells and labels (default),
 * <code>JCTableEnum.ALLCELLS</code> for all cells,
 * <code>JCTableEnum.LABEL</code> for labels only, or any other number to
 * indicate the last row/column number to use (including the label).
 * @param value the new variable estimate count
 */
public void setVariableEstimateCount(int value)
{
	this.variable_estimate_count = value;
}

/**
 * Gets the vertical scrollbar.
 * @return the current vertical scrollbar component
 */
public JScrollBar getVertSB()
{
	return(vsb);
}

/**
 * Sets the vertical scrollbar.
 * @param value the new vertical scrollbar component
 */
public void setVertSB(JScrollBar value)
{
	if (vsb != null) {
		vsb.removeAdjustmentListener(adjustmentHandler);
		remove(vsb);
		vsb = null;
	}
	vsb = value;
	vsb.addAdjustmentListener(adjustmentHandler);
	add(vsb);
	requestChange(TableChangeHandler.TABLE_RESET);
}

/**
 * Gets the tables <CODE>VertSBAttachment</CODE> value.
 * @return the current <CODE>VertSBAttachment</CODE> value; one of
 * <code>JCTableEnum.SIZE_TO_CELLS</code> or
 * <code>JCTableEnum.SIZE_TO_TABLE</code>
 * @see #setVertSBAttachment
 */
public int getVertSBAttachment()
{
	return(vsb_attach);
}

/**
 * Sets how the end of the vertical scrollbar is attached to the table.
 * When set to <CODE>JCTableEnum.SIZE_TO_CELLS</CODE> (default), the
 * scrollbar ends at the edge of the visible cells. <P>
 * When set to <CODE>JCTableEnum.SIZE_TO_TABLE</CODE>, the scrollbar ends
 * at the edge of the table. This setting should be used in conjunction
 * with <CODE>HorizSBPosition</CODE>.
 * @param value the new <CODE>VertSBAttachment</CODE> value; one of
 * <code>JCTableEnum.SIZE_TO_CELLS</code> or
 * <code>JCTableEnum.SIZE_TO_TABLE</code>
 */
public void setVertSBAttachment(int value)
{
	if (value != JCTableEnum.SIZE_TO_CELLS &&
	    value != JCTableEnum.SIZE_TO_TABLE)
	{
		throw new IllegalArgumentException("Parameter value must be one of " +
		                                   "JCTableEnum.SIZE_TO_CELLS, " +
										   "JCTableEnum.SIZE_TO_TABLE");
	}
	vsb_attach = value;
	requestChange(TableChangeHandler.TABLE_LAYOUT);
	if (vsb != null) {
		vsb.invalidate();
		vsb.validate();
	}
}

/**
 * Gets the table's <CODE>VertSBDisplay</CODE> value to determine when to
 * display scrollbars.
 * @return the current <CODE>VertSBDisplay</CODE> value; one of
 * <code>JCTableEnum.SCROLLBAR_ALWAYS</code>,
 * <code>JCTableEnum.SCROLLBAR_NEVER</code>, or
 * <code>JCTableEnum.SCROLLBAR_AS_NEEDED</code>
 * @see #setVertSBDisplay
 */
public int getVertSBDisplay()
{
	return(vsb_display);
}

/**
 * Sets when to display the vertical scrollbar. Valid values are:
 * <PRE>
 * JCTableEnum.SCROLLBAR_ALWAYS        always displayed
 * JCTableEnum.SCROLLBAR_NEVER         never displayed
 * JCTableEnum.SCROLLBAR_AS_NEEDED     displayed when the rows or
 *                                   columns are not visible</PRE>
 * @param value the new <CODE>VertSBDisplay</CODE> value; one of
 * <code>JCTableEnum.SCROLLBAR_ALWAYS</code>,
 * <code>JCTableEnum.SCROLLBAR_NEVER</code>, or
 * <code>JCTableEnum.SCROLLBAR_AS_NEEDED</code>
 */
public void setVertSBDisplay(int value)
{
	if (value != JCTableEnum.SCROLLBAR_ALWAYS &&
	    value != JCTableEnum.SCROLLBAR_NEVER  &&
	    value != JCTableEnum.SCROLLBAR_AS_NEEDED)
	{
		throw new IllegalArgumentException("Parameter value must be one of " +
		                                   "JCTableEnum.SCROLLBAR_ALWAYS, " +
		                                   "JCTableEnum.SCROLLBAR_NEVER, " +
										   "JCTableEnum.SCROLLBAR_AS_NEEDED");
	}
	vsb_display = value;
	requestChange(TableChangeHandler.TABLE_RESET);
}

/**
 * Gets the table's <CODE>VertSBOffset</CODE> value.
 * @return the current <CODE>VertSBOffset</CODE> value
 * @see #setVertSBOffset
 */
public int getVertSBOffset() {
	return(vsb_offset);
}

/**
 * Sets the distance (in pixels) between the vertical scrollbar and
 * the table (default is 0).
 * @param value the new <CODE>VertSBOffset</CODE> value
 */
public void setVertSBOffset(int value)
{
	vsb_offset = value;
	if (vsb_offset < 0) {
		vsb_offset = 0;
	}
	requestChange(TableChangeHandler.TABLE_LAYOUT);
}

/**
 * Gets the table's <CODE>VertSBPosition</CODE> value.
 * @return the current <CODE>VertSBPosition</CODE> value; one of
 * <code>JCTableEnum.POSITION_BY_CELLS</code> or
 * <code>JCTableEnum.POSITION_AT_SIDE</code>
 * @see #setVertSBPosition
 */
public int getVertSBPosition()
{
	return(vsb_position);
}

/**
 * Sets how the vertical scrollbar is attached to the table. When set
 * to <CODE>JCTableEnum.POSITION_BY_CELLS</CODE> (default), the scrollbar
 * is attached to the cell/labels. <P> When set to
 * <CODE>JCTableEnum.POSITION_AT_SIDE</CODE>, the scrollbar is attached to the
 * edge of the table within its container.
 * @param value the new <CODE>VertSBPosition</CODE> value; one of
 * <code>JCTableEnum.POSITION_BY_CELLS</code> or
 * <code>JCTableEnum.POSITION_AT_SIDE</code>
 */
public void setVertSBPosition(int value)
{
	if (value != JCTableEnum.POSITION_BY_CELLS &&
	    value != JCTableEnum.POSITION_AT_SIDE)
	{
		throw new IllegalArgumentException("Parameter value must be one of " +
		                                   "JCTableEnum.POSITION_BY_CELLS, " +
										   "JCTableEnum.POSITION_AT_SIDE");
	}
	vsb_position = value;
	requestChange(TableChangeHandler.TABLE_LAYOUT);
}

/**
 * Gets the table's <CODE>VertSBTrack</CODE> value.
 * @return the current <CODE>VertSBTrack</CODE> value; one of
 * <code>JCTableEnum.TRACK_LIVE</code>, <code>JCTableEnum.TRACK_ROW_NUMBER</CODE>, or
 * <code>JCTableEnum.TRACK_COLUMN</code>
 * @see #setVertSBTrack
 */
public int getVertSBTrack()
{
	return(vsb_track);
}

/**
 * Sets how the vertical scrollbar works during tracking. When set to
 * <CODE>JCTableEnum.TRACK_LIVE</CODE>, THE table redisplays during scrollbar
 * tracking. When set to <CODE>JCTableEnum.TRACK_ROW_NUMBER</CODE>, the table
 * redisplays after tracking completes and displays the number of the top row
 * during the track. Use <CODE>JCTableEnum.TRACK_COLUMN</CODE> and
 * <CODE>setVertSBTrackColumn</CODE> to specify a column number in order to use
 * data from the row to indicate the top-most row.
 * @param value the new <CODE>VertSBTrack</CODE> value; one of
 * <code>JCTableEnum.TRACK_LIVE</code>,
 * <code>JCTableEnum.TRACK_ROW_NUMBER</code>, or
 * <code>JCTableEnum.TRACK_COLUMN</code>
 */
public void setVertSBTrack(int value)
{
	if (value != JCTableEnum.TRACK_LIVE       &&
	    value != JCTableEnum.TRACK_ROW_NUMBER &&
	    value != JCTableEnum.TRACK_COLUMN)
	{
		throw new IllegalArgumentException("Parameter value must be one of " +
		                                   "JCTableEnum.TRACK_LIVE, " +
		                                   "JCTableEnum.TRACK_ROW_NUMBER, " +
										   "JCTableEnum.TRACK_COLUMN");
	}
	this.vsb_track = value;
}

/**
 * Gets the column number that determines the text to display when the
 * track indicator is displayed.
 * @return the current column number that determines the text to display when
 *     the track indicator is displayed
 * @see #setVertSBTrackColumn
 */
public int getVertSBTrackColumn()
{
	return(vsb_track_column);
}

/**
 * Sets the column number that determines the text to display when the
 * track indicator is displayed. Use <CODE>JCTableEnum.LABEL</CODE>
 * to display the data from the row label or a valid column number
 * to display cell data.
 * @param value the new column number that determines the text to display when
 *     the track indicator is displayed
 */
public void setVertSBTrackColumn(int value)
{
	this.vsb_track_column = value;
}

/**
 * Gets the number of non-frozen columns visible in the table.
 * @return the current number of non-frozen columns visible in the table
 * @see #setVisibleColumns
 */
public int getVisibleColumns()
{
	return(visible_columns);
}

/**
 * Sets the number of non-frozen columns used to determine the preferred size
 * of the table. This value is used in preferred size calculations and
 * is not updated as the table is resized. Use <CODE>getNumVisibleColumns</CODE>
 * to obtain the actual number currently visible. <P>Setting this property to
 * <CODE>JCTableEnum.NOVALUE</CODE> causes the table to attempt to resize to
 * make all columns visible.
 * @param value the new number of non-frozen columns visible in the table
 */
public void setVisibleColumns(int value)
{
	if (value <= 0) {
		visible_columns = JCTableEnum.NOVALUE;
	}
	else {
		visible_columns = value;
	}
	requestChange(TableChangeHandler.TABLE_RESET);
}

/**
 * Gets the number of non-frozen rows visible in the table.
 * @return the current number of non-frozen rows visible in the table
 * @see #setVisibleRows
 */
public int getVisibleRows()
{
	return(visible_rows);
}

/**
 * Sets the number of non-frozen rows used to determine the preferred size
 * of the table. This value is used in preferred size calculations and
 * is not updated as the table is resized. Use <CODE>getNumVisibleRows</CODE>
 * to obtain the actual number currently visible. <P>Setting this property to
 * <CODE>JCTableEnum.NOVALUE</CODE> causes the table to attempt to resize
 * to make all rows visible.
 * @param value the new number of non-frozen rows visible in the table
 */
public void setVisibleRows(int value)
{
	if (value <= 0) {
		visible_rows = JCTableEnum.NOVALUE;
	}
	else {
		visible_rows = value;
	}
	requestChange(TableChangeHandler.TABLE_RESET);
}


//
//
// Public Methods
// ----------------------------------------------------------------------------
//

/**
 * Initiates edit for the given cell, if the cell is editable.
 * @param row the row index of the cell
 * @param column the column index of the cell
 * @return <code>false</code> if edit is not possible because the cell is
 * non-existant or not editable/traversable
 */
public boolean beginEdit(int row, int column)
{
	if (!isValidCell(row, column)) {
		return(false);
	}
	oldFocusOwner = getFocusManager().findFocusOwner(this);
	return(getEditTraverseHandler().traverse(row, column, true, null,
										     JCTableEnum.TRAVERSE_TO_CELL,
                                             true));
}

/**
 * Cancels interactive editing of a cell and, as an option, hides the cell
 * editor. Any text entered or changed in the current cell by the user is
 * discarded.
 * @param hide determines whether or not to hide the cell editor
 * @return <code>false</code> if no edit is in progress
 */
public boolean cancelEdit(boolean hide)
{
	oldFocusOwner = getFocusManager().findFocusOwner(this);
	return(getEditTraverseHandler().cancel(hide));
}

/**
 * Returns the bounds of a cell.
 * @param row the row index of the cell
 * @param column the column index of the cell
 * @return a rectangle which contains the bounds of the cell
 */
public Rectangle getCellBounds(int row, int column)
{
	return(cellAreaHandler.getBounds(row, column));
}

/**
 * Returns the position of the top-left corner of a cell.
 * @param row the row index of the cell
 * @param column the column index of the cell
 * @return a <CODE>Point</CODE> object which contains the position of the
 *     top-left corner of a cell
 */
public Point getCellPosition(int row, int column)
{
	Rectangle r = cellAreaHandler.getBounds(row, column);
	if (r != null) {
		return(new Point(r.x, r.y));
	}
	return null;
}

/**
 * Returns <code>true</code> if the given cell is visible.
 * @param row the row index of the cell
 * @param column the column index of the cell
 * @return <code>true</code> if the cell is visible
 */
public boolean isCellVisible(int row, int column)
{
	JCCellRange cr = cellAreaHandler.getVisibleCells();
	return(cr.inside(row, column));
}

/**
 * Commits interactive cell editing internally, saving any changes made by
 * the user.
 * @param hide determines whether to hide the editor
 * @return <code>false</code> if no edit is in progress or the commit is
 * rejected
 */
public boolean commitEdit(boolean hide)
{
	return(getEditTraverseHandler().commit(hide));
}

/**
 * Returns the actual pixel width for a column.
 * @param column the index of the column
 * @return the pixel width for the given column
 */
public int getColumnPixelWidth(int column)
{
	if (cellLayout != null) {
		return(cellLayout.getWidth(column));
	}
	return(0);
}

/**
 * Returns <code>true</code> if the given column is visible.
 * @param column the index of the column
 * @return <code>true</code> if the given column is visible
 */
public boolean isColumnVisible(int column)
{
	JCCellRange cr = cellAreaHandler.getVisibleCells();
	return(column >= cr.start_column && column <= cr.end_column);
}

/**
 * Returns <code>true</code> if the full content of the cell is not displayed
 * because of the size of the cell.
 * @param row the row index of the cell
 * @param column the column index of the cell
 * @return <code>true</code> if the content of the cell is clipped
 */
public boolean isContentClipped(int row, int column)
{
	return(cellAreaHandler.isContentClipped(row, column));
}

/**
 * Gets the current cell in the table. See <CODE>getCurrentRow</CODE> and
 * <CODE>getCurrentColumn</CODE> for the definition of current.
 * @return the position of the current cell
 * @see #getCurrentRow
 * @see #getCurrentColumn
 */
public JCCellPosition getCurrentCell()
{
	return(new JCCellPosition(current_row, current_column));
}

/**
 * Returns <code>true</code> if the given row and column is the current cell.
 * @param row the row index of the cell
 * @param column the column index of the cell
 * @return <code>true</code> if the cell at (row, column) is the current cell
 */
public boolean isCurrentCell(int row, int column)
{
	return(row == current_row && column == current_column);
}

/**
 * Sets the current cell in the table. If the cell is not traversable,
 * the next traversable cell is set as current.
 * @param cell the position of the cell to make current
 */
public void setCurrentCell(JCCellPosition cell)
{
	setCurrentCell(cell.row, cell.column, JCTableEnum.TRAVERSE_RIGHT);
}

/**
 * Sets the current cell in the table given a row and column. If
 * the cell is not traversable, the next traversable cell is set
 * as current.
 * @param row the row index of the cell
 * @param column the column index of the cell
 */
public void setCurrentCell(int row, int column)
{
	setCurrentCell(row, column, JCTableEnum.TRAVERSE_RIGHT);
}

/**
 * Sets the current cell in the table given a row and column. If
 * the cell is not traversable, the next traversable cell in
 * the given direction is set as current.
 * @param row the row index of the cell
 * @param column the column index of the cell
 * @param direction the direction to go if the indicated cell is not traversable
 */
protected void setCurrentCell(int row, int column, int direction)
{
    // record the old position for quick repaint below
    int oldRow      = current_row;
    int oldColumn   = current_column;

	this.current_row = Math.max(row, 0);
	this.current_column = Math.max(column, 0);

	// ensures that the current cell is a valid cell
	if (!isValidCell(current_row, current_column)) {
		current_row = JCTableEnum.NOVALUE;
		current_column = JCTableEnum.NOVALUE;
		return;
	}

	// ensures that the current cell is traversable
	if (!isTraversable(current_row, current_column)) {
		keyHandler.traverse(direction);
	}

	// don't need to move to 0,0 on initial traversal since user
	// has explicitly set location
	needs_initial_traverse = false;

    // quick repaint: do the source and destination cells only
    requestChange(TableChangeHandler.CELL_REPAINT, row, column);
    requestChange(TableChangeHandler.CELL_REPAINT, oldRow, oldColumn);
}

/**
 * Gets the current row in the table. The current row is defined as: the row
 * containing an editor, the row containing the focus rectangle, or the top row
 * in the last selected region.
 * @return the current row
 */
public int getCurrentRow()
{
	return(current_row);
}

/**
 * Gets the current column in the table. The current column is defined as: the
 * column containing an editor, the column containing the focus rectangle, or
 * the left column in the last selected region.
 * @return the current column
 */
public int getCurrentColumn()
{
	return(current_column);
}

/**
 * Gets the component associated with the currently displayed editor. If no
 * cell is being edited, <CODE>getEditingComponent</CODE> returns null.
 * @return the currently displayed editor, if any
 */
public Component getEditingComponent()
{
    if (getEditTraverseHandler().getCellEditor() == null) {
        return(null);
    }
    return(getEditTraverseHandler().getCellEditor().getComponent());
}

/**
 * Returns <code>true</code> if there are components in table.
 * @return whether there are components in this table
 */
public boolean hasComponents()
{
	return(getComponentModel() != null);
}

/**
 * Returns <code>true</code> if the row and column consititute a valid cell.
 * @param row the row index of the cell
 * @param column the column index of the cell
 * @return <code>true</code> if the row/column combination consititutes a valid
 * cell
 */
public final static boolean isCell(int row, int column)
{
	return(row >= 0 && column >= 0);
}

/**
 * Returns <code>true</code> if the row and column consititute a valid column
 * label.
 * @param row the row index of the cell
 * @param column the column index of the cell
 * @return <code>true</code> if the row/column combination consititutes a valid
 * column label
 */
public final static boolean isColumnLabel(int row, int column)
{
	return(row == JCTableEnum.LABEL && column >= 0);
}

/**
 * Allows users to drag a column and drop it into a new location by
 * remapping the internal order of columns. This method does not
 * affect the data source. Use <CODE>JCTableEnum.MAXINT</CODE>
 * as the end point in order to move the column to the end.
 * @param column1 the original location of the column
 * @param column2 the new location of the column
 * @return an indication to whether the drag suceeded
 */
public boolean dragColumn(int column1, int column2)
{
	if (dataView instanceof SortableDataViewModel) {
		return((SortableDataViewModel)dataView).dragColumn(column1, column2);
	}
	else {
		throw new JCTableException(dataView + " does not support the " +
								   "SortableDataViewModel interface");
	}
}

/**
 * Allows users to drag a row and drop it into a new location by
 * remapping the internal order of rows. This method does not
 * affect the data source. Use <CODE>JCTableEnum.MAXINT</CODE>
 * as the end point in order to move the row to the end.
 * @param row1 the original location of the row
 * @param row2 the new location of the row
 * @return an indication to whether the drag suceeded
 */
public boolean dragRow(int row1, int row2)
{
	if (dataView instanceof SortableDataViewModel) {
		return((SortableDataViewModel)dataView).dragRow(row1, row2);
	}
	else {
		throw new JCTableException(dataView + " does not support the " +
		                           "SortableDataViewModel interface");
	}
}

/**
 * Returns <code>true</code> if the given cell is editable.
 * @param row the row index of the cell
 * @param column the column index of the cell
 * @return whether the cell at (row, column) is editable
 */
public boolean isEditable(int row, int column)
{
	// check if cell is a label
	if (!isCell(row, column)) {
		return(false);
	}

	// check if in a spanned cell range
	if (spanHandler != null) {
		JCCellRange cr = getSpannedRange(row, column);
		if (cr != null && !spanHandler.isControllingCell(cr, row, column)) {
			return(isEditable(cr.start_row, cr.start_column));
		}
	}

	// return false if the style specifies that the cell is non-traversable
	CellStyleModel style = getCellStyle(row, column);
	if (style == null || !style.isEditable()) {
		return(false);
	}
	return(true);
}

/**
 * Overrides <code>JComponent.isFocusTraversable</code> and returns
 * <code>true</code>, indicating that the table is a focusable component.
 * @return <code>true</code>
 */
public boolean isFocusTraversable()
{
	return(true);
}

/**
 * Returns <code>true</code> if the row and column constitute a valid label.
 * @param row the row index of the possible label
 * @param column the column index of the possible label
 * @return an indication of whether the row/column combination is a valid label
 */
public final static boolean isLabel(int row, int column)
{
	return(row == JCTableEnum.LABEL ? column >= 0
				                    : (column == JCTableEnum.LABEL ? row >= 0
												                   : false));
}

/**
 * Returns <code>true</code> if the row and column constitute a valid row label.
 * @param row the row index of the possible label
 * @param column the column index of the possible label
 * @return an indication of whether the row/column combination is a valid label
 */
public final static boolean isRowLabel(int row, int column)
{
	return(row >= 0 && column == JCTableEnum.LABEL);
}

/**
 * Scrolls a frozen or non-frozen cell or label so it is visible on the screen.
 * @param row the row index of the cell
 * @param column the column index of the cell
 * @return <code>false</code> if the position is invalid
 */
public boolean makeVisible(int row, int column)
{
	// abort if the table hasn't been realized
	if (!isDisplayable()) {
		return(false);
	}

	if (row == JCTableEnum.LABEL || row == JCTableEnum.ALL) {
		return(makeColumnVisible(column));
	}
	else if (column == JCTableEnum.LABEL || column == JCTableEnum.ALL) {
		return(makeRowVisible(row));
	}
	else {
		if (!makeRowVisible(row)) {
			return(false);
		}
		return(makeColumnVisible(column));
	}
}

/**
 * Scrolls a column so it is visible on the screen.
 * @param column the column index of the cell
 * @return <code>false</code> if the column is invalid
 */
public boolean makeColumnVisible(int column)
{
	// abort if the table hasn't been realized
	if (!isDisplayable() || cellAreaHandler.cellArea == null) {
		return(false);
	}

	// Frozen columns are always visible
    if (column < frozen_columns || min_cell_visibility == 0) {
        return(true);
    }

    column = Math.max(0, Math.min(column, getDataView().getNumColumns()-1));
    int value = getColumnVisibleValue(column);
    if (value == JCTableEnum.NOVALUE) {
        return(true);
    }

/*
    if (value < 0                  ||
	    value > cell_total_height  ||
		clip.vert_origin == value)
	{
        return(true);
	}
*/

	hsb.setValue(value);

/*
    if (repaint_enabled) {
        isJumping = false;
        vert_sb.setValue((int)value);
        isJumping = true;
    }
    else {
        needs_repaint = true;
        clip.vert_origin = (int)value;
    }
*/
	return true;
}

/**
 * Determines the new value of the horizontal scrollbar needed to scroll
 * the column onto the screen. If the whole column does not fit,
 * it is scrolled so that its left edge is visible.
 * @param column the column for which the horizontal scrollbar value needs to
 *    be computed
 * @return the computed horizontal scrollbar value
 */
private int getColumnVisibleValue(int column)
{
	double value = 0;

	int column_width = cellLayout.getWidth(column);
	double fraction = column_width * (100.0 - min_cell_visibility) / 100.0;

	int horizontal_origin = -cellAreaHandler.getCellOrigin().x;
	int cell_area_width = cellAreaHandler.cellArea.getWidth();

	// Get the position of the column accounting for the layout of labels and
    // frozen cells
	int x = (cellLayout.getColumnPosition(column) -
             cellAreaHandler.cellArea.getStandardOffsetX());

	// Cell position is before the horizontal origin
	if (x <= horizontal_origin) {
		value = Math.min(horizontal_origin, x + fraction);
	}
	// Cell position and width exceeds the position of the visible area
	else if (x + column_width > horizontal_origin + cell_area_width) {
		int offset = x + column_width - (cell_area_width + horizontal_origin);
		if (x - offset < horizontal_origin) {
			value = x;
		}
		else {
			if (offset <= fraction) {
				value = JCTableEnum.NOVALUE;
			}
			else {
				value = horizontal_origin + offset - fraction;
			}
		}
	}
	// Cell is already visible, no adjustment required
	else {
		value = JCTableEnum.NOVALUE;
	}
	return((int)value);
}

/**
 * Scrolls a row so it is visible on the screen.
 * @param row the row index of the cell
 * @return <code>false</code> if the row is invalid
 */
public boolean makeRowVisible(int row)
{
	// abort if the table hasn't been realized
	if (!isDisplayable() || cellAreaHandler.cellArea == null) {
		return(false);
	}
	// Frozen rows are always visible
    if (row < frozen_rows || min_cell_visibility == 0) {
        return(true);
    }

    row = Math.max(0, Math.min(row, getDataView().getNumRows()-1));
    int value = getRowVisibleValue(row);
    if (value == JCTableEnum.NOVALUE) {
        return(true);
    }

/*
    if (value < 0                  ||
	    value > cell_total_height  ||
		clip.vert_origin == value)
	{
        return(true);
	}
*/

	vsb.setValue(value);

/*
    if (repaint_enabled) {
        isJumping = false;
        vert_sb.setValue((int)value);
        isJumping = true;
    }
    else {
        needs_repaint = true;
        clip.vert_origin = (int)value;
    }
*/
	return(true);
}

/**
 * Determines the new value of the vertical scrollbar needed to scroll
 * this cell onto the screen. If the whole cell won't fit, it is scrolled so
 * that its left edge is visible.
 * @param row the row for which the vertical scrollbar value needs to
 *    be computed
 * @return the computed vertical scrollbar value
 */
private int getRowVisibleValue(int row)
{
	double value = 0;

	int row_height = cellLayout.getHeight(row);
	double fraction = row_height * (100.0 - min_cell_visibility) / 100.0;

	int vertical_origin = -cellAreaHandler.getCellOrigin().y;
	int cell_area_height = cellAreaHandler.cellArea.getHeight();

	// Get the position of the row accounting for the layout of labels and
    // frozen cells
	int y = (cellLayout.getRowPosition(row) -
             cellAreaHandler.cellArea.getStandardOffsetY());

	// Cell position is before the vertical origin
	if (y <= vertical_origin) {
		value = Math.min(vertical_origin, y + fraction);
	}
	// cell position and height exceeds the position of the visible area
	else if (y + row_height > vertical_origin + cell_area_height) {
		int offset = y + row_height - (cell_area_height + vertical_origin);
		if (y - offset < vertical_origin) {
			value = y;
		}
		else {
			if (offset <= fraction) {
				value = JCTableEnum.NOVALUE;
			}
			else {
				value = vertical_origin + offset - fraction;
			}
		}
	}
	// Cell is already visible, no adjustment required
	else {
		value = JCTableEnum.NOVALUE;
	}
	return((int)value);
}

/**
 * Returns <code>true</code> if the given row is visible.
 * @param row the row index of row whose visibility is required
 * @return whether the given row is visible
 */
public boolean isRowVisible(int row)
{
	JCCellRange cr = cellAreaHandler.getVisibleCells();
	return(row >= cr.start_row && row <= cr.end_row);
}

/**
 * Gets the number of columns in the table from the data source.
 * @return the current number of columns in the table's data source
 */
public int getNumColumns()
{
	if (dataView != null) {
		return(dataView.getNumColumns());
	}
	return(0);
}

/**
 * Gets the number of rows in the table from the data source.
 * @return the current number of rows in the table's data source
*/
public int getNumRows()
{
	if (dataView != null) {
		return(dataView.getNumRows());
	}
	return(0);
}

/**
 * Gets the number of visible columns. This value reflects the number
 * of scrollable columns that are currently visible.
 * @return the current number of visible columns in the table
 */
public int getNumVisibleColumns()
{
	JCCellRange cr = getVisibleCells();
	return(cr.end_column - cr.start_column + 1);
}

/**
 * Gets the number of visible rows. This value reflects the number
 * of scrollable rows that are currently visible.
 * @return the current number of visible rows in the table
 */
public int getNumVisibleRows()
{
	JCCellRange cr = getVisibleCells();
	return(cr.end_row - cr.start_row + 1);
}

/**
 * Returns the <code>JCCellPosition</code> of the given component in table.
 * Returns null if the component is not found.
 * @param c the component whose position is required
 * @return the cell position of the component
 */
public JCCellPosition getPosition(Component c)
{
	ComponentModel componentHandler = getComponentModel();
	if (componentHandler == null) {
		return(null);
	}
	return(componentHandler.findPosition(c));
}

/**
 * Resets the internal column mapping array.
 */
public void resetSwappedColumns()
{
	if (dataView instanceof SortableDataViewModel) {
    	((SortableDataViewModel)dataView).resetColumnMap();
	}
}

/**
 * Resets the internal row mapping array.
 */
public void resetSwappedRows()
{
	if (dataView instanceof SortableDataViewModel) {
	    ((SortableDataViewModel)dataView).resetRowMap();
	}
}

/**
 * Resets the internal row mapping array used for sorted rows.
 * <code>resetSwappedRows</code> and <code>resetSortedRows</code> perform the same function.
 */
public void resetSortedRows()
{
	if (dataView instanceof SortableDataViewModel) {
	    ((SortableDataViewModel)dataView).resetRowMap();
	}
}


public boolean isResizable(int row ,int column)
{
	return (!((allow_cell_resize == JCTableEnum.RESIZE_NONE) ||     // Resize policy
			  (allow_resize_by == JCTableEnum.RESIZE_BY_LABELS &&   // Resize by labels only
               row > -1 && column > -1) ||
			  (allow_resize_by == JCTableEnum.RESIZE_BY_CELLS &&    // Resize by cells only
               (row == -1 || column == -1)) ||
			  (row < JCTableEnum.NOVALUE ||                         // Invalid cell
               column < JCTableEnum.NOVALUE)));
}

/**
 * Returns the actual pixel height for a row.
 * @param row the index of the row whose pixel height is required
 * @return the pixel height of the given row
 */
public int getRowPixelHeight(int row)
{
	if (cellLayout != null) {
		return(cellLayout.getHeight(row));
	}
	return(0);
}

/**
 * Sorts the rows in the table based on a specified column and direction.
 * Sorting does not affect the datasource and requires a
 * <CODE>SortableDataViewModel</CODE>.
 * @param column the column on which to base the sort
 * @param direction either <code>Sort.ASCENDING</code> or
 * <code>Sort.DESCENDING</code>
 * @return whether sort succeeded
 */
public boolean sortByColumn(int column, int direction)
{
	return(Sort.sortByColumn(this, column, direction,
	                         getFrozenRows(), getNumRows() - 1, null));
}

/**
 * Sorts a range of rows in the table based on a specified column and direction.
 * Sorting does not affect the datasource and requires a
 * <CODE>SortableDataViewModel</CODE>.
 * @param column the column on which to base the sort
 * @param direction either <code>Sort.ASCENDING</code> or <code>Sort.DESCENDING</code>
 * @param start_row the start row of the sort
 * @param end_row  the end row of the sort
 * @return whether sort succeeded
 */
public boolean sortByColumn(int column, int direction,
			                int start_row, int end_row)
{
	return(Sort.sortByColumn(this, column, direction,
	                         start_row, end_row, null));
}

/**
 * Sorts the rows in a table based on multiple columns and direction. Sorting
 * does not affect the datasource and requires a
 * <CODE>SortableDataViewModel</CODE>.
 * @param column a list of columns by which to sort
 * @param direction a corresponding list of directions to sort
 * @return whether sort succeeded
 */
public boolean sortByColumn(int column[], int direction[])
{
	return(Sort.sortByColumn(this, column, direction,
	                         getFrozenRows(), getNumRows() - 1, null));
}

/**
 * Sorts a range of rows in the table based on multiple columns and directions.
 * Sorting does not affect the datasource and requires a
 * <CODE>SortableDataViewModel</CODE>.
 * @param column a list of columns by which to sort
 * @param direction a corresponding list of directions to sort
 * @param start_row the start row of the sort
 * @param end_row  the end row of the sort
 * @return whether sort succeeded
 */
public boolean sortByColumn(int column[], int direction[],
			                int start_row, int end_row)
{
	return(Sort.sortByColumn(this, column, direction,
	                         start_row, end_row, null));
}

/**
 * Swaps two columns in the table without modifying the data source. The
 * swapped columns retain their properties except for selected cells and
 * spanned cells.
 * @param column1 the first column to swap
 * @param column2 the second column to swap
 * @return whether swap succeeded
 */
public boolean swapColumns(int column1, int column2)
{
	if(dataView instanceof SortableDataViewModel) {
		return(((SortableDataViewModel)dataView).swapColumns(column1, column2));
	}
	else {
		throw new JCTableException(dataView + " does not support the " +
		                           "SortableDataViewModel interface");
	}
}

/**
 * Swaps two rows in the table without modifying the data source. The swapped
 * rows retain all their properties except spanned cells. If the
 * selection policy is currently
 * set to <CODE>JCTblEnum.SELECT_MULTIRANGE</CODE>,
 * the selected cells follow the new row order.
 * @see #setSelectionPolicy
 * @param row1 the first row to swap
 * @param row2 the second row to swap
 * @return whether swap succeeded
 */
public boolean swapRows(int row1, int row2)
{
    if (dataView instanceof SortableDataViewModel) {
		return(((SortableDataViewModel)dataView).swapRows(row1, row2));
	}
	else {
		throw new JCTableException(dataView + " does not support the " +
		                           "SortableDataViewModel interface");
	}
}

/**
 * Attempts to move the current cell focus to a new location.
 * <CODE>commitEdit</CODE> is first called to commit the edit in the current
 * cell. If <CODE>commitEdit</CODE> returns <code>false</code>,
 * <CODE>traverse</CODE> returns <code>false</code> and does not move the focus.
 * If the cell is not traversable, the method returns <code>false</code>.</P>
 * @param row the row to traverse to
 * @param column the column to traverse to
 * @param show_editor if <code>true</code>, the editing component is displayed
 * in the cell
 * @param select if <code>true</code> and the <CODE>SelectionPolicy</CODE>
 * allows it, the cell is selected
 * @return whether traverse suceeded
 */
public boolean traverse(int row, int column, boolean show_editor, boolean select)
{
	oldFocusOwner = getFocusManager().findFocusOwner(this);
	return(getEditTraverseHandler().traverse(row, column, select, null,
										     JCTableEnum.TRAVERSE_TO_CELL,
											 show_editor));
}

/**
 * Attempts to move the current cell focus to a new location given a direction.
 * @param direction the direction to traverse; one of
 * <code>JCTableEnum.TRAVERSE_DOWN</code>,
 * <code>JCTableEnum.TRAVERSE_UP</code>,
 * <code>JCTableEnum.TRAVERSE_LEFT</code>,
 * <code>JCTableEnum.TRAVERSE_RIGHT</code>,
 * <code>JCTableEnum.TRAVERSE_PAGEUP</code>,
 * <code>JCTableEnum.TRAVERSE_PAGEDOWN</code>,
 * <code>JCTableEnum.TRAVERSE_HOME</code>,
 * <code>JCTableEnum.TRAVERSE_END</code>,
 * <code>JCTableEnum.TRAVERSE_TOP</code>, or
 * <code>JCTableEnum.TRAVERSE_BOTTOM</code>
 * @return whether traverse succeeded
 */
public boolean traverse(int direction)
{
	oldFocusOwner = getFocusManager().findFocusOwner(this);
	if (keyHandler != null) {
		return(keyHandler.traverse(direction));
	}
	else {
		return(false);
	}
}

/**
 * Returns <code>true</code> if the given cell is traversable.
 * @param row the row index of the cell
 * @param column the column index of the cell
 * @return whether the given cell is traversable
 */
public boolean isTraversable(int row, int column)
{
	CellStyleModel style = getCellStyle(row, column);

	// Return false if the style specifies that the cell is non-traversable
	if (cellLayout == null || style == null || !style.isTraversable()) {
		return(false);
	}

	// Return false if the cell is hidden
	if (cellLayout.getWidth(column) <= 0 || cellLayout.getHeight(row) <= 0) {
		return(false);
	}

	// Check if in a spanned cell range
	if (spanHandler != null) {
		JCCellRange cr = getSpannedRange(row, column);
		// If this cell is in a span, return false if the cell isn't
        // the controller.
		if (cr != null) {
			return(spanHandler.isControllingCell(cr, row, column));
		}
	}
	return(true);
}

/**
 * Returns <code>true</code> if the given cell is valid.
 * @param row the row index of the cell
 * @param column the column index of the cell
 * @return Is the given cell valid?
 */
protected boolean isValidCell(int row, int column)
{
	return(row >= JCTableEnum.LABEL && row < getNumRows() &&
		   column >= JCTableEnum.LABEL && column < getNumColumns());
}

/**
 * Returns the range of scrollable cells that are currently visible.
 * @return the range of scrollable cells that are currently visible
 */
public JCCellRange getVisibleCells()
{
	return(cellAreaHandler.getVisibleCells());
}

/**
 * Returns the cell located at the x, y position.
 * @param x the x position
 * @param y the y position
 * @return the cell position of the cell located at (x, y)
 */
public JCCellPosition XYToCell(int x, int y)
{
	JCCellPosition cp = new JCCellPosition();
	cp.row = cellAreaHandler.getRow(x, y);
	cp.column = cellAreaHandler.getColumn(x, y);
	return(cp);
}

/**
 * Returns the scrollbar tracking state. During the tracking the table does
 * not adjust according to the scrollbar value, but redisplays after the
 * tracking completes.
 * @return the scrollbar tracking state
 */
public boolean isTracking()
{
	return(adjustmentHandler.isTracking());
}

//
//
// ActionHandling
// ----------------------------------------------------------------------------
//

/**
 * Adds an <CODE>ActionInitiator</CODE>/action pair to table (stored in a
 * <code>TableAction</code>).
 * @see #addAction(TableAction)
 * @param initiator the key or mouse event initiating the action
 * @param action the action to add
 */
public void addAction(ActionInitiator initiator, int action)
{
	addAction(new TableAction(initiator, action));
}

/**
 * Adds a <CODE>TableAction</CODE> to table. A <CODE>TableAction</CODE>
 * maps an event to a particular action on table. Valid
 * actions include:
 * <UL>
 * <LI>JCTableEnum.COLUMN_SELECT_ACTION</LI>
 * <LI>JCTableEnum.COLUMN_SORT_ACTION</LI>
 * <LI>JCTableEnum.COLUMN_DRAG_ACTION</LI>
 * <LI>JCTableEnum.ROW_SELECT_ACTION</LI>
 * <LI>JCTableEnum.ROW_DRAG_ACTION</LI>
 * <LI>JCTableEnum.COPY_ACTION</LI>
 * <LI>JCTableEnum.PASTE_ACTION</LI>
 * </UL>
 * @param tableAction defines a mapping from a user event to an action
 *     performed on the table
 */
public void addAction(TableAction tableAction)
{
	if (actions == null) {
		actions = new Vector();
	}
	actions.addElement(tableAction);
}

/**
 * Returns the action mapped to the given event. If no action
 * exists, <CODE>JCTableEnum.NOVALUE</CODE> is returned.
 * @param event the event to which an action is to be mapped
 * @param row the row on which the action is to be taken
 * @param column the column on which the action is to be taken
 * @return the action mapped to the event on the row/column
 */
public int getAction(AWTEvent event, int row, int column)
{
	if (actions != null) {
		for (int i = 0; i < actions.size(); i++) {
			TableAction tableAction = (TableAction)actions.elementAt(i);
			if (tableAction.isMatch(event, row, column)) {
				return(tableAction.getAction());
			}
		}
	}
	return(JCTableEnum.NOVALUE);
}

/**
 * Removes an action from table.
 * @param tableAction the action to be removed
 */
public void removeAction(TableAction tableAction)
{
	removeAction(tableAction.getInitiator(), tableAction.getAction());
}

/**
 * Removes an <CODE>ActionInitiator</CODE>/action pair from table.
 * @param initiator the key or mouse event initiating the action
 * @param action the action to remove
 */
public void removeAction(ActionInitiator initiator, int action)
{
	if (actions != null) {
		for (int i = 0; i < actions.size(); i++) {
			TableAction tableAction = (TableAction)actions.elementAt(i);
			if(tableAction.getInitiator() == initiator &&
			   tableAction.action == action)
			{
			   	actions.removeElement(tableAction);
			}
		}
		if (actions.size() == 0) {
			actions = null;
		}
	}
}

/**
 * Removes any <CODE>ActionInitiator</CODE> set for the given action type.
 * @param action the action to be removed
 */
public void removeAction(int action)
{
	if (actions != null) {
		for (int i = 0; i < actions.size(); i++) {
			TableAction tableAction = (TableAction)actions.elementAt(i);
			if (tableAction.action == action) {
				actions.removeElement(tableAction);
			}
		}
		if (actions.size() == 0) {
			actions = null;
		}
	}
}

/**
 * Removes all actions from table.
 */
public void removeAllActions()
{
	actions = null;
}

//
//
// Event Handling
// ----------------------------------------------------------------------------
//

protected void processMouseEvent(MouseEvent e)
{
//  
	super.processMouseEvent(e);

	if (e.isConsumed()) {
		return;
	}

	int id = e.getID();
	switch(id) {
	case MouseEvent.MOUSE_PRESSED:
    	if (popupMenuEnabled && (e.getModifiers() &
            InputEvent.BUTTON3_MASK) != 0)
        {
	        showPopupMenu(e.getX(), e.getY());
            return;
    	}
		mouseHandler.mousePressed(e);
		break;
	case MouseEvent.MOUSE_RELEASED:
		mouseHandler.mouseReleased(e);
		break;
	case MouseEvent.MOUSE_CLICKED:
		mouseHandler.mouseClicked(e);
		break;
	case MouseEvent.MOUSE_EXITED:
		mouseHandler.mouseExited(e);
		break;
	case MouseEvent.MOUSE_ENTERED:
		mouseHandler.mouseEntered(e);
		break;
	}
}

protected void processMouseMotionEvent(MouseEvent e)
{
	super.processMouseMotionEvent(e);

	if (e.isConsumed()) {
		return;
	}

	int id = e.getID();
	switch(id) {
	case MouseEvent.MOUSE_MOVED:
		mouseHandler.mouseMoved(e);
		break;
	case MouseEvent.MOUSE_DRAGGED:
		mouseHandler.mouseDragged(e);
		break;
	}
}

protected void processKeyEvent(KeyEvent e)
{
	
	if (e.getID() == KeyEvent.KEY_PRESSED &&
        e.getKeyCode() == KeyEvent.VK_TAB)
    {
		e.consume();
	}
	else {
		super.processKeyEvent(e);

		// PARTNER RELEASE RELEASE "L" CONSUMES KeyPressed WHEN OTHER
        // COMPONENTS ARE ON THE SCREEN:
		//if (e.isConsumed()) {
		//	return;
		//}
	}

	int id = e.getID();
	switch(id) {
	case KeyEvent.KEY_PRESSED:
		keyHandler.keyPressed(e);
		break;
	case KeyEvent.KEY_RELEASED:
		keyHandler.keyReleased(e);
		break;
	case KeyEvent.KEY_TYPED:
		keyHandler.keyTyped(e);
		break;
	}

    JCCellEditor cellEditor = getEditTraverseHandler().getCellEditor();
    if (cellEditor != null &&
        cellEditor instanceof JCCellTextEditor &&
        !e.isConsumed())
    {
        ((JCCellTextEditor)cellEditor).processKeyEvent(e);
    }
}

void dispatchMouseEvent(MouseEvent e)
{
	Point loc = e.getComponent().getLocation();

	MouseEvent new_event = new MouseEvent(this, e.getID(), e.getWhen(),
							              e.getModifiers(), e.getX()+loc.x,
										  e.getY()+loc.y, e.getClickCount(),
		                                  e.isPopupTrigger());
	processEvent(new_event);
}

void dispatchKeyEvent(KeyEvent e)
{
	KeyEvent new_event = new KeyEvent(this, e.getID(), e.getWhen(),
							          e.getModifiers(), e.getKeyCode(),
									  e.getKeyChar());
	processEvent(new_event);
}

public void mouseDragged(MouseEvent e)
{
	dispatchMouseEvent(e);
}

public void mouseMoved(MouseEvent e)
{
	dispatchMouseEvent(e);
}

public void mousePressed(MouseEvent e)
{
	dispatchMouseEvent(e);
}

public void mouseReleased(MouseEvent e)
{
	dispatchMouseEvent(e);
}

public void mouseClicked(MouseEvent e)
{
	dispatchMouseEvent(e);
}

public void mouseEntered(MouseEvent e)
{
	dispatchMouseEvent(e);
}

public void mouseExited(MouseEvent e)
{
	dispatchMouseEvent(e);
}

public void keyPressed(KeyEvent e)
{
	dispatchKeyEvent(e);
}

public void keyReleased(KeyEvent e)
{
	dispatchKeyEvent(e);
}

public void keyTyped(KeyEvent e)
{
	dispatchKeyEvent(e);
}

//
//
// Selection Handling
// ----------------------------------------------------------------------------
//

/**
 * Returns the <code>SelectionModel</code> manager.
 * @return the <code>SelectionModel</code> manager
 */
public SelectionModel getSelectionModel()
{
	return(selectionHandler);
}

/**
 * Creates the default <code>SelectionModel</code> manager.
 */
protected void createDefaultSelectionModel()
{
	try {
		SelectionModel oldSelectionHandler = selectionHandler;
		Class c = Class.forName("com.klg.jclass.table.JCSelection");
		SelectionModel selectionHandler = (SelectionModel)c.newInstance();
		selectionHandler.setTable(this);
		setSelectionModel(selectionHandler);

		addToMoveables(selectionHandler, oldSelectionHandler);
	}
	catch(Exception e) {
		System.out.println("Selection initialization error: " + e.getMessage());
	}
}

/**
 * Sets the <code>SelectionModel</code> manager.
 * @param selectionHandler the new selection handler
 */
public void setSelectionModel(SelectionModel selectionHandler)
{
	if (selectionHandler != null) {
		selectionHandler.clearSelection();
	}
	this.selectionHandler = selectionHandler;
}

/**
 * Selects a range of rows.
 * @param start_row the start row of the selection
 * @param end_row the end row of the selection
 * @return whether selection succeeded
 */
public boolean setRowSelection(int start_row, int end_row)
{
	if (selectionHandler == null) {
		return(false);
	}
	return(selectionHandler.setRowSelection(start_row, end_row));
}

/**
 * Selects a range of columns.
 * @param start_column the start column of the selection
 * @param end_column the end column of the selection
 * @return whether selection succeeded
 */
public boolean setColumnSelection(int start_column, int end_column)
{
	if (selectionHandler == null) {
		return(false);
	}
	return(selectionHandler.setColumnSelection(start_column, end_column));
}

/**
 * Selects a range of cells.
 * @param start_row the start row of the selection
 * @param end_row the end row of the selection
 * @param start_column the start column of the selection
 * @param end_column the end column of the selection
 * @return whether selection succeeded
 */
public boolean setSelection(int start_row, int start_column,
			                int end_row, int end_column)
{
	if (selectionHandler == null) {
		return(false);
	}
	return(selectionHandler.setSelection(start_row, start_column,
										 end_row, end_column));
}

/**
 * Adds a range of rows to the current selection.
 * @param start_row the start row of the selection to add
 * @param end_row the end row of the selection to add
 * @return whether selection addition succeeded
 */
public boolean addRowSelection(int start_row, int end_row)
{
	if (selectionHandler == null) {
		return(false);
	}
	return(selectionHandler.addRowSelection(start_row, end_row));
}

/**
 * Adds a range of columns to the current selection.
 * @param start_column the start column of the selection to add
 * @param end_column the end column of the selection to add
 * @return whether selection addition succeeded
 */
public boolean addColumnSelection(int start_column, int end_column)
{
	if (selectionHandler == null) {
		return(false);
	}
	return(selectionHandler.addColumnSelection(start_column, end_column));
}

/**
 * Adds a range of cells to the current selection.
 * @param start_row the start row of the selection to add
 * @param end_row the end row of the selection to add
 * @param start_column the start column of the selection to add
 * @param end_column the end column of the selection to add
 * @return whether selection addition succeeded
 */
public boolean addSelection(int start_row, int start_column,
			                int end_row, int end_column)
{
	if (selectionHandler == null) {
		return(false);
	}
	return(selectionHandler.addSelection(start_row, start_column,
										 end_row, end_column));
}

/**
 * Clears all selections.
 */
public void clearSelection()
{
	if( selectionHandler != null) {
		selectionHandler.clearSelection();
	}
}

/**
 * Selects all cells.
 * @return whether selection succeeded
 */
public boolean selectAll()
{
	if (selectionHandler == null) {
		return(false);
	}
	return(selectionHandler.selectAll());
}

/**
 * Gets the table's <CODE>SelectedCells</CODE> value.
 * @return collection of <code>JCCellRange</code> objects
 * @see #setSelectedCells
 */
public Collection getSelectedCells()
{
	if (selectionHandler != null) {
		return(selectionHandler.getSelectedCells());
	}
	else {
		return(null);
	}
}

/**
 * Deselects all selected cells.
 * @see #setSelectedCells
 */
public void clearSelectedCells()
{
	if (selectionHandler != null) {
		selectionHandler.clearSelection();
	}
}

/**
 * Sets the currently-selected ranges of cells. Each element
 * of the Collection is an instance of a <CODE>JCCellRange</CODE>.
 * @param v the collection which contains a list of ranges to select
 * @return whether selection succeeded
 */
public boolean setSelectedCells(Collection v)
{
	if (selectionHandler == null) {
		return(false);
	}
	return(selectionHandler.setSelection(v));
}

/**
 * Sets a range of selected cells.
 * To unselect all cells, call <CODE>clearSelectedCells()</CODE>.
 * @param r the range to select
 * @return whether selection succeeded
 * @see #clearSelectedCells
 */
public boolean setSelectedCells(JCCellRange r)
{
	if (selectionHandler == null) {
		return(false);
	}
	return(selectionHandler.setSelection(r));
}

/**
 * Determines whether the cell is selected.
 * @param row the row index of the cell
 * @param column the column index of the cell
 * @return whether cell at (row, column) is selected
 */
public boolean isSelected(int row, int column)
{
	if (selectionHandler == null) {
		return(false);
	}
	if (isLabel(row, column) && !select_include_labels) {
		return(false);
	}
	return(selectionHandler.isSelected(row, column));
}

public void repaintSelected()
{
	if (!isDisplayable()) {
		return;
	}

	if (selectionHandler == null) {
		return;
	}

	Collection col = selectionHandler.getSelectedCells();
	if (col == null) {
		return;
	}
	JCCellRange cr = new JCCellRange(0,0,0,0);
	Iterator it = col.iterator();
	while (it.hasNext()) {
		cr = (JCCellRange) it.next();
		cellAreaHandler.getCellAreaRenderer().repaintRange(cr);
	}
}

//
//
// Span Handling
// ----------------------------------------------------------------------------
//

/**
 * Returns the span manager.
 * @return the span manager
 */
public SpanHandler getSpanHandler()
{
	if (spanHandler == null) {
		createDefaultSpanHandler();
	}
	return(spanHandler);
}

/**
 * Creates the default span manager.
 */
protected void createDefaultSpanHandler()
{
	try {
		Class c = Class.forName("com.klg.jclass.table.SpanHandler");
		SpanHandler spanHandler = (SpanHandler)c.newInstance();
		spanHandler.setTable(this);
		setSpanHandler(spanHandler);
	}
	catch(Exception e) {
		System.out.println("SpanHandler initialization error: " +
                           e.getMessage());
	}
}

/**
 * Sets the span manager.
 * @param spanHandler the new span manager
 */
public void setSpanHandler(SpanHandler spanHandler)
{
	this.spanHandler = spanHandler;
}

/**
 * Adds a <code>JCCellRange</code> as a new spanned region. An
 * <CODE>IllegalArgumentException</CODE> is thrown if the
 * <code>JCCellRange</code> is invalid or overlaps an existing spanned cell
 * range.
 * @param cr the range defining the new spanned region to add
 * @exception java.lang.IllegalArgumentException Thrown if a cell range
 *     overlaps an existing spanned cell range.
 */
public void addSpannedRange(JCCellRange cr) throws IllegalArgumentException
{
	if (spanHandler == null) {
		createDefaultSpanHandler();
	}
	spanHandler.addSpannedRange(cr);
	requestChange(TableChangeHandler.TABLE_REPAINT);
}

/**
 * Removes all spanned cell ranges.
 */
public void clearSpannedRanges()
{
	if (spanHandler == null) {
		return;
	}

	// Remove all cell components. Those that should
	// stay will be re-added on repaint.
	ComponentModel componentHandler = getComponentModel();
	if (componentHandler != null) {
		componentHandler.removeCellComponents();
	}

	spanHandler.clearSpannedRanges();
	requestChange(TableChangeHandler.TABLE_REPAINT);
}

/**
 * Returns a Collection containing all of the spanned cell ranges.
 * @return Collection containing all of the spanned cell ranges
 */
public Collection getSpannedRanges()
{
	if (spanHandler == null) {
		return(null);
	}
	return(spanHandler.getSpannedRanges());
}

/**
 * Removes a cell range for the list of spanned ranges.
 * @param cr the range to remove
 */
public void removeSpannedRange(JCCellRange cr)
{
	if (spanHandler == null) {
		return;
	}
	spanHandler.removeSpannedRange(cr);
	requestChange(TableChangeHandler.TABLE_REPAINT);
}

/**
 * Sets the spanned cell ranges to a Collection of <code>JCCellRanges</code>.
 * Ranges are validated in order of iteration in the Collection.
 * @param ranges the collection of ranges to which the spanned cell ranges are
 *    to be set
 * @exception java.lang.IllegalArgumentException Thrown if a cell range
 *     overlaps an existing spanned cell range.
 */
public void setSpannedRanges(Collection ranges) throws IllegalArgumentException
{
	if (spanHandler == null) {
		createDefaultSpanHandler();
	}
	spanHandler.setSpannedRanges(ranges);
	requestChange(TableChangeHandler.TABLE_REPAINT);
}

/**
 * Returns a <code>JCCellRange</code> that represents that the spanned region
 * contains the given row and column. If the row and column aren't in a span,
 * null is returned.
 * @param row the row within the spanned range
 * @param column the column within the spanned range
 * @return the spanned range containing the row and column, if any
 */
public JCCellRange getSpannedRange(int row, int column)
{
	if (spanHandler == null) {
		return(null);
	}
	return(spanHandler.getSpannedRange(row, column));
}

/**
 * Gets the value of the <code>PointerTraverseForcesEdit</code> property.
 * <p>
 * <code>PointerTraverseForcesEdit</code> is used to determine what happens
 * when a user selects a new editable cell when another edit
 * is active and the value in the editor cannot be committed.
 * In JClass 4.0.1 and earlier, the only possible behaviour was to
 * warn the user, cancel the edit and start editing in the
 * new cell.  This corresponds to a value of <code>true</code> for this
 * property.  If <code>PointerTraverseForcesEdit</code> is set to
 * <code>false</code>, the current editor remains active, and traversal doesn't
 * occur.
 * @return whether pointer traversal forces an edit
 */
public boolean getPointerTraverseForcesEdit()
{
	return(pointer_traverse_forces_edit);
}

/**
 * Sets the value of the <code>PointerTraverseForcesEdit</code> property.
 * <p>
 * <code>PointerTraverseForcesEdit</code> is used to determine what happens
 * when a user selects a new editable cell when another edit
 * is active and the value in the editor cannot be committed.
 * In JClass 4.0.1 and earlier, the only possible behaviour was to
 * warn the user, cancel the edit and start editing in the
 * new cell.  This corresponds to a value of <code>true</code> for this
 * property.  If <code>PointerTraverseForcesEdit</code> is set to
 * <code>false</code>, the current editor remains active, and traversal doesn't
 * occur.
 * @param nval the new value of <code>PointerTraverseForcesEdit</code>
 */
public void setPointerTraverseForcesEdit(boolean nval)
{
	pointer_traverse_forces_edit = nval;
}

//
//
// Internal Methods
// ----------------------------------------------------------------------------
//

public void paintComponent(Graphics gc)
{

 

	// if not storing the image, paint the exposed area
	if (!store_image_enabled)
	{
		cellAreaHandler.getCellAreaRenderer().paintComponent(gc);
		return;
	}

	int width = getWidth();
	int height = getHeight();

	int cell_origin_x = cellAreaHandler.getCellOrigin().x;
	int cell_origin_y = cellAreaHandler.getCellOrigin().y;

	if (store_image == null) {
		store_image = createImage(width, height);
		paintUsingStoredImage(gc);
	}
	else {
		if (store_origin.x == cell_origin_x &&
            store_origin.y == cell_origin_y)
        {
			paintUsingStoredImage(gc);
		}
		else {
			int dx = cell_origin_x - store_origin.x;
			int dy = cell_origin_y - store_origin.y;

			Rectangle cellRect = cellAreaHandler.cellRect;
			CellArea area = cellAreaHandler.cellArea;
			if (area != null) {
				cellRect = area.getBounds();
			}

			if ((dx != 0 && dy != 0)    ||
			     mouseHandler.SELECTING ||
				 mouseHandler.DRAGGING)
			{
				paintUsingStoredImage(gc);
			}
			else {
				Graphics sgc = getStoredGraphics(gc);
				if (dy < 0) {
					sgc.copyArea(0, cellRect.y - dy, getWidth(),
					             cellRect.height + dy, dx, dy);
					sgc.clipRect(0, cellRect.y + cellRect.height + dy,
					             getWidth(), -dy);
				}
				else if (dy > 0) {
					sgc.copyArea(0, cellRect.y, getWidth(),
					             cellRect.height - dy, dx, dy);
					sgc.clipRect(0, cellRect.y, getWidth(), dy);
				}
				else if (dx < 0) {
					sgc.copyArea(cellRect.x - dx, 0, cellRect.width + dx,
					             getHeight(), dx, dy);
					sgc.clipRect(cellRect.x + cellRect.width + dx, 0, -dx,
					             getHeight());
				}
				else if (dx > 0) {
					sgc.copyArea(cellRect.x, 0, cellRect.width - dx,
					             getHeight(), dx, dy);
					sgc.clipRect(cellRect.x, 0, dx, getHeight());
				}

				// repaint the damaged region
				cellAreaHandler.getCellAreaRenderer().paintComponent(sgc);
				gc.drawImage(store_image, 0, 0, this);
				sgc.dispose();
			}
		}
	}

	store_origin.x = cell_origin_x;
	store_origin.y = cell_origin_y;
}

private void paintUsingStoredImage(Graphics gc)
{
	Graphics sgc = getStoredGraphics(gc);

	Rectangle r = getBounds();
	sgc.setColor(getBackground());
	sgc.fillRect(r.x, r.y, r.width, r.height);

	cellAreaHandler.getCellAreaRenderer().paintComponent(sgc);
	gc.drawImage(store_image, 0, 0, this);
}

private Graphics getStoredGraphics(Graphics gc)
{
	Graphics sgc = store_image.getGraphics();
	sgc.setColor(gc.getColor());
	sgc.setFont(gc.getFont());
	sgc.setClip(gc.getClipBounds());
	return(sgc);
}

public void reshape(int x, int y, int width, int height)
{
	if (width != getWidth() || height != getHeight()) {
		store_image = null;
	}
	super.reshape(x, y, width, height);
}

/**
 * Returns copy of the <CODE>CellSize</CODE> object for a row.
 * @param row the index of the row
 * @return a copy of the <CODE>CellSize</CODE> object for the given row
 * @see #setRowCellSize
 */
protected CellSize getRowCellSize(int row)
{
	return((CellSize)rowHeights.getValue(row));
}

/**
 * Sets the <code>CellSize</code> object for a row.
 * @param row the index of the row
 * @param value the new <code>CellSize</code> value for the given row
 */
protected void setRowCellSize(int row, CellSize value)
{
	if (rowHeights != null) {
		rowHeights.setValue(row, value);
		requestChange(TableChangeHandler.TABLE_RECALC_ROW_HEIGHTS,
		              row, JCTableEnum.NOVALUE);
	}
}

/**
 * Returns the <CODE>CellSize</CODE> object for a column.
 * @param column the index of the column
 * @return a copy of the <CODE>CellSize</CODE> object for the given column
 * @see #setColumnCellSize
 */
protected CellSize getColumnCellSize(int column)
{
	return((CellSize)columnWidths.getValue(column));
}

/**
 * Sets the <code>CellSize</code> object for a column.
 * @param column the index of the column
 * @param value the new <code>CellSize</code> value for the given column
 */
protected void setColumnCellSize(int column, CellSize value)
{
	if (columnWidths != null) {
		columnWidths.setValue(column, value);
		requestChange(TableChangeHandler.TABLE_RECALC_COLUMN_WIDTHS,
		              JCTableEnum.NOVALUE, column);
	}
}

/**
 * Forces required calculations based on a change in the width for a column.
 * @param column the column to base changes on
 */
protected void setColumnWidthChanged(int column)
{
	if (column < JCTableEnum.LABEL) {
		requestChange(TableChangeHandler.TABLE_RECALC_COLUMN_WIDTHS);
	}
	else {
		requestChange(TableChangeHandler.TABLE_RECALC_COLUMN_WIDTHS,
		              JCTableEnum.ALL, column);
	}
}

/**
 * Returns <code>true</code> if a relayout is required.
 * @return whether a relayout is needed
 * @see #setRelayoutRequired
 */
protected boolean isRelayoutRequired()
{
	return(needs_relayout);
}

/**
 * If set to <code>true</code>, this method forces JClass LiveTable to relayout
 * its contents when <code>setRepaintEnabled()</code> is set back to
 * <code>true</code>.
 * @param value if <code>true</code>, a relayout will be done
 * @see #setRepaintEnabled
 * @see #isRelayoutRequired
 * @see #setRecalcRequired
 */
protected void setRelayoutRequired(boolean value)
{
	needs_relayout = value;
}

/**
 * Returns <code>true</code> if a recalc is required.
 * @return Is a recalc needed?
 * @see #setRecalcRequired
 */
protected boolean isRecalcRequired()
{
	return(needs_recalc);
}

/**
 * If set to <code>true</code>, this method forces JClass LiveTable
 * to recalculate all internal dimensions when
 * <code>setRepaintEnabled()</code> is set back to <code>true</code>.
 * @param value if <code>true</code>, a recalc will be done
 * @see #setRepaintEnabled
 */
protected void setRecalcRequired(boolean value)
{
	needs_recalc = value;
}

/**
 * Returns <code>true</code> if repaint is required.
 * @return Is a repaint needed?
 * @see #setRepaintRequired
 */
protected boolean isRepaintRequired()
{
	return(needs_repaint);
}

/**
 * If set to <code>true</code>, this method forces JClass LiveTable
 * to repaint the entire contents when <code>setRepaintEnabled()</code>
 * is set back to <code>true</code>.
 * @param value if <code>true</code>, a repaint will be done
 * @see #setRepaintEnabled
 */
protected void setRepaintRequired(boolean value)
{
	needs_repaint = value;
}

/**
 * Forces required calculations based on a change in the height of a row.
 * @param row the row to base the changes on
 */
protected void setRowHeightChanged(int row)
{
	if (row < JCTableEnum.LABEL) {
		requestChange(TableChangeHandler.TABLE_RECALC_ROW_HEIGHTS);
	}
	else {
		requestChange(TableChangeHandler.TABLE_RECALC_ROW_HEIGHTS,
		              row, JCTableEnum.ALL);
	}
}

/**
 * Returns the vertical offset of cell text based on the margin and border.
 * @return the vertical offset of cell text based on the margin and border
 */
protected final int getTextHeightOffset()
{
	return(margin_height + cell_border_width);
}

/**
 * Returns the horizontal offset of cell text based on the margin and border.
 * @return the horizontal offset of cell text based on the margin and border
 */
protected final int getTextWidthOffset()
{
	return(margin_width + cell_border_width);
}

/**
 * Sends a request to process a change. Valid change flags include:
 * <UL>
 * <LI><CODE>TableChangeHandler.TABLE_RESET</CODE></LI>
 * <LI><CODE>TableChangeHandler.TABLE_REPAINT</CODE></LI>
 * <LI><CODE>TableChangeHandler.TABLE_LAYOUT</CODE></LI>
 * <LI><CODE>TableChangeHandler.TABLE_RECALC_ROW_HEIGHTS</CODE></LI>
 * <LI><CODE>TableChangeHandler.TABLE_RECALC_COLUMN_WIDTHS</CODE></LI>
 * </UL>
 * @param flags indicates the type of change(s)
 * @see #requestChange(int, int, int)
 */
protected void requestChange(int flags)
{
	requestChange(flags, JCTableEnum.NOVALUE, JCTableEnum.NOVALUE);
}

/**
 * Sends a request to process a change for the given row and column. Valid
 * change flags include:
 * <UL>
 * <LI><CODE>TableChangeHandler.TABLE_RESET</CODE></LI>
 * <LI><CODE>TableChangeHandler.TABLE_REPAINT</CODE></LI>
 * <LI><CODE>TableChangeHandler.TABLE_LAYOUT</CODE></LI>
 * <LI><CODE>TableChangeHandler.TABLE_RECALC_ROW_HEIGHTS</CODE></LI>
 * <LI><CODE>TableChangeHandler.TABLE_RECALC_COLUMN_WIDTHS</CODE></LI>
 * <LI><CODE>TableChangeHandler.CELL_RECALC_HEIGHT</CODE></LI>
 * <LI><CODE>TableChangeHandler.CELL_RECALC_WIDTH</CODE></LI>
 * <LI><CODE>TableChangeHandler.CELL_REPAINT</CODE></LI>
 * </UL>
 * @param flags indicates the type of change(s)
 * @param row the row index of the cell to be changed
 * @param column the column index of the cell to be changed
 */
protected void requestChange(int flags, int row, int column)
{
	if (isDisplayable()) {
		tableChangeHandler.process(flags, row, column);
	}
}

/**
 * Sends a request to process a change for a range of cells.
 * @param flags indicates the type of change(s)
 * @param cr the range for which to process the change requests
 * @see #requestChange(int, int, int)
 */
protected void requestChange(int flags, JCCellRange cr)
{
	if (isDisplayable()) {
		int rcnt = Math.min(cr.end_row, getNumRows() - 1);
		int ccnt = Math.min(cr.end_column, getNumColumns() - 1);
		for (int r = cr.start_row; r <= rcnt; r++) {
			for (int c = cr.start_column; c <= ccnt; c++) {
				tableChangeHandler.process(flags, r, c);
			}
		}
	}
}

/**
 * Determines the column that contains the pixel position.
 * @param x the x pixel position
 * @return the column containing the x pixel position
 */
protected int XtoColumn(int x)
{
	return(cellLayout.getColumn(x, 0));
}

/**
 * Determines the row that contains the pixel position.
 * @param y the y pixel position
 * @return the row containing the y pixel position
 */
protected int YtoRow(int y)
{
	return(cellLayout.getRow(0, y));
}

/**
 * Databound subclasses of table can override this method to return the
 * correct datatype for the column.
 * @param column the column index of the column whose class is required
 * @return the class of the given column
 */
protected Class getColumnClass(int column)
{
	return(null);
}

//
//
// Listeners
// ----------------------------------------------------------------------------
//

/**
 * Convenience method for firing <code>JCCellDisplayEvents</code>.
 * @param event the event to be fired
 */
protected void fireJCCellDisplayEvent(JCCellDisplayEvent event)
{
    Enumeration e = JCListenerList.elements(cellDisplayListeners);
    for (;e.hasMoreElements();) {
        JCCellDisplayListener listener = (JCCellDisplayListener)e.nextElement();
        listener.cellDisplay(event);
    }
}

/**
 * Adds the specified listener to receive instances
 * of <CODE>JCCellDisplayEvent</CODE> objects.
 * @param l the listener to be added
 * @see JCCellDisplayEvent
 */
public void addCellDisplayListener(JCCellDisplayListener l)
{
	cellDisplayListeners = JCListenerList.add(cellDisplayListeners, l);
}

/**
 * Removes the specified listener so it no longer receives
 * <CODE>JCCellDisplayEvent</CODE> objects.
 * @param l the listener to be removed
 * @see #addCellDisplayListener
 */
public void removeCellDisplayListener(JCCellDisplayListener l)
{
	cellDisplayListeners = JCListenerList.remove(cellDisplayListeners, l);
}

/**
 * Convenience method for firing <code>JCEditCellEvents</code>.
 * @param event the event to be fired
 */
protected void fireJCEditCellEvent(JCEditCellEvent event)
{
    Enumeration e = JCListenerList.elements(editCellListeners);
    for (;e.hasMoreElements();) {
        JCEditCellListener listener = (JCEditCellListener)e.nextElement();
        switch(event.getType()) {
        case JCEditCellEvent.BEFORE_EDIT_CELL:
            listener.beforeEditCell(event);
            break;
		case JCEditCellEvent.EDIT_CELL:
			listener.editCell(event);
			break;
        case JCEditCellEvent.AFTER_EDIT_CELL:
            listener.afterEditCell(event);
            break;
        }
    }
}

/**
 * Adds the specified listener to receive instances
 * of <CODE>JCEditCellEvent</CODE> objects.
 * @param l the listener to be added
 * @see JCEditCellEvent
 */
public void addEditCellListener(JCEditCellListener l)
{
	editCellListeners = JCListenerList.add(editCellListeners, l);
}

/**
 * Removes the specified listener so it no longer receives
 * <CODE>JCEditCellEvent</CODE> objects.
 * @param l the listener to be removed
 * @see #addEditCellListener
 */
public void removeEditCellListener(JCEditCellListener l)
{
	editCellListeners = JCListenerList.remove(editCellListeners, l);
}

/**
 * Convenience method for firing <code>JCPaintEvents</code>.
 * @param event the event to be fired
 */
protected void fireJCPaintEvent(JCPaintEvent event)
{
    Enumeration e = JCListenerList.elements(paintListeners);
    for (;e.hasMoreElements();) {
        JCPaintListener listener = (JCPaintListener)e.nextElement();
        switch(event.getType()) {
        case JCPaintEvent.BEFORE_PAINT:
            listener.beforePaint(event);
            break;
        case JCPaintEvent.AFTER_PAINT:
            listener.afterPaint(event);
            break;
        }
    }
}

/**
 * Adds the specified listener to receive instances
 * of <CODE>JCPaintEvent</CODE> objects.
 * @param l the listener to be added
 * @see JCPaintEvent
 */
public void addPaintListener(JCPaintListener l)
{
	paintListeners = JCListenerList.add(paintListeners, l);
}

/**
 * Removes the specified listener so it no longer receives
 * <CODE>JCPaintEvent</CODE> objects.
 * @param l the listener to be removed
 * @see #addPaintListener
 */
public void removePaintListener(JCPaintListener l)
{
    paintListeners = JCListenerList.remove(paintListeners, l);
}

/**
 * Convenience method for firing <code>JCResizeCellEvents</code>.
 * @param event the event to be fired
 */
protected void fireJCResizeCellEvent(JCResizeCellEvent event)
{
	Enumeration e = JCListenerList.elements(resizeListeners);
	for (;e.hasMoreElements();) {
		JCResizeCellListener listener = (JCResizeCellListener)e.nextElement();
		switch(event.getType()) {
		case JCResizeCellEvent.BEFORE_RESIZE:
			listener.beforeResizeCell(event);
			break;
		case JCResizeCellEvent.RESIZE:
			listener.resizeCell(event);
			break;
		case JCResizeCellEvent.AFTER_RESIZE:
			listener.afterResizeCell(event);
		}
	}
}

/**
 * Convenience method for firing <code>BEFORE_RESIZE</code> and
 * <code>AFTER_RESIZE</code> events. Called by
 * <code>setChartWidth/Height</code>, <code>setPixelWidth/Height</code>, and
 * <code>setMarginWidth/Height</code>.
 * @param row the row being resized
 * @param column the column being resized
 * @param oldWidth the original column width
 * @param oldHeight the original row height
 * @param newWidth the new column width
 * @param newHeight the new row height
 * @return the event
 */
protected JCResizeCellEvent doResizeEvents(int row, int column, int oldWidth,
							               int oldHeight, int newWidth,
                                           int newHeight)
{

	JCResizeCellEvent event =
        new JCResizeCellEvent(this, JCResizeCellEvent.BEFORE_RESIZE,
                              row, column, oldWidth, oldHeight);

	fireJCResizeCellEvent(event);

	// Abort if cancelled.
	if (event.isCancelled()) {
		event.type = JCResizeCellEvent.AFTER_RESIZE;
		return(event);
	}

	// Fire the RESIZE event.
	event = new JCResizeCellEvent(this, JCResizeCellEvent.RESIZE,
								  row, column, oldWidth, oldHeight);
	event.setNewColumnWidth(newWidth);
	event.setNewRowHeight(newHeight);
	fireJCResizeCellEvent(event);

	// Restore on cancel or change width on acceptance.
	if (event.isCancelled()) {
		event.setNewColumnWidth(JCTableEnum.NOVALUE);
		event.setNewRowHeight(JCTableEnum.NOVALUE);
	}

	event.type = JCResizeCellEvent.AFTER_RESIZE;
	return(event);
}

/**
 * Adds the specified listener to receive instances of
 * <CODE>JCResizeCellEvent</CODE> objects.
 * @param l the listener to be added
 * @see JCResizeCellEvent
 */
public void addResizeCellListener(JCResizeCellListener l)
{
	resizeListeners = JCListenerList.add(resizeListeners, l);
}

/**
 * Removes the specified listener so it no longer receives
 * <CODE>JCResizeCellListener</CODE> objects.
 * @param l the listener to be removed
 * @see #addResizeCellListener
 */
public void removeResizeCellListener(JCResizeCellListener l)
{
    resizeListeners = JCListenerList.remove(resizeListeners, l);
}

/**
 * Convenience method for firing <code>JCResizeCellEvents</code> to
 * <CODE>JCResizeCellMotionListeners</CODE>.
 * @param event the event to be fired
 */
protected void fireJCResizeCellMotionEvent(JCResizeCellEvent event)
{
	Enumeration e = JCListenerList.elements(resizeMotionListeners);
	for (;e.hasMoreElements();) {
		JCResizeCellMotionListener listener =
            (JCResizeCellMotionListener)e.nextElement();
		listener.resizeCellDragged(event);
	}
}

/**
 * Adds the specified listener to receive instances of
 * <CODE>JCResizeCellEvent</CODE> objects during the resize drag
 * process.
 * @param l the listener to be added
 * @see JCResizeCellEvent
 */
public void addResizeCellMotionListener(JCResizeCellMotionListener l)
{
	resizeMotionListeners = JCListenerList.add(resizeMotionListeners, l);
}

/**
 * Removes the specified listener so it no longer receives
 * <CODE>JCResizeCellMotionListener</CODE> objects.
 * @param l the listener to be removed
 * @see #addResizeCellMotionListener
 */
public void removeResizeCellMotionListener(JCResizeCellMotionListener l)
{
    resizeMotionListeners = JCListenerList.remove(resizeMotionListeners, l);
}

/**
 * Convenience method for firing <code>JCScrollEvents</code>.
 * @param event the event to be fired
 */
protected void fireJCScrollEvent(JCScrollEvent event)
{
	Enumeration e = JCListenerList.elements(scrollListeners);
	for(;e.hasMoreElements();) {
		JCScrollListener listener = (JCScrollListener)e.nextElement();
		switch(event.getType()) {
		case JCScrollEvent.SCROLL:
			listener.scroll(event);
			break;
		case JCScrollEvent.AFTER_SCROLL:
			listener.afterScroll(event);
			break;
		}
	}
}

/**
 * Adds the specified listener to receive instances of
 * <CODE>JCScrollEvent</CODE> objects.
 * @param l the listener to be added
 * @see JCScrollEvent
 */
public void addScrollListener(JCScrollListener l)
{
	scrollListeners = JCListenerList.add(scrollListeners, l);
}

/**
 * Removes the specified listener so it no longer receives
 * <CODE>JCScrollListener</CODE> objects.
 * @param l the listener to be removed
 * @see #addScrollListener
 */
public void removeScrollListener(JCScrollListener l)
{
    scrollListeners = JCListenerList.remove(scrollListeners, l);
}

/**
 * Convenience method for firing <code>JCSelectEvents</code>.
 * @param event the event to be fired
 */
protected void fireJCSelectEvent(JCSelectEvent event)
{
    Enumeration e = JCListenerList.elements(selectListeners);
    for (;e.hasMoreElements();) {
        JCSelectListener listener = (JCSelectListener)e.nextElement();
        switch(event.getType()) {
        case JCSelectEvent.BEFORE_SELECTION:
            listener.beforeSelect(event);
            break;
		case JCSelectEvent.SELECTION:
			listener.select(event);
			break;
        case JCSelectEvent.AFTER_SELECTION:
            listener.afterSelect(event);
            break;
        }
    }
}

/**
 * Adds the specified listener to receive instances
 * of <CODE>JCSelectEvent</CODE> objects.
 * @param l the listener to be added
 * @see JCSelectEvent
 */
public void addSelectListener(JCSelectListener l)
{
	selectListeners = JCListenerList.add(selectListeners, l);
}

/**
 * Removes the specified listener so it no longer receives
 * <CODE>JCSelectEvent</CODE> objects.
 * @param l the listener to be removed
 * @see #addSelectListener
 */
public void removeSelectListener(JCSelectListener l)
{
    selectListeners = JCListenerList.remove(selectListeners, l);
}

/**
 * Convenience method for firing <code>JCSortEvents</code>.
 * @param event the event to be fired
 */
protected void fireJCSortEvent(JCSortEvent event)
{
	Enumeration e = JCListenerList.elements(sortListeners);
	for (;e.hasMoreElements();) {
		JCSortListener listener = (JCSortListener)e.nextElement();
		listener.sort(event);
	}
}

/**
 * Adds the specified listener to receive instances of
 * <CODE>JCSortEvent</CODE> objects.
 * @param l the listener to be added
 * @see JCSortEvent
 */
public void addSortListener(JCSortListener l)
{
	sortListeners = JCListenerList.add(sortListeners, l);
}

/**
 * Removes the specified listener so it no longer receives
 * <CODE>JCSortListener</CODE> objects.
 * @param l the listener to be removed
 * @see #addSortListener
 */
public void removeSortListener(JCSortListener l)
{
    sortListeners = JCListenerList.remove(sortListeners, l);
}

/**
 * Convenience method for firing <code>JCTraverseCellEvents</code>.
 * @param event the event to be fired
 */
protected void fireJCTraverseCellEvent(JCTraverseCellEvent event)
{
	Enumeration e = JCListenerList.elements(traverseListeners);
	for(;e.hasMoreElements();) {
		JCTraverseCellListener listener =
            (JCTraverseCellListener)e.nextElement();
		switch(event.getType()) {
		case JCTraverseCellEvent.TRAVERSE_CELL:
			listener.traverseCell(event);
			break;
		case JCTraverseCellEvent.AFTER_TRAVERSE_CELL:
			listener.afterTraverseCell(event);
			break;
		}
	}
}

/**
 * Adds the specified listener to receive instances of
 * <CODE>JCTraverseCellEvent</CODE> objects.
 * @param l the listener to add
 * @see JCTraverseCellEvent
 */
public void addTraverseCellListener(JCTraverseCellListener l)
{
	traverseListeners = JCListenerList.add(traverseListeners, l);
}

/**
 * Removes the specified listener so it no longer receives
 * <CODE>JCTraverseCellListener</CODE> objects.
 * @param l the listener to remove
 * @see #addTraverseCellListener
 */
public void removeTraverseCellListener(JCTraverseCellListener l)
{
    traverseListeners = JCListenerList.remove(traverseListeners, l);
}




/**
 * Ensures that <code>JCTable</code> can be disposed.
 */
public void dispose()
{
	setLayout(null);

	removeAll();

	// Remove the link to the ui
	ui = null;

	if (hsb != null) {
		hsb.removeAdjustmentListener(adjustmentHandler);
		hsb = null;
	}
	if (vsb != null) {
		vsb.removeAdjustmentListener(adjustmentHandler);
		vsb = null;
	}
	removeTableMouseWheelListener();

	default_label_style = null;
	default_cell_style = null;
	moveables = null;

	if (adjustmentHandler != null) {
		adjustmentHandler.dispose();
		adjustmentHandler = null;
	}

	cellAreaHandler = null;
	cellLayout = null;
	cellStyles = null;
	columnWidths = null;

	if(dataView instanceof TableDataView) {
		((TableDataView)dataView).dispose();
	}
	dataView = null;

	CellSizeHandler.cachedFontHeights = null;

	focusManager.setEditTraverseHandler(null);
	keyHandler = null;
	mouseHandler = null;
	rowHeights = null;
	selectionHandler = null;
	spanHandler = null;
	tableChangeHandler = null;
	tableLayout = null;
	userData = null;
}

//
//
// PopupMenu support
// ----------------------------------------------------------------------------
//

/**
 * Creates a new popup menu.
 * @return the new popup menu
 */
protected TablePopupMenu createPopupMenu()
{
	return(new TablePopupMenu(this));
}

/**
 * Displays the popup menu at the given coordinates.
 * @param x the x position to place the popup menu
 * @param y the y position to place the popup menu
 * @return whether the popup succeeded
 */
protected boolean showPopupMenu(int x, int y)
{
	popupMenu.show(this, x, y);
	return(true);
}

/**
 * Processes table popup menu actions.
 * @param event the event generated by the action
 */
public void actionPerformed(ActionEvent event)
{
	String command = event.getActionCommand();
	if (command.equals(TablePopupMenu.commands[0]))	{       // "Hide Column..."
		hideShowColumns(true);
	}
	else if (command.equals(TablePopupMenu.commands[1])) {  // "Show Column..."
		hideShowColumns(false);
	}
	else if (command.equals(TablePopupMenu.commands[2])) {  // "Go to Row..."
		goToRow();
	}
}

/**
 * Displays a dialog box to select columns to show or hide.
 * @param hide if <code>true</code>, hide columns; if <code>false</code>,
 * show columns
 */
protected void hideShowColumns(boolean hide)
{
	// Create a column label list
    TableDataModel dataSource = getDataSource();
	List dialogLabelList = new ArrayList();
	for (int i = 0; i < dataSource.getNumColumns(); i++) {
		if (hide != isColumnHidden(i)) {
			int dc = i;
			if (dataView instanceof SortableDataViewModel) {
			    // The given column is a view column which must be
			    // mapped to a data column in order to hide it
				SortableDataViewModel sdvm =
				    (SortableDataViewModel)getDataView();
				dc = sdvm.getDataColumn(i);
			}
		    dialogLabelList.add(dataSource.getTableColumnLabel(dc));
		}
	}
	if (dialogLabelList.size() == 0) {
		String err = LocaleBundle.string(LocaleBundle.JCTABLE_ERROR);
		String msg = (hide ? LocaleBundle.string(LocaleBundle.NOCOLUMNS_HIDE) :
					  LocaleBundle.string(LocaleBundle.NOCOLUMNS_SHOW));
    	JCMessageHelper.showError(err, msg);
        return;
    }

	Frame frame = (Frame)SwingUtilities.getAncestorOfClass(Frame.class, this);
	PickColumnsDialog dialog = new PickColumnsDialog(frame, hide,
													 dialogLabelList.toArray());
	dialog.pack();
	dialog.center();
	dialog.show();
	int[] selected = dialog.getSelected();
	if (selected != null && selected.length != 0) {
		if (!hide) {
			int numRows = getNumRows();
			setVariableEstimateCount(numRows < 100 ? numRows : 100);
		}
		setRepaintEnabled(false);
		int selectedItem = 0;
		int menuItem = -1;
		for (int i = 0; i < dataSource.getNumColumns(); i++) {
			if (hide != isColumnHidden(i)){
				menuItem++;
				// This works because the selected array is in increasing order
				if (menuItem == selected[selectedItem]) {
					// Hide or show the column
					int vc = i;
					if (dataView instanceof SortableDataViewModel) {
					    // The given column is a data column which must be
					    // mapped to a view column in order to hide it
						SortableDataViewModel sdvm =
						    (SortableDataViewModel)dataView;
						vc = sdvm.getViewColumn(i);
					}
					setColumnHidden(vc, hide);
					selectedItem++;
					if (selectedItem >= selected.length) {
						break;
					}
				}
			}
		}
		setRepaintEnabled(true);
	}
}

/**
 * Displays a dialog box to select a row to traverse to.
 */
protected void goToRow()
{
    if ((getCurrentCell() == null) && !traverse(0,0,false,false)) {
        return;
    }
	GotoRowDialog dialog = new GotoRowDialog(
    	(Frame)SwingUtilities.getAncestorOfClass(Frame.class, this),
        getCurrentCell().row);
	dialog.open();
	traverse(dialog.getRow(), getCurrentCell().column, false, false);
}


 

/**
 * Overridden to record the old focus owner when table requests focus.  Not
 * used in JDK 1.4 or later.
 */
public void requestFocus()
{
	oldFocusOwner = getFocusManager().findFocusOwner(this);
	
//  
	super.requestFocus();
}

/**
 * This method gets called when table gains focus.
 * @param event the event generated when the table gains focus
 */
public void focusGained(FocusEvent event)
{
    
	getFocusManager().transferFocusCycle();
//	
}

/**
 * This method gets called when table loses focus.
 * @param event the event generated when the table loses focus
 */
public void focusLost(FocusEvent event)
{
    
//  
}

/*
 * Gets the value of the <code>License</code> property.
 * The <code>License</code> property displays license status information for
 * Quest Software in the Bean box if it's invalid.
 */

 

/*
 * Sets the value of the <code>License</code> property.
 * The <code>License</code> property displays license status information for
 * Quest Software in the Bean box if it's invalid.
 */

 

}
