/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCTableEnum.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import javax.swing.SwingConstants;
import com.klg.jclass.cell.JCCellInfo;

/**
 * These are commonly used variables and constants in JClass <code>JCTable</code>.
 * Java does not incorporate the idea of enums -- <CODE>JCTableEnum</CODE>
 * provides this functionality.
 */
public interface JCTableEnum {

/** A cell position constant that indicates all rows or all columns including labels. */
public static final int ALL                     = -998;
/** A cell position constant that indicates all cells in a row or column excluding the label. */
public static final int ALLCELLS                = -997;
/** Constant indicating all visible rows or columns. */
public static final int VISIBLE_CELLS            = -996;
/** A cell position constant that indicates the row or column label. */
public static final int LABEL                   = -1;
/** Constant indicating the largest possible integer. */
public static final int MAXINT                  = Integer.MAX_VALUE;
/** Constant indicating that the row or column is not found. */
public static final int NOTFOUND                = -999;
/** Constant indicating that the row or column has no value. */
public static final int NOVALUE                 = -999;


// Actions --------------------------------------------------------------------

/** A <code>TableAction</code> constant that specifies column selection. */
public static final int COLUMN_SELECT_ACTION    = 1;
/** A <code>TableAction</code> constant that specifies column sort. */
public static final int COLUMN_SORT_ACTION      = 2;
/** A <code>TableAction</code> constant that specifies column drag. */
public static final int COLUMN_DRAG_ACTION      = 3;
/** A <code>TableAction</code> constant that specifies row selection. */
public static final int ROW_SELECT_ACTION       = 4;
/** A <code>TableAction</code> constant that specifies row drag. */
public static final int ROW_DRAG_ACTION         = 5;
/** A <code>TableAction</code> constant that specifies copy. */
public static final int COPY_ACTION             = 6;
/** A <code>TableAction</code> constant that specifies paste. */
public static final int PASTE_ACTION            = 7;


// Alignment ------------------------------------------------------------------

/** Left alignment constant. */
public static final int LEFT                    = SwingConstants.LEFT;
/** Center alignment constant. */
public static final int CENTER                  = SwingConstants.CENTER;
/** Right alignment constant. */
public static final int RIGHT                   = SwingConstants.RIGHT;
/** Top alignment constant. */
public static final int TOP                     = SwingConstants.TOP;
/** Bottom alignment constant. */
public static final int BOTTOM                  = SwingConstants.BOTTOM;

// Old constants for compatibility with version 3.x

/** A table 3 compatibility alignment constant. */
public static final int TOPLEFT                 = 0;
/** A table 3 compatibility alignment constant. */
public static final int TOPCENTER               = 1;
/** A table 3 compatibility alignment constant. */
public static final int TOPRIGHT                = 2;
/** A table 3 compatibility alignment constant. */
public static final int MIDDLELEFT              = 3;
/** A table 3 compatibility alignment constant. */
public static final int MIDDLECENTER            = 4;
/** A table 3 compatibility alignment constant. */
public static final int MIDDLERIGHT             = 5;
/** A table 3 compatibility alignment constant. */
public static final int BOTTOMLEFT              = 6;
/** A table 3 compatibility alignment constant. */
public static final int BOTTOMCENTER            = 7;
/** A table 3 compatibility alignment constant. */
public static final int BOTTOMRIGHT             = 8;


// AutoScroll -----------------------------------------------------------------

/** A constant that disables auto scroll. */
public static final int AUTO_SCROLL_NONE        = 0;
/** A constant that enables row level auto scroll. */
public static final int AUTO_SCROLL_ROW         = 1;
/** A constant that enables column level auto scroll. */
public static final int AUTO_SCROLL_COLUMN      = 2;
/** A constant that enables row and column level auto scroll. */
public static final int AUTO_SCROLL_BOTH        = 3;


// Borders --------------------------------------------------------------------

/** A <code>CellStyleModel</code> <code>BorderSide</code> property constant indicating no borders. */
public static final int BORDERSIDE_NONE         = 0;
/** A <code>CellStyleModel</code> <code>BorderSide</code> property constant indicating left borders. */
public static final int BORDERSIDE_LEFT         = 1;
/** A <code>CellStyleModel</code> <code>BorderSide</code> property constant indicating top borders. */
public static final int BORDERSIDE_TOP          = 2;
/** A <code>CellStyleModel</code> <code>BorderSide</code> property constant indicating bottom borders. */
public static final int BORDERSIDE_BOTTOM       = 4;
/** A <code>CellStyleModel</code> <code>BorderSide</code> property constant indicating right borders. */
public static final int BORDERSIDE_RIGHT        = 8;
/** A <code>CellStyleModel</code> <code>BorderSide</code> property constant indicating all borders. */
public static final int BORDERSIDE_ALL          = 15;

/** A <code>JCCellBorder</code> constant indicating no border. */
public static final int BORDER_NONE             = 0;
/** A <code>JCCellBorder</code> constant indicating an etched in border. */
public static final int BORDER_ETCHED_IN        = 1;
/** A <code>JCCellBorder</code> constant indicating an etched out border. */
public static final int BORDER_ETCHED_OUT       = 2;
/** A <code>JCCellBorder</code> constant indicating a simple inset border. */
public static final int BORDER_IN               = 3;
/** A <code>JCCellBorder</code> constant indicating a simple raised border. */
public static final int BORDER_OUT              = 4;
/** A <code>JCCellBorder</code> constant indicating a plain border. */
public static final int BORDER_PLAIN            = 5;
/** A <code>JCCellBorder</code> constant indicating a frame inset border. */
public static final int BORDER_FRAME_IN         = 6;
/** A <code>JCCellBorder</code> constant indicating a frame raised border. */
public static final int BORDER_FRAME_OUT        = 7;
/** A <code>JCCellBorder</code> constant indicating a thin border. */
public static final int BORDER_THIN             = 8;

/** A <code>CellStyleModel</code> <code>CellBorderColorMode</code> constant indicating
 * that the border color is based on the <code>CellBorderColor</code> property.
 */
public static final int USE_CELL_BORDER_COLOR   = 0;
/** A <code>CellStyleModel</code> <code>CellBorderColorMode</code> constant indicating
 * that the border color is based on the background of the cell or the selected
 * background for selected cells.
 */
public static final int BASE_ON_BACKGROUND      = 1;
/** A <code>CellStyleModel</code> <code>CellBorderColorMode</code> constant indicating
 * that the border color is based on the foreground of the cell or the selected
 * foreground for selected cells.
 */
public static final int BASE_ON_FOREGROUND      = 2;


// Cell Dimensions ------------------------------------------------------------

/** A <code>PixelWidth</code>/<code>Height</code> constant that fixes the size to the current value. */
public static final int AS_IS                   = 33000;
/** A <code>PixelWidth</code>/<code>Height</code> constant that sets the size to variable. */
public static final int VARIABLE                = 33001;
/** A <code>PixelWidth</code>/<code>Height</code> constant that sets the size to variable
 * for the number of rows/columns specified in the <code>VariableEstimateCount</code> property.
 */
public static final int VARIABLE_ESTIMATE       = 33002;


// Clip Arrows ----------------------------------------------------------------

/** A <code>CellStyleModel</code> <code>ClipHints</code> constant that indicates clip arrows should not be shown. */
public static final int SHOW_NONE               = JCCellInfo.SHOW_NONE;
/** A <code>CellStyleModel</code> <code>ClipHints</code> constant the indicates only horizontal clip arrows
should be shown. */
public static final int SHOW_HORIZONTAL         = JCCellInfo.SHOW_HORIZONTAL;
/** A <code>CellStyleModel</code> <code>ClipHints</code> constant that indicates only vertical clip
 * arrows should be shown.
 */
public static final int SHOW_VERTICAL           = JCCellInfo.SHOW_VERTICAL;
/** A <code>CellStyleModel</code> <code>ClipHints</code> constant that indicates that horizontal and
 * vertical clip arrows should be shown.
 */
public static final int SHOW_ALL                = JCCellInfo.SHOW_ALL;

// Old constants for compatibility with version 3.x

/** A table 3 compatibility clip arrow constant. */
public static final int CLIP_ARROWS_NONE        = SHOW_NONE;
/** A table 3 compatibility clip arrow constant. */
public static final int CLIP_ARROWS_RIGHT       = SHOW_HORIZONTAL;
/** A table 3 compatibility clip arrow constant. */
public static final int CLIP_ARROWS_DOWN        = SHOW_VERTICAL;
/** A table 3 compatibility clip arrow constant. */
public static final int CLIP_ARROWS_BOTH        = SHOW_ALL;


// Edit width/height policies -------------------------------------------------

/** An <code>EditHeightPolicy</code>/<code>EditWidthPolicy</code> constant that indicates
 * editors should be sized to the size of the cell.
 */
public static final int EDIT_SIZE_TO_CELL          = 0;
/** An <code>EditHeightPolicy</code>/<code>EditWidthPolicy</code> constant that indicates
 * editors should be sized to at least their minimum size.
 */
public static final int EDIT_ENSURE_MINIMUM_SIZE   = 1;
/** An <code>EditHeightPolicy</code>/<code>EditWidthPolicy</code> constant that indicates
 * editors should be sizes to at least their preferred size.
 */
public static final int EDIT_ENSURE_PREFERRED_SIZE = 2;


// Focus Indicator ------------------------------------------------------------

/** A <code>FocusIndicator</code> constant that sets no focus. */
public static final int FOCUS_NONE              = 0;
/** A <code>FocusIndicator</code> constant that sets the focus as a highlighted background. */
public static final int FOCUS_HIGHLIGHT         = 1;
/** A <code>FocusIndicator</code> constant that sets the focus to a 2-pixel width rectangle. */
public static final int FOCUS_RECTANGLE         = 2;
/** A <code>FocusIndicator</code> constant that sets the focus to a 1-pixel width rectangle. */
public static final int FOCUS_THIN_RECTANGLE    = 3;
/** A <code>FocusIndicator</code> constant that sets the focus to a dashed rectangle. */
public static final int FOCUS_DASHED_RECTANGLE  = 4;


// Frozen Cells and Labels ----------------------------------------------------

/** A <code>FrozenColumnPlacement</code> constant that positions frozen columns on the right of table. */
public static final int PLACE_RIGHT             = RIGHT;
/** A <code>FrozenColumnPlacement</code> constant that positions frozen columns on the left of table. */
public static final int PLACE_LEFT              = LEFT;
/** A <code>FrozenColumnPlacement</code> constant that positions frozen rows on the top of table. */
public static final int PLACE_TOP               = TOP;
/** A <code>FrozenColumnPlacement</code> constant that positions frozen rows on the bottom of table. */
public static final int PLACE_BOTTOM            = BOTTOM;


// Jump Mode ------------------------------------------------------------------

/** A <code>JumpScroll</code> constant that disables jump scroll. */
public static final int JUMP_NONE               = 0;
/** A <code>JumpScroll</code> constant that enables horizontal jump scroll. */
public static final int JUMP_HORIZONTAL         = 1;
/** A <code>JumpScroll</code> constant that enables vertical jump scroll. */
public static final int JUMP_VERTICAL           = 2;
/** A <code>JumpScroll</code> constant that enables horizontal and vertical jump scroll. */
public static final int JUMP_ALL                = 3;


// Repeat Background/Foreground -----------------------------------------------

/** A <code>CellStyleModel</code> <code>RepeatBackground</code>/<code>RepeatForeground</code>
 *  constant that disables repeating colors.
 */
public static final int REPEAT_NONE             = 0;
/** A <code>CellStyleModel</code> <code>RepeatBackground</code>/<code>RepeatForeground</code>
 * constant that enables repeating colors by row. The colors are determined by the
 * <code>RepeatBackgroundColors</code> and <code>RepeatForegroundColors</code> properties.
 */
public static final int REPEAT_ROW              = 1;
/** A <code>CellStyleModel</code> <code>RepeatBackground</code>/<code>RepeatForeground</code>
 * constant that enables repeating colors by column. The colors are determined by the
 * <code>RepeatBackgroundColors</code> and <code>RepeatForegroundColors</code> properties. */
public static final int REPEAT_COLUMN           = 2;


// ResultSetDataSource --------------------------------------------------------

/** A <code>JCResultSetDataSource</code> constant indicating boolean data. */
public static final int TYPE_BOOLEAN            = 0;
/** A <code>JCResultSetDataSource</code> constant indicating date data. */
public static final int TYPE_DATE               = 1;
/** A <code>JCResultSetDataSource</code> constant indicating double data. */
public static final int TYPE_DOUBLE             = 2;
/** A <code>JCResultSetDataSource</code> constant indicating float data. */
public static final int TYPE_FLOAT              = 3;
/** A <code>JCResultSetDataSource</code> constant indicating integer data. */
public static final int TYPE_INTEGER            = 4;
/** A <code>JCResultSetDataSource</code> constant indicating String data. */
public static final int TYPE_STRING             = 5;


// Resize ---------------------------------------------------------------------

/** An <code>AllowCellResize</code> constant that disables interactive resize. */
public static final int RESIZE_NONE             = 0;
/** An <code>AllowCellResize</code> constant that enables interactive row resize. */
public static final int RESIZE_ROW              = 1;
/** An <code>AllowCellResize</code> constant that enables interactive column resize. */
public static final int RESIZE_COLUMN           = 2;
/** An <code>AllowCellResize</code> constant that enables interactive resize. */
public static final int RESIZE_ALL              = 4;

/** An <code>AllowResizeBy</code> constant that allows resize by cells (and not labels). */
public static final int RESIZE_BY_CELLS         = 1;
/** An <code>AllowResizeBy</code> constant that allows resize by labels (and not cells). */
public static final int RESIZE_BY_LABELS        = 2;
/** An <code>AllowResizeBy</code> constant that allows resize by cells and labels. */
public static final int RESIZE_BY_ALL           = 4;


// Scrollbar ------------------------------------------------------------------

/** A <code>HorizSBDisplay</code>/<code>VertSBDisplay</code> constant that indicates
 * scrollbars are displayed only if required.
 */
public static final int SCROLLBAR_AS_NEEDED     = 0;
/** A <code>HorizSBDisplay</code>/<code>VertSBDisplay</code> constant that indicates
 * scrollbars are always displayed.
 */
public static final int SCROLLBAR_ALWAYS        = 1;
/** A <code>HorizSBDisplay</code>/<code>VertSBDisplay</code> constant that indicates
 * scrollbars are never displayed.
 */
public static final int SCROLLBAR_NEVER         = 2;

// Old constants for compatibility with version 3.x

/** A table 3 compatibility <code>HorizSBDisplay</code>/<code>VertSBDisplay</code> constant. */
public static final int SBDISPLAY_AS_NEEDED     = SCROLLBAR_AS_NEEDED;
/** A table 3 compatibility <code>HorizSBDisplay</code>/<code>VertSBDisplay</code> constant. */
public static final int SBDISPLAY_ALWAYS        = SCROLLBAR_ALWAYS;
/** A table 3 compatibility <code>HorizSBDisplay</code>/<code>VertSBDisplay</code> constant. */
public static final int SBDISPLAY_NEVER         = SCROLLBAR_NEVER;

/** A <code>HorizSBPosition</code>/<code>VertSBPosition</code> constant that positions
 * the scrollbar to the right or bottom of the cell area.
 */
public static final int POSITION_BY_CELLS       = 0;
/** A <code>HorizSBPosition</code>/<code>VertSBPosition</code> constant that positions
 * the scrollbar to the right or bottom edge of the area allocated to table.
 */
public static final int POSITION_AT_SIDE        = 1;

// Old constants for compatibility with version 3.x

/** A table 3 compatibility <code>HorizSBPosition</code>/<code>VertSBPosition</code> constant. */
public static final int SBPOSITION_CELLS        = POSITION_BY_CELLS;
/** A table 3 compatibility <code>HorizSBPosition</code>/<code>VertSBPosition</code> constant. */
public static final int SBPOSITION_SIDE         = POSITION_AT_SIDE;

/** A <code>HorizSBAttachment</code>/<code>VertSBAttachment</code> constant the sets the
 * height/width of the scrollbar to the size of the cell area.
 */
public static final int SIZE_TO_CELLS           = 0;
/** A <code>HorizSBAttachment</code>/<code>VertSBAttachment</code> constant that sets the
 * height/width of the scrollbar to the height/width of table.
 */
public static final int SIZE_TO_TABLE           = 1;

// Old constants for compatibility with version 3.x

/** A table 3 compatibility <code>HorizSBAttachment</code>/<code>VertSBAttachment</code> constant. */
public static final int ATTACH_CELLS            = SIZE_TO_CELLS;
/** A table 3 compatibility <code>HorizSBAttachment</code>/<code>VertSBAttachment</code> constant. */
public static final int ATTACH_SIDE             = SIZE_TO_TABLE;

/** A <code>HorizSBTrack</code>/<code>VertSBTrack</code> constant that indicates full table
 * reposition during the scroll.
 */
public static final int TRACK_LIVE              = 0;
/** A <code>VertSBTrack</code> constant that indicates a scroll indicator should
 * present the top row number during scroll rather than repositionning the table.
 */
public static final int TRACK_ROW_NUMBER        = 1;
/** A <code>VertSBTrack></code> constant that indicates a scroll indicator should
 * present the data from the column in the <code>VertSBTrackColumn</code> of the top row
 * during scroll rather than repositionning the table. For example, use <code>JCTableEnum.LABEL</code>
 * in order to display the label of the top row in the scroll indicator.
 */
public static final int TRACK_COLUMN            = 2;
/** A <code>HorizSBTrack</code> constant that indicates a scroll indicator should
 * present the left column number during scroll rather than repositionning the table.
 */
public static final int TRACK_COLUMN_NUMBER     = 1;
/** A <code>HorizSBTrack</code> constant that indicates a scroll indicator should
 * present the data from the row in the <code>HorizSBTrackRow</code> of the left column
 * during scroll rather than repositionning the table. For example, use <code>JCTableEnum.LABEL</code>
 * in order to display the label of the left column in the scroll indicator.
 */
public static final int TRACK_ROW               = 2;


// Selection ------------------------------------------------------------------

/** A <code>SelectionPolicy</code> constant that disables selection. */
public static final int SELECT_NONE             = 0;
/** A <code>SelectionPolicy</code> constant that allows a single cell to be selected. */
public static final int SELECT_SINGLE           = 1;
/** A <code>SelectionPolicy</code> constant that allows a range of cells to be selected. */
public static final int SELECT_RANGE            = 2;
/** A <code>SelectionPolicy</code> constant that allows multiply ranges of cells to be selected. */
public static final int SELECT_MULTIRANGE       = 3;

/** A <code>SelectedBackgroundMode</code>/<code>SelectedForegroundMode</code> constant that indicates
 * the selected color is based on the <code>SelectedBackground</code> property.
 */
public static final int USE_SELECTED_BACKGROUND = 0;
/** A <code>SelectedBackgroundMode</code>/<code>SelectedForegroundMode</code> constant that indicates
 * the selected color is based on the <code>SelectedForeground</code> property.
 */
public static final int USE_SELECTED_FOREGROUND = 1;
/** A <code>SelectedBackgroundMode</code> constant that indicates the selected color is
 * based on the cell background.
 */
public static final int USE_CELL_BACKGROUND     = 2;
/** A <code>SelectedForegroundMode</code> constant that indicates the selected color is
 * based on the cell foreground.
 */
public static final int USE_CELL_FOREGROUND     = 3;


// Traverse -------------------------------------------------------------------

/** A <code>JCTraverseCellEvent</code> <code>TraverseType</code> that indicates the traversal was
 * initiated by a mouse click.
 */
public static final int TRAVERSE_POINTER        = 0;
/** A <code>JCTraverseCellEvent</code> <code>TraverseType</code> that indicates the traversal was
 * initiated by the down arrow.
 */
public static final int TRAVERSE_DOWN           = 1;
/** A <code>JCTraverseCellEvent</code> <code>TraverseType</code> that indicates the traversal was
 * initiated by the up arrow.
 */
public static final int TRAVERSE_UP             = 2;
/** A <code>JCTraverseCellEvent</code> <code>TraverseType</code> that indicates the traversal was
 * initiated by the left arrow.
 */
public static final int TRAVERSE_LEFT           = 3;
/** A <code>JCTraverseCellEvent</code> <code>TraverseType</code> that indicates the traversal was
 * initiated by the right arrow.
 */
public static final int TRAVERSE_RIGHT          = 4;
/** A <code>JCTraverseCellEvent</code> <code>TraverseType</code> that indicates the traversal was
 * initiated by a page up.
 */
public static final int TRAVERSE_PAGEUP         = 5;
/** A <code>JCTraverseCellEvent</code> <code>TraverseType</code> that indicates the traversal was
 * initiated by a page down.
 */
public static final int TRAVERSE_PAGEDOWN       = 6;
/** A <code>JCTraverseCellEvent</code> <code>TraverseType</code> that indicates the traversal was
 * initiated by the home key.
 */
public static final int TRAVERSE_HOME           = 7;
/** A <code>JCTraverseCellEvent</code> <code>TraverseType</code> that indicates the traversal was
 * initiated by the end key.
 */
public static final int TRAVERSE_END            = 8;
/** A <code>JCTraverseCellEvent</code> <code>TraverseType</code> that indicates the traversal was
 * initiated by control home.
 */
public static final int TRAVERSE_TOP            = 9;
/** A <code>JCTraverseCellEvent</code> <code>TraverseType</code> that indicates the traversal was
 * initiated by control end.
 */
public static final int TRAVERSE_BOTTOM         = 10;
/** A <code>JCTraverseCellEvent</code> <code>TraverseType</code> that indicates the traversal was
 * initiated programmatically.
 */
public static final int TRAVERSE_TO_CELL        = 11;

// Data Format ---------

/** File format value indicating that table should try and detect the
 * format. */
public static final int DETECT = -1;

/** Default file format for table data sources. */
public static final int STANDARD = 0;

/** Comma-separated values in a table data souce. */
public static final int CSV = 1;

/** XML file format in a table data souce. */
public static final int XML = 2;

/** Custom file format in a table data souce. */
public static final int CUSTOM = 3;

}
