/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: KeyActionInitiator.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import java.awt.AWTEvent;
import java.awt.event.KeyEvent;
import java.io.Serializable;

/**
 * <CODE>KeyActionInitiator</CODE> are used by <CODE>TableAction</CODE> to
 * associate a combination of key code and modifier to a specific action 
 * on table.
 */
public class KeyActionInitiator implements ActionInitiator, Serializable {
	
/** Key code. */
public int keycode;
	
/** Modifier for the key code. */
public int modifier;

/**
 * Creates a new <CODE>KeyActionInitiator</CODE> given a key and modifier.
 * <P>Valid modifiers are:
 * <UL>
 * <LI><CODE>InputEvent.ALT_GRAPH_MASK</CODE></LI>
 * <LI><CODE>InputEvent.ALT_MASK</CODE></LI>
 * <LI><CODE>InputEvent.CTRL_MASK</CODE></LI>
 * <LI><CODE>InputEvent.META_MASK</CODE></LI>
 * <LI><CODE>InputEvent.SHIFT_MASK</CODE></LI>
 * <LI><CODE>ActionInitiator.NO_MODIFIER_MASK</CODE></LI>
 * <LI><CODE>ActionInitiator.ANY_MODIFIER_MASK</CODE></LI>
 * </UL>
 * @param keycode The key code
 * @param modifier The modifier for the key code
 */
public KeyActionInitiator(int keycode, int modifier)
{
    this.keycode = keycode;
    this.modifier = modifier;
}

/**
 * Returns true if the given event matches the ActionInitiator settings.
 */
public boolean isMatch(AWTEvent e)
{
    if (!(e instanceof KeyEvent)) {
        return(false);
    }
    KeyEvent ke = (KeyEvent)e;

    if (modifier == ANY_MODIFIER_MASK) {
        return(keycode == ke.getKeyCode());
    }
    else {
        return(keycode == ke.getKeyCode() && modifier == ke.getModifiers());
    }
}
	
}
