/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Moveable.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

/**
 * Moveable defines the interface required by classes that DataViewSupport
 * can move/add/delete and swap.
 */
public interface Moveable {

/**
 * Adjust the series after moving columns.
 */
public void moveColumns(int start_column, int num_columns, int destination);
	
/**
 * Adjust the series after moving rows.
 */
public void moveRows(int start_row, int num_rows, int destination);

/**
 * Reorders columns after the column mapping changes.
 */
public void remapColumns(int new_positions[]);
	
/**
 * Reorders rows after the row mapping changes.
 */
public void remapRows(int new_positions[]);

/**
 * Adjusts the series after adding or deleting columns.
 * @param start_column The start position
 * @param num_columns The number of columns
 * @param direction Use <CODE>+1</CODE> for insert and <CODE>-1</CODE>
 * for delete.
 */
public void shiftColumn(int start_column, int num_columns, int direction);

/**
 * Adjusts the series after adding or deleting rows.
 * @param start_row The start position
 * @param num_rows The number of rows
 * @param direction Use <CODE>+1</CODE> for insert and <CODE>-1</CODE>
 * for delete.
 */
public void shiftRow(int start_row, int num_rows, int direction);

/**
 * Adjusts the series based on two columns swapping position.
 * @param column1 The first column to swap
 * @param column2 The second column to swap
 */
public void swapColumns(int column1, int column2);
	
/**
 * Adjusts the series based on two rows swapping position.
 * @param row1 The first row to swap
 * @param row2 The second row to swap
 */
public void swapRows(int row1, int row2);

}
