/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: PrintCellAreaHandler.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import java.awt.Point;
import java.awt.Rectangle;

/**
 * <CODE>PrintCellAreaHandler</CODE> controls the creation and removal of
 * <CODE>CellAreas</CODE> within the table during printing. It overrides the
 * getBounds methods of <CODE>CellAreaHandler</CODE> because cell layout during
 * printing is controlled by <CODE>JCPrintTable</CODE>.
 */
public class PrintCellAreaHandler extends CellAreaHandler {

protected JCCellRange page_range;

/**
 * Creates a new controller for Areas.
 */
public PrintCellAreaHandler(JCTable table)
{
    super(table);
}

/**
 * Returns the bounds of the cell accounting for the position of the CellArea
 * that manages the cell.
 */
public Rectangle getBounds(int row, int column)
{
    return(getBounds(row, column, new Rectangle()));
}

/**
 * Returns the bounds of a cell accounting for the position of the CellArea
 * that manages the cell, using the given Rectangle.
 */
public Rectangle getBounds(int row, int column, Rectangle r)
{
    r = table.getCellLayout().getBounds(row, column, r);

    // Adjust width for spans
    if (table.spanHandler != null) {
        JCCellRange cr = table.spanHandler.getSpannedRange(row, column);
        if (cr != null) {
            // Cell is spanned and is the controlling cell
            if (table.spanHandler.isControllingCell(cr, row, column)) {
                // Adjust the height of the bounding rectangle to cover all
                // spanned rows
                for (int sr = row + 1; sr <= cr.end_row; sr++) {
                    r.height += getBounds(sr, column).height;
                }
                // Adjust the width of the bounding rectangle to cover all
                // spanned columns
                for (int sc = column + 1; sc <= cr.end_column; sc++) {
                    r.width += getBounds(row, sc).width;
                }
            }
        }
    }

    Point p = ((JCPrintTable)table).getBounds(row, column);
    r.x = p.x;
    r.y = p.y;

    return(r);
}

/**
 * Override determineThinSides to work to end of page range rather than
 * end of table.
 */
protected int determineThinSides(int row, int column)
{
    if (page_range != null) {
        int end_row = page_range.end_row;
        int end_column = page_range.end_column;
        if (table.getFrozenColumns() > 0 &&
            table.getFrozenColumnPlacement() == JCTableEnum.PLACE_RIGHT)
        {
            end_column = table.getFrozenColumns() - 1;
        }
        if (table.getFrozenRows() > 0 &&
            table.getFrozenRowPlacement() == JCTableEnum.PLACE_BOTTOM)
        {
            end_row = table.getFrozenRows() - 1;
        }
        return(determineThinSides(row, column, end_row, end_column));
    }
    else {
        return(super.determineThinSides(row, column));
    }
}

}
