/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: RowColumnSeriesModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import java.util.List;

/**
 * <CODE>RowColumnSeriesModel</CODE> defines the interface required by classes
 * that  can store row or column based properties.
 */
public interface RowColumnSeriesModel {

/**
 * Gets the default member value. The default value is always equivalent to the
 * ALL position.
 */
public Object getDefault();
						 
/**
 * Creates or sets the default member to the specified value. The default value
 * is always equivalent to the ALL position.
 */
public void setDefault(Object value);

/**
 * Returns the value at the specified position.
 * @return null if the series is empty.
 */
public Object getValue(int position);
	
/**
 * Sets the specified cell to the value.
 * @return true if the series was changed after adding the value.
 */
public boolean setValue(int position, Object value);

/**
 * Sets a range of cells to the value.
 * @return true if the series was changed after adding the value.
 */
public boolean setValue(int start_position, int end_position, Object value);

/**
 * Returns an iterator for an ordered set of series values.
 */
public List getSeriesValues();

/**
 * Adjust the series after moving.
 */
public void move(int start, int num, int destination);
	
/**
 * Reorders after the mapping changes.
 */
public void remap(int new_positions[]);
	
/**
 * Adjusts the series after adding or deleting.
 * @param start the start position
 * @param num the number of rows
 * @param direction Use <CODE>+1</CODE> for insert and <CODE>-1</CODE>
 * for delete.
 */
public void shift(int start, int num, int direction);

/**
 * Adjusts the series based on two entries swap position.
 */
public void swap(int position1, int position2);

}
