/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: SelectionModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import java.util.Collection;

/**
 * The <CODE>SelectionModel</CODE> interface represents the current cell
 * selection state in  Table. Although a default implementation is supplied that
 * supports NONE, SINGLE, RANGE  and MULTIRANGE selection, the model opens up
 * possibilities that previously could only be down through the
 * <CODE>JCSelectListener</CODE> interface.
 */
public interface SelectionModel extends Moveable {
	
/** Constant that indicates that the cell range represents the start of
    selection. */
public static final int SELECT = 0;
/** Constant that indicates that the cell range will be added as a new range to
    the selection. */
public static final int ADD = 1;
/** Constant that indicates that the given cell will be extend to the current
    selection. */
public static final int EXTEND = 2;								
/** Constant that indicates that the given cell range is being deselected. */
public static final int DESELECT = 3;
/** Constant that indicates that the given cell is the end of selection. */
public static final int END = 4;

/** A SelectionPolicy constant that indicates that no selection is allowed. */
public static final int SELECT_NONE = JCTableEnum.SELECT_NONE;
/** A SelectionPolicy constant that indicates that a single cell may be
    selected. */
public static final int SELECT_SINGLE = JCTableEnum.SELECT_SINGLE;
/** A SelectionPolicy constant that indicates that a range of cells, rows or
    columnsmay be selected. */
public static final int SELECT_RANGE = JCTableEnum.SELECT_RANGE;
/** A SelectionPolicy constant that indicates that multiple ranges of cells,
    rows or columns may be selected. */
public static final int SELECT_MULTIRANGE = JCTableEnum.SELECT_MULTIRANGE;

/**
 * Returns the JCTable controlled by the SelectionModel.
 */
public JCTable getTable();
	
/**
 * Sets the JCTable controlled by the SelectionModel.
 */
public void setTable(JCTable table);

/**
 * Returns the policy of selection supported by the model. Valid policies
 * are <CODE>SELECT_NONE</CODE>, <CODE>SELECT_SINGLE</CODE>,
 * <CODE>SELECT_RANGE</CODE>, and <CODE>SELECT_MULTIRANGE</CODE>.
 */
public int getSelectionPolicy();

/**
 * Sets the policy of selection supported by the model. Valid policies
 * are <CODE>SELECT_NONE</CODE>, <CODE>SELECT_SINGLE</CODE>,
 * <CODE>SELECT_RANGE</CODE>, and <CODE>SELECT_MULTIRANGE</CODE>. 
 */
public void setSelectionPolicy(int value);

/**
 * Adds a range of columns to the selection.
 * @return true if the model supports multi-range column selection.
 */
public boolean addColumnSelection(int start_column, int end_column);

/**
 * Adds a range of rows to the selection.
 * @return true if the model supports multi-range row selection.
 */
public boolean addRowSelection(int start_row, int end_row);

/**
 * Adds a range of cells to the selection.
 * @return true if the model supports multi-range selection.
 */
public boolean addSelection(int start_row, int start_column,
                            int end_row, int end_column);

/**
 * Adds a JCCellRange to the selection.
 * @return true if the model supports multi-range selection.
 */
public boolean addSelection(JCCellRange selection);

/**
 * Removes duplicate ranges from the selection.
 */
public void cleanup();

/**
 * Clears all selected cells.
 */
public void clearSelection();

/**
 * Extends the current selection to the given row and column.
 * @return true if the model supports range or multi-range selection.
 */
public boolean extendSelection(int row, int column);

/**
 * Returns a Collection containing all of the selected regions as JCCellRange
 * objects.
 */
public Collection getSelectedCells();

/**
 * Returns true if the given row/column is selected.
 */
public boolean isSelected(int row, int column);

/**
 * Selects all cells.
 */
public boolean selectAll();

/**
 * Sets a range of columns as the current selection.
 * @return true if the model supports column selection.
 */
public boolean setColumnSelection(int start_column, int end_column);

/**
 * Sets a range of rows as the current selection.
 * @return true if the model supports row selection.
 */
public boolean setRowSelection(int start_row, int end_row);

/**
 * Sets a range of cells as the current selection.
 * @return true if the model supports range or multi-range selection.
 */
public boolean setSelection(int start_row, int start_column,
                            int end_row, int end_column);

/**
 * Sets the selection to a Collection of JCCellRange objects.
 * @return true if the model supports the selection.
 */
public boolean setSelection(Collection selected_cells);

/**
 * Sets the selection to a JCCellRange.
 * @return true if the model supports the selection.
 */
public boolean setSelection(JCCellRange selection);

/**
 * Adds, extends, deselected or ends a selection based on a row or column. This
 * method is called by the mouse and key process handler and should not be
 * used for programmatic manipulation of selected regions.
 *
 * @return true if the model supports the selection.
 */
public boolean select(int state, int row, int column);

}
