/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: SeriesModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import java.util.Collection;
import java.util.List;

/**
 * <CODE>SeriesModel</CODE> defines the interface required by classes that can
 * store  row/column based properties (for example, editors, renderers, styles).
 * Moving series storage to an interface definition allows better control over
 * sparse or dense property setting.
 */
public interface SeriesModel extends Moveable {

/**
 * Gets the default member value. The default value is always equivalent to the
 * (ALL, ALL) cell position.
 */
public Object getDefault();
						 
/**
 * Creates or sets the default member to the specified value. The default value
 * is always equivalent to the (ALL, ALL) cell position.
 */
public void setDefault(Object value);

/**
 * Returns the value at the specified cell, or within the cell's row or column.
 * @return null if the series is empty.
 */
public Object getValue(int row, int column);

/**
 * Sets the specified cell to the value.
 * @return true if the series was changed after adding the value.
 */
public boolean setValue(int row, int column, Object value);

/**
 * Sets a range of cells to the value.
 * @return true if the series was changed after adding the value.
 */
public boolean setValue(int start_row, int start_column,
                        int end_row, int end_column, Object value);

/**
 * Sets the cell range to the value.
 * @return true if the series was changed after adding the value.
 */
public boolean setValue(JCCellRange cr, Object value);

/**
 * Returns a Collection of unique objects used in JCTable.
 * @return a Collection of unique instances of objects in the series.
 */
public Collection getUniqueValues();

/**
 * Returns a Collection of JCCellRanges that contain the given object.
 * @return a Collection of <CODE>JCCellRanges</CODE> objects that as
 * set to the given value.
 */
public Collection getCellRanges(Object value);

/**
 * Returns an iterator for an ordered set of series values.
 */
public List getSeriesValues();

}
