/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Sort.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import java.util.Arrays;
import java.util.Comparator;

/**
 * Static methods to process column sorting.
 */
public class Sort {

public static final int ASCENDING = 0;
public static final int DESCENDING = 1;

/**
 * Sorts a table by column and direction.
 */
public static boolean sortByColumn(JCTable table, int column,
                                   int direction, Comparator comparator)
{
    return sortByColumn(table, column, direction, table.getFrozenRows(),
                        table.getNumRows() - 1, comparator);
}

/**
 * Sorts a table by column and direction for a range of rows.
 */
public static boolean sortByColumn(JCTable table, int column, int direction,
                                   int start_row, int end_row,
                                   Comparator comparator)
{
    int columns[] = {column};
    int directions[] = {direction};
    return sortByColumn(table, columns, directions,
                        start_row, end_row, comparator);
}

/**
 * Sorts a table by multiple columns.
 */
public static boolean sortByColumn(JCTable table, int columns[],
                                   int directions[], Comparator comparator)
{
    return sortByColumn(table, columns, directions, table.getFrozenRows(),
                        table.getNumRows() - 1, comparator);
}

/**
 * Sorts a table by multiple columns for a range of rows.
 */
public static boolean sortByColumn(JCTable table, int columns[],
                                   int directions[], int start_row, int end_row,
                                   Comparator comparator)
{
    if (comparator == null) {
        comparator = new TableDataComparator();
    }

    if (columns.length == 1) {
        table.sort_column = columns[0];
        table.sort_direction = directions[0];
    }
    else {
        table.sort_column = JCTableEnum.NOVALUE;
        table.sort_direction = JCTableEnum.NOVALUE;
    }

    DataViewModel dataView = table.getDataView();
    if (!(dataView instanceof SortableDataViewModel)) {
        throw new JCTableException(dataView + " does not support the " +
                                   "SortableDataViewModel interface");
    }
    SortableDataViewModel sortableDataView = (SortableDataViewModel)dataView;

    int rows = table.getNumRows();

    if (start_row < 0) {
        start_row = 0;
    }
    if (end_row >= rows) {
        end_row = rows - 1;
    }


    ObjectComparator c = new ObjectComparator(sortableDataView.getDataSource(),
                                              columns, directions, comparator);
    Integer[] o = new Integer[rows];
    for (int i = 0; i < rows; i++) {
        o[i] = new Integer(sortableDataView.getDataRow(i));
    }
    synchronized (o) {
        Arrays.sort(o, start_row, end_row + 1, c);
        int idx[] = new int[rows];
        for (int r = 0; r < rows; r++) {
            idx[r] = o[r].intValue();
        }
        sortableDataView.setRowMap(idx);  // replaces the current array
    }

    if (table.sortListeners != null) {
        JCSortEvent ev = new JCSortEvent(table, columns,
                                         sortableDataView.getRowMap());
        table.fireJCSortEvent(ev);
    }

    return(true);
}

} // Sort


class ObjectComparator implements Comparator {

protected TableDataModel  tableData;
protected int[]           columns;
protected int[]           directions;
protected Comparator      comparator;

ObjectComparator(TableDataModel tableData, int[] columns, int[] directions,
                 Comparator comparator)
{
    this.tableData = tableData;
    this.columns = columns;
    this.directions = directions;
    this.comparator = comparator;
}

/**
 * A method that compares the provided object to the current object
 * @param o1 the first object to be compared.
 * @param o2 the second object to be compared.
 * @return 1 if o1 > o2, 0 if o1 == o2, -1 if o1 < o2
 */
public int compare(Object o1, Object o2)
{
    int result = 0;
    for (int level = 0; level < columns.length && result == 0; level++) {
        int column = columns[level];
        Object item1 = tableData.getTableDataItem(((Integer)o1).intValue(),
                                                  column);
        Object item2 = tableData.getTableDataItem(((Integer)o2).intValue(),
                                                  column);
        result = comparator.compare(item1, item2);
        if (directions[level] == Sort.DESCENDING) {
            result = -result;
        }
    }
    return(result);
}

} // ObjectComparator


