/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: SortableDataViewModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

/**
 * The <CODE>SortableDataViewModel</CODE> adds methods to sort, drag and swap rows
 * and columns in the view.
 */
public interface SortableDataViewModel extends DataViewModel {

/**
 * Returns an array of column numbers that map view to data. For example,
 * columnMap[1] = 3 indicates that column 1 is column 3 in the data source.
 */
public int[] getColumnMap();
	
/**
 * Returns an array of row numbers that map view to data. For example,
 * rowMap[1] = 3 indicates that row 1 is row 3 in the data source.
 */
public int[] getRowMap();
		
/**
 * Returns the data column mapped to the view column.
 */
public int getDataColumn(int column);

/**
 * Returns the data row mapped to the view row.
 */
public int getDataRow(int row);
	
/**
 * Returns the column in the view for the data column.
 */
public int getViewColumn(int column);
	
/**
 * Returns the row in the view for the data row.
 */
public int getViewRow(int row);

/**
 * Reset the mapped columns array to a one-to-one mapping.
 */
public void resetColumnMap();

/**
 * Reset the mapped rows array to a one-to-one mapping.
 */
public void resetRowMap();

/**
 * Swaps two columns. Returns true if the columns are valid and the swap occurs.
 */
public boolean swapColumns(int column1, int column2);

/**
 * Swaps two rows. Returns true if the rows are valid and the swap occurs.
 */
public boolean swapRows(int row1, int row2);

/**
 * Sets a new column map array.
 */
public boolean setColumnMap(int[] new_map);
	
/**
 * Sets a new row map array.
 */
public boolean setRowMap(int[] new_map);
	
/**
 * Drags a column to a new position. Use JCTableEnum.MAXINT for the
 * destination if the first column should be at the end.
 */
public boolean dragColumn(int column1, int column2);
	
/**
 * Drags a row to a new position. Use JCTableEnum.MAXINT for the
 * destination if the first row should be at the end.
 */
public boolean dragRow(int row1, int row2);

}
