/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TableAction.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import java.awt.AWTEvent;
import java.io.Serializable;

/**
 * <CODE>TableAction</CODE> is responsible for defining a 
 * mapping from a user event to an action performed on table.
 * For example, <CODE>TableAction</CODE> can be used to map
 * a <STRONG>Shift+LeftClick</STRONG> to a sort event.
 */
public class TableAction implements Serializable {

protected ActionInitiator          initiator;
protected int                      action;

/** Action constant for column selection. */
public static final int COLUMN_SELECT_ACTION = JCTableEnum.COLUMN_SELECT_ACTION;
/** Action constant for column sort. */
public static final int COLUMN_SORT_ACTION   = JCTableEnum.COLUMN_SORT_ACTION;
/** Action constant for column drag. */
public static final int COLUMN_DRAG_ACTION   = JCTableEnum.COLUMN_DRAG_ACTION;
/** Action constant for row selection. */
public static final int ROW_SELECT_ACTION    = JCTableEnum.ROW_SELECT_ACTION;
/** Action constant for row drag. */
public static final int ROW_DRAG_ACTION      = JCTableEnum.ROW_DRAG_ACTION;
/** Action constant for copy range. */
public static final int COPY_ACTION          = JCTableEnum.COPY_ACTION;
/** Action constant for paste range. */
public static final int PASTE_ACTION         = JCTableEnum.PASTE_ACTION;

/**
 * A public constructor that creates a mapping from a series of
 * modifiers to a action.
 */
public TableAction(ActionInitiator initiator, int action)
{
    this.initiator = initiator;
    setAction(action);
}

/**
 * Gets the <CODE>ActionInitiator</CODE> for this
 * <CODE>TableAction</CODE>.
 * @see #setInitiator
 */
public ActionInitiator getInitiator()
{
    return initiator;
}

/**
 * Sets the <CODE>ActionInitiator</CODE> for this
 * <CODE>TableAction</CODE>.
 * @see KeyActionInitiator
 * @see MouseActionInitiator
 */
public void setInitiator(ActionInitiator initiator)
{
    this.initiator = initiator;
}

/**
 * Gets the value of the <CODE>Action</CODE> property for this
 * <CODE>TableAction</CODE> instance.
 * The <CODE>Action</CODE> property determines what action will be
 * performed when the specified AWT event occurs.
 */
public int getAction()
{
    return(action);
}

/**
 * Sets the value of the action property for this <CODE>TableAction</CODE>
 * instance. The <CODE>Action</CODE> property determines what action will
 * be performed when the specified AWT event occurs.
 */
public void setAction(int action)
{
    this.action = action;
}

/**
 * Returns true if the event has a matching action set for the
 * given row and column.
 */
public boolean isMatch(AWTEvent event, int row, int column)
{
    boolean match = initiator.isMatch(event);
    // Return false if there is no match
    if (!match) {
        return(false);
    }

    switch (action) {
    case COLUMN_SELECT_ACTION:
    case COLUMN_SORT_ACTION:
    case COLUMN_DRAG_ACTION:
        return(row == JCTableEnum.LABEL);
    case ROW_SELECT_ACTION:
    case ROW_DRAG_ACTION:
        return(column == JCTableEnum.LABEL);
    default:
        return(true);
    }
}

}
