/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TableDataComparator.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import java.util.Calendar;
import java.util.Comparator;

/**
 * Defines a class that performs comparisons based on object
 * type for objects that support the Comparatable interface.
 * Any other object is sorted based on it's toString() value.
 */
public class TableDataComparator implements Comparator {

/** Indicates that the first object is less than the second object. */
public static final int LESS_THAN = -1;
/** Indicates that the first object is equal to the second object. */
public static final int EQUAL = 0;
/** Indicates that the first objet is greater than the second object. */
public static final int GREATER_THAN = 1;
	
/**
 * A method that compares object a to object b
 * @param a The first object
 * @param b The second object
 * @return 1 if a > b, 0 if a == b, -1 if a < b
 */
public int compare(Object a, Object b)
{
    if (a == b) {
        return(EQUAL);
    }
    else if (a == null) {
        return(LESS_THAN);
    }
    else if (b == null) {
        return(GREATER_THAN);
    }

	// If both objects are of the same class, they can be compared,
	// otherwise, the toString values are compared
	if (a.getClass() == b.getClass()) {
		// If both implement the Comparable interface, it can be
		// used to compare the objects
		if (a instanceof Comparable && b instanceof Comparable) {
			return(((Comparable)a).compareTo(b));
		}
		// If both are Calendars, use a special comparison
		else if (a instanceof Calendar && b instanceof Calendar) {
			Calendar ac = (Calendar)a;
			Calendar bc = (Calendar)b;
			if (ac.equals(bc)) {
				return(EQUAL);
			}
			else if (ac.before(bc)) {
				return(LESS_THAN);
			}
			else {
				return(GREATER_THAN);
			}
		}
		// Otherwise, compare the string values
		else {
			return(a.toString().compareTo(b.toString()));
		}
	}
	// Objects are different classes so compare their string values
	else {
		return(a.toString().compareTo(b.toString()));
	}
}

}
