/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TableDataView.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import java.io.Serializable;

/**
 * The <CODE>TableDataView</CODE> serves an an intermediary between the
 * <CODE>Table</CODE> and the data source (<CODE>TableData</CODE>).
 */
public class TableDataView extends    BaseDataView
			               implements SortableDataViewModel, Serializable {

protected TableDataModel     dataSource;
protected boolean            isEditable = false;

private int[]                column_map;
private int[]                row_map;
private boolean              needs_column_map = false;
private boolean              needs_row_map = false;

/**
 * Creates a TableDataView instance that initializes the editor and renderer
 * tables but does not link it to table.
 */
public TableDataView()
{
	super();
}

/**
 * Creates a TableDataView instance and initializes the editor and renderer
 * tables
 * @param table The table controlled by this DataView.
 */
public TableDataView(JCTable table)
{
	super(table);
}

/**
 * Method to change the data based on the passed in event
 * @param e The event used to change the data
 */
public void dataChanged(JCTableDataEvent e)
{
	int flag = 0, pwidth, pheight;

	int view_row = e.getRow();
	if (view_row >= 0) {
		view_row = getViewRow(view_row);
	}

	int view_column = e.getColumn();
	if (view_column >= 0) {
		view_column = getViewColumn(view_column);
	}

	EditTraverseHandler editTraverseHandler = table.getEditTraverseHandler();
	switch(e.getCommand()) {
	case JCTableDataEvent.CHANGE_VALUE:
		flag = 0;
		pwidth = table.getPixelWidth(view_column);
		if (pwidth == JCTableEnum.VARIABLE ||
		    pwidth == JCTableEnum.VARIABLE_ESTIMATE)
		{
			flag |= TableChangeHandler.CELL_RECALC_WIDTH;
		}
		pheight = table.getPixelHeight(view_row);
		if (pheight == JCTableEnum.VARIABLE ||
		    pheight == JCTableEnum.VARIABLE_ESTIMATE)
		{
			flag |= TableChangeHandler.CELL_RECALC_HEIGHT;
		}
		if (flag != 0) {
			table.requestChange(flag, view_row, view_column);
		}
		repaintRange(view_row, view_column, view_row, view_column);
		break;
	case JCTableDataEvent.CHANGE_ROW:
		if (editTraverseHandler.getCellEditor() != null &&
		    view_row == table.current_row)
		{
			table.cancelEdit(true);
		}
		repaintRange(view_row, JCTableEnum.LABEL, view_row, getNumColumns());
		break;
	case JCTableDataEvent.ADD_ROW:
		if (editTraverseHandler.getCellEditor() != null) {
			table.cancelEdit(true);
		}
		DataViewSupport.addRow(table, view_row, 1,
		                       getNumRows(), getNumColumns());
		repaintRange(e.getRow(), JCTableEnum.LABEL,
		             getNumRows(), getNumColumns());
		break;
	case JCTableDataEvent.REMOVE_ROW:
		if (editTraverseHandler.getCellEditor() != null) {
			table.cancelEdit(true);
		}
		if (needs_row_map) {
			for (int r = 0; r < e.getNumAffected(); r++) {
				int vr = getViewRow(e.getRow() + r);
				row_map = removeMapElement(row_map, vr);
				DataViewSupport.deleteRow(table, vr, 1, getNumRows(),
                                          getNumColumns());
			}
		}
		else {
			DataViewSupport.deleteRow(table, e.getRow(), e.getNumAffected(),
									  getNumRows(), getNumColumns());
		}
		table.doLayout();
		break;
	case JCTableDataEvent.CHANGE_COLUMN:
		if (editTraverseHandler.getCellEditor() != null &&
		    view_column == table.current_column)
		{
			table.cancelEdit(true);
		}
		repaintRange(JCTableEnum.LABEL, view_column,
		             getNumRows(), e.getColumn());
		break;
	case JCTableDataEvent.ADD_COLUMN:
		if (editTraverseHandler.getCellEditor() != null) {
			table.cancelEdit(true);
		}
		DataViewSupport.addColumn(table, view_column, 1,
								  getNumRows(), getNumColumns());
		repaintRange(JCTableEnum.LABEL, view_column,
		             getNumRows(), getNumColumns());
		break;
	case JCTableDataEvent.REMOVE_COLUMN:
		if (editTraverseHandler.getCellEditor() != null) {
			table.cancelEdit(true);
		}
		if (needs_column_map) {
			for (int c = 0; c < e.getNumAffected(); c++) {
				int vc = getViewColumn(e.getColumn() + c);
				column_map = removeMapElement(column_map, vc);
				DataViewSupport.deleteColumn(table, vc, 1,
											 getNumRows(), getNumColumns());
			}
		}
		else {
			DataViewSupport.deleteColumn(table, e.getColumn(),
                                         e.getNumAffected(),
										 getNumRows(), getNumColumns());
		}
		table.doLayout();
		break;
	case JCTableDataEvent.CHANGE_ROW_LABEL:
		flag = 0;
		pwidth = table.getPixelWidth(JCTableEnum.LABEL);
		if (pwidth == JCTableEnum.VARIABLE ||
		    pwidth == JCTableEnum.VARIABLE_ESTIMATE)
		{
			flag |= TableChangeHandler.CELL_RECALC_WIDTH;
		}
		pheight = table.getPixelHeight(view_row);
		if (pheight == JCTableEnum.VARIABLE ||
		    pheight == JCTableEnum.VARIABLE_ESTIMATE)
		{
			flag |= TableChangeHandler.CELL_RECALC_HEIGHT;
		}
		if (flag != 0) {
			table.requestChange(flag, view_row, JCTableEnum.LABEL);
		}
		repaintRange(view_row, JCTableEnum.LABEL, view_row, JCTableEnum.LABEL);
		break;
	case JCTableDataEvent.CHANGE_COLUMN_LABEL:
		flag = 0;
		pwidth = table.getPixelWidth(view_column);
		if (pwidth == JCTableEnum.VARIABLE ||
		    pwidth == JCTableEnum.VARIABLE_ESTIMATE)
		{
			flag |= TableChangeHandler.CELL_RECALC_WIDTH;
		}
		pheight = table.getPixelHeight(JCTableEnum.LABEL);
		if (pheight == JCTableEnum.VARIABLE ||
		    pheight == JCTableEnum.VARIABLE_ESTIMATE)
		{
			flag |= TableChangeHandler.CELL_RECALC_HEIGHT;
		}
		if (flag != 0) {
			table.requestChange(flag, JCTableEnum.LABEL, view_column);
		}
		repaintRange(JCTableEnum.LABEL,view_column,
		             JCTableEnum.LABEL, view_column);
		break;
	case JCTableDataEvent.MOVE_ROW:
		if (editTraverseHandler.getCellEditor() != null) {
			table.cancelEdit(true);
		}
		resetRowMap();
		DataViewSupport.moveRows(table, e.getRow(), e.getNumAffected(),
		                         e.getDestination());
		table.doLayout();
		table.repaint();
		break;
	case JCTableDataEvent.MOVE_COLUMN:
		if (editTraverseHandler.getCellEditor() != null) {
			table.cancelEdit(true);
		}
		resetColumnMap();
		DataViewSupport.moveColumns(table, e.getColumn(), e.getNumAffected(),
		                            e.getDestination());
		table.doLayout();
		table.repaint();
		break;
	case JCTableDataEvent.NUM_ROWS:
		int rows = dataSource.getNumRows();
		// Cancel outstanding edit if outside of the number of new rows
		if (editTraverseHandler.getCellEditor() != null &&
		    rows <= table.current_row)
		{
			table.cancelEdit(true);
		}
		// Make sure the current row isn't invalid
		if (table.current_row >= rows) {
			table.setCurrentCell(rows - 1, table.current_column,
			                     JCTableEnum.TRAVERSE_UP);
		}
		resetRowMap();
		table.requestChange(TableChangeHandler.TABLE_RESET);
		break;
	case JCTableDataEvent.NUM_COLUMNS:
		int columns = dataSource.getNumColumns();
		// Cancel outstanding edit if outside of the number of new columns
		if (editTraverseHandler.getCellEditor() != null &&
		    columns <= table.current_column)
		{
			table.cancelEdit(true);
		}
		// Make sure the current column isn't invalid
		if (table.current_column >= columns) {
			table.setCurrentCell(table.current_row, columns - 1,
			                     JCTableEnum.TRAVERSE_LEFT);
		}
		resetColumnMap();
		table.requestChange(TableChangeHandler.TABLE_RESET);
		break;
	case JCTableDataEvent.RESET:
		table.requestChange(TableChangeHandler.TABLE_RESET);
		break;
	}
}

/**
 * Returns the bound data source.
 * @return The current bound data source
 */
public TableDataModel getDataSource()
{
	return(dataSource);
}

/**
 * Sets the bound data source.
 * @param tableData The new bound data source
 */
public void setDataSource(TableDataModel tableData)
{
	boolean dataSourceReset = false;

	if (tableData == null) {
		return;
	}

	table.setRepaintEnabled(false);

	if (dataSource != null) {
	    dataSourceReset = true;
		dataSource.removeTableDataListener(this);
	}
	// Cancel any outstanding edits
	EditTraverseHandler editTraverseHandler = table.getEditTraverseHandler();
	if (editTraverseHandler != null) {
		editTraverseHandler.cancel(true);
	}

	dataSource = tableData;
	dataSource.addTableDataListener(this);

	if (dataSource instanceof EditableTableDataModel) {
		isEditable = true;
	}
	else {
		isEditable = false;
	}

	if (dataSourceReset) {
	    // Invalidate the current cell and reset the origin
	    ;
	}

	// Force move to the first traversable cell
	if (dataSourceReset) {
		if (editTraverseHandler != null) {
			editTraverseHandler.traverseInitial();
		}
	}

	resetColumnMap();
	resetRowMap();

	table.setRecalcRequired(true);
	table.setRepaintEnabled(true);
}

/**
 * Returns true if the data source implements EditableTableData.
 * @return Is the data source editable?
 */
public boolean isEditable()
{
	return(isEditable);
}

/**
 * Returns the object given a row or column. This convenience method handles
 * both cells and labels.
 * @param row The row index
 * @param column The column index
 * @return The data object based on the given row and column
 */
public Object getObject(int row, int column)
{
	if (dataSource == null) {
		return(null);
	}

	if (row >= 0 && column >= 0) {
		return(dataSource.getTableDataItem(getDataRow(row),
		                                   getDataColumn(column)));
	}
	else if (row == JCTableEnum.LABEL) {
		return(dataSource.getTableColumnLabel(getDataColumn(column)));
	}
	else {
		return(dataSource.getTableRowLabel(getDataRow(row)));
	}
}

/**
 * Sets the cell value for editable data sources.
 * @param o The new cell value
 * @param row The row index
 * @param column The column index
 * @return The status of the edit.  Note that if the datasource is not
 *    editable, no edit is done and true is returned.
 */
public boolean setTableDataItem(Object o, int row, int column)
{
	if (dataSource == null) {
		return(false);
	}

	if (isEditable) {
		EditableTableDataModel edm = (EditableTableDataModel)dataSource;
		return(edm.setTableDataItem(o, getDataRow(row), getDataColumn(column)));
	}
	return(true);
}

/**
 * Returns the value for the cell at the specified location.<P>
 * @param row The row index
 * @param column The column index
 * @return The data object based on the given row and column
 */
public Object getTableDataItem(int row, int column)
{
	if (dataSource == null) {
		return(null);
	}
	return(dataSource.getTableDataItem(getDataRow(row), getDataColumn(column)));
}

/**
 * Returns the number of rows of data the data source contains.
 * @return The number of rows in the datasource
 */
public int getNumRows()
{
	if (dataSource == null) {
		return(0);
	}
	return(dataSource.getNumRows());
}

/**
 * Returns the number of columns of data the data source contains.
 * @return The number of columns in the datasource
 */
public int getNumColumns()
{
	if (dataSource == null) {
		return(0);
	}
	return(dataSource.getNumColumns());
}

/**
 * Returns the value for the row label at the specified row.<P>
 * @param row The row index
 * @return The label value for the given row
 */
public Object getTableRowLabel(int row)
{
	if (dataSource == null) {
		return(null);
	}
	return(dataSource.getTableRowLabel(getDataRow(row)));
}

/**
 * Returns the value for the column label at the specified column.<P>
 * @param column The column index
 * @return The label value for the given column
 */
public Object getTableColumnLabel(int column)
{
	if (dataSource == null) {
		return(null);
	}
	return(dataSource.getTableColumnLabel(getDataColumn(column)));
}

/**
 * Adds a <CODE>JCTableDataListener</CODE> to this object.
 * @param l The table data listenere to be added
 */
public void addTableDataListener(JCTableDataListener l)
{
	if (dataSource == null) {
		return;
	}
	dataSource.addTableDataListener(l);
}

/**
 * Removes a <CODE>JCTableDataListener</CODE> from this object
 * @param l The table data listenere to be removed
 */
public void removeTableDataListener(JCTableDataListener l)
{
	if (dataSource == null) {
		return;
	}
	dataSource.removeTableDataListener(l);
}

/**
 * Returns an array of column numbers that map view to data. For example,
 * columnMap[1] = 3 indicates that column 1 is column 3 in the data source.
 * @return A array of integers mapping column numbers to where they reside
 *    in the data source
 */
public int[] getColumnMap()
{
	int columns = dataSource.getNumColumns();
	if (column_map == null || column_map.length < columns) {
		resizeColumnMap();
	}
	return(column_map);
}

/**
 * Returns an array of row numbers that map view to data. For example,
 * rowMap[1] = 3 indicates that row 1 is row 3 in the data source.
 * @return A array of integers mapping row numbers to where they reside
 *    in the data source
 */
public int[] getRowMap()
{
	int rows = dataSource.getNumRows();
	if (row_map == null || row_map.length < rows) {
		resizeRowMap();
	}
	return(row_map);
}

/**
 * Returns the data column mapped to the view column.
 * @param column The column number in the view
 * @return The column where the view column resides in the datasource
 */
public int getDataColumn(int column)
{
	// Only lookup the column if the mapped columns array is needed
	if (column_map == null) {
		return(column);
	}

	int columns = dataSource.getNumColumns();
	if (column_map.length < columns) {
		resizeColumnMap();
	}
	if (column >= 0 && column < columns) {
		return(column_map[column]);
	}
	else {
		return(JCTableEnum.NOVALUE);
	}
}

/**
 * Returns the data row mapped to the view row.
 * @param row The row number in the view
 * @return The row where the view row resides in the datasource
 */
public int getDataRow(int row)
{
	// Only lookup the row if the mapped rows array is needed
	if (row_map == null) {
		return(row);
	}

	int rows = dataSource.getNumRows();
	if (row_map.length < rows) {
		resizeRowMap();
	}
	if (row >= 0 && row < rows) {
		return(row_map[row]);
	}
	else {
		return(JCTableEnum.NOVALUE);
	}
}

/**
 * Returns the column in the view for the data column.
 * @param column The column number in the datasource
 * @return The view column corresponding to the datasource column
 */
public int getViewColumn(int column)
{
	// Only lookup the column if the mapped columns array is needed
	if (column_map == null) {
		return(column);
	}

	int columns = dataSource.getNumColumns();
	if (column_map == null || column_map.length < columns) {
		resizeColumnMap();
	}
	if (column < columns) {
		if (column_map[column] == column) {
			return(column);
		}

		for (int i = 0; i < columns; i++) {
			if (column_map[i] == column) {
				return(i);
			}
		}
	}
	return(JCTableEnum.NOVALUE);
}

/**
 * Returns the row in the view for the data row.
 * @param row The row number in the datasource
 * @return The view row corresponding to the datasource row
 */
public int getViewRow(int row)
{
	// Only lookup the row if the mapped rows array is needed
	if (row_map == null) {
		return(row);
	}

	int rows = dataSource.getNumRows();
	if (row_map == null || row_map.length < rows) {
		resizeRowMap();
	}
	if (row < rows) {
		if (row_map[row] == row) {
			return(row);
		}

		for (int i = 0; i < rows; i++) {
			if (row_map[i] == row) {
				return(i);
			}
		}
	}
	return(JCTableEnum.NOVALUE);
}

/**
 * Initializes the mapped columns array any time it is accessed and
 * the number of columns in the datasource does not match the number
 * of columns in the map array.
 */
protected void resizeColumnMap()
{
	// Don't do anything if the mapped columns array isn't needed
	if (!needs_column_map) {
		return;
	}

	int columns = dataSource.getNumColumns();

	int[] temp = new int[columns];
	for (int i = 0; i < columns; i++) {
		if (column_map == null || i >= column_map.length) {
			temp[i] = i;
		}
		else {
			temp[i] = column_map[i];
		}
	}
	column_map = temp;
}

/**
 * Initializes the mapped rows array any time it is accessed and
 * the number of rows in the datasource does not match the number
 * of rows in the map array.
 */
protected void resizeRowMap()
{
	// Don't do anything if the mapped rows array isn't needed
	if (!needs_row_map) {
		return;
	}

	int rows = dataSource.getNumRows();

	int[] temp = new int[rows];
	for (int i = 0; i < rows; i++) {
		if (row_map == null || i >= row_map.length) {
			temp[i] = i;
		}
		else {
			temp[i] = row_map[i];
		}
	}
	row_map = temp;
}

/**
 * Reset the mapped columns array to a one-to-one mapping.
 */
public void resetColumnMap()
{
	needs_column_map = false;

	if (column_map != null) {
		int columns = column_map.length;
		int[] temp = new int[columns];
		for (int i = 0; i < columns; i++) {
			temp[i] = i;
		}
		setColumnMap(temp);
	}
	column_map = null;

	table.sort_column = JCTableEnum.NOVALUE;
	table.sort_direction = JCTableEnum.NOVALUE;
}

/**
 * Reset the mapped rows array to a one-to-one mapping.
 */
public void resetRowMap()
{
	needs_row_map = false;

	if (row_map != null) {
		int rows = row_map.length;
		int[] temp = new int[rows];
		for (int i = 0; i < rows; i++) {
			temp[i] = i;
		}
		setRowMap(temp);
	}
	row_map = null;

	table.sort_column    = JCTableEnum.NOVALUE;
	table.sort_direction = JCTableEnum.NOVALUE;
}

/**
 * Repositions a row or column to a new location in the mapped array.
 * @param array The input array of row or column indices
 * @param item1 The current position of the row/column to be moved
 * @param item2 The destination of the row/column
 * @return The new shifted array
 */
protected int[] shiftMappedArray(int[] array, int item1, int item2)
{
	int size = array.length;
	int[] temp = new int[size];

	// Handle case where destination is the last item
	if (item2 == JCTableEnum.MAXINT) {
		for (int i = 0; i < item1; i++) {
			temp[i] = array[i];
		}
		for (int i = item1 + 1; i < size; i++) {
			temp[i - 1] = array[i];
		}
		temp[size - 1] = array[item1];
	}
	// Handle case where item1 is less than item2
	else if (item1 < item2) {
		for (int i = 0; i < size; i++) {
			temp[i] = array[i];
		}
		for (int i = item1 + 1; i < item2; i++) {
			temp[i - 1] = array[i];
		}
		temp[item2 - 1] = array[item1];
	}
	// Handle case where item1 is greater than item2
	else if (item1 > item2) {
		for (int i = 0; i < size; i++) {
			temp[i] = array[i];
		}
		for (int i = item2; i < item1; i++) {
			temp[i + 1] = array[i];
		}
		temp[item2] = array[item1];
	}
	return(temp);
}

/**
 * Removes an element from the map array.
 * @param array The input array of row or column indices
 * @param element The element to be removed
 * @return The new array with the element removed
 */
protected int[] removeMapElement(int[] array, int element)
{
	if (array == null) {
		return null;
	}

	int size = array.length;
	int[] temp = new int[size - 1];
	for (int i = 0; i < size; i++) {
		if (i < element) {
			temp[i] = array[i];
		}
		else if (i > element) {
			temp[i - 1] = array[i];
		}
	}
	return(temp);
}

/**
 * Swaps two columns. Returns true if the columns are valid and the swap occurs.
 * @param column1 The view index of the first column to be swapped
 * @param column2 The view index of the second column to be swapped
 * @return Whether swap succeeded.
 */
public boolean swapColumns(int column1, int column2)
{
	int columns = dataSource.getNumColumns();

	if (column1 < 0 || column1 > columns) {
		return(false);
	}
	if (column2 < 0 || column2 > columns) {
		return(false);
	}
	if (column1 == column2) {
		return(true);
	}

	// Cancel edit if in progress
	table.cancelEdit(true);

	needs_column_map = true;

	if (column_map == null || column_map.length < columns) {
		resizeColumnMap();
	}

	int c1 = column_map[column1];
	column_map[column1] = column_map[column2];
	column_map[column2] = c1;

	DataViewSupport.swapColumns(table, column1, column2);
	table.doLayout();
	table.repaint();

	return(true);
}

/**
 * Swaps two rows. Returns true if the rows are valid and the swap occurs.
 * @param row1 The view index of the first row to be swapped
 * @param row2 The view index of the second row to be swapped
 * @return Whether swap succeeded.
 */
public boolean swapRows(int row1, int row2)
{
	int rows = dataSource.getNumRows();

	if (row1 < 0 || row1 > rows) {
		return(false);
	}
	if (row2 < 0 || row2 > rows) {
		return(false);
	}
	if (row1 == row2) {
		return(true);
	}

	// cancel edit if in progress
	table.cancelEdit(true);

	needs_row_map = true;

	if (row_map == null || row_map.length < rows) {
		resizeRowMap();
	}

	int r1 = row_map[row1];
	row_map[row1] = row_map[row2];
	row_map[row2] = r1;

	DataViewSupport.swapRows(table, row1, row2);
	table.doLayout();
	table.repaint();

	return(true);
}

/**
 * Sets a new column map array.
 * @param new_map Integer array where initial unsorted new_map resides
 * @return If process succeeded
 */
public boolean setColumnMap(int[] new_map)
{
	int columns = dataSource.getNumColumns();

	if (new_map.length != columns) {
		return(false);
	}

	// Cancel edit if in progress
	table.cancelEdit(true);

	needs_column_map = true;

	if (column_map == null || column_map.length < columns) {
		resizeColumnMap();
	}

	if (table.isSeriesDataSorted()) {
		if (DataViewSupport.swapColumnArrays(table, column_map, new_map)) {
			column_map = new_map;
		}
	}
	else {
		column_map = new_map;
	}
	table.repaint();

	return(true);
}

/**
 * Sets a new row map array.
 * @param new_map Integer array where initial unsorted new_map resides
 * @return If process succeeded
 */
public boolean setRowMap(int[] new_map)
{
	int rows = dataSource.getNumRows();

	if (new_map.length != rows) {
		return(false);
	}

	// Cancel edit if in progress
	table.cancelEdit(true);

	needs_row_map = true;

	if (row_map == null || row_map.length < rows) {
		resizeRowMap();
	}

	if (table.isSeriesDataSorted()) {
		if (DataViewSupport.swapRowArrays(table, row_map, new_map)) {
			row_map = new_map;
		}
	}
	else {
		row_map = new_map;
	}
	table.repaint();

	return(true);
}

/**
 * Drags a column to a new position. Use JCTableEnum.MAXINT for the
 * destination if the first column should be at the end.
 * @param column1 Current index of column to be dragged
 * @param column2 Index where dragged column is to be placed
 * @return Whether drag succeeded
 */
public boolean dragColumn(int column1, int column2)
{
	int columns = dataSource.getNumColumns();
	int frozenColumns = table.getFrozenColumns();

	if (column1 < 0 || column1 > columns) {
		return(false);
	}
	if (column2 < 0) {
		return(false);
	}
	if (column1 < frozenColumns || column2 < frozenColumns) {
		return(false);
	}

	if (column1 == column2) {
		return(true);
	}

	needs_column_map = true;

	if (column_map == null || column_map.length < columns) {
		resizeColumnMap();
	}

	// Recalc the mapped array
	column_map = shiftMappedArray(column_map, column1, column2);

	// Move the series
	boolean status = DataViewSupport.dragColumn(table, column1, column2);
	table.doLayout();
	table.repaint();

	return(status);
}

/**
 * Drags a row to a new position. Use JCTableEnum.MAXINT for the
 * destination if the first row should be at the end.
 * @param row1 Current index of row to be dragged
 * @param row2 Index where dragged row is to be placed
 * @return Whether drag succeeded
 */
public boolean dragRow(int row1, int row2)
{
	int rows = dataSource.getNumRows();
	int frozenRows = table.getFrozenRows();

	if (row1 < 0 || row1 > rows) {
		return(false);
	}
	if (row2 < 0) {
		return(false);
	}
	if (row1 < frozenRows || row2 < frozenRows) {
		return(false);
	}
	if (row1 == row2) {
		return(true);
	}

	needs_row_map = true;

	if (row_map == null || row_map.length < rows) {
		resizeRowMap();
	}

	// Recalc the mapped array
	row_map = shiftMappedArray(row_map, row1, row2);

	// Move the series
	boolean status = DataViewSupport.dragRow(table, row1, row2);
	table.doLayout();
	table.repaint();

	return(status);
}

/**
 * Issues a range repaint.  Defers to the appropriate cell area renderer.
 * @param start_row The starting row of the range to be repainted
 * @param start_column The starting column of the range to be repainted
 * @param end_row The ending row of the range to be repainted
 * @param end_column The ending column of the range to be repainted
 */
protected void repaintRange(int start_row, int start_column,
                            int end_row, int end_column)
{
	CellAreaRenderer car = table.getCellAreaHandler().getCellAreaRenderer();
	car.repaintRange(start_row, start_column, end_row, end_column);
}

/**
 * Ensures that TableDataView can be disposed.
 */
public void dispose()
{
	if (dataSource != null) {
		dataSource.removeTableDataListener(this);
	}
}

}
