/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TableFocusTraversalPolicy.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import com.klg.jclass.util.JCUtil;

import java.awt.*;

/**
 * <CODE>TableFocusTraversalPolicy</CODE>
 */
public class TableFocusTraversalPolicy extends FocusTraversalPolicy {

protected JCTable     table = null;
protected Component   component = null;

TableFocusTraversalPolicy(JCTable table, Component component)
{
	super();
	this.table     = table;
	this.component = component;
}

/**
 * Returns the Component that should receive the focus after aComponent.
 * focusCycleRoot must be a focus cycle root of aComponent.
 *
 * @param focusCycleRoot a focus cycle root of aComponent
 * @param aComponent a (possibly indirect) child of focusCycleRoot, or
 *        focusCycleRoot itself
 * @return the Component that should receive the focus after aComponent, or
 *         null if no suitable Component can be found
 * @throws IllegalArgumentException if focusCycleRoot is not a focus cycle
 *         root of aComponent, or if either focusCycleRoot or aComponent is
 *         null
 */
public Component getComponentAfter(Container focusCycleRoot,
                                   Component aComponent)
{
	if (focusCycleRoot == null) {
		throw new IllegalArgumentException("Null focusCycleRoot");
	}
	else if (aComponent == null) {
		throw new IllegalArgumentException("Null aComponent");
	}
	else if (!aComponent.isFocusCycleRoot(focusCycleRoot)) {
		throw new
		    IllegalArgumentException("focusCyleRoot:" +
		                             JCUtil.getComponentName(focusCycleRoot) +
			                         " is not a focus cycle root of aComponent:" +
			                         JCUtil.getComponentName(aComponent));
	}
	if (aComponent == table && component != null) {
		return(component);
	}
	return(table);
}

/**
 * Returns the Component that should receive the focus before aComponent.
 * focusCycleRoot must be a focus cycle root of aComponent.
 *
 * @param focusCycleRoot a focus cycle root of aComponent
 * @param aComponent a (possibly indirect) child of focusCycleRoot, or
 *        focusCycleRoot itself
 * @return the Component that should receive the focus before aComponent,
 *         or null if no suitable Component can be found
 * @throws IllegalArgumentException if focusCycleRoot is not a focus cycle
 *         root of aComponent, or if either focusCycleRoot or aComponent is
 *         null
 */
public Component getComponentBefore(Container focusCycleRoot,
                                    Component aComponent)
{
	if (focusCycleRoot == null) {
		throw new IllegalArgumentException("Null focusCycleRoot");
	}
	else if (aComponent == null) {
		throw new IllegalArgumentException("Null aComponent");
	}
	else if (!aComponent.isFocusCycleRoot(focusCycleRoot)) {
		throw new
		    IllegalArgumentException("focusCyleRoot:" +
		                             JCUtil.getComponentName(focusCycleRoot) +
			                         " is not a focus cycle root of aComponent:" +
			                         JCUtil.getComponentName(aComponent));
	}
	if (aComponent == table && component != null) {
		return(component);
	}
	return(table);
}

/**
 * Returns the first Component in the traversal cycle. This method is used
 * to determine the next Component to focus when traversal wraps in the
 * forward direction.
 *
 * @param focusCycleRoot the focus cycle root whose first Component is to
 *        be returned
 * @return the first Component in the traversal cycle when focusCycleRoot
 *         is the focus cycle root, or null if no suitable Component can be
 *         found
 * @throws IllegalArgumentException if focusCycleRoot is null
 */
public Component getFirstComponent(Container focusCycleRoot)
{
	if (focusCycleRoot == null) {
		throw new IllegalArgumentException("Null focusCycleRoot");
	}
	return(table);
}

/**
 * Returns the last Component in the traversal cycle. This method is used
 * to determine the next Component to focus when traversal wraps in the
 * reverse direction.
 *
 * @param focusCycleRoot the focus cycle root whose last Component is to be
 *         returned
 * @return the last Component in the traversal cycle when focusCycleRoot is
 *         the focus cycle root, or null if no suitable Component can be
 *         found
 * @throws IllegalArgumentException if focusCycleRoot is null
 */
public Component getLastComponent(Container focusCycleRoot)
{
	if (focusCycleRoot == null) {
		throw new IllegalArgumentException("Null focusCycleRoot");
	}
	return((component != null) ? component : table);
}

/**
 * Returns the default Component to focus. This Component will be the first
 * to receive focus when traversing down into a new focus traversal cycle
 * rooted at focusCycleRoot.
 *
 * @param focusCycleRoot the focus cycle root whose default Component is to
 *        be returned
 * @return the default Component in the traversal cycle when focusCycleRoot
 *         is the focus cycle root, or null if no suitable Component can
 *         be found
 * @throws IllegalArgumentException if focusCycleRoot is null
 */
public Component getDefaultComponent(Container focusCycleRoot)
{
	if (focusCycleRoot == null) {
		throw new IllegalArgumentException("Null focusCycleRoot");
	}
	return(table);
}

}
