/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: TableListenerPropagator.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:34 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import com.klg.jclass.table.data.AbstractDataSource;
import com.klg.jclass.util.formulae.MathScalar;
import com.klg.jclass.util.formulae.Operation;
import com.klg.jclass.util.formulae.TableExpressionList;
import com.klg.jclass.util.formulae.TableReference;

/**
 * Wrap a formula and listen for changes to table cells which
 * are operands for this formula. Propagate the change, if any,
 * so others up the food chain can re-evaluate themselves.
 */
public class TableListenerPropagator implements JCTableDataListener {

protected Operation  operation;
protected int        row, column;

/**
 * Wrap a formula and listen for changes to table cells which
 * are operands for this formula. Propagate the change, if any,
 * so others up the food chain can re-evaluate themselves.
 */
public TableListenerPropagator(Operation operation, int row, int column)
{
    this.operation = operation;
    this.row = row;
    this.column = column;

}

/**
 * Invoked when the data source is changed.
 */
public void dataChanged(JCTableDataEvent event)
{
    // See if the change affects any of the cells we depend upon
    int row1 = event.getRow();
    int column1 = event.getColumn();

    // If we are affected, then fire our own change event
    // to let others (who may depend upon us) know
    if (event.getSource() != this && dependant(row1, column1)) {
        AbstractDataSource dataModel = (AbstractDataSource)event.getSource();
        JCTableDataEvent evt =
            new JCTableDataEvent(dataModel, row, column, 0, 0,
                                 JCTableDataEvent.CHANGE_VALUE);
        dataModel.fireTableDataEvent(evt);
    }
}

/**
 * Determine if the change occurred in any of the cells from which
 * we derive our value.
 */
public boolean dependant(int row1, int column1)
{
    int x, y, w, h;

    // If we are a table expression list, see if we are bounded by its rectangle
    if (operation.leftOperand != null &&
        operation.leftOperand instanceof TableExpressionList)
    {
        TableExpressionList l = (TableExpressionList)operation.leftOperand;
        x = ((MathScalar)l.getStartRow().evaluate()).numberValue().intValue();
        y = ((MathScalar)l.getStartColumn().evaluate()).numberValue().intValue();
        w = ((MathScalar)l.getEndColumn().evaluate()).numberValue().intValue();
        h = ((MathScalar)l.getEndRow().evaluate()).numberValue().intValue();

        if (row1 >= x && row1 <= h && column1 >= y && column1 <= w) {
            return(true);
        }
    }

    // If we are a TableReference, see if we equal it
    if (operation.leftOperand != null &&
        operation.leftOperand instanceof TableReference)
    {
        TableReference ref = ((TableReference)operation.leftOperand);
        x = ((MathScalar)ref.getRow().evaluate()).numberValue().intValue();
        y = ((MathScalar)ref.getColumn().evaluate()).numberValue().intValue();

        if (row1 == x && column1 == y) {
            return(true);
        }
    }

    // If we are a table expression list, see if we are bounded by its rectangle
    if (operation.rightOperand != null &&
        operation.rightOperand instanceof TableExpressionList)
    {
        TableExpressionList r = (TableExpressionList)operation.rightOperand;
        x = ((MathScalar)r.getStartRow().evaluate()).numberValue().intValue();
        y = ((MathScalar)r.getStartColumn().evaluate()).numberValue().intValue();
        w = ((MathScalar)r.getEndColumn().evaluate()).numberValue().intValue();
        h = ((MathScalar)r.getEndRow().evaluate()).numberValue().intValue();

        if (row1 >= x && row1 <= h && column1 >= y && column1 <= w) {
            return(true);
        }
    }

    // If we are a TableReference, see if we equal it
    if (operation.rightOperand != null &&
        operation.rightOperand instanceof TableReference)
    {
        TableReference ref = ((TableReference)operation.rightOperand);
        x = ((MathScalar)ref.getRow().evaluate()).numberValue().intValue();
        y = ((MathScalar)ref.getColumn().evaluate()).numberValue().intValue();

        if (row1 == x && column1 == y) {
            return (true);
        }
    }

    // Nope, don't care
    return(false);
}

}
