/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TableMouseWheelListener.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table;

import java.awt.event.MouseWheelListener;
import java.awt.event.MouseWheelEvent;
import java.io.Serializable;
import javax.swing.JScrollBar;

/**
 * <CODE>TableMouseWheelListener</CODE> responds to
 * <CODE>MouseWheelEvents</CODE> and passes adjust the scrollbars appropriately.
 */
public class TableMouseWheelListener implements MouseWheelListener,
                                                Serializable
{

protected JCTable table;

/**
 * Note the table code to add a mouse wheel listener expects exactly one
 * constructor.  If you need to add a constructor, you must change the
 * corresponding code to deal with this.
 * @param table The table to which the listener is added
 */
public TableMouseWheelListener(JCTable table)
{
	setTable(table);
}

/**
 * Set the table to which this listener will change in response to
 * MouseWheelEvents
 * @param table The new table
 */
public void setTable(JCTable table)
{
	this.table = table;
}

/**
 * Get the table to which this listener will change in response to
 * MouseWheelEvents
 * @return The current table
 */
public JCTable getTable()
{
	return(table);
}

/**
 * Respond to mouse wheel being moved
 * @param e The event generated by the mouse wheel movement
 */
public void mouseWheelMoved(MouseWheelEvent e)
{
	if (table == null || e == null) {
		return;
	}

    if (e.getScrollAmount() != 0) {
        JScrollBar toScroll = table.getVertSB();
        int direction = 0;
        // Find which scrollbar to scroll, or return if none
        if (toScroll == null || !toScroll.isVisible()) {
            toScroll = table.getHorizSB();
            if (toScroll == null || !toScroll.isVisible()) {
                return;
            }
        }
        direction = (e.getWheelRotation() < 0) ? -1 : 1;
        if (e.getScrollType() == MouseWheelEvent.WHEEL_UNIT_SCROLL) {
            scrollByUnits(toScroll, direction, e.getScrollAmount());
        }
        else if (e.getScrollType() == MouseWheelEvent.WHEEL_BLOCK_SCROLL) {
            scrollByBlock(toScroll, direction);
		}
    }
}

/**
 * Scroll by block
 * @param scrollbar The scroll bar to change
 * @param direction The direction of change
 */
protected void scrollByBlock(JScrollBar scrollbar, int direction)
{
    int oldValue = scrollbar.getValue();
	int blockIncrement = scrollbar.getBlockIncrement(direction);
	int delta = blockIncrement * ((direction > 0) ? +1 : -1);
	scrollbar.setValue(oldValue + delta);
}

/**
 * Scroll by units
 * @param scrollbar The scroll bar to change
 * @param direction The direction of change
 * @param units The number of units to scroll
 */
protected void scrollByUnits(JScrollBar scrollbar, int direction, int units)
{
    int delta = units;

	if (direction > 0) {
	    delta *= scrollbar.getUnitIncrement(direction);
	}
	else {
	    delta *= -scrollbar.getUnitIncrement(direction);
	}

	int oldValue = scrollbar.getValue();
	int newValue = oldValue + delta;

	// Check for overflow.
	if (delta > 0 && newValue < oldValue) {
        newValue = scrollbar.getMaximum();
	}
	else if (delta < 0 && newValue > oldValue) {
	    newValue = scrollbar.getMinimum();
	}
	scrollbar.setValue(newValue);
}

}
