/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: BasePopupMenu.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table.beans;

import java.awt.Menu;
import java.awt.MenuItem;
import java.awt.PopupMenu;
import java.awt.event.ActionListener;

/**
 * BasePopupMenu is the popup menu to initiate component specific
 * commands at runtime.
 */
public abstract class BasePopupMenu extends PopupMenu {

/**
 * BasePopupMenu constructor
 */
public BasePopupMenu(ActionListener listener) {
	super();

    String[] commands = getCommands();
	// setup the default popup menu list
	for (int i=0; i<commands.length; i++)
    	addItem(commands[i], listener);
}

/**
 * Returns component specific command set.
 */
protected abstract String[] getCommands();

/**
 * Adds new item to the menu.
 *
 * @param command menu item command string.
 * @param listener action listener for the menu items.
 */
public void addItem(String command, ActionListener listener) {
    MenuItem menuItem = new MenuItem(command);
    menuItem.setActionCommand(command);
    menuItem.addActionListener(listener);
    add(menuItem);
}

/**
 * Enable or disable a menu item in the popup menu.
 *
 * @param command The key string.
 * @param enable True if enable else false.
 *
 * @return True if menu item is found and is enabled or is disabled.
 */
public boolean enableMenuItem(String key, boolean enable) {
    return enableMenuItem(this, key, enable);
}

/**
 * Enable or disable a menu item in the popup menu.
 *
 * @param menu The menu level.
 * @param command The command string.
 * @param enable True if enable else false.
 *
 * @return True if menu item is found and is enabled or is disabled.
 */
public static boolean enableMenuItem(Menu menu, String command, boolean enable) {
	int count = menu.getItemCount();
	for (int i = 0; i < count; i++) {
		MenuItem menuItem = menu.getItem(i);
		String itemCommand = menuItem.getActionCommand();
		if (itemCommand.equals(command)) {
			menuItem.setEnabled(enable);
			return true;
		}
		if (menuItem instanceof Menu && enableMenuItem((Menu) menuItem, command, enable))
        	return true;
	}
	return false;
}

}
