/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CellSizeEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table.beans;

import com.klg.jclass.table.JCCellRange;
import com.klg.jclass.table.JCResizeCellEvent;
import com.klg.jclass.table.JCResizeCellListener;
import com.klg.jclass.table.JCTableEnum;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.EventObject;


/**
 * Editor for cellSize series.
 */
public class CellSizeEditor extends TablePropertyEditor
                            implements ActionListener, FocusListener,
                                       ItemListener, JCResizeCellListener
{

protected JComboBox choiceWidth;
protected JComboBox choiceHeight;
protected JTextField textWidth;
protected JTextField textHeight;

protected static final String AS_IS 	= "As is";
protected static final String VARIABLE 	= "Variable";

protected static EnumString[] EnumCellSize = {
	new EnumString(JCTableEnum.AS_IS, AS_IS),
	new EnumString(JCTableEnum.VARIABLE, VARIABLE),
};

/**
 * Returns the current property value as a String instance.
 */
public String getAsText()
{
	return(tableView.getCellSize().toString());
}

/**
 * Returns a fragment of Java code representing an initializer
 * for the current value.
 */
public String getJavaInitializationString()
{
    return(tableView.getCellSize().getJavaInitializationString());
}

/**
 * Returns the current value of the property.
 */
public Object getValue()
{
    return(tableView.getCellSize());
}

/**
 *	Sets a new property value.
 */
public void setValue(Object value)
{
    if (value == null) {
		return;
    }

	tableView.setCellSize((CellSizeWrapper)value);
	super.setValue(value);
}

//================== ActionListener interface method ==================

public void actionPerformed(ActionEvent e)
{
	if (!updateComponents) {
		return;
    }
	super.actionPerformed(e);
    textChanged(e);
}

//================== FocusListener interface methods ==================

public void focusGained(FocusEvent e) {}

public void focusLost(FocusEvent e)
{
	if (!updateComponents) {
		return;
    }
	super.focusLost(e);
    textChanged(e);
}

//================== JCResizeCellListener interface methods ==================

/**
 * Invoked before the row and/or column is resized.
 * We don't use it in this class.
 */
public void beforeResizeCell(JCResizeCellEvent e) {
}

/**
 * Invoked after the mouse is released.
 * The width or height is not updated.
 * We don't use it in this class.
 */
public void resizeCell(JCResizeCellEvent e) {}

/**
 * Invoked after the row and/or column is resized and all listeners have been
 * notified of the new value through <CODE>resizeCell</CODE> and the new values
 * has been set.
 */
public void afterResizeCell(JCResizeCellEvent e)
{
	if (!updateComponents) {
		return;
    }
	updatePropertyPanel(tableView.getSelectedRange());
}

//================== ItemListener interface methods ==================

public void itemStateChanged(ItemEvent e)
{
	if (!updateComponents) {
		return;
    }
	super.itemStateChanged(e);
    itemChanged(e);
}

//================== Class protected methods ==================

protected void init()
{
	super.init(false);
	tableView.setAllowCellResize(JCTableEnum.RESIZE_ALL);
	tableView.setAllowResizeBy(JCTableEnum.RESIZE_BY_CELLS);
	tableView.addResizeCellListener(this);
    updateCellSize(tableView.getSelectedRange());
}

protected JComponent makePropertyPanel()
{
  // Create color panel
	choiceWidth = new JComboBox();
	choiceHeight = new JComboBox();
	textWidth = new JTextField(3);
	textHeight = new JTextField(3);

	choiceWidth.addItem(VARIABLE);
	choiceWidth.addItem(AS_IS);
	choiceHeight.addItem(VARIABLE);
	choiceHeight.addItem(AS_IS);

	// Create property panel
	GroupPanel p = new GroupPanel("Cell size");
	p.setLayout(new FlowLayout(FlowLayout.LEFT, 10, 6));
	p.add(new JLabel("Width"));
	p.add(choiceWidth);
	p.add(textWidth);
	p.add(new JLabel("pixels  "));
	p.add(new JLabel("Height"));
	p.add(choiceHeight);
	p.add(textHeight);
	p.add(new JLabel("pixels  "));

	textWidth.addActionListener(this);
	textHeight.addActionListener(this);
	textWidth.addFocusListener(this);
	textHeight.addFocusListener(this);
	choiceWidth.addItemListener(this);
	choiceHeight.addItemListener(this);

	return(p);
}

protected void updatePropertyPanel(JCCellRange c)
{
	updateCellSize(c);
	support.firePropertyChange("cellSize", null, getValue());
}

private void updateCellSize(JCCellRange range)
{
    updateComponents = false;
	if (range == null) {
		textHeight.setEnabled(false);
		textWidth.setEnabled(false);
		choiceHeight.setEnabled(false);
		choiceWidth.setEnabled(false);
    }
    else {
		choiceHeight.setEnabled(true);
		choiceWidth.setEnabled(true);

    	setSelectedItem(choiceHeight,
        	            tableView.getPixelHeight(range.start_row) ==
	                        JCTableEnum.VARIABLE ? VARIABLE : AS_IS);
		setSelectedItem(choiceWidth,
        	            tableView.getPixelWidth(range.start_column) ==
            	            JCTableEnum.VARIABLE ? VARIABLE : AS_IS);
		enableText(textHeight, choiceHeight,
			       tableView.getRowPixelHeight(getFirstCell(range.start_row)));
		enableText(textWidth, choiceWidth,
			       tableView.getColumnPixelWidth(getFirstCell(range.start_column)));
	}
    updateComponents = true;
}

private void setPixelHeight(JCCellRange range, int height)
{
    if (range.start_row == JCTableEnum.ALL ||
        range.start_row == JCTableEnum.ALLCELLS)
    {
        tableView.setPixelHeight(range.start_row, height);
    }
    else {
        for (int i = range.start_row; i <= range.end_row; i++) {
            tableView.setPixelHeight(i, height);
        }
    }
}

private void setPixelWidth(JCCellRange range, int width)
{
   if (range.start_column == JCTableEnum.ALL ||
       range.start_column == JCTableEnum.ALLCELLS)
   {
        tableView.setPixelWidth(range.start_column, width);
    }
    else {
        for (int i=range.start_column; i<=range.end_column; i++) {
            tableView.setPixelWidth(i, width);
        }
    }
}

private boolean enableText(JTextField tf, JComboBox cb, int v)
{
	boolean enabled = (cb.getSelectedItem()).equals(AS_IS);
	tf.setEnabled(enabled);
	tf.setText(enabled ? v+"" : "");
    return(enabled);
}

private void itemChanged(ItemEvent e)
{
	JCCellRange range = tableView.getRangedSelection();
	if (range == null) {
		return;
    }
	Object o = e.getSource();
	if (o.equals(choiceHeight)) {
    	int height =
        	tableView.getRowPixelHeight(getFirstCell(range.start_row));
		boolean variable = !enableText(textHeight, choiceHeight, height);
        setPixelHeight(range, variable ? JCTableEnum.VARIABLE : height);
	}
	else if (o.equals(choiceWidth)) {
    	int width =
        	tableView.getColumnPixelWidth(getFirstCell(range.start_column));
		boolean variable = !enableText(textWidth, choiceWidth, width);
        setPixelWidth(range, variable ? JCTableEnum.VARIABLE : width);
    }
    else {
    	return;
	}
	support.firePropertyChange("cellSize", null, getValue());
}

private void textChanged(EventObject e)
{
	JCCellRange range = tableView.getRangedSelection();
	if (range == null) {
		return;
    }
	Object o = e.getSource();
	if (o.equals(textHeight)) {
        int height = getInt(textHeight);
        if (height < 0) {
        	return;
        }
        setPixelHeight(range, height);
	}
	else if (o.equals(textWidth)) {
        int width = getInt(textWidth);
        if (width < 0) {
        	return;
        }
        setPixelWidth(range, width);
    }
    else {
    	return;
	}
	support.firePropertyChange("cellSize", null, getValue());
}

private int getInt(JTextField field)
{
	try {
		return(Math.max(Math.min((Integer.valueOf(field.getText())).intValue(),
                                 1024),
                        0));
	}
	catch (NumberFormatException ex) {}
    errorMessageBox("Invalid dimension.");
    return(Integer.MIN_VALUE);
}

}

