/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ColorEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table.beans;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.TextEvent;
import java.awt.event.TextListener;

import java.beans.PropertyEditorSupport;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.text.Document;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import com.klg.jclass.util.swing.JCSwingTypeConverter;


/**
 * Custom property editor for java.awt.Color-type properties.
 * Allows selection of color from a list, or more detailed
 * color specification using a launchable color chooser.
 */
public class ColorEditor extends PropertyEditorSupport {

    static String color_names[] = {
	"Black", "Blue", "Cyan", "DarkGray", "Gray", "Green", "LightGray",
	"LightBlue", "Magenta", "Orange", "Pink", "Red", "Yellow", "White", "Custom"};


    ColorChooser chooser = null;

    public ColorEditor() {
        chooser = new ColorChooser(this);
    }

    public String[] getTags() { return color_names; }

    public void setAsText(String text) throws IllegalArgumentException {
        Color color = null;
        if (text.equals("Custom")) {
            chooser = new ColorChooser(this);
            chooser.pack();
            chooser.show();
        } else {
        	color = JCSwingTypeConverter.toColor(text);
        	if (color == null)
            	throw new IllegalArgumentException("invalid color: " + text);
        	setValue(color);
        }

    }

    protected boolean eqColor(Color col1, Color col2) {
        if ( (col1.getRed() == col2.getRed()) && (col1.getGreen() == col2.getGreen()) && (col1.getBlue() == col2.getBlue()) ) {
            return true;
        }
        return false;
    }

    public String getAsText() {
        Color color = getColor();
       	if (color == null) {
    		return "Black";
	    } else if (eqColor(color,Color.black)) {
		    return "Black";
    	} else if (eqColor(color,Color.blue)) {
	    	return "Blue";
    	} else if (eqColor(color,Color.cyan)) {
	    	return "Cyan";
    	} else if (eqColor(color,Color.darkGray)) {
	    	return "DarkGray";
    	} else if (eqColor(color,Color.gray)) {
	    	return "Gray";
    	} else if (eqColor(color,Color.green)) {
	    	return "Green";
    	} else if (eqColor(color,JCSwingTypeConverter.lightBlue)) {
	    	return "LightBlue";
    	} else if (eqColor(color,Color.lightGray)) {
	    	return "LightGray";
    	} else if (eqColor(color,Color.magenta)) {
    		return "Magenta";
	    } else if (eqColor(color,Color.orange)) {
		    return "Orange";
    	} else if (eqColor(color,Color.pink)) {
	    	return "Pink";
    	} else if (eqColor(color,Color.red)) {
	    	return "Red";
    	} else if (eqColor(color,Color.white)) {
	    	return "White";
    	} else if (eqColor(color,Color.yellow)) {
	    	return "Yellow";
    	} else {
	    	return "Custom";
    	}

    }

    public String getJavaInitializationString() {
	    if (getValue() == null) return "Color.black";
	    return "new Color(" + getColor().getRGB() + ")";
    }

    Color getColor() {
        return (Color) getValue();
    }

    void selectColor(Color color) {
    	if (color != null)
	    	setColor(color);

    }

    private void setColor(Color color) {
	    if (color != null) {
            setValue(color);
    		firePropertyChange();
	    }
    }



    public JFrame findParent() {
/*	    JComponent c = this;
    	while (c != null && !(c instanceof JFrame))
	    	c = c.getParent();
    	if (c == null) {
	    	if (dialogFrame == null)
		    	dialogFrame = new JFrame();
    		c = dialogFrame;
	    }
    	return (JFrame)c; */
    	return new JFrame();
    }

}

class ColorChooser extends JDialog implements ActionListener, DocumentListener {

    private ColorEditor editor;

    private JTextField red;
    private JTextField green;
    private JTextField blue;
    private ColorEditorSample sample;
    private JButton buttonOK;

    ColorChooser(ColorEditor editor) {
    	super(new Frame(), "Select Custom Color", true);

    	this.editor = editor;

    	Color color = editor.getColor();
	    if (color == null)
		    color = Color.black;

    	//Create left panel
	    red = new JTextField(""+color.getRed(), 4);
    	green = new JTextField(""+color.getGreen(), 4);
	    blue = new JTextField(""+color.getBlue(), 4);

    	JPanel p1 = new JPanel();
		p1.setBorder(BorderFactory.createTitledBorder("RGB color"));
	    p1.setLayout(new GridBagLayout());
    	GridBagConstraints c = new GridBagConstraints();
	    c.gridwidth = 2;
    	c.gridx = GridBagConstraints.RELATIVE;
	    c.gridy = 1;
    	c.anchor = GridBagConstraints.NORTHWEST;
	    c.insets = new Insets(4,4,4,4);
    	p1.add(new JLabel("Red"), c);
	    p1.add(red, c);
    	c.gridy++;
	    p1.add(new JLabel("Green"), c);
    	p1.add(green, c);
	    c.gridy++;
    	p1.add(new JLabel("Blue"), c);
	    p1.add(blue, c);

    	//Create right panel
	    sample = new ColorEditorSample(color);
    	JPanel p2 = new JPanel();
	    p2.setLayout(new FlowLayout(FlowLayout.RIGHT));
    	p2.add(sample);

    	//Create bottom panel
	    buttonOK = new JButton("OK");
    	JButton buttonCancel = new JButton("Cancel");
	    JPanel p3 = new JPanel();
    	p3.setLayout(new FlowLayout(FlowLayout.RIGHT));
	    p3.add(buttonOK);
    	p3.add(buttonCancel);

    	getContentPane().setLayout(new BorderLayout(10, 4));
		getContentPane().add(p1, "Center");
		getContentPane().add(p2, "East");
		getContentPane().add(p3, "South");

        red.getDocument().addDocumentListener(this);
        green.getDocument().addDocumentListener(this);
        blue.getDocument().addDocumentListener(this);

	    buttonCancel.addActionListener(this);
    	buttonOK.addActionListener(this);

	    //Initialize this dialog to its preferred size.
    	pack();
    }

    public void actionPerformed(ActionEvent event) {
	    Object source = event.getSource();
    	if (source == buttonOK)
	    	editor.selectColor(sample.getColor());

    	setVisible(false);
    }

    public void insertUpdate(DocumentEvent e) {
        sample.setColor(new Color(getColor(red), getColor(green), getColor(blue)));
    }

    public void removeUpdate(DocumentEvent e) {
        sample.setColor(new Color(getColor(red), getColor(green), getColor(blue)));
    }

    public void changedUpdate(DocumentEvent e) {
        sample.setColor(new Color(getColor(red), getColor(green), getColor(blue)));
    }

    private int getColor(JTextField field) {
	    try {
		    int i = (new Integer(field.getText())).intValue();
    		if (i < 0)
	    		i = 0;
		    if (i > 255)
			    i = 255;

    		return i;
	    }
    	catch (NumberFormatException ex) {
	    	return 0;
    	}
    }

}

// ********************************************

class ColorEditorSample extends JComponent {
	Color color;


public ColorEditorSample(Color color) {
	this.color = color != null ? color : Color.black;
}

public void setColor(Color color) {
	this.color = color != null ? color : Color.black;
	repaint();
}

public Color getColor() {
	return color;
}

public void paint(Graphics g) {
	Dimension d = getSize();

	g.setColor(color);
	g.fillRect(0, 0, d.width, d.height);
}

public Dimension getPreferredSize() {
	return new Dimension(120,120);
}

public Dimension getMinimumSize() {
	return new Dimension(100,100);
}


} // ColorSample


/*
class ColorDialog extends JDialog implements ActionListener, TextListener {

private AppearanceEditor editor;
private boolean foreground;

private JTextField red;
private JTextField green;
private JTextField blue;
private ColorSample sample;
private JButton buttonOK;

ColorDialog(AppearanceEditor editor, boolean foreground) {
	super(editor.findParent(), (foreground?"Foreground":"Background")+" Color", true);

	this.editor = editor;
	this.foreground = foreground;

	Color color = editor.getColor(foreground);
	if (color == null)
		color = Color.black;

	//Create left panel
	red = new JTextField(""+color.getRed(), 4);
	green = new JTextField(""+color.getGreen(), 4);
	blue = new JTextField(""+color.getBlue(), 4);

	GroupPanel p1 = new GroupPanel("RGB color");
	p1.setLayout(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();
	c.gridwidth = 2;
	c.gridx = GridBagConstraints.RELATIVE;
	c.gridy = 1;
	c.anchor = GridBagConstraints.NORTHWEST;
	c.insets = new Insets(4,4,4,4);
	p1.add(new JLabel("Red"), c);
	p1.add(red, c);
	c.gridy++;
	p1.add(new JLabel("Green"), c);
	p1.add(green, c);
	c.gridy++;
	p1.add(new JLabel("Blue"), c);
	p1.add(blue, c);

	//Create right panel
	sample = new ColorSample(color);
	JPanel p2 = new JPanel();
	p2.setLayout(new FlowLayout(FlowLayout.RIGHT));
	p2.add(sample);

	//Create bottom panel
	buttonOK = new JButton("OK");
	JButton buttonCancel = new JButton("Cancel");
	JPanel p3 = new JPanel();
	p3.setLayout(new FlowLayout(FlowLayout.RIGHT));
	p3.add(buttonOK);
	p3.add(buttonCancel);

	setLayout(new BorderLayout(10, 4));
	add("Center", p1);
	add("East", p2);
	add("South", p3);

	red.addTextListener(this);
	green.addTextListener(this);
	blue.addTextListener(this);

	buttonCancel.addActionListener(this);
	buttonOK.addActionListener(this);

	//Initialize this dialog to its preferred size.
	pack();
}

public void actionPerformed(ActionEvent event) {
	Object source = event.getSource();
	if (source == buttonOK)
		editor.selectColor(sample.getColor(), foreground);

	setVisible(false);
}

public void textValueChanged(TextEvent e) {
	sample.setColor(new Color(getColor(red), getColor(green), getColor(blue)));
}

private int getColor(JTextField field) {
	try {
		int i = (new Integer(field.getText())).intValue();
		if (i < 0)
			i = 0;
		if (i > 255)
			i = 255;

		return i;
	}
	catch (NumberFormatException ex) {
		return 0;
	}
}

} // ColorDialog
*/
