/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table.beans;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import java.io.File;
import java.util.EventObject;
import java.util.StringTokenizer;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JFileChooser;

import com.klg.jclass.table.EditableTableDataModel;
import com.klg.jclass.table.JCCellRange;
import com.klg.jclass.table.JCTableEnum;
import com.klg.jclass.table.JCTableCellInfo;
import com.klg.jclass.table.JCTableDataEvent;
import com.klg.jclass.table.JCTableDataListener;
import com.klg.jclass.table.TableDataModel;
import com.klg.jclass.table.data.JCBeanFileDataSource;
import com.klg.jclass.table.data.JCEditableVectorDataSource;





import java.util.Vector;


/**
 *	Sub-class this to create a series editor for series with enum strings
 *	and type in TCTblConvert.enum_strings.
 * @since JClass 4.0K for Java 2. If you are using JClass 4.0J,
 * which is based on JDK 1.1, some of the return types for this class'
 * fields or methods are based on the corresponding
 * <code>com.sun.java.util</code> class.
 */
public class DataEditor extends TablePropertyEditor implements ActionListener,
	ItemListener, FocusListener, JCTableDataListener {

private JRadioButton chbxTable;
private JRadioButton chbxFile;
private JTextField textFName;
private JButton buttonColumns;
private JButton buttonClear;
private JButton buttonRows;
private JButton buttonBrowse;

private Point textDialogLocation = null;
private JCEditableVectorDataSource tableSource = null;


/**
 *	Constructs a DataEditor editor.
 */
public DataEditor() {
	super(SELECT_NONE);
	tableSource = (JCEditableVectorDataSource)tableView.getDataSource();
	updatePixelWidth();
	repaint();
}

/**
 * Returns the current value of the property.
 */
public Object getValue() {
	tableView.commitEdit(true);
	return tableView.getData();
}

/**
 *	Sets a new property value.
 */
public void setValue(Object value) {
	if (value == null) {
		return;
    }
	if (tableView.getDataSource() != null) {
		tableView.getDataSource().removeTableDataListener(this);
    }
	tableView.setData((DataWrapper)value);

	TableDataModel dataSource = tableView.getDataSource();
	if (dataSource != null) {
		dataSource.addTableDataListener(this);
    }
	boolean file = dataSource instanceof JCBeanFileDataSource;
	if (file) {
		textFName.setText(((JCBeanFileDataSource)dataSource).getName());
    }
	setStateFile(file);
	super.setValue(value);
}

/**
 * Returns the current property value as a String instance.
 */
public String getAsText() {
	return tableView.getData().toString();
}

/**
 * Returns a fragment of Java code representing an initializer
 * for the current value.
 */
public String getJavaInitializationString() {
	return tableView.getData().getJavaInitializationString();
}

//================== ActionListener interface method ==================

public void actionPerformed(ActionEvent e) {
	super.actionPerformed(e);
    textChanged(e);
    buttonPressed(e);
}

//================== FocusListener interface methods ==================

public void focusGained(FocusEvent e) {}

public void focusLost(FocusEvent e) {
	super.focusLost(e);
    textChanged(e);
}

//================== JCTableDataListener interface methods ==================

public void dataChanged(JCTableDataEvent e) {
	updatePixelWidth();
    updateData();
}

//================== ItemListener interface methods ==================

public void itemStateChanged(ItemEvent e) {
	super.itemStateChanged(e);
    itemChanged(e);
}

//================== Class protected methods ==================

protected void init() {
	super.init(true);
	tableView.setSelectionPolicy(JCTableEnum.SELECT_NONE);
	tableView.setAllowCellResize(JCTableEnum.RESIZE_ALL);
}

protected JComponent makePropertyPanel() {
	chbxTable = new JRadioButton("table", true);
	chbxFile = new JRadioButton("file", false);
	ButtonGroup checkSource = new ButtonGroup();
    checkSource.add(chbxTable);
    checkSource.add(chbxFile);
	textFName = new JTextField();
	buttonClear = new JButton("Clear Cells");
	buttonColumns = new JButton("Column Labels...");
	buttonRows = new JButton("Row Labels...");
	buttonBrowse = new JButton("Browse...");

	// Create property panel
	GroupPanel p = new GroupPanel("Data source");
	p.setLayout(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();
	c.gridwidth = 1;
    c.fill = GridBagConstraints.HORIZONTAL;
	c.gridx = GridBagConstraints.RELATIVE;
	c.gridy = 1;
	c.anchor = GridBagConstraints.NORTHWEST;
	c.insets = new Insets(2,2,2,2);
	p.add(chbxTable, c);
	p.add(buttonClear, c);
	p.add(buttonColumns, c);
	p.add(buttonRows, c);
	c.gridy++;
	p.add(chbxFile, c);
	c.gridwidth = 3;
	p.add(textFName, c);
	c.gridwidth = 1;
	p.add(buttonBrowse, c);

	chbxTable.addItemListener(this);
	chbxFile.addItemListener(this);
	buttonClear.addActionListener(this);
	buttonColumns.addActionListener(this);
	buttonRows.addActionListener(this);
	buttonBrowse.addActionListener(this);
	textFName.addActionListener(this);
	textFName.addFocusListener(this);

	return p;
}

protected void updatePropertyPanel(JCCellRange c) {
//	support.firePropertyChange("dataSource", null, getValue());
}

protected void updateData() {
	support.firePropertyChange("dataSource", null, getValue());
}

private void setStateFile(boolean isFile) {
	if (isFile) {
    	setSelected(chbxFile, true);
		tableView.setPixelWidth(JCTableEnum.ALL, JCTableEnum.VARIABLE);
	}
	else {
    	setSelected(chbxTable, true);
		tableSource = (JCEditableVectorDataSource)tableView.getDataSource();
		updatePixelWidth();
	}
	setSelected(!isFile);
}

private void browseFile() {
	JFileChooser chooser = new JFileChooser();
	chooser.setDialogTitle("Load Data Source");
	chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
	if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
		File file = chooser.getSelectedFile();
		if (file != null) {
            textFName.setText(file.getAbsolutePath());
            showSource();
        }
    }
}

private void clearData() {
	tableView.setRepaintEnabled(false);
	tableSource.removeTableDataListener(this);
	for(int r = 0; r < tableSource.getNumRows(); r++) {
		for(int c = 0; c < tableSource.getNumColumns(); c++) {
			tableSource.setCell(r, c, "");
        }
    }
	tableSource.addTableDataListener(this);
	updatePixelWidth();
	tableView.setRepaintEnabled(true);
    
    updateData();
}

private void updatePixelWidth() {
	boolean isEmpty;
	for(int c = 0; c < tableSource.getNumColumns(); c++) {
		isEmpty = isEmpty(tableSource.getTableColumnLabel(c));
		if (isEmpty)
			for(int r = 0; r < tableSource.getNumRows(); r++) {
				Object o = tableSource.getTableDataItem(r, c);
				if (!isEmpty(o)) {
					isEmpty = false;
					break;
				}
			}
		if (isEmpty) {
			tableView.setPixelWidth(c, 70);
        }
		else {
			tableView.setPixelWidth(c, JCTableEnum.VARIABLE);
        }
	}
}

private boolean isEmpty(Object o) {
	return o == null || !(o instanceof String) || ((String)o).trim().length() == 0;
}

private void editLabels(boolean rows) {
	TextDialog textDialog = new TextDialog(//findParent(), 
	//fix for PR14838 - VisualAge is not providing  Parent.
		new JFrame(), (rows?"Row":"Column")+" Labels");
	JCEditableVectorDataSource dataSource = (JCEditableVectorDataSource)tableView.getDataSource();
	String s = rows ?
		toString(dataSource.getRowLabels(), "\n") :
        toString(dataSource.getColumnLabels(), "\n");

	textDialog.setText(s);
	textDialog.pack();
	if (textDialogLocation == null) {
        Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
        textDialogLocation = new Point(Math.max((screenDim.width - getSize().width)/2, 0),
        	Math.max((screenDim.height - getSize().height)/2, 0));
    }
    textDialog.setLocation(textDialogLocation);
	textDialog.show();
	textDialogLocation = textDialog.getLocation();
	if (textDialog.getResult()) {
		s = textDialog.getText();
		if (rows) {
			dataSource.setRowLabels(toVector(s, "\n"));
        }
		else {
			dataSource.setColumnLabels(toVector(s, "\n"));
        }
        updateData();
	}
	repaint();
}

private void setSelected(boolean table) {
	buttonColumns.setEnabled(table);
	buttonClear.setEnabled(table);
	buttonRows.setEnabled(table);
}

protected void showSource() {
	TableDataModel dataSource = tableView.getDataSource();
	String name = textFName.getText().trim();
	if(name.length() <1) {
	    return;
	}
    if (dataSource instanceof JCBeanFileDataSource &&
    		((JCBeanFileDataSource)dataSource).getName().equalsIgnoreCase(name)) {
    	return;
    }

	tableView.commitEdit(true);
	try {
		java.io.FileInputStream stream = new java.io.FileInputStream(name);
		JCBeanFileDataSource file = new JCBeanFileDataSource(name, stream);
/*
		TableDataModel source = tableView.getDataSource();
		if (source instanceof EditableTableDataModel) {
			tableSource = (JCEditableVectorDataSource)source;
        }
*/
		setDataSource(file);
		setStateFile(true);
		updateSizePanel();
		updateData();
		return;
	}
	catch (java.io.FileNotFoundException e) {
		if (name.length() == 0) {
			errorMessageBox("Please enter data file name first.");
        }
		else {
        	if (chbxFile.isSelected()) {
				errorMessageBox("Specified data file is not found.");
            }
        }
	}
	catch (java.io.IOException e) {
		errorMessageBox("Specified data file is not valid: "+e);
	}
	setStateFile(false);
}

protected void setDataSource(TableDataModel dataSource) {
    if (tableView.getDataSource() != null) {
        tableView.getDataSource().removeTableDataListener(this);
    }
    tableView.setDataSource(dataSource);
    tableSource.addTableDataListener(this);
}

private void itemChanged(ItemEvent e) {
	Object o = e.getSource();
	if (o.equals(chbxTable) || o.equals(chbxFile)) {
        tableView.commitEdit(true);
        if (chbxTable.isSelected()) {
            if (tableSource == null) {
                tableSource = new JCEditableVectorDataSource();
            }
            setDataSource(tableSource);

            setStateFile(false);
            updateSizePanel();
            updateData();
        }
        else {
            showSource();
        }
    }
}

private void buttonPressed(ActionEvent e) {
	Object o = e.getSource();
	if (o.equals(buttonBrowse)) {
		browseFile();
    }
	else if (o.equals(buttonClear)) {
		clearData();
    }
	else if (o.equals(buttonColumns) || o.equals(buttonRows)) {
		editLabels(o.equals(buttonRows));
    }
}

private void textChanged(EventObject e) {
	Object o = e.getSource();
	if (o.equals(textFName)) {
		showSource();
    }
}

/**
 * Converts a delimited list of tokens to a Vector.
 * @param s the string to convert
 * @param delimiter the delimiter that separates the tokens in the string
 */
public static Vector toVector(String s, String delimiter) {
    if (s == null) {
		return null;
    }
    StringTokenizer st = new StringTokenizer(s, delimiter, true);
    Vector v = new Vector();
    while (st.hasMoreTokens()) {
        String token = st.nextToken();
        if (token.equalsIgnoreCase(delimiter)) {
            token = "";
        }
        else if (st.hasMoreTokens()) {
            st.nextToken();
        }
		v.addElement(token);
    }
    return v;
}

public static String toString(Vector v, String delimiter) {
	if (v == null) {
    	return "";
    }
	StringBuffer buffer = new StringBuffer();
	for (int i = 0; i < v.size(); i++) {
        Object o = v.elementAt(i);
        if (o instanceof String) {
            buffer.append((String)o);
        }
        buffer.append(delimiter);
	}
    buffer.setLength(buffer.length()-1);
    return buffer.toString();
}


//======================== Inner classes ========================

/**
 * Dialog object used to edit row and column labels.
 */
class TextDialog extends JDialog implements ActionListener {

    /** JLabel that contains the text */
    private JTextArea text;

    /** OK and buttonCancel buttons to dismiss the dialog */
    private JButton buttonOK;
    private JButton buttonCancel;

    private boolean result = false;

    /**
     * The default constructor.  Creates the
     * visual objects inside the dialog - text area for the
     * text, and two buttons to dismiss the dialog.
     */
    TextDialog(Frame parent, String title) {
        super(parent, title, true);
        getContentPane().setLayout(new BorderLayout(0, 10));

        JPanel panelText = new JPanel();
        panelText.setLayout(new BorderLayout());
        text = new JTextArea("", 15, 50);
        panelText.add("Center", text);

        JPanel panelButton = new JPanel();
        panelButton.setLayout(new FlowLayout(FlowLayout.RIGHT, 8, 8));
        buttonOK = new JButton("OK");
        panelButton.add(buttonOK);
        buttonCancel = new JButton("Cancel");
        panelButton.add(buttonCancel);

        getContentPane().add("Center", panelText);
        getContentPane().add("South", panelButton);

        buttonCancel.addActionListener(this);
        buttonOK.addActionListener(this);

        //Initialize this dialog to its preferred size.
        setLocation();
        pack();
        repaint();
    }

    public void setText(String s) {
        text.setText(s);
    }

    public boolean getResult() {
        return result;
    }

    public String getText() {
        return text.getText();
    }

	//================== ActionListener interface method ==================

    public void actionPerformed(ActionEvent event) {
        Object o = event.getSource();
        if (o instanceof JButton) {
            result = o.equals(buttonOK);
            setVisible(false);
        }
    }

    private void setLocation() {
        Container parent = getParent();
        Point p = parent.getLocation();
        Dimension d = parent.getSize();
        setLocation(p.x+d.width/2, p.y+d.height/2);
    }

} // TextDialog

} // DataEditor


