/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DataWrapper.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:34 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table.beans;

import com.klg.jclass.table.TableDataModel;

/**
 * DataWrapper wraps up string representations of the table data.
 * It is used for the data property in LiveTable.
 */
public class DataWrapper implements java.io.Serializable {

public boolean isFile;

public String      fileName = null;

public int         rows = 0;
public int         columns = 0;
public String[]    rowLabels = null;
public String[]    columnLabels = null;
public String[][]  cellValues = null;

/**
 * Creates an DataWrapper for the given dimentions and the three arrays
 * representing the labels table data.
 * @param rows int number of rows in the table data model.
 * @param columns int number of columns in the table data model.
 * @param rowLabels String[] row labels.
 * @param columnLabels String[] column labels.
 * @param cellValues String[][] cell data.
 */
public DataWrapper(int rows, int columns, String[] rowLabels,
                   String[] columnLabels, String[][] cellValues)
{
    isFile = false;
    this.columns = columns;
    this.rows = rows;
    this.rowLabels = rowLabels;
    this.columnLabels = columnLabels;
    this.cellValues = cellValues;
}

/**
 * Creates an DataWrapper for the given dimentions and the three arrays
 * representing the labels table data.
 * @param fileName String data source file name.
 */
public DataWrapper(String fileName)
{
    isFile = true;
    this.fileName = fileName;
}

/**
 * Creates a DataWrapper given a general Table Data Model
 * @param model The Table Data Model
 */
public DataWrapper(TableDataModel model)
{
    isFile = false;
    if (model == null) {
        return;
    }
    rows = Math.max(model.getNumRows(), 0);
    columns = Math.max(model.getNumColumns(), 0);
    rowLabels = new String[rows];
    for (int i = 0; i < rows; i++ ) {
        Object o = model.getTableRowLabel(i);
        rowLabels[i] = (o == null) ? null : o.toString();
    }
    columnLabels = new String[columns];
    for (int i = 0; i < columns; i++) {
        Object o = model.getTableRowLabel(i);
        columnLabels[i] = (o == null) ? null : o.toString();
    }
    cellValues = new String[rows][columns];
    for (int i = 0; i < rows; i++) {
        for (int j = 0; j < columns; j++) {
            Object o = model.getTableDataItem(i, j);
            cellValues[i][j] = (o == null) ? null : o.toString();
        }
    }
}

/**
 * Returns a representation of this wrapper values as a string.
 * @return a string that must be used by a property editor.
 */
public String toString()
{
    return "...";
}

/**
 * Returns a fragment of Java code representing an initializer
 * for the current value.
 * @return a string that must be used by a property editor.
 */
public String getJavaInitializationString()
{
    return "new " + getClass().getName() + "(" +
        (isFile ? makeFileName() : (rows + "," + columns + ",\n\t\t" +
        "new String[] {" + makeArrayString(rowLabels) + "},\n\t\t" +
        "new String[] {" + makeArrayString(columnLabels) + "},\n\t\t" +
        "new String[][] {\n\t\t\t" +
        makeArrayString(cellValues) + "}")) + ")";
}

protected String makeFileName()
{
    String s = fileName;
    System.out.println(s);
    s = s.replace('\\', '/');
    System.out.println(s);
    return "\"" + s + "\"";
}

protected String makeArrayString(String[] stringArray)
{
    String s = "";
    for (int i = 0; i < stringArray.length; i++) {
        s += "\"" + stringArray[i] + "\"" +
            (i < stringArray.length - 1 ? "," : "");
    }
    return s;
}

protected String makeArrayString(String[][] stringArray)
{
    String s = "";
    for (int i = 0; i < stringArray.length; i++) {
        s += "{" + makeArrayString(stringArray[i]) + "}" +
            (i < stringArray.length - 1 ? ",\n\t\t\t" : "");
    }
    return s;
}

} // DataWrapper
