/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: FrozenCellLayoutEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:35 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table.beans;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import java.util.EventObject;

import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.klg.jclass.table.JCCellRange;
import com.klg.jclass.table.JCTableEnum;


/**
 * Editor for frozenCellLayout series.
 */
public class FrozenCellLayoutEditor extends TablePropertyEditor implements
	ActionListener, ItemListener {

protected JTextField textRow;
protected JTextField textColumn;
protected JComboBox columnPlacement;
protected JComboBox rowPlacement;

protected static final String LEFT 		= "Left";
protected static final String RIGHT 	= "Right";
protected static final String TOP 		= "Top";
protected static final String BOTTOM 	= "Bottom";

protected static EnumString[] EnumColPosition = {
	new EnumString(JCTableEnum.PLACE_LEFT, LEFT),
	new EnumString(JCTableEnum.PLACE_RIGHT, RIGHT),
};

protected static EnumString[] EnumRowPosition = {
	new EnumString(JCTableEnum.PLACE_TOP, TOP),
	new EnumString(JCTableEnum.PLACE_BOTTOM, BOTTOM),
};


/**
 *	Constructs a FrozenCellLayoutEditor editor.
 */
public FrozenCellLayoutEditor() {
	super(SELECT_NONE);
}

/**
 *	Constructs a FrozenCellLayoutEditor editor.
 */
public FrozenCellLayoutEditor(int selectionPolicy) {
	super(selectionPolicy);
}

/**
 * Sets a new property value.
 */
public void setValue(Object value) {
	if (value == null) {
		return;
    }
	tableView.setFrozenCellLayout((FrozenCellLayoutWrapper)value);
	super.setValue(value);
    updateFrozenCellLayout();
}

/**
 * Returns the current value of the property.
 */
public Object getValue() {
	return tableView.getFrozenCellLayout();
}

/**
 * Returns a fragment of Java code representing an initializer
 * for the current value.
 */
public String getJavaInitializationString() {
	return tableView.getFrozenCellLayout().getJavaInitializationString();
}

/**
 * Returns the current property value as a String instance.
 */
public String getAsText() {
	return getValue().toString();
}


//================== ActionListener interface method ==================

public void actionPerformed(ActionEvent e) {
	super.actionPerformed(e);
    textChanged(e);
}

//================== FocusListener interface methods ==================

public void focusGained(FocusEvent e) {}

public void focusLost(FocusEvent e) {
	super.focusLost(e);
    textChanged(e);
}


//================== ItemListener interface methods ==================

public void itemStateChanged(ItemEvent e) {
	super.itemStateChanged(e);
    itemChanged(e);
}

//================== Class protected methods ==================

protected void init() {
	super.init(false);
	tableView.setSelectionPolicy(JCTableEnum.SELECT_NONE);
}

protected JComponent makePropertyPanel() {
	GridBagConstraints c = new GridBagConstraints();
    c.fill = GridBagConstraints.HORIZONTAL;
	c.gridx = GridBagConstraints.RELATIVE;
	c.gridy = 0;
	c.anchor = GridBagConstraints.NORTHWEST;
	c.insets = new Insets(4,4,4,4);

	// Create frozen cells components
	textRow = new JTextField("0", 2);
	textColumn = new JTextField("0", 2);

	GroupPanel p1 = new GroupPanel("Frozen Cells");
	p1.setLayout(new GridBagLayout());
	c.gridy = 0;
	if (selectionPolicy != SHOW_COLUMNS) {
        p1.add(new JLabel("Rows"), c);
        p1.add(textRow, c);

        textRow.addActionListener(this);
        textRow.addFocusListener(this);
        c.gridy++;
    }
	p1.add(new JLabel("Columns"), c);
	p1.add(textColumn, c);

	// Create frozen cells placement components
	GroupPanel p2 = new GroupPanel("Frozen Cells Placement");
	p2.setLayout(new GridBagLayout());
	c.gridy = 0;
	if (selectionPolicy != SHOW_COLUMNS) {
		rowPlacement = new JComboBox();
		rowPlacement.addItem(TOP);
		rowPlacement.addItem(BOTTOM);
		rowPlacement.setEnabled(false);

        p2.add(new JLabel("Row"), c);
        p2.add(rowPlacement, c);

		rowPlacement.addItemListener(this);
        c.gridy++;
	}
	columnPlacement = new JComboBox();
	columnPlacement.addItem(LEFT);
	columnPlacement.addItem(RIGHT);
	columnPlacement.setEnabled(false);
    p2.add(new JLabel("Column"), c);
    p2.add(columnPlacement, c);

	// Create property panel
	JPanel p = new JPanel();
	p.setLayout(new GridBagLayout());
	c.anchor = GridBagConstraints.CENTER;
    c.fill = GridBagConstraints.BOTH;
	c.insets = new Insets(2,2,2,2);
	c.gridx = GridBagConstraints.RELATIVE;
    c.weightx = 1;
	p.add(p1, c);
	p.add(p2, c);

	textColumn.addActionListener(this);
	textColumn.addFocusListener(this);
	columnPlacement.addItemListener(this);

	return p;
}

protected void updatePropertyPanel(JCCellRange c) {
    updateFrozenCellLayout();
    support.firePropertyChange("frozenCellLayout", null, getValue());
}

protected void updateFrozenCellLayout() {
    int rows = tableView.getFrozenRows();
    int columns = tableView.getFrozenColumns();

    textRow.setText(rows+"");
    textColumn.setText(columns+"");

	columnPlacement.setEnabled(columns > 0);
	setSelectedItem(columnPlacement,
    	toString(EnumColPosition, tableView.getFrozenColumnPlacement()));
	if (selectionPolicy != SHOW_COLUMNS) {
		rowPlacement.setEnabled(rows > 0);
        setSelectedItem(rowPlacement,
            toString(EnumRowPosition, tableView.getFrozenRowPlacement()));
    }
}

private void textChanged(EventObject e) {
	Object o = e.getSource();
	if (o.equals(textRow)) {
	    int rows = getInt(textRow.getText());
        tableView.setFrozenRows(rows);
        if (selectionPolicy != SHOW_COLUMNS) {
            rowPlacement.setEnabled(rows > 0);
        }
    }
	else if (o.equals(textColumn)) {
        int columns = getInt(textColumn.getText());
        tableView.setFrozenColumns(columns);
        columnPlacement.setEnabled(columns > 0);
    }
    else {
    	return;
    }
    support.firePropertyChange("frozenCellLayout", null, getValue());
}

private void itemChanged(ItemEvent e) {
	Object o = e.getSource();
	if (o.equals(columnPlacement)) {
		tableView.setFrozenColumnPlacement(
        	toIntValue(EnumColPosition, (String)columnPlacement.getSelectedItem()));
	}
	if (selectionPolicy != SHOW_COLUMNS && o.equals(rowPlacement)) {
		String vert = (String)rowPlacement.getSelectedItem();
		tableView.setFrozenRowPlacement(
        	toIntValue(EnumRowPosition, (String)rowPlacement.getSelectedItem()));
	}
    else {
    	return;
    }
	support.firePropertyChange("frozenCellLayout", null, getValue());
}

}	// FrozenCellLayoutEditor


