/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCLiveCellStyle.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:35 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table.beans;

import java.awt.Color;
import java.awt.Font;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;
import javax.swing.border.SoftBevelBorder;

import com.klg.jclass.table.CellBorderModel;
import com.klg.jclass.table.JCCellStyle;
import com.klg.jclass.table.JCCellBorder;
import com.klg.jclass.table.JCTableEnum;
import com.klg.jclass.table.SwingCellBorder;
import com.klg.jclass.table.TableCellInfoModel;
import com.klg.jclass.cell.JCCellRenderer;
import com.klg.jclass.util.swing.JCSwingTypeConverter;


/**
 * The <CODE>JCLiveCellStyle</CODE> class provides a default implementation of
 * the <CODE>CellStyleModel</CODE> interface.
 */
public class JCLiveCellStyle extends JCCellStyle {

LiveTable table = null;

// Swing Borders
public static final int BEVEL_BORDER		= JCTableEnum.BORDER_THIN+1;
public static final int SOFT_BEVEL_BORDER   = JCTableEnum.BORDER_THIN+2;

public JCLiveCellStyle(LiveTable table) {
	super();
	this.table = table;
}

public JCLiveCellStyle(LiveTable table, JCCellRenderer renderer) {
	super();
	this.table = table;
    if (renderer != null) {
//	    this.renderer = renderer;
//		renderer_set = true;
        setCellRenderer(renderer);
    }
}

public JCLiveCellStyle(LiveTable table, JCLiveCellStyle style) {
	super(style);
	this.table = table;
}

public JCLiveCellStyle(
        String           foreground,
        String           background,
        String           font,
        boolean          editable,
        boolean          traversable,
        int              horizontal_alignment,
        int              vertical_alignment,
        int              border_type,
        int              border_sides,
        int              clip_hints,
        int              cell_border_color_mode,
        String           cell_border_color) {
    this.foreground = JCSwingTypeConverter.toColor(foreground);
    this.background = JCSwingTypeConverter.toColor(background);
    this.font = JCSwingTypeConverter.toFont(font);
    this.editable = editable;
    this.traversable = traversable;
    this.horizontal_alignment = horizontal_alignment;
    this.vertical_alignment = vertical_alignment;
    this.border_sides = border_sides;
    this.clip_hints = clip_hints;
    this.cell_border_color_mode = cell_border_color_mode;
    this.cell_border_color = JCSwingTypeConverter.toColor(cell_border_color);
	// do this last so that color mode has already been set
    setCellBorderType(border_type);
}

/**
 * Sets the live table component for this cell style.
 * @param t
 */
public void setTable(LiveTable t)
{
	this.table = t;
	// propogate table into border component if we can
	if (border instanceof SwingCellBorder) {
		((SwingCellBorder)border).setComponent(this.table);
	}
}

public Object clone() {
	JCLiveCellStyle style = new JCLiveCellStyle(table);
    copyInto(style);
	return style;
}

/**
 * Returns the <code>CellBorderModel</code> type for the cell.
 * @return <code>CellBorderModel</code> for the cell
 */
public int getCellBorderType() {
	CellBorderModel border = getCellBorder();
	if (border instanceof JCCellBorder) {
    	return ((JCCellBorder)border).getBorderType();
    }
	if (border instanceof SwingCellBorder) {
    	return getSwingCellBorderType(((SwingCellBorder)border).getCellBorder());
    }
    else {
    	return JCTableEnum.NOVALUE;
    }
}

/**
 * Sets the <code>CellBorder</code> object for the <code>borderType</code> value
 * on the cell.
 * @param borderType
 */
public void setCellBorderType(int borderType) {
	// for bevel/soft bevel, figure out appropriate border colors
	// this is largely a copy of some similar code in CellAreaRenderer
	Color borderHighlightColor = null;
	Color borderShadowColor = null;
	if (borderType == BEVEL_BORDER || borderType == SOFT_BEVEL_BORDER) {
    	int mode = cell_border_color_mode;
		Color baseBorderColor = cell_border_color;
	    if((baseBorderColor == null && 
				mode == JCTableEnum.USE_CELL_BORDER_COLOR) ||
                    mode == JCTableEnum.BASE_ON_BACKGROUND) {
            baseBorderColor = background;
        }
        else if(mode == JCTableEnum.BASE_ON_FOREGROUND) {
            baseBorderColor = foreground;
        }
		if (baseBorderColor != null) {
            if(baseBorderColor.equals(Color.white)) {
                borderHighlightColor = TableCellInfoModel.WHITE_BRIGHTER;
                borderShadowColor = TableCellInfoModel.WHITE_DARKER;
            }
            else if(baseBorderColor.equals(Color.black)) {
                borderHighlightColor = TableCellInfoModel.BLACK_BRIGHTER;
                borderShadowColor = TableCellInfoModel.BLACK_DARKER;
            }
            else {
                borderHighlightColor = baseBorderColor.brighter();
                borderShadowColor = baseBorderColor.darker();
            }
		}
	}

	if (borderType == BEVEL_BORDER) {
		if (borderHighlightColor != null && borderShadowColor != null) {
    		border = new SwingCellBorder(table, new BevelBorder(
				BevelBorder.RAISED, borderHighlightColor, borderShadowColor ));
		} else {
    		border = new SwingCellBorder(table, new BevelBorder( BevelBorder.RAISED));
		}
    }
	else if (borderType == SOFT_BEVEL_BORDER) {
		if (borderHighlightColor != null && borderShadowColor != null) {
    		border = new SwingCellBorder(table, new SoftBevelBorder(
				BevelBorder.RAISED, borderHighlightColor, borderShadowColor));
		} else {
    		border = new SwingCellBorder(table, new SoftBevelBorder(BevelBorder.RAISED));
		}
    }
    else {
	    border = new JCCellBorder(borderType);
    }
}

/**
 * Overrides the color setting to possibly redo the Bevel/Soft Bevel borders.
 * @param value
 */
public void setCellBorderColor(Color value)
{
	super.setCellBorderColor(value);
	int type = getCellBorderType();
	if (type == BEVEL_BORDER || type == SOFT_BEVEL_BORDER) {
		setCellBorderType(type);
	}
}

/**
 * Overrides the color mode setting to possibly redo the Bevel/Soft Bevel borders.
 * @param value
 */
public void setCellBorderColorMode(int value)
{
	super.setCellBorderColorMode(value);
	int type = getCellBorderType();
	if (type == BEVEL_BORDER || type == SOFT_BEVEL_BORDER) {
		setCellBorderType(type);
	}
}

/**
 * Overrides the foreground setting to possibly redo the Bevel/Soft Bevel borders.
 * @param value
 */
public void setForeground(Color value)
{
	super.setForeground(value);
	int type = getCellBorderType();
	if (type == BEVEL_BORDER || type == SOFT_BEVEL_BORDER) {
		setCellBorderType(type);
	}
}

/**
 * Overrides the background setting to possibly redo the Bevel/Soft Bevel borders.
 * @param value
 */
public void setBackground(Color value)
{
	super.setBackground(value);
	int type = getCellBorderType();
	if (type == BEVEL_BORDER || type == SOFT_BEVEL_BORDER) {
		setCellBorderType(type);
	}
}


protected int getSwingCellBorderType(Border border) {
	// check for SoftBevel first because SoftBevel is an instance of Bevel
	if (border instanceof SoftBevelBorder) {
    	return SOFT_BEVEL_BORDER;
    } else if (border instanceof BevelBorder) {
    	return BEVEL_BORDER;
    } else {
    	return JCTableEnum.NOVALUE;
    }
}

public boolean isEditableSet() {
    return editable_set;
}

public boolean isTraversableSet() {
    return traversable_set;
}

public boolean isHorizontalAlignmentSet() {
    return horizontal_alignment_set;
}

public boolean isVerticalAlignmentSet() {
    return vertical_alignment_set;
}

public boolean isCellBorderSidesSet() {
    return border_sides_set;
}

public boolean isCellBorderTypeSet() {
    return getCellBorder() != null;
}

public boolean isClipHintsSet() {
    return clip_hints_set;
}

public boolean isBackgroundSet() {
    return background != null;
}

public boolean isForegroundSet() {
    return foreground != null;
}

public boolean isFontSet() {
    return font != null;
}

public boolean isCellBorderColorModeSet() {
    return cell_border_color_mode_set;
}

public boolean isCellBorderColorSet() {
    return cell_border_color != null;
}

}


