/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: LabelLayoutEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:35 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table.beans;


import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.FlowLayout;
import java.awt.Insets;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import java.util.EventObject;

import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.klg.jclass.table.JCCellRange;
import com.klg.jclass.table.JCTableEnum;


/**
 * Editor for the table label layout.
 */
public class LabelLayoutEditor extends TablePropertyEditor implements
	ActionListener, FocusListener, ItemListener {

protected JCheckBox rowLabelDisplay;
protected JCheckBox columnLabelDisplay;
protected JTextField rowLabelOffset;
protected JTextField columnLabelOffset;
protected JComboBox columnLabelPlacement;
protected JComboBox rowLabelPlacement;

protected static final String LEFT 		= "Left";
protected static final String RIGHT 	= "Right";
protected static final String TOP 		= "Top";
protected static final String BOTTOM 	= "Bottom";

protected static EnumString[] EnumRowPosition = {
	new EnumString(JCTableEnum.PLACE_LEFT, LEFT),
	new EnumString(JCTableEnum.PLACE_RIGHT, RIGHT),
};

protected static EnumString[] EnumColPosition = {
	new EnumString(JCTableEnum.PLACE_TOP, TOP),
	new EnumString(JCTableEnum.PLACE_BOTTOM, BOTTOM),
};

/**
 *	Constructs a LabelLayoutEditor editor.
 */
public LabelLayoutEditor() {
	super(SELECT_NONE);
}

/**
 *	Sets a new property value.
 */
public void setValue(Object value) {
	if (value == null) {
		return;
    }
	tableView.setLabelLayout((LabelLayoutWrapper)value);
    updateLabelLayout();
	super.setValue(value);
}

/**
 * Returns the current value of the property.
 */
public Object getValue() {
	return tableView.getLabelLayout();
}

/**
 * Returns a fragment of Java code representing an initializer
 * for the current value.
 */
public String getJavaInitializationString() {
	return tableView.getLabelLayout().getJavaInitializationString();
}

/**
 * Returns the current property value as a String instance.
 */
public String getAsText() {
	return getValue().toString();
}

//================== ActionListener interface method ==================

public void actionPerformed(ActionEvent e) {
	super.actionPerformed(e);
    textChanged(e);
    checkChanged(e);
}

//================== FocusListener interface methods ==================

public void focusGained(FocusEvent e) {}

public void focusLost(FocusEvent e) {
	super.focusLost(e);
    textChanged(e);
}

//================== ItemListener interface methods ==================

public void itemStateChanged(ItemEvent e) {
	super.itemStateChanged(e);
    itemChanged(e);
}

//================== Class protected methods ==================

protected void init() {
	super.init(false);
	tableView.setSelectionPolicy(JCTableEnum.SELECT_NONE);
}

protected JComponent makePropertyPanel() {
	GridBagConstraints c = new GridBagConstraints();
    c.fill = GridBagConstraints.HORIZONTAL;
	c.gridx = GridBagConstraints.RELATIVE;
	c.gridy = 0;
	c.anchor = GridBagConstraints.NORTHWEST;
	c.insets = new Insets(4,4,4,4);

	// Create label offset display
    rowLabelDisplay = new JCheckBox("Row label display");
    columnLabelDisplay = new JCheckBox("Column label display");

	GroupPanel p1 = new GroupPanel("Label Display");
	p1.setLayout(new GridBagLayout());
	p1.add(rowLabelDisplay, c);
	c.gridy++;
	p1.add(columnLabelDisplay, c);

	// Create label offset components
	rowLabelOffset = new JTextField("0", 2);
	columnLabelOffset = new JTextField("0", 2);

	GroupPanel p2 = new GroupPanel("Labels Offset");
	p2.setLayout(new GridBagLayout());
	c.gridy = 0;
	p2.add(new JLabel("Row"), c);
	p2.add(rowLabelOffset, c);
	p2.add(new JLabel("pixels  "), c);
	c.gridy++;
	p2.add(new JLabel("Column"), c);
	p2.add(columnLabelOffset, c);
	p2.add(new JLabel("pixels "), c);

	// Create label placement components
	columnLabelPlacement = new JComboBox();
	columnLabelPlacement.addItem(TOP);
	columnLabelPlacement.addItem(BOTTOM);
	rowLabelPlacement = new JComboBox();
	rowLabelPlacement.addItem(LEFT);
	rowLabelPlacement.addItem(RIGHT);

	GroupPanel p3 = new GroupPanel("Labels Placement");
	p3.setLayout(new GridBagLayout());
	c.gridy = 0;
	p3.add(new JLabel("Row"), c);
	p3.add(rowLabelPlacement, c);
	c.gridy++;
	p3.add(new JLabel("Column"), c);
	p3.add(columnLabelPlacement, c);

	// Create property panel
	JPanel p = new JPanel();
	p.setLayout(new FlowLayout(FlowLayout.LEFT, 10, 6));
	p.add(p1);
	p.add(p2);
	p.add(p3);

	rowLabelDisplay.addActionListener(this);
	columnLabelDisplay.addActionListener(this);
	rowLabelOffset.addActionListener(this);
	columnLabelOffset.addActionListener(this);
	rowLabelOffset.addFocusListener(this);
	columnLabelOffset.addFocusListener(this);
	rowLabelPlacement.addItemListener(this);
	columnLabelPlacement.addItemListener(this);

	return p;
}

protected void updatePropertyPanel(JCCellRange c) {
    updateLabelLayout();
    support.firePropertyChange("labelLayout", null, getValue());
}

protected void updateLabelLayout() {
	rowLabelDisplay.setSelected(tableView.isRowLabelDisplay());
	columnLabelDisplay.setSelected(tableView.isColumnLabelDisplay());
	columnLabelOffset.setText(tableView.getColumnLabelOffset()+"");
	rowLabelOffset.setText(tableView.getRowLabelOffset()+"");
    setSelectedItem(rowLabelPlacement,
        toString(EnumRowPosition, tableView.getRowLabelPlacement()));
    setSelectedItem(columnLabelPlacement,
        toString(EnumColPosition, tableView.getColumnLabelPlacement()));
}

private void checkChanged(ActionEvent e) {
	Object o = e.getSource();
	if (o.equals(rowLabelDisplay)) {
		boolean rowDisplay = rowLabelDisplay.isSelected();
        tableView.setRowLabelDisplay(rowDisplay);
        rowLabelOffset.setEnabled(rowDisplay);
        rowLabelPlacement.setEnabled(rowDisplay);
    }
	if (o.equals(columnLabelDisplay)) {
        boolean columnDisplay = columnLabelDisplay.isSelected();
        tableView.setColumnLabelDisplay(columnDisplay);
        columnLabelOffset.setEnabled(columnDisplay);
        columnLabelPlacement.setEnabled(columnDisplay);
    }
    else {
    	return;
    }
	support.firePropertyChange("labelLayout", null, getValue());
}

private void itemChanged(ItemEvent e) {
	Object o = e.getSource();
	if (o.equals(rowLabelPlacement)) {
		tableView.setRowLabelPlacement(toIntValue(EnumRowPosition,
        	(String)rowLabelPlacement.getSelectedItem()));
    }
	if (o.equals(columnLabelPlacement)) {
		tableView.setColumnLabelPlacement(toIntValue(EnumColPosition,
        	(String)columnLabelPlacement.getSelectedItem()));
    }
    else {
    	return;
    }
    support.firePropertyChange("labelLayout", null, getValue());
}

private void textChanged(EventObject e) {
	Object o = e.getSource();
	if (o.equals(rowLabelOffset)) {
        tableView.setRowLabelOffset(getInt(rowLabelOffset.getText()));
    }
	else if (o.equals(columnLabelOffset)) {
        tableView.setColumnLabelOffset(getInt(columnLabelOffset.getText()));
	}
    else {
    	return;
    }
	support.firePropertyChange("labelLayout", null, getValue());
}

}	// LabelLayoutEditor

