/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: LiveTable.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:35 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table.beans;

import com.klg.jclass.cell.JCCellRenderer;
import com.klg.jclass.table.*;
import com.klg.jclass.table.data.JCBeanFileDataSource;
import com.klg.jclass.table.data.JCEditableVectorDataSource;
import com.klg.jclass.table.data.JCTableModelDataSource;
import com.klg.jclass.table.data.JCVectorDataSource;

import javax.swing.table.TableModel;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;


/**
 * This is the LiveTable bean class.
 * @since JClass 4.0K for Java 2. If you are using JClass 4.0J,
 * which is based on JDK 1.1, some of the return types for this class'
 * fields or methods are based on the corresponding
 * <code>com.sun.java.util</code> class.
 */
public class LiveTable extends JCTable {
//implements JCPrintListener

protected JCCellRenderer cellRenderer = null;
protected static int numColumns = 5;
protected static int numRows = 10;


/**
 * Default constructor - creates an empty 10-row x 5-column table
 */
public LiveTable()
{
	this(true);
}

/**
 * Default constructor - creates an empty 10-row x 5-column table
 * with editable or non-editable data source
 */
public LiveTable(boolean editable)
{
	super();
	makeDataSource(editable);
	setPopupMenuEnabled(true);
//  addPrintListener(this);
}

//======================== Superclass extension methods ========================

/**
 * Gets the table's <CODE>frameBorderType</CODE> property value.
 * @see #setFrameBorderType
 */
public int getFrameBorderType()
{
	CellBorderModel border = getFrameBorder();
    int retval = JCTableEnum.BORDER_NONE;
    if (border instanceof JCCellBorder) {
        retval = ((JCCellBorder) border).getBorderType();
    }
	return(retval);
}

/**
 * Sets the CellBorder model of the borders surrounding cell and label areas
 * according with the borderType value.
 * @see #getFrameBorderType
 */
public void setFrameBorderType(int borderType)
{
    setFrameBorder(new JCCellBorder(borderType));
    requestChange(TableChangeHandler.TABLE_REPAINT);
}


//======================== Property handling methods ========================

/**
 * Provided for beans property editors - has no effect.
 */
public void setAbout(String s) {}

/**
 * Returns vendor information.
 */
public String getAbout()
{
	return(JCVersion.getVersionString());
}

/**
 * Gets the data property.
 * @see #setData
 */
public DataWrapper getData()
{
	TableDataModel dataSource = getDataSource();
    DataWrapper dataWrapper = null;
    if (dataSource instanceof JCBeanFileDataSource) {
        dataWrapper =
            new DataWrapper(((JCBeanFileDataSource) dataSource).getName());
    }
    else if (dataSource instanceof JCVectorDataSource) {
        JCVectorDataSource vds = (JCVectorDataSource)dataSource;
        dataWrapper = new DataWrapper(dataSource.getNumRows(),
                                      dataSource.getNumColumns(),
                                      createArray(vds.getRowLabels()),
                                      createArray(vds.getColumnLabels()),
                                      createArray2D(vds.getCells()));
    }
    else {
        dataWrapper = new DataWrapper(dataSource);
    }
    return(dataWrapper);
}

/**
 * Sets the table data.
 */
public void setData(DataWrapper wrapper)
{
	if (wrapper == null) {
		return;
    }
	if (wrapper.isFile) {
		setDataSource(new JCBeanFileDataSource(wrapper.fileName));
	}
	else {
		setDataSource(new JCEditableVectorDataSource(
            wrapper.rows, wrapper.columns,
			createVector(wrapper.rowLabels),
            createVector(wrapper.columnLabels),
            createVector(wrapper.cellValues)));
    }
}

/**
 * Gets the frozenCellLayout property.
 * @see #setFrozenCellLayout
 */
public FrozenCellLayoutWrapper getFrozenCellLayout()
{
	return(new FrozenCellLayoutWrapper(frozen_rows, frozen_columns,
                                       frozen_row_placement,
                                       frozen_column_placement));
}

/**
 * Sets the frozenCellLayout property.
 * @see #getFrozenCellLayout
 */
public void setFrozenCellLayout(FrozenCellLayoutWrapper wrapper)
{
	if (wrapper == null) {
		return;
    }
	frozen_rows = wrapper.rows;
	frozen_row_placement = wrapper.rowPlacement;
	frozen_columns = wrapper.columns;
	frozen_column_placement = wrapper.columnPlacement;

	requestChange(TableChangeHandler.TABLE_LAYOUT);
}

/**
 * Gets the labelLayout property.
 * @see #setLabelLayout
 */
public LabelLayoutWrapper getLabelLayout() {
	return(new LabelLayoutWrapper(row_label_display, column_label_display,
                                  row_label_offset, column_label_offset,
                                  row_label_placement, column_label_placement));
}

/**
 * Sets the labelLayout property.
 * @see #getLabelLayout
 */
public void setLabelLayout(LabelLayoutWrapper wrapper)
{
	if (wrapper == null) {
		return;
    }

	row_label_display      = wrapper.rowDisplay;
	row_label_offset       = wrapper.rowOffset;
	row_label_placement    = wrapper.rowPlacement;
	column_label_display   = wrapper.columnDisplay;
	column_label_offset    = wrapper.columnOffset;
	column_label_placement = wrapper.columnPlacement;
	requestChange(TableChangeHandler.TABLE_RESET);
}

/**
 * Gets the sBLayout property.
 * @see #setSBLayout
 */
public SBLayoutWrapper getSBLayout()
{
	return(new SBLayoutWrapper(hsb_display, vsb_display, hsb_attach, vsb_attach,
                               hsb_offset, vsb_offset, hsb_position,
                               vsb_position, hsb_track, vsb_track,
                               hsb_track_row, vsb_track_column));
}

/**
 * Sets the sBLayout property.
 * @see #getSBLayout
 */
public void setSBLayout(SBLayoutWrapper wrapper)
{
	if (wrapper == null) {
		return;
    }
	hsb_display   = wrapper.hsbDisplay;
	hsb_attach    = wrapper.hsbAttach;
	hsb_offset    = wrapper.hsbOffset;
	hsb_position  = wrapper.hsbPosition;
	hsb_track     = wrapper.hsbTrack;
	hsb_track_row = wrapper.hsbTrackRow;

	vsb_display      = wrapper.vsblDisplay;
	vsb_attach       = wrapper.vsbAttach;
	vsb_offset       = wrapper.vsbOffset;
	vsb_position     = wrapper.vsbPosition;
	vsb_track        = wrapper.vsbTrack;
	vsb_track_column = wrapper.vsbTrackColumn;

	requestChange(TableChangeHandler.TABLE_LAYOUT);
}

/**
 * Gets the styles property.
 * @see #setStyles
 */
public StyleWrapper getStyles()
{
    List list = getCellStyleValues();
    CellRangeValue[] styles = new CellRangeValue[list.size()];
    Iterator iterator = list.iterator();
    for (int i = 0; iterator.hasNext(); i++) {
        styles[i] = (CellRangeValue)iterator.next();
    }
	return(new StyleWrapper(styles));
}

/**
 * Sets the styles property.
 * @see #getStyles
 */
public void setStyles(StyleWrapper wrapper)
{
	if (wrapper == null) {
		return;
    }
    for (int i=0; i<wrapper.styles.length; i++) {
    	CellRangeValue rangeValue = wrapper.styles[i];
        CellStyleModel style = (CellStyleModel)rangeValue.value;
		// JCLiveCellStyle needs a table component, otherwise
		// borders won't render
		if (style instanceof JCLiveCellStyle) {
			((JCLiveCellStyle) style).setTable(this);
		}
    	setCellStyle(new JCCellRange(rangeValue.start_row,
                                     rangeValue.start_column,
                                     rangeValue.end_row,
                                     rangeValue.end_column),
                     style);
    }
    requestChange(TableChangeHandler.TABLE_RESET);
}

/**
 * Gets the swingDataModel property.
 * @see #setSwingDataModel
 */
public void setSwingDataModel(TableModel model)
{
	JCTableModelDataSource dataSource = new JCTableModelDataSource(model);
    if (dataSource.isEnabled()) {
	    setDataSource(dataSource);
    }
}

/**
 * Gets the swingDataModel property.
 * @see #setSwingDataModel
 */
public TableModel getSwingDataModel()
{
	TableDataModel dataSource = getDataSource();
    TableModel tm = null;
    if (dataSource instanceof JCTableModelDataSource) {
        tm = ((JCTableModelDataSource) dataSource).getTableModel();
    }
	return(tm);
}

/**
 * Gets the textPositioning property.
 * @see #setCellSize
 */
public CellSizeWrapper getCellSize()
{
	return(new CellSizeWrapper(makePositionsArray(getRowHeightValues()),
                               makePositionsArray(getColumnWidthValues())));
}

protected PositionCellSize[] makePositionsArray(List list)
{
    PositionCellSize[] positions = new PositionCellSize[list.size()];
    Iterator iterator = list.iterator();
    for (int i = 0; iterator.hasNext(); i++) {
        Object o = iterator.next();
        if (!(o instanceof PositionCellSize)) {
            throw new IllegalArgumentException("Range item is not a " +
                                               "PositionCellSize instance");
        }
        positions[i] = (PositionCellSize)o;
    }
    return(positions);
}

/**
 * Sets the cellSize property.
 * @see #getCellSize
 */
public void setCellSize(CellSizeWrapper wrapper)
{
	if (wrapper == null) {
		return;
    }
    for (int i = 0; i < wrapper.rowPositions.length; i++) {
        setRowCellSize(wrapper.rowPositions[i].position,
                       wrapper.rowPositions[i].cell_size);
    }
    for (int i = 0; i < wrapper.columnPositions.length; i++) {
        setColumnCellSize(wrapper.columnPositions[i].position,
                          wrapper.columnPositions[i].cell_size);
    }
}

/**
 * Gets the spannedCells property.
 * @see #setSpannedCells
 */
public SpannedCellsWrapper getSpannedCells()
{
    Collection ranges = getSpannedRanges();
    JCCellRange[] values = null;
    if (ranges == null) {
        values = new JCCellRange[0];
    }
    else {
        values = new JCCellRange[ranges.size()];
        Iterator rangesIterator = ranges.iterator();
        for (int i = 0; rangesIterator.hasNext(); i++) {
            Object o = rangesIterator.next();
            if (!(o instanceof JCCellRange)) {
                throw new IllegalArgumentException("Range item is not a " +
                                                   "JCCellRange instance");
            }
            values[i] = (JCCellRange)o;
        }
    }
	return(new SpannedCellsWrapper(values));
}

/**
 * Sets the spannedCells property.
 * @see #getSpannedCells
 */
public void setSpannedCells(SpannedCellsWrapper wrapper)
{
	if (wrapper == null) {
		return;
    }
	Vector v = new Vector();
	for (int i = 0; i < wrapper.spannedCells.length; i++) {
        v.addElement(wrapper.spannedCells[i]);
    }
    setSpannedRanges(v);
}


//=============== Internal initializing and conversion methods ===============

/*
 * Populates the table with some data to ensure initial dimensions
 * and cell values to show state of cells.
 */
protected void makeDataSource(boolean editable)
{
	JCVectorDataSource dataSource = editable ? new JCEditableVectorDataSource()
                                             : new JCVectorDataSource();

	dataSource.setNumRows(numRows);
	dataSource.setNumColumns(numColumns);
	setVisibleRows(numRows);
	setVisibleColumns(numColumns);
	for (int c = 0; c < dataSource.getNumColumns(); c++) {
		dataSource.setColumnLabel(c, "Col: " + c);
    }
	for (int r = 0; r < dataSource.getNumRows(); r++) {
		dataSource.setRowLabel(r, "Row: " + r);
		//c ell data shortened to eliminate load time on large tables.
		for (int c = 0; c < dataSource.getNumColumns(); c++)
			dataSource.setCell(r, c, "Cell");
	}
	setDataSource(dataSource);
	setNumEditorRows(numRows);
	setNumEditorColumns(numColumns);
}

/**
 * Returns the new CellStyleModel object.
 */
public CellStyleModel createCellStyle()
{
	return(new JCLiveCellStyle(this, cellRenderer));
}

/**
 *
 */
public void setCellRenderer(JCCellRenderer renderer)
{
	cellRenderer = renderer;
}


	/**
	 * Returns the number of columns in the editor
	 *
	 * @return numColumns the number of editor columns
	 */
public static int getNumEditorColumns()
{
	return(numColumns);
}


	/**
	 * Sets the number of columns in the editor,
	 * and therefore, in the table
	 *
	 * @param   nc  int number of columns
	 */
public static void setNumEditorColumns(int nc)
{
	numColumns = nc;
}


	/**
	 * Returns the number of rows in the editor
	 *
	 * @return   numRows the number of editor rows
	 */
public static int getNumEditorRows()
{
	return(numRows);
}


	/**
	 * Sets the number of rows in the editor, and
	 * and therefore, in the table
	 *
	 * @param   nr
	 */
public static void setNumEditorRows(int nr)
{
	numRows = nr;
}

protected JCCellRange getSelectedRange()
{
	Collection collection = getSelectedCells();
	if (collection == null) {
		return(null);
	}
	Iterator iterator = collection.iterator();
    return(iterator.hasNext() ? (JCCellRange)iterator.next() : null);
}

protected String[] createArray(Vector v)
{
    String[] s = new String[v.size()];
    for (int i = 0; i < v.size(); i++) {
        s[i] = (String)v.get(i);
    }
    return(s);
}

protected String[][] createArray2D(Vector v)
{
    int v2Size = 0;
	for (int i = 0; i < v.size(); i++) {
		Object o = v.elementAt(i);
        if (o instanceof Vector) {
            v2Size = Math.max(((Vector)o).size(), v2Size);
        }
	}

    String[][] s = new String[v.size()][v2Size];
	for (int i=0; i<v.size(); i++) {
		Object o = v.elementAt(i);
        if (o instanceof Vector) {
            s[i] = new String[((Vector)o).size()];
            for (int j = 0; j < ((Vector)o).size(); j++) {
				Object e = ((Vector)o).elementAt(j);
                if (e instanceof String) {
                    s[i][j] = (String)e;
                }
            }
        }
	}
    return(s);
}

protected Vector createVector(String[] s)
{
    if (s == null) {
        return(null);
    }
    Vector v = new Vector(s.length);
    for (int i = 0; i < s.length; i++) {
        v.add(s[i]);
    }
    return(v);
}

protected Vector createVector(String[][] s)
{
	if (s == null || s.length == 0) {
		return(null);
    }
	Vector v = new Vector();
	for (int i = 0; i < s.length; i++) {
		Vector rowV = new Vector();
    	for (int j = 0; j < s[i].length; j++) {
			rowV.addElement(s[i][j]);
        }
        v.addElement(rowV);
    }
	return(v);
}

/*
public void setPrintSettings(PrintWrapper pw)
{
	if (pw == null) {
        return;
    }

	if (!(pw instanceof PrintWrapper)) {
		throw new IllegalArgumentException("Object is not an instance of " +
                                           "PrintWrapper.");
    }

    printHeader = pw.printHeader;
    printFooter = pw.printFooter;
    showPrintMenu = pw.showPrintMenu;
    showPrintPreview = pw.showPrintPreview;

	if (popupMenu != null) {
	    popupMenu.enableMenuItem(popupMenu,
                                 TablePopupMenu.commands[3],showPrintMenu);
		popupMenu.enableMenuItem(popupMenu,
                                 TablePopupMenu.commands[4],showPrintPreview);
	}
}

public PrintWrapper getPrintSettings()
{
    return(new PrintWrapper(printHeader, printFooter, showPrintMenu,
                            showPrintPreview));
}

public void printPageHeader(JCPrintEvent e)
{
    Graphics gc = e.getGraphics();
    Rectangle r = gc.getClipRect();
    FontMetrics fm = gc.getFontMetrics();
    gc.drawString(replacePage(printHeader, e.getPage()), 0, r.height/2);
}

public void printPageBody(JCPrintEvent e) {}

public void printPageFooter(JCPrintEvent e)
{
    Graphics gc = e.getGraphics();
    Rectangle r = gc.getClipRect();
    FontMetrics fm = gc.getFontMetrics();
    gc.drawString(replacePage(printFooter, e.getPage()), 0, r.height/2);
}

public void printEnd(JCPrintEvent e) {}

String replacePage(String source, int page)
{
    String output = new String(source);
    for(;;) {
        int next = output.indexOf("^P");
        if (next == -1) {
            return(output);
        }
        output = source.substring(0, next) + String.valueOf(page);
        if (source.length() > (next + 2)) {
            output = output + source.substring(next + 2);
        }
    }
}
*/

} // LiveTable


