/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: SBLayoutEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:35 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table.beans;

import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import java.util.EventObject;

import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.Document;

import com.klg.jclass.table.JCCellRange;
import com.klg.jclass.table.JCTableEnum;

/**
 * Editor for sbLayout series.
 */
public class SBLayoutEditor extends TablePropertyEditor implements
	ActionListener, FocusListener, ItemListener {

protected JComboBox horizPosition;
protected JComboBox vertPosition;
protected JComboBox horizAttachment;
protected JComboBox vertAttachment;
protected JComboBox horizDisplay;
protected JComboBox vertDisplay;
protected JComboBox horizTrack;
protected JComboBox vertTrack;
protected JTextField horizOffset;
protected JTextField vertOffset;
protected JTextField vertTrackColumn;
protected JTextField horizTrackRow;

protected static final String ALWAYS 	= "Always";
protected static final String NEVER 	= "Never";
protected static final String AS_NEEDED = "As needed";

protected static final String BY_CELLS	= "By cells";
protected static final String AT_SIDE	= "At side";

protected static final String TO_CELLS  = "To cells";
protected static final String TO_TABLE  = "To table";

protected static final String LIVE 				= "Live";
protected static final String COLUMN_NUMBER 	= "Column number";
protected static final String ROW 				= "Row";
protected static final String ROW_NUMBER 		= "Row number";
protected static final String COLUMN 			= "Column";

protected static EnumString[] EnumDisplay = {
	new EnumString(JCTableEnum.SCROLLBAR_ALWAYS, ALWAYS),
	new EnumString(JCTableEnum.SCROLLBAR_NEVER, NEVER),
	new EnumString(JCTableEnum.SCROLLBAR_AS_NEEDED, AS_NEEDED),
};

protected static EnumString[] EnumPosition = {
	new EnumString(JCTableEnum.POSITION_BY_CELLS, BY_CELLS),
	new EnumString(JCTableEnum.POSITION_AT_SIDE, AT_SIDE),
};

protected static EnumString[] EnumAttachment = {
	new EnumString(JCTableEnum.SIZE_TO_CELLS, TO_CELLS),
	new EnumString(JCTableEnum.SIZE_TO_TABLE, TO_TABLE),
};

protected static EnumString[] EnumTrackRow = {
	new EnumString(JCTableEnum.TRACK_LIVE, LIVE),
	new EnumString(JCTableEnum.TRACK_COLUMN_NUMBER, COLUMN_NUMBER),
	new EnumString(JCTableEnum.TRACK_ROW, ROW),
};

protected static EnumString[] EnumTrackColumn = {
	new EnumString(JCTableEnum.TRACK_LIVE, LIVE),
	new EnumString(JCTableEnum.TRACK_ROW_NUMBER, ROW_NUMBER),
	new EnumString(JCTableEnum.TRACK_COLUMN, COLUMN),
};

/**
 *	Constructs a SBLayoutEditor editor.
 */
public SBLayoutEditor() {
	super(SELECT_NONE);
}

/**
 *	Sets a new property value.
 */
public void setValue(Object value) {
	if (value == null) {
		return;
    }
	tableView.setSBLayout((SBLayoutWrapper)value);
    updateSBLayout();
	super.setValue(value);
}

/**
 * Returns the current value of the property.
 */
public Object getValue() {
	return tableView.getSBLayout();
}

/**
 * Returns the current property value as a String instance.
 */
public String getAsText() {
	return getValue().toString();
}

/**
 * Returns a fragment of Java code representing an initializer
 * for the current value.
 */
public String getJavaInitializationString() {
	return tableView.getSBLayout().getJavaInitializationString();
}

//================== ActionListener interface method ==================

public void actionPerformed(ActionEvent e) {
	super.actionPerformed(e);
    textChanged(e);
}

//================== FocusListener interface methods ==================

public void focusGained(FocusEvent e) {}

public void focusLost(FocusEvent e) {
	super.focusLost(e);
    textChanged(e);
}

//================== ItemListener interface methods ==================

public void itemStateChanged(ItemEvent e) {
	super.itemStateChanged(e);
    itemChanged(e);
}

//================== Class protected methods ==================

protected JComponent makePropertyPanel() {
	GridBagConstraints c = new GridBagConstraints();
	c.anchor = GridBagConstraints.NORTHWEST;
    c.fill = GridBagConstraints.HORIZONTAL;
	c.insets = new Insets(4,4,4,4);
	c.gridx = GridBagConstraints.RELATIVE;

	// Create scrollbar display policy components
	horizDisplay = new JComboBox();
	horizDisplay.addItem(ALWAYS);
	horizDisplay.addItem(NEVER);
	horizDisplay.addItem(AS_NEEDED);
	vertDisplay = new JComboBox();
	vertDisplay.addItem(ALWAYS);
	vertDisplay.addItem(NEVER);
	vertDisplay.addItem(AS_NEEDED);
	GroupPanel p1 = new GroupPanel("Scrollbar Display");
	p1.setLayout(new GridBagLayout());
	c.gridy = 0;
	p1.add(new JLabel("Horizontal"), c);
	p1.add(horizDisplay, c);
	c.gridy++;
	p1.add(new JLabel("Vertical"), c);
	p1.add(vertDisplay, c);

	// Create scrollbar position components
	horizPosition = new JComboBox();
	horizPosition.addItem(BY_CELLS);
	horizPosition.addItem(AT_SIDE);
	vertPosition = new JComboBox();
	vertPosition.addItem(BY_CELLS);
	vertPosition.addItem(AT_SIDE);
	GroupPanel p2 = new GroupPanel("Scrollbar Position");
	p2.setLayout(new GridBagLayout());
	c.gridy = 0;
	p2.add(new JLabel("Horizontal"), c);
	p2.add(horizPosition, c);
	c.gridy++;
	p2.add(new JLabel("Vertical"), c);
	p2.add(vertPosition, c);

	// Create scrollbar attachment components
	horizAttachment = new JComboBox();
	horizAttachment.addItem(TO_CELLS);
	horizAttachment.addItem(TO_TABLE);
	vertAttachment = new JComboBox();
	vertAttachment.addItem(TO_CELLS);
	vertAttachment.addItem(TO_TABLE);
	GroupPanel p3 = new GroupPanel("Scrollbar Attachment");
	p3.setLayout(new GridBagLayout());
	c.gridy = 0;
	p3.add(new JLabel("Horizontal"), c);
	p3.add(horizAttachment, c);
	c.gridy++;
	p3.add(new JLabel("Vertical"), c);
	p3.add(vertAttachment, c);

	// Create scrollbar offset components
	horizOffset = new JTextField("0", 2);
	vertOffset = new JTextField("0", 2);
	GroupPanel p4 = new GroupPanel("Scrollbar Offset");
	p4.setLayout(new GridBagLayout());
	c.gridy = 0;
	p4.add(new JLabel("Horizontal"), c);
	p4.add(horizOffset, c);
	p4.add(new JLabel("pixels  "), c);
	c.gridy++;
	p4.add(new JLabel("Vertical"), c);
	p4.add(vertOffset, c);
	p4.add(new JLabel("pixels  "), c);

	// Create scrollbar tracking components
	horizTrack = new JComboBox();
	horizTrack.addItem(LIVE);
	horizTrack.addItem(COLUMN_NUMBER);
	horizTrack.addItem(ROW);
	vertTrack = new JComboBox();
	vertTrack.addItem(LIVE);
	vertTrack.addItem(ROW_NUMBER);
	vertTrack.addItem(COLUMN);
	vertTrackColumn = new JTextField("0", 3);
	horizTrackRow = new JTextField("0", 3);
	GroupPanel p5 = new GroupPanel("Scrollbar Tracking");
	p5.setLayout(new GridBagLayout());
	c.gridy = 0;
	p5.add(new JLabel("Horizontal"), c);
	p5.add(horizTrack, c);
	p5.add(horizTrackRow, c);
	p5.add(new JLabel("row "), c);
	c.gridy++;
	p5.add(new JLabel("Vertical"), c);
	p5.add(vertTrack, c);
	p5.add(vertTrackColumn, c);
	p5.add(new JLabel("column "), c);

	// Create property panel
	JPanel p = new JPanel();
	p.setLayout(new GridBagLayout());
	c.anchor = GridBagConstraints.CENTER;
    c.fill = GridBagConstraints.BOTH;
	c.insets = new Insets(2,2,2,2);
	c.gridx = GridBagConstraints.RELATIVE;
	p.add(p1, c);
	p.add(p2, c);
	p.add(p3, c);
	c.gridy++;
	p.add(p4, c);
	c.gridwidth = 2;
	p.add(p5, c);

	horizOffset.addActionListener(this);
	vertOffset.addActionListener(this);
	horizOffset.addFocusListener(this);
	vertOffset.addFocusListener(this);
	vertTrackColumn.addActionListener(this);
	horizTrackRow.addActionListener(this);
	vertTrackColumn.addFocusListener(this);
	horizTrackRow.addFocusListener(this);
	horizDisplay.addItemListener(this);
	vertDisplay.addItemListener(this);
	horizTrack.addItemListener(this);
	vertTrack.addItemListener(this);
	horizPosition.addItemListener(this);
	vertPosition.addItemListener(this);
	horizAttachment.addItemListener(this);
	vertAttachment.addItemListener(this);
    
	return p;
}

protected void updatePropertyPanel(JCCellRange c) {
    updateSBLayout();
    support.firePropertyChange("sbLayout", null, getValue());
}

protected void updateSBLayout() {
    setSelectedItem(horizDisplay,
    	toString(EnumDisplay, tableView.getHorizSBDisplay()));
	setSelectedItem(vertDisplay,
    	toString(EnumDisplay,tableView.getVertSBDisplay()));

	setSelectedItem(horizTrack,
    	toString(EnumTrackRow, tableView.getHorizSBTrack()));
	setSelectedItem(vertTrack,
    	toString(EnumTrackColumn, tableView.getVertSBTrack()));

	setSelectedItem(horizPosition,
    	toString(EnumPosition, tableView.getHorizSBPosition()));
	setSelectedItem(vertPosition,
    	toString(EnumPosition, tableView.getVertSBPosition()));

	setSelectedItem(horizAttachment,
    	toString(EnumAttachment, tableView.getHorizSBAttachment()));
	setSelectedItem(vertAttachment,
    	toString(EnumAttachment, tableView.getVertSBAttachment()));

	updateTextField(vertOffset, tableView.getVertSBOffset());
	updateTextField(horizOffset, tableView.getHorizSBOffset());
	updateTextField(vertTrackColumn, tableView.getVertSBTrackColumn());
	updateTextField(horizTrackRow, tableView.getHorizSBTrackRow());

    setEnabled();
}

protected void setEnabled() {
    boolean hDisplay =
    	!((String)horizDisplay.getSelectedItem()).equalsIgnoreCase(NEVER);
    boolean vDisplay =
    	!((String)vertDisplay.getSelectedItem()).equalsIgnoreCase(NEVER);

    horizAttachment.setEnabled(hDisplay);
    horizPosition.setEnabled(hDisplay);
	horizOffset.setEnabled(hDisplay);
	horizTrack.setEnabled(hDisplay);
	horizTrackRow.setEnabled(hDisplay &&
    	((String)horizTrack.getSelectedItem()).equalsIgnoreCase(ROW));

    vertAttachment.setEnabled(vDisplay);
    vertPosition.setEnabled(vDisplay);
	vertOffset.setEnabled(vDisplay);
	vertTrack.setEnabled(vDisplay);
	vertTrackColumn.setEnabled(vDisplay &&
    	((String)vertTrack.getSelectedItem()).equalsIgnoreCase(COLUMN));
}

protected void updateTextField(JTextField text, int value) {
	String s = value+"";
	if (!s.equalsIgnoreCase(text.getText())) {
		text.setText(s);
    }
}

private void textChanged(EventObject e) {
	Object o = e.getSource();
	if (o.equals(horizOffset)) {
		tableView.setHorizSBOffset(getInt(horizOffset.getText()));
    }
    else if (o.equals(vertOffset)) {
		tableView.setVertSBOffset(getInt(vertOffset.getText()));
    }
	else if (o.equals(horizTrackRow)) {
		tableView.setHorizSBTrackRow(getInt(horizTrackRow.getText()));
    }
	else if (o.equals(vertTrackColumn)) {
		tableView.setVertSBTrackColumn(getInt(vertTrackColumn.getText()));
    }
    else {
    	return;
    }
	support.firePropertyChange("sbLayout", null, getValue());
}

private void itemChanged(ItemEvent e) {
	Object o = e.getSource();
	if (o.equals(horizPosition)) {
		tableView.setHorizSBPosition(toIntValue(EnumPosition,
        	(String)horizPosition.getSelectedItem()));
    }
    else if (o.equals(vertPosition)) {
		tableView.setVertSBPosition(toIntValue(EnumPosition,
        	(String)vertPosition.getSelectedItem()));
    }
    else if (o.equals(horizAttachment)) {
		tableView.setHorizSBAttachment(toIntValue(EnumAttachment,
        	(String)horizAttachment.getSelectedItem()));
    }
    else if (o.equals(vertAttachment)) {
		tableView.setVertSBAttachment(toIntValue(EnumAttachment,
        	(String)vertAttachment.getSelectedItem()));
    }
	else if (o.equals(horizDisplay)) {
		tableView.setHorizSBDisplay(toIntValue(EnumDisplay,
        	(String)horizDisplay.getSelectedItem()));
    }
    else if (o.equals(vertDisplay)) {
		tableView.setVertSBDisplay(toIntValue(EnumDisplay,
        	(String)vertDisplay.getSelectedItem()));
    }
    else if (o.equals(horizTrack)) {
		tableView.setHorizSBTrack(toIntValue(EnumTrackRow,
        	(String)horizTrack.getSelectedItem()));
    }
    else if (o.equals(vertTrack)) {
		tableView.setVertSBTrack(toIntValue(EnumTrackColumn,
        	(String)vertTrack.getSelectedItem()));
    }
    else {
    	return;
    }
    setEnabled();
    support.firePropertyChange("sbLayout", null, getValue());
}

}	// SBLayoutEditor

