/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: SpannedCellsEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:35 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table.beans;





import java.util.Collection;


import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;

import com.klg.jclass.table.JCCellRange;
import com.klg.jclass.table.JCTableEnum;
import com.klg.jclass.table.JCTableCellInfo;

/**
 * Custom property editor for SpannedCells property.
 * Allows visual specification of cell spanning information.
 * 
 * @see com.klg.jclass.table.beans.TablePropertyEditor
 */
public class SpannedCellsEditor extends TablePropertyEditor implements ActionListener {

private JButton buttonSpan;
private JButton buttonUnspan;

/**
 *	Constructs SpannedCellsEditor editor.
 */
public SpannedCellsEditor() {
	super(

 
	SELECT_CELLS);
}

/**
 *	Sets the series value.
 */
public void setValue(Object value) {
	if (value == null) {
		return;
    }
	tableView.setSpannedCells((SpannedCellsWrapper)value);
	super.setValue(value);
}

/**
 * @return The value of the property.
 */
public Object getValue() {
	return tableView.getSpannedCells();
}

public String getAsText() {
	return tableView.getSpannedCells().toString();
}

/**
 *	Returns a string which is passed to the String setter of the property.
 */
public String getJavaInitializationString() {
	return tableView.getSpannedCells().getJavaInitializationString();
}

public void actionPerformed(ActionEvent e) {
	super.actionPerformed(e);
	Object o = e.getSource();
	if (o.equals(buttonSpan) || o.equals(buttonUnspan)) {
		updateSpan(o.equals(buttonSpan));
		JCCellRange range = tableView.getSelectedRange();
		if (range != null) {
			updatePropertyPanel(range);
        }
	}
}

protected void updatePropertyPanel(JCCellRange c) {
	support.firePropertyChange("spanningCells", null, getValue());
	if (c == null) {
    	return;
    }
//    System.out.println("canSpan="+canSpan(c));
	buttonSpan.setEnabled(canSpan(c));
	buttonUnspan.setEnabled(isSpanned(c));
}

protected JComponent makePropertyPanel() {
	GroupPanel panel = new GroupPanel("Spanning cells");
	panel.setLayout(new FlowLayout());
	buttonSpan = new JButton("Span");
	buttonUnspan = new JButton("Unspan");
	buttonSpan.setEnabled(false);
	buttonUnspan.setEnabled(false);
	panel.add(buttonSpan);
	panel.add(buttonUnspan);

	buttonSpan.addActionListener(this);
	buttonUnspan.addActionListener(this);

	return panel;
}
/*
private String getStringValue(String delimiter) {
	return LiveTable.rangesToString((Collection)getValue());
}
*/
private void updateSpan(boolean toSpan) {
	JCCellRange c = tableView.getSelectedRange();
	if (c == null) {
		return;
    }
	if (toSpan) {

 
		tableView.addSpannedRange(c);
	}
	else {
    	removeSpan(c);
    }
}

private boolean canSpan(JCCellRange range) {
	return !isSpanned(range) && range != null &&
    	(range.start_row != range.end_row || range.start_column != range.end_column);
}

private boolean isSpanned(JCCellRange range) {
	if (range == null) {
		return false;
    }
    int end_row = range.end_row == JCTableEnum.MAXINT ?
    	tableView.getNumRows() : range.end_row;
    int end_column = range.end_column == JCTableEnum.MAXINT ?
    	tableView.getNumColumns() : range.end_column;
	for (int i=range.start_row; i<=end_row; i++) {
		for (int j=range.start_column; j<=end_column; j++) {
			if (tableView.getSpannedRange(i, j) != null) {
				return true;
            }
        }
	}
	return false;
}

private void removeSpan(JCCellRange range) {
	if (range == null) {
    	return;
    }
    int end_row = range.end_row == JCTableEnum.MAXINT ?
    	tableView.getNumRows() : range.end_row;
    int end_column = range.end_column == JCTableEnum.MAXINT ?
    	tableView.getNumColumns() : range.end_column;
	for (int i=range.start_row; i<=end_row; i++) {
		for (int j=range.start_column; j<=end_column; j++) {
            JCCellRange cr = tableView.getSpannedRange(i, j);
            if (cr != null) {
                tableView.removeSpannedRange(cr);
            }
        }
    }
}

} // SpannedCellsEditor

