/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: StyleEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:35 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table.beans;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.TextListener;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Point;
import java.awt.Insets;
import java.awt.Toolkit;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JColorChooser;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.text.Document;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import java.util.EventObject;

import com.klg.jclass.table.CellRangeValue;
import com.klg.jclass.table.CellStyleModel;
import com.klg.jclass.table.JCCellRange;
import com.klg.jclass.table.JCCellPosition;
import com.klg.jclass.table.JCTableCellInfo;
import com.klg.jclass.table.JCTableEnum;
import com.klg.jclass.table.TableChangeHandler;
import com.klg.jclass.table.data.JCVectorDataSource;

import com.klg.jclass.util.swing.JCFontEvent;
import com.klg.jclass.util.swing.JCFontChooserBar;
import com.klg.jclass.util.swing.JCFontListener;
import com.klg.jclass.util.swing.JCSwingTypeConverter;


/**
 * Editor for appearance series.
 */
public class StyleEditor extends TablePropertyEditor implements ActionListener,
	ItemListener, JCFontListener {

protected JCFontChooserBar fontBar;
protected JButton fgButton;
protected JButton bgButton;
protected JButton borderColorBtn;
protected JComboBox horizontalAlignment;
protected JComboBox verticalAlignment;
protected JComboBox borderSides;
protected JComboBox borderType;
protected JComboBox borderColorModeCB;
protected JComboBox clipHints;
protected JCheckBox editable;
protected JCheckBox traversable;
protected PropertyEditorTable sampleTable;

protected static final String BOTTOM 	= "Bottom";
protected static final String CENTER 	= "Center";
protected static final String LEFT 		= "Left";
protected static final String RIGHT 	= "Right";
protected static final String TOP 		= "Top";

protected static EnumString[] EnumAlignment = {
	new EnumString(CellStyleModel.BOTTOM, BOTTOM),
	new EnumString(CellStyleModel.CENTER, CENTER),
	new EnumString(CellStyleModel.LEFT, LEFT),
	new EnumString(CellStyleModel.RIGHT, RIGHT),
	new EnumString(CellStyleModel.TOP, TOP),
};

protected static final String BORDER_ETCHED_IN = "Etched in";
protected static final String BORDER_ETCHED_OUT = "Etched out";
protected static final String BORDER_FRAME_IN = "Frame in";
protected static final String BORDER_FRAME_OUT = "Frame out";
protected static final String BORDER_IN = "Border in";
protected static final String BORDER_OUT = "Border out";
protected static final String BORDER_PLAIN = "Plain";
protected static final String BORDER_NONE = "None";
protected static final String BORDER_THIN = "Thin";
protected static final String BEVEL_BORDER = "Bevel";
protected static final String SOFT_BEVEL_BORDER = "Soft bevel";

protected static EnumString[] EnumBorderType = {
	new EnumString(JCTableEnum.BORDER_ETCHED_IN, BORDER_ETCHED_IN),
	new EnumString(JCTableEnum.BORDER_ETCHED_OUT, BORDER_ETCHED_OUT),
	new EnumString(JCTableEnum.BORDER_FRAME_IN, BORDER_FRAME_IN),
	new EnumString(JCTableEnum.BORDER_FRAME_OUT, BORDER_FRAME_OUT),
	new EnumString(JCTableEnum.BORDER_IN, BORDER_IN),
	new EnumString(JCTableEnum.BORDER_OUT, BORDER_OUT),
	new EnumString(JCTableEnum.BORDER_PLAIN, BORDER_PLAIN),
	new EnumString(JCTableEnum.BORDER_NONE, BORDER_NONE),
	new EnumString(JCTableEnum.BORDER_THIN, BORDER_THIN),
	new EnumString(JCLiveCellStyle.BEVEL_BORDER, BEVEL_BORDER),
	new EnumString(JCLiveCellStyle.SOFT_BEVEL_BORDER, SOFT_BEVEL_BORDER),
};

protected static final String BORDERSIDE_ALL = "All";
protected static final String BORDERSIDE_BOTTOM = "Bottom";
protected static final String BORDERSIDE_LEFT = "Left";
protected static final String BORDERSIDE_NONE = "None";
protected static final String BORDERSIDE_RIGHT = "Right";
protected static final String BORDERSIDE_TOP = "Top";

protected static EnumString[] EnumBorderSides = {
	new EnumString(JCTableEnum.BORDERSIDE_ALL, BORDERSIDE_ALL),
	new EnumString(JCTableEnum.BORDERSIDE_BOTTOM, BORDERSIDE_BOTTOM),
	new EnumString(JCTableEnum.BORDERSIDE_LEFT, BORDERSIDE_LEFT),
	new EnumString(JCTableEnum.BORDERSIDE_NONE, BORDERSIDE_NONE),
	new EnumString(JCTableEnum.BORDERSIDE_RIGHT, BORDERSIDE_RIGHT),
	new EnumString(JCTableEnum.BORDERSIDE_TOP, BORDERSIDE_TOP),
};

protected static final String BASE_ON_BACKGROUND = "Cell background";
protected static final String BASE_ON_FOREGROUND = "Cell foreground";
protected static final String USE_CELL_BORDER_COLOR = "Cell border";

protected static EnumString[] EnumBorderColorMode = {
	new EnumString(JCTableEnum.BASE_ON_BACKGROUND, BASE_ON_BACKGROUND),
	new EnumString(JCTableEnum.BASE_ON_FOREGROUND, BASE_ON_FOREGROUND),
	new EnumString(JCTableEnum.USE_CELL_BORDER_COLOR, USE_CELL_BORDER_COLOR),
};

protected static final String SHOW_NONE = "None";
protected static final String SHOW_HORIZONTAL = "Horizontal";
protected static final String SHOW_VERTICAL = "Vertical";              
protected static final String SHOW_ALL = "All";

protected static EnumString[] EnumClipHints = {
	new EnumString(JCTableEnum.SHOW_NONE, SHOW_NONE),
	new EnumString(JCTableEnum.SHOW_HORIZONTAL, SHOW_HORIZONTAL),
	new EnumString(JCTableEnum.SHOW_VERTICAL, SHOW_VERTICAL),
	new EnumString(JCTableEnum.SHOW_ALL, SHOW_ALL),
};


private static final int sampleColumns = 2;
private static final int sampleRows = 3;



 

/**
 *	Constructs StyleEditor editor.
 */
public StyleEditor() {
	super(SELECT_CELLS);
}

/**
 *	Sets a new property value.
 */
public void setValue(Object value) {
	if (value == null) {
		return;
    }
	tableView.setStyles((StyleWrapper)value);
    updateStyle();
	super.setValue(value);
}

/**
 * Returns the current value of the property.
 */
public Object getValue() {
	return tableView.getStyles();
}

/**
 * Returns the current property value as a String instance.
 */
public String getAsText() {
	return tableView.getStyles().toString();
}

/**
 * Returns a fragment of Java code representing an initializer
 * for the current value.
 */
public String getJavaInitializationString() {
	return tableView.getStyles().getJavaInitializationString();
}

//================== ActionListener interface methods ==================

public void actionPerformed(ActionEvent e) {
	if (!updateComponents) {
		return;
    }
	super.actionPerformed(e);
    setStyleProperty(e);
}

//================== ItemListener interface methods ==================

public void itemStateChanged(ItemEvent e) {
	if (!updateComponents) {
		return;
    }
	super.itemStateChanged(e);
    setStyleProperty(e);
}

//================== JCFontListener interface methods ==================

/** Font is changing.  Listeners can changed the font and/or underline indication. */
public void fontChanging(JCFontEvent e) {}

/** Font has been changed. */
public void fontChanged(JCFontEvent e) {
	if (!updateComponents) {
		return;
    }
    setStyleProperty(e);
}

//================== Class protected methods ==================

protected JComponent makePropertyPanel() {
	// Create color panel
    tableView.setTraversable(true);

	fgButton = new JButton("Foreground...");
    fgButton.getAccessibleContext().setAccessibleName("Foreground color");
    fgButton.getAccessibleContext().setAccessibleDescription("Choose a foreground color");
	bgButton = new JButton("Background...");
    bgButton.getAccessibleContext().setAccessibleName("Background color");
    bgButton.getAccessibleContext().setAccessibleDescription("Choose a background color");

    fontBar = new JCFontChooserBar();
    fontBar.setStyleControls(JCFontChooserBar.BOLD+JCFontChooserBar.ITALIC);
    editable = new JCheckBox("Editable");
    traversable = new JCheckBox("Traversable");
	sampleTable = new SampleTable();

	horizontalAlignment = new JComboBox();
	horizontalAlignment.addItem(LEFT);
	horizontalAlignment.addItem(CENTER);
	horizontalAlignment.addItem(RIGHT);

	verticalAlignment = new JComboBox();
	verticalAlignment.addItem(TOP);
	verticalAlignment.addItem(CENTER);
	verticalAlignment.addItem(BOTTOM);

	borderSides = new JComboBox();
    initComboBox(borderSides, EnumBorderSides);

	borderType = new JComboBox();
    initComboBox(borderType, EnumBorderType);

	clipHints = new JComboBox();
    initComboBox(clipHints, EnumClipHints);

	borderColorModeCB = new JComboBox();
    initComboBox(borderColorModeCB, EnumBorderColorMode);

	borderColorBtn = new JButton("Color...");
    borderColorBtn.getAccessibleContext().setAccessibleName("Border color");
    borderColorBtn.getAccessibleContext().setAccessibleDescription("Choose a border color");

	JPanel stylePanel = new JPanel();
	stylePanel.setLayout(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();
    c.fill = GridBagConstraints.BOTH;
	c.insets = new Insets(2,2,2,2);
	c.anchor = GridBagConstraints.NORTHWEST;

	JPanel leftPanel = new JPanel();
	leftPanel.setLayout(new GridBagLayout());
	GridBagConstraints c0 = new GridBagConstraints();
    c0.fill = GridBagConstraints.BOTH;
	c0.insets = new Insets(2,2,2,2);
	c0.anchor = GridBagConstraints.NORTHWEST;
	c0.gridx = 0;
	c0.gridy = 0;
	GroupPanel colorPanel = new GroupPanel("Color");
	colorPanel.add(fgButton);
	colorPanel.add(bgButton);
	leftPanel.add(colorPanel, c0);

	GroupPanel fontPanel = new GroupPanel("Font");
	fontPanel.add(fontBar);
	c0.gridy++;
	leftPanel.add(fontPanel, c0);

	GroupPanel alignmentPanel = new GroupPanel("Alignment");
	alignmentPanel.setLayout(new GridBagLayout());
	c0.gridx = 0;
	c0.gridy = 0;
	c0.anchor = GridBagConstraints.NORTHWEST;
	alignmentPanel.add(new JLabel("Vertical"), c0);
	c0.gridx++;
	alignmentPanel.add(verticalAlignment, c0);
	c0.gridx++;
	alignmentPanel.add(new JLabel("  Horizontal"), c0);
	c0.gridx++;
	alignmentPanel.add(horizontalAlignment, c0);
	c0.gridx = 0;
	c0.gridy = 2;
	leftPanel.add(alignmentPanel, c0);

	c.gridheight = 2;
	c.gridwidth = 1;
	c.gridx = 0;
	c.gridy = 0;
	stylePanel.add(leftPanel, c);

	GroupPanel bordersPanel = new GroupPanel("Borders");
	bordersPanel.setLayout(new GridBagLayout());
	c0.gridx = 0;
	c0.gridy = 0;
	bordersPanel.add(new JLabel("Sides"), c0);
	c0.gridx++;
	bordersPanel.add(borderSides, c0);
	c0.gridx = 0;
	c0.gridy++;
	bordersPanel.add(new JLabel("Type"), c0);
	c0.gridx++;
	bordersPanel.add(borderType, c0);
	c0.gridx = 0;
	c0.gridy++;
	bordersPanel.add(new JLabel("Clip hints"), c0);
	c0.gridx++;
	bordersPanel.add(clipHints, c0);
	c0.gridx = 0;
	c0.gridy++;
	bordersPanel.add(new JLabel("Color mode"), c0);
	c0.gridx++;
	bordersPanel.add(borderColorModeCB, c0);
	c0.gridy++;
	bordersPanel.add(borderColorBtn, c0);
	c.gridheight = 2;
	c.gridx = 1;
	c.gridy = 0;
	stylePanel.add(bordersPanel, c);

	GroupPanel interPanel = new GroupPanel("Interaction");
	interPanel.add(editable);
	interPanel.add(traversable);
	c.gridheight = 1;
	c.gridx = 2;
	c.gridy = 0;
	stylePanel.add(interPanel, c);

	GroupPanel samplePanel = new GroupPanel("Sample");
	samplePanel.setLayout(new GridBagLayout());
	samplePanel.add(sampleTable, c0);
	c.gridx = 2;
	c.gridy = 1;
	stylePanel.add(samplePanel, c);

	fgButton.addActionListener(this);
	bgButton.addActionListener(this);
    borderColorBtn.addActionListener(this);
    fontBar.addJCFontListener(this);
	editable.addActionListener(this);
	traversable.addActionListener(this);
	horizontalAlignment.addItemListener(this);
	verticalAlignment.addItemListener(this);
    borderSides.addItemListener(this);
    borderType.addItemListener(this);
	clipHints.addItemListener(this);
    borderColorModeCB.addItemListener(this);

	return stylePanel;
}

protected void updatePropertyPanel(JCCellRange c) {
	updateStyle();
	support.firePropertyChange("styles", null, getValue());
}

protected synchronized void updateStyle() {
	JCCellRange range = tableView.getSelectedRange();
	if (range == null) {
    	setEnabledAll(false);
		return;
    }
   	setEnabledAll(true);
    JCLiveCellStyle style =
        (JCLiveCellStyle)tableView.getCellStyle(range.start_row, range.start_column);
    updateComponents = false;

    updateSampleTable(range);

    fontBar.setSelectedFont(style.getFont());
   	setSelected(editable, style.isEditable());
   	setSelected(traversable, style.isTraversable());

    setSelectedItem(horizontalAlignment,
        toString(EnumAlignment, style.getHorizontalAlignment()));
    setSelectedItem(verticalAlignment,
        toString(EnumAlignment, style.getVerticalAlignment()));
    setSelectedItem(borderSides,
        toString(EnumBorderSides, style.getCellBorderSides()));
    setSelectedItem(borderType,
        toString(EnumBorderType, style.getCellBorderType()));
    setSelectedItem(clipHints,
        toString(EnumClipHints, style.getClipHints()));
    setSelectedItem(borderColorModeCB,
        toString(EnumBorderColorMode, style.getCellBorderColorMode()));
                                                             
    updateComponents = true;
}

protected void setEnabledAll(boolean b) {
    fontBar.setEnabled(b);
   	editable.setEnabled(b);
   	traversable.setEnabled(b);
    horizontalAlignment.setEnabled(b);
    verticalAlignment.setEnabled(b);
    borderSides.setEnabled(b);
    borderType.setEnabled(b);
    clipHints.setEnabled(b);
    borderColorModeCB.setEnabled(b);
}

protected void updateSampleTable(JCCellRange range) {
    // Set row labels
    JCLiveCellStyle style = (JCLiveCellStyle)tableView.getCellStyle(JCTableEnum.LABEL,
    	range.start_column).clone();
    sampleTable.setCellStyle(
        new JCCellRange(JCTableEnum.LABEL,0,JCTableEnum.LABEL,JCTableEnum.MAXINT),
        style);

    // Set column labels
    style = (JCLiveCellStyle)tableView.getCellStyle(range.start_row,
    	JCTableEnum.LABEL).clone();
    sampleTable.setCellStyle(
        new JCCellRange(0,JCTableEnum.LABEL,JCTableEnum.MAXINT,JCTableEnum.LABEL),
        style);

    // Set cells
    if ((range.start_row != JCTableEnum.LABEL) &&
        (range.start_column != JCTableEnum.LABEL))
    {
        style = (JCLiveCellStyle)tableView.getCellStyle(range.start_row,
            range.start_column).clone();
        sampleTable.setCellStyle(
            new JCCellRange(0,0,JCTableEnum.MAXINT,JCTableEnum.MAXINT),
            style);
    }
}

private void setStyleProperty(EventObject e) {
	JCCellRange range = tableView.getSelectedRange();
	if (range == null) {
		return;
    }
    Object o = e.getSource();
                                                   
    int row = getFirstCell(range.start_row);
    int column = getFirstCell(range.start_column);
    JCLiveCellStyle style =
        (JCLiveCellStyle)tableView.getCellStyle(row, column).clone();

	if (o.equals(fgButton)) {
    	Color color = style.getForeground();
    	Color newColor = selectColor(true, color);
	    if (newColor == null || newColor == color) {
        	return;
        }
        style.setForeground(newColor);
    }
	else if (o.equals(bgButton)) {
    	Color color = style.getBackground();
    	Color newColor = selectColor(false, color);
	    if (newColor == null || newColor == color) {
        	return;
        }
        style.setBackground(newColor);
    }
	else if (o.equals(borderColorBtn)) {
    	Color color = style.getBackground();
    	Color newColor = selectColor(false, color);
	    if (newColor == null || newColor == color) {
        	return;
        }
        style.setCellBorderColor(newColor);
    }
	else if (o.equals(fontBar)) {
    	Font font = ((JCFontEvent)e).getFont();

 
        style.setFont(font);
    }
	else if (o.equals(editable)) {
	    style.setEditable(editable.isSelected());
    }
	else if (o.equals(traversable)) {
	    style.setTraversable(traversable.isSelected());
    }
	else if (o.equals(horizontalAlignment)) {
    	int horizontal = toIntValue(EnumAlignment, (String)horizontalAlignment.getSelectedItem());

 
		style.setHorizontalAlignment(horizontal);
    }
	else if (o.equals(verticalAlignment)) {
        int vertical = toIntValue(EnumAlignment, (String)verticalAlignment.getSelectedItem());

 
		style.setVerticalAlignment(vertical);
    }
	else if (o.equals(borderSides)) {
		style.setCellBorderSides(toIntValue(EnumBorderSides,
        	(String)borderSides.getSelectedItem()));
    }
	else if (o.equals(borderType)) {
		style.setCellBorderType(toIntValue(EnumBorderType,
        	(String)borderType.getSelectedItem()));
    }
	else if (o.equals(borderType)) {
		style.setCellBorderColorMode(toIntValue(EnumBorderColorMode,
        	(String)borderColorModeCB.getSelectedItem()));
    }
	else if (o.equals(clipHints)) {
		style.setClipHints(toIntValue(EnumClipHints,
        	(String)clipHints.getSelectedItem()));
    }
	else if (o.equals(borderColorModeCB)) {
		style.setCellBorderColorMode(toIntValue(EnumBorderColorMode,
        	(String)borderColorModeCB.getSelectedItem()));
    }
	else {
		return;
    }
    tableView.setCellStyle(tableView.getRangedSelection(), style);
//	tableView.requestChange(TableChangeHandler.TABLE_RESET);
//	tableView.requestChange(TableChangeHandler.TABLE_RESET, range);

    updateSampleTable(range);
	support.firePropertyChange("styles", null, getValue());
}

protected Color selectColor(boolean foreground, Color color) {
	String title = (foreground ? "Foreground":"Background") + " Color";


 

    return JColorChooser.showDialog(findParent(), title, color);
}

class SampleTable extends PropertyEditorTable {

    /**
     *	Initializes the layout of the custom editor.
     */
    SampleTable() {
    	super(false);
        setSelectionPolicy(JCTableEnum.SELECT_NONE);
        clearSelectedCells();
        setAllowCellResize(JCTableEnum.RESIZE_NONE);
		setFocusIndicator(JCTableEnum.FOCUS_NONE);
        setColumnLabelDisplay(true);
        setRowLabelDisplay(true);
        setPixelWidth(JCTableEnum.ALL, JCTableEnum.VARIABLE);
        setPixelHeight(JCTableEnum.ALL, JCTableEnum.VARIABLE);
    }

    /*
     * Populates the table with some data to ensure initial dimensions
     * and cell values to show state of cells.
     */
    protected void makeDataSource(boolean editable) {
        JCVectorDataSource dataSource = new JCVectorDataSource();

        dataSource.setNumRows(sampleRows);
        dataSource.setNumColumns(sampleColumns);
        setVisibleRows(sampleRows);
        setVisibleColumns(sampleColumns);
        for(int c = 0; c < dataSource.getNumColumns(); c++)
            dataSource.setColumnLabel(c, "Column: "+c);
        for(int r = 0; r < dataSource.getNumRows(); r++) {
            dataSource.setRowLabel(r, "Row: "+r);
            for(int c = 0; c < dataSource.getNumColumns(); c++)
                dataSource.setCell(r,c,"Cell");
        }
        setDataSource(dataSource);
    }

} // SampleTable


 

} // StyleEditor


