/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TablePropertyEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:35 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table.beans;


import com.klg.jclass.table.*;
import com.klg.jclass.table.data.JCBeanFileDataSource;
import com.klg.jclass.table.data.JCVectorDataSource;
import com.klg.jclass.util.swing.JCMessageHelper;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyEditor;
import java.util.EventObject;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 *	This editor provides the framework to edit a selection in a table.
 *	Sub-class this class to edit specific property.
 */
abstract public class TablePropertyEditor extends JPanel
	                                      implements PropertyEditor,
                                                     ActionListener,
                                                     ItemListener,
    			                                     JCSelectListener,
                                                     FocusListener
{

/**
 * Tables shows in the custom editor and it is used as a temporary storage
 * of the values for the actual table.beans. After user modifies this temporary table,
 * the modified property is copied back to the actual table.
 */
protected PropertyEditorTable tableView = null;

protected PropertyChangeSupport support = new PropertyChangeSupport(this);

/** Init value of the property to be used to reset initial value. */
protected Object initValue = null;

/** New line delimiter for displaying a multi-line string in one line. */
protected String nl = " ";

/** Supports selection? */
protected int selectionPolicy;

/** No selection is allowed */
static protected final int SELECT_NONE = 0;
/** Only cells, but not labels can be selected */
static protected final int SELECT_CELLS = 1;
/** All cells and labels can be selected */
static protected final int SELECT_ALL = 2;
/** Only columns can be selected */
static protected final int SELECT_COLUMNS = 3;
/** No selection is allowed, only columns are shown */
static protected final int SHOW_COLUMNS = 4;


protected JComboBox choiceRow = null;
protected JComboBox choiceColumn = null;
protected JTextField textRow = null;
protected JTextField textColumn = null;
protected JTextField textWidth = null;
protected JTextField textHeight = null;

protected JButton buttonResetAll = null;

protected static String numberDelimiters = ",:- ";

protected static final String ALLCELLS  = "allcells";
protected static final String ALL_CELLS = "all cells";
protected static final String ALL 		= "all";
protected static final String LABEL 	= "label";
protected static final String MAXINT	= "maxint";
protected static final String NOVALUE   = "novalue";
protected static final String RANGE 	= "range";

protected static EnumString[] EnumCells = {
	new EnumString(JCTableEnum.ALL, ALL),
	new EnumString(JCTableEnum.ALLCELLS, ALL_CELLS),
	new EnumString(JCTableEnum.LABEL, LABEL),
};

protected static EnumString[] EnumRange = {
	new EnumString(JCTableEnum.ALL, ALL),
	new EnumString(JCTableEnum.ALLCELLS, ALL_CELLS),
	new EnumString(JCTableEnum.LABEL, LABEL),
    new EnumString(JCTableEnum.MAXINT, MAXINT),
	new EnumString(JCTableEnum.NOVALUE, NOVALUE),
};

protected boolean updateComponents = true;

private boolean messageBoxExposed = false;

/**
 *	Constructs TablePropertyEditor editor.
 */
public TablePropertyEditor()
{
	this(SELECT_ALL);
}

/**
 *	Constructs TablePropertyEditor editor.
 */
public TablePropertyEditor(int selectionPolicy)
{
    super();
    this.selectionPolicy = selectionPolicy;
    init();
}


 


/**
 *	Initializes the layout of the custom editor.
 */
protected void init()
{
	init(false);
}

/**
 *	Initializes the layout of the custom editor.
 */
protected void init(boolean editable)
{
	tableView = new PropertyEditorTable(editable);

	// Initialize tableView.
	tableView.setName("tableView");
	tableView.clearSelectedCells();
	tableView.setAllowCellResize(JCTableEnum.RESIZE_NONE);
	tableView.setColumnLabelDisplay(true);
	tableView.setRowLabelDisplay(true);
	tableView.setPixelWidth(JCTableEnum.ALL, JCTableEnum.VARIABLE);
	tableView.setPixelHeight(JCTableEnum.ALL, JCTableEnum.VARIABLE);
    tableView.setSelectionModel(new PropertyEditorSelection(tableView));
	tableView.setSelectionPolicy(selectionPolicy == SELECT_NONE ?
    	JCTableEnum.SELECT_NONE : JCTableEnum.SELECT_RANGE);

	BorderLayout mainLayout = new BorderLayout();
	mainLayout.setVgap(6);
	mainLayout.setHgap(6);
	setLayout(mainLayout);
	add("North", makePropertyPanel());
	add("Center", tableView);
	add("East", makeControlPanel());

	updateSizePanel();
	tableView.addSelectListener(this);
	textWidth.addFocusListener(this);
	textHeight.addFocusListener(this);
	textWidth.addActionListener(this);
	textHeight.addActionListener(this);


 
}

public PropertyEditorTable getTableView()
{
	return(tableView);
}

/**
 *	Sets the series value.
 */
public void setValue(Object value)
{
	if (initValue == null) {
		initValue = value;
    }
}

/**
 * Returns the custom editor.
 */
public Component getCustomEditor()
{
	return(this);
}

/**
 * Returns the tag values for this property.  Tags are supported
 * in this class.
 */
public String[] getTags()
{
	return(null);
}

/**
 * Always returns True meaning the paintValue method is honored.
 */
public boolean isPaintable()
{
	return(true);
}

/**
 * Sets the property value as text.
 */
public void setAsText(String text)
{
    setValue(text);
}

/**
 * Always returns true since this class is for custom editors.
 */
public boolean supportsCustomEditor()
{
	return(true);
}

/**
 * Displays selection value as string
 */
public void paintValue(Graphics gc, Rectangle box)
{
	String s = getAsText();
	if (s == null) {
		s = new String("");
	}
	FontMetrics fm = gc.getFontMetrics();
	int vpad = (box.height - fm.getAscent())/2;
	gc.drawString(s, box.x + 2, box.y + box.height - vpad);
}

public void addPropertyChangeListener(PropertyChangeListener l)
{
	support.addPropertyChangeListener(l);
}

public void removePropertyChangeListener(PropertyChangeListener l)
{
	support.removePropertyChangeListener(l);
}

//================== ActionListener interface method ==================

public void actionPerformed(ActionEvent e)
{
	if (e.getSource().equals(buttonResetAll)) {
		resetValue();
	}
    else {
		textChanged(e);
    }
}

//================== FocusListener interface methods ==================

public void focusGained(FocusEvent e) {}

public void focusLost(FocusEvent e)
{
	textChanged(e);
}

//================== ItemListener interface methods ==================

public void itemStateChanged(ItemEvent e)
{
	itemChanged(e);
}

//================== JCSelectListener interface methods ==================

/**
 * Invoked whenever a select or deselect is initiated.
 */
public void beforeSelect(JCSelectEvent e) {}

/**
 * Invoked during the selection process.
 */
public void select(JCSelectEvent e) {}

/**
 * Invoked whenever a select or deselect is completed.
 */
public void afterSelect(JCSelectEvent e)
{
	selectionChanged(e);
}

//================== Class protected methods ==================

protected static int toIntValue(EnumString[] EnumStrings, String value)
{
    return(PropertyEditorTable.toIntValue(EnumStrings, value));
}

protected static String toString(EnumString[] EnumStrings, int v)
{
    return(PropertyEditorTable.toString(EnumStrings, v));
}

protected static void initComboBox(JComboBox cb, EnumString[] EnumStrings)
{
    for (int i = 0; i < EnumStrings.length; i++) {
        cb.addItem(EnumStrings[i].name);
    }
}

protected static int getFirstCell(int startCell)
{
    if (startCell == JCTableEnum.ALL) {
        startCell = JCTableEnum.LABEL;
    }
    else if (startCell == JCTableEnum.ALLCELLS) {
        startCell = 0;
    }
	return(startCell);
}

public Frame findParent()
{
	return((Frame)SwingUtilities.getAncestorOfClass(Frame.class, this));
}

/**
 * Resets to the initial value
 */
private void resetValue()
{
	if (initValue != null) {
		setValue(initValue);
		updatePropertyPanel(tableView.getSelectedRange());
	}
}

abstract protected JComponent makePropertyPanel();

abstract protected void updatePropertyPanel(JCCellRange c);

protected JComponent makeSelectionPanel()
{
	choiceRow = new JComboBox();
	choiceColumn = new JComboBox();
	textRow = new JTextField("0", 5);
	textColumn = new JTextField("0", 5);

	if (selectionPolicy == SELECT_COLUMNS) {
		choiceRow.addItem(ALL);
    }
	else {
		choiceRow.addItem(RANGE);
		choiceRow.addItem(ALL_CELLS);
		if (selectionPolicy == SELECT_ALL) {
			choiceRow.addItem(ALL);
        }
		if (selectionPolicy == SELECT_ALL || selectionPolicy == SELECT_CELLS) {
			choiceRow.addItem(LABEL);
		}
	}
	choiceColumn.addItem(RANGE);
	choiceColumn.addItem(ALL_CELLS);
	if (selectionPolicy == SELECT_ALL || selectionPolicy == SELECT_COLUMNS) {
		choiceColumn.addItem(ALL);
    }
    if (selectionPolicy == SELECT_ALL || selectionPolicy == SELECT_CELLS ||
    		selectionPolicy == SELECT_COLUMNS) {
		choiceColumn.addItem(LABEL);
	}

	GroupPanel panelRange = new GroupPanel("Selected region");
	panelRange.setLayout(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();
	c.gridx = GridBagConstraints.RELATIVE;
	c.gridy = 1;
	c.anchor = GridBagConstraints.NORTHWEST;
	c.insets = new Insets(4,4,4,4);
	panelRange.add(new JLabel("Row"), c);
	panelRange.add(choiceRow, c);
	panelRange.add(textRow, c);
	c.gridy++;
	panelRange.add(new JLabel("Column"), c);
	panelRange.add(choiceColumn, c);
	panelRange.add(textColumn, c);

	choiceRow.addItemListener(this);
	choiceColumn.addItemListener(this);
	textRow.addFocusListener(this);
	textColumn.addFocusListener(this);
	textRow.addActionListener(this);
	textColumn.addActionListener(this);

	return(panelRange);
}

protected JComponent makeSizeResetPanel()
{
	textWidth = new JTextField(3);
	textHeight = new JTextField(3);
	buttonResetAll = new JButton("Reset All");

	GroupPanel panelSize = new GroupPanel("Table size");
	panelSize.setLayout(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();
	c.gridx = GridBagConstraints.RELATIVE;
	c.gridy = 1;
	c.anchor = GridBagConstraints.NORTHWEST;
	c.insets = new Insets(4,4,4,4);
	panelSize.add(textHeight, c);
	panelSize.add(new JLabel("rows"), c);
	panelSize.add(textWidth, c);
	panelSize.add(new JLabel("columns"), c);
	c.fill = GridBagConstraints.HORIZONTAL;

	// Create button panel
	JPanel panelButtons = new JPanel();
	panelButtons.setLayout(new FlowLayout(FlowLayout.RIGHT));
	panelButtons.add(buttonResetAll);

	JPanel panelSizeReset = new JPanel();
	BorderLayout layout = new BorderLayout();
	layout.setVgap(6);
	panelSizeReset.setLayout(layout);
	panelSizeReset.add("Center", panelSize);
	panelSizeReset.add("South", panelButtons);

	buttonResetAll.addActionListener(this);

	return(panelSizeReset);
}

protected JComponent makeControlPanel()
{
	JPanel panelControl = new JPanel();
	BorderLayout layoutSelection = new BorderLayout();
	layoutSelection.setVgap(10);
	panelControl.setLayout(layoutSelection);
	if (selectionPolicy != SELECT_NONE && selectionPolicy != SHOW_COLUMNS) {
		panelControl.add("North", makeSelectionPanel());
	}
	panelControl.add("South", makeSizeResetPanel());
	return(panelControl);
}

protected void updateSize()
{
	if (updateTableSize()) {
		updateSizePanel();
    }
}

/**
 * Updates the sizes reflected in the size panel of the editor
 */
protected void updateSizePanel()
{
	TableDataModel dataSource = tableView.getDataSource();
	boolean table = !(dataSource instanceof JCBeanFileDataSource);
	enable(textWidth, table, 0, 0);
	enable(textHeight, table, 0, 0);
	if (table) {
		String height = "" + dataSource.getNumRows();
		String width  = "" + dataSource.getNumColumns();
		if (height.compareTo(textHeight.getText()) != 0) {
			textHeight.setText(height);
        }
		if (width.compareTo(textWidth.getText()) != 0) {
			textWidth.setText(width);
        }
	}
}

/**
 * Updates the size of the table in the series editor
 */
protected boolean updateTableSize()
{
	boolean changed = false;
	TableDataModel dataSource = tableView.getDataSource();
	if (dataSource instanceof JCVectorDataSource) {
		int w = LiveTable.getNumEditorColumns();
		if (dataSource.getNumColumns() != w) {
			((JCVectorDataSource)dataSource).setNumColumns(w);
			changed = true;
		}
		int h = LiveTable.getNumEditorRows();
		if (dataSource.getNumRows() != h) {
			((JCVectorDataSource)dataSource).setNumRows(h);
			changed = true;
		}
	}
	return(changed);
}

protected void errorMessageBox(String message)
{
	if (messageBoxExposed) {
		return;
    }
	messageBoxExposed = true;
	JCMessageHelper.showError("LiveTable Bean Error", message);
	messageBoxExposed = false;
}

protected void updateSelection()
{
	int startRow    = getStart((String)choiceRow.getSelectedItem(),
                               textRow.getText());
	int endRow =     getEnd((String)choiceRow.getSelectedItem(),
                            textRow.getText());
	int startColumn = getStart((String)choiceColumn.getSelectedItem(),
                               textColumn.getText());
	int endColumn   = getEnd((String)choiceColumn.getSelectedItem(),
                             textColumn.getText());

    if (startRow == Integer.MIN_VALUE || startRow == Integer.MIN_VALUE ||
    	startColumn == Integer.MIN_VALUE || endColumn == Integer.MIN_VALUE ||
        (startRow == JCTableEnum.LABEL && startColumn == JCTableEnum.LABEL))
    {
        errorMessageBox("The selected region is invalid.");
        changeSelection();
        return;
    }
    updateSelection(startRow, startColumn, endRow, endColumn);
}

protected void updateSelection(int startRow, int startColumn,
							   int endRow, int endColumn)
{
	if (endRow == JCTableEnum.NOVALUE) {
		endRow = startRow;
    }
	if (endColumn == JCTableEnum.NOVALUE) {
		endColumn = startColumn;
    }
	if (startRow == JCTableEnum.NOVALUE || startColumn == JCTableEnum.NOVALUE) {
		tableView.clearSelectedCells();
    }
	else {
		tableView.clearSelectedCells();
		JCCellRange c = new JCCellRange(startRow, startColumn, endRow,
                                        endColumn);
		if (c.start_row == JCTableEnum.ALL) {
			c.start_row = JCTableEnum.LABEL;
			c.end_row = JCTableEnum.MAXINT;
		}
		if (c.start_column == JCTableEnum.ALL) {
			c.start_column = JCTableEnum.LABEL;
			c.end_column = JCTableEnum.MAXINT;
		}
		if (c.start_row == JCTableEnum.ALLCELLS) {
			c.start_row = 0;
			c.end_row = tableView.getNumRows() - 1;
		}
		if (c.start_column == JCTableEnum.ALLCELLS) {
			c.start_column = 0;
			c.end_column = tableView.getNumColumns() - 1;
		}
        JCCellRange range = tableView.getSelectedRange();
		if (range == null || !c.equals(range)) {
			tableView.setSelectedCells(c);
			updatePropertyPanel(tableView.getSelectedRange());
        }
	}
}



protected static int getInt(String text)
{
	try {
		return Integer.valueOf(text).intValue();
	}
	catch (NumberFormatException exception) {}
	return(0);
}

protected void enable(JTextField field, boolean enabled, int start, int end)
{
	String s = start != end ? (start < end ? start + "," + end
                                           : end + "," + start)
                            : start + "";
	field.setEnabled(enabled);
	field.setText(enabled ? s : "");
}

protected boolean setChoice(JComboBox choice, int value)
{
	if (value == JCTableEnum.ALL) {
		setSelectedItem(choice, ALL);
    }
	else if (value == JCTableEnum.ALLCELLS) {
		setSelectedItem(choice, ALL_CELLS);
    }
	else if (value == JCTableEnum.LABEL) {
		setSelectedItem(choice, LABEL);
    }
	else {
		setSelectedItem(choice, RANGE);
		return(true);
	}
	return(false);
}

protected static void setSelectedItem(JComboBox choice, String select)
{
	if (!select.equals(choice.getSelectedItem())) {
	    choice.setSelectedItem(select);
    }
}

protected static void setSelected(JRadioButton button, boolean selected)
{
    if (!button.isSelected()) {
        button.setSelected(selected);
    }
}

protected static void setSelected(JCheckBox check, boolean selected)
{
    if (check.isSelected() != selected) {
        check.setSelected(selected);
    }
}

protected static int getStart(String s, String range)
{
	return(s.equals(RANGE) ? getRangeValue(range).start
                           : toIntValue(EnumCells, s));
}

private static int getEnd(String s, String range)
{
	return(s.equals(RANGE) ? getRangeValue(range).end
                           : toIntValue(EnumCells, s));
}

private void itemChanged(ItemEvent e)
{
	Object o = e.getSource();
	if (o.equals(choiceRow) || o.equals(choiceColumn)) {
		JCCellRange range = tableView.getSelectedRange();
        if (range == null) {
			return;
        }
		if (o.equals(choiceRow)) {
			enable(textRow, (choiceRow.getSelectedItem()).equals(RANGE),
            	range.start_row, range.end_row);
        }
		else if (o.equals(choiceColumn)) {
			enable(textColumn, (choiceColumn.getSelectedItem()).equals(RANGE),
				range.start_column, range.end_column);
        }
        updateSelection();
//		updateSelection(range.start_row, range.start_column,
//        	            range.end_row, range.end_column);
	}
}

private void changeSelection()
{
	JCCellRange range = tableView.getRangedSelection();
	if (range == null) {
		return;
    }
    updateComponents = false;

	enable(textColumn, setChoice(choiceColumn, range.start_column),
    	   range.start_column, range.end_column);
	enable(textRow, setChoice(choiceRow, range.start_row),
    	   range.start_row, range.end_row);
	updatePropertyPanel(range);

    updateComponents = true;
}

private void selectionChanged(JCSelectEvent e)
{
	if (e.getAction() == JCSelectEvent.DESELECT) {
    	return;
    }
    changeSelection();
}

private void textChanged(EventObject e)
{
	Object source = e.getSource();
	if (source.equals(textHeight)) {
    	int height = getInt(textHeight.getText());
        if (height == LiveTable.getNumEditorRows()) {
        	return;
        }
        LiveTable.setNumEditorRows(height);
        TableDataModel dataSource = tableView.getDataSource();
        int sourceHeight = dataSource.getNumRows();
      
        if (dataSource instanceof JCVectorDataSource && height != sourceHeight){
        	// If the new height is less, delete Rows
        	if (height < sourceHeight ){
	        	int dif = sourceHeight - height;
	        	((JCVectorDataSource)dataSource).deleteRows(sourceHeight - dif,
                                                            dif);
        	}
            // If the new height is more, add Rows
            else if (height > sourceHeight) {
        		int dif = height - sourceHeight;
        		Vector rowVector = new Vector();
        		for (int i = 0; i < dataSource.getNumColumns(); i++) {
        			rowVector.addElement("Cell");
                }
        		for (int i = 0; i < dif; i++) {
        		    ((JCVectorDataSource)dataSource).addRow(JCTableEnum.MAXINT,
                                                            "Row: " +
                                                            (sourceHeight + i),
                                                            rowVector);
                }
        	}
        }
    }
	else if (source.equals(textWidth)) {
    	int width = getInt(textWidth.getText());
        if (width == LiveTable.getNumEditorColumns()) {
        	return;
        }
        LiveTable.setNumEditorColumns(width);
        TableDataModel dataSource = tableView.getDataSource();
        int sourceWidth = dataSource.getNumColumns();
      
        if (dataSource instanceof JCVectorDataSource && width != sourceWidth){
        	// If the new width is less, delete columns.
        	if (width < sourceWidth ){
	        	int dif = sourceWidth - width;
	        	((JCVectorDataSource)dataSource).deleteColumns(sourceWidth - dif,
                                                               dif);
        	}
            // If the new width is more, add columns.
            else if (width > sourceWidth) {
        		int dif = width - sourceWidth;
        		Vector colVector = new Vector();
        		for (int i = 0; i < dataSource.getNumRows();i++) {
        			colVector.addElement("Cell");
                }
        		for (int i = 0; i < dif; i++) {
        		    ((JCVectorDataSource)dataSource).addColumn(JCTableEnum.MAXINT,
                                                               "Col: " +
                                                               (sourceWidth + i),
                                                               colVector);
                }
        	}
        }
    }
	else if (source.equals(textRow) || source.equals(textColumn)) {
		updateSelection();
    }
}

protected static RangeValue getRangeValue(String value)
{
	return(getRangeValue(value, numberDelimiters));
}

protected static RangeValue getRangeValue(String value, String delimiters)
{
	value = value.trim();
	if (value.length() > 0) {
        StringTokenizer st = new StringTokenizer(value, delimiters);
        while (st.hasMoreTokens()) {
            int start = toIntValue(EnumRange, st.nextToken());
            int end = st.hasMoreTokens() ? toIntValue(EnumRange, st.nextToken())
                                         : start;
            if (start == Integer.MIN_VALUE || end == Integer.MIN_VALUE) {
            	break;
            }
            return new RangeValue(start, end);
        }
    }
    throw new IllegalArgumentException("Invalid style property string value: " +
                                       value);
}


//======================== Inner classes ========================

/**
 * The PropertyEditorSelection class supports row and column level selection.
 */
class PropertyEditorSelection extends JCSelection
{
	protected boolean labelSelection = false;

	public PropertyEditorSelection()
    {
		super();
	}

	public PropertyEditorSelection(JCTable table)
    {
		super(table);
	}

    /**
     * Returns true if the given row/column is selected.
     * @return true if the selection is accepted.
     */
    public boolean isSelected(int row, int column)
    {
        return(super.isSelected(row, column));
    }

    /**
     * Sets the selection to a JCCellRange.
     * @return true if the selection is accepted.
     */
    public boolean setSelection(JCCellRange range)
    {
        if (TablePropertyEditor.this.selectionPolicy == SELECT_CELLS) {
            if (range.start_row == JCTableEnum.LABEL &&
            	range.end_row == JCTableEnum.MAXINT)
            {
                range.start_row = 0;
            }
            if (range.start_column == JCTableEnum.LABEL &&
            	range.end_column == JCTableEnum.MAXINT)
            {
                range.start_column = 0;
            }
        }
        return(super.setSelection(range));
    }

} // PropertyEditorSelection


protected class GroupPanel extends JPanel
{
    GroupPanel(String title) {
	    super();
	    setBorder(BorderFactory.createTitledBorder(title));
    }
} // GroupPanel


/**
 * Class internal to TablePropertyEditor that manages
 * a range: start position and end position.
 */
protected static class RangeValue
{
	int start;
    int end;

    RangeValue(int start, int end)
    {
        this.start = start;
        this.end = end;
    }
} // RangeValue

}

