/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AbstractVectorDataSource.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:35 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table.data;

import java.util.Vector;

/**
 * The <CODE>AbstractVectorDataSource</CODE> abstract class implements the
 * <CODE>TableDataModel</CODE> interface and support for block moves, insertions
 * and deletions to Vector.
 */
public abstract class AbstractVectorDataSource extends AbstractDataSource {

/**
 * Default constructor, constructs a vector with initial
 * capacity of 1 and a growth capacityIncrement of 10.
 */
protected static Vector createVector()
{
    return(new Vector(1, 10));
}

/**
 * Constructs a vector with the specified initial size and a capacityIncrement
 * of 10
 * @param initialCapacity initial size of vector
 */
protected static Vector createVector(int initialCapacity)
{
    return(new Vector(initialCapacity, 10));
}

/**
 * Constructs an empty vector with the specified initial capacity and
 * capacity increment.
 * @param   initialCapacity     the initial capacity of the vector.
 * @param   capacityIncrement   the amount by which the capacity is
 * increased when the vector overflows.  If 0, the capacity of the vector is
 * doubled each time it needs to grow.
 */
protected static Vector createVector(int initialCapacity, int capacityIncrement)
{
    return(new Vector(initialCapacity, capacityIncrement));
}

/**
 * Creates a new Vector instance from a Vector. Note that any elements that are
 * Vectors are recursively recreated.
 * @param vector instance to be recreated.
 */
protected static Vector createVector(Vector vector)
{
    Vector v = new Vector();
    if (vector != null) {
        for (int i = 0; i < vector.size(); i++) {
            Object obj = vector.elementAt(i);
            if (obj != null && obj instanceof Vector) {
                Vector jv = createVector((Vector)obj);
                v.addElement(jv);
            }
            else {
                v.addElement(vector.elementAt(i));
            }
        }
    }
    return(v);
}

/**
 * Creates a JCVector using an array of objects.
 * If the array is null, the vector is cleared.
 * @param array array to use as the source for this vector
 */
protected static Vector createVector(Object[] array)
{
    Vector v = new Vector(array.length);
    if (array != null) {
        for (int i = 0; i < array.length; i++) {
            v.addElement(array[i]);
        }
    }
    return(v);
}

/**
 * Deletes a block starting at the specified index. Elements with an index
 * greater than the end point of the block are moved up.
 * @param index the element to remove
 * @param length the number of elements to delete
 */
protected static final synchronized void removeElementsAt(Vector vector,
                                                          int index, int length)
{
    int last = Math.min(index + length, vector.size());
    for (int i = index; i < last; i++) {
        vector.removeElementAt(index);
    }
}

/**
 * Sets the element at the specified index to be the specified object.
 * The previous element at this position is discarded.
 * In contrast to Vector.setElementAt(), the size of the vector is increased
 * if necessary.
 * @param index the specified index
 * @param value the value set to the element
 */
protected static synchronized void setElementAt(Vector vector, int index,
                                                Object value)
{
    if (value != null) {
        if (vector.size() < index + 1) {
            vector.setSize(index + 1);
        }
        vector.setElementAt(value, index);
    }
    else if (index < vector.size()) {
        vector.setElementAt(null, index);
    }
}

/**
 * Trims null values from the end.
 * @see java.util.Vector#trimToSize
 */
protected static final synchronized void strip(Vector vector)
{
    for (int i = vector.size() - 1; i >= 0; i--) {
        if (vector.elementAt(i) != null) {
            vector.setSize(++i);
            vector.trimToSize();
            return;
        }
    }
}

}
