/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCAppletDataSource.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:35 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table.data;

import java.applet.Applet;
import java.io.IOException;
import java.io.StringReader;

import com.klg.jclass.table.JCTableEnum;

/**
 * An <CODE>JCAppletDataSource</CODE> object reads in data from the 
 * <CODE>DATA</CODE> tag of an applet.
 *
 * <P><STRONG>Note:</STRONG><BR>
 *  In Netscape, new lines are stripped; make sure that if you
 *  spread the data across multiple lines, a space appears at the
 *  beginning of each line.
 *
 * @see JCInputStreamDataSource
 */
public class JCAppletDataSource extends JCInputStreamDataSource {

/**
 * Pulls the data from the <CODE>DATA</CODE> param tag associated with
 * the specified applet.
 * @exception IOException An <CODE>IOException</CODE> will be thrown if
 * there is a problem reading the data or if there is
 * no <CODE>DATA</CODE> parameter.
 * @param applet
 */
public JCAppletDataSource(Applet applet) throws IOException {
	String s = applet.getParameter("DATA");

	if(s == null) {
		throw new IOException("No DATA tag");
	}

	StringReader reader = new StringReader(s);
	read(reader);
}

/**
 * Pulls the data from the <CODE>DATA</CODE> param tag associated with
 * the specified applet and uses a custom data parser to make sense of it.
 * @exception IOException An <CODE>IOException</CODE> will be thrown if
 * there is a problem reading the data or if there is
 * no <CODE>DATA</CODE> parameter.
 * @param applet
 * @param prs
 */
public JCAppletDataSource(Applet applet, JCFileFormatParser prs) 
												throws IOException {
	String s = applet.getParameter("DATA");

	if(s == null) {
		throw new IOException("No DATA tag");
	}

	setDataFormat(JCTableEnum.CUSTOM);
	setCustomParser(prs);
	StringReader reader = new StringReader(s);
	read(reader);
}

}
