/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCFileDataSource.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:36 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table.data;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import com.klg.jclass.table.JCTableEnum;

/**
 * Creates an <CODE>JCInputStreamDataSource</CODE> from a file.
 * @see JCInputStreamDataSource
 */
public class JCFileDataSource extends JCInputStreamDataSource {
	
/**
 * Loads data from a named file into JClass LiveTable.
 * 
 * @param fname the name of the file to be used
 * @exception java.io.FileNotFoundException Thrown if the file isn't there.
 * @exception java.io.IOException Thrown if some other sort of i/o problem is
 * encountered.
 */
public JCFileDataSource(String fname) throws FileNotFoundException, IOException {
	this(fname, JCTableEnum.DETECT);
}

/**
 * Loads data from a named file into LiveTable.
 * 
 * @param fname the name of the file to be used
 * @param newFormat data format of the file; one of <code>STANDARD</code> or <code>CSV</code>
 * @exception java.io.FileNotFoundException Thrown if the file isn't there.
 * @exception java.io.IOException Thrown if some other sort of i/o problem is
 * encountered.
 */
public JCFileDataSource(String fname, int newFormat) 
		 throws FileNotFoundException, IOException {
	super(new FileInputStream(fname), newFormat);
}

/**
 * Loads data from a named file into LiveTable using a custom data parser.
 * 
 * @param fname the name of the file to be used
 * @param prs custom data parser used to parse data from the file
 * @exception java.io.FileNotFoundException Thrown if the file isn't there.
 * @exception java.io.IOException Thrown if some other sort of i/o problem is
 * encountered.
 */
public JCFileDataSource(String fname, JCFileFormatParser prs) 
		 throws FileNotFoundException, IOException {
	super(new FileInputStream(fname), prs);
}

/**
 * Loads data from a java.io.File instance into JClass LiveTable.
 *
 * @param f a <code>File</code> object representing the file to be opened
 * @exception java.io.FileNotFoundException Thrown if the file isn't there.
 * @exception java.io.IOException Thrown if some other sort of i/o problem is
 * encountered.
 * @see java.io.File
 */
public JCFileDataSource(File f) throws FileNotFoundException, IOException {
	this(f, JCTableEnum.DETECT);
}

/**
 * Loads data from a <i>java.io.File</i> instance into JClass LiveTable.
 *
 * @param f a <code>File</code> object representing the file to be opened
 * @param newFormat data format of the file; one of <code>STANDARD</code> or <code>CSV</code>
 * @exception java.io.FileNotFoundException Thrown if the file isn't there.
 * @exception java.io.IOException Thrown if some other sort of i/o problem is
 * encountered.
 * @see java.io.File
 */
public JCFileDataSource(File f, int newFormat) 
		 throws FileNotFoundException, IOException {
	super(new FileInputStream(f), newFormat);
}

/**
 * Loads data from a <i>java.io.File</i> instance into JClass LiveTable using a
 * custom data parser.
 *
 * @param f a <code>File</code> object representing the file to be opened
 * @param prs custom data parser used to parse data from the file
 * @exception java.io.FileNotFoundException Thrown if the file isn't there.
 * @exception java.io.IOException Thrown if some other sort of i/o problem is
 * encountered.
 * @see java.io.File
 */
public JCFileDataSource(File f, JCFileFormatParser prs) 
		 throws FileNotFoundException, IOException {
	super(new FileInputStream(f), prs);
}

}
