/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCResultSetDataSource.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:36 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table.data;

import java.sql.*;
import java.util.Properties;
import com.klg.jclass.table.JCTableEnum;





import java.util.Vector;


/**
 * This class is a simple read-only JDBC database source that retrieves
 * a result set and stores it internally.
 * @since JClass 4.0K for Java 2. If you are using JClass 4.0J,
 * which is based on JDK 1.1, some of the return types for this class'
 * fields or methods are based on the corresponding
 * <code>com.sun.java.util</code> class.
 *
 * @see com.klg.jclass.table.TableDataModel
 */
public class JCResultSetDataSource extends JCVectorDataSource {

protected Connection dbcon;
protected String query;
protected int column_count;
protected int column_type[];

/**
 * Creates a new <CODE>JCResultSetDataSource</CODE> object.
 */
public JCResultSetDataSource() {
	this.dbcon = null;
	this.query = null;
}

/**
 * Creates new <CODE>JCResultSetDataSource</CODE> object given a <CODE>Connection</CODE>
 * object and query String.
 * @param dbcon
 * @param query
 */
public JCResultSetDataSource(Connection dbcon, String query) {
	this.dbcon = dbcon;
	this.query = query;
}

/**
 * Executes the query and stores its results in a <CODE>JCVector</CODE>.
 * 
 * @exception java.sql.SQLException  If a database access error occurred.
 * @return
 */
public boolean executeQuery() throws SQLException {
	boolean success = false;
	Vector row_data = new Vector();
	int row_count = -1;

	if(dbcon == null) {
		return success;
	}
	
	try {
		Statement stmt = dbcon.createStatement();
		ResultSet rs = stmt.executeQuery(query);
		ResultSetMetaData rsmd = rs.getMetaData();

		column_count = rsmd.getColumnCount();
		setNumColumns(column_count);
		column_type = new int[column_count];

		// get the column names and types
		for(int c = 0; c < column_count; c++) {
			setColumnLabel(c, rsmd.getColumnName(c+1));
			column_type[c] = rsmd.getColumnType(c+1);
		}
		
		while(rs.next()) {
			row_count++;
			// add each row to the datasource
			for(int c = 0; c < column_count; c++) {
				setCell(row_count, c, rs.getObject(c+1));
			}
		}
	
		success = true;

		// free resources 
		rs.close();
		stmt.close();
	}
	catch(SQLException e) {
		throw(e);
	}
	setNumRows(row_count + 1);
	return success;
}

/**
 * Returns a JDBC database connection.
 * @return JDBC database connection
 */
public Connection getConnection() {
	return dbcon;
}

/**
 * Sets the JDBC database connection.
 * @param dbcon
 */
public void setConnection(Connection dbcon) {
	this.dbcon = dbcon;
}

/**
 * Returns the SQL statement.
 * @return SQL statement
 */
public String getQuery() {
	return query;
}

/**
 * Sets the SQL statement.
 * @param query
 */
public void setQuery(String query) {
	this.query = query;
}

/**
 * Returns an array of column types.
 * @return array of column types
 */
public int[] getColumnTypes() {
	return column_type;
}

/**
 * Returns the JDBC column type for a column.
 * @return JDBC column type
 * @param column
 */
public int getColumnType(int column) {
	if(column < 0 || column >= columns) {
		return -1;
	}
	return column_type[column];
}

/**
 * Returns the table datatype for a column.
 * @return table datatype for a column
 * @param column
 */
public int getColumnDatatype(int column) {
	int type = JCTableEnum.TYPE_STRING;

	if(column < 0 || column >= columns) {
		return type;		
	}

	switch(column_type[column]) {
	case Types.BIT:
	case Types.TINYINT:
	case Types.SMALLINT:
	case Types.INTEGER:
		type = JCTableEnum.TYPE_INTEGER;
		break;
	case Types.BIGINT:
	case Types.DOUBLE:
		type = JCTableEnum.TYPE_DOUBLE;
		break;
	case Types.CHAR:
	case Types.VARCHAR:
	case Types.LONGVARCHAR:
		type = JCTableEnum.TYPE_STRING;
		break;
	case Types.DECIMAL:
	case Types.FLOAT:
	case Types.NUMERIC:
	case Types.REAL:
		type = JCTableEnum.TYPE_FLOAT;
		break;
	case Types.DATE:
	case Types.TIME:
	case Types.TIMESTAMP:
		type = JCTableEnum.TYPE_DATE;
		break;
	default:
		type = JCTableEnum.TYPE_STRING;
	}

	return type;
}

/**
 * Establishes a connection to the data source.
 * 
 * @param driver the class name of the JDBC driver to use to
 * access your database (for example, the JDBC-ODBC driver
 * can be referenced as "com.ms.jdbc.odbc.JdbcOdbcDriver")
 * <BR><B>Note:</b> A valid driver must be passed in order to connect
 * to a database.
 * 
 * @param url a database url of the form 
 * <code> jdbc:<em>subprotocol</em>:<em>subname</em></code>
 * (for example, if
 * you've exposed the <code>JClassDemo</code> using an ODBC driver, a
 * valid URL is "<code>jdbc:odbc:JClassDemo</code>")
 * 
 * @param properties a list of arbitrary String tag/value pairs as
 * connection arguments; normally at least a <i>user</i> and
 * <i>password</i> property should be included
 * 
 * @return <code>True</code> if a connection was made successfully; <code>false</code> otherwise
 * 
 * @exception java.sql.SQLException  If a database access error occurred.
 */
public boolean connect(String driver, String url, Properties properties) throws SQLException {
	boolean success = false;

	try {
		Class.forName(driver).newInstance();
		this.dbcon = DriverManager.getConnection(url, properties);
		success = true;
	}
	catch(SQLException se) {
		throw(se);
	}
	catch(Exception e) {
	    System.out.println("Unable to load driver: " + e.getMessage());
	}

	return success;
}

}
