/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCSpreadLabel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:36 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table.data;

/** 
 * <code>JCSpreadLabel</code> contains routines to create a spreadsheet label.
 */
public class JCSpreadLabel {

static String symbols = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

/**
 *  Prints a base26 number, with 'length' digits.
 *  These are filled with 'A', the zero value.
 * @param cp
 * @param depth
 * @param length
 * @param value
 */
private static void base26(StringBuffer cp, int length, int depth, int value) {
	if (value > 25)	{
		base26(cp, length, depth+1, value/26);
	}
	else {
		while (depth++ < length) {
			cp.append(symbols.charAt(0));
		}
	}
	cp.append(symbols.charAt(value%26));
}

/**
 * Makes a spreadsheet column label. There are 26 labels of length 1,
 * 26*26 of length 2, etc. In proper base 26, there are 26 of length 1,
 * 26*26-26 of length 2, 26*26*26-26*26+26 of length 3, etc.  This
 * algorithm shifts the number back into the proper range,
 * calls base26 to format it with the correct number of digits.
 * @param value
 * @return
 */
private static String column_label(int value) {
	int i, num;
	StringBuffer buffer = new StringBuffer();

	for (i=0, num=0; value >= 26*(num+1); num=26*(num+1), i++) {
		;
	}
	base26(buffer, i+1, 1, value - num);
	return buffer.toString();
}

/**
 * Creates a row or column-style spreadsheet label. Rows start at
 * 1, and columns start at A.  Set row to a negative number to
 * convert column values, and vice versa.
 * @param row the row number to convert
 * @param column the column number to convert
 * @return Spreadsheet-style row or column label
 */
public static String create(int row, int column) {
	/* Row label */
	if (column < 0) {
		return Integer.toString(row+1);
	}
	/* Column label */
	else {
		return column_label(column);
	}
}

}
