/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCTableModelDataSource.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:36 $ $Locker:  $  Quest Software Inc.


package com.klg.jclass.table.data;

import com.klg.jclass.table.EditableTableDataModel;
import com.klg.jclass.table.JCTableDataListener;
import com.klg.jclass.table.JCTableDataEvent;
import com.klg.jclass.table.data.AbstractDataSource;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

/**
 * The <CODE>JCTableModelDataSource</CODE> class defines a wrapper for <code>JTable</code>
 * <code>TableModel</code> objects.
 */
public class JCTableModelDataSource extends AbstractDataSource
	implements EditableTableDataModel, TableModelListener {

protected TableModel model = null;

/**
 * Creates a new <CODE>JCTableModelDataSource</CODE> object containing a Swing
 * <CODE>TableModel</CODE> object.
 * @param model
 */
public JCTableModelDataSource(TableModel model) {
	this.model = model;
	this.model.addTableModelListener(this);
}

/**
 * Returns the current <code>TableModel</code> instance assiciated with the class.
 * @return the <code>TableModel</code> instance if it is available; null otherwise
 */
public TableModel getTableModel() {
	return model;
}

/**
 * Returns the number of rows of data that the data source contains.
 * @return the number of rows in the data model if it is available; 0 otherwise
 */
public int getNumRows() {
	return (model == null) ? 0 : model.getRowCount();
}

/**
 * Returns the number of columns of data that the data source contains.
 * @return the number of columns in the data model, if it is available; 0 otherwise
 */
public int getNumColumns() {
	return (model == null) ? 0 : model.getColumnCount();
}

/**
 * Returns the value for the row label at the specified row. Because <code>TableModel</code> does not
 * support row labels, return value is always an empty String.
 * @param row the row number
 * @return an empty String
 */
public Object getTableRowLabel(int row) {
	return "";
}

/**
 * Returns the value for the column label at the specified column.
 * @param column the column number
 * @return the columns label in the data model, if it is available; an empty String otherwise
 */
public Object getTableColumnLabel(int column) {
	return (model == null) ? "" : model.getColumnName(column);
}

/**
 * Returns the value for the cell at the specified location.
 * Must not be called if the data source is not enabled.
 * @param row the row number
 * @param column the column number
 * @return the cell value in the data model;
 */
public Object getTableDataItem(int row, int column) {
	return model.getValueAt(row, column);
}

/**
 * Sets a value in the data source. Must not be called if the
 * data source is not enabled.
 * @param o the cell value in the data model
 * @param row the row number
 * @param column the column number
 * @return boolean <code>true</code>
 */
public boolean setTableDataItem(Object o, int row, int column) {
	model.setValueAt(o, row, column);
	return true;
}

/**
 * Determines whether the cell can be edited.
 * @return <code>true</code> if the data model is available; <code>false</code> otherwise
 */
public boolean isEnabled() {
	return model != null;
}

/**
 * Determines whether the cell can be edited. Must not be called
 * if the data source is not enabled.
 * @param row the row number
 * @param column the column number
 * @return <code>true</code> if the cell can be edited; <code>false</code> otherwise
 */
public boolean isCellEditable(int row, int column) {
     return model.isCellEditable(row, column);
}

/**
 * Maps <code>TableModelEvents</code> to <code>TableDataEvents</code> based on the information
 * about the exact range of cells, rows, or columns that changed.
 * @param e the <code>TableModelEvent</code> instance
 */
public void tableChanged(TableModelEvent e) {
	int firstRow = e.getFirstRow();
    if (firstRow == TableModelEvent.HEADER_ROW) {
		this.fireDataReset();
        return;
	}

	int type = e.getType();
	int column = e.getColumn();
    int numRows = e.getLastRow()-firstRow+1;
	if (type == TableModelEvent.UPDATE) {
        if (column == TableModelEvent.ALL_COLUMNS) {
            fireTableDataEvent(new JCTableDataEvent(this, 0, column, this.getNumColumns(), 0, JCTableDataEvent.CHANGE_COLUMN));
        }
        else {
            if (numRows == 1) {
				this.fireValueChanged(firstRow, column);
            }
            else {
	            fireTableDataEvent(new JCTableDataEvent(this, firstRow, 0, numRows, 0, JCTableDataEvent.CHANGE_ROW));
            }
        }
    }
	else if (type == TableModelEvent.INSERT || type == TableModelEvent.DELETE) {
        fireNumRowsChanged();
    }
}

}


