/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCURLDataSource.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:36 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table.data;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * A simple class that uses a URL to create a data source object.
 * @see JCInputStreamDataSource
 */
public class JCURLDataSource extends JCInputStreamDataSource {

/**
 * @param url a String representing the location of the data file, for example,
 * <I>http://www.quest.com/data.file</I>
 * @exception MalformedURLException If the specified URL is not in a 
 * valid format.
 * @exception IOException If there is some problem reading the file.
 */
public JCURLDataSource(String url) throws MalformedURLException, IOException {
	super((new URL(url)).openStream());
}

/**
 * @param url a String representing the location of the data file, for example,
 * <I>http://www.quest.com/data.file</I>
 * @param prs the parser to use to interpret the incoming data
 * @exception MalformedURLException If the specified URL is not in a 
 * valid format.
 * @exception IOException If there is some problem reading the file.
 */
public JCURLDataSource(String url, JCFileFormatParser prs) 
					throws MalformedURLException, IOException {
	super((new URL(url)).openStream(), prs);
}

/**
 * @param host the WWW hostname
 * @param file the fully-qualified name of the file on the server
 * @exception MalformedURLException If the specified URL is not in a 
 * valid format.
 * @exception IOException If there is some problem reading the file.
 */
public JCURLDataSource(String host, String file) throws MalformedURLException, IOException {
	super((new URL("http",host,file)).openStream());
}

/**
 * @param host the WWW hostname
 * @param file the fully-qualified name of the file on the server
 * @param prs the parser to use to interpret the incoming data
 * @exception MalformedURLException If the specified URL is not in a 
 * valid format.
 * @exception IOException If there is some problem reading the file.
 */
public JCURLDataSource(String host, String file, JCFileFormatParser prs) 
							throws MalformedURLException, IOException {
	super((new URL("http",host,file)).openStream(), prs);
}

/**
 * @param protocol the protocol to use to access the file (currently only
 * http is supported)
 * @param host the WWW hostname
 * @param file the fully-qualified name of the file on the server
 * @exception MalformedURLException If the specified URL is not in a 
 * valid format.
 * @exception IOException If there is some problem reading the file.
 */
public JCURLDataSource(String protocol, String host, String file)
	throws MalformedURLException, IOException {
	super((new URL(protocol,host,file)).openStream());
}

/**
 * @param protocol the protocol to use to access the file (currently only
 * http is supported)
 * @param host the WWW hostname
 * @param file the fully-qualified name of the file on the server
 * @param prs the parser to use to interpret the incoming data
 * @exception MalformedURLException If the specified URL is not in a 
 * valid format.
 * @exception IOException If there is some problem reading the file.
 */
public JCURLDataSource(String protocol, String host, String file,
											JCFileFormatParser prs)
	throws MalformedURLException, IOException {
	super((new URL(protocol,host,file)).openStream(), prs);
}

/**
 * @param base a URL object representing the directory in which to find
 * the file
 * @param file the name of the file on the server
 * @exception MalformedURLException If the specified URL is not in a 
 * valid format.
 * @exception IOException If there is some problem reading the file.
 * @see java.net.URL
 */
public JCURLDataSource(URL base, String file) throws MalformedURLException, IOException {
	super((new URL(base,file)).openStream());
}

/**
 * @param base a URL object representing the directory in which to find
 * the file
 * @param file the name of the file on the server
 * @param prs the parser to use to interpret the incoming data
 * @exception MalformedURLException If the specified URL is not in a 
 * valid format.
 * @exception IOException If there is some problem reading the file.
 * @see java.net.URL
 */
public JCURLDataSource(URL base, String file, JCFileFormatParser prs) throws MalformedURLException, IOException {
	super((new URL(base,file)).openStream(), prs);
}

}
