/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCXMLFormatParser.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:36 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.table.data;

import com.klg.jclass.util.xml.JCTablePopulator;
import com.klg.jclass.util.xml.JCTableXMLParser;

import java.io.IOException;
import java.io.Reader;
import java.util.Vector;

/**
 * This class takes in a stream of data and parses
 * it under the assumption it is in the defined XML
 * format JClass LiveTable uses.  This format is defined in
 * <i>com.klg.jclass.util.xml.JCTableXMLParser</i>.
 *
 */
public class JCXMLFormatParser implements JCFileFormatParser, JCTablePopulator {

/**
 * The data source to populate with the data.
 */
protected JCVectorDataSource dataSource = null;

/**
 * Reads and parses data in the defined XML format and places it
 * inside a LiveTable data source.
 *
 * @param r reader open on an active stream
 * @param ds <code>JCVectorDataSource</code> instance to act as a
 * container for the data read in from r
 * @throws IOException
 */
public void readAndParse(Reader r, JCVectorDataSource ds) throws IOException
{
    dataSource = ds;
    JCTableXMLParser parser = new JCTableXMLParser();
    parser.readAndParse(r, this);
}

/**
 * Populates the table with the given data.  Implemented from the
 * <code>JCTablePopulator</code> interface.
 * @param rowV a vector of vectors, where each inside vector represents
 *             one row in the table
 * @param rowLabelV a vector containing the row labels for this table
 * @param colLabelV a vector containing the column labels for this table
 * @param nrows the number of rows in this table
 * @param ncols the number of columns in this table
 */
public void setTableData(Vector rowV, Vector rowLabelV, Vector colLabelV,
                         int nrows, int ncols)
{
    for (int i = 0; i < colLabelV.size(); i++) {
        dataSource.getColumnLabels().addElement(colLabelV.elementAt(i));
    }
    for (int i = 0; i < rowLabelV.size(); i++) {
        dataSource.getRowLabels().addElement(rowLabelV.elementAt(i));
    }
    for (int i = 0; i < rowV.size(); i++) {
        dataSource.getCells().addElement(rowV.elementAt(i));
    }
    dataSource.setNumRows(nrows);
    dataSource.setNumColumns(ncols);
}

}
