/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: DataTable.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:36 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.table.db;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Component;

import java.util.Enumeration;
import java.util.StringTokenizer;

import com.klg.jclass.cell.JCCellEditor;
import com.klg.jclass.cell.JCCellRenderer;
import com.klg.jclass.cell.editors.*;
import com.klg.jclass.cell.renderers.JCImageCellRenderer;
import com.klg.jclass.cell.renderers.JCCheckBoxCellRenderer;

import com.klg.jclass.datasource.ColumnModel;
import com.klg.jclass.datasource.DataModel;
import com.klg.jclass.datasource.DataModelException;
import com.klg.jclass.datasource.DataTableModel;
import com.klg.jclass.datasource.MetaDataModel;
import com.klg.jclass.datasource.VirtualColumnModel;
import com.klg.jclass.datasource.BindingModel;

import com.klg.jclass.table.JCCellStyle;
import com.klg.jclass.table.JCCellPosition;
import com.klg.jclass.table.JCCellRange;
import com.klg.jclass.table.JCTableEnum;
import com.klg.jclass.table.JCTraverseCellEvent;
import com.klg.jclass.table.JCTraverseCellListener;
import com.klg.jclass.table.TableDataModel;
import com.klg.jclass.table.TablePopupMenu;
import com.klg.jclass.table.beans.LiveTable;
import com.klg.jclass.table.data.AbstractDataSource;
import com.klg.jclass.table.data.JCVectorDataSource;
import com.klg.jclass.table.resources.LocaleBundle;

import com.klg.jclass.util.swing.JCMessageHelper;


/**
 * This is data aware <code>Table</code> subclass.
 * It displays data from the data source and
 * allows to change the data and save them back to the data source.
 * This component navigates the result set of a data bean and can insert and delete
 * records, cancel, save and requery data in the data source.
 * The class is not a bean, but a common ancestor of all data aware table beans.
 */
public class DataTable extends LiveTable implements JCTraverseCellListener {

protected boolean useDatasourceEditable = true;

// List of strings for the popup menu
final static public String[] commands = new String[] {
	LocaleBundle.string(LocaleBundle.INSERT_ROW),
	LocaleBundle.string(LocaleBundle.DELETE_ROWS),
	LocaleBundle.string(LocaleBundle.CANCEL_ROWS),
	LocaleBundle.string(LocaleBundle.CANCEL_ALL),
    LocaleBundle.string(LocaleBundle.REQUERY_ROW_AND),
	LocaleBundle.string(LocaleBundle.REQUERY_ALL),
	LocaleBundle.string(LocaleBundle.COMMIT_ROWS),
	LocaleBundle.string(LocaleBundle.COMMIT_ALL),
};

/**
 * Protected <code>DataTable</code> constructor.
 * The constructor must not be called
 * directly, but only from it's subclasses.
 */
protected DataTable() {
	super();

	// Sets the alignment for row labels centered
//	setAlignment(JCTableEnum.ALL, JCTableEnum.LABEL, JCTableEnum.MIDDLECENTER);

	setCurrentCell(0,0,JCTableEnum.TRAVERSE_RIGHT);	// ???

    // Selection policy supports rows selection
    setSelectionPolicy(JCTableEnum.SELECT_RANGE);

	// Turns off select indication for labels
	setSelectIncludeLabels(false);

	// Turns on the popup menu
    setPopupMenuEnabled(true);

	// Disables popup menu items by default
    updatePopupMenu();

    addTraverseCellListener(this);
}

/**
 * Creates data binding specific portion of the popup menu.
 */
protected TablePopupMenu createPopupMenu() {
	TablePopupMenu popupMenu = super.createPopupMenu();
    popupMenu.addSeparator();
	for (int i=0; i<commands.length; i++) {
    	popupMenu.addItem(commands[i], this);
    }
	return popupMenu;
}

/**
 * Handles the popup menu commands.
 */
public void actionPerformed(ActionEvent event) {
	super.actionPerformed(event);
	String command = event.getActionCommand();
    if (command.equals(commands[0]))
        insert();		// Adds a new record in the current table.
    else if (command.equals(commands[1]))
        delete();  		// Removes selected rows in the current table
    else if (command.equals(commands[2]))
        cancelRows(); 	// Cancels selected rows in the current table
    else if (command.equals(commands[3]))
        cancelAll();	// Cancels all edits made
    else if (command.equals(commands[4]))
        requeryRows();   // Requeries current row and details from the database
    else if (command.equals(commands[5]))
        requeryAll();      // Requeries all data from the database
    else if (command.equals(commands[6]))
        commitRows();	// Updates selected rows in the current table
    else if (command.equals(commands[7]))
        commitAll();	// Updates all changes made
}

/**
 * Overwrites the superclass <code>setDataSource</code>
 * to support enabling/disabling menu items.
 */
public void setDataSource(TableDataModel dataSource) {
    super.setDataSource(dataSource);
    updatePopupMenu();
}

/**
 * Binds the table to the given <code>DataModel</code> and to the
 * root <code>MetaDataModel</code>.
 */
public void setDataSource(DataModel dataModel) {
    initDataBinding(new DataBoundSource(this, dataModel));
}

/**
 * Binds the table to the given
 * <code>DataModel</code> and <code>MetaDataModel</code>.
 */
public void setDataSource(DataModel dataModel, MetaDataModel metaDataModel) {
    initDataBinding(new DataBoundSource(this, dataModel, metaDataModel));
}

/**
 * Sets the <code>DataBinding</code> property value to the table.
 *
 * Sets a table datasource for the data binding hierarchy string,
 * starting from a <code>JCTreeData</code>
 * instance name and containing full sequence of
 * <code>MetaDataModel</code> levels starting from the root
 * to the level the table is bound with.
 *
 * @see #getDataBinding
 */
protected void setDataSource(String dataBinding) {
    initDataBinding(new DataBoundSource(this, dataBinding));
}

/**
 * Sets the data source to a DataModel and binds
 * the <code>MetaData</code> to
 * the <code>MetaDataModel</code> represented by
 * a "path" of <code>MetaDataModel</code>
 * descriptions separated by '|' (e.g. Orders|Customers).
 */
protected void setDataSource(DataModel dataModel, String metaDescription) {
    initDataBinding(new DataBoundSource(this, dataModel, metaDescription));
}

/**
 * Bind using a <code>Binding</code>
 */
protected void setDataSource(BindingModel binding) {
    initDataBinding(new DataBoundSource(this, binding));
}

/**
 * Verifies that a new data source is valid and binds the table.
 */
protected void initDataBinding(DataBoundSource dataSource) {
    if (dataSource == null || !dataSource.dataBinding.isEnabled()) {
    	return;
	}
	TableDataModel td = getDataSource();
	if(td != null && td instanceof AbstractDataSource) {
	    ((AbstractDataSource)td).dispose();
	}
	// wipe out any existing renderers before setting the datasource
//	setCellRenderer(JCTableEnum.ALLCELLS, JCTableEnum.ALLCELLS, null);
	setDataSource(dataSource);
	updateEditors();
}

/**
 * Returns the table's value of the <code>DataBinding</code> object property.
 * Returns the data binding hierarchy string, starting from a
 * <code>JCTreeData</code> instance name
 * and containing full sequence of <code>MetaDataModel</code>
 * levels starting from the root to the level
 * the table is bound with.
 *
 * <code>DataBinding</code> property specifies a string,
 * describing a <code>JCTreeData</code> instance and
 * meta data level of the instance this table is bound with.
 * The format of the property
 * is as follows:
 * <pre>
 *   &lt;JCTreeData name&gt;:&lt;MetaDataModel description&gt;[|&lt;MetaDataModel description&gt;...]
 * </pre>
 *
 * @see #setDataBinding
 * @see com.klg.jclass.datasource.beans.JCTreeData
 * @see com.klg.jclass.datasource.MetaDataModel
 */
protected String getDataBinding() {
	DataBoundSource dbs = getDataBeanSource();
	return dbs != null ? dbs.getDataBinding() : "";
}

/**
 * Sets up parameters to bind the <code>DataBoundSource</code>
 * to a <code>MetaDataModel</code> in the
 * given <code>DataModel</code>.
 */
protected boolean setDataBinding(DataModel dataModel, MetaDataModel metaDataModel) {
	DataBoundSource dbs = getDataBeanSource();
	if (dbs != null) {
		dbs.dataBinding.setDataBinding(metaDataModel.getBinding());
        return true;
    }
    return false;
}

/**
 * Enables/disables the popup menu items, according to the current
 * state of the data source.
 */
void updatePopupMenu() {
	if (popupMenu == null) {
    	return;
    }
	DataBoundSource dbs = getDataBeanSource();
    boolean enabled = dbs != null && dbs.dataBinding.isEnabled();
    boolean available = enabled && dbs.dataBinding.isDataAvailable();
    boolean positioned = available && dbs.dataBinding.getNumRows()>0 &&
    	getSelectedCells() != null;
	popupMenu.enableMenuItem(LocaleBundle.INSERT_ROW, available);
	popupMenu.enableMenuItem(LocaleBundle.DELETE_ROWS, positioned);
	popupMenu.enableMenuItem(LocaleBundle.CANCEL_ROWS, positioned);
	popupMenu.enableMenuItem(LocaleBundle.CANCEL_ALL, enabled);
    popupMenu.enableMenuItem(LocaleBundle.REQUERY_ROW_AND, positioned);
	popupMenu.enableMenuItem(LocaleBundle.REQUERY_ALL, enabled);
	popupMenu.enableMenuItem(LocaleBundle.COMMIT_ROWS, positioned);
	popupMenu.enableMenuItem(LocaleBundle.COMMIT_ALL, enabled);
}

/**
 * Cancel the editing and updates the table state, because the data source data
 * table was changed.
 */
protected void tableChanged() {
	cancelEdit(true);
    clearSelectedCells();
    updatePopupMenu();
    updateEditors();
}

/**
 * Saves the value from the current editor to the data source before
 * the current row has been changed.
 */
protected boolean beforeCurrentRowChange() {
	return commitEdit(true);
}

/**
 * Traverses to another row according to the current data source row position.
 */
void setCurrentCell(int row) {
	JCCellPosition currentCell = getCurrentCell();
    if (getCurrentRow() != row) {
	    traverse(row != -1 ? row : 0,
        	currentCell != null ? currentCell.column : 0, false, false);
    }
}

/*
 * Intertnal <code>getSelection()</code> with en error handling.
 */
protected JCCellRange getSelection() {
	JCCellRange range = getSelectedRange();
	if (range == null) {
    	JCMessageHelper.showError("DataTable Error", "No rows selected.");
        return null;
    }
	clearSelectedCells();
    return range;
}

/*
 * Adds a new record in the current table.
 * The same as the add button.
 */
protected void insert() {
	((DataBoundSource)getDataSource()).dataBinding.addRow();
}

/*
 * Removes the current record in the current table.
 * The same as the delete button.
 */
protected void delete() {
	JCCellRange range = getSelection();
	if (range != null) {
		setRepaintEnabled(false);
		((DataBoundSource)getDataSource()).dataBinding.deleteRows(range.start_row,
    		range.end_row-range.start_row+1);
		setRepaintEnabled(true);
    }
}

/*
 * Cancels the changes made in the selected rows.
 */
protected void cancelRows() {
	JCCellRange range = getSelection();
	if (range != null) {
		setRepaintEnabled(false);
		((DataBoundSource)getDataSource()).dataBinding.cancelRows(range.start_row,
    		range.end_row-range.start_row+1);
		setRepaintEnabled(true);
    }
}

/*
 * Cancels all the changes made in the data source.
 */
protected void cancelAll() {
	((DataBoundSource)getDataSource()).dataBinding.cancelAll();
}

/*
 * Requeries the selected rows and details from the database.
 */
protected void requeryRows() {
	JCCellRange range = getSelection();
	if (range != null) {
		setRepaintEnabled(false);
		((DataBoundSource)getDataSource()).dataBinding.requeryRowsAndDetails(range.start_row,
    		range.end_row-range.start_row+1);
		setRepaintEnabled(true);
    }
}

/*
 * Requeries the dataset from the database.
 */
protected void requeryAll() {
	((DataBoundSource)getDataSource()).dataBinding.requeryAll();
}

/*
 * Commits the selected rows in the database.
 */
protected void commitRows() {
	JCCellRange range = getSelection();
	if (range != null) {
		setRepaintEnabled(false);
		((DataBoundSource)getDataSource()).dataBinding.commitRows(range.start_row,
    		range.end_row-range.start_row+1);
    	setRepaintEnabled(true);
    }
}

/*
 * Commits all changes made in the data source.
 */
protected void commitAll() {
	((DataBoundSource)getDataSource()).dataBinding.commitAll();
}

/**
 * Invoked before a cell is traversed to. The event's <CODE>setCancelled</CODE>
 * method can be called to prevent the traversal or the destination cell
 * can be modified.
 * @see JCTraverseCellEvent#setCancelled
 * @see JCTraverseCellEvent#setNextRow
 * @see JCTraverseCellEvent#setNextColumn
 */
public void traverseCell(JCTraverseCellEvent e) {
    int row = e.getNextRow();
	DataBoundSource dbs = getDataBeanSource();
	if (dbs != null && dbs.dataBinding.getCurrentRowIndex() != row) {
        if (!dbs.dataBinding.moveToRow(row)) {
            e.setCancelled(true);
        }
    }
}

/**
 * Invoked after the cell is traversed to if all listeners notified through
 * <CODE>traverseCell</CODE> accepted the destination. If a listener
 * cancelled the traversal in <CODE>traverseCell</CODE>,  afterTraverseCell will
 * be called with the cancelled flag set and the destination the same as the
 * source location.
 * @see #traverseCell
 * @see JCTraverseCellEvent#isCancelled
 */
public void afterTraverseCell(JCTraverseCellEvent e) {
	updatePopupMenu();
}

/**
 * Returns DataBoundSource for the table.
 */
protected DataBoundSource getDataBeanSource() {
	TableDataModel dataSource = getDataSource();
	return (dataSource != null && dataSource instanceof DataBoundSource) ?
		(DataBoundSource)dataSource : null;
}

/**
 * Returns the IDE specific metadata object bound to the table.
 */
protected Object getNativeMetaData() {
	DataBoundSource dbs = getDataBeanSource();
	return dbs != null ? dbs.getNativeMetaData() : null;
}

/**
 * Sets the useDatasourceEditable property.
 * The property determines whether the editable column state
 * is defined by the datasource or by the table.
 * @see #getUseDatasourceEditable
 */
public void setUseDatasourceEditable(boolean b) {
	useDatasourceEditable = b;
	cancelEdit(true);
    updateEditors();
}

/**
 * Gets the <code>useDatasourceEditable</code> property.
 * @see #setUseDatasourceEditable
 */
public boolean getUseDatasourceEditable() {
	return useDatasourceEditable;
}

/**
 * Updates cell editors according to the datasource table column types.
 */
protected void updateEditors() {
	DataBoundSource dataSource = getDataBeanSource();
	if (dataSource == null ||
    	!dataSource.dataBinding.isEnabled() || !dataSource.dataBinding.isDataAvailable())
		return;

	for (int column = 0; column < getNumColumns(); column++) {
		String columnName = (String)dataSource.getTableColumnLabel(column);

        JCCellEditor editor = null;
        JCCellRenderer renderer = null;
        switch (dataSource.getJavaColumnType(columnName)) {
            case MetaDataModel.TYPE_BOOLEAN:
                editor = new JCCheckBoxCellEditor();
                renderer = new JCCheckBoxCellRenderer();
                break;
            case MetaDataModel.TYPE_SHORT:
                editor = new JCIntegerCellEditor();
                break;
            case MetaDataModel.TYPE_BYTE:
                editor = new JCIntegerCellEditor();
                break;
            case MetaDataModel.TYPE_INTEGER:
                editor = new JCIntegerCellEditor();
                break;
            case MetaDataModel.TYPE_LONG:
                editor = new JCLongCellEditor();
                break;
            case MetaDataModel.TYPE_DOUBLE:
                editor = new JCDoubleCellEditor();
                break;
            case MetaDataModel.TYPE_FLOAT:
                editor = new JCDoubleCellEditor();
                break;
            case MetaDataModel.TYPE_BIG_DECIMAL:
                editor = new JCBigDecimalCellEditor();
                break;
            case MetaDataModel.TYPE_SQL_DATE:
                editor = new JCDateCellEditor();
                break;
            case MetaDataModel.TYPE_SQL_TIME:
                editor = new JCDateCellEditor();
                break;
            case MetaDataModel.TYPE_SQL_TIMESTAMP:
                editor = new JCDateCellEditor();
                break;
            case MetaDataModel.TYPE_UTIL_DATE:
                editor = new JCDateCellEditor();
                break;
            case MetaDataModel.TYPE_BYTE_ARRAY: // assumes all binary data is image type
                renderer =  new JCImageCellRenderer();
                break;
            case MetaDataModel.TYPE_STRING:
            case MetaDataModel.TYPE_OBJECT:
            default:
                editor = new JCStringCellEditor();
        }
        JCCellStyle style = (JCCellStyle)getCellStyle(JCTableEnum.ALLCELLS, column).clone();
        if (useDatasourceEditable) {
	        style.setEditable(dataSource.dataBinding.isColumnEditable(columnName));
        }
        if (editor != null) {
//	        setCellEditor(JCTableEnum.ALLCELLS, column, editor);
	        style.setCellEditor(editor);
		}
        if (renderer != null) {
//	        setCellRenderer(JCTableEnum.ALLCELLS, column, renderer);
	        style.setCellRenderer(renderer);
		}
        setCellStyle(JCTableEnum.ALLCELLS, column, style);
	}
}

/**
 * Returns the class of a given column in the datasource.
 */
protected Class getColumnClass(int column) {
    DataBoundSource dataSource = getDataBeanSource();
	if(dataSource == null)
	    return null;
	try {
	    String columnName = (String)dataSource.getTableColumnLabel(column);
	    switch(dataSource.getJavaColumnType(columnName)) {
		case MetaDataModel.TYPE_BOOLEAN:
		    return Class.forName("java.lang.Boolean");
		case MetaDataModel.TYPE_SHORT:
		    return Class.forName("java.lang.Short");
		case MetaDataModel.TYPE_BYTE:
		    return Class.forName("java.lang.Byte");
		case MetaDataModel.TYPE_INTEGER:
		    return Class.forName("java.lang.Integer");
		case MetaDataModel.TYPE_LONG:
		    return Class.forName("java.lang.Long");
		case MetaDataModel.TYPE_DOUBLE:
		    return Class.forName("java.lang.Double");
		case MetaDataModel.TYPE_FLOAT:
		    return Class.forName("java.lang.Float");
		case MetaDataModel.TYPE_BIG_DECIMAL:
		    return Class.forName("java.math.BigDecimal");
		case MetaDataModel.TYPE_SQL_DATE:
		    return Class.forName("java.sql.Date");
		case MetaDataModel.TYPE_SQL_TIME:
		    return Class.forName("java.sql.Time");
		case MetaDataModel.TYPE_SQL_TIMESTAMP:
		    return Class.forName("java.sql.Timestamp");
		case MetaDataModel.TYPE_BYTE_ARRAY: // assumes all binary data is image type
			return null;
		case MetaDataModel.TYPE_STRING:
		default:
		    return Class.forName("java.lang.String");
		}
	} catch(Exception e) {
	    return null;
	}
}


/*
 * Populates the table with some data to ensure initial
 * dimensions while it's not bound.
 */
protected void makeDataSource() {
	JCVectorDataSource dataSource = new JCVectorDataSource();
	dataSource.setNumRows(getNumRows());
	dataSource.setNumColumns(getNumColumns());
	for(int c = 0; c < dataSource.getNumColumns(); c++)
		dataSource.setColumnLabel(c, "Column: "+c);
	for(int r = 0; r < dataSource.getNumRows(); r++)
		dataSource.setRowLabel(r, "Row: "+r);
	setDataSource(dataSource);
}

}	// DataTable

