/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: TableDataBinding.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:36 $ $Locker:  $  Quest Software Inc. 

package com.klg.jclass.table.db;

import com.klg.jclass.datasource.util.NavigatorDataBinding;
import com.klg.jclass.datasource.DataModelException;


/**
 * This is the core data binding handling class for
 * LiveTable.  Most of the key functionality is in
 * NavigatorDataBinding.
 */
class TableDataBinding extends NavigatorDataBinding {

/** Data-bound table instance.  */
DataTable dataTable;

/** Data-bound table data source */
DataBoundSource dataSource;

/**
 * Creates a data binding object give an DataTable and
 * its corresponding data source
 * @param table data-bound table instance
 * @param dataSource external data source for data-bound table.
 */
TableDataBinding(DataTable table, DataBoundSource dataSource) {
	super(table);
	this.dataSource = dataSource;
    dataTable = table;
}


/**
 * Returns current zero-based row index for the DataTableModel instance.
 * @return -1 if current bookmark is invalid.
 */
public int getCurrentRowIndex() {
    return super.getCurrentRowIndex();
}

/**
 * Changes the current row in the component.
 */
protected void currentRowChanged(int rowIndex) {
	dataTable.setCurrentCell(rowIndex);
}

/**
 * Checks if all listeners have accepted the move.
 */
protected boolean beforeCurrentRowChange() {
	return dataTable.beforeCurrentRowChange();
}

/**
 * Process a move to row for any metadata level called from dataModelChanged()
 */
protected void tableChanged() {
	dataSource.fireDataReset();
    dataTable.tableChanged();
}

/**
 * Must be called after row data were changed.
 */
protected void rowDataChanged(int rowIndex) {
	dataSource.fireRowChanged(rowIndex);
	dataSource.fireRowLabelChanged(rowIndex);
}

/**
 * Must be called after a row was inserted.
 */
protected void rowInserted() {
	dataSource.fireNumRowsChanged();
    dataTable.updatePopupMenu();
}

/**
 * Must be called after a row was deleted.
 */
protected void rowDeleted() {
	dataSource.fireNumRowsChanged();
    dataTable.updatePopupMenu();
}

/**
 * Must be called after a single cell value was changed.
 */
protected void cellDataChanged(int row, String columnName) {
//    try {
        int column = binding.getColumnIndex(columnName);
        dataSource.fireValueChanged(row, column);
		dataSource.fireRowLabelChanged(row);
//    } catch(DataModelException dme) {
//        dme.printStackTrace();
//    }
}

/**
 * Sets DataModel current row.
 */
boolean moveToRow(int row) {
	return super.moveToRowIndex(row);
}

}	// TableDataBinding

