/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Changeable.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:37 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util;

/**
 * The Changeable interface is used as a template for
 * objects that have a changed flag or changed semantic
 * involving some kind of recalculation.  This is very
 * common inside JCChart.  All ChartRegion and 
 * ChartInteriorRegion derived objects 
 * make use of Changeable, either by directly 
 * implementing the interface or by deriving from TrackChange
 * (an abstract class that implements Changeable).
 */
public interface Changeable {
  
// Mask values to be used with the ChangedFlag property through setChanged(),
// getChanged(), and isChanged().
public final static int NO_CHANGE            = 0;
public final static int REDRAW               = 1;
public final static int RECALC               = 2;
public final static int LAYOUT               = 4;
public final static int NOTIFY_PARENT_LAYOUT = 8;
public final static int BASIC_CHANGE_MASK    = REDRAW | RECALC;

/**
 * Gets the value of the Changed property.
 * The Changed property determines whether the object needs
 * to be recalculated.
 * @return True if the object has been changed and needs
 * recalculation, false otherwise.
 */
public boolean getChanged();

/**
 * Checks the value of the ChangedFlag property for the existence of
 * the specified mask.
 * The ChangedFlag property determines the type of action (recalc, redraw)
 * that needs to happen to the object.
 * @return True if the ChangedFlag contains the given mask (e.g. RECALC,
 * REDRAW, etc.), false otherwise.
 */
public boolean getChanged(int mask);

/**
 * Gets the value of the Changed property.
 * The Changed property determines whether the object needs
 * to be recalculated.
 * @return True if the object has been changed and needs
 * recalculation, false otherwise.
 */
public boolean isChanged();

/**
 * Checks the value of the ChangedFlag property for the existence of
 * the specified mask.
 * The ChangedFlag property determines the type of action (recalc, redraw)
 * that needs to happen to the object.
 * @return True if the ChangedFlag contains the given mask (e.g. RECALC,
 * REDRAW, etc.), false otherwise.
 */
public boolean isChanged(int mask);

/**
 * Sets the value of the Changed property and updates the
 * parent object.
 * The Changed property determines whether the object needs
 * to be recalculated.
 * @param n New value for Changed.
 * @param cf Or-ed series of bit flags indicating type of change.
 */
public void setChanged(boolean n, int cf);

/**
 * Sets the value of the Changed property.
 * The Changed property determines whether the object needs
 * to be recalculated.
 * @param n New value for Changed.
 * @param cf Or-ed series of bit flags indicating type of change.
 * @param updateParent If true, the parent is updated.  Otherwise,
 * the parent is not updated.  The "update" semantics are up to the
 * implementor
 */
public void setChanged(boolean n, int cf, boolean updateParent);


/* DAM comments */
public boolean isBatched();

}
