/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ImageMapInfo.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:38 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util;

import java.io.Serializable;

/**
 * This class contains information for image maps for a given entity (point,
 * cluster, series, header, footer, legend, etc).
 */
public class ImageMapInfo implements Serializable {

public final static int URL   = 0;
public final static int EXTRA = 1;

/** The URL for this image map entity. */
protected String  url = null;

/** The extra URL information for this image map entity. */
protected String  extra = null;

/**
 * No args constructor.
 */
public ImageMapInfo() {}

/**
 * Constructor which specifies the URL and extra URL information.
 * @param url The image map URL.
 * @param extra The image map extra URL information.
 */
public ImageMapInfo(String url, String extra)
{
    this.url = url;
    this.extra = extra;
}

/**
 * Retrieve the current URL for this image map entity.
 * @return The current URL.
 */
public String getUrl()
{
    return(url);
}

/**
 * Set the URL for this image map entity.
 * @param url The new URL.
 */
public void setUrl(String url)
{
    this.url = url;
}

/**
 * Retrieve the current extra URL information for this image map entity.
 * @return The current extra URL information,
 */
public String getExtra()
{
    return(extra);
}

/**
 * Set the extra URL information for this image map entity.
 * @param extra The new extra URL information.
 */
public void setExtra(String extra)
{
    this.extra = extra;
}

/**
 * Return the appropriate String based on the tag.
 * @param tag The integer tag.
 * @return The String based on the tag.
 */
public String getString(int tag)
{
    String s = null;
    if (tag == URL) {
        s = url;
    }
    else if (tag == EXTRA) {
        s = extra;
    }
    return(s);
}

/**
 * Is this object empty?
 * @return false, if one of the url or extra fields is non-empty.
 */
public boolean isEmpty()
{
    return((url == null || url.length() == 0) &&
           (extra == null || extra.length() == 0));
}

/**
 * Are all of the ImageMapInfo objects in the array empty?
 * @param info The array to check.
 * @return false, if one of the objects in the array is non-empty.
 */
public static boolean isEmpty(ImageMapInfo[] info)
{
    if (info != null) {
        for (int i = 0; i < info.length; i++) {
            if (info[i] != null && !info[i].isEmpty()) {
                return(false);
            }
        }
    }
    return(true);
}

/**
 * Are all of the ImageMapInfo objects in the array empty?
 * @param info The array to check.
 * @return false, if one of the objects in the array is non-empty.
 */
public static boolean isEmpty(ImageMapInfo[][] info)
{
    if (info != null) {
        for (int i = 0; i < info.length; i++) {
            if (info[i] != null && !ImageMapInfo.isEmpty(info[i])) {
                return(false);
            }
        }
    }
    return(true);
}

/**
 * Are all of the ImageMapInfo objects in the array empty?
 * @param info The array to check.
 * @return false, if one of the objects in the array is non-empty.
 */
public static boolean isEmpty(ImageMapInfo[][][] info)
{
    if (info != null) {
        for (int i = 0; i < info.length; i++) {
            if (info[i] != null && !ImageMapInfo.isEmpty(info[i])) {
                return (false);
            }
        }
    }
    return (true);
}

/**
 * Check to see if the object passed in is empty or contains only empty
 * ImageMapInfo objects.
 * @param info The object to check.  It must be of type ImageMapInfo or an
 * array of ImageMapInfo objects of one, two, or three dimensions.
 * @return false, if the object is non-empty or one of the ImageMapInfo objects
 * that the object references is non-empty.
 * @throws IllegalArgumentException if the passed in argument is not an
 * ImageMapInfo object or an array of ImageMapInfo objects of one, two, or
 * three dimensions.
 */
public static boolean isEmpty(Object info)
{
    if (info instanceof ImageMapInfo) {
        return(((ImageMapInfo)info).isEmpty());
    }
    else if (info instanceof ImageMapInfo[]) {
        return(ImageMapInfo.isEmpty((ImageMapInfo[])info));
    }
    else if (info instanceof ImageMapInfo[][]) {
        return(ImageMapInfo.isEmpty((ImageMapInfo[][])info));
    }
    else if (info instanceof ImageMapInfo[][][]) {
        return(ImageMapInfo.isEmpty((ImageMapInfo[][][])info));
    }
    else if (info != null) {
        throw new IllegalArgumentException("Argument must be of type " +
                                           "ImageMapInfo or an array of " +
                                           "ImageMapInfo objects of " +
                                           "dimension one, two, or three.");
    }
    return(true);
}

/**
 * Create an array of ImageMapInfo objects from an array of URL Strings and an
 * array of extra URL information Strings;
 * @param urls The array of URL Strings.
 * @param extras The array of extra URL information Strings.
 * @return A new array of ImageMapInfo objects which combines the two arrays.
 */
public static ImageMapInfo[] createImageMapInfoArray(String[] urls,
                                                     String[] extras)
{
    if (urls == null && extras == null) {
        return(null);
    }

    int sizeURLs   = (urls == null)   ? 0 : urls.length;
    int sizeExtras = (extras == null) ? 0 : extras.length;
    int maxSize = Math.max(sizeURLs, sizeExtras);

    ImageMapInfo[] imageMapInfoArray = new ImageMapInfo[maxSize];
    for (int i = 0; i < maxSize; i++) {
        String URL   = (urls == null || sizeURLs < maxSize) ? null : urls[i];
        String extra =
            (extras == null || sizeExtras < maxSize) ? null : extras[i];
        imageMapInfoArray[i] = new ImageMapInfo(URL, extra);
    }
    return(imageMapInfoArray);
}

/**
 * Create an doubly subscripted array of ImageMapInfo objects from doubly
 * subscripted arrays of URL Strings and extra URL information Strings;
 * @param urls A doubly subscripted array of URL Strings.
 * @param extras A doubly subscripted array of extra URL information Strings.
 * @return A new doubly subscripted array of ImageMapInfo objects which combines
 * the two doubly subscripted arrays.
 */
public static ImageMapInfo[][] createImageMapInfoArray(String[][] urls,
                                                       String[][] extras)
{
    if (urls == null && extras == null) {
        return(null);
    }

    int sizeURLs   = (urls == null)   ? 0 : urls.length;
    int sizeExtras = (extras == null) ? 0 : extras.length;
    int maxSize = Math.max(sizeURLs, sizeExtras);

    ImageMapInfo[][] imageMapInfoArray = new ImageMapInfo[maxSize][];
    for (int i = 0; i < maxSize; i++) {
        String[] URL = (urls == null || sizeURLs < maxSize) ? null : urls[i];
        String[] extra =
            (extras == null || sizeExtras < maxSize) ? null : extras[i];
        imageMapInfoArray[i] = createImageMapInfoArray(URL, extra);
    }
    return(imageMapInfoArray);
}

/**
 * Create an triply subscripted array of ImageMapInfo objects from triply
 * subscripted arrays of URL Strings and extra URL information Strings;
 * @param urls A triply subscripted array of URL Strings.
 * @param extras A triply subscripted array of extra URL information Strings.
 * @return A new triply subscripted array of ImageMapInfo objects which combines
 * the two triply subscripted arrays.
 */
public static ImageMapInfo[][][] createImageMapInfoArray(String[][][] urls,
                                                         String[][][] extras)
{
    if (urls == null && extras == null) {
        return(null);
    }

    int sizeURLs   = (urls == null)   ? 0 : urls.length;
    int sizeExtras = (extras == null) ? 0 : extras.length;
    int maxSize = Math.max(sizeURLs, sizeExtras);

    ImageMapInfo[][][] imageMapInfoArray = new ImageMapInfo[maxSize][][];
    for (int i = 0; i < maxSize; i++) {
        String[][] URL = (urls == null || sizeURLs < maxSize) ? null : urls[i];
        String[][] extra =
            (extras == null || sizeExtras < maxSize) ? null : extras[i];
        imageMapInfoArray[i] = createImageMapInfoArray(URL, extra);
    }
    return(imageMapInfoArray);
}

/**
 * Create an array of ImageMapInfo objects (of dimension 0, 1, 2, or 3) from
 * two string arrays of URL and extra URL information.
 * <p>
 * <b>Note:</b> The dimensions of arrays passed in to the urls and extras
 * arguments must match.
 * @param urls List of URL strings.  Should be an array (possibly
 * multi-dimensional) of String objects.
 * @param extras List of extra URL information strings.  Should be an array
 * (possibly multi-dimensional) of String objects.
 * @return A new array of image map objects which match the dimension of the
 * passed in parameters.
 */
public static Object createImageMapInfoArray(Object urls, Object extras)
{
    Object infoArray = null;
    if (urls == null && extras == null) {
        return(infoArray);
    }

    if ((urls instanceof String[][][] || urls == null) &&
         (extras instanceof String[][][] || extras == null))
    {
        infoArray = createImageMapInfoArray((String[][][])urls,
                                            (String[][][])extras);
    }
    else if ((urls instanceof String[][] || urls == null) &&
             (extras instanceof String[][] || extras == null))
    {
        infoArray = createImageMapInfoArray((String[][])urls,
                                            (String[][])extras);
    }
    else if ((urls instanceof String[] || urls == null) &&
             (extras instanceof String[] || extras == null))
    {
        infoArray = createImageMapInfoArray((String[])urls, (String[])extras);
    }
    else {
        infoArray = new ImageMapInfo(urls != null ? urls.toString() : null,
                                     extras != null ? extras.toString() : null);
    }
    return(infoArray);
}

public String toString()
{
    return("Url:" + url + " Extra:" + extra);
}

}
