/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCCSVParser.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:38 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util;

import java.io.Reader;
import java.io.IOException;
import java.io.StreamTokenizer;
import java.util.Vector;

/**
 * Utility class for loading comma-separated values.
 * Given a stream of the format:
 * <PRE>
 *  token1, token2
 *  token3, token4
 * </PRE>
 * This class is capapble of loading the stream into
 * a vector of vectors.
 */
public class JCCSVParser {

/** The stream tokenizer used to read the data. */
protected StreamTokenizer tokenizer;

/**
 * Configures the tokenizer to read the CSV stream.
 * Important settings include <code>whitespaceChars</code>, which
 * denotes "," as a whitespace character.  This is
 * important because <code>StreamTokenizer</code> uses whitespace
 * as a token delimiter.  Also important is the
 * <code>eolIsSignificant()</code> setting of <code>true</code>.
 * This returns EOL as a token that we can use to identify a new
 * row of data.
 * @param r
 */
protected void initTokenizer(Reader r)
{
	tokenizer = new StreamTokenizer(r);
	tokenizer.commentChar('#');
	tokenizer.quoteChar(JCFormatParserUtil.QUOTE);
	tokenizer.slashSlashComments(false);
	tokenizer.slashStarComments(false);
	tokenizer.eolIsSignificant(true);
	tokenizer.lowerCaseMode(false);
	tokenizer.parseNumbers();
	tokenizer.whitespaceChars(',', ',');
}

/**
 * This method reads the stream provided by <i>r</i> and
 * puts the parsed contents into the Vector <i>vv</i>.
 *
 * @param r an input reader
 * @param vv a Vector instance; if this is null,
 * no parsing will occur
 * @return
 * @throws IOException
 */
public int readAndParse(Reader r, Vector vv) throws IOException
{
	if (vv == null) {
        return(0);
    }
	initTokenizer(r);

	Object token = null;
	Vector v = new Vector();
	int numColumns = 0;
	try {
		while ((token = JCFormatParserUtil.getNextToken(tokenizer)) != null) {
			if (token.equals("" + StreamTokenizer.TT_EOL)) {
				numColumns = Math.max(numColumns, v.size());
				vv.addElement(v);
				v = new Vector();
				continue;
			}
			v.addElement(token);
		}
	}
	catch (IOException ioe) {}

	if (v != null && v.size() > 0) {
		// This catches the "last array" case
		vv.addElement(v);

		// This catches the 1 row no EOL case
		numColumns = Math.max(numColumns, v.size());
	}

	tokenizer = null;
	return(numColumns);
}

}
