/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCEnvironment.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:38 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util;

import javax.swing.*;
import java.applet.Applet;
import java.applet.AppletContext;
import java.awt.*;

/** Static routines to provide environment information. */
public class JCEnvironment {

public static final int WORKING_UNKNOWN = 0;
public static final int WORKING_YES     = 1;
public static final int WORKING_NO      = 2;

//
// Environnmental problems
//

/**
 * Swing bug where file chooser does not allow multiple selection of files
 * to work correctly.
 */
public static final String FILE_CHOOSER_MULTI_SELECTS =
    "FileChooserMultiSelects";

/**
 * Swing bug where Motif Tree Leaf Look and Feel Icon always draws from
 * <i>x = 0</i>, no matter what <i>x</i> value you pass it.
 */
public static final String MOTIF_TREE_LEAF_ICON = "MotifTreeLeafIcon";

/**
 * Motif bug where calling <code>setResizable(false)</code> causes a dialog to
 * grow without bound.
 */
public static final String SET_RESIZABLE_TO_FALSE_ON_DIALOGS =
    "SetResizableToFalseOnDialogs";

/**
 * Motif bug where calculating the prefered size of the spinbox renderer is
 * initialy calculated smaller then it needs to be once it is displayed.  This
 * will return an ajustment value that can be added to the width.
 **/

public static final String MOTIF_PREFERED_WIDTH_AJUSTMENT =
    "MotifPreferedWidthAjustment";

/**
 * Netscape bug that prevents HTML in Swing components from
 * appearing.
 */
public static final String HTML_IN_COMPONENTS = "HTMLInComponents";

/**
 * Calls to the Printable interface's <code>print()</code> method can happen
 * multiple times per page.  In some cases (jdk 1.4 & Windows for one), it is
 * the second call that has the real graphics class needed for correct
 * formatting based on printer font metrics.
 */
public static final String PRINTING_USE_2ND_CALL = "PrintingUse2ndCall";

/**
 * Refers to a selection bug in <code>JTable</code> in which the selection
 * model's anchor and lead do not get updated correctly if rows are added or
 * deleted from the table's model.  This manifests itself in
 * <code>JCTreeTable</code> because we add/delete rows every time we expand or
 * collapse a node.
 */
public static final String FOCUS_RECT_IN_JTABLE = "FocusRectInJTable";

protected static String javaVersionString = null;
protected static String vendorString = null;
protected static int javaVersion = -1;

/**
 * Returns <code>YES</code> or <code>NO</code> if the answer to the question is
 * known; otherwise it returns <code>UNKNOWN</code>.<p>
 * This will be extended in the future as platform specific bugs
 * appear, since many problems are specific to many versions or
 * many different platforms.  All knowledge of the problem will be
 * encasulated (hidden) in this class to make it easier to track and
 * implement multiple manifestations of the problem and to make the
 * workarounds more legible and maintainable.<p>
 * A String based lookup method is used so that the API remains
 * constant, even as this class handles more and more divergent cases.
 * @return <code>YES</code> or <code>NO</code> if the answer to the question is
 * known; otherwise
 * it returns <code>UNKNOWN</code>
 * @param feature
 */
public static int isWorking(String feature)
{
	if (feature == null) {
		throw new NullPointerException("Feature may not be null");
	}

	if (feature.equals(MOTIF_TREE_LEAF_ICON)) {
		return(WORKING_NO);
	}

	if (feature.equals(SET_RESIZABLE_TO_FALSE_ON_DIALOGS)) {
		return(canSetResizableToFalseOnDialogs());
	}

	if (feature.equals(MOTIF_PREFERED_WIDTH_AJUSTMENT)) {
		return(motifPreferredWidthAjustment());
	}

	if (feature.equals(HTML_IN_COMPONENTS)) {
		return(htmlInComponents());
	}

	if (feature.equals(FILE_CHOOSER_MULTI_SELECTS)) {
		return(fileChooserMultiSelects());
	}

	if (feature.equals(PRINTING_USE_2ND_CALL)) {
		return(printingUse2ndCall());
	}

    if (feature.equals(FOCUS_RECT_IN_JTABLE)) {
        return(focusRectInJTable());
    }

	return(WORKING_UNKNOWN);
}

/**
 * Returns the version of the JDK for easy numeric comparison.
 * Return value looks like the following: 120 for 1.2.0.<p>
 * An additional reason for this is that historically the vendor
 * provided version is sometimes inconsistent.
 * If the version String is not found, or if there is a parsing
 * error, then the return value is -1.
 * @return version of the JCK
 */
public static int getJavaVersion()
{
	if (javaVersion != -1) {
		return(javaVersion);
	}

	String s = getVersionString();
    if (s == null) {
        javaVersion = -1;
        return(javaVersion);
    }

	// Loop through returned version stripping all non-digits
	StringBuffer s1 = new StringBuffer();
	for (int i = 0; i < s.length(); i++) {
		char c = s.charAt(i);
		if (Character.isDigit(c))	{
			// only add digits
			s1.append(c);
		}
		// Don't want a beta version appended to number
		if (s1.length() >= 3) {
			break;
		}
	}
	try {
		// will create a version "115" for JDK 1.1.5
		javaVersion = Integer.parseInt(s1.toString());
		if (javaVersion > 0 && javaVersion < 100) {
			// Must only contain major release #
			javaVersion *= 10;
		}
	}
	catch (java.lang.NumberFormatException e) {
		// Need to catch the parse in case the VM returns no version info
		javaVersion = -1; // garbage version number
	}

	return(javaVersion);
}


/**
 * Retrieves and caches the version String.
 * @return version String
 */
public static String getVersionString()
{
	if (javaVersionString == null) {
		try {
			javaVersionString = System.getProperty("java.version");
		}
		catch (Exception e) {
			javaVersionString = null;
		}
	}

	return(javaVersionString);
}

/**
 * Is the current Look and Feel Motif?
 * @return true, if the current L&F is Motif.
 */
static public boolean isMotif()
{
    String lf = UIManager.getLookAndFeel().getID();
	if (lf != null && lf.equals("Motif"))
    {
		return(true);
	}
	return(false);
}

/**
 * Is the current Look and Feel Windows?
 * @return true, if the current L&F is Windows.
 */
static public boolean isWindows()
{
    String lf = UIManager.getLookAndFeel().getID();
    if (lf != null && lf.equals("Windows")) {
        return (true);
    }
    return(false);
}

/**
 * Is the current Look and Feel Metal?
 * @return true, if the current L&F is Metal.
 */
static public boolean isMetal()
{
    String lf = UIManager.getLookAndFeel().getID();
    if (lf != null && lf.equals("Metal")) {
        return (true);
    }
    return (false);
}

static private int canSetResizableToFalseOnDialogs()
{
	if (isMotif()) {
		return(WORKING_NO);
	}
	return(WORKING_YES);
}


static private int motifPreferredWidthAjustment()
{
    if (isMotif()) {
        return(4);
    }
    else {
        return(0);
    }
}

/**
 * Returns <code>WORKING_YES</code> if HTML in Swing components
 * are working.
 * @return <code>WORKING_YES</code> if HTML in Swing components are working
 */
static private int htmlInComponents()
{
	String s = getVendorString();
	if (s.indexOf("Netscape") != -1) {
		return(WORKING_NO);
	}
	return(WORKING_YES);
}

/**
 * Returns <code>WORKING_YES</code> if JDK version &#62= 1.3.0.
 * @return <code>WORKING_YES</code> if JDK version &#62=1.3.0
 */
static private int fileChooserMultiSelects() {
	if (getJavaVersion() < 130) {
		return(WORKING_NO);
	}
	return(WORKING_YES);
}

/**
 * Returns <code>WORKING_YES</code> if Printing should use the 2nd Call
 * to the Printable interface's print method, ignorning the 1st call.
 * This happens with jdk 1.4 on Windows, and perhaps other platforms.
 * @return <code>WORKING_YES</code> if Printing should use the 2nd Call
 * to the Printable interface's print method, ignorning the 1st call
 */
static private int printingUse2ndCall()
{
	if (getJavaVersion() >= 140) {
		return(WORKING_YES);
	}
	return(WORKING_NO);
}

/**
 * Currently broken in all JDKs.
 * @return
 */
static private int focusRectInJTable()
{
    return(WORKING_NO);
}

/**
 * Globally useful <code>isDesignTime</code> check. This will take into
 * account environments that don't set <code>java.beans.isDesignTime</code>.
 * @return
 */
public static boolean isDesignTime()
{
	if (getVendorString().indexOf("Microsoft") != -1) {
		// No Microsoft IDEs, therefore not design time.
		// This avoids Microsoft's non-standard security
		// exception, which extends a non-standard exception
		// class and CANNOT be caught.
		return(false);
	}
	return(java.beans.Beans.isDesignTime() || isNetBeansDesignTime());
}

/**
 * Environment is design-time in NetBeans.
 * @return
 */
public static boolean isNetBeansDesignTime()
{
	try {
		String s = System.getProperty("netbeans.design.time");
		return(s != null);
	}
	catch (Throwable t) {}
	return(false);
}

/** Environment is Symantec Visual Cafe.
 * @return
 */
public static boolean isVisualCafe()
{
    String s = getVendorString();
    return(s.indexOf("Symantec") != -1);
}

/**
 * Retrieves and caches the vendor String.
 * @return
 */
public static String getVendorString()
{
    if (vendorString == null) {
        try {
            vendorString = System.getProperty("java.vendor");
        }
        catch (Exception e) {
            vendorString = null;
        }
    }
    return(vendorString);
}

/**
 * Follows the component's parents until
 * a parent applet is found.  Returns null if no applet is found.
 * @return null if no applet is found
 * @param comp
 */
public static Applet getApplet(Component comp)
{
	if (comp == null) {
        return(null);
    }
	if (comp instanceof Applet) {
        return((Applet) comp);
    }
	Container tmp    = comp.getParent();
	Container parent = comp.getParent();
	while (true) {
		if (parent == null || parent instanceof Applet) {
			break;
		}
		parent = tmp;
		tmp = (tmp == null) ? null : tmp.getParent();
	}
	return((Applet) parent);
}

/**
 * Gets the component's applet context. The applet context
 * lets an applet control the applet's environment, which is
 * usually the browser or the applet viewer.
 * @return null if the component is not in an applet or the applet is not in
 * a browser
 * @see java.applet.Applet#getAppletContext
 * @param applet
 */
public static AppletContext getAppletContext(Applet applet)
{
	if (applet != null) {
		// AWT bug - null exception thrown
		try {
			return(applet.getAppletContext());
		}
        catch (Exception e) {}
	}
	return(null);
}

/**
 * Returns <code>true</code> if the component is in an applet in a browser
 * (in other words, its <code>AppletContext</code> is not null).
 * @return <code>true</code> if the component is in an applet in a browser
 * @param comp
 */
public static boolean inBrowser(Component comp)
{
	return(getAppletContext(getApplet(comp)) != null);
}

/**
 * Returns <code>true</code> if the PJA toolkit is currently being used.
 * @return <code>true</code> if the PJA toolkit is currently being used.
 */
public static boolean isPJAToolkit()
{
    boolean pjaToolkit = false;
    Toolkit toolkit = Toolkit.getDefaultToolkit();
    String name = toolkit.getClass().getName();
    if (name != null && name.equals("com.eteks.awt.PJAToolkit")) {
        pjaToolkit = true;
    }
    return(pjaToolkit);
}

}
