/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCFormatParserUtil.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:38 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util;

import java.io.IOException;
import java.io.StreamTokenizer;

/**
 * This is a utility class made up of static
 * methods useful for parsing data for component
 * data sources.  It is not intended for general
 * public consumption, but it must be a public
 * class in order to be accessible by other
 * JClass packages.
 */
public class JCFormatParserUtil {

/** What JClass data soruces recognize as a quote character. */
public final static char QUOTE = '\'';

/**
 * Returns the next token from a stream tokenizer.  This
 * special call tries to convert numbers of the format
 * "2.13e-12" into real numbers.  Without this method,
 * the stream tokenizer will split them into two
 * tokens: "2.13" and "e-12".
 * 
 * @param tokenizer active <code>StreamTokenizer</code> from which to extract
 * and parse the next token
 * @return
 * @throws IOException
 */
public static Object getNextToken(StreamTokenizer tokenizer) throws IOException
{
	Object value = null;
	tokenizer.nextToken();
	switch(tokenizer.ttype) {
	case QUOTE:
		value = new String(tokenizer.sval);
		break;
	case StreamTokenizer.TT_WORD:
		value = new String(tokenizer.sval);
		break;
	case StreamTokenizer.TT_NUMBER:
		double rval = tokenizer.nval;
		// Check if next token is e/E
		tokenizer.nextToken();
		if (tokenizer.ttype == StreamTokenizer.TT_WORD  &&
			tokenizer.sval != null                      &&
			(tokenizer.sval.charAt(0) == 'e' ||
             tokenizer.sval.charAt(0) == 'E'))
        {
			if (tokenizer.sval.length() == 1) {
				tokenizer.nextToken();
				if(tokenizer.ttype == StreamTokenizer.TT_NUMBER)
					rval = rval * Math.pow(10.0, tokenizer.nval);
			}
            else {
				String s = tokenizer.sval;
				// scientific notation
				int mult = Integer.valueOf(s.substring(1)).intValue();
				rval = rval * Math.pow(10.0, mult);
			}
		}
        else {
			tokenizer.pushBack();
		}
		value = new Double(rval);
		break;
	case StreamTokenizer.TT_EOF:
		value = null;
		break;
	case StreamTokenizer.TT_EOL:
		value = new String("" + StreamTokenizer.TT_EOL);
		break;
	default:
		displayErrorMessage(tokenizer, null);
		throw new IOException("Error in data file format - " + tokenizer);
	}
	return(value);
}

/**
 * Displays a tokenizer-related error message.  This
 * isn't intended for public consumption, although you
 * could use it for displaying your own tokenizer-related
 * error messages
 *<p>
 * Uses the
 * provided tokenizer to display line numbers and token in an
 * error message using the following format:
 * <pre>
 * Error reading data: line NNN: &#60strError&#62:&#60token&#62
 * </pre>
 * Where <code>strError</code> is the String passed in as a parameter and
 * token is the current token in the tokenizer.
 *
 * @param tokenizer active <code>StreamTokenizer</code>, preferably the
 * one that has encountered some kind of error
 * @param strError String to embed into the error message,
 * as shown above
 */
public static void displayErrorMessage(StreamTokenizer tokenizer,
									   String strError)
{
	if (strError == null) {
		strError = new String("Expected data value and not ");
	}
	strError = "Error reading data: line " + tokenizer.lineno() + ": " +
		       strError;

	if (tokenizer.ttype == JCFormatParserUtil.QUOTE) {
		strError += "\'" + tokenizer.sval + "\'";
	}
	else if (tokenizer.ttype == StreamTokenizer.TT_WORD) {
		strError += tokenizer.sval;
	}
	else if (tokenizer.ttype == StreamTokenizer.TT_NUMBER) {
		strError += tokenizer.nval;
	}
	else {
		strError += tokenizer.toString();
	}
	System.err.println(strError);
}

}
