/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCIPAddress.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:38 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.util;

import java.io.Serializable;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.StringTokenizer;

/**
 * The <code>JCIPAddress</CODE> class stores an IP address and it will not try
 * to resolve the IP address on the network.
 */
public class JCIPAddress implements Serializable {

protected int[] address = new int[4];

public static int MAX_VALUE = 255;
public static int MIN_VALUE = 0;

/**
 * Creates a <code>JCIPAddress</code> given a String.
 * @param ip format for the String should be <i>%d.%d.%d.%d.</i>
 */
public JCIPAddress(String ip)
{
    parseStringIP(ip);
}

/**
 * Creates a <code>JCIPAddress</code> given a <code>java.net.InetAddress</code>.
 * It calls <code>InetAddress.getHostAddress</code> to obtain the IP.
 * @param inet
 */
public JCIPAddress(InetAddress inet)
{
    parseStringIP(inet.getHostAddress());
}

/**
 * Creates a <code>JCIPAddress</code> an array of ints.
 * @param address
 */
public JCIPAddress(int[] address)
{
    int count = Math.min(4, address.length);

    for (int i = 0; i < count; i++) {
        this.address[i] = address[i];
    }
}

/**
 * Given an IP String, it will store each subfield in the address array.
 * @param ip
 */
protected void parseStringIP(String ip)
{
    if (ip == null || ip.length() == 0) {
        return;
    }

    StringTokenizer tokenizer = new StringTokenizer(ip, ".");
    if (tokenizer.countTokens() != 4) {
        return;
    }

    address[0] = Integer.valueOf(tokenizer.nextToken()).intValue();
    address[1] = Integer.valueOf(tokenizer.nextToken()).intValue();
    address[2] = Integer.valueOf(tokenizer.nextToken()).intValue();
    address[3] = Integer.valueOf(tokenizer.nextToken()).intValue();
}

/**
 * Returns the IP as a String. It will be in the following format:
 * <i>%d.%d.%d.%d.</i>
 * @return IP as a String
 */
public String toString()
{
    return(address[0] + "." + address[1] + "." + address[2] + "." + address[3]);
}

/**
 * Returns the IP. The IP will be in a <code>java.net.InetAddress</code> object.
 * If the IP is invalid, null will be returned.
 * @return IP in a <code>java.net.InetAddress</code> object
 */
public InetAddress getInetAddress()
{
    InetAddress inet = null;
    try {
        inet = InetAddress.getByName(toString());
    }
    catch (UnknownHostException e) {}

    return(inet);
}

/**
 * Returns the IP address as an array of ints.
 * @return IP address as an array of ints
 */
public int[] getIPAddress()
{
    return(address);
}

}
