/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCListenerList.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:38 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util;

import java.util.Enumeration;

/**
 * <CODE>JCListenerList</CODE> is a class that assists with
 * keeping track of event listeners in a thread-safe manner.
 * <p>
 * To add a listener using a <CODE>JCListenerList</CODE>:
 * <pre>
 * JCListenerList someList = null;
 * ...
 * public synchronized void addSomeListener(SomeListener l) {
 *     someList = JCListenerList.add(someList, l);
 * }
 * </pre>
 * <p>
 * To remove a listener:
 * <pre>
 * public synchronized void removeSomeListener(SomeListener l) {
 *     someList = JCListenerList.remove(someList, l);
 * }
 * </pre>
 * <p>
 * The use of static methods on the <CODE>JCListenerList</CODE>
 * class prevents any
 * problems from occurring if the list being modified is null.
 * <p>
 * To send events to the listener in the list, simply get the
 * <code>Enumeration</code> of the list and walk through the elements.
 * There is no ordering guarantee.
 */

public class JCListenerList implements java.io.Serializable
{

protected JCListenerList  next = null;
protected Object          listener = null;

private JCListenerList(JCListenerList list, Object l)
{
	next = list;
	listener = l;
}

public static JCListenerList add(JCListenerList list, Object newListener)
{
	return(new JCListenerList(list, newListener));
}

public static JCListenerList remove(JCListenerList list, Object oldListener)
{
	JCListenerList newList = null;

	if (list == null) {
		return(null);
    }

	while (list != null) {
		if (list.listener != oldListener) {
			newList = add(newList, list.listener);
        }
		list = list.next;
	}

	return(newList);
}

public static Enumeration elements(JCListenerList list)
{
	return(new JCListenerListEnumeration(list));
}

}
