/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCLocaleManager.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:38 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util;

import javax.swing.*;
import java.io.Serializable;
import java.util.*;

/**
 * <code>JCLocaleManager</code> manages an association between a
 * resource bundle name and a <code>ResourceBundle</code>.  JClass uses it
 * internally to provide uniform
 * access to resource bundle information.
 * <p>
 * <code>JCLocaleManager</code> also stores standard key names for
 * JarMaster, and <code>JCMDIPane</code> for accelerators, mnemonics,
 * actions, and menu names.
 */
public class JCLocaleManager implements JCLocaleManagerModel, Serializable
{
//
// Standard Prefixes
//

static public final String ACTION = "Action";
static public final String MENU = "Menu";
static public final String TITLE = "Title";
static public final String WEEKDAY = "Weekday";
static public final String MONTH = "Month";

//
// Standard Suffixes
//

static public final String MNEMONIC = "Mnemonic";
static public final String ACCELERATOR = "Accelerator";

//
// PROVIDED KEYS
//

//
// Standard Menu Headers
//

public static final String MENU_EDIT			= MENU + "Edit";
public static final String MENU_EDIT_MNEMONIC	= MENU_EDIT + MNEMONIC;
public static final String MENU_FILE			= MENU + "File";
public static final String MENU_FILE_MNEMONIC	= MENU_FILE + MNEMONIC;
public static final String MENU_HELP			= MENU + "Help";
public static final String MENU_HELP_MNEMONIC	= MENU_HELP + MNEMONIC;
public static final String MENU_TOOLS			= MENU + "Tools";
public static final String MENU_TOOLS_MNEMONIC	= MENU_TOOLS + MNEMONIC;
public static final String MENU_VIEW			= MENU + "View";
public static final String MENU_VIEW_MNEMONIC	= MENU_VIEW + MNEMONIC;
	// MDI
public static final String MENU_WINDOWS			= MENU + "Windows";
public static final String MENU_WINDOWS_MNEMONIC = MENU_WINDOWS + MNEMONIC;

//
// Standard Action Items
//

// Keys marked MDI and WIZARD should be moved to the Elements package
// when it's created.

public static final String ACTION_APPLY				= ACTION + "Apply";
public static final String ACTION_APPLY_MNEMONIC	= ACTION_APPLY + MNEMONIC;
public static final String ACTION_ABOUT				= ACTION + "About";
public static final String ACTION_ABOUT_ACCELERATOR	= ACTION_ABOUT + ACCELERATOR;
public static final String ACTION_ABOUT_MNEMONIC	= ACTION_ABOUT + MNEMONIC;
	// MDI
public static final String ACTION_ARRANGE			= ACTION + "Arrange";
public static final String ACTION_ARRANGE_MNEMONIC	= ACTION_ARRANGE + MNEMONIC;

public static final String ACTION_BROWSE			= ACTION + "Browse";
public static final String ACTION_BROWSE_ACCELERATOR= ACTION_BROWSE + ACCELERATOR;
public static final String ACTION_BROWSE_MNEMONIC	= ACTION_BROWSE + MNEMONIC;
public static final String ACTION_CANCEL			= ACTION + "Cancel";
public static final String ACTION_CANCEL_MNEMONIC	= ACTION_CANCEL + MNEMONIC;
	// MDI
public static final String ACTION_CASCADE			= ACTION + "Cascade";
public static final String ACTION_CASCADE_MNEMONIC	= ACTION_CASCADE + MNEMONIC;
public static final String ACTION_CLOSE				= ACTION + "Close";
public static final String ACTION_CLOSE_ACCELERATOR	= ACTION_CLOSE + ACCELERATOR;
public static final String ACTION_CLOSE_MNEMONIC 	= ACTION_CLOSE + MNEMONIC;
public static final String ACTION_COPY				= ACTION + "Copy";
public static final String ACTION_COPY_ACCELERATOR 	= ACTION_COPY + ACCELERATOR;
public static final String ACTION_COPY_MNEMONIC		= ACTION_COPY + MNEMONIC;
public static final String ACTION_CUT				= ACTION + "Cut";
public static final String ACTION_CUT_ACCELERATOR 	= ACTION_CUT + ACCELERATOR;
public static final String ACTION_CUT_MNEMONIC		= ACTION_CUT + MNEMONIC;
public static final String ACTION_DELETE			= ACTION + "Delete";
public static final String ACTION_DELETE_ACCELERATOR = ACTION_DELETE + ACCELERATOR;
public static final String ACTION_DELETE_MNEMONIC 	= ACTION_DELETE + MNEMONIC;
public static final String ACTION_EXIT				= ACTION + "Exit";
public static final String ACTION_EXIT_ACCELERATOR 	= ACTION_EXIT + ACCELERATOR;
public static final String ACTION_EXIT_MNEMONIC 	= ACTION_EXIT + MNEMONIC;
	// WIZARD
public static final String ACTION_FINISH				= ACTION + "Finish";
public static final String ACTION_FINISH_ACCELERATOR 	= ACTION_FINISH + ACCELERATOR;
public static final String ACTION_FINISH_MNEMONIC		= ACTION_FINISH + MNEMONIC;
public static final String ACTION_HELP				= ACTION + "Help";
public static final String ACTION_HELP_ACCELERATOR 	= ACTION_HELP + ACCELERATOR;
public static final String ACTION_HELP_MNEMONIC		= ACTION_HELP + MNEMONIC;
	// MDI
public static final String ACTION_MAXIMIZE 	= ACTION + "Maximize";
public static final String ACTION_MAXIMIZE_MNEMONIC = ACTION_MAXIMIZE + MNEMONIC;
	// MDI
public static final String ACTION_MINIMIZE 	= ACTION + "Minimize";
public static final String ACTION_MINIMIZE_MNEMONIC = ACTION_MINIMIZE + MNEMONIC;
public static final String ACTION_NEW				= ACTION + "New";
public static final String ACTION_NEW_ACCELERATOR 	= ACTION_NEW + ACCELERATOR;
public static final String ACTION_NEW_MNEMONIC		= ACTION_NEW + MNEMONIC;

	// WIZARD
public static final String ACTION_NEXT 	               = ACTION + "Next";
public static final String ACTION_NEXT_MNEMONIC        = ACTION_NEXT + MNEMONIC;
public static final String ACTION_OKAY				   = ACTION + "Okay";
public static final String ACTION_OKAY_MNEMONIC		   = ACTION_OKAY + MNEMONIC;
public static final String ACTION_OPEN				   = ACTION + "Open";
public static final String ACTION_OPEN_ACCELERATOR 	   = ACTION_OPEN + ACCELERATOR;
public static final String ACTION_OPEN_MNEMONIC		   = ACTION_OPEN + MNEMONIC;
public static final String ACTION_PASTE				   = ACTION + "Paste";
public static final String ACTION_PASTE_ACCELERATOR	   = ACTION_PASTE + ACCELERATOR;
public static final String ACTION_PASTE_MNEMONIC	   = ACTION_PASTE + MNEMONIC;
public static final String ACTION_PRINT                = ACTION + "Print";
public static final String ACTION_PRINT_ACCELERATOR    = ACTION_PRINT + ACCELERATOR;
public static final String ACTION_PRINT_MNEMONIC       = ACTION_PRINT + MNEMONIC;
public static final String ACTION_PREFERENCES          = ACTION + "Preferences";
public static final String ACTION_PREFERENCES_MNEMONIC = ACTION_PREFERENCES + MNEMONIC;
public static final String ACTION_YES                  = ACTION + "Yes";
public static final String ACTION_YES_MNEMONIC         = ACTION_OKAY + MNEMONIC;
public static final String ACTION_YES_TO_ALL           = ACTION + "YesToAll";
public static final String ACTION_YES_TO_ALL_MNEMONIC  = ACTION_OKAY + MNEMONIC;
public static final String ACTION_NO                   = ACTION + "No";
public static final String ACTION_NO_MNEMONIC          = ACTION_OKAY + MNEMONIC;
public static final String ACTION_NO_TO_ALL            = ACTION + "NoToAll";
public static final String ACTION_NO_TO_ALL_MNEMONIC   = ACTION_OKAY + MNEMONIC;
	// WIZARD
public static final String ACTION_PREVIOUS 	= ACTION + "Previous";
public static final String ACTION_PREVIOUS_MNEMONIC = ACTION_PREVIOUS + MNEMONIC;
public static final String ACTION_REDO				= ACTION + "Redo";
public static final String ACTION_REDO_ACCELERATOR	= ACTION_REDO + ACCELERATOR;
public static final String ACTION_REDO_MNEMONIC		= ACTION_REDO + MNEMONIC;
public static final String ACTION_RENAME			= ACTION + "Rename";
public static final String ACTION_RENAME_ACCELERATOR= ACTION_RENAME + ACCELERATOR;
public static final String ACTION_RENAME_MNEMONIC	= ACTION_RENAME + MNEMONIC;
public static final String ACTION_SELECTALL			= ACTION + "SelectAll";
public static final String ACTION_SELECTALL_ACCELERATOR	= ACTION_SELECTALL + ACCELERATOR;
public static final String ACTION_SELECTALL_MNEMONIC = ACTION_SELECTALL + MNEMONIC;
public static final String ACTION_SAVE				= ACTION + "Save";
public static final String ACTION_SAVE_ACCELERATOR 	= ACTION_SAVE + ACCELERATOR;
public static final String ACTION_SAVE_MNEMONIC 	= ACTION_SAVE + MNEMONIC;
public static final String ACTION_SAVEAS			= ACTION + "SaveAs";
public static final String ACTION_SAVEAS_ACCELERATOR= ACTION_SAVEAS + ACCELERATOR;
public static final String ACTION_SAVEAS_MNEMONIC 	= ACTION_SAVEAS + MNEMONIC;
	// MDI
public static final String ACTION_TILEH				= ACTION + "Tileh";
public static final String ACTION_TILEH_MNEMONIC	= ACTION_TILEH + MNEMONIC;
	// MDI
public static final String ACTION_TILEV				= ACTION + "Tilev";
public static final String ACTION_TILEV_MNEMONIC	= ACTION_TILEV + MNEMONIC;
public static final String ACTION_UNDO				= ACTION + "Undo";
public static final String ACTION_UNDO_ACCELERATOR	= ACTION_UNDO + ACCELERATOR;
public static final String ACTION_UNDO_MNEMONIC		= ACTION_UNDO + MNEMONIC;
	// MDI
public static final String ACTION_UNDOMAX			= ACTION + "Undomax";
public static final String ACTION_UNDOMAX_ACCELERATOR = ACTION_UNDOMAX + ACCELERATOR;
public static final String ACTION_UNDOMAX_MNEMONIC	= ACTION_UNDOMAX + MNEMONIC;
public static final String ACTION_WIZARD			= ACTION + "Wizard";
public static final String ACTION_WIZARD_ACCELERATOR = ACTION_WIZARD + ACCELERATOR;
public static final String ACTION_WIZARD_MNEMONIC	= ACTION_WIZARD + MNEMONIC;

public static final String TITLE_GOTOROW 			= TITLE + "GotoRow";

// Days of the Week
public static final String WEEKDAY1 = WEEKDAY + "1";
public static final String WEEKDAY2 = WEEKDAY + "2";
public static final String WEEKDAY3 = WEEKDAY + "3";
public static final String WEEKDAY4 = WEEKDAY + "4";
public static final String WEEKDAY5 = WEEKDAY + "5";
public static final String WEEKDAY6 = WEEKDAY + "6";
public static final String WEEKDAY7 = WEEKDAY + "7";

// Months of the Year (zero based)
public static final String MONTH0  = MONTH + "0";
public static final String MONTH1  = MONTH + "1";
public static final String MONTH2  = MONTH + "2";
public static final String MONTH3  = MONTH + "3";
public static final String MONTH4  = MONTH + "4";
public static final String MONTH5  = MONTH + "5";
public static final String MONTH6  = MONTH + "6";
public static final String MONTH7  = MONTH + "7";
public static final String MONTH8  = MONTH + "8";
public static final String MONTH9  = MONTH + "9";
public static final String MONTH10 = MONTH + "10";
public static final String MONTH11 = MONTH + "11";

    
//Font labels
public static final String FONT_LABEL = "Font: ";
public static final String SIZE_LABEL = "Size: ";
public static final String STYLE = "Style";
public static final String BOLD = "Bold";
public static final String ITALIC = "Italic";
public static final String UNDERLINE = "Underline";
public static final String SAMPLE = "Sample";
public static final String CHOOSE_FONT = "Choose Font";
public static final String FONT = "Font";
public static final String FONT_SIZE = "Font Size";

public static final String defaultLocale =
    "com.klg.jclass.util.resources.LocaleInfo";
static protected final JCLocaleManager default_manager = new JCLocaleManager();

static public JCLocaleManager getDefault()
{
	return(default_manager);
}

/** The locale is the same for all the locales we manage. */
protected static Locale locale = Locale.getDefault();
protected HashMap bundles = new HashMap();

//
// Constructors
//

/**
 * Creates a locale manager for the default locale.
 */
public JCLocaleManager()
{
	this(defaultLocale);
}

/**
 * Creates a locale manager and loads the bundle specified.
 * @param bundle The bundle to load
 */
public JCLocaleManager(String bundle)
{
	if (locale == null) {
		locale = Locale.getDefault();
	}

	// Load default bundle
	ResourceBundle rb = ResourceBundle.getBundle(bundle, locale);
	bundles.put(defaultLocale, rb);
}

/**
 * Returns the locale used by this manager.
 * @return locale used by this manager
 */
public static Locale getLocale()
{
	return(locale);
}

public Collection getBundles()
{
	return(bundles.values());
}

public void add(String bundle_name)
{
	if (bundles.containsKey(bundle_name)) {
		return;
	}

	ResourceBundle rb = ResourceBundle.getBundle(bundle_name, locale);
	bundles.put(bundle_name, rb);
}

//
// ResourceBundle-like accessor methods
//

public String getString(String key)
{
	Iterator bundles = getBundles().iterator();
	String string = null;
	while (bundles.hasNext() == true) {
		ResourceBundle rb = (ResourceBundle)bundles.next();
		try {
			string = rb.getString(key);
			break;
		}
		catch (MissingResourceException mre) {
			continue;
		}
	}
	return(string);
}

public Object getObject(String key)
{
	Object o = null;
	Iterator bundles = getBundles().iterator();
	while (bundles.hasNext() == true) {
		ResourceBundle rb = (ResourceBundle)bundles.next();
		try {
			o = rb.getObject(key);
			break;
		}
		catch (MissingResourceException mre) {
			continue;
		}
	}
	return(o);
}

public int getMnemonic(String key)
{
	Integer i = (Integer)getObject(key);
	return(i.intValue());
}

public KeyStroke getKeyStroke(String key)
{
	return((KeyStroke)getObject(key));
}

public final static KeyStroke getCtrlKey(int keyCode)
{
	return(KeyStroke.getKeyStroke(keyCode, java.awt.Event.CTRL_MASK));
}

public final static KeyStroke getKey(int keyCode)
{
	return(KeyStroke.getKeyStroke(keyCode, 0));
}

}
