/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCMappingSort.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:38 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util;

import java.util.Stack;

/**
 * <code>JCMappingSort</code> class.
 */
public class JCMappingSort {

private JCIntComparator  comparator = null;
private int              indices[] = null;

/**
 * Private <code>JCMappingSort</code> constructor. Must not be called directly.
 * @param comparator the comparator for the sort
 * @param indices the array of indices which get sorted
 */
private JCMappingSort(JCIntComparator comparator, int indices[])
{
	this.comparator = comparator;
    this.indices = indices;
}

/**
 * Sorts the indices of elements using the quicksort algorithm.
 * @param comparator the comparator for the sort
 * @param indices the array of indices which get sorted
 */
public static void sort(JCIntComparator comparator, int indices[])
{
	sort(comparator, indices, 0, indices.length-1);
}

/**
 * Sorts the indices of elements using the quicksort algorithm.
 * @param comparator the comparator for the sort
 * @param indices the array of indices which get sorted
 * @param start the start position in the array to sort
 * @param end the end position in the array to sort
 */
public static void sort(JCIntComparator comparator, int indices[],
				        int start, int end)
{
	JCMappingSort mappingSort = new JCMappingSort(comparator, indices);
    mappingSort.sortArray(start, end);
}

/**
 * Swaps two indices.
 * @param i the first index
 * @param j the second index
 */
private synchronized void swap(int i, int j)
{
    int tmp = indices[i];
    indices[i] = indices[j];
    indices[j] = tmp;
}

/**
 * Sorts the indices of elements using the quicksort algorithm.
 * @param start the start position in the array to sort
 * @param end the end position in the array to sort
 */
private synchronized void sortArray(int start, int end)
{
	// Do nothing if array contains fewer than two elements
	if (start >= end) {
	    return;
    }

    Stack stack = new Stack();
    stack.push(new int[] {start, end});

    while (!stack.empty()) {
        int[] bounds = (int[]) stack.pop();
        start = bounds[0];
        end   = bounds[1];
        if (start >= end) {
            continue;
        }

		int last = start;
	    for (int i = start + 1; i <= end; i++) {
		    if (comparator.compare(indices[start], indices[i]) > 0) {
			    swap(++last, i);
            }
		}

		// Do nothing if the elements are equal
		if (comparator.compare(indices[start], indices[last]) != 0) {
			swap(start, last);
		}

        stack.push(new int[] {start, last - 1});
        stack.push(new int[] {last + 1, end});
	}
}

}



