/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: RenderProperties.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:38 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util;

/**
 * Interface used to access specific component render properties.
 * <P>
 * Internal use only. Don't use.
 */
public interface RenderProperties {

// Anti-aliasing enum values
public final static int ANTI_ALIASING_DEFAULT = 0;
public final static int ANTI_ALIASING_ON = 1;
public final static int ANTI_ALIASING_OFF = 2;

/**
 * Gets the antiAliasing value.  This value determines whether antiAliasing is
 * on when painting the component and all its subcomponents.  If set to
 * <code>ANTI_ALIASING_ON</code> or <code>ANTI_ALIASING_OFF</code>, the anti
 * aliasing rendering hint will be explicity turned on or off on the graphics
 * object used by the component to paint its children.  If set to
 * <code>ANTI_ALIASING_DEFAULT</code>, the graphics object will be untouched
 * with respect to anti aliasing when the component is painted.
 * @return The current antiAliasing value.
 */
public int getAntiAliasing();

/**
 * Sets the antiAliasing value.  This value determines whether antiAliasing is
 * on when painting the component and all its subcomponents.  If set to
 * <code>ANTI_ALIASING_ON</code> or <code>ANTI_ALIASING_OFF</code>, the anti
 * aliasing rendering hint will be explicity turned on or off on the graphics
 * object used by the component to paint its children.  If set to
 * <code>ANTI_ALIASING_DEFAULT</code>, the graphics object will be untouched
 * with respect to anti aliasing when the component is painted.
 * @param antiAliasing The new antiAliasing value.
 */
public void setAntiAliasing(int antiAliasing);

/**
 * Sets whether the component computes its pixel values using doubles rather
 * than integers.
 * @param usingDoubles The new value of usingDoubles.
 */
public void setUsingDoublePixels(boolean usingDoubles);

/**
 * Gets whether the component computes its pixel values using doubles rather
 * than integers.
 * @return true, if the component computes its pixel values using doubles.
 */
public boolean isUsingDoublePixels();

}
