/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ServerJPanel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:39 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util;

import com.klg.jclass.util.swing.encode.JCEncodeComponent;
import com.klg.jclass.util.swing.encode.JPEGEncoder;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

public class ServerJPanel extends JPanel implements ServerRenderable {

/** Temporary Graphics object used to draw without showing to screen.*/
private Graphics prevSetGraphics = null;

/**
 * Overrides <code>javax.swing.JComponent addNotify()</code>.
 */
public void addNotify()
{
    super.addNotify();
}

/**
 * Draws a snapshot of this panel into the provided image.
 *
 * @param img the image to draw this panel into.
 */
public void snapshot(Image img)
{
    ServerRenderer.snapshot(this, img);
}

/**
 * Convenience method.  Creates a <code>FileOutputStream</code> from a filename.
 * Returns null if the file could not be created.
 *
 * @param filename the name of the file to create
 * @return FileOutputStream object based on a specified filename.
 * @see java.io.FileOutputStream
 */
public FileOutputStream createFileStream(String filename)
{
    FileOutputStream fos = null;
    try {
        fos = new FileOutputStream(filename);
    }
    catch (IOException e) {
        e.printStackTrace();
    }
    return fos;
}

/**
 * Convenience method. Encodes the current panel
 * to the specified file as a GIF.
 *
 * @param filename name of file to write the GIF to.
 */
public void encodeToGIFFile(String filename)
{
	FileOutputStream fos = createFileStream(filename);
	if (fos != null)
		encodeAsGIF(fos);
	try {
		fos.close();
	}
    catch (IOException e) {
		e.printStackTrace();
	}
}

/**
 * Convenience method. Encodes the current panel to
 * the specified file as a PNG.
 *
 * @param filename name of file to write the PNG to.
 */
public void encodeToPNGFile(String filename)
{
	FileOutputStream fos = createFileStream(filename);
	if (fos != null)
		encodeAsPNG(fos);
	try {
		fos.close();
	}
    catch (IOException e) {
		e.printStackTrace();
	}
}

/**
 * Convenience method. Encodes the current panel to
 * the specified file as a JPEG.
 *
 * @param filename name of file to write the JPEG to.
 */
public void encodeToJPEGFile(String filename)
{
	FileOutputStream fos = createFileStream(filename);
	if (fos != null)
		encodeAsJPEG(fos);
	try {
		fos.close();
	}
    catch (IOException e) {
		e.printStackTrace();
	}
}

/**
 * Convenience method. Encodes the current panel to
 * the specified file as a JPEG.
 *
 * @param filename name of file to write the JPEG to.
 * @param quality quality value for JPEG encoding.  This is a value between 0
 *      and 1 that determines how much the image quality is traded off with the
 *      amount of compression done to the image.  A value of 1 gives the best
 *      image quality and the worst compression.  A value of 0 gives the best
 *      compression but the worst image quality.
 */
public void encodeToJPEGFile(String filename, float quality)
{
	FileOutputStream fos = createFileStream(filename);
	if (fos != null)
		encodeAsJPEG(fos, quality);
	try {
		fos.close();
	}
    catch (IOException e) {
		e.printStackTrace();
	}
}

/**
 * Convenience method. Encodes the current panel to
 * the specified output stream as a GIF.
 *
 * @param os output stream to write the GIF to.
 */
public void encodeAsGIF(OutputStream os)
{
	encode(JCEncodeComponent.GIF, os);
}

/**
 * Convenience method. Encodes the current panel to
 * the specified output stream as a PNG.
 *
 * @param os output stream to write the PNG to.
 */
public void encodeAsPNG(OutputStream os)
{
    encode(JCEncodeComponent.PNG, os);
}

/**
 * Convenience method. Encodes the current panel to
 * the specified output stream as a JPEG.
 *
 * @param os output stream to write the JPEG to.
 */
public void encodeAsJPEG(OutputStream os)
{
    encode(JCEncodeComponent.JPEG, os);
}

/**
 * Convenience method. Encodes the current panel to
 * the specified output stream as a JPEG.
 *
 * @param os output stream to write the JPEG to.
 * @param quality quality value for JPEG encoding.  This is a value between 0
 *      and 1 that determines how much the image quality is traded off with the
 *      amount of compression done to the image.  A value of 1 gives the best
 *      image quality and the worst compression.  A value of 0 gives the best
 *      compression but the worst image quality.
 */
public void encodeAsJPEG(OutputStream os, float quality)
{
    Image img = snapshot();
    JPEGEncoder jpeg = new JPEGEncoder();
    jpeg.setQuality(quality);
    try {
        jpeg.saveImage(img, os);
    }
    catch (Exception e) {
        e.printStackTrace();
    }
}

/**
 * Encodes the current panel to the specified output stream using specified
 * encoding.  The list of encodings are given in
 * <code>com.klg.jclass.util.swing.encode.JCEncodeComponent</code>.
 *
 * @param encoding type of encoding.
 * @param os output stream to write the encoding to.
 */
public void encode(JCEncodeComponent.Encoding encoding, OutputStream os)
{
    Image img = snapshot();
    encodeImage(encoding, img, os);
}

/**
 * Encodes the image to the output stream using the encoding.  The list
 * of possible encodings are given in
 * <code>com.klg.jclass.util.swing.encode.JCEncodeComponent</code>.
 *
 * @param encoding type of encoding.
 * @param img image to encode.
 * @param os output stream to write the encoding to.
 */
public void encodeImage(JCEncodeComponent.Encoding encoding, Image img,
                        OutputStream os)
{
    try {
        JCEncodeComponent.encode(encoding, img, os);
    }
    catch (Exception e) {
        e.printStackTrace();
    }
}

/**
 * Takes a snapshot of the current panel.
 * @return Image object containing snapshot of panel.
 */
public Image snapshot()
{
    return(snapshot(BufferedImage.TYPE_INT_ARGB));
}

/**
 * Takes a snapshot of the current panel and places it in an image of the
 * specified type.  The image types are as specified in the
 * <code>BufferedImage</code> class.
 *
 * @param imagetype the type of image to write to, as defined in the
 *     java.awt.image.BufferedImage class
 * @return Image object that contains the snapshot of the panel.
 * @see java.awt.image.BufferedImage
 */
public Image snapshot(int imagetype)
{
    Image img = ServerRenderer.createNewImage(this, imagetype);
    ServerRenderer.snapshot(this, img);
    return(img);
}

//ServerRenderable methods
/**
 * Overrides parent <code>getGraphics()</code>. If another Graphics object has
 * been set to be the current one, will return that. Otherwise, call the
 * superclass' <code>getGraphics()</code> method.
 * @return the Graphics object to use to draw this chart.
 */
public Graphics getGraphics()
{
    if (prevSetGraphics == null) {
        return(super.getGraphics());
    }
    return(prevSetGraphics);
}

/**
 * Provides a way for the panel to draw to a custom Graphics object.
 * Call with a null to specify use of the default Graphics object used
 * to draw to the screen.
 * @param g Graphics object to use for drawing
 */
public void setGraphics(Graphics g)
{
    prevSetGraphics = g;
}

/**
 * Internal use only.  Don't use.
 */
public Image serverSnapshot()
{
    return(ServerRenderer.snapshot(this));
}

/**
 * Internal use only.  Don't use.
 */
public void serverPaint(Graphics igc)
{
    ServerRenderer.paintTopLevelContainer(igc, this);
}

}

