/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ServerLocaleBundleBase.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:39 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util;

import java.util.Locale;
import java.util.Hashtable;
import java.util.ResourceBundle;

/** 
 * Abstract parent class for server-style handling of locale bundles.
 * Server-style handling caches each requested locale bundle for easy
 * retrieval.  Locale bundles are cached by Locale and the resulting
 * cache table is itself cached by the bundle path.  Using this method,
 * several instances of a single product can have different locales and
 * the correct locale bundle will be queried.  (The client-side products
 * have tended to have a single static locale bundle serving the
 * entire product.  This is not acceptable for the server case, where
 * the locale may change with each new servlet request.)
 *
 * Subclasses should provide access methods that determine the bundle
 * using the getBundle() methods below. 
 */
public abstract class ServerLocaleBundleBase {

// cache of Hashtable caches, each Hashtable contining the ResourceBundles
// so far looked up for a bundle path
protected static Hashtable cacheCache = new Hashtable();

/**
 * Get a resource bundle for the specified bundle path and the locale
 * as defined by the specified language and country.
 */
public static ResourceBundle getBundle(String path,
										String language, String country)
{
	Locale locale = new Locale(language, country);
	return getBundle(path, locale);
}

/**
 * Get a resource bundle for the specified language and country from
 * the passed bundle cache.  If one does not exist, create one for
 * the passed bundle path and store it in the bundle cache.
 */
public static ResourceBundle getBundle(Hashtable bundleCache,
								String language, String country, String path)
{
	Locale locale = new Locale(language, country);
	return getBundle(bundleCache, locale, path);
}

/**
 * Get a resource bundle for the specified bundle path and the specified locale.
 */
public static ResourceBundle getBundle(String path, Locale locale)
{
	Hashtable bundleCache = getCache(path);
	return getBundle(bundleCache, locale, path);
}

/**
 * Get a resource bundle from the specified bundle cache and for the specified
 * locale.  If one does not exist, create it for the specified path and
 * store it in the bundle cache.
 */
public static ResourceBundle getBundle(Hashtable bundleCache, Locale locale,
																String path)
{
    if (bundleCache.contains(locale)) {
        return (ResourceBundle) bundleCache.get(locale);
    } else {
        ResourceBundle bundle = ResourceBundle.getBundle(path, locale);
        bundleCache.put(locale, bundle);
        return bundle;
    }
}

/**
 * Get the bundle cache for the specified bundle path.  If one does
 * not exist, create it and store it in the cache of bundle caches.
 */
public static Hashtable getCache(String path)
{
	if (cacheCache.contains(path)) {
		return (Hashtable) cacheCache.get(path);
	} else {
		Hashtable newCache = new Hashtable();
		cacheCache.put(path, newCache);
		return newCache;
	}
}

}
