/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: ServerRenderer.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:39 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util;

import java.awt.*;
import java.awt.image.BufferedImage;

public class ServerRenderer {

static public void paintComponent(Graphics igc, Component c)
{
	if (!(c instanceof ServerRenderable)) {
		return;
	}

	ServerRenderable sr = (ServerRenderable) c;

	sr.setGraphics(igc);

	paintComponentAndChildren(igc, c);

    sr.setGraphics(null);
}

static protected void paintComponentAndChildren(Graphics igc, Component c)
{
	// Component may need to be layed out -- make sure this is done
	c.doLayout();

	// Paint component
    c.paint(igc);

	// now see if component has children and deal with them accordingly
	if (c instanceof Container) {
		Container container = (Container) c;

        // Set anti-aliasing if set on server chart
        Object save = null;
        Object antiAliasHint = null;
        if (c instanceof RenderProperties) {
            RenderProperties rp = (RenderProperties)c;
            int antiAliasing = rp.getAntiAliasing();
            if (antiAliasing == RenderProperties.ANTI_ALIASING_ON) {
                antiAliasHint = RenderingHints.VALUE_ANTIALIAS_ON;
            }
            else if (antiAliasing == RenderProperties.ANTI_ALIASING_OFF) {
                antiAliasHint = RenderingHints.VALUE_ANTIALIAS_OFF;
            }
        }
        if (antiAliasHint != null) {
            Graphics2D g2d = (Graphics2D)igc;
            save = g2d.getRenderingHint(RenderingHints.KEY_ANTIALIASING);
            if (antiAliasHint != save) {
                g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                                     antiAliasHint);
            }
        }

		// Save old clip rect
		Rectangle oldClipRect = igc.getClipBounds();
		// Our own paint-children loop
		for (int i = container.getComponentCount() - 1; i >= 0; i--) {
			Component child = container.getComponent(i);
			if (child.isVisible()) {
				Rectangle bounds = child.getBounds();
				int clipWidth;
				if (bounds.x + bounds.width <= oldClipRect.width) {
					clipWidth = bounds.width;
				}
                else {
					clipWidth = oldClipRect.x + oldClipRect.width - bounds.x;
				}

				int clipHeight;
				if (bounds.y + bounds.height <= oldClipRect.height) {
					clipHeight = bounds.height;
				}
                else {
					clipHeight = oldClipRect.y + oldClipRect.height - bounds.y;
				}

				// Only paint components that can be seen
				if (clipWidth > 0 && clipHeight > 0) {
					// Translate into child's co-ordinates & clip to child
					igc.translate(bounds.x, bounds.y);
					igc.setClip(0, 0, clipWidth, clipHeight);

					// Paint the darn thing
                   	paintComponentAndChildren(igc, child);

					// Translate back
					igc.translate(-bounds.x, -bounds.y);
				}
			}
		}
		// Reset old clip and anti-aliasing
		igc.setClip(oldClipRect.x, oldClipRect.y,
				    oldClipRect.width, oldClipRect.height);

        if (save != null && save != antiAliasHint) {
            Graphics2D g2d = (Graphics2D)igc;
            g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, save);
        }
	}
}

/**
* Used to paint a container and its children in a specified graphics object
*
* @param igc the Graphics object where the painting is made
* @param c the Container object that is paint
*/
static public void paintContainer(Graphics igc, Container c)
{
    if (!(c instanceof ServerRenderable)) {
		return;
	}

    ServerRenderable sr = (ServerRenderable) c;

    sr.setGraphics(igc);

    paintContainerAndChildren(igc, c);

    sr.setGraphics(null);
}

// This is possibly redundant -- paintComponentAndChildren should
// be called instead wherever possible
static protected void paintContainerAndChildren(Graphics igc, Container c)
{
    c.doLayout();
    c.paint(igc);

    // save old clip rect
    Rectangle oldClipRect = igc.getClipBounds();
    for (int i = c.getComponentCount() - 1; i >= 0; i--) {
        Component child = c.getComponent(i);
        if (child.isVisible()) {
            Rectangle bounds = child.getBounds();
            // Translate into child's co-ordinates & clip to child
            igc.translate(bounds.x, bounds.y);
            igc.setClip(0, 0, bounds.width, bounds.height);
            if (child instanceof Container) {
                paintContainerAndChildren(igc, (Container)child);
            }
            else {
                child.paint(igc);
            }
            igc.translate(-bounds.x, -bounds.y);
        }
    }
    // Reset old clip
	igc.setClip(oldClipRect.x, oldClipRect.y,
				oldClipRect.width, oldClipRect.height);
}

/** Used to paint the components that lay on a Server Top Level Container
*
* @param igc the Graphics object where the painting is made
* @param c the server top level that contains the components to be painted.
*/
static public void paintTopLevelContainer(Graphics igc, ServerJPanel c)
{
    if (c == null) {
	    return;
    }

    c.setGraphics(igc);
    c.doLayout();
    c.paint(igc);

    Rectangle oldClipRect = igc.getClipBounds();
    for (int i = c.getComponentCount() - 1; i >= 0; i--) {
        Component child = c.getComponent(i);
        if (child.isVisible()) {
    		Rectangle bounds = child.getBounds();
		    // Translate into child's co-ordinates & clip to child
            igc.translate(bounds.x, bounds.y);
            igc.setClip(0, 0, bounds.width, bounds.height);
		    if (child instanceof Container) {
			    paintContainer(igc, (Container)child);
            }
		    else {
			    child.paint(igc);
    		}
    		igc.translate(-bounds.x, -bounds.y);
        }
    }
    // Reset old clip
    igc.setClip(oldClipRect.x, oldClipRect.y,
				oldClipRect.width, oldClipRect.height);

    c.setGraphics(null);
}

static public Image snapshot(Component c)
{
	Image img = createNewImage(c, BufferedImage.TYPE_INT_ARGB);
	snapshot(c, img);
	return(img);
}

static public Image createNewImage(Component c, int imagetype)
{
    if (c == null) {
        throw new IllegalArgumentException("Passed in component is null.");
    }
    Dimension size = c.getSize();
    if (size.width == 0 || size.height == 0) {
        throw new IllegalArgumentException("Size of component is zero.");
    }
	return createNewImage(size.width, size.height, imagetype);
}

static public Image createNewImage(int width, int height, int imagetype)
{
    if (width == 0 || height == 0) {
        throw new IllegalArgumentException("Width and height must be positive");
    }

	Image bi = new BufferedImage(width, height, imagetype);
	return(bi);
}

static public boolean snapshot(Component c, Image image)
{
	if (!(c instanceof ServerRenderable)) {
		return(false);
    }

	ServerRenderable sr = (ServerRenderable) c;
    if (image != null) {
        Graphics igc = image.getGraphics();
        // Gotta set the clip, or else paint throws an exception
        Dimension size = c.getSize();
        igc.setClip(0, 0, size.width, size.height);
        sr.serverPaint(igc);
    }
	return(true);
}

}
