/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/


package com.klg.jclass.util;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.ListResourceBundle;

/**
 * Allow a ResourceBundle implementation to be built up programmatically
 * from a series of name/value pairs rather than a static file.
 * It is probably best to add all pairs before passing it off
 * to the class or method that will be operating on it.
 */
public class SettableListResourceBundle extends ListResourceBundle
{

/** Contents array passed back to parent class. */
protected Object[][] contents = null;

/** Storage of name/value pairs in format that is easy to add to. */
protected Hashtable properties = new Hashtable();

/** True if contents needs to be regenerated.  False if nothing has changed. */
protected boolean regen = true;

/** Constructor.  Nothing to do. */
public SettableListResourceBundle()
{
}

/**
 * Add name/value pair to resource bundle.
 * @param name name part of name/value pair
 * @param value value part of name/value pair
 */
public void add(String name, String value)
{
	properties.put(name, value);
	// pair added, must regenerate contents when next asked
	regen = true;
}

/**
 * Get contents of this reource bundle as an array.  This
 * method supplied the parent class with the information it expect.
 */
public Object[][] getContents()
{
	if (regen) {
		contents = new String[properties.size()][2];
		int i = 0;
		Enumeration hashEnumeration = properties.keys();
		while (hashEnumeration.hasMoreElements()) {
			contents[i][0] = (String) hashEnumeration.nextElement();
			contents[i][1] = properties.get(contents[i][0]);
			i++;
		}
		regen = false;
	}

	return contents;
}

}
