/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: Version.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:39 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util;

import java.util.Hashtable;

/**
 * Version is a base class for classes that determine,
 * display and verify the version number for a JClass
 * package.
 *
 * java.lang.Package provides all the information needed
 * for version checking.  However, Package only works if a
 * class from the package has been loaded.  JCVersion classes
 * are used as a reference point to ensure that the package
 * has been loaded into the VM.
 */
public class Version {

/**
 * Stores the version string.
 */
protected static Hashtable versionHashTable = new Hashtable();

public static final String VERSION = " 6.3.0";

// For client licensing
public static final String CLIENT_LICENSE_VERSION = "6.3";

// For server licensing
public static final String LICENSE_VERSION = "5.0";

// For general consumption
public static final String BUILD_NUMBER = Version.getBuildNumber();

/**
 * Retrieves the version string for the specified package.
 * @param pkg Name of the package for which to retrieve the version number.
 * @return the version String for the specified package package
 */
public static String getVersionString(String pkg)
{
    if (pkg == null) {
        pkg = JCVersion.PACKAGE;
    }
    String versionString = (String) versionHashTable.get(pkg);
    if (versionString == null) {
        Package p = Package.getPackage(pkg);
        if (p == null) {
            versionString = pkg + VERSION;
        }
        else {
            versionString = p.getSpecificationTitle() + " " +
                            p.getImplementationVersion();
        }



 
        versionHashTable.put(pkg, versionString);
    }
    return(versionString);
}

/**
 * Gets the version number for the current package.
 * @return the version number for this package
 */
public static String getVersionString()
{
    return(getVersionString(JCVersion.PACKAGE));
}

private static String getBuildNumber() {
    String number = " JCEE500_SE630-20040427-1124";
    // This tricky trick is so that the build replacement of the tag
    // won't affect this check
    if ( number.equals(" @@" + "BUILD_NUMBER@@") ){
        // Not a nightly build. Might want to tack todays date onto
        // the end of the number
        number = " DeveloperBuild";
    }

    return number;
}

}
