/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AbstractLabel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:39 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.calendar;

import com.klg.jclass.util.value.JCValueEvent;
import com.klg.jclass.util.value.JCValueListener;
import com.klg.jclass.util.value.JCValueModel;

import javax.swing.*;
import java.awt.event.ActionListener;
import java.util.Locale;

/**
 * AbstractLabel
 */
public abstract class AbstractLabel extends JLabel
									implements CalendarComponent,
											   JCValueListener
{

protected JCValueModel  model;
protected Locale        locale = null;

/**
 * Set the current date being used by the calendar component
 * @param model The new calendar value model
 */
public void setCalendarModel(JCValueModel model)
{
	if (this.model == model) {
		return;
	}
	if (this.model != null) {
		this.model.removeValueListener(this);
	}

	firePropertyChange("calendarmodel", this.model, model);
	this.model = model;
	this.model.addValueListener(this);
}

/**
 * Special dates are ignored for labels
 * @param special_dates ignored
 */
public void setSpecialDates(JCCalendar special_dates) {}

/**
 * Set the locale being used by the calendar component
 * @param locale The new locale to be used
 */
public void setLocale(Locale locale)
{
	if (this.locale == locale) {
		return;
	}
	if (locale == null) {
		throw new IllegalArgumentException("null is an invalid value " +
                                           "for locale");
	}

	firePropertyChange("locale", this.locale, locale);
	this.locale = locale;
}

/**
 * Gets the locale of this component.
 * @return This component's locale;
 * @see #setLocale
 */
public Locale getLocale()
{
    return(locale);
}

/**
 * Add listener to receive action event when data is changed by the user
 * @param l The action listener to add
 */
public void addActionListener(ActionListener l)
{
	// labels have no actions
}

/**
 * remove listener to receive action event when data is changed by the user
 * @param l The action listener to remove
 */
public void removeActionListener(ActionListener l)
{
	// labels have no actions
}

//
// JCValueModel implementation
//

/**
 * The value is changing.  The listener may further change the new_value or
 * abort the change altogether.  If the case of an abort (call
 * event.setAllowChange(false)) the "valueChanged"  method will never be
 * called.
 * @param event The event generated by the value changing
 */
public void valueChanging(JCValueEvent event) {}

/**
 * The method informs the listener that the value has been changed.
 * @param event The event generated by the value being changed
 */
abstract public void valueChanged(JCValueEvent event);

}
