/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: CalendarComponent.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:39 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.calendar;

import com.klg.jclass.util.value.JCValueModel;

import java.awt.event.ActionListener;
import java.util.Locale;

/**
 * Interface that defines the generic components of a Calendar Display/Editor.
 * This is useful when assembling the different implementations that combine to
 * build a single display.
 */
public interface CalendarComponent {

/**
 * Sets the current date being used by the calendar component.
 * @param model The new calendar value model
 */
public void setCalendarModel(JCValueModel model);

/**
 * Sets the special dates being used by the calendar component.
 * @param special_dates The new list of special dates stored within a
 *     JCCalendar object
 */
public void setSpecialDates(JCCalendar special_dates);

/**
 * Sets the locale being used by the calendar component.
 * @param locale The new locale to be used
 */
public void setLocale(Locale locale);

/**
 * Adds an action listener to detect specific actions on this component.
 * @param l The action listener to be added
 */
public void addActionListener(ActionListener l);

/**
 * Removes an action listener on this component.
 * @param l The action listener to be removed
 */
public void removeActionListener(ActionListener l);

}
