/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: DateTimeEditor.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:39 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.util.calendar;

import com.klg.jclass.util.swing.JCBrace;
import com.klg.jclass.util.swing.JCColumnLayout;
import com.klg.jclass.util.value.CalendarValueModel;
import com.klg.jclass.util.value.JCValueModel;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Calendar;
import java.util.Locale;

/**
 *  Class that wraps a JCDateChooser and a Time Component together.
 */
public class DateTimeEditor extends CalendarContainer {

/** The date chooser. */
protected JCDateChooser          dateChooser = null;

/** The time component.  Either a TimeSpin or TimeLabel object. */
protected CalendarComponent      timeComponent = null;

/** The container which holds the time component. */
protected CalendarContainer      timeContainer = null;

/**
 * A button that sets the current value on the popup calendar and pops down
 * the popup.
 */
protected JButton                applyButton = null;

/** Controls whether the timeComponent is shown. */
protected boolean                showTimeComponent = true;

/** Add an apply button to the popup. */
protected boolean                showApplyButton = false;

/** If true, popdown popup if user clicks on day table. */
protected boolean                hidePopupOnDayTableClick = false;

/**
 * Create a DateTimeChooser with default values and no associated popupEditor.
 */
public DateTimeEditor()
{
    this(JCDateChooser.DUAL_SPIN, true, false, null, null, null, null, null);
}

/**
 * Create a DateTimeChooser that specifies the type of chooser, whether or
 * not to show a time component, the locale, and an intial date/time.
 * @param chooserType The type of date chooser.  Must be one of
 * JCDateChooser.QUICK_SELECT, JCDateChooser.DUAL_SPIN,
 * JCDateChooser.SPIN_POPDOWN, or JCDateChooser.READ_ONLY.  Note that
 * specifying JCDateChooser.SPIN_POPDOWN will cause JCDateChooser.DUAL_SPIN to
 * be used because we cannot have a popdown within a popdown.
 * @param showTimeComponent Determines whether the time component is shown or
 * not.
 * @param showApplyButton If true, an apply button is shown.
 * @param l The locale to use.
 * @param c The intial date and time.
 */
public DateTimeEditor(int chooserType, boolean showTimeComponent,
                      boolean showApplyButton, Locale l, Calendar c)
{
    this(chooserType, showTimeComponent, showApplyButton, l, c,
         null, null, null);
}

/**
 * Create a DateTimeChooser that specifies the type of chooser, whether or
 * not to show a time component, the locale, an intial date/time, and gives a
 * list of month, shortMonth, and day strings to use for the calendar.
 * @param chooserType The type of date chooser.  Must be one of
 * JCDateChooser.QUICK_SELECT, JCDateChooser.DUAL_SPIN,
 * JCDateChooser.SPIN_POPDOWN, or JCDateChooser.READ_ONLY.  Note that
 * specifying JCDateChooser.SPIN_POPDOWN will cause JCDateChooser.DUAL_SPIN to
 * be used because we cannot have a popdown within a popdown.
 * @param showTimeComponent Determines whether the time component is shown or
 * not.
 * @param showApplyButton If true, an apply button is shown.
 * @param l The locale to use
 * @param c The intial date and time.
 * @param months The month strings to use for the calendar.  Note that this
 * array must be the correct size for the locale (or longer).
 * @param shortMonths The shortMonth strings to use for the calendar.  Note that
 * this array must be the correct size for the locale (or longer).
 * @param days The day strings to use for the calendar.  Note that this
 * array must be the correct size for the locale (or longer).
 */
public DateTimeEditor(int chooserType, boolean showTimeComponent,
                      boolean showApplyButton, Locale l, Calendar c,
                      String[] months, String[] shortMonths, String[] days)
{
    if (l == null) {
        l = Locale.getDefault();
    }
    this.setLocale(l);

    if (c == null) {
        c = Calendar.getInstance(getLocale());
    }
    JCValueModel valueModel = new CalendarValueModel(c);

    setLayout(new JCColumnLayout(SwingConstants.CENTER));

    // Setup JCDateChooser
    dateChooser = new DateChooser(valueModel, getLocale(), chooserType,
                                  months, shortMonths, days);

    this.showTimeComponent = showTimeComponent;
    if (showTimeComponent) {
        timeContainer = createTimeContainer(c);
    }
    this.showApplyButton = showApplyButton;
    if (showApplyButton) {
        applyButton = createApplyButton();
    }
    setCalendarModel(valueModel);
    updateLayout(false);
}

/**
 * Create the time component based on the passed in type.  Its always a
 * TimeSpin object unless the type is JCDateChooser.READ_ONLY in which case
 * its a TimeLabel object.
 * @param type The type of chooser
 * @param c The initial time
 * @return The new time component
 */
protected CalendarComponent createTimeComponent(int type, Calendar c)
{
    CalendarComponent comp = null;
    // Setup time component
    if (type == JCDateChooser.READ_ONLY) {
        comp = new TimeLabel(new CalendarValueModel(c), getLocale());
    }
    else {
        TimeSpin timeSpin = new TimeSpin(c.getTime(), getLocale());
        timeSpin.setMinimumDate(dateChooser.getMinimumDate());
        timeSpin.setMaximumDate(dateChooser.getMaximumDate());
        comp = timeSpin;
    }
    if (userFont != null) {
        ((Component)comp).setFont(userFont);
    }
    return(comp);
}

/**
 * Update the time component in response to the user changing the chooser type.
 * @param type The new chooser type
 */
protected void updateTimeComponent(int type)
{
    if (timeContainer == null || timeComponent == null) {
        return;
    }

    // Change only if the time component does currently match the new type
    if (type == JCDateChooser.READ_ONLY ^ timeComponent instanceof TimeLabel) {
        timeContainer.remove((Component)timeComponent);
        Calendar c = (Calendar)model.getValue();
        timeComponent = createTimeComponent(type, c);
        timeContainer.add((Component)timeComponent);
        updateLayout(true);
    }
}

/**
 * Create the time component and the the container which holds it.
 * @param c An initial time for the time component
 * @return The new time container
 */
protected CalendarContainer createTimeContainer(Calendar c)
{
    // Setup time component container
    CalendarContainer container = new CalendarContainer();

    // Create time component
    timeComponent = createTimeComponent(dateChooser.getChooserType(), c);
    container.add((Component)timeComponent);
    return(container);
}

/**
 * Returns the time component.  This may be either a time label (if
 * the chooser type is JCDateChooser.READ_ONLY), a time spinner, or null
 * is showTimeComponent is false.
 * @return The time component. This may be null, if showTimeComponent is false.
 * @see com.klg.jclass.util.calendar.TimeSpin
 * @see com.klg.jclass.util.calendar.TimeLabel
 */
public CalendarComponent getTimeComponent()
{
    return(timeComponent);
}

/**
 * Fire a popup event to all listeners.  Since this is not a popup, this method
 * does nothing here.  Meant to be overridden.
 */
protected void firePopupEvent() {}

/**
 * Create an apply button.
 * @return The newly created apply button.
 */
protected JButton createApplyButton()
{
    JButton button = new JButton("Apply");
    button.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e)
        {
            firePopupEvent();
        }
    });
    return(button);
}

/**
 * Return the applyButton.
 * @return The apply button.  This may be null, if showApplyButton is false.
 */
public JButton getApplyButton()
{
    return(applyButton);
}

/**
 * Update the layout so that we get the dateChooser, then the timeContainer if
 * it is required, and then the apply button if it is required.
 */
protected void updateLayout(boolean doLayout)
{
    removeAll();
    add(dateChooser);
    if (timeContainer != null && showTimeComponent) {
        add(timeContainer);
    }
    if (applyButton != null && showApplyButton) {
        add(applyButton);
        add(JCBrace.createVerticalBrace(3));
    }
    if (doLayout) {
        doLayout();
    }
}

/**
 * Sets the date chooser type.
 * @param type The type of chooser; must be one of
 *     <code>JCDateChooser.QUICK_SELECT</code>,
 *     <code>JCDateChooser.DUAL_SPIN,</code>, or
 *     <code>JCDateChooser.READ_ONLY</code>.  The chooser cannot be of
 *     type <code>JCDateChooser.SPIN_POPDOWN</code> because we cannot have
 *     a popdown within a popdown.
 */
public void setChooserType(int type)
{
    if (getChooserType() == type) {
        return;
    }

    dateChooser.setChooserType(type);
    if (timeComponent != null) {
        updateTimeComponent(type);
    }
}

/**
 * Gets the date chooser type.
 * @return an int representing the chooser type, one of
 * <ul>
 * <li> <code>JCDateChooser.QUICK_SELECT</code></li>
 * <li> <code>JCDateChooser.DUAL_SPIN</code> </li>
 * <li> <code>JCDateChooser.READ_ONLY</code> </li>.  The chooser cannot be of
 *     type <code>JCDateChooser.SPIN_POPDOWN</code> because we cannot have
 *     a popdown within a popdown.
 * </ul>
 */
public int getChooserType()
{
    return(dateChooser.getChooserType());
}

/**
 * Sets whether the time component should appear or not.
 * @param showTimeComponent If true, the time component will appear
 */
public void setShowTimeComponent(boolean showTimeComponent)
{
    if (this.showTimeComponent == showTimeComponent) {
        return;
    }

    if (timeContainer == null) {
        Calendar c = (Calendar)model.getValue();
        timeContainer = createTimeContainer(c);
    }
    this.showTimeComponent = showTimeComponent;
    updateLayout(true);
}

/**
 * Returns whether the time component should appear or not.
 * @return The current value of showTimeComponent
 */
public boolean getShowTimeComponent()
{
    return(showTimeComponent);
}

/**
 * Get whether an apply button be shown in the popup.  If an apply button is
 * shown, clicking on the button applies the value to the model and hides the
 * popup.
 * @return The current value of showApplyButton.
 */
public boolean getShowApplyButton()
{
    return(showApplyButton);
}

/**
 * Set whether an apply button be shown in the popup.  If an apply button is
 * shown, clicking on the button applies the value to the model and hides the
 * popup.
 * @param showApplyButton The new value of showApplyButton.  If true, an apply
 * button is shown in the popup.
 */
public void setShowApplyButton(boolean showApplyButton)
{
    if (this.showApplyButton == showApplyButton) {
        return;
    }

    if (applyButton == null) {
        applyButton = createApplyButton();
    }
    this.showApplyButton = showApplyButton;
    updateLayout(true);
}

/**
 * Get whether the popup will popdown when the user selects a day on the
 * day table.
 * @return The current value of hidePopupOnDayTableClick.
 */
public boolean getHidePopupOnDayTableClick()
{
    return(hidePopupOnDayTableClick);
}

/**
 * Set whether the popup will popdown when the user selects a day on the
 * day table.
 * @param hidePopupOnDayTableClick The new value of hidePopupOnDayTableClick. If
 * true, the popup will popdown when the user selects a day on the day table.
 */
public void setHidePopupOnDayTableClick(boolean hidePopupOnDayTableClick)
{
    this.hidePopupOnDayTableClick = hidePopupOnDayTableClick;
}

/**
 * Returns the value in the data model.
 * @return The current value in the data model
 */
public Object getValue()
{
    return(model.getValue());
}

/**
 * Sets the value in the data model.
 * @param value The value for the data model.
 */
public void setValue(Object value)
{
    model.setValue(value);
}

/**
 * Returns the minimum roll date. The calendar cannot change before the date.
 * @return the minimum date
 * @see java.util.Calendar
 * @see #setMinimumDate
 * @see #getMaximumDate
 * @see #setMaximumDate
 */
public Calendar getMinimumDate()
{
    return(dateChooser.getMinimumDate());
}

/**
 * Sets the minimum date. The calendar cannot change to before this date.
 * @param minDate the new minimum date
 * @see java.util.Calendar
 * @see #getMinimumDate
 * @see #getMaximumDate
 * @see #setMaximumDate
 */
public void setMinimumDate(Calendar minDate)
{
    dateChooser.setMinimumDate(minDate);
    if (timeComponent instanceof TimeSpin) {
        ((TimeSpin)timeComponent).setMinimumDate(minDate);
    }
}

/**
 * Returns the maximum roll date. The calendar cannot change later than this
 * date.
 * @return the maximum date
 * @see java.util.Calendar
 * @see #setMinimumDate
 * @see #getMinimumDate
 * @see #setMaximumDate
 */
public Calendar getMaximumDate()
{
    return(dateChooser.getMaximumDate());
}

/**
 * Sets the maximum roll date. The calendar cannot change later than this date.
 * @param maxDate the new maximum date
 * @see java.util.Calendar
 * @see #setMinimumDate
 * @see #getMinimumDate
 * @see #getMaximumDate
 */
public void setMaximumDate(Calendar maxDate)
{
    dateChooser.setMaximumDate(maxDate);
    if (timeComponent instanceof TimeSpin) {
        ((TimeSpin)timeComponent).setMaximumDate(maxDate);
    }
}

/**
 * Returns the <code>JCDateChooser</code> component.
 * @see com.klg.jclass.util.calendar.JCDateChooser
 * @return The date chooser component
 */
public JCDateChooser getDateChooser()
{
    return(dateChooser);
}

/**
 * The date chooser component.
 */
protected class DateChooser extends JCDateChooser {

    /**
     * Constructs a localized date chooser of a particular type, set to the
     * given date.
     * @param current_date The calendar value model which stores the date
     * @param locale The locale to use
     * @param type The type of chooser; must be one of
     * <code>JCDateChooser.QUICK_SELECT</code>,
     * <code>JCDateChooser.DUAL_SPIN</code>, or
     * <code>JCDateChooser.READ_ONLY</code>.  The chooser cannot be of
     * type <code>JCDateChooser.SPIN_POPDOWN</code> because we cannot have
     * a popdown within a popdown.
     */
    public DateChooser(JCValueModel current_date, Locale locale, int type)
    {
        super(current_date, locale,
              (type == JCDateChooser.SPIN_POPDOWN) ? JCDateChooser.DUAL_SPIN
                                                   : type);
    }

    /**
     * Constructs a localized date chooser of a particular type, set to the
     * given date.
     * @param current_date The calendar value model which stores the date
     * @param locale The locale to use
     * @param type The type of chooser; must be one of
     * <code>JCDateChooser.QUICK_SELECT</code>,
     * <code>JCDateChooser.DUAL_SPIN</code>, or
     * <code>JCDateChooser.READ_ONLY</code>.  The chooser cannot be of
     * type <code>JCDateChooser.SPIN_POPDOWN</code> because we cannot have
     * a popdown within a popdown.
     * @param months A list of month names to use (null means use locale
     * specific names)
     * @param shortMonths A list of short month names to use (null means use
     * locale specific names)
     * @param days A list of day names to use (null means use locale specific
     * names)
     */
    public DateChooser(JCValueModel current_date, Locale locale,
                       int type, String[] months, String[] shortMonths,
                       String[] days)
    {
        super(current_date, locale,
              (type == JCDateChooser.SPIN_POPDOWN) ? JCDateChooser.DUAL_SPIN
                                                   : type,
              months, shortMonths, days);
    }

    /**
     * Sets the date chooser type.
     * @param type The type of chooser; must be one of
     *     <code>JCDateChooser.QUICK_SELECT</code>,
     *     <code>JCDateChooser.DUAL_SPIN,</code>, or
     *     <code>JCDateChooser.READ_ONLY</code>.  The chooser cannot be of
     *     type <code>JCDateChooser.SPIN_POPDOWN</code> because we cannot have
     *     a popdown within a popdown.
     */
    public void setChooserType(int type)
    {
        // Spin popdown not allowed (get a popdown within a popdown)
        if (type == JCDateChooser.SPIN_POPDOWN) {
            type = JCDateChooser.DUAL_SPIN;
        }

        super.setChooserType(type);
    }

    /**
     * Method called when the action happens.
     * @param event The event generated by the action
     */
    public void actionPerformed(ActionEvent event)
    {
        // Only want to process events from the DayTable
        if (event.getSource() == getDayComponent() &&
            event instanceof DateActionEvent)
        {
            DateActionEvent d = (DateActionEvent)event;
            if (getHidePopupOnDayTableClick() ||  d.getHidePopup()) {
                firePopupEvent();
            }
        }
    }
}

}
