/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: DayTable.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:39 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.util.calendar;

import com.klg.jclass.util.JCListenerList;
import com.klg.jclass.util.JCLocaleManager;
import com.klg.jclass.util.JCEnvironment;
import com.klg.jclass.util.value.JCValueEvent;
import com.klg.jclass.util.value.JCValueListener;
import com.klg.jclass.util.value.JCValueModel;
import com.klg.jclass.util.value.CalendarValueModel;

import javax.swing.*;
import javax.swing.plaf.ComponentUI;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.border.SoftBevelBorder;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.*;

import java.awt.*;
import java.awt.event.*;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.Locale;

/**
 * DayTable is the table of days used to form the day portion of a calendar.
 */
public class DayTable extends JScrollPane
			          implements CalendarComponent, MouseListener,
					             MouseMotionListener
{

transient JCLocaleManager li = JCLocaleManager.getDefault();

protected JCValueModel   selectedDate;
protected JCCalendar     specialDates;
protected int            componentSpacing = 4;

// normal background = UIManager.get"Label.background"
// normal foreground = "Label.foreground"
protected Color          otherBackground;     // "Label.background"
protected Color          otherForeground;     // "Label.disabledForeground"
protected Color          specialBackground;   // "Label.background"
protected Color          specialForeground;   // "TextField.foreground"
protected Color          selectionBackground; // "TextField.selectionBackground"
protected Color          selectionForeground; // "TextField.selectionForeground"

protected int            tableCellSize;
protected int            cellSize = 0;
protected Locale         locale;
protected String[]       days = null;
protected boolean        userSetDays = false;
protected int            firstDayOfWeek;
protected int            year;
protected int            month;
protected int            day;
protected int            startingCell;
protected boolean        isDragging = false;

/* We always have 6 rows (+1 for header) so that we can draw spillover months */
protected static int     NUM_ROWS = 6;
protected static int     NUM_DAYS = 7;
protected static int     MIN_CELL_SIZE = 25;
protected JTable         table;
protected boolean        editable = true;

/** A list of action listeners */
protected JCListenerList actionListeners = null;

/** Date used when click-dragging date. */
protected Calendar       intermediateDate = null;

/** The model used for the table */
protected DayModel       dayModel;

/**
 * Creates a DayTable with specified a calendar, list of special dates, and
 * locale.
 * <b>Note:</b>The locale passed in is used to calculate locale specific values,
 * like the default day of week names and the first day of week, not the locale
 * of calendar model's calendar value
 * @param date The calendar value model to use (which stores a date)
 * @param special_dates The list of special dates for this calendar stored
 *     within a JCCalendar object
 * @param locale The locale to be used
 * @see java.util.Calendar
 * @see java.util.Locale
 */
public DayTable(JCValueModel date, JCCalendar special_dates, Locale locale)
{
	this(date, special_dates, locale, null);
}

/**
 * Creates a DayTable with a specified calendar model, list of special dates,
 * locale, and set of day strings.
 * <b>Note:</b>The locale passed in is used to calculate locale specific values,
 * like the default day of week names and the first day of week, not the locale
 * of calendar model's calendar value
 * @param date The calendar value model to use (which stores a date)
 * @param special_dates The list of special dates for this calendar stored
 *     within a JCCalendar object
 * @param locale The locale to be used
 * @param days A list of day of week names to use (null means use locale
 * specific names)
 * @see java.util.Calendar
 * @see java.util.Locale
 */
public DayTable(JCValueModel date, JCCalendar special_dates, Locale locale,
	            String[] days)
{
	this.days = days;
	if (this.days != null) {
		userSetDays = true;
	}
	table = new JTable();
	setHorizontalScrollBarPolicy(HORIZONTAL_SCROLLBAR_NEVER);
	setVerticalScrollBarPolicy(VERTICAL_SCROLLBAR_NEVER);
	setViewportView(table);

	table.setIntercellSpacing(new Dimension(0, 0));
	table.setDefaultRenderer(Integer.class, new Renderer());
	table.getTableHeader().setReorderingAllowed(false);
	table.getTableHeader().setResizingAllowed(false);

	// Set the selected calendar
    if (date == null) {
        date = new CalendarValueModel(Calendar.getInstance());
    }
	selectedDate = date;
	specialDates = special_dates;

    if (locale == null) {
        locale = Locale.getDefault();
    }

    setLocale(locale);
	setCalendarModel(date);

	table.addMouseListener(this);
	table.addMouseMotionListener(this);
}

/**
 * Set whether the DayTable will allow the selection of a day or not.
 * @param e true for editable false for non-editable.
 */
public void setEditable(boolean e)
{
	editable = e;
}

/**
 * Test whether the DayTable is editable.
 * @return Can the dates be changed interactively on this DayTable?
 */
public boolean isEditable()
{
	return(editable);
}

/**
 * Set the background color of the selected date.
 * @param c The new selected date background color
 */
public void setSelectedDateBackground(Color c)
{
	if (this.selectionBackground == c) {
		return;
	}
	if (c == null) {
		throw new IllegalArgumentException("null is an invalid color");
	}

	firePropertyChange("selectionBackground", this.selectionBackground, c);
	this.selectionBackground = c;
	repaint();
}

/**
 * Set the foreground color of selected date.
  * @param c The new selected date foreground color
*/
public void setSelectedDateForeground(Color c)
{
	if (this.selectionForeground == c) {
		return;
	}
	if (c == null) {
		throw new IllegalArgumentException("null is an invalid color");
	}

	firePropertyChange("selectionForeground", this.selectionForeground, c);
	this.selectionForeground = c;
	repaint();
}

/**
 * Set the background color of special dates.
 * @param c The new special date background color
 */
public void setSpecialDateBackground(Color c)
{
	if (this.specialBackground == c) {
		return;
	}
	if (c == null) {
		throw new IllegalArgumentException("null is an invalid color");
	}

	firePropertyChange("specialBackground", this.specialBackground, c);
	this.specialBackground = c;
	repaint();
}

/**
 * Set the foreground color of special dates.
 * @param c The new special date foreground color
 */
public void setSpecialDateForeground(Color c)
{
	if (this.specialForeground == c) {
		return;
	}
	if (c == null) {
		throw new IllegalArgumentException("null is an invalid color");
	}

	firePropertyChange("specialForeground", this.specialForeground, c);
	this.specialForeground = c;
	repaint();
}

/**
 * Set the background color of other information (not normal dates and
 * special dates.
 * @param c The new other date background color
 */
public void setOtherDateBackground(Color c)
{
	if (this.otherBackground == c) {
		return;
	}
	if (c == null) {
		throw new IllegalArgumentException("null is an invalid color");
	}

	firePropertyChange("otherBackground", this.otherBackground, c);
	this.otherBackground = c;
	repaint();
}

/**
 * Set the foreground color of other information (not normal dates and
 * special dates.
 * @param c The new other date foreground color
 */
public void setOtherDateForeground(Color c)
{
	if (this.otherForeground == c) {
		return;
	}
	if (c == null) {
		throw new IllegalArgumentException("null is an invalid color");
	}

	firePropertyChange("otherForeground", this.otherForeground, c);
	this.otherForeground = c;
	repaint();
}

/**
 * Set the locale the DayTable should use.
 * @param l The locale object to use.
 */
public void setLocale(Locale l)
{
    // Don't do anything if table is null (i.e. before the constructor is
    // executed.  This happens in JDK 1.4.0).
    if (table != null) {
        if (l == null) {
            throw new IllegalArgumentException("null is an invalid value " +
                                               "for locale");
        }
        if (locale == l) {
            return;
        }
        firePropertyChange("locale", locale, l);
        locale = l;

        // Calculate the first day of week. If the calendar returns a non-zero
        // value (such as 1 in European locales), this means Sunday is not the
        // first day of the week.  We must convert to a 0-based mechanism.
        Calendar c = Calendar.getInstance(locale);
        firstDayOfWeek = (c.getFirstDayOfWeek() + 6) % NUM_DAYS;

        calculateCalendar();

        if (dayModel != null && selectedDate != null) {
            selectedDate.removeValueListener(dayModel);
        }
        dayModel = new DayModel(locale);
        if (selectedDate != null) {
            selectedDate.addValueListener(dayModel);
        }

        // Post a complete table changed event
        table.setLocale(locale);
        table.setModel(dayModel);
        adjustSize();
    }
}

/**
  * Get the locale object being used by the DayTable.
  * @return The current locale object
  */
public Locale getLocale()
{
	return(locale);
}

/**
 * Override setFont to store the table font.
 * @param font The new font
 */
public void setFont(Font font)
{
    super.setFont(font);
    if (table != null) {
        table.setFont(font);
        table.getTableHeader().setFont(font);
        adjustSize();
    }
}

    /**
 * Set the value model the DayTable should use.  The model must a calendar
 * value model.
 * <b>Note:</b>The DayTable's current locale is used to calculate locale
 * specific values, like the default day of week names and the first day of
 * week, not the locale of calendar model's calendar value
 * @param model The calendar value model to use.
 */
public void setCalendarModel(JCValueModel model)
{
	if (dayModel != null && selectedDate != null) {
		selectedDate.removeValueListener(dayModel);
	}
	selectedDate = model;
	selectedDate.addValueListener(dayModel);
    calculateCalendar();
	table.repaint();
}

/**
 * Get the value model the DayTable is using.
 * @return The current calendar value model
 */
public JCValueModel getCalendarModel()
{
	return(selectedDate);
}

/**
 * Set special dates to appear on the DayTable.
 * @param sd The new list of special dates stored within a JCCalendar object
 */
public void setSpecialDates(JCCalendar sd)
{
	specialDates = sd;
}

/**
 * Get the special dates being used by the DayTable.
 * @param sd Unused
 * @deprecated As the parameter is unused, should use getSpecialDates()
 * @return The current list of special dates stored within a JCCalendar object
 */
public JCCalendar getSpecialDates(JCCalendar sd)
{
	return(getSpecialDates());
}

/**
 * Get the special dates being used by the DayTable.
 * @return The current list of special dates stored within a JCCalendar object
 */
public JCCalendar getSpecialDates()
{
	return(specialDates);
}

/**
 * Add listener to receive action event when data is changed by the user.
 * @param l The action listener to be added
 */
public void addActionListener(ActionListener l)
{
	actionListeners = JCListenerList.add(actionListeners, l);
}

/**
 * Remove listener to receive action event when data is changed by the user.
 * @param l The action listener to be removed
 */
public void removeActionListener(ActionListener l)
{
	actionListeners = JCListenerList.remove(actionListeners, l);
}

/**
 * Adjust the size of the table
 */
protected void adjustSize()
{
    // Calculate cell size and sets the row and column sizes
    calculateCellSize();
    Dimension headerPrefSize = table.getTableHeader().getPreferredSize();
    table.setPreferredSize(new Dimension(NUM_DAYS * tableCellSize,
                                         (NUM_ROWS * tableCellSize) +
                                         headerPrefSize.height));
}

/**
 * Override addNotify to also set the size of the table.
 *
 * @see JComponent#addNotify
 */
public void addNotify()
{
	super.addNotify();
    adjustSize();
}

/**
 * Sets the size of the cells in the table.  If this value is greater than zero,
 * this value is used for all cells (regardless of whether the text fits).
 * @param cellSize The new cellSize (in pixels)
 */
public void setCellSize(int cellSize)
{
    this.cellSize = cellSize;
}

/**
 * Returns the current cell size used.  If the user has specificied a cell size
 * that is greater than zero, this value is returned. Otherwise, return the
 * computed cell size based on the size of the text.
 * @return The current table cell size used.
 */
public int getCellSize()
{
    return((cellSize > 0) ? cellSize : tableCellSize);
}

/**
 * Find out which cell to begin drawing - first day of week may not start
 * on Sunday.
 */
protected void calculateCalendar()
{
    Calendar c = (Calendar)selectedDate.getValue();
    month      = c.get(c.MONTH);
    year       = c.get(c.YEAR);
    day        = c.get(c.DATE);
    startingCell = JCCalendar.dayOfWeek(1, month, year);
    if (firstDayOfWeek != 0) {
        startingCell = ((startingCell - firstDayOfWeek) + NUM_DAYS) % NUM_DAYS;
    }
}

/**
 * Calculates the cell size. <p>
 * Calendar cells are square so the cell size is the maximum
 * of 1) the max width of the days of the week, 2) the width of
 * a double digit, and 3) the height of a digit.
 */
protected void calculateCellSize()
{
    if (table == null) {
        return;
    }
    TableColumnModel columnModel = table.getColumnModel();
    if (cellSize > 0) {
        tableCellSize = cellSize;
    }
    else {
        tableCellSize = MIN_CELL_SIZE;
        TableModel model = table.getModel();
        for (int col = 0; col < columnModel.getColumnCount(); col++) {
            TableColumn column = columnModel.getColumn(col);

            // Calculate header cell width for this column first.
            TableCellRenderer renderer = column.getHeaderRenderer();
            // If no renderer set then we use default one
            if (renderer == null) {
                renderer = table.getTableHeader().getDefaultRenderer();
            }
            Component comp =
                renderer.getTableCellRendererComponent(table,
                                                       column.getHeaderValue(),
                                                       false, false, -1, -1);
            int headerCellWidth =
                comp.getPreferredSize().width + componentSpacing;

            // Next calculate cell widths for the current column.
            renderer = column.getCellRenderer();
            // If no renderer set then we use default one
            if (renderer == null) {
                Class columnClass = model.getColumnClass(col);
                renderer = table.getDefaultRenderer(columnClass);
            }
            int colCellWidth = 0;
            for (int row = 0; row < table.getRowCount(); row++) {
                Object value = model.getValueAt(row, col);
                comp = renderer.getTableCellRendererComponent(table, value,
                                                              false, false,
                                                              row, col);
                colCellWidth = Math.max(colCellWidth,
                                        comp.getPreferredSize().width);
            }

            // Update the table cell size
            int cellWidth = Math.max(colCellWidth + componentSpacing,
                                     headerCellWidth);
            tableCellSize = Math.max(tableCellSize, cellWidth);
        }
    }

	// Set height of rows to the newly calculated cell size
	table.setRowHeight(tableCellSize);

	// SetColumnWidths to cellSize
	Enumeration e = columnModel.getColumns();
	while (e.hasMoreElements()) {
		TableColumn column = (TableColumn) e.nextElement();
		column.setMinWidth(tableCellSize);
        if (JCEnvironment.getJavaVersion() < 130) {
            column.setWidth(tableCellSize);
        }
	}
}

/**
 * Fire the Action events.
 */
public void fireActionEvent()
{
    fireActionEvent(false);
}

protected void fireActionEvent(boolean hidePopup)
{
    ActionEvent ae = new DateActionEvent(this, ActionEvent.ACTION_PERFORMED,
                                         "DateSelected", hidePopup);
    Enumeration e = JCListenerList.elements(actionListeners);
    while (e.hasMoreElements()) {
        ActionListener listener = (ActionListener)e.nextElement();
        listener.actionPerformed(ae);
    }
}

//
// MouseListener implementation
//

/**
 * Returns the day given the row and column on the DayTable.
 * @param row The row index
 * @param column The column index
 * @return The day based on the row and column
 */
public int dayFromRowColumn(int row, int column)
{
    return(column - (startingCell - 1) + (row * NUM_DAYS));
}

/**
 * Calculate the day on the table which corresponds to the passed in point.
 * @param point A point on the table.
 * @return An integer representing the day selected.  Note that a negative value
 * means that the point selected is in the previous month.  A value greater than
 * the number of days in the month means the point selected is in the next
 * month.
 */
protected int getDayFromPoint(Point point)
{
    int row = table.rowAtPoint(point);
    int column = table.columnAtPoint(point);
    return(dayFromRowColumn(row, column));
}

/**
 * Return a copy of the current selected date with the day changed to the
 * passed in value.
 * @param dayValue The day to use.
 * @return A Calendar object which is a copy of the selected date which the date
 * changed to the passed in value.
 */
protected Calendar getCalendarFromDay(int dayValue)
{
    Calendar c = JCCalendar.copyCalendar((Calendar)selectedDate.getValue());
    c.set(c.DATE, dayValue);
    return(c);
}

/**
 * Calculate the day on the table which corresponds to the passed in point.
 * @param point A point on the table
 * @return A Calendar object with the date corresponding to the passed in point.
 */
protected Calendar getCalendarFromPoint(Point point)
{
    return(getCalendarFromDay(getDayFromPoint(point)));
}

/**
 * Is passed in day, within the current month?
 * @param dayValue The day value to check.
 * @return true, if the day is within the current month.
 */
protected boolean isDayInCurrentMonth(int dayValue)
{
    return(dayValue >= 1 && dayValue <= JCCalendar.daysInMonth(month, year));
}

/**
 * Is the passed in calendar valid?  If the parent is a JCDateChooser, check
 * the date thru its bounds validator. If there is no parent, the all dates are
 * valid.
 * @param c The date (in a Calendar object) to check
 * @return true, if the date is valid.
 */
protected boolean isValidDate(Calendar c)
{
    boolean valid = true;
    Container parent = getParent();
    if (parent instanceof JCDateChooser) {
        valid = ((JCDateChooser)parent).boundsValidator.isValid(c);
    }
    return(valid);
}

/**
 * React to a mousePressed event.
 * @param ev The mouse event generated
 */
public void mousePressed(MouseEvent ev)
{
	if (!editable) {
		return;
	}
    Calendar c = getCalendarFromPoint(ev.getPoint());
    if (isValidDate(c)) {
	    intermediateDate = c;
        table.repaint();
    }
}

/**
 * React to a mouseClicked event.
 * @param e The mouse event generated
 */
public void mouseClicked(MouseEvent e) {}

/**
 * React to a mouseReleased event.
 * @param ev The mouse event generated
 */
public void mouseReleased(MouseEvent ev)
{
	if (!editable) {
        if (ev.getClickCount() >= 2) {
            fireActionEvent(true);
        }
		return;
	}
    intermediateDate = null;

    // If dragging, make sure mouse release is in the current month.  Note that
    // if not dragging, any day is valid.
    int dayValue = getDayFromPoint(ev.getPoint());
    boolean validDay = (!isDragging || isDayInCurrentMonth(dayValue));
    isDragging = false;
    if (!validDay) {
        return;
    }

    Calendar c = getCalendarFromDay(dayValue);
    boolean hidePopup = false;
    if (c.equals(selectedDate.getValue()) && ev.getClickCount() >= 2) {
        // The user has double clicked on the calendar on the same date.  Note
        // that we could get a click count of more than 1 with a different date
        // if the user double clicks on a cell outside the current month.  In
        // this case we want to hide the popup.
        hidePopup = true;
    }
    else {
	    selectedDate.setValue(c);
    }
    fireActionEvent(hidePopup);
	table.repaint();
}

/**
 * mouseEntered.
 * @param e The mouse event generated
 */
public void mouseEntered(MouseEvent e) {}

/**
 * mouseExited.
 * @param e The mouse event generated
 */
public void mouseExited(MouseEvent e) {}

//
// MouseMotionListener implementation
//

/**
 * mouseDragged.
 * @param e The mouse event generated
 */
public void mouseDragged(MouseEvent e)
{
	if (!editable) {
		return;
	}
    int dayValue = getDayFromPoint(e.getPoint());
    if (isDayInCurrentMonth(dayValue)) {
        Calendar c = getCalendarFromDay(dayValue);
        if (isValidDate(c)) {
            intermediateDate = c;
            isDragging = true;
            table.repaint();
        }
    }
}

/**
 * mouseMoved.
 * @param e The mouse event generated
 */
public void mouseMoved(MouseEvent e) {}

/**
 * Gets the preferred size of the DayTable.
 * @return The calculated preferred size of this component
 */
public Dimension getPreferredSize()
{
    Insets insets = getInsets();
    int heightAdj =
        JCEnvironment.isWorking(JCEnvironment.MOTIF_PREFERED_WIDTH_AJUSTMENT) +
        insets.top + insets.bottom;
    Dimension headerPrefSize = table.getTableHeader().getPreferredSize();
    Dimension dimension = new Dimension(NUM_DAYS * tableCellSize +
                                        insets.left + insets.right,
                                        (NUM_ROWS * tableCellSize) +
                                        headerPrefSize.height + heightAdj);
    return(dimension);
}

/**
 * Test to see if the day at a certain row and column is selected.
 * @param row The row index
 * @param column The column index
 * @return Whether the day is selected or not
 */
public boolean isSelectedDate(int row, int column)
{
	int dayValue = dayFromRowColumn(row, column);
	if (intermediateDate != null) {
		Calendar c = intermediateDate;
		if (dayValue == c.get(c.DATE)) {
			return(true);
		}
	}
	else if (dayValue == day) {
		return(true);
	}
	return(false);
}

/**
 * Test to see if the day at a certain row and column is in the month that has
 * been selected to be displayed.
 * @param row The row index
 * @param column The column index
 * @return Whether the day is in the currently displayed month
 */
public boolean isSelectedMonth(int row, int column)
{
	int dayValue = dayFromRowColumn(row, column);
	if (isDayInCurrentMonth(dayValue))  {
		return(true);
	}
	return(false);
}

/**
 * Set the UI.
 * @param ui The new ui
 */
protected void setUI(ComponentUI ui)
{
	super.setUI(ui);
	inferLookAndFeel();
}

/**
 * Update the UI.
 */
public void updateUI()
{
	super.updateUI();
	inferLookAndFeel();
}

/**
 * Set Look and Feel specific values (note that these overrided user specified
 * values).
 */
protected void inferLookAndFeel()
{
	setBackground(UIManager.getColor("Label.background"));
	setForeground(UIManager.getColor("Label.foreground"));
	setSpecialDateBackground(UIManager.getColor("Label.background"));
	setSpecialDateForeground(Color.blue);
	setSelectedDateBackground(UIManager.getColor("TextField.selectionBackground"));
	setSelectedDateForeground(UIManager.getColor("TextField.selectionForeground"));
	setOtherDateBackground(UIManager.getColor("Label.background"));
	setOtherDateForeground(UIManager.getColor("Label.disabledForeground"));
}

/**
 * Use a table cell renderer to draw current day in reverse video
 */
protected class Renderer extends JLabel implements TableCellRenderer,
							                       java.io.Serializable {

	protected Border bevel = new SoftBevelBorder(BevelBorder.LOWERED);
	protected Border empty = new EmptyBorder(0, 0, 0, 0);

    /**
     * Get the table cell renderer component.
	 * @param table The table for which to render
	 * @param value The value in the cell
	 * @param isSelected Is the cell selected?
	 * @param hasFocus Does the cell have focus?
	 * @param row The row index of the cell
	 * @param column The column index of the cell
	 * @return The component which will renderer for the given cell.  Namely
	 *    this component.
     */
	public Component getTableCellRendererComponent(JTable table,
												   Object value,
												   boolean isSelected,
												   boolean hasFocus,
												   int row,
												   int column)
	{
		setHorizontalAlignment(SwingConstants.CENTER);
        setFont(table.getFont());

		// Check if the cell we are drawing is the selected month
		if (isSelectedDate(row, column)) {
			setForeground(selectionForeground);
			setBackground(selectionBackground);
			setBorder(bevel);
			setOpaque(true);
		}
		else {
			if (isSelectedMonth(row, column)) {
				int currentDay = dayFromRowColumn(row, column);
				if (specialDates != null &&
					specialDates.isSpecialDate(year, month, currentDay,
											   row + 1))
				{
					setForeground(specialForeground);
					setBackground(specialBackground);
				}
				else {
					setForeground(DayTable.this.getForeground());
					setBackground(DayTable.this.getBackground());
				}
			}
			else {
				setForeground(otherForeground);
				setBackground(otherBackground);
			}
			setBorder(empty);
			setOpaque(true);
		}

		if (value == null) {
			setText("");
		}
		else {
			setText(value.toString());
		}
		return(this);
	}
}

/**
 * Build data model wrapper around DateSymbols that
 * builds a 7x6 table of month with the current date for picking.
 */
protected class DayModel implements TableModel, JCValueListener,
									java.io.Serializable
{
	protected JCListenerList tableListeners = null;

    /**
     * Construct a DayModel with a certain locale.
	 * @param l The locale from which the short weekday names is garnered
     */
	public DayModel(Locale l)
	{
		if (!userSetDays) {
			days = JCCalendar.getDays(l);
		}
	}

	//
	// TableModel Implementation
	//

    /**
     * Get the number of rows.
	 * @return The current number of rows
     */
	public int getRowCount()
	{
		return(NUM_ROWS);
	}

    /**
     * Get the number of columns.
	 * @return The current number of columns
     */
	public int getColumnCount()
	{
		return(NUM_DAYS);
	}

    /**
     * Get the name of a particular column.
	 * @param columnIndex The index of column whose name is required
	 * @return The column name of the given column
     */
	public String getColumnName(int columnIndex)
	{
		return(days[(columnIndex + firstDayOfWeek) % 7]);
	}

    /**
     * Get the class of a particular column.
	 * @param columnIndex The index of column whose class type is required
	 * @return The class type of the given column
     */
	public Class getColumnClass(int columnIndex)
	{
		return(Integer.class);
	}

    /**
     * Test to see if the cell is editable.
	 * @param rowIndex The row index of the cell
	 * @param columnIndex The column index of the cell
	 * @return false as all cells are not editable
     */
	public boolean isCellEditable(int rowIndex, int columnIndex)
	{
		return(false);
	}

    /**
     * Return the value at the given row and column.
	 * @param rowIndex The row index of the cell
	 * @param columnIndex The column index of the cell
	 * @return The value for the given row and column indices
     */
	public Object getValueAt(int rowIndex, int columnIndex)
	{
		int dayValue = dayFromRowColumn(rowIndex, columnIndex);
		if (dayValue < 1) {
			int prev_year;
			int prev_month;
			if ((month - 1) < 0) {
				// Handle wrap around
				prev_month = 11;
				prev_year = year - 1;
			}
			else {
				prev_month = month - 1;
				prev_year = year;
			}
			return(new Integer(JCCalendar.daysInMonth(prev_month, prev_year) -
							   (startingCell - columnIndex - 1)));
		}
		else if (dayValue > JCCalendar.daysInMonth(month, year)) {
			return(new Integer(dayValue - JCCalendar.daysInMonth(month, year)));
		}
		else {
			return(new Integer(dayValue));
		}
	}

    /**
     * Set the value at the given row and column.  Since cells are readonly
	 * this method always throws an exception.
	 * @param aValue The value to be set
	 * @param rowIndex The row index of the cell
	 * @param columnIndex The column index of the cell
	 * @throws IllegalArgumentException
     */
	public void setValueAt(Object aValue, int rowIndex, int columnIndex)
	{
		throw new IllegalArgumentException("This TableModel is READ ONLY");
	}

    /**
     * Add a table listener to the model.
	 * @param l The table model listener to be added
     */
	public void addTableModelListener(TableModelListener l)
	{
		tableListeners = JCListenerList.add(tableListeners, l);
	}

    /**
     * Remove a table listener from the model.
	 * @param l The table model listener to be removed
     */
	public void removeTableModelListener(TableModelListener l)
	{
		tableListeners = JCListenerList.remove(tableListeners, l);
	}

    /**
     * Return a copy of the calender.
	 * @param cal The calendar to be copied
	 * @return The new replica calendar
     */
	protected Calendar copyCalendar(Calendar cal)
	{
		if (cal == null) {
			return(null);
		}
		return((Calendar) cal.clone());
	}

	//
	// JCValueModel implementation
	//

	/**
	 * The value is changing.  The listener may further change the new_value or
	 * abort the change altogether.  If the case of an abort (call
	 * event.setAllowChange(false)) the "valueChanged"  method will never be
	 * called.
	 * @param event The event generated by the value changing
	 */
	public void valueChanging(JCValueEvent event) {}

	/**
	 * The method informs the listener that the value has been changed.
	 * @param event The event generated by the value being changed
	 */
	public void valueChanged(JCValueEvent event)
	{
		if (event == null) {
			return;
		}

		calculateCalendar();

		TableModelEvent tableEvent = new TableModelEvent(this);
		Enumeration e = JCListenerList.elements(tableListeners);
		while (e.hasMoreElements()) {
			TableModelListener listener = (TableModelListener)e.nextElement();
			listener.tableChanged(tableEvent);
		}
	}
}

}


