/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCDateChooser.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:39 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.calendar;

import com.klg.jclass.util.JCVersion;
import com.klg.jclass.util.JCEnvironment;
import com.klg.jclass.util.value.CalendarValueModel;
import com.klg.jclass.util.value.JCValueModel;

import javax.swing.*;
import javax.swing.table.TableColumn;
import javax.swing.plaf.ComponentUI;
import java.awt.*;
import java.io.Serializable;
import java.util.Calendar;
import java.util.Locale;
import java.util.Enumeration;

/**
 * Date chooser component.  Displays a calendar and allows the user to select a
 * date.
 */
public class JCDateChooser extends CalendarContainer
			               implements Serializable
{

/** Valid value for <code>ChooserType</code>. */
public static final int QUICK_SELECT = 1;
/** Valid value for <code>ChooserType</code>. */
public static final int DUAL_SPIN    = 2;
/** Valid value for <code>ChooserType</code>. */
public static final int SPIN_POPDOWN = 3;
/** Valid value for <code>ChooserType</code>. */
public static final int READ_ONLY    = 4;

protected int                chooserType;
protected CalendarComponent  year;  // comp that renders/edits the current year
protected CalendarComponent  month; // comp that renders/edits the current month
protected CalendarComponent  day;   // comp that renders/edits the current day

protected String[]           months = null;
protected String[]           shortMonths = null;
protected String[]           days = null;

// TODO: componentSpacing should be exposed as a property
protected int componentSpacing = 4;

protected BoundsValidator boundsValidator = new BoundsValidator(this);

/**
 * Constructs a default date chooser.  Will show the current date.
 */
public JCDateChooser()
{
	this(new CalendarValueModel(Calendar.getInstance()),
		 Locale.getDefault(), QUICK_SELECT, null, null, null);
}

/**
 * Creates a calendar using the specified Calendar and locale.
 * The <code>Calendar</code> object must be created with the specified locale
 * already set.
 * @see java.util.Calendar
 * @see java.util.Locale
 * @param current_date the <code>Calendar</code> object must be created with the
 * specified locale
 * @param locale the locale to use
 */
public JCDateChooser(JCValueModel current_date, Locale locale)
{
	this(current_date, locale, QUICK_SELECT, null, null, null);
}

/**
 * Constructs a localized date chooser of a particular type, set to the
 * given date.
 * @param current_date the calendar value model which stores the date
 * @param locale the locale to use
 * @param chooser_type the type of chooser; must be one of
 *     <code>JCDateChooser.QUICK_SELECT</code>,
 *     <code>JCDateChooser.DUAL_SPIN</code>,
 *     <code>JCDateChooser.SPIN_POPDOWN</code>, or
 *     <code>JCDateChooser.READ_ONLY</code>
 */
public JCDateChooser(JCValueModel current_date, Locale locale,
					 int chooser_type)
{
	this(current_date, locale, chooser_type, null, null, null);
}

/**
 * Constructs a localized date chooser of a particular type, set to the
 * given date.
 * @param current_date the calendar value model which stores the date
 * @param locale the locale to use
 * @param chooser_type the type of chooser; must be one of
 *     <code>JCDateChooser.QUICK_SELECT</code>,
 *     <code>JCDateChooser.DUAL_SPIN</code>,
 *     <code>JCDateChooser.SPIN_POPDOWN</code>, or
 *     <code>JCDateChooser.READ_ONLY</code>
 * @param months a list of month names to use (null means use locale specific
 *     names)
 * @param shortMonths a list of short month names to use (null means use locale
 *     specific names)
 * @param days a list of day names to use (null means use locale specific names)
 */
public JCDateChooser(JCValueModel current_date, Locale locale,
					 int chooser_type, String[] months, String[] shortMonths,
					 String[] days)
{
	this.months      = months;
	this.shortMonths = shortMonths;
	this.days        = days;
	setCalendarModel(current_date);
	setLocale(locale);
	setLayout(new Layout());
	setChooserType(chooser_type);
}

/**
 * Gets the date chooser type.
 * @return an int representing the chooser type, one of
 * <ul>
 * <li> <code>JCDateChooser.QUICK_SELECT</code></li>
 * <li> <code>JCDateChooser.DUAL_SPIN</code> </li>
 * <li> <code>JCDateChooser.SPIN_POPDOWN</code> </li>
 * <li> <code>JCDateChooser.READ_ONLY</code> </li>
 * </ul>
 */
public int getChooserType()
{
	return(chooserType);
}

/**
 * Sets the date chooser type.
 * @param type the type of chooser; must be one of
 *     <code>JCDateChooser.QUICK_SELECT</code>,
 *     <code>JCDateChooser.DUAL_SPIN,</code>
 *     <code>JCDateChooser.SPIN_POPDOWN</code>, or
 *     <code>JCDateChooser.READ_ONLY</code>
 */
public void setChooserType(int type)
{
    if (type < JCDateChooser.QUICK_SELECT || type > JCDateChooser.READ_ONLY) {
        throw new IllegalArgumentException("Chooser type must be one of " +
                                           "JCDateChooser.QUICK_SELECT," +
                                           "JCDateChooser.DUAL_SPIN," +
                                           "JCDateChooser.SPIN_POPDOWN, " +
                                           "or JCDateChooser.READ_ONLY");
    }

	if (chooserType == type) {
		return;
	}

	createComponents(type);
	chooserType = type;
}

/**
 * Calculates the container size.
 */
protected void forceRelayout()
{
	if (getParent() != null) {
		getParent().invalidate();
	}
}

/**
 * Gets the version.
 * @return the version String
 */
public String getVersion()
{
	return(JCVersion.getVersionString());
}

/**
 * Provided for Beans property editors - has no effect.
 * @param s ignored
 */
public void setVersion(String s) {}

/**
 * Gets the value of the <code>About</code> property.
 * The <code>About</code> property displays contact information for
 * Quest Software in the Bean box.
 * @return the about String
 */
public String getAbout()
{
    return(JCVersion.getVersionString());
}

/**
 * Provided for Beans property editors - has no effect.
 * The <code>About</code> property displays contact information for
 * Quest Software in the bean box.
 * @param s ignored
 */
public synchronized void setAbout(String s) {}

/**
 * Returns the selected calendar. From the <code>Calendar</code> class, the
 * year, month, and date can be retrieved.
 * @return the current calendar value model's value
 * @see java.util.Calendar
 * @see #setValue
 */
public Calendar getValue()
{
	return((Calendar) model.getValue());
}

/**
 * Sets the selected Calendar. <br><B>Note:</b> The Calendar set must have the
 * same locale as the existing Calendar, otherwise the results are undefined.
 * @param calendar the new value for the current calendar value model
 * @see java.util.Calendar
 * @see #getValue
 */
public void setValue(Calendar calendar)
{
	model.setValue(calendar);
}

/**
 * Returns the minimum roll date. The calendar cannot change before the date.
 * @return the minimum date
 * @see java.util.Calendar
 * @see #setMinimumDate
 * @see #getMaximumDate
 * @see #setMaximumDate
 */
public Calendar getMinimumDate()
{
	return(boundsValidator.getMinimumDate());
}

/**
 * Sets the minimum date. The calendar cannot change to before this date.
 * @param minDate the new minimum date
 * @see java.util.Calendar
 * @see #getMinimumDate
 * @see #getMaximumDate
 * @see #setMaximumDate
 */
public void setMinimumDate(Calendar minDate)
{
	boundsValidator.setMinimumDate(minDate);
}

/**
 * Returns the maximum roll date. The calendar cannot change later than this
 * date.
 * @return the maximum date
 * @see java.util.Calendar
 * @see #setMinimumDate
 * @see #getMinimumDate
 * @see #setMaximumDate
 */
public Calendar getMaximumDate()
{
	return(boundsValidator.getMaximumDate());
}

/**
 * Sets the maximum roll date. The calendar cannot change later than this date.
 * @param maxDate the new maximum date
 * @see java.util.Calendar
 * @see #setMinimumDate
 * @see #getMinimumDate
 * @see #getMaximumDate
 */
public void setMaximumDate(Calendar maxDate)
{
	boundsValidator.setMaximumDate(maxDate);
}

/**
 * Checks if the passed in months array is long enough.
 * @param months the months array to check
 * @param isShort Are these short month names?
 * @return Is the month array OK to use?
 */
protected boolean checkMonths(String[] months, boolean isShort)
{
	boolean ret = true;
	String[] localeMonths = JCCalendar.getMonths(getLocale(), isShort);
	if (months != null && months.length < localeMonths.length) {
		ret = false;
		throw new IllegalArgumentException("The months array must be at " +
                                           "least " + localeMonths.length +
                                           " in size.");
	}
	return(ret);
}

/**
 * Gets the months array used for month names.  If the return value is null,
 * the locale specific month names will be used.
 * @return the current months array used for month names
 */
public String[] getMonths()
{
	return(months);
}

/**
 * Sets the months array used for month names.  If the value is null,
 * the locale specific month names will be used.
 * @param months the new months array used for month names
 */
public void setMonths(String[] months)
{
	if (checkMonths(months, false)) {
	    this.months = months;
	    createComponents(chooserType);
	}
}

/**
 * Gets the months array used for month names.  If the return value is null,
 * the locale specific month names will be used.
 * @return the current months array used for month names
 */
public String[] getShortMonths()
{
	return(shortMonths);
}

/**
 * Sets the months array used for short month names.  If the value is null,
 * the locale specific short month names will be used.
 * @param shortMonths the new short months array used
 */
public void setShortMonths(String[] shortMonths)
{
	if (checkMonths(shortMonths, true)) {
	    this.shortMonths = shortMonths;
	    createComponents(chooserType);
	}
}

/**
 * Gets the days array used for day names.  If the return value is null,
 * the locale specific day names will be used.
 * @return the current days array used for day names
 */
public String[] getDays()
{
	return(days);
}

/**
 * Sets the days array used for day names.  If the value is null,
 * the locale specific day names will be used.
 * @param days the new days array used for day names
 */
public void setDays(String[] days)
{
	String[] localeDays = JCCalendar.getDays(getLocale());
	if (days != null && days.length < localeDays.length) {
		throw new IllegalArgumentException("The days array must be at least " +
		                                   localeDays.length + " in size.");
	}
	this.days = days;
	createComponents(chooserType);
}

/**
 * Sets the UI.
 * @param ui the new UI
 */
public void setUI(ComponentUI ui)
{
	super.setUI(ui);
}

/**
 * Sets the current date being used by the calendar component.
 * @param model the new calendar value model
 */
public void setCalendarModel(JCValueModel model)
{
	JCValueModel old_model = this.model;
	super.setCalendarModel(model);
	if (old_model != null) {
		old_model.removeValueListener(boundsValidator);
	}
	if (model != null) {
		model.addValueListener(boundsValidator);
	}
}


/**
 * Returns the <code>CalendarComponent</code> used in the day portion of the
 * chooser.
 * @return the component which displays the day
 */
public CalendarComponent getDayComponent()
{
	return(day);
}

/**
 * Returns the <code>CalendarComponent</code> used in the Month portion of the
 * chooser.
 * @return the component which displays the month
 */
public CalendarComponent getMonthComponent()
{
	return(month);
}

/**
 * Returns the <code>CalendarComponent</code> used in the year portion of the
 * chooser.
 * @return the component which displays the year
 */
public CalendarComponent getYearComponent()
{
	return(year);
}

/**
 * Creates the date chooser components based on the chooser type.  Creates a
 * component for selecting the year, the month, and the day.
 * @param type the type of chooser; must be one of
 *     <code>JCDateChooser.QUICK_SELECT</code>,
 *     <code>JCDateChooser.DUAL_SPIN</code>,
 *     <code>JCDateChooser.SPIN_POPDOWN</code>, or
 *     <code>JCDateChooser.READ_ONLY</code>
 */
protected void createComponents(int type)
{
	JComponent old_year = (JComponent) year;
	JComponent old_month = (JComponent) month;
	JComponent old_day = (JComponent) day;

	switch (type) {
	case SPIN_POPDOWN:
		year = new YearSpin(model, locale);
		month = new MonthPopdown(model, locale, months);
		day = new DayTable(model, specialDates, locale, days);
		break;
	case DUAL_SPIN:
		year = new YearSpin(model, locale);
		month = new MonthSpin(model, locale, months);
		day = new DayTable(model, specialDates, locale, days);
		break;
	case QUICK_SELECT:
		year = new YearSpin(model, locale);
		month = new MonthTable(model, locale, shortMonths);
		day = new DayTable(model, specialDates, locale, days);
		break;
	case READ_ONLY:
		year = new YearLabel(model, locale);
		month = new MonthLabel(model, locale, months);
		day = new DayTable(model, specialDates, locale, days);
		((DayTable)day).setEditable(false);
		break;
	default:
		throw new IllegalArgumentException("Invalid value for ChooserType");
	}

    if (userFont != null) {
        ((Component)year).setFont(userFont);
        ((Component)month).setFont(userFont);
        ((Component)day).setFont(userFont);
    }

	if (getComponentCount() != 0) {
		// Remove any components that are currently set to us
		removeAll();
	}

	add((Component) year);
	add((Component) month);
	add((Component) day);

	if (old_year != null) {
		remove(old_year);
	}
	if (old_month != null) {
		remove(old_month);
	}
	if (old_day != null) {
		remove(old_day);
	}
}

/**
 * Inner class to lay out components differently depending on type.
 */
private class Layout implements LayoutManager2, java.io.Serializable {

	//
	// LayoutManager Implementation
	//

	/**
	 * Adds a component to the layout.
     * @param name the String to be associated with the component
     * @param comp the component to be added
	 */
	public void addLayoutComponent(String name, Component comp) {}

	/**
	 * Removes a component from the layout.
	 * @param comp the component to be removed
	 */
	public void removeLayoutComponent(Component comp) {}

	/**
	 * Returns the preferred layout size of the containers within their parent.
	 * @param parent the container to be laid out
	 * @return the preferred size of the layout
	 */
	public Dimension preferredLayoutSize(Container parent)
	{
		Component y = (Component) year;
		Component m = (Component) month;
		Component d = (Component) day;
		Dimension ys = y.getPreferredSize();
		Dimension ms = m.getPreferredSize();
		Dimension ds = d.getPreferredSize();
		if (chooserType == QUICK_SELECT) {
			return(new Dimension(Math.max(ds.width, Math.max(ys.width,
															 ms.width)),
								 ms.height + ys.height + ds.height +
								 (3 * componentSpacing)));
		}
		else {
			return(new Dimension(Math.max(ds.width,
									  ys.width + ms.width + componentSpacing),
								 Math.max(ms.height, ys.height) +
								 (2 * componentSpacing) + ds.height));
		}
	}

	/**
	 * Returns the minimum layout size of the containers within their parent.
	 * @param parent The container to be laid out
	 * @return The minimum size of the layout
	 */
	public Dimension minimumLayoutSize(Container parent)
	{
		Component y = (Component) year;
		Component m = (Component) month;
		Component d = (Component) day;
		Dimension ys = y.getMinimumSize();
		Dimension ms = m.getMinimumSize();
		Dimension ds = d.getMinimumSize();
		if (chooserType == QUICK_SELECT) {
			return(new Dimension(Math.max(ds.width, Math.max(ys.width,
													         ms.width)),
								 ms.height + ys.height + ds.height +
								 componentSpacing));
		}
		else {
			return(new Dimension(Math.max(ds.width, ys.width + ms.width),
								 Math.max(ms.height, ys.height) + ds.height +
								 componentSpacing));

		}
	}

	/**
	 * Lays out the components within the specified parent.
     * @param parent the container to be laid out
	 */
	public void layoutContainer(Container parent)
	{
		Component yr = (Component) year;
		Component m = (Component) month;
		Component d = (Component) day;
		Dimension yps = yr.getPreferredSize();
		Dimension mps = m.getPreferredSize();
		Dimension dps = d.getPreferredSize();
		Dimension ps = parent.getSize();

		int x, y, w, h;

		if (chooserType == QUICK_SELECT) {
			//
			// Align components vertically in a centered column
			//

			// YEAR

			y = componentSpacing;   // leave a gap at the top
			h = yps.height;
			if (ps.width > yps.width) {
				// actual width is greater preferred
				x = (ps.width - yps.width) / 2;
				w = yps.width;
			}
			else {
				x = 0;
				w = ps.width;
			}
			/* System.out.println("setting year bounds to " + x + ", " + y +
							   ", " + w + ", " + h); */
			yr.setBounds(x, y, w, h);

			// MONTH

			y += h; // add height of previous component
			y += componentSpacing; // add spacing
			h = mps.height;
			x = 0;
			w = ps.width;
			/* System.out.println("setting month bounds to " + x + ", " + y +
							   ", " + w + ", " + h); */
			m.setBounds(x, y, w, h);

            // Fix problem in JDK 1.2.2 where monthTable columns do not resize
            // correctly on a locale or font change.
            if (JCEnvironment.getJavaVersion() < 130) {
                MonthTable monthTable = ((MonthTable)m);
                Enumeration e = monthTable.getColumnModel().getColumns();
                while (e.hasMoreElements()) {
                    TableColumn column = (TableColumn)e.nextElement();
                    Insets insets = monthTable.getInsets();
                    int newWidth = (w + insets.left + insets.right + 5) / 6;
                    column.setMinWidth(newWidth);
                }
            }
		}
		else {
			// Put year and month in a horizontal row and place the
			// day beneath it

			// YEAR and MONTH
			y = componentSpacing;   // leave a gap at the top
			h = Math.max(yps.height, mps.height);
			if (ps.width > (yps.width + mps.width)) {
				// actual width is greater than preferred
				x = (ps.width - (yps.width + mps.width)) / 2;
				w = -1;
			}
			else {
				x = 0;
				w = ps.width / 2;
			}
			int wth;
			if (w == -1) {
				// use preferred width
				wth = yps.width;
			}
			else {
				wth = w;
			}
			yr.setBounds(x, y, wth, h);

			x += wth;
			x += componentSpacing; // add spacing
			if (w == -1) {
				// Use preferred width
				wth = mps.width;
			}
			else {
				// Compensate for odd width division errors
				wth = ps.width - w;
			}
			m.setBounds(x, y, wth, h);
		}
		// DAY

		y += h; // add height of previous component
		y += componentSpacing; // add spacing
		h = dps.height;
		x = 0;
		w = ps.width;
		/* System.out.println("setting day bounds to " + x + ", " + y +
		                      ", " + w + ", " + h); */
		d.setBounds(x, y, w, h);
	}

	//
	// LayoutManager2 Implementation
	//

	/**
	 * Adds a component to the layout manager with the specified constraint.
     * @param comp the component to be added
     * @param constraints where/how the component is added to the layout
	 */
	public void addLayoutComponent(Component comp, Object constraints) {}

	/**
	 * Returns the maximum layout size of the component.
	 * @param parent the container to be laid out
	 * @return the maximum size of the layout
	 */
	public Dimension maximumLayoutSize(Container parent)
	{
		return(preferredLayoutSize(parent));
	}

	/**
     * Returns the alignment along the <i>X</i> axis.  This specifies how
     * the component would like to be aligned relative to other
     * components.  The value should be a number between 0 and 1,
     * where 0 represents alignment along the origin, 1 is aligned
     * the furthest away from the origin, 0.5 is centered, etc.
	 * @param target the container whose <i>X</i> alignment is required
	 * @return the <i>X</i> layout alignment
	 */
	public float getLayoutAlignmentX(Container target)
	{
		return((float) 0.0);
	}

	/**
     * Returns the alignment along the <i>Y</i> axis.  This specifies how
     * the component would like to be aligned relative to other
     * components.  The value should be a number between 0 and 1,
     * where 0 represents alignment along the origin, 1 is aligned
     * the furthest away from the origin, 0.5 is centered, etc.
	 * @param target the container whose <i>Y</i> alignment is required
	 * @return the <i>Y</i> layout alignment
	 */
	public float getLayoutAlignmentY(Container target)
	{
		return((float) 0.0);
	}

	/**
	 * Invalidates the current layout.  Removes any information
	 * that might be cached regarding the current layout.
	 * @param target the container whose layout is to be invalidated
	 */
	public void invalidateLayout(Container target)
	{
		// TODO - add layout caching
	}

} // End of Layout inner class

/**
* Sets whether or not this component is enabled.
* A component that is enabled may respond to user input,
* while a component that is not enabled cannot respond to
* user input.  Some components may alter their visual
* representation when they are disabled in order to
* provide feedback to the user that they cannot take input.
* @param enabled true if this component should be enabled, false otherwise
* @see Component#isEnabled
* @see Component#isLightweight
*/
public void setEnabled(boolean enabled)
{
   super.setEnabled(enabled);
   ((JComponent)getYearComponent()).setEnabled(enabled);
   ((DayTable)getDayComponent()).setEditable(enabled);
   ((JComponent)getMonthComponent()).setEnabled(enabled);
}


}
