/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: MonthLabel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:39 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.calendar;

import java.util.Calendar;
import java.util.Locale;
import com.klg.jclass.util.value.JCValueModel;
import com.klg.jclass.util.value.JCValueEvent;

/**
 * Label shows the months of the year.
 */
public class MonthLabel extends AbstractLabel {

protected boolean   userSetMonths = false;
protected String[]  months = null;

/**
 * Construct a MonthLabel for the given value model and locale.
 * @param model The calendar value model
 * @param l The locale object to use
 */
public MonthLabel(JCValueModel model, Locale l)
{
	this(model, l, null);
}

/**
 * Construct a MonthLabel for the given value model and locale.
 * @param model The calendar value model
 * @param l The locale object to use
 * @param months A list of months to use (null means use locale specific names)
 */
public MonthLabel(JCValueModel model, Locale l, String[] months)
{
	this.months = months;
	if (this.months != null) {
		userSetMonths = true;
	}
	setLocale(l);
	setCalendarModel(model);
}

/**
 * Set the locale the MonthLabel should use.
 * @param locale The locale object to use.
 */
public void setLocale(Locale locale)
{
	super.setLocale(locale);

	if (!userSetMonths) {
	    // Reset the names of the months, based on the new locale.
	    months = JCCalendar.getMonths(locale, false);
	}
}

/**
 * Set the current date being used by the calendar component
 * @param model The new calendar value model
 */
public void setCalendarModel(JCValueModel model)
{
	super.setCalendarModel(model);
	Calendar c = (Calendar) model.getValue();
	int month = c.get(c.MONTH);
	setText(months[month]);
}

/**
 * The method informs the listener that the value has been changed.
 * @param event The event generated by the value change
 */
public void valueChanged(JCValueEvent event)
{
	Calendar c = (Calendar) model.getValue();
	int month = c.get(c.MONTH);
	setText(months[month]);
}

}
