/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: MonthSpin.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:40 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.calendar;

import com.klg.jclass.util.JCListenerList;
import com.klg.jclass.util.swing.DefaultSpinBoxModel;
import com.klg.jclass.util.swing.JCSpinBox;
import com.klg.jclass.util.value.JCValueEvent;
import com.klg.jclass.util.value.JCValueListener;
import com.klg.jclass.util.value.JCValueModel;

import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Calendar;
import java.util.Locale;

/**
 * SpinBox that shows months of the year.
 */
public class MonthSpin extends JCSpinBox
                       implements CalendarComponent, JCValueListener,
					              ItemListener {

protected Locale          locale = null;
protected JCValueModel    calendarModel;
protected int             selectedMonth = 0;
protected String[]        months;
protected boolean         userSetMonths = false;
protected JCListenerList  actionListeners = null;
protected boolean         ignoreChange = false;

/**
 * Constructs a MonthSpin with the given value model and locale.
 * @param model The new calendar value model
 * @param l The locale to use
 */
public MonthSpin(JCValueModel model, Locale l)
{
	this(model, l, null);
}

/**
 * Constructs a MonthSpin with the given value model and locale.
 * @param model The new calendar value model
 * @param l The locale to use
 * @param months A list of months to use (null means use locale specific names)
 */
public MonthSpin(JCValueModel model, Locale l, String[] months)
{
	this.months = months;
	if (this.months != null) {
		userSetMonths = true;
	}
	setLocale(l);
	setCalendarModel(model);
	addItemListener(this);
}

/**
 * Set the locale to use.
 * @param l The new locale to use
 */
public void setLocale(Locale l)
{
	if (l == null) {
		l = Locale.getDefault();
	}

	if (locale == l) {
		return;
	}
    super.setLocale(l);
	locale = l;

	boolean savedIndex = false;
	int     oldIndex   = 0;

	// If there was already a selected month (e.g. from default date),
	// remember it.
	if (getModel() != null) {
        oldIndex = getSelectedIndex();
        savedIndex = true;
	}

	if (!userSetMonths) {
	    // Reset the names of the months, based on the new locale.
	    months = JCCalendar.getMonths(locale, false);
	}
	setModel(new DefaultSpinBoxModel(months));

	if (savedIndex) {
		// Restore the previous selection. Avoids out-of-sync between index and
		// display text. Set index to different value first or the change won't
        // take effect.
        setSelectedIndex((oldIndex + 1) % months.length);
		setSelectedIndex(oldIndex);
	}

	repaint();
}

/**
 * Get the locale that is being used.
 * @return The current locale being used.
 */
public Locale getLocale()
{
	return(locale);
}

/**
 * Set the value model.
 * @param model The new calendar value model
 */
public void setCalendarModel(JCValueModel model)
{
	if (calendarModel != null) {
		calendarModel.removeValueListener(this);
	}

	this.calendarModel = model;
	valueChanged(null);
	calendarModel.addValueListener(this);
	repaint();
}

/**
 * Get the value model.
 * @return The current calendar value model
 */
public JCValueModel getCalendarModel()
{
	return(calendarModel);
}

/**
 * This component ignores special dates
 * @param sd The new list of special dates stored within a JCCalendar object
 */
public void setSpecialDates(JCCalendar sd) {}

/**
 * Get the special dates being used by the DayTable.
 * @param sd Unused
 * @deprecated As the parameter is unused, should use getSpecialDates()
 * @return The current list of special dates stored within a JCCalendar object
 */
public JCCalendar getSpecialDates(JCCalendar sd)
{
	return(getSpecialDates());
}

/**
 * Get the special dates being used by the DayTable.
 * @return The current list of special dates stored within a JCCalendar object
 */
public JCCalendar getSpecialDates()
{
	return(null);
}

/**
 * Add listener to receive action event when data is changed by the user
 * @param l The action listener to be added
 */
public void addActionListener(ActionListener l)
{
	actionListeners = JCListenerList.add(actionListeners, l);
}

/**
 * Remove listener to receive action event when data is changed by the user
 * @param l The action listener to be removed
 */
public void removeActionListener(ActionListener l)
{
	actionListeners = JCListenerList.remove(actionListeners, l);
}

//
// ItemListener implementation
//

/**
 * State Changed.
 * @param e The event caused by the item being changed
 */
public void itemStateChanged(ItemEvent e)
{
	if (ignoreChange) {
		return;
	}

	ignoreChange = true;
	DefaultSpinBoxModel spinModel = (DefaultSpinBoxModel)getModel();
	selectedMonth = spinModel.getIndexOf(getModel().getSelectedItem());
	Calendar c = JCCalendar.copyCalendar((Calendar) calendarModel.getValue());
	c.set(c.MONTH, selectedMonth);
	calendarModel.setValue(c);

    // Check to see if calendar value was rejected.  Reset to old value.
    int index = ((Calendar)calendarModel.getValue()).get(Calendar.MONTH);
    if (index != selectedMonth) {
        setSelectedIndex(index);
        selectedMonth = index;
    }
	ignoreChange = false;
}

//
// JCValueModel implementation
//

/**
 * The value is changing.  The listener may further change the new_value or
 * abort the change altogether.  If the case of an abort (call
 * event.setAllowChange(false)) the "valueChanged"  method will never be
 * called.
 * @param event The event generated by the value changing
 */
public void valueChanging(JCValueEvent event) {}

/**
 * The method informs the listener that the value has been changed.
 * @param event The event generated by the value being changed
 */
public void valueChanged(JCValueEvent event)
{
	if (ignoreChange) {
		return;
	}

	ignoreChange = true;
	setSelectedIndex(((Calendar) calendarModel.getValue()).get(Calendar.MONTH));
	ignoreChange = false;
}

}
