/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: MonthTable.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:40 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.calendar;

import com.klg.jclass.util.JCListenerList;
import com.klg.jclass.util.JCLocaleManager;
import com.klg.jclass.util.JCEnvironment;
import com.klg.jclass.util.value.JCValueEvent;
import com.klg.jclass.util.value.JCValueListener;
import com.klg.jclass.util.value.JCValueModel;
import com.klg.jclass.util.value.CalendarValueModel;

import javax.swing.*;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.border.SoftBevelBorder;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableColumn;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.Locale;

/**
 * MonthTable is used to display a Table of selectable months for use with a
 * calendar.
 */
public class MonthTable extends JTable
			            implements CalendarComponent, MouseListener
{

transient JCLocaleManager  li = JCLocaleManager.getDefault();

protected Locale           locale = null;
protected int              tableCellWidth = 0;
protected JCValueModel     calendarModel;
protected MonthModel       monthModel;
protected int              selectedMonth = 0;
protected String[]         months = null;
protected boolean          userSetMonths = false;
protected JCListenerList   actionListeners = null;

/**
 * Construct a MonthTable with the given value model and locale.
 * @param model The new calendar value model
 * @param l The locale to use
 */
public MonthTable(JCValueModel model, Locale l)
{
	this(model, l, null);
}

/**
 * Construct a MonthTable with the given value model, locale, and month names.
 * @param model The new calendar value model
 * @param l The locale to use
 * @param months A list of months to use (null means use locale specific names)
 */
public MonthTable(JCValueModel model, Locale l, String[] months)
{
	this.months = months;
	if (this.months != null) {
		userSetMonths = true;
	}
	setIntercellSpacing(new Dimension(0, 0));
	setDefaultRenderer(String.class, new Renderer());
	setLocale(l);
	setCalendarModel(model);
	addMouseListener(this);
}

/**
 * Set the locale to use.
 * @param l The new locale to use.
 */
public void setLocale(Locale l)
{
	if (l == null) {
		l = Locale.getDefault();
	}

	if (locale == l) {
		return;
	}
	firePropertyChange("locale", locale, l);
	locale = l;

	if (monthModel != null && calendarModel != null) {
		calendarModel.removeValueListener(monthModel);
	}

	monthModel = new MonthModel(locale);
	if (calendarModel != null) {
		calendarModel.addValueListener(monthModel);
	}
	// Post a complete table changed event
	setModel(monthModel);
    adjustSize();
	repaint();
}

/**
 * Get the locale being used.
 * @return The current locale being used.
 */
public Locale getLocale()
{
	return(locale);
}

/**
 * Override setFont to adjust the size of the table.
 * @param font The new font.
 */
public void setFont(Font font)
{
    super.setFont(font);
    adjustSize();
}

/**
 * Set the value model to use.
 * @param model The new calendar value model
 */
public void setCalendarModel(JCValueModel model)
{
    if (model == null || model.getValueClass() != Calendar.class) {
        throw new IllegalArgumentException("Model's value class must be of " +
                                           "type Calendar.");
    }

	if (monthModel != null && calendarModel != null) {
		calendarModel.removeValueListener(monthModel);
	}
	this.calendarModel = model;
	calendarModel.addValueListener(monthModel);
	repaint();
}

/**
 * Get the value model being used.
 * @return The current calendar value model.
 */
public JCValueModel getCalendarModel()
{
	return(calendarModel);
}

/**
 * This component ignores special dates.
 * @param sd The new list of special dates stored within a JCCalendar object
 */
public void setSpecialDates(JCCalendar sd) {}

/**
 * Get the special dates being used by the DayTable.
 * @param sd Unused
 * @deprecated As the parameter is unused, should use getSpecialDates()
 * @return The current list of special dates stored within a JCCalendar object
 */
public JCCalendar getSpecialDates(JCCalendar sd)
{
	return(getSpecialDates());
}

/**
 * Get the special dates being used by the DayTable.
 * @return The current list of special dates stored within a JCCalendar object
 */
public JCCalendar getSpecialDates()
{
	return(null);
}

/**
 * Add listener to receive action event when data is changed by the user.
 * @param l The action listener to be added.
 */
public void addActionListener(ActionListener l)
{
	actionListeners = JCListenerList.add(actionListeners, l);
}

/**
 * remove listener to receive action event when data is changed by the user.
 * @param l The action listener to be removed.
 */
public void removeActionListener(ActionListener l)
{
	actionListeners = JCListenerList.remove(actionListeners, l);
}

/**
 * Calls configureEnclosingScrollPane aswell as setting up the table row height
 * based on the font metrics.
 */
public void addNotify()
{
	super.addNotify();
    adjustSize();
}

/**
 * Adjust the size of the table.
 */
protected void adjustSize()
{
    // Calculate cell size and sets the row and column sizes
    calculateCellSize();
    setPreferredSize(new Dimension(tableCellWidth * 6,
                                   getRowHeight() * 2));
}

/**
 * Calculates the cell size. The cell widths and heights are caculated
 * separately.  The table height will be six times the cell width and two times
 * the cell height.
 */
protected void calculateCellSize()
{
    TableModel model = getModel();
    TableColumnModel columnModel = getColumnModel();
    if (model == null || columnModel == null || calendarModel == null) {
        return;
    }
    tableCellWidth = 0;
    int cellHeight = 0;
    for (int col = 0; col < columnModel.getColumnCount(); col++) {
        TableColumn column = columnModel.getColumn(col);
        TableCellRenderer renderer = column.getCellRenderer();
        // If no renderer set then we use default one
        if (renderer == null) {
            Class columnClass = model.getColumnClass(col);
            renderer = getDefaultRenderer(columnClass);
        }
        for (int row = 0; row < getRowCount(); row++) {
            Object value = model.getValueAt(row, col);
            Component c = renderer.getTableCellRendererComponent(this, value,
                                                                 false, false,
                                                                 row, col);
            Dimension cellDim = c.getPreferredSize();
            tableCellWidth = Math.max(tableCellWidth, cellDim.width);
            cellHeight = Math.max(cellHeight, cellDim.height);
        }
    }

    // Set height of rows to the newly calculated cell size
    setRowHeight(cellHeight);

    // SetColumnWidths to cellSize
    Enumeration e = columnModel.getColumns();
    while (e.hasMoreElements()) {
        TableColumn column = (TableColumn)e.nextElement();
        column.setMinWidth(tableCellWidth);
        if (JCEnvironment.getJavaVersion() < 130) {
            column.setWidth(tableCellWidth);
        }
    }
}

/**
 * Get the preferred size.
 * @return The preferred size calculated from the the preferred sizes of each
 * of the cells.
 * @see #calculateCellSize
 */
public Dimension getPreferredSize()
{
	return(new Dimension(tableCellWidth * 6, getRowHeight() * 2));
}

//
// MouseListener implementation
//

/**
 * Mouse pressed.
 * @param e The event generated
 */
public void mousePressed(MouseEvent e)
{
	int row = rowAtPoint(e.getPoint());
	int column = columnAtPoint(e.getPoint());

	int month = (row * 6) + column;
	Calendar c = JCCalendar.copyCalendar((Calendar) calendarModel.getValue());
	c.set(c.MONTH, month);
	calendarModel.setValue(c);
}

/**
 * Mouse clicked.
 * @param e The event generated
 */
public void mouseClicked(MouseEvent e) {}

/**
 * Mouse released.
 * @param e The event generated
 */
public void mouseReleased(MouseEvent e) {}

/**
 * Mouse Entered.
 * @param e The event generated
 */
public void mouseEntered(MouseEvent e) {}

/**
 * Mouse Exited.
 * @param e The event generated
 */
public void mouseExited(MouseEvent e) {}

/**
 * Use a table cell renderer to draw current month in reverse video.
 */
protected class Renderer extends JLabel implements TableCellRenderer
{
	protected Border bevel = new SoftBevelBorder(BevelBorder.LOWERED);
	protected Border empty = new EmptyBorder(0, 0, 0, 0);

	/**
	 * Get the renderer component for the cell at (row, column).  This method
	 * intializes sets properties on this object in preparation for rendering
	 * and then passes it back as the renderer.
	 * @param table The JCTable on which this renderer is being used
	 * @param value The value to render
	 * @param isSelected Is the cell selected?
	 * @param hasFocus Does the cell have focus?
	 * @param row The row index of the cell
	 * @param column The column index of the cell
	 * @return The renderer which happens to be this object
	 */
	public Component getTableCellRendererComponent(JTable table,
												   Object value,
												   boolean isSelected,
												   boolean hasFocus,
												   int row,
												   int column)
	{
		setHorizontalAlignment(SwingConstants.CENTER);
        setFont(table.getFont());

		// Check if the cell we are drawing is the selected month
		Calendar c = (Calendar) calendarModel.getValue();
		int m = c.get(c.MONTH);
		if (m == (column + (row * getColumnCount()))) {
			// ??? should really get these values from the parent component
			setForeground(UIManager.getColor("TextField.selectionForeground"));
			setBackground(UIManager.getColor("TextField.selectionBackground"));
			setBorder(bevel);
			setOpaque(true);
		}
		else {
			// ??? should really get these values from the parent component
			setBackground(UIManager.getColor("Label.background"));
			setForeground(UIManager.getColor("Label.foreground"));
			setBorder(empty);
			setOpaque(true);
		}
		setText((String) value);
		return(this);
	}
}

/**
 * Build data model wrapper around DateSymbols that
 * builds a 2x6 table of the months for picking.
 */
protected class MonthModel implements TableModel, JCValueListener,
						              java.io.Serializable
{
	protected JCListenerList  tableListeners = null;
	protected int             numRows = 2;

	/**
     * Construct a MonthModel with a certain locale.
	 * @param l The locale from which the short month names is garnered
	 */
	public MonthModel(Locale l)
	{
		if (!userSetMonths) {
		    months = JCCalendar.getMonths(l, true);
		}
	}

	//
	// TableModel Implementation
	//

    /**
     * Get the number of rows.
	 * @return The current number of rows
     */
	public int getRowCount()
	{
		return(numRows);
	}

    /**
     * Get the number of columns.
	 * @return The current number of columns
     */
	public int getColumnCount()
	{
		return(months.length / numRows);
	}

    /**
     * Get the name of a particular column.
	 * @param columnIndex The index of column whose name is required
	 * @return The column name of the given column
     */
	public String getColumnName(int columnIndex)
	{
		return(null);
	}

    /**
     * Get the class of a particular column.
	 * @param columnIndex The index of column whose class type is required
	 * @return The class type of the given column
     */
	public Class getColumnClass(int columnIndex)
	{
		return(String.class);
	}

    /**
     * Test to see if the cell is editable.
	 * @param rowIndex The row index of the cell
	 * @param columnIndex The column index of the cell
	 * @return false as all cells are not editable
     */
	public boolean isCellEditable(int rowIndex, int columnIndex)
	{
		return(false);
	}

    /**
     * Return the value at the given row and column.
	 * @param rowIndex The row index of the cell
	 * @param columnIndex The column index of the cell
	 * @return The value for the given row and column indices
     */
	public Object getValueAt(int rowIndex, int columnIndex)
	{
		return(months[columnIndex + (rowIndex * getColumnCount())]);
	}

    /**
     * Set the value at the given row and column.  Since cells are readonly
	 * this method always throws an exception.
	 * @param aValue The value to be set
	 * @param rowIndex The row index of the cell
	 * @param columnIndex The column index of the cell
	 * @throws IllegalArgumentException
     */
	public void setValueAt(Object aValue, int rowIndex, int columnIndex)
	{
		throw new IllegalArgumentException("This TableModel is READ ONLY");
	}

    /**
     * Add a table listener to the model.
	 * @param l The table model listener to be added
     */
	public void addTableModelListener(TableModelListener l)
	{
		tableListeners = JCListenerList.add(tableListeners, l);
	}

    /**
     * Remove a table listener from the model.
	 * @param l The table model listener to be removed
     */
	public void removeTableModelListener(TableModelListener l)
	{
		tableListeners = JCListenerList.remove(tableListeners, l);
	}

	//
	// JCValueModel implementation
	//

	/**
	 * The value is changing.  The listener may further change the new_value or
	 * abort the change altogether.  If the case of an abort (call
	 * event.setAllowChange(false)) the "valueChanged"  method will never be
	 * called.
     * @param event The event generated by the value changing
	 */
	public void valueChanging(JCValueEvent event) {}

	/**
	 * The method informs the listener that the value has been changed.
     * @param event The event generated by the value being changed
	 */
	public void valueChanged(JCValueEvent event)
	{
		if (event == null) {
			return;
		}

		TableModelEvent tableEvent = new TableModelEvent(this);
		Enumeration e = JCListenerList.elements(tableListeners);
		while (e.hasMoreElements()) {
			TableModelListener listener = (TableModelListener)e.nextElement();
			listener.tableChanged(tableEvent);
		}
	}
}

}
