/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TimeLabel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:40 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.calendar;

import com.klg.jclass.util.value.JCValueEvent;
import com.klg.jclass.util.value.JCValueModel;

import java.text.DateFormat;
import java.util.Calendar;
import java.util.Locale;

/**
 * Label that shows the time.
 */
public class TimeLabel extends AbstractLabel {

protected DateFormat dateFormat;

/**
 * Construct a TimeLabel for the given value model and locale.
 * @param model The calendar value model
 * @param l The locale object to use
 */
public TimeLabel(JCValueModel model, Locale l)
{
    setLocale(l);
    setCalendarModel(model);
}

/**
 * Construct a TimeLabel for the given value model and date formatter.
 * @param model The calendar value model
 * @param dateFormat The date formatter to use
 */
public TimeLabel(JCValueModel model, DateFormat dateFormat)
{
    setDateFormat(dateFormat);
    setCalendarModel(model);
}

/**
 * Get the date formatter to use for formatting the date.
 * @return The current date formatter
 */
public DateFormat getDateFormat()
{
    return(dateFormat);
}

/**
 * Set the date formatter to use for formatting the date.
 * @param dateFormat The new date formatter
 */
public void setDateFormat(DateFormat dateFormat)
{
    this.dateFormat = dateFormat;
    updateLabel();
}

/**
 * Set the locale the TimeLabel object should use.  Also update the formatter to
 * be based on the new locale.
 * @param l The locale object to use.
 */
public void setLocale(Locale l)
{
    super.setLocale(l);

    // Rest the format to the medium time format for the new locale
    setDateFormat(DateFormat.getTimeInstance(DateFormat.MEDIUM, getLocale()));
}

/**
 * Set the current date being used by the calendar component
 * @param model The new calendar value model
 */
public void setCalendarModel(JCValueModel model)
{
	super.setCalendarModel(model);
	updateLabel();
}

/**
 * The method informs the listener that the value has been changed.
 * @param event The event generated by the value change
 */
public void valueChanged(JCValueEvent event)
{
    updateLabel();
}

/**
 * Create label text based on the current model value and the date formatter
 * and set the text on the label.
 */
protected void updateLabel()
{
    if (model != null && dateFormat != null) {
        Calendar c = (Calendar)model.getValue();
        if (c != null) {
            setText(dateFormat.format(c.getTime()));
        }
    }
}

}
