/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: YearSpin.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:40 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.calendar;

import com.klg.jclass.util.JCListenerList;
import com.klg.jclass.util.swing.JCSpinNumberBox;
import com.klg.jclass.util.value.JCValueEvent;
import com.klg.jclass.util.value.JCValueListener;
import com.klg.jclass.util.value.JCValueModel;

import java.awt.event.ActionListener;
import java.awt.*;
import java.text.DecimalFormat;
import java.util.Calendar;
import java.util.Locale;

/**
 * Container to manage CalendarComponent children.  That is, any calls to
 * the calendar component interface methods are automatically passed to any
 * calendar component children
 */
public class YearSpin extends JCSpinNumberBox implements CalendarComponent {

protected JCValueModel      calendarModel;
protected JCCalendar        specialDates;
protected Locale            locale = null;
protected boolean           ignoreChange = false;
protected JCValueListener   l = new ModelConduit();
protected JCListenerList    actionListeners = null;

/**
 * Constructs a YearSpin with the given value model and locale.
 * @param model The new calendar value model
 * @param locale The locale to use
 */
public YearSpin(JCValueModel model, Locale locale)
{
	setLocale(locale);
	setValueRange(new Range(new Integer(0), new Integer(9999)));
	setCalendarModel(model);
	setNumberFormat(new DecimalFormat("#"));
}

/**
 * Set the current date being used by the calendar component
 * @param model The new calendar value model
 */
public void setCalendarModel(JCValueModel model)
{
	Calendar c = (Calendar) model.getValue();
	if (c != null) {
		int year = c.get(c.YEAR);
		ignoreChange = true;
		setValue(new Integer(year));
		ignoreChange = false;
	}
	else {
		ignoreChange = true;
		setValue(null);
		ignoreChange = false;
	}

	if (calendarModel != null) {
		calendarModel.removeValueListener(l);
	}
    if (valueModel != null) {
        valueModel.removeValueListener(l);
    }
    calendarModel = model;
	calendarModel.addValueListener(l);
	valueModel.addValueListener(l);
}

/**
 * Get the value model.
 * @return The current calendar value model
 */
public JCValueModel getCalendarModel()
{
	return(calendarModel);
}

/**
 * Set the special dates being used by the calendar component
 * @param special_dates The new list of special dates stored within a
 *     JCCalendar object
 */
public void setSpecialDates(JCCalendar special_dates)
{
	this.specialDates = special_dates;
}

/**
 * Set the locale being used by the calendar component
 * @param l The new locale to be used
 */
public void setLocale(Locale l)
{
	if (l == null) {
		l = Locale.getDefault();
	}

	if (locale == l) {
		return;
	}
    super.setLocale(l);
	locale = l;
}

/**
 * Get the locale that is being used.
 * @return The current locale being used.
 */
public Locale getLocale()
{
	return(locale);
}

/**
 * Add listener to receive action event when data is changed by the user
 * @param l The action listener to be added
 */
public void addActionListener(ActionListener l)
{
	actionListeners = JCListenerList.add(actionListeners, l);
}

/**
 * Remove listener to receive action event when data is changed by the user
 * @param l The action listener to be removed
 */
public void removeActionListener(ActionListener l)
{
	actionListeners = JCListenerList.remove(actionListeners, l);
}

/**
 * Map changes from number value model to calendar value model and vice versa
 */
class ModelConduit implements JCValueListener, java.io.Serializable {

	public ModelConduit() {}

    private Calendar getUpdatedCalendar(int year)
    {
        Calendar cal = (Calendar)calendarModel.getValue();
        Calendar c = JCCalendar.copyCalendar(cal);
        c.set(c.YEAR, year);
        return(c);
    }

	/**
	 * The value is changing.  The listener may further change the new_value or
	 * abort the change altogether.  If the case of an abort (call
	 * event.setAllowChange(false)) the "valueChanged"  method will never be
	 * called.
     * @param e The event generated by the value changing
	 */
	public void valueChanging(JCValueEvent e)
    {
        if (ignoreChange || e.getSource() == calendarModel) {
            return;
        }
        Container container = getParent();
        if (container instanceof JCDateChooser) {
            JCDateChooser dc = (JCDateChooser)container;
            int year = ((Number)e.getNewValue()).intValue();
            Calendar c = getUpdatedCalendar(year);
            if (!dc.boundsValidator.isValid(c)) {
                e.setAllowChange(false);
                Toolkit t = getToolkit();
                if (t != null) {
                    t.beep();
                }
            }
        }
    }

    /**
	 * The method informs the listener that the value has been changed.
     * @param e The event generated by the value being changed
	 */
	public void valueChanged(JCValueEvent e)
	{
		if (!ignoreChange && e.getSource() == calendarModel) {
			Calendar c = (Calendar)e.getNewValue();
			int year = c.get(c.YEAR);
			if (getValue().intValue() != year) {
				ignoreChange = true;
				setValue(new Integer(year));
				ignoreChange = false;
			}
		}
		else if (!ignoreChange && e.getSource() == getValueModel()) {
            Calendar c = getUpdatedCalendar(getValue().intValue());
			ignoreChange = true;
			calendarModel.setValue(c);
			ignoreChange = false;
		}
	}
}

}
