/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: Abs.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:40 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.formulae;


import java.util.Iterator;




/**
 * This is a concrete implementation for the absolute value operation.
 * Absolute value is the positive magnitude of a number
 * This is a unary operator, *???but can also work on lists???*.
 */
public class Abs extends Operation {

/**
 * Create an operation to compute the absolute value of the given Expression
 */
public Abs(Expression operand) {
	super(operand);
}

/**
 * Create an operation to compute the absolute value of the given Number
 */
public Abs(Number operand) {
	super(operand);
}

// NOTE:  We can extend this to magnitude of a vector, but we'd need
//        to introduce the concept of a measure on the vector space.

/**
 * Return the absolute value of the given value
 * @return The absolute value of the given operand
 */
public Result evaluate() {

	// Can't do anything with no data
	if (leftOperand == null && rightOperand == null) {
		throw new IllegalArgumentException("Absolute value is not defined on zero operands");
	}

	// Abs is a unary operator, so error if two operands exist
	if (leftOperand != null && rightOperand != null) {
		throw new IllegalArgumentException("Absolute value is not a binary operator");
	}

	// Select the correct operand
	if (leftOperand == null) {
		return(getResult(rightOperand));
	}
	else { // rightOperand == null
		return(getResult(leftOperand));
	}
}

private Result getResult(Expression value) {

	// Evaluate a nested operation
	if (value instanceof Operation || value instanceof ExpressionReference) {
		value = value.evaluate();
	}

	// Compute the absolute value of a single element
	if (value instanceof MathValue) {

		MathValue mvValue = (MathValue) value;
		return(doAbs(mvValue));
	}

	// Compute the absolute value of each element of a list
	else if (value instanceof ExpressionList) {

		ExpressionList xlValue = (ExpressionList) value;

		// get an iterator over the list
		Iterator valueIter = xlValue.iterator();
		ExpressionList result = new MathExpressionList();

		// Take the absolute value of each element and create the result
		while (valueIter.hasNext()) {
			Expression valueElem = (Expression) valueIter.next();

			result.add(getResult(valueElem));
		}

		// return the result list
		return(result);
	}

	else {
		throw new IllegalArgumentException("Unrecognised math type to Absolute value");
	}
}

/**
 * Compute the absolute value of a math value
 */
private MathValue doAbs(MathValue value) {

	Number result;

	// Absolute value of a scalar quantity
	if (value instanceof MathScalar) {

		MathScalar sValue = (MathScalar) value;
		result = absSimple(sValue.realValue);
	}

	else {
		throw new IllegalArgumentException("Can't compute the absolute value of a vector or matrix.");
	}

	return new MathScalar(result);
}

/**
 * Compute the absolute value of a Number
 */
private Number absSimple(Number value) {

	Number result = null;

	// Don't need to change Integers
	if (value instanceof Integer) {
		result = new Integer(Math.abs(value.intValue()));
	}

	// Round a non-integer
	if (value instanceof Double) {
		result = new Double(Math.abs(value.doubleValue()));
	}

	return(result);
}

}
