/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: Average.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:40 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.formulae;

/**
 * This is a concrete implementation for the average operation.
 * Average (arithmetic mean) is defined as the total value of all
 * elements, divided by the number of elements.
 */
public class Average extends Operation {

/**
 * Average across the given operand (usually a list of expressions)
 */
public Average(Expression operand) {
	super(operand);
}

/**
 * Average a pair of Expressions
 */
public Average(Expression left, Expression right) {
	super(left, right);
}

/**
 * Average an Expression and a Number
 */
public Average(Expression left, Number right) {
	super(left, right);
}

/**
 * Average a Number and an Expression
 */
public Average(Number left, Expression right) {
	super(left, right);
}

/**
 * Average two Numbers
 */
public Average(Number left, Number right) {
	super(left, right);
}

/**
 * Return the result of averaging the operands
 * @return The average value of all the operands
 */
public Result evaluate() {

	MathValue sum, count;
	Operation sumOp, countOp, divOp;

	// Construct a sum operator to total the values
	sumOp = new Sum(leftOperand, rightOperand);

	// Construct a count operator to count the values
	countOp = new Count(leftOperand, rightOperand);

	// Divide the sum by the count
	divOp = new Divide(sumOp, countOp);

	// Evaluate the expression and return the result.
	return(divOp.evaluate());
}

}
