/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: Count.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:40 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.formulae;


import java.util.Iterator;




/**
 * This is a concrete implementation for the count operation.
 * Count determines the total number of elements across its operands.
 */
public class Count extends Operation {

/**
 * Count the given operand (usually a list of expressions)
 */
public Count(Expression operand) {
	super(operand);
}

/**
 * Count a pair of operands
 */
public Count(Expression operand1, Expression operand2) {
	super(operand1, operand2);
}

/**
 * Return the result of counting the operands
 * @return The count of the number of elements in the operands
 */
public Result evaluate() {

	int count = 0;

	// if there is a left operand, count the number elements in it
	if (leftOperand != null) {
		count += countOperands(leftOperand);
	}

	// if there is a left operand, add the number elements in it
	if (rightOperand != null) {
		count += countOperands(rightOperand);
	}

	// return the total number of elements in the left and right operands
	return(new MathScalar(new Integer(count)));
}

/**
 * Count the elements in the given operand
 */
private int countOperands(Expression operand) {

	// Values and operations are considered single elements
	if (operand instanceof MathValue ||
		operand instanceof ExpressionReference ||
		operand instanceof Operation) {
		return(1);
	}

	// Count the number of elements in list
	else if (operand instanceof ExpressionList) {

		int count = 0;

		// Process the elements of the list
		Iterator eli = ((ExpressionList) operand).iterator();
		while (eli.hasNext()) {

			// Accumulate the elements in each entry
			count += countOperands((Expression) eli.next());
		}

		// return the number of elements in this list and all sub-lists
		return(count);
	}

	else {
		throw new IllegalArgumentException("Unrecognized operand type to Count");
	}
}

}
