/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: ExpressionList.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:40 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.formulae;


import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;




/**
 * An expression list is a list (or set) of expressions
 */
public abstract class ExpressionList implements Result, List, Cloneable {

/**
 * As a Result Expression, the value of this list is itself
 * @return This ExpressionList
 */
public Result evaluate() {
	return this;
}

/**
 * Implement the clone method defined by the Expression interface
 */
public Object clone() throws CloneNotSupportedException {
	return(super.clone());
}

// NOTE:  The remainder of this object implements the methods defined by
//        the List interface

/**
 * Add the given element at the specified index in this List
 * @param index The position to insert the given element
 * @param obj The element to be added to this List
 * @throws UnsupportedOperationException add is not supported by this List
 */
// NOTE:  This is optional
public void add(int index, Object obj) {

	// Throw an exception since we don't support this
	throw new UnsupportedOperationException("ExpressionList object do not implement add(int, Object)");
}

/**
 * Append the given element to the end of this List
 * @param obj The element to be added to this List
 * @return True if the addition of the element was possible
 * @throws UnsupportedOperationException add is not supported by this List
 */
// NOTE:  This is optional
public boolean add(Object obj) {

	// Throw an exception since we don't support this
	throw new UnsupportedOperationException("ExpressionList object do not implement add(Object)");
}

/**
 * Append all the elements of the given collection to the end of this List
 * @param coll The collection to be added to this List
 * @return True if this operation changed the List
 * @throws UnsupportedOperationException addAll is not supported by this List
 */
// NOTE:  This is optional
public boolean addAll(Collection coll) {

	// Throw an exception since we don't support this
	throw new UnsupportedOperationException("ExpressionList object do not implement addAll(Collection)");
}

/**
 * Insert the elements of the given collection at the given position in
 * this List
 * @param coll The collection to be added to this List
 * @param index The index in this List at which to insert the elements
 * @return True if this operation changed the List
 * @throws UnsupportedOperationException addAll is not supported by this List
 */
// NOTE:  This is optional
public boolean addAll(int index, Collection coll) {

	// Throw an exception since we don't support this
	throw new UnsupportedOperationException("ExpressionList object do not implement addAll(int, Collection)");
}

/**
 * Clear all elements from this List
 * @throws UnsupportedOperationException clear is not supported by this List
 */
// NOTE:  This is optional
public void clear() {

	// Throw an exception since we don't support this
	throw new UnsupportedOperationException("ExpressionList object do not implement clear()");
}

/**
 * Determine if the List contain the specified element
 * @param obj The element to be searched for in this List
 * @return True if the List contains the given object
 */
public abstract boolean contains(Object obj);

/**
 * Determine if all the elements of the given Collection are in the List
 * @param coll The collection whose elements are sought in this List
 * @return True if every element of the Collection is in this List
 */
public boolean containsAll(Collection coll) {

	// Get an iterator over the elements of the collection
	Iterator iter = coll.iterator();
	boolean all_found = true;

	// Check each element until we're done or one is missing
	while (iter.hasNext() && all_found) {
		all_found = this.contains(iter.next());
	}

	// Return if all the elements were found
	return(all_found);
}

/**
 * Compare the given object and this List, given that equality implies
 * the Lists contain the same entries in the same order.
 * @param obj the object whose equality with this List is tested
 * @return True if the given object satisfies "equality" with this List
 */
public abstract boolean equals(Object obj);

/**
 * Retrieve the element at a given position in this List
 * @param index The desired element's position
 * @return The element found at the given position
 */
public abstract Object get(int index);

/**
 * Find the position (first occurence) of a given object in the List.
 * -1 is returned if the element is not found in the List.
 * @param obj The object to be found in the List
 * @return The location of the object in the List (or -1)
 */
public abstract int indexOf(Object obj);

/**
 * Determine if the List contains no elements
 * @return True if this List contains no elements
 */
public boolean isEmpty() {

	// List is empty if it contains zero elements
	return(this.size() == 0);
}

/**
 * Get an iterator over the elements in this List
 * @return an iterator object which enumerates the elements in this List
 */
public abstract Iterator iterator();

/**
 * Find the position of the last occurence of a given object in the List.
 * -1 is returned if the element is not found in the List.
 * @param obj The object to be found in the List
 * @return The last location the object is found in the List (or -1)
 */
public abstract int lastIndexOf(Object o);

/**
 * Get a ListIterator over the elements of the List
 * @return A ListIterator over this List
 */
public ListIterator listIterator() {

	// Get a ListIterator starting at the first element
	return(this.listIterator(0));
}

/**
 * Get a ListIterator over the elements of the List beginning at the
 * the given position in the list
 * @param index The position at which to initialise the ListIterator
 * @return A ListIterator over this List
 */
public abstract ListIterator listIterator(int index);

/**
 * Remove the element at the given position from the List
 * @param index The position of the element to be removed from this List
 * @return The Object that has been removed from the List
 * @throws UnsupportedOperationException remove is not supported by this List
 */
// NOTE:  This is optional
public Object remove(int index) {

	// Throw an exception since we don't support this
	throw new UnsupportedOperationException("ExpressionList object do not implement remove(int)");
}

/**
 * Remove the given element from the List (if it exists)
 * @param obj The element to remove from this List
 * @return True if the given element was in the List
 * @throws UnsupportedOperationException remove is not supported by this List
 */
// NOTE:  This is optional
public boolean remove(Object o) {

	// Throw an exception since we don't support this
	throw new UnsupportedOperationException("ExpressionList object do not implement remove(Object)");
}

/**
 * Remove all elements of the given collection from the List (if they exist)
 * @param coll The Collection whose elements are to be removed from the List
 * @return True if any of the given elements was in the List
 * @throws UnsupportedOperationException removeAll is not supported by this List
 */
// NOTE:  This is optional
public boolean removeAll(Collection coll) {

	// Throw an exception since we don't support this
	throw new UnsupportedOperationException("ExpressionList object do not implement removeAll(Collection)");
}

/**
 * Remove all elements in this List which are not in the given Collection.
 * @param coll The Collection whose elements are to be kept in the List
 * @return True if any elements were removed from the List
 * @throws UnsupportedOperationException retainAll is not supported by this List
 */
// NOTE:  This is optional
public boolean retainAll(Collection c) {

	// Throw an exception since we don't support this
	throw new UnsupportedOperationException("ExpressionList object do not implement retainAll(Collection)");
}

/**
 * Set the element at a given position in this List
 * @param index The desired position of the new element
 * @param obj The object to be stored at the given location
 * @return The element previously found at that position
 */
// NOTE:  This is optional
public Object set(int index, Object obj) {

	// Throw an exception since we don't support this
	throw new UnsupportedOperationException("ExpressionList object do not implement set(int, Object)");
}

/**
 * Return the number of elements in this List
 * @return the cardinality of this List
 */
public abstract int size();

/**
 * Get a List which is a subsequence of this List (the ordering is the same)
 * @param startIndex The first element of the sublist
 * @param endIndex The element after the last element of the sublist
 * @return A List which contains the elements of the specified subset
 */
public abstract List subList(int startIndex, int endIndex);

/**
 * Get an array view of all the elements in this List
 * NOTE:  The Iterator's ordering is used
 * @return an Array of all the elements in this List
 */
public Object[] toArray() {

	// Create an array to store the elements in
	Object[] array = new Object[this.size()];

	// Fill in the array
	return(this.toArray(array));
}

/**
 * Fill a given array with a view of all the elements in this List
 * NOTE:  The Iterator's ordering is used
 * @param inArray The array in which to store the elements
 * @return an Array of all the elements in this List
 */
public abstract Object[] toArray(Object inArray[]);

}
