/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: GeometricMean.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:40 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.formulae;

/**
 * This operation computes the geometric mean of a set of numbers
 * Geometric mean is defined as the Nth root of the product of a set
 * of N numbers.
 */
public class GeometricMean extends Operation {

/**
 * The geometric mean of the given list of elements
 */
public GeometricMean(Expression operand) {
	super(operand);
}

/**
 * Geometric mean of a pair of Expressions
 */
public GeometricMean(Expression left, Expression right) {
	super(left, right);
}

/**
 * Geometric mean of an Expression and a Number
 */
public GeometricMean(Expression left, Number right) {
	super(left, right);
}

/**
 * Geometric mean of a Number and an Expression
 */
public GeometricMean(Number left, Expression right) {
	super(left, right);
}

/**
 * Geometric mean of two Numbers
 */
public GeometricMean(Number left, Number right) {
	super(left, right);
}

/**
 * Return the result of computing the geometric mean of
 * the operands
 * @return The geometric mean of all the operands
 */
public Result evaluate() {

	MathScalar count, min;
	Operation minOp, countOp, productOp, powerOp;

	// Construct a count operator to count the values
	countOp = new Count(leftOperand, rightOperand);
	count = (MathScalar) countOp.evaluate();

	// We must have a non-zero number of elements
	if (count.realValue.intValue() == 0) {
		throw new IllegalArgumentException("Must have at least 1 element to compute the geometric mean");
	}

	// Find the minimum value of the set
	minOp = new Min(leftOperand, rightOperand);
	// NOTE:  This will fail (correctly) if a vector or matrix is provided
	min = (MathScalar) minOp.evaluate();

	// The minimum must be strictly positive
	if (min.realValue.doubleValue() <= 0) {
		throw new IllegalArgumentException("Geometric mean:  Cannot use negative or zero values");
	}

	// Construct a product operator to multiply the values
	productOp = new Product(leftOperand, rightOperand);

	// Compute the Nth root (the exponent is 1/N)
	powerOp = new Power(productOp, new Divide(new MathScalar(1.0), count));

	// Return the geometric mean of the data set
	return(powerOp.evaluate());
}

}
