/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: MathExpressionList.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:40 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.formulae;


import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Vector;




/**
 * A MathExpressionList is a List of Expressions
 */
public class MathExpressionList extends ExpressionList {

/** We store the data in this List in a Vector */
protected List		dataVector = new Vector();

/**
 * Create an empty MathExpressionList
 */
public MathExpressionList() {
}

/**
 * Create a MathExpressionList from an array of Expressions
 * @param exprs An array of Expressions
 */
public MathExpressionList(Expression[] exprs) {

	// Store the Expressions in the List
	for (int i = 0; i < exprs.length; i++) {
		dataVector.add(exprs[i]);
	}
}

/**
 * Return a clone of this ExpressionList (all members are cloned as well)
 */
public Object clone() {

	// Get the expressions stored by this List
	Object[] valueArray = this.toArray();
	Expression[] fixedArray = new Expression[valueArray.length];

	// Clone all expressions
	for (int i = 0; i < valueArray.length; i++) {
		try {
			fixedArray[i] = (Expression) ((Expression) valueArray[i]).clone();
		}
		catch (CloneNotSupportedException e) {
			fixedArray[i] = null;
		}
	}

	// Create and return a new List
	return((Object) new MathExpressionList(fixedArray));
}

// NOTE:  The remainder of this object implements the methods defined by
//        the List interface

/**
 * Add the given element at the specified index in this List
 * @param index The position to insert the given element
 * @param obj The element to be added to this List
 */
// NOTE:  This is optional
public void add(int index, Object obj) {

	// Make sure only Expressions are added to ExpressionLists
	if (!(obj instanceof Expression)) {
		throw new IllegalArgumentException("Must add only an Expression to ExpressionLists");
	}

	// Add the Expression to the List
	dataVector.add(index, obj);
}

/**
 * Append the given element to the end of this List
 * @param obj The element to be added to this List
 * @return True if the addition of the element was possible
 */
// NOTE:  This is optional
public boolean add(Object obj) {

	// Make sure only Expressions are added to ExpressionLists
	if (!(obj instanceof Expression)) {
		throw new IllegalArgumentException("Must add only an Expression to ExpressionLists");
	}

	// Add the Expression to the List
	return(dataVector.add(obj));
}

/**
 * Append all the elements of the given collection to the end of this List
 * @param coll The collection to be added to this List
 * @return True if this operation changed the List
 */
// NOTE:  This is optional
public boolean addAll(Collection coll) {

	// get an iterator over the collection
	Iterator ci = coll.iterator();

	// Verify the elements of the Collection are all Expressions
	while (ci.hasNext()) {

		// Get an element from the collection
		Object obj = ci.next();

		// Make sure only Expressions are added to ExpressionLists
		if (!(obj instanceof Expression)) {
			throw new IllegalArgumentException("Must add only an Expression to ExpressionLists");
		}
	}

	// Add the Expressions to the List
	return(dataVector.addAll(coll));
}

/**
 * Insert the elements of the given collection at the given position in
 * this List
 * @param coll The collection to be added to this List
 * @param index The index in this List at which to insert the elements
 * @return True if this operation changed the List
 */
// NOTE:  This is optional
public boolean addAll(int index, Collection coll) {

	// get an iterator over the collection
	Iterator ci = coll.iterator();

	// Verify the elements of the Collection are all Expressions
	while (ci.hasNext()) {

		// Get an element from the collection
		Object obj = ci.next();

		// Make sure only Expressions are added to ExpressionLists
		if (!(obj instanceof Expression)) {
			throw new IllegalArgumentException("Must add only an Expression to ExpressionLists");
		}
	}

	// Add the Expressions to the List
	return(dataVector.addAll(index, coll));
}

/**
 * Clear all elements from this List
 */
// NOTE:  This is optional
public void clear() {

	// Call clear on the contents List
	dataVector.clear();
}

/**
 * Determine if the List contain the specified element
 * @param obj The element to be searched for in this List
 * @return True if the List contains the given object
 */
public boolean contains(Object obj) {

	// pass the check through to the contents List
	return(dataVector.contains(obj));
}

/**
 * Determine if all the elements of the given Collection are in the List
 * @param coll The collection whose elements are sought in this List
 * @return True if every element of the Collection is in this List
 */
public boolean containsAll(Collection coll) {

	// pass the check through to the contents List
	return(dataVector.containsAll(coll));
}

/**
 * Compare the given object and this List, given that equality implies
 * the Lists contain the same entries in the same order.
 * @param obj the object whose equality with this List is tested
 * @return True if the given object satisfies "equality" with this List
 */
public boolean equals(Object obj) {

	// Both objects must be MathExpressionLists
	if (!(obj instanceof MathExpressionList)) {
		return(false);
	}

	// Compare the contents of the data lists
	return(dataVector.equals(((MathExpressionList) obj).dataVector));
}

/**
 * Retrieve the element at a given position in this List
 * @param index The desired element's position
 * @return The element found at the given position
 */
public Object get(int index) {

	return(dataVector.get(index));
}

/**
 * Return a hash code derived from this List
 * @return the hash code of this List
 */
public int hashCode() {
	return(31 * dataVector.hashCode() + super.hashCode());
}

/**
 * Find the position (first occurence) of a given object in the List.
 * -1 is returned if the element is not found in the List.
 * @param obj The object to be found in the List
 * @return The location of the object in the List (or -1)
 */
public int indexOf(Object obj) {

	// Get the index of the object in the contents List
	return(dataVector.indexOf(obj));
}

/**
 * Determine if the List contains no elements
 * @return True if this List contains no elements
 */
public boolean isEmpty() {

	// Check if the contents list has any elements
	return(dataVector.isEmpty());
}

/**
 * Get an iterator over the elements in this List
 * @return an iterator object which enumerates the elements in this List
 */
public Iterator iterator() {

	// Return an iterator over the contents list
	return(dataVector.iterator());
}

/**
 * Find the position of the last occurence of a given object in the List.
 * -1 is returned if the element is not found in the List.
 * @param obj The object to be found in the List
 * @return The last location the object is found in the List (or -1)
 */
public int lastIndexOf(Object o) {

	// Get the index of the last occurence of the object in the contents List
	return(dataVector.lastIndexOf(o));
}

/**
 * Get a ListIterator over the elements of the List
 * @return A ListIterator over this List
 */
public ListIterator listIterator() {

	// Return an iterator over the contents list
	return(dataVector.listIterator());
}

/**
 * Get a ListIterator over the elements of the List beginning at the
 * the given position in the list
 * @param index The position at which to initialise the ListIterator
 * @return A ListIterator over this List
 */
public ListIterator listIterator(int index) {

	// Return an iterator over the contents list from the given index
	return(dataVector.listIterator(index));
}

/**
 * Remove the element at the given position from the List
 * @param index The position of the element to be removed from this List
 * @return The Object that has been removed from the List
 */
// NOTE:  This is optional
public Object remove(int index) {

	// Remove the selected element from the contents list
	return(dataVector.remove(index));
}

/**
 * Remove the given element from the List (if it exists)
 * @param obj The element to remove from this List
 * @return True if the given element was in the List
 */
// NOTE:  This is optional
public boolean remove(Object o) {

	// Remove the selected object from the contents list
	return(dataVector.remove(o));
}

/**
 * Remove all elements of the given collection from the List (if they exist)
 * @param coll The Collection whose elements are to be removed from the List
 * @return True if any of the given elements was in the List
 */
// NOTE:  This is optional
public boolean removeAll(Collection coll) {

	// Remove the elements of the collection from the contents list
	return(dataVector.removeAll(coll));
}

/**
 * Remove all elements in this List which are not in the given Collection.
 * @param coll The Collection whose elements are to be kept in the List
 * @return True if any elements were removed from the List
 */
// NOTE:  This is optional
public boolean retainAll(Collection c) {

	// Keep only the elements of the collection in the contents list
	return(dataVector.retainAll(c));
}

/**
 * Set the element at a given position in this List
 * @param index The desired position of the new element
 * @param obj The object to be stored at the given location
 * @return The element previously found at that position
 */
// NOTE:  This is optional
public Object set(int index, Object obj) {

	// Make sure only Expressions are added to ExpressionLists
	if (!(obj instanceof Expression)) {
		throw new IllegalArgumentException("Must add only an Expression to ExpressionLists");
	}

	// Set the Expression at the given location
	return(dataVector.set(index, obj));
}

/**
 * Return the number of elements in this List
 * @return the cardinality of this List
 */
public int size() {

	// Ask the contents list to report its size
	return(dataVector.size());
}

/**
 * Get a List which is a subsequence of this List (the ordering is the same)
 * @param startIndex The first element of the sublist
 * @param endIndex The element after the last element of the sublist
 * @return A List which contains the elements of the specified subset
 */
public List subList(int startIndex, int endIndex) {

	// Create a new MathExpressionList
	MathExpressionList sub = new MathExpressionList();

	// Use a sublist of the contents List for the sublist's contents
	sub.dataVector = dataVector.subList(startIndex, endIndex);

	return(sub);
}

/**
 * Fill a given array with a view of all the elements in this List
 * NOTE:  The Iterator's ordering is used
 * @param inArray The array in which to store the elements
 * @return an Array of all the elements in this List
 */
public Object[] toArray(Object[] inArray) {

	// Make sure the array is large enough
	if (inArray.length < dataVector.size()) {
		inArray = new Object[dataVector.size()];
	}
	
	// Get an iterator over the contents vector
	Iterator iter = dataVector.iterator();
	int index = 0;

	// Copy the elements from the List to the array
	while (iter.hasNext()) {
		inArray[index++] = iter.next();
	}

	// return the completed array
	return(inArray);
}

}
