/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: MathMatrix.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:40 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.formulae;




/**
 * The class of matrices (we almost undoubtedly do not ever need to go
 * beyond this point, since nobody in their right mind would ever try
 * to do tensor math in a spreadsheet).
 * NOTE:  We probably do not need to implement this for the first version
 *        but I include it here as a potential consideration for future
 *        development
 */
public class MathMatrix extends MathValue {

/** The values are stored in an array */
protected Number[][] realValues = null;
/** Rows and Columns define the number of entries in the matrix */
protected int        rows = 0, columns = 0;

/**
 * Create a default matrix (all entries initialised to 0)
 */
public MathMatrix() {
	int DEFAULT_SIZE = 3;
	rows = DEFAULT_SIZE;
	columns = DEFAULT_SIZE;

	// Use the default size for length
	realValues = new Number[DEFAULT_SIZE][DEFAULT_SIZE];
	for (int i = 0; i < DEFAULT_SIZE; i++) {
		for (int j = 0; j < DEFAULT_SIZE; j++) {
			realValues[i][j] = new Double(0.0);
		}
	}
}

/**
 * Create a matrix from an array of double values
 */
public MathMatrix(double[][] values) {

	// Determine the number of columns
	rows = values.length;
	for (int i = 0; i < rows; i++) {
		if (values[i].length > columns) {
			columns = values[i].length;
		}
	}

	// Size the matrix
	realValues = new Number[rows][columns];

	// Copy the values to the matrix
	for (int i = 0; i < rows; i++) {
		for (int j = 0; j < values[i].length; j++) {
			realValues[i][j] = new Double(values[i][j]);
		}
		if (values[i].length < columns) {
			for (int j = values[i].length; j < columns; j++) {
				realValues[i][j] = new Double(0.0);
			}
		}
	}
}

/**
 * Create a matrix from an array of integer values
 */
public MathMatrix(int[][] values) {

	// Determine the number of columns
	rows = values.length;
	for (int i = 0; i < rows; i++) {
		if (values[i].length > columns) {
			columns = values[i].length;
		}
	}

	// Size the matrix
	realValues = new Number[rows][columns];

	// Copy the values to the matrix
	for (int i = 0; i < rows; i++) {
		for (int j = 0; j < values[i].length; j++) {
			realValues[i][j] = new Integer(values[i][j]);
		}
		if (values[i].length < columns) {
			for (int j = values[i].length; j < columns; j++) {
				realValues[i][j] = new Integer(0);
			}
		}
	}
}

/**
 * Create a matrix from an array of Numbers
 */
public MathMatrix(Number[][] values) {

	// Determine the number of columns
	rows = values.length;
	for (int i = 0; i < rows; i++) {
		if (values[i].length > columns) {
			columns = values[i].length;
		}
	}

	// Size the matrix
	realValues = new Number[rows][columns];

	// Copy the values to the matrix
	for (int i = 0; i < rows; i++) {
		for (int j = 0; j < values[i].length; j++) {
			realValues[i][j] = cloneNumber(values[i][j]);
		}
		if (values[i].length < columns) {
			for (int j = values[i].length; j < columns; j++) {
				realValues[i][j] = new Integer(0);
			}
		}
	}
}

/**
 * Get the contents of this MathValue as a Number
 * @return A Number which is the contents of this MathValue
 */
public Number numberValue() {

	// We can't return the contents of a Scalar as a matrix
	throw new UnsupportedOperationException("Can't return a matrix as a scalar");
}

/**
 * Get the contents of this MathValue as a vector of Numbers
 * @return A vector of Numbers which is the contents of this MathValue
 */
public Number[] vectorValue() {

	// We can't return the contents of a Scalar as a matrix
	throw new UnsupportedOperationException("Can't return a matrix as a vector");
}

/**
 * Get the contents of this MathValue as a matrix of Numbers
 * @return A matrix of Numbers which is the contents of this MathValue
 */
public Number[][] matrixValue() {

	return(realValues);
}

/**
 * Retrieve the value at a particular index in the matrix
 * @param row, column The location at which to find the value
 * @return The MathValue stored at a given location
 */
public Expression getValueAt(int row, int column) {

	// Check for an out-of-bounds operation
	if (row < 0 || row >= rows) {
		throw new ArrayIndexOutOfBoundsException("MathMatrix index " +
										row + " is out of range [0.." +
										(rows - 1) + "]");
	}
	if (column < 0 || column >= columns) {
		throw new ArrayIndexOutOfBoundsException("MathMatrix index " +
										column + " is out of range [0.." +
										(columns - 1) + "]");
	}

	return(new MathScalar(realValues[row][column]));
}

/**
 * Set the value at a particular index in the matrix
 * @param row, column The location at which to store the value
 * @param value The MathValue to store at the given location
 */
public void setValueAt(int row, int column, Number value) {

	// Check for an out-of-bounds operation
	if (row < 0 || row >= rows) {
		throw new ArrayIndexOutOfBoundsException("MathMatrix index " +
										row + " is out of range [0.." +
										(rows - 1) + "]");
	}
	if (column < 0 || column >= columns) {
		throw new ArrayIndexOutOfBoundsException("MathMatrix index " +
										column + " is out of range [0.." +
										(columns - 1) + "]");
	}

	realValues[row][column] = cloneNumber(value);
}

private Number cloneNumber(Number n) {
	if (n instanceof Integer) {
		return(new Integer(n.intValue()));
	}
	else { // n instanceof Double
		return(new Double(n.doubleValue()));
	}
}

/**
 * Return a string representation of this value
 * @return A string containing this value
 */
public String toString() {

	int				i, j, pad, longest = 0;
	int[]			sblen = new int[rows];
	String			padString = "                         ";
	StringBuffer	top, output;
	StringBuffer[]	sb = new StringBuffer[rows];

	// Create the stringbuffers
	for (i = 0; i < rows; i++) {
		sb[i] = new StringBuffer();
	}

	// Store the columns of numbers in the stringbuffers, last column first
	for (j = columns - 1; j >= 0; j--) {

		// Store a column of numbers in the stringbuffers
		for (i = 0; i < rows; i++) {
			sb[i].insert(0, realValues[i][j].toString());

			// Track the longest stringbuffer
			sblen[i] = sb[i].length();
			if (sblen[i] > longest) {
				longest = sblen[i];
			}
		}

		// Pad all other strings to the same length
		for (i = 0; i < rows; i++) {
			pad = longest - sblen[i] + 1;
			sb[i].insert(0, padString.substring(0, pad));
		}
	}

	// Add brackets for the matrix
	for (i = 0; i < rows; i++) {
		sb[i].insert(0, "|");
		sb[i].append(" |\n");
	}

	// Build the top/bottom of the brackets
	top = new StringBuffer("--");
	while (top.length() + padString.length() < longest + 2) {
		top.append(padString);
	}
	top.append(padString.substring(0, longest + 2 - top.length()));
	top.append("--");

	// Assemble the matrix
	output = new StringBuffer();
	output.append(top);
	output.append("\n");
	for (i = 0; i < rows; i++) {
		output.append(sb[i]);
	}
	output.append(top);

	return(output.toString());
}

}
