/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: MathScalar.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:40 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.formulae;




/**
 * MathScalar is a scalar constant represented as a MathValue.
 * By encapsulating it in this fashion we can potentially extend
 * this class in future to support complex numbers as well as
 * integer and real numbers.
 */
public class MathScalar extends MathValue {

/** A real number stored in the appropriate format */
protected Number realValue;
/** FUTURE:  The real coefficient of i for complex numbers */
// protected Number imaginaryValue

/**
 * Create a MathScalar (default value 0.0)
 */
public MathScalar() {
	realValue = new Double(0.0);
}

/**
 * Create a MathScalar from a double
 */
public MathScalar(double value) {
	realValue = new Double(value);
}

/**
 * Create a MathScalar from an integer
 */
public MathScalar(int value) {
	realValue = new Integer(value);
}

// NOTE:  How many other data-types (Float, Long, BigDecimal, etc.)
//        do we need to support?

/** 
 * Create a MathScalar from a Number
 */
public MathScalar(Number value) {
	realValue = cloneNumber(value);
}

private Number cloneNumber(Number n) {
	if (n instanceof Integer) {
		return(new Integer(n.intValue()));
	}
	else { // n instanceof Double
		return(new Double(n.doubleValue()));
	}
}

/**
 * Get the contents of this MathValue as a Number
 * @return A Number which is the contents of this MathValue
 */
public Number numberValue() {

	return(realValue);
}

/**
 * Get the contents of this MathValue as a vector of Numbers
 * @return A vector of Numbers which is the contents of this MathValue
 */
public Number[] vectorValue() {

	// We can't return the contents of a Scalar as a matrix
	throw new UnsupportedOperationException("Can't return a scalar as a vector");
}

/**
 * Get the contents of this MathValue as a matrix of Numbers
 * @return A matrix of Numbers which is the contents of this MathValue
 */
public Number[][] matrixValue() {

	// We can't return the contents of a Scalar as a matrix
	throw new UnsupportedOperationException("Can't return a scalar as a matrix");
}

/**
 * Return a string representation of this value
 * @return A string containing this value
 */
public String toString() {
	return(realValue.toString());
}

}
