/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: MathValue.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:40 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.formulae;

import java.text.NumberFormat;

/**
 * The class MathValue forms the root for all derived constant-based
 * result/data classes.
 */
public abstract class MathValue implements Result, Cloneable {

/** The format used to output values stored in results */
public NumberFormat dataFormat;

/**
 * Satisfy the Expression interface by returning the stored value
 * (no evaluation is required).
 * @return The value this object represents
 */
public Result evaluate() {
	return this;
}

/**
 * Retrieve the NumberFormat associated with this data
 * @return The NumberFormat used to convert this data to a String
 */
public NumberFormat getDataFormat() {
	return(dataFormat);
}

/**
 * Set a NumberFormat to use on the contents of this MathValue
 * @param format The NumberFormat to be used to format this as a String
 */
public void setDataFormat(NumberFormat format) {
	this.dataFormat = format;
}

/**
 * Get the contents of this MathValue as a Number
 * @return A Number which is the contents of this MathValue
 */
public abstract Number numberValue();

/**
 * Get the contents of this MathValue as a vector of Numbers
 * @return A vector of Numbers which is the contents of this MathValue
 */
public abstract Number[] vectorValue();

/**
 * Get the contents of this MathValue as a matrix of Numbers
 * @return A matrix of Numbers which is the contents of this MathValue
 */
public abstract Number[][] matrixValue();

/**
 * Return an Expression which is a copy of this MathValue
 */
public Object clone() {

	Expression newValue;

	try {
    	newValue = (Expression) super.clone();
	}
	catch (CloneNotSupportedException e) {
		return(null);
	}

    return(newValue);
}

}
