/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: MathVector.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:40 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.formulae;




/**
 * The class of vectors (in a linear algebra sense) for matrix math.
 * NOTE:  We probably do not need to implement this for the first version
 *        but I include it here as a potential consideration for future
 *        development
 */
public class MathVector extends MathValue {

// NOTE:  We may wish to use a Vector for ease
/** The values are stored in an array */
protected Number[] realValues = null;
/** Size is the number of elements in the vector */
protected int      size = 0;

/**
 * Create a default vector (all entries initialised to 0)
 */
public MathVector() {
	int DEFAULT_SIZE = 3;
	size = DEFAULT_SIZE;

	// Use the default size for length
	realValues = new Number[DEFAULT_SIZE];
	for (int i = 0; i < DEFAULT_SIZE; i++) {
		realValues[i] = new Double(0.0);
	}
}

/**
 * Create a vector from an array of double values
 */
public MathVector(double[] values) {

	// Size the vector
	size = values.length;
	realValues = new Number[size];

	// Copy the values to the vector
	for (int i = 0; i < size; i++) {
		realValues[i] = new Double(values[i]);
	}
}

/**
 * Create a vector from an array of integer values
 */
public MathVector(int[] values) {

	// Size the vector
	size = values.length;
	realValues = new Number[size];

	// Copy the values to the vector
	for (int i = 0; i < size; i++) {
		realValues[i] = new Integer(values[i]);
	}
}

/**
 * Create a vector from an array of Numbers
 */
public MathVector(Number[] values) {

	// Size the vector
	size = values.length;
	realValues = new Number[size];

	// Copy the values to the vector
	for (int i = 0; i < size; i++) {
		realValues[i] = cloneNumber(values[i]);
	}
}

/**
 * Get the contents of this MathValue as a Number
 * @return A Number which is the contents of this MathValue
 */
public Number numberValue() {

	// We can't return the contents of a Scalar as a matrix
	throw new UnsupportedOperationException("Can't return a vector as a scalar");
}

/**
 * Get the contents of this MathValue as a vector of Numbers
 * @return A vector of Numbers which is the contents of this MathValue
 */
public Number[] vectorValue() {

	return(realValues);
}

/**
 * Get the contents of this MathValue as a matrix of Numbers
 * @return A matrix of Numbers which is the contents of this MathValue
 */
public Number[][] matrixValue() {

	// We can't return the contents of a Scalar as a matrix
	throw new UnsupportedOperationException("Can't return a vector as a matrix");
}
/**
 * Retrieve the value at a particular index in the vector
 * @param index The location at which to find the value
 * @return The MathValue stored at a given location
 */
public Expression getValueAt(int index) {

	// Check for an out-of-bounds operation
	if (index < 0 || index >= size) {
		throw new ArrayIndexOutOfBoundsException("MathVector index " +
										index + " is out of range [0.." +
										(size - 1) + "]");
	}

	return(new MathScalar(realValues[index]));
}

/**
 * Set the value at a particular index in the vector
 * @param index The location at which to store the value
 * @param value The MathValue to store at the given location
 */
public void setValueAt(int index, Number value) {

	// Check for an out-of-bounds operation
	if (index < 0 || index >= size) {
		throw new ArrayIndexOutOfBoundsException("MathVector index " +
										index + " is out of range [0.." +
										(size - 1) + "]");
	}

	realValues[index] = cloneNumber(value);
}

private Number cloneNumber(Number n) {
	if (n instanceof Integer) {
		return(new Integer(n.intValue()));
	}
	else { // n instanceof Double
		return(new Double(n.doubleValue()));
	}
}

/**
 * Output the value of this vector as a string
 * @return A string representation of this vector
 */
public String toString() {

	StringBuffer sb = new StringBuffer("( ");

	// List the elements of the vector in order
	for (int i = 0; i < size; i++) {
		sb.append(realValues[i].toString());
		sb.append(" ");
	}
	sb.append(")");
	return(sb.toString());
}

}
