/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: Median.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:40 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.formulae;


import java.util.Iterator;
import java.util.Comparator;




/**
 * This operation computes the median value in a set of elements.
 * The Median of a set is the middle element of the set, (*** or the
 * average of the two middle values of a set. ***)
 */
public class Median extends Operation {

/**
 * Determine the Median of the given set
 */
public Median(Expression operand) {
	super(operand);
}

/**
 * Determine the Median of a pair of sets
 */
public Median(Expression operand1, Expression operand2) {
	super(operand1, operand2);
}

/**
 * Return the median of the operands
 * @return The result of determining the middle value of the set
 */
public Result evaluate() {

	if (leftOperand == null && rightOperand == null) {
		throw new IllegalArgumentException("Median is not defined on zero operands");
	}

	// Construct a single list of all the operands
	ExpressionList bigList = new MathExpressionList();

	// If the left operand is non-null, add it to the list
	if (leftOperand != null) {
		addElements(bigList, leftOperand);
	}

	// If the right operand is non-null, add it to the list
	if (rightOperand != null) {
		addElements(bigList, rightOperand);
	}

	int elementCount = bigList.size();
	int location = (elementCount + 1) / 2;

	// Find the element at the given location in the list of MathValues
	MathValue result = findLocation(bigList, location);

	// return the median value
	return(result);
}

/**
 * Add elements from the given operand to a list
 */
private void addElements(ExpressionList target, Expression operand) {

	// Store a math value
	if (operand instanceof MathValue) {

		// We can only find the median in a set of MathScalars
		if (operand instanceof MathVector ||
			operand instanceof MathMatrix) {
			throw new IllegalArgumentException("Median:  No ordering defined for vectors or matrices");
		}

		target.add(operand);
	}

	// Store the result of an operation
	else if (operand instanceof Operation ||
			 operand instanceof ExpressionReference) {
		target.add(operand.evaluate());
	}

	// If this is a list, copy the elements
	else if (operand instanceof ExpressionList) {
		Iterator eli = ((ExpressionList) operand).iterator();

		// Add all elements of the list to the target
		while (eli.hasNext()) {
			Expression elem = (Expression) eli.next();
			addElements(target, elem);
		}
	}

	else {
		throw new IllegalArgumentException("Median:  Unrecognised math type in ExpressionList");
	}
}

/**
 * Find the element at the given location in the list
 */
private MathValue findLocation(ExpressionList list, int location) {

	// get a pivot to sort the list
	MathValue pivot = (MathValue) list.get(location);

	// Get a comparator to compare two MathValues as real numbers
	Comparator realCmp = new RealComparator();

	// Create lists to store the subsets
	ExpressionList less = new MathExpressionList();
	ExpressionList greater = new MathExpressionList();
	ExpressionList equal = new MathExpressionList();

	Iterator eli = list.iterator();

	// loop through the elements of the list
	while (eli.hasNext()) {
		MathValue elem = (MathValue) eli.next();

		// Store the element in the correct list
		if (realCmp.compare(elem, pivot) < 0) {
			less.add(elem);
		}
		else if (realCmp.compare(elem, pivot) > 0) {
			greater.add(elem);
		}
		else {
			equal.add(elem);
		}
	}

	// determine which list holds the desired element
	if (less.size() >= (location + 1)) {
		return(findLocation(less, location));
	}
	else {
		location -= less.size();
	}

	if (equal.size() >= (location + 1)) {
		return(pivot);
	}
	else {
		location -= equal.size();
	}

	if (greater.size() >= (location + 1)) {
		return(findLocation(greater, location));
	}
	else {
		throw new IllegalArgumentException("Median:  invalid MathValues in list");
	}
}

}
