/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: RealComparator.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:41 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.formulae;


import java.util.Comparator;




/**
 * A RealComparator compares two real-valued MathValues
 * NOTE:  We implement this so that (eventually, if we desire) we can add
 *        additional comparators to provide such things as a mechanism for
 *        comparison of vectors or complex numbers
 */

public final class RealComparator implements Comparator {

public RealComparator() { }

public int compare(Object o1, Object o2) {

	// Two nulls must compare as equal (see Comparator interface description)
	if (o1 == null && o2 == null) {
		return(0);
	}

	if (!(o1 instanceof MathScalar) || !(o2 instanceof MathScalar)) {
		throw new IllegalArgumentException("Can only compare two scalar values");
	}

	MathScalar value1 = (MathScalar) o1;
	MathScalar value2 = (MathScalar) o2;

	double dval1 = value1.realValue.doubleValue();
	double dval2 = value2.realValue.doubleValue();

	if (dval1 > dval2) {
		return(1);
	}
	else if (dval1 < dval2) {
		return(-1);
	}
	else {
		// numbers are equal
		return(0);
	}
}

public boolean equals(Object obj) {

	// Since we only have one RealComparator, we equal all others
	if (obj instanceof RealComparator) {
		return(true);
	}
	else {
		// all other Comparators, non-Comparators and obj == null
		return(false);
	}
}

}
