/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: Root.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:41 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.formulae;


import java.util.Iterator;




/**
 * This is a concrete implementation for square roots
 * This is a unary operator, *???but can also work on lists???*.
 */
public class Root extends Operation {

/**
 * Define a square root on the given Expression
 */
public Root(Expression operand) {
	super(operand);
}

/**
 * Define a square root on the given Number
 */
public Root(Number operand) {
	super(operand);
}

/**
 * Return the square root of the given value
 * @return The square root of the given operand
 */
public Result evaluate() {

	// Can't do anything with no data
	if (leftOperand == null && rightOperand == null) {
		throw new IllegalArgumentException("Square root is not defined on zero operands");
	}

	// Square root is a unary operator, so error if two operands exist
	if (leftOperand != null && rightOperand != null) {
		throw new IllegalArgumentException("Square root is not a binary operator");
	}

	// Select the correct operand
	if (leftOperand == null) {
		return(getResult(rightOperand));
	}
	else { // rightOperand == null
		return(getResult(leftOperand));
	}
}

private Result getResult(Expression value) {

	// Evaluate a nested operation
	if (value instanceof Operation || value instanceof ExpressionReference) {
		value = value.evaluate();
	}

	// Compute the square root of a single element
	if (value instanceof MathValue) {

		MathValue mvValue = (MathValue) value;
		return(doRoot(mvValue));
	}

	// Compute the square root of each element of a list
	else if (value instanceof ExpressionList) {

		ExpressionList xlValue = (ExpressionList) value;

		// get an iterator over the list
		Iterator valueIter = xlValue.iterator();
		ExpressionList result = new MathExpressionList();

		// Take the square root of each element and create the result
		while (valueIter.hasNext()) {
			Expression valueElem = (Expression) valueIter.next();

			result.add(getResult(valueElem));
		}

		// return the result list
		return(result);
	}

	else {
		throw new IllegalArgumentException("Unrecognised math type to Square root");
	}
}

/**
 * Take the square root of a math value
 */
private MathValue doRoot(MathValue value) {

	Number result;

	// Take the square root of a scalar quantity
	if (value instanceof MathScalar) {

		MathScalar sValue = (MathScalar) value;
		result = rootSimple(sValue.realValue);
	}

	else {
		throw new IllegalArgumentException("Can't take the square root of a vector or matrix.");
	}

	return new MathScalar(result);
}

/**
 * Compute the square root of a Number
 */
private Number rootSimple(Number value) {

	Number result = null;

	// We always return a double
	result = new Double(Math.sqrt(value.doubleValue()));

	return(result);
}

}
