/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: Round.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:41 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.formulae;


import java.util.Iterator;




/**
 * This is a concrete implementation for the round operation.
 * Round is defined as nearest integer, with rounding done to
 * an even number if the operand is exactly between two integers.
 * This is a unary operator, *???but can also work on lists???*.
 */
public class Round extends Operation {

/**
 * Define a rounding operation on the given Expression
 */
public Round(Expression operand) {
	super(operand);
}

/**
 * Define a rounding operation on the given Number
 */
public Round(Number operand) {
	super(operand);
}

/**
 * Return the nearest integer to the given value
 * @return The rounded value of the given operand
 */
public Result evaluate() {

	// Can't do anything with no data
	if (leftOperand == null && rightOperand == null) {
		throw new IllegalArgumentException("Round is not defined on zero operands");
	}

	// Round is a unary operator, so error if two operands exist
	if (leftOperand != null && rightOperand != null) {
		throw new IllegalArgumentException("Round is not a binary operator");
	}

	// Select the correct operand
	if (leftOperand == null) {
		return(getResult(rightOperand));
	}
	else { // rightOperand == null
		return(getResult(leftOperand));
	}
}

private Result getResult(Expression value) {

	// Evaluate a nested operation
	if (value instanceof Operation || value instanceof ExpressionReference) {
		value = value.evaluate();
	}

	// Compute the rounding of a single element
	if (value instanceof MathValue) {

		MathValue mvValue = (MathValue) value;
		return(doRound(mvValue));
	}

	// Compute the rounding of each element of a list
	else if (value instanceof ExpressionList) {

		ExpressionList xlValue = (ExpressionList) value;

		// get an iterator over the list
		Iterator valueIter = xlValue.iterator();
		ExpressionList result = new MathExpressionList();

		// Round each element of the list and create the result
		while (valueIter.hasNext()) {
			Expression valueElem = (Expression) valueIter.next();

			result.add(getResult(valueElem));
		}

		// return the result list
		return(result);
	}

	else {
		throw new IllegalArgumentException("Unrecognised math type to Round");
	}
}

/**
 * Round a math value
 */
private MathValue doRound(MathValue value) {

	Number result;

	// Round a scalar quantity
	if (value instanceof MathScalar) {

		MathScalar sValue = (MathScalar) value;
		result = roundSimple(sValue.realValue);
	}

	else {
		throw new IllegalArgumentException("Can't Round a vector or matrix.");
	}

	return new MathScalar(result);
}

/**
 * Compute the rounding of a Number
 */
private Number roundSimple(Number value) {

	Number result = null;

	// Don't need to change Integers
	if (value instanceof Integer) {
		result = new Integer(value.intValue());
	}

	// Round a non-integer
	if (value instanceof Double) {
		result = new Double(Math.round(value.floatValue()));
	}

	return(result);
}

}
