/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: Sort.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:41 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.formulae;


import java.util.Iterator;
import java.util.Collections;




/**
 * This operation returns a sorted list of the given elements
 * (secondary/nested lists are flattened)
 */
public class Sort extends Operation {

/**
 * Determine the Sorted order of the given set
 */
public Sort(Expression operand) {
	super(operand);
}

/**
 * Determine the Sorted order of a pair of sets
 */
public Sort(Expression operand1, Expression operand2) {
	super(operand1, operand2);
}

/**
 * Return the sorted result of the operands
 * @return The result of sorting the set
 */
public Result evaluate() {

	if (leftOperand == null && rightOperand == null) {
		throw new IllegalArgumentException("Sort is not defined on zero operands");
	}

	// Construct a single list of all the operands
	ExpressionList bigList = new MathExpressionList();

	// If the left operand is non-null, add it to the list
	if (leftOperand != null) {
		addElements(bigList, leftOperand);
	}

	// If the right operand is non-null, add it to the list
	if (rightOperand != null) {
		addElements(bigList, rightOperand);
	}

	// Sort a list of scalar (non-complex) MathValues
	Collections.sort(bigList, new RealComparator());

	// return the sorted list
	return(bigList);
}

/**
 * Add elements from the given operand to a list
 */
private void addElements(ExpressionList target, Expression operand) {

	// Store a math value
	if (operand instanceof MathValue) {
		target.add(operand);
	}

	// Store the result of an operation
	else if (operand instanceof Operation ||
			 operand instanceof ExpressionReference) {
		target.add(operand.evaluate());
	}

	// If this is a list, copy the elements
	else if (operand instanceof ExpressionList) {
		Iterator eli = ((ExpressionList) operand).iterator();

		// Add all elements of the list to the target
		while (eli.hasNext()) {
			Expression elem = (Expression) eli.next();
			addElements(target, elem);
		}
	}

	else {
		throw new IllegalArgumentException("Sort:  Unrecognised math type in ExpressionList");
	}
}

}
