/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: TableExpressionList.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:41 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.formulae;


import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;




import java.awt.Rectangle;

import com.klg.jclass.util.JCTableDataModel;

/**
 * A TableExpressionList represents an examination of a rectangular
 * range of cells stored in a JCTableDataModel
 */
public class TableExpressionList extends ExpressionList {

/** The table data from which the data is to be read */
protected JCTableDataModel		tableData;
/** The start row of this expression's range of cells in the data model */
protected Expression			startRow;
/** The end row of this expression's range of cells in the data model */
protected Expression			endRow;
/** The start column of this expression's range of cells in the data model */
protected Expression			startColumn;
/** The end column of this expression's range of cells in the data model */
protected Expression			endColumn;
/** A Rectangle describing the range of cells, assuming the expressions
    can be computed to a final result */
protected Rectangle				range;

/**
 * Create a table expression from a table data model and a range of cells
 * @param dataModel The JCTableDataModel whose cells are referenced by this object
 * @param range The (rectangular or linear) range of cells
 */
public TableExpressionList(JCTableDataModel tdm, Rectangle range) {

	// Store the tabledata
	tableData = tdm;

	// Check the range for validity and store it
	checkRange(range);
	this.range = range;

	// Store the extents of the range
	startRow = new MathScalar(range.y);
	endRow = new MathScalar(range.y + range.height - 1);
	startColumn = new MathScalar(range.x);
	endColumn = new MathScalar(range.x + range.width - 1);
}

/**
 * Create a table expression from a table data model and the row/column
 * locations of a range of cells
 * @param dataModel The JCTableDataModel whose cells are referenced by this object
 * @param firstRow the first row to be used in the range
 * @param lastRow the last row to be used in the range
 * @param firstColumn the first column to be used in the range
 * @param lastColumn the last column to be used in the range
 */
public TableExpressionList(JCTableDataModel tdm, Expression firstRow,
						   Expression lastRow, Expression firstColumn,
						   Expression lastColumn) {

	// Store the tabledata
	tableData = tdm;

	// Compute the range of cells defined by the Expressions
	range = computeRange(firstRow, firstColumn, lastRow, lastColumn);

	// Store the extents of the range (even if they're not defined yet)
	startRow = firstRow;
	endRow = lastRow;
	startColumn = firstColumn;
	endColumn = lastColumn;
}

/**
 * Return the range of the table data being used by this expression
 * @return A Rectangle describing the referenced cells of the table data
 */
public Rectangle getRange() {

	// if the range is not computed yet, compute it
	if (range == null) {
		return(computeRange(startRow, startColumn, endRow, endColumn));
	}

	return(new Rectangle(range));
}

/**
 * Set a range of cells to be accessed by this expression
 * @param newRange A range of cells in the table data
 */
public void setRange(Rectangle newRange) {

	// Check that the range is permissible
	checkRange(newRange);

	// set the range
	this.range = new Rectangle(newRange);

	// Store the extents of the range
	startRow = new MathScalar(range.y);
	endRow = new MathScalar(range.y + range.height - 1);
	startColumn = new MathScalar(range.x);
	endColumn = new MathScalar(range.x + range.width - 1);
}

/**
 * Get the first column of the range
 * @return The number of the first column in the range
 */
public Expression getStartColumn() {
	return(startColumn);
}

/**
 * Set the first column of the range
 * @param column The first column in the range
 */
public void setStartColumn(Expression column) {

	// Verify the range
	range = computeRange(startRow, column, endRow, endColumn);

	// ??? Should we clone this?
	startColumn = column;
}

/**
 * Get the last column of the range
 * @return The number of the last column in the range
 */
public Expression getEndColumn() {
	return(endColumn);
}

/**
 * Set the last column of the range
 * @param column The last column in the range
 */
public void setEndColumn(Expression column) {

	// Verify the range
	range = computeRange(startRow, startColumn, endRow, column);

	// ??? Should we clone this?
	endColumn = column;
}

/**
 * Get the first row of the range
 * @return The number of the first row in the range
 */
public Expression getStartRow() {
	return(startRow);
}

/**
 * Set the first row of the range
 * @param row The first row in the range
 */
public void setStartRow(Expression row) {

	// Verify the range
	range = computeRange(row, startColumn, endRow, endColumn);

	// ??? Should we clone this?
	startRow = row;
}

/**
 * Get the last row of the range
 * @return The number of the last row in the range
 */
public Expression getEndRow() {
	return(endRow);
}

/**
 * Set the last row of the range
 * @param row The last row in the range
 */
public void setEndRow(Expression row) {

	// Verify the range
	range = computeRange(startRow, startColumn, row, endColumn);

	// ??? Should we clone this?
	endRow = row;
}

/**
 * Set a table data object
 */
/*
protected void setTableData(JCTableDataModel tdm) {

	// Store the tabledata
	tableData = tdm;

	// Check the range for validity against the new data
	if (range != null) {
		checkRange(range);
	}
}
*/

/**
 * Verify a range as being within the scope of the given table data
 */
private void checkRange(Rectangle r) {

	// Check for start < 0 or extent < 0 or either beyond the scope of the data
	if (r.x < 0 || r.width < 0 || r.x + r.width > tableData.getNumColumns()) {
		throw new IllegalArgumentException("Column selection is out of range");
	}

	if (r.y < 0 || r.height < 0 || r.y + r.height > tableData.getNumRows()) {
		throw new IllegalArgumentException("Row selection is out of range");
	}
}

/**
 * Compute the range from the Expressions, if necessary and possible
 * @return The range of cells delimited in the DataModel
 */
private Rectangle computeRange(Expression firstRow, Expression firstColumn,
							   Expression lastRow, Expression lastColumn) {

	// Attempt to evaluate the Expressions to get row and column numbers
	try {

		// Get the start and end row and columns from the Expressions
		int start_column = ((MathValue) firstColumn.evaluate()).numberValue().intValue();
		int end_column = ((MathValue) lastColumn.evaluate()).numberValue().intValue();
		int start_row = ((MathValue) firstRow.evaluate()).numberValue().intValue();
		int end_row = ((MathValue) lastRow.evaluate()).numberValue().intValue();

		// Verify the range and store it if it's valid
		Rectangle temp_range = new Rectangle(start_column, start_row,
											 end_column - start_column + 1,
											 end_row - start_row + 1);
		checkRange(temp_range);
		return(temp_range);
	}

	// We catch exceptions thrown for undefined variables
	catch (ArithmeticException e) {
		return(null);
	}
}

/**
 * Make sure the range has been evaluated; throw an exception if it can
 * not yet be evaluted.
 */
private void updateRange() {

	// If the range has been evaluated, we don't need to do anything
	if (range != null) {
		return;
	}

	// Try to evaluate the range
	range = computeRange(startRow, startColumn, endRow, endColumn);

	// If the range failed to evaluate, this will throw an appropriate exception
	if (range == null) {
		Result start_row = startRow.evaluate();
		Result start_column = startColumn.evaluate();
		Result end_row = endRow.evaluate();
		Result end_column = endColumn.evaluate();
	}
}
/**
 * Return an Expression which is a clone of this TableExpressionList
 * (NOTE:  The table values are not cloned and parametrized indices are
 *         computed)
 */
public Object clone() {

	TableExpressionList newList;

	try {
		newList = (TableExpressionList) super.clone();
	}
	catch (CloneNotSupportedException e) {
		return(null);
	}

	if (startRow != null) {
		try {
			newList.startRow = (Expression) startRow.evaluate().clone();
		}
		catch (CloneNotSupportedException e) {
			newList.startRow = new MathScalar(0);
		}
	}
	if (startColumn != null) {
		try {
			newList.startColumn = (Expression) startColumn.evaluate().clone();
		}
		catch (CloneNotSupportedException e) {
			newList.startColumn = new MathScalar(0);
		}
	}
	if (endRow != null) {
		try {
			newList.endRow = (Expression) endRow.evaluate().clone();
		}
		catch (CloneNotSupportedException e) {
			newList.endRow = new MathScalar(0);
		}
	}
	if (endColumn != null) {
		try {
			newList.endColumn = (Expression) endColumn.evaluate().clone();
		}
		catch (CloneNotSupportedException e) {
			newList.endColumn = new MathScalar(0);
		}
	}
	if (range != null) {
		newList.range = new Rectangle(range);
	}

	return(newList);
}

// NOTE:  The remainder of this object implements the methods defined by
//        the List interface

/**
 * Determine if the List contain the specified element
 * @param obj The element to be searched for in this List
 * @return True if the List contains the given object
 */
public boolean contains(Object obj) {

	// ??? How do we do this?  Do we check individual table cell
	//     entries?  Sublists?  The table itself?
	return(false);
}

/**
 * Compare the given object and this List, given that equality implies
 * the Lists contain the same entries in the same order.
 * @param obj the object whose equality with this List is tested
 * @return True if the given object satisfies "equality" with this List
 */
public boolean equals(Object obj) {

	// Can only be equal to another TableExpressionList
	if (!(obj instanceof TableExpressionList)) {
		return(false);
	}

	TableExpressionList tel = (TableExpressionList) obj;

	// TableData objects must be equivalent (how to compare otherwise?)
	if (tel.tableData != tableData) {
		return(false);
	}

	// if range is defined (evaluated) then it must be equal
	if (range != null) {
		return(range.equals(tel.range));
	}

	// Here we have a case where the range contains undefined variables
	// The only way we have of testing equality is if the expressions are
	// identical, since we can't algebraically compare them.
	return(startColumn == tel.startColumn && endColumn == tel.endColumn &&
		   startRow == tel.startRow && endRow == tel.endRow);
}

/**
 * Retrieve the element at a given position in this List
 * @param index The desired element's position
 * @return The element found at the given position
 */
public Object get(int index) {

	// Make sure the range can be and has been evaluated
	updateRange();

	// NOTE If we have a 2-d range we return another TableExpressionList
	//      which iterates over the selected row

	// Make sure the index is in range
	if ((index < 0) ||
	    (range.height == 1 && index >= range.width) ||
		(range.width == 1 && index >= range.height)) {

		throw new ArrayIndexOutOfBoundsException("Index is outside the table Expression's range");
	}

	// Return the exact element if we have a one-dimensional list
	if (range.height == 1) {
		Object entry = tableData.getTableDataItem(range.y, range.x + index);
		return(encapsulateValue(entry));
	}
	else if (range.width == 1) {
		Object entry = tableData.getTableDataItem(range.y + index, range.x);
		return(encapsulateValue(entry));
	}

	// otherwise (2-d) we return another TableExpressionList
	else {
		return(new TableExpressionList(tableData,
									   new Rectangle(range.x, range.y + index,
									   				 range.width, 1)));
	}
}

/**
 * Return a hash code derived from this List
 * @return the hash code of this List
 */
public int hashCode() {
	// use Object's hashcode function
	return(super.hashCode());
}

/**
 * Find the position (first occurence) of a given object in the List.
 * -1 is returned if the element is not found in the List.
 * @param obj The object to be found in the List
 * @return The location of the object in the List (or -1)
 */
public int indexOf(Object obj) {
	// ??? See contains() for an examination of this problem
	return(-1);
}

/**
 * Get an iterator over the elements in this List
 * @return an iterator object which enumerates the elements in this List
 */
public Iterator iterator() {
	return((Iterator) listIterator(0));
}

/**
 * Find the position of the last occurence of a given object in the List.
 * -1 is returned if the element is not found in the List.
 * @param obj The object to be found in the List
 * @return The last location the object is found in the List (or -1)
 */
public int lastIndexOf(Object o) {
	// ??? See contains() for an examination of this problem
	return(-1);
}

/**
 * Get a ListIterator over the elements of the List beginning at the
 * the given position in the list
 * @param index The position at which to initialise the ListIterator
 * @return A ListIterator over this List
 */
public ListIterator listIterator(int index) {

	// update the range and create an iterator
	updateRange();
	return(new TableExpressionListIterator(this, index));
}

/**
 * Return the number of elements in this List
 * @return the cardinality of this List
 */
public int size() {

	// update the range
	updateRange();

	// Return the number of rows in a vertical or 2-d data set
	if (range.height > 1) {
		return(range.height);
	}

	// otherwise return the number of columns (in one row)
	else {
		return(range.width);
	}
}

/**
 * Get a List which is a subsequence of this List (the ordering is the same)
 * @param startIndex The first element of the sublist
 * @param endIndex The element after the last element of the sublist
 * @return A List which contains the elements of the specified subset
 */
public List subList(int startIndex, int endIndex) {

	// Error-checking
	if (startIndex < 0 || endIndex < startIndex) {
		throw new ArrayIndexOutOfBoundsException("start must be non-negative and range must be increasing");
	}

	// update the range
	updateRange();

	// Select from the range of rows if there are more than one
	if (range.height > 1) {

		// Check that the index is in range.
		if (endIndex >= range.height) {
			throw new ArrayIndexOutOfBoundsException("end index past the end of the List");
		}

		// Return the selected range of rows
		return(new TableExpressionList(tableData,
									   new Rectangle(range.x,
									   				 range.y + startIndex,
													 range.width,
													 endIndex - startIndex)));
	}

	// otherwise select from the range of columns (in one row)
	else {

		// Check that the index is in range.
		if (endIndex >= range.width) {
			throw new ArrayIndexOutOfBoundsException("end index past the end of the List");
		}

		// Return the selected range of rows
		return(new TableExpressionList(tableData,
									   new Rectangle(range.x + startIndex,
									   				 range.y,
													 endIndex - startIndex,
													 range.height)));

	}
}

/**
 * Fill a given array with a view of all the elements in this List
 * NOTE:  The Iterator's ordering is used
 * @param inArray The array in which to store the elements
 * @return an Array of all the elements in this List
 */
public Object[] toArray(Object inArray[]) {

	// update the range
	updateRange();

	// NOTE If we have a 2-d set of data we have to return an array of sublists

	// Copy a vertical list of cells to the array
	if (range.width == 1) {
		for (int i = 0; i < range.height; i++) {
			inArray[i] = tableData.getTableDataItem(range.y + i, range.x);
		}
	}

	// Copy a horizontal list of cells to the array
	else if (range.height == 1) {
		for (int i = 0; i < range.width; i++) {
			inArray[i] = tableData.getTableDataItem(range.y, range.x + i);
		}
	}

	// Copy a set of sublists to the array
	else {
		for (int i = 0; i < range.height; i++) {
			inArray[i] = new TableExpressionList(tableData,
												 new Rectangle(range.x,
											   				   range.y + i,
															   range.width,
															   1));
		}
	}

	return(inArray);
}

/**
 * Create a copy of this TableExpressionList (not necessarily a clone, since
 * we may choose to use hard co-ordinates rather than parametric ones)
 * @param fixLocation Indicates if the row and column positions are to be
 *                    evaluated before being set
 * @return A new TableExpressionList which is a copy of this one (it will
 *         point to the same table cell this one currently does)
 */
protected Expression copy(boolean fixLocation) {

	// If we are fixing the location, evaluate the co-ordinates
	if (fixLocation) {
		range = computeRange(startRow, startColumn, endRow, endColumn);
	}

	// if we're not fixing the range or there is no valid current range
	if (!fixLocation || range == null) {
		// create a copy using the same expressions
		return(new TableExpressionList(tableData, startRow, endRow,
									   startColumn, endColumn));
	}

	// otherwise use the computed range
	else {
		return(new TableExpressionList(tableData, range));
	}
}


/**
 * Take an item of data retrieved from the table, and if it is a numeric
 * data type, wrap it in a MathValue so it can be manipulated in other
 * Expressions.
 * @param obj The object found in the table data
 * @return An object (possibly the same) which encapsulates any Numbers.
 */
private Object encapsulateValue(Object obj) {

	// All Expressions can be returned intact
	if (obj instanceof Expression) {
		return(obj);
	}

	// Numbers should be wrapped into a MathValue
	else if (obj instanceof Number) {
		return(new MathScalar((Number) obj));
	}
	else if (obj instanceof Number[]) {
		return(new MathVector((Number[]) obj));
	}
	else if (obj instanceof Number[][]) {
		return(new MathMatrix((Number[][]) obj));
	}

	// Try to convert strings to numbers
	else if (obj instanceof String) {
		String s = (String) obj;
		if (s.indexOf('.') >= -1) {
			return(new MathScalar(Double.valueOf(s)));
		}
		else {
			return(new MathScalar(Integer.valueOf(s)));
		}
	}

	// No other types can be handled for now
	else {
		throw new ClassCastException("Cannot return \"" + obj + "\" as a MathValue");
	}
}

}
