/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: TableExpressionListIterator.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:41 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.formulae;


import java.util.Iterator;
import java.util.ListIterator;




import java.util.NoSuchElementException;

/**
 * TableExpressionListIterator implements the ListIterator interface to
 * allow iteration over the elements of a TableExpressionList.
 */
public class TableExpressionListIterator implements Iterator, ListIterator {

/** The ExpressionList we're iterating over */
protected TableExpressionList	tableList;
/** The current position in the list */
protected int					currentPosition = 0;
/** The last position of the list */
protected int					endPosition = 0;
/** The position in the list of the last element read by next() or previous() */
protected int					lastAccess = -1;

/**
 * Create an iterator over the given TableExpressionList
 * @param tel The TableExpressionList to iterate over
 */
public TableExpressionListIterator(TableExpressionList tel) {

	tableList = tel;
	endPosition = tel.size();
}

/**
 * Create an iterator over the given TableExpressionList, starting
 * at the given initial position.
 * @param tel The TableExpressionList to iterate over
 * @param index The initial position in the list
 */
public TableExpressionListIterator(TableExpressionList tel, int firstIndex) {

	// Verify that the index is in range
	if (firstIndex < 0 || firstIndex >= tel.size()) {
		throw new IndexOutOfBoundsException("Index not in range of the List size");
	}

	tableList = tel;
	endPosition = tel.size();
	currentPosition = firstIndex;
}

/**
 * Determines if there is another element in the forward direction
 * @return True if there is another (following) element in the List
 */
public boolean hasNext() {

	return(currentPosition < endPosition);
}

/**
 * Determines if there is previous element in the List (backward)
 * @return True if there is a preceding element in the List
 */
public boolean hasPrevious() {

	return(currentPosition > 0);
}

/**
 * Get the next element in the List (forward direction)
 * @return The next element in the List
 */
public Object next() {

	// Verify that there is another element available
	if (currentPosition >= endPosition) {
		throw new NoSuchElementException("No more elements in List.");
	}

	// Get the next element and advance the pointer
	Object element = tableList.get(currentPosition);
	lastAccess = currentPosition;
	currentPosition++;

	return(element);
}

/**
 * Get the index of the element that would be returned if next() were called
 * @return The index of the next element
 */
public int nextIndex() {
	return(currentPosition);
}

/**
 * Get the previous element in the List (backward direction)
 * @return The previous element in the List
 */
public Object previous() {

	// Verify that there is another element available
	if (currentPosition <= 0) {
		throw new NoSuchElementException("No previous elements in List.");
	}

	// Adjust the pointer and get the next element
	currentPosition--;
	Object element = tableList.get(currentPosition);
	lastAccess = currentPosition;

	return(element);
}

/**
 * Get the index of the element that would be returned if previous() were called
 * @return The index of the previous element
 */
public int previousIndex() {
	return(currentPosition - 1);
}

/**
 * Remove the last accessed element of the List
 * NOTE:  remove() is not supported on TableExpressionLists
 */
public void remove() {
	throw new UnsupportedOperationException("TableExpressionList does not support remove()");
}

/**
 * Replace the last accessed element of the List
 * NOTE:  set() is not supported on TableExpressionLists
 * @param obj The object to replace the last element with
 */
public void set(Object obj) {
	throw new UnsupportedOperationException("TableExpressionList does not support set()");
}

/**
 * Insert a new element into the list at the current position
 * NOTE:  add() is not supported on TableExpressionLists
 * @param obj The object to insert into the List
 */
public void add(Object obj) {
	throw new UnsupportedOperationException("TableExpressionList does not support add()");
}

}
