/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: Trunc.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:41 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.formulae;


import java.util.Iterator;




/**
 * This is a concrete implementation for the trunc (truncation)
 * operation.  Trunc is also known as "integer part," the result
 * is equivalent to "round to zero," since it rounds to the nearest
 * integer closer to zero.
 * This is a unary operator, *???but can also work on lists???*.
 */
public class Trunc extends Operation {

/**
 * Define a truncation operation on the given Expression
 */
public Trunc(Expression operand) {
	super(operand);
}

/**
 * Define a truncation operation on the given Number
 */
public Trunc(Number operand) {
	super(operand);
}

/**
 * Return the integer part of a given number (round to zero)
 * @return The integer part of the given operand
 */
public Result evaluate() {

	// Can't do anything with no data
	if (leftOperand == null && rightOperand == null) {
		throw new IllegalArgumentException("Trunc is not defined on zero operands");
	}

	// Trunc is a unary operator, so error if two operands exist
	if (leftOperand != null && rightOperand != null) {
		throw new IllegalArgumentException("Trunc is not a binary operator");
	}

	// Select the correct operand
	if (leftOperand == null) {
		return(getResult(rightOperand));
	}
	else { // rightOperand == null
		return(getResult(leftOperand));
	}
}

private Result getResult(Expression value) {

	// Evaluate a nested operation
	if (value instanceof Operation || value instanceof ExpressionReference) {
		value = value.evaluate();
	}

	// Truncate the value of a single element
	if (value instanceof MathValue) {

		MathValue mvValue = (MathValue) value;
		return(doTrunc(mvValue));
	}

	// Truncate the value of each element of a list
	else if (value instanceof ExpressionList) {

		ExpressionList xlValue = (ExpressionList) value;

		// get an iterator over the list
		Iterator valueIter = xlValue.iterator();
		ExpressionList result = new MathExpressionList();

		// Truncate each element of the list and create the result
		while (valueIter.hasNext()) {
			Expression valueElem = (Expression) valueIter.next();

			result.add(getResult(valueElem));
		}

		// return the result list
		return(result);
	}

	else {
		throw new IllegalArgumentException("Unrecognised math type to Trunc");
	}
}

/**
 * Truncate a math value
 */
private MathValue doTrunc(MathValue value) {

	Number result;

	// Truncate a scalar quantity
	if (value instanceof MathScalar) {

		MathScalar sValue = (MathScalar) value;

		result = truncSimple(sValue.realValue);
	}

	else {
		throw new IllegalArgumentException("Can't perform Trunc on a vector or matrix.");
	}

	return new MathScalar(result);
}

/**
 * Compute trunc of a Number
 */
private Number truncSimple(Number value) {

	Number result = null;

	// Don't need to change Integers
	if (value instanceof Integer) {
		result = new Integer(value.intValue());
	}

	// Truncate a non-integer
	if (value instanceof Double) {

		double d = value.doubleValue();

		// Round number > 0 down, and round < 0 up.
		if (d >= 0) {
			result = new Double(Math.floor(d));
		}
		else {
			result = new Double(Math.ceil(d));
		}
	}

	return(result);
}

}
