/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: DefineShape.java,v $ $Revision: 1.2 $
// $Date: 2006-03-17 21:13:41 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.graphics;

import java.util.List;

/**
 * Implemented by classes supporting the definition of shapes during
 * drawing, usually a subclass of java.awt.Graphics.  A shape is a
 * series of draw commands that is associated with a shape id and
 * a reference key from the object doing the drawing, such as
 * a data index.
 */
public interface DefineShape
{

/** 
 * Begin a shape definition.
 */
public void beginShape();

/**
 * Begin a shape definition, associating the generated shape with
 * the given reference key.  This reference key overrides the globally
 * set key.
 */
public void beginShape(Object key);

/**
 * Globally mark all future shape definitions to be associated with
 * the given reference key.
 */
public void markShape(Object key);

/**
 * Clear the global reference object to be associated with
 * future created shapes.
 */
public void unmarkShape();

/**
 * Set the shape placement property.  If true, a generated shape will
 * automatically be made visible.  If false, it will be defined but not
 * yet made visible.
 */
public void setShapePlacement(boolean place);

/**
 * Record attempted object placements (although the objects
 * may not actually be placed, if placement is overridden by
 * setShapePlacement()).  A value of true will begin the recording.
 * A list of recorded shape ids can be retrieved at any time
 * by calling getRecordedShapeIds().  A value of false will turn
 * off the recording and clear out the recorded lsit.
 * @param record true if turning recording on, false if turning
 *        recording off.
 */
public void recordPlacementAttempts(boolean record);

/**
 * Return list of shape ids corresponding to attempted object
 * placements recorded when recordPlacementAttempts() is called
 * with a value of true.
 * @return a List of Integer objects containing shape ids for
 *         all objects that were placed or intended for placement
 *         since recording was turned on with recordPlacementAttempts(true).
 */
public List getRecordedShapeIds();

/**
 * End a shape definition using the previously set value of the
 * shape placement property.
 */
public int endShape();

/**
 * End a shape definition using the given shape placement value.  If true,
 * a generated shape will automatically be made visible.  If false, it
 * will be defined but not yet made visible.
 */
public int endShape(boolean place);

/**
 * Get the id number of the last shape generated.
 */
public int getLastShapeId();

/**
 * Get the list of reference keys in drawing order.
 */
public List getKeysInDrawingOrder();

/**
 * Look up the list of shapes associated with this reference object.
 */
public List lookupShapes(Object key);
	
}
