/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: Graphics2DBase.java,v $ $Revision: 1.2 $ 
// $Date: 2006-03-17 21:13:41 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.graphics;



import java.awt.*;
import java.awt.RenderingHints.Key;
import java.awt.font.FontRenderContext;
import java.awt.font.GlyphVector;
import java.awt.font.TextAttribute;
import java.awt.geom.*;
import java.awt.image.BufferedImage;
import java.awt.image.BufferedImageOp;
import java.awt.image.ImageObserver;
import java.awt.image.RenderedImage;
import java.awt.image.renderable.RenderableImage;
import java.io.OutputStream;
import java.text.AttributedCharacterIterator;
import java.text.CharacterIterator;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

/**
 * Overrides Graphics2D to provide the Java interface
 * for encoding implementations.
 */

public abstract class Graphics2DBase extends Graphics2D implements Cloneable {

protected OutputStream 	os; 			// where we write the output
protected boolean finished = false;     // if true, will no longer output


// These define the output window for the graphics
protected Shape				clip		= new Rectangle2D.Double();

protected boolean			clipSet		= false; // is there a non-zero clip?
protected AffineTransform	transform	= new AffineTransform(); //default
protected java.awt.Point	translation	= new java.awt.Point(0, 0);

// These attributes store state for output attributes
protected boolean		underline		= false;
protected boolean		subscript		= false;		
protected Font			currentFont;
protected Color			currentColor;
protected Stroke		currentStroke 	= new BasicStroke();
protected boolean 		dashMode		= false; // draw dashed lines?
protected int			windingRule		= GeneralPath.WIND_NON_ZERO;

// These flags record what state needs to be updated in the printer
protected boolean		fontChanged		= true; // has the font changed?
protected boolean		colorChanged	= true; // has the color changed?
protected boolean		strokeChanged	= true; // has linewidth changed?

// Used for formatting/structuring the output
protected Hashtable		loadedFonts		= new Hashtable();
protected Buffer		out				= new Buffer();
protected int			maxDecimalDigits= 4; // see discussion in pdf manual
protected DecimalFormat decimalFormat	= new DecimalFormat();

// These attributes are used to track the current graphics
protected boolean		isFrameGraphics = false;
protected Graphics		parent			= null;
protected static Map	contextMap      = new Hashtable();
protected boolean		isLive			= true;

/**
 *
 * Constructor
 *
 */
public Graphics2DBase(OutputStream os)
{
    this.os = os;
    // set the default color
    setColor(new Color(0, 0, 0));

    // set up the formatting of decimal numbers to
    // use "." as decimal and always show it. Also
    // don't use "," between thousands
    DecimalFormatSymbols dfs = decimalFormat.getDecimalFormatSymbols();
    dfs.setDecimalSeparator('.');
    decimalFormat.setDecimalFormatSymbols(dfs);

    decimalFormat.setDecimalSeparatorAlwaysShown(true);
    decimalFormat.setGroupingUsed(false);
}

/**
 * Overrides g2d method.
 */
public void draw3DRect(int x, int y, int width, int height,
                       boolean raised)
{
    Paint p = getPaint();

    Color c = getColor();
    Color brighter = c.brighter();
    Color darker = c.darker();

    setColor(raised ? brighter : darker);
    fillRect(x, y, 1, height + 1);
    fillRect(x + 1, y, width - 1, 1);
    setColor(raised ? darker : brighter);
    fillRect(x + 1, y + height, width, 1);
    fillRect(x + width, y, 1, height);

    setPaint(p);
}

/**
 * Overrides g2d method.
 */
public void fill3DRect(int x, int y, int width, int height,
                       boolean raised)
{
    Paint p = getPaint();

    Color c = getColor();
    Color brighter = c.brighter();
    Color darker = c.darker();

    if (!raised) {
        setColor(darker);
    }
    else if (p != c) {
        setColor(c);
    }
    fillRect(x + 1, y + 1, width - 2, height - 2);
    setColor(raised ? brighter : darker);
    fillRect(x, y, 1, height);
    fillRect(x + 1, y, width - 2, 1);
    setColor(raised ? darker : brighter);
    fillRect(x + 1, y + height - 1, width - 1, 1);
    fillRect(x + width - 1, y, 1, height - 1);

    setPaint(p);
}

/**
 * Overrides g2d method.
 */
public void draw(Shape s)
{
    if (s instanceof GeneralPath || s instanceof Line2D) {
        // NOTE:  We use an identity transform since we've already
        //        transformed our context.
        PathIterator path = s.getPathIterator(new AffineTransform());
        drawLine(path);
    }
    else if (s instanceof Rectangle2D) {
        drawRect((Rectangle2D)s);
    }
    else if (s instanceof RoundRectangle2D) {
        drawRoundRect((RoundRectangle2D)s);
    }
    else if (s instanceof Polygon) {
        drawPolygon((Polygon)s);
    }
    else if (s instanceof Arc2D) {
        drawArc((Arc2D)s);
    }
    else if (s instanceof Ellipse2D) {
        drawOval((Ellipse2D)s);
    }
}

/**
 * Overrides g2d method.
 */
public void fill(Shape s)
{
    if (s instanceof GeneralPath || s instanceof Line2D) {
        fillShape(s);
    }
    else if (s instanceof Rectangle2D) {
        fillRect((Rectangle2D)s);
    }
    else if (s instanceof RoundRectangle2D) {
        fillRoundRect((RoundRectangle2D)s);
    }
    else if (s instanceof Polygon) {
        fillPolygon((Polygon)s);
    }
    else if (s instanceof Arc2D) {
        fillArc((Arc2D)s);
    }
    else if (s instanceof Ellipse2D) {
        fillOval((Ellipse2D)s);
    }
}

/**
 * Default implementation of a fillShape method.  All curves are
 * turned into lines and the provided shape is reduced to a polygon.
 * Ideally, this should be overridden and implemented correctly in
 * any subclasses.
 */
public void fillShape(Shape s)
{
    // NOTE:  We use an identity transform since we've already
    //        transformed our context.
    PathIterator path = s.getPathIterator(new AffineTransform());

    double lastMoveX = Double.MAX_VALUE, lastMoveY = Double.MAX_VALUE;

    // make a polygon out of the path
    Polygon polygon = new Polygon();
    while (!path.isDone()) {
        float pts[] = new float[6];
        switch (path.currentSegment(pts)) {
        case PathIterator.SEG_MOVETO:
            polygon.addPoint((int)pts[0], (int)pts[1]);
            lastMoveX = pts[0];
            lastMoveY = pts[1];
            break;
        case PathIterator.SEG_LINETO:
            polygon.addPoint((int)pts[0], (int)pts[1]);
            break;
        case PathIterator.SEG_CUBICTO:
            polygon.addPoint((int)pts[4], (int)pts[5]);
            break;
        case PathIterator.SEG_QUADTO:
            polygon.addPoint((int)pts[2], (int)pts[3]);
            break;
        case PathIterator.SEG_CLOSE:
            if (lastMoveX != Double.MAX_VALUE &&
                lastMoveY != Double.MAX_VALUE) {
                polygon.addPoint((int)lastMoveX, (int)lastMoveY);
            }
            break;
        default:
            break;
        }
        path.next();
    }

    // set the filling rule
    windingRule = ((GeneralPath)s).getWindingRule();

    // fill it
    fillPolygon(polygon);
}

/**
 * Overrides g2d method.
 */
public boolean drawImage(Image img, AffineTransform xform, ImageObserver obs)
{
    // not implemented
    
    return true;
}

/**
 * Overrides g2d method.
 */
public void drawImage(BufferedImage img, BufferedImageOp op, int x, int y)
{
    // not implemented
    
}

/**
 * Overrides g2d method.
 */
public void drawRenderedImage(RenderedImage img, AffineTransform xform)
{
    // not implemented
    
}

/**
 * Overrides g2d method.
 */
public void drawRenderableImage(RenderableImage img, AffineTransform xform)
{
    // not implemented
    
}

/**
 * Overrides g2d method.
 */
public void drawString(AttributedCharacterIterator iter, int x, int y)
{
    drawString(iter, (float)x, (float)y);
}

/**
 * Overrides g2d method.
 */
public void drawString(AttributedCharacterIterator iter, float x, float y)
{
    doString(iter, x, y);
}

/**
 * Overrides g2d method.
 */
public void drawGlyphVector(GlyphVector g, float x, float y)
{
    fill(g.getOutline(x, y));
}

/**
 * Overrides g2d method.
 */
public boolean hit(Rectangle rect, Shape s, boolean onStroke)
{
    // not implemented
    
    return true;
}

/**
 * Overrides g2d method.
 */
public GraphicsConfiguration getDeviceConfiguration()
{
    // not implemented
    
    return null;
}

/**
 * Overrides g2d method.
 */
public void setComposite(Composite comp)
{
    // not implemented
    
}

/**
 * Overrides g2d method.
 */
public void setPaint(Paint paint)
{
    // only implemented for Color
    if (paint instanceof Color) {
        setColor((Color)paint);
        // we don't support GradientPaint at the moment,
        // but we can at least behavive nicely until we do
    }
    else if (paint instanceof GradientPaint) {
        setColor(((GradientPaint)paint).getColor1());
    }
}

/**
 * Overrides g2d method.
 */
public void setStroke(Stroke stroke)
{
    // set the stroke and indicate refresh required
    if (!this.currentStroke.equals(stroke)) {
        this.currentStroke = stroke;
        strokeChanged = true;
    }
}

/**
 * Overrides g2d method.
 */
public void setRenderingHint(Key hintKey, Object hintValue)
{
    // not implemented
    
}

/**
 * Overrides g2d method.
 */
public Object getRenderingHint(Key hintKey)
{
    // not implemented
    
    return null;
}

/**
 * Overrides g2d method.
 */
public void setRenderingHints(Map hints)
{
    // not implemented
    
}

/**
 * Overrides g2d method.
 */
public void addRenderingHints(Map hints)
{
    // not implemented
    
}

/**
 * Overrides g2d method.
 */
public RenderingHints getRenderingHints()
{
    // not implemented
    
    return null;
}

/**
 * Overrides g2d method.
 */
public void translate(int x, int y)
{
    // not implemented
    
}

/**
 * Overrides g2d method.
 */
public void translate(double tx, double ty)
{
    // not implemented
    
}

/**
 * Overrides g2d method.
 */
public void rotate(double theta)
{
    // not implemented
    
}

/**
 * Overrides g2d method.
 */
public void rotate(double theta, double x, double y)
{
    // not implemented
    
}

/**
 * Overrides g2d method.
 */
public void scale(double sx, double sy)
{
    // not implemented
    
}

/**
 * Overrides g2d method.
 */
public void shear(double shx, double shy)
{
    // not implemented
    
}

/**
 * Overrides g2d method.
 */
public void transform(AffineTransform Tx)
{
    // not implemented
    
}

/**
 * Overrides g2d method.
 */
public void setTransform(AffineTransform transform)
{
    this.transform = transform;
}

/**
 * Overrides g2d method.
 */
public AffineTransform getTransform()
{
    return transform;
}

/**
 * Get the cumulative transform from all frame graphics in a single
 * hierarchy.  This gives us the transform relative to the page.
 * @return AffineTranform object holding transform that takes
 *         an object from page level to the position of this graphics object.
 */
public AffineTransform getCumulativeTransform()
{
    AffineTransform cumulativeTransform = null;
    Graphics2DBase currentGraphics = this;
    while (currentGraphics != null) {
        // only deal with frame graphics classes since non-frame graphics
        // classes already accumulate the transform
        if (currentGraphics.isFrameGraphics) {
            if (cumulativeTransform == null) {
                cumulativeTransform =
                    (AffineTransform)currentGraphics.getTransform().clone();
            }
            else {
                cumulativeTransform.preConcatenate(
                    currentGraphics.getTransform());
            }
        }
        currentGraphics = (Graphics2DBase)currentGraphics.parent;
    }
    return cumulativeTransform;
}

/**
 * Overrides g2d method.
 */
public Paint getPaint()
{
    // only implemented for Color
    return (getColor());
}

/**
 * Overrides g2d method.
 */
public Composite getComposite()
{
    // not implemented
    
    return null;
}

/**
 * Overrides g2d method.
 */
public void setBackground(Color color)
{
    // not implemented
    
}

/**
 * Overrides g2d method.
 */
public Color getBackground()
{
    // not implemented
    
    return null;
}

/**
 * Overrides g2d method.
 */
public Stroke getStroke()
{
    return currentStroke;
}

/**
 * Overrides g2d method.
 */
public void clip(Shape s)
{
    // not implemented
    
}

/**
 * Overrides g2d method.
 */
public FontRenderContext getFontRenderContext()
{
    // NOTE:  We use an identity transform since we've already
    //        transformed our context.
    return new FontRenderContext(new AffineTransform(), false, false);
}

//////////////////// Some Graphics (not 2D) methods /////////

/**
 * Overrides Graphics method
 */
public void drawRect(int x, int y, int width, int height)
{
    draw(new Rectangle2D.Double((double)x, (double)y,
                                (double)width, (double)height));
}

/**
 * Overrides Graphics method
 */
public void fillRect(int x, int y, int width, int height)
{
    fill(new Rectangle2D.Double((double)x, (double)y,
                                (double)width, (double)height));
}

/**
 * Overrides Graphics method
 */
public void fillRoundRect(int x, int y, int width, int height,
                          int arcWidth, int arcHeight)
{

    fill(new RoundRectangle2D.Double((double)x, (double)y,
                                     (double)width, (double)height,
                                     (double)arcWidth, (double)arcHeight));
}

/**
 * Overrides Graphics method
 */
public void drawRoundRect(int x, int y, int width, int height,
                          int arcWidth, int arcHeight)
{

    draw(new RoundRectangle2D.Double((double)x, (double)y,
                                     (double)width, (double)height,
                                     (double)arcWidth, (double)arcHeight));
}

/**
 * Overrides Graphics method
 */
public void drawArc(int x, int y, int width, int height,
                    int arcStart, int arcSweep)
{

    draw(new Arc2D.Double(x, y, width, height, arcStart, arcSweep, Arc2D.PIE));
}

/**
 * Overrides Graphics method
 */
public void fillArc(int x, int y, int width, int height,
                    int arcStart, int arcSweep)
{

    fill(new Arc2D.Double(x, y, width, height, arcStart, arcSweep, Arc2D.PIE));
}

/**
 * Overrides Graphics method
 */
public Graphics create()
{

    Graphics2DBase gc2;

    try {
        gc2 = (Graphics2DBase)this.clone();
    }
    catch (CloneNotSupportedException e) {
        return (null);
    }

    // replace the transformation if this is a frame, copy it otherwise
    if (this.isFrameGraphics) {
        gc2.transform = new AffineTransform();
    }
    else {
        gc2.transform = new AffineTransform(this.transform);
    }

    // Copy the clip window
    if (this.clip instanceof GeneralPath) {
        gc2.clip = (Shape)((GeneralPath)this.clip).clone();
    }
    else if (this.clip instanceof Rectangle2D) {
        gc2.clip = (Shape)((Rectangle2D)this.clip).clone();
    }
    else {
        gc2.clip = this.clip;
    }

    // We need to output font, color, etc. information for the new graphics
    gc2.fontChanged = true;
    gc2.colorChanged = true;
    gc2.strokeChanged = true;

    // Set a parent for the new graphics
    gc2.parent = this;

    // This is not automatically a frame
    gc2.isFrameGraphics = false;

    return gc2;
}

/**
 * Overrides Graphics method
 */
public Graphics create(int x, int y, int width, int height)
{

    Graphics gc2 = create();
    gc2.translate(x, y);
    gc2.clipRect(0, 0, width, height);

    return gc2;
}

/**
 * Overrides Graphics method
 */
public Color getColor()
{
    return currentColor;
}

/**
 * Overrides Graphics method
 */
public void setColor(Color c)
{

    if (c == null || !c.equals(this.currentColor)) {
        this.currentColor = c;
        colorChanged = true;
    }
}

/**
 * Overrides Graphics method
 */
public void setPaintMode()
{

    // not implemented
    
}

/**
 * Overrides Graphics method
 */
public void setXORMode(Color c1)
{

    // not implemented
    
}

/**
 * Overrides Graphics method
 */
public Font getFont()
{

    if (currentFont == null) {
        // use the default
        currentFont = Font.decode("TimesRoman-12");

        // and set it
        setFont(currentFont);
    }
    return currentFont;
}

/**
 * Overrides Graphics method
 */
public FontMetrics getFontMetrics()
{
    return getFontMetrics(getFont());
}

/**
 * Overrides Graphics method
 */
public Rectangle getClipBounds()
{
    return clip.getBounds();
}

/**
 * Overrides Graphics method
 */
public void clipRect(int x, int y, int width, int height)
{
    // not implemented
    
}

/**
 * Overrides Graphics method
 */
public Shape getClip()
{
    return clip;
}

/**
 * Overrides Graphics method
 */
public void setClip(int x, int y, int width, int height)
{

    setClip(new Rectangle2D.Double((double)x, (double)y,
                                   (double)width, (double)height));
}

/**
 * Overrides Graphics method
 */
public void setClip(Shape newClip)
{

    if (newClip instanceof Rectangle2D) {
        Rectangle2D r2d = (Rectangle2D)newClip;
        setClipPW(r2d.getX(), r2d.getY(), r2d.getWidth(), r2d.getHeight());
    }
    else {
        throw new IllegalArgumentException("setClip:  only Rectangles " +
                                           "are supported");
    }
}

/**
 * Overrides Graphics method
 */
public void copyArea(int x, int y, int width, int height, int dx, int dy)
{
    // not implemented
    
}

public void clearRect(int x, int y, int width, int height)
{
    // not implemented
    
}

/**
 * Overrides Graphics method
 */
public void drawOval(int x, int y, int width, int height)
{
    draw(new Arc2D.Double((double)x, (double)y,
                          (double)width, (double)height,
                          0.0, 360.0, Arc2D.CHORD));
}

/**
 * Overrides Graphics method.  Draw an Oval given an Ellipse2D
 */
protected void drawOval(Ellipse2D e2d)
{
    draw(new Arc2D.Double(e2d.getX(), e2d.getY(),
                          e2d.getWidth(), e2d.getHeight(),
                          0.0, 360.0, Arc2D.CHORD));
}

/**
 * Overrides Graphics method
 */
public void fillOval(int x, int y, int width, int height)
{
    fill(new Arc2D.Double((double)x, (double)y,
                          (double)width, (double)height,
                          0.0, 360.0, Arc2D.CHORD));
}

/**
 * Overrides Graphics method.  Fill an Oval given an Ellipse2D
 */
protected void fillOval(Ellipse2D e2d)
{
    fill(new Arc2D.Double(e2d.getX(), e2d.getY(),
                          e2d.getWidth(), e2d.getHeight(),
                          0.0, 360.0, Arc2D.CHORD));
}

/**
 * Overrides Graphics method
 */
public void drawPolygon(Polygon p)
{
    drawPolygon(p.xpoints, p.ypoints, p.npoints);
}

/**
 * Overrides Graphics method
 */
public void fillPolygon(Polygon p)
{
    fillPolygon(p.xpoints, p.ypoints, p.npoints);
}

/**
 * Overrides Graphics method
 */
public void drawChars(char data[], int offset, int length, int x, int y)
{
    drawString(new String(data, offset, length), x, y);
}

/**
 * Overrides Graphics method
 */
public void drawBytes(byte data[], int offset, int length, int x, int y)
{
    drawString(new String(data, offset, length), x, y);
}

/**
 * Overrides Graphics method
 */
public boolean drawImage(Image img, int x, int y, ImageObserver observer)
{

    return drawImage(img, x, y,
                     img.getWidth(observer), img.getHeight(observer),
                     null, observer);
}

/**
 * Overrides Graphics method
 */
public boolean drawImage(Image img, int x, int y, int width, int height,
                         ImageObserver observer)
{

    return drawImage(img, x, y, width, height, null, observer);
}

/**
 * Overrides Graphics method
 */
public boolean drawImage(Image img, int x, int y, Color bgcolor,
                         ImageObserver observer)
{

    return drawImage(img, x, y, img.getWidth(observer), img.getHeight(observer),
                     bgcolor, observer);
}

/**
 * Overrides Graphics method
 */
public boolean drawImage(Image img, int dx1, int dy1, int dx2, int dy2,
                         int sx1, int sy1, int sx2, int sy2,
                         ImageObserver observer)
{
    return true;
}

/**
 * Overrides Graphics method
 */
public boolean drawImage(Image img, int dx1, int dy1, int dx2, int dy2,
                         int sx1, int sy1, int sx2, int sy2,
                         Color bgcolor, ImageObserver observer)
{
    return true;
}

/**
 * Overrides Graphics method
 */
public void dispose()
{
    Graphics rootGC, frameGC, oldFrameGC = null, oldGC = null;

    // Don't dispose of graphics more than once
    if (!isLive) {
        return;
    }

    // Get the root Graphics from which this was ultimately derived
    rootGC = getPageRoot();
    frameGC = getFrameRoot();

    // Check if the root Graphics appears in the context map
    if (contextMap.containsKey(rootGC)) {

        // Use the root Graphics to determine the last GC used
        oldFrameGC = (Graphics)contextMap.get(rootGC);
    }

    // if this frame is current, check the current graphics
    if (oldFrameGC == frameGC) {

        // if we're not at the page and a current frame is stored, get it
        if (rootGC != oldFrameGC && contextMap.containsKey(oldFrameGC)) {
            oldGC = (Graphics)contextMap.get(oldFrameGC);
        }

        // if this is the current frame ending
        if (this == oldFrameGC) {

            // if the old gc isn't the frame
            if (oldGC != null && oldGC != frameGC) {
                // exit the enclosed level (oldGC)
                ((Graphics2DBase)oldGC).endContext();
            }

            // exit the current level (this == oldFrameGC)
            this.endContext();

            // update the context
            removeContext(oldFrameGC);

            // Unless this is the page itself, note the page context now
            // becomes our parent context
            if (this != rootGC) {
                contextMap.put(rootGC, parent);
            }
        }
        // otherwise we are a context within a frame
        else {
            // if we match the old context
            if (oldGC == this) {

                // exit this context
                this.endContext();

                // update the current frame for the outer context
                contextMap.put(frameGC, frameGC);
            }
        }
    }

    // Mark that this graphics has been disposed
    isLive = false;
}

// Remove a context (and any subordinate contexts) from the context map
private void removeContext(Graphics oldGC)
{

    // if this context is in the map get it's current value
    if (contextMap.containsKey(oldGC)) {
        Graphics subGC = (Graphics)contextMap.get(oldGC);

        // if this is a different, non-null context, make sure it's deleted
        if (subGC != null && subGC != oldGC) {
            removeContext(subGC);
        }
    }

    // Remove this gc from the map
    contextMap.remove(oldGC);
}

/**
 * Overrides Graphics method
 */
public void finalize()
{
    dispose();
}

/**
 * Overrides Graphics method
 */
public String toString()
{
    return getClass().getName() + "[font=" + getFont() + ",color=" +
           getColor() + "]";
}

/**
 * Overrides Graphics method
 */
/*
public Rectangle getClipRect() {
return getClipBounds();
}
*/

/**
 * Overrides Graphics method
 */
public boolean hitClip(int x, int y, int width, int height)
{
    return true;
}

/**
 * Overrides Graphics method
 */
public Rectangle getClipBounds(Rectangle r)
{
    Rectangle2D bounds = clip.getBounds2D();

    r.x = (int)bounds.getX();
    r.y = (int)bounds.getY();
    r.width = (int)bounds.getWidth();
    r.height = (int)bounds.getHeight();

    return r;
}

/**
 * Overrides Graphics method
 */
public void drawString(String string, int x, int y)
{
    drawString(string, (float)x, (float)y);
}

/**
 * Overrides Graphics method
 */
public void drawLine(int x1, int y1, int x2, int y2)
{

    draw(new Line2D.Double((double)x1, (double)y1, (double)x2, (double)y2));
}

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
////////////////////              Miscellaneous               ////////////
//////////////////// Some non-Graphics2D non Graphics methods ////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

/**
 * Define the location of a string to be printed
 */
protected void initializeOutString(float x, float y)
{

    out.x = x;
    // FIX out.y = - FontMetricsPS.asPoints(y - yOffset, getFont().getSize());
    out.y = y;
    out.text.delete(0, out.text.length());
}

/**
 * Writing to the output stream is funneled through here.
 * @param string The string to write.
 */
protected void output(String string) throws java.io.IOException
{
    if (finished) {
        return;
    }

    os.write(string.getBytes());
    os.write('\n');
}

/**
 * Draw a line given a PathIterator
 */
protected void drawLine(PathIterator path)
{
    Point2D.Double source = new Point2D.Double();
    while (!path.isDone()) {
        float pts[] = new float[6];
        switch (path.currentSegment(pts)) {
        case PathIterator.SEG_MOVETO:
            source.setLocation(pts[0], pts[1]);
            break;
        case PathIterator.SEG_LINETO:
            drawLinePW(source.getX(),
                       source.getY(),
                       (double)pts[0],
                       (double)pts[1]);
            source.setLocation(pts[0], pts[1]);
            break;
        case PathIterator.SEG_CUBICTO:
            break;
        case PathIterator.SEG_QUADTO:
            drawQuadraticPW(source.getX(),
                            source.getY(),
                            (double)pts[0],
                            (double)pts[1],
                            (double)pts[2],
                            (double)pts[3]);
            source.setLocation(pts[2], pts[3]);
            break;
        }
        path.next();
    }
}

/**
 * Draw a Rectangle given a Rectangle
 */
protected void drawRect(Rectangle2D r2d)
{
    // do driver-dependant draw
    drawRectPW(r2d.getX(), r2d.getY(), r2d.getWidth(), r2d.getHeight());
}

/**
 * Draw an Arc given an Arc2D
 */
protected void drawArc(Arc2D a2d)
{
    // do driver-dependant draw
    drawArcPW(a2d.getX(), a2d.getY(), a2d.getWidth(), a2d.getHeight(),
              a2d.getAngleStart(), a2d.getAngleExtent(), a2d.getArcType());
}

/**
 * Draw a RoundRectangle2D given a RoundRectangle2D
 */
protected void drawRoundRect(RoundRectangle2D rr2d)
{
    // do driver-dependant draw
    drawRoundRectPW(rr2d.getX(), rr2d.getY(), rr2d.getWidth(), rr2d.getHeight(),
                    rr2d.getArcWidth(), rr2d.getArcHeight());
}

/**
 * Fill an Arc given an Arc2D
 */
protected void fillArc(Arc2D a2d)
{
    // do driver-dependant draw
    fillArcPW(a2d.getX(), a2d.getY(), a2d.getWidth(), a2d.getHeight(),
              a2d.getAngleStart(), a2d.getAngleExtent(), a2d.getArcType());
}

/**
 * Fill a Rectangle given a Rectangle
 */
protected void fillRect(Rectangle2D r2d)
{
    // do driver-dependant draw
    fillRectPW(r2d.getX(), r2d.getY(), r2d.getWidth(), r2d.getHeight());
}

/**
 * Fill a RoundRectangle2D given a RoundRectangle2D
 */
protected void fillRoundRect(RoundRectangle2D rr2d)
{
    // do driver-dependant draw
    fillRoundRectPW(rr2d.getX(), rr2d.getY(), rr2d.getWidth(), rr2d.getHeight(),
                    rr2d.getArcWidth(), rr2d.getArcHeight());
}

protected void doAttributes(AttributedCharacterIterator iter)
{

    // start with a clean state
    underline = false;
    subscript = false;

    // get the list of attributes to be applied to this string
    Set set = iter.getAttributes().keySet();

    // get the iterator over them
    Iterator attributeIterator = set.iterator();

    // iterate over them and set the state in gc
    while (attributeIterator.hasNext()) {

        // process the next attribute
        TextAttribute attribute = (TextAttribute)attributeIterator.next();

        // change the font for this run
        if (attribute.equals(TextAttribute.FONT)) {
            Font newFont = (Font)iter.getAttributes().get(attribute);
            setFont(newFont);
        }
        // turn on underline for this run
        else if (attribute.equals(TextAttribute.UNDERLINE)) {
            Object o = iter.getAttributes().get(attribute);
            if (o == TextAttribute.UNDERLINE_ON) {
                underline = true;
            }
        }
        // turn on super- or sub-scripting
        else if (attribute.equals(TextAttribute.SUPERSCRIPT)) {
            Object o = iter.getAttributes().get(attribute);
            if (o == TextAttribute.SUPERSCRIPT_SUPER) {
                // ratio
            }
            else {
                // subscript
                // ratio
            }
            subscript = true;
        }
        else if (attribute.equals(TextAttribute.SIZE)) {
            Object o = iter.getAttributes().get(attribute);
            Font f = null;
            if (o instanceof Integer) {
                int newSize = ((Integer)o).intValue();

                f = getFont().deriveFont(getFont().getStyle(), newSize);
            }
            else if (o instanceof Float) {
                float newSize = ((Float)o).floatValue();

                f = getFont().deriveFont(getFont().getStyle(), newSize);
            }
            else {
                System.out.println("Unknown type for attribute SIZE");
                return;
            }
            setFont(f);
        }
        else {
            // case TextAttribute.BACKGROUND:
            // case TextAttribute.BIDI_EMBEDDING:
            // case TextAttribute.CHAR_REPLACEMENT:
            // case TextAttribute.FAMILY:
            // case TextAttribute.FOREGROUND:
            // case TextAttribute.INPUT_METHOD_HIGHLIGHT:
            // case TextAttribute.JUSTIFICATION:
            // case TextAttribute.POSTURE:
            // case TextAttribute.RUN_DIRECTION:
            // case TextAttribute.STRIKETHROUGH:
            // case TextAttribute.SWAP_COLORS:
            // case TextAttribute.TRANSFORM:
            // case TextAttribute.WEIGHT:
            // case TextAttribute.WIDTH:
            String s = "only FONT/SIZE/UNDERLINE/SUPERSCRIPT supported";
            throw new java.lang.RuntimeException(s);
        }
    }
}

/**
 * Draws an attributed string
 */
protected void doString(AttributedCharacterIterator iter, float x, float y)
{

    // so we can restore it when done
    // Font originalFont = getFont();

    // assume string is monotonic and make buffer big enough for it
    StringBuffer sb = new StringBuffer(iter.getEndIndex());

    // A 'run' is a subset of the string such that all attributes are the same
    // over its length. Process a 'run' at a time. In the best case a 'run' is
    // the entire string. Worst case, no two contiguous characters share the
    // same set of attributes.
    for (char c = iter.first(); c != CharacterIterator.DONE; c = iter.next()) {

        // if this is the start of a new run ...
        if (iter.getIndex() == iter.getRunStart()) {

            // print out string accumulated so far
            if (sb.length() > 0) {

                // call the device-dependant drawString
                drawString(sb.toString(), x, y);

                // add the length of the string we just printed
                // to the overall length
                FontMetrics fm = getFontMetrics();
                x += fm.getStringBounds(sb.toString(), this).getWidth();

                // clear the buffer


                sb.delete(0, sb.length());
            }

            // and set up for next run
            doAttributes(iter);

        }

        // finally, add the char
        sb.append(c);
    }

    // flush the remainder
    drawString(sb.toString(), x, y);

    // reset
    underline = false;
    subscript = false;
    // setFont(originalFont);
}

/**
 * Set the output stream to which data should be sent.
 * Called by JCPrinter.setOutputStream.
 */
public void setOutputStream(OutputStream os)
{
    this.os = os;
}

/**
 * Get the output stream to which data should be sent.
 */
public OutputStream getOutputStream()
{
    return os;
}

/**
 *
 * Set the maximum number of digits to follow
 * the decimal point. The default is 4. Fewer
 * digits make for smaller files but are less precise.
 * Larger values give diminishing returns considering
 * the resolution of most output devices.
 *
 * Used when outputting strings.
 *
 */
public void setMaxDecimalDigits(int maxDecimalDigits)
{
    this.maxDecimalDigits = maxDecimalDigits;
}

/**
 *
 * Get the maximum number of digits to follow
 * the decimal point. The default is 4. Fewer
 * digits make for smaller files but are less precise.
 * Larger values give diminishing returns considering
 * the resolution of most output devices.
 *
 * Used when outputting strings.
 *
 */
public int getMaxDecimalDigits()
{
    return maxDecimalDigits;
}

/**
 *
 * Truncate decimal numbers so they have getMaxDecimalDigits()
 * following the decimal point.
 *
 */
public double truncateDecimalDigits(double d)
{
    decimalFormat.setMaximumFractionDigits(maxDecimalDigits);
    double newD = Double.valueOf(decimalFormat.format(d)).doubleValue();
    if (newD == -0.0) {
        newD = 0.0;
    }
    return newD;
}

/**
 *
 * Truncate decimal numbers so they have getMaxDecimalDigits()
 * following the decimal point.
 *
 */
public double truncateDecimalDigits(float d)
{
    decimalFormat.setMaximumFractionDigits(maxDecimalDigits);
    double newD = Double.valueOf(decimalFormat.format(d)).doubleValue();
    return newD;
}

/**
 *
 * Set the winding rule which determines how polygons are filled.
 *
 */
protected void setWindingRule(int windingRule)
{
    this.windingRule = windingRule;
}

/**
 *
 * Get the winding rule which determines how polygons are filled.
 *
 */
protected int getWindingRule()
{
    return windingRule;
}

/**
 * Find the root graphics (created by the printer) from which this
 * Graphics ultimately derives
 */
protected Graphics getPageRoot()
{
    if (parent == null) {
        return(this);
    }
    return(((Graphics2DBase)parent).getPageRoot());
}

/**
 * Find the frame from which this graphics was derived
 */
protected Graphics getFrameRoot()
{
    if (isFrameGraphics) {
        return(this);
    }
    else if (parent == null) {
        throw new IllegalArgumentException("no frame root for graphics");
        // return(null);
    }
    return(((Graphics2DBase)parent).getFrameRoot());
}

/**
 * INTERNAL USE ONLY
 * Mark this graphics as corresponding to a JCFrame
 */
public void setFrameRoot()
{
    isFrameGraphics = true;
}

/**
 * INTERNAL USE ONLY
 * Return true if this graphics corresponds to a JCFrame
 */
public boolean isFrameRoot()
{
    return isFrameGraphics;
}

/**
 * Determine if this context is the one currently in use by the printer.
 */
protected boolean isContextCurrent()
{
    Graphics rootGC, oldFrameGC, frameGC, oldGC;

    // Get the frame and page GCs for this graphics
    rootGC = getPageRoot();
    frameGC = getFrameRoot();

    // if the root Graphics doesn't appear in the context map,
    // then it isn't being used (and hence isn't current)
    if (!contextMap.containsKey(rootGC)) {
        return(false);
    }

    // if the frame Graphics doesn't appear in the context map,
    // then it isn't being used (and hence isn't current)
    if (!contextMap.containsKey(frameGC)) {
        return(false);
    }

    // Use the root Graphics to determine the last GC used
    oldGC = (Graphics)contextMap.get(frameGC);
    oldFrameGC = (Graphics)contextMap.get(rootGC);

    // return if the current context is the stored (in use) context
    return(oldGC == this && oldFrameGC == frameGC);
}

/**
 *
 * If this context is not the printer's context, update the printer's context.
 *
 * NOTES:
 *   - Contexts can be marked as "frames," which correspond to frames in the
 *     document (the page is also considered a frame).
 *   - Frames are permitted to contain other frames or non-frame contexts.
 *   - Non-frame contexts are not permitted to be nested.
 *   - Because the parent is the context a given graphics was derived from,
 *     it is possible for a given context to be several levels below a frame.
 *     We only care about the last frame context from which it was parented.
 *   - Since the user may only derive Graphics from provided Graphics objects,
 *     we are guaranteed that the root will always be a "frame."
 *   - If a gc is a frame context, "gc.getFrameRoot()" will return gc.
 *
 * This has a number of possible cases:
 * 	 1)  The printer's context is a frame and this context is inside it
 *       (This context may or may not be a frame)
 *          - apply the new context
 * 	 2)  The printer's context and this context are peers (belong to same frame)
 *       (Either context may or may not be a frame)
 *          - exit the printer's context
 *          - apply the new context
 *   3)  This context belongs to a different frame than the printer's context
 *       (Either context may or may not be a frame)
 *          - find the common frame context ancestor of the two contexts
 *            (guaranteed to both belong to the same root on a page)
 *          - exit contexts on the printer up to the ancestor
 *			- apply new contexts from the ancestor down to the new context
 *
 * In general the approach is:
 *   - If the this context is also the printer's context, return.
 *   - Exit any non-frame context
 *   - If the frame context is different, exit any nested frames up to
 *     the common ancestor of the new and old frames
 *   - reapply any nested frame contexts from the common ancestor frame
 *     down to the frame of this context
 *   - Apply a non-frame context if required
 *
 * contextMap special cases:
 *   In general contextMap stores a mapping from page to current frame and
 *   current frame to current context.  (The path of frames between the
 *   page and current frame is implicit in the parent pointers.)  In the
 *   case where a frame (or page) has no current internal context (the
 *   frame is being drawn into or the previous internal context has been
 *   discarded by the dispose() method), that frame is considered to be
 *   it's own current context.  Thus there are checks, after retrieving
 *   oldGC = contextMap.get(frameGC), that oldGC != frameGC.
 *
 **/
protected void updateContext()
{

    Graphics rootGC, frameGC, oldFrameGC = null, oldGC = null;

    // Determine the this graphic's frame and page
    rootGC = getPageRoot();
    frameGC = getFrameRoot();

    // if the page has a frame stored, get it
    if (contextMap.containsKey(rootGC)) {
        oldFrameGC = (Graphics)contextMap.get(rootGC);

        // unless the page (and not a contained frame) is current,
        // get the current graphics of the old frame
        if (rootGC != oldFrameGC && contextMap.containsKey(oldFrameGC)) {
            oldGC = (Graphics)contextMap.get(oldFrameGC);
        }
    }

    // We can skip the rest if the frame and graphics are the same
    if (this == oldGC && frameGC == oldFrameGC) {
        return;
    }

    // if there is no old frame on the page
    if (oldFrameGC == null) {

        // Store the new current frame
        contextMap.put(rootGC, frameGC);

        // if we are working in a subframe of the frame we enter the frame first
        if (this != frameGC) {
            ((Graphics2DBase)frameGC).startContext();
        }
    }

    // if the old frame is the current frame
    else if (frameGC == oldFrameGC) {

        // If there is a current non-frame graphics, exit it
        if (oldGC != oldFrameGC && oldGC != null) {
            ((Graphics2DBase)oldGC).endContext();
        }
    }

    // cope with different frames
    else {

        // ignore if "oldFrame" is the page
        if (oldFrameGC != rootGC) {

            // Build the paths to root of the two frame contexts
            Graphics[] gcList1, gcList2;
            gcList1 = ((Graphics2DBase)frameGC).buildParentArray();
            gcList2 = ((Graphics2DBase)oldFrameGC).buildParentArray();

            // find the common frame ancestor
            int ancestor = 0;
            while (ancestor < gcList1.length &&
                   ancestor < gcList2.length &&
                   gcList1[ancestor] == gcList2[ancestor])
            {
                ancestor++;
            }
            ancestor--;

            // If there is a current non-frame graphics, exit it
            if (oldGC != oldFrameGC && oldGC != null) {
                ((Graphics2DBase)oldGC).endContext();
            }

            // Exit all old frame contexts and reapply the new ones
            for (int i = gcList2.length - 1; i > ancestor; i--) {
                ((Graphics2DBase)gcList2[i]).endContext();
            }
            for (int i = ancestor + 1; i < gcList1.length - 1; i++) {
                ((Graphics2DBase)gcList1[i]).startContext();
            }
        }

        // Store the new frame context
        contextMap.put(rootGC, frameGC);
    }

    // store and enter the new graphics
    contextMap.put(frameGC, this);
    this.startContext();

    // Make sure setup code is output for the next operations.
    colorChanged = true;
    fontChanged = true;
    strokeChanged = true;

    // underline, subscript, etc. (AttributedCharacter stuff) ???
}

protected Graphics[] buildParentArray()
{
    return (buildParentArray(1));
}

protected Graphics[] buildParentArray(int level)
{

    Graphics[] parentArray;

    if (parent != null) {
        parentArray = ((Graphics2DBase)parent).buildParentArray(level + 1);
        parentArray[parentArray.length - level] = this;
    }
    else {
        parentArray = new Graphics[level];
        parentArray[0] = this;
    }
    return (parentArray);
}

/**
 * Return 'finished' state.  If true, graphics object is no longer outputting.
 */
public boolean isFinishedOutput()
{
    return finished;
}

/**
 * If set to true, graphics object will no longer output.
 */
public void setFinishedOutput(boolean fin)
{
    finished = fin;
}

////////////////////////////// Device-specific methods ////////////////////
///////////////////////////////////////////////////////////////////////////
// These methods must be overridden in the driver-specific subclasses
// to output the actual device commands.
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////

/**
 * Create a new Graphics class that is the same as the existing graphics
 * class.  This is different than create since it creates an independent
 * graphics class that is unrelated to previous graphics classes.
 * To be overridden by subclasses.
 */
public abstract Graphics newGraphics();

/**
 * INTERNAL USE ONLY
 * Must be overridden to provide device-specific functionality
 */
public abstract void setClipPW(double x, double y, double width, double height);

/**
 * INTERNAL USE ONLY
 * Must be overridden to provide device-specific functionality
 */
public abstract void drawLinePW(double x, double y, double x2, double y2);

public void drawQuadraticPW(double x1, double y1,
                            double xc, double yc, double x2, double y2)
{
// not implemented
    drawLinePW(x1, y1, x2, y2);
}

/**
 * INTERNAL USE ONLY
 * Must be overridden to provide device-specific functionality
 */
public abstract void drawRectPW(double x, double y, double width,
                                double height);

/**
 * INTERNAL USE ONLY
 * Must be overridden to provide device-specific functionality
 */
public abstract void fillRectPW(double x, double y, double width,
                                double height);

/**
 * INTERNAL USE ONLY
 * Must be overridden to provide device-specific functionality
 */
public abstract void drawRoundRectPW(double x, double y, double width,
                                     double height, double arcWidth,
                                     double arcHeight);

/**
 * INTERNAL USE ONLY
 * Must be overridden to provide device-specific functionality
 */
public abstract void fillRoundRectPW(double x, double y, double width,
                                     double height, double arcWidth,
                                     double arcHeight);

/**
 * INTERNAL USE ONLY
 * Must be overridden to provide device-specific functionality
 */
public abstract void fillArcPW(double x, double y, double width, double height,
                               double arcStart, double arcSweep, int arcType);

/**
 * INTERNAL USE ONLY
 * Must be overridden to provide device-specific functionality
 */
public abstract void drawArcPW(double x, double y, double width, double height,
                               double arcStart, double arcSweep, int arcType);

/**
 * INTERNAL USE ONLY
 * Must be overridden to provide device-specific functionality
 */
public abstract void startContext();

/**
 * INTERNAL USE ONLY
 * Must be overridden to provide device-specific functionality
 */
public abstract void endContext();


////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////
//////////////////// Inner class for string buffer /////////////////
////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////



protected class Buffer {
    public StringBuffer text = new StringBuffer();
    public int size; 	// font pt size
    public double width;	// width of string, can be different
    // than length of chars
    public double x, y;	// origin of text
}
/////////////////// End of Inner class //////////////////////

}
