/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JClassInfo.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:42 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.internal;

import com.klg.jclass.util.swing.JCIconCreator;

import javax.swing.*;
import java.awt.*;
import java.io.*;
import java.util.Properties;

/**
 *  <p>The <i>JCLASS_HOME</i> environment variable is set up by the JClass installation
 *  program.  <i>JCLASS_HOME</i> cannot be accessed directly inside the Java VM because
 *  <code>java.lang.System.getenv()</code> is deprecated.
 *  <p>To use <i>JCLASS_HOME</i>, java must be run with the <i>-DJCLASS_HOME=[path]</i>
 *  argument.  To pass the environment variable <i>JCLASS_HOME</i> into java,
 *  use the following:
 *  <p><code>java -DJCLASS_HOME="${JCLASS_HOME}" class</code>
 *  <p> or at a DOS prompt,
 *  <p><code>java -DJCLASS_HOME="%JCLASS_HOME%" class</code>
 *  <p>If JCLASS_HOME is undefined, the path in the system property
 *  "user.home"/jclass is used.
 */
public class JClassInfo {

static private String[] company_logo = {
"DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",
"DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",
"DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",
"DDDDDDddzz...nhoii##pCahDDDDDDDDDDDDDDDDd.nhhhn.dDDDd...dDDDDDd...Dd.........zDDDz.hhhhmdDDz...........zDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",
"DDDiib##kaaCCCCCCCCCCCahDDDDDDDDDDDDDDdm#CCCCCCC#ndD.CCcnDDDDDoCCCzoCCCCCCCCCfddhpCCCCCClmdfCCCCCCCCCCCfDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",
"DDDCCCCCCCCCCCCCCCCCCCahDDDdddddDDDDDDACCc#onh#cCCoD.CCgmDDDDDhaCCzhcCgbbbbbfhdjCCaiAh#CCk.Afbb#cCafbbbADDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",
"DDDCCCCCCCCCCCCCCCCCCCaADdxvrerqxdDDD.gCcfdDDDDbcCC..CCamDDDDDhaCCzhcClDDDDDDD.lCCiDDDzlploDDDDngCpzDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",
"DDDCCCCCCCCCCCCCCCCCCCguwqeeeeeeeqdDDoCCkmDDDDD.pCCj.CCgnDDDDDhaCCzhcCp.z.z.zdzlCCln.zDdzzdDDDDngCkzDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",
"DDDCCCCCCCCCCCCCCCCCCCCBreeqvssqresDdbCCldDDDDDD#CCfmcCgmDDDDDhaCCzhcCcppppkkADjgCCCpfjh.dDDDDDAgCkzDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",
"DDDCCCCCCCCCCCCCCCCCCC#eeqxDDDDDwrsDdfCC#dDDDDDD#CCfmcCgmDDDDDhaCCzhcCckkkkkkoDDmb#kcCCCkb.DDDDngCkzDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",
"DDDCCCCccCCCCCCCCCCCC#.qvdDwsvqswwwDDjCCp.DDDDDz#CCj.CCamDDDDDoaCczhcCpm.m.mmdzzzz.mhi#CCCbDDDDAgCkzDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",
"DDDCCCCapgCCCCCCCCCC#zzuDdvreeeeesDDDnCCaoDDDDDhgClzdCCaoDDDDDiCCpdhcClDDDDDDDhl#fdDDDdpCCpDDDDngCkzDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",
"DDDCCCCCpblCCCCCCCCfzdfAwqeeeqqreexDDdfCCkhzdzAlCCozdbCClnzdzhgCCoDhcCkmmmmmm.oCCCo.dznkCCbDDDDngCkzDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",
"DDDCCCcgC#jjfaCCg#odzbCBqeevwdDdsesDDDzbCCCl#lCCCCkpizbCCgl#lCCCizDoCCCagaaaa#miCCCl#laCCbzDDDDhCCazDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",
"DDDCCCafkCpim.nnmdDm#CyteesDDxsxdxxDDDDznj#kgpbibbbbhDzhi#pgkfindDDmbbbbbbbbbjzzhb#kglljAzDDDDDmibidDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",
"DDDCCCCljlCapin.zhfgCahrrwDsreeerxDDDDDDDdz..zdddddzdDDDdzz.zzdDDDDDddddddddddDDDdz..zzdDDDDDDDDdddDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",
"DDDCCckCfniCCCgkkaCCpmdvwdqeeereeexDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",
"DDDCCcl#C#mmikCCCCljdDoAdreevxdwvesDDDDDdmAhhnzDDDDDDDznhhhnzDDDDzmmmmmmmmmzdmmmmmmmmmmmm.mmmDDDDdmmm.DDDDzmmmDDzmmm.DDDDDzmmmmmmm.dDDDzmmmmmmmmm.DDD",
"DDDCCCgjfCgjddmnnmdD.faBrerwDDDDDsxDDDdjlaCCCckjdDDD.bpaCCCakb.DDnkaaaaaacgongcaaaaaaaaaljaal.DDDnkccpzDDdjccbdDiaacpzDDDdjcaaaacaklizDnaaaaaaaac#DDD",
"DDDCCCClnbCCabndDzAfaCyteqdDDDDDDDDDDdfCCgbobaCCidDngCCgboigCCanDAgCCaaaaaghmgaaaCCCcaaa#maCahDDDbcCCCmDDzlCahDmpCCCCoDDDdjCCcaaaaCCClDACCCaaaaacfDDD",
"DDDCCCCCf.ogcCCCCCCCcpAqqDDDDDDDDDDDDnCCgnDDDhkgpzzkCC#dDDDdfCCkzngCkDDDDDDDDDDDd#CCoDDDDdfCCiDD.lCcCCjDDmaCl.DiCcpaCldDDdiCC#zDDDicCc.nCCgzDDDDDDDDD",
"DDDCCCCCC#.zjpaaaaagizzudDDDDDDDDDDDDnCCajzdddz..DocCamDDDDDmkCChAgCkddddddDDDDDz#CCjDDDDDhCCfzDhgCiaCgDDoCCbDmkCl.lCChDDdjCC#zDDdoaCgznCCg.dddddDDDD",
"DDDCCCCCCCaodzmAhhn.D.buDDDDDDDDDDDDD.pCCCk#bo.dDDbcCkdDDDDDzfCCjhgCc#f#f#b.DDDDz#CCjDDDDD.aCgmDbCazfCCmdfCghDiCCoDoaCpdDdjCCkoAobgCgoDACCc##f###ADDD",
"DDDCCCCCCCCCaimzdddzhkChDDDDDDDDDDDDDDzblgcCCCClnDbCCkdDDDDDdbCCjogCCccccckmDDDDz#CCjDDDDDdbCCo.lCbDhcCidkC#zmgCp.DdpCCnDdjCCCCCCCCcjDDACCCagggggoDDD",
"DDDCCCCCCCCCCCCa#f#aCCahDDDDDDDDDDDDDdDDdnjbfgCCCmbcCkzDDDDDz#CCjhgCaoohoohzDDDDz#CCjDDDDDDhCCbjgChD.aC#nCCiDiCCloAnlCC#DdjCCgbbifgCChDACCghmmmmmzDDD",
"DDDCCCCCCCCCCCCCCCCCCCahDDDDDDDDDDDDdjiihDDDDhgCCohCCanDDDDDngCCAngCkDDDDDDDDDDDz#CCjDDDDDDmkCkkCg.Dd#CglCgm.kCCCCCCCCCCmDiCC#DDDDbCC#DACCkdDDDDDDDDD",
"DDDCCCCCCCCCCCCCCCCCCCahDDDDDDDDDDDDdlCCgmzd.ogCCmd#CCpmzdzn#CClzngCkdDDDDDDDDDDz#CCjDDDDDDzbCCCCbdDDjCCCC#djCClibbbilCC#djCC#zDDDoaCpdnCCghmmmmmmDDD",
"DDDCCCCCCCCCCCCCCCCCCCahDDDDDDDDDDDDDmgCCgfflCCcbdD.#CCaff#aCC#mDhaCgzDDDDDDDDDDzlCCjDDDDDDDoCCCCADDD.cCCCi.gCanDDDDDmcCCnoCCl.DDDhaCc.nCCCCcCcCCgDDD",
"DDDhhjjifflpgaccCCCCCCcoDDDDDDDDDDDDDDmblaCCalbAdDDDzofgCCCkfozDDm#lfdDDDDDDDDDDzblphDDDDDDD.#ppf.DDDDfll#mhkpidDDDDDDblljhllbzDDD.flpAmllllplplp#DDD",
"DDDDDDDdzz..mmnAojibf##nDDDDDDDDDDDDDDDd.mmmm.dDDDDDDDz.mmmmzDDDDd...DDDDDDDDDDDDz.mzDDDDDDDd....DDDDDz...Dz..zDDDDDDDz...z...DDDDd...zd..........DDD",
"DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",
"DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD"
};

// Careful -- this message is repeated 6 times in varying formats and
// combinations.  If you change one, make sure all are changed.
public static final String product_info =
"<B>For information about JClass products:</B>"+
"<TABLE><TR><TD>JClass General Info:</TD>"+
"<TD><A HREF=http://www.quest.com/solutions/>"+
"http://www.quest.com/solutions/</A></TD></TR>"+
"<TR><TD>JClass Developers Forum:</TD>"+
"<TD><A HREF=http://qlist1.quest.com:8090/read/all_forums/>"+
"http://qlist1.quest.com:8090/read/all_forums/</A></TD></TR></TABLE>"+
"<p>";

public static final String product_info_txt =
"For information about JClass products:\n"+
"  JClass General Info:     http://www.quest.com/solutions/\n"+
"  JClass Developers Forum: http://qlist1.quest.com:8090/read/all_forums/\n"+
"\n";

private static final String defaultProperties[][] = {
// key                                      // default value
{ "datasource.data_bean_component.customizer.show_tips_at_start",   "true" },
{ "datasource.tree_data_bean_component.customizer.show_tips_at_start",  "true" },
{ "datasource.jdbc.drivers",                                        "sun.jdbc.odbc.JdbcOdbcDriver" },
{ "datasource.jdbc.urls",                                           "" },
{ "datasource.jdbc.urls.delimiter",                                 "#!#*#" },
{ "higrid.higrid_bean_component.customizer.show_tips_at_start",     "true" },

};

// Part of message displayed at bottom of eval expiry dialog box
// Careful -- this message is repeated 6 times in varying formats and
// combinations.  If you change one, make sure all are changed.
private static final String evalMessage =
"<p>To purchase permanent JClass product licenses, including "+
"technical support services and/or source code, or to "+
"inquire about any of the other tools Quest Software provides "+
"Java developers, you can go direct to Quest Software, or to a reseller. </p>"+
"<p>To find a list of resellers near you, please go to "+
"<A HREF=http://www.quest.com/partners/>"+
"http://www.quest.com/partners/</a></p>"+
"<p>To contact Quest Software directly, please go to "+
"<A HREF=http://www.quest.com>http://www.quest.com</A> "+
"or call: "+
"<TABLE><TR><TD>North America:</TD><TD>1 800 663 4723</TD></TR>"+
"<TR><TD>Europe:</TD><TD>+31 (0) 20 510 67 00</TD></TR>"+
"<TR><TD>Rest of World:</TD><TD>+1 416 594 1026</TD></TR></TABLE>"+
"</p>";

private static final String evalMessage_txt =
"To purchase permanent JClass product licenses, including\n "+
"technical support services and/or source code, or to\n "+
"inquire about any of the other tools Quest Software provides\n "+
"Java developers, you can go direct to Quest Software, or to a reseller.\n\n"+
"To find a list of resellers near you, please go to\n"+
"http://www.quest.com/partners/\n\n"+
"To contact Quest Software directly, please go to\n "+
"http://www.quest.com or call: \n"+
"  North America: 1 800 663 4723\n"+
"  Europe:        +31 (0) 20 510 67 00\n"+
"  Rest of World: +1 416 594 1026\n"+
"\n";

// Careful -- this message is repeated 6 times in varying formats and
// combinations.  If you change one, make sure all are changed.
public static final String docsupMessage =
"<B>For product documentation and support:</B>" +
"<TABLE><TR><TD>Local:</TD><TD>JCLASS_HOME/docs</TD></TR>"+
"<TR><TD>Online at Quest Software:</TD>"+
"<TD><A HREF=http://www.quest.com/solutions/>http://www.quest.com/solutions/</A></TD></TR>"+
"<TR><TD>Documentation Centre:</TD>"+
"<TD><A HREF=http://questsupportlink.quest.com/support_downloads/Resources/JClass/documentation.html>"+
"http://questsupportlink.quest.com/support_downloads/Resources/JClass/documentation.html</A></TD></TR>"+
"<TR><TD>Knowledge Base:</TD>"+
"<TD><A HREF=http://questsupportlink.quest.com/eSupport/SolutionFind.asp>"+
"http://questsupportlink.quest.com/eSupport/SolutionFind.asp</A></TD></TR></TABLE>" +
"<p>";

public static final String docsupMessage_server =
"<B>For product documentation and support:</B>" +
"<TABLE><TR><TD>Local:</TD><TD>JCLASS_SERVER_HOME/docs</TD></TR>"+
"<TR><TD>Online at Quest Software:</TD>"+
"<TD><A HREF=http://www.quest.com/solutions/>http://www.quest.com/solutions/</A></TD></TR>"+
"<TR><TD>Documentation Centre:</TD>"+
"<TD><A HREF=http://questsupportlink.quest.com/support_downloads/Resources/JClass/documentation.html>"+
"http://questsupportlink.quest.com/support_downloads/Resources/JClass/documentation.html</A></TD></TR>"+
"<TR><TD>Knowledge Base:</TD>"+
"<TD><A HREF=http://questsupportlink.quest.com/eSupport/SolutionFind.asp>"+
"http://questsupportlink.quest.com/eSupport/SolutionFind.asp</A></TD></TR></TABLE>" +
"<p>";

public static final String docsupMessage_both =
"<font size=2>For product documentation and support:" +
"<TABLE><TR><TD>Local:</TD><TD>JCLASS_HOME/docs</TD></TR>"+
"<TR><TD> </TD><TD>JCLASS_SERVER_HOME/docs</TD></TR>"+
"<TR><TD>Online at Quest Software:</TD>"+
"<TD><A HREF=http://www.quest.com/solutions/>http://www.quest.com/solutions/</A></TD></TR>"+
"<TR><TD>Documentation Centre:</TD>"+
"<TD><A HREF=http://questsupportlink.quest.com/support_downloads/Resources/JClass/documentation.html>"+
"http://questsupportlink.quest.com/support_downloads/Resources/JClass/documentation.html</A></TD></TR>"+
"<TR><TD>Knowledge Base:</TD>"+
"<TD><A HREF=http://questsupportlink.quest.com/eSupport/SolutionFind.asp>"+
"http://questsupportlink.quest.com/eSupport/SolutionFind.asp</A></TD></TR></TABLE>" +
"<p>";

public static final String docsupMessage_txt =
"For product documentation and support:\n"+
"  Local:              JCLASS_HOME/docs\n"+
"  Online at Quest Software: http://www.quest.com/solutions/\n"+
"  Documentation Centre: http://questsupportlink.quest.com/support_downloads/Resources/JClass/documentation.html\n"+
"  Knowledge Base:     http://questsupportlink.quest.com/eSupport/SolutionFind.asp\n"+
"\n";

public static final String docsupMessage_txt_server =
"For product documentation and support:\n"+
"  Local:              JCLASS_SERVER_HOME/docs\n"+
"  Online at Quest Software: http://www.quest.com/solutions/\n"+
"  Documentation Centre: http://questsupportlink.quest.com/support_downloads/Resources/JClass/documentation.html\n"+
"  Knowledge Base:     http://questsupportlink.quest.com/eSupport/SolutionFind.asp\n"+
"\n";

public static final String docsupMessage_txt_both =
"For product documentation and support:\n"+
"  Local:              JCLASS_HOME/docs\n"+
"                      JCLASS_SERVER_HOME/docs\n"+
"  Online at Quest Software: http://www.quest.com/solutions/\n"+
"  Documentation Centre: http://questsupportlink.quest.com/support_downloads/Resources/JClass/documentation.html\n"+
"  Knowledge Base:     http://questsupportlink.quest.com/eSupport/SolutionFind.asp\n"+
"\n";


// package names (jclass.XXX) of JClass Eval products
private static final String[] evalPackageNames = {
    "chart",
    "datasource",
    "field",
    "higrid",
    "jarmaster",
    "page",
    "swing",
    "table",
    "chart3d"
};

private static final String[] evalServerPackageNames = {
    "schart",
    null,
    null,
    null,
    null,
    null,
    null,
    null,
    null
};

/**
 * Creates the KL logo using the background color from
 * the specified component
 */
static public Image makeKLImage(Component comp)
{
    return makeKLImage((comp == null) ? null : comp.getBackground());
}

/**
 * Creates the KL logo with the specified background color.
 */
static public Image makeKLImage(Color background)
{
    return ((ImageIcon)makeKLIcon(background)).getImage();
}

/**
 * Creates the KL Icon using the background color from
 * the specified component
 */
static public Icon makeKLIcon(Component comp)
{
    return makeKLIcon((comp == null) ? null : comp.getBackground());
}

/**
 * Creates the KL Icon, with the specified background color.
 */
static public Icon makeKLIcon(Color background)
{
    JCIconCreator icon = new JCIconCreator(149, 32);
    if (background != null) {
        icon.setColor('G', background);
    }
    else {
        // Transparent
        icon.setColor('G', null);
    }
    icon.setColor('.', new Color(0x2f2f2f));
    icon.setColor('#', new Color(0xc6c6c6));
    icon.setColor('a', new Color(0xf4f4f4));
    icon.setColor('b', new Color(0xa6a6a6));
    icon.setColor('c', new Color(0xfafafa));
    icon.setColor('d', new Color(0x11100f));
    icon.setColor('e', new Color(0xf9d133));
    icon.setColor('f', new Color(0xb7b7b7));
    icon.setColor('g', new Color(0xececec));
    icon.setColor('h', new Color(0x686868));
    icon.setColor('i', new Color(0x979797));
    icon.setColor('j', new Color(0x888888));
    icon.setColor('k', new Color(0xe2e2e2));
    icon.setColor('l', new Color(0xd2d2d2));
    icon.setColor('m', new Color(0x444444));
    icon.setColor('n', new Color(0x585858));
    icon.setColor('o', new Color(0x767676));
    icon.setColor('p', new Color(0xdbdbdb));
    icon.setColor('q', new Color(0xab9023));
    icon.setColor('r', new Color(0xd4b129));
    icon.setColor('s', new Color(0x6a5914));
    icon.setColor('t', new Color(0xeacc59));
    icon.setColor('u', new Color(0x726a4d));
    icon.setColor('v', new Color(0x90791f));
    icon.setColor('w', new Color(0x352c0a));
    icon.setColor('x', new Color(0x493d0f));
    icon.setColor('y', new Color(0xfdf9ed));
    icon.setColor('z', new Color(0x1e1e1d));
    icon.setColor('A', new Color(0x615f58));
    icon.setColor('B', new Color(0xb1a77f));
    icon.setColor('C', new Color(0xffffff));
    icon.setColor('D', new Color(0x000000));
    icon.setPixels(company_logo);
    return icon.getIcon();
}

public static String getEvalMessage()
{
    return getEvalMessage(true, false);
}

public static String getEvalMessage(boolean client, boolean server)
{
    if (client && server) {
        return evalMessage + product_info + docsupMessage_both;
    }
    else if (server) {
        return evalMessage + product_info + docsupMessage_server;
    }
    else {
        return evalMessage + product_info + docsupMessage;
    }
}

public static String getEvalMessageNonHTML()
{
    return getEvalMessageNonHTML(true, false);
}

public static String getEvalMessageNonHTML(boolean client, boolean server)
{
    if (client && server) {
        return evalMessage_txt + product_info_txt + docsupMessage_txt_both;
    }
    else if (server) {
        return evalMessage_txt + product_info_txt + docsupMessage_txt_server;
    }
    else {
        return evalMessage_txt + product_info_txt + docsupMessage_txt;
    }
}

/**
 *  Returns package names of eval products.
 *  @return Array of package name strings.
 */
public static String[] getEvalPackageNames()
{
    return evalPackageNames;
}

/**
 *  Returns package names of serverside eval products.
 *  @return Array of package name strings.
 */
public static String[] getEvalServerPackageNames()
{
    return evalServerPackageNames;
}

/**
 * Gets a JClass property from <code>jclass.properties</code>. If
 * <code>jclass.properties</code> is not found, an empty
 * <code>jclass.properties</code> is created with default properties. <p>The
 * key in <code>jclass.properties</code> should be "jclass.[property_name]".
 * @param property_name name of the property that is to be returned.
 */
public static String getProperty(String property_name)
{
    return getJClassProperty("jclass." + property_name);
}

/**
 * Gets a JClass product-specific property from <code>jclass.properties</code>.
 * If <code>jclass.properties</code> is not found, an empty
 * <code>jclass.properties</code> is created with default properties. <p>The
 * key in <code>jclass.properties</code> should be "[product].[property_name]".
 * @param product JClass product name. e.g. higrid
 * @param property_name name of the property that is to be returned.
 */
public static String getProperty(String product, String property_name)
{
    return getJClassProperty(product + "." + property_name);
}

/**
 * Gets a JClass product-and-version-specific property from
 * <code>jclass.properties</code>. If <code>jclass.properties</code> is not
 * found, an empty <code>jclass.properties</code> is created with default
 * properties. <p>The key in <code>jclass.properties</code> should be
 * "[product].[version].[property_name]".
 *  @param product JClass product name. e.g. higrid
 *  @param version JClass product verions. e.g. 310S
 *  @param property_name name of the property that is to be returned.
 */
public static String getProperty(String product, String version,
                                 String property_name)
{
    return getJClassProperty(product + "." + version + "." + property_name);
}

/**
 * Sets a JClass property in <code>jclass.properties</code>.
 * If <code>jclass.properties</code> is not found, an empty
 * <code>jclass.properties</code> is created with default properties. <p>The
 * key in <code>jclass.properties</code> should be "jclass.[property_name]".
 * @param property_name name of the property that is to be set.
 * @param value value of the property to be set.
 */
public static void setProperty(String property_name, String value)
{
    setJClassProperty("jclass." + property_name, value);
}

/**
 * Sets a JClass product-specific property in <code>jclass.properties</code>.
 * If <code>jclass.properties</code> is not found, an empty
 * <code>jclass.properties</code> is created with default properties. <p>The
 * key in <code>jclass.properties</code> should be "[product].[property_name]".
 * @param product JClass product name. e.g. higrid
 * @param property_name name of the property that is to be set.
 * @param value value of the property to be set.
 */
public static void setProperty(String product, String property_name, String value)
{
    setJClassProperty(product + "." + property_name, value);
}

/**
 * Sets a JClass product-and-version-specific property in
 * <code>jclass.properties</code>. If <code>jclass.properties</code> is not
 * found, an empty <code>jclass.properties</code> is created with default
 * properties.  <p>The key in <code>jclass.properties</code> should be
 * "[product].[version].[property_name]".
 * @param product JClass product name. e.g. higrid
 * @param version JClass product verions. e.g. 310S
 * @param property_name name of the property that is to be set.
 * @param value value of the property to be set.
 */
public static void setProperty(String product, String version,
                               String property_name, String value)
{
    setJClassProperty(product + "." + version + "." + property_name, value);
}

/** Gets the path of the <code>jclass.properties</code> file.
 *  The path in ${JCLASS_HOME} is used if it is set.  Otherwise,
 *  The path in the system property "user.home"/jclass is used.
 */
public static String getPropertyFilePath()
{
    return(getPropertyFilePath("JCLASS_HOME"));
}

/** Gets the path of the <code>jclass.properties</code> file.
 *  The path in ${home} is used if it is set.  Otherwise,
 *  The path in the system property "user.home"/jclass is used.
 */
public static String getPropertyFilePath(String home)
{
    String path = null;
    try {
        path = System.getProperty(home);
    }
    catch (Throwable t) {
        System.out.println("WARNING: Error retrieving " + home);
    }

    if (path == null) {
        try {
            // JCLASS_HOME is not defined.  Use "user.home"/jclass
            // user.home may end with "/" or "\".  Thus use "." and
            //  file.separator after it to make sure the path is correct.
            String userHome = new String(System.getProperty("user.home"));
            String separator = System.getProperty("file.separator");
            if (!userHome.endsWith(separator)) {
                userHome = userHome + separator;
            }
            path = userHome + "." + separator + "jclass";
        }
        catch (Throwable t) {
            System.out.println("WARNING: Error retrieving user.home");
        }
    }
    if (path == null) {
        path = ".";
    }
    return(path);
}

/** Gets the path and name of the <code>jclass.properties</code> file.
 *  The path in ${JCLASS_HOME} is used if it is set.  Otherwise,
 *  The path in the system property "user.home"/jclass is used.
 */
public static String getPropertyFile()
{
    return(getPropertyFilePath() + System.getProperty("file.separator") +
           "jclass.properties");
}


/**
 * Gets the properties in <code>jclass.properties</code>.
 * If <code>jclass.properties</code> is not found, an empty
 * <code>jclass.properties</code> is created with default properties.
 */
private static Properties getJClassProperties()
{
    String file = getPropertyFile();
    Properties p = new Properties();
    try {
        p.load(new BufferedInputStream(new FileInputStream(file)));
    }
    catch (FileNotFoundException fnfe) {
        // Create an empty jclass.properties.
        try {
            createPropertyFile();
            p.load(new BufferedInputStream(new FileInputStream(file)));
        }
        catch (IOException ioe) {}
    }
    catch (Exception ex) {
        ex.printStackTrace();
    }

    return(p);
}

/** Gets a property from <code>jclass.properties</code>.
 *  @param key key in the properties file.
 */
private static String getJClassProperty(String key)
{
    Properties p = getJClassProperties();
    return(p.getProperty(key));
}

/** Sets a property from <code>jclass.properties</code>.
 *  @param key key in the properties file.
 *  @param value value of the property.
 */
private static void setJClassProperty(String key, String value)
{
    Properties p = getJClassProperties();
    p.put(key, value);
    try {
        BufferedOutputStream fos =
            new BufferedOutputStream(new FileOutputStream(getPropertyFile()));

        p.store(fos, getPropertyFile());

        // Force property to be saved.
        fos.flush();
        fos.close();
    }
    catch (IOException ioe) {
        ioe.printStackTrace();
    }
}


/** Create an empty <code>jclass.properties</code> file.
 *  If any directories do not exist in the path, they will be created.
 *  @see #getPropertyFile
 */
private static void createPropertyFile() throws IOException
{
    File dir = new File(getPropertyFilePath());
    dir.mkdirs();
    FileWriter fw = new FileWriter(getPropertyFile());
    for (int i = 0; i < defaultProperties.length; i++) {
        fw.write(defaultProperties[i][0] + "=" + defaultProperties[i][1]);
        fw.write(System.getProperty("line.separator"));
    }
    fw.close();
}

/*
 *
 *  FOR DEBUGGING OF GIF IMAGE TESTING ONLY
 *
public static void main(String args[])
{
    com.klg.jclass.util.swing.JCExitFrame f =
        new com.klg.jclass.util.swing.JCExitFrame("Logo Test");
    JPanel panel = new JPanel();
    panel.add(new JLabel(JClassInfo.makeKLIcon((Color)null)));

    f.getContentPane().setLayout(new BorderLayout());
    f.getContentPane().add(panel, BorderLayout.CENTER);
    f.setVisible(true);
    f.pack();
    f.show();
}
*/

}
