/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: OutputProperties.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:42 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.io;

/**
 * Encapsulate the properties that may be needed to save a file (such as a data
 * file or image file)
 */
public class OutputProperties extends Properties
{

protected String    outputFileName;   // Actual output file to write to
protected String    propertyName;     // Name of the property
protected String    saveType;         // Type of file to save
protected int       fileAccess;       // Type of access to file

/**
 * Default constructor.  Use default values for all properties.
 */
public OutputProperties()
{
    this(null, null, null, DEFAULT_ACCESS);
}

/**
 * Constructor which specifies output fileName, propertyName, saveType,
 * and fileAccess
 * @param propertyName Name to write to the property
 * @param fileAccess Type of access to file.  Must be one of
 *     <code>Properties.DEFAULT_ACCESS</code>,
 *     <code>Properties.ABSOLUTE</code>,
 *     <code>Properties.RESOLVING_CLASS</code>,
 *     <code>Properties.URL</code>,
 *     <code>Properties.RELATIVE_URL</code>,
 *     <code>Properties.SERVLET</code>
 */
public OutputProperties(String propertyName, int fileAccess)
{
    this(null, propertyName, null, fileAccess);
}

/**
 * Constructor which specifies output fileName, propertyName, saveType,
 * and fileAccess
 * @param outputFileName Name of actual output file to write to (if
 *     null, nothing is written out)
 * @param propertyName Name to write to the property
 * @param saveType The type of file to save
 * @param fileAccess Type of access to file.  Must be one of
 *     <code>Properties.DEFAULT_ACCESS</code>,
 *     <code>Properties.ABSOLUTE</code>,
 *     <code>Properties.RESOLVING_CLASS</code>,
 *     <code>Properties.URL</code>,
 *     <code>Properties.RELATIVE_URL</code>,
 *     <code>Properties.SERVLET</code>
 */
public OutputProperties(String outputFileName, String propertyName,
                        String saveType, int fileAccess)
{
    setOutputFileName(outputFileName);
    setPropertyName(propertyName);
    setSaveType(saveType);
    setFileAccess(fileAccess);
}

/**
 * Returns the file name of the actual output file to write to (if null, nothing
 * is written)
 * @return The file name of the actual output file to write to
 */
public String getOutputFileName()
{
    return(outputFileName);
}

/**
 * Sets the file name of the actual output file to write to (if null, nothing
 * is written)
 * @param outputFileName The file name of the actual output file to write to
 */
public void setOutputFileName(String outputFileName)
{
    this.outputFileName = outputFileName;
}

/**
 * Gets the name to write to the property.  This name is interpreted based
 * on the file access type.  It could be a filename or a Url.
 * @return The name to write to the property
 */
public String getPropertyName()
{
    return(propertyName);
}

/**
 * Sets the name to write to the property.  This name is interpreted based on
 * the file access type.  It could be a filename or a Url.
 * @param propertyName The new name to write to the property
 */
public void setPropertyName(String propertyName)
{
    this.propertyName = propertyName;
}

/**
 * Gets the type of file to save.  This save type might differ, depending on
 * how this object is used.
 * @return  The type of file to save
 */
public String getSaveType()
{
    return(saveType);
}

/**
 * Sets the type of file to save.  This save type might differ, depending on
 * how this object is used.
 * @param saveType The new way to save the file
 */
public void setSaveType(String saveType)
{
    this.saveType = saveType;
}

/**
 * Gets the type of file access.  Will be one of:
 *     <code>Properties.DEFAULT_ACCESS</code>,
 *     <code>Properties.ABSOLUTE</code>,
 *     <code>Properties.RESOLVING_CLASS</code>,
 *     <code>Properties.URL</code>,
 *     <code>Properties.RELATIVE_URL</code>,
 *     <code>Properties.SERVLET</code>
 * @return The current type of file access.
 */
public int getFileAccess()
{
    return(fileAccess);
}

/**
 * Sets the type of file access.  Must be one of:
 *     <code>Properties.DEFAULT_ACCESS</code>,
 *     <code>Properties.ABSOLUTE</code>,
 *     <code>Properties.RESOLVING_CLASS</code>,
 *     <code>Properties.URL</code>,
 *     <code>Properties.RELATIVE_URL</code>,
 *     <code>Properties.SERVLET</code>
 * @param fileAccess The new file access type
 */
public void setFileAccess(int fileAccess)
{
    if (fileAccess < DEFAULT_ACCESS ||fileAccess > SERVLET) {
        throw new IllegalArgumentException("File access type must be one of:" +
                                           "Properties.DEFAULT_ACCESS," +
                                           "Properties.ABSOLUTE," +
                                           "Properties.RESOLVING_CLASS," +
                                           "Properties.URL," +
                                           "Properties.RELATIVE_URL," +
                                           "Properties.SERVLET");
    }
    this.fileAccess = fileAccess;
}

}
