/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: Properties.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:42 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.io;

import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.io.resources.LocaleBundle;

/**
 * Definitions for load and save properties
 */
public class Properties implements java.io.Serializable, Cloneable
{

/** Ways to access a file */
public final static int DEFAULT_ACCESS  = 0;
public final static int ABSOLUTE        = 1;
public final static int RESOLVING_CLASS = 2;
public final static int URL             = 3;
public final static int RELATIVE_URL    = 4;
public final static int SERVLET         = 5;

// File access type values/strings
public final static int[] fileAccess_values = {
    DEFAULT_ACCESS, ABSOLUTE, RESOLVING_CLASS, URL, RELATIVE_URL, SERVLET
};
public final static String[] fileAccess_strings = {
    "Default", "Absolute", "Resolving_Class", "Url", "Relative_Url", "Servlet"
};
public final static String[] fileAccess_i18n_strings = {
    LocaleBundle.string(LocaleBundle.DEFAULT),
    LocaleBundle.string(LocaleBundle.ABSOLUTE),
    LocaleBundle.string(LocaleBundle.RESOLVING_CLASS),
    LocaleBundle.string(LocaleBundle.URL),
    LocaleBundle.string(LocaleBundle.RELATIVE_URL),
    LocaleBundle.string(LocaleBundle.SERVLET)
};

public final static String DEFAULT_CHARSET_NAME = "JISAutoDetect";
public final static String CHARSET_ASCII = "US-ASCII";
public final static String CHARSET_ISO = "ISO-8859-1";
public final static String CHARSET_UTF_8 = "UTF-8";

public final static String[] charsets = {
    DEFAULT_CHARSET_NAME, CHARSET_ASCII, CHARSET_ISO, CHARSET_UTF_8
};

/**
 * If true, don't throw an exception if internal resources (such as
 * data files) are not found or experience io or parsing problems.
 */
protected boolean ignoreExternalResourceExceptions = false;

/**
 * Get the file access enum from its string representation.  Default access is
 * absolute access. May be over-ridden to provide different behaviour for
 * sub-classes.
 * @param accessString The string version of the fileAccess
 * @param useI18n Use the internationalized strings
 * @return The enum corresponding to the string
 */
public int getFileAccessEnumFromString(String accessString, boolean useI18n)
{
    String[] strings = useI18n ? fileAccess_i18n_strings : fileAccess_strings;
    return(JCTypeConverter.toEnum(accessString, strings, fileAccess_values,
                                  getDefaultAccess()));
}

/**
 * Get the file access string representation from its enum value.
 * @param accessValue The enum version of the fileAccess
 * @param useI18n Use the internationalized strings
 * @return The String reprsentation of the fileAccess enum
 */
public String getFileAccessStringFromEnum(int accessValue, boolean useI18n)
{
    String[] strings = useI18n ? fileAccess_i18n_strings : fileAccess_strings;
    return(JCTypeConverter.fromEnum(accessValue, strings, fileAccess_values));
}

/**
 * Return the default access for this Properties object
 * @return The default access type
 */
public int getDefaultAccess()
{
    return(ABSOLUTE);
}

/**
 * Get the property value which determines whether exceptions caused
 * by loading or saving external resources that are referenced within the xml
 * (data files, for example) are reported.  These exceptions can
 * include JCIOException, IOException, ParserConfigurationException
 * and SAXException.  By default this property is false.
 * @return true, if no exceptions caused by loading external resources
 *         are to be thrown.
 */
public boolean ignoreExternalResourceExceptions()
{
    return(ignoreExternalResourceExceptions);
}

/**
 * Set the property value which determines whether exceptions caused
 * by loading or saving external resources that are referenced within the xml
 * (data files, for example) are reported.  These exceptions can
 * include JCIOException, IOException, ParserConfigurationException
 * and SAXException.
 * @param ignore If true, no exceptions resulting from loading external
 * resources will be thrown when an XML file is parsed.
 */
public void setIgnoreExternalResourceExceptions(boolean ignore)
{
    this.ignoreExternalResourceExceptions = ignore;
}

/**
 * Implement Cloneable.
 */
public Object clone()
{
	Object cloneObject;
	try {
		cloneObject = super.clone();
	}
    catch (CloneNotSupportedException cnse) {
		cloneObject = null;
	}
	return(cloneObject);
}

}
