/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCLegendItem.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:42 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.legend;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;

/**
 * <code>JCLegendItem</code> is used by <code>JCLegend</code> to encapsulate
 * one item in a legend. It is a data structure internal to the legend.  A
 * legend item has up to  two components: a symbol and a text portion.
 */
public class JCLegendItem implements java.io.Serializable {

public Point     pos = 
					new Point(0, 0); // position of legend item
public Point     symbolPos = 
					new Point(0, 0); // position of symbol within legend item
public Point     textPos = 
					new Point(0, 0); // position of text within legend item

public Dimension dim = null;         // size of legend item
public Dimension symbolDim = null;   // size of symbol portion
public Dimension textDim = null;     // size of text portion

public int drawType = JCLegend.NONE;  // type of symbol to draw in the legend
								      // next to the text.  See JCLegend for
									  // possible values.
						
public Rectangle pickRectangle = null; // the rectangle to use for pick
									   // operations.  Optional.  If null,
									   // a default pick rectangle will be
									   // constructed from the pos and dim.

// Note, these may be null
public Object	itemInfo;			   // implementation-defined object
									   // to pass along with the legend item.
									   // Often used to describe data being
									   // associated with this legend item.

public Object    symbol = null;     // the symbol.  Usually null, as symbol
									// is drawn by legend depending on data
									// information.  If the symbol is draw as 
                                    // an image, this member will contain
 									// the image.
public Object    contents = null;   // the text.  Usually a String.

/**
 * Default constructor.
 */
public JCLegendItem() {}

/**
 * Constructor creates new legend item from existing legend item.
 * @param item
 */
public JCLegendItem(JCLegendItem item)
{
	if (item.pos != null) {
		this.pos = new Point(item.pos.x, item.pos.y);
	}
	if (item.symbolPos != null) {
		this.symbolPos = new Point(item.symbolPos.x, item.symbolPos.y);
	}
	if (item.textPos != null) {
		this.textPos = new Point(item.textPos.x, item.textPos.y);
	}
	if (item.dim != null) {
		this.dim = new Dimension(item.dim.width, item.dim.height);
	}
	if (item.symbolDim != null) {
		this.symbolDim = new Dimension(item.symbolDim.width, 
										item.symbolDim.height);
	}
	if (item.textDim != null) {
		this.textDim = new Dimension(item.textDim.width, item.textDim.height);
	}

	this.drawType = item.drawType;

	if (item.pickRectangle != null) {
		this.pickRectangle = new Rectangle(item.pickRectangle.x,
                                           item.pickRectangle.y,
                                           item.pickRectangle.width,
                                           item.pickRectangle.height);
	}

	this.itemInfo = item.itemInfo;
	this.symbol   = item.symbol;
	this.contents = item.contents;
}

}
