/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCLegendRenderer.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:42 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.legend;

import java.awt.Graphics;
import java.awt.Font;
import java.awt.Color;

/** 
 * <code>JCLegendRenderer</code> is an interface implemented by a class that
 * wishes to help render the legend.
 */
public interface JCLegendRenderer {

/** 
 * Used for implementing custom legend item draws.  This method is called
 * when a legend item's draw type has been specified as
 * <code>JCLegend.CUSTOM_ALL</code>.
 * If implemented, the method should draw both the symbol and text portions
 * of the specified legend item.  If <code>CUSTOM_ALL</code> is not used by this
 * legend implementation, simply return out of the method.
 * 
 * @param gc graphics object used to draw the legend
 * @param useFont the font being used in this legend
 * @param thisItem the <code>JCLegendItem</code> object representing the 
 *     legend item to draw
 */
public void drawLegendItem(Graphics gc, Font useFont, JCLegendItem thisItem);

/** 
 * Used for implementing custom symbol draws for a legend item.  This method
 * is called when a legend item's draw type has been specified as
 * <code>JCLegend.CUSTOM_SYMBOL</code>.  If <code>CUSTOM_SYMBOL</code> is not
 * used by this legend implementation, simply return out of the method.
 * 
 * @param gc graphics object used to draw the legend
 * @param useFont the font being used in this legend
 * @param thisItem the <code>JCLegendItem</code> object representing the 
 *     legend item to draw
 */
public void drawLegendItemSymbol(Graphics gc, Font useFont,
                                 JCLegendItem thisItem);

/** 
 * Used for implementing custom text draws for a legend item.  This method
 * is called when a legend item's contents field contains an object other
 * than a String object.
 * 
 * @param gc graphics object used to draw the legend
 * @param useFont the font being used in this legend
 * @param thisItem the <code>JCLegendItem</code> object representing the 
 *     legend item to draw
 */
public void drawLegendItemText(Graphics gc, Font useFont,
                               JCLegendItem thisItem);

/** 
 * Retrieves the outline color to use when drawing the legend item's symbol.
 * If it returns null, the legend will use its own foreground color.
 * 
 * @param thisItem the <code>JCLegendItem</code> object representing the 
 *     legend item in question
 * @return The current outline color
 */
public Color getOutlineColor(JCLegendItem thisItem);

/** 
 * Called to set the provided <code>Graphics</code> object with the appropriate
 * paint settings for use in drawing this legend item's symbol.
 * 
 * @param gc the <code>Graphics</code> object to change
 * @param thisItem the <code>JCLegendItem</code> object representing the legend
 *      item in question
 */
public void setFillGraphics(Graphics gc, JCLegendItem thisItem);

}
