/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: LegendComponentLayout.java,v $ $Revision: 1.2 $
//          $Date: 2006-03-17 21:13:42 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.util.legend;

import com.klg.jclass.util.JCNumberUtil;
import com.klg.jclass.util.swing.JCRectUtil;
import com.klg.jclass.util.swing.JCSwingUtilities;

 

import javax.swing.*;
import java.awt.*;
import java.io.Serializable;
import java.util.Vector;

/**
 *  A layout manager that implements a default layout for large JClass
 *  components, such as JClass Chart and JClass Gauge, that  comprise a
 *  drawing area, a header, a footer, and a legend (a JCLegend object).
 *  This layout places the drawing area class in the leftover space after
 *  the legend,  the header and the footer have been layed out, unless
 *  specific layout hints were set on the object in JCChart. Any component
 *  using this layout manager must implement
 *  <code>LegendComponentLayoutUser</code>.
 */
public class LegendComponentLayout implements LayoutManager, Serializable
{

private int offset = 5;
private int outerSpacingWidth = 0;
private int outerSpacingHeight = 0;

/**
 * Adds the specified component with the specified name to the layout.
 * Not used in this layout.
 * @param name the name of the component
 * @param comp the component to be added
 */
public void addLayoutComponent(String name, Component comp) {}

/**
 * Removes the specified component from the layout. Not used in this layout.
 * @param comp the component to be removed
 */
public void removeLayoutComponent(Component comp) {}

/**
 * Determines the preferred size of the container by laying out
 * the container's header, footer, legend, and drawing area.
 * If container does not implement <code>LegendComponentLayoutUser</code>, this
 * method returns a size of zero.
 * @param parent the container in which to do the layout. Must be a class
 *               that implements <code>LegendComponentLayoutUser</code>
 * @return the preferred dimensions of the container
 */
public Dimension preferredLayoutSize(Container parent)
{
    LegendComponentLayoutUser jccomponent = null;

    if (parent instanceof LegendComponentLayoutUser) {
        jccomponent = (LegendComponentLayoutUser)parent;
    }
    else {
        return(new Dimension(0, 0));
    }

    Graphics parentGraphics = null;

 

    Dimension prefSize = null;
    try {
        JComponent header = jccomponent.getHeader();
        JComponent footer = jccomponent.getFooter();
        JComponent jccomponentArea = jccomponent.getComponentArea();
        JCLegend legend = jccomponent.getLegend();

        Insets margins = parent.getInsets();

        prefSize = jccomponentArea.getPreferredSize();
        if (header != null && header.isVisible()) {
            Dimension headerPrefSize;
            if (parentGraphics != null) {
                headerPrefSize =
                    JCSwingUtilities.getPreferredSizeFromGraphics(header,
                                                                  parentGraphics,
                                                                  false);
            }
            else {
                headerPrefSize = header.getPreferredSize();
            }
            prefSize.width = Math.max(headerPrefSize.width, prefSize.width);
            prefSize.height += headerPrefSize.height + offset;
        }
        if (footer != null && footer.isVisible()) {
            Dimension footerPrefSize;
            if (parentGraphics != null) {
                footerPrefSize =
                    JCSwingUtilities.getPreferredSizeFromGraphics(footer,
                                                                  parentGraphics,
                                                                  false);
            }
            else {
                footerPrefSize = footer.getPreferredSize();
            }
            prefSize.width = Math.max(footerPrefSize.width, prefSize.width);
            prefSize.height += footerPrefSize.height + offset;
        }
        if (legend != null) {
            Dimension lPrefSize = legend.getPreferredSize();
            if (legend.isVisible()) {
                int anc = legend.getAnchor();
                switch (anc) {
                case JCLegend.NORTHWEST:
                case JCLegend.SOUTHWEST:
                case JCLegend.NORTHEAST:
                case JCLegend.SOUTHEAST:
                    prefSize.width += lPrefSize.width + offset;
                    prefSize.height += lPrefSize.height + offset;
                    break;
                case JCLegend.NORTH:
                case JCLegend.SOUTH:
                    prefSize.width = Math.max(prefSize.width, lPrefSize.width);
                    prefSize.height += lPrefSize.height + offset;
                    break;
                case JCLegend.WEST:
                case JCLegend.EAST:
                    prefSize.width += lPrefSize.width + offset;
                    prefSize.height = Math.max(prefSize.height,
                                               lPrefSize.height);
                    break;
                }
            }
        }
        prefSize.width += margins.left + margins.right;
        prefSize.height += margins.top + margins.bottom;
    }
    catch (Exception e) {
        // Just making sure.
        e.printStackTrace(System.out);
    }

 
    return(prefSize);
}

/**
 * Determines the minimum size of the container using this layout.
 * If container does not implement <code>LegendComponentLayoutUser</code>, this method
 * returns a size of zero.
 * @param parent the container in which to do the layout. Must be a class
 *               that implements <code>LegendComponentLayoutUser</code>
 * @return the minimum dimensions of this container
 */
public Dimension minimumLayoutSize(Container parent)
{
    return(preferredLayoutSize(parent));
}

/**
 * Lays out the specified container. If container does not implement
 * <code>LegendComponentLayoutUser</code>, no layout is done.
 * @param parent the container in which to do the layout. Must be a class
 *               that implements <code>LegendComponentLayoutUser</code>
 */
public void layoutContainer(Container parent)
{

    LegendComponentLayoutUser jccomponent = null;
    if (parent instanceof LegendComponentLayoutUser) {
        jccomponent = (LegendComponentLayoutUser)parent;
    }
    else {
        return;
    }

    Dimension sz = new Dimension();

    sz.width  = jccomponent.getDrawingAreaWidth();
    sz.height = jccomponent.getDrawingAreaHeight();

    Graphics parentGraphics = null;

 

    JComponent header = jccomponent.getHeader();
    JComponent footer = jccomponent.getFooter();
    JComponent jccomponentArea = jccomponent.getComponentArea();
    JCLegend legend = jccomponent.getLegend();

    Insets margins = parent.getInsets();

    Rectangle componentAreaHints = jccomponent.getLayoutHints(jccomponentArea);
    Rectangle legendHints = jccomponent.getLayoutHints(legend);
    Rectangle headerHints = jccomponent.getLayoutHints(header);
    Rectangle footerHints = jccomponent.getLayoutHints(footer);

    Dimension head;
    if (header == null) {
        head = new Dimension(0, 0);
    }
    else if (parentGraphics != null) {
        head = JCSwingUtilities.getPreferredSizeFromGraphics(header,
                                                             parentGraphics,
                                                             false);
    }
    else {
        head = header.getPreferredSize();
    }
    if (header != null && header.isVisible() && header instanceof JLabel) {
        JCNumberUtil.fudgeItalicLabel(header.getFont(), head);
    }

    Dimension foot;
    if (footer == null) {
        foot = new Dimension(0, 0);
    }
    else if (parentGraphics != null) {
        foot = JCSwingUtilities.getPreferredSizeFromGraphics(footer,
                                                             parentGraphics,
                                                             false);
    }
    else {
        foot = footer.getPreferredSize();
    }
    if (footer != null && footer.isVisible() && footer instanceof JLabel) {
        JCNumberUtil.fudgeItalicLabel(footer.getFont(), foot);
    }

    Dimension legd = legend.getPreferredSize();

    // Layout philosophy - guarantee space for legend, header and footer.
    // JCChart gets remainder of space.

    Dimension ca = jccomponentArea.getSize();
    Point co = jccomponentArea.getLocation();
    Point lo = legend.getLocation();
    Point ho = (header == null ? new Point(0, 0) : header.getLocation());
    Point fo = (footer == null ? new Point(0, 0) : footer.getLocation());

    // Apply hints to location, size if >= 0
    if (legendHints.x != Integer.MAX_VALUE) {
        lo.x = legendHints.x;
    }
    if (legendHints.y != Integer.MAX_VALUE) {
        lo.y = legendHints.y;
    }
    if (legendHints.width != Integer.MAX_VALUE) {
        legd.width = legendHints.width;
    }
    if (legendHints.height != Integer.MAX_VALUE) {
        legd.height = legendHints.height;
    }

    if (componentAreaHints.x != Integer.MAX_VALUE) {
        co.x = componentAreaHints.x;
    }
    if (componentAreaHints.y != Integer.MAX_VALUE) {
        co.y = componentAreaHints.y;
    }
    if (componentAreaHints.width != Integer.MAX_VALUE) {
        ca.width = componentAreaHints.width;
    }
    if (componentAreaHints.height != Integer.MAX_VALUE) {
        ca.height = componentAreaHints.height;
    }

    if (headerHints.x != Integer.MAX_VALUE) {
        ho.x = headerHints.x;
    }
    if (headerHints.y != Integer.MAX_VALUE) {
        ho.y = headerHints.y;
    }
    if (headerHints.width != Integer.MAX_VALUE) {
        head.width = headerHints.width;
    }
    if (headerHints.height != Integer.MAX_VALUE) {
        head.height = headerHints.height;
    }

    if (footerHints.x != Integer.MAX_VALUE) {
        fo.x = footerHints.x;
    }
    if (footerHints.y != Integer.MAX_VALUE) {
        fo.y = footerHints.y;
    }
    if (footerHints.width != Integer.MAX_VALUE) {
        foot.width = footerHints.width;
    }
    if (footerHints.height != Integer.MAX_VALUE) {
        foot.height = footerHints.height;
    }

    co.x -= margins.left;
    co.y -= margins.top;
    lo.x -= margins.left;
    lo.y -= margins.top;
    ho.x -= margins.left;
    ho.y -= margins.top;
    fo.x -= margins.left;
    fo.y -= margins.top;

    outerSpacingWidth = sz.width / 30;
    outerSpacingHeight = sz.height / 30;

    Rectangle win = new Rectangle(0, 0, sz.width, sz.height);

    /* Calculate the position of the legend. */
    int anchor = legend.getAnchor();

    // Calculate legend top if no hint set
    if (legendHints.y == Integer.MAX_VALUE) {
        switch (anchor & JCLegend.NORTHSOUTH_MASK) {
        case JCLegend.NORTH:
            /*
             * Check to see if header is actually significant in location
             * calculation.
             */
            switch (anchor & JCLegend.EASTWEST_MASK) {
            case JCLegend.EAST:
            case JCLegend.WEST:
                lo.y = win.y + outerSpacingHeight;
                break;
            default :
                if (header != null && header.isVisible())
                    lo.y = (win.y + (2 * outerSpacingHeight) +
                           head.height);
                else
                    lo.y = win.y + outerSpacingHeight;
                break;
            }
            break;
        case JCLegend.SOUTH:
            /*
             * Check to see if footer is actually significant in location
             * calculation.
             */
            switch (anchor & JCLegend.EASTWEST_MASK) {
            case JCLegend.EAST:
            case JCLegend.WEST:
                lo.y = (win.y + win.height) - outerSpacingHeight -
                       legd.height;
                break;
            default :
                if (footer != null && footer.isVisible())
                    lo.y = (win.y + win.height) -
                           (2 * outerSpacingHeight) -
                           (legd.height + foot.height);
                else
                    lo.y = (win.y + win.height) - outerSpacingHeight -
                           legd.height;
                break;
            }
            break;
        default :
            lo.y = (win.height - legd.height) / 2;
            break;
        }
        lo.y = JCNumberUtil.clamp(lo.y, 0, win.height);
    }

    // Calculate legend left if no hint set
    if (legendHints.x == Integer.MAX_VALUE) {
        switch (anchor & JCLegend.EASTWEST_MASK) {
        case JCLegend.EAST:
            lo.x = (win.x + win.width) - outerSpacingWidth - legd.width;
            break;
        case JCLegend.WEST:
            lo.x = win.x + outerSpacingWidth;
            break;
        default :
            lo.x = (win.width - legd.width) / 2;
            break;
        }
        lo.x = JCNumberUtil.clamp(lo.x, 0, win.width);
    }

    /*
     *  (DEFAULT CASE)
     *  Calculate the position of the header. This is done assuming the
     *  header is centered in the window, although in reality it is
     *  centred over the graph. In order to position the graph, however,
     *  you need to know where the header is.
     */
    Rectangle hdr_pos = new Rectangle(ho, head);

    // Calc header position if no hints set
    if (headerHints.x == Integer.MAX_VALUE) {
        hdr_pos.x = (win.width - hdr_pos.width) / 2;
        hdr_pos.x = JCNumberUtil.clamp(hdr_pos.x, 0, win.width);
    }
    if (headerHints.y == Integer.MAX_VALUE) {
        hdr_pos.y = win.y + outerSpacingHeight;
        hdr_pos.y = JCNumberUtil.clamp(hdr_pos.y, 0, win.height);
    }

    /*
     *  (DEFAULT CASE)
     *  Calculate the position of the footer. This is done assuming the
     *  footer is centred in the window, although in reality it is
     *  centred over the graph. In order to position the graph, however,
     *  you need to know where the footer is.
     */
    Rectangle ftr_pos = new Rectangle(fo, foot);

    // Calc footer position if no hints set
    if (footerHints.x == Integer.MAX_VALUE) {
        ftr_pos.x = (win.width - ftr_pos.width) / 2;
        ftr_pos.x = JCNumberUtil.clamp(ftr_pos.x, 0, win.width);
    }
    if (footerHints.y == Integer.MAX_VALUE) {
        ftr_pos.y = win.y + win.height - ftr_pos.height - outerSpacingHeight;
        ftr_pos.y = JCNumberUtil.clamp(ftr_pos.y, 0, win.height);
    }

    Rectangle save_pos = new Rectangle(co, ca);
    if (componentAreaHints.x == Integer.MAX_VALUE     ||
        componentAreaHints.y == Integer.MAX_VALUE     ||
        componentAreaHints.width == Integer.MAX_VALUE ||
        componentAreaHints.height == Integer.MAX_VALUE)
    {
        /*
         *  Take the container's dimensions and remove the space allocated for
         *  the header, footer, and legend. What you are left with is a
         *  non-overlapping list of rectangles.
         */
        Vector rl = new Vector();

        JCRectUtil.rlInit(win, rl);
        if (legend.isVisible()) {
            JCRectUtil.rlRemove(new Rectangle(lo, legd), rl, rl);
        }

        Rectangle diff_pos;
        if (header != null && header.isVisible()) {
            diff_pos = new Rectangle(hdr_pos.x, hdr_pos.y,
                                     hdr_pos.width, hdr_pos.height);
            if (headerHints.x == Integer.MAX_VALUE) {
                /* If header x position is default, header will be centred
                 * over the graph, so we make sure that the graph
                 * doesn't intersect the header by extending the width of
                 * the header rectangle to the window edge.
                 */
                diff_pos.x = 0;
                diff_pos.width = win.width;
            }
            JCRectUtil.rlRemove(diff_pos, rl, rl);
        }

        if (footer != null && footer.isVisible()) {
            diff_pos = new Rectangle(ftr_pos.x, ftr_pos.y,
                                     ftr_pos.width, ftr_pos.height);
            if (footerHints.x == Integer.MAX_VALUE) {
                /* If footer x position is default, footer will be centred
                 * below the graph, so we make sure that the graph
                 * doesn't intersect the footer by extending the width of
                 * the footer rectangle to the window edge.
                 */
                diff_pos.x = 0;
                diff_pos.width = win.width;
            }
            JCRectUtil.rlRemove(diff_pos, rl, rl);
        }

        /*
         *  Calculate the largest rectancle left on the container by analyzing
         *  the rect list.
         */
       Rectangle lg_r;
       if (rl.size() > 0) {
            lg_r = JCRectUtil.largestRect(rl);
        }
        else {
            lg_r = new Rectangle(0, 0, 0, 0);
        }

        int x_boundary = lg_r.width / 40;
        int y_boundary = lg_r.height / 40;

        // If rectangle at edges, outer boundary
        ca.width = lg_r.width;
        if (lg_r.x == win.x) {
            co.x = lg_r.x + outerSpacingWidth;
            ca.width -= outerSpacingWidth;
        }
        else {
            co.x = lg_r.x + x_boundary;
            ca.width -= x_boundary;
        }
        if (lg_r.x + lg_r.width == win.x + win.width) {
            ca.width -= outerSpacingWidth;
        }
        else {
            ca.width -= x_boundary;
        }

        ca.height = lg_r.height;
        if (lg_r.y == win.y) {
            co.y = lg_r.y + outerSpacingHeight;
            ca.height -= outerSpacingHeight;
        }
        else {
            co.y = lg_r.y + y_boundary;
            ca.height -= y_boundary;
        }
        if (lg_r.y + lg_r.height == win.y + win.height) {
            ca.height -= outerSpacingHeight;
        }
        else {
            ca.height -= y_boundary;
        }
    }

    if (componentAreaHints.x != Integer.MAX_VALUE) {
        if (componentAreaHints.width == Integer.MAX_VALUE) {
            ca.width -= save_pos.x - co.x;
            ca.width = Math.max(0, ca.width);
        }
        co.x = save_pos.x;
    }
    if (componentAreaHints.y != Integer.MAX_VALUE) {
        if (componentAreaHints.height == Integer.MAX_VALUE) {
            ca.height -= save_pos.y - co.y;
            ca.height = Math.max(0, ca.height);
        }
        co.y = save_pos.y;
    }
    if (componentAreaHints.height != Integer.MAX_VALUE) {
        ca.height = save_pos.height;
    }
    if (componentAreaHints.width != Integer.MAX_VALUE) {
        ca.width = save_pos.width;
    }

    /*
     *  Small initial window size can cause negetive values,
     *  which result in unreasonable unsigned values.
     */
    if (ca.width > Integer.MAX_VALUE) {
        ca.width = 0;
    }
    if (ca.height > Integer.MAX_VALUE) {
        ca.height = 0;
    }

    /*
     *  (DEFAULT CASE)
     *  Calculate the true position of the header.
     */
    if (headerHints.x == Integer.MAX_VALUE) {
        ho.x = co.x +
            (ca.width - hdr_pos.width) / 2;
        ho.x = Math.max(0, ho.x);
    }
    if (headerHints.y == Integer.MAX_VALUE) {
        ho.y = hdr_pos.y;
    }

    /*
     *  (DEFAULT CASE)
     *  Calculate the true position of the footer.
     */
    if (footerHints.x == Integer.MAX_VALUE) {
        fo.x = co.x +
            (ca.width - ftr_pos.width) / 2;
        fo.x = Math.max(0, fo.x);
    }
    if (footerHints.y == Integer.MAX_VALUE) {
        fo.y = ftr_pos.y;
    }

    int offsety = 0;
    legend.setBounds(lo.x + margins.left, lo.y + margins.top + offsety,
                     legd.width, legd.height);
    if (header != null) {
        header.setBounds(ho.x + margins.left, ho.y + margins.top + offsety,
                         head.width, head.height);
    }
    if (footer != null) {
        footer.setBounds(fo.x + margins.left, fo.y + margins.top + offsety,
                         foot.width, foot.height);
    }
    jccomponentArea.setBounds(co.x + margins.left, co.y + margins.top + offsety,
                              ca.width, ca.height);

 
}

}
