/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: LegendComponentLayoutUser.java,v $ $Revision: 1.2 $ 
//          $Date: 2006-03-17 21:13:43 $ $Locker:  $ Quest Software Inc. 

package com.klg.jclass.util.legend;

import java.awt.Rectangle;
import java.awt.Dimension;
import javax.swing.JComponent;

/**
 *  A layout manager shared by chart and gauge.  That layout
 *  places the JCChartArea or JCGaugeArea class in the leftover space after the
 *  JCLegend and the header and footer JComponent's have been layed out, unless
 *  specific layout hints were set on the object in JCChart/JCGauge.
 */
public interface LegendComponentLayoutUser
{

/**
 * Get the height of the drawing area represented by this component.
 */
public int getDrawingAreaHeight();

/**
 * Get the width of the drawing area represented by this component.
 */
public int getDrawingAreaWidth();

/**
 * Gets the Header property.
 * The Header property controls the object that controls the
 * display of the header.
 * @return head the JComponent object associated with the header.  By
 * default it is an instance of JLabel.
 */
public JComponent getHeader();

/**
 * Sets the Footer property.
 * The Footer property controls the object that controls the
 * display of the footer.
 * @return the JComponent object associated with the footer.  By
 * default it is an instance of JLabel.
 */
public JComponent getFooter();

/**
 * Return the JClass component's sub-component on which the actual
 * drawing of the chart or gauge will be performed.
 * @return the JCComponent on which to draw
 */
public JComponent getComponentArea();

/**
 * Gets the Legend property
 * The Legend property controls the object that controls
 * the display of the legend.
 * @return the JCLegend object associated with the legend
 */
public JCLegend getLegend();

/**
 * Get layout hints for the specified component child.
 * @param child chart child -- either the componentArea, legend, header, or
 * footer.
 * @return Rectangle object containing the desired layout hints.  Null is
 * child is not one of the valid children of chart.
 */
public Rectangle getLayoutHints(JComponent child);

/**
 * Return the preferred size of this component.
 * @return the preferred size of this component
 */
public Dimension getPreferredSize();

}
