/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: AbstractSpinBox.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:44 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.swing;

import com.klg.jclass.util.JCEnvironment;
import com.klg.jclass.util.JCListenerList;

import com.klg.jclass.util.value.JCValueEvent;
import com.klg.jclass.util.value.JCValueListener;
import com.klg.jclass.util.value.JCValueModel;
import com.klg.jclass.util.value.ObjectValueModel;

import javax.swing.*;
import javax.swing.border.AbstractBorder;
import javax.swing.border.Border;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.UIResource;
import javax.swing.plaf.basic.BasicArrowButton;
import javax.swing.plaf.metal.MetalLookAndFeel;
import java.awt.*;
import java.awt.event.*;
import java.io.Serializable;
import java.util.Enumeration;
import java.util.Locale;

/**
 * Abstract base for creating spinboxes.
 */
public abstract class AbstractSpinBox extends JComponent {

protected boolean 			 continuousScroll = false;
protected JCSpinBoxEditor  	 editor;
protected JCSpinBoxRenderer  renderer;
protected Font               userFont = null;
protected JPanel             renderPanel = new JPanel(new BorderLayout());
protected JButton			 downArrow, upArrow;
protected boolean   		 arrowKeySpinningAllowed = true;
protected Listener 			 l = new Listener();
protected RendererListener   rl = new RendererListener();
protected Timer 			 buttonTimer;
protected JButton 			 currentButton = null;
protected JCValueModel       valueModel;
protected JCListenerList     valueListeners;
protected boolean            editable = true;

// Do we need to compute preferred size?
private boolean              recalcPrefSize = true;

static protected int         minimumHeight = 20;
static protected boolean     editorIsValidateRoot = true;
static final protected       Border metalBorder = new MetalBorder();
static final                 Dimension defaultPreferredSize = new Dimension(1,1);

protected final static int NONE      = 0;
protected final static int INCREMENT = 1;
protected final static int DECREMENT = -1;

/**
 * Constructor for the abstract protion of the spinbox.
 */
public AbstractSpinBox(JCValueModel model, JCSpinBoxEditor editor,
                       JCSpinBoxRenderer renderer)
{
	super.setLayout(null);
	setValueModel(model);
	setEditor(editor);
    setRenderer(renderer);

	// Add the arrows
	configureArrowButtons();
}

/**
 * Creates an empty Spin Control.
 */
public AbstractSpinBox(JCValueModel model, JCSpinBoxEditor editor)
{
    this(model, editor, new DefaultRenderer());
}

/**
 * Creates an empty Spin Control.
 */
public AbstractSpinBox()
{
	this(new ObjectValueModel(), new DefaultEditor());
}

/**
 * Should arrow buttons continuously scroll when the
 * the mouse button is held down, or discretely
 * (jump) scroll on each mouse click?
 *
 * @param continuousScroll a boolean, true
 * means scroll continuously while the mouse is held down,
 * false (the default) means scroll one value per mouse click.
 */
public void setContinuousScroll(boolean continuousScroll)
{
	this.continuousScroll = continuousScroll;
	buttonTimer.removeActionListener(l);
	if (continuousScroll == true) {
		buttonTimer.addActionListener(l);
	}
}

/**
 * Return the continuousScroll property.
 *
 * @return true if continuous scrolling is in effect, false otherwise.
 */
public boolean isContinuousScroll()
{
	return(continuousScroll);
}

/**
 * Set UI.
 */
public void setUI(ComponentUI ui)
{
	super.setUI(ui);
	configureArrowButtons();
}

/**
 * Update UI.
 */
public void updateUI()
{
	setUI(null);
}

/**
 * Adds a listener to listen for changes to the value.
 */
public void addValueListener(JCValueListener listener)
{
	valueListeners = JCListenerList.add(valueListeners, listener);
}

/**
 * Removes a listener that listens for changes to the value.
 */
public void removeValueListener(JCValueListener listener)
{
	valueListeners = JCListenerList.remove(valueListeners, listener);
}

/**
 * Set the value model.
 */
public void setValueModel(JCValueModel model)
{
	JCValueModel old_model = getValueModel();
	if (old_model == model) {
		return;
	}
	if (old_model != null) {
		old_model.removeValueListener(l);
	}
	this.valueModel = model;
	firePropertyChange("valueModel", old_model, model);
	valueModel.addValueListener(l);
}

/**
 * Get the value model.
 */
public JCValueModel getValueModel()
{
	return(valueModel);
}

/**
 * Set the value displayed by the spin box.
 * @param o The new value
 */
public void setValue(Object o)
{
	if (compareToValue(o) == false) {
		// Only set the value if it is not the same
		JCValueModel model = getValueModel();
		if (model != null) {
            if (o != null) {
       			model.setValue(o);
            }
		}
	}
}

/**
 * Compares any object to the stored value.
 */
public boolean compareToValue(Object compare_value)
{
	JCValueModel model = getValueModel();
	if (model == null) {
		return(false);
	}
	Object value = model.getValue();
	if (value == null) {
		if (compare_value == null) {
			return(true);
		}
		else {
			return(false);
		}
	}
	return(value.equals(compare_value));
}

/**
 * Creates the increment arrow.
 */
protected JButton createIncrArrow()
{
	String lf = UIManager.getLookAndFeel().getID();
	if (lf.equals("Metal")) {
		return(new MetalArrowButton(ArrowIcon.UP));
	}
	else {
		return(new ArrowButton(BasicArrowButton.NORTH));
	}
}

/**
 * Creates the decrement arrow.
 */
protected JButton createDecrArrow()
{
	String lf = UIManager.getLookAndFeel().getID();
	if (lf.equals("Metal")) {
		return(new MetalArrowButton(ArrowIcon.DOWN));
	}
	else {
		return(new ArrowButton(BasicArrowButton.SOUTH));
	}
}

/**
 * Creates and sets up the spin arrows.
 */
public void configureArrowButtons()
{
	// Remove old arrow buttons.
	if (upArrow != null) {
		remove(upArrow);
		upArrow = null;
	}
	if (downArrow != null) {
		remove(downArrow);
		downArrow = null;
	}

	// Stop the buttonTimer if it is exists.
	if (buttonTimer != null) {
		buttonTimer.stop();
	}

	buttonTimer = null;

	// add the arrows
	add(upArrow = createIncrArrow());
	add(downArrow = createDecrArrow());

  	setEnabled(isEnabled());

	// Setup the arrows so focus will not go the buttons when clicked on.
	upArrow.setRequestFocusEnabled(false);
	upArrow.setPreferredSize(new Dimension(14,14));
	downArrow.setRequestFocusEnabled(false);
	downArrow.setPreferredSize(new Dimension(14,14));

	upArrow.removeMouseListener(l);
	downArrow.removeMouseListener(l);

	upArrow.addMouseListener(l);
	downArrow.addMouseListener(l);

    buttonTimer = new Timer(200, l);
    if (!continuousScroll) {
        // Comply with scroll property
    	buttonTimer.removeActionListener(l);
    }
    buttonTimer.setInitialDelay(0);
}

/**
 * Request the focus for this component.
 */
public void requestFocus()
{
	// A container should never have focus so pass this request on to
	// our first traversable child; if we have no children then we do nothing
    if (editable) {
        if (editor.getEditorComponent() != null &&
		    JCEnvironment.getJavaVersion() < 140)
		{
            editor.getEditorComponent().requestFocus();
			return;
        }
	}
    super.requestFocus();
}

/**
 * Determines whether the <code>JCSpinBox</code> field is editable. An editable
 * <code>JCSpinBox</code> allows the user to type into the field or selected an
 * item from the list to initialize the field, after which it can be edited.
 * (The editing affects only the field, the list item remains intact.) A non
 * editable <code>JCSpinBox</code> displays the selected item inthe field, but
 * the selection cannot be modified.
 *
 * @param aFlag A boolean value, where true indicates that the field is editable
 */
public void setEditable(boolean aFlag)
{
	boolean changed = (aFlag != editable);
	editable = aFlag;
	if (changed) {
        swapEditorRenderer();
		firePropertyChange("editable", !editable, editable);
	}
}

/**
 * Returns true if the JCSpinBox is editable.
 *
 * @return true if the JCSpinBox is editable, else false
 */
public boolean isEditable()
{
	return(editable);
}

/**
 * Set the enable state of this component.
 */
public void setEnabled(boolean b)
{
    super.setEnabled(b);
    if ((downArrow != null) && (upArrow != null)) {
        downArrow.setEnabled(b);
        upArrow.setEnabled(b);
    }

    if (this.isEditable()) {
        if (editor.getEditorComponent() != null) {
            editor.getEditorComponent().setEnabled(b);
        }
    }
}

/**
 * Test to see if the component is enabled.
 */
public boolean isEnabled()
{
    return(super.isEnabled());
}

/**
 * Test to see if this component is focus traversable
 */
public boolean isFocusTraversable()
{
    return(!isEditable());
}

/**
 * Sets the editor used to paint and edit the selected item in the
 * <code>JCSpinBox</code> field. The editor is used only if the receiving
 * JCSpinBox is editable. If not editable, the combo box uses the renderer to
 * paint the selected item.
 *
 * @param editor that displays the selected item
 * @see #setRenderer
 */
public void setEditor(JCSpinBoxEditor editor)
{
	JCSpinBoxEditor old_editor = editor;

	// Remove the previous editor
    if (this.editor != null) {
        Component editorComponent = this.editor.getEditorComponent();
        remove(editorComponent);
        editorComponent.removeFocusListener(l);
        editorComponent.removeKeyListener(l);
    }

    this.editor = editor;
    Component editorc = editor.getEditorComponent();
    if (userFont != null) {
        editorc.setFont(userFont);
    }

    if (isEditable()) {
        setEnabled(isEnabled());
        add(editorc);
        editorc.addFocusListener(l);
        editorc.addKeyListener(l);
    }

	firePropertyChange("editor", old_editor, editor);
	// Force relayout
	if (getParent() != null) {
		getParent().invalidate();
	}
}

/**
 * Get the editor being used by this spinbox.
 */
public JCSpinBoxEditor getEditor()
{
	return(editor);
}

/**
 * Set the renderer for this spinbox to use.
 */
public void setRenderer(JCSpinBoxRenderer aRenderer)
{
    JCSpinBoxRenderer oldRenderer = renderer;

    if (this.renderer != null) {
        Component oldComponent = this.renderer.getComponent(this,null);
        renderPanel.remove(oldComponent);
        oldComponent.removeMouseListener(rl);
    }
    renderer = aRenderer;

    JCValueModel vm = getValueModel();
    Object value = (vm == null) ? null : vm.getValue();
    Component c = renderer.getComponent(this, value);
    if (userFont != null) {
        c.setFont(userFont);
    }
    renderPanel.add(c, BorderLayout.CENTER);
    c.enableInputMethods(true);
    c.addMouseListener(rl);

    if (!isEditable()) {
        remove(renderPanel);
        add(renderPanel);
        this.addFocusListener(l);
        this.addKeyListener(l);
    }

    firePropertyChange( "renderer", oldRenderer, renderer );
	if (getParent() != null) {
		getParent().invalidate();
	}
}

/**
 * Get the renderer being used by this spinbox
 */
public JCSpinBoxRenderer getRenderer()
{
    return(renderer);
}

/* Swap the renderer and editor based on editable condition*/
protected void swapEditorRenderer() {

    Component e = editor.getEditorComponent();
    remove(e);
    e.removeFocusListener(l);
    e.removeKeyListener(l);

    remove(renderPanel);
    this.removeFocusListener(l);
    this.removeKeyListener(l);

    if (this.isEditable()) {
        this.setRequestFocusEnabled(false);
        add(e);
        e.setEnabled(isEnabled());
        e.addFocusListener(l);
        e.addKeyListener(l);
    }
    else {
        this.setRequestFocusEnabled(true);
        add(renderPanel);
        this.addFocusListener(l);
        this.addKeyListener(l);
    }
    repaint();
}

/** Gets the decrement (down) button. */
public JButton getDecrementArrow()
{
	return(downArrow);
}

/** Gets the increment (up) button. */
public JButton getIncrementArrow()
{
	return(upArrow);
}

/**
 * Returns whether or not the up/down arrow keys can be used for spinning.
 * @see #setArrowKeySpinningAllowed
 */
public boolean isArrowKeySpinningAllowed()
{
	return(arrowKeySpinningAllowed);
}

/**
 * Sets whether or not the up/down arrow keys can be used for spinning.
 * This is <code>true</code> by default, and useful to turn off if you wish
 * to use the arrow keys for traversal.
 */
public void setArrowKeySpinningAllowed(boolean value)
{
	arrowKeySpinningAllowed = value;
}

/**
 * Returns the sum of the text field's and arrow button's preferred widths.
 * Commonly overridden by implementing class depeding on its data metrics.
 */
protected int preferredWidthOfView()
{
    int width = 0;

	Component c = null;
    if (isEditable()) {
        c = editor.getEditorComponent();
    }
    else {
        c = renderer.getComponent(this, getValueModel().getValue());
    }
	if (c != null) {
	    width = c.getPreferredSize().width;
	}
	return(width + upArrow.getPreferredSize().width);
}

/**
 * Returns the preferred size of the Spin Field.
 */
public Dimension getPreferredSize()
{
	Dimension size = super.getPreferredSize();
	if ((size == null) || size.equals(defaultPreferredSize) || recalcPrefSize) {
		// The border must be properly set before the sizes can be determined.
		doBorder();
        Insets insets = renderPanel.getInsets();
        size = new Dimension(preferredWidthOfView() +
                             upArrow.getPreferredSize().width +
                             insets.left + insets.right,
                             preferredHeightOfView());
		setPreferredSize(size);
		recalcPrefSize = false;
	}
	return(size);
}

/**
 * Returns the text field's preferred height. Designed to be overridden
 */
protected int preferredHeightOfView()
{
    int height = 0;

	Component c = null;
    if (isEditable()) {
		c = editor.getEditorComponent();
    }
    else {
		c = renderer.getComponent(this, getValueModel().getValue());
        Insets insets = renderPanel.getInsets();
        height = insets.top + insets.bottom;
    }
	if (c != null) {
		height += c.getPreferredSize().height;
	}
    height = Math.max(height, minimumHeight);
    return(height);
}

/** Sets background of the Spin field */
public void setBackground(Color background)
{
	super.setBackground(background);
//	downArrow.setBackground(background);
//	upArrow.setBackground(background);
}

/** Sets foreground color of the Spin field */
public void setForeground(Color foreground)
{
	super.setForeground(foreground);
//	downArrow.setForeground(foreground);
//	upArrow.setForeground(foreground);
}

/**
 * Sets the font for this component.  Overrides JComponent's setFont to make
 * sure that the font is passed on to the editor/renderer and to make sure the
 * preferred size gets recalculated.
 *
 * @param f The desired <code>Font</code> for this component
 * @see javax.swing.JComponent#setFont
 */
public void setFont(Font f)
{
    super.setFont(f);
    userFont = f;
    Component c = editor.getEditorComponent();
    if (c != null && userFont != null) {
        c.setFont(userFont);
    }
    c = renderer.getComponent(this, getValueModel().getValue());
    if (c != null && userFont != null) {
        c.setFont(userFont);
    }
    recalcPrefSize = true;
}

/**
 * Sets the locale of this component.  Overrides the setLocale of
 * java.awt.Component to make sure the preferred sized gets recalculated.
 * @param l the locale to become this component's locale
 * @see java.awt.Component#setLocale
 */
public void setLocale(Locale l)
{
    super.setLocale(l);
    recalcPrefSize = true;
}

/**
 * Positions the text field and arrow buttons.
 */
public synchronized void doLayout()
{
	Dimension size = getSize();

	int text_height = minimumHeight;

	if (text_height < size.height) {
		text_height = size.height;
	}

	int btn_height = text_height / 2;
    int btn_width  = btn_height;
	int btn_x      = getSize().width - btn_width;

	editor.getEditorComponent().setBounds(0, 0, getSize().width - btn_width,
	                                      btn_height * 2);

    JCValueModel vm = getValueModel();
    Object value = (vm == null) ? null : vm.getValue();
    Component rendererComponent = renderer.getComponent(this, value);

    renderPanel.setBounds(0, 0, getSize().width - btn_width, btn_height * 2);

    doBorder();

    rendererComponent.setFont(this.getFont());

//  rendererComponent.setBounds(0, 0, getSize().width - btn_width,
//                              btn_height * 2);

	upArrow.setBounds(btn_x, 0, btn_width, btn_height);
	downArrow.setBounds(btn_x, btn_height, btn_width, btn_height);
	checkArrowButtons();
}


protected void doFocusedLook(Graphics g)
{
    // Look is managed by editor
    if (isEditable()) {
        return;
    }

    // Setup look of renderer
    JCValueModel vm = getValueModel();
    Object value = (vm == null) ? null : vm.getValue();
    Component r = renderer.getComponent(this, value);
   	String lf = UIManager.getLookAndFeel().getID();

    r.setForeground(UIManager.getColor("ComboBox.foreground"));
    r.setBackground(UIManager.getColor("ComboBox.background"));

    if (lf.equals("Motif")) {
        g.setColor(UIManager.getColor("activeCaptionBorder"));
        g.drawRect(0, 0, getSize().width - 1, getSize().height - 1);
    }
    else if (!lf.equals("Metal")) {
        r.setBackground(UIManager.getColor("ComboBox.selectionBackground"));
        r.setForeground(UIManager.getColor("ComboBox.selectionForeground"));
    }
}

protected void doNonFocusedLook(Graphics g)
{
    // Look is managed by editor
    if (isEditable()) {
        return;
    }

    // Setup look of renderer
    JCValueModel vm = getValueModel();
    Object value = (vm == null) ? null : vm.getValue();
    Component r = renderer.getComponent(this, value);

    r.setBackground(UIManager.getColor("ComboBox.background"));
    r.setForeground(UIManager.getColor("ComboBox.foreground"));
}

protected void doBorder()
{
    // Metal is treated as a special case becase the UIManager border is no
	// border which doesn't look right so an etched border is applied.
   	String lf = UIManager.getLookAndFeel().getID();
	if (lf.equals("Metal")) {
        renderPanel.setBorder(metalBorder);
    }
    else {
        renderPanel.setBorder(UIManager.getBorder("ComboBox.border"));
    }
}

/**
 * Paint the spinbox.
 */
public void paint(Graphics g)
{
    // Paint the button as usual
    super.paint(g);

    if (isEditable()) {
        return;
    }

    if (hasFocus()) {
        doFocusedLook(g);
    }
    else {
        doNonFocusedLook(g);
    }
}

/**
 * For use by concrete implementations of the <code>checkArrowButtons</code> to
 * safely disable the buttons as necessary.
 */
public void setArrowEnabled(JButton arrow, boolean enabled)
{
    if (isEnabled() == true) {
        if (arrow.isEnabled() != enabled) {
            arrow.doClick();
            arrow.setEnabled(enabled);
        }
    }
}

/**
 * Abstract method that check if arrows should be enabled or disabled.  Use
 * setArrowEnabled to enable/disable the arrow buttons.  This will do so
 * in a thread and state friendly manner.
 * @see #setArrowEnabled
 */
protected abstract void checkArrowButtons();

/**
 * Return the new value generated when spinning up a notch
 */
protected abstract Object spinUp();

/**
 * Return the new value generated when spinning down a notch
 */
protected abstract Object spinDown();

/**
 * This method can be overridded by the subclass
 */
protected void processValueChanging(JCValueEvent ev)
{
	JCValueEvent new_ev = new JCValueEvent(this,
										   ev.getOldValue(),
										   ev.getNewValue());
    new_ev.setAllowChange(ev.getAllowChange());

	// Post value changing event
	Enumeration e = JCListenerList.elements(valueListeners);
	while (e.hasMoreElements()) {
		JCValueListener listener = (JCValueListener) e.nextElement();
		listener.valueChanging(new_ev);
	}

	// Pass through any changes the listeners just called might have changed
	ev.setAllowChange(new_ev.getAllowChange());
	ev.setNewValue(new_ev.getNewValue());
}

/**
 * This method can be overridded by the subclass
 */
protected void processValueChanged(JCValueEvent ev)
{
	JCValueEvent new_ev = new JCValueEvent(this,
										   ev.getOldValue(),
										   ev.getNewValue());
	// Post value changing event
	Enumeration e = JCListenerList.elements(valueListeners);
	while (e.hasMoreElements()) {
		JCValueListener listener = (JCValueListener) e.nextElement();
		listener.valueChanged(new_ev);
	}
}

/**
 * Overridden in JCSpinBox to add keySelectionManager functionality
 */
protected void handleKeyPressed(KeyEvent e) {}

/**
 * Sets the layout manager for this container.  This method is
 * overridden to prevent the layout manager from being set.
 */
public final void setLayout(LayoutManager mgr) {}

protected void processFocusLost(FocusEvent e)
{
	if (isEditable()) {
		// Handle commiting and outstanding edits from text field
		setValue(editor.getItem());
		checkArrowButtons();
	}
	return;
}

class RendererListener implements MouseListener, Serializable {
	//
	// Mouse Listener
	//
	public void mouseClicked(MouseEvent e)
	{
        requestFocus();
	}

	public void mouseEntered(MouseEvent e) {}

	public void mouseExited(MouseEvent e) {}

	public void mousePressed(MouseEvent e) {}

	public void mouseReleased(MouseEvent e) {}
}

class Listener implements KeyListener, MouseListener, ActionListener,
			              JCValueListener, FocusListener, Serializable
{
	//
	// Mouse Listener
	//
	public void mouseClicked(MouseEvent e) {}

	public void mouseEntered(MouseEvent e)
	{
		JButton button = (JButton)e.getSource();

		if (button.isEnabled() == true &&
			button.equals(currentButton) &&
		    currentButton.getModel().isPressed() == true)
		{
			buttonTimer.start();
		}

	}

	public void mouseExited(MouseEvent e)
	{
		if (e.getSource().equals(currentButton) &&
		    currentButton.getModel().isPressed() == true)
		{
    		buttonTimer.stop();
		}
	}

	public void mousePressed(MouseEvent e)
	{
		currentButton = (JButton)e.getSource();

		buttonTimer.start();

        if (!continuousScroll) {
        	this.actionPerformed(null);
        }
        requestFocus();
	}

	public void mouseReleased(MouseEvent e)
	{
		currentButton = null;

		if (buttonTimer.isRunning()) {
			buttonTimer.stop();
		}

        repaint();
	}

	//
	// Key Listener
	//

	/**
	 * Invoked when a key has been typed. This event occurs when a key press is
	 * followed by a key release. (Not true in all cases except the general.)
	 */
	public void keyTyped(KeyEvent e) {}

	/**
	 * Invoked when a key has been pressed.
	 */
	public void keyPressed(KeyEvent e)
	{
		int kc;
		Object new_obj = null;

		kc = e.getKeyCode();
		switch(kc) {
		case KeyEvent.VK_UP:
			// commit any edits before spinning
			if (editable && compareToValue(editor.getItem()) == false) {
				setValue(editor.getItem());
			}
			if (upArrow.isEnabled() == true && arrowKeySpinningAllowed) {
				new_obj = spinUp();
				e.consume();
				setValue(new_obj);
				checkArrowButtons();
			}
			break;
		case KeyEvent.VK_DOWN:
			// commit any edits before spinning
			if (editable && compareToValue(editor.getItem()) == false) {
				setValue(editor.getItem());
			}
			if (downArrow.isEnabled() == true && arrowKeySpinningAllowed) {
				new_obj = spinDown();
				e.consume();
				setValue(new_obj);
				checkArrowButtons();
			}
			break;
		case KeyEvent.VK_ENTER:
		case KeyEvent.VK_ESCAPE:
			// commit any edits
			if (editable && compareToValue(editor.getItem()) == false) {
				setValue(editor.getItem());
				checkArrowButtons();
			}
			break;
		default:
			// pass on key events to implementation specific handling
			handleKeyPressed(e);
		}
	}

	/**
	 * Invoked when a key has been released.
	 */
	public void keyReleased(KeyEvent e) {}

	//
	// ActionListener Implementation
	//

	/**
	 * This method is invoked because a timer started by a click on
	 * the arrow button went off
	 */
	public void actionPerformed(ActionEvent ev)
	{
		Object new_value = null;

		if ((currentButton == null) || (currentButton.isEnabled() == false)) {
			return;
		}

        
		if (upArrow.equals(currentButton)) {
			// Commit any edits before spinning
			if (editable && compareToValue(editor.getItem()) == false) {
				setValue(editor.getItem());
			}
			new_value = spinUp();
		}
		else if (downArrow.equals(currentButton)) {
			// Commit any edits before spinning
			if (editable && compareToValue(editor.getItem()) == false) {
				setValue(editor.getItem());
			}
			new_value = spinDown();
		}
		else {
			return;
		}

        

		setValue(new_value);
		checkArrowButtons();
        

		buttonTimer.start();
	}

	//
	// Value Listener
	//

	/**
	 * The value is changing.  The listener may further change the new_value or
	 * abort the change altogether.  If the case of an abort (call
	 * event.setAllowChange(false)) the "valueChanged"  method will never be
	 * called.
	 */
	public void valueChanging(JCValueEvent event)
	{
		processValueChanging(event);
	}

	/**
	 * The method informs the listener that the value has been changed.
	 */
	public void valueChanged(JCValueEvent event)
	{
		processValueChanged(event);
	}

	//
	// Focus listener
	//

	public void focusGained(FocusEvent event)
	{
        repaint();
    }

	public void focusLost(FocusEvent event)
	{
		processFocusLost(event);
        repaint();
	}
} // End of Listener


static class DefaultEditor extends JTextField implements JCSpinBoxEditor
{
	/**
	 * Returns the component that should be added to the tree hierarchy for
	 * this editor
	 */
	public Component getEditorComponent()
	{
		return(this);
	}

	/**
	 * Sets the item that should be edited. Cancel any editing if necessary
	 **/
	public void setItem(Object anObject)
	{
		if (anObject == null) {
			setText("");
		}
		else {
			setText(anObject.toString());
		}
	}

	/** Returns the edited item **/
	public Object getItem()
	{
		return(getText());
	}

} // End of DefaultEditor

static class DefaultRenderer extends JLabel implements JCSpinBoxRenderer
{
    public DefaultRenderer()
	{
        super();
	    setOpaque(true);
    }

    // Make this the validate root so the entire
    // container is not being laid out every time
    // there is a spin.
    public boolean isValidateRoot()
	{
        return(editorIsValidateRoot);
    }

    public Component getComponent(AbstractSpinBox spinBox, Object value)
	{
        if (value != null) {
            setText(value.toString());
        }
        else {
            setText(" ");
        }
        return(this);
    }
} // End of DefaultRenderer

/**
 * Arrow button for Metal look and feel.
 */
public class MetalArrowButton extends JButton
{
    /**
     * Create an arrow button.
     */
	public MetalArrowButton(int direction)
	{
		super(new ArrowIcon(direction));
	}
};

/**
 * Arrow button used to match look and feel.
 */
public class ArrowButton extends    BasicArrowButton
                         implements java.io.Serializable
{
    /**
     * Create an arrow button.
     */
    public ArrowButton(int direction)
	{
        super(direction);
    }
}

/**
 * Generates a border for the Metal look and feel.
 */
public static class MetalBorder extends    AbstractBorder
                                implements UIResource, java.io.Serializable
{
    protected static Insets borderInsets = new Insets(4, 4, 4, 4);

    /**
     * Paint the border.
     */
    public void paintBorder(Component c, Graphics g, int x, int y,
	                        int w, int h)
	{
        if (c.isEnabled()) {
            // MetalUtils.drawButtonBorder(g, x, y, w, h, false);
            g.translate( x, y);
            g.setColor(MetalLookAndFeel.getControlDarkShadow());
            g.drawRect(0, 0, w - 2, h - 2);
            g.setColor(MetalLookAndFeel.getControlHighlight());
            g.drawRect(1, 1, w - 2, h - 2);
            g.setColor(MetalLookAndFeel.getControl());
            g.drawLine(0, h - 1, 1, h - 2);
            g.drawLine(w - 1, 0, w - 2, 1);
            g.translate(-x, -y);
            if (c.getParent().hasFocus()) {
                g.setColor(MetalLookAndFeel.getFocusColor());
                g.drawRect(x + 2, y + 2, w - 6, h - 6);
            }
        }
        else { // Disabled state
            // MetalUtils.drawDisabledBorder(g, x, y, w - 1, h - 1);
            g.translate(x, y);
            g.setColor(MetalLookAndFeel.getControlShadow());
            g.drawRect(0, 0, w - 1, h - 1);
        }
    }

    /**
     * Get the insets used by the border.
     */
    public Insets getBorderInsets(Component c)
	{
        return(borderInsets);
    }
}


}
