/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: ArrowIcon.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:44 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.swing;

import javax.swing.*;
import java.io.Serializable;
import java.awt.*;

/**
 * Creates ArrowIcon's for buttons.
 */
public class ArrowIcon implements Icon, Serializable
{

public static final int UP   = 0;
public static final int DOWN = 1;

protected int width     = 6;
protected int height    = 4;
protected int direction = UP;

/**
 * Constructor which sets the direction of the arrow
 * @param direction The direction of the icon.  Must be one of UP or DOWN.
 */
public ArrowIcon(int direction)
{
    if (direction != UP && direction != DOWN) {
        throw new IllegalArgumentException("ArrowIcon's direction must be " +
                                           "one of UP or DOWN.");
    }
    this.direction = direction;
}

/**
 * Paints the arrow icon.
 * @param c Component used to get the foreground color
 * @param g The graphics context used to draw
 * @param x The x position at which to paint
 * @param y The y position at which to paint
 */
public void paintIcon(Component c, Graphics g, int x, int y)
{
    int iconWidth = getIconWidth();

    // Draw the arrow in the foreground color of the button.
    Color foreground = c.getForeground();
    if (c.isEnabled() == false) {
        foreground = UIManager.getColor("Button.disabledText");
    }

    g.setColor(foreground);
    g.translate(x, y);

    // Draw either up or down arrow
    if (direction == DOWN) {
        g.drawLine(0, 0, iconWidth - 2, 0);
        g.drawLine(1, 1, 1 + (iconWidth - 4), 1);
        g.drawLine(2, 2, 2 + (iconWidth - 6), 2);
    }
    else {
        g.drawLine(2, 0, 2 + (iconWidth - 6), 0);
        g.drawLine(1, 1, 1 + (iconWidth - 4), 1);
        g.drawLine(0, 2, iconWidth - 2, 2);
    }

    g.translate(-x, -y);
}

/**
 * Get the icon width
 * @return The icon width
 */
public int getIconWidth()
{
    return(width);
}

/**
 * Get the icon height
 * @return The icon height
 */
public int getIconHeight()
{
    return(height);
}

}




