/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: BasePopupMenu.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:44 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.swing;

import java.awt.event.ActionListener;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.MenuElement;

/**
 * BasePopupMenu is the popup menu to initiate component specific
 * commands at runtime.
 */
public abstract class BasePopupMenu extends JPopupMenu
{

/**
 * BasePopupMenu constructor
 */
public BasePopupMenu(ActionListener listener)
{
	super();

    String[] commands = getCommands();
	// Setup the default popup menu list
    if (commands != null) {
        for (int i = 0; i < commands.length; i++) {
            addItem(commands[i], listener);
        }
    }
}

/**
 * Returns component specific text set.
 */
protected abstract String[] getCommands();

/**
 * Adds new item to the menu.
 *
 * @param text menu item text string.
 * @param listener action listener for the menu items.
 */
public void addItem(String text, ActionListener listener)
{
    JMenuItem menuItem = new JMenuItem(text);
    menuItem.setActionCommand(text);
    menuItem.addActionListener(listener);
    add(menuItem);
}

/**
 * Enable or disable a menu item in the popup menu.
 *
 * @param text The text string.
 * @param enable True if enable else false.
 *
 * @return True if menu item is found and is enabled or is disabled.
 */
public boolean enableMenuItem(String text, boolean enable)
{
    return(enableMenuItem(this, text, enable));
}

/**
 * Enable or disable a menu item in the popup menu.
 *
 * @param menu The menu level.
 * @param text The text string.
 * @param enable True if enable else false.
 *
 * @return True if menu item is found and is enabled or is disabled.
 */
public static boolean enableMenuItem(MenuElement menu, String text,
                                     boolean enable)
{
	MenuElement[] elements = menu.getSubElements();
	for (int i = 0; i < elements.length; i++) {
    	if (elements[i] instanceof JMenuItem) {
			String itemCommand = ((JMenuItem)elements[i]).getActionCommand();
            if (itemCommand.equals(text)) {
                ((JMenuItem)elements[i]).setEnabled(enable);
                return(true);
            }
		}
		else {
        	return(enableMenuItem(elements[i], text, enable));
        }
	}
	return(false);
}

}
