/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: GotoRowDialog.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:44 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.swing;

import com.klg.jclass.util.JCLocaleManager;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Displays a dialog box asking the user for a row number to advance to.
 */
public class GotoRowDialog extends JDialog implements ActionListener
{

private int         row = 0;
private JTextField  text = null;
private JButton     buttonOK;
private JButton     buttonCancel;

/**
 * Construct a Goto Row dialog.
 * @param parent      The parent frame to display the dialog over.
 * @param currentRow  The current row number.  This is the initial value.
 */
public GotoRowDialog(Frame parent, int currentRow)
{
	super(parent, 
		  (JCLocaleManager.getDefault() == null) ? "" : 
	  JCLocaleManager.getDefault().getString(JCLocaleManager.TITLE_GOTOROW), 
		  true);

    row = currentRow;

	// Create top  panel
    text = new JTextField("" + currentRow, 15);
    text.selectAll();
	JPanel p1 = new JPanel();
	p1.setLayout(new FlowLayout(FlowLayout.RIGHT));
	p1.add(text);

	// Create bottom panel
	JCLocaleManager li = JCLocaleManager.getDefault();
	buttonOK = new JButton((li == null) ? "Okay" : 
						   li.getString(JCLocaleManager.ACTION_OKAY));
	buttonCancel = new JButton((li == null) ? "Cancel" : 
						   li.getString(JCLocaleManager.ACTION_CANCEL));
	JPanel p2 = new JPanel();
	p2.setLayout(new FlowLayout(FlowLayout.RIGHT));
	p2.add(buttonOK);
	p2.add(buttonCancel);

	getContentPane().setLayout(new BorderLayout(10, 4));
	getContentPane().add("Center", p1);
	getContentPane().add("South", p2);

	buttonCancel.addActionListener(this);
	buttonOK.addActionListener(this);

	//Initialize this dialog to its preferred size.
	pack();
}

/**
 * Reacts to the pressing of the OK and Cancel buttons.
 */
public void actionPerformed(ActionEvent event)
{
	Object source = event.getSource();
	if (source == buttonOK) {
	    try {
		    row = Integer.valueOf(text.getText()).intValue();
			setVisible(false);
		}
        catch (NumberFormatException e) {
//        	MessageDialog messageBox =
//              new MessageDialog(null, "Error", "Invalid integer number.");
//			messageBox.centerDialog();
//    		messageBox.run();
		}
	}
	else if (source == buttonCancel) {
		setVisible(false);
    }
}

/**
 * Returns the chosen row.
 */
public int getRow()
{
	return(row);
}

/**
 * Centers the dialog relative to the parent frame or screen.
 */
protected void center()
{
	int x = 0;
	int y = 0;
	Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
	x = (screenDim.width - getSize().width) / 2;
	y = (screenDim.height - getSize().height) / 2;
	x = Math.max(x, 0);
	y = Math.max(y, 0);
    setLocation(x, y);
}

/**
 * Opens the dialog window in the center of the screen.
 */
public void open()
{
	center();
	show();
}

} 
