/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCBox.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:45 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.swing;

import java.awt.LayoutManager;
import javax.swing.JPanel;
import com.klg.jclass.util.JCVersion;

/**
 * <code>JCBox</code> is a Swing container that uses
 * <code>JCElasticLayout</code> to lay out components in single row or column.
 * Use the orientation property within an IDE to control the orientation of the
 * Box.  The <code>JCSpring</code> and <code>JCBrace</code> components are
 * useful Beans to use in conjunction with this container.
 * @see JCSpring
 * @see JCBrace
 * @see JCElasticLayout
 * @see JCElastic
 */
public class JCBox extends JPanel {

/** Constant for use with constructor and <code>setOrientation()</code>. */
public static final int HORIZONTAL = JCElasticLayout.HORIZONTAL;
/** Constant for use with constructor and <code>setOrientation()</code>. */
public static final int VERTICAL = JCElasticLayout.VERTICAL;

private int h_instance_count = 0;
private int v_instance_count = 0;

/**
 * Orientation of the Box; valid values are <code>HORIZONTAL</code> or
 * <code>VERTICAL</code>.
 */
protected int orientation;

/**
 * Create a <code>HORIZONTAL</code> <code>JCBox</code> Container.
 */
public JCBox()
{
    this(HORIZONTAL);
}

public JCBox(int orientation)
{
    // Validate orientation
    checkOrientation(orientation);

    if (orientation == HORIZONTAL) {
        setLayout(new JCElasticLayout(HORIZONTAL));
        setName("HorizontalBox" + h_instance_count++);
    }
    else if (orientation == VERTICAL) {
        setLayout(new JCElasticLayout(VERTICAL));
        setName("VerticalBox" + v_instance_count++);
    }
}

public static JCBox createHorizontalBox()
{
    return(new JCBox(HORIZONTAL));
}

public static JCBox createVerticalBox()
{
    return(new JCBox(VERTICAL));
}

/**
 * Provided for Beans property editors - has no effect.
 * @param s
 */
public void setAbout(String s) {}

/**
 * Returns vendor information.
 * @return vendor information
 */
public String getAbout()
{
    return(JCVersion.getVersionString());
}

/**
 * Sets the orientation of the Box.
 * @param orientation valid values are <code>HORIZONTAL</code> and
 * <code>VERTICAL</code>
 */
public void setOrientation(int orientation)
{
    // Validate orientation
    checkOrientation(orientation);

    if (this.orientation == orientation) {
        // No change so bail
        return;
    }

    this.orientation = orientation;
    if (orientation == HORIZONTAL) {
        super.setLayout(new JCElasticLayout(HORIZONTAL));
    }
    else {
        super.setLayout(new JCElasticLayout(VERTICAL));
    }
}

/**
 * Returns the current orientation.
 * @return current orientation
 */
public int getOrientation()
{
    return(orientation);
}

/**
 * Sets the alignment for the layout.
 * @see com.klg.jclass.util.swing.JCElasticLayout
 * @param alignment The new alignment
 */
public void setAlignment(int alignment)
{
    ((JCElasticLayout)getLayout()).setAlignment(alignment);
    invalidate();
}

/**
 * Returns the alignment for the layout.
 * @see com.klg.jclass.util.swing.JCElasticLayout
 * @return alignment for the layout
 */
public int getAlignment()
{
    return(((JCElasticLayout)getLayout()).getAlignment());
}

/**
 * Validate an orientation value.
 * @param o The orientation to check
 */
public void checkOrientation(int o)
{
    if (o != HORIZONTAL && o != VERTICAL) {
        throw new IllegalArgumentException("Orientation must be one " +
                                           "of HORIZONTAL or VERTICAL");
    }
}

/**
 * Overrides <code>setLayout</code> so that it only accepts instances of
 * <code>JCElasticLayout</code>.
 * @param mgr The new layout manager.  Must be of type
 * <code>JCElasticLayout</code>.
 */
public void setLayout(LayoutManager mgr)
{
    // Ensure that layout manager being set is an instance of JCElasticLayout
    if (mgr instanceof JCElasticLayout) {
        super.setLayout(mgr);
    }
}

}
