/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCBrace.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:45 $ $Locker:  $  Quest Software Inc.
package com.klg.jclass.util.swing;

import java.awt.Dimension;
import java.awt.Component;

import com.klg.jclass.util.JCVersion;

/**
 * An implementation of a component that participates in layout but has no
 * view.  It is called a "brace" since its main function is to reserve space
 * for the purpose of controlling layout.<p>
 * A brace usually has a minimum and preferred size that are equal, and
 * an unlimited maximum size.
 */
public class JCBrace extends Component {

public static final int HORIZONTAL = 0;
public static final int VERTICAL   = 1;

protected int orientation;

protected Dimension minimumSize;
protected Dimension preferredSize;
protected Dimension maximumSize;

/**
 * Default constructor.
 * Creates a horizontal shape.
 */
public JCBrace()
{
    minimumSize = new Dimension(10, 0);
    preferredSize = new Dimension(10, 0);
    maximumSize = new Dimension(10, Short.MAX_VALUE);
}

/**
 * Creates a shape with the given constraints.
 * @param minimum_size   minimum size
 * @param preferred_size  preferred size
 * @param maximum_size   maximum size
 */
public JCBrace(Dimension minimum_size,
               Dimension preferred_size,
               Dimension maximum_size)
{
    minimumSize = minimum_size;
    preferredSize = preferred_size;
    maximumSize = maximum_size;
}

public JCBrace(int length, int orientation)
{
    this.orientation = orientation;
    minimumSize = new Dimension();
    preferredSize = new Dimension();
    maximumSize = new Dimension();
    setLength(length);
}

/**
 * Creates a horizontal <code>JCBrace</code> with the specified width.
 * @return
 * @param width
 */
static public JCBrace createHorizontalBrace(int width)
{
    return(new JCBrace(width, HORIZONTAL));
}

/**
 * Creates a vertical <code>JCBrace</code> with the specified width.
 * @return
 * @param height
 */
static public JCBrace createVerticalBrace(int height)
{
    return(new JCBrace(height, VERTICAL));
}

public int getOrientation()
{
    return(orientation);
}

public void setOrientation(int orientation)
{
    if (this.orientation == orientation) {
        return;
    }
    if ((orientation == HORIZONTAL) || (orientation == VERTICAL)) {
        int length = getLength();
        this.orientation = orientation;
        setLength(length);
    }
}

public int getLength()
{
    if (orientation == HORIZONTAL) {
        return(minimumSize.width);
    }
    else if (orientation == VERTICAL) {
        return(minimumSize.height);
    }
    return(0);
}

public void setLength(int length)
{
    if (orientation == HORIZONTAL) {
        minimumSize.setSize(length, 0);
        preferredSize.setSize(length, 0);
        maximumSize.setSize(length, Short.MAX_VALUE);
    }
    else if (orientation == VERTICAL) {
        minimumSize.setSize(0, length);
        preferredSize.setSize(0, length);
        maximumSize.setSize(Short.MAX_VALUE, length);
    }
}


/**
 * Provided for Beans property editors - has no effect.
 * @param s
 */
public void setAbout(String s) {}

/**
 * Returns vendor information.
 * @return vendor information
 */
public String getAbout()
{
    return(JCVersion.getVersionString());
}

//
// Overrides of Components Size methods
//

/**
 * Returns the minimum size of the component.
 * @return minimum size of the component
 */
public Dimension getMinimumSize()
{
    return(minimumSize);
}

/**
 * Returns the preferred size of the component.
 * @return preferred size of the component
 */
public Dimension getPreferredSize()
{
    return(preferredSize);
}

/**
 * Returns the maximum size of the component.
 * @return maximum size of the component
 */
public Dimension getMaximumSize()
{
    return(maximumSize);
}

}
