/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCCheckBoxList.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:45 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.swing;

import com.klg.jclass.util.JCVersion;

import javax.swing.*;
import java.awt.*;
import java.util.Vector;

/**
* Subclass of the <code>JList</code> component that implements a
* <code>JCheckBox</code> as the cell renderer.
*/
public class JCCheckBoxList extends JList {

/**
 * Default constructor.
 */
public JCCheckBoxList()
{
    super();
    setCellRenderer(new CheckBoxCellRenderer());
}

/**
 * Constructor which takes an initial ListModel.
 * @param dataModel The list model
 */
public JCCheckBoxList(ListModel dataModel)
{
    super(dataModel);
    setCellRenderer(new CheckBoxCellRenderer());
}

/**
 * Constructor which takes array of objects as the initial list.
 * @param list The initial list of objects
 */
public JCCheckBoxList(Object list[])
{
    super(list);
    setCellRenderer(new CheckBoxCellRenderer());
}

/**
 * Constructor that takes a vector as its initial list.
 * @param list The initial list vector
 */
public JCCheckBoxList(Vector list)
{
    super(list);
    setCellRenderer(new CheckBoxCellRenderer());
}

/**
 * Provided for Beans property editors - has no effect.
 * @param s ignored
 */
public void setAbout(String s) {}

/**
 * Returns vendor information.
 * @return vendor information
 */
public String getAbout()
{
    return(JCVersion.getVersionString());
}

/**
 * Resets the UI property with the value from the current look and feel.
 *
 * @see UIManager#getUI
 */
public void updateUI()
{
    super.updateUI();
    setCellRenderer(new CheckBoxCellRenderer());
}

/**
 * New renderer that draws a checkbox.
 */
class CheckBoxCellRenderer extends JCheckBox implements ListCellRenderer
{
    /**
     * Default constructor.
     */
    public CheckBoxCellRenderer()
    {
        setOpaque(true);
    }

    /**
     * Return a JCheckBox component that has been configured to display the
     * specified value and whether that item is selected.
     *
     * @param list The JList we're painting.
     * @param value The value returned by list.getModel().getElementAt(index).
     * @param index The cell's index.
     * @param isSelected True if the specified cell was selected.
     * @param cellHasFocus True if the specified cell has the focus.
     * @return A component whose paint() method will render the specified value.
     *
     * @see JList
     * @see ListSelectionModel
     * @see ListModel
     */
    public Component getListCellRendererComponent(JList list, Object value,
                                                  int index, boolean isSelected,
                                                  boolean cellHasFocus)
    {
        setText(value.toString());
        setSelected(isSelected);
        return(this);
    }
}

}




