/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCExitFrame.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:45 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.swing;

import javax.swing.JFrame;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.Dimension;
import java.awt.Toolkit;
import com.klg.jclass.util.JCVersion;

/**
 * <code>JCExitFrame</code> is a <code>JFrame</code> extension that exits the
 * application when the frame is closed.
 */
public class JCExitFrame extends JFrame
{

private boolean exit = true;

/**
 * Default constructor.
 */
public JCExitFrame()
{
	super();
	addWindowListener(new Exiter());
}

/**
 * Constructor with title.
 * @param title title to appear at top of frame
 */
public JCExitFrame(String title)
{
	super(title);
	addWindowListener(new Exiter());
}

/**
 * Provided for Beans property editors - has no effect.
 * @param s
 */
public void setAbout(String s) {}

/**
 * Returns vendor information.
 * @return vendor information
 */
public String getAbout()
{
    return(JCVersion.getVersionString());
}

/** Gets the <code>exitOnClose</code> value.
 * @see #setExitOnClose
 * @return <code>exitOnClose</code> value
 */
public boolean getExitOnClose()
{
	return(exit);
}

/**
 * Determines whether the application should exit when the user closes
 * the frame (default is <code>true</code>).<p>
 * If set to <code>false</code>, the frame is hidden; it can be made visible
 * later by calling <code>setVisible()</code>.
 * @see javax.swing.JFrame#setDefaultCloseOperation
 * @see java.awt.Component#setVisible
 * @param v The new value of exitOnClose
 */
public void setExitOnClose(boolean v)
{
	exit = v;
}

public void center()
{
    center(this);
}

static public void center(JFrame frame)
{
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension windowSize = frame.getSize();
    frame.setLocation((screenSize.width - windowSize.width) / 2,
                      (screenSize.height - windowSize.height) / 2);
}

public void sizeToScreen(double ratio)
{
    sizeToScreen(this, ratio);
}

static public void sizeToScreen(JFrame frame, double ratio)
{
    if (ratio > 1.0) {
        ratio = 1.0;
    }
    else if (ratio <= 0.0) {
        return;
    }
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    frame.setSize((int)Math.rint(screenSize.width * ratio),
                  (int)Math.rint(screenSize.height * ratio));
}

/**
 * Inner class used to listen for a window closing message,
 * then exit the application.
 */
public class Exiter extends WindowAdapter {
    /**
     * Overrides window closing event to catch the click on the close box.
     * @param e The window event
     */
    public void windowClosing(WindowEvent e)
    {
        setVisible(false);
        if (exit) {
            dispose();
            System.exit(0);
        }
    }
} // End Exiter

}



