/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCFontChooser.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:45 $  $Locker:  $  Quest Software Inc.


package com.klg.jclass.util.swing;

import java.awt.Font;
import java.util.Enumeration;


import java.util.Map;
import java.awt.font.TextAttribute;
import java.awt.GraphicsEnvironment;





import javax.swing.JComponent;

import com.klg.jclass.util.JCListenerList;

/**
 * <code>JCFontChooser</code> is the base class for
 * <code>JCFontChooserBar</code> and <code>JCFontChooserPane</code>.
 * It provides common data and methods for both components.
 */
public abstract class JCFontChooser extends JComponent {

/** Font listener list. */
private JCListenerList  fontListeners = null;;

/** Selected font. */
private String          name = "";
private int             size = 10;
private boolean         bold = false;
private boolean         italic = false;
private boolean         underline = false;

/** Old font. */
protected Font          oldFont;
protected boolean       oldUnderline;

/** Font lists. */
private String[]        nameList;
private static String[] sizeList = {"8", "9", "10", "11", "12", "13", "14",
                                    "16", "18", "20", "24", "28", "32", "36",
                                    "48", "72"};

/** Default font. */
private static final String DEFAULT_NAME = "Serif";
private static final int    DEFAULT_SIZE = 12;
private static final int    DEFAULT_STYLE = Font.PLAIN;

/** The no style support constant. */
public static final int NONE	  = 0;
/** The bold style support constant. */
public static final int BOLD	  = 1;
/** The italic style support constant. */
public static final int ITALIC 	  = 2;
/** The underline style support constant. */
public static final int UNDERLINE = 4;
/** The all styles support constant. */
public static final int ALL		  = 7;

/** Creates a font chooser with a default inital font. */
public JCFontChooser()
{
    this(DEFAULT_NAME, DEFAULT_STYLE, DEFAULT_SIZE, false);
}

/** Creates a font chooser with a specified initial font.
 * @param font the initial font
 */
public JCFontChooser(Font font)
{
    this(font.getName(), font.getStyle(), font.getSize(), isUnderline(font));
}

/** Creates a font chooser with a specified initial font and underline.
 * @param font the initial font
 * @param underline <code>true</code> if underline is on
 */
public JCFontChooser(Font font, boolean underline)
{
    this(font.getName(), font.getStyle(), font.getSize(), underline);
}

/** Creates a font chooser with a specified initial font and underline.
 * @param name the font name
 * @param style the style constant for the font
 * @param size the point size of the font
 * @param underline <code>true</code> if underline is on
 */
public JCFontChooser(String name, int style, int size, boolean underline)
{
    this.setFontValues(name, style, size, underline);
}

/** Sets the font values.
 * @param name the font name
 * @param style the style constant for the font
 * @param size the point size of the font
 * @param underline <code>true</code> if underline is on
 */
protected void setFontValues(String name, int style, int size,
                             boolean underline)
{
    this.setFontName(name);
    this.setFontSize(size);
    this.bold = (style & Font.BOLD) != 0;
    this.italic = (style & Font.ITALIC) != 0;
    this.setUnderline(underline);
}

/** Sets the font values.
 * @param font the selected font
 * @param underline <code>true</code> if underline is on
 */
protected void setFontValues(Font font, boolean underline)
{
    this.setFontValues(font.getName(), font.getStyle(), font.getSize(),
                       underline);
}

/** Sets the selected font.
 * @param name the font name
 * @param style the style constant for the font
 * @param size the point size of the font
 * @param underline <code>true</code> if underline is on
 */
protected void setSelectedFont(String name, int style, int size,
                               boolean underline)
{
    this.setFontValues(name, style, size, underline);
}

/** Sets the selected font.
 * @param font the selected font
 * @param underline <code>true</code> if underline is on
 */
protected void setSelectedFont(Font font, boolean underline)
{
    this.setSelectedFont(font.getName(), font.getStyle(), font.getSize(),
                         underline);
}

/** Sets the selected font.
 * @param font the selected font
 */
protected void setSelectedFont(Font font)
{
    this.setSelectedFont(font.getName(), font.getStyle(), font.getSize(),
                         isUnderline(font));
}

/** Gets the selected font.
 * @return the current selected font
 */
public Font getSelectedFont()
{
    int style = 0;
    if (bold || italic) {
        if (bold) {
            style += Font.BOLD;
        }
        if (italic) {
            style += Font.ITALIC;
        }
    }
    else {
        style = Font.PLAIN;
    }
    Font font1 = new Font(name, style, size);

    Map attributes = font1.getAttributes();
    if (underline) {
        attributes.put(TextAttribute.UNDERLINE, TextAttribute.UNDERLINE_ON);
    }
    return(new Font(attributes));
}

/** Sets the font name.
 * @param name the font name
 */
protected void setFontName(String name)
{
    this.name = name;
    if (name == null || name.length() == 0) {
        this.name = new String(DEFAULT_NAME);
    }
}

/** Returns the font name.
 * @return the font name
 */
protected String getFontName()
{
    return(name);
}

/** Sets the font size.
 * @param size the point size of the font
 */
protected void setFontSize(int size)
{
    this.size = (size > 0) ? size : 1;
}

/** Returns the font size.
 * @return the font size
 */
protected int getFontSize()
{
    return(size);
}

/** Sets the bold indication.
 * @param bold <code>true</code> if bold is on
 */
protected void setBold(boolean bold)
{
    this.bold = bold;
}

/** Returns the bold indication.
 * @return <code>true</code> if bold is on
 */
protected boolean isBold()
{
    return(bold);
}

/** Sets the italic indication.
 * @param italic <code>true</code> if underline is on
 */
protected void setItalic(boolean italic)
{
    this.italic = italic;
}

/** Returns the italic indication.
 * @return <code>true</code> if italic is on
 */
protected boolean isItalic()
{
    return(italic);
}

/**
 * Sets the underline indication.
 * @param underline <code>true</code> if underline is on
 */
protected void setUnderline(boolean underline)
{
    this.underline = underline;
}

/**
 * Returns the underline indication.
 * @return <code>true</code> if underline is on
 */
public boolean isUnderline()
{
    return(underline);
}

/**
 * Sets the font name list.
 */
protected void setNameList()
{
   GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
    String[] fonts = ge.getAvailableFontFamilyNames();

    // filter out the bold and italic stuff
    int count = 0;
    for (int i = 0; i < fonts.length; i++) {
        if (fonts[i].endsWith(".bold") ||
            fonts[i].endsWith(".bolditalic") ||
            fonts[i].endsWith(".italic")) {
            continue;
        }
        count++;
    }
    nameList = new String[count];
    int index = 0;
    for (int i = 0; i < fonts.length; i++) {
        if (fonts[i].endsWith(".bold") ||
            fonts[i].endsWith(".bolditalic") ||
            fonts[i].endsWith(".italic")) {
            continue;
        }
        nameList[index++] = fonts[i];
    }
}

/** Sets the font name list.
 * @param nameList array of font names
 */
protected void setNameList(String[] nameList)
{
    this.nameList = nameList;
}

/** Gets the font name list.
 * @return font name list
 */
public String[] getNameList()
{
    return(nameList);
}

/** Gets the font size list.
 * @return font size list
 */
protected static String[] getSizeList()
{
    return(sizeList);
}

/**
 * Adds the specified listener to receive <code>JCFontEvent</code> objects.
 * @param listener a listener
 */
public synchronized void addJCFontListener(JCFontListener listener)
{
    fontListeners = JCListenerList.add(fontListeners, listener);
}

/**
 * Removes the specified listener so it no longer receive
 *  <code>JCFontEvent</code> objects.
 * @param listener a listener
 */
public synchronized void removeJCFontListener(JCFontListener listener)
{
    fontListeners = JCListenerList.remove(fontListeners, listener);
}

/**
 * Notifies registered listeners of a <code>fontChanging JCFontEvent</code>.
 * @param event the <code>JCFontEvent</code>
 */
protected void fireJCFontChangingEvent(JCFontEvent event)
{
    Enumeration e = JCListenerList.elements(fontListeners);
    while (e.hasMoreElements()) {
        JCFontListener listener = (JCFontListener)e.nextElement();
        listener.fontChanging(event);
    }
}

/**
 * Notifies registered listeners of a <code>fontChanged JCFontEvent</code>.
 * @param event the <code>JCFontEvent</code>
 */
protected void fireJCFontChangedEvent(JCFontEvent event)
{
    Enumeration e = JCListenerList.elements(fontListeners);
    while (e.hasMoreElements()) {
        JCFontListener listener = (JCFontListener)e.nextElement();
        listener.fontChanged(event);
    }
}

/** Sets the old font. */
protected void setOldFont()
{
    oldFont = this.getSelectedFont();
    oldUnderline = this.isUnderline();
}

/** Sets the underline text attribute for a given font.
 * @param font the initial font
 * @param underline the underline indication
 * @return the new font with the indicated underline text attribute
 */
public static Font setUnderline(Font font, boolean underline)
{

    Map attributes = font.getAttributes();
    try {
        attributes.remove(TextAttribute.UNDERLINE);
    }
    catch (java.lang.UnsupportedOperationException e) {
    }
    if (underline) {
        attributes.put(TextAttribute.UNDERLINE, TextAttribute.UNDERLINE_ON);
    }
    return(new Font(attributes));
}

/** Gets the underline text attribute for a given font.
 * @param font the initial font
 * @return <code>true</code> if underline is on; otherwise, <code>false</code>
 */
public static boolean isUnderline(Font font)
{
    Map attributes = font.getAttributes();
    Object o = attributes.get(TextAttribute.UNDERLINE);
    if (o != null && o.equals(TextAttribute.UNDERLINE_ON)) {
        return(true);
    }
    return(false);

}

}
