/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCFontChooserBar.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:45 $  $Locker:  $  Quest Software Inc.


package com.klg.jclass.util.swing;

import com.klg.jclass.util.JCVersion;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

/**
 * <code>JCFontChooserBar</code> provides a pane of controls designed to allow
 * a user to manipulate and select a font.  It is suitable for
 * use in a <code>JToolbar</code>.
 */
public class JCFontChooserBar extends JCFontChooser
                              implements ActionListener, ItemListener {

/** The no style support constant. */
public static final int NONE = JCFontChooser.NONE;
/** The bold style support constant. */
public static final int BOLD = JCFontChooser.BOLD;
/** The italic style support constant. */
public static final int ITALIC = JCFontChooser.ITALIC;
/** The underline style support constant. */
public static final int UNDERLINE = JCFontChooser.UNDERLINE;
/** The all styles support constant. */
public static final int ALL = JCFontChooser.ALL;

// icon images
private static final String boldIcon[] = {
"             ",
"             ",
"             ",
"  XXXXXXXX   ",
"   XXX  XXX  ",
"   XXX  XXX  ",
"   XXX  XXX  ",
"   XXXXXXX   ",
"   XXX  XXX  ",
"   XXX  XXX  ",
"   XXX  XXX  ",
"  XXXXXXXX   ",
"             ",
"             ",
"             " };

private static final String italicIcon[] = {
"             ",
"             ",
"             ",
"       XXXXXX",
"        XXX  ",
"       XXX   ",
"      XXX    ",
"     XXX     ",
"    XXX      ",
"   XXX       ",
"  XXX        ",
"XXXXXX       ",
"             ",
"             ",
"             " };

private static final String underlineIcon[] = {
"             ",
"             ",
"             ",
"  XXXX XXXX  ",
"   XX   XX   ",
"   XX   XX   ",
"   XX   XX   ",
"   XX   XX   ",
"   XX   XX   ",
"   XX   XX   ",
"   XX   XX   ",
"    XXXXX    ",
"             ",
"  XXXXXXXXX  ",
"             " };

protected JComboBox      nameCombobox;
protected JComboBox      sizeCombobox;
protected JToggleButton  boldStyleButton;
protected JToggleButton  italicStyleButton;
protected JToggleButton  underlineStyleButton;
protected int            styleControls = ALL;
protected boolean        toolTipEnabled;

/** Creates a font chooser bar with default initial font.*/
public JCFontChooserBar()
{
    super();
    init();
}

/** Creates a font chooser bar with a specified initial font.
 * @param font the initial font
 */
public JCFontChooserBar(Font font)
{
    super(font);
    init();
}

/** Creates a font chooser bar with a specified initial font and underline.
 * @param font the initial font
 * @param underline <code>true</code> if underline is on
 */
public JCFontChooserBar(Font font, boolean underline)
{
    super(font, underline);
    init();
}

/** Creates a font chooser bar with a specified initial font and underline.
 * @param name the font name
 * @param style the style constant for the font
 * @param size the point size of the font
 * @param underline <code>true</code> if underline is on
 */
public JCFontChooserBar(String name, int style, int size, boolean underline)
{
    super(name, style, size, underline);
    init();
}

/** Initializes the component. */
protected void init()
{
    // set the layout
    setLayout(new JCRowLayout());

    // add the font name combobox
    nameCombobox = new JComboBox();
    setNameList();
    setNameCombobox();
    nameCombobox.addItemListener(this);
    add(nameCombobox);

    // add the font size combobox
    sizeCombobox = new JComboBox();
    for (int i = 0; i < getSizeList().length; i++) {
        sizeCombobox.addItem(getSizeList()[i]);
    }
    sizeCombobox.setEditable(true);
    sizeCombobox.addItemListener(this);
    add(sizeCombobox);

    // add the spacer
    add(JCBrace.createHorizontalBrace(8));

    // setup icons
    JCIconCreator ic = new JCIconCreator(13, 15);
    ic.setColor('X', Color.black);
    ic.setPixels(boldIcon);

    // add the bold button
    boldStyleButton = new JToggleButton(ic.getIcon());
    boldStyleButton.addActionListener(this);
    add(boldStyleButton);

    // add the italic button
    ic.clear();
    ic.setPixels(italicIcon);
    italicStyleButton = new JToggleButton(ic.getIcon());
    italicStyleButton.addActionListener(this);
    add(italicStyleButton);

    // add the underline button
    ic.clear();
    ic.setPixels(underlineIcon);
    underlineStyleButton = new JToggleButton(ic.getIcon());
    underlineStyleButton.addActionListener(this);
    add(underlineStyleButton);

    // update the UI
    setFontChooserBar();
    setToolTipEnabled(true);

    // set the preferred sizes
    Dimension dim = nameCombobox.getPreferredSize();
    sizeCombobox.setPreferredSize(new Dimension(dim.height * 2 + 10,
                                                dim.height));
    boldStyleButton.setPreferredSize(new Dimension(dim.height, dim.height));
    italicStyleButton.setPreferredSize(new Dimension(dim.height, dim.height));
    underlineStyleButton.setPreferredSize(new Dimension(dim.height,
                                                        dim.height));
}

/** Populates the font name combobox. */
protected void setNameCombobox()
{
   // for (int i = 0; i < getNameList().length; i++) {
     //   nameCombobox.addItem(getNameList()[i]);
    //}
    //PR34125 the above is incorrect.  set should set, not append.
    nameCombobox.setModel(new DefaultComboBoxModel(getNameList()));
    nameCombobox.setSelectedItem(getFontName());
}

/** Sets the font name list.
 * @param nameList array of font names
 */
public void setNameList(String[] nameList)
{
    super.setNameList(nameList);
    setNameCombobox();
}

/**
 * Updates the control buttons.
 */
protected void updateControls()
{
    remove(boldStyleButton);
    remove(italicStyleButton);
    remove(underlineStyleButton);

    if ((styleControls & BOLD) != 0) {
        add(boldStyleButton);
    }
    if ((styleControls & ITALIC) != 0) {
        add(italicStyleButton);
    }
    if ((styleControls & UNDERLINE) != 0) {
        add(underlineStyleButton);
    }
    invalidate();
}

/**
 * Provided for Beans property editors - has no effect.
 * @param s
 */
public void setAbout(String s) {}

/**
 * Returns vendor information.
 * @return vendor information
 */
public String getAbout()
{
    return(JCVersion.getVersionString());
}

/**
 * Sets the font style support variable.
 * @param controls
 */
public void setStyleControls(int controls)
{
    styleControls = controls;
    updateControls();
}

/**
 * Gets the font style support variable.
 * @return font style support variable
 */
public int getStyleControls()
{
    return(styleControls);
}

/** Sets the UI. */
protected void setFontChooserBar()
{
    nameCombobox.setSelectedItem(getFontName());
    sizeCombobox.setSelectedItem(new Integer(getFontSize()).toString());
    boldStyleButton.setSelected(isBold());
    italicStyleButton.setSelected(isItalic());
    underlineStyleButton.setSelected(isUnderline());
}

/** Sets the <i>tool tips text enabled</i> indication.
 * @param toolTipEnabled <code>true</code> to enable
 */
public void setToolTipEnabled(boolean toolTipEnabled)
{
    this.toolTipEnabled = toolTipEnabled;
    if (toolTipEnabled) {
        // i18n for text
        nameCombobox.setToolTipText("Font");
        sizeCombobox.setToolTipText("Font Size");
        boldStyleButton.setToolTipText("Bold");
        italicStyleButton.setToolTipText("Italic");
        underlineStyleButton.setToolTipText("Underline");
    }
    else {
        nameCombobox.setToolTipText(null);
        sizeCombobox.setToolTipText(null);
        boldStyleButton.setToolTipText(null);
        italicStyleButton.setToolTipText(null);
        underlineStyleButton.setToolTipText(null);
    }
}

/** Gets the <i>tool tips text enabled</i> indication.
 * @return tool tips text enabled
 */
public boolean isToolTipEnabled()
{
    return(this.toolTipEnabled);
}

/** Implementation for the <code>ActionListener</code>.
 * @param e
 */
public void actionPerformed(ActionEvent e)
{
    if (e.getSource() == boldStyleButton) {
        setOldFont();
        setBold(((JToggleButton)e.getSource()).isSelected());
        fireJCFontEvent(JCFontEvent.BOLD_STYLE_CHANGE);
    }
    else if (e.getSource() == italicStyleButton) {
        setOldFont();
        setItalic(((JToggleButton)e.getSource()).isSelected());
        fireJCFontEvent(JCFontEvent.ITALIC_STYLE_CHANGE);
    }
    else if (e.getSource() == underlineStyleButton) {
        setOldFont();
        setUnderline(((JToggleButton)e.getSource()).isSelected());
        fireJCFontEvent(JCFontEvent.UNDERLINE_STYLE_CHANGE);
    }
}

/** Implementation for the <code>ItemListener</code>.
 * @param e
 */
public void itemStateChanged(ItemEvent e)
{
    // interested only in item selected
    if (e.getStateChange() == ItemEvent.SELECTED) {
        if (e.getSource() == nameCombobox) {
            setOldFont();
            setFontName(e.getItem().toString());
            fireJCFontEvent(JCFontEvent.FONT_NAME_CHANGE);
        }
        else if (e.getSource() == sizeCombobox) {
            try {
                setOldFont();
                setFontSize(Integer.parseInt(e.getItem().toString()));
                fireJCFontEvent(JCFontEvent.FONT_SIZE_CHANGE);
            }
            catch (NumberFormatException ex) {
            }
        }
    }
}

/** Fires the font changing and font changed event.
 *  @param event the font change event
 */
protected void fireJCFontEvent(int event)
{
    // Fire the font changing event
    JCFontEvent fontEvent = new JCFontEvent(this, event, oldFont,
                                            oldUnderline, getSelectedFont(),
                                            isUnderline(), true);
    fireJCFontChangingEvent(fontEvent);

    // If font change is cancelled, reset to old font
    if (fontEvent.isCancelFontChange()) {
        setFontValues(oldFont, oldUnderline);
        setFontChooserBar();
    }
    // If font changed has been overridden, set to new font
    else if (fontEvent.isFontChanged()) {
        setFontValues(fontEvent.getFont(), fontEvent.getUnderline());
        setFontChooserBar();
    }

    // Fire the font changed event
    if (!fontEvent.isCancelFontChange()) {
        fireJCFontChangedEvent(new JCFontEvent(this, event, oldFont,
                                               oldUnderline, getSelectedFont(),
                                               isUnderline(), false));
    }

}

/** Sets the selected font.
 * @param name the font name
 * @param style the style constant for the font
 * @param size the point size of the font
 * @param underline <code>true</code> if underline is on
 */
public void setSelectedFont(String name, int style, int size, boolean underline)
{
    setOldFont();
    super.setSelectedFont(name, style, size, underline);
    setFontChooserBar();
    fireJCFontEvent(JCFontEvent.FONT_CHANGE);
}

/** Sets the selected font.
 * @param font the selected font
 * @param underline <code>true</code> if underline is on
 */
public void setSelectedFont(Font font, boolean underline)
{
    setSelectedFont(font.getName(), font.getStyle(), font.getSize(), underline);
}

/** Sets the selected font.
 * @param font the selected font
 */
public void setSelectedFont(Font font)
{
    setSelectedFont(font.getName(), font.getStyle(), font.getSize(),
                    JCFontChooser.isUnderline(font));
}

}
