/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCFontChooserPane.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:45 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.swing;

import com.klg.jclass.util.JCEnvironment;
import com.klg.jclass.util.JCVersion;
import com.klg.jclass.util.JCLocaleManager;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.awt.event.*;
import java.awt.font.FontRenderContext;
import java.awt.font.TextAttribute;
import java.awt.font.TextLayout;
import java.io.Serializable;
import java.text.AttributedCharacterIterator;
import java.text.AttributedString;

/**
 * <code>JCFontChooserPane</code> provides a pane of controls designed to allow
 * a user to manipulate and select a font.  It is suitable for
 * use in a tab pane or a dialog window.
 */
public class JCFontChooserPane extends JCFontChooser
                               implements ActionListener,
                                          FocusListener,
                                          ListSelectionListener,
                                          Serializable
{

/** The no style support constant. */
public static final int      NONE = JCFontChooser.NONE;
/** The bold style support constant. */
public static final int      BOLD = JCFontChooser.BOLD;
/** The italic style support constant. */
public static final int      ITALIC = JCFontChooser.ITALIC;
/** The underline style support constant. */
public static final int      UNDERLINE = JCFontChooser.UNDERLINE;
/** The all styles support constant. */
public static final int      ALL = JCFontChooser.ALL;

private static final int     SAMPLE_PREFERRED_HEIGHT = 100;
private static final int     SAMPLE_TEXT_X_LIMIT = 5;
private static final int     SAMPLE_TEXT_Y_LIMIT = 95;
private static final String  SAMPLE_TEXT = "AaBbYyZz";

protected JTextField  fontNameTextField;
protected JList       fontNameList;
protected JTextField  fontSizeTextField;
protected JList       fontSizeList;
protected JCheckBox   boldCheckbox;
protected JCheckBox   italicCheckbox;
protected JCheckBox   underlineCheckbox;
protected JPanel      stylePanel;

protected boolean     toolTipEnabled;
protected int         styleControls	= ALL;

private JPanel        listPanel;
private SamplePanel   samplePanel;
private boolean       ignoreEvents;

/** Creates a font chooser pane with default initial font. */
public JCFontChooserPane()
{
    super();
    init();
}

/** Creates a font chooser pane with a specified initial font.
 * @param font the initial font
 */
public JCFontChooserPane(Font font)
{
    super(font);
    init();
}

/** Creates a font chooser pane with a specified initial font and underline.
 * @param font the initial font
 * @param underline <code>true</code> if underline is on
 */
public JCFontChooserPane(Font font, boolean underline)
{
    super(font, underline);
    init();
}

/** Creates a font chooser pane with a specified initial font and underline.
 * @param name the font name
 * @param style the style constant for the font
 * @param size the point size of the font
 * @param underline <code>true</code> if underline is on
 */
public JCFontChooserPane(String name, int style, int size, boolean underline)
{
    super(name, style, size, underline);
    init();
}

/** Initializes the component. */
// i18n needed for the labels and titles
protected void init()
{
    // set the layout
	setLayout(new JCGridLayout(3,1));
	JCLocaleManager localeManager = JCLocaleManager.getDefault();
    // add the font name
    JLabel fontLabel = new JLabel(localeManager.getString(
                                         JCLocaleManager.FONT_LABEL));
    fontNameTextField = new JTextField();
    fontNameTextField.setEnabled(false);
    fontNameList = new JList();
    fontNameList.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
	setNameList();
	fontNameList.setListData(getNameList());
	fontNameList.addListSelectionListener(this);

    JPanel namePanel = new JPanel(new BorderLayout());
    namePanel.add(fontNameTextField, BorderLayout.NORTH);
    namePanel.add(new JScrollPane(fontNameList), BorderLayout.CENTER);

    // Add the font size
    JLabel sizeLabel = new JLabel(localeManager.getString(
                                               JCLocaleManager.SIZE_LABEL));
    fontSizeTextField = new JTextField(5);
    fontSizeTextField.setEditable(true);
    fontSizeList = new JList();
    fontSizeList.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
	fontSizeList.setListData(getSizeList());
	fontSizeTextField.addActionListener(this);
	fontSizeTextField.addFocusListener(this);
	fontSizeList.addListSelectionListener(this);

    JPanel sizePanel = new JPanel(new BorderLayout());
    sizePanel.add(fontSizeTextField, BorderLayout.NORTH);
    sizePanel.add(new JScrollPane(fontSizeList), BorderLayout.CENTER);

    listPanel = new JPanel(new JCGridLayout(3, 2, 5, 5));
    listPanel.add(fontLabel);
    listPanel.add(sizeLabel);
    listPanel.add(namePanel);
    listPanel.add(sizePanel);
    add(listPanel);

    // Add the style checkboxes
	stylePanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
	stylePanel.setBorder(new TitledBorder(localeManager.getString(
                                                JCLocaleManager.STYLE)));
	boldCheckbox = new JCheckBox(localeManager.getString(
                                                  JCLocaleManager.BOLD));
	italicCheckbox = new JCheckBox(localeManager.getString(
                                                 JCLocaleManager.ITALIC));
	underlineCheckbox = new JCheckBox(localeManager.getString(
					    JCLocaleManager.UNDERLINE));

	boldCheckbox.addActionListener(this);
	italicCheckbox.addActionListener(this);
	underlineCheckbox.addActionListener(this);

	stylePanel.add(boldCheckbox);
	stylePanel.add(italicCheckbox);
	stylePanel.add(underlineCheckbox);

    add(stylePanel);

    // Add the sample panel
    samplePanel = new SamplePanel();
    samplePanel.setLayout(new FlowLayout());
    samplePanel.setBorder(new TitledBorder(localeManager.getString(
					         JCLocaleManager.SAMPLE)));
    add(samplePanel);

    // Update the UI
    setFontChooserPane();
    setToolTipEnabled(true);
}

/** Sets the font name list.
 * @param nameList array of font names
 */
public void setNameList(String[] nameList)
{
    super.setNameList(nameList);
    //PR34124 events should not be happening
    //while data is reset.
    fontNameList.removeListSelectionListener(this);
	fontNameList.setListData(nameList);
    fontNameList.addListSelectionListener(this);
    fontNameList.setSelectedValue(getFontName(), true);
}

/** Sets the UI. */
protected void setFontChooserPane()
{
	fontNameList.setSelectedValue(getFontName(), true);
	fontNameTextField.setText(getFontName());
	fontSizeList.setSelectedValue((new Integer(getFontSize())).toString(),
                                  true);
	fontSizeTextField.setText((new Integer(getFontSize())).toString());
    boldCheckbox.setSelected(isBold());
    italicCheckbox.setSelected(isItalic());
    underlineCheckbox.setSelected(isUnderline());
    updateSamplePanel();
}

/** Updates the sample text area. */
protected void updateSamplePanel()
{
    samplePanel.repaint();
}

/**
 * Updates the dialog controls.
 */
protected void updateControls()
{
    boldCheckbox.setVisible((styleControls & BOLD) != 0);
    italicCheckbox.setVisible((styleControls & ITALIC) != 0);
    underlineCheckbox.setVisible((styleControls & UNDERLINE) != 0);

    remove(stylePanel);
    if (styleControls != NONE) {
        add(stylePanel, 1);
    }
}

/**
 * Provided for Beans property editors - has no effect.
 * @param s
 */
public void setAbout(String s) {}

/**
 * Returns vendor information.
 * @return vendor information
 */
public String getAbout()
{
    return(JCVersion.getVersionString());
}

/**
 * Sets the font style support variable.
 * @param controls
 */
public void setStyleControls(int controls)
{
	styleControls = controls;
    updateControls();
}

/**
 * Gets the font style support variable.
 * @return font style support variable
 */
public int getStyleControls()
{
	return(styleControls);
}

/** Sets the <i>tool tips text enabled</i> indication.If enabled 
 * (the default setting), the parts of the fontchooserpane will have 
 * individual tooltips suggesting their purpose. Setting the toolTipText for 
 * the JCFontChooserPane 
 * after this method is called, will make the general fontchooserpane tooltip
 * enabled, in spite of this setting.
 * @param toolTipEnabled <code>true</code> to enable
 */
public void setToolTipEnabled(boolean toolTipEnabled){
    JCLocaleManager localeManager = JCLocaleManager.getDefault();
    this.toolTipEnabled = toolTipEnabled;
    if (toolTipEnabled) {
        // i18n for text

    	fontNameList.setToolTipText(localeManager.getString(
                                                  JCLocaleManager.FONT));
    	fontNameTextField.setToolTipText(localeManager.getString(
						JCLocaleManager.FONT));
    	fontSizeList.setToolTipText(localeManager.getString(
					  JCLocaleManager.FONT_SIZE));
    	fontSizeTextField.setToolTipText(localeManager.getString(
				     JCLocaleManager.FONT_SIZE));
    	boldCheckbox.setToolTipText(localeManager.getString(
					   JCLocaleManager.BOLD));
    	italicCheckbox.setToolTipText(localeManager.getString(
				       JCLocaleManager.ITALIC));
    	underlineCheckbox.setToolTipText(localeManager.getString(
					 JCLocaleManager.UNDERLINE));
    }
    else {
	setToolTipText(null);
    	fontNameList.setToolTipText(null);
    	fontNameTextField.setToolTipText(null);
    	fontSizeList.setToolTipText(null);
    	fontSizeTextField.setToolTipText(null);
    	boldCheckbox.setToolTipText(null);
    	italicCheckbox.setToolTipText(null);
    	underlineCheckbox.setToolTipText(null);
    }
}

/** Gets the tool tips text enabled indication.
 * @return tool tips text enabled
 */
public boolean isToolTipEnabled()
{
    return(this.toolTipEnabled);
}

/** Implementation for the <code>ActionListener</code>.
 * @param e
 */
public void actionPerformed(ActionEvent e)
{
    if (e.getSource() == boldCheckbox) {
        setOldFont();
        setBold(((JCheckBox)e.getSource()).isSelected());
        updateSamplePanel();
        fireJCFontEvent(JCFontEvent.BOLD_STYLE_CHANGE);
    }
    else if (e.getSource() == italicCheckbox) {
        setOldFont();
        setItalic(((JCheckBox)e.getSource()).isSelected());
        updateSamplePanel();
        fireJCFontEvent(JCFontEvent.ITALIC_STYLE_CHANGE);
    }
    else if (e.getSource() == underlineCheckbox) {
        setOldFont();
        setUnderline(((JCheckBox)e.getSource()).isSelected());
        updateSamplePanel();
        fireJCFontEvent(JCFontEvent.UNDERLINE_STYLE_CHANGE);
    }
    else if (e.getSource() == fontSizeTextField) {
        processFontSizeTextChange();
    }
}

/** Implementation for the <code>FocusListener</code>.
 * @param e
 */
public void focusGained(FocusEvent e) {};

public void focusLost(FocusEvent e)
{
    if (e.getSource() == fontSizeTextField) {
        processFontSizeTextChange();
    }
}

/** Implementation for the </code>ListSelectionListener</code>.
 * @param e
 */
public void valueChanged(ListSelectionEvent e)
{
    if (ignoreEvents || e.getValueIsAdjusting()) {
	    return;
    }
    if (e.getSource() == fontNameList) {
        setOldFont();
        setFontName(fontNameList.getSelectedValue().toString());
    	fontNameTextField.setText(getFontName());
        updateSamplePanel();
        fireJCFontEvent(JCFontEvent.FONT_NAME_CHANGE);
    }
    else if (e.getSource() == fontSizeList) {
        try {
            setOldFont();
            Object selectedValue = fontSizeList.getSelectedValue();
            setFontSize(Integer.parseInt(selectedValue.toString()));
        	fontSizeTextField.setText((new Integer(getFontSize())).toString());
            updateSamplePanel();
            fireJCFontEvent(JCFontEvent.FONT_SIZE_CHANGE);
        }
        catch (NumberFormatException ex) {}
    }
}

/** Helper method that processes font size text field changes. */
protected void processFontSizeTextChange()
{
    setOldFont();
    setFontSize(Integer.parseInt(fontSizeTextField.getText()));
    ignoreEvents = true;
    fontSizeList.clearSelection();
	fontSizeList.setSelectedValue((new Integer(getFontSize())).toString(),
                                  true);
    ignoreEvents = false;
    updateSamplePanel();
    fireJCFontEvent(JCFontEvent.FONT_SIZE_CHANGE);
}

/** Fires the font changing and font changed events.
 *  @param event the font change event
 */
protected void fireJCFontEvent(int event)
{
    // Fire the font changing event
    JCFontEvent fontEvent = new JCFontEvent(this, event, oldFont, oldUnderline,
                                            getSelectedFont(), isUnderline(),
                                            true);
    fireJCFontChangingEvent(fontEvent);

    // If font change is cancelled, reset to old font
    if (fontEvent.isCancelFontChange()) {
        setFontValues(oldFont, oldUnderline);
        setFontChooserPane();
    }
    // If font changed has been overridden, set to new font
    else if (fontEvent.isFontChanged()) {
        setFontValues(fontEvent.getFont(), fontEvent.getUnderline());
        setFontChooserPane();
    }

    // Fire the font changed event
    if (!fontEvent.isCancelFontChange()) {
        fireJCFontChangedEvent(new JCFontEvent(this, event, oldFont,
                                               oldUnderline, getSelectedFont(),
                                               isUnderline(), false));
    }
}

/** Sets the selected font.
 * @param name the font name
 * @param style the style constant for the font
 * @param size the point size of the font
 * @param underline <code>true</code> if underline is on
 */
public void setSelectedFont(String name, int style, int size,
                            boolean underline)
{
    setOldFont();
    super.setSelectedFont(name, style, size, underline);
    setFontChooserPane();
    fireJCFontEvent(JCFontEvent.FONT_CHANGE);
}

/** Sets the selected font.
 * @param font the selected font
 * @param underline <code>true</code> if underline is on
 */
public void setSelectedFont(Font font, boolean underline)
{
    setSelectedFont(font.getName(), font.getStyle(), font.getSize(),
                    underline);
}

/** Sets the selected font.
 * @param font the selected font
 */
public void setSelectedFont(Font font)
{
    setSelectedFont(font.getName(), font.getStyle(), font.getSize(),
                    JCFontChooser.isUnderline(font));
}


/**
 * Inner Class: <code>SamplePanel</code> is used for the Sample text.
 */
class SamplePanel extends JPanel
{

    public Dimension getMinimumSize()
    {
        return(getPreferredSize());
    }

    public Dimension getMaximumSize()
    {
        return(getPreferredSize());
    }

    public Dimension getPreferredSize()
    {
        if (listPanel != null) {
            Dimension dim = listPanel.getPreferredSize();
            return (new Dimension(dim.width, SAMPLE_PREFERRED_HEIGHT));
        }
        return(new Dimension(234, SAMPLE_PREFERRED_HEIGHT));
    }

    public void paint(Graphics g)
    {
        super.paint(g);

        Graphics2D g2;
        g2 = (Graphics2D) g;
        FontRenderContext frc = g2.getFontRenderContext();
        Font font = getSelectedFont();

        // Note: need this hack section to make changing font with just
        // style change to draw properly
        Font fontX = new Font(font.getName(), font.getStyle(),
                              font.getSize() + 2);
        AttributedString atsX = new AttributedString("Hack");
        atsX.addAttribute(TextAttribute.FONT, fontX);
        AttributedCharacterIterator iterX = atsX.getIterator();
        TextLayout tlX = new TextLayout(iterX, frc);
        // end of hack section

        AttributedString ats = new AttributedString(SAMPLE_TEXT);
        ats.addAttribute(TextAttribute.FONT, font);
        if (isUnderline()) {
            ats.addAttribute(TextAttribute.UNDERLINE,
                             TextAttribute.UNDERLINE_ON);
        }
        AttributedCharacterIterator iter = ats.getIterator();
        TextLayout tl = new TextLayout(iter, frc);
        g2.setColor(Color.black);

        Rectangle rect = g2.getClipBounds();
        FontMetrics fm = g2.getFontMetrics(font);
        int x = (rect.x + rect.width - fm.stringWidth(SAMPLE_TEXT)) / 2;
        x = Math.max(SAMPLE_TEXT_X_LIMIT, x);
        int y = (SAMPLE_PREFERRED_HEIGHT + fm.getAscent()) / 2;
        y = Math.min(SAMPLE_TEXT_Y_LIMIT, y);
        tl.draw(g2, (float)x, (float)y);
    }

} // SamplePanel


/**
 * Shows a modal font-chooser dialog and blocks until the
 * dialog is hidden.
 * @return
 * @param component
 * @param controls
 * @param initialFont
 * @param title
 */
public static Font showDialog(Component component, String title, int controls,
                              Font initialFont)
{
    final JCFontChooserPane pane =
        new JCFontChooserPane(initialFont != null ? initialFont
                                                  : new Font("Dialog",
                                                             Font.PLAIN, 12));
    pane.setStyleControls(controls);

    OKButtonListener okListener = new OKButtonListener(pane);

    JDialog dialog = new FontChooserDialog(component, title, true, pane,
                                           okListener);
    dialog.addWindowListener(new ClosingWindowAdapter());
    dialog.addComponentListener(new DisposingAdapter());

    dialog.show();
    return(okListener.getFont());
}

private static class ClosingWindowAdapter extends WindowAdapter
	                                      implements Serializable
 {

    public void windowClosing(WindowEvent e)
    {
        Window w = e.getWindow();
        w.setVisible(false);
    }

} // ClosingWindowAdapter


private static class DisposingAdapter extends ComponentAdapter
		                              implements Serializable
{
    public void componentHidden(ComponentEvent e)
    {
        Window w = (Window)e.getComponent();
        w.dispose();
    }

} // DisposingAdapter

} // JCFontChooserPane

/**
 * Class which builds a font chooser dialog consisting of
 * a <code>JCFontChooserPane</code> with <B>Ok</B>, <B>Cancel</B>, and
 * <B>Reset</B> buttons.
 */
class FontChooserDialog extends JDialog
{
    private Font               initialFont;
    private JCFontChooserPane  chooserPane;

    public FontChooserDialog(Component c, String title, boolean modal,
                             JCFontChooserPane chooserPane,
                             ActionListener okListener)
    {
        super(JOptionPane.getFrameForComponent(c), title, modal);
        String s = JCEnvironment.SET_RESIZABLE_TO_FALSE_ON_DIALOGS;
        if (JCEnvironment.isWorking(s) == JCEnvironment.WORKING_YES) {
            // Only set resizable if its not going to make the app behave crazy
            setResizable(false);
        }

        this.chooserPane = chooserPane;

        Container contentPane = getContentPane();
        contentPane.setLayout(new BorderLayout());
        contentPane.add(chooserPane, BorderLayout.CENTER);

        // Creates lower button panel
        JPanel buttonPane = new JPanel();
        buttonPane.setLayout(new FlowLayout(FlowLayout.CENTER));
        JButton okButton = new JButton("OK");	// Needs localization!
        getRootPane().setDefaultButton(okButton);
        okButton.setActionCommand("OK");
        if (okListener != null) {
            okButton.addActionListener(okListener);
        }
        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setVisible(false);
            }
        });
        buttonPane.add(okButton);

        JButton cancelButton = new JButton("Cancel");	// Needs localization!

        // The following few lines are used to register esc to close the dialog
        ActionListener cancelKeyAction = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setVisible(false);
            }
        };

        KeyStroke cancelKeyStroke =
            KeyStroke.getKeyStroke((char)KeyEvent.VK_ESCAPE);
        cancelButton.registerKeyboardAction(cancelKeyAction,
                                            cancelKeyStroke,
                                            JComponent.WHEN_IN_FOCUSED_WINDOW);
        // end esc handling

        cancelButton.setActionCommand("cancel");
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setVisible(false);
            }
        });
        buttonPane.add(cancelButton);

        JButton resetButton = new JButton("Reset");		// Needs localization!
        resetButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                reset();
            }
        });
        buttonPane.add(resetButton);
        contentPane.add(buttonPane, BorderLayout.SOUTH);

        pack();
        setLocationRelativeTo(c);
    }

    public void show() {
        initialFont = chooserPane.getSelectedFont();
        super.show();
    }

    public void reset() {
        chooserPane.setSelectedFont(initialFont);
    }

} // FontChooserDialog

class OKButtonListener implements ActionListener, Serializable
{
    Font               font;
    JCFontChooserPane  chooser;

    public OKButtonListener(JCFontChooserPane c)
    {
        chooser = c;
    }

    public void actionPerformed(ActionEvent e)
    {
        font = chooser.getSelectedFont();
    }

    public Font getFont()
    {
        return(font);
    }
} // OKButtonListener

