/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCFontEvent.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:45 $  $Locker:  $  Quest Software Inc.


package com.klg.jclass.util.swing;

import java.awt.Font;
import java.util.EventObject;

/**
 * Fired for font related events.
 */
public class JCFontEvent extends EventObject {

/** Font change event. */
public static final int FONT_CHANGE             = 1;

/** Font name change event. */
public static final int FONT_NAME_CHANGE        = 2;

/** Font size change event. */
public static final int FONT_SIZE_CHANGE        = 3;

/** Font bold style change event. */
public static final int BOLD_STYLE_CHANGE       = 4;

/** Font italic style change event. */
public static final int ITALIC_STYLE_CHANGE     = 5;

/** Font underline style change event. */
public static final int UNDERLINE_STYLE_CHANGE  = 6;

// Properties
private int      event = (-1);
private Font     oldFont = null;
private boolean  oldUnderline = false;
private Font     font = null;
private boolean  underline = false;
private boolean  cancelFontChange = false;
private boolean  fontChanging = false;
private boolean  fontChanged = false;

/**
 * Creates a <code>JCFontEvent</code>.
 * @param event the event
 * @param source a <code>JCFontChooser</code> class
 * @param oldFont the old font
 * @param oldUnderline the old underline indication
 * @param font the new font
 * @param underline the new underline indication
 * @param fontChanging the font changing indication
 */
public JCFontEvent(Object source, int event, Font oldFont, boolean oldUnderline,
                   Font font, boolean underline, boolean fontChanging)
{
    super(source);
    this.event = event;
    this.oldFont = oldFont;
    this.oldUnderline = oldUnderline;
    this.font = font;
    this.underline = underline;
    this.fontChanging = fontChanging;
    this.fontChanged = false;
    this.cancelFontChange = false;
}

/** Retrieves the event.
 * @return the event
 */
public int getEvent()
{
    return(event);
}

/** Retrieves the old font.
 * @return old font
 */
public Font getOldFont()
{
    return(oldFont);
}

/** Retrieves the old underline indication.
 * @return old underlint indication
 */
public boolean getOldUnderline()
{
    return(oldUnderline);
}

/** Retrieves the new font.
 * @return new font
 */
public Font getFont()
{
    return(font);
}

/** Retrieves the new underline indication.
 * @return new underline indication
 */
public boolean getUnderline()
{
    return(underline);
}

/** Sets the new font.
 *  This is available only in <code>FontChanging()</code>.
 * @param font new font
 */
public void setFont(Font font)
{
    if (fontChanging) {
        this.font = font;
        this.underline = JCFontChooser.isUnderline(font);
        fontChanged = true;
    }
}

/** Sets the new font and underline indication.
 *  This is available only in <code>FontChanging()</code>.
 * @param font new font
 * @param underline underline indication
 */
public void setFont(Font font, boolean underline)
{
    if (fontChanging) {
        this.font = font;
        this.underline = underline;
        fontChanged = true;
    }
}

/** Sets the new underline indication.
 *  This is available only in <code>FontChanging()</code>.
 * @param underline underline indication
 */
public void setUnderline(boolean underline)
{
    if (fontChanging) {
        this.underline = underline;
        fontChanged = true;
    }
}

/** Gets the <code>cancelFontChange</code> indication.
 * @return <code>cancelFontChange</code> indication
 */
public boolean isCancelFontChange()
{
    return(this.cancelFontChange);
}

/** Sets the <code>cancelFontChange</code> indication.
 * @param cancelFontChange
 */
public void setCancelFontChange(boolean cancelFontChange)
{
    if (fontChanging) {
        this.cancelFontChange = cancelFontChange;
    }
}

/** Gets the <code>fontChanging</code> indication.
 * @return <code>fontChanging</code> indication
 */
public boolean isFontChanging()
{
    return(this.fontChanging);
}

/** Sets the <code>fontChanging</code> indication.
 * @param fontChanging
 */
void setFontChanging(boolean fontChanging)
{
    this.fontChanging = fontChanging;
}

/** Gets the <code>fontChanged</code> indication.
 * @return <code>fontChanged</code> indication
 */
public boolean isFontChanged()
{
    return(this.fontChanged);
}

/** Sets the <code>fontChanged</code> indication.
 * @param fontChanged
 */
void setFontChanged(boolean fontChanged)
{
    this.fontChanged = fontChanged;
}

}
