/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCHTMLPane.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:45 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.swing;


import com.klg.jclass.util.JCVersion;

import javax.swing.*;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.Document;
import java.awt.*;
import java.io.IOException;
import java.net.URL;

/**
 * Extension of <code>JEditorPane</code> that only handles HTML content and implements
 * to code necessary to traverse links when clicked on by the user.  By
 * default this pane is non-editable.
 */
public class JCHTMLPane extends JEditorPane implements java.io.Serializable {

protected boolean  inLink = false;
protected Cursor   defaultCursor = null;
protected Cursor   handCursor = null;
protected Cursor   waitCursor = null;

/**
 * Constructs a blank HTML pane.
 */
public JCHTMLPane()
{
    super();
    setEditable(false);
    setContentType("text/html");
    addHyperlinkListener(new LinkListener());
}

/**
 * Constructs an HTML pane with the specified URL.
 * @param url
 */
public JCHTMLPane(URL url)
{
    super();
    setEditable(false);
    setContentType("text/html");

    Document document = getDocument();
    try {
        setPage(url);
    }
    catch (IOException e) {
        // restore old document on failure
        setDocument(document);
        getToolkit().beep();
    }
    addHyperlinkListener(new LinkListener());
}

/**
 * Constructs an HTML pane with the specified HTML text.
 * @param text
 */
public JCHTMLPane(String text)
{
    super();
    setEditable(false);
    setContentType("text/html");

    setText(text);
    addHyperlinkListener(new LinkListener());
}

/**
 * Provided for Beans property editors - has no effect.
 * @param s
 */
public void setAbout(String s) {}

/**
 * Returns vendor information.
 * @return vendor information
 */
public String getAbout()
{
    return(JCVersion.getVersionString());
}

public URL getPage()
{
    return(super.getPage());
}

public void setPage(URL url) throws IOException
{
    super.setPage(url);
}


protected void enterLink()
{
    
    inLink = true;
    switchCursorToHand();
}

protected void exitLink()
{
    
    inLink = false;
    restoreCursor();
}

/**
 * Follows the reference in a link.
 * @param url the URL to follow
 */
protected void followLink(URL url)
{
    
    // create a page loader to follow the link
    new PageLoader(url);
}

protected void switchCursorToHand()
{
    if (defaultCursor == null) {
        defaultCursor = JCHTMLPane.this.getCursor();
    }

    if (handCursor == null) {
        handCursor = Cursor.getPredefinedCursor(Cursor.HAND_CURSOR);
    }
    setCursor(handCursor);
}

protected void restoreCursor()
{
    if (inLink) {
        setCursor(handCursor);
    }
    else {
        setCursor(defaultCursor);
    }
}

protected void switchCursorToWait()
{
    if (defaultCursor == null) {
        defaultCursor = JCHTMLPane.this.getCursor();
    }
    if (waitCursor == null) {
        waitCursor = Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);
    }
    setCursor(waitCursor);
}

class LinkListener implements HyperlinkListener, java.io.Serializable {
    /**
     * Notification of a change relative to a hyperlink.
     * @param e
     */
    public void hyperlinkUpdate(HyperlinkEvent e)
    {
        
        HyperlinkEvent.EventType et = e.getEventType();
        if (et == HyperlinkEvent.EventType.ACTIVATED) {
            followLink(e.getURL());
        }
        else if (et == HyperlinkEvent.EventType.ENTERED) {
            // change to a hand
            enterLink();
        }
        else if (et == HyperlinkEvent.EventType.EXITED) {
            // restore cursor
            exitLink();
        }
        else {
            
        }
    }
} // LinkListener

/**
 * Temporary class that loads synchronously (although
 * later than the request so that a cursor change
 * can be done).
 */
class PageLoader extends JCSwingWorker implements java.io.Serializable {
    protected URL         url;
    protected Cursor      savedCursor;
    protected JCHTMLPane  pane;

    /**
     * @param url to set on the parent pane
     */
    public PageLoader(URL url)
    {
        this(JCHTMLPane.this, url);
    }

    /**
     * @param pane html pane to set the specified url in
     * @param url to set on the specified pane
     */
    public PageLoader(JCHTMLPane pane, URL url)
    {
        this.url = url;
        this.pane = pane;
    }

    /**
     * Overridden method is invoked on creation.
     * Guaranteed to be run inside the event dispatch thread
     * before <code>construct()</code> begins.
     */
    public void start()
    {
        switchCursorToWait();
    }

    /**
     * Overridden method that is invoked in its own thread.
     * @return
     */
    public Object construct()
    {
        Runnable r = new Runnable() {
            public void run()
            {
                Document document = pane.getDocument();
                try {
                    pane.setPage(url);
                }
                catch (IOException e) {
                    // restore old document on failure
                    pane.setDocument(document);
                    getToolkit().beep();
                }
            }
        };
        javax.swing.SwingUtilities.invokeLater(r);
        return(null);
    }

    /**
     * Guaranteed to be run inside the event dispatch thread
     * after <code>construct()</code> has been completed.
     */
    public void finished()
    {
        // restore the original cursor
        restoreCursor();
        // force a repaint
        Container parent = pane.getParent();
        parent.repaint();
    }
} // PageLoader

}
