/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCHelpPane.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:45 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.swing;


import com.klg.jclass.util.JCVersion;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.net.URL;
import java.util.Stack;

/**
 * Class that contains two <code>JCHTMLPane</code>s.  Clicks on links in the pane
 * on the left will cause the clicked-on URL to be loaded in the pane
 * on the right.
 */
public class JCHelpPane extends JSplitPane {

protected JCHTMLPane    contentsPane;
protected JCHTMLPane    viewPane;
protected JCHTMLPane    titlePane;
protected JSplitPane    horizontalPane;
protected JSplitPane    verticalPane = null;

protected boolean       useToolBar = true;
protected JToolBar      toolBarCreated;
protected JToolBar      toolBarButtonsAddedTo;
protected JToolBar      toolBarListenersAddedTo;
protected JButton       forwardButton;
protected JButton       backButton;
protected JCAction      forwardAction = new ActionForward();
protected JCAction      backAction = new ActionBack();
protected PageListener  pageListener;

// URL Stack
protected Stack         urlStack = new Stack();
protected int           currentStackPosition = -1;
protected int           movingThroughStack = 0;

public JCHelpPane()
{
    contentsPane = new ContentsPane();
    viewPane = new JCHTMLPane();
    setupPanes();
}

public JCHelpPane(URL contents, URL view)
{
    contentsPane = new ContentsPane(contents);
    viewPane = new JCHTMLPane(view);
    setupPanes();
}

public JCHelpPane(URL contents, URL view, URL title)
{
    contentsPane = new ContentsPane(contents);
    viewPane = new JCHTMLPane(view);
    if (title != null) {
        titlePane = new JCHTMLPane(title);
    }
    setupPanes();
}

public JCHelpPane(String contents, String view)
{
    contentsPane = new ContentsPane(contents);
    viewPane = new JCHTMLPane(view);
    setupPanes();
}

public JCHelpPane(String contents, String view, String title)
{
    contentsPane = new ContentsPane(contents);
    viewPane = new JCHTMLPane(view);
    if (title != null) {
        titlePane = new JCHTMLPane(title);
    }
    setupPanes();
}

public JSplitPane getHorizontalPane()
{
    return(horizontalPane);
}

public JSplitPane getVerticalPane()
{
    return(verticalPane);
}

public void resetHelpStack()
{
    urlStack = new Stack();
    currentStackPosition = -1;
    movingThroughStack = 0;
    backAction.setEnabled(false);
    forwardAction.setEnabled(false);
}

/**
 * Provided for Beans property editors - has no effect.
 * @param s
 */
public void setAbout(String s) {}

/**
 * Returns vendor information.
 * @return vendor information
 */
public String getAbout()
{
    return(JCVersion.getVersionString());
}

/**
 * Returns the HTML pane on the left hand side.
 * @return the HTML pane on the left hand side
 */
public JCHTMLPane getContentsPane()
{
    return(contentsPane);
}

/**
 * Returns the HTML pane on the right hand side.
 * @return the HTML pane on the right hand side
 */
public JCHTMLPane getViewPane()
{
    return(viewPane);
}

/**
 * Sets the contents page.  That is, set the HTML pane on the left hand
 * side.
 * @param page
 */
public void setContentsPage(URL page)
{
    try {
        contentsPane.setPage(page);
    }
    catch (IOException ex) {
        System.out.println("IOException: " + ex);
    }
}

/**
 * Returns the URL displayed in the contents pane.
 * @return URL displayed in the contents pane
 */
public URL getContentsPage()
{
    return(contentsPane.getPage());
}

/**
 * Sets the view page.  That is, sets the HTML pane on the right hand
 * side.
 * @param page
 */
public void setViewPage(URL page)
{
    try {
        viewPane.setPage(page);
    }
    catch (IOException ex) {
        System.out.println("IOException: " + ex);
    }
}

/**
 * Returns the URL displayed in the contents pane.
 * @return URL displayed in the contents pane
 */
public URL getViewPage()
{
    return(viewPane.getPage());
}

/**
 * Sets the title page.  That is, sets the HTML pane at the top.
 * @param page
 */
public void setTitlePage(URL page)
{
    if (page != null && titlePane != null) {
        try {
            titlePane.setPage(page);
        }
        catch (IOException ex) {
            System.out.println("IOException: " + ex);
        }
        return;
    }

    if (page == null) {
        titlePane = null;
    }
    else if (titlePane == null) {
        // create a new title pane
        titlePane = new JCHTMLPane(page);
    }

    // call method to redo layout to include or exclude the title pane
    setupPanes();
}

/**
 * Returns the URL displayed in the title pane.
 * @return URL displayed in the title pane
 */
public URL getTitlePage()
{
    if (titlePane != null) {
        return(titlePane.getPage());
    }
    else {
        return(null);
    }
}

/**
 * If this method returns <code>true</code>, the component traverses up the tree
 * to find its root pane container and adds a tool bar to it if one does not
 * exist.  If one exists, it will add the HTML navigation buttons to the
 * existing toolbar.  If two buttons exist in the tool bar named
 * <B>HelpHTMLBack</B> and <B>HelpHTMLForward</B>, then it will not add the
 * buttons, but rather add listeners to those buttons.
 * @param use
 */
public void setUseToolBar(boolean use)
{
    if (use == useToolBar) {
        return;
    }

    if (use == false) {
        // have to undo anything that might have been done
        if (toolBarButtonsAddedTo != null) {
            // remove the buttons
            toolBarButtonsAddedTo.remove(backButton);
            toolBarButtonsAddedTo.remove(forwardButton);
            backButton = null;
            forwardButton = null;
            toolBarButtonsAddedTo = null;
        }
        else if (toolBarListenersAddedTo != null) {
            backButton.removeActionListener(backAction);
            forwardButton.removeActionListener(forwardAction);
            forwardButton = null;
            backButton = null;
            toolBarListenersAddedTo = null;
        }
        else if (toolBarCreated != null) {
            Container parent = toolBarCreated.getParent();
            parent.remove(toolBarCreated);
            forwardButton = null;
            backButton = null;
            toolBarCreated = null;
        }
    }
    else if (isDisplayable()) {
        connectToToolBar();
    }
    useToolBar = use;
}

/**
 */
public boolean isUseToolBar()
{
    return useToolBar;
}

/**
 * When this object is added to its parent.
 */
public void addNotify()
{
    super.addNotify();
    if (useToolBar == true) {
        connectToToolBar();
    }
}

/**
 * Implementation of toolbar activation behavior as described in
 * <code>setUseToolBar</code>.
 * @see #setUseToolBar
 */
protected void connectToToolBar()
{
    // find the root pane container
    RootPaneContainer rpc = (RootPaneContainer)
        SwingUtilities.getAncestorOfClass(RootPaneContainer.class, this);

    if (rpc == null) {
        //
        return;
    }

    JComponent jc = (JComponent)rpc.getContentPane();

    // look through jc for a toolbar
    JToolBar toolbar = null;
    Component kids[] = jc.getComponents();
    for (int i = 0; i < kids.length; i++) {
        if (kids[i] instanceof JToolBar) {
            toolbar = (JToolBar)kids[i];
            break;
        }
    }

    if (toolbar == null) {
        // toolbar is null so we create one
        toolBarCreated = new JToolBar();
        toolbar = toolBarCreated;
        jc.add(toolbar, "North");
        jc.invalidate();
        jc.validate();
    }
    else {
        // we found a toolbar; so now we look to see if it has forward and
        // back buttons
        // ??? this is where we set toolBarListenersAddedTo and
        // ??? toolBarButtonsAddedTo as necessary
    }

    if (backButton == null) {
        backButton = toolbar.add(backAction);
    }

    if (forwardButton == null) {
        forwardButton = toolbar.add(forwardAction);
    }

    // add listeners to the buttons
    // ???
}

protected void setupPanes()
{
    // clear any configuration that may exist
    if (this.getComponentCount() != 0) {
        this.setTopComponent(null);
        this.setBottomComponent(null);
    }

    if (pageListener == null) {
        // create page listener and install on viewPane
        pageListener = new PageListener();
        viewPane.addPropertyChangeListener(pageListener);
    }

    if (titlePane != null) {
        verticalPane = this;

        // make sure the vertical pane is vertical
        verticalPane.setOrientation(VERTICAL_SPLIT);

        // Create ScrollPane for title component
        JScrollPane title_scrollpane = new JScrollPane();
        title_scrollpane.setViewportView(titlePane);
        int never = title_scrollpane.VERTICAL_SCROLLBAR_NEVER;
        title_scrollpane.setVerticalScrollBarPolicy(never);
        verticalPane.setTopComponent(title_scrollpane);

        // If the horizontalPane equals this then we must create a new one or we
        // will be added this to this later on.
        if ((horizontalPane == this) || (horizontalPane == null)) {
            // need to create one
            horizontalPane = new JSplitPane();
        }
        else if (horizontalPane.getComponentCount() != 0) {
            horizontalPane.setLeftComponent(null);
            horizontalPane.setRightComponent(null);
        }

        verticalPane.setBottomComponent(horizontalPane);
        verticalPane.setDividerLocation(60);
    }
    else {
        verticalPane = null;
        horizontalPane = this;
        // make sure the horizontal pane is horizontal
        horizontalPane.setOrientation(HORIZONTAL_SPLIT);
    }

    // Create ScrollPane for contents component
    JScrollPane left_scrollpane = new JScrollPane();
    // place the contents on the left hand side of the split pane
    left_scrollpane.setViewportView(contentsPane);
    horizontalPane.setLeftComponent(left_scrollpane);

    // create ScrollPane for view component
    JScrollPane right_scrollpane = new JScrollPane();
    // Place the view on the Right hand side of the split pane
    right_scrollpane.setViewportView(viewPane);
    horizontalPane.setRightComponent(right_scrollpane);
    horizontalPane.setDividerLocation(200);
}

protected void back()
{
    if (!backAction.isEnabled()) {
        // back's are not allowed so we ignore the request
        return;
    }

    movingThroughStack++;

    // move back down the stack
    currentStackPosition -= 1;

    enableButtons();

    URL new_page = (URL)urlStack.elementAt(currentStackPosition);
    setViewPage(new_page);
}

protected void forward()
{
    if (!forwardAction.isEnabled()) {
        // forward's are not allowed so we ignore the request
        return;
    }

    movingThroughStack++;

    // move forward up the stack
    currentStackPosition += 1;

    enableButtons();

    URL new_page = (URL)urlStack.elementAt(currentStackPosition);
    setViewPage(new_page);
}

protected void enableButtons()
{
    if (currentStackPosition < 1) {
        // we have no where to go back to
        backAction.setEnabled(false);
    }
    else {
        backAction.setEnabled(true);
    }

    if (currentStackPosition == (urlStack.size() - 1)) {
        // we have no where to move forward to
        // we have no where to go back to
        forwardAction.setEnabled(false);
    }
    else {
        forwardAction.setEnabled(true);
    }
}

class PageListener implements PropertyChangeListener {
    public void propertyChange(PropertyChangeEvent e)
    {
        String propertyName = e.getPropertyName();
        if (propertyName.equals("page")) {
            if (movingThroughStack == 0) {
                // Blow stuff off the stack down to the current page
                for (int i = urlStack.size() - 1;
                     i > currentStackPosition;
                     i--)
                {
                    urlStack.pop();
                }
                urlStack.push(e.getNewValue());
                currentStackPosition = urlStack.size() - 1;
            }

            enableButtons();

            // Decrement flag counter; this is a bit of a hack but in
            // in practice the theoretical holes the the mechanism should
            // not show up
            if (movingThroughStack != 0) {
                movingThroughStack--;
            }
        }
    }
}

/**
 * Inner class that launches any click on URLs in the view pane of
 * its parent.
 */
class ContentsPane extends JCHTMLPane {
    public ContentsPane()
    {
        super();
    }

    public ContentsPane(URL url)
    {
        super(url);
    }

    public ContentsPane(String html_text)
    {
        super(html_text);
    }

    /**
     * Override this method so that we follow the link into the different
     * pane.
     * @param url the URL to follow
     */
    protected void followLink(URL url)
    {
        
        // create a page loader to follow the link
        new PageLoader(viewPane, url);
    }

}

class ActionForward extends JCAction {
    ActionForward()
    {
        super("forward");
        setEnabled(false);
    }

    public void actionPerformed(ActionEvent e)
    {
        forward();
    }
}

class ActionBack extends JCAction {
    ActionBack()
    {
        super("back");
        setEnabled(false);
    }

    public void actionPerformed(ActionEvent e)
    {
        back();
    }
}

}
