/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCRowSortModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.swing;

//import java.util.Comparator;

/**
 * Interface to specify methods for a sortable rows datasource.
 */
public interface JCRowSortModel {

/** Valid value for SortOrder. */
static public final int FORWARD = 1;
/** Valid value for SortOrder. */
static public final int REVERSE = -1;

/**
 * Returns whether the data is automatically sorted according
 * to the current comparator whenever a data change occurs.
 * @return whether data is automatically sorted
 */
public boolean getAutoSort();

/**
 * Sets whether the data should be automatically sorted when it changes.
 * @param autoSort
 */
public void setAutoSort(boolean autoSort);

/**
 * Sets the key columns used to sort the table model when clicking
 * on a specified column.
 * @param column index of column in <code>TableModel</code>
 * @param keyColumns columns to use as keys to sort when sorting
 *     on the specified column
 */
public void setKeyColumns(int column, int keyColumns[]);

/**
 * Returns the key columns used to sort the table model when clicking
 * on the specified column.  If the user has not set the key columns,
 * this returns an array of length 1 with the specified column
 * as the only member.
 * @return key columns
 * @param column
 */
public int[] getKeyColumns(int column);

/**
 * Returns whether or not the data set is currently sorted.
 * @return whether or not the data set is currently sorted
 */
public boolean isDataSorted();

/**
 * Sorts data by current column if set to <code>true</code> and the data needs
 * sorting; returns the data set to its native unsorted state if set to
 * <code>false</code>.
 * @param sorted
 */
public void setDataSorted(boolean sorted);

/**
 * Gets the sorting column. This should return -1 if the data is unsorted.
 * @return sorting column
 */
public int getSortColumn();

/**
 * Sets the column to use the next time a sort is performed.
 * @param column only valid column indicies are allowed
 */
public void setSortColumn(int column);

/**
 * Gets the sorting order to use the next time a sort is performed.
 * @return sorting order
 *
 */
public int getSortOrder();

/**
 * Sets the sorting order.
 * @param order
 */
public void setSortOrder(final int order);

/**
 * Toggles the sorting order.
 */
public void toggleSortOrder();

}
