/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCSortableTable.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.swing;

import com.klg.jclass.cell.EditorRendererRegistry;
import com.klg.jclass.cell.JCCellEditor;
import com.klg.jclass.cell.JCCellRenderer;
import com.klg.jclass.cell.JCCellInfo;
import com.klg.jclass.cell.swing.JCCellEditorToJTable;
import com.klg.jclass.cell.swing.JCCellRendererToJTable;
import com.klg.jclass.util.JCEnvironment;
import com.klg.jclass.util.JCVersion;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.event.TableModelEvent;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Enumeration;
import java.util.Vector;

/**
 * <code>JCSortableTable</code> class.
 */
public class JCSortableTable extends JTable {

protected Icon                 upIcon   = new SortIcon(true);
protected Icon                 downIcon = new SortIcon(false);
protected JCRowSortTableModel  sortModel;
protected MouseListener        tableHeaderMouseListener;
private static final Border    emptyBorder = new EmptyBorder(0, 0, 0, 0);

/**
 * <code>JCSortableTable</code> is a subclass of <code>JTable</code> that
 * internally wraps any <code>TableModel</code> it is given with a
 * <code>JCRowSortTableModel</code> and provides a <code>Comparator</code>
 * that has a configureable list of the column indexes that it uses for sorting.
 * Clicking on a column header invokes the sorting behavior tied to that column,
 * clicking again reverses the sort.
 */
public JCSortableTable()
{
    super();

    if (JCEnvironment.isDesignTime() == true) {
        Vector row1 = new Vector();
        row1.addElement("Row 1, Column 1");
        row1.addElement("Row 1, Column 2");
        Vector row2 = new Vector();
        row2.addElement("Row 2, Column 1");
        row2.addElement("Row 2, Column 2");
        Vector table = new Vector();
        table.addElement(row1);
        table.addElement(row2);
        Vector columns = new Vector();
        columns.addElement("Column 1");
        columns.addElement("Column 2");
        setModel(new DefaultTableModel(table,columns));
    }
}

/**
 * <code>JCSortableTable</code> is a subclass of <code>JTable</code> that
 * internally wraps any <code>TableModel</code> it is given with a
 * <code>JCRowSortTableModel</code> and provides a <code>Comparator</code>
 * that has a configureable list of the column indexes that it uses for sorting.
 * Clicking on a column header invokes the sorting behavior tied to that column,
 * clicking again reverses the sort.
 * @param dm the data model for the table
 */
public JCSortableTable(TableModel dm)
{
	super(dm);
}

/**
 * <code>JCSortableTable</code> is a subclass of <code>JTable</code> that
 * internally wraps any <code>TableModel</code> it is given with a
 * <code>JCRowSortTableModel</code> and provides a <code>Comparator</code>
 * that has a configureable list of the column indexes that it uses for sorting.
 * Clicking on a column header invokes the sorting behavior tied to that column,
 * clicking again reverses the sort.
 * @param dm the data model for the table
 * @param cm the column model for the table
 */
public JCSortableTable(TableModel dm, TableColumnModel cm)
{
	super(dm, cm);
	if (!checkColumnModel(cm)) {
		throw new IllegalArgumentException("Column model incompatible with " +
		                                   "JCSortableTable.  Use " +
										   "createSortableTableColumn() to " +
										   "create table columns.");
	}
}

/**
 * <code>JCSortableTable</code> is a subclass of <code>JTable</code> that
 * internally wraps any <code>TableModel</code> it is given with a
 * <code>JCRowSortTableModel</code> and provides a <code>Comparator</code>
 * that has a configureable list of the column indexes that it uses for sorting.
 * Clicking on a column header invokes the sorting behavior tied to that column,
 * clicking again reverses the sort.
 * @param dm the data model for the table
 * @param cm the column model for the table
 * @param sm the row selection model for the table
 */
public JCSortableTable(TableModel dm, TableColumnModel cm,
                       ListSelectionModel sm)
{
	super(dm, cm, sm);
	if (!checkColumnModel(cm)) {
		throw new IllegalArgumentException("Column model incompatible with " +
		                                   "JCSortableTable.  Use " +
										   "createSortableTableColumn() to " +
										   "create table columns.");
	}
}

/**
 * <code>JCSortableTable</code> is a subclass of <code>JTable</code> that
 * internally wraps any <code>TableModel</code> it is given with a
 * <code>JCRowSortTableModel</code> and provides a <code>Comparator</code>
 * that has a configureable list of the column indexes that it uses for sorting.
 * Clicking on a column header invokes the sorting behavior tied to that column,
 * clicking again reverses the sort.
 * @param numRows the number of rows the table holds
 * @param numColumns the number of columns the table holds
 * @see javax.swing.table.DefaultTableModel
 */
public JCSortableTable(int numRows, int numColumns)
{
    super(numRows, numColumns);
}

/**
 * <code>JCSortableTable</code> is a subclass of <code>JTable</code> that
 * internally wraps any <code>TableModel</code> it is given with a
 * <code>JCRowSortTableModel</code> and provides a <code>Comparator</code>
 * that has a configureable list of the column indexes that it uses for sorting.
 * Clicking on a column header invokes the sorting behavior tied to that column,
 * clicking again reverses the sort.
 * @since JClass 4.0K for Java 2.
 * @param rowData the data for the new table
 * @param columnNames the names of each column
 */
public JCSortableTable(final Vector rowData, final Vector columnNames)
{
    super(rowData, columnNames);
}

/**
 * <code>JCSortableTable</code> is a subclass of <code>JTable</code> that
 * internally wraps any <code>TableModel</code> it is given with a
 * <code>JCRowSortTableModel</code> and provides a <code>Comparator</code>
 * that has a configureable list of the column indexes that it uses for sorting.
 * Clicking on a column header invokes the sorting behavior tied to that column,
 * clicking again reverses the sort.
 * @param rowData the data for the new table
 * @param columnNames the names of each column
 */
public JCSortableTable(final Object[][] rowData, final Object[] columnNames)
{
    super(rowData, columnNames);
}

/**
 * Override this method to allow a different default preferred size for
 * design time.
 * @return the preferred size of the table
 */
public Dimension getPreferredSize()
{
    // Use a nice, compact size for design-time
    if (JCEnvironment.isDesignTime()) {
        return(new Dimension(200, 200));
    }
    else {
        return(super.getPreferredSize());
    }
}

/**
 * Provided for Beans property editors - has no effect.
 * @param s the about String
 */
public void setAbout(String s) {}

/**
 * Returns vendor information.
 * @return the current about String which is the version String
 */
public String getAbout()
{
    return(JCVersion.getVersionString());
}

/**
 * Sets the data model for this table to <code>newModel</code> and registers
 * for listner notifications from the new data model.
 * It wraps the <code>TableModel</code> with a <code>JCRowSortTableModel</code>
 * and adds a mouse listener to the new table header.
 * @param newModel the new data model for the table
 */
public void setModel(TableModel newModel)
{
	if (newModel instanceof JCRowSortTableModel) {
		// Model set is sortable
		sortModel = (JCRowSortTableModel) newModel;
		super.setModel(newModel);
	}
	else {
		// Model is not sortable so we wrap it
		sortModel = new DefaultRowSortTableModel(newModel);
		super.setModel((TableModel) sortModel);
	}
    removeTableHeaderMouseListener();
    addTableHeaderMouseListener();
}

/**
 * Returns the unsorted row index of specified sorted row.
 * @param row the unsorted row index
 * @return the sorted row index
 */
public int getSortedRow(int row)
{
	return(sortModel.getSortedRow(row));
}

/**
 * Returns the unsorted row index of specified sorted row.
 * @param sortedRow the sorted row index
 * @return the unsorted row index
 */
public int getUnsortedRow(int sortedRow)
{
	return(sortModel.getUnsortedRow(sortedRow));
}

/**
 * Returns whether the data is automatically sorted when it changes
 * according the current comparator.
 * @return the current value of <code>autoSort</code>
 */
public boolean getAutoSort()
{
	return(sortModel.getAutoSort());
}

/**
 * Sets whether the data should be automatically sorted when it changes.
 * @param autoSort the new value of <code>autoSort</code>
 */
public void setAutoSort(boolean autoSort)
{
	sortModel.setAutoSort(autoSort);
}

/**
 * Sets the key columns used to sort the table model when clicking
 * on specified column.
 * @param column index of columns in <code>TableModel</code>
 * @param keyColumns columns to use as keys to sort when sorting on the
 *     specified column
 */
public void setKeyColumns(int column, int keyColumns[])
{
	((JCRowSortTableModel)dataModel).setKeyColumns(column, keyColumns);
}

/**
 * Returns the key columns used to sort the table model when clicking
 * on the specified column.  If the user has not set the key columns,
 * the method returns an array of length 1 with the specified column
 * as the only member.
 * @param column the column for which the key columns are required
 * @return the columns to use as keys to sort when sorting on the
 *     specified column
 */
public int[] getKeyColumns(int column)
{
	return ((JCRowSortTableModel)dataModel).getKeyColumns(column);
}

/**
 * Gets the icon used for forward sorting.
 * @return the current icon used for forward sorting
 */
public Icon getUpIcon()
{
    return(upIcon);
}

/**
 * Sets the icon used for forward sorting.
 * @param upIcon the new icon used for forward sorting
 */
public void setUpIcon(Icon upIcon)
{
    this.upIcon = upIcon;
}

/**
 * Gets the icon used for reverse sorting.
 * @return the current icon used for reverse sorting
 */
public Icon getDownIcon()
{
    return(downIcon);
}

/**
 * Sets the icon used for reverse sorting.
 * @param downIcon the new icon used for reverse sorting
 */
public void setDownIcon(Icon downIcon)
{
    this.downIcon = downIcon;
}

/**
 * Overwrites the default implementation to add a <code>MouseListener</code>
 * to the new table header.
 * @param newHeader the new table header
 */
public void setTableHeader(JTableHeader newHeader)
{
    if (getTableHeaderMouseListener() != null) {
        removeTableHeaderMouseListener();
    }
	super.setTableHeader(newHeader);
    addTableHeaderMouseListener();
}

/**
 * Gets the cell editor for a given cell.
 * @param row the row index of the specified cell
 * @param column the column index of the specified cell
 * @return the current cell editor for the cell
 */
public TableCellEditor getCellEditor(int row, int column)
{
    ComponentOrientation o = getComponentOrientation();
    TableColumn tableColumn = getColumnModel().getColumn(column);
    TableCellEditor editor = tableColumn.getCellEditor();
    if ((editor == null) && (tableColumn instanceof JCTableColumn)) {
		EditorRendererRegistry registry =
		    EditorRendererRegistry.getCentralRegistry();
		Class c = getColumnClass(column);
		String type = ((JCTableColumn)tableColumn).getRegistryType();
		JCCellEditor jcEditor = registry.getCellEditorInstance(c, type);
        if (jcEditor == null) {
            editor = null;
        }
        else {
            editor = new JCCellEditorToJTable(jcEditor);
        }
    }
    if (editor == null) {
        editor = getDefaultEditor(getColumnClass(column));
    }
    Component ec = editor.getTableCellEditorComponent(this, null, false,
                                                      row, column);
    setComponentOrientation(ec, o, false);
    return(editor);
}

    /**
 * Gets the cell renderer for a given cell.
 * @param row the row index of the specified cell
 * @param column the column index of the specified cell
 * @return the current cell renderer for the cell
 */
public TableCellRenderer getCellRenderer(int row, int column)
{
    ComponentOrientation o = getComponentOrientation();
    TableColumn tableColumn = getColumnModel().getColumn(column);
    TableCellRenderer renderer = tableColumn.getCellRenderer();
    if ((renderer == null) && (tableColumn instanceof JCTableColumn)) {
		EditorRendererRegistry registry =
		    EditorRendererRegistry.getCentralRegistry();
		Class c = getColumnClass(column);
		String type = ((JCTableColumn)tableColumn).getRegistryType();
        JCCellRenderer jcRenderer = registry.getCellRendererInstance(c, type);
        if (jcRenderer == null) {
            renderer = null;
        }
        else {
            // Note that JCLightCellRenderer's use the horizontal alignment
            // so we need to set it here.
            JCCellRendererToJTable jcCellRenderer =
                new JCCellRendererToJTable(jcRenderer);
            int align = o.isLeftToRight() ? JCCellInfo.LEFT : JCCellInfo.RIGHT;
            jcCellRenderer.setHorizontalAlignment(align);
            renderer = jcCellRenderer;
        }
    }
    if (renderer == null) {
        renderer = getDefaultRenderer(getColumnClass(column));
    }
    Component rc = renderer.getTableCellRendererComponent(this, null, false,
                                                          false, row, column);
    setComponentOrientation(rc, o, false);
    return(renderer);
}

/**
 * Overwrites the default implementation to remove a <code>MouseListener</code>
 * from the table header.
 */
protected void configureEnclosingScrollPane()
{
    removeTableHeaderMouseListener();
	super.configureEnclosingScrollPane();
    addTableHeaderMouseListener();
}

/**
 * Adds a <code>MouseListener</code> to the table header.
 */
protected void addTableHeaderMouseListener()
{
    if (getTableHeader() != null) {
        if (getTableHeaderMouseListener() == null) {
            setTableHeaderMouseListener(createTableHeaderMouseListener());
        }
        getTableHeader().addMouseListener(getTableHeaderMouseListener());
    }
}

/**
 * Removes a <code>MouseListener</code> from the table header.
 */
protected void removeTableHeaderMouseListener()
{
    if (getTableHeader() != null) {
        getTableHeader().removeMouseListener(getTableHeaderMouseListener());
    }
}

/**
 * Gets the current table header mouse listener.
 * @return the current table header mouse listener
 */
protected MouseListener getTableHeaderMouseListener()
{
    return(tableHeaderMouseListener);
}

/**
 * Sets the table header mouse listener.
 * @param l the new table header mouse listener
 */
protected void setTableHeaderMouseListener(MouseListener l)
{
    tableHeaderMouseListener = l;
}

/**
 * Creates a new table header mouse listener.
 * @return the new table header mouse listener
 */
protected MouseAdapter createTableHeaderMouseListener()
{
    return(new TableHeaderMouseListener());
};

/**
 * <code>MouseAdapter</code> for the table header.
 */
protected class TableHeaderMouseListener extends MouseAdapter
							             implements java.io.Serializable
{
    protected boolean isPopupTrigger;

    /**
     * Invoked when a mouse button has been pressed on a component.
     * @param e the event generated by the mouse press
     */
    public void mousePressed(MouseEvent e)
    {
        isPopupTrigger = e.isPopupTrigger();
    }

	/**
     * Invoked when a mouse button has been released on a component.
	 * @param e the event generated by the mouse release
	 */
    public void mouseReleased(MouseEvent e)
    {
        isPopupTrigger = (isPopupTrigger || e.isPopupTrigger());
    }

    public void mouseClicked(MouseEvent e)
    {
        // If either the mouse press or mouse release is a popup trigger then
        // do not do the sort.  Note that whether it happens on mouse press
        // or release is system dependent.
        if (!isPopupTrigger) {
            int column = columnAtPoint(e.getPoint());
            if (column >= 0) {
                sort(column);
            }
        }
    }
}

/**
 * Sorts rows using the quicksort algorithm.
 * @param column the column to sort on
 */
public void sort(int column)
{
    if (!(dataModel instanceof JCRowSortTableModel)) {
		String s = "TableModel is not an instance of JCRowSortTableModel";
        throw new IllegalArgumentException(s);
    }

    // Close the current editor
    if (isEditing() && cellEditor != null) {
        if (!cellEditor.stopCellEditing()) {
			String s = "Sort cannot be performed while a cell is editing.";
			JCMessageHelper.showError("Error", s);
            return;
        }
    }

	// Transform the column to be relative to the data source instead of the
	// view
	column = convertColumnIndexToModel(column);
    JCRowSortTableModel jcRowSortTableModel = (JCRowSortTableModel)dataModel;
    if (jcRowSortTableModel.getSortColumn() == column) {
		// Toggle the sorting order
        jcRowSortTableModel.toggleSortOrder();
	}
	else {
        jcRowSortTableModel.setSortColumn(column);
	}

	if (jcRowSortTableModel.isDataSorted() == false) {
		// If autosort was on then we don't need to force it to sort again
        jcRowSortTableModel.setDataSorted(true);
	}
}

/**
 * Restores the unsorted order.
 */
public void unsort()
{
    if (!(dataModel instanceof JCRowSortTableModel)) {
		String s = "TableModel is not an instance of JCRowSortTableModel";
        throw new IllegalArgumentException(s);
    }
    ((JCRowSortTableModel)dataModel).setDataSorted(false);
}

/**
 * Overrides super class method to check for a change in sorting.
 * @param e the event that caused the table to change
 */
public void tableChanged(TableModelEvent e)
{
    // Remove any sort icons we may have
    clearSortIcons();

   	if (sortModel.isDataSorted()) {
        int sortColumn = ((JCRowSortTableModel)dataModel).getSortColumn();
        if (sortColumn >= 0) {
		    setSortIcon(convertColumnIndexToView(sortColumn),
            	        ((JCRowSortTableModel)dataModel).getSortOrder());
        }
	}
	super.tableChanged(e);
}

/**
 * Overridden from superclass to allow autocreation of our own column model.
 * @see javax.swing.JTable#createDefaultColumnsFromModel
 */
public void createDefaultColumnsFromModel()
{
	TableModel m = getModel();
	if (m != null) {
		// Remove any current columns
		TableColumnModel cm = getColumnModel();
		cm.removeColumnModelListener(this);
		while (cm.getColumnCount() > 0) {
			cm.removeColumn(cm.getColumn(0));
		}

		// Create new columns from the data model info
		for (int i = 0; i < m.getColumnCount(); i++) {
			TableColumn newColumn = createSortableTableColumn(i);
			addColumn(newColumn);
		}
		cm.addColumnModelListener(this);
	}
}

/**
 * Makes sure passed in column model will work with <code>JCSortableTable</code>.
 * @param cm the column model
 * @return a boolean indicating whether the column model is compatible with
 *     <code>JCSortableTable</code>
 */
protected boolean checkColumnModel(TableColumnModel cm)
{
	if (cm != null) {
		// First check if we have the correct type of columns
		for (int i = 0; i < cm.getColumnCount(); i++) {
			TableColumn column = cm.getColumn(i);
			if (!(column instanceof JCTableColumn)) {
				return(false);
			}
		}

		// Passed the first check.  Now see if a header renderer is set.
		// If not, set one.
		for (int i = 0; i < cm.getColumnCount(); i++) {
			TableColumn column = cm.getColumn(i);
			if (column instanceof SortableTableColumn ||
			    column.getHeaderRenderer() != null)
			{
				continue;
			}
            boolean isLeft = getComponentOrientation().isLeftToRight();
			column.setHeaderRenderer(new SortableHeaderRenderer(isLeft));
		}
	}
	// If null, we return true because JTable checks for this and throws the
	// appropriate exception.
	return(true);
}

/**
 * This <code>JTable</code> method is overridden to check and see if the passed
 * in column model is compatible with <code>JCSortableTable</code>.
 * @param columnModel the new column model
 */
public void setColumnModel(TableColumnModel columnModel)
{
	if (!checkColumnModel(columnModel)) {
		throw new IllegalArgumentException("Column model incompatible with " +
		                                   "JCSortableTable.  Use " +
										   "createSortableTableColumn() to " +
										   "create table columns.");
	}
	super.setColumnModel(columnModel);
}

/**
 * This method creates a <code>TableColumn</code> for use in
 * <code>JCSorttableTable</code>. Override it in subclasses to easily change
 * which type of Column is created by default.
 * @param column the column for which the <code>TableColumn</code> is to be
 *     created
 * @return the newly created <code>TableColumn</code>
 */
public TableColumn createSortableTableColumn(int column)
{
	return(new SortableTableColumn(column));
}

/**
 * Removes sorting icons from the column headers.
 */
protected void clearSortIcons()
{
	for (int i = 0; i < columnModel.getColumnCount(); i++) {
    	setSortIcon(i, 0);
    }
}

/**
 * Sets or removes the sorting icon.
 * @param column the column on which the sort icon is to be set
 */
protected void setSortIcon(int column)
{
	setSortIcon(column, ((JCRowSortTableModel)dataModel).getSortOrder());
}


/**
 * Looks at the renderer (and possibly its children) and sets the sort icon on
 * any <code>JLabel</code>s or <code>JButton</code>s that are found.
 * @param renderer the renderer on which the <code>sortIcon</code> is to be set
 * @param sortIcon the new <code>sortIcon</code> to be used for the renderer
 */
protected void setSortIcon(Component renderer, Icon sortIcon)
{
	if (renderer instanceof SortableColumnHeader) {
        ((SortableColumnHeader)renderer).setSortIcon(sortIcon);
        tableHeader.repaint();
	}
    else if (renderer instanceof JLabel) {
    	((JLabel)renderer).setHorizontalTextPosition(SwingConstants.LEADING);
        ((JLabel)renderer).setIcon(sortIcon);
        tableHeader.repaint();
    }
    else if (renderer instanceof JButton) {
    	((JButton)renderer).setHorizontalTextPosition(SwingConstants.LEADING);
        ((JButton)renderer).setIcon(sortIcon);
        ((AbstractTableModel)dataModel).fireTableDataChanged();
        tableHeader.repaint();
    }
	else if ((renderer instanceof Container) &&
			 ((Container) renderer).getComponentCount() != 0)
	{
		// We have children so we search through them for a label or JButton
		Component kids[] = ((Container) renderer).getComponents();
		for (int i = 0; i < kids.length; i++) {
			setSortIcon(kids[i], sortIcon);
		}
	}
}

/**
 * Sets or removes the sorting icon.
 * @param column the column on which the sort icon is to be set
 * @param order Determines whether the up or down icon is to be used.  Possible
 *     values are <code>JCRowSortModel.FORWARD</code> and
 *     <code>JCRowSortModel.REVERSE</code>. Any other values causes no icon to
 *     be used.
 */
protected void setSortIcon(int column, final int order)
{
    Icon sortIcon = null;
	if (order == JCRowSortModel.FORWARD) {
		sortIcon = getDownIcon();
	}
	else if (order == JCRowSortModel.REVERSE) {
		sortIcon = getUpIcon();
	}

    TableColumn tableColumn = columnModel.getColumn(column);
    TableCellRenderer renderer = tableColumn.getHeaderRenderer();

	if (renderer != null) {
	    Component comp =
	        renderer.getTableCellRendererComponent(this,
											       tableColumn.getHeaderValue(),
											       /* ??? */ false,
											       false, -1, column);
	    setSortIcon(comp, sortIcon);
	}
}

/**
 * Set component orientation on component c to the passed in orientation if its
 * different.  Also, only change the orientation if it has not been explicitly
 * set before.
 * @param c The component whose orientation may be set.
 * @param o The new orientation
 * @param force If true, force an orientation change, even if it has been
 * explicitly set before.
 */
protected static void setComponentOrientation(Component c,
                                              ComponentOrientation o,
                                              boolean force)
{
    if (c == null || o == null) {
        return;
    }
    ComponentOrientation co = c.getComponentOrientation();
    if (o.isLeftToRight() != co.isLeftToRight() &&
        (force || co == ComponentOrientation.UNKNOWN))
    {
        // Only change the orientation if it has not been explicitly set before.
        // Note this means that once its set, if the user switches the table's
        // orientation, we may or may not switch it back (depending on whether
        // o is UNKNOWN or not).
        c.setComponentOrientation(o);
        c.repaint();
    }
}

/**
 * Override so that the layout on the header renderers can be redone.
 * <p>
 * <b>Note:</b>
 * Due to problems with column ordering in JTable, JCSortableTable and
 * JCTreeTable will not behave properly for a right to left orientation for
 * JDKs prior to 1.4.0.
 * @param o The new component orientation.
 */
public void setComponentOrientation(ComponentOrientation o)
{
    super.setComponentOrientation(o);
    reLayoutColumnRenderers();
    if (isEditing()) {
        Component ec = getEditorComponent();
        setComponentOrientation(ec, o, false);
    }
    revalidate();
    getTableHeader().repaint();
    repaint();
}

/**
 * Iterate through the columns to get the renderer so that we can reset the
 * horizontal alignment on JCCellRenderers.
 */
protected void reLayoutColumnRenderers()
{
    // Enumerate through the columns
    Enumeration e = getColumnModel().getColumns();
    while (e.hasMoreElements()) {
        TableColumn column = (TableColumn)e.nextElement();
        TableCellRenderer renderer = column.getCellRenderer();
        if (renderer instanceof JCCellRendererToJTable) {
            boolean isLeft = getComponentOrientation().isLeftToRight();
            int align = isLeft ? JCCellInfo.LEFT : JCCellInfo.RIGHT;
            ((JCCellRendererToJTable)renderer).setHorizontalAlignment(align);
        }
    }
}

/**
 * Internal sorting icon class.
 */
public class SortIcon implements Icon, java.io.Serializable
{
	protected boolean  isUp;
    protected Color    color        = Color.black;
	protected Color    outlineColor = Color.gray;
    protected int      width        = 6;
    protected int      height       = 6;
    protected int      xpoints[]    = new int[3];
    protected int      ypoints[]    = new int[3];

    /**
     * Constructs a new <code>SortIcon</code> instance for up or down arrow.
	 *
	 * @param isUp if <code>true</code>, this icon draws an up arrow instead of
     * a down arrow
     */
    public SortIcon(final boolean isUp)
	{
    	this.isUp = isUp;
    }

    /**
     * Constructs a new <code>SortIcon</code> instance.
	 *
	 * @param isUp if <code>true</code>, this icon draws an up arrow instead of
     * a down arrow
	 * @param fillColor the fill color of the arrow
	 * @param outlineColor the outline color of the arrow
     */
    public SortIcon(final boolean isUp, Color fillColor, Color outlineColor)
	{
    	this.isUp         = isUp;
		this.color        = fillColor;
		this.outlineColor = outlineColor;
    }

    /**
     * Returns the icon's fill color.
	 * @return the current icon's fill color
     */
    public Color getColor()
	{
    	return(color);
    }

    /**
     * Sets the icon's fill color.
	 * @param color the new icon's fill color
     */
    public void setColor(Color color)
	{
		if (color != null) {
    	    this.color = color;
		}
    }

    /**
     * Returns the icon's outline color.
	 * @return the current icon's outline color
     */
    public Color getOutlineColor()
	{
    	return(outlineColor);
    }

    /**
     * Sets the icon's outline color.
	 * @param color the new icon's outline color
     */
    public void setOutlineColor(Color color)
	{
		if (color != null) {
    	    this.outlineColor = color;
		}
    }

	//================== Icon interface methods ==================

    /**
     * Draws the icon at the specified location.
	 * @param c Not used. Some icon implementations use this Component argument
	 *          to get properties useful for  painting; for example, the
     *          foreground or background color.
	 * @param g the graphics object to do the painting
	 * @param x the <i>X</i> position at which to paint
	 * @param y the <i>Y</i> position at which to paint
     */
    public void paintIcon(Component c, Graphics g, int x, int y)
	{
        if (isUp) {
            // draw up arrow
            xpoints[0] = x;
            ypoints[0] = y + height;
            xpoints[1] = x + width/2;
            ypoints[1] = y;
            xpoints[2] = x + width;
            ypoints[2] = y + height;
        }
        else {
            // draw down arrow
            xpoints[0] = x;
            ypoints[0] = y;
            xpoints[1] = x + width/2;
            ypoints[1] = y + height;
            xpoints[2] = x + width;
            ypoints[2] = y;
        }
    	g.setColor(color);
        g.fillPolygon(xpoints, ypoints, 3);
    	g.setColor(outlineColor);
        g.drawPolygon(xpoints, ypoints, 3);
    }

    /**
     * Returns the icon's width.
	 * @return the current icon's width
     */
    public int getIconWidth()
	{
    	return(width);
    }

    /**
     * Returns the icon's height.
	 * @return the current icon's height
     */
    public int getIconHeight()
	{
    	return(height);
    }
} // SortIcon

/**
 * Overridden version of <code>TableColumn</code> that returns
 * <code>SortableColumnRenderer</code> instead of the normal default.
 */
protected class SortableTableColumn extends JCTableColumn
				                    implements java.io.Serializable
{
    public SortableTableColumn(int column)
    {
        super(column);
        setHeaderRenderer(createDefaultHeaderRenderer());
    }

	/**
	 * Overridden to provide a renderer that reserves a location on
	 * the right for a sort icon if one exists.
	 * @return the new header renderer
	 */
    protected TableCellRenderer createDefaultHeaderRenderer()
	{
        // The following is done because this method may be called from
        // JCSortableTable's or subclass constructor in which case
        // JCSortableTable.this has not yet been set.
        boolean isLeft = ComponentOrientation.UNKNOWN.isLeftToRight();
        if (JCSortableTable.this != null) {
            isLeft =
                JCSortableTable.this.getComponentOrientation().isLeftToRight();
        }
		TableCellRenderer renderer = new SortableHeaderRenderer(isLeft);
		return(renderer);
    }
} // SortableTableColunm

/**
 * Renderer/editor that has two components: the label and the
 * sort order icon, which is stored in a button.
 */
protected class SortableHeaderRenderer extends JPanel
							           implements TableCellRenderer,
										          SortableColumnHeader,
										          java.io.Serializable
{
	protected JLabel           label      = new JLabel("label");
	protected JButton          sortButton = new JButton();
	protected JCBrace          brace      = JCBrace.createHorizontalBrace(3);
    protected Icon             sortIcon   = null;
    protected boolean          currentLayoutLeft;

    /**
     * Constructor that sets up the layout using a left to right orientation.
     * No icon (stored in the <code>sortButton</code>) to start.
     */
    public SortableHeaderRenderer()
    {
        this(true);
    }

	/**
	 * Constructor that sets up the layout using a passed in orientation
     * indicator. No icon (stored in the <code>sortButton</code>) to start.
     * @param orientationLeft If true, the renderer is oriented leftToRight.
	 */
    public SortableHeaderRenderer(boolean orientationLeft)
    {
        JCElasticLayout l = new JCElasticLayout(JCElasticLayout.HORIZONTAL);
        l.setAlignment(SwingConstants.CENTER);
        setLayout(l);

        sortButton.setBorder(emptyBorder);
        sortButton.setVerticalAlignment(SwingConstants.CENTER);
        currentLayoutLeft = label.getComponentOrientation().isLeftToRight();
        reLayout(orientationLeft);
    }

    /**
     * Is the most recent layout, left to right oriented.
     * @return true if the most recent layout is left to right.
     */
    public boolean isCurrentLayoutLeft()
    {
        return(currentLayoutLeft);
    }

    /**
     * Set properties on the header renderer component based on the table
     * header.
     * @param table The table whose header is used to set the properties.
     */
    protected void setProperties(JTable table)
    {
        if (table == null) {
            return;
        }

        JTableHeader header = table.getTableHeader();
        if (header != null) {
            label.setForeground(header.getForeground());
            label.setBackground(header.getBackground());
            label.setFont(header.getFont());
            sortButton.setBackground(header.getBackground());
            sortButton.setForeground(header.getForeground());
            this.setBackground(header.getBackground());
            this.setForeground(header.getForeground());
        }
    }

	/**
     *  Returns the component used for drawing the cell.  This method is
     *  used to configure the renderer appropriately before drawing.
     *
     * @param table the <code>JTable</code> that is asking the  renderer to
	 *     draw; can be <code>null</code>
     * @param value	The value of the cell to be rendered.  It is up to the
	 *     specific renderer to interpret and draw the value.  For example, if
     *	   <code>value</code> is the String <code>true</code>, it could be
     *     rendered as a String, or it could be rendered as a check box that is
     *     checked. <code>null</code> is a valid value.
     * @param isSelected <code>true</code> if the cell is to be rendered with
     *     the selection highlighted; otherwise <code>false</code>
     * @param hasFocus If <code>true</code>, render cell appropriately.  For
     *     example, put a special border on the cell, if the cell can be edited,
     *     rendered in the color used to indicate editing.
     * @param row The row index of the cell being drawn.  When drawing the
	 *     header, the value of <code>row</code> is -1
     * @param column the column index of the cell being drawn
	 * @return the component used for drawing the cell at the specified row
	 *     and column index with the given value
	 */
	public Component getTableCellRendererComponent(JTable table,
							                       Object value,
												   boolean isSelected,
												   boolean hasFocus,
						                           int row, int column)
	{
		if (table != null) {
		    TableColumn tableColumn = table.getColumnModel().getColumn(column);
			if (tableColumn != null && tableColumn instanceof JCTableColumn) {
				JCTableColumn jcTableColumn = (JCTableColumn)tableColumn;
				int alignment =
                    jcTableColumn.getHeaderLabelHorizontalAlignment();
				if (alignment >= 0) {
					label.setHorizontalAlignment(alignment);
				}
			}

            setProperties(table);
        }

        setBorder(UIManager.getBorder("TableHeader.cellBorder"));
		label.setText((value == null) ? "" : value.toString());
        ComponentOrientation o = getComponentOrientation();
        boolean isLeft = true;
        if (table == null || o != ComponentOrientation.UNKNOWN) {
            isLeft = o.isLeftToRight();
        }
        else {
            isLeft = table.getComponentOrientation().isLeftToRight();
        }
        if (isLeft != currentLayoutLeft) {
            reLayout(isLeft);
        }
		return(this);
	}

    /**
	 * Sets the icon to use to draw the sort icon.
	 * @param sortIcon the new sort icon to use
	 */
	public void setSortIcon(Icon sortIcon)
	{
        this.sortIcon = sortIcon;
        if (sortIcon != null) {
            sortButton.setIcon(sortIcon);
        }
        reLayout(currentLayoutLeft);
	}

    /**
     * Remove all components and add the components back in based on their
     * availability and the table's component orientation.
     * @param orientationLeft If true, layout in left to right orientation.
     */
    public void reLayout(boolean orientationLeft)
    {
        removeAll();

        // Switch orientation on the label if necessary
        if (orientationLeft != currentLayoutLeft) {
            ComponentOrientation o =
                orientationLeft ? ComponentOrientation.LEFT_TO_RIGHT
                                : ComponentOrientation.RIGHT_TO_LEFT;
            JCSortableTable.setComponentOrientation(label, o, true);
        }

        // Layout according to orienation
        if (orientationLeft) {
           add(label, JCElasticLayout.HORIZONTALLY_ELASTIC_CONSTRAINT);
        }
        if (sortIcon != null) {
            add(sortButton);
            add(brace);
        }
        if (!orientationLeft) {
            add(label, JCElasticLayout.HORIZONTALLY_ELASTIC_CONSTRAINT);
            add(JCBrace.createHorizontalBrace(1));
        }
        currentLayoutLeft = orientationLeft;
    }
}

/**
 * Interface required by the column header component (if not a
 * <code>JLabel</code> or <code>JButton</code>) to allow setting of the
 * sort icon.
 */
static public interface SortableColumnHeader
{
	/**
	 * Sets the icon to use to draw the sort icon.
	 * @param sortIcon the new sort icon to use
	 */
	public void setSortIcon(Icon sortIcon);
}

}
