/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCSpinBox.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.swing;


import com.klg.jclass.util.JCListenerList;
import com.klg.jclass.util.JCVersion;
import com.klg.jclass.util.value.JCValueEvent;

import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.io.Serializable;
import java.util.Enumeration;
import java.util.Vector;


/**
 *  A spinbox component.  This spin box will display a list of objects.  The
 *  renderer and editor can be replaced to display the and edit the objects in
 *  any form.  The default editor and renderer is designed to work with Strings.
 */
public class JCSpinBox extends AbstractSpinBox implements ItemSelectable  {

protected JCSpinBoxModel       model;
protected JCListenerList       itemListeners = null;
protected JCListenerList       actionListeners = null;
protected String               actionCommand = "spinBoxChanged";
protected ListDataListener     listDataListener = new ListListener();
protected boolean              modelChanging = false;
protected KeySelectionManager  keySelectionManager;
protected Object               lastSelectedItem = null;
protected boolean              firstPaint = true;

/**
 * Creates a <code>JCSpinBox</code> that takes its items from an existing
 * <code>JCSpinBoxModel</code>.
 * @param aModel the <code>SpinBoxModel</code> that provides the displayed list
 * of items
 */
public JCSpinBox(JCSpinBoxModel aModel)
{
    super();
	setModel(aModel);
	setOpaque(true);
	updateUI();
}

/**
 * Creates a <code>JCSpinBox</code> that contains the elements in the specified
 * array.
 * @param items
 */
public JCSpinBox(final Object items[])
{
	super();
	setModel(new DefaultSpinBoxModel(items));
	updateUI();
}

/**
 * Creates a <code>JCSpinBox</code> that contains the elements in the specified
 * <code>Vector</code>.
 * @since JClass 4.0K for Java 2. If you are using JClass 4.0J,
 * which is based on JDK 1.1, this method's parameter must be
 * a <code>com.sun.java.util.collections.Vector</code>.
 * @param items
 */
public JCSpinBox(Vector items)
{
	super();
	setModel(new DefaultSpinBoxModel(items));
	updateUI();
}

/**
 * Creates a <code>JCSpinBox</code> with a default data model.
 * The default data model is an empty list of objects.
 * Use <code>addItem</code> to add items.
 */
public JCSpinBox()
{
	super();
	setModel(new DefaultSpinBoxModel());
	updateUI();
}

/**
 * Paints the spinbox.
 * @param g
 */
public void paint(Graphics g)
{
    // If not editable and the first painting, spin to the first item.
    // a non editable spinbox should not be blank.  This will prevent it.
    if ((!this.isEditable()) && firstPaint) {
        firstPaint = false;
        // but first see if the user has explicitly set a value
        Object value = getModel().getSelectedItem();
        if (value == null) {
        	value = getItemAt(0);
        }
        setValue(value);
    }

    super.paint(g);
}


/**
 * Returns the new value generated when spinning up a notch. Currently we only
 * support wraparound lists in this control.
 * @return new value generated when spinning up a notch
 */
protected Object spinUp()
{
	int i = getSelectedIndex();
	if (++i < getItemCount()) {
		
		return(getItemAt(i));
	}
	else {
		
		return(getItemAt(0));
	}
}

/**
 * Returns the new value generated when spinning down a notch. Currently we only
 * support wraparound lists in this control.
 * @return new value generated when spinning down a notch
 */
protected Object spinDown()
{
	int i = getSelectedIndex();
	if (--i >= 0) {
		
		return getItemAt(i);
	}
	else {
		
		return getItemAt(getItemCount() - 1);
	}
}

/**
 * Returns an array containing the selected item. This method is implemented for
 * compatibility with <code>ItemSelectable</code>.
 *
 * @return an array of Objects containing one element -- the selected item
 */
public Object[] getSelectedObjects()
{
	Object selectedObject = getSelectedItem();
	if (selectedObject == null) {
		return(new Object[0]);
	}
	else {
		Object result[] = new Object[1];
		result[0] = selectedObject;
		return(result);
	}
}

protected void processValueChanged(JCValueEvent event)
{
	if (modelChanging) {
		
		// We are invoked from a ListDataEvent so do not propigate
		// the change to the model
		return;
	}

	
	model.setSelectedItem(event.getNewValue());
	// Call super to notify value listeners of change
	super.processValueChanged(event);
}

/**
 * Disables the arrow buttons if the end of the range is reached. This is an
 * empty implementation since all spinning is wrap around.
 */
protected void checkArrowButtons() {}

/**
 * Provided for Beans property editors - has no effect.
 * @param s
 */
public void setAbout(String s) {}

/**
 * Returns vendor information.
 * @return vendor information
 */
public String getAbout()
{
    return(JCVersion.getVersionString());
}


/**
 * Sets the data model that the <code>JCSpinBox</code> uses to obtain the list
 * of items.
 * @param aModel the <code>JCSpinBoxModel</code> that provides the displayed
 * list of items
 */
public void setModel(JCSpinBoxModel aModel)
{
	JCSpinBoxModel old_model = model;
	if (model != null) {
		model.removeListDataListener(listDataListener);
	}
	model = aModel;
	firePropertyChange("model", old_model, model);
	model.addListDataListener(listDataListener);

    if (isEditable() == false) {
        this.setValue(model.getElementAt(0));
    }
	invalidate();
}

/**
 * Returns the data model currently used by the <code>JCSpinBox</code>.
 *
 * @return the <code>JCSpinBoxModel</code> that provides the displayed list of
 * items
 */
public JCSpinBoxModel getModel()
{
	return(model);
}


/**
 * Returns the list item at the specified index.
 *
 * @param index  an int indicating the list position, where the first
 *               item starts at zero
 * @return the Object at that list position
 */
public Object getItemAt(int index)
{
	return(model.getElementAt(index));
}

/**
 * Returns the number of items in the list.
 *
 * @return an int equal to the number of items in the list
 */
public int getItemCount()
{
	return(model.getSize());
}

/**
 * Sets the object that translates a keyboard character into a list
 * selection. Typically, the first selection with a matching first
 * character becomes the selected item.
 * @param aManager
 */
public void setKeySelectionManager(KeySelectionManager aManager)
{
	keySelectionManager = aManager;
}

/**
 * Returns the list's key-selection manager.
 *
 * @return the <code>KeySelectionManager</code> currently in use
 */
public KeySelectionManager getKeySelectionManager()
{
	return(keySelectionManager);
}

/**
 * Selects the list item that correponds to the specified keyboard character
 * and returns <code>true</code> if there is an item corresponding to that
 * character.
 *
 * @param keyChar a char, typically this is a keyboard key typed by the user
 * @return
 */
public boolean selectWithKeyChar(char keyChar)
{
	int index;

	if (keySelectionManager == null) {
		keySelectionManager = new DefaultKeySelectionManager();
	}

	index = keySelectionManager.selectionForKey(keyChar, getModel());
	if (index != -1) {
		setSelectedIndex(index);
		return(true);
	}
	else {
		return(false);
    }
}

/**
 * Selects the item at index <code>index</code>.
 *
 * @param index an int specifying the list item to select, where 0 specifies
 *                the first item in the list
 */
public void setSelectedIndex(int index)
{
	int size = model.getSize();

	if (index == -1 ) {
	    setSelectedItem(null);
	}
	else if (index < -1 || index >= size ) {
		throw new IllegalArgumentException("setSelectedIndex: " + index +
		                                   " out of bounds");
	}
	else {
	    setSelectedItem(model.getElementAt(index));
	}
}

/**
 * Returns the index of the currently selected item in the list. The result
 * is not always defined if the <code>JCSpinBox</code> box allows selected
 * items that are not in the list.<br>
 * Returns -1 if there is no selected item or if the user specified an item
 * which is not in the list.
 * @return an int specifying the currently selected list item, where 0
 *                specifies the first item in the list, or -1 if no item
 *                is selected or if the currently selected item is not
 *                in the list
 */
public int getSelectedIndex()
{
	Object o = model.getSelectedItem();
	int i, size;
	Object obj;

	for (i = 0, size = model.getSize(); i < size; i++) {
		obj = model.getElementAt(i);
		if (obj.equals(o)) {
			return(i);
		}
	}
	return(-1);
}


/**
 * Sets the selected item in the <code>JCSpinBox</code> by specifying the object
 * in the list. If <code>item</code> is in the list, the list displays with
 * <code>item</code> selected. If the object does not exist in the list,
 * the default data model selects the first item in the list.
 *
 * @param item the list object to select
 */
public void setSelectedItem(Object item)
{
	//model.setSelectedItem(item);
    setValue(item);
}

/**
 * Returns the currently selected item.
 *
 * @return  the currently selected list object from the data model
 */
public Object getSelectedItem()
{
	return(model.getSelectedItem());
}

/**
 * Inserts an item into the item list at a given index.
 * This method works only if the <code>JCSpinBox</code> uses a data model that
 * implements <code>JCMutableSpinBoxModel</code>.
 *
 * @param o the Object to add to the list
 * @param index an int specifying the position at which to add the item
 */
public void insertItemAt(Object o, int index)
{
	checkSpinBoxMutableModel();
	((JCSpinBoxMutableModel)model).insertElementAt(o, index);
}

/**
 * Adds an item to the item list.
 * This method works only if the <code>JComboBox</code> uses a
 * <code>JCSpinBoxMutableModel</code>.
 *
 * @param o the Object to add to the list
 */
public void addItem(Object o)
{
	checkSpinBoxMutableModel();
	((JCSpinBoxMutableModel)model).addElement(o);
}

/**
 * Removes an item from the item list.
 * This method works only if the <code>JCSpinBox</code> uses a model that
 * implements <code>JCSpinBoxMutableModel</code>.
 * <code>JCSpinBox</code> uses the default data model when created with the
 * empty constructor and no other model has been set.
 *
 * @param o the object to remove from the item list
 */
public void removeItem(Object o)
{
	checkSpinBoxMutableModel();
	((JCSpinBoxMutableModel)model).removeElement(o);
}

/**
 * Removes the item at <code>anIndex</code>
 * This method works only if the <code>JCSpinBox</code> model implements
 * <code>JCSpinBoxMutableModel</code>.
 *
 * @param anIndex  an int specifying the idex of the item to remove, where 0
 *                 indicates the first item in the list
 */
public void removeItemAt(int anIndex)
{
	checkSpinBoxMutableModel();
	((JCSpinBoxMutableModel)model).removeElementAt(anIndex);
}

/**
 * Removes all items from the item list.
 * This method works only if the <code>JCSpinBox</code> model implements
 * <code>JCSpinBoxMutableModel</code>.
 */
public void removeAllItems()
{
	checkSpinBoxMutableModel();
	JCSpinBoxMutableModel mutemodel = (JCSpinBoxMutableModel) model;
	int size = mutemodel.getSize();

	if (mutemodel instanceof DefaultSpinBoxModel ) {
	    ((DefaultSpinBoxModel)mutemodel).removeAllElements();
	}
	else {
	    for (int i = 0; i < size; ++i ) {
	        Object element = mutemodel.getElementAt( 0 );
			mutemodel.removeElement(element);
	    }
	}
}

/**
 * Adds a <code>java.awt.event.ItemListener</code>. <code>aListener</code> will
 * receive an event when the selected item changes.
 *
 * @param l the <code>ItemListener</code> that is to be notified
 */
public void addItemListener(ItemListener l)
{
	itemListeners = JCListenerList.add(itemListeners, l);
}

/** Removes an <code>ItemListener</code>.
 *
 * @param l the <code>ItemListener</code> to remove
 */
public void removeItemListener(ItemListener l)
{
	itemListeners = JCListenerList.remove(itemListeners, l);
}

/**
 * Adds an <code>ActionListener</code>.
 *
 * @param l the <code>actionListener</code> that is to be notified
 */
public void addActionListener(ActionListener l)
{
	actionListeners = JCListenerList.add(actionListeners, l);
}

/** Removes an <code>ActionListener</code>.
 *
 * @param l the <code>ActionListener</code> to remove
 */
public void removeActionListener(ActionListener l)
{
	actionListeners = JCListenerList.remove(actionListeners, l);
}

/**
 * Sets the action command that should be included in the event
 * sent to action listeners.
 *
 * @param aCommand  A String containing the "command" that is sent
 *                  to action listeners. The same listener can then
 *                  do different things depending on the command it
 *                  receives.
 */
public void setActionCommand(String aCommand)
{
	actionCommand = aCommand;
}

/**
 * Returns the action command that is included in the event sent to
 *  action listeners.
 *
 * @return  the String containing the "command" that is sent
 *          to action listeners
 */
public String getActionCommand()
{
	return(actionCommand);
}

/**
 * Overrides preferred width to make sure the width is big enough for
 * the formatted max and min values.
 * @return
 */
protected int preferredWidthOfView()
{
	int width = 10;
	Component ec;

	for (int i = 0; i < getItemCount(); i++) {
        if (isEditable()) {
            editor.setItem(getItemAt(i));
    		ec = editor.getEditorComponent();
    		width = Math.max(width, ec.getPreferredSize().width);
        }
        else {
            ec = renderer.getComponent(this, getItemAt(i));
            // Add 8 to account for border
    		width = Math.max(width, ec.getPreferredSize().width + 8);
        }
	}

	// Make sure the editor/renderer is set back to its proper value
	if (isEditable()) {
		editor.setItem(valueModel.getValue());
	}
	else {
		renderer.getComponent(this, valueModel.getValue());
	}

	return(width);
}

/**
 * Checks if the current model is mutable.
 */
protected void checkSpinBoxMutableModel()
{
	if (!(model instanceof JCSpinBoxMutableModel) ) {
		throw new InternalError("Cannot use this method with a " +
		                        "non-Mutable data model.");
	}
}

/**
 * This method is invoked by listeners on the data model when the model changes.
 */
protected void modelChanged()
{
	// Set a flag to make sure that ValueModel doesn't try to update SpinModel

	modelChanging = true;

   /*
    * Disabled because it is causing duplicate valuechanging messages and
    * doesn't appear to do anything useful because valueModel is aready set to
    * the value of the SelectedItem.
    valueModel.setValue(model.getSelectedItem());
    */
	editor.setItem(model.getSelectedItem());
    renderer.getComponent(this, model.getSelectedItem());
	modelChanging = false;

	Object selectedItem = model.getSelectedItem();
	Enumeration e;
	ItemEvent ie;
	ItemListener listener;
	if (lastSelectedItem != selectedItem) {
		if (lastSelectedItem != null ) {
			ie = new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED,
							   lastSelectedItem, ItemEvent.DESELECTED);
			// post item event
			e = JCListenerList.elements(itemListeners);
			while (e.hasMoreElements()) {
				listener = (ItemListener) e.nextElement();
				listener.itemStateChanged(ie);
			}
		}
	}

	lastSelectedItem = selectedItem;

	if (selectedItem != null ) {
		ie = new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED,
						   selectedItem, ItemEvent.SELECTED);
		// post item event
		e = JCListenerList.elements(itemListeners);
		while (e.hasMoreElements()) {
			listener = (ItemListener) e.nextElement();
			listener.itemStateChanged(ie);
		}
	}
}

class ListListener implements ListDataListener, java.io.Serializable
{
	//
	// ListDataListener
	//
	public void intervalAdded(ListDataEvent e) {
		modelChanged();
	}

	public void intervalRemoved(ListDataEvent e) {
		modelChanged();
	}

	public void contentsChanged(ListDataEvent e) {
		modelChanged();
	}

} // End of ListListener

/**
 * Overridden in <code>JCSpinBox</code> to add <code>keySelectionManager</code>
 * functionality.
 * @param e
 */
protected void handleKeyPressed(KeyEvent e)
{
	if (isEditable() == false) {
		if (selectWithKeyChar(e.getKeyChar()) ) {
		    e.consume();
		}
	}
}

/**
 * The interface that defines a <code>KeySelectionManager</code>. To qualify as
 * a <code>KeySelectionManager</code>, the class needs to implement the method
 * that identifies the list index given a character and the
 * combo box data model.
 */
public interface KeySelectionManager {
	/**
	 * Given <code>aKey</code> and the model, returns the row
	 *  that should become selected. Return -1 if no match was
	 *  found.
	 *
	 * @param aKey a char value, usually indicating a keyboard key that
	 *             	was pressed
	 * @param aModel a <code>JCSpinBoxModel</code> -- the component's data
     *               model, containing the list of selectable items
	 * @return an int equal to the selected row, where 0 is the
	 *         first item and -1 is none.
	 */
	int selectionForKey(char aKey, JCSpinBoxModel aModel);
} // end of KeySelectionManager


class DefaultKeySelectionManager implements KeySelectionManager, Serializable
{
	public int selectionForKey(char aKey, JCSpinBoxModel aModel)
	{
		int i, c;
		int currentSelection = -1;
		Object selectedItem = aModel.getSelectedItem();
		String v;
		String pattern;

		if (selectedItem != null) {
			selectedItem = selectedItem.toString();

			for (i = 0, c = aModel.getSize(); i < c; i++) {
				if (selectedItem.equals(aModel.getElementAt(i).toString())) {
					currentSelection  =  i;
					break;
				}
			}
		}

		pattern = ("" + aKey).toLowerCase();
		aKey = pattern.charAt(0);

		for (i = ++currentSelection, c = aModel.getSize() ; i < c ; i++ ) {
			v = aModel.getElementAt(i).toString().toLowerCase();
			if (v.length() > 0 && v.charAt(0) == aKey ) {
				return(i);
            }
		}

        for (i = 0; i < currentSelection; i++) {
            v = aModel.getElementAt(i).toString().toLowerCase();
            if (v.length() > 0 && v.charAt(0) == aKey) {
                return (i);
            }
        }
		return(-1);
	}
} // end DefaultKeySelectionManager

}
