/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCSpring.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.swing;

import java.awt.Dimension;
import java.awt.Component;

import com.klg.jclass.util.JCVersion;

/**
 * Stretchable concrete implementable of the <code>JCElastic</code> interface,
 * which specifies components as stretchable for the
 * <code>JCElasticLayout</code> manager and its subclasses.
 */
public class JCSpring extends Component
                      implements JCElastic, java.io.Serializable
{

protected final Dimension zero = new Dimension(0, 0);
protected final Dimension max = new Dimension(Short.MAX_VALUE,
											  Short.MAX_VALUE);

protected int horizontalElasticity;
protected int verticalElasticity;

protected int h_count = 0;
protected int b_count = 0;
protected int v_count = 0;

/**
 * Creates a completely stretchable component with an elasticity of 1 in
 * both horizontal and vertical directions.
 */
public JCSpring()
{
    this(1, 1);
}

/**
 * Creates a spring that is only horizontally stretchable with an
 * elasticity of 1.
 * @return
 */
static public JCSpring createHorizontalSpring()
{
    return(new JCSpring(1, 0));
}

/**
 * Creates a spring that is only horizontally stretchable with the specified
 * elasticity.
 * @return
 * @param elasticity
 */
static public JCSpring createHorizontalSpring(int elasticity)
{
    return(new JCSpring(elasticity, 0));
}

/**
 * Creates a spring that is only vertically stretchable with an elasticity of 1.
 * @return
 */
static public JCSpring createVerticalSpring()
{
    return(new JCSpring(0, 1));
}

/**
 * Creates a spring that is only vertically stretchable with the specified
 * elasticity.
 * @return
 * @param elasticity
 */
static public JCSpring createVerticalSpring(int elasticity)
{
    return(new JCSpring(0, elasticity));
}


/**
 * Creates a component that is stretchable in the direction specified.
 * @param horizontal_elasticity
 * @param vertical_elasticity
 */
public JCSpring(int horizontal_elasticity, int vertical_elasticity)
{
    if (horizontal_elasticity != 0 && vertical_elasticity != 0) {
        setName("BidirecionalSpring" + b_count++);
    }
    else if (horizontal_elasticity != 0) {
        setName("HorizontalSpring" + h_count++);
    }
    else if (vertical_elasticity != 0) {
        setName("VerticalSpring" + v_count++);
    }

    this.verticalElasticity = vertical_elasticity;
    this.horizontalElasticity = horizontal_elasticity;
}

/**
 * Provided for Beans property editors - has no effect.
 * @param s
 */
public void setAbout(String s) {}

/**
 * Returns vendor information.
 * @return vendor information
 */
public String getAbout()
{
    return(JCVersion.getVersionString());
}

public int getHorizontalElasticity()
{
    return(horizontalElasticity);
}

public void setHorizontalElasticity(int elasticity)
{
    horizontalElasticity = elasticity;
}

public int getVerticalElasticity()
{
    return(verticalElasticity);
}

public void setVerticalElasticity(int elasticity)
{
    verticalElasticity = elasticity;
}

/**
 * Returns the minimum size of the component.
 * @return minimum size of component
 */
public Dimension getMinimumSize()
{
    return(zero);
}

/**
 * Returns the preferred size of the component.
 * @return preferred size of the component
 */
public Dimension getPreferredSize()
{
    return(zero);
}

/**
 * Returns the maximum size of the component.
 * @return maximum size of the component
 */
public Dimension getMaximumSize()
{
    return(max);
}

}
