/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCSwingTypeConverter.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.swing;

import java.awt.Color;
import java.awt.Font;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Component;
import java.lang.reflect.Method;

import javax.swing.*;
import javax.swing.border.*;

import com.klg.jclass.util.JCTypeConverter;
import com.klg.jclass.util.JCStringTokenizer;
import com.klg.jclass.util.JCEnvironment;

/**
 * <code>JCSwingTypeConverter</code> contains static methods for converting
 * to/from AWT and Swing data types.
 */
public class JCSwingTypeConverter
{

/** The color light blue. */
public static final Color lightBlue = new Color(173, 216, 230);

// Titled Border position values/strings
public final static int[] positionValues = {
    TitledBorder.DEFAULT_POSITION, TitledBorder.ABOVE_TOP, TitledBorder.TOP,
	TitledBorder.BELOW_TOP, TitledBorder.ABOVE_BOTTOM, TitledBorder.BOTTOM,
	TitledBorder.BELOW_BOTTOM
};
public final static String[] positionStrings = {
    "Default", "Above_Top", "Top", "Below_Top",
	"Above_Bottom", "Bottom", "Below_Bottom"
};

// Titled Border justification values/strings
public final static int[] justificationValues = {
    TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.LEFT,
	TitledBorder.CENTER, TitledBorder.RIGHT, TitledBorder.LEADING,
    TitledBorder.TRAILING
};
public final static String[] justificationStrings = {
    "Default", "Left", "Center", "Right", "Leading", "Trailing"
};

// Horizontal Alignment values/strings
public final static int[] horizontalAlignmentValues = {
    SwingConstants.LEFT, SwingConstants.CENTER, SwingConstants.RIGHT,
    SwingConstants.LEADING, SwingConstants.TRAILING
};
public final static String[] horizontalAlignmentStrings = {
    "Left", "Center", "Right", "Leading", "Trailing"
};

// Vertical Alignment values/strings
public final static int[] verticalAlignmentValues = {
    SwingConstants.TOP, SwingConstants.CENTER, SwingConstants.BOTTOM
};
public final static String[] verticalAlignmentStrings = {
    "Top", "Center", "Bottom"
};

/**
 * Converts a String to an array of <code>Color</code>s.
 * @param s the String containing color names separated by commas
 * @param def the default value, returned if a parse error occurs
 * @return the newly created color list
 */
public static Color[] toColorList(String s, Color[] def)
{
    Color[] c = toColorList(s);
    return(s != null ? c : def);
}

/**
 * Converts a String to an array of <code>Color</code>s.
 * @param s the comma-separated color names
 * @return the newly created color list
 */
public static Color[] toColorList(String s)
{
    if (s == null) {
		return(null);
    }
    JCStringTokenizer string = new JCStringTokenizer(s);
    Color list[] = new Color[string.countTokens(',')];
    for (int i=0; string.hasMoreTokens(); i++) {
		String token = string.nextToken(',').trim();
		if ((list[i] = toColor(token)) == null) {
			JCTypeConverter.error(s, token);
		}
    }
    return(list);
}

/**
 * Converts a String to a <code>Color</code>.
 * @param s the String containing a color name
 * @param def the default value, returned if a parse error occurs
 * @return the newly created <code>Color</code>
 */
public static Color toColor(String s, Color def)
{
    Color c = toColor(s);
    return(s != null ? c : def);
}

/**
 * Converts a String to a <code>Color</code>.
 * @param s a color enum ("black", "pink", etc.);
 * hex value in format "#RRGGBB", "#RRRRGGGGBBBB" or "#N";
 * or rgb values in format "r-g-b"
 * @return the newly created <code>Color</code>
 */
public static Color toColor(String s)
{
    if (s == null) {
		return(null);
    }
    JCStringTokenizer st = new JCStringTokenizer(s);
    String token = st.nextToken('-');
    if (token == null) {
		return(null);
    }
    token = token.trim();

    if (token.equalsIgnoreCase("black")) {
		return(Color.black);
    }
    if (token.equalsIgnoreCase("blue")) {
		return(Color.blue);
    }
    if (token.equalsIgnoreCase("cyan")) {
		return(Color.cyan);
    }
    if (token.equalsIgnoreCase("darkGray")) {
		return(Color.darkGray);
    }
    if (token.equalsIgnoreCase("darkGrey")) {
		return(Color.darkGray);
    }
    if (token.equalsIgnoreCase("gray")) {
		return(Color.gray);
    }
    if (token.equalsIgnoreCase("grey")) {
		return(Color.gray);
    }
    if (token.equalsIgnoreCase("green")) {
		return(Color.green);
    }
    if (token.equalsIgnoreCase("lightGray")) {
		return(Color.lightGray);
    }
    if (token.equalsIgnoreCase("lightGrey")) {
		return(Color.lightGray);
    }
    if (token.equalsIgnoreCase("lightBlue")) {
		return(lightBlue);
    }
    if (token.equalsIgnoreCase("magenta")) {
		return(Color.magenta);
    }
    if (token.equalsIgnoreCase("orange")) {
		return(Color.orange);
    }
    if (token.equalsIgnoreCase("pink")) {
		return(Color.pink);
    }
    if (token.equalsIgnoreCase("red")) {
		return(Color.red);
    }
    if (token.equalsIgnoreCase("white")) {
		return(Color.white);
    }
    if (token.equalsIgnoreCase("yellow")) {
		return(Color.yellow);
    }

    try {
	    // Parse a hex string
		if (token.startsWith("#")) {
			if (token.length() >= 13) {
                return(new Color(Integer.valueOf(token.substring(1,5),
													   16).intValue()/256,
								 Integer.valueOf(token.substring(5,9),
													   16).intValue()/256,
								 Integer.valueOf(token.substring(9,13),
													   16).intValue()/256));
			}
			else if (token.length() >= 7) {
                return(new Color(Integer.valueOf(token.substring(1,3),
												 16).intValue(),
								 Integer.valueOf(token.substring(3,5),
												 16).intValue(),
								 Integer.valueOf(token.substring(5,7),
												 16).intValue()));
			}
			else {
				return(new Color(Integer.valueOf(token.substring(1),
												 16).intValue()));
			}
		}

		// Otherwise parse red-green-blue values
		int r = Integer.parseInt(token);
		int g = Integer.parseInt(st.nextToken('-'));
		int b = Integer.parseInt(st.nextToken('-'));
		return(new Color(r,g,b));
    }
    // ATTN GV Violation
    catch (Exception e) {
		return(null);
    }
}

/** Converts list to a comma-separated list of tokens.
 * @see #fromColor
 * @param list the list of colors to be converted to a string
 * @return the String containing a list of comma separated colors
 */
public static String fromColorList(Color[] list)
{
    if (list == null) {
		return("");
	}
    StringBuffer s = new StringBuffer();

    for (int i = 0; i < list.length; i++) {
		if (i > 0) {
			s.append(",");
		}
		s.append(fromColor(list[i]));
    }
    return(new String(s));
}

/**
 * Converts color to one of the <code>Color</code> enums, or format "r-g-b" .
 * @param c the color to be converted
 * @return a String representation of the passed in color
 */
public static String fromColor(Color c)
{
    if (c == null) {
		return(null);
    }

    if (c.equals(Color.black)) {
		return("black");
    }
    if (c.equals(Color.blue)) {
		return("blue");
    }
    if (c.equals(Color.cyan)) {
		return("cyan");
    }
    if (c.equals(Color.darkGray)) {
		return("darkGray");
    }
    if (c.equals(Color.gray)) {
		return("gray");
    }
    if (c.equals(Color.green)) {
		return("green");
    }
    if (c.equals(lightBlue)) {
		return("lightBlue");
    }
    if (c.equals(Color.lightGray)) {
		return("lightGray");
    }
    if (c.equals(Color.magenta)) {
		return("magenta");
    }
    if (c.equals(Color.orange)) {
		return("orange");
    }
    if (c.equals(Color.pink)) {
		return("pink");
    }
    if (c.equals(Color.red)) {
		return("red");
    }
    if (c.equals(Color.white)) {
		return("white");
    }
    if (c.equals(Color.yellow)) {
		return("yellow");
    }

    StringBuffer sb = new StringBuffer();
    sb.append(c.getRed());
    sb.append("-");
    sb.append(c.getGreen());
    sb.append("-");
    sb.append(c.getBlue());
    return(sb.toString());
}

/**
 * Converts a font name to a <code>Font</code> instance.
 * @param s the String containing font name
 * @param def the default value, returned if a parse error occurs
 * @return the font based on the inputted font name
 */
public static Font toFont(String s, Font def)
{
    Font f = toFont(s);
    return(f != null ? f : def);
}

/**
 * Converts a font name in format "name-style-size", e.g. "Helvetica-plain-10"
 * @param s the String representation of the Font
 * @return the font based on the inputted font name (returns null on error)
*/
public static Font toFont(String s)
{
    if (s == null) {
		return(null);
    }
    int style = Font.PLAIN;
    JCStringTokenizer string = new JCStringTokenizer(s);
    if (string.countTokens('-') != 3) {
		return(null);
    }
    String font_name = string.nextToken('-');
    if (font_name == null) {
		return(null);
    }
    font_name = font_name.trim();

    String token = string.nextToken('-');
    if (token == null) {
		style = Font.PLAIN;
    }
    else {
	    token = token.trim();
	    if (token.equalsIgnoreCase("PLAIN")) {
	        style = Font.PLAIN;
	    }
	    if (token.equalsIgnoreCase("ITALIC")) {
	        style = Font.ITALIC;
	    }
	    if (token.equalsIgnoreCase("BOLD")) {
	        style = Font.BOLD;
	    }
	    if (token.equalsIgnoreCase("BOLDITALIC")) {
	        style = Font.BOLD + Font.ITALIC;
	    }
	}
    String size_string = string.nextToken('-');
    int size = 8;
    if (size_string != null) {
		size = Integer.parseInt(size_string.trim());
    }

    return(new Font(font_name, style, size));
}

/**
 * Returns font in format "name-style-size".
 * @param f the font to be converted to a String
 * @return a String representation of the font
 */
public static String fromFont(Font f)
{
    if (f == null) {
        return(null);
    }
    StringBuffer sb = new StringBuffer();

    sb.append(f.getName());
    sb.append("-");
    switch (f.getStyle()) {
	case Font.ITALIC :
		sb.append("ITALIC");
		break;
	case Font.BOLD :
		sb.append("BOLD");
		break;
	case (Font.BOLD + Font.ITALIC):
	    sb.append("BOLDITALIC");
	    break;
	case Font.PLAIN :
	default:
		sb.append("PLAIN");
		break;
    }
    sb.append("-");
    sb.append(f.getSize());

    return(sb.toString());
}

/** Converts a String to an <code>Insets</code> instance.
 * @param s the String containing the top, left, bottom, and right
 *     values, separated by commas
 * @param def the default value, returned if the String could not be converted
 * @return the <code>Insets</code> object created from the String
 */
public static Insets toInsets(String s, Insets def)
{
    if (s == null) {
		return(def);
    }
    int[] v = JCTypeConverter.toIntList(s, ',', null);
    if (v != null && v.length == 4) {
		return(new Insets(v[0], v[1], v[2], v[3]));
    }
    return(def);
}

/**
 * Creates a String from an awt <code>Insets</code> value.
 * @param insets the <code>Insets</code object to convert to a String
 * @return the String representation of the <code>Insets</code> object passed in
 */
public static String fromInsets(Insets insets)
{
    return(insets.top + "," + insets.left + "," +
	       insets.bottom + "," + insets.right);
}


/** Converts a String to a <code>Dimension</code> instance.
 * @param s the String in the format <i>[width]x[height]</i>
 * @param def the default value, returned if the String could not be converted
 * @return the <code>Dimension</code> object created from the String
 */
public static Dimension toDimension(String s, Dimension def)
{
    int[] v = JCTypeConverter.toIntList(s, 'x', null);
    if (v != null && v.length == 2) {
		return(new Dimension(v[0], v[1]));
    }
    return(def);
}

/** Converts a String to a <code>Point</code> instance.
 * @param s the String in the format <i>[x]</i>,<i>[y]</i>
 * @param def the default value, returned if the String could not be converted
 * @return the <code>Point</code> object created from the String
 */
public static Point toPoint(String s, Point def)
{
    int[] v = JCTypeConverter.toIntList(s, ',', null);
    if (v != null && v.length == 2) {
		return(new Point(v[0], v[1]));
    }
    return(def);
}

/**
 * Converts an <code>Insets</code> object to a code String list of integers.
 * @param insets an <code>Insets</code> object
 * @return a code generation String consisting of a series of comma separated
 * integers
 */
public static String fromInsetsToIntList(Insets insets)
{
    return(fromInsetsToIntList(insets, ", "));
}

/**
 * Converts an <code>Insets</code> object to a code String list of integers.
 * @param insets an <code>Insets</code> object
 * @param separator the separator used between each number
 * @return a code generation String consisting of a series of comma separated
 * integers
 */
public static String fromInsetsToIntList(Insets insets, String separator)
{
    if (insets == null) {
        return("0" + separator + "0" + separator + "0" + separator + "0");
    }
    StringBuffer sb = new StringBuffer("");
    sb.append(insets.top);
    sb.append(separator);
    sb.append(insets.left);
    sb.append(separator);
    sb.append(insets.bottom);
    sb.append(separator);
    sb.append(insets.right);

    return(sb.toString());
}

/**
 * Converts a border name in format "name|param|param|etc."; for example,
 * "empty|10|10|10|10"
 * @param s the String to convert to a <code>Border</code> object
 * @return the border object created from the String
 */
public static Border toBorder(String s)
{
	Border border = null;
	char delim = '|';
    if (s == null) {
		return(null);
    }

    JCStringTokenizer string = new JCStringTokenizer(s);
    String border_name = string.nextToken(delim);
    if (border_name == null) {
		return(null);
    }
    border_name = border_name.trim();

	if (border_name.equalsIgnoreCase("empty")) {
   		String insets1 = string.nextToken(delim);
   		String insets2 = string.nextToken(delim);
   		String insets3 = string.nextToken(delim);
   		String insets4 = string.nextToken(delim);
		if (insets1 == null || insets2 == null ||
		    insets3 == null || insets4 == null)
		{
			return(null);
		}
		border =
		    BorderFactory.createEmptyBorder(new Integer(insets1).intValue(),
		                                    new Integer(insets2).intValue(),
											new Integer(insets3).intValue(),
											new Integer(insets4).intValue());
	}
	else if (border_name.equalsIgnoreCase("bevel")) {
   		String type   = string.nextToken(delim);
   		String color1 = string.nextToken(delim);
   		String color2 = string.nextToken(delim);

		if (type != null) {
			int value = type.equalsIgnoreCase("raised") ? BevelBorder.RAISED
				                                        : BevelBorder.LOWERED;
		    if (color1 == null || color2 == null) {
			    border = new BevelBorder(value);
			}
		    else {
			    border = new BevelBorder(value, toColor(color1),
                                         toColor(color2));
			}
		}
	}
	else if (border_name.equalsIgnoreCase("softbevel")) {
   		String type   = string.nextToken(delim);
   		String color1 = string.nextToken(delim);
   		String color2 = string.nextToken(delim);

		if (type != null) {
			int value = type.equalsIgnoreCase("raised") ? BevelBorder.RAISED
				                                        : BevelBorder.LOWERED;
		    if (color1 == null || color2 == null) {
			    border = new SoftBevelBorder(value);
		    }
		    else {
			   border = new SoftBevelBorder(value, toColor(color1),
			                                toColor(color2));
		    }
		}
	}
	else if (border_name.equalsIgnoreCase("etched")) {
   		String type   = string.nextToken(delim);
   		String color1 = string.nextToken(delim);
   		String color2 = string.nextToken(delim);

		if (type != null) {
			int value = type.equalsIgnoreCase("raised") ? EtchedBorder.RAISED
				                                        : EtchedBorder.LOWERED;
		    if (color1 == null || color2 == null) {
			    border = new EtchedBorder(value);
		    }
		    else {
			    border = new EtchedBorder(value, toColor(color1),
				                          toColor(color2));
		    }
		}
	}
	else if (border_name.equalsIgnoreCase("matte")) {
   		String insets1 = string.nextToken(delim);
   		String insets2 = string.nextToken(delim);
   		String insets3 = string.nextToken(delim);
   		String insets4 = string.nextToken(delim);
		if (insets1 == null || insets2 == null ||
		    insets3 == null || insets4 == null)
		{
			return(null);
		}
   		String color = string.nextToken(delim);

		border =
		    BorderFactory.createMatteBorder(new Integer(insets1).intValue(),
			                                new Integer(insets2).intValue(),
			                                new Integer(insets3).intValue(),
			                                new Integer(insets4).intValue(),
			                                toColor(color));
	}
	else if (border_name.equals("line")) {
   		String color = string.nextToken(delim);
   		String thick = string.nextToken(delim);
		Color c = toColor(color, Color.black);
		int thickness = (thick == null) ? 1 : new Integer(thick).intValue();
		thickness = Math.max(thickness, 1);
        if (JCEnvironment.getJavaVersion() < 130) {
            border = new LineBorder(c, thickness);
        }
        else {
            String round = string.nextToken(delim);
            boolean rounded = JCTypeConverter.toBoolean(round, false);
		    border = new LineBorder(c, thickness, rounded);
        }
	}
	else if (border_name.equals("titled")) {
   		String title = string.nextToken(delim);
		String position = string.nextToken(delim);
		String justification = string.nextToken(delim);
		String color = string.nextToken(delim);
		String font = string.nextToken(delim);

		if (title == null) {
			return(null);
		}
		int pos = JCTypeConverter.toEnum(position, positionStrings,
		                                 positionValues,
										 TitledBorder.DEFAULT_POSITION);
		int just = JCTypeConverter.toEnum(justification, justificationStrings,
		                                  justificationValues,
										  TitledBorder.DEFAULT_JUSTIFICATION);
        // Before JDK1.3.0, Leading and Trailing were not available
        if (JCEnvironment.getJavaVersion() < 130 && just > TitledBorder.RIGHT) {
            just = TitledBorder.DEFAULT_JUSTIFICATION;
        }
		Color c = toColor(color);
		Font f = toFont(font);
		border = BorderFactory.createTitledBorder(null, title, just, pos, f, c);
	}

	return(border);
}

/**
 * Returns border in format "name|param|param".
 * @param border the border object to convert
 * @param comp the component to get default values from (for example: insets, colors)
 * @return the String representation of the passed in Border object
 */
public static String fromBorder(Border border, Component comp)
{
    if (border == null) {
        return(null);
    }
    StringBuffer sb = new StringBuffer();
	String delim = "|";

	if (border instanceof MatteBorder) {
		sb.append("matte");
	}
	else if (border instanceof EmptyBorder) {
		sb.append("empty");
	}
	else if (border instanceof SoftBevelBorder) {
		sb.append("softbevel");
	}
	else if (border instanceof BevelBorder) {
		sb.append("bevel");
	}
	else if (border instanceof EtchedBorder) {
		sb.append("etched");
	}
	else if (border instanceof LineBorder) {
		sb.append("line");
	}
	else if (border instanceof TitledBorder) {
		sb.append("titled");
	}
	else {
		return(null);
	}

    sb.append(delim);

	if (border instanceof MatteBorder) {
	    MatteBorder matte = (MatteBorder)border;
		sb.append(fromInsetsToIntList(matte.getBorderInsets(comp), delim));
		sb.append(delim);
		Color color = getMatteBorderColor(matte, comp);
		sb.append(fromColor(color));
	}
	else if (border instanceof EmptyBorder) {
		EmptyBorder empty = (EmptyBorder)border;
		sb.append(fromInsetsToIntList(empty.getBorderInsets(comp), delim));
	}
	else if (border instanceof BevelBorder) {
		BevelBorder bevel = (BevelBorder)border;
		int type = bevel.getBevelType();
        if (type == BevelBorder.RAISED) {
            sb.append("raised");
        }
		else {
            sb.append("lowered");
        }

        // Don't write out highlight & shadow if they haven't been specified.
        Color highlight = bevel.getHighlightInnerColor();
        Color shadow = bevel.getShadowOuterColor();
        if (highlight != null || shadow != null) {
            sb.append(delim);
            if (highlight == null) {
                highlight = bevel.getHighlightInnerColor(comp);
            }
            String hColor = fromColor(highlight);
            sb.append((hColor == null) ? "" : hColor);
            if (shadow == null) {
                shadow = bevel.getShadowOuterColor(comp);
            }
            String sColor = fromColor(shadow);
            sb.append(delim);
            sb.append((sColor == null) ? "" : sColor);
        }
	}
	else if (border instanceof EtchedBorder) {
		EtchedBorder etched = (EtchedBorder)border;
		int type = etched.getEtchType();
        if (type == EtchedBorder.RAISED) {
            sb.append("raised");
        }
		else {
            sb.append("lowered");
        }
    	sb.append(delim);
        sb.append(fromColor(etched.getHighlightColor(comp)));
        sb.append(delim);
        sb.append(fromColor(etched.getShadowColor(comp)));
	}
	else if (border instanceof LineBorder) {
		LineBorder line = (LineBorder)border;
        sb.append(fromColor(line.getLineColor()));
        sb.append(delim);
        sb.append(line.getThickness());
        if (JCEnvironment.getJavaVersion() >= 130) {
            sb.append(delim);
            sb.append("" + line.getRoundedCorners());
        }
	}
	else if (border instanceof TitledBorder) {
		TitledBorder titled = (TitledBorder)border;
        sb.append(titled.getTitle());
		sb.append(delim);
		sb.append(JCTypeConverter.fromEnum(titled.getTitlePosition(),
		                                   positionStrings,
										   positionValues));
		sb.append(delim);
		sb.append(JCTypeConverter.fromEnum(titled.getTitleJustification(),
		                                   justificationStrings,
										   justificationValues));
		sb.append(delim);
		sb.append(fromColor(titled.getTitleColor()));
		sb.append(delim);
		sb.append(fromFont(titled.getTitleFont()));
	}

    return(sb.toString());
}

/**
 * As of JDK 1.3, <code>MatteBorder</code> has a <code>getMatteColor()</code>
 * method.  See if its there and invoke if it is.  Otherwise, return the
 * component's background color.
 * @param matte the <code>MatteBorder</code> from which the color is needed
 * @param comp the component which holds the border
 * @return the <code>MatteBorder</code>'s color
 */
public static Color getMatteBorderColor(MatteBorder matte, Component comp)
{
	Color color = (comp == null) ? Color.black : comp.getBackground();
	if (matte == null) {
		return(color);
	}
    Class c = matte.getClass();
	Method[] methods = c.getMethods();
    for (int i = 0; i < methods.length; i++) {
	    if (methods[i] != null &&
		    methods[i].getName().equals("getMatteColor"))
		{
		    try {
			    color = (Color)methods[i].invoke(matte, null);
			}
			catch (Exception e) {
			    e.printStackTrace();
			}
		}
	}
	return(color);
}

/**
 * Takes in an AWT alignment float, and spits out the equivalent
 * SwingConstant enum.
 * @param componentAlignment
 * @param defaultValue the value to return if the value is not found.
 * @return the matching SwingConstant.
 */
public static int getVerticalSwingConstantFromAWT(float componentAlignment,
                                                  int defaultValue)
{
    if (componentAlignment == Component.BOTTOM_ALIGNMENT) {
        return(SwingConstants.BOTTOM);
    }
    else if (componentAlignment == Component.CENTER_ALIGNMENT) {
        return(SwingConstants.CENTER);
    }
    else if (componentAlignment == Component.TOP_ALIGNMENT) {
        return(SwingConstants.TOP);
    }
    return(defaultValue);
}

/**
 * Takes in an AWT alignment float, and spits out the equivalent
 * SwingConstant enum.
 * @param componentAlignment
 * @param defaultValue the value to return if the value is not found.
 * @return the matching SwingConstant.
 */
public static int getHorizontalSwingConstantFromAWT(float componentAlignment,
                                                    int defaultValue)
{
    if (componentAlignment == Component.CENTER_ALIGNMENT) {
        return(SwingConstants.CENTER);
    }
    else if (componentAlignment == Component.LEFT_ALIGNMENT) {
        return(SwingConstants.LEFT);
    }
    else if (componentAlignment == Component.RIGHT_ALIGNMENT) {

        return(SwingConstants.RIGHT);
    }
    return(defaultValue);
}

/**
 * Spits out an AWT alignment float from the equivalent
 * SwingConstant enum.
 * @param swingValue
 * @param defaultValue the value to return if the value is not found.
 * @return the matching Component alignment constant.
 */
public static float getVerticalAWTFromSwingConstant(int swingValue,
                                                    float defaultValue)
{
    if (swingValue == SwingConstants.BOTTOM) {
        return(Component.BOTTOM_ALIGNMENT);
    }
    else if (swingValue == SwingConstants.CENTER) {
        return(Component.CENTER_ALIGNMENT);
    }
    else if (swingValue == SwingConstants.TOP) {
        return(Component.TOP_ALIGNMENT);
    }
    return(defaultValue);
}

/**
 * Spits out an AWT alignment float from the equivalent
 * SwingConstant enum.
  * @param swingValue
 * @param defaultValue the value to return if the value is not found.
 * @param isLeftToRight determines whether trailing and leading are
 * right or left respectively (Can be determinded by calling
 * <code>Component.getComponentOrientation.isLeftToRight();</code>
 * @return the matching Component alignment constant.
 */
public static float getHorizontalAWTFromSwingConstant(int swingValue,
                                                      float defaultValue,
                                                      boolean isLeftToRight)
{
    if (swingValue == SwingConstants.CENTER) {
        return(Component.CENTER_ALIGNMENT);
    }
    else if (swingValue == SwingConstants.LEFT) {
        return(Component.LEFT_ALIGNMENT);
    }
    else if (swingValue == SwingConstants.RIGHT) {
        return(Component.RIGHT_ALIGNMENT);
    }
    else if (swingValue == SwingConstants.TRAILING) {
        if (isLeftToRight) {
            return(Component.RIGHT_ALIGNMENT);
        }
        else {
            return(Component.LEFT_ALIGNMENT);
        }

    }
    else if (swingValue == SwingConstants.LEADING) {
        if (isLeftToRight) {
            return(Component.LEFT_ALIGNMENT);
        }
        else {
            return(Component.RIGHT_ALIGNMENT);
        }
    }
    return(defaultValue);
}

}
