/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCTableColumn.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.swing;

import javax.swing.table.*;
import javax.swing.SwingConstants;

/**
 * JCTableColumn extends TableColumn to add an EditorRendererRegistry type and
 * a header label horizontal alignment specification.
 */
public class JCTableColumn extends TableColumn {

protected String  registryType = null;
protected int     headerLabelHorizontalAlignment = -1; // Unset

/**
 * Default constructor
 */
public JCTableColumn()
{
    super();
}

/**
 * Constructor which specifies the registry type.  The
 * <code>EditorRendererRegistry</code> allows multipule editors and renderers to
 * be registered for a particular class by defining a different modifier type
 * for each.
 * @param registryType The registry modifier type (can be null)
 */
public JCTableColumn(String registryType)
{
    this();
    this.registryType = registryType;
}

/**
 * Constructor which specifies the model index of this column.  The
 * <code>modelIndex</code> is the index of the column in the model which will
 * supply the data for this column in the table.  The <code>modelIndex</code>
 * does not change as the columns are reordered in the view.
 * @param modelIndex The model index for this column
 */
public JCTableColumn(int modelIndex)
{
    super(modelIndex);
}

/**
 * Constructor which specifies the registry type and the model index.  The
 * <code>EditorRendererRegistry</code> allows multipule editors and renderers
 * to be registered for a particular class by defining a different modifier type
 * for each. The <code>modelIndex</code> is the index of the column in the model
 * which will supply the data for this column in the table.  The
 * <code>modelIndex</code> does not change as the columns are reordered in the
 * view.
 * @param modelIndex The model index for this column
 * @param registryType The registry modifier type (can be null)
 */
public JCTableColumn(int modelIndex, String registryType)
{
    this(modelIndex);
    this.registryType = registryType;
}

/**
 * Constructor which specifies the model index and the width of this column. The
 * <code>modelIndex</code> is the index of the column in the model which will
 * supply the data for this column in the table.  The <code>modelIndex</code>
 * does not change as the columns are reordered in the view.  The width
 * parameter is used to set both the  <code>preferredWidth</code> for this
 * column and the initial width.
 * @param modelIndex The model index for this column
 * @param width The preferred and initial width of this column
 */
public JCTableColumn(int modelIndex, int width)
{
    super(modelIndex, width);
}

/**
 * Constructor which specifies the registry type, width, and model index.  The
 * <code>EditorRendererRegistry</code> allows multipule editors and renderers to
 * be registered for a particular class by defining a different modifier type
 * for each. The <code>modelIndex</code> is the index of the column in the model
 * which will supply the data for this column in the table.  The
 * <code>modelIndex</code> does not change as the columns are reordered in the
 * view. The width parameter is used to set both the <code>preferredWidth</code>
 * for this column and the initial width.
 * @param modelIndex The model index for this column
 * @param width The preferred and initial width of this column
 * @param registryType The registry modifier type (can be null)
 */
public JCTableColumn(int modelIndex, int width, String registryType)
{
    this(modelIndex, width);
    this.registryType = registryType;
}

/**
 * Constructor which specifies the model index, width, and a cell renderer and
 * editor for this column.  The <code>modelIndex</code> is the index of the
 * column in the model which will supply the data for this column in the table.
 * The <code>modelIndex</code> does not change as the columns are reordered in
 * the view.  The width parameter is used to set both the
 * <code>preferredWidth</code> for this column and the initial width.  The
 * renderer and editor are the objects  used respectively to render and edit
 * values in this column. When  these are <code>null</code>, default values,
 * provided by the <code>getDefaultRenderer</code> and
 * <code>getDefaultEditor</code> methods in the <code>JTable</code> class are
 * used to provide defaults based on the type of the data in this column.
 * @param modelIndex The model index for this column
 * @param width The preferred and initial width of this column
 * @param cellRenderer The cell renderer for this column
 * @param cellEditor The cell editor for this column
 * @see javax.swing.JTable#getDefaultRenderer(Class)
 * @see javax.swing.JTable#getDefaultEditor(Class)
 * @see javax.swing.JTable#getCellRenderer(int, int)
 * @see javax.swing.JTable#getCellEditor(int, int)
 */
public JCTableColumn(int modelIndex, int width, TableCellRenderer cellRenderer,
	                 TableCellEditor cellEditor)
{
    super(modelIndex, width, cellRenderer, cellEditor);
}

/**
 * Constructor which specifies the model index, width, a cell renderer and
 * editor, and a registry type for this column.  The <code>modelIndex</code> is
 * the index of the column in the model which will supply the data for this
 * column in the table. The <code>modelIndex</code> does not change as the
 * columns are reordered in the view.  The width parameter is used to set both
 * the <code>preferredWidth</code> for this column and the initial width.  The
 * renderer and editor are the objects  used respectively to render and edit
 * values in this column. When  these are <code>null</code>, default values,
 * provided by the <code>getDefaultRenderer</code> and
 * <code>getDefaultEditor</code> methods in the <code>JTable</code> class are
 * used to provide defaults based on the type of the data in this column.  The
 * <code>EditorRendererRegistry</code> allows multipule editors and renderers
 * to be registered for a particular class by defining a different modifier type
 * for each.
 * @param modelIndex The model index for this column
 * @param width The preferred and initial width of this column
 * @param cellRenderer The cell renderer for this column
 * @param cellEditor The cell editor for this column
 * @param registryType The registry modifier type (can be null)
 * @see javax.swing.JTable#getDefaultRenderer(Class)
 * @see javax.swing.JTable#getDefaultEditor(Class)
 * @see javax.swing.JTable#getCellRenderer(int, int)
 * @see javax.swing.JTable#getCellEditor(int, int)
 */
public JCTableColumn(int modelIndex, int width, TableCellRenderer cellRenderer,
                     TableCellEditor cellEditor, String registryType)
{
    this(modelIndex, width, cellRenderer, cellEditor);
    this.registryType = registryType;
}

/**
 * Get the current registry type. The <code>EditorRendererRegistry</code>
 * allows multipule editors and renderers to be registered for a particular
 * class by defining a different modifier type for each.
 * @return The current registry type.  Can be null.
 */
public String getRegistryType()
{
    return(registryType);
}

/**
 * Set the current registry type. The <code>EditorRendererRegistry</code>
 * allows multipule editors and renderers to be registered for a particular
 * class by defining a different modifier type for each.
 * @param registryType The new registry type (can be null)
 */
public void setRegistryType(String registryType)
{
    this.registryType = registryType;
}

/**
 * Returns the <code>TableCellEditor</code> used by the <code>JTable</code> to
 * edit values for this column.  When the <code>cellEditor</code> is
 * <code>null</code>, the <code>JTable</code> uses a default editor based on the
 * class of the cells in that column. The default value for a
 * <code>cellEditor</code> is <code>null</code>.
 * @return The current <code>cellEditor</code> for this column
 * @see	#setCellEditor
 * @see	javax.swing.JTable#setDefaultEditor
 */
public TableCellEditor getCellEditor()
{
    TableCellEditor editor = super.getCellEditor();

    /* If I could get at the JTable or the model I could do this
    if (editor == null) {
        EditorRendererRegistry registry =
            EditorRendererRegistry.getCentralRegistry();
        editor = registry.getCellEditorInstance(getColumnClass(getModelIndex()),
                                                registryType);
    }
    */
    return(editor);
}

/**
 * Sets the horizontal alignment for the <code>JLabel</code> within the header
 * of this column. This is used by the default header renderer for
 * <code>JCSortableTable</code> and <code>JCTreeTable</code>. By default this
 * value is a non-valid alignment value, in which case the renderer will use
 * the default <code>JLabel</code> horizontal alignment value, which is
 * <code>SwingConstants.LEADING</code>.
 *
 * @see javax.swing.JLabel
 * @param alignment the new header label horizontal alignment value
 */
public void setHeaderLabelHorizontalAlignment(int alignment)
{
	if (alignment == headerLabelHorizontalAlignment) {
		return;
	}
    if ((alignment == SwingConstants.LEFT)    ||
        (alignment == SwingConstants.CENTER)  ||
        (alignment == SwingConstants.RIGHT)   ||
        (alignment == SwingConstants.LEADING) ||
        (alignment == SwingConstants.TRAILING))
	{
        headerLabelHorizontalAlignment = alignment;
    }
    else {
        throw new IllegalArgumentException("Bad horizontal alignment value");
    }
}

/**
 * Gets the horizontal alignment for the <code>JLabel</code> within the header
 * of this column. This is used by the default header renderer for
 * <code>JCSortableTable</code> and <code>JCTreeTable</code>. By default this
 * value is a non-valid alignment value, in which case the renderer will use
 * the default <code>JLabel</code> horizontal alignment value, which is
 * <code>SwingConstants.LEADING</code>.
 *
 * @see javax.swing.JLabel
 * @return the current header label horizontal alignment value
 */
public int getHeaderLabelHorizontalAlignment()
{
	return(headerLabelHorizontalAlignment);
}

}
