/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCWordWrap.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:46 $  $Locker:  $  Quest Software Inc.
package com.klg.jclass.util.swing;

import java.awt.FontMetrics;
import java.util.StringTokenizer;

/**
 * <CODE>JCWordWrap</CODE> provides a static method <CODE>wrapText</CODE>, which
 * performs basic word-wrap logic on a String given a line width and
 * new line delimiter.
 */
public class JCWordWrap {

/**
 * Returns a word-wrapped String given a line width and delimiter, such as a
 * newline.  Word-wrap logic breaks lines by spaces and provides no hyphenation
 * logic. The original String is returned if the number of characters is less
 * than 10.  Note that the word delimiter is always a space.
 * @param text The original text to wrap
 * @param fm The font metrics used to decide whether text fits on a line
 * @param width The width of the line
 * @param delimiter The delimiter to use for line wrapping.
 * @param isLeftAligned No longer used
 * @return The newly created word-wrapped String
 */
public static String wrapText(String text, FontMetrics fm, int width,
			                  String delimiter, boolean isLeftAligned)
{
    if (width < 10) {
        return(text);
    }

    if (text == null || text.length() == 0) {
        return(text);
    }

    // Set the line delimiter to newline if one isn't passed in
    if (delimiter == null) {
        delimiter = "\n";
    }

    // Return the input string if its length is less than the wrap width
    if (fm.stringWidth(text) < width) {
        return(text);
    }

    int token_width = 0;
    int line_width  = 0;
    StringBuffer buffer = new StringBuffer(text.length());

    // Create a string tokenizer using space as the delimiter
    StringTokenizer tokenizer = new StringTokenizer(text, " ", true);
    while (tokenizer.hasMoreTokens()) {
        String token = tokenizer.nextToken();
        token_width = fm.stringWidth(token);
        if (line_width + token_width > width) {
            // We've gone over the width.  Add delimiter and reset line width.
            if (token.equals(" ")) {
                // We don't want a space as the first item of the next line
                buffer.append(delimiter);
                line_width = 0;
            }
            else {
                buffer.append(delimiter + token);
                line_width = token_width;
            }
        }
        else {
            buffer.append(token);
            line_width += token_width;
        }
    }
    return(buffer.toString());
}

/**
 * Returns a String with each occurence of an old delimiter replaced by a new
 * delimiter.
 * @param text The text in which the replacement happens
 * @param oldDelimiter The delimiter text to be replaced
 * @param newDelimiter The new text to replace the old
 * @return A string in which the replacement has happened.  If there are no
 * occurrences of the old delimiter in the text, the passed in String is
 * returned.
 */
public static String replace(String text, String oldDelimiter,
			                 String newDelimiter)
{
    if (oldDelimiter.equals(newDelimiter)) {
        return(text);
    }

    int pos = text.indexOf(oldDelimiter);
    if (pos != -1) {
        return(replace(text.substring(0, pos) + newDelimiter +
                       text.substring(pos + oldDelimiter.length()),
                       oldDelimiter, newDelimiter));
    }
    return(text);
}

}
