/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: TextRenderer.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.swing;

import com.klg.jclass.util.JCEnvironment;



import javax.swing.JLabel;
import javax.swing.CellRendererPane;
import javax.swing.plaf.basic.BasicHTML;
import java.awt.Dimension;
import java.awt.Container;
import java.awt.Graphics;

/**
 * TextRenderer is a class that extends the swing JLabel class functionality 
 * for use as a text renderer.  In this way, we get things like HTML
 * rendering for free because it is already implemented in JLabel.
 */
public class TextRenderer extends JLabel
{

/**
 * Object used for synchroizing access to Swing objects that are problematic
 * in threads.
 */



public TextRenderer()
{
	super();
}

public boolean isShowing()
{
	return(true);
}

/**
 *  If html encoded text, let the renderer component figure out the size
 */
public Dimension calcHTMLTextSize(String text)
{



   		setText(text);




    return(getPreferredSize());
}

public void drawHTMLText(Graphics gc, CellRendererPane crp, Container cont,
					     String text, int x, int y, int w, int h)
{
    // Set text in rendering component, get parent region to draw
    // into and get the renderer pane we can use for that




   		setText(text);




    crp.paintComponent(gc, this, cont, x, y, w, h, false);
}

/**
 * Check for HTML encoded text.  Check is the same as that in
 * javax.swing.plaf.basic.BasicHTML.isHTMLString().  We can't use
 * that call in versions up to and including JDK 1.2.2 since the class isn't
 * public, so we repeat the code here in that case. For a string to be HTML
 * encoded, it needs to begin with "<html>" (ignoring case).
 * @param s the string to check for HTML encoding.
 * @return true if string is encoded in html, false otherwise.
 */
public static boolean isHTML(String s)
{
    if (JCEnvironment.getJavaVersion() >= 130) {
        return(BasicHTML.isHTMLString(s));
    }
    else {
        if (s != null) {
            if ((s.length() >= 6)    &&
                (s.charAt(0) == '<') &&
                (s.charAt(5) == '>'))
            {
                String tag = s.substring(1, 5);
                return(tag.equalsIgnoreCase("html"));
            }
        }
        return(false);
    }
}

/**
 * Attempt to calculate the size of one line of a multi-line piece of text.
 * If this is just straight text, it can only be one line anyways, so we
 * just return the ascent from the font metrics.  If this object contains
 * HTML encoded text, however, we try to make a guess at the height of the
 * first line by counting line breaks.  If the text uses different font
 * sizes on multiple lines, this calculation will be off.
 * Note : a full recalc should have been done on this object first.
 */
public Dimension calcHTMLLineSize(String text, int w, int h)
{
    // If HTML encoded string, try to figure out number of line breaks
    // and divide the full height by that value
    int paraCount = 1;
    paraCount += countInString("<p>", text);
    paraCount += countInString("<br>", text);
    paraCount += countInString("<P>", text);
    paraCount += countInString("<BR>", text);

    return(new Dimension(w, h/paraCount));
}

/**
 * Count the occurrences of string token within larger target string
 */
public static int countInString(String token, String target)
{
    int count = 0;
    int textIndex = 0;
    int tmpIndex = 0;
    while ((tmpIndex = target.indexOf(token, textIndex)) >= 0) {
        count++;
        textIndex = tmpIndex + 1;
    }
    return(count);
}


}
