/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DateChooserEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.swing.beans;

import com.klg.jclass.beans.AbstractBeanEditor;
import com.klg.jclass.util.calendar.JCDateChooser;
import com.klg.jclass.util.swing.JCExitFrame;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DateFormat;
import java.util.Calendar;
import java.util.Locale;

public class DateChooserEditor extends AbstractBeanEditor
                               implements ActionListener
{

public static final String DATECHOOSER_HELP = "DataChooser";

protected Calendar       date;
protected JCDateChooser  dateChooser;

public DateChooserEditor()
{
    super("com.klg.jclass.util.swing.beans.resources.LocaleInfo",
          DATECHOOSER_HELP, "Date Chooser Editor");
    //setPreferredSize(new Dimension(300,200));
}

/**
 * Returns the actual editor component which will be wrapped with the html
 * help pane.
 */
public Component getEditorComponent()
{
    // Add the style checkboxes
    JPanel panel = new JPanel();
    BoxLayout layout = new BoxLayout(panel, BoxLayout.Y_AXIS);
    panel.setLayout(layout);

    dateChooser = new JCDateChooser();
    dateChooser.addActionListener(this);

    panel.add(dateChooser);
    return(panel);
}

/**
 * Returns the code generation string.
 */
public String getJavaInitializationString()
{
    // Build the string array
    String init_string = "null";
    if (date != null) {
        init_string = "new java.util.GregorianCalendar(" +
                      date.get(Calendar.YEAR) + ", " +
                      date.get(Calendar.MONTH) + ", " +
                      date.get(Calendar.DAY_OF_MONTH) + ")";
    }
    return(init_string);
}

/**
 * Returns the value of the property.
 */
public Object getValue()
{
    return((date == null) ? null : date.clone());
}

/**
 * Returns a text representation of the value.
 */
protected String getValueAsText()
{
    String s = "none";
    if (date != null) {
        DateFormat df = DateFormat.getDateInstance(DateFormat.MEDIUM,
                                                   Locale.getDefault());
        s = df.format(date.getTime());
    }
    return(s);
}

/**
 * Sets the value of the property.
 */
public void setValue(Object o)
{
    if (o == null) {
        date = null;
    }
    else {
        date = (Calendar)((Calendar)o).clone();
    }
    if (dateChooser != null) {
        Calendar c = (date == null) ? Calendar.getInstance() : date;
        dateChooser.setValue(c);
    }
}

/**
 * Implementation for the <code>ActionListener</code>.
 * @param e The event generated by the action.
 */
public void actionPerformed(ActionEvent e)
{
    if (e.getSource() == dateChooser) {
        date = dateChooser.getValue();
        support.firePropertyChange(null, null, null);
    }
}

public static void main(String[] args)
{
    JFrame f = new JCExitFrame();
    f.getContentPane().add(new DateChooserEditor());
    f.pack();
    f.show();
}

}
