/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: DateTimeBeanEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.swing.beans;

import com.klg.jclass.beans.AbstractBeanEditor;
import com.klg.jclass.util.value.CalendarValueModel;
import com.klg.jclass.util.value.JCValueEvent;
import com.klg.jclass.util.value.JCValueListener;
import com.klg.jclass.util.swing.JCExitFrame;
import com.klg.jclass.util.calendar.DateTimeEditor;

import javax.swing.*;
import java.awt.*;
import java.text.DateFormat;
import java.util.Calendar;
import java.util.Locale;
import java.util.Date;

public class DateTimeBeanEditor extends AbstractBeanEditor
                                implements JCValueListener
{

public static final String DATETIME_HELP = "DateTimeEditor";

protected Date                date;
protected CalendarValueModel  model;
protected DateTimeEditor      dateTimeEditor;
protected boolean             updatingModel = false;

public DateTimeBeanEditor()
{
    super("com.klg.jclass.util.swing.beans.resources.LocaleInfo",
          DATETIME_HELP, "Date Time Editor");
}

/**
 * Returns the actual editor component which will be wrapped with the html
 * help pane.
 */
public Component getEditorComponent()
{
    // Add the style checkboxes
    JPanel panel = new JPanel();
    BoxLayout layout = new BoxLayout(panel, BoxLayout.Y_AXIS);
    panel.setLayout(layout);

    dateTimeEditor = new DateTimeEditor();
    Calendar c = Calendar.getInstance();
    if (date != null) {
        c.setTime(date);
    }
    model = new CalendarValueModel(c);
    model.addValueListener(this);
    dateTimeEditor.setCalendarModel(model);

    panel.add(dateTimeEditor);
    return(panel);
}

/**
 * Returns the code generation string.
 */
public String getJavaInitializationString()
{
    // Build the string array
    String init_string = "null";
    if (date != null) {
        init_string = "new java.util.Date(" + date.getTime() + "l)";
    }
    return(init_string);
}

/**
 * Returns the value of the property.
 */
public Object getValue()
{
    return((date == null) ? null : date.clone());
}

/**
 * Returns a text representation of the value.
 */
protected String getValueAsText()
{
    String s = "none";
    if (date != null) {
        DateFormat df = DateFormat.getDateTimeInstance(DateFormat.MEDIUM,
                                                       DateFormat.MEDIUM,
                                                       Locale.getDefault());
        s = df.format(date);
    }
    return(s);
}

/**
 * Sets the value of the property.
 */
public void setValue(Object o)
{
    if (o == null) {
        date = null;
    }
    else {
        date = (Date)((Date)o).clone();
    }
    if (model != null) {
        updatingModel = true;
        Calendar c = Calendar.getInstance();
        if (date != null) {
            c.setTime(date);
        }
        model.setValue(c);
        updatingModel = false;
    }
}

/**
 * The value is changing.  Ignored.
 * @param event The event generated by the value changing
 */
public void valueChanging(JCValueEvent event) {}

/**
 * The method informs a listener that the value has been changed.
 * @param event The event generated by the value changed
 */
public void valueChanged(JCValueEvent event)
{
    if (updatingModel) {
        return;
    }

    Calendar c = (Calendar)event.getNewValue();
    date = (c == null) ? null : c.getTime();
    support.firePropertyChange(null, null, null);
}

public static void main(String[] args)
{
    JFrame f = new JCExitFrame();
    f.getContentPane().add(new DateTimeEditor());
    f.pack();
    f.show();
}

}
