/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: StyleControlsEditor.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:46 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.swing.beans;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.*;
import java.awt.FlowLayout;
import com.klg.jclass.util.swing.JCGridLayout;
import com.klg.jclass.util.swing.JCFontChooser;
import com.klg.jclass.beans.AbstractBeanEditor;

public class StyleControlsEditor extends AbstractBeanEditor implements ActionListener {

public static final String STYLECONTROLS_HELP = "StyleControls";
public int styleControls;

protected JCheckBox boldCheckbox;
protected JCheckBox italicCheckbox;

protected JCheckBox underlineCheckbox;


public StyleControlsEditor() {
	super("com.klg.jclass.util.swing.beans.resources.LocaleInfo", STYLECONTROLS_HELP,
		  "Style Controls Editor");
}

/**
 * Returns the actual editor component which will be wrapped with the html
 * help pane.
 */
public Component getEditorComponent() {
    // add the style checkboxes
	JPanel stylePanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
	stylePanel.setBorder(new TitledBorder("Style"));

	boldCheckbox = new JCheckBox("Bold");
	boldCheckbox.addActionListener(this);
	stylePanel.add(boldCheckbox);

	italicCheckbox = new JCheckBox("Italic");
	italicCheckbox.addActionListener(this);
	stylePanel.add(italicCheckbox);


	underlineCheckbox = new JCheckBox("Underline");
	underlineCheckbox.addActionListener(this);
	stylePanel.add(underlineCheckbox);


    return stylePanel;
}


/**
 * Returns the code generation string.
 */
public String getJavaInitializationString() {
    if(styleControls == JCFontChooser.NONE) {
      return new String("com.klg.jclass.util.swing.JCFontChooser.NONE");
    }

    String str = "";
    boolean itemAdded = false;

    if((styleControls & JCFontChooser.BOLD) != 0) {
        str += "com.klg.jclass.util.swing.JCFontChooser.BOLD";
        itemAdded = true;
    }

    if((styleControls & JCFontChooser.ITALIC) != 0) {
        if(itemAdded) {
            str += " | ";
        }
        str += "com.klg.jclass.util.swing.JCFontChooser.ITALIC";
        itemAdded = true;
    }


    if((styleControls & JCFontChooser.UNDERLINE) != 0) {
        if(itemAdded) {
            str += " | ";
        }
        str += "com.klg.jclass.util.swing.JCFontChooser.UNDERLINE";
        itemAdded = true;
    }


	return str;
}

/**
 * Returns the value of the property.
 */
public Object getValue() {
	return new Integer(styleControls);
}

/**
 * Returns a text representation of the value.
 */
protected String getValueAsText() {
	return null;
}

/**
 * Sets the value of the property.
 */
public void setValue(Object o) {
    styleControls = ((Integer)o).intValue();
    updateControls();
}

/**
 * Updates the dialog controls.
 */
protected void updateControls() {
    boldCheckbox.setSelected((styleControls & JCFontChooser.BOLD) != 0);
    italicCheckbox.setSelected((styleControls & JCFontChooser.ITALIC) != 0);

    underlineCheckbox.setSelected((styleControls & JCFontChooser.UNDERLINE) != 0);

}

/** Implementation for the <code>ActionListener</code>.
 */
public void actionPerformed(ActionEvent e) {
    if ((e.getSource() == boldCheckbox)
        || (e.getSource() == italicCheckbox)

        || (e.getSource() == underlineCheckbox)

        ){
        styleControls = JCFontChooser.NONE;
        if(boldCheckbox.isSelected() == true) {
            styleControls |= JCFontChooser.BOLD;
        }
        if(italicCheckbox.isSelected() == true) {
            styleControls |= JCFontChooser.ITALIC;
        }

        if(underlineCheckbox.isSelected() == true) {
            styleControls |= JCFontChooser.UNDERLINE;
        }

        support.firePropertyChange(null,null,null);
    }
}


public static void main(String[] args) {
	JFrame f = new JFrame();
	f.getContentPane().add(new StyleControlsEditor());
	f.pack();
	f.show();
}

}
