/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: AbstractImageEncoder.java,v $ $Revision: 1.2 $
//          $Date: 2006-03-17 21:13:47 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.util.swing.encode;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.OutputStream;

/**
 * This is the base class for encoding a component to a stream file.
 */
public abstract class AbstractImageEncoder implements Encoder {

public void encode(Image image, OutputStream os)
    throws IOException, EncoderException
{
    saveImage(image, os);
}

public void encode(Component component, OutputStream os)
    throws IOException, EncoderException
{
    encode(snapshot(component), os);
}

public static Image snapshot(Component component)
{
    Dimension size = component.getSize();
    Image img;
    try {
        img = new BufferedImage(size.width, size.height,
                                BufferedImage.TYPE_INT_ARGB);
    }
    catch (Exception e) {
        img = component.createImage(size.width, size.height);
    }

    if (img != null) {
        Graphics igc = img.getGraphics();
        //Gotta set the clip, or else paint throws an exception
        igc.setClip(0, 0, component.getSize().width,
                    component.getSize().height);
        component.paint(igc);

        //check for Java3d usage
        snapshotJava3d(component, igc);

    }
    return(img);
}

/**
 * Run through the component and its children looking for those that
 * implement Java3dImageable.  If such a component is found, then
 * the image of its Java3D canvas is drawn into the image being built.
 * @param component the component to test for Java3d Image drawing capability
 * @param igc the Graphics object representing the parent image being drawn to
 */
protected static void snapshotJava3d(Component component, Graphics igc)
{
    if (component instanceof Java3dImageable) {
        Point location = new Point();
        Image subImage =
            ((Java3dImageable)component).getJava3dAsImage(location);
        if (subImage != null) {
            igc.drawImage(subImage, location.x, location.y, null);
        }
    }

    if (component instanceof Container &&
        ((Container)component).getComponentCount() > 0)
    {
        Component[] children = ((Container)component).getComponents();
        for (int i = 0; i < children.length; i++) {
            Point location = children[i].getLocation();
            igc.translate(location.x, location.y);
            snapshotJava3d(children[i], igc);
            igc.translate(-location.x, -location.y);
        }
    }
}

public abstract void saveImage(Image image, OutputStream os)
    throws IOException, EncoderException;

public static byte createByte(int b7, int b6, int b5, int b4, int b3, int b2,
                              int b1, int b0)
{
    byte bits = 0;
    if (b0 == 1) {
        bits = (byte)(bits | 1);
    }
    if (b1 == 1) {
        bits = (byte)(bits | 2);
    }
    if (b2 == 1) {
        bits = (byte)(bits | 4);
    }
    if (b3 == 1) {
        bits = (byte)(bits | 8);
    }
    if (b4 == 1) {
        bits = (byte)(bits | 16);
    }
    if (b5 == 1) {
        bits = (byte)(bits | 32);
    }
    if (b6 == 1) {
        bits = (byte)(bits | 64);
    }
    if (b7 == 1) {
        bits = (byte)(bits | 128);
    }
    return(bits);
}

public static byte byteFromInt(int value)
{
    return ((byte)(value & 255));
}

public static byte[] bytesFromInt(int value)
{
    byte[] buf = new byte[2];
    buf[0] = ((byte)((value >> 8) & 255));
    buf[1] = ((byte)((value) & 255));
    return(buf);
}

public static byte[] bytesFromLong(long value)
{
    byte[] buf = new byte[4];
    buf[0] = ((byte)((value >> 24) & 255));
    buf[1] = ((byte)((value >> 16) & 255));
    buf[2] = ((byte)((value >> 8) & 255));
    buf[3] = ((byte)((value) & 255));
    return(buf);
}

public static byte byteFromChar(char ochar)
{
    int temp = (int)ochar;
    byte bits = 0;

    for (int i = 0; i <= 7; i++) {
        if ((temp & ((byte)Math.pow(2, i))) != 0) {
            bits = (byte)(bits | ((byte)Math.pow(2, i)));
        }
    }
    return(bits);
}

/**
 * Compress the given color into one 8 bit representation.
 * @param clr integer representation of rgb value (lowest byte is blue,
 *            second lowest byte is green, third lowest byte is red)
 * @return color compressed into 8-bit representation
 */
public byte compressColor(int clr)
{
    return compressColor((clr >> 16) & 255, (clr >> 8) & 255, clr & 255);
}

/**
 * Compress the given color into one 8 bit representation.
 * @param red value of the red portion of the color
 * @param green value of the green portion of the color
 * @param blue value of the blue portion of the color
 * @return color compressed into 8-bit representation
 */
public byte compressColor(int red, int green, int blue)
{
    // take 3 most significatnt bits of red, 3 most significant bits of
    // green and 2 most significant bits of blue to form 8-bit compression
    // of color values

    return (byte)((red & 224) | ((green >> 3) & 28) | ((blue >> 6) & 3));
}

protected void error(String msg) throws EncoderException
{
    throw new EncoderException(msg);
}

}

