/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JPEGEncoder.java,v $ $Revision: 1.2 $ 
//          $Date: 2006-03-17 21:13:47 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.util.swing.encode;

import com.sun.image.codec.jpeg.JPEGCodec;
import com.sun.image.codec.jpeg.JPEGEncodeParam;
import com.sun.image.codec.jpeg.JPEGImageEncoder;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.OutputStream;

/**
 * This class encodes a JPEG file from an Image to an OutputStream.  It uses
 * the sun com.sun.image.codec.jpeg package to do the encoding, and hence
 * is usable only under jdk 1.2.2 or later.
*/
public class JPEGEncoder extends AbstractImageEncoder {

public float quality = 1.0f;

public JPEGEncoder() {}

public void saveImage(Image img, OutputStream ofile)
    throws IOException, EncoderException
{
    if (img == null) {
        error("JPEG encoding error: Image is NULL.");
    }

    BufferedImage bi;

    // If image is a buffered image of usable type, we can just use it
    if (img instanceof BufferedImage &&
        ((BufferedImage)img).getType() != BufferedImage.TYPE_INT_ARGB)
    {
        bi = (BufferedImage)img;
    }
    else {
        // Otherwise, create a new buffered image and paint our existing image
        // into it
        bi = new BufferedImage(img.getWidth(null), img.getHeight(null),
                               BufferedImage.TYPE_INT_RGB);
        Graphics2D biGC = bi.createGraphics();
        biGC.drawImage(img, 0, 0, null);
    }

    try {
        JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(ofile);
        JPEGEncodeParam param = encoder.getDefaultJPEGEncodeParam(bi);
        param.setQuality(quality, false);
        encoder.setJPEGEncodeParam(param);
        encoder.encode(bi);
    }
    catch (Exception e) {
        error("JPEG encoding error : " + e.getMessage());
        e.printStackTrace();
    }
}

/**
 * Get quality factor for JPEG encoding.  The quality factor is a floating
 * point number between 0 and 1.  A value of 1 provides the best quality
 * image but features the words compression.  A value of 0 provides the
 * best compression but the worst quality image.
 * @return qlty The quality factor being used.  Must be between 0 and 1.
 */
public float getQuality()
{
    return(quality);
}

/**
 * Set quality factor for JPEG encoding.  The quality factor is a floating
 * point number between 0 and 1.  A value of 1 provides the best quality
 * image but features the words compression.  A value of 0 provides the
 * best compression but the worst quality image.
 * @param qlty The quality factor to set.  Must be between 0 and 1.
 *             If outside these ranges, the value will be clipped to 0 or 1.
 */
public void setQuality(float qlty)
{
    if (qlty < 0.0f) {
        qlty = 0.0f;
    }
    else if (qlty > 1.0f) {
        qlty = 1.0f;
    }
    quality = qlty;
}

}
