/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: AbstractVectorEncoder.java,v $ $Revision: 1.2 $
//          $Date: 2006-03-17 21:13:47 $ $Locker:  $ Quest Software Inc.

package com.klg.jclass.util.swing.encode.page;

import java.awt.Component;
import java.awt.Image;
import java.awt.Rectangle;

import java.io.IOException;
import java.io.OutputStream;

import javax.swing.Icon;
import javax.swing.ImageIcon;

import com.klg.jclass.util.swing.encode.Encoder;
import com.klg.jclass.util.swing.encode.EncoderException;
import com.klg.jclass.page.JCDocument;
import com.klg.jclass.page.JCFlow;
import com.klg.jclass.page.JCFrame;
import com.klg.jclass.page.JCPage;
import com.klg.jclass.page.JCUnit;

/**
 * This is the base class for encoding a component to a stream file.
 * It needs JClass PageLayout to compile and to be used.
 */
public abstract class AbstractVectorEncoder implements Encoder {

protected Image img = null;
protected Component comp = null;
protected OutputStream outfile = null;

public void encode(Image image, OutputStream os)
			throws IOException, EncoderException {

	img = image;
	outfile = os;

	JCDocument doc = createOutput(os);
	JCFlow flow = new JCFlow(doc);
	flow.getCurrentFrame().pasteImage(img, new JCUnit.Point());
	doc.print();
}

public void encode(Component component, OutputStream os)
			throws IOException, EncoderException {

	comp = component;
	outfile = os;

	JCDocument doc = createOutput(os);
	JCFlow flow = new JCFlow(doc);
	flow.getCurrentFrame().pasteComponent(component, new JCUnit.Point());
	doc.print();
}

protected abstract JCDocument createOutput(OutputStream os)
							  throws IOException, EncoderException;

protected JCUnit.Dimension getObjectSize() throws EncoderException {

	JCUnit.Dimension size = null;

	if (img != null) {
		Icon tmp_icon = new ImageIcon(img);
		size = new JCUnit.Dimension(JCUnit.POINTS, tmp_icon.getIconWidth(),
									tmp_icon.getIconHeight());
	}
	else if (comp != null) {
		Rectangle rect = comp.getBounds();
		size = new JCUnit.Dimension(JCUnit.POINTS, (double) rect.width,
									(double) rect.height);
	}
	else {
		error("No object to encode");
	}

	return(size);
}

protected JCPage createTemplatePage(JCUnit.Dimension size) {

	// Create a page of the correct size and mark it as a template
	JCPage template = new JCPage("template", new JCUnit.Point(), size);
	template.setPageType(JCPage.PAGE_TYPE_TEMPLATE);
	template.setFlowPageTemplate(template);
	template.setFlowSectionTemplate(template);

	// Create the content frame for the page and add it.
	JCFrame frame = new JCFrame("body", new JCUnit.Point(), size);
	template.getFrameList().add(frame);
	template.getFlowFrameList().add(frame);

	// return the template page
	return(template);
}

protected void error(String msg) throws EncoderException {
    throw new EncoderException(msg);
}

}
