/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: EmptyTreeTableModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:48 $  $Locker:  $  Quest Software Inc.
package com.klg.jclass.util.treetable;

import java.beans.*;
import javax.swing.tree.*;
import com.klg.jclass.util.treetable.JCTreeTableModel;
import com.klg.jclass.util.JCEnvironment;

/**
 * An empty tree table model used by JCTreeTable and JCTreeExplorer in
 * their default constructors.  This model provides a simple tree table
 * when invoked at design time, and does nothing under normal conditions.
 * This way the tree table or tree explorer can display some values in a
 * visual designer environment.
 */
public class EmptyTreeTableModel extends DefaultTreeModel
			                     implements JCTreeTableModel
{

/**
 * Constructs a tree table model.
 * @param root a TreeNode object that is the root of the tree
 */
public EmptyTreeTableModel(TreeNode root)
{
    super(root);
}

/**
 * Constructs a tree table model.
 * @param root a TreeNode object that is the root of the tree
 * @param asksAllowsChildren a boolean, false if any node can
 *        have children, true if each node is asked to see if
 *        it can have children
 */
public EmptyTreeTableModel(TreeNode root, boolean asksAllowsChildren)
{
    super(root, asksAllowsChildren);
}

/**
 * Creates a simple tree model.
 * @return The root of the new default tree
 */
public static MutableTreeNode makeDefaultTree()
{
    DefaultMutableTreeNode root =
        new DefaultMutableTreeNode(new String("Root"));
    root.add(new DefaultMutableTreeNode(new String("Child 1")));
    root.add(new DefaultMutableTreeNode(new String("Child 2")));
    return(root);
}


// JCTreeTableModel methods

/**
 * Returns a simple string object if running at design time, otherwise
 * returns null.
 *
 * @param node The node for which the value is wanted
 * @param column The column of the node for which the value is wanted
 * @return The value the given column of the given node
 */
public Object getValueAt(Object node, int column)
{
    if (JCEnvironment.isDesignTime() == true) {
        return(new String("Column " + column));
    }
    return(null);
}

/**
 * Returns that the treeTable is not editable.
 * @param node The node for which the cell editable value is wanted
 * @param column The column of the node for which the cell editable value
 *               is wanted
 * @return The value the given column of the given node
 */
public boolean isCellEditable(Object node, int column)
{
    return(false);
}

/**
 * Does nothing.
 * @param value The value to be set
 * @param node The node for which the value is to be set
 * @param column The column of the node for which the value is to be set
 */
public void setValueAt(Object value, Object node, int column) {}

/**
 * Returns the Object class type.
 * @param column The column whose class is needed.
 * @return Always return the Object class for this model
 */
public Class getColumnClass(int column)
{
    return(Object.class);
}

/**
 * At design time returns there are two columns, otherwise return zero columns.
 * @return The current column count
 */
public int getColumnCount()
{
    if (Beans.isDesignTime() == true) {
        return(2);
    }
    return(0);
}

/**
 * At design time returns headers for each column, otherwise returns null.
 * @param column The column whose name is required
 * @return The current name of the given column
 */
public String getColumnName(int column)
{
    if (Beans.isDesignTime() == true) {
        return(new String("Column Header " + column));
    }
    return(null);
}

}
