/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCTreeTableModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:48 $  $Locker:  $  Quest Software Inc.
package com.klg.jclass.util.treetable;

import javax.swing.tree.TreeModel;

/**
 * Model that provides a tree and treetable interface. Note that specific
 * implementations need to implement </code>javax.swing.table.TableModel</code>
 * and <code>javax.swing.tree.TreeModel</code> for them to work properly.
 * <p>
 * This interface requires that <i>some</i> of the <code>TableModel</code>'s
 * methods be implemented.  If the implementor of this model wishes the
 * <code>JCTreeTable</code> component to be able to switch dynamically to a full
 * table view of the data then they should complete the implementation
 * of the <code>TableModel</code> as well as specifying that the class also
 * implements it. <p>For reference, the methods needed to complete a
 * <code>TableModel</code> implementation are:<br>
 *  <pre>
 *   public int getRowCount();
 *   public Object getValueAt(int row, int column);
 *   public boolean isCellEditable(int row, int column);
 *   public void addTableModelListener(TableModelListener l);
 *   public void removeTableModelListener(TableModelListener l);
 *   public void setValueAt(Object value, int row, int column);
 * </pre>
 */
public interface JCTreeTableModel extends TreeModel {

//
// The following are the new methods
//

/**
 * Returns the value of the specific node and column.
 * @param node the node to be queried
 * @param column the index of the column
 * @return the value at the given node and column
 * @see #setValueAt
 */
public Object getValueAt(Object node, int column);

/**
 * Returns whether a particular cell is editable given the node and column.
 * @param node the node to be checked for editabiltily
 * @param column the index of the column
 * @return whether the cell at the given node and column is editable
 * @see #setValueAt
 */
public boolean isCellEditable(Object node, int column);

/**
 * Sets the value in the cell corresponding to the node and the column to the
 * given value.
 *
 * @param value the new value
 * @param node the node whose value is to be changed
 * @param column the column whose value is to be changed
 * @see #getValueAt
 * @see #isCellEditable
 */
public void setValueAt(Object value, Object node, int column);

//
// The following methods as a subset of javax.swing.table.TableModel
//

/**
 * Returns a class used to determine a renderer and editor for all the cell
 * values in the column.
 *
 * @param column the index of the column
 * @return the class used to determine a renderer and editor for the column
 */
public Class getColumnClass(int column);

/**
 * Returns the number of columns in the model.
 * @return the current number of columns
 */
public int getColumnCount();

/**
 * The name of the column at the specified column index.
 * @param column the index of the column whose name is required
 * @return the name of the column at the given column index
 */
public String getColumnName(int column);

}
