/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: NodeChildrenTable.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:48 $  $Locker:  $  Quest Software Inc.
package com.klg.jclass.util.treetable;

import com.klg.jclass.util.JCListenerList;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;
import javax.swing.tree.TreePath;
import java.io.Serializable;
import java.util.Enumeration;

/**
 * Class to map the children of a particular node in a
 * <code>JCTreeTableModel</code> into a standard Swing <code>TableModel</code>.
 */
public class NodeChildrenTable implements TableModel, Serializable
{

protected JCListenerList    listeners = null;
protected TreePath          path = null;
protected JCTreeTableModel  treetable;

/**
 * Creates a <code>NodeChildrenTable</code> based on the specified treetable.
 * Note that the parent node must be set before this model is
 * considered "non-empty"
 * @param treetable The tree table model
 * @see #setParentPath
 */
public NodeChildrenTable(JCTreeTableModel treetable)
{
	this.treetable = treetable;
}

/**
 * Creates a <code>NodeChildrenTable</code> based on the specified treetable and
 * parent node.  To change the parent node call <code>setParentPath()</code>.
 * @param treetable The tree table model
 * @param path The parent path node
 */
public NodeChildrenTable(JCTreeTableModel treetable, TreePath path)
{
	this(treetable);
	setParentPath(path);
}

/**
 * Returns the path of the parent node.
 * @return The current parent path node
 */
public TreePath getParentPath()
{
	return(path);
}

/**
 * Sets the path of the node whose children are to be mapped into
 * <code>TableModel</code>.
 * @param path The new parent path node
 */
public void setParentPath(TreePath path)
{
	if (this.path == path) {
		return;
	}

	if (this.path != null && this.path.equals(path)) {
		return;
	}

	this.path = path;
	TableModelEvent tme = new TableModelEvent(this);
	Enumeration e = JCListenerList.elements(listeners);
	while (e.hasMoreElements()) {
		TableModelListener listener = (TableModelListener)e.nextElement();
		listener.tableChanged(tme);
	}
}

/**
 * Return</code> whether the specified row in the <code>TableModel</code>
 * mapping is a Leaf or not.
 * @param row The row to check
 * @return Whether the row is a leaf
 */
public boolean isLeaf(int row)
{
    Object last = path.getLastPathComponent();
	return(treetable.isLeaf(treetable.getChild(last, row)));
}

/**
 * Adds a standard table model listener.  This class will listen for
 * appropriate changes to the tree and pass them on via this listener.
 * @param l The listener to add
 */
public void addTableModelListener(TableModelListener l)
{
	listeners = JCListenerList.add(listeners, l);
}

/**
 * Removes the table model listener
 * @param l The listener to remove
 */
public void removeTableModelListener(TableModelListener l)
{
    listeners = JCListenerList.remove(listeners, l);
}

public Class getColumnClass(int column)
{
	return(treetable.getColumnClass(column));
}

public int getColumnCount()
{
	return(treetable.getColumnCount());
}

public String getColumnName(int column)
{
	return(treetable.getColumnName(column));
}

public int getRowCount()
{
	if (path == null) {
		return(0);
	}
	return(treetable.getChildCount(path.getLastPathComponent()));
}

public Object getValueAt(int row, int column)
{
    Object last = path.getLastPathComponent();
	return(treetable.getValueAt(treetable.getChild(last, row), column));
}

public boolean isCellEditable(int row, int column)
{
    Object last = path.getLastPathComponent();
	return(treetable.isCellEditable(treetable.getChild(last, row), column));
}

public void setValueAt(Object value, int row, int column)
{
    Object last = path.getLastPathComponent();
	treetable.setValueAt(value, treetable.getChild(last, row), column);
}

}


