/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: AbstractValueModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:48 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.value;

import java.io.Serializable;
import java.util.Enumeration;
import com.klg.jclass.util.JCListenerList;


/**
 * This class is a base class for default value models.
 */
abstract public class AbstractValueModel implements JCValueModel, Serializable
{

/** The value for this value model. */
protected Object          value;

/**
 * A list of value listeners who get notified when the value is about to
 * change or has changed.
 */
protected JCListenerList  listeners;

/**
 * Adds a listener to listen for changes to the value.
 * @param listener The listener to add
 */
public void addValueListener(JCValueListener listener)
{
    listeners = JCListenerList.add(listeners, listener);
}

/**
 * Removes a listener that listens for changes to the value.
 * @param listener The listener to remove
 */
public void removeValueListener(JCValueListener listener)
{
    listeners = JCListenerList.remove(listeners, listener);
}

/**
 * Sets the internal core value.  Calls setValue(value, true). In other words,
 * dontProcessEqualValues is set to true.
 * @param value The new value
 */
public void setValue(Object value)
{
    setValue(value, true);
}

/**
 * Sets the internal core value but optionally exits before setting the
 * value (after the valueChanging but before the valueChanged event) if
 * the new and old values are equal.
 * @param value The new value
 * @param dontProcessEqualValues If true and the new value is the same object
 * as the new value, don't send any events (i.e. don't go thru the valueChanging
 * and valueChanged process).
 */
public void setValue(Object value, boolean dontProcessEqualValues)
{
    

    JCValueEvent event = new JCValueEvent(this, this.value, value);

    boolean isEqual = (dontProcessEqualValues &&
                       event.getNewValue() == event.getOldValue());

    // Post value changing event
    if (event != null && !isEqual) {
        
        Enumeration e = JCListenerList.elements(listeners);
        while (e.hasMoreElements()) {
            JCValueListener listener = (JCValueListener)e.nextElement();
            listener.valueChanging(event);
        }
    }

    if (event.getAllowChange() == false) {
        // Listener requested that this change be aborted
        
        return;
    }

    // Reset isEqual because a listener may have changed the value during the
    // value changing process.
    isEqual = (dontProcessEqualValues &&
               event.getNewValue() == event.getOldValue());
    if (isEqual) {
        return;
    }

    
    this.value = event.getNewValue();
    // Post value changed event
    Enumeration e = JCListenerList.elements(listeners);
    while (e.hasMoreElements()) {
        JCValueListener listener = (JCValueListener)e.nextElement();
        listener.valueChanged(event);
    }
}

/**
 * Returns the internal core value.
 * @return The current value
 */
public Object getValue()
{
    return(value);
}

/**
 * Returns the type of the core value. It is necessary to store the
 * class of the value separate from the object since if the value object is
 * null then the class information is lost.
 * @return The class of the value for this value model
 */
abstract public Class getValueClass();

}
