/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

// RCSID -- $RCSfile: JCXMLDTDResolver.java,v $ $Revision: 1.2 $ $Date: 2006-03-17 21:13:49 $ $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.xml;

import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.URL;

/**
 * Resolves requests for JClass DTDs to the correct place in the classpath.
 */
public class JCXMLDTDResolver implements EntityResolver {

protected String  jclassDTDName = "";
protected Class   resolvingClass = null;

/**
 * Constructor.  Takes the DTD name to be intercepted from resolve
 * requests.  Should only be used if this class will not be shared
 * across jars.
 * @param jclassDTDName
 */
public JCXMLDTDResolver(String jclassDTDName)
{
    this.jclassDTDName = jclassDTDName;
}

/**
 * Constructor.  Takes the DTD name to be intercepted from resolve
 * requests and a class that provides the space from which it should
 * be located.  This should be used if this class may appear in
 * several different jars (as in most JClass product lines).
 * @param jclassDTDName
 * @param resolvingClass
 */
public JCXMLDTDResolver(String jclassDTDName, Class resolvingClass)
{
    this(jclassDTDName);
    this.resolvingClass = resolvingClass;
}

public InputSource resolveEntity(java.lang.String publicId,
                                 java.lang.String systemId)
    throws SAXException, java.io.IOException
{
    if (resolvingClass == null) {
        resolvingClass = getClass();
    }

    // intercept use of the named jclass dtd and try to find it
    // in the classpath (e.g. in the jar)
    if (jclassDTDName != null      &&
        !jclassDTDName.equals("")  &&
        systemId != null           &&
        systemId.indexOf(jclassDTDName) != -1)
    {
        URL resourceURL = resolvingClass.getResource(
            "/com/klg/jclass/xml-dtd/" + jclassDTDName);
        if (resourceURL != null) {
            try {
                InputSource inSource =
                    new InputSource(resourceURL.openStream());
                // set the system id with the logical name
                // for resolving any sub-entities
                inSource.setSystemId(systemId);
                return(inSource);
            }
            catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }
        else {
            System.err.println("Warning: /com/klg/jclass/xml-dtd/" +
                               jclassDTDName + " not found on classpath");
        }
    }
    // If it's not the dtd named in the constructor, is it an entity dtd?
    else if (systemId != null && !systemId.equals("")) {
        // Try to see if it's a sub .dtd
		String dtdName = "";
		// look for text after last slash
		if (systemId.lastIndexOf("/") < systemId.length() - 1) {
			dtdName = systemId.substring(systemId.lastIndexOf("/") + 1);
		// or if no slash, try the whole string
		} else {
			dtdName = systemId;
		}
		dtdName = dtdName.trim();

		// only resolve DTDs -- allow entity xml files to pass through
		if (dtdName.endsWith(".dtd") || dtdName.endsWith(".DTD")) {
        	URL resourceURL = resolvingClass.getResource(
            	"/com/klg/jclass/xml-dtd/" + dtdName);
        	if (resourceURL != null) {
            	try {
                	InputSource inSource =
                    	new InputSource(resourceURL.openStream());
                	// Set the system id with the logical name
                	// for resolving any sub-entities
                	inSource.setSystemId(systemId);
                	return(inSource);
            	}
            	catch (IOException ioe) {
                	ioe.printStackTrace();
            	}
        	}
        	else {
            	System.err.println("Warning: /com/klg/jclass/xml-dtd/" +
                               	dtdName + " not found on classpath");
        	}
		}
    }

    // Not found in classpath or not the named jclass dtd, so
    // revert to default handling
    return(null);
}

// Stolen from Sun's old Resolver implementation
static public InputSource createInputSource(File file) throws IOException
{
    InputSource retval;
    String path;
    retval = new InputSource(new FileInputStream(file));

    // On JDK 1.2 and later, simplify this:
    //  "path = file.toURL ().toString ()".
    path = file.getAbsolutePath();
    if (File.separatorChar != '/') {
        path = path.replace(File.separatorChar, '/');
    }
    if (!path.startsWith("/")) {
        path = "/" + path;
    }
    if (!path.endsWith("/") && file.isDirectory()) {
        path = path + "/";
    }

    retval.setSystemId("file:" + path);
    return(retval);
}

}

