/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: JCXMLTableModel.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:49 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.util.xml;

import javax.swing.table.DefaultTableModel;
import java.io.*;
import java.util.Vector;

/**
 * Populates a swing <code>TableModel</code> with data taken from an incoming
 * stream of data formatted according to the XML specification in
 * <code>com.klg.jclass.util.xml.JCTableXMLParser</code>.
 */
public class JCXMLTableModel extends DefaultTableModel
							 implements JCTablePopulator
{

/** Parser used to parse the XML-formatted data. */
protected JCTableXMLParser parser;

/**
 * Populates this <code>TableModel</code> from the given input stream.
 * @param i the input stream of incoming data
 * @throws IOException
 */
public JCXMLTableModel(InputStream i) throws IOException
{
    super();
    parser = new JCTableXMLParser();
    readFromStream(i);
}

/**
 * Default constructor.  Creates parser class but expects user to
 * explicitly call <code>readFromStream()</code> or <code>read()</code>.
 * @throws IOException
 */
public JCXMLTableModel() throws IOException
{
    super();
    parser = new JCTableXMLParser();
}

/**
 * Reads data from an <code>InputStream</code> to populate this table.
 * @param i input stream of incoming data
 * @throws IOException
 */
protected void readFromStream(InputStream i) throws IOException
{
    Reader r = new BufferedReader(new InputStreamReader(i, "JISAutoDetect"));
    read(r);
}

/**
 * Reads and parses data to populate this table.
 * @param r reader open on an active stream
 * @throws IOException
 */
public void read(Reader r) throws IOException
{
    parser.readAndParse(r, this);
}

/**
 * Populates the table with the given data.  Implemented from the
 * <code>JCTablePopulator</code> interface.
 * @param rowV a vector of vectors, where each inside vector represents
 *             one row in the table
 * @param rowLabelV a vector containing the row labels for this table
 * @param colLabelV a vector containing the column labels for this table
 * @param nrows the number of rows in this table
 * @param ncols the number of columns in this table
 */
public void setTableData(Vector rowV, Vector rowLabelV, Vector colLabelV,
                         int nrows, int ncols)
{
   // Swing TableModel has no row labels, so ignore 'em
   setDataVector(rowV, colLabelV);
}

}
