/* 
______________________________________________________________________
*
* Copyright (c) 1996-2004 QUEST SOFTWARE INC.  All Rights Reserved.
* http://java.quest.com
*
* This software is the confidential and proprietary information of
* Quest Software Inc. ("Confidential Information").  You shall not disclose
* such Confidential Information and shall use it only in accordance with the
* terms of the license agreement you entered into with Quest Software.
*
* QUEST SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
* TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE, OR NON-INFRINGEMENT. QUEST SOFTWARE SHALL NOT BE LIABLE FOR ANY
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
* ______________________________________________________________________
*/

//   RCSID -- $RCSfile: IdeMetaData.java,v $ $Revision: 1.2 $
//            $Date: 2006-03-17 21:13:09 $  $Locker:  $  Quest Software Inc.

package com.klg.jclass.datasource;

import java.util.Enumeration;

import javax.swing.tree.*;

/**
 * Exploits commonality betwee ides.
 */
public abstract class IdeMetaData extends BaseMetaData implements IdeMetaDataModel {

static final long serialVersionUID = 7967981952927611609L;

public static long globalBookmarkBeforeLastMove = DataTableModel.ROW_NOT_FOUND;    // see undoDependantMove below
public static DataTableModel globalDataTableBeforeLastMove = null;                 // see undoDependantMove below

public IdeMetaData(DataModel dm) {
    super(dm);
}

/**
 * Responds to navigation events from ide underlying data sources.
 * @param row an int which is the row in the ide
 */
public void navigated(int row) {
    if (!getDataModel().getEventsEnabled()) {
        return;
    }

    // for vcafe so dependant moves can be undone
    globalBookmarkBeforeLastMove  = getDataModel().getCurrentGlobalBookmark();
    globalDataTableBeforeLastMove = getDataModel().getCurrentGlobalTable();

    try {

        MetaDataModel parentMeta = (MetaDataModel) getParent();
        DataTableModel dt = null;
        if (parentMeta == null) {
            // at root
            dt = (DataTableModel) getDataModel().getDataTableTree().getRoot();
        }
        else {
            DataTableModel parentTable = parentMeta.getCurrentDataTable();
            if (parentTable != null) {
                dt = parentTable.getTable(parentMeta.getCurrentBookmark(), this); // getCurrentDataTable();
            }
        }
        if (dt == null) {
            return;
        }

        long bookmark = dt.getRowIdentifier(row);
        setCurrentBookmark(bookmark);
        setCurrentDataTable(dt);

        if (dt.getMetaData().getDataModel().getCurrentGlobalBookmark() != bookmark ||
            getCurrentBookmark() != bookmark) {
            getDataModel().moveToRow((DataModelListener) dt, dt, bookmark, false);
        }
    }
    catch (Exception e) {
        System.out.println("navigated: " + e.toString());
    }
}

/**
 * Requery the current row in this data table and its subtree.
 *
 * @param originator the DataModelListener which initiated this action
 * @param dt the DataTableModel for which the current row and
 * subtree is to be requeried.
 */
public void requeryCurrentRowAndBelow(DataModelListener originator, DataTableModel dt) {
    try {
        if (dt == null) return;
        long parentBookmark         = dt.getParentBookmark();
        MetaDataModel parentMeta    = (MetaDataModel) dt.getMetaData().getParent();
        DataTableModel parentData = null;
        int eventToSend = DataModelEvent.AFTER_REQUERY_ROW_AND_DETAILS;
        if (parentMeta == null) {
            // this is the root
            parentData = dt;
            eventToSend = DataModelEvent.AFTER_RESET;
        }
        else {
            parentData   = getDataModel().getCurrentDataTable(originator, parentMeta);
        }

        if (parentData != null) {
            if (eventToSend == DataModelEvent.AFTER_RESET) {
                // redo the root
                ((IdeDataTableModel)parentData).requeryFromIdeDataSource();
                getDataModel().clearCurrentPath((MetaDataModel) this);
                setCurrentBookmark(DataTableModel.ROW_NOT_FOUND);
            }
            else {
                ((MutableTreeNode)dt).setParent(null);
                getDataModel().clearCurrentPath((MetaDataModel) parentMeta);
                parentData.deleteCacheChildren(originator);
            }
        }

        DataModelEvent dmEvent = new DataModelEvent(getDataModel(), originator, parentData, parentBookmark,
                                                    "", eventToSend);
        getDataModel().fireDataModelEvent(dmEvent);
    }
    catch (Exception e) {
        System.out.println(e.toString());
    }
}

/**
 * We have received an event from the IDE data source telling us
 * that changes applied to a row have been posted.
 *
 * @param originator the DataModelListener which initiated this action
 * @param dt the IdeDataTableModel in which the change occurred
 * @param bookmark a long which uniquely identifies a row
 */
public void handleIDEUpdateCellRequest(IdeDataTableModel dt, long bookmark) {
    // request listeners permissions to update
    try {
        // FIX: at this point we do not know which cell was edited, so lets use the first modified
        String firstModifiedColumn = null;
        for (int col = 0; col < getColumnCount(); col++) {
            String column = getColumnIdentifier(col);
            Object newCell = dt.getCell2(bookmark, column);
            Object oldCell = dt.getResultData(bookmark, getColumnIdentifier(col));
            if ((newCell == null && oldCell != null) || (oldCell == null && newCell != null) ||
                ((oldCell != null && newCell != null) && (!newCell.equals(oldCell)))) {
                firstModifiedColumn = column;
                break;
            }
        }
        // if not detected, use the last column since the key is usually at the beginning and read-only
        if (firstModifiedColumn == null) {
            firstModifiedColumn = getColumnIdentifier(getColumnCount() - 1);
        }
        // check okay to update this cell
        if (!((BaseDataTable)dt).prepareCellUpdate((DataModelListener) dt, bookmark, firstModifiedColumn)) {
            return;
        }
        ((BaseDataTable)dt).setRowDirty(bookmark);

    }
    catch (Exception e) {
        System.out.println(e.toString());
        return;
    }
    turnOffDataModelEvents();
    try {

        // dt.moveToRow(bookmark);
        for (int col = 0; col < getColumnCount(); col++) {
            String column = getColumnIdentifier(col);
            Object newCell = dt.getCell2(bookmark, column);
            Object oldCell = dt.getResultData(bookmark, getColumnIdentifier(col));
            // only update modified columns
            // check for nulls and whether values have changed
            if ((newCell == null && oldCell != null) || (oldCell == null && newCell != null) ||
                ((oldCell != null && newCell != null) && (!newCell.equals(oldCell)))) {
                dt.setValueAt2(bookmark,column, newCell);
            }
        }
    }
    catch (Exception e) {
        System.out.println(e.toString());
        try {
            DataModelEvent dmEvent = new DataModelEvent(getDataModel(), (DataModelListener) dt, dt, bookmark, "", DataModelEvent.AFTER_CHANGE_OF_ROW_DATA);
            getDataModel().fireDataModelEvent(dmEvent);
        }
        catch (DataModelException ex) {
        }
    }
    turnOnDataModelEvents();
    try {
        DataModelEvent dmEvent = new DataModelEvent(getDataModel(), (DataModelListener) dt, dt, bookmark, "", DataModelEvent.AFTER_CHANGE_OF_ROW_DATA);
        getDataModel().fireDataModelEvent(dmEvent);
    }
    catch (DataModelException ex) {
    }
}

/**
 * This simply calls dataModel.enableDataModelEvents() but
 * remembers the previous state.  That is, if they were turned
 * off by the user, don't turn them back on.
 */
public void turnOffDataModelEvents() {
    getDataModel().enableDataModelEvents(false);
}
/**
 * If events were originally on turn them back on.
 */
public void turnOnDataModelEvents() {
    getDataModel().enableDataModelEvents(true); // eventsOriginallyOn);
}

/**
 * For internal use only.
 * Recursively refreshes child dataSets of the root
 * Called by requeryLevel in the root DataTable.
 * @exception DataModelException If an exception occurs while refreshing the data sets.
 */
public void cleanChildDataSets() throws DataModelException {
    // for each of the root's children, recurse and clean
    for (Enumeration e = children(); e.hasMoreElements();) {
        cleanChildDataSets((IdeMetaData) e.nextElement());
    }
}

/**
 * For internal use only.
 * Called by cleanChildDataSet to do the work.
 * @exception DataModelException If an exception occurs while refreshing the data sets.
 */
protected void cleanChildDataSets(IdeMetaData md) throws DataModelException {
    md.refreshDataSet();
    for (Enumeration e = md.children(); e.hasMoreElements();) {
        cleanChildDataSets((IdeMetaData) e.nextElement());
    }
}

/**
 * Overrides BaseMetaData.setCommitPolicy to only allow
 * COMMIT_LEAVING_RECORD to be selected. This is the only
 * policy currently supported in IDE environments.
 *
 * @param commit_policy an int, must be COMMIT_LEAVING_RECORD.
 * @see com.klg.jclass.datasource.BaseMetaData#getShowDeletedRows
 * @exception DataModelException If set fails.
 */
public void setCommitPolicy(int commit_policy) throws DataModelException {
    if (commit_policy != MetaDataModel.COMMIT_LEAVING_RECORD) {
        Exception e = new Exception(LocaleBundle.string(LocaleBundle.policy_not_supported));
        throw new DataModelException(DataModelEvent.INITIALIZE, e);
    }
    super.setCommitPolicy(commit_policy);
}

/**
 * handle the non-heirarchical case where the ide has
 * its data sets linked in master-detail relationships
 * and there are jclass components bound in single-level
 * to each master-detail level.  That is, each jclass
 * component does not know about anybody else and they
 * each have their own data model.
 *
 * If this level receives a reset then the parent has
 * navigated, so refresh this level.
 * @exception Exception If an exception occurs while refreshing the data set.
 */
protected boolean handledAsSingleLevelBindingMove() throws Exception {
    try {
        TreeNode metaRoot = (TreeNode)getDataModel().getMetaDataTree().getRoot();
        if (metaRoot.getChildCount() == 0) {
            turnOffDataModelEvents();
            getDataModel().requeryAll(new DataModelAdapter());
            turnOnDataModelEvents();
            return true;
        }
    }
    catch (Exception e) {
        turnOnDataModelEvents();
    }
    return false;
}

}
